;;;; chicken.keyword.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.keyword
      'library
      (scheme#list)
      '((keyword? . chicken.keyword#keyword?)
        (get-keyword . chicken.keyword#get-keyword)
        (keyword->string . chicken.keyword#keyword->string)
        (string->keyword . chicken.keyword#string->keyword))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
