;;;; chicken.compiler.user-pass.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.user-pass
      'user-pass
      (scheme#list)
      '((user-options-pass . chicken.compiler.user-pass#user-options-pass)
        (user-read-pass . chicken.compiler.user-pass#user-read-pass)
        (user-preprocessor-pass
          .
          chicken.compiler.user-pass#user-preprocessor-pass)
        (user-pass . chicken.compiler.user-pass#user-pass)
        (user-post-analysis-pass
          .
          chicken.compiler.user-pass#user-post-analysis-pass))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
