;;;; chicken.random.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.random
      'extras
      (scheme#list)
      '((set-pseudo-random-seed! . chicken.random#set-pseudo-random-seed!)
        (pseudo-random-integer . chicken.random#pseudo-random-integer)
        (pseudo-random-real . chicken.random#pseudo-random-real)
        (random-bytes . chicken.random#random-bytes))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
