;;;; chicken.fixnum.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.fixnum
      'library
      (scheme#list)
      '((fx/? . chicken.fixnum#fx/?)
        (fx*? . chicken.fixnum#fx*?)
        (fx-? . chicken.fixnum#fx-?)
        (fx+? . chicken.fixnum#fx+?)
        (fxrem . chicken.fixnum#fxrem)
        (fxmod . chicken.fixnum#fxmod)
        (fxgcd . chicken.fixnum#fxgcd)
        (fx/ . chicken.fixnum#fx/)
        (fxlen . chicken.fixnum#fxlen)
        (fxeven? . chicken.fixnum#fxeven?)
        (fxodd? . chicken.fixnum#fxodd?)
        (fxshr . chicken.fixnum#fxshr)
        (fxshl . chicken.fixnum#fxshl)
        (fxnot . chicken.fixnum#fxnot)
        (fxxor . chicken.fixnum#fxxor)
        (fxior . chicken.fixnum#fxior)
        (fxand . chicken.fixnum#fxand)
        (fxneg . chicken.fixnum#fxneg)
        (fxmax . chicken.fixnum#fxmax)
        (fxmin . chicken.fixnum#fxmin)
        (fx<= . chicken.fixnum#fx<=)
        (fx>= . chicken.fixnum#fx>=)
        (fx< . chicken.fixnum#fx<)
        (fx> . chicken.fixnum#fx>)
        (fx= . chicken.fixnum#fx=)
        (fx* . chicken.fixnum#fx*)
        (fx- . chicken.fixnum#fx-)
        (fx+ . chicken.fixnum#fx+)
        (fixnum-precision . chicken.fixnum#fixnum-precision)
        (fixnum-bits . chicken.fixnum#fixnum-bits)
        (most-negative-fixnum . chicken.fixnum#most-negative-fixnum)
        (most-positive-fixnum . chicken.fixnum#most-positive-fixnum))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
