;;;; chicken.compiler.optimizer.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.optimizer
      'optimizer
      (scheme#list)
      '((scan-toplevel-assignments
          .
          chicken.compiler.optimizer#scan-toplevel-assignments)
        (perform-high-level-optimizations
          .
          chicken.compiler.optimizer#perform-high-level-optimizations)
        (transform-direct-lambdas!
          .
          chicken.compiler.optimizer#transform-direct-lambdas!)
        (determine-loop-and-dispatch
          .
          chicken.compiler.optimizer#determine-loop-and-dispatch)
        (eq-inline-operator . chicken.compiler.optimizer#eq-inline-operator)
        (membership-test-operators
          .
          chicken.compiler.optimizer#membership-test-operators)
        (membership-unfold-limit
          .
          chicken.compiler.optimizer#membership-unfold-limit)
        (default-optimization-passes
          .
          chicken.compiler.optimizer#default-optimization-passes)
        (rewrite . chicken.compiler.optimizer#rewrite))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
