;;;; chicken.memory.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.memory
      'lolevel
      (scheme#list)
      '((address->pointer . chicken.memory#address->pointer)
        (align-to-word . chicken.memory#align-to-word)
        (allocate . chicken.memory#allocate)
        (free . chicken.memory#free)
        (make-pointer-vector . chicken.memory#make-pointer-vector)
        (move-memory! . chicken.memory#move-memory!)
        (object->pointer . chicken.memory#object->pointer)
        (pointer+ . chicken.memory#pointer+)
        (pointer->address . chicken.memory#pointer->address)
        (pointer->object . chicken.memory#pointer->object)
        (pointer-f32-ref . chicken.memory#pointer-f32-ref)
        (pointer-f32-set! . chicken.memory#pointer-f32-set!)
        (pointer-f64-ref . chicken.memory#pointer-f64-ref)
        (pointer-f64-set! . chicken.memory#pointer-f64-set!)
        (pointer-like? . chicken.memory#pointer-like?)
        (pointer-s16-ref . chicken.memory#pointer-s16-ref)
        (pointer-s16-set! . chicken.memory#pointer-s16-set!)
        (pointer-s32-ref . chicken.memory#pointer-s32-ref)
        (pointer-s32-set! . chicken.memory#pointer-s32-set!)
        (pointer-s64-ref . chicken.memory#pointer-s64-ref)
        (pointer-s64-set! . chicken.memory#pointer-s64-set!)
        (pointer-s8-ref . chicken.memory#pointer-s8-ref)
        (pointer-s8-set! . chicken.memory#pointer-s8-set!)
        (pointer-tag . chicken.memory#pointer-tag)
        (pointer-u16-ref . chicken.memory#pointer-u16-ref)
        (pointer-u16-set! . chicken.memory#pointer-u16-set!)
        (pointer-u32-ref . chicken.memory#pointer-u32-ref)
        (pointer-u32-set! . chicken.memory#pointer-u32-set!)
        (pointer-u64-ref . chicken.memory#pointer-u64-ref)
        (pointer-u64-set! . chicken.memory#pointer-u64-set!)
        (pointer-u8-ref . chicken.memory#pointer-u8-ref)
        (pointer-u8-set! . chicken.memory#pointer-u8-set!)
        (pointer-vector . chicken.memory#pointer-vector)
        (pointer-vector-fill! . chicken.memory#pointer-vector-fill!)
        (pointer-vector-length . chicken.memory#pointer-vector-length)
        (pointer-vector-ref . chicken.memory#pointer-vector-ref)
        (pointer-vector-set! . chicken.memory#pointer-vector-set!)
        (pointer-vector? . chicken.memory#pointer-vector?)
        (pointer=? . chicken.memory#pointer=?)
        (pointer? . chicken.memory#pointer?)
        (tag-pointer . chicken.memory#tag-pointer)
        (tagged-pointer? . chicken.memory#tagged-pointer?))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
