;;;; chicken.compiler.scrutinizer.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.scrutinizer
      'scrutinizer
      (scheme#list)
      '((scrutinize . chicken.compiler.scrutinizer#scrutinize)
        (load-type-database . chicken.compiler.scrutinizer#load-type-database)
        (emit-types-file . chicken.compiler.scrutinizer#emit-types-file)
        (validate-type . chicken.compiler.scrutinizer#validate-type)
        (check-and-validate-type
          .
          chicken.compiler.scrutinizer#check-and-validate-type)
        (install-specializations
          .
          chicken.compiler.scrutinizer#install-specializations)
        (match-types . chicken.compiler.scrutinizer#match-types)
        (refine-types . chicken.compiler.scrutinizer#refine-types)
        (type<=? . chicken.compiler.scrutinizer#type<=?))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
