;;;; chicken.load.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.load
      'eval
      (scheme#list)
      '((dynamic-load-libraries . chicken.load#dynamic-load-libraries)
        (set-dynamic-load-mode! . chicken.load#set-dynamic-load-mode!)
        (load-library . chicken.load#load-library)
        (load-noisily . chicken.load#load-noisily)
        (load-relative . chicken.load#load-relative)
        (load-verbose . chicken.load#load-verbose)
        (provide . chicken.load#provide)
        (provided? . chicken.load#provided?)
        (require . chicken.load#require))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
