;;;; chicken.errno.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.errno
      'posix
      (scheme#list)
      '((errno/xdev . chicken.errno#errno/xdev)
        (errno/wouldblock . chicken.errno#errno/wouldblock)
        (errno/srch . chicken.errno#errno/srch)
        (errno/spipe . chicken.errno#errno/spipe)
        (errno/rofs . chicken.errno#errno/rofs)
        (errno/range . chicken.errno#errno/range)
        (errno/pipe . chicken.errno#errno/pipe)
        (errno/perm . chicken.errno#errno/perm)
        (errno/nxio . chicken.errno#errno/nxio)
        (errno/notty . chicken.errno#errno/notty)
        (errno/notempty . chicken.errno#errno/notempty)
        (errno/notdir . chicken.errno#errno/notdir)
        (errno/nosys . chicken.errno#errno/nosys)
        (errno/nospc . chicken.errno#errno/nospc)
        (errno/nomem . chicken.errno#errno/nomem)
        (errno/nolck . chicken.errno#errno/nolck)
        (errno/noexec . chicken.errno#errno/noexec)
        (errno/noent . chicken.errno#errno/noent)
        (errno/nodev . chicken.errno#errno/nodev)
        (errno/nfile . chicken.errno#errno/nfile)
        (errno/nametoolong . chicken.errno#errno/nametoolong)
        (errno/mlink . chicken.errno#errno/mlink)
        (errno/mfile . chicken.errno#errno/mfile)
        (errno/isdir . chicken.errno#errno/isdir)
        (errno/io . chicken.errno#errno/io)
        (errno/inval . chicken.errno#errno/inval)
        (errno/intr . chicken.errno#errno/intr)
        (errno/ilseq . chicken.errno#errno/ilseq)
        (errno/fbig . chicken.errno#errno/fbig)
        (errno/fault . chicken.errno#errno/fault)
        (errno/exist . chicken.errno#errno/exist)
        (errno/dom . chicken.errno#errno/dom)
        (errno/deadlk . chicken.errno#errno/deadlk)
        (errno/child . chicken.errno#errno/child)
        (errno/busy . chicken.errno#errno/busy)
        (errno/badf . chicken.errno#errno/badf)
        (errno/again . chicken.errno#errno/again)
        (errno/acces . chicken.errno#errno/acces)
        (errno/2big . chicken.errno#errno/2big)
        (errno . chicken.errno#errno))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
