;;;; chicken.bitwise.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.bitwise
      'library
      (scheme#list)
      '((arithmetic-shift . chicken.bitwise#arithmetic-shift)
        (integer-length . chicken.bitwise#integer-length)
        (bit->boolean . chicken.bitwise#bit->boolean)
        (bitwise-not . chicken.bitwise#bitwise-not)
        (bitwise-xor . chicken.bitwise#bitwise-xor)
        (bitwise-ior . chicken.bitwise#bitwise-ior)
        (bitwise-and . chicken.bitwise#bitwise-and))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
