;;;; chicken.plist.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.plist
      'library
      (scheme#list)
      '((get . chicken.plist#get)
        (get-properties . chicken.plist#get-properties)
        (put! . chicken.plist#put!)
        (remprop! . chicken.plist#remprop!)
        (symbol-plist . chicken.plist#symbol-plist))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
