;;;; chicken.sort.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.sort
      'data-structures
      (scheme#list)
      '((merge . chicken.sort#merge)
        (merge! . chicken.sort#merge!)
        (sort . chicken.sort#sort)
        (sort! . chicken.sort#sort!)
        (sorted? . chicken.sort#sorted?)
        (topological-sort . chicken.sort#topological-sort))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
