;;;; chicken.pathname.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.pathname
      'pathname
      (scheme#list)
      '((absolute-pathname? . chicken.pathname#absolute-pathname?)
        (decompose-directory . chicken.pathname#decompose-directory)
        (decompose-pathname . chicken.pathname#decompose-pathname)
        (directory-null? . chicken.pathname#directory-null?)
        (make-absolute-pathname . chicken.pathname#make-absolute-pathname)
        (make-pathname . chicken.pathname#make-pathname)
        (normalize-pathname . chicken.pathname#normalize-pathname)
        (pathname-directory . chicken.pathname#pathname-directory)
        (pathname-extension . chicken.pathname#pathname-extension)
        (pathname-file . chicken.pathname#pathname-file)
        (pathname-replace-directory
          .
          chicken.pathname#pathname-replace-directory)
        (pathname-replace-extension
          .
          chicken.pathname#pathname-replace-extension)
        (pathname-replace-file . chicken.pathname#pathname-replace-file)
        (pathname-strip-directory . chicken.pathname#pathname-strip-directory)
        (pathname-strip-extension . chicken.pathname#pathname-strip-extension))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
