;;;; chicken.compiler.lfa2.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.lfa2
      'lfa2
      (scheme#list)
      '((perform-secondary-flow-analysis
          .
          chicken.compiler.lfa2#perform-secondary-flow-analysis)
        (perform-unboxing . chicken.compiler.lfa2#perform-unboxing))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
