;;;; chicken.process-context.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.process-context
      'library
      (scheme#list)
      '((argv . chicken.process-context#argv)
        (argc+argv . chicken.process-context#argc+argv)
        (command-line-arguments
          .
          chicken.process-context#command-line-arguments)
        (program-name . chicken.process-context#program-name)
        (executable-pathname . chicken.process-context#executable-pathname)
        (change-directory . chicken.process-context#change-directory)
        (current-directory . chicken.process-context#current-directory)
        (get-environment-variable
          .
          chicken.process-context#get-environment-variable)
        (get-environment-variables
          .
          chicken.process-context#get-environment-variables)
        (set-environment-variable!
          .
          chicken.process-context#set-environment-variable!)
        (unset-environment-variable!
          .
          chicken.process-context#unset-environment-variable!))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
