;;;; chicken.continuation.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.continuation
      'continuation
      (scheme#list)
      '((continuation? . chicken.continuation#continuation?)
        (continuation-capture . chicken.continuation#continuation-capture)
        (continuation-graft . chicken.continuation#continuation-graft)
        (continuation-return . chicken.continuation#continuation-return))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
