;;;; chicken.format.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.format
      'extras
      (scheme#list)
      '((format . chicken.format#format)
        (fprintf . chicken.format#fprintf)
        (printf . chicken.format#printf)
        (sprintf . chicken.format#sprintf))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
