;;;; chicken.io.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.io
      'extras
      (scheme#list)
      '((read-list . chicken.io#read-list)
        (read-buffered . chicken.io#read-buffered)
        (read-byte . chicken.io#read-byte)
        (read-line . chicken.io#read-line)
        (read-lines . chicken.io#read-lines)
        (read-string . chicken.io#read-string)
        (read-string! . chicken.io#read-string!)
        (read-token . chicken.io#read-token)
        (write-byte . chicken.io#write-byte)
        (write-line . chicken.io#write-line)
        (write-string . chicken.io#write-string))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
