;;;; chicken.read-syntax.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.read-syntax
      'read-syntax
      (scheme#list)
      '((copy-read-table . chicken.read-syntax#copy-read-table)
        (current-read-table . chicken.read-syntax#current-read-table)
        (define-reader-ctor . chicken.read-syntax#define-reader-ctor)
        (set-read-syntax! . chicken.read-syntax#set-read-syntax!)
        (set-sharp-read-syntax! . chicken.read-syntax#set-sharp-read-syntax!)
        (set-parameterized-read-syntax!
          .
          chicken.read-syntax#set-parameterized-read-syntax!))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
