;;;; chicken.locative.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.locative
      'lolevel
      (scheme#list)
      '((locative? . chicken.locative#locative?)
        (make-locative . chicken.locative#make-locative)
        (make-weak-locative . chicken.locative#make-weak-locative)
        (locative-ref . chicken.locative#locative-ref)
        (locative-set! . chicken.locative#locative-set!)
        (locative->object . chicken.locative#locative->object))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
