;;;; chicken.string.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.string
      'data-structures
      (scheme#list)
      '((conc . chicken.string#conc)
        (->string . chicken.string#->string)
        (string-chop . chicken.string#string-chop)
        (string-chomp . chicken.string#string-chomp)
        (string-compare3 . chicken.string#string-compare3)
        (string-compare3-ci . chicken.string#string-compare3-ci)
        (reverse-list->string . chicken.string#reverse-list->string)
        (reverse-string-append . chicken.string#reverse-string-append)
        (string-intersperse . chicken.string#string-intersperse)
        (string-split . chicken.string#string-split)
        (string-translate . chicken.string#string-translate)
        (string-translate* . chicken.string#string-translate*)
        (substring=? . chicken.string#substring=?)
        (substring-ci=? . chicken.string#substring-ci=?)
        (substring-index . chicken.string#substring-index)
        (substring-index-ci . chicken.string#substring-index-ci))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
