;;;; chicken.compiler.c-backend.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.c-backend
      'c-backend
      (scheme#list)
      '((generate-code . chicken.compiler.c-backend#generate-code)
        (foreign-type-declaration
          .
          chicken.compiler.c-backend#foreign-type-declaration))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
