;;;; chicken.flonum.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.flonum
      'library
      (scheme#list)
      '((flonum-print-precision . chicken.flonum#flonum-print-precision)
        (fpinteger? . chicken.flonum#fpinteger?)
        (fpabs . chicken.flonum#fpabs)
        (fpsqrt . chicken.flonum#fpsqrt)
        (fplog . chicken.flonum#fplog)
        (fpexpt . chicken.flonum#fpexpt)
        (fpexp . chicken.flonum#fpexp)
        (fpatan2 . chicken.flonum#fpatan2)
        (fpatan . chicken.flonum#fpatan)
        (fpacos . chicken.flonum#fpacos)
        (fpasin . chicken.flonum#fpasin)
        (fptan . chicken.flonum#fptan)
        (fpcos . chicken.flonum#fpcos)
        (fpsin . chicken.flonum#fpsin)
        (fpceiling . chicken.flonum#fpceiling)
        (fpround . chicken.flonum#fpround)
        (fptruncate . chicken.flonum#fptruncate)
        (fpfloor . chicken.flonum#fpfloor)
        (fpmin . chicken.flonum#fpmin)
        (fpmax . chicken.flonum#fpmax)
        (fpneg . chicken.flonum#fpneg)
        (fp<= . chicken.flonum#fp<=)
        (fp>= . chicken.flonum#fp>=)
        (fp< . chicken.flonum#fp<)
        (fp> . chicken.flonum#fp>)
        (fp= . chicken.flonum#fp=)
        (fp/? . chicken.flonum#fp/?)
        (fpgcd . chicken.flonum#fpgcd)
        (fp/ . chicken.flonum#fp/)
        (fp* . chicken.flonum#fp*)
        (fp- . chicken.flonum#fp-)
        (fp+ . chicken.flonum#fp+)
        (flonum-minimum-decimal-exponent
          .
          chicken.flonum#flonum-minimum-decimal-exponent)
        (flonum-maximum-decimal-exponent
          .
          chicken.flonum#flonum-maximum-decimal-exponent)
        (flonum-minimum-exponent . chicken.flonum#flonum-minimum-exponent)
        (flonum-maximum-exponent . chicken.flonum#flonum-maximum-exponent)
        (flonum-decimal-precision . chicken.flonum#flonum-decimal-precision)
        (flonum-precision . chicken.flonum#flonum-precision)
        (flonum-epsilon . chicken.flonum#flonum-epsilon)
        (flonum-radix . chicken.flonum#flonum-radix)
        (minimum-flonum . chicken.flonum#minimum-flonum)
        (maximum-flonum . chicken.flonum#maximum-flonum))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
