;;;; chicken.platform.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.platform
      'library
      (scheme#list)
      '((build-platform . chicken.platform#build-platform)
        (chicken-version . chicken.platform#chicken-version)
        (chicken-home . chicken.platform#chicken-home)
        (feature? . chicken.platform#feature?)
        (features . chicken.platform#features)
        (machine-byte-order . chicken.platform#machine-byte-order)
        (machine-type . chicken.platform#machine-type)
        (repository-path . chicken.platform#repository-path)
        (installation-repository . chicken.platform#installation-repository)
        (register-feature! . chicken.platform#register-feature!)
        (unregister-feature! . chicken.platform#unregister-feature!)
        (software-type . chicken.platform#software-type)
        (software-version . chicken.platform#software-version)
        (return-to-host . chicken.platform#return-to-host)
        (system-config-directory . chicken.platform#system-config-directory)
        (system-cache-directory . chicken.platform#system-cache-directory))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
