;;;; chicken.compiler.batch-driver.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.batch-driver
      'batch-driver
      (scheme#list)
      '((compile-source-file
          .
          chicken.compiler.batch-driver#compile-source-file))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
