;;;; chicken.compiler.compiler-syntax.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.compiler-syntax
      'compiler-syntax
      (scheme#list)
      '((compiler-syntax-statistics
          .
          chicken.compiler.compiler-syntax#compiler-syntax-statistics))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
