;;;; chicken.time.posix.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.time.posix
      'posix
      (scheme#list)
      '((seconds->utc-time . chicken.time.posix#seconds->utc-time)
        (utc-time->seconds . chicken.time.posix#utc-time->seconds)
        (seconds->local-time . chicken.time.posix#seconds->local-time)
        (seconds->string . chicken.time.posix#seconds->string)
        (local-time->seconds . chicken.time.posix#local-time->seconds)
        (string->time . chicken.time.posix#string->time)
        (time->string . chicken.time.posix#time->string)
        (local-timezone-abbreviation
          .
          chicken.time.posix#local-timezone-abbreviation))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
