;;;; chicken.compiler.c-platform.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.compiler.c-platform
      'c-platform
      (scheme#list)
      '((default-declarations
          .
          chicken.compiler.c-platform#default-declarations)
        (default-profiling-declarations
          .
          chicken.compiler.c-platform#default-profiling-declarations)
        (default-units . chicken.compiler.c-platform#default-units)
        (valid-compiler-options
          .
          chicken.compiler.c-platform#valid-compiler-options)
        (valid-compiler-options-with-argument
          .
          chicken.compiler.c-platform#valid-compiler-options-with-argument)
        (target-include-file . chicken.compiler.c-platform#target-include-file)
        (words-per-flonum . chicken.compiler.c-platform#words-per-flonum))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
