;;;; chicken.tcp.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.tcp
      'tcp
      (scheme#list)
      '((tcp-close . chicken.tcp#tcp-close)
        (tcp-listen . chicken.tcp#tcp-listen)
        (tcp-connect . chicken.tcp#tcp-connect)
        (tcp-accept . chicken.tcp#tcp-accept)
        (tcp-accept-ready? . chicken.tcp#tcp-accept-ready?)
        (tcp-listener? . chicken.tcp#tcp-listener?)
        (tcp-addresses . chicken.tcp#tcp-addresses)
        (tcp-abandon-port . chicken.tcp#tcp-abandon-port)
        (tcp-listener-port . chicken.tcp#tcp-listener-port)
        (tcp-listener-fileno . chicken.tcp#tcp-listener-fileno)
        (tcp-port-numbers . chicken.tcp#tcp-port-numbers)
        (tcp-buffer-size . chicken.tcp#tcp-buffer-size)
        (tcp-read-timeout . chicken.tcp#tcp-read-timeout)
        (tcp-write-timeout . chicken.tcp#tcp-write-timeout)
        (tcp-accept-timeout . chicken.tcp#tcp-accept-timeout)
        (tcp-connect-timeout . chicken.tcp#tcp-connect-timeout))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
