;;;; chicken.file.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.file
      'file
      (scheme#list)
      '((create-directory . chicken.file#create-directory)
        (delete-directory . chicken.file#delete-directory)
        (create-temporary-file . chicken.file#create-temporary-file)
        (create-temporary-directory . chicken.file#create-temporary-directory)
        (delete-file . chicken.file#delete-file)
        (delete-file* . chicken.file#delete-file*)
        (copy-file . chicken.file#copy-file)
        (move-file . chicken.file#move-file)
        (rename-file . chicken.file#rename-file)
        (file-exists? . chicken.file#file-exists?)
        (directory-exists? . chicken.file#directory-exists?)
        (file-readable? . chicken.file#file-readable?)
        (file-writable? . chicken.file#file-writable?)
        (file-executable? . chicken.file#file-executable?)
        (directory . chicken.file#directory)
        (find-files . chicken.file#find-files)
        (glob . chicken.file#glob))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
