;;;; chicken.eval.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.eval
      'eval
      (scheme#list)
      '((eval-handler . chicken.eval#eval-handler)
        (module-environment . chicken.eval#module-environment))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
