;;;; srfi-4.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'srfi-4
      'srfi-4
      (scheme#list)
      '((blob->f32vector . srfi-4#blob->f32vector)
        (blob->f32vector/shared . srfi-4#blob->f32vector/shared)
        (blob->f64vector . srfi-4#blob->f64vector)
        (blob->f64vector/shared . srfi-4#blob->f64vector/shared)
        (blob->s16vector . srfi-4#blob->s16vector)
        (blob->s16vector/shared . srfi-4#blob->s16vector/shared)
        (blob->s32vector . srfi-4#blob->s32vector)
        (blob->s32vector/shared . srfi-4#blob->s32vector/shared)
        (blob->s64vector . srfi-4#blob->s64vector)
        (blob->s64vector/shared . srfi-4#blob->s64vector/shared)
        (blob->s8vector . srfi-4#blob->s8vector)
        (blob->s8vector/shared . srfi-4#blob->s8vector/shared)
        (blob->u16vector . srfi-4#blob->u16vector)
        (blob->u16vector/shared . srfi-4#blob->u16vector/shared)
        (blob->u32vector . srfi-4#blob->u32vector)
        (blob->u32vector/shared . srfi-4#blob->u32vector/shared)
        (blob->u64vector . srfi-4#blob->u64vector)
        (blob->u64vector/shared . srfi-4#blob->u64vector/shared)
        (blob->u8vector . srfi-4#blob->u8vector)
        (blob->u8vector/shared . srfi-4#blob->u8vector/shared)
        (f32vector . srfi-4#f32vector)
        (f32vector->blob . srfi-4#f32vector->blob)
        (f32vector->blob/shared . srfi-4#f32vector->blob/shared)
        (f32vector->list . srfi-4#f32vector->list)
        (f32vector-length . srfi-4#f32vector-length)
        (f32vector-ref . srfi-4#f32vector-ref)
        (f32vector-set! . srfi-4#f32vector-set!)
        (f32vector? . srfi-4#f32vector?)
        (f64vector . srfi-4#f64vector)
        (f64vector->blob . srfi-4#f64vector->blob)
        (f64vector->blob/shared . srfi-4#f64vector->blob/shared)
        (f64vector->list . srfi-4#f64vector->list)
        (f64vector-length . srfi-4#f64vector-length)
        (f64vector-ref . srfi-4#f64vector-ref)
        (f64vector-set! . srfi-4#f64vector-set!)
        (f64vector? . srfi-4#f64vector?)
        (s8vector . srfi-4#s8vector)
        (s8vector->blob . srfi-4#s8vector->blob)
        (s8vector->blob/shared . srfi-4#s8vector->blob/shared)
        (s8vector->list . srfi-4#s8vector->list)
        (s8vector-length . srfi-4#s8vector-length)
        (s8vector-ref . srfi-4#s8vector-ref)
        (s8vector-set! . srfi-4#s8vector-set!)
        (s8vector? . srfi-4#s8vector?)
        (s16vector . srfi-4#s16vector)
        (s16vector->blob . srfi-4#s16vector->blob)
        (s16vector->blob/shared . srfi-4#s16vector->blob/shared)
        (s16vector->list . srfi-4#s16vector->list)
        (s16vector-length . srfi-4#s16vector-length)
        (s16vector-ref . srfi-4#s16vector-ref)
        (s16vector-set! . srfi-4#s16vector-set!)
        (s16vector? . srfi-4#s16vector?)
        (s32vector . srfi-4#s32vector)
        (s32vector->blob . srfi-4#s32vector->blob)
        (s32vector->blob/shared . srfi-4#s32vector->blob/shared)
        (s32vector->list . srfi-4#s32vector->list)
        (s32vector-length . srfi-4#s32vector-length)
        (s32vector-ref . srfi-4#s32vector-ref)
        (s32vector-set! . srfi-4#s32vector-set!)
        (s32vector? . srfi-4#s32vector?)
        (s64vector . srfi-4#s64vector)
        (s64vector->blob . srfi-4#s64vector->blob)
        (s64vector->blob/shared . srfi-4#s64vector->blob/shared)
        (s64vector->list . srfi-4#s64vector->list)
        (s64vector-length . srfi-4#s64vector-length)
        (s64vector-ref . srfi-4#s64vector-ref)
        (s64vector-set! . srfi-4#s64vector-set!)
        (s64vector? . srfi-4#s64vector?)
        (u8vector . srfi-4#u8vector)
        (u8vector->blob . srfi-4#u8vector->blob)
        (u8vector->blob/shared . srfi-4#u8vector->blob/shared)
        (u8vector->list . srfi-4#u8vector->list)
        (u8vector-length . srfi-4#u8vector-length)
        (u8vector-ref . srfi-4#u8vector-ref)
        (u8vector-set! . srfi-4#u8vector-set!)
        (u8vector? . srfi-4#u8vector?)
        (u16vector . srfi-4#u16vector)
        (u16vector->blob . srfi-4#u16vector->blob)
        (u16vector->blob/shared . srfi-4#u16vector->blob/shared)
        (u16vector->list . srfi-4#u16vector->list)
        (u16vector-length . srfi-4#u16vector-length)
        (u16vector-ref . srfi-4#u16vector-ref)
        (u16vector-set! . srfi-4#u16vector-set!)
        (u16vector? . srfi-4#u16vector?)
        (u32vector . srfi-4#u32vector)
        (u32vector->blob . srfi-4#u32vector->blob)
        (u32vector->blob/shared . srfi-4#u32vector->blob/shared)
        (u32vector->list . srfi-4#u32vector->list)
        (u32vector-length . srfi-4#u32vector-length)
        (u32vector-ref . srfi-4#u32vector-ref)
        (u32vector-set! . srfi-4#u32vector-set!)
        (u32vector? . srfi-4#u32vector?)
        (u64vector . srfi-4#u64vector)
        (u64vector->blob . srfi-4#u64vector->blob)
        (u64vector->blob/shared . srfi-4#u64vector->blob/shared)
        (u64vector->list . srfi-4#u64vector->list)
        (u64vector-length . srfi-4#u64vector-length)
        (u64vector-ref . srfi-4#u64vector-ref)
        (u64vector-set! . srfi-4#u64vector-set!)
        (u64vector? . srfi-4#u64vector?)
        (list->f32vector . srfi-4#list->f32vector)
        (list->f64vector . srfi-4#list->f64vector)
        (list->s16vector . srfi-4#list->s16vector)
        (list->s32vector . srfi-4#list->s32vector)
        (list->s64vector . srfi-4#list->s64vector)
        (list->s8vector . srfi-4#list->s8vector)
        (list->u16vector . srfi-4#list->u16vector)
        (list->u32vector . srfi-4#list->u32vector)
        (list->u8vector . srfi-4#list->u8vector)
        (list->u64vector . srfi-4#list->u64vector)
        (make-f32vector . srfi-4#make-f32vector)
        (make-f64vector . srfi-4#make-f64vector)
        (make-s16vector . srfi-4#make-s16vector)
        (make-s32vector . srfi-4#make-s32vector)
        (make-s64vector . srfi-4#make-s64vector)
        (make-s8vector . srfi-4#make-s8vector)
        (make-u16vector . srfi-4#make-u16vector)
        (make-u32vector . srfi-4#make-u32vector)
        (make-u64vector . srfi-4#make-u64vector)
        (make-u8vector . srfi-4#make-u8vector)
        (number-vector? . srfi-4#number-vector?)
        (read-u8vector . srfi-4#read-u8vector)
        (read-u8vector! . srfi-4#read-u8vector!)
        (release-number-vector . srfi-4#release-number-vector)
        (subf32vector . srfi-4#subf32vector)
        (subf64vector . srfi-4#subf64vector)
        (subs16vector . srfi-4#subs16vector)
        (subs32vector . srfi-4#subs32vector)
        (subs64vector . srfi-4#subs64vector)
        (subs8vector . srfi-4#subs8vector)
        (subu16vector . srfi-4#subu16vector)
        (subu8vector . srfi-4#subu8vector)
        (subu32vector . srfi-4#subu32vector)
        (subu64vector . srfi-4#subu64vector)
        (write-u8vector . srfi-4#write-u8vector))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
