;;;; chicken.repl.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.repl
      'repl
      (scheme#list)
      '((quit . chicken.repl#quit)
        (repl . chicken.repl#repl)
        (repl-prompt . chicken.repl#repl-prompt)
        (reset . chicken.repl#reset)
        (reset-handler . chicken.repl#reset-handler))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
