;;;; chicken.blob.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.blob
      'library
      (scheme#list)
      '((blob->string . chicken.blob#blob->string)
        (string->blob . chicken.blob#string->blob)
        (blob? . chicken.blob#blob?)
        (blob=? . chicken.blob#blob=?)
        (blob-size . chicken.blob#blob-size)
        (make-blob . chicken.blob#make-blob))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
