;;;; chicken.pretty-print.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.pretty-print
      'extras
      (scheme#list)
      '((pp . chicken.pretty-print#pp)
        (pretty-print . chicken.pretty-print#pretty-print)
        (pretty-print-width . chicken.pretty-print#pretty-print-width))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
