;;;; chicken.irregex.import.scm - GENERATED BY CHICKEN 5.3.0rc4 -*- Scheme -*-

(##sys#with-environment
  (lambda ()
    (##sys#register-compiled-module
      'chicken.irregex
      'irregex
      (scheme#list)
      '((irregex . chicken.irregex#irregex)
        (irregex? . chicken.irregex#irregex?)
        (string->sre . chicken.irregex#string->sre)
        (maybe-string->sre . chicken.irregex#maybe-string->sre)
        (sre->irregex . chicken.irregex#sre->irregex)
        (irregex-names . chicken.irregex#irregex-names)
        (irregex-num-submatches . chicken.irregex#irregex-num-submatches)
        (string->irregex . chicken.irregex#string->irregex)
        (make-irregex-chunker . chicken.irregex#make-irregex-chunker)
        (irregex-extract . chicken.irregex#irregex-extract)
        (irregex-fold . chicken.irregex#irregex-fold)
        (irregex-match . chicken.irregex#irregex-match)
        (irregex-match? . chicken.irregex#irregex-match?)
        (irregex-search . chicken.irregex#irregex-search)
        (irregex-split . chicken.irregex#irregex-split)
        (irregex-replace . chicken.irregex#irregex-replace)
        (irregex-replace/all . chicken.irregex#irregex-replace/all)
        (irregex-fold/chunked . chicken.irregex#irregex-fold/chunked)
        (irregex-match/chunked . chicken.irregex#irregex-match/chunked)
        (irregex-search/chunked . chicken.irregex#irregex-search/chunked)
        (irregex-match-data? . chicken.irregex#irregex-match-data?)
        (irregex-match-names . chicken.irregex#irregex-match-names)
        (irregex-match-start-index . chicken.irregex#irregex-match-start-index)
        (irregex-match-end-index . chicken.irregex#irregex-match-end-index)
        (irregex-match-num-submatches
          .
          chicken.irregex#irregex-match-num-submatches)
        (irregex-match-substring . chicken.irregex#irregex-match-substring)
        (irregex-match-valid-index?
          .
          chicken.irregex#irregex-match-valid-index?)
        (irregex-match-start-chunk . chicken.irregex#irregex-match-start-chunk)
        (irregex-match-end-chunk . chicken.irregex#irregex-match-end-chunk)
        (irregex-match-subchunk . chicken.irregex#irregex-match-subchunk)
        (glob->sre . chicken.irregex#glob->sre)
        (sre->string . chicken.irregex#sre->string)
        (irregex-opt . chicken.irregex#irregex-opt)
        (irregex-quote . chicken.irregex#irregex-quote))
      (scheme#list)
      (scheme#list))))

;; END OF FILE
