/* Generated from c-backend.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: c-backend.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -no-lambda-info -no-trace -emit-import-library chicken.compiler.c-backend -output-file c-backend.c
   unit: c-backend
   uses: eval support c-platform compiler data-structures internal extras library
*/
#include "chicken.h"

static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_eval_toplevel)
C_externimport void C_ccall C_eval_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_support_toplevel)
C_externimport void C_ccall C_support_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_c_2dplatform_toplevel)
C_externimport void C_ccall C_c_2dplatform_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_compiler_toplevel)
C_externimport void C_ccall C_compiler_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_data_2dstructures_toplevel)
C_externimport void C_ccall C_data_2dstructures_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_internal_toplevel)
C_externimport void C_ccall C_internal_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_extras_toplevel)
C_externimport void C_ccall C_extras_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[883];
static double C_possibly_force_alignment;


#define return(x) C_cblock C_r = (C_fix((C_word)(x))); goto C_ret; C_cblockend
C_regparm static C_word C_fcall stub2860(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_word lit=(C_word )(C_a0);
return(C_header_size(lit));
C_ret:
#undef return

return C_r;}

#define return(x) C_cblock C_r = (C_fix((C_word)(x))); goto C_ret; C_cblockend
C_regparm static C_word C_fcall stub2856(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_word lit=(C_word )(C_a0);

#ifdef C_SIXTY_FOUR
return((C_header_bits(lit) >> (24 + 32)) & 0xff);
#else
return((C_header_bits(lit) >> 24) & 0xff);
#endif

C_ret:
#undef return

return C_r;}

C_noret_decl(f14744)
static void C_ccall f14744(C_word c,C_word *av) C_noret;
C_noret_decl(f14749)
static void C_ccall f14749(C_word c,C_word *av) C_noret;
C_noret_decl(f14754)
static void C_ccall f14754(C_word c,C_word *av) C_noret;
C_noret_decl(f14762)
static void C_ccall f14762(C_word c,C_word *av) C_noret;
C_noret_decl(f14767)
static void C_ccall f14767(C_word c,C_word *av) C_noret;
C_noret_decl(f14772)
static void C_ccall f14772(C_word c,C_word *av) C_noret;
C_noret_decl(f14785)
static void C_ccall f14785(C_word c,C_word *av) C_noret;
C_noret_decl(f14789)
static void C_ccall f14789(C_word c,C_word *av) C_noret;
C_noret_decl(f14796)
static void C_ccall f14796(C_word c,C_word *av) C_noret;
C_noret_decl(f14801)
static void C_ccall f14801(C_word c,C_word *av) C_noret;
C_noret_decl(f14874)
static void C_ccall f14874(C_word c,C_word *av) C_noret;
C_noret_decl(f14881)
static void C_ccall f14881(C_word c,C_word *av) C_noret;
C_noret_decl(f14902)
static void C_ccall f14902(C_word c,C_word *av) C_noret;
C_noret_decl(f_10007)
static void C_fcall f_10007(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10363)
static void C_ccall f_10363(C_word c,C_word *av) C_noret;
C_noret_decl(f_10366)
static void C_ccall f_10366(C_word c,C_word *av) C_noret;
C_noret_decl(f_10369)
static void C_ccall f_10369(C_word c,C_word *av) C_noret;
C_noret_decl(f_10372)
static void C_ccall f_10372(C_word c,C_word *av) C_noret;
C_noret_decl(f_10375)
static void C_ccall f_10375(C_word c,C_word *av) C_noret;
C_noret_decl(f_10378)
static void C_ccall f_10378(C_word c,C_word *av) C_noret;
C_noret_decl(f_10387)
static void C_ccall f_10387(C_word c,C_word *av) C_noret;
C_noret_decl(f_10390)
static void C_ccall f_10390(C_word c,C_word *av) C_noret;
C_noret_decl(f_10393)
static void C_ccall f_10393(C_word c,C_word *av) C_noret;
C_noret_decl(f_10396)
static void C_ccall f_10396(C_word c,C_word *av) C_noret;
C_noret_decl(f_10399)
static void C_ccall f_10399(C_word c,C_word *av) C_noret;
C_noret_decl(f_10402)
static void C_ccall f_10402(C_word c,C_word *av) C_noret;
C_noret_decl(f_10405)
static void C_ccall f_10405(C_word c,C_word *av) C_noret;
C_noret_decl(f_10424)
static void C_ccall f_10424(C_word c,C_word *av) C_noret;
C_noret_decl(f_10426)
static void C_fcall f_10426(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10436)
static void C_ccall f_10436(C_word c,C_word *av) C_noret;
C_noret_decl(f_10475)
static void C_ccall f_10475(C_word c,C_word *av) C_noret;
C_noret_decl(f_10477)
static void C_fcall f_10477(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10499)
static void C_ccall f_10499(C_word c,C_word *av) C_noret;
C_noret_decl(f_10511)
static void C_fcall f_10511(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10521)
static void C_ccall f_10521(C_word c,C_word *av) C_noret;
C_noret_decl(f_10534)
static void C_fcall f_10534(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10538)
static void C_ccall f_10538(C_word c,C_word *av) C_noret;
C_noret_decl(f_10541)
static void C_ccall f_10541(C_word c,C_word *av) C_noret;
C_noret_decl(f_10544)
static void C_ccall f_10544(C_word c,C_word *av) C_noret;
C_noret_decl(f_10547)
static void C_ccall f_10547(C_word c,C_word *av) C_noret;
C_noret_decl(f_10553)
static void C_ccall f_10553(C_word c,C_word *av) C_noret;
C_noret_decl(f_10556)
static void C_ccall f_10556(C_word c,C_word *av) C_noret;
C_noret_decl(f_10559)
static void C_ccall f_10559(C_word c,C_word *av) C_noret;
C_noret_decl(f_10564)
static void C_fcall f_10564(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10574)
static void C_ccall f_10574(C_word c,C_word *av) C_noret;
C_noret_decl(f_10577)
static void C_ccall f_10577(C_word c,C_word *av) C_noret;
C_noret_decl(f_10596)
static void C_ccall f_10596(C_word c,C_word *av) C_noret;
C_noret_decl(f_10608)
static void C_ccall f_10608(C_word c,C_word *av) C_noret;
C_noret_decl(f_10610)
static void C_ccall f_10610(C_word c,C_word *av) C_noret;
C_noret_decl(f_10612)
static void C_fcall f_10612(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10617)
static void C_fcall f_10617(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10687)
static void C_fcall f_10687(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10804)
static void C_fcall f_10804(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10825)
static void C_fcall f_10825(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10957)
static void C_fcall f_10957(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10969)
static void C_fcall f_10969(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10984)
static void C_ccall f_10984(C_word c,C_word *av) C_noret;
C_noret_decl(f_10988)
static void C_fcall f_10988(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11032)
static void C_ccall f_11032(C_word c,C_word *av) C_noret;
C_noret_decl(f_11049)
static void C_ccall f_11049(C_word c,C_word *av) C_noret;
C_noret_decl(f_11062)
static void C_ccall f_11062(C_word c,C_word *av) C_noret;
C_noret_decl(f_11066)
static void C_ccall f_11066(C_word c,C_word *av) C_noret;
C_noret_decl(f_11070)
static void C_ccall f_11070(C_word c,C_word *av) C_noret;
C_noret_decl(f_11088)
static void C_ccall f_11088(C_word c,C_word *av) C_noret;
C_noret_decl(f_11090)
static void C_fcall f_11090(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11115)
static void C_ccall f_11115(C_word c,C_word *av) C_noret;
C_noret_decl(f_11139)
static void C_ccall f_11139(C_word c,C_word *av) C_noret;
C_noret_decl(f_11156)
static void C_ccall f_11156(C_word c,C_word *av) C_noret;
C_noret_decl(f_11173)
static void C_ccall f_11173(C_word c,C_word *av) C_noret;
C_noret_decl(f_11190)
static void C_ccall f_11190(C_word c,C_word *av) C_noret;
C_noret_decl(f_11207)
static void C_ccall f_11207(C_word c,C_word *av) C_noret;
C_noret_decl(f_11224)
static void C_ccall f_11224(C_word c,C_word *av) C_noret;
C_noret_decl(f_11253)
static void C_ccall f_11253(C_word c,C_word *av) C_noret;
C_noret_decl(f_11257)
static void C_ccall f_11257(C_word c,C_word *av) C_noret;
C_noret_decl(f_11278)
static void C_ccall f_11278(C_word c,C_word *av) C_noret;
C_noret_decl(f_11280)
static void C_fcall f_11280(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11290)
static void C_fcall f_11290(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11305)
static void C_ccall f_11305(C_word c,C_word *av) C_noret;
C_noret_decl(f_11704)
static void C_fcall f_11704(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11706)
static void C_fcall f_11706(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11734)
static void C_fcall f_11734(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11761)
static void C_fcall f_11761(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11986)
static void C_fcall f_11986(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11995)
static void C_fcall f_11995(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12004)
static void C_ccall f_12004(C_word c,C_word *av) C_noret;
C_noret_decl(f_12102)
static void C_ccall f_12102(C_word c,C_word *av) C_noret;
C_noret_decl(f_12369)
static void C_fcall f_12369(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12371)
static void C_fcall f_12371(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12438)
static void C_ccall f_12438(C_word c,C_word *av) C_noret;
C_noret_decl(f_12444)
static void C_ccall f_12444(C_word c,C_word *av) C_noret;
C_noret_decl(f_12447)
static void C_ccall f_12447(C_word c,C_word *av) C_noret;
C_noret_decl(f_12450)
static void C_ccall f_12450(C_word c,C_word *av) C_noret;
C_noret_decl(f_12462)
static void C_ccall f_12462(C_word c,C_word *av) C_noret;
C_noret_decl(f_12468)
static void C_ccall f_12468(C_word c,C_word *av) C_noret;
C_noret_decl(f_12471)
static void C_ccall f_12471(C_word c,C_word *av) C_noret;
C_noret_decl(f_12474)
static void C_ccall f_12474(C_word c,C_word *av) C_noret;
C_noret_decl(f_12486)
static void C_fcall f_12486(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12489)
static void C_ccall f_12489(C_word c,C_word *av) C_noret;
C_noret_decl(f_12495)
static void C_ccall f_12495(C_word c,C_word *av) C_noret;
C_noret_decl(f_12498)
static void C_ccall f_12498(C_word c,C_word *av) C_noret;
C_noret_decl(f_12501)
static void C_ccall f_12501(C_word c,C_word *av) C_noret;
C_noret_decl(f_12513)
static void C_ccall f_12513(C_word c,C_word *av) C_noret;
C_noret_decl(f_12519)
static void C_ccall f_12519(C_word c,C_word *av) C_noret;
C_noret_decl(f_12522)
static void C_ccall f_12522(C_word c,C_word *av) C_noret;
C_noret_decl(f_12525)
static void C_ccall f_12525(C_word c,C_word *av) C_noret;
C_noret_decl(f_12540)
static void C_ccall f_12540(C_word c,C_word *av) C_noret;
C_noret_decl(f_12546)
static void C_ccall f_12546(C_word c,C_word *av) C_noret;
C_noret_decl(f_12549)
static void C_ccall f_12549(C_word c,C_word *av) C_noret;
C_noret_decl(f_12552)
static void C_ccall f_12552(C_word c,C_word *av) C_noret;
C_noret_decl(f_12567)
static void C_ccall f_12567(C_word c,C_word *av) C_noret;
C_noret_decl(f_12573)
static void C_ccall f_12573(C_word c,C_word *av) C_noret;
C_noret_decl(f_12576)
static void C_ccall f_12576(C_word c,C_word *av) C_noret;
C_noret_decl(f_12579)
static void C_ccall f_12579(C_word c,C_word *av) C_noret;
C_noret_decl(f_12594)
static void C_ccall f_12594(C_word c,C_word *av) C_noret;
C_noret_decl(f_12600)
static void C_ccall f_12600(C_word c,C_word *av) C_noret;
C_noret_decl(f_12603)
static void C_ccall f_12603(C_word c,C_word *av) C_noret;
C_noret_decl(f_12606)
static void C_ccall f_12606(C_word c,C_word *av) C_noret;
C_noret_decl(f_12621)
static void C_ccall f_12621(C_word c,C_word *av) C_noret;
C_noret_decl(f_12627)
static void C_ccall f_12627(C_word c,C_word *av) C_noret;
C_noret_decl(f_12630)
static void C_ccall f_12630(C_word c,C_word *av) C_noret;
C_noret_decl(f_12633)
static void C_ccall f_12633(C_word c,C_word *av) C_noret;
C_noret_decl(f_12645)
static void C_ccall f_12645(C_word c,C_word *av) C_noret;
C_noret_decl(f_12651)
static void C_ccall f_12651(C_word c,C_word *av) C_noret;
C_noret_decl(f_12654)
static void C_ccall f_12654(C_word c,C_word *av) C_noret;
C_noret_decl(f_12657)
static void C_ccall f_12657(C_word c,C_word *av) C_noret;
C_noret_decl(f_12669)
static void C_ccall f_12669(C_word c,C_word *av) C_noret;
C_noret_decl(f_12675)
static void C_ccall f_12675(C_word c,C_word *av) C_noret;
C_noret_decl(f_12678)
static void C_ccall f_12678(C_word c,C_word *av) C_noret;
C_noret_decl(f_12681)
static void C_ccall f_12681(C_word c,C_word *av) C_noret;
C_noret_decl(f_12702)
static void C_ccall f_12702(C_word c,C_word *av) C_noret;
C_noret_decl(f_12706)
static void C_fcall f_12706(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12738)
static void C_ccall f_12738(C_word c,C_word *av) C_noret;
C_noret_decl(f_12744)
static void C_ccall f_12744(C_word c,C_word *av) C_noret;
C_noret_decl(f_12747)
static void C_ccall f_12747(C_word c,C_word *av) C_noret;
C_noret_decl(f_12750)
static void C_ccall f_12750(C_word c,C_word *av) C_noret;
C_noret_decl(f_12762)
static void C_ccall f_12762(C_word c,C_word *av) C_noret;
C_noret_decl(f_12768)
static void C_ccall f_12768(C_word c,C_word *av) C_noret;
C_noret_decl(f_12771)
static void C_ccall f_12771(C_word c,C_word *av) C_noret;
C_noret_decl(f_12774)
static void C_ccall f_12774(C_word c,C_word *av) C_noret;
C_noret_decl(f_12786)
static void C_ccall f_12786(C_word c,C_word *av) C_noret;
C_noret_decl(f_12792)
static void C_ccall f_12792(C_word c,C_word *av) C_noret;
C_noret_decl(f_12795)
static void C_ccall f_12795(C_word c,C_word *av) C_noret;
C_noret_decl(f_12798)
static void C_ccall f_12798(C_word c,C_word *av) C_noret;
C_noret_decl(f_12810)
static void C_ccall f_12810(C_word c,C_word *av) C_noret;
C_noret_decl(f_12816)
static void C_ccall f_12816(C_word c,C_word *av) C_noret;
C_noret_decl(f_12819)
static void C_ccall f_12819(C_word c,C_word *av) C_noret;
C_noret_decl(f_12822)
static void C_ccall f_12822(C_word c,C_word *av) C_noret;
C_noret_decl(f_12834)
static void C_ccall f_12834(C_word c,C_word *av) C_noret;
C_noret_decl(f_12840)
static void C_ccall f_12840(C_word c,C_word *av) C_noret;
C_noret_decl(f_12843)
static void C_ccall f_12843(C_word c,C_word *av) C_noret;
C_noret_decl(f_12846)
static void C_ccall f_12846(C_word c,C_word *av) C_noret;
C_noret_decl(f_12874)
static void C_ccall f_12874(C_word c,C_word *av) C_noret;
C_noret_decl(f_12880)
static void C_ccall f_12880(C_word c,C_word *av) C_noret;
C_noret_decl(f_12883)
static void C_ccall f_12883(C_word c,C_word *av) C_noret;
C_noret_decl(f_12886)
static void C_ccall f_12886(C_word c,C_word *av) C_noret;
C_noret_decl(f_12898)
static void C_ccall f_12898(C_word c,C_word *av) C_noret;
C_noret_decl(f_12904)
static void C_ccall f_12904(C_word c,C_word *av) C_noret;
C_noret_decl(f_12907)
static void C_ccall f_12907(C_word c,C_word *av) C_noret;
C_noret_decl(f_12910)
static void C_ccall f_12910(C_word c,C_word *av) C_noret;
C_noret_decl(f_12922)
static void C_ccall f_12922(C_word c,C_word *av) C_noret;
C_noret_decl(f_12928)
static void C_ccall f_12928(C_word c,C_word *av) C_noret;
C_noret_decl(f_12931)
static void C_ccall f_12931(C_word c,C_word *av) C_noret;
C_noret_decl(f_12934)
static void C_ccall f_12934(C_word c,C_word *av) C_noret;
C_noret_decl(f_13156)
static void C_fcall f_13156(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13165)
static void C_fcall f_13165(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13231)
static void C_ccall f_13231(C_word c,C_word *av) C_noret;
C_noret_decl(f_13253)
static void C_ccall f_13253(C_word c,C_word *av) C_noret;
C_noret_decl(f_13281)
static void C_fcall f_13281(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13340)
static void C_ccall f_13340(C_word c,C_word *av) C_noret;
C_noret_decl(f_13347)
static void C_ccall f_13347(C_word c,C_word *av) C_noret;
C_noret_decl(f_13367)
static void C_ccall f_13367(C_word c,C_word *av) C_noret;
C_noret_decl(f_13374)
static void C_ccall f_13374(C_word c,C_word *av) C_noret;
C_noret_decl(f_13387)
static void C_ccall f_13387(C_word c,C_word *av) C_noret;
C_noret_decl(f_13394)
static void C_ccall f_13394(C_word c,C_word *av) C_noret;
C_noret_decl(f_13417)
static void C_ccall f_13417(C_word c,C_word *av) C_noret;
C_noret_decl(f_13425)
static void C_ccall f_13425(C_word c,C_word *av) C_noret;
C_noret_decl(f_13447)
static void C_ccall f_13447(C_word c,C_word *av) C_noret;
C_noret_decl(f_13455)
static void C_ccall f_13455(C_word c,C_word *av) C_noret;
C_noret_decl(f_13459)
static void C_ccall f_13459(C_word c,C_word *av) C_noret;
C_noret_decl(f_13461)
static void C_ccall f_13461(C_word c,C_word *av) C_noret;
C_noret_decl(f_13490)
static void C_ccall f_13490(C_word c,C_word *av) C_noret;
C_noret_decl(f_3245)
static void C_ccall f_3245(C_word c,C_word *av) C_noret;
C_noret_decl(f_3248)
static void C_ccall f_3248(C_word c,C_word *av) C_noret;
C_noret_decl(f_3251)
static void C_ccall f_3251(C_word c,C_word *av) C_noret;
C_noret_decl(f_3254)
static void C_ccall f_3254(C_word c,C_word *av) C_noret;
C_noret_decl(f_3257)
static void C_ccall f_3257(C_word c,C_word *av) C_noret;
C_noret_decl(f_3260)
static void C_ccall f_3260(C_word c,C_word *av) C_noret;
C_noret_decl(f_3263)
static void C_ccall f_3263(C_word c,C_word *av) C_noret;
C_noret_decl(f_3266)
static void C_ccall f_3266(C_word c,C_word *av) C_noret;
C_noret_decl(f_3707)
static void C_fcall f_3707(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_3721)
static void C_ccall f_3721(C_word c,C_word *av) C_noret;
C_noret_decl(f_4279)
static void C_fcall f_4279(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4285)
static void C_fcall f_4285(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4299)
static void C_ccall f_4299(C_word c,C_word *av) C_noret;
C_noret_decl(f_4303)
static void C_ccall f_4303(C_word c,C_word *av) C_noret;
C_noret_decl(f_4459)
static void C_ccall f_4459(C_word c,C_word *av) C_noret;
C_noret_decl(f_4471)
static void C_ccall f_4471(C_word c,C_word *av) C_noret;
C_noret_decl(f_4547)
static void C_ccall f_4547(C_word c,C_word *av) C_noret;
C_noret_decl(f_4567)
static void C_fcall f_4567(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4577)
static void C_ccall f_4577(C_word c,C_word *av) C_noret;
C_noret_decl(f_4590)
static void C_fcall f_4590(C_word t0,C_word t1) C_noret;
C_noret_decl(f_4599)
static void C_ccall f_4599(C_word c,C_word *av) C_noret;
C_noret_decl(f_4604)
static void C_fcall f_4604(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4614)
static void C_ccall f_4614(C_word c,C_word *av) C_noret;
C_noret_decl(f_4655)
static void C_ccall f_4655(C_word c,C_word *av) C_noret;
C_noret_decl(f_4661)
static void C_ccall f_4661(C_word c,C_word *av) C_noret;
C_noret_decl(f_4666)
static void C_ccall f_4666(C_word c,C_word *av) C_noret;
C_noret_decl(f_4674)
static void C_ccall f_4674(C_word c,C_word *av) C_noret;
C_noret_decl(f_4678)
static void C_ccall f_4678(C_word c,C_word *av) C_noret;
C_noret_decl(f_4688)
static void C_ccall f_4688(C_word c,C_word *av) C_noret;
C_noret_decl(f_4699)
static void C_ccall f_4699(C_word c,C_word *av) C_noret;
C_noret_decl(f_4703)
static void C_ccall f_4703(C_word c,C_word *av) C_noret;
C_noret_decl(f_4706)
static void C_ccall f_4706(C_word c,C_word *av) C_noret;
C_noret_decl(f_4708)
static void C_fcall f_4708(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4712)
static void C_ccall f_4712(C_word c,C_word *av) C_noret;
C_noret_decl(f_4720)
static void C_fcall f_4720(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_4723)
static void C_fcall f_4723(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4727)
static void C_ccall f_4727(C_word c,C_word *av) C_noret;
C_noret_decl(f_4730)
static void C_ccall f_4730(C_word c,C_word *av) C_noret;
C_noret_decl(f_4733)
static void C_ccall f_4733(C_word c,C_word *av) C_noret;
C_noret_decl(f_4860)
static void C_ccall f_4860(C_word c,C_word *av) C_noret;
C_noret_decl(f_4874)
static void C_ccall f_4874(C_word c,C_word *av) C_noret;
C_noret_decl(f_4893)
static void C_ccall f_4893(C_word c,C_word *av) C_noret;
C_noret_decl(f_4896)
static void C_ccall f_4896(C_word c,C_word *av) C_noret;
C_noret_decl(f_4899)
static void C_ccall f_4899(C_word c,C_word *av) C_noret;
C_noret_decl(f_4902)
static void C_ccall f_4902(C_word c,C_word *av) C_noret;
C_noret_decl(f_4905)
static void C_ccall f_4905(C_word c,C_word *av) C_noret;
C_noret_decl(f_4908)
static void C_ccall f_4908(C_word c,C_word *av) C_noret;
C_noret_decl(f_4951)
static void C_fcall f_4951(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_4961)
static void C_ccall f_4961(C_word c,C_word *av) C_noret;
C_noret_decl(f_4964)
static void C_ccall f_4964(C_word c,C_word *av) C_noret;
C_noret_decl(f_4967)
static void C_ccall f_4967(C_word c,C_word *av) C_noret;
C_noret_decl(f_4999)
static void C_ccall f_4999(C_word c,C_word *av) C_noret;
C_noret_decl(f_5002)
static void C_ccall f_5002(C_word c,C_word *av) C_noret;
C_noret_decl(f_5005)
static void C_ccall f_5005(C_word c,C_word *av) C_noret;
C_noret_decl(f_5038)
static void C_ccall f_5038(C_word c,C_word *av) C_noret;
C_noret_decl(f_5041)
static void C_ccall f_5041(C_word c,C_word *av) C_noret;
C_noret_decl(f_5057)
static void C_ccall f_5057(C_word c,C_word *av) C_noret;
C_noret_decl(f_5060)
static void C_ccall f_5060(C_word c,C_word *av) C_noret;
C_noret_decl(f_5076)
static void C_ccall f_5076(C_word c,C_word *av) C_noret;
C_noret_decl(f_5079)
static void C_ccall f_5079(C_word c,C_word *av) C_noret;
C_noret_decl(f_5103)
static void C_ccall f_5103(C_word c,C_word *av) C_noret;
C_noret_decl(f_5109)
static void C_fcall f_5109(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5128)
static void C_ccall f_5128(C_word c,C_word *av) C_noret;
C_noret_decl(f_5131)
static void C_ccall f_5131(C_word c,C_word *av) C_noret;
C_noret_decl(f_5143)
static void C_ccall f_5143(C_word c,C_word *av) C_noret;
C_noret_decl(f_5149)
static void C_fcall f_5149(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5168)
static void C_ccall f_5168(C_word c,C_word *av) C_noret;
C_noret_decl(f_5171)
static void C_ccall f_5171(C_word c,C_word *av) C_noret;
C_noret_decl(f_5183)
static void C_ccall f_5183(C_word c,C_word *av) C_noret;
C_noret_decl(f_5189)
static void C_fcall f_5189(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5208)
static void C_ccall f_5208(C_word c,C_word *av) C_noret;
C_noret_decl(f_5211)
static void C_ccall f_5211(C_word c,C_word *av) C_noret;
C_noret_decl(f_5223)
static void C_ccall f_5223(C_word c,C_word *av) C_noret;
C_noret_decl(f_5226)
static void C_ccall f_5226(C_word c,C_word *av) C_noret;
C_noret_decl(f_5242)
static void C_ccall f_5242(C_word c,C_word *av) C_noret;
C_noret_decl(f_5245)
static void C_ccall f_5245(C_word c,C_word *av) C_noret;
C_noret_decl(f_5248)
static void C_ccall f_5248(C_word c,C_word *av) C_noret;
C_noret_decl(f_5251)
static void C_ccall f_5251(C_word c,C_word *av) C_noret;
C_noret_decl(f_5275)
static void C_ccall f_5275(C_word c,C_word *av) C_noret;
C_noret_decl(f_5278)
static void C_ccall f_5278(C_word c,C_word *av) C_noret;
C_noret_decl(f_5281)
static void C_ccall f_5281(C_word c,C_word *av) C_noret;
C_noret_decl(f_5284)
static void C_ccall f_5284(C_word c,C_word *av) C_noret;
C_noret_decl(f_5312)
static void C_ccall f_5312(C_word c,C_word *av) C_noret;
C_noret_decl(f_5315)
static void C_ccall f_5315(C_word c,C_word *av) C_noret;
C_noret_decl(f_5318)
static void C_ccall f_5318(C_word c,C_word *av) C_noret;
C_noret_decl(f_5321)
static void C_ccall f_5321(C_word c,C_word *av) C_noret;
C_noret_decl(f_5341)
static void C_ccall f_5341(C_word c,C_word *av) C_noret;
C_noret_decl(f_5344)
static void C_ccall f_5344(C_word c,C_word *av) C_noret;
C_noret_decl(f_5347)
static void C_ccall f_5347(C_word c,C_word *av) C_noret;
C_noret_decl(f_5350)
static void C_ccall f_5350(C_word c,C_word *av) C_noret;
C_noret_decl(f_5373)
static void C_ccall f_5373(C_word c,C_word *av) C_noret;
C_noret_decl(f_5374)
static void C_fcall f_5374(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5378)
static void C_ccall f_5378(C_word c,C_word *av) C_noret;
C_noret_decl(f_5381)
static void C_ccall f_5381(C_word c,C_word *av) C_noret;
C_noret_decl(f_5387)
static void C_ccall f_5387(C_word c,C_word *av) C_noret;
C_noret_decl(f_5396)
static void C_ccall f_5396(C_word c,C_word *av) C_noret;
C_noret_decl(f_5403)
static void C_fcall f_5403(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5413)
static void C_ccall f_5413(C_word c,C_word *av) C_noret;
C_noret_decl(f_5447)
static void C_ccall f_5447(C_word c,C_word *av) C_noret;
C_noret_decl(f_5450)
static void C_ccall f_5450(C_word c,C_word *av) C_noret;
C_noret_decl(f_5479)
static void C_ccall f_5479(C_word c,C_word *av) C_noret;
C_noret_decl(f_5521)
static void C_ccall f_5521(C_word c,C_word *av) C_noret;
C_noret_decl(f_5525)
static void C_ccall f_5525(C_word c,C_word *av) C_noret;
C_noret_decl(f_5556)
static void C_ccall f_5556(C_word c,C_word *av) C_noret;
C_noret_decl(f_5559)
static void C_ccall f_5559(C_word c,C_word *av) C_noret;
C_noret_decl(f_5562)
static void C_ccall f_5562(C_word c,C_word *av) C_noret;
C_noret_decl(f_5573)
static void C_ccall f_5573(C_word c,C_word *av) C_noret;
C_noret_decl(f_5577)
static void C_ccall f_5577(C_word c,C_word *av) C_noret;
C_noret_decl(f_5604)
static void C_ccall f_5604(C_word c,C_word *av) C_noret;
C_noret_decl(f_5607)
static void C_ccall f_5607(C_word c,C_word *av) C_noret;
C_noret_decl(f_5618)
static void C_ccall f_5618(C_word c,C_word *av) C_noret;
C_noret_decl(f_5622)
static void C_ccall f_5622(C_word c,C_word *av) C_noret;
C_noret_decl(f_5625)
static void C_ccall f_5625(C_word c,C_word *av) C_noret;
C_noret_decl(f_5628)
static void C_ccall f_5628(C_word c,C_word *av) C_noret;
C_noret_decl(f_5639)
static void C_ccall f_5639(C_word c,C_word *av) C_noret;
C_noret_decl(f_5643)
static void C_ccall f_5643(C_word c,C_word *av) C_noret;
C_noret_decl(f_5680)
static void C_ccall f_5680(C_word c,C_word *av) C_noret;
C_noret_decl(f_5683)
static void C_fcall f_5683(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5689)
static void C_fcall f_5689(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5693)
static void C_ccall f_5693(C_word c,C_word *av) C_noret;
C_noret_decl(f_5702)
static void C_ccall f_5702(C_word c,C_word *av) C_noret;
C_noret_decl(f_5705)
static void C_ccall f_5705(C_word c,C_word *av) C_noret;
C_noret_decl(f_5708)
static void C_ccall f_5708(C_word c,C_word *av) C_noret;
C_noret_decl(f_5724)
static void C_ccall f_5724(C_word c,C_word *av) C_noret;
C_noret_decl(f_5727)
static void C_ccall f_5727(C_word c,C_word *av) C_noret;
C_noret_decl(f_5730)
static void C_ccall f_5730(C_word c,C_word *av) C_noret;
C_noret_decl(f_5731)
static void C_fcall f_5731(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5735)
static void C_ccall f_5735(C_word c,C_word *av) C_noret;
C_noret_decl(f_5738)
static void C_ccall f_5738(C_word c,C_word *av) C_noret;
C_noret_decl(f_5747)
static void C_ccall f_5747(C_word c,C_word *av) C_noret;
C_noret_decl(f_5755)
static void C_ccall f_5755(C_word c,C_word *av) C_noret;
C_noret_decl(f_5761)
static void C_ccall f_5761(C_word c,C_word *av) C_noret;
C_noret_decl(f_5764)
static void C_ccall f_5764(C_word c,C_word *av) C_noret;
C_noret_decl(f_5772)
static void C_fcall f_5772(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5782)
static void C_ccall f_5782(C_word c,C_word *av) C_noret;
C_noret_decl(f_5809)
static void C_fcall f_5809(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5819)
static void C_ccall f_5819(C_word c,C_word *av) C_noret;
C_noret_decl(f_5846)
static void C_ccall f_5846(C_word c,C_word *av) C_noret;
C_noret_decl(f_5853)
static void C_ccall f_5853(C_word c,C_word *av) C_noret;
C_noret_decl(f_5859)
static void C_ccall f_5859(C_word c,C_word *av) C_noret;
C_noret_decl(f_5862)
static void C_ccall f_5862(C_word c,C_word *av) C_noret;
C_noret_decl(f_5865)
static void C_ccall f_5865(C_word c,C_word *av) C_noret;
C_noret_decl(f_5874)
static void C_ccall f_5874(C_word c,C_word *av) C_noret;
C_noret_decl(f_5877)
static void C_ccall f_5877(C_word c,C_word *av) C_noret;
C_noret_decl(f_5880)
static void C_ccall f_5880(C_word c,C_word *av) C_noret;
C_noret_decl(f_5883)
static void C_ccall f_5883(C_word c,C_word *av) C_noret;
C_noret_decl(f_5893)
static void C_ccall f_5893(C_word c,C_word *av) C_noret;
C_noret_decl(f_5904)
static void C_ccall f_5904(C_word c,C_word *av) C_noret;
C_noret_decl(f_5908)
static void C_ccall f_5908(C_word c,C_word *av) C_noret;
C_noret_decl(f_5911)
static void C_ccall f_5911(C_word c,C_word *av) C_noret;
C_noret_decl(f_5924)
static void C_ccall f_5924(C_word c,C_word *av) C_noret;
C_noret_decl(f_5930)
static void C_fcall f_5930(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5933)
static void C_ccall f_5933(C_word c,C_word *av) C_noret;
C_noret_decl(f_5945)
static void C_ccall f_5945(C_word c,C_word *av) C_noret;
C_noret_decl(f_5948)
static void C_ccall f_5948(C_word c,C_word *av) C_noret;
C_noret_decl(f_5951)
static void C_ccall f_5951(C_word c,C_word *av) C_noret;
C_noret_decl(f_5954)
static void C_ccall f_5954(C_word c,C_word *av) C_noret;
C_noret_decl(f_5961)
static void C_ccall f_5961(C_word c,C_word *av) C_noret;
C_noret_decl(f_5971)
static void C_ccall f_5971(C_word c,C_word *av) C_noret;
C_noret_decl(f_5979)
static void C_ccall f_5979(C_word c,C_word *av) C_noret;
C_noret_decl(f_5987)
static void C_ccall f_5987(C_word c,C_word *av) C_noret;
C_noret_decl(f_6000)
static void C_ccall f_6000(C_word c,C_word *av) C_noret;
C_noret_decl(f_6004)
static void C_ccall f_6004(C_word c,C_word *av) C_noret;
C_noret_decl(f_6012)
static void C_ccall f_6012(C_word c,C_word *av) C_noret;
C_noret_decl(f_6020)
static void C_ccall f_6020(C_word c,C_word *av) C_noret;
C_noret_decl(f_6027)
static void C_ccall f_6027(C_word c,C_word *av) C_noret;
C_noret_decl(f_6032)
static void C_ccall f_6032(C_word c,C_word *av) C_noret;
C_noret_decl(f_6039)
static void C_ccall f_6039(C_word c,C_word *av) C_noret;
C_noret_decl(f_6043)
static void C_ccall f_6043(C_word c,C_word *av) C_noret;
C_noret_decl(f_6046)
static void C_ccall f_6046(C_word c,C_word *av) C_noret;
C_noret_decl(f_6049)
static void C_ccall f_6049(C_word c,C_word *av) C_noret;
C_noret_decl(f_6052)
static void C_ccall f_6052(C_word c,C_word *av) C_noret;
C_noret_decl(f_6055)
static void C_ccall f_6055(C_word c,C_word *av) C_noret;
C_noret_decl(f_6058)
static void C_ccall f_6058(C_word c,C_word *av) C_noret;
C_noret_decl(f_6064)
static void C_fcall f_6064(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6077)
static void C_ccall f_6077(C_word c,C_word *av) C_noret;
C_noret_decl(f_6082)
static void C_ccall f_6082(C_word c,C_word *av) C_noret;
C_noret_decl(f_6105)
static void C_ccall f_6105(C_word c,C_word *av) C_noret;
C_noret_decl(f_6109)
static void C_ccall f_6109(C_word c,C_word *av) C_noret;
C_noret_decl(f_6112)
static void C_ccall f_6112(C_word c,C_word *av) C_noret;
C_noret_decl(f_6132)
static void C_ccall f_6132(C_word c,C_word *av) C_noret;
C_noret_decl(f_6139)
static void C_ccall f_6139(C_word c,C_word *av) C_noret;
C_noret_decl(f_6143)
static void C_ccall f_6143(C_word c,C_word *av) C_noret;
C_noret_decl(f_6147)
static void C_ccall f_6147(C_word c,C_word *av) C_noret;
C_noret_decl(f_6190)
static void C_ccall f_6190(C_word c,C_word *av) C_noret;
C_noret_decl(f_6196)
static void C_ccall f_6196(C_word c,C_word *av) C_noret;
C_noret_decl(f_6199)
static void C_ccall f_6199(C_word c,C_word *av) C_noret;
C_noret_decl(f_6200)
static void C_fcall f_6200(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6204)
static void C_ccall f_6204(C_word c,C_word *av) C_noret;
C_noret_decl(f_6207)
static void C_ccall f_6207(C_word c,C_word *av) C_noret;
C_noret_decl(f_6216)
static void C_ccall f_6216(C_word c,C_word *av) C_noret;
C_noret_decl(f_6224)
static void C_ccall f_6224(C_word c,C_word *av) C_noret;
C_noret_decl(f_6230)
static void C_ccall f_6230(C_word c,C_word *av) C_noret;
C_noret_decl(f_6235)
static void C_fcall f_6235(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6245)
static void C_ccall f_6245(C_word c,C_word *av) C_noret;
C_noret_decl(f_6272)
static void C_fcall f_6272(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6282)
static void C_ccall f_6282(C_word c,C_word *av) C_noret;
C_noret_decl(f_6309)
static void C_ccall f_6309(C_word c,C_word *av) C_noret;
C_noret_decl(f_6316)
static void C_ccall f_6316(C_word c,C_word *av) C_noret;
C_noret_decl(f_6319)
static void C_ccall f_6319(C_word c,C_word *av) C_noret;
C_noret_decl(f_6322)
static void C_ccall f_6322(C_word c,C_word *av) C_noret;
C_noret_decl(f_6350)
static void C_ccall f_6350(C_word c,C_word *av) C_noret;
C_noret_decl(f_6363)
static void C_ccall f_6363(C_word c,C_word *av) C_noret;
C_noret_decl(f_6367)
static void C_ccall f_6367(C_word c,C_word *av) C_noret;
C_noret_decl(f_6370)
static void C_ccall f_6370(C_word c,C_word *av) C_noret;
C_noret_decl(f_6373)
static void C_ccall f_6373(C_word c,C_word *av) C_noret;
C_noret_decl(f_6376)
static void C_ccall f_6376(C_word c,C_word *av) C_noret;
C_noret_decl(f_6379)
static void C_ccall f_6379(C_word c,C_word *av) C_noret;
C_noret_decl(f_6382)
static void C_ccall f_6382(C_word c,C_word *av) C_noret;
C_noret_decl(f_6385)
static void C_ccall f_6385(C_word c,C_word *av) C_noret;
C_noret_decl(f_6397)
static void C_ccall f_6397(C_word c,C_word *av) C_noret;
C_noret_decl(f_6409)
static void C_ccall f_6409(C_word c,C_word *av) C_noret;
C_noret_decl(f_6427)
static void C_ccall f_6427(C_word c,C_word *av) C_noret;
C_noret_decl(f_6447)
static void C_ccall f_6447(C_word c,C_word *av) C_noret;
C_noret_decl(f_6454)
static void C_ccall f_6454(C_word c,C_word *av) C_noret;
C_noret_decl(f_6458)
static void C_ccall f_6458(C_word c,C_word *av) C_noret;
C_noret_decl(f_6487)
static void C_ccall f_6487(C_word c,C_word *av) C_noret;
C_noret_decl(f_6490)
static void C_ccall f_6490(C_word c,C_word *av) C_noret;
C_noret_decl(f_6497)
static void C_ccall f_6497(C_word c,C_word *av) C_noret;
C_noret_decl(f_6510)
static void C_ccall f_6510(C_word c,C_word *av) C_noret;
C_noret_decl(f_6513)
static void C_ccall f_6513(C_word c,C_word *av) C_noret;
C_noret_decl(f_6529)
static void C_ccall f_6529(C_word c,C_word *av) C_noret;
C_noret_decl(f_6532)
static void C_ccall f_6532(C_word c,C_word *av) C_noret;
C_noret_decl(f_6565)
static void C_ccall f_6565(C_word c,C_word *av) C_noret;
C_noret_decl(f_6568)
static void C_ccall f_6568(C_word c,C_word *av) C_noret;
C_noret_decl(f_6577)
static void C_ccall f_6577(C_word c,C_word *av) C_noret;
C_noret_decl(f_6601)
static void C_ccall f_6601(C_word c,C_word *av) C_noret;
C_noret_decl(f_6621)
static void C_ccall f_6621(C_word c,C_word *av) C_noret;
C_noret_decl(f_6624)
static void C_ccall f_6624(C_word c,C_word *av) C_noret;
C_noret_decl(f_6639)
static void C_ccall f_6639(C_word c,C_word *av) C_noret;
C_noret_decl(f_6643)
static void C_ccall f_6643(C_word c,C_word *av) C_noret;
C_noret_decl(f_6655)
static void C_ccall f_6655(C_word c,C_word *av) C_noret;
C_noret_decl(f_6658)
static void C_ccall f_6658(C_word c,C_word *av) C_noret;
C_noret_decl(f_6669)
static void C_ccall f_6669(C_word c,C_word *av) C_noret;
C_noret_decl(f_6673)
static void C_ccall f_6673(C_word c,C_word *av) C_noret;
C_noret_decl(f_6685)
static void C_ccall f_6685(C_word c,C_word *av) C_noret;
C_noret_decl(f_6688)
static void C_ccall f_6688(C_word c,C_word *av) C_noret;
C_noret_decl(f_6691)
static void C_ccall f_6691(C_word c,C_word *av) C_noret;
C_noret_decl(f_6694)
static void C_ccall f_6694(C_word c,C_word *av) C_noret;
C_noret_decl(f_6705)
static void C_ccall f_6705(C_word c,C_word *av) C_noret;
C_noret_decl(f_6713)
static void C_ccall f_6713(C_word c,C_word *av) C_noret;
C_noret_decl(f_6722)
static void C_ccall f_6722(C_word c,C_word *av) C_noret;
C_noret_decl(f_6725)
static void C_ccall f_6725(C_word c,C_word *av) C_noret;
C_noret_decl(f_6728)
static void C_ccall f_6728(C_word c,C_word *av) C_noret;
C_noret_decl(f_6741)
static void C_fcall f_6741(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6749)
static void C_ccall f_6749(C_word c,C_word *av) C_noret;
C_noret_decl(f_6752)
static void C_ccall f_6752(C_word c,C_word *av) C_noret;
C_noret_decl(f_6762)
static void C_ccall f_6762(C_word c,C_word *av) C_noret;
C_noret_decl(f_6765)
static void C_ccall f_6765(C_word c,C_word *av) C_noret;
C_noret_decl(f_6768)
static void C_ccall f_6768(C_word c,C_word *av) C_noret;
C_noret_decl(f_6771)
static void C_ccall f_6771(C_word c,C_word *av) C_noret;
C_noret_decl(f_6801)
static void C_ccall f_6801(C_word c,C_word *av) C_noret;
C_noret_decl(f_6804)
static void C_ccall f_6804(C_word c,C_word *av) C_noret;
C_noret_decl(f_6807)
static void C_ccall f_6807(C_word c,C_word *av) C_noret;
C_noret_decl(f_6810)
static void C_ccall f_6810(C_word c,C_word *av) C_noret;
C_noret_decl(f_6813)
static void C_ccall f_6813(C_word c,C_word *av) C_noret;
C_noret_decl(f_6816)
static void C_ccall f_6816(C_word c,C_word *av) C_noret;
C_noret_decl(f_6838)
static void C_ccall f_6838(C_word c,C_word *av) C_noret;
C_noret_decl(f_6924)
static void C_fcall f_6924(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6930)
static void C_fcall f_6930(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6940)
static void C_ccall f_6940(C_word c,C_word *av) C_noret;
C_noret_decl(f_6943)
static void C_ccall f_6943(C_word c,C_word *av) C_noret;
C_noret_decl(f_6960)
static void C_fcall f_6960(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6966)
static void C_fcall f_6966(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6985)
static void C_ccall f_6985(C_word c,C_word *av) C_noret;
C_noret_decl(f_6997)
static void C_ccall f_6997(C_word c,C_word *av) C_noret;
C_noret_decl(f_7001)
static void C_ccall f_7001(C_word c,C_word *av) C_noret;
C_noret_decl(f_7003)
static void C_fcall f_7003(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_7015)
static void C_fcall f_7015(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7018)
static void C_ccall f_7018(C_word c,C_word *av) C_noret;
C_noret_decl(f_7021)
static void C_ccall f_7021(C_word c,C_word *av) C_noret;
C_noret_decl(f_7024)
static void C_ccall f_7024(C_word c,C_word *av) C_noret;
C_noret_decl(f_7027)
static void C_ccall f_7027(C_word c,C_word *av) C_noret;
C_noret_decl(f_7036)
static void C_fcall f_7036(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7046)
static void C_ccall f_7046(C_word c,C_word *av) C_noret;
C_noret_decl(f_7049)
static void C_ccall f_7049(C_word c,C_word *av) C_noret;
C_noret_decl(f_7052)
static void C_ccall f_7052(C_word c,C_word *av) C_noret;
C_noret_decl(f_7072)
static void C_ccall f_7072(C_word c,C_word *av) C_noret;
C_noret_decl(f_7087)
static void C_ccall f_7087(C_word c,C_word *av) C_noret;
C_noret_decl(f_7090)
static void C_ccall f_7090(C_word c,C_word *av) C_noret;
C_noret_decl(f_7093)
static void C_ccall f_7093(C_word c,C_word *av) C_noret;
C_noret_decl(f_7096)
static void C_ccall f_7096(C_word c,C_word *av) C_noret;
C_noret_decl(f_7099)
static void C_ccall f_7099(C_word c,C_word *av) C_noret;
C_noret_decl(f_7121)
static void C_ccall f_7121(C_word c,C_word *av) C_noret;
C_noret_decl(f_7124)
static void C_ccall f_7124(C_word c,C_word *av) C_noret;
C_noret_decl(f_7132)
static void C_fcall f_7132(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7136)
static void C_ccall f_7136(C_word c,C_word *av) C_noret;
C_noret_decl(f_7139)
static void C_ccall f_7139(C_word c,C_word *av) C_noret;
C_noret_decl(f_7142)
static void C_ccall f_7142(C_word c,C_word *av) C_noret;
C_noret_decl(f_7145)
static void C_ccall f_7145(C_word c,C_word *av) C_noret;
C_noret_decl(f_7148)
static void C_ccall f_7148(C_word c,C_word *av) C_noret;
C_noret_decl(f_7151)
static void C_ccall f_7151(C_word c,C_word *av) C_noret;
C_noret_decl(f_7154)
static void C_ccall f_7154(C_word c,C_word *av) C_noret;
C_noret_decl(f_7157)
static void C_ccall f_7157(C_word c,C_word *av) C_noret;
C_noret_decl(f_7169)
static void C_ccall f_7169(C_word c,C_word *av) C_noret;
C_noret_decl(f_7179)
static void C_fcall f_7179(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7189)
static void C_ccall f_7189(C_word c,C_word *av) C_noret;
C_noret_decl(f_7209)
static void C_ccall f_7209(C_word c,C_word *av) C_noret;
C_noret_decl(f_7222)
static void C_ccall f_7222(C_word c,C_word *av) C_noret;
C_noret_decl(f_7234)
static void C_ccall f_7234(C_word c,C_word *av) C_noret;
C_noret_decl(f_7237)
static void C_ccall f_7237(C_word c,C_word *av) C_noret;
C_noret_decl(f_7239)
static void C_fcall f_7239(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7264)
static void C_ccall f_7264(C_word c,C_word *av) C_noret;
C_noret_decl(f_7275)
static void C_ccall f_7275(C_word c,C_word *av) C_noret;
C_noret_decl(f_7285)
static void C_ccall f_7285(C_word c,C_word *av) C_noret;
C_noret_decl(f_7289)
static void C_ccall f_7289(C_word c,C_word *av) C_noret;
C_noret_decl(f_7291)
static void C_fcall f_7291(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7298)
static void C_ccall f_7298(C_word c,C_word *av) C_noret;
C_noret_decl(f_7312)
static void C_ccall f_7312(C_word c,C_word *av) C_noret;
C_noret_decl(f_7318)
static void C_ccall f_7318(C_word c,C_word *av) C_noret;
C_noret_decl(f_7321)
static void C_ccall f_7321(C_word c,C_word *av) C_noret;
C_noret_decl(f_7324)
static void C_ccall f_7324(C_word c,C_word *av) C_noret;
C_noret_decl(f_7329)
static void C_fcall f_7329(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7339)
static void C_ccall f_7339(C_word c,C_word *av) C_noret;
C_noret_decl(f_7345)
static void C_ccall f_7345(C_word c,C_word *av) C_noret;
C_noret_decl(f_7348)
static void C_ccall f_7348(C_word c,C_word *av) C_noret;
C_noret_decl(f_7351)
static void C_ccall f_7351(C_word c,C_word *av) C_noret;
C_noret_decl(f_7354)
static void C_ccall f_7354(C_word c,C_word *av) C_noret;
C_noret_decl(f_7369)
static void C_fcall f_7369(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7377)
static void C_ccall f_7377(C_word c,C_word *av) C_noret;
C_noret_decl(f_7387)
static void C_fcall f_7387(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7397)
static void C_ccall f_7397(C_word c,C_word *av) C_noret;
C_noret_decl(f_7438)
static void C_fcall f_7438(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7448)
static void C_ccall f_7448(C_word c,C_word *av) C_noret;
C_noret_decl(f_7461)
static void C_fcall f_7461(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7486)
static void C_ccall f_7486(C_word c,C_word *av) C_noret;
C_noret_decl(f_7495)
static void C_fcall f_7495(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7499)
static void C_ccall f_7499(C_word c,C_word *av) C_noret;
C_noret_decl(f_7508)
static void C_ccall f_7508(C_word c,C_word *av) C_noret;
C_noret_decl(f_7511)
static void C_ccall f_7511(C_word c,C_word *av) C_noret;
C_noret_decl(f_7514)
static void C_fcall f_7514(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7517)
static void C_ccall f_7517(C_word c,C_word *av) C_noret;
C_noret_decl(f_7520)
static void C_ccall f_7520(C_word c,C_word *av) C_noret;
C_noret_decl(f_7523)
static void C_ccall f_7523(C_word c,C_word *av) C_noret;
C_noret_decl(f_7526)
static void C_ccall f_7526(C_word c,C_word *av) C_noret;
C_noret_decl(f_7529)
static void C_ccall f_7529(C_word c,C_word *av) C_noret;
C_noret_decl(f_7532)
static void C_ccall f_7532(C_word c,C_word *av) C_noret;
C_noret_decl(f_7535)
static void C_ccall f_7535(C_word c,C_word *av) C_noret;
C_noret_decl(f_7538)
static void C_ccall f_7538(C_word c,C_word *av) C_noret;
C_noret_decl(f_7541)
static void C_ccall f_7541(C_word c,C_word *av) C_noret;
C_noret_decl(f_7544)
static void C_ccall f_7544(C_word c,C_word *av) C_noret;
C_noret_decl(f_7547)
static void C_ccall f_7547(C_word c,C_word *av) C_noret;
C_noret_decl(f_7568)
static void C_ccall f_7568(C_word c,C_word *av) C_noret;
C_noret_decl(f_7591)
static void C_ccall f_7591(C_word c,C_word *av) C_noret;
C_noret_decl(f_7594)
static void C_ccall f_7594(C_word c,C_word *av) C_noret;
C_noret_decl(f_7597)
static void C_ccall f_7597(C_word c,C_word *av) C_noret;
C_noret_decl(f_7600)
static void C_ccall f_7600(C_word c,C_word *av) C_noret;
C_noret_decl(f_7616)
static void C_ccall f_7616(C_word c,C_word *av) C_noret;
C_noret_decl(f_7619)
static void C_ccall f_7619(C_word c,C_word *av) C_noret;
C_noret_decl(f_7622)
static void C_ccall f_7622(C_word c,C_word *av) C_noret;
C_noret_decl(f_7633)
static void C_ccall f_7633(C_word c,C_word *av) C_noret;
C_noret_decl(f_7642)
static void C_ccall f_7642(C_word c,C_word *av) C_noret;
C_noret_decl(f_7650)
static void C_fcall f_7650(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7660)
static void C_ccall f_7660(C_word c,C_word *av) C_noret;
C_noret_decl(f_7673)
static void C_fcall f_7673(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7684)
static void C_fcall f_7684(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7694)
static void C_ccall f_7694(C_word c,C_word *av) C_noret;
C_noret_decl(f_7710)
static void C_ccall f_7710(C_word c,C_word *av) C_noret;
C_noret_decl(f_7713)
static void C_ccall f_7713(C_word c,C_word *av) C_noret;
C_noret_decl(f_7716)
static void C_fcall f_7716(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7719)
static void C_fcall f_7719(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7728)
static void C_ccall f_7728(C_word c,C_word *av) C_noret;
C_noret_decl(f_7731)
static void C_ccall f_7731(C_word c,C_word *av) C_noret;
C_noret_decl(f_7734)
static void C_ccall f_7734(C_word c,C_word *av) C_noret;
C_noret_decl(f_7737)
static void C_ccall f_7737(C_word c,C_word *av) C_noret;
C_noret_decl(f_7740)
static void C_ccall f_7740(C_word c,C_word *av) C_noret;
C_noret_decl(f_7743)
static void C_ccall f_7743(C_word c,C_word *av) C_noret;
C_noret_decl(f_7750)
static void C_ccall f_7750(C_word c,C_word *av) C_noret;
C_noret_decl(f_7757)
static void C_ccall f_7757(C_word c,C_word *av) C_noret;
C_noret_decl(f_7763)
static void C_ccall f_7763(C_word c,C_word *av) C_noret;
C_noret_decl(f_7771)
static void C_fcall f_7771(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7781)
static void C_ccall f_7781(C_word c,C_word *av) C_noret;
C_noret_decl(f_7794)
static void C_fcall f_7794(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7800)
static void C_fcall f_7800(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7810)
static void C_ccall f_7810(C_word c,C_word *av) C_noret;
C_noret_decl(f_7825)
static void C_ccall f_7825(C_word c,C_word *av) C_noret;
C_noret_decl(f_7831)
static void C_ccall f_7831(C_word c,C_word *av) C_noret;
C_noret_decl(f_7834)
static void C_ccall f_7834(C_word c,C_word *av) C_noret;
C_noret_decl(f_7837)
static void C_ccall f_7837(C_word c,C_word *av) C_noret;
C_noret_decl(f_7840)
static void C_ccall f_7840(C_word c,C_word *av) C_noret;
C_noret_decl(f_7848)
static void C_fcall f_7848(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7855)
static void C_ccall f_7855(C_word c,C_word *av) C_noret;
C_noret_decl(f_7861)
static void C_ccall f_7861(C_word c,C_word *av) C_noret;
C_noret_decl(f_7891)
static void C_ccall f_7891(C_word c,C_word *av) C_noret;
C_noret_decl(f_7904)
static void C_ccall f_7904(C_word c,C_word *av) C_noret;
C_noret_decl(f_7908)
static void C_ccall f_7908(C_word c,C_word *av) C_noret;
C_noret_decl(f_7932)
static void C_ccall f_7932(C_word c,C_word *av) C_noret;
C_noret_decl(f_7935)
static void C_ccall f_7935(C_word c,C_word *av) C_noret;
C_noret_decl(f_7941)
static void C_ccall f_7941(C_word c,C_word *av) C_noret;
C_noret_decl(f_7943)
static void C_fcall f_7943(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7967)
static void C_fcall f_7967(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7992)
static void C_ccall f_7992(C_word c,C_word *av) C_noret;
C_noret_decl(f_8006)
static void C_ccall f_8006(C_word c,C_word *av) C_noret;
C_noret_decl(f_8024)
static void C_ccall f_8024(C_word c,C_word *av) C_noret;
C_noret_decl(f_8031)
static void C_ccall f_8031(C_word c,C_word *av) C_noret;
C_noret_decl(f_8052)
static void C_fcall f_8052(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8072)
static void C_ccall f_8072(C_word c,C_word *av) C_noret;
C_noret_decl(f_8093)
static void C_ccall f_8093(C_word c,C_word *av) C_noret;
C_noret_decl(f_8097)
static void C_ccall f_8097(C_word c,C_word *av) C_noret;
C_noret_decl(f_8118)
static void C_ccall f_8118(C_word c,C_word *av) C_noret;
C_noret_decl(f_8122)
static void C_ccall f_8122(C_word c,C_word *av) C_noret;
C_noret_decl(f_8131)
static void C_fcall f_8131(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8138)
static void C_fcall f_8138(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8144)
static void C_ccall f_8144(C_word c,C_word *av) C_noret;
C_noret_decl(f_8188)
static void C_ccall f_8188(C_word c,C_word *av) C_noret;
C_noret_decl(f_8197)
static void C_ccall f_8197(C_word c,C_word *av) C_noret;
C_noret_decl(f_8206)
static void C_ccall f_8206(C_word c,C_word *av) C_noret;
C_noret_decl(f_8212)
static void C_ccall f_8212(C_word c,C_word *av) C_noret;
C_noret_decl(f_8239)
static void C_ccall f_8239(C_word c,C_word *av) C_noret;
C_noret_decl(f_8242)
static void C_ccall f_8242(C_word c,C_word *av) C_noret;
C_noret_decl(f_8249)
static void C_ccall f_8249(C_word c,C_word *av) C_noret;
C_noret_decl(f_8283)
static void C_ccall f_8283(C_word c,C_word *av) C_noret;
C_noret_decl(f_8285)
static void C_fcall f_8285(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8299)
static void C_fcall f_8299(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8320)
static void C_ccall f_8320(C_word c,C_word *av) C_noret;
C_noret_decl(f_8324)
static void C_ccall f_8324(C_word c,C_word *av) C_noret;
C_noret_decl(f_8336)
static void C_ccall f_8336(C_word c,C_word *av) C_noret;
C_noret_decl(f_8349)
static void C_ccall f_8349(C_word c,C_word *av) C_noret;
C_noret_decl(f_8353)
static void C_ccall f_8353(C_word c,C_word *av) C_noret;
C_noret_decl(f_8359)
static void C_fcall f_8359(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8366)
static void C_ccall f_8366(C_word c,C_word *av) C_noret;
C_noret_decl(f_8369)
static void C_ccall f_8369(C_word c,C_word *av) C_noret;
C_noret_decl(f_8371)
static void C_fcall f_8371(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8373)
static void C_fcall f_8373(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8381)
static void C_ccall f_8381(C_word c,C_word *av) C_noret;
C_noret_decl(f_8384)
static void C_ccall f_8384(C_word c,C_word *av) C_noret;
C_noret_decl(f_8387)
static void C_ccall f_8387(C_word c,C_word *av) C_noret;
C_noret_decl(f_8390)
static void C_ccall f_8390(C_word c,C_word *av) C_noret;
C_noret_decl(f_8393)
static void C_ccall f_8393(C_word c,C_word *av) C_noret;
C_noret_decl(f_8396)
static void C_ccall f_8396(C_word c,C_word *av) C_noret;
C_noret_decl(f_8399)
static void C_fcall f_8399(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8405)
static void C_ccall f_8405(C_word c,C_word *av) C_noret;
C_noret_decl(f_8408)
static void C_ccall f_8408(C_word c,C_word *av) C_noret;
C_noret_decl(f_8411)
static void C_ccall f_8411(C_word c,C_word *av) C_noret;
C_noret_decl(f_8414)
static void C_ccall f_8414(C_word c,C_word *av) C_noret;
C_noret_decl(f_8417)
static void C_ccall f_8417(C_word c,C_word *av) C_noret;
C_noret_decl(f_8420)
static void C_ccall f_8420(C_word c,C_word *av) C_noret;
C_noret_decl(f_8423)
static void C_ccall f_8423(C_word c,C_word *av) C_noret;
C_noret_decl(f_8426)
static void C_ccall f_8426(C_word c,C_word *av) C_noret;
C_noret_decl(f_8429)
static void C_ccall f_8429(C_word c,C_word *av) C_noret;
C_noret_decl(f_8432)
static void C_ccall f_8432(C_word c,C_word *av) C_noret;
C_noret_decl(f_8435)
static void C_ccall f_8435(C_word c,C_word *av) C_noret;
C_noret_decl(f_8438)
static void C_ccall f_8438(C_word c,C_word *av) C_noret;
C_noret_decl(f_8441)
static void C_ccall f_8441(C_word c,C_word *av) C_noret;
C_noret_decl(f_8444)
static void C_ccall f_8444(C_word c,C_word *av) C_noret;
C_noret_decl(f_8447)
static void C_ccall f_8447(C_word c,C_word *av) C_noret;
C_noret_decl(f_8450)
static void C_ccall f_8450(C_word c,C_word *av) C_noret;
C_noret_decl(f_8453)
static void C_ccall f_8453(C_word c,C_word *av) C_noret;
C_noret_decl(f_8456)
static void C_ccall f_8456(C_word c,C_word *av) C_noret;
C_noret_decl(f_8459)
static void C_ccall f_8459(C_word c,C_word *av) C_noret;
C_noret_decl(f_8462)
static void C_ccall f_8462(C_word c,C_word *av) C_noret;
C_noret_decl(f_8465)
static void C_fcall f_8465(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8468)
static void C_ccall f_8468(C_word c,C_word *av) C_noret;
C_noret_decl(f_8471)
static void C_ccall f_8471(C_word c,C_word *av) C_noret;
C_noret_decl(f_8474)
static void C_ccall f_8474(C_word c,C_word *av) C_noret;
C_noret_decl(f_8477)
static void C_ccall f_8477(C_word c,C_word *av) C_noret;
C_noret_decl(f_8480)
static void C_ccall f_8480(C_word c,C_word *av) C_noret;
C_noret_decl(f_8484)
static void C_ccall f_8484(C_word c,C_word *av) C_noret;
C_noret_decl(f_8491)
static void C_ccall f_8491(C_word c,C_word *av) C_noret;
C_noret_decl(f_8502)
static void C_ccall f_8502(C_word c,C_word *av) C_noret;
C_noret_decl(f_8508)
static void C_ccall f_8508(C_word c,C_word *av) C_noret;
C_noret_decl(f_8511)
static void C_ccall f_8511(C_word c,C_word *av) C_noret;
C_noret_decl(f_8520)
static void C_fcall f_8520(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8528)
static void C_ccall f_8528(C_word c,C_word *av) C_noret;
C_noret_decl(f_8542)
static void C_ccall f_8542(C_word c,C_word *av) C_noret;
C_noret_decl(f_8557)
static void C_ccall f_8557(C_word c,C_word *av) C_noret;
C_noret_decl(f_8560)
static void C_ccall f_8560(C_word c,C_word *av) C_noret;
C_noret_decl(f_8577)
static void C_fcall f_8577(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8587)
static void C_ccall f_8587(C_word c,C_word *av) C_noret;
C_noret_decl(f_8617)
static void C_ccall f_8617(C_word c,C_word *av) C_noret;
C_noret_decl(f_8621)
static void C_ccall f_8621(C_word c,C_word *av) C_noret;
C_noret_decl(f_8624)
static void C_ccall f_8624(C_word c,C_word *av) C_noret;
C_noret_decl(f_8627)
static void C_ccall f_8627(C_word c,C_word *av) C_noret;
C_noret_decl(f_8630)
static void C_ccall f_8630(C_word c,C_word *av) C_noret;
C_noret_decl(f_8633)
static void C_ccall f_8633(C_word c,C_word *av) C_noret;
C_noret_decl(f_8636)
static void C_ccall f_8636(C_word c,C_word *av) C_noret;
C_noret_decl(f_8645)
static void C_ccall f_8645(C_word c,C_word *av) C_noret;
C_noret_decl(f_8648)
static void C_ccall f_8648(C_word c,C_word *av) C_noret;
C_noret_decl(f_8659)
static void C_ccall f_8659(C_word c,C_word *av) C_noret;
C_noret_decl(f_8680)
static void C_fcall f_8680(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8696)
static void C_fcall f_8696(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8704)
static void C_ccall f_8704(C_word c,C_word *av) C_noret;
C_noret_decl(f_8707)
static void C_ccall f_8707(C_word c,C_word *av) C_noret;
C_noret_decl(f_8717)
static void C_ccall f_8717(C_word c,C_word *av) C_noret;
C_noret_decl(f_8720)
static void C_ccall f_8720(C_word c,C_word *av) C_noret;
C_noret_decl(f_8723)
static void C_ccall f_8723(C_word c,C_word *av) C_noret;
C_noret_decl(f_8732)
static void C_fcall f_8732(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8756)
static void C_ccall f_8756(C_word c,C_word *av) C_noret;
C_noret_decl(f_8759)
static void C_ccall f_8759(C_word c,C_word *av) C_noret;
C_noret_decl(f_8762)
static void C_ccall f_8762(C_word c,C_word *av) C_noret;
C_noret_decl(f_8765)
static void C_ccall f_8765(C_word c,C_word *av) C_noret;
C_noret_decl(f_8774)
static void C_ccall f_8774(C_word c,C_word *av) C_noret;
C_noret_decl(f_8790)
static void C_fcall f_8790(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8850)
static void C_ccall f_8850(C_word c,C_word *av) C_noret;
C_noret_decl(f_8863)
static void C_fcall f_8863(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8873)
static void C_ccall f_8873(C_word c,C_word *av) C_noret;
C_noret_decl(f_8888)
static void C_fcall f_8888(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8890)
static void C_fcall f_8890(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8898)
static void C_ccall f_8898(C_word c,C_word *av) C_noret;
C_noret_decl(f_8918)
static void C_fcall f_8918(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8928)
static void C_ccall f_8928(C_word c,C_word *av) C_noret;
C_noret_decl(f_8952)
static void C_ccall f_8952(C_word c,C_word *av) C_noret;
C_noret_decl(f_8975)
static void C_ccall f_8975(C_word c,C_word *av) C_noret;
C_noret_decl(f_8978)
static void C_ccall f_8978(C_word c,C_word *av) C_noret;
C_noret_decl(f_8981)
static void C_ccall f_8981(C_word c,C_word *av) C_noret;
C_noret_decl(f_8997)
static void C_ccall f_8997(C_word c,C_word *av) C_noret;
C_noret_decl(f_9000)
static void C_ccall f_9000(C_word c,C_word *av) C_noret;
C_noret_decl(f_9014)
static void C_ccall f_9014(C_word c,C_word *av) C_noret;
C_noret_decl(f_9039)
static void C_ccall f_9039(C_word c,C_word *av) C_noret;
C_noret_decl(f_9043)
static void C_ccall f_9043(C_word c,C_word *av) C_noret;
C_noret_decl(f_9051)
static void C_fcall f_9051(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9061)
static void C_ccall f_9061(C_word c,C_word *av) C_noret;
C_noret_decl(f_9075)
static void C_ccall f_9075(C_word c,C_word *av) C_noret;
C_noret_decl(f_9079)
static void C_ccall f_9079(C_word c,C_word *av) C_noret;
C_noret_decl(f_9082)
static void C_ccall f_9082(C_word c,C_word *av) C_noret;
C_noret_decl(f_9085)
static void C_ccall f_9085(C_word c,C_word *av) C_noret;
C_noret_decl(f_9088)
static void C_ccall f_9088(C_word c,C_word *av) C_noret;
C_noret_decl(f_9091)
static void C_ccall f_9091(C_word c,C_word *av) C_noret;
C_noret_decl(f_9094)
static void C_ccall f_9094(C_word c,C_word *av) C_noret;
C_noret_decl(f_9097)
static void C_ccall f_9097(C_word c,C_word *av) C_noret;
C_noret_decl(f_9100)
static void C_ccall f_9100(C_word c,C_word *av) C_noret;
C_noret_decl(f_9103)
static void C_ccall f_9103(C_word c,C_word *av) C_noret;
C_noret_decl(f_9106)
static void C_ccall f_9106(C_word c,C_word *av) C_noret;
C_noret_decl(f_9121)
static void C_ccall f_9121(C_word c,C_word *av) C_noret;
C_noret_decl(f_9126)
static void C_ccall f_9126(C_word c,C_word *av) C_noret;
C_noret_decl(f_9144)
static void C_ccall f_9144(C_word c,C_word *av) C_noret;
C_noret_decl(f_9148)
static void C_ccall f_9148(C_word c,C_word *av) C_noret;
C_noret_decl(f_9157)
static void C_ccall f_9157(C_word c,C_word *av) C_noret;
C_noret_decl(f_9162)
static void C_fcall f_9162(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9172)
static void C_ccall f_9172(C_word c,C_word *av) C_noret;
C_noret_decl(f_9190)
static void C_ccall f_9190(C_word c,C_word *av) C_noret;
C_noret_decl(f_9196)
static void C_ccall f_9196(C_word c,C_word *av) C_noret;
C_noret_decl(f_9201)
static void C_fcall f_9201(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9211)
static void C_ccall f_9211(C_word c,C_word *av) C_noret;
C_noret_decl(f_9224)
static void C_ccall f_9224(C_word c,C_word *av) C_noret;
C_noret_decl(f_9242)
static void C_ccall f_9242(C_word c,C_word *av) C_noret;
C_noret_decl(f_9243)
static void C_fcall f_9243(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9251)
static void C_ccall f_9251(C_word c,C_word *av) C_noret;
C_noret_decl(f_9264)
static void C_ccall f_9264(C_word c,C_word *av) C_noret;
C_noret_decl(f_9271)
static void C_ccall f_9271(C_word c,C_word *av) C_noret;
C_noret_decl(f_9274)
static void C_ccall f_9274(C_word c,C_word *av) C_noret;
C_noret_decl(f_9277)
static void C_ccall f_9277(C_word c,C_word *av) C_noret;
C_noret_decl(f_9280)
static void C_ccall f_9280(C_word c,C_word *av) C_noret;
C_noret_decl(f_9283)
static void C_ccall f_9283(C_word c,C_word *av) C_noret;
C_noret_decl(f_9288)
static void C_fcall f_9288(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9298)
static void C_ccall f_9298(C_word c,C_word *av) C_noret;
C_noret_decl(f_9316)
static void C_ccall f_9316(C_word c,C_word *av) C_noret;
C_noret_decl(f_9330)
static void C_ccall f_9330(C_word c,C_word *av) C_noret;
C_noret_decl(f_9332)
static void C_fcall f_9332(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9341)
static void C_fcall f_9341(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9367)
static void C_fcall f_9367(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9377)
static void C_ccall f_9377(C_word c,C_word *av) C_noret;
C_noret_decl(f_9408)
static void C_fcall f_9408(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9414)
static void C_ccall f_9414(C_word c,C_word *av) C_noret;
C_noret_decl(f_9422)
static void C_ccall f_9422(C_word c,C_word *av) C_noret;
C_noret_decl(f_9425)
static void C_fcall f_9425(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9431)
static void C_ccall f_9431(C_word c,C_word *av) C_noret;
C_noret_decl(f_9439)
static void C_ccall f_9439(C_word c,C_word *av) C_noret;
C_noret_decl(f_9446)
static void C_ccall f_9446(C_word c,C_word *av) C_noret;
C_noret_decl(f_9468)
static void C_ccall f_9468(C_word c,C_word *av) C_noret;
C_noret_decl(f_9476)
static void C_fcall f_9476(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9486)
static void C_ccall f_9486(C_word c,C_word *av) C_noret;
C_noret_decl(f_9499)
static void C_fcall f_9499(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9505)
static void C_ccall f_9505(C_word c,C_word *av) C_noret;
C_noret_decl(f_9508)
static void C_ccall f_9508(C_word c,C_word *av) C_noret;
C_noret_decl(f_9519)
static void C_fcall f_9519(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9529)
static void C_ccall f_9529(C_word c,C_word *av) C_noret;
C_noret_decl(f_9544)
static void C_fcall f_9544(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9548)
static void C_ccall f_9548(C_word c,C_word *av) C_noret;
C_noret_decl(f_9551)
static void C_ccall f_9551(C_word c,C_word *av) C_noret;
C_noret_decl(f_9554)
static void C_ccall f_9554(C_word c,C_word *av) C_noret;
C_noret_decl(f_9560)
static void C_ccall f_9560(C_word c,C_word *av) C_noret;
C_noret_decl(f_9563)
static void C_ccall f_9563(C_word c,C_word *av) C_noret;
C_noret_decl(f_9566)
static void C_ccall f_9566(C_word c,C_word *av) C_noret;
C_noret_decl(f_9569)
static void C_ccall f_9569(C_word c,C_word *av) C_noret;
C_noret_decl(f_9572)
static void C_ccall f_9572(C_word c,C_word *av) C_noret;
C_noret_decl(f_9575)
static void C_ccall f_9575(C_word c,C_word *av) C_noret;
C_noret_decl(f_9578)
static void C_ccall f_9578(C_word c,C_word *av) C_noret;
C_noret_decl(f_9581)
static void C_ccall f_9581(C_word c,C_word *av) C_noret;
C_noret_decl(f_9584)
static void C_ccall f_9584(C_word c,C_word *av) C_noret;
C_noret_decl(f_9587)
static void C_ccall f_9587(C_word c,C_word *av) C_noret;
C_noret_decl(f_9590)
static void C_ccall f_9590(C_word c,C_word *av) C_noret;
C_noret_decl(f_9593)
static void C_ccall f_9593(C_word c,C_word *av) C_noret;
C_noret_decl(f_9596)
static void C_ccall f_9596(C_word c,C_word *av) C_noret;
C_noret_decl(f_9605)
static void C_ccall f_9605(C_word c,C_word *av) C_noret;
C_noret_decl(f_9609)
static void C_ccall f_9609(C_word c,C_word *av) C_noret;
C_noret_decl(f_9613)
static void C_ccall f_9613(C_word c,C_word *av) C_noret;
C_noret_decl(f_9617)
static void C_ccall f_9617(C_word c,C_word *av) C_noret;
C_noret_decl(f_9623)
static void C_ccall f_9623(C_word c,C_word *av) C_noret;
C_noret_decl(f_9629)
static void C_ccall f_9629(C_word c,C_word *av) C_noret;
C_noret_decl(f_9632)
static void C_ccall f_9632(C_word c,C_word *av) C_noret;
C_noret_decl(f_9638)
static void C_ccall f_9638(C_word c,C_word *av) C_noret;
C_noret_decl(f_9647)
static void C_ccall f_9647(C_word c,C_word *av) C_noret;
C_noret_decl(f_9650)
static void C_ccall f_9650(C_word c,C_word *av) C_noret;
C_noret_decl(f_9653)
static void C_ccall f_9653(C_word c,C_word *av) C_noret;
C_noret_decl(f_9659)
static void C_ccall f_9659(C_word c,C_word *av) C_noret;
C_noret_decl(f_9662)
static void C_ccall f_9662(C_word c,C_word *av) C_noret;
C_noret_decl(f_9680)
static void C_ccall f_9680(C_word c,C_word *av) C_noret;
C_noret_decl(f_9683)
static void C_ccall f_9683(C_word c,C_word *av) C_noret;
C_noret_decl(f_9686)
static void C_ccall f_9686(C_word c,C_word *av) C_noret;
C_noret_decl(f_9689)
static void C_ccall f_9689(C_word c,C_word *av) C_noret;
C_noret_decl(f_9692)
static void C_ccall f_9692(C_word c,C_word *av) C_noret;
C_noret_decl(f_9717)
static void C_ccall f_9717(C_word c,C_word *av) C_noret;
C_noret_decl(f_9721)
static void C_ccall f_9721(C_word c,C_word *av) C_noret;
C_noret_decl(f_9739)
static void C_fcall f_9739(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_9746)
static void C_fcall f_9746(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9749)
static void C_ccall f_9749(C_word c,C_word *av) C_noret;
C_noret_decl(f_9791)
static void C_ccall f_9791(C_word c,C_word *av) C_noret;
C_noret_decl(f_9796)
static void C_fcall f_9796(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9806)
static void C_ccall f_9806(C_word c,C_word *av) C_noret;
C_noret_decl(f_9818)
static void C_ccall f_9818(C_word c,C_word *av) C_noret;
C_noret_decl(f_9821)
static void C_ccall f_9821(C_word c,C_word *av) C_noret;
C_noret_decl(f_9828)
static void C_ccall f_9828(C_word c,C_word *av) C_noret;
C_noret_decl(f_9836)
static void C_ccall f_9836(C_word c,C_word *av) C_noret;
C_noret_decl(f_9846)
static void C_ccall f_9846(C_word c,C_word *av) C_noret;
C_noret_decl(f_9857)
static void C_fcall f_9857(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9867)
static void C_ccall f_9867(C_word c,C_word *av) C_noret;
C_noret_decl(f_9882)
static void C_fcall f_9882(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9886)
static void C_ccall f_9886(C_word c,C_word *av) C_noret;
C_noret_decl(f_9889)
static void C_ccall f_9889(C_word c,C_word *av) C_noret;
C_noret_decl(f_9892)
static void C_ccall f_9892(C_word c,C_word *av) C_noret;
C_noret_decl(f_9895)
static void C_ccall f_9895(C_word c,C_word *av) C_noret;
C_noret_decl(f_9901)
static void C_ccall f_9901(C_word c,C_word *av) C_noret;
C_noret_decl(f_9903)
static void C_fcall f_9903(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_9913)
static void C_fcall f_9913(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9922)
static void C_fcall f_9922(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9934)
static void C_fcall f_9934(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9946)
static void C_fcall f_9946(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9958)
static void C_fcall f_9958(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9970)
static void C_fcall f_9970(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9976)
static void C_ccall f_9976(C_word c,C_word *av) C_noret;
C_noret_decl(f_9980)
static void C_fcall f_9980(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(C_c_2dbackend_toplevel)
C_externexport void C_ccall C_c_2dbackend_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(va15385)
static void C_fcall va15385(C_word t0,C_word t1) C_noret;
C_noret_decl(va15391)
static void C_fcall va15391(C_word t0,C_word t1) C_noret;
C_noret_decl(va15423)
static void C_fcall va15423(C_word t0,C_word t1) C_noret;
C_noret_decl(va15425)
static void C_fcall va15425(C_word t0,C_word t1) C_noret;
C_noret_decl(va15433)
static void C_fcall va15433(C_word t0,C_word t1) C_noret;
C_noret_decl(va15435)
static void C_fcall va15435(C_word t0,C_word t1) C_noret;

C_noret_decl(trf_10007)
static void C_ccall trf_10007(C_word c,C_word *av) C_noret;
static void C_ccall trf_10007(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10007(t0,t1);}

C_noret_decl(trf_10426)
static void C_ccall trf_10426(C_word c,C_word *av) C_noret;
static void C_ccall trf_10426(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10426(t0,t1,t2,t3);}

C_noret_decl(trf_10477)
static void C_ccall trf_10477(C_word c,C_word *av) C_noret;
static void C_ccall trf_10477(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10477(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10511)
static void C_ccall trf_10511(C_word c,C_word *av) C_noret;
static void C_ccall trf_10511(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10511(t0,t1,t2);}

C_noret_decl(trf_10534)
static void C_ccall trf_10534(C_word c,C_word *av) C_noret;
static void C_ccall trf_10534(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10534(t0,t1,t2);}

C_noret_decl(trf_10564)
static void C_ccall trf_10564(C_word c,C_word *av) C_noret;
static void C_ccall trf_10564(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10564(t0,t1,t2,t3);}

C_noret_decl(trf_10612)
static void C_ccall trf_10612(C_word c,C_word *av) C_noret;
static void C_ccall trf_10612(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10612(t0,t1);}

C_noret_decl(trf_10617)
static void C_ccall trf_10617(C_word c,C_word *av) C_noret;
static void C_ccall trf_10617(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10617(t0,t1,t2);}

C_noret_decl(trf_10687)
static void C_ccall trf_10687(C_word c,C_word *av) C_noret;
static void C_ccall trf_10687(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10687(t0,t1);}

C_noret_decl(trf_10804)
static void C_ccall trf_10804(C_word c,C_word *av) C_noret;
static void C_ccall trf_10804(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10804(t0,t1);}

C_noret_decl(trf_10825)
static void C_ccall trf_10825(C_word c,C_word *av) C_noret;
static void C_ccall trf_10825(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10825(t0,t1);}

C_noret_decl(trf_10957)
static void C_ccall trf_10957(C_word c,C_word *av) C_noret;
static void C_ccall trf_10957(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10957(t0,t1);}

C_noret_decl(trf_10969)
static void C_ccall trf_10969(C_word c,C_word *av) C_noret;
static void C_ccall trf_10969(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10969(t0,t1);}

C_noret_decl(trf_10988)
static void C_ccall trf_10988(C_word c,C_word *av) C_noret;
static void C_ccall trf_10988(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10988(t0,t1,t2);}

C_noret_decl(trf_11090)
static void C_ccall trf_11090(C_word c,C_word *av) C_noret;
static void C_ccall trf_11090(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11090(t0,t1,t2);}

C_noret_decl(trf_11280)
static void C_ccall trf_11280(C_word c,C_word *av) C_noret;
static void C_ccall trf_11280(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11280(t0,t1,t2);}

C_noret_decl(trf_11290)
static void C_ccall trf_11290(C_word c,C_word *av) C_noret;
static void C_ccall trf_11290(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11290(t0,t1);}

C_noret_decl(trf_11704)
static void C_ccall trf_11704(C_word c,C_word *av) C_noret;
static void C_ccall trf_11704(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11704(t0,t1);}

C_noret_decl(trf_11706)
static void C_ccall trf_11706(C_word c,C_word *av) C_noret;
static void C_ccall trf_11706(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11706(t0,t1);}

C_noret_decl(trf_11734)
static void C_ccall trf_11734(C_word c,C_word *av) C_noret;
static void C_ccall trf_11734(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11734(t0,t1);}

C_noret_decl(trf_11761)
static void C_ccall trf_11761(C_word c,C_word *av) C_noret;
static void C_ccall trf_11761(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11761(t0,t1);}

C_noret_decl(trf_11986)
static void C_ccall trf_11986(C_word c,C_word *av) C_noret;
static void C_ccall trf_11986(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11986(t0,t1);}

C_noret_decl(trf_11995)
static void C_ccall trf_11995(C_word c,C_word *av) C_noret;
static void C_ccall trf_11995(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11995(t0,t1);}

C_noret_decl(trf_12369)
static void C_ccall trf_12369(C_word c,C_word *av) C_noret;
static void C_ccall trf_12369(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12369(t0,t1,t2);}

C_noret_decl(trf_12371)
static void C_ccall trf_12371(C_word c,C_word *av) C_noret;
static void C_ccall trf_12371(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12371(t0,t1);}

C_noret_decl(trf_12486)
static void C_ccall trf_12486(C_word c,C_word *av) C_noret;
static void C_ccall trf_12486(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12486(t0,t1);}

C_noret_decl(trf_12706)
static void C_ccall trf_12706(C_word c,C_word *av) C_noret;
static void C_ccall trf_12706(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12706(t0,t1,t2);}

C_noret_decl(trf_13156)
static void C_ccall trf_13156(C_word c,C_word *av) C_noret;
static void C_ccall trf_13156(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13156(t0,t1);}

C_noret_decl(trf_13165)
static void C_ccall trf_13165(C_word c,C_word *av) C_noret;
static void C_ccall trf_13165(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13165(t0,t1);}

C_noret_decl(trf_13281)
static void C_ccall trf_13281(C_word c,C_word *av) C_noret;
static void C_ccall trf_13281(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13281(t0,t1);}

C_noret_decl(trf_3707)
static void C_ccall trf_3707(C_word c,C_word *av) C_noret;
static void C_ccall trf_3707(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_3707(t0,t1,t2,t3);}

C_noret_decl(trf_4279)
static void C_ccall trf_4279(C_word c,C_word *av) C_noret;
static void C_ccall trf_4279(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4279(t0,t1,t2);}

C_noret_decl(trf_4285)
static void C_ccall trf_4285(C_word c,C_word *av) C_noret;
static void C_ccall trf_4285(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4285(t0,t1,t2);}

C_noret_decl(trf_4567)
static void C_ccall trf_4567(C_word c,C_word *av) C_noret;
static void C_ccall trf_4567(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4567(t0,t1,t2);}

C_noret_decl(trf_4590)
static void C_ccall trf_4590(C_word c,C_word *av) C_noret;
static void C_ccall trf_4590(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_4590(t0,t1);}

C_noret_decl(trf_4604)
static void C_ccall trf_4604(C_word c,C_word *av) C_noret;
static void C_ccall trf_4604(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4604(t0,t1,t2);}

C_noret_decl(trf_4708)
static void C_ccall trf_4708(C_word c,C_word *av) C_noret;
static void C_ccall trf_4708(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4708(t0,t1,t2);}

C_noret_decl(trf_4720)
static void C_ccall trf_4720(C_word c,C_word *av) C_noret;
static void C_ccall trf_4720(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_4720(t0,t1,t2,t3,t4);}

C_noret_decl(trf_4723)
static void C_ccall trf_4723(C_word c,C_word *av) C_noret;
static void C_ccall trf_4723(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4723(t0,t1,t2,t3);}

C_noret_decl(trf_4951)
static void C_ccall trf_4951(C_word c,C_word *av) C_noret;
static void C_ccall trf_4951(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_4951(t0,t1,t2,t3,t4);}

C_noret_decl(trf_5109)
static void C_ccall trf_5109(C_word c,C_word *av) C_noret;
static void C_ccall trf_5109(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5109(t0,t1);}

C_noret_decl(trf_5149)
static void C_ccall trf_5149(C_word c,C_word *av) C_noret;
static void C_ccall trf_5149(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5149(t0,t1);}

C_noret_decl(trf_5189)
static void C_ccall trf_5189(C_word c,C_word *av) C_noret;
static void C_ccall trf_5189(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5189(t0,t1);}

C_noret_decl(trf_5374)
static void C_ccall trf_5374(C_word c,C_word *av) C_noret;
static void C_ccall trf_5374(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5374(t0,t1,t2,t3);}

C_noret_decl(trf_5403)
static void C_ccall trf_5403(C_word c,C_word *av) C_noret;
static void C_ccall trf_5403(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5403(t0,t1,t2,t3);}

C_noret_decl(trf_5683)
static void C_ccall trf_5683(C_word c,C_word *av) C_noret;
static void C_ccall trf_5683(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5683(t0,t1);}

C_noret_decl(trf_5689)
static void C_ccall trf_5689(C_word c,C_word *av) C_noret;
static void C_ccall trf_5689(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5689(t0,t1);}

C_noret_decl(trf_5731)
static void C_ccall trf_5731(C_word c,C_word *av) C_noret;
static void C_ccall trf_5731(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5731(t0,t1,t2,t3);}

C_noret_decl(trf_5772)
static void C_ccall trf_5772(C_word c,C_word *av) C_noret;
static void C_ccall trf_5772(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5772(t0,t1,t2,t3);}

C_noret_decl(trf_5809)
static void C_ccall trf_5809(C_word c,C_word *av) C_noret;
static void C_ccall trf_5809(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5809(t0,t1,t2,t3);}

C_noret_decl(trf_5930)
static void C_ccall trf_5930(C_word c,C_word *av) C_noret;
static void C_ccall trf_5930(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5930(t0,t1);}

C_noret_decl(trf_6064)
static void C_ccall trf_6064(C_word c,C_word *av) C_noret;
static void C_ccall trf_6064(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6064(t0,t1);}

C_noret_decl(trf_6200)
static void C_ccall trf_6200(C_word c,C_word *av) C_noret;
static void C_ccall trf_6200(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6200(t0,t1,t2,t3);}

C_noret_decl(trf_6235)
static void C_ccall trf_6235(C_word c,C_word *av) C_noret;
static void C_ccall trf_6235(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6235(t0,t1,t2,t3);}

C_noret_decl(trf_6272)
static void C_ccall trf_6272(C_word c,C_word *av) C_noret;
static void C_ccall trf_6272(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6272(t0,t1,t2,t3);}

C_noret_decl(trf_6741)
static void C_ccall trf_6741(C_word c,C_word *av) C_noret;
static void C_ccall trf_6741(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6741(t0,t1,t2,t3);}

C_noret_decl(trf_6924)
static void C_ccall trf_6924(C_word c,C_word *av) C_noret;
static void C_ccall trf_6924(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6924(t0,t1,t2,t3);}

C_noret_decl(trf_6930)
static void C_ccall trf_6930(C_word c,C_word *av) C_noret;
static void C_ccall trf_6930(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6930(t0,t1,t2);}

C_noret_decl(trf_6960)
static void C_ccall trf_6960(C_word c,C_word *av) C_noret;
static void C_ccall trf_6960(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6960(t0,t1,t2);}

C_noret_decl(trf_6966)
static void C_ccall trf_6966(C_word c,C_word *av) C_noret;
static void C_ccall trf_6966(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6966(t0,t1,t2);}

C_noret_decl(trf_7003)
static void C_ccall trf_7003(C_word c,C_word *av) C_noret;
static void C_ccall trf_7003(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_7003(t0,t1,t2,t3,t4);}

C_noret_decl(trf_7015)
static void C_ccall trf_7015(C_word c,C_word *av) C_noret;
static void C_ccall trf_7015(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7015(t0,t1);}

C_noret_decl(trf_7036)
static void C_ccall trf_7036(C_word c,C_word *av) C_noret;
static void C_ccall trf_7036(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7036(t0,t1,t2,t3);}

C_noret_decl(trf_7132)
static void C_ccall trf_7132(C_word c,C_word *av) C_noret;
static void C_ccall trf_7132(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7132(t0,t1);}

C_noret_decl(trf_7179)
static void C_ccall trf_7179(C_word c,C_word *av) C_noret;
static void C_ccall trf_7179(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7179(t0,t1,t2);}

C_noret_decl(trf_7239)
static void C_ccall trf_7239(C_word c,C_word *av) C_noret;
static void C_ccall trf_7239(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7239(t0,t1,t2);}

C_noret_decl(trf_7291)
static void C_ccall trf_7291(C_word c,C_word *av) C_noret;
static void C_ccall trf_7291(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7291(t0,t1);}

C_noret_decl(trf_7329)
static void C_ccall trf_7329(C_word c,C_word *av) C_noret;
static void C_ccall trf_7329(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7329(t0,t1,t2,t3);}

C_noret_decl(trf_7369)
static void C_ccall trf_7369(C_word c,C_word *av) C_noret;
static void C_ccall trf_7369(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7369(t0,t1,t2);}

C_noret_decl(trf_7387)
static void C_ccall trf_7387(C_word c,C_word *av) C_noret;
static void C_ccall trf_7387(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7387(t0,t1,t2);}

C_noret_decl(trf_7438)
static void C_ccall trf_7438(C_word c,C_word *av) C_noret;
static void C_ccall trf_7438(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7438(t0,t1,t2);}

C_noret_decl(trf_7461)
static void C_ccall trf_7461(C_word c,C_word *av) C_noret;
static void C_ccall trf_7461(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7461(t0,t1,t2);}

C_noret_decl(trf_7495)
static void C_ccall trf_7495(C_word c,C_word *av) C_noret;
static void C_ccall trf_7495(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7495(t0,t1);}

C_noret_decl(trf_7514)
static void C_ccall trf_7514(C_word c,C_word *av) C_noret;
static void C_ccall trf_7514(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7514(t0,t1);}

C_noret_decl(trf_7650)
static void C_ccall trf_7650(C_word c,C_word *av) C_noret;
static void C_ccall trf_7650(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7650(t0,t1,t2);}

C_noret_decl(trf_7673)
static void C_ccall trf_7673(C_word c,C_word *av) C_noret;
static void C_ccall trf_7673(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7673(t0,t1);}

C_noret_decl(trf_7684)
static void C_ccall trf_7684(C_word c,C_word *av) C_noret;
static void C_ccall trf_7684(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7684(t0,t1,t2,t3);}

C_noret_decl(trf_7716)
static void C_ccall trf_7716(C_word c,C_word *av) C_noret;
static void C_ccall trf_7716(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7716(t0,t1);}

C_noret_decl(trf_7719)
static void C_ccall trf_7719(C_word c,C_word *av) C_noret;
static void C_ccall trf_7719(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7719(t0,t1);}

C_noret_decl(trf_7771)
static void C_ccall trf_7771(C_word c,C_word *av) C_noret;
static void C_ccall trf_7771(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7771(t0,t1,t2);}

C_noret_decl(trf_7794)
static void C_ccall trf_7794(C_word c,C_word *av) C_noret;
static void C_ccall trf_7794(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7794(t0,t1);}

C_noret_decl(trf_7800)
static void C_ccall trf_7800(C_word c,C_word *av) C_noret;
static void C_ccall trf_7800(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7800(t0,t1,t2,t3);}

C_noret_decl(trf_7848)
static void C_ccall trf_7848(C_word c,C_word *av) C_noret;
static void C_ccall trf_7848(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7848(t0,t1,t2);}

C_noret_decl(trf_7943)
static void C_ccall trf_7943(C_word c,C_word *av) C_noret;
static void C_ccall trf_7943(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7943(t0,t1,t2,t3);}

C_noret_decl(trf_7967)
static void C_ccall trf_7967(C_word c,C_word *av) C_noret;
static void C_ccall trf_7967(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7967(t0,t1,t2);}

C_noret_decl(trf_8052)
static void C_ccall trf_8052(C_word c,C_word *av) C_noret;
static void C_ccall trf_8052(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8052(t0,t1,t2,t3);}

C_noret_decl(trf_8131)
static void C_ccall trf_8131(C_word c,C_word *av) C_noret;
static void C_ccall trf_8131(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8131(t0,t1,t2,t3);}

C_noret_decl(trf_8138)
static void C_ccall trf_8138(C_word c,C_word *av) C_noret;
static void C_ccall trf_8138(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8138(t0,t1);}

C_noret_decl(trf_8285)
static void C_ccall trf_8285(C_word c,C_word *av) C_noret;
static void C_ccall trf_8285(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8285(t0,t1,t2);}

C_noret_decl(trf_8299)
static void C_ccall trf_8299(C_word c,C_word *av) C_noret;
static void C_ccall trf_8299(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8299(t0,t1,t2,t3);}

C_noret_decl(trf_8359)
static void C_ccall trf_8359(C_word c,C_word *av) C_noret;
static void C_ccall trf_8359(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8359(t0,t1,t2,t3);}

C_noret_decl(trf_8371)
static void C_ccall trf_8371(C_word c,C_word *av) C_noret;
static void C_ccall trf_8371(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8371(t0,t1);}

C_noret_decl(trf_8373)
static void C_ccall trf_8373(C_word c,C_word *av) C_noret;
static void C_ccall trf_8373(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8373(t0,t1,t2);}

C_noret_decl(trf_8399)
static void C_ccall trf_8399(C_word c,C_word *av) C_noret;
static void C_ccall trf_8399(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8399(t0,t1);}

C_noret_decl(trf_8465)
static void C_ccall trf_8465(C_word c,C_word *av) C_noret;
static void C_ccall trf_8465(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8465(t0,t1);}

C_noret_decl(trf_8520)
static void C_ccall trf_8520(C_word c,C_word *av) C_noret;
static void C_ccall trf_8520(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8520(t0,t1,t2,t3);}

C_noret_decl(trf_8577)
static void C_ccall trf_8577(C_word c,C_word *av) C_noret;
static void C_ccall trf_8577(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8577(t0,t1,t2);}

C_noret_decl(trf_8680)
static void C_ccall trf_8680(C_word c,C_word *av) C_noret;
static void C_ccall trf_8680(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8680(t0,t1,t2,t3);}

C_noret_decl(trf_8696)
static void C_ccall trf_8696(C_word c,C_word *av) C_noret;
static void C_ccall trf_8696(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8696(t0,t1,t2,t3);}

C_noret_decl(trf_8732)
static void C_ccall trf_8732(C_word c,C_word *av) C_noret;
static void C_ccall trf_8732(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8732(t0,t1);}

C_noret_decl(trf_8790)
static void C_ccall trf_8790(C_word c,C_word *av) C_noret;
static void C_ccall trf_8790(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8790(t0,t1);}

C_noret_decl(trf_8863)
static void C_ccall trf_8863(C_word c,C_word *av) C_noret;
static void C_ccall trf_8863(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8863(t0,t1,t2);}

C_noret_decl(trf_8888)
static void C_ccall trf_8888(C_word c,C_word *av) C_noret;
static void C_ccall trf_8888(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8888(t0,t1);}

C_noret_decl(trf_8890)
static void C_ccall trf_8890(C_word c,C_word *av) C_noret;
static void C_ccall trf_8890(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8890(t0,t1,t2,t3);}

C_noret_decl(trf_8918)
static void C_ccall trf_8918(C_word c,C_word *av) C_noret;
static void C_ccall trf_8918(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8918(t0,t1,t2);}

C_noret_decl(trf_9051)
static void C_ccall trf_9051(C_word c,C_word *av) C_noret;
static void C_ccall trf_9051(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9051(t0,t1,t2);}

C_noret_decl(trf_9162)
static void C_ccall trf_9162(C_word c,C_word *av) C_noret;
static void C_ccall trf_9162(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9162(t0,t1,t2);}

C_noret_decl(trf_9201)
static void C_ccall trf_9201(C_word c,C_word *av) C_noret;
static void C_ccall trf_9201(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9201(t0,t1,t2);}

C_noret_decl(trf_9243)
static void C_ccall trf_9243(C_word c,C_word *av) C_noret;
static void C_ccall trf_9243(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9243(t0,t1,t2);}

C_noret_decl(trf_9288)
static void C_ccall trf_9288(C_word c,C_word *av) C_noret;
static void C_ccall trf_9288(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9288(t0,t1,t2);}

C_noret_decl(trf_9332)
static void C_ccall trf_9332(C_word c,C_word *av) C_noret;
static void C_ccall trf_9332(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9332(t0,t1);}

C_noret_decl(trf_9341)
static void C_ccall trf_9341(C_word c,C_word *av) C_noret;
static void C_ccall trf_9341(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9341(t0,t1,t2);}

C_noret_decl(trf_9367)
static void C_ccall trf_9367(C_word c,C_word *av) C_noret;
static void C_ccall trf_9367(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9367(t0,t1);}

C_noret_decl(trf_9408)
static void C_ccall trf_9408(C_word c,C_word *av) C_noret;
static void C_ccall trf_9408(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9408(t0,t1,t2);}

C_noret_decl(trf_9425)
static void C_ccall trf_9425(C_word c,C_word *av) C_noret;
static void C_ccall trf_9425(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9425(t0,t1,t2);}

C_noret_decl(trf_9476)
static void C_ccall trf_9476(C_word c,C_word *av) C_noret;
static void C_ccall trf_9476(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9476(t0,t1,t2);}

C_noret_decl(trf_9499)
static void C_ccall trf_9499(C_word c,C_word *av) C_noret;
static void C_ccall trf_9499(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9499(t0,t1);}

C_noret_decl(trf_9519)
static void C_ccall trf_9519(C_word c,C_word *av) C_noret;
static void C_ccall trf_9519(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9519(t0,t1,t2);}

C_noret_decl(trf_9544)
static void C_ccall trf_9544(C_word c,C_word *av) C_noret;
static void C_ccall trf_9544(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9544(t0,t1,t2);}

C_noret_decl(trf_9739)
static void C_ccall trf_9739(C_word c,C_word *av) C_noret;
static void C_ccall trf_9739(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_9739(t0,t1,t2,t3,t4);}

C_noret_decl(trf_9746)
static void C_ccall trf_9746(C_word c,C_word *av) C_noret;
static void C_ccall trf_9746(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9746(t0,t1);}

C_noret_decl(trf_9796)
static void C_ccall trf_9796(C_word c,C_word *av) C_noret;
static void C_ccall trf_9796(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9796(t0,t1,t2);}

C_noret_decl(trf_9857)
static void C_ccall trf_9857(C_word c,C_word *av) C_noret;
static void C_ccall trf_9857(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9857(t0,t1,t2);}

C_noret_decl(trf_9882)
static void C_ccall trf_9882(C_word c,C_word *av) C_noret;
static void C_ccall trf_9882(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9882(t0,t1,t2);}

C_noret_decl(trf_9903)
static void C_ccall trf_9903(C_word c,C_word *av) C_noret;
static void C_ccall trf_9903(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_9903(t0,t1,t2,t3,t4);}

C_noret_decl(trf_9913)
static void C_ccall trf_9913(C_word c,C_word *av) C_noret;
static void C_ccall trf_9913(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9913(t0,t1);}

C_noret_decl(trf_9922)
static void C_ccall trf_9922(C_word c,C_word *av) C_noret;
static void C_ccall trf_9922(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9922(t0,t1);}

C_noret_decl(trf_9934)
static void C_ccall trf_9934(C_word c,C_word *av) C_noret;
static void C_ccall trf_9934(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9934(t0,t1);}

C_noret_decl(trf_9946)
static void C_ccall trf_9946(C_word c,C_word *av) C_noret;
static void C_ccall trf_9946(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9946(t0,t1);}

C_noret_decl(trf_9958)
static void C_ccall trf_9958(C_word c,C_word *av) C_noret;
static void C_ccall trf_9958(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9958(t0,t1);}

C_noret_decl(trf_9970)
static void C_ccall trf_9970(C_word c,C_word *av) C_noret;
static void C_ccall trf_9970(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9970(t0,t1);}

C_noret_decl(trf_9980)
static void C_ccall trf_9980(C_word c,C_word *av) C_noret;
static void C_ccall trf_9980(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9980(t0,t1,t2);}

C_noret_decl(trva15385)
static void C_ccall trva15385(C_word c,C_word *av) C_noret;
static void C_ccall trva15385(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15385(t0,t1);}

C_noret_decl(trva15391)
static void C_ccall trva15391(C_word c,C_word *av) C_noret;
static void C_ccall trva15391(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15391(t0,t1);}

C_noret_decl(trva15423)
static void C_ccall trva15423(C_word c,C_word *av) C_noret;
static void C_ccall trva15423(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15423(t0,t1);}

C_noret_decl(trva15425)
static void C_ccall trva15425(C_word c,C_word *av) C_noret;
static void C_ccall trva15425(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15425(t0,t1);}

C_noret_decl(trva15433)
static void C_ccall trva15433(C_word c,C_word *av) C_noret;
static void C_ccall trva15433(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15433(t0,t1);}

C_noret_decl(trva15435)
static void C_ccall trva15435(C_word c,C_word *av) C_noret;
static void C_ccall trva15435(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va15435(t0,t1);}

/* f14744 in k5575 in k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14744,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14749 in k5620 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14749,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14754 in k5641 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14754,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14762 in k5857 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f14762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f14762,c,av);}
/* c-backend.scm:358: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[140];
f_4547(3,av2);}}

/* f14767 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14767,c,av);}
/* c-backend.scm:74: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[194];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14772 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14772,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14785 in k6314 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f14785,c,av);}
/* c-backend.scm:432: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* f14789 in k6377 in k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f14789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14789,c,av);}
/* c-backend.scm:464: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(41);
f_4547(4,av2);}}

/* f14796 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14796,c,av);}
/* c-backend.scm:74: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[194];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14801 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f14801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14801,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14874 in k7287 in k9104 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f14874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14874,c,av);}
/* c-backend.scm:75: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[117];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14881 in k9146 in for-each-loop1782 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f14881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f14881,c,av);}
/* c-backend.scm:74: chicken.string#string-translate* */
t2=C_fast_retrieve(lf[116]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[194];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f14902 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f14902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f14902,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10005 in k9974 in k9968 in k9956 in k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_fcall f_10007(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_10007,2,t0,t1);}
if(C_truep(t1)){
/* c-backend.scm:1198: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[547];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[548]);
if(C_truep(t2)){
/* c-backend.scm:1199: compute-size */
t3=((C_word*)((C_word*)t0)[5])[1];
f_9903(t3,((C_word*)t0)[2],C_i_cadr(((C_word*)t0)[6]),((C_word*)t0)[7],((C_word*)t0)[3]);}
else{
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_10363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_10363,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10366,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:1208: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}

/* k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_10366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_10366,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10369,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[10])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10475,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1210: cleanup */
f_9332(t3,((C_word*)t0)[10]);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_10369(2,av2);}}}

/* k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_10369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10369,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10372,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:1211: generate-foreign-callback-header */
f_10534(t2,lf[603],((C_word*)t0)[9]);}

/* k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_10372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,7)))){
C_save_and_reclaim((void *)f_10372,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10375,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_string_equal_p(lf[597],((C_word*)t0)[8]))){
/* c-backend.scm:1212: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(123);
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[598];
av2[5]=((C_word*)t0)[8];
av2[6]=lf[599];
av2[7]=lf[600];
f_4547(8,av2);}}
else{
/* c-backend.scm:1212: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(123);
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[598];
av2[5]=((C_word*)t0)[8];
av2[6]=lf[599];
av2[7]=lf[601];
f_4547(8,av2);}}}

/* k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_10375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10375,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10378,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:1214: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[596];
f_4547(4,av2);}}

/* k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in ... */
static void C_ccall f_10378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10378,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10390,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1219: scheme#reverse */
t3=*((C_word*)lf[595]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10385 in for-each-loop2208 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_10387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_10387,c,av);}
/* c-backend.scm:1217: gen */
t2=lf[4];{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[591];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
av2[6]=lf[592];
av2[7]=C_SCHEME_TRUE;
av2[8]=lf[593];
f_4547(9,av2);}}

/* k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in ... */
static void C_ccall f_10390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10390,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10393,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1220: scheme#reverse */
t3=*((C_word*)lf[595]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in ... */
static void C_ccall f_10393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10393,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10396,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10426,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_10426(t6,t2,((C_word*)t0)[6],t1);}

/* k10394 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_ccall f_10396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_10396,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10399,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_eqp(lf[580],((C_word*)t0)[3]);
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_10399(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10424,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1222: foreign-argument-conversion */
f_11704(t4,((C_word*)t0)[3]);}}

/* k10397 in k10394 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_10399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_10399,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10402,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1223: gen */
t3=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[589];
av2[3]=((C_word*)t0)[4];
av2[4]=C_make_character(44);
av2[5]=((C_word*)t0)[5];
av2[6]=C_make_character(41);
f_4547(7,av2);}}

/* k10400 in k10397 in k10394 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in ... */
static void C_ccall f_10402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10402,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10405,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_eqp(lf[580],((C_word*)t0)[3]);
if(C_truep(t3)){
/* c-backend.scm:1225: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[588];
f_4547(3,av2);}}
else{
/* c-backend.scm:1224: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_make_character(41);
f_4547(3,av2);}}}

/* k10403 in k10400 in k10397 in k10394 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in ... */
static void C_ccall f_10405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10405,c,av);}
/* c-backend.scm:1225: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[588];
f_4547(3,av2);}}

/* k10422 in k10394 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_10424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10424,c,av);}
/* c-backend.scm:1222: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[590];
av2[4]=t1;
f_4547(5,av2);}}

/* for-each-loop2208 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_fcall f_10426(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_10426,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10436,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t7=C_slot(t2,C_fix(0));
t8=C_slot(t3,C_fix(0));
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10387,a[2]=t6,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1217: foreign-result-conversion */
f_12369(t9,t8,lf[594]);}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k10434 in for-each-loop2208 in k10391 in k10388 in k10376 in k10373 in k10370 in k10367 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_10436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10436,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10426(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10473 in k10364 in k10361 in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_10475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10475,c,av);}
/* c-backend.scm:1210: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[604];
av2[4]=t1;
av2[5]=lf[605];
f_4547(6,av2);}}

/* loop in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_fcall f_10477(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_10477,5,t0,t1,t2,t3,t4);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_cdr(t2);
t6=C_i_cdr(t3);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10499,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,a[5]=t6,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1207: compute-size */
t8=((C_word*)((C_word*)t0)[3])[1];
f_9903(t8,t7,C_u_i_car(t2),C_u_i_car(t3),t4);}}

/* k10497 in loop in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_10499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10499,c,av);}
/* c-backend.scm:1206: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10477(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* for-each-loop2031 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10511(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_10511,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10521,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1164: g2032 */
t4=((C_word*)t0)[3];
f_9882(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10519 in for-each-loop2031 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10521,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10511(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10534(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_10534,3,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10538,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1229: chicken.compiler.support#foreign-callback-stub-name */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[676]);
C_word av2[3];
av2[0]=*((C_word*)lf[676]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_10538,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10541,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1230: chicken.compiler.support#foreign-callback-stub-qualifiers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[675]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[675]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10541,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10544,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1231: chicken.compiler.support#foreign-callback-stub-return-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[609]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[609]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10544,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10547,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1232: chicken.compiler.support#foreign-callback-stub-argument-types */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[608]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[608]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10547,c,av);}
a=C_alloc(8);
t2=C_i_length(t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10553,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:1234: make-argument-list */
f_9425(t3,t2,lf[674]);}

/* k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_10553,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10556,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10608,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1235: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=lf[673];
tp(4,av2);}}

/* k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10556(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_10556,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10559,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10564,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_10564(t6,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k10557 in k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10559,c,av);}
/* c-backend.scm:1241: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* loop in k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10564(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_10564,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10574,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10596,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1238: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t5;
av2[2]=C_i_car(t3);
av2[3]=C_i_car(t2);
tp(4,av2);}}}

/* k10572 in loop in k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10574,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10577,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[4])))){
/* c-backend.scm:1239: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
/* c-backend.scm:1240: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10564(t3,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}}

/* k10575 in k10572 in loop in k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10577,c,av);}
/* c-backend.scm:1240: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10564(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}

/* k10594 in loop in k10554 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10596,c,av);}
/* c-backend.scm:1238: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_4547(3,av2);}}

/* k10606 in k10551 in k10545 in k10542 in k10539 in k10536 in chicken.compiler.c-backend#generate-foreign-callback-header in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_10608,c,av);}
/* c-backend.scm:1235: gen */
t2=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=C_make_character(32);
av2[5]=t1;
av2[6]=((C_word*)t0)[4];
av2[7]=C_make_character(32);
av2[8]=((C_word*)t0)[5];
av2[9]=C_make_character(40);
f_4547(10,av2);}}

/* chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_10610,c,av);}
a=C_alloc(13);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10612,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10617,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t6=C_eqp(t2,lf[582]);
if(C_truep(t6)){
/* c-backend.scm:1250: str */
t7=t5;
f_10617(t7,t1,lf[680]);}
else{
t7=C_eqp(t2,lf[17]);
t8=(C_truep(t7)?t7:C_eqp(t2,lf[586]));
if(C_truep(t8)){
/* c-backend.scm:1251: str */
t9=t5;
f_10617(t9,t1,lf[681]);}
else{
t9=C_eqp(t2,lf[583]);
t10=(C_truep(t9)?t9:C_eqp(t2,lf[587]));
if(C_truep(t10)){
/* c-backend.scm:1252: str */
t11=t5;
f_10617(t11,t1,lf[682]);}
else{
t11=C_eqp(t2,lf[584]);
t12=(C_truep(t11)?t11:C_eqp(t2,lf[535]));
if(C_truep(t12)){
/* c-backend.scm:1253: str */
t13=t5;
f_10617(t13,t1,lf[683]);}
else{
t13=C_eqp(t2,lf[585]);
t14=(C_truep(t13)?t13:C_eqp(t2,lf[568]));
if(C_truep(t14)){
/* c-backend.scm:1254: str */
t15=t5;
f_10617(t15,t1,lf[684]);}
else{
t15=C_eqp(t2,lf[577]);
t16=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10687,a[2]=t5,a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t15)){
t17=t16;
f_10687(t17,t15);}
else{
t17=C_eqp(t2,lf[570]);
t18=t16;
f_10687(t18,(C_truep(t17)?t17:C_eqp(t2,lf[14])));}}}}}}}

/* err in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10612(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_10612,2,t0,t1);}
/* c-backend.scm:1247: chicken.compiler.support#quit-compiling */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[677]);
C_word av2[4];
av2[0]=*((C_word*)lf[677]+1);
av2[1]=t1;
av2[2]=lf[678];
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}

/* str in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10617(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_10617,3,t0,t1,t2);}
/* c-backend.scm:1248: scheme#string-append */
t3=*((C_word*)lf[152]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[679];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10687(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10687,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1255: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],lf[685]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[566]);
if(C_truep(t2)){
/* c-backend.scm:1256: str */
t3=((C_word*)t0)[2];
f_10617(t3,((C_word*)t0)[3],lf[686]);}
else{
t3=C_eqp(((C_word*)t0)[4],lf[567]);
if(C_truep(t3)){
/* c-backend.scm:1257: str */
t4=((C_word*)t0)[2];
f_10617(t4,((C_word*)t0)[3],lf[687]);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[578]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[571]));
if(C_truep(t5)){
/* c-backend.scm:1258: str */
t6=((C_word*)t0)[2];
f_10617(t6,((C_word*)t0)[3],lf[688]);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[565]);
if(C_truep(t6)){
/* c-backend.scm:1259: str */
t7=((C_word*)t0)[2];
f_10617(t7,((C_word*)t0)[3],lf[689]);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[537]);
if(C_truep(t7)){
/* c-backend.scm:1260: str */
t8=((C_word*)t0)[2];
f_10617(t8,((C_word*)t0)[3],lf[690]);}
else{
t8=C_eqp(((C_word*)t0)[4],lf[579]);
if(C_truep(t8)){
/* c-backend.scm:1261: str */
t9=((C_word*)t0)[2];
f_10617(t9,((C_word*)t0)[3],lf[691]);}
else{
t9=C_eqp(((C_word*)t0)[4],lf[569]);
if(C_truep(t9)){
/* c-backend.scm:1262: str */
t10=((C_word*)t0)[2];
f_10617(t10,((C_word*)t0)[3],lf[692]);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[581]);
if(C_truep(t10)){
/* c-backend.scm:1263: str */
t11=((C_word*)t0)[2];
f_10617(t11,((C_word*)t0)[3],lf[693]);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[572]);
if(C_truep(t11)){
/* c-backend.scm:1264: str */
t12=((C_word*)t0)[2];
f_10617(t12,((C_word*)t0)[3],lf[694]);}
else{
t12=C_eqp(((C_word*)t0)[4],lf[533]);
if(C_truep(t12)){
/* c-backend.scm:1265: str */
t13=((C_word*)t0)[2];
f_10617(t13,((C_word*)t0)[3],lf[695]);}
else{
t13=C_eqp(((C_word*)t0)[4],lf[574]);
t14=(C_truep(t13)?t13:C_eqp(((C_word*)t0)[4],lf[573]));
if(C_truep(t14)){
/* c-backend.scm:1266: str */
t15=((C_word*)t0)[2];
f_10617(t15,((C_word*)t0)[3],lf[696]);}
else{
t15=C_eqp(((C_word*)t0)[4],lf[550]);
t16=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10804,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t15)){
t17=t16;
f_10804(t17,t15);}
else{
t17=C_eqp(((C_word*)t0)[4],lf[552]);
if(C_truep(t17)){
t18=t16;
f_10804(t18,t17);}
else{
t18=C_eqp(((C_word*)t0)[4],lf[768]);
t19=t16;
f_10804(t19,(C_truep(t18)?t18:C_eqp(((C_word*)t0)[4],lf[769])));}}}}}}}}}}}}}}}

/* k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10804(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10804,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1267: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],lf[697]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[575]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[4],lf[576]));
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[698];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[699]);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10825,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t4)){
t6=t5;
f_10825(t6,t4);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[765]);
if(C_truep(t6)){
t7=t5;
f_10825(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[766]);
t8=t5;
f_10825(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[4],lf[767])));}}}}}

/* k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10825(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10825,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1269: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],lf[700]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[701]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[4],lf[702]));
if(C_truep(t3)){
/* c-backend.scm:1270: str */
t4=((C_word*)t0)[2];
f_10617(t4,((C_word*)t0)[3],lf[703]);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[704]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[705]));
if(C_truep(t5)){
/* c-backend.scm:1271: str */
t6=((C_word*)t0)[2];
f_10617(t6,((C_word*)t0)[3],lf[706]);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[707]);
t7=(C_truep(t6)?t6:C_eqp(((C_word*)t0)[4],lf[708]));
if(C_truep(t7)){
/* c-backend.scm:1272: str */
t8=((C_word*)t0)[2];
f_10617(t8,((C_word*)t0)[3],lf[709]);}
else{
t8=C_eqp(((C_word*)t0)[4],lf[710]);
t9=(C_truep(t8)?t8:C_eqp(((C_word*)t0)[4],lf[711]));
if(C_truep(t9)){
/* c-backend.scm:1273: str */
t10=((C_word*)t0)[2];
f_10617(t10,((C_word*)t0)[3],lf[712]);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[713]);
t11=(C_truep(t10)?t10:C_eqp(((C_word*)t0)[4],lf[714]));
if(C_truep(t11)){
/* c-backend.scm:1274: str */
t12=((C_word*)t0)[2];
f_10617(t12,((C_word*)t0)[3],lf[715]);}
else{
t12=C_eqp(((C_word*)t0)[4],lf[716]);
t13=(C_truep(t12)?t12:C_eqp(((C_word*)t0)[4],lf[717]));
if(C_truep(t13)){
/* c-backend.scm:1275: str */
t14=((C_word*)t0)[2];
f_10617(t14,((C_word*)t0)[3],lf[718]);}
else{
t14=C_eqp(((C_word*)t0)[4],lf[719]);
t15=(C_truep(t14)?t14:C_eqp(((C_word*)t0)[4],lf[720]));
if(C_truep(t15)){
/* c-backend.scm:1276: str */
t16=((C_word*)t0)[2];
f_10617(t16,((C_word*)t0)[3],lf[721]);}
else{
t16=C_eqp(((C_word*)t0)[4],lf[722]);
t17=(C_truep(t16)?t16:C_eqp(((C_word*)t0)[4],lf[723]));
if(C_truep(t17)){
/* c-backend.scm:1277: str */
t18=((C_word*)t0)[2];
f_10617(t18,((C_word*)t0)[3],lf[724]);}
else{
t18=C_eqp(((C_word*)t0)[4],lf[725]);
t19=(C_truep(t18)?t18:C_eqp(((C_word*)t0)[4],lf[726]));
if(C_truep(t19)){
/* c-backend.scm:1278: str */
t20=((C_word*)t0)[2];
f_10617(t20,((C_word*)t0)[3],lf[727]);}
else{
t20=C_eqp(((C_word*)t0)[4],lf[728]);
t21=(C_truep(t20)?t20:C_eqp(((C_word*)t0)[4],lf[729]));
if(C_truep(t21)){
/* c-backend.scm:1279: str */
t22=((C_word*)t0)[2];
f_10617(t22,((C_word*)t0)[3],lf[730]);}
else{
t22=C_eqp(((C_word*)t0)[4],lf[543]);
t23=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10957,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t22)){
t24=t23;
f_10957(t24,t22);}
else{
t24=C_eqp(((C_word*)t0)[4],lf[539]);
if(C_truep(t24)){
t25=t23;
f_10957(t25,t24);}
else{
t25=C_eqp(((C_word*)t0)[4],lf[558]);
if(C_truep(t25)){
t26=t23;
f_10957(t26,t25);}
else{
t26=C_eqp(((C_word*)t0)[4],lf[562]);
t27=t23;
f_10957(t27,(C_truep(t26)?t26:C_eqp(((C_word*)t0)[4],lf[561])));}}}}}}}}}}}}}}}

/* k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10957(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10957,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1281: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],lf[731]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[559]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10969,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_10969(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[560]);
if(C_truep(t4)){
t5=t3;
f_10969(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[563]);
t6=t3;
f_10969(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[4],lf[564])));}}}}

/* k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10969(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10969,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1283: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],lf[732]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[580]);
if(C_truep(t2)){
/* c-backend.scm:1284: str */
t3=((C_word*)t0)[2];
f_10617(t3,((C_word*)t0)[3],lf[733]);}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10984,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[4]))){
/* c-backend.scm:1286: chicken.compiler.support#lookup-foreign-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[557]);
C_word av2[3];
av2[0]=*((C_word*)lf[557]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_10984(2,av2);}}}}}

/* k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_10984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10984,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10988,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1286: g2377 */
t3=t2;
f_10988(t3,((C_word*)t0)[3],t1);}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[4]))){
/* c-backend.scm:1289: str */
t2=((C_word*)t0)[5];
f_10617(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
if(C_truep(C_i_listp(((C_word*)t0)[4]))){
t2=C_u_i_length(((C_word*)t0)[4]);
t3=C_eqp(C_fix(2),t2);
t4=(C_truep(t3)?C_u_i_memq(C_i_car(((C_word*)t0)[4]),lf[734]):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_i_cadr(((C_word*)t0)[4]);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11032,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1297: scheme#string-append */
t7=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[735];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t5=C_eqp(C_fix(2),t2);
t6=(C_truep(t5)?C_eqp(lf[546],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[4]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11049,a[2]=((C_word*)t0)[3],a[3]=t7,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1300: scheme#string-append */
t9=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[736];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t7=C_fixnum_greaterp(t2,C_fix(2));
t8=(C_truep(t7)?C_eqp(lf[737],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11062,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11066,a[2]=t9,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1305: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t10;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
av2[3]=lf[742];
tp(4,av2);}}
else{
t9=C_eqp(t2,C_fix(2));
t10=(C_truep(t9)?C_eqp(lf[548],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11139,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1312: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t11;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}
else{
t11=C_eqp(t2,C_fix(2));
t12=(C_truep(t11)?C_eqp(lf[744],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t12)){
t13=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11156,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1314: chicken.string#->string */
t14=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t14;
av2[1]=t13;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t14+1)))(3,av2);}}
else{
t13=C_eqp(t2,C_fix(2));
t14=(C_truep(t13)?C_eqp(lf[747],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t14)){
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11173,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1316: chicken.string#->string */
t16=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t16;
av2[1]=t15;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t16+1)))(3,av2);}}
else{
t15=C_eqp(t2,C_fix(2));
t16=(C_truep(t15)?C_eqp(lf[750],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t16)){
t17=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11190,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1318: chicken.string#->string */
t18=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t18;
av2[1]=t17;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t18+1)))(3,av2);}}
else{
t17=C_eqp(t2,C_fix(3));
t18=(C_truep(t17)?C_u_i_memq(C_i_car(((C_word*)t0)[4]),lf[753]):C_SCHEME_FALSE);
if(C_truep(t18)){
t19=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11207,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1320: chicken.string#->string */
t20=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t20;
av2[1]=t19;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t20+1)))(3,av2);}}
else{
t19=C_eqp(t2,C_fix(3));
t20=(C_truep(t19)?C_eqp(lf[556],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t20)){
t21=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11224,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1322: chicken.string#->string */
t22=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t22;
av2[1]=t21;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t22+1)))(3,av2);}}
else{
t21=C_fixnum_greater_or_equal_p(t2,C_fix(3));
t22=(C_truep(t21)?C_eqp(lf[553],C_i_car(((C_word*)t0)[4])):C_SCHEME_FALSE);
if(C_truep(t22)){
t23=C_i_cadr(((C_word*)t0)[4]);
t24=C_i_caddr(((C_word*)t0)[4]);
t25=C_u_i_cdr(((C_word*)t0)[4]);
t26=C_u_i_cdr(t25);
t27=C_u_i_cdr(t26);
t28=C_i_nullp(t27);
t29=(C_truep(t28)?lf[756]:C_i_car(t27));
t30=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11253,a[2]=((C_word*)t0)[3],a[3]=t29,a[4]=((C_word*)t0)[2],a[5]=t24,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1328: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t30;
av2[2]=t23;
av2[3]=lf[764];
tp(4,av2);}}
else{
/* c-backend.scm:1339: err */
t23=((C_word*)t0)[6];
f_10612(t23,((C_word*)t0)[3]);}}}}}}}}}}}
else{
/* c-backend.scm:1340: err */
t2=((C_word*)t0)[6];
f_10612(t2,((C_word*)t0)[3]);}}}}

/* g2377 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_10988(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_10988,3,t0,t1,t2);}
/* c-backend.scm:1288: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t1;
av2[2]=C_i_vector_ref(t2,C_fix(0));
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}

/* k11030 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11032,c,av);}
/* c-backend.scm:1297: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k11047 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11049,c,av);}
/* c-backend.scm:1300: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k11060 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11062(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11062,c,av);}
/* c-backend.scm:1303: str */
t2=((C_word*)t0)[2];
f_10617(t2,((C_word*)t0)[3],t1);}

/* k11064 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11066(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_11066,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11070,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_cddr(((C_word*)t0)[3]);
t8=C_i_check_list_2(t7,lf[308]);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11088,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11090,a[2]=t5,a[3]=t11,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t13=((C_word*)t11)[1];
f_11090(t13,t9,t7);}

/* k11068 in k11064 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11070,c,av);}
/* c-backend.scm:1304: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[738];
av2[4]=t1;
av2[5]=lf[739];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k11086 in k11064 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11088,c,av);}
/* c-backend.scm:1307: chicken.string#string-intersperse */
t2=C_fast_retrieve(lf[301]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[740];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop2390 in k11064 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11090(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_11090,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11115,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[243]+1);
/* c-backend.scm:1308: g2413 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[741];
tp(4,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11113 in map-loop2390 in k11064 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11115,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11090(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11137 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11139,c,av);}
/* c-backend.scm:1312: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[743];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11154 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11156(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11156,c,av);}
/* c-backend.scm:1314: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[745];
av2[3]=t1;
av2[4]=lf[746];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k11171 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11173,c,av);}
/* c-backend.scm:1316: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[748];
av2[3]=t1;
av2[4]=lf[749];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k11188 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11190,c,av);}
/* c-backend.scm:1318: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[751];
av2[3]=t1;
av2[4]=lf[752];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k11205 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11207,c,av);}
/* c-backend.scm:1320: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[754];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11222 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11224(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11224,c,av);}
/* c-backend.scm:1322: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[755];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_11253,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11257,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[5],lf[308]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11278,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11280,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_11280(t12,t8,((C_word*)t0)[5]);}

/* k11255 in k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_11257,c,av);}
/* c-backend.scm:1327: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[757];
av2[5]=((C_word*)t0)[5];
av2[6]=lf[758];
av2[7]=t1;
av2[8]=lf[759];
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}

/* k11276 in k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11278,c,av);}
/* c-backend.scm:1331: chicken.string#string-intersperse */
t2=C_fast_retrieve(lf[301]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[760];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop2438 in k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11280(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_11280,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11290,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11305,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=C_slot(t2,C_fix(0));
t6=C_eqp(lf[761],t5);
if(C_truep(t6)){
t7=t3;
f_11290(t7,C_a_i_cons(&a,2,lf[762],C_SCHEME_END_OF_LIST));}
else{
/* c-backend.scm:1335: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t4;
av2[2]=t5;
av2[3]=lf[763];
tp(4,av2);}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11288 in map-loop2438 in k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11290(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_11290,2,t0,t1);}
t2=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t1);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t4=((C_word*)((C_word*)t0)[3])[1];
f_11280(t4,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11303 in map-loop2438 in k11251 in k10982 in k10967 in k10955 in k10823 in k10802 in k10685 in chicken.compiler.c-backend#foreign-type-declaration in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_11305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_11305,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];
f_11290(t2,C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST));}

/* chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11704(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_11704,2,t1,t2);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11706,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=C_eqp(t2,lf[582]);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[771];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(t2,lf[17]);
t6=(C_truep(t5)?t5:C_eqp(t2,lf[583]));
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=lf[772];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t2,lf[586]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11734,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t7)){
t9=t8;
f_11734(t9,t7);}
else{
t9=C_eqp(t2,lf[577]);
if(C_truep(t9)){
t10=t8;
f_11734(t10,t9);}
else{
t10=C_eqp(t2,lf[578]);
if(C_truep(t10)){
t11=t8;
f_11734(t11,t10);}
else{
t11=C_eqp(t2,lf[584]);
if(C_truep(t11)){
t12=t8;
f_11734(t12,t11);}
else{
t12=C_eqp(t2,lf[585]);
t13=t8;
f_11734(t13,(C_truep(t12)?t12:C_eqp(t2,lf[587])));}}}}}}}

/* err in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11706(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_11706,2,t0,t1);}
/* c-backend.scm:1347: chicken.compiler.support#quit-compiling */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[677]);
C_word av2[4];
av2[0]=*((C_word*)lf[677]+1);
av2[1]=t1;
av2[2]=lf[770];
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}

/* k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11734(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11734,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[773];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[579]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[774];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[581]);
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[775];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[572]);
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[776];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[3],lf[574]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11761,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_11761(t7,t5);}
else{
t7=C_eqp(((C_word*)t0)[3],lf[573]);
t8=t6;
f_11761(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[3],lf[533])));}}}}}}

/* k11759 in k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11761(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11761,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[777];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[570]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[3],lf[571]));
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[778];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[565]);
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[779];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[3],lf[566]);
if(C_truep(t5)){
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[780];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[3],lf[567]);
if(C_truep(t6)){
t7=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t7;
av2[1]=lf[781];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[3],lf[537]);
if(C_truep(t7)){
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[782];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(((C_word*)t0)[3],lf[569]);
if(C_truep(t8)){
t9=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[783];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(((C_word*)t0)[3],lf[535]);
t10=(C_truep(t9)?t9:C_eqp(((C_word*)t0)[3],lf[568]));
if(C_truep(t10)){
t11=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t11;
av2[1]=lf[784];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(((C_word*)t0)[3],lf[768]);
if(C_truep(t11)){
t12=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t12;
av2[1]=lf[785];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=C_eqp(((C_word*)t0)[3],lf[769]);
if(C_truep(t12)){
t13=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t13;
av2[1]=lf[786];
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t13=C_eqp(((C_word*)t0)[3],lf[550]);
if(C_truep(t13)){
t14=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t14;
av2[1]=lf[787];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_eqp(((C_word*)t0)[3],lf[552]);
if(C_truep(t14)){
t15=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t15;
av2[1]=lf[788];
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t15=C_eqp(((C_word*)t0)[3],lf[699]);
if(C_truep(t15)){
t16=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t16;
av2[1]=lf[789];
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}
else{
t16=C_eqp(((C_word*)t0)[3],lf[765]);
if(C_truep(t16)){
t17=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t17;
av2[1]=lf[790];
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
t17=C_eqp(((C_word*)t0)[3],lf[766]);
if(C_truep(t17)){
t18=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t18;
av2[1]=lf[791];
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
t18=C_eqp(((C_word*)t0)[3],lf[767]);
if(C_truep(t18)){
t19=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t19;
av2[1]=lf[792];
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t19=C_eqp(((C_word*)t0)[3],lf[701]);
if(C_truep(t19)){
t20=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t20;
av2[1]=lf[793];
((C_proc)(void*)(*((C_word*)t20+1)))(2,av2);}}
else{
t20=C_eqp(((C_word*)t0)[3],lf[702]);
if(C_truep(t20)){
t21=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t21;
av2[1]=lf[794];
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}
else{
t21=C_eqp(((C_word*)t0)[3],lf[707]);
if(C_truep(t21)){
t22=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t22;
av2[1]=lf[795];
((C_proc)(void*)(*((C_word*)t22+1)))(2,av2);}}
else{
t22=C_eqp(((C_word*)t0)[3],lf[708]);
if(C_truep(t22)){
t23=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t23;
av2[1]=lf[796];
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}
else{
t23=C_eqp(((C_word*)t0)[3],lf[710]);
if(C_truep(t23)){
t24=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t24;
av2[1]=lf[797];
((C_proc)(void*)(*((C_word*)t24+1)))(2,av2);}}
else{
t24=C_eqp(((C_word*)t0)[3],lf[711]);
if(C_truep(t24)){
t25=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t25;
av2[1]=lf[798];
((C_proc)(void*)(*((C_word*)t25+1)))(2,av2);}}
else{
t25=C_eqp(((C_word*)t0)[3],lf[704]);
if(C_truep(t25)){
t26=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t26;
av2[1]=lf[799];
((C_proc)(void*)(*((C_word*)t26+1)))(2,av2);}}
else{
t26=C_eqp(((C_word*)t0)[3],lf[705]);
if(C_truep(t26)){
t27=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t27;
av2[1]=lf[800];
((C_proc)(void*)(*((C_word*)t27+1)))(2,av2);}}
else{
t27=C_eqp(((C_word*)t0)[3],lf[713]);
if(C_truep(t27)){
t28=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t28;
av2[1]=lf[801];
((C_proc)(void*)(*((C_word*)t28+1)))(2,av2);}}
else{
t28=C_eqp(((C_word*)t0)[3],lf[714]);
if(C_truep(t28)){
t29=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t29;
av2[1]=lf[802];
((C_proc)(void*)(*((C_word*)t29+1)))(2,av2);}}
else{
t29=C_eqp(((C_word*)t0)[3],lf[716]);
if(C_truep(t29)){
t30=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t30;
av2[1]=lf[803];
((C_proc)(void*)(*((C_word*)t30+1)))(2,av2);}}
else{
t30=C_eqp(((C_word*)t0)[3],lf[717]);
if(C_truep(t30)){
t31=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t31;
av2[1]=lf[804];
((C_proc)(void*)(*((C_word*)t31+1)))(2,av2);}}
else{
t31=C_eqp(((C_word*)t0)[3],lf[719]);
if(C_truep(t31)){
t32=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t32;
av2[1]=lf[805];
((C_proc)(void*)(*((C_word*)t32+1)))(2,av2);}}
else{
t32=C_eqp(((C_word*)t0)[3],lf[720]);
if(C_truep(t32)){
t33=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t33;
av2[1]=lf[806];
((C_proc)(void*)(*((C_word*)t33+1)))(2,av2);}}
else{
t33=C_eqp(((C_word*)t0)[3],lf[722]);
if(C_truep(t33)){
t34=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t34;
av2[1]=lf[807];
((C_proc)(void*)(*((C_word*)t34+1)))(2,av2);}}
else{
t34=C_eqp(((C_word*)t0)[3],lf[723]);
if(C_truep(t34)){
t35=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t35;
av2[1]=lf[808];
((C_proc)(void*)(*((C_word*)t35+1)))(2,av2);}}
else{
t35=C_eqp(((C_word*)t0)[3],lf[725]);
if(C_truep(t35)){
t36=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t36;
av2[1]=lf[809];
((C_proc)(void*)(*((C_word*)t36+1)))(2,av2);}}
else{
t36=C_eqp(((C_word*)t0)[3],lf[726]);
if(C_truep(t36)){
t37=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t37;
av2[1]=lf[810];
((C_proc)(void*)(*((C_word*)t37+1)))(2,av2);}}
else{
t37=C_eqp(((C_word*)t0)[3],lf[728]);
if(C_truep(t37)){
t38=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t38;
av2[1]=lf[811];
((C_proc)(void*)(*((C_word*)t38+1)))(2,av2);}}
else{
t38=C_eqp(((C_word*)t0)[3],lf[729]);
if(C_truep(t38)){
t39=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t39;
av2[1]=lf[812];
((C_proc)(void*)(*((C_word*)t39+1)))(2,av2);}}
else{
t39=C_eqp(((C_word*)t0)[3],lf[539]);
t40=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11986,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t39)){
t41=t40;
f_11986(t41,t39);}
else{
t41=C_eqp(((C_word*)t0)[3],lf[562]);
if(C_truep(t41)){
t42=t40;
f_11986(t42,t41);}
else{
t42=C_eqp(((C_word*)t0)[3],lf[563]);
t43=t40;
f_11986(t43,(C_truep(t42)?t42:C_eqp(((C_word*)t0)[3],lf[564])));}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k11984 in k11759 in k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11986(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11986,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[813];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[543]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t2)){
t4=t3;
f_11995(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[558]);
if(C_truep(t4)){
t5=t3;
f_11995(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[559]);
if(C_truep(t5)){
t6=t3;
f_11995(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[560]);
t7=t3;
f_11995(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[3],lf[561])));}}}}}

/* k11993 in k11984 in k11759 in k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_11995(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11995,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[814];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[14]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[815];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12004,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
/* c-backend.scm:1396: chicken.compiler.support#lookup-foreign-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[557]);
C_word av2[3];
av2[0]=*((C_word*)lf[557]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_12004(2,av2);}}}}}

/* k12002 in k11993 in k11984 in k11759 in k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12004,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
/* c-backend.scm:1398: foreign-argument-conversion */
f_11704(((C_word*)t0)[2],C_i_vector_ref(t1,C_fix(0)));}
else{
t2=C_i_listp(((C_word*)t0)[3]);
t3=(C_truep(t2)?C_fixnum_greater_or_equal_p(C_u_i_length(((C_word*)t0)[3]),C_fix(2)):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[3]);
t5=C_eqp(t4,lf[550]);
if(C_truep(t5)){
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=lf[816];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(t4,lf[552]);
if(C_truep(t6)){
t7=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[817];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t4,lf[554]);
if(C_truep(t7)){
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=lf[818];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(t4,lf[555]);
if(C_truep(t8)){
t9=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t9;
av2[1]=lf[819];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(t4,lf[768]);
if(C_truep(t9)){
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=lf[820];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(t4,lf[769]);
if(C_truep(t10)){
t11=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t11;
av2[1]=lf[821];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(t4,lf[553]);
if(C_truep(t11)){
t12=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t12;
av2[1]=lf[822];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=C_eqp(t4,lf[548]);
if(C_truep(t12)){
/* c-backend.scm:1408: foreign-argument-conversion */
f_11704(((C_word*)t0)[2],C_i_cadr(((C_word*)t0)[3]));}
else{
t13=C_eqp(t4,lf[750]);
if(C_truep(t13)){
t14=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t14;
av2[1]=lf[823];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_eqp(t4,lf[546]);
if(C_truep(t14)){
t15=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12102,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1411: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t15;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
av2[3]=lf[826];
tp(4,av2);}}
else{
t15=C_eqp(t4,lf[556]);
if(C_truep(t15)){
/* c-backend.scm:1414: scheme#string-append */
t16=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t16;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[827];
av2[3]=C_i_cadr(((C_word*)t0)[3]);
av2[4]=lf[828];
((C_proc)(void*)(*((C_word*)t16+1)))(5,av2);}}
else{
/* c-backend.scm:1415: err */
t16=((C_word*)t0)[4];
f_11706(t16,((C_word*)t0)[2]);}}}}}}}}}}}}
else{
/* c-backend.scm:1416: err */
t4=((C_word*)t0)[4];
f_11706(t4,((C_word*)t0)[2]);}}}

/* k12100 in k12002 in k11993 in k11984 in k11759 in k11732 in chicken.compiler.c-backend#foreign-argument-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12102(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12102,c,av);}
/* c-backend.scm:1411: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[824];
av2[3]=t1;
av2[4]=lf[825];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_12369(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_12369,3,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12371,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=C_eqp(t2,lf[17]);
t6=(C_truep(t5)?t5:C_eqp(t2,lf[583]));
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=lf[830];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t2,lf[577]);
t8=(C_truep(t7)?t7:C_eqp(t2,lf[578]));
if(C_truep(t8)){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[831];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(t2,lf[584]);
t10=(C_truep(t9)?t9:C_eqp(t2,lf[585]));
if(C_truep(t10)){
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=lf[832];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(t2,lf[579]);
if(C_truep(t11)){
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=lf[833];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=C_eqp(t2,lf[581]);
if(C_truep(t12)){
t13=t1;{
C_word av2[2];
av2[0]=t13;
av2[1]=lf[834];
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t13=C_eqp(t2,lf[586]);
if(C_truep(t13)){
t14=t1;{
C_word av2[2];
av2[0]=t14;
av2[1]=lf[835];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_eqp(t2,lf[587]);
if(C_truep(t14)){
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=lf[836];
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t15=C_eqp(t2,lf[533]);
t16=(C_truep(t15)?t15:C_eqp(t2,lf[574]));
if(C_truep(t16)){
t17=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12438,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1432: chicken.base#open-output-string */
t18=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t18;
av2[1]=t17;
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
t17=C_eqp(t2,lf[573]);
if(C_truep(t17)){
t18=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12462,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1433: chicken.base#open-output-string */
t19=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t19;
av2[1]=t18;
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t18=C_eqp(t2,lf[543]);
t19=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12486,a[2]=t1,a[3]=t3,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
if(C_truep(t18)){
t20=t19;
f_12486(t20,t18);}
else{
t20=C_eqp(t2,lf[539]);
if(C_truep(t20)){
t21=t19;
f_12486(t21,t20);}
else{
t21=C_eqp(t2,lf[552]);
if(C_truep(t21)){
t22=t19;
f_12486(t22,t21);}
else{
t22=C_eqp(t2,lf[562]);
if(C_truep(t22)){
t23=t19;
f_12486(t23,t22);}
else{
t23=C_eqp(t2,lf[558]);
if(C_truep(t23)){
t24=t19;
f_12486(t24,t23);}
else{
t24=C_eqp(t2,lf[563]);
if(C_truep(t24)){
t25=t19;
f_12486(t25,t24);}
else{
t25=C_eqp(t2,lf[564]);
if(C_truep(t25)){
t26=t19;
f_12486(t26,t25);}
else{
t26=C_eqp(t2,lf[559]);
if(C_truep(t26)){
t27=t19;
f_12486(t27,t26);}
else{
t27=C_eqp(t2,lf[560]);
if(C_truep(t27)){
t28=t19;
f_12486(t28,t27);}
else{
t28=C_eqp(t2,lf[561]);
if(C_truep(t28)){
t29=t19;
f_12486(t29,t28);}
else{
t29=C_eqp(t2,lf[575]);
t30=t19;
f_12486(t30,(C_truep(t29)?t29:C_eqp(t2,lf[576])));}}}}}}}}}}}}}}}}}}}}

/* err in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_12371(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_12371,2,t0,t1);}
/* c-backend.scm:1423: chicken.compiler.support#quit-compiling */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[677]);
C_word av2[4];
av2[0]=*((C_word*)lf[677]+1);
av2[1]=t1;
av2[2]=lf[829];
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}

/* k12436 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12438,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12444,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1432: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[837];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12442 in k12436 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12444,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12447,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1432: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12445 in k12442 in k12436 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12447,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12450,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1432: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12448 in k12445 in k12442 in k12436 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12450,c,av);}
/* c-backend.scm:1432: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12460 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12462,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12468,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1433: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[838];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12466 in k12460 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12468,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12471,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1433: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12469 in k12466 in k12460 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12471,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12474,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1433: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12472 in k12469 in k12466 in k12460 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12474,c,av);}
/* c-backend.scm:1433: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_12486(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12486,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12489,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1437: chicken.base#open-output-string */
t3=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[550]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12513,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1438: chicken.base#open-output-string */
t4=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[4],lf[570]);
t4=(C_truep(t3)?t3:C_eqp(((C_word*)t0)[4],lf[571]));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12540,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1439: chicken.base#open-output-string */
t6=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[4],lf[565]);
t6=(C_truep(t5)?t5:C_eqp(((C_word*)t0)[4],lf[567]));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12567,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1440: chicken.base#open-output-string */
t8=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[4],lf[537]);
t8=(C_truep(t7)?t7:C_eqp(((C_word*)t0)[4],lf[566]));
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12594,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1441: chicken.base#open-output-string */
t10=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=C_eqp(((C_word*)t0)[4],lf[535]);
t10=(C_truep(t9)?t9:C_eqp(((C_word*)t0)[4],lf[568]));
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1442: chicken.base#open-output-string */
t12=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t12;
av2[1]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t11=C_eqp(((C_word*)t0)[4],lf[569]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12645,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1443: chicken.base#open-output-string */
t13=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t13;
av2[1]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t12=C_eqp(((C_word*)t0)[4],lf[572]);
if(C_truep(t12)){
t13=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12669,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1444: chicken.base#open-output-string */
t14=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t14;
av2[1]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t13=C_eqp(((C_word*)t0)[4],lf[14]);
if(C_truep(t13)){
t14=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t14;
av2[1]=lf[849];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_eqp(((C_word*)t0)[4],lf[580]);
t15=(C_truep(t14)?t14:C_eqp(((C_word*)t0)[4],lf[582]));
if(C_truep(t15)){
t16=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t16;
av2[1]=lf[850];
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}
else{
t16=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12702,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[4]))){
/* c-backend.scm:1448: chicken.compiler.support#lookup-foreign-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[557]);
C_word av2[3];
av2[0]=*((C_word*)lf[557]+1);
av2[1]=t16;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
t17=t16;{
C_word av2[2];
av2[0]=t17;
av2[1]=C_SCHEME_FALSE;
f_12702(2,av2);}}}}}}}}}}}}}

/* k12487 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12489(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12489,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12495,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1437: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[840];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12493 in k12487 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12495,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12498,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1437: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12496 in k12493 in k12487 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12498,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12501,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1437: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[839];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12499 in k12496 in k12493 in k12487 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12501,c,av);}
/* c-backend.scm:1437: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12511 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12513,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12519,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1438: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[842];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12517 in k12511 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12519,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12522,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1438: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12520 in k12517 in k12511 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12522,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12525,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1438: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[841];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12523 in k12520 in k12517 in k12511 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12525,c,av);}
/* c-backend.scm:1438: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12538 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12540,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12546,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1439: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[843];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12544 in k12538 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12546,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12549,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1439: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12547 in k12544 in k12538 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12549,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12552,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1439: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12550 in k12547 in k12544 in k12538 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12552(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12552,c,av);}
/* c-backend.scm:1439: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12565 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12567,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12573,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1440: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[844];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12571 in k12565 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12573(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12573,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12576,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1440: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12574 in k12571 in k12565 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12576,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12579,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1440: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12577 in k12574 in k12571 in k12565 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12579,c,av);}
/* c-backend.scm:1440: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12592 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12594,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12600,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1441: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[845];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12598 in k12592 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12600,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12603,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1441: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12601 in k12598 in k12592 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12603,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12606,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1441: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12604 in k12601 in k12598 in k12592 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12606,c,av);}
/* c-backend.scm:1441: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12619 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12621,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12627,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1442: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[846];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12625 in k12619 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12627,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12630,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1442: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12628 in k12625 in k12619 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12630,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12633,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1442: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12631 in k12628 in k12625 in k12619 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12633,c,av);}
/* c-backend.scm:1442: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12643 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12645,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12651,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1443: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[847];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12649 in k12643 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12651,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12654,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1443: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12652 in k12649 in k12643 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12654,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12657,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1443: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12655 in k12652 in k12649 in k12643 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12657,c,av);}
/* c-backend.scm:1443: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12667 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12669,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12675,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1444: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[848];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12673 in k12667 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12675,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12678,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1444: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12676 in k12673 in k12667 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12678,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12681,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1444: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12679 in k12676 in k12673 in k12667 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12681,c,av);}
/* c-backend.scm:1444: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12702,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12706,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1448: g2737 */
t3=t2;
f_12706(t3,((C_word*)t0)[3],t1);}
else{
t2=C_i_listp(((C_word*)t0)[4]);
t3=(C_truep(t2)?C_fixnum_greater_or_equal_p(C_u_i_length(((C_word*)t0)[4]),C_fix(2)):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[4]);
t5=C_eqp(t4,lf[551]);
t6=(C_truep(t5)?t5:C_eqp(t4,lf[552]));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12738,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1454: chicken.base#open-output-string */
t8=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(t4,lf[546]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12762,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1456: chicken.base#open-output-string */
t9=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_eqp(t4,lf[554]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12786,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1458: chicken.base#open-output-string */
t10=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=C_eqp(t4,lf[555]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12810,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1460: chicken.base#open-output-string */
t11=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t10=C_eqp(t4,lf[556]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12834,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1462: chicken.base#open-output-string */
t12=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t11=C_eqp(t4,lf[548]);
if(C_truep(t11)){
/* c-backend.scm:1463: foreign-result-conversion */
f_12369(((C_word*)t0)[3],C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[2]);}
else{
t12=C_eqp(t4,lf[549]);
t13=(C_truep(t12)?t12:C_eqp(t4,lf[550]));
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12874,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1465: chicken.base#open-output-string */
t15=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t14=C_eqp(t4,lf[553]);
if(C_truep(t14)){
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12898,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1466: chicken.base#open-output-string */
t16=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t16;
av2[1]=t15;
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}
else{
t15=C_eqp(t4,lf[750]);
if(C_truep(t15)){
t16=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12922,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1467: chicken.base#open-output-string */
t17=C_fast_retrieve(lf[358]);{
C_word *av2=av;
av2[0]=t17;
av2[1]=t16;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
/* c-backend.scm:1468: err */
t16=((C_word*)t0)[5];
f_12371(t16,((C_word*)t0)[3]);}}}}}}}}}}
else{
/* c-backend.scm:1469: err */
t4=((C_word*)t0)[5];
f_12371(t4,((C_word*)t0)[3]);}}}

/* g2737 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_12706(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_12706,3,t0,t1,t2);}
/* c-backend.scm:1450: foreign-result-conversion */
f_12369(t1,C_i_vector_ref(t2,C_fix(0)),((C_word*)t0)[2]);}

/* k12736 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12738,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12744,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1454: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[852];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12742 in k12736 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12744,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12747,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1454: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12745 in k12742 in k12736 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12747,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12750,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1454: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[851];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12748 in k12745 in k12742 in k12736 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12750,c,av);}
/* c-backend.scm:1454: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12760 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12762,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12768,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1456: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[854];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12766 in k12760 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12768(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12768,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12771,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1456: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12769 in k12766 in k12760 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12771,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12774,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1456: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[853];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12772 in k12769 in k12766 in k12760 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12774,c,av);}
/* c-backend.scm:1456: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12784 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12786,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12792,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1458: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[856];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12790 in k12784 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12792,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12795,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1458: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12793 in k12790 in k12784 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12795,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12798,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1458: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[855];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12796 in k12793 in k12790 in k12784 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12798,c,av);}
/* c-backend.scm:1458: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12808 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12810,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12816,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1460: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[858];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12814 in k12808 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12816,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12819,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1460: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12817 in k12814 in k12808 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12819,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12822,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1460: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[857];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12820 in k12817 in k12814 in k12808 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12822,c,av);}
/* c-backend.scm:1460: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12832 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12834,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12840,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1462: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[860];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12838 in k12832 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12840,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12843,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1462: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12841 in k12838 in k12832 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12843,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12846,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1462: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[859];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12844 in k12841 in k12838 in k12832 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12846,c,av);}
/* c-backend.scm:1462: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12872 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12874,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12880,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1465: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[862];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12878 in k12872 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12880,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12883,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1465: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12881 in k12878 in k12872 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12883,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12886,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1465: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[861];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12884 in k12881 in k12878 in k12872 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12886,c,av);}
/* c-backend.scm:1465: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12896 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12898,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12904,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1466: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[864];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12902 in k12896 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12904,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12907,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1466: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12905 in k12902 in k12896 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12907,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12910,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1466: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[863];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12908 in k12905 in k12902 in k12896 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12910,c,av);}
/* c-backend.scm:1466: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12920 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12922,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12928,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1467: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[865];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12926 in k12920 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12928,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12931,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1467: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12929 in k12926 in k12920 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12931,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12934,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1467: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k12932 in k12929 in k12926 in k12920 in k12700 in k12484 in chicken.compiler.c-backend#foreign-result-conversion in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_12934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12934,c,av);}
/* c-backend.scm:1467: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_13156(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_13156,2,t1,t2);}
a=C_alloc(13);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13165,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13231,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=C_eqp(C_SCHEME_TRUE,t2);
if(C_truep(t5)){
t6=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t7=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=lf[867];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t6=C_eqp(C_SCHEME_FALSE,t2);
if(C_truep(t6)){
t7=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t8=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=t7;
av2[3]=lf[868];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
if(C_truep(C_charp(t2))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13253,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1505: encode-size */
f_13165(t7,C_fix(C_character_code(t2)));}
else{
if(C_truep(C_i_nullp(t2))){
t7=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t8=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=t7;
av2[3]=lf[870];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
if(C_truep(C_eofp(t2))){
t7=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t8=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=t7;
av2[3]=lf[871];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t7=*((C_word*)lf[371]+1);
t8=C_eqp(*((C_word*)lf[371]+1),t2);
if(C_truep(t8)){
t9=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t10=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t1;
av2[2]=t9;
av2[3]=lf[872];
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13281,a[2]=t2,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnump(t2))){
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13490,a[2]=t9,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1510: chicken.compiler.support#big-fixnum? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[367]);
C_word av2[3];
av2[0]=*((C_word*)lf[367]+1);
av2[1]=t10;
av2[2]=t2;
tp(3,av2);}}
else{
t10=t9;
f_13281(t10,C_SCHEME_FALSE);}}}}}}}}

/* encode-size in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_13165(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,3)))){
C_save_and_reclaim_args((void *)trf_13165,2,t1,t2);}
a=C_alloc(29);
t3=C_i_fixnum_length(t2);
if(C_truep(C_fixnum_greaterp(t3,C_fix(24)))){
/* c-backend.scm:1494: chicken.compiler.support#quit-compiling */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[677]);
C_word av2[4];
av2[0]=*((C_word*)lf[677]+1);
av2[1]=t1;
av2[2]=lf[866];
av2[3]=t2;
tp(4,av2);}}
else{
t4=C_s_a_i_arithmetic_shift(&a,2,t2,C_fix(-16));
t5=C_s_a_i_bitwise_and(&a,2,C_fix(255),t4);
t6=C_make_character(C_unfix(t5));
t7=C_s_a_i_arithmetic_shift(&a,2,t2,C_fix(-8));
t8=C_s_a_i_bitwise_and(&a,2,C_fix(255),t7);
t9=C_make_character(C_unfix(t8));
t10=C_s_a_i_bitwise_and(&a,2,C_fix(255),t2);
t11=C_make_character(C_unfix(t10));
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_a_i_string(&a,3,t6,t9,t11);
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}

/* k13229 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_13231,c,av);}
a=C_alloc(2);
t2=C_a_i_string(&a,1,C_make_character(254));
/* c-backend.scm:1501: scheme#string-append */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k13251 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13253,c,av);}
/* c-backend.scm:1505: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[869];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_13281(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(40,0,3)))){
C_save_and_reclaim_args((void *)trf_13281,2,t0,t1);}
a=C_alloc(40);
if(C_truep(t1)){
t2=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],C_fix(-24));
t3=C_s_a_i_bitwise_and(&a,2,C_fix(255),t2);
t4=C_make_character(C_unfix(t3));
t5=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],C_fix(-16));
t6=C_s_a_i_bitwise_and(&a,2,C_fix(255),t5);
t7=C_make_character(C_unfix(t6));
t8=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],C_fix(-8));
t9=C_s_a_i_bitwise_and(&a,2,C_fix(255),t8);
t10=C_make_character(C_unfix(t9));
t11=C_s_a_i_bitwise_and(&a,2,C_fix(255),((C_word*)t0)[2]);
t12=C_make_character(C_unfix(t11));
t13=C_a_i_string(&a,4,t4,t7,t10,t12);
/* ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[142]);
C_word av2[4];
av2[0]=*((C_word*)lf[142]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[873];
av2[3]=t13;
tp(4,av2);}}
else{
if(C_truep(C_i_exact_integerp(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13340,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word av2[4];
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=C_fix(16);
tp(4,av2);}}
else{
if(C_truep(C_i_flonump(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13367,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* ##sys#flonum->string */
t3=C_fast_retrieve(lf[877]);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=C_fix(10);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13374,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1528: chicken.keyword#keyword? */
t3=C_fast_retrieve(lf[366]);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}}

/* k13338 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13340,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13347,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1525: encode-size */
f_13165(t2,C_block_size(t1));}

/* k13345 in k13338 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13347,c,av);}
/* c-backend.scm:1525: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[874];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13365 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13367,c,av);}
/* c-backend.scm:1527: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[875];
av2[3]=t1;
av2[4]=lf[876];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_13374,c,av);}
a=C_alloc(11);
t2=(C_truep(t1)?t1:C_i_symbolp(((C_word*)t0)[2]));
if(C_truep(t2)){
t3=C_slot(((C_word*)t0)[2],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13387,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1532: encode-size */
f_13165(t4,C_i_string_length(t3));}
else{
if(C_truep(C_immp(((C_word*)t0)[2]))){
/* c-backend.scm:1536: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[881];
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}
else{
if(C_truep(C_byteblockp(((C_word*)t0)[2]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13417,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=stub2856(C_SCHEME_UNDEFINED,((C_word*)t0)[2]);
t5=C_make_character(C_unfix(t4));
t6=C_a_i_string(&a,1,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13425,a[2]=t3,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1541: encode-size */
f_13165(t7,stub2860(C_SCHEME_UNDEFINED,((C_word*)t0)[2]));}
else{
t3=stub2860(C_SCHEME_UNDEFINED,((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13447,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t5=stub2856(C_SCHEME_UNDEFINED,((C_word*)t0)[2]);
t6=C_make_character(C_unfix(t5));
t7=C_a_i_string(&a,1,t6);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13455,a[2]=t4,a[3]=t7,a[4]=((C_word*)t0)[2],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1548: encode-size */
f_13165(t8,t3);}}}}

/* k13385 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_13387,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13394,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1533: chicken.keyword#keyword? */
t3=C_fast_retrieve(lf[366]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13392 in k13385 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13394(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_13394,c,av);}
if(C_truep(t1)){
/* c-backend.scm:1530: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[878];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[879];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
/* c-backend.scm:1530: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[878];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[880];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* k13415 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13417,c,av);}
/* c-backend.scm:1538: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[142]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[142]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k13423 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13425,c,av);}
/* c-backend.scm:1539: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13445 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13447,c,av);}
/* c-backend.scm:1545: chicken.string#string-intersperse */
t2=C_fast_retrieve(lf[301]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[882];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13453 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_13455,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13459,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13461,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1549: list-tabulate */
f_4279(t2,((C_word*)t0)[5],t3);}

/* k13457 in k13453 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_13459,c,av);}
a=C_alloc(11);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3707,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_3707(t6,((C_word*)t0)[3],((C_word*)t0)[4],t2);}

/* a13460 in k13453 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13461,c,av);}
/* c-backend.scm:1549: encode-literal */
f_13156(t1,C_slot(((C_word*)t0)[2],t2));}

/* k13488 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_13490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13490,c,av);}
t2=((C_word*)t0)[2];
f_13281(t2,C_i_not(t1));}

/* k3243 */
static void C_ccall f_3245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3245,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3248,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_eval_toplevel(2,av2);}}

/* k3246 in k3243 */
static void C_ccall f_3248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3248,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3251,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_data_2dstructures_toplevel(2,av2);}}

/* k3249 in k3246 in k3243 */
static void C_ccall f_3251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3251,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3254,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_extras_toplevel(2,av2);}}

/* k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3254,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3257,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_c_2dplatform_toplevel(2,av2);}}

/* k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3257,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3260,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_compiler_toplevel(2,av2);}}

/* k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3260,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3263,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_internal_toplevel(2,av2);}}

/* k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3263,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3266,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_support_toplevel(2,av2);}}

/* k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3266(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(44,c,11)))){
C_save_and_reclaim((void *)f_3266,c,av);}
a=C_alloc(44);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=C_mutate(&lf[2] /* (set! chicken.compiler.c-backend#list-tabulate ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4279,tmp=(C_word)a,a+=2,tmp));
t5=lf[3] /* chicken.compiler.c-backend#output */ =C_SCHEME_FALSE;;
t6=C_mutate(&lf[4] /* (set! chicken.compiler.c-backend#gen ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4547,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[7] /* (set! chicken.compiler.c-backend#gen-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4590,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate((C_word*)lf[9]+1 /* (set! chicken.compiler.c-backend#generate-code ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4699,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate(&lf[222] /* (set! chicken.compiler.c-backend#toplevel ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9316,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate(&lf[489] /* (set! chicken.compiler.c-backend#cleanup ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9332,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate(&lf[339] /* (set! chicken.compiler.c-backend#make-variable-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9408,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate(&lf[342] /* (set! chicken.compiler.c-backend#make-argument-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9425,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate(&lf[286] /* (set! chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9499,tmp=(C_word)a,a+=2,tmp));
t14=C_mutate(&lf[602] /* (set! chicken.compiler.c-backend#generate-foreign-callback-header ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_10534,tmp=(C_word)a,a+=2,tmp));
t15=C_mutate((C_word*)lf[243]+1 /* (set! chicken.compiler.c-backend#foreign-type-declaration ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_10610,tmp=(C_word)a,a+=2,tmp));
t16=C_mutate(&lf[242] /* (set! chicken.compiler.c-backend#foreign-argument-conversion ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11704,tmp=(C_word)a,a+=2,tmp));
t17=C_mutate(&lf[237] /* (set! chicken.compiler.c-backend#foreign-result-conversion ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12369,tmp=(C_word)a,a+=2,tmp));
t18=C_mutate(&lf[385] /* (set! chicken.compiler.c-backend#encode-literal ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13156,tmp=(C_word)a,a+=2,tmp));
t19=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t19;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}

/* loop in k13457 in k13453 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_3707(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_3707,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3721,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:95: loop */
t6=t4;
t7=C_i_car(t3);
t8=C_u_i_cdr(t3);
t1=t6;
t2=t7;
t3=t8;
goto loop;}}

/* k3719 in loop in k13457 in k13453 in k13372 in k13279 in chicken.compiler.c-backend#encode-literal in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_3721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_3721,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.c-backend#list-tabulate in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4279(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_4279,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4285,a[2]=t2,a[3]=t5,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_4285(t7,t1,C_fix(0));}

/* loop in chicken.compiler.c-backend#list-tabulate in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4285(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_4285,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4299,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:190: proc */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}}

/* k4297 in loop in chicken.compiler.c-backend#list-tabulate in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4299(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_4299,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4303,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:190: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4285(t3,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k4301 in k4297 in loop in chicken.compiler.c-backend#list-tabulate in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_4303,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a4458 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_4459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4459,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a4470 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_4471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4471,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.c-backend#gen in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_4547,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4567,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_4567(t6,t1,t2);}

/* for-each-loop642 in chicken.compiler.c-backend#gen in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4567(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_4567,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4577,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_eqp(C_SCHEME_TRUE,t4);
if(C_truep(t5)){
/* c-backend.scm:64: scheme#newline */
t6=*((C_word*)lf[5]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t3;
av2[2]=lf[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
/* c-backend.scm:65: scheme#display */
t6=*((C_word*)lf[6]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[3];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k4575 in for-each-loop642 in chicken.compiler.c-backend#gen in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4577,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_4567(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.c-backend#gen-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4590(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_4590,2,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4599,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:71: chicken.base#intersperse */
t4=C_fast_retrieve(lf[8]);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=C_make_character(32);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k4597 in chicken.compiler.c-backend#gen-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_4599,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4604,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_4604(t5,((C_word*)t0)[2],t1);}

/* for-each-loop662 in k4597 in chicken.compiler.c-backend#gen-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4604(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_4604,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4614,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:70: scheme#display */
t4=*((C_word*)lf[6]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
av2[3]=lf[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k4612 in for-each-loop662 in k4597 in chicken.compiler.c-backend#gen-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4614,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_4604(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k4653 in chicken.compiler.c-backend#toplevel in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4655,c,av);}
/* c-backend.scm:76: chicken.internal#string->c-identifier */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[510]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[510]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k4659 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_4661,c,av);}
a=C_alloc(2);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4666,tmp=(C_word)a,a+=2,tmp);
/* c-backend.scm:87: chicken.sort#sort! */
t3=C_fast_retrieve(lf[666]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a4665 in k4659 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4666(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_4666,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4674,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:87: scheme#symbol->string */
t5=*((C_word*)lf[631]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k4672 in a4665 in k4659 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4674(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_4674,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4678,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:88: scheme#symbol->string */
t3=*((C_word*)lf[631]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k4676 in k4672 in a4665 in k4659 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4678,c,av);}
/* c-backend.scm:87: scheme#string<? */
t2=*((C_word*)lf[665]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a4687 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_4688,c,av);}
a=C_alloc(6);
t4=C_a_i_cons(&a,2,t2,t3);
t5=C_a_i_cons(&a,2,t4,((C_word*)((C_word*)t0)[2])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10=av[10];
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(c!=11) C_bad_argc_2(c,11,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,4)))){
C_save_and_reclaim((void *)f_4699,c,av);}
a=C_alloc(20);
t11=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_4703,a[2]=t4,a[3]=t7,a[4]=t6,a[5]=t2,a[6]=t3,a[7]=t9,a[8]=t5,a[9]=t1,a[10]=t10,tmp=(C_word)a,a+=11,tmp);
t12=C_SCHEME_END_OF_LIST;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4661,a[2]=t11,a[3]=t13,tmp=(C_word)a,a+=4,tmp);
t15=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4688,a[2]=t13,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:81: chicken.internal#hash-table-for-each */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[667]);
C_word *av2=av;
av2[0]=*((C_word*)lf[667]+1);
av2[1]=t14;
av2[2]=t15;
av2[3]=t4;
tp(4,av2);}}

/* k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_4703,c,av);}
a=C_alloc(20);
t2=C_SCHEME_FALSE;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_4706,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t1,a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],tmp=(C_word)a,a+=13,tmp);
t5=C_fast_retrieve(lf[663]);
t6=C_a_i_fixnum_plus(&a,2,C_fast_retrieve(lf[663]),C_fix(1));
/* c-backend.scm:98: chicken.flonum#flonum-print-precision */
t7=C_fast_retrieve(lf[664]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(82,c,5)))){
C_save_and_reclaim((void *)f_4706,c,av);}
a=C_alloc(82);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_UNDEFINED;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_SCHEME_UNDEFINED;
t25=(*a=C_VECTOR_TYPE|1,a[1]=t24,tmp=(C_word)a,a+=2,tmp);
t26=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4708,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp));
t27=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4720,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t28=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7132,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t29=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7291,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp));
t30=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7495,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp));
t31=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7673,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp));
t32=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7794,a[2]=t19,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp));
t33=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7848,a[2]=t17,tmp=(C_word)a,a+=3,tmp));
t34=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8131,a[2]=t21,tmp=(C_word)a,a+=3,tmp));
t35=C_set_block_item(t21,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8285,a[2]=t23,tmp=(C_word)a,a+=3,tmp));
t36=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8359,tmp=(C_word)a,a+=2,tmp));
t37=C_set_block_item(t25,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8371,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=((C_word*)t0)[6],a[5]=t15,a[6]=t17,a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp));
t38=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9075,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[8],a[6]=t25,a[7]=((C_word*)t0)[12],a[8]=t13,a[9]=((C_word*)t0)[9],a[10]=t11,a[11]=t9,a[12]=t7,tmp=(C_word)a,a+=13,tmp);
/* c-backend.scm:974: chicken.compiler.support#debugging */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[490]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[490]+1);
av2[1]=t38;
av2[2]=lf[661];
av2[3]=lf[662];
tp(4,av2);}}

/* find-lambda in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4708(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_4708,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4712,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:103: chicken.internal#hash-table-ref */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[12]);
C_word av2[4];
av2[0]=*((C_word*)lf[12]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
tp(4,av2);}}

/* k4710 in find-lambda in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4712,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* c-backend.scm:104: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[11];
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}}

/* expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4720(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,5)))){
C_save_and_reclaim_args((void *)trf_4720,5,t0,t1,t2,t3,t4);}
a=C_alloc(27);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_4723,a[2]=t6,a[3]=t4,a[4]=t12,a[5]=t8,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp));
t14=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6924,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t15=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6960,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t16=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7003,a[2]=t6,a[3]=t10,a[4]=t4,tmp=(C_word)a,a+=5,tmp));
/* c-backend.scm:600: expr */
t17=((C_word*)t6)[1];
f_4723(t17,t1,t2,t3);}

/* expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4723(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_4723,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_4727,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=t2,tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:110: chicken.compiler.support#node-subexpressions */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[266]);
C_word av2[3];
av2[0]=*((C_word*)lf[266]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}

/* k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4727(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_4727,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_4730,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:111: chicken.compiler.support#node-parameters */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[134]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[134]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
tp(3,av2);}}

/* k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_4730,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_4733,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* c-backend.scm:112: chicken.compiler.support#node-class */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[184]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[184]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[11];
tp(3,av2);}}

/* k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,6)))){
C_save_and_reclaim((void *)f_4733,c,av);}
a=C_alloc(24);
t2=C_eqp(t1,lf[13]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=C_eqp(t3,lf[14]);
if(C_truep(t4)){
if(C_truep(C_i_cadr(((C_word*)t0)[2]))){
/* c-backend.scm:116: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[15];
f_4547(3,av2);}}
else{
/* c-backend.scm:116: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[16];
f_4547(3,av2);}}}
else{
t5=C_eqp(t3,lf[17]);
if(C_truep(t5)){
t6=C_i_cadr(((C_word*)t0)[2]);
/* c-backend.scm:117: gen */
t7=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[18];
av2[3]=C_fix(C_character_code(t6));
av2[4]=C_make_character(41);
f_4547(5,av2);}}
else{
t6=C_eqp(t3,lf[19]);
if(C_truep(t6)){
/* c-backend.scm:118: gen */
t7=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[20];
f_4547(3,av2);}}
else{
t7=C_eqp(t3,lf[21]);
if(C_truep(t7)){
/* c-backend.scm:119: gen */
t8=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[22];
av2[3]=C_i_cadr(((C_word*)t0)[2]);
av2[4]=C_make_character(41);
f_4547(5,av2);}}
else{
t8=C_eqp(t3,lf[23]);
if(C_truep(t8)){
/* c-backend.scm:120: gen */
t9=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[24];
f_4547(3,av2);}}
else{
/* c-backend.scm:121: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[25];
tp(3,av2);}}}}}}}
else{
t3=C_eqp(t1,lf[26]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[2]);
if(C_truep(C_i_vectorp(t4))){
/* c-backend.scm:126: gen */
t5=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[27];
av2[3]=C_i_vector_ref(t4,C_fix(0));
av2[4]=lf[28];
f_4547(5,av2);}}
else{
/* c-backend.scm:127: gen */
t5=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[29];
av2[3]=C_i_car(((C_word*)t0)[2]);
av2[4]=C_make_character(93);
f_4547(5,av2);}}}
else{
t4=C_eqp(t1,lf[30]);
if(C_truep(t4)){
t5=C_i_car(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4860,a[2]=t5,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:131: gen */
t7=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[33];
f_4547(3,av2);}}
else{
t5=C_eqp(t1,lf[34]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4893,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:139: gen */
t7=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[37];
f_4547(4,av2);}}
else{
t6=C_eqp(t1,lf[38]);
if(C_truep(t6)){
/* c-backend.scm:148: gen */
t7=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[39];
av2[3]=C_i_car(((C_word*)t0)[2]);
f_4547(4,av2);}}
else{
t7=C_eqp(t1,lf[40]);
if(C_truep(t7)){
t8=C_i_car(((C_word*)t0)[2]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4951,a[2]=t10,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp));
t12=((C_word*)t10)[1];
f_4951(t12,((C_word*)t0)[3],((C_word*)t0)[5],((C_word*)t0)[6],t8);}
else{
t8=C_eqp(t1,lf[41]);
if(C_truep(t8)){
t9=C_i_car(((C_word*)t0)[2]);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4999,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:161: gen */
t11=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(102);
av2[4]=t9;
av2[5]=C_make_character(61);
f_4547(6,av2);}}
else{
t9=C_eqp(t1,lf[42]);
if(C_truep(t9)){
/* c-backend.scm:167: gen */
t10=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t10;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(102);
av2[3]=C_i_car(((C_word*)t0)[2]);
f_4547(4,av2);}}
else{
t10=C_eqp(t1,lf[43]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5038,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:170: gen */
t12=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t12;
av2[1]=t11;
av2[2]=lf[45];
f_4547(3,av2);}}
else{
t11=C_eqp(t1,lf[46]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5057,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:175: gen */
t13=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t13;
av2[1]=t12;
av2[2]=lf[48];
f_4547(3,av2);}}
else{
t12=C_eqp(t1,lf[49]);
if(C_truep(t12)){
t13=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5076,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:180: gen */
t14=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t14;
av2[1]=t13;
av2[2]=lf[51];
f_4547(3,av2);}}
else{
t13=C_eqp(t1,lf[52]);
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5103,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:185: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t14;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t14=C_eqp(t1,lf[62]);
if(C_truep(t14)){
t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5143,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:194: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t15;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t15=C_eqp(t1,lf[68]);
if(C_truep(t15)){
t16=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5183,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:203: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t16;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t16=C_eqp(t1,lf[73]);
if(C_truep(t16)){
t17=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5223,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:212: gen */
t18=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t18;
av2[1]=t17;
av2[2]=lf[75];
f_4547(3,av2);}}
else{
t17=C_eqp(t1,lf[76]);
if(C_truep(t17)){
t18=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5242,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:217: gen */
t19=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t19;
av2[1]=t18;
av2[2]=lf[77];
f_4547(3,av2);}}
else{
t18=C_eqp(t1,lf[78]);
if(C_truep(t18)){
t19=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5275,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:224: gen */
t20=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t20;
av2[1]=t19;
av2[2]=lf[81];
f_4547(3,av2);}}
else{
t19=C_eqp(t1,lf[82]);
if(C_truep(t19)){
t20=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5312,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:231: gen */
t21=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t21;
av2[1]=t20;
av2[2]=lf[84];
f_4547(3,av2);}}
else{
t20=C_eqp(t1,lf[85]);
if(C_truep(t20)){
t21=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5341,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:238: gen */
t22=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t22;
av2[1]=t21;
av2[2]=lf[87];
f_4547(3,av2);}}
else{
t21=C_eqp(t1,lf[88]);
if(C_truep(t21)){
t22=C_i_car(((C_word*)t0)[2]);
t23=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5373,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[5],a[5]=t22,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:246: gen */
t24=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t24;
av2[1]=t23;
av2[2]=lf[95];
av2[3]=t22;
av2[4]=C_make_character(44);
f_4547(5,av2);}}
else{
t22=C_eqp(t1,lf[96]);
if(C_truep(t22)){
t23=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5447,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:256: gen */
t24=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t24;
av2[1]=t23;
av2[2]=lf[98];
f_4547(3,av2);}}
else{
t23=C_eqp(t1,lf[99]);
if(C_truep(t23)){
/* c-backend.scm:260: gen */
t24=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t24;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(116);
av2[3]=C_i_car(((C_word*)t0)[2]);
f_4547(4,av2);}}
else{
t24=C_eqp(t1,lf[100]);
if(C_truep(t24)){
t25=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5479,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:263: gen */
t26=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t26;
av2[1]=t25;
av2[2]=C_make_character(116);
av2[3]=C_i_car(((C_word*)t0)[2]);
av2[4]=C_make_character(61);
f_4547(5,av2);}}
else{
t25=C_eqp(t1,lf[101]);
if(C_truep(t25)){
t26=C_i_car(((C_word*)t0)[2]);
t27=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_caddr(((C_word*)t0)[2]))){
if(C_truep(t27)){
/* c-backend.scm:272: gen */
t28=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t28;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[102];
av2[3]=t26;
av2[4]=lf[103];
f_4547(5,av2);}}
else{
t28=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5521,a[2]=((C_word*)t0)[3],a[3]=t26,tmp=(C_word)a,a+=4,tmp);
t29=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5525,a[2]=t28,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:274: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[108]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[108]+1);
av2[1]=t29;
av2[2]=C_i_cadddr(((C_word*)t0)[2]);
tp(3,av2);}}}
else{
if(C_truep(t27)){
/* c-backend.scm:276: gen */
t28=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t28;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[109];
av2[3]=t26;
av2[4]=lf[110];
f_4547(5,av2);}}
else{
/* c-backend.scm:277: gen */
t28=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t28;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[111];
av2[3]=t26;
av2[4]=lf[112];
f_4547(5,av2);}}}}
else{
t26=C_eqp(t1,lf[113]);
if(C_truep(t26)){
t27=C_i_car(((C_word*)t0)[2]);
t28=C_i_cadr(((C_word*)t0)[2]);
t29=C_i_caddr(((C_word*)t0)[2]);
t30=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5556,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t29,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t28)){
/* c-backend.scm:284: gen */
t31=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t31;
av2[1]=t30;
av2[2]=lf[119];
av2[3]=t27;
av2[4]=lf[120];
f_4547(5,av2);}}
else{
/* c-backend.scm:285: gen */
t31=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t31;
av2[1]=t30;
av2[2]=lf[121];
av2[3]=t27;
av2[4]=lf[122];
f_4547(5,av2);}}}
else{
t27=C_eqp(t1,lf[123]);
if(C_truep(t27)){
t28=C_i_car(((C_word*)t0)[2]);
t29=C_i_cadr(((C_word*)t0)[2]);
t30=C_i_caddr(((C_word*)t0)[2]);
if(C_truep(t29)){
t31=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5604,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t32=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5618,a[2]=t31,a[3]=t28,tmp=(C_word)a,a+=4,tmp);
t33=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5622,a[2]=t32,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:296: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[108]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[108]+1);
av2[1]=t33;
av2[2]=t30;
tp(3,av2);}}
else{
t31=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5625,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t32=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5639,a[2]=t31,a[3]=t28,tmp=(C_word)a,a+=4,tmp);
t33=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5643,a[2]=t32,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:301: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[108]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[108]+1);
av2[1]=t33;
av2[2]=t30;
tp(3,av2);}}}
else{
t28=C_eqp(t1,lf[130]);
if(C_truep(t28)){
/* c-backend.scm:305: gen */
t29=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t29;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[131];
f_4547(3,av2);}}
else{
t29=C_eqp(t1,lf[132]);
if(C_truep(t29)){
t30=C_i_cdr(((C_word*)t0)[5]);
t31=C_i_length(t30);
t32=C_a_i_fixnum_plus(&a,2,t31,C_fix(1));
t33=C_i_car(((C_word*)t0)[2]);
t34=C_i_cadr(((C_word*)t0)[2]);
t35=C_i_cddr(((C_word*)t0)[2]);
t36=C_i_pairp(t35);
t37=(C_truep(t36)?C_i_caddr(((C_word*)t0)[2]):C_SCHEME_FALSE);
t38=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_5680,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=t32,a[6]=((C_word*)t0)[8],a[7]=t30,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[4],a[10]=t31,a[11]=((C_word*)t0)[7],a[12]=((C_word*)t0)[9],a[13]=t34,a[14]=t37,a[15]=t33,a[16]=((C_word*)t0)[10],a[17]=((C_word*)t0)[11],a[18]=t36,tmp=(C_word)a,a+=19,tmp);
/* c-backend.scm:316: chicken.compiler.support#source-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[198]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[198]+1);
av2[1]=t38;
av2[2]=t37;
tp(3,av2);}}
else{
t30=C_eqp(t1,lf[199]);
if(C_truep(t30)){
t31=C_i_length(((C_word*)t0)[5]);
t32=C_a_i_fixnum_plus(&a,2,t31,C_fix(1));
t33=C_i_car(((C_word*)t0)[2]);
t34=C_i_cadr(((C_word*)t0)[2]);
t35=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6190,a[2]=t33,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[3],a[6]=t31,a[7]=((C_word*)t0)[5],a[8]=t32,a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[9],a[11]=t34,tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:414: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=t35;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t31=C_eqp(t1,lf[203]);
if(C_truep(t31)){
t32=C_i_cdr(((C_word*)t0)[5]);
t33=C_i_length(t32);
t34=C_a_i_fixnum_plus(&a,2,t33,C_fix(1));
t35=C_i_car(((C_word*)t0)[2]);
t36=C_i_caddr(((C_word*)t0)[2]);
t37=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_6350,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=t32,a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[4],a[9]=t36,a[10]=t35,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* c-backend.scm:441: chicken.compiler.support#source-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[198]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[198]+1);
av2[1]=t37;
av2[2]=t36;
tp(3,av2);}}
else{
t32=C_eqp(t1,lf[215]);
if(C_truep(t32)){
/* c-backend.scm:467: gen */
t33=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t33;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[216];
av2[3]=C_i_car(((C_word*)t0)[2]);
av2[4]=lf[217];
f_4547(5,av2);}}
else{
t33=C_eqp(t1,lf[218]);
if(C_truep(t33)){
t34=C_i_length(((C_word*)t0)[5]);
t35=C_a_i_fixnum_plus(&a,2,t34,C_fix(1));
t36=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6487,a[2]=((C_word*)t0)[3],a[3]=t35,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:474: gen */
t37=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t37;
av2[1]=t36;
av2[2]=C_make_character(123);
f_4547(3,av2);}}
else{
t34=C_eqp(t1,lf[224]);
if(C_truep(t34)){
t35=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6510,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:479: gen */
t36=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t36;
av2[1]=t35;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[226];
f_4547(4,av2);}}
else{
t35=C_eqp(t1,lf[227]);
if(C_truep(t35)){
t36=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6529,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:484: gen */
t37=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t37;
av2[1]=t36;
av2[2]=C_i_car(((C_word*)t0)[2]);
av2[3]=C_make_character(40);
f_4547(4,av2);}}
else{
t36=C_eqp(t1,lf[228]);
if(C_truep(t36)){
t37=C_i_car(((C_word*)t0)[2]);
if(C_truep(((C_word*)((C_word*)t0)[10])[1])){
/* c-backend.scm:489: gen */
t38=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t38;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[229];
av2[3]=t37;
av2[4]=lf[230];
av2[5]=lf[231];
av2[6]=lf[232];
f_4547(7,av2);}}
else{
/* c-backend.scm:489: gen */
t38=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t38;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[229];
av2[3]=t37;
av2[4]=lf[230];
av2[5]=lf[233];
av2[6]=lf[232];
f_4547(7,av2);}}}
else{
t37=C_eqp(t1,lf[234]);
if(C_truep(t37)){
t38=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6565,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:493: gen */
t39=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t39;
av2[1]=t38;
av2[2]=C_i_car(((C_word*)t0)[2]);
av2[3]=lf[235];
av2[4]=C_i_length(((C_word*)t0)[5]);
f_4547(5,av2);}}
else{
t38=C_eqp(t1,lf[236]);
if(C_truep(t38)){
t39=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6601,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:501: foreign-result-conversion */
f_12369(t39,C_i_cadr(((C_word*)t0)[2]),lf[238]);}
else{
t39=C_eqp(t1,lf[239]);
if(C_truep(t39)){
t40=C_i_cadr(((C_word*)t0)[2]);
t41=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6621,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t42=C_i_car(((C_word*)t0)[2]);
t43=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6639,a[2]=t41,a[3]=t42,a[4]=t40,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:505: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t43;
av2[2]=t40;
av2[3]=lf[244];
tp(4,av2);}}
else{
t40=C_eqp(t1,lf[245]);
if(C_truep(t40)){
t41=C_i_car(((C_word*)t0)[2]);
t42=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6655,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t43=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6669,a[2]=t42,a[3]=t41,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:511: foreign-result-conversion */
f_12369(t43,t41,lf[250]);}
else{
t41=C_eqp(t1,lf[251]);
if(C_truep(t41)){
t42=C_i_car(((C_word*)t0)[2]);
t43=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6685,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t42,tmp=(C_word)a,a+=7,tmp);
t44=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6713,a[2]=t43,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:517: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t44;
av2[2]=t42;
av2[3]=lf[256];
tp(4,av2);}}
else{
t42=C_eqp(t1,lf[257]);
if(C_truep(t42)){
t43=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6722,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:524: gen */
t44=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t44;
av2[1]=t43;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[261];
f_4547(4,av2);}}
else{
t43=C_eqp(t1,lf[262]);
if(C_truep(t43)){
t44=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6801,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:539: gen */
t45=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t45;
av2[1]=t44;
av2[2]=lf[264];
f_4547(3,av2);}}
else{
t44=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6838,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:547: chicken.compiler.support#node-class */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[184]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[184]+1);
av2[1]=t44;
av2[2]=((C_word*)t0)[12];
tp(3,av2);}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k4858 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_4860,c,av);}
a=C_alloc(3);
if(C_truep(C_i_nanp(((C_word*)t0)[2]))){
/* c-backend.scm:132: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[31];
f_4547(3,av2);}}
else{
if(C_truep(C_i_infinitep(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4874,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_negativep(((C_word*)t0)[2]))){
/* c-backend.scm:134: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(45);
f_4547(3,av2);}}
else{
/* c-backend.scm:135: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[32];
f_4547(3,av2);}}}
else{
/* c-backend.scm:136: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[2];
f_4547(3,av2);}}}}

/* k4872 in k4858 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4874,c,av);}
/* c-backend.scm:135: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[32];
f_4547(3,av2);}}

/* k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4893(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_4893,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4896,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:140: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k4894 in k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_4896,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4899,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:141: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
f_4547(3,av2);}}

/* k4897 in k4894 in k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_4899,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4902,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:142: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k4900 in k4897 in k4894 in k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_4902,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4905,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:143: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(125);
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[35];
f_4547(5,av2);}}

/* k4903 in k4900 in k4897 in k4894 in k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_4905,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4908,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:144: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_caddr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k4906 in k4903 in k4900 in k4897 in k4894 in k4891 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_4908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4908,c,av);}
/* c-backend.scm:145: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(125);
f_4547(3,av2);}}

/* loop in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_4951(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_4951,5,t0,t1,t2,t3,t4);}
a=C_alloc(8);
if(C_truep(C_i_greaterp(t4,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_4961,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[2],a[6]=t1,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:153: gen */
t6=lf[4];{
C_word av2[6];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=t3;
av2[5]=C_make_character(61);
f_4547(6,av2);}}
else{
/* c-backend.scm:157: expr */
t5=((C_word*)((C_word*)t0)[3])[1];
f_4723(t5,t1,C_i_car(t2),t3);}}

/* k4959 in loop in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_4961,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_4964,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:154: expr */
t3=((C_word*)((C_word*)t0)[7])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[2]),((C_word*)t0)[3]);}

/* k4962 in k4959 in loop in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_4964,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_4967,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:155: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k4965 in k4962 in k4959 in loop in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4967(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,4)))){
C_save_and_reclaim((void *)f_4967,c,av);}
a=C_alloc(58);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=C_s_a_i_minus(&a,2,((C_word*)t0)[4],C_fix(1));
/* c-backend.scm:156: loop */
t5=((C_word*)((C_word*)t0)[5])[1];
f_4951(t5,((C_word*)t0)[6],t2,t3,t4);}

/* k4997 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_4999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_4999,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5002,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:162: expr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5000 in k4997 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_5002,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5005,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:163: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k5003 in k5000 in k4997 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5005(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5005,c,av);}
/* c-backend.scm:164: expr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_4723(t2,((C_word*)t0)[3],C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5036 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5038,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5041,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:171: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5039 in k5036 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5041,c,av);}
/* c-backend.scm:172: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[44];
f_4547(3,av2);}}

/* k5055 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5057(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5057,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5060,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:176: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5058 in k5055 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5060(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5060,c,av);}
/* c-backend.scm:177: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[47];
f_4547(3,av2);}}

/* k5074 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_5076,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5079,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:181: expr */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[5]),((C_word*)t0)[6]);}

/* k5077 in k5074 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_5079,c,av);}
a=C_alloc(29);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_s_a_i_plus(&a,2,t2,C_fix(1));
/* c-backend.scm:182: gen */
t4=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[50];
av2[3]=t3;
av2[4]=C_make_character(93);
f_4547(5,av2);}}

/* k5101 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_5103,c,av);}
a=C_alloc(9);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5109,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5128,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:187: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k5107 in k5101 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5109(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,6)))){
C_save_and_reclaim_args((void *)trf_5109,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* c-backend.scm:190: gen */
t3=lf[4];{
C_word av2[7];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[53];
av2[3]=t2;
av2[4]=lf[54];
av2[5]=((C_word*)t0)[3];
av2[6]=lf[55];
f_4547(7,av2);}}
else{
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:191: gen */
t3=lf[4];{
C_word av2[7];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[56];
av2[3]=t2;
av2[4]=lf[57];
av2[5]=((C_word*)t0)[2];
av2[6]=lf[58];
f_4547(7,av2);}}}

/* k5126 in k5101 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5128,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5131,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];
f_5109(t3,C_i_not(t1));}
else{
/* c-backend.scm:188: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k5129 in k5126 in k5101 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5131(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5131,c,av);}
t2=((C_word*)t0)[2];
f_5109(t2,C_i_not(t1));}

/* k5141 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_5143,c,av);}
a=C_alloc(9);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5149,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5168,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:196: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k5147 in k5141 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5149(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,6)))){
C_save_and_reclaim_args((void *)trf_5149,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* c-backend.scm:199: gen */
t3=lf[4];{
C_word av2[5];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[63];
av2[3]=t2;
av2[4]=lf[64];
f_4547(5,av2);}}
else{
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:200: gen */
t3=lf[4];{
C_word av2[7];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[65];
av2[3]=t2;
av2[4]=lf[66];
av2[5]=((C_word*)t0)[2];
av2[6]=lf[67];
f_4547(7,av2);}}}

/* k5166 in k5141 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5168,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5171,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];
f_5149(t3,C_i_not(t1));}
else{
/* c-backend.scm:197: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k5169 in k5166 in k5141 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5171,c,av);}
t2=((C_word*)t0)[2];
f_5149(t2,C_i_not(t1));}

/* k5181 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_5183,c,av);}
a=C_alloc(9);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5189,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5208,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:205: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k5187 in k5181 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5189(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,4)))){
C_save_and_reclaim_args((void *)trf_5189,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* c-backend.scm:208: gen */
t3=lf[4];{
C_word av2[5];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[69];
av2[3]=t2;
av2[4]=lf[70];
f_4547(5,av2);}}
else{
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:209: gen */
t3=lf[4];{
C_word av2[5];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[71];
av2[3]=t2;
av2[4]=lf[72];
f_4547(5,av2);}}}

/* k5206 in k5181 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5208(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5208,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5211,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];
f_5189(t3,C_i_not(t1));}
else{
/* c-backend.scm:206: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k5209 in k5206 in k5181 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5211,c,av);}
t2=((C_word*)t0)[2];
f_5189(t2,C_i_not(t1));}

/* k5221 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5223(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5223,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5226,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:213: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5224 in k5221 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5226,c,av);}
/* c-backend.scm:214: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[74];
f_4547(3,av2);}}

/* k5240 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_5242,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5245,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:218: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5243 in k5240 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_5245,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5248,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:219: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(44);
av2[3]=C_i_car(((C_word*)t0)[6]);
av2[4]=C_make_character(44);
f_4547(5,av2);}}

/* k5246 in k5243 in k5240 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5248,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5251,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:220: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5249 in k5246 in k5243 in k5240 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5251,c,av);}
/* c-backend.scm:221: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5273 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_5275,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5278,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:225: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5276 in k5273 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,4)))){
C_save_and_reclaim((void *)f_5278,c,av);}
a=C_alloc(35);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5281,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_i_car(((C_word*)t0)[6]);
t4=C_s_a_i_plus(&a,2,t3,C_fix(1));
/* c-backend.scm:226: gen */
t5=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[79];
av2[3]=t4;
av2[4]=lf[80];
f_4547(5,av2);}}

/* k5279 in k5276 in k5273 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5281,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5284,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:227: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5282 in k5279 in k5276 in k5273 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5284,c,av);}
/* c-backend.scm:228: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5310 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_5312,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:232: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5313 in k5310 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_5315,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:233: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[83];
f_4547(3,av2);}}

/* k5316 in k5313 in k5310 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5318,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5321,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:234: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5319 in k5316 in k5313 in k5310 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5321,c,av);}
/* c-backend.scm:235: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5339 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5341(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_5341,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5344,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:239: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5342 in k5339 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5344(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_5344,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5347,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:240: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[86];
f_4547(3,av2);}}

/* k5345 in k5342 in k5339 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5347,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5350,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:241: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5348 in k5345 in k5342 in k5339 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5350,c,av);}
/* c-backend.scm:242: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5373,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5374,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5387,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:252: list-tabulate */
f_4279(t3,((C_word*)t0)[5],*((C_word*)lf[94]+1));}

/* g821 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5374(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_5374,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5378,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:249: gen */
t5=lf[4];{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[89];
av2[3]=t3;
av2[4]=lf[90];
f_4547(5,av2);}}

/* k5376 in g821 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5378,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5381,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:250: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k5379 in k5376 in g821 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5381,c,av);}
/* c-backend.scm:251: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(44);
f_4547(3,av2);}}

/* k5385 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5387,c,av);}
a=C_alloc(10);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t3=C_i_check_list_2(t1,lf[91]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5396,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5403,a[2]=t6,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_5403(t8,t4,((C_word*)t0)[2],t1);}

/* k5394 in k5385 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_5396,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* c-backend.scm:253: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[92];
av2[3]=t2;
av2[4]=lf[93];
f_4547(5,av2);}}

/* for-each-loop820 in k5385 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5403(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_5403,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5413,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:247: g821 */
t7=((C_word*)t0)[3];
f_5374(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k5411 in for-each-loop820 in k5385 in k5371 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5413,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_5403(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k5445 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5447,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5450,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:257: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5448 in k5445 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5450,c,av);}
/* c-backend.scm:258: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[97];
f_4547(3,av2);}}

/* k5477 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5479,c,av);}
/* c-backend.scm:264: expr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_4723(t2,((C_word*)t0)[3],C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5519 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_5521,c,av);}
/* c-backend.scm:273: gen */
t2=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[104];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[105];
av2[5]=t1;
av2[6]=lf[106];
f_4547(7,av2);}}

/* k5523 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5525,c,av);}
/* c-backend.scm:274: chicken.compiler.support#c-ify-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[107]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[107]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5556(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_5556,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5559,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5573,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5577,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:286: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[108]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[108]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k5557 in k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5559,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5562,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:287: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5560 in k5557 in k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5562(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5562,c,av);}
/* c-backend.scm:288: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5571 in k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5573(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5573,c,av);}
/* c-backend.scm:286: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[114];
av2[3]=t1;
av2[4]=lf[115];
f_4547(5,av2);}}

/* k5575 in k5554 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5577,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14744,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t3=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5602 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5604,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5607,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:297: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5605 in k5602 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5607(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5607,c,av);}
/* c-backend.scm:298: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k5616 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_5618,c,av);}
/* c-backend.scm:295: gen */
t2=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[124];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[125];
av2[5]=t1;
av2[6]=lf[126];
f_4547(7,av2);}}

/* k5620 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5622,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14749,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t3=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5623 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5625,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5628,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:302: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k5626 in k5623 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5628(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5628,c,av);}
/* c-backend.scm:303: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k5637 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5639(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_5639,c,av);}
/* c-backend.scm:300: gen */
t2=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[127];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[128];
av2[5]=t1;
av2[6]=lf[129];
f_4547(7,av2);}}

/* k5641 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5643,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14754,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t3=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_5680,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_5683,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=t1,a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
if(C_truep(((C_word*)t0)[18])){
t3=C_i_cdddr(((C_word*)t0)[2]);
t4=C_i_pairp(t3);
t5=t2;
f_5683(t5,(C_truep(t4)?C_i_cadddr(((C_word*)t0)[2]):C_SCHEME_FALSE));}
else{
t3=t2;
f_5683(t3,C_SCHEME_FALSE);}}

/* k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5683(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,2)))){
C_save_and_reclaim_args((void *)trf_5683,2,t0,t1);}
a=C_alloc(25);
t2=(C_truep(t1)?C_i_car(C_i_cddddr(((C_word*)t0)[2])):C_SCHEME_FALSE);
t3=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_5689,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t1,a[9]=((C_word*)t0)[9],a[10]=t2,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
if(C_truep(t2)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6143,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6147,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:319: find-lambda */
t6=((C_word*)((C_word*)t0)[18])[1];
f_4708(t6,t5,t1);}
else{
t4=t3;
f_5689(t4,C_SCHEME_FALSE);}}

/* k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_5689(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,2)))){
C_save_and_reclaim_args((void *)trf_5689,2,t0,t1);}
a=C_alloc(27);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_5693,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=t1,a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
if(C_truep(((C_word*)t0)[15])){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6112,a[2]=((C_word*)t0)[16],a[3]=((C_word*)t0)[17],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fast_retrieve(lf[191]))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6132,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14767,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:74: chicken.string#->string */
t7=C_fast_retrieve(lf[118]);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[18];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6139,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14772,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t7=C_fast_retrieve(lf[118]);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[18];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_5693(2,av2);}}}

/* k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_5693(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_5693,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_6109,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],tmp=(C_word)a,a+=16,tmp);
/* c-backend.scm:328: chicken.compiler.support#node-class */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[184]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[184]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k5700 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_5702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_5702,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5705,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:330: push-args */
t3=((C_word*)((C_word*)t0)[5])[1];
f_7003(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],lf[135]);}

/* k5703 in k5700 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_5705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_5705,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5708,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:331: chicken.compiler.support#node-parameters */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[134]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[134]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k5706 in k5703 in k5700 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_5708,c,av);}
/* c-backend.scm:332: gen */
t2=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_i_car(t1);
av2[4]=C_make_character(40);
av2[5]=((C_word*)t0)[3];
av2[6]=lf[133];
f_4547(7,av2);}}

/* k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_5724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,5)))){
C_save_and_reclaim((void *)f_5724,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5727,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:336: chicken.compiler.core#lambda-literal-temporaries */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[139]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[139]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_5853,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[12],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[13],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[11])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_5853(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5908,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[14],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:351: gen */
t4=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=((C_word*)t0)[3];
av2[5]=C_make_character(61);
f_4547(6,av2);}}}}

/* k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_5727(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_5727,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5730,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5846,a[2]=t1,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:337: list-tabulate */
f_4279(t2,((C_word*)t0)[7],t3);}

/* k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_5730,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5731,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_check_list_2(t1,lf[91]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5747,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=t1,a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5809,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_5809(t8,t4,((C_word*)t0)[8],t1);}

/* g909 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_fcall f_5731(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_5731,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5735,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:340: gen */
t5=lf[4];{
C_word av2[6];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=t3;
av2[5]=C_make_character(61);
f_4547(6,av2);}}

/* k5733 in g909 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5735(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5735,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5738,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:341: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k5736 in k5733 in g909 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_5738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5738,c,av);}
/* c-backend.scm:342: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_5747,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5755,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:346: list-tabulate */
f_4279(t2,((C_word*)t0)[6],*((C_word*)lf[94]+1));}

/* k5753 in k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5755,c,av);}
a=C_alloc(10);
t2=C_i_check_list_2(t1,lf[91]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5761,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5772,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_5772(t7,t3,((C_word*)t0)[5],t1);}

/* k5759 in k5753 in k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_5761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_5761,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5764,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
/* c-backend.scm:348: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[136];
f_4547(4,av2);}}
else{
/* c-backend.scm:347: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[137];
av2[4]=((C_word*)t0)[4];
av2[5]=C_make_character(59);
f_4547(6,av2);}}}

/* k5762 in k5759 in k5753 in k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_5764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5764,c,av);}
/* c-backend.scm:348: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[136];
f_4547(4,av2);}}

/* for-each-loop919 in k5753 in k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_fcall f_5772(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,7)))){
C_save_and_reclaim_args((void *)trf_5772,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5782,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:345: gen */
t7=lf[4];{
C_word av2[8];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=C_slot(t3,C_fix(0));
av2[5]=lf[138];
av2[6]=C_slot(t2,C_fix(0));
av2[7]=C_make_character(59);
f_4547(8,av2);}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k5780 in for-each-loop919 in k5753 in k5745 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_5782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5782,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_5772(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* for-each-loop908 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_fcall f_5809(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_5809,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5819,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:338: g909 */
t7=((C_word*)t0)[3];
f_5731(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k5817 in for-each-loop908 in k5728 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5819,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_5809(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a5845 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_5846,c,av);}
a=C_alloc(33);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15385,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=t3;
va15385(t4,C_s_a_i_plus(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]));}

/* k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_5853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5853,c,av);}
a=C_alloc(10);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5859,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:355: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[8];
av2[4]=C_make_character(40);
f_4547(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5874,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* c-backend.scm:360: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(123);
f_4547(3,av2);}}}

/* k5857 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_5859,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5862,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14762,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:357: expr-args */
t4=((C_word*)((C_word*)t0)[3])[1];
f_6924(t4,t3,((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
/* c-backend.scm:356: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(116);
av2[3]=((C_word*)t0)[5];
av2[4]=C_make_character(44);
f_4547(5,av2);}}}

/* k5860 in k5857 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5862(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5862,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5865,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:357: expr-args */
t3=((C_word*)((C_word*)t0)[3])[1];
f_6924(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k5863 in k5860 in k5857 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5865,c,av);}
/* c-backend.scm:358: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[140];
f_4547(3,av2);}}

/* k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_5874,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5877,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5893,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_not(((C_word*)t0)[9]))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5904,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
av2[3]=C_fix(10);
tp(4,av2);}}
else{
/* c-backend.scm:361: push-args */
t4=((C_word*)((C_word*)t0)[6])[1];
f_7003(t4,t2,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}}

/* k5875 in k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5877(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_5877,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5880,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:362: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[5];
av2[4]=C_make_character(40);
f_4547(5,av2);}}

/* k5878 in k5875 in k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5880,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5883,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
/* c-backend.scm:364: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[141];
f_4547(3,av2);}}
else{
/* c-backend.scm:363: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_make_character(44);
f_4547(4,av2);}}}

/* k5881 in k5878 in k5875 in k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_5883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5883,c,av);}
/* c-backend.scm:364: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[141];
f_4547(3,av2);}}

/* k5891 in k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5893(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5893,c,av);}
/* c-backend.scm:361: push-args */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7003(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k5902 in k5872 in k5851 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5904,c,av);}
/* ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[142]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[142]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[143];
av2[3]=t1;
tp(4,av2);}}

/* k5906 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_5908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5908,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5911,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:352: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k5909 in k5906 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5911,c,av);}
/* c-backend.scm:353: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k5922 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_5924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5924,c,av);}
t2=C_eqp(((C_word*)t0)[2],t1);
if(C_truep(t2)){
/* c-backend.scm:335: chicken.compiler.core#lambda-literal-looping */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[145]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[145]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_5724(2,av2);}}}

/* k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_fcall f_5930(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_5930,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5933,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:369: chicken.compiler.support#node-parameters */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[134]);
C_word av2[3];
av2[0]=*((C_word*)lf[134]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6043,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* c-backend.scm:399: gen */
t3=lf[4];{
C_word av2[6];
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=((C_word*)t0)[6];
av2[5]=C_make_character(61);
f_4547(6,av2);}}}

/* k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_5933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_5933,c,av);}
a=C_alloc(14);
t2=C_i_car(t1);
t3=C_i_cadr(t1);
t4=C_i_caddr(t1);
t5=C_SCHEME_FALSE;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5945,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t6,a[8]=t4,a[9]=t2,a[10]=t3,a[11]=t1,tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:374: gen */
t8=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[172];
f_4547(4,av2);}}

/* k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_5945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_5945,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5948,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_fast_retrieve(lf[149]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5961,a[2]=((C_word*)t0)[7],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[8])){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5971,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=C_fix(10);
tp(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5979,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=C_fix(10);
tp(4,av2);}}}
else{
if(C_truep(((C_word*)t0)[8])){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5987,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[10],a[4]=t2,a[5]=((C_word*)t0)[11],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6012,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=C_fix(10);
tp(4,av2);}}
else{
if(C_truep(((C_word*)t0)[10])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6020,a[2]=((C_word*)t0)[7],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6027,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=C_fix(10);
tp(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6032,a[2]=((C_word*)t0)[7],a[3]=t2,a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6039,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=C_fix(10);
tp(4,av2);}}}}}

/* k5946 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5948(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_5948,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5951,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:395: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k5949 in k5946 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_5951(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_5951,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5954,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:396: push-args */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7003(t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)((C_word*)t0)[7])[1]);}

/* k5952 in k5949 in k5946 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_5954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_5954,c,av);}
/* c-backend.scm:397: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[147];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[148];
f_4547(6,av2);}}

/* k5959 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5961,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* c-backend.scm:380: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[150];
av2[3]=((C_word*)((C_word*)t0)[2])[1];
av2[4]=lf[151];
f_4547(5,av2);}}

/* k5969 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5971,c,av);}
/* c-backend.scm:378: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[153];
av2[3]=t1;
av2[4]=lf[154];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k5977 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5979,c,av);}
/* c-backend.scm:379: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[155];
av2[3]=t1;
av2[4]=lf[156];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k5985 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_5987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_5987,c,av);}
a=C_alloc(7);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
if(C_truep(((C_word*)t0)[3])){
/* c-backend.scm:384: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[157];
av2[3]=((C_word*)((C_word*)t0)[2])[1];
av2[4]=lf[158];
f_4547(5,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6000,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6004,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:386: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[108]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[108]+1);
av2[1]=t4;
av2[2]=C_i_cadddr(((C_word*)t0)[5]);
tp(3,av2);}}}

/* k5998 in k5985 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_6000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_6000,c,av);}
/* c-backend.scm:385: gen */
t2=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[159];
av2[3]=((C_word*)((C_word*)t0)[3])[1];
av2[4]=lf[160];
av2[5]=t1;
av2[6]=lf[161];
f_4547(7,av2);}}

/* k6002 in k5985 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_6004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6004,c,av);}
/* c-backend.scm:386: chicken.compiler.support#c-ify-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[107]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[107]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k6010 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6012(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6012,c,av);}
/* c-backend.scm:382: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[162];
av2[3]=t1;
av2[4]=lf[163];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k6018 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6020(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6020,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* c-backend.scm:390: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[164];
av2[3]=((C_word*)((C_word*)t0)[2])[1];
av2[4]=lf[165];
f_4547(5,av2);}}

/* k6025 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6027,c,av);}
/* c-backend.scm:389: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[166];
av2[3]=t1;
av2[4]=lf[167];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k6030 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6032,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* c-backend.scm:394: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[168];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[169];
f_4547(5,av2);}}

/* k6037 in k5943 in k5931 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6039(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6039,c,av);}
/* c-backend.scm:393: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[170];
av2[3]=t1;
av2[4]=lf[171];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_6043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_6043,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6046,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:400: expr */
t3=((C_word*)((C_word*)t0)[8])[1];
f_4723(t3,t2,((C_word*)t0)[9],((C_word*)t0)[4]);}

/* k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_6046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_6046,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6049,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:401: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[183];
f_4547(3,av2);}}

/* k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_6049,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6052,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6077,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6082,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fix(10);
tp(4,av2);}}

/* k6050 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_6052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6052,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6055,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:403: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[181];
f_4547(4,av2);}}

/* k6053 in k6050 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_6055(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_6055,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6058,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_fast_retrieve(lf[175]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6064,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fast_retrieve(lf[175]))){
t5=t4;
f_6064(t5,C_fast_retrieve(lf[175]));}
else{
t5=C_fast_retrieve(lf[180]);
if(C_truep(C_fast_retrieve(lf[180]))){
t6=C_fast_retrieve(lf[180]);
t7=t4;
f_6064(t7,C_fast_retrieve(lf[180]));}
else{
t6=t4;
f_6064(t6,((C_word*)t0)[5]);}}}

/* k6056 in k6053 in k6050 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_6058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6058,c,av);}
/* c-backend.scm:407: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[173];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[174];
f_4547(5,av2);}}

/* k6062 in k6053 in k6050 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_fcall f_6064(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_6064,2,t0,t1);}
if(C_truep(t1)){
/* c-backend.scm:405: gen */
t2=lf[4];{
C_word av2[5];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[176];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[177];
f_4547(5,av2);}}
else{
/* c-backend.scm:406: gen */
t2=lf[4];{
C_word av2[5];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[178];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[179];
f_4547(5,av2);}}}

/* k6075 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_6077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6077,c,av);}
/* c-backend.scm:402: push-args */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7003(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k6080 in k6047 in k6044 in k6041 in k5928 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_6082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6082,c,av);}
/* ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[142]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[142]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[182];
av2[3]=t1;
tp(4,av2);}}

/* k6103 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6105,c,av);}
t2=C_eqp(lf[101],t1);
if(C_truep(t2)){
if(C_truep(C_i_not(C_fast_retrieve(lf[175])))){
t3=C_i_not(C_fast_retrieve(lf[180]));
t4=((C_word*)t0)[2];
f_5930(t4,(C_truep(t3)?C_i_not(((C_word*)t0)[3]):C_SCHEME_FALSE));}
else{
t3=((C_word*)t0)[2];
f_5930(t3,C_SCHEME_FALSE);}}
else{
t3=((C_word*)t0)[2];
f_5930(t3,C_SCHEME_FALSE);}}

/* k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_6109,c,av);}
a=C_alloc(20);
t2=C_eqp(lf[38],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5702,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:329: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(123);
f_4547(3,av2);}}
else{
if(C_truep(((C_word*)t0)[8])){
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_5724,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[13],a[11]=((C_word*)t0)[14],a[12]=((C_word*)t0)[8],a[13]=((C_word*)t0)[5],a[14]=((C_word*)t0)[4],tmp=(C_word)a,a+=15,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5924,a[2]=((C_word*)t0)[8],a[3]=t3,a[4]=((C_word*)t0)[12],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:334: chicken.compiler.core#lambda-literal-id */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[146]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[146]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[12];
tp(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5930,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[15],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6105,a[2]=t3,a[3]=((C_word*)t0)[15],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:365: chicken.compiler.support#node-class */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[184]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[184]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}}}

/* k6110 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6112,c,av);}
t2=(C_truep(C_fast_retrieve(lf[185]))?((C_word*)t0)[2]:C_SCHEME_FALSE);
if(C_truep(t2)){
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* c-backend.scm:326: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[186];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[187];
av2[6]=lf[188];
av2[7]=lf[189];
f_4547(8,av2);}}
else{
/* c-backend.scm:326: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[186];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[187];
av2[6]=lf[190];
av2[7]=lf[189];
f_4547(8,av2);}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_5693(2,av2);}}}

/* k6130 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6132,c,av);}
/* c-backend.scm:323: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[192];
av2[4]=t1;
av2[5]=lf[193];
f_4547(6,av2);}}

/* k6137 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6139,c,av);}
/* c-backend.scm:324: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[195];
av2[4]=t1;
av2[5]=lf[196];
f_4547(6,av2);}}

/* k6141 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6143,c,av);}
t2=((C_word*)t0)[2];
f_5689(t2,C_i_zerop(t1));}

/* k6145 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6147,c,av);}
/* c-backend.scm:319: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_6190,c,av);}
a=C_alloc(8);
t2=C_i_zerop(t1);
if(C_truep(((C_word*)t0)[2])){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6196,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:416: chicken.compiler.core#lambda-literal-temporaries */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[139]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[139]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6316,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[4],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:429: gen */
t4=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[11];
av2[3]=C_make_character(40);
f_4547(4,av2);}}}

/* k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_6196,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6199,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6309,a[2]=t1,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:417: list-tabulate */
f_4279(t2,((C_word*)t0)[5],t3);}

/* k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_6199,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6200,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_check_list_2(t1,lf[91]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6216,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6272,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_6272(t8,t4,((C_word*)t0)[6],t1);}

/* g1026 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6200(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_6200,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6204,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:420: gen */
t5=lf[4];{
C_word av2[6];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=t3;
av2[5]=C_make_character(61);
f_4547(6,av2);}}

/* k6202 in g1026 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6204,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6207,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:421: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6205 in k6202 in g1026 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6207,c,av);}
/* c-backend.scm:422: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k6214 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_6216,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6224,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:426: list-tabulate */
f_4279(t2,((C_word*)t0)[4],*((C_word*)lf[94]+1));}

/* k6222 in k6214 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6224(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_6224,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[91]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6230,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6235,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_6235(t7,t3,((C_word*)t0)[3],t1);}

/* k6228 in k6222 in k6214 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6230(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6230,c,av);}
/* c-backend.scm:427: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[200];
f_4547(4,av2);}}

/* for-each-loop1036 in k6222 in k6214 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_fcall f_6235(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,7)))){
C_save_and_reclaim_args((void *)trf_6235,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6245,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:425: gen */
t7=lf[4];{
C_word av2[8];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(116);
av2[4]=C_slot(t3,C_fix(0));
av2[5]=lf[201];
av2[6]=C_slot(t2,C_fix(0));
av2[7]=C_make_character(59);
f_4547(8,av2);}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k6243 in for-each-loop1036 in k6222 in k6214 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_6245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6245,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6235(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* for-each-loop1025 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6272(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_6272,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6282,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:418: g1026 */
t7=((C_word*)t0)[3];
f_6200(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k6280 in for-each-loop1025 in k6197 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6282(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6282,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6272(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a6308 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_6309,c,av);}
a=C_alloc(33);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15391,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=t3;
va15391(t4,C_s_a_i_plus(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]));}

/* k6314 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_6316,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6319,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14785,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:431: expr-args */
t4=((C_word*)((C_word*)t0)[3])[1];
f_6924(t4,t3,((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
/* c-backend.scm:430: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[202];
f_4547(3,av2);}}}

/* k6317 in k6314 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6319(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6319,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6322,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:431: expr-args */
t3=((C_word*)((C_word*)t0)[3])[1];
f_6924(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6320 in k6317 in k6314 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6322,c,av);}
/* c-backend.scm:432: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_6350,c,av);}
a=C_alloc(18);
t2=C_i_cadddr(((C_word*)t0)[2]);
t3=C_i_cddddr(((C_word*)t0)[2]);
t4=C_i_car(t3);
t5=C_i_not(C_i_zerop(t4));
t6=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_6363,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t5,a[9]=t4,a[10]=t2,a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=t1,tmp=(C_word)a,a+=15,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6458,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:445: find-lambda */
t8=((C_word*)((C_word*)t0)[12])[1];
f_4708(t8,t7,t2);}

/* k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_6363,c,av);}
a=C_alloc(16);
t2=C_i_zerop(t1);
t3=C_u_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_6367,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t2,a[7]=((C_word*)t0)[7],a[8]=t3,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* c-backend.scm:447: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_make_character(40);
f_4547(3,av2);}}

/* k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_6367,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6370,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[12])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6427,a[2]=((C_word*)t0)[13],a[3]=((C_word*)t0)[14],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fast_retrieve(lf[191]))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6447,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14796,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:74: chicken.string#->string */
t6=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[15];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6454,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14801,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t6=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[15];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_6370(2,av2);}}}

/* k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_6370,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6373,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:455: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[205];
av2[4]=((C_word*)t0)[11];
av2[5]=C_make_character(40);
f_4547(6,av2);}}

/* k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_6373,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_6376,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[9])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6409,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:457: gen */
t4=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[204];
av2[3]=((C_word*)t0)[10];
av2[4]=C_make_character(41);
f_4547(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_6376(2,av2);}}}

/* k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_6376,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6379,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_6379(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6397,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:460: expr */
t4=((C_word*)((C_word*)t0)[7])[1];
f_4723(t4,t3,((C_word*)t0)[8],((C_word*)t0)[5]);}}

/* k6377 in k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_6379(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6379,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6382,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
/* c-backend.scm:462: expr-args */
t3=((C_word*)((C_word*)t0)[4])[1];
f_6924(t3,t2,((C_word*)t0)[3],((C_word*)t0)[5]);}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14789,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:463: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(41);
f_4547(3,av2);}}}

/* k6380 in k6377 in k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_6382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6382,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6385,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:463: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k6383 in k6380 in k6377 in k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_6385(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6385,c,av);}
/* c-backend.scm:464: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_make_character(41);
f_4547(4,av2);}}

/* k6395 in k6374 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_6397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6397,c,av);}
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
/* c-backend.scm:461: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_6379(2,av2);}}}

/* k6407 in k6371 in k6368 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6409,c,av);}
t2=C_i_not(((C_word*)t0)[2]);
if(C_truep(t2)){
if(C_truep(t2)){
/* c-backend.scm:458: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_6376(2,av2);}}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
/* c-backend.scm:458: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_6376(2,av2);}}}}

/* k6425 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6427(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6427,c,av);}
t2=(C_truep(C_fast_retrieve(lf[185]))?((C_word*)t0)[2]:C_SCHEME_FALSE);
if(C_truep(t2)){
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* c-backend.scm:453: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[206];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[207];
av2[6]=lf[208];
av2[7]=lf[209];
f_4547(8,av2);}}
else{
/* c-backend.scm:453: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[206];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[207];
av2[6]=lf[210];
av2[7]=lf[209];
f_4547(8,av2);}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_6370(2,av2);}}}

/* k6445 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6447,c,av);}
/* c-backend.scm:450: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[211];
av2[4]=t1;
av2[5]=lf[212];
f_4547(6,av2);}}

/* k6452 in k6365 in k6361 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6454,c,av);}
/* c-backend.scm:451: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[213];
av2[4]=t1;
av2[5]=lf[214];
f_4547(6,av2);}}

/* k6456 in k6348 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6458,c,av);}
/* c-backend.scm:445: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k6485 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_6487,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6490,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:475: push-args */
t3=((C_word*)((C_word*)t0)[5])[1];
f_7003(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],lf[223]);}

/* k6488 in k6485 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6490,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6497,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:476: toplevel */
t3=lf[222];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[4]);
f_9316(3,av2);}}

/* k6495 in k6488 in k6485 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6497(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6497,c,av);}
/* c-backend.scm:476: gen */
t2=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[219];
av2[4]=t1;
av2[5]=lf[220];
av2[6]=((C_word*)t0)[3];
av2[7]=lf[221];
f_4547(8,av2);}}

/* k6508 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6510,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6513,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:480: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6511 in k6508 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6513,c,av);}
/* c-backend.scm:481: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[225];
f_4547(3,av2);}}

/* k6527 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6529,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6532,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:485: expr-args */
t3=((C_word*)((C_word*)t0)[3])[1];
f_6924(t3,t2,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6530 in k6527 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6532,c,av);}
/* c-backend.scm:486: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k6563 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_6565,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6568,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6577,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:496: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
/* c-backend.scm:498: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}}

/* k6566 in k6563 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6568,c,av);}
/* c-backend.scm:498: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k6575 in k6563 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6577,c,av);}
/* c-backend.scm:497: expr-args */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6924(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6599 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6601,c,av);}
/* c-backend.scm:501: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_i_car(((C_word*)t0)[3]);
av2[4]=C_make_character(41);
f_4547(5,av2);}}

/* k6619 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6621,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6624,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:506: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6622 in k6619 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6624,c,av);}
/* c-backend.scm:507: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[240];
f_4547(3,av2);}}

/* k6637 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6639(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_6639,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6643,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:505: foreign-argument-conversion */
f_11704(t2,((C_word*)t0)[4]);}

/* k6641 in k6637 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6643,c,av);}
/* c-backend.scm:505: gen */
t2=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(40);
av2[3]=((C_word*)t0)[3];
av2[4]=lf[241];
av2[5]=((C_word*)t0)[4];
av2[6]=C_make_character(41);
av2[7]=t1;
f_4547(8,av2);}}

/* k6653 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6655,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6658,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:512: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6656 in k6653 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6658,c,av);}
/* c-backend.scm:513: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[246];
f_4547(3,av2);}}

/* k6667 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_6669,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6673,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:511: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[249];
tp(4,av2);}}

/* k6671 in k6667 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6673,c,av);}
/* c-backend.scm:511: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[247];
av2[4]=t1;
av2[5]=lf[248];
f_4547(6,av2);}}

/* k6683 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_6685,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6688,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:518: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6686 in k6683 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_6688,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6691,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6705,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:519: foreign-argument-conversion */
f_11704(t3,((C_word*)t0)[6]);}

/* k6689 in k6686 in k6683 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6691(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6691,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6694,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:520: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6692 in k6689 in k6686 in k6683 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6694,c,av);}
/* c-backend.scm:521: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[252];
f_4547(3,av2);}}

/* k6703 in k6686 in k6683 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6705,c,av);}
/* c-backend.scm:519: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[253];
av2[3]=t1;
f_4547(4,av2);}}

/* k6711 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6713,c,av);}
/* c-backend.scm:517: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[254];
av2[3]=t1;
av2[4]=lf[255];
f_4547(5,av2);}}

/* k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_6722,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:525: expr */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[3]),((C_word*)t0)[5]);}

/* k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_6725,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6728,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:526: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[260];
f_4547(3,av2);}}

/* k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_6728,c,av);}
a=C_alloc(7);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cdr(((C_word*)t0)[3]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6741,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_6741(t7,((C_word*)t0)[6],t2,t3);}

/* doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6741(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_6741,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_zerop(t2))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6749,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:530: gen */
t5=lf[4];{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[258];
f_4547(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6762,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t3,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:533: gen */
t5=lf[4];{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[259];
f_4547(4,av2);}}}

/* k6747 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6749,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6752,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:531: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6750 in k6747 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6752,c,av);}
/* c-backend.scm:532: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(125);
f_4547(3,av2);}}

/* k6760 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_6762,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6765,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:534: expr */
t3=((C_word*)((C_word*)t0)[6])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[5]),((C_word*)t0)[7]);}

/* k6763 in k6760 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_6765,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6768,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:535: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(58);
f_4547(3,av2);}}

/* k6766 in k6763 in k6760 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_6768(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6768,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6771,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:536: expr */
t3=((C_word*)((C_word*)t0)[6])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[5]),((C_word*)t0)[7]);}

/* k6769 in k6766 in k6763 in k6760 in doloop1125 in k6726 in k6723 in k6720 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_6771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_6771,c,av);}
a=C_alloc(29);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_6741(t3,((C_word*)t0)[4],t2,C_i_cddr(((C_word*)t0)[5]));}

/* k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6801,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6804,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:540: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6802 in k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6804,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6807,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:541: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[263];
f_4547(3,av2);}}

/* k6805 in k6802 in k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6807,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6810,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:542: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6808 in k6805 in k6802 in k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6810,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6813,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:543: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(58);
f_4547(3,av2);}}

/* k6811 in k6808 in k6805 in k6802 in k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6813,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6816,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:544: expr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4723(t3,t2,C_i_caddr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k6814 in k6811 in k6808 in k6805 in k6802 in k6799 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_6816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6816,c,av);}
/* c-backend.scm:545: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k6836 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6838(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6838,c,av);}
/* c-backend.scm:547: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[265];
av2[3]=t1;
tp(4,av2);}}

/* expr-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6924(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_6924,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6930,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_6930(t7,t1,t2);}

/* loop in expr-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6930(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_6930,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6940,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t4=C_eqp(t2,((C_word*)t0)[5]);
if(C_truep(t4)){
t5=t3;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_6940(2,av2);}}
else{
/* c-backend.scm:552: gen */
t5=lf[4];{
C_word av2[3];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_make_character(44);
f_4547(3,av2);}}}}

/* k6938 in loop in expr-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6940,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6943,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:553: expr */
t3=((C_word*)((C_word*)t0)[5])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[6]);}

/* k6941 in k6938 in loop in expr-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6943,c,av);}
/* c-backend.scm:554: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6930(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* contains-restop? in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6960(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_6960,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6966,a[2]=t4,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_6966(t6,t1,t2);}

/* loop in contains-restop? in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_6966(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_6966,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7001,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:562: chicken.compiler.support#node-class */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[184]);
C_word av2[3];
av2[0]=*((C_word*)lf[184]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}}

/* k6983 in k6999 in loop in contains-restop? in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6985,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* c-backend.scm:564: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_6966(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* k6995 in k6999 in loop in contains-restop? in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_6997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6997,c,av);}
/* c-backend.scm:563: contains-restop? */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6960(t2,((C_word*)t0)[3],t1);}

/* k6999 in loop in contains-restop? in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7001(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_7001,c,av);}
a=C_alloc(9);
t2=C_eqp(t1,lf[52]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6985,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6997,a[2]=((C_word*)t0)[5],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:563: chicken.compiler.support#node-subexpressions */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[266]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[266]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}}

/* push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7003(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,2)))){
C_save_and_reclaim_args((void *)trf_7003,5,t0,t1,t2,t3,t4);}
a=C_alloc(19);
t5=C_i_length(t2);
t6=(C_truep(t4)?C_fix(1):C_fix(0));
t7=C_a_i_fixnum_plus(&a,2,t5,t6);
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7015,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t1,a[6]=t2,a[7]=t7,a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[4],tmp=(C_word)a,a+=10,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7121,a[2]=t8,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:569: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word av2[3];
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t9;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7015(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_7015,2,t0,t1);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_7018,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:571: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word av2[3];
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}

/* k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_7018,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_7021,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:572: chicken.compiler.core#lambda-literal-rest-argument-mode */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[284]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[284]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
tp(3,av2);}}

/* k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_7021,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7024,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_i_not(((C_word*)t0)[7]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7072,a[2]=t2,a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t3)){
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t3;
f_7072(2,av2);}}
else{
if(C_truep(C_i_lessp(((C_word*)t0)[9],((C_word*)t0)[8]))){
t5=C_eqp(t1,lf[283]);
if(C_truep(t5)){
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
f_7072(2,av2);}}
else{
/* c-backend.scm:581: contains-restop? */
t6=((C_word*)((C_word*)t0)[10])[1];
f_6960(t6,t4,((C_word*)t0)[6]);}}
else{
/* c-backend.scm:581: contains-restop? */
t5=((C_word*)((C_word*)t0)[10])[1];
f_6960(t5,t4,((C_word*)t0)[6]);}}}

/* k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_7024,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7027,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[2])){
/* c-backend.scm:592: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[270];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[271];
f_4547(6,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7027(2,av2);}}}

/* k7025 in k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_7027,c,av);}
a=C_alloc(7);
t2=(C_truep(((C_word*)t0)[2])?C_fix(1):C_fix(0));
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7036,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_7036(t6,((C_word*)t0)[5],t2,((C_word*)t0)[6]);}

/* doloop1178 in k7025 in k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7036(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_7036,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t3))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7046,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:596: gen */
t5=lf[4];{
C_word av2[6];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[268];
av2[4]=t2;
av2[5]=lf[269];
f_4547(6,av2);}}}

/* k7044 in doloop1178 in k7025 in k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7046,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7049,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:597: expr */
t3=((C_word*)((C_word*)t0)[6])[1];
f_4723(t3,t2,C_i_car(((C_word*)t0)[5]),((C_word*)t0)[7]);}

/* k7047 in k7044 in doloop1178 in k7025 in k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7049,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7052,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:598: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[267];
f_4547(3,av2);}}

/* k7050 in k7047 in k7044 in doloop1178 in k7025 in k7022 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_7052,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_7036(t3,((C_word*)t0)[4],t2,C_u_i_cdr(((C_word*)t0)[5]));}

/* k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_7072,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
/* c-backend.scm:582: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[272];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[273];
f_4547(6,av2);}}
else{
if(C_truep(C_i_greater_or_equalp(((C_word*)t0)[4],((C_word*)t0)[3]))){
/* c-backend.scm:584: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[274];
f_4547(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7087,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:586: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[282];
f_4547(4,av2);}}}}

/* k7085 in k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_7087,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7090,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:587: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[280];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[281];
f_4547(6,av2);}}

/* k7088 in k7085 in k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7090,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7093,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:588: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[279];
f_4547(4,av2);}}

/* k7091 in k7088 in k7085 in k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7093,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7096,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:589: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[278];
f_4547(4,av2);}}

/* k7094 in k7091 in k7088 in k7085 in k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7096(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_7096,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7099,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:590: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[276];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[277];
f_4547(6,av2);}}

/* k7097 in k7094 in k7091 in k7088 in k7085 in k7070 in k7019 in k7016 in k7013 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7099(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7099,c,av);}
/* c-backend.scm:591: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[275];
f_4547(4,av2);}}

/* k7119 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7121,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7124,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];
f_7015(t3,C_i_not(t1));}
else{
/* c-backend.scm:570: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k7122 in k7119 in push-args in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7124(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7124,c,av);}
t2=((C_word*)t0)[2];
f_7015(t2,C_i_not(t1));}

/* header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7132(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,2)))){
C_save_and_reclaim_args((void *)trf_7132,2,t0,t1);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7136,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7222,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7234,a[2]=t3,a[3]=t6,a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7275,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:607: chicken.platform#chicken-version */
t10=C_fast_retrieve(lf[307]);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7136,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7139,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:610: gen-list */
f_4590(t2,((C_word*)t0)[3]);}

/* k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_7139,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7142,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_not(C_fast_retrieve(lf[295])))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7142(2,av2);}}
else{
/* c-backend.scm:612: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[296];
av2[4]=C_fast_retrieve(lf[295]);
f_4547(5,av2);}}}

/* k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7142,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7145,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_nullp(C_fast_retrieve(lf[293])))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7145(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7209,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:614: gen */
t4=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[294];
f_4547(4,av2);}}}

/* k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7145,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7148,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:616: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[292];
f_4547(4,av2);}}

/* k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7148(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_7148,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7151,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:617: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[289];
av2[4]=C_fast_retrieve(lf[290]);
av2[5]=lf[291];
f_4547(6,av2);}}

/* k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7151,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7154,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_fast_retrieve(lf[285]))){
/* c-backend.scm:619: generate-foreign-callback-stub-prototypes */
f_9499(t2,C_fast_retrieve(lf[287]));}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7154(2,av2);}}}

/* k7152 in k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7154,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7157,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_pairp(C_fast_retrieve(lf[288])))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7169,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:621: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}
else{
if(C_truep(C_fast_retrieve(lf[285]))){
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* c-backend.scm:624: generate-foreign-callback-stub-prototypes */
f_9499(((C_word*)t0)[2],C_fast_retrieve(lf[287]));}}}

/* k7155 in k7152 in k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7157,c,av);}
if(C_truep(C_fast_retrieve(lf[285]))){
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* c-backend.scm:624: generate-foreign-callback-stub-prototypes */
f_9499(((C_word*)t0)[2],C_fast_retrieve(lf[287]));}}

/* k7167 in k7152 in k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7169,c,av);}
a=C_alloc(5);
t2=C_fast_retrieve(lf[288]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7179,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_7179(t6,((C_word*)t0)[2],C_fast_retrieve(lf[288]));}

/* for-each-loop1241 in k7167 in k7152 in k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7179(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7179,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7189,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:622: gen */
t4=lf[4];{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_slot(t2,C_fix(0));
f_4547(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7187 in for-each-loop1241 in k7167 in k7152 in k7149 in k7146 in k7143 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7189,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7179(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k7207 in k7140 in k7137 in k7134 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7209(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7209,c,av);}
/* c-backend.scm:615: gen-list */
f_4590(((C_word*)t0)[2],C_fast_retrieve(lf[293]));}

/* k7220 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7222(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_7222,c,av);}
/* c-backend.scm:603: gen */
t2=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[297];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[298];
av2[5]=C_SCHEME_TRUE;
av2[6]=lf[299];
av2[7]=C_SCHEME_TRUE;
av2[8]=t1;
av2[9]=lf[300];
f_4547(10,av2);}}

/* k7232 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7234(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7234,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7237,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7239,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_7239(t6,t2,t1);}

/* k7235 in k7232 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7237(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7237,c,av);}
/* c-backend.scm:605: chicken.string#string-intersperse */
t2=C_fast_retrieve(lf[301]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[302];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop1206 in k7232 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7239(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_7239,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7264,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:606: g1229 */
t4=*((C_word*)lf[152]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[303];
av2[3]=C_slot(t2,C_fix(0));
av2[4]=lf[304];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7262 in map-loop1206 in k7232 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7264,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_7239(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k7273 in header in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7275,c,av);}
/* c-backend.scm:607: chicken.string#string-split */
t2=C_fast_retrieve(lf[305]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[306];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k7283 in k9104 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,10)))){
C_save_and_reclaim((void *)f_7285,c,av);}
/* c-backend.scm:627: gen */
t2=lf[4];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[501];
av2[5]=C_SCHEME_TRUE;
av2[6]=t1;
av2[7]=lf[502];
av2[8]=C_SCHEME_TRUE;
av2[9]=lf[503];
av2[10]=C_SCHEME_TRUE;
f_4547(11,av2);}}

/* k7287 in k9104 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7289,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14874,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:75: chicken.string#->string */
t3=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7291(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_7291,2,t0,t1);}
a=C_alloc(5);
t2=C_i_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7298,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:636: gen */
t4=lf[4];{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[321];
f_4547(5,av2);}}

/* k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_7298,c,av);}
a=C_alloc(17);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=lf[222];
t7=C_fast_retrieve(lf[293]);
t8=C_i_check_list_2(C_fast_retrieve(lf[293]),lf[308]);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7312,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7461,a[2]=t4,a[3]=t11,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t13=((C_word*)t11)[1];
f_7461(t13,t9,C_fast_retrieve(lf[293]));}

/* k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7312,c,av);}
a=C_alloc(10);
t2=C_i_check_list_2(t1,lf[91]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7438,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7438(t7,t3,t1);}

/* k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_7318,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7321,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(((C_word*)t0)[4],C_fix(0));
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_7321(2,av2);}}
else{
/* c-backend.scm:643: gen */
t4=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[315];
av2[5]=((C_word*)t0)[4];
av2[6]=lf[316];
f_4547(7,av2);}}}

/* k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7321,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7324,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:644: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[314];
f_4547(4,av2);}}

/* k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_7324,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7329,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_7329(t5,((C_word*)t0)[2],C_fix(0),((C_word*)t0)[3]);}

/* doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7329(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7329,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t3))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7339,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:648: ##sys#lambda-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[313]);
C_word av2[3];
av2[0]=*((C_word*)lf[313]+1);
av2[1]=t4;
av2[2]=C_i_car(t3);
tp(3,av2);}}}

/* k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,11)))){
C_save_and_reclaim((void *)f_7339,c,av);}
a=C_alloc(23);
t2=C_i_string_length(t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7345,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t4=C_s_a_i_arithmetic_shift(&a,2,t2,C_fix(-16));
t5=C_s_a_i_arithmetic_shift(&a,2,t2,C_fix(-8));
t6=C_s_a_i_bitwise_and(&a,2,C_fix(255),t5);
/* c-backend.scm:650: gen */
t7=lf[4];{
C_word *av2;
if(c >= 12) {
  av2=av;
} else {
  av2=C_alloc(12);
}
av2[0]=t7;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[311];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[312];
av2[6]=t4;
av2[7]=C_make_character(44);
av2[8]=t6;
av2[9]=C_make_character(44);
av2[10]=C_u_fixnum_and(C_fix(255),t2);
av2[11]=C_make_character(41);
f_4547(12,av2);}}

/* k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_7345,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7348,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7387,a[2]=((C_word*)t0)[6],a[3]=t4,a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_7387(t6,t2,C_fix(0));}

/* k7346 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_7348,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7351,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[6],C_fix(7));
t4=C_s_a_i_bitwise_and(&a,2,C_fix(16777208),t3);
t5=C_s_a_u_i_integer_minus(&a,2,t4,((C_word*)t0)[6]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7369,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_7369(t9,t2,t5);}

/* k7349 in k7346 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7351,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7354,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:661: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[309];
f_4547(3,av2);}}

/* k7352 in k7349 in k7346 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_7354,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_7329(t3,((C_word*)t0)[4],t2,C_u_i_cdr(((C_word*)t0)[5]));}

/* doloop1318 in k7346 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_fcall f_7369(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7369,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_zerop(t2))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7377,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:660: gen */
t4=lf[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[310];
f_4547(3,av2);}}}

/* k7375 in doloop1318 in k7346 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_7377,c,av);}
a=C_alloc(29);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_7369(t3,((C_word*)t0)[4],t2);}

/* doloop1317 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7387(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7387,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7397,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=C_i_string_ref(((C_word*)t0)[4],t2);
/* c-backend.scm:657: gen */
t5=lf[4];{
C_word av2[4];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_make_character(44);
av2[3]=C_fix(C_character_code(t4));
f_4547(4,av2);}}}

/* k7395 in doloop1317 in k7343 in k7337 in doloop1277 in k7322 in k7319 in k7316 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7397,c,av);}
a=C_alloc(5);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_7387(t3,((C_word*)t0)[4],t2);}

/* for-each-loop1269 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7438(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,9)))){
C_save_and_reclaim_args((void *)trf_7438,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7448,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
/* c-backend.scm:639: gen */
t5=lf[4];{
C_word av2[10];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[317];
av2[4]=t4;
av2[5]=lf[318];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[319];
av2[8]=t4;
av2[9]=lf[320];
f_4547(10,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7446 in for-each-loop1269 in k7310 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7448,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7438(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* map-loop1283 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7461(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7461,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7486,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:641: g1289 */
t4=lf[222];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_9316(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7484 in map-loop1283 in k7296 in declarations in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7486(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7486,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_7461(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7495(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_7495,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7499,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:664: gen */
t3=lf[4];{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}

/* k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7499,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7650,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_7650(t6,((C_word*)t0)[3],((C_word*)t0)[2]);}

/* k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7508,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7511,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:670: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_7511,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7514,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7642,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:671: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
t3=t2;
f_7514(t3,C_SCHEME_FALSE);}}

/* k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7514(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(38,0,3)))){
C_save_and_reclaim_args((void *)trf_7514,2,t0,t1);}
a=C_alloc(38);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7517,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7633,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t4=C_s_a_i_minus(&a,2,((C_word*)t0)[6],C_fix(1));
/* c-backend.scm:672: make-variable-list */
f_9408(t3,t4,lf[340]);}
else{
/* c-backend.scm:672: make-variable-list */
f_9408(t3,((C_word*)t0)[6],lf[340]);}}

/* k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7517(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7517,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7520,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:673: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7520,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7523,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:674: chicken.compiler.core#lambda-literal-allocated */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[338]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[338]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7523(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_7523,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7526,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:675: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}

/* k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_7526,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7529,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_eqp(lf[326],((C_word*)t0)[7]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7591,a[2]=t2,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:677: gen */
t5=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[332];
av2[3]=((C_word*)t0)[7];
av2[4]=lf[333];
av2[5]=C_SCHEME_TRUE;
f_4547(6,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7616,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:685: toplevel */
t5=lf[222];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_fast_retrieve(lf[295]);
f_9316(3,av2);}}}

/* k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7529,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7532,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:689: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(40);
f_4547(3,av2);}}

/* k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7532,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7535,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7535(2,av2);}}
else{
/* c-backend.scm:690: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[325];
f_4547(3,av2);}}}

/* k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_7535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_7535,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7538,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(C_truep(((C_word*)t0)[3])?C_i_not(C_i_zerop(((C_word*)t0)[6])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7568,a[2]=((C_word*)t0)[5],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:692: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[324];
f_4547(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_7538(2,av2);}}}

/* k7536 in k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_7538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7538,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7541,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
if(C_truep(((C_word*)t0)[4])){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=lf[4];
av2[3]=((C_word*)t0)[5];
C_apply(4,av2);}}
else{
/* c-backend.scm:696: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[323];
f_4547(3,av2);}}}
else{
if(C_truep(((C_word*)t0)[3])){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=lf[4];
av2[3]=((C_word*)t0)[5];
C_apply(4,av2);}}
else{
/* c-backend.scm:696: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[323];
f_4547(3,av2);}}}}

/* k7539 in k7536 in k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_7541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7541,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7544,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:697: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(41);
f_4547(3,av2);}}

/* k7542 in k7539 in k7536 in k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_7544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7544,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7547,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
/* c-backend.scm:699: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}
else{
/* c-backend.scm:698: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[322];
f_4547(3,av2);}}}

/* k7545 in k7542 in k7539 in k7536 in k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_7547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7547,c,av);}
/* c-backend.scm:699: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* k7566 in k7533 in k7530 in k7527 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_7568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7568,c,av);}
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
/* c-backend.scm:693: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_7538(2,av2);}}}

/* k7589 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7591,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7594,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:678: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[331];
f_4547(3,av2);}}

/* k7592 in k7589 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7594,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7597,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
/* c-backend.scm:679: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[329];
f_4547(3,av2);}}
else{
/* c-backend.scm:679: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[330];
f_4547(3,av2);}}}

/* k7595 in k7592 in k7589 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_7597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7597,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7600,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:681: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[327];
f_4547(3,av2);}}
else{
/* c-backend.scm:682: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[328];
f_4547(3,av2);}}}

/* k7598 in k7595 in k7592 in k7589 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_7600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7600,c,av);}
/* c-backend.scm:683: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
f_4547(3,av2);}}

/* k7614 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7616(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_7616,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7619,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:686: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[336];
av2[3]=t1;
av2[4]=lf[337];
av2[5]=C_SCHEME_TRUE;
f_4547(6,av2);}}

/* k7617 in k7614 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7619(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7619,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7622,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:687: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[335];
f_4547(3,av2);}}

/* k7620 in k7617 in k7614 in k7524 in k7521 in k7518 in k7515 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_7622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7622,c,av);}
/* c-backend.scm:688: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[334];
av2[3]=((C_word*)t0)[3];
f_4547(4,av2);}}

/* k7631 in k7512 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7633,c,av);}
/* c-backend.scm:672: chicken.base#intersperse */
t2=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k7640 in k7509 in k7506 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7642,c,av);}
t2=((C_word*)t0)[2];
f_7514(t2,C_i_zerop(t1));}

/* for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7650(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_7650,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7660,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_car(t4);
t6=C_u_i_cdr(t4);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7508,a[2]=t3,a[3]=t5,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:669: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word av2[3];
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t7;
av2[2]=t6;
tp(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7658 in for-each-loop1338 in k7497 in prototypes in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7660,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7650(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7673(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7673,2,t0,t1);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7771,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_7771(t6,t1,((C_word*)t0)[2]);}

/* doloop1403 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7684(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,7)))){
C_save_and_reclaim_args((void *)trf_7684,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7694,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:711: gen */
t5=lf[4];{
C_word av2[8];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[344];
av2[4]=t2;
av2[5]=lf[345];
av2[6]=t3;
av2[7]=lf[346];
f_4547(8,av2);}}}

/* k7692 in doloop1403 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,3)))){
C_save_and_reclaim((void *)f_7694,c,av);}
a=C_alloc(58);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=((C_word*)((C_word*)t0)[4])[1];
f_7684(t4,((C_word*)t0)[5],t2,t3);}

/* k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_7710,c,av);}
a=C_alloc(11);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7713,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:718: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}

/* k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_7713,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7716,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7763,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:719: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}
else{
t3=t2;
f_7716(t3,C_SCHEME_FALSE);}}

/* k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7716(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(39,0,2)))){
C_save_and_reclaim_args((void *)trf_7716,2,t0,t1);}
a=C_alloc(39);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7719,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t1)){
t3=((C_word*)((C_word*)t0)[4])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[4])+1,C_s_a_i_minus(&a,2,t3,C_fix(1)));
t5=t2;
f_7719(t5,t4);}
else{
t3=t2;
f_7719(t3,C_SCHEME_UNDEFINED);}}

/* k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7719(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_7719,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7757,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:721: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word av2[3];
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}

/* k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_7728,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7731,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:724: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[347];
av2[4]=((C_word*)t0)[4];
av2[5]=lf[348];
f_4547(6,av2);}}

/* k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(40,c,4)))){
C_save_and_reclaim((void *)f_7731,c,av);}
a=C_alloc(40);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7734,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_s_a_i_minus(&a,2,t3,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7684,a[2]=t3,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_7684(t8,t2,C_fix(0),t4);}

/* k7732 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7734(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_7734,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7737,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:726: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=C_make_character(40);
f_4547(5,av2);}}

/* k7735 in k7732 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_7737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7737,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7740,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:727: make-argument-list */
f_9425(t2,((C_word*)((C_word*)t0)[3])[1],lf[343]);}

/* k7738 in k7735 in k7732 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_7740(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7740,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7743,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7750,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:728: chicken.base#intersperse */
t4=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k7741 in k7738 in k7735 in k7732 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_7743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7743,c,av);}
/* c-backend.scm:729: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[341];
f_4547(3,av2);}}

/* k7748 in k7738 in k7735 in k7732 in k7729 in k7726 in k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_7750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7750,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[4];
av2[3]=t1;
C_apply(4,av2);}}

/* k7755 in k7717 in k7714 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7757(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_7757,c,av);}
a=C_alloc(5);
t2=C_i_not(t1);
t3=(C_truep(t2)?((C_word*)t0)[2]:C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7728,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:722: gen */
t5=lf[4];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[349];
av2[5]=((C_word*)t0)[5];
av2[6]=lf[350];
av2[7]=C_SCHEME_TRUE;
av2[8]=lf[351];
av2[9]=((C_word*)t0)[5];
av2[10]=lf[352];
f_4547(11,av2);}}
else{
t4=((C_word*)((C_word*)t0)[6])[1];
f_7771(t4,((C_word*)t0)[7],C_slot(((C_word*)t0)[8],C_fix(1)));}}

/* k7761 in k7711 in k7708 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7763,c,av);}
t2=((C_word*)t0)[2];
f_7716(t2,C_i_zerop(t1));}

/* for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7771(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_7771,3,t0,t1,t2);}
a=C_alloc(13);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7781,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_car(t4);
t6=C_u_i_cdr(t4);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7710,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t2,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:717: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word av2[3];
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t7;
av2[2]=t6;
tp(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7779 in for-each-loop1393 in trampolines in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7781,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7771(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7794(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_7794,2,t0,t1);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7800,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_7800(t5,t1,C_fix(0),((C_word*)t0)[3]);}

/* doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7800(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_7800,4,t0,t1,t2,t3);}
a=C_alloc(12);
if(C_truep(C_i_nullp(t3))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7810,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t5=C_i_car(t3);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7825,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=t5,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:736: chicken.base#open-output-string */
t7=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k7808 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_7810,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_7800(t3,((C_word*)t0)[4],t2,C_u_i_cdr(((C_word*)t0)[5]));}

/* k7823 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_7825,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7831,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:736: ##sys#print */
t4=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[357];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k7829 in k7823 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_7831,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7834,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:736: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k7832 in k7829 in k7823 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7834,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7837,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:736: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t2;
av2[2]=C_make_character(93);
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}

/* k7835 in k7832 in k7829 in k7823 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7837,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7840,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:736: chicken.base#get-output-string */
t3=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7838 in k7835 in k7832 in k7829 in k7823 in doloop1432 in literal-frame in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7840,c,av);}
/* c-backend.scm:736: gen-lit */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8131(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7848(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7848,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7855,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:742: chicken.compiler.support#immediate? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[368]);
C_word av2[3];
av2[0]=*((C_word*)lf[368]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7855,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7861,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:743: chicken.compiler.support#big-fixnum? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[367]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[367]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7861,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(2);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_bignump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(2);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(((C_word*)t0)[3]))){
t2=C_fast_retrieve(lf[359]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fast_retrieve(lf[359]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(7);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7891,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:748: chicken.keyword#keyword? */
t3=C_fast_retrieve(lf[366]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}}}}

/* k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_7891,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(7);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7904,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:749: literal-size */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7848(t3,t2,C_u_i_car(((C_word*)t0)[3]));}
else{
if(C_truep(C_i_vectorp(((C_word*)t0)[3]))){
t2=C_block_size(((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7932,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t5,a[5]=((C_word*)t0)[4],a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:752: scheme#vector->list */
t8=*((C_word*)lf[361]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8006,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:753: chicken.compiler.support#block-variable-literal? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[365]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}}}

/* k7902 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7904,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7908,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:749: literal-size */
t3=((C_word*)((C_word*)t0)[3])[1];
f_7848(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k7906 in k7902 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_7908,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15423,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=t2;
va15423(t3,C_s_a_i_plus(&a,2,C_fix(3),((C_word*)t0)[3]));}

/* k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_7932,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7935,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7967,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_7967(t6,t2,t1);}

/* k7933 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_7935,c,av);}
a=C_alloc(9);
t2=C_i_check_list_2(t1,lf[360]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7941,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7943,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7943(t7,t3,t1,C_fix(0));}

/* k7939 in k7933 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_7941,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15425,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=t2;
va15425(t3,C_s_a_i_plus(&a,2,C_fix(1),((C_word*)t0)[3]));}

/* foldl1461 in k7933 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7943(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,3)))){
C_save_and_reclaim_args((void *)trf_7943,4,t0,t1,t2,t3);}
a=C_alloc(29);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=C_slot(t2,C_fix(0));
t6=C_s_a_i_plus(&a,2,t3,t5);
t8=t1;
t9=t4;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* map-loop1467 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_7967(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7967,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7992,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:752: g1473 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_7848(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7990 in map-loop1467 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_7992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7992,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_7967(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8006,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_immp(((C_word*)t0)[3]))){
/* c-backend.scm:739: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[362];
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}
else{
if(C_truep(C_lambdainfop(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8024,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:756: ##sys#bytevector? */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[364]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[364]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}}}

/* k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_8024,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8031,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:756: chicken.compiler.support#bytes->words */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[363]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[363]+1);
av2[1]=t2;
av2[2]=C_block_size(((C_word*)t0)[3]);
tp(3,av2);}}
else{
if(C_truep(C_structurep(((C_word*)t0)[3]))){
t2=C_block_size(((C_word*)t0)[3]);
t3=C_a_i_fixnum_plus(&a,2,C_fix(2),t2);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8052,a[2]=t2,a[3]=t5,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_8052(t7,((C_word*)t0)[2],C_fix(0),t3);}
else{
if(C_truep(C_i_ratnump(((C_word*)t0)[3]))){
t2=C_block_size(((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8093,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:765: literal-size */
t4=((C_word*)((C_word*)t0)[4])[1];
f_7848(t4,t3,C_u_i_ratnum_num(((C_word*)t0)[3]));}
else{
if(C_truep(C_i_cplxnump(((C_word*)t0)[3]))){
t2=C_block_size(((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8118,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:768: literal-size */
t4=((C_word*)((C_word*)t0)[4])[1];
f_7848(t4,t3,C_u_i_cplxnum_real(((C_word*)t0)[3]));}
else{
/* c-backend.scm:739: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[362];
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}}}}}

/* k8029 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8031(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_8031,c,av);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,C_fix(2),t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8052(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(35,0,2)))){
C_save_and_reclaim_args((void *)trf_8052,4,t0,t1,t2,t3);}
a=C_alloc(35);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_s_a_i_plus(&a,2,t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8072,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:762: literal-size */
t6=((C_word*)((C_word*)t0)[4])[1];
f_7848(t6,t5,C_slot(((C_word*)t0)[5],t2));}}

/* k8070 in loop in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_8072,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
/* c-backend.scm:762: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8052(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k8091 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8093,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8097,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:766: literal-size */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7848(t3,t2,C_u_i_ratnum_denom(((C_word*)t0)[5]));}

/* k8095 in k8091 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_8097,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15433,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=t2;
va15433(t3,C_s_a_i_plus(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k8116 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8118,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8122,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:769: literal-size */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7848(t3,t2,C_u_i_cplxnum_imag(((C_word*)t0)[5]));}

/* k8120 in k8116 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_8122,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)va15435,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=t2;
va15435(t3,C_s_a_i_plus(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8131(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_8131,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8138,a[2]=t1,a[3]=t3,a[4]=t2,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnump(t2))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8283,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:774: chicken.compiler.support#big-fixnum? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[367]);
C_word av2[3];
av2[0]=*((C_word*)lf[367]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}
else{
t5=t4;
f_8138(t5,C_SCHEME_FALSE);}}

/* k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8138(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,6)))){
C_save_and_reclaim_args((void *)trf_8138,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* c-backend.scm:775: gen */
t2=lf[4];{
C_word av2[7];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=lf[369];
av2[5]=((C_word*)t0)[4];
av2[6]=lf[370];
f_4547(7,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8144,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:776: chicken.compiler.support#block-variable-literal? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[365]);
C_word av2[3];
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}}

/* k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_8144,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=*((C_word*)lf[371]+1);
t3=C_eqp(((C_word*)t0)[3],*((C_word*)lf[371]+1));
if(C_truep(t3)){
/* c-backend.scm:778: gen */
t4=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=lf[372];
f_4547(5,av2);}}
else{
if(C_truep(C_booleanp(((C_word*)t0)[3]))){
if(C_truep(((C_word*)t0)[3])){
/* c-backend.scm:780: gen */
t4=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=C_make_character(61);
av2[5]=lf[373];
av2[6]=C_make_character(59);
f_4547(7,av2);}}
else{
/* c-backend.scm:780: gen */
t4=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=C_make_character(61);
av2[5]=lf[374];
av2[6]=C_make_character(59);
f_4547(7,av2);}}}
else{
if(C_truep(C_charp(((C_word*)t0)[3]))){
/* c-backend.scm:782: gen */
t4=lf[4];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=lf[375];
av2[5]=C_fix(C_character_code(((C_word*)t0)[3]));
av2[6]=lf[376];
f_4547(7,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8188,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:783: chicken.keyword#keyword? */
t5=C_fast_retrieve(lf[366]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}}}

/* k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8188(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8188,c,av);}
a=C_alloc(6);
t2=(C_truep(t1)?t1:C_i_symbolp(((C_word*)t0)[2]));
if(C_truep(t2)){
t3=C_slot(((C_word*)t0)[2],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8197,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:785: chicken.compiler.support#c-ify-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[107]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[107]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}
else{
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
/* c-backend.scm:793: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=lf[383];
f_4547(5,av2);}}
else{
t3=C_immp(((C_word*)t0)[2]);
t4=C_i_not(t3);
t5=(C_truep(t4)?C_lambdainfop(((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_fixnump(((C_word*)t0)[2]);
t7=(C_truep(t6)?t6:C_i_not(C_immp(((C_word*)t0)[2])));
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8239,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:797: gen */
t9=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[4];
av2[4]=lf[386];
f_4547(5,av2);}}
else{
/* c-backend.scm:739: chicken.compiler.support#bomb */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[10]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[10]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[362];
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}}}}}

/* k8195 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_8197,c,av);}
a=C_alloc(6);
t2=C_block_size(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8212,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:787: chicken.keyword#keyword? */
t4=C_fast_retrieve(lf[366]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8204 in k8210 in k8195 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_8206,c,av);}
/* c-backend.scm:791: gen */
t2=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[379];
av2[4]=((C_word*)t0)[4];
av2[5]=C_make_character(44);
av2[6]=((C_word*)t0)[5];
av2[7]=lf[380];
av2[8]=((C_word*)t0)[6];
av2[9]=lf[381];
f_4547(10,av2);}}

/* k8210 in k8195 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_8212,c,av);}
a=C_alloc(7);
t2=(C_truep(t1)?lf[377]:lf[378]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8206,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:790: gen */
t4=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=lf[382];
f_4547(5,av2);}}

/* k8237 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_8239,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8242,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8249,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:798: encode-literal */
f_13156(t3,((C_word*)t0)[4]);}

/* k8240 in k8237 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8242,c,av);}
/* c-backend.scm:799: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[384];
f_4547(3,av2);}}

/* k8247 in k8237 in k8186 in k8142 in k8136 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8249(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8249,c,av);}
/* c-backend.scm:798: gen-string-constant */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8285(t2,((C_word*)t0)[3],t1);}

/* k8281 in gen-lit in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8283,c,av);}
t2=((C_word*)t0)[2];
f_8138(t2,C_i_not(t1));}

/* gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8285(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_8285,3,t0,t1,t2);}
a=C_alloc(9);
t3=C_block_size(t2);
t4=C_fixnum_divide(t3,C_fix(80));
t5=C_fixnum_modulo(t3,C_fix(80));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8299,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t7,tmp=(C_word)a,a+=7,tmp));
t9=((C_word*)t7)[1];
f_8299(t9,t1,t4,C_fix(0));}

/* doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8299(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(41,0,4)))){
C_save_and_reclaim_args((void *)trf_8299,4,t0,t1,t2,t3);}
a=C_alloc(41);
if(C_truep(C_i_zerop(t2))){
t4=C_eqp(((C_word*)t0)[2],C_fix(0));
t5=(C_truep(t4)?t4:C_i_not(C_eqp(((C_word*)t0)[3],C_fix(0))));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8320,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8324,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:810: string-like-substring */
f_8359(t7,((C_word*)t0)[5],t3,((C_word*)t0)[2]);}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8336,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[6],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8349,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8353,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=C_s_a_i_plus(&a,2,t3,C_fix(80));
/* c-backend.scm:811: string-like-substring */
f_8359(t6,((C_word*)t0)[5],t3,t7);}}

/* k8318 in doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8320,c,av);}
/* c-backend.scm:810: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_4547(3,av2);}}

/* k8322 in doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8324,c,av);}
/* c-backend.scm:810: chicken.compiler.support#c-ify-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[107]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[107]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k8334 in doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,3)))){
C_save_and_reclaim((void *)f_8336,c,av);}
a=C_alloc(58);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[3],C_fix(80));
t4=((C_word*)((C_word*)t0)[4])[1];
f_8299(t4,((C_word*)t0)[5],t2,t3);}

/* k8347 in doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8349,c,av);}
/* c-backend.scm:811: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
f_4547(4,av2);}}

/* k8351 in doloop1532 in gen-string-constant in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8353,c,av);}
/* c-backend.scm:811: chicken.compiler.support#c-ify-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[107]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[107]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* string-like-substring in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8359(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(35,0,2)))){
C_save_and_reclaim_args((void *)trf_8359,4,t1,t2,t3,t4);}
a=C_alloc(35);
t5=C_s_a_i_minus(&a,2,t4,t3);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8366,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:815: scheme#make-string */
t7=*((C_word*)lf[388]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k8364 in string-like-substring in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_8366,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8369,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:816: ##sys#copy-bytes */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[387]);
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=*((C_word*)lf[387]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
av2[5]=C_fix(0);
av2[6]=((C_word*)t0)[5];
tp(7,av2);}}

/* k8367 in k8364 in string-like-substring in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8369,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8371(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_8371,2,t0,t1);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8373,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=C_i_check_list_2(((C_word*)t0)[8],lf[91]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9051,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_9051(t7,t1,((C_word*)t0)[8]);}

/* g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8373(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_8373,3,t0,t1,t2);}
a=C_alloc(11);
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8381,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t4,a[6]=t3,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:824: chicken.compiler.core#lambda-literal-argument-count */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[61]);
C_word av2[3];
av2[0]=*((C_word*)lf[61]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_8381,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8384,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:825: chicken.compiler.support#real-name */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[500]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[500]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[10];
tp(4,av2);}}

/* k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_8384,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_8387,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:826: chicken.compiler.core#lambda-literal-allocated */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[338]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[338]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_8387,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8390,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9043,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:827: chicken.compiler.core#lambda-literal-callee-signatures */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[499]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[499]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_8390,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8393,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* c-backend.scm:828: chicken.compiler.core#lambda-literal-rest-argument */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[497]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[497]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_8393,c,av);}
a=C_alloc(17);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_8396,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
/* c-backend.scm:829: chicken.compiler.core#lambda-literal-customizable */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[60]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[60]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_8396,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_8399,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9039,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:830: chicken.compiler.core#lambda-literal-closure-size */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[197]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[197]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}
else{
t3=t2;
f_8399(t3,C_SCHEME_FALSE);}}

/* k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_8399(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(47,0,3)))){
C_save_and_reclaim_args((void *)trf_8399,2,t0,t1);}
a=C_alloc(47);
t2=(C_truep(t1)?C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1)):C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(0)));
t3=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_8405,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t2,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=t1,a[17]=((C_word*)t0)[15],tmp=(C_word)a,a+=18,tmp);
/* c-backend.scm:832: make-variable-list */
f_9408(t3,((C_word*)t0)[2],lf[496]);}

/* k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_8405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_8405,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_8408,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=t1,tmp=(C_word)a,a+=19,tmp);
/* c-backend.scm:833: make-argument-list */
f_9425(t2,((C_word*)t0)[6],lf[495]);}

/* k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_8408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_8408,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_8411,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=t1,tmp=(C_word)a,a+=19,tmp);
if(C_truep(((C_word*)t0)[16])){
/* c-backend.scm:834: chicken.base#intersperse */
t3=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cdr(((C_word*)t0)[18]);
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* c-backend.scm:834: chicken.base#intersperse */
t3=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[18];
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_8411(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_8411,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_8414,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=t1,a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
if(C_truep(((C_word*)t0)[16])){
/* c-backend.scm:835: chicken.base#intersperse */
t3=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cdr(((C_word*)t0)[18]);
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* c-backend.scm:835: chicken.base#intersperse */
t3=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[18];
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_8414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_8414,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_8417,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],tmp=(C_word)a,a+=20,tmp);
/* c-backend.scm:836: chicken.compiler.core#lambda-literal-external */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[494]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[494]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_8417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_8417,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_8420,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=t1,a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
/* c-backend.scm:837: chicken.compiler.core#lambda-literal-looping */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[145]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[145]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_8420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_8420,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8423,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t1,a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
/* c-backend.scm:838: chicken.compiler.core#lambda-literal-direct */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[59]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[59]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_8423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_8423,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_8426,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],tmp=(C_word)a,a+=23,tmp);
/* c-backend.scm:839: chicken.compiler.core#lambda-literal-rest-argument-mode */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[284]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[284]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_8426(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,2)))){
C_save_and_reclaim((void *)f_8426,c,av);}
a=C_alloc(24);
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_8429,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=t1,a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],tmp=(C_word)a,a+=24,tmp);
/* c-backend.scm:840: chicken.compiler.core#lambda-literal-temporaries */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[139]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[139]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in ... */
static void C_ccall f_8429(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,2)))){
C_save_and_reclaim((void *)f_8429,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8432,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],tmp=(C_word)a,a+=25,tmp);
/* c-backend.scm:841: chicken.compiler.core#lambda-literal-float-temporaries */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[493]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[493]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in ... */
static void C_ccall f_8432(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_8432,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8435,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=t1,a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],tmp=(C_word)a,a+=26,tmp);
/* c-backend.scm:842: toplevel */
t3=lf[222];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fast_retrieve(lf[295]);
f_9316(3,av2);}}

/* k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in ... */
static void C_ccall f_8435(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_8435,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_8438,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=t1,a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],tmp=(C_word)a,a+=27,tmp);
if(C_truep(((C_word*)t0)[20])){
/* c-backend.scm:843: chicken.compiler.support#debugging */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[490]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[490]+1);
av2[1]=t2;
av2[2]=lf[491];
av2[3]=lf[492];
av2[4]=((C_word*)t0)[9];
tp(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8438(2,av2);}}}

/* k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in ... */
static void C_ccall f_8438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_8438,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_8441,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],a[26]=((C_word*)t0)[26],tmp=(C_word)a,a+=27,tmp);
/* c-backend.scm:844: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
f_4547(4,av2);}}

/* k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in ... */
static void C_ccall f_8441(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_8441,c,av);}
a=C_alloc(29);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8444,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],tmp=(C_word)a,a+=26,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9014,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:845: cleanup */
f_9332(t3,((C_word*)t0)[26]);}

/* k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in ... */
static void C_ccall f_8444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,2)))){
C_save_and_reclaim((void *)f_8444,c,av);}
a=C_alloc(32);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8447,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],tmp=(C_word)a,a+=26,tmp);
t3=C_eqp(lf[326],((C_word*)t0)[9]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8975,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:847: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[483];
f_4547(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8997,a[2]=t2,a[3]=((C_word*)t0)[19],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:854: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[486];
f_4547(3,av2);}}}

/* k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in ... */
static void C_ccall f_8447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_8447,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8450,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],tmp=(C_word)a,a+=26,tmp);
/* c-backend.scm:858: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(40);
f_4547(3,av2);}}

/* k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in ... */
static void C_ccall f_8450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_8450,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8453,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],tmp=(C_word)a,a+=26,tmp);
if(C_truep(((C_word*)t0)[3])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8453(2,av2);}}
else{
/* c-backend.scm:859: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[478];
f_4547(3,av2);}}}

/* k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in ... */
static void C_ccall f_8453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,2)))){
C_save_and_reclaim((void *)f_8453,c,av);}
a=C_alloc(30);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_8456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],tmp=(C_word)a,a+=26,tmp);
t3=(C_truep(((C_word*)t0)[10])?C_i_not(C_i_zerop(((C_word*)t0)[12])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8952,a[2]=((C_word*)t0)[25],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:861: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[477];
f_4547(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_8456(2,av2);}}}

/* k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in ... */
static void C_ccall f_8456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,3)))){
C_save_and_reclaim((void *)f_8456,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8459,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],tmp=(C_word)a,a+=25,tmp);
if(C_truep(((C_word*)t0)[3])){
if(C_truep(((C_word*)t0)[3])){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=lf[4];
av2[3]=((C_word*)t0)[25];
C_apply(4,av2);}}
else{
/* c-backend.scm:865: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[476];
f_4547(3,av2);}}}
else{
if(C_truep(((C_word*)t0)[10])){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=lf[4];
av2[3]=((C_word*)t0)[25];
C_apply(4,av2);}}
else{
/* c-backend.scm:865: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[476];
f_4547(3,av2);}}}}

/* k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in ... */
static void C_ccall f_8459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,2)))){
C_save_and_reclaim((void *)f_8459,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8462,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],tmp=(C_word)a,a+=25,tmp);
/* c-backend.scm:866: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[475];
f_4547(3,av2);}}

/* k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in ... */
static void C_ccall f_8462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,2)))){
C_save_and_reclaim((void *)f_8462,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8465,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],tmp=(C_word)a,a+=25,tmp);
t3=C_eqp(((C_word*)t0)[22],lf[283]);
if(C_truep(t3)){
t4=C_set_block_item(((C_word*)t0)[5],0,C_SCHEME_FALSE);
t5=t2;
f_8465(t5,t4);}
else{
t4=t2;
f_8465(t4,C_SCHEME_UNDEFINED);}}

/* k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in ... */
static void C_fcall f_8465(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,3)))){
C_save_and_reclaim_args((void *)trf_8465,2,t0,t1);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8468,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],tmp=(C_word)a,a+=25,tmp);
/* c-backend.scm:868: gen */
t3=lf[4];{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[474];
f_4547(4,av2);}}

/* k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in ... */
static void C_ccall f_8468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(31,c,3)))){
C_save_and_reclaim((void *)f_8468,c,av);}
a=C_alloc(31);
t2=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_8471,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],tmp=(C_word)a,a+=25,tmp);
t3=(C_truep(((C_word*)t0)[3])?((C_word*)t0)[3]:((C_word*)t0)[10]);
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_8471(2,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8918,a[2]=((C_word*)t0)[6],a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_8918(t7,t2,C_fix(0));}}

/* k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in ... */
static void C_ccall f_8471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(90,c,5)))){
C_save_and_reclaim((void *)f_8471,c,av);}
a=C_alloc(90);
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_8474,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],tmp=(C_word)a,a+=24,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
/* c-backend.scm:874: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[468];
av2[4]=((C_word*)t0)[6];
av2[5]=C_make_character(59);
f_4547(6,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8850,a[2]=((C_word*)t0)[24],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8888,a[2]=t3,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[15])){
t5=C_s_a_i_minus(&a,2,((C_word*)t0)[6],C_fix(1));
t6=t4;
f_8888(t6,C_s_a_i_plus(&a,2,((C_word*)t0)[11],t5));}
else{
t5=t4;
f_8888(t5,C_s_a_i_plus(&a,2,((C_word*)t0)[11],C_fix(0)));}}}

/* k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in ... */
static void C_ccall f_8474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,4)))){
C_save_and_reclaim((void *)f_8474,c,av);}
a=C_alloc(30);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_8477,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],tmp=(C_word)a,a+=16,tmp);
t3=C_eqp(lf[326],((C_word*)t0)[9]);
if(C_truep(t3)){
t4=C_i_check_list_2(((C_word*)t0)[16],lf[360]);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8617,a[2]=((C_word*)t0)[16],a[3]=t2,a[4]=((C_word*)t0)[17],a[5]=((C_word*)t0)[12],a[6]=((C_word*)t0)[18],a[7]=((C_word*)t0)[19],tmp=(C_word)a,a+=8,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8680,a[2]=((C_word*)t0)[20],a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_8680(t9,t5,((C_word*)t0)[16],C_fix(0));}
else{
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8717,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[18],a[6]=((C_word*)t0)[21],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:915: gen */
t5=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[453];
f_4547(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8756,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[12],a[6]=((C_word*)t0)[18],a[7]=((C_word*)t0)[22],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[23],a[10]=((C_word*)t0)[15],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[10])){
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_8756(2,av2);}}
else{
/* c-backend.scm:921: gen */
t5=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[467];
f_4547(4,av2);}}}}}

/* k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_ccall f_8477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_8477,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8480,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=C_eqp(lf[326],((C_word*)t0)[9]);
t4=C_i_not(t3);
t5=(C_truep(t4)?C_i_not(((C_word*)t0)[10]):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8502,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t2,a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[14],tmp=(C_word)a,a+=11,tmp);
t7=(C_truep(((C_word*)t0)[15])?C_i_not(((C_word*)t0)[3]):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8577,a[2]=((C_word*)t0)[6],a[3]=t9,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_8577(t11,t6,C_fix(0));}
else{
t8=t6;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_8502(2,av2);}}}
else{
/* c-backend.scm:963: gen */
t6=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t2;
av2[2]=C_make_character(125);
f_4547(3,av2);}}}

/* k8478 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_8480,c,av);}
a=C_alloc(10);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8484,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8491,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=t3,a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:966: chicken.compiler.core#lambda-literal-body */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[389]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[389]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k8482 in k8478 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8484,c,av);}
/* c-backend.scm:971: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(125);
f_4547(3,av2);}}

/* k8489 in k8478 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_8491,c,av);}
a=C_alloc(29);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:965: expression */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4720(t3,((C_word*)t0)[5],t1,t2,((C_word*)t0)[6]);}
else{
/* c-backend.scm:965: expression */
t2=((C_word*)((C_word*)t0)[4])[1];
f_4720(t2,((C_word*)t0)[5],t1,((C_word*)t0)[3],((C_word*)t0)[6]);}}

/* k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,11)))){
C_save_and_reclaim((void *)f_8502,c,av);}
a=C_alloc(8);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8508,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:947: gen */
t3=lf[4];{
C_word *av2;
if(c >= 12) {
  av2=av;
} else {
  av2=C_alloc(12);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[394];
av2[4]=((C_word*)t0)[6];
av2[5]=lf[395];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[396];
av2[8]=((C_word*)t0)[3];
av2[9]=lf[397];
av2[10]=((C_word*)t0)[7];
av2[11]=lf[398];
f_4547(12,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8542,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t3=(C_truep(((C_word*)t0)[8])?C_i_greaterp(((C_word*)t0)[9],C_fix(0)):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8557,a[2]=t2,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:956: gen */
t5=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[402];
av2[4]=((C_word*)t0)[6];
av2[5]=C_make_character(44);
av2[6]=((C_word*)t0)[9];
av2[7]=C_make_character(44);
f_4547(8,av2);}}
else{
/* c-backend.scm:960: gen */
t4=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[403];
av2[4]=((C_word*)t0)[6];
av2[5]=lf[404];
f_4547(6,av2);}}}}

/* k8506 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,7)))){
C_save_and_reclaim((void *)f_8508,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8511,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:949: gen */
t3=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[391];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[392];
av2[6]=((C_word*)t0)[2];
av2[7]=lf[393];
f_4547(8,av2);}}

/* k8509 in k8506 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in ... */
static void C_ccall f_8511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,4)))){
C_save_and_reclaim((void *)f_8511,c,av);}
a=C_alloc(34);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8520,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_8520(t6,((C_word*)t0)[3],t2,((C_word*)t0)[4]);}

/* doloop1699 in k8509 in k8506 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in ... */
static void C_fcall f_8520(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_8520,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_zerop(t3))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8528,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:953: gen */
t5=lf[4];{
C_word av2[6];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[390];
av2[4]=t2;
av2[5]=C_make_character(59);
f_4547(6,av2);}}}

/* k8526 in doloop1699 in k8509 in k8506 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in ... */
static void C_ccall f_8528(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,3)))){
C_save_and_reclaim((void *)f_8528,c,av);}
a=C_alloc(58);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=((C_word*)((C_word*)t0)[4])[1];
f_8520(t4,((C_word*)t0)[5],t2,t3);}

/* k8540 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8542(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8542,c,av);}
if(C_truep(C_i_greaterp(((C_word*)t0)[2],C_fix(0)))){
/* c-backend.scm:962: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[399];
av2[4]=((C_word*)t0)[2];
av2[5]=lf[400];
f_4547(6,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_8480(2,av2);}}}

/* k8555 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8557,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8560,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=lf[4];
av2[3]=((C_word*)t0)[3];
C_apply(4,av2);}}

/* k8558 in k8555 in k8500 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in ... */
static void C_ccall f_8560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8560,c,av);}
/* c-backend.scm:958: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[401];
f_4547(3,av2);}}

/* doloop1691 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_fcall f_8577(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,7)))){
C_save_and_reclaim_args((void *)trf_8577,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8587,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:945: gen */
t4=lf[4];{
C_word av2[8];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[405];
av2[4]=t2;
av2[5]=lf[406];
av2[6]=t2;
av2[7]=lf[407];
f_4547(8,av2);}}}

/* k8585 in doloop1691 in k8475 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_8587,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_8577(t3,((C_word*)t0)[4],t2);}

/* k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_ccall f_8617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,9)))){
C_save_and_reclaim((void *)f_8617,c,av);}
a=C_alloc(9);
t2=C_u_i_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8621,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
t4=C_fast_retrieve(lf[295]);
if(C_truep(C_fast_retrieve(lf[295]))){
t5=C_fast_retrieve(lf[295]);
t6=C_fast_retrieve(lf[295]);
/* c-backend.scm:887: gen */
t7=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t7;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[439];
av2[4]=C_SCHEME_TRUE;
av2[5]=lf[440];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[441];
av2[8]=C_fast_retrieve(lf[295]);
av2[9]=lf[442];
f_4547(10,av2);}}
else{
/* c-backend.scm:887: gen */
t5=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[439];
av2[4]=C_SCHEME_TRUE;
av2[5]=lf[440];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[441];
av2[8]=((C_word*)t0)[7];
av2[9]=lf[442];
f_4547(10,av2);}}}

/* k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_8621,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8624,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_fast_retrieve(lf[185]))){
/* c-backend.scm:891: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[438];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8624(2,av2);}}}

/* k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_8624,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8627,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_fast_retrieve(lf[436]))){
/* c-backend.scm:893: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[437];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8627(2,av2);}}}

/* k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in ... */
static void C_ccall f_8627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,7)))){
C_save_and_reclaim((void *)f_8627,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8630,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_fast_retrieve(lf[295]))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8630(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8659,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_fast_retrieve(lf[432]))){
/* c-backend.scm:896: gen */
t4=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[433];
av2[4]=C_fast_retrieve(lf[432]);
av2[5]=lf[434];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[435];
f_4547(8,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_8659(2,av2);}}}}

/* k8628 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in ... */
static void C_ccall f_8630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,35)))){
C_save_and_reclaim((void *)f_8630,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8633,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:900: gen */
t3=lf[4];{
C_word *av2;
if(c >= 36) {
  av2=av;
} else {
  av2=C_alloc(36);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[412];
av2[4]=((C_word*)t0)[5];
av2[5]=lf[413];
av2[6]=((C_word*)t0)[6];
av2[7]=lf[414];
av2[8]=C_SCHEME_TRUE;
av2[9]=lf[415];
av2[10]=((C_word*)t0)[5];
av2[11]=lf[416];
av2[12]=((C_word*)t0)[6];
av2[13]=lf[417];
av2[14]=C_SCHEME_TRUE;
av2[15]=lf[418];
av2[16]=((C_word*)t0)[7];
av2[17]=lf[419];
av2[18]=C_SCHEME_TRUE;
av2[19]=lf[420];
av2[20]=C_SCHEME_TRUE;
av2[21]=lf[421];
av2[22]=((C_word*)t0)[8];
av2[23]=lf[422];
av2[24]=C_SCHEME_TRUE;
av2[25]=lf[423];
av2[26]=C_SCHEME_TRUE;
av2[27]=lf[424];
av2[28]=((C_word*)t0)[8];
av2[29]=lf[425];
av2[30]=C_SCHEME_TRUE;
av2[31]=lf[426];
av2[32]=C_SCHEME_TRUE;
av2[33]=lf[427];
av2[34]=((C_word*)t0)[5];
av2[35]=lf[428];
f_4547(36,av2);}}

/* k8631 in k8628 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in ... */
static void C_ccall f_8633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_8633,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8636,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_eqp(((C_word*)t0)[3],C_fix(0));
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8645,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:910: gen */
t5=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[410];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[411];
f_4547(6,av2);}}
else{
/* c-backend.scm:913: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(123);
f_4547(3,av2);}}}

/* k8634 in k8631 in k8628 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in ... */
static void C_ccall f_8636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8636,c,av);}
/* c-backend.scm:913: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(123);
f_4547(3,av2);}}

/* k8643 in k8631 in k8628 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in ... */
static void C_ccall f_8645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8645,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8648,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:911: literal-frame */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7794(t3,t2);}

/* k8646 in k8643 in k8631 in k8628 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in ... */
static void C_ccall f_8648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8648,c,av);}
/* c-backend.scm:912: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[408];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[409];
f_4547(6,av2);}}

/* k8657 in k8625 in k8622 in k8619 in k8615 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in ... */
static void C_ccall f_8659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8659,c,av);}
if(C_truep(C_fast_retrieve(lf[429]))){
/* c-backend.scm:899: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[430];
av2[4]=C_fast_retrieve(lf[429]);
av2[5]=lf[431];
f_4547(6,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_8630(2,av2);}}}

/* foldl1644 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_fcall f_8680(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_8680,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8696,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8707,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:885: g1651 */
t7=t5;
f_8696(t7,t6,t3,C_slot(t2,C_fix(0)));}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* g1651 in foldl1644 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_fcall f_8696(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_8696,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8704,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:885: literal-size */
t5=((C_word*)((C_word*)t0)[2])[1];
f_7848(t5,t4,t3);}

/* k8702 in g1651 in foldl1644 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_8704,c,av);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8705 in foldl1644 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8707,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8680(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k8715 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_ccall f_8717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_8717,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8720,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8732,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_not(C_fast_retrieve(lf[175])))){
if(C_truep(C_i_not(C_fast_retrieve(lf[452])))){
t4=C_i_greaterp(((C_word*)t0)[3],C_fix(2));
t5=t3;
f_8732(t5,(C_truep(t4)?C_i_not(((C_word*)t0)[6]):C_SCHEME_FALSE));}
else{
t4=t3;
f_8732(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_8732(t4,C_SCHEME_FALSE);}}

/* k8718 in k8715 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,9)))){
C_save_and_reclaim((void *)f_8720,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8723,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fast_retrieve(lf[447]))){
/* c-backend.scm:918: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[448];
f_4547(4,av2);}}
else{
/* c-backend.scm:919: gen */
t3=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[443];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[444];
av2[6]=((C_word*)t0)[4];
av2[7]=lf[445];
av2[8]=((C_word*)t0)[5];
av2[9]=lf[446];
f_4547(10,av2);}}}

/* k8721 in k8718 in k8715 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_8723,c,av);}
/* c-backend.scm:919: gen */
t2=lf[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[443];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[444];
av2[6]=((C_word*)t0)[4];
av2[7]=lf[445];
av2[8]=((C_word*)t0)[5];
av2[9]=lf[446];
f_4547(10,av2);}}

/* k8730 in k8715 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_fcall f_8732(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,7)))){
C_save_and_reclaim_args((void *)trf_8732,2,t0,t1);}
if(C_truep(t1)){
/* c-backend.scm:917: gen */
t2=lf[4];{
C_word av2[8];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[449];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[450];
av2[6]=((C_word*)t0)[3];
av2[7]=lf[451];
f_4547(8,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_8720(2,av2);}}}

/* k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_ccall f_8756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_8756,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8759,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[2])){
if(C_truep(C_i_not(C_fast_retrieve(lf[175])))){
if(C_truep(C_i_not(C_fast_retrieve(lf[436])))){
/* c-backend.scm:923: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[466];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8759(2,av2);}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8759(2,av2);}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8759(2,av2);}}}

/* k8757 in k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8759(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_8759,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8762,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[10])){
/* c-backend.scm:924: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[465];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8762(2,av2);}}}

/* k8760 in k8757 in k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in ... */
static void C_ccall f_8762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_8762,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8765,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8790,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[9])){
if(C_truep(C_i_not(C_fast_retrieve(lf[175])))){
t4=C_i_not(C_fast_retrieve(lf[452]));
t5=t3;
f_8790(t5,(C_truep(t4)?C_i_not(((C_word*)t0)[3]):C_SCHEME_FALSE));}
else{
t4=t3;
f_8790(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_8790(t4,C_SCHEME_FALSE);}}

/* k8763 in k8760 in k8757 in k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in ... */
static void C_ccall f_8765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_8765,c,av);}
a=C_alloc(6);
if(C_truep(C_i_not(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8774,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fast_retrieve(lf[447]))){
/* c-backend.scm:933: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[458];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8774(2,av2);}}}
else{
/* c-backend.scm:939: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=C_make_character(123);
f_4547(3,av2);}}}

/* k8772 in k8763 in k8760 in k8757 in k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in ... */
static void C_ccall f_8774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_8774,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* c-backend.scm:934: gen */
t2=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[454];
av2[4]=((C_word*)t0)[4];
av2[5]=lf[455];
av2[6]=((C_word*)t0)[5];
av2[7]=lf[456];
f_4547(8,av2);}}
else{
/* c-backend.scm:934: gen */
t2=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[454];
av2[4]=((C_word*)t0)[4];
av2[5]=lf[457];
av2[6]=((C_word*)t0)[5];
av2[7]=lf[456];
f_4547(8,av2);}}}

/* k8788 in k8760 in k8757 in k8754 in k8472 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in ... */
static void C_fcall f_8790(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,7)))){
C_save_and_reclaim_args((void *)trf_8790,2,t0,t1);}
if(C_truep(t1)){
t2=C_eqp(((C_word*)t0)[2],lf[283]);
if(C_truep(t2)){
if(C_truep(C_i_greaterp(((C_word*)t0)[3],C_fix(2)))){
/* c-backend.scm:928: gen */
t3=lf[4];{
C_word av2[8];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[459];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[460];
av2[6]=((C_word*)t0)[3];
av2[7]=lf[461];
f_4547(8,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8765(2,av2);}}}
else{
/* c-backend.scm:929: gen */
t3=lf[4];{
C_word av2[8];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[462];
av2[4]=((C_word*)t0)[3];
av2[5]=lf[463];
av2[6]=((C_word*)t0)[3];
av2[7]=lf[464];
f_4547(8,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_8765(2,av2);}}}

/* k8848 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in ... */
static void C_ccall f_8850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8850,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8863,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_8863(t6,((C_word*)t0)[3],((C_word*)t0)[2]);}

/* for-each-loop1620 in k8848 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_fcall f_8863(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_8863,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8873,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:882: gen */
t4=lf[4];{
C_word av2[6];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[469];
av2[4]=C_slot(t2,C_fix(0));
av2[5]=C_make_character(59);
f_4547(6,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k8871 in for-each-loop1620 in k8848 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8873,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8863(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k8886 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in ... */
static void C_fcall f_8888(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_8888,2,t0,t1);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8890,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_8890(t5,((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* doloop1612 in k8886 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in ... */
static void C_fcall f_8890(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_8890,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_zerop(t3))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8898,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:879: gen */
t5=lf[4];{
C_word av2[6];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[470];
av2[4]=t2;
av2[5]=C_make_character(59);
f_4547(6,av2);}}}

/* k8896 in doloop1612 in k8886 in k8469 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in ... */
static void C_ccall f_8898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,3)))){
C_save_and_reclaim((void *)f_8898,c,av);}
a=C_alloc(58);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=((C_word*)((C_word*)t0)[4])[1];
f_8890(t4,((C_word*)t0)[5],t2,t3);}

/* doloop1607 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in ... */
static void C_fcall f_8918(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,7)))){
C_save_and_reclaim_args((void *)trf_8918,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8928,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:872: gen */
t4=lf[4];{
C_word av2[8];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[471];
av2[4]=t2;
av2[5]=lf[472];
av2[6]=t2;
av2[7]=lf[473];
f_4547(8,av2);}}}

/* k8926 in doloop1607 in k8466 in k8463 in k8460 in k8457 in k8454 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in ... */
static void C_ccall f_8928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_8928,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_8918(t3,((C_word*)t0)[4],t2);}

/* k8950 in k8451 in k8448 in k8445 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in ... */
static void C_ccall f_8952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8952,c,av);}
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
/* c-backend.scm:862: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(44);
f_4547(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_8456(2,av2);}}}

/* k8973 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in ... */
static void C_ccall f_8975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8975,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8978,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
/* c-backend.scm:848: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[481];
f_4547(3,av2);}}
else{
/* c-backend.scm:848: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[482];
f_4547(3,av2);}}}

/* k8976 in k8973 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in ... */
static void C_ccall f_8978(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8978,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8981,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:850: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[479];
f_4547(3,av2);}}
else{
/* c-backend.scm:851: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[480];
f_4547(3,av2);}}}

/* k8979 in k8976 in k8973 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in ... */
static void C_ccall f_8981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8981,c,av);}
/* c-backend.scm:852: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
f_4547(3,av2);}}

/* k8995 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in ... */
static void C_ccall f_8997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_8997,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9000,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fast_retrieve(lf[295]))){
/* c-backend.scm:857: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[484];
av2[5]=((C_word*)t0)[3];
f_4547(6,av2);}}
else{
/* c-backend.scm:856: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[485];
f_4547(4,av2);}}}

/* k8998 in k8995 in k8442 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in ... */
static void C_ccall f_9000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9000,c,av);}
/* c-backend.scm:857: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[484];
av2[5]=((C_word*)t0)[3];
f_4547(6,av2);}}

/* k9012 in k8439 in k8436 in k8433 in k8430 in k8427 in k8424 in k8421 in k8418 in k8415 in k8412 in k8409 in k8406 in k8403 in k8397 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in ... */
static void C_ccall f_9014(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9014,c,av);}
/* c-backend.scm:845: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[487];
av2[3]=t1;
av2[4]=lf[488];
av2[5]=C_SCHEME_TRUE;
f_4547(6,av2);}}

/* k9037 in k8394 in k8391 in k8388 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9039(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9039,c,av);}
t2=((C_word*)t0)[2];
f_8399(t2,C_i_zerop(t1));}

/* k9041 in k8385 in k8382 in k8379 in g1550 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9043,c,av);}{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[498]+1);
av2[3]=C_fix(0);
av2[4]=t1;
C_apply(5,av2);}}

/* for-each-loop1549 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9051(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9051,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9061,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:820: g1550 */
t4=((C_word*)t0)[3];
f_8373(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9059 in for-each-loop1549 in procedures in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9061(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9061,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9051(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_9075,c,av);}
a=C_alloc(11);
t2=C_mutate(&lf[3] /* (set! chicken.compiler.c-backend#output ...) */,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9079,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:976: header */
t4=((C_word*)((C_word*)t0)[12])[1];
f_7132(t4,t3);}

/* k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_9079,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9082,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* c-backend.scm:977: declarations */
t3=((C_word*)((C_word*)t0)[10])[1];
f_7291(t3,t2);}

/* k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_9082,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9085,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t3=C_fast_retrieve(lf[658]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9446,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1076: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}

/* k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_9085,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9088,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t3=C_fast_retrieve(lf[612]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9544,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
t5=C_fast_retrieve(lf[612]);
t6=C_i_check_list_2(C_fast_retrieve(lf[612]),lf[91]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9857,a[2]=t8,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_9857(t10,t2,C_fast_retrieve(lf[612]));}

/* k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_9088,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9091,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:980: prototypes */
t3=((C_word*)((C_word*)t0)[9])[1];
f_7495(t3,t2);}

/* k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9091(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_9091,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9094,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=C_fast_retrieve(lf[287]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9882,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
t5=C_fast_retrieve(lf[287]);
t6=C_i_check_list_2(C_fast_retrieve(lf[287]),lf[91]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10511,a[2]=t8,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_10511(t10,t2,C_fast_retrieve(lf[287]));}

/* k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_9094,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9097,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:982: trampolines */
t3=((C_word*)((C_word*)t0)[7])[1];
f_7673(t3,t2);}

/* k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_9097,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9100,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fast_retrieve(lf[185]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9121,a[2]=t2,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:993: gen */
t4=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[532];
f_4547(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_9100(2,av2);}}}

/* k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9100,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9103,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:985: procedures */
t3=((C_word*)((C_word*)t0)[5])[1];
f_8371(t3,t2);}

/* k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,8)))){
C_save_and_reclaim((void *)f_9103,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9106,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9242,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_i_length(((C_word*)t0)[4]);
t5=C_a_i_fixnum_plus(&a,2,t4,C_fix(1));
/* c-backend.scm:1011: gen */
t6=lf[4];{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[521];
av2[5]=C_SCHEME_TRUE;
av2[6]=lf[522];
av2[7]=t5;
av2[8]=lf[523];
f_4547(9,av2);}}

/* k9104 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_9106(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9106,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7285,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7289,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:629: chicken.base#get-output-string */
t4=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fast_retrieve(lf[504]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9121,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9190,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9224,tmp=(C_word)a,a+=2,tmp);
/* c-backend.scm:1004: chicken.sort#sort */
t4=C_fast_retrieve(lf[531]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_9126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_9126,c,av);}
a=C_alloc(8);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[91]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9157,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9162,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t8=((C_word*)t6)[1];
f_9162(t8,t4,t2);}

/* k9142 in for-each-loop1782 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9144,c,av);}
/* c-backend.scm:1001: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[527];
av2[3]=t1;
av2[4]=lf[528];
f_4547(5,av2);}}

/* k9146 in for-each-loop1782 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9148(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9148,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f14881,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:74: chicken.string#->string */
t3=C_fast_retrieve(lf[118]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k9155 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9157,c,av);}
/* c-backend.scm:1003: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[525];
f_4547(3,av2);}}

/* for-each-loop1782 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_fcall f_9162(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_9162,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9172,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
if(C_truep(C_i_not(t4))){
/* c-backend.scm:1000: gen */
t5=lf[4];{
C_word av2[3];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[526];
f_4547(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9144,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9148,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1001: chicken.string#->string */
t7=C_fast_retrieve(lf[118]);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9170 in for-each-loop1782 in k9124 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9172,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9162(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_9190,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[91]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9196,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9201,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_9201(t7,t3,t1);}

/* k9194 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_9196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9196,c,av);}
/* c-backend.scm:1005: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[524];
f_4547(4,av2);}}

/* for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_fcall f_9201(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_9201,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9211,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9126,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:996: gen */
t6=lf[4];{
C_word av2[6];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[529];
av2[4]=C_i_cadr(t4);
av2[5]=lf[530];
f_4547(6,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9209 in for-each-loop1769 in k9188 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_9211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9211,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9201(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a9223 in k9119 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9224(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9224,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_lessp(C_i_car(t2),C_i_car(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_9242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_9242,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9243,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9274,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9288,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_9288(t7,t3,((C_word*)t0)[4]);}

/* g1815 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_fcall f_9243(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_9243,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9251,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9271,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1017: chicken.internal#string->c-identifier */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[510]);
C_word av2[3];
av2[0]=*((C_word*)lf[510]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k9249 in g1815 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9251,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[326],((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9264,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1019: toplevel */
t4=lf[222];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fast_retrieve(lf[295]);
f_9316(3,av2);}}
else{
/* c-backend.scm:1020: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[2];
av2[3]=lf[507];
f_4547(4,av2);}}}

/* k9262 in k9249 in g1815 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9264,c,av);}
/* c-backend.scm:1019: gen */
t2=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[505];
av2[3]=t1;
av2[4]=lf[506];
f_4547(5,av2);}}

/* k9269 in g1815 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_9271,c,av);}
/* c-backend.scm:1017: gen */
t2=lf[4];{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[508];
av2[4]=((C_word*)t0)[3];
av2[5]=C_make_character(58);
av2[6]=t1;
av2[7]=lf[509];
f_4547(8,av2);}}

/* k9272 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_9274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9274,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9277,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1022: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[520];
f_4547(4,av2);}}

/* k9275 in k9272 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9277,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9280,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1023: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[519];
f_4547(4,av2);}}

/* k9278 in k9275 in k9272 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_9280,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9283,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1024: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[518];
f_4547(5,av2);}}

/* k9281 in k9278 in k9275 in k9272 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_9283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,14)))){
C_save_and_reclaim((void *)f_9283,c,av);}
/* c-backend.scm:1025: gen */
t2=lf[4];{
C_word *av2;
if(c >= 15) {
  av2=av;
} else {
  av2=C_alloc(15);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[511];
av2[3]=C_SCHEME_TRUE;
av2[4]=lf[512];
av2[5]=C_SCHEME_TRUE;
av2[6]=lf[513];
av2[7]=C_SCHEME_TRUE;
av2[8]=lf[514];
av2[9]=C_SCHEME_TRUE;
av2[10]=lf[515];
av2[11]=C_SCHEME_TRUE;
av2[12]=lf[516];
av2[13]=C_SCHEME_TRUE;
av2[14]=lf[517];
f_4547(15,av2);}}

/* for-each-loop1814 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_fcall f_9288(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9288,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9298,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1013: g1815 */
t4=((C_word*)t0)[3];
f_9243(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9296 in for-each-loop1814 in k9240 in k9101 in k9098 in k9095 in k9092 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9298,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9288(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.c-backend#toplevel in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9316,c,av);}
a=C_alloc(6);
if(C_truep(C_i_not(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[668];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9330,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4655,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:76: chicken.string#->string */
t5=C_fast_retrieve(lf[118]);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k9328 in chicken.compiler.c-backend#toplevel in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9330(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9330,c,av);}
/* c-backend.scm:1038: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[669];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.c-backend#cleanup in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9332(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_9332,2,t1,t2);}
a=C_alloc(10);
t3=C_SCHEME_FALSE;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_i_string_length(t2);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9341,a[2]=t5,a[3]=t4,a[4]=t2,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_9341(t9,t1,C_fix(0));}

/* loop in chicken.compiler.c-backend#cleanup in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9341(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_9341,3,t0,t1,t2);}
a=C_alloc(18);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t3=((C_word*)((C_word*)t0)[3])[1];
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=(C_truep(t3)?t3:((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_string_ref(((C_word*)t0)[4],t2);
t4=C_u_i_char_lessp(t3,C_make_character(32));
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9367,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
if(C_truep(t4)){
t6=t5;
f_9367(t6,t4);}
else{
t6=C_u_i_char_greaterp(t3,C_make_character(126));
if(C_truep(t6)){
t7=t5;
f_9367(t7,t6);}
else{
if(C_truep(C_u_i_char_equalp(t3,C_make_character(42)))){
t7=C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],C_fix(1));
if(C_truep(C_i_integer_lessp(t2,t7))){
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t9=t5;
f_9367(t9,C_u_i_char_equalp(C_make_character(47),C_i_string_ref(((C_word*)t0)[4],t8)));}
else{
t8=t5;
f_9367(t8,C_SCHEME_FALSE);}}
else{
t7=t5;
f_9367(t7,C_SCHEME_FALSE);}}}}}

/* k9365 in loop in chicken.compiler.c-backend#cleanup in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9367(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9367,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_i_string_set(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3],C_make_character(126));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:1057: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9341(t4,((C_word*)t0)[5],t3);}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9377,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1054: scheme#string-copy */
t3=C_fast_retrieve(lf[670]);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}
else{
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_i_string_set(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3],((C_word*)t0)[7]);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:1057: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9341(t4,((C_word*)t0)[5],t3);}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:1057: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_9341(t3,((C_word*)t0)[5],t2);}}}

/* k9375 in k9365 in loop in chicken.compiler.c-backend#cleanup in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9377,c,av);}
a=C_alloc(5);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_i_string_set(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3],C_make_character(126));
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* c-backend.scm:1057: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_9341(t5,((C_word*)t0)[5],t4);}

/* chicken.compiler.c-backend#make-variable-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9408(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_9408,3,t1,t2,t3);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9414,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1063: list-tabulate */
f_4279(t1,t2,t4);}

/* a9413 in chicken.compiler.c-backend#make-variable-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9414,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9422,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=C_fix(10);
tp(4,av2);}}

/* k9420 in a9413 in chicken.compiler.c-backend#make-variable-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9422,c,av);}
/* c-backend.scm:1065: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[671];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.c-backend#make-argument-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9425(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_9425,3,t1,t2,t3);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9431,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1068: list-tabulate */
f_4279(t1,t2,t4);}

/* a9430 in chicken.compiler.c-backend#make-argument-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9431,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9439,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[144]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[144]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=C_fix(10);
tp(4,av2);}}

/* k9437 in a9430 in chicken.compiler.c-backend#make-argument-list in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9439,c,av);}
/* c-backend.scm:1070: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k9444 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9446,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9476,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_9476(t6,((C_word*)t0)[3],((C_word*)t0)[2]);}

/* k9466 in for-each-loop1873 in k9444 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9468,c,av);}
/* c-backend.scm:1082: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
av2[5]=C_make_character(59);
f_4547(6,av2);}}

/* for-each-loop1873 in k9444 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9476(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_9476,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9486,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_vector_ref(t4,C_fix(0));
t6=C_i_vector_ref(t4,C_fix(1));
t7=C_i_vector_ref(t4,C_fix(2));
t8=(C_truep(t7)?lf[659]:lf[660]);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9468,a[2]=t3,a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1082: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t9;
av2[2]=t6;
av2[3]=t5;
tp(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9484 in for-each-loop1873 in k9444 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9486(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9486,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9476(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9499(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9499,2,t1,t2);}
a=C_alloc(5);
t3=C_i_check_list_2(t2,lf[91]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9519,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_9519(t7,t1,t2);}

/* k9503 in for-each-loop1897 in chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9505,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9508,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1092: generate-foreign-callback-header */
f_10534(t2,lf[672],((C_word*)t0)[3]);}

/* k9506 in k9503 in for-each-loop1897 in chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9508,c,av);}
/* c-backend.scm:1093: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(59);
f_4547(3,av2);}}

/* for-each-loop1897 in chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9519(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_9519,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9529,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9505,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1091: gen */
t6=lf[4];{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9527 in for-each-loop1897 in chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9529,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9519(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9544(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9544,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9548,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1099: chicken.compiler.core#foreign-stub-id */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[657]);
C_word av2[3];
av2[0]=*((C_word*)lf[657]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9548,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9551,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1100: chicken.compiler.support#real-name2 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[610]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[610]+1);
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}

/* k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9551(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9551,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9554,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1101: chicken.compiler.core#foreign-stub-argument-types */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[656]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[656]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_9554,c,av);}
a=C_alloc(8);
t2=C_i_length(t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9560,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* c-backend.scm:1103: chicken.compiler.core#foreign-stub-return-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[655]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[655]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_9560,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9563,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:1104: chicken.compiler.core#foreign-stub-name */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[654]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[654]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}

/* k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_9563,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9566,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* c-backend.scm:1105: chicken.compiler.core#foreign-stub-body */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[653]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[653]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
tp(3,av2);}}

/* k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_9566(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_9566,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9569,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* c-backend.scm:1106: chicken.compiler.core#foreign-stub-argument-names */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[652]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[652]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}

/* k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_9569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_9569,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9572,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_9572(2,av2);}}
else{
t3=C_a_i_list(&a,1,C_SCHEME_FALSE);
if(C_truep(C_i_nullp(t3))){
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f14902,tmp=(C_word)a,a+=2,tmp);
/* mini-srfi-1.scm:228: list-tabulate */
f_4279(t2,((C_word*)t0)[5],t4);}
else{
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4471,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:228: list-tabulate */
f_4279(t2,((C_word*)t0)[5],t5);}}}

/* k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_ccall f_9572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_9572,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9575,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:1107: foreign-result-conversion */
f_12369(t2,((C_word*)t0)[4],lf[651]);}

/* k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_ccall f_9575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_9575,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9578,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* c-backend.scm:1108: chicken.compiler.core#foreign-stub-cps */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[650]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[650]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[11];
tp(3,av2);}}

/* k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_ccall f_9578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_9578,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9581,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* c-backend.scm:1109: chicken.compiler.core#foreign-stub-callback */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[649]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[649]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
tp(3,av2);}}

/* k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_ccall f_9581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_9581,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_9584,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* c-backend.scm:1110: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}

/* k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_ccall f_9584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_9584,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9587,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)t0)[13])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9846,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1112: cleanup */
f_9332(t3,((C_word*)t0)[13]);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_9587(2,av2);}}}

/* k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in ... */
static void C_ccall f_9587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_9587,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9590,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:1114: gen */
t3=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[645];
av2[4]=((C_word*)t0)[10];
av2[5]=lf[646];
f_4547(6,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_9590(2,av2);}}}

/* k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in ... */
static void C_ccall f_9590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,11)))){
C_save_and_reclaim((void *)f_9590,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9593,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[6])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9791,a[2]=((C_word*)t0)[8],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1117: gen */
t4=lf[4];{
C_word *av2;
if(c >= 12) {
  av2=av;
} else {
  av2=C_alloc(12);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[636];
av2[4]=((C_word*)t0)[12];
av2[5]=lf[637];
av2[6]=C_SCHEME_TRUE;
av2[7]=lf[638];
av2[8]=((C_word*)t0)[12];
av2[9]=lf[639];
av2[10]=C_SCHEME_TRUE;
av2[11]=lf[640];
f_4547(12,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9818,a[2]=t2,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1124: gen */
t4=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[644];
av2[4]=((C_word*)t0)[12];
av2[5]=C_make_character(40);
f_4547(6,av2);}}}

/* k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in ... */
static void C_ccall f_9593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_9593,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9596,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* c-backend.scm:1127: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[632];
f_4547(4,av2);}}

/* k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_ccall f_9596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_9596,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9638,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4459,tmp=(C_word)a,a+=2,tmp);
/* mini-srfi-1.scm:225: list-tabulate */
f_4279(t2,((C_word*)t0)[8],t3);}

/* k9603 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9605,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9609,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1134: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[630];
tp(4,av2);}}

/* k9607 in k9603 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in ... */
static void C_ccall f_9609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9609,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9613,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1135: foreign-argument-conversion */
f_11704(t2,((C_word*)t0)[5]);}

/* k9611 in k9607 in k9603 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in ... */
static void C_ccall f_9613(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,10)))){
C_save_and_reclaim((void *)f_9613,c,av);}
/* c-backend.scm:1130: gen */
t2=lf[4];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=lf[627];
av2[5]=((C_word*)t0)[4];
av2[6]=C_make_character(41);
av2[7]=t1;
av2[8]=lf[628];
av2[9]=((C_word*)t0)[5];
av2[10]=lf[629];
f_4547(11,av2);}}

/* k9615 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9617,c,av);}
/* c-backend.scm:1131: foreign-type-declaration */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[243]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[243]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k9621 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9623,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[353]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9629,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1133: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[355]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[355]+1);
av2[1]=t3;
av2[2]=C_make_character(116);
av2[3]=t1;
tp(4,av2);}}

/* k9627 in k9621 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in ... */
static void C_ccall f_9629(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_9629,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9632,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1133: ##sys#print */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9630 in k9627 in k9621 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in ... */
static void C_ccall f_9632(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9632,c,av);}
/* c-backend.scm:1133: chicken.base#get-output-string */
t2=C_fast_retrieve(lf[354]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_9638(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,5)))){
C_save_and_reclaim((void *)f_9638,c,av);}
a=C_alloc(15);
t2=C_i_check_list_2(t1,lf[91]);
t3=C_i_check_list_2(((C_word*)t0)[2],lf[91]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9647,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9739,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t8=((C_word*)t6)[1];
f_9739(t8,t4,((C_word*)t0)[11],t1,((C_word*)t0)[2]);}

/* k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in ... */
static void C_ccall f_9647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_9647,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9650,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:1137: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[626];
f_4547(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_9650(2,av2);}}}

/* k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in ... */
static void C_ccall f_9650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_9650,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9653,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9659,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1139: gen */
t4=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=((C_word*)t0)[3];
av2[4]=C_SCHEME_TRUE;
av2[5]=lf[618];
f_4547(6,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9680,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t4=C_eqp(((C_word*)t0)[6],lf[580]);
if(C_truep(C_i_not(t4))){
/* c-backend.scm:1149: gen */
t5=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[625];
av2[4]=((C_word*)t0)[9];
f_4547(5,av2);}}
else{
/* c-backend.scm:1150: gen */
t5=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
f_4547(3,av2);}}}}

/* k9651 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9653,c,av);}
/* c-backend.scm:1160: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(125);
f_4547(3,av2);}}

/* k9657 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9659,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9662,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1141: gen */
t3=lf[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[617];
av2[4]=C_SCHEME_TRUE;
f_4547(5,av2);}}

/* k9660 in k9657 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in ... */
static void C_ccall f_9662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9662,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* c-backend.scm:1143: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[613];
av2[4]=C_SCHEME_TRUE;
av2[5]=lf[614];
f_4547(6,av2);}}
else{
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:1145: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[615];
f_4547(4,av2);}}
else{
/* c-backend.scm:1146: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[616];
f_4547(4,av2);}}}}

/* k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_9680,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9683,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1151: gen */
t3=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=C_make_character(40);
f_4547(4,av2);}}

/* k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in ... */
static void C_ccall f_9683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_9683,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9686,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9717,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9721,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1152: make-argument-list */
f_9425(t4,((C_word*)t0)[6],lf[624]);}

/* k9684 in k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in ... */
static void C_ccall f_9686(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9686,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9689,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_eqp(((C_word*)t0)[5],lf[580]);
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_9689(2,av2);}}
else{
/* c-backend.scm:1153: gen */
t4=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_make_character(41);
f_4547(3,av2);}}}

/* k9687 in k9684 in k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in ... */
static void C_ccall f_9689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9689,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9692,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1154: gen */
t3=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[623];
f_4547(3,av2);}}

/* k9690 in k9687 in k9684 in k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in ... */
static void C_ccall f_9692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9692,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* c-backend.scm:1156: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[619];
av2[4]=C_SCHEME_TRUE;
av2[5]=lf[620];
f_4547(6,av2);}}
else{
if(C_truep(((C_word*)t0)[4])){
/* c-backend.scm:1158: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[621];
f_4547(3,av2);}}
else{
/* c-backend.scm:1159: gen */
t2=lf[4];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[622];
f_4547(4,av2);}}}}

/* k9715 in k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in ... */
static void C_ccall f_9717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9717,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[4];
av2[3]=t1;
C_apply(4,av2);}}

/* k9719 in k9681 in k9678 in k9648 in k9645 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in ... */
static void C_ccall f_9721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9721,c,av);}
/* c-backend.scm:1152: chicken.base#intersperse */
t2=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in ... */
static void C_fcall f_9739(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_9739,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9746,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(t2))){
t6=C_i_pairp(t3);
t7=t5;
f_9746(t7,(C_truep(t6)?C_i_pairp(t4):C_SCHEME_FALSE));}
else{
t6=t5;
f_9746(t6,C_SCHEME_FALSE);}}

/* k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in ... */
static void C_fcall f_9746(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,2)))){
C_save_and_reclaim_args((void *)trf_9746,2,t0,t1);}
a=C_alloc(20);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9749,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_slot(((C_word*)t0)[4],C_fix(0));
t4=C_slot(((C_word*)t0)[5],C_fix(0));
t5=C_slot(((C_word*)t0)[6],C_fix(0));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9605,a[2]=t2,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9617,a[2]=t6,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t5)){
/* c-backend.scm:1133: scheme#symbol->string */
t8=*((C_word*)lf[631]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9623,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* c-backend.scm:1133: chicken.base#open-output-string */
t9=C_fast_retrieve(lf[358]);{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9747 in k9744 in for-each-loop1950 in k9636 in k9594 in k9591 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in ... */
static void C_ccall f_9749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9749,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9739(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k9789 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in ... */
static void C_ccall f_9791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9791,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9796,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_9796(t5,((C_word*)t0)[3],C_fix(0));}

/* doloop1963 in k9789 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_fcall f_9796(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(34,0,7)))){
C_save_and_reclaim_args((void *)trf_9796,3,t0,t1,t2);}
a=C_alloc(34);
if(C_truep(C_i_greater_or_equalp(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9806,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=C_s_a_i_plus(&a,2,t2,C_fix(3));
/* c-backend.scm:1122: gen */
t5=lf[4];{
C_word av2[8];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[633];
av2[4]=t2;
av2[5]=lf[634];
av2[6]=t4;
av2[7]=lf[635];
f_4547(8,av2);}}}

/* k9804 in doloop1963 in k9789 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in ... */
static void C_ccall f_9806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_9806,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=((C_word*)((C_word*)t0)[3])[1];
f_9796(t3,((C_word*)t0)[4],t2);}

/* k9816 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in ... */
static void C_ccall f_9818(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9818,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9821,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9828,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9836,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* c-backend.scm:1125: make-variable-list */
f_9408(t4,((C_word*)t0)[3],lf[643]);}

/* k9819 in k9816 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_ccall f_9821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9821,c,av);}
/* c-backend.scm:1126: gen */
t2=lf[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[641];
f_4547(3,av2);}}

/* k9826 in k9816 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_ccall f_9828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9828,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[4];
av2[3]=t1;
C_apply(4,av2);}}

/* k9834 in k9816 in k9588 in k9585 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_ccall f_9836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9836,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[642],t1);
/* c-backend.scm:1125: chicken.base#intersperse */
t3=C_fast_retrieve(lf[8]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=C_make_character(44);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k9844 in k9582 in k9579 in k9576 in k9573 in k9570 in k9567 in k9564 in k9561 in k9558 in k9552 in k9549 in k9546 in g1921 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in ... */
static void C_ccall f_9846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9846,c,av);}
/* c-backend.scm:1112: gen */
t2=lf[4];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_TRUE;
av2[3]=lf[647];
av2[4]=t1;
av2[5]=lf[648];
f_4547(6,av2);}}

/* for-each-loop1920 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9857(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9857,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9867,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1097: g1921 */
t4=((C_word*)t0)[3];
f_9544(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9865 in for-each-loop1920 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9867,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9857(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall f_9882(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9882,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9886,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1166: chicken.compiler.support#foreign-callback-stub-id */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[611]);
C_word av2[3];
av2[0]=*((C_word*)lf[611]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9886,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9889,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1167: chicken.compiler.support#real-name2 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[610]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[610]+1);
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}

/* k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9889(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9889,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9892,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* c-backend.scm:1168: chicken.compiler.support#foreign-callback-stub-return-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[609]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[609]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_ccall f_9892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_9892,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9895,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* c-backend.scm:1169: chicken.compiler.support#foreign-callback-stub-argument-types */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[608]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[608]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 in ... */
static void C_ccall f_9895(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9895,c,av);}
a=C_alloc(9);
t2=C_i_length(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9901,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t1,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* c-backend.scm:1171: make-argument-list */
f_9425(t3,t2,lf[607]);}

/* k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in ... */
static void C_ccall f_9901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,5)))){
C_save_and_reclaim((void *)f_9901,c,av);}
a=C_alloc(21);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9903,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10363,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10477,a[2]=t7,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_10477(t9,t5,((C_word*)t0)[6],t1,lf[606]);}

/* compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in ... */
static void C_fcall f_9903(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_9903,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_eqp(t2,lf[17]);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9913,a[2]=t1,a[3]=t4,a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t5)){
t7=t6;
f_9913(t7,t5);}
else{
t7=C_eqp(t2,lf[577]);
if(C_truep(t7)){
t8=t6;
f_9913(t8,t7);}
else{
t8=C_eqp(t2,lf[578]);
if(C_truep(t8)){
t9=t6;
f_9913(t9,t8);}
else{
t9=C_eqp(t2,lf[579]);
if(C_truep(t9)){
t10=t6;
f_9913(t10,t9);}
else{
t10=C_eqp(t2,lf[14]);
if(C_truep(t10)){
t11=t6;
f_9913(t11,t10);}
else{
t11=C_eqp(t2,lf[580]);
if(C_truep(t11)){
t12=t6;
f_9913(t12,t11);}
else{
t12=C_eqp(t2,lf[581]);
if(C_truep(t12)){
t13=t6;
f_9913(t13,t12);}
else{
t13=C_eqp(t2,lf[582]);
if(C_truep(t13)){
t14=t6;
f_9913(t14,t13);}
else{
t14=C_eqp(t2,lf[583]);
if(C_truep(t14)){
t15=t6;
f_9913(t15,t14);}
else{
t15=C_eqp(t2,lf[584]);
if(C_truep(t15)){
t16=t6;
f_9913(t16,t15);}
else{
t16=C_eqp(t2,lf[585]);
if(C_truep(t16)){
t17=t6;
f_9913(t17,t16);}
else{
t17=C_eqp(t2,lf[586]);
t18=t6;
f_9913(t18,(C_truep(t17)?t17:C_eqp(t2,lf[587])));}}}}}}}}}}}}

/* k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_fcall f_9913(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_9913,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[533]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9922,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_9922(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[574]);
if(C_truep(t4)){
t5=t3;
f_9922(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[550]);
if(C_truep(t5)){
t6=t3;
f_9922(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[552]);
if(C_truep(t6)){
t7=t3;
f_9922(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[575]);
t8=t3;
f_9922(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[4],lf[576])));}}}}}}

/* k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in ... */
static void C_fcall f_9922(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9922,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1180: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[534];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[535]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9934,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_9934(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[568]);
if(C_truep(t4)){
t5=t3;
f_9934(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[569]);
if(C_truep(t5)){
t6=t3;
f_9934(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[570]);
if(C_truep(t6)){
t7=t3;
f_9934(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[571]);
if(C_truep(t7)){
t8=t3;
f_9934(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[4],lf[572]);
t9=t3;
f_9934(t9,(C_truep(t8)?t8:C_eqp(((C_word*)t0)[4],lf[573])));}}}}}}}

/* k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in ... */
static void C_fcall f_9934(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9934,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1183: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[536];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[537]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9946,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_9946(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[565]);
if(C_truep(t4)){
t5=t3;
f_9946(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[566]);
t6=t3;
f_9946(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[4],lf[567])));}}}}

/* k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in ... */
static void C_fcall f_9946(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9946,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1186: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[538];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[539]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9958,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_9958(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[562]);
if(C_truep(t4)){
t5=t3;
f_9958(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[563]);
if(C_truep(t5)){
t6=t3;
f_9958(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[563]);
t7=t3;
f_9958(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[4],lf[564])));}}}}}

/* k9956 in k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in ... */
static void C_fcall f_9958(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,7)))){
C_save_and_reclaim_args((void *)trf_9958,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1188: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[8];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[540];
av2[4]=((C_word*)t0)[4];
av2[5]=lf[541];
av2[6]=((C_word*)t0)[4];
av2[7]=lf[542];
((C_proc)(void*)(*((C_word*)t2+1)))(8,av2);}}
else{
t2=C_eqp(((C_word*)t0)[5],lf[543]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9970,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_9970(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[5],lf[558]);
if(C_truep(t4)){
t5=t3;
f_9970(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[5],lf[559]);
if(C_truep(t5)){
t6=t3;
f_9970(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[5],lf[560]);
t7=t3;
f_9970(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[5],lf[561])));}}}}}

/* k9968 in k9956 in k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in chicken.compiler.c-backend#generate-code in ... */
static void C_fcall f_9970(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,5)))){
C_save_and_reclaim_args((void *)trf_9970,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* c-backend.scm:1190: scheme#string-append */
t2=*((C_word*)lf[152]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[544];
av2[4]=((C_word*)t0)[4];
av2[5]=lf[545];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9976,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[6]))){
/* c-backend.scm:1192: chicken.compiler.support#lookup-foreign-type */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[557]);
C_word av2[3];
av2[0]=*((C_word*)lf[557]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_9976(2,av2);}}}}

/* k9974 in k9968 in k9956 in k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in k4701 in ... */
static void C_ccall f_9976(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_9976,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9980,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* c-backend.scm:1192: g2167 */
t3=t2;
f_9980(t3,((C_word*)t0)[5],t1);}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t2=C_u_i_car(((C_word*)t0)[6]);
t3=C_eqp(t2,lf[546]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10007,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t3)){
t5=t4;
f_10007(t5,t3);}
else{
t5=C_eqp(t2,lf[549]);
if(C_truep(t5)){
t6=t4;
f_10007(t6,t5);}
else{
t6=C_eqp(t2,lf[550]);
if(C_truep(t6)){
t7=t4;
f_10007(t7,t6);}
else{
t7=C_eqp(t2,lf[551]);
if(C_truep(t7)){
t8=t4;
f_10007(t8,t7);}
else{
t8=C_eqp(t2,lf[552]);
if(C_truep(t8)){
t9=t4;
f_10007(t9,t8);}
else{
t9=C_eqp(t2,lf[553]);
if(C_truep(t9)){
t10=t4;
f_10007(t10,t9);}
else{
t10=C_eqp(t2,lf[554]);
if(C_truep(t10)){
t11=t4;
f_10007(t11,t10);}
else{
t11=C_eqp(t2,lf[555]);
t12=t4;
f_10007(t12,(C_truep(t11)?t11:C_eqp(t2,lf[556])));}}}}}}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* g2167 in k9974 in k9968 in k9956 in k9944 in k9932 in k9920 in k9911 in compute-size in k9899 in k9893 in k9890 in k9887 in k9884 in g2032 in k9089 in k9086 in k9083 in k9080 in k9077 in k9073 in k4704 in ... */
static void C_fcall f_9980(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_9980,3,t0,t1,t2);}
/* c-backend.scm:1193: compute-size */
t3=((C_word*)((C_word*)t0)[2])[1];
f_9903(t3,t1,C_i_vector_ref(t2,C_fix(0)),((C_word*)t0)[3],((C_word*)t0)[4]);}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_c_2dbackend_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("c-backend"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_c_2dbackend_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(1730))){
C_save(t1);
C_rereclaim2(1730*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,883);
lf[0]=C_h_intern(&lf[0],9, C_text("c-backend"));
lf[1]=C_h_intern(&lf[1],27, C_text("chicken.compiler.c-backend#"));
lf[5]=C_h_intern(&lf[5],14, C_text("scheme#newline"));
lf[6]=C_h_intern(&lf[6],14, C_text("scheme#display"));
lf[8]=C_h_intern(&lf[8],24, C_text("chicken.base#intersperse"));
lf[9]=C_h_intern(&lf[9],40, C_text("chicken.compiler.c-backend#generate-code"));
lf[10]=C_h_intern(&lf[10],29, C_text("chicken.compiler.support#bomb"));
lf[11]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021can\047t find lambda"));
lf[12]=C_h_intern(&lf[12],31, C_text("chicken.internal#hash-table-ref"));
lf[13]=C_h_intern(&lf[13],16, C_text("##core#immediate"));
lf[14]=C_h_intern(&lf[14],4, C_text("bool"));
lf[15]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_SCHEME_TRUE"));
lf[16]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_SCHEME_FALSE"));
lf[17]=C_h_intern(&lf[17],4, C_text("char"));
lf[18]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_make_character("));
lf[19]=C_h_intern(&lf[19],3, C_text("nil"));
lf[20]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_SCHEME_END_OF_LIST"));
lf[21]=C_h_intern(&lf[21],3, C_text("fix"));
lf[22]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006C_fix("));
lf[23]=C_h_intern(&lf[23],3, C_text("eof"));
lf[24]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_SCHEME_END_OF_FILE"));
lf[25]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015bad immediate"));
lf[26]=C_h_intern(&lf[26],14, C_text("##core#literal"));
lf[27]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013((C_word)li"));
lf[28]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[29]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[30]=C_h_intern(&lf[30],12, C_text("##core#float"));
lf[31]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003NAN"));
lf[32]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010INFINITY"));
lf[33]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(double)"));
lf[34]=C_h_intern(&lf[34],2, C_text("if"));
lf[35]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005else{"));
lf[36]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003)){"));
lf[37]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013if(C_truep("));
lf[38]=C_h_intern(&lf[38],11, C_text("##core#proc"));
lf[39]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(C_word)"));
lf[40]=C_h_intern(&lf[40],11, C_text("##core#bind"));
lf[41]=C_h_intern(&lf[41],16, C_text("##core#let_float"));
lf[42]=C_h_intern(&lf[42],21, C_text("##core#float-variable"));
lf[43]=C_h_intern(&lf[43],18, C_text("##core#unbox_float"));
lf[44]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[45]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023C_flonum_magnitude("));
lf[46]=C_h_intern(&lf[46],16, C_text("##core#box_float"));
lf[47]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[48]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_flonum(&a,"));
lf[49]=C_h_intern(&lf[49],10, C_text("##core#ref"));
lf[50]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)["));
lf[51]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012((C_word\052)"));
lf[52]=C_h_intern(&lf[52],15, C_text("##core#rest-car"));
lf[53]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_get_rest_arg(c,"));
lf[54]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004,av,"));
lf[55]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004,t0)"));
lf[56]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_u_i_list_ref(t"));
lf[57]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001,"));
lf[58]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[59]=C_h_intern(&lf[59],43, C_text("chicken.compiler.core#lambda-literal-direct"));
lf[60]=C_h_intern(&lf[60],49, C_text("chicken.compiler.core#lambda-literal-customizable"));
lf[61]=C_h_intern(&lf[61],51, C_text("chicken.compiler.core#lambda-literal-argument-count"));
lf[62]=C_h_intern(&lf[62],17, C_text("##core#rest-null\077"));
lf[63]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_rest_nullp(c,"));
lf[64]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[65]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036C_mk_bool(C_unfix(C_i_length(t"));
lf[66]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006)) >= "));
lf[67]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[68]=C_h_intern(&lf[68],18, C_text("##core#rest-length"));
lf[69]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_fix(c - "));
lf[70]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[71]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_u_i_length(t"));
lf[72]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[73]=C_h_intern(&lf[73],12, C_text("##core#unbox"));
lf[74]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004)[1]"));
lf[75]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012((C_word\052)"));
lf[76]=C_h_intern(&lf[76],15, C_text("##core#update_i"));
lf[77]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_set_block_item("));
lf[78]=C_h_intern(&lf[78],13, C_text("##core#update"));
lf[79]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)+"));
lf[80]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001,"));
lf[81]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_mutate(((C_word \052)"));
lf[82]=C_h_intern(&lf[82],18, C_text("##core#updatebox_i"));
lf[83]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,0,"));
lf[84]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_set_block_item("));
lf[85]=C_h_intern(&lf[85],16, C_text("##core#updatebox"));
lf[86]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004)+1,"));
lf[87]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_mutate(((C_word \052)"));
lf[88]=C_h_intern(&lf[88],14, C_text("##core#closure"));
lf[89]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002a["));
lf[90]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002]="));
lf[91]=C_h_intern(&lf[91],8, C_text("for-each"));
lf[92]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021tmp=(C_word)a,a+="));
lf[93]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005,tmp)"));
lf[94]=C_h_intern(&lf[94],17, C_text("chicken.base#add1"));
lf[95]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023(\052a=C_CLOSURE_TYPE|"));
lf[96]=C_h_intern(&lf[96],10, C_text("##core#box"));
lf[97]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030,tmp=(C_word)a,a+=2,tmp)"));
lf[98]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031(\052a=C_VECTOR_TYPE|1,a[1]="));
lf[99]=C_h_intern(&lf[99],12, C_text("##core#local"));
lf[100]=C_h_intern(&lf[100],15, C_text("##core#setlocal"));
lf[101]=C_h_intern(&lf[101],13, C_text("##core#global"));
lf[102]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[103]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[104]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_retrieve2(lf["));
lf[105]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011],C_text("));
lf[106]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002))"));
lf[107]=C_h_intern(&lf[107],37, C_text("chicken.compiler.support#c-ify-string"));
lf[108]=C_h_intern(&lf[108],20, C_text("##sys#symbol->string"));
lf[109]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\052((C_word\052)lf["));
lf[110]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004]+1)"));
lf[111]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023C_fast_retrieve(lf["));
lf[112]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002])"));
lf[113]=C_h_intern(&lf[113],16, C_text("##core#setglobal"));
lf[114]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012 /\052 (set! "));
lf[115]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011 ...) \052/,"));
lf[116]=C_h_intern(&lf[116],32, C_text("chicken.string#string-translate\052"));
lf[117]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376B\000\000\002\052/\376B\000\000\003\052_/\376\377\016"));
lf[118]=C_h_intern(&lf[118],23, C_text("chicken.string#->string"));
lf[119]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_mutate(&lf["));
lf[120]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[121]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_mutate((C_word\052)lf["));
lf[122]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003]+1"));
lf[123]=C_h_intern(&lf[123],18, C_text("##core#setglobal_i"));
lf[124]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[125]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005] /\052 "));
lf[126]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005 \052/ ="));
lf[127]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_set_block_item(lf["));
lf[128]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005] /\052 "));
lf[129]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 \052/,0,"));
lf[130]=C_h_intern(&lf[130],16, C_text("##core#undefined"));
lf[131]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022C_SCHEME_UNDEFINED"));
lf[132]=C_h_intern(&lf[132],11, C_text("##core#call"));
lf[133]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007,av2);}"));
lf[134]=C_h_intern(&lf[134],40, C_text("chicken.compiler.support#node-parameters"));
lf[135]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0010"));
lf[136]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012goto loop;"));
lf[137]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002c="));
lf[138]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002=t"));
lf[139]=C_h_intern(&lf[139],48, C_text("chicken.compiler.core#lambda-literal-temporaries"));
lf[140]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[141]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006av2);}"));
lf[142]=C_h_intern(&lf[142],19, C_text("##sys#string-append"));
lf[143]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[144]=C_h_intern(&lf[144],20, C_text("##sys#number->string"));
lf[145]=C_h_intern(&lf[145],44, C_text("chicken.compiler.core#lambda-literal-looping"));
lf[146]=C_h_intern(&lf[146],39, C_text("chicken.compiler.core#lambda-literal-id"));
lf[147]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003tp("));
lf[148]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007,av2);}"));
lf[149]=C_h_intern(&lf[149],48, C_text("chicken.compiler.core#no-global-procedure-checks"));
lf[150]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024(void\052)(\052((C_word\052)("));
lf[151]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005)+1))"));
lf[152]=C_h_intern(&lf[152],20, C_text("scheme#string-append"));
lf[153]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[154]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[155]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\052((C_word\052)lf["));
lf[156]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004]+1)"));
lf[157]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_fast_retrieve_proc("));
lf[158]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[159]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030C_retrieve2_symbol_proc("));
lf[160]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,C_text("));
lf[161]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002))"));
lf[162]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[163]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[164]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_fast_retrieve_proc("));
lf[165]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[166]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\052((C_word\052)lf["));
lf[167]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004]+1)"));
lf[168]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037C_fast_retrieve_symbol_proc(lf["));
lf[169]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002])"));
lf[170]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\052((C_word\052)lf["));
lf[171]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004]+1)"));
lf[172]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023{C_proc tp=(C_proc)"));
lf[173]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)("));
lf[174]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007,av2);}"));
lf[175]=C_h_intern(&lf[175],31, C_text("chicken.compiler.support#unsafe"));
lf[176]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024(void\052)(\052((C_word\052)t"));
lf[177]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004+1))"));
lf[178]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_fast_retrieve_proc(t"));
lf[179]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[180]=C_h_intern(&lf[180],41, C_text("chicken.compiler.core#no-procedure-checks"));
lf[181]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011((C_proc)"));
lf[182]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[183]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002;{"));
lf[184]=C_h_intern(&lf[184],35, C_text("chicken.compiler.support#node-class"));
lf[185]=C_h_intern(&lf[185],37, C_text("chicken.compiler.core#emit-debug-info"));
lf[186]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032C_debugger(&(C_debug_info["));
lf[187]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003]),"));
lf[188]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0060,NULL"));
lf[189]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[190]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004c,av"));
lf[191]=C_h_intern(&lf[191],37, C_text("chicken.compiler.core#emit-trace-info"));
lf[192]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_trace(C_text(\042"));
lf[193]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004\042));"));
lf[194]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376B\000\000\001\134\376B\000\000\002\134\134\376\377\016"));
lf[195]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003/\052 "));
lf[196]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 \052/"));
lf[197]=C_h_intern(&lf[197],49, C_text("chicken.compiler.core#lambda-literal-closure-size"));
lf[198]=C_h_intern(&lf[198],44, C_text("chicken.compiler.support#source-info->string"));
lf[199]=C_h_intern(&lf[199],14, C_text("##core#recurse"));
lf[200]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012goto loop;"));
lf[201]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002=t"));
lf[202]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003t0,"));
lf[203]=C_h_intern(&lf[203],18, C_text("##core#direct_call"));
lf[204]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_a_i(&a,"));
lf[205]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[206]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032C_debugger(&(C_debug_info["));
lf[207]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003]),"));
lf[208]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0060,NULL"));
lf[209]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002),"));
lf[210]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004c,av"));
lf[211]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_trace(\042"));
lf[212]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\042),"));
lf[213]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003/\052 "));
lf[214]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 \052/"));
lf[215]=C_h_intern(&lf[215],14, C_text("##core#provide"));
lf[216]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_a_i_provide(&a,1,lf["));
lf[217]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002])"));
lf[218]=C_h_intern(&lf[218],15, C_text("##core#callunit"));
lf[219]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002C_"));
lf[220]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[221]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007,av2);}"));
lf[223]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022C_SCHEME_UNDEFINED"));
lf[224]=C_h_intern(&lf[224],13, C_text("##core#return"));
lf[225]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[226]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007return("));
lf[227]=C_h_intern(&lf[227],13, C_text("##core#inline"));
lf[228]=C_h_intern(&lf[228],18, C_text("##core#debug-event"));
lf[229]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032C_debugger(&(C_debug_info["));
lf[230]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003]),"));
lf[231]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0060,NULL"));
lf[232]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[233]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004c,av"));
lf[234]=C_h_intern(&lf[234],22, C_text("##core#inline_allocate"));
lf[235]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004(&a,"));
lf[236]=C_h_intern(&lf[236],17, C_text("##core#inline_ref"));
lf[238]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001a"));
lf[239]=C_h_intern(&lf[239],20, C_text("##core#inline_update"));
lf[240]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025),C_SCHEME_UNDEFINED)"));
lf[241]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002=("));
lf[243]=C_h_intern(&lf[243],51, C_text("chicken.compiler.c-backend#foreign-type-declaration"));
lf[244]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[245]=C_h_intern(&lf[245],21, C_text("##core#inline_loc_ref"));
lf[246]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003)))"));
lf[247]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\052(("));
lf[248]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021\052)C_data_pointer("));
lf[249]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[250]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001a"));
lf[251]=C_h_intern(&lf[251],24, C_text("##core#inline_loc_update"));
lf[252]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025),C_SCHEME_UNDEFINED)"));
lf[253]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003))="));
lf[254]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004((\052("));
lf[255]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021\052)C_data_pointer("));
lf[256]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[257]=C_h_intern(&lf[257],13, C_text("##core#switch"));
lf[258]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010default:"));
lf[259]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005case "));
lf[260]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002){"));
lf[261]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007switch("));
lf[262]=C_h_intern(&lf[262],11, C_text("##core#cond"));
lf[263]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)\077"));
lf[264]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011(C_truep("));
lf[265]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010bad form"));
lf[266]=C_h_intern(&lf[266],44, C_text("chicken.compiler.support#node-subexpressions"));
lf[267]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001;"));
lf[268]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004av2["));
lf[269]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002]="));
lf[270]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007av2[0]="));
lf[271]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001;"));
lf[272]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_word av2["));
lf[273]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002];"));
lf[274]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_word \052av2=av;"));
lf[275]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001}"));
lf[276]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016  av2=C_alloc("));
lf[277]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[278]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010} else {"));
lf[279]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011  av2=av;"));
lf[280]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010if(c >= "));
lf[281]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003) {"));
lf[282]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_word \052av2;"));
lf[283]=C_h_intern(&lf[283],4, C_text("none"));
lf[284]=C_h_intern(&lf[284],55, C_text("chicken.compiler.core#lambda-literal-rest-argument-mode"));
lf[285]=C_h_intern(&lf[285],43, C_text("chicken.compiler.core#external-protos-first"));
lf[287]=C_h_intern(&lf[287],47, C_text("chicken.compiler.support#foreign-callback-stubs"));
lf[288]=C_h_intern(&lf[288],42, C_text("chicken.compiler.core#foreign-declarations"));
lf[289]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012#include \042"));
lf[290]=C_h_intern(&lf[290],47, C_text("chicken.compiler.c-platform#target-include-file"));
lf[291]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\042"));
lf[292]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\052/"));
lf[293]=C_h_intern(&lf[293],32, C_text("chicken.compiler.core#used-units"));
lf[294]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011   uses: "));
lf[295]=C_h_intern(&lf[295],31, C_text("chicken.compiler.core#unit-name"));
lf[296]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011   unit: "));
lf[297]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022/\052 Generated from "));
lf[298]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030 by the CHICKEN compiler"));
lf[299]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031   http://www.call-cc.org"));
lf[300]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021   command line: "));
lf[301]=C_h_intern(&lf[301],33, C_text("chicken.string#string-intersperse"));
lf[302]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[303]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003   "));
lf[304]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[305]=C_h_intern(&lf[305],27, C_text("chicken.string#string-split"));
lf[306]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[307]=C_h_intern(&lf[307],32, C_text("chicken.platform#chicken-version"));
lf[308]=C_h_intern(&lf[308],3, C_text("map"));
lf[309]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002};"));
lf[310]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002,0"));
lf[311]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026static C_char C_TLS li"));
lf[312]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026[] C_aligned={C_lihdr("));
lf[313]=C_h_intern(&lf[313],25, C_text("##sys#lambda-info->string"));
lf[314]=C_decode_literal(C_heaptop,C_text("\376B\000\000)static double C_possibly_force_alignment;"));
lf[315]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027static C_TLS C_word lf["));
lf[316]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002];"));
lf[317]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_noret_decl(C_"));
lf[318]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[319]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036C_externimport void C_ccall C_"));
lf[320]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036(C_word c,C_word \052av) C_noret;"));
lf[321]=C_decode_literal(C_heaptop,C_text("\376B\000\000+static C_PTABLE_ENTRY \052create_ptable(void);"));
lf[322]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 C_noret"));
lf[323]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word \052av"));
lf[324]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_word \052a"));
lf[325]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_word c,"));
lf[326]=C_h_intern(&lf[326],8, C_text("toplevel"));
lf[327]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_fcall "));
lf[328]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_ccall "));
lf[329]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_word "));
lf[330]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005void "));
lf[331]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007static "));
lf[332]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_noret_decl("));
lf[333]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[334]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002C_"));
lf[335]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034C_externexport void C_ccall "));
lf[336]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_noret_decl(C_"));
lf[337]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[338]=C_h_intern(&lf[338],46, C_text("chicken.compiler.core#lambda-literal-allocated"));
lf[340]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[341]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003);}"));
lf[343]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[344]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_word t"));
lf[345]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004=av["));
lf[346]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002];"));
lf[347]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026static void C_ccall tr"));
lf[348]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026(C_word c,C_word \052av){"));
lf[349]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_noret_decl(tr"));
lf[350]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[351]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026static void C_ccall tr"));
lf[352]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036(C_word c,C_word \052av) C_noret;"));
lf[353]=C_h_intern(&lf[353],6, C_text("format"));
lf[354]=C_h_intern(&lf[354],30, C_text("chicken.base#get-output-string"));
lf[355]=C_h_intern(&lf[355],18, C_text("##sys#write-char-0"));
lf[356]=C_h_intern(&lf[356],11, C_text("##sys#print"));
lf[357]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003lf["));
lf[358]=C_h_intern(&lf[358],31, C_text("chicken.base#open-output-string"));
lf[359]=C_h_intern(&lf[359],44, C_text("chicken.compiler.c-platform#words-per-flonum"));
lf[360]=C_h_intern(&lf[360],5, C_text("foldl"));
lf[361]=C_h_intern(&lf[361],19, C_text("scheme#vector->list"));
lf[362]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035type of literal not supported"));
lf[363]=C_h_intern(&lf[363],37, C_text("chicken.compiler.support#bytes->words"));
lf[364]=C_h_intern(&lf[364],17, C_text("##sys#bytevector\077"));
lf[365]=C_h_intern(&lf[365],48, C_text("chicken.compiler.support#block-variable-literal\077"));
lf[366]=C_h_intern(&lf[366],24, C_text("chicken.keyword#keyword\077"));
lf[367]=C_h_intern(&lf[367],36, C_text("chicken.compiler.support#big-fixnum\077"));
lf[368]=C_h_intern(&lf[368],35, C_text("chicken.compiler.support#immediate\077"));
lf[369]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007=C_fix("));
lf[370]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[371]=C_h_intern(&lf[371],21, C_text("##sys#undefined-value"));
lf[372]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024=C_SCHEME_UNDEFINED;"));
lf[373]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_SCHEME_TRUE"));
lf[374]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_SCHEME_FALSE"));
lf[375]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022=C_make_character("));
lf[376]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[377]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_h_intern_kw"));
lf[378]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_h_intern"));
lf[379]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002(&"));
lf[380]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011, C_text("));
lf[381]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003));"));
lf[382]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001="));
lf[383]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026=C_SCHEME_END_OF_LIST;"));
lf[384]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003));"));
lf[386]=C_decode_literal(C_heaptop,C_text("\376B\000\000#=C_decode_literal(C_heaptop,C_text("));
lf[387]=C_h_intern(&lf[387],16, C_text("##sys#copy-bytes"));
lf[388]=C_h_intern(&lf[388],18, C_text("scheme#make-string"));
lf[389]=C_h_intern(&lf[389],41, C_text("chicken.compiler.core#lambda-literal-body"));
lf[390]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_word t"));
lf[391]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[392]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023=C_build_rest(&a,c,"));
lf[393]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005,av);"));
lf[394]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032C_save_and_reclaim((void\052)"));
lf[395]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,c,av);}"));
lf[396]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015a=C_alloc((c-"));
lf[397]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020)\052C_SIZEOF_PAIR+"));
lf[398]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[399]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012a=C_alloc("));
lf[400]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[401]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003);}"));
lf[402]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042C_save_and_reclaim_args((void \052)tr"));
lf[403]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033C_save_and_reclaim((void \052)"));
lf[404]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,c,av);}"));
lf[405]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003av["));
lf[406]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003]=t"));
lf[407]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001;"));
lf[408]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022C_register_lf2(lf,"));
lf[409]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022,create_ptable());"));
lf[410]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023C_initialize_lf(lf,"));
lf[411]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[412]=C_decode_literal(C_heaptop,C_text("\376B\000\000+C_check_nursery_minimum(C_calculate_demand("));
lf[413]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,c,"));
lf[414]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003));"));
lf[415]=C_decode_literal(C_heaptop,C_text("\376B\000\000+if(C_unlikely(!C_demand(C_calculate_demand("));
lf[416]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,c,"));
lf[417]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005)))){"));
lf[418]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034C_save_and_reclaim((void\052)C_"));
lf[419]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,c,av);}"));
lf[420]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027toplevel_initialized=1;"));
lf[421]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032if(C_unlikely(!C_demand_2("));
lf[422]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004))){"));
lf[423]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_save(t1);"));
lf[424]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_rereclaim2("));
lf[425]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023\052sizeof(C_word),1);"));
lf[426]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016t1=C_restore;}"));
lf[427]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012a=C_alloc("));
lf[428]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[429]=C_h_intern(&lf[429],39, C_text("chicken.compiler.core#target-stack-size"));
lf[430]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_resize_stack("));
lf[431]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[432]=C_h_intern(&lf[432],38, C_text("chicken.compiler.core#target-heap-size"));
lf[433]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032C_set_or_change_heap_size("));
lf[434]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004,1);"));
lf[435]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_heap_size_is_fixed=1;"));
lf[436]=C_h_intern(&lf[436],53, C_text("chicken.compiler.core#disable-stack-overflow-checking"));
lf[437]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033C_disable_overflow_check=1;"));
lf[438]=C_decode_literal(C_heaptop,C_text("\376B\000\000$C_register_debug_info(C_debug_info);"));
lf[439]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word \052a;"));
lf[440]=C_decode_literal(C_heaptop,C_text("\376B\000\000=if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}"));
lf[441]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036else C_toplevel_entry(C_text(\042"));
lf[442]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004\042));"));
lf[443]=C_decode_literal(C_heaptop,C_text("\376B\000\000.if(C_unlikely(!C_demand(C_calculate_demand((c-"));
lf[444]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021)\052C_SIZEOF_PAIR +"));
lf[445]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,c,"));
lf[446]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005)))){"));
lf[447]=C_h_intern(&lf[447],41, C_text("chicken.compiler.core#insert-timer-checks"));
lf[448]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_check_for_interrupt;"));
lf[449]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005if(c<"));
lf[450]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025) C_bad_min_argc_2(c,"));
lf[451]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005,t0);"));
lf[452]=C_h_intern(&lf[452],36, C_text("chicken.compiler.core#no-argc-checks"));
lf[453]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word \052a;"));
lf[454]=C_decode_literal(C_heaptop,C_text("\376B\000\000+if(C_unlikely(!C_demand(C_calculate_demand("));
lf[455]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,0,"));
lf[456]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005)))){"));
lf[457]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,c,"));
lf[458]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_check_for_interrupt;"));
lf[459]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005if(c<"));
lf[460]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025) C_bad_min_argc_2(c,"));
lf[461]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005,t0);"));
lf[462]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006if(c!="));
lf[463]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021) C_bad_argc_2(c,"));
lf[464]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005,t0);"));
lf[465]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005loop:"));
lf[466]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_stack_overflow_check;"));
lf[467]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word \052a;"));
lf[468]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_word t"));
lf[469]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010double f"));
lf[470]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_word t"));
lf[471]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_word t"));
lf[472]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004=av["));
lf[473]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002];"));
lf[474]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_word tmp;"));
lf[475]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002){"));
lf[476]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word \052av"));
lf[477]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_word \052a"));
lf[478]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_word c,"));
lf[479]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_fcall "));
lf[480]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_ccall "));
lf[481]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_word "));
lf[482]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005void "));
lf[483]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007static "));
lf[484]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017void C_ccall C_"));
lf[485]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022C_main_entry_point"));
lf[486]=C_decode_literal(C_heaptop,C_text("\376B\000\000(static C_TLS int toplevel_initialized=0;"));
lf[487]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003/\052 "));
lf[488]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 \052/"));
lf[490]=C_h_intern(&lf[490],34, C_text("chicken.compiler.support#debugging"));
lf[491]=C_h_intern(&lf[491],1, C_text("o"));
lf[492]=C_decode_literal(C_heaptop,C_text("\376B\000\000 dropping unused closure argument"));
lf[493]=C_h_intern(&lf[493],54, C_text("chicken.compiler.core#lambda-literal-float-temporaries"));
lf[494]=C_h_intern(&lf[494],45, C_text("chicken.compiler.core#lambda-literal-external"));
lf[495]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[496]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[497]=C_h_intern(&lf[497],50, C_text("chicken.compiler.core#lambda-literal-rest-argument"));
lf[498]=C_h_intern(&lf[498],10, C_text("scheme#max"));
lf[499]=C_h_intern(&lf[499],54, C_text("chicken.compiler.core#lambda-literal-callee-signatures"));
lf[500]=C_h_intern(&lf[500],34, C_text("chicken.compiler.support#real-name"));
lf[501]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002/\052"));
lf[502]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\052/"));
lf[503]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021/\052 end of file \052/"));
lf[504]=C_h_intern(&lf[504],51, C_text("chicken.compiler.support#collected-debugging-output"));
lf[505]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002C_"));
lf[506]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002},"));
lf[507]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002},"));
lf[508]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011{C_text(\042"));
lf[509]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012\042),(void\052)"));
lf[510]=C_h_intern(&lf[510],37, C_text("chicken.internal#string->c-identifier"));
lf[511]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001{"));
lf[512]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027#ifdef C_ENABLE_PTABLES"));
lf[513]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016return ptable;"));
lf[514]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005#else"));
lf[515]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014return NULL;"));
lf[516]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006#endif"));
lf[517]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001}"));
lf[518]=C_decode_literal(C_heaptop,C_text("\376B\000\000\052static C_PTABLE_ENTRY \052create_ptable(void)"));
lf[519]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006#endif"));
lf[520]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015{NULL,NULL}};"));
lf[521]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027#ifdef C_ENABLE_PTABLES"));
lf[522]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035static C_PTABLE_ENTRY ptable["));
lf[523]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005] = {"));
lf[524]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022{0,0,NULL,NULL}};\012"));
lf[525]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002},"));
lf[526]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005NULL,"));
lf[527]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_text(\042"));
lf[528]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\042),"));
lf[529]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001{"));
lf[530]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003,0,"));
lf[531]=C_h_intern(&lf[531],17, C_text("chicken.sort#sort"));
lf[532]=C_decode_literal(C_heaptop,C_text("\376B\000\000$static C_DEBUG_INFO C_debug_info[]={"));
lf[533]=C_h_intern(&lf[533],5, C_text("float"));
lf[534]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002+3"));
lf[535]=C_h_intern(&lf[535],16, C_text("unsigned-integer"));
lf[536]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024+C_SIZEOF_FIX_BIGNUM"));
lf[537]=C_h_intern(&lf[537],18, C_text("unsigned-integer64"));
lf[538]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023+C_SIZEOF_BIGNUM(2)"));
lf[539]=C_h_intern(&lf[539],8, C_text("c-string"));
lf[540]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004+2+("));
lf[541]=C_decode_literal(C_heaptop,C_text("\376B\000\000!==NULL\0771:C_bytestowords(C_strlen("));
lf[542]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003)))"));
lf[543]=C_h_intern(&lf[543],16, C_text("nonnull-c-string"));
lf[544]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033+2+C_bytestowords(C_strlen("));
lf[545]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002))"));
lf[546]=C_h_intern(&lf[546],3, C_text("ref"));
lf[547]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002+3"));
lf[548]=C_h_intern(&lf[548],5, C_text("const"));
lf[549]=C_h_intern(&lf[549],7, C_text("pointer"));
lf[550]=C_h_intern(&lf[550],9, C_text("c-pointer"));
lf[551]=C_h_intern(&lf[551],15, C_text("nonnull-pointer"));
lf[552]=C_h_intern(&lf[552],17, C_text("nonnull-c-pointer"));
lf[553]=C_h_intern(&lf[553],8, C_text("function"));
lf[554]=C_h_intern(&lf[554],8, C_text("instance"));
lf[555]=C_h_intern(&lf[555],16, C_text("nonnull-instance"));
lf[556]=C_h_intern(&lf[556],12, C_text("instance-ref"));
lf[557]=C_h_intern(&lf[557],44, C_text("chicken.compiler.support#lookup-foreign-type"));
lf[558]=C_h_intern(&lf[558],17, C_text("nonnull-c-string\052"));
lf[559]=C_h_intern(&lf[559],25, C_text("nonnull-unsigned-c-string"));
lf[560]=C_h_intern(&lf[560],26, C_text("nonnull-unsigned-c-string\052"));
lf[561]=C_h_intern(&lf[561],6, C_text("symbol"));
lf[562]=C_h_intern(&lf[562],9, C_text("c-string\052"));
lf[563]=C_h_intern(&lf[563],17, C_text("unsigned-c-string"));
lf[564]=C_h_intern(&lf[564],18, C_text("unsigned-c-string\052"));
lf[565]=C_h_intern(&lf[565],9, C_text("integer64"));
lf[566]=C_h_intern(&lf[566],6, C_text("size_t"));
lf[567]=C_h_intern(&lf[567],7, C_text("ssize_t"));
lf[568]=C_h_intern(&lf[568],18, C_text("unsigned-integer32"));
lf[569]=C_h_intern(&lf[569],4, C_text("long"));
lf[570]=C_h_intern(&lf[570],7, C_text("integer"));
lf[571]=C_h_intern(&lf[571],9, C_text("integer32"));
lf[572]=C_h_intern(&lf[572],13, C_text("unsigned-long"));
lf[573]=C_h_intern(&lf[573],6, C_text("number"));
lf[574]=C_h_intern(&lf[574],6, C_text("double"));
lf[575]=C_h_intern(&lf[575],13, C_text("c-string-list"));
lf[576]=C_h_intern(&lf[576],14, C_text("c-string-list\052"));
lf[577]=C_h_intern(&lf[577],3, C_text("int"));
lf[578]=C_h_intern(&lf[578],5, C_text("int32"));
lf[579]=C_h_intern(&lf[579],5, C_text("short"));
lf[580]=C_h_intern(&lf[580],4, C_text("void"));
lf[581]=C_h_intern(&lf[581],14, C_text("unsigned-short"));
lf[582]=C_h_intern(&lf[582],13, C_text("scheme-object"));
lf[583]=C_h_intern(&lf[583],13, C_text("unsigned-char"));
lf[584]=C_h_intern(&lf[584],12, C_text("unsigned-int"));
lf[585]=C_h_intern(&lf[585],14, C_text("unsigned-int32"));
lf[586]=C_h_intern(&lf[586],4, C_text("byte"));
lf[587]=C_h_intern(&lf[587],13, C_text("unsigned-byte"));
lf[588]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002;}"));
lf[589]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033C_callback_wrapper((void \052)"));
lf[590]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007return "));
lf[591]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002x="));
lf[592]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[593]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_save(x);"));
lf[594]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001a"));
lf[595]=C_h_intern(&lf[595],14, C_text("scheme#reverse"));
lf[596]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035C_callback_adjust_stack(a,s);"));
lf[597]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0010"));
lf[598]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_word x,s="));
lf[599]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004,\052a="));
lf[600]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_stack_pointer;"));
lf[601]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_alloc(s);"));
lf[603]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[604]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010/\052 from "));
lf[605]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 \052/"));
lf[606]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0010"));
lf[607]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[608]=C_h_intern(&lf[608],61, C_text("chicken.compiler.support#foreign-callback-stub-argument-types"));
lf[609]=C_h_intern(&lf[609],58, C_text("chicken.compiler.support#foreign-callback-stub-return-type"));
lf[610]=C_h_intern(&lf[610],35, C_text("chicken.compiler.support#real-name2"));
lf[611]=C_h_intern(&lf[611],49, C_text("chicken.compiler.support#foreign-callback-stub-id"));
lf[612]=C_h_intern(&lf[612],42, C_text("chicken.compiler.core#foreign-lambda-stubs"));
lf[613]=C_decode_literal(C_heaptop,C_text("\376B\000\000.C_k=C_restore_callback_continuation2(C_level);"));
lf[614]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_kontinue(C_k,C_r);"));
lf[615]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_kontinue(C_k,C_r);"));
lf[616]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013return C_r;"));
lf[617]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015#undef return"));
lf[618]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006C_ret:"));
lf[619]=C_decode_literal(C_heaptop,C_text("\376B\000\000.C_k=C_restore_callback_continuation2(C_level);"));
lf[620]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_kontinue(C_k,C_r);"));
lf[621]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_kontinue(C_k,C_r);"));
lf[622]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013return C_r;"));
lf[623]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[624]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[625]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004C_r="));
lf[626]=C_decode_literal(C_heaptop,C_text("\376B\000\0003int C_level=C_save_callback_continuation(&C_a,C_k);"));
lf[627]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002=("));
lf[628]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003C_a"));
lf[629]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002);"));
lf[630]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[631]=C_h_intern(&lf[631],21, C_text("scheme#symbol->string"));
lf[632]=C_decode_literal(C_heaptop,C_text("\376B\000\0002C_word C_r=C_SCHEME_UNDEFINED,\052C_a=(C_word\052)C_buf;"));
lf[633]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_word C_a"));
lf[634]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006=C_av["));
lf[635]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002];"));
lf[636]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_noret_decl("));
lf[637]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[638]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024static void C_ccall "));
lf[639]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032(C_word C_c,C_word \052C_av){"));
lf[640]=C_decode_literal(C_heaptop,C_text("\376B\000\000!C_word C_k=C_av[1],C_buf=C_av[2];"));
lf[641]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002){"));
lf[642]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_word C_buf"));
lf[643]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003C_a"));
lf[644]=C_decode_literal(C_heaptop,C_text("\376B\000\000 C_regparm static C_word C_fcall "));
lf[645]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042#define return(x) C_cblock C_r = ("));
lf[646]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036(x))); goto C_ret; C_cblockend"));
lf[647]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010/\052 from "));
lf[648]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 \052/"));
lf[649]=C_h_intern(&lf[649],43, C_text("chicken.compiler.core#foreign-stub-callback"));
lf[650]=C_h_intern(&lf[650],38, C_text("chicken.compiler.core#foreign-stub-cps"));
lf[651]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003C_a"));
lf[652]=C_h_intern(&lf[652],49, C_text("chicken.compiler.core#foreign-stub-argument-names"));
lf[653]=C_h_intern(&lf[653],39, C_text("chicken.compiler.core#foreign-stub-body"));
lf[654]=C_h_intern(&lf[654],39, C_text("chicken.compiler.core#foreign-stub-name"));
lf[655]=C_h_intern(&lf[655],46, C_text("chicken.compiler.core#foreign-stub-return-type"));
lf[656]=C_h_intern(&lf[656],49, C_text("chicken.compiler.core#foreign-stub-argument-types"));
lf[657]=C_h_intern(&lf[657],37, C_text("chicken.compiler.core#foreign-stub-id"));
lf[658]=C_h_intern(&lf[658],40, C_text("chicken.compiler.core#external-variables"));
lf[659]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[660]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007static "));
lf[661]=C_h_intern(&lf[661],1, C_text("p"));
lf[662]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030code generation phase..."));
lf[663]=C_h_intern(&lf[663],46, C_text("chicken.flonum#flonum-maximum-decimal-exponent"));
lf[664]=C_h_intern(&lf[664],37, C_text("chicken.flonum#flonum-print-precision"));
lf[665]=C_h_intern(&lf[665],15, C_text("scheme#string<\077"));
lf[666]=C_h_intern(&lf[666],18, C_text("chicken.sort#sort!"));
lf[667]=C_h_intern(&lf[667],36, C_text("chicken.internal#hash-table-for-each"));
lf[668]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010toplevel"));
lf[669]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011_toplevel"));
lf[670]=C_h_intern(&lf[670],18, C_text("scheme#string-copy"));
lf[671]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_word "));
lf[672]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_externexport "));
lf[673]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[674]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001t"));
lf[675]=C_h_intern(&lf[675],57, C_text("chicken.compiler.support#foreign-callback-stub-qualifiers"));
lf[676]=C_h_intern(&lf[676],51, C_text("chicken.compiler.support#foreign-callback-stub-name"));
lf[677]=C_h_intern(&lf[677],39, C_text("chicken.compiler.support#quit-compiling"));
lf[678]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031illegal foreign type `~A\047"));
lf[679]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[680]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006C_word"));
lf[681]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006C_char"));
lf[682]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017unsigned C_char"));
lf[683]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014unsigned int"));
lf[684]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005C_u32"));
lf[685]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003int"));
lf[686]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006size_t"));
lf[687]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007ssize_t"));
lf[688]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005C_s32"));
lf[689]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005C_s64"));
lf[690]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005C_u64"));
lf[691]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005short"));
lf[692]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004long"));
lf[693]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016unsigned short"));
lf[694]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015unsigned long"));
lf[695]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005float"));
lf[696]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006double"));
lf[697]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006void \052"));
lf[698]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_char \052\052"));
lf[699]=C_h_intern(&lf[699],4, C_text("blob"));
lf[700]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017unsigned char \052"));
lf[701]=C_h_intern(&lf[701],9, C_text("u16vector"));
lf[702]=C_h_intern(&lf[702],17, C_text("nonnull-u16vector"));
lf[703]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020unsigned short \052"));
lf[704]=C_h_intern(&lf[704],8, C_text("s8vector"));
lf[705]=C_h_intern(&lf[705],16, C_text("nonnull-s8vector"));
lf[706]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015signed char \052"));
lf[707]=C_h_intern(&lf[707],9, C_text("u32vector"));
lf[708]=C_h_intern(&lf[708],17, C_text("nonnull-u32vector"));
lf[709]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016unsigned int \052"));
lf[710]=C_h_intern(&lf[710],9, C_text("u64vector"));
lf[711]=C_h_intern(&lf[711],17, C_text("nonnull-u64vector"));
lf[712]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_u64 \052"));
lf[713]=C_h_intern(&lf[713],9, C_text("s16vector"));
lf[714]=C_h_intern(&lf[714],17, C_text("nonnull-s16vector"));
lf[715]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007short \052"));
lf[716]=C_h_intern(&lf[716],9, C_text("s32vector"));
lf[717]=C_h_intern(&lf[717],17, C_text("nonnull-s32vector"));
lf[718]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005int \052"));
lf[719]=C_h_intern(&lf[719],9, C_text("s64vector"));
lf[720]=C_h_intern(&lf[720],17, C_text("nonnull-s64vector"));
lf[721]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_s64 \052"));
lf[722]=C_h_intern(&lf[722],9, C_text("f32vector"));
lf[723]=C_h_intern(&lf[723],17, C_text("nonnull-f32vector"));
lf[724]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007float \052"));
lf[725]=C_h_intern(&lf[725],9, C_text("f64vector"));
lf[726]=C_h_intern(&lf[726],17, C_text("nonnull-f64vector"));
lf[727]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010double \052"));
lf[728]=C_h_intern(&lf[728],14, C_text("pointer-vector"));
lf[729]=C_h_intern(&lf[729],22, C_text("nonnull-pointer-vector"));
lf[730]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007void \052\052"));
lf[731]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006char \052"));
lf[732]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017unsigned char \052"));
lf[733]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004void"));
lf[734]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\017\001nonnull-pointer\376\003\000\000\002\376\001\000\000\011\001c-pointer\376\003\000\000\002\376\001\000\000\016\001schem"
"e-pointer\376\003\000\000\002\376\001\000\000\026\001nonnull-scheme-pointer\376\003\000\000\002\376\001\000\000\021\001nonnull-c-pointer\376\377\016"));
lf[735]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\052"));
lf[736]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001&"));
lf[737]=C_h_intern(&lf[737],8, C_text("template"));
lf[738]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001<"));
lf[739]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002> "));
lf[740]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001,"));
lf[741]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[742]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[743]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006const "));
lf[744]=C_h_intern(&lf[744],6, C_text("struct"));
lf[745]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007struct "));
lf[746]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[747]=C_h_intern(&lf[747],5, C_text("union"));
lf[748]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006union "));
lf[749]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[750]=C_h_intern(&lf[750],4, C_text("enum"));
lf[751]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005enum "));
lf[752]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[753]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\001instance\376\003\000\000\002\376\001\000\000\020\001nonnull-instance\376\377\016"));
lf[754]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\052"));
lf[755]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001&"));
lf[756]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[757]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 (\052"));
lf[758]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)("));
lf[759]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[760]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001,"));
lf[761]=C_h_intern(&lf[761],3, C_text("..."));
lf[762]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003..."));
lf[763]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[764]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[765]=C_h_intern(&lf[765],12, C_text("nonnull-blob"));
lf[766]=C_h_intern(&lf[766],8, C_text("u8vector"));
lf[767]=C_h_intern(&lf[767],16, C_text("nonnull-u8vector"));
lf[768]=C_h_intern(&lf[768],14, C_text("scheme-pointer"));
lf[769]=C_h_intern(&lf[769],22, C_text("nonnull-scheme-pointer"));
lf[770]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042illegal foreign argument type `~A\047"));
lf[771]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[772]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031C_character_code((C_word)"));
lf[773]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_unfix("));
lf[774]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_unfix("));
lf[775]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030(unsigned short)C_unfix("));
lf[776]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_num_to_unsigned_long("));
lf[777]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_c_double("));
lf[778]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_num_to_int("));
lf[779]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_num_to_int64("));
lf[780]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030(size_t)C_num_to_uint64("));
lf[781]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030(ssize_t)C_num_to_int64("));
lf[782]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_num_to_uint64("));
lf[783]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_num_to_long("));
lf[784]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_num_to_unsigned_int("));
lf[785]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_data_pointer_or_null("));
lf[786]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_data_pointer("));
lf[787]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_c_pointer_or_null("));
lf[788]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_c_pointer_nn("));
lf[789]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_c_bytevector_or_null("));
lf[790]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_c_bytevector("));
lf[791]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_c_u8vector_or_null("));
lf[792]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_c_u8vector("));
lf[793]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_u16vector_or_null("));
lf[794]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_u16vector("));
lf[795]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_u32vector_or_null("));
lf[796]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_u32vector("));
lf[797]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_u64vector_or_null("));
lf[798]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_u64vector("));
lf[799]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_c_s8vector_or_null("));
lf[800]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_c_s8vector("));
lf[801]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_s16vector_or_null("));
lf[802]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_s16vector("));
lf[803]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_s32vector_or_null("));
lf[804]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_s32vector("));
lf[805]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_s64vector_or_null("));
lf[806]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_s64vector("));
lf[807]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_f32vector_or_null("));
lf[808]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_f32vector("));
lf[809]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026C_c_f64vector_or_null("));
lf[810]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_c_f64vector("));
lf[811]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033C_c_pointer_vector_or_null("));
lf[812]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023C_c_pointer_vector("));
lf[813]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_string_or_null("));
lf[814]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013C_c_string("));
lf[815]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010C_truep("));
lf[816]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_c_pointer_or_null("));
lf[817]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_c_pointer_nn("));
lf[818]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_c_pointer_or_null("));
lf[819]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_c_pointer_nn("));
lf[820]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_data_pointer_or_null("));
lf[821]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_data_pointer("));
lf[822]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024C_c_pointer_or_null("));
lf[823]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_num_to_int("));
lf[824]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\052("));
lf[825]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020)C_c_pointer_nn("));
lf[826]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\052"));
lf[827]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\052("));
lf[828]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021\052)C_c_pointer_nn("));
lf[829]=C_decode_literal(C_heaptop,C_text("\376B\000\000 illegal foreign return type `~A\047"));
lf[830]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031C_make_character((C_word)"));
lf[831]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_fix((C_word)"));
lf[832]=C_decode_literal(C_heaptop,C_text("\376B\000\000%C_fix(C_MOST_POSITIVE_FIXNUM&(C_word)"));
lf[833]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015C_fix((short)"));
lf[834]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_fix(0xffff&(C_word)"));
lf[835]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_fix((char)"));
lf[836]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023C_fix(0xff&(C_word)"));
lf[837]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_flonum(&"));
lf[838]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_number(&"));
lf[839]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[840]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[841]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[842]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_mpointer_or_false(&"));
lf[843]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_int_to_num(&"));
lf[844]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_int64_to_num(&"));
lf[845]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021C_uint64_to_num(&"));
lf[846]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027C_unsigned_int_to_num(&"));
lf[847]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017C_long_to_num(&"));
lf[848]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030C_unsigned_long_to_num(&"));
lf[849]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_mk_bool("));
lf[850]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011((C_word)"));
lf[851]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[852]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[853]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011,(void\052)&"));
lf[854]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[855]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[856]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_mpointer_or_false(&"));
lf[857]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[858]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[859]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011,(void\052)&"));
lf[860]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[861]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[862]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_mpointer_or_false(&"));
lf[863]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010,(void\052)"));
lf[864]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_mpointer(&"));
lf[865]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016C_int_to_num(&"));
lf[866]=C_decode_literal(C_heaptop,C_text("\376B\000\000=Encoded literal size of ~S is too large (must fit in 24 bits)"));
lf[867]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\377\006\001"));
lf[868]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\377\006\000"));
lf[869]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\377\012"));
lf[870]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\377\016"));
lf[871]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\377>"));
lf[872]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\377\036"));
lf[873]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\377\001"));
lf[874]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\302"));
lf[875]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001U"));
lf[876]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\000"));
lf[877]=C_h_intern(&lf[877],20, C_text("##sys#flonum->string"));
lf[878]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\001"));
lf[879]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\002"));
lf[880]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\001"));
lf[881]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037invalid literal - cannot encode"));
lf[882]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
C_register_lf2(lf,883,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3245,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_library_toplevel(2,av2);}}

/* va15385 in a5845 in k5725 in k5722 in k6107 in k5691 in k5687 in k5681 in k5678 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in ... */
static void C_fcall va15385(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15385,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* va15391 in a6308 in k6194 in k6188 in k4731 in k4728 in k4725 in expr in expression in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall va15391(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15391,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* va15423 in k7906 in k7902 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall va15423(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15423,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* va15425 in k7939 in k7933 in k7930 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall va15425(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15425,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* va15433 in k8095 in k8091 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall va15433(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15433,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* va15435 in k8120 in k8116 in k8022 in k8004 in k7889 in k7859 in k7853 in literal-size in k4704 in k4701 in chicken.compiler.c-backend#generate-code in k3264 in k3261 in k3258 in k3255 in k3252 in k3249 in k3246 in k3243 */
static void C_fcall va15435(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trva15435,2,t0,t1);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_s_a_i_plus(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[821] = {
{C_text("f14744:c_2dbackend_2escm"),(void*)f14744},
{C_text("f14749:c_2dbackend_2escm"),(void*)f14749},
{C_text("f14754:c_2dbackend_2escm"),(void*)f14754},
{C_text("f14762:c_2dbackend_2escm"),(void*)f14762},
{C_text("f14767:c_2dbackend_2escm"),(void*)f14767},
{C_text("f14772:c_2dbackend_2escm"),(void*)f14772},
{C_text("f14785:c_2dbackend_2escm"),(void*)f14785},
{C_text("f14789:c_2dbackend_2escm"),(void*)f14789},
{C_text("f14796:c_2dbackend_2escm"),(void*)f14796},
{C_text("f14801:c_2dbackend_2escm"),(void*)f14801},
{C_text("f14874:c_2dbackend_2escm"),(void*)f14874},
{C_text("f14881:c_2dbackend_2escm"),(void*)f14881},
{C_text("f14902:c_2dbackend_2escm"),(void*)f14902},
{C_text("f_10007:c_2dbackend_2escm"),(void*)f_10007},
{C_text("f_10363:c_2dbackend_2escm"),(void*)f_10363},
{C_text("f_10366:c_2dbackend_2escm"),(void*)f_10366},
{C_text("f_10369:c_2dbackend_2escm"),(void*)f_10369},
{C_text("f_10372:c_2dbackend_2escm"),(void*)f_10372},
{C_text("f_10375:c_2dbackend_2escm"),(void*)f_10375},
{C_text("f_10378:c_2dbackend_2escm"),(void*)f_10378},
{C_text("f_10387:c_2dbackend_2escm"),(void*)f_10387},
{C_text("f_10390:c_2dbackend_2escm"),(void*)f_10390},
{C_text("f_10393:c_2dbackend_2escm"),(void*)f_10393},
{C_text("f_10396:c_2dbackend_2escm"),(void*)f_10396},
{C_text("f_10399:c_2dbackend_2escm"),(void*)f_10399},
{C_text("f_10402:c_2dbackend_2escm"),(void*)f_10402},
{C_text("f_10405:c_2dbackend_2escm"),(void*)f_10405},
{C_text("f_10424:c_2dbackend_2escm"),(void*)f_10424},
{C_text("f_10426:c_2dbackend_2escm"),(void*)f_10426},
{C_text("f_10436:c_2dbackend_2escm"),(void*)f_10436},
{C_text("f_10475:c_2dbackend_2escm"),(void*)f_10475},
{C_text("f_10477:c_2dbackend_2escm"),(void*)f_10477},
{C_text("f_10499:c_2dbackend_2escm"),(void*)f_10499},
{C_text("f_10511:c_2dbackend_2escm"),(void*)f_10511},
{C_text("f_10521:c_2dbackend_2escm"),(void*)f_10521},
{C_text("f_10534:c_2dbackend_2escm"),(void*)f_10534},
{C_text("f_10538:c_2dbackend_2escm"),(void*)f_10538},
{C_text("f_10541:c_2dbackend_2escm"),(void*)f_10541},
{C_text("f_10544:c_2dbackend_2escm"),(void*)f_10544},
{C_text("f_10547:c_2dbackend_2escm"),(void*)f_10547},
{C_text("f_10553:c_2dbackend_2escm"),(void*)f_10553},
{C_text("f_10556:c_2dbackend_2escm"),(void*)f_10556},
{C_text("f_10559:c_2dbackend_2escm"),(void*)f_10559},
{C_text("f_10564:c_2dbackend_2escm"),(void*)f_10564},
{C_text("f_10574:c_2dbackend_2escm"),(void*)f_10574},
{C_text("f_10577:c_2dbackend_2escm"),(void*)f_10577},
{C_text("f_10596:c_2dbackend_2escm"),(void*)f_10596},
{C_text("f_10608:c_2dbackend_2escm"),(void*)f_10608},
{C_text("f_10610:c_2dbackend_2escm"),(void*)f_10610},
{C_text("f_10612:c_2dbackend_2escm"),(void*)f_10612},
{C_text("f_10617:c_2dbackend_2escm"),(void*)f_10617},
{C_text("f_10687:c_2dbackend_2escm"),(void*)f_10687},
{C_text("f_10804:c_2dbackend_2escm"),(void*)f_10804},
{C_text("f_10825:c_2dbackend_2escm"),(void*)f_10825},
{C_text("f_10957:c_2dbackend_2escm"),(void*)f_10957},
{C_text("f_10969:c_2dbackend_2escm"),(void*)f_10969},
{C_text("f_10984:c_2dbackend_2escm"),(void*)f_10984},
{C_text("f_10988:c_2dbackend_2escm"),(void*)f_10988},
{C_text("f_11032:c_2dbackend_2escm"),(void*)f_11032},
{C_text("f_11049:c_2dbackend_2escm"),(void*)f_11049},
{C_text("f_11062:c_2dbackend_2escm"),(void*)f_11062},
{C_text("f_11066:c_2dbackend_2escm"),(void*)f_11066},
{C_text("f_11070:c_2dbackend_2escm"),(void*)f_11070},
{C_text("f_11088:c_2dbackend_2escm"),(void*)f_11088},
{C_text("f_11090:c_2dbackend_2escm"),(void*)f_11090},
{C_text("f_11115:c_2dbackend_2escm"),(void*)f_11115},
{C_text("f_11139:c_2dbackend_2escm"),(void*)f_11139},
{C_text("f_11156:c_2dbackend_2escm"),(void*)f_11156},
{C_text("f_11173:c_2dbackend_2escm"),(void*)f_11173},
{C_text("f_11190:c_2dbackend_2escm"),(void*)f_11190},
{C_text("f_11207:c_2dbackend_2escm"),(void*)f_11207},
{C_text("f_11224:c_2dbackend_2escm"),(void*)f_11224},
{C_text("f_11253:c_2dbackend_2escm"),(void*)f_11253},
{C_text("f_11257:c_2dbackend_2escm"),(void*)f_11257},
{C_text("f_11278:c_2dbackend_2escm"),(void*)f_11278},
{C_text("f_11280:c_2dbackend_2escm"),(void*)f_11280},
{C_text("f_11290:c_2dbackend_2escm"),(void*)f_11290},
{C_text("f_11305:c_2dbackend_2escm"),(void*)f_11305},
{C_text("f_11704:c_2dbackend_2escm"),(void*)f_11704},
{C_text("f_11706:c_2dbackend_2escm"),(void*)f_11706},
{C_text("f_11734:c_2dbackend_2escm"),(void*)f_11734},
{C_text("f_11761:c_2dbackend_2escm"),(void*)f_11761},
{C_text("f_11986:c_2dbackend_2escm"),(void*)f_11986},
{C_text("f_11995:c_2dbackend_2escm"),(void*)f_11995},
{C_text("f_12004:c_2dbackend_2escm"),(void*)f_12004},
{C_text("f_12102:c_2dbackend_2escm"),(void*)f_12102},
{C_text("f_12369:c_2dbackend_2escm"),(void*)f_12369},
{C_text("f_12371:c_2dbackend_2escm"),(void*)f_12371},
{C_text("f_12438:c_2dbackend_2escm"),(void*)f_12438},
{C_text("f_12444:c_2dbackend_2escm"),(void*)f_12444},
{C_text("f_12447:c_2dbackend_2escm"),(void*)f_12447},
{C_text("f_12450:c_2dbackend_2escm"),(void*)f_12450},
{C_text("f_12462:c_2dbackend_2escm"),(void*)f_12462},
{C_text("f_12468:c_2dbackend_2escm"),(void*)f_12468},
{C_text("f_12471:c_2dbackend_2escm"),(void*)f_12471},
{C_text("f_12474:c_2dbackend_2escm"),(void*)f_12474},
{C_text("f_12486:c_2dbackend_2escm"),(void*)f_12486},
{C_text("f_12489:c_2dbackend_2escm"),(void*)f_12489},
{C_text("f_12495:c_2dbackend_2escm"),(void*)f_12495},
{C_text("f_12498:c_2dbackend_2escm"),(void*)f_12498},
{C_text("f_12501:c_2dbackend_2escm"),(void*)f_12501},
{C_text("f_12513:c_2dbackend_2escm"),(void*)f_12513},
{C_text("f_12519:c_2dbackend_2escm"),(void*)f_12519},
{C_text("f_12522:c_2dbackend_2escm"),(void*)f_12522},
{C_text("f_12525:c_2dbackend_2escm"),(void*)f_12525},
{C_text("f_12540:c_2dbackend_2escm"),(void*)f_12540},
{C_text("f_12546:c_2dbackend_2escm"),(void*)f_12546},
{C_text("f_12549:c_2dbackend_2escm"),(void*)f_12549},
{C_text("f_12552:c_2dbackend_2escm"),(void*)f_12552},
{C_text("f_12567:c_2dbackend_2escm"),(void*)f_12567},
{C_text("f_12573:c_2dbackend_2escm"),(void*)f_12573},
{C_text("f_12576:c_2dbackend_2escm"),(void*)f_12576},
{C_text("f_12579:c_2dbackend_2escm"),(void*)f_12579},
{C_text("f_12594:c_2dbackend_2escm"),(void*)f_12594},
{C_text("f_12600:c_2dbackend_2escm"),(void*)f_12600},
{C_text("f_12603:c_2dbackend_2escm"),(void*)f_12603},
{C_text("f_12606:c_2dbackend_2escm"),(void*)f_12606},
{C_text("f_12621:c_2dbackend_2escm"),(void*)f_12621},
{C_text("f_12627:c_2dbackend_2escm"),(void*)f_12627},
{C_text("f_12630:c_2dbackend_2escm"),(void*)f_12630},
{C_text("f_12633:c_2dbackend_2escm"),(void*)f_12633},
{C_text("f_12645:c_2dbackend_2escm"),(void*)f_12645},
{C_text("f_12651:c_2dbackend_2escm"),(void*)f_12651},
{C_text("f_12654:c_2dbackend_2escm"),(void*)f_12654},
{C_text("f_12657:c_2dbackend_2escm"),(void*)f_12657},
{C_text("f_12669:c_2dbackend_2escm"),(void*)f_12669},
{C_text("f_12675:c_2dbackend_2escm"),(void*)f_12675},
{C_text("f_12678:c_2dbackend_2escm"),(void*)f_12678},
{C_text("f_12681:c_2dbackend_2escm"),(void*)f_12681},
{C_text("f_12702:c_2dbackend_2escm"),(void*)f_12702},
{C_text("f_12706:c_2dbackend_2escm"),(void*)f_12706},
{C_text("f_12738:c_2dbackend_2escm"),(void*)f_12738},
{C_text("f_12744:c_2dbackend_2escm"),(void*)f_12744},
{C_text("f_12747:c_2dbackend_2escm"),(void*)f_12747},
{C_text("f_12750:c_2dbackend_2escm"),(void*)f_12750},
{C_text("f_12762:c_2dbackend_2escm"),(void*)f_12762},
{C_text("f_12768:c_2dbackend_2escm"),(void*)f_12768},
{C_text("f_12771:c_2dbackend_2escm"),(void*)f_12771},
{C_text("f_12774:c_2dbackend_2escm"),(void*)f_12774},
{C_text("f_12786:c_2dbackend_2escm"),(void*)f_12786},
{C_text("f_12792:c_2dbackend_2escm"),(void*)f_12792},
{C_text("f_12795:c_2dbackend_2escm"),(void*)f_12795},
{C_text("f_12798:c_2dbackend_2escm"),(void*)f_12798},
{C_text("f_12810:c_2dbackend_2escm"),(void*)f_12810},
{C_text("f_12816:c_2dbackend_2escm"),(void*)f_12816},
{C_text("f_12819:c_2dbackend_2escm"),(void*)f_12819},
{C_text("f_12822:c_2dbackend_2escm"),(void*)f_12822},
{C_text("f_12834:c_2dbackend_2escm"),(void*)f_12834},
{C_text("f_12840:c_2dbackend_2escm"),(void*)f_12840},
{C_text("f_12843:c_2dbackend_2escm"),(void*)f_12843},
{C_text("f_12846:c_2dbackend_2escm"),(void*)f_12846},
{C_text("f_12874:c_2dbackend_2escm"),(void*)f_12874},
{C_text("f_12880:c_2dbackend_2escm"),(void*)f_12880},
{C_text("f_12883:c_2dbackend_2escm"),(void*)f_12883},
{C_text("f_12886:c_2dbackend_2escm"),(void*)f_12886},
{C_text("f_12898:c_2dbackend_2escm"),(void*)f_12898},
{C_text("f_12904:c_2dbackend_2escm"),(void*)f_12904},
{C_text("f_12907:c_2dbackend_2escm"),(void*)f_12907},
{C_text("f_12910:c_2dbackend_2escm"),(void*)f_12910},
{C_text("f_12922:c_2dbackend_2escm"),(void*)f_12922},
{C_text("f_12928:c_2dbackend_2escm"),(void*)f_12928},
{C_text("f_12931:c_2dbackend_2escm"),(void*)f_12931},
{C_text("f_12934:c_2dbackend_2escm"),(void*)f_12934},
{C_text("f_13156:c_2dbackend_2escm"),(void*)f_13156},
{C_text("f_13165:c_2dbackend_2escm"),(void*)f_13165},
{C_text("f_13231:c_2dbackend_2escm"),(void*)f_13231},
{C_text("f_13253:c_2dbackend_2escm"),(void*)f_13253},
{C_text("f_13281:c_2dbackend_2escm"),(void*)f_13281},
{C_text("f_13340:c_2dbackend_2escm"),(void*)f_13340},
{C_text("f_13347:c_2dbackend_2escm"),(void*)f_13347},
{C_text("f_13367:c_2dbackend_2escm"),(void*)f_13367},
{C_text("f_13374:c_2dbackend_2escm"),(void*)f_13374},
{C_text("f_13387:c_2dbackend_2escm"),(void*)f_13387},
{C_text("f_13394:c_2dbackend_2escm"),(void*)f_13394},
{C_text("f_13417:c_2dbackend_2escm"),(void*)f_13417},
{C_text("f_13425:c_2dbackend_2escm"),(void*)f_13425},
{C_text("f_13447:c_2dbackend_2escm"),(void*)f_13447},
{C_text("f_13455:c_2dbackend_2escm"),(void*)f_13455},
{C_text("f_13459:c_2dbackend_2escm"),(void*)f_13459},
{C_text("f_13461:c_2dbackend_2escm"),(void*)f_13461},
{C_text("f_13490:c_2dbackend_2escm"),(void*)f_13490},
{C_text("f_3245:c_2dbackend_2escm"),(void*)f_3245},
{C_text("f_3248:c_2dbackend_2escm"),(void*)f_3248},
{C_text("f_3251:c_2dbackend_2escm"),(void*)f_3251},
{C_text("f_3254:c_2dbackend_2escm"),(void*)f_3254},
{C_text("f_3257:c_2dbackend_2escm"),(void*)f_3257},
{C_text("f_3260:c_2dbackend_2escm"),(void*)f_3260},
{C_text("f_3263:c_2dbackend_2escm"),(void*)f_3263},
{C_text("f_3266:c_2dbackend_2escm"),(void*)f_3266},
{C_text("f_3707:c_2dbackend_2escm"),(void*)f_3707},
{C_text("f_3721:c_2dbackend_2escm"),(void*)f_3721},
{C_text("f_4279:c_2dbackend_2escm"),(void*)f_4279},
{C_text("f_4285:c_2dbackend_2escm"),(void*)f_4285},
{C_text("f_4299:c_2dbackend_2escm"),(void*)f_4299},
{C_text("f_4303:c_2dbackend_2escm"),(void*)f_4303},
{C_text("f_4459:c_2dbackend_2escm"),(void*)f_4459},
{C_text("f_4471:c_2dbackend_2escm"),(void*)f_4471},
{C_text("f_4547:c_2dbackend_2escm"),(void*)f_4547},
{C_text("f_4567:c_2dbackend_2escm"),(void*)f_4567},
{C_text("f_4577:c_2dbackend_2escm"),(void*)f_4577},
{C_text("f_4590:c_2dbackend_2escm"),(void*)f_4590},
{C_text("f_4599:c_2dbackend_2escm"),(void*)f_4599},
{C_text("f_4604:c_2dbackend_2escm"),(void*)f_4604},
{C_text("f_4614:c_2dbackend_2escm"),(void*)f_4614},
{C_text("f_4655:c_2dbackend_2escm"),(void*)f_4655},
{C_text("f_4661:c_2dbackend_2escm"),(void*)f_4661},
{C_text("f_4666:c_2dbackend_2escm"),(void*)f_4666},
{C_text("f_4674:c_2dbackend_2escm"),(void*)f_4674},
{C_text("f_4678:c_2dbackend_2escm"),(void*)f_4678},
{C_text("f_4688:c_2dbackend_2escm"),(void*)f_4688},
{C_text("f_4699:c_2dbackend_2escm"),(void*)f_4699},
{C_text("f_4703:c_2dbackend_2escm"),(void*)f_4703},
{C_text("f_4706:c_2dbackend_2escm"),(void*)f_4706},
{C_text("f_4708:c_2dbackend_2escm"),(void*)f_4708},
{C_text("f_4712:c_2dbackend_2escm"),(void*)f_4712},
{C_text("f_4720:c_2dbackend_2escm"),(void*)f_4720},
{C_text("f_4723:c_2dbackend_2escm"),(void*)f_4723},
{C_text("f_4727:c_2dbackend_2escm"),(void*)f_4727},
{C_text("f_4730:c_2dbackend_2escm"),(void*)f_4730},
{C_text("f_4733:c_2dbackend_2escm"),(void*)f_4733},
{C_text("f_4860:c_2dbackend_2escm"),(void*)f_4860},
{C_text("f_4874:c_2dbackend_2escm"),(void*)f_4874},
{C_text("f_4893:c_2dbackend_2escm"),(void*)f_4893},
{C_text("f_4896:c_2dbackend_2escm"),(void*)f_4896},
{C_text("f_4899:c_2dbackend_2escm"),(void*)f_4899},
{C_text("f_4902:c_2dbackend_2escm"),(void*)f_4902},
{C_text("f_4905:c_2dbackend_2escm"),(void*)f_4905},
{C_text("f_4908:c_2dbackend_2escm"),(void*)f_4908},
{C_text("f_4951:c_2dbackend_2escm"),(void*)f_4951},
{C_text("f_4961:c_2dbackend_2escm"),(void*)f_4961},
{C_text("f_4964:c_2dbackend_2escm"),(void*)f_4964},
{C_text("f_4967:c_2dbackend_2escm"),(void*)f_4967},
{C_text("f_4999:c_2dbackend_2escm"),(void*)f_4999},
{C_text("f_5002:c_2dbackend_2escm"),(void*)f_5002},
{C_text("f_5005:c_2dbackend_2escm"),(void*)f_5005},
{C_text("f_5038:c_2dbackend_2escm"),(void*)f_5038},
{C_text("f_5041:c_2dbackend_2escm"),(void*)f_5041},
{C_text("f_5057:c_2dbackend_2escm"),(void*)f_5057},
{C_text("f_5060:c_2dbackend_2escm"),(void*)f_5060},
{C_text("f_5076:c_2dbackend_2escm"),(void*)f_5076},
{C_text("f_5079:c_2dbackend_2escm"),(void*)f_5079},
{C_text("f_5103:c_2dbackend_2escm"),(void*)f_5103},
{C_text("f_5109:c_2dbackend_2escm"),(void*)f_5109},
{C_text("f_5128:c_2dbackend_2escm"),(void*)f_5128},
{C_text("f_5131:c_2dbackend_2escm"),(void*)f_5131},
{C_text("f_5143:c_2dbackend_2escm"),(void*)f_5143},
{C_text("f_5149:c_2dbackend_2escm"),(void*)f_5149},
{C_text("f_5168:c_2dbackend_2escm"),(void*)f_5168},
{C_text("f_5171:c_2dbackend_2escm"),(void*)f_5171},
{C_text("f_5183:c_2dbackend_2escm"),(void*)f_5183},
{C_text("f_5189:c_2dbackend_2escm"),(void*)f_5189},
{C_text("f_5208:c_2dbackend_2escm"),(void*)f_5208},
{C_text("f_5211:c_2dbackend_2escm"),(void*)f_5211},
{C_text("f_5223:c_2dbackend_2escm"),(void*)f_5223},
{C_text("f_5226:c_2dbackend_2escm"),(void*)f_5226},
{C_text("f_5242:c_2dbackend_2escm"),(void*)f_5242},
{C_text("f_5245:c_2dbackend_2escm"),(void*)f_5245},
{C_text("f_5248:c_2dbackend_2escm"),(void*)f_5248},
{C_text("f_5251:c_2dbackend_2escm"),(void*)f_5251},
{C_text("f_5275:c_2dbackend_2escm"),(void*)f_5275},
{C_text("f_5278:c_2dbackend_2escm"),(void*)f_5278},
{C_text("f_5281:c_2dbackend_2escm"),(void*)f_5281},
{C_text("f_5284:c_2dbackend_2escm"),(void*)f_5284},
{C_text("f_5312:c_2dbackend_2escm"),(void*)f_5312},
{C_text("f_5315:c_2dbackend_2escm"),(void*)f_5315},
{C_text("f_5318:c_2dbackend_2escm"),(void*)f_5318},
{C_text("f_5321:c_2dbackend_2escm"),(void*)f_5321},
{C_text("f_5341:c_2dbackend_2escm"),(void*)f_5341},
{C_text("f_5344:c_2dbackend_2escm"),(void*)f_5344},
{C_text("f_5347:c_2dbackend_2escm"),(void*)f_5347},
{C_text("f_5350:c_2dbackend_2escm"),(void*)f_5350},
{C_text("f_5373:c_2dbackend_2escm"),(void*)f_5373},
{C_text("f_5374:c_2dbackend_2escm"),(void*)f_5374},
{C_text("f_5378:c_2dbackend_2escm"),(void*)f_5378},
{C_text("f_5381:c_2dbackend_2escm"),(void*)f_5381},
{C_text("f_5387:c_2dbackend_2escm"),(void*)f_5387},
{C_text("f_5396:c_2dbackend_2escm"),(void*)f_5396},
{C_text("f_5403:c_2dbackend_2escm"),(void*)f_5403},
{C_text("f_5413:c_2dbackend_2escm"),(void*)f_5413},
{C_text("f_5447:c_2dbackend_2escm"),(void*)f_5447},
{C_text("f_5450:c_2dbackend_2escm"),(void*)f_5450},
{C_text("f_5479:c_2dbackend_2escm"),(void*)f_5479},
{C_text("f_5521:c_2dbackend_2escm"),(void*)f_5521},
{C_text("f_5525:c_2dbackend_2escm"),(void*)f_5525},
{C_text("f_5556:c_2dbackend_2escm"),(void*)f_5556},
{C_text("f_5559:c_2dbackend_2escm"),(void*)f_5559},
{C_text("f_5562:c_2dbackend_2escm"),(void*)f_5562},
{C_text("f_5573:c_2dbackend_2escm"),(void*)f_5573},
{C_text("f_5577:c_2dbackend_2escm"),(void*)f_5577},
{C_text("f_5604:c_2dbackend_2escm"),(void*)f_5604},
{C_text("f_5607:c_2dbackend_2escm"),(void*)f_5607},
{C_text("f_5618:c_2dbackend_2escm"),(void*)f_5618},
{C_text("f_5622:c_2dbackend_2escm"),(void*)f_5622},
{C_text("f_5625:c_2dbackend_2escm"),(void*)f_5625},
{C_text("f_5628:c_2dbackend_2escm"),(void*)f_5628},
{C_text("f_5639:c_2dbackend_2escm"),(void*)f_5639},
{C_text("f_5643:c_2dbackend_2escm"),(void*)f_5643},
{C_text("f_5680:c_2dbackend_2escm"),(void*)f_5680},
{C_text("f_5683:c_2dbackend_2escm"),(void*)f_5683},
{C_text("f_5689:c_2dbackend_2escm"),(void*)f_5689},
{C_text("f_5693:c_2dbackend_2escm"),(void*)f_5693},
{C_text("f_5702:c_2dbackend_2escm"),(void*)f_5702},
{C_text("f_5705:c_2dbackend_2escm"),(void*)f_5705},
{C_text("f_5708:c_2dbackend_2escm"),(void*)f_5708},
{C_text("f_5724:c_2dbackend_2escm"),(void*)f_5724},
{C_text("f_5727:c_2dbackend_2escm"),(void*)f_5727},
{C_text("f_5730:c_2dbackend_2escm"),(void*)f_5730},
{C_text("f_5731:c_2dbackend_2escm"),(void*)f_5731},
{C_text("f_5735:c_2dbackend_2escm"),(void*)f_5735},
{C_text("f_5738:c_2dbackend_2escm"),(void*)f_5738},
{C_text("f_5747:c_2dbackend_2escm"),(void*)f_5747},
{C_text("f_5755:c_2dbackend_2escm"),(void*)f_5755},
{C_text("f_5761:c_2dbackend_2escm"),(void*)f_5761},
{C_text("f_5764:c_2dbackend_2escm"),(void*)f_5764},
{C_text("f_5772:c_2dbackend_2escm"),(void*)f_5772},
{C_text("f_5782:c_2dbackend_2escm"),(void*)f_5782},
{C_text("f_5809:c_2dbackend_2escm"),(void*)f_5809},
{C_text("f_5819:c_2dbackend_2escm"),(void*)f_5819},
{C_text("f_5846:c_2dbackend_2escm"),(void*)f_5846},
{C_text("f_5853:c_2dbackend_2escm"),(void*)f_5853},
{C_text("f_5859:c_2dbackend_2escm"),(void*)f_5859},
{C_text("f_5862:c_2dbackend_2escm"),(void*)f_5862},
{C_text("f_5865:c_2dbackend_2escm"),(void*)f_5865},
{C_text("f_5874:c_2dbackend_2escm"),(void*)f_5874},
{C_text("f_5877:c_2dbackend_2escm"),(void*)f_5877},
{C_text("f_5880:c_2dbackend_2escm"),(void*)f_5880},
{C_text("f_5883:c_2dbackend_2escm"),(void*)f_5883},
{C_text("f_5893:c_2dbackend_2escm"),(void*)f_5893},
{C_text("f_5904:c_2dbackend_2escm"),(void*)f_5904},
{C_text("f_5908:c_2dbackend_2escm"),(void*)f_5908},
{C_text("f_5911:c_2dbackend_2escm"),(void*)f_5911},
{C_text("f_5924:c_2dbackend_2escm"),(void*)f_5924},
{C_text("f_5930:c_2dbackend_2escm"),(void*)f_5930},
{C_text("f_5933:c_2dbackend_2escm"),(void*)f_5933},
{C_text("f_5945:c_2dbackend_2escm"),(void*)f_5945},
{C_text("f_5948:c_2dbackend_2escm"),(void*)f_5948},
{C_text("f_5951:c_2dbackend_2escm"),(void*)f_5951},
{C_text("f_5954:c_2dbackend_2escm"),(void*)f_5954},
{C_text("f_5961:c_2dbackend_2escm"),(void*)f_5961},
{C_text("f_5971:c_2dbackend_2escm"),(void*)f_5971},
{C_text("f_5979:c_2dbackend_2escm"),(void*)f_5979},
{C_text("f_5987:c_2dbackend_2escm"),(void*)f_5987},
{C_text("f_6000:c_2dbackend_2escm"),(void*)f_6000},
{C_text("f_6004:c_2dbackend_2escm"),(void*)f_6004},
{C_text("f_6012:c_2dbackend_2escm"),(void*)f_6012},
{C_text("f_6020:c_2dbackend_2escm"),(void*)f_6020},
{C_text("f_6027:c_2dbackend_2escm"),(void*)f_6027},
{C_text("f_6032:c_2dbackend_2escm"),(void*)f_6032},
{C_text("f_6039:c_2dbackend_2escm"),(void*)f_6039},
{C_text("f_6043:c_2dbackend_2escm"),(void*)f_6043},
{C_text("f_6046:c_2dbackend_2escm"),(void*)f_6046},
{C_text("f_6049:c_2dbackend_2escm"),(void*)f_6049},
{C_text("f_6052:c_2dbackend_2escm"),(void*)f_6052},
{C_text("f_6055:c_2dbackend_2escm"),(void*)f_6055},
{C_text("f_6058:c_2dbackend_2escm"),(void*)f_6058},
{C_text("f_6064:c_2dbackend_2escm"),(void*)f_6064},
{C_text("f_6077:c_2dbackend_2escm"),(void*)f_6077},
{C_text("f_6082:c_2dbackend_2escm"),(void*)f_6082},
{C_text("f_6105:c_2dbackend_2escm"),(void*)f_6105},
{C_text("f_6109:c_2dbackend_2escm"),(void*)f_6109},
{C_text("f_6112:c_2dbackend_2escm"),(void*)f_6112},
{C_text("f_6132:c_2dbackend_2escm"),(void*)f_6132},
{C_text("f_6139:c_2dbackend_2escm"),(void*)f_6139},
{C_text("f_6143:c_2dbackend_2escm"),(void*)f_6143},
{C_text("f_6147:c_2dbackend_2escm"),(void*)f_6147},
{C_text("f_6190:c_2dbackend_2escm"),(void*)f_6190},
{C_text("f_6196:c_2dbackend_2escm"),(void*)f_6196},
{C_text("f_6199:c_2dbackend_2escm"),(void*)f_6199},
{C_text("f_6200:c_2dbackend_2escm"),(void*)f_6200},
{C_text("f_6204:c_2dbackend_2escm"),(void*)f_6204},
{C_text("f_6207:c_2dbackend_2escm"),(void*)f_6207},
{C_text("f_6216:c_2dbackend_2escm"),(void*)f_6216},
{C_text("f_6224:c_2dbackend_2escm"),(void*)f_6224},
{C_text("f_6230:c_2dbackend_2escm"),(void*)f_6230},
{C_text("f_6235:c_2dbackend_2escm"),(void*)f_6235},
{C_text("f_6245:c_2dbackend_2escm"),(void*)f_6245},
{C_text("f_6272:c_2dbackend_2escm"),(void*)f_6272},
{C_text("f_6282:c_2dbackend_2escm"),(void*)f_6282},
{C_text("f_6309:c_2dbackend_2escm"),(void*)f_6309},
{C_text("f_6316:c_2dbackend_2escm"),(void*)f_6316},
{C_text("f_6319:c_2dbackend_2escm"),(void*)f_6319},
{C_text("f_6322:c_2dbackend_2escm"),(void*)f_6322},
{C_text("f_6350:c_2dbackend_2escm"),(void*)f_6350},
{C_text("f_6363:c_2dbackend_2escm"),(void*)f_6363},
{C_text("f_6367:c_2dbackend_2escm"),(void*)f_6367},
{C_text("f_6370:c_2dbackend_2escm"),(void*)f_6370},
{C_text("f_6373:c_2dbackend_2escm"),(void*)f_6373},
{C_text("f_6376:c_2dbackend_2escm"),(void*)f_6376},
{C_text("f_6379:c_2dbackend_2escm"),(void*)f_6379},
{C_text("f_6382:c_2dbackend_2escm"),(void*)f_6382},
{C_text("f_6385:c_2dbackend_2escm"),(void*)f_6385},
{C_text("f_6397:c_2dbackend_2escm"),(void*)f_6397},
{C_text("f_6409:c_2dbackend_2escm"),(void*)f_6409},
{C_text("f_6427:c_2dbackend_2escm"),(void*)f_6427},
{C_text("f_6447:c_2dbackend_2escm"),(void*)f_6447},
{C_text("f_6454:c_2dbackend_2escm"),(void*)f_6454},
{C_text("f_6458:c_2dbackend_2escm"),(void*)f_6458},
{C_text("f_6487:c_2dbackend_2escm"),(void*)f_6487},
{C_text("f_6490:c_2dbackend_2escm"),(void*)f_6490},
{C_text("f_6497:c_2dbackend_2escm"),(void*)f_6497},
{C_text("f_6510:c_2dbackend_2escm"),(void*)f_6510},
{C_text("f_6513:c_2dbackend_2escm"),(void*)f_6513},
{C_text("f_6529:c_2dbackend_2escm"),(void*)f_6529},
{C_text("f_6532:c_2dbackend_2escm"),(void*)f_6532},
{C_text("f_6565:c_2dbackend_2escm"),(void*)f_6565},
{C_text("f_6568:c_2dbackend_2escm"),(void*)f_6568},
{C_text("f_6577:c_2dbackend_2escm"),(void*)f_6577},
{C_text("f_6601:c_2dbackend_2escm"),(void*)f_6601},
{C_text("f_6621:c_2dbackend_2escm"),(void*)f_6621},
{C_text("f_6624:c_2dbackend_2escm"),(void*)f_6624},
{C_text("f_6639:c_2dbackend_2escm"),(void*)f_6639},
{C_text("f_6643:c_2dbackend_2escm"),(void*)f_6643},
{C_text("f_6655:c_2dbackend_2escm"),(void*)f_6655},
{C_text("f_6658:c_2dbackend_2escm"),(void*)f_6658},
{C_text("f_6669:c_2dbackend_2escm"),(void*)f_6669},
{C_text("f_6673:c_2dbackend_2escm"),(void*)f_6673},
{C_text("f_6685:c_2dbackend_2escm"),(void*)f_6685},
{C_text("f_6688:c_2dbackend_2escm"),(void*)f_6688},
{C_text("f_6691:c_2dbackend_2escm"),(void*)f_6691},
{C_text("f_6694:c_2dbackend_2escm"),(void*)f_6694},
{C_text("f_6705:c_2dbackend_2escm"),(void*)f_6705},
{C_text("f_6713:c_2dbackend_2escm"),(void*)f_6713},
{C_text("f_6722:c_2dbackend_2escm"),(void*)f_6722},
{C_text("f_6725:c_2dbackend_2escm"),(void*)f_6725},
{C_text("f_6728:c_2dbackend_2escm"),(void*)f_6728},
{C_text("f_6741:c_2dbackend_2escm"),(void*)f_6741},
{C_text("f_6749:c_2dbackend_2escm"),(void*)f_6749},
{C_text("f_6752:c_2dbackend_2escm"),(void*)f_6752},
{C_text("f_6762:c_2dbackend_2escm"),(void*)f_6762},
{C_text("f_6765:c_2dbackend_2escm"),(void*)f_6765},
{C_text("f_6768:c_2dbackend_2escm"),(void*)f_6768},
{C_text("f_6771:c_2dbackend_2escm"),(void*)f_6771},
{C_text("f_6801:c_2dbackend_2escm"),(void*)f_6801},
{C_text("f_6804:c_2dbackend_2escm"),(void*)f_6804},
{C_text("f_6807:c_2dbackend_2escm"),(void*)f_6807},
{C_text("f_6810:c_2dbackend_2escm"),(void*)f_6810},
{C_text("f_6813:c_2dbackend_2escm"),(void*)f_6813},
{C_text("f_6816:c_2dbackend_2escm"),(void*)f_6816},
{C_text("f_6838:c_2dbackend_2escm"),(void*)f_6838},
{C_text("f_6924:c_2dbackend_2escm"),(void*)f_6924},
{C_text("f_6930:c_2dbackend_2escm"),(void*)f_6930},
{C_text("f_6940:c_2dbackend_2escm"),(void*)f_6940},
{C_text("f_6943:c_2dbackend_2escm"),(void*)f_6943},
{C_text("f_6960:c_2dbackend_2escm"),(void*)f_6960},
{C_text("f_6966:c_2dbackend_2escm"),(void*)f_6966},
{C_text("f_6985:c_2dbackend_2escm"),(void*)f_6985},
{C_text("f_6997:c_2dbackend_2escm"),(void*)f_6997},
{C_text("f_7001:c_2dbackend_2escm"),(void*)f_7001},
{C_text("f_7003:c_2dbackend_2escm"),(void*)f_7003},
{C_text("f_7015:c_2dbackend_2escm"),(void*)f_7015},
{C_text("f_7018:c_2dbackend_2escm"),(void*)f_7018},
{C_text("f_7021:c_2dbackend_2escm"),(void*)f_7021},
{C_text("f_7024:c_2dbackend_2escm"),(void*)f_7024},
{C_text("f_7027:c_2dbackend_2escm"),(void*)f_7027},
{C_text("f_7036:c_2dbackend_2escm"),(void*)f_7036},
{C_text("f_7046:c_2dbackend_2escm"),(void*)f_7046},
{C_text("f_7049:c_2dbackend_2escm"),(void*)f_7049},
{C_text("f_7052:c_2dbackend_2escm"),(void*)f_7052},
{C_text("f_7072:c_2dbackend_2escm"),(void*)f_7072},
{C_text("f_7087:c_2dbackend_2escm"),(void*)f_7087},
{C_text("f_7090:c_2dbackend_2escm"),(void*)f_7090},
{C_text("f_7093:c_2dbackend_2escm"),(void*)f_7093},
{C_text("f_7096:c_2dbackend_2escm"),(void*)f_7096},
{C_text("f_7099:c_2dbackend_2escm"),(void*)f_7099},
{C_text("f_7121:c_2dbackend_2escm"),(void*)f_7121},
{C_text("f_7124:c_2dbackend_2escm"),(void*)f_7124},
{C_text("f_7132:c_2dbackend_2escm"),(void*)f_7132},
{C_text("f_7136:c_2dbackend_2escm"),(void*)f_7136},
{C_text("f_7139:c_2dbackend_2escm"),(void*)f_7139},
{C_text("f_7142:c_2dbackend_2escm"),(void*)f_7142},
{C_text("f_7145:c_2dbackend_2escm"),(void*)f_7145},
{C_text("f_7148:c_2dbackend_2escm"),(void*)f_7148},
{C_text("f_7151:c_2dbackend_2escm"),(void*)f_7151},
{C_text("f_7154:c_2dbackend_2escm"),(void*)f_7154},
{C_text("f_7157:c_2dbackend_2escm"),(void*)f_7157},
{C_text("f_7169:c_2dbackend_2escm"),(void*)f_7169},
{C_text("f_7179:c_2dbackend_2escm"),(void*)f_7179},
{C_text("f_7189:c_2dbackend_2escm"),(void*)f_7189},
{C_text("f_7209:c_2dbackend_2escm"),(void*)f_7209},
{C_text("f_7222:c_2dbackend_2escm"),(void*)f_7222},
{C_text("f_7234:c_2dbackend_2escm"),(void*)f_7234},
{C_text("f_7237:c_2dbackend_2escm"),(void*)f_7237},
{C_text("f_7239:c_2dbackend_2escm"),(void*)f_7239},
{C_text("f_7264:c_2dbackend_2escm"),(void*)f_7264},
{C_text("f_7275:c_2dbackend_2escm"),(void*)f_7275},
{C_text("f_7285:c_2dbackend_2escm"),(void*)f_7285},
{C_text("f_7289:c_2dbackend_2escm"),(void*)f_7289},
{C_text("f_7291:c_2dbackend_2escm"),(void*)f_7291},
{C_text("f_7298:c_2dbackend_2escm"),(void*)f_7298},
{C_text("f_7312:c_2dbackend_2escm"),(void*)f_7312},
{C_text("f_7318:c_2dbackend_2escm"),(void*)f_7318},
{C_text("f_7321:c_2dbackend_2escm"),(void*)f_7321},
{C_text("f_7324:c_2dbackend_2escm"),(void*)f_7324},
{C_text("f_7329:c_2dbackend_2escm"),(void*)f_7329},
{C_text("f_7339:c_2dbackend_2escm"),(void*)f_7339},
{C_text("f_7345:c_2dbackend_2escm"),(void*)f_7345},
{C_text("f_7348:c_2dbackend_2escm"),(void*)f_7348},
{C_text("f_7351:c_2dbackend_2escm"),(void*)f_7351},
{C_text("f_7354:c_2dbackend_2escm"),(void*)f_7354},
{C_text("f_7369:c_2dbackend_2escm"),(void*)f_7369},
{C_text("f_7377:c_2dbackend_2escm"),(void*)f_7377},
{C_text("f_7387:c_2dbackend_2escm"),(void*)f_7387},
{C_text("f_7397:c_2dbackend_2escm"),(void*)f_7397},
{C_text("f_7438:c_2dbackend_2escm"),(void*)f_7438},
{C_text("f_7448:c_2dbackend_2escm"),(void*)f_7448},
{C_text("f_7461:c_2dbackend_2escm"),(void*)f_7461},
{C_text("f_7486:c_2dbackend_2escm"),(void*)f_7486},
{C_text("f_7495:c_2dbackend_2escm"),(void*)f_7495},
{C_text("f_7499:c_2dbackend_2escm"),(void*)f_7499},
{C_text("f_7508:c_2dbackend_2escm"),(void*)f_7508},
{C_text("f_7511:c_2dbackend_2escm"),(void*)f_7511},
{C_text("f_7514:c_2dbackend_2escm"),(void*)f_7514},
{C_text("f_7517:c_2dbackend_2escm"),(void*)f_7517},
{C_text("f_7520:c_2dbackend_2escm"),(void*)f_7520},
{C_text("f_7523:c_2dbackend_2escm"),(void*)f_7523},
{C_text("f_7526:c_2dbackend_2escm"),(void*)f_7526},
{C_text("f_7529:c_2dbackend_2escm"),(void*)f_7529},
{C_text("f_7532:c_2dbackend_2escm"),(void*)f_7532},
{C_text("f_7535:c_2dbackend_2escm"),(void*)f_7535},
{C_text("f_7538:c_2dbackend_2escm"),(void*)f_7538},
{C_text("f_7541:c_2dbackend_2escm"),(void*)f_7541},
{C_text("f_7544:c_2dbackend_2escm"),(void*)f_7544},
{C_text("f_7547:c_2dbackend_2escm"),(void*)f_7547},
{C_text("f_7568:c_2dbackend_2escm"),(void*)f_7568},
{C_text("f_7591:c_2dbackend_2escm"),(void*)f_7591},
{C_text("f_7594:c_2dbackend_2escm"),(void*)f_7594},
{C_text("f_7597:c_2dbackend_2escm"),(void*)f_7597},
{C_text("f_7600:c_2dbackend_2escm"),(void*)f_7600},
{C_text("f_7616:c_2dbackend_2escm"),(void*)f_7616},
{C_text("f_7619:c_2dbackend_2escm"),(void*)f_7619},
{C_text("f_7622:c_2dbackend_2escm"),(void*)f_7622},
{C_text("f_7633:c_2dbackend_2escm"),(void*)f_7633},
{C_text("f_7642:c_2dbackend_2escm"),(void*)f_7642},
{C_text("f_7650:c_2dbackend_2escm"),(void*)f_7650},
{C_text("f_7660:c_2dbackend_2escm"),(void*)f_7660},
{C_text("f_7673:c_2dbackend_2escm"),(void*)f_7673},
{C_text("f_7684:c_2dbackend_2escm"),(void*)f_7684},
{C_text("f_7694:c_2dbackend_2escm"),(void*)f_7694},
{C_text("f_7710:c_2dbackend_2escm"),(void*)f_7710},
{C_text("f_7713:c_2dbackend_2escm"),(void*)f_7713},
{C_text("f_7716:c_2dbackend_2escm"),(void*)f_7716},
{C_text("f_7719:c_2dbackend_2escm"),(void*)f_7719},
{C_text("f_7728:c_2dbackend_2escm"),(void*)f_7728},
{C_text("f_7731:c_2dbackend_2escm"),(void*)f_7731},
{C_text("f_7734:c_2dbackend_2escm"),(void*)f_7734},
{C_text("f_7737:c_2dbackend_2escm"),(void*)f_7737},
{C_text("f_7740:c_2dbackend_2escm"),(void*)f_7740},
{C_text("f_7743:c_2dbackend_2escm"),(void*)f_7743},
{C_text("f_7750:c_2dbackend_2escm"),(void*)f_7750},
{C_text("f_7757:c_2dbackend_2escm"),(void*)f_7757},
{C_text("f_7763:c_2dbackend_2escm"),(void*)f_7763},
{C_text("f_7771:c_2dbackend_2escm"),(void*)f_7771},
{C_text("f_7781:c_2dbackend_2escm"),(void*)f_7781},
{C_text("f_7794:c_2dbackend_2escm"),(void*)f_7794},
{C_text("f_7800:c_2dbackend_2escm"),(void*)f_7800},
{C_text("f_7810:c_2dbackend_2escm"),(void*)f_7810},
{C_text("f_7825:c_2dbackend_2escm"),(void*)f_7825},
{C_text("f_7831:c_2dbackend_2escm"),(void*)f_7831},
{C_text("f_7834:c_2dbackend_2escm"),(void*)f_7834},
{C_text("f_7837:c_2dbackend_2escm"),(void*)f_7837},
{C_text("f_7840:c_2dbackend_2escm"),(void*)f_7840},
{C_text("f_7848:c_2dbackend_2escm"),(void*)f_7848},
{C_text("f_7855:c_2dbackend_2escm"),(void*)f_7855},
{C_text("f_7861:c_2dbackend_2escm"),(void*)f_7861},
{C_text("f_7891:c_2dbackend_2escm"),(void*)f_7891},
{C_text("f_7904:c_2dbackend_2escm"),(void*)f_7904},
{C_text("f_7908:c_2dbackend_2escm"),(void*)f_7908},
{C_text("f_7932:c_2dbackend_2escm"),(void*)f_7932},
{C_text("f_7935:c_2dbackend_2escm"),(void*)f_7935},
{C_text("f_7941:c_2dbackend_2escm"),(void*)f_7941},
{C_text("f_7943:c_2dbackend_2escm"),(void*)f_7943},
{C_text("f_7967:c_2dbackend_2escm"),(void*)f_7967},
{C_text("f_7992:c_2dbackend_2escm"),(void*)f_7992},
{C_text("f_8006:c_2dbackend_2escm"),(void*)f_8006},
{C_text("f_8024:c_2dbackend_2escm"),(void*)f_8024},
{C_text("f_8031:c_2dbackend_2escm"),(void*)f_8031},
{C_text("f_8052:c_2dbackend_2escm"),(void*)f_8052},
{C_text("f_8072:c_2dbackend_2escm"),(void*)f_8072},
{C_text("f_8093:c_2dbackend_2escm"),(void*)f_8093},
{C_text("f_8097:c_2dbackend_2escm"),(void*)f_8097},
{C_text("f_8118:c_2dbackend_2escm"),(void*)f_8118},
{C_text("f_8122:c_2dbackend_2escm"),(void*)f_8122},
{C_text("f_8131:c_2dbackend_2escm"),(void*)f_8131},
{C_text("f_8138:c_2dbackend_2escm"),(void*)f_8138},
{C_text("f_8144:c_2dbackend_2escm"),(void*)f_8144},
{C_text("f_8188:c_2dbackend_2escm"),(void*)f_8188},
{C_text("f_8197:c_2dbackend_2escm"),(void*)f_8197},
{C_text("f_8206:c_2dbackend_2escm"),(void*)f_8206},
{C_text("f_8212:c_2dbackend_2escm"),(void*)f_8212},
{C_text("f_8239:c_2dbackend_2escm"),(void*)f_8239},
{C_text("f_8242:c_2dbackend_2escm"),(void*)f_8242},
{C_text("f_8249:c_2dbackend_2escm"),(void*)f_8249},
{C_text("f_8283:c_2dbackend_2escm"),(void*)f_8283},
{C_text("f_8285:c_2dbackend_2escm"),(void*)f_8285},
{C_text("f_8299:c_2dbackend_2escm"),(void*)f_8299},
{C_text("f_8320:c_2dbackend_2escm"),(void*)f_8320},
{C_text("f_8324:c_2dbackend_2escm"),(void*)f_8324},
{C_text("f_8336:c_2dbackend_2escm"),(void*)f_8336},
{C_text("f_8349:c_2dbackend_2escm"),(void*)f_8349},
{C_text("f_8353:c_2dbackend_2escm"),(void*)f_8353},
{C_text("f_8359:c_2dbackend_2escm"),(void*)f_8359},
{C_text("f_8366:c_2dbackend_2escm"),(void*)f_8366},
{C_text("f_8369:c_2dbackend_2escm"),(void*)f_8369},
{C_text("f_8371:c_2dbackend_2escm"),(void*)f_8371},
{C_text("f_8373:c_2dbackend_2escm"),(void*)f_8373},
{C_text("f_8381:c_2dbackend_2escm"),(void*)f_8381},
{C_text("f_8384:c_2dbackend_2escm"),(void*)f_8384},
{C_text("f_8387:c_2dbackend_2escm"),(void*)f_8387},
{C_text("f_8390:c_2dbackend_2escm"),(void*)f_8390},
{C_text("f_8393:c_2dbackend_2escm"),(void*)f_8393},
{C_text("f_8396:c_2dbackend_2escm"),(void*)f_8396},
{C_text("f_8399:c_2dbackend_2escm"),(void*)f_8399},
{C_text("f_8405:c_2dbackend_2escm"),(void*)f_8405},
{C_text("f_8408:c_2dbackend_2escm"),(void*)f_8408},
{C_text("f_8411:c_2dbackend_2escm"),(void*)f_8411},
{C_text("f_8414:c_2dbackend_2escm"),(void*)f_8414},
{C_text("f_8417:c_2dbackend_2escm"),(void*)f_8417},
{C_text("f_8420:c_2dbackend_2escm"),(void*)f_8420},
{C_text("f_8423:c_2dbackend_2escm"),(void*)f_8423},
{C_text("f_8426:c_2dbackend_2escm"),(void*)f_8426},
{C_text("f_8429:c_2dbackend_2escm"),(void*)f_8429},
{C_text("f_8432:c_2dbackend_2escm"),(void*)f_8432},
{C_text("f_8435:c_2dbackend_2escm"),(void*)f_8435},
{C_text("f_8438:c_2dbackend_2escm"),(void*)f_8438},
{C_text("f_8441:c_2dbackend_2escm"),(void*)f_8441},
{C_text("f_8444:c_2dbackend_2escm"),(void*)f_8444},
{C_text("f_8447:c_2dbackend_2escm"),(void*)f_8447},
{C_text("f_8450:c_2dbackend_2escm"),(void*)f_8450},
{C_text("f_8453:c_2dbackend_2escm"),(void*)f_8453},
{C_text("f_8456:c_2dbackend_2escm"),(void*)f_8456},
{C_text("f_8459:c_2dbackend_2escm"),(void*)f_8459},
{C_text("f_8462:c_2dbackend_2escm"),(void*)f_8462},
{C_text("f_8465:c_2dbackend_2escm"),(void*)f_8465},
{C_text("f_8468:c_2dbackend_2escm"),(void*)f_8468},
{C_text("f_8471:c_2dbackend_2escm"),(void*)f_8471},
{C_text("f_8474:c_2dbackend_2escm"),(void*)f_8474},
{C_text("f_8477:c_2dbackend_2escm"),(void*)f_8477},
{C_text("f_8480:c_2dbackend_2escm"),(void*)f_8480},
{C_text("f_8484:c_2dbackend_2escm"),(void*)f_8484},
{C_text("f_8491:c_2dbackend_2escm"),(void*)f_8491},
{C_text("f_8502:c_2dbackend_2escm"),(void*)f_8502},
{C_text("f_8508:c_2dbackend_2escm"),(void*)f_8508},
{C_text("f_8511:c_2dbackend_2escm"),(void*)f_8511},
{C_text("f_8520:c_2dbackend_2escm"),(void*)f_8520},
{C_text("f_8528:c_2dbackend_2escm"),(void*)f_8528},
{C_text("f_8542:c_2dbackend_2escm"),(void*)f_8542},
{C_text("f_8557:c_2dbackend_2escm"),(void*)f_8557},
{C_text("f_8560:c_2dbackend_2escm"),(void*)f_8560},
{C_text("f_8577:c_2dbackend_2escm"),(void*)f_8577},
{C_text("f_8587:c_2dbackend_2escm"),(void*)f_8587},
{C_text("f_8617:c_2dbackend_2escm"),(void*)f_8617},
{C_text("f_8621:c_2dbackend_2escm"),(void*)f_8621},
{C_text("f_8624:c_2dbackend_2escm"),(void*)f_8624},
{C_text("f_8627:c_2dbackend_2escm"),(void*)f_8627},
{C_text("f_8630:c_2dbackend_2escm"),(void*)f_8630},
{C_text("f_8633:c_2dbackend_2escm"),(void*)f_8633},
{C_text("f_8636:c_2dbackend_2escm"),(void*)f_8636},
{C_text("f_8645:c_2dbackend_2escm"),(void*)f_8645},
{C_text("f_8648:c_2dbackend_2escm"),(void*)f_8648},
{C_text("f_8659:c_2dbackend_2escm"),(void*)f_8659},
{C_text("f_8680:c_2dbackend_2escm"),(void*)f_8680},
{C_text("f_8696:c_2dbackend_2escm"),(void*)f_8696},
{C_text("f_8704:c_2dbackend_2escm"),(void*)f_8704},
{C_text("f_8707:c_2dbackend_2escm"),(void*)f_8707},
{C_text("f_8717:c_2dbackend_2escm"),(void*)f_8717},
{C_text("f_8720:c_2dbackend_2escm"),(void*)f_8720},
{C_text("f_8723:c_2dbackend_2escm"),(void*)f_8723},
{C_text("f_8732:c_2dbackend_2escm"),(void*)f_8732},
{C_text("f_8756:c_2dbackend_2escm"),(void*)f_8756},
{C_text("f_8759:c_2dbackend_2escm"),(void*)f_8759},
{C_text("f_8762:c_2dbackend_2escm"),(void*)f_8762},
{C_text("f_8765:c_2dbackend_2escm"),(void*)f_8765},
{C_text("f_8774:c_2dbackend_2escm"),(void*)f_8774},
{C_text("f_8790:c_2dbackend_2escm"),(void*)f_8790},
{C_text("f_8850:c_2dbackend_2escm"),(void*)f_8850},
{C_text("f_8863:c_2dbackend_2escm"),(void*)f_8863},
{C_text("f_8873:c_2dbackend_2escm"),(void*)f_8873},
{C_text("f_8888:c_2dbackend_2escm"),(void*)f_8888},
{C_text("f_8890:c_2dbackend_2escm"),(void*)f_8890},
{C_text("f_8898:c_2dbackend_2escm"),(void*)f_8898},
{C_text("f_8918:c_2dbackend_2escm"),(void*)f_8918},
{C_text("f_8928:c_2dbackend_2escm"),(void*)f_8928},
{C_text("f_8952:c_2dbackend_2escm"),(void*)f_8952},
{C_text("f_8975:c_2dbackend_2escm"),(void*)f_8975},
{C_text("f_8978:c_2dbackend_2escm"),(void*)f_8978},
{C_text("f_8981:c_2dbackend_2escm"),(void*)f_8981},
{C_text("f_8997:c_2dbackend_2escm"),(void*)f_8997},
{C_text("f_9000:c_2dbackend_2escm"),(void*)f_9000},
{C_text("f_9014:c_2dbackend_2escm"),(void*)f_9014},
{C_text("f_9039:c_2dbackend_2escm"),(void*)f_9039},
{C_text("f_9043:c_2dbackend_2escm"),(void*)f_9043},
{C_text("f_9051:c_2dbackend_2escm"),(void*)f_9051},
{C_text("f_9061:c_2dbackend_2escm"),(void*)f_9061},
{C_text("f_9075:c_2dbackend_2escm"),(void*)f_9075},
{C_text("f_9079:c_2dbackend_2escm"),(void*)f_9079},
{C_text("f_9082:c_2dbackend_2escm"),(void*)f_9082},
{C_text("f_9085:c_2dbackend_2escm"),(void*)f_9085},
{C_text("f_9088:c_2dbackend_2escm"),(void*)f_9088},
{C_text("f_9091:c_2dbackend_2escm"),(void*)f_9091},
{C_text("f_9094:c_2dbackend_2escm"),(void*)f_9094},
{C_text("f_9097:c_2dbackend_2escm"),(void*)f_9097},
{C_text("f_9100:c_2dbackend_2escm"),(void*)f_9100},
{C_text("f_9103:c_2dbackend_2escm"),(void*)f_9103},
{C_text("f_9106:c_2dbackend_2escm"),(void*)f_9106},
{C_text("f_9121:c_2dbackend_2escm"),(void*)f_9121},
{C_text("f_9126:c_2dbackend_2escm"),(void*)f_9126},
{C_text("f_9144:c_2dbackend_2escm"),(void*)f_9144},
{C_text("f_9148:c_2dbackend_2escm"),(void*)f_9148},
{C_text("f_9157:c_2dbackend_2escm"),(void*)f_9157},
{C_text("f_9162:c_2dbackend_2escm"),(void*)f_9162},
{C_text("f_9172:c_2dbackend_2escm"),(void*)f_9172},
{C_text("f_9190:c_2dbackend_2escm"),(void*)f_9190},
{C_text("f_9196:c_2dbackend_2escm"),(void*)f_9196},
{C_text("f_9201:c_2dbackend_2escm"),(void*)f_9201},
{C_text("f_9211:c_2dbackend_2escm"),(void*)f_9211},
{C_text("f_9224:c_2dbackend_2escm"),(void*)f_9224},
{C_text("f_9242:c_2dbackend_2escm"),(void*)f_9242},
{C_text("f_9243:c_2dbackend_2escm"),(void*)f_9243},
{C_text("f_9251:c_2dbackend_2escm"),(void*)f_9251},
{C_text("f_9264:c_2dbackend_2escm"),(void*)f_9264},
{C_text("f_9271:c_2dbackend_2escm"),(void*)f_9271},
{C_text("f_9274:c_2dbackend_2escm"),(void*)f_9274},
{C_text("f_9277:c_2dbackend_2escm"),(void*)f_9277},
{C_text("f_9280:c_2dbackend_2escm"),(void*)f_9280},
{C_text("f_9283:c_2dbackend_2escm"),(void*)f_9283},
{C_text("f_9288:c_2dbackend_2escm"),(void*)f_9288},
{C_text("f_9298:c_2dbackend_2escm"),(void*)f_9298},
{C_text("f_9316:c_2dbackend_2escm"),(void*)f_9316},
{C_text("f_9330:c_2dbackend_2escm"),(void*)f_9330},
{C_text("f_9332:c_2dbackend_2escm"),(void*)f_9332},
{C_text("f_9341:c_2dbackend_2escm"),(void*)f_9341},
{C_text("f_9367:c_2dbackend_2escm"),(void*)f_9367},
{C_text("f_9377:c_2dbackend_2escm"),(void*)f_9377},
{C_text("f_9408:c_2dbackend_2escm"),(void*)f_9408},
{C_text("f_9414:c_2dbackend_2escm"),(void*)f_9414},
{C_text("f_9422:c_2dbackend_2escm"),(void*)f_9422},
{C_text("f_9425:c_2dbackend_2escm"),(void*)f_9425},
{C_text("f_9431:c_2dbackend_2escm"),(void*)f_9431},
{C_text("f_9439:c_2dbackend_2escm"),(void*)f_9439},
{C_text("f_9446:c_2dbackend_2escm"),(void*)f_9446},
{C_text("f_9468:c_2dbackend_2escm"),(void*)f_9468},
{C_text("f_9476:c_2dbackend_2escm"),(void*)f_9476},
{C_text("f_9486:c_2dbackend_2escm"),(void*)f_9486},
{C_text("f_9499:c_2dbackend_2escm"),(void*)f_9499},
{C_text("f_9505:c_2dbackend_2escm"),(void*)f_9505},
{C_text("f_9508:c_2dbackend_2escm"),(void*)f_9508},
{C_text("f_9519:c_2dbackend_2escm"),(void*)f_9519},
{C_text("f_9529:c_2dbackend_2escm"),(void*)f_9529},
{C_text("f_9544:c_2dbackend_2escm"),(void*)f_9544},
{C_text("f_9548:c_2dbackend_2escm"),(void*)f_9548},
{C_text("f_9551:c_2dbackend_2escm"),(void*)f_9551},
{C_text("f_9554:c_2dbackend_2escm"),(void*)f_9554},
{C_text("f_9560:c_2dbackend_2escm"),(void*)f_9560},
{C_text("f_9563:c_2dbackend_2escm"),(void*)f_9563},
{C_text("f_9566:c_2dbackend_2escm"),(void*)f_9566},
{C_text("f_9569:c_2dbackend_2escm"),(void*)f_9569},
{C_text("f_9572:c_2dbackend_2escm"),(void*)f_9572},
{C_text("f_9575:c_2dbackend_2escm"),(void*)f_9575},
{C_text("f_9578:c_2dbackend_2escm"),(void*)f_9578},
{C_text("f_9581:c_2dbackend_2escm"),(void*)f_9581},
{C_text("f_9584:c_2dbackend_2escm"),(void*)f_9584},
{C_text("f_9587:c_2dbackend_2escm"),(void*)f_9587},
{C_text("f_9590:c_2dbackend_2escm"),(void*)f_9590},
{C_text("f_9593:c_2dbackend_2escm"),(void*)f_9593},
{C_text("f_9596:c_2dbackend_2escm"),(void*)f_9596},
{C_text("f_9605:c_2dbackend_2escm"),(void*)f_9605},
{C_text("f_9609:c_2dbackend_2escm"),(void*)f_9609},
{C_text("f_9613:c_2dbackend_2escm"),(void*)f_9613},
{C_text("f_9617:c_2dbackend_2escm"),(void*)f_9617},
{C_text("f_9623:c_2dbackend_2escm"),(void*)f_9623},
{C_text("f_9629:c_2dbackend_2escm"),(void*)f_9629},
{C_text("f_9632:c_2dbackend_2escm"),(void*)f_9632},
{C_text("f_9638:c_2dbackend_2escm"),(void*)f_9638},
{C_text("f_9647:c_2dbackend_2escm"),(void*)f_9647},
{C_text("f_9650:c_2dbackend_2escm"),(void*)f_9650},
{C_text("f_9653:c_2dbackend_2escm"),(void*)f_9653},
{C_text("f_9659:c_2dbackend_2escm"),(void*)f_9659},
{C_text("f_9662:c_2dbackend_2escm"),(void*)f_9662},
{C_text("f_9680:c_2dbackend_2escm"),(void*)f_9680},
{C_text("f_9683:c_2dbackend_2escm"),(void*)f_9683},
{C_text("f_9686:c_2dbackend_2escm"),(void*)f_9686},
{C_text("f_9689:c_2dbackend_2escm"),(void*)f_9689},
{C_text("f_9692:c_2dbackend_2escm"),(void*)f_9692},
{C_text("f_9717:c_2dbackend_2escm"),(void*)f_9717},
{C_text("f_9721:c_2dbackend_2escm"),(void*)f_9721},
{C_text("f_9739:c_2dbackend_2escm"),(void*)f_9739},
{C_text("f_9746:c_2dbackend_2escm"),(void*)f_9746},
{C_text("f_9749:c_2dbackend_2escm"),(void*)f_9749},
{C_text("f_9791:c_2dbackend_2escm"),(void*)f_9791},
{C_text("f_9796:c_2dbackend_2escm"),(void*)f_9796},
{C_text("f_9806:c_2dbackend_2escm"),(void*)f_9806},
{C_text("f_9818:c_2dbackend_2escm"),(void*)f_9818},
{C_text("f_9821:c_2dbackend_2escm"),(void*)f_9821},
{C_text("f_9828:c_2dbackend_2escm"),(void*)f_9828},
{C_text("f_9836:c_2dbackend_2escm"),(void*)f_9836},
{C_text("f_9846:c_2dbackend_2escm"),(void*)f_9846},
{C_text("f_9857:c_2dbackend_2escm"),(void*)f_9857},
{C_text("f_9867:c_2dbackend_2escm"),(void*)f_9867},
{C_text("f_9882:c_2dbackend_2escm"),(void*)f_9882},
{C_text("f_9886:c_2dbackend_2escm"),(void*)f_9886},
{C_text("f_9889:c_2dbackend_2escm"),(void*)f_9889},
{C_text("f_9892:c_2dbackend_2escm"),(void*)f_9892},
{C_text("f_9895:c_2dbackend_2escm"),(void*)f_9895},
{C_text("f_9901:c_2dbackend_2escm"),(void*)f_9901},
{C_text("f_9903:c_2dbackend_2escm"),(void*)f_9903},
{C_text("f_9913:c_2dbackend_2escm"),(void*)f_9913},
{C_text("f_9922:c_2dbackend_2escm"),(void*)f_9922},
{C_text("f_9934:c_2dbackend_2escm"),(void*)f_9934},
{C_text("f_9946:c_2dbackend_2escm"),(void*)f_9946},
{C_text("f_9958:c_2dbackend_2escm"),(void*)f_9958},
{C_text("f_9970:c_2dbackend_2escm"),(void*)f_9970},
{C_text("f_9976:c_2dbackend_2escm"),(void*)f_9976},
{C_text("f_9980:c_2dbackend_2escm"),(void*)f_9980},
{C_text("toplevel:c_2dbackend_2escm"),(void*)C_c_2dbackend_toplevel},
{C_text("va15385:c_2dbackend_2escm"),(void*)va15385},
{C_text("va15391:c_2dbackend_2escm"),(void*)va15391},
{C_text("va15423:c_2dbackend_2escm"),(void*)va15423},
{C_text("va15425:c_2dbackend_2escm"),(void*)va15425},
{C_text("va15433:c_2dbackend_2escm"),(void*)va15433},
{C_text("va15435:c_2dbackend_2escm"),(void*)va15435},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.compiler.c-backend#partition 
o|hiding unexported module binding: chicken.compiler.c-backend#span 
o|hiding unexported module binding: chicken.compiler.c-backend#take 
o|hiding unexported module binding: chicken.compiler.c-backend#drop 
o|hiding unexported module binding: chicken.compiler.c-backend#split-at 
o|hiding unexported module binding: chicken.compiler.c-backend#append-map 
o|hiding unexported module binding: chicken.compiler.c-backend#every 
o|hiding unexported module binding: chicken.compiler.c-backend#any 
o|hiding unexported module binding: chicken.compiler.c-backend#cons* 
o|hiding unexported module binding: chicken.compiler.c-backend#concatenate 
o|hiding unexported module binding: chicken.compiler.c-backend#delete 
o|hiding unexported module binding: chicken.compiler.c-backend#first 
o|hiding unexported module binding: chicken.compiler.c-backend#second 
o|hiding unexported module binding: chicken.compiler.c-backend#third 
o|hiding unexported module binding: chicken.compiler.c-backend#fourth 
o|hiding unexported module binding: chicken.compiler.c-backend#fifth 
o|hiding unexported module binding: chicken.compiler.c-backend#delete-duplicates 
o|hiding unexported module binding: chicken.compiler.c-backend#alist-cons 
o|hiding unexported module binding: chicken.compiler.c-backend#filter 
o|hiding unexported module binding: chicken.compiler.c-backend#filter-map 
o|hiding unexported module binding: chicken.compiler.c-backend#remove 
o|hiding unexported module binding: chicken.compiler.c-backend#unzip1 
o|hiding unexported module binding: chicken.compiler.c-backend#last 
o|hiding unexported module binding: chicken.compiler.c-backend#list-index 
o|hiding unexported module binding: chicken.compiler.c-backend#lset-adjoin/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#lset-difference/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#lset-union/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#lset-intersection/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#list-tabulate 
o|hiding unexported module binding: chicken.compiler.c-backend#lset<=/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#lset=/eq? 
o|hiding unexported module binding: chicken.compiler.c-backend#length+ 
o|hiding unexported module binding: chicken.compiler.c-backend#find 
o|hiding unexported module binding: chicken.compiler.c-backend#find-tail 
o|hiding unexported module binding: chicken.compiler.c-backend#iota 
o|hiding unexported module binding: chicken.compiler.c-backend#make-list 
o|hiding unexported module binding: chicken.compiler.c-backend#posq 
o|hiding unexported module binding: chicken.compiler.c-backend#posv 
o|hiding unexported module binding: chicken.compiler.c-backend#output 
o|hiding unexported module binding: chicken.compiler.c-backend#gen 
o|hiding unexported module binding: chicken.compiler.c-backend#gen-list 
o|hiding unexported module binding: chicken.compiler.c-backend#backslashify 
o|hiding unexported module binding: chicken.compiler.c-backend#uncommentify 
o|hiding unexported module binding: chicken.compiler.c-backend#c-identifier 
o|hiding unexported module binding: chicken.compiler.c-backend#table->sorted-alist 
o|hiding unexported module binding: chicken.compiler.c-backend#emit-debug-table 
o|hiding unexported module binding: chicken.compiler.c-backend#emit-procedure-table 
o|hiding unexported module binding: chicken.compiler.c-backend#toplevel 
o|hiding unexported module binding: chicken.compiler.c-backend#cleanup 
o|hiding unexported module binding: chicken.compiler.c-backend#make-variable-list 
o|hiding unexported module binding: chicken.compiler.c-backend#make-argument-list 
o|hiding unexported module binding: chicken.compiler.c-backend#generate-external-variables 
o|hiding unexported module binding: chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes 
o|hiding unexported module binding: chicken.compiler.c-backend#generate-foreign-stubs 
o|hiding unexported module binding: chicken.compiler.c-backend#generate-foreign-callback-stubs 
o|hiding unexported module binding: chicken.compiler.c-backend#generate-foreign-callback-header 
o|hiding unexported module binding: chicken.compiler.c-backend#foreign-argument-conversion 
o|hiding unexported module binding: chicken.compiler.c-backend#foreign-result-conversion 
o|hiding unexported module binding: chicken.compiler.c-backend#encode-literal 
S|applied compiler syntax:
S|  chicken.format#sprintf		20
S|  scheme#for-each		22
S|  chicken.base#foldl		5
S|  scheme#map		8
S|  chicken.base#foldr		3
o|eliminated procedure checks: 160 
o|specializations:
o|  1 (scheme#number->string float)
o|  1 (scheme#string-length string)
o|  1 (scheme#number->string * *)
o|  1 (chicken.base#exact-integer? *)
o|  1 (scheme#cdddr (pair * (pair * pair)))
o|  3 (scheme#>= fixnum fixnum)
o|  1 (scheme#> fixnum fixnum)
o|  2 (scheme#memq * list)
o|  8 (scheme#= fixnum fixnum)
o|  1 (scheme#< integer integer)
o|  1 (chicken.base#sub1 fixnum)
o|  2 (scheme#char=? char char)
o|  1 (scheme#char>? char char)
o|  1 (scheme#char<? char char)
o|  4 (scheme#length list)
o|  1 (scheme#modulo fixnum fixnum)
o|  1 (scheme#imag-part cplxnum)
o|  1 (scheme#real-part cplxnum)
o|  1 (scheme#denominator ratnum)
o|  1 (scheme#numerator ratnum)
o|  1 (scheme#vector-length vector)
o|  20 (##sys#check-output-port * * *)
o|  1 (scheme#- integer integer)
o|  1 (chicken.bitwise#bitwise-and fixnum fixnum)
o|  4 (scheme#zero? integer)
o|  3 (scheme#string-append string string)
o|  9 (scheme#number->string *)
o|  14 (scheme#zero? *)
o|  7 (chicken.base#add1 fixnum)
o|  13 (chicken.base#sub1 *)
o|  12 (chicken.base#add1 *)
o|  1 (scheme#negative? *)
o|  1 (chicken.base#infinite? *)
o|  1 (chicken.base#nan? *)
o|  285 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  5 (scheme#+ fixnum fixnum)
o|  1 (scheme#eqv? * *)
o|  16 (##sys#check-list (or pair list) *)
o|  36 (scheme#cdr pair)
o|  13 (scheme#car pair)
(o e)|safe calls: 1578 
o|safe globals: (chicken.compiler.c-backend#encode-literal chicken.compiler.c-backend#foreign-result-conversion chicken.compiler.c-backend#foreign-argument-conversion chicken.compiler.c-backend#foreign-type-declaration chicken.compiler.c-backend#generate-foreign-callback-header chicken.compiler.c-backend#generate-foreign-callback-stubs chicken.compiler.c-backend#generate-foreign-stubs chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes chicken.compiler.c-backend#generate-external-variables chicken.compiler.c-backend#make-argument-list chicken.compiler.c-backend#make-variable-list chicken.compiler.c-backend#cleanup chicken.compiler.c-backend#toplevel chicken.compiler.c-backend#emit-procedure-table chicken.compiler.c-backend#emit-debug-table chicken.compiler.c-backend#generate-code chicken.compiler.c-backend#table->sorted-alist chicken.compiler.c-backend#c-identifier chicken.compiler.c-backend#uncommentify chicken.compiler.c-backend#backslashify chicken.compiler.c-backend#gen-list chicken.compiler.c-backend#gen chicken.compiler.c-backend#output chicken.compiler.c-backend#posv chicken.compiler.c-backend#posq chicken.compiler.c-backend#make-list chicken.compiler.c-backend#iota chicken.compiler.c-backend#find-tail chicken.compiler.c-backend#find chicken.compiler.c-backend#length+ chicken.compiler.c-backend#lset=/eq? chicken.compiler.c-backend#lset<=/eq? chicken.compiler.c-backend#list-tabulate chicken.compiler.c-backend#lset-intersection/eq? chicken.compiler.c-backend#lset-union/eq? chicken.compiler.c-backend#lset-difference/eq? chicken.compiler.c-backend#lset-adjoin/eq? chicken.compiler.c-backend#list-index chicken.compiler.c-backend#last chicken.compiler.c-backend#unzip1 chicken.compiler.c-backend#remove chicken.compiler.c-backend#filter-map chicken.compiler.c-backend#filter chicken.compiler.c-backend#alist-cons chicken.compiler.c-backend#delete-duplicates chicken.compiler.c-backend#fifth chicken.compiler.c-backend#fourth chicken.compiler.c-backend#third chicken.compiler.c-backend#second chicken.compiler.c-backend#first chicken.compiler.c-backend#delete chicken.compiler.c-backend#concatenate chicken.compiler.c-backend#cons* chicken.compiler.c-backend#any chicken.compiler.c-backend#every chicken.compiler.c-backend#append-map chicken.compiler.c-backend#split-at chicken.compiler.c-backend#drop chicken.compiler.c-backend#take chicken.compiler.c-backend#span chicken.compiler.c-backend#partition) 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#partition 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#span 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#drop 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#split-at 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#append-map 
o|inlining procedure: k3648 
o|inlining procedure: k3648 
o|inlining procedure: k3679 
o|inlining procedure: k3679 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#concatenate 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#delete-duplicates 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#alist-cons 
o|inlining procedure: k3896 
o|inlining procedure: k3896 
o|inlining procedure: k3888 
o|inlining procedure: k3888 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#filter-map 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#remove 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#unzip1 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#last 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#list-index 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset-adjoin/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset-difference/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset-union/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset-intersection/eq? 
o|inlining procedure: k4287 
o|inlining procedure: k4287 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset<=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#lset=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#length+ 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#find 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#find-tail 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#posq 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#posv 
o|inlining procedure: k4569 
o|contracted procedure: "(c-backend.scm:61) g643650" 
o|inlining procedure: k4551 
o|inlining procedure: k4551 
o|inlining procedure: k4569 
o|inlining procedure: k4606 
o|contracted procedure: "(c-backend.scm:69) g663670" 
o|inlining procedure: k4606 
o|inlining procedure: k4713 
o|inlining procedure: k4713 
o|inlining procedure: k4734 
o|inlining procedure: k4753 
o|inlining procedure: k4753 
o|inlining procedure: "(c-backend.scm:116) chicken.compiler.c-backend#second" 
o|inlining procedure: k4759 
o|inlining procedure: "(c-backend.scm:117) chicken.compiler.c-backend#second" 
o|inlining procedure: k4759 
o|inlining procedure: k4785 
o|inlining procedure: "(c-backend.scm:119) chicken.compiler.c-backend#second" 
o|inlining procedure: k4785 
o|substituted constant variable: a4811 
o|substituted constant variable: a4813 
o|substituted constant variable: a4815 
o|substituted constant variable: a4817 
o|substituted constant variable: a4819 
o|inlining procedure: "(c-backend.scm:115) chicken.compiler.c-backend#first" 
o|inlining procedure: k4734 
o|inlining procedure: k4829 
o|inlining procedure: k4829 
o|inlining procedure: "(c-backend.scm:127) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:124) chicken.compiler.c-backend#first" 
o|inlining procedure: k4849 
o|inlining procedure: k4868 
o|inlining procedure: k4868 
o|inlining procedure: "(c-backend.scm:130) chicken.compiler.c-backend#first" 
o|inlining procedure: k4849 
o|inlining procedure: k4924 
o|inlining procedure: "(c-backend.scm:148) chicken.compiler.c-backend#first" 
o|inlining procedure: k4924 
o|inlining procedure: k4953 
o|inlining procedure: k4953 
o|inlining procedure: "(c-backend.scm:151) chicken.compiler.c-backend#first" 
o|inlining procedure: k4988 
o|inlining procedure: "(c-backend.scm:164) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:162) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:160) chicken.compiler.c-backend#first" 
o|inlining procedure: k4988 
o|inlining procedure: "(c-backend.scm:167) chicken.compiler.c-backend#first" 
o|inlining procedure: k5030 
o|inlining procedure: "(c-backend.scm:171) chicken.compiler.c-backend#first" 
o|inlining procedure: k5030 
o|inlining procedure: "(c-backend.scm:176) chicken.compiler.c-backend#first" 
o|inlining procedure: k5068 
o|inlining procedure: "(c-backend.scm:182) chicken.compiler.c-backend#first" 
o|inlining procedure: k5068 
o|inlining procedure: k5110 
o|inlining procedure: k5110 
o|inlining procedure: k5129 
o|inlining procedure: k5129 
o|inlining procedure: "(c-backend.scm:186) chicken.compiler.c-backend#second" 
o|inlining procedure: k5135 
o|inlining procedure: k5169 
o|inlining procedure: k5169 
o|inlining procedure: "(c-backend.scm:195) chicken.compiler.c-backend#second" 
o|inlining procedure: k5135 
o|inlining procedure: k5190 
o|inlining procedure: k5190 
o|inlining procedure: k5209 
o|inlining procedure: k5209 
o|inlining procedure: "(c-backend.scm:204) chicken.compiler.c-backend#second" 
o|inlining procedure: k5215 
o|inlining procedure: k5215 
o|inlining procedure: "(c-backend.scm:219) chicken.compiler.c-backend#first" 
o|inlining procedure: k5267 
o|inlining procedure: "(c-backend.scm:226) chicken.compiler.c-backend#first" 
o|inlining procedure: k5267 
o|inlining procedure: k5333 
o|inlining procedure: k5333 
o|inlining procedure: k5405 
o|inlining procedure: k5405 
o|inlining procedure: "(c-backend.scm:245) chicken.compiler.c-backend#first" 
o|inlining procedure: k5439 
o|inlining procedure: k5439 
o|inlining procedure: "(c-backend.scm:260) chicken.compiler.c-backend#first" 
o|inlining procedure: k5471 
o|inlining procedure: "(c-backend.scm:263) chicken.compiler.c-backend#first" 
o|inlining procedure: k5471 
o|inlining procedure: k5506 
o|inlining procedure: "(c-backend.scm:275) chicken.compiler.c-backend#fourth" 
o|inlining procedure: k5506 
o|inlining procedure: "(c-backend.scm:269) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:268) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:267) chicken.compiler.c-backend#first" 
o|inlining procedure: k5539 
o|inlining procedure: "(c-backend.scm:282) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:281) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:280) chicken.compiler.c-backend#first" 
o|inlining procedure: k5539 
o|inlining procedure: k5599 
o|inlining procedure: k5599 
o|inlining procedure: "(c-backend.scm:293) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:292) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:291) chicken.compiler.c-backend#first" 
o|inlining procedure: k5644 
o|inlining procedure: k5644 
o|inlining procedure: k5694 
o|inlining procedure: "(c-backend.scm:332) chicken.compiler.c-backend#first" 
o|inlining procedure: k5694 
o|inlining procedure: k5719 
o|inlining procedure: k5774 
o|contracted procedure: "(c-backend.scm:344) g920943" 
o|inlining procedure: k5774 
o|inlining procedure: k5811 
o|inlining procedure: k5811 
o|inlining procedure: k5719 
o|inlining procedure: k5881 
o|inlining procedure: k5881 
o|inlining procedure: k5891 
o|substituted constant variable: a5900 
o|inlining procedure: k5891 
o|inlining procedure: k5925 
o|inlining procedure: k5981 
o|inlining procedure: "(c-backend.scm:386) chicken.compiler.c-backend#fourth" 
o|inlining procedure: k5981 
o|inlining procedure: "(c-backend.scm:372) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:371) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:370) chicken.compiler.c-backend#first" 
o|inlining procedure: k5925 
o|propagated global variable: tmp992994 chicken.compiler.support#unsafe 
o|propagated global variable: tmp992994 chicken.compiler.support#unsafe 
o|propagated global variable: tmp995997 chicken.compiler.core#no-procedure-checks 
o|inlining procedure: k6071 
o|propagated global variable: tmp995997 chicken.compiler.core#no-procedure-checks 
o|inlining procedure: k6071 
o|substituted constant variable: a6078 
o|inlining procedure: k6087 
o|inlining procedure: k6087 
o|inlining procedure: k6113 
o|inlining procedure: k6123 
o|inlining procedure: k6123 
o|inlining procedure: k6113 
o|inlining procedure: k6151 
o|inlining procedure: "(c-backend.scm:317) chicken.compiler.c-backend#fourth" 
o|inlining procedure: k6151 
o|inlining procedure: "(c-backend.scm:315) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:313) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:312) chicken.compiler.c-backend#first" 
o|inlining procedure: k6171 
o|inlining procedure: k6237 
o|contracted procedure: "(c-backend.scm:424) g10371060" 
o|inlining procedure: k6237 
o|inlining procedure: k6274 
o|inlining procedure: k6274 
o|inlining procedure: "(c-backend.scm:413) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:412) chicken.compiler.c-backend#first" 
o|inlining procedure: k6171 
o|inlining procedure: k6398 
o|inlining procedure: k6398 
o|inlining procedure: k6410 
o|inlining procedure: k6410 
o|inlining procedure: k6428 
o|inlining procedure: k6438 
o|inlining procedure: k6438 
o|inlining procedure: k6428 
o|inlining procedure: "(c-backend.scm:442) chicken.compiler.c-backend#fourth" 
o|inlining procedure: "(c-backend.scm:440) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:438) chicken.compiler.c-backend#first" 
o|inlining procedure: k6461 
o|inlining procedure: "(c-backend.scm:467) chicken.compiler.c-backend#first" 
o|inlining procedure: k6461 
o|substituted constant variable: a6484 
o|inlining procedure: "(c-backend.scm:476) chicken.compiler.c-backend#first" 
o|inlining procedure: k6502 
o|inlining procedure: "(c-backend.scm:480) chicken.compiler.c-backend#first" 
o|inlining procedure: k6502 
o|inlining procedure: "(c-backend.scm:484) chicken.compiler.c-backend#first" 
o|inlining procedure: k6540 
o|inlining procedure: k6554 
o|inlining procedure: k6554 
o|inlining procedure: "(c-backend.scm:489) chicken.compiler.c-backend#first" 
o|inlining procedure: k6540 
o|inlining procedure: k6566 
o|inlining procedure: k6566 
o|inlining procedure: "(c-backend.scm:493) chicken.compiler.c-backend#first" 
o|inlining procedure: k6589 
o|inlining procedure: "(c-backend.scm:501) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:501) chicken.compiler.c-backend#second" 
o|inlining procedure: k6589 
o|inlining procedure: "(c-backend.scm:506) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:505) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:504) chicken.compiler.c-backend#second" 
o|inlining procedure: k6644 
o|inlining procedure: "(c-backend.scm:512) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:510) chicken.compiler.c-backend#first" 
o|inlining procedure: k6644 
o|inlining procedure: "(c-backend.scm:520) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:518) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:516) chicken.compiler.c-backend#first" 
o|inlining procedure: k6714 
o|inlining procedure: k6743 
o|inlining procedure: k6743 
o|inlining procedure: "(c-backend.scm:527) chicken.compiler.c-backend#first" 
o|inlining procedure: "(c-backend.scm:525) chicken.compiler.c-backend#first" 
o|inlining procedure: k6714 
o|inlining procedure: "(c-backend.scm:544) chicken.compiler.c-backend#third" 
o|inlining procedure: "(c-backend.scm:542) chicken.compiler.c-backend#second" 
o|inlining procedure: "(c-backend.scm:540) chicken.compiler.c-backend#first" 
o|substituted constant variable: a6840 
o|substituted constant variable: a6842 
o|substituted constant variable: a6844 
o|substituted constant variable: a6846 
o|substituted constant variable: a6848 
o|substituted constant variable: a6850 
o|substituted constant variable: a6852 
o|substituted constant variable: a6854 
o|substituted constant variable: a6856 
o|substituted constant variable: a6858 
o|substituted constant variable: a6860 
o|substituted constant variable: a6862 
o|substituted constant variable: a6864 
o|substituted constant variable: a6866 
o|substituted constant variable: a6868 
o|substituted constant variable: a6870 
o|substituted constant variable: a6872 
o|substituted constant variable: a6874 
o|substituted constant variable: a6876 
o|substituted constant variable: a6878 
o|substituted constant variable: a6880 
o|substituted constant variable: a6882 
o|substituted constant variable: a6884 
o|substituted constant variable: a6886 
o|substituted constant variable: a6888 
o|substituted constant variable: a6890 
o|substituted constant variable: a6892 
o|substituted constant variable: a6894 
o|substituted constant variable: a6896 
o|substituted constant variable: a6898 
o|substituted constant variable: a6900 
o|substituted constant variable: a6902 
o|substituted constant variable: a6904 
o|substituted constant variable: a6906 
o|substituted constant variable: a6908 
o|substituted constant variable: a6910 
o|substituted constant variable: a6912 
o|substituted constant variable: a6914 
o|substituted constant variable: a6916 
o|substituted constant variable: a6918 
o|substituted constant variable: a6920 
o|substituted constant variable: a6922 
o|inlining procedure: k6932 
o|inlining procedure: k6932 
o|inlining procedure: k6968 
o|inlining procedure: k6968 
o|inlining procedure: k6986 
o|inlining procedure: k6986 
o|inlining procedure: k7038 
o|inlining procedure: k7038 
o|inlining procedure: k7076 
o|inlining procedure: k7076 
o|inlining procedure: k7106 
o|inlining procedure: k7106 
o|inlining procedure: k7122 
o|inlining procedure: k7122 
o|inlining procedure: k7158 
o|inlining procedure: k7158 
o|inlining procedure: k7181 
o|contracted procedure: "(c-backend.scm:622) g12421249" 
o|inlining procedure: k7181 
o|propagated global variable: g12481250 chicken.compiler.core#foreign-declarations 
o|inlining procedure: k7241 
o|contracted procedure: "(c-backend.scm:606) g12121221" 
o|inlining procedure: k7241 
o|inlining procedure: k7331 
o|inlining procedure: k7331 
o|inlining procedure: k7371 
o|inlining procedure: k7371 
o|substituted constant variable: a7385 
o|inlining procedure: k7389 
o|inlining procedure: k7389 
o|substituted constant variable: a7420 
o|inlining procedure: k7440 
o|contracted procedure: "(c-backend.scm:637) g12701278" 
o|inlining procedure: k7440 
o|inlining procedure: k7463 
o|inlining procedure: k7463 
o|propagated global variable: g12951299 chicken.compiler.core#used-units 
o|inlining procedure: k7652 
o|contracted procedure: "(c-backend.scm:665) g13391346" 
o|inlining procedure: k7545 
o|inlining procedure: k7545 
o|inlining procedure: k7569 
o|inlining procedure: k7569 
o|inlining procedure: k7598 
o|inlining procedure: k7598 
o|inlining procedure: k7611 
o|inlining procedure: k7611 
o|inlining procedure: k7635 
o|inlining procedure: k7635 
o|inlining procedure: k7652 
o|inlining procedure: k7720 
o|contracted procedure: "(c-backend.scm:725) restore1401" 
o|inlining procedure: k7686 
o|inlining procedure: k7686 
o|inlining procedure: k7720 
o|inlining procedure: k7773 
o|inlining procedure: k7773 
o|inlining procedure: k7802 
o|inlining procedure: k7802 
o|substituted constant variable: a7827 
o|substituted constant variable: a7828 
o|inlining procedure: k7850 
o|inlining procedure: k7850 
o|inlining procedure: k7862 
o|inlining procedure: k7862 
o|inlining procedure: k7874 
o|propagated global variable: r787514080 chicken.compiler.c-platform#words-per-flonum 
o|inlining procedure: k7874 
o|inlining procedure: k7886 
o|inlining procedure: k7886 
o|inlining procedure: k7913 
o|inlining procedure: k7945 
o|inlining procedure: k7945 
o|substituted constant variable: g14601463 
o|inlining procedure: k7969 
o|inlining procedure: k7969 
o|inlining procedure: k7913 
o|inlining procedure: k8007 
o|inlining procedure: "(c-backend.scm:754) bad-literal712" 
o|inlining procedure: k8007 
o|inlining procedure: k8019 
o|inlining procedure: k8019 
o|substituted constant variable: a8049 
o|inlining procedure: k8054 
o|inlining procedure: k8054 
o|inlining procedure: k8077 
o|inlining procedure: k8077 
o|inlining procedure: "(c-backend.scm:770) bad-literal712" 
o|inlining procedure: k8133 
o|inlining procedure: k8133 
o|inlining procedure: k8148 
o|inlining procedure: k8148 
o|inlining procedure: k8167 
o|inlining procedure: k8167 
o|inlining procedure: k8170 
o|inlining procedure: k8170 
o|inlining procedure: k8213 
o|inlining procedure: k8213 
o|inlining procedure: k8228 
o|inlining procedure: k8228 
o|inlining procedure: "(c-backend.scm:800) bad-literal712" 
o|substituted constant variable: a8294 
o|inlining procedure: k8301 
o|inlining procedure: k8301 
o|inlining procedure: k8493 
o|inlining procedure: k8493 
o|inlining procedure: k8503 
o|inlining procedure: k8522 
o|inlining procedure: k8522 
o|inlining procedure: k8503 
o|inlining procedure: k8579 
o|inlining procedure: k8579 
o|inlining procedure: k8634 
o|inlining procedure: k8634 
o|inlining procedure: k8660 
o|inlining procedure: k8660 
o|propagated global variable: tmp16571659 chicken.compiler.core#unit-name 
o|inlining procedure: k8676 
o|propagated global variable: tmp16571659 chicken.compiler.core#unit-name 
o|inlining procedure: k8676 
o|inlining procedure: k8682 
o|inlining procedure: k8682 
o|substituted constant variable: g16431646 
o|inlining procedure: k8712 
o|inlining procedure: k8739 
o|inlining procedure: k8739 
o|inlining procedure: k8712 
o|inlining procedure: k8779 
o|inlining procedure: k8779 
o|inlining procedure: k8791 
o|inlining procedure: k8791 
o|inlining procedure: k8809 
o|inlining procedure: k8809 
o|inlining procedure: k8833 
o|inlining procedure: k8833 
o|inlining procedure: k8865 
o|contracted procedure: "(c-backend.scm:880) g16211628" 
o|inlining procedure: k8865 
o|inlining procedure: k8892 
o|inlining procedure: k8892 
o|inlining procedure: k8907 
o|inlining procedure: k8907 
o|inlining procedure: k8920 
o|inlining procedure: k8920 
o|inlining procedure: k8953 
o|inlining procedure: k8953 
o|inlining procedure: k8979 
o|inlining procedure: k8979 
o|inlining procedure: k8992 
o|inlining procedure: k8992 
o|inlining procedure: k9019 
o|inlining procedure: k9019 
o|inlining procedure: k9026 
o|inlining procedure: k9026 
o|inlining procedure: k9033 
o|inlining procedure: k9033 
o|inlining procedure: k9053 
o|inlining procedure: k9053 
o|contracted procedure: "(c-backend.scm:987) trailer707" 
o|contracted procedure: "(c-backend.scm:986) chicken.compiler.c-backend#emit-procedure-table" 
o|inlining procedure: k9252 
o|inlining procedure: k9252 
o|inlining procedure: k9290 
o|inlining procedure: k9290 
o|contracted procedure: "(c-backend.scm:984) chicken.compiler.c-backend#emit-debug-table" 
o|inlining procedure: k9203 
o|contracted procedure: "(c-backend.scm:994) g17701777" 
o|inlining procedure: k9164 
o|contracted procedure: "(c-backend.scm:997) g17831790" 
o|inlining procedure: k9129 
o|inlining procedure: k9129 
o|inlining procedure: k9164 
o|inlining procedure: "(c-backend.scm:996) chicken.compiler.c-backend#second" 
o|inlining procedure: k9203 
o|contracted procedure: "(c-backend.scm:981) chicken.compiler.c-backend#generate-foreign-callback-stubs" 
o|inlining procedure: k9905 
o|inlining procedure: k9905 
o|inlining procedure: k9926 
o|inlining procedure: k9926 
o|inlining procedure: k9950 
o|inlining procedure: k9950 
o|inlining procedure: k9977 
o|inlining procedure: k9977 
o|inlining procedure: k9999 
o|inlining procedure: k9999 
o|substituted constant variable: a10025 
o|inlining procedure: k10029 
o|inlining procedure: k10029 
o|inlining procedure: k10041 
o|inlining procedure: k10041 
o|inlining procedure: k10053 
o|inlining procedure: k10053 
o|inlining procedure: k10065 
o|inlining procedure: k10065 
o|substituted constant variable: a10072 
o|substituted constant variable: a10074 
o|substituted constant variable: a10076 
o|substituted constant variable: a10078 
o|substituted constant variable: a10080 
o|substituted constant variable: a10082 
o|substituted constant variable: a10084 
o|substituted constant variable: a10086 
o|substituted constant variable: a10088 
o|inlining procedure: k10098 
o|inlining procedure: k10098 
o|inlining procedure: k10110 
o|inlining procedure: k10110 
o|substituted constant variable: a10117 
o|substituted constant variable: a10119 
o|substituted constant variable: a10121 
o|substituted constant variable: a10123 
o|substituted constant variable: a10125 
o|inlining procedure: k10129 
o|inlining procedure: k10129 
o|inlining procedure: k10141 
o|inlining procedure: k10141 
o|substituted constant variable: a10148 
o|substituted constant variable: a10150 
o|substituted constant variable: a10152 
o|substituted constant variable: a10154 
o|substituted constant variable: a10156 
o|inlining procedure: k10160 
o|inlining procedure: k10160 
o|substituted constant variable: a10173 
o|substituted constant variable: a10175 
o|substituted constant variable: a10177 
o|substituted constant variable: a10179 
o|inlining procedure: k10183 
o|inlining procedure: k10183 
o|inlining procedure: k10195 
o|inlining procedure: k10195 
o|inlining procedure: k10207 
o|inlining procedure: k10207 
o|substituted constant variable: a10214 
o|substituted constant variable: a10216 
o|substituted constant variable: a10218 
o|substituted constant variable: a10220 
o|substituted constant variable: a10222 
o|substituted constant variable: a10224 
o|substituted constant variable: a10226 
o|inlining procedure: k10230 
o|inlining procedure: k10230 
o|inlining procedure: k10242 
o|inlining procedure: k10242 
o|substituted constant variable: a10255 
o|substituted constant variable: a10257 
o|substituted constant variable: a10259 
o|substituted constant variable: a10261 
o|substituted constant variable: a10263 
o|substituted constant variable: a10265 
o|inlining procedure: k10269 
o|inlining procedure: k10269 
o|inlining procedure: k10281 
o|inlining procedure: k10281 
o|inlining procedure: k10293 
o|inlining procedure: k10293 
o|inlining procedure: k10305 
o|inlining procedure: k10305 
o|inlining procedure: k10317 
o|inlining procedure: k10317 
o|inlining procedure: k10329 
o|inlining procedure: k10329 
o|substituted constant variable: a10336 
o|substituted constant variable: a10338 
o|substituted constant variable: a10340 
o|substituted constant variable: a10342 
o|substituted constant variable: a10344 
o|substituted constant variable: a10346 
o|substituted constant variable: a10348 
o|substituted constant variable: a10350 
o|substituted constant variable: a10352 
o|substituted constant variable: a10354 
o|substituted constant variable: a10356 
o|substituted constant variable: a10358 
o|substituted constant variable: a10360 
o|inlining procedure: k10403 
o|inlining procedure: k10403 
o|inlining procedure: k10428 
o|contracted procedure: "(c-backend.scm:1215) g22092217" 
o|inlining procedure: k10428 
o|inlining procedure: k10463 
o|inlining procedure: k10463 
o|inlining procedure: k10479 
o|inlining procedure: k10479 
o|propagated global variable: stubs2027 chicken.compiler.support#foreign-callback-stubs 
o|inlining procedure: k10513 
o|inlining procedure: k10513 
o|contracted procedure: "(c-backend.scm:979) chicken.compiler.c-backend#generate-foreign-stubs" 
o|inlining procedure: k9651 
o|inlining procedure: k9669 
o|inlining procedure: k9669 
o|inlining procedure: k9651 
o|inlining procedure: k9699 
o|inlining procedure: k9699 
o|inlining procedure: k9741 
o|contracted procedure: "(c-backend.scm:1128) g19511971" 
o|inlining procedure: k9615 
o|substituted constant variable: a9625 
o|substituted constant variable: a9626 
o|inlining procedure: k9615 
o|inlining procedure: k9741 
o|inlining procedure: k9780 
o|inlining procedure: k9780 
o|contracted procedure: "(c-backend.scm:1136) chicken.compiler.c-backend#iota" 
o|inlining procedure: k9798 
o|inlining procedure: k9798 
o|contracted procedure: "(c-backend.scm:1106) chicken.compiler.c-backend#make-list" 
o|propagated global variable: stubs1916 chicken.compiler.core#foreign-lambda-stubs 
o|inlining procedure: k9859 
o|inlining procedure: k9859 
o|contracted procedure: "(c-backend.scm:978) chicken.compiler.c-backend#generate-external-variables" 
o|inlining procedure: k9478 
o|contracted procedure: "(c-backend.scm:1077) g18741881" 
o|inlining procedure: k9478 
o|propagated global variable: a9114 chicken.flonum#flonum-maximum-decimal-exponent 
o|substituted constant variable: a9115 
o|contracted procedure: "(c-backend.scm:94) chicken.compiler.c-backend#table->sorted-alist" 
o|inlining procedure: k9318 
o|inlining procedure: k9318 
o|contracted procedure: "(c-backend.scm:1038) chicken.compiler.c-backend#c-identifier" 
o|inlining procedure: k9343 
o|inlining procedure: k9343 
o|substituted constant variable: a9364 
o|inlining procedure: k9378 
o|inlining procedure: k9378 
o|substituted constant variable: a9385 
o|inlining procedure: k9386 
o|inlining procedure: k9386 
o|substituted constant variable: a9393 
o|inlining procedure: k9394 
o|substituted constant variable: a9400 
o|inlining procedure: k9394 
o|inlining procedure: k9521 
o|contracted procedure: "(c-backend.scm:1089) g18981905" 
o|inlining procedure: k9521 
o|inlining procedure: k10566 
o|inlining procedure: k10566 
o|inlining procedure: k10622 
o|inlining procedure: k10622 
o|inlining procedure: k10643 
o|inlining procedure: k10643 
o|inlining procedure: k10667 
o|inlining procedure: k10667 
o|inlining procedure: k10691 
o|inlining procedure: k10691 
o|inlining procedure: k10709 
o|inlining procedure: k10709 
o|inlining procedure: k10730 
o|inlining procedure: k10730 
o|inlining procedure: k10748 
o|inlining procedure: k10748 
o|inlining procedure: k10766 
o|inlining procedure: k10766 
o|inlining procedure: k10784 
o|inlining procedure: k10784 
o|inlining procedure: k10808 
o|inlining procedure: k10808 
o|inlining procedure: k10829 
o|inlining procedure: k10829 
o|inlining procedure: k10853 
o|inlining procedure: k10853 
o|inlining procedure: k10877 
o|inlining procedure: k10877 
o|inlining procedure: k10901 
o|inlining procedure: k10901 
o|inlining procedure: k10925 
o|inlining procedure: k10925 
o|inlining procedure: k10949 
o|inlining procedure: k10949 
o|inlining procedure: k10973 
o|inlining procedure: k10973 
o|inlining procedure: k11000 
o|inlining procedure: k11000 
o|inlining procedure: k11016 
o|inlining procedure: k11016 
o|inlining procedure: k11050 
o|inlining procedure: k11092 
o|contracted procedure: "(c-backend.scm:1308) g23962405" 
o|propagated global variable: g24132414 chicken.compiler.c-backend#foreign-type-declaration 
o|inlining procedure: k11092 
o|inlining procedure: k11050 
o|inlining procedure: k11144 
o|inlining procedure: k11144 
o|inlining procedure: k11178 
o|inlining procedure: k11178 
o|inlining procedure: k11212 
o|inlining procedure: k11212 
o|inlining procedure: k11282 
o|contracted procedure: "(c-backend.scm:1332) g24442453" 
o|inlining procedure: k11264 
o|inlining procedure: k11264 
o|inlining procedure: k11282 
o|substituted constant variable: a11333 
o|substituted constant variable: a11345 
o|substituted constant variable: a11353 
o|substituted constant variable: a11355 
o|substituted constant variable: a11367 
o|substituted constant variable: a11379 
o|substituted constant variable: a11391 
o|substituted constant variable: a11403 
o|substituted constant variable: a11415 
o|substituted constant variable: a11426 
o|substituted constant variable: a11435 
o|substituted constant variable: a11436 
o|substituted constant variable: a11448 
o|inlining procedure: k11452 
o|inlining procedure: k11452 
o|substituted constant variable: a11465 
o|substituted constant variable: a11467 
o|substituted constant variable: a11469 
o|substituted constant variable: a11471 
o|inlining procedure: k11475 
o|inlining procedure: k11475 
o|inlining procedure: k11487 
o|inlining procedure: k11487 
o|substituted constant variable: a11494 
o|substituted constant variable: a11496 
o|substituted constant variable: a11498 
o|substituted constant variable: a11500 
o|substituted constant variable: a11502 
o|substituted constant variable: a11507 
o|substituted constant variable: a11509 
o|substituted constant variable: a11514 
o|substituted constant variable: a11516 
o|substituted constant variable: a11521 
o|substituted constant variable: a11523 
o|substituted constant variable: a11528 
o|substituted constant variable: a11530 
o|substituted constant variable: a11535 
o|substituted constant variable: a11537 
o|substituted constant variable: a11542 
o|substituted constant variable: a11544 
o|substituted constant variable: a11549 
o|substituted constant variable: a11551 
o|substituted constant variable: a11556 
o|substituted constant variable: a11558 
o|substituted constant variable: a11563 
o|substituted constant variable: a11565 
o|substituted constant variable: a11570 
o|substituted constant variable: a11572 
o|inlining procedure: k11576 
o|inlining procedure: k11576 
o|substituted constant variable: a11589 
o|substituted constant variable: a11591 
o|substituted constant variable: a11593 
o|substituted constant variable: a11595 
o|substituted constant variable: a11600 
o|substituted constant variable: a11602 
o|inlining procedure: k11606 
o|inlining procedure: k11606 
o|substituted constant variable: a11619 
o|substituted constant variable: a11621 
o|substituted constant variable: a11623 
o|substituted constant variable: a11625 
o|substituted constant variable: a11630 
o|substituted constant variable: a11632 
o|substituted constant variable: a11634 
o|substituted constant variable: a11636 
o|substituted constant variable: a11638 
o|substituted constant variable: a11640 
o|substituted constant variable: a11642 
o|substituted constant variable: a11644 
o|substituted constant variable: a11646 
o|substituted constant variable: a11651 
o|substituted constant variable: a11653 
o|substituted constant variable: a11655 
o|substituted constant variable: a11657 
o|inlining procedure: k11661 
o|inlining procedure: k11661 
o|substituted constant variable: a11668 
o|substituted constant variable: a11670 
o|substituted constant variable: a11672 
o|substituted constant variable: a11677 
o|substituted constant variable: a11679 
o|substituted constant variable: a11684 
o|substituted constant variable: a11686 
o|substituted constant variable: a11691 
o|substituted constant variable: a11693 
o|substituted constant variable: a11698 
o|substituted constant variable: a11700 
o|substituted constant variable: a11702 
o|inlining procedure: k11711 
o|inlining procedure: k11711 
o|inlining procedure: k11726 
o|inlining procedure: k11726 
o|inlining procedure: k11741 
o|inlining procedure: k11741 
o|inlining procedure: k11753 
o|inlining procedure: k11753 
o|inlining procedure: k11771 
o|inlining procedure: k11771 
o|inlining procedure: k11783 
o|inlining procedure: k11783 
o|inlining procedure: k11795 
o|inlining procedure: k11795 
o|inlining procedure: k11810 
o|inlining procedure: k11810 
o|inlining procedure: k11822 
o|inlining procedure: k11822 
o|inlining procedure: k11834 
o|inlining procedure: k11834 
o|inlining procedure: k11846 
o|inlining procedure: k11846 
o|inlining procedure: k11858 
o|inlining procedure: k11858 
o|inlining procedure: k11870 
o|inlining procedure: k11870 
o|inlining procedure: k11882 
o|inlining procedure: k11882 
o|inlining procedure: k11894 
o|inlining procedure: k11894 
o|inlining procedure: k11906 
o|inlining procedure: k11906 
o|inlining procedure: k11918 
o|inlining procedure: k11918 
o|inlining procedure: k11930 
o|inlining procedure: k11930 
o|inlining procedure: k11942 
o|inlining procedure: k11942 
o|inlining procedure: k11954 
o|inlining procedure: k11954 
o|inlining procedure: k11966 
o|inlining procedure: k11966 
o|inlining procedure: k11978 
o|inlining procedure: k11978 
o|inlining procedure: k11996 
o|inlining procedure: k11996 
o|contracted procedure: "(c-backend.scm:1396) g25302531" 
o|inlining procedure: k12020 
o|inlining procedure: k12035 
o|inlining procedure: k12035 
o|inlining procedure: k12047 
o|inlining procedure: k12047 
o|inlining procedure: k12059 
o|inlining procedure: k12059 
o|inlining procedure: k12071 
o|inlining procedure: k12071 
o|inlining procedure: k12090 
o|inlining procedure: k12090 
o|substituted constant variable: a12124 
o|substituted constant variable: a12126 
o|substituted constant variable: a12128 
o|substituted constant variable: a12130 
o|substituted constant variable: a12132 
o|substituted constant variable: a12134 
o|substituted constant variable: a12136 
o|substituted constant variable: a12138 
o|substituted constant variable: a12140 
o|substituted constant variable: a12142 
o|substituted constant variable: a12144 
o|inlining procedure: k12020 
o|substituted constant variable: a12156 
o|substituted constant variable: a12164 
o|inlining procedure: k12168 
o|inlining procedure: k12168 
o|inlining procedure: k12180 
o|inlining procedure: k12180 
o|substituted constant variable: a12187 
o|substituted constant variable: a12189 
o|substituted constant variable: a12191 
o|substituted constant variable: a12193 
o|substituted constant variable: a12195 
o|inlining procedure: k12199 
o|inlining procedure: k12199 
o|substituted constant variable: a12212 
o|substituted constant variable: a12214 
o|substituted constant variable: a12216 
o|substituted constant variable: a12218 
o|substituted constant variable: a12220 
o|substituted constant variable: a12222 
o|substituted constant variable: a12224 
o|substituted constant variable: a12226 
o|substituted constant variable: a12228 
o|substituted constant variable: a12230 
o|substituted constant variable: a12232 
o|substituted constant variable: a12234 
o|substituted constant variable: a12236 
o|substituted constant variable: a12238 
o|substituted constant variable: a12240 
o|substituted constant variable: a12242 
o|substituted constant variable: a12244 
o|substituted constant variable: a12246 
o|substituted constant variable: a12248 
o|substituted constant variable: a12250 
o|substituted constant variable: a12252 
o|substituted constant variable: a12254 
o|substituted constant variable: a12256 
o|substituted constant variable: a12258 
o|substituted constant variable: a12260 
o|substituted constant variable: a12262 
o|substituted constant variable: a12264 
o|substituted constant variable: a12266 
o|substituted constant variable: a12268 
o|substituted constant variable: a12270 
o|substituted constant variable: a12272 
o|substituted constant variable: a12274 
o|substituted constant variable: a12279 
o|substituted constant variable: a12281 
o|substituted constant variable: a12283 
o|substituted constant variable: a12285 
o|substituted constant variable: a12287 
o|substituted constant variable: a12289 
o|substituted constant variable: a12291 
o|substituted constant variable: a12296 
o|substituted constant variable: a12298 
o|inlining procedure: k12302 
o|inlining procedure: k12302 
o|substituted constant variable: a12309 
o|substituted constant variable: a12311 
o|substituted constant variable: a12313 
o|substituted constant variable: a12315 
o|substituted constant variable: a12317 
o|substituted constant variable: a12319 
o|inlining procedure: k12323 
o|inlining procedure: k12323 
o|inlining procedure: k12335 
o|inlining procedure: k12335 
o|substituted constant variable: a12348 
o|substituted constant variable: a12350 
o|substituted constant variable: a12352 
o|substituted constant variable: a12354 
o|substituted constant variable: a12356 
o|substituted constant variable: a12358 
o|substituted constant variable: a12363 
o|substituted constant variable: a12365 
o|substituted constant variable: a12367 
o|inlining procedure: k12376 
o|inlining procedure: k12376 
o|inlining procedure: k12394 
o|inlining procedure: k12394 
o|inlining procedure: k12409 
o|inlining procedure: k12409 
o|inlining procedure: k12421 
o|inlining procedure: k12421 
o|substituted constant variable: a12440 
o|substituted constant variable: a12441 
o|substituted constant variable: a12464 
o|substituted constant variable: a12465 
o|inlining procedure: k12454 
o|inlining procedure: k12454 
o|substituted constant variable: a12491 
o|substituted constant variable: a12492 
o|substituted constant variable: a12515 
o|substituted constant variable: a12516 
o|inlining procedure: k12505 
o|inlining procedure: k12505 
o|substituted constant variable: a12542 
o|substituted constant variable: a12543 
o|substituted constant variable: a12569 
o|substituted constant variable: a12570 
o|inlining procedure: k12556 
o|inlining procedure: k12556 
o|substituted constant variable: a12596 
o|substituted constant variable: a12597 
o|substituted constant variable: a12623 
o|substituted constant variable: a12624 
o|inlining procedure: k12610 
o|inlining procedure: k12610 
o|substituted constant variable: a12647 
o|substituted constant variable: a12648 
o|substituted constant variable: a12671 
o|substituted constant variable: a12672 
o|inlining procedure: k12661 
o|inlining procedure: k12661 
o|inlining procedure: k12691 
o|inlining procedure: k12691 
o|inlining procedure: k12718 
o|substituted constant variable: a12740 
o|substituted constant variable: a12741 
o|substituted constant variable: a12764 
o|substituted constant variable: a12765 
o|inlining procedure: k12754 
o|inlining procedure: k12754 
o|substituted constant variable: a12788 
o|substituted constant variable: a12789 
o|substituted constant variable: a12812 
o|substituted constant variable: a12813 
o|inlining procedure: k12802 
o|inlining procedure: k12802 
o|substituted constant variable: a12836 
o|substituted constant variable: a12837 
o|inlining procedure: k12850 
o|inlining procedure: k12850 
o|substituted constant variable: a12876 
o|substituted constant variable: a12877 
o|substituted constant variable: a12900 
o|substituted constant variable: a12901 
o|inlining procedure: k12890 
o|inlining procedure: k12890 
o|substituted constant variable: a12924 
o|substituted constant variable: a12925 
o|substituted constant variable: a12942 
o|substituted constant variable: a12944 
o|substituted constant variable: a12949 
o|substituted constant variable: a12951 
o|substituted constant variable: a12953 
o|substituted constant variable: a12955 
o|substituted constant variable: a12957 
o|substituted constant variable: a12959 
o|substituted constant variable: a12961 
o|substituted constant variable: a12966 
o|substituted constant variable: a12968 
o|inlining procedure: k12718 
o|substituted constant variable: a12980 
o|substituted constant variable: a12991 
o|substituted constant variable: a12993 
o|substituted constant variable: a12995 
o|substituted constant variable: a12997 
o|substituted constant variable: a12999 
o|substituted constant variable: a13004 
o|substituted constant variable: a13006 
o|substituted constant variable: a13011 
o|substituted constant variable: a13013 
o|substituted constant variable: a13018 
o|substituted constant variable: a13020 
o|substituted constant variable: a13025 
o|substituted constant variable: a13027 
o|substituted constant variable: a13029 
o|inlining procedure: k13033 
o|inlining procedure: k13033 
o|inlining procedure: k13045 
o|inlining procedure: k13045 
o|inlining procedure: k13057 
o|inlining procedure: k13057 
o|inlining procedure: k13069 
o|inlining procedure: k13069 
o|inlining procedure: k13081 
o|inlining procedure: k13081 
o|substituted constant variable: a13094 
o|substituted constant variable: a13096 
o|substituted constant variable: a13098 
o|substituted constant variable: a13100 
o|substituted constant variable: a13102 
o|substituted constant variable: a13104 
o|substituted constant variable: a13106 
o|substituted constant variable: a13108 
o|substituted constant variable: a13110 
o|substituted constant variable: a13112 
o|substituted constant variable: a13114 
o|substituted constant variable: a13116 
o|substituted constant variable: a13118 
o|substituted constant variable: a13123 
o|substituted constant variable: a13125 
o|substituted constant variable: a13127 
o|substituted constant variable: a13129 
o|substituted constant variable: a13131 
o|substituted constant variable: a13133 
o|substituted constant variable: a13138 
o|substituted constant variable: a13140 
o|substituted constant variable: a13145 
o|substituted constant variable: a13147 
o|substituted constant variable: a13152 
o|substituted constant variable: a13154 
o|inlining procedure: k13167 
o|inlining procedure: k13167 
o|contracted procedure: "(c-backend.scm:1502) finish2854" 
o|inlining procedure: k13235 
o|inlining procedure: k13235 
o|inlining procedure: k13258 
o|inlining procedure: k13258 
o|inlining procedure: k13270 
o|inlining procedure: k13270 
o|substituted constant variable: a13285 
o|inlining procedure: k13334 
o|substituted constant variable: a13354 
o|inlining procedure: k13334 
o|inlining procedure: k13369 
o|inlining procedure: k13389 
o|inlining procedure: k13389 
o|inlining procedure: k13369 
o|inlining procedure: k13408 
o|inlining procedure: "(c-backend.scm:1541) getsize2852" 
o|inlining procedure: "(c-backend.scm:1540) getbits2851" 
o|inlining procedure: k13408 
o|contracted procedure: "(c-backend.scm:1546) chicken.compiler.c-backend#cons*" 
o|inlining procedure: k3709 
o|inlining procedure: k3709 
o|inlining procedure: "(c-backend.scm:1547) getbits2851" 
o|inlining procedure: "(c-backend.scm:1544) getsize2852" 
o|replaced variables: 3237 
o|removed binding forms: 635 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#every 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#any 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#first 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#second 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#third 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#fourth 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#filter 
o|substituted constant variable: r428813503 
o|substituted constant variable: r475413514 
o|substituted constant variable: r475413514 
o|substituted constant variable: r475413516 
o|substituted constant variable: r475413516 
o|inlining procedure: k4872 
o|inlining procedure: k5762 
o|substituted constant variable: r589213772 
o|substituted constant variable: r589213772 
o|propagated global variable: r607213798 chicken.compiler.core#no-procedure-checks 
o|substituted constant variable: r608813801 
o|substituted constant variable: r612413803 
o|substituted constant variable: r612413803 
o|substituted constant variable: r612413805 
o|substituted constant variable: r612413805 
o|substituted constant variable: r615213814 
o|substituted constant variable: r643913851 
o|substituted constant variable: r643913851 
o|substituted constant variable: r643913853 
o|substituted constant variable: r643913853 
o|substituted constant variable: r655513896 
o|substituted constant variable: r655513896 
o|substituted constant variable: r655513898 
o|substituted constant variable: r655513898 
o|substituted constant variable: r696914006 
o|substituted constant variable: r761214059 
o|substituted constant variable: r761214059 
o|substituted constant variable: r761214061 
o|substituted constant variable: r761214061 
o|contracted procedure: "(c-backend.scm:713) g13941409" 
o|removed side-effect free assignment to unused variable: bad-literal712 
o|substituted constant variable: r785114076 
o|substituted constant variable: r786314078 
o|substituted constant variable: r788714082 
o|substituted constant variable: r816814112 
o|substituted constant variable: r816814112 
o|substituted constant variable: r816814114 
o|substituted constant variable: r816814114 
o|propagated global variable: r867714151 chicken.compiler.core#unit-name 
o|substituted constant variable: r874014159 
o|substituted constant variable: r878014161 
o|substituted constant variable: r878014161 
o|substituted constant variable: r878014163 
o|substituted constant variable: r878014163 
o|substituted constant variable: r881014168 
o|substituted constant variable: r883414170 
o|substituted constant variable: r890814177 
o|substituted constant variable: r890814177 
o|substituted constant variable: r899314193 
o|substituted constant variable: r899314193 
o|substituted constant variable: r899314195 
o|substituted constant variable: r899314195 
o|inlining procedure: k8998 
o|substituted constant variable: r903414205 
o|substituted constant variable: r903414205 
o|substituted constant variable: r903414207 
o|substituted constant variable: r903414207 
o|substituted constant variable: r1046414288 
o|substituted constant variable: r1046414288 
o|substituted constant variable: r1046414290 
o|substituted constant variable: r1046414290 
o|propagated global variable: stubs2027 chicken.compiler.support#foreign-callback-stubs 
o|substituted constant variable: r978114317 
o|propagated global variable: stubs1916 chicken.compiler.core#foreign-lambda-stubs 
o|substituted constant variable: r931914325 
o|inlining procedure: k9368 
o|inlining procedure: k9368 
o|substituted constant variable: r939514334 
o|substituted constant variable: r1080914357 
o|substituted constant variable: r1126514388 
o|substituted constant variable: r1171214403 
o|substituted constant variable: r1172714405 
o|substituted constant variable: r1174214407 
o|substituted constant variable: r1175414409 
o|substituted constant variable: r1177214411 
o|substituted constant variable: r1178414413 
o|substituted constant variable: r1179614415 
o|substituted constant variable: r1181114417 
o|substituted constant variable: r1182314419 
o|substituted constant variable: r1183514421 
o|substituted constant variable: r1184714423 
o|substituted constant variable: r1185914425 
o|substituted constant variable: r1187114427 
o|substituted constant variable: r1188314429 
o|substituted constant variable: r1189514431 
o|substituted constant variable: r1190714433 
o|substituted constant variable: r1191914435 
o|substituted constant variable: r1193114437 
o|substituted constant variable: r1194314439 
o|substituted constant variable: r1195514441 
o|substituted constant variable: r1196714443 
o|substituted constant variable: r1197914445 
o|substituted constant variable: r1199714447 
o|substituted constant variable: r1203614450 
o|substituted constant variable: r1204814452 
o|substituted constant variable: r1206014454 
o|substituted constant variable: r1237714473 
o|substituted constant variable: r1239514475 
o|substituted constant variable: r1241014477 
o|substituted constant variable: r1242214479 
o|substituted constant variable: r1269214491 
o|removed side-effect free assignment to unused variable: getbits2851 
o|removed side-effect free assignment to unused variable: getsize2852 
o|substituted constant variable: r1323614515 
o|substituted constant variable: r1325914517 
o|substituted constant variable: r1327114519 
o|substituted constant variable: r1339014524 
o|substituted constant variable: r1339014524 
o|substituted constant variable: r1339014526 
o|substituted constant variable: r1339014526 
o|replaced variables: 384 
o|removed binding forms: 2538 
o|inlining procedure: "(c-backend.scm:286) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: "(c-backend.scm:296) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: "(c-backend.scm:301) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: k5860 
o|inlining procedure: "(c-backend.scm:323) chicken.compiler.c-backend#backslashify" 
o|inlining procedure: "(c-backend.scm:324) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: "(c-backend.scm:318) chicken.compiler.c-backend#fifth" 
o|inlining procedure: k6317 
o|inlining procedure: k6380 
o|inlining procedure: k6416 
o|inlining procedure: "(c-backend.scm:450) chicken.compiler.c-backend#backslashify" 
o|inlining procedure: "(c-backend.scm:451) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: "(c-backend.scm:443) chicken.compiler.c-backend#fifth" 
o|inlining procedure: k7103 
o|inlining procedure: k7155 
o|inlining procedure: k7554 
o|inlining procedure: k7554 
o|propagated global variable: r867714151 chicken.compiler.core#unit-name 
o|inlining procedure: k8721 
o|inlining procedure: k8827 
o|inlining procedure: k8827 
o|inlining procedure: k8938 
o|inlining procedure: k8938 
o|inlining procedure: "(c-backend.scm:628) chicken.compiler.c-backend#uncommentify" 
o|inlining procedure: "(c-backend.scm:1001) chicken.compiler.c-backend#backslashify" 
o|inlining procedure: k4464 
o|inlining procedure: k9355 
o|inlining procedure: k10575 
o|inlining procedure: k11303 
o|inlining procedure: k13427 
o|inlining procedure: k13435 
o|inlining procedure: k13475 
o|converted assignments to bindings: (encode-size2853) 
o|simplifications: ((let . 1)) 
o|replaced variables: 4 
o|removed binding forms: 492 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#fifth 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#backslashify 
o|removed side-effect free assignment to unused variable: chicken.compiler.c-backend#uncommentify 
o|substituted constant variable: r710414816 
o|substituted constant variable: r710414816 
o|inlining procedure: k7779 
o|substituted constant variable: r882814859 
o|substituted constant variable: r882814860 
o|substituted constant variable: r446514899 
o|substituted constant variable: r1130414921 
o|inlining procedure: k13229 
o|inlining procedure: k13229 
o|inlining procedure: k13229 
o|inlining procedure: k13229 
o|inlining procedure: k13229 
o|contracted procedure: k13438 
o|replaced variables: 27 
o|removed binding forms: 16 
o|removed conditional forms: 3 
o|substituted constant variable: r1323015036 
o|substituted constant variable: r1323015040 
o|substituted constant variable: r1323015044 
o|substituted constant variable: r1323015048 
o|substituted constant variable: r1323015052 
o|removed binding forms: 37 
o|removed binding forms: 6 
o|simplifications: ((let . 27) (if . 81) (##core#call . 884)) 
o|  call simplifications:
o|    scheme#eof-object?
o|    chicken.fixnum#fxlen
o|    scheme#integer->char	9
o|    scheme#string	10
o|    scheme#list?	3
o|    chicken.fixnum#fx>	2
o|    scheme#string-set!	3
o|    scheme#string=?
o|    scheme#-	4
o|    chicken.fixnum#fx/
o|    chicken.base#void	2
o|    scheme#boolean?
o|    scheme#char?	2
o|    chicken.base#fixnum?	3
o|    scheme#string?	2
o|    chicken.base#bignum?
o|    chicken.base#flonum?	2
o|    scheme#symbol?	7
o|    ##sys#immediate?	4
o|    ##sys#generic-structure?
o|    chicken.base#ratnum?
o|    chicken.base#cplxnum?
o|    ##sys#size	8
o|    scheme#apply	9
o|    scheme#string-length	3
o|    chicken.bitwise#arithmetic-shift	7
o|    scheme#string-ref	3
o|    chicken.bitwise#bitwise-and	9
o|    ##sys#setslot	5
o|    scheme#<	2
o|    scheme#>=	8
o|    scheme#null?	13
o|    scheme#cdr	7
o|    scheme#length	11
o|    scheme#cddr	4
o|    scheme#cdddr
o|    scheme#cddddr	2
o|    scheme#cadddr	4
o|    ##sys#check-list	24
o|    scheme#not	42
o|    scheme#+	22
o|    scheme#>	5
o|    scheme#caddr	9
o|    scheme#vector?	2
o|    scheme#vector-ref	8
o|    scheme#car	89
o|    scheme#char->integer	4
o|    scheme#cadr	39
o|    scheme#pair?	49
o|    scheme#eq?	328
o|    ##sys#slot	84
o|    chicken.fixnum#fx>=	4
o|    chicken.fixnum#fx+
o|    scheme#cons	16
o|contracted procedure: k4290 
o|contracted procedure: k4305 
o|contracted procedure: k4572 
o|contracted procedure: k4582 
o|contracted procedure: k4586 
o|contracted procedure: k4554 
o|contracted procedure: k4609 
o|contracted procedure: k4619 
o|contracted procedure: k4623 
o|contracted procedure: k4737 
o|contracted procedure: k4740 
o|contracted procedure: k4746 
o|contracted procedure: k4756 
o|contracted procedure: k4762 
o|contracted procedure: k4773 
o|contracted procedure: k4769 
o|contracted procedure: k4779 
o|contracted procedure: k4788 
o|contracted procedure: k4795 
o|contracted procedure: k4801 
o|contracted procedure: k4823 
o|contracted procedure: k4826 
o|contracted procedure: k4832 
o|contracted procedure: k4839 
o|contracted procedure: k4846 
o|contracted procedure: k4852 
o|contracted procedure: k4855 
o|contracted procedure: k4888 
o|contracted procedure: k4913 
o|contracted procedure: k4917 
o|contracted procedure: k4921 
o|contracted procedure: k4927 
o|contracted procedure: k4934 
o|contracted procedure: k4940 
o|contracted procedure: k4947 
o|contracted procedure: k4956 
o|contracted procedure: k4978 
o|contracted procedure: k4985 
o|contracted procedure: k4991 
o|contracted procedure: k4994 
o|contracted procedure: k5010 
o|contracted procedure: k5014 
o|contracted procedure: k5020 
o|contracted procedure: k5027 
o|contracted procedure: k5033 
o|contracted procedure: k5046 
o|contracted procedure: k5052 
o|contracted procedure: k5065 
o|contracted procedure: k5071 
o|contracted procedure: k5088 
o|contracted procedure: k5084 
o|contracted procedure: k5092 
o|contracted procedure: k5098 
o|contracted procedure: k5104 
o|contracted procedure: k5117 
o|contracted procedure: k5138 
o|contracted procedure: k5144 
o|contracted procedure: k5157 
o|contracted procedure: k5178 
o|contracted procedure: k5184 
o|contracted procedure: k5197 
o|contracted procedure: k5218 
o|contracted procedure: k5231 
o|contracted procedure: k5237 
o|contracted procedure: k5256 
o|contracted procedure: k5260 
o|contracted procedure: k5264 
o|contracted procedure: k5270 
o|contracted procedure: k5289 
o|contracted procedure: k5297 
o|contracted procedure: k5293 
o|contracted procedure: k5301 
o|contracted procedure: k5307 
o|contracted procedure: k5326 
o|contracted procedure: k5330 
o|contracted procedure: k5336 
o|contracted procedure: k5355 
o|contracted procedure: k5359 
o|contracted procedure: k5365 
o|contracted procedure: k5368 
o|contracted procedure: k5388 
o|contracted procedure: k5391 
o|contracted procedure: k5433 
o|contracted procedure: k5408 
o|contracted procedure: k5418 
o|contracted procedure: k5422 
o|contracted procedure: k5426 
o|contracted procedure: k5430 
o|contracted procedure: k5442 
o|contracted procedure: k5455 
o|contracted procedure: k5461 
o|contracted procedure: k5468 
o|contracted procedure: k5474 
o|contracted procedure: k5484 
o|contracted procedure: k5488 
o|contracted procedure: k5494 
o|contracted procedure: k5497 
o|contracted procedure: k5500 
o|contracted procedure: k5503 
o|contracted procedure: k5527 
o|contracted procedure: k5542 
o|contracted procedure: k5545 
o|contracted procedure: k5548 
o|contracted procedure: k5551 
o|contracted procedure: k5567 
o|contracted procedure: k5587 
o|contracted procedure: k5590 
o|contracted procedure: k5593 
o|contracted procedure: k5596 
o|contracted procedure: k5612 
o|contracted procedure: k5633 
o|contracted procedure: k5647 
o|contracted procedure: k5656 
o|contracted procedure: k5659 
o|contracted procedure: k5662 
o|contracted procedure: k5666 
o|contracted procedure: k5669 
o|contracted procedure: k6168 
o|contracted procedure: k5672 
o|contracted procedure: k5675 
o|contracted procedure: k5697 
o|contracted procedure: k5713 
o|contracted procedure: k5742 
o|contracted procedure: k5756 
o|contracted procedure: k5802 
o|contracted procedure: k5777 
o|contracted procedure: k5787 
o|contracted procedure: k5791 
o|contracted procedure: k5795 
o|contracted procedure: k5799 
o|contracted procedure: k5839 
o|contracted procedure: k5814 
o|contracted procedure: k5824 
o|contracted procedure: k5828 
o|contracted procedure: k5832 
o|contracted procedure: k5836 
o|contracted procedure: k5894 
o|contracted procedure: k5915 
o|contracted procedure: k5934 
o|contracted procedure: k5937 
o|contracted procedure: k5940 
o|contracted procedure: k6006 
o|contracted procedure: k6084 
o|contracted procedure: k6090 
o|contracted procedure: k6096 
o|contracted procedure: k6116 
o|contracted procedure: k382814778 
o|contracted procedure: k6161 
o|contracted procedure: k6154 
o|contracted procedure: k6174 
o|contracted procedure: k6177 
o|contracted procedure: k6181 
o|contracted procedure: k6184 
o|contracted procedure: k6211 
o|contracted procedure: k6225 
o|contracted procedure: k6265 
o|contracted procedure: k6240 
o|contracted procedure: k6250 
o|contracted procedure: k6254 
o|contracted procedure: k6258 
o|contracted procedure: k6262 
o|contracted procedure: k6302 
o|contracted procedure: k6277 
o|contracted procedure: k6287 
o|contracted procedure: k6291 
o|contracted procedure: k6295 
o|contracted procedure: k6299 
o|contracted procedure: k6332 
o|contracted procedure: k6335 
o|contracted procedure: k6338 
o|contracted procedure: k6342 
o|contracted procedure: k6345 
o|contracted procedure: k6351 
o|contracted procedure: k382814807 
o|contracted procedure: k6354 
o|contracted procedure: k6357 
o|contracted procedure: k6389 
o|contracted procedure: k6401 
o|contracted procedure: k6413 
o|contracted procedure: k6416 
o|contracted procedure: k6431 
o|contracted procedure: k6464 
o|contracted procedure: k6471 
o|contracted procedure: k6477 
o|contracted procedure: k6480 
o|contracted procedure: k6499 
o|contracted procedure: k6505 
o|contracted procedure: k6518 
o|contracted procedure: k6524 
o|contracted procedure: k6537 
o|contracted procedure: k6543 
o|contracted procedure: k6550 
o|contracted procedure: k6560 
o|contracted procedure: k6572 
o|contracted procedure: k6582 
o|contracted procedure: k6586 
o|contracted procedure: k6592 
o|contracted procedure: k6603 
o|contracted procedure: k6607 
o|contracted procedure: k6613 
o|contracted procedure: k6616 
o|contracted procedure: k6629 
o|contracted procedure: k6633 
o|contracted procedure: k6647 
o|contracted procedure: k6650 
o|contracted procedure: k6663 
o|contracted procedure: k6677 
o|contracted procedure: k6680 
o|contracted procedure: k6699 
o|contracted procedure: k6707 
o|contracted procedure: k6717 
o|contracted procedure: k6733 
o|contracted procedure: k6737 
o|contracted procedure: k6757 
o|contracted procedure: k6778 
o|contracted procedure: k6782 
o|contracted procedure: k6786 
o|contracted procedure: k6790 
o|contracted procedure: k6796 
o|contracted procedure: k6821 
o|contracted procedure: k6825 
o|contracted procedure: k6829 
o|contracted procedure: k6935 
o|contracted procedure: k6950 
o|contracted procedure: k6953 
o|contracted procedure: k6971 
o|contracted procedure: k6974 
o|contracted procedure: k6977 
o|contracted procedure: k7005 
o|contracted procedure: k7010 
o|contracted procedure: k7032 
o|contracted procedure: k7041 
o|contracted procedure: k7061 
o|contracted procedure: k7067 
o|contracted procedure: k7079 
o|contracted procedure: k7112 
o|contracted procedure: k7103 
o|contracted procedure: k7164 
o|contracted procedure: k7184 
o|contracted procedure: k7194 
o|contracted procedure: k7198 
o|contracted procedure: k7204 
o|contracted procedure: k7213 
o|contracted procedure: k7224 
o|contracted procedure: k7244 
o|contracted procedure: k7247 
o|contracted procedure: k7250 
o|contracted procedure: k7258 
o|contracted procedure: k7266 
o|contracted procedure: k7293 
o|contracted procedure: k7304 
o|contracted procedure: k7307 
o|contracted procedure: k7313 
o|contracted procedure: k7334 
o|contracted procedure: k7340 
o|contracted procedure: k7364 
o|contracted procedure: k7392 
o|contracted procedure: k7408 
o|contracted procedure: k7404 
o|contracted procedure: k7412 
o|contracted procedure: k7423 
o|contracted procedure: k7416 
o|contracted procedure: k7427 
o|contracted procedure: k7430 
o|contracted procedure: k7443 
o|contracted procedure: k7453 
o|contracted procedure: k7457 
o|contracted procedure: k7466 
o|contracted procedure: k7469 
o|contracted procedure: k7472 
o|contracted procedure: k7480 
o|contracted procedure: k7488 
o|propagated global variable: g12951299 chicken.compiler.core#used-units 
o|contracted procedure: k7643 
o|contracted procedure: k7655 
o|contracted procedure: k7665 
o|contracted procedure: k7669 
o|contracted procedure: k7502 
o|contracted procedure: k7563 
o|contracted procedure: k7572 
o|contracted procedure: k7627 
o|contracted procedure: k7586 
o|contracted procedure: k7764 
o|contracted procedure: k7776 
o|contracted procedure: k7786 
o|contracted procedure: k7790 
o|contracted procedure: k7704 
o|contracted procedure: k7751 
o|contracted procedure: k7723 
o|contracted procedure: k7689 
o|contracted procedure: k778614977 
o|contracted procedure: k7805 
o|contracted procedure: k7819 
o|contracted procedure: k7865 
o|contracted procedure: k7871 
o|contracted procedure: k7877 
o|contracted procedure: k7883 
o|contracted procedure: k7895 
o|contracted procedure: k7916 
o|contracted procedure: k7923 
o|contracted procedure: k7927 
o|contracted procedure: k7936 
o|contracted procedure: k7948 
o|contracted procedure: k7955 
o|contracted procedure: k7963 
o|contracted procedure: k7959 
o|contracted procedure: k7972 
o|contracted procedure: k7975 
o|contracted procedure: k7978 
o|contracted procedure: k7986 
o|contracted procedure: k7994 
o|contracted procedure: k8010 
o|contracted procedure: k8033 
o|contracted procedure: k8039 
o|contracted procedure: k8042 
o|contracted procedure: k8057 
o|contracted procedure: k8066 
o|contracted procedure: k8074 
o|contracted procedure: k8080 
o|contracted procedure: k8087 
o|contracted procedure: k8105 
o|contracted procedure: k8112 
o|contracted procedure: k8271 
o|contracted procedure: k8151 
o|propagated global variable: r8272 ##sys#undefined-value 
o|contracted procedure: k8160 
o|contracted procedure: k8173 
o|contracted procedure: k8180 
o|contracted procedure: k8189 
o|contracted procedure: k8192 
o|contracted procedure: k8198 
o|contracted procedure: k8201 
o|contracted procedure: k8216 
o|contracted procedure: k8264 
o|contracted procedure: k8260 
o|contracted procedure: k8222 
o|contracted procedure: k8231 
o|contracted procedure: k8257 
o|contracted procedure: k8274 
o|contracted procedure: k8287 
o|contracted procedure: k8290 
o|contracted procedure: k8308 
o|contracted procedure: k8329 
o|contracted procedure: k8343 
o|contracted procedure: k8355 
o|contracted procedure: k8361 
o|contracted procedure: k8375 
o|contracted procedure: k8400 
o|contracted procedure: k8606 
o|contracted procedure: k8599 
o|contracted procedure: k8497 
o|contracted procedure: k8516 
o|contracted procedure: k8533 
o|contracted procedure: k8537 
o|contracted procedure: k8546 
o|contracted procedure: k8552 
o|contracted procedure: k8570 
o|contracted procedure: k8582 
o|contracted procedure: k8609 
o|contracted procedure: k8612 
o|contracted procedure: k8653 
o|contracted procedure: k8640 
o|contracted procedure: k8685 
o|contracted procedure: k8692 
o|contracted procedure: k8709 
o|contracted procedure: k8736 
o|contracted procedure: k8742 
o|contracted procedure: k8748 
o|contracted procedure: k8769 
o|contracted procedure: k8794 
o|contracted procedure: k8800 
o|contracted procedure: k8812 
o|contracted procedure: k8818 
o|contracted procedure: k8836 
o|contracted procedure: k8827 
o|contracted procedure: k8856 
o|contracted procedure: k8868 
o|contracted procedure: k8878 
o|contracted procedure: k8882 
o|contracted procedure: k8911 
o|contracted procedure: k8923 
o|contracted procedure: k8934 
o|contracted procedure: k8947 
o|contracted procedure: k8956 
o|contracted procedure: k9008 
o|contracted procedure: k8970 
o|contracted procedure: k9019 
o|contracted procedure: k9026 
o|contracted procedure: k9044 
o|contracted procedure: k9056 
o|contracted procedure: k9066 
o|contracted procedure: k9070 
o|contracted procedure: k9245 
o|contracted procedure: k9255 
o|contracted procedure: k9293 
o|contracted procedure: k9303 
o|contracted procedure: k9307 
o|contracted procedure: k9312 
o|contracted procedure: k9191 
o|contracted procedure: k9206 
o|contracted procedure: k9216 
o|contracted procedure: k9220 
o|contracted procedure: k9149 
o|contracted procedure: k9152 
o|contracted procedure: k9167 
o|contracted procedure: k9177 
o|contracted procedure: k9181 
o|contracted procedure: k9132 
o|contracted procedure: k9185 
o|contracted procedure: k9230 
o|contracted procedure: k9234 
o|contracted procedure: k9896 
o|contracted procedure: k9908 
o|contracted procedure: k9917 
o|contracted procedure: k9929 
o|contracted procedure: k9941 
o|contracted procedure: k9953 
o|contracted procedure: k9965 
o|contracted procedure: k9986 
o|contracted procedure: k9995 
o|contracted procedure: k10002 
o|contracted procedure: k10014 
o|contracted procedure: k10021 
o|contracted procedure: k10026 
o|contracted procedure: k10032 
o|contracted procedure: k10038 
o|contracted procedure: k10044 
o|contracted procedure: k10050 
o|contracted procedure: k10056 
o|contracted procedure: k10062 
o|contracted procedure: k10089 
o|contracted procedure: k10095 
o|contracted procedure: k10101 
o|contracted procedure: k10107 
o|contracted procedure: k10126 
o|contracted procedure: k10132 
o|contracted procedure: k10138 
o|contracted procedure: k10157 
o|contracted procedure: k10163 
o|contracted procedure: k10180 
o|contracted procedure: k10186 
o|contracted procedure: k10192 
o|contracted procedure: k10198 
o|contracted procedure: k10204 
o|contracted procedure: k10227 
o|contracted procedure: k10233 
o|contracted procedure: k10239 
o|contracted procedure: k10245 
o|contracted procedure: k10266 
o|contracted procedure: k10272 
o|contracted procedure: k10278 
o|contracted procedure: k10284 
o|contracted procedure: k10290 
o|contracted procedure: k10296 
o|contracted procedure: k10302 
o|contracted procedure: k10308 
o|contracted procedure: k10314 
o|contracted procedure: k10320 
o|contracted procedure: k10326 
o|contracted procedure: k10409 
o|contracted procedure: k10415 
o|contracted procedure: k10456 
o|contracted procedure: k10431 
o|contracted procedure: k10441 
o|contracted procedure: k10445 
o|contracted procedure: k10449 
o|contracted procedure: k10453 
o|contracted procedure: k10466 
o|contracted procedure: k10482 
o|contracted procedure: k10489 
o|contracted procedure: k10493 
o|contracted procedure: k10504 
o|contracted procedure: k10516 
o|contracted procedure: k10526 
o|contracted procedure: k10530 
o|propagated global variable: stubs2027 chicken.compiler.support#foreign-callback-stubs 
o|contracted procedure: k9555 
o|contracted procedure: k9639 
o|contracted procedure: k9642 
o|contracted procedure: k9708 
o|contracted procedure: k9732 
o|contracted procedure: k9722 
o|contracted procedure: k9754 
o|contracted procedure: k9758 
o|contracted procedure: k9762 
o|contracted procedure: k9766 
o|contracted procedure: k9770 
o|contracted procedure: k9774 
o|contracted procedure: k9777 
o|contracted procedure: k9783 
o|contracted procedure: k9801 
o|contracted procedure: k9813 
o|contracted procedure: k9830 
o|contracted procedure: k4473 
o|contracted procedure: k4464 
o|contracted procedure: k9850 
o|contracted procedure: k9862 
o|contracted procedure: k9872 
o|contracted procedure: k9876 
o|propagated global variable: stubs1916 chicken.compiler.core#foreign-lambda-stubs 
o|contracted procedure: k9469 
o|contracted procedure: k9481 
o|contracted procedure: k9491 
o|contracted procedure: k9495 
o|contracted procedure: k9449 
o|contracted procedure: k9452 
o|contracted procedure: k9455 
o|contracted procedure: k9462 
o|contracted procedure: k4680 
o|contracted procedure: k4684 
o|contracted procedure: k4695 
o|contracted procedure: k4691 
o|contracted procedure: k9321 
o|contracted procedure: k9334 
o|contracted procedure: k9346 
o|contracted procedure: k9352 
o|inlining procedure: k9355 
o|inlining procedure: k9355 
o|contracted procedure: k9402 
o|contracted procedure: k9512 
o|contracted procedure: k9524 
o|contracted procedure: k9534 
o|contracted procedure: k9538 
o|contracted procedure: k10548 
o|contracted procedure: k10569 
o|contracted procedure: k10585 
o|contracted procedure: k10598 
o|contracted procedure: k10602 
o|contracted procedure: k10625 
o|contracted procedure: k10634 
o|contracted procedure: k10637 
o|contracted procedure: k10646 
o|contracted procedure: k10649 
o|contracted procedure: k10658 
o|contracted procedure: k10661 
o|contracted procedure: k10670 
o|contracted procedure: k10673 
o|contracted procedure: k10682 
o|contracted procedure: k10694 
o|contracted procedure: k10703 
o|contracted procedure: k10712 
o|contracted procedure: k10715 
o|contracted procedure: k10724 
o|contracted procedure: k10733 
o|contracted procedure: k10742 
o|contracted procedure: k10751 
o|contracted procedure: k10760 
o|contracted procedure: k10769 
o|contracted procedure: k10778 
o|contracted procedure: k10787 
o|contracted procedure: k10790 
o|contracted procedure: k10799 
o|contracted procedure: k10811 
o|contracted procedure: k10814 
o|contracted procedure: k10820 
o|contracted procedure: k10832 
o|contracted procedure: k10835 
o|contracted procedure: k10844 
o|contracted procedure: k10847 
o|contracted procedure: k10856 
o|contracted procedure: k10859 
o|contracted procedure: k10868 
o|contracted procedure: k10871 
o|contracted procedure: k10880 
o|contracted procedure: k10883 
o|contracted procedure: k10892 
o|contracted procedure: k10895 
o|contracted procedure: k10904 
o|contracted procedure: k10907 
o|contracted procedure: k10916 
o|contracted procedure: k10919 
o|contracted procedure: k10928 
o|contracted procedure: k10931 
o|contracted procedure: k10940 
o|contracted procedure: k10943 
o|contracted procedure: k10952 
o|contracted procedure: k10964 
o|contracted procedure: k10976 
o|contracted procedure: k10994 
o|contracted procedure: k11003 
o|contracted procedure: k11012 
o|contracted procedure: k11026 
o|contracted procedure: k11043 
o|contracted procedure: k11072 
o|contracted procedure: k11080 
o|contracted procedure: k11083 
o|contracted procedure: k11095 
o|contracted procedure: k11098 
o|contracted procedure: k11101 
o|contracted procedure: k11109 
o|contracted procedure: k11117 
o|contracted procedure: k11124 
o|contracted procedure: k11141 
o|contracted procedure: k11158 
o|contracted procedure: k11175 
o|contracted procedure: k11192 
o|contracted procedure: k11209 
o|contracted procedure: k11226 
o|contracted procedure: k11235 
o|contracted procedure: k11238 
o|contracted procedure: k11313 
o|contracted procedure: k11244 
o|contracted procedure: k11259 
o|contracted procedure: k11273 
o|contracted procedure: k11285 
o|contracted procedure: k11291 
o|contracted procedure: k11299 
o|contracted procedure: k11307 
o|contracted procedure: k11267 
o|contracted procedure: k11322 
o|contracted procedure: k11329 
o|contracted procedure: k11334 
o|contracted procedure: k11341 
o|contracted procedure: k11346 
o|contracted procedure: k11350 
o|contracted procedure: k11356 
o|contracted procedure: k11363 
o|contracted procedure: k11368 
o|contracted procedure: k11375 
o|contracted procedure: k11380 
o|contracted procedure: k11387 
o|contracted procedure: k11392 
o|contracted procedure: k11399 
o|contracted procedure: k11404 
o|contracted procedure: k11411 
o|contracted procedure: k11416 
o|contracted procedure: k11423 
o|contracted procedure: k11428 
o|contracted procedure: k11432 
o|contracted procedure: k11441 
o|contracted procedure: k11449 
o|contracted procedure: k11455 
o|contracted procedure: k11472 
o|contracted procedure: k11478 
o|contracted procedure: k11484 
o|contracted procedure: k11573 
o|contracted procedure: k11579 
o|contracted procedure: k11603 
o|contracted procedure: k11609 
o|contracted procedure: k11658 
o|contracted procedure: k11714 
o|contracted procedure: k11720 
o|contracted procedure: k11723 
o|contracted procedure: k11729 
o|contracted procedure: k11738 
o|contracted procedure: k11744 
o|contracted procedure: k11750 
o|contracted procedure: k11756 
o|contracted procedure: k11765 
o|contracted procedure: k11768 
o|contracted procedure: k11774 
o|contracted procedure: k11780 
o|contracted procedure: k11786 
o|contracted procedure: k11792 
o|contracted procedure: k11798 
o|contracted procedure: k11804 
o|contracted procedure: k11807 
o|contracted procedure: k11813 
o|contracted procedure: k11819 
o|contracted procedure: k11825 
o|contracted procedure: k11831 
o|contracted procedure: k11837 
o|contracted procedure: k11843 
o|contracted procedure: k11849 
o|contracted procedure: k11855 
o|contracted procedure: k11861 
o|contracted procedure: k11867 
o|contracted procedure: k11873 
o|contracted procedure: k11879 
o|contracted procedure: k11885 
o|contracted procedure: k11891 
o|contracted procedure: k11897 
o|contracted procedure: k11903 
o|contracted procedure: k11909 
o|contracted procedure: k11915 
o|contracted procedure: k11921 
o|contracted procedure: k11927 
o|contracted procedure: k11933 
o|contracted procedure: k11939 
o|contracted procedure: k11945 
o|contracted procedure: k11951 
o|contracted procedure: k11957 
o|contracted procedure: k11963 
o|contracted procedure: k11969 
o|contracted procedure: k11975 
o|contracted procedure: k11981 
o|contracted procedure: k11990 
o|contracted procedure: k11999 
o|contracted procedure: k12014 
o|contracted procedure: k12148 
o|contracted procedure: k12023 
o|contracted procedure: k12026 
o|contracted procedure: k12032 
o|contracted procedure: k12038 
o|contracted procedure: k12044 
o|contracted procedure: k12050 
o|contracted procedure: k12056 
o|contracted procedure: k12062 
o|contracted procedure: k12068 
o|contracted procedure: k12074 
o|contracted procedure: k12081 
o|contracted procedure: k12087 
o|contracted procedure: k12093 
o|contracted procedure: k12104 
o|contracted procedure: k12110 
o|contracted procedure: k12117 
o|contracted procedure: k12157 
o|contracted procedure: k12165 
o|contracted procedure: k12171 
o|contracted procedure: k12177 
o|contracted procedure: k12196 
o|contracted procedure: k12202 
o|contracted procedure: k12299 
o|contracted procedure: k12320 
o|contracted procedure: k12326 
o|contracted procedure: k12332 
o|contracted procedure: k12338 
o|contracted procedure: k12379 
o|contracted procedure: k12382 
o|contracted procedure: k12388 
o|contracted procedure: k12391 
o|contracted procedure: k12397 
o|contracted procedure: k12400 
o|contracted procedure: k12406 
o|contracted procedure: k12412 
o|contracted procedure: k12418 
o|contracted procedure: k12424 
o|contracted procedure: k12430 
o|contracted procedure: k12433 
o|contracted procedure: k12457 
o|contracted procedure: k12481 
o|contracted procedure: k12508 
o|contracted procedure: k12532 
o|contracted procedure: k12535 
o|contracted procedure: k12559 
o|contracted procedure: k12562 
o|contracted procedure: k12586 
o|contracted procedure: k12589 
o|contracted procedure: k12613 
o|contracted procedure: k12616 
o|contracted procedure: k12640 
o|contracted procedure: k12664 
o|contracted procedure: k12688 
o|contracted procedure: k12694 
o|contracted procedure: k12697 
o|contracted procedure: k12712 
o|contracted procedure: k12972 
o|contracted procedure: k12721 
o|contracted procedure: k12724 
o|contracted procedure: k12730 
o|contracted procedure: k12733 
o|contracted procedure: k12757 
o|contracted procedure: k12781 
o|contracted procedure: k12805 
o|contracted procedure: k12829 
o|contracted procedure: k12853 
o|contracted procedure: k12860 
o|contracted procedure: k12866 
o|contracted procedure: k12869 
o|contracted procedure: k12893 
o|contracted procedure: k12917 
o|contracted procedure: k12981 
o|contracted procedure: k13030 
o|contracted procedure: k13036 
o|contracted procedure: k13042 
o|contracted procedure: k13048 
o|contracted procedure: k13054 
o|contracted procedure: k13060 
o|contracted procedure: k13066 
o|contracted procedure: k13072 
o|contracted procedure: k13078 
o|contracted procedure: k13084 
o|contracted procedure: k13212 
o|contracted procedure: k13170 
o|contracted procedure: k13208 
o|contracted procedure: k13204 
o|contracted procedure: k13180 
o|contracted procedure: k13200 
o|contracted procedure: k13196 
o|contracted procedure: k13184 
o|contracted procedure: k13192 
o|contracted procedure: k13188 
o|contracted procedure: k13222 
o|contracted procedure: k13232 
o|contracted procedure: k1322215039 
o|contracted procedure: k13238 
o|contracted procedure: k1322215043 
o|contracted procedure: k13244 
o|contracted procedure: k13255 
o|contracted procedure: k13261 
o|contracted procedure: k1322215047 
o|contracted procedure: k13267 
o|contracted procedure: k1322215051 
o|contracted procedure: k13492 
o|contracted procedure: k13273 
o|propagated global variable: r13493 ##sys#undefined-value 
o|contracted procedure: k1322215055 
o|contracted procedure: k13331 
o|contracted procedure: k13327 
o|contracted procedure: k13291 
o|contracted procedure: k13323 
o|contracted procedure: k13319 
o|contracted procedure: k13295 
o|contracted procedure: k13315 
o|contracted procedure: k13311 
o|contracted procedure: k13299 
o|contracted procedure: k13307 
o|contracted procedure: k13303 
o|contracted procedure: k13287 
o|contracted procedure: k13349 
o|contracted procedure: k13358 
o|contracted procedure: k13375 
o|contracted procedure: k13378 
o|contracted procedure: k13396 
o|contracted procedure: k13402 
o|contracted procedure: k13431 
o|contracted procedure: k13419 
o|contracted procedure: k13471 
o|contracted procedure: k13449 
o|contracted procedure: k3712 
o|contracted procedure: k3723 
o|contracted procedure: k13467 
o|contracted procedure: k13481 
o|simplifications: ((if . 13) (##core#call . 12) (let . 266)) 
o|  call simplifications:
o|    scheme#+	12
o|removed binding forms: 816 
o|contracted procedure: k5684 
o|contracted procedure: k8234 
o|contracted procedure: k8311 
o|contracted procedure: k9355 
o|contracted procedure: k11019 
o|contracted procedure: k11036 
o|contracted procedure: k11053 
o|contracted procedure: k11130 
o|contracted procedure: k11147 
o|contracted procedure: k11164 
o|contracted procedure: k11181 
o|contracted procedure: k11198 
o|contracted procedure: k11215 
o|contracted procedure: k11232 
o|removed binding forms: 14 
o|customizable procedures: (k13279 loop299 encode-size2853 k12484 err2543 g27372738 k11732 k11759 k11984 k11993 err2465 k10685 k10802 k10823 k10955 k10967 err2265 k11288 map-loop24382456 map-loop23902415 g23772378 str2266 loop2254 for-each-loop18971910 k9365 loop1844 header706 declarations708 for-each-loop18731887 g19211928 for-each-loop19202021 doloop19631964 k9744 for-each-loop19501989 prototypes709 g20322039 for-each-loop20312240 loop2201 chicken.compiler.c-backend#generate-foreign-callback-header for-each-loop22082222 k9911 k9920 k9932 k9944 k9956 k9968 k10005 g21672168 compute-size2048 trampolines710 for-each-loop17821793 for-each-loop17691802 procedures717 g18151822 for-each-loop18141828 g15501557 for-each-loop15491733 k8397 chicken.compiler.c-backend#cleanup k8463 doloop16071608 k8886 doloop16121613 for-each-loop16201631 k8788 k8730 g16511652 foldl16441648 literal-frame711 doloop16911692 doloop16991700 expression705 doloop15321533 string-like-substring716 k8136 chicken.compiler.c-backend#encode-literal gen-string-constant715 va15435 va15433 loop1497 map-loop14671484 foldl14611491 va15425 literal-size713 va15423 gen-lit714 doloop14321433 k7714 k7717 doloop14031404 chicken.compiler.c-backend#make-argument-list for-each-loop13931426 k7512 chicken.compiler.c-backend#make-variable-list for-each-loop13381382 map-loop12831300 for-each-loop12691307 doloop13171319 doloop13181323 doloop12771312 map-loop12061231 chicken.compiler.c-backend#gen-list for-each-loop12411252 chicken.compiler.c-backend#generate-foreign-callback-stub-prototypes k7013 doloop11781191 contains-restop?727 loop1153 loop1147 doloop11251126 chicken.compiler.c-backend#foreign-argument-conversion chicken.compiler.c-backend#foreign-result-conversion va15391 g10261045 for-each-loop10251052 for-each-loop10361065 k5681 find-lambda704 k5687 k5928 k6062 expr-args726 va15385 g909928 for-each-loop908935 for-each-loop919948 push-args728 chicken.compiler.c-backend#list-tabulate g821829 for-each-loop820836 k5187 k5147 k5107 loop759 expr725 for-each-loop662673 for-each-loop642653 loop517) 
o|calls to known targets: 835 
o|identified direct recursive calls: f_7943 1 
o|unused rest argument: _57714901 f14902 
o|unused rest argument: _577 f_4471 
o|identified direct recursive calls: f_3707 1 
o|fast box initializations: 68 
o|fast global references: 405 
o|fast global assignments: 14 
o|dropping unused closure argument: f_10534 
o|dropping unused closure argument: f_11704 
o|dropping unused closure argument: f_12369 
o|dropping unused closure argument: f_13156 
o|dropping unused closure argument: f_13165 
o|dropping unused closure argument: f_4279 
o|dropping unused closure argument: f_4590 
o|dropping unused closure argument: f_8359 
o|dropping unused closure argument: f_9332 
o|dropping unused closure argument: f_9408 
o|dropping unused closure argument: f_9425 
o|dropping unused closure argument: f_9499 
*/
/* end of file */
