/* Generated from support.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: support.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -no-lambda-info -no-trace -emit-import-library chicken.compiler.support -output-file support.c
   unit: support
   uses: eval expand data-structures port pathname internal extras file library
*/
#include "chicken.h"

static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_eval_toplevel)
C_externimport void C_ccall C_eval_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_expand_toplevel)
C_externimport void C_ccall C_expand_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_data_2dstructures_toplevel)
C_externimport void C_ccall C_data_2dstructures_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_port_toplevel)
C_externimport void C_ccall C_port_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_pathname_toplevel)
C_externimport void C_ccall C_pathname_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_internal_toplevel)
C_externimport void C_ccall C_internal_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_extras_toplevel)
C_externimport void C_ccall C_extras_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_file_toplevel)
C_externimport void C_ccall C_file_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[550];
static double C_possibly_force_alignment;


#define return(x) C_cblock C_r = (C_fix((C_word)(x))); goto C_ret; C_cblockend
C_regparm static C_word C_fcall stub4284(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_word lit=(C_word )(C_a0);
return(C_header_size(lit));
C_ret:
#undef return

return C_r;}

/* from k7329 */
C_regparm static C_word C_fcall stub932(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
int t0=(int )C_unfix(C_a0);
C_r=C_fix((C_word)C_wordstobytes(t0));
return C_r;}

/* from k7322 */
C_regparm static C_word C_fcall stub927(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
int t0=(int )C_unfix(C_a0);
C_r=C_fix((C_word)C_bytestowords(t0));
return C_r;}

C_noret_decl(f19850)
static void C_ccall f19850(C_word c,C_word *av) C_noret;
C_noret_decl(f_10004)
static void C_ccall f_10004(C_word c,C_word *av) C_noret;
C_noret_decl(f_10032)
static void C_ccall f_10032(C_word c,C_word *av) C_noret;
C_noret_decl(f_10056)
static void C_ccall f_10056(C_word c,C_word *av) C_noret;
C_noret_decl(f_10090)
static void C_ccall f_10090(C_word c,C_word *av) C_noret;
C_noret_decl(f_10094)
static void C_ccall f_10094(C_word c,C_word *av) C_noret;
C_noret_decl(f_10104)
static void C_fcall f_10104(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10128)
static void C_ccall f_10128(C_word c,C_word *av) C_noret;
C_noret_decl(f_10143)
static void C_ccall f_10143(C_word c,C_word *av) C_noret;
C_noret_decl(f_10155)
static void C_ccall f_10155(C_word c,C_word *av) C_noret;
C_noret_decl(f_10180)
static void C_fcall f_10180(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10205)
static void C_ccall f_10205(C_word c,C_word *av) C_noret;
C_noret_decl(f_10235)
static void C_ccall f_10235(C_word c,C_word *av) C_noret;
C_noret_decl(f_10237)
static void C_fcall f_10237(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10262)
static void C_ccall f_10262(C_word c,C_word *av) C_noret;
C_noret_decl(f_10293)
static void C_fcall f_10293(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10305)
static void C_ccall f_10305(C_word c,C_word *av) C_noret;
C_noret_decl(f_10309)
static void C_ccall f_10309(C_word c,C_word *av) C_noret;
C_noret_decl(f_10332)
static void C_ccall f_10332(C_word c,C_word *av) C_noret;
C_noret_decl(f_10343)
static void C_fcall f_10343(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10350)
static void C_ccall f_10350(C_word c,C_word *av) C_noret;
C_noret_decl(f_10361)
static void C_ccall f_10361(C_word c,C_word *av) C_noret;
C_noret_decl(f_10363)
static void C_fcall f_10363(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10388)
static void C_ccall f_10388(C_word c,C_word *av) C_noret;
C_noret_decl(f_10418)
static void C_ccall f_10418(C_word c,C_word *av) C_noret;
C_noret_decl(f_10420)
static void C_fcall f_10420(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10445)
static void C_ccall f_10445(C_word c,C_word *av) C_noret;
C_noret_decl(f_10459)
static void C_ccall f_10459(C_word c,C_word *av) C_noret;
C_noret_decl(f_10469)
static void C_ccall f_10469(C_word c,C_word *av) C_noret;
C_noret_decl(f_10471)
static void C_fcall f_10471(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10496)
static void C_ccall f_10496(C_word c,C_word *av) C_noret;
C_noret_decl(f_10569)
static void C_ccall f_10569(C_word c,C_word *av) C_noret;
C_noret_decl(f_10575)
static void C_fcall f_10575(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10601)
static void C_ccall f_10601(C_word c,C_word *av) C_noret;
C_noret_decl(f_10605)
static void C_ccall f_10605(C_word c,C_word *av) C_noret;
C_noret_decl(f_10621)
static void C_ccall f_10621(C_word c,C_word *av) C_noret;
C_noret_decl(f_10627)
static void C_ccall f_10627(C_word c,C_word *av) C_noret;
C_noret_decl(f_10633)
static void C_ccall f_10633(C_word c,C_word *av) C_noret;
C_noret_decl(f_10639)
static void C_ccall f_10639(C_word c,C_word *av) C_noret;
C_noret_decl(f_10643)
static void C_ccall f_10643(C_word c,C_word *av) C_noret;
C_noret_decl(f_10646)
static void C_ccall f_10646(C_word c,C_word *av) C_noret;
C_noret_decl(f_10660)
static void C_ccall f_10660(C_word c,C_word *av) C_noret;
C_noret_decl(f_10663)
static void C_ccall f_10663(C_word c,C_word *av) C_noret;
C_noret_decl(f_10666)
static void C_ccall f_10666(C_word c,C_word *av) C_noret;
C_noret_decl(f_10673)
static void C_ccall f_10673(C_word c,C_word *av) C_noret;
C_noret_decl(f_10675)
static void C_fcall f_10675(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10690)
static void C_fcall f_10690(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10723)
static void C_ccall f_10723(C_word c,C_word *av) C_noret;
C_noret_decl(f_10751)
static void C_ccall f_10751(C_word c,C_word *av) C_noret;
C_noret_decl(f_10755)
static void C_ccall f_10755(C_word c,C_word *av) C_noret;
C_noret_decl(f_10779)
static void C_ccall f_10779(C_word c,C_word *av) C_noret;
C_noret_decl(f_10811)
static void C_ccall f_10811(C_word c,C_word *av) C_noret;
C_noret_decl(f_10820)
static void C_fcall f_10820(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10824)
static void C_ccall f_10824(C_word c,C_word *av) C_noret;
C_noret_decl(f_10830)
static void C_ccall f_10830(C_word c,C_word *av) C_noret;
C_noret_decl(f_10836)
static void C_ccall f_10836(C_word c,C_word *av) C_noret;
C_noret_decl(f_10839)
static void C_ccall f_10839(C_word c,C_word *av) C_noret;
C_noret_decl(f_10844)
static void C_fcall f_10844(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10854)
static void C_ccall f_10854(C_word c,C_word *av) C_noret;
C_noret_decl(f_10869)
static void C_ccall f_10869(C_word c,C_word *av) C_noret;
C_noret_decl(f_10871)
static void C_fcall f_10871(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10896)
static void C_ccall f_10896(C_word c,C_word *av) C_noret;
C_noret_decl(f_10917)
static void C_fcall f_10917(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10942)
static void C_ccall f_10942(C_word c,C_word *av) C_noret;
C_noret_decl(f_10964)
static void C_ccall f_10964(C_word c,C_word *av) C_noret;
C_noret_decl(f_10972)
static void C_fcall f_10972(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11023)
static void C_ccall f_11023(C_word c,C_word *av) C_noret;
C_noret_decl(f_11030)
static void C_ccall f_11030(C_word c,C_word *av) C_noret;
C_noret_decl(f_11033)
static void C_ccall f_11033(C_word c,C_word *av) C_noret;
C_noret_decl(f_11062)
static void C_ccall f_11062(C_word c,C_word *av) C_noret;
C_noret_decl(f_11070)
static void C_ccall f_11070(C_word c,C_word *av) C_noret;
C_noret_decl(f_11086)
static void C_ccall f_11086(C_word c,C_word *av) C_noret;
C_noret_decl(f_11089)
static void C_ccall f_11089(C_word c,C_word *av) C_noret;
C_noret_decl(f_11095)
static void C_ccall f_11095(C_word c,C_word *av) C_noret;
C_noret_decl(f_11115)
static void C_ccall f_11115(C_word c,C_word *av) C_noret;
C_noret_decl(f_11138)
static void C_ccall f_11138(C_word c,C_word *av) C_noret;
C_noret_decl(f_11143)
static void C_fcall f_11143(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11147)
static void C_ccall f_11147(C_word c,C_word *av) C_noret;
C_noret_decl(f_11150)
static void C_ccall f_11150(C_word c,C_word *av) C_noret;
C_noret_decl(f_11156)
static void C_ccall f_11156(C_word c,C_word *av) C_noret;
C_noret_decl(f_11159)
static void C_ccall f_11159(C_word c,C_word *av) C_noret;
C_noret_decl(f_11176)
static void C_fcall f_11176(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11186)
static void C_ccall f_11186(C_word c,C_word *av) C_noret;
C_noret_decl(f_11188)
static void C_fcall f_11188(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11213)
static void C_ccall f_11213(C_word c,C_word *av) C_noret;
C_noret_decl(f_11224)
static void C_ccall f_11224(C_word c,C_word *av) C_noret;
C_noret_decl(f_11232)
static void C_ccall f_11232(C_word c,C_word *av) C_noret;
C_noret_decl(f_11240)
static void C_ccall f_11240(C_word c,C_word *av) C_noret;
C_noret_decl(f_11253)
static void C_ccall f_11253(C_word c,C_word *av) C_noret;
C_noret_decl(f_11255)
static void C_fcall f_11255(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11303)
static void C_fcall f_11303(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11328)
static void C_ccall f_11328(C_word c,C_word *av) C_noret;
C_noret_decl(f_11347)
static void C_ccall f_11347(C_word c,C_word *av) C_noret;
C_noret_decl(f_11352)
static void C_fcall f_11352(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11362)
static void C_ccall f_11362(C_word c,C_word *av) C_noret;
C_noret_decl(f_11364)
static void C_fcall f_11364(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11389)
static void C_ccall f_11389(C_word c,C_word *av) C_noret;
C_noret_decl(f_11411)
static void C_fcall f_11411(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11462)
static void C_fcall f_11462(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11506)
static void C_ccall f_11506(C_word c,C_word *av) C_noret;
C_noret_decl(f_11553)
static void C_ccall f_11553(C_word c,C_word *av) C_noret;
C_noret_decl(f_11581)
static void C_ccall f_11581(C_word c,C_word *av) C_noret;
C_noret_decl(f_11585)
static void C_ccall f_11585(C_word c,C_word *av) C_noret;
C_noret_decl(f_11589)
static void C_ccall f_11589(C_word c,C_word *av) C_noret;
C_noret_decl(f_11608)
static void C_ccall f_11608(C_word c,C_word *av) C_noret;
C_noret_decl(f_11620)
static void C_ccall f_11620(C_word c,C_word *av) C_noret;
C_noret_decl(f_11622)
static void C_fcall f_11622(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11664)
static void C_fcall f_11664(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11674)
static void C_ccall f_11674(C_word c,C_word *av) C_noret;
C_noret_decl(f_11696)
static void C_ccall f_11696(C_word c,C_word *av) C_noret;
C_noret_decl(f_11723)
static void C_ccall f_11723(C_word c,C_word *av) C_noret;
C_noret_decl(f_11743)
static void C_fcall f_11743(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11796)
static void C_ccall f_11796(C_word c,C_word *av) C_noret;
C_noret_decl(f_11843)
static void C_ccall f_11843(C_word c,C_word *av) C_noret;
C_noret_decl(f_11860)
static void C_ccall f_11860(C_word c,C_word *av) C_noret;
C_noret_decl(f_11866)
static void C_fcall f_11866(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11880)
static void C_ccall f_11880(C_word c,C_word *av) C_noret;
C_noret_decl(f_11884)
static void C_ccall f_11884(C_word c,C_word *av) C_noret;
C_noret_decl(f_11890)
static void C_ccall f_11890(C_word c,C_word *av) C_noret;
C_noret_decl(f_11928)
static void C_ccall f_11928(C_word c,C_word *av) C_noret;
C_noret_decl(f_11932)
static void C_ccall f_11932(C_word c,C_word *av) C_noret;
C_noret_decl(f_11935)
static void C_ccall f_11935(C_word c,C_word *av) C_noret;
C_noret_decl(f_11938)
static void C_ccall f_11938(C_word c,C_word *av) C_noret;
C_noret_decl(f_11973)
static void C_fcall f_11973(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12017)
static void C_ccall f_12017(C_word c,C_word *av) C_noret;
C_noret_decl(f_12019)
static void C_fcall f_12019(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12044)
static void C_ccall f_12044(C_word c,C_word *av) C_noret;
C_noret_decl(f_12059)
static void C_fcall f_12059(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12088)
static void C_ccall f_12088(C_word c,C_word *av) C_noret;
C_noret_decl(f_12090)
static void C_fcall f_12090(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12115)
static void C_ccall f_12115(C_word c,C_word *av) C_noret;
C_noret_decl(f_12124)
static void C_ccall f_12124(C_word c,C_word *av) C_noret;
C_noret_decl(f_12127)
static void C_fcall f_12127(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12133)
static void C_ccall f_12133(C_word c,C_word *av) C_noret;
C_noret_decl(f_12181)
static void C_ccall f_12181(C_word c,C_word *av) C_noret;
C_noret_decl(f_12184)
static void C_ccall f_12184(C_word c,C_word *av) C_noret;
C_noret_decl(f_12190)
static void C_ccall f_12190(C_word c,C_word *av) C_noret;
C_noret_decl(f_12198)
static void C_ccall f_12198(C_word c,C_word *av) C_noret;
C_noret_decl(f_12206)
static void C_fcall f_12206(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12216)
static void C_ccall f_12216(C_word c,C_word *av) C_noret;
C_noret_decl(f_12235)
static void C_ccall f_12235(C_word c,C_word *av) C_noret;
C_noret_decl(f_12239)
static void C_ccall f_12239(C_word c,C_word *av) C_noret;
C_noret_decl(f_12244)
static void C_ccall f_12244(C_word c,C_word *av) C_noret;
C_noret_decl(f_12250)
static void C_ccall f_12250(C_word c,C_word *av) C_noret;
C_noret_decl(f_12253)
static void C_ccall f_12253(C_word c,C_word *av) C_noret;
C_noret_decl(f_12258)
static void C_fcall f_12258(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12268)
static void C_ccall f_12268(C_word c,C_word *av) C_noret;
C_noret_decl(f_12283)
static void C_ccall f_12283(C_word c,C_word *av) C_noret;
C_noret_decl(f_12285)
static void C_ccall f_12285(C_word c,C_word *av) C_noret;
C_noret_decl(f_12292)
static void C_ccall f_12292(C_word c,C_word *av) C_noret;
C_noret_decl(f_12313)
static void C_fcall f_12313(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12347)
static void C_ccall f_12347(C_word c,C_word *av) C_noret;
C_noret_decl(f_12350)
static void C_fcall f_12350(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12375)
static void C_ccall f_12375(C_word c,C_word *av) C_noret;
C_noret_decl(f_12381)
static void C_ccall f_12381(C_word c,C_word *av) C_noret;
C_noret_decl(f_12407)
static void C_ccall f_12407(C_word c,C_word *av) C_noret;
C_noret_decl(f_12440)
static void C_ccall f_12440(C_word c,C_word *av) C_noret;
C_noret_decl(f_12442)
static void C_ccall f_12442(C_word c,C_word *av) C_noret;
C_noret_decl(f_12448)
static void C_ccall f_12448(C_word c,C_word *av) C_noret;
C_noret_decl(f_12454)
static void C_fcall f_12454(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12458)
static void C_ccall f_12458(C_word c,C_word *av) C_noret;
C_noret_decl(f_12481)
static void C_ccall f_12481(C_word c,C_word *av) C_noret;
C_noret_decl(f_12492)
static void C_ccall f_12492(C_word c,C_word *av) C_noret;
C_noret_decl(f_12498)
static void C_ccall f_12498(C_word c,C_word *av) C_noret;
C_noret_decl(f_12501)
static void C_fcall f_12501(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12509)
static C_word C_fcall f_12509(C_word t0,C_word t1);
C_noret_decl(f_12535)
static void C_fcall f_12535(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12557)
static void C_ccall f_12557(C_word c,C_word *av) C_noret;
C_noret_decl(f_12582)
static void C_fcall f_12582(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12604)
static void C_ccall f_12604(C_word c,C_word *av) C_noret;
C_noret_decl(f_12622)
static void C_fcall f_12622(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12653)
static void C_ccall f_12653(C_word c,C_word *av) C_noret;
C_noret_decl(f_12705)
static void C_ccall f_12705(C_word c,C_word *av) C_noret;
C_noret_decl(f_12711)
static void C_ccall f_12711(C_word c,C_word *av) C_noret;
C_noret_decl(f_12731)
static void C_ccall f_12731(C_word c,C_word *av) C_noret;
C_noret_decl(f_12737)
static void C_ccall f_12737(C_word c,C_word *av) C_noret;
C_noret_decl(f_12763)
static void C_fcall f_12763(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12777)
static void C_fcall f_12777(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12785)
static void C_ccall f_12785(C_word c,C_word *av) C_noret;
C_noret_decl(f_12840)
static void C_ccall f_12840(C_word c,C_word *av) C_noret;
C_noret_decl(f_12869)
static void C_ccall f_12869(C_word c,C_word *av) C_noret;
C_noret_decl(f_12966)
static void C_ccall f_12966(C_word c,C_word *av) C_noret;
C_noret_decl(f_12972)
static void C_ccall f_12972(C_word c,C_word *av) C_noret;
C_noret_decl(f_12979)
static void C_fcall f_12979(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12982)
static void C_ccall f_12982(C_word c,C_word *av) C_noret;
C_noret_decl(f_13005)
static void C_ccall f_13005(C_word c,C_word *av) C_noret;
C_noret_decl(f_13007)
static void C_ccall f_13007(C_word c,C_word *av) C_noret;
C_noret_decl(f_13013)
static void C_ccall f_13013(C_word c,C_word *av) C_noret;
C_noret_decl(f_13020)
static void C_fcall f_13020(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13023)
static void C_ccall f_13023(C_word c,C_word *av) C_noret;
C_noret_decl(f_13042)
static void C_ccall f_13042(C_word c,C_word *av) C_noret;
C_noret_decl(f_13044)
static void C_ccall f_13044(C_word c,C_word *av) C_noret;
C_noret_decl(f_13050)
static void C_ccall f_13050(C_word c,C_word *av) C_noret;
C_noret_decl(f_13063)
static void C_ccall f_13063(C_word c,C_word *av) C_noret;
C_noret_decl(f_13091)
static void C_ccall f_13091(C_word c,C_word *av) C_noret;
C_noret_decl(f_13093)
static void C_ccall f_13093(C_word c,C_word *av) C_noret;
C_noret_decl(f_13114)
static void C_ccall f_13114(C_word c,C_word *av) C_noret;
C_noret_decl(f_13121)
static void C_ccall f_13121(C_word c,C_word *av) C_noret;
C_noret_decl(f_13127)
static void C_ccall f_13127(C_word c,C_word *av) C_noret;
C_noret_decl(f_13133)
static void C_ccall f_13133(C_word c,C_word *av) C_noret;
C_noret_decl(f_13142)
static void C_ccall f_13142(C_word c,C_word *av) C_noret;
C_noret_decl(f_13151)
static void C_ccall f_13151(C_word c,C_word *av) C_noret;
C_noret_decl(f_13160)
static void C_ccall f_13160(C_word c,C_word *av) C_noret;
C_noret_decl(f_13169)
static void C_ccall f_13169(C_word c,C_word *av) C_noret;
C_noret_decl(f_13178)
static void C_ccall f_13178(C_word c,C_word *av) C_noret;
C_noret_decl(f_13204)
static void C_ccall f_13204(C_word c,C_word *av) C_noret;
C_noret_decl(f_13207)
static void C_ccall f_13207(C_word c,C_word *av) C_noret;
C_noret_decl(f_13218)
static void C_ccall f_13218(C_word c,C_word *av) C_noret;
C_noret_decl(f_13220)
static void C_ccall f_13220(C_word c,C_word *av) C_noret;
C_noret_decl(f_13274)
static void C_ccall f_13274(C_word c,C_word *av) C_noret;
C_noret_decl(f_13280)
static void C_ccall f_13280(C_word c,C_word *av) C_noret;
C_noret_decl(f_13286)
static void C_ccall f_13286(C_word c,C_word *av) C_noret;
C_noret_decl(f_13292)
static void C_fcall f_13292(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13317)
static void C_fcall f_13317(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13332)
static void C_fcall f_13332(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13350)
static void C_ccall f_13350(C_word c,C_word *av) C_noret;
C_noret_decl(f_13400)
static void C_ccall f_13400(C_word c,C_word *av) C_noret;
C_noret_decl(f_13415)
static void C_fcall f_13415(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13455)
static void C_fcall f_13455(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13458)
static void C_ccall f_13458(C_word c,C_word *av) C_noret;
C_noret_decl(f_13473)
static void C_fcall f_13473(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13497)
static void C_fcall f_13497(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13523)
static void C_fcall f_13523(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13529)
static void C_ccall f_13529(C_word c,C_word *av) C_noret;
C_noret_decl(f_13535)
static void C_ccall f_13535(C_word c,C_word *av) C_noret;
C_noret_decl(f_13538)
static void C_ccall f_13538(C_word c,C_word *av) C_noret;
C_noret_decl(f_13541)
static void C_ccall f_13541(C_word c,C_word *av) C_noret;
C_noret_decl(f_13544)
static void C_ccall f_13544(C_word c,C_word *av) C_noret;
C_noret_decl(f_13566)
static void C_fcall f_13566(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13572)
static void C_ccall f_13572(C_word c,C_word *av) C_noret;
C_noret_decl(f_13578)
static void C_ccall f_13578(C_word c,C_word *av) C_noret;
C_noret_decl(f_13581)
static void C_ccall f_13581(C_word c,C_word *av) C_noret;
C_noret_decl(f_13584)
static void C_ccall f_13584(C_word c,C_word *av) C_noret;
C_noret_decl(f_13587)
static void C_ccall f_13587(C_word c,C_word *av) C_noret;
C_noret_decl(f_13610)
static void C_fcall f_13610(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13613)
static void C_ccall f_13613(C_word c,C_word *av) C_noret;
C_noret_decl(f_13654)
static void C_fcall f_13654(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13657)
static void C_ccall f_13657(C_word c,C_word *av) C_noret;
C_noret_decl(f_13672)
static void C_fcall f_13672(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13699)
static void C_fcall f_13699(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13742)
static void C_ccall f_13742(C_word c,C_word *av) C_noret;
C_noret_decl(f_13746)
static void C_fcall f_13746(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13773)
static void C_fcall f_13773(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13776)
static void C_ccall f_13776(C_word c,C_word *av) C_noret;
C_noret_decl(f_13811)
static void C_ccall f_13811(C_word c,C_word *av) C_noret;
C_noret_decl(f_13847)
static void C_ccall f_13847(C_word c,C_word *av) C_noret;
C_noret_decl(f_14350)
static void C_ccall f_14350(C_word c,C_word *av) C_noret;
C_noret_decl(f_14356)
static void C_fcall f_14356(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14366)
static void C_ccall f_14366(C_word c,C_word *av) C_noret;
C_noret_decl(f_14377)
static void C_fcall f_14377(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14387)
static void C_ccall f_14387(C_word c,C_word *av) C_noret;
C_noret_decl(f_14398)
static void C_ccall f_14398(C_word c,C_word *av) C_noret;
C_noret_decl(f_14402)
static void C_ccall f_14402(C_word c,C_word *av) C_noret;
C_noret_decl(f_14413)
static void C_ccall f_14413(C_word c,C_word *av) C_noret;
C_noret_decl(f_14417)
static void C_ccall f_14417(C_word c,C_word *av) C_noret;
C_noret_decl(f_14428)
static void C_ccall f_14428(C_word c,C_word *av) C_noret;
C_noret_decl(f_14434)
static void C_ccall f_14434(C_word c,C_word *av) C_noret;
C_noret_decl(f_14438)
static void C_ccall f_14438(C_word c,C_word *av) C_noret;
C_noret_decl(f_14442)
static void C_fcall f_14442(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14461)
static void C_ccall f_14461(C_word c,C_word *av) C_noret;
C_noret_decl(f_14467)
static void C_ccall f_14467(C_word c,C_word *av) C_noret;
C_noret_decl(f_14470)
static void C_fcall f_14470(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14479)
static void C_ccall f_14479(C_word c,C_word *av) C_noret;
C_noret_decl(f_14489)
static void C_fcall f_14489(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14498)
static void C_fcall f_14498(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14510)
static void C_fcall f_14510(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14522)
static void C_fcall f_14522(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14534)
static void C_fcall f_14534(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14540)
static void C_ccall f_14540(C_word c,C_word *av) C_noret;
C_noret_decl(f_14544)
static void C_fcall f_14544(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14571)
static void C_fcall f_14571(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14936)
static void C_ccall f_14936(C_word c,C_word *av) C_noret;
C_noret_decl(f_14942)
static void C_ccall f_14942(C_word c,C_word *av) C_noret;
C_noret_decl(f_14954)
static void C_ccall f_14954(C_word c,C_word *av) C_noret;
C_noret_decl(f_14964)
static void C_fcall f_14964(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14976)
static void C_fcall f_14976(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14982)
static void C_ccall f_14982(C_word c,C_word *av) C_noret;
C_noret_decl(f_14986)
static void C_fcall f_14986(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15013)
static void C_fcall f_15013(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15386)
static void C_ccall f_15386(C_word c,C_word *av) C_noret;
C_noret_decl(f_15392)
static void C_ccall f_15392(C_word c,C_word *av) C_noret;
C_noret_decl(f_15396)
static void C_ccall f_15396(C_word c,C_word *av) C_noret;
C_noret_decl(f_15512)
static void C_fcall f_15512(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15540)
static void C_ccall f_15540(C_word c,C_word *av) C_noret;
C_noret_decl(f_15660)
static void C_ccall f_15660(C_word c,C_word *av) C_noret;
C_noret_decl(f_15667)
static void C_ccall f_15667(C_word c,C_word *av) C_noret;
C_noret_decl(f_15670)
static void C_ccall f_15670(C_word c,C_word *av) C_noret;
C_noret_decl(f_15673)
static void C_ccall f_15673(C_word c,C_word *av) C_noret;
C_noret_decl(f_15697)
static void C_fcall f_15697(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15772)
static void C_fcall f_15772(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15859)
static void C_fcall f_15859(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15892)
static void C_fcall f_15892(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15910)
static void C_fcall f_15910(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15932)
static void C_fcall f_15932(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16314)
static void C_ccall f_16314(C_word c,C_word *av) C_noret;
C_noret_decl(f_16318)
static void C_ccall f_16318(C_word c,C_word *av) C_noret;
C_noret_decl(f_16320)
static void C_fcall f_16320(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16352)
static void C_fcall f_16352(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16360)
static void C_fcall f_16360(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16370)
static void C_ccall f_16370(C_word c,C_word *av) C_noret;
C_noret_decl(f_16416)
static void C_fcall f_16416(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16424)
static void C_fcall f_16424(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16434)
static void C_ccall f_16434(C_word c,C_word *av) C_noret;
C_noret_decl(f_16469)
static void C_ccall f_16469(C_word c,C_word *av) C_noret;
C_noret_decl(f_16472)
static void C_fcall f_16472(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16506)
static void C_fcall f_16506(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16525)
static void C_ccall f_16525(C_word c,C_word *av) C_noret;
C_noret_decl(f_16531)
static void C_ccall f_16531(C_word c,C_word *av) C_noret;
C_noret_decl(f_16535)
static void C_ccall f_16535(C_word c,C_word *av) C_noret;
C_noret_decl(f_16561)
static void C_ccall f_16561(C_word c,C_word *av) C_noret;
C_noret_decl(f_16570)
static void C_ccall f_16570(C_word c,C_word *av) C_noret;
C_noret_decl(f_16581)
static void C_ccall f_16581(C_word c,C_word *av) C_noret;
C_noret_decl(f_16600)
static void C_ccall f_16600(C_word c,C_word *av) C_noret;
C_noret_decl(f_16612)
static void C_ccall f_16612(C_word c,C_word *av) C_noret;
C_noret_decl(f_16656)
static void C_fcall f_16656(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16658)
static void C_fcall f_16658(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16670)
static void C_fcall f_16670(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16680)
static void C_ccall f_16680(C_word c,C_word *av) C_noret;
C_noret_decl(f_16694)
static void C_ccall f_16694(C_word c,C_word *av) C_noret;
C_noret_decl(f_16699)
static void C_ccall f_16699(C_word c,C_word *av) C_noret;
C_noret_decl(f_16723)
static void C_ccall f_16723(C_word c,C_word *av) C_noret;
C_noret_decl(f_16729)
static void C_ccall f_16729(C_word c,C_word *av) C_noret;
C_noret_decl(f_16735)
static void C_ccall f_16735(C_word c,C_word *av) C_noret;
C_noret_decl(f_16744)
static void C_ccall f_16744(C_word c,C_word *av) C_noret;
C_noret_decl(f_16752)
static void C_ccall f_16752(C_word c,C_word *av) C_noret;
C_noret_decl(f_16758)
static void C_ccall f_16758(C_word c,C_word *av) C_noret;
C_noret_decl(f_16761)
static void C_ccall f_16761(C_word c,C_word *av) C_noret;
C_noret_decl(f_16764)
static void C_ccall f_16764(C_word c,C_word *av) C_noret;
C_noret_decl(f_16767)
static void C_ccall f_16767(C_word c,C_word *av) C_noret;
C_noret_decl(f_16770)
static void C_ccall f_16770(C_word c,C_word *av) C_noret;
C_noret_decl(f_16775)
static void C_ccall f_16775(C_word c,C_word *av) C_noret;
C_noret_decl(f_16779)
static void C_ccall f_16779(C_word c,C_word *av) C_noret;
C_noret_decl(f_16791)
static void C_ccall f_16791(C_word c,C_word *av) C_noret;
C_noret_decl(f_16796)
static void C_ccall f_16796(C_word c,C_word *av) C_noret;
C_noret_decl(f_16798)
static void C_ccall f_16798(C_word c,C_word *av) C_noret;
C_noret_decl(f_16804)
static void C_ccall f_16804(C_word c,C_word *av) C_noret;
C_noret_decl(f_16811)
static void C_ccall f_16811(C_word c,C_word *av) C_noret;
C_noret_decl(f_16814)
static void C_fcall f_16814(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16818)
static void C_ccall f_16818(C_word c,C_word *av) C_noret;
C_noret_decl(f_16824)
static void C_ccall f_16824(C_word c,C_word *av) C_noret;
C_noret_decl(f_16830)
static void C_ccall f_16830(C_word c,C_word *av) C_noret;
C_noret_decl(f_16857)
static void C_ccall f_16857(C_word c,C_word *av) C_noret;
C_noret_decl(f_16859)
static void C_fcall f_16859(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_16873)
static void C_ccall f_16873(C_word c,C_word *av) C_noret;
C_noret_decl(f_16883)
static void C_ccall f_16883(C_word c,C_word *av) C_noret;
C_noret_decl(f_16896)
static void C_ccall f_16896(C_word c,C_word *av) C_noret;
C_noret_decl(f_16911)
static void C_ccall f_16911(C_word c,C_word *av) C_noret;
C_noret_decl(f_16915)
static void C_ccall f_16915(C_word c,C_word *av) C_noret;
C_noret_decl(f_16922)
static void C_ccall f_16922(C_word c,C_word *av) C_noret;
C_noret_decl(f_16926)
static void C_ccall f_16926(C_word c,C_word *av) C_noret;
C_noret_decl(f_16931)
static void C_ccall f_16931(C_word c,C_word *av) C_noret;
C_noret_decl(f_16935)
static void C_ccall f_16935(C_word c,C_word *av) C_noret;
C_noret_decl(f_16943)
static void C_ccall f_16943(C_word c,C_word *av) C_noret;
C_noret_decl(f_16949)
static void C_ccall f_16949(C_word c,C_word *av) C_noret;
C_noret_decl(f_16956)
static void C_ccall f_16956(C_word c,C_word *av) C_noret;
C_noret_decl(f_16959)
static void C_ccall f_16959(C_word c,C_word *av) C_noret;
C_noret_decl(f_16962)
static void C_ccall f_16962(C_word c,C_word *av) C_noret;
C_noret_decl(f_16967)
static void C_ccall f_16967(C_word c,C_word *av) C_noret;
C_noret_decl(f_16987)
static void C_ccall f_16987(C_word c,C_word *av) C_noret;
C_noret_decl(f_16991)
static void C_ccall f_16991(C_word c,C_word *av) C_noret;
C_noret_decl(f_17002)
static void C_ccall f_17002(C_word c,C_word *av) C_noret;
C_noret_decl(f_17017)
static void C_ccall f_17017(C_word c,C_word *av) C_noret;
C_noret_decl(f_17029)
static void C_ccall f_17029(C_word c,C_word *av) C_noret;
C_noret_decl(f_17036)
static void C_ccall f_17036(C_word c,C_word *av) C_noret;
C_noret_decl(f_17066)
static void C_ccall f_17066(C_word c,C_word *av) C_noret;
C_noret_decl(f_17090)
static void C_ccall f_17090(C_word c,C_word *av) C_noret;
C_noret_decl(f_17105)
static void C_ccall f_17105(C_word c,C_word *av) C_noret;
C_noret_decl(f_17108)
static void C_ccall f_17108(C_word c,C_word *av) C_noret;
C_noret_decl(f_17114)
static void C_ccall f_17114(C_word c,C_word *av) C_noret;
C_noret_decl(f_17123)
static void C_ccall f_17123(C_word c,C_word *av) C_noret;
C_noret_decl(f_17126)
static void C_ccall f_17126(C_word c,C_word *av) C_noret;
C_noret_decl(f_17165)
static void C_ccall f_17165(C_word c,C_word *av) C_noret;
C_noret_decl(f_17171)
static void C_ccall f_17171(C_word c,C_word *av) C_noret;
C_noret_decl(f_17177)
static void C_ccall f_17177(C_word c,C_word *av) C_noret;
C_noret_decl(f_17180)
static void C_ccall f_17180(C_word c,C_word *av) C_noret;
C_noret_decl(f_17186)
static void C_ccall f_17186(C_word c,C_word *av) C_noret;
C_noret_decl(f_17192)
static void C_ccall f_17192(C_word c,C_word *av) C_noret;
C_noret_decl(f_17198)
static void C_ccall f_17198(C_word c,C_word *av) C_noret;
C_noret_decl(f_17204)
static void C_ccall f_17204(C_word c,C_word *av) C_noret;
C_noret_decl(f_17226)
static void C_ccall f_17226(C_word c,C_word *av) C_noret;
C_noret_decl(f_17228)
static void C_fcall f_17228(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17262)
static void C_fcall f_17262(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17296)
static void C_ccall f_17296(C_word c,C_word *av) C_noret;
C_noret_decl(f_17299)
static void C_ccall f_17299(C_word c,C_word *av) C_noret;
C_noret_decl(f_17327)
static void C_ccall f_17327(C_word c,C_word *av) C_noret;
C_noret_decl(f_17334)
static void C_ccall f_17334(C_word c,C_word *av) C_noret;
C_noret_decl(f_17349)
static void C_ccall f_17349(C_word c,C_word *av) C_noret;
C_noret_decl(f_17355)
static void C_ccall f_17355(C_word c,C_word *av) C_noret;
C_noret_decl(f_17358)
static void C_ccall f_17358(C_word c,C_word *av) C_noret;
C_noret_decl(f_17395)
static void C_ccall f_17395(C_word c,C_word *av) C_noret;
C_noret_decl(f_17410)
static void C_ccall f_17410(C_word c,C_word *av) C_noret;
C_noret_decl(f_17420)
static void C_ccall f_17420(C_word c,C_word *av) C_noret;
C_noret_decl(f_17423)
static void C_ccall f_17423(C_word c,C_word *av) C_noret;
C_noret_decl(f_17435)
static void C_ccall f_17435(C_word c,C_word *av) C_noret;
C_noret_decl(f_17441)
static void C_ccall f_17441(C_word c,C_word *av) C_noret;
C_noret_decl(f_17447)
static void C_ccall f_17447(C_word c,C_word *av) C_noret;
C_noret_decl(f_17450)
static void C_ccall f_17450(C_word c,C_word *av) C_noret;
C_noret_decl(f_17452)
static void C_fcall f_17452(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17459)
static void C_fcall f_17459(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17465)
static void C_ccall f_17465(C_word c,C_word *av) C_noret;
C_noret_decl(f_17476)
static void C_ccall f_17476(C_word c,C_word *av) C_noret;
C_noret_decl(f_17524)
static void C_ccall f_17524(C_word c,C_word *av) C_noret;
C_noret_decl(f_17526)
static C_word C_fcall f_17526(C_word t0,C_word t1);
C_noret_decl(f_17532)
static void C_ccall f_17532(C_word c,C_word *av) C_noret;
C_noret_decl(f_17536)
static void C_ccall f_17536(C_word c,C_word *av) C_noret;
C_noret_decl(f_17541)
static void C_fcall f_17541(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17569)
static void C_ccall f_17569(C_word c,C_word *av) C_noret;
C_noret_decl(f_17577)
static void C_ccall f_17577(C_word c,C_word *av) C_noret;
C_noret_decl(f_17580)
static void C_ccall f_17580(C_word c,C_word *av) C_noret;
C_noret_decl(f_17583)
static void C_ccall f_17583(C_word c,C_word *av) C_noret;
C_noret_decl(f_17586)
static void C_ccall f_17586(C_word c,C_word *av) C_noret;
C_noret_decl(f_17589)
static void C_ccall f_17589(C_word c,C_word *av) C_noret;
C_noret_decl(f_17592)
static void C_ccall f_17592(C_word c,C_word *av) C_noret;
C_noret_decl(f_17593)
static void C_fcall f_17593(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17603)
static void C_ccall f_17603(C_word c,C_word *av) C_noret;
C_noret_decl(f_17609)
static void C_ccall f_17609(C_word c,C_word *av) C_noret;
C_noret_decl(f_17621)
static void C_ccall f_17621(C_word c,C_word *av) C_noret;
C_noret_decl(f_17624)
static void C_ccall f_17624(C_word c,C_word *av) C_noret;
C_noret_decl(f_17627)
static void C_ccall f_17627(C_word c,C_word *av) C_noret;
C_noret_decl(f_17632)
static void C_fcall f_17632(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17645)
static void C_ccall f_17645(C_word c,C_word *av) C_noret;
C_noret_decl(f_17648)
static void C_ccall f_17648(C_word c,C_word *av) C_noret;
C_noret_decl(f_17665)
static void C_fcall f_17665(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17675)
static void C_ccall f_17675(C_word c,C_word *av) C_noret;
C_noret_decl(f_17688)
static void C_ccall f_17688(C_word c,C_word *av) C_noret;
C_noret_decl(f_17692)
static void C_ccall f_17692(C_word c,C_word *av) C_noret;
C_noret_decl(f_17710)
static void C_ccall f_17710(C_word c,C_word *av) C_noret;
C_noret_decl(f_17714)
static void C_ccall f_17714(C_word c,C_word *av) C_noret;
C_noret_decl(f_17731)
static void C_ccall f_17731(C_word c,C_word *av) C_noret;
C_noret_decl(f_17737)
static void C_ccall f_17737(C_word c,C_word *av) C_noret;
C_noret_decl(f_17747)
static void C_ccall f_17747(C_word c,C_word *av) C_noret;
C_noret_decl(f_17750)
static void C_ccall f_17750(C_word c,C_word *av) C_noret;
C_noret_decl(f_17766)
static void C_ccall f_17766(C_word c,C_word *av) C_noret;
C_noret_decl(f_17771)
static void C_fcall f_17771(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17775)
static void C_ccall f_17775(C_word c,C_word *av) C_noret;
C_noret_decl(f_17792)
static void C_ccall f_17792(C_word c,C_word *av) C_noret;
C_noret_decl(f_17803)
static void C_ccall f_17803(C_word c,C_word *av) C_noret;
C_noret_decl(f_17815)
static void C_ccall f_17815(C_word c,C_word *av) C_noret;
C_noret_decl(f_17818)
static void C_ccall f_17818(C_word c,C_word *av) C_noret;
C_noret_decl(f_17826)
static void C_ccall f_17826(C_word c,C_word *av) C_noret;
C_noret_decl(f_17831)
static void C_ccall f_17831(C_word c,C_word *av) C_noret;
C_noret_decl(f_17844)
static void C_ccall f_17844(C_word c,C_word *av) C_noret;
C_noret_decl(f_17855)
static void C_ccall f_17855(C_word c,C_word *av) C_noret;
C_noret_decl(f_17877)
static void C_ccall f_17877(C_word c,C_word *av) C_noret;
C_noret_decl(f_17879)
static void C_ccall f_17879(C_word c,C_word *av) C_noret;
C_noret_decl(f_17899)
static void C_ccall f_17899(C_word c,C_word *av) C_noret;
C_noret_decl(f_17919)
static void C_ccall f_17919(C_word c,C_word *av) C_noret;
C_noret_decl(f_17927)
static void C_ccall f_17927(C_word c,C_word *av) C_noret;
C_noret_decl(f_17936)
static void C_ccall f_17936(C_word c,C_word *av) C_noret;
C_noret_decl(f_17941)
static void C_ccall f_17941(C_word c,C_word *av) C_noret;
C_noret_decl(f_17945)
static void C_ccall f_17945(C_word c,C_word *av) C_noret;
C_noret_decl(f_17966)
static void C_ccall f_17966(C_word c,C_word *av) C_noret;
C_noret_decl(f_17981)
static void C_ccall f_17981(C_word c,C_word *av) C_noret;
C_noret_decl(f_17987)
static void C_ccall f_17987(C_word c,C_word *av) C_noret;
C_noret_decl(f_17998)
static void C_ccall f_17998(C_word c,C_word *av) C_noret;
C_noret_decl(f_18009)
static void C_ccall f_18009(C_word c,C_word *av) C_noret;
C_noret_decl(f_18020)
static void C_ccall f_18020(C_word c,C_word *av) C_noret;
C_noret_decl(f_18024)
static void C_ccall f_18024(C_word c,C_word *av) C_noret;
C_noret_decl(f_18030)
static void C_ccall f_18030(C_word c,C_word *av) C_noret;
C_noret_decl(f_18042)
static void C_ccall f_18042(C_word c,C_word *av) C_noret;
C_noret_decl(f_18046)
static void C_ccall f_18046(C_word c,C_word *av) C_noret;
C_noret_decl(f_18058)
static void C_ccall f_18058(C_word c,C_word *av) C_noret;
C_noret_decl(f_18066)
static void C_fcall f_18066(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18076)
static void C_ccall f_18076(C_word c,C_word *av) C_noret;
C_noret_decl(f_18091)
static void C_ccall f_18091(C_word c,C_word *av) C_noret;
C_noret_decl(f_18097)
static void C_ccall f_18097(C_word c,C_word *av) C_noret;
C_noret_decl(f_18100)
static void C_ccall f_18100(C_word c,C_word *av) C_noret;
C_noret_decl(f_18103)
static void C_ccall f_18103(C_word c,C_word *av) C_noret;
C_noret_decl(f_18106)
static void C_ccall f_18106(C_word c,C_word *av) C_noret;
C_noret_decl(f_18109)
static void C_ccall f_18109(C_word c,C_word *av) C_noret;
C_noret_decl(f_18113)
static void C_ccall f_18113(C_word c,C_word *av) C_noret;
C_noret_decl(f_18115)
static void C_ccall f_18115(C_word c,C_word *av) C_noret;
C_noret_decl(f_18122)
static void C_ccall f_18122(C_word c,C_word *av) C_noret;
C_noret_decl(f_18129)
static void C_ccall f_18129(C_word c,C_word *av) C_noret;
C_noret_decl(f_18140)
static void C_ccall f_18140(C_word c,C_word *av) C_noret;
C_noret_decl(f_18144)
static void C_ccall f_18144(C_word c,C_word *av) C_noret;
C_noret_decl(f_18147)
static void C_ccall f_18147(C_word c,C_word *av) C_noret;
C_noret_decl(f_18152)
static void C_ccall f_18152(C_word c,C_word *av) C_noret;
C_noret_decl(f_18158)
static void C_ccall f_18158(C_word c,C_word *av) C_noret;
C_noret_decl(f_18165)
static void C_ccall f_18165(C_word c,C_word *av) C_noret;
C_noret_decl(f_18168)
static void C_ccall f_18168(C_word c,C_word *av) C_noret;
C_noret_decl(f_18171)
static void C_ccall f_18171(C_word c,C_word *av) C_noret;
C_noret_decl(f_18174)
static void C_ccall f_18174(C_word c,C_word *av) C_noret;
C_noret_decl(f_5348)
static void C_ccall f_5348(C_word c,C_word *av) C_noret;
C_noret_decl(f_5351)
static void C_ccall f_5351(C_word c,C_word *av) C_noret;
C_noret_decl(f_5354)
static void C_ccall f_5354(C_word c,C_word *av) C_noret;
C_noret_decl(f_5357)
static void C_ccall f_5357(C_word c,C_word *av) C_noret;
C_noret_decl(f_5360)
static void C_ccall f_5360(C_word c,C_word *av) C_noret;
C_noret_decl(f_5363)
static void C_ccall f_5363(C_word c,C_word *av) C_noret;
C_noret_decl(f_5366)
static void C_ccall f_5366(C_word c,C_word *av) C_noret;
C_noret_decl(f_5369)
static void C_ccall f_5369(C_word c,C_word *av) C_noret;
C_noret_decl(f_5372)
static void C_ccall f_5372(C_word c,C_word *av) C_noret;
C_noret_decl(f_5482)
static void C_fcall f_5482(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5500)
static void C_ccall f_5500(C_word c,C_word *av) C_noret;
C_noret_decl(f_5540)
static void C_fcall f_5540(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_5554)
static void C_ccall f_5554(C_word c,C_word *av) C_noret;
C_noret_decl(f_5743)
static void C_fcall f_5743(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5749)
static void C_fcall f_5749(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5771)
static void C_ccall f_5771(C_word c,C_word *av) C_noret;
C_noret_decl(f_5777)
static void C_fcall f_5777(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5783)
static void C_fcall f_5783(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5793)
static void C_ccall f_5793(C_word c,C_word *av) C_noret;
C_noret_decl(f_5807)
static void C_fcall f_5807(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5813)
static void C_fcall f_5813(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5827)
static void C_ccall f_5827(C_word c,C_word *av) C_noret;
C_noret_decl(f_6036)
static void C_fcall f_6036(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6044)
static void C_fcall f_6044(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6052)
static C_word C_fcall f_6052(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_6070)
static void C_ccall f_6070(C_word c,C_word *av) C_noret;
C_noret_decl(f_6109)
static void C_fcall f_6109(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6143)
static void C_fcall f_6143(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6149)
static C_word C_fcall f_6149(C_word t0);
C_noret_decl(f_6202)
static void C_fcall f_6202(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6208)
static void C_fcall f_6208(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6391)
static void C_fcall f_6391(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6409)
static void C_ccall f_6409(C_word c,C_word *av) C_noret;
C_noret_decl(f_6504)
static void C_fcall f_6504(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6517)
static void C_ccall f_6517(C_word c,C_word *av) C_noret;
C_noret_decl(f_6655)
static void C_ccall f_6655(C_word c,C_word *av) C_noret;
C_noret_decl(f_6659)
static void C_ccall f_6659(C_word c,C_word *av) C_noret;
C_noret_decl(f_6673)
static void C_ccall f_6673(C_word c,C_word *av) C_noret;
C_noret_decl(f_6684)
static void C_ccall f_6684(C_word c,C_word *av) C_noret;
C_noret_decl(f_6687)
static void C_fcall f_6687(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6702)
static void C_ccall f_6702(C_word c,C_word *av) C_noret;
C_noret_decl(f_6708)
static void C_ccall f_6708(C_word c,C_word *av) C_noret;
C_noret_decl(f_6711)
static void C_fcall f_6711(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6717)
static void C_ccall f_6717(C_word c,C_word *av) C_noret;
C_noret_decl(f_6721)
static void C_ccall f_6721(C_word c,C_word *av) C_noret;
C_noret_decl(f_6724)
static void C_ccall f_6724(C_word c,C_word *av) C_noret;
C_noret_decl(f_6733)
static void C_ccall f_6733(C_word c,C_word *av) C_noret;
C_noret_decl(f_6741)
static void C_ccall f_6741(C_word c,C_word *av) C_noret;
C_noret_decl(f_6748)
static void C_ccall f_6748(C_word c,C_word *av) C_noret;
C_noret_decl(f_6753)
static void C_fcall f_6753(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6763)
static void C_ccall f_6763(C_word c,C_word *av) C_noret;
C_noret_decl(f_6776)
static void C_fcall f_6776(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6783)
static void C_ccall f_6783(C_word c,C_word *av) C_noret;
C_noret_decl(f_6786)
static void C_ccall f_6786(C_word c,C_word *av) C_noret;
C_noret_decl(f_6795)
static void C_ccall f_6795(C_word c,C_word *av) C_noret;
C_noret_decl(f_6798)
static void C_ccall f_6798(C_word c,C_word *av) C_noret;
C_noret_decl(f_6801)
static void C_ccall f_6801(C_word c,C_word *av) C_noret;
C_noret_decl(f_6804)
static void C_ccall f_6804(C_word c,C_word *av) C_noret;
C_noret_decl(f_6807)
static void C_ccall f_6807(C_word c,C_word *av) C_noret;
C_noret_decl(f_6810)
static void C_ccall f_6810(C_word c,C_word *av) C_noret;
C_noret_decl(f_6816)
static void C_ccall f_6816(C_word c,C_word *av) C_noret;
C_noret_decl(f_6819)
static void C_ccall f_6819(C_word c,C_word *av) C_noret;
C_noret_decl(f_6826)
static void C_ccall f_6826(C_word c,C_word *av) C_noret;
C_noret_decl(f_6828)
static void C_ccall f_6828(C_word c,C_word *av) C_noret;
C_noret_decl(f_6831)
static void C_fcall f_6831(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6833)
static void C_fcall f_6833(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6840)
static void C_ccall f_6840(C_word c,C_word *av) C_noret;
C_noret_decl(f_6843)
static void C_ccall f_6843(C_word c,C_word *av) C_noret;
C_noret_decl(f_6846)
static void C_ccall f_6846(C_word c,C_word *av) C_noret;
C_noret_decl(f_6860)
static void C_ccall f_6860(C_word c,C_word *av) C_noret;
C_noret_decl(f_6865)
static void C_fcall f_6865(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6875)
static void C_ccall f_6875(C_word c,C_word *av) C_noret;
C_noret_decl(f_6892)
static void C_ccall f_6892(C_word c,C_word *av) C_noret;
C_noret_decl(f_6895)
static void C_ccall f_6895(C_word c,C_word *av) C_noret;
C_noret_decl(f_6898)
static void C_ccall f_6898(C_word c,C_word *av) C_noret;
C_noret_decl(f_6901)
static void C_ccall f_6901(C_word c,C_word *av) C_noret;
C_noret_decl(f_6907)
static void C_ccall f_6907(C_word c,C_word *av) C_noret;
C_noret_decl(f_6916)
static void C_ccall f_6916(C_word c,C_word *av) C_noret;
C_noret_decl(f_6923)
static void C_ccall f_6923(C_word c,C_word *av) C_noret;
C_noret_decl(f_6925)
static void C_ccall f_6925(C_word c,C_word *av) C_noret;
C_noret_decl(f_6929)
static void C_ccall f_6929(C_word c,C_word *av) C_noret;
C_noret_decl(f_6932)
static void C_ccall f_6932(C_word c,C_word *av) C_noret;
C_noret_decl(f_6939)
static void C_ccall f_6939(C_word c,C_word *av) C_noret;
C_noret_decl(f_6941)
static void C_ccall f_6941(C_word c,C_word *av) C_noret;
C_noret_decl(f_6945)
static void C_fcall f_6945(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6948)
static void C_ccall f_6948(C_word c,C_word *av) C_noret;
C_noret_decl(f_6949)
static void C_fcall f_6949(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6959)
static void C_ccall f_6959(C_word c,C_word *av) C_noret;
C_noret_decl(f_6962)
static void C_ccall f_6962(C_word c,C_word *av) C_noret;
C_noret_decl(f_6967)
static void C_fcall f_6967(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6977)
static void C_ccall f_6977(C_word c,C_word *av) C_noret;
C_noret_decl(f_6994)
static void C_ccall f_6994(C_word c,C_word *av) C_noret;
C_noret_decl(f_6997)
static void C_ccall f_6997(C_word c,C_word *av) C_noret;
C_noret_decl(f_7000)
static void C_ccall f_7000(C_word c,C_word *av) C_noret;
C_noret_decl(f_7003)
static void C_ccall f_7003(C_word c,C_word *av) C_noret;
C_noret_decl(f_7006)
static void C_ccall f_7006(C_word c,C_word *av) C_noret;
C_noret_decl(f_7015)
static void C_ccall f_7015(C_word c,C_word *av) C_noret;
C_noret_decl(f_7018)
static void C_ccall f_7018(C_word c,C_word *av) C_noret;
C_noret_decl(f_7021)
static void C_ccall f_7021(C_word c,C_word *av) C_noret;
C_noret_decl(f_7038)
static void C_ccall f_7038(C_word c,C_word *av) C_noret;
C_noret_decl(f_7092)
static void C_ccall f_7092(C_word c,C_word *av) C_noret;
C_noret_decl(f_7098)
static C_word C_fcall f_7098(C_word t0,C_word t1);
C_noret_decl(f_7133)
static void C_ccall f_7133(C_word c,C_word *av) C_noret;
C_noret_decl(f_7139)
static void C_fcall f_7139(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7161)
static void C_ccall f_7161(C_word c,C_word *av) C_noret;
C_noret_decl(f_7170)
static void C_ccall f_7170(C_word c,C_word *av) C_noret;
C_noret_decl(f_7182)
static void C_ccall f_7182(C_word c,C_word *av) C_noret;
C_noret_decl(f_7186)
static void C_ccall f_7186(C_word c,C_word *av) C_noret;
C_noret_decl(f_7188)
static void C_fcall f_7188(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7210)
static void C_fcall f_7210(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7217)
static void C_fcall f_7217(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7221)
static void C_ccall f_7221(C_word c,C_word *av) C_noret;
C_noret_decl(f_7225)
static void C_ccall f_7225(C_word c,C_word *av) C_noret;
C_noret_decl(f_7231)
static void C_ccall f_7231(C_word c,C_word *av) C_noret;
C_noret_decl(f_7253)
static void C_ccall f_7253(C_word c,C_word *av) C_noret;
C_noret_decl(f_7269)
static void C_ccall f_7269(C_word c,C_word *av) C_noret;
C_noret_decl(f_7273)
static void C_ccall f_7273(C_word c,C_word *av) C_noret;
C_noret_decl(f_7294)
static void C_ccall f_7294(C_word c,C_word *av) C_noret;
C_noret_decl(f_7317)
static void C_ccall f_7317(C_word c,C_word *av) C_noret;
C_noret_decl(f_7319)
static void C_ccall f_7319(C_word c,C_word *av) C_noret;
C_noret_decl(f_7326)
static void C_ccall f_7326(C_word c,C_word *av) C_noret;
C_noret_decl(f_7333)
static void C_ccall f_7333(C_word c,C_word *av) C_noret;
C_noret_decl(f_7346)
static void C_ccall f_7346(C_word c,C_word *av) C_noret;
C_noret_decl(f_7377)
static void C_ccall f_7377(C_word c,C_word *av) C_noret;
C_noret_decl(f_7389)
static void C_ccall f_7389(C_word c,C_word *av) C_noret;
C_noret_decl(f_7403)
static void C_ccall f_7403(C_word c,C_word *av) C_noret;
C_noret_decl(f_7405)
static void C_fcall f_7405(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7431)
static void C_ccall f_7431(C_word c,C_word *av) C_noret;
C_noret_decl(f_7445)
static void C_fcall f_7445(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7451)
static void C_fcall f_7451(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7466)
static void C_ccall f_7466(C_word c,C_word *av) C_noret;
C_noret_decl(f_7482)
static void C_ccall f_7482(C_word c,C_word *av) C_noret;
C_noret_decl(f_7490)
static void C_ccall f_7490(C_word c,C_word *av) C_noret;
C_noret_decl(f_7494)
static void C_ccall f_7494(C_word c,C_word *av) C_noret;
C_noret_decl(f_7496)
static void C_ccall f_7496(C_word c,C_word *av) C_noret;
C_noret_decl(f_7507)
static void C_ccall f_7507(C_word c,C_word *av) C_noret;
C_noret_decl(f_7509)
static void C_fcall f_7509(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_7526)
static void C_ccall f_7526(C_word c,C_word *av) C_noret;
C_noret_decl(f_7540)
static void C_ccall f_7540(C_word c,C_word *av) C_noret;
C_noret_decl(f_7574)
static void C_ccall f_7574(C_word c,C_word *av) C_noret;
C_noret_decl(f_7586)
static void C_ccall f_7586(C_word c,C_word *av) C_noret;
C_noret_decl(f_7602)
static void C_ccall f_7602(C_word c,C_word *av) C_noret;
C_noret_decl(f_7632)
static void C_ccall f_7632(C_word c,C_word *av) C_noret;
C_noret_decl(f_7636)
static void C_fcall f_7636(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7676)
static void C_ccall f_7676(C_word c,C_word *av) C_noret;
C_noret_decl(f_7678)
static void C_ccall f_7678(C_word c,C_word *av) C_noret;
C_noret_decl(f_7694)
static void C_ccall f_7694(C_word c,C_word *av) C_noret;
C_noret_decl(f_7700)
static void C_ccall f_7700(C_word c,C_word *av) C_noret;
C_noret_decl(f_7715)
static void C_ccall f_7715(C_word c,C_word *av) C_noret;
C_noret_decl(f_7732)
static void C_ccall f_7732(C_word c,C_word *av) C_noret;
C_noret_decl(f_7734)
static void C_ccall f_7734(C_word c,C_word *av) C_noret;
C_noret_decl(f_7740)
static void C_fcall f_7740(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7764)
static void C_fcall f_7764(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7780)
static void C_ccall f_7780(C_word c,C_word *av) C_noret;
C_noret_decl(f_7790)
static void C_ccall f_7790(C_word c,C_word *av) C_noret;
C_noret_decl(f_7795)
static void C_ccall f_7795(C_word c,C_word *av) C_noret;
C_noret_decl(f_7809)
static void C_ccall f_7809(C_word c,C_word *av) C_noret;
C_noret_decl(f_7812)
static void C_ccall f_7812(C_word c,C_word *av) C_noret;
C_noret_decl(f_7813)
static void C_ccall f_7813(C_word c,C_word *av) C_noret;
C_noret_decl(f_7817)
static void C_ccall f_7817(C_word c,C_word *av) C_noret;
C_noret_decl(f_7822)
static void C_ccall f_7822(C_word c,C_word *av) C_noret;
C_noret_decl(f_7828)
static void C_ccall f_7828(C_word c,C_word *av) C_noret;
C_noret_decl(f_7834)
static void C_ccall f_7834(C_word c,C_word *av) C_noret;
C_noret_decl(f_7842)
static void C_ccall f_7842(C_word c,C_word *av) C_noret;
C_noret_decl(f_7845)
static void C_ccall f_7845(C_word c,C_word *av) C_noret;
C_noret_decl(f_7853)
static void C_ccall f_7853(C_word c,C_word *av) C_noret;
C_noret_decl(f_7855)
static void C_fcall f_7855(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7859)
static void C_ccall f_7859(C_word c,C_word *av) C_noret;
C_noret_decl(f_7881)
static void C_ccall f_7881(C_word c,C_word *av) C_noret;
C_noret_decl(f_7887)
static void C_fcall f_7887(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7891)
static void C_ccall f_7891(C_word c,C_word *av) C_noret;
C_noret_decl(f_7904)
static void C_ccall f_7904(C_word c,C_word *av) C_noret;
C_noret_decl(f_7912)
static void C_fcall f_7912(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7918)
static void C_ccall f_7918(C_word c,C_word *av) C_noret;
C_noret_decl(f_7929)
static void C_ccall f_7929(C_word c,C_word *av) C_noret;
C_noret_decl(f_7931)
static void C_ccall f_7931(C_word c,C_word *av) C_noret;
C_noret_decl(f_7934)
static void C_ccall f_7934(C_word c,C_word *av) C_noret;
C_noret_decl(f_7940)
static C_word C_fcall f_7940(C_word t0,C_word t1);
C_noret_decl(f_7979)
static void C_ccall f_7979(C_word c,C_word *av) C_noret;
C_noret_decl(f_7984)
static void C_ccall f_7984(C_word c,C_word *av) C_noret;
C_noret_decl(f_7988)
static void C_ccall f_7988(C_word c,C_word *av) C_noret;
C_noret_decl(f_7992)
static void C_ccall f_7992(C_word c,C_word *av) C_noret;
C_noret_decl(f_8043)
static void C_ccall f_8043(C_word c,C_word *av) C_noret;
C_noret_decl(f_8080)
static void C_ccall f_8080(C_word c,C_word *av) C_noret;
C_noret_decl(f_8082)
static void C_fcall f_8082(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8132)
static void C_ccall f_8132(C_word c,C_word *av) C_noret;
C_noret_decl(f_8136)
static void C_ccall f_8136(C_word c,C_word *av) C_noret;
C_noret_decl(f_8150)
static void C_ccall f_8150(C_word c,C_word *av) C_noret;
C_noret_decl(f_8154)
static void C_ccall f_8154(C_word c,C_word *av) C_noret;
C_noret_decl(f_8162)
static C_word C_fcall f_8162(C_word t0,C_word t1);
C_noret_decl(f_8168)
static void C_ccall f_8168(C_word c,C_word *av) C_noret;
C_noret_decl(f_8172)
static void C_ccall f_8172(C_word c,C_word *av) C_noret;
C_noret_decl(f_8214)
static void C_ccall f_8214(C_word c,C_word *av) C_noret;
C_noret_decl(f_8218)
static void C_ccall f_8218(C_word c,C_word *av) C_noret;
C_noret_decl(f_8266)
static void C_ccall f_8266(C_word c,C_word *av) C_noret;
C_noret_decl(f_8270)
static void C_ccall f_8270(C_word c,C_word *av) C_noret;
C_noret_decl(f_8275)
static void C_ccall f_8275(C_word c,C_word *av) C_noret;
C_noret_decl(f_8285)
static void C_ccall f_8285(C_word c,C_word *av) C_noret;
C_noret_decl(f_8292)
static void C_ccall f_8292(C_word c,C_word *av) C_noret;
C_noret_decl(f_8299)
static void C_fcall f_8299(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8326)
static void C_ccall f_8326(C_word c,C_word *av) C_noret;
C_noret_decl(f_8332)
static void C_ccall f_8332(C_word c,C_word *av) C_noret;
C_noret_decl(f_8342)
static void C_ccall f_8342(C_word c,C_word *av) C_noret;
C_noret_decl(f_8345)
static void C_ccall f_8345(C_word c,C_word *av) C_noret;
C_noret_decl(f_8348)
static void C_ccall f_8348(C_word c,C_word *av) C_noret;
C_noret_decl(f_8361)
static void C_ccall f_8361(C_word c,C_word *av) C_noret;
C_noret_decl(f_8363)
static void C_fcall f_8363(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8398)
static void C_ccall f_8398(C_word c,C_word *av) C_noret;
C_noret_decl(f_8404)
static void C_ccall f_8404(C_word c,C_word *av) C_noret;
C_noret_decl(f_8410)
static void C_ccall f_8410(C_word c,C_word *av) C_noret;
C_noret_decl(f_8419)
static void C_ccall f_8419(C_word c,C_word *av) C_noret;
C_noret_decl(f_8428)
static void C_ccall f_8428(C_word c,C_word *av) C_noret;
C_noret_decl(f_8437)
static void C_ccall f_8437(C_word c,C_word *av) C_noret;
C_noret_decl(f_8446)
static void C_ccall f_8446(C_word c,C_word *av) C_noret;
C_noret_decl(f_8455)
static void C_ccall f_8455(C_word c,C_word *av) C_noret;
C_noret_decl(f_8465)
static void C_ccall f_8465(C_word c,C_word *av) C_noret;
C_noret_decl(f_8467)
static void C_ccall f_8467(C_word c,C_word *av) C_noret;
C_noret_decl(f_8473)
static void C_ccall f_8473(C_word c,C_word *av) C_noret;
C_noret_decl(f_8488)
static void C_ccall f_8488(C_word c,C_word *av) C_noret;
C_noret_decl(f_8503)
static void C_ccall f_8503(C_word c,C_word *av) C_noret;
C_noret_decl(f_8506)
static void C_fcall f_8506(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8573)
static void C_ccall f_8573(C_word c,C_word *av) C_noret;
C_noret_decl(f_8575)
static void C_fcall f_8575(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8600)
static void C_ccall f_8600(C_word c,C_word *av) C_noret;
C_noret_decl(f_8623)
static void C_ccall f_8623(C_word c,C_word *av) C_noret;
C_noret_decl(f_8626)
static void C_fcall f_8626(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8629)
static void C_ccall f_8629(C_word c,C_word *av) C_noret;
C_noret_decl(f_8636)
static void C_ccall f_8636(C_word c,C_word *av) C_noret;
C_noret_decl(f_8683)
static void C_ccall f_8683(C_word c,C_word *av) C_noret;
C_noret_decl(f_8687)
static void C_ccall f_8687(C_word c,C_word *av) C_noret;
C_noret_decl(f_8692)
static void C_fcall f_8692(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8709)
static void C_ccall f_8709(C_word c,C_word *av) C_noret;
C_noret_decl(f_8717)
static void C_ccall f_8717(C_word c,C_word *av) C_noret;
C_noret_decl(f_8719)
static void C_fcall f_8719(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8744)
static void C_ccall f_8744(C_word c,C_word *av) C_noret;
C_noret_decl(f_8780)
static void C_ccall f_8780(C_word c,C_word *av) C_noret;
C_noret_decl(f_8814)
static void C_ccall f_8814(C_word c,C_word *av) C_noret;
C_noret_decl(f_8845)
static void C_fcall f_8845(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_8868)
static void C_ccall f_8868(C_word c,C_word *av) C_noret;
C_noret_decl(f_8889)
static void C_ccall f_8889(C_word c,C_word *av) C_noret;
C_noret_decl(f_8911)
static void C_ccall f_8911(C_word c,C_word *av) C_noret;
C_noret_decl(f_8919)
static void C_ccall f_8919(C_word c,C_word *av) C_noret;
C_noret_decl(f_8923)
static void C_ccall f_8923(C_word c,C_word *av) C_noret;
C_noret_decl(f_8931)
static void C_ccall f_8931(C_word c,C_word *av) C_noret;
C_noret_decl(f_8952)
static void C_ccall f_8952(C_word c,C_word *av) C_noret;
C_noret_decl(f_8956)
static void C_ccall f_8956(C_word c,C_word *av) C_noret;
C_noret_decl(f_8968)
static void C_ccall f_8968(C_word c,C_word *av) C_noret;
C_noret_decl(f_8995)
static void C_fcall f_8995(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9007)
static void C_ccall f_9007(C_word c,C_word *av) C_noret;
C_noret_decl(f_9009)
static void C_fcall f_9009(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9034)
static void C_ccall f_9034(C_word c,C_word *av) C_noret;
C_noret_decl(f_9068)
static void C_fcall f_9068(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9094)
static void C_ccall f_9094(C_word c,C_word *av) C_noret;
C_noret_decl(f_9096)
static void C_fcall f_9096(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9121)
static void C_ccall f_9121(C_word c,C_word *av) C_noret;
C_noret_decl(f_9205)
static void C_ccall f_9205(C_word c,C_word *av) C_noret;
C_noret_decl(f_9207)
static void C_fcall f_9207(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9232)
static void C_ccall f_9232(C_word c,C_word *av) C_noret;
C_noret_decl(f_9272)
static void C_ccall f_9272(C_word c,C_word *av) C_noret;
C_noret_decl(f_9313)
static void C_fcall f_9313(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9342)
static void C_ccall f_9342(C_word c,C_word *av) C_noret;
C_noret_decl(f_9344)
static void C_fcall f_9344(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9369)
static void C_ccall f_9369(C_word c,C_word *av) C_noret;
C_noret_decl(f_9405)
static void C_ccall f_9405(C_word c,C_word *av) C_noret;
C_noret_decl(f_9407)
static void C_fcall f_9407(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9432)
static void C_ccall f_9432(C_word c,C_word *av) C_noret;
C_noret_decl(f_9444)
static void C_ccall f_9444(C_word c,C_word *av) C_noret;
C_noret_decl(f_9450)
static void C_ccall f_9450(C_word c,C_word *av) C_noret;
C_noret_decl(f_9473)
static void C_ccall f_9473(C_word c,C_word *av) C_noret;
C_noret_decl(f_9475)
static void C_fcall f_9475(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9500)
static void C_ccall f_9500(C_word c,C_word *av) C_noret;
C_noret_decl(f_9511)
static void C_fcall f_9511(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9515)
static void C_ccall f_9515(C_word c,C_word *av) C_noret;
C_noret_decl(f_9518)
static void C_ccall f_9518(C_word c,C_word *av) C_noret;
C_noret_decl(f_9525)
static void C_ccall f_9525(C_word c,C_word *av) C_noret;
C_noret_decl(f_9539)
static void C_ccall f_9539(C_word c,C_word *av) C_noret;
C_noret_decl(f_9645)
static void C_ccall f_9645(C_word c,C_word *av) C_noret;
C_noret_decl(f_9647)
static void C_fcall f_9647(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9672)
static void C_ccall f_9672(C_word c,C_word *av) C_noret;
C_noret_decl(f_9690)
static void C_ccall f_9690(C_word c,C_word *av) C_noret;
C_noret_decl(f_9693)
static void C_ccall f_9693(C_word c,C_word *av) C_noret;
C_noret_decl(f_9699)
static void C_ccall f_9699(C_word c,C_word *av) C_noret;
C_noret_decl(f_9705)
static void C_fcall f_9705(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9739)
static void C_fcall f_9739(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9752)
static void C_ccall f_9752(C_word c,C_word *av) C_noret;
C_noret_decl(f_9754)
static void C_fcall f_9754(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9779)
static void C_ccall f_9779(C_word c,C_word *av) C_noret;
C_noret_decl(f_9809)
static void C_ccall f_9809(C_word c,C_word *av) C_noret;
C_noret_decl(f_9811)
static void C_fcall f_9811(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9836)
static void C_ccall f_9836(C_word c,C_word *av) C_noret;
C_noret_decl(f_9909)
static void C_ccall f_9909(C_word c,C_word *av) C_noret;
C_noret_decl(f_9912)
static void C_ccall f_9912(C_word c,C_word *av) C_noret;
C_noret_decl(f_9921)
static void C_ccall f_9921(C_word c,C_word *av) C_noret;
C_noret_decl(f_9925)
static void C_ccall f_9925(C_word c,C_word *av) C_noret;
C_noret_decl(f_9929)
static void C_ccall f_9929(C_word c,C_word *av) C_noret;
C_noret_decl(f_9931)
static void C_fcall f_9931(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9979)
static void C_fcall f_9979(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(C_support_toplevel)
C_externexport void C_ccall C_support_toplevel(C_word c,C_word *av) C_noret;

C_noret_decl(trf_10104)
static void C_ccall trf_10104(C_word c,C_word *av) C_noret;
static void C_ccall trf_10104(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10104(t0,t1,t2,t3);}

C_noret_decl(trf_10180)
static void C_ccall trf_10180(C_word c,C_word *av) C_noret;
static void C_ccall trf_10180(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10180(t0,t1,t2);}

C_noret_decl(trf_10237)
static void C_ccall trf_10237(C_word c,C_word *av) C_noret;
static void C_ccall trf_10237(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10237(t0,t1,t2);}

C_noret_decl(trf_10293)
static void C_ccall trf_10293(C_word c,C_word *av) C_noret;
static void C_ccall trf_10293(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10293(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10343)
static void C_ccall trf_10343(C_word c,C_word *av) C_noret;
static void C_ccall trf_10343(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10343(t0,t1);}

C_noret_decl(trf_10363)
static void C_ccall trf_10363(C_word c,C_word *av) C_noret;
static void C_ccall trf_10363(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10363(t0,t1,t2);}

C_noret_decl(trf_10420)
static void C_ccall trf_10420(C_word c,C_word *av) C_noret;
static void C_ccall trf_10420(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10420(t0,t1,t2);}

C_noret_decl(trf_10471)
static void C_ccall trf_10471(C_word c,C_word *av) C_noret;
static void C_ccall trf_10471(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10471(t0,t1,t2);}

C_noret_decl(trf_10575)
static void C_ccall trf_10575(C_word c,C_word *av) C_noret;
static void C_ccall trf_10575(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10575(t0,t1,t2);}

C_noret_decl(trf_10675)
static void C_ccall trf_10675(C_word c,C_word *av) C_noret;
static void C_ccall trf_10675(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10675(t0,t1,t2,t3);}

C_noret_decl(trf_10690)
static void C_ccall trf_10690(C_word c,C_word *av) C_noret;
static void C_ccall trf_10690(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10690(t0,t1,t2,t3);}

C_noret_decl(trf_10820)
static void C_ccall trf_10820(C_word c,C_word *av) C_noret;
static void C_ccall trf_10820(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10820(t0,t1,t2);}

C_noret_decl(trf_10844)
static void C_ccall trf_10844(C_word c,C_word *av) C_noret;
static void C_ccall trf_10844(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10844(t0,t1,t2);}

C_noret_decl(trf_10871)
static void C_ccall trf_10871(C_word c,C_word *av) C_noret;
static void C_ccall trf_10871(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10871(t0,t1,t2);}

C_noret_decl(trf_10917)
static void C_ccall trf_10917(C_word c,C_word *av) C_noret;
static void C_ccall trf_10917(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10917(t0,t1,t2);}

C_noret_decl(trf_10972)
static void C_ccall trf_10972(C_word c,C_word *av) C_noret;
static void C_ccall trf_10972(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10972(t0,t1,t2,t3);}

C_noret_decl(trf_11143)
static void C_ccall trf_11143(C_word c,C_word *av) C_noret;
static void C_ccall trf_11143(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11143(t0,t1,t2);}

C_noret_decl(trf_11176)
static void C_ccall trf_11176(C_word c,C_word *av) C_noret;
static void C_ccall trf_11176(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11176(t0,t1,t2);}

C_noret_decl(trf_11188)
static void C_ccall trf_11188(C_word c,C_word *av) C_noret;
static void C_ccall trf_11188(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11188(t0,t1,t2);}

C_noret_decl(trf_11255)
static void C_ccall trf_11255(C_word c,C_word *av) C_noret;
static void C_ccall trf_11255(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11255(t0,t1,t2,t3);}

C_noret_decl(trf_11303)
static void C_ccall trf_11303(C_word c,C_word *av) C_noret;
static void C_ccall trf_11303(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11303(t0,t1,t2);}

C_noret_decl(trf_11352)
static void C_ccall trf_11352(C_word c,C_word *av) C_noret;
static void C_ccall trf_11352(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11352(t0,t1,t2);}

C_noret_decl(trf_11364)
static void C_ccall trf_11364(C_word c,C_word *av) C_noret;
static void C_ccall trf_11364(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11364(t0,t1,t2);}

C_noret_decl(trf_11411)
static void C_ccall trf_11411(C_word c,C_word *av) C_noret;
static void C_ccall trf_11411(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11411(t0,t1,t2,t3);}

C_noret_decl(trf_11462)
static void C_ccall trf_11462(C_word c,C_word *av) C_noret;
static void C_ccall trf_11462(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11462(t0,t1,t2);}

C_noret_decl(trf_11622)
static void C_ccall trf_11622(C_word c,C_word *av) C_noret;
static void C_ccall trf_11622(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11622(t0,t1,t2,t3);}

C_noret_decl(trf_11664)
static void C_ccall trf_11664(C_word c,C_word *av) C_noret;
static void C_ccall trf_11664(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11664(t0,t1,t2);}

C_noret_decl(trf_11743)
static void C_ccall trf_11743(C_word c,C_word *av) C_noret;
static void C_ccall trf_11743(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11743(t0,t1,t2,t3);}

C_noret_decl(trf_11866)
static void C_ccall trf_11866(C_word c,C_word *av) C_noret;
static void C_ccall trf_11866(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11866(t0,t1,t2);}

C_noret_decl(trf_11973)
static void C_ccall trf_11973(C_word c,C_word *av) C_noret;
static void C_ccall trf_11973(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11973(t0,t1,t2);}

C_noret_decl(trf_12019)
static void C_ccall trf_12019(C_word c,C_word *av) C_noret;
static void C_ccall trf_12019(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12019(t0,t1,t2);}

C_noret_decl(trf_12059)
static void C_ccall trf_12059(C_word c,C_word *av) C_noret;
static void C_ccall trf_12059(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12059(t0,t1,t2);}

C_noret_decl(trf_12090)
static void C_ccall trf_12090(C_word c,C_word *av) C_noret;
static void C_ccall trf_12090(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12090(t0,t1,t2);}

C_noret_decl(trf_12127)
static void C_ccall trf_12127(C_word c,C_word *av) C_noret;
static void C_ccall trf_12127(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12127(t0,t1,t2);}

C_noret_decl(trf_12206)
static void C_ccall trf_12206(C_word c,C_word *av) C_noret;
static void C_ccall trf_12206(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12206(t0,t1,t2);}

C_noret_decl(trf_12258)
static void C_ccall trf_12258(C_word c,C_word *av) C_noret;
static void C_ccall trf_12258(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12258(t0,t1,t2);}

C_noret_decl(trf_12313)
static void C_ccall trf_12313(C_word c,C_word *av) C_noret;
static void C_ccall trf_12313(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12313(t0,t1);}

C_noret_decl(trf_12350)
static void C_ccall trf_12350(C_word c,C_word *av) C_noret;
static void C_ccall trf_12350(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12350(t0,t1);}

C_noret_decl(trf_12454)
static void C_ccall trf_12454(C_word c,C_word *av) C_noret;
static void C_ccall trf_12454(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12454(t0,t1);}

C_noret_decl(trf_12501)
static void C_ccall trf_12501(C_word c,C_word *av) C_noret;
static void C_ccall trf_12501(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12501(t0,t1,t2,t3);}

C_noret_decl(trf_12535)
static void C_ccall trf_12535(C_word c,C_word *av) C_noret;
static void C_ccall trf_12535(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12535(t0,t1,t2,t3);}

C_noret_decl(trf_12582)
static void C_ccall trf_12582(C_word c,C_word *av) C_noret;
static void C_ccall trf_12582(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12582(t0,t1,t2,t3);}

C_noret_decl(trf_12622)
static void C_ccall trf_12622(C_word c,C_word *av) C_noret;
static void C_ccall trf_12622(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12622(t0,t1,t2,t3);}

C_noret_decl(trf_12763)
static void C_ccall trf_12763(C_word c,C_word *av) C_noret;
static void C_ccall trf_12763(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12763(t0,t1);}

C_noret_decl(trf_12777)
static void C_ccall trf_12777(C_word c,C_word *av) C_noret;
static void C_ccall trf_12777(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12777(t0,t1,t2);}

C_noret_decl(trf_12979)
static void C_ccall trf_12979(C_word c,C_word *av) C_noret;
static void C_ccall trf_12979(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12979(t0,t1);}

C_noret_decl(trf_13020)
static void C_ccall trf_13020(C_word c,C_word *av) C_noret;
static void C_ccall trf_13020(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13020(t0,t1);}

C_noret_decl(trf_13292)
static void C_ccall trf_13292(C_word c,C_word *av) C_noret;
static void C_ccall trf_13292(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13292(t0,t1,t2);}

C_noret_decl(trf_13317)
static void C_ccall trf_13317(C_word c,C_word *av) C_noret;
static void C_ccall trf_13317(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13317(t0,t1);}

C_noret_decl(trf_13332)
static void C_ccall trf_13332(C_word c,C_word *av) C_noret;
static void C_ccall trf_13332(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13332(t0,t1);}

C_noret_decl(trf_13415)
static void C_ccall trf_13415(C_word c,C_word *av) C_noret;
static void C_ccall trf_13415(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13415(t0,t1);}

C_noret_decl(trf_13455)
static void C_ccall trf_13455(C_word c,C_word *av) C_noret;
static void C_ccall trf_13455(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13455(t0,t1);}

C_noret_decl(trf_13473)
static void C_ccall trf_13473(C_word c,C_word *av) C_noret;
static void C_ccall trf_13473(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13473(t0,t1);}

C_noret_decl(trf_13497)
static void C_ccall trf_13497(C_word c,C_word *av) C_noret;
static void C_ccall trf_13497(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13497(t0,t1);}

C_noret_decl(trf_13523)
static void C_ccall trf_13523(C_word c,C_word *av) C_noret;
static void C_ccall trf_13523(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13523(t0,t1);}

C_noret_decl(trf_13566)
static void C_ccall trf_13566(C_word c,C_word *av) C_noret;
static void C_ccall trf_13566(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13566(t0,t1);}

C_noret_decl(trf_13610)
static void C_ccall trf_13610(C_word c,C_word *av) C_noret;
static void C_ccall trf_13610(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13610(t0,t1);}

C_noret_decl(trf_13654)
static void C_ccall trf_13654(C_word c,C_word *av) C_noret;
static void C_ccall trf_13654(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13654(t0,t1);}

C_noret_decl(trf_13672)
static void C_ccall trf_13672(C_word c,C_word *av) C_noret;
static void C_ccall trf_13672(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13672(t0,t1);}

C_noret_decl(trf_13699)
static void C_ccall trf_13699(C_word c,C_word *av) C_noret;
static void C_ccall trf_13699(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13699(t0,t1);}

C_noret_decl(trf_13746)
static void C_ccall trf_13746(C_word c,C_word *av) C_noret;
static void C_ccall trf_13746(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13746(t0,t1,t2);}

C_noret_decl(trf_13773)
static void C_ccall trf_13773(C_word c,C_word *av) C_noret;
static void C_ccall trf_13773(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13773(t0,t1);}

C_noret_decl(trf_14356)
static void C_ccall trf_14356(C_word c,C_word *av) C_noret;
static void C_ccall trf_14356(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14356(t0,t1);}

C_noret_decl(trf_14377)
static void C_ccall trf_14377(C_word c,C_word *av) C_noret;
static void C_ccall trf_14377(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14377(t0,t1);}

C_noret_decl(trf_14442)
static void C_ccall trf_14442(C_word c,C_word *av) C_noret;
static void C_ccall trf_14442(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14442(t0,t1,t2);}

C_noret_decl(trf_14470)
static void C_ccall trf_14470(C_word c,C_word *av) C_noret;
static void C_ccall trf_14470(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14470(t0,t1);}

C_noret_decl(trf_14489)
static void C_ccall trf_14489(C_word c,C_word *av) C_noret;
static void C_ccall trf_14489(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14489(t0,t1);}

C_noret_decl(trf_14498)
static void C_ccall trf_14498(C_word c,C_word *av) C_noret;
static void C_ccall trf_14498(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14498(t0,t1);}

C_noret_decl(trf_14510)
static void C_ccall trf_14510(C_word c,C_word *av) C_noret;
static void C_ccall trf_14510(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14510(t0,t1);}

C_noret_decl(trf_14522)
static void C_ccall trf_14522(C_word c,C_word *av) C_noret;
static void C_ccall trf_14522(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14522(t0,t1);}

C_noret_decl(trf_14534)
static void C_ccall trf_14534(C_word c,C_word *av) C_noret;
static void C_ccall trf_14534(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14534(t0,t1);}

C_noret_decl(trf_14544)
static void C_ccall trf_14544(C_word c,C_word *av) C_noret;
static void C_ccall trf_14544(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14544(t0,t1,t2);}

C_noret_decl(trf_14571)
static void C_ccall trf_14571(C_word c,C_word *av) C_noret;
static void C_ccall trf_14571(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14571(t0,t1);}

C_noret_decl(trf_14964)
static void C_ccall trf_14964(C_word c,C_word *av) C_noret;
static void C_ccall trf_14964(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14964(t0,t1);}

C_noret_decl(trf_14976)
static void C_ccall trf_14976(C_word c,C_word *av) C_noret;
static void C_ccall trf_14976(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14976(t0,t1);}

C_noret_decl(trf_14986)
static void C_ccall trf_14986(C_word c,C_word *av) C_noret;
static void C_ccall trf_14986(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14986(t0,t1,t2);}

C_noret_decl(trf_15013)
static void C_ccall trf_15013(C_word c,C_word *av) C_noret;
static void C_ccall trf_15013(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15013(t0,t1);}

C_noret_decl(trf_15512)
static void C_ccall trf_15512(C_word c,C_word *av) C_noret;
static void C_ccall trf_15512(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15512(t0,t1);}

C_noret_decl(trf_15697)
static void C_ccall trf_15697(C_word c,C_word *av) C_noret;
static void C_ccall trf_15697(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15697(t0,t1);}

C_noret_decl(trf_15772)
static void C_ccall trf_15772(C_word c,C_word *av) C_noret;
static void C_ccall trf_15772(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15772(t0,t1);}

C_noret_decl(trf_15859)
static void C_ccall trf_15859(C_word c,C_word *av) C_noret;
static void C_ccall trf_15859(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15859(t0,t1);}

C_noret_decl(trf_15892)
static void C_ccall trf_15892(C_word c,C_word *av) C_noret;
static void C_ccall trf_15892(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15892(t0,t1);}

C_noret_decl(trf_15910)
static void C_ccall trf_15910(C_word c,C_word *av) C_noret;
static void C_ccall trf_15910(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15910(t0,t1);}

C_noret_decl(trf_15932)
static void C_ccall trf_15932(C_word c,C_word *av) C_noret;
static void C_ccall trf_15932(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15932(t0,t1);}

C_noret_decl(trf_16320)
static void C_ccall trf_16320(C_word c,C_word *av) C_noret;
static void C_ccall trf_16320(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16320(t0,t1,t2);}

C_noret_decl(trf_16352)
static void C_ccall trf_16352(C_word c,C_word *av) C_noret;
static void C_ccall trf_16352(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16352(t0,t1);}

C_noret_decl(trf_16360)
static void C_ccall trf_16360(C_word c,C_word *av) C_noret;
static void C_ccall trf_16360(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16360(t0,t1,t2);}

C_noret_decl(trf_16416)
static void C_ccall trf_16416(C_word c,C_word *av) C_noret;
static void C_ccall trf_16416(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16416(t0,t1);}

C_noret_decl(trf_16424)
static void C_ccall trf_16424(C_word c,C_word *av) C_noret;
static void C_ccall trf_16424(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16424(t0,t1,t2);}

C_noret_decl(trf_16472)
static void C_ccall trf_16472(C_word c,C_word *av) C_noret;
static void C_ccall trf_16472(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16472(t0,t1,t2,t3);}

C_noret_decl(trf_16506)
static void C_ccall trf_16506(C_word c,C_word *av) C_noret;
static void C_ccall trf_16506(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16506(t0,t1);}

C_noret_decl(trf_16656)
static void C_ccall trf_16656(C_word c,C_word *av) C_noret;
static void C_ccall trf_16656(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16656(t0,t1,t2,t3);}

C_noret_decl(trf_16658)
static void C_ccall trf_16658(C_word c,C_word *av) C_noret;
static void C_ccall trf_16658(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16658(t0,t1,t2);}

C_noret_decl(trf_16670)
static void C_ccall trf_16670(C_word c,C_word *av) C_noret;
static void C_ccall trf_16670(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16670(t0,t1,t2);}

C_noret_decl(trf_16814)
static void C_ccall trf_16814(C_word c,C_word *av) C_noret;
static void C_ccall trf_16814(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16814(t0,t1);}

C_noret_decl(trf_16859)
static void C_ccall trf_16859(C_word c,C_word *av) C_noret;
static void C_ccall trf_16859(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_16859(t0,t1,t2,t3,t4);}

C_noret_decl(trf_17228)
static void C_ccall trf_17228(C_word c,C_word *av) C_noret;
static void C_ccall trf_17228(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17228(t0,t1,t2);}

C_noret_decl(trf_17262)
static void C_ccall trf_17262(C_word c,C_word *av) C_noret;
static void C_ccall trf_17262(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17262(t0,t1,t2);}

C_noret_decl(trf_17452)
static void C_ccall trf_17452(C_word c,C_word *av) C_noret;
static void C_ccall trf_17452(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17452(t0,t1);}

C_noret_decl(trf_17459)
static void C_ccall trf_17459(C_word c,C_word *av) C_noret;
static void C_ccall trf_17459(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17459(t0,t1,t2);}

C_noret_decl(trf_17541)
static void C_ccall trf_17541(C_word c,C_word *av) C_noret;
static void C_ccall trf_17541(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17541(t0,t1,t2,t3);}

C_noret_decl(trf_17593)
static void C_ccall trf_17593(C_word c,C_word *av) C_noret;
static void C_ccall trf_17593(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17593(t0,t1,t2);}

C_noret_decl(trf_17632)
static void C_ccall trf_17632(C_word c,C_word *av) C_noret;
static void C_ccall trf_17632(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17632(t0,t1,t2);}

C_noret_decl(trf_17665)
static void C_ccall trf_17665(C_word c,C_word *av) C_noret;
static void C_ccall trf_17665(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17665(t0,t1,t2);}

C_noret_decl(trf_17771)
static void C_ccall trf_17771(C_word c,C_word *av) C_noret;
static void C_ccall trf_17771(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17771(t0,t1);}

C_noret_decl(trf_18066)
static void C_ccall trf_18066(C_word c,C_word *av) C_noret;
static void C_ccall trf_18066(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18066(t0,t1,t2);}

C_noret_decl(trf_5482)
static void C_ccall trf_5482(C_word c,C_word *av) C_noret;
static void C_ccall trf_5482(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5482(t0,t1,t2);}

C_noret_decl(trf_5540)
static void C_ccall trf_5540(C_word c,C_word *av) C_noret;
static void C_ccall trf_5540(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_5540(t0,t1,t2,t3,t4);}

C_noret_decl(trf_5743)
static void C_ccall trf_5743(C_word c,C_word *av) C_noret;
static void C_ccall trf_5743(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5743(t0,t1,t2);}

C_noret_decl(trf_5749)
static void C_ccall trf_5749(C_word c,C_word *av) C_noret;
static void C_ccall trf_5749(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5749(t0,t1,t2);}

C_noret_decl(trf_5777)
static void C_ccall trf_5777(C_word c,C_word *av) C_noret;
static void C_ccall trf_5777(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5777(t0,t1,t2);}

C_noret_decl(trf_5783)
static void C_ccall trf_5783(C_word c,C_word *av) C_noret;
static void C_ccall trf_5783(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5783(t0,t1,t2);}

C_noret_decl(trf_5807)
static void C_ccall trf_5807(C_word c,C_word *av) C_noret;
static void C_ccall trf_5807(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5807(t0,t1,t2);}

C_noret_decl(trf_5813)
static void C_ccall trf_5813(C_word c,C_word *av) C_noret;
static void C_ccall trf_5813(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5813(t0,t1,t2,t3);}

C_noret_decl(trf_6036)
static void C_ccall trf_6036(C_word c,C_word *av) C_noret;
static void C_ccall trf_6036(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6036(t0,t1,t2);}

C_noret_decl(trf_6044)
static void C_ccall trf_6044(C_word c,C_word *av) C_noret;
static void C_ccall trf_6044(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6044(t0,t1,t2,t3);}

C_noret_decl(trf_6109)
static void C_ccall trf_6109(C_word c,C_word *av) C_noret;
static void C_ccall trf_6109(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6109(t0,t1,t2);}

C_noret_decl(trf_6143)
static void C_ccall trf_6143(C_word c,C_word *av) C_noret;
static void C_ccall trf_6143(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6143(t0,t1);}

C_noret_decl(trf_6202)
static void C_ccall trf_6202(C_word c,C_word *av) C_noret;
static void C_ccall trf_6202(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6202(t0,t1,t2);}

C_noret_decl(trf_6208)
static void C_ccall trf_6208(C_word c,C_word *av) C_noret;
static void C_ccall trf_6208(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6208(t0,t1,t2,t3);}

C_noret_decl(trf_6391)
static void C_ccall trf_6391(C_word c,C_word *av) C_noret;
static void C_ccall trf_6391(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6391(t0,t1,t2);}

C_noret_decl(trf_6504)
static void C_ccall trf_6504(C_word c,C_word *av) C_noret;
static void C_ccall trf_6504(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6504(t0,t1,t2);}

C_noret_decl(trf_6687)
static void C_ccall trf_6687(C_word c,C_word *av) C_noret;
static void C_ccall trf_6687(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6687(t0,t1,t2);}

C_noret_decl(trf_6711)
static void C_ccall trf_6711(C_word c,C_word *av) C_noret;
static void C_ccall trf_6711(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6711(t0,t1);}

C_noret_decl(trf_6753)
static void C_ccall trf_6753(C_word c,C_word *av) C_noret;
static void C_ccall trf_6753(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6753(t0,t1,t2);}

C_noret_decl(trf_6776)
static void C_ccall trf_6776(C_word c,C_word *av) C_noret;
static void C_ccall trf_6776(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6776(t0,t1,t2);}

C_noret_decl(trf_6831)
static void C_ccall trf_6831(C_word c,C_word *av) C_noret;
static void C_ccall trf_6831(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6831(t0,t1,t2);}

C_noret_decl(trf_6833)
static void C_ccall trf_6833(C_word c,C_word *av) C_noret;
static void C_ccall trf_6833(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6833(t0,t1,t2);}

C_noret_decl(trf_6865)
static void C_ccall trf_6865(C_word c,C_word *av) C_noret;
static void C_ccall trf_6865(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6865(t0,t1,t2);}

C_noret_decl(trf_6945)
static void C_ccall trf_6945(C_word c,C_word *av) C_noret;
static void C_ccall trf_6945(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6945(t0,t1);}

C_noret_decl(trf_6949)
static void C_ccall trf_6949(C_word c,C_word *av) C_noret;
static void C_ccall trf_6949(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6949(t0,t1,t2);}

C_noret_decl(trf_6967)
static void C_ccall trf_6967(C_word c,C_word *av) C_noret;
static void C_ccall trf_6967(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6967(t0,t1,t2);}

C_noret_decl(trf_7139)
static void C_ccall trf_7139(C_word c,C_word *av) C_noret;
static void C_ccall trf_7139(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7139(t0,t1,t2,t3);}

C_noret_decl(trf_7188)
static void C_ccall trf_7188(C_word c,C_word *av) C_noret;
static void C_ccall trf_7188(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7188(t0,t1,t2);}

C_noret_decl(trf_7210)
static void C_ccall trf_7210(C_word c,C_word *av) C_noret;
static void C_ccall trf_7210(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7210(t0,t1);}

C_noret_decl(trf_7217)
static void C_ccall trf_7217(C_word c,C_word *av) C_noret;
static void C_ccall trf_7217(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7217(t0,t1);}

C_noret_decl(trf_7405)
static void C_ccall trf_7405(C_word c,C_word *av) C_noret;
static void C_ccall trf_7405(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7405(t0,t1,t2);}

C_noret_decl(trf_7445)
static void C_ccall trf_7445(C_word c,C_word *av) C_noret;
static void C_ccall trf_7445(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7445(t0,t1,t2,t3);}

C_noret_decl(trf_7451)
static void C_ccall trf_7451(C_word c,C_word *av) C_noret;
static void C_ccall trf_7451(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7451(t0,t1,t2,t3);}

C_noret_decl(trf_7509)
static void C_ccall trf_7509(C_word c,C_word *av) C_noret;
static void C_ccall trf_7509(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_7509(t0,t1,t2,t3,t4);}

C_noret_decl(trf_7636)
static void C_ccall trf_7636(C_word c,C_word *av) C_noret;
static void C_ccall trf_7636(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7636(t0,t1);}

C_noret_decl(trf_7740)
static void C_ccall trf_7740(C_word c,C_word *av) C_noret;
static void C_ccall trf_7740(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7740(t0,t1,t2);}

C_noret_decl(trf_7764)
static void C_ccall trf_7764(C_word c,C_word *av) C_noret;
static void C_ccall trf_7764(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7764(t0,t1);}

C_noret_decl(trf_7855)
static void C_ccall trf_7855(C_word c,C_word *av) C_noret;
static void C_ccall trf_7855(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7855(t0,t1);}

C_noret_decl(trf_7887)
static void C_ccall trf_7887(C_word c,C_word *av) C_noret;
static void C_ccall trf_7887(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7887(t0,t1,t2);}

C_noret_decl(trf_7912)
static void C_ccall trf_7912(C_word c,C_word *av) C_noret;
static void C_ccall trf_7912(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7912(t0,t1,t2);}

C_noret_decl(trf_8082)
static void C_ccall trf_8082(C_word c,C_word *av) C_noret;
static void C_ccall trf_8082(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8082(t0,t1,t2);}

C_noret_decl(trf_8299)
static void C_ccall trf_8299(C_word c,C_word *av) C_noret;
static void C_ccall trf_8299(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8299(t0,t1,t2);}

C_noret_decl(trf_8363)
static void C_ccall trf_8363(C_word c,C_word *av) C_noret;
static void C_ccall trf_8363(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8363(t0,t1,t2);}

C_noret_decl(trf_8506)
static void C_ccall trf_8506(C_word c,C_word *av) C_noret;
static void C_ccall trf_8506(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8506(t0,t1,t2);}

C_noret_decl(trf_8575)
static void C_ccall trf_8575(C_word c,C_word *av) C_noret;
static void C_ccall trf_8575(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8575(t0,t1,t2);}

C_noret_decl(trf_8626)
static void C_ccall trf_8626(C_word c,C_word *av) C_noret;
static void C_ccall trf_8626(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8626(t0,t1);}

C_noret_decl(trf_8692)
static void C_ccall trf_8692(C_word c,C_word *av) C_noret;
static void C_ccall trf_8692(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8692(t0,t1,t2);}

C_noret_decl(trf_8719)
static void C_ccall trf_8719(C_word c,C_word *av) C_noret;
static void C_ccall trf_8719(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8719(t0,t1,t2);}

C_noret_decl(trf_8845)
static void C_ccall trf_8845(C_word c,C_word *av) C_noret;
static void C_ccall trf_8845(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_8845(t0,t1,t2,t3,t4);}

C_noret_decl(trf_8995)
static void C_ccall trf_8995(C_word c,C_word *av) C_noret;
static void C_ccall trf_8995(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8995(t0,t1);}

C_noret_decl(trf_9009)
static void C_ccall trf_9009(C_word c,C_word *av) C_noret;
static void C_ccall trf_9009(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9009(t0,t1,t2);}

C_noret_decl(trf_9068)
static void C_ccall trf_9068(C_word c,C_word *av) C_noret;
static void C_ccall trf_9068(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9068(t0,t1);}

C_noret_decl(trf_9096)
static void C_ccall trf_9096(C_word c,C_word *av) C_noret;
static void C_ccall trf_9096(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9096(t0,t1,t2);}

C_noret_decl(trf_9207)
static void C_ccall trf_9207(C_word c,C_word *av) C_noret;
static void C_ccall trf_9207(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9207(t0,t1,t2);}

C_noret_decl(trf_9313)
static void C_ccall trf_9313(C_word c,C_word *av) C_noret;
static void C_ccall trf_9313(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9313(t0,t1);}

C_noret_decl(trf_9344)
static void C_ccall trf_9344(C_word c,C_word *av) C_noret;
static void C_ccall trf_9344(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9344(t0,t1,t2);}

C_noret_decl(trf_9407)
static void C_ccall trf_9407(C_word c,C_word *av) C_noret;
static void C_ccall trf_9407(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9407(t0,t1,t2);}

C_noret_decl(trf_9475)
static void C_ccall trf_9475(C_word c,C_word *av) C_noret;
static void C_ccall trf_9475(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9475(t0,t1,t2);}

C_noret_decl(trf_9511)
static void C_ccall trf_9511(C_word c,C_word *av) C_noret;
static void C_ccall trf_9511(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9511(t0,t1);}

C_noret_decl(trf_9647)
static void C_ccall trf_9647(C_word c,C_word *av) C_noret;
static void C_ccall trf_9647(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9647(t0,t1,t2);}

C_noret_decl(trf_9705)
static void C_ccall trf_9705(C_word c,C_word *av) C_noret;
static void C_ccall trf_9705(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9705(t0,t1,t2);}

C_noret_decl(trf_9739)
static void C_ccall trf_9739(C_word c,C_word *av) C_noret;
static void C_ccall trf_9739(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9739(t0,t1);}

C_noret_decl(trf_9754)
static void C_ccall trf_9754(C_word c,C_word *av) C_noret;
static void C_ccall trf_9754(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9754(t0,t1,t2);}

C_noret_decl(trf_9811)
static void C_ccall trf_9811(C_word c,C_word *av) C_noret;
static void C_ccall trf_9811(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9811(t0,t1,t2);}

C_noret_decl(trf_9931)
static void C_ccall trf_9931(C_word c,C_word *av) C_noret;
static void C_ccall trf_9931(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9931(t0,t1,t2,t3);}

C_noret_decl(trf_9979)
static void C_ccall trf_9979(C_word c,C_word *av) C_noret;
static void C_ccall trf_9979(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9979(t0,t1,t2);}

/* f19850 in chicken.compiler.support#print-version in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f19850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f19850,c,av);}
/* support.scm:1791: chicken.base#print */
t2=*((C_word*)lf[237]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k10002 in map-loop1861 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10004,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9979(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10030 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10032,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list3(&a,3,((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10054 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10056,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[195],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10090,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10094,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_i_cdr(((C_word*)t0)[3]);
t4=C_i_cdr(((C_word*)t0)[4]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10104,a[2]=((C_word*)t0)[5],a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_10104(t8,t2,t3,t4);}

/* k10092 in k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10094,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[197],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10104(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_10104,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10128,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:618: walk */
t5=((C_word*)((C_word*)t0)[2])[1];
f_9705(t5,t4,C_i_car(t3));}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10155,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* support.scm:619: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_9705(t6,t5,C_i_car(t3));}}

/* k10126 in loop in k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10128,c,av);}
a=C_alloc(9);
t2=C_a_i_list(&a,2,lf[168],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10141 in k10153 in loop in k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10143,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10153 in loop in k10088 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10155(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_10155,c,av);}
a=C_alloc(10);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10143,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:620: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_10104(t4,t3,C_u_i_cdr(((C_word*)t0)[5]),C_u_i_cdr(((C_word*)t0)[6]));}

/* map-loop1910 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10180(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10180,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10205,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:622: g1916 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10203 in map-loop1910 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10205,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10180(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10233 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10235,c,av);}
a=C_alloc(6);
/* support.scm:623: cons* */
f_5807(((C_word*)t0)[2],lf[189],C_a_i_list(&a,2,((C_word*)t0)[3],t1));}

/* map-loop1936 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10237(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10237,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10262,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:623: g1942 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10260 in map-loop1936 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10262,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10237(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* loop in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10293(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,0,2)))){
C_save_and_reclaim_args((void *)trf_10293,5,t0,t1,t2,t3,t4);}
a=C_alloc(36);
if(C_truep(C_i_zerop(t2))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10305,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:628: scheme#reverse */
t6=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_s_a_i_minus(&a,2,t2,C_fix(1));
t6=C_i_cdr(t3);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10332,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t5,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* support.scm:629: walk */
t8=((C_word*)((C_word*)t0)[2])[1];
f_9705(t8,t7,C_u_i_car(t3));}}

/* k10303 in loop in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10305,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10309,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:628: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_9705(t3,t2,C_i_car(((C_word*)t0)[4]));}

/* k10307 in k10303 in loop in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10309,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[198],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10330 in loop in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10332,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* support.scm:629: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10293(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2);}

/* k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10343(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_10343,2,t0,t1);}
a=C_alloc(21);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10350,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* support.scm:631: walk */
t3=((C_word*)((C_word*)t0)[6])[1];
f_9705(t3,t2,C_i_car(((C_word*)t0)[2]));}
else{
t2=C_eqp(((C_word*)t0)[4],lf[177]);
if(C_truep(t2)){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10418,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10420,a[2]=t5,a[3]=t10,a[4]=((C_word*)t0)[6],a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_10420(t12,t8,((C_word*)t0)[2]);}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10459,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10469,a[2]=t3,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10471,a[2]=t6,a[3]=t11,a[4]=((C_word*)t0)[6],a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_10471(t13,t9,((C_word*)t0)[2]);}}}

/* k10348 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_10350,c,av);}
a=C_alloc(19);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[125]);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10361,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10363,a[2]=t4,a[3]=t10,a[4]=((C_word*)t0)[6],a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_10363(t12,t8,t6);}

/* k10359 in k10348 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10361,c,av);}
a=C_alloc(9);
/* support.scm:631: cons* */
f_5807(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,3,((C_word*)t0)[4],((C_word*)t0)[5],t1));}

/* map-loop1979 in k10348 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10363(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10363,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10388,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:631: g1985 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10386 in map-loop1979 in k10348 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10388(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10388,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10363(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10416 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10418,c,av);}
a=C_alloc(6);
/* support.scm:633: cons* */
f_5807(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,((C_word*)t0)[4],t1));}

/* map-loop2005 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10420(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10420,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10445,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:633: g2011 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10443 in map-loop2005 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10445(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10445,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10420(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10457 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10459,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10467 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10469(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10469,c,av);}
/* support.scm:634: scheme#append */
t2=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop2031 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10471(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10471,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10496,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:634: g2037 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10494 in map-loop2031 in k10341 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10496,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10471(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.support#fold-boolean in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10569,c,av);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10575,a[2]=t2,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_10575(t7,t1,t3);}

/* fold in chicken.compiler.support#fold-boolean in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10575(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_10575,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_cddr(t2);
if(C_truep(C_i_nullp(t3))){{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
C_apply(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10601,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:642: proc */
t5=((C_word*)t0)[2];{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
av2[3]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k10599 in fold in chicken.compiler.support#fold-boolean in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10601,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10605,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:643: fold */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10575(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k10603 in k10599 in fold in chicken.compiler.support#fold-boolean in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_10605,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[172],lf[206],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_10621,c,av);}
a=C_alloc(7);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10627,a[2]=t3,a[3]=t6,a[4]=t5,a[5]=t7,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* support.scm:647: ##sys#decompose-lambda-list */
t9=*((C_word*)lf[225]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
av2[2]=t2;
av2[3]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}

/* a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_10627,c,av);}
a=C_alloc(13);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10633,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10639,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* support.scm:650: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}

/* a10632 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_10633,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5540,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_5540(t5,t1,((C_word*)t0)[2],C_SCHEME_END_OF_LIST,((C_word*)t0)[3]);}

/* a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10639(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_10639,c,av);}
a=C_alloc(24);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10643,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[5])){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_i_check_list_2(((C_word*)t0)[6],lf[125]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10917,a[2]=t7,a[3]=t11,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t13=((C_word*)t11)[1];
f_10917(t13,t4,((C_word*)t0)[6]);}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[6];
f_10643(2,av2);}}}

/* k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_10643,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10646,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[8])){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[9],lf[125]);
t8=C_i_check_list_2(t1,lf[125]);
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10964,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[11],tmp=(C_word)a,a+=6,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11411,a[2]=t5,a[3]=t11,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t13=((C_word*)t11)[1];
f_11411(t13,t9,((C_word*)t0)[9],t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[11];
f_10646(2,av2);}}}

/* k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_10646,c,av);}
a=C_alloc(22);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t7=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10660,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10871,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_10871(t11,t7,((C_word*)t0)[2]);}

/* k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_10660,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10663,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10869,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[6],a[5]=t2,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* support.scm:656: last */
f_6143(t3,((C_word*)t0)[5]);}

/* k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,4)))){
C_save_and_reclaim((void *)f_10663,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10666,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[2])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10820,a[2]=((C_word*)t0)[6],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10830,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:664: db-get-list */
t5=*((C_word*)lf[134]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[212];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_10666(2,av2);}}}

/* k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10666(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_10666,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10673,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* support.scm:668: take */
f_5482(t2,((C_word*)t0)[5],((C_word*)t0)[10]);}

/* k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_10673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10673,c,av);}
a=C_alloc(11);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10675,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t3,tmp=(C_word)a,a+=9,tmp));
t5=((C_word*)t3)[1];
f_10675(t5,((C_word*)t0)[8],t1,((C_word*)t0)[9]);}

/* loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_10675(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_10675,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t2))){
if(C_truep(((C_word*)t0)[2])){
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10690,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=t5,tmp=(C_word)a,a+=8,tmp));
t7=((C_word*)t5)[1];
f_10690(t7,t1,((C_word*)t0)[4],((C_word*)t0)[7]);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_i_car(t2);
t5=C_a_i_list1(&a,1,t4);
t6=C_i_car(t3);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10811,a[2]=t6,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* support.scm:694: loop */
t9=t7;
t10=C_u_i_cdr(t2);
t11=C_u_i_cdr(t3);
t1=t9;
t2=t10;
t3=t11;
goto loop;}}

/* loop2 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_fcall f_10690(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_10690,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10755,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:677: db-get-list */
t5=*((C_word*)lf[134]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[209];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=C_i_car(t3);
t5=C_a_i_list1(&a,1,t4);
t6=C_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10779,a[2]=t6,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* support.scm:690: loop2 */
t9=t7;
t10=C_u_i_cdr(t2);
t11=C_u_i_cdr(t3);
t1=t9;
t2=t10;
t3=t11;
goto loop;}}

/* k10721 in k10749 in k10753 in loop2 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in ... */
static void C_ccall f_10723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_10723,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10749 in k10753 in loop2 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_10751(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,2)))){
C_save_and_reclaim((void *)f_10751,c,av);}
a=C_alloc(35);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10723,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_nullp(((C_word*)t0)[4]))){
/* support.scm:682: qnode */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=C_i_length(((C_word*)t0)[4]);
t5=C_a_i_fixnum_times(&a,2,C_fix(3),t4);
t6=C_a_i_list2(&a,2,lf[208],t5);
t7=C_a_i_record4(&a,4,lf[141],lf[177],t6,((C_word*)t0)[4]);
t8=C_a_i_list2(&a,2,t7,((C_word*)t0)[2]);
t9=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],t2,t8);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}

/* k10753 in loop2 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_10755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10755,c,av);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10751,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:680: last */
f_6143(t2,((C_word*)t0)[5]);}}

/* k10777 in loop2 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_10779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_10779,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10809 in loop in k10671 in k10664 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_10811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_10811,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* g2138 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10820(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_10820,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10824,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:662: db-put! */
t4=*((C_word*)lf[131]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[211];
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k10822 in g2138 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_10824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10824,c,av);}
/* support.scm:663: db-put! */
t2=*((C_word*)lf[131]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[210];
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k10828 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_10830,c,av);}
a=C_alloc(11);
t2=C_i_check_list_2(t1,lf[44]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10836,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10844,a[2]=t5,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_10844(t7,t3,t1);}

/* k10834 in k10828 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_10836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_10836,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10839,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:665: db-put! */
t3=*((C_word*)lf[131]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[211];
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k10837 in k10834 in k10828 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_10839(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10839,c,av);}
/* support.scm:666: db-put! */
t2=*((C_word*)lf[131]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[212];
av2[5]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* for-each-loop2137 in k10828 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_10844(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_10844,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10854,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:661: g2138 */
t4=((C_word*)t0)[3];
f_10820(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10852 in for-each-loop2137 in k10828 in k10661 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_10854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10854,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10844(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10869,c,av);}
a=C_alloc(9);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11462,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t3,tmp=(C_word)a,a+=7,tmp));
/* support.scm:780: walk */
t5=((C_word*)t3)[1];
f_11462(t5,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* map-loop2110 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10871(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10871,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10896,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:655: chicken.base#gensym */
t4=*((C_word*)lf[97]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[219];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10894 in map-loop2110 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10896,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10871(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* map-loop2082 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10917(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10917,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10942,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:651: g2088 */
t4=*((C_word*)lf[97]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10940 in map-loop2082 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10942,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10917(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_10964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_10964,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10972,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp));
/* support.scm:743: walk */
t5=((C_word*)t3)[1];
f_10972(t5,((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_10972(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_10972,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=C_slot(t2,C_fix(3));
t5=C_slot(t2,C_fix(2));
t6=C_slot(t2,C_fix(1));
t7=C_eqp(t6,lf[85]);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_record4(&a,4,lf[141],t6,t5,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(t6,lf[154]);
if(C_truep(t8)){
t9=C_i_car(t5);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11023,a[2]=t1,a[3]=t9,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11033,a[2]=((C_word*)t0)[2],a[3]=t10,a[4]=t9,tmp=(C_word)a,a+=5,tmp);
/* support.scm:709: db-get */
t12=*((C_word*)lf[127]+1);{
C_word av2[5];
av2[0]=t12;
av2[1]=t11;
av2[2]=((C_word*)t0)[3];
av2[3]=t9;
av2[4]=lf[222];
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}
else{
t9=C_eqp(t6,lf[124]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11070,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=t4,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t11=C_i_car(t5);
/* support.scm:699: chicken.base#alist-ref */
t12=*((C_word*)lf[220]+1);{
C_word av2[6];
av2[0]=t12;
av2[1]=t10;
av2[2]=t11;
av2[3]=t3;
av2[4]=*((C_word*)lf[221]+1);
av2[5]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(6,av2);}}
else{
t10=C_eqp(t6,lf[96]);
if(C_truep(t10)){
t11=C_i_car(t5);
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11086,a[2]=t11,a[3]=t3,a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* support.scm:718: walk */
t15=t12;
t16=C_i_car(t4);
t17=t3;
t1=t15;
t2=t16;
t3=t17;
goto loop;}
else{
t11=C_eqp(t6,lf[118]);
if(C_truep(t11)){
t12=C_i_caddr(t5);
t13=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11138,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* support.scm:726: ##sys#decompose-lambda-list */
t14=*((C_word*)lf[225]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t1;
av2[2]=t12;
av2[3]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}
else{
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11347,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=t4,a[5]=t1,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* support.scm:741: tree-copy */
t13=*((C_word*)lf[226]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}}}}}}

/* k11021 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11023,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11030,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:699: chicken.base#alist-ref */
t3=*((C_word*)lf[220]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=*((C_word*)lf[221]+1);
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k11028 in k11021 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11030,c,av);}
/* support.scm:711: varnode */
t2=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11031 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11033(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11033,c,av);}
if(C_truep(t1)){
/* support.scm:710: cfk */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_11023(2,av2);}}}

/* k11060 in k11068 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11062(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_11062,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[124],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11068 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_11070,c,av);}
a=C_alloc(7);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11062,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:715: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_10972(t4,t3,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k11084 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_11086,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11089,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* support.scm:719: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11087 in k11084 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,5)))){
C_save_and_reclaim((void *)f_11089,c,av);}
a=C_alloc(14);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11095,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* support.scm:721: db-put! */
t5=*((C_word*)lf[131]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
av2[3]=t1;
av2[4]=lf[223];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}

/* k11093 in k11087 in k11084 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11095(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_11095,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11115,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:724: walk */
t4=((C_word*)((C_word*)t0)[5])[1];
f_10972(t4,t3,C_i_cadr(((C_word*)t0)[6]),((C_word*)t0)[7]);}

/* k11113 in k11093 in k11087 in k11084 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_11115,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11138(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_11138,c,av);}
a=C_alloc(26);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11143,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t10=C_i_check_list_2(t2,lf[125]);
t11=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11156,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=t3,a[7]=t4,a[8]=((C_word*)t0)[6],a[9]=t2,tmp=(C_word)a,a+=10,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11303,a[2]=t7,a[3]=t13,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t15=((C_word*)t13)[1];
f_11303(t15,t11,t2);}

/* g2282 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11143(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11143,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11147,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:730: chicken.base#gensym */
t4=*((C_word*)lf[97]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k11145 in g2282 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_11147,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11150,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:731: db-put! */
t3=*((C_word*)lf[131]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[223];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k11148 in k11145 in g2282 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11150(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11150,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11156(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,4)))){
C_save_and_reclaim((void *)f_11156,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11159,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(t1,lf[125]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11253,a[2]=t2,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11255,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_11255(t12,t8,((C_word*)t0)[9],t1);}

/* k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_11159,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11224,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* support.scm:737: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[224];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* g2349 in k11230 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_fcall f_11176(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_11176,3,t0,t1,t2);}
/* support.scm:740: g2366 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10972(t3,t1,t2,((C_word*)t0)[3]);}

/* k11184 in k11230 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_11186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_11186,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[118],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2343 in k11230 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_fcall f_11188(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11188,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11213,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:740: g2349 */
t4=((C_word*)t0)[4];
f_11176(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11211 in map-loop2343 in k11230 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_11213(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11213,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11188(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11224(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,5)))){
C_save_and_reclaim((void *)f_11224,c,av);}
a=C_alloc(14);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11232,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11240,a[2]=t3,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[9])){
/* support.scm:699: chicken.base#alist-ref */
t5=*((C_word*)lf[220]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[4];
av2[4]=*((C_word*)lf[221]+1);
av2[5]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
/* support.scm:738: build-lambda-list */
t5=*((C_word*)lf[54]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[8];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k11230 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_11232(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,3)))){
C_save_and_reclaim((void *)f_11232,c,av);}
a=C_alloc(33);
t2=C_i_cadddr(((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,((C_word*)t0)[3],((C_word*)t0)[4],t1,t2);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11176,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t9=C_i_check_list_2(((C_word*)t0)[7],lf[125]);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11186,a[2]=((C_word*)t0)[8],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11188,a[2]=t6,a[3]=t12,a[4]=t8,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_11188(t14,t10,((C_word*)t0)[7]);}

/* k11238 in k11222 in k11157 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_11240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11240,c,av);}
/* support.scm:738: build-lambda-list */
t2=*((C_word*)lf[54]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11251 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11253,c,av);}
/* support.scm:734: scheme#append */
t2=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop2306 in k11154 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_11255(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_11255,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_cons(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* map-loop2276 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11303(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11303,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11328,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:729: g2282 */
t4=((C_word*)t0)[4];
f_11143(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11326 in map-loop2276 in a11137 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11328,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11303(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11345 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_11347,c,av);}
a=C_alloc(22);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11352,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(((C_word*)t0)[4],lf[125]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11362,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11364,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_11364(t12,t8,((C_word*)t0)[4]);}

/* g2388 in k11345 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11352(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_11352,3,t0,t1,t2);}
/* support.scm:742: g2405 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10972(t3,t1,t2,((C_word*)t0)[3]);}

/* k11360 in k11345 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11362(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_11362,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2382 in k11345 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11364(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11364,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11389,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:742: g2388 */
t4=((C_word*)t0)[4];
f_11352(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11387 in map-loop2382 in k11345 in walk in k10962 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11389,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11364(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* map-loop2195 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11411(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_11411,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_cons(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11462(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_11462,3,t0,t1,t2);}
a=C_alloc(9);
t3=C_slot(t2,C_fix(3));
t4=C_slot(t2,C_fix(2));
t5=C_slot(t2,C_fix(1));
t6=C_eqp(t5,lf[213]);
if(C_truep(t6)){
t7=C_i_car(t4);
t8=C_eqp(((C_word*)t0)[2],t7);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11506,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t10=C_i_length(((C_word*)t0)[3]);
t11=C_i_cadr(t4);
/* support.scm:755: qnode */
t12=*((C_word*)lf[155]+1);{
C_word av2[3];
av2[0]=t12;
av2[1]=t9;
av2[2]=C_i_less_or_equalp(t10,t11);
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t7=C_eqp(t5,lf[215]);
if(C_truep(t7)){
t8=C_i_car(t4);
t9=C_eqp(((C_word*)t0)[2],t8);
if(C_truep(t9)){
t10=C_i_cadr(t4);
t11=C_i_length(((C_word*)t0)[3]);
if(C_truep(C_i_greaterp(t11,t10))){
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11553,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:762: varnode */
t13=*((C_word*)lf[153]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=C_i_list_ref(((C_word*)t0)[3],t10);
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t12=C_a_i_list1(&a,1,lf[216]);
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11581,a[2]=t12,a[3]=t1,a[4]=t2,a[5]=t10,tmp=(C_word)a,a+=6,tmp);
/* support.scm:765: qnode */
t14=*((C_word*)lf[155]+1);{
C_word av2[3];
av2[0]=t14;
av2[1]=t13;
av2[2]=t11;
((C_proc)(void*)(*((C_word*)t14+1)))(3,av2);}}}
else{
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t8=C_eqp(t5,lf[217]);
if(C_truep(t8)){
t9=C_i_car(t4);
t10=C_eqp(((C_word*)t0)[2],t9);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11608,a[2]=t4,a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:770: collect! */
t12=*((C_word*)lf[133]+1);{
C_word av2[6];
av2[0]=t12;
av2[1]=t11;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[209];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t12+1)))(6,av2);}}
else{
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}
else{
t9=C_i_check_list_2(t3,lf[44]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11664,a[2]=t11,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp));
t13=((C_word*)t11)[1];
f_11664(t13,t1,t3);}}}}

/* k11504 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11506,c,av);}
/* support.scm:755: copy-node! */
t2=*((C_word*)lf[214]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11551 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11553,c,av);}
/* support.scm:762: copy-node! */
t2=*((C_word*)lf[214]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11579 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_11581,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11585,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:765: qnode */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11583 in k11579 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_11585,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11589,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* support.scm:765: qnode */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11587 in k11583 in k11579 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_11589(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_11589,c,av);}
a=C_alloc(14);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_record4(&a,4,lf[141],lf[172],((C_word*)t0)[4],t2);
/* support.scm:763: copy-node! */
t4=*((C_word*)lf[214]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
av2[2]=t3;
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11606 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_11608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,2)))){
C_save_and_reclaim((void *)f_11608,c,av);}
a=C_alloc(34);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_s_a_i_plus(&a,2,t2,C_fix(1));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11620,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:772: varnode */
t5=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11618 in k11606 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11620(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11620,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11622,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_11622(t5,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* lp in k11618 in k11606 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_fcall f_11622(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(40,0,3)))){
C_save_and_reclaim_args((void *)trf_11622,4,t0,t1,t2,t3);}
a=C_alloc(40);
if(C_truep(C_i_zerop(t2))){
/* support.scm:774: copy-node! */
t4=*((C_word*)lf[214]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=C_s_a_i_minus(&a,2,t2,C_fix(1));
t5=C_a_i_list1(&a,1,lf[218]);
t6=C_a_i_list1(&a,1,t3);
t7=C_a_i_record4(&a,4,lf[141],lf[172],t5,t6);
/* support.scm:775: lp */
t9=t1;
t10=t4;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}

/* for-each-loop2465 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_11664(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11664,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11674,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:778: g2466 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_11462(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11672 in for-each-loop2465 in walk in k10867 in k10658 in k10644 in k10641 in a10638 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_11674(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11674,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11664(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.support#replace-rest-op-with-list-ops in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,4)))){
C_save_and_reclaim((void *)f_11696,c,av);}
a=C_alloc(34);
t5=C_eqp(t2,lf[215]);
if(C_truep(t5)){
t6=C_a_i_list1(&a,1,lf[228]);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11723,a[2]=t3,a[3]=t1,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
/* support.scm:787: qnode */
t8=*((C_word*)lf[155]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=C_i_cadr(t4);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t6=C_eqp(t2,lf[217]);
if(C_truep(t6)){
t7=C_i_cadr(t4);
t8=C_s_a_i_plus(&a,2,t7,C_fix(1));
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11743,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t12=((C_word*)t10)[1];
f_11743(t12,t1,t8,t3);}
else{
t7=C_eqp(t2,lf[213]);
if(C_truep(t7)){
t8=C_a_i_list1(&a,1,lf[230]);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11796,a[2]=t3,a[3]=t1,a[4]=t8,tmp=(C_word)a,a+=5,tmp);
/* support.scm:798: qnode */
t10=*((C_word*)lf[155]+1);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
av2[2]=C_i_cadr(t4);
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t8=C_eqp(t2,lf[232]);
if(C_truep(t8)){
t9=C_a_i_list1(&a,1,lf[233]);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11843,a[2]=t3,a[3]=t1,a[4]=t9,tmp=(C_word)a,a+=5,tmp);
/* support.scm:803: qnode */
t11=*((C_word*)lf[155]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=C_i_cadr(t4);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
/* support.scm:804: bomb */
t9=*((C_word*)lf[13]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[234];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}}}}}

/* k11721 in chicken.compiler.support#replace-rest-op-with-list-ops in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_11723,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[172],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* lp in chicken.compiler.support#replace-rest-op-with-list-ops in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11743(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(40,0,3)))){
C_save_and_reclaim_args((void *)trf_11743,4,t0,t1,t2,t3);}
a=C_alloc(40);
if(C_truep(C_i_zerop(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_s_a_i_minus(&a,2,t2,C_fix(1));
t5=C_a_i_list1(&a,1,lf[229]);
t6=C_a_i_list1(&a,1,t3);
t7=C_a_i_record4(&a,4,lf[141],lf[172],t5,t6);
/* support.scm:793: lp */
t9=t1;
t10=t4;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}

/* k11794 in chicken.compiler.support#replace-rest-op-with-list-ops in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,1)))){
C_save_and_reclaim((void *)f_11796,c,av);}
a=C_alloc(22);
t2=C_a_i_list1(&a,1,lf[231]);
t3=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t4=C_a_i_record4(&a,4,lf[141],lf[172],t2,t3);
t5=C_a_i_list2(&a,2,t1,t4);
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[172],((C_word*)t0)[4],t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k11841 in chicken.compiler.support#replace-rest-op-with-list-ops in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_11843,c,av);}
a=C_alloc(11);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[172],((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#tree-copy in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11860,c,av);}
a=C_alloc(5);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11866,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_11866(t6,t1,t2);}

/* rec in chicken.compiler.support#tree-copy in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_11866(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11866,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11880,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:810: rec */
t5=t3;
t6=C_u_i_car(t2);
t1=t5;
t2=t6;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11878 in rec in chicken.compiler.support#tree-copy in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11880,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11884,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:810: rec */
t3=((C_word*)((C_word*)t0)[3])[1];
f_11866(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k11882 in k11878 in rec in chicken.compiler.support#tree-copy in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_11884,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#copy-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_11890,c,av);}
a=C_alloc(5);
t3=C_slot(t2,C_fix(1));
t4=C_slot(t2,C_fix(2));
t5=C_slot(t2,C_fix(3));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_record4(&a,4,lf[141],t3,t4,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* chicken.compiler.support#copy-node! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11928,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11932,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:819: node-class-set! */
t5=*((C_word*)lf[145]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=C_slot(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k11930 in chicken.compiler.support#copy-node! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11932,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11935,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:820: node-parameters-set! */
t3=*((C_word*)lf[149]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_slot(((C_word*)t0)[4],C_fix(2));
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11933 in k11930 in chicken.compiler.support#copy-node! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11935,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11938,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:821: node-subexpressions-set! */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_slot(((C_word*)t0)[4],C_fix(3));
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11936 in k11933 in k11930 in chicken.compiler.support#copy-node! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_11938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11938,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* walk in k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_11973(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_11973,3,t0,t1,t2);}
a=C_alloc(18);
t3=C_slot(t2,C_fix(1));
t4=C_slot(t2,C_fix(2));
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_slot(t2,C_fix(3));
t10=C_i_check_list_2(t9,lf[125]);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12017,a[2]=t4,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12019,a[2]=t7,a[3]=t13,a[4]=((C_word*)t0)[2],a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t15=((C_word*)t13)[1];
f_12019(t15,t11,t9);}

/* k12015 in walk in k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_12017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_12017,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop2571 in walk in k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_fcall f_12019(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12019,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12044,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:828: g2577 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_11973(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12042 in map-loop2571 in walk in k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_12044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12044,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_12019(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* walk in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12059(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_12059,3,t0,t1,t2);}
a=C_alloc(18);
t3=C_i_car(t2);
t4=C_i_cadr(t2);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_u_i_cdr(t2);
t10=C_u_i_cdr(t9);
t11=C_i_check_list_2(t10,lf[125]);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12088,a[2]=t1,a[3]=t3,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12090,a[2]=t7,a[3]=t14,a[4]=((C_word*)t0)[2],a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_12090(t16,t12,t10);}

/* k12086 in walk in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_12088,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2610 in walk in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12090(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12090,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12115,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:832: g2616 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12059(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12113 in map-loop2610 in walk in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12115,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_12090(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12124(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_12124,c,av);}
a=C_alloc(22);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12127,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_END_OF_LIST;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_END_OF_LIST;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12181,a[2]=t1,a[3]=t10,a[4]=t12,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
t14=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12285,a[2]=t10,a[3]=t12,a[4]=t8,a[5]=t6,a[6]=t4,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
/* support.scm:847: chicken.internal#hash-table-for-each */
t15=*((C_word*)lf[139]+1);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t13;
av2[2]=t14;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t15+1)))(4,av2);}}

/* uses-foreign-stubs? in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12127(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12127,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12133,a[2]=((C_word*)t0)[2],a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];{
C_word av2[3];
av2[0]=t6;
av2[1]=t1;
av2[2]=t2;
f_12133(3,av2);}}

/* walk in uses-foreign-stubs? in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12133,c,av);}
t3=C_slot(t2,C_fix(1));
t4=C_eqp(t3,lf[172]);
if(C_truep(t4)){
t5=C_slot(t2,C_fix(2));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_i_memq(C_i_car(t5),((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* support.scm:844: any */
f_5777(t1,((C_word*)((C_word*)t0)[3])[1],C_slot(t2,C_fix(3)));}}

/* k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_12181,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12184,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12235,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* support.scm:868: scheme#with-output-to-file */
t4=*((C_word*)lf[250]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12184,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12190,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[3])[1]))){
/* support.scm:879: debugging */
t3=*((C_word*)lf[22]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[242];
av2[3]=lf[243];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_12190(2,av2);}}}

/* k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12190,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12198,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7482,tmp=(C_word)a,a+=2,tmp);
/* support.scm:281: chicken.sort#sort */
t5=*((C_word*)lf[241]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12196 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12198,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[44]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12206,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_12206(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2736 in k12196 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12206(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_12206,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12216,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:880: g2752 */
t4=*((C_word*)lf[237]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[238];
av2[3]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12214 in for-each-loop2736 in k12196 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12216,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12206(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12235,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12239,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12283,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:870: chicken.platform#chicken-version */
t4=*((C_word*)lf[249]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12239,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12250,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:876: scheme#reverse */
t3=*((C_word*)lf[80]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12242 in for-each-loop2714 in k12248 in k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12244(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12244,c,av);}
/* support.scm:875: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12248 in k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_12250,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12253,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12258,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_12258(t6,t2,t1);}

/* k12251 in k12248 in k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12253,c,av);}
/* support.scm:877: chicken.base#print */
t2=*((C_word*)lf[237]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[244];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* for-each-loop2714 in k12248 in k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12258(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_12258,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12268,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12244,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:874: chicken.pretty-print#pp */
t6=*((C_word*)lf[245]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12266 in for-each-loop2714 in k12248 in k12237 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12268,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12258(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k12281 in a12234 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_12283,c,av);}
/* support.scm:870: chicken.base#print */
t2=*((C_word*)lf[237]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[246];
av2[3]=t1;
av2[4]=lf[247];
av2[5]=((C_word*)t0)[3];
av2[6]=lf[248];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12285,c,av);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12292,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=t1,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* support.scm:849: variable-visible? */
t5=*((C_word*)lf[260]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_12292,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_i_assq(lf[251],((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12440,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* tweaks.scm:60: ##sys#get */
t4=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[259];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12313(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_12313,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
if(C_truep(C_i_assq(lf[253],((C_word*)t0)[2]))){
t2=C_i_cdr(((C_word*)t0)[3]);
t3=C_slot(t2,C_fix(2));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12407,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[8],a[8]=t3,a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* support.scm:857: db-get */
t5=*((C_word*)lf[127]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[10];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[257];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12347,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12350,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=C_eqp(t1,lf[254]);
if(C_truep(t3)){
t4=t2;
f_12350(t4,C_SCHEME_TRUE);}
else{
t4=C_eqp(t1,lf[255]);
t5=t2;
f_12350(t5,(C_truep(t4)?C_SCHEME_FALSE:C_i_lessp(C_i_cadddr(((C_word*)t0)[8]),((C_word*)t0)[9])));}}

/* k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12350(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_12350,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12381,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* support.scm:864: uses-foreign-stubs? */
t3=((C_word*)t0)[7];
f_12127(t3,t2,C_u_i_cdr(((C_word*)t0)[6]));}
else{
t2=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12373 in k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_12375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_12375,c,av);}
a=C_alloc(9);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)((C_word*)t0)[3])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k12379 in k12348 in k12345 in k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_12381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12381,c,av);}
a=C_alloc(13);
if(C_truep(C_i_not(t1))){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12375,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t5=C_u_i_cdr(((C_word*)t0)[6]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11973,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_11973(t9,t4,t5);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12405 in k12311 in k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12407(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12407,c,av);}
a=C_alloc(10);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12347,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* tweaks.scm:60: ##sys#get */
t3=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[256];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12438 in k12290 in a12284 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_12440,c,av);}
a=C_alloc(11);
t2=C_i_structurep(t1,lf[141]);
if(C_truep(C_i_not(t2))){
t3=C_i_assq(lf[252],((C_word*)t0)[2]);
t4=C_i_not(t3);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12313,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t4)){
t6=t5;
f_12313(t6,t4);}
else{
t6=C_i_cdr(t3);
t7=t5;
f_12313(t7,C_i_not(C_eqp(lf[258],t6)));}}
else{
t3=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_12442,c,av);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12448,tmp=(C_word)a,a+=2,tmp);
/* support.scm:884: scheme#with-input-from-file */
t4=*((C_word*)lf[263]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12448,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12454,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_12454(t5,t1);}

/* loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12454(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_12454,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12458,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:887: scheme#read */
t3=*((C_word*)lf[83]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12458,c,av);}
a=C_alloc(13);
if(C_truep(C_eofp(t1))){
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12481,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_i_car(t1);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12492,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_i_cadr(t1);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12059,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_12059(t9,t4,t5);}}

/* k12479 in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12481,c,av);}
/* support.scm:893: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12454(t2,((C_word*)t0)[3]);}

/* k12490 in k12456 in loop in a12447 in chicken.compiler.support#load-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12492(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12492,c,av);}
a=C_alloc(3);
t2=C_a_i_list(&a,1,t1);
if(C_truep(C_i_nullp(t2))){
/* tweaks.scm:57: ##sys#put! */
t3=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[259];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* tweaks.scm:57: ##sys#put! */
t3=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[259];
av2[4]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_12498,c,av);}
a=C_alloc(27);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12501,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t14=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12535,a[2]=t8,a[3]=t10,tmp=(C_word)a,a+=4,tmp));
t15=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12582,a[2]=t8,a[3]=t12,a[4]=t10,tmp=(C_word)a,a+=5,tmp));
t16=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12705,a[2]=t6,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:927: matchn */
t17=((C_word*)t12)[1];
f_12582(t17,t16,t2,t3);}

/* resolve in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12501(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12501,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_assq(t2,((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12509,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:902: g2798 */
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(
/* support.scm:902: g2798 */
  f_12509(t5,t4)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_i_memq(t2,((C_word*)t0)[3]))){
t5=((C_word*)((C_word*)t0)[2])[1];
t6=C_a_i_cons(&a,2,t2,t3);
t7=C_a_i_cons(&a,2,t6,t5);
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,t7);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_eqp(t2,t3);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}

/* g2798 in resolve in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_12509(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_i_equalp(((C_word*)t0)[2],C_i_cdr(t1)));}

/* match1 in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12535(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12535,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t3);
if(C_truep(C_i_not(t4))){
/* support.scm:909: resolve */
t5=((C_word*)((C_word*)t0)[2])[1];
f_12501(t5,t1,t3,t2);}
else{
t5=C_i_pairp(t2);
if(C_truep(C_i_not(t5))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12557,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:911: match1 */
t8=t6;
t9=C_i_car(t2);
t10=C_i_car(t3);
t1=t8;
t2=t9;
t3=t10;
goto loop;}}}

/* k12555 in match1 in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12557,c,av);}
if(C_truep(t1)){
/* support.scm:911: match1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12535(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* matchn in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12582(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_12582,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_pairp(t3);
if(C_truep(C_i_not(t4))){
/* support.scm:916: resolve */
t5=((C_word*)((C_word*)t0)[2])[1];
f_12501(t5,t1,t3,t2);}
else{
t5=C_slot(t2,C_fix(1));
t6=C_i_car(t3);
t7=C_eqp(t5,t6);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12604,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* support.scm:918: match1 */
t9=((C_word*)((C_word*)t0)[4])[1];
f_12535(t9,t8,C_slot(t2,C_fix(2)),C_i_cadr(t3));}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}

/* k12602 in matchn in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_12604,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(3));
t3=C_i_cddr(((C_word*)t0)[3]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12622,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_12622(t7,((C_word*)t0)[6],t2,t3);}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* loop in k12602 in matchn in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12622(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12622,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_nullp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_pairp(t3);
if(C_truep(C_i_not(t4))){
/* support.scm:922: resolve */
t5=((C_word*)((C_word*)t0)[2])[1];
f_12501(t5,t1,t3,t2);}
else{
if(C_truep(C_i_nullp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12653,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:924: matchn */
t6=((C_word*)((C_word*)t0)[4])[1];
f_12582(t6,t5,C_i_car(t2),C_i_car(t3));}}}}

/* k12651 in loop in k12602 in matchn in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12653,c,av);}
if(C_truep(t1)){
/* support.scm:925: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12622(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12703 in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_12705,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12711,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:930: debugging */
t3=*((C_word*)lf[22]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[265];
av2[3]=lf[266];
av2[4]=C_slot(((C_word*)t0)[4],C_fix(1));
av2[5]=C_slot(((C_word*)t0)[4],C_fix(2));
av2[6]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(7,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12709 in k12703 in chicken.compiler.support#match-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12711(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12711,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12731,c,av);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12737,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
av2[2]=t2;
f_12737(3,av2);}}

/* walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12737,c,av);}
a=C_alloc(7);
t3=C_slot(t2,C_fix(3));
t4=C_slot(t2,C_fix(1));
t5=C_eqp(t4,lf[154]);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12763,a[2]=t1,a[3]=t4,a[4]=t2,a[5]=((C_word*)t0)[2],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t5)){
t7=t6;
f_12763(t7,t5);}
else{
t7=C_eqp(t4,lf[85]);
if(C_truep(t7)){
t8=t6;
f_12763(t8,t7);}
else{
t8=C_eqp(t4,lf[159]);
t9=t6;
f_12763(t9,(C_truep(t8)?t8:C_eqp(t4,lf[174])));}}}

/* k12761 in walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12763(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_12763,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[118]);
if(C_truep(t2)){
t3=C_slot(((C_word*)t0)[4],C_fix(2));
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12777,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=*((C_word*)lf[269]+1);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6504,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_6504(t10,((C_word*)t0)[2],*((C_word*)lf[269]+1));}
else{
t3=C_eqp(((C_word*)t0)[3],lf[158]);
if(C_truep(t3)){
if(C_truep(t3)){
/* support.scm:946: any */
f_5777(((C_word*)t0)[2],((C_word*)((C_word*)t0)[5])[1],((C_word*)t0)[6]);}
else{
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[96]);
if(C_truep(t4)){
/* support.scm:946: any */
f_5777(((C_word*)t0)[2],((C_word*)((C_word*)t0)[5])[1],((C_word*)t0)[6]);}
else{
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}}}

/* a12776 in k12761 in walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12777(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_12777,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12785,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:944: foreign-callback-stub-id */
t4=*((C_word*)lf[268]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12783 in a12776 in k12761 in walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12785,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#simple-lambda-node? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_12840,c,av);}
a=C_alloc(6);
t3=C_slot(t2,C_fix(2));
t4=C_i_caddr(t3);
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_i_car(t4):C_SCHEME_FALSE);
if(C_truep(t6)){
if(C_truep(C_i_cadr(t3))){
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12869,a[2]=t6,a[3]=t8,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t2;
f_12869(3,av2);}}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* rec in chicken.compiler.support#simple-lambda-node? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12869,c,av);}
t3=C_slot(t2,C_fix(1));
t4=C_eqp(t3,lf[179]);
if(C_truep(t4)){
t5=C_slot(t2,C_fix(3));
t6=C_i_car(t5);
t7=C_slot(t6,C_fix(1));
t8=C_eqp(lf[154],t7);
if(C_truep(t8)){
t9=C_slot(t6,C_fix(2));
t10=C_i_car(t9);
t11=C_eqp(((C_word*)t0)[2],t10);
if(C_truep(t11)){
/* support.scm:962: every */
f_5743(t1,((C_word*)((C_word*)t0)[3])[1],C_i_cdr(t5));}
else{
t12=t1;{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t5=C_eqp(t3,lf[189]);
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* support.scm:964: every */
f_5743(t1,((C_word*)((C_word*)t0)[3])[1],C_slot(t2,C_fix(3)));}}}

/* chicken.compiler.support#dump-undefined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_12966,c,av);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12972,tmp=(C_word)a,a+=2,tmp);
/* support.scm:970: chicken.internal#hash-table-for-each */
t4=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a12971 in chicken.compiler.support#dump-undefined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12972,c,av);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12979,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13005,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:972: chicken.keyword#keyword? */
t6=*((C_word*)lf[275]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k12977 in a12971 in chicken.compiler.support#dump-undefined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_12979(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_12979,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12982,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:975: scheme#write */
t3=*((C_word*)lf[272]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12980 in k12977 in a12971 in chicken.compiler.support#dump-undefined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_12982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12982,c,av);}
/* support.scm:976: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13003 in a12971 in chicken.compiler.support#dump-undefined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13005(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13005,c,av);}
if(C_truep(C_i_not(t1))){
t2=C_i_assq(lf[273],((C_word*)t0)[2]);
t3=((C_word*)t0)[3];
f_12979(t3,(C_truep(t2)?C_i_not(C_i_assq(lf[274],((C_word*)t0)[2])):C_SCHEME_FALSE));}
else{
t2=((C_word*)t0)[3];
f_12979(t2,C_SCHEME_FALSE);}}

/* chicken.compiler.support#dump-defined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13007(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_13007,c,av);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13013,tmp=(C_word)a,a+=2,tmp);
/* support.scm:980: chicken.internal#hash-table-for-each */
t4=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a13012 in chicken.compiler.support#dump-defined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13013,c,av);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13020,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13042,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:982: chicken.keyword#keyword? */
t6=*((C_word*)lf[275]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k13018 in a13012 in chicken.compiler.support#dump-defined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13020(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_13020,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13023,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:985: scheme#write */
t3=*((C_word*)lf[272]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13021 in k13018 in a13012 in chicken.compiler.support#dump-defined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13023,c,av);}
/* support.scm:986: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13040 in a13012 in chicken.compiler.support#dump-defined-globals in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13042,c,av);}
if(C_truep(C_i_not(t1))){
t2=C_i_assq(lf[273],((C_word*)t0)[2]);
t3=((C_word*)t0)[3];
f_13020(t3,(C_truep(t2)?C_i_assq(lf[274],((C_word*)t0)[2]):C_SCHEME_FALSE));}
else{
t2=((C_word*)t0)[3];
f_13020(t2,C_SCHEME_FALSE);}}

/* chicken.compiler.support#dump-global-refs in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_13044,c,av);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13050,tmp=(C_word)a,a+=2,tmp);
/* support.scm:990: chicken.internal#hash-table-for-each */
t4=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a13049 in chicken.compiler.support#dump-global-refs in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13050(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_13050,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13091,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:992: chicken.keyword#keyword? */
t5=*((C_word*)lf[275]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13061 in k13089 in a13049 in chicken.compiler.support#dump-global-refs in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13063(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13063,c,av);}
/* support.scm:995: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13089 in a13049 in chicken.compiler.support#dump-global-refs in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13091(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_13091,c,av);}
a=C_alloc(9);
t2=C_i_not(t1);
t3=(C_truep(t2)?C_i_assq(lf[273],((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_assq(lf[209],((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13063,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t4)){
t6=C_i_cdr(t4);
t7=C_i_length(t6);
t8=C_a_i_list2(&a,2,((C_word*)t0)[4],t7);
/* support.scm:994: scheme#write */
t9=*((C_word*)lf[272]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t5;
av2[2]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t6=C_a_i_list2(&a,2,((C_word*)t0)[4],C_fix(0));
/* support.scm:994: scheme#write */
t7=*((C_word*)lf[272]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}
else{
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* ##sys#toplevel-definition-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13093,c,av);}
a=C_alloc(4);
if(C_truep(C_u_i_namespaced_symbolp(t2))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17936,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1749: variable-hidden? */
t6=*((C_word*)lf[281]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
if(C_truep(C_i_not(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13114,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1006: debugging */
t6=*((C_word*)lf[22]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[190];
av2[3]=lf[283];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}

/* k13112 in ##sys#toplevel-definition-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13114,c,av);}
/* support.scm:1007: hide-variable */
t2=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.support#make-foreign-callback-stub in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_13121,c,av);}
a=C_alloc(7);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_record6(&a,6,lf[285],t2,t3,t4,t5,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13127,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[285]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub-id in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13133,c,av);}
t3=C_i_check_structure_2(t2,lf[285],lf[287]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13142,c,av);}
t3=C_i_check_structure_2(t2,lf[285],lf[289]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub-qualifiers in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13151,c,av);}
t3=C_i_check_structure_2(t2,lf[285],lf[291]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub-return-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13160,c,av);}
t3=C_i_check_structure_2(t2,lf[285],lf[293]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(4));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#foreign-callback-stub-argument-types in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13169,c,av);}
t3=C_i_check_structure_2(t2,lf[285],lf[295]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#register-foreign-callback-stub! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13178,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13204,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t4;
av2[2]=*((C_word*)lf[284]+1);
av2[3]=t2;
av2[4]=t3;
C_apply(5,av2);}}

/* k13202 in chicken.compiler.support#register-foreign-callback-stub! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_13204,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,*((C_word*)lf[269]+1));
t3=C_mutate((C_word*)lf[269]+1 /* (set! chicken.compiler.support#foreign-callback-stubs ...) */,t2);
t4=C_SCHEME_END_OF_LIST;
if(C_truep(C_i_nullp(t4))){
/* tweaks.scm:57: ##sys#put! */
t5=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[297];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
/* tweaks.scm:57: ##sys#put! */
t5=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[297];
av2[4]=C_i_car(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* chicken.compiler.support#clear-foreign-type-table! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_13207,c,av);}
a=C_alloc(3);
if(C_truep(lf[298])){
/* support.scm:1035: scheme#vector-fill! */
t2=*((C_word*)lf[300]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[298];
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13218,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1036: scheme#make-vector */
t3=*((C_word*)lf[301]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(301);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k13216 in chicken.compiler.support#clear-foreign-type-table! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13218(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13218,c,av);}
t2=C_mutate(&lf[298] /* (set! chicken.compiler.support#foreign-type-table ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#register-foreign-type! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13220,c,av);}
a=C_alloc(4);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=C_rest_nullp(c,4);
t7=C_rest_nullp(c,5);
t8=(C_truep(t7)?C_SCHEME_FALSE:C_get_rest_arg(c,5,av,4,t0));
t9=C_rest_nullp(c,5);
t10=(C_truep(t8)?t5:C_SCHEME_FALSE);
t11=(C_truep(t5)?C_a_i_vector3(&a,3,t3,t10,t8):C_a_i_vector3(&a,3,t3,t10,C_SCHEME_FALSE));
/* support.scm:1044: chicken.internal#hash-table-set! */
t12=*((C_word*)lf[132]+1);{
C_word av2[5];
av2[0]=t12;
av2[1]=t1;
av2[2]=lf[298];
av2[3]=t2;
av2[4]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}

/* chicken.compiler.support#lookup-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13274,c,av);}
/* support.scm:1052: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[128]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[298];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_13280,c,av);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13286,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14350,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1069: follow-without-loop */
f_7445(t1,t3,t4,t5);}

/* a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_13286,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13292,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_13292(t7,t1,t2);}

/* repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13292(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_13292,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_eqp(t2,lf[306]);
t4=(C_truep(t3)?t3:C_eqp(t2,lf[307]));
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(*((C_word*)lf[10]+1))?((C_word*)t0)[2]:C_a_i_list(&a,2,lf[308],((C_word*)t0)[2]));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(t2,lf[309]);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13317,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t5)){
t7=t6;
f_13317(t7,t5);}
else{
t7=C_eqp(t2,lf[396]);
if(C_truep(t7)){
t8=t6;
f_13317(t8,t7);}
else{
t8=C_eqp(t2,lf[397]);
if(C_truep(t8)){
t9=t6;
f_13317(t9,t8);}
else{
t9=C_eqp(t2,lf[398]);
if(C_truep(t9)){
t10=t6;
f_13317(t10,t9);}
else{
t10=C_eqp(t2,lf[399]);
t11=t6;
f_13317(t11,(C_truep(t10)?t10:C_eqp(t2,lf[400])));}}}}}}

/* k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13317(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_13317,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=(C_truep(*((C_word*)lf[10]+1))?((C_word*)t0)[3]:C_a_i_list(&a,2,lf[310],((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[311]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13332,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_13332(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[394]);
t5=t3;
f_13332(t5,(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[395])));}}}

/* k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13332(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_13332,2,t0,t1);}
a=C_alloc(15);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=(C_truep(*((C_word*)lf[10]+1))?((C_word*)t0)[3]:C_a_i_list(&a,2,lf[312],((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[313]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[4],lf[314]));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13350,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1080: chicken.base#gensym */
t5=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[319]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[320]));
if(C_truep(t5)){
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(*((C_word*)lf[10]+1))?((C_word*)t0)[3]:C_a_i_list(&a,2,lf[315],((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[4],lf[321]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13400,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1092: chicken.base#gensym */
t8=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[4],lf[323]);
if(C_truep(t7)){
if(C_truep(*((C_word*)lf[10]+1))){
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_a_i_list(&a,2,lf[316],lf[321]);
t9=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t9;
av2[1]=C_a_i_list(&a,3,lf[322],t8,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t8=C_eqp(((C_word*)t0)[4],lf[324]);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13455,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t8)){
t10=t9;
f_13455(t10,t8);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[385]);
if(C_truep(t10)){
t11=t9;
f_13455(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[386]);
if(C_truep(t11)){
t12=t9;
f_13455(t12,t11);}
else{
t12=C_eqp(((C_word*)t0)[4],lf[387]);
if(C_truep(t12)){
t13=t9;
f_13455(t13,t12);}
else{
t13=C_eqp(((C_word*)t0)[4],lf[388]);
if(C_truep(t13)){
t14=t9;
f_13455(t14,t13);}
else{
t14=C_eqp(((C_word*)t0)[4],lf[389]);
if(C_truep(t14)){
t15=t9;
f_13455(t15,t14);}
else{
t15=C_eqp(((C_word*)t0)[4],lf[390]);
if(C_truep(t15)){
t16=t9;
f_13455(t16,t15);}
else{
t16=C_eqp(((C_word*)t0)[4],lf[391]);
if(C_truep(t16)){
t17=t9;
f_13455(t17,t16);}
else{
t17=C_eqp(((C_word*)t0)[4],lf[392]);
t18=t9;
f_13455(t18,(C_truep(t17)?t17:C_eqp(((C_word*)t0)[4],lf[393])));}}}}}}}}}}}}}}

/* k13348 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,1)))){
C_save_and_reclaim((void *)f_13350,c,av);}
a=C_alloc(42);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=(C_truep(*((C_word*)lf[10]+1))?t1:C_a_i_list(&a,2,lf[315],t1));
t5=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t6=C_a_i_list(&a,4,lf[317],t1,t4,t5);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_list(&a,3,lf[318],t3,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k13398 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_13400,c,av);}
a=C_alloc(29);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13415,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[10]+1))){
t5=t4;
f_13415(t5,t1);}
else{
t5=C_a_i_list(&a,2,lf[316],lf[321]);
t6=t4;
f_13415(t6,C_a_i_list(&a,3,lf[322],t5,t1));}}

/* k13413 in k13398 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13415(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,1)))){
C_save_and_reclaim_args((void *)trf_13415,2,t0,t1);}
a=C_alloc(27);
t2=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t3=C_a_i_list(&a,4,lf[317],((C_word*)t0)[2],t1,t2);
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,3,lf[318],((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13455(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_13455,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13458,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1105: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[325]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13497,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_13497(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[376]);
if(C_truep(t4)){
t5=t3;
f_13497(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[377]);
if(C_truep(t5)){
t6=t3;
f_13497(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[378]);
if(C_truep(t6)){
t7=t3;
f_13497(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[379]);
if(C_truep(t7)){
t8=t3;
f_13497(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[4],lf[380]);
if(C_truep(t8)){
t9=t3;
f_13497(t9,t8);}
else{
t9=C_eqp(((C_word*)t0)[4],lf[381]);
if(C_truep(t9)){
t10=t3;
f_13497(t10,t9);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[382]);
if(C_truep(t10)){
t11=t3;
f_13497(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[383]);
t12=t3;
f_13497(t12,(C_truep(t11)?t11:C_eqp(((C_word*)t0)[4],lf[384])));}}}}}}}}}}

/* k13456 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_13458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_13458,c,av);}
a=C_alloc(29);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13473,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[10]+1))){
t5=t4;
f_13473(t5,t1);}
else{
t5=C_a_i_list(&a,2,lf[316],((C_word*)t0)[4]);
t6=t4;
f_13473(t6,C_a_i_list(&a,3,lf[322],t5,t1));}}

/* k13471 in k13456 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13473(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,1)))){
C_save_and_reclaim_args((void *)trf_13473,2,t0,t1);}
a=C_alloc(27);
t2=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t3=C_a_i_list(&a,4,lf[317],((C_word*)t0)[2],t1,t2);
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,3,lf[318],((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13497(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_13497,2,t0,t1);}
a=C_alloc(15);
if(C_truep(t1)){
if(C_truep(*((C_word*)lf[10]+1))){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_slot(C_u_i_assq(((C_word*)t0)[4],lf[326]),C_fix(1));
t3=C_a_i_list(&a,2,lf[316],t2);
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,3,lf[322],t3,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[327]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13523,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_13523(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[371]);
if(C_truep(t4)){
t5=t3;
f_13523(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[372]);
if(C_truep(t5)){
t6=t3;
f_13523(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[373]);
if(C_truep(t6)){
t7=t3;
f_13523(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[374]);
t8=t3;
f_13523(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[4],lf[375])));}}}}}}

/* k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_13523(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_13523,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_slot(C_u_i_assq(((C_word*)t0)[2],((C_word*)t0)[3]),C_fix(1));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13529,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1124: chicken.base#open-output-string */
t4=*((C_word*)lf[334]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[2],lf[335]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13566,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_13566(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[2],lf[366]);
if(C_truep(t4)){
t5=t3;
f_13566(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[2],lf[367]);
if(C_truep(t5)){
t6=t3;
f_13566(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[2],lf[368]);
if(C_truep(t6)){
t7=t3;
f_13566(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[2],lf[369]);
t8=t3;
f_13566(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[2],lf[370])));}}}}}}

/* k13527 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_13529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_13529,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[328]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13535,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1124: ##sys#print */
t4=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[333];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k13533 in k13527 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_13535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13535,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13538,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1124: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13536 in k13533 in k13527 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_13538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13538,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13541,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1124: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[332];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13539 in k13536 in k13533 in k13527 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_13541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13541,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13544,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1124: chicken.base#get-output-string */
t3=*((C_word*)lf[331]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13542 in k13539 in k13536 in k13533 in k13527 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_13544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,1)))){
C_save_and_reclaim((void *)f_13544,c,av);}
a=C_alloc(18);
if(C_truep(*((C_word*)lf[10]+1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_a_i_list(&a,3,lf[329],t1,lf[309]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,3,lf[330],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_13566(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_13566,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_slot(t2,C_fix(1));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13572,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1132: chicken.base#open-output-string */
t5=*((C_word*)lf[334]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[2],lf[339]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13610,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_13610(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[2],lf[364]);
t5=t3;
f_13610(t5,(C_truep(t4)?t4:C_eqp(((C_word*)t0)[2],lf[365])));}}}

/* k13570 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_13572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_13572,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[328]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13578,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1132: ##sys#print */
t4=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[338];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k13576 in k13570 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_13578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13578,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13581,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1132: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13579 in k13576 in k13570 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_13581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13581,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13584,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1132: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[337];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13582 in k13579 in k13576 in k13570 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_13584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13584,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13587,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1132: chicken.base#get-output-string */
t3=*((C_word*)lf[331]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13585 in k13582 in k13579 in k13576 in k13570 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in ... */
static void C_ccall f_13587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,1)))){
C_save_and_reclaim((void *)f_13587,c,av);}
a=C_alloc(18);
if(C_truep(*((C_word*)lf[10]+1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_a_i_list(&a,3,lf[329],t1,lf[309]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,3,lf[336],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_13610(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_13610,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13613,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1138: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[341]);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[340],((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[4],lf[342]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13654,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t3)){
t5=t4;
f_13654(t5,t3);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[361]);
if(C_truep(t5)){
t6=t4;
f_13654(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[362]);
t7=t4;
f_13654(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[4],lf[363])));}}}}}

/* k13611 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_13613(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,1)))){
C_save_and_reclaim((void *)f_13613,c,av);}
a=C_alloc(42);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[340],t1);
t5=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t6=C_a_i_list(&a,4,lf[317],t1,t4,t5);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_list(&a,3,lf[318],t3,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_fcall f_13654(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_13654,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13657,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1146: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[345]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13699,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_13699(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[4],lf[359]);
t5=t3;
f_13699(t5,(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[360])));}}}

/* k13655 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_13657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_13657,c,av);}
a=C_alloc(26);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13672,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[10]+1))){
t5=t4;
f_13672(t5,C_a_i_list(&a,2,lf[343],t1));}
else{
t5=C_a_i_list(&a,2,lf[344],t1);
t6=t4;
f_13672(t6,C_a_i_list(&a,2,lf[343],t5));}}

/* k13670 in k13655 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_fcall f_13672(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,1)))){
C_save_and_reclaim_args((void *)trf_13672,2,t0,t1);}
a=C_alloc(27);
t2=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t3=C_a_i_list(&a,4,lf[317],((C_word*)t0)[2],t1,t2);
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,3,lf[318],((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_fcall f_13699(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_13699,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
if(C_truep(*((C_word*)lf[10]+1))){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[343],((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_a_i_list(&a,2,lf[344],((C_word*)t0)[3]);
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[343],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[346]);
if(C_truep(t2)){
if(C_truep(*((C_word*)lf[10]+1))){
t3=C_a_i_list(&a,2,lf[180],((C_word*)t0)[3]);
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,2,lf[343],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_a_i_list(&a,2,lf[180],((C_word*)t0)[3]);
t4=C_a_i_list(&a,2,lf[344],t3);
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=C_a_i_list(&a,2,lf[343],t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13742,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[4]))){
/* support.scm:1162: lookup-foreign-type */
t4=*((C_word*)lf[303]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_13742(2,av2);}}}}}

/* k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_13742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13742,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13746,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1162: g3285 */
t3=t2;
f_13746(t3,((C_word*)t0)[3],t1);}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t2=C_u_i_car(((C_word*)t0)[4]);
t3=C_eqp(t2,lf[347]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13773,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t3)){
t5=t4;
f_13773(t5,t3);}
else{
t5=C_eqp(t2,lf[357]);
if(C_truep(t5)){
t6=t4;
f_13773(t6,t5);}
else{
t6=C_eqp(t2,lf[358]);
t7=t4;
f_13773(t7,(C_truep(t6)?t6:C_eqp(t2,lf[339])));}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* g3285 in k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in ... */
static void C_fcall f_13746(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_13746,3,t0,t1,t2);}
/* support.scm:1163: next */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_i_vector_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13771 in k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in ... */
static void C_fcall f_13773(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_13773,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13776,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1167: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[348]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[4],lf[349]));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13811,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1173: chicken.base#gensym */
t5=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[314]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13847,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1179: chicken.base#gensym */
t6=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[4],lf[319]);
if(C_truep(t5)){
t6=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(*((C_word*)lf[10]+1))?((C_word*)t0)[2]:C_a_i_list(&a,2,lf[315],((C_word*)t0)[2]));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[4],lf[352]);
if(C_truep(t6)){
t7=C_a_i_list(&a,2,lf[316],lf[350]);
t8=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_list(&a,3,lf[351],((C_word*)t0)[2],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[4],lf[353]);
if(C_truep(t7)){
/* support.scm:1192: repeat */
t8=((C_word*)((C_word*)t0)[5])[1];
f_13292(t8,((C_word*)t0)[3],C_i_cadr(((C_word*)t0)[6]));}
else{
t8=C_eqp(((C_word*)t0)[4],lf[354]);
if(C_truep(t8)){
if(C_truep(*((C_word*)lf[10]+1))){
t9=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t9;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_a_i_list(&a,3,lf[329],lf[355],lf[309]);
t10=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_a_i_list(&a,3,lf[330],((C_word*)t0)[2],t9);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t9=C_eqp(((C_word*)t0)[4],lf[356]);
if(C_truep(t9)){
t10=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t10;
av2[1]=(C_truep(t9)?C_a_i_list(&a,2,lf[340],((C_word*)t0)[2]):((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(((C_word*)t0)[4],lf[341]);
t11=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t11;
av2[1]=(C_truep(t10)?C_a_i_list(&a,2,lf[340],((C_word*)t0)[2]):((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}}}}}}}

/* k13774 in k13771 in k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in ... */
static void C_ccall f_13776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,1)))){
C_save_and_reclaim((void *)f_13776,c,av);}
a=C_alloc(42);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[340],t1);
t5=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t6=C_a_i_list(&a,4,lf[317],t1,t4,t5);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_list(&a,3,lf[318],t3,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k13809 in k13771 in k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in ... */
static void C_ccall f_13811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(51,c,1)))){
C_save_and_reclaim((void *)f_13811,c,av);}
a=C_alloc(51);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[316],lf[350]);
t5=C_a_i_list(&a,3,lf[351],((C_word*)t0)[2],t4);
t6=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t7=C_a_i_list(&a,4,lf[317],t1,t5,t6);
t8=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_list(&a,3,lf[318],t3,t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* k13845 in k13771 in k13740 in k13697 in k13652 in k13608 in k13564 in k13521 in k13495 in k13453 in k13330 in k13315 in repeat in a13285 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in ... */
static void C_ccall f_13847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,1)))){
C_save_and_reclaim((void *)f_13847,c,av);}
a=C_alloc(42);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=(C_truep(*((C_word*)lf[10]+1))?t1:C_a_i_list(&a,2,lf[315],t1));
t5=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t6=C_a_i_list(&a,4,lf[317],t1,t4,t5);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_list(&a,3,lf[318],t3,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* a14349 in chicken.compiler.support#foreign-type-check in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14350,c,av);}
/* support.scm:1204: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[401];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#foreign-type-result-converter in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14356(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_14356,2,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_symbolp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14366,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1211: lookup-foreign-type */
t4=*((C_word*)lf[303]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14364 in chicken.compiler.support#foreign-type-result-converter in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14366,c,av);}
if(C_truep(t1)){
t2=C_i_vector_ref(t1,C_fix(2));
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#foreign-type-argument-converter in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14377(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_14377,2,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_symbolp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14387,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1217: lookup-foreign-type */
t4=*((C_word*)lf[303]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14385 in chicken.compiler.support#foreign-type-argument-converter in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14387,c,av);}
if(C_truep(t1)){
t2=C_i_vector_ref(t1,C_fix(1));
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#foreign-type-convert-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14398,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14402,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1222: foreign-type-result-converter */
f_14356(t4,t3);}

/* k14400 in chicken.compiler.support#foreign-type-convert-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14402,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#foreign-type-convert-argument in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14413,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14417,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1227: foreign-type-argument-converter */
f_14377(t4,t3);}

/* k14415 in chicken.compiler.support#foreign-type-convert-argument in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14417,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#final-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_14428,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14434,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14461,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1232: follow-without-loop */
f_7445(t1,t2,t3,t4);}

/* a14433 in chicken.compiler.support#final-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14434,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14438,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_symbolp(t2))){
/* support.scm:1235: lookup-foreign-type */
t5=*((C_word*)lf[303]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14438(2,av2);}}}

/* k14436 in a14433 in chicken.compiler.support#final-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14438,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14442,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1235: g3380 */
t3=t2;
f_14442(t3,((C_word*)t0)[3],t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g3380 in k14436 in a14433 in chicken.compiler.support#final-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14442(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_14442,3,t0,t1,t2);}
/* support.scm:1236: next */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_i_vector_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a14460 in chicken.compiler.support#final-foreign-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14461,c,av);}
/* support.scm:1238: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[407];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_14467,c,av);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14470,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14479,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14936,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1246: follow-without-loop */
f_7445(t1,t2,t4,t5);}

/* err in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14470(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_14470,2,t0,t1);}
/* support.scm:1245: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[409];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_14479,c,av);}
a=C_alloc(6);
t4=C_eqp(t2,lf[306]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14489,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t4)){
t6=t5;
f_14489(t6,t4);}
else{
t6=C_eqp(t2,lf[309]);
if(C_truep(t6)){
t7=t5;
f_14489(t7,t6);}
else{
t7=C_eqp(t2,lf[373]);
if(C_truep(t7)){
t8=t5;
f_14489(t8,t7);}
else{
t8=C_eqp(t2,lf[410]);
if(C_truep(t8)){
t9=t5;
f_14489(t9,t8);}
else{
t9=C_eqp(t2,lf[411]);
if(C_truep(t9)){
t10=t5;
f_14489(t10,t9);}
else{
t10=C_eqp(t2,lf[335]);
if(C_truep(t10)){
t11=t5;
f_14489(t11,t10);}
else{
t11=C_eqp(t2,lf[412]);
if(C_truep(t11)){
t12=t5;
f_14489(t12,t11);}
else{
t12=C_eqp(t2,lf[307]);
if(C_truep(t12)){
t13=t5;
f_14489(t13,t12);}
else{
t13=C_eqp(t2,lf[396]);
if(C_truep(t13)){
t14=t5;
f_14489(t14,t13);}
else{
t14=C_eqp(t2,lf[397]);
if(C_truep(t14)){
t15=t5;
f_14489(t15,t14);}
else{
t15=C_eqp(t2,lf[398]);
if(C_truep(t15)){
t16=t5;
f_14489(t16,t15);}
else{
t16=C_eqp(t2,lf[399]);
t17=t5;
f_14489(t17,(C_truep(t16)?t16:C_eqp(t2,lf[400])));}}}}}}}}}}}}

/* k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14489(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14489,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[342]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14498,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_14498(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[345]);
if(C_truep(t4)){
t5=t3;
f_14498(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[339]);
if(C_truep(t5)){
t6=t3;
f_14498(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[341]);
if(C_truep(t6)){
t7=t3;
f_14498(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[3],lf[346]);
if(C_truep(t7)){
t8=t3;
f_14498(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[3],lf[361]);
if(C_truep(t8)){
t9=t3;
f_14498(t9,t8);}
else{
t9=C_eqp(((C_word*)t0)[3],lf[359]);
if(C_truep(t9)){
t10=t3;
f_14498(t10,t9);}
else{
t10=C_eqp(((C_word*)t0)[3],lf[362]);
if(C_truep(t10)){
t11=t3;
f_14498(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[3],lf[363]);
if(C_truep(t11)){
t12=t3;
f_14498(t12,t11);}
else{
t12=C_eqp(((C_word*)t0)[3],lf[360]);
if(C_truep(t12)){
t13=t3;
f_14498(t13,t12);}
else{
t13=C_eqp(((C_word*)t0)[3],lf[364]);
t14=t3;
f_14498(t14,(C_truep(t13)?t13:C_eqp(((C_word*)t0)[3],lf[365])));}}}}}}}}}}}}

/* k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14498(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14498,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* support.scm:1256: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(3);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[367]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14510,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_14510(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[374]);
if(C_truep(t4)){
t5=t3;
f_14510(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[372]);
if(C_truep(t5)){
t6=t3;
f_14510(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[366]);
if(C_truep(t6)){
t7=t3;
f_14510(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[3],lf[327]);
t8=t3;
f_14510(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[3],lf[369])));}}}}}}

/* k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14510(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14510,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* support.scm:1258: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(6);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[311]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14522,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_14522(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[394]);
t5=t3;
f_14522(t5,(C_truep(t4)?t4:C_eqp(((C_word*)t0)[3],lf[395])));}}}

/* k14520 in k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14522(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14522,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* support.scm:1260: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(4);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[371]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14534,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_14534(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[370]);
if(C_truep(t4)){
t5=t3;
f_14534(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[368]);
t6=t3;
f_14534(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[3],lf[375])));}}}}

/* k14532 in k14520 in k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14534(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14534,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* support.scm:1262: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(7);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14540,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[4]))){
/* support.scm:1264: lookup-foreign-type */
t3=*((C_word*)lf[303]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_14540(2,av2);}}}}

/* k14538 in k14532 in k14520 in k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_14540,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14544,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1264: g3501 */
t3=t2;
f_14544(t3,((C_word*)t0)[3],t1);}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t2=C_u_i_car(((C_word*)t0)[4]);
t3=C_eqp(t2,lf[347]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14571,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t3)){
t5=t4;
f_14571(t5,t3);}
else{
t5=C_eqp(t2,lf[356]);
if(C_truep(t5)){
t6=t4;
f_14571(t6,t5);}
else{
t6=C_eqp(t2,lf[357]);
if(C_truep(t6)){
t7=t4;
f_14571(t7,t6);}
else{
t7=C_eqp(t2,lf[339]);
if(C_truep(t7)){
t8=t4;
f_14571(t8,t7);}
else{
t8=C_eqp(t2,lf[341]);
if(C_truep(t8)){
t9=t4;
f_14571(t9,t8);}
else{
t9=C_eqp(t2,lf[358]);
if(C_truep(t9)){
t10=t4;
f_14571(t10,t9);}
else{
t10=C_eqp(t2,lf[348]);
if(C_truep(t10)){
t11=t4;
f_14571(t11,t10);}
else{
t11=C_eqp(t2,lf[349]);
t12=t4;
f_14571(t12,(C_truep(t11)?t11:C_eqp(t2,lf[352])));}}}}}}}}
else{
/* support.scm:1273: err */
t2=((C_word*)t0)[5];
f_14470(t2,((C_word*)t0)[3]);}}}

/* g3501 in k14538 in k14532 in k14520 in k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_14544(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_14544,3,t0,t1,t2);}
/* support.scm:1265: next */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_i_vector_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14569 in k14538 in k14532 in k14520 in k14508 in k14496 in k14487 in a14478 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_14571(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_14571,2,t0,t1);}
if(C_truep(t1)){
/* support.scm:1269: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(3);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[353]);
if(C_truep(t2)){
/* support.scm:1270: next */
t3=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cadr(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[354]);
if(C_truep(t3)){
/* support.scm:1271: words->bytes */
t4=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(6);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* support.scm:1272: err */
t4=((C_word*)t0)[6];
f_14470(t4,((C_word*)t0)[2]);}}}}

/* a14935 in chicken.compiler.support#estimate-foreign-result-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14936,c,av);}
/* support.scm:1274: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[413];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_14942,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14954,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15386,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1279: follow-without-loop */
f_7445(t1,t2,t3,t4);}

/* a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14954,c,av);}
a=C_alloc(5);
t4=C_eqp(t2,lf[306]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14964,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;
f_14964(t6,t4);}
else{
t6=C_eqp(t2,lf[309]);
if(C_truep(t6)){
t7=t5;
f_14964(t7,t6);}
else{
t7=C_eqp(t2,lf[373]);
if(C_truep(t7)){
t8=t5;
f_14964(t8,t7);}
else{
t8=C_eqp(t2,lf[410]);
if(C_truep(t8)){
t9=t5;
f_14964(t9,t8);}
else{
t9=C_eqp(t2,lf[335]);
if(C_truep(t9)){
t10=t5;
f_14964(t10,t9);}
else{
t10=C_eqp(t2,lf[307]);
if(C_truep(t10)){
t11=t5;
f_14964(t11,t10);}
else{
t11=C_eqp(t2,lf[396]);
if(C_truep(t11)){
t12=t5;
f_14964(t12,t11);}
else{
t12=C_eqp(t2,lf[374]);
if(C_truep(t12)){
t13=t5;
f_14964(t13,t12);}
else{
t13=C_eqp(t2,lf[366]);
if(C_truep(t13)){
t14=t5;
f_14964(t14,t13);}
else{
t14=C_eqp(t2,lf[397]);
if(C_truep(t14)){
t15=t5;
f_14964(t15,t14);}
else{
t15=C_eqp(t2,lf[398]);
if(C_truep(t15)){
t16=t5;
f_14964(t16,t15);}
else{
t16=C_eqp(t2,lf[339]);
if(C_truep(t16)){
t17=t5;
f_14964(t17,t16);}
else{
t17=C_eqp(t2,lf[341]);
if(C_truep(t17)){
t18=t5;
f_14964(t18,t17);}
else{
t18=C_eqp(t2,lf[367]);
if(C_truep(t18)){
t19=t5;
f_14964(t19,t18);}
else{
t19=C_eqp(t2,lf[372]);
if(C_truep(t19)){
t20=t5;
f_14964(t20,t19);}
else{
t20=C_eqp(t2,lf[311]);
if(C_truep(t20)){
t21=t5;
f_14964(t21,t20);}
else{
t21=C_eqp(t2,lf[342]);
if(C_truep(t21)){
t22=t5;
f_14964(t22,t21);}
else{
t22=C_eqp(t2,lf[346]);
if(C_truep(t22)){
t23=t5;
f_14964(t23,t22);}
else{
t23=C_eqp(t2,lf[314]);
if(C_truep(t23)){
t24=t5;
f_14964(t24,t23);}
else{
t24=C_eqp(t2,lf[319]);
if(C_truep(t24)){
t25=t5;
f_14964(t25,t24);}
else{
t25=C_eqp(t2,lf[399]);
if(C_truep(t25)){
t26=t5;
f_14964(t26,t25);}
else{
t26=C_eqp(t2,lf[400]);
if(C_truep(t26)){
t27=t5;
f_14964(t27,t26);}
else{
t27=C_eqp(t2,lf[327]);
if(C_truep(t27)){
t28=t5;
f_14964(t28,t27);}
else{
t28=C_eqp(t2,lf[369]);
if(C_truep(t28)){
t29=t5;
f_14964(t29,t28);}
else{
t29=C_eqp(t2,lf[362]);
if(C_truep(t29)){
t30=t5;
f_14964(t30,t29);}
else{
t30=C_eqp(t2,lf[363]);
if(C_truep(t30)){
t31=t5;
f_14964(t31,t30);}
else{
t31=C_eqp(t2,lf[360]);
if(C_truep(t31)){
t32=t5;
f_14964(t32,t31);}
else{
t32=C_eqp(t2,lf[345]);
if(C_truep(t32)){
t33=t5;
f_14964(t33,t32);}
else{
t33=C_eqp(t2,lf[361]);
if(C_truep(t33)){
t34=t5;
f_14964(t34,t33);}
else{
t34=C_eqp(t2,lf[359]);
if(C_truep(t34)){
t35=t5;
f_14964(t35,t34);}
else{
t35=C_eqp(t2,lf[364]);
t36=t5;
f_14964(t36,(C_truep(t35)?t35:C_eqp(t2,lf[365])));}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k14962 in a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14964(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14964,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
/* support.scm:1288: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[394]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14976,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t2)){
t4=t3;
f_14976(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[371]);
if(C_truep(t4)){
t5=t3;
f_14976(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[370]);
if(C_truep(t5)){
t6=t3;
f_14976(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[368]);
t7=t3;
f_14976(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[3],lf[375])));}}}}}

/* k14974 in k14962 in a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14976(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14976,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
/* support.scm:1290: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(2);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14982,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[4]))){
/* support.scm:1292: lookup-foreign-type */
t3=*((C_word*)lf[303]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_14982(2,av2);}}}}

/* k14980 in k14974 in k14962 in a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_14982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_14982,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14986,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1292: g3659 */
t3=t2;
f_14986(t3,((C_word*)t0)[3],t1);}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t2=C_u_i_car(((C_word*)t0)[4]);
t3=C_eqp(t2,lf[347]);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15013,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t3)){
t5=t4;
f_15013(t5,t3);}
else{
t5=C_eqp(t2,lf[356]);
if(C_truep(t5)){
t6=t4;
f_15013(t6,t5);}
else{
t6=C_eqp(t2,lf[357]);
if(C_truep(t6)){
t7=t4;
f_15013(t7,t6);}
else{
t7=C_eqp(t2,lf[339]);
if(C_truep(t7)){
t8=t4;
f_15013(t8,t7);}
else{
t8=C_eqp(t2,lf[341]);
if(C_truep(t8)){
t9=t4;
f_15013(t9,t8);}
else{
t9=C_eqp(t2,lf[358]);
if(C_truep(t9)){
t10=t4;
f_15013(t10,t9);}
else{
t10=C_eqp(t2,lf[314]);
if(C_truep(t10)){
t11=t4;
f_15013(t11,t10);}
else{
t11=C_eqp(t2,lf[319]);
t12=t4;
f_15013(t12,(C_truep(t11)?t11:C_eqp(t2,lf[354])));}}}}}}}}
else{
/* support.scm:1278: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[415];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}}

/* g3659 in k14980 in k14974 in k14962 in a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_14986(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_14986,3,t0,t1,t2);}
/* support.scm:1293: next */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_i_vector_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15011 in k14980 in k14974 in k14962 in a14953 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15013(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_15013,2,t0,t1);}
if(C_truep(t1)){
/* support.scm:1298: words->bytes */
t2=*((C_word*)lf[68]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[353]);
if(C_truep(t2)){
/* support.scm:1299: next */
t3=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cadr(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
/* support.scm:1278: quit-compiling */
t3=*((C_word*)lf[37]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[415];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}}

/* a15385 in chicken.compiler.support#estimate-foreign-result-location-size in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15386,c,av);}
/* support.scm:1302: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[416];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#finish-foreign-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_15392,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15396,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1308: chicken.syntax#strip-syntax */
t5=*((C_word*)lf[430]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k15394 in chicken.compiler.support#finish-foreign-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_15396,c,av);}
a=C_alloc(21);
t2=C_eqp(t1,lf[342]);
t3=(C_truep(t2)?t2:C_eqp(t1,lf[362]));
if(C_truep(t3)){
t4=C_a_i_list(&a,2,lf[316],C_fix(0));
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_list(&a,3,lf[418],((C_word*)t0)[3],t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_eqp(t1,lf[345]);
if(C_truep(t4)){
t5=C_a_i_list(&a,2,lf[316],C_fix(0));
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_list(&a,3,lf[419],((C_word*)t0)[3],t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_eqp(t1,lf[361]);
t6=(C_truep(t5)?t5:C_eqp(t1,lf[363]));
if(C_truep(t6)){
t7=C_a_i_list(&a,2,lf[316],C_fix(0));
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_list(&a,3,lf[420],((C_word*)t0)[3],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(t1,lf[359]);
t8=(C_truep(t7)?t7:C_eqp(t1,lf[360]));
if(C_truep(t8)){
t9=C_a_i_list(&a,2,lf[316],C_fix(0));
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_a_i_list(&a,3,lf[421],((C_word*)t0)[3],t9);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=C_eqp(t1,lf[346]);
if(C_truep(t9)){
t10=C_a_i_list(&a,2,lf[316],C_fix(0));
t11=C_a_i_list(&a,3,lf[418],((C_word*)t0)[3],t10);
t12=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_a_i_list(&a,2,lf[422],t11);
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t10=C_eqp(t1,lf[364]);
if(C_truep(t10)){
t11=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t12=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_a_i_list(&a,3,lf[423],((C_word*)t0)[3],t11);
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t11=C_eqp(t1,lf[365]);
if(C_truep(t11)){
t12=C_a_i_list(&a,2,lf[316],C_SCHEME_FALSE);
t13=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t13;
av2[1]=C_a_i_list(&a,3,lf[424],((C_word*)t0)[3],t12);
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
if(C_truep(C_i_listp(t1))){
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15512,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t13=C_i_car(t1);
t14=C_eqp(t13,lf[353]);
if(C_truep(t14)){
t15=C_i_length(t1);
t16=C_eqp(C_fix(2),t15);
t17=t12;
f_15512(t17,(C_truep(t16)?C_u_i_memq(C_i_cadr(t1),lf[429]):C_SCHEME_FALSE));}
else{
t15=t12;
f_15512(t15,C_SCHEME_FALSE);}}
else{
t12=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t12;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}}}}}}}}

/* k15510 in k15394 in chicken.compiler.support#finish-foreign-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15512(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_15512,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
/* support.scm:1325: finish-foreign-result */
t2=*((C_word*)lf[417]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cadr(((C_word*)t0)[3]);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_i_length(((C_word*)t0)[3]);
t3=C_eqp(C_fix(3),t2);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[3]);
t5=C_eqp(t4,lf[348]);
t6=(C_truep(t5)?t5:C_eqp(t4,lf[349]));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15540,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1329: chicken.base#gensym */
t8=*((C_word*)lf[97]+1);{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(t4,lf[352]);
if(C_truep(t7)){
t8=C_i_caddr(((C_word*)t0)[3]);
t9=C_a_i_list(&a,2,lf[316],lf[350]);
t10=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_a_i_list(&a,4,lf[427],t8,t9,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}
else{
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k15538 in k15510 in k15394 in chicken.compiler.support#finish-foreign-result in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(60,c,1)))){
C_save_and_reclaim((void *)f_15540,c,av);}
a=C_alloc(60);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[425],t1);
t5=C_a_i_list(&a,2,lf[426],t4);
t6=C_i_caddr(((C_word*)t0)[3]);
t7=C_a_i_list(&a,2,lf[316],lf[350]);
t8=C_a_i_list(&a,4,lf[427],t6,t7,t1);
t9=C_a_i_list(&a,4,lf[428],t1,t5,t8);
t10=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_a_i_list(&a,3,lf[96],t3,t9);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15660,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15667,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=C_eqp(t3,lf[433]);
if(C_truep(t5)){
/* support.scm:1348: foreign-type-argument-converter */
f_14377(t4,t2);}
else{
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_15667(2,av2);}}}

/* k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15667(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15667,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15670,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_15670(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[458]);
if(C_truep(t3)){
/* support.scm:1349: foreign-type-result-converter */
f_14356(t2,((C_word*)t0)[4]);}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_15670(2,av2);}}}}

/* k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15670(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15670,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[170];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15673,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1353: final-foreign-type */
t3=*((C_word*)lf[406]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_15673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_15673,c,av);}
a=C_alloc(6);
t2=C_eqp(t1,lf[411]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[432];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_eqp(t1,lf[306]);
t4=(C_truep(t3)?t3:C_eqp(t1,lf[307]));
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=lf[306];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(t1,lf[309]);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15697,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t5)){
t7=t6;
f_15697(t7,t5);}
else{
t7=C_eqp(t1,lf[396]);
if(C_truep(t7)){
t8=t6;
f_15697(t8,t7);}
else{
t8=C_eqp(t1,lf[373]);
if(C_truep(t8)){
t9=t6;
f_15697(t9,t8);}
else{
t9=C_eqp(t1,lf[335]);
if(C_truep(t9)){
t10=t6;
f_15697(t10,t9);}
else{
t10=C_eqp(t1,lf[397]);
if(C_truep(t10)){
t11=t6;
f_15697(t11,t10);}
else{
t11=C_eqp(t1,lf[398]);
if(C_truep(t11)){
t12=t6;
f_15697(t12,t11);}
else{
t12=C_eqp(t1,lf[399]);
t13=t6;
f_15697(t13,(C_truep(t12)?t12:C_eqp(t1,lf[400])));}}}}}}}}}

/* k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15697(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15697,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[164];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[311]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[3],lf[394]));
if(C_truep(t3)){
t4=C_eqp(((C_word*)t0)[4],lf[433]);
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?lf[395]:lf[311]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[314]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[3],lf[319]));
if(C_truep(t5)){
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[170];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[3],lf[313]);
if(C_truep(t6)){
t7=C_eqp(((C_word*)t0)[4],lf[433]);
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?lf[434]:lf[313]);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[3],lf[320]);
if(C_truep(t7)){
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[313];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(((C_word*)t0)[3],lf[321]);
if(C_truep(t8)){
t9=C_eqp(((C_word*)t0)[4],lf[433]);
t10=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t10;
av2[1]=(C_truep(t9)?lf[435]:lf[321]);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=C_eqp(((C_word*)t0)[3],lf[323]);
if(C_truep(t9)){
t10=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t10;
av2[1]=lf[321];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(((C_word*)t0)[3],lf[324]);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15772,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t10)){
t12=t11;
f_15772(t12,t10);}
else{
t12=C_eqp(((C_word*)t0)[3],lf[385]);
if(C_truep(t12)){
t13=t11;
f_15772(t13,t12);}
else{
t13=C_eqp(((C_word*)t0)[3],lf[386]);
if(C_truep(t13)){
t14=t11;
f_15772(t14,t13);}
else{
t14=C_eqp(((C_word*)t0)[3],lf[387]);
if(C_truep(t14)){
t15=t11;
f_15772(t15,t14);}
else{
t15=C_eqp(((C_word*)t0)[3],lf[388]);
if(C_truep(t15)){
t16=t11;
f_15772(t16,t15);}
else{
t16=C_eqp(((C_word*)t0)[3],lf[389]);
if(C_truep(t16)){
t17=t11;
f_15772(t17,t16);}
else{
t17=C_eqp(((C_word*)t0)[3],lf[390]);
if(C_truep(t17)){
t18=t11;
f_15772(t18,t17);}
else{
t18=C_eqp(((C_word*)t0)[3],lf[391]);
if(C_truep(t18)){
t19=t11;
f_15772(t19,t18);}
else{
t19=C_eqp(((C_word*)t0)[3],lf[392]);
t20=t11;
f_15772(t20,(C_truep(t19)?t19:C_eqp(((C_word*)t0)[3],lf[393])));}}}}}}}}}}}}}}}}

/* k15770 in k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15772(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_15772,2,t0,t1);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_eqp(((C_word*)t0)[2],lf[433]);
if(C_truep(t2)){
t3=C_a_i_list(&a,2,lf[436],((C_word*)t0)[3]);
t4=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list(&a,3,lf[437],lf[438],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[436],((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[325]);
if(C_truep(t2)){
t3=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[439];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[377]);
if(C_truep(t3)){
t4=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[440];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[376]);
if(C_truep(t4)){
t5=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[441];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[3],lf[378]);
if(C_truep(t5)){
t6=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[442];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[3],lf[379]);
if(C_truep(t6)){
t7=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t7;
av2[1]=lf[443];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t0)[3],lf[380]);
if(C_truep(t7)){
t8=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[444];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(((C_word*)t0)[3],lf[381]);
if(C_truep(t8)){
t9=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[445];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(((C_word*)t0)[3],lf[382]);
if(C_truep(t9)){
t10=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t10;
av2[1]=lf[446];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(((C_word*)t0)[3],lf[383]);
if(C_truep(t10)){
t11=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t11;
av2[1]=lf[447];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(((C_word*)t0)[3],lf[384]);
if(C_truep(t11)){
t12=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t12;
av2[1]=lf[448];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=C_eqp(((C_word*)t0)[3],lf[372]);
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15859,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t12)){
t14=t13;
f_15859(t14,t12);}
else{
t14=C_eqp(((C_word*)t0)[3],lf[374]);
if(C_truep(t14)){
t15=t13;
f_15859(t15,t14);}
else{
t15=C_eqp(((C_word*)t0)[3],lf[368]);
if(C_truep(t15)){
t16=t13;
f_15859(t16,t15);}
else{
t16=C_eqp(((C_word*)t0)[3],lf[375]);
if(C_truep(t16)){
t17=t13;
f_15859(t17,t16);}
else{
t17=C_eqp(((C_word*)t0)[3],lf[327]);
if(C_truep(t17)){
t18=t13;
f_15859(t18,t17);}
else{
t18=C_eqp(((C_word*)t0)[3],lf[369]);
if(C_truep(t18)){
t19=t13;
f_15859(t19,t18);}
else{
t19=C_eqp(((C_word*)t0)[3],lf[371]);
if(C_truep(t19)){
t20=t13;
f_15859(t20,t19);}
else{
t20=C_eqp(((C_word*)t0)[3],lf[370]);
t21=t13;
f_15859(t21,(C_truep(t20)?t20:C_eqp(((C_word*)t0)[3],lf[366])));}}}}}}}}}}}}}}}}}}}

/* k15857 in k15770 in k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15859(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15859,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[372];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[339]);
if(C_truep(t2)){
t3=C_eqp(lf[433],((C_word*)t0)[4]);
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=(C_truep(t3)?lf[449]:lf[450]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[341]);
if(C_truep(t3)){
t4=C_eqp(lf[433],((C_word*)t0)[4]);
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?lf[451]:lf[357]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[342]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15892,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t4)){
t6=t5;
f_15892(t6,t4);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[361]);
if(C_truep(t6)){
t7=t5;
f_15892(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[3],lf[362]);
t8=t5;
f_15892(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[3],lf[363])));}}}}}}

/* k15890 in k15857 in k15770 in k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_15892(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15892,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[452];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[364]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[3],lf[365]));
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[453];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[345]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15910,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t4)){
t6=t5;
f_15910(t6,t4);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[359]);
t7=t5;
f_15910(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[3],lf[360])));}}}}

/* k15908 in k15890 in k15857 in k15770 in k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_fcall f_15910(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15910,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[454];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[346]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[346];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t3=C_u_i_car(((C_word*)t0)[4]);
t4=C_eqp(t3,lf[347]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15932,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t4)){
t6=t5;
f_15932(t6,t4);}
else{
t6=C_eqp(t3,lf[357]);
if(C_truep(t6)){
t7=t5;
f_15932(t7,t6);}
else{
t7=C_eqp(t3,lf[358]);
t8=t5;
f_15932(t8,(C_truep(t7)?t7:C_eqp(t3,lf[339])));}}}
else{
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[170];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}}

/* k15930 in k15908 in k15890 in k15857 in k15770 in k15695 in k15671 in k15668 in k15665 in chicken.compiler.support#foreign-type->scrutiny-type in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_15932(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_15932,2,t0,t1);}
if(C_truep(t1)){
t2=C_eqp(lf[433],((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=(C_truep(t2)?lf[455]:lf[456]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[353]);
if(C_truep(t2)){
/* support.scm:1412: foreign-type->scrutiny-type */
t3=*((C_word*)lf[431]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=C_i_cadr(((C_word*)t0)[5]);
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=C_eqp(((C_word*)t0)[4],lf[354]);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[372];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[356]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[4],lf[341]));
if(C_truep(t5)){
t6=C_eqp(lf[433],((C_word*)t0)[2]);
t7=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t7;
av2[1]=(C_truep(t6)?lf[457]:lf[357]);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[170];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}}

/* chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_16314,c,av);}
a=C_alloc(13);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16318,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16320,a[2]=t8,a[3]=t3,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_16320(t10,t6,t2);}

/* k16316 in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16318,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16320(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_16320,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_slot(t2,C_fix(3));
t4=C_slot(t2,C_fix(1));
t5=C_eqp(t4,lf[154]);
t6=(C_truep(t5)?t5:C_eqp(t4,lf[124]));
if(C_truep(t6)){
t7=C_slot(t2,C_fix(2));
t8=C_i_car(t7);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16352,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t10=C_i_memq(t8,((C_word*)t0)[3]);
t11=(C_truep(t10)?C_i_not(C_i_memq(t8,((C_word*)((C_word*)t0)[4])[1])):C_SCHEME_FALSE);
if(C_truep(t11)){
t12=C_a_i_cons(&a,2,t8,((C_word*)((C_word*)t0)[4])[1]);
t13=C_mutate(((C_word *)((C_word*)t0)[4])+1,t12);
t14=t9;
f_16352(t14,t13);}
else{
t12=t9;
f_16352(t12,C_SCHEME_UNDEFINED);}}
else{
t7=C_eqp(t4,lf[85]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16416,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t7)){
t9=t8;
f_16416(t9,t7);}
else{
t9=C_eqp(t4,lf[159]);
t10=t8;
f_16416(t10,(C_truep(t9)?t9:C_eqp(t4,lf[171])));}}}

/* k16350 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16352(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_16352,2,t0,t1);}
a=C_alloc(6);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[44]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16360,a[2]=t4,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_16360(t6,((C_word*)t0)[4],((C_word*)t0)[2]);}

/* for-each-loop3945 in k16350 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16360(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16360,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16370,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1433: g3946 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_16320(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16368 in for-each-loop3945 in k16350 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16370,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16360(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k16414 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16416(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_16416,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_check_list_2(((C_word*)t0)[3],lf[44]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16424,a[2]=t4,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_16424(t6,((C_word*)t0)[2],((C_word*)t0)[3]);}}

/* for-each-loop3970 in k16414 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16424(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16424,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16434,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1435: g3971 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_16320(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16432 in for-each-loop3970 in k16414 in walk in chicken.compiler.support#scan-used-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16434,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16424(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16469(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_16469,c,av);}
a=C_alloc(23);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_END_OF_LIST;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16472,a[2]=t5,a[3]=t7,a[4]=t3,a[5]=t9,a[6]=t11,tmp=(C_word)a,a+=7,tmp));
t13=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16656,a[2]=t9,tmp=(C_word)a,a+=3,tmp));
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16694,a[2]=t1,a[3]=t5,a[4]=t7,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1473: walk */
t15=((C_word*)t9)[1];
f_16472(t15,t14,t2,C_SCHEME_END_OF_LIST);}

/* walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16472(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_16472,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=C_slot(t2,C_fix(3));
t5=C_slot(t2,C_fix(2));
t6=C_slot(t2,C_fix(1));
t7=C_eqp(t6,lf[85]);
t8=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_16506,a[2]=t1,a[3]=t6,a[4]=t5,a[5]=t3,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=t4,a[11]=((C_word*)t0)[6],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t7)){
t9=t8;
f_16506(t9,t7);}
else{
t9=C_eqp(t6,lf[159]);
if(C_truep(t9)){
t10=t8;
f_16506(t10,t9);}
else{
t10=C_eqp(t6,lf[171]);
if(C_truep(t10)){
t11=t8;
f_16506(t11,t10);}
else{
t11=C_eqp(t6,lf[174]);
t12=t8;
f_16506(t12,(C_truep(t11)?t11:C_eqp(t6,lf[184])));}}}}

/* k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16506(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_16506,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[154]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[4]);
if(C_truep(C_i_memq(t3,((C_word*)t0)[5]))){
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16525,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[7],a[5]=t3,a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* support.scm:1453: lset-adjoin/eq? */
f_6202(t4,((C_word*)((C_word*)t0)[6])[1],C_a_i_list(&a,1,t3));}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[124]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[4]);
if(C_truep(C_i_memq(t4,((C_word*)t0)[5]))){
/* support.scm:1459: walk */
t5=((C_word*)((C_word*)t0)[9])[1];
f_16472(t5,((C_word*)t0)[2],C_i_car(((C_word*)t0)[10]),((C_word*)t0)[5]);}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16561,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* support.scm:1458: lset-adjoin/eq? */
f_6202(t5,((C_word*)((C_word*)t0)[6])[1],C_a_i_list(&a,1,t4));}}
else{
t4=C_eqp(((C_word*)t0)[3],lf[96]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16570,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* support.scm:1461: walk */
t6=((C_word*)((C_word*)t0)[9])[1];
f_16472(t6,t5,C_i_car(((C_word*)t0)[10]),((C_word*)t0)[5]);}
else{
t5=C_eqp(((C_word*)t0)[3],lf[118]);
if(C_truep(t5)){
t6=C_i_caddr(((C_word*)t0)[4]);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16600,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1464: ##sys#decompose-lambda-list */
t8=*((C_word*)lf[225]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=((C_word*)t0)[2];
av2[2]=t6;
av2[3]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* support.scm:1468: walkeach */
t6=((C_word*)((C_word*)t0)[11])[1];
f_16656(t6,((C_word*)t0)[2],((C_word*)t0)[10],((C_word*)t0)[5]);}}}}}}

/* k16523 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16525,c,av);}
a=C_alloc(5);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16531,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1454: variable-visible? */
t4=*((C_word*)lf[260]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k16529 in k16523 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_16531,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16535,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1455: lset-adjoin/eq? */
f_6202(t2,((C_word*)((C_word*)t0)[3])[1],C_a_i_list(&a,1,((C_word*)t0)[4]));}}

/* k16533 in k16529 in k16523 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16535,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k16559 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16561,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* support.scm:1459: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_16472(t3,((C_word*)t0)[4],C_i_car(((C_word*)t0)[5]),((C_word*)t0)[6]);}

/* k16568 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16570,c,av);}
a=C_alloc(5);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16581,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1462: scheme#append */
t4=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k16579 in k16568 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16581,c,av);}
/* support.scm:1462: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16472(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* a16599 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16600,c,av);}
a=C_alloc(5);
t5=C_i_car(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16612,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1467: scheme#append */
t7=*((C_word*)lf[58]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k16610 in a16599 in k16504 in walk in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16612,c,av);}
/* support.scm:1467: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16472(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* walkeach in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16656(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_16656,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16658,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_i_check_list_2(t2,lf[44]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16670,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_16670(t9,t1,t2);}

/* g4038 in walkeach in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16658(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16658,3,t0,t1,t2);}
/* support.scm:1471: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_16472(t3,t1,t2,((C_word*)t0)[3]);}

/* for-each-loop4037 in walkeach in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16670(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16670,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16680,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1471: g4038 */
t4=((C_word*)t0)[3];
f_16658(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16678 in for-each-loop4037 in walkeach in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16680,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16670(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k16692 in chicken.compiler.support#scan-free-variables in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16694,c,av);}
/* support.scm:1474: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
C_values(4,av2);}}

/* chicken.compiler.support#chop-separator in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16699,c,av);}
a=C_alloc(5);
t3=C_i_string_length(t2);
t4=C_a_i_fixnum_difference(&a,2,t3,C_fix(1));
if(C_truep(C_i_integer_greaterp(t4,C_fix(0)))){
t5=C_i_string_ref(t2,t4);
if(C_truep((C_truep(C_eqp(t5,C_make_character(92)))?C_SCHEME_TRUE:(C_truep(C_eqp(t5,C_make_character(47)))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
/* support.scm:1483: scheme#substring */
t6=*((C_word*)lf[462]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(0);
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* chicken.compiler.support#make-block-variable-literal in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16723,c,av);}
a=C_alloc(3);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record2(&a,2,lf[464],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#block-variable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16729(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16729,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[464]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#block-variable-literal-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16735(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16735,c,av);}
t3=C_i_check_structure_2(t2,lf[464],lf[467]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_16744,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16752,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1499: chicken.base#open-output-string */
t4=*((C_word*)lf[334]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_16752,c,av);}
a=C_alloc(8);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[328]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16758,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16779,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_nullp(((C_word*)t0)[3]))){
/* support.scm:1500: chicken.base#gensym */
t5=*((C_word*)lf[97]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* support.scm:1499: ##sys#print */
t5=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_i_car(((C_word*)t0)[3]);
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16758,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16761,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1499: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(45);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16759 in k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16761,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16764,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16775,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1501: chicken.time#current-seconds */
t4=*((C_word*)lf[469]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k16762 in k16759 in k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_16764,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16767,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1499: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_random_fixnum(C_fix(1000));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16765 in k16762 in k16759 in k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16767,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16770,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1499: chicken.base#get-output-string */
t3=*((C_word*)lf[331]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16768 in k16765 in k16762 in k16759 in k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16770,c,av);}
/* support.scm:1498: scheme#string->symbol */
t2=*((C_word*)lf[468]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16773 in k16759 in k16756 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16775,c,av);}
/* support.scm:1499: ##sys#print */
t2=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k16777 in k16750 in chicken.compiler.support#make-random-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16779,c,av);}
/* support.scm:1499: ##sys#print */
t2=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.support#clear-real-name-table! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_16791,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16796,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1517: scheme#make-vector */
t3=*((C_word*)lf[301]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(997);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16794 in chicken.compiler.support#clear-real-name-table! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16796,c,av);}
t2=C_mutate(&lf[470] /* (set! chicken.compiler.support#real-name-table ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#set-real-name! in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16798,c,av);}
/* support.scm:1520: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[470];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* chicken.compiler.support#get-real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16804,c,av);}
/* support.scm:1525: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[128]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[470];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +8,c,3)))){
C_save_and_reclaim((void*)f_16811,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+8);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16814,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16830,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* support.scm:1537: resolve */
f_16814(t5,t2);}

/* resolve in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16814(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_16814,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16818,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1532: chicken.internal#hash-table-ref */
t4=*((C_word*)lf[128]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[470];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k16816 in resolve in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16818(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16818,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16824,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1534: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[128]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[470];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16822 in k16816 in resolve in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16824,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_16830,c,av);}
a=C_alloc(6);
if(C_truep(C_i_not(t1))){
/* support.scm:1538: ##sys#symbol->string */
t2=*((C_word*)lf[180]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t2=C_u_i_car(((C_word*)t0)[4]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16926,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1541: ##sys#symbol->string */
t4=*((C_word*)lf[180]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* support.scm:1555: ##sys#symbol->string */
t2=*((C_word*)lf[180]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}}

/* k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_16857,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16859,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_16859(t5,((C_word*)t0)[4],((C_word*)t0)[5],C_fix(0),t1);}

/* loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_16859(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_16859,5,t0,t1,t2,t3,t4);}
a=C_alloc(8);
if(C_truep(C_i_greaterp(t3,C_fix(20)))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16873,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=C_a_i_cons(&a,2,lf[476],t2);
/* support.scm:1546: scheme#reverse */
t7=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16883,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* support.scm:1548: resolve */
f_16814(t5,t4);}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16922,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1554: scheme#reverse */
t6=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}

/* k16871 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16873,c,av);}
/* support.scm:1546: chicken.string#string-intersperse */
t2=*((C_word*)lf[474]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[475];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k16881 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16883,c,av);}
a=C_alloc(8);
t2=C_eqp(t1,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16896,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1550: scheme#reverse */
t4=*((C_word*)lf[80]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16915,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
/* support.scm:1551: scheme#symbol->string */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k16894 in k16881 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16896,c,av);}
/* support.scm:1550: chicken.string#string-intersperse */
t2=*((C_word*)lf[474]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[477];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k16909 in k16913 in k16881 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16911,c,av);}
/* support.scm:1551: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16859(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k16913 in k16881 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16915(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_16915,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
t3=C_fixnum_plus(((C_word*)t0)[3],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16911,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:1553: db-get */
t5=*((C_word*)lf[127]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
av2[4]=lf[478];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k16920 in loop in k16855 in k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16922,c,av);}
/* support.scm:1554: chicken.string#string-intersperse */
t2=*((C_word*)lf[474]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[479];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k16924 in k16828 in chicken.compiler.support#real-name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_16926,c,av);}
a=C_alloc(9);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16857,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:1543: db-get */
t4=*((C_word*)lf[127]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[478];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* chicken.compiler.support#real-name2 in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16931,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16935,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1558: chicken.internal#hash-table-ref */
t5=*((C_word*)lf[128]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[470];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k16933 in chicken.compiler.support#real-name2 in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16935,c,av);}
if(C_truep(t1)){
/* support.scm:1559: real-name */
t2=*((C_word*)lf[181]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#display-real-name-table in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_16943,c,av);}
a=C_alloc(2);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16949,tmp=(C_word)a,a+=2,tmp);
/* support.scm:1562: chicken.internal#hash-table-for-each */
t3=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[470];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a16948 in chicken.compiler.support#display-real-name-table in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16949,c,av);}
a=C_alloc(5);
t4=*((C_word*)lf[24]+1);
t5=*((C_word*)lf[24]+1);
t6=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16956,a[2]=t1,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1564: ##sys#print */
t8=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}

/* k16954 in a16948 in chicken.compiler.support#display-real-name-table in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16956,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16959,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1564: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(9);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16957 in k16954 in a16948 in chicken.compiler.support#display-real-name-table in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_16959,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16962,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1564: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16960 in k16957 in k16954 in a16948 in chicken.compiler.support#display-real-name-table in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16962(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16962,c,av);}
/* support.scm:1564: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#source-info->string in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16967(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_16967,c,av);}
a=C_alloc(13);
if(C_truep(C_i_listp(t2))){
t3=C_i_car(t2);
t4=C_i_cadr(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16987,a[2]=t1,a[3]=t3,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16991,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=C_i_string_length(t3);
t8=C_a_i_fixnum_difference(&a,2,C_fix(4),t7);
/* support.scm:1571: scheme#max */
t9=*((C_word*)lf[487]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t9;
av2[1]=t6;
av2[2]=C_fix(0);
av2[3]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
/* support.scm:1572: chicken.string#->string */
t3=*((C_word*)lf[66]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k16985 in chicken.compiler.support#source-info->string in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_16987,c,av);}
/* support.scm:1571: chicken.string#conc */
t2=*((C_word*)lf[483]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[484];
av2[4]=t1;
av2[5]=lf[485];
av2[6]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k16989 in chicken.compiler.support#source-info->string in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_16991(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16991,c,av);}
/* support.scm:1571: scheme#make-string */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_make_character(32);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#source-info->name in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17002,c,av);}
if(C_truep(C_i_listp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* support.scm:1575: chicken.string#->string */
t3=*((C_word*)lf[66]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* chicken.compiler.support#source-info->line in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17017,c,av);}
t3=C_i_listp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_i_car(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#call-info in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_17029,c,av);}
a=C_alloc(4);
t4=C_i_cdr(t2);
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_i_cadr(t2):C_SCHEME_FALSE);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17036,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t6)){
if(C_truep(C_i_listp(t6))){
t8=C_i_car(t6);
t9=C_i_cadr(t6);
/* support.scm:1585: chicken.string#conc */
t10=*((C_word*)lf[483]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t10;
av2[1]=t7;
av2[2]=lf[491];
av2[3]=t8;
av2[4]=lf[492];
av2[5]=t3;
((C_proc)(void*)(*((C_word*)t10+1)))(6,av2);}}
else{
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}

/* k17034 in chicken.compiler.support#call-info in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17036,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17066(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_17066,c,av);}
a=C_alloc(17);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_i_check_list_2(t3,lf[125]);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17090,a[2]=t2,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17262,a[2]=t7,a[3]=t12,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t14=((C_word*)t12)[1];
f_17262(t14,t10,t3);}

/* k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_17090,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(t1,lf[125]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17226,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17228,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_17228(t11,t7,t1);}

/* k17103 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17105,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17108,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1597: g4235 */
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17106 in k17103 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_17108,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17114,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1598: chicken.condition#condition? */
t3=*((C_word*)lf[497]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17112 in k17106 in k17103 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17114,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
/* support.scm:1598: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17123,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_i_length(((C_word*)t0)[4]);
t4=C_eqp(C_fix(1),t3);
if(C_truep(t4)){
/* support.scm:1600: encodeable-literal? */
t5=lf[496];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[4]);
f_17395(3,av2);}}
else{
t5=t2;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_17123(2,av2);}}}}

/* k17121 in k17112 in k17106 in k17103 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17123,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17126,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1601: debugging */
t3=*((C_word*)lf[22]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[190];
av2[3]=lf[494];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=C_eqp(C_fix(1),C_u_i_length(((C_word*)t0)[4]));
if(C_truep(t2)){
/* support.scm:1604: k */
t3=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* support.scm:1606: bomb */
t3=*((C_word*)lf[13]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[495];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}}

/* k17124 in k17121 in k17112 in k17106 in k17103 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17126,c,av);}
/* support.scm:1602: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_SCHEME_TRUE;
av2[3]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17165,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17171,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17180,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1597: chicken.condition#with-exception-handler */
t5=*((C_word*)lf[106]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a17170 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17171,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17177,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1597: k4232 */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a17176 in a17170 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17177(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17177,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a17179 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17180,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17186,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17198,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1597: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a17185 in a17179 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17186,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17192,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1597: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=*((C_word*)lf[498]+1);
C_call_with_values(4,av2);}}

/* a17191 in a17185 in a17179 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17192,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
C_apply(4,av2);}}

/* a17197 in a17179 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +3,c,2)))){
C_save_and_reclaim((void*)f_17198,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+3);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17204,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1597: k4232 */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a17203 in a17197 in a17179 in a17164 in k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17204,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k17224 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_17226,c,av);}
a=C_alloc(12);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_slot(((C_word*)t0)[2],C_fix(0));
if(C_truep(C_i_closurep(t3))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17105,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17165,a[2]=t3,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1597: scheme#call-with-current-continuation */
t6=*((C_word*)lf[107]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
/* support.scm:1607: bomb */
t4=*((C_word*)lf[13]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[499];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* map-loop4202 in k17088 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17228(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_17228,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_a_i_list(&a,2,lf[85],t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop4171 in chicken.compiler.support#constant-form-eval in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17262(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_17262,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_slot(t3,C_fix(2));
t5=C_i_car(t4);
t6=C_a_i_cons(&a,2,t5,C_SCHEME_END_OF_LIST);
t7=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t6);
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,t6);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17296(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_17296,c,av);}
a=C_alloc(14);
t5=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17299,tmp=(C_word)a,a+=2,tmp);
t6=C_i_car(t3);
t7=C_slot(t6,C_fix(1));
t8=C_eqp(lf[154],t7);
if(C_truep(t8)){
t9=C_slot(C_u_i_car(t3),C_fix(2));
t10=C_i_car(t9);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17327,a[2]=t3,a[3]=t4,a[4]=t1,a[5]=t10,tmp=(C_word)a,a+=6,tmp);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17349,a[2]=t11,a[3]=t5,a[4]=t3,a[5]=t10,tmp=(C_word)a,a+=6,tmp);
/* tweaks.scm:51: ##sys#get */
t13=*((C_word*)lf[182]+1);{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t10;
av2[3]=lf[503];
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}
else{
/* support.scm:1619: k */
t9=t4;{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}}

/* constant-node? in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17299(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17299,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(lf[85],C_slot(t2,C_fix(1)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17325 in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17327(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17327,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17334,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1617: constant-form-eval */
t4=*((C_word*)lf[493]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* support.scm:1618: k */
t2=((C_word*)t0)[3];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* a17333 in k17325 in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17334(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17334,c,av);}
/* support.scm:1617: k */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k17347 in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_17349,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17355,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1614: foldable? */
t3=*((C_word*)lf[502]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_17327(2,av2);}}}

/* k17353 in k17347 in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17355,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17358,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
if(C_truep(t1)){
/* support.scm:1616: every */
f_5743(((C_word*)t0)[2],((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_17327(2,av2);}}}
else{
/* support.scm:1615: predicate? */
t3=*((C_word*)lf[501]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k17356 in k17353 in k17347 in chicken.compiler.support#maybe-constant-fold-call in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17358(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17358,c,av);}
if(C_truep(t1)){
/* support.scm:1616: every */
f_5743(((C_word*)t0)[2],((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_17327(2,av2);}}}

/* chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17395,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17410,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1629: immediate? */
t4=*((C_word*)lf[89]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17410(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_17410,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_exact_integerp(((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17420,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17435,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1634: scheme#call-with-current-continuation */
t4=*((C_word*)lf[107]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_i_flonump(((C_word*)t0)[3]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
t3=C_slot(((C_word*)t0)[3],C_fix(1));
t4=C_i_string_length(t3);
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_fixnum_less_or_equal_p(C_i_integer_length(t4),C_fix(24));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_byteblockp(((C_word*)t0)[3]))){
t3=stub4284(C_SCHEME_UNDEFINED,((C_word*)t0)[3]);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_less_or_equal_p(C_i_integer_length(t3),C_fix(24));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=stub4284(C_SCHEME_UNDEFINED,((C_word*)t0)[3]);
if(C_truep(C_fixnum_less_or_equal_p(C_i_integer_length(t3),C_fix(24)))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17524,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17526,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6391,a[2]=t3,a[3]=t5,a[4]=t7,tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_6391(t9,t4,C_fix(0));}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}}}}}

/* k17418 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17420,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17423,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1634: g4293 */
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17421 in k17418 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17423,c,av);}
if(C_truep(t1)){
t2=C_i_string_length(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fixnum_less_or_equal_p(C_i_integer_length(t2),C_fix(24));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17435(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17435,c,av);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17441,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17450,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1634: chicken.condition#with-exception-handler */
t5=*((C_word*)lf[106]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a17440 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17441(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,2)))){
C_save_and_reclaim((void *)f_17441,c,av);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17447,tmp=(C_word)a,a+=2,tmp);
/* support.scm:1634: k4290 */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a17446 in a17440 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17447,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a17449 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_17450,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17452,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17459,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17476,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* tmp15335 */
t5=t2;
f_17452(t5,t4);}

/* tmp15335 in a17449 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17452(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_17452,2,t0,t1);}
/* ##sys#number->string */
t2=*((C_word*)lf[504]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=C_fix(16);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* tmp25336 in a17449 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17459(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_17459,3,t0,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17465,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1634: k4290 */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a17464 in tmp25336 in a17449 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17465,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k17474 in a17449 in a17434 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17476,c,av);}
a=C_alloc(3);
/* tmp25336 */
t2=((C_word*)t0)[2];
f_17459(t2,((C_word*)t0)[3],C_a_i_list(&a,1,t1));}

/* k17522 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17524(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17524,c,av);}
/* support.scm:1645: every */
f_5743(((C_word*)t0)[2],lf[496],t1);}

/* a17525 in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_17526(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_slot(((C_word*)t0)[2],t1));}

/* chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_17532,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17536,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17541,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_17541(t7,t3,C_fix(0),t2);}

/* k17534 in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17536(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17536,c,av);}
/* support.scm:1670: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17541(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_17541,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=C_slot(t3,C_fix(1));
t5=C_slot(t3,C_fix(2));
t6=C_slot(t3,C_fix(3));
t7=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17569,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t6,a[5]=t3,a[6]=t1,a[7]=t5,a[8]=t4,tmp=(C_word)a,a+=9,tmp);
/* support.scm:1658: scheme#make-string */
t8=*((C_word*)lf[486]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
av2[3]=C_make_character(32);
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}

/* k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_17569,c,av);}
a=C_alloc(16);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t3=*((C_word*)lf[24]+1);
t4=*((C_word*)lf[24]+1);
t5=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17577,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t3,a[9]=((C_word*)t0)[8],a[10]=t1,tmp=(C_word)a,a+=11,tmp);
/* support.scm:1660: ##sys#write-char-0 */
t7=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_17577,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17580,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* support.scm:1660: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_17580,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17583,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* support.scm:1660: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(60);
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_17583,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17586,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* support.scm:1660: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_17586,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17589,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* support.scm:1660: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17589(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_17589,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17592,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* support.scm:1660: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_17592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_17592,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17593,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_check_list_2(((C_word*)t0)[4],lf[44]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17603,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17665,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_17665(t8,t4,((C_word*)t0)[4]);}

/* g4333 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_17593(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_17593,3,t0,t1,t2);}
/* support.scm:1661: g4355 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_17541(t3,t1,((C_word*)t0)[3],t2);}

/* k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_ccall f_17603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_17603,c,av);}
a=C_alloc(9);
t2=C_block_size(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17609,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_fixnum_greaterp(t2,C_fix(4)))){
t4=*((C_word*)lf[24]+1);
t5=*((C_word*)lf[24]+1);
t6=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17621,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* support.scm:1664: ##sys#write-char-0 */
t8=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_make_character(91);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* ##sys#write-char/port */
t4=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(62);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k17607 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_17609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17609,c,av);}
/* ##sys#write-char/port */
t2=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(62);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_17621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17621,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17624,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1664: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[4],C_fix(4));
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17622 in k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in ... */
static void C_ccall f_17624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_17624,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17627,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17632,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_17632(t6,t2,C_fix(5));}

/* k17625 in k17622 in k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_ccall f_17627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17627,c,av);}
/* ##sys#write-char/port */
t2=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(93);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* doloop4369 in k17622 in k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in ... */
static void C_fcall f_17632(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_17632,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=*((C_word*)lf[24]+1);
t4=*((C_word*)lf[24]+1);
t5=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17645,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* support.scm:1667: ##sys#write-char-0 */
t7=*((C_word*)lf[26]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_make_character(32);
av2[3]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}

/* k17643 in doloop4369 in k17622 in k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in ... */
static void C_ccall f_17645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17645,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17648,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1667: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[5],((C_word*)t0)[4]);
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17646 in k17643 in doloop4369 in k17622 in k17619 in k17601 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in ... */
static void C_ccall f_17648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17648,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_17632(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop4332 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in ... */
static void C_fcall f_17665(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_17665,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17675,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1661: g4333 */
t4=((C_word*)t0)[3];
f_17593(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k17673 in for-each-loop4332 in k17590 in k17587 in k17584 in k17581 in k17578 in k17575 in k17567 in loop in chicken.compiler.support#dump-nodes in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in ... */
static void C_ccall f_17675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17675,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_17665(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.support#read-info-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_17688,c,av);}
a=C_alloc(9);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17692,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t6=C_eqp(lf[508],t2);
t7=(C_truep(t6)?C_i_symbolp(C_i_car(t3)):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=C_i_car(t3);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17710,a[2]=t3,a[3]=t5,a[4]=t8,tmp=(C_word)a,a+=5,tmp);
/* support.scm:1681: chicken.string#conc */
t10=*((C_word*)lf[483]+1);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
av2[2]=*((C_word*)lf[509]+1);
av2[3]=lf[510];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}

/* k17690 in chicken.compiler.support#read-info-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17692,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17708 in chicken.compiler.support#read-info-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17710,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17714,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:1682: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[128]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[136]+1);
av2[3]=C_u_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17712 in k17708 in chicken.compiler.support#read-info-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_17714,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,t1);
/* support.scm:1677: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=*((C_word*)lf[136]+1);
av2[3]=((C_word*)t0)[5];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
/* support.scm:1677: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=*((C_word*)lf[136]+1);
av2[3]=((C_word*)t0)[5];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* chicken.compiler.support#read/source-info in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17731,c,av);}
/* support.scm:1687: ##sys#read */
t3=*((C_word*)lf[512]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=*((C_word*)lf[507]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17737,c,av);}
a=C_alloc(4);
if(C_truep(C_i_char_equalp(C_make_character(62),t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17747,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* ##sys#read-char/port */
t5=*((C_word*)lf[517]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* support.scm:1699: old-hook */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_17747,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17750,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17766,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1702: chicken.base#open-output-string */
t4=*((C_word*)lf[334]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k17748 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_17750,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,2,lf[514],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[515],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17766(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_17766,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17771,a[2]=t3,a[3]=t1,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_17771(t5,((C_word*)t0)[3]);}

/* loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_17771(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_17771,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17775,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* ##sys#read-char/port */
t3=*((C_word*)lf[517]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17775,c,av);}
a=C_alloc(5);
if(C_truep(C_eofp(t1))){
/* support.scm:1706: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[516];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(10)))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17792,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1708: scheme#newline */
t3=*((C_word*)lf[23]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(60)))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17803,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* ##sys#read-char/port */
t3=*((C_word*)lf[517]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17826,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* ##sys#write-char/port */
t3=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}}}

/* k17790 in k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17792,c,av);}
/* support.scm:1709: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17771(t2,((C_word*)t0)[3]);}

/* k17801 in k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17803,c,av);}
a=C_alloc(6);
t2=C_eqp(C_make_character(35),t1);
if(C_truep(t2)){
/* support.scm:1713: chicken.base#get-output-string */
t3=*((C_word*)lf[331]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17815,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* ##sys#write-char/port */
t4=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(60);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k17813 in k17801 in k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17815(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17815,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17818,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* ##sys#write-char/port */
t3=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17816 in k17813 in k17801 in k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17818(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17818,c,av);}
/* support.scm:1717: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17771(t2,((C_word*)t0)[3]);}

/* k17824 in k17773 in loop in k17764 in k17745 in ##sys#user-read-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17826,c,av);}
/* support.scm:1720: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17771(t2,((C_word*)t0)[3]);}

/* chicken.compiler.support#big-fixnum? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17831,c,av);}
a=C_alloc(4);
if(C_truep(C_fixnump(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17844,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1727: chicken.platform#feature? */
t4=*((C_word*)lf[518]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[519];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17842 in chicken.compiler.support#big-fixnum? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17844,c,av);}
if(C_truep(t1)){
t2=C_fixnum_greaterp(((C_word*)t0)[2],C_fix(1073741823));
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:C_fixnum_lessp(((C_word*)t0)[2],C_fix(-1073741824)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#small-bignum? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17855,c,av);}
a=C_alloc(4);
if(C_truep(C_i_bignump(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17877,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1733: chicken.platform#feature? */
t4=*((C_word*)lf[518]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[519];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17875 in chicken.compiler.support#small-bignum? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17877(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17877,c,av);}
t2=C_i_not(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_fixnum_less_or_equal_p(C_i_integer_length(((C_word*)t0)[3]),C_fix(62)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#hide-variable in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17879(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17879,c,av);}
a=C_alloc(3);
t3=C_a_i_list(&a,1,lf[521]);
if(C_truep(C_i_nullp(t3))){
/* tweaks.scm:57: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[280];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* tweaks.scm:57: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[280];
av2[4]=C_i_car(t3);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* chicken.compiler.support#export-variable in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17899,c,av);}
a=C_alloc(3);
t3=C_a_i_list(&a,1,lf[523]);
if(C_truep(C_i_nullp(t3))){
/* tweaks.scm:57: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[280];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* tweaks.scm:57: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[280];
av2[4]=C_i_car(t3);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* chicken.compiler.support#variable-hidden? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17919,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17927,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1746: ##sys#get */
t4=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[280];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k17925 in chicken.compiler.support#variable-hidden? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17927(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17927,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(t1,lf[521]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17934 in ##sys#toplevel-definition-hook in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17936,c,av);}
if(C_truep(t1)){
/* support.scm:1749: chicken.plist#remprop! */
t2=*((C_word*)lf[279]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[280];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#variable-visible? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17941,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17945,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1752: ##sys#get */
t5=*((C_word*)lf[182]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[280];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k17943 in chicken.compiler.support#variable-visible? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17945,c,av);}
t2=C_eqp(t1,lf[521]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_eqp(t1,lf[523]);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_SCHEME_TRUE:C_i_not(((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* chicken.compiler.support#mark-variable in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17966,c,av);}
if(C_truep(C_rest_nullp(c,4))){
/* support.scm:1762: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* support.scm:1762: ##sys#put! */
t4=*((C_word*)lf[262]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=C_get_rest_arg(c,4,av,4,t0);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* chicken.compiler.support#variable-mark in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17981,c,av);}
/* support.scm:1765: ##sys#get */
t4=*((C_word*)lf[182]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* chicken.compiler.support#intrinsic? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17987,c,av);}
/* tweaks.scm:60: ##sys#get */
t3=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[503];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.compiler.support#foldable? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_17998(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17998,c,av);}
/* tweaks.scm:60: ##sys#get */
t3=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[527];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.compiler.support#predicate? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18009(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18009,c,av);}
/* tweaks.scm:60: ##sys#get */
t3=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[528];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18020(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_18020,c,av);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18024,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18113,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1776: chicken.platform#repository-path */
t5=*((C_word*)lf[536]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_18024,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18030,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18091,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1777: chicken.base#open-output-string */
t4=*((C_word*)lf[334]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18030,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18058,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1784: scheme#call-with-input-file */
t3=*((C_word*)lf[531]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[82]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18040 in for-each-loop4545 in k18056 in k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18042,c,av);}
/* support.scm:1781: ##sys#put! */
t2=*((C_word*)lf[262]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[530];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k18044 in for-each-loop4545 in k18056 in k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18046,c,av);}
a=C_alloc(3);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_a_i_list1(&a,1,t3);
/* support.scm:1783: scheme#append */
t5=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k18056 in k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18058,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[44]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18066,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_18066(t6,((C_word*)t0)[2],t1);}

/* for-each-loop4545 in k18056 in k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_18066(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_18066,3,t0,t1,t2);}
a=C_alloc(13);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18076,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_car(t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18042,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18046,a[2]=t4,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* support.scm:1783: ##sys#get */
t8=*((C_word*)lf[182]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t7;
av2[2]=t5;
av2[3]=lf[530];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k18074 in for-each-loop4545 in k18056 in k18028 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18076,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_18066(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18091(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_18091,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[328]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18097,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* support.scm:1777: ##sys#print */
t4=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[534];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k18095 in k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_18097,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18100,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1777: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18098 in k18095 in k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_18100,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18103,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1777: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[533];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18101 in k18098 in k18095 in k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18103,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18106,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:1777: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18104 in k18101 in k18098 in k18095 in k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18106(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18106,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18109,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1777: chicken.base#get-output-string */
t3=*((C_word*)lf[331]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k18107 in k18104 in k18101 in k18098 in k18095 in k18089 in k18022 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18109,c,av);}
/* support.scm:1777: debugging */
t2=*((C_word*)lf[22]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[532];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k18111 in chicken.compiler.support#load-identifier-database in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18113,c,av);}
/* support.scm:1776: chicken.load#find-file */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#print-version in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18115,c,av);}
a=C_alloc(6);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?C_SCHEME_FALSE:C_get_rest_arg(c,2,av,2,t0));
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18122,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(t3)){
/* support.scm:1790: chicken.base#print* */
t5=*((C_word*)lf[538]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[539];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f19850,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1791: chicken.platform#chicken-version */
t6=*((C_word*)lf[249]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k18120 in chicken.compiler.support#print-version in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18122,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18129,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1791: chicken.platform#chicken-version */
t3=*((C_word*)lf[249]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k18127 in k18120 in chicken.compiler.support#print-version in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18129(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18129,c,av);}
/* support.scm:1791: chicken.base#print */
t2=*((C_word*)lf[237]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.support#print-usage in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18140(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18140,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18144,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:1796: print-version */
t3=*((C_word*)lf[537]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k18142 in chicken.compiler.support#print-usage in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18144,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18147,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:1797: scheme#newline */
t3=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k18145 in k18142 in chicken.compiler.support#print-usage in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18147,c,av);}
/* support.scm:1798: scheme#display */
t2=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[541];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.support#print-debug-options in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18152(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18152,c,av);}
/* support.scm:1930: scheme#display */
t2=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[543];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a18157 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18158(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_18158,c,av);}
a=C_alloc(5);
t4=C_i_check_port_2(t3,C_fix(2),C_SCHEME_TRUE,lf[32]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18165,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:477: ##sys#print */
t6=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[544];
av2[3]=C_SCHEME_FALSE;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k18163 in a18157 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_18165,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18168,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:477: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[4],C_fix(1));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18166 in k18163 in a18157 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18168,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18171,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:477: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18169 in k18166 in k18163 in a18157 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_18171,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18174,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:477: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[4],C_fix(2));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18172 in k18169 in k18166 in k18163 in a18157 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_18174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18174,c,av);}
/* support.scm:477: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(62);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k5346 */
static void C_ccall f_5348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5348,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5351,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_eval_toplevel(2,av2);}}

/* k5349 in k5346 */
static void C_ccall f_5351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5351,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5354,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_data_2dstructures_toplevel(2,av2);}}

/* k5352 in k5349 in k5346 */
static void C_ccall f_5354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5354,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5357,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_extras_toplevel(2,av2);}}

/* k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5357,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5360,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_file_toplevel(2,av2);}}

/* k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5360,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5363,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_internal_toplevel(2,av2);}}

/* k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5363,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5366,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_pathname_toplevel(2,av2);}}

/* k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5366,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5369,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_port_toplevel(2,av2);}}

/* k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_5369,c,av);}
a=C_alloc(19);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5372,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t4;
C_expand_toplevel(2,av2);}}

/* k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_5372,c,av);}
a=C_alloc(19);
t2=C_mutate(&lf[2] /* (set! chicken.compiler.support#take ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5482,tmp=(C_word)a,a+=2,tmp));
t3=C_mutate(&lf[3] /* (set! chicken.compiler.support#every ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5743,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate(&lf[4] /* (set! chicken.compiler.support#any ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5777,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate(&lf[5] /* (set! chicken.compiler.support#cons* ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5807,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate(&lf[6] /* (set! chicken.compiler.support#last ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6143,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[7] /* (set! chicken.compiler.support#lset-adjoin/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6202,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate((C_word*)lf[8]+1 /* (set! chicken.compiler.support#number-type ...) */,lf[9]);
t9=C_set_block_item(lf[10] /* chicken.compiler.support#unsafe */,0,C_SCHEME_FALSE);
t10=C_mutate((C_word*)lf[11]+1 /* (set! chicken.compiler.support#compiler-cleanup-hook ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6655,tmp=(C_word)a,a+=2,tmp));
t11=C_set_block_item(lf[12] /* chicken.compiler.support#debugging-chicken */,0,C_SCHEME_END_OF_LIST);
t12=C_mutate((C_word*)lf[13]+1 /* (set! chicken.compiler.support#bomb ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6659,tmp=(C_word)a,a+=2,tmp));
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6684,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:121: chicken.base#open-output-string */
t14=*((C_word*)lf[334]+1);{
C_word *av2=av;
av2[0]=t14;
av2[1]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}

/* chicken.compiler.support#take in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5482(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_5482,3,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5500,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:56: take */
t7=t5;
t8=C_u_i_cdr(t2);
t9=C_fixnum_difference(t3,C_fix(1));
t1=t7;
t2=t8;
t3=t9;
goto loop;}}

/* k5498 in chicken.compiler.support#take in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_5500,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in a10632 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5540(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_5540,5,t0,t1,t2,t3,t4);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5554,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:67: scheme#reverse */
t6=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_fixnum_difference(t2,C_fix(1));
t6=C_i_car(t4);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:68: loop */
t9=t1;
t10=t5;
t11=t7;
t12=C_u_i_cdr(t4);
t1=t9;
t2=t10;
t3=t11;
t4=t12;
goto loop;}}

/* k5552 in loop in a10632 in a10626 in chicken.compiler.support#inline-lambda-bindings in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5554,c,av);}
/* mini-srfi-1.scm:67: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* chicken.compiler.support#every in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5743(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_5743,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5749,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_5749(t7,t1,t3);}

/* loop in chicken.compiler.support#every in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5749(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_5749,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5771,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:82: pred */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k5769 in loop in chicken.compiler.support#every in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5771,c,av);}
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:83: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_5749(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.compiler.support#any in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5777(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_5777,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5783,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_5783(t7,t1,t3);}

/* loop in chicken.compiler.support#any in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5783(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_5783,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5793,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:88: pred */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k5791 in loop in chicken.compiler.support#any in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5793,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:89: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_5783(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.compiler.support#cons* in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5807(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_5807,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5813,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_5813(t7,t1,t2,t3);}

/* loop in chicken.compiler.support#cons* in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_5813(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_5813,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5827,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:95: loop */
t6=t4;
t7=C_i_car(t3);
t8=C_u_i_cdr(t3);
t1=t6;
t2=t7;
t3=t8;
goto loop;}}

/* k5825 in loop in chicken.compiler.support#cons* in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_5827(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_5827,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* foldr408 in k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6036(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_6036,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6044,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6070,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g413 in foldr408 in k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6044(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_6044,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=(
/* mini-srfi-1.scm:135: pred */
  f_8162(((C_word*)t0)[2],t2)
);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6052,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:135: g423 */
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(
/* mini-srfi-1.scm:135: g423 */
  f_6052(C_a_i(&a,3),t5,t4)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* g423 in g413 in foldr408 in k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_6052(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_a_i_cons(&a,2,t1,((C_word*)t0)[2]));}

/* k6068 in foldr408 in k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6070,c,av);}
/* mini-srfi-1.scm:134: g413 */
t2=((C_word*)t0)[2];
f_6044(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* map-loop436 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6109(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_6109,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#last in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6143(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,0,2)))){
C_save_and_reclaim_args((void *)trf_6143,2,t1,t2);}
a=C_alloc(2);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6149,tmp=(C_word)a,a+=2,tmp);
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=(
  f_6149(t2)
);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* loop in chicken.compiler.support#last in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_6149(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
return(C_u_i_car(t1));}
else{
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}}

/* chicken.compiler.support#lset-adjoin/eq? in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6202(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_6202,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6208,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_6208(t7,t1,t3,t2);}

/* loop in chicken.compiler.support#lset-adjoin/eq? in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6208(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_6208,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_memq(t4,t3))){
/* mini-srfi-1.scm:160: loop */
t9=t1;
t10=C_u_i_cdr(t2);
t11=t3;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:161: loop */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* loop in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6391(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_6391,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(
/* mini-srfi-1.scm:190: proc */
  f_17526(((C_word*)t0)[3],t2)
);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6409,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:190: loop */
t6=t4;
t7=C_fixnum_plus(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}

/* k6407 in loop in k17408 in chicken.compiler.support#encodeable-literal? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6409,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k12761 in walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6504(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6504,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6517,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:216: pred */
t4=((C_word*)t0)[3];
f_12777(t4,t3,C_i_car(t2));}}

/* k6515 in loop in k12761 in walk in chicken.compiler.support#expression-has-side-effects? in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6517(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6517,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:217: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_6504(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[3]));}}

/* chicken.compiler.support#compiler-cleanup-hook in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6655,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#bomb in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,3)))){
C_save_and_reclaim((void*)f_6659,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6673,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:117: scheme#string-append */
t4=*((C_word*)lf[15]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[16];
av2[3]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* support.scm:118: chicken.base#error */
t3=*((C_word*)lf[14]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[17];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k6671 in chicken.compiler.support#bomb in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6673,c,av);}{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[14]+1);
av2[3]=t1;
av2[4]=C_u_i_cdr(((C_word*)t0)[3]);
C_apply(5,av2);}}

/* k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6684(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,5)))){
C_save_and_reclaim((void *)f_6684,c,av);}
a=C_alloc(47);
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.support#collected-debugging-output ...) */,t1);
t3=C_mutate(&lf[19] /* (set! chicken.compiler.support#+logged-debugging-modes+ ...) */,lf[20]);
t4=C_mutate(&lf[21] /* (set! chicken.compiler.support#test-debugging-mode ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6687,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate((C_word*)lf[22]+1 /* (set! chicken.compiler.support#debugging ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6708,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate((C_word*)lf[34]+1 /* (set! chicken.compiler.support#with-debugging-output ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6828,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate((C_word*)lf[37]+1 /* (set! chicken.compiler.support#quit-compiling ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6925,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate((C_word*)lf[42]+1 /* (set! ##sys#syntax-error-hook ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6941,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate((C_word*)lf[51]+1 /* (set! chicken.syntax#syntax-error ...) */,*((C_word*)lf[42]+1));
t10=C_mutate((C_word*)lf[52]+1 /* (set! chicken.compiler.support#emit-syntax-trace-info ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7038,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate((C_word*)lf[53]+1 /* (set! chicken.compiler.support#check-signature ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7092,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate((C_word*)lf[54]+1 /* (set! chicken.compiler.support#build-lambda-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7133,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate((C_word*)lf[55]+1 /* (set! chicken.compiler.support#c-ify-string ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7170,tmp=(C_word)a,a+=2,tmp));
t14=C_mutate((C_word*)lf[65]+1 /* (set! chicken.compiler.support#valid-c-identifier? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7269,tmp=(C_word)a,a+=2,tmp));
t15=C_mutate((C_word*)lf[67]+1 /* (set! chicken.compiler.support#bytes->words ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7319,tmp=(C_word)a,a+=2,tmp));
t16=C_mutate((C_word*)lf[68]+1 /* (set! chicken.compiler.support#words->bytes ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7326,tmp=(C_word)a,a+=2,tmp));
t17=C_mutate((C_word*)lf[69]+1 /* (set! chicken.compiler.support#check-and-open-input-file ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7333,tmp=(C_word)a,a+=2,tmp));
t18=C_mutate((C_word*)lf[76]+1 /* (set! chicken.compiler.support#close-checked-input-file ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7377,tmp=(C_word)a,a+=2,tmp));
t19=C_mutate((C_word*)lf[79]+1 /* (set! chicken.compiler.support#fold-inner ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7389,tmp=(C_word)a,a+=2,tmp));
t20=C_mutate(&lf[81] /* (set! chicken.compiler.support#follow-without-loop ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7445,tmp=(C_word)a,a+=2,tmp));
t21=C_mutate((C_word*)lf[82]+1 /* (set! chicken.compiler.support#read-expressions ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7496,tmp=(C_word)a,a+=2,tmp));
t22=C_mutate((C_word*)lf[84]+1 /* (set! chicken.compiler.support#constant? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7540,tmp=(C_word)a,a+=2,tmp));
t23=C_mutate((C_word*)lf[88]+1 /* (set! chicken.compiler.support#collapsable-literal? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7602,tmp=(C_word)a,a+=2,tmp));
t24=C_mutate((C_word*)lf[89]+1 /* (set! chicken.compiler.support#immediate? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7632,tmp=(C_word)a,a+=2,tmp));
t25=C_mutate((C_word*)lf[91]+1 /* (set! chicken.compiler.support#basic-literal? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7678,tmp=(C_word)a,a+=2,tmp));
t26=C_mutate((C_word*)lf[93]+1 /* (set! chicken.compiler.support#canonicalize-begin-body ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7734,tmp=(C_word)a,a+=2,tmp));
t27=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7809,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:344: chicken.condition#condition-predicate */
t28=*((C_word*)lf[549]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t28;
av2[1]=t27;
av2[2]=lf[547];
((C_proc)(void*)(*((C_word*)t28+1)))(3,av2);}}

/* chicken.compiler.support#test-debugging-mode in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6687(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_6687,3,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_symbolp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_memq(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6702,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:128: any */
f_5777(t1,t4,t2);}}

/* a6701 in chicken.compiler.support#test-debugging-mode in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6702,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +17,c,3)))){
C_save_and_reclaim((void*)f_6708,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+17);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6711,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t10=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6776,a[2]=t2,tmp=(C_word)a,a+=3,tmp));
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6795,a[2]=t1,a[3]=t8,a[4]=t2,a[5]=t6,tmp=(C_word)a,a+=6,tmp);
/* support.scm:143: test-debugging-mode */
f_6687(t11,t2,*((C_word*)lf[12]+1));}

/* text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6711(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_6711,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6717,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:132: chicken.port#with-output-to-string */
t3=*((C_word*)lf[31]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6717,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6721,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:134: scheme#display */
t3=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_6721,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6724,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6733,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:136: scheme#display */
t4=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[30];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* support.scm:140: scheme#newline */
t3=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k6722 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6724,c,av);}
/* support.scm:140: scheme#newline */
t2=*((C_word*)lf[23]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k6731 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6733,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6753,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_6753(t5,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* k6739 in for-each-loop704 in k6731 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6741,c,av);}
/* support.scm:138: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k6746 in for-each-loop704 in k6731 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6748,c,av);}
/* support.scm:138: ##sys#print */
t2=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* for-each-loop704 in k6731 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6753(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_6753,3,t0,t1,t2);}
a=C_alloc(13);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6763,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[24]+1);
t6=*((C_word*)lf[24]+1);
t7=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6741,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6748,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* support.scm:138: scheme#force */
t10=*((C_word*)lf[28]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k6761 in for-each-loop704 in k6731 in k6719 in a6716 in text in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6763,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6753(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* dump in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6776(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_6776,3,t0,t1,t2);}
a=C_alloc(5);
t3=*((C_word*)lf[18]+1);
t4=*((C_word*)lf[18]+1);
t5=C_i_check_port_2(*((C_word*)lf[18]+1),C_fix(2),C_SCHEME_TRUE,lf[32]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6783,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:142: ##sys#print */
t7=*((C_word*)lf[27]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[18]+1);
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k6781 in dump in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6783,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6786,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:142: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(124);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k6784 in k6781 in dump in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6786,c,av);}
/* support.scm:142: ##sys#print */
t2=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_6795,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6798,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:144: text */
t3=((C_word*)((C_word*)t0)[5])[1];
f_6711(t3,t2);}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6816,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6819,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* support.scm:151: test-debugging-mode */
f_6687(t3,((C_word*)t0)[4],lf[19]);}}

/* k6796 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6798,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6801,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:145: scheme#display */
t3=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k6799 in k6796 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6801,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6804,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* support.scm:146: chicken.base#flush-output */
t3=*((C_word*)lf[33]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k6802 in k6799 in k6796 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_6804,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6807,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6810,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* support.scm:147: test-debugging-mode */
f_6687(t3,((C_word*)t0)[5],lf[19]);}

/* k6805 in k6802 in k6799 in k6796 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6807,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k6808 in k6802 in k6799 in k6796 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6810,c,av);}
if(C_truep(t1)){
/* support.scm:148: dump */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6776(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k6814 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6816,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k6817 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6819,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6826,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:152: text */
t3=((C_word*)((C_word*)t0)[4])[1];
f_6711(t3,t2);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k6824 in k6817 in k6793 in chicken.compiler.support#debugging in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6826,c,av);}
/* support.scm:152: dump */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6776(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_6828,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6831,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6892,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* support.scm:163: test-debugging-mode */
f_6687(t5,t2,*((C_word*)lf[12]+1));}

/* collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6831(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_6831,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6833,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6860,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:162: chicken.string#string-split */
t5=*((C_word*)lf[35]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[36];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* g756 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6833(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_6833,3,t0,t1,t2);}
a=C_alloc(5);
t3=*((C_word*)lf[18]+1);
t4=*((C_word*)lf[18]+1);
t5=C_i_check_port_2(*((C_word*)lf[18]+1),C_fix(2),C_SCHEME_TRUE,lf[32]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6840,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
/* support.scm:159: ##sys#print */
t7=*((C_word*)lf[27]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_u_i_car(((C_word*)t0)[2]);
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[18]+1);
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
/* support.scm:159: ##sys#print */
t7=*((C_word*)lf[27]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_FALSE;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}}

/* k6838 in g756 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6840,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6843,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:159: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(124);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k6841 in k6838 in g756 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_6843,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6846,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:159: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6844 in k6841 in k6838 in g756 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6846,c,av);}
/* support.scm:159: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k6858 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_6860,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6865,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_6865(t5,((C_word*)t0)[3],t1);}

/* for-each-loop755 in k6858 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6865(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6865,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6875,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:157: g756 */
t4=((C_word*)t0)[3];
f_6833(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k6873 in for-each-loop755 in k6858 in collect in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6875,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6865(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6892,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6895,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:164: chicken.port#with-output-to-string */
t3=*((C_word*)lf[31]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6916,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* support.scm:169: test-debugging-mode */
f_6687(t2,((C_word*)t0)[4],lf[19]);}}

/* k6893 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6895(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6895,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6898,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:165: scheme#display */
t3=*((C_word*)lf[29]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k6896 in k6893 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6898,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6901,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* support.scm:166: chicken.base#flush-output */
t3=*((C_word*)lf[33]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k6899 in k6896 in k6893 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6901,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6907,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:167: test-debugging-mode */
f_6687(t2,((C_word*)t0)[5],lf[19]);}

/* k6905 in k6899 in k6896 in k6893 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6907,c,av);}
if(C_truep(t1)){
/* support.scm:168: collect */
t2=((C_word*)t0)[2];
f_6831(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k6914 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6916(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6916,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6923,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:170: chicken.port#with-output-to-string */
t3=*((C_word*)lf[31]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k6921 in k6914 in k6890 in chicken.compiler.support#with-debugging-output in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6923,c,av);}
/* support.scm:170: collect */
t2=((C_word*)t0)[2];
f_6831(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.support#quit-compiling in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_6925,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+9);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
t4=*((C_word*)lf[38]+1);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6929,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6939,a[2]=t5,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:174: scheme#string-append */
t7=*((C_word*)lf[15]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[41];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k6927 in chicken.compiler.support#quit-compiling in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6929,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6932,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:175: scheme#newline */
t3=*((C_word*)lf[23]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k6930 in k6927 in chicken.compiler.support#quit-compiling in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6932,c,av);}
/* support.scm:176: chicken.base#exit */
t2=*((C_word*)lf[39]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k6937 in chicken.compiler.support#quit-compiling in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6939,c,av);}{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[40]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
av2[5]=((C_word*)t0)[4];
C_apply(6,av2);}}

/* ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +10,c,2)))){
C_save_and_reclaim((void*)f_6941,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+10);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t6=*((C_word*)lf[38]+1);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6945,a[2]=t6,a[3]=t5,a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_symbolp(((C_word*)t4)[1]))){
t8=((C_word*)t4)[1];
t9=C_i_car(((C_word*)t5)[1]);
t10=C_set_block_item(t4,0,t9);
t11=C_i_cdr(((C_word*)t5)[1]);
t12=C_set_block_item(t5,0,t11);
t13=t7;
f_6945(t13,t8);}
else{
t8=t7;
f_6945(t8,C_SCHEME_FALSE);}}

/* k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6945(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_6945,2,t0,t1);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6948,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=C_i_check_port_2(((C_word*)t0)[2],C_fix(2),C_SCHEME_TRUE,lf[32]);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6994,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* support.scm:187: ##sys#print */
t5=*((C_word*)lf[27]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[49];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_i_check_port_2(((C_word*)t0)[2],C_fix(2),C_SCHEME_TRUE,lf[32]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7015,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* support.scm:188: ##sys#print */
t5=*((C_word*)lf[27]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[50];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6948(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_6948,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6949,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_i_check_list_2(t3,lf[44]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6959,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6967,a[2]=t7,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_6967(t9,t5,t3);}

/* g800 in k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6949(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_6949,3,t0,t1,t2);}
/* support.scm:189: g833 */
t3=*((C_word*)lf[40]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[43];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6957 in k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_6959,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6962,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:190: chicken.base#print-call-chain */
t3=*((C_word*)lf[45]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_fix(0);
av2[4]=*((C_word*)lf[46]+1);
av2[5]=lf[47];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k6960 in k6957 in k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6962(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6962,c,av);}
/* support.scm:191: chicken.base#exit */
t2=*((C_word*)lf[39]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(70);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* for-each-loop799 in k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_6967(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6967,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6977,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* support.scm:189: g800 */
t4=((C_word*)t0)[3];
f_6949(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k6975 in for-each-loop799 in k6946 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6977(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6977,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6967(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k6992 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6994(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_6994,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6997,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:187: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6995 in k6992 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_6997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_6997,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7000,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:187: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[48];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6998 in k6995 in k6992 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_7000,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7003,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:187: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k7001 in k6998 in k6995 in k6992 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7003(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7003,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7006,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:187: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7004 in k7001 in k6998 in k6995 in k6992 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7006,c,av);}
/* support.scm:187: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k7013 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_7015,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7018,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:188: ##sys#print */
t3=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k7016 in k7013 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7018,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7021,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:188: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7019 in k7016 in k7013 in k6943 in ##sys#syntax-error-hook in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7021,c,av);}
/* support.scm:188: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#emit-syntax-trace-info in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7038,c,av);}
t4=*((C_word*)lf[46]+1);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_emit_syntax_trace_info(t2,t3,C_slot(*((C_word*)lf[46]+1),C_fix(14)));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.compiler.support#check-signature in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_7092,c,av);}
a=C_alloc(2);
t5=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7098,tmp=(C_word)a,a+=2,tmp);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(
  f_7098(t3,t4)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* loop in chicken.compiler.support#check-signature in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_7098(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t2))){
return(C_i_nullp(t1));}
else{
t3=C_i_symbolp(t2);
if(C_truep(t3)){
return(t3);}
else{
if(C_truep(C_i_nullp(t1))){
return(C_SCHEME_FALSE);}
else{
t5=C_i_cdr(t1);
t6=C_i_cdr(t2);
t1=t5;
t2=t6;
goto loop;}}}}

/* chicken.compiler.support#build-lambda-list in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_7133,c,av);}
a=C_alloc(6);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7139,a[2]=t4,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_7139(t8,t1,t2,t3);}

/* loop in chicken.compiler.support#build-lambda-list in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7139(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(33,0,3)))){
C_save_and_reclaim_args((void *)trf_7139,4,t0,t1,t2,t3);}
a=C_alloc(33);
t4=C_i_zerop(t3);
t5=(C_truep(t4)?t4:C_i_nullp(t2));
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[2]:C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7161,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=C_u_i_cdr(t2);
t9=C_s_a_i_minus(&a,2,t3,C_fix(1));
/* support.scm:219: loop */
t11=t7;
t12=t8;
t13=t9;
t1=t11;
t2=t12;
t3=t13;
goto loop;}}

/* k7159 in loop in chicken.compiler.support#build-lambda-list in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7161,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7170,c,av);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7182,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7186,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#string->list */
t5=*((C_word*)lf[60]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7180 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7182,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(34),t1);
/* ##sys#list->string */
t3=*((C_word*)lf[56]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7186,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7188,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_7188(t5,((C_word*)t0)[2],t1);}

/* loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7188(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_7188,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[57];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_fix(C_character_code(t3));
t5=C_fixnum_lessp(t4,C_fix(32));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7210,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t4,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t5)){
t7=t6;
f_7210(t7,t5);}
else{
t7=C_fixnum_greater_or_equal_p(t4,C_fix(127));
t8=t6;
f_7210(t8,(C_truep(t7)?t7:C_u_i_memq(t3,lf[64])));}}}

/* k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7210(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7210,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7217,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnum_lessp(((C_word*)t0)[5],C_fix(8)))){
t3=t2;
f_7217(t3,lf[62]);}
else{
t3=C_fixnum_lessp(((C_word*)t0)[5],C_fix(64));
t4=t2;
f_7217(t4,(C_truep(t3)?lf[63]:C_SCHEME_END_OF_LIST));}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7253,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* support.scm:238: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_7188(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}}

/* k7215 in k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7217(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_7217,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7221,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7231,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* ##sys#fixnum->string */
t4=*((C_word*)lf[61]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=C_fix(8);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k7219 in k7215 in k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7221,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7225,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* support.scm:237: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7188(t3,t2,C_u_i_cdr(((C_word*)t0)[5]));}

/* k7223 in k7219 in k7215 in k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_7225,c,av);}
/* support.scm:232: scheme#append */
t2=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[59];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k7229 in k7215 in k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7231,c,av);}
/* ##sys#string->list */
t2=*((C_word*)lf[60]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k7251 in k7208 in loop in k7184 in chicken.compiler.support#c-ify-string in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7253,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#valid-c-identifier? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7269,c,av);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7273,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7317,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:242: chicken.string#->string */
t5=*((C_word*)lf[66]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7271 in chicken.compiler.support#valid-c-identifier? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7273(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_7273,c,av);}
a=C_alloc(2);
if(C_truep(C_i_pairp(t1))){
t2=C_u_i_car(t1);
t3=C_u_i_char_alphabeticp(t2);
t4=(C_truep(t3)?t3:C_u_i_char_equalp(C_make_character(95),t2));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7294,tmp=(C_word)a,a+=2,tmp);
/* support.scm:246: every */
f_5743(((C_word*)t0)[2],t5,C_u_i_cdr(t1));}
else{
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a7293 in k7271 in chicken.compiler.support#valid-c-identifier? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7294,c,av);}
t3=C_u_i_char_alphabeticp(t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_u_i_char_numericp(t2);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?t4:C_u_i_char_equalp(C_make_character(95),t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k7315 in chicken.compiler.support#valid-c-identifier? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7317,c,av);}
/* ##sys#string->list */
t2=*((C_word*)lf[60]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.support#bytes->words in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7319(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7319,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=stub927(C_SCHEME_UNDEFINED,C_i_foreign_fixnum_argumentp(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#words->bytes in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7326,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=stub932(C_SCHEME_UNDEFINED,C_i_foreign_fixnum_argumentp(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#check-and-open-input-file in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,2)))){
C_save_and_reclaim((void*)f_7333,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(C_i_string_equal_p(t2,lf[70]))){
t4=*((C_word*)lf[71]+1);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=*((C_word*)lf[71]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7346,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:256: chicken.file#file-exists? */
t5=*((C_word*)lf[75]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k7344 in chicken.compiler.support#check-and-open-input-file in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7346,c,av);}
if(C_truep(t1)){
/* support.scm:256: scheme#open-input-file */
t2=*((C_word*)lf[72]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_i_nullp(((C_word*)t0)[4]);
t3=(C_truep(t2)?t2:C_i_not(C_i_car(((C_word*)t0)[4])));
if(C_truep(t3)){
/* support.scm:258: quit-compiling */
t4=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[73];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* support.scm:259: quit-compiling */
t4=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[74];
av2[3]=C_i_car(((C_word*)t0)[4]);
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}}

/* chicken.compiler.support#close-checked-input-file in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7377,c,av);}
if(C_truep(C_i_string_equal_p(t3,lf[77]))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* support.scm:262: scheme#close-input-port */
t4=*((C_word*)lf[78]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* chicken.compiler.support#fold-inner in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7389,c,av);}
a=C_alloc(4);
t4=C_i_cdr(t3);
if(C_truep(C_i_nullp(t4))){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7403,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:267: scheme#reverse */
t6=*((C_word*)lf[80]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k7401 in chicken.compiler.support#fold-inner in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7403(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7403,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7405,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_7405(t5,((C_word*)t0)[3],t1);}

/* fold in k7401 in chicken.compiler.support#fold-inner in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7405(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7405,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_i_cddr(t2);
if(C_truep(C_i_nullp(t3))){
t4=C_u_i_cdr(t2);
t5=C_u_i_car(t4);
t6=C_u_i_car(t2);
t7=C_a_i_list2(&a,2,t5,t6);{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t7;
C_apply(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7431,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* support.scm:272: fold */
t9=t4;
t10=C_u_i_cdr(t2);
t1=t9;
t2=t10;
goto loop;}}

/* k7429 in fold in k7401 in chicken.compiler.support#fold-inner in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7431,c,av);}
a=C_alloc(6);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_list2(&a,2,t1,t2);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t3;
C_apply(4,av2);}}

/* chicken.compiler.support#follow-without-loop in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7445(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_7445,4,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7451,a[2]=t4,a[3]=t6,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_7451(t8,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in chicken.compiler.support#follow-without-loop in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7451(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7451,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_member(t2,t3))){
/* support.scm:277: abort */
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7466,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* support.scm:278: proc */
t5=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* a7465 in loop in chicken.compiler.support#follow-without-loop in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7466(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7466,c,av);}
a=C_alloc(3);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* support.scm:278: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_7451(t4,t1,t2,t3);}

/* a7481 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7482(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7482,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7490,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* support.scm:281: scheme#symbol->string */
t5=*((C_word*)lf[240]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7488 in a7481 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7490,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7494,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:281: scheme#symbol->string */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7492 in k7488 in a7481 in k12188 in k12182 in k12179 in chicken.compiler.support#emit-global-inline-file in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7494,c,av);}
/* support.scm:281: scheme#string<? */
t2=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.support#read-expressions in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7496,c,av);}
a=C_alloc(4);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?*((C_word*)lf[71]+1):C_get_rest_arg(c,2,av,2,t0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7507,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:284: scheme#read */
t5=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7505 in chicken.compiler.support#read-expressions in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_7507,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7509,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_7509(t5,((C_word*)t0)[3],t1,C_fix(0),C_SCHEME_END_OF_LIST);}

/* doloop978 in k7505 in chicken.compiler.support#read-expressions in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7509(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_7509,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_eofp(t2))){
/* support.scm:287: scheme#reverse */
t5=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t1;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7526,a[2]=t3,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[2],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* support.scm:284: scheme#read */
t6=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k7524 in doloop978 in k7505 in chicken.compiler.support#read-expressions in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,4)))){
C_save_and_reclaim((void *)f_7526,c,av);}
a=C_alloc(32);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=((C_word*)((C_word*)t0)[5])[1];
f_7509(t4,((C_word*)t0)[6],t1,t2,t3);}

/* chicken.compiler.support#constant? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7540,c,av);}
a=C_alloc(4);
t3=C_i_numberp(t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_charp(t2);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_stringp(t2);
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_booleanp(t2);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eofp(t2);
if(C_truep(t7)){
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7574,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:298: chicken.blob#blob? */
t9=*((C_word*)lf[87]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}}}}}}

/* k7572 in chicken.compiler.support#constant? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7574,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_vectorp(((C_word*)t0)[3]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7586,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:300: ##sys#srfi-4-vector? */
t4=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}}

/* k7584 in k7572 in chicken.compiler.support#constant? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7586,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_pairp(((C_word*)t0)[3]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_eqp(lf[85],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#collapsable-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7602(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7602,c,av);}
t3=C_booleanp(t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_charp(t2);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eofp(t2);
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_numberp(t2);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?t6:C_i_symbolp(t2));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}}

/* chicken.compiler.support#immediate? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7632(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7632,c,av);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7636,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fixnump(t2))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7676,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* support.scm:311: big-fixnum? */
t5=*((C_word*)lf[90]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t3;
f_7636(t4,C_SCHEME_FALSE);}}

/* k7634 in chicken.compiler.support#immediate? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7636(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,1)))){
C_save_and_reclaim_args((void *)trf_7636,2,t0,t1);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(C_SCHEME_UNDEFINED,((C_word*)t0)[3]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_nullp(((C_word*)t0)[3]);
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eofp(((C_word*)t0)[3]);
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_charp(((C_word*)t0)[3]);
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?t5:C_booleanp(((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}}

/* k7674 in chicken.compiler.support#immediate? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7676,c,av);}
t2=((C_word*)t0)[2];
f_7636(t2,C_i_not(t1));}

/* chicken.compiler.support#basic-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7678,c,av);}
a=C_alloc(4);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_symbolp(t2);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7694,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:321: constant? */
t6=*((C_word*)lf[84]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}

/* k7692 in chicken.compiler.support#basic-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7694,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7700,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_vectorp(((C_word*)t0)[3]))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7732,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:322: scheme#vector->list */
t4=*((C_word*)lf[92]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_7700(2,av2);}}}}

/* k7698 in k7692 in chicken.compiler.support#basic-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7700,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7715,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:324: basic-literal? */
t3=*((C_word*)lf[91]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k7713 in k7698 in k7692 in chicken.compiler.support#basic-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7715,c,av);}
if(C_truep(t1)){
/* support.scm:325: basic-literal? */
t2=*((C_word*)lf[91]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k7730 in k7692 in chicken.compiler.support#basic-literal? in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7732,c,av);}
/* support.scm:322: every */
f_5743(((C_word*)t0)[2],*((C_word*)lf[91]+1),t1);}

/* chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7734(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7734,c,av);}
a=C_alloc(5);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7740,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_7740(t6,t1,t2);}

/* loop in chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7740(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_7740,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[94];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cdr(t2);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_u_i_car(t2);
t5=C_i_equalp(t4,lf[95]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7764,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_7764(t7,t5);}
else{
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7795,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:336: constant? */
t8=*((C_word*)lf[84]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}}}

/* k7762 in loop in chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7764(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7764,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
/* support.scm:338: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7740(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7790,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* support.scm:339: chicken.base#gensym */
t3=*((C_word*)lf[97]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[98];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k7778 in k7788 in k7762 in loop in chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_7780,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[96],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k7788 in k7762 in loop in chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7790(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_7790,c,av);}
a=C_alloc(13);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_list(&a,2,t1,t2);
t4=C_a_i_list(&a,1,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7780,a[2]=((C_word*)t0)[3],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:340: loop */
t6=((C_word*)((C_word*)t0)[4])[1];
f_7740(t6,t5,C_u_i_cdr(((C_word*)t0)[2]));}

/* k7793 in loop in chicken.compiler.support#canonicalize-begin-body in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7795,c,av);}
t2=((C_word*)t0)[2];
f_7764(t2,(C_truep(t1)?t1:C_i_equalp(((C_word*)t0)[3],lf[99])));}

/* k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7809(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7809,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7812,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* support.scm:345: chicken.condition#condition-property-accessor */
t3=*((C_word*)lf[546]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[547];
av2[3]=lf[548];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(51,c,6)))){
C_save_and_reclaim((void *)f_7812,c,av);}
a=C_alloc(51);
t2=C_mutate((C_word*)lf[100]+1 /* (set! chicken.compiler.support#string->expr ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7813,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t3=C_mutate((C_word*)lf[108]+1 /* (set! chicken.compiler.support#llist-length ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7931,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate((C_word*)lf[109]+1 /* (set! chicken.compiler.support#llist-match? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7934,tmp=(C_word)a,a+=2,tmp));
t5=lf[110] /* chicken.compiler.support#profile-info-vector-name */ =C_SCHEME_FALSE;;
t6=C_mutate((C_word*)lf[111]+1 /* (set! chicken.compiler.support#reset-profile-info-vector-name! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7979,tmp=(C_word)a,a+=2,tmp));
t7=lf[114] /* chicken.compiler.support#profile-lambda-list */ =C_SCHEME_END_OF_LIST;;
t8=lf[115] /* chicken.compiler.support#profile-lambda-index */ =C_fix(0);;
t9=C_mutate((C_word*)lf[116]+1 /* (set! chicken.compiler.support#expand-profile-lambda ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7988,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate((C_word*)lf[122]+1 /* (set! chicken.compiler.support#profiling-prelude-exps ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8043,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate((C_word*)lf[127]+1 /* (set! chicken.compiler.support#db-get ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8132,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate((C_word*)lf[129]+1 /* (set! chicken.compiler.support#db-get-all ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8150,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate((C_word*)lf[131]+1 /* (set! chicken.compiler.support#db-put! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8168,tmp=(C_word)a,a+=2,tmp));
t14=C_mutate((C_word*)lf[133]+1 /* (set! chicken.compiler.support#collect! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8214,tmp=(C_word)a,a+=2,tmp));
t15=C_mutate((C_word*)lf[134]+1 /* (set! chicken.compiler.support#db-get-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8266,tmp=(C_word)a,a+=2,tmp));
t16=C_mutate((C_word*)lf[135]+1 /* (set! chicken.compiler.support#get-line ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8275,tmp=(C_word)a,a+=2,tmp));
t17=C_mutate((C_word*)lf[137]+1 /* (set! chicken.compiler.support#get-line-2 ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8285,tmp=(C_word)a,a+=2,tmp));
t18=C_mutate((C_word*)lf[138]+1 /* (set! chicken.compiler.support#display-line-number-database ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8326,tmp=(C_word)a,a+=2,tmp));
t19=C_mutate((C_word*)lf[140]+1 /* (set! chicken.compiler.support#make-node ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8398,tmp=(C_word)a,a+=2,tmp));
t20=C_mutate((C_word*)lf[142]+1 /* (set! chicken.compiler.support#node? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8404,tmp=(C_word)a,a+=2,tmp));
t21=C_mutate((C_word*)lf[143]+1 /* (set! chicken.compiler.support#node-class ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8410,tmp=(C_word)a,a+=2,tmp));
t22=C_mutate((C_word*)lf[145]+1 /* (set! chicken.compiler.support#node-class-set! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8419,tmp=(C_word)a,a+=2,tmp));
t23=C_mutate((C_word*)lf[147]+1 /* (set! chicken.compiler.support#node-parameters ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8428,tmp=(C_word)a,a+=2,tmp));
t24=C_mutate((C_word*)lf[149]+1 /* (set! chicken.compiler.support#node-parameters-set! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8437,tmp=(C_word)a,a+=2,tmp));
t25=C_mutate((C_word*)lf[150]+1 /* (set! chicken.compiler.support#node-subexpressions ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8446,tmp=(C_word)a,a+=2,tmp));
t26=C_mutate((C_word*)lf[152]+1 /* (set! chicken.compiler.support#node-subexpressions-set! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8455,tmp=(C_word)a,a+=2,tmp));
t27=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8465,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t28=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18158,tmp=(C_word)a,a+=2,tmp);
/* support.scm:475: chicken.base#set-record-printer! */
t29=*((C_word*)lf[545]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t29;
av2[1]=t27;
av2[2]=lf[141];
av2[3]=t28;
((C_proc)(void*)(*((C_word*)t29+1)))(4,av2);}}

/* chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_7813,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7817,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7822,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* support.scm:347: scheme#call-with-current-continuation */
t5=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7815 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7817,c,av);}
/* support.scm:346: g1086 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7822,c,av);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7828,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7853,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:347: chicken.condition#with-exception-handler */
t5=*((C_word*)lf[106]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a7827 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7828,c,av);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7834,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* support.scm:347: k1083 */
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a7833 in a7827 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_7834,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7842,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7845,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:350: exn? */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7840 in a7833 in a7827 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7842,c,av);}
/* support.scm:348: quit-compiling */
t2=*((C_word*)lf[37]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[101];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k7843 in a7833 in a7827 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7845,c,av);}
if(C_truep(t1)){
/* support.scm:351: exn-msg */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
/* support.scm:352: chicken.string#->string */
t2=*((C_word*)lf[66]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7853,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7855,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7912,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7929,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* tmp14965 */
t5=t2;
f_7855(t5,t4);}

/* tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7855(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7855,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7859,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7881,tmp=(C_word)a,a+=2,tmp);
/* support.scm:353: chicken.port#with-input-from-string */
t4=*((C_word*)lf[105]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k7857 in tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7859,c,av);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[102];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_cdr(t1);
t3=C_i_nullp(t2);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_u_i_car(t1):C_a_i_cons(&a,2,lf[103],t1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a7880 in tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7881,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7887,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_7887(t5,t1,C_SCHEME_END_OF_LIST);}

/* loop in a7880 in tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7887(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7887,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7891,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* support.scm:357: scheme#read */
t4=*((C_word*)lf[83]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k7889 in loop in a7880 in tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7891,c,av);}
a=C_alloc(3);
if(C_truep(C_eofp(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7904,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:359: scheme#reverse */
t3=*((C_word*)lf[80]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
/* support.scm:360: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7887(t3,((C_word*)t0)[2],t2);}}

/* k7902 in k7889 in loop in a7880 in tmp14965 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7904,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[104]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* tmp24966 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_7912(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_7912,3,t0,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7918,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* support.scm:347: k1083 */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a7917 in tmp24966 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7918,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k7927 in a7852 in a7821 in chicken.compiler.support#string->expr in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7929,c,av);}
a=C_alloc(3);
/* tmp24966 */
t2=((C_word*)t0)[2];
f_7912(t2,((C_word*)t0)[3],C_a_i_list(&a,1,t1));}

/* chicken.compiler.support#llist-length in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7931,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#llist-match? in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_7934,c,av);}
a=C_alloc(2);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7940,tmp=(C_word)a,a+=2,tmp);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(
  f_7940(t2,t3)
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* loop in chicken.compiler.support#llist-match? in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_7940(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t1))){
return(C_i_nullp(t2));}
else{
t3=C_i_symbolp(t1);
if(C_truep(t3)){
return(t3);}
else{
if(C_truep(C_i_nullp(t2))){
return(C_i_not_pair_p(t1));}
else{
t5=C_i_cdr(t1);
t6=C_i_cdr(t2);
t1=t5;
t2=t6;
goto loop;}}}}

/* chicken.compiler.support#reset-profile-info-vector-name! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7979,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7984,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:381: make-random-name */
t3=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[113];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7982 in chicken.compiler.support#reset-profile-info-vector-name! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7984,c,av);}
t2=C_mutate(&lf[110] /* (set! chicken.compiler.support#profile-info-vector-name ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#expand-profile-lambda in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7988,c,av);}
a=C_alloc(7);
t5=lf[115];
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7992,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* support.scm:388: chicken.base#gensym */
t7=*((C_word*)lf[97]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k7990 in chicken.compiler.support#expand-profile-lambda in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_7992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(131,c,1)))){
C_save_and_reclaim((void *)f_7992,c,av);}
a=C_alloc(131);
t2=lf[114];
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,t3,lf[114]);
t5=C_mutate(&lf[114] /* (set! chicken.compiler.support#profile-lambda-list ...) */,t4);
t6=C_mutate(&lf[115] /* (set! chicken.compiler.support#profile-lambda-index ...) */,C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1)));
t7=C_a_i_list(&a,2,lf[85],((C_word*)t0)[2]);
t8=C_a_i_list(&a,3,lf[117],t7,lf[110]);
t9=C_a_i_list(&a,3,lf[118],C_SCHEME_END_OF_LIST,t8);
t10=C_a_i_list(&a,3,lf[118],((C_word*)t0)[4],((C_word*)t0)[5]);
t11=C_a_i_list(&a,3,lf[119],t10,t1);
t12=C_a_i_list(&a,3,lf[118],C_SCHEME_END_OF_LIST,t11);
t13=C_a_i_list(&a,2,lf[85],((C_word*)t0)[2]);
t14=C_a_i_list(&a,3,lf[120],t13,lf[110]);
t15=C_a_i_list(&a,3,lf[118],C_SCHEME_END_OF_LIST,t14);
t16=C_a_i_list(&a,4,lf[121],t9,t12,t15);
t17=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t17;
av2[1]=C_a_i_list(&a,3,lf[118],t1,t16);
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}

/* chicken.compiler.support#profiling-prelude-exps in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(46,c,3)))){
C_save_and_reclaim((void *)f_8043,c,av);}
a=C_alloc(46);
t3=C_i_length(lf[114]);
t4=C_a_i_list(&a,2,lf[85],t3);
t5=C_a_i_list(&a,2,lf[85],t2);
t6=C_a_i_list(&a,3,lf[123],t4,t5);
t7=C_a_i_list(&a,3,lf[124],lf[110],t6);
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=lf[114];
t13=C_i_check_list_2(lf[114],lf[125]);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8080,a[2]=t1,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8082,a[2]=t10,a[3]=t16,a[4]=t11,tmp=(C_word)a,a+=5,tmp));
t18=((C_word*)t16)[1];
f_8082(t18,t14,lf[114]);}

/* k8078 in chicken.compiler.support#profiling-prelude-exps in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_8080,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1141 in chicken.compiler.support#profiling-prelude-exps in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8082(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(27,0,2)))){
C_save_and_reclaim_args((void *)trf_8082,3,t0,t1,t2);}
a=C_alloc(27);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_list(&a,2,lf[85],t4);
t6=C_u_i_cdr(t3);
t7=C_a_i_list(&a,2,lf[85],t6);
t8=C_a_i_list(&a,4,lf[126],lf[110],t5,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t1=t13;
t2=t14;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#db-get in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_8132,c,av);}
a=C_alloc(4);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8136,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:413: chicken.internal#hash-table-ref */
t6=*((C_word*)lf[128]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k8134 in chicken.compiler.support#db-get in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8136,c,av);}
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_slot(t2,C_fix(1)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8150(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +4,c,3)))){
C_save_and_reclaim((void*)f_8150,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+4);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8154,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:419: chicken.internal#hash-table-ref */
t6=*((C_word*)lf[128]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_8154,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8162,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t3=C_i_check_list_2(((C_word*)t0)[2],lf[130]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6036,a[2]=t2,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_6036(t7,((C_word*)t0)[3],((C_word*)t0)[2]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a8161 in k8152 in chicken.compiler.support#db-get-all in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static C_word C_fcall f_8162(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_i_assq(t1,((C_word*)t0)[2]));}

/* chicken.compiler.support#db-put! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_8168,c,av);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8172,a[2]=t4,a[3]=t1,a[4]=t5,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* support.scm:425: chicken.internal#hash-table-ref */
t7=*((C_word*)lf[128]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k8170 in chicken.compiler.support#db-put! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8172,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],t1);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_setslot(t2,C_fix(1),((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(((C_word*)t0)[4])){
t3=C_slot(t1,C_fix(1));
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[4]);
t5=C_a_i_cons(&a,2,t4,t3);
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_i_setslot(t1,C_fix(1),t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}
else{
if(C_truep(((C_word*)t0)[4])){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[4]);
t3=C_a_i_list1(&a,1,t2);
/* support.scm:430: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* chicken.compiler.support#collect! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_8214,c,av);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8218,a[2]=t4,a[3]=t5,a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* support.scm:433: chicken.internal#hash-table-ref */
t7=*((C_word*)lf[128]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k8216 in chicken.compiler.support#collect! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8218(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_8218,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],t1);
if(C_truep(t2)){
t3=C_slot(t2,C_fix(1));
t4=C_a_i_cons(&a,2,((C_word*)t0)[3],t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_setslot(t2,C_fix(1),t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t3=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t4=C_slot(t1,C_fix(1));
t5=C_a_i_cons(&a,2,((C_word*)t0)[2],t3);
t6=C_a_i_cons(&a,2,t5,t4);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_setslot(t1,C_fix(1),t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_list1(&a,1,t2);
/* support.scm:438: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* chicken.compiler.support#db-get-list in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8266(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_8266,c,av);}
a=C_alloc(3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8270,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:441: db-get */
t6=*((C_word*)lf[127]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k8268 in chicken.compiler.support#db-get-list in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8270,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#get-line in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8275,c,av);}
/* support.scm:448: db-get */
t3=*((C_word*)lf[127]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=*((C_word*)lf[136]+1);
av2[3]=C_i_car(t2);
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* chicken.compiler.support#get-line-2 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8285,c,av);}
a=C_alloc(5);
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8292,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:452: chicken.internal#hash-table-ref */
t5=*((C_word*)lf[128]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=*((C_word*)lf[136]+1);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k8290 in chicken.compiler.support#get-line-2 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8292,c,av);}
a=C_alloc(3);
t2=(C_truep(t1)?C_i_assq(((C_word*)t0)[2],C_i_cdr(t1)):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8299,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* support.scm:453: g1224 */
t4=t3;
f_8299(t4,((C_word*)t0)[3],t2);}
else{
/* support.scm:455: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
C_values(4,av2);}}}

/* g1224 in k8290 in chicken.compiler.support#get-line-2 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8299(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_8299,3,t0,t1,t2);}
/* support.scm:454: scheme#values */{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=C_i_car(((C_word*)t0)[2]);
av2[3]=C_i_cdr(t2);
C_values(4,av2);}}

/* chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,4)))){
C_save_and_reclaim((void *)f_8326,c,av);}
a=C_alloc(2);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8332,tmp=(C_word)a,a+=2,tmp);
/* support.scm:458: chicken.internal#hash-table-for-each */
t3=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=*((C_word*)lf[136]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_8332,c,av);}
a=C_alloc(5);
if(C_truep(t3)){
t4=*((C_word*)lf[24]+1);
t5=*((C_word*)lf[24]+1);
t6=C_i_check_port_2(*((C_word*)lf[24]+1),C_fix(2),C_SCHEME_TRUE,lf[25]);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8342,a[2]=t1,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* support.scm:460: ##sys#print */
t8=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[24]+1);
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k8340 in a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8342,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8345,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* support.scm:460: ##sys#write-char-0 */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k8343 in k8340 in a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_8345,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8348,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[4],lf[125]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8361,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8363,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_8363(t12,t8,((C_word*)t0)[4]);}

/* k8346 in k8343 in k8340 in a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8348,c,av);}
/* support.scm:460: ##sys#write-char-0 */
t2=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k8359 in k8343 in k8340 in a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8361,c,av);}
/* support.scm:460: ##sys#print */
t2=*((C_word*)lf[27]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1236 in k8343 in k8340 in a8331 in chicken.compiler.support#display-line-number-database in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8363(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_8363,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cdr(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.support#make-node in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8398,c,av);}
a=C_alloc(5);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_record4(&a,4,lf[141],t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.compiler.support#node? in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8404(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8404,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[141]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.support#node-class in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8410(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8410,c,av);}
t3=C_i_check_structure_2(t2,lf[141],lf[144]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#node-class-set! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8419,c,av);}
t4=C_i_check_structure_2(t2,lf[141],C_SCHEME_FALSE);
/* support.scm:468: ##sys#block-set! */
t5=*((C_word*)lf[146]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(1);
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* chicken.compiler.support#node-parameters in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8428,c,av);}
t3=C_i_check_structure_2(t2,lf[141],lf[148]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#node-parameters-set! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8437,c,av);}
t4=C_i_check_structure_2(t2,lf[141],C_SCHEME_FALSE);
/* support.scm:468: ##sys#block-set! */
t5=*((C_word*)lf[146]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(2);
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* chicken.compiler.support#node-subexpressions in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8446,c,av);}
t3=C_i_check_structure_2(t2,lf[141],lf[151]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#node-subexpressions-set! in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8455,c,av);}
t4=C_i_check_structure_2(t2,lf[141],C_SCHEME_FALSE);
/* support.scm:468: ##sys#block-set! */
t5=*((C_word*)lf[146]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(3);
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(162,c,8)))){
C_save_and_reclaim((void *)f_8465,c,av);}
a=C_alloc(162);
t2=C_mutate((C_word*)lf[140]+1 /* (set! chicken.compiler.support#make-node ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8467,tmp=(C_word)a,a+=2,tmp));
t3=C_mutate((C_word*)lf[153]+1 /* (set! chicken.compiler.support#varnode ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8473,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate((C_word*)lf[155]+1 /* (set! chicken.compiler.support#qnode ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8488,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate((C_word*)lf[156]+1 /* (set! chicken.compiler.support#build-node-graph ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8503,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate((C_word*)lf[192]+1 /* (set! chicken.compiler.support#build-expression-tree ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9699,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate((C_word*)lf[205]+1 /* (set! chicken.compiler.support#fold-boolean ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_10569,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate((C_word*)lf[207]+1 /* (set! chicken.compiler.support#inline-lambda-bindings ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_10621,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate((C_word*)lf[227]+1 /* (set! chicken.compiler.support#replace-rest-op-with-list-ops ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11696,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate((C_word*)lf[226]+1 /* (set! chicken.compiler.support#tree-copy ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11860,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate((C_word*)lf[235]+1 /* (set! chicken.compiler.support#copy-node ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11890,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate((C_word*)lf[214]+1 /* (set! chicken.compiler.support#copy-node! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11928,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate((C_word*)lf[236]+1 /* (set! chicken.compiler.support#emit-global-inline-file ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12124,tmp=(C_word)a,a+=2,tmp));
t14=C_mutate((C_word*)lf[261]+1 /* (set! chicken.compiler.support#load-inline-file ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12442,tmp=(C_word)a,a+=2,tmp));
t15=C_mutate((C_word*)lf[264]+1 /* (set! chicken.compiler.support#match-node ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12498,tmp=(C_word)a,a+=2,tmp));
t16=C_mutate((C_word*)lf[267]+1 /* (set! chicken.compiler.support#expression-has-side-effects? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12731,tmp=(C_word)a,a+=2,tmp));
t17=C_mutate((C_word*)lf[270]+1 /* (set! chicken.compiler.support#simple-lambda-node? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12840,tmp=(C_word)a,a+=2,tmp));
t18=C_mutate((C_word*)lf[271]+1 /* (set! chicken.compiler.support#dump-undefined-globals ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12966,tmp=(C_word)a,a+=2,tmp));
t19=C_mutate((C_word*)lf[276]+1 /* (set! chicken.compiler.support#dump-defined-globals ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13007,tmp=(C_word)a,a+=2,tmp));
t20=C_mutate((C_word*)lf[277]+1 /* (set! chicken.compiler.support#dump-global-refs ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13044,tmp=(C_word)a,a+=2,tmp));
t21=C_mutate((C_word*)lf[278]+1 /* (set! ##sys#toplevel-definition-hook ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13093,tmp=(C_word)a,a+=2,tmp));
t22=C_set_block_item(lf[269] /* chicken.compiler.support#foreign-callback-stubs */,0,C_SCHEME_END_OF_LIST);
t23=C_mutate((C_word*)lf[284]+1 /* (set! chicken.compiler.support#make-foreign-callback-stub ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13121,tmp=(C_word)a,a+=2,tmp));
t24=C_mutate((C_word*)lf[286]+1 /* (set! chicken.compiler.support#foreign-callback-stub? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13127,tmp=(C_word)a,a+=2,tmp));
t25=C_mutate((C_word*)lf[268]+1 /* (set! chicken.compiler.support#foreign-callback-stub-id ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13133,tmp=(C_word)a,a+=2,tmp));
t26=C_mutate((C_word*)lf[288]+1 /* (set! chicken.compiler.support#foreign-callback-stub-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13142,tmp=(C_word)a,a+=2,tmp));
t27=C_mutate((C_word*)lf[290]+1 /* (set! chicken.compiler.support#foreign-callback-stub-qualifiers ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13151,tmp=(C_word)a,a+=2,tmp));
t28=C_mutate((C_word*)lf[292]+1 /* (set! chicken.compiler.support#foreign-callback-stub-return-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13160,tmp=(C_word)a,a+=2,tmp));
t29=C_mutate((C_word*)lf[294]+1 /* (set! chicken.compiler.support#foreign-callback-stub-argument-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13169,tmp=(C_word)a,a+=2,tmp));
t30=C_mutate((C_word*)lf[296]+1 /* (set! chicken.compiler.support#register-foreign-callback-stub! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13178,tmp=(C_word)a,a+=2,tmp));
t31=lf[298] /* chicken.compiler.support#foreign-type-table */ =C_SCHEME_FALSE;;
t32=C_mutate((C_word*)lf[299]+1 /* (set! chicken.compiler.support#clear-foreign-type-table! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13207,tmp=(C_word)a,a+=2,tmp));
t33=C_mutate((C_word*)lf[302]+1 /* (set! chicken.compiler.support#register-foreign-type! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13220,tmp=(C_word)a,a+=2,tmp));
t34=C_mutate((C_word*)lf[303]+1 /* (set! chicken.compiler.support#lookup-foreign-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13274,tmp=(C_word)a,a+=2,tmp));
t35=lf[304];
t36=C_mutate((C_word*)lf[305]+1 /* (set! chicken.compiler.support#foreign-type-check ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13280,a[2]=t35,tmp=(C_word)a,a+=3,tmp));
t37=C_mutate(&lf[402] /* (set! chicken.compiler.support#foreign-type-result-converter ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14356,tmp=(C_word)a,a+=2,tmp));
t38=C_mutate(&lf[403] /* (set! chicken.compiler.support#foreign-type-argument-converter ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14377,tmp=(C_word)a,a+=2,tmp));
t39=C_mutate((C_word*)lf[404]+1 /* (set! chicken.compiler.support#foreign-type-convert-result ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14398,tmp=(C_word)a,a+=2,tmp));
t40=C_mutate((C_word*)lf[405]+1 /* (set! chicken.compiler.support#foreign-type-convert-argument ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14413,tmp=(C_word)a,a+=2,tmp));
t41=C_mutate((C_word*)lf[406]+1 /* (set! chicken.compiler.support#final-foreign-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14428,tmp=(C_word)a,a+=2,tmp));
t42=C_mutate((C_word*)lf[408]+1 /* (set! chicken.compiler.support#estimate-foreign-result-size ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14467,tmp=(C_word)a,a+=2,tmp));
t43=C_mutate((C_word*)lf[414]+1 /* (set! chicken.compiler.support#estimate-foreign-result-location-size ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14942,tmp=(C_word)a,a+=2,tmp));
t44=C_mutate((C_word*)lf[417]+1 /* (set! chicken.compiler.support#finish-foreign-result ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15392,tmp=(C_word)a,a+=2,tmp));
t45=C_mutate((C_word*)lf[431]+1 /* (set! chicken.compiler.support#foreign-type->scrutiny-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15660,tmp=(C_word)a,a+=2,tmp));
t46=C_mutate((C_word*)lf[459]+1 /* (set! chicken.compiler.support#scan-used-variables ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16314,tmp=(C_word)a,a+=2,tmp));
t47=C_mutate((C_word*)lf[460]+1 /* (set! chicken.compiler.support#scan-free-variables ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16469,tmp=(C_word)a,a+=2,tmp));
t48=C_mutate((C_word*)lf[461]+1 /* (set! chicken.compiler.support#chop-separator ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16699,tmp=(C_word)a,a+=2,tmp));
t49=C_mutate((C_word*)lf[463]+1 /* (set! chicken.compiler.support#make-block-variable-literal ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16723,tmp=(C_word)a,a+=2,tmp));
t50=C_mutate((C_word*)lf[465]+1 /* (set! chicken.compiler.support#block-variable-literal? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16729,tmp=(C_word)a,a+=2,tmp));
t51=C_mutate((C_word*)lf[466]+1 /* (set! chicken.compiler.support#block-variable-literal-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16735,tmp=(C_word)a,a+=2,tmp));
t52=C_mutate((C_word*)lf[112]+1 /* (set! chicken.compiler.support#make-random-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16744,tmp=(C_word)a,a+=2,tmp));
t53=lf[470] /* chicken.compiler.support#real-name-table */ =C_SCHEME_FALSE;;
t54=C_mutate((C_word*)lf[471]+1 /* (set! chicken.compiler.support#clear-real-name-table! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16791,tmp=(C_word)a,a+=2,tmp));
t55=C_mutate((C_word*)lf[472]+1 /* (set! chicken.compiler.support#set-real-name! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16798,tmp=(C_word)a,a+=2,tmp));
t56=C_mutate((C_word*)lf[473]+1 /* (set! chicken.compiler.support#get-real-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16804,tmp=(C_word)a,a+=2,tmp));
t57=C_mutate((C_word*)lf[181]+1 /* (set! chicken.compiler.support#real-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16811,tmp=(C_word)a,a+=2,tmp));
t58=C_mutate((C_word*)lf[480]+1 /* (set! chicken.compiler.support#real-name2 ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16931,tmp=(C_word)a,a+=2,tmp));
t59=C_mutate((C_word*)lf[481]+1 /* (set! chicken.compiler.support#display-real-name-table ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16943,tmp=(C_word)a,a+=2,tmp));
t60=C_mutate((C_word*)lf[482]+1 /* (set! chicken.compiler.support#source-info->string ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16967,tmp=(C_word)a,a+=2,tmp));
t61=C_mutate((C_word*)lf[488]+1 /* (set! chicken.compiler.support#source-info->name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17002,tmp=(C_word)a,a+=2,tmp));
t62=C_mutate((C_word*)lf[489]+1 /* (set! chicken.compiler.support#source-info->line ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17017,tmp=(C_word)a,a+=2,tmp));
t63=C_mutate((C_word*)lf[490]+1 /* (set! chicken.compiler.support#call-info ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17029,tmp=(C_word)a,a+=2,tmp));
t64=C_mutate((C_word*)lf[493]+1 /* (set! chicken.compiler.support#constant-form-eval ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17066,tmp=(C_word)a,a+=2,tmp));
t65=C_mutate((C_word*)lf[500]+1 /* (set! chicken.compiler.support#maybe-constant-fold-call ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17296,tmp=(C_word)a,a+=2,tmp));
t66=C_mutate(&lf[496] /* (set! chicken.compiler.support#encodeable-literal? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17395,tmp=(C_word)a,a+=2,tmp));
t67=C_mutate((C_word*)lf[505]+1 /* (set! chicken.compiler.support#dump-nodes ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17532,tmp=(C_word)a,a+=2,tmp));
t68=C_mutate((C_word*)lf[507]+1 /* (set! chicken.compiler.support#read-info-hook ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17688,tmp=(C_word)a,a+=2,tmp));
t69=C_mutate((C_word*)lf[511]+1 /* (set! chicken.compiler.support#read/source-info ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17731,tmp=(C_word)a,a+=2,tmp));
t70=*((C_word*)lf[513]+1);
t71=C_mutate((C_word*)lf[513]+1 /* (set! ##sys#user-read-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17737,a[2]=t70,tmp=(C_word)a,a+=3,tmp));
t72=C_mutate((C_word*)lf[90]+1 /* (set! chicken.compiler.support#big-fixnum? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17831,tmp=(C_word)a,a+=2,tmp));
t73=C_mutate((C_word*)lf[520]+1 /* (set! chicken.compiler.support#small-bignum? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17855,tmp=(C_word)a,a+=2,tmp));
t74=C_mutate((C_word*)lf[282]+1 /* (set! chicken.compiler.support#hide-variable ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17879,tmp=(C_word)a,a+=2,tmp));
t75=C_mutate((C_word*)lf[522]+1 /* (set! chicken.compiler.support#export-variable ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17899,tmp=(C_word)a,a+=2,tmp));
t76=C_mutate((C_word*)lf[281]+1 /* (set! chicken.compiler.support#variable-hidden? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17919,tmp=(C_word)a,a+=2,tmp));
t77=C_mutate((C_word*)lf[260]+1 /* (set! chicken.compiler.support#variable-visible? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17941,tmp=(C_word)a,a+=2,tmp));
t78=C_mutate((C_word*)lf[524]+1 /* (set! chicken.compiler.support#mark-variable ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17966,tmp=(C_word)a,a+=2,tmp));
t79=C_mutate((C_word*)lf[525]+1 /* (set! chicken.compiler.support#variable-mark ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17981,tmp=(C_word)a,a+=2,tmp));
t80=C_mutate((C_word*)lf[526]+1 /* (set! chicken.compiler.support#intrinsic? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17987,tmp=(C_word)a,a+=2,tmp));
t81=C_mutate((C_word*)lf[502]+1 /* (set! chicken.compiler.support#foldable? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17998,tmp=(C_word)a,a+=2,tmp));
t82=C_mutate((C_word*)lf[501]+1 /* (set! chicken.compiler.support#predicate? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18009,tmp=(C_word)a,a+=2,tmp));
t83=C_mutate((C_word*)lf[529]+1 /* (set! chicken.compiler.support#load-identifier-database ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18020,tmp=(C_word)a,a+=2,tmp));
t84=C_mutate((C_word*)lf[537]+1 /* (set! chicken.compiler.support#print-version ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18115,tmp=(C_word)a,a+=2,tmp));
t85=C_mutate((C_word*)lf[540]+1 /* (set! chicken.compiler.support#print-usage ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18140,tmp=(C_word)a,a+=2,tmp));
t86=C_mutate((C_word*)lf[542]+1 /* (set! chicken.compiler.support#print-debug-options ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_18152,tmp=(C_word)a,a+=2,tmp));
t87=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t87;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t87+1)))(2,av2);}}

/* chicken.compiler.support#make-node in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8467,c,av);}
a=C_alloc(5);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_record4(&a,4,lf[141],t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.compiler.support#varnode in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_8473,c,av);}
a=C_alloc(8);
t3=C_a_i_list1(&a,1,t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[154],t3,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#qnode in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_8488,c,av);}
a=C_alloc(8);
t3=C_a_i_list1(&a,1,t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[85],t3,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_8503,c,av);}
a=C_alloc(12);
t3=C_fix(0);
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8506,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9690,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:578: walk */
t9=((C_word*)t6)[1];
f_8506(t9,t8,t2);}

/* walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8506(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_8506,3,t0,t1,t2);}
a=C_alloc(20);
if(C_truep(C_i_symbolp(t2))){
/* support.scm:488: varnode */
t3=*((C_word*)lf[153]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_i_structurep(t2,lf[141]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
/* support.scm:490: bomb */
t4=*((C_word*)lf[13]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[157];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_symbolp(t4))){
t5=C_u_i_car(t2);
t6=C_eqp(t5,lf[158]);
t7=(C_truep(t6)?t6:C_eqp(t5,lf[159]));
if(C_truep(t7)){
t8=C_u_i_car(t2);
t9=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t10=t9;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=((C_word*)t11)[1];
t13=C_u_i_cdr(t2);
t14=C_i_check_list_2(t13,lf[125]);
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8573,a[2]=t1,a[3]=t8,tmp=(C_word)a,a+=4,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8575,a[2]=t11,a[3]=t17,a[4]=((C_word*)t0)[2],a[5]=t12,tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_8575(t19,t15,t13);}
else{
t8=C_eqp(t5,lf[85]);
if(C_truep(t8)){
t9=C_i_cadr(t2);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8623,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8626,a[2]=t10,a[3]=t9,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_numberp(t9))){
t12=C_eqp(lf[164],*((C_word*)lf[8]+1));
t13=t11;
f_8626(t13,(C_truep(t12)?C_i_not(C_i_integerp(t9)):C_SCHEME_FALSE));}
else{
t12=t11;
f_8626(t12,C_SCHEME_FALSE);}}
else{
t9=C_eqp(t5,lf[96]);
if(C_truep(t9)){
t10=C_i_cadr(t2);
t11=C_i_caddr(t2);
if(C_truep(C_i_nullp(t10))){
/* support.scm:508: walk */
t22=t1;
t23=t11;
t1=t22;
t2=t23;
goto loop;}
else{
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8683,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t11,tmp=(C_word)a,a+=6,tmp);
t13=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t14=t13;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=((C_word*)t15)[1];
t17=C_i_check_list_2(t10,lf[125]);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6109,a[2]=t15,a[3]=t19,a[4]=t16,tmp=(C_word)a,a+=5,tmp));
t21=((C_word*)t19)[1];
f_6109(t21,t12,t10);}}
else{
t10=C_eqp(t5,lf[165]);
t11=(C_truep(t10)?t10:C_eqp(t5,lf[118]));
if(C_truep(t11)){
t12=C_i_cadr(t2);
t13=C_a_i_list1(&a,1,t12);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8780,a[2]=t1,a[3]=t13,tmp=(C_word)a,a+=4,tmp);
/* support.scm:514: walk */
t22=t14;
t23=C_i_caddr(t2);
t1=t22;
t2=t23;
goto loop;}
else{
t12=C_eqp(t5,lf[166]);
if(C_truep(t12)){
t13=C_i_cadr(t2);
t14=C_i_caddr(t2);
t15=C_a_i_list2(&a,2,t13,t14);
t16=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8814,a[2]=t1,a[3]=t15,tmp=(C_word)a,a+=4,tmp);
/* support.scm:518: walk */
t22=t16;
t23=C_i_cadddr(t2);
t1=t22;
t2=t23;
goto loop;}
else{
t13=C_eqp(t5,lf[167]);
if(C_truep(t13)){
t14=C_i_cdddr(t2);
t15=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8968,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t14,tmp=(C_word)a,a+=6,tmp);
t16=C_u_i_cdr(t2);
t17=C_u_i_cdr(t16);
/* support.scm:521: walk */
t22=t15;
t23=C_u_i_car(t17);
t1=t22;
t2=t23;
goto loop;}
else{
t14=C_eqp(t5,lf[171]);
if(C_truep(t14)){
t15=C_i_cadr(t2);
t16=C_u_i_car(t2);
t17=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8995,a[2]=t2,a[3]=t1,a[4]=t16,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(t15))){
t18=C_eqp(lf[85],C_u_i_car(t15));
if(C_truep(t18)){
t19=C_i_cadr(t15);
t20=t17;
f_8995(t20,C_a_i_list1(&a,1,t19));}
else{
t19=t17;
f_8995(t19,C_a_i_list1(&a,1,t15));}}
else{
t18=t17;
f_8995(t18,C_a_i_list1(&a,1,t15));}}
else{
t15=C_eqp(t5,lf[172]);
t16=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9068,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t5,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t15)){
t17=t16;
f_9068(t17,t15);}
else{
t17=C_eqp(t5,lf[188]);
t18=t16;
f_9068(t18,(C_truep(t17)?t17:C_eqp(t5,lf[189])));}}}}}}}}}
else{
t5=C_a_i_list1(&a,1,C_SCHEME_FALSE);
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9645,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9647,a[2]=t8,a[3]=t12,a[4]=((C_word*)t0)[2],a[5]=t9,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_9647(t14,t10,t2);}}}}}

/* k8571 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8573(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8573,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],C_SCHEME_END_OF_LIST,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1366 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8575(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8575,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8600,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:493: g1372 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8598 in map-loop1366 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8600,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8575(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8621 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8623,c,av);}
/* support.scm:496: qnode */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k8624 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8626(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_8626,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8629,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* support.scm:500: chicken.base#warning */
t3=*((C_word*)lf[162]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[163];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* support.scm:496: qnode */
t2=*((C_word*)lf[155]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* k8627 in k8624 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8629(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8629,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8636,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* support.scm:502: scheme#truncate */
t3=*((C_word*)lf[161]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8634 in k8627 in k8624 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8636,c,av);}
/* support.scm:502: scheme#inexact->exact */
t2=*((C_word*)lf[160]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,3)))){
C_save_and_reclaim((void *)f_8683,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8687,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8692,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t8=C_i_cadr(((C_word*)t0)[4]);
t9=C_i_check_list_2(t8,lf[125]);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8709,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8719,a[2]=t5,a[3]=t12,a[4]=t7,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_8719(t14,t10,t8);}

/* k8685 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8687,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[96],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g1409 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8692(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_8692,3,t0,t1,t2);}
/* support.scm:511: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8506(t3,t1,C_i_cadr(t2));}

/* k8707 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8709,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8717,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* support.scm:512: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8506(t3,t2,((C_word*)t0)[4]);}

/* k8715 in k8707 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8717,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* support.scm:511: scheme#append */
t3=*((C_word*)lf[58]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* map-loop1403 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8719(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8719,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8744,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:511: g1409 */
t4=((C_word*)t0)[4];
f_8692(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8742 in map-loop1403 in k8681 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8744,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8719(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8778 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_8780,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[165],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k8812 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_8814,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[166],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8845(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_8845,5,t0,t1,t2,t3,t4);}
a=C_alloc(14);
if(C_truep(C_i_nullp(t2))){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8889,a[2]=t5,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* support.scm:525: scheme#reverse */
t7=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=C_i_caar(t2);
t6=C_eqp(lf[168],t5);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[2]);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8931,a[2]=t7,a[3]=t1,a[4]=t4,a[5]=((C_word*)t0)[3],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t9=C_a_i_cons(&a,2,lf[170],t3);
/* support.scm:531: scheme#reverse */
t10=*((C_word*)lf[80]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t8;
av2[2]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t7=C_u_i_cdr(t2);
t8=C_i_caar(t2);
t9=C_a_i_cons(&a,2,t8,t3);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8952,a[2]=t4,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t7,a[6]=t9,tmp=(C_word)a,a+=7,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8956,a[2]=((C_word*)t0)[3],a[3]=t10,tmp=(C_word)a,a+=4,tmp);
/* support.scm:535: scheme#cadar */
t12=*((C_word*)lf[169]+1);{
C_word av2[3];
av2[0]=t12;
av2[1]=t11;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}}

/* k8866 in k8887 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8868(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8868,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[167],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8887 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8889(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_8889,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8868,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_record4(&a,4,lf[141],lf[159],C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[4]);
/* support.scm:526: scheme#reverse */
t6=*((C_word*)lf[80]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k8909 in k8929 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8911,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[167],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8917 in k8929 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8919,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* support.scm:532: scheme#reverse */
t3=*((C_word*)lf[80]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8921 in k8929 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8923,c,av);}
/* support.scm:532: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8506(t2,((C_word*)t0)[3],t1);}

/* k8929 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_8931,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8911,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8919,a[2]=((C_word*)t0)[4],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8923,a[2]=((C_word*)t0)[5],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* support.scm:532: scheme#cadar */
t6=*((C_word*)lf[169]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k8950 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_8952,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* support.scm:533: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8845(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2);}

/* k8954 in loop in k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8956,c,av);}
/* support.scm:535: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8506(t2,((C_word*)t0)[3],t1);}

/* k8966 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_8968(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_8968,c,av);}
a=C_alloc(10);
t2=C_a_i_list1(&a,1,t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8845,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t4,tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_8845(t6,((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_END_OF_LIST,t2);}

/* k8993 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_8995(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_8995,2,t0,t1);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_u_i_cdr(t6);
t8=C_i_check_list_2(t7,lf[125]);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9007,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9009,a[2]=t4,a[3]=t11,a[4]=((C_word*)t0)[5],a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_9009(t13,t9,t7);}

/* k9005 in k8993 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9007(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9007,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1474 in k8993 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9009(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9009,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9034,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:541: g1480 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9032 in map-loop1474 in k8993 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9034,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9009(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9068(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_9068,2,t0,t1);}
a=C_alloc(21);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_a_i_list1(&a,1,t3);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_u_i_cdr(((C_word*)t0)[2]);
t10=C_u_i_cdr(t9);
t11=C_i_check_list_2(t10,lf[125]);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9094,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9096,a[2]=t7,a[3]=t14,a[4]=((C_word*)t0)[4],a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_9096(t16,t12,t10);}
else{
t2=C_eqp(((C_word*)t0)[5],lf[173]);
if(C_truep(t2)){
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_u_i_cdr(((C_word*)t0)[2]);
t5=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t5;
av2[1]=C_a_i_record4(&a,4,lf[141],t3,t4,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t3=C_eqp(((C_word*)t0)[5],lf[174]);
if(C_truep(t3)){
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_a_i_list2(&a,2,t4,C_SCHEME_TRUE);
t6=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[174],t5,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[5],lf[124]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[5],lf[175]));
if(C_truep(t5)){
t6=C_i_cadr(((C_word*)t0)[2]);
t7=C_a_i_list1(&a,1,t6);
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=C_u_i_cdr(((C_word*)t0)[2]);
t13=C_u_i_cdr(t12);
t14=C_i_check_list_2(t13,lf[125]);
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9205,a[2]=((C_word*)t0)[3],a[3]=t7,tmp=(C_word)a,a+=4,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9207,a[2]=t10,a[3]=t17,a[4]=((C_word*)t0)[4],a[5]=t11,tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_9207(t19,t15,t13);}
else{
t6=C_eqp(((C_word*)t0)[5],lf[176]);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[2]);
t8=C_i_cadr(t7);
t9=C_i_caddr(((C_word*)t0)[2]);
t10=C_i_cadr(t9);
t11=C_i_cadddr(((C_word*)t0)[2]);
t12=C_i_cadr(t11);
t13=C_i_cddddr(((C_word*)t0)[2]);
t14=C_i_car(t13);
t15=C_i_cadr(t14);
t16=C_a_i_list4(&a,4,t8,t10,t12,t15);
t17=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9272,a[2]=((C_word*)t0)[3],a[3]=t16,tmp=(C_word)a,a+=4,tmp);
/* support.scm:557: walk */
t18=((C_word*)((C_word*)t0)[4])[1];
f_8506(t18,t17,C_i_list_ref(((C_word*)t0)[2],C_fix(5)));}
else{
t7=C_eqp(((C_word*)t0)[5],lf[177]);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9313,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t7)){
t9=t8;
f_9313(t9,t7);}
else{
t9=C_eqp(((C_word*)t0)[5],lf[184]);
if(C_truep(t9)){
t10=t8;
f_9313(t10,t9);}
else{
t10=C_eqp(((C_word*)t0)[5],lf[185]);
if(C_truep(t10)){
t11=t8;
f_9313(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[5],lf[186]);
t12=t8;
f_9313(t12,(C_truep(t11)?t11:C_eqp(((C_word*)t0)[5],lf[187])));}}}}}}}}}

/* k9092 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9094,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1511 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9096(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9096,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9121,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:543: g1517 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9119 in map-loop1511 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9121,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9096(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9203 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9205,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[124],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1555 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9207(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9207,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9232,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:551: g1561 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9230 in map-loop1555 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9232(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9232,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9207(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9270 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9272(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_9272,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[176],((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9313(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,4)))){
C_save_and_reclaim_args((void *)trf_9313,2,t0,t1);}
a=C_alloc(20);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_i_cddr(((C_word*)t0)[2]);
t9=C_i_check_list_2(t8,lf[125]);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9342,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9344,a[2]=t6,a[3]=t12,a[4]=((C_word*)t0)[4],a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_9344(t14,t10,t8);}
else{
t2=C_eqp(((C_word*)t0)[5],lf[178]);
if(C_truep(t2)){
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_u_i_cdr(((C_word*)t0)[2]);
t9=C_i_check_list_2(t8,lf[125]);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9405,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9407,a[2]=t6,a[3]=t12,a[4]=((C_word*)t0)[4],a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_9407(t14,t10,t8);}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9444,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9450,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* support.scm:564: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}}}

/* k9340 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9342,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1604 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9344(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9344,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9369,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:560: g1610 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9367 in map-loop1604 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9369,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9344(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9403 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9405,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[179],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1635 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9407(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9407,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9432,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:562: g1641 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9430 in map-loop1635 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9432(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9432,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9407(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a9443 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9444,c,av);}
/* support.scm:564: get-line-2 */
t2=*((C_word*)lf[137]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_9450,c,av);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9511,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9539,a[2]=((C_word*)t0)[4],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* tweaks.scm:60: ##sys#get */
t6=*((C_word*)lf[182]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=lf[183];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k9471 in k9513 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9473,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[179],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1681 in k9513 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9475(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9475,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9500,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:576: g1687 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9498 in map-loop1681 in k9513 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 in ... */
static void C_ccall f_9500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9500,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9475(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9511(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_9511,2,t0,t1);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9515,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[5])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9518,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* support.scm:572: real-name */
t4=*((C_word*)lf[181]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* support.scm:575: ##sys#symbol->string */
t3=*((C_word*)lf[180]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k9513 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_9515,c,av);}
a=C_alloc(23);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[3],lf[125]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9473,a[2]=((C_word*)t0)[4],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9475,a[2]=t5,a[3]=t10,a[4]=((C_word*)t0)[5],a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_9475(t12,t8,((C_word*)t0)[3]);}

/* k9516 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_9518,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9525,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
f_9515(2,av2);}}
else{
/* support.scm:574: ##sys#symbol->string */
t3=*((C_word*)lf[180]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k9523 in k9516 in k9509 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_9525,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
f_9515(2,av2);}}

/* k9537 in a9449 in k9311 in k9066 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_9539,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t2,C_fix(1)));
t4=((C_word*)t0)[3];
f_9511(t4,C_SCHEME_TRUE);}
else{
t2=((C_word*)t0)[3];
f_9511(t2,C_SCHEME_FALSE);}}

/* k9643 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_9645,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record4(&a,4,lf[141],lf[179],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1712 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9647(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9647,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9672,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:577: g1718 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9670 in map-loop1712 in walk in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9672,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9647(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9688 in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_9690,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9693,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
if(C_truep(C_i_positivep(t3))){
/* support.scm:580: debugging */
t4=*((C_word*)lf[22]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[190];
av2[3]=lf[191];
av2[4]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9691 in k9688 in chicken.compiler.support#build-node-graph in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9693(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9693,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9699,c,av);}
a=C_alloc(5);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9705,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_9705(t6,t1,t2);}

/* walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9705(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_9705,3,t0,t1,t2);}
a=C_alloc(7);
t3=C_slot(t2,C_fix(3));
t4=C_slot(t2,C_fix(2));
t5=C_slot(t2,C_fix(1));
t6=C_eqp(t5,lf[158]);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9739,a[2]=t3,a[3]=t1,a[4]=t5,a[5]=((C_word*)t0)[2],a[6]=t4,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t6)){
t8=t7;
f_9739(t8,t6);}
else{
t8=C_eqp(t5,lf[203]);
t9=t7;
f_9739(t9,(C_truep(t8)?t8:C_eqp(t5,lf[204])));}}

/* k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9739(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,5)))){
C_save_and_reclaim_args((void *)trf_9739,2,t0,t1);}
a=C_alloc(20);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9752,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9754,a[2]=t4,a[3]=t9,a[4]=((C_word*)t0)[5],a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_9754(t11,t7,((C_word*)t0)[2]);}
else{
t2=C_eqp(((C_word*)t0)[4],lf[193]);
if(C_truep(t2)){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9809,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9811,a[2]=t5,a[3]=t10,a[4]=((C_word*)t0)[5],a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_9811(t12,t8,((C_word*)t0)[2]);}
else{
t3=C_eqp(((C_word*)t0)[4],lf[154]);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_car(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[85]);
if(C_truep(t4)){
t5=C_i_car(((C_word*)t0)[6]);
t6=C_booleanp(t5);
if(C_truep(t6)){
if(C_truep(t6)){
t7=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t7;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_u_i_car(((C_word*)t0)[6]);
t8=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_list(&a,2,lf[85],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t7=C_i_stringp(t5);
if(C_truep(t7)){
if(C_truep(t7)){
t8=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t8;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_u_i_car(((C_word*)t0)[6]);
t9=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t9;
av2[1]=C_a_i_list(&a,2,lf[85],t8);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t8=C_i_numberp(t5);
t9=(C_truep(t8)?t8:C_charp(t5));
if(C_truep(t9)){
t10=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t10;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_u_i_car(((C_word*)t0)[6]);
t11=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t11;
av2[1]=C_a_i_list(&a,2,lf[85],t10);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}}
else{
t5=C_eqp(((C_word*)t0)[4],lf[96]);
if(C_truep(t5)){
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t11=t10;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=((C_word*)t12)[1];
t14=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9909,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=t8,a[7]=t9,a[8]=t12,a[9]=t13,tmp=(C_word)a,a+=10,tmp);
/* support.scm:599: chicken.base#butlast */
t15=*((C_word*)lf[194]+1);{
C_word av2[3];
av2[0]=t15;
av2[1]=t14;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t15+1)))(3,av2);}}
else{
t6=C_eqp(((C_word*)t0)[4],lf[118]);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[6]);
t8=(C_truep(t7)?lf[165]:lf[118]);
t9=C_i_caddr(((C_word*)t0)[6]);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10032,a[2]=((C_word*)t0)[3],a[3]=t8,a[4]=t9,tmp=(C_word)a,a+=5,tmp);
/* support.scm:606: walk */
t11=((C_word*)((C_word*)t0)[5])[1];
f_9705(t11,t10,C_i_car(((C_word*)t0)[2]));}
else{
t7=C_eqp(((C_word*)t0)[4],lf[166]);
if(C_truep(t7)){
t8=C_i_car(((C_word*)t0)[6]);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10056,a[2]=((C_word*)t0)[3],a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* support.scm:608: walk */
t10=((C_word*)((C_word*)t0)[5])[1];
f_9705(t10,t9,C_i_car(((C_word*)t0)[2]));}
else{
t8=C_eqp(((C_word*)t0)[4],lf[196]);
if(C_truep(t8)){
/* support.scm:610: walk */
t9=((C_word*)((C_word*)t0)[5])[1];
f_9705(t9,((C_word*)t0)[3],C_i_car(((C_word*)t0)[2]));}
else{
t9=C_eqp(((C_word*)t0)[4],lf[167]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10090,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* support.scm:613: walk */
t11=((C_word*)((C_word*)t0)[5])[1];
f_9705(t11,t10,C_i_car(((C_word*)t0)[2]));}
else{
t10=C_eqp(((C_word*)t0)[4],lf[179]);
if(C_truep(t10)){
t11=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t12=t11;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=((C_word*)t13)[1];
t15=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10180,a[2]=t13,a[3]=t17,a[4]=((C_word*)t0)[5],a[5]=t14,tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_10180(t19,((C_word*)t0)[3],((C_word*)t0)[2]);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[189]);
if(C_truep(t11)){
t12=C_i_car(((C_word*)t0)[6]);
t13=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t14=t13;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=((C_word*)t15)[1];
t17=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t18=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10235,a[2]=((C_word*)t0)[3],a[3]=t12,tmp=(C_word)a,a+=4,tmp);
t19=C_SCHEME_UNDEFINED;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_set_block_item(t20,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10237,a[2]=t15,a[3]=t20,a[4]=((C_word*)t0)[5],a[5]=t16,tmp=(C_word)a,a+=6,tmp));
t22=((C_word*)t20)[1];
f_10237(t22,t18,((C_word*)t0)[2]);}
else{
t12=C_eqp(((C_word*)t0)[4],lf[159]);
if(C_truep(t12)){
t13=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t13;
av2[1]=C_a_i_list1(&a,1,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t13=C_eqp(((C_word*)t0)[4],lf[198]);
if(C_truep(t13)){
t14=C_i_car(((C_word*)t0)[6]);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10293,a[2]=((C_word*)t0)[5],a[3]=t16,tmp=(C_word)a,a+=4,tmp));
t18=((C_word*)t16)[1];
f_10293(t18,((C_word*)t0)[3],t14,((C_word*)t0)[2],C_SCHEME_END_OF_LIST);}
else{
t14=C_eqp(((C_word*)t0)[4],lf[199]);
t15=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10343,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t14)){
t16=t15;
f_10343(t16,t14);}
else{
t16=C_eqp(((C_word*)t0)[4],lf[200]);
if(C_truep(t16)){
t17=t15;
f_10343(t17,t16);}
else{
t17=C_eqp(((C_word*)t0)[4],lf[201]);
t18=t15;
f_10343(t18,(C_truep(t17)?t17:C_eqp(((C_word*)t0)[4],lf[202])));}}}}}}}}}}}}}}}}

/* k9750 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9752,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1769 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9754(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9754,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9779,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:589: g1775 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9777 in map-loop1769 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9779,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9754(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9807 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9809(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_9809,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[193],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop1798 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9811(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9811,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9836,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:591: g1804 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9834 in map-loop1798 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9836,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9811(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9909(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_9909,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9912,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9979,a[2]=((C_word*)t0)[8],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_9979(t6,t2,t1);}

/* k9910 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_9912,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[125]);
t3=C_i_check_list_2(t1,lf[125]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9921,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9931,a[2]=((C_word*)t0)[6],a[3]=t6,a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_9931(t8,t4,((C_word*)t0)[2],t1);}

/* k9919 in k9910 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_9921,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9925,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9929,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* support.scm:600: last */
f_6143(t3,((C_word*)t0)[4]);}

/* k9923 in k9919 in k9910 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_9925,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[96],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k9927 in k9919 in k9910 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_ccall f_9929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9929,c,av);}
/* support.scm:600: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9705(t2,((C_word*)t0)[3],t1);}

/* map-loop1840 in k9910 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9931(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_9931,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_list2(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* map-loop1861 in k9907 in k9737 in walk in chicken.compiler.support#build-expression-tree in k8463 in k7810 in k7807 in k6682 in k5370 in k5367 in k5364 in k5361 in k5358 in k5355 in k5352 in k5349 in k5346 */
static void C_fcall f_9979(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9979,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10004,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* support.scm:599: g1867 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_9705(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_support_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("support"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_support_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(4077))){
C_save(t1);
C_rereclaim2(4077*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,550);
lf[0]=C_h_intern(&lf[0],7, C_text("support"));
lf[1]=C_h_intern(&lf[1],25, C_text("chicken.compiler.support#"));
lf[8]=C_h_intern(&lf[8],36, C_text("chicken.compiler.support#number-type"));
lf[9]=C_h_intern(&lf[9],7, C_text("generic"));
lf[10]=C_h_intern(&lf[10],31, C_text("chicken.compiler.support#unsafe"));
lf[11]=C_h_intern(&lf[11],46, C_text("chicken.compiler.support#compiler-cleanup-hook"));
lf[12]=C_h_intern(&lf[12],42, C_text("chicken.compiler.support#debugging-chicken"));
lf[13]=C_h_intern(&lf[13],29, C_text("chicken.compiler.support#bomb"));
lf[14]=C_h_intern(&lf[14],18, C_text("chicken.base#error"));
lf[15]=C_h_intern(&lf[15],20, C_text("scheme#string-append"));
lf[16]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032[internal compiler error] "));
lf[17]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031[internal compiler error]"));
lf[18]=C_h_intern(&lf[18],51, C_text("chicken.compiler.support#collected-debugging-output"));
lf[20]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001x\376\003\000\000\002\376\001\000\000\001\001S\376\377\016"));
lf[22]=C_h_intern(&lf[22],34, C_text("chicken.compiler.support#debugging"));
lf[23]=C_h_intern(&lf[23],14, C_text("scheme#newline"));
lf[24]=C_h_intern(&lf[24],21, C_text("##sys#standard-output"));
lf[25]=C_h_intern(&lf[25],6, C_text("printf"));
lf[26]=C_h_intern(&lf[26],18, C_text("##sys#write-char-0"));
lf[27]=C_h_intern(&lf[27],11, C_text("##sys#print"));
lf[28]=C_h_intern(&lf[28],12, C_text("scheme#force"));
lf[29]=C_h_intern(&lf[29],14, C_text("scheme#display"));
lf[30]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[31]=C_h_intern(&lf[31],34, C_text("chicken.port#with-output-to-string"));
lf[32]=C_h_intern(&lf[32],7, C_text("fprintf"));
lf[33]=C_h_intern(&lf[33],25, C_text("chicken.base#flush-output"));
lf[34]=C_h_intern(&lf[34],46, C_text("chicken.compiler.support#with-debugging-output"));
lf[35]=C_h_intern(&lf[35],27, C_text("chicken.string#string-split"));
lf[36]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[37]=C_h_intern(&lf[37],39, C_text("chicken.compiler.support#quit-compiling"));
lf[38]=C_h_intern(&lf[38],20, C_text("##sys#standard-error"));
lf[39]=C_h_intern(&lf[39],17, C_text("chicken.base#exit"));
lf[40]=C_h_intern(&lf[40],22, C_text("chicken.format#fprintf"));
lf[41]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010\012Error: "));
lf[42]=C_h_intern(&lf[42],23, C_text("##sys#syntax-error-hook"));
lf[43]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005\011~s~%"));
lf[44]=C_h_intern(&lf[44],8, C_text("for-each"));
lf[45]=C_h_intern(&lf[45],29, C_text("chicken.base#print-call-chain"));
lf[46]=C_h_intern(&lf[46],20, C_text("##sys#current-thread"));
lf[47]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025\012\011Expansion history:\012"));
lf[48]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003): "));
lf[49]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017\012Syntax error ("));
lf[50]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017\012Syntax error: "));
lf[51]=C_h_intern(&lf[51],27, C_text("chicken.syntax#syntax-error"));
lf[52]=C_h_intern(&lf[52],47, C_text("chicken.compiler.support#emit-syntax-trace-info"));
lf[53]=C_h_intern(&lf[53],40, C_text("chicken.compiler.support#check-signature"));
lf[54]=C_h_intern(&lf[54],42, C_text("chicken.compiler.support#build-lambda-list"));
lf[55]=C_h_intern(&lf[55],37, C_text("chicken.compiler.support#c-ify-string"));
lf[56]=C_h_intern(&lf[56],18, C_text("##sys#list->string"));
lf[57]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000\042\376\377\016"));
lf[58]=C_h_intern(&lf[58],13, C_text("scheme#append"));
lf[59]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000\134\376\377\016"));
lf[60]=C_h_intern(&lf[60],18, C_text("##sys#string->list"));
lf[61]=C_h_intern(&lf[61],20, C_text("##sys#fixnum->string"));
lf[62]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\0000\376\003\000\000\002\376\377\012\000\0000\376\377\016"));
lf[63]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\0000\376\377\016"));
lf[64]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000\042\376\003\000\000\002\376\377\012\000\000\047\376\003\000\000\002\376\377\012\000\000\134\376\003\000\000\002\376\377\012\000\000\077\376\003\000\000\002\376\377\012\000\000\052\376\377\016"));
lf[65]=C_h_intern(&lf[65],44, C_text("chicken.compiler.support#valid-c-identifier\077"));
lf[66]=C_h_intern(&lf[66],23, C_text("chicken.string#->string"));
lf[67]=C_h_intern(&lf[67],37, C_text("chicken.compiler.support#bytes->words"));
lf[68]=C_h_intern(&lf[68],37, C_text("chicken.compiler.support#words->bytes"));
lf[69]=C_h_intern(&lf[69],50, C_text("chicken.compiler.support#check-and-open-input-file"));
lf[70]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001-"));
lf[71]=C_h_intern(&lf[71],20, C_text("##sys#standard-input"));
lf[72]=C_h_intern(&lf[72],22, C_text("scheme#open-input-file"));
lf[73]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024Can not open file ~s"));
lf[74]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031(~a) can not open file ~s"));
lf[75]=C_h_intern(&lf[75],25, C_text("chicken.file#file-exists\077"));
lf[76]=C_h_intern(&lf[76],49, C_text("chicken.compiler.support#close-checked-input-file"));
lf[77]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001-"));
lf[78]=C_h_intern(&lf[78],23, C_text("scheme#close-input-port"));
lf[79]=C_h_intern(&lf[79],35, C_text("chicken.compiler.support#fold-inner"));
lf[80]=C_h_intern(&lf[80],14, C_text("scheme#reverse"));
lf[82]=C_h_intern(&lf[82],41, C_text("chicken.compiler.support#read-expressions"));
lf[83]=C_h_intern(&lf[83],11, C_text("scheme#read"));
lf[84]=C_h_intern(&lf[84],34, C_text("chicken.compiler.support#constant\077"));
lf[85]=C_h_intern(&lf[85],5, C_text("quote"));
lf[86]=C_h_intern(&lf[86],20, C_text("##sys#srfi-4-vector\077"));
lf[87]=C_h_intern(&lf[87],18, C_text("chicken.blob#blob\077"));
lf[88]=C_h_intern(&lf[88],45, C_text("chicken.compiler.support#collapsable-literal\077"));
lf[89]=C_h_intern(&lf[89],35, C_text("chicken.compiler.support#immediate\077"));
lf[90]=C_h_intern(&lf[90],36, C_text("chicken.compiler.support#big-fixnum\077"));
lf[91]=C_h_intern(&lf[91],39, C_text("chicken.compiler.support#basic-literal\077"));
lf[92]=C_h_intern(&lf[92],19, C_text("scheme#vector->list"));
lf[93]=C_h_intern(&lf[93],48, C_text("chicken.compiler.support#canonicalize-begin-body"));
lf[94]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[95]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[96]=C_h_intern(&lf[96],3, C_text("let"));
lf[97]=C_h_intern(&lf[97],19, C_text("chicken.base#gensym"));
lf[98]=C_h_intern(&lf[98],1, C_text("t"));
lf[99]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\012\001##sys#void\376\377\016"));
lf[100]=C_h_intern(&lf[100],37, C_text("chicken.compiler.support#string->expr"));
lf[101]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042cannot parse expression: ~s [~a]~%"));
lf[102]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[103]=C_h_intern(&lf[103],5, C_text("begin"));
lf[104]=C_h_intern(&lf[104],13, C_text("scheme#values"));
lf[105]=C_h_intern(&lf[105],35, C_text("chicken.port#with-input-from-string"));
lf[106]=C_h_intern(&lf[106],40, C_text("chicken.condition#with-exception-handler"));
lf[107]=C_h_intern(&lf[107],37, C_text("scheme#call-with-current-continuation"));
lf[108]=C_h_intern(&lf[108],37, C_text("chicken.compiler.support#llist-length"));
lf[109]=C_h_intern(&lf[109],37, C_text("chicken.compiler.support#llist-match\077"));
lf[111]=C_h_intern(&lf[111],56, C_text("chicken.compiler.support#reset-profile-info-vector-name!"));
lf[112]=C_h_intern(&lf[112],41, C_text("chicken.compiler.support#make-random-name"));
lf[113]=C_h_intern(&lf[113],12, C_text("profile-info"));
lf[116]=C_h_intern(&lf[116],46, C_text("chicken.compiler.support#expand-profile-lambda"));
lf[117]=C_h_intern(&lf[117],19, C_text("##sys#profile-entry"));
lf[118]=C_h_intern(&lf[118],13, C_text("##core#lambda"));
lf[119]=C_h_intern(&lf[119],11, C_text("##sys#apply"));
lf[120]=C_h_intern(&lf[120],18, C_text("##sys#profile-exit"));
lf[121]=C_h_intern(&lf[121],18, C_text("##sys#dynamic-wind"));
lf[122]=C_h_intern(&lf[122],47, C_text("chicken.compiler.support#profiling-prelude-exps"));
lf[123]=C_h_intern(&lf[123],27, C_text("##sys#register-profile-info"));
lf[124]=C_h_intern(&lf[124],4, C_text("set!"));
lf[125]=C_h_intern(&lf[125],3, C_text("map"));
lf[126]=C_h_intern(&lf[126],30, C_text("##sys#set-profile-info-vector!"));
lf[127]=C_h_intern(&lf[127],31, C_text("chicken.compiler.support#db-get"));
lf[128]=C_h_intern(&lf[128],31, C_text("chicken.internal#hash-table-ref"));
lf[129]=C_h_intern(&lf[129],35, C_text("chicken.compiler.support#db-get-all"));
lf[130]=C_h_intern(&lf[130],5, C_text("foldr"));
lf[131]=C_h_intern(&lf[131],32, C_text("chicken.compiler.support#db-put!"));
lf[132]=C_h_intern(&lf[132],32, C_text("chicken.internal#hash-table-set!"));
lf[133]=C_h_intern(&lf[133],33, C_text("chicken.compiler.support#collect!"));
lf[134]=C_h_intern(&lf[134],36, C_text("chicken.compiler.support#db-get-list"));
lf[135]=C_h_intern(&lf[135],33, C_text("chicken.compiler.support#get-line"));
lf[136]=C_h_intern(&lf[136],26, C_text("##sys#line-number-database"));
lf[137]=C_h_intern(&lf[137],35, C_text("chicken.compiler.support#get-line-2"));
lf[138]=C_h_intern(&lf[138],53, C_text("chicken.compiler.support#display-line-number-database"));
lf[139]=C_h_intern(&lf[139],36, C_text("chicken.internal#hash-table-for-each"));
lf[140]=C_h_intern(&lf[140],34, C_text("chicken.compiler.support#make-node"));
lf[141]=C_h_intern(&lf[141],29, C_text("chicken.compiler.support#node"));
lf[142]=C_h_intern(&lf[142],30, C_text("chicken.compiler.support#node\077"));
lf[143]=C_h_intern(&lf[143],35, C_text("chicken.compiler.support#node-class"));
lf[144]=C_h_intern(&lf[144],10, C_text("node-class"));
lf[145]=C_h_intern(&lf[145],40, C_text("chicken.compiler.support#node-class-set!"));
lf[146]=C_h_intern(&lf[146],16, C_text("##sys#block-set!"));
lf[147]=C_h_intern(&lf[147],40, C_text("chicken.compiler.support#node-parameters"));
lf[148]=C_h_intern(&lf[148],15, C_text("node-parameters"));
lf[149]=C_h_intern(&lf[149],45, C_text("chicken.compiler.support#node-parameters-set!"));
lf[150]=C_h_intern(&lf[150],44, C_text("chicken.compiler.support#node-subexpressions"));
lf[151]=C_h_intern(&lf[151],19, C_text("node-subexpressions"));
lf[152]=C_h_intern(&lf[152],49, C_text("chicken.compiler.support#node-subexpressions-set!"));
lf[153]=C_h_intern(&lf[153],32, C_text("chicken.compiler.support#varnode"));
lf[154]=C_h_intern(&lf[154],15, C_text("##core#variable"));
lf[155]=C_h_intern(&lf[155],30, C_text("chicken.compiler.support#qnode"));
lf[156]=C_h_intern(&lf[156],41, C_text("chicken.compiler.support#build-node-graph"));
lf[157]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016bad expression"));
lf[158]=C_h_intern(&lf[158],2, C_text("if"));
lf[159]=C_h_intern(&lf[159],16, C_text("##core#undefined"));
lf[160]=C_h_intern(&lf[160],21, C_text("scheme#inexact->exact"));
lf[161]=C_h_intern(&lf[161],15, C_text("scheme#truncate"));
lf[162]=C_h_intern(&lf[162],20, C_text("chicken.base#warning"));
lf[163]=C_decode_literal(C_heaptop,C_text("\376B\000\0006literal is out of range - will be truncated to integer"));
lf[164]=C_h_intern(&lf[164],6, C_text("fixnum"));
lf[165]=C_h_intern(&lf[165],6, C_text("lambda"));
lf[166]=C_h_intern(&lf[166],10, C_text("##core#the"));
lf[167]=C_h_intern(&lf[167],15, C_text("##core#typecase"));
lf[168]=C_h_intern(&lf[168],4, C_text("else"));
lf[169]=C_h_intern(&lf[169],12, C_text("scheme#cadar"));
lf[170]=C_h_intern(&lf[170],1, C_text("\052"));
lf[171]=C_h_intern(&lf[171],16, C_text("##core#primitive"));
lf[172]=C_h_intern(&lf[172],13, C_text("##core#inline"));
lf[173]=C_h_intern(&lf[173],18, C_text("##core#debug-event"));
lf[174]=C_h_intern(&lf[174],11, C_text("##core#proc"));
lf[175]=C_h_intern(&lf[175],11, C_text("##core#set!"));
lf[176]=C_h_intern(&lf[176],31, C_text("##core#foreign-callback-wrapper"));
lf[177]=C_h_intern(&lf[177],22, C_text("##core#inline_allocate"));
lf[178]=C_h_intern(&lf[178],10, C_text("##core#app"));
lf[179]=C_h_intern(&lf[179],11, C_text("##core#call"));
lf[180]=C_h_intern(&lf[180],20, C_text("##sys#symbol->string"));
lf[181]=C_h_intern(&lf[181],34, C_text("chicken.compiler.support#real-name"));
lf[182]=C_h_intern(&lf[182],9, C_text("##sys#get"));
lf[183]=C_h_intern(&lf[183],36, C_text("##compiler#always-bound-to-procedure"));
lf[184]=C_h_intern(&lf[184],17, C_text("##core#inline_ref"));
lf[185]=C_h_intern(&lf[185],20, C_text("##core#inline_update"));
lf[186]=C_h_intern(&lf[186],21, C_text("##core#inline_loc_ref"));
lf[187]=C_h_intern(&lf[187],24, C_text("##core#inline_loc_update"));
lf[188]=C_h_intern(&lf[188],14, C_text("##core#provide"));
lf[189]=C_h_intern(&lf[189],15, C_text("##core#callunit"));
lf[190]=C_h_intern(&lf[190],1, C_text("o"));
lf[191]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033eliminated procedure checks"));
lf[192]=C_h_intern(&lf[192],46, C_text("chicken.compiler.support#build-expression-tree"));
lf[193]=C_h_intern(&lf[193],14, C_text("##core#closure"));
lf[194]=C_h_intern(&lf[194],20, C_text("chicken.base#butlast"));
lf[195]=C_h_intern(&lf[195],3, C_text("the"));
lf[196]=C_h_intern(&lf[196],17, C_text("##core#the/result"));
lf[197]=C_h_intern(&lf[197],17, C_text("compiler-typecase"));
lf[198]=C_h_intern(&lf[198],11, C_text("##core#bind"));
lf[199]=C_h_intern(&lf[199],12, C_text("##core#unbox"));
lf[200]=C_h_intern(&lf[200],10, C_text("##core#ref"));
lf[201]=C_h_intern(&lf[201],13, C_text("##core#update"));
lf[202]=C_h_intern(&lf[202],15, C_text("##core#update_i"));
lf[203]=C_h_intern(&lf[203],10, C_text("##core#box"));
lf[204]=C_h_intern(&lf[204],11, C_text("##core#cond"));
lf[205]=C_h_intern(&lf[205],37, C_text("chicken.compiler.support#fold-boolean"));
lf[206]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376B\000\000\005C_and\376\377\016"));
lf[207]=C_h_intern(&lf[207],47, C_text("chicken.compiler.support#inline-lambda-bindings"));
lf[208]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_a_i_list"));
lf[209]=C_h_intern(&lf[209],10, C_text("references"));
lf[210]=C_h_intern(&lf[210],10, C_text("rest-null\077"));
lf[211]=C_h_intern(&lf[211],8, C_text("rest-cdr"));
lf[212]=C_h_intern(&lf[212],17, C_text("derived-rest-vars"));
lf[213]=C_h_intern(&lf[213],17, C_text("##core#rest-null\077"));
lf[214]=C_h_intern(&lf[214],35, C_text("chicken.compiler.support#copy-node!"));
lf[215]=C_h_intern(&lf[215],15, C_text("##core#rest-car"));
lf[216]=C_decode_literal(C_heaptop,C_text("\376B\000\000$C_rest_arg_out_of_bounds_error_value"));
lf[217]=C_h_intern(&lf[217],15, C_text("##core#rest-cdr"));
lf[218]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_i_cdr"));
lf[219]=C_h_intern(&lf[219],4, C_text("rarg"));
lf[220]=C_h_intern(&lf[220],22, C_text("chicken.base#alist-ref"));
lf[221]=C_h_intern(&lf[221],10, C_text("scheme#eq\077"));
lf[222]=C_h_intern(&lf[222],12, C_text("contractable"));
lf[223]=C_h_intern(&lf[223],16, C_text("inline-transient"));
lf[224]=C_h_intern(&lf[224],1, C_text("f"));
lf[225]=C_h_intern(&lf[225],27, C_text("##sys#decompose-lambda-list"));
lf[226]=C_h_intern(&lf[226],34, C_text("chicken.compiler.support#tree-copy"));
lf[227]=C_h_intern(&lf[227],54, C_text("chicken.compiler.support#replace-rest-op-with-list-ops"));
lf[228]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014C_i_list_ref"));
lf[229]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_i_cdr"));
lf[230]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025C_i_greater_or_equalp"));
lf[231]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_i_length"));
lf[232]=C_h_intern(&lf[232],18, C_text("##core#rest-length"));
lf[233]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_i_length"));
lf[234]=C_decode_literal(C_heaptop,C_text("\376B\000\000gUnknown rest op node class while undoing rest op for explicitly consed rest"
" arg. This shouldn\047t happen!"));
lf[235]=C_h_intern(&lf[235],34, C_text("chicken.compiler.support#copy-node"));
lf[236]=C_h_intern(&lf[236],48, C_text("chicken.compiler.support#emit-global-inline-file"));
lf[237]=C_h_intern(&lf[237],18, C_text("chicken.base#print"));
lf[238]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[239]=C_h_intern(&lf[239],15, C_text("scheme#string<\077"));
lf[240]=C_h_intern(&lf[240],21, C_text("scheme#symbol->string"));
lf[241]=C_h_intern(&lf[241],17, C_text("chicken.sort#sort"));
lf[242]=C_h_intern(&lf[242],1, C_text("i"));
lf[243]=C_decode_literal(C_heaptop,C_text("\376B\000\0001the following procedures can be globally inlined:"));
lf[244]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015; END OF FILE"));
lf[245]=C_h_intern(&lf[245],23, C_text("chicken.pretty-print#pp"));
lf[246]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027; GENERATED BY CHICKEN "));
lf[247]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 FROM "));
lf[248]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[249]=C_h_intern(&lf[249],32, C_text("chicken.platform#chicken-version"));
lf[250]=C_h_intern(&lf[250],26, C_text("scheme#with-output-to-file"));
lf[251]=C_h_intern(&lf[251],11, C_text("local-value"));
lf[252]=C_h_intern(&lf[252],5, C_text("value"));
lf[253]=C_h_intern(&lf[253],9, C_text("inlinable"));
lf[254]=C_h_intern(&lf[254],3, C_text("yes"));
lf[255]=C_h_intern(&lf[255],2, C_text("no"));
lf[256]=C_h_intern(&lf[256],17, C_text("##compiler#inline"));
lf[257]=C_h_intern(&lf[257],11, C_text("hidden-refs"));
lf[258]=C_h_intern(&lf[258],7, C_text("unknown"));
lf[259]=C_h_intern(&lf[259],24, C_text("##compiler#inline-global"));
lf[260]=C_h_intern(&lf[260],42, C_text("chicken.compiler.support#variable-visible\077"));
lf[261]=C_h_intern(&lf[261],41, C_text("chicken.compiler.support#load-inline-file"));
lf[262]=C_h_intern(&lf[262],10, C_text("##sys#put!"));
lf[263]=C_h_intern(&lf[263],27, C_text("scheme#with-input-from-file"));
lf[264]=C_h_intern(&lf[264],35, C_text("chicken.compiler.support#match-node"));
lf[265]=C_h_intern(&lf[265],1, C_text("a"));
lf[266]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007matched"));
lf[267]=C_h_intern(&lf[267],53, C_text("chicken.compiler.support#expression-has-side-effects\077"));
lf[268]=C_h_intern(&lf[268],49, C_text("chicken.compiler.support#foreign-callback-stub-id"));
lf[269]=C_h_intern(&lf[269],47, C_text("chicken.compiler.support#foreign-callback-stubs"));
lf[270]=C_h_intern(&lf[270],44, C_text("chicken.compiler.support#simple-lambda-node\077"));
lf[271]=C_h_intern(&lf[271],47, C_text("chicken.compiler.support#dump-undefined-globals"));
lf[272]=C_h_intern(&lf[272],12, C_text("scheme#write"));
lf[273]=C_h_intern(&lf[273],6, C_text("global"));
lf[274]=C_h_intern(&lf[274],8, C_text("assigned"));
lf[275]=C_h_intern(&lf[275],24, C_text("chicken.keyword#keyword\077"));
lf[276]=C_h_intern(&lf[276],45, C_text("chicken.compiler.support#dump-defined-globals"));
lf[277]=C_h_intern(&lf[277],41, C_text("chicken.compiler.support#dump-global-refs"));
lf[278]=C_h_intern(&lf[278],30, C_text("##sys#toplevel-definition-hook"));
lf[279]=C_h_intern(&lf[279],22, C_text("chicken.plist#remprop!"));
lf[280]=C_h_intern(&lf[280],21, C_text("##compiler#visibility"));
lf[281]=C_h_intern(&lf[281],41, C_text("chicken.compiler.support#variable-hidden\077"));
lf[282]=C_h_intern(&lf[282],38, C_text("chicken.compiler.support#hide-variable"));
lf[283]=C_decode_literal(C_heaptop,C_text("\376B\000\000 hiding unexported module binding"));
lf[284]=C_h_intern(&lf[284],51, C_text("chicken.compiler.support#make-foreign-callback-stub"));
lf[285]=C_h_intern(&lf[285],46, C_text("chicken.compiler.support#foreign-callback-stub"));
lf[286]=C_h_intern(&lf[286],47, C_text("chicken.compiler.support#foreign-callback-stub\077"));
lf[287]=C_h_intern(&lf[287],24, C_text("foreign-callback-stub-id"));
lf[288]=C_h_intern(&lf[288],51, C_text("chicken.compiler.support#foreign-callback-stub-name"));
lf[289]=C_h_intern(&lf[289],26, C_text("foreign-callback-stub-name"));
lf[290]=C_h_intern(&lf[290],57, C_text("chicken.compiler.support#foreign-callback-stub-qualifiers"));
lf[291]=C_h_intern(&lf[291],32, C_text("foreign-callback-stub-qualifiers"));
lf[292]=C_h_intern(&lf[292],58, C_text("chicken.compiler.support#foreign-callback-stub-return-type"));
lf[293]=C_h_intern(&lf[293],33, C_text("foreign-callback-stub-return-type"));
lf[294]=C_h_intern(&lf[294],61, C_text("chicken.compiler.support#foreign-callback-stub-argument-types"));
lf[295]=C_h_intern(&lf[295],36, C_text("foreign-callback-stub-argument-types"));
lf[296]=C_h_intern(&lf[296],56, C_text("chicken.compiler.support#register-foreign-callback-stub!"));
lf[297]=C_h_intern(&lf[297],26, C_text("##compiler#callback-lambda"));
lf[299]=C_h_intern(&lf[299],50, C_text("chicken.compiler.support#clear-foreign-type-table!"));
lf[300]=C_h_intern(&lf[300],19, C_text("scheme#vector-fill!"));
lf[301]=C_h_intern(&lf[301],18, C_text("scheme#make-vector"));
lf[302]=C_h_intern(&lf[302],47, C_text("chicken.compiler.support#register-foreign-type!"));
lf[303]=C_h_intern(&lf[303],44, C_text("chicken.compiler.support#lookup-foreign-type"));
lf[304]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376\001\000\000\007\001integer\376B\000\000\003int\376\003\000\000\002\376\003\000\000\002\376\001\000\000\020\001unsigned-integer\376B\000\000\014unsigned int"
"\376\003\000\000\002\376\003\000\000\002\376\001\000\000\011\001integer32\376B\000\000\005C_s32\376\003\000\000\002\376\003\000\000\002\376\001\000\000\022\001unsigned-integer32\376B\000\000\005C_u32\376"
"\003\000\000\002\376\003\000\000\002\376\001\000\000\011\001integer64\376B\000\000\005C_s64\376\003\000\000\002\376\003\000\000\002\376\001\000\000\022\001unsigned-integer64\376B\000\000\005C_u64\376\003"
"\000\000\002\376\003\000\000\002\376\001\000\000\005\001short\376B\000\000\005short\376\003\000\000\002\376\003\000\000\002\376\001\000\000\016\001unsigned-short\376B\000\000\016unsigned short\376\003"
"\000\000\002\376\003\000\000\002\376\001\000\000\004\001long\376B\000\000\004long\376\003\000\000\002\376\003\000\000\002\376\001\000\000\015\001unsigned-long\376B\000\000\015unsigned long\376\003\000\000\002\376"
"\003\000\000\002\376\001\000\000\007\001ssize_t\376B\000\000\007ssize_t\376\003\000\000\002\376\003\000\000\002\376\001\000\000\006\001size_t\376B\000\000\006size_t\376\377\016"));
lf[305]=C_h_intern(&lf[305],43, C_text("chicken.compiler.support#foreign-type-check"));
lf[306]=C_h_intern(&lf[306],4, C_text("char"));
lf[307]=C_h_intern(&lf[307],13, C_text("unsigned-char"));
lf[308]=C_h_intern(&lf[308],27, C_text("##sys#foreign-char-argument"));
lf[309]=C_h_intern(&lf[309],3, C_text("int"));
lf[310]=C_h_intern(&lf[310],29, C_text("##sys#foreign-fixnum-argument"));
lf[311]=C_h_intern(&lf[311],5, C_text("float"));
lf[312]=C_h_intern(&lf[312],29, C_text("##sys#foreign-flonum-argument"));
lf[313]=C_h_intern(&lf[313],4, C_text("blob"));
lf[314]=C_h_intern(&lf[314],14, C_text("scheme-pointer"));
lf[315]=C_h_intern(&lf[315],28, C_text("##sys#foreign-block-argument"));
lf[316]=C_h_intern(&lf[316],12, C_text("##core#quote"));
lf[317]=C_h_intern(&lf[317],9, C_text("##core#if"));
lf[318]=C_h_intern(&lf[318],10, C_text("##core#let"));
lf[319]=C_h_intern(&lf[319],22, C_text("nonnull-scheme-pointer"));
lf[320]=C_h_intern(&lf[320],12, C_text("nonnull-blob"));
lf[321]=C_h_intern(&lf[321],14, C_text("pointer-vector"));
lf[322]=C_h_intern(&lf[322],37, C_text("##sys#foreign-struct-wrapper-argument"));
lf[323]=C_h_intern(&lf[323],22, C_text("nonnull-pointer-vector"));
lf[324]=C_h_intern(&lf[324],8, C_text("u8vector"));
lf[325]=C_h_intern(&lf[325],16, C_text("nonnull-u8vector"));
lf[326]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376\001\000\000\020\001nonnull-u8vector\376\001\000\000\010\001u8vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonnull-u16vector\376"
"\001\000\000\011\001u16vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\020\001nonnull-s8vector\376\001\000\000\010\001s8vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonn"
"ull-s16vector\376\001\000\000\011\001s16vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonnull-u32vector\376\001\000\000\011\001u32vector\376\003\000\000"
"\002\376\003\000\000\002\376\001\000\000\021\001nonnull-s32vector\376\001\000\000\011\001s32vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonnull-u64vector\376\001\000"
"\000\011\001u64vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonnull-s64vector\376\001\000\000\011\001s64vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonn"
"ull-f32vector\376\001\000\000\011\001f32vector\376\003\000\000\002\376\003\000\000\002\376\001\000\000\021\001nonnull-f64vector\376\001\000\000\011\001f64vector\376\377\016"));
lf[327]=C_h_intern(&lf[327],9, C_text("integer32"));
lf[328]=C_h_intern(&lf[328],6, C_text("format"));
lf[329]=C_h_intern(&lf[329],13, C_text("foreign-value"));
lf[330]=C_h_intern(&lf[330],37, C_text("##sys#foreign-ranged-integer-argument"));
lf[331]=C_h_intern(&lf[331],30, C_text("chicken.base#get-output-string"));
lf[332]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014) \052 CHAR_BIT"));
lf[333]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007sizeof("));
lf[334]=C_h_intern(&lf[334],31, C_text("chicken.base#open-output-string"));
lf[335]=C_h_intern(&lf[335],14, C_text("unsigned-short"));
lf[336]=C_h_intern(&lf[336],46, C_text("##sys#foreign-unsigned-ranged-integer-argument"));
lf[337]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014) \052 CHAR_BIT"));
lf[338]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007sizeof("));
lf[339]=C_h_intern(&lf[339],9, C_text("c-pointer"));
lf[340]=C_h_intern(&lf[340],30, C_text("##sys#foreign-pointer-argument"));
lf[341]=C_h_intern(&lf[341],17, C_text("nonnull-c-pointer"));
lf[342]=C_h_intern(&lf[342],8, C_text("c-string"));
lf[343]=C_h_intern(&lf[343],19, C_text("##sys#make-c-string"));
lf[344]=C_h_intern(&lf[344],29, C_text("##sys#foreign-string-argument"));
lf[345]=C_h_intern(&lf[345],16, C_text("nonnull-c-string"));
lf[346]=C_h_intern(&lf[346],6, C_text("symbol"));
lf[347]=C_h_intern(&lf[347],3, C_text("ref"));
lf[348]=C_h_intern(&lf[348],8, C_text("instance"));
lf[349]=C_h_intern(&lf[349],12, C_text("instance-ref"));
lf[350]=C_h_intern(&lf[350],4, C_text("this"));
lf[351]=C_h_intern(&lf[351],8, C_text("slot-ref"));
lf[352]=C_h_intern(&lf[352],16, C_text("nonnull-instance"));
lf[353]=C_h_intern(&lf[353],5, C_text("const"));
lf[354]=C_h_intern(&lf[354],4, C_text("enum"));
lf[355]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026sizeof(int) \052 CHAR_BIT"));
lf[356]=C_h_intern(&lf[356],15, C_text("nonnull-pointer"));
lf[357]=C_h_intern(&lf[357],7, C_text("pointer"));
lf[358]=C_h_intern(&lf[358],8, C_text("function"));
lf[359]=C_h_intern(&lf[359],17, C_text("nonnull-c-string\052"));
lf[360]=C_h_intern(&lf[360],26, C_text("nonnull-unsigned-c-string\052"));
lf[361]=C_h_intern(&lf[361],9, C_text("c-string\052"));
lf[362]=C_h_intern(&lf[362],17, C_text("unsigned-c-string"));
lf[363]=C_h_intern(&lf[363],18, C_text("unsigned-c-string\052"));
lf[364]=C_h_intern(&lf[364],13, C_text("c-string-list"));
lf[365]=C_h_intern(&lf[365],14, C_text("c-string-list\052"));
lf[366]=C_h_intern(&lf[366],13, C_text("unsigned-long"));
lf[367]=C_h_intern(&lf[367],16, C_text("unsigned-integer"));
lf[368]=C_h_intern(&lf[368],6, C_text("size_t"));
lf[369]=C_h_intern(&lf[369],18, C_text("unsigned-integer32"));
lf[370]=C_h_intern(&lf[370],18, C_text("unsigned-integer64"));
lf[371]=C_h_intern(&lf[371],9, C_text("integer64"));
lf[372]=C_h_intern(&lf[372],7, C_text("integer"));
lf[373]=C_h_intern(&lf[373],5, C_text("short"));
lf[374]=C_h_intern(&lf[374],4, C_text("long"));
lf[375]=C_h_intern(&lf[375],7, C_text("ssize_t"));
lf[376]=C_h_intern(&lf[376],17, C_text("nonnull-u16vector"));
lf[377]=C_h_intern(&lf[377],16, C_text("nonnull-s8vector"));
lf[378]=C_h_intern(&lf[378],17, C_text("nonnull-s16vector"));
lf[379]=C_h_intern(&lf[379],17, C_text("nonnull-u32vector"));
lf[380]=C_h_intern(&lf[380],17, C_text("nonnull-s32vector"));
lf[381]=C_h_intern(&lf[381],17, C_text("nonnull-u64vector"));
lf[382]=C_h_intern(&lf[382],17, C_text("nonnull-s64vector"));
lf[383]=C_h_intern(&lf[383],17, C_text("nonnull-f32vector"));
lf[384]=C_h_intern(&lf[384],17, C_text("nonnull-f64vector"));
lf[385]=C_h_intern(&lf[385],9, C_text("u16vector"));
lf[386]=C_h_intern(&lf[386],8, C_text("s8vector"));
lf[387]=C_h_intern(&lf[387],9, C_text("s16vector"));
lf[388]=C_h_intern(&lf[388],9, C_text("u32vector"));
lf[389]=C_h_intern(&lf[389],9, C_text("s32vector"));
lf[390]=C_h_intern(&lf[390],9, C_text("u64vector"));
lf[391]=C_h_intern(&lf[391],9, C_text("s64vector"));
lf[392]=C_h_intern(&lf[392],9, C_text("f32vector"));
lf[393]=C_h_intern(&lf[393],9, C_text("f64vector"));
lf[394]=C_h_intern(&lf[394],6, C_text("double"));
lf[395]=C_h_intern(&lf[395],6, C_text("number"));
lf[396]=C_h_intern(&lf[396],12, C_text("unsigned-int"));
lf[397]=C_h_intern(&lf[397],4, C_text("byte"));
lf[398]=C_h_intern(&lf[398],13, C_text("unsigned-byte"));
lf[399]=C_h_intern(&lf[399],5, C_text("int32"));
lf[400]=C_h_intern(&lf[400],14, C_text("unsigned-int32"));
lf[401]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042foreign type `~S\047 refers to itself"));
lf[404]=C_h_intern(&lf[404],52, C_text("chicken.compiler.support#foreign-type-convert-result"));
lf[405]=C_h_intern(&lf[405],54, C_text("chicken.compiler.support#foreign-type-convert-argument"));
lf[406]=C_h_intern(&lf[406],43, C_text("chicken.compiler.support#final-foreign-type"));
lf[407]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042foreign type `~S\047 refers to itself"));
lf[408]=C_h_intern(&lf[408],53, C_text("chicken.compiler.support#estimate-foreign-result-size"));
lf[409]=C_decode_literal(C_heaptop,C_text("\376B\000\0008cannot compute size for unknown foreign type `~S\047 result"));
lf[410]=C_h_intern(&lf[410],4, C_text("bool"));
lf[411]=C_h_intern(&lf[411],4, C_text("void"));
lf[412]=C_h_intern(&lf[412],13, C_text("scheme-object"));
lf[413]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042foreign type `~S\047 refers to itself"));
lf[414]=C_h_intern(&lf[414],62, C_text("chicken.compiler.support#estimate-foreign-result-location-size"));
lf[415]=C_decode_literal(C_heaptop,C_text("\376B\000\0005cannot compute size of location for foreign type `~S\047"));
lf[416]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042foreign type `~S\047 refers to itself"));
lf[417]=C_h_intern(&lf[417],46, C_text("chicken.compiler.support#finish-foreign-result"));
lf[418]=C_h_intern(&lf[418],19, C_text("##sys#peek-c-string"));
lf[419]=C_h_intern(&lf[419],27, C_text("##sys#peek-nonnull-c-string"));
lf[420]=C_h_intern(&lf[420],28, C_text("##sys#peek-and-free-c-string"));
lf[421]=C_h_intern(&lf[421],36, C_text("##sys#peek-and-free-nonnull-c-string"));
lf[422]=C_h_intern(&lf[422],19, C_text("##sys#intern-symbol"));
lf[423]=C_h_intern(&lf[423],24, C_text("##sys#peek-c-string-list"));
lf[424]=C_h_intern(&lf[424],33, C_text("##sys#peek-and-free-c-string-list"));
lf[425]=C_h_intern(&lf[425],19, C_text("##sys#null-pointer\077"));
lf[426]=C_h_intern(&lf[426],3, C_text("not"));
lf[427]=C_h_intern(&lf[427],4, C_text("make"));
lf[428]=C_h_intern(&lf[428],3, C_text("and"));
lf[429]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\001c-string\376\003\000\000\002\376\001\000\000\011\001c-string\052\376\003\000\000\002\376\001\000\000\021\001unsigned-c-string\376\003\000\000\002\376\001\000\000\022\001un"
"signed-c-string\052\376\003\000\000\002\376\001\000\000\020\001nonnull-c-string\376\003\000\000\002\376\001\000\000\021\001nonnull-c-string\052\376\003\000\000\002\376\001\000\000"
"\030\001nonnull-unsigned-string\052\376\377\016"));
lf[430]=C_h_intern(&lf[430],27, C_text("chicken.syntax#strip-syntax"));
lf[431]=C_h_intern(&lf[431],52, C_text("chicken.compiler.support#foreign-type->scrutiny-type"));
lf[432]=C_h_intern(&lf[432],9, C_text("undefined"));
lf[433]=C_h_intern(&lf[433],3, C_text("arg"));
lf[434]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\004\001blob\376\377\016"));
lf[435]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\016\001pointer-vector\376\377\016"));
lf[436]=C_h_intern(&lf[436],6, C_text("struct"));
lf[437]=C_h_intern(&lf[437],2, C_text("or"));
lf[438]=C_h_intern(&lf[438],5, C_text("false"));
lf[439]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\010\001u8vector\376\377\016"));
lf[440]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\010\001s8vector\376\377\016"));
lf[441]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001u16vector\376\377\016"));
lf[442]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001s16vector\376\377\016"));
lf[443]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001u32vector\376\377\016"));
lf[444]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001s32vector\376\377\016"));
lf[445]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001u64vector\376\377\016"));
lf[446]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001s64vector\376\377\016"));
lf[447]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001f32vector\376\377\016"));
lf[448]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001struct\376\003\000\000\002\376\001\000\000\011\001f64vector\376\377\016"));
lf[449]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\010\001locative\376\377\016"));
lf[450]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\007\001pointer\376\377\016"));
lf[451]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\010\001locative\376\377\016"));
lf[452]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\006\001string\376\377\016"));
lf[453]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\001list-of\376\003\000\000\002\376\001\000\000\006\001string\376\377\016"));
lf[454]=C_h_intern(&lf[454],6, C_text("string"));
lf[455]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\010\001locative\376\377\016"));
lf[456]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\007\001pointer\376\377\016"));
lf[457]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\010\001locative\376\377\016"));
lf[458]=C_h_intern(&lf[458],6, C_text("result"));
lf[459]=C_h_intern(&lf[459],44, C_text("chicken.compiler.support#scan-used-variables"));
lf[460]=C_h_intern(&lf[460],44, C_text("chicken.compiler.support#scan-free-variables"));
lf[461]=C_h_intern(&lf[461],39, C_text("chicken.compiler.support#chop-separator"));
lf[462]=C_h_intern(&lf[462],16, C_text("scheme#substring"));
lf[463]=C_h_intern(&lf[463],52, C_text("chicken.compiler.support#make-block-variable-literal"));
lf[464]=C_h_intern(&lf[464],47, C_text("chicken.compiler.support#block-variable-literal"));
lf[465]=C_h_intern(&lf[465],48, C_text("chicken.compiler.support#block-variable-literal\077"));
lf[466]=C_h_intern(&lf[466],52, C_text("chicken.compiler.support#block-variable-literal-name"));
lf[467]=C_h_intern(&lf[467],27, C_text("block-variable-literal-name"));
lf[468]=C_h_intern(&lf[468],21, C_text("scheme#string->symbol"));
lf[469]=C_h_intern(&lf[469],28, C_text("chicken.time#current-seconds"));
lf[471]=C_h_intern(&lf[471],47, C_text("chicken.compiler.support#clear-real-name-table!"));
lf[472]=C_h_intern(&lf[472],39, C_text("chicken.compiler.support#set-real-name!"));
lf[473]=C_h_intern(&lf[473],38, C_text("chicken.compiler.support#get-real-name"));
lf[474]=C_h_intern(&lf[474],33, C_text("chicken.string#string-intersperse"));
lf[475]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 in "));
lf[476]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003..."));
lf[477]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 in "));
lf[478]=C_h_intern(&lf[478],12, C_text("contained-in"));
lf[479]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 in "));
lf[480]=C_h_intern(&lf[480],35, C_text("chicken.compiler.support#real-name2"));
lf[481]=C_h_intern(&lf[481],48, C_text("chicken.compiler.support#display-real-name-table"));
lf[482]=C_h_intern(&lf[482],44, C_text("chicken.compiler.support#source-info->string"));
lf[483]=C_h_intern(&lf[483],19, C_text("chicken.string#conc"));
lf[484]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001:"));
lf[485]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[486]=C_h_intern(&lf[486],18, C_text("scheme#make-string"));
lf[487]=C_h_intern(&lf[487],10, C_text("scheme#max"));
lf[488]=C_h_intern(&lf[488],42, C_text("chicken.compiler.support#source-info->name"));
lf[489]=C_h_intern(&lf[489],42, C_text("chicken.compiler.support#source-info->line"));
lf[490]=C_h_intern(&lf[490],34, C_text("chicken.compiler.support#call-info"));
lf[491]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[492]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002) "));
lf[493]=C_h_intern(&lf[493],43, C_text("chicken.compiler.support#constant-form-eval"));
lf[494]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032folded constant expression"));
lf[495]=C_decode_literal(C_heaptop,C_text("\376B\000\000Dattempt to constant-fold call to procedure that has multiple results"));
lf[497]=C_h_intern(&lf[497],28, C_text("chicken.condition#condition\077"));
lf[498]=C_h_intern(&lf[498],10, C_text("##sys#list"));
lf[499]=C_decode_literal(C_heaptop,C_text("\376B\000\000.attempt to constant-fold call to non-procedure"));
lf[500]=C_h_intern(&lf[500],49, C_text("chicken.compiler.support#maybe-constant-fold-call"));
lf[501]=C_h_intern(&lf[501],35, C_text("chicken.compiler.support#predicate\077"));
lf[502]=C_h_intern(&lf[502],34, C_text("chicken.compiler.support#foldable\077"));
lf[503]=C_h_intern(&lf[503],20, C_text("##compiler#intrinsic"));
lf[504]=C_h_intern(&lf[504],20, C_text("##sys#number->string"));
lf[505]=C_h_intern(&lf[505],35, C_text("chicken.compiler.support#dump-nodes"));
lf[506]=C_h_intern(&lf[506],21, C_text("##sys#write-char/port"));
lf[507]=C_h_intern(&lf[507],39, C_text("chicken.compiler.support#read-info-hook"));
lf[508]=C_h_intern(&lf[508],9, C_text("list-info"));
lf[509]=C_h_intern(&lf[509],29, C_text("##sys#current-source-filename"));
lf[510]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001:"));
lf[511]=C_h_intern(&lf[511],41, C_text("chicken.compiler.support#read/source-info"));
lf[512]=C_h_intern(&lf[512],10, C_text("##sys#read"));
lf[513]=C_h_intern(&lf[513],20, C_text("##sys#user-read-hook"));
lf[514]=C_h_intern(&lf[514],15, C_text("foreign-declare"));
lf[515]=C_h_intern(&lf[515],7, C_text("declare"));
lf[516]=C_decode_literal(C_heaptop,C_text("\376B\000\000&unexpected end of `#> ... <#\047 sequence"));
lf[517]=C_h_intern(&lf[517],20, C_text("##sys#read-char/port"));
lf[518]=C_h_intern(&lf[518],25, C_text("chicken.platform#feature\077"));
lf[519]=C_h_intern_kw(&lf[519],5, C_text("64bit"));
lf[520]=C_h_intern(&lf[520],38, C_text("chicken.compiler.support#small-bignum\077"));
lf[521]=C_h_intern(&lf[521],6, C_text("hidden"));
lf[522]=C_h_intern(&lf[522],40, C_text("chicken.compiler.support#export-variable"));
lf[523]=C_h_intern(&lf[523],8, C_text("exported"));
lf[524]=C_h_intern(&lf[524],38, C_text("chicken.compiler.support#mark-variable"));
lf[525]=C_h_intern(&lf[525],38, C_text("chicken.compiler.support#variable-mark"));
lf[526]=C_h_intern(&lf[526],35, C_text("chicken.compiler.support#intrinsic\077"));
lf[527]=C_h_intern(&lf[527],19, C_text("##compiler#foldable"));
lf[528]=C_h_intern(&lf[528],20, C_text("##compiler#predicate"));
lf[529]=C_h_intern(&lf[529],49, C_text("chicken.compiler.support#load-identifier-database"));
lf[530]=C_h_intern(&lf[530],9, C_text("##core#db"));
lf[531]=C_h_intern(&lf[531],27, C_text("scheme#call-with-input-file"));
lf[532]=C_h_intern(&lf[532],1, C_text("p"));
lf[533]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 ..."));
lf[534]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034loading identifier database "));
lf[535]=C_h_intern(&lf[535],22, C_text("chicken.load#find-file"));
lf[536]=C_h_intern(&lf[536],32, C_text("chicken.platform#repository-path"));
lf[537]=C_h_intern(&lf[537],38, C_text("chicken.compiler.support#print-version"));
lf[538]=C_h_intern(&lf[538],19, C_text("chicken.base#print\052"));
lf[539]=C_decode_literal(C_heaptop,C_text("\376B\000\000KCHICKEN\012(c) 2008-2021, The CHICKEN Team\012(c) 2000-2007, Felix L. Winkelmann\012"
));
lf[540]=C_h_intern(&lf[540],36, C_text("chicken.compiler.support#print-usage"));
lf[541]=C_decode_literal(C_heaptop,C_text("\376B\000\031\317Usage: chicken FILENAME [OPTION ...]\012\012  `chicken\047 is the CHICKEN compiler.\012"
"  \012  FILENAME should be a complete source file name with extension, or \042-\042 for\012 "
" standard input. OPTION may be one of the following:\012\012  General options:\012\012    -h"
"elp                        display this text and exit\012    -version              "
"       display compiler version and exit\012    -release                     print "
"release number and exit\012    -verbose                     display information on "
"compilation progress\012\012  File and pathname options:\012\012    -output-file FILENAME   "
"     specifies output-filename, default is \047out.c\047\012    -include-path PATHNAME   "
"    specifies alternative path for included files\012    -to-stdout                "
"   write compiled file to stdout instead of file\012\012  Language options:\012\012    -feat"
"ure SYMBOL              register feature identifier\012    -no-feature SYMBOL      "
"     disable built-in feature identifier\012\012  Syntax related options:\012\012    -case-i"
"nsensitive            don\047t preserve case of read symbols\012    -keyword-style STY"
"LE         allow alternative keyword syntax\012                                  (p"
"refix, suffix or none)\012    -no-parentheses-synonyms     disables list delimiter "
"synonyms\012    -no-symbol-escape            disables support for escaped symbols\012 "
"   -r5rs-syntax                 disables the CHICKEN extensions to\012             "
"                     R5RS syntax\012    -compile-syntax              macros are mad"
"e available at run-time\012    -emit-import-library MODULE  write compile-time modu"
"le information into\012                                  separate file\012    -emit-al"
"l-import-libraries   emit import-libraries for all defined modules\012    -no-compi"
"ler-syntax          disable expansion of compiler-macros\012    -module NAME       "
"          wrap compiled code in a module\012    -module-registration         always"
" generate module registration code\012    -no-module-registration      never genera"
"te module registration code\012                                  (overrides `-modul"
"e-registration\047)\012\012  Translation options:\012\012    -explicit-use                do no"
"t use units \047library\047 and \047eval\047 by\012                                  default\012  "
"  -check-syntax                stop compilation after macro-expansion\012    -analy"
"ze-only                stop compilation after first analysis pass\012\012  Debugging o"
"ptions:\012\012    -no-warnings                 disable warnings\012    -debug-level NUMB"
"ER          set level of available debugging information\012    -no-trace          "
"          disable tracing information\012    -debug-info                  enable de"
"bug-information in compiled code for use\012                                  with "
"an external debugger\012    -profile                     executable emits profiling"
" information \012    -profile-name FILENAME       name of the generated profile inf"
"ormation file\012    -accumulate-profile          executable emits profiling inform"
"ation in\012                                  append mode\012    -no-lambda-info      "
"        omit additional procedure-information\012    -emit-types-file FILENAME    w"
"rite type-declaration information into file\012    -consult-types-file FILENAME loa"
"d additional type database\012\012  Optimization options:\012\012    -optimize-level NUMBER "
"      enable certain sets of optimization options\012    -optimize-leaf-routines   "
"   enable leaf routine optimization\012    -no-usual-integrations       standard pr"
"ocedures may be redefined\012    -unsafe                      disable all safety ch"
"ecks\012    -local                       assume globals are only modified in curren"
"t\012                                  file\012    -block                       enable"
" block-compilation\012    -disable-interrupts          disable interrupts in compil"
"ed code\012    -fixnum-arithmetic           assume all numbers are fixnums\012    -dis"
"able-stack-overflow-checks  disables detection of stack-overflows\012    -inline   "
"                   enable inlining\012    -inline-limit LIMIT          set inlining"
" threshold\012    -inline-global               enable cross-module inlining\012    -sp"
"ecialize                  perform type-based specialization of primitive calls\012 "
"   -emit-inline-file FILENAME   generate file with globally inlinable\012          "
"                        procedures (implies -inline -local)\012    -consult-inline-"
"file FILENAME  explicitly load inline file\012    -no-argc-checks              disa"
"ble argument count checks\012    -no-bound-checks             disable bound variabl"
"e checks\012    -no-procedure-checks         disable procedure call checks\012    -no-"
"procedure-checks-for-usual-bindings\012                                   disable p"
"rocedure call checks only for usual\012                                   bindings\012"
"    -no-procedure-checks-for-toplevel-bindings\012                                 "
"  disable procedure call checks for toplevel\012                                   "
"bindings\012    -strict-types                assume variable do not change their ty"
"pe\012    -clustering                  combine groups of local procedures into disp"
"atch\012                                   loop\012    -lfa2                        pe"
"rform additional lightweight flow-analysis pass\012    -unroll-limit LIMIT         "
" specifies inlining limit for self-recursive calls\012\012  Configuration options:\012\012  "
"  -unit NAME                   compile file as a library unit\012    -uses NAME    "
"               declare library unit as used.\012    -heap-size NUMBER            sp"
"ecifies heap-size of compiled executable\012    -nursery NUMBER  -stack-size NUMBER"
"\012                                 specifies nursery size of compiled executable\012"
"    -extend FILENAME             load file before compilation commences\012    -pre"
"lude EXPRESSION          add expression to front of source file\012    -postlude EX"
"PRESSION         add expression to end of source file\012    -prologue FILENAME    "
"       include file before main source file\012    -epilogue FILENAME           inc"
"lude file after main source file\012    -dynamic                     compile as dyn"
"amically loadable code\012    -require-extension NAME      require and import exten"
"sion NAME\012\012  Obscure options:\012\012    -debug MODES                 display debuggin"
"g output for the given modes\012    -raw                         do not generate im"
"plicit init- and exit code                           \012    -emit-external-prototy"
"pes-first\012                                 emit prototypes for callbacks before "
"foreign\012                                  declarations\012    -regenerate-import-li"
"braries emit import libraries even when unchanged\012    -ignore-repository        "
"   do not refer to repository for extensions\012    -setup-mode                  pr"
"efer the current directory when locating extensions\012"));
lf[542]=C_h_intern(&lf[542],44, C_text("chicken.compiler.support#print-debug-options"));
lf[543]=C_decode_literal(C_heaptop,C_text("\376B\000\006\344\012Available debugging options:\012\012     a          show node-matching during si"
"mplification\012     b          show breakdown of time needed for each compiler pas"
"s\012     c          print every expression before macro-expansion\012     d          "
"lists all assigned global variables\012     e          show information about speci"
"alizations\012     h          you already figured that out\012     i          show inf"
"ormation about inlining\012     m          show GC statistics during compilation\012  "
"   n          print the line-number database \012     o          show performed opt"
"imizations\012     p          display information about what the compiler is curren"
"tly doing\012     r          show invocation parameters\012     s          show progra"
"m-size information and other statistics\012     t          show time needed for com"
"pilation\012     u          lists all unassigned global variable references\012     x "
"         display information about experimental features\012     D          when pr"
"inting nodes, use node-tree output\012     I          show inferred type informatio"
"n for unexported globals\012     N          show the real-name mapping table\012     P"
"          show expressions after specialization\012     S          show application"
"s of compiler syntax\012     T          show expressions after converting to node t"
"ree\012     1          show source expressions\012     2          show canonicalized e"
"xpressions\012     3          show expressions converted into CPS\012     4          s"
"how database after each analysis pass\012     5          show expressions after eac"
"h optimization pass\012     6          show expressions after each inlining pass\012  "
"   7          show expressions after complete optimization\012     8          show "
"database after final analysis\012     9          show expressions after closure con"
"version\012\012"));
lf[544]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007#<node "));
lf[545]=C_h_intern(&lf[545],32, C_text("chicken.base#set-record-printer!"));
lf[546]=C_h_intern(&lf[546],45, C_text("chicken.condition#condition-property-accessor"));
lf[547]=C_h_intern(&lf[547],3, C_text("exn"));
lf[548]=C_h_intern(&lf[548],7, C_text("message"));
lf[549]=C_h_intern(&lf[549],37, C_text("chicken.condition#condition-predicate"));
C_register_lf2(lf,550,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5348,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_library_toplevel(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[782] = {
{C_text("f19850:support_2escm"),(void*)f19850},
{C_text("f_10004:support_2escm"),(void*)f_10004},
{C_text("f_10032:support_2escm"),(void*)f_10032},
{C_text("f_10056:support_2escm"),(void*)f_10056},
{C_text("f_10090:support_2escm"),(void*)f_10090},
{C_text("f_10094:support_2escm"),(void*)f_10094},
{C_text("f_10104:support_2escm"),(void*)f_10104},
{C_text("f_10128:support_2escm"),(void*)f_10128},
{C_text("f_10143:support_2escm"),(void*)f_10143},
{C_text("f_10155:support_2escm"),(void*)f_10155},
{C_text("f_10180:support_2escm"),(void*)f_10180},
{C_text("f_10205:support_2escm"),(void*)f_10205},
{C_text("f_10235:support_2escm"),(void*)f_10235},
{C_text("f_10237:support_2escm"),(void*)f_10237},
{C_text("f_10262:support_2escm"),(void*)f_10262},
{C_text("f_10293:support_2escm"),(void*)f_10293},
{C_text("f_10305:support_2escm"),(void*)f_10305},
{C_text("f_10309:support_2escm"),(void*)f_10309},
{C_text("f_10332:support_2escm"),(void*)f_10332},
{C_text("f_10343:support_2escm"),(void*)f_10343},
{C_text("f_10350:support_2escm"),(void*)f_10350},
{C_text("f_10361:support_2escm"),(void*)f_10361},
{C_text("f_10363:support_2escm"),(void*)f_10363},
{C_text("f_10388:support_2escm"),(void*)f_10388},
{C_text("f_10418:support_2escm"),(void*)f_10418},
{C_text("f_10420:support_2escm"),(void*)f_10420},
{C_text("f_10445:support_2escm"),(void*)f_10445},
{C_text("f_10459:support_2escm"),(void*)f_10459},
{C_text("f_10469:support_2escm"),(void*)f_10469},
{C_text("f_10471:support_2escm"),(void*)f_10471},
{C_text("f_10496:support_2escm"),(void*)f_10496},
{C_text("f_10569:support_2escm"),(void*)f_10569},
{C_text("f_10575:support_2escm"),(void*)f_10575},
{C_text("f_10601:support_2escm"),(void*)f_10601},
{C_text("f_10605:support_2escm"),(void*)f_10605},
{C_text("f_10621:support_2escm"),(void*)f_10621},
{C_text("f_10627:support_2escm"),(void*)f_10627},
{C_text("f_10633:support_2escm"),(void*)f_10633},
{C_text("f_10639:support_2escm"),(void*)f_10639},
{C_text("f_10643:support_2escm"),(void*)f_10643},
{C_text("f_10646:support_2escm"),(void*)f_10646},
{C_text("f_10660:support_2escm"),(void*)f_10660},
{C_text("f_10663:support_2escm"),(void*)f_10663},
{C_text("f_10666:support_2escm"),(void*)f_10666},
{C_text("f_10673:support_2escm"),(void*)f_10673},
{C_text("f_10675:support_2escm"),(void*)f_10675},
{C_text("f_10690:support_2escm"),(void*)f_10690},
{C_text("f_10723:support_2escm"),(void*)f_10723},
{C_text("f_10751:support_2escm"),(void*)f_10751},
{C_text("f_10755:support_2escm"),(void*)f_10755},
{C_text("f_10779:support_2escm"),(void*)f_10779},
{C_text("f_10811:support_2escm"),(void*)f_10811},
{C_text("f_10820:support_2escm"),(void*)f_10820},
{C_text("f_10824:support_2escm"),(void*)f_10824},
{C_text("f_10830:support_2escm"),(void*)f_10830},
{C_text("f_10836:support_2escm"),(void*)f_10836},
{C_text("f_10839:support_2escm"),(void*)f_10839},
{C_text("f_10844:support_2escm"),(void*)f_10844},
{C_text("f_10854:support_2escm"),(void*)f_10854},
{C_text("f_10869:support_2escm"),(void*)f_10869},
{C_text("f_10871:support_2escm"),(void*)f_10871},
{C_text("f_10896:support_2escm"),(void*)f_10896},
{C_text("f_10917:support_2escm"),(void*)f_10917},
{C_text("f_10942:support_2escm"),(void*)f_10942},
{C_text("f_10964:support_2escm"),(void*)f_10964},
{C_text("f_10972:support_2escm"),(void*)f_10972},
{C_text("f_11023:support_2escm"),(void*)f_11023},
{C_text("f_11030:support_2escm"),(void*)f_11030},
{C_text("f_11033:support_2escm"),(void*)f_11033},
{C_text("f_11062:support_2escm"),(void*)f_11062},
{C_text("f_11070:support_2escm"),(void*)f_11070},
{C_text("f_11086:support_2escm"),(void*)f_11086},
{C_text("f_11089:support_2escm"),(void*)f_11089},
{C_text("f_11095:support_2escm"),(void*)f_11095},
{C_text("f_11115:support_2escm"),(void*)f_11115},
{C_text("f_11138:support_2escm"),(void*)f_11138},
{C_text("f_11143:support_2escm"),(void*)f_11143},
{C_text("f_11147:support_2escm"),(void*)f_11147},
{C_text("f_11150:support_2escm"),(void*)f_11150},
{C_text("f_11156:support_2escm"),(void*)f_11156},
{C_text("f_11159:support_2escm"),(void*)f_11159},
{C_text("f_11176:support_2escm"),(void*)f_11176},
{C_text("f_11186:support_2escm"),(void*)f_11186},
{C_text("f_11188:support_2escm"),(void*)f_11188},
{C_text("f_11213:support_2escm"),(void*)f_11213},
{C_text("f_11224:support_2escm"),(void*)f_11224},
{C_text("f_11232:support_2escm"),(void*)f_11232},
{C_text("f_11240:support_2escm"),(void*)f_11240},
{C_text("f_11253:support_2escm"),(void*)f_11253},
{C_text("f_11255:support_2escm"),(void*)f_11255},
{C_text("f_11303:support_2escm"),(void*)f_11303},
{C_text("f_11328:support_2escm"),(void*)f_11328},
{C_text("f_11347:support_2escm"),(void*)f_11347},
{C_text("f_11352:support_2escm"),(void*)f_11352},
{C_text("f_11362:support_2escm"),(void*)f_11362},
{C_text("f_11364:support_2escm"),(void*)f_11364},
{C_text("f_11389:support_2escm"),(void*)f_11389},
{C_text("f_11411:support_2escm"),(void*)f_11411},
{C_text("f_11462:support_2escm"),(void*)f_11462},
{C_text("f_11506:support_2escm"),(void*)f_11506},
{C_text("f_11553:support_2escm"),(void*)f_11553},
{C_text("f_11581:support_2escm"),(void*)f_11581},
{C_text("f_11585:support_2escm"),(void*)f_11585},
{C_text("f_11589:support_2escm"),(void*)f_11589},
{C_text("f_11608:support_2escm"),(void*)f_11608},
{C_text("f_11620:support_2escm"),(void*)f_11620},
{C_text("f_11622:support_2escm"),(void*)f_11622},
{C_text("f_11664:support_2escm"),(void*)f_11664},
{C_text("f_11674:support_2escm"),(void*)f_11674},
{C_text("f_11696:support_2escm"),(void*)f_11696},
{C_text("f_11723:support_2escm"),(void*)f_11723},
{C_text("f_11743:support_2escm"),(void*)f_11743},
{C_text("f_11796:support_2escm"),(void*)f_11796},
{C_text("f_11843:support_2escm"),(void*)f_11843},
{C_text("f_11860:support_2escm"),(void*)f_11860},
{C_text("f_11866:support_2escm"),(void*)f_11866},
{C_text("f_11880:support_2escm"),(void*)f_11880},
{C_text("f_11884:support_2escm"),(void*)f_11884},
{C_text("f_11890:support_2escm"),(void*)f_11890},
{C_text("f_11928:support_2escm"),(void*)f_11928},
{C_text("f_11932:support_2escm"),(void*)f_11932},
{C_text("f_11935:support_2escm"),(void*)f_11935},
{C_text("f_11938:support_2escm"),(void*)f_11938},
{C_text("f_11973:support_2escm"),(void*)f_11973},
{C_text("f_12017:support_2escm"),(void*)f_12017},
{C_text("f_12019:support_2escm"),(void*)f_12019},
{C_text("f_12044:support_2escm"),(void*)f_12044},
{C_text("f_12059:support_2escm"),(void*)f_12059},
{C_text("f_12088:support_2escm"),(void*)f_12088},
{C_text("f_12090:support_2escm"),(void*)f_12090},
{C_text("f_12115:support_2escm"),(void*)f_12115},
{C_text("f_12124:support_2escm"),(void*)f_12124},
{C_text("f_12127:support_2escm"),(void*)f_12127},
{C_text("f_12133:support_2escm"),(void*)f_12133},
{C_text("f_12181:support_2escm"),(void*)f_12181},
{C_text("f_12184:support_2escm"),(void*)f_12184},
{C_text("f_12190:support_2escm"),(void*)f_12190},
{C_text("f_12198:support_2escm"),(void*)f_12198},
{C_text("f_12206:support_2escm"),(void*)f_12206},
{C_text("f_12216:support_2escm"),(void*)f_12216},
{C_text("f_12235:support_2escm"),(void*)f_12235},
{C_text("f_12239:support_2escm"),(void*)f_12239},
{C_text("f_12244:support_2escm"),(void*)f_12244},
{C_text("f_12250:support_2escm"),(void*)f_12250},
{C_text("f_12253:support_2escm"),(void*)f_12253},
{C_text("f_12258:support_2escm"),(void*)f_12258},
{C_text("f_12268:support_2escm"),(void*)f_12268},
{C_text("f_12283:support_2escm"),(void*)f_12283},
{C_text("f_12285:support_2escm"),(void*)f_12285},
{C_text("f_12292:support_2escm"),(void*)f_12292},
{C_text("f_12313:support_2escm"),(void*)f_12313},
{C_text("f_12347:support_2escm"),(void*)f_12347},
{C_text("f_12350:support_2escm"),(void*)f_12350},
{C_text("f_12375:support_2escm"),(void*)f_12375},
{C_text("f_12381:support_2escm"),(void*)f_12381},
{C_text("f_12407:support_2escm"),(void*)f_12407},
{C_text("f_12440:support_2escm"),(void*)f_12440},
{C_text("f_12442:support_2escm"),(void*)f_12442},
{C_text("f_12448:support_2escm"),(void*)f_12448},
{C_text("f_12454:support_2escm"),(void*)f_12454},
{C_text("f_12458:support_2escm"),(void*)f_12458},
{C_text("f_12481:support_2escm"),(void*)f_12481},
{C_text("f_12492:support_2escm"),(void*)f_12492},
{C_text("f_12498:support_2escm"),(void*)f_12498},
{C_text("f_12501:support_2escm"),(void*)f_12501},
{C_text("f_12509:support_2escm"),(void*)f_12509},
{C_text("f_12535:support_2escm"),(void*)f_12535},
{C_text("f_12557:support_2escm"),(void*)f_12557},
{C_text("f_12582:support_2escm"),(void*)f_12582},
{C_text("f_12604:support_2escm"),(void*)f_12604},
{C_text("f_12622:support_2escm"),(void*)f_12622},
{C_text("f_12653:support_2escm"),(void*)f_12653},
{C_text("f_12705:support_2escm"),(void*)f_12705},
{C_text("f_12711:support_2escm"),(void*)f_12711},
{C_text("f_12731:support_2escm"),(void*)f_12731},
{C_text("f_12737:support_2escm"),(void*)f_12737},
{C_text("f_12763:support_2escm"),(void*)f_12763},
{C_text("f_12777:support_2escm"),(void*)f_12777},
{C_text("f_12785:support_2escm"),(void*)f_12785},
{C_text("f_12840:support_2escm"),(void*)f_12840},
{C_text("f_12869:support_2escm"),(void*)f_12869},
{C_text("f_12966:support_2escm"),(void*)f_12966},
{C_text("f_12972:support_2escm"),(void*)f_12972},
{C_text("f_12979:support_2escm"),(void*)f_12979},
{C_text("f_12982:support_2escm"),(void*)f_12982},
{C_text("f_13005:support_2escm"),(void*)f_13005},
{C_text("f_13007:support_2escm"),(void*)f_13007},
{C_text("f_13013:support_2escm"),(void*)f_13013},
{C_text("f_13020:support_2escm"),(void*)f_13020},
{C_text("f_13023:support_2escm"),(void*)f_13023},
{C_text("f_13042:support_2escm"),(void*)f_13042},
{C_text("f_13044:support_2escm"),(void*)f_13044},
{C_text("f_13050:support_2escm"),(void*)f_13050},
{C_text("f_13063:support_2escm"),(void*)f_13063},
{C_text("f_13091:support_2escm"),(void*)f_13091},
{C_text("f_13093:support_2escm"),(void*)f_13093},
{C_text("f_13114:support_2escm"),(void*)f_13114},
{C_text("f_13121:support_2escm"),(void*)f_13121},
{C_text("f_13127:support_2escm"),(void*)f_13127},
{C_text("f_13133:support_2escm"),(void*)f_13133},
{C_text("f_13142:support_2escm"),(void*)f_13142},
{C_text("f_13151:support_2escm"),(void*)f_13151},
{C_text("f_13160:support_2escm"),(void*)f_13160},
{C_text("f_13169:support_2escm"),(void*)f_13169},
{C_text("f_13178:support_2escm"),(void*)f_13178},
{C_text("f_13204:support_2escm"),(void*)f_13204},
{C_text("f_13207:support_2escm"),(void*)f_13207},
{C_text("f_13218:support_2escm"),(void*)f_13218},
{C_text("f_13220:support_2escm"),(void*)f_13220},
{C_text("f_13274:support_2escm"),(void*)f_13274},
{C_text("f_13280:support_2escm"),(void*)f_13280},
{C_text("f_13286:support_2escm"),(void*)f_13286},
{C_text("f_13292:support_2escm"),(void*)f_13292},
{C_text("f_13317:support_2escm"),(void*)f_13317},
{C_text("f_13332:support_2escm"),(void*)f_13332},
{C_text("f_13350:support_2escm"),(void*)f_13350},
{C_text("f_13400:support_2escm"),(void*)f_13400},
{C_text("f_13415:support_2escm"),(void*)f_13415},
{C_text("f_13455:support_2escm"),(void*)f_13455},
{C_text("f_13458:support_2escm"),(void*)f_13458},
{C_text("f_13473:support_2escm"),(void*)f_13473},
{C_text("f_13497:support_2escm"),(void*)f_13497},
{C_text("f_13523:support_2escm"),(void*)f_13523},
{C_text("f_13529:support_2escm"),(void*)f_13529},
{C_text("f_13535:support_2escm"),(void*)f_13535},
{C_text("f_13538:support_2escm"),(void*)f_13538},
{C_text("f_13541:support_2escm"),(void*)f_13541},
{C_text("f_13544:support_2escm"),(void*)f_13544},
{C_text("f_13566:support_2escm"),(void*)f_13566},
{C_text("f_13572:support_2escm"),(void*)f_13572},
{C_text("f_13578:support_2escm"),(void*)f_13578},
{C_text("f_13581:support_2escm"),(void*)f_13581},
{C_text("f_13584:support_2escm"),(void*)f_13584},
{C_text("f_13587:support_2escm"),(void*)f_13587},
{C_text("f_13610:support_2escm"),(void*)f_13610},
{C_text("f_13613:support_2escm"),(void*)f_13613},
{C_text("f_13654:support_2escm"),(void*)f_13654},
{C_text("f_13657:support_2escm"),(void*)f_13657},
{C_text("f_13672:support_2escm"),(void*)f_13672},
{C_text("f_13699:support_2escm"),(void*)f_13699},
{C_text("f_13742:support_2escm"),(void*)f_13742},
{C_text("f_13746:support_2escm"),(void*)f_13746},
{C_text("f_13773:support_2escm"),(void*)f_13773},
{C_text("f_13776:support_2escm"),(void*)f_13776},
{C_text("f_13811:support_2escm"),(void*)f_13811},
{C_text("f_13847:support_2escm"),(void*)f_13847},
{C_text("f_14350:support_2escm"),(void*)f_14350},
{C_text("f_14356:support_2escm"),(void*)f_14356},
{C_text("f_14366:support_2escm"),(void*)f_14366},
{C_text("f_14377:support_2escm"),(void*)f_14377},
{C_text("f_14387:support_2escm"),(void*)f_14387},
{C_text("f_14398:support_2escm"),(void*)f_14398},
{C_text("f_14402:support_2escm"),(void*)f_14402},
{C_text("f_14413:support_2escm"),(void*)f_14413},
{C_text("f_14417:support_2escm"),(void*)f_14417},
{C_text("f_14428:support_2escm"),(void*)f_14428},
{C_text("f_14434:support_2escm"),(void*)f_14434},
{C_text("f_14438:support_2escm"),(void*)f_14438},
{C_text("f_14442:support_2escm"),(void*)f_14442},
{C_text("f_14461:support_2escm"),(void*)f_14461},
{C_text("f_14467:support_2escm"),(void*)f_14467},
{C_text("f_14470:support_2escm"),(void*)f_14470},
{C_text("f_14479:support_2escm"),(void*)f_14479},
{C_text("f_14489:support_2escm"),(void*)f_14489},
{C_text("f_14498:support_2escm"),(void*)f_14498},
{C_text("f_14510:support_2escm"),(void*)f_14510},
{C_text("f_14522:support_2escm"),(void*)f_14522},
{C_text("f_14534:support_2escm"),(void*)f_14534},
{C_text("f_14540:support_2escm"),(void*)f_14540},
{C_text("f_14544:support_2escm"),(void*)f_14544},
{C_text("f_14571:support_2escm"),(void*)f_14571},
{C_text("f_14936:support_2escm"),(void*)f_14936},
{C_text("f_14942:support_2escm"),(void*)f_14942},
{C_text("f_14954:support_2escm"),(void*)f_14954},
{C_text("f_14964:support_2escm"),(void*)f_14964},
{C_text("f_14976:support_2escm"),(void*)f_14976},
{C_text("f_14982:support_2escm"),(void*)f_14982},
{C_text("f_14986:support_2escm"),(void*)f_14986},
{C_text("f_15013:support_2escm"),(void*)f_15013},
{C_text("f_15386:support_2escm"),(void*)f_15386},
{C_text("f_15392:support_2escm"),(void*)f_15392},
{C_text("f_15396:support_2escm"),(void*)f_15396},
{C_text("f_15512:support_2escm"),(void*)f_15512},
{C_text("f_15540:support_2escm"),(void*)f_15540},
{C_text("f_15660:support_2escm"),(void*)f_15660},
{C_text("f_15667:support_2escm"),(void*)f_15667},
{C_text("f_15670:support_2escm"),(void*)f_15670},
{C_text("f_15673:support_2escm"),(void*)f_15673},
{C_text("f_15697:support_2escm"),(void*)f_15697},
{C_text("f_15772:support_2escm"),(void*)f_15772},
{C_text("f_15859:support_2escm"),(void*)f_15859},
{C_text("f_15892:support_2escm"),(void*)f_15892},
{C_text("f_15910:support_2escm"),(void*)f_15910},
{C_text("f_15932:support_2escm"),(void*)f_15932},
{C_text("f_16314:support_2escm"),(void*)f_16314},
{C_text("f_16318:support_2escm"),(void*)f_16318},
{C_text("f_16320:support_2escm"),(void*)f_16320},
{C_text("f_16352:support_2escm"),(void*)f_16352},
{C_text("f_16360:support_2escm"),(void*)f_16360},
{C_text("f_16370:support_2escm"),(void*)f_16370},
{C_text("f_16416:support_2escm"),(void*)f_16416},
{C_text("f_16424:support_2escm"),(void*)f_16424},
{C_text("f_16434:support_2escm"),(void*)f_16434},
{C_text("f_16469:support_2escm"),(void*)f_16469},
{C_text("f_16472:support_2escm"),(void*)f_16472},
{C_text("f_16506:support_2escm"),(void*)f_16506},
{C_text("f_16525:support_2escm"),(void*)f_16525},
{C_text("f_16531:support_2escm"),(void*)f_16531},
{C_text("f_16535:support_2escm"),(void*)f_16535},
{C_text("f_16561:support_2escm"),(void*)f_16561},
{C_text("f_16570:support_2escm"),(void*)f_16570},
{C_text("f_16581:support_2escm"),(void*)f_16581},
{C_text("f_16600:support_2escm"),(void*)f_16600},
{C_text("f_16612:support_2escm"),(void*)f_16612},
{C_text("f_16656:support_2escm"),(void*)f_16656},
{C_text("f_16658:support_2escm"),(void*)f_16658},
{C_text("f_16670:support_2escm"),(void*)f_16670},
{C_text("f_16680:support_2escm"),(void*)f_16680},
{C_text("f_16694:support_2escm"),(void*)f_16694},
{C_text("f_16699:support_2escm"),(void*)f_16699},
{C_text("f_16723:support_2escm"),(void*)f_16723},
{C_text("f_16729:support_2escm"),(void*)f_16729},
{C_text("f_16735:support_2escm"),(void*)f_16735},
{C_text("f_16744:support_2escm"),(void*)f_16744},
{C_text("f_16752:support_2escm"),(void*)f_16752},
{C_text("f_16758:support_2escm"),(void*)f_16758},
{C_text("f_16761:support_2escm"),(void*)f_16761},
{C_text("f_16764:support_2escm"),(void*)f_16764},
{C_text("f_16767:support_2escm"),(void*)f_16767},
{C_text("f_16770:support_2escm"),(void*)f_16770},
{C_text("f_16775:support_2escm"),(void*)f_16775},
{C_text("f_16779:support_2escm"),(void*)f_16779},
{C_text("f_16791:support_2escm"),(void*)f_16791},
{C_text("f_16796:support_2escm"),(void*)f_16796},
{C_text("f_16798:support_2escm"),(void*)f_16798},
{C_text("f_16804:support_2escm"),(void*)f_16804},
{C_text("f_16811:support_2escm"),(void*)f_16811},
{C_text("f_16814:support_2escm"),(void*)f_16814},
{C_text("f_16818:support_2escm"),(void*)f_16818},
{C_text("f_16824:support_2escm"),(void*)f_16824},
{C_text("f_16830:support_2escm"),(void*)f_16830},
{C_text("f_16857:support_2escm"),(void*)f_16857},
{C_text("f_16859:support_2escm"),(void*)f_16859},
{C_text("f_16873:support_2escm"),(void*)f_16873},
{C_text("f_16883:support_2escm"),(void*)f_16883},
{C_text("f_16896:support_2escm"),(void*)f_16896},
{C_text("f_16911:support_2escm"),(void*)f_16911},
{C_text("f_16915:support_2escm"),(void*)f_16915},
{C_text("f_16922:support_2escm"),(void*)f_16922},
{C_text("f_16926:support_2escm"),(void*)f_16926},
{C_text("f_16931:support_2escm"),(void*)f_16931},
{C_text("f_16935:support_2escm"),(void*)f_16935},
{C_text("f_16943:support_2escm"),(void*)f_16943},
{C_text("f_16949:support_2escm"),(void*)f_16949},
{C_text("f_16956:support_2escm"),(void*)f_16956},
{C_text("f_16959:support_2escm"),(void*)f_16959},
{C_text("f_16962:support_2escm"),(void*)f_16962},
{C_text("f_16967:support_2escm"),(void*)f_16967},
{C_text("f_16987:support_2escm"),(void*)f_16987},
{C_text("f_16991:support_2escm"),(void*)f_16991},
{C_text("f_17002:support_2escm"),(void*)f_17002},
{C_text("f_17017:support_2escm"),(void*)f_17017},
{C_text("f_17029:support_2escm"),(void*)f_17029},
{C_text("f_17036:support_2escm"),(void*)f_17036},
{C_text("f_17066:support_2escm"),(void*)f_17066},
{C_text("f_17090:support_2escm"),(void*)f_17090},
{C_text("f_17105:support_2escm"),(void*)f_17105},
{C_text("f_17108:support_2escm"),(void*)f_17108},
{C_text("f_17114:support_2escm"),(void*)f_17114},
{C_text("f_17123:support_2escm"),(void*)f_17123},
{C_text("f_17126:support_2escm"),(void*)f_17126},
{C_text("f_17165:support_2escm"),(void*)f_17165},
{C_text("f_17171:support_2escm"),(void*)f_17171},
{C_text("f_17177:support_2escm"),(void*)f_17177},
{C_text("f_17180:support_2escm"),(void*)f_17180},
{C_text("f_17186:support_2escm"),(void*)f_17186},
{C_text("f_17192:support_2escm"),(void*)f_17192},
{C_text("f_17198:support_2escm"),(void*)f_17198},
{C_text("f_17204:support_2escm"),(void*)f_17204},
{C_text("f_17226:support_2escm"),(void*)f_17226},
{C_text("f_17228:support_2escm"),(void*)f_17228},
{C_text("f_17262:support_2escm"),(void*)f_17262},
{C_text("f_17296:support_2escm"),(void*)f_17296},
{C_text("f_17299:support_2escm"),(void*)f_17299},
{C_text("f_17327:support_2escm"),(void*)f_17327},
{C_text("f_17334:support_2escm"),(void*)f_17334},
{C_text("f_17349:support_2escm"),(void*)f_17349},
{C_text("f_17355:support_2escm"),(void*)f_17355},
{C_text("f_17358:support_2escm"),(void*)f_17358},
{C_text("f_17395:support_2escm"),(void*)f_17395},
{C_text("f_17410:support_2escm"),(void*)f_17410},
{C_text("f_17420:support_2escm"),(void*)f_17420},
{C_text("f_17423:support_2escm"),(void*)f_17423},
{C_text("f_17435:support_2escm"),(void*)f_17435},
{C_text("f_17441:support_2escm"),(void*)f_17441},
{C_text("f_17447:support_2escm"),(void*)f_17447},
{C_text("f_17450:support_2escm"),(void*)f_17450},
{C_text("f_17452:support_2escm"),(void*)f_17452},
{C_text("f_17459:support_2escm"),(void*)f_17459},
{C_text("f_17465:support_2escm"),(void*)f_17465},
{C_text("f_17476:support_2escm"),(void*)f_17476},
{C_text("f_17524:support_2escm"),(void*)f_17524},
{C_text("f_17526:support_2escm"),(void*)f_17526},
{C_text("f_17532:support_2escm"),(void*)f_17532},
{C_text("f_17536:support_2escm"),(void*)f_17536},
{C_text("f_17541:support_2escm"),(void*)f_17541},
{C_text("f_17569:support_2escm"),(void*)f_17569},
{C_text("f_17577:support_2escm"),(void*)f_17577},
{C_text("f_17580:support_2escm"),(void*)f_17580},
{C_text("f_17583:support_2escm"),(void*)f_17583},
{C_text("f_17586:support_2escm"),(void*)f_17586},
{C_text("f_17589:support_2escm"),(void*)f_17589},
{C_text("f_17592:support_2escm"),(void*)f_17592},
{C_text("f_17593:support_2escm"),(void*)f_17593},
{C_text("f_17603:support_2escm"),(void*)f_17603},
{C_text("f_17609:support_2escm"),(void*)f_17609},
{C_text("f_17621:support_2escm"),(void*)f_17621},
{C_text("f_17624:support_2escm"),(void*)f_17624},
{C_text("f_17627:support_2escm"),(void*)f_17627},
{C_text("f_17632:support_2escm"),(void*)f_17632},
{C_text("f_17645:support_2escm"),(void*)f_17645},
{C_text("f_17648:support_2escm"),(void*)f_17648},
{C_text("f_17665:support_2escm"),(void*)f_17665},
{C_text("f_17675:support_2escm"),(void*)f_17675},
{C_text("f_17688:support_2escm"),(void*)f_17688},
{C_text("f_17692:support_2escm"),(void*)f_17692},
{C_text("f_17710:support_2escm"),(void*)f_17710},
{C_text("f_17714:support_2escm"),(void*)f_17714},
{C_text("f_17731:support_2escm"),(void*)f_17731},
{C_text("f_17737:support_2escm"),(void*)f_17737},
{C_text("f_17747:support_2escm"),(void*)f_17747},
{C_text("f_17750:support_2escm"),(void*)f_17750},
{C_text("f_17766:support_2escm"),(void*)f_17766},
{C_text("f_17771:support_2escm"),(void*)f_17771},
{C_text("f_17775:support_2escm"),(void*)f_17775},
{C_text("f_17792:support_2escm"),(void*)f_17792},
{C_text("f_17803:support_2escm"),(void*)f_17803},
{C_text("f_17815:support_2escm"),(void*)f_17815},
{C_text("f_17818:support_2escm"),(void*)f_17818},
{C_text("f_17826:support_2escm"),(void*)f_17826},
{C_text("f_17831:support_2escm"),(void*)f_17831},
{C_text("f_17844:support_2escm"),(void*)f_17844},
{C_text("f_17855:support_2escm"),(void*)f_17855},
{C_text("f_17877:support_2escm"),(void*)f_17877},
{C_text("f_17879:support_2escm"),(void*)f_17879},
{C_text("f_17899:support_2escm"),(void*)f_17899},
{C_text("f_17919:support_2escm"),(void*)f_17919},
{C_text("f_17927:support_2escm"),(void*)f_17927},
{C_text("f_17936:support_2escm"),(void*)f_17936},
{C_text("f_17941:support_2escm"),(void*)f_17941},
{C_text("f_17945:support_2escm"),(void*)f_17945},
{C_text("f_17966:support_2escm"),(void*)f_17966},
{C_text("f_17981:support_2escm"),(void*)f_17981},
{C_text("f_17987:support_2escm"),(void*)f_17987},
{C_text("f_17998:support_2escm"),(void*)f_17998},
{C_text("f_18009:support_2escm"),(void*)f_18009},
{C_text("f_18020:support_2escm"),(void*)f_18020},
{C_text("f_18024:support_2escm"),(void*)f_18024},
{C_text("f_18030:support_2escm"),(void*)f_18030},
{C_text("f_18042:support_2escm"),(void*)f_18042},
{C_text("f_18046:support_2escm"),(void*)f_18046},
{C_text("f_18058:support_2escm"),(void*)f_18058},
{C_text("f_18066:support_2escm"),(void*)f_18066},
{C_text("f_18076:support_2escm"),(void*)f_18076},
{C_text("f_18091:support_2escm"),(void*)f_18091},
{C_text("f_18097:support_2escm"),(void*)f_18097},
{C_text("f_18100:support_2escm"),(void*)f_18100},
{C_text("f_18103:support_2escm"),(void*)f_18103},
{C_text("f_18106:support_2escm"),(void*)f_18106},
{C_text("f_18109:support_2escm"),(void*)f_18109},
{C_text("f_18113:support_2escm"),(void*)f_18113},
{C_text("f_18115:support_2escm"),(void*)f_18115},
{C_text("f_18122:support_2escm"),(void*)f_18122},
{C_text("f_18129:support_2escm"),(void*)f_18129},
{C_text("f_18140:support_2escm"),(void*)f_18140},
{C_text("f_18144:support_2escm"),(void*)f_18144},
{C_text("f_18147:support_2escm"),(void*)f_18147},
{C_text("f_18152:support_2escm"),(void*)f_18152},
{C_text("f_18158:support_2escm"),(void*)f_18158},
{C_text("f_18165:support_2escm"),(void*)f_18165},
{C_text("f_18168:support_2escm"),(void*)f_18168},
{C_text("f_18171:support_2escm"),(void*)f_18171},
{C_text("f_18174:support_2escm"),(void*)f_18174},
{C_text("f_5348:support_2escm"),(void*)f_5348},
{C_text("f_5351:support_2escm"),(void*)f_5351},
{C_text("f_5354:support_2escm"),(void*)f_5354},
{C_text("f_5357:support_2escm"),(void*)f_5357},
{C_text("f_5360:support_2escm"),(void*)f_5360},
{C_text("f_5363:support_2escm"),(void*)f_5363},
{C_text("f_5366:support_2escm"),(void*)f_5366},
{C_text("f_5369:support_2escm"),(void*)f_5369},
{C_text("f_5372:support_2escm"),(void*)f_5372},
{C_text("f_5482:support_2escm"),(void*)f_5482},
{C_text("f_5500:support_2escm"),(void*)f_5500},
{C_text("f_5540:support_2escm"),(void*)f_5540},
{C_text("f_5554:support_2escm"),(void*)f_5554},
{C_text("f_5743:support_2escm"),(void*)f_5743},
{C_text("f_5749:support_2escm"),(void*)f_5749},
{C_text("f_5771:support_2escm"),(void*)f_5771},
{C_text("f_5777:support_2escm"),(void*)f_5777},
{C_text("f_5783:support_2escm"),(void*)f_5783},
{C_text("f_5793:support_2escm"),(void*)f_5793},
{C_text("f_5807:support_2escm"),(void*)f_5807},
{C_text("f_5813:support_2escm"),(void*)f_5813},
{C_text("f_5827:support_2escm"),(void*)f_5827},
{C_text("f_6036:support_2escm"),(void*)f_6036},
{C_text("f_6044:support_2escm"),(void*)f_6044},
{C_text("f_6052:support_2escm"),(void*)f_6052},
{C_text("f_6070:support_2escm"),(void*)f_6070},
{C_text("f_6109:support_2escm"),(void*)f_6109},
{C_text("f_6143:support_2escm"),(void*)f_6143},
{C_text("f_6149:support_2escm"),(void*)f_6149},
{C_text("f_6202:support_2escm"),(void*)f_6202},
{C_text("f_6208:support_2escm"),(void*)f_6208},
{C_text("f_6391:support_2escm"),(void*)f_6391},
{C_text("f_6409:support_2escm"),(void*)f_6409},
{C_text("f_6504:support_2escm"),(void*)f_6504},
{C_text("f_6517:support_2escm"),(void*)f_6517},
{C_text("f_6655:support_2escm"),(void*)f_6655},
{C_text("f_6659:support_2escm"),(void*)f_6659},
{C_text("f_6673:support_2escm"),(void*)f_6673},
{C_text("f_6684:support_2escm"),(void*)f_6684},
{C_text("f_6687:support_2escm"),(void*)f_6687},
{C_text("f_6702:support_2escm"),(void*)f_6702},
{C_text("f_6708:support_2escm"),(void*)f_6708},
{C_text("f_6711:support_2escm"),(void*)f_6711},
{C_text("f_6717:support_2escm"),(void*)f_6717},
{C_text("f_6721:support_2escm"),(void*)f_6721},
{C_text("f_6724:support_2escm"),(void*)f_6724},
{C_text("f_6733:support_2escm"),(void*)f_6733},
{C_text("f_6741:support_2escm"),(void*)f_6741},
{C_text("f_6748:support_2escm"),(void*)f_6748},
{C_text("f_6753:support_2escm"),(void*)f_6753},
{C_text("f_6763:support_2escm"),(void*)f_6763},
{C_text("f_6776:support_2escm"),(void*)f_6776},
{C_text("f_6783:support_2escm"),(void*)f_6783},
{C_text("f_6786:support_2escm"),(void*)f_6786},
{C_text("f_6795:support_2escm"),(void*)f_6795},
{C_text("f_6798:support_2escm"),(void*)f_6798},
{C_text("f_6801:support_2escm"),(void*)f_6801},
{C_text("f_6804:support_2escm"),(void*)f_6804},
{C_text("f_6807:support_2escm"),(void*)f_6807},
{C_text("f_6810:support_2escm"),(void*)f_6810},
{C_text("f_6816:support_2escm"),(void*)f_6816},
{C_text("f_6819:support_2escm"),(void*)f_6819},
{C_text("f_6826:support_2escm"),(void*)f_6826},
{C_text("f_6828:support_2escm"),(void*)f_6828},
{C_text("f_6831:support_2escm"),(void*)f_6831},
{C_text("f_6833:support_2escm"),(void*)f_6833},
{C_text("f_6840:support_2escm"),(void*)f_6840},
{C_text("f_6843:support_2escm"),(void*)f_6843},
{C_text("f_6846:support_2escm"),(void*)f_6846},
{C_text("f_6860:support_2escm"),(void*)f_6860},
{C_text("f_6865:support_2escm"),(void*)f_6865},
{C_text("f_6875:support_2escm"),(void*)f_6875},
{C_text("f_6892:support_2escm"),(void*)f_6892},
{C_text("f_6895:support_2escm"),(void*)f_6895},
{C_text("f_6898:support_2escm"),(void*)f_6898},
{C_text("f_6901:support_2escm"),(void*)f_6901},
{C_text("f_6907:support_2escm"),(void*)f_6907},
{C_text("f_6916:support_2escm"),(void*)f_6916},
{C_text("f_6923:support_2escm"),(void*)f_6923},
{C_text("f_6925:support_2escm"),(void*)f_6925},
{C_text("f_6929:support_2escm"),(void*)f_6929},
{C_text("f_6932:support_2escm"),(void*)f_6932},
{C_text("f_6939:support_2escm"),(void*)f_6939},
{C_text("f_6941:support_2escm"),(void*)f_6941},
{C_text("f_6945:support_2escm"),(void*)f_6945},
{C_text("f_6948:support_2escm"),(void*)f_6948},
{C_text("f_6949:support_2escm"),(void*)f_6949},
{C_text("f_6959:support_2escm"),(void*)f_6959},
{C_text("f_6962:support_2escm"),(void*)f_6962},
{C_text("f_6967:support_2escm"),(void*)f_6967},
{C_text("f_6977:support_2escm"),(void*)f_6977},
{C_text("f_6994:support_2escm"),(void*)f_6994},
{C_text("f_6997:support_2escm"),(void*)f_6997},
{C_text("f_7000:support_2escm"),(void*)f_7000},
{C_text("f_7003:support_2escm"),(void*)f_7003},
{C_text("f_7006:support_2escm"),(void*)f_7006},
{C_text("f_7015:support_2escm"),(void*)f_7015},
{C_text("f_7018:support_2escm"),(void*)f_7018},
{C_text("f_7021:support_2escm"),(void*)f_7021},
{C_text("f_7038:support_2escm"),(void*)f_7038},
{C_text("f_7092:support_2escm"),(void*)f_7092},
{C_text("f_7098:support_2escm"),(void*)f_7098},
{C_text("f_7133:support_2escm"),(void*)f_7133},
{C_text("f_7139:support_2escm"),(void*)f_7139},
{C_text("f_7161:support_2escm"),(void*)f_7161},
{C_text("f_7170:support_2escm"),(void*)f_7170},
{C_text("f_7182:support_2escm"),(void*)f_7182},
{C_text("f_7186:support_2escm"),(void*)f_7186},
{C_text("f_7188:support_2escm"),(void*)f_7188},
{C_text("f_7210:support_2escm"),(void*)f_7210},
{C_text("f_7217:support_2escm"),(void*)f_7217},
{C_text("f_7221:support_2escm"),(void*)f_7221},
{C_text("f_7225:support_2escm"),(void*)f_7225},
{C_text("f_7231:support_2escm"),(void*)f_7231},
{C_text("f_7253:support_2escm"),(void*)f_7253},
{C_text("f_7269:support_2escm"),(void*)f_7269},
{C_text("f_7273:support_2escm"),(void*)f_7273},
{C_text("f_7294:support_2escm"),(void*)f_7294},
{C_text("f_7317:support_2escm"),(void*)f_7317},
{C_text("f_7319:support_2escm"),(void*)f_7319},
{C_text("f_7326:support_2escm"),(void*)f_7326},
{C_text("f_7333:support_2escm"),(void*)f_7333},
{C_text("f_7346:support_2escm"),(void*)f_7346},
{C_text("f_7377:support_2escm"),(void*)f_7377},
{C_text("f_7389:support_2escm"),(void*)f_7389},
{C_text("f_7403:support_2escm"),(void*)f_7403},
{C_text("f_7405:support_2escm"),(void*)f_7405},
{C_text("f_7431:support_2escm"),(void*)f_7431},
{C_text("f_7445:support_2escm"),(void*)f_7445},
{C_text("f_7451:support_2escm"),(void*)f_7451},
{C_text("f_7466:support_2escm"),(void*)f_7466},
{C_text("f_7482:support_2escm"),(void*)f_7482},
{C_text("f_7490:support_2escm"),(void*)f_7490},
{C_text("f_7494:support_2escm"),(void*)f_7494},
{C_text("f_7496:support_2escm"),(void*)f_7496},
{C_text("f_7507:support_2escm"),(void*)f_7507},
{C_text("f_7509:support_2escm"),(void*)f_7509},
{C_text("f_7526:support_2escm"),(void*)f_7526},
{C_text("f_7540:support_2escm"),(void*)f_7540},
{C_text("f_7574:support_2escm"),(void*)f_7574},
{C_text("f_7586:support_2escm"),(void*)f_7586},
{C_text("f_7602:support_2escm"),(void*)f_7602},
{C_text("f_7632:support_2escm"),(void*)f_7632},
{C_text("f_7636:support_2escm"),(void*)f_7636},
{C_text("f_7676:support_2escm"),(void*)f_7676},
{C_text("f_7678:support_2escm"),(void*)f_7678},
{C_text("f_7694:support_2escm"),(void*)f_7694},
{C_text("f_7700:support_2escm"),(void*)f_7700},
{C_text("f_7715:support_2escm"),(void*)f_7715},
{C_text("f_7732:support_2escm"),(void*)f_7732},
{C_text("f_7734:support_2escm"),(void*)f_7734},
{C_text("f_7740:support_2escm"),(void*)f_7740},
{C_text("f_7764:support_2escm"),(void*)f_7764},
{C_text("f_7780:support_2escm"),(void*)f_7780},
{C_text("f_7790:support_2escm"),(void*)f_7790},
{C_text("f_7795:support_2escm"),(void*)f_7795},
{C_text("f_7809:support_2escm"),(void*)f_7809},
{C_text("f_7812:support_2escm"),(void*)f_7812},
{C_text("f_7813:support_2escm"),(void*)f_7813},
{C_text("f_7817:support_2escm"),(void*)f_7817},
{C_text("f_7822:support_2escm"),(void*)f_7822},
{C_text("f_7828:support_2escm"),(void*)f_7828},
{C_text("f_7834:support_2escm"),(void*)f_7834},
{C_text("f_7842:support_2escm"),(void*)f_7842},
{C_text("f_7845:support_2escm"),(void*)f_7845},
{C_text("f_7853:support_2escm"),(void*)f_7853},
{C_text("f_7855:support_2escm"),(void*)f_7855},
{C_text("f_7859:support_2escm"),(void*)f_7859},
{C_text("f_7881:support_2escm"),(void*)f_7881},
{C_text("f_7887:support_2escm"),(void*)f_7887},
{C_text("f_7891:support_2escm"),(void*)f_7891},
{C_text("f_7904:support_2escm"),(void*)f_7904},
{C_text("f_7912:support_2escm"),(void*)f_7912},
{C_text("f_7918:support_2escm"),(void*)f_7918},
{C_text("f_7929:support_2escm"),(void*)f_7929},
{C_text("f_7931:support_2escm"),(void*)f_7931},
{C_text("f_7934:support_2escm"),(void*)f_7934},
{C_text("f_7940:support_2escm"),(void*)f_7940},
{C_text("f_7979:support_2escm"),(void*)f_7979},
{C_text("f_7984:support_2escm"),(void*)f_7984},
{C_text("f_7988:support_2escm"),(void*)f_7988},
{C_text("f_7992:support_2escm"),(void*)f_7992},
{C_text("f_8043:support_2escm"),(void*)f_8043},
{C_text("f_8080:support_2escm"),(void*)f_8080},
{C_text("f_8082:support_2escm"),(void*)f_8082},
{C_text("f_8132:support_2escm"),(void*)f_8132},
{C_text("f_8136:support_2escm"),(void*)f_8136},
{C_text("f_8150:support_2escm"),(void*)f_8150},
{C_text("f_8154:support_2escm"),(void*)f_8154},
{C_text("f_8162:support_2escm"),(void*)f_8162},
{C_text("f_8168:support_2escm"),(void*)f_8168},
{C_text("f_8172:support_2escm"),(void*)f_8172},
{C_text("f_8214:support_2escm"),(void*)f_8214},
{C_text("f_8218:support_2escm"),(void*)f_8218},
{C_text("f_8266:support_2escm"),(void*)f_8266},
{C_text("f_8270:support_2escm"),(void*)f_8270},
{C_text("f_8275:support_2escm"),(void*)f_8275},
{C_text("f_8285:support_2escm"),(void*)f_8285},
{C_text("f_8292:support_2escm"),(void*)f_8292},
{C_text("f_8299:support_2escm"),(void*)f_8299},
{C_text("f_8326:support_2escm"),(void*)f_8326},
{C_text("f_8332:support_2escm"),(void*)f_8332},
{C_text("f_8342:support_2escm"),(void*)f_8342},
{C_text("f_8345:support_2escm"),(void*)f_8345},
{C_text("f_8348:support_2escm"),(void*)f_8348},
{C_text("f_8361:support_2escm"),(void*)f_8361},
{C_text("f_8363:support_2escm"),(void*)f_8363},
{C_text("f_8398:support_2escm"),(void*)f_8398},
{C_text("f_8404:support_2escm"),(void*)f_8404},
{C_text("f_8410:support_2escm"),(void*)f_8410},
{C_text("f_8419:support_2escm"),(void*)f_8419},
{C_text("f_8428:support_2escm"),(void*)f_8428},
{C_text("f_8437:support_2escm"),(void*)f_8437},
{C_text("f_8446:support_2escm"),(void*)f_8446},
{C_text("f_8455:support_2escm"),(void*)f_8455},
{C_text("f_8465:support_2escm"),(void*)f_8465},
{C_text("f_8467:support_2escm"),(void*)f_8467},
{C_text("f_8473:support_2escm"),(void*)f_8473},
{C_text("f_8488:support_2escm"),(void*)f_8488},
{C_text("f_8503:support_2escm"),(void*)f_8503},
{C_text("f_8506:support_2escm"),(void*)f_8506},
{C_text("f_8573:support_2escm"),(void*)f_8573},
{C_text("f_8575:support_2escm"),(void*)f_8575},
{C_text("f_8600:support_2escm"),(void*)f_8600},
{C_text("f_8623:support_2escm"),(void*)f_8623},
{C_text("f_8626:support_2escm"),(void*)f_8626},
{C_text("f_8629:support_2escm"),(void*)f_8629},
{C_text("f_8636:support_2escm"),(void*)f_8636},
{C_text("f_8683:support_2escm"),(void*)f_8683},
{C_text("f_8687:support_2escm"),(void*)f_8687},
{C_text("f_8692:support_2escm"),(void*)f_8692},
{C_text("f_8709:support_2escm"),(void*)f_8709},
{C_text("f_8717:support_2escm"),(void*)f_8717},
{C_text("f_8719:support_2escm"),(void*)f_8719},
{C_text("f_8744:support_2escm"),(void*)f_8744},
{C_text("f_8780:support_2escm"),(void*)f_8780},
{C_text("f_8814:support_2escm"),(void*)f_8814},
{C_text("f_8845:support_2escm"),(void*)f_8845},
{C_text("f_8868:support_2escm"),(void*)f_8868},
{C_text("f_8889:support_2escm"),(void*)f_8889},
{C_text("f_8911:support_2escm"),(void*)f_8911},
{C_text("f_8919:support_2escm"),(void*)f_8919},
{C_text("f_8923:support_2escm"),(void*)f_8923},
{C_text("f_8931:support_2escm"),(void*)f_8931},
{C_text("f_8952:support_2escm"),(void*)f_8952},
{C_text("f_8956:support_2escm"),(void*)f_8956},
{C_text("f_8968:support_2escm"),(void*)f_8968},
{C_text("f_8995:support_2escm"),(void*)f_8995},
{C_text("f_9007:support_2escm"),(void*)f_9007},
{C_text("f_9009:support_2escm"),(void*)f_9009},
{C_text("f_9034:support_2escm"),(void*)f_9034},
{C_text("f_9068:support_2escm"),(void*)f_9068},
{C_text("f_9094:support_2escm"),(void*)f_9094},
{C_text("f_9096:support_2escm"),(void*)f_9096},
{C_text("f_9121:support_2escm"),(void*)f_9121},
{C_text("f_9205:support_2escm"),(void*)f_9205},
{C_text("f_9207:support_2escm"),(void*)f_9207},
{C_text("f_9232:support_2escm"),(void*)f_9232},
{C_text("f_9272:support_2escm"),(void*)f_9272},
{C_text("f_9313:support_2escm"),(void*)f_9313},
{C_text("f_9342:support_2escm"),(void*)f_9342},
{C_text("f_9344:support_2escm"),(void*)f_9344},
{C_text("f_9369:support_2escm"),(void*)f_9369},
{C_text("f_9405:support_2escm"),(void*)f_9405},
{C_text("f_9407:support_2escm"),(void*)f_9407},
{C_text("f_9432:support_2escm"),(void*)f_9432},
{C_text("f_9444:support_2escm"),(void*)f_9444},
{C_text("f_9450:support_2escm"),(void*)f_9450},
{C_text("f_9473:support_2escm"),(void*)f_9473},
{C_text("f_9475:support_2escm"),(void*)f_9475},
{C_text("f_9500:support_2escm"),(void*)f_9500},
{C_text("f_9511:support_2escm"),(void*)f_9511},
{C_text("f_9515:support_2escm"),(void*)f_9515},
{C_text("f_9518:support_2escm"),(void*)f_9518},
{C_text("f_9525:support_2escm"),(void*)f_9525},
{C_text("f_9539:support_2escm"),(void*)f_9539},
{C_text("f_9645:support_2escm"),(void*)f_9645},
{C_text("f_9647:support_2escm"),(void*)f_9647},
{C_text("f_9672:support_2escm"),(void*)f_9672},
{C_text("f_9690:support_2escm"),(void*)f_9690},
{C_text("f_9693:support_2escm"),(void*)f_9693},
{C_text("f_9699:support_2escm"),(void*)f_9699},
{C_text("f_9705:support_2escm"),(void*)f_9705},
{C_text("f_9739:support_2escm"),(void*)f_9739},
{C_text("f_9752:support_2escm"),(void*)f_9752},
{C_text("f_9754:support_2escm"),(void*)f_9754},
{C_text("f_9779:support_2escm"),(void*)f_9779},
{C_text("f_9809:support_2escm"),(void*)f_9809},
{C_text("f_9811:support_2escm"),(void*)f_9811},
{C_text("f_9836:support_2escm"),(void*)f_9836},
{C_text("f_9909:support_2escm"),(void*)f_9909},
{C_text("f_9912:support_2escm"),(void*)f_9912},
{C_text("f_9921:support_2escm"),(void*)f_9921},
{C_text("f_9925:support_2escm"),(void*)f_9925},
{C_text("f_9929:support_2escm"),(void*)f_9929},
{C_text("f_9931:support_2escm"),(void*)f_9931},
{C_text("f_9979:support_2escm"),(void*)f_9979},
{C_text("toplevel:support_2escm"),(void*)C_support_toplevel},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.compiler.support#partition 
o|hiding unexported module binding: chicken.compiler.support#span 
o|hiding unexported module binding: chicken.compiler.support#take 
o|hiding unexported module binding: chicken.compiler.support#drop 
o|hiding unexported module binding: chicken.compiler.support#split-at 
o|hiding unexported module binding: chicken.compiler.support#append-map 
o|hiding unexported module binding: chicken.compiler.support#every 
o|hiding unexported module binding: chicken.compiler.support#any 
o|hiding unexported module binding: chicken.compiler.support#cons* 
o|hiding unexported module binding: chicken.compiler.support#concatenate 
o|hiding unexported module binding: chicken.compiler.support#delete 
o|hiding unexported module binding: chicken.compiler.support#first 
o|hiding unexported module binding: chicken.compiler.support#second 
o|hiding unexported module binding: chicken.compiler.support#third 
o|hiding unexported module binding: chicken.compiler.support#fourth 
o|hiding unexported module binding: chicken.compiler.support#fifth 
o|hiding unexported module binding: chicken.compiler.support#delete-duplicates 
o|hiding unexported module binding: chicken.compiler.support#alist-cons 
o|hiding unexported module binding: chicken.compiler.support#filter 
o|hiding unexported module binding: chicken.compiler.support#filter-map 
o|hiding unexported module binding: chicken.compiler.support#remove 
o|hiding unexported module binding: chicken.compiler.support#unzip1 
o|hiding unexported module binding: chicken.compiler.support#last 
o|hiding unexported module binding: chicken.compiler.support#list-index 
o|hiding unexported module binding: chicken.compiler.support#lset-adjoin/eq? 
o|hiding unexported module binding: chicken.compiler.support#lset-difference/eq? 
o|hiding unexported module binding: chicken.compiler.support#lset-union/eq? 
o|hiding unexported module binding: chicken.compiler.support#lset-intersection/eq? 
o|hiding unexported module binding: chicken.compiler.support#list-tabulate 
o|hiding unexported module binding: chicken.compiler.support#lset<=/eq? 
o|hiding unexported module binding: chicken.compiler.support#lset=/eq? 
o|hiding unexported module binding: chicken.compiler.support#length+ 
o|hiding unexported module binding: chicken.compiler.support#find 
o|hiding unexported module binding: chicken.compiler.support#find-tail 
o|hiding unexported module binding: chicken.compiler.support#iota 
o|hiding unexported module binding: chicken.compiler.support#make-list 
o|hiding unexported module binding: chicken.compiler.support#posq 
o|hiding unexported module binding: chicken.compiler.support#posv 
o|hiding unexported module binding: chicken.compiler.support#constant678 
o|hiding unexported module binding: chicken.compiler.support#+logged-debugging-modes+ 
o|hiding unexported module binding: chicken.compiler.support#test-debugging-mode 
o|hiding unexported module binding: chicken.compiler.support#map-llist 
o|hiding unexported module binding: chicken.compiler.support#follow-without-loop 
o|hiding unexported module binding: chicken.compiler.support#sort-symbols 
o|hiding unexported module binding: chicken.compiler.support#profile-info-vector-name 
o|hiding unexported module binding: chicken.compiler.support#profile-lambda-list 
o|hiding unexported module binding: chicken.compiler.support#profile-lambda-index 
o|hiding unexported module binding: chicken.compiler.support#node 
o|hiding unexported module binding: chicken.compiler.support#copy-node-tree-and-rename 
o|hiding unexported module binding: chicken.compiler.support#replace-rest-ops-in-known-call! 
o|hiding unexported module binding: chicken.compiler.support#node->sexpr 
o|hiding unexported module binding: chicken.compiler.support#sexpr->node 
o|hiding unexported module binding: chicken.compiler.support#foreign-callback-stub 
o|hiding unexported module binding: chicken.compiler.support#foreign-type-table 
o|hiding unexported module binding: chicken.compiler.support#foreign-type-result-converter 
o|hiding unexported module binding: chicken.compiler.support#foreign-type-argument-converter 
o|hiding unexported module binding: chicken.compiler.support#block-variable-literal 
o|hiding unexported module binding: chicken.compiler.support#real-name-table 
o|hiding unexported module binding: chicken.compiler.support#real-name-max-depth 
o|hiding unexported module binding: chicken.compiler.support#encodeable-literal? 
o|hiding unexported module binding: chicken.compiler.support#scan-sharp-greater-string 
o|hiding unexported module binding: chicken.compiler.support#unhide-variable 
S|applied compiler syntax:
S|  chicken.format#sprintf		4
S|  chicken.format#fprintf		5
S|  chicken.format#printf		6
S|  scheme#for-each		12
S|  chicken.base#foldl		3
S|  scheme#map		34
S|  chicken.base#foldr		3
o|eliminated procedure checks: 452 
o|specializations:
o|  1 (scheme#eqv? (or eof null fixnum char boolean symbol keyword) *)
o|  1 (scheme#+ fixnum fixnum)
o|  1 (scheme#number->string * *)
o|  1 (chicken.base#exact-integer? *)
o|  2 (chicken.bitwise#integer-length *)
o|  1 (scheme#length list)
o|  1 (scheme#- fixnum fixnum)
o|  1 (scheme#> integer integer)
o|  1 (chicken.base#sub1 fixnum)
o|  4 (scheme#= fixnum fixnum)
o|  2 (scheme#assq * (list-of pair))
o|  1 (scheme#* fixnum fixnum)
o|  1 (scheme#positive? *)
o|  4 (scheme#cddr (pair * pair))
o|  1 (scheme#caddr (pair * (pair * pair)))
o|  1 (scheme#integer? *)
o|  338 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  2 (##sys#call-with-values (procedure () *) *)
o|  5 (chicken.base#add1 *)
o|  1 (scheme#cadr (pair * pair))
o|  2 (scheme#current-input-port)
o|  4 (scheme#char=? char char)
o|  1 (scheme#number->string fixnum fixnum)
o|  3 (scheme#memq * list)
o|  1 (scheme#>= fixnum fixnum)
o|  3 (scheme#< fixnum fixnum)
o|  3 (chicken.base#sub1 *)
o|  4 (scheme#zero? *)
o|  2 (chicken.base#current-error-port)
o|  15 (##sys#check-output-port * * *)
o|  1 (scheme#eqv? * *)
o|  9 (##sys#check-list (or pair list) *)
o|  55 (scheme#cdr pair)
o|  34 (scheme#car pair)
(o e)|safe calls: 1902 
(o e)|assignments to immediate values: 5 
o|safe globals: (chicken.compiler.support#bomb chicken.compiler.support#debugging-chicken chicken.compiler.support#compiler-cleanup-hook chicken.compiler.support#unsafe chicken.compiler.support#number-type chicken.compiler.support#constant678 chicken.compiler.support#posv chicken.compiler.support#posq chicken.compiler.support#make-list chicken.compiler.support#iota chicken.compiler.support#find-tail chicken.compiler.support#find chicken.compiler.support#length+ chicken.compiler.support#lset=/eq? chicken.compiler.support#lset<=/eq? chicken.compiler.support#list-tabulate chicken.compiler.support#lset-intersection/eq? chicken.compiler.support#lset-union/eq? chicken.compiler.support#lset-difference/eq? chicken.compiler.support#lset-adjoin/eq? chicken.compiler.support#list-index chicken.compiler.support#last chicken.compiler.support#unzip1 chicken.compiler.support#remove chicken.compiler.support#filter-map chicken.compiler.support#filter chicken.compiler.support#alist-cons chicken.compiler.support#delete-duplicates chicken.compiler.support#fifth chicken.compiler.support#fourth chicken.compiler.support#third chicken.compiler.support#second chicken.compiler.support#first chicken.compiler.support#delete chicken.compiler.support#concatenate chicken.compiler.support#cons* chicken.compiler.support#any chicken.compiler.support#every chicken.compiler.support#append-map chicken.compiler.support#split-at chicken.compiler.support#drop chicken.compiler.support#take chicken.compiler.support#span chicken.compiler.support#partition) 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#partition 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#span 
o|inlining procedure: k5484 
o|inlining procedure: k5484 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#drop 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#append-map 
o|inlining procedure: k5754 
o|inlining procedure: k5754 
o|inlining procedure: k5785 
o|inlining procedure: k5785 
o|merged explicitly consed rest parameter: xs339 
o|inlining procedure: k5815 
o|inlining procedure: k5815 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#concatenate 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#delete-duplicates 
o|inlining procedure: k6002 
o|inlining procedure: k6002 
o|inlining procedure: k5994 
o|inlining procedure: k5994 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#remove 
o|inlining procedure: k6151 
o|inlining procedure: k6151 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#list-index 
o|merged explicitly consed rest parameter: vals479 
o|inlining procedure: k6210 
o|inlining procedure: k6210 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#lset-difference/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#lset-union/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#lset-intersection/eq? 
o|inlining procedure: k6393 
o|inlining procedure: k6393 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#lset<=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#lset=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#length+ 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#find-tail 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#iota 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#make-list 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#posq 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#posv 
o|inlining procedure: k6661 
o|inlining procedure: k6661 
o|inlining procedure: k6689 
o|inlining procedure: k6689 
o|inlining procedure: k6722 
o|inlining procedure: k6755 
o|contracted procedure: "(support.scm:137) g705712" 
o|propagated global variable: out715718 ##sys#standard-output 
o|substituted constant variable: a6737 
o|substituted constant variable: a6738 
o|inlining procedure: k6755 
o|inlining procedure: k6722 
o|propagated global variable: out730733 chicken.compiler.support#collected-debugging-output 
o|substituted constant variable: a6779 
o|substituted constant variable: a6780 
o|propagated global variable: out730733 chicken.compiler.support#collected-debugging-output 
o|inlining procedure: k6790 
o|inlining procedure: k6790 
o|propagated global variable: out766769 chicken.compiler.support#collected-debugging-output 
o|substituted constant variable: a6836 
o|substituted constant variable: a6837 
o|inlining procedure: k6851 
o|inlining procedure: k6851 
o|inlining procedure: k6867 
o|inlining procedure: k6867 
o|inlining procedure: k6887 
o|inlining procedure: k6887 
o|inlining procedure: k6969 
o|inlining procedure: k6969 
o|substituted constant variable: a6990 
o|substituted constant variable: a6991 
o|substituted constant variable: a7011 
o|substituted constant variable: a7012 
o|contracted procedure: "(support.scm:198) thread-id846" 
o|propagated global variable: t847 ##sys#current-thread 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#map-llist 
o|inlining procedure: k7100 
o|inlining procedure: k7100 
o|inlining procedure: k7115 
o|inlining procedure: k7115 
o|inlining procedure: k7141 
o|inlining procedure: k7141 
o|inlining procedure: k7190 
o|inlining procedure: k7190 
o|substituted constant variable: a7233 
o|inlining procedure: k7237 
o|inlining procedure: k7237 
o|substituted constant variable: a7244 
o|substituted constant variable: a7246 
o|inlining procedure: k7259 
o|inlining procedure: k7259 
o|substituted constant variable: a7263 
o|substituted constant variable: a7265 
o|substituted constant variable: a7267 
o|inlining procedure: k7274 
o|inlining procedure: k7299 
o|inlining procedure: k7299 
o|substituted constant variable: a7308 
o|substituted constant variable: a7312 
o|inlining procedure: k7274 
o|inlining procedure: k7335 
o|propagated global variable: r733618258 ##sys#standard-input 
o|inlining procedure: k7335 
o|inlining procedure: k7350 
o|inlining procedure: k7350 
o|inlining procedure: k7379 
o|inlining procedure: k7379 
o|inlining procedure: k7391 
o|inlining procedure: k7391 
o|inlining procedure: k7411 
o|inlining procedure: k7411 
o|inlining procedure: k7453 
o|inlining procedure: k7453 
o|inlining procedure: k7511 
o|inlining procedure: k7511 
o|inlining procedure: k7545 
o|inlining procedure: k7545 
o|inlining procedure: k7557 
o|inlining procedure: k7557 
o|inlining procedure: k7569 
o|inlining procedure: k7569 
o|inlining procedure: k7581 
o|inlining procedure: k7581 
o|inlining procedure: k7590 
o|inlining procedure: k7590 
o|inlining procedure: k7607 
o|inlining procedure: k7607 
o|inlining procedure: k7619 
o|inlining procedure: k7619 
o|inlining procedure: k7637 
o|inlining procedure: k7637 
o|inlining procedure: k7649 
o|inlining procedure: k7649 
o|inlining procedure: k7661 
o|inlining procedure: k7661 
o|inlining procedure: k7683 
o|inlining procedure: k7683 
o|inlining procedure: k7695 
o|inlining procedure: k7695 
o|inlining procedure: k7704 
o|inlining procedure: k7704 
o|inlining procedure: k7742 
o|inlining procedure: k7742 
o|inlining procedure: k7755 
o|inlining procedure: k7755 
o|inlining procedure: k7796 
o|inlining procedure: k7796 
o|inlining procedure: k7840 
o|inlining procedure: k7840 
o|inlining procedure: k7860 
o|inlining procedure: k7860 
o|inlining procedure: k7892 
o|inlining procedure: k7892 
o|merged explicitly consed rest parameter: args10841102 
o|consed rest parameter at call site: tmp24966 1 
o|inlining procedure: k7942 
o|inlining procedure: k7942 
o|inlining procedure: k7957 
o|inlining procedure: k7957 
o|inlining procedure: k8084 
o|contracted procedure: "(support.scm:403) g11471156" 
o|inlining procedure: k8084 
o|propagated global variable: g11531157 chicken.compiler.support#profile-lambda-list 
o|inlining procedure: k8137 
o|inlining procedure: k8137 
o|inlining procedure: k8155 
o|contracted procedure: "(support.scm:421) chicken.compiler.support#filter-map" 
o|inlining procedure: k6049 
o|inlining procedure: k6049 
o|inlining procedure: k6038 
o|inlining procedure: k6038 
o|inlining procedure: k8155 
o|inlining procedure: k8173 
o|inlining procedure: k8185 
o|inlining procedure: k8185 
o|inlining procedure: k8173 
o|inlining procedure: k8219 
o|inlining procedure: k8219 
o|inlining procedure: k8271 
o|inlining procedure: k8271 
o|inlining procedure: k8296 
o|inlining procedure: k8296 
o|propagated global variable: out12301233 ##sys#standard-output 
o|substituted constant variable: a8338 
o|substituted constant variable: a8339 
o|inlining procedure: k8334 
o|inlining procedure: k8365 
o|inlining procedure: k8365 
o|propagated global variable: out12301233 ##sys#standard-output 
o|inlining procedure: k8334 
o|contracted procedure: "(support.scm:482) g13261327" 
o|contracted procedure: "(support.scm:483) g13331334" 
o|inlining procedure: k8508 
o|inlining procedure: k8508 
o|inlining procedure: k8528 
o|inlining procedure: k8528 
o|inlining procedure: k8544 
o|contracted procedure: "(support.scm:493) g13591360" 
o|inlining procedure: k8577 
o|inlining procedure: k8577 
o|inlining procedure: k8544 
o|inlining procedure: k8621 
o|inlining procedure: k8621 
o|inlining procedure: k8640 
o|inlining procedure: k8640 
o|inlining procedure: k8651 
o|contracted procedure: "(support.scm:509) g13961397" 
o|inlining procedure: k8721 
o|inlining procedure: k8721 
o|contracted procedure: "(support.scm:510) chicken.compiler.support#unzip1" 
o|inlining procedure: k6111 
o|contracted procedure: "(mini-srfi-1.scm:143) g442451" 
o|inlining procedure: k6111 
o|inlining procedure: k8651 
o|contracted procedure: "(support.scm:514) g14311432" 
o|inlining procedure: k8789 
o|contracted procedure: "(support.scm:516) g14361437" 
o|inlining procedure: "(support.scm:518) chicken.compiler.support#fourth" 
o|inlining procedure: "(support.scm:517) chicken.compiler.support#third" 
o|inlining procedure: "(support.scm:517) chicken.compiler.support#second" 
o|inlining procedure: k8789 
o|inlining procedure: k8847 
o|contracted procedure: "(support.scm:523) g14491450" 
o|contracted procedure: "(support.scm:527) g14541455" 
o|inlining procedure: k8847 
o|contracted procedure: "(support.scm:529) g14591460" 
o|inlining procedure: k8973 
o|contracted procedure: "(support.scm:538) g14661467" 
o|inlining procedure: k9011 
o|inlining procedure: k9011 
o|inlining procedure: k9043 
o|inlining procedure: k9043 
o|inlining procedure: k8973 
o|contracted procedure: "(support.scm:543) g15041505" 
o|inlining procedure: k9098 
o|inlining procedure: k9098 
o|inlining procedure: k9133 
o|contracted procedure: "(support.scm:545) g15351536" 
o|inlining procedure: k9133 
o|contracted procedure: "(support.scm:547) g15401541" 
o|inlining procedure: k9173 
o|contracted procedure: "(support.scm:549) g15481549" 
o|inlining procedure: k9209 
o|inlining procedure: k9209 
o|inlining procedure: k9173 
o|contracted procedure: "(support.scm:554) g15801581" 
o|contracted procedure: "(support.scm:556) chicken.compiler.support#fifth" 
o|inlining procedure: "(support.scm:556) chicken.compiler.support#fourth" 
o|inlining procedure: "(support.scm:556) chicken.compiler.support#third" 
o|inlining procedure: "(support.scm:553) chicken.compiler.support#second" 
o|inlining procedure: k9305 
o|contracted procedure: "(support.scm:560) g15971598" 
o|inlining procedure: k9346 
o|inlining procedure: k9346 
o|inlining procedure: "(support.scm:560) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:560) chicken.compiler.support#first" 
o|inlining procedure: k9305 
o|contracted procedure: "(support.scm:562) g16281629" 
o|inlining procedure: k9409 
o|inlining procedure: k9409 
o|contracted procedure: "(support.scm:565) g16611662" 
o|inlining procedure: k9477 
o|inlining procedure: k9477 
o|inlining procedure: k9513 
o|inlining procedure: k9523 
o|inlining procedure: k9523 
o|inlining procedure: k9513 
o|contracted procedure: "(support.scm:567) g16701671" 
o|substituted constant variable: a9543 
o|inlining procedure: k9547 
o|inlining procedure: k9547 
o|inlining procedure: k9559 
o|inlining procedure: k9559 
o|substituted constant variable: a9566 
o|substituted constant variable: a9568 
o|substituted constant variable: a9570 
o|substituted constant variable: a9572 
o|substituted constant variable: a9574 
o|substituted constant variable: a9576 
o|substituted constant variable: a9581 
o|substituted constant variable: a9583 
o|substituted constant variable: a9585 
o|substituted constant variable: a9587 
o|inlining procedure: k9591 
o|inlining procedure: k9591 
o|substituted constant variable: a9598 
o|substituted constant variable: a9600 
o|substituted constant variable: a9602 
o|substituted constant variable: a9604 
o|substituted constant variable: a9606 
o|substituted constant variable: a9608 
o|substituted constant variable: a9613 
o|substituted constant variable: a9615 
o|substituted constant variable: a9617 
o|substituted constant variable: a9619 
o|substituted constant variable: a9624 
o|substituted constant variable: a9626 
o|contracted procedure: "(support.scm:577) g17051706" 
o|inlining procedure: k9649 
o|inlining procedure: k9649 
o|contracted procedure: "(support.scm:489) g13471348" 
o|inlining procedure: k9691 
o|inlining procedure: k9691 
o|inlining procedure: k9731 
o|inlining procedure: k9756 
o|inlining procedure: k9756 
o|inlining procedure: k9731 
o|inlining procedure: k9813 
o|inlining procedure: k9813 
o|inlining procedure: k9844 
o|inlining procedure: k9844 
o|inlining procedure: k9862 
o|inlining procedure: k9862 
o|inlining procedure: k9879 
o|inlining procedure: k9879 
o|inlining procedure: k9891 
o|inlining procedure: k9933 
o|inlining procedure: k9933 
o|inlining procedure: k9981 
o|inlining procedure: k9981 
o|inlining procedure: k9891 
o|inlining procedure: "(support.scm:605) chicken.compiler.support#third" 
o|inlining procedure: "(support.scm:602) chicken.compiler.support#second" 
o|inlining procedure: k10040 
o|inlining procedure: "(support.scm:608) chicken.compiler.support#first" 
o|inlining procedure: "(support.scm:608) chicken.compiler.support#first" 
o|inlining procedure: k10040 
o|inlining procedure: "(support.scm:610) chicken.compiler.support#first" 
o|inlining procedure: k10074 
o|inlining procedure: k10106 
o|inlining procedure: k10106 
o|inlining procedure: "(support.scm:613) chicken.compiler.support#first" 
o|inlining procedure: k10074 
o|inlining procedure: k10182 
o|inlining procedure: k10182 
o|inlining procedure: k10213 
o|consed rest parameter at call site: "(support.scm:623) chicken.compiler.support#cons*" 2 
o|inlining procedure: k10239 
o|inlining procedure: k10239 
o|inlining procedure: k10213 
o|inlining procedure: k10279 
o|inlining procedure: k10295 
o|inlining procedure: k10295 
o|inlining procedure: k10279 
o|consed rest parameter at call site: "(support.scm:631) chicken.compiler.support#cons*" 2 
o|inlining procedure: k10365 
o|inlining procedure: k10365 
o|inlining procedure: k10400 
o|consed rest parameter at call site: "(support.scm:633) chicken.compiler.support#cons*" 2 
o|inlining procedure: k10422 
o|inlining procedure: k10422 
o|inlining procedure: k10400 
o|inlining procedure: k10473 
o|inlining procedure: k10473 
o|substituted constant variable: a10505 
o|inlining procedure: k10509 
o|inlining procedure: k10509 
o|substituted constant variable: a10522 
o|substituted constant variable: a10524 
o|substituted constant variable: a10526 
o|substituted constant variable: a10528 
o|substituted constant variable: a10530 
o|substituted constant variable: a10532 
o|substituted constant variable: a10534 
o|substituted constant variable: a10536 
o|substituted constant variable: a10538 
o|substituted constant variable: a10540 
o|substituted constant variable: a10542 
o|substituted constant variable: a10544 
o|substituted constant variable: a10546 
o|substituted constant variable: a10548 
o|substituted constant variable: a10550 
o|substituted constant variable: a10552 
o|inlining procedure: k10556 
o|inlining procedure: k10556 
o|substituted constant variable: a10563 
o|substituted constant variable: a10565 
o|substituted constant variable: a10567 
o|contracted procedure: "(support.scm:587) g17521753" 
o|contracted procedure: "(support.scm:586) g17491750" 
o|contracted procedure: "(support.scm:585) g17461747" 
o|inlining procedure: k10577 
o|inlining procedure: k10577 
o|contracted procedure: "(support.scm:640) g20612062" 
o|inlining procedure: "(support.scm:642) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:642) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:650) chicken.compiler.support#split-at" 
o|inlining procedure: k5542 
o|inlining procedure: k5542 
o|inlining procedure: k10677 
o|inlining procedure: k10692 
o|contracted procedure: "(support.scm:679) g21622163" 
o|inlining procedure: k10721 
o|inlining procedure: k10721 
o|contracted procedure: "(support.scm:683) g21672168" 
o|substituted constant variable: a10743 
o|inlining procedure: k10692 
o|contracted procedure: "(support.scm:688) g21722173" 
o|inlining procedure: k10677 
o|contracted procedure: "(support.scm:692) g21782179" 
o|inlining procedure: k10846 
o|inlining procedure: k10846 
o|contracted procedure: "(support.scm:656) chicken.compiler.support#replace-rest-ops-in-known-call!" 
o|inlining procedure: k11488 
o|inlining procedure: "(support.scm:755) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:754) chicken.compiler.support#first" 
o|inlining procedure: k11488 
o|inlining procedure: k11529 
o|contracted procedure: "(support.scm:763) g24442445" 
o|inlining procedure: "(support.scm:759) chicken.compiler.support#second" 
o|inlining procedure: k11529 
o|inlining procedure: "(support.scm:758) chicken.compiler.support#first" 
o|inlining procedure: k11594 
o|inlining procedure: k11624 
o|inlining procedure: k11624 
o|contracted procedure: "(support.scm:776) g24562457" 
o|inlining procedure: "(support.scm:771) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:769) chicken.compiler.support#first" 
o|inlining procedure: k11594 
o|inlining procedure: k11666 
o|inlining procedure: k11666 
o|substituted constant variable: a11687 
o|substituted constant variable: a11689 
o|substituted constant variable: a11691 
o|contracted procedure: "(support.scm:751) g24332434" 
o|contracted procedure: "(support.scm:750) g24302431" 
o|contracted procedure: "(support.scm:749) g24272428" 
o|inlining procedure: k10873 
o|contracted procedure: "(support.scm:655) g21162125" 
o|inlining procedure: k10873 
o|contracted procedure: "(support.scm:653) chicken.compiler.support#copy-node-tree-and-rename" 
o|inlining procedure: k10998 
o|contracted procedure: "(support.scm:706) g22482249" 
o|inlining procedure: k10998 
o|inlining procedure: "(support.scm:711) rename2224" 
o|inlining procedure: "(support.scm:708) chicken.compiler.support#first" 
o|inlining procedure: k11037 
o|contracted procedure: "(support.scm:713) g22552256" 
o|inlining procedure: "(support.scm:715) chicken.compiler.support#first" 
o|inlining procedure: "(support.scm:714) rename2224" 
o|inlining procedure: "(support.scm:714) chicken.compiler.support#first" 
o|inlining procedure: k11037 
o|contracted procedure: "(support.scm:722) g22642265" 
o|inlining procedure: "(support.scm:724) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:718) chicken.compiler.support#first" 
o|inlining procedure: "(support.scm:717) chicken.compiler.support#first" 
o|inlining procedure: k11124 
o|contracted procedure: "(support.scm:735) g23352336" 
o|inlining procedure: k11190 
o|inlining procedure: k11190 
o|inlining procedure: "(support.scm:739) chicken.compiler.support#fourth" 
o|inlining procedure: k11238 
o|inlining procedure: "(support.scm:738) rename2224" 
o|inlining procedure: k11238 
o|inlining procedure: "(support.scm:737) chicken.compiler.support#second" 
o|inlining procedure: k11257 
o|inlining procedure: k11257 
o|inlining procedure: k11305 
o|inlining procedure: k11305 
o|inlining procedure: "(support.scm:727) chicken.compiler.support#third" 
o|inlining procedure: k11124 
o|contracted procedure: "(support.scm:741) g23752376" 
o|inlining procedure: k11366 
o|inlining procedure: k11366 
o|substituted constant variable: a11398 
o|substituted constant variable: a11400 
o|substituted constant variable: a11402 
o|substituted constant variable: a11404 
o|substituted constant variable: a11406 
o|contracted procedure: "(support.scm:703) g22392240" 
o|contracted procedure: "(support.scm:702) g22362237" 
o|contracted procedure: "(support.scm:701) g22332234" 
o|inlining procedure: k11413 
o|inlining procedure: k11413 
o|inlining procedure: k10919 
o|inlining procedure: k10919 
o|inlining procedure: k11698 
o|contracted procedure: "(support.scm:785) g24912492" 
o|inlining procedure: "(support.scm:787) chicken.compiler.support#second" 
o|inlining procedure: k11698 
o|inlining procedure: k11745 
o|inlining procedure: k11745 
o|contracted procedure: "(support.scm:794) g24992500" 
o|inlining procedure: "(support.scm:789) chicken.compiler.support#second" 
o|inlining procedure: k11771 
o|contracted procedure: "(support.scm:796) g25052506" 
o|contracted procedure: "(support.scm:799) g25102511" 
o|inlining procedure: "(support.scm:798) chicken.compiler.support#second" 
o|inlining procedure: k11771 
o|contracted procedure: "(support.scm:801) g25152516" 
o|inlining procedure: "(support.scm:803) chicken.compiler.support#second" 
o|substituted constant variable: a11852 
o|substituted constant variable: a11854 
o|substituted constant variable: a11856 
o|substituted constant variable: a11858 
o|inlining procedure: k11868 
o|inlining procedure: k11868 
o|contracted procedure: "(support.scm:814) g25272528" 
o|contracted procedure: "(support.scm:816) g25382539" 
o|contracted procedure: "(support.scm:815) g25352536" 
o|contracted procedure: "(support.scm:814) g25322533" 
o|contracted procedure: "(support.scm:821) g25502551" 
o|contracted procedure: "(support.scm:820) g25472548" 
o|contracted procedure: "(support.scm:819) g25442545" 
o|inlining procedure: k12143 
o|contracted procedure: "(support.scm:842) g26552656" 
o|inlining procedure: k12143 
o|contracted procedure: "(support.scm:844) g26582659" 
o|substituted constant variable: a12178 
o|contracted procedure: "(support.scm:840) g26522653" 
o|inlining procedure: k12185 
o|inlining procedure: k12208 
o|contracted procedure: "(support.scm:880) g27372744" 
o|inlining procedure: k12208 
o|contracted procedure: "(support.scm:880) chicken.compiler.support#sort-symbols" 
o|inlining procedure: k12185 
o|inlining procedure: k12260 
o|contracted procedure: "(support.scm:872) g27152722" 
o|inlining procedure: k12260 
o|inlining procedure: k12287 
o|inlining procedure: k12302 
o|inlining procedure: k12320 
o|inlining procedure: k12337 
o|inlining procedure: k12357 
o|contracted procedure: "(support.scm:866) chicken.compiler.support#node->sexpr" 
o|inlining procedure: k12021 
o|inlining procedure: k12021 
o|contracted procedure: "(support.scm:828) g25882589" 
o|contracted procedure: "(support.scm:827) g25662567" 
o|contracted procedure: "(support.scm:826) g25632564" 
o|inlining procedure: k12357 
o|inlining procedure: k12387 
o|inlining procedure: k12387 
o|inlining procedure: "(support.scm:862) chicken.compiler.support#fourth" 
o|substituted constant variable: a12401 
o|substituted constant variable: a12403 
o|contracted procedure: "(support.scm:858) g27052706" 
o|inlining procedure: k12337 
o|contracted procedure: "(support.scm:856) g26942695" 
o|inlining procedure: k12320 
o|inlining procedure: k12302 
o|contracted procedure: "(support.scm:851) g26802681" 
o|contracted procedure: "(support.scm:851) g26832684" 
o|inlining procedure: k12287 
o|inlining procedure: k12459 
o|inlining procedure: k12459 
o|contracted procedure: "(support.scm:889) g27662767" 
o|contracted procedure: "(support.scm:892) chicken.compiler.support#sexpr->node" 
o|contracted procedure: "(support.scm:832) g26032604" 
o|inlining procedure: k12092 
o|inlining procedure: k12092 
o|inlining procedure: k12506 
o|inlining procedure: k12506 
o|inlining procedure: k12537 
o|inlining procedure: k12537 
o|inlining procedure: k12552 
o|inlining procedure: k12552 
o|inlining procedure: k12584 
o|inlining procedure: k12584 
o|inlining procedure: k12599 
o|inlining procedure: k12624 
o|inlining procedure: k12624 
o|inlining procedure: k12642 
o|inlining procedure: k12642 
o|contracted procedure: "(support.scm:919) g28272828" 
o|inlining procedure: k12599 
o|inlining procedure: "(support.scm:918) chicken.compiler.support#second" 
o|contracted procedure: "(support.scm:918) g28152816" 
o|inlining procedure: "(support.scm:917) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:917) g28112812" 
o|inlining procedure: k12706 
o|contracted procedure: "(support.scm:930) g28352836" 
o|contracted procedure: "(support.scm:930) g28322833" 
o|inlining procedure: k12706 
o|inlining procedure: k12755 
o|inlining procedure: k12755 
o|contracted procedure: "(support.scm:943) chicken.compiler.support#find" 
o|inlining procedure: k6506 
o|inlining procedure: k6506 
o|propagated global variable: lst585 chicken.compiler.support#foreign-callback-stubs 
o|inlining procedure: "(support.scm:942) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:942) g28702871" 
o|inlining procedure: k12795 
o|inlining procedure: k12795 
o|substituted constant variable: a12811 
o|substituted constant variable: a12813 
o|substituted constant variable: a12815 
o|inlining procedure: k12819 
o|inlining procedure: k12819 
o|substituted constant variable: a12832 
o|substituted constant variable: a12834 
o|substituted constant variable: a12836 
o|substituted constant variable: a12838 
o|contracted procedure: "(support.scm:939) g28572858" 
o|contracted procedure: "(support.scm:938) g28482849" 
o|inlining procedure: k12856 
o|inlining procedure: k12879 
o|inlining procedure: k12902 
o|inlining procedure: k12902 
o|inlining procedure: "(support.scm:961) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:961) g29102911" 
o|contracted procedure: "(support.scm:960) g29062907" 
o|inlining procedure: "(support.scm:959) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:958) g29012902" 
o|inlining procedure: k12879 
o|contracted procedure: "(support.scm:964) g29132914" 
o|substituted constant variable: a12956 
o|substituted constant variable: a12958 
o|contracted procedure: "(support.scm:956) g28972898" 
o|inlining procedure: "(support.scm:954) chicken.compiler.support#second" 
o|inlining procedure: k12856 
o|inlining procedure: "(support.scm:952) chicken.compiler.support#first" 
o|inlining procedure: "(support.scm:951) chicken.compiler.support#third" 
o|contracted procedure: "(support.scm:950) g28812882" 
o|inlining procedure: k12974 
o|inlining procedure: k12974 
o|inlining procedure: k12989 
o|inlining procedure: k12989 
o|inlining procedure: k13015 
o|inlining procedure: k13015 
o|inlining procedure: k13030 
o|inlining procedure: k13030 
o|inlining procedure: k13052 
o|inlining procedure: k13072 
o|inlining procedure: k13072 
o|inlining procedure: k13052 
o|inlining procedure: k13095 
o|contracted procedure: "(support.scm:1004) chicken.compiler.support#unhide-variable" 
o|inlining procedure: k17931 
o|inlining procedure: k17931 
o|inlining procedure: k13095 
o|contracted procedure: "(support.scm:1003) g29452946" 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#foreign-callback-stub 
o|contracted procedure: "(support.scm:1027) g29872988" 
o|inlining procedure: k13209 
o|inlining procedure: k13209 
o|inlining procedure: k13246 
o|inlining procedure: k13246 
o|inlining procedure: k13294 
o|inlining procedure: k13294 
o|inlining procedure: k13318 
o|inlining procedure: k13318 
o|inlining procedure: k13324 
o|inlining procedure: k13324 
o|inlining procedure: k13377 
o|inlining procedure: k13377 
o|inlining procedure: k13431 
o|inlining procedure: k13431 
o|inlining procedure: k13489 
o|substituted constant variable: tmap3027 
o|substituted constant variable: tmap3027 
o|inlining procedure: k13489 
o|substituted constant variable: a13531 
o|substituted constant variable: a13532 
o|inlining procedure: k13545 
o|inlining procedure: k13545 
o|substituted constant variable: a13574 
o|substituted constant variable: a13575 
o|inlining procedure: k13558 
o|inlining procedure: k13558 
o|inlining procedure: k13637 
o|inlining procedure: k13637 
o|inlining procedure: k13691 
o|inlining procedure: k13691 
o|inlining procedure: k13719 
o|inlining procedure: k13719 
o|inlining procedure: k13743 
o|inlining procedure: k13743 
o|inlining procedure: k13765 
o|inlining procedure: k13765 
o|inlining procedure: k13839 
o|inlining procedure: k13839 
o|inlining procedure: k13880 
o|inlining procedure: k13880 
o|inlining procedure: k13886 
o|inlining procedure: k13886 
o|inlining procedure: k13912 
o|inlining procedure: k13912 
o|substituted constant variable: a13944 
o|substituted constant variable: a13946 
o|substituted constant variable: a13948 
o|substituted constant variable: a13950 
o|substituted constant variable: a13952 
o|substituted constant variable: a13954 
o|substituted constant variable: a13956 
o|substituted constant variable: a13961 
o|substituted constant variable: a13963 
o|inlining procedure: k13967 
o|inlining procedure: k13967 
o|substituted constant variable: a13980 
o|substituted constant variable: a13982 
o|substituted constant variable: a13984 
o|substituted constant variable: a13986 
o|substituted constant variable: a13994 
o|inlining procedure: k13998 
o|inlining procedure: k13998 
o|substituted constant variable: a14005 
o|substituted constant variable: a14007 
o|substituted constant variable: a14009 
o|inlining procedure: k14013 
o|inlining procedure: k14013 
o|substituted constant variable: a14026 
o|substituted constant variable: a14028 
o|substituted constant variable: a14030 
o|substituted constant variable: a14032 
o|substituted constant variable: a14034 
o|inlining procedure: k14038 
o|inlining procedure: k14038 
o|substituted constant variable: a14045 
o|substituted constant variable: a14047 
o|substituted constant variable: a14049 
o|inlining procedure: k14053 
o|inlining procedure: k14053 
o|inlining procedure: k14065 
o|inlining procedure: k14065 
o|substituted constant variable: a14078 
o|substituted constant variable: a14080 
o|substituted constant variable: a14082 
o|substituted constant variable: a14084 
o|substituted constant variable: a14086 
o|substituted constant variable: a14088 
o|inlining procedure: k14092 
o|inlining procedure: k14092 
o|inlining procedure: k14104 
o|inlining procedure: k14104 
o|substituted constant variable: a14117 
o|substituted constant variable: a14119 
o|substituted constant variable: a14121 
o|substituted constant variable: a14123 
o|substituted constant variable: a14125 
o|substituted constant variable: a14127 
o|inlining procedure: k14131 
o|inlining procedure: k14131 
o|inlining procedure: k14143 
o|inlining procedure: k14143 
o|inlining procedure: k14155 
o|inlining procedure: k14155 
o|inlining procedure: k14167 
o|inlining procedure: k14167 
o|substituted constant variable: a14180 
o|substituted constant variable: a14182 
o|substituted constant variable: a14184 
o|substituted constant variable: a14186 
o|substituted constant variable: a14188 
o|substituted constant variable: a14190 
o|substituted constant variable: a14192 
o|substituted constant variable: a14194 
o|substituted constant variable: a14196 
o|substituted constant variable: a14198 
o|inlining procedure: k14202 
o|inlining procedure: k14202 
o|inlining procedure: k14214 
o|inlining procedure: k14214 
o|inlining procedure: k14226 
o|inlining procedure: k14226 
o|inlining procedure: k14238 
o|inlining procedure: k14238 
o|substituted constant variable: a14251 
o|substituted constant variable: a14253 
o|substituted constant variable: a14255 
o|substituted constant variable: a14257 
o|substituted constant variable: a14259 
o|substituted constant variable: a14261 
o|substituted constant variable: a14263 
o|substituted constant variable: a14265 
o|substituted constant variable: a14267 
o|substituted constant variable: a14269 
o|substituted constant variable: a14271 
o|substituted constant variable: a14273 
o|substituted constant variable: a14278 
o|substituted constant variable: a14280 
o|substituted constant variable: a14285 
o|substituted constant variable: a14287 
o|inlining procedure: k14291 
o|inlining procedure: k14291 
o|substituted constant variable: a14298 
o|substituted constant variable: a14300 
o|substituted constant variable: a14302 
o|inlining procedure: k14306 
o|inlining procedure: k14306 
o|inlining procedure: k14318 
o|inlining procedure: k14318 
o|substituted constant variable: a14331 
o|substituted constant variable: a14333 
o|substituted constant variable: a14335 
o|substituted constant variable: a14337 
o|substituted constant variable: a14339 
o|substituted constant variable: a14341 
o|substituted constant variable: a14346 
o|substituted constant variable: a14348 
o|inlining procedure: k14361 
o|inlining procedure: k14373 
o|inlining procedure: k14373 
o|inlining procedure: k14361 
o|inlining procedure: k14382 
o|inlining procedure: k14394 
o|inlining procedure: k14394 
o|inlining procedure: k14382 
o|inlining procedure: k14406 
o|inlining procedure: k14406 
o|inlining procedure: k14421 
o|inlining procedure: k14421 
o|inlining procedure: k14439 
o|inlining procedure: k14439 
o|removed unused formal parameters: (t3386) 
o|inlining procedure: k14481 
o|inlining procedure: k14481 
o|inlining procedure: k14502 
o|inlining procedure: k14502 
o|inlining procedure: k14526 
o|inlining procedure: k14526 
o|inlining procedure: k14556 
o|inlining procedure: k14575 
o|inlining procedure: k14575 
o|removed unused parameter to known procedure: t3386 "(support.scm:1272) err3385" 
o|substituted constant variable: a14601 
o|substituted constant variable: a14603 
o|inlining procedure: k14607 
o|inlining procedure: k14607 
o|inlining procedure: k14619 
o|inlining procedure: k14619 
o|inlining procedure: k14631 
o|inlining procedure: k14631 
o|inlining procedure: k14643 
o|inlining procedure: k14643 
o|substituted constant variable: a14650 
o|substituted constant variable: a14652 
o|substituted constant variable: a14654 
o|substituted constant variable: a14656 
o|substituted constant variable: a14658 
o|substituted constant variable: a14660 
o|substituted constant variable: a14662 
o|substituted constant variable: a14664 
o|substituted constant variable: a14666 
o|inlining procedure: k14556 
o|removed unused parameter to known procedure: t3386 "(support.scm:1273) err3385" 
o|inlining procedure: k14679 
o|inlining procedure: k14679 
o|substituted constant variable: a14692 
o|substituted constant variable: a14694 
o|substituted constant variable: a14696 
o|substituted constant variable: a14698 
o|inlining procedure: k14702 
o|inlining procedure: k14702 
o|substituted constant variable: a14709 
o|substituted constant variable: a14711 
o|substituted constant variable: a14713 
o|inlining procedure: k14717 
o|inlining procedure: k14717 
o|inlining procedure: k14729 
o|inlining procedure: k14729 
o|substituted constant variable: a14742 
o|substituted constant variable: a14744 
o|substituted constant variable: a14746 
o|substituted constant variable: a14748 
o|substituted constant variable: a14750 
o|substituted constant variable: a14752 
o|inlining procedure: k14756 
o|inlining procedure: k14756 
o|inlining procedure: k14768 
o|inlining procedure: k14768 
o|inlining procedure: k14780 
o|inlining procedure: k14780 
o|inlining procedure: k14792 
o|inlining procedure: k14792 
o|inlining procedure: k14804 
o|inlining procedure: k14804 
o|substituted constant variable: a14817 
o|substituted constant variable: a14819 
o|substituted constant variable: a14821 
o|substituted constant variable: a14823 
o|substituted constant variable: a14825 
o|substituted constant variable: a14827 
o|substituted constant variable: a14829 
o|substituted constant variable: a14831 
o|substituted constant variable: a14833 
o|substituted constant variable: a14835 
o|substituted constant variable: a14837 
o|substituted constant variable: a14839 
o|inlining procedure: k14843 
o|inlining procedure: k14843 
o|inlining procedure: k14855 
o|inlining procedure: k14855 
o|inlining procedure: k14867 
o|inlining procedure: k14867 
o|inlining procedure: k14879 
o|inlining procedure: k14879 
o|inlining procedure: k14891 
o|inlining procedure: k14891 
o|inlining procedure: k14903 
o|inlining procedure: k14903 
o|substituted constant variable: a14910 
o|substituted constant variable: a14912 
o|substituted constant variable: a14914 
o|substituted constant variable: a14916 
o|substituted constant variable: a14918 
o|substituted constant variable: a14920 
o|substituted constant variable: a14922 
o|substituted constant variable: a14924 
o|substituted constant variable: a14926 
o|substituted constant variable: a14928 
o|substituted constant variable: a14930 
o|substituted constant variable: a14932 
o|substituted constant variable: a14934 
o|inlining procedure: k14956 
o|inlining procedure: k14956 
o|inlining procedure: k14983 
o|inlining procedure: k14983 
o|inlining procedure: k15005 
o|inlining procedure: k15005 
o|inlining procedure: "(support.scm:1300) err3537" 
o|substituted constant variable: a15034 
o|inlining procedure: k15038 
o|inlining procedure: k15038 
o|inlining procedure: k15050 
o|inlining procedure: k15050 
o|inlining procedure: k15062 
o|inlining procedure: k15062 
o|inlining procedure: k15074 
o|inlining procedure: k15074 
o|substituted constant variable: a15081 
o|substituted constant variable: a15083 
o|substituted constant variable: a15085 
o|substituted constant variable: a15087 
o|substituted constant variable: a15089 
o|substituted constant variable: a15091 
o|substituted constant variable: a15093 
o|substituted constant variable: a15095 
o|substituted constant variable: a15097 
o|inlining procedure: "(support.scm:1301) err3537" 
o|inlining procedure: k15110 
o|inlining procedure: k15110 
o|inlining procedure: k15122 
o|inlining procedure: k15122 
o|substituted constant variable: a15129 
o|substituted constant variable: a15131 
o|substituted constant variable: a15133 
o|substituted constant variable: a15135 
o|substituted constant variable: a15137 
o|inlining procedure: k15141 
o|inlining procedure: k15141 
o|inlining procedure: k15153 
o|inlining procedure: k15153 
o|inlining procedure: k15165 
o|inlining procedure: k15165 
o|inlining procedure: k15177 
o|inlining procedure: k15177 
o|inlining procedure: k15189 
o|inlining procedure: k15189 
o|inlining procedure: k15201 
o|inlining procedure: k15201 
o|inlining procedure: k15213 
o|inlining procedure: k15213 
o|inlining procedure: k15225 
o|inlining procedure: k15225 
o|inlining procedure: k15237 
o|inlining procedure: k15237 
o|inlining procedure: k15249 
o|inlining procedure: k15249 
o|inlining procedure: k15261 
o|inlining procedure: k15261 
o|inlining procedure: k15273 
o|inlining procedure: k15273 
o|inlining procedure: k15285 
o|inlining procedure: k15285 
o|inlining procedure: k15297 
o|inlining procedure: k15297 
o|inlining procedure: k15309 
o|inlining procedure: k15309 
o|substituted constant variable: a15322 
o|substituted constant variable: a15324 
o|substituted constant variable: a15326 
o|substituted constant variable: a15328 
o|substituted constant variable: a15330 
o|substituted constant variable: a15332 
o|substituted constant variable: a15334 
o|substituted constant variable: a15336 
o|substituted constant variable: a15338 
o|substituted constant variable: a15340 
o|substituted constant variable: a15342 
o|substituted constant variable: a15344 
o|substituted constant variable: a15346 
o|substituted constant variable: a15348 
o|substituted constant variable: a15350 
o|substituted constant variable: a15352 
o|substituted constant variable: a15354 
o|substituted constant variable: a15356 
o|substituted constant variable: a15358 
o|substituted constant variable: a15360 
o|substituted constant variable: a15362 
o|substituted constant variable: a15364 
o|substituted constant variable: a15366 
o|substituted constant variable: a15368 
o|substituted constant variable: a15370 
o|substituted constant variable: a15372 
o|substituted constant variable: a15374 
o|substituted constant variable: a15376 
o|substituted constant variable: a15378 
o|substituted constant variable: a15380 
o|substituted constant variable: a15382 
o|substituted constant variable: a15384 
o|inlining procedure: k15397 
o|inlining procedure: k15397 
o|inlining procedure: k15426 
o|inlining procedure: k15426 
o|inlining procedure: k15458 
o|inlining procedure: k15458 
o|inlining procedure: k15488 
o|inlining procedure: k15488 
o|inlining procedure: k15507 
o|inlining procedure: k15507 
o|inlining procedure: k15529 
o|inlining procedure: k15529 
o|substituted constant variable: a15594 
o|substituted constant variable: a15599 
o|substituted constant variable: a15601 
o|substituted constant variable: a15602 
o|inlining procedure: k15610 
o|substituted constant variable: a15620 
o|inlining procedure: k15610 
o|substituted constant variable: a15621 
o|substituted constant variable: a15631 
o|substituted constant variable: a15633 
o|substituted constant variable: a15635 
o|substituted constant variable: a15640 
o|substituted constant variable: a15642 
o|substituted constant variable: a15647 
o|substituted constant variable: a15649 
o|substituted constant variable: a15651 
o|substituted constant variable: a15656 
o|substituted constant variable: a15658 
o|inlining procedure: k15662 
o|inlining procedure: k15662 
o|inlining procedure: k15680 
o|inlining procedure: k15680 
o|inlining procedure: k15698 
o|substituted constant variable: a15714 
o|inlining procedure: k15698 
o|inlining procedure: k15724 
o|substituted constant variable: a15737 
o|inlining procedure: k15724 
o|inlining procedure: k15744 
o|substituted constant variable: a15757 
o|inlining procedure: k15744 
o|inlining procedure: k15764 
o|substituted constant variable: a15790 
o|inlining procedure: k15764 
o|inlining procedure: k15797 
o|inlining procedure: k15797 
o|inlining procedure: k15809 
o|inlining procedure: k15809 
o|inlining procedure: k15821 
o|inlining procedure: k15821 
o|inlining procedure: k15833 
o|inlining procedure: k15833 
o|inlining procedure: k15845 
o|inlining procedure: k15845 
o|inlining procedure: k15860 
o|inlining procedure: k15860 
o|inlining procedure: k15878 
o|inlining procedure: k15878 
o|inlining procedure: k15884 
o|inlining procedure: k15884 
o|inlining procedure: k15902 
o|inlining procedure: k15902 
o|inlining procedure: k15917 
o|inlining procedure: k15933 
o|inlining procedure: k15933 
o|inlining procedure: k15939 
o|inlining procedure: k15939 
o|inlining procedure: k15958 
o|inlining procedure: k15958 
o|substituted constant variable: a15977 
o|substituted constant variable: a15979 
o|substituted constant variable: a15981 
o|substituted constant variable: a15983 
o|inlining procedure: k15987 
o|inlining procedure: k15987 
o|substituted constant variable: a16000 
o|substituted constant variable: a16002 
o|substituted constant variable: a16004 
o|substituted constant variable: a16006 
o|inlining procedure: k15917 
o|substituted constant variable: a16008 
o|inlining procedure: k16012 
o|inlining procedure: k16012 
o|substituted constant variable: a16019 
o|substituted constant variable: a16021 
o|substituted constant variable: a16023 
o|substituted constant variable: a16028 
o|substituted constant variable: a16030 
o|inlining procedure: k16034 
o|inlining procedure: k16034 
o|substituted constant variable: a16047 
o|substituted constant variable: a16049 
o|substituted constant variable: a16051 
o|substituted constant variable: a16053 
o|substituted constant variable: a16055 
o|substituted constant variable: a16057 
o|inlining procedure: k16061 
o|inlining procedure: k16061 
o|inlining procedure: k16073 
o|inlining procedure: k16073 
o|inlining procedure: k16085 
o|inlining procedure: k16085 
o|inlining procedure: k16097 
o|inlining procedure: k16097 
o|substituted constant variable: a16104 
o|substituted constant variable: a16106 
o|substituted constant variable: a16108 
o|substituted constant variable: a16110 
o|substituted constant variable: a16112 
o|substituted constant variable: a16114 
o|substituted constant variable: a16116 
o|substituted constant variable: a16118 
o|substituted constant variable: a16120 
o|substituted constant variable: a16122 
o|substituted constant variable: a16124 
o|substituted constant variable: a16126 
o|substituted constant variable: a16128 
o|substituted constant variable: a16130 
o|substituted constant variable: a16132 
o|substituted constant variable: a16134 
o|substituted constant variable: a16136 
o|substituted constant variable: a16138 
o|substituted constant variable: a16140 
o|inlining procedure: k16144 
o|inlining procedure: k16144 
o|inlining procedure: k16156 
o|inlining procedure: k16156 
o|inlining procedure: k16168 
o|inlining procedure: k16168 
o|inlining procedure: k16180 
o|inlining procedure: k16180 
o|substituted constant variable: a16193 
o|substituted constant variable: a16195 
o|substituted constant variable: a16197 
o|substituted constant variable: a16199 
o|substituted constant variable: a16201 
o|substituted constant variable: a16203 
o|substituted constant variable: a16205 
o|substituted constant variable: a16207 
o|substituted constant variable: a16209 
o|substituted constant variable: a16211 
o|substituted constant variable: a16213 
o|substituted constant variable: a16215 
o|substituted constant variable: a16217 
o|substituted constant variable: a16219 
o|substituted constant variable: a16224 
o|substituted constant variable: a16226 
o|substituted constant variable: a16231 
o|substituted constant variable: a16233 
o|inlining procedure: k16237 
o|inlining procedure: k16237 
o|inlining procedure: k16249 
o|inlining procedure: k16249 
o|inlining procedure: k16261 
o|inlining procedure: k16261 
o|substituted constant variable: a16274 
o|substituted constant variable: a16276 
o|substituted constant variable: a16278 
o|substituted constant variable: a16280 
o|substituted constant variable: a16282 
o|substituted constant variable: a16284 
o|substituted constant variable: a16286 
o|substituted constant variable: a16288 
o|substituted constant variable: a16293 
o|substituted constant variable: a16295 
o|substituted constant variable: a16297 
o|inlining procedure: k16298 
o|inlining procedure: k16298 
o|inlining procedure: k16338 
o|inlining procedure: k16362 
o|inlining procedure: k16362 
o|inlining procedure: "(support.scm:1430) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:1430) g39403941" 
o|inlining procedure: k16338 
o|inlining procedure: k16426 
o|inlining procedure: k16426 
o|inlining procedure: k16449 
o|inlining procedure: k16449 
o|substituted constant variable: a16456 
o|substituted constant variable: a16458 
o|substituted constant variable: a16460 
o|substituted constant variable: a16465 
o|substituted constant variable: a16467 
o|contracted procedure: "(support.scm:1428) g39333934" 
o|contracted procedure: "(support.scm:1427) g39243925" 
o|inlining procedure: k16498 
o|inlining procedure: k16498 
o|inlining procedure: k16516 
o|inlining procedure: k16516 
o|consed rest parameter at call site: "(support.scm:1455) chicken.compiler.support#lset-adjoin/eq?" 2 
o|consed rest parameter at call site: "(support.scm:1453) chicken.compiler.support#lset-adjoin/eq?" 2 
o|inlining procedure: "(support.scm:1451) chicken.compiler.support#first" 
o|inlining procedure: k16536 
o|consed rest parameter at call site: "(support.scm:1458) chicken.compiler.support#lset-adjoin/eq?" 2 
o|inlining procedure: "(support.scm:1457) chicken.compiler.support#first" 
o|inlining procedure: k16536 
o|inlining procedure: "(support.scm:1462) chicken.compiler.support#second" 
o|inlining procedure: "(support.scm:1461) chicken.compiler.support#first" 
o|inlining procedure: k16586 
o|inlining procedure: "(support.scm:1467) chicken.compiler.support#first" 
o|inlining procedure: "(support.scm:1465) chicken.compiler.support#third" 
o|inlining procedure: k16586 
o|substituted constant variable: a16617 
o|substituted constant variable: a16619 
o|substituted constant variable: a16621 
o|substituted constant variable: a16623 
o|inlining procedure: k16627 
o|inlining procedure: k16627 
o|inlining procedure: k16639 
o|inlining procedure: k16639 
o|substituted constant variable: a16646 
o|substituted constant variable: a16648 
o|substituted constant variable: a16650 
o|substituted constant variable: a16652 
o|substituted constant variable: a16654 
o|contracted procedure: "(support.scm:1448) g40104011" 
o|contracted procedure: "(support.scm:1447) g40014002" 
o|contracted procedure: "(support.scm:1446) g39983999" 
o|inlining procedure: k16672 
o|inlining procedure: k16672 
o|inlining procedure: k16705 
o|inlining procedure: k16705 
o|substituted constant variable: a16715 
o|substituted constant variable: a16720 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#block-variable-literal 
o|substituted constant variable: a16754 
o|substituted constant variable: a16755 
o|inlining procedure: k16777 
o|inlining procedure: k16777 
o|inlining procedure: k16819 
o|inlining procedure: k16819 
o|inlining procedure: k16831 
o|inlining procedure: k16831 
o|inlining procedure: k16861 
o|inlining procedure: k16861 
o|inlining procedure: k16884 
o|inlining procedure: k16884 
o|substituted constant variable: chicken.compiler.support#real-name-max-depth 
o|inlining procedure: k16936 
o|inlining procedure: k16936 
o|propagated global variable: out41354138 ##sys#standard-output 
o|substituted constant variable: a16952 
o|substituted constant variable: a16953 
o|propagated global variable: out41354138 ##sys#standard-output 
o|inlining procedure: k16969 
o|substituted constant variable: a16993 
o|inlining procedure: k16969 
o|inlining procedure: k17004 
o|inlining procedure: k17004 
o|inlining procedure: k17019 
o|inlining procedure: k17019 
o|inlining procedure: k17037 
o|inlining procedure: k17037 
o|inlining procedure: k17040 
o|inlining procedure: k17040 
o|inlining procedure: "(support.scm:1581) chicken.compiler.support#second" 
o|inlining procedure: k17097 
o|inlining procedure: k17118 
o|inlining procedure: k17118 
o|substituted constant variable: a17146 
o|substituted constant variable: a17159 
o|inlining procedure: k17097 
o|inlining procedure: k17230 
o|contracted procedure: "(support.scm:1593) g42084217" 
o|inlining procedure: k17230 
o|inlining procedure: k17264 
o|contracted procedure: "(support.scm:1592) g41774186" 
o|inlining procedure: "(support.scm:1592) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:1592) g41894190" 
o|inlining procedure: k17264 
o|contracted procedure: "(support.scm:1610) g42524253" 
o|inlining procedure: k17313 
o|inlining procedure: k17350 
o|inlining procedure: k17350 
o|contracted procedure: "(support.scm:1613) g42634264" 
o|inlining procedure: "(support.scm:1612) chicken.compiler.support#first" 
o|contracted procedure: "(support.scm:1612) g42594260" 
o|inlining procedure: k17313 
o|contracted procedure: "(support.scm:1611) g42554256" 
o|inlining procedure: k17411 
o|inlining procedure: k17411 
o|inlining procedure: k17424 
o|inlining procedure: k17424 
o|substituted constant variable: a17458 
o|merged explicitly consed rest parameter: args42914297 
o|consed rest parameter at call site: tmp25336 1 
o|inlining procedure: k17480 
o|inlining procedure: k17480 
o|inlining procedure: k17499 
o|inlining procedure: "(support.scm:1641) getsize4281" 
o|inlining procedure: k17499 
o|inlining procedure: "(support.scm:1643) getsize4281" 
o|substituted constant variable: a17571 
o|propagated global variable: out43274340 ##sys#standard-output 
o|substituted constant variable: a17573 
o|substituted constant variable: a17574 
o|propagated global variable: out43634366 ##sys#standard-output 
o|substituted constant variable: a17617 
o|substituted constant variable: a17618 
o|inlining procedure: k17607 
o|inlining procedure: k17634 
o|propagated global variable: out43724375 ##sys#standard-output 
o|substituted constant variable: a17641 
o|substituted constant variable: a17642 
o|inlining procedure: k17634 
o|propagated global variable: out43724375 ##sys#standard-output 
o|propagated global variable: out43634366 ##sys#standard-output 
o|inlining procedure: k17607 
o|inlining procedure: k17667 
o|inlining procedure: k17667 
o|propagated global variable: out43274340 ##sys#standard-output 
o|contracted procedure: "(support.scm:1657) g43244325" 
o|contracted procedure: "(support.scm:1656) g43214322" 
o|contracted procedure: "(support.scm:1655) g43184319" 
o|inlining procedure: k17690 
o|inlining procedure: k17715 
o|inlining procedure: k17715 
o|inlining procedure: k17690 
o|inlining procedure: k17739 
o|contracted procedure: "(support.scm:1697) chicken.compiler.support#scan-sharp-greater-string" 
o|inlining procedure: k17776 
o|inlining procedure: k17776 
o|substituted constant variable: a17789 
o|substituted constant variable: a17800 
o|inlining procedure: k17796 
o|substituted constant variable: a17822 
o|inlining procedure: k17796 
o|inlining procedure: k17739 
o|inlining procedure: k17833 
o|inlining procedure: k17848 
o|inlining procedure: k17848 
o|inlining procedure: k17833 
o|inlining procedure: k17857 
o|inlining procedure: k17857 
o|contracted procedure: "(support.scm:1740) g44344435" 
o|contracted procedure: "(support.scm:1743) g44504451" 
o|inlining procedure: k17946 
o|inlining procedure: k17946 
o|substituted constant variable: a17962 
o|substituted constant variable: a17964 
o|contracted procedure: "(support.scm:1767) g45014502" 
o|contracted procedure: "(support.scm:1769) g45124513" 
o|contracted procedure: "(support.scm:1770) g45234524" 
o|inlining procedure: k18025 
o|inlining procedure: k18068 
o|contracted procedure: "(support.scm:1778) g45464553" 
o|inlining procedure: k18068 
o|substituted constant variable: a18093 
o|substituted constant variable: a18094 
o|inlining procedure: k18025 
o|substituted constant variable: chicken.compiler.support#constant678 
o|substituted constant variable: a18161 
o|substituted constant variable: a18162 
o|contracted procedure: "(support.scm:477) g13121313" 
o|contracted procedure: "(support.scm:477) g13091310" 
o|substituted constant variable: chicken.compiler.support#node 
o|replaced variables: 4002 
o|removed binding forms: 706 
o|substituted constant variable: r548518196 
o|substituted constant variable: r578618200 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#first 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#second 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#third 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#fourth 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#filter 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#constant678 
o|propagated global variable: out715718 ##sys#standard-output 
o|propagated global variable: out730733 chicken.compiler.support#collected-debugging-output 
o|substituted constant variable: r679118230 
o|inlining procedure: k6805 
o|substituted constant variable: r679118231 
o|inlining procedure: k6814 
o|propagated global variable: out766769 chicken.compiler.support#collected-debugging-output 
o|converted assignments to bindings: (collect751) 
o|substituted constant variable: r711618244 
o|substituted constant variable: r719118248 
o|substituted constant variable: r723818250 
o|substituted constant variable: r723818251 
o|substituted constant variable: r727518257 
o|substituted constant variable: r759118283 
o|substituted constant variable: r770518299 
o|substituted constant variable: r774318300 
o|substituted constant variable: r786118310 
o|substituted constant variable: r813818321 
o|substituted constant variable: r603918326 
o|substituted constant variable: r815618327 
o|substituted constant variable: r827218335 
o|propagated global variable: out12301233 ##sys#standard-output 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#node 
o|substituted constant variable: c1328 
o|substituted constant variable: s1330 
o|substituted constant variable: c1335 
o|substituted constant variable: s1337 
o|substituted constant variable: p1362 
o|substituted constant variable: r864118355 
o|substituted constant variable: c1398 
o|substituted constant variable: c1433 
o|substituted constant variable: c1438 
o|substituted constant variable: c1451 
o|substituted constant variable: c1456 
o|substituted constant variable: p1457 
o|substituted constant variable: s1458 
o|substituted constant variable: c1461 
o|substituted constant variable: s1539 
o|substituted constant variable: c1542 
o|substituted constant variable: s1544 
o|substituted constant variable: c1550 
o|substituted constant variable: c1582 
o|substituted constant variable: c1630 
o|substituted constant variable: c1663 
o|substituted constant variable: mark1673 
o|substituted constant variable: c1707 
o|substituted constant variable: c2063 
o|substituted constant variable: p2064 
o|substituted constant variable: c2164 
o|substituted constant variable: c2169 
o|substituted constant variable: c2174 
o|substituted constant variable: c2180 
o|substituted constant variable: c2446 
o|substituted constant variable: c2458 
o|removed side-effect free assignment to unused variable: rename2224 
o|substituted constant variable: s2252 
o|substituted constant variable: c2257 
o|substituted constant variable: c2266 
o|substituted constant variable: c2337 
o|substituted constant variable: r1123918654 
o|substituted constant variable: r1123918654 
o|substituted constant variable: c2493 
o|substituted constant variable: c2501 
o|substituted constant variable: c2507 
o|substituted constant variable: c2512 
o|substituted constant variable: c2517 
o|substituted constant variable: r1235818720 
o|substituted constant variable: r1238818721 
o|substituted constant variable: mark2708 
o|substituted constant variable: r1233818728 
o|substituted constant variable: r1232118729 
o|substituted constant variable: r1230318730 
o|substituted constant variable: mark2686 
o|converted assignments to bindings: (uses-foreign-stubs?2642) 
o|substituted constant variable: mark2773 
o|substituted constant variable: r1255318742 
o|substituted constant variable: r1264318748 
o|substituted constant variable: r1260018750 
o|substituted constant variable: r1270718762 
o|substituted constant variable: r1275618763 
o|substituted constant variable: r650718765 
o|substituted constant variable: r1279618773 
o|substituted constant variable: r1290318779 
o|substituted constant variable: r1285718796 
o|substituted constant variable: r1299018810 
o|substituted constant variable: r1303118814 
o|substituted constant variable: r1307318818 
o|substituted constant variable: r1307318818 
o|substituted constant variable: mark2994 
o|substituted constant variable: r1324718829 
o|substituted constant variable: r1324718829 
o|substituted constant variable: r1437418905 
o|substituted constant variable: r1436218906 
o|substituted constant variable: r1439518909 
o|substituted constant variable: r1438318910 
o|substituted constant variable: r1448218917 
o|converted assignments to bindings: (err3385) 
o|removed side-effect free assignment to unused variable: err3537 
o|substituted constant variable: r1561119036 
o|substituted constant variable: r1566319037 
o|substituted constant variable: r1568119039 
o|substituted constant variable: r1579819049 
o|substituted constant variable: r1581019051 
o|substituted constant variable: r1582219053 
o|substituted constant variable: r1583419055 
o|substituted constant variable: r1584619057 
o|substituted constant variable: r1587919061 
o|substituted constant variable: r1587919062 
o|substituted constant variable: r1588519063 
o|substituted constant variable: r1590319065 
o|substituted constant variable: r1593419068 
o|substituted constant variable: r1593419069 
o|substituted constant variable: r1595919073 
o|substituted constant variable: r1591819076 
o|substituted constant variable: r1629919104 
o|substituted constant variable: r1649919118 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#real-name-max-depth 
o|converted assignments to bindings: (resolve4104) 
o|substituted constant variable: r1693719177 
o|propagated global variable: out41354138 ##sys#standard-output 
o|substituted constant variable: r1702019183 
o|substituted constant variable: r1704119187 
o|substituted constant variable: r1735119208 
o|converted assignments to bindings: (constant-node?4250) 
o|removed side-effect free assignment to unused variable: getsize4281 
o|substituted constant variable: r1742519218 
o|contracted procedure: "(support.scm:1647) chicken.compiler.support#list-tabulate" 
o|substituted constant variable: r639418212 
o|propagated global variable: out43274340 ##sys#standard-output 
o|propagated global variable: out43634366 ##sys#standard-output 
o|propagated global variable: out43724375 ##sys#standard-output 
o|substituted constant variable: r1771619245 
o|substituted constant variable: r1771619245 
o|substituted constant variable: r1783419258 
o|substituted constant variable: r1785819260 
o|substituted constant variable: mark4441 
o|substituted constant variable: mark4457 
o|substituted constant variable: r1794719263 
o|substituted constant variable: mark4504 
o|substituted constant variable: mark4515 
o|substituted constant variable: mark4526 
o|substituted constant variable: r1802619268 
o|simplifications: ((let . 5)) 
o|replaced variables: 639 
o|removed binding forms: 3383 
o|substituted constant variable: r67911823019296 
o|substituted constant variable: r67911823119298 
o|inlining procedure: "(support.scm:389) chicken.compiler.support#alist-cons" 
o|inlining procedure: "(support.scm:429) chicken.compiler.support#alist-cons" 
o|inlining procedure: "(support.scm:437) chicken.compiler.support#alist-cons" 
o|removed call to pure procedure with unused result: "(support.scm:655) ##sys#slot" 
o|inlining procedure: "(support.scm:720) chicken.compiler.support#alist-cons" 
o|inlining procedure: k12467 
o|inlining procedure: "(support.scm:904) chicken.compiler.support#alist-cons" 
o|inlining procedure: k12801 
o|inlining procedure: k13186 
o|inlining procedure: k13934 
o|inlining procedure: k14403 
o|inlining procedure: k14418 
o|inlining procedure: k16545 
o|inlining procedure: k16545 
o|inlining procedure: k17034 
o|inlining procedure: k17034 
o|inlining procedure: "(support.scm:1635) fits?4282" 
o|inlining procedure: "(support.scm:1639) fits?4282" 
o|inlining procedure: "(support.scm:1641) fits?4282" 
o|inlining procedure: "(support.scm:1644) fits?4282" 
o|converted assignments to bindings: (fits?4282) 
o|inlining procedure: "(support.scm:1680) chicken.compiler.support#alist-cons" 
o|inlining procedure: "(support.scm:1680) chicken.compiler.support#alist-cons" 
o|inlining procedure: k17883 
o|inlining procedure: k17903 
o|inlining procedure: k17968 
o|inlining procedure: k18120 
o|simplifications: ((let . 1)) 
o|replaced variables: 3 
o|removed binding forms: 764 
o|removed side-effect free assignment to unused variable: chicken.compiler.support#alist-cons 
o|inlining procedure: k9046 
o|contracted procedure: k10898 
o|substituted constant variable: r1246819740 
o|contracted procedure: k13100 
o|substituted constant variable: r1318719768 
o|substituted constant variable: r1440419772 
o|substituted constant variable: r1440419772 
o|substituted constant variable: r1441919775 
o|substituted constant variable: r1441919775 
o|inlining procedure: k16708 
o|inlining procedure: k16708 
o|substituted constant variable: r1703519796 
o|substituted constant variable: r1703519796 
o|substituted constant variable: r1703519799 
o|substituted constant variable: r1703519799 
o|contracted procedure: k17506 
o|contracted procedure: k17509 
o|substituted constant variable: z38219837 
o|substituted constant variable: r1788419843 
o|substituted constant variable: r1790419844 
o|substituted constant variable: r1796919845 
o|simplifications: ((let . 1)) 
o|replaced variables: 35 
o|removed binding forms: 23 
o|removed conditional forms: 4 
o|substituted constant variable: r904719902 
o|substituted constant variable: r1670920012 
o|simplifications: ((if . 1) (let . 1)) 
o|removed binding forms: 48 
o|removed conditional forms: 2 
o|replaced variables: 2 
o|removed binding forms: 2 
o|removed binding forms: 1 
o|simplifications: ((if . 107) (let . 52) (##core#call . 1518)) 
o|  call simplifications:
o|    chicken.base#bignum?
o|    scheme#char=?
o|    scheme#read-char	3
o|    ##sys#size
o|    chicken.fixnum#fx>	2
o|    scheme#write-char	6
o|    chicken.base#flonum?
o|    scheme#procedure?
o|    chicken.fixnum#fx+	3
o|    scheme#string-length	4
o|    scheme#string-ref
o|    scheme#list?	5
o|    scheme#vector-ref	6
o|    scheme#vector	2
o|    scheme#<
o|    scheme#>	2
o|    scheme#<=
o|    scheme#-
o|    ##sys#call-with-values	4
o|    scheme#cddddr
o|    scheme#list-ref	2
o|    scheme#cdddr
o|    scheme#caar	2
o|    scheme#cadddr	4
o|    scheme#caddr	10
o|    scheme#cadr	45
o|    ##sys#check-structure	12
o|    ##sys#block-ref	9
o|    ##sys#structure?	5
o|    ##sys#make-structure	42
o|    scheme#values	4
o|    scheme#assq	16
o|    scheme#length	8
o|    ##sys#setslot	36
o|    chicken.base#atom?
o|    ##sys#apply	3
o|    ##sys#cons	8
o|    scheme#equal?	3
o|    ##sys#list	139
o|    chicken.base#fixnum?	2
o|    scheme#number?	4
o|    scheme#char?	4
o|    scheme#string?	2
o|    scheme#boolean?	4
o|    scheme#vector?	2
o|    scheme#eq?	369
o|    scheme#eof-object?	7
o|    scheme#member
o|    scheme#cddr	4
o|    scheme#list	65
o|    scheme#string=?	2
o|    ##sys#foreign-fixnum-argument	2
o|    scheme#char-alphabetic?	2
o|    scheme#char-numeric?
o|    scheme#char->integer
o|    chicken.fixnum#fx>=	3
o|    chicken.fixnum#fx<	4
o|    scheme#string->list	3
o|    scheme#list->string
o|    ##sys#check-list	42
o|    ##sys#slot	191
o|    scheme#symbol?	16
o|    scheme#pair?	69
o|    scheme#apply	7
o|    scheme#memq	9
o|    scheme#cdr	23
o|    scheme#null?	47
o|    scheme#not	23
o|    chicken.fixnum#fx<=	7
o|    scheme#car	89
o|    chicken.fixnum#fx-	2
o|    scheme#cons	113
o|contracted procedure: k5487 
o|contracted procedure: k5494 
o|contracted procedure: k5504 
o|contracted procedure: k5751 
o|contracted procedure: k5760 
o|contracted procedure: k5773 
o|contracted procedure: k5788 
o|contracted procedure: k5803 
o|contracted procedure: k5818 
o|contracted procedure: k5829 
o|contracted procedure: k6164 
o|contracted procedure: k6154 
o|contracted procedure: k6213 
o|contracted procedure: k6239 
o|contracted procedure: k6219 
o|contracted procedure: k6233 
o|contracted procedure: k6664 
o|contracted procedure: k6692 
o|contracted procedure: k6728 
o|contracted procedure: k6758 
o|contracted procedure: k6768 
o|contracted procedure: k6772 
o|contracted procedure: k6854 
o|propagated global variable: out766769 chicken.compiler.support#collected-debugging-output 
o|contracted procedure: k6870 
o|contracted procedure: k6880 
o|contracted procedure: k6884 
o|contracted procedure: k6954 
o|contracted procedure: k6972 
o|contracted procedure: k6982 
o|contracted procedure: k6986 
o|contracted procedure: k7025 
o|contracted procedure: k7029 
o|contracted procedure: k7033 
o|contracted procedure: k7047 
o|contracted procedure: k7103 
o|contracted procedure: k7109 
o|contracted procedure: k7118 
o|contracted procedure: k7125 
o|contracted procedure: k7129 
o|contracted procedure: k7145 
o|contracted procedure: k7155 
o|contracted procedure: k7176 
o|contracted procedure: k7193 
o|contracted procedure: k7196 
o|contracted procedure: k7199 
o|contracted procedure: k7205 
o|contracted procedure: k7234 
o|contracted procedure: k7240 
o|contracted procedure: k7256 
o|contracted procedure: k7277 
o|contracted procedure: k7284 
o|contracted procedure: k7287 
o|contracted procedure: k7296 
o|contracted procedure: k7302 
o|contracted procedure: k7322 
o|contracted procedure: k7329 
o|contracted procedure: k7338 
o|contracted procedure: k7353 
o|contracted procedure: k7366 
o|contracted procedure: k7373 
o|contracted procedure: k7382 
o|contracted procedure: k7441 
o|contracted procedure: k7394 
o|contracted procedure: k7437 
o|contracted procedure: k7414 
o|inlining procedure: k7411 
o|inlining procedure: k7411 
o|contracted procedure: k7456 
o|contracted procedure: k7472 
o|contracted procedure: k7533 
o|contracted procedure: k7498 
o|contracted procedure: k7514 
o|contracted procedure: k7530 
o|contracted procedure: k7542 
o|contracted procedure: k7548 
o|contracted procedure: k7554 
o|contracted procedure: k7560 
o|contracted procedure: k7566 
o|contracted procedure: k7578 
o|contracted procedure: k7593 
o|contracted procedure: k7604 
o|contracted procedure: k7610 
o|contracted procedure: k7616 
o|contracted procedure: k7622 
o|contracted procedure: k7640 
o|contracted procedure: k7646 
o|contracted procedure: k7652 
o|contracted procedure: k7658 
o|contracted procedure: k7667 
o|contracted procedure: k7680 
o|contracted procedure: k7686 
o|contracted procedure: k7707 
o|contracted procedure: k7723 
o|contracted procedure: k7745 
o|contracted procedure: k7803 
o|contracted procedure: k7751 
o|contracted procedure: k7759 
o|contracted procedure: k7784 
o|contracted procedure: k7774 
o|contracted procedure: k7863 
o|contracted procedure: k7877 
o|contracted procedure: k7869 
o|contracted procedure: k7895 
o|contracted procedure: k7909 
o|contracted procedure: k7945 
o|contracted procedure: k7951 
o|contracted procedure: k7960 
o|contracted procedure: k7970 
o|contracted procedure: k7974 
o|contracted procedure: k597919594 
o|contracted procedure: k7994 
o|propagated global variable: z38219591 chicken.compiler.support#profile-lambda-list 
o|contracted procedure: k8039 
o|contracted procedure: k8035 
o|contracted procedure: k8007 
o|contracted procedure: k8031 
o|contracted procedure: k8027 
o|contracted procedure: k8011 
o|contracted procedure: k8023 
o|contracted procedure: k8019 
o|contracted procedure: k8015 
o|contracted procedure: k8003 
o|contracted procedure: k8128 
o|contracted procedure: k8120 
o|contracted procedure: k8124 
o|contracted procedure: k8116 
o|contracted procedure: k8049 
o|contracted procedure: k8053 
o|contracted procedure: k8075 
o|contracted procedure: k8087 
o|contracted procedure: k8109 
o|contracted procedure: k8072 
o|contracted procedure: k8062 
o|contracted procedure: k8066 
o|contracted procedure: k8105 
o|contracted procedure: k8090 
o|contracted procedure: k8093 
o|contracted procedure: k8101 
o|propagated global variable: g11531157 chicken.compiler.support#profile-lambda-list 
o|contracted procedure: k8140 
o|contracted procedure: k6029 
o|contracted procedure: k6041 
o|contracted procedure: k6064 
o|contracted procedure: k6072 
o|contracted procedure: k8176 
o|contracted procedure: k8196 
o|contracted procedure: k597919605 
o|contracted procedure: k8192 
o|contracted procedure: k8210 
o|contracted procedure: k8206 
o|contracted procedure: k8222 
o|contracted procedure: k8236 
o|contracted procedure: k8232 
o|contracted procedure: k8247 
o|contracted procedure: k8251 
o|contracted procedure: k597919612 
o|contracted procedure: k8243 
o|contracted procedure: k8262 
o|contracted procedure: k8258 
o|contracted procedure: k8281 
o|contracted procedure: k8287 
o|contracted procedure: k8305 
o|contracted procedure: k8309 
o|contracted procedure: k8322 
o|contracted procedure: k8353 
o|contracted procedure: k8356 
o|contracted procedure: k8368 
o|contracted procedure: k8390 
o|contracted procedure: k8386 
o|contracted procedure: k8371 
o|contracted procedure: k8374 
o|contracted procedure: k8382 
o|contracted procedure: k8412 
o|contracted procedure: k8421 
o|contracted procedure: k8430 
o|contracted procedure: k8439 
o|contracted procedure: k8448 
o|contracted procedure: k8457 
o|contracted procedure: k8484 
o|contracted procedure: k8499 
o|contracted procedure: k8511 
o|contracted procedure: k8525 
o|contracted procedure: k9685 
o|contracted procedure: k8531 
o|contracted procedure: k9681 
o|contracted procedure: k8540 
o|contracted procedure: k8547 
o|contracted procedure: k8550 
o|contracted procedure: k8564 
o|contracted procedure: k8568 
o|contracted procedure: k8580 
o|contracted procedure: k8583 
o|contracted procedure: k8586 
o|contracted procedure: k8594 
o|contracted procedure: k8602 
o|contracted procedure: k8611 
o|contracted procedure: k8614 
o|contracted procedure: k8637 
o|contracted procedure: k8643 
o|contracted procedure: k8654 
o|contracted procedure: k8657 
o|contracted procedure: k8660 
o|contracted procedure: k8666 
o|contracted procedure: k8689 
o|contracted procedure: k8698 
o|contracted procedure: k8701 
o|contracted procedure: k8704 
o|contracted procedure: k8711 
o|contracted procedure: k8724 
o|contracted procedure: k8727 
o|contracted procedure: k8730 
o|contracted procedure: k8738 
o|contracted procedure: k8746 
o|contracted procedure: k6094 
o|contracted procedure: k6102 
o|contracted procedure: k6114 
o|contracted procedure: k6136 
o|contracted procedure: k6132 
o|contracted procedure: k6117 
o|contracted procedure: k6120 
o|contracted procedure: k6128 
o|contracted procedure: k8755 
o|contracted procedure: k8758 
o|contracted procedure: k8786 
o|contracted procedure: k8770 
o|contracted procedure: k8774 
o|contracted procedure: k8782 
o|contracted procedure: k8792 
o|contracted procedure: k8820 
o|contracted procedure: k8824 
o|contracted procedure: k8804 
o|contracted procedure: k8808 
o|contracted procedure: k8816 
o|contracted procedure: k8830 
o|contracted procedure: k8837 
o|contracted procedure: k8841 
o|contracted procedure: k8850 
o|contracted procedure: k8883 
o|contracted procedure: k8862 
o|contracted procedure: k8879 
o|contracted procedure: k8870 
o|contracted procedure: k8962 
o|contracted procedure: k8893 
o|contracted procedure: k8925 
o|contracted procedure: k8905 
o|contracted procedure: k8913 
o|contracted procedure: k8933 
o|contracted procedure: k8958 
o|contracted procedure: k8942 
o|contracted procedure: k8946 
o|contracted procedure: k8976 
o|contracted procedure: k8979 
o|contracted procedure: k8997 
o|contracted procedure: k9002 
o|contracted procedure: k9014 
o|contracted procedure: k9017 
o|contracted procedure: k9020 
o|contracted procedure: k9028 
o|contracted procedure: k9036 
o|contracted procedure: k9052 
o|contracted procedure: k9046 
o|contracted procedure: k9043 
o|contracted procedure: k9063 
o|contracted procedure: k9130 
o|contracted procedure: k9080 
o|contracted procedure: k9084 
o|contracted procedure: k9089 
o|contracted procedure: k9101 
o|contracted procedure: k9104 
o|contracted procedure: k9107 
o|contracted procedure: k9115 
o|contracted procedure: k9123 
o|contracted procedure: k9136 
o|contracted procedure: k9154 
o|contracted procedure: k9170 
o|contracted procedure: k9166 
o|contracted procedure: k9176 
o|contracted procedure: k9179 
o|contracted procedure: k9241 
o|contracted procedure: k9191 
o|contracted procedure: k9195 
o|contracted procedure: k9200 
o|contracted procedure: k9212 
o|contracted procedure: k9215 
o|contracted procedure: k9218 
o|contracted procedure: k9226 
o|contracted procedure: k9234 
o|contracted procedure: k9247 
o|contracted procedure: k9302 
o|contracted procedure: k9250 
o|contracted procedure: k9298 
o|contracted procedure: k9278 
o|contracted procedure: k9294 
o|contracted procedure: k9282 
o|contracted procedure: k5934 
o|contracted procedure: k9290 
o|contracted procedure: k9286 
o|contracted procedure: k9262 
o|contracted procedure: k9266 
o|contracted procedure: k9274 
o|contracted procedure: k9308 
o|contracted procedure: k9323 
o|contracted procedure: k9327 
o|contracted procedure: k9331 
o|contracted procedure: k9334 
o|contracted procedure: k9337 
o|contracted procedure: k9349 
o|contracted procedure: k9352 
o|contracted procedure: k9355 
o|contracted procedure: k9363 
o|contracted procedure: k9371 
o|contracted procedure: k9380 
o|contracted procedure: k9392 
o|contracted procedure: k9396 
o|contracted procedure: k9400 
o|contracted procedure: k9412 
o|contracted procedure: k9415 
o|contracted procedure: k9418 
o|contracted procedure: k9426 
o|contracted procedure: k9434 
o|contracted procedure: k9461 
o|contracted procedure: k9465 
o|contracted procedure: k9468 
o|contracted procedure: k9480 
o|contracted procedure: k9483 
o|contracted procedure: k9486 
o|contracted procedure: k9494 
o|contracted procedure: k9502 
o|contracted procedure: k9544 
o|contracted procedure: k9550 
o|contracted procedure: k9556 
o|contracted procedure: k9588 
o|contracted procedure: k9636 
o|contracted procedure: k9640 
o|contracted procedure: k9652 
o|contracted procedure: k9655 
o|contracted procedure: k9658 
o|contracted procedure: k9666 
o|contracted procedure: k9674 
o|contracted procedure: k9712 
o|contracted procedure: k9720 
o|contracted procedure: k9728 
o|contracted procedure: k9734 
o|contracted procedure: k9744 
o|contracted procedure: k9747 
o|contracted procedure: k9759 
o|contracted procedure: k9762 
o|contracted procedure: k9765 
o|contracted procedure: k9773 
o|contracted procedure: k9781 
o|contracted procedure: k9790 
o|contracted procedure: k9801 
o|contracted procedure: k9804 
o|contracted procedure: k9797 
o|contracted procedure: k9816 
o|contracted procedure: k9819 
o|contracted procedure: k9822 
o|contracted procedure: k9830 
o|contracted procedure: k9838 
o|contracted procedure: k9847 
o|contracted procedure: k9856 
o|contracted procedure: k9859 
o|contracted procedure: k9865 
o|inlining procedure: k9868 
o|contracted procedure: k9876 
o|inlining procedure: k9868 
o|contracted procedure: k9882 
o|contracted procedure: k9894 
o|contracted procedure: k9901 
o|contracted procedure: k9904 
o|contracted procedure: k9913 
o|contracted procedure: k9916 
o|contracted procedure: k9972 
o|contracted procedure: k9936 
o|contracted procedure: k9962 
o|contracted procedure: k9966 
o|contracted procedure: k9958 
o|contracted procedure: k9939 
o|contracted procedure: k9942 
o|contracted procedure: k9950 
o|contracted procedure: k9954 
o|contracted procedure: k9984 
o|contracted procedure: k9987 
o|contracted procedure: k9990 
o|contracted procedure: k9998 
o|contracted procedure: k10006 
o|contracted procedure: k10015 
o|contracted procedure: k10037 
o|contracted procedure: k10022 
o|contracted procedure: k10026 
o|contracted procedure: k10034 
o|contracted procedure: k10043 
o|contracted procedure: k10050 
o|contracted procedure: k10058 
o|contracted procedure: k10064 
o|contracted procedure: k10071 
o|contracted procedure: k10077 
o|contracted procedure: k10084 
o|contracted procedure: k10096 
o|contracted procedure: k10100 
o|contracted procedure: k10109 
o|contracted procedure: k10115 
o|contracted procedure: k10122 
o|contracted procedure: k10130 
o|contracted procedure: k10149 
o|contracted procedure: k10137 
o|contracted procedure: k10157 
o|contracted procedure: k10161 
o|contracted procedure: k10167 
o|contracted procedure: k10170 
o|contracted procedure: k10173 
o|contracted procedure: k10185 
o|contracted procedure: k10188 
o|contracted procedure: k10191 
o|contracted procedure: k10199 
o|contracted procedure: k10207 
o|contracted procedure: k10216 
o|contracted procedure: k10223 
o|contracted procedure: k10227 
o|contracted procedure: k10230 
o|contracted procedure: k10242 
o|contracted procedure: k10245 
o|contracted procedure: k10248 
o|contracted procedure: k10256 
o|contracted procedure: k10264 
o|contracted procedure: k10273 
o|contracted procedure: k10282 
o|contracted procedure: k10289 
o|contracted procedure: k10311 
o|contracted procedure: k10318 
o|contracted procedure: k10322 
o|contracted procedure: k10326 
o|contracted procedure: k10338 
o|contracted procedure: k10352 
o|contracted procedure: k10356 
o|contracted procedure: k10368 
o|contracted procedure: k10371 
o|contracted procedure: k10374 
o|contracted procedure: k10382 
o|contracted procedure: k10390 
o|contracted procedure: k10397 
o|contracted procedure: k10403 
o|contracted procedure: k10410 
o|contracted procedure: k10413 
o|contracted procedure: k10425 
o|contracted procedure: k10428 
o|contracted procedure: k10431 
o|contracted procedure: k10439 
o|contracted procedure: k10447 
o|contracted procedure: k10461 
o|contracted procedure: k10464 
o|contracted procedure: k10476 
o|contracted procedure: k10479 
o|contracted procedure: k10482 
o|contracted procedure: k10490 
o|contracted procedure: k10498 
o|contracted procedure: k10506 
o|contracted procedure: k10512 
o|contracted procedure: k10553 
o|contracted procedure: k10617 
o|contracted procedure: k10580 
o|contracted procedure: k10595 
o|contracted procedure: k10609 
o|contracted procedure: k10613 
o|contracted procedure: k5545 
o|contracted procedure: k5559 
o|contracted procedure: k5569 
o|contracted procedure: k5563 
o|contracted procedure: k10647 
o|contracted procedure: k10655 
o|contracted procedure: k10680 
o|contracted procedure: k10695 
o|contracted procedure: k10701 
o|contracted procedure: k10713 
o|contracted procedure: k10717 
o|contracted procedure: k10724 
o|contracted procedure: k10745 
o|contracted procedure: k10739 
o|contracted procedure: k10785 
o|contracted procedure: k10765 
o|contracted procedure: k10773 
o|contracted procedure: k10769 
o|contracted procedure: k10817 
o|contracted procedure: k10797 
o|contracted procedure: k10805 
o|contracted procedure: k10801 
o|contracted procedure: k10831 
o|contracted procedure: k10849 
o|contracted procedure: k10859 
o|contracted procedure: k10863 
o|contracted procedure: k11469 
o|contracted procedure: k11477 
o|contracted procedure: k11485 
o|contracted procedure: k11491 
o|contracted procedure: k11520 
o|contracted procedure: k11497 
o|contracted procedure: k11512 
o|contracted procedure: k11516 
o|contracted procedure: k11508 
o|contracted procedure: k11526 
o|contracted procedure: k11591 
o|contracted procedure: k11532 
o|contracted procedure: k11535 
o|contracted procedure: k11538 
o|contracted procedure: k11544 
o|contracted procedure: k11555 
o|contracted procedure: k11571 
o|contracted procedure: k11575 
o|contracted procedure: k11567 
o|contracted procedure: k11597 
o|contracted procedure: k11654 
o|contracted procedure: k11603 
o|contracted procedure: k11614 
o|contracted procedure: k11646 
o|contracted procedure: k11650 
o|contracted procedure: k11642 
o|contracted procedure: k11657 
o|contracted procedure: k11669 
o|contracted procedure: k11679 
o|contracted procedure: k11683 
o|contracted procedure: k10876 
o|contracted procedure: k10879 
o|contracted procedure: k10882 
o|contracted procedure: k10890 
o|contracted procedure: k10953 
o|contracted procedure: k10956 
o|contracted procedure: k10959 
o|contracted procedure: k10979 
o|contracted procedure: k10987 
o|contracted procedure: k10995 
o|contracted procedure: k11001 
o|contracted procedure: k11015 
o|contracted procedure: k11018 
o|contracted procedure: k11040 
o|contracted procedure: k11052 
o|contracted procedure: k11056 
o|contracted procedure: k11064 
o|contracted procedure: k11072 
o|contracted procedure: k11078 
o|contracted procedure: k11081 
o|contracted procedure: k597919705 
o|contracted procedure: k11090 
o|contracted procedure: k11105 
o|contracted procedure: k11109 
o|contracted procedure: k11117 
o|contracted procedure: k11121 
o|contracted procedure: k11127 
o|contracted procedure: k11134 
o|contracted procedure: k11140 
o|contracted procedure: k11151 
o|contracted procedure: k11226 
o|contracted procedure: k11234 
o|contracted procedure: k11169 
o|contracted procedure: k11173 
o|contracted procedure: k11181 
o|contracted procedure: k11193 
o|contracted procedure: k11196 
o|contracted procedure: k11199 
o|contracted procedure: k11207 
o|contracted procedure: k11215 
o|contracted procedure: k11245 
o|contracted procedure: k11248 
o|contracted procedure: k11296 
o|contracted procedure: k11260 
o|contracted procedure: k11286 
o|contracted procedure: k11290 
o|contracted procedure: k11282 
o|contracted procedure: k11263 
o|contracted procedure: k11266 
o|contracted procedure: k11274 
o|contracted procedure: k11278 
o|contracted procedure: k11308 
o|contracted procedure: k11311 
o|contracted procedure: k11314 
o|contracted procedure: k11322 
o|contracted procedure: k11330 
o|contracted procedure: k11349 
o|contracted procedure: k11357 
o|contracted procedure: k11369 
o|contracted procedure: k11372 
o|contracted procedure: k11375 
o|contracted procedure: k11383 
o|contracted procedure: k11391 
o|contracted procedure: k11452 
o|contracted procedure: k11416 
o|contracted procedure: k11442 
o|contracted procedure: k11446 
o|contracted procedure: k11438 
o|contracted procedure: k11419 
o|contracted procedure: k11422 
o|contracted procedure: k11430 
o|contracted procedure: k11434 
o|contracted procedure: k10907 
o|contracted procedure: k10910 
o|contracted procedure: k10922 
o|contracted procedure: k10925 
o|contracted procedure: k10928 
o|contracted procedure: k10936 
o|contracted procedure: k10944 
o|contracted procedure: k11701 
o|contracted procedure: k11713 
o|contracted procedure: k11717 
o|contracted procedure: k11725 
o|contracted procedure: k11731 
o|contracted procedure: k11739 
o|contracted procedure: k11764 
o|contracted procedure: k11768 
o|contracted procedure: k11760 
o|contracted procedure: k11774 
o|contracted procedure: k11786 
o|contracted procedure: k11807 
o|contracted procedure: k11811 
o|contracted procedure: k11803 
o|contracted procedure: k11790 
o|contracted procedure: k11815 
o|contracted procedure: k11821 
o|contracted procedure: k11833 
o|contracted procedure: k11837 
o|contracted procedure: k11845 
o|contracted procedure: k11871 
o|contracted procedure: k11906 
o|contracted procedure: k11915 
o|contracted procedure: k11924 
o|contracted procedure: k11945 
o|contracted procedure: k11954 
o|contracted procedure: k11963 
o|contracted procedure: k12140 
o|contracted procedure: k12146 
o|contracted procedure: k12162 
o|contracted procedure: k12153 
o|contracted procedure: k12174 
o|contracted procedure: k12199 
o|contracted procedure: k12211 
o|contracted procedure: k12221 
o|contracted procedure: k12225 
o|contracted procedure: k12228 
o|contracted procedure: k12263 
o|contracted procedure: k12273 
o|contracted procedure: k12277 
o|contracted procedure: k12293 
o|contracted procedure: k12429 
o|contracted procedure: k12299 
o|contracted procedure: k12305 
o|contracted procedure: k12308 
o|contracted procedure: k12317 
o|contracted procedure: k12409 
o|contracted procedure: k12328 
o|contracted procedure: k12334 
o|contracted procedure: k12354 
o|contracted procedure: k12361 
o|contracted procedure: k12369 
o|contracted procedure: k12365 
o|contracted procedure: k11984 
o|contracted procedure: k11997 
o|contracted procedure: k12001 
o|contracted procedure: k12009 
o|contracted procedure: k12012 
o|contracted procedure: k11988 
o|contracted procedure: k12024 
o|contracted procedure: k12027 
o|contracted procedure: k12030 
o|contracted procedure: k12038 
o|contracted procedure: k12046 
o|contracted procedure: k12384 
o|contracted procedure: k12390 
o|contracted procedure: k12397 
o|contracted procedure: k12420 
o|contracted procedure: k12416 
o|contracted procedure: k12462 
o|contracted procedure: k12486 
o|contracted procedure: k12473 
o|contracted procedure: k12467 
o|contracted procedure: k12494 
o|contracted procedure: k12070 
o|contracted procedure: k12074 
o|contracted procedure: k12078 
o|contracted procedure: k12083 
o|contracted procedure: k12095 
o|contracted procedure: k12098 
o|contracted procedure: k12101 
o|contracted procedure: k12109 
o|contracted procedure: k12117 
o|contracted procedure: k12503 
o|contracted procedure: k12515 
o|contracted procedure: k12524 
o|contracted procedure: k597919757 
o|contracted procedure: k12528 
o|contracted procedure: k12578 
o|contracted procedure: k12540 
o|contracted procedure: k12574 
o|contracted procedure: k12549 
o|contracted procedure: k12566 
o|contracted procedure: k12570 
o|contracted procedure: k12700 
o|contracted procedure: k12587 
o|contracted procedure: k12692 
o|contracted procedure: k12696 
o|contracted procedure: k12596 
o|contracted procedure: k12614 
o|contracted procedure: k12618 
o|contracted procedure: k12627 
o|contracted procedure: k12670 
o|contracted procedure: k12636 
o|contracted procedure: k12645 
o|contracted procedure: k12662 
o|contracted procedure: k12666 
o|contracted procedure: k12679 
o|contracted procedure: k12683 
o|contracted procedure: k12718 
o|contracted procedure: k12727 
o|contracted procedure: k12744 
o|contracted procedure: k12752 
o|contracted procedure: k12758 
o|contracted procedure: k12767 
o|contracted procedure: k12792 
o|contracted procedure: k12770 
o|contracted procedure: k6509 
o|contracted procedure: k6525 
o|contracted procedure: k12798 
o|contracted procedure: k12801 
o|contracted procedure: k12816 
o|contracted procedure: k12822 
o|contracted procedure: k12847 
o|contracted procedure: k12850 
o|contracted procedure: k12959 
o|contracted procedure: k12853 
o|contracted procedure: k12862 
o|contracted procedure: k12876 
o|contracted procedure: k12882 
o|contracted procedure: k12890 
o|contracted procedure: k12893 
o|contracted procedure: k12934 
o|contracted procedure: k12899 
o|contracted procedure: k12925 
o|contracted procedure: k12916 
o|contracted procedure: k12905 
o|contracted procedure: k12912 
o|contracted procedure: k12940 
o|contracted procedure: k12952 
o|contracted procedure: k12986 
o|contracted procedure: k12992 
o|contracted procedure: k12999 
o|contracted procedure: k13027 
o|contracted procedure: k13033 
o|contracted procedure: k13082 
o|contracted procedure: k13055 
o|contracted procedure: k13058 
o|contracted procedure: k13079 
o|contracted procedure: k13072 
o|inlining procedure: k13068 
o|inlining procedure: k13068 
o|contracted procedure: k13109 
o|contracted procedure: k13135 
o|contracted procedure: k13144 
o|contracted procedure: k13153 
o|contracted procedure: k13162 
o|contracted procedure: k13171 
o|contracted procedure: k13181 
o|contracted procedure: k13192 
o|contracted procedure: k13186 
o|contracted procedure: k13267 
o|contracted procedure: k13222 
o|contracted procedure: k13261 
o|contracted procedure: k13225 
o|contracted procedure: k13255 
o|contracted procedure: k13228 
o|contracted procedure: k13249 
o|contracted procedure: k13231 
o|contracted procedure: k13242 
o|contracted procedure: k13238 
o|contracted procedure: k13297 
o|contracted procedure: k13300 
o|contracted procedure: k13312 
o|contracted procedure: k13327 
o|contracted procedure: k13342 
o|contracted procedure: k13345 
o|contracted procedure: k13374 
o|contracted procedure: k13355 
o|contracted procedure: k13363 
o|contracted procedure: k13367 
o|contracted procedure: k13359 
o|contracted procedure: k13380 
o|contracted procedure: k13383 
o|contracted procedure: k13395 
o|contracted procedure: k13428 
o|contracted procedure: k13405 
o|contracted procedure: k13417 
o|contracted procedure: k13409 
o|contracted procedure: k13424 
o|contracted procedure: k13434 
o|contracted procedure: k13444 
o|contracted procedure: k13450 
o|contracted procedure: k13486 
o|contracted procedure: k13463 
o|contracted procedure: k13475 
o|contracted procedure: k13467 
o|contracted procedure: k13482 
o|contracted procedure: k13492 
o|contracted procedure: k13509 
o|contracted procedure: k13505 
o|contracted procedure: k13518 
o|contracted procedure: k13524 
o|contracted procedure: k13552 
o|contracted procedure: k13561 
o|contracted procedure: k13599 
o|contracted procedure: k13567 
o|contracted procedure: k13595 
o|contracted procedure: k13605 
o|contracted procedure: k13634 
o|contracted procedure: k13618 
o|contracted procedure: k13626 
o|contracted procedure: k13630 
o|contracted procedure: k13622 
o|contracted procedure: k13640 
o|contracted procedure: k13649 
o|contracted procedure: k13688 
o|contracted procedure: k13662 
o|contracted procedure: k13674 
o|contracted procedure: k13666 
o|contracted procedure: k13684 
o|contracted procedure: k13694 
o|contracted procedure: k13710 
o|contracted procedure: k13716 
o|contracted procedure: k13726 
o|contracted procedure: k13737 
o|contracted procedure: k13733 
o|contracted procedure: k13752 
o|contracted procedure: k13761 
o|contracted procedure: k13768 
o|contracted procedure: k13797 
o|contracted procedure: k13781 
o|contracted procedure: k13789 
o|contracted procedure: k13793 
o|contracted procedure: k13785 
o|contracted procedure: k13803 
o|contracted procedure: k13806 
o|contracted procedure: k13836 
o|contracted procedure: k13816 
o|contracted procedure: k13832 
o|contracted procedure: k13824 
o|contracted procedure: k13828 
o|contracted procedure: k13820 
o|contracted procedure: k13842 
o|contracted procedure: k13871 
o|contracted procedure: k13852 
o|contracted procedure: k13860 
o|contracted procedure: k13864 
o|contracted procedure: k13856 
o|contracted procedure: k13877 
o|contracted procedure: k13889 
o|contracted procedure: k13896 
o|contracted procedure: k13902 
o|contracted procedure: k13909 
o|contracted procedure: k13915 
o|contracted procedure: k13925 
o|contracted procedure: k13931 
o|contracted procedure: k13934 
o|contracted procedure: k13964 
o|contracted procedure: k13970 
o|contracted procedure: k13987 
o|contracted procedure: k13995 
o|contracted procedure: k14010 
o|contracted procedure: k14016 
o|contracted procedure: k14035 
o|contracted procedure: k14050 
o|contracted procedure: k14056 
o|contracted procedure: k14062 
o|contracted procedure: k14068 
o|contracted procedure: k14089 
o|contracted procedure: k14095 
o|contracted procedure: k14101 
o|contracted procedure: k14107 
o|contracted procedure: k14128 
o|contracted procedure: k14134 
o|contracted procedure: k14140 
o|contracted procedure: k14146 
o|contracted procedure: k14152 
o|contracted procedure: k14158 
o|contracted procedure: k14164 
o|contracted procedure: k14170 
o|contracted procedure: k14199 
o|contracted procedure: k14205 
o|contracted procedure: k14211 
o|contracted procedure: k14217 
o|contracted procedure: k14223 
o|contracted procedure: k14229 
o|contracted procedure: k14235 
o|contracted procedure: k14241 
o|contracted procedure: k14288 
o|contracted procedure: k14303 
o|contracted procedure: k14309 
o|contracted procedure: k14315 
o|contracted procedure: k14321 
o|contracted procedure: k14358 
o|contracted procedure: k14370 
o|contracted procedure: k14379 
o|contracted procedure: k14391 
o|contracted procedure: k14403 
o|contracted procedure: k14418 
o|contracted procedure: k14448 
o|contracted procedure: k14454 
o|contracted procedure: k14484 
o|contracted procedure: k14493 
o|contracted procedure: k14505 
o|contracted procedure: k14517 
o|contracted procedure: k14529 
o|contracted procedure: k14550 
o|contracted procedure: k14559 
o|contracted procedure: k14566 
o|contracted procedure: k14578 
o|contracted procedure: k14585 
o|contracted procedure: k14591 
o|contracted procedure: k14604 
o|contracted procedure: k14610 
o|contracted procedure: k14616 
o|contracted procedure: k14622 
o|contracted procedure: k14628 
o|contracted procedure: k14634 
o|contracted procedure: k14640 
o|contracted procedure: k14670 
o|contracted procedure: k14676 
o|contracted procedure: k14682 
o|contracted procedure: k14699 
o|contracted procedure: k14714 
o|contracted procedure: k14720 
o|contracted procedure: k14726 
o|contracted procedure: k14732 
o|contracted procedure: k14753 
o|contracted procedure: k14759 
o|contracted procedure: k14765 
o|contracted procedure: k14771 
o|contracted procedure: k14777 
o|contracted procedure: k14783 
o|contracted procedure: k14789 
o|contracted procedure: k14795 
o|contracted procedure: k14801 
o|contracted procedure: k14807 
o|contracted procedure: k14840 
o|contracted procedure: k14846 
o|contracted procedure: k14852 
o|contracted procedure: k14858 
o|contracted procedure: k14864 
o|contracted procedure: k14870 
o|contracted procedure: k14876 
o|contracted procedure: k14882 
o|contracted procedure: k14888 
o|contracted procedure: k14894 
o|contracted procedure: k14900 
o|contracted procedure: k14959 
o|contracted procedure: k14971 
o|contracted procedure: k14992 
o|contracted procedure: k15001 
o|contracted procedure: k15008 
o|contracted procedure: k15020 
o|contracted procedure: k15027 
o|contracted procedure: k15035 
o|contracted procedure: k15041 
o|contracted procedure: k15047 
o|contracted procedure: k15053 
o|contracted procedure: k15059 
o|contracted procedure: k15065 
o|contracted procedure: k15071 
o|contracted procedure: k15101 
o|contracted procedure: k15107 
o|contracted procedure: k15113 
o|contracted procedure: k15119 
o|contracted procedure: k15138 
o|contracted procedure: k15144 
o|contracted procedure: k15150 
o|contracted procedure: k15156 
o|contracted procedure: k15162 
o|contracted procedure: k15168 
o|contracted procedure: k15174 
o|contracted procedure: k15180 
o|contracted procedure: k15186 
o|contracted procedure: k15192 
o|contracted procedure: k15198 
o|contracted procedure: k15204 
o|contracted procedure: k15210 
o|contracted procedure: k15216 
o|contracted procedure: k15222 
o|contracted procedure: k15228 
o|contracted procedure: k15234 
o|contracted procedure: k15240 
o|contracted procedure: k15246 
o|contracted procedure: k15252 
o|contracted procedure: k15258 
o|contracted procedure: k15264 
o|contracted procedure: k15270 
o|contracted procedure: k15276 
o|contracted procedure: k15282 
o|contracted procedure: k15288 
o|contracted procedure: k15294 
o|contracted procedure: k15300 
o|contracted procedure: k15306 
o|contracted procedure: k15312 
o|contracted procedure: k15400 
o|contracted procedure: k15403 
o|contracted procedure: k15410 
o|contracted procedure: k15416 
o|contracted procedure: k15423 
o|contracted procedure: k15429 
o|contracted procedure: k15432 
o|contracted procedure: k15439 
o|contracted procedure: k15445 
o|contracted procedure: k15448 
o|contracted procedure: k15455 
o|contracted procedure: k15461 
o|contracted procedure: k15472 
o|contracted procedure: k15468 
o|contracted procedure: k15478 
o|contracted procedure: k15485 
o|contracted procedure: k15491 
o|contracted procedure: k15498 
o|contracted procedure: k15504 
o|contracted procedure: k15517 
o|contracted procedure: k15604 
o|contracted procedure: k15523 
o|contracted procedure: k15526 
o|contracted procedure: k15532 
o|contracted procedure: k15535 
o|contracted procedure: k15573 
o|contracted procedure: k15545 
o|contracted procedure: k15569 
o|contracted procedure: k15553 
o|contracted procedure: k15561 
o|contracted procedure: k15565 
o|contracted procedure: k15557 
o|contracted procedure: k15549 
o|contracted procedure: k15579 
o|contracted procedure: k15586 
o|contracted procedure: k15590 
o|contracted procedure: k15627 
o|contracted procedure: k15607 
o|contracted procedure: k15623 
o|contracted procedure: k15613 
o|contracted procedure: k15617 
o|contracted procedure: k15677 
o|contracted procedure: k15683 
o|contracted procedure: k15686 
o|contracted procedure: k15692 
o|contracted procedure: k15701 
o|contracted procedure: k15704 
o|contracted procedure: k15710 
o|contracted procedure: k15718 
o|contracted procedure: k15721 
o|contracted procedure: k15727 
o|contracted procedure: k15733 
o|contracted procedure: k15741 
o|contracted procedure: k15747 
o|contracted procedure: k15753 
o|contracted procedure: k15761 
o|contracted procedure: k15767 
o|contracted procedure: k15776 
o|contracted procedure: k15783 
o|contracted procedure: k15794 
o|contracted procedure: k15800 
o|contracted procedure: k15806 
o|contracted procedure: k15812 
o|contracted procedure: k15818 
o|contracted procedure: k15824 
o|contracted procedure: k15830 
o|contracted procedure: k15836 
o|contracted procedure: k15842 
o|contracted procedure: k15848 
o|contracted procedure: k15854 
o|contracted procedure: k15863 
o|contracted procedure: k15869 
o|contracted procedure: k15875 
o|contracted procedure: k15881 
o|contracted procedure: k15887 
o|contracted procedure: k15896 
o|contracted procedure: k15899 
o|contracted procedure: k15905 
o|contracted procedure: k15914 
o|contracted procedure: k15920 
o|contracted procedure: k15927 
o|contracted procedure: k15936 
o|contracted procedure: k15942 
o|contracted procedure: k15949 
o|contracted procedure: k15955 
o|contracted procedure: k15961 
o|contracted procedure: k15964 
o|contracted procedure: k15970 
o|contracted procedure: k15984 
o|contracted procedure: k15990 
o|contracted procedure: k16009 
o|contracted procedure: k16031 
o|contracted procedure: k16037 
o|contracted procedure: k16058 
o|contracted procedure: k16064 
o|contracted procedure: k16070 
o|contracted procedure: k16076 
o|contracted procedure: k16082 
o|contracted procedure: k16088 
o|contracted procedure: k16094 
o|contracted procedure: k16141 
o|contracted procedure: k16147 
o|contracted procedure: k16153 
o|contracted procedure: k16159 
o|contracted procedure: k16165 
o|contracted procedure: k16171 
o|contracted procedure: k16177 
o|contracted procedure: k16183 
o|contracted procedure: k16234 
o|contracted procedure: k16240 
o|contracted procedure: k16246 
o|contracted procedure: k16252 
o|contracted procedure: k16258 
o|contracted procedure: k16264 
o|contracted procedure: k16301 
o|contracted procedure: k16307 
o|contracted procedure: k16327 
o|contracted procedure: k16335 
o|contracted procedure: k16341 
o|contracted procedure: k16344 
o|contracted procedure: k16405 
o|contracted procedure: k16347 
o|contracted procedure: k16353 
o|contracted procedure: k16365 
o|contracted procedure: k16375 
o|contracted procedure: k16379 
o|contracted procedure: k16386 
o|contracted procedure: k16389 
o|contracted procedure: k16396 
o|contracted procedure: k16411 
o|contracted procedure: k16417 
o|contracted procedure: k16429 
o|contracted procedure: k16439 
o|contracted procedure: k16443 
o|contracted procedure: k16446 
o|contracted procedure: k16479 
o|contracted procedure: k16487 
o|contracted procedure: k16495 
o|contracted procedure: k16501 
o|contracted procedure: k16510 
o|contracted procedure: k16513 
o|contracted procedure: k16519 
o|contracted procedure: k16539 
o|contracted procedure: k16542 
o|contracted procedure: k16555 
o|contracted procedure: k1655219787 
o|contracted procedure: k1655219791 
o|contracted procedure: k16565 
o|contracted procedure: k16575 
o|contracted procedure: k16583 
o|contracted procedure: k16589 
o|contracted procedure: k16596 
o|contracted procedure: k16606 
o|contracted procedure: k16624 
o|contracted procedure: k16630 
o|contracted procedure: k16636 
o|contracted procedure: k16663 
o|contracted procedure: k16675 
o|contracted procedure: k16685 
o|contracted procedure: k16689 
o|contracted procedure: k16702 
o|contracted procedure: k16717 
o|contracted procedure: k16737 
o|contracted procedure: k16780 
o|inlining procedure: k16777 
o|contracted procedure: k16834 
o|contracted procedure: k16843 
o|contracted procedure: k16851 
o|contracted procedure: k16864 
o|contracted procedure: k16875 
o|contracted procedure: k16887 
o|contracted procedure: k16901 
o|contracted procedure: k16905 
o|contracted procedure: k16972 
o|contracted procedure: k16975 
o|contracted procedure: k16978 
o|contracted procedure: k16995 
o|contracted procedure: k17007 
o|contracted procedure: k17022 
o|contracted procedure: k17062 
o|contracted procedure: k17055 
o|contracted procedure: k17031 
o|contracted procedure: k17043 
o|contracted procedure: k17046 
o|contracted procedure: k17049 
o|contracted procedure: k17068 
o|contracted procedure: k17085 
o|contracted procedure: k17213 
o|contracted procedure: k17221 
o|contracted procedure: k17091 
o|contracted procedure: k17094 
o|contracted procedure: k17100 
o|contracted procedure: k17131 
o|contracted procedure: k17137 
o|contracted procedure: k17161 
o|contracted procedure: k17149 
o|contracted procedure: k17156 
o|contracted procedure: k17233 
o|contracted procedure: k17255 
o|contracted procedure: k17251 
o|contracted procedure: k17236 
o|contracted procedure: k17239 
o|contracted procedure: k17247 
o|contracted procedure: k17267 
o|contracted procedure: k17289 
o|contracted procedure: k17082 
o|contracted procedure: k17285 
o|contracted procedure: k17270 
o|contracted procedure: k17273 
o|contracted procedure: k17281 
o|contracted procedure: k17310 
o|contracted procedure: k17391 
o|contracted procedure: k17387 
o|contracted procedure: k17316 
o|contracted procedure: k17373 
o|contracted procedure: k17319 
o|inlining procedure: k17356 
o|contracted procedure: k17431 
o|contracted procedure: k17477 
o|contracted procedure: k17486 
o|contracted procedure: k17489 
o|contracted procedure: k17496 
o|contracted procedure: k17515 
o|contracted procedure: k6396 
o|contracted procedure: k6411 
o|contracted procedure: k17548 
o|contracted procedure: k17556 
o|contracted procedure: k17564 
o|contracted procedure: k17598 
o|contracted procedure: k17604 
o|contracted procedure: k17613 
o|contracted procedure: k17637 
o|contracted procedure: k17653 
o|contracted procedure: k17657 
o|contracted procedure: k17661 
o|contracted procedure: k17670 
o|contracted procedure: k17680 
o|contracted procedure: k17684 
o|contracted procedure: k17700 
o|contracted procedure: k597919833 
o|inlining procedure: k17704 
o|contracted procedure: k597919840 
o|inlining procedure: k17704 
o|contracted procedure: k17720 
o|contracted procedure: k17727 
o|contracted procedure: k17742 
o|contracted procedure: k17755 
o|contracted procedure: k17779 
o|contracted procedure: k17807 
o|contracted procedure: k17836 
o|contracted procedure: k17845 
o|contracted procedure: k17860 
o|contracted procedure: k17866 
o|contracted procedure: k17889 
o|contracted procedure: k17883 
o|contracted procedure: k17909 
o|contracted procedure: k17903 
o|contracted procedure: k17949 
o|contracted procedure: k17955 
o|contracted procedure: k17974 
o|contracted procedure: k17968 
o|contracted procedure: k18059 
o|contracted procedure: k18071 
o|contracted procedure: k18081 
o|contracted procedure: k18085 
o|contracted procedure: k18033 
o|contracted procedure: k18047 
o|contracted procedure: k18051 
o|contracted procedure: k18133 
o|contracted procedure: k18117 
o|contracted procedure: k18184 
o|contracted procedure: k18193 
o|simplifications: ((if . 7) (let . 314)) 
o|removed binding forms: 1256 
o|contracted procedure: k7356 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest971972 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest971972 0 
o|contracted procedure: k8293 
o|contracted procedure: k9868 
o|inlining procedure: k10721 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest30083011 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest30083011 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest30083011 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr rest30083011 0 
o|contracted procedure: k16382 
o|contracted procedure: k17693 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest44824485 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest44824485 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest45714572 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest45714572 0 
o|simplifications: ((let . 1)) 
o|replaced variables: 3 
o|removed binding forms: 8 
(o x)|known list op on rest arg sublist: ##core#rest-null? r13226 1 
(o x)|known list op on rest arg sublist: ##core#rest-car r13226 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r13226 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r13226 1 
o|removed binding forms: 3 
o|removed binding forms: 2 
o|direct leaf routine/allocation: loop463 0 
o|direct leaf routine/allocation: loop863 0 
o|direct leaf routine/allocation: loop1108 0 
o|direct leaf routine/allocation: a8161 0 
o|direct leaf routine/allocation: g423424 3 
o|direct leaf routine/allocation: g27982799 0 
o|direct leaf routine/allocation: a17525 0 
o|converted assignments to bindings: (loop463) 
o|converted assignments to bindings: (loop863) 
o|converted assignments to bindings: (loop1108) 
o|contracted procedure: "(mini-srfi-1.scm:135) k6046" 
o|contracted procedure: "(mini-srfi-1.scm:190) k6403" 
o|simplifications: ((let . 3)) 
o|removed binding forms: 2 
o|customizable procedures: (for-each-loop45454560 loop4409 g43334347 for-each-loop43324357 doloop43694370 loop4310 loop558 tmp15335 tmp25336 map-loop41714192 map-loop42024223 resolve4104 loop4116 g40384045 for-each-loop40374048 k16504 walkeach3993 walk3992 chicken.compiler.support#lset-adjoin/eq? k16414 for-each-loop39703980 k16350 walk3921 for-each-loop39453956 k15695 k15770 k15857 k15890 k15908 k15930 k15510 k14962 k14974 k15011 g36593660 k14487 k14496 k14508 k14520 k14532 k14569 err3385 g35013502 g33803381 chicken.compiler.support#foreign-type-argument-converter chicken.compiler.support#foreign-type-result-converter chicken.compiler.support#follow-without-loop k13315 k13330 k13453 k13495 k13521 k13564 k13608 k13652 k13697 k13771 repeat3033 g32853286 k13670 k13471 k13413 k13018 k12977 k12761 a12776 loop586 matchn2789 loop2818 match12788 resolve2787 walk2601 map-loop26102627 loop2764 k12311 k12348 uses-foreign-stubs?2642 walk2558 map-loop25712591 for-each-loop27142726 for-each-loop27362754 rec2522 lp2496 map-loop20822099 map-loop21952214 g23882397 map-loop23822407 g22822291 map-loop22762296 map-loop23062325 g23492358 map-loop23432368 walk2225 map-loop21102128 walk2422 for-each-loop24652475 lp2453 g21382145 for-each-loop21372149 loop2156 loop22159 loop230 fold2059 k9737 k10341 map-loop20312048 map-loop20052022 map-loop19791996 loop1960 map-loop19361953 chicken.compiler.support#cons* map-loop19101927 loop1901 map-loop18611878 map-loop18401885 chicken.compiler.support#last map-loop17981815 walk1741 map-loop17691786 map-loop17121729 k9066 k9311 k9509 map-loop16811698 map-loop16351652 map-loop16041621 map-loop15551572 map-loop15111528 k8993 map-loop14741491 loop1441 map-loop436454 g14091418 map-loop14031421 k8624 walk1341 map-loop13661383 map-loop12361253 g12241225 foldr408411 g413414 map-loop11411162 tmp14965 tmp24966 loop1091 k7762 loop1062 k7634 doloop978979 loop958 fold951 chicken.compiler.support#every k7208 k7215 loop894 loop878 k6943 g800825 for-each-loop799835 collect751 g756763 for-each-loop755774 text700 chicken.compiler.support#test-debugging-mode dump701 for-each-loop704721 chicken.compiler.support#any loop480 loop340 loop327 loop314 chicken.compiler.support#take) 
o|calls to known targets: 530 
o|identified direct recursive calls: f_5482 1 
o|identified direct recursive calls: f_5813 1 
o|identified direct recursive calls: f_6149 1 
o|identified direct recursive calls: f_6208 2 
o|identified direct recursive calls: f_7098 1 
o|identified direct recursive calls: f_7139 1 
o|identified direct recursive calls: f_7405 1 
o|unused rest argument: rest971972 f_7496 
o|identified direct recursive calls: f_7940 1 
o|identified direct recursive calls: f_8082 1 
o|identified direct recursive calls: f_6036 1 
o|identified direct recursive calls: f_8363 1 
o|identified direct recursive calls: f_6109 1 
o|identified direct recursive calls: f_8506 4 
o|identified direct recursive calls: f_9931 1 
o|identified direct recursive calls: f_5540 1 
o|identified direct recursive calls: f_10690 1 
o|identified direct recursive calls: f_10675 1 
o|identified direct recursive calls: f_11622 1 
o|identified direct recursive calls: f_11255 1 
o|identified direct recursive calls: f_10972 1 
o|identified direct recursive calls: f_11411 1 
o|identified direct recursive calls: f_11743 1 
o|identified direct recursive calls: f_11866 1 
o|identified direct recursive calls: f_12535 1 
o|unused rest argument: rest30083011 f_13220 
o|identified direct recursive calls: f_17228 1 
o|identified direct recursive calls: f_17262 1 
o|identified direct recursive calls: f_6391 1 
o|unused rest argument: rest44824485 f_17966 
o|unused rest argument: rest45714572 f_18115 
o|fast box initializations: 94 
o|fast global references: 63 
o|fast global assignments: 22 
o|dropping unused closure argument: f_14356 
o|dropping unused closure argument: f_14377 
o|dropping unused closure argument: f_16814 
o|dropping unused closure argument: f_5482 
o|dropping unused closure argument: f_5743 
o|dropping unused closure argument: f_5777 
o|dropping unused closure argument: f_5807 
o|dropping unused closure argument: f_6143 
o|dropping unused closure argument: f_6149 
o|dropping unused closure argument: f_6202 
o|dropping unused closure argument: f_6687 
o|dropping unused closure argument: f_7098 
o|dropping unused closure argument: f_7445 
o|dropping unused closure argument: f_7940 
*/
/* end of file */
