/* Generated from scrutinizer.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: scrutinizer.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -no-lambda-info -no-trace -emit-import-library chicken.compiler.scrutinizer -output-file scrutinizer.c
   unit: scrutinizer
   uses: eval expand data-structures port pathname internal extras support library
*/
#include "chicken.h"

static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_eval_toplevel)
C_externimport void C_ccall C_eval_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_expand_toplevel)
C_externimport void C_ccall C_expand_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_data_2dstructures_toplevel)
C_externimport void C_ccall C_data_2dstructures_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_port_toplevel)
C_externimport void C_ccall C_port_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_pathname_toplevel)
C_externimport void C_ccall C_pathname_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_internal_toplevel)
C_externimport void C_ccall C_internal_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_extras_toplevel)
C_externimport void C_ccall C_extras_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_support_toplevel)
C_externimport void C_ccall C_support_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[659];
static double C_possibly_force_alignment;


C_noret_decl(f25872)
static void C_ccall f25872(C_word c,C_word *av) C_noret;
C_noret_decl(f25979)
static void C_ccall f25979(C_word c,C_word *av) C_noret;
C_noret_decl(f25994)
static void C_ccall f25994(C_word c,C_word *av) C_noret;
C_noret_decl(f26001)
static void C_ccall f26001(C_word c,C_word *av) C_noret;
C_noret_decl(f26243)
static void C_ccall f26243(C_word c,C_word *av) C_noret;
C_noret_decl(f26391)
static void C_ccall f26391(C_word c,C_word *av) C_noret;
C_noret_decl(f26456)
static void C_ccall f26456(C_word c,C_word *av) C_noret;
C_noret_decl(f26463)
static void C_ccall f26463(C_word c,C_word *av) C_noret;
C_noret_decl(f26609)
static void C_ccall f26609(C_word c,C_word *av) C_noret;
C_noret_decl(f26614)
static void C_ccall f26614(C_word c,C_word *av) C_noret;
C_noret_decl(f26623)
static void C_ccall f26623(C_word c,C_word *av) C_noret;
C_noret_decl(f26630)
static void C_ccall f26630(C_word c,C_word *av) C_noret;
C_noret_decl(f26669)
static void C_ccall f26669(C_word c,C_word *av) C_noret;
C_noret_decl(f26674)
static void C_ccall f26674(C_word c,C_word *av) C_noret;
C_noret_decl(f26679)
static void C_ccall f26679(C_word c,C_word *av) C_noret;
C_noret_decl(f26686)
static void C_ccall f26686(C_word c,C_word *av) C_noret;
C_noret_decl(f26693)
static void C_ccall f26693(C_word c,C_word *av) C_noret;
C_noret_decl(f26700)
static void C_ccall f26700(C_word c,C_word *av) C_noret;
C_noret_decl(f26707)
static void C_ccall f26707(C_word c,C_word *av) C_noret;
C_noret_decl(f27984)
static void C_ccall f27984(C_word c,C_word *av) C_noret;
C_noret_decl(f27989)
static void C_ccall f27989(C_word c,C_word *av) C_noret;
C_noret_decl(f28008)
static void C_ccall f28008(C_word c,C_word *av) C_noret;
C_noret_decl(f28236)
static void C_ccall f28236(C_word c,C_word *av) C_noret;
C_noret_decl(f28241)
static void C_ccall f28241(C_word c,C_word *av) C_noret;
C_noret_decl(f28246)
static void C_ccall f28246(C_word c,C_word *av) C_noret;
C_noret_decl(f_10003)
static void C_ccall f_10003(C_word c,C_word *av) C_noret;
C_noret_decl(f_10013)
static void C_ccall f_10013(C_word c,C_word *av) C_noret;
C_noret_decl(f_10017)
static void C_ccall f_10017(C_word c,C_word *av) C_noret;
C_noret_decl(f_10021)
static void C_ccall f_10021(C_word c,C_word *av) C_noret;
C_noret_decl(f_10023)
static void C_ccall f_10023(C_word c,C_word *av) C_noret;
C_noret_decl(f_10045)
static void C_ccall f_10045(C_word c,C_word *av) C_noret;
C_noret_decl(f_10052)
static void C_ccall f_10052(C_word c,C_word *av) C_noret;
C_noret_decl(f_10055)
static void C_ccall f_10055(C_word c,C_word *av) C_noret;
C_noret_decl(f_10058)
static void C_ccall f_10058(C_word c,C_word *av) C_noret;
C_noret_decl(f_10066)
static void C_ccall f_10066(C_word c,C_word *av) C_noret;
C_noret_decl(f_10075)
static void C_ccall f_10075(C_word c,C_word *av) C_noret;
C_noret_decl(f_10082)
static void C_ccall f_10082(C_word c,C_word *av) C_noret;
C_noret_decl(f_10089)
static void C_ccall f_10089(C_word c,C_word *av) C_noret;
C_noret_decl(f_10097)
static void C_ccall f_10097(C_word c,C_word *av) C_noret;
C_noret_decl(f_10099)
static void C_fcall f_10099(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10107)
static void C_fcall f_10107(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10111)
static void C_fcall f_10111(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10123)
static void C_ccall f_10123(C_word c,C_word *av) C_noret;
C_noret_decl(f_10158)
static void C_ccall f_10158(C_word c,C_word *av) C_noret;
C_noret_decl(f_10180)
static void C_ccall f_10180(C_word c,C_word *av) C_noret;
C_noret_decl(f_10192)
static void C_fcall f_10192(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10205)
static void C_ccall f_10205(C_word c,C_word *av) C_noret;
C_noret_decl(f_10227)
static void C_ccall f_10227(C_word c,C_word *av) C_noret;
C_noret_decl(f_10236)
static void C_ccall f_10236(C_word c,C_word *av) C_noret;
C_noret_decl(f_10238)
static void C_fcall f_10238(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10309)
static void C_ccall f_10309(C_word c,C_word *av) C_noret;
C_noret_decl(f_10312)
static void C_ccall f_10312(C_word c,C_word *av) C_noret;
C_noret_decl(f_10315)
static void C_ccall f_10315(C_word c,C_word *av) C_noret;
C_noret_decl(f_10321)
static void C_ccall f_10321(C_word c,C_word *av) C_noret;
C_noret_decl(f_10324)
static void C_ccall f_10324(C_word c,C_word *av) C_noret;
C_noret_decl(f_10327)
static void C_ccall f_10327(C_word c,C_word *av) C_noret;
C_noret_decl(f_10333)
static void C_ccall f_10333(C_word c,C_word *av) C_noret;
C_noret_decl(f_10347)
static void C_fcall f_10347(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10355)
static void C_fcall f_10355(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10389)
static void C_ccall f_10389(C_word c,C_word *av) C_noret;
C_noret_decl(f_10438)
static void C_ccall f_10438(C_word c,C_word *av) C_noret;
C_noret_decl(f_10453)
static void C_fcall f_10453(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10456)
static void C_ccall f_10456(C_word c,C_word *av) C_noret;
C_noret_decl(f_10459)
static void C_ccall f_10459(C_word c,C_word *av) C_noret;
C_noret_decl(f_10468)
static void C_fcall f_10468(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10474)
static void C_ccall f_10474(C_word c,C_word *av) C_noret;
C_noret_decl(f_10480)
static void C_ccall f_10480(C_word c,C_word *av) C_noret;
C_noret_decl(f_10483)
static void C_ccall f_10483(C_word c,C_word *av) C_noret;
C_noret_decl(f_10490)
static void C_ccall f_10490(C_word c,C_word *av) C_noret;
C_noret_decl(f_10496)
static void C_ccall f_10496(C_word c,C_word *av) C_noret;
C_noret_decl(f_10499)
static void C_ccall f_10499(C_word c,C_word *av) C_noret;
C_noret_decl(f_10502)
static void C_ccall f_10502(C_word c,C_word *av) C_noret;
C_noret_decl(f_10505)
static void C_ccall f_10505(C_word c,C_word *av) C_noret;
C_noret_decl(f_10508)
static void C_ccall f_10508(C_word c,C_word *av) C_noret;
C_noret_decl(f_10511)
static void C_ccall f_10511(C_word c,C_word *av) C_noret;
C_noret_decl(f_10519)
static void C_ccall f_10519(C_word c,C_word *av) C_noret;
C_noret_decl(f_10539)
static void C_ccall f_10539(C_word c,C_word *av) C_noret;
C_noret_decl(f_10543)
static void C_ccall f_10543(C_word c,C_word *av) C_noret;
C_noret_decl(f_10572)
static void C_ccall f_10572(C_word c,C_word *av) C_noret;
C_noret_decl(f_10579)
static void C_fcall f_10579(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10582)
static void C_fcall f_10582(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10602)
static void C_fcall f_10602(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10609)
static void C_ccall f_10609(C_word c,C_word *av) C_noret;
C_noret_decl(f_10631)
static void C_ccall f_10631(C_word c,C_word *av) C_noret;
C_noret_decl(f_10635)
static void C_ccall f_10635(C_word c,C_word *av) C_noret;
C_noret_decl(f_10642)
static void C_ccall f_10642(C_word c,C_word *av) C_noret;
C_noret_decl(f_10644)
static void C_ccall f_10644(C_word c,C_word *av) C_noret;
C_noret_decl(f_10671)
static void C_ccall f_10671(C_word c,C_word *av) C_noret;
C_noret_decl(f_10678)
static void C_fcall f_10678(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10694)
static void C_ccall f_10694(C_word c,C_word *av) C_noret;
C_noret_decl(f_10698)
static void C_ccall f_10698(C_word c,C_word *av) C_noret;
C_noret_decl(f_10700)
static void C_ccall f_10700(C_word c,C_word *av) C_noret;
C_noret_decl(f_10707)
static void C_ccall f_10707(C_word c,C_word *av) C_noret;
C_noret_decl(f_10713)
static void C_ccall f_10713(C_word c,C_word *av) C_noret;
C_noret_decl(f_10719)
static void C_ccall f_10719(C_word c,C_word *av) C_noret;
C_noret_decl(f_10722)
static void C_ccall f_10722(C_word c,C_word *av) C_noret;
C_noret_decl(f_10725)
static void C_ccall f_10725(C_word c,C_word *av) C_noret;
C_noret_decl(f_10728)
static void C_ccall f_10728(C_word c,C_word *av) C_noret;
C_noret_decl(f_10733)
static void C_ccall f_10733(C_word c,C_word *av) C_noret;
C_noret_decl(f_10739)
static void C_ccall f_10739(C_word c,C_word *av) C_noret;
C_noret_decl(f_10742)
static void C_fcall f_10742(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10749)
static void C_fcall f_10749(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10752)
static void C_ccall f_10752(C_word c,C_word *av) C_noret;
C_noret_decl(f_10777)
static void C_ccall f_10777(C_word c,C_word *av) C_noret;
C_noret_decl(f_10781)
static void C_ccall f_10781(C_word c,C_word *av) C_noret;
C_noret_decl(f_10787)
static void C_ccall f_10787(C_word c,C_word *av) C_noret;
C_noret_decl(f_10790)
static void C_ccall f_10790(C_word c,C_word *av) C_noret;
C_noret_decl(f_10800)
static void C_fcall f_10800(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10812)
static void C_fcall f_10812(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10837)
static void C_ccall f_10837(C_word c,C_word *av) C_noret;
C_noret_decl(f_10854)
static void C_fcall f_10854(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10867)
static void C_ccall f_10867(C_word c,C_word *av) C_noret;
C_noret_decl(f_10870)
static void C_ccall f_10870(C_word c,C_word *av) C_noret;
C_noret_decl(f_10873)
static void C_ccall f_10873(C_word c,C_word *av) C_noret;
C_noret_decl(f_10879)
static void C_fcall f_10879(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10885)
static void C_ccall f_10885(C_word c,C_word *av) C_noret;
C_noret_decl(f_10891)
static void C_ccall f_10891(C_word c,C_word *av) C_noret;
C_noret_decl(f_10897)
static void C_ccall f_10897(C_word c,C_word *av) C_noret;
C_noret_decl(f_10920)
static void C_ccall f_10920(C_word c,C_word *av) C_noret;
C_noret_decl(f_10935)
static void C_ccall f_10935(C_word c,C_word *av) C_noret;
C_noret_decl(f_10941)
static void C_ccall f_10941(C_word c,C_word *av) C_noret;
C_noret_decl(f_10947)
static void C_ccall f_10947(C_word c,C_word *av) C_noret;
C_noret_decl(f_10963)
static void C_ccall f_10963(C_word c,C_word *av) C_noret;
C_noret_decl(f_10969)
static void C_ccall f_10969(C_word c,C_word *av) C_noret;
C_noret_decl(f_10973)
static void C_fcall f_10973(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10981)
static void C_ccall f_10981(C_word c,C_word *av) C_noret;
C_noret_decl(f_11003)
static void C_ccall f_11003(C_word c,C_word *av) C_noret;
C_noret_decl(f_11014)
static void C_ccall f_11014(C_word c,C_word *av) C_noret;
C_noret_decl(f_11018)
static void C_ccall f_11018(C_word c,C_word *av) C_noret;
C_noret_decl(f_11027)
static void C_ccall f_11027(C_word c,C_word *av) C_noret;
C_noret_decl(f_11030)
static void C_ccall f_11030(C_word c,C_word *av) C_noret;
C_noret_decl(f_11040)
static void C_fcall f_11040(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11052)
static void C_fcall f_11052(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11077)
static void C_ccall f_11077(C_word c,C_word *av) C_noret;
C_noret_decl(f_11086)
static void C_fcall f_11086(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11096)
static void C_ccall f_11096(C_word c,C_word *av) C_noret;
C_noret_decl(f_11125)
static void C_ccall f_11125(C_word c,C_word *av) C_noret;
C_noret_decl(f_11127)
static void C_ccall f_11127(C_word c,C_word *av) C_noret;
C_noret_decl(f_11135)
static void C_ccall f_11135(C_word c,C_word *av) C_noret;
C_noret_decl(f_11143)
static void C_ccall f_11143(C_word c,C_word *av) C_noret;
C_noret_decl(f_11160)
static void C_ccall f_11160(C_word c,C_word *av) C_noret;
C_noret_decl(f_11162)
static void C_fcall f_11162(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11200)
static void C_fcall f_11200(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11229)
static void C_ccall f_11229(C_word c,C_word *av) C_noret;
C_noret_decl(f_11258)
static void C_ccall f_11258(C_word c,C_word *av) C_noret;
C_noret_decl(f_11264)
static void C_ccall f_11264(C_word c,C_word *av) C_noret;
C_noret_decl(f_11283)
static void C_ccall f_11283(C_word c,C_word *av) C_noret;
C_noret_decl(f_11289)
static void C_fcall f_11289(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11292)
static void C_fcall f_11292(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11314)
static void C_ccall f_11314(C_word c,C_word *av) C_noret;
C_noret_decl(f_11346)
static void C_ccall f_11346(C_word c,C_word *av) C_noret;
C_noret_decl(f_11349)
static void C_ccall f_11349(C_word c,C_word *av) C_noret;
C_noret_decl(f_11362)
static void C_fcall f_11362(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11383)
static void C_ccall f_11383(C_word c,C_word *av) C_noret;
C_noret_decl(f_11389)
static void C_ccall f_11389(C_word c,C_word *av) C_noret;
C_noret_decl(f_11392)
static void C_ccall f_11392(C_word c,C_word *av) C_noret;
C_noret_decl(f_11402)
static void C_ccall f_11402(C_word c,C_word *av) C_noret;
C_noret_decl(f_11421)
static void C_ccall f_11421(C_word c,C_word *av) C_noret;
C_noret_decl(f_11446)
static void C_fcall f_11446(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11456)
static void C_ccall f_11456(C_word c,C_word *av) C_noret;
C_noret_decl(f_11458)
static void C_fcall f_11458(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11468)
static void C_ccall f_11468(C_word c,C_word *av) C_noret;
C_noret_decl(f_11529)
static void C_ccall f_11529(C_word c,C_word *av) C_noret;
C_noret_decl(f_11532)
static void C_ccall f_11532(C_word c,C_word *av) C_noret;
C_noret_decl(f_11535)
static void C_ccall f_11535(C_word c,C_word *av) C_noret;
C_noret_decl(f_11538)
static void C_ccall f_11538(C_word c,C_word *av) C_noret;
C_noret_decl(f_11541)
static void C_ccall f_11541(C_word c,C_word *av) C_noret;
C_noret_decl(f_11547)
static void C_ccall f_11547(C_word c,C_word *av) C_noret;
C_noret_decl(f_11570)
static void C_ccall f_11570(C_word c,C_word *av) C_noret;
C_noret_decl(f_11574)
static void C_ccall f_11574(C_word c,C_word *av) C_noret;
C_noret_decl(f_11582)
static void C_ccall f_11582(C_word c,C_word *av) C_noret;
C_noret_decl(f_11585)
static void C_ccall f_11585(C_word c,C_word *av) C_noret;
C_noret_decl(f_11588)
static void C_ccall f_11588(C_word c,C_word *av) C_noret;
C_noret_decl(f_11591)
static void C_ccall f_11591(C_word c,C_word *av) C_noret;
C_noret_decl(f_11608)
static void C_fcall f_11608(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11618)
static void C_ccall f_11618(C_word c,C_word *av) C_noret;
C_noret_decl(f_11645)
static void C_ccall f_11645(C_word c,C_word *av) C_noret;
C_noret_decl(f_11647)
static void C_fcall f_11647(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11653)
static void C_fcall f_11653(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11663)
static void C_ccall f_11663(C_word c,C_word *av) C_noret;
C_noret_decl(f_11675)
static void C_ccall f_11675(C_word c,C_word *av) C_noret;
C_noret_decl(f_11683)
static void C_fcall f_11683(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11703)
static void C_ccall f_11703(C_word c,C_word *av) C_noret;
C_noret_decl(f_11716)
static void C_ccall f_11716(C_word c,C_word *av) C_noret;
C_noret_decl(f_11728)
static void C_ccall f_11728(C_word c,C_word *av) C_noret;
C_noret_decl(f_11735)
static void C_ccall f_11735(C_word c,C_word *av) C_noret;
C_noret_decl(f_11737)
static void C_fcall f_11737(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11762)
static void C_ccall f_11762(C_word c,C_word *av) C_noret;
C_noret_decl(f_11784)
static void C_ccall f_11784(C_word c,C_word *av) C_noret;
C_noret_decl(f_11797)
static void C_ccall f_11797(C_word c,C_word *av) C_noret;
C_noret_decl(f_11818)
static void C_ccall f_11818(C_word c,C_word *av) C_noret;
C_noret_decl(f_11843)
static void C_ccall f_11843(C_word c,C_word *av) C_noret;
C_noret_decl(f_11847)
static void C_ccall f_11847(C_word c,C_word *av) C_noret;
C_noret_decl(f_11858)
static void C_fcall f_11858(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11867)
static void C_fcall f_11867(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_11964)
static void C_ccall f_11964(C_word c,C_word *av) C_noret;
C_noret_decl(f_11989)
static void C_fcall f_11989(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11995)
static void C_ccall f_11995(C_word c,C_word *av) C_noret;
C_noret_decl(f_12011)
static void C_ccall f_12011(C_word c,C_word *av) C_noret;
C_noret_decl(f_12018)
static void C_ccall f_12018(C_word c,C_word *av) C_noret;
C_noret_decl(f_12035)
static void C_ccall f_12035(C_word c,C_word *av) C_noret;
C_noret_decl(f_12052)
static void C_fcall f_12052(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12095)
static void C_ccall f_12095(C_word c,C_word *av) C_noret;
C_noret_decl(f_12120)
static void C_fcall f_12120(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12126)
static void C_ccall f_12126(C_word c,C_word *av) C_noret;
C_noret_decl(f_12131)
static void C_ccall f_12131(C_word c,C_word *av) C_noret;
C_noret_decl(f_12137)
static void C_ccall f_12137(C_word c,C_word *av) C_noret;
C_noret_decl(f_12142)
static void C_fcall f_12142(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12148)
static void C_fcall f_12148(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12161)
static void C_ccall f_12161(C_word c,C_word *av) C_noret;
C_noret_decl(f_12178)
static void C_fcall f_12178(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12195)
static void C_fcall f_12195(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12208)
static void C_ccall f_12208(C_word c,C_word *av) C_noret;
C_noret_decl(f_12236)
static void C_fcall f_12236(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12249)
static void C_ccall f_12249(C_word c,C_word *av) C_noret;
C_noret_decl(f_12286)
static void C_ccall f_12286(C_word c,C_word *av) C_noret;
C_noret_decl(f_12327)
static void C_fcall f_12327(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12340)
static void C_ccall f_12340(C_word c,C_word *av) C_noret;
C_noret_decl(f_12370)
static void C_ccall f_12370(C_word c,C_word *av) C_noret;
C_noret_decl(f_12440)
static void C_ccall f_12440(C_word c,C_word *av) C_noret;
C_noret_decl(f_12444)
static void C_fcall f_12444(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12454)
static void C_ccall f_12454(C_word c,C_word *av) C_noret;
C_noret_decl(f_12458)
static void C_fcall f_12458(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12479)
static void C_ccall f_12479(C_word c,C_word *av) C_noret;
C_noret_decl(f_12484)
static void C_ccall f_12484(C_word c,C_word *av) C_noret;
C_noret_decl(f_12488)
static void C_ccall f_12488(C_word c,C_word *av) C_noret;
C_noret_decl(f_12491)
static void C_ccall f_12491(C_word c,C_word *av) C_noret;
C_noret_decl(f_12500)
static void C_ccall f_12500(C_word c,C_word *av) C_noret;
C_noret_decl(f_12520)
static void C_ccall f_12520(C_word c,C_word *av) C_noret;
C_noret_decl(f_12525)
static void C_ccall f_12525(C_word c,C_word *av) C_noret;
C_noret_decl(f_12529)
static void C_ccall f_12529(C_word c,C_word *av) C_noret;
C_noret_decl(f_12532)
static void C_ccall f_12532(C_word c,C_word *av) C_noret;
C_noret_decl(f_12541)
static void C_ccall f_12541(C_word c,C_word *av) C_noret;
C_noret_decl(f_12559)
static void C_ccall f_12559(C_word c,C_word *av) C_noret;
C_noret_decl(f_12578)
static void C_ccall f_12578(C_word c,C_word *av) C_noret;
C_noret_decl(f_12694)
static void C_fcall f_12694(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12706)
static void C_ccall f_12706(C_word c,C_word *av) C_noret;
C_noret_decl(f_12709)
static void C_ccall f_12709(C_word c,C_word *av) C_noret;
C_noret_decl(f_12712)
static void C_ccall f_12712(C_word c,C_word *av) C_noret;
C_noret_decl(f_12715)
static void C_ccall f_12715(C_word c,C_word *av) C_noret;
C_noret_decl(f_12721)
static void C_ccall f_12721(C_word c,C_word *av) C_noret;
C_noret_decl(f_12814)
static void C_ccall f_12814(C_word c,C_word *av) C_noret;
C_noret_decl(f_12924)
static void C_ccall f_12924(C_word c,C_word *av) C_noret;
C_noret_decl(f_12957)
static void C_ccall f_12957(C_word c,C_word *av) C_noret;
C_noret_decl(f_13026)
static void C_ccall f_13026(C_word c,C_word *av) C_noret;
C_noret_decl(f_13059)
static void C_ccall f_13059(C_word c,C_word *av) C_noret;
C_noret_decl(f_13133)
static void C_ccall f_13133(C_word c,C_word *av) C_noret;
C_noret_decl(f_13167)
static void C_ccall f_13167(C_word c,C_word *av) C_noret;
C_noret_decl(f_13209)
static void C_ccall f_13209(C_word c,C_word *av) C_noret;
C_noret_decl(f_13243)
static void C_ccall f_13243(C_word c,C_word *av) C_noret;
C_noret_decl(f_13431)
static void C_fcall f_13431(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13437)
static void C_fcall f_13437(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13470)
static void C_ccall f_13470(C_word c,C_word *av) C_noret;
C_noret_decl(f_13484)
static void C_ccall f_13484(C_word c,C_word *av) C_noret;
C_noret_decl(f_13503)
static void C_fcall f_13503(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13506)
static void C_fcall f_13506(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13512)
static void C_ccall f_13512(C_word c,C_word *av) C_noret;
C_noret_decl(f_13533)
static void C_ccall f_13533(C_word c,C_word *av) C_noret;
C_noret_decl(f_13537)
static void C_ccall f_13537(C_word c,C_word *av) C_noret;
C_noret_decl(f_13548)
static void C_ccall f_13548(C_word c,C_word *av) C_noret;
C_noret_decl(f_13550)
static void C_ccall f_13550(C_word c,C_word *av) C_noret;
C_noret_decl(f_13585)
static void C_ccall f_13585(C_word c,C_word *av) C_noret;
C_noret_decl(f_13587)
static void C_fcall f_13587(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13628)
static void C_ccall f_13628(C_word c,C_word *av) C_noret;
C_noret_decl(f_13656)
static void C_ccall f_13656(C_word c,C_word *av) C_noret;
C_noret_decl(f_13662)
static void C_ccall f_13662(C_word c,C_word *av) C_noret;
C_noret_decl(f_13674)
static void C_fcall f_13674(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13694)
static void C_ccall f_13694(C_word c,C_word *av) C_noret;
C_noret_decl(f_13697)
static void C_ccall f_13697(C_word c,C_word *av) C_noret;
C_noret_decl(f_13700)
static void C_ccall f_13700(C_word c,C_word *av) C_noret;
C_noret_decl(f_13703)
static void C_ccall f_13703(C_word c,C_word *av) C_noret;
C_noret_decl(f_13706)
static void C_ccall f_13706(C_word c,C_word *av) C_noret;
C_noret_decl(f_13709)
static void C_ccall f_13709(C_word c,C_word *av) C_noret;
C_noret_decl(f_13716)
static void C_fcall f_13716(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13724)
static void C_ccall f_13724(C_word c,C_word *av) C_noret;
C_noret_decl(f_13728)
static void C_ccall f_13728(C_word c,C_word *av) C_noret;
C_noret_decl(f_13743)
static void C_ccall f_13743(C_word c,C_word *av) C_noret;
C_noret_decl(f_13749)
static void C_ccall f_13749(C_word c,C_word *av) C_noret;
C_noret_decl(f_13759)
static void C_ccall f_13759(C_word c,C_word *av) C_noret;
C_noret_decl(f_13765)
static void C_ccall f_13765(C_word c,C_word *av) C_noret;
C_noret_decl(f_13771)
static void C_ccall f_13771(C_word c,C_word *av) C_noret;
C_noret_decl(f_13774)
static void C_ccall f_13774(C_word c,C_word *av) C_noret;
C_noret_decl(f_13777)
static void C_ccall f_13777(C_word c,C_word *av) C_noret;
C_noret_decl(f_13794)
static void C_fcall f_13794(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13819)
static void C_ccall f_13819(C_word c,C_word *av) C_noret;
C_noret_decl(f_13830)
static void C_ccall f_13830(C_word c,C_word *av) C_noret;
C_noret_decl(f_13848)
static void C_ccall f_13848(C_word c,C_word *av) C_noret;
C_noret_decl(f_13858)
static void C_ccall f_13858(C_word c,C_word *av) C_noret;
C_noret_decl(f_13874)
static void C_ccall f_13874(C_word c,C_word *av) C_noret;
C_noret_decl(f_13878)
static void C_ccall f_13878(C_word c,C_word *av) C_noret;
C_noret_decl(f_13936)
static void C_ccall f_13936(C_word c,C_word *av) C_noret;
C_noret_decl(f_13938)
static void C_fcall f_13938(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13963)
static void C_ccall f_13963(C_word c,C_word *av) C_noret;
C_noret_decl(f_13982)
static void C_ccall f_13982(C_word c,C_word *av) C_noret;
C_noret_decl(f_14001)
static void C_ccall f_14001(C_word c,C_word *av) C_noret;
C_noret_decl(f_14016)
static void C_ccall f_14016(C_word c,C_word *av) C_noret;
C_noret_decl(f_14029)
static void C_ccall f_14029(C_word c,C_word *av) C_noret;
C_noret_decl(f_14032)
static void C_ccall f_14032(C_word c,C_word *av) C_noret;
C_noret_decl(f_14068)
static void C_ccall f_14068(C_word c,C_word *av) C_noret;
C_noret_decl(f_14092)
static void C_ccall f_14092(C_word c,C_word *av) C_noret;
C_noret_decl(f_14133)
static void C_ccall f_14133(C_word c,C_word *av) C_noret;
C_noret_decl(f_14135)
static void C_fcall f_14135(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14160)
static void C_ccall f_14160(C_word c,C_word *av) C_noret;
C_noret_decl(f_14189)
static void C_ccall f_14189(C_word c,C_word *av) C_noret;
C_noret_decl(f_14191)
static void C_fcall f_14191(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14216)
static void C_ccall f_14216(C_word c,C_word *av) C_noret;
C_noret_decl(f_14250)
static void C_ccall f_14250(C_word c,C_word *av) C_noret;
C_noret_decl(f_14264)
static void C_fcall f_14264(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14289)
static void C_ccall f_14289(C_word c,C_word *av) C_noret;
C_noret_decl(f_14309)
static void C_ccall f_14309(C_word c,C_word *av) C_noret;
C_noret_decl(f_14311)
static void C_fcall f_14311(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14336)
static void C_ccall f_14336(C_word c,C_word *av) C_noret;
C_noret_decl(f_14389)
static void C_fcall f_14389(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14394)
static void C_ccall f_14394(C_word c,C_word *av) C_noret;
C_noret_decl(f_14403)
static void C_ccall f_14403(C_word c,C_word *av) C_noret;
C_noret_decl(f_14420)
static void C_ccall f_14420(C_word c,C_word *av) C_noret;
C_noret_decl(f_14422)
static void C_ccall f_14422(C_word c,C_word *av) C_noret;
C_noret_decl(f_14437)
static void C_fcall f_14437(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14445)
static void C_ccall f_14445(C_word c,C_word *av) C_noret;
C_noret_decl(f_14472)
static void C_fcall f_14472(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14518)
static void C_ccall f_14518(C_word c,C_word *av) C_noret;
C_noret_decl(f_14567)
static void C_ccall f_14567(C_word c,C_word *av) C_noret;
C_noret_decl(f_14571)
static void C_ccall f_14571(C_word c,C_word *av) C_noret;
C_noret_decl(f_14617)
static void C_ccall f_14617(C_word c,C_word *av) C_noret;
C_noret_decl(f_14621)
static void C_ccall f_14621(C_word c,C_word *av) C_noret;
C_noret_decl(f_14647)
static void C_ccall f_14647(C_word c,C_word *av) C_noret;
C_noret_decl(f_14653)
static void C_fcall f_14653(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14679)
static void C_ccall f_14679(C_word c,C_word *av) C_noret;
C_noret_decl(f_14683)
static void C_ccall f_14683(C_word c,C_word *av) C_noret;
C_noret_decl(f_14710)
static void C_fcall f_14710(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14714)
static void C_ccall f_14714(C_word c,C_word *av) C_noret;
C_noret_decl(f_14717)
static void C_ccall f_14717(C_word c,C_word *av) C_noret;
C_noret_decl(f_14742)
static void C_ccall f_14742(C_word c,C_word *av) C_noret;
C_noret_decl(f_14748)
static void C_ccall f_14748(C_word c,C_word *av) C_noret;
C_noret_decl(f_14754)
static void C_ccall f_14754(C_word c,C_word *av) C_noret;
C_noret_decl(f_14769)
static void C_ccall f_14769(C_word c,C_word *av) C_noret;
C_noret_decl(f_14773)
static void C_ccall f_14773(C_word c,C_word *av) C_noret;
C_noret_decl(f_14778)
static void C_fcall f_14778(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14797)
static void C_ccall f_14797(C_word c,C_word *av) C_noret;
C_noret_decl(f_14806)
static void C_fcall f_14806(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14810)
static void C_ccall f_14810(C_word c,C_word *av) C_noret;
C_noret_decl(f_14814)
static void C_fcall f_14814(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14824)
static void C_ccall f_14824(C_word c,C_word *av) C_noret;
C_noret_decl(f_14828)
static void C_fcall f_14828(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14844)
static void C_ccall f_14844(C_word c,C_word *av) C_noret;
C_noret_decl(f_14871)
static void C_ccall f_14871(C_word c,C_word *av) C_noret;
C_noret_decl(f_14895)
static void C_fcall f_14895(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14904)
static void C_ccall f_14904(C_word c,C_word *av) C_noret;
C_noret_decl(f_14915)
static void C_ccall f_14915(C_word c,C_word *av) C_noret;
C_noret_decl(f_14929)
static void C_fcall f_14929(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14947)
static void C_ccall f_14947(C_word c,C_word *av) C_noret;
C_noret_decl(f_14953)
static void C_ccall f_14953(C_word c,C_word *av) C_noret;
C_noret_decl(f_14960)
static void C_fcall f_14960(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14989)
static void C_ccall f_14989(C_word c,C_word *av) C_noret;
C_noret_decl(f_15068)
static void C_ccall f_15068(C_word c,C_word *av) C_noret;
C_noret_decl(f_15098)
static void C_ccall f_15098(C_word c,C_word *av) C_noret;
C_noret_decl(f_15101)
static void C_ccall f_15101(C_word c,C_word *av) C_noret;
C_noret_decl(f_15119)
static void C_ccall f_15119(C_word c,C_word *av) C_noret;
C_noret_decl(f_15171)
static void C_fcall f_15171(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15222)
static void C_fcall f_15222(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15279)
static void C_fcall f_15279(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15336)
static void C_fcall f_15336(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15345)
static void C_fcall f_15345(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15355)
static void C_ccall f_15355(C_word c,C_word *av) C_noret;
C_noret_decl(f_15362)
static void C_fcall f_15362(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15364)
static void C_fcall f_15364(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15443)
static void C_ccall f_15443(C_word c,C_word *av) C_noret;
C_noret_decl(f_15499)
static void C_fcall f_15499(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15532)
static void C_ccall f_15532(C_word c,C_word *av) C_noret;
C_noret_decl(f_15559)
static void C_fcall f_15559(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15570)
static void C_fcall f_15570(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15592)
static void C_ccall f_15592(C_word c,C_word *av) C_noret;
C_noret_decl(f_15600)
static void C_fcall f_15600(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15602)
static void C_fcall f_15602(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15628)
static void C_ccall f_15628(C_word c,C_word *av) C_noret;
C_noret_decl(f_15687)
static C_word C_fcall f_15687(C_word t0);
C_noret_decl(f_15740)
static C_word C_fcall f_15740(C_word t0);
C_noret_decl(f_15760)
static C_word C_fcall f_15760(C_word t0);
C_noret_decl(f_15819)
static void C_ccall f_15819(C_word c,C_word *av) C_noret;
C_noret_decl(f_15863)
static C_word C_fcall f_15863(C_word t0);
C_noret_decl(f_15902)
static void C_ccall f_15902(C_word c,C_word *av) C_noret;
C_noret_decl(f_15905)
static void C_ccall f_15905(C_word c,C_word *av) C_noret;
C_noret_decl(f_15910)
static void C_ccall f_15910(C_word c,C_word *av) C_noret;
C_noret_decl(f_15914)
static void C_ccall f_15914(C_word c,C_word *av) C_noret;
C_noret_decl(f_15916)
static void C_fcall f_15916(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15961)
static void C_ccall f_15961(C_word c,C_word *av) C_noret;
C_noret_decl(f_15978)
static void C_fcall f_15978(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15988)
static void C_ccall f_15988(C_word c,C_word *av) C_noret;
C_noret_decl(f_16005)
static void C_fcall f_16005(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16015)
static void C_ccall f_16015(C_word c,C_word *av) C_noret;
C_noret_decl(f_16035)
static void C_ccall f_16035(C_word c,C_word *av) C_noret;
C_noret_decl(f_16051)
static void C_fcall f_16051(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16061)
static void C_ccall f_16061(C_word c,C_word *av) C_noret;
C_noret_decl(f_16078)
static void C_fcall f_16078(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16088)
static void C_ccall f_16088(C_word c,C_word *av) C_noret;
C_noret_decl(f_16120)
static void C_ccall f_16120(C_word c,C_word *av) C_noret;
C_noret_decl(f_16162)
static void C_ccall f_16162(C_word c,C_word *av) C_noret;
C_noret_decl(f_16164)
static void C_fcall f_16164(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16189)
static void C_fcall f_16189(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16211)
static void C_fcall f_16211(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16221)
static void C_ccall f_16221(C_word c,C_word *av) C_noret;
C_noret_decl(f_16242)
static void C_fcall f_16242(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16248)
static C_word C_fcall f_16248(C_word t0,C_word t1);
C_noret_decl(f_16280)
static void C_ccall f_16280(C_word c,C_word *av) C_noret;
C_noret_decl(f_16283)
static void C_ccall f_16283(C_word c,C_word *av) C_noret;
C_noret_decl(f_16285)
static void C_ccall f_16285(C_word c,C_word *av) C_noret;
C_noret_decl(f_16293)
static void C_ccall f_16293(C_word c,C_word *av) C_noret;
C_noret_decl(f_16295)
static void C_fcall f_16295(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16303)
static void C_fcall f_16303(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16384)
static void C_fcall f_16384(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16395)
static void C_ccall f_16395(C_word c,C_word *av) C_noret;
C_noret_decl(f_16397)
static void C_fcall f_16397(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16422)
static void C_ccall f_16422(C_word c,C_word *av) C_noret;
C_noret_decl(f_16442)
static void C_ccall f_16442(C_word c,C_word *av) C_noret;
C_noret_decl(f_16468)
static void C_ccall f_16468(C_word c,C_word *av) C_noret;
C_noret_decl(f_16481)
static void C_fcall f_16481(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16491)
static void C_fcall f_16491(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16502)
static void C_ccall f_16502(C_word c,C_word *av) C_noret;
C_noret_decl(f_16504)
static void C_fcall f_16504(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16529)
static void C_ccall f_16529(C_word c,C_word *av) C_noret;
C_noret_decl(f_16545)
static void C_ccall f_16545(C_word c,C_word *av) C_noret;
C_noret_decl(f_16548)
static void C_ccall f_16548(C_word c,C_word *av) C_noret;
C_noret_decl(f_16551)
static void C_ccall f_16551(C_word c,C_word *av) C_noret;
C_noret_decl(f_16558)
static void C_ccall f_16558(C_word c,C_word *av) C_noret;
C_noret_decl(f_16570)
static void C_ccall f_16570(C_word c,C_word *av) C_noret;
C_noret_decl(f_16574)
static void C_ccall f_16574(C_word c,C_word *av) C_noret;
C_noret_decl(f_16581)
static void C_fcall f_16581(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16593)
static void C_fcall f_16593(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16618)
static void C_ccall f_16618(C_word c,C_word *av) C_noret;
C_noret_decl(f_16627)
static void C_fcall f_16627(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16655)
static void C_ccall f_16655(C_word c,C_word *av) C_noret;
C_noret_decl(f_16674)
static void C_ccall f_16674(C_word c,C_word *av) C_noret;
C_noret_decl(f_16683)
static void C_ccall f_16683(C_word c,C_word *av) C_noret;
C_noret_decl(f_16687)
static void C_ccall f_16687(C_word c,C_word *av) C_noret;
C_noret_decl(f_16750)
static void C_ccall f_16750(C_word c,C_word *av) C_noret;
C_noret_decl(f_16754)
static void C_ccall f_16754(C_word c,C_word *av) C_noret;
C_noret_decl(f_16757)
static void C_ccall f_16757(C_word c,C_word *av) C_noret;
C_noret_decl(f_16763)
static void C_ccall f_16763(C_word c,C_word *av) C_noret;
C_noret_decl(f_16768)
static void C_ccall f_16768(C_word c,C_word *av) C_noret;
C_noret_decl(f_16773)
static void C_ccall f_16773(C_word c,C_word *av) C_noret;
C_noret_decl(f_16782)
static void C_ccall f_16782(C_word c,C_word *av) C_noret;
C_noret_decl(f_16788)
static void C_ccall f_16788(C_word c,C_word *av) C_noret;
C_noret_decl(f_16793)
static void C_ccall f_16793(C_word c,C_word *av) C_noret;
C_noret_decl(f_16799)
static void C_ccall f_16799(C_word c,C_word *av) C_noret;
C_noret_decl(f_16803)
static void C_ccall f_16803(C_word c,C_word *av) C_noret;
C_noret_decl(f_16806)
static void C_ccall f_16806(C_word c,C_word *av) C_noret;
C_noret_decl(f_16809)
static void C_ccall f_16809(C_word c,C_word *av) C_noret;
C_noret_decl(f_16812)
static void C_ccall f_16812(C_word c,C_word *av) C_noret;
C_noret_decl(f_16821)
static void C_fcall f_16821(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16828)
static void C_ccall f_16828(C_word c,C_word *av) C_noret;
C_noret_decl(f_16832)
static void C_ccall f_16832(C_word c,C_word *av) C_noret;
C_noret_decl(f_16836)
static void C_ccall f_16836(C_word c,C_word *av) C_noret;
C_noret_decl(f_16840)
static void C_ccall f_16840(C_word c,C_word *av) C_noret;
C_noret_decl(f_16847)
static void C_ccall f_16847(C_word c,C_word *av) C_noret;
C_noret_decl(f_16854)
static void C_ccall f_16854(C_word c,C_word *av) C_noret;
C_noret_decl(f_16860)
static void C_ccall f_16860(C_word c,C_word *av) C_noret;
C_noret_decl(f_16863)
static void C_ccall f_16863(C_word c,C_word *av) C_noret;
C_noret_decl(f_16866)
static void C_ccall f_16866(C_word c,C_word *av) C_noret;
C_noret_decl(f_16869)
static void C_ccall f_16869(C_word c,C_word *av) C_noret;
C_noret_decl(f_16872)
static void C_ccall f_16872(C_word c,C_word *av) C_noret;
C_noret_decl(f_16875)
static void C_ccall f_16875(C_word c,C_word *av) C_noret;
C_noret_decl(f_16878)
static void C_ccall f_16878(C_word c,C_word *av) C_noret;
C_noret_decl(f_16882)
static void C_ccall f_16882(C_word c,C_word *av) C_noret;
C_noret_decl(f_16887)
static void C_fcall f_16887(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16903)
static void C_ccall f_16903(C_word c,C_word *av) C_noret;
C_noret_decl(f_16914)
static void C_fcall f_16914(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16933)
static void C_ccall f_16933(C_word c,C_word *av) C_noret;
C_noret_decl(f_16947)
static void C_ccall f_16947(C_word c,C_word *av) C_noret;
C_noret_decl(f_16961)
static void C_ccall f_16961(C_word c,C_word *av) C_noret;
C_noret_decl(f_16975)
static void C_ccall f_16975(C_word c,C_word *av) C_noret;
C_noret_decl(f_16989)
static void C_ccall f_16989(C_word c,C_word *av) C_noret;
C_noret_decl(f_17019)
static void C_ccall f_17019(C_word c,C_word *av) C_noret;
C_noret_decl(f_17038)
static void C_ccall f_17038(C_word c,C_word *av) C_noret;
C_noret_decl(f_17070)
static void C_ccall f_17070(C_word c,C_word *av) C_noret;
C_noret_decl(f_17076)
static void C_ccall f_17076(C_word c,C_word *av) C_noret;
C_noret_decl(f_17078)
static void C_fcall f_17078(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17088)
static void C_ccall f_17088(C_word c,C_word *av) C_noret;
C_noret_decl(f_17101)
static void C_ccall f_17101(C_word c,C_word *av) C_noret;
C_noret_decl(f_17108)
static void C_ccall f_17108(C_word c,C_word *av) C_noret;
C_noret_decl(f_17114)
static void C_ccall f_17114(C_word c,C_word *av) C_noret;
C_noret_decl(f_17117)
static void C_ccall f_17117(C_word c,C_word *av) C_noret;
C_noret_decl(f_17120)
static void C_ccall f_17120(C_word c,C_word *av) C_noret;
C_noret_decl(f_17123)
static void C_ccall f_17123(C_word c,C_word *av) C_noret;
C_noret_decl(f_17126)
static void C_ccall f_17126(C_word c,C_word *av) C_noret;
C_noret_decl(f_17135)
static void C_ccall f_17135(C_word c,C_word *av) C_noret;
C_noret_decl(f_17149)
static void C_ccall f_17149(C_word c,C_word *av) C_noret;
C_noret_decl(f_17155)
static void C_ccall f_17155(C_word c,C_word *av) C_noret;
C_noret_decl(f_17159)
static void C_ccall f_17159(C_word c,C_word *av) C_noret;
C_noret_decl(f_17162)
static void C_ccall f_17162(C_word c,C_word *av) C_noret;
C_noret_decl(f_17167)
static void C_ccall f_17167(C_word c,C_word *av) C_noret;
C_noret_decl(f_17174)
static void C_fcall f_17174(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17177)
static void C_ccall f_17177(C_word c,C_word *av) C_noret;
C_noret_decl(f_17183)
static void C_ccall f_17183(C_word c,C_word *av) C_noret;
C_noret_decl(f_17186)
static void C_ccall f_17186(C_word c,C_word *av) C_noret;
C_noret_decl(f_17189)
static void C_ccall f_17189(C_word c,C_word *av) C_noret;
C_noret_decl(f_17192)
static void C_ccall f_17192(C_word c,C_word *av) C_noret;
C_noret_decl(f_17195)
static void C_ccall f_17195(C_word c,C_word *av) C_noret;
C_noret_decl(f_17198)
static void C_ccall f_17198(C_word c,C_word *av) C_noret;
C_noret_decl(f_17201)
static void C_ccall f_17201(C_word c,C_word *av) C_noret;
C_noret_decl(f_17208)
static void C_ccall f_17208(C_word c,C_word *av) C_noret;
C_noret_decl(f_17212)
static void C_ccall f_17212(C_word c,C_word *av) C_noret;
C_noret_decl(f_17214)
static void C_fcall f_17214(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17235)
static void C_ccall f_17235(C_word c,C_word *av) C_noret;
C_noret_decl(f_17245)
static void C_ccall f_17245(C_word c,C_word *av) C_noret;
C_noret_decl(f_17253)
static void C_ccall f_17253(C_word c,C_word *av) C_noret;
C_noret_decl(f_17261)
static void C_ccall f_17261(C_word c,C_word *av) C_noret;
C_noret_decl(f_17269)
static void C_ccall f_17269(C_word c,C_word *av) C_noret;
C_noret_decl(f_17297)
static void C_ccall f_17297(C_word c,C_word *av) C_noret;
C_noret_decl(f_17308)
static void C_ccall f_17308(C_word c,C_word *av) C_noret;
C_noret_decl(f_17312)
static void C_ccall f_17312(C_word c,C_word *av) C_noret;
C_noret_decl(f_17317)
static void C_ccall f_17317(C_word c,C_word *av) C_noret;
C_noret_decl(f_17320)
static void C_fcall f_17320(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17325)
static void C_ccall f_17325(C_word c,C_word *av) C_noret;
C_noret_decl(f_17332)
static void C_ccall f_17332(C_word c,C_word *av) C_noret;
C_noret_decl(f_17336)
static void C_ccall f_17336(C_word c,C_word *av) C_noret;
C_noret_decl(f_17338)
static void C_fcall f_17338(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17345)
static void C_ccall f_17345(C_word c,C_word *av) C_noret;
C_noret_decl(f_17365)
static void C_ccall f_17365(C_word c,C_word *av) C_noret;
C_noret_decl(f_17371)
static void C_ccall f_17371(C_word c,C_word *av) C_noret;
C_noret_decl(f_17379)
static void C_ccall f_17379(C_word c,C_word *av) C_noret;
C_noret_decl(f_17383)
static void C_ccall f_17383(C_word c,C_word *av) C_noret;
C_noret_decl(f_17390)
static void C_ccall f_17390(C_word c,C_word *av) C_noret;
C_noret_decl(f_17396)
static void C_ccall f_17396(C_word c,C_word *av) C_noret;
C_noret_decl(f_17398)
static void C_fcall f_17398(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17423)
static void C_ccall f_17423(C_word c,C_word *av) C_noret;
C_noret_decl(f_17440)
static void C_ccall f_17440(C_word c,C_word *av) C_noret;
C_noret_decl(f_17446)
static void C_ccall f_17446(C_word c,C_word *av) C_noret;
C_noret_decl(f_17455)
static void C_ccall f_17455(C_word c,C_word *av) C_noret;
C_noret_decl(f_17494)
static void C_ccall f_17494(C_word c,C_word *av) C_noret;
C_noret_decl(f_17500)
static void C_fcall f_17500(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17503)
static void C_fcall f_17503(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17550)
static void C_ccall f_17550(C_word c,C_word *av) C_noret;
C_noret_decl(f_17560)
static void C_fcall f_17560(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17587)
static void C_ccall f_17587(C_word c,C_word *av) C_noret;
C_noret_decl(f_17591)
static void C_ccall f_17591(C_word c,C_word *av) C_noret;
C_noret_decl(f_17651)
static void C_ccall f_17651(C_word c,C_word *av) C_noret;
C_noret_decl(f_17653)
static void C_ccall f_17653(C_word c,C_word *av) C_noret;
C_noret_decl(f_17662)
static void C_fcall f_17662(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17680)
static void C_ccall f_17680(C_word c,C_word *av) C_noret;
C_noret_decl(f_17684)
static void C_fcall f_17684(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17690)
static C_word C_fcall f_17690(C_word t0,C_word t1);
C_noret_decl(f_17718)
static void C_fcall f_17718(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17749)
static void C_ccall f_17749(C_word c,C_word *av) C_noret;
C_noret_decl(f_17781)
static void C_ccall f_17781(C_word c,C_word *av) C_noret;
C_noret_decl(f_17808)
static void C_ccall f_17808(C_word c,C_word *av) C_noret;
C_noret_decl(f_17811)
static void C_ccall f_17811(C_word c,C_word *av) C_noret;
C_noret_decl(f_17848)
static void C_fcall f_17848(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17906)
static void C_ccall f_17906(C_word c,C_word *av) C_noret;
C_noret_decl(f_17940)
static void C_ccall f_17940(C_word c,C_word *av) C_noret;
C_noret_decl(f_17972)
static void C_ccall f_17972(C_word c,C_word *av) C_noret;
C_noret_decl(f_17977)
static void C_ccall f_17977(C_word c,C_word *av) C_noret;
C_noret_decl(f_17981)
static void C_ccall f_17981(C_word c,C_word *av) C_noret;
C_noret_decl(f_17992)
static void C_ccall f_17992(C_word c,C_word *av) C_noret;
C_noret_decl(f_17994)
static void C_ccall f_17994(C_word c,C_word *av) C_noret;
C_noret_decl(f_18010)
static void C_ccall f_18010(C_word c,C_word *av) C_noret;
C_noret_decl(f_18013)
static void C_ccall f_18013(C_word c,C_word *av) C_noret;
C_noret_decl(f_18029)
static void C_fcall f_18029(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18076)
static void C_ccall f_18076(C_word c,C_word *av) C_noret;
C_noret_decl(f_18078)
static void C_fcall f_18078(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18103)
static void C_ccall f_18103(C_word c,C_word *av) C_noret;
C_noret_decl(f_18122)
static void C_fcall f_18122(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18126)
static C_word C_fcall f_18126(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_18164)
static void C_ccall f_18164(C_word c,C_word *av) C_noret;
C_noret_decl(f_18170)
static void C_ccall f_18170(C_word c,C_word *av) C_noret;
C_noret_decl(f_18175)
static void C_fcall f_18175(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18200)
static void C_ccall f_18200(C_word c,C_word *av) C_noret;
C_noret_decl(f_18276)
static void C_ccall f_18276(C_word c,C_word *av) C_noret;
C_noret_decl(f_18306)
static void C_fcall f_18306(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18318)
static void C_ccall f_18318(C_word c,C_word *av) C_noret;
C_noret_decl(f_18345)
static void C_ccall f_18345(C_word c,C_word *av) C_noret;
C_noret_decl(f_18348)
static void C_ccall f_18348(C_word c,C_word *av) C_noret;
C_noret_decl(f_18352)
static void C_fcall f_18352(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18365)
static void C_fcall f_18365(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18368)
static void C_ccall f_18368(C_word c,C_word *av) C_noret;
C_noret_decl(f_18392)
static void C_ccall f_18392(C_word c,C_word *av) C_noret;
C_noret_decl(f_18400)
static void C_fcall f_18400(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18404)
static void C_ccall f_18404(C_word c,C_word *av) C_noret;
C_noret_decl(f_18415)
static void C_ccall f_18415(C_word c,C_word *av) C_noret;
C_noret_decl(f_18494)
static void C_ccall f_18494(C_word c,C_word *av) C_noret;
C_noret_decl(f_18536)
static void C_fcall f_18536(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18554)
static void C_ccall f_18554(C_word c,C_word *av) C_noret;
C_noret_decl(f_18557)
static void C_ccall f_18557(C_word c,C_word *av) C_noret;
C_noret_decl(f_18561)
static void C_fcall f_18561(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18604)
static void C_ccall f_18604(C_word c,C_word *av) C_noret;
C_noret_decl(f_18610)
static void C_ccall f_18610(C_word c,C_word *av) C_noret;
C_noret_decl(f_18615)
static void C_fcall f_18615(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18640)
static void C_ccall f_18640(C_word c,C_word *av) C_noret;
C_noret_decl(f_18685)
static void C_ccall f_18685(C_word c,C_word *av) C_noret;
C_noret_decl(f_18694)
static void C_ccall f_18694(C_word c,C_word *av) C_noret;
C_noret_decl(f_18698)
static void C_fcall f_18698(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18708)
static void C_ccall f_18708(C_word c,C_word *av) C_noret;
C_noret_decl(f_18740)
static void C_ccall f_18740(C_word c,C_word *av) C_noret;
C_noret_decl(f_18746)
static void C_ccall f_18746(C_word c,C_word *av) C_noret;
C_noret_decl(f_18748)
static void C_fcall f_18748(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18773)
static void C_ccall f_18773(C_word c,C_word *av) C_noret;
C_noret_decl(f_18905)
static void C_ccall f_18905(C_word c,C_word *av) C_noret;
C_noret_decl(f_18909)
static void C_fcall f_18909(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18913)
static void C_fcall f_18913(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18916)
static void C_ccall f_18916(C_word c,C_word *av) C_noret;
C_noret_decl(f_18948)
static void C_fcall f_18948(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18952)
static void C_ccall f_18952(C_word c,C_word *av) C_noret;
C_noret_decl(f_18965)
static void C_ccall f_18965(C_word c,C_word *av) C_noret;
C_noret_decl(f_18969)
static void C_ccall f_18969(C_word c,C_word *av) C_noret;
C_noret_decl(f_18972)
static void C_ccall f_18972(C_word c,C_word *av) C_noret;
C_noret_decl(f_18978)
static void C_ccall f_18978(C_word c,C_word *av) C_noret;
C_noret_decl(f_18980)
static void C_fcall f_18980(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19005)
static void C_ccall f_19005(C_word c,C_word *av) C_noret;
C_noret_decl(f_19020)
static void C_ccall f_19020(C_word c,C_word *av) C_noret;
C_noret_decl(f_19029)
static void C_ccall f_19029(C_word c,C_word *av) C_noret;
C_noret_decl(f_19037)
static void C_ccall f_19037(C_word c,C_word *av) C_noret;
C_noret_decl(f_19039)
static void C_ccall f_19039(C_word c,C_word *av) C_noret;
C_noret_decl(f_19054)
static void C_ccall f_19054(C_word c,C_word *av) C_noret;
C_noret_decl(f_19057)
static void C_fcall f_19057(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19069)
static void C_fcall f_19069(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19080)
static void C_fcall f_19080(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19086)
static void C_ccall f_19086(C_word c,C_word *av) C_noret;
C_noret_decl(f_19092)
static void C_ccall f_19092(C_word c,C_word *av) C_noret;
C_noret_decl(f_19108)
static void C_ccall f_19108(C_word c,C_word *av) C_noret;
C_noret_decl(f_19111)
static void C_ccall f_19111(C_word c,C_word *av) C_noret;
C_noret_decl(f_19121)
static void C_ccall f_19121(C_word c,C_word *av) C_noret;
C_noret_decl(f_19147)
static void C_fcall f_19147(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19153)
static void C_ccall f_19153(C_word c,C_word *av) C_noret;
C_noret_decl(f_19159)
static void C_ccall f_19159(C_word c,C_word *av) C_noret;
C_noret_decl(f_19175)
static void C_ccall f_19175(C_word c,C_word *av) C_noret;
C_noret_decl(f_19181)
static void C_fcall f_19181(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19206)
static void C_ccall f_19206(C_word c,C_word *av) C_noret;
C_noret_decl(f_19244)
static void C_ccall f_19244(C_word c,C_word *av) C_noret;
C_noret_decl(f_19246)
static void C_fcall f_19246(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19271)
static void C_ccall f_19271(C_word c,C_word *av) C_noret;
C_noret_decl(f_19297)
static void C_ccall f_19297(C_word c,C_word *av) C_noret;
C_noret_decl(f_19299)
static void C_fcall f_19299(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19324)
static void C_ccall f_19324(C_word c,C_word *av) C_noret;
C_noret_decl(f_19333)
static void C_fcall f_19333(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19361)
static void C_fcall f_19361(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19375)
static void C_ccall f_19375(C_word c,C_word *av) C_noret;
C_noret_decl(f_19409)
static void C_ccall f_19409(C_word c,C_word *av) C_noret;
C_noret_decl(f_19413)
static void C_ccall f_19413(C_word c,C_word *av) C_noret;
C_noret_decl(f_19447)
static void C_fcall f_19447(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19451)
static void C_ccall f_19451(C_word c,C_word *av) C_noret;
C_noret_decl(f_19459)
static void C_fcall f_19459(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19479)
static void C_ccall f_19479(C_word c,C_word *av) C_noret;
C_noret_decl(f_19495)
static void C_ccall f_19495(C_word c,C_word *av) C_noret;
C_noret_decl(f_19497)
static void C_fcall f_19497(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_19501)
static void C_ccall f_19501(C_word c,C_word *av) C_noret;
C_noret_decl(f_19564)
static void C_ccall f_19564(C_word c,C_word *av) C_noret;
C_noret_decl(f_19576)
static void C_ccall f_19576(C_word c,C_word *av) C_noret;
C_noret_decl(f_19580)
static void C_ccall f_19580(C_word c,C_word *av) C_noret;
C_noret_decl(f_19596)
static void C_ccall f_19596(C_word c,C_word *av) C_noret;
C_noret_decl(f_19600)
static void C_ccall f_19600(C_word c,C_word *av) C_noret;
C_noret_decl(f_19621)
static void C_ccall f_19621(C_word c,C_word *av) C_noret;
C_noret_decl(f_19644)
static void C_ccall f_19644(C_word c,C_word *av) C_noret;
C_noret_decl(f_19651)
static void C_ccall f_19651(C_word c,C_word *av) C_noret;
C_noret_decl(f_19654)
static void C_ccall f_19654(C_word c,C_word *av) C_noret;
C_noret_decl(f_19657)
static void C_ccall f_19657(C_word c,C_word *av) C_noret;
C_noret_decl(f_19710)
static void C_ccall f_19710(C_word c,C_word *av) C_noret;
C_noret_decl(f_19720)
static void C_ccall f_19720(C_word c,C_word *av) C_noret;
C_noret_decl(f_19747)
static void C_fcall f_19747(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_19765)
static void C_ccall f_19765(C_word c,C_word *av) C_noret;
C_noret_decl(f_19842)
static void C_fcall f_19842(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19844)
static void C_ccall f_19844(C_word c,C_word *av) C_noret;
C_noret_decl(f_19848)
static void C_ccall f_19848(C_word c,C_word *av) C_noret;
C_noret_decl(f_19851)
static void C_ccall f_19851(C_word c,C_word *av) C_noret;
C_noret_decl(f_19897)
static void C_ccall f_19897(C_word c,C_word *av) C_noret;
C_noret_decl(f_19900)
static void C_ccall f_19900(C_word c,C_word *av) C_noret;
C_noret_decl(f_19906)
static void C_fcall f_19906(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19910)
static void C_fcall f_19910(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19914)
static void C_ccall f_19914(C_word c,C_word *av) C_noret;
C_noret_decl(f_19921)
static void C_ccall f_19921(C_word c,C_word *av) C_noret;
C_noret_decl(f_19925)
static void C_ccall f_19925(C_word c,C_word *av) C_noret;
C_noret_decl(f_19937)
static void C_ccall f_19937(C_word c,C_word *av) C_noret;
C_noret_decl(f_19940)
static void C_ccall f_19940(C_word c,C_word *av) C_noret;
C_noret_decl(f_19943)
static void C_ccall f_19943(C_word c,C_word *av) C_noret;
C_noret_decl(f_19946)
static void C_ccall f_19946(C_word c,C_word *av) C_noret;
C_noret_decl(f_19949)
static void C_ccall f_19949(C_word c,C_word *av) C_noret;
C_noret_decl(f_19952)
static void C_ccall f_19952(C_word c,C_word *av) C_noret;
C_noret_decl(f_19955)
static void C_ccall f_19955(C_word c,C_word *av) C_noret;
C_noret_decl(f_19957)
static void C_ccall f_19957(C_word c,C_word *av) C_noret;
C_noret_decl(f_19966)
static void C_fcall f_19966(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19976)
static void C_fcall f_19976(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19998)
static void C_ccall f_19998(C_word c,C_word *av) C_noret;
C_noret_decl(f_20008)
static void C_ccall f_20008(C_word c,C_word *av) C_noret;
C_noret_decl(f_20011)
static void C_ccall f_20011(C_word c,C_word *av) C_noret;
C_noret_decl(f_20013)
static void C_fcall f_20013(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20057)
static void C_ccall f_20057(C_word c,C_word *av) C_noret;
C_noret_decl(f_20074)
static void C_ccall f_20074(C_word c,C_word *av) C_noret;
C_noret_decl(f_20080)
static void C_ccall f_20080(C_word c,C_word *av) C_noret;
C_noret_decl(f_20087)
static void C_ccall f_20087(C_word c,C_word *av) C_noret;
C_noret_decl(f_20111)
static void C_ccall f_20111(C_word c,C_word *av) C_noret;
C_noret_decl(f_20120)
static void C_ccall f_20120(C_word c,C_word *av) C_noret;
C_noret_decl(f_20123)
static void C_ccall f_20123(C_word c,C_word *av) C_noret;
C_noret_decl(f_20127)
static void C_ccall f_20127(C_word c,C_word *av) C_noret;
C_noret_decl(f_20131)
static void C_ccall f_20131(C_word c,C_word *av) C_noret;
C_noret_decl(f_20143)
static void C_fcall f_20143(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20173)
static void C_ccall f_20173(C_word c,C_word *av) C_noret;
C_noret_decl(f_20187)
static void C_ccall f_20187(C_word c,C_word *av) C_noret;
C_noret_decl(f_20191)
static void C_ccall f_20191(C_word c,C_word *av) C_noret;
C_noret_decl(f_20195)
static void C_ccall f_20195(C_word c,C_word *av) C_noret;
C_noret_decl(f_20218)
static void C_ccall f_20218(C_word c,C_word *av) C_noret;
C_noret_decl(f_20220)
static void C_fcall f_20220(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_20223)
static void C_fcall f_20223(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20232)
static void C_fcall f_20232(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20283)
static void C_ccall f_20283(C_word c,C_word *av) C_noret;
C_noret_decl(f_20287)
static void C_fcall f_20287(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20291)
static void C_ccall f_20291(C_word c,C_word *av) C_noret;
C_noret_decl(f_20295)
static void C_fcall f_20295(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20303)
static void C_ccall f_20303(C_word c,C_word *av) C_noret;
C_noret_decl(f_20305)
static void C_ccall f_20305(C_word c,C_word *av) C_noret;
C_noret_decl(f_20329)
static void C_fcall f_20329(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20354)
static void C_ccall f_20354(C_word c,C_word *av) C_noret;
C_noret_decl(f_20365)
static void C_ccall f_20365(C_word c,C_word *av) C_noret;
C_noret_decl(f_20375)
static void C_ccall f_20375(C_word c,C_word *av) C_noret;
C_noret_decl(f_20377)
static void C_fcall f_20377(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20402)
static void C_ccall f_20402(C_word c,C_word *av) C_noret;
C_noret_decl(f_20419)
static void C_ccall f_20419(C_word c,C_word *av) C_noret;
C_noret_decl(f_20421)
static void C_fcall f_20421(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20435)
static void C_fcall f_20435(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20451)
static void C_ccall f_20451(C_word c,C_word *av) C_noret;
C_noret_decl(f_20470)
static void C_ccall f_20470(C_word c,C_word *av) C_noret;
C_noret_decl(f_20476)
static void C_ccall f_20476(C_word c,C_word *av) C_noret;
C_noret_decl(f_20478)
static void C_fcall f_20478(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20488)
static void C_ccall f_20488(C_word c,C_word *av) C_noret;
C_noret_decl(f_20508)
static void C_ccall f_20508(C_word c,C_word *av) C_noret;
C_noret_decl(f_20511)
static void C_ccall f_20511(C_word c,C_word *av) C_noret;
C_noret_decl(f_20522)
static void C_ccall f_20522(C_word c,C_word *av) C_noret;
C_noret_decl(f_20525)
static void C_ccall f_20525(C_word c,C_word *av) C_noret;
C_noret_decl(f_20545)
static void C_fcall f_20545(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20552)
static void C_ccall f_20552(C_word c,C_word *av) C_noret;
C_noret_decl(f_20555)
static void C_ccall f_20555(C_word c,C_word *av) C_noret;
C_noret_decl(f_20581)
static void C_fcall f_20581(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20594)
static void C_ccall f_20594(C_word c,C_word *av) C_noret;
C_noret_decl(f_20596)
static void C_fcall f_20596(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20621)
static void C_ccall f_20621(C_word c,C_word *av) C_noret;
C_noret_decl(f_20636)
static void C_fcall f_20636(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20643)
static void C_ccall f_20643(C_word c,C_word *av) C_noret;
C_noret_decl(f_20646)
static void C_ccall f_20646(C_word c,C_word *av) C_noret;
C_noret_decl(f_20653)
static void C_ccall f_20653(C_word c,C_word *av) C_noret;
C_noret_decl(f_20662)
static void C_fcall f_20662(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20668)
static void C_fcall f_20668(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20681)
static void C_ccall f_20681(C_word c,C_word *av) C_noret;
C_noret_decl(f_20697)
static void C_ccall f_20697(C_word c,C_word *av) C_noret;
C_noret_decl(f_20703)
static void C_ccall f_20703(C_word c,C_word *av) C_noret;
C_noret_decl(f_20706)
static void C_ccall f_20706(C_word c,C_word *av) C_noret;
C_noret_decl(f_20709)
static void C_ccall f_20709(C_word c,C_word *av) C_noret;
C_noret_decl(f_20712)
static void C_ccall f_20712(C_word c,C_word *av) C_noret;
C_noret_decl(f_20720)
static void C_ccall f_20720(C_word c,C_word *av) C_noret;
C_noret_decl(f_20733)
static void C_ccall f_20733(C_word c,C_word *av) C_noret;
C_noret_decl(f_20735)
static void C_fcall f_20735(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20760)
static void C_ccall f_20760(C_word c,C_word *av) C_noret;
C_noret_decl(f_20776)
static void C_ccall f_20776(C_word c,C_word *av) C_noret;
C_noret_decl(f_20788)
static void C_fcall f_20788(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20797)
static void C_ccall f_20797(C_word c,C_word *av) C_noret;
C_noret_decl(f_20806)
static void C_fcall f_20806(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20865)
static void C_ccall f_20865(C_word c,C_word *av) C_noret;
C_noret_decl(f_20867)
static void C_fcall f_20867(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20892)
static void C_ccall f_20892(C_word c,C_word *av) C_noret;
C_noret_decl(f_20917)
static void C_ccall f_20917(C_word c,C_word *av) C_noret;
C_noret_decl(f_20923)
static void C_ccall f_20923(C_word c,C_word *av) C_noret;
C_noret_decl(f_20926)
static void C_ccall f_20926(C_word c,C_word *av) C_noret;
C_noret_decl(f_20929)
static void C_ccall f_20929(C_word c,C_word *av) C_noret;
C_noret_decl(f_20960)
static void C_ccall f_20960(C_word c,C_word *av) C_noret;
C_noret_decl(f_20963)
static void C_fcall f_20963(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20966)
static void C_ccall f_20966(C_word c,C_word *av) C_noret;
C_noret_decl(f_20983)
static void C_fcall f_20983(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21008)
static void C_ccall f_21008(C_word c,C_word *av) C_noret;
C_noret_decl(f_21017)
static void C_fcall f_21017(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21042)
static void C_ccall f_21042(C_word c,C_word *av) C_noret;
C_noret_decl(f_21079)
static void C_ccall f_21079(C_word c,C_word *av) C_noret;
C_noret_decl(f_21086)
static void C_ccall f_21086(C_word c,C_word *av) C_noret;
C_noret_decl(f_21090)
static void C_ccall f_21090(C_word c,C_word *av) C_noret;
C_noret_decl(f_21092)
static void C_ccall f_21092(C_word c,C_word *av) C_noret;
C_noret_decl(f_21100)
static void C_ccall f_21100(C_word c,C_word *av) C_noret;
C_noret_decl(f_21102)
static void C_fcall f_21102(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21106)
static void C_ccall f_21106(C_word c,C_word *av) C_noret;
C_noret_decl(f_21111)
static void C_fcall f_21111(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21137)
static void C_ccall f_21137(C_word c,C_word *av) C_noret;
C_noret_decl(f_21142)
static void C_fcall f_21142(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21154)
static void C_fcall f_21154(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21179)
static void C_ccall f_21179(C_word c,C_word *av) C_noret;
C_noret_decl(f_21196)
static void C_ccall f_21196(C_word c,C_word *av) C_noret;
C_noret_decl(f_21205)
static void C_ccall f_21205(C_word c,C_word *av) C_noret;
C_noret_decl(f_21207)
static void C_fcall f_21207(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21215)
static void C_ccall f_21215(C_word c,C_word *av) C_noret;
C_noret_decl(f_21219)
static void C_ccall f_21219(C_word c,C_word *av) C_noret;
C_noret_decl(f_21221)
static void C_ccall f_21221(C_word c,C_word *av) C_noret;
C_noret_decl(f_21229)
static void C_ccall f_21229(C_word c,C_word *av) C_noret;
C_noret_decl(f_21231)
static void C_fcall f_21231(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21235)
static void C_ccall f_21235(C_word c,C_word *av) C_noret;
C_noret_decl(f_21244)
static void C_ccall f_21244(C_word c,C_word *av) C_noret;
C_noret_decl(f_21250)
static void C_ccall f_21250(C_word c,C_word *av) C_noret;
C_noret_decl(f_21253)
static void C_ccall f_21253(C_word c,C_word *av) C_noret;
C_noret_decl(f_21256)
static void C_ccall f_21256(C_word c,C_word *av) C_noret;
C_noret_decl(f_21261)
static void C_fcall f_21261(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21274)
static void C_ccall f_21274(C_word c,C_word *av) C_noret;
C_noret_decl(f_21280)
static void C_ccall f_21280(C_word c,C_word *av) C_noret;
C_noret_decl(f_21283)
static void C_ccall f_21283(C_word c,C_word *av) C_noret;
C_noret_decl(f_21286)
static void C_ccall f_21286(C_word c,C_word *av) C_noret;
C_noret_decl(f_21293)
static void C_ccall f_21293(C_word c,C_word *av) C_noret;
C_noret_decl(f_21311)
static void C_fcall f_21311(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21325)
static void C_fcall f_21325(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21329)
static void C_ccall f_21329(C_word c,C_word *av) C_noret;
C_noret_decl(f_21333)
static void C_ccall f_21333(C_word c,C_word *av) C_noret;
C_noret_decl(f_21339)
static void C_ccall f_21339(C_word c,C_word *av) C_noret;
C_noret_decl(f_21347)
static void C_ccall f_21347(C_word c,C_word *av) C_noret;
C_noret_decl(f_21353)
static void C_ccall f_21353(C_word c,C_word *av) C_noret;
C_noret_decl(f_21361)
static void C_ccall f_21361(C_word c,C_word *av) C_noret;
C_noret_decl(f_21365)
static void C_ccall f_21365(C_word c,C_word *av) C_noret;
C_noret_decl(f_21371)
static void C_ccall f_21371(C_word c,C_word *av) C_noret;
C_noret_decl(f_21374)
static void C_ccall f_21374(C_word c,C_word *av) C_noret;
C_noret_decl(f_21377)
static void C_ccall f_21377(C_word c,C_word *av) C_noret;
C_noret_decl(f_21380)
static void C_ccall f_21380(C_word c,C_word *av) C_noret;
C_noret_decl(f_21383)
static void C_ccall f_21383(C_word c,C_word *av) C_noret;
C_noret_decl(f_21389)
static void C_ccall f_21389(C_word c,C_word *av) C_noret;
C_noret_decl(f_21392)
static void C_ccall f_21392(C_word c,C_word *av) C_noret;
C_noret_decl(f_21395)
static void C_ccall f_21395(C_word c,C_word *av) C_noret;
C_noret_decl(f_21398)
static void C_ccall f_21398(C_word c,C_word *av) C_noret;
C_noret_decl(f_21401)
static void C_ccall f_21401(C_word c,C_word *av) C_noret;
C_noret_decl(f_21417)
static void C_ccall f_21417(C_word c,C_word *av) C_noret;
C_noret_decl(f_21431)
static void C_fcall f_21431(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21435)
static void C_ccall f_21435(C_word c,C_word *av) C_noret;
C_noret_decl(f_21438)
static void C_ccall f_21438(C_word c,C_word *av) C_noret;
C_noret_decl(f_21451)
static void C_ccall f_21451(C_word c,C_word *av) C_noret;
C_noret_decl(f_21455)
static void C_ccall f_21455(C_word c,C_word *av) C_noret;
C_noret_decl(f_21470)
static void C_ccall f_21470(C_word c,C_word *av) C_noret;
C_noret_decl(f_21482)
static void C_fcall f_21482(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21488)
static void C_ccall f_21488(C_word c,C_word *av) C_noret;
C_noret_decl(f_21494)
static void C_ccall f_21494(C_word c,C_word *av) C_noret;
C_noret_decl(f_21501)
static void C_ccall f_21501(C_word c,C_word *av) C_noret;
C_noret_decl(f_21507)
static void C_ccall f_21507(C_word c,C_word *av) C_noret;
C_noret_decl(f_21510)
static void C_ccall f_21510(C_word c,C_word *av) C_noret;
C_noret_decl(f_21513)
static void C_ccall f_21513(C_word c,C_word *av) C_noret;
C_noret_decl(f_21516)
static void C_ccall f_21516(C_word c,C_word *av) C_noret;
C_noret_decl(f_21519)
static void C_ccall f_21519(C_word c,C_word *av) C_noret;
C_noret_decl(f_21525)
static void C_ccall f_21525(C_word c,C_word *av) C_noret;
C_noret_decl(f_21531)
static void C_ccall f_21531(C_word c,C_word *av) C_noret;
C_noret_decl(f_21534)
static void C_ccall f_21534(C_word c,C_word *av) C_noret;
C_noret_decl(f_21537)
static void C_ccall f_21537(C_word c,C_word *av) C_noret;
C_noret_decl(f_21542)
static void C_fcall f_21542(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21545)
static void C_fcall f_21545(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21553)
static void C_ccall f_21553(C_word c,C_word *av) C_noret;
C_noret_decl(f_21557)
static void C_ccall f_21557(C_word c,C_word *av) C_noret;
C_noret_decl(f_21559)
static void C_fcall f_21559(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21563)
static void C_ccall f_21563(C_word c,C_word *av) C_noret;
C_noret_decl(f_21583)
static void C_ccall f_21583(C_word c,C_word *av) C_noret;
C_noret_decl(f_21590)
static void C_ccall f_21590(C_word c,C_word *av) C_noret;
C_noret_decl(f_21611)
static void C_ccall f_21611(C_word c,C_word *av) C_noret;
C_noret_decl(f_21621)
static void C_ccall f_21621(C_word c,C_word *av) C_noret;
C_noret_decl(f_21625)
static void C_ccall f_21625(C_word c,C_word *av) C_noret;
C_noret_decl(f_21629)
static void C_ccall f_21629(C_word c,C_word *av) C_noret;
C_noret_decl(f_21633)
static void C_ccall f_21633(C_word c,C_word *av) C_noret;
C_noret_decl(f_21637)
static void C_ccall f_21637(C_word c,C_word *av) C_noret;
C_noret_decl(f_21641)
static void C_ccall f_21641(C_word c,C_word *av) C_noret;
C_noret_decl(f_21645)
static void C_ccall f_21645(C_word c,C_word *av) C_noret;
C_noret_decl(f_21649)
static void C_ccall f_21649(C_word c,C_word *av) C_noret;
C_noret_decl(f_21656)
static void C_ccall f_21656(C_word c,C_word *av) C_noret;
C_noret_decl(f_21672)
static void C_ccall f_21672(C_word c,C_word *av) C_noret;
C_noret_decl(f_21675)
static void C_fcall f_21675(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21681)
static void C_ccall f_21681(C_word c,C_word *av) C_noret;
C_noret_decl(f_21697)
static void C_ccall f_21697(C_word c,C_word *av) C_noret;
C_noret_decl(f_21703)
static void C_ccall f_21703(C_word c,C_word *av) C_noret;
C_noret_decl(f_21710)
static void C_ccall f_21710(C_word c,C_word *av) C_noret;
C_noret_decl(f_21714)
static void C_ccall f_21714(C_word c,C_word *av) C_noret;
C_noret_decl(f_21718)
static void C_ccall f_21718(C_word c,C_word *av) C_noret;
C_noret_decl(f_21722)
static void C_ccall f_21722(C_word c,C_word *av) C_noret;
C_noret_decl(f_21725)
static void C_ccall f_21725(C_word c,C_word *av) C_noret;
C_noret_decl(f_21729)
static void C_ccall f_21729(C_word c,C_word *av) C_noret;
C_noret_decl(f_21733)
static void C_ccall f_21733(C_word c,C_word *av) C_noret;
C_noret_decl(f_21739)
static void C_ccall f_21739(C_word c,C_word *av) C_noret;
C_noret_decl(f_21742)
static void C_ccall f_21742(C_word c,C_word *av) C_noret;
C_noret_decl(f_21756)
static void C_fcall f_21756(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21774)
static void C_ccall f_21774(C_word c,C_word *av) C_noret;
C_noret_decl(f_21787)
static void C_ccall f_21787(C_word c,C_word *av) C_noret;
C_noret_decl(f_21791)
static void C_ccall f_21791(C_word c,C_word *av) C_noret;
C_noret_decl(f_21795)
static void C_ccall f_21795(C_word c,C_word *av) C_noret;
C_noret_decl(f_21801)
static void C_ccall f_21801(C_word c,C_word *av) C_noret;
C_noret_decl(f_21812)
static void C_ccall f_21812(C_word c,C_word *av) C_noret;
C_noret_decl(f_21816)
static void C_ccall f_21816(C_word c,C_word *av) C_noret;
C_noret_decl(f_21820)
static void C_ccall f_21820(C_word c,C_word *av) C_noret;
C_noret_decl(f_21824)
static void C_ccall f_21824(C_word c,C_word *av) C_noret;
C_noret_decl(f_21828)
static void C_ccall f_21828(C_word c,C_word *av) C_noret;
C_noret_decl(f_21839)
static void C_ccall f_21839(C_word c,C_word *av) C_noret;
C_noret_decl(f_21843)
static void C_ccall f_21843(C_word c,C_word *av) C_noret;
C_noret_decl(f_21847)
static void C_ccall f_21847(C_word c,C_word *av) C_noret;
C_noret_decl(f_21851)
static void C_ccall f_21851(C_word c,C_word *av) C_noret;
C_noret_decl(f_21855)
static void C_ccall f_21855(C_word c,C_word *av) C_noret;
C_noret_decl(f_21862)
static void C_ccall f_21862(C_word c,C_word *av) C_noret;
C_noret_decl(f_21873)
static void C_ccall f_21873(C_word c,C_word *av) C_noret;
C_noret_decl(f_21877)
static void C_ccall f_21877(C_word c,C_word *av) C_noret;
C_noret_decl(f_21881)
static void C_ccall f_21881(C_word c,C_word *av) C_noret;
C_noret_decl(f_21897)
static void C_ccall f_21897(C_word c,C_word *av) C_noret;
C_noret_decl(f_21901)
static void C_ccall f_21901(C_word c,C_word *av) C_noret;
C_noret_decl(f_21906)
static void C_fcall f_21906(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_21911)
static void C_ccall f_21911(C_word c,C_word *av) C_noret;
C_noret_decl(f_21922)
static void C_ccall f_21922(C_word c,C_word *av) C_noret;
C_noret_decl(f_21926)
static void C_ccall f_21926(C_word c,C_word *av) C_noret;
C_noret_decl(f_21930)
static void C_ccall f_21930(C_word c,C_word *av) C_noret;
C_noret_decl(f_21934)
static void C_ccall f_21934(C_word c,C_word *av) C_noret;
C_noret_decl(f_21938)
static void C_ccall f_21938(C_word c,C_word *av) C_noret;
C_noret_decl(f_21942)
static void C_ccall f_21942(C_word c,C_word *av) C_noret;
C_noret_decl(f_21946)
static void C_ccall f_21946(C_word c,C_word *av) C_noret;
C_noret_decl(f_21950)
static void C_ccall f_21950(C_word c,C_word *av) C_noret;
C_noret_decl(f_21952)
static void C_ccall f_21952(C_word c,C_word *av) C_noret;
C_noret_decl(f_21957)
static void C_ccall f_21957(C_word c,C_word *av) C_noret;
C_noret_decl(f_21959)
static void C_ccall f_21959(C_word c,C_word *av) C_noret;
C_noret_decl(f_21971)
static void C_ccall f_21971(C_word c,C_word *av) C_noret;
C_noret_decl(f_21975)
static void C_ccall f_21975(C_word c,C_word *av) C_noret;
C_noret_decl(f_21979)
static void C_ccall f_21979(C_word c,C_word *av) C_noret;
C_noret_decl(f_21983)
static void C_ccall f_21983(C_word c,C_word *av) C_noret;
C_noret_decl(f_21987)
static void C_ccall f_21987(C_word c,C_word *av) C_noret;
C_noret_decl(f_21993)
static void C_ccall f_21993(C_word c,C_word *av) C_noret;
C_noret_decl(f_21996)
static void C_ccall f_21996(C_word c,C_word *av) C_noret;
C_noret_decl(f_21999)
static void C_ccall f_21999(C_word c,C_word *av) C_noret;
C_noret_decl(f_22006)
static void C_ccall f_22006(C_word c,C_word *av) C_noret;
C_noret_decl(f_22022)
static void C_ccall f_22022(C_word c,C_word *av) C_noret;
C_noret_decl(f_22028)
static void C_ccall f_22028(C_word c,C_word *av) C_noret;
C_noret_decl(f_22031)
static void C_ccall f_22031(C_word c,C_word *av) C_noret;
C_noret_decl(f_22034)
static void C_ccall f_22034(C_word c,C_word *av) C_noret;
C_noret_decl(f_22037)
static void C_ccall f_22037(C_word c,C_word *av) C_noret;
C_noret_decl(f_22045)
static void C_fcall f_22045(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_22050)
static void C_ccall f_22050(C_word c,C_word *av) C_noret;
C_noret_decl(f_22057)
static void C_ccall f_22057(C_word c,C_word *av) C_noret;
C_noret_decl(f_22065)
static void C_ccall f_22065(C_word c,C_word *av) C_noret;
C_noret_decl(f_22069)
static void C_ccall f_22069(C_word c,C_word *av) C_noret;
C_noret_decl(f_22073)
static void C_ccall f_22073(C_word c,C_word *av) C_noret;
C_noret_decl(f_22077)
static void C_ccall f_22077(C_word c,C_word *av) C_noret;
C_noret_decl(f_22081)
static void C_ccall f_22081(C_word c,C_word *av) C_noret;
C_noret_decl(f_22087)
static void C_ccall f_22087(C_word c,C_word *av) C_noret;
C_noret_decl(f_22090)
static void C_ccall f_22090(C_word c,C_word *av) C_noret;
C_noret_decl(f_22093)
static void C_ccall f_22093(C_word c,C_word *av) C_noret;
C_noret_decl(f_22099)
static void C_ccall f_22099(C_word c,C_word *av) C_noret;
C_noret_decl(f_22105)
static void C_ccall f_22105(C_word c,C_word *av) C_noret;
C_noret_decl(f_22108)
static void C_ccall f_22108(C_word c,C_word *av) C_noret;
C_noret_decl(f_22111)
static void C_ccall f_22111(C_word c,C_word *av) C_noret;
C_noret_decl(f_22114)
static void C_ccall f_22114(C_word c,C_word *av) C_noret;
C_noret_decl(f_22117)
static void C_ccall f_22117(C_word c,C_word *av) C_noret;
C_noret_decl(f_22120)
static void C_ccall f_22120(C_word c,C_word *av) C_noret;
C_noret_decl(f_22123)
static void C_ccall f_22123(C_word c,C_word *av) C_noret;
C_noret_decl(f_22130)
static void C_ccall f_22130(C_word c,C_word *av) C_noret;
C_noret_decl(f_22136)
static void C_ccall f_22136(C_word c,C_word *av) C_noret;
C_noret_decl(f_22139)
static void C_ccall f_22139(C_word c,C_word *av) C_noret;
C_noret_decl(f_22142)
static void C_ccall f_22142(C_word c,C_word *av) C_noret;
C_noret_decl(f_22148)
static void C_ccall f_22148(C_word c,C_word *av) C_noret;
C_noret_decl(f_22154)
static void C_ccall f_22154(C_word c,C_word *av) C_noret;
C_noret_decl(f_22157)
static void C_ccall f_22157(C_word c,C_word *av) C_noret;
C_noret_decl(f_22160)
static void C_ccall f_22160(C_word c,C_word *av) C_noret;
C_noret_decl(f_22167)
static void C_ccall f_22167(C_word c,C_word *av) C_noret;
C_noret_decl(f_22177)
static void C_ccall f_22177(C_word c,C_word *av) C_noret;
C_noret_decl(f_22180)
static void C_fcall f_22180(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22192)
static void C_ccall f_22192(C_word c,C_word *av) C_noret;
C_noret_decl(f_22196)
static void C_ccall f_22196(C_word c,C_word *av) C_noret;
C_noret_decl(f_22200)
static void C_ccall f_22200(C_word c,C_word *av) C_noret;
C_noret_decl(f_22216)
static void C_ccall f_22216(C_word c,C_word *av) C_noret;
C_noret_decl(f_22222)
static void C_ccall f_22222(C_word c,C_word *av) C_noret;
C_noret_decl(f_22225)
static void C_ccall f_22225(C_word c,C_word *av) C_noret;
C_noret_decl(f_22228)
static void C_ccall f_22228(C_word c,C_word *av) C_noret;
C_noret_decl(f_22231)
static void C_ccall f_22231(C_word c,C_word *av) C_noret;
C_noret_decl(f_22239)
static void C_ccall f_22239(C_word c,C_word *av) C_noret;
C_noret_decl(f_22242)
static void C_fcall f_22242(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22254)
static void C_ccall f_22254(C_word c,C_word *av) C_noret;
C_noret_decl(f_22258)
static void C_ccall f_22258(C_word c,C_word *av) C_noret;
C_noret_decl(f_22262)
static void C_ccall f_22262(C_word c,C_word *av) C_noret;
C_noret_decl(f_22266)
static void C_ccall f_22266(C_word c,C_word *av) C_noret;
C_noret_decl(f_22279)
static void C_ccall f_22279(C_word c,C_word *av) C_noret;
C_noret_decl(f_22285)
static void C_ccall f_22285(C_word c,C_word *av) C_noret;
C_noret_decl(f_22288)
static void C_ccall f_22288(C_word c,C_word *av) C_noret;
C_noret_decl(f_22291)
static void C_ccall f_22291(C_word c,C_word *av) C_noret;
C_noret_decl(f_22294)
static void C_ccall f_22294(C_word c,C_word *av) C_noret;
C_noret_decl(f_22298)
static void C_ccall f_22298(C_word c,C_word *av) C_noret;
C_noret_decl(f_22305)
static void C_ccall f_22305(C_word c,C_word *av) C_noret;
C_noret_decl(f_22311)
static void C_ccall f_22311(C_word c,C_word *av) C_noret;
C_noret_decl(f_22314)
static void C_ccall f_22314(C_word c,C_word *av) C_noret;
C_noret_decl(f_22317)
static void C_ccall f_22317(C_word c,C_word *av) C_noret;
C_noret_decl(f_22320)
static void C_ccall f_22320(C_word c,C_word *av) C_noret;
C_noret_decl(f_22323)
static void C_ccall f_22323(C_word c,C_word *av) C_noret;
C_noret_decl(f_22326)
static void C_ccall f_22326(C_word c,C_word *av) C_noret;
C_noret_decl(f_22330)
static void C_ccall f_22330(C_word c,C_word *av) C_noret;
C_noret_decl(f_22336)
static void C_ccall f_22336(C_word c,C_word *av) C_noret;
C_noret_decl(f_22339)
static void C_ccall f_22339(C_word c,C_word *av) C_noret;
C_noret_decl(f_22342)
static void C_ccall f_22342(C_word c,C_word *av) C_noret;
C_noret_decl(f_22345)
static void C_ccall f_22345(C_word c,C_word *av) C_noret;
C_noret_decl(f_22353)
static void C_ccall f_22353(C_word c,C_word *av) C_noret;
C_noret_decl(f_22359)
static void C_ccall f_22359(C_word c,C_word *av) C_noret;
C_noret_decl(f_22362)
static void C_fcall f_22362(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22374)
static void C_ccall f_22374(C_word c,C_word *av) C_noret;
C_noret_decl(f_22378)
static void C_ccall f_22378(C_word c,C_word *av) C_noret;
C_noret_decl(f_22382)
static void C_ccall f_22382(C_word c,C_word *av) C_noret;
C_noret_decl(f_22386)
static void C_ccall f_22386(C_word c,C_word *av) C_noret;
C_noret_decl(f_22399)
static void C_ccall f_22399(C_word c,C_word *av) C_noret;
C_noret_decl(f_22405)
static void C_ccall f_22405(C_word c,C_word *av) C_noret;
C_noret_decl(f_22408)
static void C_ccall f_22408(C_word c,C_word *av) C_noret;
C_noret_decl(f_22411)
static void C_ccall f_22411(C_word c,C_word *av) C_noret;
C_noret_decl(f_22414)
static void C_ccall f_22414(C_word c,C_word *av) C_noret;
C_noret_decl(f_22418)
static void C_ccall f_22418(C_word c,C_word *av) C_noret;
C_noret_decl(f_22425)
static void C_ccall f_22425(C_word c,C_word *av) C_noret;
C_noret_decl(f_22431)
static void C_ccall f_22431(C_word c,C_word *av) C_noret;
C_noret_decl(f_22434)
static void C_ccall f_22434(C_word c,C_word *av) C_noret;
C_noret_decl(f_22437)
static void C_ccall f_22437(C_word c,C_word *av) C_noret;
C_noret_decl(f_22440)
static void C_ccall f_22440(C_word c,C_word *av) C_noret;
C_noret_decl(f_22443)
static void C_ccall f_22443(C_word c,C_word *av) C_noret;
C_noret_decl(f_22446)
static void C_ccall f_22446(C_word c,C_word *av) C_noret;
C_noret_decl(f_22450)
static void C_ccall f_22450(C_word c,C_word *av) C_noret;
C_noret_decl(f_22456)
static void C_ccall f_22456(C_word c,C_word *av) C_noret;
C_noret_decl(f_22459)
static void C_ccall f_22459(C_word c,C_word *av) C_noret;
C_noret_decl(f_22462)
static void C_ccall f_22462(C_word c,C_word *av) C_noret;
C_noret_decl(f_22465)
static void C_ccall f_22465(C_word c,C_word *av) C_noret;
C_noret_decl(f_22473)
static void C_ccall f_22473(C_word c,C_word *av) C_noret;
C_noret_decl(f_22484)
static void C_ccall f_22484(C_word c,C_word *av) C_noret;
C_noret_decl(f_22495)
static void C_ccall f_22495(C_word c,C_word *av) C_noret;
C_noret_decl(f_22499)
static void C_ccall f_22499(C_word c,C_word *av) C_noret;
C_noret_decl(f_22503)
static void C_ccall f_22503(C_word c,C_word *av) C_noret;
C_noret_decl(f_22507)
static void C_ccall f_22507(C_word c,C_word *av) C_noret;
C_noret_decl(f_22511)
static void C_ccall f_22511(C_word c,C_word *av) C_noret;
C_noret_decl(f_22518)
static void C_ccall f_22518(C_word c,C_word *av) C_noret;
C_noret_decl(f_22529)
static void C_ccall f_22529(C_word c,C_word *av) C_noret;
C_noret_decl(f_22533)
static void C_ccall f_22533(C_word c,C_word *av) C_noret;
C_noret_decl(f_22537)
static void C_ccall f_22537(C_word c,C_word *av) C_noret;
C_noret_decl(f_22541)
static void C_ccall f_22541(C_word c,C_word *av) C_noret;
C_noret_decl(f_22545)
static void C_ccall f_22545(C_word c,C_word *av) C_noret;
C_noret_decl(f_22559)
static void C_ccall f_22559(C_word c,C_word *av) C_noret;
C_noret_decl(f_22563)
static void C_ccall f_22563(C_word c,C_word *av) C_noret;
C_noret_decl(f_22567)
static void C_ccall f_22567(C_word c,C_word *av) C_noret;
C_noret_decl(f_22581)
static void C_ccall f_22581(C_word c,C_word *av) C_noret;
C_noret_decl(f_22585)
static void C_ccall f_22585(C_word c,C_word *av) C_noret;
C_noret_decl(f_22599)
static void C_ccall f_22599(C_word c,C_word *av) C_noret;
C_noret_decl(f_22603)
static void C_ccall f_22603(C_word c,C_word *av) C_noret;
C_noret_decl(f_22607)
static void C_ccall f_22607(C_word c,C_word *av) C_noret;
C_noret_decl(f_22621)
static void C_ccall f_22621(C_word c,C_word *av) C_noret;
C_noret_decl(f_22625)
static void C_ccall f_22625(C_word c,C_word *av) C_noret;
C_noret_decl(f_22633)
static void C_ccall f_22633(C_word c,C_word *av) C_noret;
C_noret_decl(f_22647)
static void C_ccall f_22647(C_word c,C_word *av) C_noret;
C_noret_decl(f_22651)
static void C_ccall f_22651(C_word c,C_word *av) C_noret;
C_noret_decl(f_22655)
static void C_ccall f_22655(C_word c,C_word *av) C_noret;
C_noret_decl(f_22659)
static void C_ccall f_22659(C_word c,C_word *av) C_noret;
C_noret_decl(f_22679)
static void C_ccall f_22679(C_word c,C_word *av) C_noret;
C_noret_decl(f_22683)
static void C_ccall f_22683(C_word c,C_word *av) C_noret;
C_noret_decl(f_22687)
static void C_ccall f_22687(C_word c,C_word *av) C_noret;
C_noret_decl(f_22691)
static void C_ccall f_22691(C_word c,C_word *av) C_noret;
C_noret_decl(f_22695)
static void C_ccall f_22695(C_word c,C_word *av) C_noret;
C_noret_decl(f_22699)
static void C_ccall f_22699(C_word c,C_word *av) C_noret;
C_noret_decl(f_22709)
static void C_ccall f_22709(C_word c,C_word *av) C_noret;
C_noret_decl(f_22711)
static void C_fcall f_22711(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22736)
static void C_ccall f_22736(C_word c,C_word *av) C_noret;
C_noret_decl(f_22748)
static void C_ccall f_22748(C_word c,C_word *av) C_noret;
C_noret_decl(f_22755)
static void C_ccall f_22755(C_word c,C_word *av) C_noret;
C_noret_decl(f_22769)
static void C_ccall f_22769(C_word c,C_word *av) C_noret;
C_noret_decl(f_22773)
static void C_ccall f_22773(C_word c,C_word *av) C_noret;
C_noret_decl(f_22785)
static void C_ccall f_22785(C_word c,C_word *av) C_noret;
C_noret_decl(f_22797)
static void C_ccall f_22797(C_word c,C_word *av) C_noret;
C_noret_decl(f_22815)
static void C_ccall f_22815(C_word c,C_word *av) C_noret;
C_noret_decl(f_22819)
static void C_ccall f_22819(C_word c,C_word *av) C_noret;
C_noret_decl(f_22823)
static void C_ccall f_22823(C_word c,C_word *av) C_noret;
C_noret_decl(f_22827)
static void C_ccall f_22827(C_word c,C_word *av) C_noret;
C_noret_decl(f_22831)
static void C_ccall f_22831(C_word c,C_word *av) C_noret;
C_noret_decl(f_22835)
static void C_ccall f_22835(C_word c,C_word *av) C_noret;
C_noret_decl(f_22839)
static void C_ccall f_22839(C_word c,C_word *av) C_noret;
C_noret_decl(f_22842)
static void C_ccall f_22842(C_word c,C_word *av) C_noret;
C_noret_decl(f_22850)
static void C_fcall f_22850(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22861)
static void C_ccall f_22861(C_word c,C_word *av) C_noret;
C_noret_decl(f_22867)
static void C_ccall f_22867(C_word c,C_word *av) C_noret;
C_noret_decl(f_22870)
static void C_ccall f_22870(C_word c,C_word *av) C_noret;
C_noret_decl(f_22873)
static void C_ccall f_22873(C_word c,C_word *av) C_noret;
C_noret_decl(f_22876)
static void C_ccall f_22876(C_word c,C_word *av) C_noret;
C_noret_decl(f_22884)
static void C_ccall f_22884(C_word c,C_word *av) C_noret;
C_noret_decl(f_22888)
static void C_ccall f_22888(C_word c,C_word *av) C_noret;
C_noret_decl(f_22892)
static void C_ccall f_22892(C_word c,C_word *av) C_noret;
C_noret_decl(f_22896)
static void C_ccall f_22896(C_word c,C_word *av) C_noret;
C_noret_decl(f_22899)
static void C_ccall f_22899(C_word c,C_word *av) C_noret;
C_noret_decl(f_22905)
static void C_ccall f_22905(C_word c,C_word *av) C_noret;
C_noret_decl(f_22908)
static void C_ccall f_22908(C_word c,C_word *av) C_noret;
C_noret_decl(f_22911)
static void C_ccall f_22911(C_word c,C_word *av) C_noret;
C_noret_decl(f_22914)
static void C_ccall f_22914(C_word c,C_word *av) C_noret;
C_noret_decl(f_22917)
static void C_ccall f_22917(C_word c,C_word *av) C_noret;
C_noret_decl(f_22924)
static void C_ccall f_22924(C_word c,C_word *av) C_noret;
C_noret_decl(f_22928)
static void C_ccall f_22928(C_word c,C_word *av) C_noret;
C_noret_decl(f_22940)
static void C_ccall f_22940(C_word c,C_word *av) C_noret;
C_noret_decl(f_22944)
static void C_ccall f_22944(C_word c,C_word *av) C_noret;
C_noret_decl(f_22985)
static void C_ccall f_22985(C_word c,C_word *av) C_noret;
C_noret_decl(f_23002)
static void C_ccall f_23002(C_word c,C_word *av) C_noret;
C_noret_decl(f_23023)
static void C_ccall f_23023(C_word c,C_word *av) C_noret;
C_noret_decl(f_23025)
static void C_fcall f_23025(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23059)
static void C_ccall f_23059(C_word c,C_word *av) C_noret;
C_noret_decl(f_23080)
static void C_ccall f_23080(C_word c,C_word *av) C_noret;
C_noret_decl(f_23082)
static void C_fcall f_23082(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23116)
static void C_ccall f_23116(C_word c,C_word *av) C_noret;
C_noret_decl(f_23141)
static void C_ccall f_23141(C_word c,C_word *av) C_noret;
C_noret_decl(f_23143)
static void C_fcall f_23143(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23181)
static void C_ccall f_23181(C_word c,C_word *av) C_noret;
C_noret_decl(f_23206)
static void C_ccall f_23206(C_word c,C_word *av) C_noret;
C_noret_decl(f_23208)
static void C_fcall f_23208(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23248)
static void C_ccall f_23248(C_word c,C_word *av) C_noret;
C_noret_decl(f_23250)
static void C_ccall f_23250(C_word c,C_word *av) C_noret;
C_noret_decl(f_23258)
static void C_ccall f_23258(C_word c,C_word *av) C_noret;
C_noret_decl(f_23260)
static void C_ccall f_23260(C_word c,C_word *av) C_noret;
C_noret_decl(f_23276)
static void C_ccall f_23276(C_word c,C_word *av) C_noret;
C_noret_decl(f_23280)
static void C_ccall f_23280(C_word c,C_word *av) C_noret;
C_noret_decl(f_23289)
static void C_ccall f_23289(C_word c,C_word *av) C_noret;
C_noret_decl(f_23316)
static void C_ccall f_23316(C_word c,C_word *av) C_noret;
C_noret_decl(f_23327)
static void C_ccall f_23327(C_word c,C_word *av) C_noret;
C_noret_decl(f_23333)
static void C_ccall f_23333(C_word c,C_word *av) C_noret;
C_noret_decl(f_23337)
static void C_ccall f_23337(C_word c,C_word *av) C_noret;
C_noret_decl(f_23390)
static void C_ccall f_23390(C_word c,C_word *av) C_noret;
C_noret_decl(f_23394)
static void C_ccall f_23394(C_word c,C_word *av) C_noret;
C_noret_decl(f_23398)
static void C_ccall f_23398(C_word c,C_word *av) C_noret;
C_noret_decl(f_6730)
static void C_ccall f_6730(C_word c,C_word *av) C_noret;
C_noret_decl(f_6733)
static void C_ccall f_6733(C_word c,C_word *av) C_noret;
C_noret_decl(f_6736)
static void C_ccall f_6736(C_word c,C_word *av) C_noret;
C_noret_decl(f_6739)
static void C_ccall f_6739(C_word c,C_word *av) C_noret;
C_noret_decl(f_6742)
static void C_ccall f_6742(C_word c,C_word *av) C_noret;
C_noret_decl(f_6745)
static void C_ccall f_6745(C_word c,C_word *av) C_noret;
C_noret_decl(f_6748)
static void C_ccall f_6748(C_word c,C_word *av) C_noret;
C_noret_decl(f_6751)
static void C_ccall f_6751(C_word c,C_word *av) C_noret;
C_noret_decl(f_6754)
static void C_ccall f_6754(C_word c,C_word *av) C_noret;
C_noret_decl(f_6820)
static void C_fcall f_6820(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6834)
static void C_ccall f_6834(C_word c,C_word *av) C_noret;
C_noret_decl(f_6858)
static void C_ccall f_6858(C_word c,C_word *av) C_noret;
C_noret_decl(f_6864)
static void C_fcall f_6864(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6882)
static void C_ccall f_6882(C_word c,C_word *av) C_noret;
C_noret_decl(f_6922)
static void C_fcall f_6922(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_6936)
static void C_ccall f_6936(C_word c,C_word *av) C_noret;
C_noret_decl(f_6955)
static void C_fcall f_6955(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6970)
static void C_fcall f_6970(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6978)
static void C_fcall f_6978(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6986)
static void C_ccall f_6986(C_word c,C_word *av) C_noret;
C_noret_decl(f_6997)
static void C_ccall f_6997(C_word c,C_word *av) C_noret;
C_noret_decl(f_7010)
static void C_fcall f_7010(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7017)
static void C_ccall f_7017(C_word c,C_word *av) C_noret;
C_noret_decl(f_7024)
static void C_ccall f_7024(C_word c,C_word *av) C_noret;
C_noret_decl(f_7028)
static void C_ccall f_7028(C_word c,C_word *av) C_noret;
C_noret_decl(f_7040)
static void C_ccall f_7040(C_word c,C_word *av) C_noret;
C_noret_decl(f_7042)
static void C_fcall f_7042(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7089)
static void C_ccall f_7089(C_word c,C_word *av) C_noret;
C_noret_decl(f_7091)
static void C_fcall f_7091(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7125)
static void C_fcall f_7125(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7131)
static void C_fcall f_7131(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7153)
static void C_ccall f_7153(C_word c,C_word *av) C_noret;
C_noret_decl(f_7159)
static void C_fcall f_7159(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7165)
static void C_fcall f_7165(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7175)
static void C_ccall f_7175(C_word c,C_word *av) C_noret;
C_noret_decl(f_7195)
static void C_fcall f_7195(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7209)
static void C_ccall f_7209(C_word c,C_word *av) C_noret;
C_noret_decl(f_7223)
static void C_fcall f_7223(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7241)
static void C_ccall f_7241(C_word c,C_word *av) C_noret;
C_noret_decl(f_7251)
static void C_fcall f_7251(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7264)
static void C_ccall f_7264(C_word c,C_word *av) C_noret;
C_noret_decl(f_7278)
static void C_ccall f_7278(C_word c,C_word *av) C_noret;
C_noret_decl(f_7320)
static void C_fcall f_7320(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7326)
static void C_fcall f_7326(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7340)
static void C_ccall f_7340(C_word c,C_word *av) C_noret;
C_noret_decl(f_7353)
static void C_ccall f_7353(C_word c,C_word *av) C_noret;
C_noret_decl(f_7409)
static void C_fcall f_7409(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7418)
static void C_fcall f_7418(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7426)
static void C_fcall f_7426(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7430)
static void C_ccall f_7430(C_word c,C_word *av) C_noret;
C_noret_decl(f_7434)
static C_word C_fcall f_7434(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_7452)
static void C_ccall f_7452(C_word c,C_word *av) C_noret;
C_noret_decl(f_7474)
static void C_ccall f_7474(C_word c,C_word *av) C_noret;
C_noret_decl(f_7491)
static void C_fcall f_7491(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7590)
static void C_fcall f_7590(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7676)
static void C_fcall f_7676(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7701)
static void C_fcall f_7701(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7724)
static void C_ccall f_7724(C_word c,C_word *av) C_noret;
C_noret_decl(f_7729)
static void C_ccall f_7729(C_word c,C_word *av) C_noret;
C_noret_decl(f_7743)
static void C_ccall f_7743(C_word c,C_word *av) C_noret;
C_noret_decl(f_7767)
static void C_fcall f_7767(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7773)
static void C_fcall f_7773(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7787)
static void C_ccall f_7787(C_word c,C_word *av) C_noret;
C_noret_decl(f_7791)
static void C_ccall f_7791(C_word c,C_word *av) C_noret;
C_noret_decl(f_7803)
static void C_ccall f_7803(C_word c,C_word *av) C_noret;
C_noret_decl(f_7809)
static void C_fcall f_7809(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7821)
static void C_ccall f_7821(C_word c,C_word *av) C_noret;
C_noret_decl(f_7886)
static C_word C_fcall f_7886(C_word t0,C_word t1);
C_noret_decl(f_7947)
static void C_ccall f_7947(C_word c,C_word *av) C_noret;
C_noret_decl(f_7950)
static void C_fcall f_7950(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7959)
static void C_ccall f_7959(C_word c,C_word *av) C_noret;
C_noret_decl(f_8079)
static void C_ccall f_8079(C_word c,C_word *av) C_noret;
C_noret_decl(f_8087)
static void C_ccall f_8087(C_word c,C_word *av) C_noret;
C_noret_decl(f_8089)
static void C_ccall f_8089(C_word c,C_word *av) C_noret;
C_noret_decl(f_8132)
static void C_ccall f_8132(C_word c,C_word *av) C_noret;
C_noret_decl(f_8169)
static void C_fcall f_8169(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8182)
static void C_ccall f_8182(C_word c,C_word *av) C_noret;
C_noret_decl(f_8194)
static void C_ccall f_8194(C_word c,C_word *av) C_noret;
C_noret_decl(f_8197)
static void C_ccall f_8197(C_word c,C_word *av) C_noret;
C_noret_decl(f_8258)
static void C_ccall f_8258(C_word c,C_word *av) C_noret;
C_noret_decl(f_8260)
static void C_fcall f_8260(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8285)
static void C_ccall f_8285(C_word c,C_word *av) C_noret;
C_noret_decl(f_8309)
static void C_ccall f_8309(C_word c,C_word *av) C_noret;
C_noret_decl(f_8313)
static void C_ccall f_8313(C_word c,C_word *av) C_noret;
C_noret_decl(f_8343)
static void C_ccall f_8343(C_word c,C_word *av) C_noret;
C_noret_decl(f_8346)
static void C_ccall f_8346(C_word c,C_word *av) C_noret;
C_noret_decl(f_8348)
static void C_fcall f_8348(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8373)
static void C_ccall f_8373(C_word c,C_word *av) C_noret;
C_noret_decl(f_8418)
static void C_ccall f_8418(C_word c,C_word *av) C_noret;
C_noret_decl(f_8422)
static void C_fcall f_8422(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8432)
static void C_ccall f_8432(C_word c,C_word *av) C_noret;
C_noret_decl(f_8441)
static void C_ccall f_8441(C_word c,C_word *av) C_noret;
C_noret_decl(f_8461)
static void C_fcall f_8461(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8473)
static C_word C_fcall f_8473(C_word t0,C_word t1);
C_noret_decl(f_8492)
static void C_fcall f_8492(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_8496)
static void C_ccall f_8496(C_word c,C_word *av) C_noret;
C_noret_decl(f_8508)
static void C_fcall f_8508(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8546)
static void C_ccall f_8546(C_word c,C_word *av) C_noret;
C_noret_decl(f_8553)
static void C_ccall f_8553(C_word c,C_word *av) C_noret;
C_noret_decl(f_8555)
static void C_ccall f_8555(C_word c,C_word *av) C_noret;
C_noret_decl(f_8587)
static void C_ccall f_8587(C_word c,C_word *av) C_noret;
C_noret_decl(f_8617)
static void C_fcall f_8617(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_8621)
static void C_ccall f_8621(C_word c,C_word *av) C_noret;
C_noret_decl(f_8627)
static void C_ccall f_8627(C_word c,C_word *av) C_noret;
C_noret_decl(f_8639)
static void C_ccall f_8639(C_word c,C_word *av) C_noret;
C_noret_decl(f_8641)
static void C_fcall f_8641(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8669)
static void C_ccall f_8669(C_word c,C_word *av) C_noret;
C_noret_decl(f_8672)
static void C_ccall f_8672(C_word c,C_word *av) C_noret;
C_noret_decl(f_8685)
static void C_ccall f_8685(C_word c,C_word *av) C_noret;
C_noret_decl(f_8688)
static void C_ccall f_8688(C_word c,C_word *av) C_noret;
C_noret_decl(f_8691)
static void C_ccall f_8691(C_word c,C_word *av) C_noret;
C_noret_decl(f_8707)
static void C_fcall f_8707(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_8717)
static void C_ccall f_8717(C_word c,C_word *av) C_noret;
C_noret_decl(f_8723)
static void C_ccall f_8723(C_word c,C_word *av) C_noret;
C_noret_decl(f_8733)
static void C_ccall f_8733(C_word c,C_word *av) C_noret;
C_noret_decl(f_8742)
static void C_fcall f_8742(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8745)
static void C_ccall f_8745(C_word c,C_word *av) C_noret;
C_noret_decl(f_8752)
static void C_ccall f_8752(C_word c,C_word *av) C_noret;
C_noret_decl(f_8758)
static void C_ccall f_8758(C_word c,C_word *av) C_noret;
C_noret_decl(f_8763)
static void C_ccall f_8763(C_word c,C_word *av) C_noret;
C_noret_decl(f_8769)
static void C_ccall f_8769(C_word c,C_word *av) C_noret;
C_noret_decl(f_8773)
static void C_ccall f_8773(C_word c,C_word *av) C_noret;
C_noret_decl(f_8776)
static void C_ccall f_8776(C_word c,C_word *av) C_noret;
C_noret_decl(f_8779)
static void C_fcall f_8779(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8782)
static void C_ccall f_8782(C_word c,C_word *av) C_noret;
C_noret_decl(f_8785)
static void C_ccall f_8785(C_word c,C_word *av) C_noret;
C_noret_decl(f_8788)
static void C_fcall f_8788(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8791)
static void C_fcall f_8791(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8794)
static void C_ccall f_8794(C_word c,C_word *av) C_noret;
C_noret_decl(f_8807)
static void C_fcall f_8807(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8819)
static void C_fcall f_8819(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8844)
static void C_ccall f_8844(C_word c,C_word *av) C_noret;
C_noret_decl(f_8854)
static void C_fcall f_8854(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8863)
static void C_ccall f_8863(C_word c,C_word *av) C_noret;
C_noret_decl(f_8875)
static void C_ccall f_8875(C_word c,C_word *av) C_noret;
C_noret_decl(f_8882)
static void C_ccall f_8882(C_word c,C_word *av) C_noret;
C_noret_decl(f_8885)
static void C_ccall f_8885(C_word c,C_word *av) C_noret;
C_noret_decl(f_8888)
static void C_ccall f_8888(C_word c,C_word *av) C_noret;
C_noret_decl(f_8922)
static void C_fcall f_8922(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8929)
static void C_ccall f_8929(C_word c,C_word *av) C_noret;
C_noret_decl(f_8932)
static void C_ccall f_8932(C_word c,C_word *av) C_noret;
C_noret_decl(f_8938)
static void C_ccall f_8938(C_word c,C_word *av) C_noret;
C_noret_decl(f_8967)
static void C_ccall f_8967(C_word c,C_word *av) C_noret;
C_noret_decl(f_8971)
static void C_ccall f_8971(C_word c,C_word *av) C_noret;
C_noret_decl(f_8989)
static void C_ccall f_8989(C_word c,C_word *av) C_noret;
C_noret_decl(f_8992)
static void C_ccall f_8992(C_word c,C_word *av) C_noret;
C_noret_decl(f_8995)
static void C_ccall f_8995(C_word c,C_word *av) C_noret;
C_noret_decl(f_9001)
static void C_ccall f_9001(C_word c,C_word *av) C_noret;
C_noret_decl(f_9034)
static void C_ccall f_9034(C_word c,C_word *av) C_noret;
C_noret_decl(f_9038)
static void C_ccall f_9038(C_word c,C_word *av) C_noret;
C_noret_decl(f_9077)
static void C_ccall f_9077(C_word c,C_word *av) C_noret;
C_noret_decl(f_9081)
static void C_fcall f_9081(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9100)
static void C_fcall f_9100(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9104)
static void C_fcall f_9104(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9110)
static void C_fcall f_9110(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9124)
static void C_ccall f_9124(C_word c,C_word *av) C_noret;
C_noret_decl(f_9130)
static void C_ccall f_9130(C_word c,C_word *av) C_noret;
C_noret_decl(f_9144)
static void C_ccall f_9144(C_word c,C_word *av) C_noret;
C_noret_decl(f_9175)
static void C_ccall f_9175(C_word c,C_word *av) C_noret;
C_noret_decl(f_9188)
static void C_ccall f_9188(C_word c,C_word *av) C_noret;
C_noret_decl(f_9198)
static void C_ccall f_9198(C_word c,C_word *av) C_noret;
C_noret_decl(f_9200)
static void C_ccall f_9200(C_word c,C_word *av) C_noret;
C_noret_decl(f_9229)
static void C_fcall f_9229(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_9242)
static void C_ccall f_9242(C_word c,C_word *av) C_noret;
C_noret_decl(f_9258)
static void C_ccall f_9258(C_word c,C_word *av) C_noret;
C_noret_decl(f_9269)
static void C_ccall f_9269(C_word c,C_word *av) C_noret;
C_noret_decl(f_9273)
static void C_ccall f_9273(C_word c,C_word *av) C_noret;
C_noret_decl(f_9301)
static void C_ccall f_9301(C_word c,C_word *av) C_noret;
C_noret_decl(f_9305)
static void C_ccall f_9305(C_word c,C_word *av) C_noret;
C_noret_decl(f_9307)
static void C_fcall f_9307(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9341)
static C_word C_fcall f_9341(C_word *a,C_word t0);
C_noret_decl(f_9346)
static void C_fcall f_9346(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_9352)
static void C_fcall f_9352(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9357)
static void C_ccall f_9357(C_word c,C_word *av) C_noret;
C_noret_decl(f_9379)
static void C_fcall f_9379(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9386)
static void C_ccall f_9386(C_word c,C_word *av) C_noret;
C_noret_decl(f_9389)
static void C_ccall f_9389(C_word c,C_word *av) C_noret;
C_noret_decl(f_9392)
static void C_ccall f_9392(C_word c,C_word *av) C_noret;
C_noret_decl(f_9398)
static void C_ccall f_9398(C_word c,C_word *av) C_noret;
C_noret_decl(f_9402)
static void C_fcall f_9402(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9409)
static void C_ccall f_9409(C_word c,C_word *av) C_noret;
C_noret_decl(f_9417)
static void C_fcall f_9417(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9442)
static void C_ccall f_9442(C_word c,C_word *av) C_noret;
C_noret_decl(f_9451)
static void C_ccall f_9451(C_word c,C_word *av) C_noret;
C_noret_decl(f_9457)
static void C_ccall f_9457(C_word c,C_word *av) C_noret;
C_noret_decl(f_9475)
static void C_fcall f_9475(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_9479)
static void C_ccall f_9479(C_word c,C_word *av) C_noret;
C_noret_decl(f_9482)
static void C_ccall f_9482(C_word c,C_word *av) C_noret;
C_noret_decl(f_9485)
static void C_ccall f_9485(C_word c,C_word *av) C_noret;
C_noret_decl(f_9493)
static void C_ccall f_9493(C_word c,C_word *av) C_noret;
C_noret_decl(f_9521)
static void C_ccall f_9521(C_word c,C_word *av) C_noret;
C_noret_decl(f_9563)
static void C_ccall f_9563(C_word c,C_word *av) C_noret;
C_noret_decl(f_9580)
static void C_ccall f_9580(C_word c,C_word *av) C_noret;
C_noret_decl(f_9600)
static void C_ccall f_9600(C_word c,C_word *av) C_noret;
C_noret_decl(f_9617)
static void C_ccall f_9617(C_word c,C_word *av) C_noret;
C_noret_decl(f_9636)
static void C_ccall f_9636(C_word c,C_word *av) C_noret;
C_noret_decl(f_9642)
static void C_fcall f_9642(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9647)
static void C_ccall f_9647(C_word c,C_word *av) C_noret;
C_noret_decl(f_9666)
static void C_ccall f_9666(C_word c,C_word *av) C_noret;
C_noret_decl(f_9669)
static void C_ccall f_9669(C_word c,C_word *av) C_noret;
C_noret_decl(f_9673)
static void C_ccall f_9673(C_word c,C_word *av) C_noret;
C_noret_decl(f_9680)
static void C_ccall f_9680(C_word c,C_word *av) C_noret;
C_noret_decl(f_9692)
static void C_fcall f_9692(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9695)
static void C_ccall f_9695(C_word c,C_word *av) C_noret;
C_noret_decl(f_9727)
static void C_fcall f_9727(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9756)
static void C_ccall f_9756(C_word c,C_word *av) C_noret;
C_noret_decl(f_9821)
static void C_fcall f_9821(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9855)
static void C_fcall f_9855(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9865)
static void C_ccall f_9865(C_word c,C_word *av) C_noret;
C_noret_decl(f_9911)
static void C_ccall f_9911(C_word c,C_word *av) C_noret;
C_noret_decl(f_9915)
static void C_ccall f_9915(C_word c,C_word *av) C_noret;
C_noret_decl(f_9917)
static void C_ccall f_9917(C_word c,C_word *av) C_noret;
C_noret_decl(f_9940)
static void C_fcall f_9940(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_9958)
static void C_ccall f_9958(C_word c,C_word *av) C_noret;
C_noret_decl(f_9967)
static void C_ccall f_9967(C_word c,C_word *av) C_noret;
C_noret_decl(f_9970)
static void C_fcall f_9970(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9986)
static void C_fcall f_9986(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9995)
static void C_ccall f_9995(C_word c,C_word *av) C_noret;
C_noret_decl(C_scrutinizer_toplevel)
C_externexport void C_ccall C_scrutinizer_toplevel(C_word c,C_word *av) C_noret;

C_noret_decl(trf_10099)
static void C_ccall trf_10099(C_word c,C_word *av) C_noret;
static void C_ccall trf_10099(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10099(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10107)
static void C_ccall trf_10107(C_word c,C_word *av) C_noret;
static void C_ccall trf_10107(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10107(t0,t1);}

C_noret_decl(trf_10111)
static void C_ccall trf_10111(C_word c,C_word *av) C_noret;
static void C_ccall trf_10111(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10111(t0,t1,t2);}

C_noret_decl(trf_10192)
static void C_ccall trf_10192(C_word c,C_word *av) C_noret;
static void C_ccall trf_10192(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10192(t0,t1);}

C_noret_decl(trf_10238)
static void C_ccall trf_10238(C_word c,C_word *av) C_noret;
static void C_ccall trf_10238(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10238(t0,t1,t2,t3);}

C_noret_decl(trf_10347)
static void C_ccall trf_10347(C_word c,C_word *av) C_noret;
static void C_ccall trf_10347(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10347(t0,t1);}

C_noret_decl(trf_10355)
static void C_ccall trf_10355(C_word c,C_word *av) C_noret;
static void C_ccall trf_10355(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10355(t0,t1,t2,t3);}

C_noret_decl(trf_10453)
static void C_ccall trf_10453(C_word c,C_word *av) C_noret;
static void C_ccall trf_10453(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10453(t0,t1);}

C_noret_decl(trf_10468)
static void C_ccall trf_10468(C_word c,C_word *av) C_noret;
static void C_ccall trf_10468(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10468(t0,t1);}

C_noret_decl(trf_10579)
static void C_ccall trf_10579(C_word c,C_word *av) C_noret;
static void C_ccall trf_10579(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10579(t0,t1);}

C_noret_decl(trf_10582)
static void C_ccall trf_10582(C_word c,C_word *av) C_noret;
static void C_ccall trf_10582(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10582(t0,t1);}

C_noret_decl(trf_10602)
static void C_ccall trf_10602(C_word c,C_word *av) C_noret;
static void C_ccall trf_10602(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10602(t0,t1);}

C_noret_decl(trf_10678)
static void C_ccall trf_10678(C_word c,C_word *av) C_noret;
static void C_ccall trf_10678(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10678(t0,t1,t2,t3);}

C_noret_decl(trf_10742)
static void C_ccall trf_10742(C_word c,C_word *av) C_noret;
static void C_ccall trf_10742(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10742(t0,t1);}

C_noret_decl(trf_10749)
static void C_ccall trf_10749(C_word c,C_word *av) C_noret;
static void C_ccall trf_10749(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10749(t0,t1);}

C_noret_decl(trf_10800)
static void C_ccall trf_10800(C_word c,C_word *av) C_noret;
static void C_ccall trf_10800(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10800(t0,t1,t2);}

C_noret_decl(trf_10812)
static void C_ccall trf_10812(C_word c,C_word *av) C_noret;
static void C_ccall trf_10812(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10812(t0,t1,t2);}

C_noret_decl(trf_10854)
static void C_ccall trf_10854(C_word c,C_word *av) C_noret;
static void C_ccall trf_10854(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10854(t0,t1,t2,t3);}

C_noret_decl(trf_10879)
static void C_ccall trf_10879(C_word c,C_word *av) C_noret;
static void C_ccall trf_10879(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10879(t0,t1);}

C_noret_decl(trf_10973)
static void C_ccall trf_10973(C_word c,C_word *av) C_noret;
static void C_ccall trf_10973(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10973(t0,t1,t2);}

C_noret_decl(trf_11040)
static void C_ccall trf_11040(C_word c,C_word *av) C_noret;
static void C_ccall trf_11040(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11040(t0,t1,t2);}

C_noret_decl(trf_11052)
static void C_ccall trf_11052(C_word c,C_word *av) C_noret;
static void C_ccall trf_11052(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11052(t0,t1,t2);}

C_noret_decl(trf_11086)
static void C_ccall trf_11086(C_word c,C_word *av) C_noret;
static void C_ccall trf_11086(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11086(t0,t1,t2,t3);}

C_noret_decl(trf_11162)
static void C_ccall trf_11162(C_word c,C_word *av) C_noret;
static void C_ccall trf_11162(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11162(t0,t1,t2);}

C_noret_decl(trf_11200)
static void C_ccall trf_11200(C_word c,C_word *av) C_noret;
static void C_ccall trf_11200(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11200(t0,t1,t2,t3);}

C_noret_decl(trf_11289)
static void C_ccall trf_11289(C_word c,C_word *av) C_noret;
static void C_ccall trf_11289(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11289(t0,t1);}

C_noret_decl(trf_11292)
static void C_ccall trf_11292(C_word c,C_word *av) C_noret;
static void C_ccall trf_11292(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11292(t0,t1);}

C_noret_decl(trf_11362)
static void C_ccall trf_11362(C_word c,C_word *av) C_noret;
static void C_ccall trf_11362(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11362(t0,t1,t2,t3);}

C_noret_decl(trf_11446)
static void C_ccall trf_11446(C_word c,C_word *av) C_noret;
static void C_ccall trf_11446(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11446(t0,t1,t2);}

C_noret_decl(trf_11458)
static void C_ccall trf_11458(C_word c,C_word *av) C_noret;
static void C_ccall trf_11458(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11458(t0,t1,t2);}

C_noret_decl(trf_11608)
static void C_ccall trf_11608(C_word c,C_word *av) C_noret;
static void C_ccall trf_11608(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11608(t0,t1,t2);}

C_noret_decl(trf_11647)
static void C_ccall trf_11647(C_word c,C_word *av) C_noret;
static void C_ccall trf_11647(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11647(t0,t1);}

C_noret_decl(trf_11653)
static void C_ccall trf_11653(C_word c,C_word *av) C_noret;
static void C_ccall trf_11653(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11653(t0,t1,t2);}

C_noret_decl(trf_11683)
static void C_ccall trf_11683(C_word c,C_word *av) C_noret;
static void C_ccall trf_11683(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11683(t0,t1,t2,t3);}

C_noret_decl(trf_11737)
static void C_ccall trf_11737(C_word c,C_word *av) C_noret;
static void C_ccall trf_11737(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11737(t0,t1,t2);}

C_noret_decl(trf_11858)
static void C_ccall trf_11858(C_word c,C_word *av) C_noret;
static void C_ccall trf_11858(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11858(t0,t1,t2,t3);}

C_noret_decl(trf_11867)
static void C_ccall trf_11867(C_word c,C_word *av) C_noret;
static void C_ccall trf_11867(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_11867(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_11989)
static void C_ccall trf_11989(C_word c,C_word *av) C_noret;
static void C_ccall trf_11989(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11989(t0,t1,t2,t3);}

C_noret_decl(trf_12052)
static void C_ccall trf_12052(C_word c,C_word *av) C_noret;
static void C_ccall trf_12052(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12052(t0,t1,t2,t3);}

C_noret_decl(trf_12120)
static void C_ccall trf_12120(C_word c,C_word *av) C_noret;
static void C_ccall trf_12120(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12120(t0,t1,t2,t3);}

C_noret_decl(trf_12142)
static void C_ccall trf_12142(C_word c,C_word *av) C_noret;
static void C_ccall trf_12142(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12142(t0,t1,t2,t3);}

C_noret_decl(trf_12148)
static void C_ccall trf_12148(C_word c,C_word *av) C_noret;
static void C_ccall trf_12148(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12148(t0,t1,t2,t3);}

C_noret_decl(trf_12178)
static void C_ccall trf_12178(C_word c,C_word *av) C_noret;
static void C_ccall trf_12178(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12178(t0,t1,t2,t3);}

C_noret_decl(trf_12195)
static void C_ccall trf_12195(C_word c,C_word *av) C_noret;
static void C_ccall trf_12195(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12195(t0,t1,t2);}

C_noret_decl(trf_12236)
static void C_ccall trf_12236(C_word c,C_word *av) C_noret;
static void C_ccall trf_12236(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12236(t0,t1,t2);}

C_noret_decl(trf_12327)
static void C_ccall trf_12327(C_word c,C_word *av) C_noret;
static void C_ccall trf_12327(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12327(t0,t1,t2);}

C_noret_decl(trf_12444)
static void C_ccall trf_12444(C_word c,C_word *av) C_noret;
static void C_ccall trf_12444(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12444(t0,t1,t2);}

C_noret_decl(trf_12458)
static void C_ccall trf_12458(C_word c,C_word *av) C_noret;
static void C_ccall trf_12458(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12458(t0,t1,t2);}

C_noret_decl(trf_12694)
static void C_ccall trf_12694(C_word c,C_word *av) C_noret;
static void C_ccall trf_12694(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12694(t0,t1);}

C_noret_decl(trf_13431)
static void C_ccall trf_13431(C_word c,C_word *av) C_noret;
static void C_ccall trf_13431(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13431(t0,t1,t2,t3);}

C_noret_decl(trf_13437)
static void C_ccall trf_13437(C_word c,C_word *av) C_noret;
static void C_ccall trf_13437(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13437(t0,t1,t2,t3);}

C_noret_decl(trf_13503)
static void C_ccall trf_13503(C_word c,C_word *av) C_noret;
static void C_ccall trf_13503(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13503(t0,t1);}

C_noret_decl(trf_13506)
static void C_ccall trf_13506(C_word c,C_word *av) C_noret;
static void C_ccall trf_13506(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13506(t0,t1,t2);}

C_noret_decl(trf_13587)
static void C_ccall trf_13587(C_word c,C_word *av) C_noret;
static void C_ccall trf_13587(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13587(t0,t1,t2);}

C_noret_decl(trf_13674)
static void C_ccall trf_13674(C_word c,C_word *av) C_noret;
static void C_ccall trf_13674(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13674(t0,t1,t2,t3);}

C_noret_decl(trf_13716)
static void C_ccall trf_13716(C_word c,C_word *av) C_noret;
static void C_ccall trf_13716(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13716(t0,t1);}

C_noret_decl(trf_13794)
static void C_ccall trf_13794(C_word c,C_word *av) C_noret;
static void C_ccall trf_13794(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13794(t0,t1,t2,t3);}

C_noret_decl(trf_13938)
static void C_ccall trf_13938(C_word c,C_word *av) C_noret;
static void C_ccall trf_13938(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13938(t0,t1,t2);}

C_noret_decl(trf_14135)
static void C_ccall trf_14135(C_word c,C_word *av) C_noret;
static void C_ccall trf_14135(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14135(t0,t1,t2);}

C_noret_decl(trf_14191)
static void C_ccall trf_14191(C_word c,C_word *av) C_noret;
static void C_ccall trf_14191(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14191(t0,t1,t2);}

C_noret_decl(trf_14264)
static void C_ccall trf_14264(C_word c,C_word *av) C_noret;
static void C_ccall trf_14264(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14264(t0,t1,t2);}

C_noret_decl(trf_14311)
static void C_ccall trf_14311(C_word c,C_word *av) C_noret;
static void C_ccall trf_14311(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14311(t0,t1,t2);}

C_noret_decl(trf_14389)
static void C_ccall trf_14389(C_word c,C_word *av) C_noret;
static void C_ccall trf_14389(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14389(t0,t1,t2);}

C_noret_decl(trf_14437)
static void C_ccall trf_14437(C_word c,C_word *av) C_noret;
static void C_ccall trf_14437(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14437(t0,t1,t2);}

C_noret_decl(trf_14472)
static void C_ccall trf_14472(C_word c,C_word *av) C_noret;
static void C_ccall trf_14472(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14472(t0,t1,t2);}

C_noret_decl(trf_14653)
static void C_ccall trf_14653(C_word c,C_word *av) C_noret;
static void C_ccall trf_14653(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14653(t0,t1,t2,t3);}

C_noret_decl(trf_14710)
static void C_ccall trf_14710(C_word c,C_word *av) C_noret;
static void C_ccall trf_14710(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14710(t0,t1,t2,t3);}

C_noret_decl(trf_14778)
static void C_ccall trf_14778(C_word c,C_word *av) C_noret;
static void C_ccall trf_14778(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14778(t0,t1);}

C_noret_decl(trf_14806)
static void C_ccall trf_14806(C_word c,C_word *av) C_noret;
static void C_ccall trf_14806(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14806(t0,t1,t2,t3);}

C_noret_decl(trf_14814)
static void C_ccall trf_14814(C_word c,C_word *av) C_noret;
static void C_ccall trf_14814(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14814(t0,t1,t2);}

C_noret_decl(trf_14828)
static void C_ccall trf_14828(C_word c,C_word *av) C_noret;
static void C_ccall trf_14828(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14828(t0,t1,t2);}

C_noret_decl(trf_14895)
static void C_ccall trf_14895(C_word c,C_word *av) C_noret;
static void C_ccall trf_14895(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14895(t0,t1);}

C_noret_decl(trf_14929)
static void C_ccall trf_14929(C_word c,C_word *av) C_noret;
static void C_ccall trf_14929(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14929(t0,t1);}

C_noret_decl(trf_14960)
static void C_ccall trf_14960(C_word c,C_word *av) C_noret;
static void C_ccall trf_14960(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14960(t0,t1,t2,t3);}

C_noret_decl(trf_15171)
static void C_ccall trf_15171(C_word c,C_word *av) C_noret;
static void C_ccall trf_15171(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15171(t0,t1);}

C_noret_decl(trf_15222)
static void C_ccall trf_15222(C_word c,C_word *av) C_noret;
static void C_ccall trf_15222(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15222(t0,t1);}

C_noret_decl(trf_15279)
static void C_ccall trf_15279(C_word c,C_word *av) C_noret;
static void C_ccall trf_15279(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15279(t0,t1);}

C_noret_decl(trf_15336)
static void C_ccall trf_15336(C_word c,C_word *av) C_noret;
static void C_ccall trf_15336(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15336(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15345)
static void C_ccall trf_15345(C_word c,C_word *av) C_noret;
static void C_ccall trf_15345(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15345(t0,t1,t2,t3);}

C_noret_decl(trf_15362)
static void C_ccall trf_15362(C_word c,C_word *av) C_noret;
static void C_ccall trf_15362(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15362(t0,t1);}

C_noret_decl(trf_15364)
static void C_ccall trf_15364(C_word c,C_word *av) C_noret;
static void C_ccall trf_15364(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15364(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15499)
static void C_ccall trf_15499(C_word c,C_word *av) C_noret;
static void C_ccall trf_15499(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15499(t0,t1,t2);}

C_noret_decl(trf_15559)
static void C_ccall trf_15559(C_word c,C_word *av) C_noret;
static void C_ccall trf_15559(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15559(t0,t1,t2);}

C_noret_decl(trf_15570)
static void C_ccall trf_15570(C_word c,C_word *av) C_noret;
static void C_ccall trf_15570(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15570(t0,t1,t2);}

C_noret_decl(trf_15600)
static void C_ccall trf_15600(C_word c,C_word *av) C_noret;
static void C_ccall trf_15600(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15600(t0,t1);}

C_noret_decl(trf_15602)
static void C_ccall trf_15602(C_word c,C_word *av) C_noret;
static void C_ccall trf_15602(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15602(t0,t1,t2);}

C_noret_decl(trf_15916)
static void C_ccall trf_15916(C_word c,C_word *av) C_noret;
static void C_ccall trf_15916(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15916(t0,t1,t2);}

C_noret_decl(trf_15978)
static void C_ccall trf_15978(C_word c,C_word *av) C_noret;
static void C_ccall trf_15978(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15978(t0,t1,t2);}

C_noret_decl(trf_16005)
static void C_ccall trf_16005(C_word c,C_word *av) C_noret;
static void C_ccall trf_16005(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16005(t0,t1,t2);}

C_noret_decl(trf_16051)
static void C_ccall trf_16051(C_word c,C_word *av) C_noret;
static void C_ccall trf_16051(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16051(t0,t1,t2);}

C_noret_decl(trf_16078)
static void C_ccall trf_16078(C_word c,C_word *av) C_noret;
static void C_ccall trf_16078(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16078(t0,t1,t2);}

C_noret_decl(trf_16164)
static void C_ccall trf_16164(C_word c,C_word *av) C_noret;
static void C_ccall trf_16164(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16164(t0,t1,t2);}

C_noret_decl(trf_16189)
static void C_ccall trf_16189(C_word c,C_word *av) C_noret;
static void C_ccall trf_16189(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16189(t0,t1);}

C_noret_decl(trf_16211)
static void C_ccall trf_16211(C_word c,C_word *av) C_noret;
static void C_ccall trf_16211(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16211(t0,t1,t2);}

C_noret_decl(trf_16242)
static void C_ccall trf_16242(C_word c,C_word *av) C_noret;
static void C_ccall trf_16242(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16242(t0,t1,t2);}

C_noret_decl(trf_16295)
static void C_ccall trf_16295(C_word c,C_word *av) C_noret;
static void C_ccall trf_16295(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16295(t0,t1,t2,t3);}

C_noret_decl(trf_16303)
static void C_ccall trf_16303(C_word c,C_word *av) C_noret;
static void C_ccall trf_16303(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16303(t0,t1,t2);}

C_noret_decl(trf_16384)
static void C_ccall trf_16384(C_word c,C_word *av) C_noret;
static void C_ccall trf_16384(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16384(t0,t1,t2);}

C_noret_decl(trf_16397)
static void C_ccall trf_16397(C_word c,C_word *av) C_noret;
static void C_ccall trf_16397(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16397(t0,t1,t2);}

C_noret_decl(trf_16481)
static void C_ccall trf_16481(C_word c,C_word *av) C_noret;
static void C_ccall trf_16481(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16481(t0,t1);}

C_noret_decl(trf_16491)
static void C_ccall trf_16491(C_word c,C_word *av) C_noret;
static void C_ccall trf_16491(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16491(t0,t1,t2);}

C_noret_decl(trf_16504)
static void C_ccall trf_16504(C_word c,C_word *av) C_noret;
static void C_ccall trf_16504(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16504(t0,t1,t2);}

C_noret_decl(trf_16581)
static void C_ccall trf_16581(C_word c,C_word *av) C_noret;
static void C_ccall trf_16581(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16581(t0,t1,t2);}

C_noret_decl(trf_16593)
static void C_ccall trf_16593(C_word c,C_word *av) C_noret;
static void C_ccall trf_16593(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16593(t0,t1,t2);}

C_noret_decl(trf_16627)
static void C_ccall trf_16627(C_word c,C_word *av) C_noret;
static void C_ccall trf_16627(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16627(t0,t1,t2);}

C_noret_decl(trf_16821)
static void C_ccall trf_16821(C_word c,C_word *av) C_noret;
static void C_ccall trf_16821(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16821(t0,t1);}

C_noret_decl(trf_16887)
static void C_ccall trf_16887(C_word c,C_word *av) C_noret;
static void C_ccall trf_16887(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16887(t0,t1,t2);}

C_noret_decl(trf_16914)
static void C_ccall trf_16914(C_word c,C_word *av) C_noret;
static void C_ccall trf_16914(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16914(t0,t1,t2);}

C_noret_decl(trf_17078)
static void C_ccall trf_17078(C_word c,C_word *av) C_noret;
static void C_ccall trf_17078(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17078(t0,t1,t2);}

C_noret_decl(trf_17174)
static void C_ccall trf_17174(C_word c,C_word *av) C_noret;
static void C_ccall trf_17174(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17174(t0,t1);}

C_noret_decl(trf_17214)
static void C_ccall trf_17214(C_word c,C_word *av) C_noret;
static void C_ccall trf_17214(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17214(t0,t1,t2);}

C_noret_decl(trf_17320)
static void C_ccall trf_17320(C_word c,C_word *av) C_noret;
static void C_ccall trf_17320(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17320(t0,t1,t2);}

C_noret_decl(trf_17338)
static void C_ccall trf_17338(C_word c,C_word *av) C_noret;
static void C_ccall trf_17338(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17338(t0,t1,t2);}

C_noret_decl(trf_17398)
static void C_ccall trf_17398(C_word c,C_word *av) C_noret;
static void C_ccall trf_17398(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17398(t0,t1,t2);}

C_noret_decl(trf_17500)
static void C_ccall trf_17500(C_word c,C_word *av) C_noret;
static void C_ccall trf_17500(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17500(t0,t1,t2,t3);}

C_noret_decl(trf_17503)
static void C_ccall trf_17503(C_word c,C_word *av) C_noret;
static void C_ccall trf_17503(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17503(t0,t1,t2);}

C_noret_decl(trf_17560)
static void C_ccall trf_17560(C_word c,C_word *av) C_noret;
static void C_ccall trf_17560(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17560(t0,t1);}

C_noret_decl(trf_17662)
static void C_ccall trf_17662(C_word c,C_word *av) C_noret;
static void C_ccall trf_17662(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17662(t0,t1,t2);}

C_noret_decl(trf_17684)
static void C_ccall trf_17684(C_word c,C_word *av) C_noret;
static void C_ccall trf_17684(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17684(t0,t1,t2);}

C_noret_decl(trf_17718)
static void C_ccall trf_17718(C_word c,C_word *av) C_noret;
static void C_ccall trf_17718(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17718(t0,t1,t2);}

C_noret_decl(trf_17848)
static void C_ccall trf_17848(C_word c,C_word *av) C_noret;
static void C_ccall trf_17848(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17848(t0,t1,t2,t3);}

C_noret_decl(trf_18029)
static void C_ccall trf_18029(C_word c,C_word *av) C_noret;
static void C_ccall trf_18029(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18029(t0,t1,t2);}

C_noret_decl(trf_18078)
static void C_ccall trf_18078(C_word c,C_word *av) C_noret;
static void C_ccall trf_18078(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18078(t0,t1,t2);}

C_noret_decl(trf_18122)
static void C_ccall trf_18122(C_word c,C_word *av) C_noret;
static void C_ccall trf_18122(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18122(t0,t1);}

C_noret_decl(trf_18175)
static void C_ccall trf_18175(C_word c,C_word *av) C_noret;
static void C_ccall trf_18175(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18175(t0,t1,t2);}

C_noret_decl(trf_18306)
static void C_ccall trf_18306(C_word c,C_word *av) C_noret;
static void C_ccall trf_18306(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18306(t0,t1);}

C_noret_decl(trf_18352)
static void C_ccall trf_18352(C_word c,C_word *av) C_noret;
static void C_ccall trf_18352(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18352(t0,t1,t2);}

C_noret_decl(trf_18365)
static void C_ccall trf_18365(C_word c,C_word *av) C_noret;
static void C_ccall trf_18365(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18365(t0,t1);}

C_noret_decl(trf_18400)
static void C_ccall trf_18400(C_word c,C_word *av) C_noret;
static void C_ccall trf_18400(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18400(t0,t1);}

C_noret_decl(trf_18536)
static void C_ccall trf_18536(C_word c,C_word *av) C_noret;
static void C_ccall trf_18536(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18536(t0,t1,t2,t3);}

C_noret_decl(trf_18561)
static void C_ccall trf_18561(C_word c,C_word *av) C_noret;
static void C_ccall trf_18561(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18561(t0,t1,t2);}

C_noret_decl(trf_18615)
static void C_ccall trf_18615(C_word c,C_word *av) C_noret;
static void C_ccall trf_18615(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18615(t0,t1,t2);}

C_noret_decl(trf_18698)
static void C_ccall trf_18698(C_word c,C_word *av) C_noret;
static void C_ccall trf_18698(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18698(t0,t1);}

C_noret_decl(trf_18748)
static void C_ccall trf_18748(C_word c,C_word *av) C_noret;
static void C_ccall trf_18748(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18748(t0,t1,t2);}

C_noret_decl(trf_18909)
static void C_ccall trf_18909(C_word c,C_word *av) C_noret;
static void C_ccall trf_18909(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18909(t0,t1,t2);}

C_noret_decl(trf_18913)
static void C_ccall trf_18913(C_word c,C_word *av) C_noret;
static void C_ccall trf_18913(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18913(t0,t1);}

C_noret_decl(trf_18948)
static void C_ccall trf_18948(C_word c,C_word *av) C_noret;
static void C_ccall trf_18948(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18948(t0,t1,t2);}

C_noret_decl(trf_18980)
static void C_ccall trf_18980(C_word c,C_word *av) C_noret;
static void C_ccall trf_18980(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18980(t0,t1,t2);}

C_noret_decl(trf_19057)
static void C_ccall trf_19057(C_word c,C_word *av) C_noret;
static void C_ccall trf_19057(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19057(t0,t1,t2);}

C_noret_decl(trf_19069)
static void C_ccall trf_19069(C_word c,C_word *av) C_noret;
static void C_ccall trf_19069(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19069(t0,t1,t2);}

C_noret_decl(trf_19080)
static void C_ccall trf_19080(C_word c,C_word *av) C_noret;
static void C_ccall trf_19080(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19080(t0,t1,t2);}

C_noret_decl(trf_19147)
static void C_ccall trf_19147(C_word c,C_word *av) C_noret;
static void C_ccall trf_19147(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19147(t0,t1,t2);}

C_noret_decl(trf_19181)
static void C_ccall trf_19181(C_word c,C_word *av) C_noret;
static void C_ccall trf_19181(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19181(t0,t1,t2);}

C_noret_decl(trf_19246)
static void C_ccall trf_19246(C_word c,C_word *av) C_noret;
static void C_ccall trf_19246(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19246(t0,t1,t2);}

C_noret_decl(trf_19299)
static void C_ccall trf_19299(C_word c,C_word *av) C_noret;
static void C_ccall trf_19299(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19299(t0,t1,t2);}

C_noret_decl(trf_19333)
static void C_ccall trf_19333(C_word c,C_word *av) C_noret;
static void C_ccall trf_19333(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19333(t0,t1);}

C_noret_decl(trf_19361)
static void C_ccall trf_19361(C_word c,C_word *av) C_noret;
static void C_ccall trf_19361(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19361(t0,t1,t2,t3);}

C_noret_decl(trf_19447)
static void C_ccall trf_19447(C_word c,C_word *av) C_noret;
static void C_ccall trf_19447(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19447(t0,t1);}

C_noret_decl(trf_19459)
static void C_ccall trf_19459(C_word c,C_word *av) C_noret;
static void C_ccall trf_19459(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19459(t0,t1,t2);}

C_noret_decl(trf_19497)
static void C_ccall trf_19497(C_word c,C_word *av) C_noret;
static void C_ccall trf_19497(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_19497(t0,t1,t2,t3,t4);}

C_noret_decl(trf_19747)
static void C_ccall trf_19747(C_word c,C_word *av) C_noret;
static void C_ccall trf_19747(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_19747(t0,t1,t2,t3,t4);}

C_noret_decl(trf_19842)
static void C_ccall trf_19842(C_word c,C_word *av) C_noret;
static void C_ccall trf_19842(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19842(t0,t1);}

C_noret_decl(trf_19906)
static void C_ccall trf_19906(C_word c,C_word *av) C_noret;
static void C_ccall trf_19906(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19906(t0,t1);}

C_noret_decl(trf_19910)
static void C_ccall trf_19910(C_word c,C_word *av) C_noret;
static void C_ccall trf_19910(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19910(t0,t1,t2);}

C_noret_decl(trf_19966)
static void C_ccall trf_19966(C_word c,C_word *av) C_noret;
static void C_ccall trf_19966(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19966(t0,t1);}

C_noret_decl(trf_19976)
static void C_ccall trf_19976(C_word c,C_word *av) C_noret;
static void C_ccall trf_19976(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19976(t0,t1,t2,t3);}

C_noret_decl(trf_20013)
static void C_ccall trf_20013(C_word c,C_word *av) C_noret;
static void C_ccall trf_20013(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20013(t0,t1,t2,t3);}

C_noret_decl(trf_20143)
static void C_ccall trf_20143(C_word c,C_word *av) C_noret;
static void C_ccall trf_20143(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20143(t0,t1);}

C_noret_decl(trf_20220)
static void C_ccall trf_20220(C_word c,C_word *av) C_noret;
static void C_ccall trf_20220(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_20220(t0,t1,t2,t3,t4);}

C_noret_decl(trf_20223)
static void C_ccall trf_20223(C_word c,C_word *av) C_noret;
static void C_ccall trf_20223(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20223(t0,t1);}

C_noret_decl(trf_20232)
static void C_ccall trf_20232(C_word c,C_word *av) C_noret;
static void C_ccall trf_20232(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20232(t0,t1,t2);}

C_noret_decl(trf_20287)
static void C_ccall trf_20287(C_word c,C_word *av) C_noret;
static void C_ccall trf_20287(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20287(t0,t1);}

C_noret_decl(trf_20295)
static void C_ccall trf_20295(C_word c,C_word *av) C_noret;
static void C_ccall trf_20295(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20295(t0,t1,t2);}

C_noret_decl(trf_20329)
static void C_ccall trf_20329(C_word c,C_word *av) C_noret;
static void C_ccall trf_20329(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20329(t0,t1,t2);}

C_noret_decl(trf_20377)
static void C_ccall trf_20377(C_word c,C_word *av) C_noret;
static void C_ccall trf_20377(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20377(t0,t1,t2);}

C_noret_decl(trf_20421)
static void C_ccall trf_20421(C_word c,C_word *av) C_noret;
static void C_ccall trf_20421(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20421(t0,t1,t2,t3);}

C_noret_decl(trf_20435)
static void C_ccall trf_20435(C_word c,C_word *av) C_noret;
static void C_ccall trf_20435(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20435(t0,t1,t2);}

C_noret_decl(trf_20478)
static void C_ccall trf_20478(C_word c,C_word *av) C_noret;
static void C_ccall trf_20478(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20478(t0,t1,t2);}

C_noret_decl(trf_20545)
static void C_ccall trf_20545(C_word c,C_word *av) C_noret;
static void C_ccall trf_20545(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20545(t0,t1,t2);}

C_noret_decl(trf_20581)
static void C_ccall trf_20581(C_word c,C_word *av) C_noret;
static void C_ccall trf_20581(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20581(t0,t1,t2);}

C_noret_decl(trf_20596)
static void C_ccall trf_20596(C_word c,C_word *av) C_noret;
static void C_ccall trf_20596(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20596(t0,t1,t2);}

C_noret_decl(trf_20636)
static void C_ccall trf_20636(C_word c,C_word *av) C_noret;
static void C_ccall trf_20636(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20636(t0,t1);}

C_noret_decl(trf_20662)
static void C_ccall trf_20662(C_word c,C_word *av) C_noret;
static void C_ccall trf_20662(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20662(t0,t1,t2,t3);}

C_noret_decl(trf_20668)
static void C_ccall trf_20668(C_word c,C_word *av) C_noret;
static void C_ccall trf_20668(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20668(t0,t1,t2);}

C_noret_decl(trf_20735)
static void C_ccall trf_20735(C_word c,C_word *av) C_noret;
static void C_ccall trf_20735(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20735(t0,t1,t2);}

C_noret_decl(trf_20788)
static void C_ccall trf_20788(C_word c,C_word *av) C_noret;
static void C_ccall trf_20788(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20788(t0,t1);}

C_noret_decl(trf_20806)
static void C_ccall trf_20806(C_word c,C_word *av) C_noret;
static void C_ccall trf_20806(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20806(t0,t1,t2);}

C_noret_decl(trf_20867)
static void C_ccall trf_20867(C_word c,C_word *av) C_noret;
static void C_ccall trf_20867(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20867(t0,t1,t2);}

C_noret_decl(trf_20963)
static void C_ccall trf_20963(C_word c,C_word *av) C_noret;
static void C_ccall trf_20963(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20963(t0,t1);}

C_noret_decl(trf_20983)
static void C_ccall trf_20983(C_word c,C_word *av) C_noret;
static void C_ccall trf_20983(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20983(t0,t1,t2);}

C_noret_decl(trf_21017)
static void C_ccall trf_21017(C_word c,C_word *av) C_noret;
static void C_ccall trf_21017(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21017(t0,t1,t2);}

C_noret_decl(trf_21102)
static void C_ccall trf_21102(C_word c,C_word *av) C_noret;
static void C_ccall trf_21102(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21102(t0,t1);}

C_noret_decl(trf_21111)
static void C_ccall trf_21111(C_word c,C_word *av) C_noret;
static void C_ccall trf_21111(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21111(t0,t1,t2,t3);}

C_noret_decl(trf_21142)
static void C_ccall trf_21142(C_word c,C_word *av) C_noret;
static void C_ccall trf_21142(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21142(t0,t1,t2);}

C_noret_decl(trf_21154)
static void C_ccall trf_21154(C_word c,C_word *av) C_noret;
static void C_ccall trf_21154(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21154(t0,t1,t2);}

C_noret_decl(trf_21207)
static void C_ccall trf_21207(C_word c,C_word *av) C_noret;
static void C_ccall trf_21207(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21207(t0,t1);}

C_noret_decl(trf_21231)
static void C_ccall trf_21231(C_word c,C_word *av) C_noret;
static void C_ccall trf_21231(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21231(t0,t1);}

C_noret_decl(trf_21261)
static void C_ccall trf_21261(C_word c,C_word *av) C_noret;
static void C_ccall trf_21261(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21261(t0,t1,t2);}

C_noret_decl(trf_21311)
static void C_ccall trf_21311(C_word c,C_word *av) C_noret;
static void C_ccall trf_21311(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21311(t0,t1,t2,t3);}

C_noret_decl(trf_21325)
static void C_ccall trf_21325(C_word c,C_word *av) C_noret;
static void C_ccall trf_21325(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21325(t0,t1);}

C_noret_decl(trf_21431)
static void C_ccall trf_21431(C_word c,C_word *av) C_noret;
static void C_ccall trf_21431(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21431(t0,t1);}

C_noret_decl(trf_21482)
static void C_ccall trf_21482(C_word c,C_word *av) C_noret;
static void C_ccall trf_21482(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21482(t0,t1);}

C_noret_decl(trf_21542)
static void C_ccall trf_21542(C_word c,C_word *av) C_noret;
static void C_ccall trf_21542(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21542(t0,t1);}

C_noret_decl(trf_21545)
static void C_ccall trf_21545(C_word c,C_word *av) C_noret;
static void C_ccall trf_21545(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21545(t0,t1);}

C_noret_decl(trf_21559)
static void C_ccall trf_21559(C_word c,C_word *av) C_noret;
static void C_ccall trf_21559(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21559(t0,t1,t2);}

C_noret_decl(trf_21675)
static void C_ccall trf_21675(C_word c,C_word *av) C_noret;
static void C_ccall trf_21675(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21675(t0,t1);}

C_noret_decl(trf_21756)
static void C_ccall trf_21756(C_word c,C_word *av) C_noret;
static void C_ccall trf_21756(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21756(t0,t1,t2);}

C_noret_decl(trf_21906)
static void C_ccall trf_21906(C_word c,C_word *av) C_noret;
static void C_ccall trf_21906(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_21906(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_22045)
static void C_ccall trf_22045(C_word c,C_word *av) C_noret;
static void C_ccall trf_22045(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_22045(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_22180)
static void C_ccall trf_22180(C_word c,C_word *av) C_noret;
static void C_ccall trf_22180(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22180(t0,t1,t2,t3);}

C_noret_decl(trf_22242)
static void C_ccall trf_22242(C_word c,C_word *av) C_noret;
static void C_ccall trf_22242(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22242(t0,t1,t2,t3);}

C_noret_decl(trf_22362)
static void C_ccall trf_22362(C_word c,C_word *av) C_noret;
static void C_ccall trf_22362(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22362(t0,t1,t2,t3);}

C_noret_decl(trf_22711)
static void C_ccall trf_22711(C_word c,C_word *av) C_noret;
static void C_ccall trf_22711(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22711(t0,t1,t2);}

C_noret_decl(trf_22850)
static void C_ccall trf_22850(C_word c,C_word *av) C_noret;
static void C_ccall trf_22850(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22850(t0,t1,t2,t3,t4);}

C_noret_decl(trf_23025)
static void C_ccall trf_23025(C_word c,C_word *av) C_noret;
static void C_ccall trf_23025(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23025(t0,t1,t2);}

C_noret_decl(trf_23082)
static void C_ccall trf_23082(C_word c,C_word *av) C_noret;
static void C_ccall trf_23082(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23082(t0,t1,t2);}

C_noret_decl(trf_23143)
static void C_ccall trf_23143(C_word c,C_word *av) C_noret;
static void C_ccall trf_23143(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23143(t0,t1,t2);}

C_noret_decl(trf_23208)
static void C_ccall trf_23208(C_word c,C_word *av) C_noret;
static void C_ccall trf_23208(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23208(t0,t1,t2);}

C_noret_decl(trf_6820)
static void C_ccall trf_6820(C_word c,C_word *av) C_noret;
static void C_ccall trf_6820(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6820(t0,t1,t2,t3);}

C_noret_decl(trf_6864)
static void C_ccall trf_6864(C_word c,C_word *av) C_noret;
static void C_ccall trf_6864(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6864(t0,t1,t2);}

C_noret_decl(trf_6922)
static void C_ccall trf_6922(C_word c,C_word *av) C_noret;
static void C_ccall trf_6922(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_6922(t0,t1,t2,t3,t4);}

C_noret_decl(trf_6955)
static void C_ccall trf_6955(C_word c,C_word *av) C_noret;
static void C_ccall trf_6955(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6955(t0,t1,t2,t3);}

C_noret_decl(trf_6970)
static void C_ccall trf_6970(C_word c,C_word *av) C_noret;
static void C_ccall trf_6970(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6970(t0,t1,t2);}

C_noret_decl(trf_6978)
static void C_ccall trf_6978(C_word c,C_word *av) C_noret;
static void C_ccall trf_6978(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6978(t0,t1,t2,t3);}

C_noret_decl(trf_7010)
static void C_ccall trf_7010(C_word c,C_word *av) C_noret;
static void C_ccall trf_7010(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7010(t0,t1,t2);}

C_noret_decl(trf_7042)
static void C_ccall trf_7042(C_word c,C_word *av) C_noret;
static void C_ccall trf_7042(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7042(t0,t1,t2);}

C_noret_decl(trf_7091)
static void C_ccall trf_7091(C_word c,C_word *av) C_noret;
static void C_ccall trf_7091(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7091(t0,t1,t2);}

C_noret_decl(trf_7125)
static void C_ccall trf_7125(C_word c,C_word *av) C_noret;
static void C_ccall trf_7125(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7125(t0,t1,t2);}

C_noret_decl(trf_7131)
static void C_ccall trf_7131(C_word c,C_word *av) C_noret;
static void C_ccall trf_7131(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7131(t0,t1,t2);}

C_noret_decl(trf_7159)
static void C_ccall trf_7159(C_word c,C_word *av) C_noret;
static void C_ccall trf_7159(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7159(t0,t1,t2);}

C_noret_decl(trf_7165)
static void C_ccall trf_7165(C_word c,C_word *av) C_noret;
static void C_ccall trf_7165(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7165(t0,t1,t2);}

C_noret_decl(trf_7195)
static void C_ccall trf_7195(C_word c,C_word *av) C_noret;
static void C_ccall trf_7195(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7195(t0,t1,t2,t3);}

C_noret_decl(trf_7223)
static void C_ccall trf_7223(C_word c,C_word *av) C_noret;
static void C_ccall trf_7223(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7223(t0,t1,t2);}

C_noret_decl(trf_7251)
static void C_ccall trf_7251(C_word c,C_word *av) C_noret;
static void C_ccall trf_7251(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7251(t0,t1,t2);}

C_noret_decl(trf_7320)
static void C_ccall trf_7320(C_word c,C_word *av) C_noret;
static void C_ccall trf_7320(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7320(t0,t1,t2);}

C_noret_decl(trf_7326)
static void C_ccall trf_7326(C_word c,C_word *av) C_noret;
static void C_ccall trf_7326(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7326(t0,t1,t2);}

C_noret_decl(trf_7409)
static void C_ccall trf_7409(C_word c,C_word *av) C_noret;
static void C_ccall trf_7409(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7409(t0,t1,t2);}

C_noret_decl(trf_7418)
static void C_ccall trf_7418(C_word c,C_word *av) C_noret;
static void C_ccall trf_7418(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7418(t0,t1,t2);}

C_noret_decl(trf_7426)
static void C_ccall trf_7426(C_word c,C_word *av) C_noret;
static void C_ccall trf_7426(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7426(t0,t1,t2,t3);}

C_noret_decl(trf_7491)
static void C_ccall trf_7491(C_word c,C_word *av) C_noret;
static void C_ccall trf_7491(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7491(t0,t1,t2);}

C_noret_decl(trf_7590)
static void C_ccall trf_7590(C_word c,C_word *av) C_noret;
static void C_ccall trf_7590(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7590(t0,t1,t2,t3);}

C_noret_decl(trf_7676)
static void C_ccall trf_7676(C_word c,C_word *av) C_noret;
static void C_ccall trf_7676(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7676(t0,t1,t2,t3);}

C_noret_decl(trf_7701)
static void C_ccall trf_7701(C_word c,C_word *av) C_noret;
static void C_ccall trf_7701(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7701(t0,t1,t2,t3);}

C_noret_decl(trf_7767)
static void C_ccall trf_7767(C_word c,C_word *av) C_noret;
static void C_ccall trf_7767(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7767(t0,t1,t2);}

C_noret_decl(trf_7773)
static void C_ccall trf_7773(C_word c,C_word *av) C_noret;
static void C_ccall trf_7773(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7773(t0,t1,t2);}

C_noret_decl(trf_7809)
static void C_ccall trf_7809(C_word c,C_word *av) C_noret;
static void C_ccall trf_7809(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7809(t0,t1,t2);}

C_noret_decl(trf_7950)
static void C_ccall trf_7950(C_word c,C_word *av) C_noret;
static void C_ccall trf_7950(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7950(t0,t1,t2);}

C_noret_decl(trf_8169)
static void C_ccall trf_8169(C_word c,C_word *av) C_noret;
static void C_ccall trf_8169(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8169(t0,t1,t2);}

C_noret_decl(trf_8260)
static void C_ccall trf_8260(C_word c,C_word *av) C_noret;
static void C_ccall trf_8260(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8260(t0,t1,t2);}

C_noret_decl(trf_8348)
static void C_ccall trf_8348(C_word c,C_word *av) C_noret;
static void C_ccall trf_8348(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8348(t0,t1,t2);}

C_noret_decl(trf_8422)
static void C_ccall trf_8422(C_word c,C_word *av) C_noret;
static void C_ccall trf_8422(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8422(t0,t1,t2);}

C_noret_decl(trf_8461)
static void C_ccall trf_8461(C_word c,C_word *av) C_noret;
static void C_ccall trf_8461(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8461(t0,t1,t2,t3);}

C_noret_decl(trf_8492)
static void C_ccall trf_8492(C_word c,C_word *av) C_noret;
static void C_ccall trf_8492(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_8492(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_8508)
static void C_ccall trf_8508(C_word c,C_word *av) C_noret;
static void C_ccall trf_8508(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8508(t0,t1);}

C_noret_decl(trf_8617)
static void C_ccall trf_8617(C_word c,C_word *av) C_noret;
static void C_ccall trf_8617(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_8617(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_8641)
static void C_ccall trf_8641(C_word c,C_word *av) C_noret;
static void C_ccall trf_8641(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8641(t0,t1,t2);}

C_noret_decl(trf_8707)
static void C_ccall trf_8707(C_word c,C_word *av) C_noret;
static void C_ccall trf_8707(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_8707(t0,t1,t2,t3,t4);}

C_noret_decl(trf_8742)
static void C_ccall trf_8742(C_word c,C_word *av) C_noret;
static void C_ccall trf_8742(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8742(t0,t1);}

C_noret_decl(trf_8779)
static void C_ccall trf_8779(C_word c,C_word *av) C_noret;
static void C_ccall trf_8779(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8779(t0,t1);}

C_noret_decl(trf_8788)
static void C_ccall trf_8788(C_word c,C_word *av) C_noret;
static void C_ccall trf_8788(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8788(t0,t1);}

C_noret_decl(trf_8791)
static void C_ccall trf_8791(C_word c,C_word *av) C_noret;
static void C_ccall trf_8791(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8791(t0,t1);}

C_noret_decl(trf_8807)
static void C_ccall trf_8807(C_word c,C_word *av) C_noret;
static void C_ccall trf_8807(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8807(t0,t1,t2);}

C_noret_decl(trf_8819)
static void C_ccall trf_8819(C_word c,C_word *av) C_noret;
static void C_ccall trf_8819(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8819(t0,t1,t2);}

C_noret_decl(trf_8854)
static void C_ccall trf_8854(C_word c,C_word *av) C_noret;
static void C_ccall trf_8854(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8854(t0,t1);}

C_noret_decl(trf_8922)
static void C_ccall trf_8922(C_word c,C_word *av) C_noret;
static void C_ccall trf_8922(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8922(t0,t1,t2);}

C_noret_decl(trf_9081)
static void C_ccall trf_9081(C_word c,C_word *av) C_noret;
static void C_ccall trf_9081(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9081(t0,t1,t2);}

C_noret_decl(trf_9100)
static void C_ccall trf_9100(C_word c,C_word *av) C_noret;
static void C_ccall trf_9100(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9100(t0,t1);}

C_noret_decl(trf_9104)
static void C_ccall trf_9104(C_word c,C_word *av) C_noret;
static void C_ccall trf_9104(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9104(t0,t1,t2);}

C_noret_decl(trf_9110)
static void C_ccall trf_9110(C_word c,C_word *av) C_noret;
static void C_ccall trf_9110(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9110(t0,t1,t2);}

C_noret_decl(trf_9229)
static void C_ccall trf_9229(C_word c,C_word *av) C_noret;
static void C_ccall trf_9229(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_9229(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_9307)
static void C_ccall trf_9307(C_word c,C_word *av) C_noret;
static void C_ccall trf_9307(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9307(t0,t1,t2);}

C_noret_decl(trf_9346)
static void C_ccall trf_9346(C_word c,C_word *av) C_noret;
static void C_ccall trf_9346(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_9346(t0,t1,t2,t3,t4);}

C_noret_decl(trf_9352)
static void C_ccall trf_9352(C_word c,C_word *av) C_noret;
static void C_ccall trf_9352(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9352(t0,t1,t2);}

C_noret_decl(trf_9379)
static void C_ccall trf_9379(C_word c,C_word *av) C_noret;
static void C_ccall trf_9379(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9379(t0,t1,t2,t3);}

C_noret_decl(trf_9402)
static void C_ccall trf_9402(C_word c,C_word *av) C_noret;
static void C_ccall trf_9402(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9402(t0,t1,t2);}

C_noret_decl(trf_9417)
static void C_ccall trf_9417(C_word c,C_word *av) C_noret;
static void C_ccall trf_9417(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9417(t0,t1,t2);}

C_noret_decl(trf_9475)
static void C_ccall trf_9475(C_word c,C_word *av) C_noret;
static void C_ccall trf_9475(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_9475(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_9642)
static void C_ccall trf_9642(C_word c,C_word *av) C_noret;
static void C_ccall trf_9642(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9642(t0,t1);}

C_noret_decl(trf_9692)
static void C_ccall trf_9692(C_word c,C_word *av) C_noret;
static void C_ccall trf_9692(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9692(t0,t1);}

C_noret_decl(trf_9727)
static void C_ccall trf_9727(C_word c,C_word *av) C_noret;
static void C_ccall trf_9727(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9727(t0,t1,t2,t3);}

C_noret_decl(trf_9821)
static void C_ccall trf_9821(C_word c,C_word *av) C_noret;
static void C_ccall trf_9821(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9821(t0,t1,t2);}

C_noret_decl(trf_9855)
static void C_ccall trf_9855(C_word c,C_word *av) C_noret;
static void C_ccall trf_9855(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9855(t0,t1,t2);}

C_noret_decl(trf_9940)
static void C_ccall trf_9940(C_word c,C_word *av) C_noret;
static void C_ccall trf_9940(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_9940(t0,t1,t2,t3,t4);}

C_noret_decl(trf_9970)
static void C_ccall trf_9970(C_word c,C_word *av) C_noret;
static void C_ccall trf_9970(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9970(t0,t1);}

C_noret_decl(trf_9986)
static void C_ccall trf_9986(C_word c,C_word *av) C_noret;
static void C_ccall trf_9986(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9986(t0,t1);}

/* f25872 in chicken.compiler.scrutinizer#make-list in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f25872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f25872,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f25979 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f25979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f25979,c,av);}
/* scrutinizer.scm:581: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_SCHEME_END_OF_LIST;
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* f25994 in map-loop5615 in k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f25994(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f25994,c,av);}
a=C_alloc(3);
/* scrutinizer.scm:2912: string-add-indent */
f_20545(((C_word*)t0)[2],t1,C_a_i_list(&a,1,lf[316]));}

/* f26001 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f26001(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f26001,c,av);}
a=C_alloc(3);
/* scrutinizer.scm:2912: string-add-indent */
f_20545(((C_word*)t0)[2],t1,C_a_i_list(&a,1,lf[316]));}

/* f26243 in k20641 in map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f26243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f26243,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];
f_20788(t2,C_a_i_list2(&a,2,lf[123],t1));}

/* f26391 in map-loop984 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26391,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_9307(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26456 in map-loop1636 in k10717 */
static void C_ccall f26456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26456,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_11162(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26463 in k10711 in k10705 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26463(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f26463,c,av);}
a=C_alloc(20);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_10719,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=t2,a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],tmp=(C_word)a,a+=20,tmp);
/* scrutinizer.scm:678: procedure-name */
f_15171(t3,t2);}

/* f26609 in k19499 in known-length-vector-index in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f26609,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_i_pairp(t2))){
t3=C_i_car(t2);
t4=C_eqp(lf[50],t3);
if(C_truep(t4)){
t5=C_i_caddr(((C_word*)t0)[2]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19580,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2099: chicken.compiler.support#node-class */
t7=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f26614 in k19598 in vector-ref-result-type in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f26614,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19644,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2114: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f26623 in k19846 */
static void C_ccall f26623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f26623,c,av);}
a=C_alloc(10);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=C_i_caddr(((C_word*)t0)[2]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19925,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t2,a[8]=((C_word*)t0)[8],a[9]=t3,tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2185: chicken.compiler.support#node-class */
t5=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f26630 in lp in derive-result-type in append-special-case in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f26630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,4)))){
C_save_and_reclaim((void *)f26630,c,av);}
a=C_alloc(33);
t2=C_i_car(t1);
t3=C_i_pairp(t2);
t4=(C_truep(t3)?C_eqp(C_u_i_car(t2),lf[47]):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19998,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t6=C_u_i_cdr(((C_word*)t0)[3]);
t7=C_s_a_i_plus(&a,2,((C_word*)t0)[4],C_fix(1));
/* scrutinizer.scm:2254: lp */
t8=((C_word*)((C_word*)t0)[5])[1];
f_19976(t8,t5,t6,t7);}
else{
t5=C_i_pairp(t2);
t6=(C_truep(t5)?C_eqp(C_u_i_car(t2),lf[361]):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20057,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t8=C_u_i_cdr(((C_word*)t0)[3]);
t9=C_s_a_i_plus(&a,2,((C_word*)t0)[4],C_fix(1));
/* scrutinizer.scm:2262: lp */
t10=((C_word*)((C_word*)t0)[5])[1];
f_19976(t10,t7,t8,t9);}
else{
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20074,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=C_i_nullp(C_u_i_cdr(((C_word*)t0)[3]));
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20080,a[2]=((C_word*)t0)[2],a[3]=t7,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[8],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
if(C_truep(t8)){
t10=t9;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t8;
f_20080(2,av2);}}
else{
/* scrutinizer.scm:2243: match-types */
t10=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
av2[3]=lf[47];
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}}}}

/* f26669 in k20129 */
static void C_ccall f26669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f26669,c,av);}
t2=((C_word*)t0)[2];
f_20143(t2,C_i_car(t1));}

/* f26674 in k22942 in a22939 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f26674(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f26674,c,av);}
a=C_alloc(4);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_i_pairp(t2))){
t3=C_i_car(t2);
t4=C_eqp(t3,lf[47]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22985,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2238: scheme#reverse */
t6=*((C_word*)lf[367]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_u_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f26679 in map-loop4578 in a23001 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26679,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_23025(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26686 in map-loop4544 in a23058 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26686(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26686,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_23082(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26693 in map-loop4510 in a23115 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26693(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26693,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_23143(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26700 in map-loop4476 in a23180 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f26700,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t6=((C_word*)((C_word*)t0)[3])[1];
f_23208(t6,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* f26707 in k23287 in k23278 in a23275 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f26707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f26707,c,av);}
a=C_alloc(8);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23327,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2129: type-always-immediate? */
t4=lf[27];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
f_8089(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f27984 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f27984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f27984,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* f27989 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f27989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f27989,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* f28008 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f28008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f28008,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* f28236 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f28236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f28236,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* f28241 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f28241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f28241,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* f28246 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f28246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f28246,c,av);}
/* scrutinizer.scm:2521: fragment */
f_21102(((C_word*)t0)[2],C_i_car(t1));}

/* k10001 in k9984 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10003(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10003,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:518: chicken.compiler.support#db-get */
t4=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=t2;
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10011 in k10015 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10013,c,av);}
t2=((C_word*)t0)[2];
f_9986(t2,C_i_not(t1));}

/* k10015 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10017,c,av);}
a=C_alloc(3);
t2=C_eqp(t1,lf[87]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10013,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:516: chicken.compiler.support#db-get */
t4=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=((C_word*)t0)[2];
f_9986(t3,C_SCHEME_FALSE);}}

/* k10019 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10021,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10023,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:513: single */
f_8641(((C_word*)t0)[7],t1,t2);}

/* a10022 in k10019 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_10023,c,av);}
t3=lf[220];
/* scrutinizer.scm:514: g1386 */
t4=lf[220];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=t2;
f_22239(7,av2);}}

/* a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_10045,c,av);}
a=C_alloc(20);
t5=(C_truep(((C_word*)t0)[2])?C_a_i_list1(&a,1,((C_word*)t0)[2]):C_SCHEME_END_OF_LIST);
t6=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_10052,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t5,a[8]=((C_word*)t0)[7],a[9]=t3,a[10]=t2,a[11]=((C_word*)t0)[8],a[12]=((C_word*)t0)[9],a[13]=((C_word*)t0)[10],a[14]=t4,a[15]=t1,a[16]=((C_word*)t0)[11],tmp=(C_word)a,a+=17,tmp);
/* scrutinizer.scm:526: initial-argument-types */
t7=((C_word*)((C_word*)t0)[12])[1];
f_9379(t7,t6,((C_word*)t0)[2],t3);}

/* k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_10052,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10055,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=t1,a[17]=((C_word*)t0)[16],tmp=(C_word)a,a+=18,tmp);
if(C_truep(((C_word*)t0)[14])){
/* scrutinizer.scm:527: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=lf[226];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* scrutinizer.scm:527: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10055(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_10055,c,av);}
a=C_alloc(29);
t2=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_10058,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],tmp=(C_word)a,a+=17,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10227,a[2]=((C_word*)t0)[16],a[3]=t2,a[4]=((C_word*)t0)[17],a[5]=t5,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[14])){
/* scrutinizer.scm:529: chicken.base#butlast */
t8=*((C_word*)lf[225]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t8=t7;{
C_word *av2=av;
av2[0]=t8;
av2[1]=((C_word*)t0)[10];
f_10227(2,av2);}}}

/* k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(49,c,4)))){
C_save_and_reclaim((void *)f_10058,c,av);}
a=C_alloc(49);
t2=(C_truep(((C_word*)t0)[2])?C_SCHEME_UNDEFINED:C_SCHEME_UNDEFINED);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_FALSE;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_END_OF_LIST;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_FALSE;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_FALSE;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_FALSE;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10066,a[2]=t10,a[3]=((C_word*)t0)[3],a[4]=t12,a[5]=((C_word*)t0)[4],a[6]=t14,a[7]=((C_word*)t0)[5],a[8]=t4,a[9]=t6,a[10]=t8,tmp=(C_word)a,a+=11,tmp);
t16=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_10075,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[2],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=t1,tmp=(C_word)a,a+=15,tmp);
t17=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10205,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=t6,a[5]=((C_word*)t0)[4],a[6]=t8,a[7]=((C_word*)t0)[5],a[8]=t10,a[9]=t12,a[10]=t14,tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:534: ##sys#dynamic-wind */
t18=*((C_word*)lf[224]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t18;
av2[1]=((C_word*)t0)[16];
av2[2]=t15;
av2[3]=t16;
av2[4]=t17;
((C_proc)(void*)(*((C_word*)t18+1)))(5,av2);}}

/* a10065 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10066(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10066,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)((C_word*)t0)[5])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[6])+1,((C_word*)((C_word*)t0)[7])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[8])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[5])+1,((C_word*)((C_word*)t0)[9])[1]);
t7=C_mutate(((C_word *)((C_word*)t0)[7])+1,((C_word*)((C_word*)t0)[10])[1]);
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(53,c,2)))){
C_save_and_reclaim((void *)f_10075,c,av);}
a=C_alloc(53);
t2=(
/* scrutinizer.scm:537: tag */
  f_9341(C_a_i(&a,29),((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10082,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t4=C_i_car(((C_word*)t0)[9]);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10192,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=t2,a[5]=((C_word*)t0)[12],a[6]=t3,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[13])){
t6=C_a_i_cons(&a,2,((C_word*)t0)[13],lf[47]);
t7=t5;
f_10192(t7,C_a_i_cons(&a,2,t6,((C_word*)t0)[14]));}
else{
t6=t5;
f_10192(t6,((C_word*)t0)[14]);}}

/* k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,5)))){
C_save_and_reclaim((void *)f_10082,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10089,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10097,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10099,a[2]=t5,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_10099(t7,t3,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k10087 in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10089,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list1(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10095 in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_10097,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* scrutinizer.scm:553: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[223];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10099(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,4)))){
C_save_and_reclaim_args((void *)trf_10099,5,t0,t1,t2,t3,t4);}
a=C_alloc(13);
if(C_truep(C_i_zerop(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10107,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t4,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10180,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t5,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:559: chicken.compiler.support#db-get */
t7=*((C_word*)lf[67]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[5];
av2[3]=C_i_car(t3);
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}}

/* k10105 in loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_10107(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,0,4)))){
C_save_and_reclaim_args((void *)trf_10107,2,t0,t1);}
a=C_alloc(33);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10111,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:558: g1473 */
t3=t2;
f_10111(t3,((C_word*)t0)[6],t1);}
else{
t2=C_i_car(((C_word*)t0)[5]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10158,a[2]=((C_word*)t0)[6],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
/* scrutinizer.scm:573: loop */
t5=((C_word*)((C_word*)t0)[3])[1];
f_10099(t5,t3,t4,C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}}

/* g1473 in k10105 in loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_10111(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,0,4)))){
C_save_and_reclaim_args((void *)trf_10111,3,t0,t1,t2);}
a=C_alloc(33);
t3=C_i_cdr(t2);
t4=C_eqp(t3,lf[53]);
t5=(C_truep(t4)?lf[53]:C_u_i_cdr(t2));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10123,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
/* scrutinizer.scm:569: loop */
t8=((C_word*)((C_word*)t0)[3])[1];
f_10099(t8,t6,t7,C_i_cdr(((C_word*)t0)[4]),C_i_cdr(((C_word*)t0)[5]));}

/* k10121 in g1473 in k10105 in loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_10123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10123,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10156 in k10105 in loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_10158(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10158,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10178 in loop in k10080 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_10180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10180,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
t4=((C_word*)t0)[4];
f_10107(t4,C_i_assoc(t3,((C_word*)((C_word*)t0)[5])[1]));}
else{
t2=((C_word*)t0)[4];
f_10107(t2,C_SCHEME_FALSE);}}

/* k10190 in a10074 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10192(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,7)))){
C_save_and_reclaim_args((void *)trf_10192,2,t0,t1);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_list1(&a,1,((C_word*)t0)[4]);
/* scrutinizer.scm:538: walk */
t4=((C_word*)((C_word*)t0)[5])[1];
f_9475(t4,((C_word*)t0)[6],((C_word*)t0)[7],t1,t2,C_SCHEME_FALSE,t3,C_SCHEME_FALSE);}

/* a10204 in k10056 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10205,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)((C_word*)t0)[5])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[6])+1,((C_word*)((C_word*)t0)[7])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[8])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[5])+1,((C_word*)((C_word*)t0)[9])[1]);
t7=C_mutate(((C_word *)((C_word*)t0)[7])+1,((C_word*)((C_word*)t0)[10])[1]);
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* k10225 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10227(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10227,c,av);}
a=C_alloc(11);
t2=C_i_check_list_2(t1,lf[6]);
t3=C_i_check_list_2(((C_word*)t0)[2],lf[6]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10236,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10238,a[2]=((C_word*)t0)[5],a[3]=t6,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_10238(t8,t4,t1,((C_word*)t0)[2]);}

/* k10234 in k10225 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10236,c,av);}
/* scrutinizer.scm:528: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop1404 in k10225 in k10053 in k10050 in a10044 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10238(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_10238,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_cons(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,7)))){
C_save_and_reclaim((void *)f_10309,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_10312,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t1,a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],tmp=(C_word)a,a+=17,tmp);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10642,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[16],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:578: walk */
t4=((C_word*)((C_word*)t0)[17])[1];
f_9475(t4,t3,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[3],((C_word*)t0)[14],((C_word*)t0)[2],((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,2)))){
C_save_and_reclaim((void *)f_10312,c,av);}
a=C_alloc(25);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],tmp=(C_word)a,a+=18,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10631,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[13])){
/* scrutinizer.scm:582: type-typeenv */
t4=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[13];
f_15910(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f25979,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:583: type-typeenv */
t5=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t1;
f_15910(3,av2);}}}

/* k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,2)))){
C_save_and_reclaim((void *)f_10315,c,av);}
a=C_alloc(32);
t2=C_i_assq(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_10321,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=t2,a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],tmp=(C_word)a,a+=15,tmp);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10579,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[15],a[4]=t3,a[5]=((C_word*)t0)[16],a[6]=((C_word*)t0)[14],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[17],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[14])){
if(C_truep(C_i_not(t2))){
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10602,a[2]=t4,a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[7],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t6=C_eqp(((C_word*)t0)[14],lf[59]);
if(C_truep(t6)){
t7=t5;
f_10602(t7,C_i_not(t6));}
else{
t7=C_i_pairp(((C_word*)t0)[14]);
t8=t5;
f_10602(t8,(C_truep(t7)?C_i_not(C_eqp(C_u_i_car(((C_word*)t0)[14]),lf[59])):C_SCHEME_TRUE));}}
else{
t5=t4;
f_10579(t5,C_SCHEME_FALSE);}}
else{
t5=t4;
f_10579(t5,C_SCHEME_FALSE);}}

/* k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_10321,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10324,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10453,a[2]=t2,a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[12],a[6]=((C_word*)t0)[13],a[7]=((C_word*)t0)[11],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_not(((C_word*)t0)[14]))){
if(C_truep(C_i_not(((C_word*)t0)[6]))){
t4=C_eqp(lf[53],((C_word*)t0)[5]);
if(C_truep(C_i_not(t4))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10572,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:595: chicken.compiler.support#db-get */
t6=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[11];
av2[3]=((C_word*)t0)[8];
av2[4]=lf[250];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t3;
f_10453(t5,C_SCHEME_FALSE);}}
else{
t4=t3;
f_10453(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_10453(t4,C_SCHEME_FALSE);}}

/* k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_10324,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10327,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
t3=C_i_cdr(((C_word*)t0)[6]);
t4=C_eqp(lf[35],t3);
t5=(C_truep(t4)?C_i_setslot(((C_word*)t0)[6],C_fix(1),((C_word*)t0)[5]):C_SCHEME_UNDEFINED);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10347,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[9],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[10])){
t7=t6;
f_10347(t7,(C_truep(((C_word*)t0)[10])?((C_word*)t0)[5]:lf[53]));}
else{
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10438,a[2]=t6,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:623: chicken.compiler.support#db-get */
t8=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[11];
av2[3]=((C_word*)t0)[8];
av2[4]=lf[232];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_10327(2,av2);}}}

/* k10325 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10327(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10327,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10333,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:657: type-always-immediate? */
t3=lf[27];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
f_8089(3,av2);}}

/* k10331 in k10325 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,1)))){
C_save_and_reclaim((void *)f_10333,c,av);}
a=C_alloc(58);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t2,C_fix(1)));
t4=C_i_set_cdr(((C_word*)t0)[3],lf[230]);
t5=lf[16];
t6=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[231];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[231];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10345 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10347(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_10347,2,t0,t1);}
a=C_alloc(9);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10355,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=t4,tmp=(C_word)a,a+=7,tmp));
t6=((C_word*)t4)[1];
f_10355(t6,((C_word*)t0)[5],((C_word*)((C_word*)t0)[4])[1],C_SCHEME_FALSE);}

/* loop in k10345 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10355(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(17,0,3)))){
C_save_and_reclaim_args((void *)trf_10355,4,t0,t1,t2,t3);}
a=C_alloc(17);
if(C_truep(C_i_nullp(t2))){
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=((C_word*)((C_word*)t0)[4])[1];
t6=C_a_i_cons(&a,2,t4,((C_word*)t0)[5]);
t7=C_a_i_cons(&a,2,t6,t5);
t8=C_mutate(((C_word *)((C_word*)t0)[4])+1,t7);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t4=C_i_car(t2);
t5=C_i_caar(t4);
t6=C_eqp(((C_word*)t0)[2],t5);
if(C_truep(t6)){
t7=C_u_i_car(t2);
t8=C_i_cdr(t7);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10389,a[2]=t7,a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
t10=C_a_i_list(&a,3,lf[28],((C_word*)t0)[5],t8);
/* scrutinizer.scm:650: simplify-type */
f_13503(t9,t10);}
else{
/* scrutinizer.scm:655: loop */
t12=t1;
t13=C_u_i_cdr(t2);
t14=t3;
t1=t12;
t2=t13;
t3=t14;
goto loop;}}}

/* k10387 in loop in k10345 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_10389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10389,c,av);}
t2=C_i_set_cdr(((C_word*)t0)[2],t1);
t3=C_u_i_cdr(((C_word*)t0)[3]);
if(C_truep(((C_word*)t0)[4])){
/* scrutinizer.scm:654: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_10355(t4,((C_word*)t0)[6],t3,((C_word*)t0)[4]);}
else{
t4=C_i_cdar(((C_word*)t0)[2]);
/* scrutinizer.scm:654: loop */
t5=((C_word*)((C_word*)t0)[5])[1];
f_10355(t5,((C_word*)t0)[6],t3,C_eqp(((C_word*)t0)[7],t4));}}

/* k10436 in k10322 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10438,c,av);}
t2=C_i_not(t1);
t3=((C_word*)t0)[2];
f_10347(t3,(C_truep(t2)?((C_word*)t0)[3]:lf[53]));}

/* k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10453(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_10453,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:596: chicken.compiler.support#db-get */
t3=*((C_word*)lf[67]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[249];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_10324(2,av2);}}}

/* k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_10456,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10459,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_10459(2,av2);}}
else{
/* scrutinizer.scm:597: chicken.compiler.support#db-get */
t3=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[248];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10459,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10468,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_i_car(((C_word*)t0)[5]);
t4=C_eqp(t1,t3);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10543,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:599: chicken.compiler.support#variable-visible? */
t6=*((C_word*)lf[247]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=t2;
f_10468(t5,C_SCHEME_FALSE);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10324(2,av2);}}}

/* k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_10468(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_10468,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10519,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:602: chicken.compiler.support#tree-copy */
t3=*((C_word*)lf[244]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_10324(2,av2);}}}

/* k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_10474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_10474,c,av);}
a=C_alloc(10);
t2=C_i_cdar(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10480,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10490,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:605: chicken.base#open-output-string */
t5=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k10478 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_10480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_10480,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10483,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:609: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[233]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[66];
av2[4]=lf[234];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10481 in k10478 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_10483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10483,c,av);}
/* scrutinizer.scm:610: chicken.compiler.support#mark-variable */
t2=*((C_word*)lf[233]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[65];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_10490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10490,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10496,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:605: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[241];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_10496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_10496,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10499,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:605: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10497 in k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_10499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10499,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10502,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:605: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10500 in k10497 in k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_10502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_10502,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10505,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:605: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10503 in k10500 in k10497 in k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_10505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_10505,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10508,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:605: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(41);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10506 in k10503 in k10500 in k10497 in k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_10508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10508,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10511,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:605: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10509 in k10506 in k10503 in k10500 in k10497 in k10494 in k10488 in k10472 in k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in ... */
static void C_ccall f_10511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10511,c,av);}
/* scrutinizer.scm:605: chicken.compiler.support#debugging */
t2=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[237];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10517 in k10466 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_10519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_10519,c,av);}
a=C_alloc(11);
t2=C_a_i_cons(&a,2,C_SCHEME_FALSE,t1);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10474,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:603: smash-component-types! */
f_11647(t4,t3);}

/* k10537 in k10541 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_10539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10539,c,av);}
t2=((C_word*)t0)[2];
f_10468(t2,C_i_not(C_eqp(t1,lf[245])));}

/* k10541 in k10457 in k10454 in k10451 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_10543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_10543,c,av);}
a=C_alloc(3);
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];
f_10468(t3,t2);}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10539,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:600: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[246];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k10570 in k10319 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10572,c,av);}
t2=((C_word*)t0)[2];
f_10453(t2,C_i_not(t1));}

/* k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10579(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_10579,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10582,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[9])){
t3=C_set_block_item(((C_word*)t0)[10],0,C_SCHEME_TRUE);
t4=t2;
f_10582(t4,t3);}
else{
t3=t2;
f_10582(t3,C_SCHEME_UNDEFINED);}}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_10321(2,av2);}}}

/* k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10582(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,14)))){
C_save_and_reclaim_args((void *)trf_10582,2,t0,t1);}
a=C_alloc(15);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_list2(&a,2,((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22815,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2972: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word av2[15];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[254];
av2[3]=lf[255];
av2[4]=lf[256];
av2[5]=lf[257];
av2[6]=lf[258];
av2[7]=lf[259];
av2[8]=lf[260];
av2[9]=lf[261];
av2[10]=lf[262];
av2[11]=lf[263];
av2[12]=lf[264];
av2[13]=lf[265];
av2[14]=lf[266];
((C_proc)(void*)(*((C_word*)t5+1)))(15,av2);}}

/* k10600 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10602(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_10602,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10609,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:589: match-types */
t3=*((C_word*)lf[170]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];
f_10579(t2,C_SCHEME_FALSE);}}

/* k10607 in k10600 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10609,c,av);}
t2=((C_word*)t0)[2];
f_10579(t2,C_i_not(t1));}

/* k10629 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10631,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10635,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:583: type-typeenv */
t3=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
f_15910(3,av2);}}

/* k10633 in k10629 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10635(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10635,c,av);}
/* scrutinizer.scm:581: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10640 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10642,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10644,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:578: single */
f_8641(((C_word*)t0)[7],t1,t2);}

/* a10643 in k10640 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10644(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_10644,c,av);}
t3=lf[267];
/* scrutinizer.scm:579: g1502 */
t4=lf[267];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=C_i_car(((C_word*)t0)[5]);
av2[6]=t2;
f_22359(7,av2);}}

/* k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,4)))){
C_save_and_reclaim((void *)f_10671,c,av);}
a=C_alloc(36);
t2=C_i_length(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10678,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t8=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_10707,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[2],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],a[15]=((C_word*)t0)[16],a[16]=t2,a[17]=((C_word*)t0)[17],a[18]=t5,a[19]=t7,a[20]=t6,tmp=(C_word)a,a+=21,tmp);
t9=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7947,tmp=(C_word)a,a+=2,tmp);
/* mini-srfi-1.scm:225: list-tabulate */
f_7767(t8,t2,t9);}

/* g1595 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_10678(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,7)))){
C_save_and_reclaim_args((void *)trf_10678,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10694,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10698,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=t4,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:672: walk */
t6=((C_word*)((C_word*)t0)[5])[1];
f_9475(t6,t5,t2,((C_word*)t0)[6],((C_word*)t0)[2],C_SCHEME_FALSE,((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k10692 in g1595 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10694,c,av);}
a=C_alloc(6);
t2=C_a_i_list1(&a,1,t1);
t3=C_a_i_list1(&a,1,((C_word*)t0)[2]);
/* scrutinizer.scm:668: chicken.compiler.support#make-node */
t4=*((C_word*)lf[269]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[85];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10696 in g1595 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10698(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10698,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10700,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:671: single */
f_8641(((C_word*)t0)[7],t1,t2);}

/* a10699 in k10696 in g1595 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_10700,c,av);}
t3=lf[270];
/* scrutinizer.scm:673: g1616 */
t4=lf[270];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=t2;
f_21952(7,av2);}}

/* k10705 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,4)))){
C_save_and_reclaim((void *)f_10707,c,av);}
a=C_alloc(26);
t2=C_i_check_list_2(t1,lf[6]);
t3=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10713,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11200,a[2]=((C_word*)t0)[18],a[3]=t5,a[4]=((C_word*)t0)[19],a[5]=((C_word*)t0)[20],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_11200(t7,t3,((C_word*)t0)[12],t1);}

/* k10711 in k10705 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_10713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_10713,c,av);}
a=C_alloc(19);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f26463,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k10717 */
static void C_ccall f_10719(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,3)))){
C_save_and_reclaim((void *)f_10719,c,av);}
a=C_alloc(36);
t2=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_10722,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=lf[25];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11160,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11162,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_11162(t12,t8,((C_word*)t0)[4]);}

/* k10720 in k10717 */
static void C_ccall f_10722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_10722,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_10725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
if(C_truep(((C_word*)t0)[9])){
/* scrutinizer.scm:682: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=lf[276];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_10725(2,av2);}}}

/* k10723 in k10720 in k10717 */
static void C_ccall f_10725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_10725,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_10728,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=t1,a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],tmp=(C_word)a,a+=23,tmp);
if(C_truep(((C_word*)t0)[9])){
/* scrutinizer.scm:683: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=lf[165];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_10728(2,av2);}}}

/* k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,4)))){
C_save_and_reclaim((void *)f_10728,c,av);}
a=C_alloc(28);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10733,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_10739,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=((C_word*)t0)[16],a[15]=t1,a[16]=((C_word*)t0)[17],a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[4],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
/* scrutinizer.scm:684: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[22];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a10732 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10733,c,av);}
/* scrutinizer.scm:685: call-result */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8707(t2,t1,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,7)))){
C_save_and_reclaim((void *)f_10739,c,av);}
a=C_alloc(30);
t4=t2;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10742,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t3)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10787,a[2]=t5,a[3]=t1,a[4]=((C_word*)t0)[6],a[5]=t6,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:695: walk */
t8=((C_word*)((C_word*)t0)[7])[1];
f_9475(t8,t7,((C_word*)t0)[8],((C_word*)t0)[3],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12]);}
else{
t7=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_11143,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[13],a[11]=((C_word*)t0)[14],a[12]=((C_word*)t0)[15],a[13]=((C_word*)t0)[6],a[14]=((C_word*)t0)[16],a[15]=((C_word*)t0)[17],a[16]=t5,a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[19],a[19]=((C_word*)t0)[20],a[20]=t6,a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
/* scrutinizer.scm:702: chicken.compiler.support#node-class */
t8=*((C_word*)lf[88]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}

/* smash in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10742(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_10742,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10749,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_not(((C_word*)t0)[4]))){
t3=C_i_not(((C_word*)t0)[5]);
if(C_truep(t3)){
t4=t2;
f_10749(t4,t3);}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10781,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:690: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[272];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}
else{
t3=t2;
f_10749(t3,C_SCHEME_FALSE);}}

/* k10747 in smash in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10749(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_10749,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10752,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:692: smash-component-types! */
f_11647(t2,((C_word*)t0)[4]);}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10750 in k10747 in smash in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10752,c,av);}
/* scrutinizer.scm:693: smash-component-types! */
f_11647(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);}

/* k10775 in k10779 in smash in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10777,c,av);}
t2=((C_word*)t0)[2];
f_10749(t2,C_i_not(t1));}

/* k10779 in smash in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_10781,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10777,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:691: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[271];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_10749(t2,C_SCHEME_FALSE);}}

/* k10785 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10787(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10787,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10790,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:696: smash */
t3=((C_word*)t0)[5];
f_10742(t3,t2);}

/* k10788 in k10785 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10790(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_10790,c,av);}
a=C_alloc(16);
t2=C_eqp(lf[53],((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10800,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t8=((C_word*)((C_word*)t0)[2])[1];
t9=C_i_check_list_2(t8,lf[6]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10812,a[2]=t5,a[3]=t11,a[4]=t7,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_10812(t13,((C_word*)t0)[3],t8);}}

/* g1691 in k10788 in k10785 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10800(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_10800,3,t0,t1,t2);}
t3=lf[118];
/* scrutinizer.scm:701: g1708 */
t4=lf[118];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
f_16285(4,av2);}}

/* map-loop1685 in k10788 in k10785 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10812(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10812,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10837,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:701: g1691 */
t4=((C_word*)t0)[4];
f_10800(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10835 in map-loop1685 in k10788 in k10785 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10837,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10812(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10854(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,2)))){
C_save_and_reclaim_args((void *)trf_10854,4,t0,t1,t2,t3);}
a=C_alloc(19);
t4=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_11018,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=((C_word*)t0)[12],a[15]=((C_word*)t0)[13],a[16]=t3,a[17]=((C_word*)t0)[14],a[18]=((C_word*)t0)[15],tmp=(C_word)a,a+=19,tmp);
/* scrutinizer.scm:707: chicken.compiler.support#node-class */
t5=*((C_word*)lf[88]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_10867,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10870,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_10870(2,av2);}}
else{
/* scrutinizer.scm:709: chicken.base#alist-ref */
t3=*((C_word*)lf[275]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[18];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_10870,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10873,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],tmp=(C_word)a,a+=18,tmp);
/* scrutinizer.scm:710: resolve */
t3=lf[118];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[17];
av2[3]=((C_word*)t0)[9];
f_16285(4,av2);}}

/* k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_10873,c,av);}
a=C_alloc(22);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_eqp(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10879,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=t1,a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],a[15]=((C_word*)t0)[16],a[16]=((C_word*)t0)[17],a[17]=t3,tmp=(C_word)a,a+=18,tmp);
if(C_truep(((C_word*)t0)[9])){
if(C_truep(((C_word*)t0)[8])){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11003,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:714: chicken.compiler.support#db-get */
t6=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[13];
av2[3]=((C_word*)t0)[7];
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t4;
f_10879(t5,C_SCHEME_FALSE);}}
else{
t5=t4;
f_10879(t5,C_SCHEME_FALSE);}}

/* k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10879(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_10879,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10885,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:719: resolve */
t3=lf[118];{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[8];
f_16285(4,av2);}}
else{
if(C_truep(((C_word*)t0)[2])){
if(C_truep(((C_word*)t0)[9])){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10935,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[10],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10963,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:737: chicken.compiler.support#db-get */
t4=*((C_word*)lf[67]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[12];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10969,a[2]=((C_word*)t0)[13],a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[15],a[5]=((C_word*)t0)[16],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[17])){
/* scrutinizer.scm:747: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[274];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_10969(2,av2);}}}}}

/* k10883 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_10885,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10891,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t3=C_i_car(((C_word*)t0)[6]);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10920,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_not(((C_word*)t0)[2]))){
/* scrutinizer.scm:722: add-to-blist */
t5=((C_word*)((C_word*)t0)[3])[1];
f_9346(t5,t2,((C_word*)t0)[5],t3,t1);}
else{
/* scrutinizer.scm:724: refine-types */
t5=*((C_word*)lf[273]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k10889 in k10883 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_10891,c,av);}
a=C_alloc(12);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10897,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t3=C_a_i_list(&a,2,lf[69],((C_word*)t0)[7]);
/* scrutinizer.scm:730: refine-types */
t4=*((C_word*)lf[273]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10895 in k10889 in k10883 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10897,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:734: add-to-blist */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9346(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_u_i_cdr(((C_word*)t0)[5]),t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k10918 in k10883 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10920,c,av);}
/* scrutinizer.scm:722: add-to-blist */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9346(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k10933 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_10935,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10941,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:742: add-to-blist */
t3=((C_word*)((C_word*)t0)[3])[1];
f_9346(t3,t2,((C_word*)t0)[5],C_i_car(((C_word*)t0)[6]),t1);}

/* k10939 in k10933 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_10941,c,av);}
a=C_alloc(7);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10947,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:744: add-to-blist */
t3=((C_word*)((C_word*)t0)[3])[1];
f_9346(t3,t2,((C_word*)t0)[5],C_i_car(((C_word*)t0)[2]),((C_word*)t0)[6]);}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10945 in k10939 in k10933 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10947,c,av);}
/* scrutinizer.scm:745: add-to-blist */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9346(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6]);}

/* k10961 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10963(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10963,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[53];
f_10935(2,av2);}}
else{
/* scrutinizer.scm:739: refine-types */
t2=*((C_word*)lf[273]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k10967 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10969,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10973,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:716: g1768 */
t3=t2;
f_10973(t3,((C_word*)t0)[6],t1);}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g1768 in k10967 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_10973(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,5)))){
C_save_and_reclaim_args((void *)trf_10973,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10981,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:752: srt */
t4=t2;{
C_word av2[6];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k10979 in g1768 in k10967 in k10877 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_10981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10981,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11001 in k10871 in k10868 in k10865 in k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11003(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11003,c,av);}
t2=C_i_not(t1);
t3=((C_word*)t0)[2];
f_10879(t3,(C_truep(t2)?C_i_not(((C_word*)t0)[3]):C_SCHEME_FALSE));}

/* k11012 in k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11014(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_11014,c,av);}
a=C_alloc(19);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_10867,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
/* scrutinizer.scm:709: blist-type */
t4=((C_word*)((C_word*)t0)[18])[1];
f_8461(t4,t3,t2,((C_word*)t0)[10]);}

/* k11016 in g1722 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_11018,c,av);}
a=C_alloc(19);
t2=C_eqp(lf[87],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_11014,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],tmp=(C_word)a,a+=19,tmp);
/* scrutinizer.scm:708: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11025 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11027,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11030,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:759: smash */
t3=((C_word*)t0)[5];
f_10742(t3,t2);}

/* k11028 in k11025 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_11030,c,av);}
a=C_alloc(16);
t2=C_eqp(lf[53],((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(t2)){
t3=((C_word*)((C_word*)t0)[2])[1];
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11040,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t8=((C_word*)((C_word*)t0)[2])[1];
t9=C_i_check_list_2(t8,lf[6]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11052,a[2]=t5,a[3]=t11,a[4]=t7,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_11052(t13,((C_word*)t0)[3],t8);}}

/* g1792 in k11028 in k11025 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_11040(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_11040,3,t0,t1,t2);}
t3=lf[118];
/* scrutinizer.scm:762: g1809 */
t4=lf[118];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
f_16285(4,av2);}}

/* map-loop1786 in k11028 in k11025 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_11052(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11052,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11077,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:762: g1792 */
t4=((C_word*)t0)[4];
f_11040(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11075 in map-loop1786 in k11028 in k11025 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11077,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11052(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* for-each-loop1721 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_fcall f_11086(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_11086,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11096,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:705: g1722 */
t7=((C_word*)t0)[3];
f_10854(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k11094 in for-each-loop1721 in k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11096(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11096,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11086(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11123 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_11125,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_i_check_list_2(((C_word*)t0)[3],lf[212]);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11027,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11086,a[2]=t6,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_11086(t8,t4,((C_word*)t0)[3],t2);}

/* a11126 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_11127,c,av);}
a=C_alloc(5);
t2=C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],C_fix(1));
/* scrutinizer.scm:758: procedure-argument-types */
f_15336(t1,((C_word*)t0)[3],t2,((C_word*)t0)[4],C_SCHEME_END_OF_LIST);}

/* a11134 in k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11135(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_11135,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_list_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11141 in a10738 in k10726 in k10723 in k10720 in k10717 */
static void C_ccall f_11143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,7)))){
C_save_and_reclaim((void *)f_11143,c,av);}
a=C_alloc(32);
t2=C_eqp(lf[123],t1);
if(C_truep(t2)){
/* scrutinizer.scm:703: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_9475(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}
else{
t3=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_10854,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[12],a[6]=((C_word*)t0)[13],a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[15],a[10]=((C_word*)t0)[16],a[11]=((C_word*)t0)[4],a[12]=((C_word*)t0)[17],a[13]=((C_word*)t0)[6],a[14]=((C_word*)t0)[5],a[15]=((C_word*)t0)[18],tmp=(C_word)a,a+=16,tmp);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11125,a[2]=((C_word*)t0)[19],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[16],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[13],a[7]=((C_word*)t0)[20],a[8]=t3,tmp=(C_word)a,a+=9,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11127,a[2]=((C_word*)t0)[21],a[3]=((C_word*)t0)[19],a[4]=((C_word*)t0)[13],tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11135,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:756: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t4;
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}}

/* k11158 in k10717 */
static void C_ccall f_11160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11160,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[28],t1);
/* scrutinizer.scm:679: type-typeenv */
t3=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
f_15910(3,av2);}}

/* map-loop1636 in k10717 */
static void C_fcall f_11162(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11162,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop1589 in k10705 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11200(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_11200,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11229,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:667: g1595 */
t7=((C_word*)t0)[4];
f_10678(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k11227 in map-loop1589 in k10705 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11229,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11200(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,10)))){
C_save_and_reclaim((void *)f_11258,c,av);}
a=C_alloc(36);
t2=C_eqp(t1,lf[53]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11264,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t2)){
t4=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t5=lf[16];
t6=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(t1))){
t4=C_i_car(((C_word*)t0)[4]);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22599,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2856: scheme#string-append */
t7=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[279];
av2[3]=lf[280];
av2[4]=lf[281];
av2[5]=lf[282];
av2[6]=lf[283];
av2[7]=lf[284];
av2[8]=lf[285];
((C_proc)(void*)(*((C_word*)t7+1)))(9,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11283,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[3],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
t5=C_i_length(t1);
if(C_truep(C_fixnum_greaterp(t5,C_fix(1)))){
t6=C_i_car(((C_word*)t0)[4]);
t7=C_a_i_list1(&a,1,t6);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22621,a[2]=t1,a[3]=t4,a[4]=t7,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2873: scheme#string-append */
t9=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[302];
av2[3]=lf[303];
av2[4]=lf[304];
av2[5]=lf[305];
av2[6]=lf[306];
av2[7]=lf[307];
av2[8]=lf[308];
av2[9]=lf[309];
av2[10]=lf[310];
((C_proc)(void*)(*((C_word*)t9+1)))(11,av2);}}
else{
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_11283(2,av2);}}}}}

/* k11262 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_11264,c,av);}
a=C_alloc(32);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=lf[16];
t4=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_11283,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11289,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(C_i_cadr(((C_word*)t0)[10]))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11314,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:772: compatible-types? */
f_14710(t3,((C_word*)t0)[6],C_i_car(((C_word*)t0)[3]),C_SCHEME_END_OF_LIST);}
else{
t3=t2;
f_11289(t3,C_SCHEME_FALSE);}}

/* k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11289(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,0,2)))){
C_save_and_reclaim_args((void *)trf_11289,2,t0,t1);}
a=C_alloc(32);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11292,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[7])){
t3=C_set_block_item(((C_word*)t0)[8],0,C_SCHEME_TRUE);
t4=t2;
f_11292(t4,t3);}
else{
t3=t2;
f_11292(t3,C_SCHEME_UNDEFINED);}}
else{
t2=C_a_i_list1(&a,1,((C_word*)t0)[6]);
t3=lf[16];
t4=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t5=((C_word*)t0)[9];{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k11290 in k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11292(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,14)))){
C_save_and_reclaim_args((void *)trf_11292,2,t0,t1);}
a=C_alloc(11);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_car(((C_word*)t0)[3]);
t4=C_a_i_list1(&a,1,t2);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22647,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word*)t0)[5],a[5]=t3,a[6]=((C_word*)t0)[6],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2893: scheme#string-append */
t6=*((C_word*)lf[76]+1);{
C_word av2[15];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[287];
av2[3]=lf[288];
av2[4]=lf[289];
av2[5]=lf[290];
av2[6]=lf[291];
av2[7]=lf[292];
av2[8]=lf[293];
av2[9]=lf[294];
av2[10]=lf[295];
av2[11]=lf[296];
av2[12]=lf[297];
av2[13]=lf[298];
av2[14]=lf[299];
((C_proc)(void*)(*((C_word*)t6+1)))(15,av2);}}

/* k11312 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11314,c,av);}
t2=((C_word*)t0)[2];
f_11289(t2,C_i_not(t1));}

/* k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_11346,c,av);}
a=C_alloc(14);
t2=lf[24];
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_11349,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=t2,a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:779: type-typeenv */
t4=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t1);
f_15910(3,av2);}}

/* k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_11349,c,av);}
a=C_alloc(16);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_i_cdr(((C_word*)t0)[3]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_11362,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=t5,a[12]=((C_word*)t0)[12],a[13]=t1,tmp=(C_word)a,a+=14,tmp));
t7=((C_word*)t5)[1];
f_11362(t7,((C_word*)t0)[13],t2,t3);}

/* loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11362(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,18)))){
C_save_and_reclaim_args((void *)trf_11362,4,t0,t1,t2,t3);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
t4=C_i_car(((C_word*)t0)[2]);
t5=C_i_cdr(((C_word*)t0)[3]);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22679,a[2]=t1,a[3]=t5,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2914: scheme#string-append */
t7=*((C_word*)lf[76]+1);{
C_word av2[19];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[324];
av2[3]=lf[325];
av2[4]=lf[326];
av2[5]=lf[327];
av2[6]=lf[328];
av2[7]=lf[329];
av2[8]=lf[330];
av2[9]=lf[331];
av2[10]=lf[332];
av2[11]=lf[333];
av2[12]=lf[334];
av2[13]=lf[335];
av2[14]=lf[336];
av2[15]=lf[337];
av2[16]=lf[338];
av2[17]=lf[339];
av2[18]=lf[340];
((C_proc)(void*)(*((C_word*)t7+1)))(19,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_11383,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=t3,a[11]=((C_word*)t0)[11],a[12]=t2,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[2],tmp=(C_word)a,a+=15,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11421,a[2]=t4,a[3]=((C_word*)t0)[13],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:784: type-typeenv */
t6=lf[161];{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_i_car(t2);
f_15910(3,av2);}}}

/* k11381 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11383(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,5)))){
C_save_and_reclaim((void *)f_11383,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_11389,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=t1,tmp=(C_word)a,a+=15,tmp);
/* scrutinizer.scm:785: match-types */
t3=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[12]);
av2[3]=C_i_car(((C_word*)t0)[14]);
av2[4]=t1;
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k11387 in k11381 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_11389,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11392,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:787: mutate-node! */
f_17320(t2,((C_word*)t0)[4],C_i_car(((C_word*)t0)[10]));}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11402,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:790: trail-restore */
f_16242(t2,((C_word*)t0)[13],((C_word*)t0)[14]);}}

/* k11390 in k11387 in k11381 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_11392,c,av);}
/* scrutinizer.scm:788: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9475(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k11400 in k11387 in k11381 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11402,c,av);}
/* scrutinizer.scm:791: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11362(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_i_cdr(((C_word*)t0)[5]));}

/* k11419 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11421(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11421,c,av);}
/* scrutinizer.scm:784: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* g1850 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11446(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,7)))){
C_save_and_reclaim_args((void *)trf_11446,3,t0,t1,t2);}
/* scrutinizer.scm:795: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_9475(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,((C_word*)t0)[5],C_SCHEME_FALSE);}

/* k11454 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_11456,c,av);}
a=C_alloc(29);
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* for-each-loop1849 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11458(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11458,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11468,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:795: g1850 */
t4=((C_word*)t0)[3];
f_11446(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11466 in for-each-loop1849 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11468,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11458(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_11529,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11532,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_pairp(lf[23]))){
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11570,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:803: chicken.compiler.support#with-debugging-output */
t4=*((C_word*)lf[357]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[358];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_11532(2,av2);}}}

/* k11530 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_11532,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11535,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=((C_word*)((C_word*)t0)[7])[1];
if(C_truep(C_i_positivep(t3))){
/* scrutinizer.scm:812: chicken.compiler.support#debugging */
t4=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[350];
av2[3]=lf[351];
av2[4]=((C_word*)((C_word*)t0)[7])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_11535(2,av2);}}}

/* k11533 in k11530 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11535,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11538,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)((C_word*)t0)[6])[1];
if(C_truep(C_i_positivep(t3))){
/* scrutinizer.scm:814: chicken.compiler.support#debugging */
t4=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[348];
av2[3]=lf[349];
av2[4]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_11538(2,av2);}}}

/* k11536 in k11533 in k11530 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11538,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11541,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)((C_word*)t0)[5])[1];
if(C_truep(C_i_positivep(t3))){
/* scrutinizer.scm:816: chicken.compiler.support#debugging */
t4=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[346];
av2[3]=lf[347];
av2[4]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_11541(2,av2);}}}

/* k11539 in k11536 in k11533 in k11530 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11541,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11547,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)((C_word*)t0)[4])[1])){
/* scrutinizer.scm:819: chicken.compiler.support#quit-compiling */
t3=*((C_word*)lf[312]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[345];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11545 in k11539 in k11536 in k11533 in k11530 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11547,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11570,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11574,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:806: chicken.base#print */
t3=*((C_word*)lf[355]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[356];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11574,c,av);}
a=C_alloc(5);
t2=lf[23];
t3=C_i_check_list_2(lf[23],lf[212]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11608,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_11608(t7,((C_word*)t0)[2],lf[23]);}

/* k11580 in for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11582(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11582,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11585,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:809: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cdr(((C_word*)t0)[4]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11583 in k11580 in for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11585,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11588,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:809: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(32);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11586 in k11583 in k11580 in for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11588(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11588,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11591,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:809: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[4]);
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11589 in k11586 in k11583 in k11580 in for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11591,c,av);}
/* scrutinizer.scm:809: ##sys#write-char-0 */
t2=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11608(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_11608,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11618,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[352]+1);
t6=*((C_word*)lf[352]+1);
t7=C_i_check_port_2(*((C_word*)lf[352]+1),C_fix(2),C_SCHEME_TRUE,lf[353]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11582,a[2]=t3,a[3]=t5,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:809: ##sys#print */
t9=*((C_word*)lf[240]+1);{
C_word av2[5];
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[354];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[352]+1);
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11616 in for-each-loop1875 in k11572 in a11569 in k11527 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11618,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11608(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k11643 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,7)))){
C_save_and_reclaim((void *)f_11645,c,av);}
a=C_alloc(32);
t2=C_i_car(t1);
t3=(
/* scrutinizer.scm:801: tag */
  f_9341(C_a_i(&a,29),((C_word*)((C_word*)t0)[2])[1])
);
t4=C_a_i_list1(&a,1,t3);
/* scrutinizer.scm:801: walk */
t5=((C_word*)((C_word*)t0)[3])[1];
f_9475(t5,((C_word*)t0)[4],t2,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE,t4,C_SCHEME_FALSE);}

/* chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11647(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_11647,2,t1,t2);}
a=C_alloc(5);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11653,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_11653(t6,t1,t2);}

/* doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11653(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,4)))){
C_save_and_reclaim_args((void *)trf_11653,3,t0,t1,t2);}
a=C_alloc(13);
if(C_truep(C_i_nullp(t2))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11663,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_cdar(t2);
t5=C_u_i_car(t2);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11675,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11683,a[2]=t8,tmp=(C_word)a,a+=3,tmp));
t10=((C_word*)t8)[1];
f_11683(t10,t3,t4,t6);}}

/* k11661 in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11663,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11653(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* a11673 in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11675,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_setslot(((C_word*)t0)[2],C_fix(1),t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11683(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(16,0,3)))){
C_save_and_reclaim_args((void *)trf_11683,4,t0,t1,t2,t3);}
a=C_alloc(16);
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_car(t2);
t5=C_eqp(t4,lf[359]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11703,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:837: change! */
t7=t3;{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[50];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=C_eqp(t4,lf[50]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11716,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11728,a[2]=t2,a[3]=t3,a[4]=t7,a[5]=t10,a[6]=t11,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:842: chicken.base#constantly */
t13=*((C_word*)lf[360]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=lf[53];
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t7=C_eqp(t4,lf[361]);
t8=(C_truep(t7)?t7:C_eqp(t4,lf[47]));
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11784,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:846: change! */
t10=t3;{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=lf[362];
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t9=C_eqp(t4,lf[48]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11797,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:850: change! */
t11=t3;{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=C_eqp(t4,lf[29]);
if(C_truep(t10)){
t11=C_i_caddr(t2);
t12=C_i_cddr(t2);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11818,a[2]=t12,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:853: loop */
t15=t1;
t16=t11;
t17=t13;
t1=t15;
t2=t16;
t3=t17;
goto loop;}
else{
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}}}}
else{
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k11701 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11703,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11714 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11716(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11716,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11726 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_11728,c,av);}
a=C_alloc(12);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[6]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11735,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11737,a[2]=((C_word*)t0)[5],a[3]=t6,a[4]=t1,a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_11737(t8,t4,t2);}

/* k11733 in k11726 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11735(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11735,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[50],t1);
/* scrutinizer.scm:842: change! */
t3=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* map-loop1946 in k11726 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11737(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11737,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11762,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:842: g1952 */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11760 in map-loop1946 in k11726 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11762,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11737(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11782 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11784,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11795 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11797,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a11814 in loop in doloop1928 in chicken.compiler.scrutinizer#smash-component-types! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11818(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11818,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_set_car(((C_word*)t0)[2],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +15,c,2)))){
C_save_and_reclaim((void*)f_11843,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+15);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11847,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_nullp(t4))){
t6=C_a_i_list(&a,3,lf[28],t2,t3);
/* scrutinizer.scm:871: type-typeenv */
t7=lf[161];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
f_15910(3,av2);}}
else{
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_i_car(t4);
f_11847(2,av2);}}}

/* k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,4)))){
C_save_and_reclaim((void *)f_11847,c,av);}
a=C_alloc(42);
t2=C_i_nullp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_SCHEME_END_OF_LIST:C_i_cdr(((C_word*)t0)[2]));
t4=C_i_nullp(t3);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_i_car(t3));
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_i_nullp(t3);
t9=(C_truep(t8)?C_SCHEME_END_OF_LIST:C_i_cdr(t3));
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_UNDEFINED;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11858,a[2]=t13,a[3]=t21,tmp=(C_word)a,a+=4,tmp));
t23=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11989,a[2]=t21,tmp=(C_word)a,a+=3,tmp));
t24=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12052,a[2]=t7,a[3]=t15,a[4]=t21,tmp=(C_word)a,a+=5,tmp));
t25=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12120,a[2]=t7,a[3]=t21,tmp=(C_word)a,a+=4,tmp));
t26=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12142,a[2]=t21,tmp=(C_word)a,a+=3,tmp));
t27=C_set_block_item(t21,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12178,a[2]=t1,a[3]=t17,a[4]=t21,a[5]=t7,a[6]=t15,a[7]=t11,a[8]=t19,tmp=(C_word)a,a+=9,tmp));
/* scrutinizer.scm:1106: match1 */
t28=((C_word*)t21)[1];
f_12178(t28,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* match-args in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11858(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,6)))){
C_save_and_reclaim_args((void *)trf_11858,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11867,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_11867(t7,t1,t2,t3,C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* loop in match-args in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11867(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_11867,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_nullp(t3);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=(C_truep(t6)?t6:C_u_i_memq(C_i_car(t3),lf[363]));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
if(C_truep(C_i_nullp(t3))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t4)?t4:C_u_i_memq(C_i_car(t2),lf[363]));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_eqp(lf[364],t6);
if(C_truep(t7)){
/* scrutinizer.scm:884: loop */
t14=t1;
t15=C_u_i_cdr(t2);
t16=t3;
t17=C_SCHEME_TRUE;
t18=t5;
t1=t14;
t2=t15;
t3=t16;
t4=t17;
t5=t18;
goto loop;}
else{
t8=C_i_car(t3);
t9=C_eqp(lf[364],t8);
if(C_truep(t9)){
/* scrutinizer.scm:886: loop */
t14=t1;
t15=t2;
t16=C_u_i_cdr(t3);
t17=t4;
t18=C_SCHEME_TRUE;
t1=t14;
t2=t15;
t3=t16;
t4=t17;
t5=t18;
goto loop;}
else{
t10=C_eqp(lf[365],C_u_i_car(t2));
if(C_truep(t10)){
t11=(
/* scrutinizer.scm:888: rest-type */
  f_15740(C_u_i_cdr(t2))
);
/* scrutinizer.scm:888: match-rest */
t12=((C_word*)((C_word*)t0)[3])[1];
f_11989(t12,t1,t11,t3);}
else{
t11=C_eqp(lf[365],C_u_i_car(t3));
if(C_truep(t11)){
t12=(
/* scrutinizer.scm:890: rest-type */
  f_15740(C_u_i_cdr(t3))
);
/* scrutinizer.scm:890: match-rest */
t13=((C_word*)((C_word*)t0)[3])[1];
f_11989(t13,t1,t12,t2);}
else{
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11964,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:891: match1 */
t13=((C_word*)((C_word*)t0)[4])[1];
f_12178(t13,t12,C_u_i_car(t2),C_u_i_car(t3));}}}}}}}

/* k11962 in loop in match-args in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11964,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:892: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11867(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_11989(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_11989,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11995,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12011,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:896: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
C_call_with_values(4,av2);}}

/* a11994 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_11995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11995,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6820,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_6820(t5,t1,((C_word*)t0)[2],C_SCHEME_END_OF_LIST);}

/* a12010 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12011,c,av);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12018,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12035,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:897: every */
f_7125(t4,t5,t2);}

/* k12016 in a12010 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12018,c,av);}
if(C_truep(t1)){
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t2=(
/* scrutinizer.scm:902: rest-type */
  f_15740(C_u_i_cdr(((C_word*)t0)[2]))
);
/* scrutinizer.scm:902: match1 */
t3=((C_word*)((C_word*)t0)[3])[1];
f_12178(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}
else{
/* scrutinizer.scm:902: match1 */
t2=((C_word*)((C_word*)t0)[3])[1];
f_12178(t2,((C_word*)t0)[4],((C_word*)t0)[5],lf[53]);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a12034 in a12010 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12035,c,av);}
t3=C_eqp(lf[364],t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* scrutinizer.scm:900: match1 */
t4=((C_word*)((C_word*)t0)[2])[1];
f_12178(t4,t1,((C_word*)t0)[3],t2);}}

/* match-results in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12052(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12052,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_eqp(lf[53],t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(lf[53],t3);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_i_not(((C_word*)((C_word*)t0)[2])[1]);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(t2))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_i_nullp(t3);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(t3))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=(C_truep((C_truep(C_eqp(t6,lf[35]))?C_SCHEME_TRUE:(C_truep(C_eqp(t6,lf[70]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))?C_u_i_memq(C_i_car(t3),lf[368]):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12095,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:914: match1 */
t9=((C_word*)((C_word*)t0)[4])[1];
f_12178(t9,t8,C_u_i_car(t2),C_i_car(t3));}}}}}}

/* k12093 in match-results in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12095(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12095,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:915: match-results */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12052(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* rawmatch1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12120(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,4)))){
C_save_and_reclaim_args((void *)trf_12120,4,t0,t1,t2,t3);}
a=C_alloc(19);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12126,a[2]=t7,a[3]=((C_word*)t0)[2],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12131,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12137,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t7,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:919: ##sys#dynamic-wind */
t11=*((C_word*)lf[224]+1);{
C_word av2[5];
av2[0]=t11;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
av2[4]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}

/* a12125 in rawmatch1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12126,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a12130 in rawmatch1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12131(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12131,c,av);}
/* scrutinizer.scm:920: match1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12178(t2,t1,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* a12136 in rawmatch1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12137(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12137,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* every-match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12142(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_12142,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12148,a[2]=t5,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_12148(t7,t1,t2,t3);}

/* loop in every-match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12148(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12148,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_nullp(t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12161,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:925: match1 */
t6=((C_word*)((C_word*)t0)[3])[1];
f_12178(t6,t5,C_i_car(t2),C_i_car(t3));}}

/* k12159 in loop in every-match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12161,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:925: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12148(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12178(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_12178,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=C_eqp(t2,t3);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_symbolp(t2);
t6=(C_truep(t5)?C_i_assq(t2,((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12195,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:931: g2133 */
t8=t7;
f_12195(t8,t1,t6);}
else{
t7=C_i_symbolp(t3);
t8=(C_truep(t7)?C_i_assq(t3,((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12327,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:931: g2166 */
t10=t9;
f_12327(t10,t1,t8);}
else{
t9=C_eqp(t2,lf[53]);
if(C_truep(t9)){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(t3,lf[53]);
if(C_truep(t10)){
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_i_not(((C_word*)((C_word*)t0)[5])[1]);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(t2,lf[35]);
if(C_truep(t11)){
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=C_eqp(t3,lf[35]);
if(C_truep(t12)){
t13=t1;{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t13=C_eqp(t2,lf[70]);
if(C_truep(t13)){
t14=t1;{
C_word av2[2];
av2[0]=t14;
av2[1]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_eqp(t3,lf[70]);
if(C_truep(t14)){
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t15=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12440,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_symbolp(t2))){
/* scrutinizer.scm:1272: chicken.base#alist-ref */
t16=*((C_word*)lf[275]+1);{
C_word av2[5];
av2[0]=t16;
av2[1]=t15;
av2[2]=t2;
av2[3]=lf[373];
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t16+1)))(5,av2);}}
else{
t16=t15;{
C_word av2[2];
av2[0]=t16;
av2[1]=C_SCHEME_FALSE;
f_12440(2,av2);}}}}}}}}}}}}

/* g2133 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12195(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12195,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_cadr(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12208,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:936: match1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12178(t4,t3,C_i_cadr(t2),((C_word*)t0)[3]);}
else{
t3=C_i_symbolp(((C_word*)t0)[3]);
t4=(C_truep(t3)?C_i_assq(((C_word*)t0)[3],((C_word*)t0)[5]):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12236,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:935: g2147 */
t6=t5;
f_12236(t6,t1,t4);}
else{
t5=C_i_caddr(t2);
t6=C_i_not(t5);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12286,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
if(C_truep(t6)){
t8=t7;{
C_word av2[2];
av2[0]=t8;
av2[1]=t6;
f_12286(2,av2);}}
else{
/* scrutinizer.scm:950: rawmatch1 */
t8=((C_word*)((C_word*)t0)[2])[1];
f_12120(t8,t7,C_i_caddr(t2),((C_word*)t0)[3]);}}}}

/* k12206 in g2133 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12208(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12208,c,av);}
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_i_not(t2);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* scrutinizer.scm:938: rawmatch1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12120(t4,((C_word*)t0)[3],C_i_caddr(((C_word*)t0)[2]),((C_word*)t0)[5]);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g2147 in g2133 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12236(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12236,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_i_cadr(t2);
t4=C_i_not(t3);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12249,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:946: match1 */
t6=((C_word*)((C_word*)t0)[4])[1];
f_12178(t6,t5,((C_word*)t0)[3],C_i_cadr(t2));}}

/* k12247 in g2147 in g2133 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12249(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12249,c,av);}
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_i_not(t2);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* scrutinizer.scm:948: rawmatch1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12120(t4,((C_word*)t0)[3],((C_word*)t0)[5],C_i_caddr(((C_word*)t0)[2]));}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12284 in g2133 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_12286,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],lf[24]);
t3=C_mutate(&lf[24] /* (set! chicken.compiler.scrutinizer#trail ...) */,t2);
t4=C_i_cdr(((C_word*)t0)[3]);
t5=C_i_set_car(t4,((C_word*)t0)[4]);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g2166 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12327(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_12327,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_cadr(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12340,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:959: match1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12178(t4,t3,((C_word*)t0)[3],C_i_cadr(t2));}
else{
t3=C_i_caddr(t2);
t4=C_i_not(t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12370,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
if(C_truep(t4)){
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=t4;
f_12370(2,av2);}}
else{
/* scrutinizer.scm:963: rawmatch1 */
t6=((C_word*)((C_word*)t0)[2])[1];
f_12120(t6,t5,((C_word*)t0)[3],C_i_caddr(t2));}}}

/* k12338 in g2166 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12340,c,av);}
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_i_not(t2);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* scrutinizer.scm:961: rawmatch1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12120(t4,((C_word*)t0)[3],((C_word*)t0)[5],C_i_caddr(((C_word*)t0)[2]));}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12368 in g2166 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_12370,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],lf[24]);
t3=C_mutate(&lf[24] /* (set! chicken.compiler.scrutinizer#trail ...) */,t2);
t4=C_i_cdr(((C_word*)t0)[3]);
t5=C_i_set_car(t4,((C_word*)t0)[4]);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_12440,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12444,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:931: g2194 */
t3=t2;
f_12444(t3,((C_word*)t0)[4],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12454,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
/* scrutinizer.scm:1272: chicken.base#alist-ref */
t3=*((C_word*)lf[275]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[373];
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_12454(2,av2);}}}}

/* g2194 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12444(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_12444,3,t0,t1,t2);}
/* scrutinizer.scm:975: g2202 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,t2,((C_word*)t0)[3]);}

/* k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,5)))){
C_save_and_reclaim((void *)f_12454,c,av);}
a=C_alloc(20);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12458,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:931: g2205 */
t3=t2;
f_12458(t3,((C_word*)t0)[4],t1);}
else{
t2=C_i_pairp(((C_word*)t0)[3]);
t3=(C_truep(t2)?C_eqp(lf[69],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_not(((C_word*)((C_word*)t0)[5])[1]);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_FALSE;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12479,a[2]=t8,a[3]=((C_word*)t0)[5],a[4]=t6,tmp=(C_word)a,a+=5,tmp);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12484,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12500,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=t8,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:978: ##sys#dynamic-wind */
t12=*((C_word*)lf[224]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t12;
av2[1]=((C_word*)t0)[4];
av2[2]=t9;
av2[3]=t10;
av2[4]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}
else{
t4=C_i_pairp(((C_word*)t0)[7]);
t5=(C_truep(t4)?C_eqp(lf[69],C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE);
if(C_truep(t5)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t6=C_SCHEME_TRUE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_FALSE;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12520,a[2]=t9,a[3]=((C_word*)t0)[5],a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12525,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12541,a[2]=t7,a[3]=((C_word*)t0)[5],a[4]=t9,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:985: ##sys#dynamic-wind */
t13=*((C_word*)lf[224]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t13;
av2[1]=((C_word*)t0)[4];
av2[2]=t10;
av2[3]=t11;
av2[4]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}
else{
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t6=C_i_pairp(((C_word*)t0)[7]);
t7=(C_truep(t6)?C_eqp(lf[28],C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=C_i_cdr(((C_word*)t0)[7]);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12559,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:993: over-all-instantiations */
f_20220(((C_word*)t0)[4],t8,((C_word*)t0)[6],((C_word*)((C_word*)t0)[5])[1],t9);}
else{
t8=C_i_pairp(((C_word*)t0)[3]);
t9=(C_truep(t8)?C_eqp(lf[28],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
if(C_truep(t9)){
t10=C_i_cdr(((C_word*)t0)[3]);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12578,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1000: over-all-instantiations */
f_20220(((C_word*)t0)[4],t10,((C_word*)t0)[6],C_SCHEME_FALSE,t11);}
else{
t10=C_i_pairp(((C_word*)t0)[3]);
t11=(C_truep(t10)?C_eqp(lf[29],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
if(C_truep(t11)){
/* scrutinizer.scm:1006: match1 */
t12=((C_word*)((C_word*)t0)[2])[1];
f_12178(t12,((C_word*)t0)[4],C_i_caddr(((C_word*)t0)[3]),((C_word*)t0)[7]);}
else{
t12=C_i_pairp(((C_word*)t0)[7]);
t13=(C_truep(t12)?C_eqp(lf[29],C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE);
if(C_truep(t13)){
/* scrutinizer.scm:1008: match1 */
t14=((C_word*)((C_word*)t0)[2])[1];
f_12178(t14,((C_word*)t0)[4],((C_word*)t0)[3],C_i_caddr(((C_word*)t0)[7]));}
else{
t14=C_eqp(lf[84],((C_word*)t0)[3]);
if(C_truep(t14)){
t15=C_i_pairp(((C_word*)t0)[7]);
t16=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t16;
av2[1]=(C_truep(t15)?C_eqp(lf[84],C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}
else{
t15=C_eqp(lf[84],((C_word*)t0)[7]);
if(C_truep(t15)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t16=C_i_pairp(((C_word*)t0)[3]);
t17=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t17;
av2[1]=(C_truep(t16)?C_eqp(lf[84],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
t16=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t16;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}}
else{
t16=C_eqp(((C_word*)t0)[3],lf[31]);
if(C_truep(t16)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t17=C_i_pairp(((C_word*)t0)[7]);
t18=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t18;
av2[1]=(C_truep(t17)?C_eqp(lf[361],C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
t17=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t17;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}}
else{
t17=C_eqp(((C_word*)t0)[7],lf[31]);
if(C_truep(t17)){
t18=C_i_pairp(((C_word*)t0)[3]);
t19=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t19;
av2[1]=(C_truep(t18)?C_eqp(lf[361],C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t18=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12694,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t19=C_i_pairp(((C_word*)t0)[7]);
t20=t18;
f_12694(t20,(C_truep(t19)?C_eqp(C_u_i_car(((C_word*)t0)[3]),C_u_i_car(((C_word*)t0)[7])):C_SCHEME_FALSE));}
else{
t19=t18;
f_12694(t19,C_SCHEME_FALSE);}}}}}}}}}}}}}

/* g2205 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12458(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_12458,3,t0,t1,t2);}
/* scrutinizer.scm:976: g2213 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,((C_word*)t0)[3],t2);}

/* a12478 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12479,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a12483 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12484,c,av);}
a=C_alloc(5);
t2=lf[24];
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12488,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:980: match1 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_12178(t4,t3,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k12486 in a12483 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12488,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12491,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:981: trail-restore */
f_16242(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k12489 in k12486 in a12483 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12491,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a12499 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12500,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a12519 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12520,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a12524 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12525,c,av);}
a=C_alloc(5);
t2=lf[24];
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12529,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:987: match1 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_12178(t4,t3,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* k12527 in a12524 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_12529,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12532,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:988: trail-restore */
f_16242(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k12530 in k12527 in a12524 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12532,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a12540 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12541,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a12558 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12559,c,av);}
/* scrutinizer.scm:997: match1 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,((C_word*)t0)[3],t2);}

/* a12577 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12578,c,av);}
/* scrutinizer.scm:1004: match1 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,t2,((C_word*)t0)[3]);}

/* k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_12694(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,5)))){
C_save_and_reclaim_args((void *)trf_12694,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_eqp(t2,lf[84]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12706,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1022: procedure-arguments */
f_15222(t4,((C_word*)t0)[2]);}
else{
t4=C_eqp(t2,lf[52]);
if(C_truep(t4)){
t5=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t5;
av2[1]=C_i_equalp(((C_word*)t0)[2],((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(t2,lf[48]);
if(C_truep(t5)){
/* scrutinizer.scm:1029: every-match1 */
t6=((C_word*)((C_word*)t0)[7])[1];
f_12142(t6,((C_word*)t0)[4],C_u_i_cdr(((C_word*)t0)[2]),C_i_cdr(((C_word*)t0)[6]));}
else{
t6=C_eqp(t2,lf[361]);
t7=(C_truep(t6)?t6:C_eqp(t2,lf[359]));
if(C_truep(t7)){
/* scrutinizer.scm:1030: match1 */
t8=((C_word*)((C_word*)t0)[8])[1];
f_12178(t8,((C_word*)t0)[4],C_i_cadr(((C_word*)t0)[2]),C_i_cadr(((C_word*)t0)[6]));}
else{
t8=C_eqp(t2,lf[47]);
t9=(C_truep(t8)?t8:C_eqp(t2,lf[50]));
if(C_truep(t9)){
t10=C_i_length(((C_word*)t0)[2]);
t11=C_i_length(((C_word*)t0)[6]);
t12=C_eqp(t10,t11);
if(C_truep(t12)){
/* scrutinizer.scm:1033: every-match1 */
t13=((C_word*)((C_word*)t0)[7])[1];
f_12142(t13,((C_word*)t0)[4],C_i_cdr(((C_word*)t0)[2]),C_i_cdr(((C_word*)t0)[6]));}
else{
t13=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}
else{
t10=C_eqp(t2,lf[372]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12814,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1035: match1 */
t12=((C_word*)((C_word*)t0)[8])[1];
f_12178(t12,t11,C_i_caddr(((C_word*)t0)[2]),C_i_caddr(((C_word*)t0)[6]));}
else{
t11=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}}}}}
else{
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_eqp(lf[372],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t3)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[9])[1]))){
/* scrutinizer.scm:1040: match1 */
t4=((C_word*)((C_word*)t0)[8])[1];
f_12178(t4,((C_word*)t0)[4],C_i_caddr(((C_word*)t0)[2]),((C_word*)t0)[6]);}
else{
t4=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_i_pairp(((C_word*)t0)[6]);
t5=(C_truep(t4)?C_eqp(lf[372],C_u_i_car(((C_word*)t0)[6])):C_SCHEME_FALSE);
if(C_truep(t5)){
/* scrutinizer.scm:1042: match1 */
t6=((C_word*)((C_word*)t0)[8])[1];
f_12178(t6,((C_word*)t0)[4],((C_word*)t0)[2],C_i_caddr(((C_word*)t0)[6]));}
else{
t6=C_i_pairp(((C_word*)t0)[2]);
t7=(C_truep(t6)?C_eqp(lf[48],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t7)){
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t8=C_u_i_car(((C_word*)t0)[6]);
t9=C_eqp(t8,lf[361]);
if(C_truep(t9)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[9])[1]))){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12924,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1048: match1 */
t11=((C_word*)((C_word*)t0)[8])[1];
f_12178(t11,t10,C_i_cadr(((C_word*)t0)[2]),C_i_cadr(((C_word*)t0)[6]));}
else{
t10=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t10=C_eqp(t8,lf[47]);
if(C_truep(t10)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[6])))){
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12957,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1052: match1 */
t12=((C_word*)((C_word*)t0)[8])[1];
f_12178(t12,t11,C_i_cadr(((C_word*)t0)[2]),C_i_cadr(((C_word*)t0)[6]));}
else{
t11=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}
else{
t11=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}
else{
t8=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t8=C_i_pairp(((C_word*)t0)[6]);
t9=(C_truep(t8)?C_eqp(lf[48],C_u_i_car(((C_word*)t0)[6])):C_SCHEME_FALSE);
if(C_truep(t9)){
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t10=C_u_i_car(((C_word*)t0)[2]);
t11=C_eqp(t10,lf[361]);
if(C_truep(t11)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[9])[1]))){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13026,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1063: match1 */
t13=((C_word*)((C_word*)t0)[8])[1];
f_12178(t13,t12,C_i_cadr(((C_word*)t0)[2]),C_i_cadr(((C_word*)t0)[6]));}
else{
t12=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t12=C_eqp(t10,lf[47]);
if(C_truep(t12)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13059,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1067: match1 */
t14=((C_word*)((C_word*)t0)[8])[1];
f_12178(t14,t13,C_i_cadr(((C_word*)t0)[2]),C_i_cadr(((C_word*)t0)[6]));}
else{
t13=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}
else{
t13=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}}
else{
t10=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t10=C_i_pairp(((C_word*)t0)[2]);
t11=(C_truep(t10)?C_eqp(lf[47],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t11)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[9])[1]))){
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t12=C_eqp(lf[361],C_u_i_car(((C_word*)t0)[6]));
if(C_truep(t12)){
t13=C_i_cdr(((C_word*)t0)[2]);
t14=C_i_cadr(((C_word*)t0)[6]);
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13133,a[2]=((C_word*)t0)[8],a[3]=t14,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1076: over-all-instantiations */
f_20220(((C_word*)t0)[4],t13,((C_word*)t0)[10],C_SCHEME_TRUE,t15);}
else{
t13=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}
else{
t12=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t12=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t12=C_i_pairp(((C_word*)t0)[2]);
t13=(C_truep(t12)?C_eqp(lf[361],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t13)){
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t14=C_eqp(lf[47],C_u_i_car(((C_word*)t0)[6]));
if(C_truep(t14)){
t15=C_u_i_cdr(((C_word*)t0)[6]);
t16=C_i_cadr(((C_word*)t0)[2]);
t17=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13167,a[2]=((C_word*)t0)[8],a[3]=t16,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1083: over-all-instantiations */
f_20220(((C_word*)t0)[4],t15,((C_word*)t0)[10],C_SCHEME_TRUE,t17);}
else{
t15=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t15;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}}
else{
t14=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t14;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}}
else{
t14=C_i_pairp(((C_word*)t0)[2]);
t15=(C_truep(t14)?C_eqp(lf[50],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t15)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[9])[1]))){
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t16=C_eqp(lf[359],C_u_i_car(((C_word*)t0)[6]));
if(C_truep(t16)){
t17=C_i_cdr(((C_word*)t0)[2]);
t18=C_i_cadr(((C_word*)t0)[6]);
t19=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13209,a[2]=((C_word*)t0)[8],a[3]=t18,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1091: over-all-instantiations */
f_20220(((C_word*)t0)[4],t17,((C_word*)t0)[10],C_SCHEME_TRUE,t19);}
else{
t17=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t17;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}}
else{
t16=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t16;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}}
else{
t16=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t16;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}}
else{
t16=C_i_pairp(((C_word*)t0)[2]);
t17=(C_truep(t16)?C_eqp(lf[359],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t17)){
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t18=C_eqp(lf[50],C_u_i_car(((C_word*)t0)[6]));
if(C_truep(t18)){
t19=C_u_i_cdr(((C_word*)t0)[6]);
t20=C_i_cadr(((C_word*)t0)[2]);
t21=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13243,a[2]=((C_word*)t0)[8],a[3]=t20,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1098: over-all-instantiations */
f_20220(((C_word*)t0)[4],t19,((C_word*)t0)[10],C_SCHEME_TRUE,t21);}
else{
t19=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t19;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}}
else{
t18=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t18;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}}
else{
t18=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t18;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}}}}}}}}}}

/* k12704 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12706,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12709,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1023: procedure-arguments */
f_15222(t2,((C_word*)t0)[5]);}

/* k12707 in k12704 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12709,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12712,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1024: procedure-results */
f_15279(t2,((C_word*)t0)[7]);}

/* k12710 in k12707 in k12704 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12712,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12715,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1025: procedure-results */
f_15279(t2,((C_word*)t0)[7]);}

/* k12713 in k12710 in k12707 in k12704 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_12715,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12721,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1026: match-args */
t3=((C_word*)((C_word*)t0)[5])[1];
f_11858(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k12719 in k12713 in k12710 in k12707 in k12704 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12721,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:1027: match-results */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12052(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12812 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12814,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_i_not(((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cadr(((C_word*)t0)[4]);
t4=C_i_cadr(((C_word*)t0)[5]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7803,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:193: every */
f_7125(((C_word*)t0)[3],t5,t3);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12922 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12924,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:1049: match1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12178(t2,((C_word*)t0)[3],C_i_caddr(((C_word*)t0)[4]),((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12955 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_12957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12957,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_i_cddr(((C_word*)t0)[3]);
if(C_truep(C_i_nullp(t3))){
/* scrutinizer.scm:1053: match1 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_12178(t4,((C_word*)t0)[5],t2,lf[31]);}
else{
t4=C_u_i_cdr(((C_word*)t0)[3]);
t5=C_u_i_cdr(t4);
t6=C_a_i_cons(&a,2,lf[47],t5);
/* scrutinizer.scm:1053: match1 */
t7=((C_word*)((C_word*)t0)[4])[1];
f_12178(t7,((C_word*)t0)[5],t2,t6);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13024 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13026,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:1064: match1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12178(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_i_caddr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13057 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_13059,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t2))){
/* scrutinizer.scm:1068: match1 */
t3=((C_word*)((C_word*)t0)[3])[1];
f_12178(t3,((C_word*)t0)[4],lf[31],C_i_caddr(((C_word*)t0)[5]));}
else{
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_u_i_cdr(t3);
t5=C_a_i_cons(&a,2,lf[47],t4);
/* scrutinizer.scm:1068: match1 */
t6=((C_word*)((C_word*)t0)[3])[1];
f_12178(t6,((C_word*)t0)[4],t5,C_i_caddr(((C_word*)t0)[5]));}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a13129 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13133,c,av);}
/* scrutinizer.scm:1080: g23202321 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,t2,((C_word*)t0)[3]);}

/* a13163 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13167,c,av);}
/* scrutinizer.scm:1087: g23352336 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,((C_word*)t0)[3],t2);}

/* a13205 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13209(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13209,c,av);}
/* scrutinizer.scm:1095: g23512352 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,t2,((C_word*)t0)[3]);}

/* a13239 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13243,c,av);}
/* scrutinizer.scm:1102: g23662367 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_12178(t3,t1,((C_word*)t0)[3],t2);}

/* chicken.compiler.scrutinizer#match-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13431(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_13431,4,t1,t2,t3,t4);}
a=C_alloc(6);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13437,a[2]=t4,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_13437(t8,t1,t2,t3);}

/* loop in chicken.compiler.scrutinizer#match-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13437(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_13437,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_nullp(t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_equalp(lf[375],t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_car(t2);
t6=C_eqp(t5,lf[365]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13470,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1119: every */
f_7125(t1,t7,t3);}
else{
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13484,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1123: match-types */
t8=*((C_word*)lf[170]+1);{
C_word av2[6];
av2[0]=t8;
av2[1]=t7;
av2[2]=C_u_i_car(t2);
av2[3]=C_i_car(t3);
av2[4]=((C_word*)t0)[2];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t8+1)))(6,av2);}}}}}}

/* a13469 in loop in chicken.compiler.scrutinizer#match-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13470(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_13470,c,av);}
/* scrutinizer.scm:1121: match-types */
t3=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
av2[3]=t2;
av2[4]=((C_word*)t0)[3];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k13482 in loop in chicken.compiler.scrutinizer#match-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13484,c,av);}
if(C_truep(t1)){
/* scrutinizer.scm:1124: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13437(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13503(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_13503,2,t1,t2);}
a=C_alloc(21);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_END_OF_LIST;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13506,a[2]=t4,a[3]=t6,a[4]=t10,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14403,a[2]=t8,a[3]=t1,a[4]=t6,a[5]=t10,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1254: simplify */
t13=((C_word*)t10)[1];
f_13506(t13,t12,t2);}

/* simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13506(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_13506,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13512,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1139: chicken.base#call/cc */
t4=*((C_word*)lf[167]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_13512,c,av);}
a=C_alloc(24);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_eqp(t3,lf[29]);
if(C_truep(t4)){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13533,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=((C_word*)t0)[2],a[7]=t5,tmp=(C_word)a,a+=8,tmp);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=C_i_check_list_2(t5,lf[6]);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13585,a[2]=t6,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13587,a[2]=t9,a[3]=t14,a[4]=t10,tmp=(C_word)a,a+=5,tmp));
t16=((C_word*)t14)[1];
f_13587(t16,t12,t5);}
else{
t5=C_eqp(t3,lf[28]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13628,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=C_u_i_cdr(((C_word*)t0)[2]);
t12=C_i_check_list_2(t11,lf[6]);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13936,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13938,a[2]=t9,a[3]=t15,a[4]=((C_word*)t0)[5],a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t17=((C_word*)t15)[1];
f_13938(t17,t13,t11);}
else{
t6=C_eqp(t3,lf[372]);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[2]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13982,a[2]=t7,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1209: simplify */
t9=((C_word*)((C_word*)t0)[5])[1];
f_13506(t9,t8,C_i_caddr(((C_word*)t0)[2]));}
else{
t7=C_eqp(t3,lf[48]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14029,a[2]=t1,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1216: simplify */
t9=((C_word*)((C_word*)t0)[5])[1];
f_13506(t9,t8,C_i_cadr(((C_word*)t0)[2]));}
else{
t8=C_eqp(t3,lf[359]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14068,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1223: simplify */
t10=((C_word*)((C_word*)t0)[5])[1];
f_13506(t10,t9,C_i_cadr(((C_word*)t0)[2]));}
else{
t9=C_eqp(t3,lf[361]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14092,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1228: simplify */
t11=((C_word*)((C_word*)t0)[5])[1];
f_13506(t11,t10,C_i_cadr(((C_word*)t0)[2]));}
else{
t10=C_eqp(t3,lf[47]);
if(C_truep(t10)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[2])))){
t11=t1;{
C_word *av2=av;
av2[0]=t11;
av2[1]=lf[31];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t12=t11;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=((C_word*)t13)[1];
t15=C_u_i_cdr(((C_word*)t0)[2]);
t16=C_i_check_list_2(t15,lf[6]);
t17=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14133,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14135,a[2]=t13,a[3]=t19,a[4]=((C_word*)t0)[5],a[5]=t14,tmp=(C_word)a,a+=6,tmp));
t21=((C_word*)t19)[1];
f_14135(t21,t17,t15);}}
else{
t11=C_eqp(t3,lf[50]);
if(C_truep(t11)){
t12=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t13=t12;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=((C_word*)t14)[1];
t16=C_u_i_cdr(((C_word*)t0)[2]);
t17=C_i_check_list_2(t16,lf[6]);
t18=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14189,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t19=C_SCHEME_UNDEFINED;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_set_block_item(t20,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14191,a[2]=t14,a[3]=t20,a[4]=((C_word*)t0)[5],a[5]=t15,tmp=(C_word)a,a+=6,tmp));
t22=((C_word*)t20)[1];
f_14191(t22,t18,t16);}
else{
t12=C_eqp(t3,lf[84]);
if(C_truep(t12)){
t13=(
/* scrutinizer.scm:1239: named? */
  f_15687(((C_word*)t0)[2])
);
t14=(C_truep(t13)?C_i_cadr(((C_word*)t0)[2]):C_SCHEME_FALSE);
t15=(C_truep(t14)?C_i_cdddr(((C_word*)t0)[2]):C_i_cddr(((C_word*)t0)[2]));
t16=(C_truep(t14)?C_a_i_list1(&a,1,t14):C_SCHEME_END_OF_LIST);
t17=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t18=t17;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=((C_word*)t19)[1];
t21=(C_truep(t14)?C_i_caddr(((C_word*)t0)[2]):C_i_cadr(((C_word*)t0)[2]));
t22=C_i_check_list_2(t21,lf[6]);
t23=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14309,a[2]=t1,a[3]=t16,a[4]=t15,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t24=C_SCHEME_UNDEFINED;
t25=(*a=C_VECTOR_TYPE|1,a[1]=t24,tmp=(C_word)a,a+=2,tmp);
t26=C_set_block_item(t25,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14311,a[2]=t19,a[3]=t25,a[4]=((C_word*)t0)[5],a[5]=t20,tmp=(C_word)a,a+=6,tmp));
t27=((C_word*)t25)[1];
f_14311(t27,t23,t21);}
else{
t13=t1;{
C_word *av2=av;
av2[0]=t13;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}}}}}}}}}
else{
t3=C_i_assq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14389,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1141: g2692 */
t5=t4;
f_14389(t5,t1,t3);}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k13531 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13533(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_13533,c,av);}
a=C_alloc(12);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13537,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13548,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13550,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:1152: filter-map */
f_7409(t4,t5,((C_word*)t0)[7]);}

/* k13535 in k13531 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13537,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* scrutinizer.scm:1157: simplify */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13506(t3,((C_word*)t0)[4],C_i_caddr(((C_word*)t0)[5]));}

/* k13546 in k13531 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13548,c,av);}
/* scrutinizer.scm:1152: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a13549 in k13531 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13550(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13550,c,av);}
t3=C_i_pairp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t2:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13583 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13585,c,av);}
/* scrutinizer.scm:1146: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop2425 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13587(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_13587,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_symbolp(t3);
t5=(C_truep(t4)?t3:C_i_car(t3));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15902,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1520: chicken.plist#get */
t7=*((C_word*)lf[377]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=t5;
av2[3]=lf[376];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13628(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13628,c,av);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_car(t1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_length(t1);
if(C_truep(C_fixnum_greaterp(t3,C_fix(20)))){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13656,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1165: every */
f_7125(t4,lf[121],t1);}}}}

/* k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13656(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13656,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13662,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13749,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:1166: any */
f_7159(t2,t3,((C_word*)t0)[3]);}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13759,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1183: lset=/eq? */
f_7809(t2,lf[385],((C_word*)t0)[3]);}}

/* k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13662,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[84];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_car(((C_word*)t0)[3]);
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=C_i_check_list_2(t3,lf[378]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13674,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t8=((C_word*)t6)[1];
f_13674(t8,((C_word*)t0)[2],t3,t2);}}

/* foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13674(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_13674,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13743,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_slot(t2,C_fix(0));
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13694,a[2]=t5,a[3]=t3,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1170: procedure-name */
f_15171(t7,t6);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13694,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13697,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1171: procedure-arguments */
f_15222(t2,((C_word*)t0)[4]);}

/* k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13697(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13697,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13700,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1172: procedure-results */
f_15279(t2,((C_word*)t0)[5]);}

/* k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_13700,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13703,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1173: procedure-name */
f_15171(t2,((C_word*)t0)[5]);}

/* k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13703,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13706,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1174: procedure-arguments */
f_15222(t2,((C_word*)t0)[6]);}

/* k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13706,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13709,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1175: procedure-results */
f_15279(t2,((C_word*)t0)[7]);}

/* k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_13709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13709,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13716,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[6])){
if(C_truep(((C_word*)t0)[7])){
t3=C_eqp(((C_word*)t0)[6],((C_word*)t0)[7]);
t4=t2;
f_13716(t4,(C_truep(t3)?C_a_i_list1(&a,1,((C_word*)t0)[6]):C_SCHEME_END_OF_LIST));}
else{
t3=t2;
f_13716(t3,C_SCHEME_END_OF_LIST);}}
else{
t3=t2;
f_13716(t3,C_SCHEME_END_OF_LIST);}}

/* k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_13716(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_13716,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13728,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1179: merge-argument-types */
f_14472(t2,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k13722 in k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_13724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_13724,c,av);}
/* scrutinizer.scm:1176: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[379];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_13728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_13728,c,av);}
a=C_alloc(12);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13724,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14647,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1303: chicken.base#call/cc */
t5=*((C_word*)lf[167]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13741 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13743,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_13674(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* a13748 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13749,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(lf[84],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13759(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13759,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[34];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13765,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1184: lset=/eq? */
f_7809(t2,lf[384],((C_word*)t0)[6]);}}

/* k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13765,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[40];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13771,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1185: lset=/eq? */
f_7809(t2,lf[383],((C_word*)t0)[6]);}}

/* k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_13771,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[381];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13774,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13874,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1187: append-map */
f_6955(t2,t3,((C_word*)t0)[6],C_SCHEME_END_OF_LIST);}}

/* k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_13774,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13777,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13794,a[2]=((C_word*)t0)[5],a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_13794(t6,t2,t1,C_SCHEME_END_OF_LIST);}

/* k13775 in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13777,c,av);}
a=C_alloc(3);
if(C_truep(C_i_equalp(t1,C_u_i_cdr(((C_word*)t0)[2])))){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_a_i_cons(&a,2,lf[28],t1);
/* scrutinizer.scm:1206: simplify */
t3=((C_word*)((C_word*)t0)[4])[1];
f_13506(t3,((C_word*)t0)[3],t2);}}

/* loop in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13794(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_13794,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
/* scrutinizer.scm:1197: scheme#reverse */
t4=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(lf[53],t4);
if(C_truep(t5)){
/* scrutinizer.scm:1198: return */
t6=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[53];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13819,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13858,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1199: any */
f_7159(t6,t7,C_u_i_cdr(t2));}}}

/* k13817 in loop in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_13819,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
/* scrutinizer.scm:1200: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13794(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),((C_word*)t0)[5]);}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13830,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13848,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1201: any */
f_7159(t2,t3,((C_word*)t0)[5]);}}

/* k13828 in k13817 in loop in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_13830,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
/* scrutinizer.scm:1202: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13794(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),((C_word*)t0)[5]);}
else{
t2=C_u_i_cdr(((C_word*)t0)[4]);
t3=C_u_i_car(((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[5]);
/* scrutinizer.scm:1203: loop */
t5=((C_word*)((C_word*)t0)[2])[1];
f_13794(t5,((C_word*)t0)[3],t2,t4);}}

/* a13847 in k13817 in loop in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13848,c,av);}
t3=*((C_word*)lf[382]+1);
/* scrutinizer.scm:1201: g2552 */
t4=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=C_i_car(((C_word*)t0)[2]);
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a13857 in loop in k13772 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13858,c,av);}
t3=*((C_word*)lf[382]+1);
/* scrutinizer.scm:1199: g2544 */
t4=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=C_i_car(((C_word*)t0)[2]);
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a13873 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13874,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13878,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1189: simplify */
t4=((C_word*)((C_word*)t0)[3])[1];
f_13506(t4,t3,t2);}

/* k13876 in a13873 in k13769 in k13763 in k13757 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13878,c,av);}
a=C_alloc(3);
t2=C_i_pairp(t1);
t3=(C_truep(t2)?C_eqp(lf[28],C_u_i_car(t1)):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(t1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(t1,lf[35]);
if(C_truep(t4)){
/* scrutinizer.scm:1192: return */
t5=((C_word*)t0)[3];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[35];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t5=C_eqp(t1,lf[70]);
if(C_truep(t5)){
/* scrutinizer.scm:1193: return */
t6=((C_word*)t0)[3];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[53];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_list1(&a,1,t1);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}

/* k13934 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13936,c,av);}
/* scrutinizer.scm:1159: delete-duplicates */
f_7320(((C_word*)t0)[2],t1,*((C_word*)lf[374]+1));}

/* map-loop2458 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_13938(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_13938,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13963,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1159: g2464 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13961 in map-loop2458 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13963(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13963,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_13938(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_13982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_13982,c,av);}
a=C_alloc(12);
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(
/* scrutinizer.scm:1211: refinement-type? */
  f_15863(t1)
);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14001,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_i_cadr(t1);
t5=C_a_i_list(&a,1,((C_word*)t0)[2]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7676,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_7676(t9,t3,t5,t4);}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14016,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1214: delete-duplicates */
f_7320(t3,((C_word*)t0)[2],*((C_word*)lf[374]+1));}}}

/* k13999 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14001(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14001,c,av);}
a=C_alloc(9);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list3(&a,3,lf[372],t1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14014 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14016(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14016,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list3(&a,3,lf[372],t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k14027 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14029,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14032,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1217: simplify */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13506(t3,t2,C_i_caddr(((C_word*)t0)[4]));}

/* k14030 in k14027 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_14032,c,av);}
a=C_alloc(9);
t2=C_eqp(lf[53],((C_word*)t0)[2]);
t3=(C_truep(t2)?C_eqp(lf[53],t1):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[48];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_a_i_list(&a,3,lf[48],((C_word*)t0)[2],t1);
/* scrutinizer.scm:1220: canonicalize-list-type */
f_19333(((C_word*)t0)[3],t4);}}

/* k14066 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14068,c,av);}
a=C_alloc(6);
t2=C_eqp(t1,lf[53]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[50];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_u_i_car(((C_word*)t0)[3]);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list(&a,2,t3,t1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14090 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14092,c,av);}
a=C_alloc(6);
t2=C_eqp(t1,lf[53]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[47];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_u_i_car(((C_word*)t0)[3]);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list(&a,2,t3,t1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14131 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14133,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[47],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2583 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14135(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14135,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14160,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1235: g2589 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14158 in map-loop2583 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14160,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14135(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k14187 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14189,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[50],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2612 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14191(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14191,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14216,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1237: g2618 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14214 in map-loop2612 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14216,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14191(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k14248 in k14307 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_14250,c,av);}
/* scrutinizer.scm:1241: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[389];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* map-loop2667 in k14307 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14264(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14264,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14289,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1247: g2673 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14287 in map-loop2667 in k14307 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14289,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14264(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k14307 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,5)))){
C_save_and_reclaim((void *)f_14309,c,av);}
a=C_alloc(21);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14250,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_eqp(lf[53],((C_word*)t0)[4]);
if(C_truep(t4)){
/* scrutinizer.scm:1241: scheme#append */
t5=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[389];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
av2[5]=lf[53];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_i_check_list_2(((C_word*)t0)[4],lf[6]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14264,a[2]=t7,a[3]=t11,a[4]=((C_word*)t0)[5],a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_14264(t13,t3,((C_word*)t0)[4]);}}

/* map-loop2641 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14311(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_14311,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14336,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1244: g2647 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13506(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14334 in map-loop2641 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14336,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14311(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* g2692 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14389(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,4)))){
C_save_and_reclaim_args((void *)trf_14389,3,t0,t1,t2);}
a=C_alloc(13);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14394,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=((C_word*)((C_word*)t0)[2])[1];
t5=C_a_i_list(&a,1,((C_word*)t0)[3]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7590,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_7590(t9,t3,t5,t4);}

/* k14392 in g2692 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14394(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14394,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14401 in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14403(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_14403,c,av);}
a=C_alloc(11);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[2])[1]))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14420,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14422,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1257: filter-map */
f_7409(t4,t5,((C_word*)((C_word*)t0)[6])[1]);}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t3)[1];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14418 in k14401 in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14420,c,av);}
a=C_alloc(9);
t2=C_a_i_list(&a,3,lf[29],t1,((C_word*)((C_word*)t0)[2])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a14421 in k14401 in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14422,c,av);}
a=C_alloc(4);
t3=C_i_car(t2);
if(C_truep(C_i_memq(t3,((C_word*)((C_word*)t0)[2])[1]))){
t4=C_u_i_cdr(t2);
t5=C_i_assq(C_u_i_car(t2),((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14437,a[2]=t4,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1261: g2710 */
t7=t6;
f_14437(t7,t1,t5);}
else{
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* g2710 in a14421 in k14401 in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14437(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_14437,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14445,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1263: simplify */
t4=((C_word*)((C_word*)t0)[3])[1];
f_13506(t4,t3,C_i_cadr(t2));}

/* k14443 in g2710 in a14421 in k14401 in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14445(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14445,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14472(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_14472,3,t1,t2,t3);}
a=C_alloc(14);
if(C_truep(C_i_nullp(t2))){
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep((C_truep(C_eqp(t4,lf[365]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[364]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))?t3:lf[390]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[391];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(lf[365],t4);
if(C_truep(t5)){
t6=C_i_pairp(t3);
t7=(C_truep(t6)?C_eqp(lf[365],C_u_i_car(t3)):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14518,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t9=(
/* scrutinizer.scm:1288: rest-type */
  f_15740(C_u_i_cdr(t2))
);
t10=(
/* scrutinizer.scm:1289: rest-type */
  f_15740(C_i_cdr(t3))
);
t11=C_a_i_list(&a,3,lf[28],t9,t10);
/* scrutinizer.scm:1287: simplify-type */
f_13503(t8,t11);}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[392];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t6=C_eqp(lf[364],C_u_i_car(t2));
if(C_truep(t6)){
t7=C_i_pairp(t3);
t8=(C_truep(t7)?C_eqp(lf[364],C_u_i_car(t3)):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14567,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t10=C_i_cadr(t2);
t11=C_i_cadr(t3);
t12=C_a_i_list(&a,3,lf[28],t10,t11);
/* scrutinizer.scm:1294: simplify-type */
f_13503(t9,t12);}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[393];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t7=C_i_car(t3);
if(C_truep((C_truep(C_eqp(t7,lf[365]))?C_SCHEME_TRUE:(C_truep(C_eqp(t7,lf[364]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
/* scrutinizer.scm:1298: merge-argument-types */
t14=t1;
t15=t3;
t16=t2;
t1=t14;
t2=t15;
t3=t16;
goto loop;}
else{
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14617,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t9=C_u_i_car(t2);
t10=C_u_i_car(t3);
t11=C_a_i_list(&a,3,lf[28],t9,t10);
/* scrutinizer.scm:1299: simplify-type */
f_13503(t8,t11);}}}}}}

/* k14516 in chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14518,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[365],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k14565 in chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14567,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14571,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1295: merge-argument-types */
f_14472(t2,C_i_cddr(((C_word*)t0)[3]),C_i_cddr(((C_word*)t0)[4]));}

/* k14569 in k14565 in chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_14571,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[364],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14615 in chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14617,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14621,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1300: merge-argument-types */
f_14472(t2,C_u_i_cdr(((C_word*)t0)[3]),C_u_i_cdr(((C_word*)t0)[4]));}

/* k14619 in k14615 in chicken.compiler.scrutinizer#merge-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14621,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a14646 in k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_14647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_14647,c,av);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14653,a[2]=t2,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_14653(t6,t1,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* loop in a14646 in k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_14653(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_14653,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=C_i_nullp(t2);
t5=(C_truep(t4)?C_i_nullp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_not_pair_p(t2);
t7=(C_truep(t6)?t6:C_i_not_pair_p(t3));
if(C_truep(t7)){
/* scrutinizer.scm:1307: return */
t8=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t8;
av2[1]=t1;
av2[2]=lf[53];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14679,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t9=C_i_car(t2);
t10=C_i_car(t3);
t11=C_a_i_list(&a,3,lf[28],t9,t10);
/* scrutinizer.scm:1308: simplify-type */
f_13503(t8,t11);}}}

/* k14677 in loop in a14646 in k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_14679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14679,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14683,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1309: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_14653(t3,t2,C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}

/* k14681 in k14677 in loop in a14646 in k13726 in k13714 in k13707 in k13704 in k13701 in k13698 in k13695 in k13692 in foldl2501 in k13660 in k13654 in k13626 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in ... */
static void C_ccall f_14683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14683,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#compatible-types? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14710(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_14710,4,t1,t2,t3,t4);}
a=C_alloc(14);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14714,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_nullp(t4))){
t6=C_a_i_list(&a,3,lf[28],t2,t3);
/* scrutinizer.scm:1312: type-typeenv */
t7=lf[161];{
C_word av2[3];
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
f_15910(3,av2);}}
else{
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_i_car(t4);
f_14714(2,av2);}}}

/* k14712 in chicken.compiler.scrutinizer#compatible-types? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_14714,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14717,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1313: type<=? */
t3=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k14715 in k14712 in chicken.compiler.scrutinizer#compatible-types? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_14717,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* scrutinizer.scm:1314: type<=? */
t2=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k14740 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_14742,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14748,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1317: type<=? */
t3=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k14746 in k14740 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_14748,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14754,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1318: type<=? */
t3=*((C_word*)lf[382]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k14752 in k14746 in k14740 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14754,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?((C_word*)t0)[3]:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#type<=? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_14769,c,av);}
a=C_alloc(14);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14773,a[2]=t3,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_rest_nullp(c,4))){
t5=C_a_i_list(&a,3,lf[28],t2,t3);
/* scrutinizer.scm:1321: type-typeenv */
t6=lf[161];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
f_15910(3,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_get_rest_arg(c,4,av,4,t0);
f_14773(2,av2);}}}

/* k14771 in chicken.compiler.scrutinizer#type<=? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_14773,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14778,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=lf[24];
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16280,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1562: thunk */
t5=t2;
f_14778(t5,t4);}

/* a14777 in k14771 in chicken.compiler.scrutinizer#type<=? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14778(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_14778,2,t0,t1);}
/* scrutinizer.scm:1325: match-types */
t2=*((C_word*)lf[170]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_14797,c,av);}
a=C_alloc(14);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15098,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_list(&a,3,lf[28],t2,t3);
/* scrutinizer.scm:1364: type-typeenv */
t6=lf[161];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
f_15910(3,av2);}}

/* loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14806(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_14806,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14810,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_symbolp(t2))){
/* scrutinizer.scm:1272: chicken.base#alist-ref */
t5=*((C_word*)lf[275]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[373];
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14810(2,av2);}}}

/* k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_14810,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14814,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1337: g2845 */
t3=t2;
f_14814(t3,((C_word*)t0)[4],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14824,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
/* scrutinizer.scm:1272: chicken.base#alist-ref */
t3=*((C_word*)lf[275]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[373];
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_14824(2,av2);}}}}

/* g2845 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14814(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_14814,3,t0,t1,t2);}
/* scrutinizer.scm:1338: g2853 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_14806(t3,t1,t2,((C_word*)t0)[3]);}

/* k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_14824,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14828,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1337: g2856 */
t3=t2;
f_14828(t3,((C_word*)t0)[4],t1);}
else{
t2=C_i_pairp(((C_word*)t0)[3]);
t3=(C_truep(t2)?C_u_i_memq(C_u_i_car(((C_word*)t0)[3]),lf[394]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14844,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1341: loop */
t5=((C_word*)((C_word*)t0)[2])[1];
f_14806(t5,t4,C_i_caddr(((C_word*)t0)[3]),((C_word*)t0)[5]);}
else{
t4=C_i_pairp(((C_word*)t0)[5]);
t5=(C_truep(t4)?C_u_i_memq(C_u_i_car(((C_word*)t0)[5]),lf[395]):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14871,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1344: loop */
t7=((C_word*)((C_word*)t0)[2])[1];
f_14806(t7,t6,((C_word*)t0)[3],C_i_caddr(((C_word*)t0)[5]));}
else{
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14895,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t7=C_eqp(C_u_i_car(((C_word*)t0)[3]),lf[28]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15068,a[2]=((C_word*)t0)[5],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1348: lset=/eq? */
f_7809(t8,lf[397],C_u_i_cdr(((C_word*)t0)[3]));}
else{
t8=t6;
f_14895(t8,C_SCHEME_FALSE);}}
else{
t7=t6;
f_14895(t7,C_SCHEME_FALSE);}}}}}

/* g2856 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14828(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_14828,3,t0,t1,t2);}
/* scrutinizer.scm:1339: g2864 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_14806(t3,t1,((C_word*)t0)[3],t2);}

/* k14842 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14844,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list3(&a,3,t2,t3,t1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k14869 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14871(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14871,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list3(&a,3,t2,t3,t1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14895(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_14895,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_pairp(((C_word*)t0)[4]);
t3=(C_truep(t2)?C_eqp(C_u_i_car(((C_word*)t0)[4]),lf[28]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14904,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14915,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1352: filter-map */
f_7409(t4,t5,C_i_cdr(((C_word*)t0)[4]));}
else{
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14929,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[4]))){
t5=C_u_i_car(((C_word*)t0)[4]);
if(C_truep((C_truep(C_eqp(t5,lf[48]))?C_SCHEME_TRUE:(C_truep(C_eqp(t5,lf[47]))?C_SCHEME_TRUE:(C_truep(C_eqp(t5,lf[50]))?C_SCHEME_TRUE:(C_truep(C_eqp(t5,lf[359]))?C_SCHEME_TRUE:(C_truep(C_eqp(t5,lf[361]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))))){
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t6=C_eqp(C_u_i_car(((C_word*)t0)[4]),C_u_i_car(((C_word*)t0)[3]));
t7=t4;
f_14929(t7,(C_truep(t6)?C_eqp(C_i_length(((C_word*)t0)[4]),C_i_length(((C_word*)t0)[3])):C_SCHEME_FALSE));}
else{
t6=t4;
f_14929(t6,C_SCHEME_FALSE);}}
else{
t6=t4;
f_14929(t6,C_SCHEME_FALSE);}}
else{
t5=t4;
f_14929(t5,C_SCHEME_FALSE);}}}}

/* k14902 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14904,c,av);}
a=C_alloc(3);
t2=C_i_pairp(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_a_i_cons(&a,2,lf[28],t1):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a14914 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14915(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14915,c,av);}
/* scrutinizer.scm:1352: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_14806(t3,t1,t2,((C_word*)t0)[3]);}

/* k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14929(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,4)))){
C_save_and_reclaim_args((void *)trf_14929,2,t0,t1);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cdr(((C_word*)t0)[2]);
t7=C_i_cdr(((C_word*)t0)[3]);
t8=C_i_check_list_2(t6,lf[6]);
t9=C_i_check_list_2(t7,lf[6]);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14947,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14960,a[2]=t4,a[3]=t12,a[4]=((C_word*)t0)[5],a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_14960(t14,t10,t6,t7);}
else{
t2=C_a_i_list(&a,1,((C_word*)t0)[6]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14742,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_nullp(t2))){
t4=C_a_i_list(&a,3,lf[28],((C_word*)t0)[2],((C_word*)t0)[3]);
/* scrutinizer.scm:1316: type-typeenv */
t5=lf[161];{
C_word av2[3];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
f_15910(3,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_car(t2);
f_14742(2,av2);}}}}

/* k14945 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14947,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14953,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1360: every */
f_7125(t2,*((C_word*)lf[396]+1),t1);}

/* k14951 in k14945 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14953(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14953,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* map-loop2887 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_14960(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_14960,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14989,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1359: g2893 */
t7=((C_word*)((C_word*)t0)[4])[1];
f_14806(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k14987 in map-loop2887 in k14927 in k14893 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_14989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14989,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14960(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k15066 in k14822 in k14808 in loop in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15068,c,av);}
if(C_truep(t1)){
t2=C_i_pairp(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];
f_14895(t3,(C_truep(t2)?C_eqp(lf[361],C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE));}
else{
t2=((C_word*)t0)[3];
f_14895(t2,C_SCHEME_FALSE);}}

/* k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15098(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_15098,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15101,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14806,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_14806(t6,t2,((C_word*)t0)[4],((C_word*)t0)[2]);}

/* k15099 in k15096 in chicken.compiler.scrutinizer#refine-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15101(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15101,c,av);}
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* scrutinizer.scm:1368: simplify-type */
f_13503(((C_word*)t0)[3],t2);}}

/* chicken.compiler.scrutinizer#procedure-type? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15119(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
av[0]=t0;
av[1]=t1;
av[2]=t2;
C_save_and_reclaim((void *)f_15119,c,av);}
t3=C_eqp(lf[84],t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_car(t2);
t5=C_eqp(t4,lf[29]);
if(C_truep(t5)){
/* scrutinizer.scm:1376: procedure-type? */
t9=t1;
t10=C_i_caddr(t2);
t1=t9;
t2=t10;
c=3;
goto loop;}
else{
t6=C_eqp(t4,lf[84]);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t4,lf[28]);
if(C_truep(t7)){
/* scrutinizer.scm:1378: every */
f_7125(t1,lf[121],C_u_i_cdr(t2));}
else{
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* chicken.compiler.scrutinizer#procedure-name in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15171(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_15171,2,t1,t2);}
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[29]);
if(C_truep(t4)){
/* scrutinizer.scm:1384: procedure-name */
t9=t1;
t10=C_i_caddr(t2);
t1=t9;
t2=t10;
goto loop;}
else{
t5=C_eqp(t3,lf[84]);
if(C_truep(t5)){
t6=C_i_cadr(t2);
if(C_truep(C_i_stringp(t6))){
/* scrutinizer.scm:1387: scheme#string->symbol */
t7=*((C_word*)lf[252]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t7=C_i_symbolp(t6);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?t6:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#procedure-arguments in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15222(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_15222,2,t1,t2);}
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[29]);
if(C_truep(t4)){
/* scrutinizer.scm:1395: procedure-arguments */
t10=t1;
t11=C_i_caddr(t2);
t1=t10;
t2=t11;
goto loop;}
else{
t5=C_eqp(t3,lf[84]);
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=C_i_stringp(t6);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?C_i_caddr(t2):C_i_cadr(t2));
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_i_symbolp(t6);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(C_truep(t8)?C_i_caddr(t2):C_i_cadr(t2));
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
/* scrutinizer.scm:1401: chicken.compiler.support#bomb */
t6=*((C_word*)lf[343]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[398];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#procedure-results in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15279(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_15279,2,t1,t2);}
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[29]);
if(C_truep(t4)){
/* scrutinizer.scm:1406: procedure-results */
t10=t1;
t11=C_i_caddr(t2);
t1=t10;
t2=t11;
goto loop;}
else{
t5=C_eqp(t3,lf[84]);
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=C_i_stringp(t6);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?C_i_cdddr(t2):C_i_cddr(t2));
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_i_symbolp(t6);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(C_truep(t8)?C_i_cdddr(t2):C_i_cddr(t2));
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
/* scrutinizer.scm:1412: chicken.compiler.support#bomb */
t6=*((C_word*)lf[343]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[399];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15336(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_15336,5,t1,t2,t3,t4,t5);}
a=C_alloc(8);
t6=C_i_nullp(t5);
t7=(C_truep(t6)?C_SCHEME_FALSE:C_i_car(t5));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15345,a[2]=t7,a[3]=t3,a[4]=t9,a[5]=t4,tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_15345(t11,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15345(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_15345,4,t0,t1,t2,t3);}
a=C_alloc(20);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_eqp(lf[84],C_u_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_TRUE;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_fix(0);
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15355,a[2]=t1,a[3]=t7,a[4]=t9,a[5]=t11,tmp=(C_word)a,a+=6,tmp);
t13=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15362,a[2]=t9,a[3]=((C_word*)t0)[2],a[4]=t7,a[5]=t11,a[6]=t12,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
t14=C_i_cadr(t2);
t15=C_i_stringp(t14);
if(C_truep(t15)){
t16=t13;
f_15362(t16,(C_truep(t15)?C_i_caddr(t2):C_i_cadr(t2)));}
else{
t16=C_i_cadr(t2);
t17=C_i_symbolp(t16);
t18=t13;
f_15362(t18,(C_truep(t17)?C_i_caddr(t2):C_i_cadr(t2)));}}
else{
t6=C_i_pairp(t2);
t7=(C_truep(t6)?C_eqp(lf[29],C_u_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t7)){
/* scrutinizer.scm:1447: loop1 */
t19=t1;
t20=C_i_caddr(t2);
t21=t3;
t1=t19;
t2=t20;
t3=t21;
goto loop;}
else{
t8=C_i_assq(t2,((C_word*)t0)[5]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15499,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1416: g3032 */
t10=t9;
f_15499(t10,t1,t8);}
else{
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15532,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1454: make-list */
f_7950(t9,((C_word*)t0)[3],C_a_i_list(&a,1,lf[53]));}}}}

/* k15353 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_15355,c,av);}
/* scrutinizer.scm:1445: scheme#values */{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
av2[4]=((C_word*)((C_word*)t0)[4])[1];
av2[5]=((C_word*)((C_word*)t0)[5])[1];
C_values(6,av2);}}

/* k15360 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15362(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_15362,2,t0,t1);}
a=C_alloc(9);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15364,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp));
t5=((C_word*)t3)[1];
f_15364(t5,((C_word*)t0)[6],t1,((C_word*)t0)[7],C_SCHEME_FALSE);}

/* loop in k15360 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15364(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(62,0,4)))){
C_save_and_reclaim_args((void *)trf_15364,5,t0,t1,t2,t3,t4);}
a=C_alloc(62);
if(C_truep(C_i_nullp(t2))){
if(C_truep(t4)){
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_i_zerop(t3);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t5=C_i_car(t2);
t6=C_eqp(lf[364],t5);
if(C_truep(t6)){
if(C_truep(((C_word*)t0)[3])){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* scrutinizer.scm:1434: loop */
t17=t1;
t18=C_u_i_cdr(t2);
t19=t3;
t20=C_SCHEME_TRUE;
t1=t17;
t2=t18;
t3=t19;
t4=t20;
goto loop;}}
else{
t7=C_eqp(lf[365],C_u_i_car(t2));
if(C_truep(t7)){
if(C_truep(((C_word*)t0)[3])){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_i_pairp(C_u_i_cdr(t2));
t9=(C_truep(t8)?C_eqp(lf[400],C_i_cadr(t2)):C_SCHEME_FALSE);
t10=C_set_block_item(((C_word*)t0)[5],0,t9);
t11=(
/* scrutinizer.scm:1439: rest-type */
  f_15740(C_u_i_cdr(t2))
);
/* scrutinizer.scm:1439: make-list */
f_7950(t1,t3,C_a_i_list(&a,1,t11));}}
else{
t8=(C_truep(t4)?C_i_less_or_equalp(t3,C_fix(0)):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_set_block_item(((C_word*)t0)[2],0,C_i_positivep(t3));
t10=((C_word*)((C_word*)t0)[6])[1];
t11=C_mutate(((C_word *)((C_word*)t0)[6])+1,C_s_a_i_plus(&a,2,t10,C_fix(1)));
t12=C_u_i_car(t2);
t13=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15443,a[2]=t1,a[3]=t12,tmp=(C_word)a,a+=4,tmp);
t14=C_u_i_cdr(t2);
t15=C_s_a_i_minus(&a,2,t3,C_fix(1));
/* scrutinizer.scm:1444: loop */
t17=t13;
t18=t14;
t19=t15;
t20=t4;
t1=t17;
t2=t18;
t3=t19;
t4=t20;
goto loop;}}}}}

/* k15441 in loop in k15360 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15443,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g3032 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15499(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_15499,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_i_cadr(t2);
t4=(C_truep(t3)?C_i_memq(t3,((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t4)){
/* scrutinizer.scm:1452: loop1 */
t5=((C_word*)((C_word*)t0)[3])[1];
f_15345(t5,t1,lf[53],((C_word*)t0)[2]);}
else{
t5=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[2]);
/* scrutinizer.scm:1453: loop1 */
t6=((C_word*)((C_word*)t0)[3])[1];
f_15345(t6,t1,t3,t5);}}

/* k15530 in loop1 in chicken.compiler.scrutinizer#procedure-argument-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_15532,c,av);}
/* scrutinizer.scm:1454: scheme#values */{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
av2[5]=((C_word*)t0)[3];
C_values(6,av2);}}

/* loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15559(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_15559,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(((C_word*)t0)[2])){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_assq(t2,((C_word*)t0)[4]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15570,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1458: g3051 */
t5=t4;
f_15570(t5,t1,t3);}
else{
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_eqp(lf[84],C_u_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15592,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1461: chicken.base#call/cc */
t7=*((C_word*)lf[167]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=C_i_pairp(t2);
t7=(C_truep(t6)?C_eqp(lf[29],C_u_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t7)){
/* scrutinizer.scm:1470: loop1 */
t9=t1;
t10=C_i_caddr(t2);
t1=t9;
t2=t10;
goto loop;}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}}}

/* g3051 in loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_15570(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_15570,3,t0,t1,t2);}
/* scrutinizer.scm:1459: loop1 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_15559(t3,t1,C_i_cadr(t2));}

/* a15591 in loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_15592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_15592,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15600,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_i_stringp(t4);
if(C_truep(t5)){
t6=t3;
f_15600(t6,(C_truep(t5)?C_i_cdddr(((C_word*)t0)[2]):C_i_cddr(((C_word*)t0)[2])));}
else{
t6=C_i_cadr(((C_word*)t0)[2]);
t7=C_i_symbolp(t6);
t8=t3;
f_15600(t8,(C_truep(t7)?C_i_cdddr(((C_word*)t0)[2]):C_i_cddr(((C_word*)t0)[2])));}}

/* k15598 in a15591 in loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_15600(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_15600,2,t0,t1);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15602,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_15602(t5,((C_word*)t0)[3],t1);}

/* loop in k15598 in a15591 in loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_15602(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_15602,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep((C_truep(C_eqp(t2,lf[53]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[70]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
/* scrutinizer.scm:1467: return */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[53];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15628,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1468: loop */
t6=t4;
t7=C_u_i_cdr(t2);
t1=t6;
t2=t7;
goto loop;}}}

/* k15626 in loop in k15598 in a15591 in loop1 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_15628(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15628,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#named? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_15687(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_u_i_car(t1);
t3=C_eqp(t2,lf[84]);
if(C_truep(t3)){
t4=C_i_cadr(t1);
t5=C_i_nullp(t4);
if(C_truep(t5)){
return(C_i_not(t5));}
else{
t6=C_u_i_cdr(t1);
return(C_i_not(C_i_pairp(C_u_i_car(t6))));}}
else{
t4=C_eqp(t2,lf[29]);
if(C_truep(t4)){
t8=C_i_caddr(t1);
t1=t8;
goto loop;}
else{
return(C_SCHEME_FALSE);}}}
else{
return(C_SCHEME_FALSE);}}

/* chicken.compiler.scrutinizer#rest-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_15740(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
if(C_truep(C_i_nullp(t1))){
return(lf[53]);}
else{
t2=C_i_car(t1);
t3=C_eqp(lf[400],t2);
return((C_truep(t3)?lf[53]:C_u_i_car(t1)));}}

/* chicken.compiler.scrutinizer#noreturn-procedure-type? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_15760(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_u_i_car(t1);
t3=C_eqp(t2,lf[84]);
if(C_truep(t3)){
if(C_truep(C_i_listp(t1))){
t4=C_i_cadr(t1);
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_i_cddr(t1):C_i_cdddr(t1));
return(C_i_equalp(lf[401],t6));}
else{
return(C_SCHEME_FALSE);}}
else{
t4=C_eqp(t2,lf[29]);
if(C_truep(t4)){
t8=C_i_caddr(t1);
t1=t8;
goto loop;}
else{
return(C_SCHEME_FALSE);}}}
else{
return(C_SCHEME_FALSE);}}

/* chicken.compiler.scrutinizer#noreturn-type? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
av[0]=t0;
av[1]=t1;
av[2]=t2;
C_save_and_reclaim((void *)f_15819,c,av);}
t3=C_eqp(lf[70],t2);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_car(t2);
t5=C_eqp(t4,lf[28]);
if(C_truep(t5)){
/* scrutinizer.scm:1504: any */
f_7159(t1,lf[402],C_u_i_cdr(t2));}
else{
t6=C_eqp(t4,lf[29]);
if(C_truep(t6)){
/* scrutinizer.scm:1505: noreturn-type? */
t8=t1;
t9=C_i_caddr(t2);
t1=t8;
t2=t9;
c=3;
goto loop;}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* chicken.compiler.scrutinizer#refinement-type? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_15863(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_i_car(t1);
t3=C_eqp(t2,lf[372]);
if(C_truep(t3)){
return(C_SCHEME_TRUE);}
else{
t4=C_eqp(t2,lf[29]);
if(C_truep(t4)){
t6=C_i_caddr(t1);
t1=t6;
goto loop;}
else{
return(C_SCHEME_FALSE);}}}
else{
return(C_SCHEME_FALSE);}}

/* k15900 in map-loop2425 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_15902,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15905,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1522: chicken.base#gensym */
t3=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15903 in k15900 in map-loop2425 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_15905,c,av);}
a=C_alloc(14);
t2=C_a_i_putprop(&a,3,t1,lf[376],((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[4])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[4])+1,t4);
t7=((C_word*)((C_word*)t0)[5])[1];
f_13587(t7,((C_word*)t0)[6],C_slot(((C_word*)t0)[7],C_fix(1)));}

/* chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_15910,c,av);}
a=C_alloc(12);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15914,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15916,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_15916(t9,t5,t2);}

/* k15912 in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15914,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15916(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(22,0,3)))){
C_save_and_reclaim_args((void *)trf_15916,3,t0,t1,t2);}
a=C_alloc(22);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[372]);
if(C_truep(t4)){
/* scrutinizer.scm:1532: loop */
t19=t1;
t20=C_i_caddr(t2);
t1=t19;
t2=t20;
goto loop;}
else{
t5=C_eqp(t3,lf[84]);
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=C_i_stringp(t6);
t8=(C_truep(t7)?t7:C_i_symbolp(C_i_cadr(t2)));
if(C_truep(t8)){
t9=C_i_caddr(t2);
t10=C_i_check_list_2(t9,lf[212]);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15961,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16005,a[2]=t13,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t15=((C_word*)t13)[1];
f_16005(t15,t11,t9);}
else{
t9=C_i_cadr(t2);
t10=C_i_check_list_2(t9,lf[212]);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16035,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16078,a[2]=t13,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t15=((C_word*)t13)[1];
f_16078(t15,t11,t9);}}
else{
t6=C_eqp(t3,lf[29]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16120,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=C_i_cadr(t2);
t13=C_i_check_list_2(t12,lf[6]);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16162,a[2]=t7,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16164,a[2]=t10,a[3]=t16,a[4]=t11,tmp=(C_word)a,a+=5,tmp));
t18=((C_word*)t16)[1];
f_16164(t18,t14,t12);}
else{
t7=C_eqp(t3,lf[28]);
if(C_truep(t7)){
t8=C_u_i_cdr(t2);
t9=C_i_check_list_2(t8,lf[212]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16211,a[2]=t11,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t13=((C_word*)t11)[1];
f_16211(t13,t1,t8);}
else{
t8=C_SCHEME_UNDEFINED;
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15959 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_15961,c,av);}
a=C_alloc(6);
t2=C_i_cdddr(((C_word*)t0)[2]);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_u_i_cdr(t3);
t5=C_u_i_cdr(t4);
t6=C_i_check_list_2(t5,lf[212]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15978,a[2]=t8,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_15978(t10,((C_word*)t0)[4],t5);}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* for-each-loop3159 in k15959 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_15978(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_15978,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15988,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1537: g3160 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_15916(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15986 in for-each-loop3159 in k15959 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_15988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15988,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15978(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop3142 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16005(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16005,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16015,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1535: g3143 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_15916(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16013 in for-each-loop3142 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16015,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16005(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k16033 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_16035,c,av);}
a=C_alloc(6);
t2=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_u_i_cdr(t3);
t5=C_i_check_list_2(t4,lf[212]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16051,a[2]=t7,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_16051(t9,((C_word*)t0)[4],t4);}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* for-each-loop3194 in k16033 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16051(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16051,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16061,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1541: g3195 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_15916(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16059 in for-each-loop3194 in k16033 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16061(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16061,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16051(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop3177 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16078(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16078,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16088,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1539: g3178 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_15916(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16086 in for-each-loop3177 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16088,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16078(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k16118 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16120,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* scrutinizer.scm:1549: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_15916(t3,((C_word*)t0)[4],C_i_caddr(((C_word*)t0)[5]));}

/* k16160 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16162(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16162,c,av);}
/* scrutinizer.scm:1543: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop3212 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16164(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_16164,3,t0,t1,t2);}
a=C_alloc(15);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16189,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
if(C_truep(C_i_symbolp(t4))){
t5=t3;
f_16189(t5,C_a_i_list3(&a,3,t4,C_SCHEME_FALSE,C_SCHEME_FALSE));}
else{
t5=C_i_car(t4);
t6=C_i_cadr(t4);
t7=t3;
f_16189(t7,C_a_i_list3(&a,3,t5,C_SCHEME_FALSE,t6));}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16187 in map-loop3212 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16189(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_16189,2,t0,t1);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16164(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* for-each-loop3240 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16211(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16211,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16221,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1551: g3241 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_15916(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16219 in for-each-loop3240 in loop in chicken.compiler.scrutinizer#type-typeenv in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16221,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_16211(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.scrutinizer#trail-restore in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16242(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_16242,3,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16248,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(
  f_16248(t4,lf[24])
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* doloop3260 in chicken.compiler.scrutinizer#trail-restore in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_16248(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
t2=C_eqp(t1,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_SCHEME_UNDEFINED;
return(t3);}
else{
t3=C_i_car(t1);
t4=C_i_assq(t3,((C_word*)t0)[3]);
t5=C_i_cdr(t4);
t6=C_i_set_car(t5,C_SCHEME_FALSE);
t8=C_u_i_cdr(t1);
t1=t8;
goto loop;}}

/* k16278 in k14771 in chicken.compiler.scrutinizer#type<=? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16280,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16283,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1563: trail-restore */
f_16242(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k16281 in k16278 in k14771 in chicken.compiler.scrutinizer#type<=? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16283,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_16285,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16293,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16295,a[2]=t3,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_16295(t8,t4,t2,C_SCHEME_END_OF_LIST);}

/* k16291 in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16293,c,av);}
/* scrutinizer.scm:1567: simplify-type */
f_13503(((C_word*)t0)[2],t1);}

/* resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16295(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_16295,4,t0,t1,t2,t3);}
a=C_alloc(20);
t4=C_i_assq(t2,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16303,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1569: g3284 */
t6=t5;
f_16303(t6,t1,t4);}
else{
t5=C_i_pairp(t2);
if(C_truep(C_i_not(t5))){
t6=C_i_memq(t2,lf[19]);
if(C_truep(t6)){
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* scrutinizer.scm:1581: chicken.compiler.support#bomb */
t7=*((C_word*)lf[343]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[403];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
if(C_truep(C_i_memq(t2,lf[21]))){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* scrutinizer.scm:1581: chicken.compiler.support#bomb */
t7=*((C_word*)lf[343]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[403];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}}
else{
t6=C_i_car(t2);
t7=C_eqp(t6,lf[28]);
if(C_truep(t7)){
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16384,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t13=C_u_i_cdr(t2);
t14=C_i_check_list_2(t13,lf[6]);
t15=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16395,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16397,a[2]=t10,a[3]=t17,a[4]=t12,a[5]=t11,tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_16397(t19,t15,t13);}
else{
t8=C_eqp(t6,lf[69]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16442,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1585: resolve */
t20=t9;
t21=C_i_cadr(t2);
t22=t3;
t1=t20;
t2=t21;
t3=t22;
goto loop;}
else{
t9=C_eqp(t6,lf[29]);
t10=(C_truep(t9)?t9:C_eqp(t6,lf[372]));
if(C_truep(t10)){
t11=C_u_i_car(t2);
t12=C_i_cadr(t2);
t13=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16468,a[2]=t1,a[3]=t11,a[4]=t12,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1587: resolve */
t20=t13;
t21=C_i_caddr(t2);
t22=t3;
t1=t20;
t2=t21;
t3=t22;
goto loop;}
else{
t11=C_eqp(t6,lf[48]);
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16481,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=t1,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t11)){
t13=t12;
f_16481(t13,t11);}
else{
t13=C_eqp(t6,lf[47]);
if(C_truep(t13)){
t14=t12;
f_16481(t14,t13);}
else{
t14=C_eqp(t6,lf[50]);
if(C_truep(t14)){
t15=t12;
f_16481(t15,t14);}
else{
t15=C_eqp(t6,lf[359]);
t16=t12;
f_16481(t16,(C_truep(t15)?t15:C_eqp(t6,lf[361])));}}}}}}}}}

/* g3284 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16303(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_16303,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_i_cadr(t2);
t4=C_i_not(t3);
t5=(C_truep(t4)?t4:C_i_memq(t3,((C_word*)t0)[2]));
if(C_truep(t5)){
if(C_truep(C_i_caddr(t2))){
t6=C_i_caddr(t2);
t7=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[2]);
/* scrutinizer.scm:1575: resolve */
t8=((C_word*)((C_word*)t0)[4])[1];
f_16295(t8,t1,t6,t7);}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t6=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[2]);
/* scrutinizer.scm:1577: resolve */
t7=((C_word*)((C_word*)t0)[4])[1];
f_16295(t7,t1,t3,t6);}}

/* g3311 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16384(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16384,3,t0,t1,t2);}
/* scrutinizer.scm:1584: g3328 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_16295(t3,t1,t2,((C_word*)t0)[3]);}

/* k16393 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16395,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[28],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop3305 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16397(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_16397,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16422,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1584: g3311 */
t4=((C_word*)t0)[4];
f_16384(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16420 in map-loop3305 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16422,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16397(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k16440 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_16442,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[69],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16466 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_16468,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list3(&a,3,((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16481(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_16481,2,t0,t1);}
a=C_alloc(21);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16491,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t8=C_u_i_cdr(((C_word*)t0)[2]);
t9=C_i_check_list_2(t8,lf[6]);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16502,a[2]=((C_word*)t0)[5],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16504,a[2]=t5,a[3]=t12,a[4]=t7,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_16504(t14,t10,t8);}
else{
t2=C_eqp(((C_word*)t0)[6],lf[84]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16545,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1591: procedure-name */
f_15171(t3,((C_word*)t0)[2]);}
else{
t3=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* g3363 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16491(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16491,3,t0,t1,t2);}
/* scrutinizer.scm:1589: g3380 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_16295(t3,t1,t2,((C_word*)t0)[3]);}

/* k16500 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16502,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop3357 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16504(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_16504,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16529,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1589: g3363 */
t4=((C_word*)t0)[4];
f_16491(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16527 in map-loop3357 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16529,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16504(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16545,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16548,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1592: procedure-arguments */
f_15222(t2,((C_word*)t0)[5]);}

/* k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16548,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16551,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1593: procedure-results */
f_15279(t2,((C_word*)t0)[6]);}

/* k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16551(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_16551,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16558,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(C_truep(((C_word*)t0)[3])?C_a_i_list1(&a,1,((C_word*)t0)[3]):C_SCHEME_END_OF_LIST);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16570,a[2]=t2,a[3]=t3,a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16627,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_16627(t8,t4,((C_word*)t0)[6]);}

/* k16556 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16558(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16558,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[84],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16568 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_16570,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16574,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_eqp(lf[53],((C_word*)t0)[4]);
if(C_truep(t3)){
t4=C_a_i_cons(&a,2,t1,lf[53]);
/* scrutinizer.scm:1594: ##sys#append */
t5=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16581,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t9=C_i_check_list_2(((C_word*)t0)[4],lf[6]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16593,a[2]=t6,a[3]=t11,a[4]=t8,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_16593(t13,t2,((C_word*)t0)[4]);}}

/* k16572 in k16568 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_16574,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* scrutinizer.scm:1594: ##sys#append */
t3=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* g3410 in k16568 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16581(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16581,3,t0,t1,t2);}
/* scrutinizer.scm:1607: g3427 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_16295(t3,t1,t2,((C_word*)t0)[3]);}

/* map-loop3404 in k16568 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16593(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_16593,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16618,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1607: g3410 */
t4=((C_word*)t0)[4];
f_16581(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16616 in map-loop3404 in k16568 in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16618,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16593(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* loop in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16627(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_16627,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_eqp(lf[365],t3);
if(C_truep(t4)){
if(C_truep(C_i_equalp(lf[405],C_u_i_cdr(t2)))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_u_i_car(t2);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16655,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1601: loop */
t9=t6;
t10=C_u_i_cdr(t2);
t1=t9;
t2=t10;
goto loop;}}
else{
t5=C_eqp(lf[364],C_u_i_car(t2));
if(C_truep(t5)){
t6=C_u_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16674,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1603: loop */
t9=t7;
t10=C_u_i_cdr(t2);
t1=t9;
t2=t10;
goto loop;}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16683,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1604: resolve */
t7=((C_word*)((C_word*)t0)[3])[1];
f_16295(t7,t6,C_u_i_car(t2),((C_word*)t0)[4]);}}}}

/* k16653 in loop in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16655,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16672 in loop in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16674(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16674,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16681 in loop in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16683,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16687,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1604: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_16627(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k16685 in k16681 in loop in k16549 in k16546 in k16543 in k16479 in resolve in chicken.compiler.scrutinizer#resolve in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16687,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16750,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16754,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_rest_nullp(c,4))){
/* scrutinizer.scm:1614: chicken.platform#repository-path */
t5=*((C_word*)lf[431]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_get_rest_arg(c,4,av,4,t0);
f_16754(2,av2);}}}

/* k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_16754,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16757,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_not(t1))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17135,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1616: ##sys#file-exists? */
t4=*((C_word*)lf[429]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
/* scrutinizer.scm:1617: chicken.load#find-file */
t3=*((C_word*)lf[430]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16757(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16757,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16763,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17108,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1618: chicken.base#open-output-string */
t4=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_16763,c,av);}
a=C_alloc(15);
t2=C_SCHEME_FALSE;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16768,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16773,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17101,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1619: ##sys#dynamic-wind */
t9=*((C_word*)lf[224]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a16767 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16768(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16768,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[17]);
t3=C_mutate(&lf[17] /* (set! chicken.compiler.scrutinizer#scrutiny-debug ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_16773,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17070,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1680: scheme#call-with-input-file */
t3=*((C_word*)lf[424]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=*((C_word*)lf[425]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_16782,c,av);}
a=C_alloc(12);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_i_pairp(t2);
t4=(C_truep(t3)?C_u_i_cdr(C_u_i_cdr(((C_word*)t0)[2])):C_SCHEME_FALSE);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16788,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_u_i_car(t6);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16887,a[2]=((C_word*)t0)[3],a[3]=t9,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_16887(t11,t5,t7);}

/* k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_16788,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16793,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16799,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1659: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a16792 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16793,c,av);}
/* scrutinizer.scm:1659: validate-type */
t2=*((C_word*)lf[407]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16799(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_16799,c,av);}
a=C_alloc(13);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16803,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_16803(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16854,a[2]=t5,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1662: chicken.base#open-output-string */
t7=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_16803,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16821,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16847,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1665: compatible-types? */
f_14710(t4,((C_word*)t0)[6],((C_word*)t0)[5],C_SCHEME_END_OF_LIST);}
else{
t4=t3;
f_16821(t4,C_SCHEME_FALSE);}}

/* k16804 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16806,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16809,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1676: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[233]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[65];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16807 in k16804 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_16809(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16809,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16812,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1677: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[233]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[66];
av2[4]=lf[409];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16810 in k16807 in k16804 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_16812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16812,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* scrutinizer.scm:1679: install-specializations */
t2=*((C_word*)lf[408]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16819 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16821(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_16821,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16828,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16832,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1668: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[411];
av2[3]=lf[412];
av2[4]=lf[413];
av2[5]=lf[414];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_16806(2,av2);}}}

/* k16826 in k16819 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_16828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16828,c,av);}
/* scrutinizer.scm:1666: chicken.base#warning */
t2=*((C_word*)lf[90]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16830 in k16819 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_16832(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_16832,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16836,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1674: type->pp-string */
f_20636(t2,((C_word*)t0)[5]);}

/* k16834 in k16830 in k16819 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_16836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_16836,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16840,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1675: type->pp-string */
f_20636(t2,((C_word*)t0)[5]);}

/* k16838 in k16834 in k16830 in k16819 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_16840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_16840,c,av);}
/* scrutinizer.scm:1667: chicken.format#sprintf */
t2=*((C_word*)lf[410]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k16845 in k16801 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16847,c,av);}
t2=((C_word*)t0)[2];
f_16821(t2,C_i_not(t1));}

/* k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16854,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16860,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1662: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[416];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16860,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16863,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1662: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_16863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16863,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16866,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1662: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[415];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_16866(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16866,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16869,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1662: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16867 in k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_16869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16869,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16872,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1662: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k16870 in k16867 in k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_16872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16872,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16875,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16882,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1664: type->pp-string */
f_20636(t3,((C_word*)t0)[4]);}

/* k16873 in k16870 in k16867 in k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_16875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16875,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16878,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1662: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16876 in k16873 in k16870 in k16867 in k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_16878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16878,c,av);}
/* scrutinizer.scm:1661: chicken.base#warning */
t2=*((C_word*)lf[90]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16880 in k16870 in k16867 in k16864 in k16861 in k16858 in k16852 in a16798 in k16786 in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_16882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16882,c,av);}
/* scrutinizer.scm:1662: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16887(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_16887,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=C_i_vectorp(C_u_i_car(t2));
t4=(C_truep(t3)?C_eqp(lf[84],C_i_vector_ref(C_u_i_car(t2),C_fix(0))):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16903,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17019,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1630: scheme#vector->list */
t7=*((C_word*)lf[51]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=C_eqp(lf[29],C_u_i_car(t2));
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17038,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1654: adjust */
t9=t7;
t10=C_i_caddr(t2);
t1=t9;
t2=t10;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16901 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16903(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16903,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[84],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_16914(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_16914,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_eqp(t3,lf[417]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16933,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1634: chicken.compiler.support#mark-variable */
t6=*((C_word*)lf[233]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[272];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=C_eqp(t3,lf[418]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16947,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1637: chicken.compiler.support#mark-variable */
t7=*((C_word*)lf[233]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[271];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t6=C_eqp(t3,lf[419]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16961,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1640: chicken.compiler.support#mark-variable */
t8=*((C_word*)lf[233]+1);{
C_word av2[5];
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[276];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t7=C_eqp(t3,lf[420]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16975,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1643: chicken.compiler.support#mark-variable */
t9=*((C_word*)lf[233]+1);{
C_word av2[5];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[421];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t8=C_eqp(t3,lf[422]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16989,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1646: chicken.compiler.support#mark-variable */
t10=*((C_word*)lf[233]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t9;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[165];
av2[4]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
/* scrutinizer.scm:1649: chicken.compiler.support#bomb */
t9=*((C_word*)lf[343]+1);{
C_word av2[5];
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[423];
av2[3]=C_u_i_car(t2);
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}}}}}}}

/* k16931 in loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16933,c,av);}
/* scrutinizer.scm:1635: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16914(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k16945 in loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16947,c,av);}
/* scrutinizer.scm:1638: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16914(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k16959 in loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16961,c,av);}
/* scrutinizer.scm:1641: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16914(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k16973 in loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16975,c,av);}
/* scrutinizer.scm:1644: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16914(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k16987 in loop in k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_16989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16989,c,av);}
/* scrutinizer.scm:1647: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16914(t2,((C_word*)t0)[3],C_i_cddr(((C_word*)t0)[4]));}

/* k17017 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17019,c,av);}
a=C_alloc(7);
t2=C_i_cdr(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16914,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_16914(t6,((C_word*)t0)[4],t2);}

/* k17036 in adjust in k16780 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_17038,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[29],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17070,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[212]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17076,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17078,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_17078(t7,t3,t1);}

/* k17074 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17076,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17078(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_17078,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17088,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_car(t4);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16782,a[2]=t4,a[3]=t5,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1623: chicken.compiler.support#variable-mark */
t7=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=t5;
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k17086 in for-each-loop3475 in k17068 in a16772 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17088,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_17078(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a17100 in k16761 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17101(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17101,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[17]);
t3=C_mutate(&lf[17] /* (set! chicken.compiler.scrutinizer#scrutiny-debug ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17108,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17114,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1618: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[428];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k17112 in k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_17114,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17117,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1618: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17115 in k17112 in k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17117(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_17117,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17120,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1618: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[427];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17118 in k17115 in k17112 in k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17120,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17123,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1618: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17121 in k17118 in k17115 in k17112 in k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17123,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17126,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1618: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17124 in k17121 in k17118 in k17115 in k17112 in k17106 in k16755 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17126,c,av);}
/* scrutinizer.scm:1618: chicken.compiler.support#debugging */
t2=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[426];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17133 in k16752 in chicken.compiler.scrutinizer#load-type-database in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17135(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17135,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?((C_word*)t0)[3]:C_SCHEME_FALSE);
f_16757(2,av2);}}

/* chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17149,c,av);}
a=C_alloc(5);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17155,a[2]=t5,a[3]=t4,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1684: scheme#with-output-to-file */
t7=*((C_word*)lf[446]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
av2[2]=t3;
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17155(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_17155,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17159,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17317,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1686: chicken.platform#chicken-version */
t4=*((C_word*)lf[445]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_17159,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17162,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17167,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1688: chicken.internal#hash-table-for-each */
t4=*((C_word*)lf[441]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k17160 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17162(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17162,c,av);}
/* scrutinizer.scm:1719: chicken.base#print */
t2=*((C_word*)lf[355]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[433];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17167,c,av);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17174,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17308,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1690: chicken.compiler.support#variable-visible? */
t6=*((C_word*)lf[247]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17174(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_17174,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17177,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1692: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[162];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17177(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17177,c,av);}
a=C_alloc(5);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17183,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1693: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17183,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17186,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1694: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[165];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17186,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17189,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1695: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[272];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17189,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17192,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1696: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[271];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_17192,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17195,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:1697: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[276];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_17195,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17198,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:1698: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[421];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_17198,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17201,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17208,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17212,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17214,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=t1,a[7]=t6,tmp=(C_word)a,a+=8,tmp));
t8=((C_word*)t6)[1];
f_17214(t8,t4,((C_word*)t0)[9]);}

/* k17199 in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_17201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17201,c,av);}
/* scrutinizer.scm:1717: scheme#newline */
t2=*((C_word*)lf[434]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17206 in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_17208(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17208,c,av);}
/* scrutinizer.scm:1699: chicken.pretty-print#pp */
t2=*((C_word*)lf[435]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k17210 in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_17212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_17212,c,av);}
a=C_alloc(11);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7195,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_7195(t6,((C_word*)t0)[3],((C_word*)t0)[4],t2);}

/* wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_17214(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(25,0,3)))){
C_save_and_reclaim_args((void *)trf_17214,3,t0,t1,t2);}
a=C_alloc(25);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[84]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17235,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17245,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=(C_truep(((C_word*)t0)[2])?lf[436]:C_SCHEME_END_OF_LIST);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17253,a[2]=t6,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
t9=(C_truep(((C_word*)t0)[3])?C_a_i_list(&a,2,lf[422],((C_word*)t0)[3]):C_SCHEME_END_OF_LIST);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17261,a[2]=t8,a[3]=t9,tmp=(C_word)a,a+=4,tmp);
t11=(C_truep(((C_word*)t0)[4])?lf[437]:C_SCHEME_END_OF_LIST);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17269,a[2]=t10,a[3]=t11,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[5])){
t13=lf[438];
if(C_truep(((C_word*)t0)[6])){
/* scrutinizer.scm:1705: ##sys#append */
t14=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t12;
av2[2]=t13;
av2[3]=lf[439];
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}
else{
/* scrutinizer.scm:1705: ##sys#append */
t14=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t12;
av2[2]=t13;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}}
else{
t13=C_SCHEME_END_OF_LIST;
if(C_truep(((C_word*)t0)[6])){
/* scrutinizer.scm:1705: ##sys#append */
t14=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t12;
av2[2]=t13;
av2[3]=lf[439];
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}
else{
/* scrutinizer.scm:1705: ##sys#append */
t14=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t12;
av2[2]=t13;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}}}
else{
t5=C_eqp(t3,lf[29]);
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17297,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1713: wrap */
t15=t7;
t16=C_i_caddr(t2);
t1=t15;
t2=t16;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17233 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17235,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17243 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17245,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[84],t1);
/* scrutinizer.scm:1705: ##sys#list->vector */
t3=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17251 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17253,c,av);}
/* scrutinizer.scm:1705: ##sys#append */
t2=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17259 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17261(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17261,c,av);}
/* scrutinizer.scm:1705: ##sys#append */
t2=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17267 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17269,c,av);}
/* scrutinizer.scm:1705: ##sys#append */
t2=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17295 in wrap in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_17297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_17297,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[29],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17306 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17308(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17308,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17312,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1691: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[66];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_17174(t2,C_SCHEME_FALSE);}}

/* k17310 in k17306 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17312,c,av);}
t2=((C_word*)t0)[2];
f_17174(t2,C_u_i_memq(t1,lf[440]));}

/* k17315 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17317,c,av);}
/* scrutinizer.scm:1686: chicken.base#print */
t2=*((C_word*)lf[355]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[442];
av2[3]=t1;
av2[4]=lf[443];
av2[5]=((C_word*)t0)[3];
av2[6]=lf[444];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* chicken.compiler.scrutinizer#mutate-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17320(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_17320,3,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17325,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17336,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1734: chicken.compiler.support#copy-node */
t6=*((C_word*)lf[450]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k17323 in chicken.compiler.scrutinizer#mutate-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17325,c,av);}
a=C_alloc(4);
t2=C_mutate(&lf[447] /* (set! chicken.compiler.scrutinizer#node-mutations ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17332,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1735: chicken.compiler.support#build-node-graph */
t4=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17330 in k17323 in chicken.compiler.scrutinizer#mutate-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17332,c,av);}
/* scrutinizer.scm:1735: chicken.compiler.support#copy-node! */
t2=*((C_word*)lf[448]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17334 in chicken.compiler.scrutinizer#mutate-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17336,c,av);}
/* scrutinizer.scm:1734: chicken.base#alist-update! */
t2=*((C_word*)lf[186]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=lf[447];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.scrutinizer#source-node in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17338(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_17338,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_i_nullp(t3);
t5=(C_truep(t4)?*((C_word*)lf[452]+1):C_i_car(t3));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17345,a[2]=t5,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1738: chicken.base#alist-ref */
t7=*((C_word*)lf[275]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=lf[447];
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k17343 in chicken.compiler.scrutinizer#source-node in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17345,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
/* scrutinizer.scm:1739: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
/* scrutinizer.scm:1739: source-node */
f_17338(((C_word*)t0)[3],t1,C_a_i_list(&a,1,((C_word*)t0)[2]));}}

/* chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17365(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17365,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17371,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:1742: source-node */
f_17338(t1,t2,C_a_i_list(&a,1,t3));}

/* a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17371,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17379,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1745: chicken.compiler.support#node-class */
t4=*((C_word*)lf[88]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17379(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17379,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17383,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1746: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17381 in k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17383(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_17383,c,av);}
a=C_alloc(12);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=lf[453];
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17390,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t4,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1747: chicken.compiler.support#node-subexpressions */
t8=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k17388 in k17381 in k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_17390,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(t1,lf[6]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17396,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17398,a[2]=((C_word*)t0)[5],a[3]=t5,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_17398(t7,t3,t1);}

/* k17394 in k17388 in k17381 in k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17396,c,av);}
/* scrutinizer.scm:1745: chicken.compiler.support#make-node */
t2=*((C_word*)lf[269]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop3626 in k17388 in k17381 in k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17398(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_17398,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17423,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1747: g3632 */
t4=lf[453];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_17365(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17421 in map-loop3626 in k17388 in k17381 in k17377 in a17370 in chicken.compiler.scrutinizer#source-node-tree in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17423,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_17398(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k17438 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_17440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17440,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17446,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1753: chicken.compiler.support#node-class */
t3=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17444 in k17438 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_17446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17446,c,av);}
a=C_alloc(3);
t2=C_eqp(t1,lf[120]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17455,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1755: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_eqp(t1,lf[311]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17494,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1760: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_21235(2,av2);}}}}

/* k17453 in k17444 in k17438 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_17455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17455,c,av);}
t2=C_i_cdr(t1);
if(C_truep(C_i_pairp(t2))){
t3=C_i_cadr(t1);
if(C_truep(C_i_pairp(t3))){
t4=C_u_i_cdr(t1);
/* scrutinizer.scm:1758: chicken.compiler.support#source-info->line */
t5=*((C_word*)lf[510]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=C_u_i_car(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_21235(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21235(2,av2);}}}

/* k17492 in k17444 in k17438 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_17494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17494,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_car(t1);
f_21235(2,av2);}}

/* chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17500(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_17500,4,t1,t2,t3,t4);}
a=C_alloc(13);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17503,a[2]=t3,a[3]=t6,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17651,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1789: subst */
t11=((C_word*)t8)[1];
f_17503(t11,t10,t4);}

/* subst in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17503(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(58,0,2)))){
C_save_and_reclaim_args((void *)trf_17503,3,t0,t1,t2);}
a=C_alloc(58);
t3=C_i_vectorp(t2);
t4=(C_truep(t3)?C_eqp(C_fix(1),C_block_size(t2)):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_i_vector_ref(t2,C_fix(0));
if(C_truep(C_i_integerp(t5))){
if(C_truep(C_i_negativep(t5))){
t6=C_s_a_i_negate(&a,1,t5);
t7=C_s_a_i_minus(&a,2,t6,C_fix(1));
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_i_list_tail(((C_word*)t0)[2],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t6=C_s_a_i_minus(&a,2,t5,C_fix(1));
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_i_list_ref(((C_word*)t0)[2],t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
if(C_truep(C_i_symbolp(t5))){
t6=C_i_assq(t5,((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_i_cdr(t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17550,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1778: chicken.base#gensym */
t8=*((C_word*)lf[197]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17560,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_vectorp(t2))){
t6=C_block_size(t2);
t7=C_eqp(C_fix(2),t6);
if(C_truep(t7)){
t8=C_i_vector_ref(t2,C_fix(0));
t9=t5;
f_17560(t9,(C_truep(C_i_integerp(t8))?C_eqp(lf[454],C_i_vector_ref(t2,C_fix(1))):C_SCHEME_FALSE));}
else{
t8=t5;
f_17560(t8,C_SCHEME_FALSE);}}
else{
t6=t5;
f_17560(t6,C_SCHEME_FALSE);}}}

/* k17548 in subst in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17550(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_17550,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k17558 in subst in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17560(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,2)))){
C_save_and_reclaim_args((void *)trf_17560,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t3=C_s_a_i_minus(&a,2,t2,C_fix(1));
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_list_tail(((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_i_pairp(((C_word*)t0)[2]);
if(C_truep(C_i_not(t2))){
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(((C_word*)t0)[2]);
t4=C_eqp(lf[123],t3);
if(C_truep(t4)){
t5=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17587,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1788: subst */
t6=((C_word*)((C_word*)t0)[5])[1];
f_17503(t6,t5,C_u_i_car(((C_word*)t0)[2]));}}}}

/* k17585 in k17558 in subst in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17587,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17591,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1788: subst */
t3=((C_word*)((C_word*)t0)[3])[1];
f_17503(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k17589 in k17585 in k17558 in subst in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17591,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17649 in chicken.compiler.scrutinizer#specialize-node! in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17651,c,av);}
/* scrutinizer.scm:1789: mutate-node! */
f_17320(((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(40,c,4)))){
C_save_and_reclaim((void *)f_17653,c,av);}
a=C_alloc(40);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_END_OF_LIST;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_END_OF_LIST;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17684,tmp=(C_word)a,a+=2,tmp));
t19=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17718,a[2]=t15,a[3]=t17,tmp=(C_word)a,a+=4,tmp));
t20=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17848,a[2]=t9,a[3]=t17,a[4]=t11,a[5]=t5,a[6]=t13,a[7]=t7,a[8]=t3,a[9]=t15,tmp=(C_word)a,a+=10,tmp));
t21=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18905,a[2]=t5,a[3]=t7,a[4]=t9,a[5]=t11,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1967: validate */
t22=((C_word*)t17)[1];
f_17848(t22,t21,t2,C_a_i_list(&a,1,C_SCHEME_FALSE));}

/* loop in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17662(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_17662,3,t0,t1,t2);}
a=C_alloc(4);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17680,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1814: loop */
t7=t5;
t8=C_u_i_cdr(t2);
t1=t7;
t2=t8;
goto loop;}}

/* k17678 in loop in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17680,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* memq* in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17684(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_17684,3,t1,t2,t3);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17690,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(
  f_17690(t4,t3)
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* loop in memq* in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_17690(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
t2=C_i_pairp(t1);
if(C_truep(C_i_not(t2))){
return(C_SCHEME_FALSE);}
else{
t3=C_i_car(t1);
t4=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t4)){
return(t1);}
else{
t6=C_u_i_cdr(t1);
t1=t6;
goto loop;}}}

/* validate-llist in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17718(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_17718,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_symbolp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[455];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(lf[364],t4);
t6=(C_truep(t5)?t5:C_eqp(lf[456],C_u_i_car(t2)));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17749,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1826: validate-llist */
t12=t7;
t13=C_u_i_cdr(t2);
t1=t12;
t2=t13;
goto loop;}
else{
t7=C_eqp(lf[365],C_u_i_car(t2));
t8=(C_truep(t7)?t7:C_eqp(lf[457],C_u_i_car(t2)));
if(C_truep(t8)){
if(C_truep(C_i_nullp(C_u_i_cdr(t2)))){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[458];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_i_pairp(C_u_i_cdr(t2));
if(C_truep(C_i_not(t9))){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17781,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1833: validate */
t11=((C_word*)((C_word*)t0)[3])[1];
f_17848(t11,t10,C_i_cadr(t2),C_SCHEME_END_OF_LIST);}}}
else{
t9=C_eqp(lf[459],C_u_i_car(t2));
if(C_truep(t9)){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=lf[460];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17808,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1837: validate */
t11=((C_word*)((C_word*)t0)[3])[1];
f_17848(t11,t10,C_u_i_car(t2),C_SCHEME_END_OF_LIST);}}}}}}}

/* k17747 in validate-llist in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17749,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_cons(&a,2,lf[364],t1):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17779 in validate-llist in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_17781,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_list(&a,2,lf[365],t1):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k17806 in validate-llist in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17808(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17808,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17811,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1838: validate-llist */
t3=((C_word*)((C_word*)t0)[3])[1];
f_17718(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k17809 in k17806 in validate-llist in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17811,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(((C_word*)t0)[3])?(C_truep(t1)?C_a_i_cons(&a,2,((C_word*)t0)[3],t1):C_SCHEME_FALSE):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_17848(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,3)))){
C_save_and_reclaim_args((void *)trf_17848,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t5=C_i_nullp(t3);
t6=(C_truep(t5)?C_SCHEME_TRUE:C_i_car(t3));
if(C_truep(C_i_memq(((C_word*)t4)[1],lf[19]))){
t7=((C_word*)t4)[1];
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
if(C_truep(C_i_memq(((C_word*)t4)[1],lf[21]))){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=((C_word*)t4)[1];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
if(C_truep(C_i_memq(((C_word*)t4)[1],lf[461]))){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_a_i_list(&a,2,lf[52],((C_word*)t4)[1]);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(((C_word*)t4)[1],lf[462]);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[463];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_eqp(((C_word*)t4)[1],lf[464]);
if(C_truep(t8)){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(((C_word*)t4)[1],lf[465]);
if(C_truep(t9)){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=lf[35];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_eqp(((C_word*)t4)[1],lf[466]);
if(C_truep(t10)){
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=lf[467];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(((C_word*)t4)[1],lf[468]);
if(C_truep(t11)){
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=lf[469];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t12=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17906,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=t6,a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],a[12]=((C_word*)t0)[9],tmp=(C_word)a,a+=13,tmp);
if(C_truep(C_i_symbolp(((C_word*)t4)[1]))){
/* scrutinizer.scm:1849: ##sys#get */
t13=*((C_word*)lf[474]+1);{
C_word av2[4];
av2[0]=t13;
av2[1]=t12;
av2[2]=((C_word*)t4)[1];
av2[3]=lf[475];
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}
else{
t13=t12;{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
f_17906(2,av2);}}}}}}}}}}}

/* k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_17906,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_pairp(((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(C_i_not(t2))){
if(C_truep(C_i_memq(((C_word*)((C_word*)t0)[3])[1],((C_word*)((C_word*)t0)[4])[1]))){
t3=((C_word*)((C_word*)t0)[3])[1];
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=C_i_car(((C_word*)((C_word*)t0)[3])[1]);
t4=C_eqp(lf[69],t3);
if(C_truep(t4)){
t5=C_i_length(((C_word*)((C_word*)t0)[3])[1]);
t6=C_eqp(C_fix(2),t5);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17940,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=((C_word*)((C_word*)t0)[3])[1];
/* scrutinizer.scm:1855: validate */
t9=((C_word*)((C_word*)t0)[5])[1];
f_17848(t9,t7,C_i_cadr(t8),C_SCHEME_END_OF_LIST);}
else{
t7=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t5=C_i_car(((C_word*)((C_word*)t0)[3])[1]);
t6=C_eqp(lf[29],t5);
if(C_truep(t6)){
t7=C_i_length(((C_word*)((C_word*)t0)[3])[1]);
t8=C_eqp(C_fix(3),t7);
if(C_truep(t8)){
t9=((C_word*)((C_word*)t0)[3])[1];
t10=C_i_cadr(t9);
if(C_truep(C_i_listp(t10))){
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17972,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1859: chicken.base#call/cc */
t12=*((C_word*)lf[167]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t12;
av2[1]=((C_word*)t0)[2];
av2[2]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t11=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}
else{
t9=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t7=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_18122,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
t8=C_i_car(((C_word*)((C_word*)t0)[3])[1]);
t9=C_eqp(lf[123],t8);
if(C_truep(t9)){
t10=C_i_cdr(((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(C_i_pairp(t10))){
t11=((C_word*)((C_word*)t0)[3])[1];
t12=C_i_cadr(t11);
if(C_truep(C_i_symbolp(t12))){
t13=C_i_cddr(((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(C_i_nullp(t13))){
t14=((C_word*)((C_word*)t0)[3])[1];
t15=t7;
f_18122(t15,C_i_cadr(t14));}
else{
t14=t7;
f_18122(t14,C_SCHEME_FALSE);}}
else{
t13=t7;
f_18122(t13,C_SCHEME_FALSE);}}
else{
t11=t7;
f_18122(t11,C_SCHEME_FALSE);}}
else{
t10=t7;
f_18122(t10,C_SCHEME_FALSE);}}}}}}

/* k17938 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_17940,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[69],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_17972,c,av);}
a=C_alloc(29);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17977,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18029,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=((C_word*)((C_word*)t0)[4])[1];
t10=C_i_cadr(t9);
t11=C_i_check_list_2(t10,lf[6]);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18076,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18078,a[2]=t6,a[3]=t14,a[4]=t8,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_18078(t16,t12,t10);}

/* k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17977(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_17977,c,av);}
a=C_alloc(15);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17981,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17992,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17994,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t6=((C_word*)((C_word*)t0)[4])[1];
/* scrutinizer.scm:1872: filter-map */
f_7409(t4,t5,C_i_cadr(t6));}

/* k17979 in k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17981,c,av);}
a=C_alloc(3);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)((C_word*)t0)[3])[1];
/* scrutinizer.scm:1881: validate */
t4=((C_word*)((C_word*)t0)[4])[1];
f_17848(t4,((C_word*)t0)[5],C_i_caddr(t3),C_a_i_list(&a,1,((C_word*)t0)[6]));}

/* k17990 in k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17992,c,av);}
/* scrutinizer.scm:1872: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a17993 in k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_17994(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17994,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18010,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1876: validate */
t5=((C_word*)((C_word*)t0)[3])[1];
f_17848(t5,t4,C_i_cadr(t2),C_SCHEME_END_OF_LIST);}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18008 in a17993 in k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18010(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_18010,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18013,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* scrutinizer.scm:1877: return */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k18011 in k18008 in a17993 in k17975 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_18013,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g3794 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18029(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_18029,3,t0,t1,t2);}
if(C_truep(C_i_symbolp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_listp(t2))){
t3=C_eqp(C_fix(2),C_u_i_length(t2));
if(C_truep(t3)){
t4=C_i_car(t2);
if(C_truep(C_i_symbolp(t4))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* scrutinizer.scm:1868: return */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}
else{
/* scrutinizer.scm:1868: return */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}
else{
/* scrutinizer.scm:1868: return */
t3=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k18074 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18076,c,av);}
/* scrutinizer.scm:1862: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop3788 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18078(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18078,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18103,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1862: g3794 */
t4=((C_word*)t0)[4];
f_18029(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18101 in map-loop3788 in a17971 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18103,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18078(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18122(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,3)))){
C_save_and_reclaim_args((void *)trf_18122,2,t0,t1);}
a=C_alloc(16);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18126,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1841: g3830 */
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=(
/* scrutinizer.scm:1841: g3830 */
  f_18126(C_a_i(&a,3),t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t3=C_eqp(lf[28],t2);
if(C_truep(t3)){
if(C_truep(C_i_listp(((C_word*)((C_word*)t0)[4])[1]))){
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_i_cdr(((C_word*)((C_word*)t0)[4])[1]);
t9=C_i_check_list_2(t8,lf[6]);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18164,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18175,a[2]=t6,a[3]=t12,a[4]=((C_word*)t0)[5],a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_18175(t14,t10,t8);}
else{
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t5=C_eqp(lf[52],t4);
if(C_truep(t5)){
t6=C_i_length(((C_word*)((C_word*)t0)[4])[1]);
t7=C_eqp(C_fix(2),t6);
if(C_truep(t7)){
t8=((C_word*)((C_word*)t0)[4])[1];
t9=C_i_cadr(t8);
if(C_truep(C_i_symbolp(t9))){
t10=((C_word*)((C_word*)t0)[4])[1];
t11=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t10=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t8=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t6=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t7=C_eqp(lf[59],t6);
if(C_truep(t7)){
t8=C_i_length(((C_word*)((C_word*)t0)[4])[1]);
t9=C_eqp(C_fix(2),t8);
if(C_truep(t9)){
t10=((C_word*)((C_word*)t0)[4])[1];
t11=C_i_cadr(t10);
t12=C_i_symbolp(t11);
t13=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t13;
av2[1]=(C_truep(t12)?((C_word*)((C_word*)t0)[4])[1]:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t10=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}
else{
t8=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t9=C_eqp(lf[372],t8);
if(C_truep(t9)){
t10=C_i_length(((C_word*)((C_word*)t0)[4])[1]);
t11=C_eqp(C_fix(3),t10);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18276,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t13=((C_word*)((C_word*)t0)[4])[1];
/* scrutinizer.scm:1902: validate */
t14=((C_word*)((C_word*)t0)[5])[1];
f_17848(t14,t12,C_i_caddr(t13),C_SCHEME_END_OF_LIST);}
else{
t12=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t10=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_18345,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:1907: memq* */
f_17684(t10,lf[471],((C_word*)((C_word*)t0)[4])[1]);}}}}}}

/* g3830 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_18126(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
if(C_truep(C_i_memq(t1,((C_word*)((C_word*)t0)[2])[1]))){
return(t1);}
else{
t2=C_a_i_cons(&a,2,t1,((C_word*)((C_word*)t0)[2])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
return(t1);}}

/* k18162 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18164,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18170,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1894: every */
f_7125(t2,*((C_word*)lf[396]+1),t1);}

/* k18168 in k18162 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18170,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_cons(&a,2,lf[28],((C_word*)t0)[3]):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop3838 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18175(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_18175,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18200,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1893: g3844 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_17848(t4,t3,C_slot(t2,C_fix(0)),C_SCHEME_END_OF_LIST);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18198 in map-loop3838 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18200,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18175(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k18274 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18276(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_18276,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18306,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(t1))){
t3=C_eqp(C_u_i_car(t1),lf[52]);
t4=t2;
f_18306(t4,(C_truep(t3)?t3:C_i_memq(t1,lf[19])));}
else{
t3=t2;
f_18306(t3,C_i_memq(t1,lf[19]));}}

/* k18304 in k18274 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18306(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18306,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_i_cadr(t2);
if(C_truep(C_i_listp(t3))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=((C_word*)((C_word*)t0)[2])[1];
/* scrutinizer.scm:1905: every */
f_7125(t4,*((C_word*)lf[470]+1),C_i_cadr(t5));}
else{
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18316 in k18304 in k18274 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_18318,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_i_cadr(t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list3(&a,3,lf[372],t3,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_18345,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_18348,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_18348(2,av2);}}
else{
/* scrutinizer.scm:1907: memq* */
f_17684(t2,lf[473],((C_word*)((C_word*)t0)[4])[1]);}}

/* k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_18348,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18352,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1841: g3889 */
t3=t2;
f_18352(t3,((C_word*)t0)[8],t1);}
else{
t2=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
if(C_truep((C_truep(C_eqp(t2,lf[359]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[361]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
if(C_truep(C_i_listp(((C_word*)((C_word*)t0)[4])[1]))){
t3=C_i_length(((C_word*)((C_word*)t0)[4])[1]);
t4=C_eqp(C_fix(2),t3);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18494,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
t6=((C_word*)((C_word*)t0)[4])[1];
/* scrutinizer.scm:1930: validate */
t7=((C_word*)((C_word*)t0)[3])[1];
f_17848(t7,t5,C_i_cadr(t6),C_SCHEME_END_OF_LIST);}
else{
t5=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
if(C_truep((C_truep(C_eqp(t3,lf[50]))?C_SCHEME_TRUE:(C_truep(C_eqp(t3,lf[47]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
if(C_truep(C_i_listp(((C_word*)((C_word*)t0)[4])[1]))){
t4=C_i_cdr(((C_word*)((C_word*)t0)[4])[1]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18536,a[2]=((C_word*)t0)[4],a[3]=t6,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_18536(t8,((C_word*)t0)[8],t4,C_SCHEME_END_OF_LIST);}
else{
t4=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t5=C_eqp(lf[48],t4);
if(C_truep(t5)){
t6=C_i_length(((C_word*)((C_word*)t0)[4])[1]);
t7=C_eqp(C_fix(3),t6);
if(C_truep(t7)){
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=C_i_cdr(((C_word*)((C_word*)t0)[4])[1]);
t13=C_i_check_list_2(t12,lf[6]);
t14=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18604,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18615,a[2]=t10,a[3]=t16,a[4]=((C_word*)t0)[3],a[5]=t11,tmp=(C_word)a,a+=6,tmp));
t18=((C_word*)t16)[1];
f_18615(t18,t14,t12);}
else{
t8=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t6=C_i_car(((C_word*)((C_word*)t0)[4])[1]);
t7=C_eqp(lf[84],t6);
if(C_truep(t7)){
t8=C_i_cdr(((C_word*)((C_word*)t0)[4])[1]);
if(C_truep(C_i_pairp(t8))){
t9=C_i_cadr(((C_word*)((C_word*)t0)[4])[1]);
t10=C_i_symbolp(t9);
t11=(C_truep(t10)?C_i_cadr(((C_word*)((C_word*)t0)[4])[1]):((C_word*)t0)[9]);
t12=C_i_cadr(((C_word*)((C_word*)t0)[4])[1]);
t13=C_i_symbolp(t12);
t14=(C_truep(t13)?C_i_cddr(((C_word*)((C_word*)t0)[4])[1]):C_i_cdr(((C_word*)((C_word*)t0)[4])[1]));
if(C_truep(C_i_pairp(t14))){
if(C_truep(C_i_listp(C_u_i_car(t14)))){
t15=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18685,a[2]=t14,a[3]=((C_word*)t0)[8],a[4]=t11,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1951: validate-llist */
t16=((C_word*)((C_word*)t0)[10])[1];
f_17718(t16,t15,C_u_i_car(t14));}
else{
t15=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t15;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}}
else{
t15=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t15;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}}
else{
t9=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t8=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}}}}

/* g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18352(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_18352,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_car(t2);
t4=C_eqp(lf[471],t3);
t5=C_i_not(((C_word*)t0)[2]);
t6=(C_truep(t5)?t5:C_i_not(t4));
t7=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18365,a[2]=t6,a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[2])){
t8=t7;
f_18365(t8,C_SCHEME_UNDEFINED);}
else{
t8=C_set_block_item(((C_word*)t0)[7],0,t4);
t9=t7;
f_18365(t9,t8);}}

/* k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18365(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_18365,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_18368,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:1912: memq* */
f_17684(t2,lf[472],((C_word*)t0)[3]);}

/* k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18368(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_18368,c,av);}
a=C_alloc(12);
if(C_truep(C_i_not(t1))){
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18392,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)((C_word*)t0)[7])[1];
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17662,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_17662(t7,t2,t3);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_18400,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[4],a[7]=t1,a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
t3=C_i_length(((C_word*)((C_word*)t0)[7])[1]);
t4=C_eqp(C_fix(5),t3);
if(C_truep(t4)){
t5=C_i_cdr(((C_word*)((C_word*)t0)[7])[1]);
t6=C_eqp(((C_word*)t0)[3],t5);
t7=t2;
f_18400(t7,(C_truep(t6)?C_eqp(t1,C_i_cdddr(((C_word*)((C_word*)t0)[7])[1])):C_SCHEME_FALSE));}
else{
t5=t2;
f_18400(t5,C_SCHEME_FALSE);}}}

/* k18390 in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_18392,c,av);}
a=C_alloc(9);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
t4=C_a_i_cons(&a,2,lf[84],t3);
/* scrutinizer.scm:1915: validate */
t5=((C_word*)((C_word*)t0)[3])[1];
f_17848(t5,((C_word*)t0)[4],t4,C_a_i_list(&a,1,((C_word*)t0)[5]));}

/* k18398 in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18400(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,3)))){
C_save_and_reclaim_args((void *)trf_18400,2,t0,t1);}
a=C_alloc(23);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18404,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=((C_word*)((C_word*)t0)[2])[1];
t4=C_i_car(t3);
t5=C_a_i_list(&a,1,t4);
t6=((C_word*)((C_word*)t0)[2])[1];
t7=C_i_caddr(t6);
t8=C_a_i_list(&a,3,lf[84],t5,t7);
/* scrutinizer.scm:1921: validate */
t9=((C_word*)((C_word*)t0)[6])[1];
f_17848(t9,t2,t8,C_a_i_list(&a,1,((C_word*)t0)[8]));}
else{
t2=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18402 in k18398 in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18404(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18404,c,av);}
a=C_alloc(6);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18415,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1924: validate */
t4=((C_word*)((C_word*)t0)[6])[1];
f_17848(t4,t3,C_i_cadr(((C_word*)t0)[7]),C_SCHEME_END_OF_LIST);}

/* k18413 in k18402 in k18398 in k18366 in k18363 in g3889 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18415(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18415,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
if(C_truep(((C_word*)t0)[4])){
t4=((C_word*)((C_word*)t0)[2])[1];
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k18492 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_18494,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_car(((C_word*)((C_word*)t0)[2])[1]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,t2,t1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* loop in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18536(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_18536,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t4=C_i_car(((C_word*)((C_word*)t0)[2])[1]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18554,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1935: scheme#reverse */
t6=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18557,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1936: validate */
t5=((C_word*)((C_word*)t0)[4])[1];
f_17848(t5,t4,C_i_car(t2),C_SCHEME_END_OF_LIST);}}

/* k18552 in loop in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18554,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k18555 in loop in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18557,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18561,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:1935: g3935 */
t3=t2;
f_18561(t3,((C_word*)t0)[5],t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g3935 in k18555 in loop in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18561(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_18561,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_i_cdr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
/* scrutinizer.scm:1937: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_18536(t5,t1,t3,t4);}

/* k18602 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18604,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18610,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1942: every */
f_7125(t2,*((C_word*)lf[396]+1),t1);}

/* k18608 in k18602 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18610,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_cons(&a,2,lf[48],((C_word*)t0)[3]):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop3943 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18615(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_18615,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18640,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1941: g3949 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_17848(t4,t3,C_slot(t2,C_fix(0)),C_SCHEME_END_OF_LIST);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18638 in map-loop3943 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18640,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18615(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_18685,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18694,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:1953: every */
f_7125(t2,*((C_word*)lf[396]+1),t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_18694,c,av);}
a=C_alloc(22);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18698,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t4=C_eqp(lf[53],t2);
if(C_truep(t4)){
t5=t3;
f_18698(t5,t2);}
else{
if(C_truep(C_i_listp(t2))){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18740,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18748,a[2]=t7,a[3]=t11,a[4]=((C_word*)t0)[7],a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_18748(t13,t9,t2);}
else{
t5=t3;
f_18698(t5,C_SCHEME_FALSE);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18696 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18698(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_18698,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18708,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
t3=C_i_not(((C_word*)t0)[4]);
t4=(C_truep(t3)?C_a_i_list1(&a,1,((C_word*)t0)[3]):C_SCHEME_END_OF_LIST);
t5=C_a_i_cons(&a,2,((C_word*)t0)[5],t1);
/* scrutinizer.scm:1962: ##sys#append */
t6=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t2;
av2[2]=t4;
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_a_i_cons(&a,2,((C_word*)t0)[5],t1);
/* scrutinizer.scm:1962: ##sys#append */
t4=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t2;
av2[2]=C_SCHEME_END_OF_LIST;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18706 in k18696 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18708,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[84],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k18738 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18740(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18740,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18746,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1959: every */
f_7125(t2,*((C_word*)lf[396]+1),t1);}

/* k18744 in k18738 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18746(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18746,c,av);}
t2=((C_word*)t0)[2];
f_18698(t2,(C_truep(t1)?((C_word*)t0)[3]:C_SCHEME_FALSE));}

/* map-loop3985 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18748(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_18748,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18773,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1958: g3991 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_17848(t4,t3,C_slot(t2,C_fix(0)),C_SCHEME_END_OF_LIST);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18771 in map-loop3985 in k18692 in k18683 in k18346 in k18343 in k18120 in k17904 in validate in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18773,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18748(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_18905,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18909,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1967: g4016 */
t3=t2;
f_18909(t3,((C_word*)t0)[6],t1);}
else{
/* scrutinizer.scm:1983: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[6];
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
C_values(5,av2);}}}

/* g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18909(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,3)))){
C_save_and_reclaim_args((void *)trf_18909,3,t0,t1,t2);}
a=C_alloc(23);
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18913,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t1,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[4])[1]))){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18948,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18972,a[2]=t3,a[3]=t4,a[4]=t7,a[5]=t9,a[6]=t8,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:1976: delete-duplicates */
f_7320(t10,((C_word*)((C_word*)t0)[4])[1],*((C_word*)lf[374]+1));}
else{
t5=t4;
f_18913(t5,C_SCHEME_UNDEFINED);}}

/* k18911 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18913(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18913,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18916,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1978: simplify-type */
f_13503(t2,((C_word*)((C_word*)t0)[3])[1]);}

/* k18914 in k18911 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18916(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18916,c,av);}
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_i_car(((C_word*)((C_word*)t0)[2])[1]);
t3=C_eqp(t2,((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(t3)){
/* scrutinizer.scm:1979: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=C_i_cdr(((C_word*)((C_word*)t0)[2])[1]);
av2[4]=((C_word*)((C_word*)t0)[5])[1];
C_values(5,av2);}}
else{
/* scrutinizer.scm:1979: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)((C_word*)t0)[5])[1];
C_values(5,av2);}}}
else{
/* scrutinizer.scm:1979: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)((C_word*)t0)[5])[1];
C_values(5,av2);}}}

/* g4030 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18948(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_18948,3,t0,t1,t2);}
a=C_alloc(12);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18952,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18965,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18969,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1973: chicken.syntax#strip-syntax */
t6=*((C_word*)lf[478]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k18950 in g4030 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18952,c,av);}
t2=C_i_assq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k18963 in g4030 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18965(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18965,c,av);}
/* scrutinizer.scm:1973: chicken.plist#put! */
t2=*((C_word*)lf[476]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[376];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k18967 in g4030 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18969,c,av);}
/* scrutinizer.scm:1973: scheme#symbol->string */
t2=*((C_word*)lf[477]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k18970 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_18972,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(t1,lf[6]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18978,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18980,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_18980(t7,t3,t1);}

/* k18976 in k18970 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_18978(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_18978,c,av);}
a=C_alloc(9);
t2=C_a_i_list(&a,3,lf[29],t1,((C_word*)((C_word*)t0)[2])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=((C_word*)t0)[3];
f_18913(t4,t3);}

/* map-loop4024 in k18970 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_18980(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18980,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19005,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:1972: g4030 */
t4=((C_word*)t0)[4];
f_18948(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19003 in map-loop4024 in k18970 in g4016 in k18903 in chicken.compiler.scrutinizer#validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19005(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19005,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18980(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.scrutinizer#check-and-validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19020(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_19020,c,av);}
a=C_alloc(8);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19029,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19039,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1986: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t6;
av2[3]=t7;
C_call_with_values(4,av2);}}

/* a19028 in chicken.compiler.scrutinizer#check-and-validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_19029,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19037,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:1986: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[478]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k19035 in a19028 in chicken.compiler.scrutinizer#check-and-validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19037(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19037,c,av);}
/* scrutinizer.scm:1986: validate-type */
t2=*((C_word*)lf[407]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a19038 in chicken.compiler.scrutinizer#check-and-validate-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19039(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19039,c,av);}
if(C_truep(t2)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* scrutinizer.scm:1988: chicken.base#error */
t5=*((C_word*)lf[480]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[481];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19054(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_19054,c,av);}
a=C_alloc(24);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19057,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19069,a[2]=t2,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t10=C_i_check_list_2(t3,lf[6]);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19297,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19299,a[2]=t7,a[3]=t13,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t15=((C_word*)t13)[1];
f_19299(t15,t11,t3);}

/* fail in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19057(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_19057,3,t0,t1,t2);}
/* scrutinizer.scm:1992: chicken.base#error */
t3=*((C_word*)lf[480]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[482];
av2[3]=t2;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19069(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,3)))){
C_save_and_reclaim_args((void *)trf_19069,3,t0,t1,t2);}
a=C_alloc(23);
t3=C_i_listp(t2);
t4=(C_truep(t3)?C_i_listp(C_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19080,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t10=C_i_car(t2);
t11=C_i_check_list_2(t10,lf[6]);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19108,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19246,a[2]=t7,a[3]=t14,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_19246(t16,t12,t10);}
else{
/* scrutinizer.scm:2032: fail */
t5=((C_word*)t0)[3];
f_19057(t5,t1,t2);}}

/* g4127 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19080(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_19080,3,t0,t1,t2);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19086,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19092,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2009: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}

/* a19085 in g4127 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19086,c,av);}
/* scrutinizer.scm:2009: validate-type */
t2=*((C_word*)lf[407]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a19091 in g4127 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19092,c,av);}
if(C_truep(t2)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* scrutinizer.scm:2011: chicken.base#error */
t5=*((C_word*)lf[480]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[483];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}}

/* k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_19108,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19111,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19244,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2014: append-map */
f_6955(t3,lf[161],t1,C_SCHEME_END_OF_LIST);}

/* k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_19111,c,av);}
a=C_alloc(26);
t2=C_i_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19121,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
switch(t2){
case C_fix(2):
t4=C_i_cdr(((C_word*)t0)[2]);
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}
case C_fix(3):
t4=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_listp(t4))){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19147,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t10=C_i_cadr(((C_word*)t0)[2]);
t11=C_i_check_list_2(t10,lf[6]);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19175,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19181,a[2]=t7,a[3]=t14,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_19181(t16,t12,t10);}
else{
t5=C_i_cadr(((C_word*)t0)[2]);
t6=C_eqp(lf[53],t5);
if(C_truep(t6)){
t7=C_i_cdr(((C_word*)t0)[2]);
t8=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
/* scrutinizer.scm:2030: fail */
t7=((C_word*)t0)[6];
f_19057(t7,t3,((C_word*)t0)[2]);}}
default:
/* scrutinizer.scm:2031: fail */
t4=((C_word*)t0)[6];
f_19057(t4,t3,((C_word*)t0)[2]);}}

/* k19119 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19121,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g4180 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19147(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_19147,3,t0,t1,t2);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19153,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19159,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2023: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}

/* a19152 in g4180 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19153,c,av);}
/* scrutinizer.scm:2023: validate-type */
t2=*((C_word*)lf[407]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a19158 in g4180 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19159,c,av);}
if(C_truep(t2)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* scrutinizer.scm:2025: chicken.base#error */
t5=*((C_word*)lf[480]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[484];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}}

/* k19173 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_19175,c,av);}
a=C_alloc(6);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop4174 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19181(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19181,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19206,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2022: g4180 */
t4=((C_word*)t0)[4];
f_19147(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19204 in map-loop4174 in k19109 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19206,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_19181(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k19242 in k19106 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19244(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19244,c,av);}
/* scrutinizer.scm:2014: unzip1 */
t2=lf[12];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_7474(3,av2);}}

/* map-loop4121 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19246(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19246,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19271,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2008: g4127 */
t4=((C_word*)t0)[4];
f_19080(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19269 in map-loop4121 in g4105 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19271,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_19246(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k19295 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19297,c,av);}
/* scrutinizer.scm:1993: chicken.compiler.support#mark-variable */
t2=*((C_word*)lf[233]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[162];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop4099 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19299(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19299,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19324,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2005: g4105 */
t4=((C_word*)t0)[4];
f_19069(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19322 in map-loop4099 in chicken.compiler.scrutinizer#install-specializations in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19324,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_19299(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.scrutinizer#canonicalize-list-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19333(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_19333,2,t1,t2);}
a=C_alloc(9);
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(lf[48],t4);
if(C_truep(t5)){
t6=C_i_cadr(t2);
t7=C_i_caddr(t2);
t8=C_a_i_list1(&a,1,t6);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19361,a[2]=t10,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t12=((C_word*)t10)[1];
f_19361(t12,t1,t7,t8);}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* rec in chicken.compiler.scrutinizer#canonicalize-list-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19361(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_19361,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_eqp(lf[31],t2);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19375,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2048: scheme#reverse */
t6=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_i_pairp(t2);
t6=(C_truep(t5)?C_eqp(lf[48],C_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=C_i_caddr(t2);
t8=C_i_cadr(t2);
t9=C_a_i_cons(&a,2,t8,t3);
/* scrutinizer.scm:2050: rec */
t12=t1;
t13=t7;
t14=t9;
t1=t12;
t2=t13;
t3=t14;
goto loop;}
else{
t7=C_i_pairp(t2);
t8=(C_truep(t7)?C_eqp(lf[47],C_i_car(t2)):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19409,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19413,a[2]=t9,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2052: scheme#reverse */
t11=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}}}

/* k19373 in rec in chicken.compiler.scrutinizer#canonicalize-list-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19375,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[47],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19407 in rec in chicken.compiler.scrutinizer#canonicalize-list-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19409,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[47],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19411 in rec in chicken.compiler.scrutinizer#canonicalize-list-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19413,c,av);}
/* scrutinizer.scm:2052: ##sys#append */
t2=*((C_word*)lf[404]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.scrutinizer#strip-namespace in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19447(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_19447,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19451,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2060: scheme#symbol->string */
t4=*((C_word*)lf[477]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19449 in chicken.compiler.scrutinizer#strip-namespace in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_19451,c,av);}
a=C_alloc(8);
t2=C_block_size(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19459,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_19459(t6,((C_word*)t0)[3],C_fix(0));}

/* loop in k19449 in chicken.compiler.scrutinizer#strip-namespace in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19459(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_19459,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(C_subchar(((C_word*)t0)[4],t2),C_make_character(35));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19479,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2065: ##sys#substring */
t6=*((C_word*)lf[485]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fixnum_plus(t2,C_fix(1));
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
/* scrutinizer.scm:2066: loop */
t7=t1;
t8=C_fixnum_plus(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}}}

/* k19477 in loop in k19449 in chicken.compiler.scrutinizer#strip-namespace in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19479,c,av);}
/* scrutinizer.scm:2065: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
C_string_to_symbol(3,av2);}}

/* k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,6)))){
C_save_and_reclaim((void *)f_19495,c,av);}
a=C_alloc(14);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19497,tmp=(C_word)a,a+=2,tmp));
t7=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19596,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19651,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2119: ##sys#put! */
t9=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[655];
av2[3]=lf[274];
av2[4]=((C_word*)t5)[1];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* known-length-vector-index in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19497(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_19497,5,t1,t2,t3,t4,t5);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19501,a[2]=t5,a[3]=t3,a[4]=t1,a[5]=t4,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2093: chicken.compiler.support#node-subexpressions */
t7=*((C_word*)lf[86]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k19499 in known-length-vector-index in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,2)))){
C_save_and_reclaim((void *)f_19501,c,av);}
a=C_alloc(35);
if(C_truep(t1)){
t2=C_i_length(t1);
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
if(C_truep(C_i_nequalp(t2,t3))){
t4=C_i_cadr(((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26609,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19562 in k19574 in k19578 */
static void C_ccall f_19564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19564,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19574 in k19578 */
static void C_ccall f_19576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_19576,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_fixnump(t2))){
t3=C_i_length(C_u_i_cdr(((C_word*)t0)[2]));
t4=C_i_greater_or_equalp(t2,C_fix(0));
t5=(C_truep(t4)?C_i_lessp(t2,t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19564,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2106: r-index-out-of-range */
f_22045(t6,((C_word*)t0)[4],((C_word*)t0)[5],t2,t3,lf[487]);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19578 */
static void C_ccall f_19580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19580,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[123],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19576,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2100: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* vector-ref-result-type in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_19596,c,av);}
a=C_alloc(6);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19600,a[2]=t3,a[3]=t1,a[4]=t5,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2112: known-length-vector-index */
f_19497(t6,t2,t3,t4,C_fix(2));}

/* k19598 in vector-ref-result-type in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19600,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26614,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19619 in k19642 */
static void C_ccall f_19621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19621,c,av);}
a=C_alloc(3);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_i_list_ref(t2,((C_word*)t0)[3]);
t4=C_a_i_list1(&a,1,t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?t4:((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k19642 */
static void C_ccall f_19644(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_19644,c,av);}
a=C_alloc(21);
t2=C_i_cadr(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_list(&a,2,lf[123],((C_word*)t0)[3]);
t5=C_a_i_list(&a,3,lf[488],t2,t4);
/* scrutinizer.scm:2115: mutate-node! */
f_17320(t3,((C_word*)t0)[6],t5);}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_19651,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19654,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2120: ##sys#put! */
t3=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[654];
av2[3]=lf[274];
av2[4]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_19654,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19657,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23276,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2122: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[653];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_19657,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19842,tmp=(C_word)a,a+=2,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19937,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23258,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23260,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2202: list+index-call-result-type-special-case */
f_19842(t4,t5);}

/* a19709 in k19919 in k19923 */
static void C_ccall f_19710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_19710,c,av);}
a=C_alloc(5);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6922,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_6922(t6,t1,((C_word*)t0)[3],C_SCHEME_END_OF_LIST,t2);}

/* a19719 in k19919 in k19923 */
static void C_ccall f_19720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_19720,c,av);}
a=C_alloc(6);
t4=C_i_nullp(t2);
t5=(C_truep(t4)?lf[31]:C_a_i_cons(&a,2,lf[47],t2));
t6=C_i_nullp(t3);
t7=(C_truep(t6)?lf[31]:C_a_i_cons(&a,2,lf[47],t3));
/* scrutinizer.scm:2158: k */
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=t1;
av2[2]=t5;
av2[3]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}

/* lp in k19919 in k19923 */
static void C_fcall f_19747(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(32,0,4)))){
C_save_and_reclaim_args((void *)trf_19747,5,t0,t1,t2,t3,t4);}
a=C_alloc(32);
t5=C_eqp(t4,C_fix(0));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19765,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2163: scheme#reverse */
t7=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=C_i_pairp(t3);
t7=(C_truep(t6)?C_eqp(C_i_car(t3),lf[48]):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=C_i_cadr(t3);
t9=C_a_i_cons(&a,2,t8,t2);
t10=C_i_caddr(t3);
t11=C_s_a_i_minus(&a,2,t4,C_fix(1));
/* scrutinizer.scm:2166: lp */
t13=t1;
t14=t9;
t15=t10;
t16=t11;
t1=t13;
t2=t14;
t3=t15;
t4=t16;
goto loop;}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}

/* k19763 in lp in k19919 in k19923 */
static void C_ccall f_19765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_19765,c,av);}
a=C_alloc(3);
t2=C_i_nullp(t1);
t3=(C_truep(t2)?lf[31]:C_a_i_cons(&a,2,lf[47],t1));
/* scrutinizer.scm:2163: k */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=t3;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* list+index-call-result-type-special-case in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_19842(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,6)))){
C_save_and_reclaim_args((void *)trf_19842,2,t1,t2);}
a=C_alloc(3);
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19844,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_19844 in list+index-call-result-type-special-case in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_19844,c,av);}
a=C_alloc(8);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19848,a[2]=t1,a[3]=t5,a[4]=t3,a[5]=t4,a[6]=t2,a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2181: chicken.compiler.support#node-subexpressions */
t7=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k19846 */
static void C_ccall f_19848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_19848,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19851,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=C_i_length(t1);
t4=C_eqp(t3,C_fix(3));
if(C_truep(t4)){
t5=C_i_cadr(((C_word*)t0)[4]);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f26623,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t2,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t7=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19849 in k19846 */
static void C_ccall f_19851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19851,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19895 in k19919 in k19923 */
static void C_ccall f_19897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19897,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19898 in k19919 in k19923 */
static void C_ccall f_19900(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_19900,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19906,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
t3=C_eqp(((C_word*)t0)[8],lf[31]);
if(C_truep(t3)){
t4=t2;
f_19906(t4,C_fix(0));}
else{
t4=C_i_pairp(((C_word*)t0)[8]);
t5=(C_truep(t4)?C_eqp(C_u_i_car(((C_word*)t0)[8]),lf[47]):C_SCHEME_FALSE);
t6=t2;
f_19906(t6,(C_truep(t5)?C_i_length(C_i_cdr(((C_word*)t0)[8])):C_SCHEME_FALSE));}}}

/* k19904 in k19898 in k19919 in k19923 */
static void C_fcall f_19906(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_19906,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19910,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2188: g4451 */
t3=t2;
f_19910(t3,((C_word*)t0)[5],t1);}
else{
t2=((C_word*)t0)[6];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g4451 in k19904 in k19898 in k19919 in k19923 */
static void C_fcall f_19910(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,6)))){
C_save_and_reclaim_args((void *)trf_19910,3,t0,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19914,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2196: r-index-out-of-range */
f_22045(t3,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t2,lf[491]);}

/* k19912 in g4451 in k19904 in k19898 in k19919 in k19923 */
static void C_ccall f_19914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19914,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19919 in k19923 */
static void C_ccall f_19921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,6)))){
C_save_and_reclaim((void *)f_19921,c,av);}
a=C_alloc(16);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_fixnump(t2))){
if(C_truep(C_i_negativep(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19897,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2189: r-index-out-of-range */
f_22045(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2,lf[489],lf[490]);}
else{
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19900,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t4=C_i_pairp(((C_word*)t0)[7]);
if(C_truep(C_i_not(t4))){
t5=C_eqp(t2,C_fix(0));
if(C_truep(t5)){
t6=C_eqp(((C_word*)t0)[7],lf[31]);
if(C_truep(t6)){
/* scrutinizer.scm:2154: k */
t7=((C_word*)t0)[8];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t7=t3;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
f_19900(2,av2);}}}
else{
t6=t3;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_19900(2,av2);}}}
else{
t5=C_i_car(((C_word*)t0)[7]);
t6=C_eqp(t5,lf[47]);
if(C_truep(t6)){
t7=C_i_length(((C_word*)t0)[7]);
if(C_truep(C_fixnum_lessp(t2,t7))){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19710,a[2]=((C_word*)t0)[7],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19720,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2157: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t3;
av2[2]=t8;
av2[3]=t9;
C_call_with_values(4,av2);}}
else{
t8=t3;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_19900(2,av2);}}}
else{
t7=C_i_car(((C_word*)t0)[7]);
t8=C_eqp(t7,lf[48]);
if(C_truep(t8)){
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19747,a[2]=((C_word*)t0)[8],a[3]=t10,tmp=(C_word)a,a+=4,tmp));
t12=((C_word*)t10)[1];
f_19747(t12,t3,C_SCHEME_END_OF_LIST,((C_word*)t0)[7],t2);}
else{
t9=t3;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
f_19900(2,av2);}}}}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19923 */
static void C_ccall f_19925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_19925,c,av);}
a=C_alloc(9);
t2=C_eqp(lf[123],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19921,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2186: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_19937,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19940,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23248,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23250,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2208: list+index-call-result-type-special-case */
f_19842(t3,t4);}

/* k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_19940,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19943,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23181,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2211: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[647];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_19943,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19946,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23116,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2217: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[645];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19946(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_19946,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19949,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23059,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2223: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[643];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_19949,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19952,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23002,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2227: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[642];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_19952,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19955,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22940,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2231: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[367];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19955(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_19955,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19957,tmp=(C_word)a,a+=2,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20120,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2281: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[5];
av2[3]=lf[274];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* append-special-case in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_19957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_19957,c,av);}
a=C_alloc(9);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19966,a[2]=t3,a[3]=t4,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20111,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2278: derive-result-type */
t8=t6;
f_19966(t8,t7);}

/* derive-result-type in append-special-case in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_19966(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_19966,2,t0,t1);}
a=C_alloc(7);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19976,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_19976(t6,t1,t2,C_fix(1));}

/* lp in derive-result-type in append-special-case in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_19976(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_19976,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[31];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f26630,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=t4,tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k19996 */
static void C_ccall f_19998(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_19998,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20008,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2259: scheme#reverse */
t3=*((C_word*)lf[367]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20006 in k19996 */
static void C_ccall f_20008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_20008,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20011,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20013,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_20013(t6,t2,t1,((C_word*)t0)[3]);}

/* k20009 in k20006 in k19996 */
static void C_ccall f_20011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20011,c,av);}
/* scrutinizer.scm:2257: canonicalize-list-type */
f_19333(((C_word*)t0)[2],t1);}

/* foldl4656 in k20006 in k19996 */
static void C_fcall f_20013(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_20013,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=C_slot(t2,C_fix(0));
t6=C_a_i_list(&a,3,lf[48],t5,t3);
t8=t1;
t9=t4;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k20055 */
static void C_ccall f_20057(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20057,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_a_i_list(&a,3,lf[28],((C_word*)t0)[2],t1);
/* scrutinizer.scm:2264: simplify-type */
f_13503(((C_word*)t0)[3],t2);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20072 */
static void C_ccall f_20074(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20074,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20078 */
static void C_ccall f_20080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_20080,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20087,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2276: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[5];
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k20085 in k20078 */
static void C_ccall f_20087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_20087,c,av);}
/* scrutinizer.scm:2274: r-proc-call-argument-type-mismatch */
f_21906(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[47],((C_word*)t0)[7],t1);}

/* k20109 in append-special-case in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_20111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_20111,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_list1(&a,1,t1):((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_20120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20120,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20123,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2282: ##sys#put! */
t3=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[404];
av2[3]=lf[274];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_20123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_20123,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20218,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_20127,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2305: ##sys#put! */
t4=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[641];
av2[3]=lf[274];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* f_20127 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_20127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_20127,c,av);}
a=C_alloc(5);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20131,a[2]=t1,a[3]=t5,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2292: chicken.compiler.support#node-subexpressions */
t7=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k20129 */
static void C_ccall f_20131(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_20131,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_length(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20143,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
switch(t2){
case C_fix(2):
t4=t3;
f_20143(t4,lf[53]);
case C_fix(3):
t4=C_i_caddr(((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f26669,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}
default:
t4=t3;
f_20143(t4,C_SCHEME_FALSE);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20141 in k20129 */
static void C_fcall f_20143(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20143,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20195,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2298: chicken.compiler.support#node-class */
t4=*((C_word*)lf[88]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20171 in k20189 in k20193 in k20141 in k20129 */
static void C_ccall f_20173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_20173,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20187,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2302: make-list */
f_7950(t2,((C_word*)t0)[4],C_a_i_list(&a,1,((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20185 in k20171 in k20189 in k20193 in k20141 in k20129 */
static void C_ccall f_20187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_20187,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[50],t1);
t3=C_a_i_list(&a,1,t2);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t3:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k20189 in k20193 in k20141 in k20129 */
static void C_ccall f_20191(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20191,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_fixnump(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20173,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2301: scheme#<= */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t3;
av2[2]=C_fix(0);
av2[3]=t2;
av2[4]=C_fix(256);
C_less_or_equal_p(5,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20193 in k20141 in k20129 */
static void C_ccall f_20195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_20195,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[123],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20191,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2299: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_20218(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,9)))){
C_save_and_reclaim((void *)f_20218,c,av);}
a=C_alloc(36);
t2=C_mutate(&lf[369] /* (set! chicken.compiler.scrutinizer#over-all-instantiations ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_20220,tmp=(C_word)a,a+=2,tmp));
t3=C_mutate(&lf[315] /* (set! chicken.compiler.scrutinizer#string-add-indent ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_20545,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate(&lf[74] /* (set! chicken.compiler.scrutinizer#type->pp-string ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_20636,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate(&lf[142] /* (set! chicken.compiler.scrutinizer#fragment ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21102,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate(&lf[75] /* (set! chicken.compiler.scrutinizer#pp-fragment ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21207,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[323] /* (set! chicken.compiler.scrutinizer#node-source-prefix ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21231,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate(&lf[317] /* (set! chicken.compiler.scrutinizer#location-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21261,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate(&lf[515] /* (set! chicken.compiler.scrutinizer#variable-and-module ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21431,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate(&lf[91] /* (set! chicken.compiler.scrutinizer#variable-from-module ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21482,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate(&lf[102] /* (set! chicken.compiler.scrutinizer#describe-expression ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21542,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate(&lf[71] /* (set! chicken.compiler.scrutinizer#report2 ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21672,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate(&lf[169] /* (set! chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21906,tmp=(C_word)a,a+=2,tmp));
t14=C_mutate(&lf[270] /* (set! chicken.compiler.scrutinizer#r-proc-call-argument-value-count ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21952,tmp=(C_word)a,a+=2,tmp));
t15=C_mutate(&lf[486] /* (set! chicken.compiler.scrutinizer#r-index-out-of-range ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22045,tmp=(C_word)a,a+=2,tmp));
t16=C_mutate(&lf[219] /* (set! chicken.compiler.scrutinizer#r-conditional-value-count-invalid ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22177,tmp=(C_word)a,a+=2,tmp));
t17=C_mutate(&lf[220] /* (set! chicken.compiler.scrutinizer#r-let-value-count-invalid ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22239,tmp=(C_word)a,a+=2,tmp));
t18=C_mutate(&lf[267] /* (set! chicken.compiler.scrutinizer#r-assignment-value-count-invalid ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22359,tmp=(C_word)a,a+=2,tmp));
t19=C_mutate(&lf[61] /* (set! chicken.compiler.scrutinizer#r-deprecated-identifier ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22850,tmp=(C_word)a,a+=2,tmp));
t20=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t20;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t20+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_20220(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,0,3)))){
C_save_and_reclaim_args((void *)trf_20220,5,t1,t2,t3,t4,t5);}
a=C_alloc(24);
t6=C_SCHEME_END_OF_LIST;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=lf[24];
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20223,a[2]=t8,a[3]=t7,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t14=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20287,a[2]=t4,a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t15=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20419,a[2]=t2,a[3]=t3,a[4]=t12,a[5]=t10,a[6]=t4,a[7]=t5,a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2354: delete-duplicates */
f_7320(t15,t2,*((C_word*)lf[374]+1));}

/* restore in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_20223(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_20223,2,t0,t1);}
a=C_alloc(11);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20232,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t5,tmp=(C_word)a,a+=7,tmp));
t7=((C_word*)t5)[1];
f_20232(t7,t1,lf[24]);}

/* doloop4736 in restore in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20232(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_20232,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=C_mutate(&lf[24] /* (set! chicken.compiler.scrutinizer#trail ...) */,t2);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[3])[1]))){
t5=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[3])[1],((C_word*)((C_word*)t0)[4])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[4])+1,t5);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t5=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[3])[1],((C_word*)((C_word*)t0)[4])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[4])+1,t5);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20283,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2329: resolve */
t6=lf[118];{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_u_i_car(t2);
av2[3]=((C_word*)t0)[5];
f_16285(4,av2);}}}

/* k20281 in doloop4736 in restore in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_20283,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=C_i_assq(C_u_i_car(((C_word*)t0)[4]),((C_word*)t0)[5]);
t7=C_i_cdr(t6);
t8=C_i_set_car(t7,C_SCHEME_FALSE);
t9=((C_word*)((C_word*)t0)[6])[1];
f_20232(t9,((C_word*)t0)[7],C_u_i_cdr(((C_word*)t0)[4]));}

/* collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_20287(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,3)))){
C_save_and_reclaim_args((void *)trf_20287,2,t0,t1);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20291,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20365,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=lf[12];
t9=((C_word*)((C_word*)t0)[3])[1];
t10=C_i_check_list_2(t9,lf[6]);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20375,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20377,a[2]=t6,a[3]=t13,a[4]=t7,tmp=(C_word)a,a+=5,tmp));
t15=((C_word*)t13)[1];
f_20377(t15,t11,t9);}

/* k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20291(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_20291,c,av);}
a=C_alloc(17);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20295,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(t1,lf[6]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20329,a[2]=t4,a[3]=t9,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_20329(t11,((C_word*)t0)[4],t1);}

/* g4784 in k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20295(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_20295,3,t0,t1,t2);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20303,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20305,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2341: filter-map */
f_7409(t3,t4,((C_word*)((C_word*)t0)[3])[1]);}

/* k20301 in g4784 in k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_20303,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a20304 in g4784 in k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20305,c,av);}
t3=C_i_assq(((C_word*)t0)[2],t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_i_cdr(t3):(C_truep(((C_word*)t0)[3])?lf[53]:C_SCHEME_FALSE));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop4778 in k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20329(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20329,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20354,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2338: g4784 */
t4=((C_word*)t0)[4];
f_20295(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20352 in map-loop4778 in k20289 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20354,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20329(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20363 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20365(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20365,c,av);}
/* scrutinizer.scm:2337: delete-duplicates */
f_7320(((C_word*)t0)[2],t1,*((C_word*)lf[374]+1));}

/* k20373 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20375,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7223,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_7223(t5,((C_word*)t0)[2],t1);}

/* map-loop4751 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20377(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20377,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20402,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2337: g4757 */
t4=lf[12];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_7474(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20400 in map-loop4751 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20402,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20377(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_20419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_20419,c,av);}
a=C_alloc(11);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20421,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp));
t5=((C_word*)t3)[1];
f_20421(t5,((C_word*)t0)[8],t1,C_SCHEME_FALSE);}

/* loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20421(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_20421,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t4=(C_truep(t3)?t3:C_i_nullp(((C_word*)t0)[2]));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20435,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20470,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2363: collect */
t7=((C_word*)((C_word*)t0)[4])[1];
f_20287(t7,t6);}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20508,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2366: process */
t5=((C_word*)t0)[8];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* g4829 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20435(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_20435,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_car(t2);
t4=C_a_i_cons(&a,2,t3,lf[24]);
t5=C_mutate(&lf[24] /* (set! chicken.compiler.scrutinizer#trail ...) */,t4);
t6=C_i_assq(C_u_i_car(t2),((C_word*)t0)[2]);
t7=C_i_cdr(t6);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20451,a[2]=t1,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
t9=C_u_i_cdr(t2);
t10=C_a_i_cons(&a,2,lf[28],t9);
/* scrutinizer.scm:2362: simplify-type */
f_13503(t8,t10);}

/* k20449 in g4829 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20451,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_set_car(((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20468 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20470(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_20470,c,av);}
a=C_alloc(9);
t2=C_i_check_list_2(t1,lf[212]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20476,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20478,a[2]=t5,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_20478(t7,t3,t1);}

/* k20474 in k20468 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20476,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* for-each-loop4828 in k20468 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_fcall f_20478(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20478,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20488,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2358: g4829 */
t4=((C_word*)t0)[3];
f_20435(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k20486 in for-each-loop4828 in k20468 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_20488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20488,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_20478(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k20506 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_20508,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20511,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2367: restore */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20223(t3,t2);}
else{
if(C_truep(((C_word*)t0)[6])){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20522,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2370: restore */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20223(t3,t2);}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20525,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2373: restore */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20223(t3,t2);}}}

/* k20509 in k20506 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20511,c,av);}
/* scrutinizer.scm:2368: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20421(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_SCHEME_TRUE);}

/* k20520 in k20506 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20522,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20523 in k20506 in loop in k20417 in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20525,c,av);}
/* scrutinizer.scm:2374: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20421(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),((C_word*)t0)[5]);}

/* chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_20545(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_20545,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_i_nullp(t3);
t5=(C_truep(t4)?lf[492]:C_i_car(t3));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20552,a[2]=t2,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2382: chicken.string#string-split */
t7=*((C_word*)lf[497]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=lf[498];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_20552(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_20552,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20555,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20581,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20594,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20596,a[2]=t5,a[3]=t10,a[4]=t7,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_20596(t12,t8,t1);}

/* k20553 in k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20555,c,av);}
a=C_alloc(5);
t2=C_block_size(((C_word*)t0)[2]);
t3=C_a_i_fixnum_difference(&a,2,t2,C_fix(1));
t4=C_i_string_ref(((C_word*)t0)[2],t3);
t5=C_eqp(C_make_character(10),t4);
if(C_truep(t5)){
/* ##sys#string-append */
t6=*((C_word*)lf[493]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=lf[494];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* g4880 in k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20581(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_20581,3,t0,t1,t2);}
if(C_truep(C_i_string_equal_p(lf[495],t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* scrutinizer.scm:2387: scheme#string-append */
t3=*((C_word*)lf[76]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k20592 in k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20594,c,av);}
/* scrutinizer.scm:2383: chicken.string#string-intersperse */
t2=*((C_word*)lf[313]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[496];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop4874 in k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20596(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20596,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2384: g4880 */
t4=((C_word*)t0)[4];
f_20581(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20619 in map-loop4874 in k20550 in chicken.compiler.scrutinizer#string-add-indent in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20621,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20596(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_20636(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_20636,2,t1,t2);}
a=C_alloc(12);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20662,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21079,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21100,a[2]=t4,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2430: chicken.syntax#strip-syntax */
t8=*((C_word*)lf[478]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k20641 in map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_20643,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20646,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f26243,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2398: scheme#string->symbol */
t4=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_a_i_list2(&a,2,lf[500],((C_word*)t0)[3]);
/* scrutinizer.scm:2397: ##sys#error */
t4=*((C_word*)lf[501]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k20644 in k20641 in map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_20646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20646,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20653,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2398: scheme#string->symbol */
t3=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20651 in k20644 in k20641 in map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_20653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_20653,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];
f_20788(t2,C_a_i_list2(&a,2,lf[123],t1));}

/* conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_20662(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_20662,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=C_i_nullp(t3);
t5=(C_truep(t4)?C_SCHEME_END_OF_LIST:C_i_car(t3));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20668,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_i_pairp(t2);
if(C_truep(C_i_not(t7))){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20681,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2403: chicken.base#alist-ref */
t9=*((C_word*)lf[275]+1);{
C_word av2[5];
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
av2[3]=t5;
av2[4]=*((C_word*)lf[374]+1);
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t8=(
/* scrutinizer.scm:2404: refinement-type? */
  f_15863(t2)
);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20697,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2406: chicken.base#open-output-string */
t10=*((C_word*)lf[242]+1);{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=C_i_pairp(t2);
t10=(C_truep(t9)?C_u_i_car(t2):C_SCHEME_FALSE);
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20776,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t10,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
t12=C_eqp(lf[29],t10);
if(C_truep(t12)){
/* scrutinizer.scm:2410: every */
f_7125(t11,*((C_word*)lf[470]+1),C_i_cadr(t2));}
else{
t13=t11;{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
f_20776(2,av2);}}}}}

/* R in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_20668(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_20668,3,t0,t1,t2);}
a=C_alloc(3);
/* scrutinizer.scm:2400: conv */
t3=((C_word*)((C_word*)t0)[2])[1];
f_20662(t3,t1,t2,C_a_i_list(&a,1,((C_word*)t0)[3]));}

/* k20679 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20681,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20697(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,3)))){
C_save_and_reclaim((void *)f_20697,c,av);}
a=C_alloc(25);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20703,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20720,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=*((C_word*)lf[320]+1);
t10=C_i_cadr(((C_word*)t0)[3]);
t11=C_i_check_list_2(t10,lf[6]);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20733,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20735,a[2]=t7,a[3]=t14,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_20735(t16,t12,t10);}

/* k20701 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20703,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20706,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2406: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(45);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k20704 in k20701 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_20706,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20709,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2406: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_caddr(((C_word*)t0)[4]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20707 in k20704 in k20701 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_20709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20709,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20712,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2406: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20710 in k20707 in k20704 in k20701 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_20712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20712,c,av);}
/* scrutinizer.scm:2405: scheme#string->symbol */
t2=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k20718 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20720,c,av);}
/* scrutinizer.scm:2406: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k20731 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20733,c,av);}
/* scrutinizer.scm:2406: chicken.string#string-intersperse */
t2=*((C_word*)lf[313]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[499];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop4938 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20735(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20735,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20760,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2406: g4944 */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20758 in map-loop4938 in k20695 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20760(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20760,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20735(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_20776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_20776,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cadr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[6]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20797,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20806,a[2]=t4,a[3]=t10,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_20806(t12,t8,t6);}
else{
t2=C_eqp(lf[29],((C_word*)t0)[5]);
if(C_truep(t2)){
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep((C_truep(C_eqp(((C_word*)t0)[5],lf[28]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[69]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[47]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[50]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[48]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[361]))?C_SCHEME_TRUE:(C_truep(C_eqp(((C_word*)t0)[5],lf[359]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))))))){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_cdr(((C_word*)t0)[2]);
t8=C_i_check_list_2(t7,lf[6]);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20865,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20867,a[2]=t5,a[3]=t11,a[4]=((C_word*)t0)[6],a[5]=t6,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_20867(t13,t9,t7);}
else{
t3=C_eqp(lf[52],((C_word*)t0)[5]);
if(C_truep(t3)){
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(lf[84],((C_word*)t0)[5]);
if(C_truep(t4)){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20917,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=t7,a[6]=t8,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2418: procedure-arguments */
f_15222(t9,((C_word*)t0)[2]);}
else{
/* scrutinizer.scm:2429: chicken.compiler.support#bomb */
t5=*((C_word*)lf[343]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[502];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}}}}}

/* k20786 in map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_fcall f_20788(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20788,2,t0,t1);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t3);
t6=((C_word*)((C_word*)t0)[4])[1];
f_20806(t6,((C_word*)t0)[5],C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k20795 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20797,c,av);}
a=C_alloc(3);
/* scrutinizer.scm:2412: conv */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20662(t2,((C_word*)t0)[3],C_i_caddr(((C_word*)t0)[4]),C_a_i_list(&a,1,t1));}

/* map-loop4976 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20806(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_20806,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20788,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20643,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2396: chicken.plist#get */
t6=*((C_word*)lf[377]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
av2[3]=lf[376];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20863 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_20865,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop5006 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_20867(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20867,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20892,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2415: g5012 */
t4=((C_word*)t0)[4];
f_20668(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20890 in map-loop5006 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20892,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20867(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_20917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_20917,c,av);}
a=C_alloc(13);
t2=C_i_check_list_2(t1,lf[6]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20923,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21017,a[2]=((C_word*)t0)[5],a[3]=t5,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_21017(t7,t3,t1);}

/* k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_20923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_20923,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20926,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2419: procedure-results */
f_15279(t2,((C_word*)t0)[3]);}

/* k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_20926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_20926,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20929,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_eqp(lf[53],t1);
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_20929(2,av2);}}
else{
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_i_check_list_2(t1,lf[6]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20983,a[2]=t6,a[3]=t10,a[4]=((C_word*)t0)[5],a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_20983(t12,t2,t1);}}

/* k20927 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_20929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_20929,c,av);}
a=C_alloc(6);
if(C_truep(C_i_not(t1))){
t2=(C_truep(t1)?C_a_i_cons(&a,2,((C_word*)t0)[2],t1):C_a_i_cons(&a,2,((C_word*)t0)[2],lf[53]));
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[84],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20960,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2425: procedure-name */
f_15171(t2,((C_word*)t0)[4]);}}

/* k20958 in k20927 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_20960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_20960,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20963,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20966,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2426: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
av2[3]=lf[272];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;
f_20963(t3,C_SCHEME_FALSE);}}

/* k20961 in k20958 in k20927 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_fcall f_20963(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_20963,2,t0,t1);}
a=C_alloc(3);
t2=(C_truep(t1)?C_a_i_cons(&a,2,lf[471],((C_word*)t0)[2]):C_a_i_cons(&a,2,lf[473],((C_word*)t0)[2]));
/* scrutinizer.scm:2425: ##sys#append */
t3=*((C_word*)lf[404]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k20964 in k20958 in k20927 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_20966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20966,c,av);}
t2=((C_word*)t0)[2];
f_20963(t2,(C_truep(t1)?t1:C_SCHEME_FALSE));}

/* map-loop5061 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_fcall f_20983(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20983,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21008,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2422: g5067 */
t4=((C_word*)t0)[4];
f_20668(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21006 in map-loop5061 in k20924 in k20921 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21008,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20983(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* map-loop5034 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_fcall f_21017(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21017,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21042,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2418: g5040 */
t4=((C_word*)t0)[4];
f_20668(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21040 in map-loop5034 in k20915 in k20774 in conv in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21042,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_21017(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k21077 in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21079,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21086,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21090,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21092,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2433: chicken.port#with-output-to-string */
t5=*((C_word*)lf[504]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k21084 in k21077 in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21086,c,av);}
/* scrutinizer.scm:2431: string-add-indent */
f_20545(((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* k21088 in k21077 in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21090,c,av);}
/* scrutinizer.scm:2432: chicken.string#string-chomp */
t2=*((C_word*)lf[503]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a21091 in k21077 in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21092,c,av);}
/* scrutinizer.scm:2434: chicken.pretty-print#pp */
t2=*((C_word*)lf[435]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21098 in chicken.compiler.scrutinizer#type->pp-string in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21100,c,av);}
/* scrutinizer.scm:2430: conv */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20662(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21102(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21102,2,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21106,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21205,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2437: source-node-tree */
t5=lf[453];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
f_17365(3,av2);}}

/* k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21106(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21106,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21111,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_21111(t5,((C_word*)t0)[2],t1,C_fix(0));}

/* walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_21111(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_21111,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_not_pair_p(t2))){
/* scrutinizer.scm:2439: chicken.syntax#strip-syntax */
t4=*((C_word*)lf[478]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
if(C_truep(C_i_greater_or_equalp(t3,C_fix(4)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[454];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_listp(t2))){
t4=C_u_i_length(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21137,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnum_lessp(C_fix(6),t4))){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21196,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2444: take */
f_6864(t6,t2,C_fix(6));}
else{
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
f_21137(2,av2);}}}
else{
/* scrutinizer.scm:2447: chicken.syntax#strip-syntax */
t4=*((C_word*)lf[478]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}}}

/* k21135 in walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21137(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(46,c,3)))){
C_save_and_reclaim((void *)f_21137,c,av);}
a=C_alloc(46);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21142,a[2]=((C_word*)t0)[3],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=C_i_check_list_2(t1,lf[6]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21154,a[2]=t4,a[3]=t10,a[4]=t7,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_21154(t12,((C_word*)t0)[4],t1);}

/* g5122 in k21135 in walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_fcall f_21142(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_21142,3,t0,t1,t2);}
/* scrutinizer.scm:2446: g51365137 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_21111(t3,t1,t2,((C_word*)t0)[3]);}

/* map-loop5116 in k21135 in walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_fcall f_21154(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21154,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21179,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2446: g5122 */
t4=((C_word*)t0)[4];
f_21142(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21177 in map-loop5116 in k21135 in walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21179,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_21154(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k21194 in walk in k21104 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21196,c,av);}
/* scrutinizer.scm:2444: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[505];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k21203 in chicken.compiler.scrutinizer#fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21205,c,av);}
/* scrutinizer.scm:2437: chicken.compiler.support#build-expression-tree */
t2=*((C_word*)lf[506]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.scrutinizer#pp-fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21207(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_21207,2,t1,t2);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21215,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21219,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21221,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2452: chicken.port#with-output-to-string */
t6=*((C_word*)lf[504]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k21213 in chicken.compiler.scrutinizer#pp-fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21215(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21215,c,av);}
/* scrutinizer.scm:2450: string-add-indent */
f_20545(((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* k21217 in chicken.compiler.scrutinizer#pp-fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21219,c,av);}
/* scrutinizer.scm:2451: chicken.string#string-chomp */
t2=*((C_word*)lf[503]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a21220 in chicken.compiler.scrutinizer#pp-fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21221,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21229,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2454: fragment */
f_21102(t2,((C_word*)t0)[2]);}

/* k21227 in a21220 in chicken.compiler.scrutinizer#pp-fragment in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21229,c,av);}
/* scrutinizer.scm:2454: chicken.pretty-print#pp */
t2=*((C_word*)lf[435]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21231(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_21231,2,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21235,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17440,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:1750: source-node */
f_17338(t4,t2,C_SCHEME_END_OF_LIST);}

/* k21233 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21235,c,av);}
a=C_alloc(4);
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[507];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21244,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2458: chicken.base#open-output-string */
t3=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21242 in k21233 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21244(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21244,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21250,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2458: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[509];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21248 in k21242 in k21233 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21250,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21253,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2458: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21251 in k21248 in k21242 in k21233 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21253,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21256,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2458: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[508];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21254 in k21251 in k21248 in k21242 in k21233 in chicken.compiler.scrutinizer#node-source-prefix in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21256,c,av);}
/* scrutinizer.scm:2458: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21261(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_21261,3,t1,t2,t3);}
a=C_alloc(9);
t4=C_i_nullp(t3);
t5=(C_truep(t4)?lf[511]:C_i_car(t3));
if(C_truep(C_i_nullp(t2))){
/* scrutinizer.scm:2466: chicken.string#conc */
t6=*((C_word*)lf[320]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[512];
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=C_a_i_list1(&a,1,lf[513]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21311,a[2]=t5,a[3]=t8,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_21311(t10,t1,t2,t6);}}

/* k21272 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21274,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21280,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2463: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[523];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21278 in k21272 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_21280,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21283,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21293,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2463: chicken.compiler.support#real-name */
t4=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k21281 in k21278 in k21272 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21283,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21286,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2463: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[522];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21284 in k21281 in k21278 in k21272 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21286,c,av);}
/* scrutinizer.scm:2463: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21291 in k21278 in k21272 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21293,c,av);}
/* scrutinizer.scm:2463: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_21311(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_21311,4,t0,t1,t2,t3);}
a=C_alloc(14);
t4=C_i_cdr(t2);
if(C_truep(C_i_nullp(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21325,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21333,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_u_i_car(t2))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21339,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21353,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2475: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t6;
av2[2]=t7;
av2[3]=t8;
C_call_with_values(4,av2);}}
else{
t7=t5;
f_21325(t7,C_a_i_cons(&a,2,lf[521],t3));}}
else{
t5=C_u_i_cdr(t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21417,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
t7=C_u_i_car(t2);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21274,a[2]=t6,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2463: chicken.base#open-output-string */
t9=*((C_word*)lf[242]+1);{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_a_i_cons(&a,2,lf[524],t3);
/* scrutinizer.scm:2480: rec */
t10=t1;
t11=t5;
t12=t8;
t1=t10;
t2=t11;
t3=t12;
goto loop;}}}

/* k21323 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_21325(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_21325,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21329,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2479: chicken.string#conc */
t3=*((C_word*)lf[320]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[514];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21327 in k21323 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21329(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21329,c,av);}
/* scrutinizer.scm:2470: chicken.string#string-intersperse */
t2=*((C_word*)lf[313]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k21331 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_21333,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];
f_21325(t2,C_a_i_cons(&a,2,t1,((C_word*)t0)[3]));}

/* a21338 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21339,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21347,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2475: chicken.compiler.support#real-name */
t3=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21345 in a21338 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21347,c,av);}
/* scrutinizer.scm:2475: variable-and-module */
f_21431(((C_word*)t0)[2],t1);}

/* a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21353,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21361,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21383,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2476: chicken.base#open-output-string */
t6=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=lf[520];
f_21361(2,av2);}}}

/* k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21361,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21365,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2477: chicken.base#open-output-string */
t3=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k21363 in k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21365(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_21365,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21371,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2477: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[517];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21369 in k21363 in k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21371,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21374,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2477: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21372 in k21369 in k21363 in k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21374,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21377,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2477: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[516];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21375 in k21372 in k21369 in k21363 in k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21377,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21380,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2477: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21378 in k21375 in k21372 in k21369 in k21363 in k21359 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_21380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21380,c,av);}
/* scrutinizer.scm:2476: chicken.string#conc */
t2=*((C_word*)lf[320]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21383(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_21383,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21389,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2476: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[519];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21387 in k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21389,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21392,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2476: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21390 in k21387 in k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21392,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21395,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2476: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[518];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21393 in k21390 in k21387 in k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_21395,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21398,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2476: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21396 in k21393 in k21390 in k21387 in k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21398,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21401,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2476: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21399 in k21396 in k21393 in k21390 in k21387 in k21381 in a21352 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_21401(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21401,c,av);}
/* scrutinizer.scm:2476: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21415 in rec in chicken.compiler.scrutinizer#location-name in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21417,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* scrutinizer.scm:2480: rec */
t3=((C_word*)((C_word*)t0)[3])[1];
f_21311(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21431(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_21431,2,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21435,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_symbolp(t2))){
/* scrutinizer.scm:2484: scheme#symbol->string */
t4=*((C_word*)lf[477]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
f_21435(2,av2);}}}

/* k21433 in chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21435(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21435,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21438,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2485: chicken.string#string-split */
t3=*((C_word*)lf[497]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=lf[525];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21436 in k21433 in chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21438,c,av);}
a=C_alloc(4);
t2=C_i_cdr(t1);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21451,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2487: scheme#string->symbol */
t4=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(t1);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21470,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2488: scheme#string->symbol */
t4=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k21449 in k21436 in k21433 in chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21451,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21455,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2487: scheme#string->symbol */
t3=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21453 in k21449 in k21436 in k21433 in chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21455,c,av);}
/* scrutinizer.scm:2487: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_values(4,av2);}}

/* k21468 in k21436 in k21433 in chicken.compiler.scrutinizer#variable-and-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21470(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21470,c,av);}
/* scrutinizer.scm:2488: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
C_values(4,av2);}}

/* chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21482(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_21482,2,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21488,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21494,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2491: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}

/* a21487 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21488,c,av);}
/* scrutinizer.scm:2491: variable-and-module */
f_21431(t1,((C_word*)t0)[2]);}

/* a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21494,c,av);}
a=C_alloc(5);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21501,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2493: chicken.base#open-output-string */
t5=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21525,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2494: chicken.base#open-output-string */
t5=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21501,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21507,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2493: ##sys#write-char-0 */
t4=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(96);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k21505 in k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21507,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21510,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2493: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21508 in k21505 in k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21510,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21513,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2493: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[526];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21511 in k21508 in k21505 in k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21513,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21516,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2493: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21514 in k21511 in k21508 in k21505 in k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21516(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_21516,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21519,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2493: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21517 in k21514 in k21511 in k21508 in k21505 in k21499 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21519,c,av);}
/* scrutinizer.scm:2493: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21523 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_21525,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21531,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2494: ##sys#write-char-0 */
t4=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(96);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k21529 in k21523 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21531,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21534,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2494: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21532 in k21529 in k21523 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_21534,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21537,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2494: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21535 in k21532 in k21529 in k21523 in a21493 in chicken.compiler.scrutinizer#variable-from-module in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21537,c,av);}
/* scrutinizer.scm:2494: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21542(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_21542,2,t1,t2);}
a=C_alloc(14);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21545,tmp=(C_word)a,a+=2,tmp));
t8=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21559,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21656,a[2]=t6,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2518: source-node-tree */
t10=lf[453];{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
f_17365(3,av2);}}

/* p-expr in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_21545(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_21545,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21553,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2498: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[527];
av2[3]=lf[528];
av2[4]=lf[529];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21551 in p-expr in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21553,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21557,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2499: pp-fragment */
f_21207(t2,((C_word*)t0)[3]);}

/* k21555 in k21551 in p-expr in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21557,c,av);}
/* scrutinizer.scm:2498: chicken.format#sprintf */
t2=*((C_word*)lf[410]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_21559(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_21559,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21563,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21649,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2501: chicken.compiler.support#node-class */
t5=*((C_word*)lf[88]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k21561 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21563,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21590,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2515: chicken.compiler.support#node-class */
t3=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k21581 in k21588 in k21561 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21583,c,av);}
/* scrutinizer.scm:2516: p-node */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21559(t2,((C_word*)t0)[3],C_i_car(t1));}

/* k21588 in k21561 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21590,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[85],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21583,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2516: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* scrutinizer.scm:2517: p-expr */
f_21545(((C_word*)t0)[3],((C_word*)t0)[4]);}}

/* k21609 in k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21611(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,6)))){
C_save_and_reclaim((void *)f_21611,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2506: scheme#string-append */
t3=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[530];
av2[3]=lf[531];
av2[4]=lf[532];
av2[5]=lf[533];
av2[6]=lf[534];
((C_proc)(void*)(*((C_word*)t3+1)))(7,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_21563(2,av2);}}}

/* k21619 in k21609 in k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21621,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21625,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2512: variable-from-module */
f_21482(t2,((C_word*)t0)[6]);}

/* k21623 in k21619 in k21609 in k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_21625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21625,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21629,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2513: type->pp-string */
f_20636(t2,((C_word*)t0)[6]);}

/* k21627 in k21623 in k21619 in k21609 in k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_21629(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21629,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21633,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2514: p-expr */
f_21545(t2,((C_word*)t0)[6]);}

/* k21631 in k21627 in k21623 in k21619 in k21609 in k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in ... */
static void C_ccall f_21633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21633,c,av);}
/* scrutinizer.scm:2506: chicken.format#sprintf */
t2=*((C_word*)lf[410]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k21635 in k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21637(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21637,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21611,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2505: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21563(2,av2);}}}

/* k21639 in k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21641,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[87],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21637,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2504: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21563(2,av2);}}}

/* k21643 in k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21645,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21641,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2503: chicken.compiler.support#node-class */
t4=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k21647 in p-node in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21649,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[120],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21645,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2502: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21563(2,av2);}}}

/* k21654 in chicken.compiler.scrutinizer#describe-expression in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21656(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21656,c,av);}
/* scrutinizer.scm:2518: p-node */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21559(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_21672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-7)*C_SIZEOF_PAIR +23,c,2)))){
C_save_and_reclaim((void*)f_21672,c,av);}
a=C_alloc((c-7)*C_SIZEOF_PAIR+23);
t7=C_build_rest(&a,c,7,av);
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21675,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(lf[18])){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21703,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21710,a[2]=t3,a[3]=t9,tmp=(C_word)a,a+=4,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21714,a[2]=t10,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21718,a[2]=t11,tmp=(C_word)a,a+=3,tmp);
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21722,a[2]=t12,a[3]=t6,a[4]=t7,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2533: file-location */
t14=t8;
f_21675(t14,t13);}
else{
t9=C_SCHEME_UNDEFINED;
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}

/* file-location in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_21675(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,0,3)))){
C_save_and_reclaim_args((void *)trf_21675,2,t0,t1);}
a=C_alloc(2);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21681,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2525: any */
f_7159(t1,t2,((C_word*)t0)[2]);}

/* a21680 in file-location in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21681,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21697,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2525: node-source-prefix */
f_21231(t3,t2);}

/* k21695 in a21680 in file-location in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21697(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21697,c,av);}
if(C_truep(C_i_not(C_u_i_string_equal_p(lf[535],t1)))){
/* scrutinizer.scm:2526: node-source-prefix */
f_21231(((C_word*)t0)[2],((C_word*)t0)[3]);}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21701 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21703,c,av);}
/* scrutinizer.scm:2537: chicken.base#flush-output */
t2=*((C_word*)lf[536]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k21708 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21710,c,av);}
/* scrutinizer.scm:2529: report-f */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21712 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21714,c,av);}
/* scrutinizer.scm:2530: chicken.string#conc */
t2=*((C_word*)lf[320]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k21716 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21718(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21718,c,av);}
a=C_alloc(3);
/* scrutinizer.scm:2532: string-add-indent */
f_20545(((C_word*)t0)[2],t1,C_a_i_list(&a,1,lf[537]));}

/* k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21722,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t1)){
/* scrutinizer.scm:2533: chicken.string#conc */
t3=*((C_word*)lf[320]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[541];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[542];
f_21725(2,av2);}}}

/* k21723 in k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_21725,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21729,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2534: location-name */
f_21261(t2,((C_word*)t0)[5],C_a_i_list(&a,1,lf[540]));}

/* k21727 in k21723 in k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21729(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21729,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21733,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2535: chicken.base#open-output-string */
t3=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k21731 in k21727 in k21723 in k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_21733,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21739,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=t3;
av2[2]=*((C_word*)lf[539]+1);
av2[3]=t1;
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
C_apply(6,av2);}}

/* k21737 in k21731 in k21727 in k21723 in k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21739,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21742,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2535: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21740 in k21737 in k21731 in k21727 in k21723 in k21720 in chicken.compiler.scrutinizer#report2 in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21742,c,av);}
/* scrutinizer.scm:2533: chicken.string#conc */
t2=*((C_word*)lf[320]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[538];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* variable-node-name in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_21756(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_21756,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21795,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2546: chicken.compiler.support#node-class */
t4=*((C_word*)lf[88]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k21772 in k21793 in variable-node-name in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21774,c,av);}
/* scrutinizer.scm:2547: variable-node-name */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21756(t2,((C_word*)t0)[3],C_i_car(t1));}

/* k21785 in k21789 in k21793 in variable-node-name in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_21787(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21787,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k21789 in k21793 in variable-node-name in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21791,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[87],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21787,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2548: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21793 in variable-node-name in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21795,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[85],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21774,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2547: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21791,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2548: chicken.compiler.support#node-class */
t4=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,16)))){
C_save_and_reclaim((void *)f_21801,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21812,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2556: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[92];
av2[3]=lf[93];
av2[4]=lf[94];
av2[5]=lf[95];
av2[6]=lf[96];
av2[7]=lf[97];
av2[8]=lf[98];
av2[9]=lf[99];
av2[10]=lf[100];
((C_proc)(void*)(*((C_word*)t4+1)))(11,av2);}}
else{
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21839,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2574: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 17) {
  av2=av;
} else {
  av2=C_alloc(17);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[103];
av2[3]=lf[104];
av2[4]=lf[105];
av2[5]=lf[106];
av2[6]=lf[107];
av2[7]=lf[108];
av2[8]=lf[109];
av2[9]=lf[110];
av2[10]=lf[111];
av2[11]=lf[112];
av2[12]=lf[113];
av2[13]=lf[114];
av2[14]=lf[115];
av2[15]=lf[116];
av2[16]=lf[117];
((C_proc)(void*)(*((C_word*)t4+1)))(17,av2);}}}

/* k21810 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21812,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21816,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2566: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k21814 in k21810 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21816,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21820,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21828,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2567: variable-node-name */
t4=((C_word*)((C_word*)t0)[7])[1];
f_21756(t4,t3,((C_word*)t0)[8]);}

/* k21818 in k21814 in k21810 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_21820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_21820,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21824,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2568: type->pp-string */
f_20636(t2,((C_word*)t0)[7]);}

/* k21822 in k21818 in k21814 in k21810 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_21824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_21824,c,av);}
/* scrutinizer.scm:2551: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[89];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=t1;
f_21672(10,av2);}}

/* k21826 in k21814 in k21810 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_21828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21828,c,av);}
/* scrutinizer.scm:2567: variable-from-module */
f_21482(((C_word*)t0)[2],t1);}

/* k21837 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21839(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21839,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21843,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2590: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k21841 in k21837 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21843,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21847,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2591: describe-expression */
f_21542(t2,((C_word*)t0)[8]);}

/* k21845 in k21841 in k21837 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_21847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21847,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21851,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2592: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k21849 in k21845 in k21841 in k21837 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_21851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21851,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21855,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2593: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k21853 in k21849 in k21845 in k21841 in k21837 in k21799 in k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_21855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,10)))){
C_save_and_reclaim((void *)f_21855,c,av);}
/* scrutinizer.scm:2569: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[101];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=((C_word*)t0)[8];
av2[10]=t1;
f_21672(11,av2);}}

/* k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21862(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,10)))){
C_save_and_reclaim((void *)f_21862,c,av);}
a=C_alloc(13);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21873,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=t3,a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2602: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[177];
av2[3]=lf[178];
av2[4]=lf[179];
av2[5]=lf[180];
av2[6]=lf[181];
av2[7]=lf[182];
av2[8]=lf[183];
av2[9]=lf[184];
av2[10]=lf[185];
((C_proc)(void*)(*((C_word*)t5+1)))(11,av2);}}

/* k21871 in k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_21873,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21877,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2612: pp-fragment */
f_21207(t2,((C_word*)t0)[9]);}

/* k21875 in k21871 in k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21877(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21877,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21881,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2613: strip-namespace */
f_19447(t2,((C_word*)((C_word*)t0)[9])[1]);}

/* k21879 in k21875 in k21871 in k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_21881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_21881,c,av);}
a=C_alloc(14);
t2=C_i_nequalp(((C_word*)t0)[2],C_fix(1));
t3=(C_truep(t2)?lf[171]:lf[172]);
t4=C_i_nequalp(((C_word*)t0)[3],C_fix(1));
t5=(C_truep(t4)?lf[171]:lf[172]);
t6=C_i_nequalp(((C_word*)t0)[3],C_fix(1));
t7=(C_truep(t6)?lf[173]:lf[174]);
t8=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21897,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=t1,a[8]=((C_word*)t0)[2],a[9]=t3,a[10]=((C_word*)t0)[3],a[11]=t5,a[12]=t7,a[13]=((C_word*)t0)[9],tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:2617: variable-from-module */
f_21482(t8,((C_word*)((C_word*)t0)[10])[1]);}

/* k21895 in k21879 in k21875 in k21871 in k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_21897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_21897,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21901,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t1,tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:2618: type->pp-string */
f_20636(t2,((C_word*)t0)[13]);}

/* k21899 in k21895 in k21879 in k21875 in k21871 in k21860 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_21901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,15)))){
C_save_and_reclaim((void *)f_21901,c,av);}
/* scrutinizer.scm:2597: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 16) {
  av2=av;
} else {
  av2=C_alloc(16);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[175];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=((C_word*)t0)[8];
av2[10]=((C_word*)t0)[9];
av2[11]=((C_word*)t0)[10];
av2[12]=((C_word*)t0)[11];
av2[13]=((C_word*)t0)[12];
av2[14]=((C_word*)t0)[13];
av2[15]=t1;
f_21672(16,av2);}}

/* chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_21906(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8){
C_word tmp;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_21906,8,t1,t2,t3,t4,t5,t6,t7,t8);}
a=C_alloc(16);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21911,a[2]=t10,a[3]=t3,a[4]=t1,a[5]=t2,a[6]=t4,a[7]=t8,a[8]=t5,a[9]=t6,a[10]=t7,tmp=(C_word)a,a+=11,tmp);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f28236,a[2]=t11,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t13=*((C_word*)lf[86]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}

/* k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,18)))){
C_save_and_reclaim((void *)f_21911,c,av);}
a=C_alloc(15);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21922,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[3],tmp=(C_word)a,a+=12,tmp);
/* scrutinizer.scm:2627: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 19) {
  av2=av;
} else {
  av2=C_alloc(19);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[544];
av2[3]=lf[545];
av2[4]=lf[546];
av2[5]=lf[547];
av2[6]=lf[548];
av2[7]=lf[549];
av2[8]=lf[550];
av2[9]=lf[551];
av2[10]=lf[552];
av2[11]=lf[553];
av2[12]=lf[554];
av2[13]=lf[555];
av2[14]=lf[556];
av2[15]=lf[557];
av2[16]=lf[558];
av2[17]=lf[559];
av2[18]=lf[560];
((C_proc)(void*)(*((C_word*)t5+1)))(19,av2);}}

/* k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_21922,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21926,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* scrutinizer.scm:2645: pp-fragment */
f_21207(t2,((C_word*)t0)[11]);}

/* k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_21926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21926,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21930,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2647: strip-namespace */
f_19447(t2,((C_word*)((C_word*)t0)[8])[1]);}

/* k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_21930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21930,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21934,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2648: type->pp-string */
f_20636(t2,((C_word*)t0)[12]);}

/* k21932 in k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_21934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21934,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21938,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2649: type->pp-string */
f_20636(t2,((C_word*)t0)[12]);}

/* k21936 in k21932 in k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_21938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21938,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21942,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2650: describe-expression */
f_21542(t2,((C_word*)t0)[12]);}

/* k21940 in k21936 in k21932 in k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_21942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21942,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21946,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2651: variable-from-module */
f_21482(t2,((C_word*)((C_word*)t0)[12])[1]);}

/* k21944 in k21940 in k21936 in k21932 in k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_21946(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21946,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21950,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:2652: type->pp-string */
f_20636(t2,((C_word*)t0)[12]);}

/* k21948 in k21944 in k21940 in k21936 in k21932 in k21928 in k21924 in k21920 in k21909 in chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_21950(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,14)))){
C_save_and_reclaim((void *)f_21950,c,av);}
/* scrutinizer.scm:2622: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 15) {
  av2=av;
} else {
  av2=C_alloc(15);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[543];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=((C_word*)t0)[8];
av2[10]=((C_word*)t0)[9];
av2[11]=((C_word*)t0)[10];
av2[12]=((C_word*)t0)[11];
av2[13]=((C_word*)t0)[12];
av2[14]=t1;
f_21672(15,av2);}}

/* chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_21952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_21952,c,av);}
a=C_alloc(17);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21957,a[2]=t8,a[3]=t10,a[4]=t5,a[5]=t3,a[6]=t2,a[7]=t4,a[8]=t6,a[9]=t1,tmp=(C_word)a,a+=10,tmp);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f28241,a[2]=t11,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t13=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}

/* k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_21957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_21957,c,av);}
a=C_alloc(12);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21959,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp));
t4=C_i_length(((C_word*)t0)[8]);
t5=C_eqp(t4,C_fix(0));
if(C_truep(t5)){
/* scrutinizer.scm:2676: p */
t6=((C_word*)((C_word*)t0)[3])[1];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[9];
av2[2]=lf[570];
av2[3]=lf[571];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22022,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2679: chicken.base#open-output-string */
t7=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* f_21959 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_21959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,8)))){
C_save_and_reclaim((void *)f_21959,c,av);}
a=C_alloc(17);
t4=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21971,a[2]=t1,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t3,a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[3],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2662: scheme#string-append */
t6=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[563];
av2[3]=lf[564];
av2[4]=lf[565];
av2[5]=lf[566];
av2[6]=lf[567];
av2[7]=lf[568];
av2[8]=lf[569];
((C_proc)(void*)(*((C_word*)t6+1)))(9,av2);}}

/* k21969 */
static void C_ccall f_21971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21971,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21975,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2670: pp-fragment */
f_21207(t2,((C_word*)t0)[10]);}

/* k21973 in k21969 */
static void C_ccall f_21975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_21975,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21979,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_zerop(((C_word*)t0)[7]))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[561];
f_21979(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21987,a[2]=t2,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2672: chicken.base#open-output-string */
t4=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k21977 in k21973 in k21969 */
static void C_ccall f_21979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21979,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21983,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2674: describe-expression */
f_21542(t2,((C_word*)t0)[10]);}

/* k21981 in k21977 in k21973 in k21969 */
static void C_ccall f_21983(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,11)))){
C_save_and_reclaim((void *)f_21983,c,av);}
/* scrutinizer.scm:2657: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 12) {
  av2=av;
} else {
  av2=C_alloc(12);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=((C_word*)t0)[9];
av2[10]=((C_word*)t0)[10];
av2[11]=t1;
f_21672(12,av2);}}

/* k21985 in k21973 in k21969 */
static void C_ccall f_21987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_21987,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21993,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2672: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[562];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21991 in k21985 in k21973 in k21969 */
static void C_ccall f_21993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_21993,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21996,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22006,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2672: strip-namespace */
f_19447(t3,((C_word*)((C_word*)t0)[4])[1]);}

/* k21994 in k21991 in k21985 in k21973 in k21969 */
static void C_ccall f_21996(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_21996,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21999,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2672: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21997 in k21994 in k21991 in k21985 in k21973 in k21969 */
static void C_ccall f_21999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21999,c,av);}
/* scrutinizer.scm:2672: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22004 in k21991 in k21985 in k21973 in k21969 */
static void C_ccall f_22006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22006,c,av);}
/* scrutinizer.scm:2672: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22020 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22022,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22028,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2679: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[574];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22026 in k22020 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22028(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22028,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22031,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2679: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[5]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22029 in k22026 in k22020 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22031(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22031,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22034,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2679: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[573];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22032 in k22029 in k22026 in k22020 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22034,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22037,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2679: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22035 in k22032 in k22029 in k22026 in k22020 in k21955 in chicken.compiler.scrutinizer#r-proc-call-argument-value-count in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22037(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22037,c,av);}
/* scrutinizer.scm:2678: p */
t2=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[572];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_22045(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_22045,6,t1,t2,t3,t4,t5,t6);}
a=C_alloc(14);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22050,a[2]=t8,a[3]=t3,a[4]=t1,a[5]=t2,a[6]=t4,a[7]=t5,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f28246,a[2]=t9,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t11=*((C_word*)lf[86]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}

/* k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22050(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_22050,c,av);}
a=C_alloc(13);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22057,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[2],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_negativep(((C_word*)t0)[6]))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22130,a[2]=t3,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2686: chicken.base#open-output-string */
t5=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22148,a[2]=t3,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2687: chicken.base#open-output-string */
t5=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22057(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,6)))){
C_save_and_reclaim((void *)f_22057,c,av);}
a=C_alloc(14);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22065,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2693: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[580];
av2[3]=lf[581];
av2[4]=lf[582];
av2[5]=lf[583];
av2[6]=lf[584];
((C_proc)(void*)(*((C_word*)t4+1)))(7,av2);}}

/* k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_22065,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22069,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2699: pp-fragment */
f_21207(t2,((C_word*)t0)[10]);}

/* k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_22069,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22073,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2700: variable-from-module */
f_21482(t2,((C_word*)((C_word*)t0)[10])[1]);}

/* k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22073(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_22073,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22077,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_negativep(((C_word*)t0)[8]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22081,a[2]=t2,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2702: chicken.base#open-output-string */
t4=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22099,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2703: chicken.base#open-output-string */
t4=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k22075 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_22077,c,av);}
/* scrutinizer.scm:2684: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=t1;
f_21672(10,av2);}}

/* k22079 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22081(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22081,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22087,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2702: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[575];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22085 in k22079 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_22087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22087,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22090,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2702: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22088 in k22085 in k22079 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_22090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_22090,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22093,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2702: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(46);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22091 in k22088 in k22085 in k22079 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22093,c,av);}
/* scrutinizer.scm:2702: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22099(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_22099,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22105,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[579];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_22105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22105,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22108,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_22108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22108,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22111,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[578];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22109 in k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22111,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22114,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22112 in k22109 in k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in ... */
static void C_ccall f_22114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22114,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22117,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[577];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22115 in k22112 in k22109 in k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in ... */
static void C_ccall f_22117(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22117,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22120,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22118 in k22115 in k22112 in k22109 in k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in ... */
static void C_ccall f_22120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22120,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22123,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2703: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[576];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22121 in k22118 in k22115 in k22112 in k22109 in k22106 in k22103 in k22097 in k22071 in k22067 in k22063 in k22055 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in ... */
static void C_ccall f_22123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22123,c,av);}
/* scrutinizer.scm:2703: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22128 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22130,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22136,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2686: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[586];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22134 in k22128 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22136,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22139,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2686: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22137 in k22134 in k22128 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22139,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22142,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2686: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[585];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22140 in k22137 in k22134 in k22128 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22142,c,av);}
/* scrutinizer.scm:2686: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22146 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22148(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22148,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22154,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=C_i_string_ref(((C_word*)t0)[3],C_fix(0));
/* scrutinizer.scm:2687: ##sys#print */
t5=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_u_i_char_upcase(t4);
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k22152 in k22146 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_22154,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22157,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22167,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2689: scheme#substring */
t4=*((C_word*)lf[588]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fix(1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k22155 in k22152 in k22146 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_22157,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22160,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2687: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[587];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22158 in k22155 in k22152 in k22146 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22160,c,av);}
/* scrutinizer.scm:2687: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22165 in k22152 in k22146 in k22048 in chicken.compiler.scrutinizer#r-index-out-of-range in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22167,c,av);}
/* scrutinizer.scm:2687: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22177(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_22177,c,av);}
a=C_alloc(10);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22180,a[2]=t4,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t7=C_i_length(t5);
t8=C_eqp(t7,C_fix(0));
if(C_truep(t8)){
/* scrutinizer.scm:2721: p */
t9=t6;
f_22180(t9,t1,lf[596],lf[597]);}
else{
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22216,a[2]=t6,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2724: chicken.base#open-output-string */
t10=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}

/* p in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_22180(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,8)))){
C_save_and_reclaim_args((void *)trf_22180,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22192,a[2]=t1,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2709: scheme#string-append */
t6=*((C_word*)lf[76]+1);{
C_word av2[9];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[589];
av2[3]=lf[590];
av2[4]=lf[591];
av2[5]=lf[592];
av2[6]=lf[593];
av2[7]=lf[594];
av2[8]=lf[595];
((C_proc)(void*)(*((C_word*)t6+1)))(9,av2);}}

/* k22190 in p in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22192,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22196,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2717: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k22194 in k22190 in p in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22196,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22200,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2719: describe-expression */
f_21542(t2,((C_word*)t0)[8]);}

/* k22198 in k22194 in k22190 in p in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_22200,c,av);}
/* scrutinizer.scm:2707: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=t1;
f_21672(10,av2);}}

/* k22214 in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22216,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22222,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2724: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[600];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22220 in k22214 in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22222(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22222,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22225,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2724: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[5]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22223 in k22220 in k22214 in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22225,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22228,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2724: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[599];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22226 in k22223 in k22220 in k22214 in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22228(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22228,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22231,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2724: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22229 in k22226 in k22223 in k22220 in k22214 in chicken.compiler.scrutinizer#r-conditional-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22231,c,av);}
/* scrutinizer.scm:2723: p */
t2=((C_word*)t0)[2];
f_22180(t2,((C_word*)t0)[3],lf[598],t1);}

/* chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_22239,c,av);}
a=C_alloc(12);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22242,a[2]=t5,a[3]=t4,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t8=C_i_length(t6);
t9=C_eqp(t8,C_fix(0));
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22279,a[2]=t7,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2743: chicken.base#open-output-string */
t11=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22305,a[2]=t7,a[3]=t1,a[4]=t6,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2745: chicken.base#open-output-string */
t11=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}

/* p in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_22242(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,8)))){
C_save_and_reclaim_args((void *)trf_22242,4,t0,t1,t2,t3);}
a=C_alloc(16);
t4=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22254,a[2]=t1,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2730: scheme#string-append */
t6=*((C_word*)lf[76]+1);{
C_word av2[9];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[601];
av2[3]=lf[602];
av2[4]=lf[603];
av2[5]=lf[604];
av2[6]=lf[605];
av2[7]=lf[606];
av2[8]=lf[607];
((C_proc)(void*)(*((C_word*)t6+1)))(9,av2);}}

/* k22252 in p in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22254,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22258,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2738: pp-fragment */
f_21207(t2,((C_word*)t0)[9]);}

/* k22256 in k22252 in p in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22258,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22262,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2739: chicken.compiler.support#real-name */
t3=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22260 in k22256 in k22252 in p in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22262,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22266,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2741: describe-expression */
f_21542(t2,((C_word*)t0)[9]);}

/* k22264 in k22260 in k22256 in k22252 in p in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22266(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,10)))){
C_save_and_reclaim((void *)f_22266,c,av);}
/* scrutinizer.scm:2728: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=((C_word*)t0)[9];
av2[10]=t1;
f_21672(11,av2);}}

/* k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22279,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22285,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2743: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[610];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22283 in k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22285,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22288,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22298,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2743: chicken.compiler.support#real-name */
t4=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22286 in k22283 in k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22288(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22288,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22291,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2743: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[609];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22289 in k22286 in k22283 in k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22291(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22291,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22294,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2743: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22292 in k22289 in k22286 in k22283 in k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22294,c,av);}
/* scrutinizer.scm:2743: p */
t2=((C_word*)t0)[2];
f_22242(t2,((C_word*)t0)[3],t1,lf[608]);}

/* k22296 in k22283 in k22277 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22298,c,av);}
/* scrutinizer.scm:2743: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_22305,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22311,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2745: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[615];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22311,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22314,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22353,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2745: chicken.compiler.support#real-name */
t4=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22314,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22317,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2745: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[614];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22317,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22320,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2745: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[4]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22320,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22323,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2745: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[613];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22323,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22326,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2745: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_22326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22326,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22330,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2746: chicken.base#open-output-string */
t3=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k22328 in k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_22330(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_22330,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22336,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2746: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[612];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22334 in k22328 in k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22336,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22339,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2746: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[6]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22337 in k22334 in k22328 in k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in ... */
static void C_ccall f_22339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22339,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22342,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2746: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[611];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22340 in k22337 in k22334 in k22328 in k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in ... */
static void C_ccall f_22342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22342,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22345,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2746: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22343 in k22340 in k22337 in k22334 in k22328 in k22324 in k22321 in k22318 in k22315 in k22312 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in ... */
static void C_ccall f_22345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22345,c,av);}
/* scrutinizer.scm:2745: p */
t2=((C_word*)t0)[2];
f_22242(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k22351 in k22309 in k22303 in chicken.compiler.scrutinizer#r-let-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22353,c,av);}
/* scrutinizer.scm:2745: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22359(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_22359,c,av);}
a=C_alloc(12);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22362,a[2]=t5,a[3]=t4,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t8=C_i_length(t6);
t9=C_eqp(t8,C_fix(0));
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22399,a[2]=t7,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2765: chicken.base#open-output-string */
t11=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22425,a[2]=t7,a[3]=t1,a[4]=t6,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2767: chicken.base#open-output-string */
t11=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}

/* p in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_22362(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,8)))){
C_save_and_reclaim_args((void *)trf_22362,4,t0,t1,t2,t3);}
a=C_alloc(16);
t4=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22374,a[2]=t1,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2752: scheme#string-append */
t6=*((C_word*)lf[76]+1);{
C_word av2[9];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[616];
av2[3]=lf[617];
av2[4]=lf[618];
av2[5]=lf[619];
av2[6]=lf[620];
av2[7]=lf[621];
av2[8]=lf[622];
((C_proc)(void*)(*((C_word*)t6+1)))(9,av2);}}

/* k22372 in p in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22374,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22378,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2760: pp-fragment */
f_21207(t2,((C_word*)t0)[9]);}

/* k22376 in k22372 in p in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22378,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22382,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2761: strip-namespace */
f_19447(t2,((C_word*)t0)[9]);}

/* k22380 in k22376 in k22372 in p in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22382,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22386,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2763: describe-expression */
f_21542(t2,((C_word*)t0)[9]);}

/* k22384 in k22380 in k22376 in k22372 in p in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,10)))){
C_save_and_reclaim((void *)f_22386,c,av);}
/* scrutinizer.scm:2750: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 11) {
  av2=av;
} else {
  av2=C_alloc(11);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=((C_word*)t0)[9];
av2[10]=t1;
f_21672(11,av2);}}

/* k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22399,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22405,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2765: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[625];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22403 in k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22405,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22408,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22418,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2765: strip-namespace */
f_19447(t3,((C_word*)t0)[5]);}

/* k22406 in k22403 in k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22408,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22411,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2765: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[624];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22409 in k22406 in k22403 in k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22411(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22411,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22414,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2765: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22412 in k22409 in k22406 in k22403 in k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22414,c,av);}
/* scrutinizer.scm:2765: p */
t2=((C_word*)t0)[2];
f_22362(t2,((C_word*)t0)[3],t1,lf[623]);}

/* k22416 in k22403 in k22397 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22418,c,av);}
/* scrutinizer.scm:2765: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_22425,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22431,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2767: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[630];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22431,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22434,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22473,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2767: strip-namespace */
f_19447(t3,((C_word*)t0)[6]);}

/* k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22434,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22437,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2767: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[629];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22437,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22440,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2767: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[4]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22440,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22443,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2767: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[628];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22443,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22446,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2767: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_22446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22446,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22450,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2768: chicken.base#open-output-string */
t3=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k22448 in k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_22450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_22450,c,av);}
a=C_alloc(7);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2768: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[627];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22454 in k22448 in k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22456,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22459,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2768: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_length(((C_word*)t0)[6]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22457 in k22454 in k22448 in k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in ... */
static void C_ccall f_22459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22459,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22462,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2768: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[626];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22460 in k22457 in k22454 in k22448 in k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in ... */
static void C_ccall f_22462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22462,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22465,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2768: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22463 in k22460 in k22457 in k22454 in k22448 in k22444 in k22441 in k22438 in k22435 in k22432 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in ... */
static void C_ccall f_22465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22465,c,av);}
/* scrutinizer.scm:2767: p */
t2=((C_word*)t0)[2];
f_22362(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k22471 in k22429 in k22423 in chicken.compiler.scrutinizer#r-assignment-value-count-invalid in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22473,c,av);}
/* scrutinizer.scm:2767: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,14)))){
C_save_and_reclaim((void *)f_22484,c,av);}
a=C_alloc(12);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22495,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2776: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 15) {
  av2=av;
} else {
  av2=C_alloc(15);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[129];
av2[3]=lf[130];
av2[4]=lf[131];
av2[5]=lf[132];
av2[6]=lf[133];
av2[7]=lf[134];
av2[8]=lf[135];
av2[9]=lf[136];
av2[10]=lf[137];
av2[11]=lf[138];
av2[12]=lf[139];
av2[13]=lf[140];
av2[14]=lf[141];
((C_proc)(void*)(*((C_word*)t5+1)))(15,av2);}}

/* k22493 in k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22495,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22499,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2790: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k22497 in k22493 in k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22499,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22503,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2791: variable-from-module */
f_21482(t2,((C_word*)((C_word*)t0)[8])[1]);}

/* k22501 in k22497 in k22493 in k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_22503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22503,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22507,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2792: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k22505 in k22501 in k22497 in k22493 in k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_22507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22507,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22511,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2793: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k22509 in k22505 in k22501 in k22497 in k22493 in k22482 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in ... */
static void C_ccall f_22511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,8)))){
C_save_and_reclaim((void *)f_22511,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,4,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[71];
av2[3]=lf[128];
av2[4]=*((C_word*)lf[73]+1);
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=t2;
C_apply(9,av2);}}

/* k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,14)))){
C_save_and_reclaim((void *)f_22518,c,av);}
a=C_alloc(12);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22529,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2801: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 15) {
  av2=av;
} else {
  av2=C_alloc(15);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[145];
av2[3]=lf[146];
av2[4]=lf[147];
av2[5]=lf[148];
av2[6]=lf[149];
av2[7]=lf[150];
av2[8]=lf[151];
av2[9]=lf[152];
av2[10]=lf[153];
av2[11]=lf[154];
av2[12]=lf[155];
av2[13]=lf[156];
av2[14]=lf[157];
((C_proc)(void*)(*((C_word*)t5+1)))(15,av2);}}

/* k22527 in k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_22529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22529,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22533,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2815: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k22531 in k22527 in k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_22533(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22533,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22537,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2816: variable-from-module */
f_21482(t2,((C_word*)((C_word*)t0)[8])[1]);}

/* k22535 in k22531 in k22527 in k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in ... */
static void C_ccall f_22537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22537,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22541,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2817: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k22539 in k22535 in k22531 in k22527 in k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in ... */
static void C_ccall f_22541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22541,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22545,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2818: type->pp-string */
f_20636(t2,((C_word*)t0)[8]);}

/* k22543 in k22539 in k22535 in k22531 in k22527 in k22516 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in ... */
static void C_ccall f_22545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,8)))){
C_save_and_reclaim((void *)f_22545,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,4,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[71];
av2[3]=lf[144];
av2[4]=*((C_word*)lf[73]+1);
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=t2;
C_apply(9,av2);}}

/* k22557 in k8619 in always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22559,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22563,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2833: pp-fragment */
f_21207(t2,((C_word*)t0)[6]);}

/* k22561 in k22557 in k8619 in always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22563,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22567,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2834: type->pp-string */
f_20636(t2,((C_word*)t0)[6]);}

/* k22565 in k22561 in k22557 in k8619 in always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,8)))){
C_save_and_reclaim((void *)f_22567,c,av);}
a=C_alloc(6);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[71];
av2[3]=lf[72];
av2[4]=*((C_word*)lf[73]+1);
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=t2;
C_apply(9,av2);}}

/* k22579 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22581,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22585,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2847: pp-fragment */
f_21207(t2,((C_word*)t0)[5]);}

/* k22583 in k22579 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,8)))){
C_save_and_reclaim((void *)f_22585,c,av);}
a=C_alloc(3);
t2=C_a_i_list(&a,1,t1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[71];
av2[3]=lf[213];
av2[4]=*((C_word*)lf[73]+1);
av2[5]=((C_word*)t0)[3];
av2[6]=((C_word*)t0)[4];
av2[7]=((C_word*)t0)[5];
av2[8]=t2;
C_apply(9,av2);}}

/* k22597 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22599,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22603,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2864: pp-fragment */
f_21207(t2,((C_word*)t0)[6]);}

/* k22601 in k22597 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22603,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22607,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2865: type->pp-string */
f_20636(t2,((C_word*)t0)[6]);}

/* k22605 in k22601 in k22597 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22607(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_22607,c,av);}
/* scrutinizer.scm:2851: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[278];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=t1;
f_21672(9,av2);}}

/* k22619 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22621,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22625,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2883: pp-fragment */
f_21207(t2,((C_word*)t0)[7]);}

/* k22623 in k22619 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22625,c,av);}
a=C_alloc(8);
t2=C_i_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22633,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2885: type->pp-string */
f_20636(t3,((C_word*)t0)[7]);}

/* k22631 in k22623 in k22619 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_22633,c,av);}
/* scrutinizer.scm:2868: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[301];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=t1;
f_21672(10,av2);}}

/* k22645 in k11290 in k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22647,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22651,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2907: pp-fragment */
f_21207(t2,((C_word*)t0)[7]);}

/* k22649 in k22645 in k11290 in k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22651,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22655,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2908: type->pp-string */
f_20636(t2,((C_word*)t0)[7]);}

/* k22653 in k22649 in k22645 in k11290 in k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22655,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22659,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2909: type->pp-string */
f_20636(t2,((C_word*)t0)[7]);}

/* k22657 in k22653 in k22649 in k22645 in k11290 in k11287 in k11281 in k11256 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_22659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_22659,c,av);}
/* scrutinizer.scm:2888: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[286];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=t1;
f_21672(10,av2);}}

/* k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_22679,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22683,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22748,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2932: node-source-prefix */
f_21231(t3,((C_word*)t0)[5]);}

/* k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_22683,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22687,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2935: location-name */
f_21261(t2,((C_word*)t0)[7],C_SCHEME_END_OF_LIST);}

/* k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22687,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22691,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2936: pp-fragment */
f_21207(t2,((C_word*)t0)[7]);}

/* k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22691(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_22691,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22695,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f26001,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2912: type->pp-string */
f_20636(t3,((C_word*)t0)[7]);}

/* k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_22695(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_22695,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22699,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[7],lf[6]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22709,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22711,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_22711(t12,t8,((C_word*)t0)[7]);}

/* k22697 in k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_22699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_22699,c,av);}
/* scrutinizer.scm:2913: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[312]+1);{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(8,av2);}}

/* k22707 in k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_22709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22709,c,av);}
/* scrutinizer.scm:2938: chicken.string#string-intersperse */
t2=*((C_word*)lf[313]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[314];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop5615 in k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_22711(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_22711,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22736,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f25994,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2912: type->pp-string */
f_20636(t5,t4);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k22734 in map-loop5615 in k22693 in k22689 in k22685 in k22681 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22736,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_22711(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k22746 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22748,c,av);}
a=C_alloc(3);
if(C_truep(C_u_i_string_equal_p(lf[318],t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[319];
f_22683(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22755,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2934: node-source-prefix */
f_21231(t2,((C_word*)t0)[3]);}}

/* k22753 in k22746 in k22677 in loop in k11347 in k11344 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22755,c,av);}
/* scrutinizer.scm:2934: chicken.string#conc */
t2=*((C_word*)lf[320]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[321];
av2[3]=t1;
av2[4]=lf[322];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22767 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_22769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22769,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22773,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2960: pp-fragment */
f_21207(t2,((C_word*)t0)[9]);}

/* k22771 in k22767 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_22773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_22773,c,av);}
a=C_alloc(11);
t2=C_i_length(((C_word*)t0)[2]);
t3=C_u_i_length(((C_word*)t0)[2]);
t4=C_i_nequalp(t3,C_fix(1));
t5=(C_truep(t4)?lf[171]:lf[172]);
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22785,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t1,a[8]=t2,a[9]=t5,a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:2962: pp-fragment */
f_21207(t6,((C_word*)t0)[9]);}

/* k22783 in k22771 in k22767 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_22785,c,av);}
a=C_alloc(12);
t2=C_i_length(((C_word*)t0)[2]);
t3=C_u_i_length(((C_word*)t0)[2]);
t4=C_i_nequalp(t3,C_fix(1));
t5=(C_truep(t4)?lf[171]:lf[172]);
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_22797,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=t1,a[10]=t2,a[11]=t5,tmp=(C_word)a,a+=12,tmp);
/* scrutinizer.scm:2964: pp-fragment */
f_21207(t6,((C_word*)t0)[10]);}

/* k22795 in k22783 in k22771 in k22767 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,13)))){
C_save_and_reclaim((void *)f_22797,c,av);}
/* scrutinizer.scm:2941: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 14) {
  av2=av;
} else {
  av2=C_alloc(14);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[198];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=((C_word*)t0)[8];
av2[10]=((C_word*)t0)[9];
av2[11]=((C_word*)t0)[10];
av2[12]=((C_word*)t0)[11];
av2[13]=t1;
f_21672(14,av2);}}

/* k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22815(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22815,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22819,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2986: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22819,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22823,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2987: strip-namespace */
f_19447(t2,((C_word*)t0)[7]);}

/* k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_22823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22823,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22827,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2988: type->pp-string */
f_20636(t2,((C_word*)t0)[9]);}

/* k22825 in k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_22827(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_22827,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22831,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22839,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2990: chicken.compiler.support#real-name */
t4=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22829 in k22825 in k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_22831,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22835,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2992: type->pp-string */
f_20636(t2,((C_word*)t0)[9]);}

/* k22833 in k22829 in k22825 in k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,11)))){
C_save_and_reclaim((void *)f_22835,c,av);}
/* scrutinizer.scm:2967: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 12) {
  av2=av;
} else {
  av2=C_alloc(12);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[251];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=((C_word*)t0)[7];
av2[9]=((C_word*)t0)[8];
av2[10]=((C_word*)t0)[9];
av2[11]=t1;
f_21672(12,av2);}}

/* k22837 in k22825 in k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_22839(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22839,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22842,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_symbolp(t1))){
/* scrutinizer.scm:2989: variable-from-module */
f_21482(((C_word*)t0)[2],t1);}
else{
/* scrutinizer.scm:2991: scheme#string->symbol */
t3=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k22840 in k22837 in k22825 in k22821 in k22817 in k22813 in k10580 in k10577 in k10313 in k10310 in k10307 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22842,c,av);}
/* scrutinizer.scm:2989: variable-from-module */
f_21482(((C_word*)t0)[2],t1);}

/* chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_22850(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_22850,5,t1,t2,t3,t4,t5);}
a=C_alloc(7);
t6=C_i_nullp(t5);
t7=(C_truep(t6)?C_SCHEME_FALSE:C_i_car(t5));
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22861,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t7,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2996: chicken.base#open-output-string */
t9=*((C_word*)lf[242]+1);{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_22861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_22861,c,av);}
a=C_alloc(8);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22867,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2996: ##sys#print */
t4=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[639];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_22867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_22867,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22870,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22928,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2996: strip-namespace */
f_19447(t3,((C_word*)t0)[6]);}

/* k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_22870,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22873,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:2996: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_22873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22873,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22876,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2996: chicken.base#get-output-string */
t3=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in ... */
static void C_ccall f_22876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,7)))){
C_save_and_reclaim((void *)f_22876,c,av);}
a=C_alloc(12);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22884,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[2],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:3000: scheme#string-append */
t4=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[633];
av2[3]=lf[634];
av2[4]=lf[635];
av2[5]=lf[636];
av2[6]=lf[637];
av2[7]=lf[638];
((C_proc)(void*)(*((C_word*)t4+1)))(8,av2);}}

/* k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in ... */
static void C_ccall f_22884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22884,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22888,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:3007: pp-fragment */
f_21207(t2,((C_word*)t0)[8]);}

/* k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in ... */
static void C_ccall f_22888(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22888,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22892,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:3008: variable-from-module */
f_21482(t2,((C_word*)t0)[8]);}

/* k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in ... */
static void C_ccall f_22892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_22892,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22896,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[8])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22899,a[2]=t2,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:3010: chicken.base#open-output-string */
t4=*((C_word*)lf[242]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[632];
f_22896(2,av2);}}}

/* k22894 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_22896,c,av);}
/* scrutinizer.scm:2995: report2 */
t2=lf[71];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[90]+1);
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
av2[7]=((C_word*)t0)[7];
av2[8]=((C_word*)t0)[8];
av2[9]=t1;
f_21672(10,av2);}}

/* k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in ... */
static void C_ccall f_22899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_22899,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[235]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22905,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:3010: ##sys#write-char-0 */
t4=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(10);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in ... */
static void C_ccall f_22905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_22905,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22908,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:3010: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22906 in k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in ... */
static void C_ccall f_22908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22908,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22911,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:3010: ##sys#print */
t3=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[631];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22909 in k22906 in k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in ... */
static void C_ccall f_22911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22911,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22914,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22924,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:3011: variable-from-module */
f_21482(t3,((C_word*)t0)[4]);}

/* k22912 in k22909 in k22906 in k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in ... */
static void C_ccall f_22914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_22914,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22917,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:3010: ##sys#write-char-0 */
t3=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(46);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22915 in k22912 in k22909 in k22906 in k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in ... */
static void C_ccall f_22917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22917,c,av);}
/* scrutinizer.scm:3010: chicken.base#get-output-string */
t2=*((C_word*)lf[238]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22922 in k22909 in k22906 in k22903 in k22897 in k22890 in k22886 in k22882 in k22874 in k22871 in k22868 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in ... */
static void C_ccall f_22924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22924,c,av);}
/* scrutinizer.scm:3010: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22926 in k22865 in k22859 in chicken.compiler.scrutinizer#r-deprecated-identifier in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_22928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22928,c,av);}
/* scrutinizer.scm:2996: ##sys#print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a22939 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22940,c,av);}
a=C_alloc(5);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22944,a[2]=t3,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2233: chicken.compiler.support#node-subexpressions */
t7=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k22942 in a22939 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_22944(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22944,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_i_length(t1);
t3=C_eqp(t2,C_fix(2));
if(C_truep(t3)){
t4=C_i_cadr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f26674,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k22983 */
static void C_ccall f_22985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_22985,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[47],t1);
t3=C_a_i_list(&a,1,t2);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t3:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a23001 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_23002,c,av);}
a=C_alloc(15);
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=lf[25];
t11=C_i_cdr(t3);
t12=C_i_check_list_2(t11,lf[6]);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23023,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23025,a[2]=t8,a[3]=t15,a[4]=t9,tmp=(C_word)a,a+=5,tmp));
t17=((C_word*)t15)[1];
f_23025(t17,t13,t11);}

/* k23021 in a23001 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23023,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[50],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop4578 in a23001 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_23025(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_23025,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26679,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a23058 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_23059,c,av);}
a=C_alloc(15);
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=lf[25];
t11=C_i_cdr(t3);
t12=C_i_check_list_2(t11,lf[6]);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23080,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23082,a[2]=t8,a[3]=t15,a[4]=t9,tmp=(C_word)a,a+=5,tmp));
t17=((C_word*)t15)[1];
f_23082(t17,t13,t11);}

/* k23078 in a23058 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23080,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[50],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop4544 in a23058 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_23082(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_23082,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26686,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a23115 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23116(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_23116,c,av);}
a=C_alloc(15);
t6=C_i_cdr(t3);
if(C_truep(C_i_nullp(t6))){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[644];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=lf[25];
t12=C_u_i_cdr(t3);
t13=C_i_check_list_2(t12,lf[6]);
t14=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23141,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23143,a[2]=t9,a[3]=t16,a[4]=t10,tmp=(C_word)a,a+=5,tmp));
t18=((C_word*)t16)[1];
f_23143(t18,t14,t12);}}

/* k23139 in a23115 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23141,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[47],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop4510 in a23115 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_23143(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_23143,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26693,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a23180 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_23181,c,av);}
a=C_alloc(15);
t6=C_i_cdr(t3);
if(C_truep(C_i_nullp(t6))){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[646];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=lf[25];
t12=C_u_i_cdr(t3);
t13=C_i_check_list_2(t12,lf[6]);
t14=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23206,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23208,a[2]=t9,a[3]=t16,a[4]=t10,tmp=(C_word)a,a+=5,tmp));
t18=((C_word*)t16)[1];
f_23208(t18,t14,t12);}}

/* k23204 in a23180 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23206,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[47],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop4476 in a23180 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_23208(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_23208,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26700,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k23246 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_23248,c,av);}
/* scrutinizer.scm:2207: ##sys#put! */
t2=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[648];
av2[3]=lf[274];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a23249 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_23250,c,av);}
a=C_alloc(3);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list1(&a,1,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k23256 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_23258,c,av);}
/* scrutinizer.scm:2201: ##sys#put! */
t2=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[649];
av2[3]=lf[274];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a23259 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_23260,c,av);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t3))){
t4=C_i_cadr(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_list1(&a,1,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a23275 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23276(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_23276,c,av);}
a=C_alloc(6);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23280,a[2]=t3,a[3]=t1,a[4]=t5,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2124: known-length-vector-index */
f_19497(t6,t2,t3,t4,C_fix(3));}

/* k23278 in a23275 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23280,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23289,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2125: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k23287 in k23278 in a23275 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_23289,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_cadr(t1);
if(C_truep(t2)){
t3=C_i_cadddr(t1);
if(C_truep(t3)){
t4=C_i_cadddr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f26707,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=t3,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k23314 in k23325 */
static void C_ccall f_23316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23316,c,av);}
t2=lf[652];
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?t2:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k23325 */
static void C_ccall f_23327(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_23327,c,av);}
a=C_alloc(22);
t2=(C_truep(t1)?lf[650]:lf[651]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23316,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_list(&a,2,lf[123],((C_word*)t0)[4]);
t5=C_a_i_list(&a,4,t2,((C_word*)t0)[5],t4,((C_word*)t0)[6]);
/* scrutinizer.scm:2132: mutate-node! */
f_17320(t3,((C_word*)t0)[7],t5);}

/* a23332 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_23333,c,av);}
a=C_alloc(4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23337,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2078: chicken.compiler.support#node-subexpressions */
t7=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k23335 in a23332 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_23337,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_length(t1);
if(C_truep(C_fixnum_greater_or_equal_p(t2,C_fix(2)))){
t3=C_i_cadr(t1);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23398,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:2081: chicken.compiler.support#node-class */
t5=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k23388 in k23392 in k23396 in k23335 in a23332 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_23390,c,av);}
a=C_alloc(9);
t2=C_a_i_list(&a,2,lf[52],t1);
t3=C_a_i_list(&a,1,t2);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t3:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k23392 in k23396 in k23335 in a23332 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23394(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_23394,c,av);}
a=C_alloc(4);
t2=C_i_car(t1);
if(C_truep(t2)){
if(C_truep(C_i_symbolp(t2))){
t3=C_eqp(lf[656],t2);
if(C_truep(t3)){
t4=lf[657];
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?t4:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23390,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2088: strip-namespace */
f_19447(t4,t2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k23396 in k23335 in a23332 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_23398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_23398,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[123],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23394,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:2082: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k6728 */
static void C_ccall f_6730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6730,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6733,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_eval_toplevel(2,av2);}}

/* k6731 in k6728 */
static void C_ccall f_6733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6733,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6736,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_data_2dstructures_toplevel(2,av2);}}

/* k6734 in k6731 in k6728 */
static void C_ccall f_6736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6736,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6739,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_expand_toplevel(2,av2);}}

/* k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6739,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6742,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_extras_toplevel(2,av2);}}

/* k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6742,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6745,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_pathname_toplevel(2,av2);}}

/* k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6745,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6748,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_port_toplevel(2,av2);}}

/* k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6748,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6751,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_support_toplevel(2,av2);}}

/* k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6751(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6751,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6754,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_internal_toplevel(2,av2);}}

/* k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(111,c,8)))){
C_save_and_reclaim((void *)f_6754,c,av);}
a=C_alloc(111);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=C_mutate(&lf[2] /* (set! chicken.compiler.scrutinizer#take ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6864,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate(&lf[3] /* (set! chicken.compiler.scrutinizer#append-map ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6955,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate(&lf[9] /* (set! chicken.compiler.scrutinizer#every ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7125,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[7] /* (set! chicken.compiler.scrutinizer#any ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7159,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate(&lf[10] /* (set! chicken.compiler.scrutinizer#delete-duplicates ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7320,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate(&lf[11] /* (set! chicken.compiler.scrutinizer#filter-map ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7409,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate(&lf[12] /* (set! chicken.compiler.scrutinizer#unzip1 ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7474,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate(&lf[13] /* (set! chicken.compiler.scrutinizer#list-tabulate ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7767,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate(&lf[14] /* (set! chicken.compiler.scrutinizer#lset=/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7809,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate(&lf[15] /* (set! chicken.compiler.scrutinizer#make-list ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7950,tmp=(C_word)a,a+=2,tmp));
t14=lf[16] /* chicken.compiler.scrutinizer#d-depth */ =C_fix(0);;
t15=lf[17] /* chicken.compiler.scrutinizer#scrutiny-debug */ =C_SCHEME_TRUE;;
t16=lf[18] /* chicken.compiler.scrutinizer#*complain?* */ =C_SCHEME_FALSE;;
t17=C_mutate(&lf[19] /* (set! chicken.compiler.scrutinizer#constant723 ...) */,lf[20]);
t18=C_mutate(&lf[21] /* (set! chicken.compiler.scrutinizer#constant730 ...) */,lf[22]);
t19=lf[23] /* chicken.compiler.scrutinizer#specialization-statistics */ =C_SCHEME_END_OF_LIST;;
t20=lf[24] /* chicken.compiler.scrutinizer#trail */ =C_SCHEME_END_OF_LIST;;
t21=C_mutate(&lf[25] /* (set! chicken.compiler.scrutinizer#walked-result ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8079,tmp=(C_word)a,a+=2,tmp));
t22=C_mutate(&lf[27] /* (set! chicken.compiler.scrutinizer#type-always-immediate? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8089,tmp=(C_word)a,a+=2,tmp));
t23=C_mutate((C_word*)lf[36]+1 /* (set! chicken.compiler.scrutinizer#scrutinize ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8132,tmp=(C_word)a,a+=2,tmp));
t24=C_mutate(&lf[243] /* (set! chicken.compiler.scrutinizer#smash-component-types! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11647,tmp=(C_word)a,a+=2,tmp));
t25=C_mutate((C_word*)lf[170]+1 /* (set! chicken.compiler.scrutinizer#match-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11843,tmp=(C_word)a,a+=2,tmp));
t26=C_mutate(&lf[159] /* (set! chicken.compiler.scrutinizer#match-argument-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13431,tmp=(C_word)a,a+=2,tmp));
t27=C_mutate(&lf[49] /* (set! chicken.compiler.scrutinizer#simplify-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13503,tmp=(C_word)a,a+=2,tmp));
t28=C_mutate(&lf[380] /* (set! chicken.compiler.scrutinizer#merge-argument-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14472,tmp=(C_word)a,a+=2,tmp));
t29=C_mutate(&lf[300] /* (set! chicken.compiler.scrutinizer#compatible-types? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14710,tmp=(C_word)a,a+=2,tmp));
t30=C_mutate((C_word*)lf[382]+1 /* (set! chicken.compiler.scrutinizer#type<=? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14769,tmp=(C_word)a,a+=2,tmp));
t31=C_mutate((C_word*)lf[273]+1 /* (set! chicken.compiler.scrutinizer#refine-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14797,tmp=(C_word)a,a+=2,tmp));
t32=C_mutate(&lf[121] /* (set! chicken.compiler.scrutinizer#procedure-type? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15119,tmp=(C_word)a,a+=2,tmp));
t33=C_mutate(&lf[166] /* (set! chicken.compiler.scrutinizer#procedure-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15171,tmp=(C_word)a,a+=2,tmp));
t34=C_mutate(&lf[371] /* (set! chicken.compiler.scrutinizer#procedure-arguments ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15222,tmp=(C_word)a,a+=2,tmp));
t35=C_mutate(&lf[370] /* (set! chicken.compiler.scrutinizer#procedure-results ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15279,tmp=(C_word)a,a+=2,tmp));
t36=C_mutate(&lf[119] /* (set! chicken.compiler.scrutinizer#procedure-argument-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15336,tmp=(C_word)a,a+=2,tmp));
t37=C_mutate(&lf[388] /* (set! chicken.compiler.scrutinizer#named? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15687,tmp=(C_word)a,a+=2,tmp));
t38=C_mutate(&lf[366] /* (set! chicken.compiler.scrutinizer#rest-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15740,tmp=(C_word)a,a+=2,tmp));
t39=C_mutate(&lf[168] /* (set! chicken.compiler.scrutinizer#noreturn-procedure-type? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15760,tmp=(C_word)a,a+=2,tmp));
t40=C_mutate(&lf[402] /* (set! chicken.compiler.scrutinizer#noreturn-type? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15819,tmp=(C_word)a,a+=2,tmp));
t41=C_mutate(&lf[386] /* (set! chicken.compiler.scrutinizer#refinement-type? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15863,tmp=(C_word)a,a+=2,tmp));
t42=C_mutate(&lf[161] /* (set! chicken.compiler.scrutinizer#type-typeenv ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15910,tmp=(C_word)a,a+=2,tmp));
t43=C_mutate(&lf[158] /* (set! chicken.compiler.scrutinizer#trail-restore ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16242,tmp=(C_word)a,a+=2,tmp));
t44=C_mutate(&lf[118] /* (set! chicken.compiler.scrutinizer#resolve ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16285,tmp=(C_word)a,a+=2,tmp));
t45=C_mutate((C_word*)lf[406]+1 /* (set! chicken.compiler.scrutinizer#load-type-database ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16750,tmp=(C_word)a,a+=2,tmp));
t46=C_mutate((C_word*)lf[432]+1 /* (set! chicken.compiler.scrutinizer#emit-types-file ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17149,tmp=(C_word)a,a+=2,tmp));
t47=lf[447] /* chicken.compiler.scrutinizer#node-mutations */ =C_SCHEME_END_OF_LIST;;
t48=C_mutate(&lf[160] /* (set! chicken.compiler.scrutinizer#mutate-node! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17320,tmp=(C_word)a,a+=2,tmp));
t49=C_mutate(&lf[451] /* (set! chicken.compiler.scrutinizer#source-node ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17338,tmp=(C_word)a,a+=2,tmp));
t50=C_mutate(&lf[453] /* (set! chicken.compiler.scrutinizer#source-node-tree ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17365,tmp=(C_word)a,a+=2,tmp));
t51=C_mutate(&lf[125] /* (set! chicken.compiler.scrutinizer#specialize-node! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17500,tmp=(C_word)a,a+=2,tmp));
t52=C_mutate((C_word*)lf[407]+1 /* (set! chicken.compiler.scrutinizer#validate-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17653,tmp=(C_word)a,a+=2,tmp));
t53=C_mutate((C_word*)lf[479]+1 /* (set! chicken.compiler.scrutinizer#check-and-validate-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19020,tmp=(C_word)a,a+=2,tmp));
t54=C_mutate((C_word*)lf[408]+1 /* (set! chicken.compiler.scrutinizer#install-specializations ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19054,tmp=(C_word)a,a+=2,tmp));
t55=C_mutate(&lf[387] /* (set! chicken.compiler.scrutinizer#canonicalize-list-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19333,tmp=(C_word)a,a+=2,tmp));
t56=C_mutate(&lf[176] /* (set! chicken.compiler.scrutinizer#strip-namespace ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19447,tmp=(C_word)a,a+=2,tmp));
t57=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19495,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t58=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_23333,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:2076: ##sys#put! */
t59=*((C_word*)lf[640]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t59;
av2[1]=t57;
av2[2]=lf[658];
av2[3]=lf[274];
av2[4]=t58;
((C_proc)(void*)(*((C_word*)t59+1)))(5,av2);}}

/* loop in a11994 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_6820(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_6820,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6834,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:49: scheme#reverse */
t5=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(lf[365],t4);
if(C_truep(C_i_not(t5))){
t6=C_u_i_cdr(t2);
t7=C_u_i_car(t2);
t8=C_a_i_cons(&a,2,t7,t3);
/* mini-srfi-1.scm:50: loop */
t10=t1;
t11=t6;
t12=t8;
t1=t10;
t2=t11;
t3=t12;
goto loop;}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6858,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:51: scheme#reverse */
t7=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}

/* k6832 in loop in a11994 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6834,c,av);}
/* mini-srfi-1.scm:49: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* k6856 in loop in a11994 in match-rest in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6858,c,av);}
/* mini-srfi-1.scm:51: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* chicken.compiler.scrutinizer#take in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_6864(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_6864,3,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6882,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:56: take */
t7=t5;
t8=C_u_i_cdr(t2);
t9=C_fixnum_difference(t3,C_fix(1));
t1=t7;
t2=t8;
t3=t9;
goto loop;}}

/* k6880 in chicken.compiler.scrutinizer#take in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6882,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in a19709 in k19919 in k19923 */
static void C_fcall f_6922(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_6922,5,t0,t1,t2,t3,t4);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6936,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:67: scheme#reverse */
t6=*((C_word*)lf[367]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_fixnum_difference(t2,C_fix(1));
t6=C_i_car(t4);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:68: loop */
t9=t1;
t10=t5;
t11=t7;
t12=C_u_i_cdr(t4);
t1=t9;
t2=t10;
t3=t11;
t4=t12;
goto loop;}}

/* k6934 in loop in a19709 in k19919 in k19923 */
static void C_ccall f_6936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6936,c,av);}
/* mini-srfi-1.scm:67: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_6955(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_6955,4,t1,t2,t3,t4);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t4))){
t5=C_i_check_list_2(t3,lf[4]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6970,a[2]=t2,a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_6970(t9,t1,t3);}
else{
t5=C_a_i_cons(&a,2,t3,t4);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7010,a[2]=t7,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_7010(t9,t1,t5);}}

/* foldr184 in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_6970(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_6970,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6978,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6997,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g189 in foldr184 in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_6978(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_6978,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6986,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:72: proc */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k6984 in g189 in foldr184 in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6986,c,av);}
/* mini-srfi-1.scm:72: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k6995 in foldr184 in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_6997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6997,c,av);}
/* mini-srfi-1.scm:72: g189 */
t2=((C_word*)t0)[2];
f_6978(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7010(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7010,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7017,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* mini-srfi-1.scm:74: any */
f_7159(t3,*((C_word*)lf[8]+1),t2);}

/* k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_7017,c,av);}
a=C_alloc(21);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7024,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[4],lf[6]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7089,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7091,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_7091(t12,t8,((C_word*)t0)[4]);}}

/* k7022 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_7024,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7028,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7040,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7042,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_7042(t11,t7,((C_word*)t0)[4]);}

/* k7026 in k7022 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7028(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7028,c,av);}
/* mini-srfi-1.scm:76: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k7038 in k7022 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7040(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7040,c,av);}
/* mini-srfi-1.scm:77: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7010(t2,((C_word*)t0)[3],t1);}

/* map-loop226 in k7022 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7042(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_7042,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cdr(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7087 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7089,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop199 in k7015 in loop in chicken.compiler.scrutinizer#append-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7091(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_7091,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#every in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7125(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7125,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7131,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_7131(t7,t1,t3);}

/* loop in chicken.compiler.scrutinizer#every in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7131(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7131,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7153,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:82: pred */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k7151 in loop in chicken.compiler.scrutinizer#every in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7153,c,av);}
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:83: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_7131(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.compiler.scrutinizer#any in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7159(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7159,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7165,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_7165(t7,t1,t3);}

/* loop in chicken.compiler.scrutinizer#any in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7165(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7165,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7175,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:88: pred */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k7173 in loop in chicken.compiler.scrutinizer#any in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7175,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:89: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_7165(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* loop in k17210 in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_7195(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_7195,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7209,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:95: loop */
t6=t4;
t7=C_i_car(t3);
t8=C_u_i_cdr(t3);
t1=t6;
t2=t7;
t3=t8;
goto loop;}}

/* k7207 in loop in k17210 in k17196 in k17193 in k17190 in k17187 in k17184 in k17181 in k17175 in k17172 in a17166 in k17157 in a17154 in chicken.compiler.scrutinizer#emit-types-file in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_7209(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7209,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k20373 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_7223(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_7223,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7241,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:101: loop */
t6=t4;
t7=C_u_i_cdr(t2);
t1=t6;
t2=t7;
goto loop;}}

/* k7239 in loop in k20373 in collect in chicken.compiler.scrutinizer#over-all-instantiations in k20216 in k20121 in k20118 in k19953 in k19950 in k19947 in k19944 in k19941 in k19938 in k19935 in k19655 in k19652 in k19649 in k19493 in k6752 in k6749 in k6746 in ... */
static void C_ccall f_7241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7241,c,av);}
/* mini-srfi-1.scm:101: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* loop in loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7251(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7251,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7264,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:106: test */
t4=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k7262 in loop in loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7264,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
/* mini-srfi-1.scm:107: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7251(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=C_u_i_car(((C_word*)t0)[4]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7278,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:109: loop */
t4=((C_word*)((C_word*)t0)[2])[1];
f_7251(t4,t3,C_u_i_cdr(((C_word*)t0)[4]));}}

/* k7276 in k7262 in loop in loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7278,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7320(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7320,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7326,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_7326(t7,t1,t2);}

/* loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7326(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,3)))){
C_save_and_reclaim_args((void *)trf_7326,3,t0,t1,t2);}
a=C_alloc(17);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7340,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7353,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7251,a[2]=t8,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_7251(t10,t6,t4);}}

/* k7338 in loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7340,c,av);}
a=C_alloc(3);
t2=C_i_equalp(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?((C_word*)t0)[4]:C_a_i_cons(&a,2,((C_word*)t0)[5],t1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7351 in loop in chicken.compiler.scrutinizer#delete-duplicates in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7353,c,av);}
/* mini-srfi-1.scm:123: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7326(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7409(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7409,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_check_list_2(t3,lf[4]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7418,a[2]=t2,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_7418(t8,t1,t3);}

/* foldr349 in chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7418(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_7418,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7426,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7452,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g354 in foldr349 in chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7426(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_7426,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7430,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:135: pred */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7428 in g354 in foldr349 in chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_7430,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7434,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:135: g364 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* mini-srfi-1.scm:135: g364 */
  f_7434(C_a_i(&a,3),t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g364 in k7428 in g354 in foldr349 in chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_7434(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_a_i_cons(&a,2,t1,((C_word*)t0)[2]));}

/* k7450 in foldr349 in chicken.compiler.scrutinizer#filter-map in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7452,c,av);}
/* mini-srfi-1.scm:134: g354 */
t2=((C_word*)t0)[2];
f_7426(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.compiler.scrutinizer#unzip1 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_7474,c,av);}
a=C_alloc(12);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(t2,lf[6]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7491,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_7491(t11,t1,t2);}

/* map-loop377 in chicken.compiler.scrutinizer#unzip1 in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7491(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_7491,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* loop in g2692 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7590(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7590,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_memq(t4,t3))){
/* mini-srfi-1.scm:160: loop */
t9=t1;
t10=C_u_i_cdr(t2);
t11=t3;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:161: loop */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* foldl459 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7676(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_7676,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7743,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_slot(t2,C_fix(0));
t7=C_i_check_list_2(t6,lf[378]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7701,a[2]=t9,tmp=(C_word)a,a+=3,tmp));
t11=((C_word*)t9)[1];
f_7701(t11,t5,t6,t3);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* foldl475 in foldl459 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7701(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_7701,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=C_slot(t2,C_fix(0));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7724,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,a[5]=t3,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7729,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:175: any */
f_7159(t6,t7,t3);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7722 in foldl475 in foldl459 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7724,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
f_7701(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[6],((C_word*)t0)[5]);
t3=((C_word*)((C_word*)t0)[2])[1];
f_7701(t3,((C_word*)t0)[3],((C_word*)t0)[4],t2);}}

/* a7728 in foldl475 in foldl459 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7729(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7729,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7741 in foldl459 in k13980 in a13511 in simplify in chicken.compiler.scrutinizer#simplify-type in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7743,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7676(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.compiler.scrutinizer#list-tabulate in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7767(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_7767,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7773,a[2]=t2,a[3]=t5,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_7773(t7,t1,C_fix(0));}

/* loop in chicken.compiler.scrutinizer#list-tabulate in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7773(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7773,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7787,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:190: proc */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k7785 in loop in chicken.compiler.scrutinizer#list-tabulate in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7787(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7787,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7791,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:190: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_7773(t3,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k7789 in k7785 in loop in chicken.compiler.scrutinizer#list-tabulate in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_7791,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a7802 in k12812 in k12692 in k12452 in k12438 in match1 in k11845 in chicken.compiler.scrutinizer#match-types in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7803,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#lset=/eq? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7809(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7809,3,t1,t2,t3);}
a=C_alloc(3);
t4=C_i_length(t2);
t5=C_i_length(t3);
t6=C_eqp(t4,t5);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7821,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:197: every */
f_7125(t1,t7,t2);}
else{
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* a7820 in chicken.compiler.scrutinizer#lset=/eq? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7821,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in blist-type in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_7886(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t1))){
return(C_SCHEME_FALSE);}
else{
t2=(
/* mini-srfi-1.scm:216: pred */
  f_8473(((C_word*)t0)[2],C_i_car(t1))
);
if(C_truep(t2)){
return(C_u_i_car(t1));}
else{
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}}}

/* a7946 in k10669 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7947,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#make-list in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_7950(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7950,3,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t3))){
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f25872,tmp=(C_word)a,a+=2,tmp);
/* mini-srfi-1.scm:228: list-tabulate */
f_7767(t1,t2,t4);}
else{
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7959,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:228: list-tabulate */
f_7767(t1,t2,t5);}}

/* a7958 in chicken.compiler.scrutinizer#make-list in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_7959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7959,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#walked-result in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8079,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8087,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[26]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8085 in chicken.compiler.scrutinizer#walked-result in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8087,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.scrutinizer#type-always-immediate? in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
av[0]=t0;
av[1]=t1;
av[2]=t2;
C_save_and_reclaim((void *)f_8089,c,av);}
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[28]);
if(C_truep(t4)){
/* scrutinizer.scm:171: every */
f_7125(t1,lf[27],C_u_i_cdr(t2));}
else{
t5=C_eqp(t3,lf[29]);
if(C_truep(t5)){
/* scrutinizer.scm:172: type-always-immediate? */
t7=t1;
t8=C_i_caddr(t2);
t1=t7;
t2=t8;
c=3;
goto loop;}
else{
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep((C_truep(C_eqp(t2,lf[30]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[31]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[32]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[33]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[34]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[35]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))))?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(106,c,8)))){
C_save_and_reclaim((void *)f_8132,c,av);}
a=C_alloc(106);
t8=C_mutate(&lf[18] /* (set! chicken.compiler.scrutinizer#*complain?* ...) */,t4);
t9=C_SCHEME_END_OF_LIST;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_END_OF_LIST;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_FALSE;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_fix(0);
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_fix(0);
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_SCHEME_FALSE;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_fix(0);
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_SCHEME_UNDEFINED;
t24=(*a=C_VECTOR_TYPE|1,a[1]=t23,tmp=(C_word)a,a+=2,tmp);
t25=C_SCHEME_UNDEFINED;
t26=(*a=C_VECTOR_TYPE|1,a[1]=t25,tmp=(C_word)a,a+=2,tmp);
t27=C_SCHEME_UNDEFINED;
t28=(*a=C_VECTOR_TYPE|1,a[1]=t27,tmp=(C_word)a,a+=2,tmp);
t29=C_SCHEME_UNDEFINED;
t30=(*a=C_VECTOR_TYPE|1,a[1]=t29,tmp=(C_word)a,a+=2,tmp);
t31=C_SCHEME_UNDEFINED;
t32=(*a=C_VECTOR_TYPE|1,a[1]=t31,tmp=(C_word)a,a+=2,tmp);
t33=C_SCHEME_UNDEFINED;
t34=(*a=C_VECTOR_TYPE|1,a[1]=t33,tmp=(C_word)a,a+=2,tmp);
t35=C_SCHEME_UNDEFINED;
t36=(*a=C_VECTOR_TYPE|1,a[1]=t35,tmp=(C_word)a,a+=2,tmp);
t37=C_SCHEME_UNDEFINED;
t38=(*a=C_VECTOR_TYPE|1,a[1]=t37,tmp=(C_word)a,a+=2,tmp);
t39=C_SCHEME_UNDEFINED;
t40=(*a=C_VECTOR_TYPE|1,a[1]=t39,tmp=(C_word)a,a+=2,tmp);
t41=C_SCHEME_UNDEFINED;
t42=(*a=C_VECTOR_TYPE|1,a[1]=t41,tmp=(C_word)a,a+=2,tmp);
t43=C_SCHEME_UNDEFINED;
t44=(*a=C_VECTOR_TYPE|1,a[1]=t43,tmp=(C_word)a,a+=2,tmp);
t45=C_set_block_item(t24,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8169,a[2]=t24,tmp=(C_word)a,a+=3,tmp));
t46=C_set_block_item(t26,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8461,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t47=C_set_block_item(t28,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8492,a[2]=t6,a[3]=t3,a[4]=t26,tmp=(C_word)a,a+=5,tmp));
t48=C_set_block_item(t30,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8555,a[2]=t30,tmp=(C_word)a,a+=3,tmp));
t49=C_set_block_item(t32,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8617,a[2]=t30,tmp=(C_word)a,a+=3,tmp));
t50=C_set_block_item(t34,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8641,tmp=(C_word)a,a+=2,tmp));
t51=C_set_block_item(t36,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8707,a[2]=t22,a[3]=t5,a[4]=t14,tmp=(C_word)a,a+=5,tmp));
t52=C_fix(0);
t53=(*a=C_VECTOR_TYPE|1,a[1]=t52,tmp=(C_word)a,a+=2,tmp);
t54=C_set_block_item(t38,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9341,a[2]=t53,tmp=(C_word)a,a+=3,tmp));
t55=C_set_block_item(t40,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9346,a[2]=t10,a[3]=t12,tmp=(C_word)a,a+=4,tmp));
t56=C_set_block_item(t42,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9379,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t57=C_set_block_item(t44,0,(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_9475,a[2]=t24,a[3]=t28,a[4]=t38,a[5]=t14,a[6]=t5,a[7]=t16,a[8]=t44,a[9]=t40,a[10]=t10,a[11]=t32,a[12]=t34,a[13]=t12,a[14]=t3,a[15]=t42,a[16]=t18,a[17]=t6,a[18]=t7,a[19]=t20,a[20]=t36,a[21]=t26,tmp=(C_word)a,a+=22,tmp));
t58=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11529,a[2]=t1,a[3]=t20,a[4]=t18,a[5]=t16,a[6]=t22,tmp=(C_word)a,a+=7,tmp);
t59=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11645,a[2]=t38,a[3]=t44,a[4]=t58,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:801: chicken.compiler.support#node-subexpressions */
t60=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t60;
av2[1]=t59;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t60+1)))(3,av2);}}

/* constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8169(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8169,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_stringp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[37];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8182,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:197: chicken.keyword#keyword? */
t4=*((C_word*)lf[56]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8182,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[38];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[39];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8194,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:200: chicken.compiler.support#big-fixnum? */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8194,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8197,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_8197(2,av2);}}
else{
/* scrutinizer.scm:200: chicken.compiler.support#small-bignum? */
t3=*((C_word*)lf[54]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_8197,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[40];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_fixnump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[32];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_bignump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[41];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[42];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_ratnump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[43];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_cplxnump(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[44];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_booleanp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(((C_word*)t0)[3])?lf[45]:lf[46]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[31];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_listp(((C_word*)t0)[3]))){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8258,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8260,a[2]=t4,a[3]=t8,a[4]=((C_word*)t0)[4],a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t10=((C_word*)t8)[1];
f_8260(t10,t6,((C_word*)t0)[3]);}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8309,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:213: constant-result */
t3=((C_word*)((C_word*)t0)[4])[1];
f_8169(t3,t2,C_u_i_car(((C_word*)t0)[3]));}
else{
if(C_truep(C_eofp(((C_word*)t0)[3]))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[30];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_vectorp(((C_word*)t0)[3]))){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8343,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:217: scheme#vector->list */
t7=*((C_word*)lf[51]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t2=C_immp(((C_word*)t0)[3]);
t3=C_i_not(t2);
t4=(C_truep(t3)?C_structurep(((C_word*)t0)[3]):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_slot(((C_word*)t0)[3],C_fix(0));
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_list(&a,2,lf[52],t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_charp(((C_word*)t0)[3]);
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?lf[33]:lf[53]);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}}}}}}}}}}}

/* k8256 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_8258,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[47],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop822 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8260(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8260,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8285,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:210: g828 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8169(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8283 in map-loop822 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8285,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8260(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8307 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8309,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8313,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:213: constant-result */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8169(t3,t2,C_u_i_cdr(((C_word*)t0)[4]));}

/* k8311 in k8307 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8313(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_8313,c,av);}
a=C_alloc(9);
t2=C_a_i_list(&a,3,lf[48],((C_word*)t0)[2],t1);
/* scrutinizer.scm:212: simplify-type */
f_13503(((C_word*)t0)[3],t2);}

/* k8341 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8343(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_8343,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8346,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8348,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_8348(t6,t2,t1);}

/* k8344 in k8341 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8346,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[50],t1);
/* scrutinizer.scm:216: simplify-type */
f_13503(((C_word*)t0)[2],t2);}

/* map-loop854 in k8341 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8348(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8348,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8373,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:217: g860 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8169(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8371 in map-loop854 in k8341 in k8195 in k8192 in k8180 in constant-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8373,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8348(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8416 in k8506 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8418,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8422,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:224: g891 */
t3=t2;
f_8422(t3,((C_word*)t0)[5],t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[63];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g891 in k8416 in k8506 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8422(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_8422,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_eqp(t2,lf[59]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8432,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:228: r-deprecated-identifier */
f_22850(t4,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_END_OF_LIST);}
else{
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_eqp(C_u_i_car(t2),lf[59]):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8441,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t7=C_i_cadr(t2);
/* scrutinizer.scm:231: r-deprecated-identifier */
f_22850(t6,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],C_a_i_list(&a,1,t7));}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_list1(&a,1,t2);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* k8430 in g891 in k8416 in k8506 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8432(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8432,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[60];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8439 in g891 in k8416 in k8506 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8441(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8441,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[62];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* blist-type in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8461(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_8461,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8473,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=((C_word*)((C_word*)t0)[2])[1];
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7886,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t7=(
  f_7886(t6,t5)
);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?C_i_cdr(t7):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* a8472 in blist-type in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_8473(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;{}
t2=C_i_caar(t1);
t3=C_eqp(((C_word*)t0)[2],t2);
if(C_truep(t3)){
t4=C_u_i_car(t1);
return(C_i_memq(C_u_i_cdr(t4),((C_word*)t0)[3]));}
else{
return(C_SCHEME_FALSE);}}

/* variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8492(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_8492,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(9);
t7=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8496,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:245: blist-type */
t8=((C_word*)((C_word*)t0)[4])[1];
f_8461(t8,t7,t2,t6);}

/* k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_8496,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list1(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8508,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_not(((C_word*)t0)[7]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8546,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:247: chicken.compiler.support#db-get */
t4=*((C_word*)lf[67]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[68];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=t2;
f_8508(t3,C_SCHEME_FALSE);}}}

/* k8506 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8508(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_8508,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=lf[57];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_assq(((C_word*)t0)[3],((C_word*)t0)[4]);
if(C_truep(t2)){
t3=C_i_cdr(t2);
t4=C_eqp(lf[35],t3);
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[58];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_u_i_cdr(t2);
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_list1(&a,1,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8418,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:224: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}}

/* k8544 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8546,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8553,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:248: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[66];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_8508(t2,C_SCHEME_FALSE);}}

/* k8551 in k8544 in k8494 in variable-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8553,c,av);}
t2=((C_word*)t0)[2];
f_8508(t2,C_i_not(t1));}

/* always-true1 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
av[0]=t0;
av[1]=t1;
av[2]=t2;
C_save_and_reclaim((void *)f_8555,c,av);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[28]);
if(C_truep(t4)){
/* scrutinizer.scm:264: every */
f_7125(t1,((C_word*)((C_word*)t0)[2])[1],C_u_i_cdr(t2));}
else{
t5=C_eqp(t3,lf[69]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8587,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:265: always-true1 */
t8=t6;
t9=C_i_cadr(t2);
t1=t8;
t2=t9;
c=3;
goto loop;}
else{
t6=C_eqp(t3,lf[29]);
if(C_truep(t6)){
/* scrutinizer.scm:266: always-true1 */
t8=t1;
t9=C_i_caddr(t2);
t1=t8;
t2=t9;
c=3;
goto loop;}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep((C_truep(C_eqp(t2,lf[53]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[34]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[46]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[35]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[70]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))))?C_SCHEME_FALSE:C_SCHEME_TRUE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8585 in always-true1 in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8587,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8617(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_8617,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8621,a[2]=t1,a[3]=t3,a[4]=t2,a[5]=t5,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:272: always-true1 */
t7=((C_word*)((C_word*)t0)[2])[1];{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
f_8555(3,av2);}}

/* k8619 in always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,8)))){
C_save_and_reclaim((void *)f_8621,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8627,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list2(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22559,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:2825: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[77];
av2[3]=lf[78];
av2[4]=lf[79];
av2[5]=lf[80];
av2[6]=lf[81];
av2[7]=lf[82];
av2[8]=lf[83];
((C_proc)(void*)(*((C_word*)t5+1)))(9,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k8625 in k8619 in always-true in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8627,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8637 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8639(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8639,c,av);}
t2=((C_word*)t0)[2];
f_9642(t2,((C_word*)t0)[3]);}

/* single in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8641(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_8641,3,t1,t2,t3);}
a=C_alloc(4);
t4=C_eqp(lf[53],t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_length(t2);
t6=C_eqp(C_fix(1),t5);
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t5,C_fix(0));
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8669,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:287: r-value-count-mismatch */
t9=t3;{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8672,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:290: r-value-count-mismatch */
t9=t3;{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}}}}

/* k8667 in single in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8669,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[35];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8670 in single in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8672,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8683 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_8685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_8685,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8688,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:297: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[162];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k8686 in k8683 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_8688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8688,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8691,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(C_truep(((C_word*)t0)[3])?((C_word*)t0)[3]:C_SCHEME_END_OF_LIST);
if(C_truep(t1)){
/* scrutinizer.scm:298: scheme#append */
t4=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* scrutinizer.scm:298: scheme#append */
t4=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k8689 in k8686 in k8683 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_8691(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8691,c,av);}
t2=C_i_pairp(t1);
t3=((C_word*)t0)[2];
f_9100(t3,(C_truep(t2)?t1:C_SCHEME_FALSE));}

/* call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8707(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_8707,5,t0,t1,t2,t3,t4);}
a=C_alloc(21);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=lf[25];
t10=C_i_check_list_2(t3,lf[6]);
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8717,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],tmp=(C_word)a,a+=9,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9307,a[2]=t7,a[3]=t13,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t15=((C_word*)t13)[1];
f_9307(t15,t11,t3);}

/* k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_8717,c,av);}
a=C_alloc(11);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8723,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=t1,a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:304: procedure-type? */
t4=lf[121];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
f_15119(3,av2);}}

/* k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_8723,c,av);}
a=C_alloc(16);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_length(t2);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9305,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t3,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:306: make-list */
f_7950(t4,t3,C_a_i_list(&a,1,lf[53]));}

/* k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_8733,c,av);}
a=C_alloc(22);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_8742,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t3,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=t5,a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=((C_word*)t0)[12],tmp=(C_word)a,a+=15,tmp);
if(C_truep(C_i_not(((C_word*)t0)[13]))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9301,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:310: match-types */
t8=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t3)[1];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t7=t6;
f_8742(t7,C_SCHEME_FALSE);}}

/* k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8742(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,6)))){
C_save_and_reclaim_args((void *)trf_8742,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8745,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8752,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:312: resolve */
t4=lf[118];{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)((C_word*)t0)[7])[1];
f_16285(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8763,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8769,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[12],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[5],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[14],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:315: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}}

/* k8743 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8745,c,av);}
/* scrutinizer.scm:313: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[53];
av2[3]=C_SCHEME_FALSE;
C_values(4,av2);}}

/* k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_8752,c,av);}
a=C_alloc(7);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8758,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:312: resolve */
t4=lf[118];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)((C_word*)t0)[7])[1];
f_16285(4,av2);}}

/* k8756 in k8750 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_8758,c,av);}
a=C_alloc(14);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21756,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21801,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=t3,a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:2550: variable-node-name */
t6=((C_word*)t3)[1];
f_21756(t6,t5,((C_word*)t0)[2]);}

/* a8762 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8763,c,av);}
/* scrutinizer.scm:316: procedure-argument-types */
f_15336(t1,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)((C_word*)t0)[4])[1],C_SCHEME_END_OF_LIST);}

/* a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,2)))){
C_save_and_reclaim((void *)f_8769,c,av);}
a=C_alloc(30);
t6=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_8773,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=t3,a[14]=((C_word*)t0)[12],a[15]=t2,tmp=(C_word)a,a+=16,tmp);
if(C_truep(t4)){
t7=t6;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_8773(2,av2);}}
else{
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21862,a[2]=t8,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[11],a[5]=t5,a[6]=t6,a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f28008,a[2]=t9,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t11=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}}

/* k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,6)))){
C_save_and_reclaim((void *)f_8773,c,av);}
a=C_alloc(24);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_8776,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],tmp=(C_word)a,a+=15,tmp);
t3=C_i_cdr(((C_word*)t0)[10]);
t4=C_i_cdr(((C_word*)t0)[9]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9229,a[2]=t6,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_9229(t8,t2,t3,t4,((C_word*)t0)[15],C_fix(1));}

/* k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_8776,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8779,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
t3=(
/* scrutinizer.scm:334: noreturn-procedure-type? */
  f_15760(((C_word*)t0)[8])
);
if(C_truep(t3)){
t4=C_set_block_item(((C_word*)t0)[14],0,C_SCHEME_TRUE);
t5=t2;
f_8779(t5,t4);}
else{
t4=t2;
f_8779(t4,C_SCHEME_UNDEFINED);}}

/* k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_8779(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_8779,2,t0,t1);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8782,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
t3=C_u_i_cdr(((C_word*)t0)[10]);
t4=((C_word*)((C_word*)t0)[4])[1];
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15559,a[2]=((C_word*)t0)[13],a[3]=t3,a[4]=t4,a[5]=t6,tmp=(C_word)a,a+=6,tmp));
/* scrutinizer.scm:1472: loop1 */
t8=((C_word*)t6)[1];
f_15559(t8,t2,((C_word*)t0)[8]);}

/* k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_8782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_8782,c,av);}
a=C_alloc(16);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8785,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:337: procedure-name */
f_15171(t4,((C_word*)t0)[8]);}

/* k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_8785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_8785,c,av);}
a=C_alloc(23);
t2=lf[24];
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8788,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t1)){
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8885,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[4],a[6]=t1,a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=t2,a[12]=((C_word*)t0)[5],tmp=(C_word)a,a+=13,tmp);
t5=C_eqp(C_fix(1),((C_word*)t0)[13]);
if(C_truep(t5)){
/* scrutinizer.scm:341: chicken.compiler.support#variable-mark */
t6=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t1;
av2[3]=lf[165];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_8885(2,av2);}}}
else{
t4=t3;
f_8788(t4,C_SCHEME_UNDEFINED);}}

/* k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_fcall f_8788(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_8788,2,t0,t1);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8791,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8854,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[8])){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[3])[1]))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8875,a[2]=t3,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:401: procedure-type? */
t5=lf[121];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
f_15119(3,av2);}}
else{
t4=t3;
f_8854(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_8854(t4,C_SCHEME_FALSE);}}

/* k8789 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_8791(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_8791,2,t0,t1);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8794,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(lf[53],((C_word*)((C_word*)t0)[4])[1]);
if(C_truep(t3)){
t4=((C_word*)((C_word*)t0)[4])[1];
/* scrutinizer.scm:407: scheme#values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t4;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
C_values(4,av2);}}
else{
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8807,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
t9=((C_word*)((C_word*)t0)[4])[1];
t10=C_i_check_list_2(t9,lf[6]);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8819,a[2]=t6,a[3]=t12,a[4]=t8,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_8819(t14,t2,t9);}}

/* k8792 in k8789 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_8794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8794,c,av);}
/* scrutinizer.scm:407: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
C_values(4,av2);}}

/* g1145 in k8789 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_8807(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_8807,3,t0,t1,t2);}
t3=lf[118];
/* scrutinizer.scm:405: g1162 */
t4=lf[118];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=((C_word*)((C_word*)t0)[2])[1];
f_16285(4,av2);}}

/* map-loop1139 in k8789 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_8819(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8819,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8844,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:405: g1145 */
t4=((C_word*)t0)[4];
f_8807(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8842 in map-loop1139 in k8789 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_8844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8844,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8819(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8852 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_8854(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_8854,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8863,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:403: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];
f_8791(t2,C_SCHEME_UNDEFINED);}}

/* k8861 in k8852 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_8863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_8863,c,av);}
a=C_alloc(29);
t2=C_i_set_car(t1,C_SCHEME_TRUE);
t3=((C_word*)((C_word*)t0)[2])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=((C_word*)t0)[3];
f_8791(t5,t4);}

/* k8873 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_8875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8875,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8882,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:402: chicken.compiler.support#node-class */
t3=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_8854(t2,C_SCHEME_FALSE);}}

/* k8880 in k8873 in k8786 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_8882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8882,c,av);}
t2=((C_word*)t0)[2];
f_8854(t2,C_eqp(lf[120],t1));}

/* k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_8885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_8885,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8888,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_8922,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
/* scrutinizer.scm:340: g1061 */
t4=t3;
f_8922(t4,t2,t1);}
else{
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9077,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[8],a[4]=t2,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[4],tmp=(C_word)a,a+=12,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9198,a[2]=t3,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:364: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k8886 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_8888(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_8888,c,av);}
a=C_alloc(29);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_i_assoc(((C_word*)((C_word*)t0)[2])[1],lf[23]);
if(C_truep(t2)){
t3=C_i_cdr(t2);
t4=C_s_a_i_plus(&a,2,t3,C_fix(1));
t5=((C_word*)t0)[3];
f_8788(t5,C_i_set_cdr(t2,t4));}
else{
t3=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t4=C_a_i_cons(&a,2,t3,lf[23]);
t5=C_mutate(&lf[23] /* (set! chicken.compiler.scrutinizer#specialization-statistics ...) */,t4);
t6=((C_word*)t0)[3];
f_8788(t6,t5);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];
f_8788(t3,t2);}}

/* g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_fcall f_8922(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,4)))){
C_save_and_reclaim_args((void *)trf_8922,3,t0,t1,t2);}
a=C_alloc(17);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8929,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=((C_word*)t0)[5],a[7]=t1,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
t4=C_a_i_list1(&a,1,t2);
/* scrutinizer.scm:343: match-argument-types */
f_13431(t3,t4,C_i_cdr(((C_word*)t0)[8]),((C_word*)((C_word*)t0)[11])[1]);}

/* k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_8929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_8929,c,av);}
a=C_alloc(23);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8932,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t3=C_i_cadr(((C_word*)t0)[10]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22484,a[2]=t5,a[3]=((C_word*)t0)[9],a[4]=t2,a[5]=((C_word*)t0)[11],a[6]=t3,a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f27984,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t8=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8989,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:353: trail-restore */
f_16242(t2,((C_word*)t0)[12],((C_word*)((C_word*)t0)[13])[1]);}}

/* k8930 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_8932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_8932,c,av);}
a=C_alloc(15);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8938,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
t3=C_i_cdr(((C_word*)t0)[8]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8967,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_list(&a,1,lf[127]);
/* scrutinizer.scm:349: ##sys#list->vector */
t6=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k8936 in k8930 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_8938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_8938,c,av);}
a=C_alloc(6);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[122]);
t3=C_a_i_list2(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_mutate(((C_word *)((C_word*)t0)[5])+1,t3);
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k8965 in k8930 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_8967(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_8967,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8971,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_a_i_list(&a,1,C_fix(1));
/* scrutinizer.scm:349: ##sys#list->vector */
t4=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8969 in k8965 in k8930 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_8971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,4)))){
C_save_and_reclaim((void *)f_8971,c,av);}
a=C_alloc(24);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[123],C_SCHEME_TRUE);
t5=C_a_i_list(&a,3,lf[124],t3,t4);
/* scrutinizer.scm:347: specialize-node! */
f_17500(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t5);}

/* k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_ccall f_8989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_8989,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8992,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
t3=C_a_i_list(&a,2,lf[69],((C_word*)t0)[4]);
t4=C_a_i_list1(&a,1,t3);
/* scrutinizer.scm:354: match-argument-types */
f_13431(t2,t4,C_u_i_cdr(((C_word*)t0)[9]),((C_word*)((C_word*)t0)[12])[1]);}

/* k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_ccall f_8992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_8992,c,av);}
a=C_alloc(22);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=C_i_cadr(((C_word*)t0)[9]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22518,a[2]=t5,a[3]=((C_word*)t0)[8],a[4]=t2,a[5]=((C_word*)t0)[10],a[6]=t3,a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f27989,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:2521: chicken.compiler.support#node-subexpressions */
t8=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
/* scrutinizer.scm:363: trail-restore */
f_16242(((C_word*)t0)[6],((C_word*)t0)[11],((C_word*)((C_word*)t0)[12])[1]);}}

/* k8993 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_8995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_8995,c,av);}
a=C_alloc(14);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9001,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t3=C_i_cdr(((C_word*)t0)[7]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9034,a[2]=t2,a[3]=((C_word*)t0)[8],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_list(&a,1,lf[127]);
/* scrutinizer.scm:360: ##sys#list->vector */
t6=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8999 in k8993 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_9001(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_9001,c,av);}
a=C_alloc(12);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[143]);
t3=C_a_i_list(&a,2,lf[69],((C_word*)t0)[3]);
t4=C_a_i_list2(&a,2,((C_word*)t0)[3],t3);
t5=C_mutate(((C_word *)((C_word*)t0)[4])+1,t4);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k9032 in k8993 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_9034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_9034,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9038,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_a_i_list(&a,1,C_fix(1));
/* scrutinizer.scm:360: ##sys#list->vector */
t4=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k9036 in k9032 in k8993 in k8990 in k8987 in k8927 in g1061 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_9038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,4)))){
C_save_and_reclaim((void *)f_9038,c,av);}
a=C_alloc(24);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,2,lf[123],C_SCHEME_FALSE);
t5=C_a_i_list(&a,3,lf[124],t3,t4);
/* scrutinizer.scm:358: specialize-node! */
f_17500(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t5);}

/* k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_9077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_9077,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9081,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:340: g1092 */
t3=t2;
f_9081(t3,((C_word*)t0)[4],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9100,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[4],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[11])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8685,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:296: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[163];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;
f_9100(t3,C_SCHEME_FALSE);}}}

/* g1092 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_9081(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_9081,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[53]);
t4=C_i_car(t2);
t5=C_a_i_list2(&a,2,lf[123],t4);
/* scrutinizer.scm:370: mutate-node! */
f_17320(t1,((C_word*)t0)[3],t5);}

/* k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_fcall f_9100(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_9100,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9104,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:340: g1098 */
t3=t2;
f_9104(t3,((C_word*)t0)[10],t1);}
else{
t2=((C_word*)t0)[10];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_8888(2,av2);}}}

/* g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in ... */
static void C_fcall f_9104(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_9104,3,t0,t1,t2);}
a=C_alloc(13);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9110,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t4,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp));
t6=((C_word*)t4)[1];
f_9110(t6,t1,t2);}

/* loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in ... */
static void C_fcall f_9110(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,4)))){
C_save_and_reclaim_args((void *)trf_9110,3,t0,t1,t2);}
a=C_alloc(19);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_9124,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t1,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=t2,a[12]=((C_word*)t0)[9],a[13]=t4,a[14]=((C_word*)t0)[10],tmp=(C_word)a,a+=15,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9188,a[2]=t5,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:378: append-map */
f_6955(t6,lf[161],t4,C_SCHEME_END_OF_LIST);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9122 in loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_9124(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_9124,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_9130,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:380: match-argument-types */
f_13431(t2,((C_word*)t0)[13],C_i_cdr(((C_word*)t0)[14]),t1);}

/* k9128 in k9122 in loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in ... */
static void C_ccall f_9130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_9130,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
t4=C_mutate(((C_word *)((C_word*)t0)[4])+1,t3);
t5=C_mutate(((C_word *)((C_word*)t0)[5])+1,((C_word*)t0)[6]);
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_i_pairp(t6);
t8=(C_truep(t7)?C_i_cadr(((C_word*)t0)[2]):C_SCHEME_FALSE);
t9=(C_truep(t8)?C_i_caddr(((C_word*)t0)[2]):C_i_cadr(((C_word*)t0)[2]));
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9144,a[2]=t8,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:388: specialize-node! */
f_17500(t10,((C_word*)t0)[9],C_i_cdr(((C_word*)t0)[10]),t9);}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9175,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[12],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:391: trail-restore */
f_16242(t2,((C_word*)t0)[13],((C_word*)t0)[6]);}}

/* k9142 in k9128 in k9122 in loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_9144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9144,c,av);}
if(C_truep(((C_word*)t0)[2])){
t2=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)t0)[2]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9173 in k9128 in k9122 in loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in ... */
static void C_ccall f_9175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9175,c,av);}
/* scrutinizer.scm:392: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9110(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k9186 in loop in g1098 in k9098 in k9075 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in ... */
static void C_ccall f_9188(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9188,c,av);}
/* scrutinizer.scm:377: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k9196 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in ... */
static void C_ccall f_9198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,5)))){
C_save_and_reclaim((void *)f_9198,c,av);}
a=C_alloc(2);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9200,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:364: chicken.compiler.support#maybe-constant-fold-call */
t3=*((C_word*)lf[164]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a9199 in k9196 in k8883 in k8783 in k8780 in k8777 in k8774 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in ... */
static void C_ccall f_9200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9200,c,av);}
a=C_alloc(3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t2)?C_a_i_cons(&a,2,t3,t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* doloop1040 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9229(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,4)))){
C_save_and_reclaim_args((void *)trf_9229,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(19);
t6=C_i_nullp(t2);
t7=(C_truep(t6)?t6:C_i_nullp(t4));
if(C_truep(t7)){
t8=C_SCHEME_UNDEFINED;
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9242,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t5,a[6]=((C_word*)t0)[2],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t9=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9258,a[2]=t8,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t5,a[7]=((C_word*)t0)[5],a[8]=t2,a[9]=((C_word*)t0)[6],a[10]=t4,tmp=(C_word)a,a+=11,tmp);
/* scrutinizer.scm:324: match-types */
t10=*((C_word*)lf[170]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t9;
av2[2]=C_i_car(t4);
av2[3]=C_i_car(t2);
av2[4]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}}

/* k9240 in doloop1040 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,5)))){
C_save_and_reclaim((void *)f_9242,c,av);}
a=C_alloc(29);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_cdr(((C_word*)t0)[3]);
t4=C_u_i_cdr(((C_word*)t0)[4]);
t5=C_s_a_i_plus(&a,2,((C_word*)t0)[5],C_fix(1));
t6=((C_word*)((C_word*)t0)[6])[1];
f_9229(t6,((C_word*)t0)[7],t2,t3,t4,t5);}

/* k9256 in doloop1040 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_9258,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_9242(2,av2);}}
else{
t2=C_i_car(((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9269,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t2,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:331: resolve */
t4=lf[118];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(((C_word*)t0)[10]);
av2[3]=((C_word*)((C_word*)t0)[9])[1];
f_16285(4,av2);}}}

/* k9267 in k9256 in doloop1040 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9269,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9273,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:332: resolve */
t3=lf[118];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[8]);
av2[3]=((C_word*)((C_word*)t0)[9])[1];
f_16285(4,av2);}}

/* k9271 in k9267 in k9256 in doloop1040 in k8771 in a8768 in k8740 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_9273(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_9273,c,av);}
/* scrutinizer.scm:328: r-proc-call-argument-type-mismatch */
f_21906(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t1,((C_word*)t0)[8]);}

/* k9299 in k8731 in k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9301(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9301,c,av);}
t2=((C_word*)t0)[2];
f_8742(t2,C_i_not(t1));}

/* k9303 in k8721 in k8715 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_9305,c,av);}
a=C_alloc(23);
t2=C_a_i_list(&a,3,lf[84],t1,lf[53]);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8733,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t2,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* scrutinizer.scm:307: append-map */
f_6955(t3,lf[161],((C_word*)t0)[10],C_SCHEME_END_OF_LIST);}

/* map-loop984 in call-result in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9307(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9307,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f26391,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:166: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* tag in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static C_word C_fcall f_9341(C_word *a,C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
t1=((C_word*)((C_word*)t0)[2])[1];
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t1,C_fix(1)));
return(((C_word*)((C_word*)t0)[2])[1]);}

/* add-to-blist in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9346(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_9346,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9352,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t6,a[5]=t3,a[6]=t4,tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_9352(t8,t1,t2);}

/* loop in add-to-blist in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9352(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_9352,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9357,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
t4=C_a_i_cons(&a,2,t2,((C_word*)t0)[5]);
/* scrutinizer.scm:417: chicken.base#alist-update! */
t5=*((C_word*)lf[186]+1);{
C_word av2[6];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)((C_word*)t0)[2])[1];
av2[5]=*((C_word*)lf[187]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}

/* k9355 in loop in add-to-blist in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9357,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_i_assq(((C_word*)t0)[3],((C_word*)((C_word*)t0)[4])[1]);
if(C_truep(t3)){
/* scrutinizer.scm:421: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_9352(t4,((C_word*)t0)[6],C_i_cdr(t3));}
else{
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9379(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9379,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9386,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t2)){
if(C_truep(((C_word*)t0)[2])){
/* scrutinizer.scm:425: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[64]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[66];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_9386(2,av2);}}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_9386(2,av2);}}}

/* k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9386,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9389,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* scrutinizer.scm:426: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* scrutinizer.scm:432: make-list */
f_7950(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,1,lf[53]));}}

/* k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9389,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9392,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:427: type-typeenv */
t3=lf[161];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
f_15910(3,av2);}}

/* k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_9392,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9398,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:428: procedure-type? */
t3=lf[121];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
f_15119(3,av2);}}

/* k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_9398,c,av);}
a=C_alloc(20);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9402,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9409,a[2]=t4,a[3]=t6,a[4]=t5,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9451,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t9=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_9457,tmp=(C_word)a,a+=2,tmp);
/* scrutinizer.scm:430: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t7;
av2[2]=t8;
av2[3]=t9;
C_call_with_values(4,av2);}}
else{
/* scrutinizer.scm:431: make-list */
f_7950(((C_word*)t0)[3],((C_word*)t0)[5],C_a_i_list(&a,1,lf[53]));}}

/* g1205 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9402(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_9402,3,t0,t1,t2);}
t3=lf[118];
/* scrutinizer.scm:429: g1222 */
t4=lf[118];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
f_16285(4,av2);}}

/* k9407 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_9409,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[6]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9417,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_9417(t6,((C_word*)t0)[5],t1);}

/* map-loop1199 in k9407 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9417(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9417,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9442,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:429: g1205 */
t4=((C_word*)t0)[4];
f_9402(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9440 in map-loop1199 in k9407 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9442,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9417(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a9450 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_9451,c,av);}
a=C_alloc(3);
/* scrutinizer.scm:430: procedure-argument-types */
f_15336(t1,((C_word*)t0)[2],((C_word*)t0)[3],C_SCHEME_END_OF_LIST,C_a_i_list(&a,1,C_SCHEME_TRUE));}

/* a9456 in k9396 in k9390 in k9387 in k9384 in initial-argument-types in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9457(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_9457,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_list_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9475(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,2)))){
C_save_and_reclaim_args((void *)trf_9475,8,t0,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(29);
t8=(*a=C_CLOSURE_TYPE|28,a[1]=(C_word)f_9479,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=t4,a[7]=t2,a[8]=t6,a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[5],a[11]=((C_word*)t0)[6],a[12]=((C_word*)t0)[7],a[13]=((C_word*)t0)[8],a[14]=t5,a[15]=t7,a[16]=((C_word*)t0)[9],a[17]=((C_word*)t0)[10],a[18]=((C_word*)t0)[11],a[19]=((C_word*)t0)[12],a[20]=((C_word*)t0)[13],a[21]=((C_word*)t0)[14],a[22]=((C_word*)t0)[15],a[23]=((C_word*)t0)[16],a[24]=((C_word*)t0)[17],a[25]=((C_word*)t0)[18],a[26]=((C_word*)t0)[19],a[27]=((C_word*)t0)[20],a[28]=((C_word*)t0)[21],tmp=(C_word)a,a+=29,tmp);
/* scrutinizer.scm:435: chicken.compiler.support#node-subexpressions */
t9=*((C_word*)lf[86]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,2)))){
C_save_and_reclaim((void *)f_9479,c,av);}
a=C_alloc(30);
t2=(*a=C_CLOSURE_TYPE|29,a[1]=(C_word)f_9482,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],a[27]=((C_word*)t0)[26],a[28]=((C_word*)t0)[27],a[29]=((C_word*)t0)[28],tmp=(C_word)a,a+=30,tmp);
/* scrutinizer.scm:436: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[26]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9482(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(31,c,2)))){
C_save_and_reclaim((void *)f_9482,c,av);}
a=C_alloc(31);
t2=(*a=C_CLOSURE_TYPE|30,a[1]=(C_word)f_9485,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],a[27]=((C_word*)t0)[26],a[28]=((C_word*)t0)[27],a[29]=((C_word*)t0)[28],a[30]=((C_word*)t0)[29],tmp=(C_word)a,a+=31,tmp);
/* scrutinizer.scm:437: chicken.compiler.support#node-class */
t3=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(121,c,7)))){
C_save_and_reclaim((void *)f_9485,c,av);}
a=C_alloc(121);
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_plus(&a,2,lf[16],C_fix(1)));
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9493,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t5=C_eqp(t1,lf[85]);
if(C_truep(t5)){
t6=C_i_car(((C_word*)t0)[3]);
t7=C_a_i_list1(&a,1,t6);
t8=lf[16];
t9=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t6=C_eqp(t1,lf[123]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9521,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:446: constant-result */
t8=((C_word*)((C_word*)t0)[4])[1];
f_8169(t8,t7,C_i_car(((C_word*)t0)[3]));}
else{
t7=C_eqp(t1,lf[188]);
if(C_truep(t7)){
t8=lf[16];
t9=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=lf[189];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t8=C_eqp(t1,lf[190]);
if(C_truep(t8)){
t9=lf[16];
t10=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t11=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t11;
av2[1]=lf[191];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t9=C_eqp(t1,lf[87]);
if(C_truep(t9)){
/* scrutinizer.scm:449: variable-result */
t10=((C_word*)((C_word*)t0)[5])[1];
f_8492(t10,t4,C_i_car(((C_word*)t0)[3]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}
else{
t10=C_eqp(t1,lf[192]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9563,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:451: chicken.compiler.support#foreign-type->scrutiny-type */
t12=*((C_word*)lf[193]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t11;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
av2[3]=lf[194];
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}
else{
t11=C_eqp(t1,lf[195]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9580,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* scrutinizer.scm:453: chicken.compiler.support#foreign-type->scrutiny-type */
t13=*((C_word*)lf[193]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t13;
av2[1]=t12;
av2[2]=C_i_car(((C_word*)t0)[3]);
av2[3]=lf[194];
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}
else{
t12=C_eqp(t1,lf[196]);
if(C_truep(t12)){
t13=(
/* scrutinizer.scm:455: tag */
  f_9341(C_a_i(&a,29),((C_word*)((C_word*)t0)[10])[1])
);
t14=(
/* scrutinizer.scm:455: tag */
  f_9341(C_a_i(&a,29),((C_word*)((C_word*)t0)[10])[1])
);
t15=C_a_i_cons(&a,2,t13,t14);
t16=C_i_car(((C_word*)t0)[11]);
t17=((C_word*)((C_word*)t0)[12])[1];
t18=C_set_block_item(((C_word*)t0)[12],0,C_SCHEME_FALSE);
t19=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_9600,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[14],a[6]=((C_word*)t0)[15],a[7]=t4,a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[16],a[12]=((C_word*)t0)[9],a[13]=((C_word*)t0)[17],a[14]=t16,a[15]=t17,a[16]=((C_word*)t0)[2],a[17]=t15,a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],tmp=(C_word)a,a+=21,tmp);
t20=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9915,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=t16,a[5]=((C_word*)t0)[21],a[6]=t19,tmp=(C_word)a,a+=7,tmp);
/* scrutinizer.scm:459: walk */
t21=((C_word*)((C_word*)t0)[15])[1];
f_9475(t21,t20,t16,((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE,((C_word*)t0)[9],t15);}
else{
t13=C_eqp(t1,lf[124]);
if(C_truep(t13)){
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9940,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[16],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[17],a[7]=((C_word*)t0)[6],a[8]=t15,a[9]=((C_word*)t0)[22],a[10]=((C_word*)t0)[23],a[11]=((C_word*)t0)[8],a[12]=((C_word*)t0)[21],tmp=(C_word)a,a+=13,tmp));
t17=((C_word*)t15)[1];
f_9940(t17,t4,((C_word*)t0)[3],((C_word*)t0)[11],C_SCHEME_END_OF_LIST);}
else{
t14=C_eqp(t1,lf[221]);
t15=(C_truep(t14)?t14:C_eqp(t1,lf[222]));
if(C_truep(t15)){
t16=C_i_car(((C_word*)t0)[3]);
t17=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10045,a[2]=((C_word*)t0)[16],a[3]=((C_word*)t0)[19],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[22],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[23],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[15],a[11]=((C_word*)t0)[6],a[12]=((C_word*)t0)[24],tmp=(C_word)a,a+=13,tmp);
/* scrutinizer.scm:522: ##sys#decompose-lambda-list */
t18=*((C_word*)lf[227]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t18;
av2[1]=t4;
av2[2]=t16;
av2[3]=t17;
((C_proc)(void*)(*((C_word*)t18+1)))(4,av2);}}
else{
t16=C_eqp(t1,lf[228]);
t17=(C_truep(t16)?t16:C_eqp(t1,lf[229]));
if(C_truep(t17)){
t18=C_i_car(((C_word*)t0)[3]);
t19=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10309,a[2]=t18,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[25],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[19],a[9]=((C_word*)t0)[26],a[10]=((C_word*)t0)[23],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[27],a[13]=((C_word*)t0)[8],a[14]=((C_word*)t0)[7],a[15]=((C_word*)t0)[28],a[16]=((C_word*)t0)[21],a[17]=((C_word*)t0)[15],tmp=(C_word)a,a+=18,tmp);
/* scrutinizer.scm:577: chicken.compiler.support#variable-mark */
t20=*((C_word*)lf[64]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t20;
av2[1]=t19;
av2[2]=t18;
av2[3]=lf[65];
((C_proc)(void*)(*((C_word*)t20+1)))(4,av2);}}
else{
t18=C_eqp(t1,lf[268]);
if(C_truep(t18)){
t19=lf[16];
t20=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t21=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t21;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}
else{
t19=C_eqp(t1,lf[120]);
if(C_truep(t19)){
t20=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10671,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[21],a[6]=((C_word*)t0)[15],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[29],a[10]=((C_word*)t0)[19],a[11]=((C_word*)t0)[26],a[12]=((C_word*)t0)[16],a[13]=((C_word*)t0)[17],a[14]=((C_word*)t0)[18],a[15]=((C_word*)t0)[23],a[16]=((C_word*)t0)[30],a[17]=t4,tmp=(C_word)a,a+=18,tmp);
/* scrutinizer.scm:665: fragment */
f_21102(t20,((C_word*)t0)[8]);}
else{
t20=C_eqp(t1,lf[277]);
if(C_truep(t20)){
t21=C_i_car(((C_word*)t0)[3]);
t22=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11258,a[2]=t21,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[26],a[7]=((C_word*)t0)[28],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:765: walk */
t23=((C_word*)((C_word*)t0)[15])[1];
f_9475(t23,t22,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[16],((C_word*)t0)[9],((C_word*)t0)[17]);}
else{
t21=C_eqp(t1,lf[311]);
if(C_truep(t21)){
t22=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11346,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[15],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[16],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[17],a[11]=t4,tmp=(C_word)a,a+=12,tmp);
/* scrutinizer.scm:777: walk */
t23=((C_word*)((C_word*)t0)[15])[1];
f_9475(t23,t22,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE,((C_word*)t0)[9],((C_word*)t0)[17]);}
else{
t22=C_eqp(t1,lf[341]);
t23=(C_truep(t22)?t22:C_eqp(t1,lf[342]));
if(C_truep(t23)){
/* scrutinizer.scm:793: chicken.compiler.support#bomb */
t24=*((C_word*)lf[343]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t24;
av2[1]=t4;
av2[2]=lf[344];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t24+1)))(4,av2);}}
else{
t24=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11446,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp);
t25=C_i_check_list_2(((C_word*)t0)[11],lf[212]);
t26=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11456,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t27=C_SCHEME_UNDEFINED;
t28=(*a=C_VECTOR_TYPE|1,a[1]=t27,tmp=(C_word)a,a+=2,tmp);
t29=C_set_block_item(t28,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11458,a[2]=t28,a[3]=t24,tmp=(C_word)a,a+=4,tmp));
t30=((C_word*)t28)[1];
f_11458(t30,t26,((C_word*)t0)[11]);}}}}}}}}}}}}}}}}}

/* k9491 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_9493,c,av);}
a=C_alloc(29);
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k9519 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_9521,c,av);}
a=C_alloc(32);
t2=C_a_i_list1(&a,1,t1);
t3=lf[16];
t4=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k9561 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_9563,c,av);}
a=C_alloc(32);
t2=C_a_i_list1(&a,1,t1);
t3=lf[16];
t4=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k9578 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_9580,c,av);}
a=C_alloc(32);
t2=C_a_i_list1(&a,1,t1);
t3=lf[16];
t4=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,5)))){
C_save_and_reclaim((void *)f_9600,c,av);}
a=C_alloc(23);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_caddr(((C_word*)t0)[2]);
t4=((C_word*)((C_word*)t0)[3])[1];
t5=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_9911,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=t2,a[14]=t3,a[15]=((C_word*)t0)[3],a[16]=((C_word*)t0)[15],a[17]=t4,a[18]=((C_word*)t0)[16],a[19]=((C_word*)t0)[17],a[20]=((C_word*)t0)[18],a[21]=((C_word*)t0)[19],a[22]=t1,tmp=(C_word)a,a+=23,tmp);
/* scrutinizer.scm:465: always-true */
t6=((C_word*)((C_word*)t0)[20])[1];
f_8617(t6,t5,((C_word*)t0)[8],((C_word*)t0)[14],t1,((C_word*)t0)[10]);}

/* k9615 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_9617,c,av);}
/* scrutinizer.scm:468: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9475(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k9634 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_9636,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,3,lf[124],t3,((C_word*)t0)[3]);
/* scrutinizer.scm:467: mutate-node! */
f_17320(((C_word*)t0)[4],((C_word*)t0)[5],t4);}

/* k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9642(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(45,0,7)))){
C_save_and_reclaim_args((void *)trf_9642,2,t0,t1);}
a=C_alloc(45);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t2,C_fix(1)));
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9647,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9666,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=t4,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:471: chicken.base#gensym */
t6=*((C_word*)lf[197]+1);{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_9669,a[2]=((C_word*)t0)[13],a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[15],a[5]=((C_word*)t0)[16],a[6]=((C_word*)t0)[12],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[17],a[10]=((C_word*)t0)[4],a[11]=((C_word*)t0)[18],a[12]=((C_word*)t0)[19],a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[20],a[15]=((C_word*)t0)[3],a[16]=((C_word*)t0)[6],a[17]=((C_word*)t0)[8],tmp=(C_word)a,a+=18,tmp);
t3=C_u_i_car(((C_word*)t0)[18]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[9]);
/* scrutinizer.scm:474: walk */
t5=((C_word*)((C_word*)t0)[3])[1];
f_9475(t5,t2,((C_word*)t0)[17],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],t4,C_SCHEME_FALSE);}}

/* k9645 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_9647,c,av);}
/* scrutinizer.scm:472: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9475(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k9664 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9666(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_9666,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,3,lf[124],t3,((C_word*)t0)[3]);
/* scrutinizer.scm:471: mutate-node! */
f_17320(((C_word*)t0)[4],((C_word*)t0)[5],t4);}

/* k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,7)))){
C_save_and_reclaim((void *)f_9669,c,av);}
a=C_alloc(20);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t4=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_9673,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t1,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=((C_word*)t0)[12],a[15]=((C_word*)t0)[13],a[16]=((C_word*)t0)[14],tmp=(C_word)a,a+=17,tmp);
t5=C_u_i_cdr(((C_word*)t0)[11]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[13]);
/* scrutinizer.scm:477: walk */
t7=((C_word*)((C_word*)t0)[15])[1];
f_9475(t7,t4,((C_word*)t0)[6],((C_word*)t0)[16],((C_word*)t0)[8],((C_word*)t0)[17],t6,C_SCHEME_FALSE);}

/* k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_9673,c,av);}
a=C_alloc(23);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=(C_truep(((C_word*)t0)[3])?((C_word*)t0)[3]:(C_truep(((C_word*)t0)[4])?((C_word*)t0)[4]:(C_truep(((C_word*)t0)[5])?t2:C_SCHEME_FALSE)));
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9680,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[5],a[10]=t2,a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
t6=(C_truep(((C_word*)t0)[5])?C_i_not(t2):C_SCHEME_FALSE);
t7=(C_truep(t6)?t6:(C_truep(t2)?C_i_not(((C_word*)t0)[5]):C_SCHEME_FALSE));
if(C_truep(t7)){
t8=(C_truep(((C_word*)t0)[5])?C_u_i_cdr(((C_word*)t0)[13]):C_u_i_car(((C_word*)t0)[13]));
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9821,a[2]=t8,a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[15],tmp=(C_word)a,a+=5,tmp);
t10=((C_word*)((C_word*)t0)[16])[1];
t11=C_i_check_list_2(t10,lf[212]);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9855,a[2]=t13,a[3]=t9,tmp=(C_word)a,a+=4,tmp));
t15=((C_word*)t13)[1];
f_9855(t15,t5,t10);}
else{
t8=t5;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_9680(2,av2);}}}

/* k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_9680,c,av);}
a=C_alloc(29);
t2=C_eqp(lf[53],((C_word*)t0)[2]);
t3=C_i_not(t2);
t4=(C_truep(t3)?C_i_not(C_eqp(lf[53],((C_word*)t0)[3])):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9692,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_i_not(((C_word*)t0)[9]))){
if(C_truep(C_i_not(((C_word*)t0)[10]))){
t6=C_i_length(((C_word*)t0)[2]);
t7=C_i_length(((C_word*)t0)[3]);
t8=t5;
f_9692(t8,C_i_not(C_eqp(t6,t7)));}
else{
t6=t5;
f_9692(t6,C_SCHEME_FALSE);}}
else{
t6=t5;
f_9692(t6,C_SCHEME_FALSE);}}
else{
t5=lf[16];
t6=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9692(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,14)))){
C_save_and_reclaim_args((void *)trf_9692,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9695,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list2(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22769,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[4],tmp=(C_word)a,a+=10,tmp);
/* scrutinizer.scm:2946: scheme#string-append */
t5=*((C_word*)lf[76]+1);{
C_word av2[15];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[199];
av2[3]=lf[200];
av2[4]=lf[201];
av2[5]=lf[202];
av2[6]=lf[203];
av2[7]=lf[204];
av2[8]=lf[205];
av2[9]=lf[206];
av2[10]=lf[207];
av2[11]=lf[208];
av2[12]=lf[209];
av2[13]=lf[210];
av2[14]=lf[211];
((C_proc)(void*)(*((C_word*)t5+1)))(15,av2);}}
else{
if(C_truep(((C_word*)t0)[9])){
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(((C_word*)t0)[10])){
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[5],lf[6]);
t7=C_i_check_list_2(((C_word*)t0)[6],lf[6]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9727,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_9727(t11,((C_word*)t0)[11],((C_word*)t0)[5],((C_word*)t0)[6]);}}}}

/* k9693 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_ccall f_9695(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_9695,c,av);}
a=C_alloc(29);
t2=lf[16];
t3=C_mutate(&lf[16] /* (set! chicken.compiler.scrutinizer#d-depth ...) */,C_s_a_i_minus(&a,2,lf[16],C_fix(1)));
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[53];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop1334 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 in ... */
static void C_fcall f_9727(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_9727,4,t0,t1,t2,t3);}
a=C_alloc(16);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9756,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
t7=C_slot(t2,C_fix(0));
t8=C_slot(t3,C_fix(0));
t9=C_a_i_list(&a,3,lf[28],t7,t8);
/* scrutinizer.scm:503: simplify-type */
f_13503(t6,t9);}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k9754 in map-loop1334 in k9690 in k9678 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in ... */
static void C_ccall f_9756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9756,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9727(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* g1306 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9821(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_9821,3,t0,t1,t2);}
t3=C_i_cdar(t2);
t4=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=C_u_i_car(t2);
/* scrutinizer.scm:490: add-to-blist */
t6=((C_word*)((C_word*)t0)[3])[1];
f_9346(t6,t1,C_u_i_car(t5),C_i_car(((C_word*)t0)[4]),C_u_i_cdr(t2));}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* for-each-loop1305 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9855(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9855,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9865,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:485: g1306 */
t4=((C_word*)t0)[3];
f_9821(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9863 in for-each-loop1305 in k9671 in k9667 in k9640 in k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9865,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_9855(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k9909 in k9598 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(45,c,6)))){
C_save_and_reclaim((void *)f_9911,c,av);}
a=C_alloc(45);
t2=(C_truep(t1)?((C_word*)t0)[2]:C_SCHEME_FALSE);
if(C_truep(t2)){
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9617,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],tmp=(C_word)a,a+=10,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9636,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[13],a[4]=t5,a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:467: chicken.base#gensym */
t7=*((C_word*)lf[197]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_9642,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=((C_word*)t0)[16],a[15]=((C_word*)t0)[17],a[16]=((C_word*)t0)[18],a[17]=((C_word*)t0)[13],a[18]=((C_word*)t0)[19],a[19]=((C_word*)t0)[20],a[20]=((C_word*)t0)[21],tmp=(C_word)a,a+=21,tmp);
t4=C_eqp(((C_word*)t0)[22],lf[46]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8639,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t6=C_a_i_list2(&a,2,((C_word*)t0)[12],((C_word*)t0)[6]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22581,a[2]=t5,a[3]=t6,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:2841: scheme#string-append */
t8=*((C_word*)lf[76]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[214];
av2[3]=lf[215];
av2[4]=lf[216];
av2[5]=lf[217];
av2[6]=lf[218];
((C_proc)(void*)(*((C_word*)t8+1)))(7,av2);}}
else{
t5=t3;
f_9642(t5,C_SCHEME_FALSE);}}}

/* k9913 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9915(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_9915,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9917,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:459: single */
f_8641(((C_word*)t0)[6],t1,t2);}

/* a9916 in k9913 in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9917,c,av);}
t3=lf[219];
/* scrutinizer.scm:460: g1261 */
t4=lf[219];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
av2[5]=t2;
f_22177(6,av2);}}

/* loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9940(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,7)))){
C_save_and_reclaim_args((void *)trf_9940,5,t0,t1,t2,t3,t4);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
t5=C_i_car(t3);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9958,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* scrutinizer.scm:510: scheme#append */
t7=*((C_word*)lf[5]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t5=C_i_car(t2);
t6=C_i_car(t3);
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9967,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[8],a[6]=t1,a[7]=((C_word*)t0)[9],a[8]=t5,a[9]=((C_word*)t0)[10],a[10]=t6,tmp=(C_word)a,a+=11,tmp);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10021,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=((C_word*)t0)[11],a[5]=t6,a[6]=((C_word*)t0)[12],a[7]=t7,tmp=(C_word)a,a+=8,tmp);
/* scrutinizer.scm:513: walk */
t9=((C_word*)((C_word*)t0)[2])[1];
f_9475(t9,t8,t6,((C_word*)t0)[7],((C_word*)t0)[3],t5,((C_word*)t0)[5],C_SCHEME_FALSE);}}

/* k9956 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9958(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_9958,c,av);}
/* scrutinizer.scm:510: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9475(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9967(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_9967,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9970,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9986,a[2]=t2,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10017,a[2]=t3,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* scrutinizer.scm:515: chicken.compiler.support#node-class */
t5=*((C_word*)lf[88]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k9968 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9970(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_9970,2,t0,t1);}
a=C_alloc(6);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=C_u_i_car(((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[4]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[5]);
/* scrutinizer.scm:520: loop */
t7=((C_word*)((C_word*)t0)[6])[1];
f_9940(t7,((C_word*)t0)[7],t2,t3,t6);}

/* k9984 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_fcall f_9986(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9986,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10003,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* scrutinizer.scm:517: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[26]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_9970(t2,C_SCHEME_UNDEFINED);}}

/* k9993 in k10001 in k9984 in k9965 in loop in k9483 in k9480 in k9477 in walk in chicken.compiler.scrutinizer#scrutinize in k6752 in k6749 in k6746 in k6743 in k6740 in k6737 in k6734 in k6731 in k6728 */
static void C_ccall f_9995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_9995,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];
f_9970(t3,t2);}
else{
t2=((C_word*)((C_word*)t0)[3])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t4);
t6=((C_word*)t0)[2];
f_9970(t6,t5);}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_scrutinizer_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("scrutinizer"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_scrutinizer_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(3107))){
C_save(t1);
C_rereclaim2(3107*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,659);
lf[0]=C_h_intern(&lf[0],11, C_text("scrutinizer"));
lf[1]=C_h_intern(&lf[1],29, C_text("chicken.compiler.scrutinizer#"));
lf[4]=C_h_intern(&lf[4],5, C_text("foldr"));
lf[5]=C_h_intern(&lf[5],13, C_text("scheme#append"));
lf[6]=C_h_intern(&lf[6],3, C_text("map"));
lf[8]=C_h_intern(&lf[8],12, C_text("scheme#null\077"));
lf[20]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001string\376\003\000\000\002\376\001\000\000\006\001symbol\376\003\000\000\002\376\001\000\000\007\001keyword\376\003\000\000\002\376\001\000\000\004\001char\376\003\000\000\002\376\001\000\000\004\001nu"
"ll\376\003\000\000\002\376\001\000\000\007\001boolean\376\003\000\000\002\376\001\000\000\004\001true\376\003\000\000\002\376\001\000\000\005\001false\376\003\000\000\002\376\001\000\000\004\001blob\376\003\000\000\002\376\001\000\000\003\001eof"
"\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\005\001float\376\003\000\000\002\376\001\000\000\006\001number\376\003\000\000\002\376\001\000\000\007\001integer\376\003\000\000\002\376\001\000\000\006\001b"
"ignum\376\003\000\000\002\376\001\000\000\006\001ratnum\376\003\000\000\002\376\001\000\000\007\001cplxnum\376\003\000\000\002\376\001\000\000\016\001pointer-vector\376\003\000\000\002\376\001\000\000\004\001port"
"\376\003\000\000\002\376\001\000\000\007\001pointer\376\003\000\000\002\376\001\000\000\010\001locative\376\377\016"));
lf[22]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\004\001list\376\003\000\000\002\376\001\000\000\004\001pair\376\003\000\000\002\376\001\000\000\011\001procedure\376\003\000\000\002\376\001\000\000\006\001vector\376"
"\003\000\000\002\376\001\000\000\011\001undefined\376\003\000\000\002\376\001\000\000\012\001deprecated\376\003\000\000\002\376\001\000\000\010\001noreturn\376\003\000\000\002\376\001\000\000\006\001values\376\377\016"));
lf[26]=C_h_intern(&lf[26],40, C_text("chicken.compiler.support#node-parameters"));
lf[28]=C_h_intern(&lf[28],2, C_text("or"));
lf[29]=C_h_intern(&lf[29],6, C_text("forall"));
lf[30]=C_h_intern(&lf[30],3, C_text("eof"));
lf[31]=C_h_intern(&lf[31],4, C_text("null"));
lf[32]=C_h_intern(&lf[32],6, C_text("fixnum"));
lf[33]=C_h_intern(&lf[33],4, C_text("char"));
lf[34]=C_h_intern(&lf[34],7, C_text("boolean"));
lf[35]=C_h_intern(&lf[35],9, C_text("undefined"));
lf[36]=C_h_intern(&lf[36],39, C_text("chicken.compiler.scrutinizer#scrutinize"));
lf[37]=C_h_intern(&lf[37],6, C_text("string"));
lf[38]=C_h_intern(&lf[38],7, C_text("keyword"));
lf[39]=C_h_intern(&lf[39],6, C_text("symbol"));
lf[40]=C_h_intern(&lf[40],7, C_text("integer"));
lf[41]=C_h_intern(&lf[41],6, C_text("bignum"));
lf[42]=C_h_intern(&lf[42],5, C_text("float"));
lf[43]=C_h_intern(&lf[43],6, C_text("ratnum"));
lf[44]=C_h_intern(&lf[44],7, C_text("cplxnum"));
lf[45]=C_h_intern(&lf[45],4, C_text("true"));
lf[46]=C_h_intern(&lf[46],5, C_text("false"));
lf[47]=C_h_intern(&lf[47],4, C_text("list"));
lf[48]=C_h_intern(&lf[48],4, C_text("pair"));
lf[50]=C_h_intern(&lf[50],6, C_text("vector"));
lf[51]=C_h_intern(&lf[51],19, C_text("scheme#vector->list"));
lf[52]=C_h_intern(&lf[52],6, C_text("struct"));
lf[53]=C_h_intern(&lf[53],1, C_text("\052"));
lf[54]=C_h_intern(&lf[54],38, C_text("chicken.compiler.support#small-bignum\077"));
lf[55]=C_h_intern(&lf[55],36, C_text("chicken.compiler.support#big-fixnum\077"));
lf[56]=C_h_intern(&lf[56],24, C_text("chicken.keyword#keyword\077"));
lf[57]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[58]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[59]=C_h_intern(&lf[59],10, C_text("deprecated"));
lf[60]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[62]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[63]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[64]=C_h_intern(&lf[64],38, C_text("chicken.compiler.support#variable-mark"));
lf[65]=C_h_intern(&lf[65],15, C_text("##compiler#type"));
lf[66]=C_h_intern(&lf[66],22, C_text("##compiler#type-source"));
lf[67]=C_h_intern(&lf[67],31, C_text("chicken.compiler.support#db-get"));
lf[68]=C_h_intern(&lf[68],8, C_text("assigned"));
lf[69]=C_h_intern(&lf[69],3, C_text("not"));
lf[70]=C_h_intern(&lf[70],8, C_text("noreturn"));
lf[72]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023Test is always true"));
lf[73]=C_h_intern(&lf[73],12, C_text("##sys#notice"));
lf[76]=C_h_intern(&lf[76],20, C_text("scheme#string-append"));
lf[77]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032In conditional expression:"));
lf[78]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[79]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[80]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[81]=C_decode_literal(C_heaptop,C_text("\376B\000\000-Test condition has always true value of type:"));
lf[82]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[83]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[84]=C_h_intern(&lf[84],9, C_text("procedure"));
lf[85]=C_h_intern(&lf[85],17, C_text("##core#the/result"));
lf[86]=C_h_intern(&lf[86],44, C_text("chicken.compiler.support#node-subexpressions"));
lf[87]=C_h_intern(&lf[87],15, C_text("##core#variable"));
lf[88]=C_h_intern(&lf[88],35, C_text("chicken.compiler.support#node-class"));
lf[89]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021Invalid procedure"));
lf[90]=C_h_intern(&lf[90],20, C_text("chicken.base#warning"));
lf[92]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[93]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[94]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[95]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[96]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037Variable ~a is not a procedure."));
lf[97]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[98]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021It has this type:"));
lf[99]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[100]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[101]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021Invalid procedure"));
lf[103]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[104]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[105]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[106]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[107]=C_decode_literal(C_heaptop,C_text("\376B\000\000:The procedure expression does not appear to be a callable."));
lf[108]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[109]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[110]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[111]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025The expected type is:"));
lf[112]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[113]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[114]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[115]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023The actual type is:"));
lf[116]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[117]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[120]=C_h_intern(&lf[120],11, C_text("##core#call"));
lf[122]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001true\376\377\016"));
lf[123]=C_h_intern(&lf[123],5, C_text("quote"));
lf[124]=C_h_intern(&lf[124],3, C_text("let"));
lf[126]=C_h_intern(&lf[126],18, C_text("##sys#list->vector"));
lf[127]=C_h_intern(&lf[127],3, C_text("tmp"));
lf[128]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030Predicate is always true"));
lf[129]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[130]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[131]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[132]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[133]=C_decode_literal(C_heaptop,C_text("\376B\000\000&The predicate will always return true."));
lf[134]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[135]=C_decode_literal(C_heaptop,C_text("\376B\000\000 Procedure ~a is a predicate for:"));
lf[136]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[137]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[138]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[139]=C_decode_literal(C_heaptop,C_text("\376B\000\000!The given argument has this type:"));
lf[140]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[141]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[143]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001false\376\377\016"));
lf[144]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031Predicate is always false"));
lf[145]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[146]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[147]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[148]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[149]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047The predicate will always return false."));
lf[150]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[151]=C_decode_literal(C_heaptop,C_text("\376B\000\000 Procedure ~a is a predicate for:"));
lf[152]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[153]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[154]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[155]=C_decode_literal(C_heaptop,C_text("\376B\000\000!The given argument has this type:"));
lf[156]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[157]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[162]=C_h_intern(&lf[162],26, C_text("##compiler#specializations"));
lf[163]=C_h_intern(&lf[163],32, C_text("##compiler#local-specializations"));
lf[164]=C_h_intern(&lf[164],49, C_text("chicken.compiler.support#maybe-constant-fold-call"));
lf[165]=C_h_intern(&lf[165],20, C_text("##compiler#predicate"));
lf[167]=C_h_intern(&lf[167],20, C_text("chicken.base#call/cc"));
lf[170]=C_h_intern(&lf[170],40, C_text("chicken.compiler.scrutinizer#match-types"));
lf[171]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[172]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001s"));
lf[173]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002is"));
lf[174]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003are"));
lf[175]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031Wrong number of arguments"));
lf[177]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[178]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[179]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[180]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[181]=C_decode_literal(C_heaptop,C_text("\376B\000\000JProcedure `~a\047 is called with ~a argument~a but ~a argument~a ~a expected."));
lf[182]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[183]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033Procedure ~a has this type:"));
lf[184]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[185]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[186]=C_h_intern(&lf[186],26, C_text("chicken.base#alist-update!"));
lf[187]=C_h_intern(&lf[187],13, C_text("scheme#equal\077"));
lf[188]=C_h_intern(&lf[188],16, C_text("##core#undefined"));
lf[189]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[190]=C_h_intern(&lf[190],11, C_text("##core#proc"));
lf[191]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001procedure\376\377\016"));
lf[192]=C_h_intern(&lf[192],17, C_text("##core#inline_ref"));
lf[193]=C_h_intern(&lf[193],52, C_text("chicken.compiler.support#foreign-type->scrutiny-type"));
lf[194]=C_h_intern(&lf[194],6, C_text("result"));
lf[195]=C_h_intern(&lf[195],21, C_text("##core#inline_loc_ref"));
lf[196]=C_h_intern(&lf[196],2, C_text("if"));
lf[197]=C_h_intern(&lf[197],19, C_text("chicken.base#gensym"));
lf[198]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026Branch values mismatch"));
lf[199]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032In conditional expression:"));
lf[200]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[201]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[202]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[203]=C_decode_literal(C_heaptop,C_text("\376B\000\000.The branches have different numbers of values."));
lf[204]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[205]=C_decode_literal(C_heaptop,C_text("\376B\000\000#The true branch returns ~a value~a:"));
lf[206]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[207]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[208]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[209]=C_decode_literal(C_heaptop,C_text("\376B\000\000$The false branch returns ~a value~a:"));
lf[210]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[211]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[212]=C_h_intern(&lf[212],8, C_text("for-each"));
lf[213]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024Test is always false"));
lf[214]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032In conditional expression:"));
lf[215]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[216]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[217]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[218]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037Test condition is always false."));
lf[221]=C_h_intern(&lf[221],13, C_text("##core#lambda"));
lf[222]=C_h_intern(&lf[222],6, C_text("lambda"));
lf[223]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001procedure\376\377\016"));
lf[224]=C_h_intern(&lf[224],18, C_text("##sys#dynamic-wind"));
lf[225]=C_h_intern(&lf[225],20, C_text("chicken.base#butlast"));
lf[226]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[227]=C_h_intern(&lf[227],27, C_text("##sys#decompose-lambda-list"));
lf[228]=C_h_intern(&lf[228],4, C_text("set!"));
lf[229]=C_h_intern(&lf[229],11, C_text("##core#set!"));
lf[230]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\006\001\376\377\016"));
lf[231]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001undefined\376\377\016"));
lf[232]=C_h_intern(&lf[232],8, C_text("captured"));
lf[233]=C_h_intern(&lf[233],38, C_text("chicken.compiler.support#mark-variable"));
lf[234]=C_h_intern(&lf[234],9, C_text("inference"));
lf[235]=C_h_intern(&lf[235],6, C_text("format"));
lf[236]=C_h_intern(&lf[236],34, C_text("chicken.compiler.support#debugging"));
lf[237]=C_h_intern(&lf[237],1, C_text("I"));
lf[238]=C_h_intern(&lf[238],30, C_text("chicken.base#get-output-string"));
lf[239]=C_h_intern(&lf[239],18, C_text("##sys#write-char-0"));
lf[240]=C_h_intern(&lf[240],11, C_text("##sys#print"));
lf[241]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003(: "));
lf[242]=C_h_intern(&lf[242],31, C_text("chicken.base#open-output-string"));
lf[244]=C_h_intern(&lf[244],34, C_text("chicken.compiler.support#tree-copy"));
lf[245]=C_h_intern(&lf[245],2, C_text("no"));
lf[246]=C_h_intern(&lf[246],17, C_text("##compiler#inline"));
lf[247]=C_h_intern(&lf[247],42, C_text("chicken.compiler.support#variable-visible\077"));
lf[248]=C_h_intern(&lf[248],11, C_text("local-value"));
lf[249]=C_h_intern(&lf[249],5, C_text("value"));
lf[250]=C_h_intern(&lf[250],7, C_text("unknown"));
lf[251]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022Invalid assignment"));
lf[252]=C_h_intern(&lf[252],21, C_text("scheme#string->symbol"));
lf[253]=C_h_intern(&lf[253],34, C_text("chicken.compiler.support#real-name"));
lf[254]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In assignment:"));
lf[255]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[256]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[257]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[258]=C_decode_literal(C_heaptop,C_text("\376B\000\000(Variable `~a\047 is assigned invalid value."));
lf[259]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[260]=C_decode_literal(C_heaptop,C_text("\376B\000\000!The assigned value has this type:"));
lf[261]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[262]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[263]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[264]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033The declared type of ~a is:"));
lf[265]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[266]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[268]=C_h_intern(&lf[268],16, C_text("##core#primitive"));
lf[269]=C_h_intern(&lf[269],34, C_text("chicken.compiler.support#make-node"));
lf[271]=C_h_intern(&lf[271],16, C_text("##compiler#clean"));
lf[272]=C_h_intern(&lf[272],15, C_text("##compiler#pure"));
lf[273]=C_h_intern(&lf[273],41, C_text("chicken.compiler.scrutinizer#refine-types"));
lf[274]=C_h_intern(&lf[274],30, C_text("##compiler#special-result-type"));
lf[275]=C_h_intern(&lf[275],22, C_text("chicken.base#alist-ref"));
lf[276]=C_h_intern(&lf[276],18, C_text("##compiler#enforce"));
lf[277]=C_h_intern(&lf[277],10, C_text("##core#the"));
lf[278]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021Not enough values"));
lf[279]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In expression:"));
lf[280]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[281]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[282]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[283]=C_decode_literal(C_heaptop,C_text("\376B\000\0006Expression returns 0 values but is declared to return:"));
lf[284]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[285]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[286]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015Type mismatch"));
lf[287]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In expression:"));
lf[288]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[289]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[290]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[291]=C_decode_literal(C_heaptop,C_text("\376B\000\0004Expression\047s declared and actual types do not match."));
lf[292]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[293]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025The declared type is:"));
lf[294]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[295]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[296]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[297]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023The actual type is:"));
lf[298]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[299]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[301]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017Too many values"));
lf[302]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In expression:"));
lf[303]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[304]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[305]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[306]=C_decode_literal(C_heaptop,C_text("\376B\000\000#Expression returns too many values."));
lf[307]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[308]=C_decode_literal(C_heaptop,C_text("\376B\000\000;The expression returns ~a values but is declared to return:"));
lf[309]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[310]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[311]=C_h_intern(&lf[311],15, C_text("##core#typecase"));
lf[312]=C_h_intern(&lf[312],39, C_text("chicken.compiler.support#quit-compiling"));
lf[313]=C_h_intern(&lf[313],33, C_text("chicken.string#string-intersperse"));
lf[314]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\012\012"));
lf[316]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[318]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[319]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[320]=C_h_intern(&lf[320],19, C_text("chicken.string#conc"));
lf[321]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[322]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\012  "));
lf[324]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021No typecase match"));
lf[325]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~%"));
lf[326]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[327]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[328]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042In `compiler-typecase\047 expression:"));
lf[329]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[330]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004  ~a"));
lf[331]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[332]=C_decode_literal(C_heaptop,C_text("\376B\000\000,  Tested expression does not match any case."));
lf[333]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[334]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037  The expression has this type:"));
lf[335]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[336]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[337]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[338]=C_decode_literal(C_heaptop,C_text("\376B\000\000%  The specified type cases are these:"));
lf[339]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[340]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[341]=C_h_intern(&lf[341],13, C_text("##core#switch"));
lf[342]=C_h_intern(&lf[342],11, C_text("##core#cond"));
lf[343]=C_h_intern(&lf[343],29, C_text("chicken.compiler.support#bomb"));
lf[344]=C_decode_literal(C_heaptop,C_text("\376B\000\000!scrutinize: unexpected node class"));
lf[345]=C_decode_literal(C_heaptop,C_text("\376B\000\000-some variable types do not satisfy strictness"));
lf[346]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001e\376\377\016"));
lf[347]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037assignments to immediate values"));
lf[348]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001e\376\377\016"));
lf[349]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020dropped branches"));
lf[350]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001e\376\377\016"));
lf[351]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012safe calls"));
lf[352]=C_h_intern(&lf[352],21, C_text("##sys#standard-output"));
lf[353]=C_h_intern(&lf[353],6, C_text("printf"));
lf[354]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[355]=C_h_intern(&lf[355],18, C_text("chicken.base#print"));
lf[356]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020specializations:"));
lf[357]=C_h_intern(&lf[357],46, C_text("chicken.compiler.support#with-debugging-output"));
lf[358]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001e\376\377\016"));
lf[359]=C_h_intern(&lf[359],9, C_text("vector-of"));
lf[360]=C_h_intern(&lf[360],23, C_text("chicken.base#constantly"));
lf[361]=C_h_intern(&lf[361],7, C_text("list-of"));
lf[362]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\004\001pair\376\003\000\000\002\376\001\000\000\004\001null\376\377\016"));
lf[363]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\003\000\000\002\376\001\000\000\012\001#!optional\376\377\016"));
lf[364]=C_h_intern(&lf[364],10, C_text("#!optional"));
lf[365]=C_h_intern(&lf[365],6, C_text("#!rest"));
lf[367]=C_h_intern(&lf[367],14, C_text("scheme#reverse"));
lf[368]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001undefined\376\003\000\000\002\376\001\000\000\010\001noreturn\376\377\016"));
lf[372]=C_h_intern(&lf[372],6, C_text("refine"));
lf[373]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376\001\000\000\004\001pair\376\003\000\000\002\376\001\000\000\004\001pair\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\004\001li"
"st\376\003\000\000\002\376\001\000\000\007\001list-of\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\006\001vector\376\003\000\000\002\376\001\000\000\011\001vector-of\376\003\000"
"\000\002\376\001\000\000\001\001\052\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\007\001boolean\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\004\001true\376\003\000\000\002\376\001\000\000\005\001false\376"
"\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\007\001integer\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\006\001bignum\376\377\016\376\003\000\000\002"
"\376\003\000\000\002\376\001\000\000\006\001number\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\005\001float\376\003\000\000\002\376\001\000\000\006\001bignum"
"\376\003\000\000\002\376\001\000\000\006\001ratnum\376\003\000\000\002\376\001\000\000\007\001cplxnum\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\011\001procedure\376\003\000\000\002\376\001\000\000\011\001proced"
"ure\376\003\000\000\002\376\003\000\000\002\376\001\000\000\006\001#!rest\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016\376\001\000\000\001\001\052\376\377\016"));
lf[374]=C_h_intern(&lf[374],10, C_text("scheme#eq\077"));
lf[375]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[376]=C_h_intern(&lf[376],14, C_text("##core#tv-root"));
lf[377]=C_h_intern(&lf[377],17, C_text("chicken.plist#get"));
lf[378]=C_h_intern(&lf[378],5, C_text("foldl"));
lf[379]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001procedure\376\377\016"));
lf[381]=C_h_intern(&lf[381],6, C_text("number"));
lf[382]=C_h_intern(&lf[382],36, C_text("chicken.compiler.scrutinizer#type<=\077"));
lf[383]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\005\001float\376\003\000\000\002\376\001\000\000\006\001bignum\376\003\000\000\002\376\001\000\000\006\001ratnum\376\003\000\000\002\376\001\000\000\007\001cp"
"lxnum\376\377\016"));
lf[384]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\006\001bignum\376\377\016"));
lf[385]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001true\376\003\000\000\002\376\001\000\000\005\001false\376\377\016"));
lf[389]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001procedure\376\377\016"));
lf[390]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[391]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[392]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[393]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\377\016"));
lf[394]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001forall\376\003\000\000\002\376\001\000\000\006\001refine\376\377\016"));
lf[395]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001forall\376\003\000\000\002\376\001\000\000\006\001refine\376\377\016"));
lf[396]=C_h_intern(&lf[396],21, C_text("chicken.base#identity"));
lf[397]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001null\376\003\000\000\002\376\001\000\000\004\001pair\376\377\016"));
lf[398]=C_decode_literal(C_heaptop,C_text("\376B\000\000)procedure-arguments: not a procedure type"));
lf[399]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047procedure-results: not a procedure type"));
lf[400]=C_h_intern(&lf[400],6, C_text("values"));
lf[401]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\001noreturn\376\377\016"));
lf[403]=C_decode_literal(C_heaptop,C_text("\376B\000\000,resolve: can\047t resolve unknown type-variable"));
lf[404]=C_h_intern(&lf[404],12, C_text("##sys#append"));
lf[405]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001values\376\377\016"));
lf[406]=C_h_intern(&lf[406],47, C_text("chicken.compiler.scrutinizer#load-type-database"));
lf[407]=C_h_intern(&lf[407],42, C_text("chicken.compiler.scrutinizer#validate-type"));
lf[408]=C_h_intern(&lf[408],52, C_text("chicken.compiler.scrutinizer#install-specializations"));
lf[409]=C_h_intern(&lf[409],2, C_text("db"));
lf[410]=C_h_intern(&lf[410],22, C_text("chicken.format#sprintf"));
lf[411]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047Declared type for toplevel binding `~a\047"));
lf[412]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012~%~%~a~%~%"));
lf[413]=C_decode_literal(C_heaptop,C_text("\376B\000\000(  conflicts with previously loaded type:"));
lf[414]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006~%~%~a"));
lf[415]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047:"));
lf[416]=C_decode_literal(C_heaptop,C_text("\376B\000\000 Invalid type specification for `"));
lf[417]=C_h_intern_kw(&lf[417],4, C_text("pure"));
lf[418]=C_h_intern_kw(&lf[418],5, C_text("clean"));
lf[419]=C_h_intern_kw(&lf[419],7, C_text("enforce"));
lf[420]=C_h_intern_kw(&lf[420],8, C_text("foldable"));
lf[421]=C_h_intern(&lf[421],19, C_text("##compiler#foldable"));
lf[422]=C_h_intern_kw(&lf[422],9, C_text("predicate"));
lf[423]=C_decode_literal(C_heaptop,C_text("\376B\000\0003load-type-database: invalid procedure-type property"));
lf[424]=C_h_intern(&lf[424],27, C_text("scheme#call-with-input-file"));
lf[425]=C_h_intern(&lf[425],41, C_text("chicken.compiler.support#read-expressions"));
lf[426]=C_h_intern(&lf[426],1, C_text("p"));
lf[427]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005\047 ..."));
lf[428]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027loading type database `"));
lf[429]=C_h_intern(&lf[429],18, C_text("##sys#file-exists\077"));
lf[430]=C_h_intern(&lf[430],22, C_text("chicken.load#find-file"));
lf[431]=C_h_intern(&lf[431],32, C_text("chicken.platform#repository-path"));
lf[432]=C_h_intern(&lf[432],44, C_text("chicken.compiler.scrutinizer#emit-types-file"));
lf[433]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015; END OF FILE"));
lf[434]=C_h_intern(&lf[434],14, C_text("scheme#newline"));
lf[435]=C_h_intern(&lf[435],23, C_text("chicken.pretty-print#pp"));
lf[436]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\002enforce\376\377\016"));
lf[437]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\002pure\376\377\016"));
lf[438]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\002clean\376\377\016"));
lf[439]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\002foldable\376\377\016"));
lf[440]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001local\376\003\000\000\002\376\001\000\000\011\001inference\376\377\016"));
lf[441]=C_h_intern(&lf[441],36, C_text("chicken.internal#hash-table-for-each"));
lf[442]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027; GENERATED BY CHICKEN "));
lf[443]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 FROM "));
lf[444]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[445]=C_h_intern(&lf[445],32, C_text("chicken.platform#chicken-version"));
lf[446]=C_h_intern(&lf[446],26, C_text("scheme#with-output-to-file"));
lf[448]=C_h_intern(&lf[448],35, C_text("chicken.compiler.support#copy-node!"));
lf[449]=C_h_intern(&lf[449],41, C_text("chicken.compiler.support#build-node-graph"));
lf[450]=C_h_intern(&lf[450],34, C_text("chicken.compiler.support#copy-node"));
lf[452]=C_h_intern(&lf[452],13, C_text("scheme#values"));
lf[454]=C_h_intern(&lf[454],3, C_text("..."));
lf[455]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[456]=C_h_intern(&lf[456],9, C_text("&optional"));
lf[457]=C_h_intern(&lf[457],5, C_text("&rest"));
lf[458]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[459]=C_h_intern(&lf[459],5, C_text("#!key"));
lf[460]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001#!rest\376\003\000\000\002\376\001\000\000\001\001\052\376\377\016"));
lf[461]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\001u8vector\376\003\000\000\002\376\001\000\000\010\001s8vector\376\003\000\000\002\376\001\000\000\011\001u16vector\376\003\000\000\002\376\001\000\000\011\001s16vector\376\003"
"\000\000\002\376\001\000\000\011\001u32vector\376\003\000\000\002\376\001\000\000\011\001s32vector\376\003\000\000\002\376\001\000\000\011\001u64vector\376\003\000\000\002\376\001\000\000\011\001s64vector\376\003"
"\000\000\002\376\001\000\000\011\001f32vector\376\003\000\000\002\376\001\000\000\011\001f64vector\376\003\000\000\002\376\001\000\000\006\001thread\376\003\000\000\002\376\001\000\000\005\001queue\376\003\000\000\002\376\001\000\000"
"\013\001environment\376\003\000\000\002\376\001\000\000\004\001time\376\003\000\000\002\376\001\000\000\014\001continuation\376\003\000\000\002\376\001\000\000\004\001lock\376\003\000\000\002\376\001\000\000\004\001mma"
"p\376\003\000\000\002\376\001\000\000\011\001condition\376\003\000\000\002\376\001\000\000\012\001hash-table\376\003\000\000\002\376\001\000\000\014\001tcp-listener\376\377\016"));
lf[462]=C_h_intern(&lf[462],9, C_text("immediate"));
lf[463]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\003\000\000\002\376\001\000\000\003\001eof\376\003\000\000\002\376\001\000\000\004\001null\376\003\000\000\002\376\001\000\000\006\001fixnum\376\003\000\000\002\376\001\000\000\004\001char\376\003\000\000\002\376"
"\001\000\000\007\001boolean\376\377\016"));
lf[464]=C_h_intern(&lf[464],3, C_text("any"));
lf[465]=C_h_intern(&lf[465],4, C_text("void"));
lf[466]=C_h_intern(&lf[466],10, C_text("input-port"));
lf[467]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001refine\376\003\000\000\002\376\003\000\000\002\376\001\000\000\005\001input\376\377\016\376\003\000\000\002\376\001\000\000\004\001port\376\377\016"));
lf[468]=C_h_intern(&lf[468],11, C_text("output-port"));
lf[469]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001refine\376\003\000\000\002\376\003\000\000\002\376\001\000\000\006\001output\376\377\016\376\003\000\000\002\376\001\000\000\004\001port\376\377\016"));
lf[470]=C_h_intern(&lf[470],14, C_text("scheme#symbol\077"));
lf[471]=C_h_intern(&lf[471],3, C_text("-->"));
lf[472]=C_h_intern(&lf[472],1, C_text(":"));
lf[473]=C_h_intern(&lf[473],2, C_text("->"));
lf[474]=C_h_intern(&lf[474],9, C_text("##sys#get"));
lf[475]=C_h_intern(&lf[475],28, C_text("##compiler#type-abbreviation"));
lf[476]=C_h_intern(&lf[476],18, C_text("chicken.plist#put!"));
lf[477]=C_h_intern(&lf[477],21, C_text("scheme#symbol->string"));
lf[478]=C_h_intern(&lf[478],27, C_text("chicken.syntax#strip-syntax"));
lf[479]=C_h_intern(&lf[479],52, C_text("chicken.compiler.scrutinizer#check-and-validate-type"));
lf[480]=C_h_intern(&lf[480],18, C_text("chicken.base#error"));
lf[481]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026invalid type specifier"));
lf[482]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035invalid specialization format"));
lf[483]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047invalid argument type in specialization"));
lf[484]=C_decode_literal(C_heaptop,C_text("\376B\000\000%invalid result type in specialization"));
lf[485]=C_h_intern(&lf[485],15, C_text("##sys#substring"));
lf[487]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006vector"));
lf[488]=C_h_intern(&lf[488],10, C_text("##sys#slot"));
lf[489]=C_h_intern(&lf[489],8, C_text("not-used"));
lf[490]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004list"));
lf[491]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004list"));
lf[492]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[493]=C_h_intern(&lf[493],19, C_text("##sys#string-append"));
lf[494]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[495]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[496]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[497]=C_h_intern(&lf[497],27, C_text("chicken.string#string-split"));
lf[498]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[499]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001/"));
lf[500]=C_h_intern_kw(&lf[500],2, C_text("tv"));
lf[501]=C_h_intern(&lf[501],11, C_text("##sys#error"));
lf[502]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037type->pp-string: unhandled type"));
lf[503]=C_h_intern(&lf[503],27, C_text("chicken.string#string-chomp"));
lf[504]=C_h_intern(&lf[504],34, C_text("chicken.port#with-output-to-string"));
lf[505]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001...\376\377\016"));
lf[506]=C_h_intern(&lf[506],46, C_text("chicken.compiler.support#build-expression-tree"));
lf[507]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[508]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047,"));
lf[509]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011In file `"));
lf[510]=C_h_intern(&lf[510],42, C_text("chicken.compiler.support#source-info->line"));
lf[511]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[512]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021At the toplevel,\012"));
lf[513]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[514]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[516]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047,"));
lf[517]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In procedure `"));
lf[518]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047,"));
lf[519]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013In module `"));
lf[520]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[521]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030In a toplevel procedure,"));
lf[522]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047,"));
lf[523]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In procedure `"));
lf[524]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025In a local procedure,"));
lf[525]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001#"));
lf[526]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017\047 from module `"));
lf[527]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027This is the expression:"));
lf[528]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[529]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[530]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047It is a call to ~a which has this type:"));
lf[531]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[532]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[533]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[534]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[535]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[536]=C_h_intern(&lf[536],25, C_text("chicken.base#flush-output"));
lf[537]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002  "));
lf[538]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[539]=C_h_intern(&lf[539],22, C_text("chicken.format#fprintf"));
lf[540]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[541]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[542]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[543]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020Invalid argument"));
lf[544]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[545]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[546]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[547]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[548]=C_decode_literal(C_heaptop,C_text("\376B\000\0003Argument #~a to procedure `~a\047 has an invalid type:"));
lf[549]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[550]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[551]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[552]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025The expected type is:"));
lf[553]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[554]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[555]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[556]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[557]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[558]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033Procedure ~a has this type:"));
lf[559]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[560]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[561]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[562]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002 `"));
lf[563]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[564]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[565]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[566]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[567]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036Argument #~a to procedure~a ~a"));
lf[568]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[569]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[570]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032Not enough argument values"));
lf[571]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033does not return any values."));
lf[572]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030Too many argument values"));
lf[573]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032 values but 1 is expected."));
lf[574]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010returns "));
lf[575]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021a negative index "));
lf[576]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047."));
lf[577]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014 of length `"));
lf[578]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010\047 for a "));
lf[579]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007index `"));
lf[580]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In procedure call:"));
lf[581]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[582]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[583]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[584]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036Procedure ~a is called with ~a"));
lf[585]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 index"));
lf[586]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011Negative "));
lf[587]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023 index out of range"));
lf[588]=C_h_intern(&lf[588],16, C_text("scheme#substring"));
lf[589]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017In conditional:"));
lf[590]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[591]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[592]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[593]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026The test expression ~a"));
lf[594]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[595]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[596]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033Zero values for conditional"));
lf[597]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021returns 0 values."));
lf[598]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037Too many values for conditional"));
lf[599]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 values."));
lf[600]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010returns "));
lf[601]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022In let expression:"));
lf[602]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[603]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[604]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[605]=C_decode_literal(C_heaptop,C_text("\376B\000\000/Variable `~a\047 is bound to an expression that ~a"));
lf[606]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[607]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[608]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021returns 0 values."));
lf[609]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021\047 has zero values"));
lf[610]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020Let binding to `"));
lf[611]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 values."));
lf[612]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010returns "));
lf[613]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007 values"));
lf[614]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006\047 has "));
lf[615]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020Let binding to `"));
lf[616]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In assignment:"));
lf[617]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[618]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[619]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[620]=C_decode_literal(C_heaptop,C_text("\376B\000\0001Variable `~a\047 is assigned from expression that ~a"));
lf[621]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[622]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[623]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021returns 0 values."));
lf[624]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021\047 has zero values"));
lf[625]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017Assignment to `"));
lf[626]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 values."));
lf[627]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010returns "));
lf[628]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007 values"));
lf[629]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006\047 has "));
lf[630]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017Assignment to `"));
lf[631]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035The suggested alternative is "));
lf[632]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[633]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016In expression:"));
lf[634]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[635]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[636]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004~%~%"));
lf[637]=C_decode_literal(C_heaptop,C_text("\376B\000\000 Use of deprecated identifier ~a."));
lf[638]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002~a"));
lf[639]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027Deprecated identifier `"));
lf[640]=C_h_intern(&lf[640],10, C_text("##sys#put!"));
lf[641]=C_h_intern(&lf[641],18, C_text("scheme#make-vector"));
lf[642]=C_h_intern(&lf[642],12, C_text("##sys#vector"));
lf[643]=C_h_intern(&lf[643],13, C_text("scheme#vector"));
lf[644]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001null\376\377\016"));
lf[645]=C_h_intern(&lf[645],10, C_text("##sys#list"));
lf[646]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001null\376\377\016"));
lf[647]=C_h_intern(&lf[647],11, C_text("scheme#list"));
lf[648]=C_h_intern(&lf[648],16, C_text("scheme#list-tail"));
lf[649]=C_h_intern(&lf[649],15, C_text("scheme#list-ref"));
lf[650]=C_h_intern(&lf[650],14, C_text("##sys#setislot"));
lf[651]=C_h_intern(&lf[651],13, C_text("##sys#setslot"));
lf[652]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\011\001undefined\376\377\016"));
lf[653]=C_h_intern(&lf[653],18, C_text("scheme#vector-set!"));
lf[654]=C_h_intern(&lf[654],16, C_text("##sys#vector-ref"));
lf[655]=C_h_intern(&lf[655],17, C_text("scheme#vector-ref"));
lf[656]=C_h_intern(&lf[656],14, C_text("pointer-vector"));
lf[657]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\016\001pointer-vector\376\377\016"));
lf[658]=C_h_intern(&lf[658],20, C_text("##sys#make-structure"));
C_register_lf2(lf,659,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6730,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_library_toplevel(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[1322] = {
{C_text("f25872:scrutinizer_2escm"),(void*)f25872},
{C_text("f25979:scrutinizer_2escm"),(void*)f25979},
{C_text("f25994:scrutinizer_2escm"),(void*)f25994},
{C_text("f26001:scrutinizer_2escm"),(void*)f26001},
{C_text("f26243:scrutinizer_2escm"),(void*)f26243},
{C_text("f26391:scrutinizer_2escm"),(void*)f26391},
{C_text("f26456:scrutinizer_2escm"),(void*)f26456},
{C_text("f26463:scrutinizer_2escm"),(void*)f26463},
{C_text("f26609:scrutinizer_2escm"),(void*)f26609},
{C_text("f26614:scrutinizer_2escm"),(void*)f26614},
{C_text("f26623:scrutinizer_2escm"),(void*)f26623},
{C_text("f26630:scrutinizer_2escm"),(void*)f26630},
{C_text("f26669:scrutinizer_2escm"),(void*)f26669},
{C_text("f26674:scrutinizer_2escm"),(void*)f26674},
{C_text("f26679:scrutinizer_2escm"),(void*)f26679},
{C_text("f26686:scrutinizer_2escm"),(void*)f26686},
{C_text("f26693:scrutinizer_2escm"),(void*)f26693},
{C_text("f26700:scrutinizer_2escm"),(void*)f26700},
{C_text("f26707:scrutinizer_2escm"),(void*)f26707},
{C_text("f27984:scrutinizer_2escm"),(void*)f27984},
{C_text("f27989:scrutinizer_2escm"),(void*)f27989},
{C_text("f28008:scrutinizer_2escm"),(void*)f28008},
{C_text("f28236:scrutinizer_2escm"),(void*)f28236},
{C_text("f28241:scrutinizer_2escm"),(void*)f28241},
{C_text("f28246:scrutinizer_2escm"),(void*)f28246},
{C_text("f_10003:scrutinizer_2escm"),(void*)f_10003},
{C_text("f_10013:scrutinizer_2escm"),(void*)f_10013},
{C_text("f_10017:scrutinizer_2escm"),(void*)f_10017},
{C_text("f_10021:scrutinizer_2escm"),(void*)f_10021},
{C_text("f_10023:scrutinizer_2escm"),(void*)f_10023},
{C_text("f_10045:scrutinizer_2escm"),(void*)f_10045},
{C_text("f_10052:scrutinizer_2escm"),(void*)f_10052},
{C_text("f_10055:scrutinizer_2escm"),(void*)f_10055},
{C_text("f_10058:scrutinizer_2escm"),(void*)f_10058},
{C_text("f_10066:scrutinizer_2escm"),(void*)f_10066},
{C_text("f_10075:scrutinizer_2escm"),(void*)f_10075},
{C_text("f_10082:scrutinizer_2escm"),(void*)f_10082},
{C_text("f_10089:scrutinizer_2escm"),(void*)f_10089},
{C_text("f_10097:scrutinizer_2escm"),(void*)f_10097},
{C_text("f_10099:scrutinizer_2escm"),(void*)f_10099},
{C_text("f_10107:scrutinizer_2escm"),(void*)f_10107},
{C_text("f_10111:scrutinizer_2escm"),(void*)f_10111},
{C_text("f_10123:scrutinizer_2escm"),(void*)f_10123},
{C_text("f_10158:scrutinizer_2escm"),(void*)f_10158},
{C_text("f_10180:scrutinizer_2escm"),(void*)f_10180},
{C_text("f_10192:scrutinizer_2escm"),(void*)f_10192},
{C_text("f_10205:scrutinizer_2escm"),(void*)f_10205},
{C_text("f_10227:scrutinizer_2escm"),(void*)f_10227},
{C_text("f_10236:scrutinizer_2escm"),(void*)f_10236},
{C_text("f_10238:scrutinizer_2escm"),(void*)f_10238},
{C_text("f_10309:scrutinizer_2escm"),(void*)f_10309},
{C_text("f_10312:scrutinizer_2escm"),(void*)f_10312},
{C_text("f_10315:scrutinizer_2escm"),(void*)f_10315},
{C_text("f_10321:scrutinizer_2escm"),(void*)f_10321},
{C_text("f_10324:scrutinizer_2escm"),(void*)f_10324},
{C_text("f_10327:scrutinizer_2escm"),(void*)f_10327},
{C_text("f_10333:scrutinizer_2escm"),(void*)f_10333},
{C_text("f_10347:scrutinizer_2escm"),(void*)f_10347},
{C_text("f_10355:scrutinizer_2escm"),(void*)f_10355},
{C_text("f_10389:scrutinizer_2escm"),(void*)f_10389},
{C_text("f_10438:scrutinizer_2escm"),(void*)f_10438},
{C_text("f_10453:scrutinizer_2escm"),(void*)f_10453},
{C_text("f_10456:scrutinizer_2escm"),(void*)f_10456},
{C_text("f_10459:scrutinizer_2escm"),(void*)f_10459},
{C_text("f_10468:scrutinizer_2escm"),(void*)f_10468},
{C_text("f_10474:scrutinizer_2escm"),(void*)f_10474},
{C_text("f_10480:scrutinizer_2escm"),(void*)f_10480},
{C_text("f_10483:scrutinizer_2escm"),(void*)f_10483},
{C_text("f_10490:scrutinizer_2escm"),(void*)f_10490},
{C_text("f_10496:scrutinizer_2escm"),(void*)f_10496},
{C_text("f_10499:scrutinizer_2escm"),(void*)f_10499},
{C_text("f_10502:scrutinizer_2escm"),(void*)f_10502},
{C_text("f_10505:scrutinizer_2escm"),(void*)f_10505},
{C_text("f_10508:scrutinizer_2escm"),(void*)f_10508},
{C_text("f_10511:scrutinizer_2escm"),(void*)f_10511},
{C_text("f_10519:scrutinizer_2escm"),(void*)f_10519},
{C_text("f_10539:scrutinizer_2escm"),(void*)f_10539},
{C_text("f_10543:scrutinizer_2escm"),(void*)f_10543},
{C_text("f_10572:scrutinizer_2escm"),(void*)f_10572},
{C_text("f_10579:scrutinizer_2escm"),(void*)f_10579},
{C_text("f_10582:scrutinizer_2escm"),(void*)f_10582},
{C_text("f_10602:scrutinizer_2escm"),(void*)f_10602},
{C_text("f_10609:scrutinizer_2escm"),(void*)f_10609},
{C_text("f_10631:scrutinizer_2escm"),(void*)f_10631},
{C_text("f_10635:scrutinizer_2escm"),(void*)f_10635},
{C_text("f_10642:scrutinizer_2escm"),(void*)f_10642},
{C_text("f_10644:scrutinizer_2escm"),(void*)f_10644},
{C_text("f_10671:scrutinizer_2escm"),(void*)f_10671},
{C_text("f_10678:scrutinizer_2escm"),(void*)f_10678},
{C_text("f_10694:scrutinizer_2escm"),(void*)f_10694},
{C_text("f_10698:scrutinizer_2escm"),(void*)f_10698},
{C_text("f_10700:scrutinizer_2escm"),(void*)f_10700},
{C_text("f_10707:scrutinizer_2escm"),(void*)f_10707},
{C_text("f_10713:scrutinizer_2escm"),(void*)f_10713},
{C_text("f_10719:scrutinizer_2escm"),(void*)f_10719},
{C_text("f_10722:scrutinizer_2escm"),(void*)f_10722},
{C_text("f_10725:scrutinizer_2escm"),(void*)f_10725},
{C_text("f_10728:scrutinizer_2escm"),(void*)f_10728},
{C_text("f_10733:scrutinizer_2escm"),(void*)f_10733},
{C_text("f_10739:scrutinizer_2escm"),(void*)f_10739},
{C_text("f_10742:scrutinizer_2escm"),(void*)f_10742},
{C_text("f_10749:scrutinizer_2escm"),(void*)f_10749},
{C_text("f_10752:scrutinizer_2escm"),(void*)f_10752},
{C_text("f_10777:scrutinizer_2escm"),(void*)f_10777},
{C_text("f_10781:scrutinizer_2escm"),(void*)f_10781},
{C_text("f_10787:scrutinizer_2escm"),(void*)f_10787},
{C_text("f_10790:scrutinizer_2escm"),(void*)f_10790},
{C_text("f_10800:scrutinizer_2escm"),(void*)f_10800},
{C_text("f_10812:scrutinizer_2escm"),(void*)f_10812},
{C_text("f_10837:scrutinizer_2escm"),(void*)f_10837},
{C_text("f_10854:scrutinizer_2escm"),(void*)f_10854},
{C_text("f_10867:scrutinizer_2escm"),(void*)f_10867},
{C_text("f_10870:scrutinizer_2escm"),(void*)f_10870},
{C_text("f_10873:scrutinizer_2escm"),(void*)f_10873},
{C_text("f_10879:scrutinizer_2escm"),(void*)f_10879},
{C_text("f_10885:scrutinizer_2escm"),(void*)f_10885},
{C_text("f_10891:scrutinizer_2escm"),(void*)f_10891},
{C_text("f_10897:scrutinizer_2escm"),(void*)f_10897},
{C_text("f_10920:scrutinizer_2escm"),(void*)f_10920},
{C_text("f_10935:scrutinizer_2escm"),(void*)f_10935},
{C_text("f_10941:scrutinizer_2escm"),(void*)f_10941},
{C_text("f_10947:scrutinizer_2escm"),(void*)f_10947},
{C_text("f_10963:scrutinizer_2escm"),(void*)f_10963},
{C_text("f_10969:scrutinizer_2escm"),(void*)f_10969},
{C_text("f_10973:scrutinizer_2escm"),(void*)f_10973},
{C_text("f_10981:scrutinizer_2escm"),(void*)f_10981},
{C_text("f_11003:scrutinizer_2escm"),(void*)f_11003},
{C_text("f_11014:scrutinizer_2escm"),(void*)f_11014},
{C_text("f_11018:scrutinizer_2escm"),(void*)f_11018},
{C_text("f_11027:scrutinizer_2escm"),(void*)f_11027},
{C_text("f_11030:scrutinizer_2escm"),(void*)f_11030},
{C_text("f_11040:scrutinizer_2escm"),(void*)f_11040},
{C_text("f_11052:scrutinizer_2escm"),(void*)f_11052},
{C_text("f_11077:scrutinizer_2escm"),(void*)f_11077},
{C_text("f_11086:scrutinizer_2escm"),(void*)f_11086},
{C_text("f_11096:scrutinizer_2escm"),(void*)f_11096},
{C_text("f_11125:scrutinizer_2escm"),(void*)f_11125},
{C_text("f_11127:scrutinizer_2escm"),(void*)f_11127},
{C_text("f_11135:scrutinizer_2escm"),(void*)f_11135},
{C_text("f_11143:scrutinizer_2escm"),(void*)f_11143},
{C_text("f_11160:scrutinizer_2escm"),(void*)f_11160},
{C_text("f_11162:scrutinizer_2escm"),(void*)f_11162},
{C_text("f_11200:scrutinizer_2escm"),(void*)f_11200},
{C_text("f_11229:scrutinizer_2escm"),(void*)f_11229},
{C_text("f_11258:scrutinizer_2escm"),(void*)f_11258},
{C_text("f_11264:scrutinizer_2escm"),(void*)f_11264},
{C_text("f_11283:scrutinizer_2escm"),(void*)f_11283},
{C_text("f_11289:scrutinizer_2escm"),(void*)f_11289},
{C_text("f_11292:scrutinizer_2escm"),(void*)f_11292},
{C_text("f_11314:scrutinizer_2escm"),(void*)f_11314},
{C_text("f_11346:scrutinizer_2escm"),(void*)f_11346},
{C_text("f_11349:scrutinizer_2escm"),(void*)f_11349},
{C_text("f_11362:scrutinizer_2escm"),(void*)f_11362},
{C_text("f_11383:scrutinizer_2escm"),(void*)f_11383},
{C_text("f_11389:scrutinizer_2escm"),(void*)f_11389},
{C_text("f_11392:scrutinizer_2escm"),(void*)f_11392},
{C_text("f_11402:scrutinizer_2escm"),(void*)f_11402},
{C_text("f_11421:scrutinizer_2escm"),(void*)f_11421},
{C_text("f_11446:scrutinizer_2escm"),(void*)f_11446},
{C_text("f_11456:scrutinizer_2escm"),(void*)f_11456},
{C_text("f_11458:scrutinizer_2escm"),(void*)f_11458},
{C_text("f_11468:scrutinizer_2escm"),(void*)f_11468},
{C_text("f_11529:scrutinizer_2escm"),(void*)f_11529},
{C_text("f_11532:scrutinizer_2escm"),(void*)f_11532},
{C_text("f_11535:scrutinizer_2escm"),(void*)f_11535},
{C_text("f_11538:scrutinizer_2escm"),(void*)f_11538},
{C_text("f_11541:scrutinizer_2escm"),(void*)f_11541},
{C_text("f_11547:scrutinizer_2escm"),(void*)f_11547},
{C_text("f_11570:scrutinizer_2escm"),(void*)f_11570},
{C_text("f_11574:scrutinizer_2escm"),(void*)f_11574},
{C_text("f_11582:scrutinizer_2escm"),(void*)f_11582},
{C_text("f_11585:scrutinizer_2escm"),(void*)f_11585},
{C_text("f_11588:scrutinizer_2escm"),(void*)f_11588},
{C_text("f_11591:scrutinizer_2escm"),(void*)f_11591},
{C_text("f_11608:scrutinizer_2escm"),(void*)f_11608},
{C_text("f_11618:scrutinizer_2escm"),(void*)f_11618},
{C_text("f_11645:scrutinizer_2escm"),(void*)f_11645},
{C_text("f_11647:scrutinizer_2escm"),(void*)f_11647},
{C_text("f_11653:scrutinizer_2escm"),(void*)f_11653},
{C_text("f_11663:scrutinizer_2escm"),(void*)f_11663},
{C_text("f_11675:scrutinizer_2escm"),(void*)f_11675},
{C_text("f_11683:scrutinizer_2escm"),(void*)f_11683},
{C_text("f_11703:scrutinizer_2escm"),(void*)f_11703},
{C_text("f_11716:scrutinizer_2escm"),(void*)f_11716},
{C_text("f_11728:scrutinizer_2escm"),(void*)f_11728},
{C_text("f_11735:scrutinizer_2escm"),(void*)f_11735},
{C_text("f_11737:scrutinizer_2escm"),(void*)f_11737},
{C_text("f_11762:scrutinizer_2escm"),(void*)f_11762},
{C_text("f_11784:scrutinizer_2escm"),(void*)f_11784},
{C_text("f_11797:scrutinizer_2escm"),(void*)f_11797},
{C_text("f_11818:scrutinizer_2escm"),(void*)f_11818},
{C_text("f_11843:scrutinizer_2escm"),(void*)f_11843},
{C_text("f_11847:scrutinizer_2escm"),(void*)f_11847},
{C_text("f_11858:scrutinizer_2escm"),(void*)f_11858},
{C_text("f_11867:scrutinizer_2escm"),(void*)f_11867},
{C_text("f_11964:scrutinizer_2escm"),(void*)f_11964},
{C_text("f_11989:scrutinizer_2escm"),(void*)f_11989},
{C_text("f_11995:scrutinizer_2escm"),(void*)f_11995},
{C_text("f_12011:scrutinizer_2escm"),(void*)f_12011},
{C_text("f_12018:scrutinizer_2escm"),(void*)f_12018},
{C_text("f_12035:scrutinizer_2escm"),(void*)f_12035},
{C_text("f_12052:scrutinizer_2escm"),(void*)f_12052},
{C_text("f_12095:scrutinizer_2escm"),(void*)f_12095},
{C_text("f_12120:scrutinizer_2escm"),(void*)f_12120},
{C_text("f_12126:scrutinizer_2escm"),(void*)f_12126},
{C_text("f_12131:scrutinizer_2escm"),(void*)f_12131},
{C_text("f_12137:scrutinizer_2escm"),(void*)f_12137},
{C_text("f_12142:scrutinizer_2escm"),(void*)f_12142},
{C_text("f_12148:scrutinizer_2escm"),(void*)f_12148},
{C_text("f_12161:scrutinizer_2escm"),(void*)f_12161},
{C_text("f_12178:scrutinizer_2escm"),(void*)f_12178},
{C_text("f_12195:scrutinizer_2escm"),(void*)f_12195},
{C_text("f_12208:scrutinizer_2escm"),(void*)f_12208},
{C_text("f_12236:scrutinizer_2escm"),(void*)f_12236},
{C_text("f_12249:scrutinizer_2escm"),(void*)f_12249},
{C_text("f_12286:scrutinizer_2escm"),(void*)f_12286},
{C_text("f_12327:scrutinizer_2escm"),(void*)f_12327},
{C_text("f_12340:scrutinizer_2escm"),(void*)f_12340},
{C_text("f_12370:scrutinizer_2escm"),(void*)f_12370},
{C_text("f_12440:scrutinizer_2escm"),(void*)f_12440},
{C_text("f_12444:scrutinizer_2escm"),(void*)f_12444},
{C_text("f_12454:scrutinizer_2escm"),(void*)f_12454},
{C_text("f_12458:scrutinizer_2escm"),(void*)f_12458},
{C_text("f_12479:scrutinizer_2escm"),(void*)f_12479},
{C_text("f_12484:scrutinizer_2escm"),(void*)f_12484},
{C_text("f_12488:scrutinizer_2escm"),(void*)f_12488},
{C_text("f_12491:scrutinizer_2escm"),(void*)f_12491},
{C_text("f_12500:scrutinizer_2escm"),(void*)f_12500},
{C_text("f_12520:scrutinizer_2escm"),(void*)f_12520},
{C_text("f_12525:scrutinizer_2escm"),(void*)f_12525},
{C_text("f_12529:scrutinizer_2escm"),(void*)f_12529},
{C_text("f_12532:scrutinizer_2escm"),(void*)f_12532},
{C_text("f_12541:scrutinizer_2escm"),(void*)f_12541},
{C_text("f_12559:scrutinizer_2escm"),(void*)f_12559},
{C_text("f_12578:scrutinizer_2escm"),(void*)f_12578},
{C_text("f_12694:scrutinizer_2escm"),(void*)f_12694},
{C_text("f_12706:scrutinizer_2escm"),(void*)f_12706},
{C_text("f_12709:scrutinizer_2escm"),(void*)f_12709},
{C_text("f_12712:scrutinizer_2escm"),(void*)f_12712},
{C_text("f_12715:scrutinizer_2escm"),(void*)f_12715},
{C_text("f_12721:scrutinizer_2escm"),(void*)f_12721},
{C_text("f_12814:scrutinizer_2escm"),(void*)f_12814},
{C_text("f_12924:scrutinizer_2escm"),(void*)f_12924},
{C_text("f_12957:scrutinizer_2escm"),(void*)f_12957},
{C_text("f_13026:scrutinizer_2escm"),(void*)f_13026},
{C_text("f_13059:scrutinizer_2escm"),(void*)f_13059},
{C_text("f_13133:scrutinizer_2escm"),(void*)f_13133},
{C_text("f_13167:scrutinizer_2escm"),(void*)f_13167},
{C_text("f_13209:scrutinizer_2escm"),(void*)f_13209},
{C_text("f_13243:scrutinizer_2escm"),(void*)f_13243},
{C_text("f_13431:scrutinizer_2escm"),(void*)f_13431},
{C_text("f_13437:scrutinizer_2escm"),(void*)f_13437},
{C_text("f_13470:scrutinizer_2escm"),(void*)f_13470},
{C_text("f_13484:scrutinizer_2escm"),(void*)f_13484},
{C_text("f_13503:scrutinizer_2escm"),(void*)f_13503},
{C_text("f_13506:scrutinizer_2escm"),(void*)f_13506},
{C_text("f_13512:scrutinizer_2escm"),(void*)f_13512},
{C_text("f_13533:scrutinizer_2escm"),(void*)f_13533},
{C_text("f_13537:scrutinizer_2escm"),(void*)f_13537},
{C_text("f_13548:scrutinizer_2escm"),(void*)f_13548},
{C_text("f_13550:scrutinizer_2escm"),(void*)f_13550},
{C_text("f_13585:scrutinizer_2escm"),(void*)f_13585},
{C_text("f_13587:scrutinizer_2escm"),(void*)f_13587},
{C_text("f_13628:scrutinizer_2escm"),(void*)f_13628},
{C_text("f_13656:scrutinizer_2escm"),(void*)f_13656},
{C_text("f_13662:scrutinizer_2escm"),(void*)f_13662},
{C_text("f_13674:scrutinizer_2escm"),(void*)f_13674},
{C_text("f_13694:scrutinizer_2escm"),(void*)f_13694},
{C_text("f_13697:scrutinizer_2escm"),(void*)f_13697},
{C_text("f_13700:scrutinizer_2escm"),(void*)f_13700},
{C_text("f_13703:scrutinizer_2escm"),(void*)f_13703},
{C_text("f_13706:scrutinizer_2escm"),(void*)f_13706},
{C_text("f_13709:scrutinizer_2escm"),(void*)f_13709},
{C_text("f_13716:scrutinizer_2escm"),(void*)f_13716},
{C_text("f_13724:scrutinizer_2escm"),(void*)f_13724},
{C_text("f_13728:scrutinizer_2escm"),(void*)f_13728},
{C_text("f_13743:scrutinizer_2escm"),(void*)f_13743},
{C_text("f_13749:scrutinizer_2escm"),(void*)f_13749},
{C_text("f_13759:scrutinizer_2escm"),(void*)f_13759},
{C_text("f_13765:scrutinizer_2escm"),(void*)f_13765},
{C_text("f_13771:scrutinizer_2escm"),(void*)f_13771},
{C_text("f_13774:scrutinizer_2escm"),(void*)f_13774},
{C_text("f_13777:scrutinizer_2escm"),(void*)f_13777},
{C_text("f_13794:scrutinizer_2escm"),(void*)f_13794},
{C_text("f_13819:scrutinizer_2escm"),(void*)f_13819},
{C_text("f_13830:scrutinizer_2escm"),(void*)f_13830},
{C_text("f_13848:scrutinizer_2escm"),(void*)f_13848},
{C_text("f_13858:scrutinizer_2escm"),(void*)f_13858},
{C_text("f_13874:scrutinizer_2escm"),(void*)f_13874},
{C_text("f_13878:scrutinizer_2escm"),(void*)f_13878},
{C_text("f_13936:scrutinizer_2escm"),(void*)f_13936},
{C_text("f_13938:scrutinizer_2escm"),(void*)f_13938},
{C_text("f_13963:scrutinizer_2escm"),(void*)f_13963},
{C_text("f_13982:scrutinizer_2escm"),(void*)f_13982},
{C_text("f_14001:scrutinizer_2escm"),(void*)f_14001},
{C_text("f_14016:scrutinizer_2escm"),(void*)f_14016},
{C_text("f_14029:scrutinizer_2escm"),(void*)f_14029},
{C_text("f_14032:scrutinizer_2escm"),(void*)f_14032},
{C_text("f_14068:scrutinizer_2escm"),(void*)f_14068},
{C_text("f_14092:scrutinizer_2escm"),(void*)f_14092},
{C_text("f_14133:scrutinizer_2escm"),(void*)f_14133},
{C_text("f_14135:scrutinizer_2escm"),(void*)f_14135},
{C_text("f_14160:scrutinizer_2escm"),(void*)f_14160},
{C_text("f_14189:scrutinizer_2escm"),(void*)f_14189},
{C_text("f_14191:scrutinizer_2escm"),(void*)f_14191},
{C_text("f_14216:scrutinizer_2escm"),(void*)f_14216},
{C_text("f_14250:scrutinizer_2escm"),(void*)f_14250},
{C_text("f_14264:scrutinizer_2escm"),(void*)f_14264},
{C_text("f_14289:scrutinizer_2escm"),(void*)f_14289},
{C_text("f_14309:scrutinizer_2escm"),(void*)f_14309},
{C_text("f_14311:scrutinizer_2escm"),(void*)f_14311},
{C_text("f_14336:scrutinizer_2escm"),(void*)f_14336},
{C_text("f_14389:scrutinizer_2escm"),(void*)f_14389},
{C_text("f_14394:scrutinizer_2escm"),(void*)f_14394},
{C_text("f_14403:scrutinizer_2escm"),(void*)f_14403},
{C_text("f_14420:scrutinizer_2escm"),(void*)f_14420},
{C_text("f_14422:scrutinizer_2escm"),(void*)f_14422},
{C_text("f_14437:scrutinizer_2escm"),(void*)f_14437},
{C_text("f_14445:scrutinizer_2escm"),(void*)f_14445},
{C_text("f_14472:scrutinizer_2escm"),(void*)f_14472},
{C_text("f_14518:scrutinizer_2escm"),(void*)f_14518},
{C_text("f_14567:scrutinizer_2escm"),(void*)f_14567},
{C_text("f_14571:scrutinizer_2escm"),(void*)f_14571},
{C_text("f_14617:scrutinizer_2escm"),(void*)f_14617},
{C_text("f_14621:scrutinizer_2escm"),(void*)f_14621},
{C_text("f_14647:scrutinizer_2escm"),(void*)f_14647},
{C_text("f_14653:scrutinizer_2escm"),(void*)f_14653},
{C_text("f_14679:scrutinizer_2escm"),(void*)f_14679},
{C_text("f_14683:scrutinizer_2escm"),(void*)f_14683},
{C_text("f_14710:scrutinizer_2escm"),(void*)f_14710},
{C_text("f_14714:scrutinizer_2escm"),(void*)f_14714},
{C_text("f_14717:scrutinizer_2escm"),(void*)f_14717},
{C_text("f_14742:scrutinizer_2escm"),(void*)f_14742},
{C_text("f_14748:scrutinizer_2escm"),(void*)f_14748},
{C_text("f_14754:scrutinizer_2escm"),(void*)f_14754},
{C_text("f_14769:scrutinizer_2escm"),(void*)f_14769},
{C_text("f_14773:scrutinizer_2escm"),(void*)f_14773},
{C_text("f_14778:scrutinizer_2escm"),(void*)f_14778},
{C_text("f_14797:scrutinizer_2escm"),(void*)f_14797},
{C_text("f_14806:scrutinizer_2escm"),(void*)f_14806},
{C_text("f_14810:scrutinizer_2escm"),(void*)f_14810},
{C_text("f_14814:scrutinizer_2escm"),(void*)f_14814},
{C_text("f_14824:scrutinizer_2escm"),(void*)f_14824},
{C_text("f_14828:scrutinizer_2escm"),(void*)f_14828},
{C_text("f_14844:scrutinizer_2escm"),(void*)f_14844},
{C_text("f_14871:scrutinizer_2escm"),(void*)f_14871},
{C_text("f_14895:scrutinizer_2escm"),(void*)f_14895},
{C_text("f_14904:scrutinizer_2escm"),(void*)f_14904},
{C_text("f_14915:scrutinizer_2escm"),(void*)f_14915},
{C_text("f_14929:scrutinizer_2escm"),(void*)f_14929},
{C_text("f_14947:scrutinizer_2escm"),(void*)f_14947},
{C_text("f_14953:scrutinizer_2escm"),(void*)f_14953},
{C_text("f_14960:scrutinizer_2escm"),(void*)f_14960},
{C_text("f_14989:scrutinizer_2escm"),(void*)f_14989},
{C_text("f_15068:scrutinizer_2escm"),(void*)f_15068},
{C_text("f_15098:scrutinizer_2escm"),(void*)f_15098},
{C_text("f_15101:scrutinizer_2escm"),(void*)f_15101},
{C_text("f_15119:scrutinizer_2escm"),(void*)f_15119},
{C_text("f_15171:scrutinizer_2escm"),(void*)f_15171},
{C_text("f_15222:scrutinizer_2escm"),(void*)f_15222},
{C_text("f_15279:scrutinizer_2escm"),(void*)f_15279},
{C_text("f_15336:scrutinizer_2escm"),(void*)f_15336},
{C_text("f_15345:scrutinizer_2escm"),(void*)f_15345},
{C_text("f_15355:scrutinizer_2escm"),(void*)f_15355},
{C_text("f_15362:scrutinizer_2escm"),(void*)f_15362},
{C_text("f_15364:scrutinizer_2escm"),(void*)f_15364},
{C_text("f_15443:scrutinizer_2escm"),(void*)f_15443},
{C_text("f_15499:scrutinizer_2escm"),(void*)f_15499},
{C_text("f_15532:scrutinizer_2escm"),(void*)f_15532},
{C_text("f_15559:scrutinizer_2escm"),(void*)f_15559},
{C_text("f_15570:scrutinizer_2escm"),(void*)f_15570},
{C_text("f_15592:scrutinizer_2escm"),(void*)f_15592},
{C_text("f_15600:scrutinizer_2escm"),(void*)f_15600},
{C_text("f_15602:scrutinizer_2escm"),(void*)f_15602},
{C_text("f_15628:scrutinizer_2escm"),(void*)f_15628},
{C_text("f_15687:scrutinizer_2escm"),(void*)f_15687},
{C_text("f_15740:scrutinizer_2escm"),(void*)f_15740},
{C_text("f_15760:scrutinizer_2escm"),(void*)f_15760},
{C_text("f_15819:scrutinizer_2escm"),(void*)f_15819},
{C_text("f_15863:scrutinizer_2escm"),(void*)f_15863},
{C_text("f_15902:scrutinizer_2escm"),(void*)f_15902},
{C_text("f_15905:scrutinizer_2escm"),(void*)f_15905},
{C_text("f_15910:scrutinizer_2escm"),(void*)f_15910},
{C_text("f_15914:scrutinizer_2escm"),(void*)f_15914},
{C_text("f_15916:scrutinizer_2escm"),(void*)f_15916},
{C_text("f_15961:scrutinizer_2escm"),(void*)f_15961},
{C_text("f_15978:scrutinizer_2escm"),(void*)f_15978},
{C_text("f_15988:scrutinizer_2escm"),(void*)f_15988},
{C_text("f_16005:scrutinizer_2escm"),(void*)f_16005},
{C_text("f_16015:scrutinizer_2escm"),(void*)f_16015},
{C_text("f_16035:scrutinizer_2escm"),(void*)f_16035},
{C_text("f_16051:scrutinizer_2escm"),(void*)f_16051},
{C_text("f_16061:scrutinizer_2escm"),(void*)f_16061},
{C_text("f_16078:scrutinizer_2escm"),(void*)f_16078},
{C_text("f_16088:scrutinizer_2escm"),(void*)f_16088},
{C_text("f_16120:scrutinizer_2escm"),(void*)f_16120},
{C_text("f_16162:scrutinizer_2escm"),(void*)f_16162},
{C_text("f_16164:scrutinizer_2escm"),(void*)f_16164},
{C_text("f_16189:scrutinizer_2escm"),(void*)f_16189},
{C_text("f_16211:scrutinizer_2escm"),(void*)f_16211},
{C_text("f_16221:scrutinizer_2escm"),(void*)f_16221},
{C_text("f_16242:scrutinizer_2escm"),(void*)f_16242},
{C_text("f_16248:scrutinizer_2escm"),(void*)f_16248},
{C_text("f_16280:scrutinizer_2escm"),(void*)f_16280},
{C_text("f_16283:scrutinizer_2escm"),(void*)f_16283},
{C_text("f_16285:scrutinizer_2escm"),(void*)f_16285},
{C_text("f_16293:scrutinizer_2escm"),(void*)f_16293},
{C_text("f_16295:scrutinizer_2escm"),(void*)f_16295},
{C_text("f_16303:scrutinizer_2escm"),(void*)f_16303},
{C_text("f_16384:scrutinizer_2escm"),(void*)f_16384},
{C_text("f_16395:scrutinizer_2escm"),(void*)f_16395},
{C_text("f_16397:scrutinizer_2escm"),(void*)f_16397},
{C_text("f_16422:scrutinizer_2escm"),(void*)f_16422},
{C_text("f_16442:scrutinizer_2escm"),(void*)f_16442},
{C_text("f_16468:scrutinizer_2escm"),(void*)f_16468},
{C_text("f_16481:scrutinizer_2escm"),(void*)f_16481},
{C_text("f_16491:scrutinizer_2escm"),(void*)f_16491},
{C_text("f_16502:scrutinizer_2escm"),(void*)f_16502},
{C_text("f_16504:scrutinizer_2escm"),(void*)f_16504},
{C_text("f_16529:scrutinizer_2escm"),(void*)f_16529},
{C_text("f_16545:scrutinizer_2escm"),(void*)f_16545},
{C_text("f_16548:scrutinizer_2escm"),(void*)f_16548},
{C_text("f_16551:scrutinizer_2escm"),(void*)f_16551},
{C_text("f_16558:scrutinizer_2escm"),(void*)f_16558},
{C_text("f_16570:scrutinizer_2escm"),(void*)f_16570},
{C_text("f_16574:scrutinizer_2escm"),(void*)f_16574},
{C_text("f_16581:scrutinizer_2escm"),(void*)f_16581},
{C_text("f_16593:scrutinizer_2escm"),(void*)f_16593},
{C_text("f_16618:scrutinizer_2escm"),(void*)f_16618},
{C_text("f_16627:scrutinizer_2escm"),(void*)f_16627},
{C_text("f_16655:scrutinizer_2escm"),(void*)f_16655},
{C_text("f_16674:scrutinizer_2escm"),(void*)f_16674},
{C_text("f_16683:scrutinizer_2escm"),(void*)f_16683},
{C_text("f_16687:scrutinizer_2escm"),(void*)f_16687},
{C_text("f_16750:scrutinizer_2escm"),(void*)f_16750},
{C_text("f_16754:scrutinizer_2escm"),(void*)f_16754},
{C_text("f_16757:scrutinizer_2escm"),(void*)f_16757},
{C_text("f_16763:scrutinizer_2escm"),(void*)f_16763},
{C_text("f_16768:scrutinizer_2escm"),(void*)f_16768},
{C_text("f_16773:scrutinizer_2escm"),(void*)f_16773},
{C_text("f_16782:scrutinizer_2escm"),(void*)f_16782},
{C_text("f_16788:scrutinizer_2escm"),(void*)f_16788},
{C_text("f_16793:scrutinizer_2escm"),(void*)f_16793},
{C_text("f_16799:scrutinizer_2escm"),(void*)f_16799},
{C_text("f_16803:scrutinizer_2escm"),(void*)f_16803},
{C_text("f_16806:scrutinizer_2escm"),(void*)f_16806},
{C_text("f_16809:scrutinizer_2escm"),(void*)f_16809},
{C_text("f_16812:scrutinizer_2escm"),(void*)f_16812},
{C_text("f_16821:scrutinizer_2escm"),(void*)f_16821},
{C_text("f_16828:scrutinizer_2escm"),(void*)f_16828},
{C_text("f_16832:scrutinizer_2escm"),(void*)f_16832},
{C_text("f_16836:scrutinizer_2escm"),(void*)f_16836},
{C_text("f_16840:scrutinizer_2escm"),(void*)f_16840},
{C_text("f_16847:scrutinizer_2escm"),(void*)f_16847},
{C_text("f_16854:scrutinizer_2escm"),(void*)f_16854},
{C_text("f_16860:scrutinizer_2escm"),(void*)f_16860},
{C_text("f_16863:scrutinizer_2escm"),(void*)f_16863},
{C_text("f_16866:scrutinizer_2escm"),(void*)f_16866},
{C_text("f_16869:scrutinizer_2escm"),(void*)f_16869},
{C_text("f_16872:scrutinizer_2escm"),(void*)f_16872},
{C_text("f_16875:scrutinizer_2escm"),(void*)f_16875},
{C_text("f_16878:scrutinizer_2escm"),(void*)f_16878},
{C_text("f_16882:scrutinizer_2escm"),(void*)f_16882},
{C_text("f_16887:scrutinizer_2escm"),(void*)f_16887},
{C_text("f_16903:scrutinizer_2escm"),(void*)f_16903},
{C_text("f_16914:scrutinizer_2escm"),(void*)f_16914},
{C_text("f_16933:scrutinizer_2escm"),(void*)f_16933},
{C_text("f_16947:scrutinizer_2escm"),(void*)f_16947},
{C_text("f_16961:scrutinizer_2escm"),(void*)f_16961},
{C_text("f_16975:scrutinizer_2escm"),(void*)f_16975},
{C_text("f_16989:scrutinizer_2escm"),(void*)f_16989},
{C_text("f_17019:scrutinizer_2escm"),(void*)f_17019},
{C_text("f_17038:scrutinizer_2escm"),(void*)f_17038},
{C_text("f_17070:scrutinizer_2escm"),(void*)f_17070},
{C_text("f_17076:scrutinizer_2escm"),(void*)f_17076},
{C_text("f_17078:scrutinizer_2escm"),(void*)f_17078},
{C_text("f_17088:scrutinizer_2escm"),(void*)f_17088},
{C_text("f_17101:scrutinizer_2escm"),(void*)f_17101},
{C_text("f_17108:scrutinizer_2escm"),(void*)f_17108},
{C_text("f_17114:scrutinizer_2escm"),(void*)f_17114},
{C_text("f_17117:scrutinizer_2escm"),(void*)f_17117},
{C_text("f_17120:scrutinizer_2escm"),(void*)f_17120},
{C_text("f_17123:scrutinizer_2escm"),(void*)f_17123},
{C_text("f_17126:scrutinizer_2escm"),(void*)f_17126},
{C_text("f_17135:scrutinizer_2escm"),(void*)f_17135},
{C_text("f_17149:scrutinizer_2escm"),(void*)f_17149},
{C_text("f_17155:scrutinizer_2escm"),(void*)f_17155},
{C_text("f_17159:scrutinizer_2escm"),(void*)f_17159},
{C_text("f_17162:scrutinizer_2escm"),(void*)f_17162},
{C_text("f_17167:scrutinizer_2escm"),(void*)f_17167},
{C_text("f_17174:scrutinizer_2escm"),(void*)f_17174},
{C_text("f_17177:scrutinizer_2escm"),(void*)f_17177},
{C_text("f_17183:scrutinizer_2escm"),(void*)f_17183},
{C_text("f_17186:scrutinizer_2escm"),(void*)f_17186},
{C_text("f_17189:scrutinizer_2escm"),(void*)f_17189},
{C_text("f_17192:scrutinizer_2escm"),(void*)f_17192},
{C_text("f_17195:scrutinizer_2escm"),(void*)f_17195},
{C_text("f_17198:scrutinizer_2escm"),(void*)f_17198},
{C_text("f_17201:scrutinizer_2escm"),(void*)f_17201},
{C_text("f_17208:scrutinizer_2escm"),(void*)f_17208},
{C_text("f_17212:scrutinizer_2escm"),(void*)f_17212},
{C_text("f_17214:scrutinizer_2escm"),(void*)f_17214},
{C_text("f_17235:scrutinizer_2escm"),(void*)f_17235},
{C_text("f_17245:scrutinizer_2escm"),(void*)f_17245},
{C_text("f_17253:scrutinizer_2escm"),(void*)f_17253},
{C_text("f_17261:scrutinizer_2escm"),(void*)f_17261},
{C_text("f_17269:scrutinizer_2escm"),(void*)f_17269},
{C_text("f_17297:scrutinizer_2escm"),(void*)f_17297},
{C_text("f_17308:scrutinizer_2escm"),(void*)f_17308},
{C_text("f_17312:scrutinizer_2escm"),(void*)f_17312},
{C_text("f_17317:scrutinizer_2escm"),(void*)f_17317},
{C_text("f_17320:scrutinizer_2escm"),(void*)f_17320},
{C_text("f_17325:scrutinizer_2escm"),(void*)f_17325},
{C_text("f_17332:scrutinizer_2escm"),(void*)f_17332},
{C_text("f_17336:scrutinizer_2escm"),(void*)f_17336},
{C_text("f_17338:scrutinizer_2escm"),(void*)f_17338},
{C_text("f_17345:scrutinizer_2escm"),(void*)f_17345},
{C_text("f_17365:scrutinizer_2escm"),(void*)f_17365},
{C_text("f_17371:scrutinizer_2escm"),(void*)f_17371},
{C_text("f_17379:scrutinizer_2escm"),(void*)f_17379},
{C_text("f_17383:scrutinizer_2escm"),(void*)f_17383},
{C_text("f_17390:scrutinizer_2escm"),(void*)f_17390},
{C_text("f_17396:scrutinizer_2escm"),(void*)f_17396},
{C_text("f_17398:scrutinizer_2escm"),(void*)f_17398},
{C_text("f_17423:scrutinizer_2escm"),(void*)f_17423},
{C_text("f_17440:scrutinizer_2escm"),(void*)f_17440},
{C_text("f_17446:scrutinizer_2escm"),(void*)f_17446},
{C_text("f_17455:scrutinizer_2escm"),(void*)f_17455},
{C_text("f_17494:scrutinizer_2escm"),(void*)f_17494},
{C_text("f_17500:scrutinizer_2escm"),(void*)f_17500},
{C_text("f_17503:scrutinizer_2escm"),(void*)f_17503},
{C_text("f_17550:scrutinizer_2escm"),(void*)f_17550},
{C_text("f_17560:scrutinizer_2escm"),(void*)f_17560},
{C_text("f_17587:scrutinizer_2escm"),(void*)f_17587},
{C_text("f_17591:scrutinizer_2escm"),(void*)f_17591},
{C_text("f_17651:scrutinizer_2escm"),(void*)f_17651},
{C_text("f_17653:scrutinizer_2escm"),(void*)f_17653},
{C_text("f_17662:scrutinizer_2escm"),(void*)f_17662},
{C_text("f_17680:scrutinizer_2escm"),(void*)f_17680},
{C_text("f_17684:scrutinizer_2escm"),(void*)f_17684},
{C_text("f_17690:scrutinizer_2escm"),(void*)f_17690},
{C_text("f_17718:scrutinizer_2escm"),(void*)f_17718},
{C_text("f_17749:scrutinizer_2escm"),(void*)f_17749},
{C_text("f_17781:scrutinizer_2escm"),(void*)f_17781},
{C_text("f_17808:scrutinizer_2escm"),(void*)f_17808},
{C_text("f_17811:scrutinizer_2escm"),(void*)f_17811},
{C_text("f_17848:scrutinizer_2escm"),(void*)f_17848},
{C_text("f_17906:scrutinizer_2escm"),(void*)f_17906},
{C_text("f_17940:scrutinizer_2escm"),(void*)f_17940},
{C_text("f_17972:scrutinizer_2escm"),(void*)f_17972},
{C_text("f_17977:scrutinizer_2escm"),(void*)f_17977},
{C_text("f_17981:scrutinizer_2escm"),(void*)f_17981},
{C_text("f_17992:scrutinizer_2escm"),(void*)f_17992},
{C_text("f_17994:scrutinizer_2escm"),(void*)f_17994},
{C_text("f_18010:scrutinizer_2escm"),(void*)f_18010},
{C_text("f_18013:scrutinizer_2escm"),(void*)f_18013},
{C_text("f_18029:scrutinizer_2escm"),(void*)f_18029},
{C_text("f_18076:scrutinizer_2escm"),(void*)f_18076},
{C_text("f_18078:scrutinizer_2escm"),(void*)f_18078},
{C_text("f_18103:scrutinizer_2escm"),(void*)f_18103},
{C_text("f_18122:scrutinizer_2escm"),(void*)f_18122},
{C_text("f_18126:scrutinizer_2escm"),(void*)f_18126},
{C_text("f_18164:scrutinizer_2escm"),(void*)f_18164},
{C_text("f_18170:scrutinizer_2escm"),(void*)f_18170},
{C_text("f_18175:scrutinizer_2escm"),(void*)f_18175},
{C_text("f_18200:scrutinizer_2escm"),(void*)f_18200},
{C_text("f_18276:scrutinizer_2escm"),(void*)f_18276},
{C_text("f_18306:scrutinizer_2escm"),(void*)f_18306},
{C_text("f_18318:scrutinizer_2escm"),(void*)f_18318},
{C_text("f_18345:scrutinizer_2escm"),(void*)f_18345},
{C_text("f_18348:scrutinizer_2escm"),(void*)f_18348},
{C_text("f_18352:scrutinizer_2escm"),(void*)f_18352},
{C_text("f_18365:scrutinizer_2escm"),(void*)f_18365},
{C_text("f_18368:scrutinizer_2escm"),(void*)f_18368},
{C_text("f_18392:scrutinizer_2escm"),(void*)f_18392},
{C_text("f_18400:scrutinizer_2escm"),(void*)f_18400},
{C_text("f_18404:scrutinizer_2escm"),(void*)f_18404},
{C_text("f_18415:scrutinizer_2escm"),(void*)f_18415},
{C_text("f_18494:scrutinizer_2escm"),(void*)f_18494},
{C_text("f_18536:scrutinizer_2escm"),(void*)f_18536},
{C_text("f_18554:scrutinizer_2escm"),(void*)f_18554},
{C_text("f_18557:scrutinizer_2escm"),(void*)f_18557},
{C_text("f_18561:scrutinizer_2escm"),(void*)f_18561},
{C_text("f_18604:scrutinizer_2escm"),(void*)f_18604},
{C_text("f_18610:scrutinizer_2escm"),(void*)f_18610},
{C_text("f_18615:scrutinizer_2escm"),(void*)f_18615},
{C_text("f_18640:scrutinizer_2escm"),(void*)f_18640},
{C_text("f_18685:scrutinizer_2escm"),(void*)f_18685},
{C_text("f_18694:scrutinizer_2escm"),(void*)f_18694},
{C_text("f_18698:scrutinizer_2escm"),(void*)f_18698},
{C_text("f_18708:scrutinizer_2escm"),(void*)f_18708},
{C_text("f_18740:scrutinizer_2escm"),(void*)f_18740},
{C_text("f_18746:scrutinizer_2escm"),(void*)f_18746},
{C_text("f_18748:scrutinizer_2escm"),(void*)f_18748},
{C_text("f_18773:scrutinizer_2escm"),(void*)f_18773},
{C_text("f_18905:scrutinizer_2escm"),(void*)f_18905},
{C_text("f_18909:scrutinizer_2escm"),(void*)f_18909},
{C_text("f_18913:scrutinizer_2escm"),(void*)f_18913},
{C_text("f_18916:scrutinizer_2escm"),(void*)f_18916},
{C_text("f_18948:scrutinizer_2escm"),(void*)f_18948},
{C_text("f_18952:scrutinizer_2escm"),(void*)f_18952},
{C_text("f_18965:scrutinizer_2escm"),(void*)f_18965},
{C_text("f_18969:scrutinizer_2escm"),(void*)f_18969},
{C_text("f_18972:scrutinizer_2escm"),(void*)f_18972},
{C_text("f_18978:scrutinizer_2escm"),(void*)f_18978},
{C_text("f_18980:scrutinizer_2escm"),(void*)f_18980},
{C_text("f_19005:scrutinizer_2escm"),(void*)f_19005},
{C_text("f_19020:scrutinizer_2escm"),(void*)f_19020},
{C_text("f_19029:scrutinizer_2escm"),(void*)f_19029},
{C_text("f_19037:scrutinizer_2escm"),(void*)f_19037},
{C_text("f_19039:scrutinizer_2escm"),(void*)f_19039},
{C_text("f_19054:scrutinizer_2escm"),(void*)f_19054},
{C_text("f_19057:scrutinizer_2escm"),(void*)f_19057},
{C_text("f_19069:scrutinizer_2escm"),(void*)f_19069},
{C_text("f_19080:scrutinizer_2escm"),(void*)f_19080},
{C_text("f_19086:scrutinizer_2escm"),(void*)f_19086},
{C_text("f_19092:scrutinizer_2escm"),(void*)f_19092},
{C_text("f_19108:scrutinizer_2escm"),(void*)f_19108},
{C_text("f_19111:scrutinizer_2escm"),(void*)f_19111},
{C_text("f_19121:scrutinizer_2escm"),(void*)f_19121},
{C_text("f_19147:scrutinizer_2escm"),(void*)f_19147},
{C_text("f_19153:scrutinizer_2escm"),(void*)f_19153},
{C_text("f_19159:scrutinizer_2escm"),(void*)f_19159},
{C_text("f_19175:scrutinizer_2escm"),(void*)f_19175},
{C_text("f_19181:scrutinizer_2escm"),(void*)f_19181},
{C_text("f_19206:scrutinizer_2escm"),(void*)f_19206},
{C_text("f_19244:scrutinizer_2escm"),(void*)f_19244},
{C_text("f_19246:scrutinizer_2escm"),(void*)f_19246},
{C_text("f_19271:scrutinizer_2escm"),(void*)f_19271},
{C_text("f_19297:scrutinizer_2escm"),(void*)f_19297},
{C_text("f_19299:scrutinizer_2escm"),(void*)f_19299},
{C_text("f_19324:scrutinizer_2escm"),(void*)f_19324},
{C_text("f_19333:scrutinizer_2escm"),(void*)f_19333},
{C_text("f_19361:scrutinizer_2escm"),(void*)f_19361},
{C_text("f_19375:scrutinizer_2escm"),(void*)f_19375},
{C_text("f_19409:scrutinizer_2escm"),(void*)f_19409},
{C_text("f_19413:scrutinizer_2escm"),(void*)f_19413},
{C_text("f_19447:scrutinizer_2escm"),(void*)f_19447},
{C_text("f_19451:scrutinizer_2escm"),(void*)f_19451},
{C_text("f_19459:scrutinizer_2escm"),(void*)f_19459},
{C_text("f_19479:scrutinizer_2escm"),(void*)f_19479},
{C_text("f_19495:scrutinizer_2escm"),(void*)f_19495},
{C_text("f_19497:scrutinizer_2escm"),(void*)f_19497},
{C_text("f_19501:scrutinizer_2escm"),(void*)f_19501},
{C_text("f_19564:scrutinizer_2escm"),(void*)f_19564},
{C_text("f_19576:scrutinizer_2escm"),(void*)f_19576},
{C_text("f_19580:scrutinizer_2escm"),(void*)f_19580},
{C_text("f_19596:scrutinizer_2escm"),(void*)f_19596},
{C_text("f_19600:scrutinizer_2escm"),(void*)f_19600},
{C_text("f_19621:scrutinizer_2escm"),(void*)f_19621},
{C_text("f_19644:scrutinizer_2escm"),(void*)f_19644},
{C_text("f_19651:scrutinizer_2escm"),(void*)f_19651},
{C_text("f_19654:scrutinizer_2escm"),(void*)f_19654},
{C_text("f_19657:scrutinizer_2escm"),(void*)f_19657},
{C_text("f_19710:scrutinizer_2escm"),(void*)f_19710},
{C_text("f_19720:scrutinizer_2escm"),(void*)f_19720},
{C_text("f_19747:scrutinizer_2escm"),(void*)f_19747},
{C_text("f_19765:scrutinizer_2escm"),(void*)f_19765},
{C_text("f_19842:scrutinizer_2escm"),(void*)f_19842},
{C_text("f_19844:scrutinizer_2escm"),(void*)f_19844},
{C_text("f_19848:scrutinizer_2escm"),(void*)f_19848},
{C_text("f_19851:scrutinizer_2escm"),(void*)f_19851},
{C_text("f_19897:scrutinizer_2escm"),(void*)f_19897},
{C_text("f_19900:scrutinizer_2escm"),(void*)f_19900},
{C_text("f_19906:scrutinizer_2escm"),(void*)f_19906},
{C_text("f_19910:scrutinizer_2escm"),(void*)f_19910},
{C_text("f_19914:scrutinizer_2escm"),(void*)f_19914},
{C_text("f_19921:scrutinizer_2escm"),(void*)f_19921},
{C_text("f_19925:scrutinizer_2escm"),(void*)f_19925},
{C_text("f_19937:scrutinizer_2escm"),(void*)f_19937},
{C_text("f_19940:scrutinizer_2escm"),(void*)f_19940},
{C_text("f_19943:scrutinizer_2escm"),(void*)f_19943},
{C_text("f_19946:scrutinizer_2escm"),(void*)f_19946},
{C_text("f_19949:scrutinizer_2escm"),(void*)f_19949},
{C_text("f_19952:scrutinizer_2escm"),(void*)f_19952},
{C_text("f_19955:scrutinizer_2escm"),(void*)f_19955},
{C_text("f_19957:scrutinizer_2escm"),(void*)f_19957},
{C_text("f_19966:scrutinizer_2escm"),(void*)f_19966},
{C_text("f_19976:scrutinizer_2escm"),(void*)f_19976},
{C_text("f_19998:scrutinizer_2escm"),(void*)f_19998},
{C_text("f_20008:scrutinizer_2escm"),(void*)f_20008},
{C_text("f_20011:scrutinizer_2escm"),(void*)f_20011},
{C_text("f_20013:scrutinizer_2escm"),(void*)f_20013},
{C_text("f_20057:scrutinizer_2escm"),(void*)f_20057},
{C_text("f_20074:scrutinizer_2escm"),(void*)f_20074},
{C_text("f_20080:scrutinizer_2escm"),(void*)f_20080},
{C_text("f_20087:scrutinizer_2escm"),(void*)f_20087},
{C_text("f_20111:scrutinizer_2escm"),(void*)f_20111},
{C_text("f_20120:scrutinizer_2escm"),(void*)f_20120},
{C_text("f_20123:scrutinizer_2escm"),(void*)f_20123},
{C_text("f_20127:scrutinizer_2escm"),(void*)f_20127},
{C_text("f_20131:scrutinizer_2escm"),(void*)f_20131},
{C_text("f_20143:scrutinizer_2escm"),(void*)f_20143},
{C_text("f_20173:scrutinizer_2escm"),(void*)f_20173},
{C_text("f_20187:scrutinizer_2escm"),(void*)f_20187},
{C_text("f_20191:scrutinizer_2escm"),(void*)f_20191},
{C_text("f_20195:scrutinizer_2escm"),(void*)f_20195},
{C_text("f_20218:scrutinizer_2escm"),(void*)f_20218},
{C_text("f_20220:scrutinizer_2escm"),(void*)f_20220},
{C_text("f_20223:scrutinizer_2escm"),(void*)f_20223},
{C_text("f_20232:scrutinizer_2escm"),(void*)f_20232},
{C_text("f_20283:scrutinizer_2escm"),(void*)f_20283},
{C_text("f_20287:scrutinizer_2escm"),(void*)f_20287},
{C_text("f_20291:scrutinizer_2escm"),(void*)f_20291},
{C_text("f_20295:scrutinizer_2escm"),(void*)f_20295},
{C_text("f_20303:scrutinizer_2escm"),(void*)f_20303},
{C_text("f_20305:scrutinizer_2escm"),(void*)f_20305},
{C_text("f_20329:scrutinizer_2escm"),(void*)f_20329},
{C_text("f_20354:scrutinizer_2escm"),(void*)f_20354},
{C_text("f_20365:scrutinizer_2escm"),(void*)f_20365},
{C_text("f_20375:scrutinizer_2escm"),(void*)f_20375},
{C_text("f_20377:scrutinizer_2escm"),(void*)f_20377},
{C_text("f_20402:scrutinizer_2escm"),(void*)f_20402},
{C_text("f_20419:scrutinizer_2escm"),(void*)f_20419},
{C_text("f_20421:scrutinizer_2escm"),(void*)f_20421},
{C_text("f_20435:scrutinizer_2escm"),(void*)f_20435},
{C_text("f_20451:scrutinizer_2escm"),(void*)f_20451},
{C_text("f_20470:scrutinizer_2escm"),(void*)f_20470},
{C_text("f_20476:scrutinizer_2escm"),(void*)f_20476},
{C_text("f_20478:scrutinizer_2escm"),(void*)f_20478},
{C_text("f_20488:scrutinizer_2escm"),(void*)f_20488},
{C_text("f_20508:scrutinizer_2escm"),(void*)f_20508},
{C_text("f_20511:scrutinizer_2escm"),(void*)f_20511},
{C_text("f_20522:scrutinizer_2escm"),(void*)f_20522},
{C_text("f_20525:scrutinizer_2escm"),(void*)f_20525},
{C_text("f_20545:scrutinizer_2escm"),(void*)f_20545},
{C_text("f_20552:scrutinizer_2escm"),(void*)f_20552},
{C_text("f_20555:scrutinizer_2escm"),(void*)f_20555},
{C_text("f_20581:scrutinizer_2escm"),(void*)f_20581},
{C_text("f_20594:scrutinizer_2escm"),(void*)f_20594},
{C_text("f_20596:scrutinizer_2escm"),(void*)f_20596},
{C_text("f_20621:scrutinizer_2escm"),(void*)f_20621},
{C_text("f_20636:scrutinizer_2escm"),(void*)f_20636},
{C_text("f_20643:scrutinizer_2escm"),(void*)f_20643},
{C_text("f_20646:scrutinizer_2escm"),(void*)f_20646},
{C_text("f_20653:scrutinizer_2escm"),(void*)f_20653},
{C_text("f_20662:scrutinizer_2escm"),(void*)f_20662},
{C_text("f_20668:scrutinizer_2escm"),(void*)f_20668},
{C_text("f_20681:scrutinizer_2escm"),(void*)f_20681},
{C_text("f_20697:scrutinizer_2escm"),(void*)f_20697},
{C_text("f_20703:scrutinizer_2escm"),(void*)f_20703},
{C_text("f_20706:scrutinizer_2escm"),(void*)f_20706},
{C_text("f_20709:scrutinizer_2escm"),(void*)f_20709},
{C_text("f_20712:scrutinizer_2escm"),(void*)f_20712},
{C_text("f_20720:scrutinizer_2escm"),(void*)f_20720},
{C_text("f_20733:scrutinizer_2escm"),(void*)f_20733},
{C_text("f_20735:scrutinizer_2escm"),(void*)f_20735},
{C_text("f_20760:scrutinizer_2escm"),(void*)f_20760},
{C_text("f_20776:scrutinizer_2escm"),(void*)f_20776},
{C_text("f_20788:scrutinizer_2escm"),(void*)f_20788},
{C_text("f_20797:scrutinizer_2escm"),(void*)f_20797},
{C_text("f_20806:scrutinizer_2escm"),(void*)f_20806},
{C_text("f_20865:scrutinizer_2escm"),(void*)f_20865},
{C_text("f_20867:scrutinizer_2escm"),(void*)f_20867},
{C_text("f_20892:scrutinizer_2escm"),(void*)f_20892},
{C_text("f_20917:scrutinizer_2escm"),(void*)f_20917},
{C_text("f_20923:scrutinizer_2escm"),(void*)f_20923},
{C_text("f_20926:scrutinizer_2escm"),(void*)f_20926},
{C_text("f_20929:scrutinizer_2escm"),(void*)f_20929},
{C_text("f_20960:scrutinizer_2escm"),(void*)f_20960},
{C_text("f_20963:scrutinizer_2escm"),(void*)f_20963},
{C_text("f_20966:scrutinizer_2escm"),(void*)f_20966},
{C_text("f_20983:scrutinizer_2escm"),(void*)f_20983},
{C_text("f_21008:scrutinizer_2escm"),(void*)f_21008},
{C_text("f_21017:scrutinizer_2escm"),(void*)f_21017},
{C_text("f_21042:scrutinizer_2escm"),(void*)f_21042},
{C_text("f_21079:scrutinizer_2escm"),(void*)f_21079},
{C_text("f_21086:scrutinizer_2escm"),(void*)f_21086},
{C_text("f_21090:scrutinizer_2escm"),(void*)f_21090},
{C_text("f_21092:scrutinizer_2escm"),(void*)f_21092},
{C_text("f_21100:scrutinizer_2escm"),(void*)f_21100},
{C_text("f_21102:scrutinizer_2escm"),(void*)f_21102},
{C_text("f_21106:scrutinizer_2escm"),(void*)f_21106},
{C_text("f_21111:scrutinizer_2escm"),(void*)f_21111},
{C_text("f_21137:scrutinizer_2escm"),(void*)f_21137},
{C_text("f_21142:scrutinizer_2escm"),(void*)f_21142},
{C_text("f_21154:scrutinizer_2escm"),(void*)f_21154},
{C_text("f_21179:scrutinizer_2escm"),(void*)f_21179},
{C_text("f_21196:scrutinizer_2escm"),(void*)f_21196},
{C_text("f_21205:scrutinizer_2escm"),(void*)f_21205},
{C_text("f_21207:scrutinizer_2escm"),(void*)f_21207},
{C_text("f_21215:scrutinizer_2escm"),(void*)f_21215},
{C_text("f_21219:scrutinizer_2escm"),(void*)f_21219},
{C_text("f_21221:scrutinizer_2escm"),(void*)f_21221},
{C_text("f_21229:scrutinizer_2escm"),(void*)f_21229},
{C_text("f_21231:scrutinizer_2escm"),(void*)f_21231},
{C_text("f_21235:scrutinizer_2escm"),(void*)f_21235},
{C_text("f_21244:scrutinizer_2escm"),(void*)f_21244},
{C_text("f_21250:scrutinizer_2escm"),(void*)f_21250},
{C_text("f_21253:scrutinizer_2escm"),(void*)f_21253},
{C_text("f_21256:scrutinizer_2escm"),(void*)f_21256},
{C_text("f_21261:scrutinizer_2escm"),(void*)f_21261},
{C_text("f_21274:scrutinizer_2escm"),(void*)f_21274},
{C_text("f_21280:scrutinizer_2escm"),(void*)f_21280},
{C_text("f_21283:scrutinizer_2escm"),(void*)f_21283},
{C_text("f_21286:scrutinizer_2escm"),(void*)f_21286},
{C_text("f_21293:scrutinizer_2escm"),(void*)f_21293},
{C_text("f_21311:scrutinizer_2escm"),(void*)f_21311},
{C_text("f_21325:scrutinizer_2escm"),(void*)f_21325},
{C_text("f_21329:scrutinizer_2escm"),(void*)f_21329},
{C_text("f_21333:scrutinizer_2escm"),(void*)f_21333},
{C_text("f_21339:scrutinizer_2escm"),(void*)f_21339},
{C_text("f_21347:scrutinizer_2escm"),(void*)f_21347},
{C_text("f_21353:scrutinizer_2escm"),(void*)f_21353},
{C_text("f_21361:scrutinizer_2escm"),(void*)f_21361},
{C_text("f_21365:scrutinizer_2escm"),(void*)f_21365},
{C_text("f_21371:scrutinizer_2escm"),(void*)f_21371},
{C_text("f_21374:scrutinizer_2escm"),(void*)f_21374},
{C_text("f_21377:scrutinizer_2escm"),(void*)f_21377},
{C_text("f_21380:scrutinizer_2escm"),(void*)f_21380},
{C_text("f_21383:scrutinizer_2escm"),(void*)f_21383},
{C_text("f_21389:scrutinizer_2escm"),(void*)f_21389},
{C_text("f_21392:scrutinizer_2escm"),(void*)f_21392},
{C_text("f_21395:scrutinizer_2escm"),(void*)f_21395},
{C_text("f_21398:scrutinizer_2escm"),(void*)f_21398},
{C_text("f_21401:scrutinizer_2escm"),(void*)f_21401},
{C_text("f_21417:scrutinizer_2escm"),(void*)f_21417},
{C_text("f_21431:scrutinizer_2escm"),(void*)f_21431},
{C_text("f_21435:scrutinizer_2escm"),(void*)f_21435},
{C_text("f_21438:scrutinizer_2escm"),(void*)f_21438},
{C_text("f_21451:scrutinizer_2escm"),(void*)f_21451},
{C_text("f_21455:scrutinizer_2escm"),(void*)f_21455},
{C_text("f_21470:scrutinizer_2escm"),(void*)f_21470},
{C_text("f_21482:scrutinizer_2escm"),(void*)f_21482},
{C_text("f_21488:scrutinizer_2escm"),(void*)f_21488},
{C_text("f_21494:scrutinizer_2escm"),(void*)f_21494},
{C_text("f_21501:scrutinizer_2escm"),(void*)f_21501},
{C_text("f_21507:scrutinizer_2escm"),(void*)f_21507},
{C_text("f_21510:scrutinizer_2escm"),(void*)f_21510},
{C_text("f_21513:scrutinizer_2escm"),(void*)f_21513},
{C_text("f_21516:scrutinizer_2escm"),(void*)f_21516},
{C_text("f_21519:scrutinizer_2escm"),(void*)f_21519},
{C_text("f_21525:scrutinizer_2escm"),(void*)f_21525},
{C_text("f_21531:scrutinizer_2escm"),(void*)f_21531},
{C_text("f_21534:scrutinizer_2escm"),(void*)f_21534},
{C_text("f_21537:scrutinizer_2escm"),(void*)f_21537},
{C_text("f_21542:scrutinizer_2escm"),(void*)f_21542},
{C_text("f_21545:scrutinizer_2escm"),(void*)f_21545},
{C_text("f_21553:scrutinizer_2escm"),(void*)f_21553},
{C_text("f_21557:scrutinizer_2escm"),(void*)f_21557},
{C_text("f_21559:scrutinizer_2escm"),(void*)f_21559},
{C_text("f_21563:scrutinizer_2escm"),(void*)f_21563},
{C_text("f_21583:scrutinizer_2escm"),(void*)f_21583},
{C_text("f_21590:scrutinizer_2escm"),(void*)f_21590},
{C_text("f_21611:scrutinizer_2escm"),(void*)f_21611},
{C_text("f_21621:scrutinizer_2escm"),(void*)f_21621},
{C_text("f_21625:scrutinizer_2escm"),(void*)f_21625},
{C_text("f_21629:scrutinizer_2escm"),(void*)f_21629},
{C_text("f_21633:scrutinizer_2escm"),(void*)f_21633},
{C_text("f_21637:scrutinizer_2escm"),(void*)f_21637},
{C_text("f_21641:scrutinizer_2escm"),(void*)f_21641},
{C_text("f_21645:scrutinizer_2escm"),(void*)f_21645},
{C_text("f_21649:scrutinizer_2escm"),(void*)f_21649},
{C_text("f_21656:scrutinizer_2escm"),(void*)f_21656},
{C_text("f_21672:scrutinizer_2escm"),(void*)f_21672},
{C_text("f_21675:scrutinizer_2escm"),(void*)f_21675},
{C_text("f_21681:scrutinizer_2escm"),(void*)f_21681},
{C_text("f_21697:scrutinizer_2escm"),(void*)f_21697},
{C_text("f_21703:scrutinizer_2escm"),(void*)f_21703},
{C_text("f_21710:scrutinizer_2escm"),(void*)f_21710},
{C_text("f_21714:scrutinizer_2escm"),(void*)f_21714},
{C_text("f_21718:scrutinizer_2escm"),(void*)f_21718},
{C_text("f_21722:scrutinizer_2escm"),(void*)f_21722},
{C_text("f_21725:scrutinizer_2escm"),(void*)f_21725},
{C_text("f_21729:scrutinizer_2escm"),(void*)f_21729},
{C_text("f_21733:scrutinizer_2escm"),(void*)f_21733},
{C_text("f_21739:scrutinizer_2escm"),(void*)f_21739},
{C_text("f_21742:scrutinizer_2escm"),(void*)f_21742},
{C_text("f_21756:scrutinizer_2escm"),(void*)f_21756},
{C_text("f_21774:scrutinizer_2escm"),(void*)f_21774},
{C_text("f_21787:scrutinizer_2escm"),(void*)f_21787},
{C_text("f_21791:scrutinizer_2escm"),(void*)f_21791},
{C_text("f_21795:scrutinizer_2escm"),(void*)f_21795},
{C_text("f_21801:scrutinizer_2escm"),(void*)f_21801},
{C_text("f_21812:scrutinizer_2escm"),(void*)f_21812},
{C_text("f_21816:scrutinizer_2escm"),(void*)f_21816},
{C_text("f_21820:scrutinizer_2escm"),(void*)f_21820},
{C_text("f_21824:scrutinizer_2escm"),(void*)f_21824},
{C_text("f_21828:scrutinizer_2escm"),(void*)f_21828},
{C_text("f_21839:scrutinizer_2escm"),(void*)f_21839},
{C_text("f_21843:scrutinizer_2escm"),(void*)f_21843},
{C_text("f_21847:scrutinizer_2escm"),(void*)f_21847},
{C_text("f_21851:scrutinizer_2escm"),(void*)f_21851},
{C_text("f_21855:scrutinizer_2escm"),(void*)f_21855},
{C_text("f_21862:scrutinizer_2escm"),(void*)f_21862},
{C_text("f_21873:scrutinizer_2escm"),(void*)f_21873},
{C_text("f_21877:scrutinizer_2escm"),(void*)f_21877},
{C_text("f_21881:scrutinizer_2escm"),(void*)f_21881},
{C_text("f_21897:scrutinizer_2escm"),(void*)f_21897},
{C_text("f_21901:scrutinizer_2escm"),(void*)f_21901},
{C_text("f_21906:scrutinizer_2escm"),(void*)f_21906},
{C_text("f_21911:scrutinizer_2escm"),(void*)f_21911},
{C_text("f_21922:scrutinizer_2escm"),(void*)f_21922},
{C_text("f_21926:scrutinizer_2escm"),(void*)f_21926},
{C_text("f_21930:scrutinizer_2escm"),(void*)f_21930},
{C_text("f_21934:scrutinizer_2escm"),(void*)f_21934},
{C_text("f_21938:scrutinizer_2escm"),(void*)f_21938},
{C_text("f_21942:scrutinizer_2escm"),(void*)f_21942},
{C_text("f_21946:scrutinizer_2escm"),(void*)f_21946},
{C_text("f_21950:scrutinizer_2escm"),(void*)f_21950},
{C_text("f_21952:scrutinizer_2escm"),(void*)f_21952},
{C_text("f_21957:scrutinizer_2escm"),(void*)f_21957},
{C_text("f_21959:scrutinizer_2escm"),(void*)f_21959},
{C_text("f_21971:scrutinizer_2escm"),(void*)f_21971},
{C_text("f_21975:scrutinizer_2escm"),(void*)f_21975},
{C_text("f_21979:scrutinizer_2escm"),(void*)f_21979},
{C_text("f_21983:scrutinizer_2escm"),(void*)f_21983},
{C_text("f_21987:scrutinizer_2escm"),(void*)f_21987},
{C_text("f_21993:scrutinizer_2escm"),(void*)f_21993},
{C_text("f_21996:scrutinizer_2escm"),(void*)f_21996},
{C_text("f_21999:scrutinizer_2escm"),(void*)f_21999},
{C_text("f_22006:scrutinizer_2escm"),(void*)f_22006},
{C_text("f_22022:scrutinizer_2escm"),(void*)f_22022},
{C_text("f_22028:scrutinizer_2escm"),(void*)f_22028},
{C_text("f_22031:scrutinizer_2escm"),(void*)f_22031},
{C_text("f_22034:scrutinizer_2escm"),(void*)f_22034},
{C_text("f_22037:scrutinizer_2escm"),(void*)f_22037},
{C_text("f_22045:scrutinizer_2escm"),(void*)f_22045},
{C_text("f_22050:scrutinizer_2escm"),(void*)f_22050},
{C_text("f_22057:scrutinizer_2escm"),(void*)f_22057},
{C_text("f_22065:scrutinizer_2escm"),(void*)f_22065},
{C_text("f_22069:scrutinizer_2escm"),(void*)f_22069},
{C_text("f_22073:scrutinizer_2escm"),(void*)f_22073},
{C_text("f_22077:scrutinizer_2escm"),(void*)f_22077},
{C_text("f_22081:scrutinizer_2escm"),(void*)f_22081},
{C_text("f_22087:scrutinizer_2escm"),(void*)f_22087},
{C_text("f_22090:scrutinizer_2escm"),(void*)f_22090},
{C_text("f_22093:scrutinizer_2escm"),(void*)f_22093},
{C_text("f_22099:scrutinizer_2escm"),(void*)f_22099},
{C_text("f_22105:scrutinizer_2escm"),(void*)f_22105},
{C_text("f_22108:scrutinizer_2escm"),(void*)f_22108},
{C_text("f_22111:scrutinizer_2escm"),(void*)f_22111},
{C_text("f_22114:scrutinizer_2escm"),(void*)f_22114},
{C_text("f_22117:scrutinizer_2escm"),(void*)f_22117},
{C_text("f_22120:scrutinizer_2escm"),(void*)f_22120},
{C_text("f_22123:scrutinizer_2escm"),(void*)f_22123},
{C_text("f_22130:scrutinizer_2escm"),(void*)f_22130},
{C_text("f_22136:scrutinizer_2escm"),(void*)f_22136},
{C_text("f_22139:scrutinizer_2escm"),(void*)f_22139},
{C_text("f_22142:scrutinizer_2escm"),(void*)f_22142},
{C_text("f_22148:scrutinizer_2escm"),(void*)f_22148},
{C_text("f_22154:scrutinizer_2escm"),(void*)f_22154},
{C_text("f_22157:scrutinizer_2escm"),(void*)f_22157},
{C_text("f_22160:scrutinizer_2escm"),(void*)f_22160},
{C_text("f_22167:scrutinizer_2escm"),(void*)f_22167},
{C_text("f_22177:scrutinizer_2escm"),(void*)f_22177},
{C_text("f_22180:scrutinizer_2escm"),(void*)f_22180},
{C_text("f_22192:scrutinizer_2escm"),(void*)f_22192},
{C_text("f_22196:scrutinizer_2escm"),(void*)f_22196},
{C_text("f_22200:scrutinizer_2escm"),(void*)f_22200},
{C_text("f_22216:scrutinizer_2escm"),(void*)f_22216},
{C_text("f_22222:scrutinizer_2escm"),(void*)f_22222},
{C_text("f_22225:scrutinizer_2escm"),(void*)f_22225},
{C_text("f_22228:scrutinizer_2escm"),(void*)f_22228},
{C_text("f_22231:scrutinizer_2escm"),(void*)f_22231},
{C_text("f_22239:scrutinizer_2escm"),(void*)f_22239},
{C_text("f_22242:scrutinizer_2escm"),(void*)f_22242},
{C_text("f_22254:scrutinizer_2escm"),(void*)f_22254},
{C_text("f_22258:scrutinizer_2escm"),(void*)f_22258},
{C_text("f_22262:scrutinizer_2escm"),(void*)f_22262},
{C_text("f_22266:scrutinizer_2escm"),(void*)f_22266},
{C_text("f_22279:scrutinizer_2escm"),(void*)f_22279},
{C_text("f_22285:scrutinizer_2escm"),(void*)f_22285},
{C_text("f_22288:scrutinizer_2escm"),(void*)f_22288},
{C_text("f_22291:scrutinizer_2escm"),(void*)f_22291},
{C_text("f_22294:scrutinizer_2escm"),(void*)f_22294},
{C_text("f_22298:scrutinizer_2escm"),(void*)f_22298},
{C_text("f_22305:scrutinizer_2escm"),(void*)f_22305},
{C_text("f_22311:scrutinizer_2escm"),(void*)f_22311},
{C_text("f_22314:scrutinizer_2escm"),(void*)f_22314},
{C_text("f_22317:scrutinizer_2escm"),(void*)f_22317},
{C_text("f_22320:scrutinizer_2escm"),(void*)f_22320},
{C_text("f_22323:scrutinizer_2escm"),(void*)f_22323},
{C_text("f_22326:scrutinizer_2escm"),(void*)f_22326},
{C_text("f_22330:scrutinizer_2escm"),(void*)f_22330},
{C_text("f_22336:scrutinizer_2escm"),(void*)f_22336},
{C_text("f_22339:scrutinizer_2escm"),(void*)f_22339},
{C_text("f_22342:scrutinizer_2escm"),(void*)f_22342},
{C_text("f_22345:scrutinizer_2escm"),(void*)f_22345},
{C_text("f_22353:scrutinizer_2escm"),(void*)f_22353},
{C_text("f_22359:scrutinizer_2escm"),(void*)f_22359},
{C_text("f_22362:scrutinizer_2escm"),(void*)f_22362},
{C_text("f_22374:scrutinizer_2escm"),(void*)f_22374},
{C_text("f_22378:scrutinizer_2escm"),(void*)f_22378},
{C_text("f_22382:scrutinizer_2escm"),(void*)f_22382},
{C_text("f_22386:scrutinizer_2escm"),(void*)f_22386},
{C_text("f_22399:scrutinizer_2escm"),(void*)f_22399},
{C_text("f_22405:scrutinizer_2escm"),(void*)f_22405},
{C_text("f_22408:scrutinizer_2escm"),(void*)f_22408},
{C_text("f_22411:scrutinizer_2escm"),(void*)f_22411},
{C_text("f_22414:scrutinizer_2escm"),(void*)f_22414},
{C_text("f_22418:scrutinizer_2escm"),(void*)f_22418},
{C_text("f_22425:scrutinizer_2escm"),(void*)f_22425},
{C_text("f_22431:scrutinizer_2escm"),(void*)f_22431},
{C_text("f_22434:scrutinizer_2escm"),(void*)f_22434},
{C_text("f_22437:scrutinizer_2escm"),(void*)f_22437},
{C_text("f_22440:scrutinizer_2escm"),(void*)f_22440},
{C_text("f_22443:scrutinizer_2escm"),(void*)f_22443},
{C_text("f_22446:scrutinizer_2escm"),(void*)f_22446},
{C_text("f_22450:scrutinizer_2escm"),(void*)f_22450},
{C_text("f_22456:scrutinizer_2escm"),(void*)f_22456},
{C_text("f_22459:scrutinizer_2escm"),(void*)f_22459},
{C_text("f_22462:scrutinizer_2escm"),(void*)f_22462},
{C_text("f_22465:scrutinizer_2escm"),(void*)f_22465},
{C_text("f_22473:scrutinizer_2escm"),(void*)f_22473},
{C_text("f_22484:scrutinizer_2escm"),(void*)f_22484},
{C_text("f_22495:scrutinizer_2escm"),(void*)f_22495},
{C_text("f_22499:scrutinizer_2escm"),(void*)f_22499},
{C_text("f_22503:scrutinizer_2escm"),(void*)f_22503},
{C_text("f_22507:scrutinizer_2escm"),(void*)f_22507},
{C_text("f_22511:scrutinizer_2escm"),(void*)f_22511},
{C_text("f_22518:scrutinizer_2escm"),(void*)f_22518},
{C_text("f_22529:scrutinizer_2escm"),(void*)f_22529},
{C_text("f_22533:scrutinizer_2escm"),(void*)f_22533},
{C_text("f_22537:scrutinizer_2escm"),(void*)f_22537},
{C_text("f_22541:scrutinizer_2escm"),(void*)f_22541},
{C_text("f_22545:scrutinizer_2escm"),(void*)f_22545},
{C_text("f_22559:scrutinizer_2escm"),(void*)f_22559},
{C_text("f_22563:scrutinizer_2escm"),(void*)f_22563},
{C_text("f_22567:scrutinizer_2escm"),(void*)f_22567},
{C_text("f_22581:scrutinizer_2escm"),(void*)f_22581},
{C_text("f_22585:scrutinizer_2escm"),(void*)f_22585},
{C_text("f_22599:scrutinizer_2escm"),(void*)f_22599},
{C_text("f_22603:scrutinizer_2escm"),(void*)f_22603},
{C_text("f_22607:scrutinizer_2escm"),(void*)f_22607},
{C_text("f_22621:scrutinizer_2escm"),(void*)f_22621},
{C_text("f_22625:scrutinizer_2escm"),(void*)f_22625},
{C_text("f_22633:scrutinizer_2escm"),(void*)f_22633},
{C_text("f_22647:scrutinizer_2escm"),(void*)f_22647},
{C_text("f_22651:scrutinizer_2escm"),(void*)f_22651},
{C_text("f_22655:scrutinizer_2escm"),(void*)f_22655},
{C_text("f_22659:scrutinizer_2escm"),(void*)f_22659},
{C_text("f_22679:scrutinizer_2escm"),(void*)f_22679},
{C_text("f_22683:scrutinizer_2escm"),(void*)f_22683},
{C_text("f_22687:scrutinizer_2escm"),(void*)f_22687},
{C_text("f_22691:scrutinizer_2escm"),(void*)f_22691},
{C_text("f_22695:scrutinizer_2escm"),(void*)f_22695},
{C_text("f_22699:scrutinizer_2escm"),(void*)f_22699},
{C_text("f_22709:scrutinizer_2escm"),(void*)f_22709},
{C_text("f_22711:scrutinizer_2escm"),(void*)f_22711},
{C_text("f_22736:scrutinizer_2escm"),(void*)f_22736},
{C_text("f_22748:scrutinizer_2escm"),(void*)f_22748},
{C_text("f_22755:scrutinizer_2escm"),(void*)f_22755},
{C_text("f_22769:scrutinizer_2escm"),(void*)f_22769},
{C_text("f_22773:scrutinizer_2escm"),(void*)f_22773},
{C_text("f_22785:scrutinizer_2escm"),(void*)f_22785},
{C_text("f_22797:scrutinizer_2escm"),(void*)f_22797},
{C_text("f_22815:scrutinizer_2escm"),(void*)f_22815},
{C_text("f_22819:scrutinizer_2escm"),(void*)f_22819},
{C_text("f_22823:scrutinizer_2escm"),(void*)f_22823},
{C_text("f_22827:scrutinizer_2escm"),(void*)f_22827},
{C_text("f_22831:scrutinizer_2escm"),(void*)f_22831},
{C_text("f_22835:scrutinizer_2escm"),(void*)f_22835},
{C_text("f_22839:scrutinizer_2escm"),(void*)f_22839},
{C_text("f_22842:scrutinizer_2escm"),(void*)f_22842},
{C_text("f_22850:scrutinizer_2escm"),(void*)f_22850},
{C_text("f_22861:scrutinizer_2escm"),(void*)f_22861},
{C_text("f_22867:scrutinizer_2escm"),(void*)f_22867},
{C_text("f_22870:scrutinizer_2escm"),(void*)f_22870},
{C_text("f_22873:scrutinizer_2escm"),(void*)f_22873},
{C_text("f_22876:scrutinizer_2escm"),(void*)f_22876},
{C_text("f_22884:scrutinizer_2escm"),(void*)f_22884},
{C_text("f_22888:scrutinizer_2escm"),(void*)f_22888},
{C_text("f_22892:scrutinizer_2escm"),(void*)f_22892},
{C_text("f_22896:scrutinizer_2escm"),(void*)f_22896},
{C_text("f_22899:scrutinizer_2escm"),(void*)f_22899},
{C_text("f_22905:scrutinizer_2escm"),(void*)f_22905},
{C_text("f_22908:scrutinizer_2escm"),(void*)f_22908},
{C_text("f_22911:scrutinizer_2escm"),(void*)f_22911},
{C_text("f_22914:scrutinizer_2escm"),(void*)f_22914},
{C_text("f_22917:scrutinizer_2escm"),(void*)f_22917},
{C_text("f_22924:scrutinizer_2escm"),(void*)f_22924},
{C_text("f_22928:scrutinizer_2escm"),(void*)f_22928},
{C_text("f_22940:scrutinizer_2escm"),(void*)f_22940},
{C_text("f_22944:scrutinizer_2escm"),(void*)f_22944},
{C_text("f_22985:scrutinizer_2escm"),(void*)f_22985},
{C_text("f_23002:scrutinizer_2escm"),(void*)f_23002},
{C_text("f_23023:scrutinizer_2escm"),(void*)f_23023},
{C_text("f_23025:scrutinizer_2escm"),(void*)f_23025},
{C_text("f_23059:scrutinizer_2escm"),(void*)f_23059},
{C_text("f_23080:scrutinizer_2escm"),(void*)f_23080},
{C_text("f_23082:scrutinizer_2escm"),(void*)f_23082},
{C_text("f_23116:scrutinizer_2escm"),(void*)f_23116},
{C_text("f_23141:scrutinizer_2escm"),(void*)f_23141},
{C_text("f_23143:scrutinizer_2escm"),(void*)f_23143},
{C_text("f_23181:scrutinizer_2escm"),(void*)f_23181},
{C_text("f_23206:scrutinizer_2escm"),(void*)f_23206},
{C_text("f_23208:scrutinizer_2escm"),(void*)f_23208},
{C_text("f_23248:scrutinizer_2escm"),(void*)f_23248},
{C_text("f_23250:scrutinizer_2escm"),(void*)f_23250},
{C_text("f_23258:scrutinizer_2escm"),(void*)f_23258},
{C_text("f_23260:scrutinizer_2escm"),(void*)f_23260},
{C_text("f_23276:scrutinizer_2escm"),(void*)f_23276},
{C_text("f_23280:scrutinizer_2escm"),(void*)f_23280},
{C_text("f_23289:scrutinizer_2escm"),(void*)f_23289},
{C_text("f_23316:scrutinizer_2escm"),(void*)f_23316},
{C_text("f_23327:scrutinizer_2escm"),(void*)f_23327},
{C_text("f_23333:scrutinizer_2escm"),(void*)f_23333},
{C_text("f_23337:scrutinizer_2escm"),(void*)f_23337},
{C_text("f_23390:scrutinizer_2escm"),(void*)f_23390},
{C_text("f_23394:scrutinizer_2escm"),(void*)f_23394},
{C_text("f_23398:scrutinizer_2escm"),(void*)f_23398},
{C_text("f_6730:scrutinizer_2escm"),(void*)f_6730},
{C_text("f_6733:scrutinizer_2escm"),(void*)f_6733},
{C_text("f_6736:scrutinizer_2escm"),(void*)f_6736},
{C_text("f_6739:scrutinizer_2escm"),(void*)f_6739},
{C_text("f_6742:scrutinizer_2escm"),(void*)f_6742},
{C_text("f_6745:scrutinizer_2escm"),(void*)f_6745},
{C_text("f_6748:scrutinizer_2escm"),(void*)f_6748},
{C_text("f_6751:scrutinizer_2escm"),(void*)f_6751},
{C_text("f_6754:scrutinizer_2escm"),(void*)f_6754},
{C_text("f_6820:scrutinizer_2escm"),(void*)f_6820},
{C_text("f_6834:scrutinizer_2escm"),(void*)f_6834},
{C_text("f_6858:scrutinizer_2escm"),(void*)f_6858},
{C_text("f_6864:scrutinizer_2escm"),(void*)f_6864},
{C_text("f_6882:scrutinizer_2escm"),(void*)f_6882},
{C_text("f_6922:scrutinizer_2escm"),(void*)f_6922},
{C_text("f_6936:scrutinizer_2escm"),(void*)f_6936},
{C_text("f_6955:scrutinizer_2escm"),(void*)f_6955},
{C_text("f_6970:scrutinizer_2escm"),(void*)f_6970},
{C_text("f_6978:scrutinizer_2escm"),(void*)f_6978},
{C_text("f_6986:scrutinizer_2escm"),(void*)f_6986},
{C_text("f_6997:scrutinizer_2escm"),(void*)f_6997},
{C_text("f_7010:scrutinizer_2escm"),(void*)f_7010},
{C_text("f_7017:scrutinizer_2escm"),(void*)f_7017},
{C_text("f_7024:scrutinizer_2escm"),(void*)f_7024},
{C_text("f_7028:scrutinizer_2escm"),(void*)f_7028},
{C_text("f_7040:scrutinizer_2escm"),(void*)f_7040},
{C_text("f_7042:scrutinizer_2escm"),(void*)f_7042},
{C_text("f_7089:scrutinizer_2escm"),(void*)f_7089},
{C_text("f_7091:scrutinizer_2escm"),(void*)f_7091},
{C_text("f_7125:scrutinizer_2escm"),(void*)f_7125},
{C_text("f_7131:scrutinizer_2escm"),(void*)f_7131},
{C_text("f_7153:scrutinizer_2escm"),(void*)f_7153},
{C_text("f_7159:scrutinizer_2escm"),(void*)f_7159},
{C_text("f_7165:scrutinizer_2escm"),(void*)f_7165},
{C_text("f_7175:scrutinizer_2escm"),(void*)f_7175},
{C_text("f_7195:scrutinizer_2escm"),(void*)f_7195},
{C_text("f_7209:scrutinizer_2escm"),(void*)f_7209},
{C_text("f_7223:scrutinizer_2escm"),(void*)f_7223},
{C_text("f_7241:scrutinizer_2escm"),(void*)f_7241},
{C_text("f_7251:scrutinizer_2escm"),(void*)f_7251},
{C_text("f_7264:scrutinizer_2escm"),(void*)f_7264},
{C_text("f_7278:scrutinizer_2escm"),(void*)f_7278},
{C_text("f_7320:scrutinizer_2escm"),(void*)f_7320},
{C_text("f_7326:scrutinizer_2escm"),(void*)f_7326},
{C_text("f_7340:scrutinizer_2escm"),(void*)f_7340},
{C_text("f_7353:scrutinizer_2escm"),(void*)f_7353},
{C_text("f_7409:scrutinizer_2escm"),(void*)f_7409},
{C_text("f_7418:scrutinizer_2escm"),(void*)f_7418},
{C_text("f_7426:scrutinizer_2escm"),(void*)f_7426},
{C_text("f_7430:scrutinizer_2escm"),(void*)f_7430},
{C_text("f_7434:scrutinizer_2escm"),(void*)f_7434},
{C_text("f_7452:scrutinizer_2escm"),(void*)f_7452},
{C_text("f_7474:scrutinizer_2escm"),(void*)f_7474},
{C_text("f_7491:scrutinizer_2escm"),(void*)f_7491},
{C_text("f_7590:scrutinizer_2escm"),(void*)f_7590},
{C_text("f_7676:scrutinizer_2escm"),(void*)f_7676},
{C_text("f_7701:scrutinizer_2escm"),(void*)f_7701},
{C_text("f_7724:scrutinizer_2escm"),(void*)f_7724},
{C_text("f_7729:scrutinizer_2escm"),(void*)f_7729},
{C_text("f_7743:scrutinizer_2escm"),(void*)f_7743},
{C_text("f_7767:scrutinizer_2escm"),(void*)f_7767},
{C_text("f_7773:scrutinizer_2escm"),(void*)f_7773},
{C_text("f_7787:scrutinizer_2escm"),(void*)f_7787},
{C_text("f_7791:scrutinizer_2escm"),(void*)f_7791},
{C_text("f_7803:scrutinizer_2escm"),(void*)f_7803},
{C_text("f_7809:scrutinizer_2escm"),(void*)f_7809},
{C_text("f_7821:scrutinizer_2escm"),(void*)f_7821},
{C_text("f_7886:scrutinizer_2escm"),(void*)f_7886},
{C_text("f_7947:scrutinizer_2escm"),(void*)f_7947},
{C_text("f_7950:scrutinizer_2escm"),(void*)f_7950},
{C_text("f_7959:scrutinizer_2escm"),(void*)f_7959},
{C_text("f_8079:scrutinizer_2escm"),(void*)f_8079},
{C_text("f_8087:scrutinizer_2escm"),(void*)f_8087},
{C_text("f_8089:scrutinizer_2escm"),(void*)f_8089},
{C_text("f_8132:scrutinizer_2escm"),(void*)f_8132},
{C_text("f_8169:scrutinizer_2escm"),(void*)f_8169},
{C_text("f_8182:scrutinizer_2escm"),(void*)f_8182},
{C_text("f_8194:scrutinizer_2escm"),(void*)f_8194},
{C_text("f_8197:scrutinizer_2escm"),(void*)f_8197},
{C_text("f_8258:scrutinizer_2escm"),(void*)f_8258},
{C_text("f_8260:scrutinizer_2escm"),(void*)f_8260},
{C_text("f_8285:scrutinizer_2escm"),(void*)f_8285},
{C_text("f_8309:scrutinizer_2escm"),(void*)f_8309},
{C_text("f_8313:scrutinizer_2escm"),(void*)f_8313},
{C_text("f_8343:scrutinizer_2escm"),(void*)f_8343},
{C_text("f_8346:scrutinizer_2escm"),(void*)f_8346},
{C_text("f_8348:scrutinizer_2escm"),(void*)f_8348},
{C_text("f_8373:scrutinizer_2escm"),(void*)f_8373},
{C_text("f_8418:scrutinizer_2escm"),(void*)f_8418},
{C_text("f_8422:scrutinizer_2escm"),(void*)f_8422},
{C_text("f_8432:scrutinizer_2escm"),(void*)f_8432},
{C_text("f_8441:scrutinizer_2escm"),(void*)f_8441},
{C_text("f_8461:scrutinizer_2escm"),(void*)f_8461},
{C_text("f_8473:scrutinizer_2escm"),(void*)f_8473},
{C_text("f_8492:scrutinizer_2escm"),(void*)f_8492},
{C_text("f_8496:scrutinizer_2escm"),(void*)f_8496},
{C_text("f_8508:scrutinizer_2escm"),(void*)f_8508},
{C_text("f_8546:scrutinizer_2escm"),(void*)f_8546},
{C_text("f_8553:scrutinizer_2escm"),(void*)f_8553},
{C_text("f_8555:scrutinizer_2escm"),(void*)f_8555},
{C_text("f_8587:scrutinizer_2escm"),(void*)f_8587},
{C_text("f_8617:scrutinizer_2escm"),(void*)f_8617},
{C_text("f_8621:scrutinizer_2escm"),(void*)f_8621},
{C_text("f_8627:scrutinizer_2escm"),(void*)f_8627},
{C_text("f_8639:scrutinizer_2escm"),(void*)f_8639},
{C_text("f_8641:scrutinizer_2escm"),(void*)f_8641},
{C_text("f_8669:scrutinizer_2escm"),(void*)f_8669},
{C_text("f_8672:scrutinizer_2escm"),(void*)f_8672},
{C_text("f_8685:scrutinizer_2escm"),(void*)f_8685},
{C_text("f_8688:scrutinizer_2escm"),(void*)f_8688},
{C_text("f_8691:scrutinizer_2escm"),(void*)f_8691},
{C_text("f_8707:scrutinizer_2escm"),(void*)f_8707},
{C_text("f_8717:scrutinizer_2escm"),(void*)f_8717},
{C_text("f_8723:scrutinizer_2escm"),(void*)f_8723},
{C_text("f_8733:scrutinizer_2escm"),(void*)f_8733},
{C_text("f_8742:scrutinizer_2escm"),(void*)f_8742},
{C_text("f_8745:scrutinizer_2escm"),(void*)f_8745},
{C_text("f_8752:scrutinizer_2escm"),(void*)f_8752},
{C_text("f_8758:scrutinizer_2escm"),(void*)f_8758},
{C_text("f_8763:scrutinizer_2escm"),(void*)f_8763},
{C_text("f_8769:scrutinizer_2escm"),(void*)f_8769},
{C_text("f_8773:scrutinizer_2escm"),(void*)f_8773},
{C_text("f_8776:scrutinizer_2escm"),(void*)f_8776},
{C_text("f_8779:scrutinizer_2escm"),(void*)f_8779},
{C_text("f_8782:scrutinizer_2escm"),(void*)f_8782},
{C_text("f_8785:scrutinizer_2escm"),(void*)f_8785},
{C_text("f_8788:scrutinizer_2escm"),(void*)f_8788},
{C_text("f_8791:scrutinizer_2escm"),(void*)f_8791},
{C_text("f_8794:scrutinizer_2escm"),(void*)f_8794},
{C_text("f_8807:scrutinizer_2escm"),(void*)f_8807},
{C_text("f_8819:scrutinizer_2escm"),(void*)f_8819},
{C_text("f_8844:scrutinizer_2escm"),(void*)f_8844},
{C_text("f_8854:scrutinizer_2escm"),(void*)f_8854},
{C_text("f_8863:scrutinizer_2escm"),(void*)f_8863},
{C_text("f_8875:scrutinizer_2escm"),(void*)f_8875},
{C_text("f_8882:scrutinizer_2escm"),(void*)f_8882},
{C_text("f_8885:scrutinizer_2escm"),(void*)f_8885},
{C_text("f_8888:scrutinizer_2escm"),(void*)f_8888},
{C_text("f_8922:scrutinizer_2escm"),(void*)f_8922},
{C_text("f_8929:scrutinizer_2escm"),(void*)f_8929},
{C_text("f_8932:scrutinizer_2escm"),(void*)f_8932},
{C_text("f_8938:scrutinizer_2escm"),(void*)f_8938},
{C_text("f_8967:scrutinizer_2escm"),(void*)f_8967},
{C_text("f_8971:scrutinizer_2escm"),(void*)f_8971},
{C_text("f_8989:scrutinizer_2escm"),(void*)f_8989},
{C_text("f_8992:scrutinizer_2escm"),(void*)f_8992},
{C_text("f_8995:scrutinizer_2escm"),(void*)f_8995},
{C_text("f_9001:scrutinizer_2escm"),(void*)f_9001},
{C_text("f_9034:scrutinizer_2escm"),(void*)f_9034},
{C_text("f_9038:scrutinizer_2escm"),(void*)f_9038},
{C_text("f_9077:scrutinizer_2escm"),(void*)f_9077},
{C_text("f_9081:scrutinizer_2escm"),(void*)f_9081},
{C_text("f_9100:scrutinizer_2escm"),(void*)f_9100},
{C_text("f_9104:scrutinizer_2escm"),(void*)f_9104},
{C_text("f_9110:scrutinizer_2escm"),(void*)f_9110},
{C_text("f_9124:scrutinizer_2escm"),(void*)f_9124},
{C_text("f_9130:scrutinizer_2escm"),(void*)f_9130},
{C_text("f_9144:scrutinizer_2escm"),(void*)f_9144},
{C_text("f_9175:scrutinizer_2escm"),(void*)f_9175},
{C_text("f_9188:scrutinizer_2escm"),(void*)f_9188},
{C_text("f_9198:scrutinizer_2escm"),(void*)f_9198},
{C_text("f_9200:scrutinizer_2escm"),(void*)f_9200},
{C_text("f_9229:scrutinizer_2escm"),(void*)f_9229},
{C_text("f_9242:scrutinizer_2escm"),(void*)f_9242},
{C_text("f_9258:scrutinizer_2escm"),(void*)f_9258},
{C_text("f_9269:scrutinizer_2escm"),(void*)f_9269},
{C_text("f_9273:scrutinizer_2escm"),(void*)f_9273},
{C_text("f_9301:scrutinizer_2escm"),(void*)f_9301},
{C_text("f_9305:scrutinizer_2escm"),(void*)f_9305},
{C_text("f_9307:scrutinizer_2escm"),(void*)f_9307},
{C_text("f_9341:scrutinizer_2escm"),(void*)f_9341},
{C_text("f_9346:scrutinizer_2escm"),(void*)f_9346},
{C_text("f_9352:scrutinizer_2escm"),(void*)f_9352},
{C_text("f_9357:scrutinizer_2escm"),(void*)f_9357},
{C_text("f_9379:scrutinizer_2escm"),(void*)f_9379},
{C_text("f_9386:scrutinizer_2escm"),(void*)f_9386},
{C_text("f_9389:scrutinizer_2escm"),(void*)f_9389},
{C_text("f_9392:scrutinizer_2escm"),(void*)f_9392},
{C_text("f_9398:scrutinizer_2escm"),(void*)f_9398},
{C_text("f_9402:scrutinizer_2escm"),(void*)f_9402},
{C_text("f_9409:scrutinizer_2escm"),(void*)f_9409},
{C_text("f_9417:scrutinizer_2escm"),(void*)f_9417},
{C_text("f_9442:scrutinizer_2escm"),(void*)f_9442},
{C_text("f_9451:scrutinizer_2escm"),(void*)f_9451},
{C_text("f_9457:scrutinizer_2escm"),(void*)f_9457},
{C_text("f_9475:scrutinizer_2escm"),(void*)f_9475},
{C_text("f_9479:scrutinizer_2escm"),(void*)f_9479},
{C_text("f_9482:scrutinizer_2escm"),(void*)f_9482},
{C_text("f_9485:scrutinizer_2escm"),(void*)f_9485},
{C_text("f_9493:scrutinizer_2escm"),(void*)f_9493},
{C_text("f_9521:scrutinizer_2escm"),(void*)f_9521},
{C_text("f_9563:scrutinizer_2escm"),(void*)f_9563},
{C_text("f_9580:scrutinizer_2escm"),(void*)f_9580},
{C_text("f_9600:scrutinizer_2escm"),(void*)f_9600},
{C_text("f_9617:scrutinizer_2escm"),(void*)f_9617},
{C_text("f_9636:scrutinizer_2escm"),(void*)f_9636},
{C_text("f_9642:scrutinizer_2escm"),(void*)f_9642},
{C_text("f_9647:scrutinizer_2escm"),(void*)f_9647},
{C_text("f_9666:scrutinizer_2escm"),(void*)f_9666},
{C_text("f_9669:scrutinizer_2escm"),(void*)f_9669},
{C_text("f_9673:scrutinizer_2escm"),(void*)f_9673},
{C_text("f_9680:scrutinizer_2escm"),(void*)f_9680},
{C_text("f_9692:scrutinizer_2escm"),(void*)f_9692},
{C_text("f_9695:scrutinizer_2escm"),(void*)f_9695},
{C_text("f_9727:scrutinizer_2escm"),(void*)f_9727},
{C_text("f_9756:scrutinizer_2escm"),(void*)f_9756},
{C_text("f_9821:scrutinizer_2escm"),(void*)f_9821},
{C_text("f_9855:scrutinizer_2escm"),(void*)f_9855},
{C_text("f_9865:scrutinizer_2escm"),(void*)f_9865},
{C_text("f_9911:scrutinizer_2escm"),(void*)f_9911},
{C_text("f_9915:scrutinizer_2escm"),(void*)f_9915},
{C_text("f_9917:scrutinizer_2escm"),(void*)f_9917},
{C_text("f_9940:scrutinizer_2escm"),(void*)f_9940},
{C_text("f_9958:scrutinizer_2escm"),(void*)f_9958},
{C_text("f_9967:scrutinizer_2escm"),(void*)f_9967},
{C_text("f_9970:scrutinizer_2escm"),(void*)f_9970},
{C_text("f_9986:scrutinizer_2escm"),(void*)f_9986},
{C_text("f_9995:scrutinizer_2escm"),(void*)f_9995},
{C_text("toplevel:scrutinizer_2escm"),(void*)C_scrutinizer_toplevel},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.compiler.scrutinizer#partition 
o|hiding unexported module binding: chicken.compiler.scrutinizer#span 
o|hiding unexported module binding: chicken.compiler.scrutinizer#take 
o|hiding unexported module binding: chicken.compiler.scrutinizer#drop 
o|hiding unexported module binding: chicken.compiler.scrutinizer#split-at 
o|hiding unexported module binding: chicken.compiler.scrutinizer#append-map 
o|hiding unexported module binding: chicken.compiler.scrutinizer#every 
o|hiding unexported module binding: chicken.compiler.scrutinizer#any 
o|hiding unexported module binding: chicken.compiler.scrutinizer#cons* 
o|hiding unexported module binding: chicken.compiler.scrutinizer#concatenate 
o|hiding unexported module binding: chicken.compiler.scrutinizer#delete 
o|hiding unexported module binding: chicken.compiler.scrutinizer#first 
o|hiding unexported module binding: chicken.compiler.scrutinizer#second 
o|hiding unexported module binding: chicken.compiler.scrutinizer#third 
o|hiding unexported module binding: chicken.compiler.scrutinizer#fourth 
o|hiding unexported module binding: chicken.compiler.scrutinizer#fifth 
o|hiding unexported module binding: chicken.compiler.scrutinizer#delete-duplicates 
o|hiding unexported module binding: chicken.compiler.scrutinizer#alist-cons 
o|hiding unexported module binding: chicken.compiler.scrutinizer#filter 
o|hiding unexported module binding: chicken.compiler.scrutinizer#filter-map 
o|hiding unexported module binding: chicken.compiler.scrutinizer#remove 
o|hiding unexported module binding: chicken.compiler.scrutinizer#unzip1 
o|hiding unexported module binding: chicken.compiler.scrutinizer#last 
o|hiding unexported module binding: chicken.compiler.scrutinizer#list-index 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset-adjoin/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset-difference/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset-union/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset-intersection/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#list-tabulate 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset<=/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#lset=/eq? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#length+ 
o|hiding unexported module binding: chicken.compiler.scrutinizer#find 
o|hiding unexported module binding: chicken.compiler.scrutinizer#find-tail 
o|hiding unexported module binding: chicken.compiler.scrutinizer#iota 
o|hiding unexported module binding: chicken.compiler.scrutinizer#make-list 
o|hiding unexported module binding: chicken.compiler.scrutinizer#posq 
o|hiding unexported module binding: chicken.compiler.scrutinizer#posv 
o|hiding unexported module binding: chicken.compiler.scrutinizer#d-depth 
o|hiding unexported module binding: chicken.compiler.scrutinizer#scrutiny-debug 
o|hiding unexported module binding: chicken.compiler.scrutinizer#*complain?* 
o|hiding unexported module binding: chicken.compiler.scrutinizer#d 
o|hiding unexported module binding: chicken.compiler.scrutinizer#dd 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ddd 
o|hiding unexported module binding: chicken.compiler.scrutinizer#d 
o|hiding unexported module binding: chicken.compiler.scrutinizer#dd 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ddd 
o|hiding unexported module binding: chicken.compiler.scrutinizer#constant723 
o|hiding unexported module binding: chicken.compiler.scrutinizer#constant730 
o|hiding unexported module binding: chicken.compiler.scrutinizer#constant738 
o|hiding unexported module binding: chicken.compiler.scrutinizer#constant747 
o|hiding unexported module binding: chicken.compiler.scrutinizer#specialization-statistics 
o|hiding unexported module binding: chicken.compiler.scrutinizer#trail 
o|hiding unexported module binding: chicken.compiler.scrutinizer#walked-result 
o|hiding unexported module binding: chicken.compiler.scrutinizer#type-always-immediate? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#smash-component-types! 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ble-id 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ble-tag 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ble-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#ble-type-set! 
o|hiding unexported module binding: chicken.compiler.scrutinizer#match-argument-types 
o|hiding unexported module binding: chicken.compiler.scrutinizer#simplify-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#maybe-expand-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#merge-argument-types 
o|hiding unexported module binding: chicken.compiler.scrutinizer#merge-result-types 
o|hiding unexported module binding: chicken.compiler.scrutinizer#compatible-types? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#type-min 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-type? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-name 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-arguments 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-results 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-argument-types 
o|hiding unexported module binding: chicken.compiler.scrutinizer#procedure-result-types 
o|hiding unexported module binding: chicken.compiler.scrutinizer#named? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#rest-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#noreturn-procedure-type? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#noreturn-type? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#refinement-type? 
o|hiding unexported module binding: chicken.compiler.scrutinizer#make-tv 
o|hiding unexported module binding: chicken.compiler.scrutinizer#type-typeenv 
o|hiding unexported module binding: chicken.compiler.scrutinizer#trail-restore 
o|hiding unexported module binding: chicken.compiler.scrutinizer#with-trail-restore 
o|hiding unexported module binding: chicken.compiler.scrutinizer#resolve 
o|hiding unexported module binding: chicken.compiler.scrutinizer#node-mutations 
o|hiding unexported module binding: chicken.compiler.scrutinizer#mutate-node! 
o|hiding unexported module binding: chicken.compiler.scrutinizer#source-node 
o|hiding unexported module binding: chicken.compiler.scrutinizer#source-node-tree 
o|hiding unexported module binding: chicken.compiler.scrutinizer#node-line-number 
o|hiding unexported module binding: chicken.compiler.scrutinizer#node-debug-info 
o|hiding unexported module binding: chicken.compiler.scrutinizer#specialize-node! 
o|hiding unexported module binding: chicken.compiler.scrutinizer#canonicalize-list-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#strip-namespace 
o|hiding unexported module binding: chicken.compiler.scrutinizer#define-special-case 
o|hiding unexported module binding: chicken.compiler.scrutinizer#over-all-instantiations 
o|hiding unexported module binding: chicken.compiler.scrutinizer#multiples 
o|hiding unexported module binding: chicken.compiler.scrutinizer#string-add-indent 
o|hiding unexported module binding: chicken.compiler.scrutinizer#type->pp-string 
o|hiding unexported module binding: chicken.compiler.scrutinizer#fragment 
o|hiding unexported module binding: chicken.compiler.scrutinizer#pp-fragment 
o|hiding unexported module binding: chicken.compiler.scrutinizer#node-source-prefix 
o|hiding unexported module binding: chicken.compiler.scrutinizer#location-name 
o|hiding unexported module binding: chicken.compiler.scrutinizer#variable-and-module 
o|hiding unexported module binding: chicken.compiler.scrutinizer#variable-from-module 
o|hiding unexported module binding: chicken.compiler.scrutinizer#describe-expression 
o|hiding unexported module binding: chicken.compiler.scrutinizer#call-node-procedure-name 
o|hiding unexported module binding: chicken.compiler.scrutinizer#report2 
o|hiding unexported module binding: chicken.compiler.scrutinizer#report-notice 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-invalid-called-procedure-type 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-proc-call-argument-count-mismatch 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-proc-call-argument-value-count 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-index-out-of-range 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-conditional-value-count-invalid 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-let-value-count-invalid 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-assignment-value-count-invalid 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-pred-call-always-true 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-pred-call-always-false 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-cond-test-always-true 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-cond-test-always-false 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-zero-values-for-the 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-too-many-values-for-the 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-type-mismatch-in-the 
o|hiding unexported module binding: chicken.compiler.scrutinizer#fail-compiler-typecase 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-cond-branch-value-count-mismatch 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-toplevel-var-assignment-type-mismatch 
o|hiding unexported module binding: chicken.compiler.scrutinizer#r-deprecated-identifier 
S|applied compiler syntax:
S|  scheme#for-each		11
S|  chicken.format#sprintf		27
S|  chicken.format#printf		2
S|  chicken.base#foldl		5
S|  scheme#map		49
S|  chicken.base#foldr		3
o|eliminated procedure checks: 463 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|specializations:
o|  2 (scheme#string=? string string)
o|  1 (scheme#< fixnum fixnum)
o|  1 (scheme#string-append string string)
o|  1 (scheme#>= fixnum fixnum)
o|  2 (scheme#string-length string)
o|  4 (scheme#eqv? (or eof null fixnum char boolean symbol keyword) *)
o|  1 (scheme#- *)
o|  4 (scheme#negative? *)
o|  2 (scheme#integer? *)
o|  2 (scheme#vector-length vector)
o|  1 (scheme#cdddr (pair * (pair * pair)))
o|  1 (chicken.plist#put! symbol symbol *)
o|  3 (scheme#cadr (pair * pair))
o|  4 (scheme#cddr (pair * pair))
o|  4 (scheme#positive? *)
o|  2 (scheme#> fixnum fixnum)
o|  2 (chicken.base#sub1 fixnum)
o|  2 (scheme#set-cdr! pair *)
o|  8 (chicken.base#sub1 *)
o|  3 (scheme#zero? *)
o|  1 (scheme#caar (pair pair *))
o|  13 (chicken.base#add1 *)
o|  11 (scheme#length list)
o|  5 (scheme#zero? integer)
o|  16 (scheme#= fixnum fixnum)
o|  1 (scheme#cdar (pair pair *))
o|  15 (scheme#memq * list)
o|  91 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  29 (##sys#check-output-port * * *)
o|  1 (##sys#debug-mode?)
o|  1 (scheme#eqv? * *)
o|  11 (##sys#check-list (or pair list) *)
o|  123 (scheme#cdr pair)
o|  129 (scheme#car pair)
(o e)|safe calls: 2507 
(o e)|dropped branches: 2 
(o e)|assignments to immediate values: 7 
o|safe globals: (chicken.compiler.scrutinizer#strip-namespace chicken.compiler.scrutinizer#canonicalize-list-type chicken.compiler.scrutinizer#install-specializations chicken.compiler.scrutinizer#check-and-validate-type chicken.compiler.scrutinizer#validate-type chicken.compiler.scrutinizer#specialize-node! chicken.compiler.scrutinizer#node-debug-info chicken.compiler.scrutinizer#node-line-number chicken.compiler.scrutinizer#source-node-tree chicken.compiler.scrutinizer#source-node chicken.compiler.scrutinizer#mutate-node! chicken.compiler.scrutinizer#node-mutations chicken.compiler.scrutinizer#emit-types-file chicken.compiler.scrutinizer#load-type-database chicken.compiler.scrutinizer#resolve chicken.compiler.scrutinizer#with-trail-restore chicken.compiler.scrutinizer#trail-restore chicken.compiler.scrutinizer#type-typeenv chicken.compiler.scrutinizer#make-tv chicken.compiler.scrutinizer#refinement-type? chicken.compiler.scrutinizer#noreturn-type? chicken.compiler.scrutinizer#noreturn-procedure-type? chicken.compiler.scrutinizer#rest-type chicken.compiler.scrutinizer#named? chicken.compiler.scrutinizer#procedure-result-types chicken.compiler.scrutinizer#procedure-argument-types chicken.compiler.scrutinizer#procedure-results chicken.compiler.scrutinizer#procedure-arguments chicken.compiler.scrutinizer#procedure-name chicken.compiler.scrutinizer#procedure-type? chicken.compiler.scrutinizer#refine-types chicken.compiler.scrutinizer#type<=? chicken.compiler.scrutinizer#type-min chicken.compiler.scrutinizer#compatible-types? chicken.compiler.scrutinizer#merge-result-types chicken.compiler.scrutinizer#merge-argument-types chicken.compiler.scrutinizer#maybe-expand-type chicken.compiler.scrutinizer#simplify-type chicken.compiler.scrutinizer#match-argument-types chicken.compiler.scrutinizer#match-types chicken.compiler.scrutinizer#ble-type-set! chicken.compiler.scrutinizer#ble-type chicken.compiler.scrutinizer#ble-tag chicken.compiler.scrutinizer#ble-id chicken.compiler.scrutinizer#smash-component-types! chicken.compiler.scrutinizer#scrutinize chicken.compiler.scrutinizer#type-always-immediate? chicken.compiler.scrutinizer#walked-result chicken.compiler.scrutinizer#trail chicken.compiler.scrutinizer#specialization-statistics chicken.compiler.scrutinizer#constant747 chicken.compiler.scrutinizer#constant738 chicken.compiler.scrutinizer#constant730 chicken.compiler.scrutinizer#constant723 chicken.compiler.scrutinizer#ddd chicken.compiler.scrutinizer#dd chicken.compiler.scrutinizer#d chicken.compiler.scrutinizer#*complain?* chicken.compiler.scrutinizer#scrutiny-debug chicken.compiler.scrutinizer#d-depth chicken.compiler.scrutinizer#posv chicken.compiler.scrutinizer#posq chicken.compiler.scrutinizer#make-list chicken.compiler.scrutinizer#iota chicken.compiler.scrutinizer#find-tail chicken.compiler.scrutinizer#find chicken.compiler.scrutinizer#length+ chicken.compiler.scrutinizer#lset=/eq? chicken.compiler.scrutinizer#lset<=/eq? chicken.compiler.scrutinizer#list-tabulate chicken.compiler.scrutinizer#lset-intersection/eq? chicken.compiler.scrutinizer#lset-union/eq? chicken.compiler.scrutinizer#lset-difference/eq? chicken.compiler.scrutinizer#lset-adjoin/eq? chicken.compiler.scrutinizer#list-index chicken.compiler.scrutinizer#last chicken.compiler.scrutinizer#unzip1 chicken.compiler.scrutinizer#remove chicken.compiler.scrutinizer#filter-map chicken.compiler.scrutinizer#filter chicken.compiler.scrutinizer#alist-cons chicken.compiler.scrutinizer#delete-duplicates chicken.compiler.scrutinizer#fifth chicken.compiler.scrutinizer#fourth chicken.compiler.scrutinizer#third chicken.compiler.scrutinizer#second chicken.compiler.scrutinizer#first chicken.compiler.scrutinizer#delete chicken.compiler.scrutinizer#concatenate chicken.compiler.scrutinizer#cons* chicken.compiler.scrutinizer#any chicken.compiler.scrutinizer#every chicken.compiler.scrutinizer#append-map chicken.compiler.scrutinizer#split-at chicken.compiler.scrutinizer#drop chicken.compiler.scrutinizer#take chicken.compiler.scrutinizer#span chicken.compiler.scrutinizer#partition) 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#partition 
o|inlining procedure: k6866 
o|inlining procedure: k6866 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#drop 
o|merged explicitly consed rest parameter: lsts179 
o|inlining procedure: k6957 
o|inlining procedure: k6972 
o|inlining procedure: k6972 
o|inlining procedure: k6957 
o|inlining procedure: k7012 
o|inlining procedure: k7012 
o|inlining procedure: k7044 
o|contracted procedure: "(mini-srfi-1.scm:77) g232241" 
o|inlining procedure: k7044 
o|inlining procedure: k7093 
o|contracted procedure: "(mini-srfi-1.scm:76) g205214" 
o|inlining procedure: k7093 
o|inlining procedure: k7136 
o|inlining procedure: k7136 
o|inlining procedure: k7167 
o|inlining procedure: k7167 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#fifth 
o|inlining procedure: k7328 
o|inlining procedure: k7328 
o|contracted procedure: "(mini-srfi-1.scm:123) chicken.compiler.scrutinizer#delete" 
o|inlining procedure: k7253 
o|inlining procedure: k7253 
o|inlining procedure: k7384 
o|inlining procedure: k7384 
o|inlining procedure: k7376 
o|inlining procedure: k7376 
o|inlining procedure: k7431 
o|inlining procedure: k7431 
o|inlining procedure: k7420 
o|inlining procedure: k7420 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#remove 
o|inlining procedure: k7493 
o|contracted procedure: "(mini-srfi-1.scm:143) g383392" 
o|inlining procedure: k7493 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#last 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#list-index 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#lset-difference/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#lset-intersection/eq? 
o|inlining procedure: k7775 
o|inlining procedure: k7775 
o|inlining procedure: k7811 
o|inlining procedure: k7811 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#length+ 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#find-tail 
o|merged explicitly consed rest parameter: rest551553 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#posq 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#posv 
o|propagated global variable: out626629 ##sys#standard-output 
o|substituted constant variable: a8046 
o|substituted constant variable: a8047 
o|inlining procedure: k8039 
o|propagated global variable: out626629 ##sys#standard-output 
o|inlining procedure: k8039 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#dd 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#ddd 
o|inlining procedure: "(scrutinizer.scm:166) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k8091 
o|inlining procedure: k8109 
o|inlining procedure: "(scrutinizer.scm:172) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k8109 
o|substituted constant variable: a8123 
o|substituted constant variable: a8125 
o|inlining procedure: k8091 
o|substituted constant variable: a8130 
o|removed side-effect free assignment to unused variable: report773 
o|inlining procedure: k8171 
o|inlining procedure: k8171 
o|inlining procedure: k8183 
o|inlining procedure: k8183 
o|inlining procedure: k8198 
o|inlining procedure: k8198 
o|inlining procedure: k8210 
o|inlining procedure: k8210 
o|inlining procedure: k8222 
o|inlining procedure: k8222 
o|inlining procedure: k8234 
o|inlining procedure: k8234 
o|inlining procedure: k8237 
o|inlining procedure: k8237 
o|inlining procedure: k8262 
o|inlining procedure: k8262 
o|inlining procedure: k8293 
o|inlining procedure: k8293 
o|inlining procedure: k8324 
o|inlining procedure: k8350 
o|inlining procedure: k8350 
o|inlining procedure: k8324 
o|inlining procedure: k8394 
o|inlining procedure: k8394 
o|inlining procedure: k8466 
o|inlining procedure: k8466 
o|inlining procedure: k8475 
o|inlining procedure: k8475 
o|contracted procedure: "(scrutinizer.scm:237) chicken.compiler.scrutinizer#find" 
o|inlining procedure: k7888 
o|inlining procedure: k7888 
o|inlining procedure: k8497 
o|inlining procedure: k8497 
o|inlining procedure: k8512 
o|contracted procedure: "(scrutinizer.scm:245) g925926" 
o|inlining procedure: k8517 
o|inlining procedure: k8517 
o|inlining procedure: k8512 
o|contracted procedure: "(scrutinizer.scm:259) global-result795" 
o|inlining procedure: k8424 
o|consed rest parameter at call site: "(scrutinizer.scm:228) chicken.compiler.scrutinizer#r-deprecated-identifier" 4 
o|inlining procedure: k8424 
o|consed rest parameter at call site: "(scrutinizer.scm:231) chicken.compiler.scrutinizer#r-deprecated-identifier" 4 
o|inlining procedure: k8419 
o|inlining procedure: k8419 
o|inlining procedure: k8541 
o|inlining procedure: k8541 
o|inlining procedure: k8557 
o|inlining procedure: k8575 
o|inlining procedure: "(scrutinizer.scm:265) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k8575 
o|inlining procedure: "(scrutinizer.scm:266) chicken.compiler.scrutinizer#third" 
o|substituted constant variable: a8606 
o|substituted constant variable: a8608 
o|substituted constant variable: a8610 
o|inlining procedure: k8557 
o|substituted constant variable: a8615 
o|inlining procedure: k8622 
o|contracted procedure: "(scrutinizer.scm:273) chicken.compiler.scrutinizer#r-cond-test-always-true" 
o|consed rest parameter at call site: "(scrutinizer.scm:2821) chicken.compiler.scrutinizer#report-notice" 5 
o|inlining procedure: k8622 
o|inlining procedure: k8643 
o|inlining procedure: k8643 
o|inlining procedure: k8661 
o|inlining procedure: k8661 
o|inlining procedure: "(scrutinizer.scm:291) chicken.compiler.scrutinizer#first" 
o|substituted constant variable: a8677 
o|removed unused formal parameters: (typeenv980) 
o|inlining procedure: k8737 
o|contracted procedure: "(scrutinizer.scm:311) chicken.compiler.scrutinizer#r-invalid-called-procedure-type" 
o|inlining procedure: k21758 
o|inlining procedure: "(scrutinizer.scm:2547) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k21758 
o|inlining procedure: k21796 
o|inlining procedure: k21796 
o|inlining procedure: k8737 
o|consed rest parameter at call site: "(scrutinizer.scm:316) chicken.compiler.scrutinizer#procedure-argument-types" 4 
o|inlining procedure: k8821 
o|inlining procedure: k8821 
o|inlining procedure: k8864 
o|inlining procedure: k8864 
o|inlining procedure: k8889 
o|contracted procedure: "(scrutinizer.scm:395) g11261127" 
o|inlining procedure: k8889 
o|inlining procedure: k8924 
o|contracted procedure: "(scrutinizer.scm:344) chicken.compiler.scrutinizer#r-pred-call-always-true" 
o|consed rest parameter at call site: "(scrutinizer.scm:2772) chicken.compiler.scrutinizer#report-notice" 5 
o|inlining procedure: k8924 
o|inlining procedure: k8996 
o|inlining procedure: k8996 
o|contracted procedure: "(scrutinizer.scm:355) chicken.compiler.scrutinizer#r-pred-call-always-false" 
o|consed rest parameter at call site: "(scrutinizer.scm:2797) chicken.compiler.scrutinizer#report-notice" 5 
o|inlining procedure: k9078 
o|inlining procedure: k9078 
o|inlining procedure: k9112 
o|inlining procedure: k9145 
o|inlining procedure: k9145 
o|inlining procedure: "(scrutinizer.scm:386) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:387) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:384) chicken.compiler.scrutinizer#second" 
o|consed rest parameter at call site: "(scrutinizer.scm:378) chicken.compiler.scrutinizer#append-map" 3 
o|inlining procedure: "(scrutinizer.scm:376) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9112 
o|contracted procedure: "(scrutinizer.scm:371) get-specializations803" 
o|inlining procedure: k8692 
o|inlining procedure: k8692 
o|inlining procedure: k8703 
o|inlining procedure: k8703 
o|inlining procedure: k9202 
o|inlining procedure: k9202 
o|contracted procedure: "(scrutinizer.scm:336) chicken.compiler.scrutinizer#procedure-result-types" 
o|inlining procedure: k15561 
o|inlining procedure: k15561 
o|inlining procedure: "(scrutinizer.scm:1459) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15582 
o|inlining procedure: k15604 
o|inlining procedure: k15604 
o|substituted constant variable: a15614 
o|inlining procedure: "(scrutinizer.scm:1463) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1463) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15582 
o|inlining procedure: "(scrutinizer.scm:1470) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k9231 
o|inlining procedure: k9231 
o|contracted procedure: "(scrutinizer.scm:318) chicken.compiler.scrutinizer#r-proc-call-argument-count-mismatch" 
o|consed rest parameter at call site: "(scrutinizer.scm:307) chicken.compiler.scrutinizer#append-map" 3 
o|consed rest parameter at call site: "(scrutinizer.scm:306) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k9309 
o|inlining procedure: k9309 
o|inlining procedure: k9361 
o|inlining procedure: k9361 
o|removed unused formal parameters: (vars1191) 
o|inlining procedure: k9381 
o|inlining procedure: k9419 
o|inlining procedure: k9419 
o|consed rest parameter at call site: "(scrutinizer.scm:430) chicken.compiler.scrutinizer#procedure-argument-types" 4 
o|consed rest parameter at call site: "(scrutinizer.scm:431) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k9381 
o|consed rest parameter at call site: "(scrutinizer.scm:432) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k9468 
o|inlining procedure: k9468 
o|propagated global variable: a9490 chicken.compiler.scrutinizer#d-depth 
o|propagated global variable: a9495 chicken.compiler.scrutinizer#d-depth 
o|inlining procedure: "(scrutinizer.scm:445) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9509 
o|inlining procedure: "(scrutinizer.scm:446) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9509 
o|inlining procedure: k9532 
o|inlining procedure: k9532 
o|inlining procedure: "(scrutinizer.scm:449) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9551 
o|inlining procedure: "(scrutinizer.scm:451) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k9551 
o|inlining procedure: "(scrutinizer.scm:453) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9585 
o|inlining procedure: k9637 
o|inlining procedure: k9637 
o|inlining procedure: k9687 
o|contracted procedure: "(scrutinizer.scm:496) chicken.compiler.scrutinizer#r-cond-branch-value-count-mismatch" 
o|inlining procedure: k9687 
o|inlining procedure: k9699 
o|inlining procedure: k9699 
o|inlining procedure: k9729 
o|contracted procedure: "(scrutinizer.scm:502) g13401350" 
o|inlining procedure: k9729 
o|inlining procedure: k9777 
o|inlining procedure: k9777 
o|inlining procedure: k9823 
o|inlining procedure: k9823 
o|inlining procedure: k9857 
o|inlining procedure: k9857 
o|inlining procedure: k9879 
o|inlining procedure: k9879 
o|inlining procedure: k9888 
o|inlining procedure: k9888 
o|contracted procedure: "(scrutinizer.scm:469) always-false800" 
o|inlining procedure: k8634 
o|contracted procedure: "(scrutinizer.scm:278) chicken.compiler.scrutinizer#r-cond-test-always-false" 
o|consed rest parameter at call site: "(scrutinizer.scm:2837) chicken.compiler.scrutinizer#report-notice" 5 
o|inlining procedure: k8634 
o|inlining procedure: "(scrutinizer.scm:462) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:461) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:456) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k9585 
o|inlining procedure: k9942 
o|inlining procedure: k9942 
o|inlining procedure: k9990 
o|inlining procedure: k9990 
o|inlining procedure: "(scrutinizer.scm:517) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10028 
o|inlining procedure: k10101 
o|inlining procedure: k10101 
o|inlining procedure: "(scrutinizer.scm:538) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10240 
o|contracted procedure: "(scrutinizer.scm:528) g14101420" 
o|inlining procedure: k10240 
o|inlining procedure: k10289 
o|inlining procedure: k10289 
o|removed unused parameter to known procedure: vars1191 "(scrutinizer.scm:526) initial-argument-types807" 
o|inlining procedure: "(scrutinizer.scm:523) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10028 
o|inlining procedure: k10328 
o|inlining procedure: k10328 
o|inlining procedure: k10357 
o|inlining procedure: k10357 
o|inlining procedure: k10402 
o|inlining procedure: k10402 
o|inlining procedure: k10429 
o|inlining procedure: k10429 
o|inlining procedure: k10460 
o|substituted constant variable: a10492 
o|substituted constant variable: a10493 
o|removed unused parameter to known procedure: where1927 "(scrutinizer.scm:603) chicken.compiler.scrutinizer#smash-component-types!" 
o|inlining procedure: k10526 
o|inlining procedure: k10526 
o|inlining procedure: "(scrutinizer.scm:598) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10460 
o|inlining procedure: k10554 
o|inlining procedure: k10554 
o|contracted procedure: "(scrutinizer.scm:591) chicken.compiler.scrutinizer#r-toplevel-var-assignment-type-mismatch" 
o|inlining procedure: k22840 
o|inlining procedure: k22840 
o|inlining procedure: "(scrutinizer.scm:591) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10591 
o|inlining procedure: k10614 
o|inlining procedure: k10614 
o|inlining procedure: k10591 
o|inlining procedure: "(scrutinizer.scm:580) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:578) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:576) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10657 
o|inlining procedure: k10657 
o|removed unused parameter to known procedure: typeenv980 "(scrutinizer.scm:685) call-result804" 
o|inlining procedure: k10744 
o|removed unused parameter to known procedure: where1927 "(scrutinizer.scm:693) chicken.compiler.scrutinizer#smash-component-types!" 
o|removed unused parameter to known procedure: where1927 "(scrutinizer.scm:692) chicken.compiler.scrutinizer#smash-component-types!" 
o|inlining procedure: k10744 
o|inlining procedure: k10762 
o|inlining procedure: k10762 
o|inlining procedure: k10782 
o|inlining procedure: k10814 
o|inlining procedure: k10814 
o|inlining procedure: k10782 
o|inlining procedure: k10856 
o|inlining procedure: k10892 
o|inlining procedure: k10892 
o|inlining procedure: k10918 
o|inlining procedure: k10918 
o|inlining procedure: k10927 
o|inlining procedure: k10942 
o|inlining procedure: k10942 
o|inlining procedure: k10927 
o|inlining procedure: k10988 
o|inlining procedure: k10988 
o|inlining procedure: "(scrutinizer.scm:711) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:708) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k10856 
o|inlining procedure: k11031 
o|inlining procedure: k11031 
o|inlining procedure: k11054 
o|inlining procedure: k11054 
o|inlining procedure: k11088 
o|inlining procedure: k11088 
o|consed rest parameter at call site: "(scrutinizer.scm:758) chicken.compiler.scrutinizer#procedure-argument-types" 4 
o|inlining procedure: k11164 
o|inlining procedure: k11164 
o|inlining procedure: k11202 
o|inlining procedure: k11202 
o|contracted procedure: "(scrutinizer.scm:676) chicken.compiler.scrutinizer#iota" 
o|inlining procedure: k11247 
o|inlining procedure: k11268 
o|contracted procedure: "(scrutinizer.scm:767) chicken.compiler.scrutinizer#r-zero-values-for-the" 
o|inlining procedure: "(scrutinizer.scm:767) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k11268 
o|contracted procedure: "(scrutinizer.scm:774) chicken.compiler.scrutinizer#r-type-mismatch-in-the" 
o|inlining procedure: "(scrutinizer.scm:774) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:774) chicken.compiler.scrutinizer#first" 
o|consed rest parameter at call site: "(scrutinizer.scm:772) chicken.compiler.scrutinizer#compatible-types?" 3 
o|inlining procedure: "(scrutinizer.scm:772) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:771) chicken.compiler.scrutinizer#second" 
o|contracted procedure: "(scrutinizer.scm:770) chicken.compiler.scrutinizer#r-too-many-values-for-the" 
o|inlining procedure: "(scrutinizer.scm:770) chicken.compiler.scrutinizer#first" 
o|substituted constant variable: a11333 
o|inlining procedure: "(scrutinizer.scm:765) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:764) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k11247 
o|inlining procedure: k11364 
o|contracted procedure: "(scrutinizer.scm:783) chicken.compiler.scrutinizer#fail-compiler-typecase" 
o|consed rest parameter at call site: "(scrutinizer.scm:2912) chicken.compiler.scrutinizer#string-add-indent" 2 
o|inlining procedure: k22713 
o|inlining procedure: k22713 
o|consed rest parameter at call site: "(scrutinizer.scm:2935) chicken.compiler.scrutinizer#location-name" 2 
o|substituted constant variable: a22744 
o|inlining procedure: k11364 
o|inlining procedure: "(scrutinizer.scm:777) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k11434 
o|inlining procedure: k11434 
o|inlining procedure: k11460 
o|inlining procedure: k11460 
o|substituted constant variable: a11484 
o|substituted constant variable: a11486 
o|substituted constant variable: a11488 
o|substituted constant variable: a11490 
o|substituted constant variable: a11492 
o|substituted constant variable: a11494 
o|substituted constant variable: a11499 
o|substituted constant variable: a11501 
o|substituted constant variable: a11506 
o|substituted constant variable: a11508 
o|substituted constant variable: a11510 
o|substituted constant variable: a11512 
o|substituted constant variable: a11514 
o|substituted constant variable: a11516 
o|substituted constant variable: a11518 
o|substituted constant variable: a11520 
o|substituted constant variable: a11522 
o|substituted constant variable: a11524 
o|substituted constant variable: a11526 
o|inlining procedure: k11545 
o|inlining procedure: k11545 
o|inlining procedure: k11610 
o|contracted procedure: "(scrutinizer.scm:807) g18761883" 
o|propagated global variable: out18861889 ##sys#standard-output 
o|substituted constant variable: a11578 
o|substituted constant variable: a11579 
o|propagated global variable: out18861889 ##sys#standard-output 
o|inlining procedure: k11610 
o|propagated global variable: g18821884 chicken.compiler.scrutinizer#specialization-statistics 
o|inlining procedure: "(scrutinizer.scm:801) chicken.compiler.scrutinizer#first" 
o|removed unused formal parameters: (where1927) 
o|inlining procedure: k11655 
o|inlining procedure: k11655 
o|inlining procedure: k11685 
o|inlining procedure: k11705 
o|inlining procedure: k11739 
o|inlining procedure: k11739 
o|inlining procedure: k11705 
o|inlining procedure: k11786 
o|inlining procedure: k11786 
o|inlining procedure: "(scrutinizer.scm:853) chicken.compiler.scrutinizer#third" 
o|substituted constant variable: a11824 
o|substituted constant variable: a11826 
o|substituted constant variable: a11831 
o|substituted constant variable: a11833 
o|substituted constant variable: a11835 
o|substituted constant variable: a11837 
o|inlining procedure: k11685 
o|inlining procedure: k11869 
o|inlining procedure: k11881 
o|inlining procedure: k11881 
o|inlining procedure: "(scrutinizer.scm:879) optargs?2036" 
o|inlining procedure: k11869 
o|inlining procedure: k11897 
o|inlining procedure: k11897 
o|inlining procedure: "(scrutinizer.scm:882) optargs?2036" 
o|inlining procedure: k11907 
o|inlining procedure: k11907 
o|inlining procedure: k11929 
o|removed unused parameter to known procedure: opt2066 "(scrutinizer.scm:888) match-rest2035" 
o|inlining procedure: k11929 
o|removed unused parameter to known procedure: opt2066 "(scrutinizer.scm:890) match-rest2035" 
o|inlining procedure: k11959 
o|inlining procedure: k11959 
o|removed unused formal parameters: (opt2066) 
o|contracted procedure: "(scrutinizer.scm:896) chicken.compiler.scrutinizer#span" 
o|inlining procedure: k6822 
o|inlining procedure: k6822 
o|inlining procedure: k12013 
o|inlining procedure: k12023 
o|inlining procedure: k12023 
o|inlining procedure: k12013 
o|inlining procedure: k12040 
o|inlining procedure: k12040 
o|substituted constant variable: a12050 
o|inlining procedure: k12057 
o|inlining procedure: k12057 
o|inlining procedure: k12069 
o|inlining procedure: k12069 
o|inlining procedure: k12087 
o|inlining procedure: k12087 
o|substituted constant variable: a12113 
o|substituted constant variable: a12118 
o|inlining procedure: k12153 
o|inlining procedure: k12153 
o|inlining procedure: k12186 
o|inlining procedure: k12186 
o|inlining procedure: k12197 
o|inlining procedure: k12212 
o|inlining procedure: k12212 
o|inlining procedure: "(scrutinizer.scm:938) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:937) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:936) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12197 
o|inlining procedure: k12241 
o|inlining procedure: k12241 
o|inlining procedure: k12253 
o|inlining procedure: k12253 
o|inlining procedure: "(scrutinizer.scm:948) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:947) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:946) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:945) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12278 
o|inlining procedure: k12278 
o|inlining procedure: "(scrutinizer.scm:950) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:949) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:935) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12329 
o|inlining procedure: k12344 
o|inlining procedure: k12344 
o|inlining procedure: "(scrutinizer.scm:961) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:960) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:959) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12329 
o|inlining procedure: "(scrutinizer.scm:963) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:962) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:958) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12324 
o|inlining procedure: k12324 
o|inlining procedure: k12405 
o|inlining procedure: k12405 
o|inlining procedure: k12420 
o|inlining procedure: k12420 
o|inlining procedure: k12435 
o|inlining procedure: k12435 
o|inlining procedure: k12455 
o|inlining procedure: k12455 
o|inlining procedure: k12504 
o|inlining procedure: k12504 
o|inlining procedure: k12564 
o|inlining procedure: k12564 
o|inlining procedure: "(scrutinizer.scm:1006) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k12596 
o|inlining procedure: "(scrutinizer.scm:1008) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k12596 
o|inlining procedure: k12615 
o|inlining procedure: k12615 
o|inlining procedure: k12626 
o|inlining procedure: k12638 
o|inlining procedure: k12638 
o|inlining procedure: k12626 
o|inlining procedure: k12655 
o|inlining procedure: k12655 
o|inlining procedure: k12672 
o|inlining procedure: k12672 
o|inlining procedure: k12698 
o|inlining procedure: k12698 
o|inlining procedure: k12734 
o|inlining procedure: k12734 
o|inlining procedure: "(scrutinizer.scm:1030) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1030) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12769 
o|inlining procedure: k12769 
o|inlining procedure: k12809 
o|contracted procedure: "(scrutinizer.scm:1037) chicken.compiler.scrutinizer#lset<=/eq?" 
o|inlining procedure: "(scrutinizer.scm:1037) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1037) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12809 
o|inlining procedure: "(scrutinizer.scm:1035) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1035) chicken.compiler.scrutinizer#third" 
o|substituted constant variable: a12841 
o|substituted constant variable: a12846 
o|substituted constant variable: a12848 
o|substituted constant variable: a12853 
o|substituted constant variable: a12855 
o|substituted constant variable: a12857 
o|substituted constant variable: a12859 
o|substituted constant variable: a12861 
o|inlining procedure: k12862 
o|inlining procedure: "(scrutinizer.scm:1040) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k12862 
o|inlining procedure: "(scrutinizer.scm:1042) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k12894 
o|inlining procedure: k12907 
o|inlining procedure: k12919 
o|inlining procedure: "(scrutinizer.scm:1049) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k12919 
o|inlining procedure: "(scrutinizer.scm:1048) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1048) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12907 
o|inlining procedure: k12946 
o|inlining procedure: k12966 
o|inlining procedure: k12966 
o|inlining procedure: "(scrutinizer.scm:1053) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1052) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1052) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k12946 
o|substituted constant variable: a12993 
o|substituted constant variable: a12995 
o|inlining procedure: k12894 
o|inlining procedure: k13002 
o|inlining procedure: k13015 
o|inlining procedure: "(scrutinizer.scm:1064) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1063) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1063) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13015 
o|inlining procedure: k13042 
o|inlining procedure: k13054 
o|inlining procedure: "(scrutinizer.scm:1071) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k13054 
o|inlining procedure: "(scrutinizer.scm:1067) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1067) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13042 
o|substituted constant variable: a13095 
o|substituted constant variable: a13097 
o|inlining procedure: k13002 
o|inlining procedure: k13098 
o|inlining procedure: k13110 
o|inlining procedure: "(scrutinizer.scm:1080) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13110 
o|inlining procedure: k13098 
o|inlining procedure: k13146 
o|inlining procedure: "(scrutinizer.scm:1087) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13146 
o|inlining procedure: k13174 
o|inlining procedure: k13186 
o|inlining procedure: "(scrutinizer.scm:1095) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13186 
o|inlining procedure: k13174 
o|inlining procedure: k13222 
o|inlining procedure: "(scrutinizer.scm:1102) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13222 
o|inlining procedure: k13317 
o|inlining procedure: k13317 
o|inlining procedure: k13439 
o|inlining procedure: k13439 
o|inlining procedure: k13457 
o|inlining procedure: k13457 
o|inlining procedure: k13479 
o|inlining procedure: k13479 
o|inlining procedure: k13514 
o|inlining procedure: "(scrutinizer.scm:1157) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k13552 
o|inlining procedure: k13552 
o|inlining procedure: k13589 
o|contracted procedure: "(scrutinizer.scm:1146) g24312440" 
o|contracted procedure: "(scrutinizer.scm:1148) chicken.compiler.scrutinizer#make-tv" 
o|substituted constant variable: a15907 
o|inlining procedure: "(scrutinizer.scm:1147) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k13589 
o|inlining procedure: "(scrutinizer.scm:1144) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k13620 
o|inlining procedure: k13635 
o|inlining procedure: k13635 
o|inlining procedure: k13651 
o|inlining procedure: k13676 
o|contracted procedure: "(scrutinizer.scm:1168) g25082509" 
o|contracted procedure: "(scrutinizer.scm:1180) chicken.compiler.scrutinizer#merge-result-types" 
o|inlining procedure: k14655 
o|inlining procedure: k14655 
o|inlining procedure: k13735 
o|inlining procedure: k13735 
o|inlining procedure: k13676 
o|inlining procedure: k13651 
o|inlining procedure: k13760 
o|inlining procedure: k13760 
o|inlining procedure: k13778 
o|inlining procedure: k13778 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13814 
o|inlining procedure: k13814 
o|inlining procedure: k13879 
o|inlining procedure: k13879 
o|inlining procedure: k13897 
o|inlining procedure: k13897 
o|consed rest parameter at call site: "(scrutinizer.scm:1187) chicken.compiler.scrutinizer#append-map" 3 
o|substituted constant variable: a13921 
o|inlining procedure: k13940 
o|inlining procedure: k13940 
o|inlining procedure: k13620 
o|inlining procedure: k13983 
o|inlining procedure: k13983 
o|inlining procedure: "(scrutinizer.scm:1212) chicken.compiler.scrutinizer#third" 
o|contracted procedure: "(scrutinizer.scm:1212) chicken.compiler.scrutinizer#lset-union/eq?" 
o|inlining procedure: k7678 
o|contracted procedure: "(mini-srfi-1.scm:171) g466467" 
o|inlining procedure: k7703 
o|contracted procedure: "(mini-srfi-1.scm:173) g482483" 
o|inlining procedure: k7719 
o|inlining procedure: k7719 
o|inlining procedure: k7703 
o|inlining procedure: k7678 
o|inlining procedure: "(scrutinizer.scm:1212) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1209) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1208) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k14021 
o|inlining procedure: "(scrutinizer.scm:1217) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1216) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k14021 
o|inlining procedure: k14069 
o|inlining procedure: k14069 
o|inlining procedure: "(scrutinizer.scm:1223) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k14084 
o|inlining procedure: "(scrutinizer.scm:1228) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k14084 
o|inlining procedure: k14114 
o|inlining procedure: k14114 
o|inlining procedure: k14137 
o|inlining procedure: k14137 
o|inlining procedure: k14170 
o|inlining procedure: k14193 
o|inlining procedure: k14193 
o|inlining procedure: k14170 
o|inlining procedure: k14248 
o|inlining procedure: k14248 
o|inlining procedure: k14266 
o|inlining procedure: k14266 
o|inlining procedure: k14313 
o|inlining procedure: k14313 
o|inlining procedure: "(scrutinizer.scm:1244) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1244) chicken.compiler.scrutinizer#second" 
o|substituted constant variable: a14366 
o|substituted constant variable: a14368 
o|substituted constant variable: a14370 
o|substituted constant variable: a14372 
o|substituted constant variable: a14374 
o|substituted constant variable: a14376 
o|substituted constant variable: a14378 
o|substituted constant variable: a14380 
o|substituted constant variable: a14382 
o|inlining procedure: k13514 
o|contracted procedure: "(scrutinizer.scm:1251) chicken.compiler.scrutinizer#lset-adjoin/eq?" 
o|inlining procedure: k7592 
o|inlining procedure: k7592 
o|inlining procedure: k14404 
o|inlining procedure: k14424 
o|inlining procedure: k14424 
o|inlining procedure: k14404 
o|inlining procedure: k14462 
o|substituted constant variable: chicken.compiler.scrutinizer#constant747 
o|inlining procedure: k14462 
o|inlining procedure: k14474 
o|substituted constant variable: a14493 
o|inlining procedure: k14486 
o|inlining procedure: k14486 
o|inlining procedure: k14474 
o|inlining procedure: k14500 
o|inlining procedure: k14500 
o|inlining procedure: k14551 
o|inlining procedure: k14551 
o|substituted constant variable: a14607 
o|inlining procedure: k14600 
o|inlining procedure: k14600 
o|merged explicitly consed rest parameter: rest27782781 
o|inlining procedure: k14718 
o|inlining procedure: k14718 
o|contracted procedure: "(scrutinizer.scm:1322) chicken.compiler.scrutinizer#with-trail-restore" 
o|inlining procedure: k15105 
o|inlining procedure: k15105 
o|contracted procedure: "(scrutinizer.scm:1365) refine2832" 
o|inlining procedure: k14811 
o|inlining procedure: k14811 
o|inlining procedure: k14836 
o|inlining procedure: "(scrutinizer.scm:1342) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1341) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k14836 
o|inlining procedure: k14872 
o|inlining procedure: "(scrutinizer.scm:1345) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k14872 
o|inlining procedure: "(scrutinizer.scm:1344) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k14890 
o|inlining procedure: k14890 
o|inlining procedure: k14905 
o|inlining procedure: k14905 
o|inlining procedure: k14924 
o|inlining procedure: k14962 
o|inlining procedure: k14962 
o|inlining procedure: k14924 
o|contracted procedure: "(scrutinizer.scm:1362) chicken.compiler.scrutinizer#type-min" 
o|inlining procedure: k14743 
o|inlining procedure: k14743 
o|substituted constant variable: a15018 
o|inlining procedure: k15013 
o|inlining procedure: k15025 
o|inlining procedure: k15025 
o|inlining procedure: k15013 
o|inlining procedure: k15057 
o|inlining procedure: k15069 
o|inlining procedure: k15069 
o|inlining procedure: k15057 
o|substituted constant variable: a15089 
o|substituted constant variable: a15095 
o|inlining procedure: k15124 
o|inlining procedure: k15124 
o|inlining procedure: k15134 
o|inlining procedure: "(scrutinizer.scm:1376) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15134 
o|inlining procedure: k15153 
o|inlining procedure: k15153 
o|substituted constant variable: a15165 
o|substituted constant variable: a15167 
o|substituted constant variable: a15169 
o|inlining procedure: k15173 
o|inlining procedure: "(scrutinizer.scm:1384) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15193 
o|inlining procedure: k15211 
o|inlining procedure: k15211 
o|inlining procedure: k15193 
o|substituted constant variable: a15218 
o|substituted constant variable: a15220 
o|inlining procedure: k15173 
o|inlining procedure: k15224 
o|inlining procedure: "(scrutinizer.scm:1395) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15244 
o|inlining procedure: "(scrutinizer.scm:1399) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1400) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1397) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15244 
o|substituted constant variable: a15275 
o|substituted constant variable: a15277 
o|inlining procedure: k15224 
o|inlining procedure: k15281 
o|inlining procedure: "(scrutinizer.scm:1406) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15301 
o|inlining procedure: "(scrutinizer.scm:1408) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15301 
o|substituted constant variable: a15332 
o|substituted constant variable: a15334 
o|inlining procedure: k15281 
o|merged explicitly consed rest parameter: rest29832987 
o|inlining procedure: k15347 
o|inlining procedure: k15366 
o|inlining procedure: k15366 
o|inlining procedure: k15383 
o|inlining procedure: k15383 
o|inlining procedure: k15391 
o|consed rest parameter at call site: "(scrutinizer.scm:1439) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k15391 
o|inlining procedure: "(scrutinizer.scm:1424) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1425) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1423) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1423) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15347 
o|inlining procedure: "(scrutinizer.scm:1447) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15504 
o|inlining procedure: k15504 
o|inlining procedure: "(scrutinizer.scm:1450) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15496 
o|inlining procedure: k15496 
o|consed rest parameter at call site: "(scrutinizer.scm:1454) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k15689 
o|inlining procedure: k15709 
o|inlining procedure: k15709 
o|inlining procedure: k15722 
o|inlining procedure: "(scrutinizer.scm:1479) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15722 
o|substituted constant variable: a15736 
o|substituted constant variable: a15738 
o|inlining procedure: k15689 
o|inlining procedure: k15742 
o|inlining procedure: k15742 
o|inlining procedure: k15762 
o|inlining procedure: k15775 
o|inlining procedure: k15785 
o|inlining procedure: k15785 
o|inlining procedure: "(scrutinizer.scm:1493) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15775 
o|inlining procedure: k15801 
o|inlining procedure: "(scrutinizer.scm:1497) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15801 
o|substituted constant variable: a15815 
o|substituted constant variable: a15817 
o|inlining procedure: k15762 
o|inlining procedure: k15824 
o|inlining procedure: k15824 
o|inlining procedure: k15834 
o|inlining procedure: k15834 
o|inlining procedure: "(scrutinizer.scm:1505) chicken.compiler.scrutinizer#third" 
o|substituted constant variable: a15859 
o|substituted constant variable: a15861 
o|inlining procedure: k15865 
o|inlining procedure: k15880 
o|inlining procedure: "(scrutinizer.scm:1514) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15880 
o|substituted constant variable: a15894 
o|substituted constant variable: a15896 
o|inlining procedure: "(scrutinizer.scm:1512) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k15865 
o|inlining procedure: k15918 
o|inlining procedure: "(scrutinizer.scm:1532) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k15938 
o|inlining procedure: k15962 
o|inlining procedure: k15980 
o|inlining procedure: k15980 
o|inlining procedure: k15962 
o|inlining procedure: k16007 
o|inlining procedure: k16007 
o|inlining procedure: "(scrutinizer.scm:1535) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k16036 
o|inlining procedure: k16053 
o|inlining procedure: k16053 
o|inlining procedure: k16036 
o|inlining procedure: k16080 
o|inlining procedure: k16080 
o|inlining procedure: "(scrutinizer.scm:1539) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1534) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1534) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k15938 
o|inlining procedure: "(scrutinizer.scm:1549) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k16166 
o|contracted procedure: "(scrutinizer.scm:1543) g32183227" 
o|inlining procedure: k16134 
o|inlining procedure: k16134 
o|inlining procedure: "(scrutinizer.scm:1546) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1546) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k16166 
o|inlining procedure: "(scrutinizer.scm:1547) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k16197 
o|inlining procedure: k16213 
o|inlining procedure: k16213 
o|inlining procedure: k16197 
o|substituted constant variable: a16234 
o|substituted constant variable: a16236 
o|substituted constant variable: a16238 
o|substituted constant variable: a16240 
o|inlining procedure: k15918 
o|inlining procedure: k16250 
o|inlining procedure: k16250 
o|inlining procedure: k16308 
o|inlining procedure: "(scrutinizer.scm:1575) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1574) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k16308 
o|inlining procedure: "(scrutinizer.scm:1571) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k16300 
o|inlining procedure: k16300 
o|inlining procedure: k16353 
o|inlining procedure: k16353 
o|inlining procedure: k16371 
o|inlining procedure: k16399 
o|inlining procedure: k16399 
o|inlining procedure: k16371 
o|inlining procedure: "(scrutinizer.scm:1585) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k16447 
o|inlining procedure: "(scrutinizer.scm:1587) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1587) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k16447 
o|inlining procedure: k16506 
o|inlining procedure: k16506 
o|inlining procedure: k16537 
o|inlining procedure: k16572 
o|inlining procedure: k16572 
o|inlining procedure: k16595 
o|inlining procedure: k16595 
o|inlining procedure: k16629 
o|inlining procedure: k16629 
o|inlining procedure: k16641 
o|inlining procedure: k16641 
o|inlining procedure: k16660 
o|inlining procedure: k16660 
o|inlining procedure: k16537 
o|substituted constant variable: a16702 
o|inlining procedure: k16706 
o|inlining procedure: k16706 
o|inlining procedure: k16718 
o|inlining procedure: k16718 
o|substituted constant variable: a16725 
o|substituted constant variable: a16727 
o|substituted constant variable: a16729 
o|substituted constant variable: a16731 
o|substituted constant variable: a16733 
o|substituted constant variable: a16738 
o|substituted constant variable: a16740 
o|substituted constant variable: a16742 
o|substituted constant variable: a16744 
o|inlining procedure: k16758 
o|inlining procedure: k17080 
o|contracted procedure: "(scrutinizer.scm:1620) g34763483" 
o|inlining procedure: k16813 
o|inlining procedure: k16813 
o|consed rest parameter at call site: "(scrutinizer.scm:1665) chicken.compiler.scrutinizer#compatible-types?" 3 
o|substituted constant variable: a16856 
o|substituted constant variable: a16857 
o|inlining procedure: k16889 
o|inlining procedure: k16916 
o|inlining procedure: k16916 
o|inlining procedure: k16939 
o|inlining procedure: k16939 
o|inlining procedure: k16967 
o|inlining procedure: k16967 
o|substituted constant variable: a17007 
o|substituted constant variable: a17009 
o|substituted constant variable: a17011 
o|substituted constant variable: a17013 
o|substituted constant variable: a17015 
o|inlining procedure: k17022 
o|inlining procedure: "(scrutinizer.scm:1654) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1654) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k17022 
o|inlining procedure: k16889 
o|inlining procedure: k17080 
o|substituted constant variable: a17110 
o|substituted constant variable: a17111 
o|inlining procedure: k16758 
o|inlining procedure: k17130 
o|inlining procedure: k17130 
o|inlining procedure: k17169 
o|contracted procedure: "(scrutinizer.scm:1699) chicken.compiler.scrutinizer#cons*" 
o|inlining procedure: k7197 
o|inlining procedure: k7197 
o|inlining procedure: k17216 
o|inlining procedure: k17275 
o|inlining procedure: k17275 
o|inlining procedure: k17281 
o|inlining procedure: "(scrutinizer.scm:1713) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1713) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k17281 
o|substituted constant variable: a17303 
o|substituted constant variable: a17305 
o|inlining procedure: k17216 
o|inlining procedure: k17169 
o|substituted constant variable: a17313 
o|merged explicitly consed rest parameter: rest36123614 
o|inlining procedure: k17346 
o|inlining procedure: k17346 
o|consed rest parameter at call site: "(scrutinizer.scm:1739) chicken.compiler.scrutinizer#source-node" 2 
o|inlining procedure: k17400 
o|inlining procedure: k17400 
o|consed rest parameter at call site: "(scrutinizer.scm:1742) chicken.compiler.scrutinizer#source-node" 2 
o|inlining procedure: k17505 
o|inlining procedure: k17518 
o|inlining procedure: k17518 
o|inlining procedure: k17533 
o|inlining procedure: k17533 
o|inlining procedure: k17505 
o|inlining procedure: k17569 
o|inlining procedure: k17569 
o|inlining procedure: k17607 
o|inlining procedure: k17607 
o|substituted constant variable: a17627 
o|substituted constant variable: a17639 
o|inlining procedure: k17692 
o|inlining procedure: k17692 
o|inlining procedure: k17720 
o|inlining procedure: k17720 
o|inlining procedure: k17732 
o|inlining procedure: k17732 
o|inlining procedure: k17750 
o|inlining procedure: k17750 
o|inlining procedure: k17758 
o|inlining procedure: k17773 
o|inlining procedure: k17773 
o|inlining procedure: k17758 
o|inlining procedure: k17812 
o|inlining procedure: k17812 
o|inlining procedure: k17853 
o|inlining procedure: k17853 
o|inlining procedure: k17865 
o|inlining procedure: k17865 
o|inlining procedure: k17880 
o|inlining procedure: k17880 
o|inlining procedure: k17892 
o|inlining procedure: k17892 
o|inlining procedure: k17907 
o|inlining procedure: k17907 
o|inlining procedure: k17916 
o|inlining procedure: k17916 
o|inlining procedure: k17922 
o|inlining procedure: "(scrutinizer.scm:1855) chicken.compiler.scrutinizer#second" 
o|substituted constant variable: a17945 
o|inlining procedure: k17922 
o|inlining procedure: k17956 
o|inlining procedure: "(scrutinizer.scm:1881) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k17996 
o|inlining procedure: k18011 
o|inlining procedure: k18011 
o|inlining procedure: k17996 
o|inlining procedure: "(scrutinizer.scm:1879) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18031 
o|inlining procedure: k18031 
o|inlining procedure: k18052 
o|inlining procedure: k18052 
o|substituted constant variable: a18065 
o|inlining procedure: k18080 
o|inlining procedure: k18080 
o|inlining procedure: "(scrutinizer.scm:1869) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1858) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k17956 
o|substituted constant variable: a18115 
o|inlining procedure: k18128 
o|inlining procedure: k18128 
o|inlining procedure: k18123 
o|inlining procedure: k18123 
o|inlining procedure: k18147 
o|inlining procedure: k18177 
o|inlining procedure: k18177 
o|inlining procedure: k18147 
o|inlining procedure: k18208 
o|inlining procedure: k18220 
o|inlining procedure: k18220 
o|inlining procedure: "(scrutinizer.scm:1897) chicken.compiler.scrutinizer#second" 
o|substituted constant variable: a18230 
o|inlining procedure: k18208 
o|inlining procedure: k18241 
o|inlining procedure: "(scrutinizer.scm:1899) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18241 
o|substituted constant variable: a18257 
o|inlining procedure: k18262 
o|inlining procedure: k18277 
o|inlining procedure: k18313 
o|inlining procedure: "(scrutinizer.scm:1906) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18313 
o|inlining procedure: "(scrutinizer.scm:1905) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:1904) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18277 
o|contracted procedure: "(scrutinizer.scm:1903) g38733874" 
o|inlining procedure: k18298 
o|inlining procedure: k18298 
o|contracted procedure: "(scrutinizer.scm:160) g38793880" 
o|inlining procedure: k18284 
o|inlining procedure: k18284 
o|inlining procedure: "(scrutinizer.scm:1902) chicken.compiler.scrutinizer#third" 
o|substituted constant variable: a18338 
o|inlining procedure: k18262 
o|inlining procedure: k18369 
o|contracted procedure: "(scrutinizer.scm:1916) upto3704" 
o|inlining procedure: k17664 
o|inlining procedure: k17664 
o|inlining procedure: k18369 
o|inlining procedure: k18409 
o|inlining procedure: k18409 
o|inlining procedure: "(scrutinizer.scm:1921) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:1921) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k18439 
o|inlining procedure: k18439 
o|substituted constant variable: a18456 
o|substituted constant variable: a18479 
o|inlining procedure: k18472 
o|inlining procedure: k18486 
o|inlining procedure: "(scrutinizer.scm:1930) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18486 
o|substituted constant variable: a18509 
o|inlining procedure: k18472 
o|substituted constant variable: a18521 
o|inlining procedure: k18522 
o|inlining procedure: k18538 
o|inlining procedure: k18538 
o|inlining procedure: k18522 
o|inlining procedure: k18581 
o|inlining procedure: k18605 
o|inlining procedure: k18605 
o|inlining procedure: k18617 
o|inlining procedure: k18617 
o|substituted constant variable: a18648 
o|inlining procedure: k18581 
o|inlining procedure: k18659 
o|inlining procedure: k18677 
o|inlining procedure: k18689 
o|inlining procedure: k18729 
o|inlining procedure: k18750 
o|inlining procedure: k18750 
o|inlining procedure: k18729 
o|inlining procedure: k18689 
o|inlining procedure: k18677 
o|inlining procedure: k18659 
o|inlining procedure: k18842 
o|inlining procedure: k18854 
o|inlining procedure: "(scrutinizer.scm:1886) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18854 
o|inlining procedure: "(scrutinizer.scm:1884) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k18842 
o|substituted constant variable: chicken.compiler.scrutinizer#constant738 
o|inlining procedure: k18921 
o|inlining procedure: k18921 
o|inlining procedure: k18956 
o|inlining procedure: k18956 
o|inlining procedure: k18982 
o|inlining procedure: k18982 
o|inlining procedure: k18906 
o|inlining procedure: k18906 
o|inlining procedure: k19041 
o|inlining procedure: k19041 
o|inlining procedure: k19094 
o|inlining procedure: k19094 
o|inlining procedure: k19071 
o|inlining procedure: k19119 
o|inlining procedure: k19119 
o|inlining procedure: k19134 
o|inlining procedure: k19161 
o|inlining procedure: k19161 
o|inlining procedure: k19183 
o|inlining procedure: k19183 
o|inlining procedure: "(scrutinizer.scm:2027) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19134 
o|inlining procedure: "(scrutinizer.scm:2029) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:2020) chicken.compiler.scrutinizer#second" 
o|substituted constant variable: a19238 
o|substituted constant variable: a19240 
o|consed rest parameter at call site: "(scrutinizer.scm:2014) chicken.compiler.scrutinizer#append-map" 3 
o|inlining procedure: k19248 
o|inlining procedure: k19248 
o|inlining procedure: "(scrutinizer.scm:2013) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19071 
o|inlining procedure: "(scrutinizer.scm:2006) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19301 
o|inlining procedure: k19301 
o|inlining procedure: k19335 
o|inlining procedure: k19335 
o|inlining procedure: k19363 
o|inlining procedure: k19363 
o|inlining procedure: "(scrutinizer.scm:2050) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:2050) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k19397 
o|inlining procedure: k19397 
o|inlining procedure: "(scrutinizer.scm:2051) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:2049) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:2045) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:2044) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19461 
o|inlining procedure: k19461 
o|inlining procedure: k19502 
o|inlining procedure: k19514 
o|inlining procedure: k19526 
o|inlining procedure: k19538 
o|inlining procedure: k19550 
o|inlining procedure: k19550 
o|inlining procedure: "(scrutinizer.scm:2100) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19538 
o|inlining procedure: "(scrutinizer.scm:2098) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k19526 
o|inlining procedure: k19514 
o|inlining procedure: "(scrutinizer.scm:2095) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19502 
o|inlining procedure: k19604 
o|inlining procedure: k19604 
o|inlining procedure: k19610 
o|inlining procedure: "(scrutinizer.scm:2114) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19610 
o|inlining procedure: "(scrutinizer.scm:2113) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19661 
o|inlining procedure: k19661 
o|inlining procedure: k19673 
o|inlining procedure: k19685 
o|inlining procedure: k19685 
o|inlining procedure: k19673 
o|inlining procedure: k19700 
o|contracted procedure: "(scrutinizer.scm:2157) chicken.compiler.scrutinizer#split-at" 
o|inlining procedure: k6924 
o|inlining procedure: k6924 
o|inlining procedure: k19700 
o|inlining procedure: k19737 
o|inlining procedure: k19749 
o|inlining procedure: k19749 
o|inlining procedure: "(scrutinizer.scm:2167) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:2166) chicken.compiler.scrutinizer#second" 
o|inlining procedure: "(scrutinizer.scm:2165) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19737 
o|inlining procedure: "(scrutinizer.scm:2160) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:2155) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19852 
o|inlining procedure: k19852 
o|inlining procedure: k19858 
o|inlining procedure: k19870 
o|inlining procedure: k19882 
o|inlining procedure: k19891 
o|inlining procedure: k19891 
o|inlining procedure: k19907 
o|inlining procedure: k19907 
o|contracted procedure: "(scrutinizer.scm:2194) proper-list-type-length4385" 
o|inlining procedure: k19814 
o|inlining procedure: k19814 
o|inlining procedure: k19882 
o|inlining procedure: "(scrutinizer.scm:2186) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k19870 
o|inlining procedure: "(scrutinizer.scm:2184) chicken.compiler.scrutinizer#third" 
o|inlining procedure: "(scrutinizer.scm:2183) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k19858 
o|substituted constant variable: a19934 
o|inlining procedure: k19978 
o|inlining procedure: k19978 
o|inlining procedure: k19999 
o|inlining procedure: k20015 
o|contracted procedure: "(scrutinizer.scm:2258) g46634664" 
o|inlining procedure: k20015 
o|inlining procedure: k19999 
o|inlining procedure: k20049 
o|inlining procedure: k20049 
o|contracted procedure: "(scrutinizer.scm:2273) potentially-proper-list?4637" 
o|inlining procedure: k20112 
o|inlining procedure: k20112 
o|inlining procedure: k20234 
o|removed side-effect free assignment to unused variable: anyinst4729 
o|inlining procedure: k20234 
o|inlining procedure: k20310 
o|inlining procedure: k20310 
o|inlining procedure: k20331 
o|inlining procedure: k20331 
o|contracted procedure: "(scrutinizer.scm:2337) chicken.compiler.scrutinizer#concatenate" 
o|inlining procedure: k7225 
o|inlining procedure: k7225 
o|inlining procedure: k20379 
o|inlining procedure: k20379 
o|inlining procedure: k20423 
o|inlining procedure: k20480 
o|inlining procedure: k20480 
o|inlining procedure: k20423 
o|inlining procedure: k20517 
o|inlining procedure: k20517 
o|inlining procedure: k20538 
o|inlining procedure: k20538 
o|merged explicitly consed rest parameter: rest48624864 
o|inlining procedure: k20556 
o|substituted constant variable: a20566 
o|inlining procedure: k20556 
o|inlining procedure: k20583 
o|inlining procedure: k20583 
o|inlining procedure: k20598 
o|inlining procedure: k20598 
o|merged explicitly consed rest parameter: rest49114913 
o|consed rest parameter at call site: "(scrutinizer.scm:2400) conv4902" 2 
o|inlining procedure: k20673 
o|inlining procedure: k20673 
o|substituted constant variable: a20699 
o|substituted constant variable: a20700 
o|inlining procedure: "(scrutinizer.scm:2406) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k20737 
o|inlining procedure: k20737 
o|inlining procedure: "(scrutinizer.scm:2406) chicken.compiler.scrutinizer#second" 
o|contracted procedure: "(scrutinizer.scm:2411) pp-tv4901" 
o|inlining procedure: k20771 
o|consed rest parameter at call site: "(scrutinizer.scm:2412) conv4902" 2 
o|inlining procedure: "(scrutinizer.scm:2412) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k20808 
o|inlining procedure: k20808 
o|inlining procedure: "(scrutinizer.scm:2411) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k20771 
o|substituted constant variable: a20849 
o|inlining procedure: k20845 
o|inlining procedure: k20869 
o|inlining procedure: k20869 
o|inlining procedure: k20845 
o|inlining procedure: k20906 
o|inlining procedure: k20944 
o|inlining procedure: k20944 
o|inlining procedure: k20955 
o|inlining procedure: k20955 
o|inlining procedure: k20967 
o|inlining procedure: k20967 
o|inlining procedure: k20985 
o|inlining procedure: k20985 
o|inlining procedure: k21019 
o|inlining procedure: k21019 
o|inlining procedure: k20906 
o|inlining procedure: "(scrutinizer.scm:2410) chicken.compiler.scrutinizer#second" 
o|consed rest parameter at call site: "(scrutinizer.scm:2431) chicken.compiler.scrutinizer#string-add-indent" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:2430) conv4902" 2 
o|inlining procedure: k21113 
o|inlining procedure: k21113 
o|inlining procedure: k21128 
o|inlining procedure: k21156 
o|inlining procedure: k21156 
o|substituted constant variable: a21197 
o|inlining procedure: k21128 
o|consed rest parameter at call site: "(scrutinizer.scm:2450) chicken.compiler.scrutinizer#string-add-indent" 2 
o|inlining procedure: k21236 
o|substituted constant variable: a21246 
o|substituted constant variable: a21247 
o|inlining procedure: k21236 
o|contracted procedure: "(scrutinizer.scm:2457) chicken.compiler.scrutinizer#node-line-number" 
o|contracted procedure: "(scrutinizer.scm:1750) chicken.compiler.scrutinizer#node-debug-info" 
o|inlining procedure: k17447 
o|inlining procedure: k17462 
o|inlining procedure: k17462 
o|inlining procedure: k17447 
o|substituted constant variable: a17496 
o|substituted constant variable: a17498 
o|consed rest parameter at call site: "(scrutinizer.scm:1750) chicken.compiler.scrutinizer#source-node" 2 
o|merged explicitly consed rest parameter: rest51745176 
o|inlining procedure: k21294 
o|inlining procedure: k21294 
o|inlining procedure: k21313 
o|inlining procedure: k21331 
o|substituted constant variable: a21367 
o|substituted constant variable: a21368 
o|substituted constant variable: a21385 
o|substituted constant variable: a21386 
o|inlining procedure: k21331 
o|inlining procedure: k21313 
o|contracted procedure: "(scrutinizer.scm:2481) lname5182" 
o|substituted constant variable: a21276 
o|substituted constant variable: a21277 
o|inlining procedure: k21269 
o|inlining procedure: k21269 
o|inlining procedure: k21439 
o|inlining procedure: "(scrutinizer.scm:2487) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:2487) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k21439 
o|substituted constant variable: a21503 
o|substituted constant variable: a21504 
o|inlining procedure: k21496 
o|substituted constant variable: a21527 
o|substituted constant variable: a21528 
o|inlining procedure: k21496 
o|inlining procedure: k21564 
o|inlining procedure: k21564 
o|inlining procedure: "(scrutinizer.scm:2516) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k21600 
o|inlining procedure: k21612 
o|inlining procedure: k21612 
o|inlining procedure: k21600 
o|inlining procedure: "(scrutinizer.scm:2502) chicken.compiler.scrutinizer#first" 
o|inlining procedure: "(scrutinizer.scm:2521) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k21683 
o|inlining procedure: k21683 
o|substituted constant variable: a21693 
o|inlining procedure: k21698 
o|consed rest parameter at call site: "(scrutinizer.scm:2532) chicken.compiler.scrutinizer#string-add-indent" 2 
o|substituted constant variable: a21735 
o|substituted constant variable: a21736 
o|consed rest parameter at call site: "(scrutinizer.scm:2534) chicken.compiler.scrutinizer#location-name" 2 
o|inlining procedure: k21698 
o|merged explicitly consed rest parameter: args5324 
o|substituted constant variable: a21989 
o|substituted constant variable: a21990 
o|inlining procedure: k22007 
o|inlining procedure: k22007 
o|substituted constant variable: a22024 
o|substituted constant variable: a22025 
o|substituted constant variable: a22083 
o|substituted constant variable: a22084 
o|substituted constant variable: a22101 
o|substituted constant variable: a22102 
o|substituted constant variable: a22132 
o|substituted constant variable: a22133 
o|substituted constant variable: a22150 
o|substituted constant variable: a22151 
o|inlining procedure: k22201 
o|inlining procedure: k22201 
o|substituted constant variable: a22218 
o|substituted constant variable: a22219 
o|inlining procedure: k22267 
o|substituted constant variable: a22281 
o|substituted constant variable: a22282 
o|inlining procedure: k22267 
o|substituted constant variable: a22307 
o|substituted constant variable: a22308 
o|substituted constant variable: a22332 
o|substituted constant variable: a22333 
o|inlining procedure: k22387 
o|substituted constant variable: a22401 
o|substituted constant variable: a22402 
o|inlining procedure: k22387 
o|substituted constant variable: a22427 
o|substituted constant variable: a22428 
o|substituted constant variable: a22452 
o|substituted constant variable: a22453 
o|merged explicitly consed rest parameter: rest56595663 
o|substituted constant variable: a22863 
o|substituted constant variable: a22864 
o|substituted constant variable: a22901 
o|substituted constant variable: a22902 
o|contracted procedure: "(scrutinizer.scm:2306) complex-object-constructor-result-type-special-case4690" 
o|inlining procedure: k20135 
o|inlining procedure: k20135 
o|inlining procedure: k20144 
o|inlining procedure: k20156 
o|inlining procedure: k20168 
o|consed rest parameter at call site: "(scrutinizer.scm:2302) chicken.compiler.scrutinizer#make-list" 2 
o|inlining procedure: k20168 
o|inlining procedure: "(scrutinizer.scm:2299) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k20156 
o|inlining procedure: "(scrutinizer.scm:2297) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k20144 
o|inlining procedure: k20199 
o|inlining procedure: "(scrutinizer.scm:2295) chicken.compiler.scrutinizer#third" 
o|inlining procedure: k20199 
o|substituted constant variable: a20213 
o|substituted constant variable: a20215 
o|inlining procedure: k22948 
o|inlining procedure: k22948 
o|inlining procedure: k22954 
o|inlining procedure: k22966 
o|inlining procedure: k22966 
o|inlining procedure: "(scrutinizer.scm:2235) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k22954 
o|substituted constant variable: a23000 
o|inlining procedure: k23027 
o|inlining procedure: k23027 
o|inlining procedure: k23084 
o|inlining procedure: k23084 
o|inlining procedure: k23118 
o|inlining procedure: k23118 
o|inlining procedure: k23145 
o|inlining procedure: k23145 
o|inlining procedure: k23183 
o|inlining procedure: k23183 
o|inlining procedure: k23210 
o|inlining procedure: k23210 
o|inlining procedure: k23262 
o|inlining procedure: k23262 
o|inlining procedure: k23284 
o|inlining procedure: k23284 
o|inlining procedure: k23290 
o|inlining procedure: k23302 
o|inlining procedure: "(scrutinizer.scm:2128) chicken.compiler.scrutinizer#fourth" 
o|inlining procedure: k23302 
o|inlining procedure: "(scrutinizer.scm:2127) chicken.compiler.scrutinizer#fourth" 
o|inlining procedure: "(scrutinizer.scm:2126) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k23290 
o|inlining procedure: k23341 
o|inlining procedure: k23341 
o|inlining procedure: k23347 
o|inlining procedure: k23359 
o|inlining procedure: k23371 
o|inlining procedure: k23371 
o|inlining procedure: "(scrutinizer.scm:2082) chicken.compiler.scrutinizer#first" 
o|inlining procedure: k23359 
o|inlining procedure: "(scrutinizer.scm:2080) chicken.compiler.scrutinizer#second" 
o|inlining procedure: k23347 
o|substituted constant variable: a23403 
o|simplifications: ((if . 10)) 
o|replaced variables: 4767 
o|removed binding forms: 903 
o|substituted constant variable: r686723404 
o|substituted constant variable: r697323408 
o|substituted constant variable: r701323410 
o|substituted constant variable: r716823418 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#first 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#second 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#third 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#fourth 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#filter 
o|substituted constant variable: r742123431 
o|substituted constant variable: r777623434 
o|substituted constant variable: r781223437 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#d 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#constant738 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#constant747 
o|substituted constant variable: r811023452 
o|substituted constant variable: r817223455 
o|substituted constant variable: r818423457 
o|substituted constant variable: r819923459 
o|substituted constant variable: r821123461 
o|substituted constant variable: r822323463 
o|substituted constant variable: r823523465 
o|substituted constant variable: r823523466 
o|substituted constant variable: r823823467 
o|substituted constant variable: r839523477 
o|substituted constant variable: r839523478 
o|substituted constant variable: r846723480 
o|substituted constant variable: r847623482 
o|substituted constant variable: r788923483 
o|substituted constant variable: r851823488 
o|substituted constant variable: r842523491 
o|substituted constant variable: r842023494 
o|substituted constant variable: r854223496 
o|substituted constant variable: r862323512 
o|substituted constant variable: r862323513 
o|substituted constant variable: r864423514 
o|substituted constant variable: r866223516 
o|removed side-effect free assignment to unused variable: add-loc802 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:406) chicken.base#void" 
o|substituted constant variable: r886523537 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:394) chicken.base#void" 
o|substituted constant variable: r911323569 
o|substituted constant variable: r869323571 
o|substituted constant variable: r870423574 
o|substituted constant variable: r870423574 
o|substituted constant variable: r920323577 
o|substituted constant variable: r1560523586 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:309) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:420) chicken.base#void" 
o|substituted constant variable: r946923616 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:798) chicken.base#void" 
o|substituted constant variable: r953323629 
o|substituted constant variable: r968823651 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:501) chicken.base#void" 
o|substituted constant variable: r977823658 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:488) chicken.base#void" 
o|substituted constant variable: r988023664 
o|substituted constant variable: r863523667 
o|substituted constant variable: r863523668 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:566) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:533) chicken.base#void" 
o|substituted constant variable: r1029023704 
o|substituted constant variable: r1029023704 
o|substituted constant variable: r1029023706 
o|substituted constant variable: r1029023706 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:658) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:645) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:651) chicken.base#void" 
o|substituted constant variable: r1046123734 
o|substituted constant variable: r1055523736 
o|inlining procedure: k10614 
o|substituted constant variable: r1059223751 
o|substituted constant variable: r1065823767 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:732) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:720) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:740) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:751) chicken.base#void" 
o|substituted constant variable: r1098923789 
o|converted assignments to bindings: (smash1676) 
o|inlining procedure: k11262 
o|inlining procedure: k11262 
o|converted assignments to bindings: (pp-type5611) 
o|substituted constant variable: r1143523865 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:438) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:817) chicken.base#void" 
o|propagated global variable: out18861889 ##sys#standard-output 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:178) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:836) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:840) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:845) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:849) chicken.base#void" 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#ble-id 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#ble-tag 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#ble-type 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#ble-type-set! 
o|substituted constant variable: a1205023900 
o|substituted constant variable: a1205023907 
o|substituted constant variable: r1196023913 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:874) chicken.base#void" 
o|substituted constant variable: r1202423919 
o|substituted constant variable: r1202423919 
o|substituted constant variable: r1201423921 
o|removed side-effect free assignment to unused variable: optargs?2036 
o|substituted constant variable: r1227923978 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:951) chicken.base#void" 
o|substituted constant variable: r1227923979 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:964) chicken.base#void" 
o|substituted constant variable: r1242124033 
o|substituted constant variable: r1261624056 
o|substituted constant variable: r1263924059 
o|substituted constant variable: r1265624062 
o|substituted constant variable: r1281024092 
o|substituted constant variable: r1292024123 
o|substituted constant variable: r1296724136 
o|substituted constant variable: r1296724136 
o|substituted constant variable: r1294724155 
o|substituted constant variable: r1301624174 
o|substituted constant variable: r1305524182 
o|substituted constant variable: r1304324193 
o|substituted constant variable: r1300324194 
o|substituted constant variable: r1311124202 
o|substituted constant variable: r1314724210 
o|substituted constant variable: r1318724218 
o|substituted constant variable: r1322324226 
o|substituted constant variable: r1331824228 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:930) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:1107) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:1105) chicken.base#void" 
o|substituted constant variable: r1348024234 
o|substituted constant variable: r1355324242 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:1163) chicken.base#void" 
o|substituted constant variable: r1465624260 
o|substituted constant variable: r1373624263 
o|substituted constant variable: r1376124266 
o|substituted constant variable: r1407024322 
o|substituted constant variable: r1411524336 
o|substituted constant variable: r1424924344 
o|substituted constant variable: r1424924344 
o|substituted constant variable: r1442524373 
o|substituted constant variable: r1446324381 
o|substituted constant variable: r1448724384 
o|substituted constant variable: r1455224390 
o|substituted constant variable: r1487324418 
o|substituted constant variable: r1490624427 
o|substituted constant variable: r1502624437 
o|substituted constant variable: r1501424438 
o|substituted constant variable: r1507024442 
o|substituted constant variable: r1505824443 
o|substituted constant variable: r1515424454 
o|substituted constant variable: r1521224463 
o|substituted constant variable: r1519424464 
o|substituted constant variable: r1517424465 
o|substituted constant variable: r1522524489 
o|substituted constant variable: r1528224503 
o|substituted constant variable: r1536724505 
o|inlining procedure: k15373 
o|inlining procedure: k15373 
o|substituted constant variable: r1538424507 
o|substituted constant variable: r1572324557 
o|substituted constant variable: r1569024558 
o|substituted constant variable: r1574324559 
o|substituted constant variable: r1577624572 
o|substituted constant variable: r1580224579 
o|substituted constant variable: r1576324580 
o|substituted constant variable: r1588124597 
o|substituted constant variable: r1586624603 
o|substituted constant variable: r1657324720 
o|substituted constant variable: r1657324720 
o|substituted constant variable: r1663024726 
o|substituted constant variable: r1675924762 
o|substituted constant variable: r1713124764 
o|substituted constant variable: r1727624771 
o|substituted constant variable: r1727624771 
o|substituted constant variable: r1727624773 
o|substituted constant variable: r1727624773 
o|substituted constant variable: r1760824802 
o|substituted constant variable: r1769324803 
o|substituted constant variable: r1772124805 
o|substituted constant variable: r1773324807 
o|substituted constant variable: r1775124810 
o|substituted constant variable: r1777424812 
o|substituted constant variable: r1781324816 
o|substituted constant variable: r1788124821 
o|substituted constant variable: r1789324823 
o|substituted constant variable: r1791724828 
o|substituted constant variable: r1799724849 
o|substituted constant variable: r1805324858 
o|substituted constant variable: r1795724871 
o|substituted constant variable: r1814824881 
o|substituted constant variable: r1822124884 
o|substituted constant variable: r1824224897 
o|substituted constant variable: r1831424906 
o|substituted constant variable: r1827824917 
o|substituted constant variable: r1828524921 
o|substituted constant variable: r1766524929 
o|substituted constant variable: r1841024933 
o|substituted constant variable: r1844024945 
o|substituted constant variable: r1848724953 
o|substituted constant variable: r1852324958 
o|substituted constant variable: r1860624961 
o|substituted constant variable: r1873024971 
o|substituted constant variable: r1869024972 
o|substituted constant variable: r1867824973 
o|substituted constant variable: r1866024974 
o|substituted constant variable: r1885524984 
o|substituted constant variable: r1884324990 
o|inlining procedure: k18921 
o|substituted constant variable: r1892224993 
o|substituted constant variable: r1892224993 
o|converted assignments to bindings: (fail4095) 
o|substituted constant variable: r1955125089 
o|substituted constant variable: r1953925095 
o|substituted constant variable: r1952725101 
o|substituted constant variable: r1951525102 
o|substituted constant variable: r1950325108 
o|substituted constant variable: r1961125117 
o|substituted constant variable: r1966225123 
o|substituted constant variable: r1968625127 
o|substituted constant variable: r1970125132 
o|substituted constant variable: r1973825151 
o|substituted constant variable: r1989225167 
o|substituted constant variable: r1990825170 
o|substituted constant variable: r1981525171 
o|substituted constant variable: r1988325173 
o|substituted constant variable: r1987125179 
o|substituted constant variable: r1985925190 
o|substituted constant variable: r1997925191 
o|substituted constant variable: r2000025196 
o|substituted constant variable: r2005025198 
o|inlining procedure: k20072 
o|converted assignments to bindings: (derive-result-type4638) 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:2331) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:2320) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:2349) chicken.base#void" 
o|substituted constant variable: r722625207 
o|substituted constant variable: r2051825215 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:2352) chicken.base#void" 
o|substituted constant variable: r2053925217 
o|substituted constant variable: r2053925218 
o|contracted procedure: "(scrutinizer.scm:2411) g49824991" 
o|substituted constant variable: r2094525260 
o|substituted constant variable: r2094525260 
o|substituted constant variable: r2095625262 
o|substituted constant variable: r2095625262 
o|substituted constant variable: r2095625264 
o|substituted constant variable: r2095625264 
o|substituted constant variable: r2096825267 
o|converted assignments to bindings: (R4923) 
o|substituted constant variable: r2123725285 
o|substituted constant variable: r1746325289 
o|substituted constant variable: r2133225296 
o|substituted constant variable: r2133225296 
o|substituted constant variable: r2127025300 
o|substituted constant variable: r2161325324 
o|substituted constant variable: r2160125325 
o|substituted constant variable: r2168425337 
o|converted assignments to bindings: (file-location5303) 
o|converted assignments to bindings: (p5457) 
o|converted assignments to bindings: (p5479) 
o|converted assignments to bindings: (p5527) 
o|substituted constant variable: type4691 
o|substituted constant variable: r2016925353 
o|substituted constant variable: r2015725359 
o|substituted constant variable: r2014525365 
o|substituted constant variable: r2020025372 
o|converted assignments to bindings: (append-special-case4626) 
o|substituted constant variable: r2296725377 
o|substituted constant variable: r2295525383 
o|substituted constant variable: r2311925388 
o|substituted constant variable: r2318425392 
o|substituted constant variable: r2326325397 
o|substituted constant variable: r2330325407 
o|substituted constant variable: r2329125418 
o|substituted constant variable: r2337225424 
o|substituted constant variable: r2336025430 
o|substituted constant variable: r2334825436 
o|simplifications: ((let . 10)) 
o|replaced variables: 987 
o|removed binding forms: 3936 
o|inlining procedure: k7952 
o|contracted procedure: k8134 
o|inlining procedure: "(scrutinizer.scm:2821) chicken.compiler.scrutinizer#report-notice" 
o|contracted procedure: k8734 
o|contracted procedure: k8795 
o|contracted procedure: k8892 
o|inlining procedure: "(scrutinizer.scm:2772) chicken.compiler.scrutinizer#report-notice" 
o|inlining procedure: "(scrutinizer.scm:2797) chicken.compiler.scrutinizer#report-notice" 
o|inlining procedure: k15634 
o|contracted procedure: k9364 
o|contracted procedure: k9486 
o|contracted procedure: k9496 
o|contracted procedure: k9702 
o|contracted procedure: k9829 
o|inlining procedure: k9906 
o|inlining procedure: "(scrutinizer.scm:2837) chicken.compiler.scrutinizer#report-notice" 
o|inlining procedure: k9906 
o|inlining procedure: "(scrutinizer.scm:520) chicken.compiler.scrutinizer#alist-cons" 
o|inlining procedure: "(scrutinizer.scm:519) chicken.compiler.scrutinizer#alist-cons" 
o|contracted procedure: k10137 
o|inlining procedure: "(scrutinizer.scm:539) chicken.compiler.scrutinizer#alist-cons" 
o|contracted procedure: k10334 
o|contracted procedure: k10366 
o|inlining procedure: "(scrutinizer.scm:646) chicken.compiler.scrutinizer#alist-cons" 
o|contracted procedure: k10390 
o|substituted constant variable: r1061525537 
o|folded constant expression: (scheme#not (quote #f)) 
o|inlining procedure: k10629 
o|contracted procedure: k10886 
o|contracted procedure: k10901 
o|contracted procedure: k10936 
o|contracted procedure: k10975 
o|inlining procedure: "(scrutinizer.scm:2938) pp-type5611" 
o|inlining procedure: "(scrutinizer.scm:2937) pp-type5611" 
o|contracted procedure: k11542 
o|contracted procedure: k11698 
o|contracted procedure: k11711 
o|contracted procedure: k11779 
o|contracted procedure: k11792 
o|contracted procedure: k11860 
o|inlining procedure: "(mini-srfi-1.scm:50) a12000" 
o|contracted procedure: k12180 
o|contracted procedure: k12287 
o|contracted procedure: k12371 
o|contracted procedure: k13390 
o|contracted procedure: k13396 
o|inlining procedure: k13571 
o|contracted procedure: k13648 
o|inlining procedure: k13729 
o|inlining procedure: k13729 
o|inlining procedure: k7734 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:1267) chicken.base#void" 
o|removed call to pure procedure with unused result: "(scrutinizer.scm:1267) chicken.base#void" 
o|inlining procedure: k15316 
o|substituted constant variable: r153672450525651 
o|substituted constant variable: r153672450525654 
o|inlining procedure: "(scrutinizer.scm:1779) chicken.compiler.scrutinizer#alist-cons" 
o|consed rest parameter at call site: "(scrutinizer.scm:1833) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1837) validate3707" 2 
o|merged explicitly consed rest parameter: rest37553757 
o|consed rest parameter at call site: "(scrutinizer.scm:1855) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1881) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1876) validate3707" 2 
o|inlining procedure: k18040 
o|inlining procedure: k18040 
o|consed rest parameter at call site: "(scrutinizer.scm:1893) validate3707" 2 
o|inlining procedure: k18295 
o|consed rest parameter at call site: "(scrutinizer.scm:1902) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1915) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1924) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1921) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1930) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1936) validate3707" 2 
o|consed rest parameter at call site: "(scrutinizer.scm:1941) validate3707" 2 
o|inlining procedure: k18717 
o|consed rest parameter at call site: "(scrutinizer.scm:1958) validate3707" 2 
o|substituted constant variable: r1892225739 
o|consed rest parameter at call site: "(scrutinizer.scm:1967) validate3707" 2 
o|inlining procedure: k19601 
o|inlining procedure: k19601 
o|inlining procedure: k19601 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|inlining procedure: k19849 
o|substituted constant variable: r200502519825771 
o|contracted procedure: k20225 
o|contracted procedure: k20256 
o|inlining procedure: "(scrutinizer.scm:2327) chicken.compiler.scrutinizer#alist-cons" 
o|contracted procedure: k20325 
o|contracted procedure: k20410 
o|inlining procedure: k20644 
o|inlining procedure: k21415 
o|inlining procedure: k22936 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k20132 
o|inlining procedure: k22945 
o|inlining procedure: k22945 
o|inlining procedure: k22945 
o|inlining procedure: k22945 
o|inlining procedure: k22945 
o|inlining procedure: k23281 
o|inlining procedure: k23281 
o|inlining procedure: k23281 
o|inlining procedure: k23281 
o|inlining procedure: k23281 
o|inlining procedure: k23281 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|inlining procedure: k23338 
o|replaced variables: 5 
o|removed binding forms: 1145 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#alist-cons 
o|substituted constant variable: r795325869 
o|substituted constant variable: reason532025881 
o|inlining procedure: k8792 
o|substituted constant variable: reason532025903 
o|substituted constant variable: reason532025909 
o|inlining procedure: "(scrutinizer.scm:302) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|substituted constant variable: r990725933 
o|substituted constant variable: reason532025935 
o|substituted constant variable: r990725940 
o|substituted constant variable: y32225961 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|substituted constant variable: r1063025977 
o|inlining procedure: k9491 
o|inlining procedure: "(scrutinizer.scm:680) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: "(scrutinizer.scm:677) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: k9491 
o|inlining procedure: k13064 
o|substituted constant variable: r1373026039 
o|substituted constant variable: r1373026040 
o|contracted procedure: k1440724371 
o|contracted procedure: k1440724379 
o|inlining procedure: k15259 
o|inlining procedure: k15460 
o|inlining procedure: k16359 
o|inlining procedure: k17271 
o|inlining procedure: k17271 
o|substituted constant variable: r1804126130 
o|substituted constant variable: r1804126131 
o|substituted constant variable: r1829626136 
o|substituted constant variable: r1829626136 
o|substituted constant variable: r1871826146 
o|inlining procedure: "(scrutinizer.scm:2095) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r1960226161 
o|substituted constant variable: r1960226161 
o|substituted constant variable: r1960226164 
o|substituted constant variable: r1960226164 
o|inlining procedure: "(scrutinizer.scm:2113) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r1960226167 
o|substituted constant variable: r1960226167 
o|substituted constant variable: r1985026180 
o|substituted constant variable: r1985026180 
o|substituted constant variable: r1985026186 
o|substituted constant variable: r1985026186 
o|substituted constant variable: r1985026189 
o|substituted constant variable: r1985026189 
o|substituted constant variable: r1985026192 
o|substituted constant variable: r1985026192 
o|substituted constant variable: r1985026195 
o|substituted constant variable: r1985026195 
o|substituted constant variable: r1985026198 
o|substituted constant variable: r1985026198 
o|substituted constant variable: r1985026201 
o|substituted constant variable: r1985026201 
o|inlining procedure: "(scrutinizer.scm:2183) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r1985026204 
o|substituted constant variable: r1985026204 
o|substituted constant variable: r1985026207 
o|substituted constant variable: r1985026207 
o|inlining procedure: "(scrutinizer.scm:2251) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r2141626259 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#report-notice 
o|substituted constant variable: r2013326263 
o|substituted constant variable: r2013326263 
o|substituted constant variable: r2013326266 
o|substituted constant variable: r2013326266 
o|substituted constant variable: r2013326269 
o|substituted constant variable: r2013326269 
o|substituted constant variable: r2013326272 
o|substituted constant variable: r2013326272 
o|substituted constant variable: r2013326275 
o|substituted constant variable: r2013326275 
o|substituted constant variable: r2013326278 
o|substituted constant variable: r2013326278 
o|inlining procedure: "(scrutinizer.scm:2295) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r2013326281 
o|substituted constant variable: r2013326281 
o|substituted constant variable: r2294626284 
o|substituted constant variable: r2294626284 
o|substituted constant variable: r2294626287 
o|substituted constant variable: r2294626287 
o|substituted constant variable: r2294626290 
o|substituted constant variable: r2294626290 
o|inlining procedure: "(scrutinizer.scm:2235) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r2294626293 
o|substituted constant variable: r2294626293 
o|substituted constant variable: r2294626296 
o|substituted constant variable: r2294626296 
o|inlining procedure: "(scrutinizer.scm:2229) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: "(scrutinizer.scm:2225) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: "(scrutinizer.scm:2221) chicken.compiler.scrutinizer#walked-result" 
o|inlining procedure: "(scrutinizer.scm:2215) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r2328226310 
o|substituted constant variable: r2328226310 
o|inlining procedure: "(scrutinizer.scm:2128) chicken.compiler.scrutinizer#walked-result" 
o|substituted constant variable: r2328226313 
o|substituted constant variable: r2328226313 
o|substituted constant variable: r2328226316 
o|substituted constant variable: r2328226316 
o|substituted constant variable: r2328226319 
o|substituted constant variable: r2328226319 
o|substituted constant variable: r2328226322 
o|substituted constant variable: r2328226322 
o|substituted constant variable: r2333926328 
o|substituted constant variable: r2333926328 
o|substituted constant variable: r2333926331 
o|substituted constant variable: r2333926331 
o|substituted constant variable: r2333926334 
o|substituted constant variable: r2333926334 
o|substituted constant variable: r2333926337 
o|substituted constant variable: r2333926337 
o|substituted constant variable: r2333926340 
o|substituted constant variable: r2333926340 
o|substituted constant variable: r2333926343 
o|substituted constant variable: r2333926343 
o|replaced variables: 47 
o|removed binding forms: 127 
o|removed conditional forms: 43 
o|substituted constant variable: r949226399 
o|substituted constant variable: r949226403 
o|substituted constant variable: r949226407 
o|substituted constant variable: r949226421 
o|substituted constant variable: r949226433 
o|substituted constant variable: r949226437 
o|substituted constant variable: r949226443 
o|substituted constant variable: r949226471 
o|substituted constant variable: r1306526495 
o|replaced variables: 35 
o|removed binding forms: 113 
o|removed binding forms: 44 
o|simplifications: ((if . 162) (let . 167) (##core#call . 1963)) 
o|  call simplifications:
o|    scheme#cadddr	2
o|    scheme#char-upcase
o|    ##sys#apply
o|    scheme#string=?
o|    scheme#string-ref	2
o|    chicken.fixnum#fx<	2
o|    scheme#>=	2
o|    scheme#<
o|    ##sys#intern-symbol
o|    chicken.base#identity
o|    ##sys#size	4
o|    scheme#list-tail	2
o|    scheme#vector-ref	5
o|    scheme#<=	2
o|    chicken.base#atom?	3
o|    chicken.fixnum#fx>	2
o|    scheme#cdar	4
o|    scheme#list-ref	4
o|    ##sys#call-with-values	12
o|    scheme#=	3
o|    scheme#cdddr	8
o|    chicken.fixnum#fx=	3
o|    scheme#cddr	18
o|    scheme#assoc	2
o|    scheme#set-cdr!	3
o|    scheme#set-car!	7
o|    scheme#values	14
o|    scheme#assq	17
o|    scheme#cadr	112
o|    scheme#list	69
o|    scheme#caar	2
o|    scheme#string?	7
o|    scheme#symbol?	26
o|    chicken.base#fixnum?	4
o|    chicken.base#bignum?
o|    chicken.base#flonum?
o|    chicken.base#ratnum?
o|    chicken.base#cplxnum?
o|    scheme#boolean?
o|    scheme#list?	14
o|    scheme#eof-object?
o|    scheme#vector?	4
o|    ##sys#immediate?
o|    ##sys#generic-structure?
o|    scheme#char?
o|    ##sys#list	65
o|    ##sys#cons	40
o|    scheme#caddr	66
o|    scheme#length	36
o|    scheme#eq?	288
o|    scheme#memq	16
o|    chicken.fixnum#fx>=	2
o|    chicken.fixnum#fx+	3
o|    scheme#equal?	6
o|    scheme#not	79
o|    scheme#null?	76
o|    scheme#apply	5
o|    scheme#cdr	73
o|    ##sys#setslot	51
o|    ##sys#check-list	61
o|    scheme#pair?	174
o|    ##sys#slot	192
o|    chicken.fixnum#fx<=	2
o|    scheme#car	178
o|    chicken.fixnum#fx-	2
o|    scheme#cons	174
o|contracted procedure: k6869 
o|contracted procedure: k6876 
o|contracted procedure: k6886 
o|contracted procedure: k6960 
o|contracted procedure: k6963 
o|contracted procedure: k6975 
o|contracted procedure: k6991 
o|contracted procedure: k6999 
o|contracted procedure: k7006 
o|contracted procedure: k7030 
o|contracted procedure: k7047 
o|contracted procedure: k7069 
o|contracted procedure: k7065 
o|contracted procedure: k7050 
o|contracted procedure: k7053 
o|contracted procedure: k7061 
o|contracted procedure: k7076 
o|contracted procedure: k7084 
o|contracted procedure: k7096 
o|contracted procedure: k7118 
o|contracted procedure: k7114 
o|contracted procedure: k7099 
o|contracted procedure: k7102 
o|contracted procedure: k7110 
o|contracted procedure: k7133 
o|contracted procedure: k7142 
o|contracted procedure: k7155 
o|contracted procedure: k7170 
o|contracted procedure: k7185 
o|contracted procedure: k7331 
o|contracted procedure: k7334 
o|contracted procedure: k7344 
o|contracted procedure: k7256 
o|contracted procedure: k7282 
o|contracted procedure: k7411 
o|contracted procedure: k7423 
o|contracted procedure: k7446 
o|contracted procedure: k7454 
o|contracted procedure: k7476 
o|contracted procedure: k7484 
o|contracted procedure: k7496 
o|contracted procedure: k7518 
o|contracted procedure: k7514 
o|contracted procedure: k7499 
o|contracted procedure: k7502 
o|contracted procedure: k7510 
o|contracted procedure: k7778 
o|contracted procedure: k7793 
o|contracted procedure: k7827 
o|contracted procedure: k7831 
o|contracted procedure: k7814 
o|contracted procedure: k7961 
o|contracted procedure: k7952 
o|contracted procedure: k8094 
o|contracted procedure: k8101 
o|contracted procedure: k8112 
o|contracted procedure: k8119 
o|contracted procedure: k8174 
o|contracted procedure: k8186 
o|contracted procedure: k8201 
o|contracted procedure: k8207 
o|contracted procedure: k8213 
o|contracted procedure: k8219 
o|contracted procedure: k8225 
o|contracted procedure: k8231 
o|contracted procedure: k8240 
o|contracted procedure: k8246 
o|contracted procedure: k8253 
o|contracted procedure: k8265 
o|contracted procedure: k8268 
o|contracted procedure: k8271 
o|contracted procedure: k8279 
o|contracted procedure: k8287 
o|contracted procedure: k8296 
o|contracted procedure: k8303 
o|contracted procedure: k8321 
o|contracted procedure: k8327 
o|contracted procedure: k8338 
o|contracted procedure: k8334 
o|contracted procedure: k8353 
o|contracted procedure: k8356 
o|contracted procedure: k8359 
o|contracted procedure: k8367 
o|contracted procedure: k8375 
o|contracted procedure: k8407 
o|contracted procedure: k8400 
o|contracted procedure: k8384 
o|contracted procedure: k8391 
o|contracted procedure: k8397 
o|contracted procedure: k8488 
o|contracted procedure: k8478 
o|contracted procedure: k7891 
o|contracted procedure: k7907 
o|contracted procedure: k8509 
o|contracted procedure: k8529 
o|contracted procedure: k8520 
o|contracted procedure: k8427 
o|contracted procedure: k8449 
o|contracted procedure: k8436 
o|contracted procedure: k8443 
o|contracted procedure: k8538 
o|contracted procedure: k8560 
o|contracted procedure: k8567 
o|contracted procedure: k8578 
o|contracted procedure: k8589 
o|contracted procedure: k8595 
o|contracted procedure: k8602 
o|contracted procedure: k22553 
o|contracted procedure: k8646 
o|contracted procedure: k8649 
o|contracted procedure: k8655 
o|contracted procedure: k8664 
o|contracted procedure: k8709 
o|contracted procedure: k8712 
o|contracted procedure: k8718 
o|contracted procedure: k8725 
o|contracted procedure: k8728 
o|contracted procedure: k21761 
o|contracted procedure: k21768 
o|contracted procedure: k21778 
o|contracted procedure: k21806 
o|contracted procedure: k21833 
o|contracted procedure: k8801 
o|contracted procedure: k8804 
o|contracted procedure: k8812 
o|contracted procedure: k8824 
o|contracted procedure: k8827 
o|contracted procedure: k8830 
o|contracted procedure: k8838 
o|contracted procedure: k8846 
o|contracted procedure: k8855 
o|contracted procedure: k8867 
o|contracted procedure: k8895 
o|contracted procedure: k8908 
o|contracted procedure: k8919 
o|contracted procedure: k8915 
o|contracted procedure: k8941 
o|contracted procedure: k8945 
o|contracted procedure: k8961 
o|contracted procedure: k8953 
o|contracted procedure: k8957 
o|contracted procedure: k8949 
o|contracted procedure: k8973 
o|contracted procedure: k8977 
o|contracted procedure: k8981 
o|contracted procedure: k22489 
o|contracted procedure: k9008 
o|contracted procedure: k9004 
o|contracted procedure: k9012 
o|contracted procedure: k9028 
o|contracted procedure: k9020 
o|contracted procedure: k9024 
o|contracted procedure: k9016 
o|contracted procedure: k9040 
o|contracted procedure: k9044 
o|contracted procedure: k9048 
o|contracted procedure: k22523 
o|contracted procedure: k9061 
o|contracted procedure: k9055 
o|contracted procedure: k9065 
o|contracted procedure: k9069 
o|contracted procedure: k9092 
o|contracted procedure: k9088 
o|contracted procedure: k9115 
o|contracted procedure: k9119 
o|contracted procedure: k9170 
o|contracted procedure: k9132 
o|contracted procedure: k9166 
o|contracted procedure: k9159 
o|contracted procedure: k9136 
o|contracted procedure: k9139 
o|contracted procedure: k9150 
o|contracted procedure: k9182 
o|contracted procedure: k8695 
o|contracted procedure: k8699 
o|contracted procedure: k9208 
o|contracted procedure: k15564 
o|contracted procedure: k15576 
o|contracted procedure: k15675 
o|contracted procedure: k15585 
o|contracted procedure: k15607 
o|contracted procedure: k15622 
o|contracted procedure: k15651 
o|contracted procedure: k15631 
o|contracted procedure: k15647 
o|contracted procedure: k15634 
o|contracted procedure: k15667 
o|contracted procedure: k15657 
o|contracted procedure: k15664 
o|contracted procedure: k9221 
o|contracted procedure: k9225 
o|contracted procedure: k9234 
o|contracted procedure: k9237 
o|contracted procedure: k9249 
o|contracted procedure: k9263 
o|contracted procedure: k9279 
o|contracted procedure: k9283 
o|contracted procedure: k21867 
o|contracted procedure: k21902 
o|contracted procedure: k21891 
o|inlining procedure: "(scrutinizer.scm:2615) chicken.compiler.scrutinizer#multiples" 
o|inlining procedure: "(scrutinizer.scm:2614) chicken.compiler.scrutinizer#multiples" 
o|contracted procedure: k9292 
o|contracted procedure: k9312 
o|contracted procedure: k9334 
o|contracted procedure: k9330 
o|contracted procedure: k9315 
o|contracted procedure: k9318 
o|contracted procedure: k9326 
o|contracted procedure: k9358 
o|contracted procedure: k9371 
o|contracted procedure: k9375 
o|contracted procedure: k9399 
o|contracted procedure: k9410 
o|contracted procedure: k9422 
o|contracted procedure: k9425 
o|contracted procedure: k9428 
o|contracted procedure: k9436 
o|contracted procedure: k9444 
o|contracted procedure: k9499 
o|contracted procedure: k9506 
o|inlining procedure: k9491 
o|contracted procedure: k9512 
o|inlining procedure: k9491 
o|contracted procedure: k9523 
o|contracted procedure: k9529 
o|contracted procedure: k9535 
o|contracted procedure: k9541 
o|contracted procedure: k9548 
o|contracted procedure: k9554 
o|inlining procedure: k9491 
o|contracted procedure: k9565 
o|contracted procedure: k9571 
o|inlining procedure: k9491 
o|contracted procedure: k9582 
o|contracted procedure: k9588 
o|contracted procedure: k9591 
o|contracted procedure: k9594 
o|contracted procedure: k9601 
o|contracted procedure: k9604 
o|contracted procedure: k9610 
o|contracted procedure: k9630 
o|contracted procedure: k9626 
o|contracted procedure: k9622 
o|contracted procedure: k9660 
o|contracted procedure: k9656 
o|contracted procedure: k9652 
o|contracted procedure: k9675 
o|contracted procedure: k22763 
o|contracted procedure: k22775 
o|contracted procedure: k22787 
o|inlining procedure: "(scrutinizer.scm:2963) chicken.compiler.scrutinizer#multiples" 
o|inlining procedure: "(scrutinizer.scm:2961) chicken.compiler.scrutinizer#multiples" 
o|contracted procedure: k9705 
o|contracted procedure: k9717 
o|contracted procedure: k9720 
o|contracted procedure: k9768 
o|contracted procedure: k9732 
o|contracted procedure: k9735 
o|contracted procedure: k9738 
o|contracted procedure: k9746 
o|contracted procedure: k9750 
o|contracted procedure: k9758 
o|contracted procedure: k9762 
o|contracted procedure: k9714 
o|contracted procedure: k9774 
o|contracted procedure: k9780 
o|contracted procedure: k9791 
o|contracted procedure: k9795 
o|contracted procedure: k9787 
o|contracted procedure: k9809 
o|contracted procedure: k9798 
o|contracted procedure: k9805 
o|contracted procedure: k9812 
o|contracted procedure: k9815 
o|contracted procedure: k9818 
o|contracted procedure: k9845 
o|contracted procedure: k9826 
o|contracted procedure: k9839 
o|contracted procedure: k9848 
o|contracted procedure: k9860 
o|contracted procedure: k9870 
o|contracted procedure: k9874 
o|contracted procedure: k9895 
o|contracted procedure: k9901 
o|contracted procedure: k8631 
o|contracted procedure: k22575 
o|contracted procedure: k9933 
o|contracted procedure: k9945 
o|contracted procedure: k9952 
o|contracted procedure: k9959 
o|contracted procedure: k9962 
o|contracted procedure: k736125947 
o|contracted procedure: k9979 
o|contracted procedure: k9987 
o|contracted procedure: k736125954 
o|contracted procedure: k9997 
o|contracted procedure: k10004 
o|contracted procedure: k10031 
o|contracted procedure: k10034 
o|contracted procedure: k10041 
o|contracted procedure: k10047 
o|contracted procedure: k10059 
o|contracted procedure: k10091 
o|contracted procedure: k10142 
o|contracted procedure: k10134 
o|contracted procedure: k10117 
o|contracted procedure: k10127 
o|contracted procedure: k10131 
o|contracted procedure: k10152 
o|contracted procedure: k10165 
o|contracted procedure: k10172 
o|contracted procedure: k10182 
o|contracted procedure: k10186 
o|contracted procedure: k10194 
o|contracted procedure: k10198 
o|contracted procedure: k736125965 
o|contracted procedure: k10217 
o|contracted procedure: k10228 
o|contracted procedure: k10231 
o|contracted procedure: k10279 
o|contracted procedure: k10243 
o|contracted procedure: k10269 
o|contracted procedure: k10273 
o|contracted procedure: k10265 
o|contracted procedure: k10246 
o|contracted procedure: k10249 
o|contracted procedure: k10257 
o|contracted procedure: k10261 
o|contracted procedure: k10298 
o|contracted procedure: k10301 
o|contracted procedure: k10304 
o|contracted procedure: k10316 
o|contracted procedure: k10328 
o|contracted procedure: k10448 
o|contracted procedure: k10439 
o|contracted procedure: k10342 
o|contracted procedure: k10348 
o|contracted procedure: k10360 
o|contracted procedure: k10374 
o|contracted procedure: k736125974 
o|contracted procedure: k10370 
o|contracted procedure: k10426 
o|contracted procedure: k10422 
o|contracted procedure: k10380 
o|contracted procedure: k10384 
o|contracted procedure: k10393 
o|contracted procedure: k10409 
o|contracted procedure: k10402 
o|contracted procedure: k10413 
o|contracted procedure: k10429 
o|contracted procedure: k10513 
o|contracted procedure: k10469 
o|contracted procedure: k10475 
o|contracted procedure: k10545 
o|contracted procedure: k10520 
o|contracted procedure: k10523 
o|contracted procedure: k10533 
o|contracted procedure: k10551 
o|contracted procedure: k10557 
o|contracted procedure: k10574 
o|contracted procedure: k10563 
o|contracted procedure: k10587 
o|contracted procedure: k22809 
o|contracted procedure: k22843 
o|contracted procedure: k10594 
o|contracted procedure: k10611 
o|contracted procedure: k10620 
o|contracted procedure: k10614 
o|contracted procedure: k10650 
o|contracted procedure: k10654 
o|contracted procedure: k10660 
o|contracted procedure: k10666 
o|contracted procedure: k10672 
o|contracted procedure: k10675 
o|contracted procedure: k10684 
o|contracted procedure: k10688 
o|contracted procedure: k10708 
o|contracted procedure: k11196 
o|contracted procedure: k10714 
o|contracted procedure: k10756 
o|contracted procedure: k10759 
o|contracted procedure: k10768 
o|contracted procedure: k10794 
o|contracted procedure: k10797 
o|contracted procedure: k10805 
o|contracted procedure: k10817 
o|contracted procedure: k10820 
o|contracted procedure: k10823 
o|contracted procedure: k10831 
o|contracted procedure: k10839 
o|contracted procedure: k10848 
o|contracted procedure: k10859 
o|contracted procedure: k10862 
o|contracted procedure: k11005 
o|contracted procedure: k10874 
o|contracted procedure: k10910 
o|contracted procedure: k10914 
o|contracted procedure: k10921 
o|contracted procedure: k10954 
o|contracted procedure: k10958 
o|contracted procedure: k10994 
o|contracted procedure: k11019 
o|contracted procedure: k11022 
o|contracted procedure: k11034 
o|contracted procedure: k11037 
o|contracted procedure: k11045 
o|contracted procedure: k11057 
o|contracted procedure: k11060 
o|contracted procedure: k11063 
o|contracted procedure: k11071 
o|contracted procedure: k11079 
o|contracted procedure: k11116 
o|contracted procedure: k11091 
o|contracted procedure: k11101 
o|contracted procedure: k11105 
o|contracted procedure: k11109 
o|contracted procedure: k11113 
o|contracted procedure: k11155 
o|contracted procedure: k11151 
o|contracted procedure: k11167 
o|contracted procedure: k11189 
o|contracted procedure: k11185 
o|contracted procedure: k11170 
o|contracted procedure: k11173 
o|contracted procedure: k11181 
o|contracted procedure: k11241 
o|contracted procedure: k11205 
o|contracted procedure: k11208 
o|contracted procedure: k11211 
o|contracted procedure: k11219 
o|contracted procedure: k11223 
o|contracted procedure: k11231 
o|contracted procedure: k11235 
o|contracted procedure: k11250 
o|contracted procedure: k11253 
o|contracted procedure: k11259 
o|inlining procedure: k9491 
o|inlining procedure: k9491 
o|contracted procedure: k11271 
o|contracted procedure: k11278 
o|contracted procedure: k22593 
o|contracted procedure: k11297 
o|contracted procedure: k11301 
o|contracted procedure: k22641 
o|inlining procedure: k9491 
o|contracted procedure: k11305 
o|contracted procedure: k11316 
o|contracted procedure: k11330 
o|contracted procedure: k11319 
o|contracted procedure: k11326 
o|contracted procedure: k22615 
o|contracted procedure: k22627 
o|contracted procedure: k11335 
o|contracted procedure: k11341 
o|contracted procedure: k11354 
o|contracted procedure: k11358 
o|contracted procedure: k11367 
o|contracted procedure: k11374 
o|contracted procedure: k11378 
o|contracted procedure: k22701 
o|contracted procedure: k22704 
o|contracted procedure: k22716 
o|contracted procedure: k22719 
o|contracted procedure: k22722 
o|contracted procedure: k22730 
o|contracted procedure: k22738 
o|contracted procedure: k11397 
o|contracted procedure: k11409 
o|contracted procedure: k11415 
o|contracted procedure: k11423 
o|contracted procedure: k11427 
o|contracted procedure: k11431 
o|contracted procedure: k11437 
o|contracted procedure: k11440 
o|contracted procedure: k11451 
o|contracted procedure: k11463 
o|contracted procedure: k11473 
o|contracted procedure: k11477 
o|contracted procedure: k11563 
o|contracted procedure: k11601 
o|contracted procedure: k11613 
o|contracted procedure: k11623 
o|contracted procedure: k11627 
o|contracted procedure: k11598 
o|propagated global variable: g18821884 chicken.compiler.scrutinizer#specialization-statistics 
o|contracted procedure: k11631 
o|contracted procedure: k11635 
o|contracted procedure: k11658 
o|contracted procedure: k11670 
o|contracted procedure: k11688 
o|contracted procedure: k11695 
o|contracted procedure: k11708 
o|contracted procedure: k11723 
o|contracted procedure: k11730 
o|contracted procedure: k11719 
o|contracted procedure: k11742 
o|contracted procedure: k11745 
o|contracted procedure: k11748 
o|contracted procedure: k11756 
o|contracted procedure: k11764 
o|contracted procedure: k11773 
o|contracted procedure: k11776 
o|contracted procedure: k11789 
o|contracted procedure: k11804 
o|contracted procedure: k11811 
o|contracted procedure: k11815 
o|contracted procedure: k13411 
o|contracted procedure: k11848 
o|contracted procedure: k13405 
o|contracted procedure: k11851 
o|contracted procedure: k13399 
o|contracted procedure: k11854 
o|contracted procedure: k11872 
o|contracted procedure: k11878 
o|contracted procedure: k11888 
o|contracted procedure: k11894 
o|contracted procedure: k11904 
o|contracted procedure: k11985 
o|contracted procedure: k11910 
o|contracted procedure: k11981 
o|contracted procedure: k11921 
o|contracted procedure: k11932 
o|contracted procedure: k11947 
o|contracted procedure: k6825 
o|contracted procedure: k6860 
o|contracted procedure: k1200726021 
o|contracted procedure: k6838 
o|contracted procedure: k6847 
o|contracted procedure: k12026 
o|contracted procedure: k12037 
o|contracted procedure: k12054 
o|contracted procedure: k12063 
o|contracted procedure: k12072 
o|contracted procedure: k12081 
o|contracted procedure: k12106 
o|contracted procedure: k12110 
o|contracted procedure: k12115 
o|contracted procedure: k12150 
o|contracted procedure: k12170 
o|contracted procedure: k12174 
o|contracted procedure: k12183 
o|contracted procedure: k13384 
o|contracted procedure: k12189 
o|contracted procedure: k12200 
o|contracted procedure: k12223 
o|contracted procedure: k12209 
o|contracted procedure: k12219 
o|contracted procedure: k12227 
o|contracted procedure: k12312 
o|contracted procedure: k12230 
o|contracted procedure: k12272 
o|contracted procedure: k12238 
o|contracted procedure: k12264 
o|contracted procedure: k12250 
o|contracted procedure: k12260 
o|contracted procedure: k12268 
o|contracted procedure: k12309 
o|contracted procedure: k12281 
o|contracted procedure: k12291 
o|contracted procedure: k12298 
o|contracted procedure: k12294 
o|contracted procedure: k12305 
o|contracted procedure: k13378 
o|contracted procedure: k12321 
o|contracted procedure: k12332 
o|contracted procedure: k12355 
o|contracted procedure: k12341 
o|contracted procedure: k12351 
o|contracted procedure: k12359 
o|contracted procedure: k12393 
o|contracted procedure: k12365 
o|contracted procedure: k12375 
o|contracted procedure: k12382 
o|contracted procedure: k12378 
o|contracted procedure: k12389 
o|contracted procedure: k12399 
o|contracted procedure: k12408 
o|contracted procedure: k12417 
o|contracted procedure: k12423 
o|contracted procedure: k12426 
o|contracted procedure: k12432 
o|contracted procedure: k13370 
o|contracted procedure: k12469 
o|contracted procedure: k12472 
o|contracted procedure: k12496 
o|contracted procedure: k13362 
o|contracted procedure: k12507 
o|contracted procedure: k12513 
o|contracted procedure: k12537 
o|contracted procedure: k13354 
o|contracted procedure: k12548 
o|contracted procedure: k12555 
o|contracted procedure: k13346 
o|contracted procedure: k12567 
o|contracted procedure: k12574 
o|contracted procedure: k13338 
o|contracted procedure: k12586 
o|contracted procedure: k12593 
o|contracted procedure: k13330 
o|contracted procedure: k12599 
o|contracted procedure: k12606 
o|contracted procedure: k12612 
o|contracted procedure: k12618 
o|contracted procedure: k12629 
o|contracted procedure: k12635 
o|contracted procedure: k12641 
o|contracted procedure: k12652 
o|contracted procedure: k12658 
o|contracted procedure: k12664 
o|contracted procedure: k12675 
o|contracted procedure: k12681 
o|contracted procedure: k12695 
o|contracted procedure: k12701 
o|contracted procedure: k12728 
o|contracted procedure: k12737 
o|contracted procedure: k12746 
o|contracted procedure: k12752 
o|contracted procedure: k12755 
o|contracted procedure: k12762 
o|contracted procedure: k12766 
o|contracted procedure: k12772 
o|contracted procedure: k12775 
o|contracted procedure: k12796 
o|contracted procedure: k12800 
o|contracted procedure: k12781 
o|contracted procedure: k12788 
o|contracted procedure: k12792 
o|contracted procedure: k12806 
o|contracted procedure: k12815 
o|contracted procedure: k12825 
o|contracted procedure: k12829 
o|contracted procedure: k12833 
o|contracted procedure: k12837 
o|contracted procedure: k13306 
o|contracted procedure: k12865 
o|contracted procedure: k12871 
o|contracted procedure: k12878 
o|contracted procedure: k13298 
o|contracted procedure: k12884 
o|contracted procedure: k12891 
o|contracted procedure: k13290 
o|contracted procedure: k12897 
o|contracted procedure: k12903 
o|contracted procedure: k12910 
o|contracted procedure: k12916 
o|contracted procedure: k12929 
o|contracted procedure: k12933 
o|contracted procedure: k12937 
o|contracted procedure: k12943 
o|contracted procedure: k12949 
o|contracted procedure: k12962 
o|contracted procedure: k12979 
o|contracted procedure: k12969 
o|contracted procedure: k12966 
o|contracted procedure: k12983 
o|contracted procedure: k12987 
o|contracted procedure: k13282 
o|contracted procedure: k12999 
o|contracted procedure: k13005 
o|contracted procedure: k13012 
o|contracted procedure: k13018 
o|contracted procedure: k13031 
o|contracted procedure: k13035 
o|contracted procedure: k13039 
o|contracted procedure: k13045 
o|contracted procedure: k13051 
o|contracted procedure: k13081 
o|contracted procedure: k13071 
o|contracted procedure: k1306826498 
o|contracted procedure: k13064 
o|contracted procedure: k13068 
o|contracted procedure: k13085 
o|contracted procedure: k13089 
o|contracted procedure: k13274 
o|contracted procedure: k13101 
o|contracted procedure: k13107 
o|contracted procedure: k13113 
o|contracted procedure: k13119 
o|contracted procedure: k13126 
o|contracted procedure: k13130 
o|contracted procedure: k13266 
o|contracted procedure: k13143 
o|contracted procedure: k13149 
o|contracted procedure: k13155 
o|contracted procedure: k13164 
o|contracted procedure: k13258 
o|contracted procedure: k13177 
o|contracted procedure: k13183 
o|contracted procedure: k13189 
o|contracted procedure: k13195 
o|contracted procedure: k13202 
o|contracted procedure: k13206 
o|contracted procedure: k13250 
o|contracted procedure: k13219 
o|contracted procedure: k13225 
o|contracted procedure: k13231 
o|contracted procedure: k13240 
o|contracted procedure: k13314 
o|contracted procedure: k13320 
o|contracted procedure: k13417 
o|contracted procedure: k13424 
o|contracted procedure: k13442 
o|contracted procedure: k13451 
o|contracted procedure: k13454 
o|contracted procedure: k13499 
o|contracted procedure: k13463 
o|contracted procedure: k13476 
o|contracted procedure: k13495 
o|contracted procedure: k13517 
o|contracted procedure: k13524 
o|contracted procedure: k13527 
o|contracted procedure: k13542 
o|contracted procedure: k13555 
o|contracted procedure: k13559 
o|contracted procedure: k13580 
o|contracted procedure: k13592 
o|contracted procedure: k13614 
o|contracted procedure: k13574 
o|contracted procedure: k13564 
o|contracted procedure: k13610 
o|contracted procedure: k13595 
o|contracted procedure: k13598 
o|contracted procedure: k13606 
o|contracted procedure: k13623 
o|contracted procedure: k13632 
o|contracted procedure: k13923 
o|contracted procedure: k13638 
o|contracted procedure: k13918 
o|contracted procedure: k13645 
o|contracted procedure: k13663 
o|contracted procedure: k13667 
o|contracted procedure: k13679 
o|contracted procedure: k13686 
o|contracted procedure: k13745 
o|contracted procedure: k13718 
o|contracted procedure: k14703 
o|contracted procedure: k14658 
o|contracted procedure: k14664 
o|contracted procedure: k14667 
o|contracted procedure: k14693 
o|contracted procedure: k14697 
o|contracted procedure: k14689 
o|contracted procedure: k13729 
o|contracted procedure: k13781 
o|contracted procedure: k13788 
o|contracted procedure: k13799 
o|contracted procedure: k13870 
o|contracted procedure: k13808 
o|contracted procedure: k13842 
o|contracted procedure: k13854 
o|propagated global variable: g25522553 chicken.compiler.scrutinizer#type<=? 
o|contracted procedure: k13864 
o|propagated global variable: g25442545 chicken.compiler.scrutinizer#type<=? 
o|contracted procedure: k13909 
o|contracted procedure: k13882 
o|contracted procedure: k13891 
o|contracted procedure: k13900 
o|contracted procedure: k13927 
o|contracted procedure: k13931 
o|contracted procedure: k13943 
o|contracted procedure: k13946 
o|contracted procedure: k13949 
o|contracted procedure: k13957 
o|contracted procedure: k13965 
o|contracted procedure: k13974 
o|contracted procedure: k13977 
o|contracted procedure: k13986 
o|contracted procedure: k14003 
o|contracted procedure: k14007 
o|contracted procedure: k7681 
o|contracted procedure: k7688 
o|contracted procedure: k7745 
o|contracted procedure: k7694 
o|contracted procedure: k7706 
o|contracted procedure: k7713 
o|contracted procedure: k7738 
o|contracted procedure: k7734 
o|contracted procedure: k14018 
o|contracted procedure: k14024 
o|contracted procedure: k14046 
o|contracted procedure: k14036 
o|contracted procedure: k14043 
o|contracted procedure: k14053 
o|contracted procedure: k14057 
o|contracted procedure: k14063 
o|contracted procedure: k14072 
o|contracted procedure: k14081 
o|contracted procedure: k14087 
o|contracted procedure: k14096 
o|contracted procedure: k14105 
o|contracted procedure: k14111 
o|contracted procedure: k14117 
o|contracted procedure: k14124 
o|contracted procedure: k14128 
o|contracted procedure: k14140 
o|contracted procedure: k14143 
o|contracted procedure: k14146 
o|contracted procedure: k14154 
o|contracted procedure: k14162 
o|contracted procedure: k14173 
o|contracted procedure: k14180 
o|contracted procedure: k14184 
o|contracted procedure: k14196 
o|contracted procedure: k14199 
o|contracted procedure: k14202 
o|contracted procedure: k14210 
o|contracted procedure: k14218 
o|contracted procedure: k14227 
o|contracted procedure: k14230 
o|contracted procedure: k14233 
o|contracted procedure: k14240 
o|contracted procedure: k14298 
o|contracted procedure: k14301 
o|contracted procedure: k14304 
o|contracted procedure: k14244 
o|contracted procedure: k14251 
o|contracted procedure: k14254 
o|contracted procedure: k14257 
o|contracted procedure: k14269 
o|contracted procedure: k14272 
o|contracted procedure: k14275 
o|contracted procedure: k14283 
o|contracted procedure: k14291 
o|contracted procedure: k14316 
o|contracted procedure: k14319 
o|contracted procedure: k14322 
o|contracted procedure: k14330 
o|contracted procedure: k14338 
o|contracted procedure: k14383 
o|contracted procedure: k7595 
o|contracted procedure: k7621 
o|contracted procedure: k7601 
o|contracted procedure: k7615 
o|contracted procedure: k14410 
o|contracted procedure: k14414 
o|contracted procedure: k14456 
o|contracted procedure: k14427 
o|contracted procedure: k14431 
o|contracted procedure: k14447 
o|contracted procedure: k14465 
o|contracted procedure: k14477 
o|contracted procedure: k14483 
o|contracted procedure: k14490 
o|contracted procedure: k14497 
o|contracted procedure: k14637 
o|contracted procedure: k14503 
o|contracted procedure: k14537 
o|contracted procedure: k14509 
o|contracted procedure: k14520 
o|contracted procedure: k14532 
o|contracted procedure: k14548 
o|contracted procedure: k14592 
o|contracted procedure: k14554 
o|contracted procedure: k14561 
o|contracted procedure: k14573 
o|contracted procedure: k14577 
o|contracted procedure: k14585 
o|contracted procedure: k14589 
o|contracted procedure: k14581 
o|contracted procedure: k14604 
o|contracted procedure: k14627 
o|contracted procedure: k14724 
o|contracted procedure: k14731 
o|contracted procedure: k14783 
o|contracted procedure: k14790 
o|contracted procedure: k15102 
o|contracted procedure: k15108 
o|contracted procedure: k15090 
o|contracted procedure: k14839 
o|contracted procedure: k14852 
o|contracted procedure: k14856 
o|contracted procedure: k14860 
o|contracted procedure: k15084 
o|contracted procedure: k14866 
o|contracted procedure: k14879 
o|contracted procedure: k14883 
o|contracted procedure: k14887 
o|contracted procedure: k15046 
o|contracted procedure: k14899 
o|contracted procedure: k14908 
o|contracted procedure: k14921 
o|contracted procedure: k14930 
o|contracted procedure: k14933 
o|contracted procedure: k14936 
o|contracted procedure: k14939 
o|contracted procedure: k14942 
o|contracted procedure: k15001 
o|contracted procedure: k14965 
o|contracted procedure: k14968 
o|contracted procedure: k14971 
o|contracted procedure: k14979 
o|contracted procedure: k14983 
o|contracted procedure: k14991 
o|contracted procedure: k14995 
o|contracted procedure: k14755 
o|contracted procedure: k14762 
o|contracted procedure: k15010 
o|contracted procedure: k15022 
o|contracted procedure: k15028 
o|contracted procedure: k15035 
o|contracted procedure: k15039 
o|contracted procedure: k15054 
o|contracted procedure: k15060 
o|contracted procedure: k15072 
o|contracted procedure: k15115 
o|contracted procedure: k15121 
o|contracted procedure: k15130 
o|contracted procedure: k15137 
o|contracted procedure: k15144 
o|contracted procedure: k15150 
o|contracted procedure: k15156 
o|contracted procedure: k15176 
o|contracted procedure: k15183 
o|contracted procedure: k15190 
o|contracted procedure: k15196 
o|contracted procedure: k15199 
o|contracted procedure: k15205 
o|contracted procedure: k15214 
o|contracted procedure: k15227 
o|contracted procedure: k15234 
o|contracted procedure: k15241 
o|contracted procedure: k15247 
o|contracted procedure: k15250 
o|contracted procedure: k15256 
o|contracted procedure: k15259 
o|contracted procedure: k15284 
o|contracted procedure: k15291 
o|contracted procedure: k15298 
o|contracted procedure: k15304 
o|contracted procedure: k15307 
o|contracted procedure: k15313 
o|contracted procedure: k15316 
o|contracted procedure: k15549 
o|contracted procedure: k15338 
o|contracted procedure: k15541 
o|contracted procedure: k15350 
o|contracted procedure: k15369 
o|contracted procedure: k15454 
o|contracted procedure: k15380 
o|contracted procedure: k15394 
o|contracted procedure: k15413 
o|contracted procedure: k15420 
o|contracted procedure: k15428 
o|contracted procedure: k15477 
o|contracted procedure: k15457 
o|contracted procedure: k15473 
o|contracted procedure: k15460 
o|contracted procedure: k15533 
o|contracted procedure: k15483 
o|contracted procedure: k15490 
o|contracted procedure: k15493 
o|contracted procedure: k15501 
o|contracted procedure: k15507 
o|contracted procedure: k15517 
o|contracted procedure: k15692 
o|contracted procedure: k15699 
o|contracted procedure: k15719 
o|contracted procedure: k15706 
o|contracted procedure: k15709 
o|contracted procedure: k15725 
o|contracted procedure: k15732 
o|contracted procedure: k15745 
o|contracted procedure: k15756 
o|contracted procedure: k15751 
o|contracted procedure: k15765 
o|contracted procedure: k15772 
o|contracted procedure: k15778 
o|contracted procedure: k15798 
o|contracted procedure: k15788 
o|contracted procedure: k15785 
o|contracted procedure: k15804 
o|contracted procedure: k15811 
o|contracted procedure: k15821 
o|contracted procedure: k15830 
o|contracted procedure: k15837 
o|contracted procedure: k15848 
o|contracted procedure: k15855 
o|contracted procedure: k15868 
o|contracted procedure: k15871 
o|contracted procedure: k15877 
o|contracted procedure: k15883 
o|contracted procedure: k15890 
o|contracted procedure: k15921 
o|contracted procedure: k15928 
o|contracted procedure: k15935 
o|contracted procedure: k15941 
o|contracted procedure: k16108 
o|contracted procedure: k15947 
o|contracted procedure: k15953 
o|contracted procedure: k15956 
o|contracted procedure: k16001 
o|contracted procedure: k15965 
o|contracted procedure: k15971 
o|contracted procedure: k15983 
o|contracted procedure: k15993 
o|contracted procedure: k15997 
o|contracted procedure: k16010 
o|contracted procedure: k16020 
o|contracted procedure: k16024 
o|contracted procedure: k16027 
o|contracted procedure: k16030 
o|contracted procedure: k16074 
o|contracted procedure: k16039 
o|contracted procedure: k16044 
o|contracted procedure: k16056 
o|contracted procedure: k16066 
o|contracted procedure: k16070 
o|contracted procedure: k16083 
o|contracted procedure: k16093 
o|contracted procedure: k16097 
o|contracted procedure: k16104 
o|contracted procedure: k16114 
o|contracted procedure: k16125 
o|contracted procedure: k16129 
o|contracted procedure: k16154 
o|contracted procedure: k16157 
o|contracted procedure: k16169 
o|contracted procedure: k16172 
o|contracted procedure: k16175 
o|contracted procedure: k16183 
o|contracted procedure: k16191 
o|contracted procedure: k16137 
o|contracted procedure: k16147 
o|contracted procedure: k16151 
o|contracted procedure: k16200 
o|contracted procedure: k16204 
o|contracted procedure: k16216 
o|contracted procedure: k16226 
o|contracted procedure: k16230 
o|contracted procedure: k16253 
o|contracted procedure: k16272 
o|contracted procedure: k16256 
o|contracted procedure: k16268 
o|contracted procedure: k16259 
o|contracted procedure: k16297 
o|contracted procedure: k16305 
o|contracted procedure: k16311 
o|contracted procedure: k16314 
o|contracted procedure: k16320 
o|contracted procedure: k16327 
o|contracted procedure: k16331 
o|contracted procedure: k16338 
o|contracted procedure: k16746 
o|contracted procedure: k16350 
o|contracted procedure: k16356 
o|contracted procedure: k16359 
o|contracted procedure: k16368 
o|contracted procedure: k16374 
o|contracted procedure: k16381 
o|contracted procedure: k16390 
o|contracted procedure: k16402 
o|contracted procedure: k16405 
o|contracted procedure: k16408 
o|contracted procedure: k16416 
o|contracted procedure: k16424 
o|contracted procedure: k16433 
o|contracted procedure: k16444 
o|contracted procedure: k16450 
o|contracted procedure: k16453 
o|contracted procedure: k16462 
o|contracted procedure: k16470 
o|contracted procedure: k16476 
o|contracted procedure: k16488 
o|contracted procedure: k16497 
o|contracted procedure: k16509 
o|contracted procedure: k16512 
o|contracted procedure: k16515 
o|contracted procedure: k16523 
o|contracted procedure: k16531 
o|contracted procedure: k16540 
o|contracted procedure: k16560 
o|inlining procedure: k16564 
o|contracted procedure: k16575 
o|inlining procedure: k16564 
o|contracted procedure: k16578 
o|contracted procedure: k16586 
o|contracted procedure: k16598 
o|contracted procedure: k16601 
o|contracted procedure: k16604 
o|contracted procedure: k16612 
o|contracted procedure: k16620 
o|contracted procedure: k16632 
o|contracted procedure: k16695 
o|contracted procedure: k16638 
o|contracted procedure: k16644 
o|contracted procedure: k16663 
o|contracted procedure: k16703 
o|contracted procedure: k16709 
o|contracted procedure: k16715 
o|contracted procedure: k17071 
o|contracted procedure: k17083 
o|contracted procedure: k17093 
o|contracted procedure: k17097 
o|contracted procedure: k16777 
o|contracted procedure: k17065 
o|contracted procedure: k17059 
o|contracted procedure: k16783 
o|contracted procedure: k16892 
o|contracted procedure: k16910 
o|contracted procedure: k16919 
o|contracted procedure: k16922 
o|contracted procedure: k16928 
o|contracted procedure: k16942 
o|contracted procedure: k16956 
o|contracted procedure: k16970 
o|contracted procedure: k16984 
o|contracted procedure: k16994 
o|contracted procedure: k16998 
o|contracted procedure: k17025 
o|contracted procedure: k17032 
o|contracted procedure: k17040 
o|contracted procedure: k17045 
o|contracted procedure: k17052 
o|contracted procedure: k17127 
o|contracted procedure: k17139 
o|contracted procedure: k17178 
o|contracted procedure: k7200 
o|contracted procedure: k7211 
o|contracted procedure: k17219 
o|contracted procedure: k17226 
o|contracted procedure: k17239 
o|contracted procedure: k17247 
o|contracted procedure: k17255 
o|contracted procedure: k17263 
o|contracted procedure: k17284 
o|contracted procedure: k17291 
o|contracted procedure: k17299 
o|contracted procedure: k17358 
o|contracted procedure: k17340 
o|contracted procedure: k17349 
o|contracted procedure: k17385 
o|contracted procedure: k17391 
o|contracted procedure: k17403 
o|contracted procedure: k17406 
o|contracted procedure: k17409 
o|contracted procedure: k17417 
o|contracted procedure: k17425 
o|contracted procedure: k17511 
o|contracted procedure: k17536 
o|contracted procedure: k17539 
o|contracted procedure: k736126125 
o|contracted procedure: k17552 
o|contracted procedure: k17566 
o|contracted procedure: k17601 
o|contracted procedure: k17572 
o|contracted procedure: k17597 
o|contracted procedure: k17578 
o|contracted procedure: k17604 
o|contracted procedure: k17629 
o|contracted procedure: k17610 
o|contracted procedure: k17617 
o|contracted procedure: k17624 
o|contracted procedure: k17633 
o|contracted procedure: k17641 
o|contracted procedure: k17714 
o|contracted procedure: k17695 
o|contracted procedure: k17710 
o|contracted procedure: k17701 
o|contracted procedure: k17723 
o|contracted procedure: k17729 
o|contracted procedure: k17844 
o|contracted procedure: k17735 
o|contracted procedure: k17840 
o|contracted procedure: k17741 
o|contracted procedure: k17744 
o|contracted procedure: k17761 
o|contracted procedure: k17764 
o|contracted procedure: k17770 
o|contracted procedure: k17793 
o|contracted procedure: k17776 
o|contracted procedure: k17789 
o|contracted procedure: k17803 
o|contracted procedure: k18897 
o|contracted procedure: k17850 
o|contracted procedure: k17856 
o|contracted procedure: k17862 
o|contracted procedure: k17868 
o|contracted procedure: k17877 
o|contracted procedure: k17883 
o|contracted procedure: k17889 
o|contracted procedure: k17895 
o|contracted procedure: k17901 
o|contracted procedure: k18888 
o|contracted procedure: k17913 
o|contracted procedure: k17919 
o|contracted procedure: k18884 
o|contracted procedure: k17925 
o|contracted procedure: k17947 
o|contracted procedure: k17931 
o|contracted procedure: k17942 
o|contracted procedure: k18880 
o|contracted procedure: k17953 
o|contracted procedure: k18117 
o|contracted procedure: k17959 
o|contracted procedure: k18112 
o|contracted procedure: k17965 
o|contracted procedure: k17986 
o|contracted procedure: k17999 
o|contracted procedure: k18018 
o|contracted procedure: k18022 
o|contracted procedure: k18026 
o|contracted procedure: k18034 
o|contracted procedure: k18049 
o|contracted procedure: k18055 
o|contracted procedure: k18062 
o|contracted procedure: k18040 
o|contracted procedure: k18068 
o|contracted procedure: k18071 
o|contracted procedure: k18083 
o|contracted procedure: k18086 
o|contracted procedure: k18089 
o|contracted procedure: k18097 
o|contracted procedure: k18105 
o|contracted procedure: k18131 
o|contracted procedure: k18135 
o|contracted procedure: k18836 
o|contracted procedure: k18144 
o|contracted procedure: k18150 
o|contracted procedure: k18153 
o|contracted procedure: k18156 
o|contracted procedure: k18159 
o|contracted procedure: k18180 
o|contracted procedure: k18183 
o|contracted procedure: k18186 
o|contracted procedure: k18194 
o|contracted procedure: k18202 
o|contracted procedure: k18832 
o|contracted procedure: k18211 
o|contracted procedure: k18232 
o|contracted procedure: k18217 
o|contracted procedure: k18227 
o|contracted procedure: k18223 
o|contracted procedure: k18828 
o|contracted procedure: k18238 
o|contracted procedure: k18259 
o|contracted procedure: k18244 
o|contracted procedure: k18254 
o|contracted procedure: k18250 
o|contracted procedure: k18824 
o|contracted procedure: k18265 
o|contracted procedure: k18340 
o|contracted procedure: k18271 
o|contracted procedure: k18331 
o|contracted procedure: k18310 
o|contracted procedure: k18323 
o|contracted procedure: k18327 
o|contracted procedure: k18287 
o|contracted procedure: k18295 
o|contracted procedure: k18335 
o|contracted procedure: k18466 
o|contracted procedure: k18354 
o|contracted procedure: k18357 
o|contracted procedure: k18360 
o|contracted procedure: k18372 
o|contracted procedure: k18386 
o|contracted procedure: k18382 
o|contracted procedure: k17667 
o|contracted procedure: k17674 
o|contracted procedure: k18406 
o|contracted procedure: k18417 
o|contracted procedure: k18433 
o|contracted procedure: k18425 
o|contracted procedure: k18429 
o|contracted procedure: k18421 
o|contracted procedure: k18458 
o|contracted procedure: k18436 
o|contracted procedure: k18453 
o|contracted procedure: k18442 
o|contracted procedure: k18449 
o|contracted procedure: k18476 
o|contracted procedure: k18483 
o|contracted procedure: k18511 
o|contracted procedure: k18489 
o|contracted procedure: k18502 
o|contracted procedure: k18506 
o|contracted procedure: k18518 
o|contracted procedure: k18525 
o|contracted procedure: k18532 
o|contracted procedure: k18541 
o|contracted procedure: k18548 
o|contracted procedure: k18567 
o|contracted procedure: k18571 
o|contracted procedure: k18578 
o|contracted procedure: k18817 
o|contracted procedure: k18584 
o|contracted procedure: k18650 
o|contracted procedure: k18590 
o|contracted procedure: k18593 
o|contracted procedure: k18596 
o|contracted procedure: k18599 
o|contracted procedure: k18620 
o|contracted procedure: k18623 
o|contracted procedure: k18626 
o|contracted procedure: k18634 
o|contracted procedure: k18642 
o|contracted procedure: k18813 
o|contracted procedure: k18656 
o|contracted procedure: k18809 
o|contracted procedure: k18662 
o|contracted procedure: k18805 
o|contracted procedure: k18798 
o|contracted procedure: k18665 
o|contracted procedure: k18795 
o|contracted procedure: k18785 
o|contracted procedure: k18668 
o|contracted procedure: k18674 
o|contracted procedure: k18680 
o|contracted procedure: k18714 
o|contracted procedure: k18717 
o|contracted procedure: k18726 
o|contracted procedure: k18732 
o|contracted procedure: k18735 
o|contracted procedure: k18753 
o|contracted procedure: k18756 
o|contracted procedure: k18759 
o|contracted procedure: k18767 
o|contracted procedure: k18775 
o|contracted procedure: k18876 
o|contracted procedure: k18839 
o|contracted procedure: k18872 
o|contracted procedure: k18845 
o|contracted procedure: k18868 
o|contracted procedure: k18851 
o|contracted procedure: k18864 
o|contracted procedure: k18857 
o|contracted procedure: k18891 
o|contracted procedure: k18934 
o|contracted procedure: k18927 
o|contracted procedure: k18921 
o|contracted procedure: k18937 
o|contracted procedure: k18945 
o|contracted procedure: k18953 
o|contracted procedure: k18973 
o|contracted procedure: k18941 
o|contracted procedure: k18985 
o|contracted procedure: k18988 
o|contracted procedure: k18991 
o|contracted procedure: k18999 
o|contracted procedure: k19007 
o|contracted procedure: k19047 
o|contracted procedure: k19022 
o|contracted procedure: k19066 
o|contracted procedure: k19077 
o|contracted procedure: k19100 
o|contracted procedure: k19103 
o|contracted procedure: k19116 
o|contracted procedure: k19122 
o|inlining procedure: k19119 
o|contracted procedure: k19131 
o|contracted procedure: k19231 
o|contracted procedure: k19137 
o|contracted procedure: k19144 
o|contracted procedure: k19167 
o|contracted procedure: k19170 
o|contracted procedure: k19177 
o|inlining procedure: k19119 
o|contracted procedure: k19186 
o|contracted procedure: k19189 
o|contracted procedure: k19192 
o|contracted procedure: k19200 
o|contracted procedure: k19208 
o|contracted procedure: k19227 
o|contracted procedure: k19217 
o|inlining procedure: k19119 
o|contracted procedure: k19251 
o|contracted procedure: k19254 
o|contracted procedure: k19257 
o|contracted procedure: k19265 
o|contracted procedure: k19273 
o|contracted procedure: k19282 
o|contracted procedure: k19289 
o|contracted procedure: k19292 
o|contracted procedure: k19304 
o|contracted procedure: k19307 
o|contracted procedure: k19310 
o|contracted procedure: k19318 
o|contracted procedure: k19326 
o|contracted procedure: k19443 
o|contracted procedure: k19338 
o|contracted procedure: k19439 
o|contracted procedure: k19344 
o|contracted procedure: k19347 
o|contracted procedure: k19350 
o|contracted procedure: k19357 
o|contracted procedure: k19366 
o|contracted procedure: k19386 
o|contracted procedure: k19394 
o|contracted procedure: k19390 
o|contracted procedure: k19415 
o|contracted procedure: k19418 
o|contracted procedure: k19425 
o|contracted procedure: k19428 
o|contracted procedure: k19435 
o|contracted procedure: k19452 
o|contracted procedure: k19464 
o|contracted procedure: k19470 
o|contracted procedure: k19481 
o|contracted procedure: k19488 
o|contracted procedure: k19590 
o|contracted procedure: k19505 
o|contracted procedure: k19586 
o|contracted procedure: k19511 
o|contracted procedure: k19517 
o|contracted procedure: k19582 
o|contracted procedure: k19523 
o|contracted procedure: k19529 
o|contracted procedure: k19535 
o|contracted procedure: k19541 
o|contracted procedure: k19547 
o|contracted procedure: k19553 
o|contracted procedure: k19565 
o|contracted procedure: k19559 
o|contracted procedure: k19646 
o|contracted procedure: k19607 
o|contracted procedure: k19613 
o|contracted procedure: k19630 
o|contracted procedure: k19626 
o|contracted procedure: k19601 
o|contracted procedure: k19638 
o|contracted procedure: k19634 
o|contracted procedure: k19664 
o|contracted procedure: k19808 
o|contracted procedure: k19676 
o|contracted procedure: k19682 
o|contracted procedure: k19688 
o|contracted procedure: k19804 
o|contracted procedure: k19697 
o|contracted procedure: k19734 
o|contracted procedure: k19703 
o|contracted procedure: k19716 
o|contracted procedure: k6927 
o|contracted procedure: k6941 
o|contracted procedure: k6951 
o|contracted procedure: k6945 
o|contracted procedure: k19800 
o|contracted procedure: k19740 
o|contracted procedure: k19752 
o|contracted procedure: k19786 
o|contracted procedure: k19776 
o|contracted procedure: k19780 
o|contracted procedure: k19789 
o|contracted procedure: k19796 
o|contracted procedure: k19931 
o|contracted procedure: k19855 
o|contracted procedure: k19927 
o|contracted procedure: k19861 
o|contracted procedure: k19867 
o|contracted procedure: k19873 
o|contracted procedure: k19879 
o|contracted procedure: k19885 
o|contracted procedure: k19817 
o|contracted procedure: k19833 
o|contracted procedure: k19823 
o|contracted procedure: k19830 
o|contracted procedure: k19972 
o|contracted procedure: k19981 
o|contracted procedure: k19984 
o|contracted procedure: k19987 
o|contracted procedure: k20101 
o|contracted procedure: k19993 
o|contracted procedure: k20018 
o|contracted procedure: k20025 
o|contracted procedure: k20038 
o|contracted procedure: k20034 
o|contracted procedure: k20042 
o|contracted procedure: k20093 
o|contracted procedure: k20052 
o|contracted procedure: k20065 
o|contracted procedure: k20075 
o|contracted procedure: k20237 
o|contracted procedure: k20245 
o|contracted procedure: k20248 
o|contracted procedure: k20277 
o|contracted procedure: k736126220 
o|contracted procedure: k20253 
o|contracted procedure: k20259 
o|contracted procedure: k20271 
o|contracted procedure: k20262 
o|contracted procedure: k20292 
o|contracted procedure: k20307 
o|contracted procedure: k20319 
o|contracted procedure: k20334 
o|contracted procedure: k20337 
o|contracted procedure: k20340 
o|contracted procedure: k20348 
o|contracted procedure: k20356 
o|contracted procedure: k20367 
o|contracted procedure: k20370 
o|contracted procedure: k7228 
o|contracted procedure: k7235 
o|contracted procedure: k20382 
o|contracted procedure: k20385 
o|contracted procedure: k20388 
o|contracted procedure: k20396 
o|contracted procedure: k20404 
o|contracted procedure: k20426 
o|contracted procedure: k20432 
o|contracted procedure: k20465 
o|contracted procedure: k20438 
o|contracted procedure: k20459 
o|contracted procedure: k20445 
o|contracted procedure: k20453 
o|contracted procedure: k20471 
o|contracted procedure: k20483 
o|contracted procedure: k20493 
o|contracted procedure: k20497 
o|contracted procedure: k20532 
o|contracted procedure: k20541 
o|contracted procedure: k20629 
o|contracted procedure: k20547 
o|contracted procedure: k20573 
o|contracted procedure: k20568 
o|contracted procedure: k20559 
o|contracted procedure: k20578 
o|contracted procedure: k20586 
o|contracted procedure: k20601 
o|contracted procedure: k20604 
o|contracted procedure: k20607 
o|contracted procedure: k20615 
o|contracted procedure: k20623 
o|contracted procedure: k21071 
o|contracted procedure: k20664 
o|contracted procedure: k21068 
o|contracted procedure: k20676 
o|contracted procedure: k20714 
o|contracted procedure: k20722 
o|contracted procedure: k20725 
o|contracted procedure: k20728 
o|contracted procedure: k20740 
o|contracted procedure: k20743 
o|contracted procedure: k20746 
o|contracted procedure: k20754 
o|contracted procedure: k20762 
o|contracted procedure: k21063 
o|contracted procedure: k20768 
o|contracted procedure: k20777 
o|contracted procedure: k20789 
o|contracted procedure: k20792 
o|contracted procedure: k20802 
o|contracted procedure: k20811 
o|contracted procedure: k20833 
o|contracted procedure: k20829 
o|contracted procedure: k20814 
o|contracted procedure: k20817 
o|contracted procedure: k20825 
o|contracted procedure: k20658 
o|contracted procedure: k20842 
o|contracted procedure: k20854 
o|contracted procedure: k20857 
o|contracted procedure: k20860 
o|contracted procedure: k20872 
o|contracted procedure: k20875 
o|contracted procedure: k20878 
o|contracted procedure: k20886 
o|contracted procedure: k20894 
o|contracted procedure: k20903 
o|contracted procedure: k20909 
o|contracted procedure: k20912 
o|contracted procedure: k20918 
o|contracted procedure: k20933 
o|contracted procedure: k20940 
o|contracted procedure: k20951 
o|contracted procedure: k20970 
o|contracted procedure: k20973 
o|contracted procedure: k20976 
o|contracted procedure: k20988 
o|contracted procedure: k20991 
o|contracted procedure: k20994 
o|contracted procedure: k21002 
o|contracted procedure: k21010 
o|contracted procedure: k21022 
o|contracted procedure: k21025 
o|contracted procedure: k21028 
o|contracted procedure: k21036 
o|contracted procedure: k21044 
o|contracted procedure: k21053 
o|contracted procedure: k21060 
o|contracted procedure: k21116 
o|contracted procedure: k21125 
o|contracted procedure: k21131 
o|contracted procedure: k21138 
o|contracted procedure: k21147 
o|contracted procedure: k21159 
o|contracted procedure: k21162 
o|contracted procedure: k21165 
o|contracted procedure: k21173 
o|contracted procedure: k21181 
o|contracted procedure: k21187 
o|contracted procedure: k21239 
o|contracted procedure: k17450 
o|contracted procedure: k17479 
o|contracted procedure: k17459 
o|contracted procedure: k17475 
o|contracted procedure: k17465 
o|contracted procedure: k17485 
o|contracted procedure: k21424 
o|contracted procedure: k21263 
o|contracted procedure: k21297 
o|contracted procedure: k21307 
o|contracted procedure: k21421 
o|contracted procedure: k21316 
o|contracted procedure: k21349 
o|inlining procedure: k21411 
o|inlining procedure: k21411 
o|contracted procedure: k21472 
o|contracted procedure: k21442 
o|contracted procedure: k21457 
o|contracted procedure: k21461 
o|contracted procedure: k21475 
o|contracted procedure: k21570 
o|contracted procedure: k21577 
o|contracted procedure: k21591 
o|contracted procedure: k21594 
o|contracted procedure: k21597 
o|contracted procedure: k21603 
o|contracted procedure: k21664 
o|contracted procedure: k21686 
o|contracted procedure: k21916 
o|contracted procedure: k21965 
o|contracted procedure: k22041 
o|contracted procedure: k22010 
o|contracted procedure: k22059 
o|contracted procedure: k22173 
o|contracted procedure: k22169 
o|contracted procedure: k22186 
o|contracted procedure: k22235 
o|contracted procedure: k22204 
o|contracted procedure: k22248 
o|contracted procedure: k22355 
o|contracted procedure: k22270 
o|contracted procedure: k22368 
o|contracted procedure: k22475 
o|contracted procedure: k22390 
o|contracted procedure: k22929 
o|contracted procedure: k22852 
o|contracted procedure: k22878 
o|contracted procedure: k20138 
o|contracted procedure: k20147 
o|contracted procedure: k20153 
o|contracted procedure: k20159 
o|contracted procedure: k20165 
o|contracted procedure: k20181 
o|contracted procedure: k20132 
o|contracted procedure: k20196 
o|contracted procedure: k20202 
o|contracted procedure: k20209 
o|contracted procedure: k22997 
o|contracted procedure: k22951 
o|contracted procedure: k22993 
o|contracted procedure: k22957 
o|contracted procedure: k22963 
o|contracted procedure: k22989 
o|contracted procedure: k22969 
o|contracted procedure: k22979 
o|contracted procedure: k22945 
o|contracted procedure: k23012 
o|contracted procedure: k23015 
o|contracted procedure: k23018 
o|contracted procedure: k23008 
o|contracted procedure: k23030 
o|contracted procedure: k23052 
o|contracted procedure: k23048 
o|contracted procedure: k23033 
o|contracted procedure: k23036 
o|contracted procedure: k23044 
o|contracted procedure: k23069 
o|contracted procedure: k23072 
o|contracted procedure: k23075 
o|contracted procedure: k23065 
o|contracted procedure: k23087 
o|contracted procedure: k23109 
o|contracted procedure: k23105 
o|contracted procedure: k23090 
o|contracted procedure: k23093 
o|contracted procedure: k23101 
o|contracted procedure: k23177 
o|contracted procedure: k23121 
o|contracted procedure: k23132 
o|contracted procedure: k23136 
o|contracted procedure: k23128 
o|contracted procedure: k23148 
o|contracted procedure: k23170 
o|contracted procedure: k23166 
o|contracted procedure: k23151 
o|contracted procedure: k23154 
o|contracted procedure: k23162 
o|contracted procedure: k23242 
o|contracted procedure: k23186 
o|contracted procedure: k23197 
o|contracted procedure: k23201 
o|contracted procedure: k23193 
o|contracted procedure: k23213 
o|contracted procedure: k23235 
o|contracted procedure: k23231 
o|contracted procedure: k23216 
o|contracted procedure: k23219 
o|contracted procedure: k23227 
o|contracted procedure: k23265 
o|contracted procedure: k23272 
o|contracted procedure: k23293 
o|contracted procedure: k23299 
o|contracted procedure: k23329 
o|contracted procedure: k23305 
o|contracted procedure: k23311 
o|contracted procedure: k23322 
o|contracted procedure: k23318 
o|contracted procedure: k23400 
o|contracted procedure: k23344 
o|contracted procedure: k23350 
o|contracted procedure: k23356 
o|contracted procedure: k23362 
o|contracted procedure: k23368 
o|contracted procedure: k23377 
o|contracted procedure: k23384 
o|contracted procedure: k23338 
o|simplifications: ((if . 17) (let . 542)) 
o|removed binding forms: 1695 
o|inlining procedure: "(scrutinizer.scm:2771) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|inlining procedure: "(scrutinizer.scm:2796) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|inlining procedure: "(scrutinizer.scm:2615) chicken.compiler.scrutinizer#multiples" 
o|inlining procedure: "(scrutinizer.scm:2614) chicken.compiler.scrutinizer#multiples" 
o|inlining procedure: "(scrutinizer.scm:2596) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|contracted procedure: k9684 
o|inlining procedure: "(scrutinizer.scm:2963) chicken.compiler.scrutinizer#multiples" 
o|inlining procedure: "(scrutinizer.scm:2961) chicken.compiler.scrutinizer#multiples" 
o|contracted procedure: k12084 
o|inlining procedure: "(scrutinizer.scm:976) chicken.compiler.scrutinizer#maybe-expand-type" 
o|inlining procedure: "(scrutinizer.scm:975) chicken.compiler.scrutinizer#maybe-expand-type" 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest28172820 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest28172820 0 
o|inlining procedure: "(scrutinizer.scm:1339) chicken.compiler.scrutinizer#maybe-expand-type" 
o|inlining procedure: "(scrutinizer.scm:1338) chicken.compiler.scrutinizer#maybe-expand-type" 
o|contracted procedure: k15401 
o|contracted procedure: k15950 
o|contracted procedure: k16898 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest34413444 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest34413444 0 
o|contracted procedure: k17508 
o|inlining procedure: k18710 
o|inlining procedure: k18710 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest40684071 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest40684071 0 
o|contracted procedure: k19074 
o|contracted procedure: k19379 
o|contracted procedure: k19400 
o|inlining procedure: "(scrutinizer.scm:2159) list-or-null4383" 
o|inlining procedure: "(scrutinizer.scm:2158) list-or-null4383" 
o|inlining procedure: "(scrutinizer.scm:2163) list-or-null4383" 
o|contracted procedure: k19769 
o|inlining procedure: k20241 
o|inlining procedure: k20241 
o|inlining procedure: "(scrutinizer.scm:2621) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|inlining procedure: "(scrutinizer.scm:2655) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|inlining procedure: "(scrutinizer.scm:2683) chicken.compiler.scrutinizer#call-node-procedure-name" 
o|removed binding forms: 12 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#maybe-expand-type 
o|substituted constant variable: r1871128162 
o|removed side-effect free assignment to unused variable: list-or-null4383 
o|contracted procedure: "(scrutinizer.scm:2191) split-list-type4384" 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#multiples 
o|removed side-effect free assignment to unused variable: chicken.compiler.scrutinizer#call-node-procedure-name 
o|replaced variables: 43 
o|removed binding forms: 5 
o|inlining procedure: k19730 
o|inlining procedure: k19759 
o|converted assignments to bindings: (list+index-call-result-type-special-case4386) 
o|simplifications: ((let . 1)) 
o|replaced variables: 25 
o|removed binding forms: 40 
o|contracted procedure: k21883 
o|contracted procedure: k21887 
o|contracted procedure: k22779 
o|contracted procedure: k22791 
o|contracted procedure: k19726 
o|removed binding forms: 11 
o|direct leaf routine/allocation: g364365 3 
o|direct leaf routine/allocation: a8472 0 
o|direct leaf routine/allocation: tag805 29 
o|direct leaf routine/allocation: chicken.compiler.scrutinizer#named? 0 
o|direct leaf routine/allocation: chicken.compiler.scrutinizer#rest-type 0 
o|direct leaf routine/allocation: chicken.compiler.scrutinizer#noreturn-procedure-type? 0 
o|direct leaf routine/allocation: chicken.compiler.scrutinizer#refinement-type? 0 
o|direct leaf routine/allocation: doloop32603261 0 
o|direct leaf routine/allocation: loop3719 0 
o|direct leaf routine/allocation: g38303831 3 
o|contracted procedure: "(mini-srfi-1.scm:216) k7897" 
o|contracted procedure: "(scrutinizer.scm:334) k9216" 
o|contracted procedure: "(scrutinizer.scm:455) k9923" 
o|contracted procedure: "(scrutinizer.scm:455) k9927" 
o|contracted procedure: "(scrutinizer.scm:537) k10077" 
o|contracted procedure: "(scrutinizer.scm:801) k11639" 
o|contracted procedure: "(scrutinizer.scm:888) k11939" 
o|contracted procedure: "(scrutinizer.scm:890) k11954" 
o|contracted procedure: "(scrutinizer.scm:902) k12023" 
o|contracted procedure: "(scrutinizer.scm:1211) k13992" 
o|contracted procedure: "(scrutinizer.scm:1239) k14359" 
o|contracted procedure: "(scrutinizer.scm:1288) k14524" 
o|contracted procedure: "(scrutinizer.scm:1289) k14528" 
o|contracted procedure: "(scrutinizer.scm:1439) k15408" 
o|converted assignments to bindings: (doloop32603261) 
o|converted assignments to bindings: (loop3719) 
o|contracted procedure: "(scrutinizer.scm:2404) k20688" 
o|simplifications: ((let . 2)) 
o|removed binding forms: 15 
o|direct leaf routine/allocation: loop527 0 
o|contracted procedure: k8463 
o|converted assignments to bindings: (loop527) 
o|simplifications: ((let . 1)) 
o|removed binding forms: 1 
o|customizable procedures: (list+index-call-result-type-special-case4386 map-loop44764493 map-loop45104527 map-loop45444561 map-loop45784595 k20141 p5527 p5479 p5457 file-location5303 p-expr5265 p-node5266 rec5196 chicken.compiler.scrutinizer#variable-and-module k21323 g51225131 map-loop51165145 walk5105 map-loop50345051 map-loop50615078 k20961 R4923 map-loop50065023 k20786 map-loop49764994 map-loop49384955 conv4902 g48804889 map-loop48744892 restore4732 loop4812 collect4733 g48294836 for-each-loop48284843 map-loop47514768 loop287 g47844793 map-loop47784803 doloop47364737 derive-result-type4638 lp4640 foldl46564660 lp4404 loop171 k19904 g44514452 known-length-vector-index4314 chicken.compiler.scrutinizer#r-index-out-of-range loop4250 rec4230 g41054114 map-loop40994214 g41274136 map-loop41214154 fail4095 g41804189 map-loop41744207 g40164017 g40304039 map-loop40244049 k18911 k18120 map-loop39854002 k18696 map-loop39433960 g39353936 loop3923 g38893890 k18363 memq*3705 k18398 loop3710 k18304 map-loop38383855 g37943803 map-loop37883812 validate3707 validate-llist3706 k17558 subst3668 map-loop36263643 chicken.compiler.scrutinizer#source-node k17172 wrap3582 loop281 adjust3491 loop3498 k16819 for-each-loop34753554 k16479 loop3395 g34103419 map-loop34043429 g33633372 map-loop33573382 g33113320 map-loop33053330 g32843285 resolve3275 for-each-loop32403250 k16187 map-loop32123230 for-each-loop31773187 for-each-loop31943204 for-each-loop31423152 for-each-loop31593169 loop3125 g30323033 loop12993 k15360 loop3006 k14893 k14927 map-loop28872906 g28562857 g28452846 loop2836 a14777 g27102711 g26922693 loop421 map-loop26412658 map-loop26672684 map-loop26122629 map-loop25832600 chicken.compiler.scrutinizer#canonicalize-list-type foldl475479 foldl459463 map-loop24582475 chicken.compiler.scrutinizer#delete-duplicates chicken.compiler.scrutinizer#lset=/eq? loop2531 k13714 chicken.compiler.scrutinizer#merge-argument-types loop2759 foldl25012505 map-loop24252444 chicken.compiler.scrutinizer#filter-map simplify2408 loop2391 k12692 every-match12039 chicken.compiler.scrutinizer#procedure-arguments chicken.compiler.scrutinizer#procedure-results match-args2034 chicken.compiler.scrutinizer#over-all-instantiations g22052206 g21942195 g21662167 g21332134 g21472148 rawmatch12038 loop2109 match-results2037 loop150 match12040 match-rest2035 loop2044 loop1931 map-loop19461963 doloop19281929 for-each-loop18751895 g18501857 for-each-loop18491860 loop1837 chicken.compiler.scrutinizer#node-source-prefix chicken.compiler.scrutinizer#location-name chicken.compiler.scrutinizer#string-add-indent map-loop56155632 chicken.compiler.scrutinizer#compatible-types? k11287 k11290 g15951605 map-loop15891618 map-loop16361653 g17221730 for-each-loop17211776 g17921801 map-loop17861811 k10877 g17681769 smash1676 g16911700 map-loop16851710 k10747 call-result804 k10600 k10577 k10580 k10451 k10466 chicken.compiler.scrutinizer#smash-component-types! k10345 loop1554 initial-argument-types807 map-loop14041425 k10190 k10105 g14731474 loop1462 k9984 k9968 loop1373 single801 always-true799 k9640 g13061313 for-each-loop13051318 add-to-blist806 k9690 map-loop13341358 walk808 variable-result797 g12051214 map-loop11991227 loop1183 map-loop9841001 chicken.compiler.scrutinizer#make-list k8740 chicken.compiler.scrutinizer#strip-namespace chicken.compiler.scrutinizer#r-proc-call-argument-type-mismatch doloop10401041 k8777 k15598 loop3056 g30513052 loop13043 chicken.compiler.scrutinizer#procedure-name k9098 g10981099 chicken.compiler.scrutinizer#append-map loop1101 g10921093 chicken.compiler.scrutinizer#mutate-node! g10611062 chicken.compiler.scrutinizer#match-argument-types chicken.compiler.scrutinizer#trail-restore chicken.compiler.scrutinizer#fragment chicken.compiler.scrutinizer#specialize-node! k8786 k8852 k8789 g11451154 map-loop11391164 chicken.compiler.scrutinizer#procedure-argument-types chicken.compiler.scrutinizer#describe-expression chicken.compiler.scrutinizer#variable-from-module variable-node-name5331 chicken.compiler.scrutinizer#pp-fragment chicken.compiler.scrutinizer#type->pp-string blist-type796 k8506 g891892 chicken.compiler.scrutinizer#r-deprecated-identifier map-loop854871 chicken.compiler.scrutinizer#simplify-type constant-result794 map-loop822839 chicken.compiler.scrutinizer#list-tabulate chicken.compiler.scrutinizer#every loop499 map-loop377395 foldr349352 g354355 loop294 loop314 loop268 loop255 chicken.compiler.scrutinizer#any map-loop199217 map-loop226244 loop195 foldr184187 g189190 chicken.compiler.scrutinizer#take) 
o|calls to known targets: 939 
o|identified direct recursive calls: f_6864 1 
o|identified direct recursive calls: f_6970 1 
o|identified direct recursive calls: f_7042 1 
o|identified direct recursive calls: f_7091 1 
o|identified direct recursive calls: f_7418 1 
o|identified direct recursive calls: f_7491 1 
o|unused rest argument: _55925871 f25872 
o|unused rest argument: _559 f_7959 
o|identified direct recursive calls: f_8089 1 
o|identified direct recursive calls: f_7886 1 
o|identified direct recursive calls: f_8555 2 
o|identified direct recursive calls: f_15602 1 
o|identified direct recursive calls: f_15559 1 
o|identified direct recursive calls: f_10238 1 
o|identified direct recursive calls: f_10355 1 
o|identified direct recursive calls: f_11683 1 
o|identified direct recursive calls: f_11867 2 
o|identified direct recursive calls: f_6820 1 
o|identified direct recursive calls: f_7590 2 
o|identified direct recursive calls: f_14472 1 
o|unused rest argument: rest28172820 f_14769 
o|identified direct recursive calls: f_15119 1 
o|identified direct recursive calls: f_15171 1 
o|identified direct recursive calls: f_15222 1 
o|identified direct recursive calls: f_15279 1 
o|identified direct recursive calls: f_15364 2 
o|identified direct recursive calls: f_15345 1 
o|identified direct recursive calls: f_15687 1 
o|identified direct recursive calls: f_15760 1 
o|identified direct recursive calls: f_15819 1 
o|identified direct recursive calls: f_15863 1 
o|identified direct recursive calls: f_15916 1 
o|identified direct recursive calls: f_16248 1 
o|identified direct recursive calls: f_16627 2 
o|identified direct recursive calls: f_16295 2 
o|identified direct recursive calls: f_16887 1 
o|unused rest argument: rest34413444 f_16750 
o|identified direct recursive calls: f_7195 1 
o|identified direct recursive calls: f_17214 1 
o|identified direct recursive calls: f_17690 1 
o|identified direct recursive calls: f_17718 1 
o|identified direct recursive calls: f_17662 1 
o|unused rest argument: rest40684071 f_19020 
o|identified direct recursive calls: f_19361 1 
o|identified direct recursive calls: f_19459 1 
o|identified direct recursive calls: f_6922 1 
o|identified direct recursive calls: f_19747 1 
o|identified direct recursive calls: f_20013 1 
o|identified direct recursive calls: f_7223 1 
o|identified direct recursive calls: f_21311 1 
o|fast box initializations: 142 
o|fast global references: 387 
o|fast global assignments: 90 
o|dropping unused closure argument: f_11647 
o|dropping unused closure argument: f_13431 
o|dropping unused closure argument: f_13503 
o|dropping unused closure argument: f_14472 
o|dropping unused closure argument: f_14710 
o|dropping unused closure argument: f_15171 
o|dropping unused closure argument: f_15222 
o|dropping unused closure argument: f_15279 
o|dropping unused closure argument: f_15336 
o|dropping unused closure argument: f_15687 
o|dropping unused closure argument: f_15740 
o|dropping unused closure argument: f_15760 
o|dropping unused closure argument: f_15863 
o|dropping unused closure argument: f_16242 
o|dropping unused closure argument: f_17320 
o|dropping unused closure argument: f_17338 
o|dropping unused closure argument: f_17500 
o|dropping unused closure argument: f_17684 
o|dropping unused closure argument: f_19333 
o|dropping unused closure argument: f_19447 
o|dropping unused closure argument: f_19497 
o|dropping unused closure argument: f_19842 
o|dropping unused closure argument: f_20220 
o|dropping unused closure argument: f_20545 
o|dropping unused closure argument: f_20636 
o|dropping unused closure argument: f_21102 
o|dropping unused closure argument: f_21207 
o|dropping unused closure argument: f_21231 
o|dropping unused closure argument: f_21261 
o|dropping unused closure argument: f_21431 
o|dropping unused closure argument: f_21482 
o|dropping unused closure argument: f_21542 
o|dropping unused closure argument: f_21545 
o|dropping unused closure argument: f_21906 
o|dropping unused closure argument: f_22045 
o|dropping unused closure argument: f_22850 
o|dropping unused closure argument: f_6864 
o|dropping unused closure argument: f_6955 
o|dropping unused closure argument: f_7125 
o|dropping unused closure argument: f_7159 
o|dropping unused closure argument: f_7320 
o|dropping unused closure argument: f_7409 
o|dropping unused closure argument: f_7767 
o|dropping unused closure argument: f_7809 
o|dropping unused closure argument: f_7950 
o|dropping unused closure argument: f_8641 
*/
/* end of file */
