/* Generated from irregex.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: irregex.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -explicit-use -no-trace -output-file irregex.c -emit-import-library chicken.irregex
   unit: irregex
   uses: expand library
*/
#include "chicken.h"

static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_expand_toplevel)
C_externimport void C_ccall C_expand_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[502];
static double C_possibly_force_alignment;
static C_char C_TLS li0[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,118,101,99,116,111,114,45,99,111,112,121,32,118,41,0};
static C_char C_TLS li1[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,63,32,120,41,0,0,0,0};
static C_char C_TLS li2[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,110,117,109,45,115,117,98,109,97,116,99,104,101,115,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li3[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,110,97,109,101,115,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li4[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,110,101,119,45,109,97,116,99,104,101,115,32,105,114,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li5[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,114,101,115,101,116,45,109,97,116,99,104,101,115,33,32,109,41,0,0,0,0,0,0};
static C_char C_TLS li6[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,99,111,112,121,45,109,97,116,99,104,101,115,32,109,41};
static C_char C_TLS li7[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,100,97,116,97,63,32,111,98,106,41,0,0,0,0,0,0,0};
static C_char C_TLS li8[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,110,117,109,45,115,117,98,109,97,116,99,104,101,115,32,109,41};
static C_char C_TLS li9[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,110,97,109,101,115,32,109,41,0};
static C_char C_TLS li10[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,115,116,97,114,116,45,99,104,117,110,107,32,109,32,46,32,111,112,116,41,0,0,0,0,0};
static C_char C_TLS li11[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,115,116,97,114,116,45,105,110,100,101,120,32,109,32,46,32,111,112,116,41,0,0,0,0,0};
static C_char C_TLS li12[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,101,110,100,45,99,104,117,110,107,32,109,32,46,32,111,112,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li13[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,101,110,100,45,105,110,100,101,120,32,109,32,46,32,111,112,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li14[] C_aligned={C_lihdr(0,0,16),40,108,112,32,108,115,32,117,110,107,110,111,119,110,63,41};
static C_char C_TLS li15[] C_aligned={C_lihdr(0,0,60),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,110,117,109,101,114,105,99,45,105,110,100,101,120,32,108,111,99,97,116,105,111,110,32,109,32,111,112,116,41,0,0,0,0};
static C_char C_TLS li16[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,118,97,108,105,100,45,105,110,100,101,120,63,32,109,32,110,41};
static C_char C_TLS li17[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,115,117,98,115,116,114,105,110,103,32,109,32,46,32,111,112,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li18[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,45,115,117,98,99,104,117,110,107,32,109,32,46,32,111,112,116,41};
static C_char C_TLS li19[] C_aligned={C_lihdr(0,0,12),40,102,95,56,53,54,52,32,99,110,107,41,0,0,0,0};
static C_char C_TLS li20[] C_aligned={C_lihdr(0,0,12),40,102,95,56,53,52,55,32,99,110,107,41,0,0,0,0};
static C_char C_TLS li21[] C_aligned={C_lihdr(0,0,14),40,108,111,111,112,32,99,110,107,32,114,101,115,41,0,0};
static C_char C_TLS li22[] C_aligned={C_lihdr(0,0,28),40,102,95,56,52,52,54,32,99,110,107,49,32,115,116,97,114,116,32,99,110,107,50,32,101,110,100,41,0,0,0,0};
static C_char C_TLS li23[] C_aligned={C_lihdr(0,0,59),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,97,107,101,45,105,114,114,101,103,101,120,45,99,104,117,110,107,101,114,32,103,101,116,45,110,101,120,116,32,103,101,116,45,115,116,114,32,46,32,111,41,0,0,0,0,0};
static C_char C_TLS li24[] C_aligned={C_lihdr(0,0,10),40,108,112,32,115,116,97,114,116,41,0,0,0,0,0,0};
static C_char C_TLS li25[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,104,117,110,107,101,114,45,112,114,101,118,45,99,104,97,114,32,99,110,107,32,115,116,97,114,116,32,101,110,100,41,0,0,0,0,0,0,0};
static C_char C_TLS li26[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,104,117,110,107,101,114,45,110,101,120,116,45,99,104,97,114,32,99,110,107,32,115,114,99,41,0,0,0,0,0};
static C_char C_TLS li27[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,105,41};
static C_char C_TLS li28[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,115,99,97,110,45,99,104,97,114,32,115,116,114,32,99,32,46,32,111,41,0,0,0,0};
static C_char C_TLS li29[] C_aligned={C_lihdr(0,0,7),40,108,112,32,108,115,41,0};
static C_char C_TLS li30[] C_aligned={C_lihdr(0,0,11),40,97,57,49,49,55,32,115,32,97,41,0,0,0,0,0};
static C_char C_TLS li31[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,99,97,116,45,114,101,118,101,114,115,101,32,115,116,114,105,110,103,45,108,105,115,116,41};
static C_char C_TLS li32[] C_aligned={C_lihdr(0,0,10),40,108,112,32,105,32,114,101,115,41,0,0,0,0,0,0};
static C_char C_TLS li33[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,122,101,114,111,45,116,111,32,110,41,0,0,0,0,0};
static C_char C_TLS li34[] C_aligned={C_lihdr(0,0,7),40,108,112,32,108,115,41,0};
static C_char C_TLS li35[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,102,105,110,100,32,112,114,101,100,32,108,115,41,0,0};
static C_char C_TLS li36[] C_aligned={C_lihdr(0,0,4),40,108,112,41,0,0,0,0};
static C_char C_TLS li37[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,108,97,115,116,32,108,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li38[] C_aligned={C_lihdr(0,0,14),40,108,112,32,104,101,97,100,32,116,97,105,108,41,0,0};
static C_char C_TLS li39[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,97,110,121,32,112,114,101,100,32,108,115,41,0,0,0};
static C_char C_TLS li40[] C_aligned={C_lihdr(0,0,14),40,108,112,32,104,101,97,100,32,116,97,105,108,41,0,0};
static C_char C_TLS li41[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,101,118,101,114,121,32,112,114,101,100,32,108,115,41,0};
static C_char C_TLS li42[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li43[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,102,111,108,100,32,107,111,110,115,32,107,110,105,108,32,108,115,41,0,0,0,0,0};
static C_char C_TLS li44[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,102,108,97,103,45,106,111,105,110,32,97,32,98,41,0};
static C_char C_TLS li45[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,102,108,97,103,45,99,108,101,97,114,32,97,32,98,41};
static C_char C_TLS li46[] C_aligned={C_lihdr(0,0,12),40,97,57,56,49,56,32,122,32,106,51,41,0,0,0,0};
static C_char C_TLS li47[] C_aligned={C_lihdr(0,0,12),40,97,57,56,49,50,32,121,32,106,50,41,0,0,0,0};
static C_char C_TLS li48[] C_aligned={C_lihdr(0,0,11),40,97,57,55,55,56,32,120,32,106,41,0,0,0,0,0};
static C_char C_TLS li49[] C_aligned={C_lihdr(0,0,9),40,108,112,32,105,32,108,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li50[] C_aligned={C_lihdr(0,0,8),40,115,107,105,112,32,105,41};
static C_char C_TLS li51[] C_aligned={C_lihdr(0,0,14),40,97,57,57,51,53,32,115,101,120,112,32,106,41,0,0};
static C_char C_TLS li52[] C_aligned={C_lihdr(0,0,14),40,97,57,57,55,52,32,115,101,120,112,32,106,41,0,0};
static C_char C_TLS li53[] C_aligned={C_lihdr(0,0,9),40,99,111,108,108,101,99,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li54[] C_aligned={C_lihdr(0,0,17),40,115,99,97,110,32,102,114,111,109,32,105,32,114,101,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li55[] C_aligned={C_lihdr(0,0,15),40,97,49,48,49,50,49,32,115,101,120,112,32,106,41,0};
static C_char C_TLS li56[] C_aligned={C_lihdr(0,0,15),40,97,49,48,49,51,57,32,115,101,120,112,32,106,41,0};
static C_char C_TLS li57[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,106,41};
static C_char C_TLS li58[] C_aligned={C_lihdr(0,0,12),40,102,95,57,55,52,54,32,105,32,107,41,0,0,0,0};
static C_char C_TLS li59[] C_aligned={C_lihdr(0,0,14),40,97,49,48,51,49,48,32,114,101,115,32,106,41,0,0};
static C_char C_TLS li60[] C_aligned={C_lihdr(0,0,50),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,119,105,116,104,45,114,101,97,100,45,102,114,111,109,45,115,116,114,105,110,103,32,115,116,114,32,105,32,112,114,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li61[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li62[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,121,109,98,111,108,45,108,105,115,116,45,62,102,108,97,103,115,32,108,115,41,0};
static C_char C_TLS li63[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,97,121,98,101,45,115,116,114,105,110,103,45,62,115,114,101,32,111,98,106,41,0};
static C_char C_TLS li64[] C_aligned={C_lihdr(0,0,12),40,99,97,115,101,100,45,99,104,97,114,41,0,0,0,0};
static C_char C_TLS li65[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,49,52,55,49,32,103,49,52,56,51,41,0,0,0,0};
static C_char C_TLS li66[] C_aligned={C_lihdr(0,0,18),40,99,97,115,101,100,45,115,116,114,105,110,103,32,115,116,114,41,0,0,0,0,0,0};
static C_char C_TLS li67[] C_aligned={C_lihdr(0,0,9),40,99,111,108,108,101,99,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li68[] C_aligned={C_lihdr(0,0,6),40,108,112,32,105,41,0,0};
static C_char C_TLS li69[] C_aligned={C_lihdr(0,0,16),40,99,111,108,108,101,99,116,47,115,105,110,103,108,101,41};
static C_char C_TLS li70[] C_aligned={C_lihdr(0,0,16),40,108,112,32,108,115,32,116,101,114,109,32,114,101,115,41};
static C_char C_TLS li71[] C_aligned={C_lihdr(0,0,15),40,99,111,108,108,101,99,116,47,116,101,114,109,115,41,0};
static C_char C_TLS li72[] C_aligned={C_lihdr(0,0,6),40,115,97,118,101,41,0,0};
static C_char C_TLS li73[] C_aligned={C_lihdr(0,0,14),40,97,49,49,50,52,49,32,115,114,101,32,106,41,0,0};
static C_char C_TLS li74[] C_aligned={C_lihdr(0,0,6),40,106,111,105,110,41,0,0};
static C_char C_TLS li75[] C_aligned={C_lihdr(0,0,9),40,110,101,119,45,114,101,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li76[] C_aligned={C_lihdr(0,0,21),40,108,112,50,32,106,32,102,108,97,103,115,32,105,110,118,101,114,116,63,41,0,0,0};
static C_char C_TLS li77[] C_aligned={C_lihdr(0,0,14),40,97,49,50,48,48,49,32,115,114,101,32,106,41,0,0};
static C_char C_TLS li78[] C_aligned={C_lihdr(0,0,14),40,97,50,54,57,51,56,32,120,32,114,101,115,41,0,0};
static C_char C_TLS li79[] C_aligned={C_lihdr(0,0,12),40,97,49,51,53,48,51,32,99,32,106,41,0,0,0,0};
static C_char C_TLS li80[] C_aligned={C_lihdr(0,0,7),40,103,49,56,56,52,41,0};
static C_char C_TLS li81[] C_aligned={C_lihdr(0,0,13),40,97,49,51,56,51,51,32,99,104,32,106,41,0,0,0};
static C_char C_TLS li82[] C_aligned={C_lihdr(0,0,21),40,103,111,32,105,32,112,114,101,118,45,99,104,97,114,32,99,115,101,116,41,0,0,0};
static C_char C_TLS li83[] C_aligned={C_lihdr(0,0,9),40,99,111,108,108,101,99,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li84[] C_aligned={C_lihdr(0,0,15),40,108,112,32,105,32,102,114,111,109,32,114,101,115,41,0};
static C_char C_TLS li85[] C_aligned={C_lihdr(0,0,13),40,97,49,50,54,52,54,32,99,104,32,106,41,0,0,0};
static C_char C_TLS li86[] C_aligned={C_lihdr(0,0,7),40,108,112,50,32,106,41,0};
static C_char C_TLS li87[] C_aligned={C_lihdr(0,0,14),40,97,49,50,56,55,55,32,115,114,101,32,106,41,0,0};
static C_char C_TLS li88[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,105,41};
static C_char C_TLS li89[] C_aligned={C_lihdr(0,0,24),40,108,112,32,105,32,102,114,111,109,32,102,108,97,103,115,32,114,101,115,32,115,116,41};
static C_char C_TLS li90[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,62,115,114,101,32,115,116,114,32,46,32,111,41,0,0,0};
static C_char C_TLS li91[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,105,41};
static C_char C_TLS li92[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,112,97,114,115,101,45,104,101,120,45,101,115,99,97,112,101,32,115,116,114,32,105,32,101,110,100,41,0,0,0,0,0};
static C_char C_TLS li93[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,104,105,103,104,45,99,104,97,114,63,32,99,41,0,0};
static C_char C_TLS li94[] C_aligned={C_lihdr(0,0,6),40,98,121,116,101,41,0,0};
static C_char C_TLS li95[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,117,116,102,56,45,115,116,114,105,110,103,45,114,101,102,32,115,116,114,32,105,32,108,101,110,41,0,0,0,0,0};
static C_char C_TLS li96[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,117,116,102,56,45,108,111,119,101,115,116,45,100,105,103,105,116,45,111,102,45,108,101,110,103,116,104,32,108,101,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li97[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,104,97,114,45,62,117,116,102,56,45,108,105,115,116,32,99,41,0,0,0,0,0};
static C_char C_TLS li98[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,48,55,55,32,103,50,48,56,57,41,0,0,0,0};
static C_char C_TLS li99[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,48,53,49,32,103,50,48,54,51,41,0,0,0,0};
static C_char C_TLS li100[] C_aligned={C_lihdr(0,0,56),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,117,110,105,99,111,100,101,45,114,97,110,103,101,45,104,101,108,112,101,114,32,111,110,101,32,108,115,32,112,114,101,102,105,120,32,114,101,115,41};
static C_char C_TLS li101[] C_aligned={C_lihdr(0,0,10),40,97,49,52,55,55,51,32,99,41,0,0,0,0,0,0};
static C_char C_TLS li102[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,49,49,51,32,103,50,49,50,53,41,0,0,0,0};
static C_char C_TLS li103[] C_aligned={C_lihdr(0,0,45),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,117,110,105,99,111,100,101,45,114,97,110,103,101,45,117,112,45,102,114,111,109,32,108,111,45,108,115,41,0,0,0};
static C_char C_TLS li104[] C_aligned={C_lihdr(0,0,10),40,97,49,52,57,49,50,32,99,41,0,0,0,0,0,0};
static C_char C_TLS li105[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,49,52,56,32,103,50,49,54,48,41,0,0,0,0};
static C_char C_TLS li106[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,117,110,105,99,111,100,101,45,114,97,110,103,101,45,117,112,45,116,111,32,104,105,45,108,115,41,0,0,0,0,0};
static C_char C_TLS li107[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,50,51,55,32,103,50,50,52,57,41,0,0,0,0};
static C_char C_TLS li108[] C_aligned={C_lihdr(0,0,9),40,103,50,50,50,48,32,105,41,0,0,0,0,0,0,0};
static C_char C_TLS li109[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,50,55,55,32,103,50,50,56,57,41,0,0,0,0};
static C_char C_TLS li110[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,50,49,52,32,103,50,50,50,54,41,0,0,0,0};
static C_char C_TLS li111[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,49,56,52,32,103,50,49,57,54,41,0,0,0,0};
static C_char C_TLS li112[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,51,49,53,32,103,50,51,50,55,41,0,0,0,0};
static C_char C_TLS li113[] C_aligned={C_lihdr(0,0,16),40,108,112,32,108,111,45,108,115,32,104,105,45,108,115,41};
static C_char C_TLS li114[] C_aligned={C_lihdr(0,0,20),40,108,112,32,108,115,32,97,108,116,115,32,108,111,45,99,115,101,116,41,0,0,0,0};
static C_char C_TLS li115[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,62,117,116,102,56,45,112,97,116,116,101,114,110,32,99,115,101,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li116[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,32,120,32,46,32,111,41,0};
static C_char C_TLS li117[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,62,105,114,114,101,103,101,120,32,115,116,114,32,46,32,111,41,0,0,0,0,0,0,0};
static C_char C_TLS li118[] C_aligned={C_lihdr(0,0,44),40,97,50,51,52,48,53,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li119[] C_aligned={C_lihdr(0,0,9),40,114,101,99,32,115,114,101,41,0,0,0,0,0,0,0};
static C_char C_TLS li120[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,53,49,53,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li121[] C_aligned={C_lihdr(0,0,8),40,97,50,51,53,52,52,41};
static C_char C_TLS li122[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,53,51,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li123[] C_aligned={C_lihdr(0,0,8),40,97,50,51,55,51,55,41};
static C_char C_TLS li124[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,55,51,50,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li125[] C_aligned={C_lihdr(0,0,8),40,97,50,51,55,54,51,41};
static C_char C_TLS li126[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,55,53,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li127[] C_aligned={C_lihdr(0,0,8),40,97,50,51,55,57,56,41};
static C_char C_TLS li128[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,55,57,51,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li129[] C_aligned={C_lihdr(0,0,8),40,97,50,51,56,52,52,41};
static C_char C_TLS li130[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,56,51,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li131[] C_aligned={C_lihdr(0,0,8),40,97,50,51,56,54,48,41};
static C_char C_TLS li132[] C_aligned={C_lihdr(0,0,44),40,97,50,51,56,53,52,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li133[] C_aligned={C_lihdr(0,0,8),40,97,50,51,57,48,51,41};
static C_char C_TLS li134[] C_aligned={C_lihdr(0,0,44),40,97,50,51,56,57,55,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li135[] C_aligned={C_lihdr(0,0,45),40,102,95,50,51,57,57,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li136[] C_aligned={C_lihdr(0,0,8),40,97,50,52,48,50,55,41};
static C_char C_TLS li137[] C_aligned={C_lihdr(0,0,44),40,97,50,52,48,48,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li138[] C_aligned={C_lihdr(0,0,16),40,98,111,100,121,52,53,56,55,32,99,111,117,110,116,41};
static C_char C_TLS li139[] C_aligned={C_lihdr(0,0,8),40,97,50,52,48,54,55,41};
static C_char C_TLS li140[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,48,53,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li141[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,49,52,51,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li142[] C_aligned={C_lihdr(0,0,8),40,97,50,52,49,56,51,41};
static C_char C_TLS li143[] C_aligned={C_lihdr(0,0,44),40,97,50,52,49,54,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li144[] C_aligned={C_lihdr(0,0,16),40,98,111,100,121,52,54,52,53,32,99,111,117,110,116,41};
static C_char C_TLS li145[] C_aligned={C_lihdr(0,0,8),40,97,50,52,50,50,55,41};
static C_char C_TLS li146[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,50,49,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li147[] C_aligned={C_lihdr(0,0,8),40,97,50,52,51,56,56,41};
static C_char C_TLS li148[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,51,55,52,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li149[] C_aligned={C_lihdr(0,0,44),40,97,50,52,51,57,53,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li150[] C_aligned={C_lihdr(0,0,8),40,97,50,52,52,50,51,41};
static C_char C_TLS li151[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,52,48,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li152[] C_aligned={C_lihdr(0,0,44),40,97,50,52,52,51,48,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li153[] C_aligned={C_lihdr(0,0,11),40,102,95,50,52,52,57,50,32,120,41,0,0,0,0,0};
static C_char C_TLS li154[] C_aligned={C_lihdr(0,0,8),40,97,50,52,52,56,54,41};
static C_char C_TLS li155[] C_aligned={C_lihdr(0,0,9),40,97,56,55,51,53,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li156[] C_aligned={C_lihdr(0,0,9),40,97,56,55,54,50,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li157[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,52,52,55,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li158[] C_aligned={C_lihdr(0,0,44),40,97,50,52,53,48,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li159[] C_aligned={C_lihdr(0,0,8),40,97,50,52,53,52,52,41};
static C_char C_TLS li160[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,53,51,48,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li161[] C_aligned={C_lihdr(0,0,44),40,97,50,52,53,53,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li162[] C_aligned={C_lihdr(0,0,46),40,102,95,50,52,53,56,51,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,50,41,0,0};
static C_char C_TLS li163[] C_aligned={C_lihdr(0,0,8),40,97,50,52,54,52,54,41};
static C_char C_TLS li164[] C_aligned={C_lihdr(0,0,46),40,102,95,50,52,54,52,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,50,41,0,0};
static C_char C_TLS li165[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,54,56,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li166[] C_aligned={C_lihdr(0,0,24),40,108,112,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,106,32,108,101,110,41};
static C_char C_TLS li167[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,55,50,52,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li168[] C_aligned={C_lihdr(0,0,8),40,97,50,52,57,56,53,41};
static C_char C_TLS li169[] C_aligned={C_lihdr(0,0,45),40,102,95,50,52,57,51,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li170[] C_aligned={C_lihdr(0,0,8),40,97,50,53,48,56,48,41};
static C_char C_TLS li171[] C_aligned={C_lihdr(0,0,44),40,97,50,53,48,52,53,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0,0};
static C_char C_TLS li172[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,50,56,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li173[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,51,51,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li174[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,52,51,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li175[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,52,54,49,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li176[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,53,49,50,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li177[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,54,49,57,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li178[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,54,53,50,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li179[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,55,49,55,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li180[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,56,48,54,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li181[] C_aligned={C_lihdr(0,0,9),40,108,112,32,115,114,99,50,41,0,0,0,0,0,0,0};
static C_char C_TLS li182[] C_aligned={C_lihdr(0,0,45),40,102,95,50,53,57,50,56,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li183[] C_aligned={C_lihdr(0,0,9),40,108,112,32,115,114,99,50,41,0,0,0,0,0,0,0};
static C_char C_TLS li184[] C_aligned={C_lihdr(0,0,45),40,102,95,50,54,48,50,55,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li185[] C_aligned={C_lihdr(0,0,21),40,108,112,32,115,114,101,32,110,32,102,108,97,103,115,32,110,101,120,116,41,0,0,0};
static C_char C_TLS li186[] C_aligned={C_lihdr(0,0,8),40,103,114,111,119,32,105,41};
static C_char C_TLS li187[] C_aligned={C_lihdr(0,0,16),40,97,49,55,50,48,53,32,108,111,51,32,104,105,51,41};
static C_char C_TLS li188[] C_aligned={C_lihdr(0,0,18),40,108,112,50,32,108,115,32,110,32,108,111,50,32,104,105,50,41,0,0,0,0,0,0};
static C_char C_TLS li189[] C_aligned={C_lihdr(0,0,16),40,97,49,55,50,56,55,32,108,111,51,32,104,105,51,41};
static C_char C_TLS li190[] C_aligned={C_lihdr(0,0,18),40,108,112,50,32,108,115,32,110,32,108,111,50,32,104,105,50,41,0,0,0,0,0,0};
static C_char C_TLS li191[] C_aligned={C_lihdr(0,0,16),40,97,49,55,51,56,51,32,108,111,52,32,104,105,52,41};
static C_char C_TLS li192[] C_aligned={C_lihdr(0,0,16),40,97,49,55,51,54,57,32,108,111,51,32,104,105,51,41};
static C_char C_TLS li193[] C_aligned={C_lihdr(0,0,16),40,97,49,55,51,53,53,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li194[] C_aligned={C_lihdr(0,0,16),40,97,49,55,53,48,50,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li195[] C_aligned={C_lihdr(0,0,14),40,97,49,55,54,53,54,32,108,111,32,104,105,41,0,0};
static C_char C_TLS li196[] C_aligned={C_lihdr(0,0,16),40,97,49,55,54,57,53,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li197[] C_aligned={C_lihdr(0,0,16),40,97,49,55,55,52,50,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li198[] C_aligned={C_lihdr(0,0,16),40,97,49,55,56,49,54,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li199[] C_aligned={C_lihdr(0,0,16),40,97,49,55,56,52,52,32,108,111,50,32,104,105,50,41};
static C_char C_TLS li200[] C_aligned={C_lihdr(0,0,12),40,103,51,48,50,55,32,99,101,108,108,41,0,0,0,0};
static C_char C_TLS li201[] C_aligned={C_lihdr(0,0,23),40,108,112,32,115,114,101,32,110,32,108,111,32,104,105,32,114,101,116,117,114,110,41,0};
static C_char C_TLS li202[] C_aligned={C_lihdr(0,0,11),40,103,50,54,50,53,32,110,102,97,41,0,0,0,0,0};
static C_char C_TLS li203[] C_aligned={C_lihdr(0,0,11),40,103,50,54,49,52,32,110,102,97,41,0,0,0,0,0};
static C_char C_TLS li204[] C_aligned={C_lihdr(0,0,9),40,114,101,99,32,115,114,101,41,0,0,0,0,0,0,0};
static C_char C_TLS li205[] C_aligned={C_lihdr(0,0,9),40,103,50,51,56,48,32,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li206[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,51,55,52,32,103,50,51,56,54,41,0,0,0,0};
static C_char C_TLS li207[] C_aligned={C_lihdr(0,0,9),40,103,50,52,48,55,32,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li208[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,52,48,49,32,103,50,52,49,51,41,0,0,0,0};
static C_char C_TLS li209[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,52,52,52,32,103,50,52,53,54,41,0,0,0,0};
static C_char C_TLS li210[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,52,55,48,32,103,50,52,56,50,41,0,0,0,0};
static C_char C_TLS li211[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,50,53,48,52,32,103,50,53,49,54,41,0,0,0,0};
static C_char C_TLS li212[] C_aligned={C_lihdr(0,0,22),40,97,100,106,117,115,116,32,115,114,101,32,117,116,102,56,63,32,99,105,63,41,0,0};
static C_char C_TLS li213[] C_aligned={C_lihdr(0,0,38),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,62,105,114,114,101,103,101,120,32,115,114,101,32,46,32,111,41,0,0};
static C_char C_TLS li214[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,101,109,112,116,121,63,32,115,114,101,41};
static C_char C_TLS li215[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,97,110,121,63,32,115,114,101,41,0,0};
static C_char C_TLS li216[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,114,101,112,101,97,116,101,114,63,41,0};
static C_char C_TLS li217[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,98,111,115,63,32,115,114,101,41,0,0};
static C_char C_TLS li218[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,115,101,97,114,99,104,101,114,63,32,115,114,101,41,0,0,0,0,0};
static C_char C_TLS li219[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,99,111,110,115,117,109,101,114,63,32,115,114,101,41,0,0,0,0,0};
static C_char C_TLS li220[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,104,97,115,45,115,117,98,109,97,116,99,104,101,115,63,32,115,114,101,41,0,0,0,0,0,0,0};
static C_char C_TLS li221[] C_aligned={C_lihdr(0,0,15),40,99,111,117,110,116,32,115,114,101,32,115,117,109,41,0};
static C_char C_TLS li222[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,99,111,117,110,116,45,115,117,98,109,97,116,99,104,101,115,32,115,114,101,41,0,0,0,0,0,0};
static C_char C_TLS li223[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,115,101,113,117,101,110,99,101,41,0,0};
static C_char C_TLS li224[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,97,108,116,101,114,110,97,116,101,41,0};
static C_char C_TLS li225[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,51,48,54,52,32,103,51,48,55,54,41,0,0,0,0};
static C_char C_TLS li226[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,115,116,114,105,112,45,115,117,98,109,97,116,99,104,101,115,32,115,114,101,41,0,0,0,0,0,0};
static C_char C_TLS li227[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,110,97,109,101,115,32,115,114,101,32,110,32,110,97,109,101,115,41,0};
static C_char C_TLS li228[] C_aligned={C_lihdr(0,0,47),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,115,101,113,117,101,110,99,101,45,110,97,109,101,115,32,108,115,32,110,32,110,97,109,101,115,41,0};
static C_char C_TLS li229[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,51,50,48,53,32,103,51,50,49,55,41,0,0,0,0};
static C_char C_TLS li230[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,114,101,109,111,118,101,45,105,110,105,116,105,97,108,45,98,111,115,32,115,114,101,41,0,0,0,0};
static C_char C_TLS li231[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,115,101,97,114,99,104,32,120,32,115,116,114,32,46,32,111,41,0,0,0,0,0,0};
static C_char C_TLS li232[] C_aligned={C_lihdr(0,0,54),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,115,101,97,114,99,104,47,99,104,117,110,107,101,100,32,120,32,99,110,107,32,115,114,99,32,46,32,111,41,0,0};
static C_char C_TLS li233[] C_aligned={C_lihdr(0,0,7),40,108,112,50,32,105,41,0};
static C_char C_TLS li234[] C_aligned={C_lihdr(0,0,11),40,108,112,49,32,115,114,99,32,105,41,0,0,0,0,0};
static C_char C_TLS li235[] C_aligned={C_lihdr(0,0,10),40,97,49,57,55,53,57,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li236[] C_aligned={C_lihdr(0,0,13),40,108,112,50,32,105,32,115,116,97,116,101,41,0,0,0};
static C_char C_TLS li237[] C_aligned={C_lihdr(0,0,21),40,108,112,49,32,115,114,99,32,115,116,97,114,116,32,115,116,97,116,101,41,0,0,0};
static C_char C_TLS li238[] C_aligned={C_lihdr(0,0,8),40,97,49,57,50,50,49,41};
static C_char C_TLS li239[] C_aligned={C_lihdr(0,0,8),40,97,49,57,50,57,51,41};
static C_char C_TLS li240[] C_aligned={C_lihdr(0,0,19),40,108,112,32,115,114,99,50,32,115,116,114,32,105,32,101,110,100,41,0,0,0,0,0};
static C_char C_TLS li241[] C_aligned={C_lihdr(0,0,67),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,115,101,97,114,99,104,47,109,97,116,99,104,101,115,32,105,114,120,32,99,110,107,32,105,110,105,116,32,115,114,99,32,105,32,109,97,116,99,104,101,115,41,0,0,0,0,0};
static C_char C_TLS li242[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,32,105,114,120,32,115,116,114,32,46,32,111,41,0,0,0,0,0};
static C_char C_TLS li243[] C_aligned={C_lihdr(0,0,6),40,108,112,32,109,41,0,0};
static C_char C_TLS li244[] C_aligned={C_lihdr(0,0,8),40,97,49,57,54,49,52,41};
static C_char C_TLS li245[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,47,99,104,117,110,107,101,100,32,105,114,120,32,99,110,107,32,115,114,99,41,0,0,0,0,0};
static C_char C_TLS li246[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,109,97,116,99,104,63,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li247[] C_aligned={C_lihdr(0,0,7),40,103,51,53,49,52,41,0};
static C_char C_TLS li248[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,51,53,49,51,32,103,51,53,50,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li249[] C_aligned={C_lihdr(0,0,52),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,102,105,110,97,108,105,122,101,33,32,102,105,110,97,108,105,122,101,114,32,109,101,109,111,114,121,32,109,97,116,99,104,101,115,41,0,0,0,0};
static C_char C_TLS li250[] C_aligned={C_lihdr(0,0,7),40,103,51,54,54,57,41,0};
static C_char C_TLS li251[] C_aligned={C_lihdr(0,0,9),40,102,95,50,48,49,57,57,41,0,0,0,0,0,0,0};
static C_char C_TLS li252[] C_aligned={C_lihdr(0,0,9),40,103,51,55,48,56,32,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li253[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,51,54,55,56,32,103,51,54,56,53,41,0,0,0,0,0,0,0};
static C_char C_TLS li254[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,51,55,48,50,32,103,51,55,49,52,41,0,0,0,0};
static C_char C_TLS li255[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,51,54,54,56,32,103,51,54,55,53,41,0,0,0,0,0,0,0};
static C_char C_TLS li256[] C_aligned={C_lihdr(0,0,10),40,97,50,48,51,56,56,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li257[] C_aligned={C_lihdr(0,0,41),40,108,112,50,32,105,32,115,116,97,116,101,32,114,101,115,45,115,114,99,32,114,101,115,45,105,110,100,101,120,32,102,105,110,97,108,105,122,101,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li258[] C_aligned={C_lihdr(0,0,49),40,108,112,49,32,115,114,99,32,115,116,97,114,116,32,115,116,97,116,101,32,114,101,115,45,115,114,99,32,114,101,115,45,105,110,100,101,120,32,102,105,110,97,108,105,122,101,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li259[] C_aligned={C_lihdr(0,0,7),40,103,51,53,56,53,41,0};
static C_char C_TLS li260[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,51,53,56,52,32,103,51,53,57,49,41,0,0,0,0,0,0,0};
static C_char C_TLS li261[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,51,53,52,52,32,105,41,0,0};
static C_char C_TLS li262[] C_aligned={C_lihdr(0,0,79),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,100,102,97,45,109,97,116,99,104,47,108,111,110,103,101,115,116,32,100,102,97,32,99,110,107,32,115,114,99,32,115,116,97,114,116,32,101,110,100,45,115,114,99,32,101,110,100,32,109,97,116,99,104,101,115,32,105,110,100,101,120,41,0};
static C_char C_TLS li263[] C_aligned={C_lihdr(0,0,16),40,97,51,48,55,55,51,32,101,115,99,32,46,32,111,41};
static C_char C_TLS li264[] C_aligned={C_lihdr(0,0,36),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,110,102,97,45,103,101,116,45,101,112,115,105,108,111,110,115,32,105,41,0,0,0,0};
static C_char C_TLS li265[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,110,102,97,45,97,100,100,45,101,112,115,105,108,111,110,33,32,105,32,120,32,116,41};
static C_char C_TLS li266[] C_aligned={C_lihdr(0,0,24),40,110,101,119,45,115,116,97,116,101,45,110,117,109,98,101,114,32,115,116,97,116,101,41};
static C_char C_TLS li267[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,51,56,56,51,32,105,41,0,0};
static C_char C_TLS li268[] C_aligned={C_lihdr(0,0,24),40,97,100,100,45,115,116,97,116,101,33,32,110,50,32,116,114,97,110,115,45,108,115,41};
static C_char C_TLS li269[] C_aligned={C_lihdr(0,0,29),40,101,120,116,101,110,100,45,115,116,97,116,101,33,32,110,101,120,116,32,116,114,97,110,115,45,99,115,41,0,0,0};
static C_char C_TLS li270[] C_aligned={C_lihdr(0,0,25),40,97,100,100,45,99,104,97,114,45,115,116,97,116,101,33,32,110,101,120,116,32,99,104,41,0,0,0,0,0,0,0};
static C_char C_TLS li271[] C_aligned={C_lihdr(0,0,12),40,108,112,50,32,105,32,110,101,120,116,41,0,0,0,0};
static C_char C_TLS li272[] C_aligned={C_lihdr(0,0,12),40,103,52,48,49,57,32,99,101,108,108,41,0,0,0,0};
static C_char C_TLS li273[] C_aligned={C_lihdr(0,0,20),40,108,112,32,108,115,32,110,32,102,108,97,103,115,32,110,101,120,116,41,0,0,0,0};
static C_char C_TLS li274[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,52,48,50,57,32,105,41,0,0};
static C_char C_TLS li275[] C_aligned={C_lihdr(0,0,16),40,108,112,32,115,114,101,32,109,97,120,32,114,101,115,41};
static C_char C_TLS li276[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,62,110,102,97,32,115,114,101,32,105,110,105,116,45,102,108,97,103,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li277[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,97,107,101,45,109,115,116,32,110,102,97,41,0,0};
static C_char C_TLS li278[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,115,116,45,99,111,112,121,32,109,115,116,41,0,0};
static C_char C_TLS li279[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,110,102,97,45,115,116,97,116,101,45,62,109,115,116,32,110,102,97,32,115,116,97,116,101,32,109,97,112,112,105,110,103,115,41,0,0,0,0,0};
static C_char C_TLS li280[] C_aligned={C_lihdr(0,0,7),40,103,52,48,56,55,41,0};
static C_char C_TLS li281[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,52,48,56,54,32,103,52,48,57,51,41,0,0,0,0,0,0,0};
static C_char C_TLS li282[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,115,116,45,97,100,100,33,32,110,102,97,32,109,115,116,32,115,116,97,116,101,32,109,97,112,112,105,110,103,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li283[] C_aligned={C_lihdr(0,0,10),40,108,112,32,105,32,97,99,99,41,0,0,0,0,0,0};
static C_char C_TLS li284[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,109,115,116,45,102,111,108,100,32,109,115,116,32,107,111,110,115,32,107,110,105,108,41};
static C_char C_TLS li285[] C_aligned={C_lihdr(0,0,10),40,103,52,50,53,56,32,116,114,41,0,0,0,0,0,0};
static C_char C_TLS li286[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,52,50,53,50,32,103,52,50,54,52,41,0,0,0,0};
static C_char C_TLS li287[] C_aligned={C_lihdr(0,0,21),40,100,111,108,111,111,112,52,50,52,52,32,105,32,115,116,97,116,101,115,41,0,0,0};
static C_char C_TLS li288[] C_aligned={C_lihdr(0,0,21),40,108,112,32,105,32,115,116,97,116,101,115,32,105,110,100,101,120,101,115,41,0,0,0};
static C_char C_TLS li289[] C_aligned={C_lihdr(0,0,36),40,108,112,51,32,99,108,111,115,117,114,101,45,115,108,111,116,115,32,115,116,97,116,101,45,115,108,111,116,115,32,99,109,100,115,41,0,0,0,0};
static C_char C_TLS li290[] C_aligned={C_lihdr(0,0,26),40,108,112,50,32,115,116,97,116,101,45,115,117,109,109,97,114,121,32,116,32,99,109,100,115,41,0,0,0,0,0,0};
static C_char C_TLS li291[] C_aligned={C_lihdr(0,0,6),40,108,112,32,105,41,0,0};
static C_char C_TLS li292[] C_aligned={C_lihdr(0,0,15),40,108,112,32,100,102,97,45,115,116,97,116,101,115,41,0};
static C_char C_TLS li293[] C_aligned={C_lihdr(0,0,37),40,108,112,50,32,116,114,97,110,115,32,117,110,109,97,114,107,101,100,45,115,116,97,116,101,115,32,100,102,97,45,116,114,97,110,115,41,0,0,0};
static C_char C_TLS li294[] C_aligned={C_lihdr(0,0,20),40,103,52,51,48,55,32,105,110,116,101,114,115,101,99,116,105,111,110,41,0,0,0,0};
static C_char C_TLS li295[] C_aligned={C_lihdr(0,0,20),40,108,112,32,108,115,32,99,115,32,115,116,97,116,101,32,114,101,115,41,0,0,0,0};
static C_char C_TLS li296[] C_aligned={C_lihdr(0,0,24),40,97,50,50,55,53,53,32,115,116,32,109,97,112,112,105,110,103,115,32,114,101,115,41};
static C_char C_TLS li297[] C_aligned={C_lihdr(0,0,43),40,108,112,32,117,110,109,97,114,107,101,100,45,115,116,97,116,101,115,32,109,97,114,107,101,100,45,115,116,97,116,101,115,32,100,102,97,45,115,105,122,101,41,0,0,0,0,0};
static C_char C_TLS li298[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,110,102,97,45,62,100,102,97,32,110,102,97,32,111,41};
static C_char C_TLS li299[] C_aligned={C_lihdr(0,0,10),40,108,112,32,109,32,114,101,115,41,0,0,0,0,0,0};
static C_char C_TLS li300[] C_aligned={C_lihdr(0,0,12),40,100,111,108,111,111,112,52,49,56,55,41,0,0,0,0};
static C_char C_TLS li301[] C_aligned={C_lihdr(0,0,11),40,103,52,51,52,50,32,116,97,103,41,0,0,0,0,0};
static C_char C_TLS li302[] C_aligned={C_lihdr(0,0,17),40,108,112,50,32,116,114,97,110,115,32,115,116,97,99,107,41,0,0,0,0,0,0,0};
static C_char C_TLS li303[] C_aligned={C_lihdr(0,0,29),40,108,112,32,115,116,97,99,107,32,112,114,105,111,114,105,116,105,101,115,32,99,108,111,115,117,114,101,41,0,0,0};
static C_char C_TLS li304[] C_aligned={C_lihdr(0,0,17),40,97,50,51,48,52,51,32,115,116,32,109,32,114,101,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li305[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,110,102,97,45,101,112,115,105,108,111,110,45,99,108,111,115,117,114,101,32,110,102,97,32,115,116,97,116,101,115,41};
static C_char C_TLS li306[] C_aligned={C_lihdr(0,0,10),40,97,50,51,49,54,55,32,99,41,0,0,0,0,0,0};
static C_char C_TLS li307[] C_aligned={C_lihdr(0,0,10),40,97,50,51,49,57,49,32,99,41,0,0,0,0,0,0};
static C_char C_TLS li308[] C_aligned={C_lihdr(0,0,16),40,108,112,50,32,115,49,32,115,50,32,99,109,100,115,41};
static C_char C_TLS li309[] C_aligned={C_lihdr(0,0,11),40,108,112,32,116,32,99,109,100,115,41,0,0,0,0,0};
static C_char C_TLS li310[] C_aligned={C_lihdr(0,0,79),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,116,97,103,45,115,101,116,45,99,111,109,109,97,110,100,115,45,102,111,114,45,99,108,111,115,117,114,101,32,110,102,97,32,111,114,105,103,45,115,116,97,116,101,32,99,108,111,115,117,114,101,32,99,111,112,121,45,99,109,100,115,41,0};
static C_char C_TLS li311[] C_aligned={C_lihdr(0,0,45),40,102,95,50,54,49,55,48,32,99,110,107,32,105,110,105,116,32,115,114,99,32,115,116,114,32,105,32,101,110,100,32,109,97,116,99,104,101,115,32,102,97,105,108,41,0,0,0};
static C_char C_TLS li312[] C_aligned={C_lihdr(0,0,47),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,99,115,101,116,45,62,112,114,111,99,101,100,117,114,101,32,99,115,101,116,32,110,101,120,116,41,0};
static C_char C_TLS li313[] C_aligned={C_lihdr(0,0,10),40,108,112,32,105,32,114,101,115,41,0,0,0,0,0,0};
static C_char C_TLS li314[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,62,112,108,105,115,116,32,99,115,41};
static C_char C_TLS li315[] C_aligned={C_lihdr(0,0,14),40,97,50,54,52,55,55,32,99,104,32,99,115,41,0,0};
static C_char C_TLS li316[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,62,99,115,101,116,32,115,41};
static C_char C_TLS li317[] C_aligned={C_lihdr(0,0,9),40,114,101,99,32,115,114,101,41,0,0,0,0,0,0,0};
static C_char C_TLS li318[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,53,50,53,48,32,103,53,50,54,50,41,0,0,0,0};
static C_char C_TLS li319[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,53,50,55,54,32,103,53,50,56,56,41,0,0,0,0};
static C_char C_TLS li320[] C_aligned={C_lihdr(0,0,14),40,97,50,54,54,56,49,32,120,32,114,101,115,41,0,0};
static C_char C_TLS li321[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,53,51,48,52,32,103,53,51,49,54,41,0,0,0,0};
static C_char C_TLS li322[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li323[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li324[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,53,51,51,49,32,103,53,51,52,51,41,0,0,0,0};
static C_char C_TLS li325[] C_aligned={C_lihdr(0,0,12),40,108,112,32,115,114,101,32,99,105,63,41,0,0,0,0};
static C_char C_TLS li326[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,62,99,115,101,116,32,115,114,101,32,111,41,0,0,0,0,0,0,0};
static C_char C_TLS li327[] C_aligned={C_lihdr(0,0,16),40,108,112,32,108,111,119,101,114,32,117,112,112,101,114,41};
static C_char C_TLS li328[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,99,111,110,116,97,105,110,115,63,32,99,115,101,116,32,99,104,41};
static C_char C_TLS li329[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,104,97,114,45,114,97,110,103,101,115,45,117,110,105,111,110,32,98,41,0,0,0};
static C_char C_TLS li330[] C_aligned={C_lihdr(0,0,21),40,117,110,105,111,110,45,114,97,110,103,101,32,97,32,98,32,114,101,115,41,0,0,0};
static C_char C_TLS li331[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,117,110,105,111,110,32,97,32,98,41};
static C_char C_TLS li332[] C_aligned={C_lihdr(0,0,14),40,100,105,102,102,32,97,32,98,32,114,101,115,41,0,0};
static C_char C_TLS li333[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,100,105,102,102,101,114,101,110,99,101,32,97,32,98,41,0,0,0};
static C_char C_TLS li334[] C_aligned={C_lihdr(0,0,19),40,105,110,116,101,114,115,101,99,116,32,97,32,98,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li335[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,105,110,116,101,114,115,101,99,116,105,111,110,32,97,32,98,41,0};
static C_char C_TLS li336[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,99,111,109,112,108,101,109,101,110,116,32,97,41,0,0,0,0,0};
static C_char C_TLS li337[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li338[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,99,97,115,101,45,105,110,115,101,110,115,105,116,105,118,101,32,97,41,0,0,0,0,0,0,0};
static C_char C_TLS li339[] C_aligned={C_lihdr(0,0,15),40,102,95,50,55,57,51,57,32,105,32,97,99,99,41,0};
static C_char C_TLS li340[] C_aligned={C_lihdr(0,0,19),40,108,112,32,115,114,99,32,102,114,111,109,32,105,32,97,99,99,41,0,0,0,0,0};
static C_char C_TLS li341[] C_aligned={C_lihdr(0,0,57),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,102,111,108,100,47,102,97,115,116,32,105,114,120,32,107,111,110,115,32,107,110,105,108,32,115,116,114,32,46,32,111,41,0,0,0,0,0,0,0};
static C_char C_TLS li342[] C_aligned={C_lihdr(0,0,15),40,107,111,110,115,50,32,105,32,109,32,97,99,99,41,0};
static C_char C_TLS li343[] C_aligned={C_lihdr(0,0,46),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,102,111,108,100,32,105,114,120,32,107,111,110,115,32,46,32,97,114,103,115,41,0,0};
static C_char C_TLS li344[] C_aligned={C_lihdr(0,0,19),40,102,95,50,56,49,53,57,32,115,114,99,32,105,32,97,99,99,41,0,0,0,0,0};
static C_char C_TLS li345[] C_aligned={C_lihdr(0,0,16),40,108,112,32,115,116,97,114,116,32,105,32,97,99,99,41};
static C_char C_TLS li346[] C_aligned={C_lihdr(0,0,71),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,102,111,108,100,47,99,104,117,110,107,101,100,47,102,97,115,116,32,105,114,120,32,107,111,110,115,32,107,110,105,108,32,99,110,107,32,115,116,97,114,116,32,46,32,111,41,0};
static C_char C_TLS li347[] C_aligned={C_lihdr(0,0,17),40,107,111,110,115,50,32,115,32,105,32,109,32,97,99,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li348[] C_aligned={C_lihdr(0,0,54),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,102,111,108,100,47,99,104,117,110,107,101,100,32,105,114,120,32,107,111,110,115,32,46,32,97,114,103,115,41,0,0};
static C_char C_TLS li349[] C_aligned={C_lihdr(0,0,45),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,114,101,112,108,97,99,101,32,105,114,120,32,115,116,114,32,46,32,111,41,0,0,0};
static C_char C_TLS li350[] C_aligned={C_lihdr(0,0,16),40,97,50,56,50,56,49,32,105,32,109,32,97,99,99,41};
static C_char C_TLS li351[] C_aligned={C_lihdr(0,0,14),40,97,50,56,51,51,50,32,105,32,97,99,99,41,0,0};
static C_char C_TLS li352[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,114,101,112,108,97,99,101,47,97,108,108,32,105,114,120,32,115,116,114,32,46,32,111,41,0,0,0,0,0,0,0};
static C_char C_TLS li353[] C_aligned={C_lihdr(0,0,9),40,103,53,54,53,56,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li354[] C_aligned={C_lihdr(0,0,11),40,108,112,32,108,115,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li355[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,97,112,112,108,121,45,109,97,116,99,104,32,109,32,108,115,41,0,0,0,0,0,0};
static C_char C_TLS li356[] C_aligned={C_lihdr(0,0,14),40,97,50,56,54,48,53,32,105,32,109,32,97,41,0,0};
static C_char C_TLS li357[] C_aligned={C_lihdr(0,0,12),40,97,50,56,54,49,53,32,105,32,97,41,0,0,0,0};
static C_char C_TLS li358[] C_aligned={C_lihdr(0,0,45),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,101,120,116,114,97,99,116,32,105,114,120,32,115,116,114,32,46,32,111,41,0,0,0};
static C_char C_TLS li359[] C_aligned={C_lihdr(0,0,14),40,97,50,56,54,52,54,32,105,32,109,32,97,41,0,0};
static C_char C_TLS li360[] C_aligned={C_lihdr(0,0,21),40,108,112,32,108,115,32,114,101,115,32,119,97,115,45,99,104,97,114,63,41,0,0,0};
static C_char C_TLS li361[] C_aligned={C_lihdr(0,0,12),40,97,50,56,55,48,52,32,105,32,97,41,0,0,0,0};
static C_char C_TLS li362[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,115,112,108,105,116,32,105,114,120,32,115,116,114,32,46,32,111,41,0,0,0,0,0};
static C_char C_TLS li363[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,105,41};
static C_char C_TLS li364[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,116,114,105,110,103,45,115,99,97,110,45,99,104,97,114,32,115,116,114,32,99,32,46,32,111,41,0,0,0,0};
static C_char C_TLS li365[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,108,115,32,114,101,115,41,0,0,0};
static C_char C_TLS li366[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,113,117,111,116,101,32,115,116,114,41,0,0,0,0,0};
static C_char C_TLS li367[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,53,57,53,55,32,103,53,57,54,57,41,0,0,0,0};
static C_char C_TLS li368[] C_aligned={C_lihdr(0,0,10),40,97,50,57,48,51,54,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li369[] C_aligned={C_lihdr(0,0,11),40,108,112,50,32,105,32,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li370[] C_aligned={C_lihdr(0,0,15),40,108,112,49,32,108,115,32,101,109,112,116,121,63,41,0};
static C_char C_TLS li371[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,105,114,114,101,103,101,120,45,111,112,116,32,108,115,41};
static C_char C_TLS li372[] C_aligned={C_lihdr(0,0,7),40,108,112,32,108,115,41,0};
static C_char C_TLS li373[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,99,115,101,116,45,62,115,116,114,105,110,103,32,108,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li374[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,48,53,55,32,103,54,48,54,52,41,0,0,0,0,0,0,0};
static C_char C_TLS li375[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,48,55,54,32,103,54,48,56,51,41,0,0,0,0,0,0,0};
static C_char C_TLS li376[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,48,57,51,32,103,54,49,48,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li377[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,49,49,50,32,103,54,49,49,57,41,0,0,0,0,0,0,0};
static C_char C_TLS li378[] C_aligned={C_lihdr(0,0,9),40,103,54,49,51,52,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li379[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,49,51,51,32,103,54,49,52,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li380[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,49,55,49,32,103,54,49,55,56,41,0,0,0,0,0,0,0};
static C_char C_TLS li381[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,50,50,54,32,103,54,50,51,51,41,0,0,0,0,0,0,0};
static C_char C_TLS li382[] C_aligned={C_lihdr(0,0,6),40,108,112,32,120,41,0,0};
static C_char C_TLS li383[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,115,114,101,45,62,115,116,114,105,110,103,32,111,98,106,41,0,0,0,0,0,0,0};
static C_char C_TLS li384[] C_aligned={C_lihdr(0,0,14),40,108,111,111,112,50,32,114,101,115,116,32,115,41,0,0};
static C_char C_TLS li385[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,99,115,32,100,105,114,41,0,0,0};
static C_char C_TLS li386[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,105,114,114,101,103,101,120,35,103,108,111,98,45,62,115,114,101,32,115,41,0,0,0};
static C_char C_TLS li387[] C_aligned={C_lihdr(0,0,10),40,97,51,48,57,51,53,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li388[] C_aligned={C_lihdr(0,0,22),40,97,51,48,57,51,56,32,115,114,99,49,32,105,32,115,114,99,50,32,106,41,0,0};
static C_char C_TLS li389[] C_aligned={C_lihdr(0,0,14),40,97,51,48,57,55,50,32,109,32,111,117,116,41,0,0};
static C_char C_TLS li390[] C_aligned={C_lihdr(0,0,10),40,116,111,112,108,101,118,101,108,41,0,0,0,0,0,0};


C_noret_decl(f33619)
static void C_ccall f33619(C_word c,C_word *av) C_noret;
C_noret_decl(f33623)
static void C_ccall f33623(C_word c,C_word *av) C_noret;
C_noret_decl(f_10021)
static void C_fcall f_10021(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10024)
static void C_fcall f_10024(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10038)
static void C_ccall f_10038(C_word c,C_word *av) C_noret;
C_noret_decl(f_10063)
static void C_ccall f_10063(C_word c,C_word *av) C_noret;
C_noret_decl(f_10070)
static void C_ccall f_10070(C_word c,C_word *av) C_noret;
C_noret_decl(f_10089)
static void C_ccall f_10089(C_word c,C_word *av) C_noret;
C_noret_decl(f_10122)
static void C_ccall f_10122(C_word c,C_word *av) C_noret;
C_noret_decl(f_10140)
static void C_ccall f_10140(C_word c,C_word *av) C_noret;
C_noret_decl(f_10160)
static void C_ccall f_10160(C_word c,C_word *av) C_noret;
C_noret_decl(f_10240)
static void C_fcall f_10240(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10250)
static void C_fcall f_10250(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10253)
static void C_ccall f_10253(C_word c,C_word *av) C_noret;
C_noret_decl(f_10260)
static void C_ccall f_10260(C_word c,C_word *av) C_noret;
C_noret_decl(f_10263)
static void C_ccall f_10263(C_word c,C_word *av) C_noret;
C_noret_decl(f_10311)
static void C_ccall f_10311(C_word c,C_word *av) C_noret;
C_noret_decl(f_10332)
static void C_fcall f_10332(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10338)
static void C_fcall f_10338(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10356)
static void C_fcall f_10356(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10367)
static void C_fcall f_10367(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10455)
static void C_ccall f_10455(C_word c,C_word *av) C_noret;
C_noret_decl(f_10467)
static void C_ccall f_10467(C_word c,C_word *av) C_noret;
C_noret_decl(f_10471)
static void C_ccall f_10471(C_word c,C_word *av) C_noret;
C_noret_decl(f_10477)
static void C_ccall f_10477(C_word c,C_word *av) C_noret;
C_noret_decl(f_10482)
static void C_fcall f_10482(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_10485)
static C_word C_fcall f_10485(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_10507)
static void C_fcall f_10507(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10524)
static void C_ccall f_10524(C_word c,C_word *av) C_noret;
C_noret_decl(f_10527)
static void C_ccall f_10527(C_word c,C_word *av) C_noret;
C_noret_decl(f_10529)
static void C_fcall f_10529(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10563)
static void C_fcall f_10563(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10577)
static void C_ccall f_10577(C_word c,C_word *av) C_noret;
C_noret_decl(f_10581)
static void C_ccall f_10581(C_word c,C_word *av) C_noret;
C_noret_decl(f_10583)
static void C_fcall f_10583(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10590)
static void C_ccall f_10590(C_word c,C_word *av) C_noret;
C_noret_decl(f_10599)
static void C_fcall f_10599(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10619)
static void C_ccall f_10619(C_word c,C_word *av) C_noret;
C_noret_decl(f_10623)
static void C_ccall f_10623(C_word c,C_word *av) C_noret;
C_noret_decl(f_10627)
static void C_ccall f_10627(C_word c,C_word *av) C_noret;
C_noret_decl(f_10655)
static void C_fcall f_10655(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10659)
static void C_ccall f_10659(C_word c,C_word *av) C_noret;
C_noret_decl(f_10662)
static void C_fcall f_10662(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10665)
static void C_fcall f_10665(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10668)
static void C_ccall f_10668(C_word c,C_word *av) C_noret;
C_noret_decl(f_10673)
static void C_fcall f_10673(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10676)
static C_word C_fcall f_10676(C_word *a,C_word t0);
C_noret_decl(f_10746)
static void C_ccall f_10746(C_word c,C_word *av) C_noret;
C_noret_decl(f_10762)
static void C_ccall f_10762(C_word c,C_word *av) C_noret;
C_noret_decl(f_10838)
static void C_ccall f_10838(C_word c,C_word *av) C_noret;
C_noret_decl(f_10849)
static void C_ccall f_10849(C_word c,C_word *av) C_noret;
C_noret_decl(f_10863)
static void C_ccall f_10863(C_word c,C_word *av) C_noret;
C_noret_decl(f_10885)
static void C_ccall f_10885(C_word c,C_word *av) C_noret;
C_noret_decl(f_10888)
static void C_fcall f_10888(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10900)
static void C_ccall f_10900(C_word c,C_word *av) C_noret;
C_noret_decl(f_10960)
static void C_ccall f_10960(C_word c,C_word *av) C_noret;
C_noret_decl(f_10989)
static void C_fcall f_10989(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11120)
static void C_ccall f_11120(C_word c,C_word *av) C_noret;
C_noret_decl(f_11126)
static void C_ccall f_11126(C_word c,C_word *av) C_noret;
C_noret_decl(f_11141)
static void C_ccall f_11141(C_word c,C_word *av) C_noret;
C_noret_decl(f_11212)
static void C_ccall f_11212(C_word c,C_word *av) C_noret;
C_noret_decl(f_11242)
static void C_ccall f_11242(C_word c,C_word *av) C_noret;
C_noret_decl(f_11252)
static void C_fcall f_11252(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11274)
static void C_ccall f_11274(C_word c,C_word *av) C_noret;
C_noret_decl(f_11309)
static void C_ccall f_11309(C_word c,C_word *av) C_noret;
C_noret_decl(f_11324)
static void C_ccall f_11324(C_word c,C_word *av) C_noret;
C_noret_decl(f_11350)
static void C_ccall f_11350(C_word c,C_word *av) C_noret;
C_noret_decl(f_11373)
static void C_ccall f_11373(C_word c,C_word *av) C_noret;
C_noret_decl(f_11396)
static void C_ccall f_11396(C_word c,C_word *av) C_noret;
C_noret_decl(f_11438)
static void C_ccall f_11438(C_word c,C_word *av) C_noret;
C_noret_decl(f_11461)
static void C_ccall f_11461(C_word c,C_word *av) C_noret;
C_noret_decl(f_11464)
static void C_ccall f_11464(C_word c,C_word *av) C_noret;
C_noret_decl(f_11490)
static void C_ccall f_11490(C_word c,C_word *av) C_noret;
C_noret_decl(f_11494)
static void C_ccall f_11494(C_word c,C_word *av) C_noret;
C_noret_decl(f_11498)
static void C_ccall f_11498(C_word c,C_word *av) C_noret;
C_noret_decl(f_11550)
static void C_ccall f_11550(C_word c,C_word *av) C_noret;
C_noret_decl(f_11575)
static void C_ccall f_11575(C_word c,C_word *av) C_noret;
C_noret_decl(f_11578)
static void C_ccall f_11578(C_word c,C_word *av) C_noret;
C_noret_decl(f_11608)
static void C_ccall f_11608(C_word c,C_word *av) C_noret;
C_noret_decl(f_11612)
static void C_ccall f_11612(C_word c,C_word *av) C_noret;
C_noret_decl(f_11627)
static void C_ccall f_11627(C_word c,C_word *av) C_noret;
C_noret_decl(f_11630)
static void C_ccall f_11630(C_word c,C_word *av) C_noret;
C_noret_decl(f_11651)
static void C_ccall f_11651(C_word c,C_word *av) C_noret;
C_noret_decl(f_11655)
static void C_ccall f_11655(C_word c,C_word *av) C_noret;
C_noret_decl(f_11678)
static void C_ccall f_11678(C_word c,C_word *av) C_noret;
C_noret_decl(f_11709)
static void C_fcall f_11709(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_11712)
static C_word C_fcall f_11712(C_word t0,C_word t1);
C_noret_decl(f_11721)
static C_word C_fcall f_11721(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_11865)
static void C_ccall f_11865(C_word c,C_word *av) C_noret;
C_noret_decl(f_11896)
static void C_ccall f_11896(C_word c,C_word *av) C_noret;
C_noret_decl(f_11987)
static void C_ccall f_11987(C_word c,C_word *av) C_noret;
C_noret_decl(f_12002)
static void C_ccall f_12002(C_word c,C_word *av) C_noret;
C_noret_decl(f_12022)
static void C_ccall f_12022(C_word c,C_word *av) C_noret;
C_noret_decl(f_12026)
static void C_ccall f_12026(C_word c,C_word *av) C_noret;
C_noret_decl(f_12045)
static void C_fcall f_12045(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12054)
static void C_ccall f_12054(C_word c,C_word *av) C_noret;
C_noret_decl(f_12070)
static void C_ccall f_12070(C_word c,C_word *av) C_noret;
C_noret_decl(f_12073)
static void C_ccall f_12073(C_word c,C_word *av) C_noret;
C_noret_decl(f_12076)
static void C_ccall f_12076(C_word c,C_word *av) C_noret;
C_noret_decl(f_12079)
static void C_ccall f_12079(C_word c,C_word *av) C_noret;
C_noret_decl(f_12088)
static void C_fcall f_12088(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12198)
static void C_ccall f_12198(C_word c,C_word *av) C_noret;
C_noret_decl(f_12274)
static void C_ccall f_12274(C_word c,C_word *av) C_noret;
C_noret_decl(f_12301)
static void C_ccall f_12301(C_word c,C_word *av) C_noret;
C_noret_decl(f_12324)
static void C_ccall f_12324(C_word c,C_word *av) C_noret;
C_noret_decl(f_12351)
static void C_ccall f_12351(C_word c,C_word *av) C_noret;
C_noret_decl(f_12378)
static void C_ccall f_12378(C_word c,C_word *av) C_noret;
C_noret_decl(f_12409)
static void C_ccall f_12409(C_word c,C_word *av) C_noret;
C_noret_decl(f_12444)
static void C_ccall f_12444(C_word c,C_word *av) C_noret;
C_noret_decl(f_12467)
static void C_ccall f_12467(C_word c,C_word *av) C_noret;
C_noret_decl(f_12490)
static void C_ccall f_12490(C_word c,C_word *av) C_noret;
C_noret_decl(f_12521)
static void C_ccall f_12521(C_word c,C_word *av) C_noret;
C_noret_decl(f_12544)
static void C_ccall f_12544(C_word c,C_word *av) C_noret;
C_noret_decl(f_12567)
static void C_ccall f_12567(C_word c,C_word *av) C_noret;
C_noret_decl(f_12590)
static void C_ccall f_12590(C_word c,C_word *av) C_noret;
C_noret_decl(f_12613)
static void C_ccall f_12613(C_word c,C_word *av) C_noret;
C_noret_decl(f_12636)
static void C_ccall f_12636(C_word c,C_word *av) C_noret;
C_noret_decl(f_12647)
static void C_ccall f_12647(C_word c,C_word *av) C_noret;
C_noret_decl(f_12667)
static void C_ccall f_12667(C_word c,C_word *av) C_noret;
C_noret_decl(f_12671)
static void C_ccall f_12671(C_word c,C_word *av) C_noret;
C_noret_decl(f_12695)
static void C_fcall f_12695(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12698)
static void C_ccall f_12698(C_word c,C_word *av) C_noret;
C_noret_decl(f_12701)
static void C_ccall f_12701(C_word c,C_word *av) C_noret;
C_noret_decl(f_12736)
static void C_ccall f_12736(C_word c,C_word *av) C_noret;
C_noret_decl(f_12740)
static void C_ccall f_12740(C_word c,C_word *av) C_noret;
C_noret_decl(f_12767)
static void C_ccall f_12767(C_word c,C_word *av) C_noret;
C_noret_decl(f_12775)
static void C_fcall f_12775(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12835)
static void C_ccall f_12835(C_word c,C_word *av) C_noret;
C_noret_decl(f_12878)
static void C_ccall f_12878(C_word c,C_word *av) C_noret;
C_noret_decl(f_12890)
static void C_ccall f_12890(C_word c,C_word *av) C_noret;
C_noret_decl(f_12899)
static void C_ccall f_12899(C_word c,C_word *av) C_noret;
C_noret_decl(f_12919)
static void C_ccall f_12919(C_word c,C_word *av) C_noret;
C_noret_decl(f_12923)
static void C_ccall f_12923(C_word c,C_word *av) C_noret;
C_noret_decl(f_12927)
static void C_ccall f_12927(C_word c,C_word *av) C_noret;
C_noret_decl(f_12979)
static void C_ccall f_12979(C_word c,C_word *av) C_noret;
C_noret_decl(f_12995)
static void C_ccall f_12995(C_word c,C_word *av) C_noret;
C_noret_decl(f_13059)
static void C_ccall f_13059(C_word c,C_word *av) C_noret;
C_noret_decl(f_13085)
static void C_ccall f_13085(C_word c,C_word *av) C_noret;
C_noret_decl(f_13114)
static void C_ccall f_13114(C_word c,C_word *av) C_noret;
C_noret_decl(f_13142)
static void C_ccall f_13142(C_word c,C_word *av) C_noret;
C_noret_decl(f_13163)
static void C_ccall f_13163(C_word c,C_word *av) C_noret;
C_noret_decl(f_13181)
static void C_ccall f_13181(C_word c,C_word *av) C_noret;
C_noret_decl(f_13295)
static void C_fcall f_13295(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13314)
static void C_ccall f_13314(C_word c,C_word *av) C_noret;
C_noret_decl(f_13326)
static void C_ccall f_13326(C_word c,C_word *av) C_noret;
C_noret_decl(f_13329)
static void C_ccall f_13329(C_word c,C_word *av) C_noret;
C_noret_decl(f_13360)
static void C_ccall f_13360(C_word c,C_word *av) C_noret;
C_noret_decl(f_13363)
static void C_ccall f_13363(C_word c,C_word *av) C_noret;
C_noret_decl(f_13404)
static void C_fcall f_13404(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13439)
static void C_ccall f_13439(C_word c,C_word *av) C_noret;
C_noret_decl(f_13449)
static void C_ccall f_13449(C_word c,C_word *av) C_noret;
C_noret_decl(f_13456)
static void C_ccall f_13456(C_word c,C_word *av) C_noret;
C_noret_decl(f_13477)
static void C_fcall f_13477(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13487)
static void C_ccall f_13487(C_word c,C_word *av) C_noret;
C_noret_decl(f_13504)
static void C_ccall f_13504(C_word c,C_word *av) C_noret;
C_noret_decl(f_13521)
static void C_ccall f_13521(C_word c,C_word *av) C_noret;
C_noret_decl(f_13532)
static void C_ccall f_13532(C_word c,C_word *av) C_noret;
C_noret_decl(f_13533)
static C_word C_fcall f_13533(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_13554)
static void C_fcall f_13554(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13566)
static void C_fcall f_13566(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13576)
static void C_ccall f_13576(C_word c,C_word *av) C_noret;
C_noret_decl(f_13679)
static void C_ccall f_13679(C_word c,C_word *av) C_noret;
C_noret_decl(f_13688)
static void C_fcall f_13688(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13694)
static void C_ccall f_13694(C_word c,C_word *av) C_noret;
C_noret_decl(f_13697)
static void C_ccall f_13697(C_word c,C_word *av) C_noret;
C_noret_decl(f_13707)
static void C_ccall f_13707(C_word c,C_word *av) C_noret;
C_noret_decl(f_13714)
static void C_ccall f_13714(C_word c,C_word *av) C_noret;
C_noret_decl(f_13718)
static void C_ccall f_13718(C_word c,C_word *av) C_noret;
C_noret_decl(f_13762)
static void C_ccall f_13762(C_word c,C_word *av) C_noret;
C_noret_decl(f_13801)
static void C_fcall f_13801(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13811)
static void C_ccall f_13811(C_word c,C_word *av) C_noret;
C_noret_decl(f_13815)
static void C_ccall f_13815(C_word c,C_word *av) C_noret;
C_noret_decl(f_13819)
static void C_ccall f_13819(C_word c,C_word *av) C_noret;
C_noret_decl(f_13834)
static void C_ccall f_13834(C_word c,C_word *av) C_noret;
C_noret_decl(f_13842)
static void C_ccall f_13842(C_word c,C_word *av) C_noret;
C_noret_decl(f_13846)
static void C_ccall f_13846(C_word c,C_word *av) C_noret;
C_noret_decl(f_13865)
static void C_ccall f_13865(C_word c,C_word *av) C_noret;
C_noret_decl(f_13918)
static void C_fcall f_13918(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13932)
static void C_ccall f_13932(C_word c,C_word *av) C_noret;
C_noret_decl(f_13936)
static void C_ccall f_13936(C_word c,C_word *av) C_noret;
C_noret_decl(f_13940)
static void C_ccall f_13940(C_word c,C_word *av) C_noret;
C_noret_decl(f_13950)
static void C_ccall f_13950(C_word c,C_word *av) C_noret;
C_noret_decl(f_14008)
static void C_ccall f_14008(C_word c,C_word *av) C_noret;
C_noret_decl(f_14029)
static void C_fcall f_14029(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14032)
static C_word C_fcall f_14032(C_word t0,C_word t1);
C_noret_decl(f_14224)
static void C_fcall f_14224(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14262)
static void C_fcall f_14262(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14338)
static void C_fcall f_14338(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14479)
static void C_ccall f_14479(C_word c,C_word *av) C_noret;
C_noret_decl(f_14482)
static void C_ccall f_14482(C_word c,C_word *av) C_noret;
C_noret_decl(f_14503)
static void C_ccall f_14503(C_word c,C_word *av) C_noret;
C_noret_decl(f_14508)
static void C_fcall f_14508(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14540)
static void C_ccall f_14540(C_word c,C_word *av) C_noret;
C_noret_decl(f_14565)
static void C_ccall f_14565(C_word c,C_word *av) C_noret;
C_noret_decl(f_14569)
static void C_ccall f_14569(C_word c,C_word *av) C_noret;
C_noret_decl(f_14580)
static void C_ccall f_14580(C_word c,C_word *av) C_noret;
C_noret_decl(f_14588)
static void C_ccall f_14588(C_word c,C_word *av) C_noret;
C_noret_decl(f_14614)
static void C_fcall f_14614(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_14644)
static void C_ccall f_14644(C_word c,C_word *av) C_noret;
C_noret_decl(f_14654)
static void C_ccall f_14654(C_word c,C_word *av) C_noret;
C_noret_decl(f_14662)
static void C_ccall f_14662(C_word c,C_word *av) C_noret;
C_noret_decl(f_14678)
static void C_ccall f_14678(C_word c,C_word *av) C_noret;
C_noret_decl(f_14680)
static void C_fcall f_14680(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14716)
static void C_fcall f_14716(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14752)
static void C_fcall f_14752(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14772)
static void C_ccall f_14772(C_word c,C_word *av) C_noret;
C_noret_decl(f_14774)
static void C_ccall f_14774(C_word c,C_word *av) C_noret;
C_noret_decl(f_14806)
static void C_ccall f_14806(C_word c,C_word *av) C_noret;
C_noret_decl(f_14813)
static void C_ccall f_14813(C_word c,C_word *av) C_noret;
C_noret_decl(f_14819)
static void C_ccall f_14819(C_word c,C_word *av) C_noret;
C_noret_decl(f_14835)
static void C_ccall f_14835(C_word c,C_word *av) C_noret;
C_noret_decl(f_14837)
static void C_fcall f_14837(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14877)
static void C_ccall f_14877(C_word c,C_word *av) C_noret;
C_noret_decl(f_14883)
static void C_ccall f_14883(C_word c,C_word *av) C_noret;
C_noret_decl(f_14891)
static void C_fcall f_14891(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14911)
static void C_ccall f_14911(C_word c,C_word *av) C_noret;
C_noret_decl(f_14913)
static void C_ccall f_14913(C_word c,C_word *av) C_noret;
C_noret_decl(f_14945)
static void C_ccall f_14945(C_word c,C_word *av) C_noret;
C_noret_decl(f_14952)
static void C_ccall f_14952(C_word c,C_word *av) C_noret;
C_noret_decl(f_14958)
static void C_ccall f_14958(C_word c,C_word *av) C_noret;
C_noret_decl(f_14974)
static void C_ccall f_14974(C_word c,C_word *av) C_noret;
C_noret_decl(f_14976)
static void C_fcall f_14976(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15016)
static void C_ccall f_15016(C_word c,C_word *av) C_noret;
C_noret_decl(f_15022)
static void C_ccall f_15022(C_word c,C_word *av) C_noret;
C_noret_decl(f_15041)
static void C_ccall f_15041(C_word c,C_word *av) C_noret;
C_noret_decl(f_15050)
static void C_fcall f_15050(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15062)
static void C_fcall f_15062(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15074)
static void C_ccall f_15074(C_word c,C_word *av) C_noret;
C_noret_decl(f_15080)
static void C_ccall f_15080(C_word c,C_word *av) C_noret;
C_noret_decl(f_15082)
static void C_fcall f_15082(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15130)
static void C_ccall f_15130(C_word c,C_word *av) C_noret;
C_noret_decl(f_15138)
static void C_ccall f_15138(C_word c,C_word *av) C_noret;
C_noret_decl(f_15145)
static void C_ccall f_15145(C_word c,C_word *av) C_noret;
C_noret_decl(f_15151)
static void C_ccall f_15151(C_word c,C_word *av) C_noret;
C_noret_decl(f_15183)
static void C_ccall f_15183(C_word c,C_word *av) C_noret;
C_noret_decl(f_15185)
static void C_fcall f_15185(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15235)
static void C_ccall f_15235(C_word c,C_word *av) C_noret;
C_noret_decl(f_15253)
static void C_fcall f_15253(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15278)
static void C_ccall f_15278(C_word c,C_word *av) C_noret;
C_noret_decl(f_15321)
static void C_ccall f_15321(C_word c,C_word *av) C_noret;
C_noret_decl(f_15323)
static void C_fcall f_15323(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15412)
static void C_ccall f_15412(C_word c,C_word *av) C_noret;
C_noret_decl(f_15414)
static void C_fcall f_15414(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15469)
static void C_fcall f_15469(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15477)
static void C_ccall f_15477(C_word c,C_word *av) C_noret;
C_noret_decl(f_15479)
static void C_fcall f_15479(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15493)
static void C_ccall f_15493(C_word c,C_word *av) C_noret;
C_noret_decl(f_15497)
static void C_ccall f_15497(C_word c,C_word *av) C_noret;
C_noret_decl(f_15515)
static void C_ccall f_15515(C_word c,C_word *av) C_noret;
C_noret_decl(f_15524)
static void C_fcall f_15524(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15539)
static void C_ccall f_15539(C_word c,C_word *av) C_noret;
C_noret_decl(f_15591)
static void C_fcall f_15591(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15594)
static void C_fcall f_15594(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15651)
static void C_fcall f_15651(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15662)
static void C_ccall f_15662(C_word c,C_word *av) C_noret;
C_noret_decl(f_15664)
static void C_fcall f_15664(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15689)
static void C_ccall f_15689(C_word c,C_word *av) C_noret;
C_noret_decl(f_15712)
static void C_fcall f_15712(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15723)
static void C_ccall f_15723(C_word c,C_word *av) C_noret;
C_noret_decl(f_15725)
static void C_fcall f_15725(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15750)
static void C_ccall f_15750(C_word c,C_word *av) C_noret;
C_noret_decl(f_15766)
static void C_fcall f_15766(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15775)
static void C_ccall f_15775(C_word c,C_word *av) C_noret;
C_noret_decl(f_15781)
static void C_ccall f_15781(C_word c,C_word *av) C_noret;
C_noret_decl(f_15791)
static void C_ccall f_15791(C_word c,C_word *av) C_noret;
C_noret_decl(f_15798)
static void C_ccall f_15798(C_word c,C_word *av) C_noret;
C_noret_decl(f_15833)
static void C_ccall f_15833(C_word c,C_word *av) C_noret;
C_noret_decl(f_15835)
static void C_fcall f_15835(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15860)
static void C_ccall f_15860(C_word c,C_word *av) C_noret;
C_noret_decl(f_15889)
static void C_ccall f_15889(C_word c,C_word *av) C_noret;
C_noret_decl(f_15891)
static void C_fcall f_15891(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15916)
static void C_ccall f_15916(C_word c,C_word *av) C_noret;
C_noret_decl(f_15974)
static void C_fcall f_15974(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15984)
static void C_ccall f_15984(C_word c,C_word *av) C_noret;
C_noret_decl(f_15987)
static void C_ccall f_15987(C_word c,C_word *av) C_noret;
C_noret_decl(f_15989)
static void C_fcall f_15989(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16036)
static void C_ccall f_16036(C_word c,C_word *av) C_noret;
C_noret_decl(f_16043)
static void C_ccall f_16043(C_word c,C_word *av) C_noret;
C_noret_decl(f_16097)
static void C_ccall f_16097(C_word c,C_word *av) C_noret;
C_noret_decl(f_16164)
static void C_ccall f_16164(C_word c,C_word *av) C_noret;
C_noret_decl(f_16172)
static void C_ccall f_16172(C_word c,C_word *av) C_noret;
C_noret_decl(f_16174)
static void C_ccall f_16174(C_word c,C_word *av) C_noret;
C_noret_decl(f_16178)
static void C_ccall f_16178(C_word c,C_word *av) C_noret;
C_noret_decl(f_16181)
static void C_ccall f_16181(C_word c,C_word *av) C_noret;
C_noret_decl(f_16184)
static void C_ccall f_16184(C_word c,C_word *av) C_noret;
C_noret_decl(f_16187)
static void C_ccall f_16187(C_word c,C_word *av) C_noret;
C_noret_decl(f_16193)
static void C_ccall f_16193(C_word c,C_word *av) C_noret;
C_noret_decl(f_16196)
static void C_ccall f_16196(C_word c,C_word *av) C_noret;
C_noret_decl(f_16199)
static void C_ccall f_16199(C_word c,C_word *av) C_noret;
C_noret_decl(f_16202)
static void C_ccall f_16202(C_word c,C_word *av) C_noret;
C_noret_decl(f_16205)
static void C_ccall f_16205(C_word c,C_word *av) C_noret;
C_noret_decl(f_16220)
static void C_ccall f_16220(C_word c,C_word *av) C_noret;
C_noret_decl(f_16234)
static void C_ccall f_16234(C_word c,C_word *av) C_noret;
C_noret_decl(f_16239)
static void C_fcall f_16239(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16265)
static void C_ccall f_16265(C_word c,C_word *av) C_noret;
C_noret_decl(f_16269)
static void C_fcall f_16269(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16307)
static void C_ccall f_16307(C_word c,C_word *av) C_noret;
C_noret_decl(f_16324)
static void C_fcall f_16324(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16367)
static void C_fcall f_16367(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16474)
static void C_ccall f_16474(C_word c,C_word *av) C_noret;
C_noret_decl(f_16497)
static void C_fcall f_16497(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16575)
static C_word C_fcall f_16575(C_word t0);
C_noret_decl(f_16620)
static void C_ccall f_16620(C_word c,C_word *av) C_noret;
C_noret_decl(f_16637)
static void C_fcall f_16637(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16709)
static void C_ccall f_16709(C_word c,C_word *av) C_noret;
C_noret_decl(f_16713)
static void C_ccall f_16713(C_word c,C_word *av) C_noret;
C_noret_decl(f_16750)
static void C_fcall f_16750(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16826)
static void C_ccall f_16826(C_word c,C_word *av) C_noret;
C_noret_decl(f_16830)
static void C_ccall f_16830(C_word c,C_word *av) C_noret;
C_noret_decl(f_16867)
static void C_fcall f_16867(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16880)
static void C_ccall f_16880(C_word c,C_word *av) C_noret;
C_noret_decl(f_16943)
static void C_ccall f_16943(C_word c,C_word *av) C_noret;
C_noret_decl(f_16969)
static void C_ccall f_16969(C_word c,C_word *av) C_noret;
C_noret_decl(f_16982)
static void C_fcall f_16982(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16988)
static void C_ccall f_16988(C_word c,C_word *av) C_noret;
C_noret_decl(f_17009)
static void C_ccall f_17009(C_word c,C_word *av) C_noret;
C_noret_decl(f_17015)
static void C_fcall f_17015(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17045)
static void C_ccall f_17045(C_word c,C_word *av) C_noret;
C_noret_decl(f_17082)
static void C_ccall f_17082(C_word c,C_word *av) C_noret;
C_noret_decl(f_17085)
static void C_ccall f_17085(C_word c,C_word *av) C_noret;
C_noret_decl(f_17092)
static void C_ccall f_17092(C_word c,C_word *av) C_noret;
C_noret_decl(f_17094)
static void C_fcall f_17094(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_17097)
static void C_fcall f_17097(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17137)
static void C_fcall f_17137(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17153)
static void C_ccall f_17153(C_word c,C_word *av) C_noret;
C_noret_decl(f_17166)
static void C_fcall f_17166(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17173)
static void C_fcall f_17173(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_17206)
static void C_ccall f_17206(C_word c,C_word *av) C_noret;
C_noret_decl(f_17236)
static void C_ccall f_17236(C_word c,C_word *av) C_noret;
C_noret_decl(f_17251)
static void C_fcall f_17251(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_17288)
static void C_ccall f_17288(C_word c,C_word *av) C_noret;
C_noret_decl(f_17304)
static void C_ccall f_17304(C_word c,C_word *av) C_noret;
C_noret_decl(f_17308)
static void C_ccall f_17308(C_word c,C_word *av) C_noret;
C_noret_decl(f_17321)
static void C_ccall f_17321(C_word c,C_word *av) C_noret;
C_noret_decl(f_17344)
static void C_ccall f_17344(C_word c,C_word *av) C_noret;
C_noret_decl(f_17347)
static void C_ccall f_17347(C_word c,C_word *av) C_noret;
C_noret_decl(f_17354)
static void C_fcall f_17354(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17356)
static void C_ccall f_17356(C_word c,C_word *av) C_noret;
C_noret_decl(f_17370)
static void C_ccall f_17370(C_word c,C_word *av) C_noret;
C_noret_decl(f_17384)
static void C_ccall f_17384(C_word c,C_word *av) C_noret;
C_noret_decl(f_17409)
static void C_ccall f_17409(C_word c,C_word *av) C_noret;
C_noret_decl(f_17413)
static void C_ccall f_17413(C_word c,C_word *av) C_noret;
C_noret_decl(f_17490)
static void C_fcall f_17490(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17503)
static void C_ccall f_17503(C_word c,C_word *av) C_noret;
C_noret_decl(f_17539)
static void C_ccall f_17539(C_word c,C_word *av) C_noret;
C_noret_decl(f_17548)
static void C_fcall f_17548(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17648)
static void C_ccall f_17648(C_word c,C_word *av) C_noret;
C_noret_decl(f_17657)
static void C_ccall f_17657(C_word c,C_word *av) C_noret;
C_noret_decl(f_17675)
static void C_fcall f_17675(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17678)
static void C_fcall f_17678(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17696)
static void C_ccall f_17696(C_word c,C_word *av) C_noret;
C_noret_decl(f_17743)
static void C_ccall f_17743(C_word c,C_word *av) C_noret;
C_noret_decl(f_17817)
static void C_ccall f_17817(C_word c,C_word *av) C_noret;
C_noret_decl(f_17845)
static void C_ccall f_17845(C_word c,C_word *av) C_noret;
C_noret_decl(f_17870)
static void C_fcall f_17870(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17889)
static void C_fcall f_17889(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17910)
static void C_fcall f_17910(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17920)
static void C_fcall f_17920(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17928)
static void C_ccall f_17928(C_word c,C_word *av) C_noret;
C_noret_decl(f_18159)
static void C_ccall f_18159(C_word c,C_word *av) C_noret;
C_noret_decl(f_18182)
static void C_ccall f_18182(C_word c,C_word *av) C_noret;
C_noret_decl(f_18191)
static C_word C_fcall f_18191(C_word *a,C_word t0);
C_noret_decl(f_18214)
static C_word C_fcall f_18214(C_word *a,C_word t0);
C_noret_decl(f_18237)
static void C_ccall f_18237(C_word c,C_word *av) C_noret;
C_noret_decl(f_18310)
static void C_fcall f_18310(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18335)
static void C_ccall f_18335(C_word c,C_word *av) C_noret;
C_noret_decl(f_18370)
static void C_fcall f_18370(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18393)
static void C_ccall f_18393(C_word c,C_word *av) C_noret;
C_noret_decl(f_18397)
static void C_ccall f_18397(C_word c,C_word *av) C_noret;
C_noret_decl(f_18418)
static void C_fcall f_18418(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18520)
static void C_fcall f_18520(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18704)
static void C_fcall f_18704(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18726)
static void C_ccall f_18726(C_word c,C_word *av) C_noret;
C_noret_decl(f_18732)
static void C_ccall f_18732(C_word c,C_word *av) C_noret;
C_noret_decl(f_18736)
static void C_ccall f_18736(C_word c,C_word *av) C_noret;
C_noret_decl(f_18753)
static void C_fcall f_18753(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18786)
static void C_ccall f_18786(C_word c,C_word *av) C_noret;
C_noret_decl(f_18823)
static void C_ccall f_18823(C_word c,C_word *av) C_noret;
C_noret_decl(f_18825)
static void C_fcall f_18825(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18850)
static void C_ccall f_18850(C_word c,C_word *av) C_noret;
C_noret_decl(f_18918)
static void C_ccall f_18918(C_word c,C_word *av) C_noret;
C_noret_decl(f_18920)
static void C_ccall f_18920(C_word c,C_word *av) C_noret;
C_noret_decl(f_18924)
static void C_ccall f_18924(C_word c,C_word *av) C_noret;
C_noret_decl(f_18933)
static void C_fcall f_18933(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18939)
static void C_ccall f_18939(C_word c,C_word *av) C_noret;
C_noret_decl(f_18942)
static void C_ccall f_18942(C_word c,C_word *av) C_noret;
C_noret_decl(f_19010)
static void C_ccall f_19010(C_word c,C_word *av) C_noret;
C_noret_decl(f_19014)
static void C_ccall f_19014(C_word c,C_word *av) C_noret;
C_noret_decl(f_19017)
static void C_ccall f_19017(C_word c,C_word *av) C_noret;
C_noret_decl(f_19020)
static void C_ccall f_19020(C_word c,C_word *av) C_noret;
C_noret_decl(f_19023)
static void C_ccall f_19023(C_word c,C_word *av) C_noret;
C_noret_decl(f_19053)
static void C_fcall f_19053(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_19072)
static void C_ccall f_19072(C_word c,C_word *av) C_noret;
C_noret_decl(f_19097)
static void C_ccall f_19097(C_word c,C_word *av) C_noret;
C_noret_decl(f_19114)
static void C_fcall f_19114(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19118)
static void C_ccall f_19118(C_word c,C_word *av) C_noret;
C_noret_decl(f_19123)
static void C_fcall f_19123(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19130)
static void C_ccall f_19130(C_word c,C_word *av) C_noret;
C_noret_decl(f_19153)
static void C_ccall f_19153(C_word c,C_word *av) C_noret;
C_noret_decl(f_19163)
static void C_ccall f_19163(C_word c,C_word *av) C_noret;
C_noret_decl(f_19184)
static void C_ccall f_19184(C_word c,C_word *av) C_noret;
C_noret_decl(f_19202)
static void C_ccall f_19202(C_word c,C_word *av) C_noret;
C_noret_decl(f_19208)
static void C_ccall f_19208(C_word c,C_word *av) C_noret;
C_noret_decl(f_19222)
static void C_ccall f_19222(C_word c,C_word *av) C_noret;
C_noret_decl(f_19228)
static void C_fcall f_19228(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_19235)
static void C_ccall f_19235(C_word c,C_word *av) C_noret;
C_noret_decl(f_19265)
static void C_ccall f_19265(C_word c,C_word *av) C_noret;
C_noret_decl(f_19278)
static void C_ccall f_19278(C_word c,C_word *av) C_noret;
C_noret_decl(f_19285)
static void C_ccall f_19285(C_word c,C_word *av) C_noret;
C_noret_decl(f_19292)
static void C_ccall f_19292(C_word c,C_word *av) C_noret;
C_noret_decl(f_19294)
static void C_ccall f_19294(C_word c,C_word *av) C_noret;
C_noret_decl(f_19302)
static void C_ccall f_19302(C_word c,C_word *av) C_noret;
C_noret_decl(f_19306)
static void C_ccall f_19306(C_word c,C_word *av) C_noret;
C_noret_decl(f_19315)
static void C_fcall f_19315(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19321)
static void C_ccall f_19321(C_word c,C_word *av) C_noret;
C_noret_decl(f_19324)
static void C_ccall f_19324(C_word c,C_word *av) C_noret;
C_noret_decl(f_19392)
static void C_ccall f_19392(C_word c,C_word *av) C_noret;
C_noret_decl(f_19396)
static void C_ccall f_19396(C_word c,C_word *av) C_noret;
C_noret_decl(f_19399)
static void C_ccall f_19399(C_word c,C_word *av) C_noret;
C_noret_decl(f_19414)
static void C_ccall f_19414(C_word c,C_word *av) C_noret;
C_noret_decl(f_19437)
static void C_ccall f_19437(C_word c,C_word *av) C_noret;
C_noret_decl(f_19448)
static void C_ccall f_19448(C_word c,C_word *av) C_noret;
C_noret_decl(f_19484)
static void C_ccall f_19484(C_word c,C_word *av) C_noret;
C_noret_decl(f_19494)
static void C_ccall f_19494(C_word c,C_word *av) C_noret;
C_noret_decl(f_19500)
static void C_ccall f_19500(C_word c,C_word *av) C_noret;
C_noret_decl(f_19506)
static void C_ccall f_19506(C_word c,C_word *av) C_noret;
C_noret_decl(f_19516)
static void C_ccall f_19516(C_word c,C_word *av) C_noret;
C_noret_decl(f_19518)
static void C_fcall f_19518(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19528)
static void C_fcall f_19528(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19547)
static void C_ccall f_19547(C_word c,C_word *av) C_noret;
C_noret_decl(f_19560)
static void C_ccall f_19560(C_word c,C_word *av) C_noret;
C_noret_decl(f_19592)
static void C_ccall f_19592(C_word c,C_word *av) C_noret;
C_noret_decl(f_19615)
static void C_ccall f_19615(C_word c,C_word *av) C_noret;
C_noret_decl(f_19620)
static void C_ccall f_19620(C_word c,C_word *av) C_noret;
C_noret_decl(f_19627)
static void C_ccall f_19627(C_word c,C_word *av) C_noret;
C_noret_decl(f_19678)
static void C_fcall f_19678(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_19685)
static void C_ccall f_19685(C_word c,C_word *av) C_noret;
C_noret_decl(f_19688)
static void C_ccall f_19688(C_word c,C_word *av) C_noret;
C_noret_decl(f_19693)
static void C_fcall f_19693(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19745)
static void C_ccall f_19745(C_word c,C_word *av) C_noret;
C_noret_decl(f_19760)
static void C_ccall f_19760(C_word c,C_word *av) C_noret;
C_noret_decl(f_19796)
static void C_ccall f_19796(C_word c,C_word *av) C_noret;
C_noret_decl(f_19806)
static void C_ccall f_19806(C_word c,C_word *av) C_noret;
C_noret_decl(f_19816)
static void C_fcall f_19816(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19818)
static C_word C_fcall f_19818(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_19876)
static void C_fcall f_19876(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19904)
static void C_ccall f_19904(C_word c,C_word *av) C_noret;
C_noret_decl(f_19908)
static void C_ccall f_19908(C_word c,C_word *av) C_noret;
C_noret_decl(f_19913)
static void C_fcall f_19913(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19933)
static void C_ccall f_19933(C_word c,C_word *av) C_noret;
C_noret_decl(f_19935)
static void C_fcall f_19935(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8) C_noret;
C_noret_decl(f_19960)
static void C_ccall f_19960(C_word c,C_word *av) C_noret;
C_noret_decl(f_19972)
static void C_fcall f_19972(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19975)
static void C_ccall f_19975(C_word c,C_word *av) C_noret;
C_noret_decl(f_19988)
static void C_fcall f_19988(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_19992)
static void C_ccall f_19992(C_word c,C_word *av) C_noret;
C_noret_decl(f_19995)
static void C_ccall f_19995(C_word c,C_word *av) C_noret;
C_noret_decl(f_20000)
static void C_fcall f_20000(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_20010)
static void C_fcall f_20010(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20013)
static void C_ccall f_20013(C_word c,C_word *av) C_noret;
C_noret_decl(f_20023)
static void C_ccall f_20023(C_word c,C_word *av) C_noret;
C_noret_decl(f_20058)
static void C_ccall f_20058(C_word c,C_word *av) C_noret;
C_noret_decl(f_20110)
static void C_ccall f_20110(C_word c,C_word *av) C_noret;
C_noret_decl(f_20122)
static void C_ccall f_20122(C_word c,C_word *av) C_noret;
C_noret_decl(f_20146)
static void C_ccall f_20146(C_word c,C_word *av) C_noret;
C_noret_decl(f_20147)
static C_word C_fcall f_20147(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_20176)
static void C_ccall f_20176(C_word c,C_word *av) C_noret;
C_noret_decl(f_20185)
static void C_fcall f_20185(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20199)
static void C_ccall f_20199(C_word c,C_word *av) C_noret;
C_noret_decl(f_20218)
static void C_ccall f_20218(C_word c,C_word *av) C_noret;
C_noret_decl(f_20226)
static void C_fcall f_20226(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20236)
static void C_ccall f_20236(C_word c,C_word *av) C_noret;
C_noret_decl(f_20249)
static void C_fcall f_20249(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20274)
static void C_ccall f_20274(C_word c,C_word *av) C_noret;
C_noret_decl(f_20283)
static void C_fcall f_20283(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20319)
static void C_fcall f_20319(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20322)
static void C_ccall f_20322(C_word c,C_word *av) C_noret;
C_noret_decl(f_20361)
static void C_fcall f_20361(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20364)
static void C_ccall f_20364(C_word c,C_word *av) C_noret;
C_noret_decl(f_20389)
static void C_ccall f_20389(C_word c,C_word *av) C_noret;
C_noret_decl(f_20429)
static C_word C_fcall f_20429(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_20457)
static void C_fcall f_20457(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20582)
static C_word C_fcall f_20582(C_word t0,C_word t1);
C_noret_decl(f_20616)
static C_word C_fcall f_20616(C_word *a,C_word t0,C_word t1,C_word t2,C_word t3);
C_noret_decl(f_20757)
static void C_fcall f_20757(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20761)
static void C_ccall f_20761(C_word c,C_word *av) C_noret;
C_noret_decl(f_20764)
static void C_ccall f_20764(C_word c,C_word *av) C_noret;
C_noret_decl(f_20766)
static void C_fcall f_20766(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_20769)
static void C_fcall f_20769(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20779)
static void C_fcall f_20779(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20783)
static void C_fcall f_20783(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20792)
static void C_ccall f_20792(C_word c,C_word *av) C_noret;
C_noret_decl(f_20795)
static void C_ccall f_20795(C_word c,C_word *av) C_noret;
C_noret_decl(f_20804)
static void C_fcall f_20804(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20840)
static void C_fcall f_20840(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20851)
static void C_ccall f_20851(C_word c,C_word *av) C_noret;
C_noret_decl(f_20857)
static void C_fcall f_20857(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20874)
static void C_ccall f_20874(C_word c,C_word *av) C_noret;
C_noret_decl(f_20907)
static void C_ccall f_20907(C_word c,C_word *av) C_noret;
C_noret_decl(f_20913)
static void C_ccall f_20913(C_word c,C_word *av) C_noret;
C_noret_decl(f_20920)
static void C_ccall f_20920(C_word c,C_word *av) C_noret;
C_noret_decl(f_20931)
static void C_ccall f_20931(C_word c,C_word *av) C_noret;
C_noret_decl(f_20945)
static void C_fcall f_20945(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20963)
static void C_ccall f_20963(C_word c,C_word *av) C_noret;
C_noret_decl(f_20988)
static void C_ccall f_20988(C_word c,C_word *av) C_noret;
C_noret_decl(f_21015)
static void C_ccall f_21015(C_word c,C_word *av) C_noret;
C_noret_decl(f_21046)
static void C_ccall f_21046(C_word c,C_word *av) C_noret;
C_noret_decl(f_21053)
static void C_ccall f_21053(C_word c,C_word *av) C_noret;
C_noret_decl(f_21065)
static void C_ccall f_21065(C_word c,C_word *av) C_noret;
C_noret_decl(f_21095)
static void C_ccall f_21095(C_word c,C_word *av) C_noret;
C_noret_decl(f_21109)
static void C_fcall f_21109(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21112)
static void C_ccall f_21112(C_word c,C_word *av) C_noret;
C_noret_decl(f_21118)
static void C_ccall f_21118(C_word c,C_word *av) C_noret;
C_noret_decl(f_21132)
static void C_ccall f_21132(C_word c,C_word *av) C_noret;
C_noret_decl(f_21156)
static void C_fcall f_21156(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21159)
static void C_ccall f_21159(C_word c,C_word *av) C_noret;
C_noret_decl(f_21166)
static void C_ccall f_21166(C_word c,C_word *av) C_noret;
C_noret_decl(f_21183)
static void C_ccall f_21183(C_word c,C_word *av) C_noret;
C_noret_decl(f_21199)
static void C_ccall f_21199(C_word c,C_word *av) C_noret;
C_noret_decl(f_21202)
static void C_ccall f_21202(C_word c,C_word *av) C_noret;
C_noret_decl(f_21205)
static void C_ccall f_21205(C_word c,C_word *av) C_noret;
C_noret_decl(f_21211)
static void C_ccall f_21211(C_word c,C_word *av) C_noret;
C_noret_decl(f_21221)
static void C_ccall f_21221(C_word c,C_word *av) C_noret;
C_noret_decl(f_21225)
static void C_ccall f_21225(C_word c,C_word *av) C_noret;
C_noret_decl(f_21236)
static void C_ccall f_21236(C_word c,C_word *av) C_noret;
C_noret_decl(f_21240)
static void C_ccall f_21240(C_word c,C_word *av) C_noret;
C_noret_decl(f_21244)
static void C_ccall f_21244(C_word c,C_word *av) C_noret;
C_noret_decl(f_21252)
static void C_ccall f_21252(C_word c,C_word *av) C_noret;
C_noret_decl(f_21260)
static void C_ccall f_21260(C_word c,C_word *av) C_noret;
C_noret_decl(f_21275)
static void C_ccall f_21275(C_word c,C_word *av) C_noret;
C_noret_decl(f_21281)
static void C_ccall f_21281(C_word c,C_word *av) C_noret;
C_noret_decl(f_21295)
static void C_ccall f_21295(C_word c,C_word *av) C_noret;
C_noret_decl(f_21309)
static void C_ccall f_21309(C_word c,C_word *av) C_noret;
C_noret_decl(f_21315)
static void C_ccall f_21315(C_word c,C_word *av) C_noret;
C_noret_decl(f_21318)
static void C_ccall f_21318(C_word c,C_word *av) C_noret;
C_noret_decl(f_21345)
static void C_ccall f_21345(C_word c,C_word *av) C_noret;
C_noret_decl(f_21349)
static void C_ccall f_21349(C_word c,C_word *av) C_noret;
C_noret_decl(f_21369)
static void C_ccall f_21369(C_word c,C_word *av) C_noret;
C_noret_decl(f_21375)
static void C_ccall f_21375(C_word c,C_word *av) C_noret;
C_noret_decl(f_21378)
static void C_ccall f_21378(C_word c,C_word *av) C_noret;
C_noret_decl(f_21381)
static void C_ccall f_21381(C_word c,C_word *av) C_noret;
C_noret_decl(f_21397)
static void C_ccall f_21397(C_word c,C_word *av) C_noret;
C_noret_decl(f_21405)
static void C_ccall f_21405(C_word c,C_word *av) C_noret;
C_noret_decl(f_21417)
static void C_ccall f_21417(C_word c,C_word *av) C_noret;
C_noret_decl(f_21447)
static void C_ccall f_21447(C_word c,C_word *av) C_noret;
C_noret_decl(f_21453)
static void C_ccall f_21453(C_word c,C_word *av) C_noret;
C_noret_decl(f_21456)
static void C_ccall f_21456(C_word c,C_word *av) C_noret;
C_noret_decl(f_21459)
static void C_ccall f_21459(C_word c,C_word *av) C_noret;
C_noret_decl(f_21475)
static void C_ccall f_21475(C_word c,C_word *av) C_noret;
C_noret_decl(f_21483)
static void C_ccall f_21483(C_word c,C_word *av) C_noret;
C_noret_decl(f_21491)
static void C_ccall f_21491(C_word c,C_word *av) C_noret;
C_noret_decl(f_21495)
static void C_ccall f_21495(C_word c,C_word *av) C_noret;
C_noret_decl(f_21514)
static void C_fcall f_21514(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21532)
static void C_ccall f_21532(C_word c,C_word *av) C_noret;
C_noret_decl(f_21660)
static void C_ccall f_21660(C_word c,C_word *av) C_noret;
C_noret_decl(f_21666)
static void C_ccall f_21666(C_word c,C_word *av) C_noret;
C_noret_decl(f_21669)
static void C_ccall f_21669(C_word c,C_word *av) C_noret;
C_noret_decl(f_21677)
static void C_fcall f_21677(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21716)
static void C_fcall f_21716(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_21751)
static void C_fcall f_21751(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21758)
static void C_ccall f_21758(C_word c,C_word *av) C_noret;
C_noret_decl(f_21784)
static void C_ccall f_21784(C_word c,C_word *av) C_noret;
C_noret_decl(f_21880)
static void C_fcall f_21880(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21884)
static void C_ccall f_21884(C_word c,C_word *av) C_noret;
C_noret_decl(f_21897)
static void C_ccall f_21897(C_word c,C_word *av) C_noret;
C_noret_decl(f_21909)
static void C_fcall f_21909(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21913)
static void C_ccall f_21913(C_word c,C_word *av) C_noret;
C_noret_decl(f_21920)
static void C_ccall f_21920(C_word c,C_word *av) C_noret;
C_noret_decl(f_21926)
static void C_fcall f_21926(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21930)
static void C_ccall f_21930(C_word c,C_word *av) C_noret;
C_noret_decl(f_21933)
static void C_ccall f_21933(C_word c,C_word *av) C_noret;
C_noret_decl(f_21935)
static void C_fcall f_21935(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_21942)
static void C_fcall f_21942(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21949)
static C_word C_fcall f_21949(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_21984)
static void C_ccall f_21984(C_word c,C_word *av) C_noret;
C_noret_decl(f_21997)
static void C_fcall f_21997(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22098)
static void C_ccall f_22098(C_word c,C_word *av) C_noret;
C_noret_decl(f_22104)
static void C_fcall f_22104(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22110)
static void C_fcall f_22110(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22163)
static void C_fcall f_22163(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22234)
static void C_fcall f_22234(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22287)
static void C_fcall f_22287(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22296)
static void C_fcall f_22296(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22316)
static void C_ccall f_22316(C_word c,C_word *av) C_noret;
C_noret_decl(f_22345)
static C_word C_fcall f_22345(C_word t0,C_word t1);
C_noret_decl(f_22369)
static void C_fcall f_22369(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22376)
static void C_ccall f_22376(C_word c,C_word *av) C_noret;
C_noret_decl(f_22379)
static void C_ccall f_22379(C_word c,C_word *av) C_noret;
C_noret_decl(f_22382)
static void C_ccall f_22382(C_word c,C_word *av) C_noret;
C_noret_decl(f_22385)
static void C_ccall f_22385(C_word c,C_word *av) C_noret;
C_noret_decl(f_22401)
static void C_fcall f_22401(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22418)
static void C_ccall f_22418(C_word c,C_word *av) C_noret;
C_noret_decl(f_22453)
static void C_ccall f_22453(C_word c,C_word *av) C_noret;
C_noret_decl(f_22457)
static void C_fcall f_22457(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22485)
static void C_ccall f_22485(C_word c,C_word *av) C_noret;
C_noret_decl(f_22488)
static void C_fcall f_22488(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22494)
static void C_ccall f_22494(C_word c,C_word *av) C_noret;
C_noret_decl(f_22580)
static void C_ccall f_22580(C_word c,C_word *av) C_noret;
C_noret_decl(f_22583)
static void C_ccall f_22583(C_word c,C_word *av) C_noret;
C_noret_decl(f_22588)
static void C_fcall f_22588(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22598)
static void C_ccall f_22598(C_word c,C_word *av) C_noret;
C_noret_decl(f_22601)
static void C_ccall f_22601(C_word c,C_word *av) C_noret;
C_noret_decl(f_22621)
static void C_fcall f_22621(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22629)
static void C_fcall f_22629(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22662)
static void C_ccall f_22662(C_word c,C_word *av) C_noret;
C_noret_decl(f_22664)
static void C_fcall f_22664(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22689)
static void C_ccall f_22689(C_word c,C_word *av) C_noret;
C_noret_decl(f_22702)
static void C_fcall f_22702(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22741)
static void C_ccall f_22741(C_word c,C_word *av) C_noret;
C_noret_decl(f_22756)
static void C_ccall f_22756(C_word c,C_word *av) C_noret;
C_noret_decl(f_22777)
static void C_fcall f_22777(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_22795)
static void C_ccall f_22795(C_word c,C_word *av) C_noret;
C_noret_decl(f_22804)
static void C_ccall f_22804(C_word c,C_word *av) C_noret;
C_noret_decl(f_22818)
static void C_fcall f_22818(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22822)
static void C_ccall f_22822(C_word c,C_word *av) C_noret;
C_noret_decl(f_22825)
static void C_ccall f_22825(C_word c,C_word *av) C_noret;
C_noret_decl(f_22831)
static void C_ccall f_22831(C_word c,C_word *av) C_noret;
C_noret_decl(f_22834)
static void C_fcall f_22834(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22837)
static void C_ccall f_22837(C_word c,C_word *av) C_noret;
C_noret_decl(f_22854)
static void C_ccall f_22854(C_word c,C_word *av) C_noret;
C_noret_decl(f_22926)
static void C_ccall f_22926(C_word c,C_word *av) C_noret;
C_noret_decl(f_22930)
static void C_ccall f_22930(C_word c,C_word *av) C_noret;
C_noret_decl(f_22934)
static void C_ccall f_22934(C_word c,C_word *av) C_noret;
C_noret_decl(f_22936)
static void C_fcall f_22936(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22959)
static void C_fcall f_22959(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_22988)
static void C_fcall f_22988(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22992)
static void C_fcall f_22992(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23016)
static void C_fcall f_23016(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23044)
static void C_ccall f_23044(C_word c,C_word *av) C_noret;
C_noret_decl(f_23054)
static void C_fcall f_23054(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23064)
static void C_ccall f_23064(C_word c,C_word *av) C_noret;
C_noret_decl(f_23069)
static void C_fcall f_23069(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_23084)
static void C_fcall f_23084(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_23104)
static void C_fcall f_23104(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_23127)
static void C_fcall f_23127(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23159)
static void C_ccall f_23159(C_word c,C_word *av) C_noret;
C_noret_decl(f_23166)
static void C_ccall f_23166(C_word c,C_word *av) C_noret;
C_noret_decl(f_23168)
static void C_ccall f_23168(C_word c,C_word *av) C_noret;
C_noret_decl(f_23192)
static void C_ccall f_23192(C_word c,C_word *av) C_noret;
C_noret_decl(f_23236)
static void C_fcall f_23236(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23249)
static void C_fcall f_23249(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23263)
static void C_fcall f_23263(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_23290)
static void C_fcall f_23290(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_23371)
static void C_ccall f_23371(C_word c,C_word *av) C_noret;
C_noret_decl(f_23387)
static void C_ccall f_23387(C_word c,C_word *av) C_noret;
C_noret_decl(f_23397)
static void C_ccall f_23397(C_word c,C_word *av) C_noret;
C_noret_decl(f_23406)
static void C_ccall f_23406(C_word c,C_word *av) C_noret;
C_noret_decl(f_23446)
static void C_fcall f_23446(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_23449)
static void C_fcall f_23449(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23472)
static void C_ccall f_23472(C_word c,C_word *av) C_noret;
C_noret_decl(f_23488)
static void C_fcall f_23488(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23495)
static void C_ccall f_23495(C_word c,C_word *av) C_noret;
C_noret_decl(f_23515)
static void C_ccall f_23515(C_word c,C_word *av) C_noret;
C_noret_decl(f_23535)
static void C_ccall f_23535(C_word c,C_word *av) C_noret;
C_noret_decl(f_23538)
static void C_ccall f_23538(C_word c,C_word *av) C_noret;
C_noret_decl(f_23539)
static void C_ccall f_23539(C_word c,C_word *av) C_noret;
C_noret_decl(f_23545)
static void C_ccall f_23545(C_word c,C_word *av) C_noret;
C_noret_decl(f_23561)
static void C_ccall f_23561(C_word c,C_word *av) C_noret;
C_noret_decl(f_23689)
static void C_ccall f_23689(C_word c,C_word *av) C_noret;
C_noret_decl(f_23708)
static void C_ccall f_23708(C_word c,C_word *av) C_noret;
C_noret_decl(f_23731)
static void C_ccall f_23731(C_word c,C_word *av) C_noret;
C_noret_decl(f_23732)
static void C_ccall f_23732(C_word c,C_word *av) C_noret;
C_noret_decl(f_23738)
static void C_ccall f_23738(C_word c,C_word *av) C_noret;
C_noret_decl(f_23757)
static void C_ccall f_23757(C_word c,C_word *av) C_noret;
C_noret_decl(f_23758)
static void C_ccall f_23758(C_word c,C_word *av) C_noret;
C_noret_decl(f_23764)
static void C_ccall f_23764(C_word c,C_word *av) C_noret;
C_noret_decl(f_23786)
static void C_ccall f_23786(C_word c,C_word *av) C_noret;
C_noret_decl(f_23792)
static void C_ccall f_23792(C_word c,C_word *av) C_noret;
C_noret_decl(f_23793)
static void C_ccall f_23793(C_word c,C_word *av) C_noret;
C_noret_decl(f_23799)
static void C_ccall f_23799(C_word c,C_word *av) C_noret;
C_noret_decl(f_23831)
static void C_ccall f_23831(C_word c,C_word *av) C_noret;
C_noret_decl(f_23837)
static void C_ccall f_23837(C_word c,C_word *av) C_noret;
C_noret_decl(f_23839)
static void C_ccall f_23839(C_word c,C_word *av) C_noret;
C_noret_decl(f_23845)
static void C_ccall f_23845(C_word c,C_word *av) C_noret;
C_noret_decl(f_23855)
static void C_ccall f_23855(C_word c,C_word *av) C_noret;
C_noret_decl(f_23861)
static void C_ccall f_23861(C_word c,C_word *av) C_noret;
C_noret_decl(f_23885)
static void C_ccall f_23885(C_word c,C_word *av) C_noret;
C_noret_decl(f_23891)
static void C_ccall f_23891(C_word c,C_word *av) C_noret;
C_noret_decl(f_23898)
static void C_ccall f_23898(C_word c,C_word *av) C_noret;
C_noret_decl(f_23904)
static void C_ccall f_23904(C_word c,C_word *av) C_noret;
C_noret_decl(f_23987)
static void C_fcall f_23987(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23990)
static void C_fcall f_23990(C_word t0,C_word t1) C_noret;
C_noret_decl(f_23991)
static void C_ccall f_23991(C_word c,C_word *av) C_noret;
C_noret_decl(f_24004)
static void C_ccall f_24004(C_word c,C_word *av) C_noret;
C_noret_decl(f_24010)
static void C_ccall f_24010(C_word c,C_word *av) C_noret;
C_noret_decl(f_24023)
static void C_ccall f_24023(C_word c,C_word *av) C_noret;
C_noret_decl(f_24028)
static void C_ccall f_24028(C_word c,C_word *av) C_noret;
C_noret_decl(f_24059)
static void C_ccall f_24059(C_word c,C_word *av) C_noret;
C_noret_decl(f_24063)
static void C_ccall f_24063(C_word c,C_word *av) C_noret;
C_noret_decl(f_24068)
static void C_ccall f_24068(C_word c,C_word *av) C_noret;
C_noret_decl(f_24139)
static void C_fcall f_24139(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24142)
static void C_fcall f_24142(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24143)
static void C_ccall f_24143(C_word c,C_word *av) C_noret;
C_noret_decl(f_24156)
static void C_ccall f_24156(C_word c,C_word *av) C_noret;
C_noret_decl(f_24162)
static void C_ccall f_24162(C_word c,C_word *av) C_noret;
C_noret_decl(f_24172)
static void C_ccall f_24172(C_word c,C_word *av) C_noret;
C_noret_decl(f_24184)
static void C_ccall f_24184(C_word c,C_word *av) C_noret;
C_noret_decl(f_24197)
static void C_ccall f_24197(C_word c,C_word *av) C_noret;
C_noret_decl(f_24218)
static void C_ccall f_24218(C_word c,C_word *av) C_noret;
C_noret_decl(f_24228)
static void C_ccall f_24228(C_word c,C_word *av) C_noret;
C_noret_decl(f_24232)
static void C_ccall f_24232(C_word c,C_word *av) C_noret;
C_noret_decl(f_24238)
static void C_ccall f_24238(C_word c,C_word *av) C_noret;
C_noret_decl(f_24310)
static void C_ccall f_24310(C_word c,C_word *av) C_noret;
C_noret_decl(f_24360)
static void C_ccall f_24360(C_word c,C_word *av) C_noret;
C_noret_decl(f_24373)
static void C_ccall f_24373(C_word c,C_word *av) C_noret;
C_noret_decl(f_24374)
static void C_ccall f_24374(C_word c,C_word *av) C_noret;
C_noret_decl(f_24381)
static void C_ccall f_24381(C_word c,C_word *av) C_noret;
C_noret_decl(f_24389)
static void C_ccall f_24389(C_word c,C_word *av) C_noret;
C_noret_decl(f_24396)
static void C_ccall f_24396(C_word c,C_word *av) C_noret;
C_noret_decl(f_24408)
static void C_ccall f_24408(C_word c,C_word *av) C_noret;
C_noret_decl(f_24409)
static void C_ccall f_24409(C_word c,C_word *av) C_noret;
C_noret_decl(f_24416)
static void C_ccall f_24416(C_word c,C_word *av) C_noret;
C_noret_decl(f_24424)
static void C_ccall f_24424(C_word c,C_word *av) C_noret;
C_noret_decl(f_24431)
static void C_ccall f_24431(C_word c,C_word *av) C_noret;
C_noret_decl(f_24446)
static void C_ccall f_24446(C_word c,C_word *av) C_noret;
C_noret_decl(f_24447)
static void C_ccall f_24447(C_word c,C_word *av) C_noret;
C_noret_decl(f_24451)
static void C_ccall f_24451(C_word c,C_word *av) C_noret;
C_noret_decl(f_24457)
static void C_ccall f_24457(C_word c,C_word *av) C_noret;
C_noret_decl(f_24464)
static void C_ccall f_24464(C_word c,C_word *av) C_noret;
C_noret_decl(f_24473)
static void C_ccall f_24473(C_word c,C_word *av) C_noret;
C_noret_decl(f_24483)
static void C_ccall f_24483(C_word c,C_word *av) C_noret;
C_noret_decl(f_24487)
static void C_ccall f_24487(C_word c,C_word *av) C_noret;
C_noret_decl(f_24492)
static void C_ccall f_24492(C_word c,C_word *av) C_noret;
C_noret_decl(f_24509)
static void C_ccall f_24509(C_word c,C_word *av) C_noret;
C_noret_decl(f_24518)
static void C_ccall f_24518(C_word c,C_word *av) C_noret;
C_noret_decl(f_24529)
static void C_ccall f_24529(C_word c,C_word *av) C_noret;
C_noret_decl(f_24530)
static void C_ccall f_24530(C_word c,C_word *av) C_noret;
C_noret_decl(f_24534)
static void C_ccall f_24534(C_word c,C_word *av) C_noret;
C_noret_decl(f_24545)
static void C_ccall f_24545(C_word c,C_word *av) C_noret;
C_noret_decl(f_24552)
static void C_ccall f_24552(C_word c,C_word *av) C_noret;
C_noret_decl(f_24564)
static void C_ccall f_24564(C_word c,C_word *av) C_noret;
C_noret_decl(f_24567)
static void C_ccall f_24567(C_word c,C_word *av) C_noret;
C_noret_decl(f_24570)
static void C_ccall f_24570(C_word c,C_word *av) C_noret;
C_noret_decl(f_24579)
static void C_fcall f_24579(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24582)
static void C_ccall f_24582(C_word c,C_word *av) C_noret;
C_noret_decl(f_24583)
static void C_ccall f_24583(C_word c,C_word *av) C_noret;
C_noret_decl(f_24640)
static void C_ccall f_24640(C_word c,C_word *av) C_noret;
C_noret_decl(f_24641)
static void C_ccall f_24641(C_word c,C_word *av) C_noret;
C_noret_decl(f_24647)
static void C_ccall f_24647(C_word c,C_word *av) C_noret;
C_noret_decl(f_24683)
static void C_ccall f_24683(C_word c,C_word *av) C_noret;
C_noret_decl(f_24688)
static void C_ccall f_24688(C_word c,C_word *av) C_noret;
C_noret_decl(f_24720)
static void C_ccall f_24720(C_word c,C_word *av) C_noret;
C_noret_decl(f_24723)
static void C_fcall f_24723(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24724)
static void C_ccall f_24724(C_word c,C_word *av) C_noret;
C_noret_decl(f_24728)
static void C_ccall f_24728(C_word c,C_word *av) C_noret;
C_noret_decl(f_24746)
static void C_fcall f_24746(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_24759)
static void C_ccall f_24759(C_word c,C_word *av) C_noret;
C_noret_decl(f_24773)
static void C_ccall f_24773(C_word c,C_word *av) C_noret;
C_noret_decl(f_24777)
static void C_ccall f_24777(C_word c,C_word *av) C_noret;
C_noret_decl(f_24791)
static void C_ccall f_24791(C_word c,C_word *av) C_noret;
C_noret_decl(f_24797)
static void C_ccall f_24797(C_word c,C_word *av) C_noret;
C_noret_decl(f_24810)
static void C_ccall f_24810(C_word c,C_word *av) C_noret;
C_noret_decl(f_24817)
static void C_ccall f_24817(C_word c,C_word *av) C_noret;
C_noret_decl(f_24824)
static void C_ccall f_24824(C_word c,C_word *av) C_noret;
C_noret_decl(f_24846)
static void C_ccall f_24846(C_word c,C_word *av) C_noret;
C_noret_decl(f_24850)
static void C_ccall f_24850(C_word c,C_word *av) C_noret;
C_noret_decl(f_24937)
static void C_ccall f_24937(C_word c,C_word *av) C_noret;
C_noret_decl(f_24938)
static void C_ccall f_24938(C_word c,C_word *av) C_noret;
C_noret_decl(f_24986)
static void C_ccall f_24986(C_word c,C_word *av) C_noret;
C_noret_decl(f_25046)
static void C_ccall f_25046(C_word c,C_word *av) C_noret;
C_noret_decl(f_25081)
static void C_ccall f_25081(C_word c,C_word *av) C_noret;
C_noret_decl(f_25281)
static void C_ccall f_25281(C_word c,C_word *av) C_noret;
C_noret_decl(f_25301)
static void C_ccall f_25301(C_word c,C_word *av) C_noret;
C_noret_decl(f_25310)
static void C_ccall f_25310(C_word c,C_word *av) C_noret;
C_noret_decl(f_25316)
static void C_ccall f_25316(C_word c,C_word *av) C_noret;
C_noret_decl(f_25322)
static void C_ccall f_25322(C_word c,C_word *av) C_noret;
C_noret_decl(f_25339)
static void C_ccall f_25339(C_word c,C_word *av) C_noret;
C_noret_decl(f_25376)
static void C_ccall f_25376(C_word c,C_word *av) C_noret;
C_noret_decl(f_25385)
static void C_ccall f_25385(C_word c,C_word *av) C_noret;
C_noret_decl(f_25391)
static void C_ccall f_25391(C_word c,C_word *av) C_noret;
C_noret_decl(f_25397)
static void C_ccall f_25397(C_word c,C_word *av) C_noret;
C_noret_decl(f_25431)
static void C_ccall f_25431(C_word c,C_word *av) C_noret;
C_noret_decl(f_25461)
static void C_ccall f_25461(C_word c,C_word *av) C_noret;
C_noret_decl(f_25468)
static void C_ccall f_25468(C_word c,C_word *av) C_noret;
C_noret_decl(f_25505)
static void C_ccall f_25505(C_word c,C_word *av) C_noret;
C_noret_decl(f_25512)
static void C_ccall f_25512(C_word c,C_word *av) C_noret;
C_noret_decl(f_25519)
static void C_fcall f_25519(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25528)
static void C_fcall f_25528(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25547)
static void C_ccall f_25547(C_word c,C_word *av) C_noret;
C_noret_decl(f_25564)
static void C_ccall f_25564(C_word c,C_word *av) C_noret;
C_noret_decl(f_25571)
static void C_ccall f_25571(C_word c,C_word *av) C_noret;
C_noret_decl(f_25592)
static void C_ccall f_25592(C_word c,C_word *av) C_noret;
C_noret_decl(f_25612)
static void C_ccall f_25612(C_word c,C_word *av) C_noret;
C_noret_decl(f_25619)
static void C_ccall f_25619(C_word c,C_word *av) C_noret;
C_noret_decl(f_25626)
static void C_fcall f_25626(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25645)
static void C_ccall f_25645(C_word c,C_word *av) C_noret;
C_noret_decl(f_25652)
static void C_ccall f_25652(C_word c,C_word *av) C_noret;
C_noret_decl(f_25659)
static void C_fcall f_25659(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25682)
static void C_ccall f_25682(C_word c,C_word *av) C_noret;
C_noret_decl(f_25703)
static void C_ccall f_25703(C_word c,C_word *av) C_noret;
C_noret_decl(f_25710)
static void C_ccall f_25710(C_word c,C_word *av) C_noret;
C_noret_decl(f_25717)
static void C_ccall f_25717(C_word c,C_word *av) C_noret;
C_noret_decl(f_25724)
static void C_fcall f_25724(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25733)
static void C_fcall f_25733(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25753)
static void C_ccall f_25753(C_word c,C_word *av) C_noret;
C_noret_decl(f_25769)
static void C_ccall f_25769(C_word c,C_word *av) C_noret;
C_noret_decl(f_25786)
static void C_ccall f_25786(C_word c,C_word *av) C_noret;
C_noret_decl(f_25806)
static void C_ccall f_25806(C_word c,C_word *av) C_noret;
C_noret_decl(f_25810)
static void C_ccall f_25810(C_word c,C_word *av) C_noret;
C_noret_decl(f_25813)
static void C_ccall f_25813(C_word c,C_word *av) C_noret;
C_noret_decl(f_25819)
static void C_fcall f_25819(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25864)
static void C_ccall f_25864(C_word c,C_word *av) C_noret;
C_noret_decl(f_25928)
static void C_ccall f_25928(C_word c,C_word *av) C_noret;
C_noret_decl(f_25945)
static void C_ccall f_25945(C_word c,C_word *av) C_noret;
C_noret_decl(f_25947)
static void C_fcall f_25947(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25957)
static void C_ccall f_25957(C_word c,C_word *av) C_noret;
C_noret_decl(f_25963)
static void C_ccall f_25963(C_word c,C_word *av) C_noret;
C_noret_decl(f_25969)
static void C_ccall f_25969(C_word c,C_word *av) C_noret;
C_noret_decl(f_25985)
static void C_ccall f_25985(C_word c,C_word *av) C_noret;
C_noret_decl(f_25991)
static void C_ccall f_25991(C_word c,C_word *av) C_noret;
C_noret_decl(f_26013)
static void C_ccall f_26013(C_word c,C_word *av) C_noret;
C_noret_decl(f_26027)
static void C_ccall f_26027(C_word c,C_word *av) C_noret;
C_noret_decl(f_26044)
static void C_ccall f_26044(C_word c,C_word *av) C_noret;
C_noret_decl(f_26046)
static void C_fcall f_26046(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26056)
static void C_ccall f_26056(C_word c,C_word *av) C_noret;
C_noret_decl(f_26062)
static void C_ccall f_26062(C_word c,C_word *av) C_noret;
C_noret_decl(f_26068)
static void C_ccall f_26068(C_word c,C_word *av) C_noret;
C_noret_decl(f_26084)
static void C_ccall f_26084(C_word c,C_word *av) C_noret;
C_noret_decl(f_26142)
static void C_ccall f_26142(C_word c,C_word *av) C_noret;
C_noret_decl(f_26168)
static void C_fcall f_26168(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26170)
static void C_ccall f_26170(C_word c,C_word *av) C_noret;
C_noret_decl(f_26183)
static void C_ccall f_26183(C_word c,C_word *av) C_noret;
C_noret_decl(f_26202)
static void C_ccall f_26202(C_word c,C_word *av) C_noret;
C_noret_decl(f_26211)
static void C_ccall f_26211(C_word c,C_word *av) C_noret;
C_noret_decl(f_26217)
static void C_ccall f_26217(C_word c,C_word *av) C_noret;
C_noret_decl(f_26223)
static void C_ccall f_26223(C_word c,C_word *av) C_noret;
C_noret_decl(f_26229)
static void C_ccall f_26229(C_word c,C_word *av) C_noret;
C_noret_decl(f_26381)
static void C_fcall f_26381(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26393)
static void C_fcall f_26393(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_26443)
static void C_fcall f_26443(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_26461)
static void C_ccall f_26461(C_word c,C_word *av) C_noret;
C_noret_decl(f_26472)
static void C_fcall f_26472(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26478)
static void C_ccall f_26478(C_word c,C_word *av) C_noret;
C_noret_decl(f_26490)
static void C_ccall f_26490(C_word c,C_word *av) C_noret;
C_noret_decl(f_26492)
static void C_fcall f_26492(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26502)
static void C_fcall f_26502(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_26505)
static void C_fcall f_26505(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26531)
static void C_ccall f_26531(C_word c,C_word *av) C_noret;
C_noret_decl(f_26552)
static void C_ccall f_26552(C_word c,C_word *av) C_noret;
C_noret_decl(f_26556)
static void C_ccall f_26556(C_word c,C_word *av) C_noret;
C_noret_decl(f_26569)
static void C_ccall f_26569(C_word c,C_word *av) C_noret;
C_noret_decl(f_26571)
static void C_fcall f_26571(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26596)
static void C_ccall f_26596(C_word c,C_word *av) C_noret;
C_noret_decl(f_26620)
static void C_ccall f_26620(C_word c,C_word *av) C_noret;
C_noret_decl(f_26633)
static void C_ccall f_26633(C_word c,C_word *av) C_noret;
C_noret_decl(f_26635)
static void C_fcall f_26635(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26660)
static void C_ccall f_26660(C_word c,C_word *av) C_noret;
C_noret_decl(f_26682)
static void C_ccall f_26682(C_word c,C_word *av) C_noret;
C_noret_decl(f_26690)
static void C_ccall f_26690(C_word c,C_word *av) C_noret;
C_noret_decl(f_26703)
static void C_ccall f_26703(C_word c,C_word *av) C_noret;
C_noret_decl(f_26705)
static void C_fcall f_26705(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26730)
static void C_ccall f_26730(C_word c,C_word *av) C_noret;
C_noret_decl(f_26750)
static void C_ccall f_26750(C_word c,C_word *av) C_noret;
C_noret_decl(f_26760)
static void C_ccall f_26760(C_word c,C_word *av) C_noret;
C_noret_decl(f_26775)
static void C_ccall f_26775(C_word c,C_word *av) C_noret;
C_noret_decl(f_26788)
static void C_ccall f_26788(C_word c,C_word *av) C_noret;
C_noret_decl(f_26790)
static void C_fcall f_26790(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26815)
static void C_ccall f_26815(C_word c,C_word *av) C_noret;
C_noret_decl(f_26937)
static void C_ccall f_26937(C_word c,C_word *av) C_noret;
C_noret_decl(f_26939)
static void C_ccall f_26939(C_word c,C_word *av) C_noret;
C_noret_decl(f_26957)
static void C_ccall f_26957(C_word c,C_word *av) C_noret;
C_noret_decl(f_26959)
static void C_fcall f_26959(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26992)
static void C_fcall f_26992(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_27044)
static C_word C_fcall f_27044(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_27080)
static void C_ccall f_27080(C_word c,C_word *av) C_noret;
C_noret_decl(f_27088)
static void C_ccall f_27088(C_word c,C_word *av) C_noret;
C_noret_decl(f_27092)
static void C_ccall f_27092(C_word c,C_word *av) C_noret;
C_noret_decl(f_27094)
static void C_fcall f_27094(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_27108)
static void C_ccall f_27108(C_word c,C_word *av) C_noret;
C_noret_decl(f_27112)
static void C_ccall f_27112(C_word c,C_word *av) C_noret;
C_noret_decl(f_27116)
static void C_ccall f_27116(C_word c,C_word *av) C_noret;
C_noret_decl(f_27129)
static void C_ccall f_27129(C_word c,C_word *av) C_noret;
C_noret_decl(f_27133)
static void C_ccall f_27133(C_word c,C_word *av) C_noret;
C_noret_decl(f_27137)
static void C_ccall f_27137(C_word c,C_word *av) C_noret;
C_noret_decl(f_27275)
static void C_fcall f_27275(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27283)
static void C_ccall f_27283(C_word c,C_word *av) C_noret;
C_noret_decl(f_27287)
static void C_ccall f_27287(C_word c,C_word *av) C_noret;
C_noret_decl(f_27289)
static void C_fcall f_27289(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_27303)
static void C_ccall f_27303(C_word c,C_word *av) C_noret;
C_noret_decl(f_27316)
static void C_ccall f_27316(C_word c,C_word *av) C_noret;
C_noret_decl(f_27320)
static void C_ccall f_27320(C_word c,C_word *av) C_noret;
C_noret_decl(f_27366)
static void C_fcall f_27366(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27369)
static void C_fcall f_27369(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27376)
static void C_fcall f_27376(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27471)
static void C_ccall f_27471(C_word c,C_word *av) C_noret;
C_noret_decl(f_27479)
static void C_ccall f_27479(C_word c,C_word *av) C_noret;
C_noret_decl(f_27483)
static void C_ccall f_27483(C_word c,C_word *av) C_noret;
C_noret_decl(f_27485)
static void C_fcall f_27485(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_27502)
static void C_ccall f_27502(C_word c,C_word *av) C_noret;
C_noret_decl(f_27597)
static void C_fcall f_27597(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27605)
static void C_ccall f_27605(C_word c,C_word *av) C_noret;
C_noret_decl(f_27607)
static void C_fcall f_27607(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27615)
static void C_ccall f_27615(C_word c,C_word *av) C_noret;
C_noret_decl(f_27617)
static void C_fcall f_27617(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_27631)
static void C_ccall f_27631(C_word c,C_word *av) C_noret;
C_noret_decl(f_27637)
static void C_fcall f_27637(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27646)
static void C_ccall f_27646(C_word c,C_word *av) C_noret;
C_noret_decl(f_27650)
static void C_ccall f_27650(C_word c,C_word *av) C_noret;
C_noret_decl(f_27654)
static void C_ccall f_27654(C_word c,C_word *av) C_noret;
C_noret_decl(f_27658)
static void C_ccall f_27658(C_word c,C_word *av) C_noret;
C_noret_decl(f_27682)
static void C_ccall f_27682(C_word c,C_word *av) C_noret;
C_noret_decl(f_27692)
static void C_ccall f_27692(C_word c,C_word *av) C_noret;
C_noret_decl(f_27701)
static void C_ccall f_27701(C_word c,C_word *av) C_noret;
C_noret_decl(f_27705)
static void C_ccall f_27705(C_word c,C_word *av) C_noret;
C_noret_decl(f_27709)
static void C_ccall f_27709(C_word c,C_word *av) C_noret;
C_noret_decl(f_27713)
static void C_ccall f_27713(C_word c,C_word *av) C_noret;
C_noret_decl(f_27723)
static void C_ccall f_27723(C_word c,C_word *av) C_noret;
C_noret_decl(f_27738)
static void C_ccall f_27738(C_word c,C_word *av) C_noret;
C_noret_decl(f_27742)
static void C_ccall f_27742(C_word c,C_word *av) C_noret;
C_noret_decl(f_27745)
static void C_ccall f_27745(C_word c,C_word *av) C_noret;
C_noret_decl(f_27748)
static void C_ccall f_27748(C_word c,C_word *av) C_noret;
C_noret_decl(f_27757)
static void C_fcall f_27757(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27760)
static void C_fcall f_27760(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27769)
static void C_ccall f_27769(C_word c,C_word *av) C_noret;
C_noret_decl(f_27772)
static void C_ccall f_27772(C_word c,C_word *av) C_noret;
C_noret_decl(f_27780)
static void C_fcall f_27780(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_27793)
static void C_ccall f_27793(C_word c,C_word *av) C_noret;
C_noret_decl(f_27828)
static void C_ccall f_27828(C_word c,C_word *av) C_noret;
C_noret_decl(f_27831)
static void C_ccall f_27831(C_word c,C_word *av) C_noret;
C_noret_decl(f_27939)
static void C_ccall f_27939(C_word c,C_word *av) C_noret;
C_noret_decl(f_27956)
static void C_ccall f_27956(C_word c,C_word *av) C_noret;
C_noret_decl(f_27960)
static void C_ccall f_27960(C_word c,C_word *av) C_noret;
C_noret_decl(f_27961)
static void C_ccall f_27961(C_word c,C_word *av) C_noret;
C_noret_decl(f_27969)
static void C_ccall f_27969(C_word c,C_word *av) C_noret;
C_noret_decl(f_27984)
static void C_ccall f_27984(C_word c,C_word *av) C_noret;
C_noret_decl(f_27988)
static void C_ccall f_27988(C_word c,C_word *av) C_noret;
C_noret_decl(f_27991)
static void C_ccall f_27991(C_word c,C_word *av) C_noret;
C_noret_decl(f_28000)
static void C_ccall f_28000(C_word c,C_word *av) C_noret;
C_noret_decl(f_28006)
static void C_ccall f_28006(C_word c,C_word *av) C_noret;
C_noret_decl(f_28014)
static void C_fcall f_28014(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_28027)
static void C_ccall f_28027(C_word c,C_word *av) C_noret;
C_noret_decl(f_28077)
static void C_ccall f_28077(C_word c,C_word *av) C_noret;
C_noret_decl(f_28087)
static void C_ccall f_28087(C_word c,C_word *av) C_noret;
C_noret_decl(f_28101)
static void C_ccall f_28101(C_word c,C_word *av) C_noret;
C_noret_decl(f_28104)
static void C_ccall f_28104(C_word c,C_word *av) C_noret;
C_noret_decl(f_28107)
static void C_ccall f_28107(C_word c,C_word *av) C_noret;
C_noret_decl(f_28159)
static void C_ccall f_28159(C_word c,C_word *av) C_noret;
C_noret_decl(f_28166)
static void C_ccall f_28166(C_word c,C_word *av) C_noret;
C_noret_decl(f_28170)
static void C_ccall f_28170(C_word c,C_word *av) C_noret;
C_noret_decl(f_28171)
static void C_ccall f_28171(C_word c,C_word *av) C_noret;
C_noret_decl(f_28179)
static void C_ccall f_28179(C_word c,C_word *av) C_noret;
C_noret_decl(f_28194)
static void C_ccall f_28194(C_word c,C_word *av) C_noret;
C_noret_decl(f_28198)
static void C_ccall f_28198(C_word c,C_word *av) C_noret;
C_noret_decl(f_28201)
static void C_ccall f_28201(C_word c,C_word *av) C_noret;
C_noret_decl(f_28215)
static void C_ccall f_28215(C_word c,C_word *av) C_noret;
C_noret_decl(f_28219)
static void C_ccall f_28219(C_word c,C_word *av) C_noret;
C_noret_decl(f_28223)
static void C_ccall f_28223(C_word c,C_word *av) C_noret;
C_noret_decl(f_28231)
static void C_ccall f_28231(C_word c,C_word *av) C_noret;
C_noret_decl(f_28273)
static void C_ccall f_28273(C_word c,C_word *av) C_noret;
C_noret_decl(f_28277)
static void C_ccall f_28277(C_word c,C_word *av) C_noret;
C_noret_decl(f_28282)
static void C_ccall f_28282(C_word c,C_word *av) C_noret;
C_noret_decl(f_28316)
static void C_ccall f_28316(C_word c,C_word *av) C_noret;
C_noret_decl(f_28323)
static void C_ccall f_28323(C_word c,C_word *av) C_noret;
C_noret_decl(f_28331)
static void C_ccall f_28331(C_word c,C_word *av) C_noret;
C_noret_decl(f_28333)
static void C_ccall f_28333(C_word c,C_word *av) C_noret;
C_noret_decl(f_28354)
static void C_ccall f_28354(C_word c,C_word *av) C_noret;
C_noret_decl(f_28366)
static void C_fcall f_28366(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28372)
static void C_fcall f_28372(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_28399)
static void C_ccall f_28399(C_word c,C_word *av) C_noret;
C_noret_decl(f_28424)
static void C_ccall f_28424(C_word c,C_word *av) C_noret;
C_noret_decl(f_28450)
static void C_ccall f_28450(C_word c,C_word *av) C_noret;
C_noret_decl(f_28514)
static void C_ccall f_28514(C_word c,C_word *av) C_noret;
C_noret_decl(f_28546)
static void C_fcall f_28546(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28576)
static void C_ccall f_28576(C_word c,C_word *av) C_noret;
C_noret_decl(f_28597)
static void C_ccall f_28597(C_word c,C_word *av) C_noret;
C_noret_decl(f_28601)
static void C_ccall f_28601(C_word c,C_word *av) C_noret;
C_noret_decl(f_28606)
static void C_ccall f_28606(C_word c,C_word *av) C_noret;
C_noret_decl(f_28614)
static void C_ccall f_28614(C_word c,C_word *av) C_noret;
C_noret_decl(f_28616)
static void C_ccall f_28616(C_word c,C_word *av) C_noret;
C_noret_decl(f_28632)
static void C_ccall f_28632(C_word c,C_word *av) C_noret;
C_noret_decl(f_28636)
static void C_ccall f_28636(C_word c,C_word *av) C_noret;
C_noret_decl(f_28642)
static void C_fcall f_28642(C_word t0,C_word t1) C_noret;
C_noret_decl(f_28647)
static void C_ccall f_28647(C_word c,C_word *av) C_noret;
C_noret_decl(f_28661)
static void C_ccall f_28661(C_word c,C_word *av) C_noret;
C_noret_decl(f_28705)
static void C_ccall f_28705(C_word c,C_word *av) C_noret;
C_noret_decl(f_28713)
static void C_fcall f_28713(C_word t0,C_word t1) C_noret;
C_noret_decl(f_28715)
static void C_fcall f_28715(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_28756)
static void C_ccall f_28756(C_word c,C_word *av) C_noret;
C_noret_decl(f_28796)
static void C_ccall f_28796(C_word c,C_word *av) C_noret;
C_noret_decl(f_28830)
static void C_ccall f_28830(C_word c,C_word *av) C_noret;
C_noret_decl(f_28843)
static void C_fcall f_28843(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28873)
static void C_ccall f_28873(C_word c,C_word *av) C_noret;
C_noret_decl(f_28881)
static void C_ccall f_28881(C_word c,C_word *av) C_noret;
C_noret_decl(f_28885)
static void C_ccall f_28885(C_word c,C_word *av) C_noret;
C_noret_decl(f_28887)
static void C_fcall f_28887(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_28906)
static void C_ccall f_28906(C_word c,C_word *av) C_noret;
C_noret_decl(f_28930)
static void C_ccall f_28930(C_word c,C_word *av) C_noret;
C_noret_decl(f_28947)
static void C_ccall f_28947(C_word c,C_word *av) C_noret;
C_noret_decl(f_28954)
static void C_ccall f_28954(C_word c,C_word *av) C_noret;
C_noret_decl(f_28977)
static void C_ccall f_28977(C_word c,C_word *av) C_noret;
C_noret_decl(f_28998)
static void C_ccall f_28998(C_word c,C_word *av) C_noret;
C_noret_decl(f_29000)
static void C_fcall f_29000(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29010)
static void C_fcall f_29010(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29037)
static void C_ccall f_29037(C_word c,C_word *av) C_noret;
C_noret_decl(f_29067)
static void C_ccall f_29067(C_word c,C_word *av) C_noret;
C_noret_decl(f_29072)
static void C_fcall f_29072(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_29084)
static void C_fcall f_29084(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_29104)
static void C_ccall f_29104(C_word c,C_word *av) C_noret;
C_noret_decl(f_29139)
static void C_ccall f_29139(C_word c,C_word *av) C_noret;
C_noret_decl(f_29147)
static void C_ccall f_29147(C_word c,C_word *av) C_noret;
C_noret_decl(f_29183)
static void C_ccall f_29183(C_word c,C_word *av) C_noret;
C_noret_decl(f_29197)
static void C_fcall f_29197(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29201)
static void C_ccall f_29201(C_word c,C_word *av) C_noret;
C_noret_decl(f_29204)
static void C_ccall f_29204(C_word c,C_word *av) C_noret;
C_noret_decl(f_29209)
static void C_fcall f_29209(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29219)
static void C_ccall f_29219(C_word c,C_word *av) C_noret;
C_noret_decl(f_29230)
static void C_ccall f_29230(C_word c,C_word *av) C_noret;
C_noret_decl(f_29233)
static void C_ccall f_29233(C_word c,C_word *av) C_noret;
C_noret_decl(f_29240)
static void C_ccall f_29240(C_word c,C_word *av) C_noret;
C_noret_decl(f_29252)
static void C_ccall f_29252(C_word c,C_word *av) C_noret;
C_noret_decl(f_29267)
static void C_ccall f_29267(C_word c,C_word *av) C_noret;
C_noret_decl(f_29277)
static void C_ccall f_29277(C_word c,C_word *av) C_noret;
C_noret_decl(f_29281)
static void C_ccall f_29281(C_word c,C_word *av) C_noret;
C_noret_decl(f_29284)
static void C_ccall f_29284(C_word c,C_word *av) C_noret;
C_noret_decl(f_29289)
static void C_fcall f_29289(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29312)
static void C_fcall f_29312(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29315)
static void C_ccall f_29315(C_word c,C_word *av) C_noret;
C_noret_decl(f_29322)
static void C_ccall f_29322(C_word c,C_word *av) C_noret;
C_noret_decl(f_29327)
static void C_fcall f_29327(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29337)
static void C_ccall f_29337(C_word c,C_word *av) C_noret;
C_noret_decl(f_29357)
static void C_fcall f_29357(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29367)
static void C_ccall f_29367(C_word c,C_word *av) C_noret;
C_noret_decl(f_29409)
static void C_ccall f_29409(C_word c,C_word *av) C_noret;
C_noret_decl(f_29416)
static void C_ccall f_29416(C_word c,C_word *av) C_noret;
C_noret_decl(f_29421)
static void C_fcall f_29421(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29431)
static void C_ccall f_29431(C_word c,C_word *av) C_noret;
C_noret_decl(f_29451)
static void C_ccall f_29451(C_word c,C_word *av) C_noret;
C_noret_decl(f_29454)
static void C_ccall f_29454(C_word c,C_word *av) C_noret;
C_noret_decl(f_29457)
static void C_ccall f_29457(C_word c,C_word *av) C_noret;
C_noret_decl(f_29466)
static void C_ccall f_29466(C_word c,C_word *av) C_noret;
C_noret_decl(f_29471)
static void C_fcall f_29471(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29481)
static void C_ccall f_29481(C_word c,C_word *av) C_noret;
C_noret_decl(f_29505)
static void C_ccall f_29505(C_word c,C_word *av) C_noret;
C_noret_decl(f_29508)
static void C_ccall f_29508(C_word c,C_word *av) C_noret;
C_noret_decl(f_29509)
static void C_fcall f_29509(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29513)
static void C_ccall f_29513(C_word c,C_word *av) C_noret;
C_noret_decl(f_29525)
static void C_ccall f_29525(C_word c,C_word *av) C_noret;
C_noret_decl(f_29530)
static void C_fcall f_29530(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29540)
static void C_ccall f_29540(C_word c,C_word *av) C_noret;
C_noret_decl(f_29564)
static void C_fcall f_29564(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29567)
static void C_ccall f_29567(C_word c,C_word *av) C_noret;
C_noret_decl(f_29578)
static void C_ccall f_29578(C_word c,C_word *av) C_noret;
C_noret_decl(f_29585)
static void C_ccall f_29585(C_word c,C_word *av) C_noret;
C_noret_decl(f_29590)
static void C_fcall f_29590(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29600)
static void C_ccall f_29600(C_word c,C_word *av) C_noret;
C_noret_decl(f_29634)
static void C_fcall f_29634(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29637)
static void C_ccall f_29637(C_word c,C_word *av) C_noret;
C_noret_decl(f_29640)
static void C_ccall f_29640(C_word c,C_word *av) C_noret;
C_noret_decl(f_29647)
static void C_ccall f_29647(C_word c,C_word *av) C_noret;
C_noret_decl(f_29651)
static void C_ccall f_29651(C_word c,C_word *av) C_noret;
C_noret_decl(f_29664)
static void C_ccall f_29664(C_word c,C_word *av) C_noret;
C_noret_decl(f_29677)
static void C_ccall f_29677(C_word c,C_word *av) C_noret;
C_noret_decl(f_29680)
static void C_ccall f_29680(C_word c,C_word *av) C_noret;
C_noret_decl(f_29687)
static void C_ccall f_29687(C_word c,C_word *av) C_noret;
C_noret_decl(f_29698)
static void C_fcall f_29698(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29707)
static void C_fcall f_29707(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29710)
static void C_ccall f_29710(C_word c,C_word *av) C_noret;
C_noret_decl(f_29713)
static void C_ccall f_29713(C_word c,C_word *av) C_noret;
C_noret_decl(f_29720)
static void C_ccall f_29720(C_word c,C_word *av) C_noret;
C_noret_decl(f_29745)
static void C_ccall f_29745(C_word c,C_word *av) C_noret;
C_noret_decl(f_29783)
static void C_ccall f_29783(C_word c,C_word *av) C_noret;
C_noret_decl(f_29786)
static void C_ccall f_29786(C_word c,C_word *av) C_noret;
C_noret_decl(f_29789)
static void C_ccall f_29789(C_word c,C_word *av) C_noret;
C_noret_decl(f_29796)
static void C_ccall f_29796(C_word c,C_word *av) C_noret;
C_noret_decl(f_29801)
static void C_fcall f_29801(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29811)
static void C_ccall f_29811(C_word c,C_word *av) C_noret;
C_noret_decl(f_29847)
static void C_ccall f_29847(C_word c,C_word *av) C_noret;
C_noret_decl(f_30011)
static void C_ccall f_30011(C_word c,C_word *av) C_noret;
C_noret_decl(f_30024)
static void C_ccall f_30024(C_word c,C_word *av) C_noret;
C_noret_decl(f_30033)
static void C_ccall f_30033(C_word c,C_word *av) C_noret;
C_noret_decl(f_30044)
static void C_ccall f_30044(C_word c,C_word *av) C_noret;
C_noret_decl(f_30048)
static void C_ccall f_30048(C_word c,C_word *av) C_noret;
C_noret_decl(f_30050)
static void C_fcall f_30050(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_30081)
static void C_ccall f_30081(C_word c,C_word *av) C_noret;
C_noret_decl(f_30128)
static void C_ccall f_30128(C_word c,C_word *av) C_noret;
C_noret_decl(f_30148)
static void C_ccall f_30148(C_word c,C_word *av) C_noret;
C_noret_decl(f_30158)
static void C_fcall f_30158(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_30185)
static void C_ccall f_30185(C_word c,C_word *av) C_noret;
C_noret_decl(f_30207)
static void C_fcall f_30207(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30309)
static void C_ccall f_30309(C_word c,C_word *av) C_noret;
C_noret_decl(f_30774)
static void C_ccall f_30774(C_word c,C_word *av) C_noret;
C_noret_decl(f_30936)
static void C_ccall f_30936(C_word c,C_word *av) C_noret;
C_noret_decl(f_30939)
static void C_ccall f_30939(C_word c,C_word *av) C_noret;
C_noret_decl(f_30973)
static void C_ccall f_30973(C_word c,C_word *av) C_noret;
C_noret_decl(f_30977)
static void C_ccall f_30977(C_word c,C_word *av) C_noret;
C_noret_decl(f_30980)
static void C_ccall f_30980(C_word c,C_word *av) C_noret;
C_noret_decl(f_30983)
static void C_ccall f_30983(C_word c,C_word *av) C_noret;
C_noret_decl(f_30986)
static void C_ccall f_30986(C_word c,C_word *av) C_noret;
C_noret_decl(f_30989)
static void C_ccall f_30989(C_word c,C_word *av) C_noret;
C_noret_decl(f_7535)
static void C_ccall f_7535(C_word c,C_word *av) C_noret;
C_noret_decl(f_7538)
static void C_ccall f_7538(C_word c,C_word *av) C_noret;
C_noret_decl(f_7540)
static void C_fcall f_7540(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7544)
static void C_ccall f_7544(C_word c,C_word *av) C_noret;
C_noret_decl(f_7547)
static void C_ccall f_7547(C_word c,C_word *av) C_noret;
C_noret_decl(f_7553)
static void C_ccall f_7553(C_word c,C_word *av) C_noret;
C_noret_decl(f_7595)
static void C_ccall f_7595(C_word c,C_word *av) C_noret;
C_noret_decl(f_7613)
static void C_ccall f_7613(C_word c,C_word *av) C_noret;
C_noret_decl(f_7622)
static void C_fcall f_7622(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7630)
static void C_ccall f_7630(C_word c,C_word *av) C_noret;
C_noret_decl(f_7643)
static void C_ccall f_7643(C_word c,C_word *av) C_noret;
C_noret_decl(f_7645)
static void C_fcall f_7645(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7652)
static void C_ccall f_7652(C_word c,C_word *av) C_noret;
C_noret_decl(f_7654)
static void C_fcall f_7654(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7668)
static void C_ccall f_7668(C_word c,C_word *av) C_noret;
C_noret_decl(f_7686)
static void C_ccall f_7686(C_word c,C_word *av) C_noret;
C_noret_decl(f_7692)
static void C_ccall f_7692(C_word c,C_word *av) C_noret;
C_noret_decl(f_7717)
static void C_ccall f_7717(C_word c,C_word *av) C_noret;
C_noret_decl(f_7763)
static void C_ccall f_7763(C_word c,C_word *av) C_noret;
C_noret_decl(f_7776)
static void C_ccall f_7776(C_word c,C_word *av) C_noret;
C_noret_decl(f_7778)
static void C_ccall f_7778(C_word c,C_word *av) C_noret;
C_noret_decl(f_7782)
static void C_ccall f_7782(C_word c,C_word *av) C_noret;
C_noret_decl(f_7803)
static void C_ccall f_7803(C_word c,C_word *av) C_noret;
C_noret_decl(f_7807)
static void C_ccall f_7807(C_word c,C_word *av) C_noret;
C_noret_decl(f_7832)
static void C_ccall f_7832(C_word c,C_word *av) C_noret;
C_noret_decl(f_7836)
static void C_ccall f_7836(C_word c,C_word *av) C_noret;
C_noret_decl(f_7861)
static void C_ccall f_7861(C_word c,C_word *av) C_noret;
C_noret_decl(f_7865)
static void C_ccall f_7865(C_word c,C_word *av) C_noret;
C_noret_decl(f_7954)
static void C_fcall f_7954(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8022)
static void C_fcall f_8022(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8064)
static void C_ccall f_8064(C_word c,C_word *av) C_noret;
C_noret_decl(f_8066)
static void C_fcall f_8066(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8136)
static void C_ccall f_8136(C_word c,C_word *av) C_noret;
C_noret_decl(f_8138)
static void C_ccall f_8138(C_word c,C_word *av) C_noret;
C_noret_decl(f_8142)
static void C_ccall f_8142(C_word c,C_word *av) C_noret;
C_noret_decl(f_8199)
static void C_ccall f_8199(C_word c,C_word *av) C_noret;
C_noret_decl(f_8201)
static void C_ccall f_8201(C_word c,C_word *av) C_noret;
C_noret_decl(f_8205)
static void C_ccall f_8205(C_word c,C_word *av) C_noret;
C_noret_decl(f_8282)
static void C_ccall f_8282(C_word c,C_word *av) C_noret;
C_noret_decl(f_8286)
static void C_ccall f_8286(C_word c,C_word *av) C_noret;
C_noret_decl(f_8372)
static void C_ccall f_8372(C_word c,C_word *av) C_noret;
C_noret_decl(f_8406)
static void C_ccall f_8406(C_word c,C_word *av) C_noret;
C_noret_decl(f_8412)
static void C_fcall f_8412(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8446)
static void C_ccall f_8446(C_word c,C_word *av) C_noret;
C_noret_decl(f_8460)
static void C_ccall f_8460(C_word c,C_word *av) C_noret;
C_noret_decl(f_8467)
static void C_ccall f_8467(C_word c,C_word *av) C_noret;
C_noret_decl(f_8473)
static void C_fcall f_8473(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8491)
static void C_ccall f_8491(C_word c,C_word *av) C_noret;
C_noret_decl(f_8495)
static void C_ccall f_8495(C_word c,C_word *av) C_noret;
C_noret_decl(f_8499)
static void C_ccall f_8499(C_word c,C_word *av) C_noret;
C_noret_decl(f_8506)
static void C_ccall f_8506(C_word c,C_word *av) C_noret;
C_noret_decl(f_8514)
static void C_ccall f_8514(C_word c,C_word *av) C_noret;
C_noret_decl(f_8518)
static void C_ccall f_8518(C_word c,C_word *av) C_noret;
C_noret_decl(f_8522)
static void C_ccall f_8522(C_word c,C_word *av) C_noret;
C_noret_decl(f_8526)
static void C_ccall f_8526(C_word c,C_word *av) C_noret;
C_noret_decl(f_8530)
static void C_ccall f_8530(C_word c,C_word *av) C_noret;
C_noret_decl(f_8534)
static void C_ccall f_8534(C_word c,C_word *av) C_noret;
C_noret_decl(f_8538)
static void C_ccall f_8538(C_word c,C_word *av) C_noret;
C_noret_decl(f_8547)
static void C_ccall f_8547(C_word c,C_word *av) C_noret;
C_noret_decl(f_8555)
static void C_ccall f_8555(C_word c,C_word *av) C_noret;
C_noret_decl(f_8564)
static void C_ccall f_8564(C_word c,C_word *av) C_noret;
C_noret_decl(f_8622)
static void C_fcall f_8622(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8626)
static void C_ccall f_8626(C_word c,C_word *av) C_noret;
C_noret_decl(f_8640)
static void C_fcall f_8640(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8644)
static void C_ccall f_8644(C_word c,C_word *av) C_noret;
C_noret_decl(f_8657)
static void C_ccall f_8657(C_word c,C_word *av) C_noret;
C_noret_decl(f_8668)
static void C_ccall f_8668(C_word c,C_word *av) C_noret;
C_noret_decl(f_8670)
static void C_fcall f_8670(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8677)
static void C_ccall f_8677(C_word c,C_word *av) C_noret;
C_noret_decl(f_8690)
static void C_ccall f_8690(C_word c,C_word *av) C_noret;
C_noret_decl(f_8697)
static void C_ccall f_8697(C_word c,C_word *av) C_noret;
C_noret_decl(f_8736)
static void C_ccall f_8736(C_word c,C_word *av) C_noret;
C_noret_decl(f_8763)
static void C_ccall f_8763(C_word c,C_word *av) C_noret;
C_noret_decl(f_8795)
static void C_ccall f_8795(C_word c,C_word *av) C_noret;
C_noret_decl(f_8802)
static void C_ccall f_8802(C_word c,C_word *av) C_noret;
C_noret_decl(f_8817)
static void C_ccall f_8817(C_word c,C_word *av) C_noret;
C_noret_decl(f_8830)
static void C_fcall f_8830(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8873)
static void C_fcall f_8873(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8930)
static void C_fcall f_8930(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8968)
static void C_fcall f_8968(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_8971)
static void C_fcall f_8971(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8979)
static void C_ccall f_8979(C_word c,C_word *av) C_noret;
C_noret_decl(f_8992)
static void C_ccall f_8992(C_word c,C_word *av) C_noret;
C_noret_decl(f_9011)
static void C_ccall f_9011(C_word c,C_word *av) C_noret;
C_noret_decl(f_9108)
static void C_fcall f_9108(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9116)
static void C_ccall f_9116(C_word c,C_word *av) C_noret;
C_noret_decl(f_9118)
static void C_ccall f_9118(C_word c,C_word *av) C_noret;
C_noret_decl(f_9132)
static void C_ccall f_9132(C_word c,C_word *av) C_noret;
C_noret_decl(f_9137)
static C_word C_fcall f_9137(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_9158)
static void C_fcall f_9158(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9174)
static void C_fcall f_9174(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9195)
static void C_fcall f_9195(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9201)
static void C_fcall f_9201(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9214)
static void C_ccall f_9214(C_word c,C_word *av) C_noret;
C_noret_decl(f_9256)
static void C_fcall f_9256(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9271)
static C_word C_fcall f_9271(C_word t0);
C_noret_decl(f_9294)
static void C_fcall f_9294(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9310)
static void C_fcall f_9310(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9323)
static void C_ccall f_9323(C_word c,C_word *av) C_noret;
C_noret_decl(f_9337)
static void C_fcall f_9337(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9355)
static void C_fcall f_9355(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9371)
static void C_ccall f_9371(C_word c,C_word *av) C_noret;
C_noret_decl(f_9382)
static void C_fcall f_9382(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9388)
static void C_fcall f_9388(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9406)
static void C_ccall f_9406(C_word c,C_word *av) C_noret;
C_noret_decl(f_9712)
static void C_ccall f_9712(C_word c,C_word *av) C_noret;
C_noret_decl(f_9721)
static void C_ccall f_9721(C_word c,C_word *av) C_noret;
C_noret_decl(f_9739)
static void C_fcall f_9739(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9746)
static void C_fcall f_9746(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9773)
static void C_fcall f_9773(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9779)
static void C_ccall f_9779(C_word c,C_word *av) C_noret;
C_noret_decl(f_9793)
static void C_ccall f_9793(C_word c,C_word *av) C_noret;
C_noret_decl(f_9813)
static void C_ccall f_9813(C_word c,C_word *av) C_noret;
C_noret_decl(f_9819)
static void C_ccall f_9819(C_word c,C_word *av) C_noret;
C_noret_decl(f_9836)
static void C_ccall f_9836(C_word c,C_word *av) C_noret;
C_noret_decl(f_9840)
static void C_ccall f_9840(C_word c,C_word *av) C_noret;
C_noret_decl(f_9887)
static void C_fcall f_9887(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9936)
static void C_ccall f_9936(C_word c,C_word *av) C_noret;
C_noret_decl(f_9964)
static void C_fcall f_9964(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9975)
static void C_ccall f_9975(C_word c,C_word *av) C_noret;
C_noret_decl(C_irregex_toplevel)
C_externexport void C_ccall C_irregex_toplevel(C_word c,C_word *av) C_noret;

C_noret_decl(trf_10021)
static void C_ccall trf_10021(C_word c,C_word *av) C_noret;
static void C_ccall trf_10021(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10021(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10024)
static void C_ccall trf_10024(C_word c,C_word *av) C_noret;
static void C_ccall trf_10024(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10024(t0,t1);}

C_noret_decl(trf_10240)
static void C_ccall trf_10240(C_word c,C_word *av) C_noret;
static void C_ccall trf_10240(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10240(t0,t1,t2);}

C_noret_decl(trf_10250)
static void C_ccall trf_10250(C_word c,C_word *av) C_noret;
static void C_ccall trf_10250(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10250(t0,t1);}

C_noret_decl(trf_10332)
static void C_ccall trf_10332(C_word c,C_word *av) C_noret;
static void C_ccall trf_10332(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10332(t0,t1);}

C_noret_decl(trf_10338)
static void C_ccall trf_10338(C_word c,C_word *av) C_noret;
static void C_ccall trf_10338(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10338(t0,t1,t2,t3);}

C_noret_decl(trf_10356)
static void C_ccall trf_10356(C_word c,C_word *av) C_noret;
static void C_ccall trf_10356(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10356(t0,t1);}

C_noret_decl(trf_10367)
static void C_ccall trf_10367(C_word c,C_word *av) C_noret;
static void C_ccall trf_10367(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10367(t0,t1);}

C_noret_decl(trf_10482)
static void C_ccall trf_10482(C_word c,C_word *av) C_noret;
static void C_ccall trf_10482(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_10482(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_10507)
static void C_ccall trf_10507(C_word c,C_word *av) C_noret;
static void C_ccall trf_10507(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10507(t0,t1,t2);}

C_noret_decl(trf_10529)
static void C_ccall trf_10529(C_word c,C_word *av) C_noret;
static void C_ccall trf_10529(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10529(t0,t1,t2);}

C_noret_decl(trf_10563)
static void C_ccall trf_10563(C_word c,C_word *av) C_noret;
static void C_ccall trf_10563(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10563(t0,t1);}

C_noret_decl(trf_10583)
static void C_ccall trf_10583(C_word c,C_word *av) C_noret;
static void C_ccall trf_10583(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10583(t0,t1);}

C_noret_decl(trf_10599)
static void C_ccall trf_10599(C_word c,C_word *av) C_noret;
static void C_ccall trf_10599(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10599(t0,t1);}

C_noret_decl(trf_10655)
static void C_ccall trf_10655(C_word c,C_word *av) C_noret;
static void C_ccall trf_10655(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10655(t0,t1);}

C_noret_decl(trf_10662)
static void C_ccall trf_10662(C_word c,C_word *av) C_noret;
static void C_ccall trf_10662(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10662(t0,t1);}

C_noret_decl(trf_10665)
static void C_ccall trf_10665(C_word c,C_word *av) C_noret;
static void C_ccall trf_10665(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10665(t0,t1);}

C_noret_decl(trf_10673)
static void C_ccall trf_10673(C_word c,C_word *av) C_noret;
static void C_ccall trf_10673(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10673(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10888)
static void C_ccall trf_10888(C_word c,C_word *av) C_noret;
static void C_ccall trf_10888(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10888(t0,t1);}

C_noret_decl(trf_10989)
static void C_ccall trf_10989(C_word c,C_word *av) C_noret;
static void C_ccall trf_10989(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10989(t0,t1);}

C_noret_decl(trf_11252)
static void C_ccall trf_11252(C_word c,C_word *av) C_noret;
static void C_ccall trf_11252(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11252(t0,t1);}

C_noret_decl(trf_11709)
static void C_ccall trf_11709(C_word c,C_word *av) C_noret;
static void C_ccall trf_11709(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_11709(t0,t1,t2,t3,t4);}

C_noret_decl(trf_12045)
static void C_ccall trf_12045(C_word c,C_word *av) C_noret;
static void C_ccall trf_12045(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12045(t0,t1);}

C_noret_decl(trf_12088)
static void C_ccall trf_12088(C_word c,C_word *av) C_noret;
static void C_ccall trf_12088(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12088(t0,t1);}

C_noret_decl(trf_12695)
static void C_ccall trf_12695(C_word c,C_word *av) C_noret;
static void C_ccall trf_12695(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12695(t0,t1);}

C_noret_decl(trf_12775)
static void C_ccall trf_12775(C_word c,C_word *av) C_noret;
static void C_ccall trf_12775(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12775(t0,t1,t2);}

C_noret_decl(trf_13295)
static void C_ccall trf_13295(C_word c,C_word *av) C_noret;
static void C_ccall trf_13295(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13295(t0,t1,t2,t3);}

C_noret_decl(trf_13404)
static void C_ccall trf_13404(C_word c,C_word *av) C_noret;
static void C_ccall trf_13404(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13404(t0,t1,t2,t3,t4);}

C_noret_decl(trf_13477)
static void C_ccall trf_13477(C_word c,C_word *av) C_noret;
static void C_ccall trf_13477(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13477(t0,t1);}

C_noret_decl(trf_13554)
static void C_ccall trf_13554(C_word c,C_word *av) C_noret;
static void C_ccall trf_13554(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13554(t0,t1);}

C_noret_decl(trf_13566)
static void C_ccall trf_13566(C_word c,C_word *av) C_noret;
static void C_ccall trf_13566(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13566(t0,t1);}

C_noret_decl(trf_13688)
static void C_ccall trf_13688(C_word c,C_word *av) C_noret;
static void C_ccall trf_13688(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13688(t0,t1);}

C_noret_decl(trf_13801)
static void C_ccall trf_13801(C_word c,C_word *av) C_noret;
static void C_ccall trf_13801(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13801(t0,t1);}

C_noret_decl(trf_13918)
static void C_ccall trf_13918(C_word c,C_word *av) C_noret;
static void C_ccall trf_13918(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13918(t0,t1);}

C_noret_decl(trf_14029)
static void C_ccall trf_14029(C_word c,C_word *av) C_noret;
static void C_ccall trf_14029(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14029(t0,t1,t2,t3);}

C_noret_decl(trf_14224)
static void C_ccall trf_14224(C_word c,C_word *av) C_noret;
static void C_ccall trf_14224(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14224(t0,t1,t2);}

C_noret_decl(trf_14262)
static void C_ccall trf_14262(C_word c,C_word *av) C_noret;
static void C_ccall trf_14262(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14262(t0,t1);}

C_noret_decl(trf_14338)
static void C_ccall trf_14338(C_word c,C_word *av) C_noret;
static void C_ccall trf_14338(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14338(t0,t1);}

C_noret_decl(trf_14508)
static void C_ccall trf_14508(C_word c,C_word *av) C_noret;
static void C_ccall trf_14508(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14508(t0,t1,t2,t3);}

C_noret_decl(trf_14614)
static void C_ccall trf_14614(C_word c,C_word *av) C_noret;
static void C_ccall trf_14614(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_14614(t0,t1,t2,t3,t4);}

C_noret_decl(trf_14680)
static void C_ccall trf_14680(C_word c,C_word *av) C_noret;
static void C_ccall trf_14680(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14680(t0,t1,t2);}

C_noret_decl(trf_14716)
static void C_ccall trf_14716(C_word c,C_word *av) C_noret;
static void C_ccall trf_14716(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14716(t0,t1,t2);}

C_noret_decl(trf_14752)
static void C_ccall trf_14752(C_word c,C_word *av) C_noret;
static void C_ccall trf_14752(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14752(t0,t1);}

C_noret_decl(trf_14837)
static void C_ccall trf_14837(C_word c,C_word *av) C_noret;
static void C_ccall trf_14837(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14837(t0,t1,t2);}

C_noret_decl(trf_14891)
static void C_ccall trf_14891(C_word c,C_word *av) C_noret;
static void C_ccall trf_14891(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14891(t0,t1);}

C_noret_decl(trf_14976)
static void C_ccall trf_14976(C_word c,C_word *av) C_noret;
static void C_ccall trf_14976(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14976(t0,t1,t2);}

C_noret_decl(trf_15050)
static void C_ccall trf_15050(C_word c,C_word *av) C_noret;
static void C_ccall trf_15050(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15050(t0,t1,t2);}

C_noret_decl(trf_15062)
static void C_ccall trf_15062(C_word c,C_word *av) C_noret;
static void C_ccall trf_15062(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15062(t0,t1);}

C_noret_decl(trf_15082)
static void C_ccall trf_15082(C_word c,C_word *av) C_noret;
static void C_ccall trf_15082(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15082(t0,t1,t2);}

C_noret_decl(trf_15185)
static void C_ccall trf_15185(C_word c,C_word *av) C_noret;
static void C_ccall trf_15185(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15185(t0,t1,t2);}

C_noret_decl(trf_15253)
static void C_ccall trf_15253(C_word c,C_word *av) C_noret;
static void C_ccall trf_15253(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15253(t0,t1,t2);}

C_noret_decl(trf_15323)
static void C_ccall trf_15323(C_word c,C_word *av) C_noret;
static void C_ccall trf_15323(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15323(t0,t1,t2);}

C_noret_decl(trf_15414)
static void C_ccall trf_15414(C_word c,C_word *av) C_noret;
static void C_ccall trf_15414(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15414(t0,t1,t2);}

C_noret_decl(trf_15469)
static void C_ccall trf_15469(C_word c,C_word *av) C_noret;
static void C_ccall trf_15469(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15469(t0,t1);}

C_noret_decl(trf_15479)
static void C_ccall trf_15479(C_word c,C_word *av) C_noret;
static void C_ccall trf_15479(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15479(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15524)
static void C_ccall trf_15524(C_word c,C_word *av) C_noret;
static void C_ccall trf_15524(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15524(t0,t1);}

C_noret_decl(trf_15591)
static void C_ccall trf_15591(C_word c,C_word *av) C_noret;
static void C_ccall trf_15591(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15591(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15594)
static void C_ccall trf_15594(C_word c,C_word *av) C_noret;
static void C_ccall trf_15594(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15594(t0,t1,t2);}

C_noret_decl(trf_15651)
static void C_ccall trf_15651(C_word c,C_word *av) C_noret;
static void C_ccall trf_15651(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15651(t0,t1,t2);}

C_noret_decl(trf_15664)
static void C_ccall trf_15664(C_word c,C_word *av) C_noret;
static void C_ccall trf_15664(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15664(t0,t1,t2);}

C_noret_decl(trf_15712)
static void C_ccall trf_15712(C_word c,C_word *av) C_noret;
static void C_ccall trf_15712(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15712(t0,t1,t2);}

C_noret_decl(trf_15725)
static void C_ccall trf_15725(C_word c,C_word *av) C_noret;
static void C_ccall trf_15725(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15725(t0,t1,t2);}

C_noret_decl(trf_15766)
static void C_ccall trf_15766(C_word c,C_word *av) C_noret;
static void C_ccall trf_15766(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15766(t0,t1);}

C_noret_decl(trf_15835)
static void C_ccall trf_15835(C_word c,C_word *av) C_noret;
static void C_ccall trf_15835(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15835(t0,t1,t2);}

C_noret_decl(trf_15891)
static void C_ccall trf_15891(C_word c,C_word *av) C_noret;
static void C_ccall trf_15891(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15891(t0,t1,t2);}

C_noret_decl(trf_15974)
static void C_ccall trf_15974(C_word c,C_word *av) C_noret;
static void C_ccall trf_15974(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15974(t0,t1);}

C_noret_decl(trf_15989)
static void C_ccall trf_15989(C_word c,C_word *av) C_noret;
static void C_ccall trf_15989(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15989(t0,t1,t2);}

C_noret_decl(trf_16239)
static void C_ccall trf_16239(C_word c,C_word *av) C_noret;
static void C_ccall trf_16239(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16239(t0,t1,t2);}

C_noret_decl(trf_16269)
static void C_ccall trf_16269(C_word c,C_word *av) C_noret;
static void C_ccall trf_16269(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16269(t0,t1,t2);}

C_noret_decl(trf_16324)
static void C_ccall trf_16324(C_word c,C_word *av) C_noret;
static void C_ccall trf_16324(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16324(t0,t1);}

C_noret_decl(trf_16367)
static void C_ccall trf_16367(C_word c,C_word *av) C_noret;
static void C_ccall trf_16367(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16367(t0,t1);}

C_noret_decl(trf_16497)
static void C_ccall trf_16497(C_word c,C_word *av) C_noret;
static void C_ccall trf_16497(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16497(t0,t1);}

C_noret_decl(trf_16637)
static void C_ccall trf_16637(C_word c,C_word *av) C_noret;
static void C_ccall trf_16637(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16637(t0,t1);}

C_noret_decl(trf_16750)
static void C_ccall trf_16750(C_word c,C_word *av) C_noret;
static void C_ccall trf_16750(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16750(t0,t1);}

C_noret_decl(trf_16867)
static void C_ccall trf_16867(C_word c,C_word *av) C_noret;
static void C_ccall trf_16867(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16867(t0,t1);}

C_noret_decl(trf_16982)
static void C_ccall trf_16982(C_word c,C_word *av) C_noret;
static void C_ccall trf_16982(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16982(t0,t1);}

C_noret_decl(trf_17015)
static void C_ccall trf_17015(C_word c,C_word *av) C_noret;
static void C_ccall trf_17015(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17015(t0,t1);}

C_noret_decl(trf_17094)
static void C_ccall trf_17094(C_word c,C_word *av) C_noret;
static void C_ccall trf_17094(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_17094(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_17097)
static void C_ccall trf_17097(C_word c,C_word *av) C_noret;
static void C_ccall trf_17097(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17097(t0,t1,t2);}

C_noret_decl(trf_17137)
static void C_ccall trf_17137(C_word c,C_word *av) C_noret;
static void C_ccall trf_17137(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17137(t0,t1);}

C_noret_decl(trf_17166)
static void C_ccall trf_17166(C_word c,C_word *av) C_noret;
static void C_ccall trf_17166(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17166(t0,t1);}

C_noret_decl(trf_17173)
static void C_ccall trf_17173(C_word c,C_word *av) C_noret;
static void C_ccall trf_17173(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_17173(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_17251)
static void C_ccall trf_17251(C_word c,C_word *av) C_noret;
static void C_ccall trf_17251(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_17251(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_17354)
static void C_ccall trf_17354(C_word c,C_word *av) C_noret;
static void C_ccall trf_17354(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17354(t0,t1);}

C_noret_decl(trf_17490)
static void C_ccall trf_17490(C_word c,C_word *av) C_noret;
static void C_ccall trf_17490(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17490(t0,t1);}

C_noret_decl(trf_17548)
static void C_ccall trf_17548(C_word c,C_word *av) C_noret;
static void C_ccall trf_17548(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17548(t0,t1);}

C_noret_decl(trf_17675)
static void C_ccall trf_17675(C_word c,C_word *av) C_noret;
static void C_ccall trf_17675(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17675(t0,t1);}

C_noret_decl(trf_17678)
static void C_ccall trf_17678(C_word c,C_word *av) C_noret;
static void C_ccall trf_17678(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17678(t0,t1);}

C_noret_decl(trf_17870)
static void C_ccall trf_17870(C_word c,C_word *av) C_noret;
static void C_ccall trf_17870(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17870(t0,t1);}

C_noret_decl(trf_17889)
static void C_ccall trf_17889(C_word c,C_word *av) C_noret;
static void C_ccall trf_17889(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17889(t0,t1);}

C_noret_decl(trf_17910)
static void C_ccall trf_17910(C_word c,C_word *av) C_noret;
static void C_ccall trf_17910(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17910(t0,t1);}

C_noret_decl(trf_17920)
static void C_ccall trf_17920(C_word c,C_word *av) C_noret;
static void C_ccall trf_17920(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17920(t0,t1,t2);}

C_noret_decl(trf_18310)
static void C_ccall trf_18310(C_word c,C_word *av) C_noret;
static void C_ccall trf_18310(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18310(t0,t1,t2);}

C_noret_decl(trf_18370)
static void C_ccall trf_18370(C_word c,C_word *av) C_noret;
static void C_ccall trf_18370(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18370(t0,t1,t2,t3);}

C_noret_decl(trf_18418)
static void C_ccall trf_18418(C_word c,C_word *av) C_noret;
static void C_ccall trf_18418(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18418(t0,t1,t2,t3);}

C_noret_decl(trf_18520)
static void C_ccall trf_18520(C_word c,C_word *av) C_noret;
static void C_ccall trf_18520(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18520(t0,t1);}

C_noret_decl(trf_18704)
static void C_ccall trf_18704(C_word c,C_word *av) C_noret;
static void C_ccall trf_18704(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18704(t0,t1,t2,t3);}

C_noret_decl(trf_18753)
static void C_ccall trf_18753(C_word c,C_word *av) C_noret;
static void C_ccall trf_18753(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18753(t0,t1);}

C_noret_decl(trf_18825)
static void C_ccall trf_18825(C_word c,C_word *av) C_noret;
static void C_ccall trf_18825(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18825(t0,t1,t2);}

C_noret_decl(trf_18933)
static void C_ccall trf_18933(C_word c,C_word *av) C_noret;
static void C_ccall trf_18933(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18933(t0,t1);}

C_noret_decl(trf_19053)
static void C_ccall trf_19053(C_word c,C_word *av) C_noret;
static void C_ccall trf_19053(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_19053(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_19114)
static void C_ccall trf_19114(C_word c,C_word *av) C_noret;
static void C_ccall trf_19114(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19114(t0,t1,t2,t3);}

C_noret_decl(trf_19123)
static void C_ccall trf_19123(C_word c,C_word *av) C_noret;
static void C_ccall trf_19123(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19123(t0,t1,t2);}

C_noret_decl(trf_19228)
static void C_ccall trf_19228(C_word c,C_word *av) C_noret;
static void C_ccall trf_19228(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_19228(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_19315)
static void C_ccall trf_19315(C_word c,C_word *av) C_noret;
static void C_ccall trf_19315(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19315(t0,t1);}

C_noret_decl(trf_19518)
static void C_ccall trf_19518(C_word c,C_word *av) C_noret;
static void C_ccall trf_19518(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19518(t0,t1,t2);}

C_noret_decl(trf_19528)
static void C_ccall trf_19528(C_word c,C_word *av) C_noret;
static void C_ccall trf_19528(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19528(t0,t1);}

C_noret_decl(trf_19678)
static void C_ccall trf_19678(C_word c,C_word *av) C_noret;
static void C_ccall trf_19678(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_19678(t0,t1,t2,t3,t4);}

C_noret_decl(trf_19693)
static void C_ccall trf_19693(C_word c,C_word *av) C_noret;
static void C_ccall trf_19693(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19693(t0,t1,t2,t3);}

C_noret_decl(trf_19816)
static void C_ccall trf_19816(C_word c,C_word *av) C_noret;
static void C_ccall trf_19816(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19816(t0,t1,t2,t3);}

C_noret_decl(trf_19876)
static void C_ccall trf_19876(C_word c,C_word *av) C_noret;
static void C_ccall trf_19876(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19876(t0,t1,t2);}

C_noret_decl(trf_19913)
static void C_ccall trf_19913(C_word c,C_word *av) C_noret;
static void C_ccall trf_19913(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19913(t0,t1,t2);}

C_noret_decl(trf_19935)
static void C_ccall trf_19935(C_word c,C_word *av) C_noret;
static void C_ccall trf_19935(C_word c,C_word *av){
C_word t0=av[8];
C_word t1=av[7];
C_word t2=av[6];
C_word t3=av[5];
C_word t4=av[4];
C_word t5=av[3];
C_word t6=av[2];
C_word t7=av[1];
C_word t8=av[0];
f_19935(t0,t1,t2,t3,t4,t5,t6,t7,t8);}

C_noret_decl(trf_19972)
static void C_ccall trf_19972(C_word c,C_word *av) C_noret;
static void C_ccall trf_19972(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19972(t0,t1);}

C_noret_decl(trf_19988)
static void C_ccall trf_19988(C_word c,C_word *av) C_noret;
static void C_ccall trf_19988(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_19988(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_20000)
static void C_ccall trf_20000(C_word c,C_word *av) C_noret;
static void C_ccall trf_20000(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_20000(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_20010)
static void C_ccall trf_20010(C_word c,C_word *av) C_noret;
static void C_ccall trf_20010(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20010(t0,t1);}

C_noret_decl(trf_20185)
static void C_ccall trf_20185(C_word c,C_word *av) C_noret;
static void C_ccall trf_20185(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20185(t0,t1,t2);}

C_noret_decl(trf_20226)
static void C_ccall trf_20226(C_word c,C_word *av) C_noret;
static void C_ccall trf_20226(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20226(t0,t1,t2);}

C_noret_decl(trf_20249)
static void C_ccall trf_20249(C_word c,C_word *av) C_noret;
static void C_ccall trf_20249(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20249(t0,t1,t2);}

C_noret_decl(trf_20283)
static void C_ccall trf_20283(C_word c,C_word *av) C_noret;
static void C_ccall trf_20283(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20283(t0,t1,t2);}

C_noret_decl(trf_20319)
static void C_ccall trf_20319(C_word c,C_word *av) C_noret;
static void C_ccall trf_20319(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20319(t0,t1);}

C_noret_decl(trf_20361)
static void C_ccall trf_20361(C_word c,C_word *av) C_noret;
static void C_ccall trf_20361(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20361(t0,t1);}

C_noret_decl(trf_20457)
static void C_ccall trf_20457(C_word c,C_word *av) C_noret;
static void C_ccall trf_20457(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20457(t0,t1,t2);}

C_noret_decl(trf_20757)
static void C_ccall trf_20757(C_word c,C_word *av) C_noret;
static void C_ccall trf_20757(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20757(t0,t1,t2);}

C_noret_decl(trf_20766)
static void C_ccall trf_20766(C_word c,C_word *av) C_noret;
static void C_ccall trf_20766(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_20766(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_20769)
static void C_ccall trf_20769(C_word c,C_word *av) C_noret;
static void C_ccall trf_20769(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20769(t0,t1,t2);}

C_noret_decl(trf_20779)
static void C_ccall trf_20779(C_word c,C_word *av) C_noret;
static void C_ccall trf_20779(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20779(t0,t1,t2,t3);}

C_noret_decl(trf_20783)
static void C_ccall trf_20783(C_word c,C_word *av) C_noret;
static void C_ccall trf_20783(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20783(t0,t1);}

C_noret_decl(trf_20804)
static void C_ccall trf_20804(C_word c,C_word *av) C_noret;
static void C_ccall trf_20804(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20804(t0,t1,t2);}

C_noret_decl(trf_20840)
static void C_ccall trf_20840(C_word c,C_word *av) C_noret;
static void C_ccall trf_20840(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20840(t0,t1,t2,t3);}

C_noret_decl(trf_20857)
static void C_ccall trf_20857(C_word c,C_word *av) C_noret;
static void C_ccall trf_20857(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20857(t0,t1,t2,t3);}

C_noret_decl(trf_20945)
static void C_ccall trf_20945(C_word c,C_word *av) C_noret;
static void C_ccall trf_20945(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20945(t0,t1,t2,t3);}

C_noret_decl(trf_21109)
static void C_ccall trf_21109(C_word c,C_word *av) C_noret;
static void C_ccall trf_21109(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21109(t0,t1);}

C_noret_decl(trf_21156)
static void C_ccall trf_21156(C_word c,C_word *av) C_noret;
static void C_ccall trf_21156(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21156(t0,t1);}

C_noret_decl(trf_21514)
static void C_ccall trf_21514(C_word c,C_word *av) C_noret;
static void C_ccall trf_21514(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21514(t0,t1,t2);}

C_noret_decl(trf_21677)
static void C_ccall trf_21677(C_word c,C_word *av) C_noret;
static void C_ccall trf_21677(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21677(t0,t1,t2);}

C_noret_decl(trf_21716)
static void C_ccall trf_21716(C_word c,C_word *av) C_noret;
static void C_ccall trf_21716(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_21716(t0,t1,t2,t3,t4);}

C_noret_decl(trf_21751)
static void C_ccall trf_21751(C_word c,C_word *av) C_noret;
static void C_ccall trf_21751(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21751(t0,t1);}

C_noret_decl(trf_21880)
static void C_ccall trf_21880(C_word c,C_word *av) C_noret;
static void C_ccall trf_21880(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21880(t0,t1);}

C_noret_decl(trf_21909)
static void C_ccall trf_21909(C_word c,C_word *av) C_noret;
static void C_ccall trf_21909(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21909(t0,t1);}

C_noret_decl(trf_21926)
static void C_ccall trf_21926(C_word c,C_word *av) C_noret;
static void C_ccall trf_21926(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21926(t0,t1,t2,t3);}

C_noret_decl(trf_21935)
static void C_ccall trf_21935(C_word c,C_word *av) C_noret;
static void C_ccall trf_21935(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_21935(t0,t1,t2,t3,t4);}

C_noret_decl(trf_21942)
static void C_ccall trf_21942(C_word c,C_word *av) C_noret;
static void C_ccall trf_21942(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21942(t0,t1);}

C_noret_decl(trf_21997)
static void C_ccall trf_21997(C_word c,C_word *av) C_noret;
static void C_ccall trf_21997(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21997(t0,t1,t2);}

C_noret_decl(trf_22104)
static void C_ccall trf_22104(C_word c,C_word *av) C_noret;
static void C_ccall trf_22104(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22104(t0,t1);}

C_noret_decl(trf_22110)
static void C_ccall trf_22110(C_word c,C_word *av) C_noret;
static void C_ccall trf_22110(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22110(t0,t1);}

C_noret_decl(trf_22163)
static void C_ccall trf_22163(C_word c,C_word *av) C_noret;
static void C_ccall trf_22163(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22163(t0,t1,t2,t3);}

C_noret_decl(trf_22234)
static void C_ccall trf_22234(C_word c,C_word *av) C_noret;
static void C_ccall trf_22234(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22234(t0,t1,t2);}

C_noret_decl(trf_22287)
static void C_ccall trf_22287(C_word c,C_word *av) C_noret;
static void C_ccall trf_22287(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22287(t0,t1,t2,t3);}

C_noret_decl(trf_22296)
static void C_ccall trf_22296(C_word c,C_word *av) C_noret;
static void C_ccall trf_22296(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22296(t0,t1,t2,t3);}

C_noret_decl(trf_22369)
static void C_ccall trf_22369(C_word c,C_word *av) C_noret;
static void C_ccall trf_22369(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22369(t0,t1,t2);}

C_noret_decl(trf_22401)
static void C_ccall trf_22401(C_word c,C_word *av) C_noret;
static void C_ccall trf_22401(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22401(t0,t1,t2,t3,t4);}

C_noret_decl(trf_22457)
static void C_ccall trf_22457(C_word c,C_word *av) C_noret;
static void C_ccall trf_22457(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22457(t0,t1,t2,t3,t4);}

C_noret_decl(trf_22488)
static void C_ccall trf_22488(C_word c,C_word *av) C_noret;
static void C_ccall trf_22488(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22488(t0,t1);}

C_noret_decl(trf_22588)
static void C_ccall trf_22588(C_word c,C_word *av) C_noret;
static void C_ccall trf_22588(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22588(t0,t1,t2,t3);}

C_noret_decl(trf_22621)
static void C_ccall trf_22621(C_word c,C_word *av) C_noret;
static void C_ccall trf_22621(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22621(t0,t1,t2);}

C_noret_decl(trf_22629)
static void C_ccall trf_22629(C_word c,C_word *av) C_noret;
static void C_ccall trf_22629(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22629(t0,t1);}

C_noret_decl(trf_22664)
static void C_ccall trf_22664(C_word c,C_word *av) C_noret;
static void C_ccall trf_22664(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22664(t0,t1,t2);}

C_noret_decl(trf_22702)
static void C_ccall trf_22702(C_word c,C_word *av) C_noret;
static void C_ccall trf_22702(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22702(t0,t1,t2,t3,t4);}

C_noret_decl(trf_22777)
static void C_ccall trf_22777(C_word c,C_word *av) C_noret;
static void C_ccall trf_22777(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_22777(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_22818)
static void C_ccall trf_22818(C_word c,C_word *av) C_noret;
static void C_ccall trf_22818(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22818(t0,t1,t2);}

C_noret_decl(trf_22834)
static void C_ccall trf_22834(C_word c,C_word *av) C_noret;
static void C_ccall trf_22834(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22834(t0,t1);}

C_noret_decl(trf_22936)
static void C_ccall trf_22936(C_word c,C_word *av) C_noret;
static void C_ccall trf_22936(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22936(t0,t1,t2,t3,t4);}

C_noret_decl(trf_22959)
static void C_ccall trf_22959(C_word c,C_word *av) C_noret;
static void C_ccall trf_22959(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_22959(t0,t1,t2,t3);}

C_noret_decl(trf_22988)
static void C_ccall trf_22988(C_word c,C_word *av) C_noret;
static void C_ccall trf_22988(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22988(t0,t1,t2);}

C_noret_decl(trf_22992)
static void C_ccall trf_22992(C_word c,C_word *av) C_noret;
static void C_ccall trf_22992(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22992(t0,t1);}

C_noret_decl(trf_23016)
static void C_ccall trf_23016(C_word c,C_word *av) C_noret;
static void C_ccall trf_23016(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23016(t0,t1);}

C_noret_decl(trf_23054)
static void C_ccall trf_23054(C_word c,C_word *av) C_noret;
static void C_ccall trf_23054(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23054(t0,t1,t2);}

C_noret_decl(trf_23069)
static void C_ccall trf_23069(C_word c,C_word *av) C_noret;
static void C_ccall trf_23069(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_23069(t0,t1,t2,t3,t4);}

C_noret_decl(trf_23084)
static void C_ccall trf_23084(C_word c,C_word *av) C_noret;
static void C_ccall trf_23084(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_23084(t0,t1,t2,t3);}

C_noret_decl(trf_23104)
static void C_ccall trf_23104(C_word c,C_word *av) C_noret;
static void C_ccall trf_23104(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_23104(t0,t1,t2,t3,t4);}

C_noret_decl(trf_23127)
static void C_ccall trf_23127(C_word c,C_word *av) C_noret;
static void C_ccall trf_23127(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23127(t0,t1);}

C_noret_decl(trf_23236)
static void C_ccall trf_23236(C_word c,C_word *av) C_noret;
static void C_ccall trf_23236(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23236(t0,t1,t2);}

C_noret_decl(trf_23249)
static void C_ccall trf_23249(C_word c,C_word *av) C_noret;
static void C_ccall trf_23249(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23249(t0,t1);}

C_noret_decl(trf_23263)
static void C_ccall trf_23263(C_word c,C_word *av) C_noret;
static void C_ccall trf_23263(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_23263(t0,t1,t2,t3,t4);}

C_noret_decl(trf_23290)
static void C_ccall trf_23290(C_word c,C_word *av) C_noret;
static void C_ccall trf_23290(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_23290(t0,t1,t2,t3,t4);}

C_noret_decl(trf_23446)
static void C_ccall trf_23446(C_word c,C_word *av) C_noret;
static void C_ccall trf_23446(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_23446(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_23449)
static void C_ccall trf_23449(C_word c,C_word *av) C_noret;
static void C_ccall trf_23449(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23449(t0,t1,t2);}

C_noret_decl(trf_23488)
static void C_ccall trf_23488(C_word c,C_word *av) C_noret;
static void C_ccall trf_23488(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23488(t0,t1);}

C_noret_decl(trf_23987)
static void C_ccall trf_23987(C_word c,C_word *av) C_noret;
static void C_ccall trf_23987(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23987(t0,t1);}

C_noret_decl(trf_23990)
static void C_ccall trf_23990(C_word c,C_word *av) C_noret;
static void C_ccall trf_23990(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_23990(t0,t1);}

C_noret_decl(trf_24139)
static void C_ccall trf_24139(C_word c,C_word *av) C_noret;
static void C_ccall trf_24139(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24139(t0,t1);}

C_noret_decl(trf_24142)
static void C_ccall trf_24142(C_word c,C_word *av) C_noret;
static void C_ccall trf_24142(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24142(t0,t1);}

C_noret_decl(trf_24579)
static void C_ccall trf_24579(C_word c,C_word *av) C_noret;
static void C_ccall trf_24579(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24579(t0,t1);}

C_noret_decl(trf_24723)
static void C_ccall trf_24723(C_word c,C_word *av) C_noret;
static void C_ccall trf_24723(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24723(t0,t1);}

C_noret_decl(trf_24746)
static void C_ccall trf_24746(C_word c,C_word *av) C_noret;
static void C_ccall trf_24746(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_24746(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_25519)
static void C_ccall trf_25519(C_word c,C_word *av) C_noret;
static void C_ccall trf_25519(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25519(t0,t1);}

C_noret_decl(trf_25528)
static void C_ccall trf_25528(C_word c,C_word *av) C_noret;
static void C_ccall trf_25528(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25528(t0,t1);}

C_noret_decl(trf_25626)
static void C_ccall trf_25626(C_word c,C_word *av) C_noret;
static void C_ccall trf_25626(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25626(t0,t1);}

C_noret_decl(trf_25659)
static void C_ccall trf_25659(C_word c,C_word *av) C_noret;
static void C_ccall trf_25659(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25659(t0,t1);}

C_noret_decl(trf_25724)
static void C_ccall trf_25724(C_word c,C_word *av) C_noret;
static void C_ccall trf_25724(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25724(t0,t1);}

C_noret_decl(trf_25733)
static void C_ccall trf_25733(C_word c,C_word *av) C_noret;
static void C_ccall trf_25733(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25733(t0,t1);}

C_noret_decl(trf_25819)
static void C_ccall trf_25819(C_word c,C_word *av) C_noret;
static void C_ccall trf_25819(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25819(t0,t1);}

C_noret_decl(trf_25947)
static void C_ccall trf_25947(C_word c,C_word *av) C_noret;
static void C_ccall trf_25947(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25947(t0,t1,t2);}

C_noret_decl(trf_26046)
static void C_ccall trf_26046(C_word c,C_word *av) C_noret;
static void C_ccall trf_26046(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26046(t0,t1,t2);}

C_noret_decl(trf_26168)
static void C_ccall trf_26168(C_word c,C_word *av) C_noret;
static void C_ccall trf_26168(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26168(t0,t1,t2);}

C_noret_decl(trf_26381)
static void C_ccall trf_26381(C_word c,C_word *av) C_noret;
static void C_ccall trf_26381(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26381(t0,t1);}

C_noret_decl(trf_26393)
static void C_ccall trf_26393(C_word c,C_word *av) C_noret;
static void C_ccall trf_26393(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_26393(t0,t1,t2,t3);}

C_noret_decl(trf_26443)
static void C_ccall trf_26443(C_word c,C_word *av) C_noret;
static void C_ccall trf_26443(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_26443(t0,t1,t2,t3);}

C_noret_decl(trf_26472)
static void C_ccall trf_26472(C_word c,C_word *av) C_noret;
static void C_ccall trf_26472(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26472(t0,t1);}

C_noret_decl(trf_26492)
static void C_ccall trf_26492(C_word c,C_word *av) C_noret;
static void C_ccall trf_26492(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26492(t0,t1,t2);}

C_noret_decl(trf_26502)
static void C_ccall trf_26502(C_word c,C_word *av) C_noret;
static void C_ccall trf_26502(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_26502(t0,t1,t2,t3);}

C_noret_decl(trf_26505)
static void C_ccall trf_26505(C_word c,C_word *av) C_noret;
static void C_ccall trf_26505(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26505(t0,t1,t2);}

C_noret_decl(trf_26571)
static void C_ccall trf_26571(C_word c,C_word *av) C_noret;
static void C_ccall trf_26571(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26571(t0,t1,t2);}

C_noret_decl(trf_26635)
static void C_ccall trf_26635(C_word c,C_word *av) C_noret;
static void C_ccall trf_26635(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26635(t0,t1,t2);}

C_noret_decl(trf_26705)
static void C_ccall trf_26705(C_word c,C_word *av) C_noret;
static void C_ccall trf_26705(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26705(t0,t1,t2);}

C_noret_decl(trf_26790)
static void C_ccall trf_26790(C_word c,C_word *av) C_noret;
static void C_ccall trf_26790(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26790(t0,t1,t2);}

C_noret_decl(trf_26959)
static void C_ccall trf_26959(C_word c,C_word *av) C_noret;
static void C_ccall trf_26959(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26959(t0,t1,t2);}

C_noret_decl(trf_26992)
static void C_ccall trf_26992(C_word c,C_word *av) C_noret;
static void C_ccall trf_26992(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_26992(t0,t1,t2,t3);}

C_noret_decl(trf_27094)
static void C_ccall trf_27094(C_word c,C_word *av) C_noret;
static void C_ccall trf_27094(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_27094(t0,t1,t2,t3,t4);}

C_noret_decl(trf_27275)
static void C_ccall trf_27275(C_word c,C_word *av) C_noret;
static void C_ccall trf_27275(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27275(t0,t1,t2);}

C_noret_decl(trf_27289)
static void C_ccall trf_27289(C_word c,C_word *av) C_noret;
static void C_ccall trf_27289(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_27289(t0,t1,t2,t3,t4);}

C_noret_decl(trf_27366)
static void C_ccall trf_27366(C_word c,C_word *av) C_noret;
static void C_ccall trf_27366(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27366(t0,t1);}

C_noret_decl(trf_27369)
static void C_ccall trf_27369(C_word c,C_word *av) C_noret;
static void C_ccall trf_27369(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27369(t0,t1);}

C_noret_decl(trf_27376)
static void C_ccall trf_27376(C_word c,C_word *av) C_noret;
static void C_ccall trf_27376(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27376(t0,t1);}

C_noret_decl(trf_27485)
static void C_ccall trf_27485(C_word c,C_word *av) C_noret;
static void C_ccall trf_27485(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_27485(t0,t1,t2,t3,t4);}

C_noret_decl(trf_27597)
static void C_ccall trf_27597(C_word c,C_word *av) C_noret;
static void C_ccall trf_27597(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27597(t0,t1);}

C_noret_decl(trf_27607)
static void C_ccall trf_27607(C_word c,C_word *av) C_noret;
static void C_ccall trf_27607(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27607(t0,t1);}

C_noret_decl(trf_27617)
static void C_ccall trf_27617(C_word c,C_word *av) C_noret;
static void C_ccall trf_27617(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_27617(t0,t1,t2,t3);}

C_noret_decl(trf_27637)
static void C_ccall trf_27637(C_word c,C_word *av) C_noret;
static void C_ccall trf_27637(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27637(t0,t1);}

C_noret_decl(trf_27757)
static void C_ccall trf_27757(C_word c,C_word *av) C_noret;
static void C_ccall trf_27757(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27757(t0,t1);}

C_noret_decl(trf_27760)
static void C_ccall trf_27760(C_word c,C_word *av) C_noret;
static void C_ccall trf_27760(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27760(t0,t1);}

C_noret_decl(trf_27780)
static void C_ccall trf_27780(C_word c,C_word *av) C_noret;
static void C_ccall trf_27780(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_27780(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_28014)
static void C_ccall trf_28014(C_word c,C_word *av) C_noret;
static void C_ccall trf_28014(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_28014(t0,t1,t2,t3,t4);}

C_noret_decl(trf_28366)
static void C_ccall trf_28366(C_word c,C_word *av) C_noret;
static void C_ccall trf_28366(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28366(t0,t1,t2);}

C_noret_decl(trf_28372)
static void C_ccall trf_28372(C_word c,C_word *av) C_noret;
static void C_ccall trf_28372(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_28372(t0,t1,t2,t3);}

C_noret_decl(trf_28546)
static void C_ccall trf_28546(C_word c,C_word *av) C_noret;
static void C_ccall trf_28546(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28546(t0,t1,t2);}

C_noret_decl(trf_28642)
static void C_ccall trf_28642(C_word c,C_word *av) C_noret;
static void C_ccall trf_28642(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_28642(t0,t1);}

C_noret_decl(trf_28713)
static void C_ccall trf_28713(C_word c,C_word *av) C_noret;
static void C_ccall trf_28713(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_28713(t0,t1);}

C_noret_decl(trf_28715)
static void C_ccall trf_28715(C_word c,C_word *av) C_noret;
static void C_ccall trf_28715(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_28715(t0,t1,t2,t3,t4);}

C_noret_decl(trf_28843)
static void C_ccall trf_28843(C_word c,C_word *av) C_noret;
static void C_ccall trf_28843(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28843(t0,t1,t2);}

C_noret_decl(trf_28887)
static void C_ccall trf_28887(C_word c,C_word *av) C_noret;
static void C_ccall trf_28887(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_28887(t0,t1,t2,t3);}

C_noret_decl(trf_29000)
static void C_ccall trf_29000(C_word c,C_word *av) C_noret;
static void C_ccall trf_29000(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29000(t0,t1,t2);}

C_noret_decl(trf_29010)
static void C_ccall trf_29010(C_word c,C_word *av) C_noret;
static void C_ccall trf_29010(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29010(t0,t1);}

C_noret_decl(trf_29072)
static void C_ccall trf_29072(C_word c,C_word *av) C_noret;
static void C_ccall trf_29072(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_29072(t0,t1,t2,t3);}

C_noret_decl(trf_29084)
static void C_ccall trf_29084(C_word c,C_word *av) C_noret;
static void C_ccall trf_29084(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_29084(t0,t1,t2,t3);}

C_noret_decl(trf_29197)
static void C_ccall trf_29197(C_word c,C_word *av) C_noret;
static void C_ccall trf_29197(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29197(t0,t1);}

C_noret_decl(trf_29209)
static void C_ccall trf_29209(C_word c,C_word *av) C_noret;
static void C_ccall trf_29209(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29209(t0,t1,t2);}

C_noret_decl(trf_29289)
static void C_ccall trf_29289(C_word c,C_word *av) C_noret;
static void C_ccall trf_29289(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29289(t0,t1,t2);}

C_noret_decl(trf_29312)
static void C_ccall trf_29312(C_word c,C_word *av) C_noret;
static void C_ccall trf_29312(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29312(t0,t1);}

C_noret_decl(trf_29327)
static void C_ccall trf_29327(C_word c,C_word *av) C_noret;
static void C_ccall trf_29327(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29327(t0,t1,t2);}

C_noret_decl(trf_29357)
static void C_ccall trf_29357(C_word c,C_word *av) C_noret;
static void C_ccall trf_29357(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29357(t0,t1,t2);}

C_noret_decl(trf_29421)
static void C_ccall trf_29421(C_word c,C_word *av) C_noret;
static void C_ccall trf_29421(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29421(t0,t1,t2);}

C_noret_decl(trf_29471)
static void C_ccall trf_29471(C_word c,C_word *av) C_noret;
static void C_ccall trf_29471(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29471(t0,t1,t2);}

C_noret_decl(trf_29509)
static void C_ccall trf_29509(C_word c,C_word *av) C_noret;
static void C_ccall trf_29509(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29509(t0,t1,t2);}

C_noret_decl(trf_29530)
static void C_ccall trf_29530(C_word c,C_word *av) C_noret;
static void C_ccall trf_29530(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29530(t0,t1,t2);}

C_noret_decl(trf_29564)
static void C_ccall trf_29564(C_word c,C_word *av) C_noret;
static void C_ccall trf_29564(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29564(t0,t1);}

C_noret_decl(trf_29590)
static void C_ccall trf_29590(C_word c,C_word *av) C_noret;
static void C_ccall trf_29590(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29590(t0,t1,t2);}

C_noret_decl(trf_29634)
static void C_ccall trf_29634(C_word c,C_word *av) C_noret;
static void C_ccall trf_29634(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29634(t0,t1);}

C_noret_decl(trf_29698)
static void C_ccall trf_29698(C_word c,C_word *av) C_noret;
static void C_ccall trf_29698(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29698(t0,t1);}

C_noret_decl(trf_29707)
static void C_ccall trf_29707(C_word c,C_word *av) C_noret;
static void C_ccall trf_29707(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29707(t0,t1);}

C_noret_decl(trf_29801)
static void C_ccall trf_29801(C_word c,C_word *av) C_noret;
static void C_ccall trf_29801(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29801(t0,t1,t2);}

C_noret_decl(trf_30050)
static void C_ccall trf_30050(C_word c,C_word *av) C_noret;
static void C_ccall trf_30050(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_30050(t0,t1,t2,t3);}

C_noret_decl(trf_30158)
static void C_ccall trf_30158(C_word c,C_word *av) C_noret;
static void C_ccall trf_30158(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_30158(t0,t1,t2,t3);}

C_noret_decl(trf_30207)
static void C_ccall trf_30207(C_word c,C_word *av) C_noret;
static void C_ccall trf_30207(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30207(t0,t1);}

C_noret_decl(trf_7540)
static void C_ccall trf_7540(C_word c,C_word *av) C_noret;
static void C_ccall trf_7540(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7540(t0,t1);}

C_noret_decl(trf_7622)
static void C_ccall trf_7622(C_word c,C_word *av) C_noret;
static void C_ccall trf_7622(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7622(t0,t1);}

C_noret_decl(trf_7645)
static void C_ccall trf_7645(C_word c,C_word *av) C_noret;
static void C_ccall trf_7645(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7645(t0,t1);}

C_noret_decl(trf_7654)
static void C_ccall trf_7654(C_word c,C_word *av) C_noret;
static void C_ccall trf_7654(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7654(t0,t1);}

C_noret_decl(trf_7954)
static void C_ccall trf_7954(C_word c,C_word *av) C_noret;
static void C_ccall trf_7954(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7954(t0,t1,t2,t3);}

C_noret_decl(trf_8022)
static void C_ccall trf_8022(C_word c,C_word *av) C_noret;
static void C_ccall trf_8022(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8022(t0,t1);}

C_noret_decl(trf_8066)
static void C_ccall trf_8066(C_word c,C_word *av) C_noret;
static void C_ccall trf_8066(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8066(t0,t1,t2,t3);}

C_noret_decl(trf_8412)
static void C_ccall trf_8412(C_word c,C_word *av) C_noret;
static void C_ccall trf_8412(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8412(t0,t1);}

C_noret_decl(trf_8473)
static void C_ccall trf_8473(C_word c,C_word *av) C_noret;
static void C_ccall trf_8473(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8473(t0,t1,t2,t3);}

C_noret_decl(trf_8622)
static void C_ccall trf_8622(C_word c,C_word *av) C_noret;
static void C_ccall trf_8622(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8622(t0,t1,t2);}

C_noret_decl(trf_8640)
static void C_ccall trf_8640(C_word c,C_word *av) C_noret;
static void C_ccall trf_8640(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8640(t0,t1,t2,t3);}

C_noret_decl(trf_8670)
static void C_ccall trf_8670(C_word c,C_word *av) C_noret;
static void C_ccall trf_8670(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8670(t0,t1,t2);}

C_noret_decl(trf_8830)
static void C_ccall trf_8830(C_word c,C_word *av) C_noret;
static void C_ccall trf_8830(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8830(t0,t1,t2);}

C_noret_decl(trf_8873)
static void C_ccall trf_8873(C_word c,C_word *av) C_noret;
static void C_ccall trf_8873(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8873(t0,t1,t2);}

C_noret_decl(trf_8930)
static void C_ccall trf_8930(C_word c,C_word *av) C_noret;
static void C_ccall trf_8930(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8930(t0,t1,t2);}

C_noret_decl(trf_8968)
static void C_ccall trf_8968(C_word c,C_word *av) C_noret;
static void C_ccall trf_8968(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_8968(t0,t1,t2,t3,t4);}

C_noret_decl(trf_8971)
static void C_ccall trf_8971(C_word c,C_word *av) C_noret;
static void C_ccall trf_8971(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8971(t0,t1);}

C_noret_decl(trf_9108)
static void C_ccall trf_9108(C_word c,C_word *av) C_noret;
static void C_ccall trf_9108(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9108(t0,t1);}

C_noret_decl(trf_9158)
static void C_ccall trf_9158(C_word c,C_word *av) C_noret;
static void C_ccall trf_9158(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9158(t0,t1);}

C_noret_decl(trf_9174)
static void C_ccall trf_9174(C_word c,C_word *av) C_noret;
static void C_ccall trf_9174(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9174(t0,t1,t2,t3);}

C_noret_decl(trf_9195)
static void C_ccall trf_9195(C_word c,C_word *av) C_noret;
static void C_ccall trf_9195(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9195(t0,t1,t2);}

C_noret_decl(trf_9201)
static void C_ccall trf_9201(C_word c,C_word *av) C_noret;
static void C_ccall trf_9201(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9201(t0,t1,t2);}

C_noret_decl(trf_9256)
static void C_ccall trf_9256(C_word c,C_word *av) C_noret;
static void C_ccall trf_9256(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9256(t0,t1);}

C_noret_decl(trf_9294)
static void C_ccall trf_9294(C_word c,C_word *av) C_noret;
static void C_ccall trf_9294(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9294(t0,t1,t2);}

C_noret_decl(trf_9310)
static void C_ccall trf_9310(C_word c,C_word *av) C_noret;
static void C_ccall trf_9310(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9310(t0,t1,t2,t3);}

C_noret_decl(trf_9337)
static void C_ccall trf_9337(C_word c,C_word *av) C_noret;
static void C_ccall trf_9337(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9337(t0,t1,t2);}

C_noret_decl(trf_9355)
static void C_ccall trf_9355(C_word c,C_word *av) C_noret;
static void C_ccall trf_9355(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9355(t0,t1,t2,t3);}

C_noret_decl(trf_9382)
static void C_ccall trf_9382(C_word c,C_word *av) C_noret;
static void C_ccall trf_9382(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9382(t0,t1,t2,t3);}

C_noret_decl(trf_9388)
static void C_ccall trf_9388(C_word c,C_word *av) C_noret;
static void C_ccall trf_9388(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9388(t0,t1,t2,t3);}

C_noret_decl(trf_9739)
static void C_ccall trf_9739(C_word c,C_word *av) C_noret;
static void C_ccall trf_9739(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9739(t0,t1,t2,t3);}

C_noret_decl(trf_9746)
static void C_ccall trf_9746(C_word c,C_word *av) C_noret;
static void C_ccall trf_9746(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9746(t0,t1,t2,t3);}

C_noret_decl(trf_9773)
static void C_ccall trf_9773(C_word c,C_word *av) C_noret;
static void C_ccall trf_9773(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9773(t0,t1,t2,t3);}

C_noret_decl(trf_9887)
static void C_ccall trf_9887(C_word c,C_word *av) C_noret;
static void C_ccall trf_9887(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9887(t0,t1,t2);}

C_noret_decl(trf_9964)
static void C_ccall trf_9964(C_word c,C_word *av) C_noret;
static void C_ccall trf_9964(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9964(t0,t1);}

/* f33619 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f33619(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f33619,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[124],t1);
/* irregex-core.scm:848: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* f33623 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f33623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f33623,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[125],t1);
/* irregex-core.scm:848: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* scan */
static void C_fcall f_10021(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,4)))){
C_save_and_reclaim_args((void *)trf_10021,5,t0,t1,t2,t3,t4);}
a=C_alloc(23);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10024,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[2],a[6]=((C_word)li53),tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(t3,((C_word*)((C_word*)t0)[3])[1]))){
/* irregex-core.scm:676: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[78];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=C_i_string_ref(((C_word*)t0)[2],t3);
switch(t6){
case C_make_character(34):
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10063,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10070,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:678: collect */
t9=t5;
f_10024(t9,t8);
case C_make_character(92):
t7=C_a_i_fixnum_plus(&a,2,t3,C_fix(1));
t8=C_a_i_fixnum_plus(&a,2,t3,C_fix(2));
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10089,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=t7,a[5]=t8,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:679: collect */
t10=t5;
f_10024(t10,t9);
default:
t7=C_a_i_fixnum_plus(&a,2,t3,C_fix(1));
/* irregex-core.scm:680: scan */
t14=t1;
t15=t2;
t16=t7;
t17=t4;
t1=t14;
t2=t15;
t3=t16;
t4=t17;
goto loop;}}}

/* collect in scan */
static void C_fcall f_10024(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_10024,2,t0,t1);}
a=C_alloc(4);
t2=C_eqp(((C_word*)t0)[2],((C_word*)t0)[3]);
if(C_truep(t2)){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10038,a[2]=t1,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:674: scheme#substring */
t4=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k10036 in collect in scan */
static void C_ccall f_10038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10038,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10061 in scan */
static void C_ccall f_10063(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10063,c,av);}
a=C_alloc(5);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:678: k */
t3=((C_word*)t0)[3];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10068 in scan */
static void C_ccall f_10070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10070,c,av);}
/* irregex-core.scm:678: string-cat-reverse */
f_9108(((C_word*)t0)[2],t1);}

/* k10087 in scan */
static void C_ccall f_10089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10089,c,av);}
/* irregex-core.scm:679: scan */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10021(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* a10121 */
static void C_ccall f_10122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10122,c,av);}
/* irregex-core.scm:684: read */
t4=((C_word*)((C_word*)t0)[2])[1];
f_9746(t4,t1,t3,((C_word*)t0)[3]);}

/* a10139 */
static void C_ccall f_10140(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10140,c,av);}
a=C_alloc(5);
t4=C_eqp(t2,lf[79]);
if(C_truep(t4)){
/* irregex-core.scm:688: k */
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=C_make_character(32);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=C_eqp(t2,lf[80]);
if(C_truep(t5)){
/* irregex-core.scm:688: k */
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=C_make_character(10);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10160,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_numberp(t2))){
/* ##sys#number->string */
t7=*((C_word*)lf[81]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=C_fix(10);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
/* irregex-core.scm:693: scheme#symbol->string */
t7=*((C_word*)lf[82]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}}

/* k10158 in a10139 */
static void C_ccall f_10160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10160,c,av);}
/* irregex-core.scm:688: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_i_string_ref(t1,C_fix(0));
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* scan */
static void C_fcall f_10240(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_10240,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_fixnum_greater_or_equal_p(t2,((C_word*)((C_word*)t0)[2])[1]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10250,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t3)){
t5=t4;
f_10250(t5,t3);}
else{
t5=C_i_string_ref(((C_word*)t0)[4],t2);
t6=C_u_i_char_whitespacep(t5);
t7=t4;
f_10250(t7,(C_truep(t6)?t6:C_u_i_memq(t5,lf[86])));}}

/* k10248 in scan */
static void C_fcall f_10250(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_10250,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10253,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:711: scheme#substring */
t3=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* irregex-core.scm:713: scan */
t2=((C_word*)((C_word*)t0)[7])[1];
f_10240(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}}

/* k10251 in k10248 in scan */
static void C_ccall f_10253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_10253,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10260,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:712: scheme#string->number */
t3=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10258 in k10251 in k10248 in scan */
static void C_ccall f_10260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10260,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10263,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
/* irregex-core.scm:712: k */
t3=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* irregex-core.scm:712: scheme#string->symbol */
t3=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k10261 in k10258 in k10251 in k10248 in scan */
static void C_ccall f_10263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10263,c,av);}
/* irregex-core.scm:712: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a10310 in chicken.irregex#with-read-from-string in k7761 in k7536 in k7533 */
static void C_ccall f_10311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10311,c,av);}
t4=C_eqp(t2,lf[87]);
if(C_truep(t4)){
/* irregex-core.scm:716: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[88];
av2[3]=((C_word*)t0)[2];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
/* irregex-core.scm:717: proc */
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* chicken.irregex#symbol-list->flags in k7761 in k7536 in k7533 */
static void C_fcall f_10332(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_10332,2,t1,t2);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10338,a[2]=t4,a[3]=((C_word)li61),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_10338(t6,t1,t2,C_fix(0));}

/* lp in chicken.irregex#symbol-list->flags in k7761 in k7536 in k7533 */
static void C_fcall f_10338(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_10338,4,t0,t1,t2,t3);}
a=C_alloc(13);
t4=C_i_pairp(t2);
if(C_truep(C_i_not(t4))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_cdr(t2);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10356,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
t7=C_u_i_car(t2);
t8=C_eqp(t7,lf[90]);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10367,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,a[5]=t3,a[6]=t7,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
if(C_truep(t8)){
t10=t9;
f_10367(t10,t8);}
else{
t10=C_eqp(t7,lf[99]);
t11=t9;
f_10367(t11,(C_truep(t10)?t10:C_eqp(t7,lf[100])));}}}

/* k10354 in lp in chicken.irregex#symbol-list->flags in k7761 in k7536 in k7533 */
static void C_fcall f_10356(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_10356,2,t0,t1);}
/* irregex-core.scm:733: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10338(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k10365 in lp in chicken.irregex#symbol-list->flags in k7761 in k7536 in k7533 */
static void C_fcall f_10367(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_10367,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:733: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10338(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_or(((C_word*)t0)[5],C_fix(2)));}
else{
t2=C_eqp(((C_word*)t0)[6],lf[91]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[6],lf[92]));
if(C_truep(t3)){
/* irregex-core.scm:733: lp */
t4=((C_word*)((C_word*)t0)[2])[1];
f_10338(t4,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_or(((C_word*)t0)[5],C_fix(4)));}
else{
t4=C_eqp(((C_word*)t0)[6],lf[93]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[6],lf[94]));
if(C_truep(t5)){
t6=((C_word*)t0)[7];
f_10356(t6,C_fixnum_or(((C_word*)t0)[5],C_fix(8)));}
else{
t6=C_eqp(((C_word*)t0)[6],lf[95]);
t7=(C_truep(t6)?t6:C_eqp(((C_word*)t0)[6],lf[96]));
if(C_truep(t7)){
t8=((C_word*)t0)[7];
f_10356(t8,C_fixnum_or(((C_word*)t0)[5],C_fix(16)));}
else{
t8=C_eqp(((C_word*)t0)[6],lf[97]);
t9=(C_truep(t8)?t8:C_eqp(((C_word*)t0)[6],lf[98]));
if(C_truep(t9)){
t10=((C_word*)t0)[7];
f_10356(t10,C_fixnum_or(((C_word*)t0)[5],C_fix(32)));}
else{
/* irregex-core.scm:733: lp */
t10=((C_word*)((C_word*)t0)[2])[1];
f_10338(t10,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}}}}}}

/* chicken.irregex#maybe-string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10455,c,av);}
if(C_truep(C_i_stringp(t2))){
/* irregex-core.scm:745: string->sre */
t3=*((C_word*)lf[102]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,4)))){
C_save_and_reclaim((void*)f_10467,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10471,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_stringp(t2);
if(C_truep(C_i_not(t5))){
/* irregex-core.scm:748: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=lf[197];
av2[3]=lf[198];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_10471(2,av2);}}}

/* k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10471,c,av);}
a=C_alloc(5);
t2=C_i_string_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10477,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:750: symbol-list->flags */
f_10332(t3,((C_word*)t0)[4]);}

/* k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,7)))){
C_save_and_reclaim((void *)f_10477,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10482,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word)li89),tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_10482(t5,((C_word*)t0)[4],C_fix(0),C_fix(0),t1,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST);}

/* lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10482(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word t91;
C_word t92;
C_word t93;
C_word t94;
C_word t95;
C_word t96;
C_word t97;
C_word t98;
C_word t99;
C_word t100;
C_word t101;
C_word t102;
C_word t103;
C_word t104;
C_word t105;
C_word t106;
C_word t107;
C_word t108;
C_word t109;
C_word t110;
C_word t111;
C_word t112;
C_word t113;
C_word t114;
C_word t115;
C_word t116;
C_word t117;
C_word t118;
C_word t119;
C_word t120;
C_word t121;
C_word t122;
C_word t123;
C_word t124;
C_word t125;
C_word t126;
C_word t127;
C_word t128;
C_word t129;
C_word t130;
C_word t131;
C_word t132;
C_word t133;
C_word t134;
C_word t135;
C_word t136;
C_word t137;
C_word t138;
C_word t139;
C_word t140;
C_word t141;
C_word t142;
C_word t143;
C_word t144;
C_word t145;
C_word t146;
C_word t147;
C_word t148;
C_word t149;
C_word t150;
C_word t151;
C_word t152;
C_word t153;
C_word t154;
C_word t155;
C_word t156;
C_word t157;
C_word t158;
C_word t159;
C_word t160;
C_word t161;
C_word t162;
C_word t163;
C_word t164;
C_word t165;
C_word t166;
C_word t167;
C_word t168;
C_word t169;
C_word t170;
C_word t171;
C_word t172;
C_word t173;
C_word t174;
C_word t175;
C_word t176;
C_word t177;
C_word t178;
C_word t179;
C_word t180;
C_word t181;
C_word t182;
C_word t183;
C_word t184;
C_word t185;
C_word t186;
C_word t187;
C_word t188;
C_word t189;
C_word t190;
C_word t191;
C_word t192;
C_word t193;
C_word t194;
C_word t195;
C_word t196;
C_word t197;
C_word t198;
C_word t199;
C_word t200;
C_word t201;
C_word t202;
C_word t203;
C_word t204;
C_word t205;
C_word t206;
C_word t207;
C_word t208;
C_word t209;
C_word t210;
C_word t211;
C_word t212;
C_word t213;
C_word t214;
C_word t215;
C_word t216;
C_word t217;
C_word t218;
C_word t219;
C_word t220;
C_word t221;
C_word t222;
C_word t223;
C_word t224;
C_word t225;
C_word t226;
C_word t227;
C_word t228;
C_word t229;
C_word t230;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(105,0,6)))){
C_save_and_reclaim_args((void *)trf_10482,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(105);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_UNDEFINED;
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10485,a[2]=t4,a[3]=((C_word)li64),tmp=(C_word)a,a+=4,tmp));
t19=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10507,a[2]=t4,a[3]=t8,a[4]=((C_word)li66),tmp=(C_word)a,a+=5,tmp));
t20=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10563,a[2]=t2,a[3]=t3,a[4]=t5,a[5]=t10,a[6]=((C_word*)t0)[2],a[7]=((C_word)li67),tmp=(C_word)a,a+=8,tmp));
t21=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10583,a[2]=t4,a[3]=t3,a[4]=t5,a[5]=t10,a[6]=((C_word*)t0)[2],a[7]=t8,a[8]=t2,a[9]=((C_word)li69),tmp=(C_word)a,a+=10,tmp));
t22=t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10655,a[2]=t4,a[3]=t12,a[4]=((C_word)li71),tmp=(C_word)a,a+=5,tmp);
t23=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10888,a[2]=t4,a[3]=t6,a[4]=t12,a[5]=((C_word)li72),tmp=(C_word)a,a+=6,tmp));
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[3]))){
if(C_truep(C_i_pairp(t6))){
/* irregex-core.scm:843: chicken.base#error */
t24=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t24;
av2[1]=t1;
av2[2]=lf[123];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t24+1)))(4,av2);}}
else{
/* irregex-core.scm:844: collect/terms */
t24=t15;
f_10655(t24,t1);}}
else{
t24=C_i_string_ref(((C_word*)t0)[2],t2);
switch(t24){
case C_make_character(46):
t25=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t26=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t27=C_fixnum_and(t4,C_fix(8));
t28=C_eqp(C_fix(8),t27);
if(C_truep(t28)){
t29=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f33619,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t25,a[5]=t26,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:850: collect */
t30=((C_word*)t12)[1];
f_10563(t30,t29);}
else{
t29=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f33623,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t25,a[5]=t26,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:850: collect */
t30=((C_word*)t12)[1];
f_10563(t30,t29);}
case C_make_character(63):
t25=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10960,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:853: collect/single */
t26=((C_word*)t14)[1];
f_10583(t26,t25);
default:
t25=C_eqp(t24,C_make_character(43));
t26=(C_truep(t25)?t25:C_eqp(t24,C_make_character(42)));
if(C_truep(t26)){
t27=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11120,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=t6,a[8]=t24,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:874: collect/single */
t28=((C_word*)t14)[1];
f_10583(t28,t27);}
else{
switch(t24){
case C_make_character(40):
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
if(C_truep(C_i_integer_greater_or_equalp(t27,((C_word*)t0)[3]))){
/* irregex-core.scm:889: chicken.base#error */
t28=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t28;
av2[1]=t1;
av2[2]=lf[140];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t28+1)))(4,av2);}}
else{
t28=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t29=C_i_string_ref(((C_word*)t0)[2],t28);
if(C_truep(C_i_not(C_u_i_memq(t29,lf[141])))){
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t32=C_fixnum_or(t4,C_fix(1));
t33=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11212,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t32,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:891: save */
t34=((C_word*)t17)[1];
f_10888(t34,t33);}
else{
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
if(C_truep(C_i_integer_greater_or_equalp(t30,((C_word*)t0)[3]))){
/* irregex-core.scm:893: chicken.base#error */
t31=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t31;
av2[1]=t1;
av2[2]=lf[142];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t31+1)))(4,av2);}}
else{
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t32=C_i_string_ref(((C_word*)t0)[2],t31);
t33=C_eqp(t32,C_make_character(42));
if(C_truep(t33)){
t34=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t35=C_i_string_ref(((C_word*)t0)[2],t34);
t36=C_eqp(C_make_character(39),t35);
if(C_truep(t36)){
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11242,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=t6,a[7]=t12,a[8]=((C_word)li73),tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:896: with-read-from-string */
f_9739(t1,((C_word*)t0)[2],t37,t38);}
else{
/* irregex-core.scm:901: chicken.base#error */
t37=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t37;
av2[1]=t1;
av2[2]=lf[144];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t37+1)))(4,av2);}}}
else{
t34=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t35=C_i_string_ref(((C_word*)t0)[2],t34);
switch(t35){
case C_make_character(35):
t36=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11309,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t4,a[6]=t6,a[7]=t12,tmp=(C_word)a,a+=8,tmp);
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
/* irregex-core.scm:905: string-scan-char */
t38=lf[54];{
C_word av2[5];
av2[0]=t38;
av2[1]=t36;
av2[2]=((C_word*)t0)[2];
av2[3]=C_make_character(41);
av2[4]=t37;
((C_proc)(void*)(*((C_word*)t38+1)))(5,av2);}
case C_make_character(58):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_fixnum_not(C_fix(1));
t39=C_fixnum_and(t4,t38);
t40=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11350,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t36,a[5]=t37,a[6]=t39,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:908: save */
t41=((C_word*)t17)[1];
f_10888(t41,t40);
case C_make_character(61):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_fixnum_not(C_fix(1));
t39=C_fixnum_and(t4,t38);
t40=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11373,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t36,a[5]=t37,a[6]=t39,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:911: save */
t41=((C_word*)t17)[1];
f_10888(t41,t40);
case C_make_character(33):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_fixnum_not(C_fix(1));
t39=C_fixnum_and(t4,t38);
t40=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11396,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t36,a[5]=t37,a[6]=t39,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:914: save */
t41=((C_word*)t17)[1];
f_10888(t41,t40);
case C_make_character(60):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
if(C_truep(C_i_integer_greater_or_equalp(t36,((C_word*)t0)[3]))){
/* irregex-core.scm:918: chicken.base#error */
t37=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t37;
av2[1]=t1;
av2[2]=lf[147];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t37+1)))(4,av2);}}
else{
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_i_string_ref(((C_word*)t0)[2],t37);
switch(t38){
case C_make_character(61):
t39=C_a_i_fixnum_plus(&a,2,t2,C_fix(4));
t40=C_a_i_fixnum_plus(&a,2,t2,C_fix(4));
t41=C_fixnum_not(C_fix(1));
t42=C_fixnum_and(t4,t41);
t43=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11438,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t39,a[5]=t40,a[6]=t42,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:923: save */
t44=((C_word*)t17)[1];
f_10888(t44,t43);
case C_make_character(33):
t39=C_a_i_fixnum_plus(&a,2,t2,C_fix(4));
t40=C_a_i_fixnum_plus(&a,2,t2,C_fix(4));
t41=C_fixnum_not(C_fix(1));
t42=C_fixnum_and(t4,t41);
t43=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11461,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t39,a[5]=t40,a[6]=t42,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:926: save */
t44=((C_word*)t17)[1];
f_10888(t44,t43);
default:
t39=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11464,a[2]=t4,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t17,a[6]=t2,a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
t40=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t41=C_i_string_ref(((C_word*)t0)[2],t40);
if(C_truep(C_u_i_char_alphabeticp(t41))){
t42=C_a_i_fixnum_plus(&a,2,t2,C_fix(4));
/* irregex-core.scm:930: string-scan-char */
t43=lf[54];{
C_word av2[5];
av2[0]=t43;
av2[1]=t39;
av2[2]=((C_word*)t0)[2];
av2[3]=C_make_character(62);
av2[4]=t42;
((C_proc)(void*)(*((C_word*)t43+1)))(5,av2);}}
else{
t42=t39;{
C_word av2[2];
av2[0]=t42;
av2[1]=C_SCHEME_FALSE;
f_11464(2,av2);}}}}
case C_make_character(62):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_fixnum_not(C_fix(1));
t39=C_fixnum_and(t4,t38);
t40=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11550,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t36,a[5]=t37,a[6]=t39,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:939: save */
t41=((C_word*)t17)[1];
f_10888(t41,t40);
case C_make_character(40):
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
if(C_truep(C_i_integer_greater_or_equalp(t36,((C_word*)t0)[3]))){
/* irregex-core.scm:947: chicken.base#error */
t37=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t37;
av2[1]=t1;
av2[2]=lf[152];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t37+1)))(4,av2);}}
else{
t37=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t38=C_i_string_ref(((C_word*)t0)[2],t37);
if(C_truep(C_u_i_char_numericp(t38))){
t39=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11575,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=t17,a[7]=t2,tmp=(C_word)a,a+=8,tmp);
t40=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
/* irregex-core.scm:949: string-scan-char */
t41=lf[54];{
C_word av2[5];
av2[0]=t41;
av2[1]=t39;
av2[2]=((C_word*)t0)[2];
av2[3]=C_make_character(41);
av2[4]=t40;
((C_proc)(void*)(*((C_word*)t41+1)))(5,av2);}}
else{
t39=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
t40=C_i_string_ref(((C_word*)t0)[2],t39);
if(C_truep(C_u_i_char_alphabeticp(t40))){
t41=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11627,a[2]=t4,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t17,a[6]=t2,a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
t42=C_a_i_fixnum_plus(&a,2,t2,C_fix(3));
/* irregex-core.scm:956: string-scan-char */
t43=lf[54];{
C_word av2[5];
av2[0]=t43;
av2[1]=t41;
av2[2]=((C_word*)t0)[2];
av2[3]=C_make_character(41);
av2[4]=t42;
((C_proc)(void*)(*((C_word*)t43+1)))(5,av2);}}
else{
t41=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t42=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t43=C_fixnum_not(C_fix(1));
t44=C_fixnum_and(t4,t43);
t45=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11678,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t41,a[5]=t42,a[6]=t44,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:962: save */
t46=((C_word*)t17)[1];
f_10888(t46,t45);}}}
case C_make_character(123):
/* irregex-core.scm:964: chicken.base#error */
t36=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t36;
av2[1]=t1;
av2[2]=lf[155];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t36+1)))(4,av2);}
default:
t36=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t37=C_SCHEME_UNDEFINED;
t38=(*a=C_VECTOR_TYPE|1,a[1]=t37,tmp=(C_word)a,a+=2,tmp);
t39=C_set_block_item(t38,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11709,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t38,a[7]=((C_word*)t0)[4],a[8]=t6,a[9]=t12,a[10]=((C_word)li76),tmp=(C_word)a,a+=11,tmp));
t40=((C_word*)t38)[1];
f_11709(t40,t1,t36,t4,C_SCHEME_FALSE);}}}}}
case C_make_character(41):
if(C_truep(C_i_nullp(t6))){
/* irregex-core.scm:1004: chicken.base#error */
t27=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t27;
av2[1]=t1;
av2[2]=lf[160];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t27+1)))(4,av2);}}
else{
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t28=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t29=C_i_caar(t6);
t30=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11987,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t27,a[6]=t28,a[7]=t29,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1008: collect/terms */
t31=t15;
f_10655(t31,t30);}
case C_make_character(91):
t27=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12002,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=t6,a[5]=t12,a[6]=((C_word)li77),tmp=(C_word)a,a+=7,tmp);
t28=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12026,a[2]=t1,a[3]=t27,tmp=(C_word)a,a+=4,tmp);
t29=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t30=C_i_string_length(((C_word*)t0)[2]);
t31=C_fixnum_lessp(t29,t30);
t32=(C_truep(t31)?C_eqp(C_make_character(94),C_i_string_ref(((C_word*)t0)[2],t29)):C_SCHEME_FALSE);
t33=C_fixnum_and(t4,C_fix(32));
t34=C_eqp(C_fix(32),t33);
t35=C_SCHEME_UNDEFINED;
t36=(*a=C_VECTOR_TYPE|1,a[1]=t35,tmp=(C_word)a,a+=2,tmp);
t37=C_set_block_item(t36,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13404,a[2]=t30,a[3]=((C_word*)t0)[2],a[4]=t36,a[5]=t4,a[6]=t32,a[7]=t29,a[8]=t34,a[9]=((C_word)li82),tmp=(C_word)a,a+=10,tmp));
if(C_truep(t32)){
t38=C_fixnum_plus(t29,C_fix(1));
t39=C_fixnum_and(t4,C_fix(4));
t40=C_eqp(C_fix(4),t39);
if(C_truep(t40)){
t41=C_a_i_cons(&a,2,C_make_character(10),C_make_character(10));
t42=C_a_i_vector1(&a,1,t41);
/* irregex-core.scm:1303: go */
t43=((C_word*)t36)[1];
f_13404(t43,t28,t38,C_SCHEME_FALSE,t42);}
else{
t41=C_a_i_vector(&a,0);
/* irregex-core.scm:1303: go */
t42=((C_word*)t36)[1];
f_13404(t42,t28,t38,C_SCHEME_FALSE,t41);}}
else{
t38=C_a_i_vector(&a,0);
/* irregex-core.scm:1308: go */
t39=((C_word*)t36)[1];
f_13404(t39,t28,t29,C_SCHEME_FALSE,t38);}
case C_make_character(123):
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t28=C_i_integer_greater_or_equalp(t27,((C_word*)t0)[3]);
t29=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12045,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=((C_word*)t0)[2],a[10]=t14,tmp=(C_word)a,a+=11,tmp);
if(C_truep(t28)){
t30=t29;
f_12045(t30,t28);}
else{
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t31=C_i_string_ref(((C_word*)t0)[2],t30);
t32=C_u_i_char_numericp(t31);
if(C_truep(t32)){
t33=t29;
f_12045(t33,C_i_not(t32));}
else{
t33=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t34=C_i_string_ref(((C_word*)t0)[2],t33);
t35=t29;
f_12045(t35,C_i_not(C_eqp(C_make_character(44),t34)));}}
case C_make_character(92):
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
if(C_truep(C_i_integer_greater_or_equalp(t27,((C_word*)t0)[3]))){
/* irregex-core.scm:1051: chicken.base#error */
t28=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t28;
av2[1]=t1;
av2[2]=lf[177];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t28+1)))(4,av2);}}
else{
t28=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t29=C_i_string_ref(((C_word*)t0)[2],t28);
switch(t29){
case C_make_character(100):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12274,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1056: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(68):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,2,lf[179],lf[178]);
t33=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12301,a[2]=t32,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1058: collect */
t34=((C_word*)t12)[1];
f_10563(t34,t33);
case C_make_character(115):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12324,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1060: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(83):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,2,lf[179],lf[79]);
t33=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12351,a[2]=t32,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1062: collect */
t34=((C_word*)t12)[1];
f_10563(t34,t33);
case C_make_character(119):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,1,lf[180]);
t33=C_a_i_list(&a,3,lf[103],lf[181],t32);
t34=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12378,a[2]=t33,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1065: collect */
t35=((C_word*)t12)[1];
f_10563(t35,t34);
case C_make_character(87):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,1,lf[182]);
t33=C_a_i_list(&a,3,lf[103],lf[181],t32);
t34=C_a_i_list(&a,2,lf[179],t33);
t35=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12409,a[2]=t34,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1068: collect */
t36=((C_word*)t12)[1];
f_10563(t36,t35);
case C_make_character(98):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,3,lf[103],lf[183],lf[184]);
t33=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12444,a[2]=t32,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1071: collect */
t34=((C_word*)t12)[1];
f_10563(t34,t33);
case C_make_character(66):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12467,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1073: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(65):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12490,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1075: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(90):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,2,lf[131],C_make_character(10));
t33=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12521,a[2]=t32,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1078: collect */
t34=((C_word*)t12)[1];
f_10563(t34,t33);
case C_make_character(122):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12544,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1080: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(82):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12567,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1082: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(75):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12590,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1084: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(60):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12613,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1087: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(62):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12636,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1089: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(120):
t30=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12647,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=t6,a[5]=t12,a[6]=((C_word)li85),tmp=(C_word)a,a+=7,tmp);
t31=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12671,a[2]=t1,a[3]=t30,tmp=(C_word)a,a+=4,tmp);
t32=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
/* irregex-core.scm:1094: string-parse-hex-escape */
f_13295(t31,((C_word*)t0)[2],t32,((C_word*)t0)[3]);
case C_make_character(107):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_i_string_ref(((C_word*)t0)[2],t30);
if(C_truep(C_i_not(C_u_i_memq(t31,lf[189])))){
/* irregex-core.scm:1098: chicken.base#error */
t32=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t32;
av2[1]=t1;
av2[2]=lf[190];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t32+1)))(4,av2);}}
else{
t32=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12695,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=t6,a[7]=t12,a[8]=t2,tmp=(C_word)a,a+=9,tmp);
switch(t31){
case C_make_character(60):
t33=t32;
f_12695(t33,C_make_character(62));
case C_make_character(123):
t33=t32;
f_12695(t33,C_make_character(125));
case C_make_character(40):
t33=t32;
f_12695(t33,C_make_character(41));
default:
t33=C_eqp(t31,C_make_character(91));
t34=t32;
f_12695(t34,(C_truep(t33)?C_make_character(93):t31));}}
case C_make_character(81):
t30=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12767,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=t6,a[7]=((C_word*)t0)[2],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1113: collect */
t31=((C_word*)t12)[1];
f_10563(t31,t30);
case C_make_character(39):
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12878,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=t6,a[5]=t12,a[6]=((C_word)li87),tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:1130: with-read-from-string */
f_9739(t1,((C_word*)t0)[2],t30,t31);
default:
if(C_truep(C_u_i_char_numericp(t29))){
t30=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12899,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=t6,a[7]=t12,a[8]=t2,a[9]=((C_word*)t0)[2],tmp=(C_word)a,a+=10,tmp);
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_list(&a,1,t31);
t33=C_i_string_length(((C_word*)t0)[2]);
t34=C_i_pairp(t32);
t35=(C_truep(t34)?C_u_i_car(t32):C_fix(0));
t36=C_SCHEME_UNDEFINED;
t37=(*a=C_VECTOR_TYPE|1,a[1]=t36,tmp=(C_word)a,a+=2,tmp);
t38=C_set_block_item(t37,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8930,a[2]=t33,a[3]=((C_word*)t0)[2],a[4]=t37,a[5]=((C_word)li88),tmp=(C_word)a,a+=6,tmp));
t39=((C_word*)t37)[1];
f_8930(t39,t30,t35);}
else{
if(C_truep(C_u_i_char_alphabeticp(t29))){
t30=C_i_assq(t29,lf[168]);
if(C_truep(t30)){
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t32=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t33=C_i_cdr(t30);
t34=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12979,a[2]=t33,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t31,a[6]=t32,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1157: collect */
t35=((C_word*)t12)[1];
f_10563(t35,t34);}
else{
/* irregex-core.scm:1158: chicken.base#error */
t31=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t31;
av2[1]=t1;
av2[2]=lf[194];
av2[3]=((C_word*)t0)[2];
av2[4]=t29;
((C_proc)(void*)(*((C_word*)t31+1)))(5,av2);}}}
else{
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12995,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t30,a[5]=t31,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1160: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);}}}}
case C_make_character(124):
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t28=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t29=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13059,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t27,a[5]=t28,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1162: collect */
t30=((C_word*)t12)[1];
f_10563(t30,t29);
case C_make_character(94):
t27=C_fixnum_and(t4,C_fix(4));
t28=C_eqp(C_fix(4),t27);
t29=(C_truep(t28)?lf[195]:lf[186]);
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t32=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13085,a[2]=t29,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1165: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(36):
t27=C_fixnum_and(t4,C_fix(4));
t28=C_eqp(C_fix(4),t27);
t29=(C_truep(t28)?lf[196]:lf[187]);
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t31=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t32=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13114,a[2]=t29,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t30,a[6]=t31,a[7]=t4,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1168: collect */
t33=((C_word*)t12)[1];
f_10563(t33,t32);
case C_make_character(32):
t27=C_fixnum_and(t4,C_fix(16));
t28=C_eqp(C_fix(16),t27);
if(C_truep(t28)){
t29=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t31=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13142,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t29,a[5]=t30,a[6]=t4,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1171: collect */
t32=((C_word*)t12)[1];
f_10563(t32,t31);}
else{
t29=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1172: lp */
t225=t1;
t226=t29;
t227=t3;
t228=t4;
t229=t5;
t230=t6;
t1=t225;
t2=t226;
t3=t227;
t4=t228;
t5=t229;
t6=t230;
goto loop;}
case C_make_character(35):
t27=C_fixnum_and(t4,C_fix(16));
t28=C_eqp(C_fix(16),t27);
if(C_truep(t28)){
t29=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13163,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t4,a[6]=t6,a[7]=t12,tmp=(C_word)a,a+=8,tmp);
t30=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1175: string-scan-char */
t31=lf[54];{
C_word av2[5];
av2[0]=t31;
av2[1]=t29;
av2[2]=((C_word*)t0)[2];
av2[3]=C_make_character(10);
av2[4]=t30;
((C_proc)(void*)(*((C_word*)t31+1)))(5,av2);}}
else{
t29=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1178: lp */
t225=t1;
t226=t29;
t227=t3;
t228=t4;
t229=t5;
t230=t6;
t1=t225;
t2=t226;
t3=t227;
t4=t228;
t5=t229;
t6=t230;
goto loop;}
default:
t27=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1180: lp */
t225=t1;
t226=t27;
t227=t3;
t228=t4;
t229=t5;
t230=t6;
t1=t225;
t2=t226;
t3=t227;
t4=t228;
t5=t229;
t6=t230;
goto loop;}}}}}

/* cased-char in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static C_word C_fcall f_10485(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;{}
t2=C_fixnum_and(((C_word*)t0)[2],C_fix(2));
t3=C_eqp(C_fix(2),t2);
t4=(C_truep(t3)?C_u_i_char_alphabeticp(t1):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_u_i_char_upper_casep(t1);
t6=(C_truep(t5)?C_u_i_char_downcase(t1):C_u_i_char_upcase(t1));
return(C_a_i_list(&a,3,lf[103],t1,t6));}
else{
return(t1);}}

/* cased-string in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10507(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_10507,3,t0,t1,t2);}
a=C_alloc(11);
t3=C_fixnum_and(((C_word*)t0)[2],C_fix(2));
t4=C_eqp(C_fix(2),t3);
if(C_truep(t4)){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10524,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t7,a[5]=t8,tmp=(C_word)a,a+=6,tmp);
/* ##sys#string->list */
t10=*((C_word*)lf[105]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k10522 in cased-string in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10524(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_10524,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10527,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10529,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t4,a[5]=((C_word*)t0)[5],a[6]=((C_word)li65),tmp=(C_word)a,a+=7,tmp));
t6=((C_word*)t4)[1];
f_10529(t6,t2,t1);}

/* k10525 in k10522 in cased-string in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10527(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10527,c,av);}
a=C_alloc(3);
/* irregex-core.scm:762: sre-sequence */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* irregex-core.scm:762: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop1471 in k10522 in cased-string in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10529(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_10529,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=(
/* irregex-core.scm:762: g1477 */
  f_10485(C_a_i(&a,9),((C_word*)((C_word*)t0)[2])[1],C_slot(t2,C_fix(0)))
);
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* collect in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10563(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_10563,2,t0,t1);}
a=C_alloc(8);
t2=C_eqp(((C_word*)t0)[2],((C_word*)t0)[3]);
if(C_truep(t2)){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10577,a[2]=t1,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10581,a[2]=((C_word*)t0)[5],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:766: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k10575 in collect in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10577,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10579 in collect in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10581,c,av);}
/* irregex-core.scm:766: cased-string */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10507(t2,((C_word*)t0)[3],t1);}

/* collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10583(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,3)))){
C_save_and_reclaim_args((void *)trf_10583,2,t0,t1);}
a=C_alloc(17);
t2=C_fixnum_and(((C_word*)t0)[2],C_fix(32));
t3=C_eqp(C_fix(32),t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10590,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t3,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t5=(C_truep(t3)?C_fixnum_greaterp(((C_word*)t0)[8],C_fix(1)):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_fixnum_difference(((C_word*)t0)[8],C_fix(1));
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14224,a[2]=((C_word*)t0)[6],a[3]=t8,a[4]=((C_word)li68),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_14224(t10,t4,t6);}
else{
t6=t4;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_fixnum_difference(((C_word*)t0)[8],C_fix(1));
f_10590(2,av2);}}}

/* k10588 in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_10590,c,av);}
a=C_alloc(21);
if(C_truep(C_fixnum_lessp(t1,((C_word*)t0)[2]))){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10599,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10627,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[8])){
/* irregex-core.scm:779: utf8-string-ref */
f_14029(t3,((C_word*)t0)[6],t1,C_fixnum_difference(((C_word*)t0)[9],t1));}
else{
/* irregex-core.scm:778: cased-char */
t4=t2;
f_10599(t4,(
/* irregex-core.scm:778: cased-char */
  f_10485(C_a_i(&a,9),((C_word*)((C_word*)t0)[7])[1],C_i_string_ref(((C_word*)t0)[6],t1))
));}}}

/* k10597 in k10588 in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10599(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_10599,2,t0,t1);}
a=C_alloc(9);
t2=C_eqp(((C_word*)t0)[2],((C_word*)t0)[3]);
if(C_truep(t2)){
t3=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10619,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10623,a[2]=((C_word*)t0)[6],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:786: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k10617 in k10597 in k10588 in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10619(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10619,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10621 in k10597 in k10588 in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10623,c,av);}
/* irregex-core.scm:786: cased-string */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10507(t2,((C_word*)t0)[3],t1);}

/* k10625 in k10588 in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10627,c,av);}
a=C_alloc(9);
/* irregex-core.scm:778: cased-char */
t2=((C_word*)t0)[2];
f_10599(t2,(
/* irregex-core.scm:778: cased-char */
  f_10485(C_a_i(&a,9),((C_word*)((C_word*)t0)[3])[1],t1)
));}

/* collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10655(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_10655,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10659,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:792: collect */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10563(t3,t2);}

/* k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_10659,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10662,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(t1))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10885,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:795: last */
f_9256(t3,t1);}
else{
t3=t2;
f_10662(t3,C_SCHEME_FALSE);}}

/* k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10662(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_10662,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10665,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(C_truep(t1)?C_u_i_memq(C_i_car(t1),lf[121]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10863,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:801: ##sys#fast-reverse */
t5=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
if(C_truep(t1)){
t4=C_i_car(t1);
t5=t2;
f_10665(t5,C_a_i_list1(&a,1,t4));}
else{
t4=t2;
f_10665(t4,C_SCHEME_FALSE);}}}

/* k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10665(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_10665,2,t0,t1);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10668,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=C_i_car(((C_word*)t0)[4]);
if(C_truep((C_truep(C_eqp(t3,lf[119]))?C_SCHEME_TRUE:(C_truep(C_eqp(t3,lf[120]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10838,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:805: ##sys#fast-reverse */
t5=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10849,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:806: ##sys#fast-reverse */
t5=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
f_10668(2,av2);}}}

/* k10666 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_10668,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10673,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word)li70),tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_10673(t5,((C_word*)t0)[4],t1,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST);}

/* lp in k10666 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10673(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(31,0,4)))){
C_save_and_reclaim_args((void *)trf_10673,5,t0,t1,t2,t3,t4);}
a=C_alloc(31);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10676,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_nullp(t2))){
t6=(
/* irregex-core.scm:813: shift */
  f_10676(C_a_i(&a,6),t5)
);
t7=(
/* irregex-core.scm:813: sre-alternate */
  f_18214(C_a_i(&a,3),t6)
);
t8=C_fixnum_and(((C_word*)t0)[2],C_fix(1));
t9=C_eqp(C_fix(1),t8);
t10=(C_truep(t9)?C_a_i_list2(&a,2,lf[108],t7):t7);
if(C_truep(((C_word*)t0)[3])){
t11=C_i_car(((C_word*)t0)[3]);
t12=C_eqp(lf[109],t11);
if(C_truep(t12)){
t13=C_i_pairp(t10);
if(C_truep(C_i_not(t13))){
t14=t1;{
C_word av2[2];
av2[0]=t14;
av2[1]=lf[110];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t14=C_i_car(t10);
if(C_truep((C_truep(C_eqp(t14,lf[111]))?C_SCHEME_TRUE:(C_truep(C_eqp(t14,lf[112]))?C_SCHEME_TRUE:(C_truep(C_eqp(t14,lf[113]))?C_SCHEME_TRUE:(C_truep(C_eqp(t14,lf[114]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))){
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t15=C_eqp(lf[115],C_u_i_car(t10));
if(C_truep(t15)){
t16=C_i_cadr(t10);
t17=C_u_i_cdr(t10);
t18=(
/* irregex-core.scm:828: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t17))
);
t19=t1;{
C_word av2[2];
av2[0]=t19;
av2[1]=C_a_i_list(&a,3,lf[109],t16,t18);
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t16=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10746,a[2]=t10,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:830: scheme#cadadr */
t17=*((C_word*)lf[117]+1);{
C_word av2[3];
av2[0]=t17;
av2[1]=t16;
av2[2]=t10;
((C_proc)(void*)(*((C_word*)t17+1)))(3,av2);}}}}}
else{
t13=C_a_i_list(&a,1,t10);
/* irregex-core.scm:833: ##sys#append */
t14=*((C_word*)lf[118]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}}
else{
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}
else{
t6=C_i_car(t2);
t7=C_eqp(lf[103],t6);
if(C_truep(t7)){
t8=C_u_i_cdr(t2);
t9=(
/* irregex-core.scm:835: shift */
  f_10676(C_a_i(&a,6),t5)
);
/* irregex-core.scm:835: lp */
t20=t1;
t21=t8;
t22=C_SCHEME_END_OF_LIST;
t23=t9;
t1=t20;
t2=t21;
t3=t22;
t4=t23;
goto loop;}
else{
t8=C_u_i_cdr(t2);
t9=C_u_i_car(t2);
t10=C_a_i_cons(&a,2,t9,t3);
/* irregex-core.scm:836: lp */
t20=t1;
t21=t8;
t22=t10;
t23=t4;
t1=t20;
t2=t21;
t3=t22;
t4=t23;
goto loop;}}}

/* shift in lp in k10666 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static C_word C_fcall f_10676(C_word *a,C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_stack_overflow_check;{}
t1=(
/* irregex-core.scm:810: sre-sequence */
  f_18191(C_a_i(&a,3),((C_word*)t0)[2])
);
return(C_a_i_cons(&a,2,t1,((C_word*)t0)[3]));}

/* k10744 in lp in k10666 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10746(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10746,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10762,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:831: scheme#cddadr */
t3=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10760 in k10744 in lp in k10666 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,1)))){
C_save_and_reclaim((void *)f_10762,c,av);}
a=C_alloc(18);
t2=(
/* irregex-core.scm:831: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
t3=(
/* irregex-core.scm:832: sre-alternate */
  f_18214(C_a_i(&a,3),C_i_cddr(((C_word*)t0)[2]))
);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list(&a,4,lf[109],((C_word*)t0)[4],t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10836 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10838(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10838,c,av);}
/* irregex-core.scm:805: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cddr(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k10847 in k10663 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10849(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10849,c,av);}
/* irregex-core.scm:806: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cdr(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k10861 in k10660 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10863,c,av);}
a=C_alloc(6);
t2=C_i_cadr(t1);
t3=((C_word*)t0)[2];
f_10665(t3,C_a_i_list2(&a,2,lf[120],t2));}

/* k10883 in k10657 in collect/terms in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10885,c,av);}
t2=((C_word*)t0)[2];
f_10662(t2,C_u_i_memq(t1,lf[122]));}

/* save in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10888(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_10888,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10900,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:838: collect */
t3=((C_word*)((C_word*)t0)[4])[1];
f_10563(t3,t2);}

/* k10898 in save in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10900(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10900,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10958 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_10960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,4)))){
C_save_and_reclaim((void *)f_10960,c,av);}
a=C_alloc(28);
if(C_truep(C_i_nullp(t1))){
/* irregex-core.scm:855: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[126];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=C_i_car(t1);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10989,a[2]=t1,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=t4,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_pairp(t2))){
t6=C_u_i_car(t2);
t7=C_eqp(t6,lf[127]);
if(C_truep(t7)){
t8=C_u_i_cdr(t2);
t9=t5;
f_10989(t9,C_a_i_cons(&a,2,lf[128],t8));}
else{
t8=C_eqp(t6,lf[129]);
if(C_truep(t8)){
t9=C_u_i_cdr(t2);
t10=C_a_i_cons(&a,2,C_SCHEME_FALSE,t9);
t11=C_a_i_cons(&a,2,C_fix(1),t10);
t12=t5;
f_10989(t12,C_a_i_cons(&a,2,lf[130],t11));}
else{
t9=C_eqp(t6,lf[131]);
if(C_truep(t9)){
t10=C_u_i_cdr(t2);
t11=t5;
f_10989(t11,C_a_i_cons(&a,2,lf[132],t10));}
else{
t10=C_eqp(t6,lf[133]);
if(C_truep(t10)){
t11=C_u_i_cdr(t2);
t12=t5;
f_10989(t12,C_a_i_cons(&a,2,lf[130],t11));}
else{
t11=C_eqp(t6,lf[134]);
if(C_truep(t11)){
t12=C_i_cadr(t2);
t13=C_u_i_cdr(t2);
t14=C_a_i_cons(&a,2,t12,t13);
t15=t5;
f_10989(t15,C_a_i_cons(&a,2,lf[130],t14));}
else{
t12=C_eqp(t6,lf[135]);
if(C_truep(t12)){
t13=C_i_cadr(t2);
t14=C_u_i_cdr(t2);
t15=C_u_i_cdr(t14);
t16=C_a_i_cons(&a,2,C_SCHEME_FALSE,t15);
t17=C_a_i_cons(&a,2,t13,t16);
t18=t5;
f_10989(t18,C_a_i_cons(&a,2,lf[130],t17));}
else{
t13=t5;
f_10989(t13,C_a_i_list(&a,2,lf[131],t2));}}}}}}}
else{
t6=t5;
f_10989(t6,C_a_i_list(&a,2,lf[131],t2));}}}

/* k10987 in k10958 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_10989(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,6)))){
C_save_and_reclaim_args((void *)trf_10989,2,t0,t1);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
/* irregex-core.scm:857: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_10482(t4,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t3,((C_word*)t0)[8]);}

/* k11118 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11120,c,av);}
a=C_alloc(12);
t2=C_i_pairp(t1);
t3=(C_truep(t2)?C_u_i_car(t1):lf[110]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11126,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
t5=C_a_i_string(&a,1,((C_word*)t0)[8]);
/* irregex-core.scm:876: scheme#string->symbol */
t6=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k11124 in k11118 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_11126,c,av);}
a=C_alloc(11);
t2=(
/* irregex-core.scm:878: sre-repeater? */
  f_16575(((C_word*)t0)[2])
);
if(C_truep(t2)){
/* irregex-core.scm:879: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[137];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11141,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:880: sre-empty? */
t4=lf[139];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
f_16307(3,av2);}}}

/* k11139 in k11124 in k11118 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,6)))){
C_save_and_reclaim((void *)f_11141,c,av);}
a=C_alloc(19);
if(C_truep(t1)){
/* irregex-core.scm:881: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[138];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[5],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[5],C_fix(1));
t4=C_a_i_list2(&a,2,((C_word*)t0)[6],((C_word*)t0)[7]);
t5=C_i_cdr(((C_word*)t0)[4]);
t6=C_a_i_cons(&a,2,t4,t5);
/* irregex-core.scm:883: lp */
t7=((C_word*)((C_word*)t0)[8])[1];
f_10482(t7,((C_word*)t0)[2],t2,t3,((C_word*)t0)[9],t6,((C_word*)t0)[10]);}}

/* k11210 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11212,c,av);}
/* irregex-core.scm:891: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_END_OF_LIST,t1);}

/* a11241 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_11242,c,av);}
a=C_alloc(10);
t4=C_fixnum_greater_or_equal_p(t3,((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11252,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t4)){
t6=t5;
f_11252(t6,t4);}
else{
t6=C_i_string_ref(((C_word*)t0)[3],t3);
t7=t5;
f_11252(t7,C_i_not(C_eqp(C_make_character(41),t6)));}}

/* k11250 in a11241 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_11252(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_11252,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
/* irregex-core.scm:899: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[143];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_fixnum_plus(((C_word*)t0)[4],C_fix(1));
t3=C_fixnum_plus(((C_word*)t0)[4],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11274,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t3,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:900: collect */
t5=((C_word*)((C_word*)t0)[9])[1];
f_10563(t5,t4);}}

/* k11272 in k11250 in a11241 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_11274,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:900: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k11307 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_11309,c,av);}
a=C_alloc(8);
t2=C_fixnum_plus(t1,((C_word*)t0)[2]);
t3=C_fixnum_plus(t1,C_fix(1));
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11324,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:906: collect */
t5=((C_word*)((C_word*)t0)[7])[1];
f_10563(t5,t4);}

/* k11322 in k11307 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11324,c,av);}
/* irregex-core.scm:906: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,((C_word*)t0)[7]);}

/* k11348 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11350,c,av);}
/* irregex-core.scm:908: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_END_OF_LIST,t1);}

/* k11371 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11373,c,av);}
/* irregex-core.scm:910: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[145],t1);}

/* k11394 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11396,c,av);}
/* irregex-core.scm:913: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[146],t1);}

/* k11436 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11438,c,av);}
/* irregex-core.scm:922: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[148],t1);}

/* k11459 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11461,c,av);}
/* irregex-core.scm:925: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[149],t1);}

/* k11462 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11464(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_11464,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_fixnum_plus(t1,C_fix(1));
t3=C_fixnum_plus(t1,C_fix(1));
t4=C_fixnum_not(C_fix(1));
t5=C_fixnum_and(((C_word*)t0)[2],t4);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11494,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t3,a[6]=t5,a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11498,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
t8=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[6],C_fix(3));
/* irregex-core.scm:933: scheme#substring */
t9=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t7;
av2[2]=((C_word*)t0)[7];
av2[3]=t8;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
/* irregex-core.scm:936: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[150];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k11488 in k11492 in k11462 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11490,c,av);}
/* irregex-core.scm:932: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t1);}

/* k11492 in k11462 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_11494,c,av);}
a=C_alloc(14);
t2=C_a_i_list(&a,2,t1,lf[120]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11490,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:935: save */
t4=((C_word*)((C_word*)t0)[7])[1];
f_10888(t4,t3);}

/* k11496 in k11462 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11498,c,av);}
/* irregex-core.scm:933: scheme#string->symbol */
t2=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11548 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11550(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11550,c,av);}
/* irregex-core.scm:938: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[151],t1);}

/* k11573 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_11575,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11578,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11612,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[7],C_fix(3));
/* irregex-core.scm:950: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=t4;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k11576 in k11573 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_11578,c,av);}
a=C_alloc(24);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:952: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[153];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_fixnum_not(C_fix(1));
t5=C_fixnum_and(((C_word*)t0)[5],t4);
t6=C_a_i_list(&a,2,t1,lf[109]);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11608,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t3,a[6]=t5,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:954: save */
t8=((C_word*)((C_word*)t0)[7])[1];
f_10888(t8,t7);}}

/* k11606 in k11576 in k11573 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11608,c,av);}
/* irregex-core.scm:953: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t1);}

/* k11610 in k11573 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11612,c,av);}
/* irregex-core.scm:950: scheme#string->number */
t2=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11625 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_11627,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11630,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11655,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[6],C_fix(3));
/* irregex-core.scm:957: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=t4;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k11628 in k11625 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,2)))){
C_save_and_reclaim((void *)f_11630,c,av);}
a=C_alloc(24);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t4=C_fixnum_not(C_fix(1));
t5=C_fixnum_and(((C_word*)t0)[3],t4);
t6=C_a_i_list(&a,2,t1,lf[109]);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11651,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=t3,a[6]=t5,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:959: save */
t8=((C_word*)((C_word*)t0)[6])[1];
f_10888(t8,t7);}

/* k11649 in k11628 in k11625 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11651,c,av);}
/* irregex-core.scm:958: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t1);}

/* k11653 in k11625 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11655,c,av);}
/* irregex-core.scm:957: scheme#string->symbol */
t2=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11676 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11678,c,av);}
/* irregex-core.scm:961: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],lf[154],t1);}

/* lp2 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_11709(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(38,0,4)))){
C_save_and_reclaim_args((void *)trf_11709,5,t0,t1,t2,t3,t4);}
a=C_alloc(38);
t5=C_SCHEME_UNDEFINED;
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11712,a[2]=t4,a[3]=t3,a[4]=((C_word)li74),tmp=(C_word)a,a+=5,tmp);
t9=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11721,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word)li75),tmp=(C_word)a,a+=5,tmp));
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[3]))){
/* irregex-core.scm:978: chicken.base#error */
t10=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t1;
av2[2]=lf[158];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t10=C_i_string_ref(((C_word*)t0)[4],t2);
switch(t10){
case C_make_character(105):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=(
/* irregex-core.scm:982: join */
  f_11712(t5,C_fix(2))
);
/* irregex-core.scm:982: lp2 */
t28=t1;
t29=t11;
t30=t12;
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;
case C_make_character(109):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=(
/* irregex-core.scm:984: join */
  f_11712(t5,C_fix(4))
);
/* irregex-core.scm:984: lp2 */
t28=t1;
t29=t11;
t30=t12;
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;
case C_make_character(120):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=(
/* irregex-core.scm:986: join */
  f_11712(t5,C_fix(16))
);
/* irregex-core.scm:986: lp2 */
t28=t1;
t29=t11;
t30=t12;
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;
case C_make_character(117):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=(
/* irregex-core.scm:989: join */
  f_11712(t5,C_fix(32))
);
/* irregex-core.scm:989: lp2 */
t28=t1;
t29=t11;
t30=t12;
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;
case C_make_character(45):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:992: lp2 */
t28=t1;
t29=t11;
t30=t3;
t31=C_i_not(t4);
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;
case C_make_character(41):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t13=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11865,a[2]=t7,a[3]=((C_word*)t0)[7],a[4]=t1,a[5]=t11,a[6]=t12,a[7]=t3,a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:994: collect */
t14=((C_word*)((C_word*)t0)[9])[1];
f_10563(t14,t13);
case C_make_character(58):
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t13=(
/* irregex-core.scm:997: new-res */
  f_11721(C_a_i(&a,6),((C_word*)t7)[1],C_SCHEME_END_OF_LIST)
);
t14=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11896,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[7],a[5]=t1,a[6]=t11,a[7]=t12,a[8]=t3,a[9]=t13,tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:998: collect */
t15=((C_word*)((C_word*)t0)[9])[1];
f_10563(t15,t14);
default:
/* irregex-core.scm:1000: chicken.base#error */
t11=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t11;
av2[1]=t1;
av2[2]=lf[159];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}}}

/* join in lp2 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static C_word C_fcall f_11712(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
if(C_truep(((C_word*)t0)[2])){
t2=lf[64];
t3=lf[64];
return(C_fixnum_and(((C_word*)t0)[3],C_fixnum_not(t1)));}
else{
t2=lf[63];
t3=lf[63];
return((C_truep(t1)?C_fixnum_or(((C_word*)t0)[3],t1):((C_word*)t0)[3]));}}

/* new-res in lp2 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static C_word C_fcall f_11721(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;{}
t2=C_fixnum_and(((C_word*)t0)[2],C_fix(32));
t3=C_eqp(C_fix(32),t2);
t4=C_fixnum_and(((C_word*)t0)[3],C_fix(32));
t5=C_eqp(C_fix(32),t4);
t6=C_eqp(t3,t5);
return((C_truep(t6)?t1:(C_truep(t5)?C_a_i_cons(&a,2,lf[156],t1):C_a_i_cons(&a,2,lf[157],t1))));}

/* k11863 in lp2 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_11865,c,av);}
a=C_alloc(6);
t2=(
/* irregex-core.scm:994: new-res */
  f_11721(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1)
);
/* irregex-core.scm:994: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k11894 in lp2 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_11896,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
/* irregex-core.scm:997: lp */
t4=((C_word*)((C_word*)t0)[4])[1];
f_10482(t4,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],t3);}

/* k11985 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_11987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_11987,c,av);}
a=C_alloc(3);
t2=C_i_cdar(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
/* irregex-core.scm:1005: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_10482(t4,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t3,C_u_i_cdr(((C_word*)t0)[2]));}

/* a12001 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12002,c,av);}
a=C_alloc(9);
t4=C_fixnum_plus(t3,C_fix(1));
t5=C_fixnum_plus(t3,C_fix(1));
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12022,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,a[6]=t5,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1013: collect */
t7=((C_word*)((C_word*)t0)[5])[1];
f_10563(t7,t6);}

/* k12020 in a12001 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12022,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1013: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12024 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12026,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_12045(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,6)))){
C_save_and_reclaim_args((void *)trf_12045,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:1020: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12054,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[9],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1022: collect/single */
t3=((C_word*)((C_word*)t0)[10])[1];
f_10583(t3,t2);}}

/* k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12054(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_12054,c,av);}
a=C_alloc(15);
if(C_truep(C_i_nullp(t1))){
/* irregex-core.scm:1025: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[174];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_i_car(t1);
t3=C_u_i_cdr(t1);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12070,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
t5=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[6],C_fix(1));
/* irregex-core.scm:1029: string-scan-char */
t6=lf[54];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=C_make_character(125);
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}

/* k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_12070,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12073,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12198,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[8],C_fix(1));
/* irregex-core.scm:1030: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
av2[3]=t4;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k12071 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12073(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_12073,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12076,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1032: scheme#string->number */
t3=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12074 in k12071 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_12076,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12079,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[3])))){
/* irregex-core.scm:1034: scheme#string->number */
t3=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_12079(2,av2);}}}

/* k12077 in k12074 in k12071 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_12079,c,av);}
a=C_alloc(12);
t2=C_i_not(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_12088,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,tmp=(C_word)a,a+=12,tmp);
if(C_truep(t2)){
t4=t3;
f_12088(t4,t2);}
else{
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[4])))){
t4=C_i_cadr(((C_word*)t0)[4]);
t5=C_i_equalp(lf[176],t4);
t6=C_i_not(t5);
t7=t3;
f_12088(t7,(C_truep(t6)?C_i_not(t1):C_SCHEME_FALSE));}
else{
t4=t3;
f_12088(t4,C_SCHEME_FALSE);}}}

/* k12086 in k12077 in k12074 in k12071 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_12088(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,6)))){
C_save_and_reclaim_args((void *)trf_12088,2,t0,t1);}
a=C_alloc(25);
if(C_truep(t1)){
/* irregex-core.scm:1040: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[175];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_a_i_list(&a,3,lf[134],((C_word*)t0)[5],((C_word*)t0)[6]);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[7]);
/* irregex-core.scm:1042: lp */
t6=((C_word*)((C_word*)t0)[8])[1];
f_10482(t6,((C_word*)t0)[2],t2,t3,((C_word*)t0)[9],t5,((C_word*)t0)[10]);}
else{
if(C_truep(((C_word*)t0)[11])){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_a_i_list(&a,4,lf[133],((C_word*)t0)[5],((C_word*)t0)[11],((C_word*)t0)[6]);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[7]);
/* irregex-core.scm:1044: lp */
t6=((C_word*)((C_word*)t0)[8])[1];
f_10482(t6,((C_word*)t0)[2],t2,t3,((C_word*)t0)[9],t5,((C_word*)t0)[10]);}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_a_i_list(&a,3,lf[135],((C_word*)t0)[5],((C_word*)t0)[6]);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[7]);
/* irregex-core.scm:1046: lp */
t6=((C_word*)((C_word*)t0)[8])[1];
f_10482(t6,((C_word*)t0)[2],t2,t3,((C_word*)t0)[9],t5,((C_word*)t0)[10]);}}}}

/* k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_12198,c,av);}
a=C_alloc(8);
t2=C_i_string_length(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8968,a[2]=t1,a[3]=t2,a[4]=t4,a[5]=((C_word)li84),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_8968(t6,((C_word*)t0)[2],C_fix(0),C_fix(0),C_SCHEME_END_OF_LIST);}

/* k12272 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12274,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[178],t1);
/* irregex-core.scm:1056: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12299 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12301(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12301,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1058: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12322 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12324,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[79],t1);
/* irregex-core.scm:1060: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12349 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12351,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1062: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12376 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12378,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1064: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12407 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12409,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1067: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12442 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12444,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1070: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12465 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12467,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[185],t1);
/* irregex-core.scm:1073: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12488 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12490,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[186],t1);
/* irregex-core.scm:1075: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12519 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_12521,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[187],t1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
/* irregex-core.scm:1077: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_10482(t4,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t3,((C_word*)t0)[8]);}

/* k12542 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12544,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[187],t1);
/* irregex-core.scm:1080: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12565 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12567,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[80],t1);
/* irregex-core.scm:1082: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12588 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12590,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[188],t1);
/* irregex-core.scm:1084: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12611 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12613(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12613,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[183],t1);
/* irregex-core.scm:1087: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12634 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12636,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[184],t1);
/* irregex-core.scm:1089: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* a12646 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12647,c,av);}
a=C_alloc(9);
t4=C_fixnum_plus(t3,C_fix(1));
t5=C_fixnum_plus(t3,C_fix(1));
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12667,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,a[6]=t5,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1093: collect */
t7=((C_word*)((C_word*)t0)[5])[1];
f_10563(t7,t6);}

/* k12665 in a12646 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12667(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12667,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1093: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12669 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12671,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* k12693 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_12695(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_12695,2,t0,t1);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12698,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[8],C_fix(2));
/* irregex-core.scm:1100: string-scan-char */
t4=lf[54];{
C_word av2[5];
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12696 in k12693 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12698(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_12698,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12701,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[8],C_fix(3));
/* irregex-core.scm:1101: scheme#substring */
t4=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=t3;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_12701(2,av2);}}}

/* k12699 in k12696 in k12693 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12701(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12701,c,av);}
a=C_alloc(10);
t2=C_fixnum_and(((C_word*)t0)[2],C_fix(2));
t3=C_eqp(C_fix(2),t2);
t4=(C_truep(t3)?lf[191]:lf[192]);
if(C_truep(C_i_not(((C_word*)t0)[3]))){
/* irregex-core.scm:1107: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[193];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=C_fixnum_plus(((C_word*)t0)[3],C_fix(1));
t6=C_fixnum_plus(((C_word*)t0)[3],C_fix(1));
t7=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12740,a[2]=t4,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],a[5]=t5,a[6]=t6,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1109: scheme#string->symbol */
t8=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}

/* k12734 in k12738 in k12699 in k12696 in k12693 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12736,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1108: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12738 in k12699 in k12696 in k12693 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12740(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_12740,c,av);}
a=C_alloc(15);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12736,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1110: collect */
t4=((C_word*)((C_word*)t0)[9])[1];
f_10563(t4,t3);}

/* k12765 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_12767,c,av);}
a=C_alloc(18);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12775,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=t4,a[10]=((C_word)li86),tmp=(C_word)a,a+=11,tmp));
t6=((C_word*)t4)[1];
f_12775(t6,((C_word*)t0)[8],t2);}

/* lp2 in k12765 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_12775(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,6)))){
C_save_and_reclaim_args((void *)trf_12775,3,t0,t1,t2);}
a=C_alloc(29);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
/* irregex-core.scm:1117: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,t1,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(2)),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}
else{
t3=C_i_string_ref(((C_word*)t0)[8],t2);
t4=C_eqp(C_make_character(92),t3);
if(C_truep(t4)){
t5=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
if(C_truep(C_i_integer_greater_or_equalp(t5,((C_word*)t0)[2]))){
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1121: lp */
t7=((C_word*)((C_word*)t0)[3])[1];
f_10482(t7,t1,t6,C_fixnum_plus(((C_word*)t0)[4],C_fix(2)),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}
else{
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t7=C_i_string_ref(((C_word*)t0)[8],t6);
t8=C_eqp(C_make_character(69),t7);
if(C_truep(t8)){
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t10=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12835,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t9,a[6]=t10,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1124: scheme#substring */
t12=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t12;
av2[1]=t11;
av2[2]=((C_word*)t0)[8];
av2[3]=C_fixnum_plus(((C_word*)t0)[4],C_fix(2));
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}
else{
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
/* irregex-core.scm:1126: lp2 */
t13=t1;
t14=t9;
t1=t13;
t2=t14;
goto loop;}}}
else{
t5=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:1128: lp2 */
t13=t1;
t14=t5;
t1=t13;
t2=t14;
goto loop;}}}

/* k12833 in lp2 in k12765 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12835,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:1123: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* a12877 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_12878,c,av);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12890,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1132: collect */
t5=((C_word*)((C_word*)t0)[5])[1];
f_10563(t5,t4);}

/* k12888 in a12877 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12890,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1132: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12897 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_12899,c,av);}
a=C_alloc(17);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=C_fixnum_and(((C_word*)t0)[3],C_fix(2));
t4=C_eqp(C_fix(2),t3);
t5=(C_truep(t4)?lf[191]:lf[192]);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12923,a[2]=t5,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12927,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
t8=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[8],C_fix(1));
/* irregex-core.scm:1150: scheme#substring */
t9=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t7;
av2[2]=((C_word*)t0)[9];
av2[3]=t8;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* k12917 in k12921 in k12897 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12919,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1152: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k12921 in k12897 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_12923,c,av);}
a=C_alloc(14);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12919,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1151: collect */
t4=((C_word*)((C_word*)t0)[8])[1];
f_10563(t4,t3);}

/* k12925 in k12897 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12927(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12927,c,av);}
/* irregex-core.scm:1149: scheme#string->number */
t2=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12977 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_12979,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1156: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k12993 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_12995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_12995,c,av);}
/* irregex-core.scm:1160: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,((C_word*)t0)[7]);}

/* k13057 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_13059,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[103],t1);
/* irregex-core.scm:1162: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10482(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t2,((C_word*)t0)[7]);}

/* k13083 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_13085,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1165: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k13112 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_13114,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1168: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10482(t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t2,((C_word*)t0)[8]);}

/* k13140 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_13142,c,av);}
/* irregex-core.scm:1171: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,((C_word*)t0)[7]);}

/* k13161 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13163(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_13163,c,av);}
a=C_alloc(13);
t2=(C_truep(t1)?t1:C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],C_fix(1)));
t3=C_fixnum_plus(t2,C_fix(1));
t4=C_fixnum_plus(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13181,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=t4,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1177: collect */
t6=((C_word*)((C_word*)t0)[7])[1];
f_10563(t6,t5);}

/* k13179 in k13161 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_13181,c,av);}
/* irregex-core.scm:1177: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10482(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,((C_word*)t0)[7]);}

/* chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_fcall f_13295(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,4)))){
C_save_and_reclaim_args((void *)trf_13295,4,t1,t2,t3,t4);}
a=C_alloc(21);
if(C_truep(C_fixnum_greater_or_equal_p(t3,t4))){
/* irregex-core.scm:1200: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[199];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=C_i_string_ref(t2,t3);
t6=C_eqp(C_make_character(123),t5);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13314,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t8=C_a_i_fixnum_plus(&a,2,t3,C_fix(1));
t9=C_a_i_list(&a,1,t8);
t10=C_i_string_length(t2);
t11=C_i_pairp(t9);
t12=(C_truep(t11)?C_u_i_car(t9):C_fix(0));
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8873,a[2]=t10,a[3]=t2,a[4]=t14,a[5]=((C_word)li91),tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_8873(t16,t7,t12);}
else{
t7=C_a_i_fixnum_plus(&a,2,t3,C_fix(1));
if(C_truep(C_fixnum_greater_or_equal_p(t7,t4))){
/* irregex-core.scm:1211: chicken.base#error */
t8=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t8;
av2[1]=t1;
av2[2]=lf[202];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13360,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t9=C_a_i_fixnum_plus(&a,2,t3,C_fix(2));
/* irregex-core.scm:1213: scheme#substring */
t10=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t8;
av2[2]=t2;
av2[3]=t3;
av2[4]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}}}}

/* k13312 in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_ccall f_13314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13314,c,av);}
a=C_alloc(9);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:1204: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[200];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13326,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
/* irregex-core.scm:1205: scheme#substring */
t4=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k13324 in k13312 in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_ccall f_13326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13326,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13329,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1206: scheme#string->number */
t3=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=C_fix(16);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k13327 in k13324 in k13312 in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_ccall f_13329(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_13329,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_make_character(C_unfix(t1));
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list2(&a,2,t2,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* irregex-core.scm:1209: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[201];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k13358 in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_ccall f_13360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13360,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13363,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1214: scheme#string->number */
t3=*((C_word*)lf[85]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=C_fix(16);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k13361 in k13358 in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_ccall f_13363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_13363,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_make_character(C_unfix(t1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list2(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* irregex-core.scm:1217: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[203];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13404(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,5)))){
C_save_and_reclaim_args((void *)trf_13404,5,t0,t1,t2,t3,t4);}
a=C_alloc(25);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
/* irregex-core.scm:1225: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[6];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[161];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
av2[5]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
t5=C_i_string_ref(((C_word*)t0)[3],t2);
switch(t5){
case C_make_character(93):
t6=C_i_vector_length(t4);
t7=C_eqp(t6,C_fix(0));
if(C_truep(t7)){
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13439,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t8,tmp=(C_word)a,a+=5,tmp);
t10=C_a_i_cons(&a,2,C_make_character(93),C_make_character(93));
t11=C_a_i_vector1(&a,1,t10);
/* irregex-core.scm:3802: cset-union */
t12=lf[162];{
C_word av2[4];
av2[0]=t12;
av2[1]=t9;
av2[2]=t4;
av2[3]=t11;
f_27080(4,av2);}}
else{
t8=C_fixnum_and(((C_word*)t0)[5],C_fix(2));
t9=C_eqp(C_fix(2),t8);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13449,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t9)){
/* irregex-core.scm:1233: cset-case-insensitive */
f_27607(t10,t4);}
else{
t11=t10;{
C_word av2[2];
av2[0]=t11;
av2[1]=t4;
f_13449(2,av2);}}}
case C_make_character(45):
t6=C_eqp(t2,((C_word*)t0)[7]);
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_13477,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t5,a[6]=t4,a[7]=t3,a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t6)){
t8=t7;
f_13477(t8,t6);}
else{
t8=C_fixnum_plus(((C_word*)t0)[7],C_fix(1));
t9=C_eqp(t2,t8);
t10=(C_truep(t9)?C_eqp(C_make_character(94),C_i_string_ref(((C_word*)t0)[3],((C_word*)t0)[7])):C_SCHEME_FALSE);
if(C_truep(t10)){
t11=t7;
f_13477(t11,t10);}
else{
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=t7;
f_13477(t12,C_eqp(C_make_character(93),C_i_string_ref(((C_word*)t0)[3],t11)));}}
case C_make_character(91):
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t7=C_i_string_ref(((C_word*)t0)[3],t6);
t8=C_eqp(C_make_character(94),t7);
t9=(C_truep(t8)?C_a_i_fixnum_plus(&a,2,t2,C_fix(2)):C_a_i_fixnum_plus(&a,2,t2,C_fix(1)));
t10=C_i_string_ref(((C_word*)t0)[3],t9);
t11=C_eqp(t10,C_make_character(58));
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13679,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=t8,a[7]=t9,tmp=(C_word)a,a+=8,tmp);
t13=C_a_i_fixnum_plus(&a,2,t9,C_fix(1));
/* irregex-core.scm:1267: string-scan-char */
t14=lf[54];{
C_word av2[5];
av2[0]=t14;
av2[1]=t12;
av2[2]=((C_word*)t0)[3];
av2[3]=C_make_character(58);
av2[4]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(5,av2);}}
else{
t12=C_eqp(t10,C_make_character(61));
t13=(C_truep(t12)?t12:C_eqp(t10,C_make_character(46)));
if(C_truep(t13)){
/* irregex-core.scm:1276: chicken.base#error */
t14=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t14;
av2[1]=t1;
av2[2]=lf[173];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}
else{
t14=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13762,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t14,tmp=(C_word)a,a+=5,tmp);
t16=C_a_i_cons(&a,2,C_make_character(91),C_make_character(91));
t17=C_a_i_vector1(&a,1,t16);
/* irregex-core.scm:3802: cset-union */
t18=lf[162];{
C_word av2[4];
av2[0]=t18;
av2[1]=t15;
av2[2]=t4;
av2[3]=t17;
f_27080(4,av2);}}}
case C_make_character(92):
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t7=C_i_string_ref(((C_word*)t0)[3],t6);
t8=C_eqp(t7,C_make_character(100));
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13801,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t4,a[6]=t7,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[2],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t8)){
t10=t9;
f_13801(t10,t8);}
else{
t10=C_eqp(t7,C_make_character(68));
if(C_truep(t10)){
t11=t9;
f_13801(t11,t10);}
else{
t11=C_eqp(t7,C_make_character(115));
if(C_truep(t11)){
t12=t9;
f_13801(t12,t11);}
else{
t12=C_eqp(t7,C_make_character(83));
if(C_truep(t12)){
t13=t9;
f_13801(t13,t12);}
else{
t13=C_eqp(t7,C_make_character(119));
t14=t9;
f_13801(t14,(C_truep(t13)?t13:C_eqp(t7,C_make_character(87))));}}}}
default:
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13918,a[2]=t5,a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=t4,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[8])){
t7=C_fix(C_character_code(t5));
t8=t6;
f_13918(t8,C_and(C_fixnum_less_or_equal_p(C_fix(128),t7),C_fixnum_less_or_equal_p(t7,C_fix(255))));}
else{
t7=t6;
f_13918(t7,C_SCHEME_FALSE);}}}}

/* k13437 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13439,c,av);}
/* irregex-core.scm:1230: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(93),t1);}

/* k13447 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13449(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13449,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
/* irregex-core.scm:1234: cset-complement */
f_27597(t2,t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_13456(2,av2);}}}

/* k13454 in k13447 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_13456,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26937,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26939,a[2]=((C_word)li78),tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26957,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3751: scheme#vector->list */
t5=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13477(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(30,0,4)))){
C_save_and_reclaim_args((void *)trf_13477,2,t0,t1);}
a=C_alloc(30);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13487,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_cons(&a,2,((C_word*)t0)[5],((C_word*)t0)[5]);
t5=C_a_i_vector1(&a,1,t4);
/* irregex-core.scm:3802: cset-union */
t6=lf[162];{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=t5;
f_27080(4,av2);}}
else{
if(C_truep(C_i_not(((C_word*)t0)[7]))){
/* irregex-core.scm:1243: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[166];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_i_string_ref(((C_word*)t0)[8],t2);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13504,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=((C_word)li79),tmp=(C_word)a,a+=6,tmp);
t5=C_eqp(C_make_character(92),t3);
t6=(C_truep(t5)?C_i_assq(t3,lf[168]):C_SCHEME_FALSE);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13532,a[2]=((C_word*)t0)[4],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t6)){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13533,a[2]=((C_word*)t0)[2],a[3]=((C_word)li80),tmp=(C_word)a,a+=4,tmp);
t9=(
/* irregex-core.scm:1251: g1884 */
  f_13533(C_a_i(&a,6),t8,t6)
);{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t4;
av2[3]=t9;
C_apply(4,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13554,a[2]=((C_word*)t0)[2],a[3]=t7,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=t3,a[7]=((C_word*)t0)[4],a[8]=t4,a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t9=C_eqp(C_make_character(92),t3);
if(C_truep(t9)){
t10=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t11=t8;
f_13554(t11,C_eqp(C_i_string_ref(((C_word*)t0)[8],t10),C_make_character(120)));}
else{
t10=t8;
f_13554(t10,C_SCHEME_FALSE);}}}}}

/* k13485 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13487,c,av);}
/* irregex-core.scm:1241: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* a13503 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_13504,c,av);}
a=C_alloc(10);
if(C_truep(C_i_char_lessp(t2,((C_word*)t0)[2]))){
/* irregex-core.scm:1249: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[167];
av2[3]=((C_word*)t0)[2];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13521,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
t6=C_a_i_vector1(&a,1,t5);
/* irregex-core.scm:1250: cset-union */
t7=lf[162];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
av2[3]=t6;
f_27080(4,av2);}}}

/* k13519 in a13503 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13521,c,av);}
/* irregex-core.scm:1250: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,t1);}

/* k13530 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13532,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* g1884 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static C_word C_fcall f_13533(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
t2=C_i_cdr(t1);
t3=C_fixnum_plus(((C_word*)t0)[2],C_fix(3));
return(C_a_i_list2(&a,2,t2,t3));}

/* k13552 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13554(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_13554,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(3));
/* irregex-core.scm:1256: string-parse-hex-escape */
f_13295(((C_word*)t0)[3],((C_word*)t0)[4],t2,((C_word*)t0)[5]);}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13566,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[9])){
t3=C_fix(C_character_code(((C_word*)t0)[6]));
t4=t2;
f_13566(t4,C_and(C_fixnum_less_or_equal_p(C_fix(128),t3),C_fixnum_less_or_equal_p(t3,C_fix(255))));}
else{
t3=t2;
f_13566(t3,C_SCHEME_FALSE);}}}

/* k13564 in k13552 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13566(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_13566,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_fix(C_character_code(((C_word*)t0)[2]));
t3=C_i_vector_ref(lf[170],t2);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13576,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:1259: utf8-string-ref */
f_14029(t4,((C_word*)t0)[6],t5,t3);}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(2));
t3=C_a_i_list2(&a,2,((C_word*)t0)[2],t2);{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t3;
C_apply(4,av2);}}}

/* k13574 in k13564 in k13552 in k13475 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_13576,c,av);}
a=C_alloc(6);
t2=C_fixnum_plus(C_fixnum_plus(((C_word*)t0)[2],C_fix(1)),((C_word*)t0)[3]);
t3=C_a_i_list2(&a,2,t1,t2);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t3;
C_apply(4,av2);}}

/* k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_13679,c,av);}
a=C_alloc(9);
t2=C_i_not(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13688,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t2)){
t4=t3;
f_13688(t4,t2);}
else{
t4=C_fixnum_plus(t1,C_fix(1));
t5=C_i_string_ref(((C_word*)t0)[3],t4);
t6=t3;
f_13688(t6,C_i_not(C_eqp(C_make_character(93),t5)));}}

/* k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13688(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,4)))){
C_save_and_reclaim_args((void *)trf_13688,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
/* irregex-core.scm:1269: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[171];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13694,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13714,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13718,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[8],C_fix(1));
/* irregex-core.scm:1272: scheme#substring */
t6=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
av2[3]=t5;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}

/* k13692 in k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13694,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13697,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
/* irregex-core.scm:1273: cset-complement */
f_27597(t2,t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
f_13697(2,av2);}}}

/* k13695 in k13692 in k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13697(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_13697,c,av);}
a=C_alloc(10);
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13707,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1274: cset-union */
t4=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=t1;
f_27080(4,av2);}}

/* k13705 in k13695 in k13692 in k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13707,c,av);}
/* irregex-core.scm:1274: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,t1);}

/* k13712 in k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13714,c,av);}
/* irregex-core.scm:1270: sre->cset */
f_26492(((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* k13716 in k13686 in k13677 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13718(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13718,c,av);}
/* irregex-core.scm:1271: scheme#string->symbol */
t2=*((C_word*)lf[84]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k13760 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13762,c,av);}
/* irregex-core.scm:1278: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(91),t1);}

/* k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13801(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,4)))){
C_save_and_reclaim_args((void *)trf_13801,2,t0,t1);}
a=C_alloc(20);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13811,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13815,a[2]=t3,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13819,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_a_i_string(&a,2,C_make_character(92),((C_word*)t0)[6]);
/* irregex-core.scm:1285: string->sre */
t7=*((C_word*)lf[102]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[6],C_make_character(120));
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13834,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word)li81),tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13846,a[2]=((C_word*)t0)[4],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
/* irregex-core.scm:1290: string-parse-hex-escape */
f_13295(t4,((C_word*)t0)[7],t5,((C_word*)t0)[8]);}
else{
t3=C_i_assq(((C_word*)t0)[6],lf[168]);
t4=(C_truep(t3)?C_i_cdr(t3):((C_word*)t0)[6]);
t5=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2));
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13865,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t5,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t7=C_a_i_cons(&a,2,t4,t4);
t8=C_a_i_vector1(&a,1,t7);
/* irregex-core.scm:3802: cset-union */
t9=lf[162];{
C_word av2[4];
av2[0]=t9;
av2[1]=t6;
av2[2]=((C_word*)t0)[5];
av2[3]=t8;
f_27080(4,av2);}}}}

/* k13809 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13811,c,av);}
/* irregex-core.scm:1283: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,t1);}

/* k13813 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13815(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13815,c,av);}
/* irregex-core.scm:1284: cset-union */
t2=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
f_27080(4,av2);}}

/* k13817 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13819,c,av);}
/* irregex-core.scm:1285: sre->cset */
f_26492(((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* a13833 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_13834,c,av);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13842,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_cons(&a,2,t2,t2);
t6=C_a_i_vector1(&a,1,t5);
/* irregex-core.scm:3802: cset-union */
t7=lf[162];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
av2[3]=t6;
f_27080(4,av2);}}

/* k13840 in a13833 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13842,c,av);}
/* irregex-core.scm:1289: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k13844 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13846,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* k13863 in k13799 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13865,c,av);}
/* irregex-core.scm:1294: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k13916 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_13918(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,4)))){
C_save_and_reclaim_args((void *)trf_13918,2,t0,t1);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_fix(C_character_code(((C_word*)t0)[2]));
t3=C_i_vector_ref(lf[170],t2);
t4=C_fixnum_plus(((C_word*)t0)[3],t3);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13932,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t4,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[3],a[8]=t3,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1299: utf8-string-ref */
f_14029(t5,((C_word*)t0)[7],((C_word*)t0)[3],t3);}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13950,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[2]);
t5=C_a_i_vector1(&a,1,t4);
/* irregex-core.scm:3802: cset-union */
t6=lf[162];{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=t5;
f_27080(4,av2);}}}

/* k13930 in k13916 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_13932,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13936,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13940,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1300: utf8-string-ref */
f_14029(t3,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k13934 in k13930 in k13916 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13936,c,av);}
/* irregex-core.scm:1298: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k13938 in k13930 in k13916 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13940,c,av);}
a=C_alloc(5);
t2=C_a_i_cons(&a,2,t1,t1);
t3=C_a_i_vector1(&a,1,t2);
/* irregex-core.scm:3802: cset-union */
t4=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
f_27080(4,av2);}}

/* k13948 in k13916 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_13950(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13950,c,av);}
/* irregex-core.scm:1301: go */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13404(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* chicken.irregex#high-char? in k7761 in k7536 in k7533 */
static void C_ccall f_14008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14008,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fixnum_less_or_equal_p(C_fix(128),C_fix(C_character_code(t2)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#utf8-string-ref in k7761 in k7536 in k7533 */
static void C_fcall f_14029(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_14029,4,t1,t2,t3,t4);}
a=C_alloc(9);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14032,a[2]=t2,a[3]=((C_word)li94),tmp=(C_word)a,a+=4,tmp);
switch(t4){
case C_fix(1):
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_i_string_ref(t2,t3);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}
case C_fix(2):
t6=(
/* irregex-core.scm:1362: byte */
  f_14032(t5,t3)
);
t7=C_fixnum_and(t6,C_fix(31));
t8=C_fixnum_shift_left(t7,C_fix(6));
t9=(
/* irregex-core.scm:1363: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(1)))
);
t10=C_fixnum_and(t9,C_fix(63));
t11=C_a_i_fixnum_plus(&a,2,t8,t10);
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_make_character(C_unfix(t11));
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}
case C_fix(3):
t6=(
/* irregex-core.scm:1366: byte */
  f_14032(t5,t3)
);
t7=C_fixnum_and(t6,C_fix(15));
t8=C_fixnum_shift_left(t7,C_fix(12));
t9=(
/* irregex-core.scm:1367: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(1)))
);
t10=C_fixnum_and(t9,C_fix(63));
t11=C_fixnum_shift_left(t10,C_fix(6));
t12=(
/* irregex-core.scm:1368: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(2)))
);
t13=C_fixnum_and(t12,C_fix(63));
t14=C_fixnum_plus(C_fixnum_plus(t8,t11),t13);
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=C_make_character(C_unfix(t14));
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}
case C_fix(4):
t6=(
/* irregex-core.scm:1371: byte */
  f_14032(t5,t3)
);
t7=C_fixnum_and(t6,C_fix(7));
t8=C_fixnum_shift_left(t7,C_fix(18));
t9=(
/* irregex-core.scm:1372: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(1)))
);
t10=C_fixnum_and(t9,C_fix(63));
t11=C_fixnum_shift_left(t10,C_fix(12));
t12=(
/* irregex-core.scm:1373: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(2)))
);
t13=C_fixnum_and(t12,C_fix(63));
t14=C_fixnum_shift_left(t13,C_fix(6));
t15=(
/* irregex-core.scm:1374: byte */
  f_14032(t5,C_fixnum_plus(t3,C_fix(3)))
);
t16=C_fixnum_and(t15,C_fix(63));
t17=C_fixnum_plus(C_fixnum_plus(C_fixnum_plus(t8,t11),t14),t16);
t18=t1;{
C_word av2[2];
av2[0]=t18;
av2[1]=C_make_character(C_unfix(t17));
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}
default:
/* irregex-core.scm:1376: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word av2[6];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[205];
av2[3]=t2;
av2[4]=t4;
av2[5]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(6,av2);}}}

/* byte in chicken.irregex#utf8-string-ref in k7761 in k7536 in k7533 */
static C_word C_fcall f_14032(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
t2=C_i_string_ref(((C_word*)t0)[2],t1);
return(C_fix(C_character_code(t2)));}

/* lp in collect/single in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_14224(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14224,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,C_fix(0));
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[2],t2);
t5=C_fix(C_character_code(t4));
t6=C_fixnum_lessp(t5,C_fix(128));
t7=(C_truep(t6)?t6:C_fixnum_greater_or_equal_p(t5,C_fix(192)));
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_a_i_fixnum_difference(&a,2,t2,C_fix(1));
/* irregex-core.scm:1385: lp */
t10=t1;
t11=t8;
t1=t10;
t2=t11;
goto loop;}}}

/* chicken.irregex#utf8-lowest-digit-of-length in k7761 in k7536 in k7533 */
static void C_fcall f_14262(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_14262,2,t1,t2);}
switch(t2){
case C_fix(1):
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
case C_fix(2):
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_fix(192);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
case C_fix(3):
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_fix(224);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
case C_fix(4):
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_fix(240);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
default:
/* irregex-core.scm:1390: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[207];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* chicken.irregex#char->utf8-list in k7761 in k7536 in k7533 */
static void C_fcall f_14338(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_14338,2,t1,t2);}
a=C_alloc(12);
t3=C_fix(C_character_code(t2));
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(127)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list1(&a,1,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(2047)))){
t4=C_fixnum_shift_right(t3,C_fix(6));
t5=C_fixnum_or(C_fix(192),t4);
t6=C_fixnum_and(t3,C_fix(63));
t7=C_fixnum_or(C_fix(128),t6);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_list2(&a,2,t5,t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(65535)))){
t4=C_fixnum_shift_right(t3,C_fix(12));
t5=C_fixnum_or(C_fix(224),t4);
t6=C_fixnum_shift_right(t3,C_fix(6));
t7=C_fixnum_and(t6,C_fix(63));
t8=C_fixnum_or(C_fix(128),t7);
t9=C_fixnum_and(t3,C_fix(63));
t10=C_fixnum_or(C_fix(128),t9);
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_a_i_list3(&a,3,t5,t8,t10);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(2097151)))){
t4=C_fixnum_shift_right(t3,C_fix(18));
t5=C_fixnum_or(C_fix(240),t4);
t6=C_fixnum_shift_right(t3,C_fix(12));
t7=C_fixnum_and(t6,C_fix(63));
t8=C_fixnum_or(C_fix(128),t7);
t9=C_fixnum_shift_right(t3,C_fix(6));
t10=C_fixnum_and(t9,C_fix(63));
t11=C_fixnum_or(C_fix(128),t10);
t12=C_fixnum_and(t3,C_fix(63));
t13=C_fixnum_or(C_fix(128),t12);
t14=t1;{
C_word av2[2];
av2[0]=t14;
av2[1]=C_a_i_list4(&a,4,t5,t8,t11,t13);
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
/* irregex-core.scm:1413: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[209];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}}}}

/* k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_14479,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_14482,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1417: char->utf8-list */
f_14338(t2,((C_word*)t0)[8]);}

/* k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14482(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,4)))){
C_save_and_reclaim((void *)f_14482,c,av);}
a=C_alloc(36);
t2=C_eqp(C_u_i_length(((C_word*)t0)[2]),C_u_i_length(t1));
if(C_truep(C_i_not(t2))){
t3=C_i_length(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15041,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t5=C_i_car(((C_word*)t0)[2]);
t6=C_make_character(C_unfix(t5));
t7=C_u_i_car(((C_word*)t0)[2]);
t8=C_fixnum_less_or_equal_p(t7,C_fix(127));
t9=(C_truep(t8)?C_a_i_list(&a,3,lf[52],t6,C_make_character(127)):C_a_i_list(&a,3,lf[52],t6,C_make_character(255)));
t10=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t11=t10;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=((C_word*)t12)[1];
t14=C_u_i_cdr(((C_word*)t0)[2]);
t15=C_i_check_list_2(t14,lf[211]);
t16=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15321,a[2]=t9,a[3]=t3,a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t17=C_SCHEME_UNDEFINED;
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_set_block_item(t18,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15323,a[2]=t12,a[3]=t18,a[4]=t13,a[5]=((C_word)li111),tmp=(C_word)a,a+=6,tmp));
t20=((C_word*)t18)[1];
f_15323(t20,t16,t14);}
else{
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14508,a[2]=t4,a[3]=((C_word)li113),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_14508(t6,((C_word*)t0)[8],((C_word*)t0)[2],t1);}}

/* k14501 in k15039 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_14503,c,av);}
a=C_alloc(12);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=(
/* irregex-core.scm:1419: sre-alternate */
  f_18214(C_a_i(&a,3),t2)
);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* irregex-core.scm:1533: lp */
t5=((C_word*)((C_word*)t0)[4])[1];
f_15479(t5,((C_word*)t0)[5],((C_word*)t0)[6],t4,((C_word*)t0)[7]);}

/* lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_14508(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_14508,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_car(t2);
t5=C_i_car(t3);
t6=C_eqp(t4,t5);
if(C_truep(t6)){
t7=C_u_i_car(t2);
t8=C_make_character(C_unfix(t7));
if(C_truep(C_i_nullp(C_u_i_cdr(t2)))){
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
/* irregex-core.scm:1424: sre-sequence */
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=(
/* irregex-core.scm:1424: sre-sequence */
  f_18191(C_a_i(&a,3),t9)
);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14540,a[2]=t8,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1427: lp */
t12=t9;
t13=C_u_i_cdr(t2);
t14=C_u_i_cdr(t3);
t1=t12;
t2=t13;
t3=t14;
goto loop;}}
else{
t7=C_fixnum_plus(C_u_i_car(t2),C_fix(1));
t8=C_u_i_car(t3);
t9=C_eqp(t7,t8);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14565,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1429: unicode-range-up-from */
f_14752(t10,t2);}
else{
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14580,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1432: unicode-range-up-from */
f_14752(t10,t2);}}}

/* k14538 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14540,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
/* irregex-core.scm:1424: sre-sequence */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(
/* irregex-core.scm:1424: sre-sequence */
  f_18191(C_a_i(&a,3),t3)
);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k14563 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14565,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14569,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1430: unicode-range-up-to */
f_14891(t2,((C_word*)t0)[3]);}

/* k14567 in k14563 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14569,c,av);}
a=C_alloc(9);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1429: sre-alternate */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* irregex-core.scm:1429: sre-alternate */
  f_18214(C_a_i(&a,3),t2)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14578 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,3)))){
C_save_and_reclaim((void *)f_14580,c,av);}
a=C_alloc(28);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_fixnum_plus(t2,C_fix(1));
t4=C_make_character(C_unfix(t3));
t5=C_i_car(((C_word*)t0)[3]);
t6=C_fixnum_difference(t5,C_fix(1));
t7=C_make_character(C_unfix(t6));
t8=(C_truep(C_u_i_char_equalp(t4,t7))?t4:C_a_i_list(&a,3,lf[52],t4,t7));
t9=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t10=t9;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=((C_word*)t11)[1];
t13=C_u_i_cdr(((C_word*)t0)[2]);
t14=C_i_check_list_2(t13,lf[211]);
t15=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15412,a[2]=t8,a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15414,a[2]=t11,a[3]=t17,a[4]=t12,a[5]=((C_word)li112),tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_15414(t19,t15,t13);}

/* k14586 in k15410 in k14578 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_14588(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_14588,c,av);}
a=C_alloc(12);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
/* irregex-core.scm:1432: sre-alternate */
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* irregex-core.scm:1432: sre-alternate */
  f_18214(C_a_i(&a,3),t2)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_fcall f_14614(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,0,3)))){
C_save_and_reclaim_args((void *)trf_14614,5,t1,t2,t3,t4,t5);}
a=C_alloc(28);
if(C_truep(C_i_nullp(t3))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_cdr(t3);
t7=C_u_i_car(t3);
t8=C_a_i_cons(&a,2,t7,t4);
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14644,a[2]=t5,a[3]=t1,a[4]=t2,a[5]=t6,a[6]=t8,tmp=(C_word)a,a+=7,tmp);
t10=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t11=t10;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=((C_word*)t12)[1];
t14=C_i_check_list_2(t4,lf[211]);
t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14654,a[2]=t3,a[3]=t9,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14716,a[2]=t12,a[3]=t17,a[4]=t13,a[5]=((C_word)li99),tmp=(C_word)a,a+=6,tmp));
t19=((C_word*)t17)[1];
f_14716(t19,t15,t4);}}

/* k14642 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_ccall f_14644(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_14644,c,av);}
a=C_alloc(6);
t2=(
/* irregex-core.scm:1443: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[2]);
/* irregex-core.scm:1439: unicode-range-helper */
f_14614(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t3);}

/* k14652 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_ccall f_14654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14654,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14662,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1445: one */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14660 in k14652 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_ccall f_14662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_14662,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[211]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14678,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14680,a[2]=t4,a[3]=t10,a[4]=t5,a[5]=((C_word)li98),tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_14680(t12,t8,t6);}

/* k14676 in k14660 in k14652 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_ccall f_14678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14678,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1444: ##sys#append */
t3=*((C_word*)lf[118]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* map-loop2077 in k14660 in k14652 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_fcall f_14680(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_14680,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),C_make_character(255));
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop2051 in chicken.irregex#unicode-range-helper in k7761 in k7536 in k7533 */
static void C_fcall f_14716(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_14716,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_fcall f_14752(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_14752,2,t1,t2);}
a=C_alloc(13);
t3=C_i_car(t2);
t4=C_make_character(C_unfix(t3));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14772,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14774,a[2]=t2,a[3]=((C_word)li101),tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14883,a[2]=t5,a[3]=t6,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1459: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=C_u_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k14770 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_14772,c,av);}
a=C_alloc(12);
t2=(
/* irregex-core.scm:1455: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
t3=C_a_i_list2(&a,2,((C_word*)t0)[2],t2);
/* irregex-core.scm:1453: sre-sequence */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(
/* irregex-core.scm:1453: sre-sequence */
  f_18191(C_a_i(&a,3),t3)
);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a14773 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14774,c,av);}
a=C_alloc(9);
t3=C_i_car(((C_word*)t0)[2]);
t4=C_fixnum_plus(t3,C_fix(1));
t5=C_make_character(C_unfix(t4));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_list(&a,3,lf[52],t5,C_make_character(255));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k14804 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_14806,c,av);}
a=C_alloc(6);
t2=(
/* irregex-core.scm:1462: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
t3=C_a_i_list1(&a,1,t2);
/* irregex-core.scm:1456: unicode-range-helper */
f_14614(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_END_OF_LIST,t3);}

/* k14811 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_14813,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(t1,lf[211]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14819,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14837,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=((C_word)li102),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_14837(t7,t3,t1);}

/* k14817 in k14811 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14819,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14835,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1465: last */
f_9256(t2,((C_word*)t0)[3]);}

/* k14833 in k14817 in k14811 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_14835,c,av);}
a=C_alloc(12);
t2=C_make_character(C_unfix(t1));
t3=C_a_i_list(&a,3,lf[52],t2,C_make_character(255));
t4=C_a_i_list(&a,1,t3);
/* irregex-core.scm:1463: scheme#append */
t5=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* map-loop2113 in k14811 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_fcall f_14837(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_14837,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14875 in k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14877(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14877,c,av);}
/* irregex-core.scm:1464: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cdr(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k14881 in chicken.irregex#unicode-range-up-from in k7761 in k7536 in k7533 */
static void C_ccall f_14883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_14883,c,av);}
a=C_alloc(19);
t2=C_i_cdr(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14813,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t6,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14877,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1464: ##sys#fast-reverse */
t10=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t10;
av2[1]=t9;
av2[2]=C_u_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_fcall f_14891(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_14891,2,t1,t2);}
a=C_alloc(13);
t3=C_i_car(t2);
t4=C_make_character(C_unfix(t3));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14911,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14913,a[2]=t2,a[3]=((C_word)li104),tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15022,a[2]=t5,a[3]=t6,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1475: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=C_u_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k14909 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_14911,c,av);}
a=C_alloc(12);
t2=(
/* irregex-core.scm:1471: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
t3=C_a_i_list2(&a,2,((C_word*)t0)[2],t2);
/* irregex-core.scm:1469: sre-sequence */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(
/* irregex-core.scm:1469: sre-sequence */
  f_18191(C_a_i(&a,3),t3)
);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a14912 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_14913,c,av);}
a=C_alloc(9);
t3=C_i_car(((C_word*)t0)[2]);
t4=C_fixnum_difference(t3,C_fix(1));
t5=C_make_character(C_unfix(t4));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_list(&a,3,lf[52],C_make_character(128),t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k14943 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_14945,c,av);}
a=C_alloc(6);
t2=(
/* irregex-core.scm:1478: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
t3=C_a_i_list1(&a,1,t2);
/* irregex-core.scm:1472: unicode-range-helper */
f_14614(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_END_OF_LIST,t3);}

/* k14950 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_14952,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(t1,lf[211]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14958,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14976,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=((C_word)li105),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_14976(t7,t3,t1);}

/* k14956 in k14950 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14958(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14958,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14974,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1482: last */
f_9256(t2,((C_word*)t0)[3]);}

/* k14972 in k14956 in k14950 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_14974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_14974,c,av);}
a=C_alloc(12);
t2=C_make_character(C_unfix(t1));
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),t2);
t4=C_a_i_list(&a,1,t3);
/* irregex-core.scm:1479: scheme#append */
t5=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* map-loop2148 in k14950 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_fcall f_14976(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_14976,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15014 in k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_15016(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15016,c,av);}
/* irregex-core.scm:1480: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_cdr(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k15020 in chicken.irregex#unicode-range-up-to in k7761 in k7536 in k7533 */
static void C_ccall f_15022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_15022,c,av);}
a=C_alloc(19);
t2=C_i_cdr(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14945,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14952,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t6,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15016,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1480: ##sys#fast-reverse */
t10=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t10;
av2[1]=t9;
av2[2]=C_u_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* k15039 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_15041,c,av);}
a=C_alloc(11);
t2=(
/* irregex-core.scm:1486: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_14503,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1420: unicode-range-up-to */
f_14891(t3,((C_word*)t0)[7]);}

/* g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15050(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_15050,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15062,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15138,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1499: utf8-lowest-digit-of-length */
f_14262(t4,C_fixnum_plus(C_fixnum_plus(t2,((C_word*)t0)[2]),C_fix(1)));}

/* k15060 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15062(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_15062,2,t0,t1);}
a=C_alloc(11);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15074,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t4,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1503: zero-to */
f_9158(t6,C_fixnum_plus(((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k15072 in k15060 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15074(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_15074,c,av);}
a=C_alloc(12);
t2=C_i_check_list_2(t1,lf[211]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15080,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15082,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=((C_word)li107),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_15082(t7,t3,t1);}

/* k15078 in k15072 in k15060 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_15080,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:1497: sre-sequence */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* irregex-core.scm:1497: sre-sequence */
  f_18191(C_a_i(&a,3),t2)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop2237 in k15072 in k15060 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15082(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_15082,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),C_make_character(255));
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15128 in k15136 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_15130,c,av);}
a=C_alloc(9);
t2=C_make_character(C_unfix(t1));
t3=((C_word*)t0)[2];
f_15062(t3,C_a_i_list(&a,3,lf[52],((C_word*)t0)[3],t2));}

/* k15136 in g2220 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15138(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_15138,c,av);}
a=C_alloc(13);
t2=C_make_character(C_unfix(t1));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15130,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_fixnum_plus(C_fixnum_plus(((C_word*)t0)[3],((C_word*)t0)[4]),C_fix(1));
switch(t4){
case C_fix(1):
t5=((C_word*)t0)[2];
f_15062(t5,C_a_i_list(&a,3,lf[52],t2,C_make_character(127)));
case C_fix(2):
t5=((C_word*)t0)[2];
f_15062(t5,C_a_i_list(&a,3,lf[52],t2,C_make_character(223)));
case C_fix(3):
t5=((C_word*)t0)[2];
f_15062(t5,C_a_i_list(&a,3,lf[52],t2,C_make_character(239)));
case C_fix(4):
t5=((C_word*)t0)[2];
f_15062(t5,C_a_i_list(&a,3,lf[52],t2,C_make_character(247)));
default:
/* irregex-core.scm:1395: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[215];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_15145,c,av);}
a=C_alloc(14);
t2=C_i_check_list_2(t1,lf[211]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15151,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15253,a[2]=((C_word*)t0)[5],a[3]=t5,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word)li110),tmp=(C_word)a,a+=7,tmp));
t7=((C_word*)t5)[1];
f_15253(t7,t3,t1);}

/* k15149 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_15151,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15235,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t3=C_i_car(((C_word*)t0)[2]);
t4=C_fixnum_difference(t3,C_fix(1));
t5=C_make_character(C_unfix(t4));
t6=C_fix(C_character_code(t5));
/* irregex-core.scm:1508: utf8-lowest-digit-of-length */
f_14262(t2,C_i_vector_ref(lf[170],t6));}

/* k15181 in k15233 in k15149 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_15183,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(
/* irregex-core.scm:1506: sre-sequence */
  f_18191(C_a_i(&a,3),t2)
);
t4=C_a_i_list1(&a,1,t3);
/* irregex-core.scm:1487: scheme#append */
t5=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* map-loop2277 in k15233 in k15149 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15185(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_15185,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),C_make_character(255));
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15233 in k15149 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,3)))){
C_save_and_reclaim((void *)f_15235,c,av);}
a=C_alloc(28);
t2=C_make_character(C_unfix(t1));
t3=C_fixnum_difference(C_u_i_car(((C_word*)t0)[2]),C_fix(1));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_list(&a,3,lf[52],t2,t4);
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=C_u_i_cdr(((C_word*)t0)[2]);
t11=C_i_check_list_2(t10,lf[211]);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15183,a[2]=t5,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15185,a[2]=t8,a[3]=t14,a[4]=t9,a[5]=((C_word)li109),tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_15185(t16,t12,t10);}

/* map-loop2214 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15253(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15253,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15278,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1495: g2220 */
t4=((C_word*)t0)[4];
f_15050(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15276 in map-loop2214 in k15143 in k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15278,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15253(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k15319 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,3)))){
C_save_and_reclaim((void *)f_15321,c,av);}
a=C_alloc(36);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(
/* irregex-core.scm:1489: sre-sequence */
  f_18191(C_a_i(&a,3),t2)
);
t4=C_a_i_list1(&a,1,t3);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15050,a[2]=((C_word*)t0)[3],a[3]=((C_word)li108),tmp=(C_word)a,a+=4,tmp);
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15145,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t4,a[5]=t7,a[6]=t9,a[7]=t8,tmp=(C_word)a,a+=8,tmp);
t11=C_i_length(((C_word*)t0)[4]);
t12=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t13=C_s_a_u_i_integer_minus(&a,2,t11,t12);
/* irregex-core.scm:1504: zero-to */
f_9158(t10,t13);}

/* map-loop2184 in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15323(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_15323,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),C_make_character(255));
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15410 in k14578 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15412(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_15412,c,av);}
a=C_alloc(11);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(
/* irregex-core.scm:1519: sre-sequence */
  f_18191(C_a_i(&a,3),t2)
);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14588,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1434: unicode-range-up-to */
f_14891(t4,((C_word*)t0)[5]);}

/* map-loop2315 in k14578 in lp in k14480 in k14477 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15414(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_15414,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_a_i_list(&a,3,lf[52],C_make_character(128),C_make_character(255));
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15469(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_15469,2,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15477,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1526: cset->plist */
f_26381(t3,t2);}

/* k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_15477,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15479,a[2]=t3,a[3]=((C_word)li114),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_15479(t5,((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST);}

/* lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15479(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_15479,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15493,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15497,a[2]=t4,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1528: ##sys#fast-reverse */
t7=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=C_i_car(t2);
t6=C_fix(C_character_code(t5));
t7=C_fixnum_less_or_equal_p(C_fix(128),t6);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15524,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t7)){
t9=t8;
f_15524(t9,t7);}
else{
t9=C_i_cadr(t2);
t10=t8;
f_15524(t10,C_fixnum_less_or_equal_p(C_fix(128),C_fix(C_character_code(t9))));}}}

/* k15491 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15493,c,av);}
a=C_alloc(3);
/* irregex-core.scm:1528: sre-alternate */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* irregex-core.scm:1528: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k15495 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15497(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_15497,c,av);}
a=C_alloc(4);
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
/* irregex-core.scm:1528: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15515,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1531: ##sys#fast-reverse */
t3=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k15513 in k15495 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_15515,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[52],t1);
t3=C_a_i_list1(&a,1,t2);
/* irregex-core.scm:1528: scheme#append */
t4=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_fcall f_15524(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,4)))){
C_save_and_reclaim_args((void *)trf_15524,2,t0,t1);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_i_cddr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15539,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t4=C_u_i_car(((C_word*)t0)[2]);
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_u_i_car(t5);
t7=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_14479,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],a[7]=t3,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1416: char->utf8-list */
f_14338(t7,t4);}
else{
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_u_i_car(t3);
t5=C_u_i_car(((C_word*)t0)[2]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[6]);
t7=C_a_i_cons(&a,2,t4,t6);
/* irregex-core.scm:1536: lp */
t8=((C_word*)((C_word*)t0)[4])[1];
f_15479(t8,((C_word*)t0)[5],t2,((C_word*)t0)[3],t7);}}

/* k15537 in k15522 in lp in k15475 in chicken.irregex#cset->utf8-pattern in k7761 in k7536 in k7533 */
static void C_ccall f_15539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_15539,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:1533: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_15479(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2,((C_word*)t0)[6]);}

/* adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15591(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,4)))){
C_save_and_reclaim_args((void *)trf_15591,5,t0,t1,t2,t3,t4);}
a=C_alloc(29);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15594,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,a[5]=((C_word)li204),tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(t2))){
t6=C_u_i_car(t2);
t7=C_eqp(t6,lf[156]);
if(C_truep(t7)){
t8=(
/* irregex-core.scm:1546: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:1546: adjust */
t24=t1;
t25=t8;
t26=C_SCHEME_TRUE;
t27=t4;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
goto loop;}
else{
t8=C_eqp(t6,lf[157]);
if(C_truep(t8)){
t9=(
/* irregex-core.scm:1547: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:1547: adjust */
t24=t1;
t25=t9;
t26=C_SCHEME_FALSE;
t27=t4;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
goto loop;}
else{
t9=C_eqp(t6,lf[225]);
if(C_truep(t9)){
t10=C_u_i_car(t2);
t11=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t12=t11;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=((C_word*)t13)[1];
t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15651,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word)li205),tmp=(C_word)a,a+=5,tmp);
t16=C_u_i_cdr(t2);
t17=C_i_check_list_2(t16,lf[211]);
t18=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15662,a[2]=t1,a[3]=t10,tmp=(C_word)a,a+=4,tmp);
t19=C_SCHEME_UNDEFINED;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_set_block_item(t20,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15664,a[2]=t13,a[3]=t20,a[4]=t15,a[5]=t14,a[6]=((C_word)li206),tmp=(C_word)a,a+=7,tmp));
t22=((C_word*)t20)[1];
f_15664(t22,t18,t16);}
else{
t10=C_eqp(t6,lf[226]);
if(C_truep(t10)){
t11=C_u_i_car(t2);
t12=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t13=t12;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=((C_word*)t14)[1];
t16=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15712,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word)li207),tmp=(C_word)a,a+=5,tmp);
t17=C_u_i_cdr(t2);
t18=C_i_check_list_2(t17,lf[211]);
t19=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15723,a[2]=t1,a[3]=t11,tmp=(C_word)a,a+=4,tmp);
t20=C_SCHEME_UNDEFINED;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_set_block_item(t21,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15725,a[2]=t14,a[3]=t21,a[4]=t16,a[5]=t15,a[6]=((C_word)li208),tmp=(C_word)a,a+=7,tmp));
t23=((C_word*)t21)[1];
f_15725(t23,t19,t17);}
else{
t11=C_eqp(t6,lf[52]);
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15766,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,a[6]=t6,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
if(C_truep(t11)){
t13=t12;
f_15766(t13,t11);}
else{
t13=C_eqp(t6,lf[179]);
if(C_truep(t13)){
t14=t12;
f_15766(t14,t13);}
else{
t14=C_eqp(t6,lf[234]);
t15=t12;
f_15766(t15,(C_truep(t14)?t14:C_eqp(t6,lf[247])));}}}}}}}
else{
t6=C_eqp(t2,lf[124]);
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=lf[274];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_eqp(t2,lf[125]);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[275];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15974,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
t9=C_charp(t2);
t10=t8;
f_15974(t10,(C_truep(t9)?C_fixnum_less_or_equal_p(C_fix(128),C_fix(C_character_code(t2))):C_SCHEME_FALSE));}
else{
t9=t8;
f_15974(t9,C_SCHEME_FALSE);}}}}}

/* rec in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15594(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_15594,3,t0,t1,t2);}
/* irregex-core.scm:1542: adjust */
t3=((C_word*)((C_word*)t0)[2])[1];
f_15591(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* g2380 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15651(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_15651,3,t0,t1,t2);}
/* irregex-core.scm:1549: adjust */
t3=((C_word*)((C_word*)t0)[2])[1];
f_15591(t3,t1,t2,((C_word*)t0)[3],C_SCHEME_FALSE);}

/* k15660 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15662,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2374 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15664(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15664,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15689,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1549: g2380 */
t4=((C_word*)t0)[4];
f_15651(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15687 in map-loop2374 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15689,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15664(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* g2407 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15712(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_15712,3,t0,t1,t2);}
/* irregex-core.scm:1551: adjust */
t3=((C_word*)((C_word*)t0)[2])[1];
f_15591(t3,t1,t2,((C_word*)t0)[3],C_SCHEME_TRUE);}

/* k15721 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15723,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2401 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15725(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15725,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15750,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1551: g2407 */
t4=((C_word*)t0)[4];
f_15712(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15748 in map-loop2401 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15750,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15725(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15766(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_15766,2,t0,t1);}
a=C_alloc(20);
if(C_truep(t1)){
if(C_truep(C_i_not(((C_word*)t0)[2]))){
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15775,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1555: sre->cset */
f_26492(t2,((C_word*)t0)[4],C_a_i_list(&a,1,((C_word*)t0)[5]));}}
else{
t2=C_eqp(((C_word*)t0)[6],lf[127]);
if(C_truep(t2)){
t3=(
/* irregex-core.scm:1562: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[4]))
);
t4=C_eqp(t3,lf[124]);
if(C_truep(t4)){
t5=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[272];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(t3,lf[125]);
if(C_truep(t5)){
t6=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t6;
av2[1]=lf[273];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=C_u_i_cdr(((C_word*)t0)[4]);
t11=C_i_check_list_2(t10,lf[211]);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15833,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15835,a[2]=t8,a[3]=t14,a[4]=((C_word*)t0)[7],a[5]=t9,a[6]=((C_word)li209),tmp=(C_word)a,a+=7,tmp));
t16=((C_word*)t14)[1];
f_15835(t16,t12,t10);}}}
else{
t3=C_u_i_car(((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_u_i_cdr(((C_word*)t0)[4]);
t9=C_i_check_list_2(t8,lf[211]);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15889,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15891,a[2]=t6,a[3]=t12,a[4]=((C_word*)t0)[7],a[5]=t7,a[6]=((C_word)li210),tmp=(C_word)a,a+=7,tmp));
t14=((C_word*)t12)[1];
f_15891(t14,t10,t8);}}}

/* k15773 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_15775,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15781,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15798,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1556: cset->plist */
f_26381(t3,t1);}

/* k15779 in k15773 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15781,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15791,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1558: cset->utf8-pattern */
f_15469(t2,((C_word*)t0)[4]);}
else{
/* irregex-core.scm:1559: cset->utf8-pattern */
f_15469(((C_word*)t0)[3],((C_word*)t0)[4]);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k15789 in k15779 in k15773 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_15791,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,lf[225],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k15796 in k15773 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15798,c,av);}
/* irregex-core.scm:1556: any */
f_9294(((C_word*)t0)[2],lf[204],t1);}

/* k15831 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15833(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15833,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[127],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2444 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15835(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15835,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15860,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1566: g2450 */
t4=((C_word*)t0)[4];
f_15594(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15858 in map-loop2444 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15860,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15835(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k15887 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15889(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15889,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2470 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15891(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15891,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15916,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1568: g2476 */
t4=((C_word*)t0)[4];
f_15594(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15914 in map-loop2470 in k15764 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15916(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_15916,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15891(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k15972 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15974(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_15974,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15984,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1575: char->utf8-list */
f_14338(t6,((C_word*)t0)[3]);}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k15982 in k15972 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_15984,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15987,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15989,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word)li211),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_15989(t6,t2,t1);}

/* k15985 in k15982 in k15972 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_15987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_15987,c,av);}
a=C_alloc(3);
/* irregex-core.scm:1575: sre-sequence */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* irregex-core.scm:1575: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop2504 in k15982 in k15972 in adjust in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_15989(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_15989,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.irregex#irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,2)))){
C_save_and_reclaim((void*)f_16036,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16043,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1592: irregex? */
t5=*((C_word*)lf[5]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16041 in chicken.irregex#irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_16043,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_nullp(((C_word*)t0)[4]))){
t2=lf[218];
t3=C_slot(t2,C_fix(0));
if(C_truep(C_i_equalp(t3,((C_word*)t0)[3]))){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(2));
if(C_truep(C_i_equalp(t4,((C_word*)t0)[3]))){
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_slot(t2,C_fix(3));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_slot(t2,C_fix(4));
if(C_truep(C_i_equalp(t5,((C_word*)t0)[3]))){
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_slot(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_slot(t2,C_fix(6));
if(C_truep(C_i_equalp(t6,((C_word*)t0)[3]))){
t7=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_slot(t2,C_fix(7));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_slot(t2,C_fix(8));
if(C_truep(C_i_equalp(t7,((C_word*)t0)[3]))){
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_slot(t2,C_fix(9));
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16097,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_stringp(((C_word*)t0)[3]))){
/* irregex-core.scm:1597: string->irregex */
t9=*((C_word*)lf[219]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
/* irregex-core.scm:1598: sre->irregex */
t9=*((C_word*)lf[220]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}}}}}}}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[3]))){{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[219]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
C_apply(5,av2);}}
else{{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[220]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
C_apply(5,av2);}}}}}

/* k16095 in k16041 in chicken.irregex#irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16097,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(10));
t3=C_i_setslot(((C_word*)t0)[2],t2,((C_word*)t0)[3]);
t4=C_fixnum_plus(t2,C_fix(1));
t5=C_i_setslot(((C_word*)t0)[2],t4,t1);
t6=C_fixnum_plus(t2,C_fix(2));
t7=C_fixnum_modulo(t6,C_fix(10));
t8=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),t7);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* chicken.irregex#string->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_16164,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16172,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t4;
av2[2]=*((C_word*)lf[102]+1);
av2[3]=t2;
av2[4]=t3;
C_apply(5,av2);}}

/* k16170 in chicken.irregex#string->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_16172,c,av);}{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[220]+1);
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
C_apply(5,av2);}}

/* chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,2)))){
C_save_and_reclaim((void*)f_16174,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16178,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1608: symbol-list->flags */
f_10332(t4,t3);}

/* k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_16178,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16181,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=C_fixnum_and(t1,C_fix(32));
t4=C_eqp(C_fix(32),t3);
t5=C_fixnum_and(t1,C_fix(2));
t6=C_eqp(C_fix(2),t5);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15591,a[2]=t8,a[3]=((C_word)li212),tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_15591(t10,t2,((C_word*)t0)[4],t4,t6);}

/* k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_16181,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16184,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1612: sre-searcher? */
t3=lf[271];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
f_16709(3,av2);}}

/* k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16184,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16187,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
/* irregex-core.scm:1613: sre-remove-initial-bos */
t3=lf[270];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
f_18736(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
f_16187(2,av2);}}}

/* k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_16187,c,av);}
a=C_alloc(27);
t2=(C_truep(C_u_i_memq(lf[221],((C_word*)t0)[2]))?C_fix(1):(C_truep(C_u_i_memq(lf[222],((C_word*)t0)[2]))?C_fix(50):C_fix(10)));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16193,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=t2,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_u_i_memq(lf[269],((C_word*)t0)[2]))){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_16193(2,av2);}}
else{
if(C_truep(((C_word*)t0)[3])){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_TRUE;
f_16193(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16265,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_a_i_list(&a,2,lf[127],lf[124]);
t6=C_a_i_list(&a,3,lf[115],t5,t1);
/* irregex-core.scm:1621: sre->nfa */
f_20757(t4,t6,((C_word*)t0)[5]);}}}

/* k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_16193,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_16196,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
/* irregex-core.scm:1625: sre->nfa */
f_20757(t2,((C_word*)t0)[6],((C_word*)t0)[4]);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_16196(2,av2);}}}

/* k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_16196,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16199,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16239,a[2]=((C_word*)t0)[8],a[3]=((C_word)li202),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1625: g2625 */
t4=t3;
f_16239(t4,t2,t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_16199(2,av2);}}}

/* k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_16199,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_16202,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1629: sre-count-submatches */
f_16982(t2,((C_word*)t0)[7]);}

/* k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16202(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_16202,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_16205,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1630: sre-names */
f_18418(t2,((C_word*)t0)[8],C_fix(1),C_SCHEME_END_OF_LIST);}

/* k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_16205,c,av);}
a=C_alloc(14);
t2=C_a_i_list(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17082,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_pairp(t2))){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_car(t2);
f_17082(2,av2);}}
else{
/* irregex-core.scm:1726: sre-names */
f_18418(t3,((C_word*)t0)[9],C_fix(1),C_SCHEME_END_OF_LIST);}}

/* k16218 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_16220,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record8(&a,8,lf[6],C_SCHEME_FALSE,C_SCHEME_FALSE,t1,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16234(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_16234,c,av);}
a=C_alloc(21);
t2=(C_truep(t1)?C_fixnum_or(((C_word*)t0)[2],C_fix(2)):((C_word*)t0)[2]);
if(C_truep(((C_word*)t0)[3])){
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record8(&a,8,lf[6],((C_word*)t0)[3],((C_word*)t0)[5],C_SCHEME_FALSE,t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16220,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
t4=C_a_i_list(&a,2,((C_word*)t0)[9],((C_word*)t0)[8]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23397,a[2]=t6,a[3]=t4,a[4]=t3,a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
t8=C_i_pairp(t4);
t9=(C_truep(t8)?C_i_pairp(C_u_i_cdr(t4)):C_SCHEME_FALSE);
if(C_truep(t9)){
t10=t7;{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_i_cadr(t4);
f_23397(2,av2);}}
else{
/* irregex-core.scm:3126: sre-names */
f_18418(t7,((C_word*)t0)[10],C_fix(1),C_SCHEME_END_OF_LIST);}}}

/* g2625 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_16239(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_16239,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_vector_length(t2);
t4=C_a_i_fixnum_quotient_checked(&a,2,t3,C_fix(4));
t5=C_fixnum_times(((C_word*)t0)[2],t4);
/* irregex-core.scm:1627: nfa->dfa */
f_22369(t1,t2,C_a_i_list(&a,1,t5));}

/* k16263 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_16265(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16265,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16269,a[2]=((C_word*)t0)[2],a[3]=((C_word)li203),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1619: g2614 */
t3=t2;
f_16269(t3,((C_word*)t0)[3],t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_16193(2,av2);}}}

/* g2614 in k16263 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_16269(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_16269,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_vector_length(t2);
t4=C_a_i_fixnum_quotient_checked(&a,2,t3,C_fix(4));
t5=C_fixnum_times(((C_word*)t0)[2],t4);
/* irregex-core.scm:1623: nfa->dfa */
f_22369(t1,t2,C_a_i_list(&a,1,t5));}

/* chicken.irregex#sre-empty? in k7761 in k7536 in k7533 */
static void C_ccall f_16307(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16307,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[127]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16324,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;
f_16324(t6,t4);}
else{
t6=C_eqp(t3,lf[131]);
if(C_truep(t6)){
t7=t5;
f_16324(t7,t6);}
else{
t7=C_eqp(t3,lf[111]);
if(C_truep(t7)){
t8=t5;
f_16324(t8,t7);}
else{
t8=C_eqp(t3,lf[113]);
if(C_truep(t8)){
t9=t5;
f_16324(t9,t8);}
else{
t9=C_eqp(t3,lf[112]);
t10=t5;
f_16324(t10,(C_truep(t9)?t9:C_eqp(t3,lf[114])));}}}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_memq(t2,lf[276]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16322 in chicken.irregex#sre-empty? in k7761 in k7536 in k7533 */
static void C_fcall f_16324(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_16324,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[133]);
if(C_truep(t2)){
t3=C_i_cadr(((C_word*)t0)[4]);
t4=C_i_numberp(t3);
t5=C_i_not(t4);
if(C_truep(t5)){
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_u_i_cdr(((C_word*)t0)[4]);
t7=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t7;
av2[1]=C_i_zerop(C_u_i_car(t6));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[103]);
if(C_truep(t3)){
/* irregex-core.scm:1651: any */
f_9294(((C_word*)t0)[2],lf[139],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t4=C_eqp(((C_word*)t0)[3],lf[227]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16367,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t4)){
if(C_truep(t4)){
/* irregex-core.scm:1653: every */
f_9337(((C_word*)t0)[2],lf[139],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t6=C_eqp(((C_word*)t0)[3],lf[115]);
if(C_truep(t6)){
if(C_truep(t6)){
/* irregex-core.scm:1653: every */
f_9337(((C_word*)t0)[2],lf[139],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t7=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t7=C_eqp(((C_word*)t0)[3],lf[245]);
if(C_truep(t7)){
t8=t5;
f_16367(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[3],lf[108]);
if(C_truep(t8)){
t9=t5;
f_16367(t9,t8);}
else{
t9=C_eqp(((C_word*)t0)[3],lf[119]);
if(C_truep(t9)){
t10=t5;
f_16367(t10,t9);}
else{
t10=C_eqp(((C_word*)t0)[3],lf[120]);
if(C_truep(t10)){
t11=t5;
f_16367(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[3],lf[129]);
t12=t5;
f_16367(t12,(C_truep(t11)?t11:C_eqp(((C_word*)t0)[3],lf[239])));}}}}}}}}}}

/* k16365 in k16322 in chicken.irregex#sre-empty? in k7761 in k7536 in k7533 */
static void C_fcall f_16367(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16367,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:1653: every */
f_9337(((C_word*)t0)[2],lf[139],C_u_i_cdr(((C_word*)t0)[3]));}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#sre-any? in k7761 in k7536 in k7533 */
static void C_ccall f_16474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16474,c,av);}
a=C_alloc(5);
t3=C_eqp(t2,lf[124]);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_car(t2);
t5=C_eqp(t4,lf[115]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16497,a[2]=t2,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_16497(t7,t5);}
else{
t7=C_eqp(t4,lf[227]);
if(C_truep(t7)){
t8=t6;
f_16497(t8,t7);}
else{
t8=C_eqp(t4,lf[245]);
if(C_truep(t8)){
t9=t6;
f_16497(t9,t8);}
else{
t9=C_eqp(t4,lf[108]);
if(C_truep(t9)){
t10=t6;
f_16497(t10,t9);}
else{
t10=C_eqp(t4,lf[119]);
t11=t6;
f_16497(t11,(C_truep(t10)?t10:C_eqp(t4,lf[120])));}}}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k16495 in chicken.irregex#sre-any? in k7761 in k7536 in k7533 */
static void C_fcall f_16497(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16497,2,t0,t1);}
if(C_truep(t1)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
t2=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t2))){
t3=C_u_i_cdr(((C_word*)t0)[2]);
/* irregex-core.scm:1662: sre-any? */
t4=lf[277];{
C_word av2[3];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=C_u_i_car(t3);
f_16474(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[103]);
if(C_truep(t2)){
/* irregex-core.scm:1663: every */
f_9337(((C_word*)t0)[3],lf[277],C_u_i_cdr(((C_word*)t0)[2]));}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* chicken.irregex#sre-repeater? in k7761 in k7536 in k7533 */
static C_word C_fcall f_16575(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_u_i_car(t1);
t3=C_u_i_memq(t2,lf[278]);
if(C_truep(t3)){
return(t3);}
else{
t4=C_u_i_car(t1);
if(C_truep((C_truep(C_eqp(t4,lf[245]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[108]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[119]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[120]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[115]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[227]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))))){
if(C_truep(C_i_pairp(C_u_i_cdr(t1)))){
t5=C_i_cddr(t1);
if(C_truep(C_i_nullp(t5))){
t6=C_u_i_cdr(t1);
t8=C_u_i_car(t6);
t1=t8;
goto loop;}
else{
return(C_SCHEME_FALSE);}}
else{
return(C_SCHEME_FALSE);}}
else{
return(C_SCHEME_FALSE);}}}
else{
return(C_SCHEME_FALSE);}}

/* chicken.irregex#sre-bos? in k7761 in k7536 in k7533 */
static void C_ccall f_16620(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16620,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[115]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16637,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;
f_16637(t6,t4);}
else{
t6=C_eqp(t3,lf[227]);
if(C_truep(t6)){
t7=t5;
f_16637(t7,t6);}
else{
t7=C_eqp(t3,lf[245]);
if(C_truep(t7)){
t8=t5;
f_16637(t8,t7);}
else{
t8=C_eqp(t3,lf[108]);
if(C_truep(t8)){
t9=t5;
f_16637(t9,t8);}
else{
t9=C_eqp(t3,lf[119]);
t10=t5;
f_16637(t10,(C_truep(t9)?t9:C_eqp(t3,lf[120])));}}}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(lf[186],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16635 in chicken.irregex#sre-bos? in k7761 in k7536 in k7533 */
static void C_fcall f_16637(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16637,2,t0,t1);}
if(C_truep(t1)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
/* irregex-core.scm:1678: sre-bos? */
t2=lf[279];{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_i_cadr(((C_word*)t0)[2]);
f_16620(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[103]);
if(C_truep(t2)){
/* irregex-core.scm:1679: every */
f_9337(((C_word*)t0)[3],lf[279],C_u_i_cdr(((C_word*)t0)[2]));}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* chicken.irregex#sre-searcher? in k7761 in k7536 in k7533 */
static void C_ccall f_16709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16709,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16713,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1685: sre-bos? */
t4=lf[279];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
f_16620(3,av2);}}

/* k16711 in chicken.irregex#sre-searcher? in k7761 in k7536 in k7533 */
static void C_ccall f_16713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16713,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t2=C_u_i_car(((C_word*)t0)[3]);
t3=C_eqp(t2,lf[127]);
t4=(C_truep(t3)?t3:C_eqp(t2,lf[129]));
if(C_truep(t4)){
t5=(
/* irregex-core.scm:1688: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[3]))
);
/* irregex-core.scm:1688: sre-any? */
t6=lf[277];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
av2[2]=t5;
f_16474(3,av2);}}
else{
t5=C_eqp(t2,lf[115]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16750,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_16750(t7,t5);}
else{
t7=C_eqp(t2,lf[227]);
if(C_truep(t7)){
t8=t6;
f_16750(t8,t7);}
else{
t8=C_eqp(t2,lf[245]);
if(C_truep(t8)){
t9=t6;
f_16750(t9,t8);}
else{
t9=C_eqp(t2,lf[108]);
if(C_truep(t9)){
t10=t6;
f_16750(t10,t9);}
else{
t10=C_eqp(t2,lf[119]);
t11=t6;
f_16750(t11,(C_truep(t10)?t10:C_eqp(t2,lf[120])));}}}}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k16748 in k16711 in chicken.irregex#sre-searcher? in k7761 in k7536 in k7533 */
static void C_fcall f_16750(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_16750,2,t0,t1);}
if(C_truep(t1)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
/* irregex-core.scm:1690: sre-searcher? */
t2=lf[271];{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_i_cadr(((C_word*)t0)[2]);
f_16709(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[103]);
if(C_truep(t2)){
/* irregex-core.scm:1691: every */
f_9337(((C_word*)t0)[3],lf[271],C_u_i_cdr(((C_word*)t0)[2]));}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* chicken.irregex#sre-consumer? in k7761 in k7536 in k7533 */
static void C_ccall f_16826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16826,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16830,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:1696: sre-bos? */
t4=lf[279];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
f_16620(3,av2);}}

/* k16828 in chicken.irregex#sre-consumer? in k7761 in k7536 in k7533 */
static void C_ccall f_16830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16830,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[3]))){
t2=C_u_i_car(((C_word*)t0)[3]);
t3=C_eqp(t2,lf[127]);
t4=(C_truep(t3)?t3:C_eqp(t2,lf[129]));
if(C_truep(t4)){
t5=(
/* irregex-core.scm:1699: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[3]))
);
/* irregex-core.scm:1699: sre-any? */
t6=lf[277];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
av2[2]=t5;
f_16474(3,av2);}}
else{
t5=C_eqp(t2,lf[115]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16867,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_16867(t7,t5);}
else{
t7=C_eqp(t2,lf[227]);
if(C_truep(t7)){
t8=t6;
f_16867(t8,t7);}
else{
t8=C_eqp(t2,lf[245]);
if(C_truep(t8)){
t9=t6;
f_16867(t9,t8);}
else{
t9=C_eqp(t2,lf[108]);
if(C_truep(t9)){
t10=t6;
f_16867(t10,t9);}
else{
t10=C_eqp(t2,lf[119]);
t11=t6;
f_16867(t11,(C_truep(t10)?t10:C_eqp(t2,lf[120])));}}}}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k16865 in k16828 in chicken.irregex#sre-consumer? in k7761 in k7536 in k7533 */
static void C_fcall f_16867(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_16867,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16880,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1701: last */
f_9256(t2,((C_word*)t0)[2]);}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[103]);
if(C_truep(t2)){
/* irregex-core.scm:1702: every */
f_9337(((C_word*)t0)[3],lf[253],C_u_i_cdr(((C_word*)t0)[2]));}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k16878 in k16865 in k16828 in chicken.irregex#sre-consumer? in k7761 in k7536 in k7533 */
static void C_ccall f_16880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16880,c,av);}
/* irregex-core.scm:1701: sre-consumer? */
t2=lf[253];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_16826(3,av2);}}

/* chicken.irregex#sre-has-submatches? in k7761 in k7536 in k7533 */
static void C_ccall f_16943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_16943,c,av);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_u_i_memq(t3,lf[281]);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(lf[235],C_u_i_car(t2));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16969,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1709: string->sre */
t7=*((C_word*)lf[102]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
/* irregex-core.scm:1710: any */
f_9294(t1,lf[280],C_u_i_cdr(t2));}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16967 in chicken.irregex#sre-has-submatches? in k7761 in k7536 in k7533 */
static void C_ccall f_16969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16969,c,av);}
/* irregex-core.scm:1709: sre-has-submatches? */
t2=lf[280];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_16943(3,av2);}}

/* chicken.irregex#sre-count-submatches in k7761 in k7536 in k7533 */
static void C_fcall f_16982(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_16982,2,t1,t2);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16988,a[2]=t4,a[3]=((C_word)li221),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(0);
f_16988(4,av2);}}

/* count in chicken.irregex#sre-count-submatches in k7761 in k7536 in k7533 */
static void C_ccall f_16988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_16988,c,av);}
a=C_alloc(14);
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17009,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_eqp(t4,lf[245]);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17015,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,a[6]=t4,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
if(C_truep(t6)){
t8=t7;
f_17015(t8,t6);}
else{
t8=C_eqp(t4,lf[108]);
if(C_truep(t8)){
t9=t7;
f_17015(t9,t8);}
else{
t9=C_eqp(t4,lf[119]);
t10=t7;
f_17015(t10,(C_truep(t9)?t9:C_eqp(t4,lf[120])));}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k17007 in count in chicken.irregex#sre-count-submatches in k7761 in k7536 in k7533 */
static void C_ccall f_17009(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17009,c,av);}
/* irregex-core.scm:1715: fold */
f_9382(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1],C_fixnum_plus(((C_word*)t0)[4],t1),C_u_i_cdr(((C_word*)t0)[5]));}

/* k17013 in count in chicken.irregex#sre-count-submatches in k7761 in k7536 in k7533 */
static void C_fcall f_17015(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,4)))){
C_save_and_reclaim_args((void *)trf_17015,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
/* irregex-core.scm:1715: fold */
f_9382(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t2=C_eqp(((C_word*)t0)[6],lf[244]);
if(C_truep(t2)){
t3=C_fixnum_plus(C_i_cadr(((C_word*)t0)[5]),C_i_caddr(((C_word*)t0)[5]));
/* irregex-core.scm:1715: fold */
f_9382(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1],C_fixnum_plus(((C_word*)t0)[4],t3),C_u_i_cdr(((C_word*)t0)[5]));}
else{
t3=C_eqp(((C_word*)t0)[6],lf[235]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17045,a[2]=((C_word*)t0)[7],tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1720: string->sre */
t5=*((C_word*)lf[102]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_cadr(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_s_a_i_plus(&a,2,((C_word*)t0)[4],C_fix(0));
/* irregex-core.scm:1715: fold */
f_9382(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1],t4,C_u_i_cdr(((C_word*)t0)[5]));}}}}

/* k17043 in k17013 in count in chicken.irregex#sre-count-submatches in k7761 in k7536 in k7533 */
static void C_ccall f_17045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17045,c,av);}
/* irregex-core.scm:1720: sre-count-submatches */
f_16982(((C_word*)t0)[2],t1);}

/* k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_17082,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17085,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18182,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1727: sre-count-submatches */
f_16982(t3,((C_word*)t0)[10]);}

/* k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,7)))){
C_save_and_reclaim((void *)f_17085,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17092,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17094,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[10],a[5]=((C_word)li201),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_17094(t6,t2,((C_word*)t0)[11],C_fix(1),C_fix(0),C_fix(0),*((C_word*)lf[266]+1));}

/* k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_17092,c,av);}
a=C_alloc(11);
t2=C_i_vector_set(((C_word*)t0)[2],C_fix(0),t1);
t3=(C_truep(((C_word*)t0)[3])?C_fixnum_or(C_fix(0),C_fix(1)):C_fix(0));
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_16234,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:1634: sre-consumer? */
t5=lf[253];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[10];
f_16826(3,av2);}}

/* lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17094(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,6)))){
C_save_and_reclaim_args((void *)trf_17094,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(19);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17097,a[2]=t4,a[3]=t5,a[4]=t6,a[5]=((C_word)li186),tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(t2))){
if(C_truep(C_i_stringp(C_u_i_car(t2)))){
/* irregex-core.scm:1736: grow */
t8=t7;
f_17097(t8,t1,C_fix(1));}
else{
t8=C_u_i_car(t2);
t9=C_eqp(t8,lf[52]);
t10=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17137,a[2]=t7,a[3]=t1,a[4]=t8,a[5]=((C_word*)t0)[2],a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t6,a[10]=t2,a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[4],tmp=(C_word)a,a+=13,tmp);
if(C_truep(t9)){
t11=t10;
f_17137(t11,t9);}
else{
t11=C_eqp(t8,lf[179]);
if(C_truep(t11)){
t12=t10;
f_17137(t12,t11);}
else{
t12=C_eqp(t8,lf[234]);
t13=t10;
f_17137(t13,(C_truep(t12)?t12:C_eqp(t8,lf[247])));}}}}
else{
if(C_truep(C_charp(t2))){
/* irregex-core.scm:1851: grow */
t8=t7;
f_17097(t8,t1,C_fix(1));}
else{
if(C_truep(C_i_stringp(t2))){
/* irregex-core.scm:1853: grow */
t8=t7;
f_17097(t8,t1,C_block_size(t2));}
else{
if(C_truep((C_truep(C_eqp(t2,lf[124]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[125]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
/* irregex-core.scm:1855: grow */
t8=t7;
f_17097(t8,t1,C_fix(1));}
else{
if(C_truep((C_truep(C_eqp(t2,lf[110]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[186]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[187]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[195]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[196]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[183]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[184]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[185]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[264]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))))))))){
/* irregex-core.scm:1857: return */
t8=t6;{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t8=C_i_assq(t2,lf[248]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18159,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
t10=C_i_cdr(t8);
if(C_truep(C_i_closurep(t10))){
/* irregex-core.scm:1861: g3031 */
t11=C_u_i_cdr(t8);{
C_word av2[2];
av2[0]=t11;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
/* irregex-core.scm:1861: lp */
t14=t1;
t15=C_u_i_cdr(t8);
t16=t3;
t17=t4;
t18=t5;
t19=t6;
t1=t14;
t2=t15;
t3=t16;
t4=t17;
t5=t18;
t6=t19;
goto loop;}}
else{
/* irregex-core.scm:1863: chicken.base#error */
t9=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[261];
av2[3]=lf[265];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}}}}}}}

/* grow in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17097(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_17097,3,t0,t1,t2);}
t3=C_fixnum_plus(((C_word*)t0)[2],t2);
if(C_truep(((C_word*)t0)[3])){
/* irregex-core.scm:1732: return */
t4=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=C_fixnum_plus(((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* irregex-core.scm:1732: return */
t4=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17137(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_17137,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
/* irregex-core.scm:1739: grow */
t2=((C_word*)t0)[2];
f_17097(t2,((C_word*)t0)[3],C_fix(1));}
else{
t2=C_eqp(((C_word*)t0)[4],lf[235]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17153,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1741: string->sre */
t4=*((C_word*)lf[102]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(((C_word*)t0)[10]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_eqp(((C_word*)t0)[4],lf[115]);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17166,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t3)){
t5=t4;
f_17166(t5,t3);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[227]);
if(C_truep(t5)){
t6=t4;
f_17166(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[225]);
if(C_truep(t6)){
t7=t4;
f_17166(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[4],lf[226]);
t8=t4;
f_17166(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[4],lf[239])));}}}}}}

/* k17151 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17153,c,av);}
/* irregex-core.scm:1741: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17094(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17166(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,6)))){
C_save_and_reclaim_args((void *)trf_17166,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17173,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t4,a[6]=((C_word*)t0)[6],a[7]=((C_word)li188),tmp=(C_word)a,a+=8,tmp));
t6=((C_word*)t4)[1];
f_17173(t6,((C_word*)t0)[7],t2,((C_word*)t0)[8],C_fix(0),C_fix(0));}
else{
t2=C_eqp(((C_word*)t0)[9],lf[103]);
if(C_truep(t2)){
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17251,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t5,a[6]=((C_word*)t0)[6],a[7]=((C_word)li190),tmp=(C_word)a,a+=8,tmp));
t7=((C_word*)t5)[1];
f_17251(t7,((C_word*)t0)[7],t3,((C_word*)t0)[8],C_SCHEME_FALSE,C_fix(0));}
else{
t3=C_eqp(((C_word*)t0)[9],lf[109]);
if(C_truep(t3)){
t4=C_i_nullp(C_u_i_cdr(((C_word*)t0)[2]));
t5=(C_truep(t4)?t4:C_i_nullp(C_i_cddr(((C_word*)t0)[2])));
if(C_truep(t5)){
/* irregex-core.scm:1765: return */
t6=((C_word*)t0)[5];{
C_word av2[4];
av2[0]=t6;
av2[1]=((C_word*)t0)[7];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17344,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1767: sre-count-submatches */
f_16982(t6,C_u_i_car(((C_word*)t0)[2]));}}
else{
t4=C_eqp(((C_word*)t0)[9],lf[244]);
if(C_truep(t4)){
t5=(
/* irregex-core.scm:1786: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdddr(((C_word*)t0)[2]))
);
t6=C_i_cadr(((C_word*)t0)[2]);
/* irregex-core.scm:1786: lp */
t7=((C_word*)((C_word*)t0)[6])[1];
f_17094(t7,((C_word*)t0)[7],t5,C_fixnum_plus(((C_word*)t0)[8],t6),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t5=C_eqp(((C_word*)t0)[9],lf[245]);
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17490,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t5)){
t7=t6;
f_17490(t7,t5);}
else{
t7=C_eqp(((C_word*)t0)[9],lf[108]);
if(C_truep(t7)){
t8=t6;
f_17490(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[9],lf[119]);
t9=t6;
f_17490(t9,(C_truep(t8)?t8:C_eqp(((C_word*)t0)[9],lf[120])));}}}}}}}

/* lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17173(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,6)))){
C_save_and_reclaim_args((void *)trf_17173,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t6=C_fixnum_plus(((C_word*)t0)[2],t4);
if(C_truep(((C_word*)t0)[3])){
if(C_truep(t5)){
/* irregex-core.scm:1745: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_fixnum_plus(((C_word*)t0)[3],t5);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
/* irregex-core.scm:1745: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
/* irregex-core.scm:1745: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
t6=C_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17206,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t5,a[6]=((C_word*)t0)[5],a[7]=((C_word)li187),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1746: lp */
t8=((C_word*)((C_word*)t0)[6])[1];
f_17094(t8,t1,t6,t3,C_fix(0),C_fix(0),t7);}}

/* a17205 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_17206,c,av);}
a=C_alloc(10);
t4=C_i_cdr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17236,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=t3,a[7]=((C_word*)t0)[6],a[8]=t1,a[9]=t4,tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1749: sre-count-submatches */
f_16982(t5,C_u_i_car(((C_word*)t0)[2]));}

/* k17234 in a17205 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17236,c,av);}
t2=C_fixnum_plus(((C_word*)t0)[2],t1);
t3=C_fixnum_plus(((C_word*)t0)[3],((C_word*)t0)[4]);
if(C_truep(((C_word*)t0)[5])){
if(C_truep(((C_word*)t0)[6])){
/* irregex-core.scm:1748: lp2 */
t4=((C_word*)((C_word*)t0)[7])[1];
f_17173(t4,((C_word*)t0)[8],((C_word*)t0)[9],t2,t3,C_fixnum_plus(((C_word*)t0)[5],((C_word*)t0)[6]));}
else{
/* irregex-core.scm:1748: lp2 */
t4=((C_word*)((C_word*)t0)[7])[1];
f_17173(t4,((C_word*)t0)[8],((C_word*)t0)[9],t2,t3,C_SCHEME_FALSE);}}
else{
/* irregex-core.scm:1748: lp2 */
t4=((C_word*)((C_word*)t0)[7])[1];
f_17173(t4,((C_word*)t0)[8],((C_word*)t0)[9],t2,t3,C_SCHEME_FALSE);}}

/* lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17251(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,6)))){
C_save_and_reclaim_args((void *)trf_17251,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t6=(C_truep(t4)?C_fixnum_plus(((C_word*)t0)[2],t4):C_fixnum_plus(((C_word*)t0)[2],C_fix(1)));
if(C_truep(((C_word*)t0)[3])){
if(C_truep(t5)){
/* irregex-core.scm:1755: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_fixnum_plus(((C_word*)t0)[3],t5);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
/* irregex-core.scm:1755: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
/* irregex-core.scm:1755: return */
t7=((C_word*)t0)[4];{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
t6=C_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17288,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=t5,a[6]=t4,a[7]=((C_word)li189),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1756: lp */
t8=((C_word*)((C_word*)t0)[6])[1];
f_17094(t8,t1,t6,t3,C_fix(0),C_fix(0),t7);}}

/* a17287 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17288(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_17288,c,av);}
a=C_alloc(10);
t4=C_i_cdr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17321,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t4,a[6]=((C_word*)t0)[5],a[7]=t3,a[8]=((C_word*)t0)[6],a[9]=t2,tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1759: sre-count-submatches */
f_16982(t5,C_u_i_car(((C_word*)t0)[2]));}

/* k17302 in k17319 in a17287 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_17304(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_17304,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17308,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[6])){
if(C_truep(((C_word*)t0)[7])){
/* irregex-core.scm:1761: scheme#max */
t3=*((C_word*)lf[254]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* irregex-core.scm:1758: lp2 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_17251(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,C_SCHEME_FALSE);}}
else{
/* irregex-core.scm:1758: lp2 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_17251(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,C_SCHEME_FALSE);}}

/* k17306 in k17302 in k17319 in a17287 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_17308(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17308,c,av);}
/* irregex-core.scm:1758: lp2 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17251(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1);}

/* k17319 in a17287 in lp2 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17321,c,av);}
a=C_alloc(8);
t2=C_fixnum_plus(((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17304,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[8])){
/* irregex-core.scm:1760: scheme#min */
t4=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[9];
f_17304(2,av2);}}}

/* k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17344(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_17344,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17347,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:1768: sre-count-submatches */
f_16982(t2,C_i_cadr(((C_word*)t0)[2]));}

/* k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_17347,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17354,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_i_numberp(t3);
if(C_truep(t4)){
t5=t2;
f_17354(t5,(C_truep(t4)?lf[110]:C_u_i_car(C_u_i_cdr(((C_word*)t0)[2]))));}
else{
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_i_symbolp(C_u_i_car(t5));
t7=t2;
f_17354(t7,(C_truep(t6)?lf[110]:C_u_i_car(C_u_i_cdr(((C_word*)t0)[2]))));}}

/* k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17354(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,6)))){
C_save_and_reclaim_args((void *)trf_17354,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17356,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word)li193),tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1769: lp */
t3=((C_word*)((C_word*)t0)[7])[1];
f_17094(t3,((C_word*)t0)[8],t1,((C_word*)t0)[3],((C_word*)t0)[9],((C_word*)t0)[10],t2);}

/* a17355 in k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_17356(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,6)))){
C_save_and_reclaim((void *)f_17356,c,av);}
a=C_alloc(11);
t4=C_i_caddr(((C_word*)t0)[2]);
t5=C_fixnum_plus(((C_word*)t0)[3],((C_word*)t0)[4]);
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17370,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=t3,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word)li192),tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:1774: lp */
t7=((C_word*)((C_word*)t0)[7])[1];
f_17094(t7,t1,t4,t5,C_fix(0),C_fix(0),t6);}

/* a17369 in a17355 in k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_17370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,6)))){
C_save_and_reclaim((void *)f_17370,c,av);}
a=C_alloc(8);
t4=C_i_cdddr(((C_word*)t0)[2]);
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_i_cadddr(((C_word*)t0)[2]):lf[110]);
t7=C_fixnum_plus(C_fixnum_plus(((C_word*)t0)[3],((C_word*)t0)[4]),((C_word*)t0)[5]);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17384,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=t3,a[5]=((C_word*)t0)[8],a[6]=t2,a[7]=((C_word)li191),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1776: lp */
t9=((C_word*)((C_word*)t0)[9])[1];
f_17094(t9,t1,t6,t7,C_fix(0),C_fix(0),t8);}

/* a17383 in a17369 in a17355 in k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in ... */
static void C_ccall f_17384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17384,c,av);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17413,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:1781: scheme#min */
t5=*((C_word*)lf[255]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k17407 in k17411 in a17383 in a17369 in a17355 in k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in ... */
static void C_ccall f_17409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17409,c,av);}
/* irregex-core.scm:1781: return */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_fixnum_plus(((C_word*)t0)[5],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17411 in a17383 in a17369 in a17355 in k17352 in k17345 in k17342 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_17413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17413,c,av);}
a=C_alloc(6);
t2=C_fixnum_plus(((C_word*)t0)[2],t1);
if(C_truep(((C_word*)t0)[3])){
if(C_truep(((C_word*)t0)[4])){
if(C_truep(((C_word*)t0)[5])){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17409,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1783: scheme#max */
t4=*((C_word*)lf[254]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* irregex-core.scm:1781: return */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}
else{
/* irregex-core.scm:1781: return */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}
else{
/* irregex-core.scm:1781: return */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17490(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,6)))){
C_save_and_reclaim_args((void *)trf_17490,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_eqp(lf[108],C_u_i_car(((C_word*)t0)[2]));
t3=(C_truep(t2)?(
/* irregex-core.scm:1788: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[2]))
):(
/* irregex-core.scm:1788: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cddr(((C_word*)t0)[2]))
));
t4=C_fixnum_plus(((C_word*)t0)[3],C_fix(1));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17503,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word)li194),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1788: lp */
t6=((C_word*)((C_word*)t0)[6])[1];
f_17094(t6,((C_word*)t0)[7],t3,t4,((C_word*)t0)[8],((C_word*)t0)[9],t5);}
else{
t2=C_eqp(((C_word*)t0)[10],lf[192]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[10],lf[191]));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17539,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t5=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_numberp(t5))){
t6=t4;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_u_i_car(C_u_i_cdr(((C_word*)t0)[2]));
f_17539(2,av2);}}
else{
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_i_assq(C_u_i_car(t6),((C_word*)t0)[11]);
if(C_truep(t7)){
t8=t4;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_i_cdr(t7);
f_17539(2,av2);}}
else{
t8=C_u_i_cdr(((C_word*)t0)[2]);
/* irregex-core.scm:1798: chicken.base#error */
t9=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t4;
av2[2]=lf[259];
av2[3]=C_u_i_car(t8);
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}}}
else{
t4=C_eqp(((C_word*)t0)[10],lf[127]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[10],lf[128]));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17648,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
t7=(
/* irregex-core.scm:1810: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[2]))
);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17657,a[2]=((C_word)li195),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1810: lp */
t9=((C_word*)((C_word*)t0)[6])[1];
f_17094(t9,t6,t7,((C_word*)t0)[3],((C_word*)t0)[8],((C_word*)t0)[9],t8);}
else{
t6=C_eqp(((C_word*)t0)[10],lf[133]);
t7=(C_truep(t6)?t6:C_eqp(((C_word*)t0)[10],lf[130]));
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17675,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
t9=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_numberp(t9))){
t10=C_i_caddr(((C_word*)t0)[2]);
if(C_truep(C_i_numberp(t10))){
t11=C_u_i_cdr(((C_word*)t0)[2]);
t12=C_u_i_car(t11);
t13=C_u_i_cdr(((C_word*)t0)[2]);
t14=C_u_i_cdr(t13);
t15=t8;
f_17675(t15,C_fixnum_greaterp(t12,C_u_i_car(t14)));}
else{
t11=t8;
f_17675(t11,C_SCHEME_FALSE);}}
else{
t10=t8;
f_17675(t10,C_SCHEME_FALSE);}}
else{
t8=C_eqp(((C_word*)t0)[10],lf[129]);
if(C_truep(t8)){
t9=(
/* irregex-core.scm:1829: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[2]))
);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17817,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[8],a[4]=((C_word)li198),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1829: lp */
t11=((C_word*)((C_word*)t0)[6])[1];
f_17094(t11,((C_word*)t0)[7],t9,((C_word*)t0)[3],((C_word*)t0)[8],((C_word*)t0)[9],t10);}
else{
t9=C_eqp(((C_word*)t0)[10],lf[131]);
t10=(C_truep(t9)?t9:C_eqp(((C_word*)t0)[10],lf[132]));
if(C_truep(t10)){
t11=(
/* irregex-core.scm:1833: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[2]))
);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17845,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[8],a[5]=((C_word)li199),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1833: lp */
t13=((C_word*)((C_word*)t0)[6])[1];
f_17094(t13,((C_word*)t0)[7],t11,((C_word*)t0)[3],((C_word*)t0)[8],((C_word*)t0)[9],t12);}
else{
t11=C_eqp(((C_word*)t0)[10],lf[134]);
t12=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17870,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t11)){
t13=t12;
f_17870(t13,t11);}
else{
t13=C_eqp(((C_word*)t0)[10],lf[263]);
if(C_truep(t13)){
t14=t12;
f_17870(t14,t13);}
else{
t14=C_eqp(((C_word*)t0)[10],lf[135]);
t15=t12;
f_17870(t15,(C_truep(t14)?t14:C_eqp(((C_word*)t0)[10],lf[260])));}}}}}}}}}

/* a17502 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17503,c,av);}
a=C_alloc(3);
t4=C_a_i_cons(&a,2,t2,t3);
t5=C_i_vector_set(((C_word*)t0)[2],((C_word*)t0)[3],t4);
/* irregex-core.scm:1793: return */
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k17537 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_17539,c,av);}
a=C_alloc(9);
t2=C_i_not(C_i_integerp(t1));
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17548,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t2)){
t4=t3;
f_17548(t4,t2);}
else{
t4=C_block_size(((C_word*)t0)[4]);
t5=t3;
f_17548(t5,C_i_not(C_and(C_fixnum_lessp(C_fix(0),t1),C_fixnum_lessp(t1,t4))));}}

/* k17546 in k17537 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17548(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_17548,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:1802: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[256];
av2[3]=lf[257];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=C_i_vector_ref(((C_word*)t0)[4],((C_word*)t0)[5]);
if(C_truep(C_i_not(t2))){
/* irregex-core.scm:1804: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[256];
av2[3]=lf[258];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=C_i_vector_ref(((C_word*)t0)[4],((C_word*)t0)[5]);
t4=C_i_car(t3);
t5=C_i_vector_ref(((C_word*)t0)[4],((C_word*)t0)[5]);
t6=C_i_cdr(t5);
t7=C_fixnum_plus(((C_word*)t0)[6],t4);
if(C_truep(((C_word*)t0)[7])){
if(C_truep(t6)){
/* irregex-core.scm:1808: return */
t8=((C_word*)t0)[8];{
C_word av2[4];
av2[0]=t8;
av2[1]=((C_word*)t0)[2];
av2[2]=t7;
av2[3]=C_fixnum_plus(((C_word*)t0)[7],t6);
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* irregex-core.scm:1808: return */
t8=((C_word*)t0)[8];{
C_word av2[4];
av2[0]=t8;
av2[1]=((C_word*)t0)[2];
av2[2]=t7;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}
else{
/* irregex-core.scm:1808: return */
t8=((C_word*)t0)[8];{
C_word av2[4];
av2[0]=t8;
av2[1]=((C_word*)t0)[2];
av2[2]=t7;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}}}

/* k17646 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17648,c,av);}
/* irregex-core.scm:1811: return */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a17656 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17657,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k17673 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17675(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_17675,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17678,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=t2;
f_17678(t3,t1);}
else{
t3=C_u_i_cdr(((C_word*)t0)[6]);
t4=C_i_not(C_u_i_car(t3));
t5=t2;
f_17678(t5,(C_truep(t4)?C_i_caddr(((C_word*)t0)[6]):C_SCHEME_FALSE));}}

/* k17676 in k17673 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17678(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,6)))){
C_save_and_reclaim_args((void *)trf_17678,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
/* irregex-core.scm:1818: return */
t2=((C_word*)t0)[2];{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
if(C_truep(C_i_caddr(((C_word*)t0)[6]))){
t2=C_u_i_cdr(((C_word*)t0)[6]);
t3=C_u_i_cdr(t2);
t4=(
/* irregex-core.scm:1821: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t3))
);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17696,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word)li196),tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:1821: lp */
t6=((C_word*)((C_word*)t0)[7])[1];
f_17094(t6,((C_word*)t0)[3],t4,((C_word*)t0)[8],C_fix(0),C_fix(0),t5);}
else{
t2=(
/* irregex-core.scm:1825: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdddr(((C_word*)t0)[6]))
);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17743,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word)li197),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1825: lp */
t4=((C_word*)((C_word*)t0)[7])[1];
f_17094(t4,((C_word*)t0)[3],t2,((C_word*)t0)[8],C_fix(0),C_fix(0),t3);}}}

/* a17695 in k17676 in k17673 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_17696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17696,c,av);}
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_fixnum_times(t4,t2);
t6=C_fixnum_plus(((C_word*)t0)[3],t5);
if(C_truep(((C_word*)t0)[4])){
if(C_truep(t3)){
t7=C_i_caddr(((C_word*)t0)[2]);
t8=C_fixnum_times(t7,t3);
/* irregex-core.scm:1823: return */
t9=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_fixnum_plus(((C_word*)t0)[4],t8);
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
/* irregex-core.scm:1823: return */
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
/* irregex-core.scm:1823: return */
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}

/* a17742 in k17676 in k17673 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_17743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17743,c,av);}
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_fixnum_times(t4,t2);
/* irregex-core.scm:1827: return */
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=C_fixnum_plus(((C_word*)t0)[4],t5);
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* a17816 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17817,c,av);}
/* irregex-core.scm:1831: return */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=C_fixnum_plus(((C_word*)t0)[3],t2);
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a17844 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_17845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17845,c,av);}
if(C_truep(((C_word*)t0)[2])){
if(C_truep(t3)){
/* irregex-core.scm:1835: return */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fixnum_plus(((C_word*)t0)[2],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* irregex-core.scm:1835: return */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}
else{
/* irregex-core.scm:1835: return */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k17868 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17870(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_17870,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17889,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t4=C_u_i_car(((C_word*)t0)[2]);
if(C_truep((C_truep(C_eqp(t4,lf[135]))?C_SCHEME_TRUE:(C_truep(C_eqp(t4,lf[260]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_u_i_cdr(t5);
t7=t3;
f_17889(t7,C_a_i_cons(&a,2,C_SCHEME_FALSE,t6));}
else{
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_u_i_car(t5);
t7=C_u_i_cdr(((C_word*)t0)[2]);
t8=C_u_i_cdr(t7);
t9=t3;
f_17889(t9,C_a_i_cons(&a,2,t6,t8));}}
else{
t2=C_eqp(((C_word*)t0)[9],lf[111]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17910,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t2)){
t4=t3;
f_17910(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[9],lf[112]);
if(C_truep(t4)){
t5=t3;
f_17910(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[9],lf[113]);
t6=t3;
f_17910(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[9],lf[114])));}}}}

/* k17887 in k17868 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17889(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,6)))){
C_save_and_reclaim_args((void *)trf_17889,2,t0,t1);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,lf[133],t2);
/* irregex-core.scm:1837: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_17094(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k17908 in k17868 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_17910(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_17910,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
/* irregex-core.scm:1842: return */
t2=((C_word*)t0)[2];{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_i_assq(C_u_i_car(((C_word*)t0)[6]),lf[248]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17920,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word)li200),tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:1844: g3027 */
t4=t3;
f_17920(t4,((C_word*)t0)[3],t2);}
else{
/* irregex-core.scm:1849: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[261];
av2[3]=lf[262];
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}}

/* g3027 in k17908 in k17868 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_fcall f_17920(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_17920,3,t0,t1,t2);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17928,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);{
C_word av2[4];
av2[0]=0;
av2[1]=t3;
av2[2]=C_i_cdr(t2);
av2[3]=C_i_cdr(((C_word*)t0)[7]);
C_apply(4,av2);}}

/* k17926 in g3027 in k17908 in k17868 in k17488 in k17164 in k17135 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_17928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17928,c,av);}
/* irregex-core.scm:1847: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17094(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k18157 in lp in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_18159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_18159,c,av);}
/* irregex-core.scm:1861: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17094(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k18180 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_18182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18182,c,av);}
/* irregex-core.scm:1727: scheme#make-vector */
t2=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fixnum_plus(C_fix(1),t1);
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.irregex#sre-sequence in k7761 in k7536 in k7533 */
static C_word C_fcall f_18191(C_word *a,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
if(C_truep(C_i_nullp(t1))){
return(lf[110]);}
else{
t2=C_i_cdr(t1);
t3=C_i_nullp(t2);
return((C_truep(t3)?C_u_i_car(t1):C_a_i_cons(&a,2,lf[115],t1)));}}

/* chicken.irregex#sre-alternate in k7761 in k7536 in k7533 */
static C_word C_fcall f_18214(C_word *a,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
if(C_truep(C_i_nullp(t1))){
return(lf[282]);}
else{
t2=C_i_cdr(t1);
t3=C_i_nullp(t2);
return((C_truep(t3)?C_u_i_car(t1):C_a_i_cons(&a,2,lf[103],t1)));}}

/* chicken.irregex#sre-strip-submatches in k7761 in k7536 in k7533 */
static void C_ccall f_18237(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
loop:
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
av[0]=t0;
av[1]=t1;
av[2]=t2;
C_save_and_reclaim((void *)f_18237,c,av);}
a=C_alloc(13);
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_eqp(t4,lf[245]);
t6=(C_truep(t5)?t5:C_eqp(t4,lf[108]));
if(C_truep(t6)){
t7=(
/* irregex-core.scm:1888: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:1888: sre-strip-submatches */
t19=t1;
t20=t7;
t1=t19;
t2=t20;
c=3;
goto loop;}
else{
t7=C_eqp(t4,lf[119]);
t8=(C_truep(t7)?t7:C_eqp(t4,lf[120]));
if(C_truep(t8)){
t9=(
/* irregex-core.scm:1889: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cddr(t2))
);
/* irregex-core.scm:1889: sre-strip-submatches */
t19=t1;
t20=t9;
t1=t19;
t2=t20;
c=3;
goto loop;}
else{
t9=C_eqp(t4,lf[244]);
if(C_truep(t9)){
t10=(
/* irregex-core.scm:1890: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdddr(t2))
);
/* irregex-core.scm:1890: sre-strip-submatches */
t19=t1;
t20=t10;
t1=t19;
t2=t20;
c=3;
goto loop;}
else{
t10=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t11=t10;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=((C_word*)t12)[1];
t14=lf[283];
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18310,a[2]=t12,a[3]=t16,a[4]=t13,a[5]=((C_word)li225),tmp=(C_word)a,a+=6,tmp));
t18=((C_word*)t16)[1];
f_18310(t18,t1,t2);}}}}}

/* map-loop3064 in chicken.irregex#sre-strip-submatches in k7761 in k7536 in k7533 */
static void C_fcall f_18310(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18310,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18335,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1891: g3070 */
t4=lf[283];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_18237(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18333 in map-loop3064 in chicken.irregex#sre-strip-submatches in k7761 in k7536 in k7533 */
static void C_ccall f_18335(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18335,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18310(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* lp in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_18370(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_18370,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
/* irregex-core.scm:1899: ##sys#fast-reverse */
t4=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_stringp(t4))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18393,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18397,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* ##sys#string->list */
t7=*((C_word*)lf[105]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* irregex-core.scm:1903: lp */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* k18391 in lp in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18393,c,av);}
/* irregex-core.scm:1901: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18370(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4]);}

/* k18395 in lp in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18397,c,av);}
/* irregex-core.scm:1901: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.irregex#sre-names in k7761 in k7536 in k7533 */
static void C_fcall f_18418(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_18418,4,t1,t2,t3,t4);}
a=C_alloc(9);
t5=C_i_pairp(t2);
if(C_truep(C_i_not(t5))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_eqp(t6,lf[245]);
t8=(C_truep(t7)?t7:C_eqp(t6,lf[108]));
if(C_truep(t8)){
t9=(
/* irregex-core.scm:1910: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:1910: sre-names */
t28=t1;
t29=t9;
t30=C_fixnum_plus(t3,C_fix(1));
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;}
else{
t9=C_eqp(t6,lf[119]);
t10=(C_truep(t9)?t9:C_eqp(t6,lf[120]));
if(C_truep(t10)){
t11=(
/* irregex-core.scm:1912: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cddr(t2))
);
t12=C_fixnum_plus(t3,C_fix(1));
t13=C_i_cadr(t2);
t14=C_a_i_cons(&a,2,t13,t3);
t15=C_a_i_cons(&a,2,t14,t4);
/* irregex-core.scm:1912: sre-names */
t28=t1;
t29=t11;
t30=t12;
t31=t15;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;}
else{
t11=C_eqp(t6,lf[244]);
if(C_truep(t11)){
t12=(
/* irregex-core.scm:1916: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdddr(t2))
);
t13=C_i_cadr(t2);
/* irregex-core.scm:1916: sre-names */
t28=t1;
t29=t12;
t30=C_fixnum_plus(t3,t13);
t31=t4;
t1=t28;
t2=t29;
t3=t30;
t4=t31;
goto loop;}
else{
t12=C_eqp(t6,lf[115]);
t13=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18520,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t12)){
t14=t13;
f_18520(t14,t12);}
else{
t14=C_eqp(t6,lf[227]);
if(C_truep(t14)){
t15=t13;
f_18520(t15,t14);}
else{
t15=C_eqp(t6,lf[103]);
if(C_truep(t15)){
t16=t13;
f_18520(t16,t15);}
else{
t16=C_eqp(t6,lf[127]);
if(C_truep(t16)){
t17=t13;
f_18520(t17,t16);}
else{
t17=C_eqp(t6,lf[129]);
if(C_truep(t17)){
t18=t13;
f_18520(t18,t17);}
else{
t18=C_eqp(t6,lf[131]);
if(C_truep(t18)){
t19=t13;
f_18520(t19,t18);}
else{
t19=C_eqp(t6,lf[128]);
if(C_truep(t19)){
t20=t13;
f_18520(t20,t19);}
else{
t20=C_eqp(t6,lf[132]);
if(C_truep(t20)){
t21=t13;
f_18520(t21,t20);}
else{
t21=C_eqp(t6,lf[225]);
if(C_truep(t21)){
t22=t13;
f_18520(t22,t21);}
else{
t22=C_eqp(t6,lf[226]);
if(C_truep(t22)){
t23=t13;
f_18520(t23,t22);}
else{
t23=C_eqp(t6,lf[239]);
if(C_truep(t23)){
t24=t13;
f_18520(t24,t23);}
else{
t24=C_eqp(t6,lf[111]);
if(C_truep(t24)){
t25=t13;
f_18520(t25,t24);}
else{
t25=C_eqp(t6,lf[113]);
if(C_truep(t25)){
t26=t13;
f_18520(t26,t25);}
else{
t26=C_eqp(t6,lf[112]);
t27=t13;
f_18520(t27,(C_truep(t26)?t26:C_eqp(t6,lf[114])));}}}}}}}}}}}}}}}}}}

/* k18518 in chicken.irregex#sre-names in k7761 in k7536 in k7533 */
static void C_fcall f_18520(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_18520,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:1919: sre-sequence-names */
f_18704(((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[3]),((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t2=C_eqp(((C_word*)t0)[6],lf[134]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[6],lf[135]));
if(C_truep(t3)){
/* irregex-core.scm:1921: sre-sequence-names */
f_18704(((C_word*)t0)[2],C_i_cddr(((C_word*)t0)[3]),((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t4=C_eqp(((C_word*)t0)[6],lf[133]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[6],lf[130]));
if(C_truep(t5)){
/* irregex-core.scm:1923: sre-sequence-names */
f_18704(((C_word*)t0)[2],C_i_cdddr(((C_word*)t0)[3]),((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t6=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t6;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}

/* chicken.irregex#sre-sequence-names in k7761 in k7536 in k7533 */
static void C_fcall f_18704(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_18704,4,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_cdr(t2);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18732,a[2]=t3,a[3]=t1,a[4]=t5,a[5]=t2,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:1931: sre-count-submatches */
f_16982(t6,C_u_i_car(t2));}}

/* k18724 in k18730 in chicken.irregex#sre-sequence-names in k7761 in k7536 in k7533 */
static void C_ccall f_18726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18726,c,av);}
/* irregex-core.scm:1930: sre-sequence-names */
f_18704(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k18730 in chicken.irregex#sre-sequence-names in k7761 in k7536 in k7533 */
static void C_ccall f_18732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_18732,c,av);}
a=C_alloc(5);
t2=C_fixnum_plus(((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18726,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:1932: sre-names */
f_18418(t3,C_u_i_car(((C_word*)t0)[5]),((C_word*)t0)[2],((C_word*)t0)[6]);}

/* chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_ccall f_18736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_18736,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[115]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18753,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;
f_18753(t6,t4);}
else{
t6=C_eqp(t3,lf[227]);
if(C_truep(t6)){
t7=t5;
f_18753(t7,t6);}
else{
t7=C_eqp(t3,lf[245]);
if(C_truep(t7)){
t8=t5;
f_18753(t8,t7);}
else{
t8=C_eqp(t3,lf[108]);
if(C_truep(t8)){
t9=t5;
f_18753(t9,t8);}
else{
t9=C_eqp(t3,lf[119]);
if(C_truep(t9)){
t10=t5;
f_18753(t10,t9);}
else{
t10=C_eqp(t3,lf[120]);
if(C_truep(t10)){
t11=t5;
f_18753(t11,t10);}
else{
t11=C_eqp(t3,lf[127]);
t12=t5;
f_18753(t12,(C_truep(t11)?t11:C_eqp(t3,lf[129])));}}}}}}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18751 in chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_fcall f_18753(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,3)))){
C_save_and_reclaim_args((void *)trf_18753,2,t0,t1);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
if(C_truep(C_i_not(t2))){
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_eqp(lf[186],t3);
if(C_truep(t4)){
t5=C_u_i_car(((C_word*)t0)[2]);
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_u_i_cdr(t6);
t8=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_cons(&a,2,t5,t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t5=C_u_i_car(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18786,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
t7=C_u_i_cdr(((C_word*)t0)[2]);
/* irregex-core.scm:1946: sre-remove-initial-bos */
t8=lf[270];{
C_word av2[3];
av2[0]=t8;
av2[1]=t6;
av2[2]=C_u_i_car(t7);
f_18736(3,av2);}}}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[103]);
if(C_truep(t2)){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=lf[270];
t8=C_u_i_cdr(((C_word*)t0)[2]);
t9=C_i_check_list_2(t8,lf[211]);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18823,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18825,a[2]=t5,a[3]=t12,a[4]=t6,a[5]=((C_word)li229),tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_18825(t14,t10,t8);}
else{
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k18784 in k18751 in chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_ccall f_18786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_18786,c,av);}
a=C_alloc(6);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k18821 in k18751 in chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_ccall f_18823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18823,c,av);}
a=C_alloc(3);
/* irregex-core.scm:1948: sre-alternate */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* irregex-core.scm:1948: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop3205 in k18751 in chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_fcall f_18825(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18825,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18850,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1948: g3211 */
t4=lf[270];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_18736(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18848 in map-loop3205 in k18751 in chicken.irregex#sre-remove-initial-bos in k7761 in k7536 in k7533 */
static void C_ccall f_18850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18850,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18825(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word t91;
C_word t92;
C_word t93;
C_word t94;
C_word t95;
C_word t96;
C_word t97;
C_word t98;
C_word t99;
C_word t100;
C_word t101;
C_word t102;
C_word t103;
C_word t104;
C_word t105;
C_word t106;
C_word t107;
C_word t108;
C_word t109;
C_word t110;
C_word t111;
C_word t112;
C_word t113;
C_word t114;
C_word t115;
C_word t116;
C_word t117;
C_word t118;
C_word t119;
C_word t120;
C_word t121;
C_word t122;
C_word t123;
C_word t124;
C_word t125;
C_word t126;
C_word t127;
C_word t128;
C_word t129;
C_word t130;
C_word t131;
C_word t132;
C_word t133;
C_word t134;
C_word t135;
C_word t136;
C_word t137;
C_word t138;
C_word t139;
C_word t140;
C_word t141;
C_word t142;
C_word t143;
C_word t144;
C_word t145;
C_word t146;
C_word t147;
C_word t148;
C_word t149;
C_word t150;
C_word t151;
C_word t152;
C_word t153;
C_word t154;
C_word t155;
C_word t156;
C_word t157;
C_word t158;
C_word t159;
C_word t160;
C_word t161;
C_word t162;
C_word t163;
C_word t164;
C_word t165;
C_word t166;
C_word t167;
C_word t168;
C_word t169;
C_word t170;
C_word t171;
C_word t172;
C_word t173;
C_word t174;
C_word t175;
C_word t176;
C_word t177;
C_word t178;
C_word t179;
C_word t180;
C_word t181;
C_word t182;
C_word t183;
C_word t184;
C_word t185;
C_word t186;
C_word t187;
C_word t188;
C_word t189;
C_word t190;
C_word t191;
C_word t192;
C_word t193;
C_word t194;
C_word t195;
C_word t196;
C_word t197;
C_word t198;
C_word t199;
C_word t200;
C_word t201;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(1658,c,10)))){
C_save_and_reclaim((void *)f_18918,c,av);}
a=C_alloc(1658);
t2=C_mutate(&lf[285] /* (set! chicken.irregex#irregex-basic-string-chunker ...) */,t1);
t3=C_mutate((C_word*)lf[286]+1 /* (set! chicken.irregex#irregex-search ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18920,a[2]=((C_word)li231),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[287]+1 /* (set! chicken.irregex#irregex-search/chunked ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19010,a[2]=((C_word)li232),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate(&lf[292] /* (set! chicken.irregex#irregex-search/matches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19053,a[2]=((C_word)li241),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[298]+1 /* (set! chicken.irregex#irregex-match ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19302,a[2]=((C_word)li242),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[299]+1 /* (set! chicken.irregex#irregex-match/chunked ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19392,a[2]=((C_word)li245),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[304]+1 /* (set! chicken.irregex#irregex-match? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19620,a[2]=((C_word)li246),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate(&lf[305] /* (set! chicken.irregex#finalize! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19816,a[2]=((C_word)li249),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate(&lf[295] /* (set! chicken.irregex#dfa-match/longest ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19935,a[2]=((C_word)li262),tmp=(C_word)a,a+=3,tmp));
t11=C_a_i_cons(&a,2,lf[124],lf[53]);
t12=C_a_i_string(&a,1,C_make_character(10));
t13=C_a_i_list(&a,1,t12);
t14=C_a_i_list(&a,4,lf[125],lf[247],lf[53],t13);
t15=C_a_i_list(&a,6,lf[307],lf[52],C_make_character(97),C_make_character(122),C_make_character(65),C_make_character(90));
t16=C_a_i_cons(&a,2,lf[308],lf[307]);
t17=C_a_i_list(&a,8,lf[181],lf[52],C_make_character(97),C_make_character(122),C_make_character(65),C_make_character(90),C_make_character(48),C_make_character(57));
t18=C_a_i_cons(&a,2,lf[309],lf[181]);
t19=C_a_i_cons(&a,2,lf[310],lf[181]);
t20=C_a_i_list(&a,4,lf[311],lf[52],C_make_character(97),C_make_character(122));
t21=C_a_i_cons(&a,2,lf[312],lf[311]);
t22=C_a_i_list(&a,4,lf[313],lf[52],C_make_character(65),C_make_character(90));
t23=C_a_i_cons(&a,2,lf[314],lf[313]);
t24=C_a_i_list(&a,4,lf[178],lf[52],C_make_character(48),C_make_character(57));
t25=C_a_i_cons(&a,2,lf[315],lf[178]);
t26=C_a_i_cons(&a,2,lf[316],lf[178]);
t27=C_a_i_list(&a,25,lf[317],lf[103],C_make_character(33),C_make_character(34),C_make_character(35),C_make_character(37),C_make_character(38),C_make_character(39),C_make_character(40),C_make_character(41),C_make_character(42),C_make_character(44),C_make_character(45),C_make_character(46),C_make_character(47),C_make_character(58),C_make_character(59),C_make_character(63),C_make_character(64),C_make_character(91),C_make_character(92),C_make_character(93),C_make_character(95),C_make_character(123),C_make_character(125));
t28=C_a_i_cons(&a,2,lf[318],lf[317]);
t29=C_a_i_list(&a,13,lf[319],lf[103],lf[181],lf[317],C_make_character(36),C_make_character(43),C_make_character(60),C_make_character(61),C_make_character(62),C_make_character(94),C_make_character(96),C_make_character(124),C_make_character(126));
t30=C_a_i_cons(&a,2,lf[320],lf[319]);
t31=C_a_i_list(&a,4,lf[321],lf[103],C_make_character(32),C_make_character(9));
t32=C_a_i_list(&a,4,lf[322],lf[103],lf[321],C_make_character(10));
t33=C_a_i_cons(&a,2,lf[79],lf[322]);
t34=C_a_i_cons(&a,2,lf[323],lf[322]);
t35=C_a_i_list(&a,4,lf[324],lf[103],lf[319],lf[322]);
t36=C_a_i_cons(&a,2,lf[325],lf[324]);
t37=C_a_i_list(&a,4,lf[326],lf[52],C_make_character(0),C_make_character(31));
t38=C_a_i_cons(&a,2,lf[327],lf[326]);
t39=C_a_i_list(&a,5,lf[52],C_make_character(97),C_make_character(102),C_make_character(65),C_make_character(70));
t40=C_a_i_list(&a,4,lf[328],lf[103],lf[178],t39);
t41=C_a_i_cons(&a,2,lf[329],lf[328]);
t42=C_a_i_list(&a,4,lf[330],lf[52],C_make_character(0),C_make_character(127));
t43=C_a_i_list(&a,6,lf[331],lf[52],C_make_character(0),C_make_character(9),C_make_character(11),C_make_character(127));
t44=C_a_i_list(&a,3,lf[115],C_make_character(13),C_make_character(10));
t45=C_a_i_list(&a,3,lf[52],C_make_character(10),C_make_character(13));
t46=C_a_i_list(&a,4,lf[80],lf[103],t44,t45);
t47=C_a_i_list(&a,3,lf[103],lf[181],C_make_character(95));
t48=C_a_i_list(&a,2,lf[129],t47);
t49=C_a_i_list(&a,5,lf[231],lf[115],lf[183],t48,lf[184]);
t50=C_a_i_list(&a,4,lf[332],lf[52],C_make_character(128),C_make_character(193));
t51=C_a_i_list(&a,3,lf[52],C_make_character(194),C_make_character(223));
t52=C_a_i_list(&a,4,lf[333],lf[115],t51,lf[332]);
t53=C_a_i_list(&a,3,lf[52],C_make_character(224),C_make_character(239));
t54=C_a_i_list(&a,5,lf[334],lf[115],t53,lf[332],lf[332]);
t55=C_a_i_list(&a,3,lf[52],C_make_character(240),C_make_character(247));
t56=C_a_i_list(&a,6,lf[335],lf[115],t55,lf[332],lf[332],lf[332]);
t57=C_a_i_list(&a,6,lf[274],lf[103],lf[330],lf[333],lf[334],lf[335]);
t58=C_a_i_list(&a,6,lf[275],lf[103],lf[331],lf[333],lf[334],lf[335]);
t59=C_a_i_list(&a,3,lf[103],C_make_character(43),C_make_character(45));
t60=C_a_i_list(&a,2,lf[131],t59);
t61=C_a_i_list(&a,2,lf[129],lf[178]);
t62=C_a_i_list(&a,4,lf[336],lf[115],t60,t61);
t63=C_a_i_list(&a,3,lf[103],C_make_character(43),C_make_character(45));
t64=C_a_i_list(&a,2,lf[131],t63);
t65=C_a_i_list(&a,2,lf[129],lf[178]);
t66=C_a_i_list(&a,2,lf[129],lf[178]);
t67=C_a_i_list(&a,3,lf[131],C_make_character(46),t66);
t68=C_a_i_list(&a,3,lf[103],C_make_character(101),C_make_character(69));
t69=C_a_i_list(&a,3,lf[131],t68,lf[336]);
t70=C_a_i_list(&a,6,lf[337],lf[115],t64,t65,t67,t69);
t71=C_a_i_list(&a,1,lf[338]);
t72=C_a_i_list(&a,4,lf[339],lf[103],lf[308],t71);
t73=C_a_i_list(&a,1,lf[340]);
t74=C_a_i_list(&a,5,lf[341],lf[103],lf[339],lf[316],t73);
t75=C_a_i_list(&a,2,lf[127],lf[341]);
t76=C_a_i_list(&a,3,lf[115],lf[339],t75);
t77=C_a_i_list(&a,1,lf[342]);
t78=C_a_i_list(&a,2,lf[127],lf[341]);
t79=C_a_i_list(&a,3,lf[131],lf[339],t78);
t80=C_a_i_list(&a,3,lf[115],t77,t79);
t81=C_a_i_list(&a,2,lf[127],lf[343]);
t82=C_a_i_list(&a,3,lf[115],lf[344],t81);
t83=C_a_i_list(&a,5,lf[345],lf[103],t76,t80,t82);
t84=C_a_i_list(&a,2,lf[127],lf[79]);
t85=C_a_i_list(&a,2,lf[127],lf[125]);
t86=C_a_i_list(&a,5,lf[127],t84,lf[346],t85,lf[80]);
t87=C_a_i_list(&a,2,lf[129],lf[79]);
t88=C_a_i_list(&a,4,lf[347],lf[115],t86,t87);
t89=C_a_i_list(&a,3,lf[348],C_make_character(92),C_make_character(34));
t90=C_a_i_list(&a,5,lf[349],lf[115],C_make_character(34),t89,C_make_character(34));
t91=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30774,a[2]=((C_word)li263),tmp=(C_word)a,a+=3,tmp);
t92=C_a_i_cons(&a,2,lf[348],t91);
t93=C_a_i_list(&a,2,lf[52],lf[350]);
t94=C_a_i_list(&a,2,lf[131],t93);
t95=C_a_i_list(&a,2,lf[131],lf[178]);
t96=C_a_i_list(&a,5,lf[351],lf[115],t94,t95,lf[178]);
t97=C_a_i_list(&a,4,lf[134],C_fix(3),C_make_character(46),lf[351]);
t98=C_a_i_list(&a,4,lf[352],lf[115],lf[351],t97);
t99=C_a_i_list(&a,4,lf[133],C_fix(0),C_fix(4),lf[328]);
t100=C_a_i_list(&a,2,lf[131],C_make_character(58));
t101=C_a_i_list(&a,4,lf[133],C_fix(0),C_fix(4),lf[328]);
t102=C_a_i_list(&a,6,lf[133],C_fix(1),C_fix(7),C_make_character(58),t100,t101);
t103=C_a_i_list(&a,4,lf[353],lf[115],t99,t102);
t104=C_a_i_list(&a,4,lf[354],lf[103],lf[352],lf[353]);
t105=C_a_i_list(&a,4,lf[103],lf[181],C_make_character(95),C_make_character(45));
t106=C_a_i_list(&a,3,lf[355],lf[129],t105);
t107=C_a_i_list(&a,3,lf[129],C_make_character(46),lf[355]);
t108=C_a_i_list(&a,4,lf[356],lf[115],lf[355],t107);
t109=C_a_i_list(&a,3,lf[134],C_fix(2),lf[308]);
t110=C_a_i_list(&a,16,lf[103],lf[357],lf[358],lf[359],lf[360],lf[361],lf[362],lf[363],lf[364],lf[365],lf[366],lf[367],lf[368],lf[369],lf[370],t109);
t111=C_a_i_list(&a,3,lf[371],lf[226],t110);
t112=C_a_i_list(&a,3,lf[129],lf[355],C_make_character(46));
t113=C_a_i_list(&a,4,lf[372],lf[115],t112,lf[371]);
t114=C_a_i_list(&a,6,lf[103],lf[181],C_make_character(95),C_make_character(45),C_make_character(46),C_make_character(43));
t115=C_a_i_list(&a,3,lf[373],lf[129],t114);
t116=C_a_i_list(&a,5,lf[374],lf[115],lf[373],C_make_character(64),lf[356]);
t117=C_a_i_list(&a,4,lf[115],lf[375],lf[328],lf[328]);
t118=C_a_i_list(&a,14,lf[376],lf[103],lf[310],C_make_character(95),C_make_character(45),C_make_character(43),C_make_character(92),C_make_character(61),C_make_character(126),C_make_character(46),C_make_character(44),C_make_character(38),C_make_character(59),t117);
t119=C_a_i_list(&a,4,lf[115],lf[377],lf[328],lf[328]);
t120=C_a_i_list(&a,11,lf[378],lf[103],lf[310],C_make_character(95),C_make_character(45),C_make_character(43),C_make_character(92),C_make_character(61),C_make_character(126),C_make_character(38),t119);
t121=C_a_i_list(&a,2,lf[131],lf[379]);
t122=C_a_i_list(&a,3,lf[103],lf[356],lf[352]);
t123=C_a_i_list(&a,2,lf[129],lf[178]);
t124=C_a_i_list(&a,3,lf[131],lf[380],t123);
t125=C_a_i_list(&a,3,lf[103],lf[376],lf[381]);
t126=C_a_i_list(&a,2,lf[127],t125);
t127=C_a_i_list(&a,2,lf[127],lf[376]);
t128=C_a_i_list(&a,3,lf[131],lf[382],t127);
t129=C_a_i_list(&a,2,lf[127],lf[376]);
t130=C_a_i_list(&a,3,lf[131],t129,lf[378]);
t131=C_a_i_list(&a,3,lf[131],lf[383],t130);
t132=C_a_i_list(&a,5,lf[131],lf[384],t126,t128,t131);
t133=C_a_i_list(&a,8,lf[385],lf[226],lf[386],t121,lf[387],t122,t124,t132);
t134=C_a_i_list(&a,31,t42,t43,t46,t49,t50,t52,t54,t56,t57,t58,t62,t70,t72,t74,t83,t88,t90,t92,t96,t98,t103,t104,t106,t108,t111,t113,t115,t116,t118,t120,t133);
t135=C_a_i_cons(&a,2,t41,t134);
t136=C_a_i_cons(&a,2,t40,t135);
t137=C_a_i_cons(&a,2,t38,t136);
t138=C_a_i_cons(&a,2,t37,t137);
t139=C_a_i_cons(&a,2,t36,t138);
t140=C_a_i_cons(&a,2,t35,t139);
t141=C_a_i_cons(&a,2,t34,t140);
t142=C_a_i_cons(&a,2,t33,t141);
t143=C_a_i_cons(&a,2,t32,t142);
t144=C_a_i_cons(&a,2,t31,t143);
t145=C_a_i_cons(&a,2,t30,t144);
t146=C_a_i_cons(&a,2,t29,t145);
t147=C_a_i_cons(&a,2,t28,t146);
t148=C_a_i_cons(&a,2,t27,t147);
t149=C_a_i_cons(&a,2,t26,t148);
t150=C_a_i_cons(&a,2,t25,t149);
t151=C_a_i_cons(&a,2,t24,t150);
t152=C_a_i_cons(&a,2,t23,t151);
t153=C_a_i_cons(&a,2,t22,t152);
t154=C_a_i_cons(&a,2,t21,t153);
t155=C_a_i_cons(&a,2,t20,t154);
t156=C_a_i_cons(&a,2,t19,t155);
t157=C_a_i_cons(&a,2,t18,t156);
t158=C_a_i_cons(&a,2,t17,t157);
t159=C_a_i_cons(&a,2,t16,t158);
t160=C_a_i_cons(&a,2,t15,t159);
t161=C_a_i_cons(&a,2,t14,t160);
t162=C_a_i_cons(&a,2,t11,t161);
t163=C_mutate(&lf[248] /* (set! chicken.irregex#sre-named-definitions ...) */,t162);
t164=C_mutate(&lf[388] /* (set! chicken.irregex#nfa-get-epsilons ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20582,a[2]=((C_word)li264),tmp=(C_word)a,a+=3,tmp));
t165=C_mutate(&lf[389] /* (set! chicken.irregex#nfa-add-epsilon! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20616,a[2]=((C_word)li265),tmp=(C_word)a,a+=3,tmp));
t166=C_mutate(&lf[268] /* (set! chicken.irregex#sre->nfa ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20757,a[2]=((C_word)li276),tmp=(C_word)a,a+=3,tmp));
t167=C_mutate(&lf[396] /* (set! chicken.irregex#make-mst ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21880,a[2]=((C_word)li277),tmp=(C_word)a,a+=3,tmp));
t168=C_mutate(&lf[397] /* (set! chicken.irregex#mst-copy ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21909,a[2]=((C_word)li278),tmp=(C_word)a,a+=3,tmp));
t169=C_mutate(&lf[398] /* (set! chicken.irregex#nfa-state->mst ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21926,a[2]=((C_word)li279),tmp=(C_word)a,a+=3,tmp));
t170=C_mutate(&lf[399] /* (set! chicken.irregex#mst-add! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21935,a[2]=((C_word)li282),tmp=(C_word)a,a+=3,tmp));
t171=C_mutate(&lf[400] /* (set! chicken.irregex#mst-fold ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22287,a[2]=((C_word)li284),tmp=(C_word)a,a+=3,tmp));
t172=C_mutate(&lf[267] /* (set! chicken.irregex#nfa->dfa ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22369,a[2]=((C_word)li298),tmp=(C_word)a,a+=3,tmp));
t173=C_mutate(&lf[403] /* (set! chicken.irregex#nfa-epsilon-closure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23054,a[2]=((C_word)li305),tmp=(C_word)a,a+=3,tmp));
t174=C_mutate(&lf[402] /* (set! chicken.irregex#tag-set-commands-for-closure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23069,a[2]=((C_word)li310),tmp=(C_word)a,a+=3,tmp));
t175=C_mutate(&lf[223] /* (set! chicken.irregex#sre-cset->procedure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26168,a[2]=((C_word)li312),tmp=(C_word)a,a+=3,tmp));
t176=C_mutate(&lf[216] /* (set! chicken.irregex#cset->plist ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26381,a[2]=((C_word)li314),tmp=(C_word)a,a+=3,tmp));
t177=C_mutate(&lf[391] /* (set! chicken.irregex#string->cset ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26472,a[2]=((C_word)li316),tmp=(C_word)a,a+=3,tmp));
t178=C_mutate(&lf[172] /* (set! chicken.irregex#sre->cset ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26492,a[2]=((C_word)li326),tmp=(C_word)a,a+=3,tmp));
t179=C_mutate(&lf[296] /* (set! chicken.irregex#cset-contains? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26959,a[2]=((C_word)li328),tmp=(C_word)a,a+=3,tmp));
t180=C_mutate(&lf[408] /* (set! chicken.irregex#char-ranges-union ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27044,a[2]=((C_word)li329),tmp=(C_word)a,a+=3,tmp));
t181=C_mutate(&lf[162] /* (set! chicken.irregex#cset-union ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27080,a[2]=((C_word)li331),tmp=(C_word)a,a+=3,tmp));
t182=C_mutate(&lf[404] /* (set! chicken.irregex#cset-difference ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27275,a[2]=((C_word)li333),tmp=(C_word)a,a+=3,tmp));
t183=C_mutate(&lf[405] /* (set! chicken.irregex#cset-intersection ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27471,a[2]=((C_word)li335),tmp=(C_word)a,a+=3,tmp));
t184=C_mutate(&lf[164] /* (set! chicken.irregex#cset-complement ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27597,a[2]=((C_word)li336),tmp=(C_word)a,a+=3,tmp));
t185=C_mutate(&lf[165] /* (set! chicken.irregex#cset-case-insensitive ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27607,a[2]=((C_word)li338),tmp=(C_word)a,a+=3,tmp));
t186=C_mutate(&lf[410] /* (set! chicken.irregex#irregex-fold/fast ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27738,a[2]=((C_word)li341),tmp=(C_word)a,a+=3,tmp));
t187=C_mutate((C_word*)lf[415]+1 /* (set! chicken.irregex#irregex-fold ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27956,a[2]=((C_word)li343),tmp=(C_word)a,a+=3,tmp));
t188=C_mutate(&lf[417] /* (set! chicken.irregex#irregex-fold/chunked/fast ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27984,a[2]=((C_word)li346),tmp=(C_word)a,a+=3,tmp));
t189=C_mutate((C_word*)lf[420]+1 /* (set! chicken.irregex#irregex-fold/chunked ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28166,a[2]=((C_word)li348),tmp=(C_word)a,a+=3,tmp));
t190=C_mutate((C_word*)lf[422]+1 /* (set! chicken.irregex#irregex-replace ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28194,a[2]=((C_word)li349),tmp=(C_word)a,a+=3,tmp));
t191=C_mutate((C_word*)lf[426]+1 /* (set! chicken.irregex#irregex-replace/all ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28273,a[2]=((C_word)li352),tmp=(C_word)a,a+=3,tmp));
t192=C_mutate(&lf[423] /* (set! chicken.irregex#irregex-apply-match ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28366,a[2]=((C_word)li355),tmp=(C_word)a,a+=3,tmp));
t193=C_mutate((C_word*)lf[433]+1 /* (set! chicken.irregex#irregex-extract ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28597,a[2]=((C_word)li358),tmp=(C_word)a,a+=3,tmp));
t194=C_mutate((C_word*)lf[436]+1 /* (set! chicken.irregex#irregex-split ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28632,a[2]=((C_word)li362),tmp=(C_word)a,a+=3,tmp));
t195=C_mutate(&lf[54] /* (set! chicken.irregex#string-scan-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28830,a[2]=((C_word)li364),tmp=(C_word)a,a+=3,tmp));
t196=C_mutate((C_word*)lf[440]+1 /* (set! chicken.irregex#irregex-quote ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28873,a[2]=((C_word)li366),tmp=(C_word)a,a+=3,tmp));
t197=C_mutate((C_word*)lf[443]+1 /* (set! chicken.irregex#irregex-opt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28930,a[2]=((C_word)li371),tmp=(C_word)a,a+=3,tmp));
t198=C_mutate(&lf[447] /* (set! chicken.irregex#cset->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29197,a[2]=((C_word)li373),tmp=(C_word)a,a+=3,tmp));
t199=C_mutate((C_word*)lf[452]+1 /* (set! chicken.irregex#sre->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29277,a[2]=((C_word)li383),tmp=(C_word)a,a+=3,tmp));
t200=C_mutate((C_word*)lf[486]+1 /* (set! chicken.irregex#glob->sre ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30033,a[2]=((C_word)li386),tmp=(C_word)a,a+=3,tmp));
t201=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t201;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t201+1)))(2,av2);}}

/* chicken.irregex#irregex-search in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_18920,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18924,a[2]=t4,a[3]=t3,a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:1966: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[288];
av2[3]=lf[291];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_18924(2,av2);}}}

/* k18922 in chicken.irregex#irregex-search in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18924,c,av);}
a=C_alloc(6);
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_u_i_car(((C_word*)t0)[2]):C_SCHEME_FALSE);
t4=(C_truep(t3)?t3:C_fix(0));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18933,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t6=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t7=t5;
f_18933(t7,(C_truep(t6)?C_i_cadr(((C_word*)t0)[2]):C_SCHEME_FALSE));}
else{
t6=t5;
f_18933(t6,C_SCHEME_FALSE);}}

/* k18931 in k18922 in chicken.irregex#irregex-search in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_18933(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_18933,2,t0,t1);}
a=C_alloc(7);
t2=(C_truep(t1)?t1:C_i_string_length(((C_word*)t0)[2]));
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18939,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[3]))){
t4=C_i_exactp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t4))){
/* irregex-core.scm:1970: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[288];
av2[3]=lf[290];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=t3;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_18939(2,av2);}}}
else{
/* irregex-core.scm:1970: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[288];
av2[3]=lf[290];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k18937 in k18931 in k18922 in chicken.irregex#irregex-search in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_18939,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18942,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[4]))){
t3=C_i_exactp(((C_word*)t0)[4]);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:1972: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[288];
av2[3]=lf[289];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_18942(2,av2);}}}
else{
/* irregex-core.scm:1972: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[288];
av2[3]=lf[289];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k18940 in k18937 in k18931 in k18922 in chicken.irregex#irregex-search in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_18942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_18942,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
/* irregex-core.scm:1973: irregex-search/chunked */
t3=*((C_word*)lf[287]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[285];
av2[4]=t2;
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* chicken.irregex#irregex-search/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19010(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c<5) C_bad_min_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-5)*C_SIZEOF_PAIR +6,c,2)))){
C_save_and_reclaim((void*)f_19010,c,av);}
a=C_alloc((c-5)*C_SIZEOF_PAIR+6);
t5=C_build_rest(&a,c,5,av);
C_word t6;
C_word t7;
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19014,a[2]=t3,a[3]=t4,a[4]=t1,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1979: irregex */
t7=*((C_word*)lf[217]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k19012 in chicken.irregex#irregex-search/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19014(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_19014,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19017,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:1980: irregex-new-matches */
f_7622(t2,t1);}

/* k19015 in k19012 in chicken.irregex#irregex-search/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_19017,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19020,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_car(((C_word*)t0)[6]);
f_19020(2,av2);}}
else{
/* irregex-core.scm:1981: g3263 */
t3=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k19018 in k19015 in k19012 in chicken.irregex#irregex-search/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19020(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_19020,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19023,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_not(C_i_integerp(t1)))){
/* irregex-core.scm:1982: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[288];
av2[3]=lf[293];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_19023(2,av2);}}}

/* k19021 in k19018 in k19015 in k19012 in chicken.irregex#irregex-search/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,7)))){
C_save_and_reclaim((void *)f_19023,c,av);}
a=C_alloc(3);
t2=C_i_setslot(((C_word*)t0)[2],C_fix(3),((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* irregex-core.scm:1984: irregex-search/matches */
f_19053(((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[3],t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[2]);}

/* chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19053(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,9)))){
C_save_and_reclaim_args((void *)trf_19053,7,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(20);
t8=C_i_check_structure_2(t2,lf[6],lf[294]);
if(C_truep(C_slot(t2,C_fix(1)))){
t9=C_slot(t2,C_fix(4));
t10=C_fixnum_and(t9,C_fix(1));
t11=C_eqp(C_fix(1),t10);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19072,a[2]=t7,a[3]=t5,a[4]=t6,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:1994: dfa-match/longest */
f_19935(t12,C_slot(t2,C_fix(1)),t3,t5,t6,C_SCHEME_FALSE,C_SCHEME_FALSE,t7,C_fix(0));}
else{
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19097,a[2]=t2,a[3]=t3,a[4]=t7,a[5]=t1,a[6]=t5,a[7]=t6,tmp=(C_word)a,a+=8,tmp);
t13=C_slot(t2,C_fix(2));
t14=C_i_vector_ref(t3,C_fix(1));
t15=C_i_vector_ref(t3,C_fix(2));
t16=C_i_vector_ref(t3,C_fix(3));
t17=C_i_vector_ref(t3,C_fix(0));
t18=C_i_vector_ref(t13,C_fix(0));
t19=C_i_cadr(t18);
t20=C_i_cadr(t19);
t21=C_i_vector_ref(t13,t20);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19678,a[2]=t7,a[3]=t13,a[4]=t23,a[5]=t15,a[6]=t17,a[7]=t16,a[8]=t14,a[9]=((C_word)li237),tmp=(C_word)a,a+=10,tmp));
t25=((C_word*)t23)[1];
f_19678(t25,t12,t5,t6,t21);}}
else{
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19184,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t10=C_i_check_structure_2(t2,lf[6],lf[297]);
t11=C_slot(t2,C_fix(3));
t12=C_i_vector_ref(t3,C_fix(1));
t13=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19202,a[2]=t3,a[3]=t2,a[4]=t11,a[5]=t9,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:2027: g3331 */
t14=t12;{
C_word av2[3];
av2[0]=t14;
av2[1]=t13;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t14+1)))(3,av2);}}}

/* k19070 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19072,c,av);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_vector_set(t2,C_fix(0),((C_word*)t0)[3]);
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_i_vector_set(t4,C_fix(1),((C_word*)t0)[4]);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_19097,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_vector_ref(((C_word*)t0)[3],C_fix(2));
t4=C_i_vector_ref(((C_word*)t0)[3],C_fix(3));
t5=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19114,a[2]=((C_word*)t0)[4],a[3]=t7,a[4]=t3,a[5]=t5,a[6]=t2,a[7]=((C_word*)t0)[3],a[8]=t4,a[9]=((C_word)li234),tmp=(C_word)a,a+=10,tmp));
t9=((C_word*)t7)[1];
f_19114(t9,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19114(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_19114,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19118,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=t1,a[10]=t3,tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2007: get-end */
t5=((C_word*)t0)[8];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k19116 in lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_19118,c,av);}
a=C_alloc(14);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19123,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t3,a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word)li233),tmp=(C_word)a,a+=12,tmp));
t5=((C_word*)t3)[1];
f_19123(t5,((C_word*)t0)[9],((C_word*)t0)[10]);}

/* lp2 in k19116 in lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19123(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,9)))){
C_save_and_reclaim_args((void *)trf_19123,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19130,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2010: dfa-match/longest */
f_19935(t3,((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[3],t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[2],C_fix(0));}

/* k19128 in lp2 in k19116 in lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19130,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_vector_set(t2,C_fix(0),((C_word*)t0)[3]);
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_i_vector_set(t4,C_fix(1),((C_word*)t0)[4]);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[4],((C_word*)t0)[6]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19153,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2015: get-next */
t3=((C_word*)t0)[9];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
/* irregex-core.scm:2018: lp2 */
t2=((C_word*)((C_word*)t0)[10])[1];
f_19123(t2,((C_word*)t0)[5],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}}}

/* k19151 in k19128 in lp2 in k19116 in lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19153,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19163,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2016: get-start */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19161 in k19151 in k19128 in lp2 in k19116 in lp1 in k19095 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19163(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19163,c,av);}
/* irregex-core.scm:2016: lp1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_19114(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k19182 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19184,c,av);}
if(C_truep(t1)){
t2=C_i_set_i_slot(t1,C_fix(4),C_SCHEME_FALSE);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19202(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_19202,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19208,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2027: g3333 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19208(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,9)))){
C_save_and_reclaim((void *)f_19208,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t3=C_slot(((C_word*)t0)[3],C_fix(4));
t4=C_fixnum_and(t3,C_fix(1));
t5=C_eqp(C_fix(1),t4);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19222,a[2]=((C_word)li238),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:2032: matcher */
t7=((C_word*)t0)[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=((C_word*)t0)[9];
av2[7]=t1;
av2[8]=((C_word*)t0)[10];
av2[9]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(10,av2);}}
else{
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19228,a[2]=((C_word*)t0)[10],a[3]=t7,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[6],a[8]=((C_word)li240),tmp=(C_word)a,a+=9,tmp));
t9=((C_word*)t7)[1];
f_19228(t9,((C_word*)t0)[5],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],t1);}}

/* a19221 in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19222(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19222,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19228(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,9)))){
C_save_and_reclaim_args((void *)trf_19228,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(14);
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19235,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t4,a[5]=t1,a[6]=t5,a[7]=((C_word*)t0)[3],a[8]=t3,a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[5],tmp=(C_word)a,a+=11,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19294,a[2]=((C_word)li239),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:2038: matcher */
t8=((C_word*)t0)[6];{
C_word av2[10];
av2[0]=t8;
av2[1]=t6;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[7];
av2[4]=t2;
av2[5]=t3;
av2[6]=t4;
av2[7]=t5;
av2[8]=((C_word*)t0)[2];
av2[9]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(10,av2);}}

/* k19233 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_19235,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_vector_set(t2,C_fix(0),((C_word*)t0)[3]);
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_i_vector_set(t4,C_fix(1),((C_word*)t0)[4]);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(((C_word*)t0)[4],((C_word*)t0)[6]))){
/* irregex-core.scm:2043: lp */
t2=((C_word*)((C_word*)t0)[7])[1];
f_19228(t2,((C_word*)t0)[5],((C_word*)t0)[3],((C_word*)t0)[8],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),((C_word*)t0)[6]);}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19265,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2045: get-next */
t3=((C_word*)t0)[10];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k19263 in k19233 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19265(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19265,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19278,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2047: g3352 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19276 in k19263 in k19233 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_19278,c,av);}
a=C_alloc(7);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19285,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2047: g3354 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19283 in k19276 in k19263 in k19233 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_19285,c,av);}
a=C_alloc(7);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19292,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2047: g3356 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19290 in k19283 in k19276 in k19263 in k19233 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19292,c,av);}
/* irregex-core.scm:2047: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_19228(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1);}

/* a19293 in lp in k19206 in k19200 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19294,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#irregex-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_19302,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19306,a[2]=t4,a[3]=t3,a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:2054: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[300];
av2[3]=lf[303];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_19306(2,av2);}}}

/* k19304 in chicken.irregex#irregex-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19306,c,av);}
a=C_alloc(6);
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_u_i_car(((C_word*)t0)[2]):C_SCHEME_FALSE);
t4=(C_truep(t3)?t3:C_fix(0));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19315,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t6=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t7=t5;
f_19315(t7,(C_truep(t6)?C_i_cadr(((C_word*)t0)[2]):C_SCHEME_FALSE));}
else{
t6=t5;
f_19315(t6,C_SCHEME_FALSE);}}

/* k19313 in k19304 in chicken.irregex#irregex-match in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19315(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_19315,2,t0,t1);}
a=C_alloc(7);
t2=(C_truep(t1)?t1:C_i_string_length(((C_word*)t0)[2]));
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19321,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[3]))){
t4=C_i_exactp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t4))){
/* irregex-core.scm:2058: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[300];
av2[3]=lf[302];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=t3;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_19321(2,av2);}}}
else{
/* irregex-core.scm:2058: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[300];
av2[3]=lf[302];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k19319 in k19313 in k19304 in chicken.irregex#irregex-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_19321,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19324,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[4]))){
t3=C_i_exactp(((C_word*)t0)[4]);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:2060: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[300];
av2[3]=lf[301];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
/* irregex-core.scm:2061: irregex-match/chunked */
t5=*((C_word*)lf[299]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[285];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}
else{
/* irregex-core.scm:2060: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[300];
av2[3]=lf[301];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k19322 in k19319 in k19313 in k19304 in chicken.irregex#irregex-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_19324,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
/* irregex-core.scm:2061: irregex-match/chunked */
t3=*((C_word*)lf[299]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[285];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19392,c,av);}
a=C_alloc(5);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19396,a[2]=t3,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2066: irregex */
t6=*((C_word*)lf[217]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19396,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19399,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2067: irregex-new-matches */
f_7622(t2,t1);}

/* k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_19399,c,av);}
a=C_alloc(13);
t2=C_i_setslot(t1,C_fix(3),((C_word*)t0)[2]);
if(C_truep(C_slot(((C_word*)t0)[3],C_fix(1)))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19414,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_slot(((C_word*)t0)[3],C_fix(1));
t5=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19484,a[2]=t3,a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2072: g3390 */
t7=t5;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t3=C_slot(((C_word*)t0)[3],C_fix(3));
t4=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19494,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t3,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2083: g3410 */
t6=t4;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k19412 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19414,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19448,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_slot(((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:2074: g3393 */
t5=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_slot(t4,C_fix(2));
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19435 in k19446 in k19412 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19437,c,av);}
t2=C_i_vector_set(((C_word*)t0)[2],C_fix(1),t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k19446 in k19412 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19448,c,av);}
a=C_alloc(5);
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(C_fix(0),C_fix(4));
t4=C_slot(t2,C_fixnum_plus(C_fix(3),t3));
t5=C_eqp(t1,t4);
if(C_truep(t5)){
t6=C_slot(((C_word*)t0)[2],C_fix(1));
t7=C_i_vector_set(t6,C_fix(0),((C_word*)t0)[3]);
t8=C_slot(((C_word*)t0)[2],C_fix(1));
t9=C_i_vector_ref(((C_word*)t0)[4],C_fix(2));
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19437,a[2]=t8,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2078: g3404 */
t11=t9;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k19482 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_19484,c,av);}
/* irregex-core.scm:2072: dfa-match/longest */
f_19935(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[6],C_fix(0));}

/* k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_19494,c,av);}
a=C_alloc(8);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19500,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2083: g3413 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_19500,c,av);}
a=C_alloc(9);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19506,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:2083: g3416 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,9)))){
C_save_and_reclaim((void *)f_19506,c,av);}
a=C_alloc(10);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19516,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19615,a[2]=((C_word)li244),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:2088: matcher */
t5=((C_word*)t0)[6];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=((C_word*)t0)[2];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[3];
av2[7]=t1;
av2[8]=((C_word*)t0)[8];
av2[9]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(10,av2);}}

/* k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19516(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_19516,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19518,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word)li243),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_19518(t5,((C_word*)t0)[3],t1);}

/* lp in k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19518(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_19518,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19528,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t4=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19592,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t6=C_slot(t2,C_fix(1));
/* irregex-core.scm:2091: g3427 */
t7=t4;{
C_word av2[3];
av2[0]=t7;
av2[1]=t5;
av2[2]=C_slot(t6,C_fix(2));
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19526 in lp in k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19528(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_19528,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_i_set_i_slot(((C_word*)t0)[2],C_fix(4),C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_slot(((C_word*)t0)[2],C_fix(4)))){
t2=C_slot(((C_word*)t0)[2],C_fix(4));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19547,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2099: g3440 */
t4=t2;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k19545 in k19526 in lp in k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19547,c,av);}
/* irregex-core.scm:2099: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_19518(t2,((C_word*)t0)[3],t1);}

/* k19558 in k19590 in lp in k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19560,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(C_fix(0),C_fix(4));
t4=((C_word*)t0)[3];
f_19528(t4,C_eqp(t1,C_slot(t2,C_fixnum_plus(C_fix(3),t3))));}

/* k19590 in lp in k19514 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_19592,c,av);}
a=C_alloc(4);
if(C_truep(C_i_not(t1))){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19560,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=C_slot(((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:2093: g3433 */
t5=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_slot(t4,C_fix(2));
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t2=((C_word*)t0)[4];
f_19528(t2,C_SCHEME_FALSE);}}

/* a19614 in k19504 in k19498 in k19492 in k19397 in k19394 in chicken.irregex#irregex-match/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19615,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#irregex-match? in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19620(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +3,c,3)))){
C_save_and_reclaim((void*)f_19620,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+3);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19627,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t3;
av2[2]=*((C_word*)lf[298]+1);
av2[3]=t2;
C_apply(4,av2);}}

/* k19625 in chicken.irregex#irregex-match? in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19627,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19678(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_19678,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
if(C_truep(t2)){
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19685,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t1,a[9]=t3,a[10]=t4,a[11]=((C_word*)t0)[7],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:2130: get-str */
t6=((C_word*)t0)[8];{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_19685,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19688,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:2131: get-end */
t3=((C_word*)t0)[11];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_19688,c,av);}
a=C_alloc(14);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19693,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word)li236),tmp=(C_word)a,a+=12,tmp));
t5=((C_word*)t3)[1];
f_19693(t5,((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11]);}

/* lp2 in k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19693(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_19693,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_car(t3))){
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_i_vector_set(t4,C_fix(2),((C_word*)t0)[3]);
t6=C_slot(((C_word*)t0)[2],C_fix(1));
t7=C_i_vector_set(t6,C_fix(3),t2);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(t2,((C_word*)t0)[4]))){
t4=C_i_string_ref(((C_word*)t0)[5],t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19745,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=t1,a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19760,a[2]=t4,a[3]=((C_word)li235),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2142: find */
f_9195(t5,t6,C_i_cdr(t3));}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19796,a[2]=((C_word*)t0)[8],a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2149: get-next */
t5=((C_word*)t0)[10];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}

/* k19743 in lp2 in k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_19745,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t3=C_i_cadr(t1);
/* irregex-core.scm:2147: lp2 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_19693(t4,((C_word*)t0)[4],t2,C_i_vector_ref(((C_word*)t0)[5],t3));}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a19759 in lp2 in k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19760(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19760,c,av);}
t3=C_i_car(t2);
t4=C_eqp(((C_word*)t0)[2],t3);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_charp(C_u_i_car(t2));
if(C_truep(C_i_not(t5))){
/* irregex-core.scm:2145: cset-contains? */
f_26959(t1,C_u_i_car(t2),((C_word*)t0)[2]);}
else{
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* k19794 in lp2 in k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19796,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2150: get-start */
t3=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19804 in k19794 in lp2 in k19686 in k19683 in lp1 in chicken.irregex#irregex-search/matches in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19806,c,av);}
/* irregex-core.scm:2150: lp1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_19678(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5]);}

/* chicken.irregex#finalize! in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19816(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_19816,4,t1,t2,t3,t4);}
a=C_alloc(12);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19818,a[2]=t3,a[3]=t4,a[4]=((C_word)li247),tmp=(C_word)a,a+=5,tmp);
t6=C_i_check_list_2(t2,lf[306]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19876,a[2]=t5,a[3]=t8,a[4]=((C_word)li248),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_19876(t10,t1,t2);}

/* g3514 in chicken.irregex#finalize! in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_19818(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_stack_overflow_check;{}
t2=C_i_car(t1);
t3=C_i_vector_ref(((C_word*)t0)[2],C_u_i_cdr(t1));
t4=C_i_vector_ref(t3,t2);
t5=C_slot(((C_word*)t0)[3],C_fix(1));
t6=C_a_i_fixnum_times(&a,2,t2,C_fix(2));
t7=C_s_a_u_i_integer_plus(&a,2,C_fix(4),t6);
t8=(C_truep(t4)?C_i_vector_set(t5,t7,C_i_car(t4)):C_i_vector_set(t5,t7,C_SCHEME_FALSE));
t9=C_slot(((C_word*)t0)[3],C_fix(1));
t10=C_a_i_fixnum_times(&a,2,t2,C_fix(2));
t11=C_s_a_u_i_integer_plus(&a,2,C_fix(5),t10);
return((C_truep(t4)?C_i_vector_set(t9,t11,C_i_cdr(t4)):C_i_vector_set(t9,t11,C_SCHEME_FALSE)));}

/* for-each-loop3513 in chicken.irregex#finalize! in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19876(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,2)))){
C_save_and_reclaim_args((void *)trf_19876,3,t0,t1,t2);}
a=C_alloc(20);
if(C_truep(C_i_pairp(t2))){
t3=(
/* irregex-core.scm:2153: g3514 */
  f_19818(C_a_i(&a,20),((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t5=t1;
t6=C_slot(t2,C_fix(1));
t1=t5;
t2=t6;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k19902 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_19904,c,av);}
a=C_alloc(10);
t2=C_a_i_fixnum_times(&a,2,t1,C_fix(2));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19908,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2165: scheme#make-vector */
t4=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19906 in k19902 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_19908,c,av);}
a=C_alloc(9);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19913,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word)li261),tmp=(C_word)a,a+=7,tmp));
t5=((C_word*)t3)[1];
f_19913(t5,((C_word*)t0)[4],C_fix(0));}

/* doloop3544 in k19906 in k19902 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19913(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_19913,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19933,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2168: scheme#make-vector */
t5=*((C_word*)lf[4]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k19931 in doloop3544 in k19906 in k19902 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19933,c,av);}
a=C_alloc(5);
t2=C_i_vector_set(((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=((C_word*)((C_word*)t0)[4])[1];
f_19913(t4,((C_word*)t0)[5],t3);}

/* chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19935(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8,C_word t9){
C_word tmp;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,2)))){
C_save_and_reclaim_args((void *)trf_19935,9,t1,t2,t3,t4,t5,t6,t7,t8,t9);}
a=C_alloc(20);
t10=C_i_vector_ref(t3,C_fix(1));
t11=C_i_vector_ref(t3,C_fix(2));
t12=C_i_vector_ref(t3,C_fix(3));
t13=C_i_vector_ref(t3,C_fix(0));
t14=C_i_vector_ref(t2,C_fix(0));
t15=C_i_car(t14);
t16=C_i_not(C_i_zerop(t15));
t17=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_19960,a[2]=t14,a[3]=t2,a[4]=t4,a[5]=t5,a[6]=t11,a[7]=t9,a[8]=t8,a[9]=t6,a[10]=t13,a[11]=t16,a[12]=t7,a[13]=t12,a[14]=t10,a[15]=t1,tmp=(C_word)a,a+=16,tmp);
t18=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19904,a[2]=t15,a[3]=t17,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2164: irregex-match-num-submatches */
t19=*((C_word*)lf[17]+1);{
C_word av2[3];
av2[0]=t19;
av2[1]=t18;
av2[2]=t8;
((C_proc)(void*)(*((C_word*)t19+1)))(3,av2);}}

/* k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_19960,c,av);}
a=C_alloc(19);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_cadr(t2);
t4=C_i_vector_ref(((C_word*)t0)[3],t3);
t5=C_i_car(t4);
t6=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_19972,a[2]=t5,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t1,a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=t4,a[18]=t2,tmp=(C_word)a,a+=19,tmp);
if(C_truep(((C_word*)t0)[7])){
t7=C_slot(((C_word*)t0)[8],C_fix(1));
t8=C_fixnum_times(((C_word*)t0)[7],C_fix(4));
t9=C_fixnum_plus(C_fix(2),t8);
t10=C_i_vector_set(t7,t9,C_SCHEME_FALSE);
t11=C_slot(((C_word*)t0)[8],C_fix(1));
t12=C_fixnum_times(((C_word*)t0)[7],C_fix(4));
t13=t6;
f_19972(t13,C_i_vector_set(t11,C_fixnum_plus(C_fix(3),t12),C_SCHEME_FALSE));}
else{
t7=t6;
f_19972(t7,C_SCHEME_UNDEFINED);}}

/* k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19972(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(31,0,3)))){
C_save_and_reclaim_args((void *)trf_19972,2,t0,t1);}
a=C_alloc(31);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_19975,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
if(C_truep(((C_word*)t0)[12])){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20429,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word)li259),tmp=(C_word)a,a+=6,tmp);
t4=C_i_cddr(((C_word*)t0)[18]);
t5=C_i_cdr(t4);
t6=C_i_check_list_2(t5,lf[306]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20457,a[2]=t3,a[3]=t8,a[4]=((C_word)li260),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_20457(t10,t2,t5);}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_19975(2,av2);}}}

/* k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,8)))){
C_save_and_reclaim((void *)f_19975,c,av);}
a=C_alloc(17);
t2=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[3]:C_SCHEME_FALSE);
t3=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[4]:C_SCHEME_FALSE);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_19988,a[2]=t5,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=((C_word)li258),tmp=(C_word)a,a+=15,tmp));
t7=((C_word*)t5)[1];
f_19988(t7,((C_word*)t0)[16],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[17],t2,t3,((C_word*)t0)[2]);}

/* lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_19988(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,2)))){
C_save_and_reclaim_args((void *)trf_19988,8,t0,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(20);
t8=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_19992,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t2,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=t1,a[13]=t3,a[14]=t4,a[15]=t5,a[16]=t6,a[17]=t7,a[18]=((C_word*)t0)[11],a[19]=((C_word*)t0)[12],tmp=(C_word)a,a+=20,tmp);
/* irregex-core.scm:2199: get-str */
t9=((C_word*)t0)[13];{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_19992,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_19995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
t3=C_eqp(((C_word*)t0)[7],((C_word*)t0)[8]);
if(C_truep(t3)){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[18];
f_19995(2,av2);}}
else{
/* irregex-core.scm:2200: get-end */
t4=((C_word*)t0)[19];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_19995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,7)))){
C_save_and_reclaim((void *)f_19995,c,av);}
a=C_alloc(18);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_20000,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=t3,a[14]=((C_word*)t0)[12],a[15]=((C_word)li257),tmp=(C_word)a,a+=16,tmp));
t5=((C_word*)t3)[1];
f_20000(t5,((C_word*)t0)[13],((C_word*)t0)[14],((C_word*)t0)[15],((C_word*)t0)[16],((C_word*)t0)[17],((C_word*)t0)[18]);}

/* lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20000(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_20000,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(18);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t7=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_20010,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],a[12]=((C_word*)t0)[8],a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[10],tmp=(C_word)a,a+=15,tmp);
t8=(C_truep(((C_word*)t0)[5])?t4:C_SCHEME_FALSE);
if(C_truep(t8)){
t9=C_slot(((C_word*)t0)[6],C_fix(1));
t10=C_fixnum_times(((C_word*)t0)[5],C_fix(4));
t11=C_fixnum_plus(C_fix(2),t10);
t12=C_i_vector_set(t9,t11,t4);
t13=C_slot(((C_word*)t0)[6],C_fix(1));
t14=C_fixnum_times(((C_word*)t0)[5],C_fix(4));
t15=t7;
f_20010(t15,C_i_vector_set(t13,C_fixnum_plus(C_fix(3),t14),t5));}
else{
t9=t7;
f_20010(t9,C_SCHEME_UNDEFINED);}}
else{
t7=C_i_string_ref(((C_word*)t0)[11],t2);
t8=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_20110,a[2]=((C_word*)t0)[12],a[3]=t2,a[4]=((C_word*)t0)[13],a[5]=t1,a[6]=((C_word*)t0)[8],a[7]=t4,a[8]=t5,a[9]=((C_word*)t0)[14],a[10]=((C_word*)t0)[7],a[11]=t6,a[12]=((C_word*)t0)[6],a[13]=((C_word*)t0)[5],tmp=(C_word)a,a+=14,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20389,a[2]=t7,a[3]=((C_word)li256),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2221: find */
f_9195(t8,t9,C_i_cdr(t3));}}

/* k20008 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20010(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_20010,2,t0,t1);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_20013,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
t3=C_eqp(((C_word*)t0)[12],((C_word*)t0)[13]);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:2212: get-next */
t4=((C_word*)t0)[14];{
C_word av2[3];
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t2;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_20013(2,av2);}}}

/* k20011 in k20008 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_20013,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20023,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:2214: get-start */
t3=((C_word*)t0)[8];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(((C_word*)t0)[9])){
t2=C_slot(((C_word*)t0)[10],C_fix(1));
t3=C_fixnum_times(((C_word*)t0)[9],C_fix(4));
t4=C_fixnum_plus(C_fix(2),t3);
if(C_truep(C_slot(t2,t4))){
t5=C_i_not(((C_word*)t0)[7]);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20058,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t5)){
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t5)?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* irregex-core.scm:2217: finalize! */
f_19816(t6,((C_word*)t0)[7],((C_word*)t0)[11],((C_word*)t0)[10]);}}
else{
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k20021 in k20011 in k20008 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_20023,c,av);}
/* irregex-core.scm:2214: lp1 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_19988(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k20056 in k20011 in k20008 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20058,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20110(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_20110,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_i_cadr(t1);
t3=C_i_vector_ref(((C_word*)t0)[2],t2);
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20122,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t3,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[9])){
t6=C_i_cddr(t1);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20146,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=t6,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[11])){
if(C_truep(C_i_not(t4))){
/* irregex-core.scm:2236: finalize! */
f_19816(t7,((C_word*)t0)[11],((C_word*)t0)[10],((C_word*)t0)[12]);}
else{
t8=t7;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_20146(2,av2);}}}
else{
t8=t7;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_20146(2,av2);}}}
else{
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_20122(2,av2);}}}
else{
if(C_truep(((C_word*)t0)[7])){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20319,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[12],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[13])){
t3=C_slot(((C_word*)t0)[12],C_fix(1));
t4=C_fixnum_times(((C_word*)t0)[13],C_fix(4));
t5=C_fixnum_plus(C_fix(2),t4);
t6=C_i_vector_set(t3,t5,((C_word*)t0)[7]);
t7=C_slot(((C_word*)t0)[12],C_fix(1));
t8=C_fixnum_times(((C_word*)t0)[13],C_fix(4));
t9=t2;
f_20319(t9,C_i_vector_set(t7,C_fixnum_plus(C_fix(3),t8),((C_word*)t0)[8]));}
else{
t3=t2;
f_20319(t3,C_SCHEME_UNDEFINED);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20361,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[12],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[13])){
t3=C_slot(((C_word*)t0)[12],C_fix(1));
t4=C_fixnum_times(((C_word*)t0)[13],C_fix(4));
t5=t2;
f_20361(t5,C_slot(t3,C_fixnum_plus(C_fix(2),t4)));}
else{
t3=t2;
f_20361(t3,C_SCHEME_FALSE);}}}}

/* k20120 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,6)))){
C_save_and_reclaim((void *)f_20122,c,av);}
a=C_alloc(10);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:2255: lp2 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_20000(t4,((C_word*)t0)[5],t2,((C_word*)t0)[6],((C_word*)t0)[7],t3,((C_word*)t0)[2]);}
else{
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:2256: lp2 */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20000(t3,((C_word*)t0)[5],t2,((C_word*)t0)[6],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}}

/* k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20146(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_20146,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20147,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word)li250),tmp=(C_word)a,a+=6,tmp);
t3=C_i_cdr(((C_word*)t0)[5]);
t4=C_i_check_list_2(t3,lf[306]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20176,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20283,a[2]=t2,a[3]=t7,a[4]=((C_word)li255),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_20283(t9,t5,t3);}

/* g3669 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_20147(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;{}
t2=C_i_cdr(t1);
t3=C_i_vector_ref(((C_word*)t0)[2],t2);
t4=C_fixnum_plus(((C_word*)t0)[3],C_fix(1));
t5=C_a_i_cons(&a,2,((C_word*)t0)[4],t4);
return(C_i_vector_set(t3,C_u_i_car(t1),t5));}

/* k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_20176,c,av);}
a=C_alloc(21);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20185,a[2]=((C_word*)t0)[2],a[3]=((C_word)li252),tmp=(C_word)a,a+=4,tmp);
t7=C_u_i_car(((C_word*)t0)[3]);
t8=C_i_check_list_2(t7,lf[211]);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20218,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20249,a[2]=t4,a[3]=t11,a[4]=t6,a[5]=t5,a[6]=((C_word)li254),tmp=(C_word)a,a+=7,tmp));
t13=((C_word*)t11)[1];
f_20249(t13,t9,t7);}

/* g3708 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20185(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20185,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_i_vector_ref(t2,C_fix(0));
t4=C_i_vector_ref(t2,C_fix(1));
t5=C_i_vector_ref(((C_word*)t0)[2],t4);
t6=C_i_vector_ref(t2,C_fix(2));
t7=C_i_vector_ref(((C_word*)t0)[2],t6);
t8=C_i_vector_ref(t5,t3);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20199,a[2]=t7,a[3]=t3,a[4]=t8,a[5]=((C_word)li251),tmp=(C_word)a,a+=6,tmp);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* f_20199 in g3708 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20199,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_vector_set(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20216 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20218(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_20218,c,av);}
a=C_alloc(6);
t2=C_i_check_list_2(t1,lf[306]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20226,a[2]=t4,a[3]=((C_word)li253),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_20226(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3678 in k20216 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20226(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20226,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20236,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2246: execute! */
t4=C_slot(t2,C_fix(0));{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k20234 in for-each-loop3678 in k20216 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20236,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_20226(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* map-loop3702 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20249(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20249,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20274,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2247: g3708 */
t4=((C_word*)t0)[4];
f_20185(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20272 in map-loop3702 in k20174 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20274,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20249(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* for-each-loop3668 in k20144 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20283(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_20283,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=(
/* irregex-core.scm:2237: g3669 */
  f_20147(C_a_i(&a,3),((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t5=t1;
t6=C_slot(t2,C_fix(1));
t1=t5;
t2=t6;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k20317 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20319(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_20319,2,t0,t1);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20322,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
/* irregex-core.scm:2262: finalize! */
f_19816(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20320 in k20317 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20322,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k20359 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20361(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_20361,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20364,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[3])){
/* irregex-core.scm:2265: finalize! */
f_19816(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20362 in k20359 in k20108 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20364(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20364,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a20388 in lp2 in k19993 in k19990 in lp1 in k19973 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20389,c,av);}
t3=C_i_car(t2);
t4=C_eqp(((C_word*)t0)[2],t3);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_charp(C_u_i_car(t2));
if(C_truep(C_i_not(t5))){
/* irregex-core.scm:2224: cset-contains? */
f_26959(t1,C_u_i_car(t2),((C_word*)t0)[2]);}
else{
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* g3585 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_20429(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;{}
t2=C_i_cdr(t1);
t3=C_i_vector_ref(((C_word*)t0)[2],t2);
t4=C_u_i_car(t1);
t5=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
return(C_i_vector_set(t3,t4,t5));}

/* for-each-loop3584 in k19970 in k19958 in chicken.irregex#dfa-match/longest in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20457(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_20457,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=(
/* irregex-core.scm:2189: g3585 */
  f_20429(C_a_i(&a,3),((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t5=t1;
t6=C_slot(t2,C_fix(1));
t1=t5;
t2=t6;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* chicken.irregex#nfa-get-epsilons in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_20582(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;{}
t3=C_eqp(t2,C_fix(0));
if(C_truep(t3)){
return(C_SCHEME_END_OF_LIST);}
else{
t4=C_fixnum_times(t2,C_fix(4));
return(C_i_vector_ref(t1,C_fixnum_plus(t4,C_fix(1))));}}

/* chicken.irregex#nfa-add-epsilon! in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_20616(C_word *a,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_stack_overflow_check;{}
t5=(
/* irregex-core.scm:2421: nfa-get-epsilons */
  f_20582(t1,t2)
);
t6=C_i_assq(t3,t5);
if(C_truep(C_i_not(t6))){
t7=C_a_i_cons(&a,2,t3,t4);
t8=C_a_i_cons(&a,2,t7,t5);
t9=C_fixnum_times(t2,C_fix(4));
return(C_i_vector_set(t1,C_fixnum_plus(t9,C_fix(1)),t8));}
else{
t7=C_SCHEME_UNDEFINED;
return(t7);}}

/* chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20757(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_20757,3,t1,t2,t3);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20761,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_fixnum_times(&a,2,C_fix(128),C_fix(4));
/* irregex-core.scm:2447: scheme#make-vector */
t6=*((C_word*)lf[4]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,5)))){
C_save_and_reclaim((void *)f_20761,c,av);}
a=C_alloc(18);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20764,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21716,a[2]=t3,a[3]=t7,a[4]=((C_word)li275),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_21716(t9,t4,t5,C_fix(0),C_SCHEME_END_OF_LIST);}

/* k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,6)))){
C_save_and_reclaim((void *)f_20764,c,av);}
a=C_alloc(15);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20766,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t1,a[5]=((C_word)li273),tmp=(C_word)a,a+=6,tmp));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21660,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t6=C_a_i_list1(&a,1,((C_word*)t0)[4]);
/* irregex-core.scm:2664: lp */
t7=((C_word*)t3)[1];
f_20766(t7,t5,t6,C_fix(1),((C_word*)t0)[5],C_fix(0));}

/* lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20766(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(40,0,5)))){
C_save_and_reclaim_args((void *)trf_20766,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(40);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20769,a[2]=t3,a[3]=((C_word)li266),tmp=(C_word)a,a+=4,tmp));
t15=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20779,a[2]=((C_word*)t0)[2],a[3]=((C_word)li268),tmp=(C_word)a,a+=4,tmp));
t16=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20840,a[2]=t9,a[3]=t7,a[4]=((C_word)li269),tmp=(C_word)a,a+=5,tmp));
t17=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20857,a[2]=t4,a[3]=t11,a[4]=((C_word)li270),tmp=(C_word)a,a+=5,tmp));
if(C_truep(C_i_nullp(t2))){
t18=t1;{
C_word av2[2];
av2[0]=t18;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
t18=C_i_car(t2);
t19=C_eqp(lf[110],t18);
t20=(C_truep(t19)?t19:C_i_equalp(lf[390],C_u_i_car(t2)));
if(C_truep(t20)){
t21=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20907,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t9,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2495: lp */
t30=t21;
t31=C_u_i_cdr(t2);
t32=t3;
t33=t4;
t34=t5;
t1=t30;
t2=t31;
t3=t32;
t4=t33;
t5=t34;
goto loop;}
else{
if(C_truep(C_i_stringp(C_u_i_car(t2)))){
t21=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20931,a[2]=t2,a[3]=t13,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2502: lp */
t30=t21;
t31=C_u_i_cdr(t2);
t32=t3;
t33=t4;
t34=t5;
t1=t30;
t2=t31;
t3=t32;
t4=t33;
t5=t34;
goto loop;}
else{
if(C_truep(C_charp(C_u_i_car(t2)))){
t21=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20988,a[2]=t13,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2512: lp */
t30=t21;
t31=C_u_i_cdr(t2);
t32=t3;
t33=t4;
t34=t5;
t1=t30;
t2=t31;
t3=t32;
t4=t33;
t5=t34;
goto loop;}
else{
if(C_truep(C_i_symbolp(C_u_i_car(t2)))){
t21=C_i_assq(C_u_i_car(t2),lf[248]);
if(C_truep(t21)){
t22=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21015,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t3,a[6]=t4,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
t23=C_i_cdr(t21);
if(C_truep(C_i_closurep(t23))){
/* irregex-core.scm:2516: g3916 */
t24=C_u_i_cdr(t21);{
C_word av2[2];
av2[0]=t24;
av2[1]=t22;
((C_proc)(void*)(*((C_word*)t24+1)))(2,av2);}}
else{
t24=C_u_i_cdr(t21);
t25=C_u_i_cdr(t2);
t26=C_a_i_cons(&a,2,t24,t25);
/* irregex-core.scm:2516: lp */
t30=t1;
t31=t26;
t32=t3;
t33=t4;
t34=t5;
t1=t30;
t2=t31;
t3=t32;
t4=t33;
t5=t34;
goto loop;}}
else{
t22=t1;{
C_word av2[2];
av2[0]=t22;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t22+1)))(2,av2);}}}
else{
if(C_truep(C_i_pairp(C_u_i_car(t2)))){
t21=C_i_caar(t2);
if(C_truep(C_i_stringp(t21))){
t22=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21046,a[2]=t11,a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,tmp=(C_word)a,a+=9,tmp);
t23=C_fixnum_and(t4,C_fix(2));
t24=C_eqp(C_fix(2),t23);
if(C_truep(t24)){
t25=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21065,a[2]=t22,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:2527: string->cset */
f_26472(t25,C_i_caar(t2));}
else{
/* irregex-core.scm:2528: string->cset */
f_26472(t22,C_i_caar(t2));}}
else{
t22=C_i_caar(t2);
t23=C_eqp(t22,lf[115]);
t24=(C_truep(t23)?t23:C_eqp(t22,lf[227]));
if(C_truep(t24)){
t25=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21095,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,a[5]=t4,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
t26=C_u_i_car(t2);
/* irregex-core.scm:2534: scheme#append */
t27=*((C_word*)lf[73]+1);{
C_word av2[4];
av2[0]=t27;
av2[1]=t25;
av2[2]=C_u_i_cdr(t26);
av2[3]=C_u_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t27+1)))(4,av2);}}
else{
t25=C_eqp(t22,lf[225]);
t26=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21109,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t7,a[6]=t4,a[7]=t3,a[8]=t5,a[9]=t22,a[10]=t11,a[11]=t9,a[12]=((C_word*)t0)[2],a[13]=((C_word*)t0)[4],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t25)){
t27=t26;
f_21109(t27,t25);}
else{
t27=C_eqp(t22,lf[226]);
if(C_truep(t27)){
t28=t26;
f_21109(t28,t27);}
else{
t28=C_eqp(t22,lf[156]);
t29=t26;
f_21109(t29,(C_truep(t28)?t28:C_eqp(t22,lf[157])));}}}}}
else{
t21=t1;{
C_word av2[2];
av2[0]=t21;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}}}}}}}

/* new-state-number in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20769(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_20769,3,t0,t1,t2);}
/* irregex-core.scm:2472: scheme#max */
t3=*((C_word*)lf[254]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=C_fixnum_plus(C_fix(1),t2);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* add-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20779(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,3)))){
C_save_and_reclaim_args((void *)trf_20779,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20783,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t5=C_fixnum_times(t2,C_fix(4));
t6=C_i_vector_length(((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(C_fixnum_greater_or_equal_p(t5,t6))){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20792,a[2]=((C_word*)t0)[2],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t8=C_i_vector_length(((C_word*)((C_word*)t0)[2])[1]);
t9=C_a_i_fixnum_times(&a,2,C_fix(2),t8);
/* irregex-core.scm:2475: scheme#make-vector */
t10=*((C_word*)lf[4]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t7;
av2[2]=t9;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t7=t4;
f_20783(t7,C_SCHEME_UNDEFINED);}}

/* k20781 in add-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20783(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,1)))){
C_save_and_reclaim_args((void *)trf_20783,2,t0,t1);}
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t4=C_i_vector_set(t2,t3,((C_word*)t0)[4]);
t5=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k20790 in add-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_20792,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20795,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_vector_length(((C_word*)((C_word*)t0)[2])[1]);
t4=C_a_i_fixnum_difference(&a,2,t3,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20804,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t6,a[5]=((C_word)li267),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_20804(t8,t2,t4);}

/* k20793 in k20790 in add-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20795,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];
f_20783(t3,t2);}

/* doloop3883 in k20790 in add-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20804(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20804,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_fixnum_lessp(t2,C_fix(0)))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_vector_ref(((C_word*)((C_word*)t0)[2])[1],t2);
t4=C_i_vector_set(((C_word*)t0)[3],t2,t3);
t5=C_a_i_fixnum_difference(&a,2,t2,C_fix(1));
t7=t1;
t8=t5;
t1=t7;
t2=t8;
goto loop;}}

/* extend-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20840(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20840,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(t2)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20851,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2484: new-state-number */
t5=((C_word*)((C_word*)t0)[3])[1];
f_20769(t5,t4,t2);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k20849 in extend-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20851,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* irregex-core.scm:2484: add-state! */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20779(t3,((C_word*)t0)[5],t1,t2);}

/* add-char-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20857(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,3)))){
C_save_and_reclaim_args((void *)trf_20857,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=C_u_i_char_upper_casep(t3);
t5=(C_truep(t4)?C_u_i_char_downcase(t3):C_u_i_char_upcase(t3));
t6=C_fixnum_and(((C_word*)t0)[2],C_fix(2));
t7=C_eqp(C_fix(2),t6);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20874,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t9=C_a_i_cons(&a,2,t3,t3);
t10=C_a_i_vector1(&a,1,t9);
t11=C_a_i_cons(&a,2,t5,t5);
t12=C_a_i_vector1(&a,1,t11);
/* irregex-core.scm:2488: cset-union */
t13=lf[162];{
C_word av2[4];
av2[0]=t13;
av2[1]=t8;
av2[2]=t10;
av2[3]=t12;
f_27080(4,av2);}}
else{
t8=C_a_i_cons(&a,2,t3,t3);
t9=C_a_i_vector1(&a,1,t8);
/* irregex-core.scm:2489: extend-state! */
t10=((C_word*)((C_word*)t0)[3])[1];
f_20840(t10,t1,t2,t9);}}

/* k20872 in add-char-state! in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20874,c,av);}
/* irregex-core.scm:2488: extend-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20840(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k20905 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20907,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20913,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20920,a[2]=((C_word*)t0)[4],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2497: new-state-number */
t4=((C_word*)((C_word*)t0)[5])[1];
f_20769(t4,t3,t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20911 in k20905 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_20913,c,av);}
a=C_alloc(6);
t2=(
/* irregex-core.scm:2498: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[3],C_SCHEME_FALSE)
);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k20918 in k20905 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20920,c,av);}
/* irregex-core.scm:2497: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k20929 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_20931,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=C_i_string_length(C_u_i_car(((C_word*)t0)[2]));
t3=C_a_i_fixnum_difference(&a,2,t2,C_fix(1));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20945,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li271),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_20945(t7,((C_word*)t0)[4],t3,t1);}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* lp2 in k20929 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_20945(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_20945,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_fixnum_lessp(t2,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fixnum_difference(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20963,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_i_car(((C_word*)t0)[3]);
/* irregex-core.scm:2509: add-char-state! */
t7=((C_word*)((C_word*)t0)[4])[1];
f_20857(t7,t5,t3,C_i_string_ref(t6,t2));}}

/* k20961 in lp2 in k20929 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20963(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20963,c,av);}
/* irregex-core.scm:2508: lp2 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20945(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k20986 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_20988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20988,c,av);}
/* irregex-core.scm:2512: add-char-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20857(t2,((C_word*)t0)[3],t1,C_u_i_car(((C_word*)t0)[4]));}

/* k21013 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_21015,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
/* irregex-core.scm:2516: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_20766(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k21044 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_21046,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21053,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2529: lp */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20766(t3,t2,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k21051 in k21044 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21053,c,av);}
/* irregex-core.scm:2529: extend-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20840(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4]);}

/* k21063 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21065,c,av);}
/* irregex-core.scm:2527: cset-case-insensitive */
f_27607(((C_word*)t0)[2],t1);}

/* k21093 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21095(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21095,c,av);}
/* irregex-core.scm:2534: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21109(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,5)))){
C_save_and_reclaim_args((void *)trf_21109,2,t0,t1);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21112,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2536: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20766(t3,t2,C_u_i_cdr(((C_word*)t0)[2]),((C_word*)t0)[7],((C_word*)t0)[6],((C_word*)t0)[8]);}
else{
t2=C_eqp(((C_word*)t0)[9],lf[52]);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21156,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t2)){
t4=t3;
f_21156(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[9],lf[247]);
if(C_truep(t4)){
t5=t3;
f_21156(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[9],lf[234]);
t6=t3;
f_21156(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[9],lf[179])));}}}}

/* k21110 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_21112,c,av);}
a=C_alloc(7);
t2=C_i_caar(((C_word*)t0)[2]);
t3=(C_truep((C_truep(C_eqp(t2,lf[225]))?C_SCHEME_TRUE:(C_truep(C_eqp(t2,lf[156]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))?lf[64]:lf[63]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21118,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t5=C_u_i_car(((C_word*)t0)[2]);
t6=C_u_i_car(t5);
if(C_truep((C_truep(C_eqp(t6,lf[225]))?C_SCHEME_TRUE:(C_truep(C_eqp(t6,lf[226]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
/* irregex-core.scm:2536: g3943 */
t7=t3;{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=C_fix(2);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
/* irregex-core.scm:2536: g3943 */
t7=t3;{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=C_fix(32);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}

/* k21116 in k21110 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21118,c,av);}
a=C_alloc(7);
if(C_truep(((C_word*)t0)[2])){
t2=C_i_cdar(((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21132,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=t1,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2545: new-state-number */
t4=((C_word*)((C_word*)t0)[6])[1];
f_20769(t4,t3,((C_word*)t0)[2]);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21130 in k21116 in k21110 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21132,c,av);}
/* irregex-core.scm:2545: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21156(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,5)))){
C_save_and_reclaim_args((void *)trf_21156,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21159,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=C_u_i_car(((C_word*)t0)[5]);
t4=C_fixnum_and(((C_word*)t0)[7],C_fix(2));
t5=C_eqp(C_fix(2),t4);
/* irregex-core.scm:2547: sre->cset */
f_26492(t2,t3,C_a_i_list(&a,1,t5));}
else{
t2=C_eqp(((C_word*)t0)[9],lf[103]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21183,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[12],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:2552: lp */
t4=((C_word*)((C_word*)t0)[4])[1];
f_20766(t4,t3,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t3=C_eqp(((C_word*)t0)[9],lf[131]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21275,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[11],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2576: lp */
t5=((C_word*)((C_word*)t0)[4])[1];
f_20766(t5,t4,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t4=C_eqp(((C_word*)t0)[9],lf[129]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[9],lf[127]));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21309,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[11],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2585: lp */
t7=((C_word*)((C_word*)t0)[4])[1];
f_20766(t7,t6,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t6=C_eqp(((C_word*)t0)[9],lf[245]);
t7=(C_truep(t6)?t6:C_eqp(((C_word*)t0)[9],lf[108]));
if(C_truep(t7)){
t8=C_i_assq(C_u_i_car(((C_word*)t0)[5]),((C_word*)t0)[13]);
t9=C_i_cdr(t8);
t10=C_fixnum_shift_left(t9,C_fix(1));
t11=C_fixnum_plus(t10,C_fix(1));
t12=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21369,a[2]=((C_word*)t0)[12],a[3]=t10,a[4]=t11,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[7],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2630: lp */
t13=((C_word*)((C_word*)t0)[4])[1];
f_20766(t13,t12,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t8=C_eqp(((C_word*)t0)[9],lf[119]);
t9=(C_truep(t8)?t8:C_eqp(((C_word*)t0)[9],lf[120]));
if(C_truep(t9)){
t10=C_i_assq(C_u_i_car(((C_word*)t0)[5]),((C_word*)t0)[13]);
t11=C_i_cdr(t10);
t12=C_fixnum_shift_left(t11,C_fix(1));
t13=C_fixnum_plus(t12,C_fix(1));
t14=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_21447,a[2]=((C_word*)t0)[12],a[3]=t12,a[4]=t13,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[5],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2643: lp */
t15=((C_word*)((C_word*)t0)[4])[1];
f_20766(t15,t14,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t10=C_i_caar(((C_word*)t0)[5]);
t11=C_i_assq(t10,lf[248]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21514,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word)li272),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2654: g4019 */
t13=t12;
f_21514(t13,((C_word*)t0)[3],t11);}
else{
t12=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}}}}}}}

/* k21157 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_21159,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21166,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2549: lp */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20766(t3,t2,C_u_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k21164 in k21157 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21166,c,av);}
/* irregex-core.scm:2549: extend-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20840(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4]);}

/* k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_21183,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_i_cdar(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21199,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2557: new-state-number */
t4=((C_word*)((C_word*)t0)[5])[1];
f_20769(t4,t3,t1);}
else{
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21202,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t1,a[9]=((C_word*)t0)[2],tmp=(C_word)a,a+=10,tmp);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21260,a[2]=((C_word*)t0)[7],a[3]=t3,a[4]=((C_word*)t0)[8],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2560: scheme#cddar */
t5=*((C_word*)lf[393]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21197 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21199,c,av);}
/* irregex-core.scm:2557: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21202(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_21202,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21205,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21244,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[5],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2565: scheme#cadar */
t4=*((C_word*)lf[392]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21205(2,av2);}}}

/* k21203 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_21205,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21211,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21221,a[2]=((C_word*)t0)[5],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21225,a[2]=((C_word*)t0)[6],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2570: scheme#max */
t5=*((C_word*)lf[254]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21209 in k21203 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_21211,c,av);}
a=C_alloc(12);
t2=(
/* irregex-core.scm:2572: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[3],C_SCHEME_FALSE)
);
t3=(
/* irregex-core.scm:2573: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[4],C_SCHEME_FALSE)
);
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k21219 in k21203 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21221,c,av);}
/* irregex-core.scm:2570: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k21223 in k21203 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21225,c,av);}
/* irregex-core.scm:2570: new-state-number */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20769(t2,((C_word*)t0)[3],t1);}

/* k21234 in k21242 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21236,c,av);}
/* irregex-core.scm:2565: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21238 in k21242 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21240,c,av);}
/* irregex-core.scm:2566: new-state-number */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20769(t2,((C_word*)t0)[3],t1);}

/* k21242 in k21200 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21244(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_21244,c,av);}
a=C_alloc(14);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21236,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21240,a[2]=((C_word*)t0)[6],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2566: scheme#max */
t5=*((C_word*)lf[254]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k21250 in k21258 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21252(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21252,c,av);}
/* irregex-core.scm:2560: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21258 in k21181 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21260,c,av);}
a=C_alloc(13);
t2=(
/* irregex-core.scm:2560: sre-alternate */
  f_18214(C_a_i(&a,3),t1)
);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21252,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2561: new-state-number */
t5=((C_word*)((C_word*)t0)[6])[1];
f_20769(t5,t4,((C_word*)t0)[5]);}

/* k21273 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_21275,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21281,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_cdar(((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21295,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2580: new-state-number */
t5=((C_word*)((C_word*)t0)[7])[1];
f_20769(t5,t4,t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21279 in k21273 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_21281,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(
/* irregex-core.scm:2582: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[3],C_SCHEME_FALSE)
);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21293 in k21273 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21295(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21295,c,av);}
/* irregex-core.scm:2580: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21307 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_21309,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21349,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[6],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2589: new-state-number */
t4=((C_word*)((C_word*)t0)[7])[1];
f_20769(t4,t3,t1);}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21313 in k21307 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21315,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_i_cdar(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21345,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2592: new-state-number */
t5=((C_word*)((C_word*)t0)[7])[1];
f_20769(t5,t4,t1);}

/* k21316 in k21313 in k21307 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_21318,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_i_caar(((C_word*)t0)[2]);
t3=C_eqp(lf[127],t2);
if(C_truep(t3)){
t4=(
/* irregex-core.scm:2597: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[3])[1],t1,((C_word*)t0)[4],C_SCHEME_FALSE)
);
t5=(
/* irregex-core.scm:2599: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[4],t1,C_SCHEME_FALSE)
);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t4=(
/* irregex-core.scm:2599: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[4],t1,C_SCHEME_FALSE)
);
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21343 in k21313 in k21307 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21345,c,av);}
/* irregex-core.scm:2592: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21347 in k21307 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21349,c,av);}
/* irregex-core.scm:2588: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],lf[394],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_21369,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21375,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21417,a[2]=((C_word*)t0)[6],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2632: new-state-number */
t4=((C_word*)((C_word*)t0)[7])[1];
f_20769(t4,t3,t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21373 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_21375,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21378,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=(
/* irregex-core.scm:2633: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdar(((C_word*)t0)[9]))
);
t4=C_a_i_list1(&a,1,t3);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21405,a[2]=((C_word*)t0)[10],a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[11],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2634: new-state-number */
t6=((C_word*)((C_word*)t0)[8])[1];
f_20769(t6,t5,t1);}

/* k21376 in k21373 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21378,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21381,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21397,a[2]=((C_word*)t0)[8],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2635: new-state-number */
t4=((C_word*)((C_word*)t0)[9])[1];
f_20769(t4,t3,t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21381(2,av2);}}}

/* k21379 in k21376 in k21373 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_21381,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(
/* irregex-core.scm:2637: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[3],((C_word*)t0)[4])
);
t3=(
/* irregex-core.scm:2638: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7])
);
t4=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21395 in k21376 in k21373 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21397,c,av);}
/* irregex-core.scm:2635: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k21403 in k21373 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21405,c,av);}
/* irregex-core.scm:2633: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21415 in k21367 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21417,c,av);}
/* irregex-core.scm:2632: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_21447,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21453,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21495,a[2]=((C_word*)t0)[6],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2645: new-state-number */
t4=((C_word*)((C_word*)t0)[7])[1];
f_20769(t4,t3,t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_21453,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21456,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21491,a[2]=((C_word*)t0)[9],a[3]=t2,a[4]=((C_word*)t0)[10],a[5]=t1,a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2646: scheme#cddar */
t4=*((C_word*)lf[393]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k21454 in k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21456,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21459,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21475,a[2]=((C_word*)t0)[8],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2648: new-state-number */
t4=((C_word*)((C_word*)t0)[9])[1];
f_20769(t4,t3,t1);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_21459(2,av2);}}}

/* k21457 in k21454 in k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_21459,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(
/* irregex-core.scm:2650: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],t1,((C_word*)t0)[3],((C_word*)t0)[4])
);
t3=(
/* irregex-core.scm:2651: nfa-add-epsilon! */
  f_20616(C_a_i(&a,6),((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7])
);
t4=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21473 in k21454 in k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21475,c,av);}
/* irregex-core.scm:2648: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* k21481 in k21489 in k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21483,c,av);}
/* irregex-core.scm:2646: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20766(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k21489 in k21451 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21491,c,av);}
a=C_alloc(13);
t2=(
/* irregex-core.scm:2646: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21483,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2647: new-state-number */
t5=((C_word*)((C_word*)t0)[6])[1];
f_20769(t5,t4,((C_word*)t0)[5]);}

/* k21493 in k21445 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21495,c,av);}
/* irregex-core.scm:2645: add-state! */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20779(t2,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* g4019 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21514(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_21514,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
if(C_truep(C_i_closurep(t3))){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21532,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);{
C_word av2[4];
av2[0]=0;
av2[1]=t4;
av2[2]=C_u_i_cdr(t2);
av2[3]=C_i_cdar(((C_word*)t0)[2]);
C_apply(4,av2);}}
else{
/* irregex-core.scm:2660: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[395];
av2[3]=C_i_caar(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k21530 in g4019 in k21154 in k21107 in lp in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_21532,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
/* irregex-core.scm:2658: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_20766(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k21658 in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21660,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21666,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_fixnum_plus(t1,C_fix(1));
/* irregex-core.scm:2666: scheme#make-vector */
t4=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_fixnum_times(C_fix(4),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k21664 in k21658 in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21666(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_21666,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21669,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_block_size(t1);
t4=C_a_i_fixnum_difference(&a,2,t3,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21677,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t6,a[5]=((C_word)li274),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_21677(t8,t2,t4);}

/* k21667 in k21664 in k21658 in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21669,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* doloop4029 in k21664 in k21658 in k20762 in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21677(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_21677,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_fixnum_lessp(t2,C_fix(0)))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_vector_ref(((C_word*)((C_word*)t0)[2])[1],t2);
t4=C_i_vector_set(((C_word*)t0)[3],t2,t3);
t5=C_a_i_fixnum_difference(&a,2,t2,C_fix(1));
t7=t1;
t8=t5;
t1=t7;
t2=t8;
goto loop;}}

/* lp in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21716(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_21716,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_i_pairp(t2);
if(C_truep(C_i_not(t5))){
t6=C_fixnum_shift_left(t3,C_fix(1));
t7=C_i_vector_set(((C_word*)((C_word*)t0)[2])[1],C_fix(0),t6);
t8=C_i_vector_set(((C_word*)((C_word*)t0)[2])[1],C_fix(1),C_fix(-1));
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t6=C_i_car(t2);
if(C_truep(C_i_pairp(t6))){
t7=C_i_caar(t2);
t8=C_eqp(t7,lf[245]);
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21751,a[2]=t3,a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[3],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t8)){
t10=t9;
f_21751(t10,t8);}
else{
t10=C_eqp(t7,lf[108]);
if(C_truep(t10)){
t11=t9;
f_21751(t11,t10);}
else{
t11=C_eqp(t7,lf[119]);
t12=t9;
f_21751(t12,(C_truep(t11)?t11:C_eqp(t7,lf[120])));}}}
else{
/* irregex-core.scm:2468: lp */
t13=t1;
t14=C_u_i_cdr(t2);
t15=t3;
t16=t4;
t1=t13;
t2=t14;
t3=t15;
t4=t16;
goto loop;}}}

/* k21749 in lp in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21751(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_21751,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21758,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_u_i_car(((C_word*)t0)[3]);
/* irregex-core.scm:2465: scheme#append */
t4=*((C_word*)lf[73]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t2;
av2[2]=C_u_i_cdr(t3);
av2[3]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21784,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2467: scheme#append */
t3=*((C_word*)lf[73]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[3]);
av2[3]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k21756 in k21749 in lp in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_21758,c,av);}
a=C_alloc(6);
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
t3=C_u_i_car(((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[4]);
/* irregex-core.scm:2465: lp */
t6=((C_word*)((C_word*)t0)[5])[1];
f_21716(t6,((C_word*)t0)[6],t1,t2,t5);}

/* k21782 in k21749 in lp in k20759 in chicken.irregex#sre->nfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21784,c,av);}
/* irregex-core.scm:2467: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21716(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* chicken.irregex#make-mst in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21880(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_21880,2,t1,t2);}
a=C_alloc(14);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21884,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_i_vector_length(t2);
t5=C_a_i_fixnum_quotient_checked(&a,2,t4,C_fix(4));
t6=C_s_a_u_i_integer_plus(&a,2,t5,C_fix(3));
/* irregex-core.scm:2737: scheme#make-vector */
t7=*((C_word*)lf[4]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t3;
av2[2]=t6;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k21882 in chicken.irregex#make-mst in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_21884,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21897,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2738: scheme#make-vector */
t3=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21895 in k21882 in chicken.irregex#make-mst in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21897,c,av);}
t2=C_i_vector_set(((C_word*)t0)[2],C_fix(0),t1);
t3=C_i_vector_set(((C_word*)t0)[2],C_fix(1),C_fix(0));
t4=C_i_vector_set(((C_word*)t0)[2],C_fix(2),C_fix(0));
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.irregex#mst-copy in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21909(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_21909,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21913,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2745: vector-copy */
f_7540(t3,t2);}

/* k21911 in chicken.irregex#mst-copy in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21913,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21920,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2746: vector-copy */
f_7540(t2,C_i_vector_ref(((C_word*)t0)[3],C_fix(0)));}

/* k21918 in k21911 in chicken.irregex#mst-copy in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21920,c,av);}
t2=C_i_vector_set(((C_word*)t0)[2],C_fix(0),t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#nfa-state->mst in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21926(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21926,4,t1,t2,t3,t4);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21930,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2750: make-mst */
f_21880(t5,t2);}

/* k21928 in chicken.irregex#nfa-state->mst in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_21930,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21933,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2751: mst-add! */
f_21935(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k21931 in k21928 in chicken.irregex#nfa-state->mst in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21933,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#mst-add! in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21935(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_21935,5,t1,t2,t3,t4,t5);}
a=C_alloc(10);
t6=C_i_vector_ref(t3,C_fix(2));
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21942,a[2]=t3,a[3]=t4,a[4]=t5,a[5]=t8,a[6]=t2,a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t10=C_fixnum_plus(t4,C_fix(3));
t11=C_i_vector_ref(t3,t10);
if(C_truep(C_i_not(t11))){
t12=C_fixnum_plus(((C_word*)t8)[1],t4);
t13=C_set_block_item(t8,0,t12);
t14=C_i_vector_ref(t3,C_fix(1));
t15=t9;
f_21942(t15,C_i_vector_set(t3,C_fix(1),C_fixnum_plus(t14,C_fix(1))));}
else{
t12=t9;
f_21942(t12,C_SCHEME_UNDEFINED);}}

/* k21940 in chicken.irregex#mst-add! in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21942(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_21942,2,t0,t1);}
a=C_alloc(18);
t2=C_i_vector_set(((C_word*)t0)[2],C_fixnum_plus(((C_word*)t0)[3],C_fix(3)),((C_word*)t0)[4]);
t3=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21949,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=((C_word)li280),tmp=(C_word)a,a+=5,tmp);
t5=C_i_check_list_2(((C_word*)t0)[4],lf[306]);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21984,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21997,a[2]=t4,a[3]=t8,a[4]=((C_word)li281),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_21997(t10,t6,((C_word*)t0)[4]);}

/* g4087 in k21940 in chicken.irregex#mst-add! in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_21949(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_stack_overflow_check;{}
t2=C_i_car(t1);
t3=C_u_i_cdr(t1);
t4=C_i_vector_ref(((C_word*)t0)[2],t2);
t5=C_i_memq(t3,t4);
if(C_truep(C_i_not(t5))){
t6=C_fixnum_plus(((C_word*)((C_word*)t0)[3])[1],t2);
t7=C_mutate(((C_word *)((C_word*)t0)[3])+1,t6);
t8=C_a_i_cons(&a,2,t3,t4);
return(C_i_vector_set(((C_word*)t0)[2],t2,t8));}
else{
t6=C_SCHEME_UNDEFINED;
return(t6);}}

/* k21982 in k21940 in chicken.irregex#mst-add! in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_21984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,1)))){
C_save_and_reclaim((void *)f_21984,c,av);}
a=C_alloc(10);
t2=C_i_vector_length(((C_word*)t0)[2]);
t3=C_a_i_fixnum_quotient_checked(&a,2,t2,C_fix(4));
t4=C_s_a_i_modulo(&a,2,((C_word*)((C_word*)t0)[3])[1],t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_vector_set(((C_word*)t0)[5],C_fix(2),t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* for-each-loop4086 in k21940 in chicken.irregex#mst-add! in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_21997(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_21997,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=(
/* irregex-core.scm:2762: g4087 */
  f_21949(C_a_i(&a,3),((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t5=t1;
t6=C_slot(t2,C_fix(1));
t1=t5;
t2=t6;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k22096 in k22990 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22098(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_22098,c,av);}
a=C_alloc(17);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_22104,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=((C_word*)t0)[12],tmp=(C_word)a,a+=15,tmp);
t6=C_fixnum_plus(((C_word*)t0)[3],C_fix(3));
t7=C_i_vector_ref(((C_word*)t0)[2],t6);
if(C_truep(C_i_not(t7))){
t8=C_fixnum_plus(((C_word*)t4)[1],((C_word*)t0)[3]);
t9=C_set_block_item(t4,0,t8);
t10=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t11=t5;
f_22104(t11,C_i_vector_set(((C_word*)t0)[2],C_fix(1),C_fixnum_plus(t10,C_fix(1))));}
else{
t8=t5;
f_22104(t8,C_SCHEME_UNDEFINED);}}

/* k22102 in k22096 in k22990 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22104(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_22104,2,t0,t1);}
a=C_alloc(14);
t2=C_i_vector_set(((C_word*)t0)[2],C_fixnum_plus(((C_word*)t0)[3],C_fix(3)),((C_word*)t0)[4]);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22110,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t4=C_i_memq(((C_word*)t0)[11],((C_word*)t0)[12]);
if(C_truep(C_i_not(t4))){
t5=C_fixnum_plus(((C_word*)((C_word*)t0)[6])[1],((C_word*)t0)[13]);
t6=C_mutate(((C_word *)((C_word*)t0)[6])+1,t5);
t7=C_a_i_cons(&a,2,((C_word*)t0)[11],((C_word*)t0)[12]);
t8=t3;
f_22110(t8,C_i_vector_set(((C_word*)t0)[14],((C_word*)t0)[13],t7));}
else{
t5=t3;
f_22110(t5,C_SCHEME_UNDEFINED);}}

/* k22108 in k22102 in k22096 in k22990 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22110(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,3)))){
C_save_and_reclaim_args((void *)trf_22110,2,t0,t1);}
a=C_alloc(16);
t2=C_i_vector_length(((C_word*)t0)[2]);
t3=C_a_i_fixnum_quotient_checked(&a,2,t2,C_fix(4));
t4=C_s_a_i_modulo(&a,2,((C_word*)((C_word*)t0)[3])[1],t3);
t5=C_i_vector_set(((C_word*)t0)[4],C_fix(2),t4);
t6=C_i_cdr(((C_word*)t0)[5]);
t7=C_a_i_cons(&a,2,((C_word*)t0)[6],((C_word*)t0)[7]);
t8=C_a_i_cons(&a,2,t7,((C_word*)t0)[8]);
/* irregex-core.scm:3030: lp2 */
t9=((C_word*)((C_word*)t0)[9])[1];
f_22959(t9,((C_word*)t0)[10],t6,t8);}

/* lp in k22990 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22163(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_22163,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_a_i_cons(&a,2,t4,t3);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_i_caar(t2);
t5=C_eqp(t4,((C_word*)t0)[2]);
if(C_truep(t5)){
t6=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t7=C_u_i_cdr(t2);
t8=C_a_i_cons(&a,2,t6,t7);
/* irregex-core.scm:2793: scheme#append */
t9=*((C_word*)lf[73]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t1;
av2[2]=t3;
av2[3]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t6=C_u_i_cdr(t2);
t7=C_u_i_car(t2);
t8=C_a_i_cons(&a,2,t7,t3);
/* irregex-core.scm:2794: lp */
t10=t1;
t11=t6;
t12=t8;
t1=t10;
t2=t11;
t3=t12;
goto loop;}}}

/* lp in lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22234(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_22234,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_vector_ref(((C_word*)t0)[3],t2);
t5=C_i_not(t4);
t6=C_i_vector_ref(((C_word*)t0)[4],t2);
t7=C_i_not(t6);
t8=C_eqp(t5,t7);
if(C_truep(t8)){
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:2815: lp */
t11=t1;
t12=t9;
t1=t11;
t2=t12;
goto loop;}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}}

/* chicken.irregex#mst-fold in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22287(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_22287,4,t1,t2,t3,t4);}
a=C_alloc(9);
t5=C_i_vector_length(t2);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22296,a[2]=t5,a[3]=t2,a[4]=t7,a[5]=t3,a[6]=((C_word)li283),tmp=(C_word)a,a+=7,tmp));
t9=((C_word*)t7)[1];
f_22296(t9,t1,C_fix(3),t4);}

/* lp in chicken.irregex#mst-fold in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22296(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_22296,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_vector_ref(((C_word*)t0)[3],t2);
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22316,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t8=C_a_i_fixnum_difference(&a,2,t2,C_fix(3));
/* irregex-core.scm:2824: kons */
t9=((C_word*)t0)[5];{
C_word av2[5];
av2[0]=t9;
av2[1]=t7;
av2[2]=t8;
av2[3]=t5;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
/* irregex-core.scm:2824: lp */
t10=t1;
t11=t6;
t12=t3;
t1=t10;
t2=t11;
t3=t12;
goto loop;}}}

/* k22314 in lp in chicken.irregex#mst-fold in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22316,c,av);}
/* irregex-core.scm:2824: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22296(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* doloop4187 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_22345(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
t2=C_i_memq(t1,((C_word*)t0)[2]);
if(C_truep(C_i_not(t2))){
return(t1);}
else{
t4=C_fixnum_plus(t1,C_fix(1));
t1=t4;
goto loop;}}

/* chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22369(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_22369,3,t1,t2,t3);}
a=C_alloc(15);
t4=C_i_pairp(t3);
t5=(C_truep(t4)?C_u_i_car(t3):C_SCHEME_FALSE);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22376,a[2]=t2,a[3]=t5,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t7=C_i_vector_length(t2);
t8=C_a_i_fixnum_quotient_checked(&a,2,t7,C_fix(4));
t9=C_s_a_u_i_integer_minus(&a,2,t8,C_fix(1));
/* irregex-core.scm:2873: nfa-state->mst */
f_21926(t6,t2,t9,C_SCHEME_END_OF_LIST);}

/* k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_22376,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22379,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2874: nfa-epsilon-closure */
f_23054(t2,((C_word*)t0)[2],t1);}

/* k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22379(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_22379,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22382,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2877: tag-set-commands-for-closure */
f_23069(t2,((C_word*)t0)[2],((C_word*)t0)[5],t1,C_SCHEME_END_OF_LIST);}

/* k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22382,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22385,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:2878: make-mst */
f_21880(t2,((C_word*)t0)[3]);}

/* k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22385(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,5)))){
C_save_and_reclaim((void *)f_22385,c,av);}
a=C_alloc(36);
t2=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,C_SCHEME_FALSE,t2);
t4=C_a_i_cons(&a,2,((C_word*)t0)[3],t3);
t5=C_a_i_list(&a,1,t4);
t6=C_a_i_list3(&a,3,t1,C_SCHEME_FALSE,t5);
t7=C_a_i_list1(&a,1,((C_word*)t0)[3]);
t8=C_a_i_list1(&a,1,t6);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22401,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t10,a[6]=((C_word)li297),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_22401(t12,((C_word*)t0)[6],t7,t8,C_fix(0));}

/* lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22401(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,5)))){
C_save_and_reclaim_args((void *)trf_22401,5,t0,t1,t2,t3,t4);}
a=C_alloc(13);
if(C_truep(C_i_nullp(t2))){
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_i_vector_ref(((C_word*)t0)[3],C_fix(1));
t7=C_fixnum_plus(t6,C_fix(1));
t8=C_i_setslot(t5,C_fix(0),t7);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22418,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:2889: ##sys#fast-reverse */
t10=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t5=(C_truep(((C_word*)t0)[4])?C_fixnum_greaterp(t4,((C_word*)t0)[4]):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_i_car(t2);
if(C_truep(C_i_assoc(t6,t3))){
/* irregex-core.scm:2893: lp */
t11=t1;
t12=C_u_i_cdr(t2);
t13=t3;
t14=t4;
t1=t11;
t2=t12;
t3=t13;
t4=t14;
goto loop;}
else{
t7=C_u_i_car(t2);
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22453,a[2]=t2,a[3]=t7,a[4]=t3,a[5]=((C_word*)t0)[5],a[6]=t4,a[7]=((C_word*)t0)[3],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22756,a[2]=((C_word*)t0)[3],a[3]=((C_word)li296),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2954: mst-fold */
f_22287(t8,t7,t9,C_SCHEME_END_OF_LIST);}}}}

/* k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_22418,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22580,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22702,a[2]=t4,a[3]=((C_word)li288),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_22702(t6,t2,C_fix(0),t1,C_SCHEME_END_OF_LIST);}

/* k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_22453,c,av);}
a=C_alloc(11);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22457,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t4,a[7]=((C_word*)t0)[7],a[8]=((C_word)li293),tmp=(C_word)a,a+=9,tmp));
t6=((C_word*)t4)[1];
f_22457(t6,((C_word*)t0)[8],t1,t2,C_SCHEME_END_OF_LIST);}

/* lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22457(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_22457,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
if(C_truep(C_i_nullp(t2))){
t5=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t6=C_a_i_list3(&a,3,((C_word*)t0)[2],t5,t4);
t7=C_a_i_cons(&a,2,t6,((C_word*)t0)[3]);
/* irregex-core.scm:2901: lp */
t8=((C_word*)((C_word*)t0)[4])[1];
f_22401(t8,t1,t3,t7,C_fixnum_plus(((C_word*)t0)[5],C_fix(1)));}
else{
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22485,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:2904: nfa-epsilon-closure */
f_23054(t5,((C_word*)t0)[7],C_i_cdar(t2));}}

/* k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_22485,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22488,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=C_i_vector_ref(t1,C_fix(2));
t4=C_fixnum_times(t3,C_fix(4));
t5=C_fixnum_plus(t4,C_fix(2));
t6=C_i_vector_ref(((C_word*)t0)[7],t5);
t7=C_i_assoc(t1,t6);
t8=(C_truep(t7)?C_i_cdr(t7):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=t2;
f_22488(t9,t8);}
else{
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23387,a[2]=t1,a[3]=((C_word*)t0)[7],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t10=C_i_vector_ref(((C_word*)t0)[7],C_fix(0));
t11=C_i_vector_ref(t1,C_fix(0));
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23236,a[2]=t13,a[3]=t10,a[4]=t11,a[5]=t1,a[6]=((C_word)li292),tmp=(C_word)a,a+=7,tmp));
t15=((C_word*)t13)[1];
f_23236(t15,t9,((C_word*)t0)[8]);}}

/* k22486 in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22488(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_22488,2,t0,t1);}
a=C_alloc(10);
t2=(C_truep(t1)?C_i_cdr(t1):C_SCHEME_END_OF_LIST);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22494,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:2910: tag-set-commands-for-closure */
f_23069(t3,((C_word*)t0)[8],C_i_cdar(((C_word*)t0)[3]),((C_word*)t0)[2],t2);}

/* k22492 in k22486 in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_22494,c,av);}
a=C_alloc(15);
t2=(C_truep(((C_word*)t0)[2])?C_i_car(((C_word*)t0)[2]):((C_word*)t0)[3]);
t3=C_u_i_cdr(((C_word*)t0)[4]);
t4=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[5]:C_a_i_cons(&a,2,t2,((C_word*)t0)[5]));
t5=C_i_caar(((C_word*)t0)[4]);
t6=C_a_i_cons(&a,2,((C_word*)t0)[6],t1);
t7=C_a_i_cons(&a,2,t5,t6);
t8=C_a_i_cons(&a,2,t2,t7);
t9=C_a_i_cons(&a,2,t8,((C_word*)t0)[7]);
/* irregex-core.scm:2913: lp2 */
t10=((C_word*)((C_word*)t0)[8])[1];
f_22457(t10,((C_word*)t0)[9],t3,t4,t9);}

/* k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22580,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22583,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2931: scheme#make-vector */
t3=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_length(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_22583,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22588,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word)li287),tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_22588(t5,((C_word*)t0)[3],C_fix(0),((C_word*)t0)[4]);}

/* doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22588(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_22588,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22598,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=t1,a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:2935: scheme#cadar */
t5=*((C_word*)lf[392]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22598(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_22598,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22601,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:2936: scheme#caddar */
t3=*((C_word*)lf[401]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_22601,c,av);}
a=C_alloc(26);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22621,a[2]=((C_word*)t0)[2],a[3]=((C_word)li285),tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(t1,lf[211]);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22662,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22664,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,a[6]=((C_word)li286),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_22664(t12,t8,t1);}

/* g4258 in k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22621(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_22621,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22629,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_cadr(t2))){
t4=C_u_i_cdr(t2);
t5=C_u_i_car(t4);
t6=C_i_vector_length(t5);
t7=C_eqp(t6,C_fix(1));
if(C_truep(t7)){
t8=C_i_vector_ref(t5,C_fix(0));
t9=C_i_car(t8);
t10=C_i_vector_ref(t5,C_fix(0));
t11=C_i_char_equalp(t9,C_i_cdr(t10));
t12=t3;
f_22629(t12,(C_truep(t11)?C_i_car(C_i_vector_ref(t5,C_fix(0))):t5));}
else{
t8=t3;
f_22629(t8,(C_truep(C_SCHEME_FALSE)?C_i_car(C_i_vector_ref(t5,C_fix(0))):t5));}}
else{
t4=t3;
f_22629(t4,C_SCHEME_FALSE);}}

/* k22627 in g4258 in k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22629(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,1)))){
C_save_and_reclaim_args((void *)trf_22629,2,t0,t1);}
a=C_alloc(6);
t2=C_i_assoc(C_u_i_car(((C_word*)t0)[2]),((C_word*)t0)[3]);
t3=C_i_cdr(t2);
t4=C_i_cddr(((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,t3,t4);
t6=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_cons(&a,2,t1,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k22660 in k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_22662,c,av);}
a=C_alloc(8);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_i_vector_set(((C_word*)t0)[3],((C_word*)t0)[4],t2);
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[4],C_fix(1));
t5=((C_word*)((C_word*)t0)[5])[1];
f_22588(t5,((C_word*)t0)[6],t4,C_u_i_cdr(((C_word*)t0)[7]));}

/* map-loop4252 in k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22664(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_22664,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22689,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:2940: g4258 */
t4=((C_word*)t0)[4];
f_22621(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k22687 in map-loop4252 in k22599 in k22596 in doloop4244 in k22581 in k22578 in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22689,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_22664(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* lp in k22416 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22702(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_22702,5,t0,t1,t2,t3,t4);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t3))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_fixnum_plus(t2,C_fix(1));
t6=C_i_cdr(t3);
t7=C_i_caar(t3);
t8=C_a_i_cons(&a,2,t7,t2);
t9=C_a_i_cons(&a,2,t8,t4);
/* irregex-core.scm:2929: lp */
t11=t1;
t12=t5;
t13=t6;
t14=t9;
t1=t11;
t2=t12;
t3=t13;
t4=t14;
goto loop;}}

/* k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_22741,c,av);}
a=C_alloc(10);
t2=C_i_vector_length(t1);
t3=C_eqp(t2,C_fix(0));
t4=C_i_not(t3);
t5=(C_truep(t4)?t1:C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22818,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word)li294),tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:2961: g4307 */
t7=t6;
f_22818(t7,((C_word*)t0)[9],t5);}
else{
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_u_i_car(((C_word*)t0)[2]);
t8=C_a_i_cons(&a,2,t7,((C_word*)t0)[7]);
/* irregex-core.scm:2996: lp */
t9=((C_word*)((C_word*)t0)[3])[1];
f_22777(t9,((C_word*)t0)[9],t6,((C_word*)t0)[8],((C_word*)t0)[4],t8);}}

/* a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,6)))){
C_save_and_reclaim((void *)f_22756,c,av);}
a=C_alloc(8);
t5=C_eqp(t2,C_fix(0));
t6=(C_truep(t5)?C_SCHEME_END_OF_LIST:C_i_vector_ref(((C_word*)t0)[2],C_fixnum_times(t2,C_fix(4))));
if(C_truep(C_i_nullp(t6))){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_i_car(t6);
t8=C_u_i_cdr(t6);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22777,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t10,a[5]=((C_word)li295),tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_22777(t12,t1,t4,t7,t8,C_SCHEME_END_OF_LIST);}}

/* lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22777(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_22777,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t2))){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22795,a[2]=t3,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2968: nfa-state->mst */
f_21926(t6,((C_word*)t0)[2],t4,((C_word*)t0)[3]);}
else{
t6=C_i_caar(t2);
if(C_truep(C_i_equalp(t3,t6))){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22804,a[2]=t1,a[3]=t2,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:2972: mst-add! */
f_21935(t7,((C_word*)t0)[2],C_i_cdar(t2),t4,((C_word*)t0)[3]);}
else{
t7=C_i_caar(t2);
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22741,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=t4,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=t5,a[8]=t3,a[9]=t1,tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:2952: cset-intersection */
t9=lf[405];{
C_word av2[4];
av2[0]=t9;
av2[1]=t8;
av2[2]=t3;
av2[3]=t7;
f_27471(4,av2);}}}}

/* k22793 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_22795,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k22802 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22804,c,av);}
/* irregex-core.scm:2973: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22818(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_22818,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22822,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2976: cset-difference */
f_27275(t3,((C_word*)t0)[8],C_i_caar(((C_word*)t0)[2]));}

/* k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_22822,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22825,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:2977: cset-difference */
f_27275(t2,C_i_caar(((C_word*)t0)[2]),((C_word*)t0)[10]);}

/* k22823 in k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_22825,c,av);}
a=C_alloc(13);
t2=C_i_cdar(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22831,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=t1,a[12]=((C_word*)t0)[10],tmp=(C_word)a,a+=13,tmp);
t4=C_i_vector_length(t1);
t5=C_eqp(t4,C_fix(0));
if(C_truep(C_i_not(t5))){
/* irregex-core.scm:2981: mst-copy */
f_21909(t3,t2);}
else{
t6=t3;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_22831(2,av2);}}}

/* k22829 in k22823 in k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_22831,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22834,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t1)){
t3=C_a_i_cons(&a,2,((C_word*)t0)[11],t1);
t4=t2;
f_22834(t4,C_a_i_cons(&a,2,t3,((C_word*)t0)[12]));}
else{
t3=t2;
f_22834(t3,((C_word*)t0)[12]);}}

/* k22832 in k22829 in k22823 in k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22834(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_22834,2,t0,t1);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22837,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:2985: mst-add! */
f_21935(t2,((C_word*)t0)[9],((C_word*)t0)[4],((C_word*)t0)[8],((C_word*)t0)[10]);}

/* k22835 in k22832 in k22829 in k22823 in k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_22837,c,av);}
a=C_alloc(7);
t2=C_i_vector_length(((C_word*)t0)[2]);
t3=C_eqp(t2,C_fix(0));
if(C_truep(t3)){
t4=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22854,a[2]=((C_word*)t0)[5],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:2992: scheme#append */
t6=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_u_i_cdr(((C_word*)t0)[6]);
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t4=C_u_i_cdr(((C_word*)t0)[6]);
t5=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[7]);
/* irregex-core.scm:2993: lp */
t7=((C_word*)((C_word*)t0)[8])[1];
f_22777(t7,((C_word*)t0)[5],t4,((C_word*)t0)[2],((C_word*)t0)[9],t6);}}

/* k22852 in k22835 in k22832 in k22829 in k22823 in k22820 in g4307 in k22739 in lp in a22755 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_22854,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_22926,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22930,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_i_vector_length(((C_word*)t0)[2]);
t4=C_a_i_fixnum_quotient_checked(&a,2,t3,C_fix(4));
/* irregex-core.scm:3007: scheme#make-vector */
t5=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=t4;
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22930,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22934,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3008: mst-copy */
f_21909(t2,((C_word*)t0)[5]);}

/* k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_22934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_22934,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22936,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word)li303),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_22936(t5,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22936(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_22936,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
if(C_truep(C_i_nullp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_caar(t2);
t6=C_u_i_car(t2);
t7=C_u_i_cdr(t6);
t8=(
/* irregex-core.scm:3013: nfa-get-epsilons */
  f_20582(((C_word*)t0)[2],t5)
);
t9=C_u_i_cdr(t2);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22959,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=t4,a[5]=t5,a[6]=((C_word*)t0)[2],a[7]=t11,a[8]=t7,a[9]=((C_word)li302),tmp=(C_word)a,a+=10,tmp));
t13=((C_word*)t11)[1];
f_22959(t13,t1,t8,t9);}}

/* lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22959(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,4)))){
C_save_and_reclaim_args((void *)trf_22959,4,t0,t1,t2,t3);}
a=C_alloc(18);
if(C_truep(C_i_nullp(t2))){
/* irregex-core.scm:3016: lp */
t4=((C_word*)((C_word*)t0)[2])[1];
f_22936(t4,t1,t3,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t4=C_i_caar(t2);
t5=C_i_vector_ref(((C_word*)t0)[3],t4);
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[5],t5))){
t6=C_i_vector_set(((C_word*)t0)[3],t4,((C_word*)t0)[5]);
t7=C_i_cdar(t2);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22988,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word*)t0)[6],a[5]=t2,a[6]=t3,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word)li301),tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3023: g4342 */
t9=t8;
f_22988(t9,t1,t7);}
else{
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23016,a[2]=t2,a[3]=t4,a[4]=((C_word*)t0)[8],a[5]=t3,a[6]=((C_word*)t0)[7],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t9=C_fixnum_plus(t4,C_fix(3));
t10=C_i_vector_ref(((C_word*)t0)[4],t9);
if(C_truep(C_i_not(t10))){
t11=C_i_vector_ref(((C_word*)t0)[4],C_fix(2));
t12=C_fixnum_plus(t11,t4);
t13=C_i_vector_length(((C_word*)t0)[6]);
t14=C_a_i_fixnum_quotient_checked(&a,2,t13,C_fix(4));
t15=C_s_a_i_modulo(&a,2,t12,t14);
t16=C_i_vector_set(((C_word*)t0)[4],C_fix(2),t15);
t17=C_i_vector_ref(((C_word*)t0)[4],C_fix(1));
t18=C_fixnum_plus(t17,C_fix(1));
t19=C_i_vector_set(((C_word*)t0)[4],C_fix(1),t18);
t20=t8;
f_23016(t20,C_i_vector_set(((C_word*)t0)[4],C_fixnum_plus(t4,C_fix(3)),((C_word*)t0)[8]));}
else{
t11=t8;
f_23016(t11,C_i_vector_set(((C_word*)t0)[4],C_fixnum_plus(t4,C_fix(3)),((C_word*)t0)[8]));}}}
else{
/* irregex-core.scm:3035: lp2 */
t21=t1;
t22=C_u_i_cdr(t2);
t23=t3;
t1=t21;
t2=t22;
t3=t23;
goto loop;}}}

/* g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22988(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_22988,3,t0,t1,t2);}
a=C_alloc(15);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22992,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=t1,a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
t4=C_i_vector_ref(((C_word*)t0)[4],C_fix(1));
t5=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t6=C_i_vector_ref(t5,t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22345,a[2]=t6,a[3]=((C_word)li300),tmp=(C_word)a,a+=4,tmp);
t8=(
  f_22345(t7,C_fix(0))
);
if(C_truep(C_fixnum_greaterp(t8,t4))){
t9=C_i_vector_set(((C_word*)t0)[4],C_fix(1),t8);
t10=t3;
f_22992(t10,t8);}
else{
t9=t3;
f_22992(t9,t8);}}

/* k22990 in g4342 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_22992(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,4)))){
C_save_and_reclaim_args((void *)trf_22992,2,t0,t1);}
a=C_alloc(21);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t3=C_i_vector_ref(t2,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22098,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=t1,a[10]=t3,a[11]=((C_word*)t0)[3],a[12]=t2,tmp=(C_word)a,a+=13,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22163,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t6,a[5]=((C_word)li299),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_22163(t8,t4,((C_word*)t0)[10],C_SCHEME_END_OF_LIST);}

/* k23014 in lp2 in lp in k22932 in k22928 in k22924 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23016(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_23016,2,t0,t1);}
a=C_alloc(6);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[5]);
/* irregex-core.scm:3034: lp2 */
t5=((C_word*)((C_word*)t0)[6])[1];
f_22959(t5,((C_word*)t0)[7],t2,t4);}

/* a23043 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23044,c,av);}
a=C_alloc(6);
t5=C_a_i_cons(&a,2,t2,t3);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_cons(&a,2,t5,t4);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23054(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,5)))){
C_save_and_reclaim_args((void *)trf_23054,3,t1,t2,t3);}
a=C_alloc(13);
t4=C_i_vector_ref(t3,C_fix(2));
t5=C_fixnum_times(t4,C_fix(4));
t6=C_fixnum_plus(t5,C_fix(3));
t7=C_i_vector_ref(t2,t6);
t8=C_i_assoc(t3,t7);
t9=(C_truep(t8)?C_i_cdr(t8):C_SCHEME_FALSE);
if(C_truep(t9)){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23064,a[2]=t3,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22926,a[2]=t2,a[3]=t10,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23044,a[2]=((C_word)li304),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3003: mst-fold */
f_22287(t11,t3,t12,C_SCHEME_END_OF_LIST);}}

/* k23062 in chicken.irregex#nfa-epsilon-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23064(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23064,c,av);}
a=C_alloc(6);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=C_fixnum_times(t2,C_fix(4));
t4=C_fixnum_plus(t3,C_fix(3));
t5=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t6=C_i_vector_ref(((C_word*)t0)[3],t4);
t7=C_a_i_cons(&a,2,t5,t6);
t8=C_i_vector_set(((C_word*)t0)[3],t4,t7);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23069(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_23069,5,t1,t2,t3,t4,t5);}
a=C_alloc(10);
t6=C_i_vector_ref(t2,C_fix(0));
t7=C_i_vector_ref(t4,C_fix(0));
t8=C_i_vector_ref(t3,C_fix(0));
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23084,a[2]=t6,a[3]=t7,a[4]=t8,a[5]=t10,a[6]=t5,a[7]=((C_word)li309),tmp=(C_word)a,a+=8,tmp));
t12=((C_word*)t10)[1];
f_23084(t12,t1,C_fix(0),C_SCHEME_END_OF_LIST);}

/* lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23084(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_23084,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_vector_ref(((C_word*)t0)[3],t2);
t6=C_i_vector_ref(((C_word*)t0)[4],t2);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23104,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t8,a[5]=((C_word*)t0)[6],a[6]=((C_word)li308),tmp=(C_word)a,a+=7,tmp));
t10=((C_word*)t8)[1];
f_23104(t10,t1,t5,t6,t3);}}

/* lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23104(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,3)))){
C_save_and_reclaim_args((void *)trf_23104,5,t0,t1,t2,t3,t4);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
/* irregex-core.scm:3055: lp */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23084(t5,t1,C_fixnum_plus(((C_word*)t0)[3],C_fix(1)),t4);}
else{
t5=C_i_car(t2);
t6=C_i_memq(t5,t3);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23127,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t6)){
t8=t7;
f_23127(t8,t6);}
else{
t8=C_i_nullp(((C_word*)t0)[5]);
if(C_truep(C_i_not(t8))){
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23159,a[2]=t7,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23192,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word)li307),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3062: any */
f_9294(t9,t10,((C_word*)t0)[5]);}
else{
t9=t7;
f_23127(t9,C_SCHEME_FALSE);}}}}

/* k23125 in lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23127(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_23127,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
/* irregex-core.scm:3072: lp2 */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23104(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6]);}
else{
t2=C_u_i_cdr(((C_word*)t0)[4]);
t3=C_u_i_car(((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,((C_word*)t0)[7],t3);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[6]);
/* irregex-core.scm:3073: lp2 */
t6=((C_word*)((C_word*)t0)[2])[1];
f_23104(t6,((C_word*)t0)[3],t2,((C_word*)t0)[5],t5);}}

/* k23157 in lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_23159,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23166,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23168,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word)li306),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3068: any */
f_9294(t2,t3,((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[2];
f_23127(t2,C_SCHEME_FALSE);}}

/* k23164 in k23157 in lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23166,c,av);}
t2=((C_word*)t0)[2];
f_23127(t2,C_i_not(t1));}

/* a23167 in k23157 in lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23168,c,av);}
t3=C_i_vector_ref(t2,C_fix(0));
t4=C_eqp(t3,((C_word*)t0)[2]);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_eqp(C_i_vector_ref(t2,C_fix(1)),C_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* a23191 in lp2 in lp in chicken.irregex#tag-set-commands-for-closure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23192,c,av);}
t3=C_i_vector_ref(t2,C_fix(0));
t4=C_eqp(t3,((C_word*)t0)[2]);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_eqp(C_i_vector_ref(t2,C_fix(2)),C_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23236(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,3)))){
C_save_and_reclaim_args((void *)trf_23236,3,t0,t1,t2);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23249,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23371,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=C_i_caar(t2);
t6=C_i_vector_ref(t5,C_fix(2));
t7=C_i_vector_ref(((C_word*)t0)[5],C_fix(2));
t8=C_eqp(t6,t7);
if(C_truep(t8)){
t9=C_i_vector_ref(t5,C_fix(1));
t10=C_i_vector_ref(((C_word*)t0)[5],C_fix(1));
t11=C_eqp(t9,t10);
if(C_truep(t11)){
t12=C_i_vector_length(t5);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22234,a[2]=t12,a[3]=t5,a[4]=((C_word*)t0)[5],a[5]=t14,a[6]=((C_word)li291),tmp=(C_word)a,a+=7,tmp));
t16=((C_word*)t14)[1];
f_22234(t16,t4,C_fix(3));}
else{
t12=t3;
f_23249(t12,C_SCHEME_TRUE);}}
else{
t9=t3;
f_23249(t9,C_SCHEME_TRUE);}}}

/* k23247 in lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23249(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,5)))){
C_save_and_reclaim_args((void *)trf_23249,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
/* irregex-core.scm:3090: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23236(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=C_i_caar(((C_word*)t0)[4]);
t3=C_i_vector_ref(t2,C_fix(0));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23263,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=t5,a[6]=((C_word*)t0)[2],a[7]=((C_word)li290),tmp=(C_word)a,a+=8,tmp));
t7=((C_word*)t5)[1];
f_23263(t7,((C_word*)t0)[3],t3,C_fix(0),C_SCHEME_END_OF_LIST);}}

/* lp2 in k23247 in lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23263(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,5)))){
C_save_and_reclaim_args((void *)trf_23263,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t5)){
t6=C_i_caar(((C_word*)t0)[3]);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_a_i_cons(&a,2,t6,t4);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_i_vector_ref(((C_word*)t0)[4],t3);
t7=C_i_vector_ref(t2,t3);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_23290,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[3],a[7]=t9,a[8]=((C_word)li289),tmp=(C_word)a,a+=9,tmp));
t11=((C_word*)t9)[1];
f_23290(t11,t1,t6,t7,t4);}}

/* lp3 in lp2 in k23247 in lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_23290(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_23290,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
if(C_truep(C_i_nullp(t3))){
/* irregex-core.scm:3100: lp2 */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23263(t5,t1,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),t4);}
else{
/* irregex-core.scm:3101: lp */
t5=((C_word*)((C_word*)t0)[5])[1];
f_23236(t5,t1,C_i_cdr(((C_word*)t0)[6]));}}
else{
if(C_truep(C_i_nullp(t3))){
/* irregex-core.scm:3102: lp */
t5=((C_word*)((C_word*)t0)[5])[1];
f_23236(t5,t1,C_i_cdr(((C_word*)t0)[6]));}
else{
t5=C_i_cdr(t2);
t6=C_i_cdr(t3);
t7=C_u_i_car(t2);
t8=C_u_i_car(t3);
t9=C_eqp(t7,t8);
if(C_truep(t9)){
/* irregex-core.scm:3103: lp3 */
t15=t1;
t16=t5;
t17=t6;
t18=t4;
t1=t15;
t2=t16;
t3=t17;
t4=t18;
goto loop;}
else{
t10=C_u_i_car(t2);
t11=C_u_i_car(t3);
t12=C_a_i_vector3(&a,3,((C_word*)t0)[4],t10,t11);
t13=C_a_i_cons(&a,2,t12,t4);
/* irregex-core.scm:3103: lp3 */
t15=t1;
t16=t5;
t17=t6;
t18=t13;
t1=t15;
t2=t16;
t3=t17;
t4=t18;
goto loop;}}}}

/* k23369 in lp in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23371,c,av);}
t2=((C_word*)t0)[2];
f_23249(t2,C_i_not(t1));}

/* k23385 in k22483 in lp2 in k22451 in lp in k22383 in k22380 in k22377 in k22374 in chicken.irregex#nfa->dfa in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_23387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_23387,c,av);}
a=C_alloc(6);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=C_fixnum_times(t2,C_fix(4));
t4=C_fixnum_plus(t3,C_fix(2));
t5=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t6=C_i_vector_ref(((C_word*)t0)[3],t4);
t7=C_a_i_cons(&a,2,t5,t6);
t8=C_i_vector_set(((C_word*)t0)[3],t4,t7);
t9=((C_word*)t0)[4];
f_22488(t9,t1);}

/* k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,10)))){
C_save_and_reclaim((void *)f_23397,c,av);}
a=C_alloc(10);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=C_i_pairp(((C_word*)t0)[3]);
t4=(C_truep(t3)?C_u_i_car(((C_word*)t0)[3]):C_fix(0));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23406,a[2]=((C_word)li118),tmp=(C_word)a,a+=3,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23446,a[2]=t7,a[3]=((C_word*)t0)[2],a[4]=((C_word)li185),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_23446(t9,((C_word*)t0)[4],((C_word*)t0)[5],C_fix(1),t4,t5);}

/* a23405 in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23406,c,av);}
t10=C_slot(t8,C_fix(1));
t11=C_i_car(t3);
t12=C_i_vector_set(t10,C_fix(0),t11);
t13=C_slot(t8,C_fix(1));
t14=C_i_vector_set(t13,C_fix(1),C_u_i_cdr(t3));
t15=C_slot(t8,C_fix(1));
t16=C_i_vector_set(t15,C_fix(2),t4);
t17=C_slot(t8,C_fix(1));
t18=C_i_vector_set(t17,C_fix(3),t6);
t19=C_i_setslot(t8,C_fix(4),t9);
t20=t1;{
C_word *av2=av;
av2[0]=t20;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t20+1)))(2,av2);}}

/* lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_23446(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,10)))){
C_save_and_reclaim_args((void *)trf_23446,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(18);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23449,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,a[5]=t5,a[6]=((C_word)li119),tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(t2))){
if(C_truep(C_i_stringp(C_u_i_car(t2)))){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23472,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t8=C_u_i_car(t2);
t9=C_fixnum_and(t4,C_fix(2));
t10=C_eqp(C_fix(2),t9);
/* irregex-core.scm:3143: sre->cset */
f_26492(t7,t8,C_a_i_list(&a,1,t10));}
else{
t7=C_u_i_car(t2);
t8=C_eqp(t7,lf[179]);
t9=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_23488,a[2]=t1,a[3]=t5,a[4]=t4,a[5]=t2,a[6]=t7,a[7]=t6,a[8]=((C_word*)t0)[2],a[9]=t3,a[10]=((C_word*)t0)[3],tmp=(C_word)a,a+=11,tmp);
if(C_truep(t8)){
t10=t9;
f_23488(t10,t8);}
else{
t10=C_eqp(t7,lf[247]);
if(C_truep(t10)){
t11=t9;
f_23488(t11,t10);}
else{
t11=C_eqp(t7,lf[234]);
t12=t9;
f_23488(t12,(C_truep(t11)?t11:C_eqp(t7,lf[52])));}}}}
else{
if(C_truep(C_i_symbolp(t2))){
t7=C_eqp(t2,lf[124]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25281,a[2]=t5,a[3]=((C_word)li172),tmp=(C_word)a,a+=4,tmp);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_eqp(t2,lf[125]);
if(C_truep(t8)){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25339,a[2]=t5,a[3]=((C_word)li173),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_eqp(t2,lf[186]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25431,a[2]=t5,a[3]=((C_word)li174),tmp=(C_word)a,a+=4,tmp);
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t10=C_eqp(t2,lf[195]);
if(C_truep(t10)){
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25461,a[2]=t5,a[3]=((C_word)li175),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t11=C_eqp(t2,lf[183]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25512,a[2]=t5,a[3]=((C_word)li176),tmp=(C_word)a,a+=4,tmp);
t13=t1;{
C_word av2[2];
av2[0]=t13;
av2[1]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t12=C_eqp(t2,lf[187]);
if(C_truep(t12)){
t13=t1;{
C_word av2[2];
av2[0]=t13;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25619,a[2]=t5,a[3]=((C_word)li177),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t13=C_eqp(t2,lf[196]);
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25652,a[2]=t5,a[3]=((C_word)li178),tmp=(C_word)a,a+=4,tmp);
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t14=C_eqp(t2,lf[184]);
if(C_truep(t14)){
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25717,a[2]=t5,a[3]=((C_word)li179),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t15=C_eqp(t2,lf[185]);
if(C_truep(t15)){
t16=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25806,a[2]=t5,a[3]=((C_word)li180),tmp=(C_word)a,a+=4,tmp);
t17=t1;{
C_word av2[2];
av2[0]=t17;
av2[1]=t16;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
t16=C_eqp(t2,lf[110]);
if(C_truep(t16)){
t17=t1;{
C_word av2[2];
av2[0]=t17;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
t17=C_i_assq(t2,lf[248]);
if(C_truep(t17)){
/* irregex-core.scm:3584: rec */
t18=t6;
f_23449(t18,t1,C_i_cdr(t17));}
else{
/* irregex-core.scm:3585: chicken.base#error */
t18=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t18;
av2[1]=t1;
av2[2]=lf[249];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t18+1)))(4,av2);}}}}}}}}}}}}}
else{
if(C_truep(C_charp(t2))){
t7=C_fixnum_and(t4,C_fix(2));
t8=C_eqp(C_fix(2),t7);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(C_truep(t8)?(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25928,a[2]=t5,a[3]=t2,a[4]=((C_word)li182),tmp=(C_word)a,a+=5,tmp):(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26027,a[2]=t2,a[3]=t5,a[4]=((C_word)li184),tmp=(C_word)a,a+=5,tmp));
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
if(C_truep(C_i_stringp(t2))){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26142,a[2]=t6,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* ##sys#string->list */
t8=*((C_word*)lf[105]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
/* irregex-core.scm:3638: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[251];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}}}}

/* rec in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_23449(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_23449,3,t0,t1,t2);}
/* irregex-core.scm:3138: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23446(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k23470 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23472(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23472,c,av);}
/* irregex-core.scm:3142: sre-cset->procedure */
f_26168(((C_word*)t0)[2],t1,((C_word*)t0)[3]);}

/* k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_23488(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(39,0,10)))){
C_save_and_reclaim_args((void *)trf_23488,2,t0,t1);}
a=C_alloc(39);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23495,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_fixnum_and(((C_word*)t0)[4],C_fix(2));
t4=C_eqp(C_fix(2),t3);
/* irregex-core.scm:3148: sre->cset */
f_26492(t2,((C_word*)t0)[5],C_a_i_list(&a,1,t4));}
else{
t2=C_eqp(((C_word*)t0)[6],lf[103]);
if(C_truep(t2)){
t3=C_i_length(C_u_i_cdr(((C_word*)t0)[5]));
switch(t3){
case C_fix(0):
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23515,a[2]=((C_word)li120),tmp=(C_word)a,a+=3,tmp);
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}
case C_fix(1):
/* irregex-core.scm:3153: rec */
t4=((C_word*)t0)[7];
f_23449(t4,((C_word*)t0)[2],C_i_cadr(((C_word*)t0)[5]));
default:
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23535,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3155: rec */
t5=((C_word*)t0)[7];
f_23449(t5,t4,C_i_cadr(((C_word*)t0)[5]));}}
else{
t3=C_eqp(((C_word*)t0)[6],lf[225]);
if(C_truep(t3)){
t4=(
/* irregex-core.scm:3165: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t5=C_fixnum_not(C_fix(2));
/* irregex-core.scm:3165: lp */
t6=((C_word*)((C_word*)t0)[8])[1];
f_23446(t6,((C_word*)t0)[2],t4,((C_word*)t0)[9],C_fixnum_and(((C_word*)t0)[4],t5),((C_word*)t0)[3]);}
else{
t4=C_eqp(((C_word*)t0)[6],lf[226]);
if(C_truep(t4)){
t5=(
/* irregex-core.scm:3170: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3170: lp */
t6=((C_word*)((C_word*)t0)[8])[1];
f_23446(t6,((C_word*)t0)[2],t5,((C_word*)t0)[9],C_fixnum_or(((C_word*)t0)[4],C_fix(2)),((C_word*)t0)[3]);}
else{
t5=C_eqp(((C_word*)t0)[6],lf[156]);
if(C_truep(t5)){
t6=(
/* irregex-core.scm:3175: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3175: lp */
t7=((C_word*)((C_word*)t0)[8])[1];
f_23446(t7,((C_word*)t0)[2],t6,((C_word*)t0)[9],C_fixnum_or(((C_word*)t0)[4],C_fix(32)),((C_word*)t0)[3]);}
else{
t6=C_eqp(((C_word*)t0)[6],lf[157]);
if(C_truep(t6)){
t7=(
/* irregex-core.scm:3177: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t8=C_fixnum_not(C_fix(32));
/* irregex-core.scm:3177: lp */
t9=((C_word*)((C_word*)t0)[8])[1];
f_23446(t9,((C_word*)t0)[2],t7,((C_word*)t0)[9],C_fixnum_and(((C_word*)t0)[4],t8),((C_word*)t0)[3]);}
else{
t7=C_eqp(((C_word*)t0)[6],lf[115]);
t8=(C_truep(t7)?t7:C_eqp(((C_word*)t0)[6],lf[227]));
if(C_truep(t8)){
t9=C_i_length(C_u_i_cdr(((C_word*)t0)[5]));
switch(t9){
case C_fix(0):
t10=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t10;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}
case C_fix(1):
/* irregex-core.scm:3181: rec */
t10=((C_word*)t0)[7];
f_23449(t10,((C_word*)t0)[2],C_i_cadr(((C_word*)t0)[5]));
default:
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23689,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t11=(
/* irregex-core.scm:3183: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cddr(((C_word*)t0)[5]))
);
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23708,a[2]=((C_word*)t0)[8],a[3]=t10,a[4]=t11,a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3184: sre-count-submatches */
f_16982(t12,C_i_cadr(((C_word*)t0)[5]));}}
else{
t9=C_eqp(((C_word*)t0)[6],lf[131]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23731,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t11=(
/* irregex-core.scm:3189: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3189: rec */
t12=((C_word*)t0)[7];
f_23449(t12,t10,t11);}
else{
t10=C_eqp(((C_word*)t0)[6],lf[132]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23757,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t12=(
/* irregex-core.scm:3194: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3194: rec */
t13=((C_word*)t0)[7];
f_23449(t13,t11,t12);}
else{
t11=C_eqp(((C_word*)t0)[6],lf[127]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23786,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t13=(
/* irregex-core.scm:3200: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3200: sre-empty? */
t14=lf[139];{
C_word av2[3];
av2[0]=t14;
av2[1]=t12;
av2[2]=t13;
f_16307(3,av2);}}
else{
t12=C_eqp(((C_word*)t0)[6],lf[128]);
if(C_truep(t12)){
t13=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23831,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
t14=(
/* irregex-core.scm:3210: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3210: sre-empty? */
t15=lf[139];{
C_word av2[3];
av2[0]=t15;
av2[1]=t13;
av2[2]=t14;
f_16307(3,av2);}}
else{
t13=C_eqp(((C_word*)t0)[6],lf[129]);
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23885,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
t15=(
/* irregex-core.scm:3229: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
/* irregex-core.scm:3229: sre-empty? */
t16=lf[139];{
C_word av2[3];
av2[0]=t16;
av2[1]=t14;
av2[2]=t15;
f_16307(3,av2);}}
else{
t14=C_eqp(((C_word*)t0)[6],lf[134]);
if(C_truep(t14)){
t15=C_i_cadr(((C_word*)t0)[5]);
t16=C_u_i_cdr(((C_word*)t0)[5]);
t17=C_u_i_car(t16);
t18=C_u_i_cdr(((C_word*)t0)[5]);
t19=C_u_i_cdr(t18);
t20=C_a_i_cons(&a,2,t17,t19);
t21=C_a_i_cons(&a,2,t15,t20);
t22=C_a_i_cons(&a,2,lf[133],t21);
/* irregex-core.scm:3244: rec */
t23=((C_word*)t0)[7];
f_23449(t23,((C_word*)t0)[2],t22);}
else{
t15=C_eqp(((C_word*)t0)[6],lf[135]);
if(C_truep(t15)){
t16=C_i_cadr(((C_word*)t0)[5]);
t17=C_u_i_cdr(((C_word*)t0)[5]);
t18=C_u_i_cdr(t17);
t19=C_a_i_cons(&a,2,C_SCHEME_FALSE,t18);
t20=C_a_i_cons(&a,2,t16,t19);
t21=C_a_i_cons(&a,2,lf[133],t20);
/* irregex-core.scm:3246: rec */
t22=((C_word*)t0)[7];
f_23449(t22,((C_word*)t0)[2],t21);}
else{
t16=C_eqp(((C_word*)t0)[6],lf[133]);
if(C_truep(t16)){
t17=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23987,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
t18=C_i_cadr(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t18))){
t19=C_i_caddr(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t19))){
t20=C_u_i_cdr(((C_word*)t0)[5]);
t21=C_u_i_car(t20);
t22=C_u_i_cdr(((C_word*)t0)[5]);
t23=C_u_i_cdr(t22);
t24=t17;
f_23987(t24,C_fixnum_greaterp(t21,C_u_i_car(t23)));}
else{
t20=t17;
f_23987(t20,C_SCHEME_FALSE);}}
else{
t19=t17;
f_23987(t19,C_SCHEME_FALSE);}}
else{
t17=C_eqp(((C_word*)t0)[6],lf[130]);
if(C_truep(t17)){
t18=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24139,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
t19=C_i_cadr(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t19))){
t20=C_i_caddr(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t20))){
t21=C_u_i_cdr(((C_word*)t0)[5]);
t22=C_u_i_car(t21);
t23=C_u_i_cdr(((C_word*)t0)[5]);
t24=C_u_i_cdr(t23);
t25=t18;
f_24139(t25,C_fixnum_greaterp(t22,C_u_i_car(t24)));}
else{
t21=t18;
f_24139(t21,C_SCHEME_FALSE);}}
else{
t20=t18;
f_24139(t20,C_SCHEME_FALSE);}}
else{
t18=C_eqp(((C_word*)t0)[6],lf[231]);
if(C_truep(t18)){
t19=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24310,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t20=C_u_i_cdr(((C_word*)t0)[5]);
t21=C_a_i_list(&a,1,lf[184]);
/* irregex-core.scm:3317: ##sys#append */
t22=*((C_word*)lf[118]+1);{
C_word av2[4];
av2[0]=t22;
av2[1]=t19;
av2[2]=t20;
av2[3]=t21;
((C_proc)(void*)(*((C_word*)t22+1)))(4,av2);}}
else{
t19=C_eqp(((C_word*)t0)[6],lf[232]);
if(C_truep(t19)){
t20=C_a_i_list(&a,3,lf[103],lf[181],lf[233]);
t21=C_u_i_cdr(((C_word*)t0)[5]);
t22=C_a_i_cons(&a,2,lf[103],t21);
t23=C_a_i_list(&a,3,lf[234],t20,t22);
t24=C_a_i_list(&a,2,lf[129],t23);
t25=C_a_i_list(&a,4,lf[115],lf[183],t24,lf[184]);
/* irregex-core.scm:3319: rec */
t26=((C_word*)t0)[7];
f_23449(t26,((C_word*)t0)[2],t25);}
else{
t20=C_eqp(((C_word*)t0)[6],lf[235]);
if(C_truep(t20)){
t21=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24360,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3322: string->sre */
t22=*((C_word*)lf[102]+1);{
C_word av2[3];
av2[0]=t22;
av2[1]=t21;
av2[2]=C_i_cadr(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t22+1)))(3,av2);}}
else{
t21=C_eqp(((C_word*)t0)[6],lf[111]);
if(C_truep(t21)){
t22=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24373,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t23=(
/* irregex-core.scm:3325: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t24=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24396,a[2]=((C_word)li149),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3325: lp */
t25=((C_word*)((C_word*)t0)[8])[1];
f_23446(t25,t22,t23,((C_word*)t0)[9],((C_word*)t0)[4],t24);}
else{
t22=C_eqp(((C_word*)t0)[6],lf[112]);
if(C_truep(t22)){
t23=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24408,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t24=(
/* irregex-core.scm:3335: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t25=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24431,a[2]=((C_word)li152),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3335: lp */
t26=((C_word*)((C_word*)t0)[8])[1];
f_23446(t26,t23,t24,((C_word*)t0)[9],((C_word*)t0)[4],t25);}
else{
t23=C_eqp(((C_word*)t0)[6],lf[113]);
t24=(C_truep(t23)?t23:C_eqp(((C_word*)t0)[6],lf[114]));
if(C_truep(t24)){
t25=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24446,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t26=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24518,a[2]=((C_word*)t0)[8],a[3]=t25,a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3346: scheme#append */
t27=*((C_word*)lf[73]+1);{
C_word av2[4];
av2[0]=t27;
av2[1]=t26;
av2[2]=C_u_i_cdr(((C_word*)t0)[5]);
av2[3]=lf[238];
((C_proc)(void*)(*((C_word*)t27+1)))(4,av2);}}
else{
t25=C_eqp(((C_word*)t0)[6],lf[239]);
if(C_truep(t25)){
t26=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24529,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t27=(
/* irregex-core.scm:3362: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t28=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24552,a[2]=((C_word)li161),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3362: lp */
t29=((C_word*)((C_word*)t0)[8])[1];
f_23446(t29,t26,t27,((C_word*)t0)[9],((C_word*)t0)[4],t28);}
else{
t26=C_eqp(((C_word*)t0)[6],lf[109]);
if(C_truep(t26)){
t27=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24564,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3372: sre-count-submatches */
f_16982(t27,C_i_cadr(((C_word*)t0)[5]));}
else{
t27=C_eqp(((C_word*)t0)[6],lf[192]);
t28=(C_truep(t27)?t27:C_eqp(((C_word*)t0)[6],lf[191]));
if(C_truep(t28)){
t29=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24720,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t30=C_i_cadr(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t30))){
t31=t29;{
C_word av2[2];
av2[0]=t31;
av2[1]=C_u_i_car(C_u_i_cdr(((C_word*)t0)[5]));
f_24720(2,av2);}}
else{
t31=C_u_i_cdr(((C_word*)t0)[5]);
t32=C_i_assq(C_u_i_car(t31),((C_word*)((C_word*)t0)[10])[1]);
if(C_truep(t32)){
t33=t29;{
C_word av2[2];
av2[0]=t33;
av2[1]=C_i_cdr(t32);
f_24720(2,av2);}}
else{
t33=C_u_i_cdr(((C_word*)t0)[5]);
/* irregex-core.scm:3404: chicken.base#error */
t34=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t34;
av2[1]=t29;
av2[2]=lf[243];
av2[3]=C_u_i_car(t33);
((C_proc)(void*)(*((C_word*)t34+1)))(4,av2);}}}}
else{
t29=C_eqp(((C_word*)t0)[6],lf[244]);
if(C_truep(t29)){
t30=(
/* irregex-core.scm:3444: sre-sequence */
  f_18191(C_a_i(&a,3),C_i_cdddr(((C_word*)t0)[5]))
);
t31=C_i_cadr(((C_word*)t0)[5]);
/* irregex-core.scm:3444: lp */
t32=((C_word*)((C_word*)t0)[8])[1];
f_23446(t32,((C_word*)t0)[2],t30,C_fixnum_plus(((C_word*)t0)[9],t31),((C_word*)t0)[4],((C_word*)t0)[3]);}
else{
t30=C_eqp(((C_word*)t0)[6],lf[245]);
t31=(C_truep(t30)?t30:C_eqp(((C_word*)t0)[6],lf[108]));
if(C_truep(t31)){
t32=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24937,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t33=(
/* irregex-core.scm:3447: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[5]))
);
t34=C_fixnum_plus(((C_word*)t0)[9],C_fix(1));
t35=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25046,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[3],a[4]=((C_word)li171),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3447: lp */
t36=((C_word*)((C_word*)t0)[8])[1];
f_23446(t36,t32,t33,t34,((C_word*)t0)[4],t35);}
else{
t32=C_eqp(((C_word*)t0)[6],lf[119]);
t33=(C_truep(t32)?t32:C_eqp(((C_word*)t0)[6],lf[120]));
if(C_truep(t33)){
t34=C_i_cddr(((C_word*)t0)[5]);
t35=C_a_i_cons(&a,2,lf[108],t34);
/* irregex-core.scm:3477: rec */
t36=((C_word*)t0)[7];
f_23449(t36,((C_word*)t0)[2],t35);}
else{
/* irregex-core.scm:3479: chicken.base#error */
t34=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t34;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[246];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t34+1)))(4,av2);}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k23493 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23495,c,av);}
/* irregex-core.scm:3147: sre-cset->procedure */
f_26168(((C_word*)t0)[2],t1,((C_word*)t0)[3]);}

/* f_23515 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23515,c,av);}
/* irregex-core.scm:3152: fail */
t10=t9;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k23533 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_23535,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23538,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(
/* irregex-core.scm:3156: sre-alternate */
  f_18214(C_a_i(&a,3),C_i_cddr(((C_word*)t0)[3]))
);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23561,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3157: sre-count-submatches */
f_16982(t4,C_i_cadr(((C_word*)t0)[3]));}

/* k23536 in k23533 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_23538,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23539,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word)li122),tmp=(C_word)a,a+=5,tmp);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_23539 in k23536 in k23533 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23539,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23545,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li121),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3161: first */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23544 */
static void C_ccall f_23545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23545,c,av);}
/* irregex-core.scm:3163: rest */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23559 in k23533 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_23561,c,av);}
/* irregex-core.scm:3156: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23446(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_plus(((C_word*)t0)[5],t1),((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k23687 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_23689,c,av);}
/* irregex-core.scm:3187: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23446(t2,((C_word*)t0)[3],C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],t1);}

/* k23706 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_23708,c,av);}
/* irregex-core.scm:3183: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23446(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_plus(((C_word*)t0)[5],t1),((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k23729 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_23731,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23732,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word)li124),tmp=(C_word)a,a+=5,tmp);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_23732 in k23729 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23732,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23738,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li123),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3191: body */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23737 */
static void C_ccall f_23738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23738,c,av);}
/* irregex-core.scm:3192: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23755 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23757(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_23757,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23758,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word)li126),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_23758 in k23755 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23758,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23764,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li125),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3196: next */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23763 */
static void C_ccall f_23764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23764,c,av);}
/* irregex-core.scm:3197: body */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23784 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_23786,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
/* irregex-core.scm:3201: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[228];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23792,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(
/* irregex-core.scm:3203: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[3]))
);
t4=C_a_i_list2(&a,2,lf[129],t3);
/* irregex-core.scm:3203: rec */
t5=((C_word*)t0)[5];
f_23449(t5,t2,t4);}}

/* k23790 in k23784 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_23792,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23793,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li128),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_23793 in k23790 in k23784 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23793,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23799,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li127),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3205: body */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23798 */
static void C_ccall f_23799(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23799,c,av);}
/* irregex-core.scm:3207: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23829 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,10)))){
C_save_and_reclaim((void *)f_23831,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
/* irregex-core.scm:3211: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[229];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23837,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=(
/* irregex-core.scm:3215: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[3]))
);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23855,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word)li132),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3215: lp */
t7=((C_word*)((C_word*)t0)[5])[1];
f_23446(t7,t4,t5,((C_word*)t0)[6],((C_word*)t0)[7],t6);}}

/* k23835 in k23829 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_23837,c,av);}
a=C_alloc(5);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23839,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word)li130),tmp=(C_word)a,a+=5,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_23839 in k23835 in k23829 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23839(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23839,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23845,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li129),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3224: next */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23844 */
static void C_ccall f_23845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23845,c,av);}
/* irregex-core.scm:3226: body */
t2=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* a23854 in k23829 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23855,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23861,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li131),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3219: next */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23860 in a23854 in k23829 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23861,c,av);}
/* irregex-core.scm:3221: body */
t2=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23883 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_23885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,10)))){
C_save_and_reclaim((void *)f_23885,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
/* irregex-core.scm:3230: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[230];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23891,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t5=(
/* irregex-core.scm:3234: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(((C_word*)t0)[3]))
);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23898,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word)li134),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3234: lp */
t7=((C_word*)((C_word*)t0)[5])[1];
f_23446(t7,t4,t5,((C_word*)t0)[6],((C_word*)t0)[7],t6);}}

/* k23889 in k23883 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23891,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)((C_word*)t0)[2])[1];
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a23897 in k23883 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_23898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_23898,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23904,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li133),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3238: body */
t11=((C_word*)((C_word*)t0)[3])[1];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a23903 in a23897 in k23883 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_23904,c,av);}
/* irregex-core.scm:3240: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_23987(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_23987,2,t0,t1);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23990,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t1)){
t3=t2;
f_23990(t3,t1);}
else{
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=C_i_not(C_u_i_car(t3));
t5=t2;
f_23990(t5,(C_truep(t4)?C_i_caddr(((C_word*)t0)[3]):C_SCHEME_FALSE));}}

/* k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_fcall f_23990(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,0,10)))){
C_save_and_reclaim_args((void *)trf_23990,2,t0,t1);}
a=C_alloc(28);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23991,a[2]=((C_word)li135),tmp=(C_word)a,a+=3,tmp);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_u_i_cdr(((C_word*)t0)[3]);
t11=C_u_i_car(t10);
t12=C_i_caddr(((C_word*)t0)[3]);
t13=C_u_i_cdr(((C_word*)t0)[3]);
t14=C_u_i_cdr(t13);
t15=(
/* irregex-core.scm:3258: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t14))
);
t16=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24004,a[2]=t5,a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=t9,a[6]=((C_word*)t0)[5],a[7]=t7,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word)li138),tmp=(C_word)a,a+=11,tmp);
t17=C_set_block_item(t3,0,t11);
t18=C_set_block_item(t5,0,t12);
t19=C_set_block_item(t7,0,t15);
t20=C_set_block_item(t9,0,t16);
t21=(C_truep(C_i_zerop(((C_word*)t3)[1]))?(C_truep(((C_word*)t5)[1])?C_i_zerop(((C_word*)t5)[1]):C_SCHEME_FALSE):C_SCHEME_FALSE);
if(C_truep(t21)){
t22=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t22;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t22+1)))(2,av2);}}
else{
t22=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24059,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t9,a[5]=((C_word)li140),tmp=(C_word)a,a+=6,tmp);
t23=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t23;
av2[1]=t22;
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}}}

/* f_23991 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_23991(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23991,c,av);}
/* irregex-core.scm:3253: fail */
t10=t9;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* body4587 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,10)))){
C_save_and_reclaim((void *)f_24004,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24010,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li137),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3261: lp */
t4=((C_word*)((C_word*)t0)[6])[1];
f_23446(t4,t1,((C_word*)((C_word*)t0)[7])[1],((C_word*)t0)[8],((C_word*)t0)[9],t3);}

/* a24009 in body4587 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in ... */
static void C_ccall f_24010(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,9)))){
C_save_and_reclaim((void *)f_24010,c,av);}
a=C_alloc(14);
t10=(C_truep(((C_word*)((C_word*)t0)[2])[1])?C_eqp(((C_word*)t0)[3],((C_word*)((C_word*)t0)[2])[1]):C_SCHEME_FALSE);
if(C_truep(t10)){
/* irregex-core.scm:3266: next */
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}
else{
t11=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_24023,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t6,a[10]=t7,a[11]=t8,a[12]=t9,a[13]=t1,tmp=(C_word)a,a+=14,tmp);
/* irregex-core.scm:3267: body */
t12=((C_word*)((C_word*)t0)[6])[1];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=C_fixnum_plus(C_fix(1),((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}

/* k24021 in a24009 in body4587 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_24023(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,9)))){
C_save_and_reclaim((void *)f_24023,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_24028,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word)li136),tmp=(C_word)a,a+=14,tmp);
/* irregex-core.scm:3265: g4606 */
t3=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[13];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=((C_word*)t0)[9];
av2[7]=((C_word*)t0)[10];
av2[8]=((C_word*)t0)[11];
av2[9]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}

/* a24027 in k24021 in a24009 in body4587 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in ... */
static void C_ccall f_24028(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24028,c,av);}
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]))){
/* irregex-core.scm:3271: next */
t2=((C_word*)t0)[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=((C_word*)t0)[9];
av2[7]=((C_word*)t0)[10];
av2[8]=((C_word*)t0)[11];
av2[9]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3272: fail */
t2=((C_word*)t0)[12];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* f_24059 in k23988 in k23985 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_24059,c,av);}
a=C_alloc(13);
t10=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_24063,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,a[12]=t1,tmp=(C_word)a,a+=13,tmp);
/* irregex-core.scm:3276: body */
t11=((C_word*)((C_word*)t0)[4])[1];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}

/* k24061 */
static void C_ccall f_24063(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,9)))){
C_save_and_reclaim((void *)f_24063,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_24068,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word)li139),tmp=(C_word)a,a+=13,tmp);
/* irregex-core.scm:3275: g4618 */
t3=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[12];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}

/* a24067 in k24061 */
static void C_ccall f_24068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24068,c,av);}
if(C_truep(C_i_zerop(((C_word*)((C_word*)t0)[2])[1]))){
/* irregex-core.scm:3279: next */
t2=((C_word*)t0)[3];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3280: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_fcall f_24139(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_24139,2,t0,t1);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24142,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t1)){
t3=t2;
f_24142(t3,t1);}
else{
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=C_i_not(C_u_i_car(t3));
t5=t2;
f_24142(t5,(C_truep(t4)?C_i_caddr(((C_word*)t0)[3]):C_SCHEME_FALSE));}}

/* k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_fcall f_24142(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,0,10)))){
C_save_and_reclaim_args((void *)trf_24142,2,t0,t1);}
a=C_alloc(28);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24143,a[2]=((C_word)li141),tmp=(C_word)a,a+=3,tmp);
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_u_i_cdr(((C_word*)t0)[3]);
t11=C_u_i_car(t10);
t12=C_i_caddr(((C_word*)t0)[3]);
t13=C_u_i_cdr(((C_word*)t0)[3]);
t14=C_u_i_cdr(t13);
t15=(
/* irregex-core.scm:3292: sre-sequence */
  f_18191(C_a_i(&a,3),C_u_i_cdr(t14))
);
t16=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24156,a[2]=t3,a[3]=t9,a[4]=t5,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t7,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word)li144),tmp=(C_word)a,a+=11,tmp);
t17=C_set_block_item(t3,0,t11);
t18=C_set_block_item(t5,0,t12);
t19=C_set_block_item(t7,0,t15);
t20=C_set_block_item(t9,0,t16);
t21=(C_truep(C_i_zerop(((C_word*)t3)[1]))?(C_truep(((C_word*)t5)[1])?C_i_zerop(((C_word*)t5)[1]):C_SCHEME_FALSE):C_SCHEME_FALSE);
t22=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t22;
av2[1]=(C_truep(t21)?((C_word*)t0)[4]:(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24218,a[2]=t3,a[3]=t9,a[4]=((C_word*)t0)[4],a[5]=((C_word)li146),tmp=(C_word)a,a+=6,tmp));
((C_proc)(void*)(*((C_word*)t22+1)))(2,av2);}}}

/* f_24143 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24143,c,av);}
/* irregex-core.scm:3287: fail */
t10=t9;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* body4645 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24156(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,10)))){
C_save_and_reclaim((void *)f_24156,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24162,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li143),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3295: lp */
t4=((C_word*)((C_word*)t0)[6])[1];
f_23446(t4,t1,((C_word*)((C_word*)t0)[7])[1],((C_word*)t0)[8],((C_word*)t0)[9],t3);}

/* a24161 in body4645 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in ... */
static void C_ccall f_24162(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,9)))){
C_save_and_reclaim((void *)f_24162,c,av);}
a=C_alloc(14);
if(C_truep(C_fixnum_lessp(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]))){
t10=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24172,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3300: body */
t11=((C_word*)((C_word*)t0)[4])[1];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=C_fixnum_plus(C_fix(1),((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_24184,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t9,a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t6,a[10]=t7,a[11]=t8,a[12]=((C_word*)t0)[4],a[13]=((C_word)li142),tmp=(C_word)a,a+=14,tmp);
/* irregex-core.scm:3302: next */
t11=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}}

/* k24170 in a24161 in body4645 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_24172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24172,c,av);}
/* irregex-core.scm:3299: g4663 */
t2=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* a24183 in a24161 in body4645 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_24184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_24184,c,av);}
a=C_alloc(11);
t2=(C_truep(((C_word*)((C_word*)t0)[2])[1])?C_eqp(((C_word*)t0)[3],((C_word*)((C_word*)t0)[2])[1]):C_SCHEME_FALSE);
if(C_truep(t2)){
/* irregex-core.scm:3305: fail */
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24197,a[2]=t1,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[4],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3306: body */
t4=((C_word*)((C_word*)t0)[12])[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fixnum_plus(C_fix(1),((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k24195 in a24183 in a24161 in body4645 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in ... */
static void C_ccall f_24197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24197,c,av);}
/* irregex-core.scm:3304: g4666 */
t2=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* f_24218 in k24140 in k24137 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24218(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_24218,c,av);}
a=C_alloc(12);
if(C_truep(C_i_zerop(((C_word*)((C_word*)t0)[2])[1]))){
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_24228,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t5,a[6]=t6,a[7]=t7,a[8]=t8,a[9]=t9,a[10]=((C_word*)t0)[3],a[11]=((C_word)li145),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3312: next */
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24238,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3315: body */
t11=((C_word*)((C_word*)t0)[3])[1];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}}

/* a24227 */
static void C_ccall f_24228(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_24228,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24232,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3314: body */
t3=((C_word*)((C_word*)t0)[10])[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k24230 in a24227 */
static void C_ccall f_24232(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24232,c,av);}
/* irregex-core.scm:3313: g4678 */
t2=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k24236 */
static void C_ccall f_24238(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24238,c,av);}
/* irregex-core.scm:3311: g4680 */
t2=t1;{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k24308 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24310(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_24310,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,lf[183],t1);
t3=C_a_i_cons(&a,2,lf[115],t2);
/* irregex-core.scm:3317: rec */
t4=((C_word*)t0)[2];
f_23449(t4,((C_word*)t0)[3],t3);}

/* k24358 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24360,c,av);}
/* irregex-core.scm:3322: rec */
t2=((C_word*)t0)[2];
f_23449(t2,((C_word*)t0)[3],t1);}

/* k24371 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_24373,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24374,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word)li148),tmp=(C_word)a,a+=5,tmp);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_24374 in k24371 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,9)))){
C_save_and_reclaim((void *)f_24374,c,av);}
a=C_alloc(15);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_24381,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24389,a[2]=((C_word)li147),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3330: check */
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t10;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(10,av2);}}

/* k24379 */
static void C_ccall f_24381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24381,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:3331: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3332: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a24388 */
static void C_ccall f_24389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24389,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a24395 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24396,c,av);}
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k24406 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_24408,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24409,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li151),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24409 in k24406 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,9)))){
C_save_and_reclaim((void *)f_24409,c,av);}
a=C_alloc(15);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_24416,a[2]=t9,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t6,a[10]=t7,a[11]=t8,tmp=(C_word)a,a+=12,tmp);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24424,a[2]=((C_word)li150),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3340: check */
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t10;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(10,av2);}}

/* k24414 */
static void C_ccall f_24416(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24416,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:3341: fail */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* irregex-core.scm:3342: next */
t2=((C_word*)t0)[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=((C_word*)t0)[9];
av2[7]=((C_word*)t0)[10];
av2[8]=((C_word*)t0)[11];
av2[9]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}}

/* a24423 */
static void C_ccall f_24424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24424,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a24430 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24431,c,av);}
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k24444 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,10)))){
C_save_and_reclaim((void *)f_24446,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24447,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word)li157),tmp=(C_word)a,a+=6,tmp);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_24447 in k24444 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,7)))){
C_save_and_reclaim((void *)f_24447,c,av);}
a=C_alloc(25);
t10=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_24451,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,a[7]=t4,a[8]=t5,a[9]=t6,a[10]=t7,a[11]=t8,a[12]=t9,a[13]=((C_word*)t0)[4],tmp=(C_word)a,a+=14,tmp);
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8736,a[2]=t4,a[3]=t2,a[4]=((C_word)li155),tmp=(C_word)a,a+=5,tmp);
t12=C_i_vector_ref(t2,C_fix(1));
t13=C_i_vector_ref(t2,C_fix(2));
t14=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8763,a[2]=t4,a[3]=t6,a[4]=t2,a[5]=((C_word)li156),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:399: make-irregex-chunker */
t15=*((C_word*)lf[45]+1);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t10;
av2[2]=t11;
av2[3]=t12;
av2[4]=t13;
av2[5]=t14;
av2[6]=C_i_vector_ref(t2,C_fix(4));
av2[7]=C_i_vector_ref(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t15+1)))(8,av2);}}

/* k24449 */
static void C_ccall f_24451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_24451,c,av);}
a=C_alloc(15);
t2=C_i_vector_ref(t1,C_fix(1));
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_24457,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
/* irregex-core.scm:3351: g4748 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k24455 in k24449 */
static void C_ccall f_24457(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_24457,c,av);}
a=C_alloc(17);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_vector_ref(((C_word*)t0)[3],C_fix(3));
t4=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_24464,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[3],a[15]=t1,a[16]=t2,tmp=(C_word)a,a+=17,tmp);
/* irregex-core.scm:3351: g4752 */
t5=t3;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k24462 in k24455 in k24449 */
static void C_ccall f_24464(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,9)))){
C_save_and_reclaim((void *)f_24464,c,av);}
a=C_alloc(22);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_eqp(t2,lf[113]);
t4=(C_truep(t3)?(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24492,a[2]=((C_word)li153),tmp=(C_word)a,a+=3,tmp):*((C_word*)lf[236]+1));
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_24473,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24483,a[2]=t4,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_u_i_car(((C_word*)t0)[6]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24487,a[2]=((C_word)li154),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3356: check */
t9=((C_word*)t0)[13];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t9;
av2[1]=t6;
av2[2]=((C_word*)t0)[14];
av2[3]=((C_word*)t0)[6];
av2[4]=t7;
av2[5]=((C_word*)t0)[15];
av2[6]=((C_word*)t0)[16];
av2[7]=t1;
av2[8]=((C_word*)t0)[11];
av2[9]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(10,av2);}}

/* k24471 in k24462 in k24455 in k24449 */
static void C_ccall f_24473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24473,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:3358: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3359: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24481 in k24462 in k24455 in k24449 */
static void C_ccall f_24483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24483,c,av);}
/* irregex-core.scm:3355: g4754 */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a24486 in k24462 in k24455 in k24449 */
static void C_ccall f_24487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24487,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_24492 in k24462 in k24455 in k24449 */
static void C_ccall f_24492(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24492,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a24508 in k24516 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24509,c,av);}
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k24516 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,10)))){
C_save_and_reclaim((void *)f_24518,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,lf[237],t1);
t3=(
/* irregex-core.scm:3345: sre-sequence */
  f_18191(C_a_i(&a,3),t2)
);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24509,a[2]=((C_word)li158),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3345: lp */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23446(t5,((C_word*)t0)[3],t3,((C_word*)t0)[4],((C_word*)t0)[5],t4);}

/* k24527 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_24529,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24530,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li160),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24530 in k24527 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,9)))){
C_save_and_reclaim((void *)f_24530,c,av);}
a=C_alloc(14);
t10=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24534,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t7,a[9]=t8,a[10]=t9,tmp=(C_word)a,a+=11,tmp);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24545,a[2]=((C_word)li159),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3367: once */
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t10;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(10,av2);}}

/* k24532 */
static void C_ccall f_24534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24534,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:3369: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=t1;
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3370: fail */
t2=((C_word*)t0)[10];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a24544 */
static void C_ccall f_24545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24545,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a24551 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24552(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24552,c,av);}
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_24564,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_24567,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3373: lp */
t3=((C_word*)((C_word*)t0)[5])[1];
f_23446(t3,t2,C_i_caddr(((C_word*)t0)[2]),((C_word*)t0)[7],C_fixnum_plus(((C_word*)t0)[6],t1),((C_word*)t0)[8]);}

/* k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,10)))){
C_save_and_reclaim((void *)f_24567,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24570,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=C_i_cdddr(((C_word*)t0)[2]);
if(C_truep(C_i_pairp(t3))){
t4=C_i_cadddr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24683,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
t6=C_u_i_cdr(((C_word*)t0)[2]);
t7=C_u_i_cdr(t6);
/* irregex-core.scm:3377: sre-count-submatches */
f_16982(t5,C_u_i_car(t7));}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24688,a[2]=((C_word)li165),tmp=(C_word)a,a+=3,tmp);
f_24570(2,av2);}}}

/* k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_24570,c,av);}
a=C_alloc(10);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_numberp(t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_24579,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t3)){
t5=t4;
f_24579(t5,t3);}
else{
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=t4;
f_24579(t6,C_i_symbolp(C_u_i_car(t5)));}}

/* k24577 in k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in ... */
static void C_fcall f_24579(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_24579,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24582,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_u_i_cdr(((C_word*)t0)[5]);
if(C_truep(C_i_symbolp(C_u_i_car(t3)))){
t4=C_u_i_cdr(((C_word*)t0)[5]);
t5=C_i_assq(C_u_i_car(t4),((C_word*)((C_word*)t0)[6])[1]);
if(C_truep(t5)){
t6=t2;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_i_cdr(t5);
f_24582(2,av2);}}
else{
/* irregex-core.scm:3389: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t2;
av2[2]=lf[240];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}
else{
t4=t2;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_cadr(((C_word*)t0)[5]);
f_24582(2,av2);}}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24640,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3396: lp */
t3=((C_word*)((C_word*)t0)[7])[1];
f_23446(t3,t2,C_i_cadr(((C_word*)t0)[5]),((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[2]);}}

/* k24580 in k24577 in k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_24582(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,10)))){
C_save_and_reclaim((void *)f_24582,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24583,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li162),tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24583 in k24580 in k24577 in k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in ... */
static void C_ccall f_24583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24583,c,av);}
t10=C_slot(t8,C_fix(1));
t11=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t12=C_fixnum_plus(C_fix(2),t11);
if(C_truep(C_slot(t10,t12))){
/* irregex-core.scm:3393: pass */
t13=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t13;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t13+1)))(10,av2);}}
else{
/* irregex-core.scm:3394: fail */
t13=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t13;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t13+1)))(10,av2);}}}

/* k24638 in k24577 in k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in ... */
static void C_ccall f_24640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_24640,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24641,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li164),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24641 in k24638 in k24577 in k24568 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in ... */
static void C_ccall f_24641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,9)))){
C_save_and_reclaim((void *)f_24641,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_24647,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,a[7]=t6,a[8]=t7,a[9]=t8,a[10]=t9,a[11]=((C_word)li163),tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3398: test */
t11=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(10,av2);}}

/* a24646 */
static void C_ccall f_24647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24647,c,av);}
/* irregex-core.scm:3399: fail */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=((C_word*)t0)[7];
av2[7]=((C_word*)t0)[8];
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* k24681 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_24683,c,av);}
/* irregex-core.scm:3375: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23446(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_plus(C_fixnum_plus(((C_word*)t0)[5],((C_word*)t0)[6]),t1),((C_word*)t0)[7],((C_word*)t0)[8]);}

/* f_24688 in k24565 in k24562 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24688,c,av);}
/* irregex-core.scm:3381: fail */
t10=t9;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* k24718 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_24720,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24723,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_eqp(C_u_i_car(((C_word*)t0)[4]),lf[191]);
if(C_truep(t3)){
t4=t2;
f_24723(t4,(C_truep(t3)?*((C_word*)lf[241]+1):*((C_word*)lf[242]+1)));}
else{
t4=C_fixnum_and(((C_word*)t0)[5],C_fix(2));
t5=C_eqp(C_fix(2),t4);
t6=t2;
f_24723(t6,(C_truep(t5)?*((C_word*)lf[241]+1):*((C_word*)lf[242]+1)));}}

/* k24721 in k24718 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_fcall f_24723(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,10)))){
C_save_and_reclaim_args((void *)trf_24723,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24724,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word)li167),tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24724 in k24721 in k24718 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in ... */
static void C_ccall f_24724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_24724,c,av);}
a=C_alloc(13);
t10=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_24728,a[2]=t9,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t3,a[7]=t8,a[8]=((C_word*)t0)[3],a[9]=t4,a[10]=t5,a[11]=t6,a[12]=t7,tmp=(C_word)a,a+=13,tmp);
/* irregex-core.scm:3410: irregex-match-substring */
t11=*((C_word*)lf[40]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=t8;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}

/* k24726 */
static void C_ccall f_24728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,8)))){
C_save_and_reclaim((void *)f_24728,c,av);}
a=C_alloc(13);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:3412: fail */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_i_string_length(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24746,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[8],a[8]=t1,a[9]=t4,a[10]=((C_word)li166),tmp=(C_word)a,a+=11,tmp));
t6=((C_word*)t4)[1];
f_24746(t6,((C_word*)t0)[3],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12],C_fix(0),t2);}}

/* lp in k24726 */
static void C_fcall f_24746(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,4)))){
C_save_and_reclaim_args((void *)trf_24746,8,t0,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(20);
t8=C_fixnum_difference(t5,t4);
if(C_truep(C_fixnum_less_or_equal_p(t7,t8))){
t9=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_24759,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=t3,a[8]=t4,a[9]=t7,a[10]=t5,a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[6],tmp=(C_word)a,a+=13,tmp);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24773,a[2]=((C_word*)t0)[7],a[3]=t9,a[4]=t3,a[5]=t4,a[6]=t7,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:3423: scheme#substring */
t11=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[8];
av2[3]=t6;
av2[4]=C_i_string_length(((C_word*)t0)[8]);
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}
else{
t9=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24791,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=t4,a[5]=t6,a[6]=((C_word*)t0)[9],a[7]=t1,a[8]=t7,a[9]=((C_word*)t0)[6],a[10]=t2,tmp=(C_word)a,a+=11,tmp);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24846,a[2]=((C_word*)t0)[7],a[3]=t9,a[4]=t3,a[5]=t4,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
t11=C_fixnum_difference(t5,t4);
/* irregex-core.scm:3430: scheme#substring */
t12=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t12;
av2[1]=t10;
av2[2]=((C_word*)t0)[8];
av2[3]=t6;
av2[4]=C_fixnum_plus(t6,t11);
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}}

/* k24757 in lp in k24726 */
static void C_ccall f_24759(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_24759,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:3425: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=C_fixnum_plus(((C_word*)t0)[8],((C_word*)t0)[9]);
av2[7]=((C_word*)t0)[10];
av2[8]=((C_word*)t0)[11];
av2[9]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3427: fail */
t2=((C_word*)t0)[12];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24771 in lp in k24726 */
static void C_ccall f_24773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_24773,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24777,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3424: scheme#substring */
t3=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=C_fixnum_plus(((C_word*)t0)[5],((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k24775 in k24771 in lp in k24726 */
static void C_ccall f_24777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24777,c,av);}
/* irregex-core.scm:3423: compare */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k24789 in lp in k24726 */
static void C_ccall f_24791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_24791,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_24797,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3432: g4864 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3442: fail */
t2=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24795 in k24789 in lp in k24726 */
static void C_ccall f_24797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_24797,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_24810,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3434: g4866 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3440: fail */
t2=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24808 in k24795 in k24789 in lp in k24726 */
static void C_ccall f_24810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_24810,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24817,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3434: g4868 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k24815 in k24808 in k24795 in k24789 in lp in k24726 */
static void C_ccall f_24817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_24817,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_24824,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=t1,a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3434: g4870 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k24822 in k24815 in k24808 in k24795 in k24789 in lp in k24726 */
static void C_ccall f_24824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,7)))){
C_save_and_reclaim((void *)f_24824,c,av);}
a=C_alloc(15);
t2=C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_s_a_u_i_integer_plus(&a,2,((C_word*)t0)[4],t2);
t4=C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* irregex-core.scm:3434: lp */
t5=((C_word*)((C_word*)t0)[5])[1];
f_24746(t5,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],t1,t3,C_fixnum_difference(((C_word*)t0)[10],t4));}

/* k24844 in lp in k24726 */
static void C_ccall f_24846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_24846,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24850,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3431: scheme#substring */
t3=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k24848 in k24844 in lp in k24726 */
static void C_ccall f_24850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24850,c,av);}
/* irregex-core.scm:3430: compare */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k24935 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_24937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,10)))){
C_save_and_reclaim((void *)f_24937,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24938,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li169),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_24938 in k24935 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_24938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,9)))){
C_save_and_reclaim((void *)f_24938,c,av);}
a=C_alloc(8);
t10=C_slot(t8,C_fix(1));
t11=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t12=C_slot(t10,t11);
t13=C_slot(t8,C_fix(1));
t14=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t15=C_fixnum_plus(C_fix(1),t14);
t16=C_slot(t13,t15);
t17=C_slot(t8,C_fix(1));
t18=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t19=C_i_vector_set(t17,t18,t4);
t20=C_slot(t8,C_fix(1));
t21=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t22=C_fixnum_plus(C_fix(1),t21);
t23=C_i_vector_set(t20,t22,t6);
t24=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24986,a[2]=t8,a[3]=((C_word*)t0)[2],a[4]=t12,a[5]=t16,a[6]=t9,a[7]=((C_word)li168),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3469: body */
t25=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t25;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t24;
((C_proc)(void*)(*((C_word*)t25+1)))(10,av2);}}

/* a24985 */
static void C_ccall f_24986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24986,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t4=C_i_vector_set(t2,t3,((C_word*)t0)[4]);
t5=C_slot(((C_word*)t0)[2],C_fix(1));
t6=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t7=C_fixnum_plus(C_fix(1),t6);
t8=C_i_vector_set(t5,t7,((C_word*)t0)[5]);
/* irregex-core.scm:3475: fail */
t9=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* a25045 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,9)))){
C_save_and_reclaim((void *)f_25046,c,av);}
a=C_alloc(8);
t10=C_slot(t8,C_fix(1));
t11=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t12=C_fixnum_plus(C_fix(2),t11);
t13=C_slot(t10,t12);
t14=C_slot(t8,C_fix(1));
t15=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t16=C_slot(t14,C_fixnum_plus(C_fix(3),t15));
t17=C_slot(t8,C_fix(1));
t18=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t19=C_fixnum_plus(C_fix(2),t18);
t20=C_i_vector_set(t17,t19,t4);
t21=C_slot(t8,C_fix(1));
t22=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t23=C_fixnum_plus(C_fix(3),t22);
t24=C_i_vector_set(t21,t23,t6);
t25=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25081,a[2]=t8,a[3]=((C_word*)t0)[2],a[4]=t13,a[5]=t16,a[6]=t9,a[7]=((C_word)li170),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3457: next */
t26=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t26;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t25;
((C_proc)(void*)(*((C_word*)t26+1)))(10,av2);}}

/* a25080 in a25045 in k23486 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 in ... */
static void C_ccall f_25081(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25081,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t4=C_fixnum_plus(C_fix(2),t3);
t5=C_i_vector_set(t2,t4,((C_word*)t0)[4]);
t6=C_slot(((C_word*)t0)[2],C_fix(1));
t7=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t8=C_fixnum_plus(C_fix(3),t7);
t9=C_i_vector_set(t6,t8,((C_word*)t0)[5]);
/* irregex-core.scm:3463: fail */
t10=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}

/* f_25281 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,9)))){
C_save_and_reclaim((void *)f_25281,c,av);}
a=C_alloc(8);
if(C_truep(C_fixnum_lessp(t6,t7))){
/* irregex-core.scm:3485: next */
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=C_fixnum_plus(t6,C_fix(1));
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(10,av2);}}
else{
t10=C_i_vector_ref(t2,C_fix(0));
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25301,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=t8,a[7]=t9,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3486: g4960 */
t12=t10;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}

/* k25299 */
static void C_ccall f_25301(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25301,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25310,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3488: g4965 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3492: fail */
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25308 in k25299 */
static void C_ccall f_25310(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_25310,c,av);}
a=C_alloc(10);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_25316,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3488: g4967 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25314 in k25308 in k25299 */
static void C_ccall f_25316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_25316,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_25322,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3488: g4969 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25320 in k25314 in k25308 in k25299 */
static void C_ccall f_25322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_25322,c,av);}
/* irregex-core.scm:3491: next */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=C_fixnum_plus(((C_word*)t0)[8],C_fix(1));
av2[7]=t1;
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}

/* f_25339 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,9)))){
C_save_and_reclaim((void *)f_25339,c,av);}
a=C_alloc(8);
if(C_truep(C_fixnum_lessp(t6,t7))){
t10=C_i_string_ref(t5,t6);
t11=C_eqp(C_make_character(10),t10);
if(C_truep(C_i_not(t11))){
t12=C_a_i_fixnum_plus(&a,2,t6,C_fix(1));
/* irregex-core.scm:3497: next */
t13=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t13;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t12;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t13+1)))(10,av2);}}
else{
/* irregex-core.scm:3498: fail */
t12=t9;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t10=C_i_vector_ref(t2,C_fix(0));
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25376,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=t8,a[7]=t9,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3499: g4980 */
t12=t10;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}

/* k25374 */
static void C_ccall f_25376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25376,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25385,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3501: g4985 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3507: fail */
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25383 in k25374 */
static void C_ccall f_25385(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_25385,c,av);}
a=C_alloc(10);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_25391,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3501: g4987 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25389 in k25383 in k25374 */
static void C_ccall f_25391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_25391,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_25397,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3501: g4989 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25395 in k25389 in k25383 in k25374 */
static void C_ccall f_25397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_25397,c,av);}
a=C_alloc(5);
t2=C_i_string_ref(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_eqp(C_make_character(10),t2);
if(C_truep(C_i_not(t3))){
t4=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:3505: next */
t5=((C_word*)t0)[4];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
av2[4]=((C_word*)t0)[8];
av2[5]=((C_word*)t0)[2];
av2[6]=t4;
av2[7]=t1;
av2[8]=((C_word*)t0)[9];
av2[9]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t5+1)))(10,av2);}}
else{
/* irregex-core.scm:3506: fail */
t4=((C_word*)t0)[10];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* f_25431 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_25431,c,av);}
t10=C_i_car(t3);
t11=C_eqp(t4,t10);
t12=(C_truep(t11)?C_i_eqvp(t6,C_u_i_cdr(t3)):C_SCHEME_FALSE);
if(C_truep(t12)){
/* irregex-core.scm:3511: next */
t13=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t13;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t6;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t13+1)))(10,av2);}}
else{
/* irregex-core.scm:3512: fail */
t13=t9;{
C_word *av2=av;
av2[0]=t13;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}

/* f_25461 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_25461,c,av);}
a=C_alloc(20);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25468,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
t11=C_i_vector_ref(t2,C_fix(2));
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25505,a[2]=t6,a[3]=t10,a[4]=t5,a[5]=t2,a[6]=t3,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3515: g5009 */
t13=t11;{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}

/* k25466 */
static void C_ccall f_25468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,9)))){
C_save_and_reclaim((void *)f_25468,c,av);}
t2=C_i_not(t1);
t3=(C_truep(t2)?t2:C_eqp(C_make_character(10),t1));
if(C_truep(t3)){
/* irregex-core.scm:3519: next */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t4+1)))(10,av2);}}
else{
/* irregex-core.scm:3520: fail */
t4=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k25503 */
static void C_ccall f_25505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25505,c,av);}
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[2],t1))){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_string_ref(((C_word*)t0)[4],C_fixnum_difference(((C_word*)t0)[2],C_fix(1)));
f_25468(2,av2);}}
else{
/* irregex-core.scm:3517: chunker-prev-char */
f_8640(((C_word*)t0)[3],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* f_25512 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,2)))){
C_save_and_reclaim((void *)f_25512,c,av);}
a=C_alloc(28);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25519,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25528,a[2]=t6,a[3]=t7,a[4]=t5,a[5]=t10,a[6]=t2,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
t12=C_i_vector_ref(t2,C_fix(2));
t13=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25612,a[2]=t6,a[3]=t5,a[4]=t11,a[5]=t2,a[6]=t3,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3523: g5023 */
t14=t12;{
C_word *av2=av;
av2[0]=t14;
av2[1]=t13;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t14+1)))(3,av2);}}

/* k25517 */
static void C_fcall f_25519(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,9)))){
C_save_and_reclaim_args((void *)trf_25519,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:3534: next */
t2=((C_word*)t0)[2];{
C_word av2[10];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3535: fail */
t2=((C_word*)t0)[11];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25526 */
static void C_fcall f_25528(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_25528,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
if(C_truep(C_fixnum_lessp(((C_word*)t0)[2],((C_word*)t0)[3]))){
t2=C_i_string_ref(((C_word*)t0)[4],((C_word*)t0)[2]);
t3=C_u_i_char_alphabeticp(t2);
t4=((C_word*)t0)[5];
f_25519(t4,(C_truep(t3)?t3:C_u_i_char_numericp(t2)));}
else{
t2=C_i_vector_ref(((C_word*)t0)[6],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25547,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3529: g5030 */
t4=t2;{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}
else{
t2=((C_word*)t0)[5];
f_25519(t2,C_SCHEME_FALSE);}}

/* k25545 in k25526 */
static void C_ccall f_25547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_25547,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25564,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3532: g5033 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];
f_25519(t2,C_SCHEME_FALSE);}}

/* k25562 in k25545 in k25526 */
static void C_ccall f_25564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_25564,c,av);}
a=C_alloc(4);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25571,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3532: g5035 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25569 in k25562 in k25545 in k25526 */
static void C_ccall f_25571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25571,c,av);}
t2=C_i_string_ref(((C_word*)t0)[2],t1);
t3=C_u_i_char_alphabeticp(t2);
t4=((C_word*)t0)[3];
f_25519(t4,(C_truep(t3)?t3:C_u_i_char_numericp(t2)));}

/* k25590 in k25610 */
static void C_ccall f_25592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25592,c,av);}
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];
f_25528(t3,t2);}
else{
t3=C_u_i_char_alphabeticp(t1);
t4=(C_truep(t3)?t3:C_u_i_char_numericp(t1));
t5=((C_word*)t0)[2];
f_25528(t5,C_i_not(t4));}}

/* k25610 */
static void C_ccall f_25612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_25612,c,av);}
a=C_alloc(3);
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[2],t1))){
t2=C_fixnum_difference(((C_word*)t0)[2],C_fix(1));
t3=C_i_string_ref(((C_word*)t0)[3],t2);
t4=C_u_i_char_alphabeticp(t3);
t5=(C_truep(t4)?t4:C_u_i_char_numericp(t3));
t6=((C_word*)t0)[4];
f_25528(t6,C_i_not(t5));}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25592,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3525: chunker-prev-char */
f_8640(t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* f_25619 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25619(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_25619,c,av);}
a=C_alloc(15);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25626,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(t6,t7))){
t11=C_i_vector_ref(t2,C_fix(0));
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25645,a[2]=t10,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3538: g5046 */
t13=t11;{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t11=t10;
f_25626(t11,C_SCHEME_FALSE);}}

/* k25624 */
static void C_fcall f_25626(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,9)))){
C_save_and_reclaim_args((void *)trf_25626,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:3539: next */
t2=((C_word*)t0)[2];{
C_word av2[10];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3540: fail */
t2=((C_word*)t0)[11];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25643 */
static void C_ccall f_25645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25645,c,av);}
t2=((C_word*)t0)[2];
f_25626(t2,C_i_not(t1));}

/* f_25652 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_25652,c,av);}
a=C_alloc(16);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25659,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_fixnum_lessp(t6,t7))){
t11=t10;
f_25659(t11,C_eqp(C_make_character(10),C_i_string_ref(t5,t6)));}
else{
t11=C_i_vector_ref(t2,C_fix(0));
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25682,a[2]=t10,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3545: g5057 */
t13=t11;{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}}

/* k25657 */
static void C_fcall f_25659(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,9)))){
C_save_and_reclaim_args((void *)trf_25659,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:3551: next */
t2=((C_word*)t0)[2];{
C_word av2[10];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3552: fail */
t2=((C_word*)t0)[11];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25680 */
static void C_ccall f_25682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_25682,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];
f_25659(t2,C_SCHEME_TRUE);}
else{
t2=C_i_vector_ref(((C_word*)t0)[3],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25703,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3549: g5059 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k25701 in k25680 */
static void C_ccall f_25703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_25703,c,av);}
a=C_alloc(4);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25710,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3549: g5061 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25708 in k25701 in k25680 */
static void C_ccall f_25710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25710,c,av);}
t2=((C_word*)t0)[2];
f_25659(t2,C_eqp(C_make_character(10),C_i_string_ref(((C_word*)t0)[3],t1)));}

/* f_25717 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_25717,c,av);}
a=C_alloc(23);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25724,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25733,a[2]=t2,a[3]=t6,a[4]=t5,a[5]=t10,a[6]=t3,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_fixnum_lessp(t6,t7))){
t12=C_i_string_ref(t5,t6);
t13=C_u_i_char_alphabeticp(t12);
t14=(C_truep(t13)?t13:C_u_i_char_numericp(t12));
t15=t11;
f_25733(t15,C_i_not(t14));}
else{
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25786,a[2]=t11,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3557: chunker-next-char */
f_8670(t12,t2,t4);}}

/* k25722 */
static void C_fcall f_25724(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,9)))){
C_save_and_reclaim_args((void *)trf_25724,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:3563: next */
t2=((C_word*)t0)[2];{
C_word av2[10];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3564: fail */
t2=((C_word*)t0)[11];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25731 */
static void C_fcall f_25733(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_25733,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25769,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3559: g5076 */
t4=t2;{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];
f_25724(t2,C_SCHEME_FALSE);}}

/* k25751 in k25767 in k25731 */
static void C_ccall f_25753(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25753,c,av);}
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];
f_25724(t3,t2);}
else{
t3=C_u_i_char_alphabeticp(t1);
t4=((C_word*)t0)[2];
f_25724(t4,(C_truep(t3)?t3:C_u_i_char_numericp(t1)));}}

/* k25767 in k25731 */
static void C_ccall f_25769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_25769,c,av);}
a=C_alloc(3);
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[2],t1))){
t2=C_fixnum_difference(((C_word*)t0)[2],C_fix(1));
t3=C_i_string_ref(((C_word*)t0)[3],t2);
t4=C_u_i_char_alphabeticp(t3);
t5=((C_word*)t0)[4];
f_25724(t5,(C_truep(t4)?t4:C_u_i_char_numericp(t3)));}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25753,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3561: chunker-prev-char */
f_8640(t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* k25784 */
static void C_ccall f_25786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25786,c,av);}
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];
f_25733(t3,t2);}
else{
t3=C_u_i_char_alphabeticp(t1);
t4=(C_truep(t3)?t3:C_u_i_char_numericp(t1));
t5=((C_word*)t0)[2];
f_25733(t5,C_i_not(t4));}}

/* f_25806 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_25806,c,av);}
a=C_alloc(12);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25810,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_fixnum_lessp(t6,t7))){
t11=t10;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_i_string_ref(t5,t6);
f_25810(2,av2);}}
else{
/* irregex-core.scm:3569: chunker-next-char */
f_8670(t10,t2,t4);}}

/* k25808 */
static void C_ccall f_25810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_25810,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_25813,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
t3=C_i_vector_ref(((C_word*)t0)[4],C_fix(2));
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25864,a[2]=((C_word*)t0)[8],a[3]=t2,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3570: g5092 */
t5=t3;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k25811 in k25808 */
static void C_ccall f_25813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_25813,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25819,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[12])){
if(C_truep(t1)){
t3=C_u_i_char_alphabeticp(((C_word*)t0)[12]);
t4=(C_truep(t3)?t3:C_u_i_char_numericp(((C_word*)t0)[12]));
if(C_truep(t4)){
t5=C_u_i_char_alphabeticp(t1);
t6=t2;
f_25819(t6,(C_truep(t5)?t5:C_u_i_char_numericp(t1)));}
else{
t5=C_u_i_char_alphabeticp(t1);
t6=(C_truep(t5)?t5:C_u_i_char_numericp(t1));
t7=t2;
f_25819(t7,C_i_not(t6));}}
else{
t3=t2;
f_25819(t3,C_SCHEME_FALSE);}}
else{
t3=t2;
f_25819(t3,C_SCHEME_FALSE);}}

/* k25817 in k25811 in k25808 */
static void C_fcall f_25819(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,9)))){
C_save_and_reclaim_args((void *)trf_25819,2,t0,t1);}
if(C_truep(t1)){
/* irregex-core.scm:3577: next */
t2=((C_word*)t0)[2];{
C_word av2[10];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=((C_word*)t0)[8];
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(10,av2);}}
else{
/* irregex-core.scm:3578: fail */
t2=((C_word*)t0)[11];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k25862 in k25808 */
static void C_ccall f_25864(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25864,c,av);}
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[2],t1))){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_string_ref(((C_word*)t0)[4],C_fixnum_difference(((C_word*)t0)[2],C_fix(1)));
f_25813(2,av2);}}
else{
/* irregex-core.scm:3572: chunker-prev-char */
f_8640(((C_word*)t0)[3],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* f_25928 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_25928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_25928,c,av);}
a=C_alloc(12);
if(C_truep(C_fixnum_greater_or_equal_p(t6,t7))){
t10=C_i_vector_ref(t2,C_fix(0));
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25945,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t8,a[6]=t9,a[7]=((C_word*)t0)[3],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3591: g5119 */
t12=t10;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_26013,a[2]=t6,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3603: scheme#char-ci=? */
t11=*((C_word*)lf[250]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_string_ref(t5,t6);
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}}

/* k25943 */
static void C_ccall f_25945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_25945,c,av);}
a=C_alloc(12);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_25947,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word)li181),tmp=(C_word)a,a+=10,tmp));
t5=((C_word*)t3)[1];
f_25947(t5,((C_word*)t0)[8],t1);}

/* lp in k25943 */
static void C_fcall f_25947(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_25947,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(t2)){
t3=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_25957,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3593: g5110 */
t5=t3;{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* irregex-core.scm:3602: fail */
t3=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k25955 in lp in k25943 */
static void C_ccall f_25957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_25957,c,av);}
a=C_alloc(12);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25963,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3593: g5112 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25961 in k25955 in lp in k25943 */
static void C_ccall f_25963(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_25963,c,av);}
a=C_alloc(13);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_25969,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* irregex-core.scm:3593: g5114 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k25967 in k25961 in k25955 in lp in k25943 */
static void C_ccall f_25969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_25969,c,av);}
a=C_alloc(12);
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],t1))){
t2=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25985,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3597: g5116 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25991,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[9],a[9]=t1,a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3598: scheme#char-ci=? */
t3=*((C_word*)lf[250]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
av2[3]=C_i_string_ref(((C_word*)t0)[9],((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k25983 in k25967 in k25961 in k25955 in lp in k25943 */
static void C_ccall f_25985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25985,c,av);}
/* irregex-core.scm:3597: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_25947(t2,((C_word*)t0)[3],t1);}

/* k25989 in k25967 in k25961 in k25955 in lp in k25943 */
static void C_ccall f_25991(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_25991,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:3599: next */
t3=((C_word*)t0)[3];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=t2;
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}
else{
/* irregex-core.scm:3601: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k26011 */
static void C_ccall f_26013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_26013,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:3604: next */
t3=((C_word*)t0)[3];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=t2;
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}
else{
/* irregex-core.scm:3605: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* f_26027 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_26027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,9)))){
C_save_and_reclaim((void *)f_26027,c,av);}
a=C_alloc(9);
if(C_truep(C_fixnum_greater_or_equal_p(t6,t7))){
t10=C_i_vector_ref(t2,C_fix(0));
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_26044,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=t8,a[7]=t9,a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3609: g5143 */
t12=t10;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t10=C_i_string_ref(t5,t6);
if(C_truep(C_i_char_equalp(((C_word*)t0)[2],t10))){
t11=C_a_i_fixnum_plus(&a,2,t6,C_fix(1));
/* irregex-core.scm:3622: next */
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
av2[6]=t11;
av2[7]=t7;
av2[8]=t8;
av2[9]=t9;
((C_proc)(void*)(*((C_word*)t12+1)))(10,av2);}}
else{
/* irregex-core.scm:3623: fail */
t11=t9;{
C_word *av2=av;
av2[0]=t11;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}

/* k26042 */
static void C_ccall f_26044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_26044,c,av);}
a=C_alloc(12);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_26046,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word)li183),tmp=(C_word)a,a+=10,tmp));
t5=((C_word*)t3)[1];
f_26046(t5,((C_word*)t0)[8],t1);}

/* lp in k26042 */
static void C_fcall f_26046(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_26046,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(t2)){
t3=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_26056,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3611: g5134 */
t5=t3;{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* irregex-core.scm:3620: fail */
t3=((C_word*)t0)[8];{
C_word av2[2];
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26054 in lp in k26042 */
static void C_ccall f_26056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_26056,c,av);}
a=C_alloc(12);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_26062,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3611: g5136 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26060 in k26054 in lp in k26042 */
static void C_ccall f_26062(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_26062,c,av);}
a=C_alloc(13);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_26068,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* irregex-core.scm:3611: g5138 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26066 in k26060 in k26054 in lp in k26042 */
static void C_ccall f_26068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_26068,c,av);}
a=C_alloc(5);
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],t1))){
t2=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26084,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3615: g5140 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_i_string_ref(((C_word*)t0)[7],((C_word*)t0)[2]);
if(C_truep(C_i_char_equalp(((C_word*)t0)[8],t2))){
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:3617: next */
t4=((C_word*)t0)[9];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[10];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[7];
av2[6]=t3;
av2[7]=t1;
av2[8]=((C_word*)t0)[11];
av2[9]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t4+1)))(10,av2);}}
else{
/* irregex-core.scm:3619: fail */
t3=((C_word*)t0)[12];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k26082 in k26066 in k26060 in k26054 in lp in k26042 */
static void C_ccall f_26084(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26084,c,av);}
/* irregex-core.scm:3615: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_26046(t2,((C_word*)t0)[3],t1);}

/* k26140 in lp in k23395 in k16232 in k17090 in k17083 in k17080 in k16203 in k16200 in k16197 in k16194 in k16191 in k16185 in k16182 in k16179 in k16176 in chicken.irregex#sre->irregex in k7761 in k7536 in k7533 */
static void C_ccall f_26142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26142,c,av);}
a=C_alloc(3);
t2=(
/* irregex-core.scm:3626: sre-sequence */
  f_18191(C_a_i(&a,3),t1)
);
/* irregex-core.scm:3626: rec */
t3=((C_word*)t0)[2];
f_23449(t3,((C_word*)t0)[3],t2);}

/* chicken.irregex#sre-cset->procedure in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26168(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,10)))){
C_save_and_reclaim_args((void *)trf_26168,3,t1,t2,t3);}
a=C_alloc(5);
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26170,a[2]=t3,a[3]=t2,a[4]=((C_word)li311),tmp=(C_word)a,a+=5,tmp);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* f_26170 in chicken.irregex#sre-cset->procedure in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=10) C_bad_argc_2(c,10,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_26170,c,av);}
a=C_alloc(12);
if(C_truep(C_fixnum_lessp(t6,t7))){
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_26183,a[2]=t6,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t2,a[6]=t3,a[7]=t4,a[8]=t5,a[9]=t7,a[10]=t8,a[11]=t9,tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3650: cset-contains? */
f_26959(t10,((C_word*)t0)[3],C_i_string_ref(t5,t6));}
else{
t10=C_i_vector_ref(t2,C_fix(0));
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_26202,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=t8,a[7]=t9,a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3653: g5184 */
t12=t10;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}

/* k26181 */
static void C_ccall f_26183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_26183,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:3651: next */
t3=((C_word*)t0)[3];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=t2;
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}
else{
/* irregex-core.scm:3652: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k26200 */
static void C_ccall f_26202(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_26202,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_26211,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* irregex-core.scm:3655: g5189 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3661: fail */
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k26209 in k26200 */
static void C_ccall f_26211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_26211,c,av);}
a=C_alloc(11);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_26217,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3655: g5191 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26215 in k26209 in k26200 */
static void C_ccall f_26217(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_26217,c,av);}
a=C_alloc(12);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_26223,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3655: g5193 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26221 in k26215 in k26209 in k26200 */
static void C_ccall f_26223(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_26223,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_26229,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3658: cset-contains? */
f_26959(t2,((C_word*)t0)[11],C_i_string_ref(((C_word*)t0)[8],((C_word*)t0)[2]));}

/* k26227 in k26221 in k26215 in k26209 in k26200 */
static void C_ccall f_26229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,9)))){
C_save_and_reclaim((void *)f_26229,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* irregex-core.scm:3659: next */
t3=((C_word*)t0)[3];{
C_word *av2;
if(c >= 10) {
  av2=av;
} else {
  av2=C_alloc(10);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
av2[6]=t2;
av2[7]=((C_word*)t0)[9];
av2[8]=((C_word*)t0)[10];
av2[9]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(10,av2);}}
else{
/* irregex-core.scm:3660: fail */
t2=((C_word*)t0)[11];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#cset->plist in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26381(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_26381,2,t1,t2);}
a=C_alloc(12);
t3=C_i_vector_length(t2);
t4=C_a_i_fixnum_difference(&a,2,t3,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26393,a[2]=t2,a[3]=t6,a[4]=((C_word)li313),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_26393(t8,t1,t4,C_SCHEME_END_OF_LIST);}

/* lp in chicken.irregex#cset->plist in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26393(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_26393,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_eqp(t2,C_fix(-1));
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_fixnum_difference(t2,C_fix(1));
t6=C_i_vector_ref(((C_word*)t0)[2],t2);
t7=C_i_car(t6);
t8=C_i_vector_ref(((C_word*)t0)[2],t2);
t9=C_i_cdr(t8);
t10=C_a_i_cons(&a,2,t9,t3);
t11=C_a_i_cons(&a,2,t7,t10);
/* irregex-core.scm:3690: lp */
t13=t1;
t14=t5;
t15=t11;
t1=t13;
t2=t14;
t3=t15;
goto loop;}}

/* lp in k26758 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26443(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_26443,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_cddr(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26461,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_u_i_car(t2);
t7=C_u_i_cdr(t2);
t8=C_u_i_car(t7);
t9=C_a_i_cons(&a,2,t6,t8);
t10=C_a_i_vector1(&a,1,t9);
/* irregex-core.scm:3697: cset-union */
t11=lf[162];{
C_word av2[4];
av2[0]=t11;
av2[1]=t5;
av2[2]=t10;
av2[3]=t3;
f_27080(4,av2);}}}

/* k26459 in lp in k26758 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26461,c,av);}
/* irregex-core.scm:3697: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_26443(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.irregex#string->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26472(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_26472,2,t1,t2);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26478,a[2]=((C_word)li315),tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_vector(&a,0);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26490,a[2]=t1,a[3]=t3,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* ##sys#string->list */
t6=*((C_word*)lf[105]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* a26477 in chicken.irregex#string->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26478(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_26478,c,av);}
a=C_alloc(5);
t4=C_a_i_cons(&a,2,t2,t2);
t5=C_a_i_vector1(&a,1,t4);
/* irregex-core.scm:3802: cset-union */
t6=lf[162];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=t3;
av2[3]=t5;
f_27080(4,av2);}}

/* k26488 in chicken.irregex#string->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26490,c,av);}
/* irregex-core.scm:3700: fold */
f_9382(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26492(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_26492,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t3);
t5=(C_truep(t4)?C_u_i_car(t3):C_SCHEME_FALSE);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26502,a[2]=t7,a[3]=((C_word)li325),tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_26502(t9,t1,t2,t5);}

/* lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26502(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,4)))){
C_save_and_reclaim_args((void *)trf_26502,4,t0,t1,t2,t3);}
a=C_alloc(18);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26505,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word)li317),tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(t2))){
if(C_truep(C_i_stringp(C_u_i_car(t2)))){
if(C_truep(t3)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26531,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3712: string->cset */
f_26472(t5,C_u_i_car(t2));}
else{
/* irregex-core.scm:3713: string->cset */
f_26472(t1,C_u_i_car(t2));}}
else{
t5=C_u_i_car(t2);
t6=C_eqp(t5,lf[179]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26552,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26556,a[2]=t2,a[3]=t7,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3717: rec */
t9=t4;
f_26505(t9,t8,C_i_cadr(t2));}
else{
t7=C_eqp(t5,lf[234]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26620,a[2]=t2,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3719: rec */
t9=t4;
f_26505(t9,t8,C_i_cadr(t2));}
else{
t8=C_eqp(t5,lf[247]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26682,a[2]=((C_word)li320),tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26690,a[2]=t2,a[3]=t1,a[4]=t9,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3722: rec */
t11=t4;
f_26505(t11,t10,C_i_cadr(t2));}
else{
t9=C_eqp(t5,lf[52]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26750,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26760,a[2]=t10,tmp=(C_word)a,a+=3,tmp);
t12=C_u_i_cdr(t2);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18370,a[2]=t14,a[3]=((C_word)li323),tmp=(C_word)a,a+=4,tmp));
t16=((C_word*)t14)[1];
f_18370(t16,t11,t12,C_SCHEME_END_OF_LIST);}
else{
t10=C_eqp(t5,lf[103]);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26775,a[2]=t2,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3730: rec */
t12=t4;
f_26505(t12,t11,C_i_cadr(t2));}
else{
t11=C_eqp(t5,lf[225]);
if(C_truep(t11)){
t12=(
/* irregex-core.scm:3732: sre-alternate */
  f_18214(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:3732: lp */
t17=t1;
t18=t12;
t19=C_SCHEME_FALSE;
t1=t17;
t2=t18;
t3=t19;
goto loop;}
else{
t12=C_eqp(t5,lf[226]);
if(C_truep(t12)){
t13=(
/* irregex-core.scm:3734: sre-alternate */
  f_18214(C_a_i(&a,3),C_u_i_cdr(t2))
);
/* irregex-core.scm:3734: lp */
t17=t1;
t18=t13;
t19=C_SCHEME_TRUE;
t1=t17;
t2=t18;
t3=t19;
goto loop;}
else{
/* irregex-core.scm:3736: chicken.base#error */
t13=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t13;
av2[1]=t1;
av2[2]=lf[406];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}}}}}}}}}
else{
if(C_truep(C_charp(t2))){
if(C_truep(t3)){
t5=C_a_i_cons(&a,2,t2,t2);
t6=C_a_i_vector1(&a,1,t5);
/* irregex-core.scm:3738: cset-case-insensitive */
f_27607(t1,t6);}
else{
t5=C_a_i_cons(&a,2,t2,t2);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_vector1(&a,1,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
if(C_truep(C_i_stringp(t2))){
t5=C_a_i_list1(&a,1,t2);
/* irregex-core.scm:3740: rec */
t6=t4;
f_26505(t6,t1,t5);}
else{
t5=C_i_assq(t2,lf[248]);
if(C_truep(t5)){
/* irregex-core.scm:3744: rec */
t6=t4;
f_26505(t6,t1,C_i_cdr(t5));}
else{
/* irregex-core.scm:3745: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[407];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}}}

/* rec in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26505(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_26505,3,t0,t1,t2);}
/* irregex-core.scm:3707: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_26502(t3,t1,t2,((C_word*)t0)[3]);}

/* k26529 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26531,c,av);}
/* irregex-core.scm:3712: cset-case-insensitive */
f_27607(((C_word*)t0)[2],t1);}

/* k26550 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26552(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26552,c,av);}
/* irregex-core.scm:3716: cset-complement */
f_27597(((C_word*)t0)[2],t1);}

/* k26554 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26556(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_26556,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[211]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26569,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26571,a[2]=t4,a[3]=t10,a[4]=((C_word*)t0)[4],a[5]=t5,a[6]=((C_word)li318),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_26571(t12,t8,t6);}

/* k26567 in k26554 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26569,c,av);}
/* irregex-core.scm:3717: fold */
f_9382(((C_word*)t0)[2],lf[162],((C_word*)t0)[3],t1);}

/* map-loop5250 in k26554 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26571(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26571,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26596,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3717: g5256 */
t4=((C_word*)t0)[4];
f_26505(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26594 in map-loop5250 in k26554 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26596,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_26571(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k26618 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26620(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_26620,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[211]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26633,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26635,a[2]=t4,a[3]=t10,a[4]=((C_word*)t0)[4],a[5]=t5,a[6]=((C_word)li319),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_26635(t12,t8,t6);}

/* k26631 in k26618 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26633,c,av);}
/* irregex-core.scm:3719: fold */
f_9382(((C_word*)t0)[2],lf[405],((C_word*)t0)[3],t1);}

/* map-loop5276 in k26618 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26635(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26635,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26660,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3719: g5282 */
t4=((C_word*)t0)[4];
f_26505(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26658 in map-loop5276 in k26618 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26660,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_26635(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a26681 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26682,c,av);}
/* irregex-core.scm:3721: cset-difference */
f_27275(t1,t3,t2);}

/* k26688 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_26690,c,av);}
a=C_alloc(19);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[211]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26703,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26705,a[2]=t4,a[3]=t10,a[4]=((C_word*)t0)[5],a[5]=t5,a[6]=((C_word)li321),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_26705(t12,t8,t6);}

/* k26701 in k26688 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26703,c,av);}
/* irregex-core.scm:3721: fold */
f_9382(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* map-loop5304 in k26688 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26705(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26705,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26730,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3723: g5310 */
t4=((C_word*)t0)[4];
f_26505(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26728 in map-loop5304 in k26688 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26730,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_26705(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k26748 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26750,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* irregex-core.scm:3727: cset-case-insensitive */
f_27607(((C_word*)t0)[3],t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k26758 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26760(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_26760,c,av);}
a=C_alloc(7);
t2=C_a_i_vector(&a,0);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26443,a[2]=t4,a[3]=((C_word)li322),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_26443(t6,((C_word*)t0)[2],t1,t2);}

/* k26773 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_26775,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_i_check_list_2(t6,lf[211]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26788,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26790,a[2]=t4,a[3]=t10,a[4]=((C_word*)t0)[4],a[5]=t5,a[6]=((C_word)li324),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_26790(t12,t8,t6);}

/* k26786 in k26773 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26788,c,av);}
/* irregex-core.scm:3730: fold */
f_9382(((C_word*)t0)[2],lf[162],((C_word*)t0)[3],t1);}

/* map-loop5331 in k26773 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26790(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26790,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26815,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3730: g5337 */
t4=((C_word*)t0)[4];
f_26505(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26813 in map-loop5331 in k26773 in lp in chicken.irregex#sre->cset in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_26815(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26815,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_26790(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k26935 in k13454 in k13447 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_26937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_26937,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,lf[52],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list2(&a,2,t2,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a26938 in k13454 in k13447 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_26939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_26939,c,av);}
a=C_alloc(6);
t4=C_i_car(t2);
t5=C_u_i_cdr(t2);
t6=C_a_i_cons(&a,2,t5,t3);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_cons(&a,2,t4,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k26955 in k13454 in k13447 in go in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_26957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26957,c,av);}
/* irregex-core.scm:3749: fold */
f_9382(((C_word*)t0)[2],((C_word*)t0)[3],C_SCHEME_END_OF_LIST,t1);}

/* chicken.irregex#cset-contains? in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26959(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_26959,3,t1,t2,t3);}
a=C_alloc(8);
t4=C_block_size(t2);
switch(t4){
case C_fix(0):
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}
case C_fix(1):
t5=C_i_vector_ref(t2,C_fix(0));
t6=C_u_i_cdr(t5);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=(C_truep(C_u_i_char_less_or_equal_p(t3,t6))?C_u_i_char_less_or_equal_p(C_u_i_car(t5),t3):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}
default:
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26992,a[2]=t2,a[3]=t3,a[4]=t6,a[5]=((C_word)li327),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_26992(t8,t1,C_fix(0),t4);}}

/* lp in chicken.irregex#cset-contains? in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_26992(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_26992,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=C_fixnum_plus(t3,t2);
t5=C_fixnum_shift_right(t4,C_fix(1));
t6=C_i_vector_ref(((C_word*)t0)[2],t5);
t7=C_u_i_cdr(t6);
if(C_truep(C_u_i_char_lessp(t7,((C_word*)t0)[3]))){
t8=C_a_i_fixnum_plus(&a,2,t5,C_fix(1));
if(C_truep(C_fixnum_lessp(t8,t3))){
/* irregex-core.scm:3768: lp */
t10=t1;
t11=t8;
t12=t3;
t1=t10;
t2=t11;
t3=t12;
goto loop;}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
if(C_truep(C_i_char_lessp(((C_word*)t0)[3],C_u_i_car(t6)))){
if(C_truep(C_fixnum_lessp(t2,t5))){
/* irregex-core.scm:3770: lp */
t10=t1;
t11=t2;
t12=t5;
t1=t10;
t2=t11;
t3=t12;
goto loop;}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}

/* chicken.irregex#char-ranges-union in k18916 in k7761 in k7536 in k7533 */
static C_word C_fcall f_27044(C_word *a,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;{}
t3=C_i_car(t1);
t4=C_i_car(t2);
t5=C_i_char_less_or_equal_p(t3,t4);
t6=(C_truep(t5)?C_u_i_car(t1):C_u_i_car(t2));
if(C_truep(C_i_char_greater_or_equal_p(C_u_i_cdr(t1),C_u_i_cdr(t2)))){
t7=C_u_i_cdr(t1);
return(C_a_i_cons(&a,2,t6,t7));}
else{
t7=C_u_i_cdr(t2);
return(C_a_i_cons(&a,2,t6,t7));}}

/* chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27080,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27088,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3778: scheme#vector->list */
t5=*((C_word*)lf[163]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27088,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27092,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3779: scheme#vector->list */
t3=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_27092,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27094,a[2]=t3,a[3]=((C_word)li330),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_27094(t5,((C_word*)t0)[2],((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27094(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,4)))){
C_save_and_reclaim_args((void *)trf_27094,5,t0,t1,t2,t3,t4);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27108,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27112,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27116,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3782: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
if(C_truep(C_i_nullp(t3))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27129,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27133,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27137,a[2]=t6,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3783: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t5=C_i_car(t2);
t6=C_i_car(t3);
t7=C_i_cdr(t5);
t8=C_fix(C_character_code(t7));
t9=C_a_i_fixnum_plus(&a,2,t8,C_fix(1));
t10=C_i_car(t6);
t11=C_fix(C_character_code(t10));
if(C_truep(C_i_integer_lessp(t9,t11))){
t12=C_u_i_cdr(t2);
t13=C_a_i_cons(&a,2,t5,t4);
/* irregex-core.scm:3791: union-range */
t22=t1;
t23=t12;
t24=t3;
t25=t13;
t1=t22;
t2=t23;
t3=t24;
t4=t25;
goto loop;}
else{
t12=C_u_i_car(t5);
t13=C_fix(C_character_code(t12));
t14=C_u_i_cdr(t6);
t15=C_fix(C_character_code(t14));
t16=C_a_i_fixnum_plus(&a,2,t15,C_fix(1));
if(C_truep(C_i_integer_greaterp(t13,t16))){
t17=C_u_i_cdr(t3);
t18=C_a_i_cons(&a,2,t6,t4);
/* irregex-core.scm:3793: union-range */
t22=t1;
t23=t17;
t24=t2;
t25=t18;
t1=t22;
t2=t23;
t3=t24;
t4=t25;
goto loop;}
else{
if(C_truep(C_i_char_greater_or_equal_p(C_u_i_cdr(t5),C_u_i_car(t6)))){
t17=(
/* irregex-core.scm:3795: char-ranges-union */
  f_27044(C_a_i(&a,6),t5,t6)
);
t18=C_u_i_cdr(t2);
t19=C_a_i_cons(&a,2,t17,t18);
/* irregex-core.scm:3795: union-range */
t22=t1;
t23=t19;
t24=C_u_i_cdr(t3);
t25=t4;
t1=t22;
t2=t23;
t3=t24;
t4=t25;
goto loop;}
else{
t17=C_u_i_cdr(t2);
t18=(
/* irregex-core.scm:3799: char-ranges-union */
  f_27044(C_a_i(&a,6),t5,t6)
);
t19=C_u_i_cdr(t3);
t20=C_a_i_cons(&a,2,t18,t19);
/* irregex-core.scm:3798: union-range */
t22=t1;
t23=t17;
t24=t20;
t25=t4;
t1=t22;
t2=t23;
t3=t24;
t4=t25;
goto loop;}}}}}}

/* k27106 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27108,c,av);}
/* irregex-core.scm:3782: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27110 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27112,c,av);}
/* irregex-core.scm:3782: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27114 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27116(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27116,c,av);}
/* irregex-core.scm:3782: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k27127 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27129(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27129,c,av);}
/* irregex-core.scm:3783: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27131 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27133,c,av);}
/* irregex-core.scm:3783: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27135 in union-range in k27090 in k27086 in chicken.irregex#cset-union in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27137(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27137,c,av);}
/* irregex-core.scm:3783: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27275(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_27275,3,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27283,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3811: scheme#vector->list */
t5=*((C_word*)lf[163]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27283,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27287,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3812: scheme#vector->list */
t3=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_27287,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27289,a[2]=t3,a[3]=((C_word)li332),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_27289(t5,((C_word*)t0)[2],((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27289(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,4)))){
C_save_and_reclaim_args((void *)trf_27289,5,t0,t1,t2,t3,t4);}
a=C_alloc(17);
if(C_truep(C_i_nullp(t2))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27303,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3814: ##sys#fast-reverse */
t6=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
if(C_truep(C_i_nullp(t3))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27316,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27320,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3815: ##sys#fast-reverse */
t7=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=C_i_car(t2);
t6=C_i_car(t3);
t7=C_i_cdr(t5);
t8=C_i_car(t6);
if(C_truep(C_i_char_lessp(t7,t8))){
t9=C_u_i_cdr(t2);
t10=C_a_i_cons(&a,2,t5,t4);
/* irregex-core.scm:3821: diff */
t18=t1;
t19=t9;
t20=t3;
t21=t10;
t1=t18;
t2=t19;
t3=t20;
t4=t21;
goto loop;}
else{
if(C_truep(C_i_char_greaterp(C_u_i_car(t5),C_u_i_cdr(t6)))){
/* irregex-core.scm:3823: diff */
t18=t1;
t19=t2;
t20=C_u_i_cdr(t3);
t21=t4;
t1=t18;
t2=t19;
t3=t20;
t4=t21;
goto loop;}
else{
t9=C_i_char_less_or_equal_p(C_u_i_car(t6),C_u_i_car(t5));
t10=(C_truep(t9)?C_i_char_greater_or_equal_p(C_u_i_cdr(t6),C_u_i_cdr(t5)):C_SCHEME_FALSE);
if(C_truep(t10)){
/* irregex-core.scm:3826: diff */
t18=t1;
t19=C_u_i_cdr(t2);
t20=t3;
t21=t4;
t1=t18;
t2=t19;
t3=t20;
t4=t21;
goto loop;}
else{
t11=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_27366,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=t2,a[7]=t5,a[8]=t6,tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_char_lessp(C_u_i_car(t5),C_u_i_car(t6)))){
t12=C_u_i_car(t5);
t13=C_u_i_car(t6);
t14=C_fix(C_character_code(t13));
t15=C_a_i_fixnum_difference(&a,2,t14,C_fix(1));
t16=C_make_character(C_unfix(t15));
t17=t11;
f_27366(t17,C_a_i_cons(&a,2,t12,t16));}
else{
t12=t11;
f_27366(t12,C_SCHEME_FALSE);}}}}}}}

/* k27301 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27303,c,av);}
/* irregex-core.scm:3814: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27314 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27316,c,av);}
/* irregex-core.scm:3815: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27318 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27320,c,av);}
/* irregex-core.scm:3815: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k27364 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27366(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_27366,2,t0,t1);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27369,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_char_greaterp(C_u_i_cdr(((C_word*)t0)[7]),C_u_i_cdr(((C_word*)t0)[8])))){
t3=C_u_i_cdr(((C_word*)t0)[8]);
t4=C_fix(C_character_code(t3));
t5=C_a_i_fixnum_plus(&a,2,t4,C_fix(1));
t6=C_make_character(C_unfix(t5));
t7=C_u_i_cdr(((C_word*)t0)[7]);
t8=t2;
f_27369(t8,C_a_i_cons(&a,2,t6,t7));}
else{
t3=t2;
f_27369(t3,C_SCHEME_FALSE);}}

/* k27367 in k27364 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27369(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_27369,2,t0,t1);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27376,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
t3=C_u_i_cdr(((C_word*)t0)[7]);
t4=t2;
f_27376(t4,C_a_i_cons(&a,2,t1,t3));}
else{
t3=t2;
f_27376(t3,C_u_i_cdr(((C_word*)t0)[7]));}}

/* k27374 in k27367 in k27364 in diff in k27285 in k27281 in chicken.irregex#cset-difference in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27376(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_27376,2,t0,t1);}
a=C_alloc(3);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* irregex-core.scm:3833: diff */
t3=((C_word*)((C_word*)t0)[4])[1];
f_27289(t3,((C_word*)t0)[5],t1,((C_word*)t0)[6],t2);}
else{
/* irregex-core.scm:3833: diff */
t2=((C_word*)((C_word*)t0)[4])[1];
f_27289(t2,((C_word*)t0)[5],t1,((C_word*)t0)[6],((C_word*)t0)[3]);}}

/* chicken.irregex#cset-intersection in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27471,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27479,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3844: scheme#vector->list */
t5=*((C_word*)lf[163]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k27477 in chicken.irregex#cset-intersection in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27479,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27483,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3845: scheme#vector->list */
t3=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27481 in k27477 in chicken.irregex#cset-intersection in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_27483,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27485,a[2]=t3,a[3]=((C_word)li334),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_27485(t5,((C_word*)t0)[2],((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* intersect in k27481 in k27477 in chicken.irregex#cset-intersection in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27485(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_27485,5,t0,t1,t2,t3,t4);}
a=C_alloc(6);
t5=C_i_nullp(t2);
t6=(C_truep(t5)?t5:C_i_nullp(t3));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27502,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3848: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=C_i_car(t2);
t8=C_i_car(t3);
t9=C_i_cdr(t7);
t10=C_i_car(t8);
if(C_truep(C_i_char_lessp(t9,t10))){
/* irregex-core.scm:3853: intersect */
t25=t1;
t26=C_u_i_cdr(t2);
t27=t3;
t28=t4;
t1=t25;
t2=t26;
t3=t27;
t4=t28;
goto loop;}
else{
if(C_truep(C_i_char_greaterp(C_u_i_car(t7),C_u_i_cdr(t8)))){
/* irregex-core.scm:3855: intersect */
t25=t1;
t26=t2;
t27=C_u_i_cdr(t3);
t28=t4;
t1=t25;
t2=t26;
t3=t27;
t4=t28;
goto loop;}
else{
t11=C_u_i_car(t8);
t12=C_u_i_car(t7);
t13=C_i_char_lessp(t11,t12);
t14=(C_truep(t13)?t12:t11);
t15=C_u_i_cdr(t7);
t16=C_u_i_cdr(t8);
t17=C_i_char_lessp(t15,t16);
t18=(C_truep(t17)?C_a_i_cons(&a,2,t14,t15):C_a_i_cons(&a,2,t14,t16));
t19=C_i_char_greaterp(C_u_i_cdr(t7),C_u_i_cdr(t18));
t20=(C_truep(t19)?t2:C_u_i_cdr(t2));
t21=C_i_char_greaterp(C_u_i_cdr(t8),C_u_i_cdr(t18));
t22=(C_truep(t21)?t3:C_u_i_cdr(t3));
t23=C_a_i_cons(&a,2,t18,t4);
/* irregex-core.scm:3859: intersect */
t25=t1;
t26=t20;
t27=t22;
t28=t23;
t1=t25;
t2=t26;
t3=t27;
t4=t28;
goto loop;}}}}

/* k27500 in intersect in k27481 in k27477 in chicken.irregex#cset-intersection in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27502,c,av);}
/* irregex-core.scm:3848: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.irregex#cset-complement in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27597(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_27597,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27605,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3866: sre->cset */
f_26492(t3,lf[53],C_SCHEME_END_OF_LIST);}

/* k27603 in chicken.irregex#cset-complement in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27605,c,av);}
/* irregex-core.scm:3866: cset-difference */
f_27275(((C_word*)t0)[2],t1,((C_word*)t0)[3]);}

/* chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27607(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_27607,2,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27615,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3870: scheme#vector->list */
t4=*((C_word*)lf[163]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_27615,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27617,a[2]=t3,a[3]=((C_word)li337),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_27617(t5,((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27617(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_27617,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27631,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3871: ##sys#fast-reverse */
t5=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27637,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t5=C_i_caar(t2);
if(C_truep(C_u_i_char_alphabeticp(t5))){
t6=C_u_i_car(t2);
t7=t4;
f_27637(t7,C_u_i_char_alphabeticp(C_u_i_cdr(t6)));}
else{
t6=t4;
f_27637(t6,C_SCHEME_FALSE);}}}

/* k27629 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27631,c,av);}
/* irregex-core.scm:3871: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27637(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(22,0,2)))){
C_save_and_reclaim_args((void *)trf_27637,2,t0,t1);}
a=C_alloc(22);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27646,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27650,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27654,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27658,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27682,a[2]=((C_word*)t0)[2],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27692,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3877: ##sys#fast-reverse */
t9=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27701,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27705,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27709,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27713,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27723,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:3882: ##sys#fast-reverse */
t8=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}

/* k27644 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27646,c,av);}
/* irregex-core.scm:3874: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_27617(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k27648 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27650,c,av);}
/* irregex-core.scm:3875: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27652 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27654,c,av);}
/* irregex-core.scm:3876: scheme#vector->list */
t2=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27656 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_27658,c,av);}
a=C_alloc(5);
t2=C_i_caar(((C_word*)t0)[2]);
t3=C_u_i_char_upper_casep(t2);
t4=(C_truep(t3)?C_u_i_char_downcase(t2):C_u_i_char_upcase(t2));
t5=C_i_cdar(((C_word*)t0)[2]);
t6=C_u_i_char_upper_casep(t5);
t7=(C_truep(t6)?C_u_i_char_downcase(t5):C_u_i_char_upcase(t5));
t8=C_a_i_cons(&a,2,t4,t7);
t9=C_a_i_vector1(&a,1,t8);
/* irregex-core.scm:3877: cset-union */
t10=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t10;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t9;
f_27080(4,av2);}}

/* k27680 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_27682,c,av);}
a=C_alloc(2);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_vector1(&a,1,t2);
/* irregex-core.scm:3877: cset-union */
t4=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t3;
f_27080(4,av2);}}

/* k27690 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27692,c,av);}
/* irregex-core.scm:3877: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27699 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27701(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27701,c,av);}
/* irregex-core.scm:3881: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_27617(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k27703 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27705,c,av);}
/* irregex-core.scm:3881: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27707 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27709,c,av);}
/* irregex-core.scm:3881: scheme#vector->list */
t2=*((C_word*)lf[163]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27711 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(2,c,3)))){
C_save_and_reclaim((void *)f_27713,c,av);}
a=C_alloc(2);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_vector1(&a,1,t2);
/* irregex-core.scm:3882: cset-union */
t4=lf[162];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t3;
f_27080(4,av2);}}

/* k27721 in k27635 in lp in k27613 in chicken.irregex#cset-case-insensitive in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27723,c,av);}
/* irregex-core.scm:3882: scheme#list->vector */
t2=*((C_word*)lf[409]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word *a;
if(c<6) C_bad_min_argc_2(c,6,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-6)*C_SIZEOF_PAIR +8,c,4)))){
C_save_and_reclaim((void*)f_27738,c,av);}
a=C_alloc((c-6)*C_SIZEOF_PAIR+8);
t6=C_build_rest(&a,c,6,av);
C_word t7;
C_word t8;
C_word t9;
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27742,a[2]=t6,a[3]=t5,a[4]=t3,a[5]=t1,a[6]=t4,a[7]=t2,tmp=(C_word)a,a+=8,tmp);
t8=C_i_stringp(t5);
if(C_truep(C_i_not(t8))){
/* irregex-core.scm:3889: chicken.base#error */
t9=*((C_word*)lf[30]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t7;
av2[2]=lf[411];
av2[3]=lf[414];
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t9=t7;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_UNDEFINED;
f_27742(2,av2);}}}

/* k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_27742,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27745,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:3890: irregex */
t3=*((C_word*)lf[217]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_27745,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27748,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3891: irregex-new-matches */
f_7622(t2,t1);}

/* k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_27748,c,av);}
a=C_alloc(13);
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_u_i_car(((C_word*)t0)[2]):C_SCHEME_FALSE);
t4=(C_truep(t3)?t3:(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27939,a[2]=((C_word)li339),tmp=(C_word)a,a+=3,tmp));
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27757,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[2],tmp=(C_word)a,a+=10,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t6=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t7=t5;
f_27757(t7,(C_truep(t6)?C_i_cadr(((C_word*)t0)[2]):C_fix(0)));}
else{
t6=t5;
f_27757(t6,C_fix(0));}}

/* k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27757(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_27757,2,t0,t1);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27760,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[9]))){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[9])))){
t3=C_i_cddr(((C_word*)t0)[9]);
t4=C_i_pairp(t3);
t5=t2;
f_27760(t5,(C_truep(t4)?C_i_caddr(((C_word*)t0)[9]):C_i_string_length(((C_word*)t0)[2])));}
else{
t3=t2;
f_27760(t3,C_i_string_length(((C_word*)t0)[2]));}}
else{
t3=t2;
f_27760(t3,C_i_string_length(((C_word*)t0)[2]));}}

/* k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27760(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,4)))){
C_save_and_reclaim_args((void *)trf_27760,2,t0,t1);}
a=C_alloc(25);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_27769,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],a[8]=t3,a[9]=((C_word*)t0)[8],a[10]=t2,a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[9],tmp=(C_word)a,a+=13,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[3]))){
t5=C_i_exactp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t5))){
/* irregex-core.scm:3900: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t4;
av2[2]=lf[411];
av2[3]=lf[413];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t6=t4;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_27769(2,av2);}}}
else{
/* irregex-core.scm:3900: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[411];
av2[3]=lf[413];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_27769,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_27772,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[3]))){
t3=C_i_exactp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:3902: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[411];
av2[3]=lf[412];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_27772(2,av2);}}}
else{
/* irregex-core.scm:3902: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[411];
av2[3]=lf[412];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k27770 in k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,6)))){
C_save_and_reclaim((void *)f_27772,c,av);}
a=C_alloc(13);
t2=lf[285];
t3=C_i_setslot(((C_word*)t0)[2],C_fix(3),lf[285]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_27780,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t5,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word)li340),tmp=(C_word)a,a+=11,tmp));
t7=((C_word*)t5)[1];
f_27780(t7,((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[11],((C_word*)t0)[12]);}

/* lp in k27770 in k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_27780(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,7)))){
C_save_and_reclaim_args((void *)trf_27780,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(12);
if(C_truep(C_fixnum_greater_or_equal_p(t4,((C_word*)t0)[2]))){
/* irregex-core.scm:3906: finish */
t6=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=t3;
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_27793,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,a[5]=t5,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3907: irregex-search/matches */
f_19053(t6,((C_word*)t0)[4],lf[285],((C_word*)t0)[9],t2,t4,((C_word*)t0)[7]);}}

/* k27791 in lp in k27770 in k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_27793,c,av);}
a=C_alloc(11);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:3915: finish */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=C_slot(t1,C_fix(1));
t3=C_slot(t2,C_fix(1));
t4=C_slot(t1,C_fix(1));
t5=C_fixnum_times(C_fix(0),C_fix(4));
t6=C_slot(t4,C_fixnum_plus(C_fix(3),t5));
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_27828,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t6,a[6]=t3,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3918: kons */
t8=((C_word*)t0)[11];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}}

/* k27826 in k27791 in lp in k27770 in k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_27828,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_27831,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:3919: irregex-reset-matches! */
f_7645(t2,((C_word*)t0)[10]);}

/* k27829 in k27826 in k27791 in lp in k27770 in k27767 in k27758 in k27755 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_27831,c,av);}
a=C_alloc(9);
t2=C_slot(((C_word*)t0)[2],C_fix(4));
t3=C_fixnum_and(t2,C_fix(2));
t4=C_eqp(C_fix(2),t3);
if(C_truep(t4)){
/* irregex-core.scm:3922: finish */
t5=((C_word*)t0)[3];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=C_eqp(((C_word*)t0)[5],((C_word*)t0)[7]);
if(C_truep(t5)){
t6=C_a_i_list3(&a,3,((C_word*)t0)[8],((C_word*)t0)[5],((C_word*)t0)[9]);
/* irregex-core.scm:3925: lp */
t7=((C_word*)((C_word*)t0)[10])[1];
f_27780(t7,((C_word*)t0)[4],t6,((C_word*)t0)[5],C_fixnum_plus(((C_word*)t0)[5],C_fix(1)),((C_word*)t0)[6]);}
else{
t6=C_a_i_list3(&a,3,((C_word*)t0)[8],((C_word*)t0)[5],((C_word*)t0)[9]);
/* irregex-core.scm:3927: lp */
t7=((C_word*)((C_word*)t0)[10])[1];
f_27780(t7,((C_word*)t0)[4],t6,((C_word*)t0)[5],((C_word*)t0)[5],((C_word*)t0)[6]);}}}

/* f_27939 in k27746 in k27743 in k27740 in chicken.irregex#irregex-fold/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_27939,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.irregex#irregex-fold in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_27956,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27960,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_i_closurep(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:3930: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[411];
av2[3]=lf[416];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_27960(2,av2);}}}

/* k27958 in chicken.irregex#irregex-fold in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_27960,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27961,a[2]=((C_word*)t0)[2],a[3]=((C_word)li342),tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[410];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
av2[5]=((C_word*)t0)[5];
C_apply(6,av2);}}

/* kons2 in k27958 in chicken.irregex#irregex-fold in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_27961,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27969,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3931: irregex-copy-matches */
f_7654(t5,t3);}

/* k27967 in kons2 in k27958 in chicken.irregex#irregex-fold in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_27969,c,av);}
/* irregex-core.scm:3931: kons */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(c<7) C_bad_min_argc_2(c,7,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-7)*C_SIZEOF_PAIR +8,c,2)))){
C_save_and_reclaim((void*)f_27984,c,av);}
a=C_alloc((c-7)*C_SIZEOF_PAIR+8);
t7=C_build_rest(&a,c,7,av);
C_word t8;
C_word t9;
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27988,a[2]=t7,a[3]=t6,a[4]=t5,a[5]=t3,a[6]=t1,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3935: irregex */
t9=*((C_word*)lf[217]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_27988,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_27991,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3936: irregex-new-matches */
f_7622(t2,t1);}

/* k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_27991(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_27991,c,av);}
a=C_alloc(13);
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_u_i_car(((C_word*)t0)[2]):C_SCHEME_FALSE);
t4=(C_truep(t3)?t3:(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28159,a[2]=((C_word)li344),tmp=(C_word)a,a+=3,tmp));
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_28000,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t6=C_i_pairp(((C_word*)t0)[2]);
t7=(C_truep(t6)?C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=t5;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_i_cadr(((C_word*)t0)[2]);
f_28000(2,av2);}}
else{
/* irregex-core.scm:3938: g5555 */
t8=C_i_vector_ref(((C_word*)t0)[4],C_fix(2));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}

/* k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_28000,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_28006,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t2,a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[2],a[10]=t1,a[11]=((C_word*)t0)[9],tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_i_not(C_i_integerp(t1)))){
/* irregex-core.scm:3942: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[418];
av2[3]=lf[419];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_28006(2,av2);}}}

/* k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,5)))){
C_save_and_reclaim((void *)f_28006,c,av);}
a=C_alloc(12);
t2=C_i_setslot(((C_word*)t0)[2],C_fix(3),((C_word*)t0)[3]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_28014,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=t4,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word)li345),tmp=(C_word)a,a+=10,tmp));
t6=((C_word*)t4)[1];
f_28014(t6,((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11]);}

/* lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28014(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,7)))){
C_save_and_reclaim_args((void *)trf_28014,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
if(C_truep(C_i_not(t2))){
/* irregex-core.scm:3946: finish */
t5=((C_word*)t0)[2];{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_28027,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],tmp=(C_word)a,a+=12,tmp);
/* irregex-core.scm:3947: irregex-search/matches */
f_19053(t5,((C_word*)t0)[5],((C_word*)t0)[3],((C_word*)t0)[8],t2,t3,((C_word*)t0)[6]);}}

/* k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_28027,c,av);}
a=C_alloc(9);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:3949: finish */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=C_slot(t1,C_fix(1));
t3=C_slot(t2,C_fix(2));
t4=C_slot(t1,C_fix(1));
t5=C_fixnum_times(C_fix(0),C_fix(4));
t6=C_slot(t4,C_fixnum_plus(C_fix(3),t5));
t7=C_eqp(t3,((C_word*)t0)[4]);
t8=(C_truep(t7)?C_eqp(t6,((C_word*)t0)[5]):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=C_i_vector_ref(((C_word*)t0)[7],C_fix(3));
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28101,a[2]=t6,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:3953: g5570 */
t11=t9;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_28104,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=t6,a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3957: kons */
t10=((C_word*)t0)[11];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t10;
av2[1]=t9;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=t1;
av2[5]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t10+1)))(6,av2);}}}}

/* k28075 in k28099 in k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28077,c,av);}
a=C_alloc(6);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28087,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3955: g5575 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k28085 in k28075 in k28099 in k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_28087,c,av);}
/* irregex-core.scm:3955: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_28014(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5]);}

/* k28099 in k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28101(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_28101,c,av);}
a=C_alloc(6);
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],t1))){
t2=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28077,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:3954: g5573 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:3956: lp */
t2=((C_word*)((C_word*)t0)[4])[1];
f_28014(t2,((C_word*)t0)[5],((C_word*)t0)[7],C_fixnum_plus(((C_word*)t0)[2],C_fix(1)),((C_word*)t0)[6]);}}

/* k28102 in k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28104(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_28104,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_28107,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:3958: irregex-reset-matches! */
f_7645(t2,((C_word*)t0)[8]);}

/* k28105 in k28102 in k28025 in lp in k28004 in k27998 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28107(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_28107,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(4));
t3=C_fixnum_and(t2,C_fix(2));
t4=C_eqp(C_fix(2),t3);
if(C_truep(t4)){
/* irregex-core.scm:3960: finish */
t5=((C_word*)t0)[3];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
/* irregex-core.scm:3961: lp */
t5=((C_word*)((C_word*)t0)[8])[1];
f_28014(t5,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* f_28159 in k27989 in k27986 in chicken.irregex#irregex-fold/chunked/fast in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28159,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.irregex#irregex-fold/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_28166,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28170,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_i_closurep(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:3964: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[418];
av2[3]=lf[421];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_28170(2,av2);}}}

/* k28168 in chicken.irregex#irregex-fold/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_28170,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28171,a[2]=((C_word*)t0)[2],a[3]=((C_word)li347),tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[417];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
av2[5]=((C_word*)t0)[5];
C_apply(6,av2);}}

/* kons2 in k28168 in chicken.irregex#irregex-fold/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_28171,c,av);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28179,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:3965: irregex-copy-matches */
f_7654(t6,t4);}

/* k28177 in kons2 in k28168 in chicken.irregex#irregex-fold/chunked in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_28179,c,av);}
/* irregex-core.scm:3965: kons */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=t1;
av2[5]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_28194,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28198,a[2]=t1,a[3]=t3,a[4]=t4,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:3969: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[424];
av2[3]=lf[425];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_28198(2,av2);}}}

/* k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_28198,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28201,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3970: irregex-search */
t3=*((C_word*)lf[286]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k28199 in k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_28201,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28215,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=C_slot(t1,C_fix(1));
t4=C_fixnum_times(C_fix(0),C_fix(4));
/* irregex-core.scm:3973: scheme#substring */
t5=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_slot(t3,C_fixnum_plus(C_fix(3),t4));
av2[4]=C_block_size(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k28213 in k28199 in k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28215(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_28215,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28219,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28223,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3974: irregex-apply-match */
f_28366(t3,((C_word*)t0)[3],((C_word*)t0)[5]);}

/* k28217 in k28213 in k28199 in k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_28219,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* irregex-core.scm:3972: string-cat-reverse */
f_9108(((C_word*)t0)[3],t2);}

/* k28221 in k28213 in k28199 in k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28223(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_28223,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28231,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_slot(((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:3975: scheme#substring */
t4=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fix(0);
av2[4]=C_slot(t3,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k28229 in k28221 in k28213 in k28199 in k28196 in chicken.irregex#irregex-replace in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28231,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* irregex-core.scm:3974: scheme#append */
t3=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28273(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_28273,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28277,a[2]=t4,a[3]=t3,a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:3980: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[427];
av2[3]=lf[428];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_28277(2,av2);}}}

/* k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,6)))){
C_save_and_reclaim((void *)f_28277,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28282,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li350),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28333,a[2]=((C_word*)t0)[3],a[3]=((C_word)li351),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3981: irregex-fold/fast */
t4=lf[410];{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
av2[5]=((C_word*)t0)[3];
av2[6]=t3;
f_27738(7,av2);}}

/* a28281 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28282(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_28282,c,av);}
a=C_alloc(7);
t5=C_slot(t3,C_fix(1));
t6=C_slot(t5,C_fix(1));
if(C_truep(C_fixnum_greater_or_equal_p(t2,t6))){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28316,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3986: irregex-apply-match */
f_28366(t7,t3,((C_word*)t0)[2]);}
else{
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28323,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:3987: irregex-apply-match */
f_28366(t7,t3,((C_word*)t0)[2]);}}

/* k28314 in a28281 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_28316,c,av);}
/* irregex-core.scm:3986: scheme#append */
t2=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k28321 in a28281 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_28323,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28331,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:3988: scheme#substring */
t3=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k28329 in k28321 in a28281 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28331(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28331,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:3987: scheme#append */
t3=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a28332 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_28333,c,av);}
a=C_alloc(4);
t4=C_i_string_length(((C_word*)t0)[2]);
if(C_truep(C_fixnum_greater_or_equal_p(t2,t4))){
/* irregex-core.scm:3993: string-cat-reverse */
f_9108(t1,t3);}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28354,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:3995: scheme#substring */
t6=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}

/* k28352 in a28332 in k28275 in chicken.irregex#irregex-replace/all in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_28354,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:3993: string-cat-reverse */
f_9108(((C_word*)t0)[3],t2);}

/* chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28366(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_28366,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28372,a[2]=t5,a[3]=t2,a[4]=((C_word)li354),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_28372(t7,t1,t3,C_SCHEME_END_OF_LIST);}

/* lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28372(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_28372,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_integerp(t4))){
t5=C_u_i_cdr(t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28399,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:4004: irregex-match-substring */
t7=*((C_word*)lf[40]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
if(C_truep(C_i_closurep(C_u_i_car(t2)))){
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28424,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:4006: g5631 */
t8=t6;{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
if(C_truep(C_i_symbolp(C_u_i_car(t2)))){
t5=C_u_i_car(t2);
t6=C_eqp(t5,lf[430]);
if(C_truep(t6)){
t7=C_u_i_cdr(t2);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28450,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
t9=C_slot(((C_word*)t0)[3],C_fix(1));
t10=C_slot(t9,C_fix(0));
t11=C_i_car(t10);
t12=C_slot(((C_word*)t0)[3],C_fix(1));
/* irregex-core.scm:4011: scheme#substring */
t13=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t13;
av2[1]=t8;
av2[2]=t11;
av2[3]=C_fix(0);
av2[4]=C_slot(t12,C_fix(1));
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}
else{
t7=C_eqp(t5,lf[431]);
if(C_truep(t7)){
t8=C_slot(((C_word*)t0)[3],C_fix(1));
t9=C_slot(t8,C_fix(0));
t10=C_i_car(t9);
t11=C_u_i_cdr(t2);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28514,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t11,tmp=(C_word)a,a+=6,tmp);
t13=C_slot(((C_word*)t0)[3],C_fix(1));
t14=C_fixnum_times(C_fix(0),C_fix(4));
/* irregex-core.scm:4018: scheme#substring */
t15=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t15;
av2[1]=t12;
av2[2]=t10;
av2[3]=C_slot(t13,C_fixnum_plus(C_fix(3),t14));
av2[4]=C_i_string_length(t10);
((C_proc)(void*)(*((C_word*)t15+1)))(5,av2);}}
else{
t8=C_u_i_car(t2);
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28576,a[2]=t8,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:4024: irregex-match-names */
t10=*((C_word*)lf[19]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}}}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* irregex-core.scm:4029: lp */
t16=t1;
t17=t5;
t18=t7;
t1=t16;
t2=t17;
t3=t18;
goto loop;}}}}}

/* k28397 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28399,c,av);}
a=C_alloc(3);
t2=(C_truep(t1)?C_a_i_cons(&a,2,t1,((C_word*)t0)[2]):C_a_i_cons(&a,2,lf[429],((C_word*)t0)[2]));
/* irregex-core.scm:4003: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_28372(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k28422 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28424,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:4006: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_28372(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k28448 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28450,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:4010: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_28372(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k28512 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28514(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28514,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:4017: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_28372(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* g5658 in k28574 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28546(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_28546,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_i_cdr(t2);
t4=C_i_cdr(((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,t3,t4);
/* irregex-core.scm:4025: lp */
t6=((C_word*)((C_word*)t0)[3])[1];
f_28372(t6,t1,t5,((C_word*)t0)[4]);}

/* k28574 in lp in chicken.irregex#irregex-apply-match in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_28576,c,av);}
a=C_alloc(6);
t2=C_i_assq(((C_word*)t0)[2],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28546,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word)li353),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:4023: g5658 */
t4=t3;
f_28546(t4,((C_word*)t0)[6],t2);}
else{
/* irregex-core.scm:4027: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[6];
av2[2]=lf[432];
av2[3]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* chicken.irregex#irregex-extract in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_28597,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28601,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:4032: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[434];
av2[3]=lf[435];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_28601(2,av2);}}}

/* k28599 in chicken.irregex#irregex-extract in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,8)))){
C_save_and_reclaim((void *)f_28601,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28606,a[2]=((C_word)li356),tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28616,a[2]=((C_word)li357),tmp=(C_word)a,a+=3,tmp);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[410];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
av2[5]=C_SCHEME_END_OF_LIST;
av2[6]=((C_word*)t0)[4];
av2[7]=t3;
av2[8]=((C_word*)t0)[5];
C_apply(9,av2);}}

/* a28605 in k28599 in chicken.irregex#irregex-extract in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28606,c,av);}
a=C_alloc(4);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28614,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:4035: irregex-match-substring */
t6=*((C_word*)lf[40]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k28612 in a28605 in k28599 in chicken.irregex#irregex-extract in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_28614,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a28615 in k28599 in chicken.irregex#irregex-extract in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28616(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28616,c,av);}
/* irregex-core.scm:4038: ##sys#fast-reverse */
t4=*((C_word*)lf[69]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28632(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_28632,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28636,a[2]=t4,a[3]=t3,a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_i_stringp(t3);
if(C_truep(C_i_not(t6))){
/* irregex-core.scm:4042: chicken.base#error */
t7=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[438];
av2[3]=lf[439];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_28636(2,av2);}}}

/* k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28636,c,av);}
a=C_alloc(6);
t2=C_i_pairp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_u_i_car(((C_word*)t0)[2]):C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28642,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t5=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t6=t4;
f_28642(t6,(C_truep(t5)?C_i_cadr(((C_word*)t0)[2]):C_i_string_length(((C_word*)t0)[3])));}
else{
t5=t4;
f_28642(t5,C_i_string_length(((C_word*)t0)[3]));}}

/* k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28642(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,8)))){
C_save_and_reclaim_args((void *)trf_28642,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28647,a[2]=((C_word*)t0)[2],a[3]=((C_word)li359),tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28705,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word)li361),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:4045: irregex-fold/fast */
t4=lf[410];{
C_word av2[9];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
av2[5]=((C_word*)t0)[2];
av2[6]=t3;
av2[7]=((C_word*)t0)[5];
av2[8]=t1;
f_27738(9,av2);}}

/* a28646 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_28647,c,av);}
a=C_alloc(4);
t5=C_slot(t3,C_fix(1));
t6=C_slot(t5,C_fix(1));
t7=C_eqp(t2,t6);
if(C_truep(t7)){
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28661,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t9=C_slot(t3,C_fix(1));
/* irregex-core.scm:4052: scheme#substring */
t10=*((C_word*)lf[46]+1);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=C_slot(t9,C_fix(1));
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}}

/* k28659 in a28646 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_28661,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a28704 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_28705,c,av);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28713,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t5)){
t6=t4;
f_28713(t6,t3);}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28796,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:4056: scheme#substring */
t7=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}}

/* k28711 in a28704 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28713(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_28713,2,t0,t1);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28715,a[2]=t3,a[3]=((C_word)li360),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_28715(t5,((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}

/* lp in k28711 in a28704 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28715(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_28715,5,t0,t1,t2,t3,t4);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_car(t2);
if(C_truep(C_charp(t5))){
t6=C_u_i_cdr(t2);
t7=(C_truep(t4)?t4:C_i_nullp(t3));
if(C_truep(t7)){
t8=C_u_i_car(t2);
t9=C_a_i_string(&a,1,t8);
t10=C_a_i_cons(&a,2,t9,t3);
/* irregex-core.scm:4062: lp */
t12=t1;
t13=t6;
t14=t10;
t15=C_SCHEME_TRUE;
t1=t12;
t2=t13;
t3=t14;
t4=t15;
goto loop;}
else{
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28756,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t6,tmp=(C_word)a,a+=6,tmp);
t9=C_u_i_car(t2);
t10=C_a_i_string(&a,1,t9);
/* irregex-core.scm:4065: scheme#string-append */
t11=*((C_word*)lf[437]+1);{
C_word av2[4];
av2[0]=t11;
av2[1]=t8;
av2[2]=t10;
av2[3]=C_i_car(t3);
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}}
else{
t6=C_u_i_cdr(t2);
t7=C_u_i_car(t2);
t8=C_a_i_cons(&a,2,t7,t3);
/* irregex-core.scm:4068: lp */
t12=t1;
t13=t6;
t14=t8;
t15=C_SCHEME_FALSE;
t1=t12;
t2=t13;
t3=t14;
t4=t15;
goto loop;}}}

/* k28754 in lp in k28711 in a28704 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_28756,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t1,t2);
/* irregex-core.scm:4062: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_28715(t4,((C_word*)t0)[4],((C_word*)t0)[5],t3,C_SCHEME_TRUE);}

/* k28794 in a28704 in k28640 in k28634 in chicken.irregex#irregex-split in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_28796,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];
f_28713(t2,C_a_i_cons(&a,2,t1,((C_word*)t0)[3]));}

/* chicken.irregex#string-scan-char in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_28830,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+9);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
t5=C_i_string_length(t2);
t6=C_i_pairp(t4);
t7=(C_truep(t6)?C_get_rest_arg(c,4,av,4,t0):C_fix(0));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28843,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=t9,a[6]=((C_word)li363),tmp=(C_word)a,a+=7,tmp));
t11=((C_word*)t9)[1];
f_28843(t11,t1,t7);}

/* scan in chicken.irregex#string-scan-char in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28843(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_28843,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t2);
t5=C_eqp(((C_word*)t0)[4],t4);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-utils.scm:14: scan */
t8=t1;
t9=t6;
t1=t8;
t2=t9;
goto loop;}}}

/* chicken.irregex#irregex-quote in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28873,c,av);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28881,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28885,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#string->list */
t5=*((C_word*)lf[105]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k28879 in chicken.irregex#irregex-quote in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28881,c,av);}
/* ##sys#list->string */
t2=*((C_word*)lf[441]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k28883 in chicken.irregex#irregex-quote in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_28885,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28887,a[2]=t3,a[3]=((C_word)li365),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_28887(t5,((C_word*)t0)[2],t1,C_SCHEME_END_OF_LIST);}

/* loop in k28883 in chicken.irregex#irregex-quote in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_28887(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_28887,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
/* irregex-utils.scm:20: ##sys#fast-reverse */
t4=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28906,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[2],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* irregex-utils.scm:22: string-scan-char */
t6=lf[54];{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[442];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* k28904 in loop in k28883 in chicken.irregex#irregex-quote in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_28906,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,C_make_character(92),((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,((C_word*)t0)[4],t3);
/* irregex-utils.scm:23: loop */
t5=((C_word*)((C_word*)t0)[5])[1];
f_28887(t5,((C_word*)t0)[6],t2,t4);}
else{
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[3]);
/* irregex-utils.scm:24: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_28887(t4,((C_word*)t0)[6],t2,t3);}}

/* chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_28930,c,av);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[444];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cdr(t2);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29067,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:42: scheme#make-vector */
t5=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_fix(256);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}}

/* k28945 in k29102 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_28947,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28954,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* ##sys#list->string */
t3=*((C_word*)lf[441]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_a_i_cons(&a,2,lf[103],((C_word*)t0)[4]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(((C_word*)t0)[3])?C_a_i_list(&a,2,lf[131],t2):t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k28952 in k28945 in k29102 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_28954,c,av);}
a=C_alloc(9);
t2=C_a_i_list1(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(((C_word*)t0)[3])?C_a_i_list(&a,2,lf[131],t2):t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k28975 in k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28977(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_28977,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28998,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29000,a[2]=t4,a[3]=t8,a[4]=t5,a[5]=((C_word)li367),tmp=(C_word)a,a+=6,tmp));
t10=((C_word*)t8)[1];
f_29000(t10,t6,((C_word*)t0)[3]);}
else{
t2=C_a_i_cons(&a,2,lf[115],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
/* irregex-utils.scm:51: lp2 */
t4=((C_word*)((C_word*)t0)[5])[1];
f_29084(t4,((C_word*)t0)[6],((C_word*)t0)[7],t3);}}

/* k28996 in k28975 in k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_28998(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_28998,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[437]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop5957 in k28975 in k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29000(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_29000,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29010,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
if(C_truep(C_charp(t4))){
t5=C_a_i_string(&a,1,t4);
t6=t3;
f_29010(t6,C_a_i_cons(&a,2,t5,C_SCHEME_END_OF_LIST));}
else{
t5=t3;
f_29010(t5,C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST));}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k29008 in map-loop5957 in k28975 in k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29010(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_29010,2,t0,t1);}
t2=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t1);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t4=((C_word*)((C_word*)t0)[3])[1];
f_29000(t4,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a29036 in k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29037(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29037,c,av);}
t3=C_i_stringp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t3:C_charp(t2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29067(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_29067,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29072,a[2]=t1,a[3]=t3,a[4]=((C_word)li370),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_29072(t5,((C_word*)t0)[2],((C_word*)t0)[3],C_SCHEME_FALSE);}

/* lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29072(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_29072,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29084,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word)li369),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_29084(t7,t1,C_fix(0),C_SCHEME_END_OF_LIST);}
else{
t4=C_i_car(t2);
t5=C_i_string_length(t4);
t6=C_eqp(t5,C_fix(0));
if(C_truep(t6)){
/* irregex-utils.scm:60: lp1 */
t11=t1;
t12=C_u_i_cdr(t2);
t13=C_SCHEME_TRUE;
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
t7=C_i_string_ref(t4,C_fix(0));
t8=C_fix(C_character_code(t7));
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_29183,a[2]=((C_word*)t0)[2],a[3]=t8,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* irregex-utils.scm:65: scheme#substring */
t10=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t9;
av2[2]=t4;
av2[3]=C_fix(1);
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}}}

/* lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29084(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,3)))){
C_save_and_reclaim_args((void *)trf_29084,4,t0,t1,t2,t3);}
a=C_alloc(19);
t4=C_eqp(t2,C_fix(256));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29104,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:47: ##sys#fast-reverse */
t6=*((C_word*)lf[69]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_make_character(C_unfix(t2));
t6=C_i_vector_ref(((C_word*)t0)[3],t2);
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
if(C_truep(C_i_nullp(t6))){
/* irregex-utils.scm:51: lp2 */
t11=t1;
t12=t7;
t13=t3;
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
if(C_truep(C_i_equalp(t6,lf[446]))){
t8=C_a_i_cons(&a,2,t5,t3);
/* irregex-utils.scm:51: lp2 */
t11=t1;
t12=t7;
t13=t8;
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29139,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_29147,a[2]=t5,a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=t7,a[7]=t8,tmp=(C_word)a,a+=8,tmp);
/* irregex-utils.scm:55: irregex-opt */
t10=*((C_word*)lf[443]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}}}}

/* k29102 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29104(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29104,c,av);}
a=C_alloc(6);
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
t3=C_u_i_car(t1);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(((C_word*)t0)[3])?C_a_i_list(&a,2,lf[131],t3):t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28947,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:31: every */
f_9337(t3,*((C_word*)lf[445]+1),t1);}}

/* k29137 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_29139,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-utils.scm:51: lp2 */
t3=((C_word*)((C_word*)t0)[3])[1];
f_29084(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k29145 in lp2 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_29147,c,av);}
a=C_alloc(17);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=C_i_cdr(t2);
if(C_truep(C_i_nullp(t3))){
t4=C_u_i_car(t2);
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[3]);
/* irregex-utils.scm:51: lp2 */
t6=((C_word*)((C_word*)t0)[4])[1];
f_29084(t6,((C_word*)t0)[5],((C_word*)t0)[6],t5);}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28977,a[2]=((C_word*)t0)[7],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29037,a[2]=((C_word)li368),tmp=(C_word)a,a+=3,tmp);
/* irregex-utils.scm:35: every */
f_9337(t4,t5,t2);}}

/* k29181 in lp1 in k29065 in chicken.irregex#irregex-opt in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_29183,c,av);}
a=C_alloc(3);
t2=C_i_vector_ref(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t1,t2);
t4=C_i_vector_set(((C_word*)t0)[2],((C_word*)t0)[3],t3);
/* irregex-utils.scm:66: lp1 */
t5=((C_word*)((C_word*)t0)[4])[1];
f_29072(t5,((C_word*)t0)[5],C_u_i_cdr(((C_word*)t0)[6]),((C_word*)t0)[7]);}

/* chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29197(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_29197,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29201,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:71: chicken.base#open-output-string */
t4=*((C_word*)lf[451]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29201,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29204,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29209,a[2]=t4,a[3]=t1,a[4]=((C_word)li372),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_29209(t6,t2,((C_word*)t0)[3]);}

/* k29202 in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29204,c,av);}
/* irregex-utils.scm:82: chicken.base#get-output-string */
t2=*((C_word*)lf[448]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29209(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_29209,3,t0,t1,t2);}
a=C_alloc(16);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29219,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(C_u_i_car(t2)))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29230,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29252,a[2]=t4,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t6=C_i_caar(t2);
t7=C_a_i_string(&a,1,t6);
/* irregex-utils.scm:77: irregex-quote */
t8=*((C_word*)lf[440]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t5;
av2[2]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29267,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t5=C_u_i_car(t2);
t6=C_a_i_string(&a,1,t5);
/* irregex-utils.scm:80: irregex-quote */
t7=*((C_word*)lf[440]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t4;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29217 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29219,c,av);}
/* irregex-utils.scm:81: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_29209(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* k29228 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29230(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_29230,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29233,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* ##sys#write-char/port */
t3=*((C_word*)lf[450]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(45);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k29231 in k29228 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29233(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29233,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29240,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_cdar(((C_word*)t0)[4]);
t4=C_a_i_string(&a,1,t3);
/* irregex-utils.scm:79: irregex-quote */
t5=*((C_word*)lf[440]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k29238 in k29231 in k29228 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29240,c,av);}
/* irregex-utils.scm:79: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29250 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29252(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29252,c,av);}
/* irregex-utils.scm:77: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29265 in lp in k29199 in chicken.irregex#cset->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29267(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29267,c,av);}
/* irregex-utils.scm:80: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29277,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29281,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:85: chicken.base#open-output-string */
t4=*((C_word*)lf[451]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_29281,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29284,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29289,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word)li382),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_29289(t6,t2,((C_word*)t0)[3]);}

/* k29282 in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29284,c,av);}
/* irregex-utils.scm:153: chicken.base#get-output-string */
t2=*((C_word*)lf[448]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29289(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_29289,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_eqp(t3,lf[227]);
t5=(C_truep(t4)?t4:C_eqp(t3,lf[115]));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29312,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(t2)))){
t7=C_i_cddr(t2);
t8=C_i_pairp(t7);
t9=t6;
f_29312(t9,(C_truep(t8)?C_i_not(C_eqp(t2,((C_word*)t0)[4])):C_SCHEME_FALSE));}
else{
t7=t6;
f_29312(t7,C_SCHEME_FALSE);}}
else{
t6=C_eqp(t3,lf[108]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29409,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:96: scheme#display */
t8=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[456];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t7=C_eqp(t3,lf[120]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29451,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:98: scheme#display */
t9=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[459];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t8=C_eqp(t3,lf[103]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29505,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:101: scheme#display */
t10=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t9;
av2[2]=lf[462];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t9=C_eqp(t3,lf[127]);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_29564,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t9)){
t11=t10;
f_29564(t11,t9);}
else{
t11=C_eqp(t3,lf[129]);
if(C_truep(t11)){
t12=t10;
f_29564(t12,t11);}
else{
t12=C_eqp(t3,lf[131]);
if(C_truep(t12)){
t13=t10;
f_29564(t13,t12);}
else{
t13=C_eqp(t3,lf[128]);
t14=t10;
f_29564(t14,(C_truep(t13)?t13:C_eqp(t3,lf[132])));}}}}}}}}
else{
if(C_truep(C_i_symbolp(t2))){
t3=C_eqp(t2,lf[186]);
t4=(C_truep(t3)?t3:C_eqp(t2,lf[195]));
if(C_truep(t4)){
/* irregex-utils.scm:143: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[481];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=C_eqp(t2,lf[187]);
t6=(C_truep(t5)?t5:C_eqp(t2,lf[196]));
if(C_truep(t6)){
/* irregex-utils.scm:144: scheme#display */
t7=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[482];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t7=C_eqp(t2,lf[124]);
t8=(C_truep(t7)?t7:C_eqp(t2,lf[125]));
if(C_truep(t8)){
/* irregex-utils.scm:145: scheme#display */
t9=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[483];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
/* irregex-utils.scm:146: chicken.base#error */
t9=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[484];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}}}}
else{
if(C_truep(C_i_stringp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30011,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:148: irregex-quote */
t4=*((C_word*)lf[440]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
if(C_truep(C_charp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30024,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_string(&a,1,t2);
/* irregex-utils.scm:150: irregex-quote */
t5=*((C_word*)lf[440]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* irregex-utils.scm:152: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[485];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}}}}

/* k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29312(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_29312,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:93: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[454];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29357,a[2]=t5,a[3]=((C_word*)t0)[5],a[4]=((C_word)li375),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_29357(t7,((C_word*)t0)[3],t2);}}

/* k29313 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29315,c,av);}
a=C_alloc(11);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29322,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29327,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=((C_word)li374),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_29327(t8,t4,t2);}

/* k29320 in k29313 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29322,c,av);}
/* irregex-utils.scm:93: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[453];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6057 in k29313 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29327(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29327,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29337,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:93: g6058 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29335 in for-each-loop6057 in k29313 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29337,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29327(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop6076 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29357(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29357,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29367,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:94: g6077 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29365 in for-each-loop6076 in k29310 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29367,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29357(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29407 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29409,c,av);}
a=C_alloc(11);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29416,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29421,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=((C_word)li376),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_29421(t8,t4,t2);}

/* k29414 in k29407 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29416(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29416,c,av);}
/* irregex-utils.scm:96: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[455];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6093 in k29407 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29421(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29421,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29431,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:96: g6094 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29429 in for-each-loop6093 in k29407 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29431,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29421(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29451,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29454,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:98: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k29452 in k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29454,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29457,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:98: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[458];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k29455 in k29452 in k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29457(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29457,c,av);}
a=C_alloc(11);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29466,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29471,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=((C_word)li377),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_29471(t8,t4,t2);}

/* k29464 in k29455 in k29452 in k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29466(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29466,c,av);}
/* irregex-utils.scm:99: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[457];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6112 in k29455 in k29452 in k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29471(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29471,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29481,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:99: g6113 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29479 in for-each-loop6112 in k29455 in k29452 in k29449 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29481,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29471(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29505,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29508,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:102: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_29289(t3,t2,C_i_cadr(((C_word*)t0)[4]));}

/* k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_29508,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29509,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li378),tmp=(C_word)a,a+=5,tmp);
t3=C_i_cddr(((C_word*)t0)[4]);
t4=C_i_check_list_2(t3,lf[306]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29525,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29530,a[2]=t7,a[3]=t2,a[4]=((C_word)li379),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_29530(t9,t5,t3);}

/* g6134 in k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29509(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_29509,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29513,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:103: scheme#display */
t4=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[460];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k29511 in g6134 in k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29513,c,av);}
/* irregex-utils.scm:103: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_29289(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k29523 in k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29525,c,av);}
/* irregex-utils.scm:104: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[461];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6133 in k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29530(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29530,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29540,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:103: g6134 */
t4=((C_word*)t0)[3];
f_29509(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29538 in for-each-loop6133 in k29506 in k29503 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29540,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29530(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29564(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_29564,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29567,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_i_cddr(((C_word*)t0)[3]);
if(C_truep(C_i_pairp(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29578,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:108: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[464];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
/* irregex-utils.scm:109: lp */
t4=((C_word*)((C_word*)t0)[5])[1];
f_29289(t4,t2,C_i_cadr(((C_word*)t0)[3]));}}
else{
t2=C_eqp(((C_word*)t0)[6],lf[465]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29634,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=C_i_cadr(((C_word*)t0)[3]);
if(C_truep(C_i_pairp(t4))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29664,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* irregex-utils.scm:113: scheme#caadr */
t6=*((C_word*)lf[471]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t3;
f_29634(t5,C_SCHEME_FALSE);}}
else{
t3=C_eqp(((C_word*)t0)[6],lf[470]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29677,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:119: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[473];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t4=C_eqp(((C_word*)t0)[6],lf[247]);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_29698,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t4)){
t6=t5;
f_29698(t6,t4);}
else{
t6=C_eqp(((C_word*)t0)[6],lf[234]);
if(C_truep(t6)){
t7=t5;
f_29698(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[6],lf[52]);
t8=t5;
f_29698(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[6],lf[179])));}}}}}}

/* k29565 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29567,c,av);}
/* irregex-utils.scm:110: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_u_i_car(((C_word*)t0)[3]);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29576 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29578,c,av);}
a=C_alloc(11);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29585,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29590,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=((C_word)li380),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_29590(t8,t4,t2);}

/* k29583 in k29576 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29585,c,av);}
/* irregex-utils.scm:108: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[463];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6171 in k29576 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29590(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29590,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29600,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:108: g6172 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29598 in for-each-loop6171 in k29576 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29600,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29590(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29632 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29634(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_29634,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29637,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:114: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[468];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* irregex-utils.scm:117: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[469];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k29635 in k29632 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29637(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_29637,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29640,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29647,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29651,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* irregex-utils.scm:115: scheme#cdadr */
t5=*((C_word*)lf[467]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k29638 in k29635 in k29632 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29640,c,av);}
/* irregex-utils.scm:116: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[466];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29645 in k29635 in k29632 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29647,c,av);}
/* irregex-utils.scm:115: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29649 in k29635 in k29632 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29651,c,av);}
/* irregex-utils.scm:115: cset->string */
f_29197(((C_word*)t0)[2],t1);}

/* k29662 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29664(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29664,c,av);}
t2=((C_word*)t0)[2];
f_29634(t2,C_eqp(lf[470],t1));}

/* k29675 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29677(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_29677,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29680,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29687,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:120: cset->string */
f_29197(t3,C_u_i_cdr(((C_word*)t0)[4]));}

/* k29678 in k29675 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29680,c,av);}
/* irregex-utils.scm:121: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[472];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29685 in k29675 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29687,c,av);}
/* irregex-utils.scm:120: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29698(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_29698,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_eqp(C_make_character(126),C_u_i_car(((C_word*)t0)[2]));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29707,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_29707(t4,t2);}
else{
t4=C_eqp(lf[247],C_u_i_car(((C_word*)t0)[2]));
if(C_truep(t4)){
t5=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t6=t3;
f_29707(t6,(C_truep(t5)?C_eqp(lf[124],C_i_cadr(((C_word*)t0)[2])):C_SCHEME_FALSE));}
else{
t5=t3;
f_29707(t5,C_SCHEME_FALSE);}}}
else{
t2=C_eqp(((C_word*)t0)[6],lf[225]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[6],lf[226]));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29783,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:132: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[479];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
if(C_truep(C_i_stringp(C_u_i_car(((C_word*)t0)[2])))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29847,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* ##sys#string->list */
t5=*((C_word*)lf[105]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* irregex-utils.scm:140: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[480];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}}}

/* k29705 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29707(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_29707,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29710,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:126: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[475];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29745,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-utils.scm:130: sre->cset */
f_26492(t2,((C_word*)t0)[4],C_SCHEME_END_OF_LIST);}}

/* k29708 in k29705 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_29710,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29713,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29720,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_eqp(C_make_character(126),C_u_i_car(((C_word*)t0)[4]));
if(C_truep(t4)){
/* irregex-utils.scm:127: cset->string */
f_29197(t3,C_u_i_cdr(((C_word*)t0)[4]));}
else{
/* irregex-utils.scm:127: cset->string */
f_29197(t3,C_i_cddr(((C_word*)t0)[4]));}}

/* k29711 in k29708 in k29705 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29713,c,av);}
/* irregex-utils.scm:128: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[474];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29718 in k29708 in k29705 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29720,c,av);}
/* irregex-utils.scm:127: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k29743 in k29705 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_29745,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[470],t1);
/* irregex-utils.scm:130: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_29289(t3,((C_word*)t0)[3],t2);}

/* k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29783,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29786,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_eqp(C_u_i_car(((C_word*)t0)[2]),lf[225]);
if(C_truep(t3)){
/* irregex-utils.scm:133: scheme#display */
t4=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[478];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_29786(2,av2);}}}

/* k29784 in k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29786,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29789,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-utils.scm:134: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[477];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k29787 in k29784 in k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29789,c,av);}
a=C_alloc(11);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_check_list_2(t2,lf[306]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29796,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29801,a[2]=t6,a[3]=((C_word*)t0)[5],a[4]=((C_word)li381),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_29801(t8,t4,t2);}

/* k29794 in k29787 in k29784 in k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29796,c,av);}
/* irregex-utils.scm:136: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[476];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6226 in k29787 in k29784 in k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_29801(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_29801,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29811,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* irregex-utils.scm:135: g6227 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_29289(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k29809 in for-each-loop6226 in k29787 in k29784 in k29781 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29811,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29801(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k29845 in k29696 in k29562 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_29847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_29847,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[470],t1);
/* irregex-utils.scm:139: lp */
t3=((C_word*)((C_word*)t0)[2])[1];
f_29289(t3,((C_word*)t0)[3],t2);}

/* k30009 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30011,c,av);}
/* irregex-utils.scm:148: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k30022 in lp in k29279 in chicken.irregex#sre->string in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30024,c,av);}
/* irregex-utils.scm:150: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30033(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30033,c,av);}
a=C_alloc(6);
t3=C_i_check_string_2(t2,lf[487]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30044,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30048,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#string->list */
t6=*((C_word*)lf[105]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k30042 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30044,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[227],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30048(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_30048,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30050,a[2]=t3,a[3]=((C_word)li385),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_30050(t5,((C_word*)t0)[2],t1,C_SCHEME_TRUE);}

/* loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_30050(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(61,0,4)))){
C_save_and_reclaim_args((void *)trf_30050,4,t0,t1,t2,t3);}
a=C_alloc(61);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_u_i_cdr(t2);
if(C_truep(C_i_char_equalp(t4,C_make_character(42)))){
if(C_truep(t3)){
t6=C_a_i_list(&a,1,lf[488]);
t7=C_a_i_list(&a,2,lf[179],t6);
t8=C_a_i_list(&a,1,lf[489]);
t9=C_a_i_list(&a,2,lf[179],t8);
t10=C_a_i_list(&a,2,lf[127],t9);
t11=C_a_i_list(&a,3,lf[227],t7,t10);
t12=C_a_i_list(&a,1,lf[490]);
t13=C_a_i_list(&a,2,lf[179],t12);
t14=C_a_i_list(&a,2,lf[127],t13);
t15=C_a_i_list(&a,3,lf[103],t11,t14);
t16=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30081,a[2]=t1,a[3]=t15,tmp=(C_word)a,a+=4,tmp);
/* irregex.scm:241: loop */
t18=t16;
t19=t5;
t20=C_SCHEME_FALSE;
t1=t18;
t2=t19;
t3=t20;
goto loop;}
else{
t6=C_a_i_list(&a,1,lf[491]);
t7=C_a_i_list(&a,2,lf[179],t6);
t8=C_a_i_list(&a,2,lf[127],t7);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30128,a[2]=t1,a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* irregex.scm:242: loop */
t18=t9;
t19=t5;
t20=C_SCHEME_FALSE;
t1=t18;
t2=t19;
t3=t20;
goto loop;}}
else{
if(C_truep(C_u_i_char_equalp(t4,C_make_character(63)))){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30148,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex.scm:243: loop */
t18=t6;
t19=t5;
t20=C_SCHEME_FALSE;
t1=t18;
t2=t19;
t3=t20;
goto loop;}
else{
if(C_truep(C_u_i_char_equalp(t4,C_make_character(91)))){
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30158,a[2]=((C_word*)t0)[2],a[3]=t7,a[4]=((C_word)li384),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_30158(t9,t1,t5,C_SCHEME_END_OF_LIST);}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30309,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* irregex.scm:265: loop */
t18=t6;
t19=t5;
t20=C_u_i_memq(t4,lf[493]);
t1=t18;
t2=t19;
t3=t20;
goto loop;}}}}}

/* k30079 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30081(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30081,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30126 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30128,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30146 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30148(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30148,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[124],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop2 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_30158(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_30158,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_pairp(t2);
if(C_truep(C_i_not(t4))){
/* irregex.scm:247: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[487];
av2[3]=lf[492];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=C_i_car(t2);
if(C_truep(C_i_char_equalp(C_make_character(93),t5))){
t6=C_i_length(t3);
t7=C_fixnum_greaterp(t6,C_fix(1));
t8=(C_truep(t7)?C_a_i_cons(&a,2,lf[103],t3):C_i_car(t3));
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30185,a[2]=t1,a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* irregex.scm:253: loop */
t10=((C_word*)((C_word*)t0)[2])[1];
f_30050(t10,t9,C_u_i_cdr(t2),C_SCHEME_FALSE);}
else{
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30207,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(t2)))){
t7=C_i_cddr(t2);
if(C_truep(C_i_pairp(t7))){
t8=C_u_i_cdr(t2);
t9=t6;
f_30207(t9,C_i_char_equalp(C_make_character(45),C_u_i_car(t8)));}
else{
t8=t6;
f_30207(t8,C_SCHEME_FALSE);}}
else{
t7=t6;
f_30207(t7,C_SCHEME_FALSE);}}}}

/* k30183 in loop2 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30185(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30185,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30205 in loop2 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_fcall f_30207(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_30207,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_i_cdddr(((C_word*)t0)[2]);
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_u_i_cdr(((C_word*)t0)[2]);
t5=C_u_i_cdr(t4);
t6=C_u_i_car(t5);
t7=C_a_i_list(&a,3,lf[52],t3,t6);
t8=C_a_i_cons(&a,2,t7,((C_word*)t0)[3]);
/* irregex.scm:257: loop2 */
t9=((C_word*)((C_word*)t0)[4])[1];
f_30158(t9,((C_word*)t0)[5],t2,t8);}
else{
t2=C_i_pairp(C_u_i_cdr(((C_word*)t0)[2]));
t3=(C_truep(t2)?C_i_char_equalp(C_make_character(45),C_u_i_car(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_cddr(((C_word*)t0)[2]);
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_u_i_car(t5);
t7=C_a_i_list(&a,2,lf[179],t6);
t8=C_a_i_cons(&a,2,t7,((C_word*)t0)[3]);
/* irregex.scm:261: loop2 */
t9=((C_word*)((C_word*)t0)[4])[1];
f_30158(t9,((C_word*)t0)[5],t4,t8);}
else{
t4=C_u_i_cdr(((C_word*)t0)[2]);
t5=C_u_i_car(((C_word*)t0)[2]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[3]);
/* irregex.scm:264: loop2 */
t7=((C_word*)((C_word*)t0)[4])[1];
f_30158(t7,((C_word*)t0)[5],t4,t6);}}}

/* k30307 in loop in k30046 in chicken.irregex#glob->sre in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30309,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a30773 in k18916 in k7761 in k7536 in k7533 */
static void C_ccall f_30774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +30,c,1)))){
C_save_and_reclaim((void*)f_30774,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+30);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
t4=C_a_i_cons(&a,2,t2,t3);
t5=C_a_i_cons(&a,2,lf[179],t4);
t6=C_a_i_list(&a,3,lf[115],t2,lf[124]);
t7=C_a_i_list(&a,3,lf[103],t5,t6);
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_list(&a,2,lf[127],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* a30935 in k7761 in k7536 in k7533 */
static void C_ccall f_30936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30936,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a30938 in k7761 in k7536 in k7533 */
static void C_ccall f_30939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30939,c,av);}
/* irregex-core.scm:1963: scheme#substring */
t6=*((C_word*)lf[46]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=C_i_car(t2);
av2[3]=t3;
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* a30972 in k7536 in k7533 */
static void C_ccall f_30973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_30973,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30977,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:159: irregex-match-num-submatches */
t5=*((C_word*)lf[17]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k30975 in a30972 in k7536 in k7533 */
static void C_ccall f_30977(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30977,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30980,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:160: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[500];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k30978 in k30975 in a30972 in k7536 in k7533 */
static void C_ccall f_30980(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30980,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30983,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:161: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k30981 in k30978 in k30975 in a30972 in k7536 in k7533 */
static void C_ccall f_30983(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30983,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30986,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:162: scheme#display */
t3=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[499];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k30984 in k30981 in k30978 in k30975 in a30972 in k7536 in k7533 */
static void C_ccall f_30986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_30986,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30989,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(((C_word*)t0)[4],C_fix(0));
t4=(C_truep(t3)?t3:C_fixnum_greaterp(((C_word*)t0)[4],C_fix(1)));
if(C_truep(t4)){
/* irregex-core.scm:163: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[498];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
/* irregex-core.scm:164: scheme#display */
t5=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[497];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k30987 in k30984 in k30981 in k30978 in k30975 in a30972 in k7536 in k7533 */
static void C_ccall f_30989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30989,c,av);}
/* irregex-core.scm:164: scheme#display */
t2=*((C_word*)lf[449]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[497];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k7533 */
static void C_ccall f_7535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7535,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7538,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_expand_toplevel(2,av2);}}

/* k7536 in k7533 */
static void C_ccall f_7538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,4)))){
C_save_and_reclaim((void *)f_7538,c,av);}
a=C_alloc(36);
t2=C_mutate(&lf[2] /* (set! chicken.irregex#vector-copy ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7540,a[2]=((C_word)li0),tmp=(C_word)a,a+=3,tmp));
t3=C_mutate((C_word*)lf[5]+1 /* (set! chicken.irregex#irregex? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7553,a[2]=((C_word)li1),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[7]+1 /* (set! chicken.irregex#irregex-num-submatches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7595,a[2]=((C_word)li2),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[9]+1 /* (set! chicken.irregex#irregex-names ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7613,a[2]=((C_word)li3),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate(&lf[11] /* (set! chicken.irregex#irregex-new-matches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7622,a[2]=((C_word)li4),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate(&lf[13] /* (set! chicken.irregex#irregex-reset-matches! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7645,a[2]=((C_word)li5),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate(&lf[15] /* (set! chicken.irregex#irregex-copy-matches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7654,a[2]=((C_word)li6),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[16]+1 /* (set! chicken.irregex#irregex-match-data? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7686,a[2]=((C_word)li7),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[17]+1 /* (set! chicken.irregex#irregex-match-num-submatches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7692,a[2]=((C_word)li8),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[19]+1 /* (set! chicken.irregex#irregex-match-names ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7717,a[2]=((C_word)li9),tmp=(C_word)a,a+=3,tmp));
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7763,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30973,a[2]=((C_word)li389),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:157: chicken.base#set-record-printer! */
t14=*((C_word*)lf[501]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t14;
av2[1]=t12;
av2[2]=lf[12];
av2[3]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}

/* chicken.irregex#vector-copy in k7536 in k7533 */
static void C_fcall f_7540(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_7540,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7544,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:89: scheme#make-vector */
t4=*((C_word*)lf[4]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_vector_length(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7542 in chicken.irregex#vector-copy in k7536 in k7533 */
static void C_ccall f_7544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7544,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7547,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:90: chicken.base#vector-copy! */
t3=*((C_word*)lf[3]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7545 in k7542 in chicken.irregex#vector-copy in k7536 in k7533 */
static void C_ccall f_7547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7547,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#irregex? in k7536 in k7533 */
static void C_ccall f_7553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7553,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[6]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#irregex-num-submatches in k7536 in k7533 */
static void C_ccall f_7595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7595,c,av);}
t3=C_i_check_structure_2(t2,lf[6],lf[8]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.irregex#irregex-names in k7536 in k7533 */
static void C_ccall f_7613(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7613,c,av);}
t3=C_i_check_structure_2(t2,lf[6],lf[10]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fix(7));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.irregex#irregex-new-matches in k7536 in k7533 */
static void C_fcall f_7622(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_7622,2,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7630,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7643,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:122: irregex-num-submatches */
t5=*((C_word*)lf[7]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7628 in chicken.irregex#irregex-new-matches in k7536 in k7533 */
static void C_ccall f_7630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_7630,c,av);}
a=C_alloc(6);
t2=C_slot(((C_word*)t0)[2],C_fix(7));
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record5(&a,5,lf[12],t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7641 in chicken.irregex#irregex-new-matches in k7536 in k7533 */
static void C_ccall f_7643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7643,c,av);}
a=C_alloc(10);
t2=C_a_i_fixnum_plus(&a,2,C_fix(2),t1);
t3=C_s_a_u_i_integer_times(&a,2,C_fix(4),t2);
/* irregex-core.scm:122: scheme#make-vector */
t4=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* chicken.irregex#irregex-reset-matches! in k7536 in k7533 */
static void C_fcall f_7645(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_7645,2,t1,t2);}
a=C_alloc(4);
t3=C_slot(t2,C_fix(1));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7652,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:125: scheme#vector-fill! */
t5=*((C_word*)lf[14]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k7650 in chicken.irregex#irregex-reset-matches! in k7536 in k7533 */
static void C_ccall f_7652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7652,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#irregex-copy-matches in k7536 in k7533 */
static void C_fcall f_7654(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_7654,2,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_structurep(t2,lf[12]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7668,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:131: vector-copy */
f_7540(t3,C_slot(t2,C_fix(1)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7666 in chicken.irregex#irregex-copy-matches in k7536 in k7533 */
static void C_ccall f_7668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_7668,c,av);}
a=C_alloc(6);
t2=C_slot(((C_word*)t0)[2],C_fix(2));
t3=C_slot(((C_word*)t0)[2],C_fix(3));
t4=C_slot(((C_word*)t0)[2],C_fix(4));
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_record5(&a,5,lf[12],t1,t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.irregex#irregex-match-data? in k7536 in k7533 */
static void C_ccall f_7686(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7686,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[12]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#irregex-match-num-submatches in k7536 in k7533 */
static void C_ccall f_7692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_7692,c,av);}
a=C_alloc(5);
t3=C_i_check_structure_2(t2,lf[12],lf[18]);
t4=C_slot(t2,C_fix(1));
t5=C_block_size(t4);
t6=C_fixnum_divide(t5,C_fix(4));
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_fixnum_difference(&a,2,t6,C_fix(2));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* chicken.irregex#irregex-match-names in k7536 in k7533 */
static void C_ccall f_7717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7717,c,av);}
t3=C_i_check_structure_2(t2,lf[12],lf[20]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k7761 in k7536 in k7533 */
static void C_ccall f_7763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(216,c,6)))){
C_save_and_reclaim((void *)f_7763,c,av);}
a=C_alloc(216);
t2=C_mutate((C_word*)lf[21]+1 /* (set! chicken.irregex#irregex-match-start-chunk ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7778,a[2]=((C_word)li10),tmp=(C_word)a,a+=3,tmp));
t3=C_mutate((C_word*)lf[24]+1 /* (set! chicken.irregex#irregex-match-start-index ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7803,a[2]=((C_word)li11),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[26]+1 /* (set! chicken.irregex#irregex-match-end-chunk ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7832,a[2]=((C_word)li12),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[28]+1 /* (set! chicken.irregex#irregex-match-end-index ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7861,a[2]=((C_word)li13),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate(&lf[22] /* (set! chicken.irregex#irregex-match-numeric-index ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7954,a[2]=((C_word)li15),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[36]+1 /* (set! chicken.irregex#irregex-match-valid-index? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8138,a[2]=((C_word)li16),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[40]+1 /* (set! chicken.irregex#irregex-match-substring ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8201,a[2]=((C_word)li17),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[42]+1 /* (set! chicken.irregex#irregex-match-subchunk ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8282,a[2]=((C_word)li18),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[45]+1 /* (set! chicken.irregex#make-irregex-chunker ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8372,a[2]=((C_word)li23),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate(&lf[50] /* (set! chicken.irregex#chunker-prev-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8640,a[2]=((C_word)li25),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate(&lf[51] /* (set! chicken.irregex#chunker-next-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8670,a[2]=((C_word)li26),tmp=(C_word)a,a+=3,tmp));
t13=C_a_i_list(&a,5,lf[52],C_make_character(0),C_make_character(55295),C_make_character(57344),C_make_character(1114111));
t14=C_mutate(&lf[53] /* (set! chicken.irregex#*all-chars* ...) */,t13);
t15=C_mutate(&lf[54] /* (set! chicken.irregex#string-scan-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8817,a[2]=((C_word)li28),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate(&lf[47] /* (set! chicken.irregex#string-cat-reverse ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9108,a[2]=((C_word)li31),tmp=(C_word)a,a+=3,tmp));
t17=C_mutate(&lf[57] /* (set! chicken.irregex#zero-to ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9158,a[2]=((C_word)li33),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate(&lf[58] /* (set! chicken.irregex#find ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9195,a[2]=((C_word)li35),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate(&lf[59] /* (set! chicken.irregex#last ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9256,a[2]=((C_word)li37),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate(&lf[61] /* (set! chicken.irregex#any ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9294,a[2]=((C_word)li39),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate(&lf[62] /* (set! chicken.irregex#every ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9337,a[2]=((C_word)li41),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate(&lf[56] /* (set! chicken.irregex#fold ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9382,a[2]=((C_word)li43),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate(&lf[63] /* (set! chicken.irregex#flag-join ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9712,a[2]=((C_word)li44),tmp=(C_word)a,a+=3,tmp));
t24=C_mutate(&lf[64] /* (set! chicken.irregex#flag-clear ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9721,a[2]=((C_word)li45),tmp=(C_word)a,a+=3,tmp));
t25=C_a_i_list1(&a,1,lf[65]);
t26=C_mutate(&lf[66] /* (set! chicken.irregex#close-token ...) */,t25);
t27=C_mutate(&lf[67] /* (set! chicken.irregex#with-read-from-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9739,a[2]=((C_word)li60),tmp=(C_word)a,a+=3,tmp));
t28=C_mutate(&lf[89] /* (set! chicken.irregex#symbol-list->flags ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10332,a[2]=((C_word)li62),tmp=(C_word)a,a+=3,tmp));
t29=C_mutate((C_word*)lf[101]+1 /* (set! chicken.irregex#maybe-string->sre ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10455,a[2]=((C_word)li63),tmp=(C_word)a,a+=3,tmp));
t30=C_mutate((C_word*)lf[102]+1 /* (set! chicken.irregex#string->sre ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10467,a[2]=((C_word)li90),tmp=(C_word)a,a+=3,tmp));
t31=C_a_i_cons(&a,2,C_make_character(110),C_make_character(10));
t32=C_a_i_cons(&a,2,C_make_character(114),C_make_character(13));
t33=C_a_i_cons(&a,2,C_make_character(116),C_make_character(9));
t34=C_a_i_cons(&a,2,C_make_character(97),C_make_character(7));
t35=C_a_i_cons(&a,2,C_make_character(101),C_make_character(27));
t36=C_a_i_cons(&a,2,C_make_character(102),C_make_character(12));
t37=C_a_i_list(&a,6,t31,t32,t33,t34,t35,t36);
t38=C_mutate(&lf[168] /* (set! chicken.irregex#posix-escape-sequences ...) */,t37);
t39=C_mutate(&lf[169] /* (set! chicken.irregex#string-parse-hex-escape ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13295,a[2]=((C_word)li92),tmp=(C_word)a,a+=3,tmp));
t40=C_mutate(&lf[204] /* (set! chicken.irregex#high-char? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14008,a[2]=((C_word)li93),tmp=(C_word)a,a+=3,tmp));
t41=C_mutate(&lf[106] /* (set! chicken.irregex#utf8-string-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14029,a[2]=((C_word)li95),tmp=(C_word)a,a+=3,tmp));
t42=C_mutate(&lf[206] /* (set! chicken.irregex#utf8-lowest-digit-of-length ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14262,a[2]=((C_word)li96),tmp=(C_word)a,a+=3,tmp));
t43=C_mutate(&lf[208] /* (set! chicken.irregex#char->utf8-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14338,a[2]=((C_word)li97),tmp=(C_word)a,a+=3,tmp));
t44=C_mutate(&lf[210] /* (set! chicken.irregex#unicode-range-helper ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14614,a[2]=((C_word)li100),tmp=(C_word)a,a+=3,tmp));
t45=C_mutate(&lf[212] /* (set! chicken.irregex#unicode-range-up-from ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14752,a[2]=((C_word)li103),tmp=(C_word)a,a+=3,tmp));
t46=C_mutate(&lf[213] /* (set! chicken.irregex#unicode-range-up-to ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14891,a[2]=((C_word)li106),tmp=(C_word)a,a+=3,tmp));
t47=C_mutate(&lf[214] /* (set! chicken.irregex#cset->utf8-pattern ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15469,a[2]=((C_word)li115),tmp=(C_word)a,a+=3,tmp));
t48=C_mutate((C_word*)lf[217]+1 /* (set! chicken.irregex#irregex ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16036,a[2]=((C_word)li116),tmp=(C_word)a,a+=3,tmp));
t49=C_mutate((C_word*)lf[219]+1 /* (set! chicken.irregex#string->irregex ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16164,a[2]=((C_word)li117),tmp=(C_word)a,a+=3,tmp));
t50=C_mutate((C_word*)lf[220]+1 /* (set! chicken.irregex#sre->irregex ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16174,a[2]=((C_word)li213),tmp=(C_word)a,a+=3,tmp));
t51=C_mutate(&lf[139] /* (set! chicken.irregex#sre-empty? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16307,a[2]=((C_word)li214),tmp=(C_word)a,a+=3,tmp));
t52=C_mutate(&lf[277] /* (set! chicken.irregex#sre-any? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16474,a[2]=((C_word)li215),tmp=(C_word)a,a+=3,tmp));
t53=C_mutate(&lf[136] /* (set! chicken.irregex#sre-repeater? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16575,a[2]=((C_word)li216),tmp=(C_word)a,a+=3,tmp));
t54=C_mutate(&lf[279] /* (set! chicken.irregex#sre-bos? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16620,a[2]=((C_word)li217),tmp=(C_word)a,a+=3,tmp));
t55=C_mutate(&lf[271] /* (set! chicken.irregex#sre-searcher? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16709,a[2]=((C_word)li218),tmp=(C_word)a,a+=3,tmp));
t56=C_mutate(&lf[253] /* (set! chicken.irregex#sre-consumer? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16826,a[2]=((C_word)li219),tmp=(C_word)a,a+=3,tmp));
t57=C_mutate(&lf[280] /* (set! chicken.irregex#sre-has-submatches? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16943,a[2]=((C_word)li220),tmp=(C_word)a,a+=3,tmp));
t58=C_mutate(&lf[224] /* (set! chicken.irregex#sre-count-submatches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16982,a[2]=((C_word)li222),tmp=(C_word)a,a+=3,tmp));
t59=C_mutate(&lf[104] /* (set! chicken.irregex#sre-sequence ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18191,a[2]=((C_word)li223),tmp=(C_word)a,a+=3,tmp));
t60=C_mutate(&lf[107] /* (set! chicken.irregex#sre-alternate ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18214,a[2]=((C_word)li224),tmp=(C_word)a,a+=3,tmp));
t61=C_mutate(&lf[283] /* (set! chicken.irregex#sre-strip-submatches ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18237,a[2]=((C_word)li226),tmp=(C_word)a,a+=3,tmp));
t62=C_mutate(&lf[252] /* (set! chicken.irregex#sre-names ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18418,a[2]=((C_word)li227),tmp=(C_word)a,a+=3,tmp));
t63=C_mutate(&lf[284] /* (set! chicken.irregex#sre-sequence-names ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18704,a[2]=((C_word)li228),tmp=(C_word)a,a+=3,tmp));
t64=C_mutate(&lf[270] /* (set! chicken.irregex#sre-remove-initial-bos ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18736,a[2]=((C_word)li230),tmp=(C_word)a,a+=3,tmp));
t65=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18918,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t66=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30936,a[2]=((C_word)li387),tmp=(C_word)a,a+=3,tmp);
t67=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30939,a[2]=((C_word)li388),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:1958: make-irregex-chunker */
t68=*((C_word*)lf[45]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t68;
av2[1]=t65;
av2[2]=t66;
av2[3]=*((C_word*)lf[494]+1);
av2[4]=*((C_word*)lf[495]+1);
av2[5]=*((C_word*)lf[496]+1);
av2[6]=t67;
((C_proc)(void*)(*((C_word*)t68+1)))(7,av2);}}

/* k7774 in k8140 in chicken.irregex#irregex-match-valid-index? in k7761 in k7536 in k7533 */
static void C_ccall f_7776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7776,c,av);}
t2=C_i_assq(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.irregex#irregex-match-start-chunk in k7761 in k7536 in k7533 */
static void C_ccall f_7778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_7778,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7782,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:232: irregex-match-numeric-index */
f_7954(t4,lf[23],t2,t3);}

/* k7780 in chicken.irregex#irregex-match-start-chunk in k7761 in k7536 in k7533 */
static void C_ccall f_7782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7782,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_slot(C_slot(((C_word*)t0)[3],C_fix(1)),C_fixnum_times(t1,C_fix(4))):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#irregex-match-start-index in k7761 in k7536 in k7533 */
static void C_ccall f_7803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_7803,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7807,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:235: irregex-match-numeric-index */
f_7954(t4,lf[25],t2,t3);}

/* k7805 in chicken.irregex#irregex-match-start-index in k7761 in k7536 in k7533 */
static void C_ccall f_7807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7807,c,av);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(t1,C_fix(4));
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fixnum_plus(C_fix(1),t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#irregex-match-end-chunk in k7761 in k7536 in k7533 */
static void C_ccall f_7832(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_7832,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7836,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:238: irregex-match-numeric-index */
f_7954(t4,lf[27],t2,t3);}

/* k7834 in chicken.irregex#irregex-match-end-chunk in k7761 in k7536 in k7533 */
static void C_ccall f_7836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7836,c,av);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(t1,C_fix(4));
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fixnum_plus(C_fix(2),t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#irregex-match-end-index in k7761 in k7536 in k7533 */
static void C_ccall f_7861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_7861,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7865,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:241: irregex-match-numeric-index */
f_7954(t4,lf[29],t2,t3);}

/* k7863 in chicken.irregex#irregex-match-end-index in k7761 in k7536 in k7533 */
static void C_ccall f_7865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7865,c,av);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(t1,C_fix(4));
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fixnum_plus(C_fix(3),t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#irregex-match-numeric-index in k7761 in k7536 in k7533 */
static void C_fcall f_7954(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7954,4,t1,t2,t3,t4);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8136,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:268: irregex-match-data? */
t6=*((C_word*)lf[16]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k8020 in k8134 in chicken.irregex#irregex-match-numeric-index in k7761 in k7536 in k7533 */
static void C_fcall f_8022(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_8022,2,t0,t1);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_fixnum_times(((C_word*)t0)[3],C_fix(4));
t4=C_fixnum_plus(C_fix(1),t3);
t5=C_slot(t2,t4);
t6=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?((C_word*)t0)[3]:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* irregex-core.scm:279: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=lf[33];
av2[4]=((C_word*)t0)[2];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* k8062 in k8134 in chicken.irregex#irregex-match-numeric-index in k7761 in k7536 in k7533 */
static void C_ccall f_8064(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_8064,c,av);}
a=C_alloc(9);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8066,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=((C_word)li14),tmp=(C_word)a,a+=7,tmp));
t5=((C_word*)t3)[1];
f_8066(t5,((C_word*)t0)[5],t1,C_SCHEME_TRUE);}

/* lp in k8062 in k8134 in chicken.irregex#irregex-match-numeric-index in k7761 in k7536 in k7533 */
static void C_fcall f_8066(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_8066,4,t0,t1,t2,t3);}
if(C_truep(C_i_nullp(t2))){
if(C_truep(t3)){
/* irregex-core.scm:286: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[35];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_i_caar(t2);
t5=C_eqp(((C_word*)t0)[3],t4);
if(C_truep(t5)){
t6=C_u_i_car(t2);
t7=C_u_i_cdr(t6);
t8=C_slot(((C_word*)t0)[4],C_fix(1));
t9=C_fixnum_times(t7,C_fix(4));
if(C_truep(C_slot(t8,t9))){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_i_cdar(t2);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
/* irregex-core.scm:290: lp */
t11=t1;
t12=C_u_i_cdr(t2);
t13=C_SCHEME_FALSE;
t1=t11;
t2=t12;
t3=t13;
goto loop;}}
else{
/* irregex-core.scm:291: lp */
t11=t1;
t12=C_u_i_cdr(t2);
t13=t3;
t1=t11;
t2=t12;
t3=t13;
goto loop;}}}

/* k8134 in chicken.irregex#irregex-match-numeric-index in k7761 in k7536 in k7533 */
static void C_ccall f_8136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,6)))){
C_save_and_reclaim((void *)f_8136,c,av);}
a=C_alloc(11);
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:269: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[31];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=C_i_pairp(((C_word*)t0)[5]);
if(C_truep(C_i_not(t2))){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cdr(((C_word*)t0)[5]);
if(C_truep(C_i_pairp(t3))){{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[30]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=lf[32];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
C_apply(7,av2);}}
else{
t4=C_u_i_car(((C_word*)t0)[5]);
if(C_truep(C_i_numberp(t4))){
t5=(C_truep(C_i_integerp(t4))?C_i_exactp(t4):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8022,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t7=C_slot(((C_word*)t0)[4],C_fix(1));
if(C_truep(C_fixnum_greater_or_equal_p(t4,C_fix(0)))){
t8=C_fixnum_times(t4,C_fix(4));
t9=C_block_size(t7);
t10=C_a_i_fixnum_difference(&a,2,t9,C_fix(4));
t11=t6;
f_8022(t11,C_fixnum_lessp(t8,t10));}
else{
t8=t6;
f_8022(t8,C_SCHEME_FALSE);}}
else{
/* irregex-core.scm:280: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[34];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8064,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:281: irregex-match-names */
t6=*((C_word*)lf[19]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}}}

/* chicken.irregex#irregex-match-valid-index? in k7761 in k7536 in k7533 */
static void C_ccall f_8138(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_8138,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8142,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8199,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:294: irregex-match-data? */
t6=*((C_word*)lf[16]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k8140 in chicken.irregex#irregex-match-valid-index? in k7761 in k7536 in k7533 */
static void C_ccall f_8142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_8142,c,av);}
a=C_alloc(5);
if(C_truep(C_i_integerp(((C_word*)t0)[2]))){
t2=C_i_exactp(((C_word*)t0)[2]);
if(C_truep(C_i_not(t2))){
/* irregex-core.scm:298: chicken.base#error */
t3=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[37];
av2[3]=lf[38];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=C_slot(((C_word*)t0)[4],C_fix(1));
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],C_fix(0)))){
t4=C_fixnum_times(((C_word*)t0)[2],C_fix(4));
t5=C_block_size(t3);
t6=C_a_i_fixnum_difference(&a,2,t5,C_fix(4));
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_fixnum_lessp(t4,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7776,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:227: irregex-match-names */
t3=*((C_word*)lf[19]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k8197 in chicken.irregex#irregex-match-valid-index? in k7761 in k7536 in k7533 */
static void C_ccall f_8199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8199,c,av);}
if(C_truep(C_i_not(t1))){
/* irregex-core.scm:295: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[37];
av2[3]=lf[39];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_8142(2,av2);}}}

/* chicken.irregex#irregex-match-substring in k7761 in k7536 in k7533 */
static void C_ccall f_8201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_8201,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8205,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:303: irregex-match-numeric-index */
f_7954(t4,lf[41],t2,t3);}

/* k8203 in chicken.irregex#irregex-match-substring in k7761 in k7536 in k7533 */
static void C_ccall f_8205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8205,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(3));
if(C_truep(t1)){
t3=C_i_vector_ref(t2,C_fix(4));
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_fixnum_times(t1,C_fix(4));
t6=C_slot(t4,t5);
t7=C_slot(((C_word*)t0)[2],C_fix(1));
t8=C_fixnum_times(t1,C_fix(4));
t9=C_fixnum_plus(C_fix(1),t8);
t10=C_slot(t7,t9);
t11=C_slot(((C_word*)t0)[2],C_fix(1));
t12=C_fixnum_times(t1,C_fix(4));
t13=C_fixnum_plus(C_fix(2),t12);
t14=C_slot(t11,t13);
t15=C_slot(((C_word*)t0)[2],C_fix(1));
t16=C_fixnum_times(t1,C_fix(4));
/* irregex-core.scm:305: g895 */
t17=t3;{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t17;
av2[1]=((C_word*)t0)[3];
av2[2]=t6;
av2[3]=t10;
av2[4]=t14;
av2[5]=C_slot(t15,C_fixnum_plus(C_fix(3),t16));
((C_proc)(void*)(*((C_word*)t17+1)))(6,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.irregex#irregex-match-subchunk in k7761 in k7536 in k7533 */
static void C_ccall f_8282(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_8282,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8286,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:313: irregex-match-numeric-index */
f_7954(t4,lf[44],t2,t3);}

/* k8284 in chicken.irregex#irregex-match-subchunk in k7761 in k7536 in k7533 */
static void C_ccall f_8286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8286,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(3));
t3=C_i_vector_ref(t2,C_fix(5));
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:317: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[43];
av2[3]=((C_word*)t0)[2];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
if(C_truep(t1)){
t4=C_slot(((C_word*)t0)[2],C_fix(1));
t5=C_fixnum_times(t1,C_fix(4));
t6=C_slot(t4,t5);
t7=C_slot(((C_word*)t0)[2],C_fix(1));
t8=C_fixnum_times(t1,C_fix(4));
t9=C_fixnum_plus(C_fix(1),t8);
t10=C_slot(t7,t9);
t11=C_slot(((C_word*)t0)[2],C_fix(1));
t12=C_fixnum_times(t1,C_fix(4));
t13=C_fixnum_plus(C_fix(2),t12);
t14=C_slot(t11,t13);
t15=C_slot(((C_word*)t0)[2],C_fix(1));
t16=C_fixnum_times(t1,C_fix(4));
/* irregex-core.scm:318: get-subchunk */
t17=t3;{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t17;
av2[1]=((C_word*)t0)[3];
av2[2]=t6;
av2[3]=t10;
av2[4]=t14;
av2[5]=C_slot(t15,C_fixnum_plus(C_fix(3),t16));
((C_proc)(void*)(*((C_word*)t17+1)))(6,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_ccall f_8372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +33,c,6)))){
C_save_and_reclaim((void*)f_8372,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+33);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_get_rest_arg(c,4,av,4,t0):C_SCHEME_FALSE);
t7=(C_truep(t6)?t6:(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8564,a[2]=((C_word)li19),tmp=(C_word)a,a+=3,tmp));
t8=C_i_pairp(t4);
t9=(C_truep(t8)?C_i_cdr(t4):t4);
t10=C_i_pairp(t9);
t11=(C_truep(t10)?C_u_i_car(t9):C_SCHEME_FALSE);
t12=(C_truep(t11)?t11:(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8547,a[2]=t3,a[3]=((C_word)li20),tmp=(C_word)a,a+=4,tmp));
t13=C_i_pairp(t9);
t14=(C_truep(t13)?C_u_i_cdr(t9):t9);
t15=C_i_pairp(t14);
t16=(C_truep(t15)?C_u_i_car(t14):C_SCHEME_FALSE);
t17=(C_truep(t16)?t16:(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8446,a[2]=t3,a[3]=t7,a[4]=t12,a[5]=t2,a[6]=((C_word)li22),tmp=(C_word)a,a+=7,tmp));
t18=C_i_pairp(t14);
t19=(C_truep(t18)?C_u_i_cdr(t14):t14);
t20=C_i_pairp(t19);
t21=(C_truep(t20)?C_u_i_car(t19):C_SCHEME_FALSE);
t22=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8406,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t7,a[6]=t12,a[7]=t17,a[8]=t21,tmp=(C_word)a,a+=9,tmp);
t23=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8412,a[2]=t22,a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t7,a[7]=t12,a[8]=t17,a[9]=t21,tmp=(C_word)a,a+=10,tmp);
if(C_truep(C_i_closurep(t2))){
if(C_truep(C_i_closurep(t3))){
t24=C_i_closurep(t7);
t25=t23;
f_8412(t25,(C_truep(t24)?C_i_not(C_i_closurep(t17)):C_SCHEME_TRUE));}
else{
t24=t23;
f_8412(t24,C_SCHEME_TRUE);}}
else{
t24=t23;
f_8412(t24,C_SCHEME_TRUE);}}

/* k8404 in chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_ccall f_8406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_8406,c,av);}
a=C_alloc(7);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_vector6(&a,6,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8410 in chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_fcall f_8412(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_8412,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* irregex-core.scm:356: chicken.base#error */
t2=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[48];
av2[3]=lf[49];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_a_i_vector6(&a,6,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* f_8446 in chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_ccall f_8446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_8446,c,av);}
a=C_alloc(11);
t6=C_eqp(t2,t4);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8460,a[2]=t1,a[3]=t3,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:336: get-str */
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8467,a[2]=t4,a[3]=t5,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=t1,a[9]=t3,a[10]=t2,tmp=(C_word)a,a+=11,tmp);
/* irregex-core.scm:337: get-next */
t8=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}

/* k8458 */
static void C_ccall f_8460(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8460,c,av);}
/* irregex-core.scm:336: scheme#substring */
t2=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8465 */
static void C_ccall f_8467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_8467,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8530,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,tmp=(C_word)a,a+=10,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8534,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:338: get-str */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* loop in k8528 in k8465 */
static void C_fcall f_8473(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_8473,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8491,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8495,a[2]=t5,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:343: get-str */
t7=((C_word*)t0)[5];{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8506,a[2]=t3,a[3]=((C_word*)t0)[6],a[4]=t1,a[5]=((C_word*)t0)[7],a[6]=t2,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* irregex-core.scm:347: get-next */
t6=((C_word*)t0)[8];{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k8489 in loop in k8528 in k8465 */
static void C_ccall f_8491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8491,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:342: string-cat-reverse */
f_9108(((C_word*)t0)[3],t2);}

/* k8493 in loop in k8528 in k8465 */
static void C_ccall f_8495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8495,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8499,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:344: get-start */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8497 in k8493 in loop in k8528 in k8465 */
static void C_ccall f_8499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8499,c,av);}
/* irregex-core.scm:343: scheme#substring */
t2=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8504 in loop in k8528 in k8465 */
static void C_ccall f_8506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_8506,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8514,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8518,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:348: get-str */
t4=((C_word*)t0)[8];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8512 in k8504 in loop in k8528 in k8465 */
static void C_ccall f_8514(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8514,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* irregex-core.scm:347: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8473(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k8516 in k8504 in loop in k8528 in k8465 */
static void C_ccall f_8518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_8518,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8522,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:349: get-start */
t3=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8520 in k8516 in k8504 in loop in k8528 in k8465 */
static void C_ccall f_8522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8522,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8526,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:350: get-end */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8524 in k8520 in k8516 in k8504 in loop in k8528 in k8465 */
static void C_ccall f_8526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8526,c,av);}
/* irregex-core.scm:348: scheme#substring */
t2=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8528 in k8465 */
static void C_ccall f_8530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_8530,c,av);}
a=C_alloc(15);
t2=C_a_i_list1(&a,1,t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8473,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t4,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word)li21),tmp=(C_word)a,a+=10,tmp));
t6=((C_word*)t4)[1];
f_8473(t6,((C_word*)t0)[8],((C_word*)t0)[9],t2);}

/* k8532 in k8465 */
static void C_ccall f_8534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8534,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8538,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:340: get-end */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8536 in k8532 in k8465 */
static void C_ccall f_8538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8538,c,av);}
/* irregex-core.scm:338: scheme#substring */
t2=*((C_word*)lf[46]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* f_8547 in chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_ccall f_8547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8547,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8555,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:330: get-str */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8553 */
static void C_ccall f_8555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8555,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_string_length(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_8564 in chicken.irregex#make-irregex-chunker in k7761 in k7536 in k7533 */
static void C_ccall f_8564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8564,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* lp in chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_fcall f_8622(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8622,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8626,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:371: get-next */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8624 in lp in chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_ccall f_8626(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8626,c,av);}
t2=C_eqp(t1,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(t1)){
/* irregex-core.scm:374: lp */
t3=((C_word*)((C_word*)t0)[5])[1];
f_8622(t3,((C_word*)t0)[3],t1);}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_fcall f_8640(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_8640,4,t1,t2,t3,t4);}
a=C_alloc(12);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8644,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=C_eqp(t3,t4);
if(C_truep(t6)){
t7=t5;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
f_8644(2,av2);}}
else{
t7=C_i_vector_ref(t2,C_fix(0));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8622,a[2]=t4,a[3]=t9,a[4]=t7,a[5]=((C_word)li24),tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_8622(t11,t5,t3);}}

/* k8642 in chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_ccall f_8644(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8644,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8657,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:379: g994 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k8655 in k8642 in chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_ccall f_8657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8657,c,av);}
a=C_alloc(4);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8668,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:380: g996 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8666 in k8655 in k8642 in chicken.irregex#chunker-prev-char in k7761 in k7536 in k7533 */
static void C_ccall f_8668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8668,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_string_ref(((C_word*)t0)[3],C_fixnum_difference(t1,C_fix(1)));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.irregex#chunker-next-char in k7761 in k7536 in k7533 */
static void C_fcall f_8670(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_8670,3,t1,t2,t3);}
a=C_alloc(4);
t4=C_i_vector_ref(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8677,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:383: g1002 */
t6=t4;{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k8675 in chicken.irregex#chunker-next-char in k7761 in k7536 in k7533 */
static void C_ccall f_8677(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8677,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8690,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:385: g1005 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k8688 in k8675 in chicken.irregex#chunker-next-char in k7761 in k7536 in k7533 */
static void C_ccall f_8690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8690,c,av);}
a=C_alloc(4);
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8697,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:385: g1007 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8695 in k8688 in k8675 in chicken.irregex#chunker-next-char in k7761 in k7536 in k7533 */
static void C_ccall f_8697(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8697,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_string_ref(((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a8735 */
static void C_ccall f_8736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8736,c,av);}
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:400: g1024 */
t4=C_i_vector_ref(((C_word*)t0)[3],C_fix(0));{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a8762 */
static void C_ccall f_8763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_8763,c,av);}
a=C_alloc(7);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* irregex-core.scm:408: g1035 */
t4=C_i_vector_ref(((C_word*)t0)[4],C_fix(3));{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}
else{
t4=C_i_vector_ref(((C_word*)t0)[4],C_fix(0));
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8802,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:409: g1031 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k8793 in k8800 in a8762 */
static void C_ccall f_8795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8795,c,av);}
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* irregex-core.scm:408: g1035 */
t2=C_i_vector_ref(((C_word*)t0)[4],C_fix(3));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* k8800 in a8762 */
static void C_ccall f_8802(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_8802,c,av);}
a=C_alloc(6);
if(C_truep(C_i_not(t1))){
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8795,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:410: g1033 */
t4=t2;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* irregex-core.scm:408: g1035 */
t2=C_i_vector_ref(((C_word*)t0)[2],C_fix(3));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* chicken.irregex#string-scan-char in k7761 in k7536 in k7533 */
static void C_ccall f_8817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_8817,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+9);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
t5=C_i_string_length(t2);
t6=C_i_pairp(t4);
t7=(C_truep(t6)?C_get_rest_arg(c,4,av,4,t0):C_fix(0));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8830,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=t9,a[6]=((C_word)li27),tmp=(C_word)a,a+=7,tmp));
t11=((C_word*)t9)[1];
f_8830(t11,t1,t7);}

/* scan in chicken.irregex#string-scan-char in k7761 in k7536 in k7533 */
static void C_fcall f_8830(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8830,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t2);
t5=C_eqp(((C_word*)t0)[4],t4);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:437: scan */
t8=t1;
t9=t6;
t1=t8;
t2=t9;
goto loop;}}}

/* scan in chicken.irregex#string-parse-hex-escape in k7761 in k7536 in k7533 */
static void C_fcall f_8873(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8873,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t2);
t5=C_eqp(C_make_character(125),t4);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:445: scan */
t8=t1;
t9=t6;
t1=t8;
t2=t9;
goto loop;}}}

/* scan in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_8930(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8930,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t2);
t5=C_u_i_char_numericp(t4);
if(C_truep(C_i_not(t5))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:452: scan */
t8=t1;
t9=t6;
t1=t8;
t2=t9;
goto loop;}}}

/* lp in k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_8968(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,4)))){
C_save_and_reclaim_args((void *)trf_8968,5,t0,t1,t2,t3,t4);}
a=C_alloc(23);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8971,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,a[6]=((C_word)li83),tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[3]))){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8992,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:458: collect */
t7=t5;
f_8971(t7,t6);}
else{
t6=C_i_string_ref(((C_word*)t0)[2],t2);
t7=C_eqp(C_make_character(44),t6);
if(C_truep(t7)){
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9011,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t8,a[5]=t9,tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:459: collect */
t11=t5;
f_8971(t11,t10);}
else{
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:460: lp */
t12=t1;
t13=t8;
t14=t3;
t15=t4;
t1=t12;
t2=t13;
t3=t14;
t4=t15;
goto loop;}}}

/* collect in lp in k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_fcall f_8971(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_8971,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8979,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:457: scheme#substring */
t3=*((C_word*)lf[46]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8977 in collect in lp in k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_8979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_8979,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8990 in lp in k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_8992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8992,c,av);}
/* irregex-core.scm:458: ##sys#fast-reverse */
t2=*((C_word*)lf[69]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k9009 in lp in k12196 in k12068 in k12052 in k12043 in lp in k10475 in k10469 in chicken.irregex#string->sre in k7761 in k7536 in k7533 */
static void C_ccall f_9011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9011,c,av);}
/* irregex-core.scm:459: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8968(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* chicken.irregex#string-cat-reverse in k7761 in k7536 in k7533 */
static void C_fcall f_9108(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_9108,2,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9116,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9118,a[2]=((C_word)li30),tmp=(C_word)a,a+=3,tmp);
/* irregex-core.scm:484: fold */
f_9382(t3,t4,C_fix(0),t2);}

/* k9114 in chicken.irregex#string-cat-reverse in k7761 in k7536 in k7533 */
static void C_ccall f_9116(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9116,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9132,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:488: scheme#make-string */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a9117 in chicken.irregex#string-cat-reverse in k7761 in k7536 in k7533 */
static void C_ccall f_9118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9118,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_plus(C_i_string_length(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k9130 in k9114 in chicken.irregex#string-cat-reverse in k7761 in k7536 in k7533 */
static void C_ccall f_9132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9132,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9137,a[2]=t1,a[3]=((C_word)li29),tmp=(C_word)a,a+=4,tmp);
t3=(
  f_9137(t2,((C_word*)t0)[2],((C_word*)t0)[3])
);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* lp in k9130 in k9114 in chicken.irregex#string-cat-reverse in k7761 in k7536 in k7533 */
static C_word C_fcall f_9137(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t2))){
t3=C_u_i_car(t2);
t4=C_i_string_length(t3);
t5=C_fixnum_difference(t1,t4);
t6=C_substring_copy(t3,((C_word*)t0)[2],C_fix(0),t4,t5);
t8=t5;
t9=C_u_i_cdr(t2);
t1=t8;
t2=t9;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
return(t3);}}

/* chicken.irregex#zero-to in k7761 in k7536 in k7533 */
static void C_fcall f_9158(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_9158,2,t1,t2);}
a=C_alloc(6);
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(0)))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_fixnum_difference(t2,C_fix(1));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9174,a[2]=t5,a[3]=((C_word)li32),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_9174(t7,t1,t3,C_SCHEME_END_OF_LIST);}}

/* lp in chicken.irregex#zero-to in k7761 in k7536 in k7533 */
static void C_fcall f_9174(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_9174,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_zerop(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,C_fix(0),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fixnum_difference(t2,C_fix(1));
t5=C_a_i_cons(&a,2,t2,t3);
/* irregex-core.scm:506: lp */
t7=t1;
t8=t4;
t9=t5;
t1=t7;
t2=t8;
t3=t9;
goto loop;}}

/* chicken.irregex#find in k7761 in k7536 in k7533 */
static void C_fcall f_9195(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9195,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9201,a[2]=t5,a[3]=t2,a[4]=((C_word)li34),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_9201(t7,t1,t3);}

/* lp in chicken.irregex#find in k7761 in k7536 in k7533 */
static void C_fcall f_9201(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9201,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9214,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:513: pred */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k9212 in lp in chicken.irregex#find in k7761 in k7536 in k7533 */
static void C_ccall f_9214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9214,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* irregex-core.scm:514: lp */
t2=((C_word*)((C_word*)t0)[4])[1];
f_9201(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[3]));}}

/* chicken.irregex#last in k7761 in k7536 in k7533 */
static void C_fcall f_9256(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_9256,2,t1,t2);}
a=C_alloc(3);
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
/* irregex-core.scm:524: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[60];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9271,a[2]=((C_word)li36),tmp=(C_word)a,a+=3,tmp);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(
  f_9271(t2)
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* lp in chicken.irregex#last in k7761 in k7536 in k7533 */
static C_word C_fcall f_9271(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
t2=C_i_cdr(t1);
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}
else{
return(C_u_i_car(t1));}}

/* chicken.irregex#any in k7761 in k7536 in k7533 */
static void C_fcall f_9294(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_9294,3,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_pairp(t3))){
t4=C_u_i_car(t3);
t5=C_u_i_cdr(t3);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9310,a[2]=t2,a[3]=t7,a[4]=((C_word)li38),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_9310(t9,t1,t4,t5);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* lp in chicken.irregex#any in k7761 in k7536 in k7533 */
static void C_fcall f_9310(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9310,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t3))){
/* irregex-core.scm:534: pred */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9323,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:535: pred */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k9321 in lp in chicken.irregex#any in k7761 in k7536 in k7533 */
static void C_ccall f_9323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9323,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* irregex-core.scm:535: lp */
t2=((C_word*)((C_word*)t0)[3])[1];
f_9310(t2,((C_word*)t0)[2],C_i_car(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.irregex#every in k7761 in k7536 in k7533 */
static void C_fcall f_9337(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_9337,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_nullp(t3);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_car(t3);
t6=C_u_i_cdr(t3);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9355,a[2]=t2,a[3]=t8,a[4]=((C_word)li40),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_9355(t10,t1,t5,t6);}}

/* lp in chicken.irregex#every in k7761 in k7536 in k7533 */
static void C_fcall f_9355(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9355,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t3))){
/* irregex-core.scm:541: pred */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9371,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:542: pred */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k9369 in lp in chicken.irregex#every in k7761 in k7536 in k7533 */
static void C_ccall f_9371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9371,c,av);}
if(C_truep(t1)){
/* irregex-core.scm:542: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9355(t2,((C_word*)t0)[3],C_i_car(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.irregex#fold in k7761 in k7536 in k7533 */
static void C_fcall f_9382(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_9382,4,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9388,a[2]=t6,a[3]=t2,a[4]=((C_word)li42),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_9388(t8,t1,t4,t3);}

/* lp in chicken.irregex#fold in k7761 in k7536 in k7533 */
static void C_fcall f_9388(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9388,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9406,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:548: kons */
t6=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_u_i_car(t2);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* k9404 in lp in chicken.irregex#fold in k7761 in k7536 in k7533 */
static void C_ccall f_9406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9406,c,av);}
/* irregex-core.scm:548: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9388(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.irregex#flag-join in k7761 in k7536 in k7533 */
static void C_ccall f_9712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9712,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_fixnum_or(t2,t3):t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.irregex#flag-clear in k7761 in k7536 in k7533 */
static void C_ccall f_9721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9721,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_and(t2,C_fixnum_not(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.irregex#with-read-from-string in k7761 in k7536 in k7533 */
static void C_fcall f_9739(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_9739,4,t1,t2,t3,t4);}
a=C_alloc(15);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_i_string_length(t2);
t10=C_set_block_item(t6,0,t9);
t11=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9746,a[2]=t6,a[3]=t2,a[4]=t8,a[5]=((C_word)li58),tmp=(C_word)a,a+=6,tmp));
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10311,a[2]=t2,a[3]=t4,a[4]=((C_word)li59),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:714: read */
t13=((C_word*)t8)[1];
f_9746(t13,t1,t3,t12);}

/* f_9746 in chicken.irregex#with-read-from-string in k7761 in k7536 in k7533 */
static void C_fcall f_9746(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,5)))){
C_save_and_reclaim_args((void *)trf_9746,4,t0,t1,t2,t3);}
a=C_alloc(19);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)((C_word*)t0)[2])[1]))){
/* irregex-core.scm:631: chicken.base#error */
t4=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[68];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t2);
switch(t4){
case C_make_character(40):
t5=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9773,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t7,a[6]=((C_word)li49),tmp=(C_word)a,a+=7,tmp));
t9=((C_word*)t7)[1];
f_9773(t9,t1,t5,C_SCHEME_END_OF_LIST);
case C_make_character(41):
t5=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:655: k */
t6=t3;{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[66];
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}
case C_make_character(59):
t5=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9887,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=t7,a[7]=((C_word)li50),tmp=(C_word)a,a+=8,tmp));
t9=((C_word*)t7)[1];
f_9887(t9,t1,t5);
default:
t5=C_eqp(t4,C_make_character(39));
t6=(C_truep(t5)?t5:C_eqp(t4,C_make_character(96)));
if(C_truep(t6)){
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9936,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t3,a[5]=((C_word)li51),tmp=(C_word)a,a+=6,tmp);
/* irregex-core.scm:662: read */
t41=t1;
t42=t7;
t43=t8;
t1=t41;
t2=t42;
t3=t43;
goto loop;}
else{
switch(t4){
case C_make_character(44):
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9964,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
if(C_truep(C_fixnum_lessp(t8,((C_word*)((C_word*)t0)[2])[1]))){
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t10=t7;
f_9964(t10,C_eqp(C_make_character(64),C_i_string_ref(((C_word*)t0)[3],t9)));}
else{
t9=t7;
f_9964(t9,C_SCHEME_FALSE);}
case C_make_character(34):
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t8=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10021,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t10,a[6]=((C_word)li54),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_10021(t12,t1,t7,t8,C_SCHEME_END_OF_LIST);
case C_make_character(35):
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t8=C_i_string_ref(((C_word*)t0)[3],t7);
switch(t8){
case C_make_character(59):
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10122,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word)li55),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:684: read */
t41=t1;
t42=t9;
t43=t10;
t1=t41;
t2=t42;
t3=t43;
goto loop;
case C_make_character(92):
t9=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10140,a[2]=t3,a[3]=((C_word)li56),tmp=(C_word)a,a+=4,tmp);
/* irregex-core.scm:686: read */
t41=t1;
t42=t9;
t43=t10;
t1=t41;
t2=t42;
t3=t43;
goto loop;
default:
t9=C_eqp(t8,C_make_character(116));
t10=(C_truep(t9)?t9:C_eqp(t8,C_make_character(102)));
if(C_truep(t10)){
t11=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t12=C_i_string_ref(((C_word*)t0)[3],t11);
t13=C_eqp(C_make_character(116),t12);
t14=C_a_i_fixnum_plus(&a,2,t2,C_fix(2));
/* irregex-core.scm:697: k */
t15=t3;{
C_word av2[4];
av2[0]=t15;
av2[1]=t1;
av2[2]=t13;
av2[3]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(4,av2);}}
else{
/* irregex-core.scm:699: chicken.base#error */
t11=*((C_word*)lf[30]+1);{
C_word av2[4];
av2[0]=t11;
av2[1]=t1;
av2[2]=lf[83];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}}
default:
if(C_truep(C_u_i_char_whitespacep(C_i_string_ref(((C_word*)t0)[3],t2)))){
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
/* irregex-core.scm:703: read */
t41=t1;
t42=t7;
t43=t3;
t1=t41;
t2=t42;
t3=t43;
goto loop;}
else{
t7=C_a_i_fixnum_plus(&a,2,t2,C_fix(1));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10240,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=t9,a[7]=((C_word)li57),tmp=(C_word)a,a+=8,tmp));
t11=((C_word*)t9)[1];
f_10240(t11,t1,t7);}}}}}}

/* lp */
static void C_fcall f_9773(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_9773,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9779,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li48),tmp=(C_word)a,a+=8,tmp);
/* irregex-core.scm:636: read */
t5=((C_word*)((C_word*)t0)[4])[1];
f_9746(t5,t1,t2,t4);}

/* a9778 in lp */
static void C_ccall f_9779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_9779,c,av);}
a=C_alloc(7);
t4=C_eqp(t2,lf[66]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9793,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:641: ##sys#fast-reverse */
t6=*((C_word*)lf[69]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_eqp(t2,lf[70]);
if(C_truep(t5)){
if(C_truep(C_i_nullp(((C_word*)t0)[3]))){
/* irregex-core.scm:644: chicken.base#error */
t6=*((C_word*)lf[30]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[71];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9813,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word)li47),tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:645: read */
t7=((C_word*)((C_word*)t0)[5])[1];
f_9746(t7,t1,t3,t6);}}
else{
t6=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
/* irregex-core.scm:653: lp */
t7=((C_word*)((C_word*)t0)[6])[1];
f_9773(t7,t1,t3,t6);}}}

/* k9791 in a9778 in lp */
static void C_ccall f_9793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9793,c,av);}
/* irregex-core.scm:641: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a9812 in a9778 in lp */
static void C_ccall f_9813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_9813,c,av);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9819,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=((C_word)li46),tmp=(C_word)a,a+=7,tmp);
/* irregex-core.scm:646: read */
t5=((C_word*)((C_word*)t0)[5])[1];
f_9746(t5,t1,t3,t4);}

/* a9818 in a9812 in a9778 in lp */
static void C_ccall f_9819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_9819,c,av);}
a=C_alloc(10);
t4=C_eqp(t2,lf[66]);
if(C_truep(C_i_not(t4))){
/* irregex-core.scm:648: chicken.base#error */
t5=*((C_word*)lf[30]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[72];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9836,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9840,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:649: ##sys#fast-reverse */
t7=*((C_word*)lf[69]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k9834 in a9818 in a9812 in a9778 in lp */
static void C_ccall f_9836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9836,c,av);}
/* irregex-core.scm:649: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k9838 in a9818 in a9812 in a9778 in lp */
static void C_ccall f_9840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9840,c,av);}
a=C_alloc(3);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
/* irregex-core.scm:649: scheme#append */
t4=*((C_word*)lf[73]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* skip */
static void C_fcall f_9887(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_9887,3,t0,t1,t2);}
t3=C_fixnum_greater_or_equal_p(t2,((C_word*)((C_word*)t0)[2])[1]);
t4=(C_truep(t3)?t3:C_eqp(C_make_character(10),C_i_string_ref(((C_word*)t0)[3],t2)));
if(C_truep(t4)){
/* irregex-core.scm:659: read */
t5=((C_word*)((C_word*)t0)[4])[1];
f_9746(t5,t1,C_fixnum_plus(t2,C_fix(1)),((C_word*)t0)[5]);}
else{
/* irregex-core.scm:660: skip */
t6=t1;
t7=C_fixnum_plus(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}

/* a9935 */
static void C_ccall f_9936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9936,c,av);}
a=C_alloc(6);
t4=C_i_string_ref(((C_word*)t0)[2],((C_word*)t0)[3]);
t5=C_eqp(C_make_character(39),t4);
t6=(C_truep(t5)?lf[74]:lf[75]);
t7=C_a_i_list2(&a,2,t6,t2);
/* irregex-core.scm:665: k */
t8=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t8;
av2[1]=t1;
av2[2]=t7;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}

/* k9962 */
static void C_fcall f_9964(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_9964,2,t0,t1);}
a=C_alloc(10);
t2=(C_truep(t1)?lf[76]:lf[77]);
t3=(C_truep(t1)?C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(2)):C_a_i_fixnum_plus(&a,2,((C_word*)t0)[2],C_fix(1)));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9975,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word)li52),tmp=(C_word)a,a+=5,tmp);
/* irregex-core.scm:670: read */
t5=((C_word*)((C_word*)t0)[4])[1];
f_9746(t5,((C_word*)t0)[5],t3,t4);}

/* a9974 in k9962 */
static void C_ccall f_9975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
C_check_for_interrupt;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9975,c,av);}
a=C_alloc(6);
t4=C_a_i_list2(&a,2,((C_word*)t0)[2],t2);
/* irregex-core.scm:670: k */
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_irregex_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("irregex"));
C_check_nursery_minimum(C_calculate_demand(19,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void*)C_irregex_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(2387))){
C_save(t1);
C_rereclaim2(2387*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(19);
C_initialize_lf(lf,502);
lf[0]=C_h_intern(&lf[0],7, C_text("irregex"));
lf[1]=C_h_intern(&lf[1],16, C_text("chicken.irregex#"));
lf[3]=C_h_intern(&lf[3],25, C_text("chicken.base#vector-copy!"));
lf[4]=C_h_intern(&lf[4],18, C_text("scheme#make-vector"));
lf[5]=C_h_intern(&lf[5],24, C_text("chicken.irregex#irregex\077"));
lf[6]=C_h_intern(&lf[6],6, C_text("regexp"));
lf[7]=C_h_intern(&lf[7],38, C_text("chicken.irregex#irregex-num-submatches"));
lf[8]=C_h_intern(&lf[8],22, C_text("irregex-num-submatches"));
lf[9]=C_h_intern(&lf[9],29, C_text("chicken.irregex#irregex-names"));
lf[10]=C_h_intern(&lf[10],13, C_text("irregex-names"));
lf[12]=C_h_intern(&lf[12],12, C_text("regexp-match"));
lf[14]=C_h_intern(&lf[14],19, C_text("scheme#vector-fill!"));
lf[16]=C_h_intern(&lf[16],35, C_text("chicken.irregex#irregex-match-data\077"));
lf[17]=C_h_intern(&lf[17],44, C_text("chicken.irregex#irregex-match-num-submatches"));
lf[18]=C_h_intern(&lf[18],28, C_text("irregex-match-num-submatches"));
lf[19]=C_h_intern(&lf[19],35, C_text("chicken.irregex#irregex-match-names"));
lf[20]=C_h_intern(&lf[20],19, C_text("irregex-match-names"));
lf[21]=C_h_intern(&lf[21],41, C_text("chicken.irregex#irregex-match-start-chunk"));
lf[23]=C_h_intern(&lf[23],25, C_text("irregex-match-start-chunk"));
lf[24]=C_h_intern(&lf[24],41, C_text("chicken.irregex#irregex-match-start-index"));
lf[25]=C_h_intern(&lf[25],25, C_text("irregex-match-start-index"));
lf[26]=C_h_intern(&lf[26],39, C_text("chicken.irregex#irregex-match-end-chunk"));
lf[27]=C_h_intern(&lf[27],23, C_text("irregex-match-end-chunk"));
lf[28]=C_h_intern(&lf[28],39, C_text("chicken.irregex#irregex-match-end-index"));
lf[29]=C_h_intern(&lf[29],23, C_text("irregex-match-end-index"));
lf[30]=C_h_intern(&lf[30],18, C_text("chicken.base#error"));
lf[31]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016not match data"));
lf[32]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022too many arguments"));
lf[33]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021not a valid index"));
lf[34]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[35]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022unknown match name"));
lf[36]=C_h_intern(&lf[36],42, C_text("chicken.irregex#irregex-match-valid-index\077"));
lf[37]=C_h_intern(&lf[37],26, C_text("irregex-match-valid-index\077"));
lf[38]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[39]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016not match data"));
lf[40]=C_h_intern(&lf[40],39, C_text("chicken.irregex#irregex-match-substring"));
lf[41]=C_h_intern(&lf[41],23, C_text("irregex-match-substring"));
lf[42]=C_h_intern(&lf[42],38, C_text("chicken.irregex#irregex-match-subchunk"));
lf[43]=C_decode_literal(C_heaptop,C_text("\376B\000\0000this chunk type does not support match subchunks"));
lf[44]=C_h_intern(&lf[44],22, C_text("irregex-match-subchunk"));
lf[45]=C_h_intern(&lf[45],36, C_text("chicken.irregex#make-irregex-chunker"));
lf[46]=C_h_intern(&lf[46],16, C_text("scheme#substring"));
lf[48]=C_h_intern(&lf[48],20, C_text("make-irregex-chunker"));
lf[49]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023expected a procdure"));
lf[52]=C_h_intern(&lf[52],1, C_text("/"));
lf[55]=C_h_intern(&lf[55],18, C_text("scheme#make-string"));
lf[60]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035can\047t take last of empty list"));
lf[65]=C_h_intern(&lf[65],5, C_text("close"));
lf[68]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031unterminated embedded SRE"));
lf[69]=C_h_intern(&lf[69],18, C_text("##sys#fast-reverse"));
lf[70]=C_h_intern(&lf[70],1, C_text("."));
lf[71]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017bad dotted form"));
lf[72]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017bad dotted form"));
lf[73]=C_h_intern(&lf[73],13, C_text("scheme#append"));
lf[74]=C_h_intern(&lf[74],5, C_text("quote"));
lf[75]=C_h_intern(&lf[75],10, C_text("quasiquote"));
lf[76]=C_h_intern(&lf[76],15, C_text("uquote-splicing"));
lf[77]=C_h_intern(&lf[77],7, C_text("unquote"));
lf[78]=C_decode_literal(C_heaptop,C_text("\376B\000\000#unterminated string in embedded SRE"));
lf[79]=C_h_intern(&lf[79],5, C_text("space"));
lf[80]=C_h_intern(&lf[80],7, C_text("newline"));
lf[81]=C_h_intern(&lf[81],20, C_text("##sys#number->string"));
lf[82]=C_h_intern(&lf[82],21, C_text("scheme#symbol->string"));
lf[83]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad # syntax in simplified SRE"));
lf[84]=C_h_intern(&lf[84],21, C_text("scheme#string->symbol"));
lf[85]=C_h_intern(&lf[85],21, C_text("scheme#string->number"));
lf[86]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000;\376\003\000\000\002\376\377\012\000\000(\376\003\000\000\002\376\377\012\000\000)\376\003\000\000\002\376\377\012\000\000\042\376\003\000\000\002\376\377\012\000\000#\376\003\000\000\002\376\377\012\000\000\134\376\377\016"));
lf[87]=C_h_intern(&lf[87],11, C_text("close-token"));
lf[88]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025unexpected \047)\047 in SRE"));
lf[90]=C_h_intern(&lf[90],1, C_text("i"));
lf[91]=C_h_intern(&lf[91],1, C_text("m"));
lf[92]=C_h_intern(&lf[92],10, C_text("multi-line"));
lf[93]=C_h_intern(&lf[93],1, C_text("s"));
lf[94]=C_h_intern(&lf[94],11, C_text("single-line"));
lf[95]=C_h_intern(&lf[95],1, C_text("x"));
lf[96]=C_h_intern(&lf[96],12, C_text("ignore-space"));
lf[97]=C_h_intern(&lf[97],1, C_text("u"));
lf[98]=C_h_intern(&lf[98],4, C_text("utf8"));
lf[99]=C_h_intern(&lf[99],2, C_text("ci"));
lf[100]=C_h_intern(&lf[100],16, C_text("case-insensitive"));
lf[101]=C_h_intern(&lf[101],33, C_text("chicken.irregex#maybe-string->sre"));
lf[102]=C_h_intern(&lf[102],27, C_text("chicken.irregex#string->sre"));
lf[103]=C_h_intern(&lf[103],2, C_text("or"));
lf[105]=C_h_intern(&lf[105],18, C_text("##sys#string->list"));
lf[108]=C_h_intern(&lf[108],8, C_text("submatch"));
lf[109]=C_h_intern(&lf[109],2, C_text("if"));
lf[110]=C_h_intern(&lf[110],7, C_text("epsilon"));
lf[111]=C_h_intern(&lf[111],10, C_text("look-ahead"));
lf[112]=C_h_intern(&lf[112],14, C_text("neg-look-ahead"));
lf[113]=C_h_intern(&lf[113],11, C_text("look-behind"));
lf[114]=C_h_intern(&lf[114],15, C_text("neg-look-behind"));
lf[115]=C_h_intern(&lf[115],3, C_text("seq"));
lf[116]=C_h_intern(&lf[116],13, C_text("scheme#cddadr"));
lf[117]=C_h_intern(&lf[117],13, C_text("scheme#cadadr"));
lf[118]=C_h_intern(&lf[118],12, C_text("##sys#append"));
lf[119]=C_h_intern(&lf[119],2, C_text("=>"));
lf[120]=C_h_intern(&lf[120],14, C_text("submatch-named"));
lf[121]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001=>\376\003\000\000\002\376\001\000\000\016\001submatch-named\376\377\016"));
lf[122]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001atomic\376\003\000\000\002\376\001\000\000\002\001if\376\003\000\000\002\376\001\000\000\012\001look-ahead\376\003\000\000\002\376\001\000\000\016\001neg-look-ahead\376\003\000\000"
"\002\376\001\000\000\013\001look-behind\376\003\000\000\002\376\001\000\000\017\001neg-look-behind\376\003\000\000\002\376\001\000\000\002\001=>\376\003\000\000\002\376\001\000\000\016\001submatch-nam"
"ed\376\003\000\000\002\376\001\000\000\006\001w/utf8\376\003\000\000\002\376\001\000\000\010\001w/noutf8\376\377\016"));
lf[123]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unterminated parenthesis in regexp"));
lf[124]=C_h_intern(&lf[124],3, C_text("any"));
lf[125]=C_h_intern(&lf[125],4, C_text("nonl"));
lf[126]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034\077 can\047t follow empty pattern"));
lf[127]=C_h_intern(&lf[127],1, C_text("\052"));
lf[128]=C_h_intern(&lf[128],2, C_text("\052\077"));
lf[129]=C_h_intern(&lf[129],1, C_text("+"));
lf[130]=C_h_intern(&lf[130],3, C_text("\052\052\077"));
lf[131]=C_h_intern(&lf[131],1, C_text("\077"));
lf[132]=C_h_intern(&lf[132],2, C_text("\077\077"));
lf[133]=C_h_intern(&lf[133],2, C_text("\052\052"));
lf[134]=C_h_intern(&lf[134],1, C_text("="));
lf[135]=C_h_intern(&lf[135],2, C_text(">="));
lf[137]=C_decode_literal(C_heaptop,C_text("\376B\000\000)duplicate repetition (e.g. \052\052) in pattern"));
lf[138]=C_decode_literal(C_heaptop,C_text("\376B\000\000%can\047t repeat empty pattern (e.g. ()\052)"));
lf[140]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unterminated parenthesis in regexp"));
lf[141]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000\077\376\003\000\000\002\376\377\012\000\000\052\376\377\016"));
lf[142]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unterminated parenthesis in regexp"));
lf[143]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037unterminated (\052\047...) SRE escape"));
lf[144]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047bad regexp syntax: (\052FOO) not supported"));
lf[145]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\012\001look-ahead\376\377\016"));
lf[146]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\016\001neg-look-ahead\376\377\016"));
lf[147]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unterminated parenthesis in regexp"));
lf[148]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\013\001look-behind\376\377\016"));
lf[149]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\017\001neg-look-behind\376\377\016"));
lf[150]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024invalid (\077< sequence"));
lf[151]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001atomic\376\377\016"));
lf[152]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unterminated parenthesis in regexp"));
lf[153]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035invalid conditional reference"));
lf[154]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001if\376\377\016"));
lf[155]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036unsupported Perl-style cluster"));
lf[156]=C_h_intern(&lf[156],6, C_text("w/utf8"));
lf[157]=C_h_intern(&lf[157],8, C_text("w/noutf8"));
lf[158]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022incomplete cluster"));
lf[159]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036unknown regex cluster modifier"));
lf[160]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026too many )\047s in regexp"));
lf[161]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023incomplete char set"));
lf[163]=C_h_intern(&lf[163],19, C_text("scheme#vector->list"));
lf[166]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014bad char-set"));
lf[167]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032inverted range in char-set"));
lf[170]=C_decode_literal(C_heaptop,C_text("\376\000\000\001\000\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000"
"\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376"
"\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000"
"\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001"
"\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001"
"\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000"
"\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377"
"\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000"
"\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376"
"\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000"
"\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001"
"\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001"
"\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000"
"\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377"
"\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000"
"\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376"
"\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\001\376\377\001\000\000\000\002\376\377\001\000"
"\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002"
"\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001"
"\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\002\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000"
"\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377\001\000\000\000\003\376\377"
"\001\000\000\000\003\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\004\376\377\001\000\000\000\005\376\377\001\000\000\000\005\376\377\001\000\000"
"\000\005\376\377\001\000\000\000\005\376\377\001\000\000\000\006\376\377\001\000\000\000\006\376\377\001\000\000\000\000\376\377\001\000\000\000\000"));
lf[171]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032incomplete character class"));
lf[173]=C_decode_literal(C_heaptop,C_text("\376B\000\000!collating sequences not supported"));
lf[174]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034{ can\047t follow empty pattern"));
lf[175]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035invalid {n} repetition syntax"));
lf[176]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[177]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032incomplete escape sequence"));
lf[178]=C_h_intern(&lf[178],7, C_text("numeric"));
lf[179]=C_h_intern(&lf[179],1, C_text("~"));
lf[180]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001_"));
lf[181]=C_h_intern(&lf[181],12, C_text("alphanumeric"));
lf[182]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001_"));
lf[183]=C_h_intern(&lf[183],3, C_text("bow"));
lf[184]=C_h_intern(&lf[184],3, C_text("eow"));
lf[185]=C_h_intern(&lf[185],3, C_text("nwb"));
lf[186]=C_h_intern(&lf[186],3, C_text("bos"));
lf[187]=C_h_intern(&lf[187],3, C_text("eos"));
lf[188]=C_h_intern(&lf[188],5, C_text("reset"));
lf[189]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000<\376\003\000\000\002\376\377\012\000\000{\376\003\000\000\002\376\377\012\000\000\047\376\377\016"));
lf[190]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad \134k usage, expected \134k<...>"));
lf[191]=C_h_intern(&lf[191],10, C_text("backref-ci"));
lf[192]=C_h_intern(&lf[192],7, C_text("backref"));
lf[193]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032unterminated named backref"));
lf[194]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027unknown escape sequence"));
lf[195]=C_h_intern(&lf[195],3, C_text("bol"));
lf[196]=C_h_intern(&lf[196],3, C_text("eol"));
lf[197]=C_h_intern(&lf[197],11, C_text("string->sre"));
lf[198]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021expected a string"));
lf[199]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025incomplete hex escape"));
lf[200]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033incomplete hex brace escape"));
lf[201]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024bad hex brace escape"));
lf[202]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025incomplete hex escape"));
lf[203]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016bad hex escape"));
lf[205]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid utf8 length"));
lf[207]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid utf8 length"));
lf[209]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037unicode codepoint out of range:"));
lf[211]=C_h_intern(&lf[211],3, C_text("map"));
lf[215]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid utf8 length"));
lf[217]=C_h_intern(&lf[217],23, C_text("chicken.irregex#irregex"));
lf[218]=C_decode_literal(C_heaptop,C_text("\376\000\000\000\013\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\006\000\376\377\001\000\000\000\000"));
lf[219]=C_h_intern(&lf[219],31, C_text("chicken.irregex#string->irregex"));
lf[220]=C_h_intern(&lf[220],28, C_text("chicken.irregex#sre->irregex"));
lf[221]=C_h_intern(&lf[221],5, C_text("small"));
lf[222]=C_h_intern(&lf[222],4, C_text("fast"));
lf[225]=C_h_intern(&lf[225],6, C_text("w/case"));
lf[226]=C_h_intern(&lf[226],8, C_text("w/nocase"));
lf[227]=C_h_intern(&lf[227],1, C_text(":"));
lf[228]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024invalid sre: empty \052"));
lf[229]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025invalid sre: empty \052\077"));
lf[230]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024invalid sre: empty +"));
lf[231]=C_h_intern(&lf[231],4, C_text("word"));
lf[232]=C_h_intern(&lf[232],5, C_text("word+"));
lf[233]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001_"));
lf[234]=C_h_intern(&lf[234],1, C_text("&"));
lf[235]=C_h_intern(&lf[235],12, C_text("posix-string"));
lf[236]=C_h_intern(&lf[236],10, C_text("scheme#not"));
lf[237]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\003\001any\376\377\016"));
lf[238]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001eos\376\377\016"));
lf[239]=C_h_intern(&lf[239],6, C_text("atomic"));
lf[240]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037unknown named backref in SRE IF"));
lf[241]=C_h_intern(&lf[241],18, C_text("scheme#string-ci=\077"));
lf[242]=C_h_intern(&lf[242],15, C_text("scheme#string=\077"));
lf[243]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025unknown backreference"));
lf[244]=C_h_intern(&lf[244],3, C_text("dsm"));
lf[245]=C_h_intern(&lf[245],1, C_text("$"));
lf[246]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027unknown regexp operator"));
lf[247]=C_h_intern(&lf[247],1, C_text("-"));
lf[249]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016unknown regexp"));
lf[250]=C_h_intern(&lf[250],16, C_text("scheme#char-ci=\077"));
lf[251]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016unknown regexp"));
lf[254]=C_h_intern(&lf[254],10, C_text("scheme#max"));
lf[255]=C_h_intern(&lf[255],10, C_text("scheme#min"));
lf[256]=C_h_intern(&lf[256],10, C_text("sre-length"));
lf[257]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025invalid backreference"));
lf[258]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035invalid forward backreference"));
lf[259]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025unknown backreference"));
lf[260]=C_h_intern(&lf[260],3, C_text(">=\077"));
lf[261]=C_h_intern(&lf[261],17, C_text("sre-length-ranges"));
lf[262]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024unknown sre operator"));
lf[263]=C_h_intern(&lf[263],2, C_text("=\077"));
lf[264]=C_h_intern(&lf[264],6, C_text("commit"));
lf[265]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013unknown sre"));
lf[266]=C_h_intern(&lf[266],11, C_text("scheme#cons"));
lf[269]=C_h_intern(&lf[269],9, C_text("backtrack"));
lf[272]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\003\001any\376\377\016"));
lf[273]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\004\001nonl\376\377\016"));
lf[274]=C_h_intern(&lf[274],8, C_text("utf8-any"));
lf[275]=C_h_intern(&lf[275],9, C_text("utf8-nonl"));
lf[276]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\001epsilon\376\003\000\000\002\376\001\000\000\003\001bos\376\003\000\000\002\376\001\000\000\003\001eos\376\003\000\000\002\376\001\000\000\003\001bol\376\003\000\000\002\376\001\000\000\003\001eol\376\003\000\000\002\376"
"\001\000\000\003\001bow\376\003\000\000\002\376\001\000\000\003\001eow\376\003\000\000\002\376\001\000\000\006\001commit\376\377\016"));
lf[278]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001\052\376\003\000\000\002\376\001\000\000\001\001+\376\377\016"));
lf[281]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001$\376\003\000\000\002\376\001\000\000\010\001submatch\376\003\000\000\002\376\001\000\000\002\001=>\376\003\000\000\002\376\001\000\000\016\001submatch-named\376\377\016"));
lf[282]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\002\001or\376\377\016"));
lf[286]=C_h_intern(&lf[286],30, C_text("chicken.irregex#irregex-search"));
lf[287]=C_h_intern(&lf[287],38, C_text("chicken.irregex#irregex-search/chunked"));
lf[288]=C_h_intern(&lf[288],14, C_text("irregex-search"));
lf[289]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[290]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[291]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[293]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016not an integer"));
lf[294]=C_h_intern(&lf[294],11, C_text("irregex-dfa"));
lf[297]=C_h_intern(&lf[297],11, C_text("irregex-nfa"));
lf[298]=C_h_intern(&lf[298],29, C_text("chicken.irregex#irregex-match"));
lf[299]=C_h_intern(&lf[299],37, C_text("chicken.irregex#irregex-match/chunked"));
lf[300]=C_h_intern(&lf[300],13, C_text("irregex-match"));
lf[301]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[302]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[303]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[304]=C_h_intern(&lf[304],30, C_text("chicken.irregex#irregex-match\077"));
lf[306]=C_h_intern(&lf[306],8, C_text("for-each"));
lf[307]=C_h_intern(&lf[307],10, C_text("alphabetic"));
lf[308]=C_h_intern(&lf[308],5, C_text("alpha"));
lf[309]=C_h_intern(&lf[309],8, C_text("alphanum"));
lf[310]=C_h_intern(&lf[310],5, C_text("alnum"));
lf[311]=C_h_intern(&lf[311],10, C_text("lower-case"));
lf[312]=C_h_intern(&lf[312],5, C_text("lower"));
lf[313]=C_h_intern(&lf[313],10, C_text("upper-case"));
lf[314]=C_h_intern(&lf[314],5, C_text("upper"));
lf[315]=C_h_intern(&lf[315],3, C_text("num"));
lf[316]=C_h_intern(&lf[316],5, C_text("digit"));
lf[317]=C_h_intern(&lf[317],11, C_text("punctuation"));
lf[318]=C_h_intern(&lf[318],5, C_text("punct"));
lf[319]=C_h_intern(&lf[319],7, C_text("graphic"));
lf[320]=C_h_intern(&lf[320],5, C_text("graph"));
lf[321]=C_h_intern(&lf[321],5, C_text("blank"));
lf[322]=C_h_intern(&lf[322],10, C_text("whitespace"));
lf[323]=C_h_intern(&lf[323],5, C_text("white"));
lf[324]=C_h_intern(&lf[324],8, C_text("printing"));
lf[325]=C_h_intern(&lf[325],5, C_text("print"));
lf[326]=C_h_intern(&lf[326],7, C_text("control"));
lf[327]=C_h_intern(&lf[327],5, C_text("cntrl"));
lf[328]=C_h_intern(&lf[328],9, C_text("hex-digit"));
lf[329]=C_h_intern(&lf[329],6, C_text("xdigit"));
lf[330]=C_h_intern(&lf[330],5, C_text("ascii"));
lf[331]=C_h_intern(&lf[331],10, C_text("ascii-nonl"));
lf[332]=C_h_intern(&lf[332],14, C_text("utf8-tail-char"));
lf[333]=C_h_intern(&lf[333],11, C_text("utf8-2-char"));
lf[334]=C_h_intern(&lf[334],11, C_text("utf8-3-char"));
lf[335]=C_h_intern(&lf[335],11, C_text("utf8-4-char"));
lf[336]=C_h_intern(&lf[336],7, C_text("integer"));
lf[337]=C_h_intern(&lf[337],4, C_text("real"));
lf[338]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016!$%&\052/:<=>\077^_~"));
lf[339]=C_h_intern(&lf[339],14, C_text("symbol-initial"));
lf[340]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004+-.@"));
lf[341]=C_h_intern(&lf[341],17, C_text("symbol-subsequent"));
lf[342]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002+-"));
lf[343]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[344]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002.."));
lf[345]=C_h_intern(&lf[345],6, C_text("symbol"));
lf[346]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001;"));
lf[347]=C_h_intern(&lf[347],10, C_text("sexp-space"));
lf[348]=C_h_intern(&lf[348],6, C_text("escape"));
lf[349]=C_h_intern(&lf[349],6, C_text("string"));
lf[350]=C_decode_literal(C_heaptop,C_text("\376B\000\000\00212"));
lf[351]=C_h_intern(&lf[351],10, C_text("ipv4-digit"));
lf[352]=C_h_intern(&lf[352],12, C_text("ipv4-address"));
lf[353]=C_h_intern(&lf[353],12, C_text("ipv6-address"));
lf[354]=C_h_intern(&lf[354],10, C_text("ip-address"));
lf[355]=C_h_intern(&lf[355],11, C_text("domain-atom"));
lf[356]=C_h_intern(&lf[356],6, C_text("domain"));
lf[357]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004arpa"));
lf[358]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003com"));
lf[359]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003gov"));
lf[360]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003mil"));
lf[361]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003net"));
lf[362]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003org"));
lf[363]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003edu"));
lf[364]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004aero"));
lf[365]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003biz"));
lf[366]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004coop"));
lf[367]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004info"));
lf[368]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006museum"));
lf[369]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004name"));
lf[370]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003pro"));
lf[371]=C_h_intern(&lf[371],16, C_text("top-level-domain"));
lf[372]=C_h_intern(&lf[372],13, C_text("domain/common"));
lf[373]=C_h_intern(&lf[373],16, C_text("email-local-part"));
lf[374]=C_h_intern(&lf[374],5, C_text("email"));
lf[375]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001%"));
lf[376]=C_h_intern(&lf[376],8, C_text("url-char"));
lf[377]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001%"));
lf[378]=C_h_intern(&lf[378],14, C_text("url-final-char"));
lf[379]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001s"));
lf[380]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001:"));
lf[381]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001/"));
lf[382]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\077"));
lf[383]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001#"));
lf[384]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001/"));
lf[385]=C_h_intern(&lf[385],8, C_text("http-url"));
lf[386]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004http"));
lf[387]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003://"));
lf[390]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[392]=C_h_intern(&lf[392],12, C_text("scheme#cadar"));
lf[393]=C_h_intern(&lf[393],12, C_text("scheme#cddar"));
lf[394]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\001epsilon\376\377\016"));
lf[395]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034non-procedure in op position"));
lf[401]=C_h_intern(&lf[401],13, C_text("scheme#caddar"));
lf[406]=C_decode_literal(C_heaptop,C_text("\376B\000\000!not a valid sre char-set operator"));
lf[407]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030not a valid sre char-set"));
lf[409]=C_h_intern(&lf[409],19, C_text("scheme#list->vector"));
lf[411]=C_h_intern(&lf[411],12, C_text("irregex-fold"));
lf[412]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[413]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024not an exact integer"));
lf[414]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[415]=C_h_intern(&lf[415],28, C_text("chicken.irregex#irregex-fold"));
lf[416]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017not a procedure"));
lf[418]=C_h_intern(&lf[418],20, C_text("irregex-fold/chunked"));
lf[419]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016not an integer"));
lf[420]=C_h_intern(&lf[420],36, C_text("chicken.irregex#irregex-fold/chunked"));
lf[421]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017not a procedure"));
lf[422]=C_h_intern(&lf[422],31, C_text("chicken.irregex#irregex-replace"));
lf[424]=C_h_intern(&lf[424],15, C_text("irregex-replace"));
lf[425]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[426]=C_h_intern(&lf[426],35, C_text("chicken.irregex#irregex-replace/all"));
lf[427]=C_h_intern(&lf[427],19, C_text("irregex-replace/all"));
lf[428]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[429]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[430]=C_h_intern(&lf[430],3, C_text("pre"));
lf[431]=C_h_intern(&lf[431],4, C_text("post"));
lf[432]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031unknown match replacement"));
lf[433]=C_h_intern(&lf[433],31, C_text("chicken.irregex#irregex-extract"));
lf[434]=C_h_intern(&lf[434],15, C_text("irregex-extract"));
lf[435]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[436]=C_h_intern(&lf[436],29, C_text("chicken.irregex#irregex-split"));
lf[437]=C_h_intern(&lf[437],20, C_text("scheme#string-append"));
lf[438]=C_h_intern(&lf[438],13, C_text("irregex-split"));
lf[439]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014not a string"));
lf[440]=C_h_intern(&lf[440],29, C_text("chicken.irregex#irregex-quote"));
lf[441]=C_h_intern(&lf[441],18, C_text("##sys#list->string"));
lf[442]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017\134|[](){}.\052+\077^$#"));
lf[443]=C_h_intern(&lf[443],27, C_text("chicken.irregex#irregex-opt"));
lf[444]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[445]=C_h_intern(&lf[445],12, C_text("scheme#char\077"));
lf[446]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376B\000\000\000\376\377\016"));
lf[448]=C_h_intern(&lf[448],30, C_text("chicken.base#get-output-string"));
lf[449]=C_h_intern(&lf[449],14, C_text("scheme#display"));
lf[450]=C_h_intern(&lf[450],21, C_text("##sys#write-char/port"));
lf[451]=C_h_intern(&lf[451],31, C_text("chicken.base#open-output-string"));
lf[452]=C_h_intern(&lf[452],27, C_text("chicken.irregex#sre->string"));
lf[453]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[454]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003(\077:"));
lf[455]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[456]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[457]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[458]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001>"));
lf[459]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003(\077<"));
lf[460]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001|"));
lf[461]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[462]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003(\077:"));
lf[463]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[464]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003(\077:"));
lf[465]=C_h_intern(&lf[465],3, C_text("not"));
lf[466]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[467]=C_h_intern(&lf[467],12, C_text("scheme#cdadr"));
lf[468]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002[^"));
lf[469]=C_decode_literal(C_heaptop,C_text("\376B\000\000(can\047t represent general \047not\047 in strings"));
lf[470]=C_h_intern(&lf[470],4, C_text("cset"));
lf[471]=C_h_intern(&lf[471],12, C_text("scheme#caadr"));
lf[472]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[473]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001["));
lf[474]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001]"));
lf[475]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002[^"));
lf[476]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[477]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001:"));
lf[478]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001-"));
lf[479]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002(\077"));
lf[480]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024unknown sre operator"));
lf[481]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001^"));
lf[482]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001$"));
lf[483]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[484]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022unknown sre symbol"));
lf[485]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023unknown sre pattern"));
lf[486]=C_h_intern(&lf[486],25, C_text("chicken.irregex#glob->sre"));
lf[487]=C_h_intern(&lf[487],9, C_text("glob->sre"));
lf[488]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003./\134"));
lf[489]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002/\134"));
lf[490]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003./\134"));
lf[491]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002/\134"));
lf[492]=C_decode_literal(C_heaptop,C_text("\376B\000\000!unexpected end of character class"));
lf[493]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000\134\376\003\000\000\002\376\377\012\000\000/\376\377\016"));
lf[494]=C_h_intern(&lf[494],10, C_text("scheme#car"));
lf[495]=C_h_intern(&lf[495],11, C_text("scheme#cadr"));
lf[496]=C_h_intern(&lf[496],12, C_text("scheme#caddr"));
lf[497]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002)>"));
lf[498]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002es"));
lf[499]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011 submatch"));
lf[500]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020#<regexp-match ("));
lf[501]=C_h_intern(&lf[501],32, C_text("chicken.base#set-record-printer!"));
C_register_lf2(lf,502,create_ptable());{}
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7535,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t4;
C_library_toplevel(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[1211] = {
{C_text("f33619:irregex_2escm"),(void*)f33619},
{C_text("f33623:irregex_2escm"),(void*)f33623},
{C_text("f_10021:irregex_2escm"),(void*)f_10021},
{C_text("f_10024:irregex_2escm"),(void*)f_10024},
{C_text("f_10038:irregex_2escm"),(void*)f_10038},
{C_text("f_10063:irregex_2escm"),(void*)f_10063},
{C_text("f_10070:irregex_2escm"),(void*)f_10070},
{C_text("f_10089:irregex_2escm"),(void*)f_10089},
{C_text("f_10122:irregex_2escm"),(void*)f_10122},
{C_text("f_10140:irregex_2escm"),(void*)f_10140},
{C_text("f_10160:irregex_2escm"),(void*)f_10160},
{C_text("f_10240:irregex_2escm"),(void*)f_10240},
{C_text("f_10250:irregex_2escm"),(void*)f_10250},
{C_text("f_10253:irregex_2escm"),(void*)f_10253},
{C_text("f_10260:irregex_2escm"),(void*)f_10260},
{C_text("f_10263:irregex_2escm"),(void*)f_10263},
{C_text("f_10311:irregex_2escm"),(void*)f_10311},
{C_text("f_10332:irregex_2escm"),(void*)f_10332},
{C_text("f_10338:irregex_2escm"),(void*)f_10338},
{C_text("f_10356:irregex_2escm"),(void*)f_10356},
{C_text("f_10367:irregex_2escm"),(void*)f_10367},
{C_text("f_10455:irregex_2escm"),(void*)f_10455},
{C_text("f_10467:irregex_2escm"),(void*)f_10467},
{C_text("f_10471:irregex_2escm"),(void*)f_10471},
{C_text("f_10477:irregex_2escm"),(void*)f_10477},
{C_text("f_10482:irregex_2escm"),(void*)f_10482},
{C_text("f_10485:irregex_2escm"),(void*)f_10485},
{C_text("f_10507:irregex_2escm"),(void*)f_10507},
{C_text("f_10524:irregex_2escm"),(void*)f_10524},
{C_text("f_10527:irregex_2escm"),(void*)f_10527},
{C_text("f_10529:irregex_2escm"),(void*)f_10529},
{C_text("f_10563:irregex_2escm"),(void*)f_10563},
{C_text("f_10577:irregex_2escm"),(void*)f_10577},
{C_text("f_10581:irregex_2escm"),(void*)f_10581},
{C_text("f_10583:irregex_2escm"),(void*)f_10583},
{C_text("f_10590:irregex_2escm"),(void*)f_10590},
{C_text("f_10599:irregex_2escm"),(void*)f_10599},
{C_text("f_10619:irregex_2escm"),(void*)f_10619},
{C_text("f_10623:irregex_2escm"),(void*)f_10623},
{C_text("f_10627:irregex_2escm"),(void*)f_10627},
{C_text("f_10655:irregex_2escm"),(void*)f_10655},
{C_text("f_10659:irregex_2escm"),(void*)f_10659},
{C_text("f_10662:irregex_2escm"),(void*)f_10662},
{C_text("f_10665:irregex_2escm"),(void*)f_10665},
{C_text("f_10668:irregex_2escm"),(void*)f_10668},
{C_text("f_10673:irregex_2escm"),(void*)f_10673},
{C_text("f_10676:irregex_2escm"),(void*)f_10676},
{C_text("f_10746:irregex_2escm"),(void*)f_10746},
{C_text("f_10762:irregex_2escm"),(void*)f_10762},
{C_text("f_10838:irregex_2escm"),(void*)f_10838},
{C_text("f_10849:irregex_2escm"),(void*)f_10849},
{C_text("f_10863:irregex_2escm"),(void*)f_10863},
{C_text("f_10885:irregex_2escm"),(void*)f_10885},
{C_text("f_10888:irregex_2escm"),(void*)f_10888},
{C_text("f_10900:irregex_2escm"),(void*)f_10900},
{C_text("f_10960:irregex_2escm"),(void*)f_10960},
{C_text("f_10989:irregex_2escm"),(void*)f_10989},
{C_text("f_11120:irregex_2escm"),(void*)f_11120},
{C_text("f_11126:irregex_2escm"),(void*)f_11126},
{C_text("f_11141:irregex_2escm"),(void*)f_11141},
{C_text("f_11212:irregex_2escm"),(void*)f_11212},
{C_text("f_11242:irregex_2escm"),(void*)f_11242},
{C_text("f_11252:irregex_2escm"),(void*)f_11252},
{C_text("f_11274:irregex_2escm"),(void*)f_11274},
{C_text("f_11309:irregex_2escm"),(void*)f_11309},
{C_text("f_11324:irregex_2escm"),(void*)f_11324},
{C_text("f_11350:irregex_2escm"),(void*)f_11350},
{C_text("f_11373:irregex_2escm"),(void*)f_11373},
{C_text("f_11396:irregex_2escm"),(void*)f_11396},
{C_text("f_11438:irregex_2escm"),(void*)f_11438},
{C_text("f_11461:irregex_2escm"),(void*)f_11461},
{C_text("f_11464:irregex_2escm"),(void*)f_11464},
{C_text("f_11490:irregex_2escm"),(void*)f_11490},
{C_text("f_11494:irregex_2escm"),(void*)f_11494},
{C_text("f_11498:irregex_2escm"),(void*)f_11498},
{C_text("f_11550:irregex_2escm"),(void*)f_11550},
{C_text("f_11575:irregex_2escm"),(void*)f_11575},
{C_text("f_11578:irregex_2escm"),(void*)f_11578},
{C_text("f_11608:irregex_2escm"),(void*)f_11608},
{C_text("f_11612:irregex_2escm"),(void*)f_11612},
{C_text("f_11627:irregex_2escm"),(void*)f_11627},
{C_text("f_11630:irregex_2escm"),(void*)f_11630},
{C_text("f_11651:irregex_2escm"),(void*)f_11651},
{C_text("f_11655:irregex_2escm"),(void*)f_11655},
{C_text("f_11678:irregex_2escm"),(void*)f_11678},
{C_text("f_11709:irregex_2escm"),(void*)f_11709},
{C_text("f_11712:irregex_2escm"),(void*)f_11712},
{C_text("f_11721:irregex_2escm"),(void*)f_11721},
{C_text("f_11865:irregex_2escm"),(void*)f_11865},
{C_text("f_11896:irregex_2escm"),(void*)f_11896},
{C_text("f_11987:irregex_2escm"),(void*)f_11987},
{C_text("f_12002:irregex_2escm"),(void*)f_12002},
{C_text("f_12022:irregex_2escm"),(void*)f_12022},
{C_text("f_12026:irregex_2escm"),(void*)f_12026},
{C_text("f_12045:irregex_2escm"),(void*)f_12045},
{C_text("f_12054:irregex_2escm"),(void*)f_12054},
{C_text("f_12070:irregex_2escm"),(void*)f_12070},
{C_text("f_12073:irregex_2escm"),(void*)f_12073},
{C_text("f_12076:irregex_2escm"),(void*)f_12076},
{C_text("f_12079:irregex_2escm"),(void*)f_12079},
{C_text("f_12088:irregex_2escm"),(void*)f_12088},
{C_text("f_12198:irregex_2escm"),(void*)f_12198},
{C_text("f_12274:irregex_2escm"),(void*)f_12274},
{C_text("f_12301:irregex_2escm"),(void*)f_12301},
{C_text("f_12324:irregex_2escm"),(void*)f_12324},
{C_text("f_12351:irregex_2escm"),(void*)f_12351},
{C_text("f_12378:irregex_2escm"),(void*)f_12378},
{C_text("f_12409:irregex_2escm"),(void*)f_12409},
{C_text("f_12444:irregex_2escm"),(void*)f_12444},
{C_text("f_12467:irregex_2escm"),(void*)f_12467},
{C_text("f_12490:irregex_2escm"),(void*)f_12490},
{C_text("f_12521:irregex_2escm"),(void*)f_12521},
{C_text("f_12544:irregex_2escm"),(void*)f_12544},
{C_text("f_12567:irregex_2escm"),(void*)f_12567},
{C_text("f_12590:irregex_2escm"),(void*)f_12590},
{C_text("f_12613:irregex_2escm"),(void*)f_12613},
{C_text("f_12636:irregex_2escm"),(void*)f_12636},
{C_text("f_12647:irregex_2escm"),(void*)f_12647},
{C_text("f_12667:irregex_2escm"),(void*)f_12667},
{C_text("f_12671:irregex_2escm"),(void*)f_12671},
{C_text("f_12695:irregex_2escm"),(void*)f_12695},
{C_text("f_12698:irregex_2escm"),(void*)f_12698},
{C_text("f_12701:irregex_2escm"),(void*)f_12701},
{C_text("f_12736:irregex_2escm"),(void*)f_12736},
{C_text("f_12740:irregex_2escm"),(void*)f_12740},
{C_text("f_12767:irregex_2escm"),(void*)f_12767},
{C_text("f_12775:irregex_2escm"),(void*)f_12775},
{C_text("f_12835:irregex_2escm"),(void*)f_12835},
{C_text("f_12878:irregex_2escm"),(void*)f_12878},
{C_text("f_12890:irregex_2escm"),(void*)f_12890},
{C_text("f_12899:irregex_2escm"),(void*)f_12899},
{C_text("f_12919:irregex_2escm"),(void*)f_12919},
{C_text("f_12923:irregex_2escm"),(void*)f_12923},
{C_text("f_12927:irregex_2escm"),(void*)f_12927},
{C_text("f_12979:irregex_2escm"),(void*)f_12979},
{C_text("f_12995:irregex_2escm"),(void*)f_12995},
{C_text("f_13059:irregex_2escm"),(void*)f_13059},
{C_text("f_13085:irregex_2escm"),(void*)f_13085},
{C_text("f_13114:irregex_2escm"),(void*)f_13114},
{C_text("f_13142:irregex_2escm"),(void*)f_13142},
{C_text("f_13163:irregex_2escm"),(void*)f_13163},
{C_text("f_13181:irregex_2escm"),(void*)f_13181},
{C_text("f_13295:irregex_2escm"),(void*)f_13295},
{C_text("f_13314:irregex_2escm"),(void*)f_13314},
{C_text("f_13326:irregex_2escm"),(void*)f_13326},
{C_text("f_13329:irregex_2escm"),(void*)f_13329},
{C_text("f_13360:irregex_2escm"),(void*)f_13360},
{C_text("f_13363:irregex_2escm"),(void*)f_13363},
{C_text("f_13404:irregex_2escm"),(void*)f_13404},
{C_text("f_13439:irregex_2escm"),(void*)f_13439},
{C_text("f_13449:irregex_2escm"),(void*)f_13449},
{C_text("f_13456:irregex_2escm"),(void*)f_13456},
{C_text("f_13477:irregex_2escm"),(void*)f_13477},
{C_text("f_13487:irregex_2escm"),(void*)f_13487},
{C_text("f_13504:irregex_2escm"),(void*)f_13504},
{C_text("f_13521:irregex_2escm"),(void*)f_13521},
{C_text("f_13532:irregex_2escm"),(void*)f_13532},
{C_text("f_13533:irregex_2escm"),(void*)f_13533},
{C_text("f_13554:irregex_2escm"),(void*)f_13554},
{C_text("f_13566:irregex_2escm"),(void*)f_13566},
{C_text("f_13576:irregex_2escm"),(void*)f_13576},
{C_text("f_13679:irregex_2escm"),(void*)f_13679},
{C_text("f_13688:irregex_2escm"),(void*)f_13688},
{C_text("f_13694:irregex_2escm"),(void*)f_13694},
{C_text("f_13697:irregex_2escm"),(void*)f_13697},
{C_text("f_13707:irregex_2escm"),(void*)f_13707},
{C_text("f_13714:irregex_2escm"),(void*)f_13714},
{C_text("f_13718:irregex_2escm"),(void*)f_13718},
{C_text("f_13762:irregex_2escm"),(void*)f_13762},
{C_text("f_13801:irregex_2escm"),(void*)f_13801},
{C_text("f_13811:irregex_2escm"),(void*)f_13811},
{C_text("f_13815:irregex_2escm"),(void*)f_13815},
{C_text("f_13819:irregex_2escm"),(void*)f_13819},
{C_text("f_13834:irregex_2escm"),(void*)f_13834},
{C_text("f_13842:irregex_2escm"),(void*)f_13842},
{C_text("f_13846:irregex_2escm"),(void*)f_13846},
{C_text("f_13865:irregex_2escm"),(void*)f_13865},
{C_text("f_13918:irregex_2escm"),(void*)f_13918},
{C_text("f_13932:irregex_2escm"),(void*)f_13932},
{C_text("f_13936:irregex_2escm"),(void*)f_13936},
{C_text("f_13940:irregex_2escm"),(void*)f_13940},
{C_text("f_13950:irregex_2escm"),(void*)f_13950},
{C_text("f_14008:irregex_2escm"),(void*)f_14008},
{C_text("f_14029:irregex_2escm"),(void*)f_14029},
{C_text("f_14032:irregex_2escm"),(void*)f_14032},
{C_text("f_14224:irregex_2escm"),(void*)f_14224},
{C_text("f_14262:irregex_2escm"),(void*)f_14262},
{C_text("f_14338:irregex_2escm"),(void*)f_14338},
{C_text("f_14479:irregex_2escm"),(void*)f_14479},
{C_text("f_14482:irregex_2escm"),(void*)f_14482},
{C_text("f_14503:irregex_2escm"),(void*)f_14503},
{C_text("f_14508:irregex_2escm"),(void*)f_14508},
{C_text("f_14540:irregex_2escm"),(void*)f_14540},
{C_text("f_14565:irregex_2escm"),(void*)f_14565},
{C_text("f_14569:irregex_2escm"),(void*)f_14569},
{C_text("f_14580:irregex_2escm"),(void*)f_14580},
{C_text("f_14588:irregex_2escm"),(void*)f_14588},
{C_text("f_14614:irregex_2escm"),(void*)f_14614},
{C_text("f_14644:irregex_2escm"),(void*)f_14644},
{C_text("f_14654:irregex_2escm"),(void*)f_14654},
{C_text("f_14662:irregex_2escm"),(void*)f_14662},
{C_text("f_14678:irregex_2escm"),(void*)f_14678},
{C_text("f_14680:irregex_2escm"),(void*)f_14680},
{C_text("f_14716:irregex_2escm"),(void*)f_14716},
{C_text("f_14752:irregex_2escm"),(void*)f_14752},
{C_text("f_14772:irregex_2escm"),(void*)f_14772},
{C_text("f_14774:irregex_2escm"),(void*)f_14774},
{C_text("f_14806:irregex_2escm"),(void*)f_14806},
{C_text("f_14813:irregex_2escm"),(void*)f_14813},
{C_text("f_14819:irregex_2escm"),(void*)f_14819},
{C_text("f_14835:irregex_2escm"),(void*)f_14835},
{C_text("f_14837:irregex_2escm"),(void*)f_14837},
{C_text("f_14877:irregex_2escm"),(void*)f_14877},
{C_text("f_14883:irregex_2escm"),(void*)f_14883},
{C_text("f_14891:irregex_2escm"),(void*)f_14891},
{C_text("f_14911:irregex_2escm"),(void*)f_14911},
{C_text("f_14913:irregex_2escm"),(void*)f_14913},
{C_text("f_14945:irregex_2escm"),(void*)f_14945},
{C_text("f_14952:irregex_2escm"),(void*)f_14952},
{C_text("f_14958:irregex_2escm"),(void*)f_14958},
{C_text("f_14974:irregex_2escm"),(void*)f_14974},
{C_text("f_14976:irregex_2escm"),(void*)f_14976},
{C_text("f_15016:irregex_2escm"),(void*)f_15016},
{C_text("f_15022:irregex_2escm"),(void*)f_15022},
{C_text("f_15041:irregex_2escm"),(void*)f_15041},
{C_text("f_15050:irregex_2escm"),(void*)f_15050},
{C_text("f_15062:irregex_2escm"),(void*)f_15062},
{C_text("f_15074:irregex_2escm"),(void*)f_15074},
{C_text("f_15080:irregex_2escm"),(void*)f_15080},
{C_text("f_15082:irregex_2escm"),(void*)f_15082},
{C_text("f_15130:irregex_2escm"),(void*)f_15130},
{C_text("f_15138:irregex_2escm"),(void*)f_15138},
{C_text("f_15145:irregex_2escm"),(void*)f_15145},
{C_text("f_15151:irregex_2escm"),(void*)f_15151},
{C_text("f_15183:irregex_2escm"),(void*)f_15183},
{C_text("f_15185:irregex_2escm"),(void*)f_15185},
{C_text("f_15235:irregex_2escm"),(void*)f_15235},
{C_text("f_15253:irregex_2escm"),(void*)f_15253},
{C_text("f_15278:irregex_2escm"),(void*)f_15278},
{C_text("f_15321:irregex_2escm"),(void*)f_15321},
{C_text("f_15323:irregex_2escm"),(void*)f_15323},
{C_text("f_15412:irregex_2escm"),(void*)f_15412},
{C_text("f_15414:irregex_2escm"),(void*)f_15414},
{C_text("f_15469:irregex_2escm"),(void*)f_15469},
{C_text("f_15477:irregex_2escm"),(void*)f_15477},
{C_text("f_15479:irregex_2escm"),(void*)f_15479},
{C_text("f_15493:irregex_2escm"),(void*)f_15493},
{C_text("f_15497:irregex_2escm"),(void*)f_15497},
{C_text("f_15515:irregex_2escm"),(void*)f_15515},
{C_text("f_15524:irregex_2escm"),(void*)f_15524},
{C_text("f_15539:irregex_2escm"),(void*)f_15539},
{C_text("f_15591:irregex_2escm"),(void*)f_15591},
{C_text("f_15594:irregex_2escm"),(void*)f_15594},
{C_text("f_15651:irregex_2escm"),(void*)f_15651},
{C_text("f_15662:irregex_2escm"),(void*)f_15662},
{C_text("f_15664:irregex_2escm"),(void*)f_15664},
{C_text("f_15689:irregex_2escm"),(void*)f_15689},
{C_text("f_15712:irregex_2escm"),(void*)f_15712},
{C_text("f_15723:irregex_2escm"),(void*)f_15723},
{C_text("f_15725:irregex_2escm"),(void*)f_15725},
{C_text("f_15750:irregex_2escm"),(void*)f_15750},
{C_text("f_15766:irregex_2escm"),(void*)f_15766},
{C_text("f_15775:irregex_2escm"),(void*)f_15775},
{C_text("f_15781:irregex_2escm"),(void*)f_15781},
{C_text("f_15791:irregex_2escm"),(void*)f_15791},
{C_text("f_15798:irregex_2escm"),(void*)f_15798},
{C_text("f_15833:irregex_2escm"),(void*)f_15833},
{C_text("f_15835:irregex_2escm"),(void*)f_15835},
{C_text("f_15860:irregex_2escm"),(void*)f_15860},
{C_text("f_15889:irregex_2escm"),(void*)f_15889},
{C_text("f_15891:irregex_2escm"),(void*)f_15891},
{C_text("f_15916:irregex_2escm"),(void*)f_15916},
{C_text("f_15974:irregex_2escm"),(void*)f_15974},
{C_text("f_15984:irregex_2escm"),(void*)f_15984},
{C_text("f_15987:irregex_2escm"),(void*)f_15987},
{C_text("f_15989:irregex_2escm"),(void*)f_15989},
{C_text("f_16036:irregex_2escm"),(void*)f_16036},
{C_text("f_16043:irregex_2escm"),(void*)f_16043},
{C_text("f_16097:irregex_2escm"),(void*)f_16097},
{C_text("f_16164:irregex_2escm"),(void*)f_16164},
{C_text("f_16172:irregex_2escm"),(void*)f_16172},
{C_text("f_16174:irregex_2escm"),(void*)f_16174},
{C_text("f_16178:irregex_2escm"),(void*)f_16178},
{C_text("f_16181:irregex_2escm"),(void*)f_16181},
{C_text("f_16184:irregex_2escm"),(void*)f_16184},
{C_text("f_16187:irregex_2escm"),(void*)f_16187},
{C_text("f_16193:irregex_2escm"),(void*)f_16193},
{C_text("f_16196:irregex_2escm"),(void*)f_16196},
{C_text("f_16199:irregex_2escm"),(void*)f_16199},
{C_text("f_16202:irregex_2escm"),(void*)f_16202},
{C_text("f_16205:irregex_2escm"),(void*)f_16205},
{C_text("f_16220:irregex_2escm"),(void*)f_16220},
{C_text("f_16234:irregex_2escm"),(void*)f_16234},
{C_text("f_16239:irregex_2escm"),(void*)f_16239},
{C_text("f_16265:irregex_2escm"),(void*)f_16265},
{C_text("f_16269:irregex_2escm"),(void*)f_16269},
{C_text("f_16307:irregex_2escm"),(void*)f_16307},
{C_text("f_16324:irregex_2escm"),(void*)f_16324},
{C_text("f_16367:irregex_2escm"),(void*)f_16367},
{C_text("f_16474:irregex_2escm"),(void*)f_16474},
{C_text("f_16497:irregex_2escm"),(void*)f_16497},
{C_text("f_16575:irregex_2escm"),(void*)f_16575},
{C_text("f_16620:irregex_2escm"),(void*)f_16620},
{C_text("f_16637:irregex_2escm"),(void*)f_16637},
{C_text("f_16709:irregex_2escm"),(void*)f_16709},
{C_text("f_16713:irregex_2escm"),(void*)f_16713},
{C_text("f_16750:irregex_2escm"),(void*)f_16750},
{C_text("f_16826:irregex_2escm"),(void*)f_16826},
{C_text("f_16830:irregex_2escm"),(void*)f_16830},
{C_text("f_16867:irregex_2escm"),(void*)f_16867},
{C_text("f_16880:irregex_2escm"),(void*)f_16880},
{C_text("f_16943:irregex_2escm"),(void*)f_16943},
{C_text("f_16969:irregex_2escm"),(void*)f_16969},
{C_text("f_16982:irregex_2escm"),(void*)f_16982},
{C_text("f_16988:irregex_2escm"),(void*)f_16988},
{C_text("f_17009:irregex_2escm"),(void*)f_17009},
{C_text("f_17015:irregex_2escm"),(void*)f_17015},
{C_text("f_17045:irregex_2escm"),(void*)f_17045},
{C_text("f_17082:irregex_2escm"),(void*)f_17082},
{C_text("f_17085:irregex_2escm"),(void*)f_17085},
{C_text("f_17092:irregex_2escm"),(void*)f_17092},
{C_text("f_17094:irregex_2escm"),(void*)f_17094},
{C_text("f_17097:irregex_2escm"),(void*)f_17097},
{C_text("f_17137:irregex_2escm"),(void*)f_17137},
{C_text("f_17153:irregex_2escm"),(void*)f_17153},
{C_text("f_17166:irregex_2escm"),(void*)f_17166},
{C_text("f_17173:irregex_2escm"),(void*)f_17173},
{C_text("f_17206:irregex_2escm"),(void*)f_17206},
{C_text("f_17236:irregex_2escm"),(void*)f_17236},
{C_text("f_17251:irregex_2escm"),(void*)f_17251},
{C_text("f_17288:irregex_2escm"),(void*)f_17288},
{C_text("f_17304:irregex_2escm"),(void*)f_17304},
{C_text("f_17308:irregex_2escm"),(void*)f_17308},
{C_text("f_17321:irregex_2escm"),(void*)f_17321},
{C_text("f_17344:irregex_2escm"),(void*)f_17344},
{C_text("f_17347:irregex_2escm"),(void*)f_17347},
{C_text("f_17354:irregex_2escm"),(void*)f_17354},
{C_text("f_17356:irregex_2escm"),(void*)f_17356},
{C_text("f_17370:irregex_2escm"),(void*)f_17370},
{C_text("f_17384:irregex_2escm"),(void*)f_17384},
{C_text("f_17409:irregex_2escm"),(void*)f_17409},
{C_text("f_17413:irregex_2escm"),(void*)f_17413},
{C_text("f_17490:irregex_2escm"),(void*)f_17490},
{C_text("f_17503:irregex_2escm"),(void*)f_17503},
{C_text("f_17539:irregex_2escm"),(void*)f_17539},
{C_text("f_17548:irregex_2escm"),(void*)f_17548},
{C_text("f_17648:irregex_2escm"),(void*)f_17648},
{C_text("f_17657:irregex_2escm"),(void*)f_17657},
{C_text("f_17675:irregex_2escm"),(void*)f_17675},
{C_text("f_17678:irregex_2escm"),(void*)f_17678},
{C_text("f_17696:irregex_2escm"),(void*)f_17696},
{C_text("f_17743:irregex_2escm"),(void*)f_17743},
{C_text("f_17817:irregex_2escm"),(void*)f_17817},
{C_text("f_17845:irregex_2escm"),(void*)f_17845},
{C_text("f_17870:irregex_2escm"),(void*)f_17870},
{C_text("f_17889:irregex_2escm"),(void*)f_17889},
{C_text("f_17910:irregex_2escm"),(void*)f_17910},
{C_text("f_17920:irregex_2escm"),(void*)f_17920},
{C_text("f_17928:irregex_2escm"),(void*)f_17928},
{C_text("f_18159:irregex_2escm"),(void*)f_18159},
{C_text("f_18182:irregex_2escm"),(void*)f_18182},
{C_text("f_18191:irregex_2escm"),(void*)f_18191},
{C_text("f_18214:irregex_2escm"),(void*)f_18214},
{C_text("f_18237:irregex_2escm"),(void*)f_18237},
{C_text("f_18310:irregex_2escm"),(void*)f_18310},
{C_text("f_18335:irregex_2escm"),(void*)f_18335},
{C_text("f_18370:irregex_2escm"),(void*)f_18370},
{C_text("f_18393:irregex_2escm"),(void*)f_18393},
{C_text("f_18397:irregex_2escm"),(void*)f_18397},
{C_text("f_18418:irregex_2escm"),(void*)f_18418},
{C_text("f_18520:irregex_2escm"),(void*)f_18520},
{C_text("f_18704:irregex_2escm"),(void*)f_18704},
{C_text("f_18726:irregex_2escm"),(void*)f_18726},
{C_text("f_18732:irregex_2escm"),(void*)f_18732},
{C_text("f_18736:irregex_2escm"),(void*)f_18736},
{C_text("f_18753:irregex_2escm"),(void*)f_18753},
{C_text("f_18786:irregex_2escm"),(void*)f_18786},
{C_text("f_18823:irregex_2escm"),(void*)f_18823},
{C_text("f_18825:irregex_2escm"),(void*)f_18825},
{C_text("f_18850:irregex_2escm"),(void*)f_18850},
{C_text("f_18918:irregex_2escm"),(void*)f_18918},
{C_text("f_18920:irregex_2escm"),(void*)f_18920},
{C_text("f_18924:irregex_2escm"),(void*)f_18924},
{C_text("f_18933:irregex_2escm"),(void*)f_18933},
{C_text("f_18939:irregex_2escm"),(void*)f_18939},
{C_text("f_18942:irregex_2escm"),(void*)f_18942},
{C_text("f_19010:irregex_2escm"),(void*)f_19010},
{C_text("f_19014:irregex_2escm"),(void*)f_19014},
{C_text("f_19017:irregex_2escm"),(void*)f_19017},
{C_text("f_19020:irregex_2escm"),(void*)f_19020},
{C_text("f_19023:irregex_2escm"),(void*)f_19023},
{C_text("f_19053:irregex_2escm"),(void*)f_19053},
{C_text("f_19072:irregex_2escm"),(void*)f_19072},
{C_text("f_19097:irregex_2escm"),(void*)f_19097},
{C_text("f_19114:irregex_2escm"),(void*)f_19114},
{C_text("f_19118:irregex_2escm"),(void*)f_19118},
{C_text("f_19123:irregex_2escm"),(void*)f_19123},
{C_text("f_19130:irregex_2escm"),(void*)f_19130},
{C_text("f_19153:irregex_2escm"),(void*)f_19153},
{C_text("f_19163:irregex_2escm"),(void*)f_19163},
{C_text("f_19184:irregex_2escm"),(void*)f_19184},
{C_text("f_19202:irregex_2escm"),(void*)f_19202},
{C_text("f_19208:irregex_2escm"),(void*)f_19208},
{C_text("f_19222:irregex_2escm"),(void*)f_19222},
{C_text("f_19228:irregex_2escm"),(void*)f_19228},
{C_text("f_19235:irregex_2escm"),(void*)f_19235},
{C_text("f_19265:irregex_2escm"),(void*)f_19265},
{C_text("f_19278:irregex_2escm"),(void*)f_19278},
{C_text("f_19285:irregex_2escm"),(void*)f_19285},
{C_text("f_19292:irregex_2escm"),(void*)f_19292},
{C_text("f_19294:irregex_2escm"),(void*)f_19294},
{C_text("f_19302:irregex_2escm"),(void*)f_19302},
{C_text("f_19306:irregex_2escm"),(void*)f_19306},
{C_text("f_19315:irregex_2escm"),(void*)f_19315},
{C_text("f_19321:irregex_2escm"),(void*)f_19321},
{C_text("f_19324:irregex_2escm"),(void*)f_19324},
{C_text("f_19392:irregex_2escm"),(void*)f_19392},
{C_text("f_19396:irregex_2escm"),(void*)f_19396},
{C_text("f_19399:irregex_2escm"),(void*)f_19399},
{C_text("f_19414:irregex_2escm"),(void*)f_19414},
{C_text("f_19437:irregex_2escm"),(void*)f_19437},
{C_text("f_19448:irregex_2escm"),(void*)f_19448},
{C_text("f_19484:irregex_2escm"),(void*)f_19484},
{C_text("f_19494:irregex_2escm"),(void*)f_19494},
{C_text("f_19500:irregex_2escm"),(void*)f_19500},
{C_text("f_19506:irregex_2escm"),(void*)f_19506},
{C_text("f_19516:irregex_2escm"),(void*)f_19516},
{C_text("f_19518:irregex_2escm"),(void*)f_19518},
{C_text("f_19528:irregex_2escm"),(void*)f_19528},
{C_text("f_19547:irregex_2escm"),(void*)f_19547},
{C_text("f_19560:irregex_2escm"),(void*)f_19560},
{C_text("f_19592:irregex_2escm"),(void*)f_19592},
{C_text("f_19615:irregex_2escm"),(void*)f_19615},
{C_text("f_19620:irregex_2escm"),(void*)f_19620},
{C_text("f_19627:irregex_2escm"),(void*)f_19627},
{C_text("f_19678:irregex_2escm"),(void*)f_19678},
{C_text("f_19685:irregex_2escm"),(void*)f_19685},
{C_text("f_19688:irregex_2escm"),(void*)f_19688},
{C_text("f_19693:irregex_2escm"),(void*)f_19693},
{C_text("f_19745:irregex_2escm"),(void*)f_19745},
{C_text("f_19760:irregex_2escm"),(void*)f_19760},
{C_text("f_19796:irregex_2escm"),(void*)f_19796},
{C_text("f_19806:irregex_2escm"),(void*)f_19806},
{C_text("f_19816:irregex_2escm"),(void*)f_19816},
{C_text("f_19818:irregex_2escm"),(void*)f_19818},
{C_text("f_19876:irregex_2escm"),(void*)f_19876},
{C_text("f_19904:irregex_2escm"),(void*)f_19904},
{C_text("f_19908:irregex_2escm"),(void*)f_19908},
{C_text("f_19913:irregex_2escm"),(void*)f_19913},
{C_text("f_19933:irregex_2escm"),(void*)f_19933},
{C_text("f_19935:irregex_2escm"),(void*)f_19935},
{C_text("f_19960:irregex_2escm"),(void*)f_19960},
{C_text("f_19972:irregex_2escm"),(void*)f_19972},
{C_text("f_19975:irregex_2escm"),(void*)f_19975},
{C_text("f_19988:irregex_2escm"),(void*)f_19988},
{C_text("f_19992:irregex_2escm"),(void*)f_19992},
{C_text("f_19995:irregex_2escm"),(void*)f_19995},
{C_text("f_20000:irregex_2escm"),(void*)f_20000},
{C_text("f_20010:irregex_2escm"),(void*)f_20010},
{C_text("f_20013:irregex_2escm"),(void*)f_20013},
{C_text("f_20023:irregex_2escm"),(void*)f_20023},
{C_text("f_20058:irregex_2escm"),(void*)f_20058},
{C_text("f_20110:irregex_2escm"),(void*)f_20110},
{C_text("f_20122:irregex_2escm"),(void*)f_20122},
{C_text("f_20146:irregex_2escm"),(void*)f_20146},
{C_text("f_20147:irregex_2escm"),(void*)f_20147},
{C_text("f_20176:irregex_2escm"),(void*)f_20176},
{C_text("f_20185:irregex_2escm"),(void*)f_20185},
{C_text("f_20199:irregex_2escm"),(void*)f_20199},
{C_text("f_20218:irregex_2escm"),(void*)f_20218},
{C_text("f_20226:irregex_2escm"),(void*)f_20226},
{C_text("f_20236:irregex_2escm"),(void*)f_20236},
{C_text("f_20249:irregex_2escm"),(void*)f_20249},
{C_text("f_20274:irregex_2escm"),(void*)f_20274},
{C_text("f_20283:irregex_2escm"),(void*)f_20283},
{C_text("f_20319:irregex_2escm"),(void*)f_20319},
{C_text("f_20322:irregex_2escm"),(void*)f_20322},
{C_text("f_20361:irregex_2escm"),(void*)f_20361},
{C_text("f_20364:irregex_2escm"),(void*)f_20364},
{C_text("f_20389:irregex_2escm"),(void*)f_20389},
{C_text("f_20429:irregex_2escm"),(void*)f_20429},
{C_text("f_20457:irregex_2escm"),(void*)f_20457},
{C_text("f_20582:irregex_2escm"),(void*)f_20582},
{C_text("f_20616:irregex_2escm"),(void*)f_20616},
{C_text("f_20757:irregex_2escm"),(void*)f_20757},
{C_text("f_20761:irregex_2escm"),(void*)f_20761},
{C_text("f_20764:irregex_2escm"),(void*)f_20764},
{C_text("f_20766:irregex_2escm"),(void*)f_20766},
{C_text("f_20769:irregex_2escm"),(void*)f_20769},
{C_text("f_20779:irregex_2escm"),(void*)f_20779},
{C_text("f_20783:irregex_2escm"),(void*)f_20783},
{C_text("f_20792:irregex_2escm"),(void*)f_20792},
{C_text("f_20795:irregex_2escm"),(void*)f_20795},
{C_text("f_20804:irregex_2escm"),(void*)f_20804},
{C_text("f_20840:irregex_2escm"),(void*)f_20840},
{C_text("f_20851:irregex_2escm"),(void*)f_20851},
{C_text("f_20857:irregex_2escm"),(void*)f_20857},
{C_text("f_20874:irregex_2escm"),(void*)f_20874},
{C_text("f_20907:irregex_2escm"),(void*)f_20907},
{C_text("f_20913:irregex_2escm"),(void*)f_20913},
{C_text("f_20920:irregex_2escm"),(void*)f_20920},
{C_text("f_20931:irregex_2escm"),(void*)f_20931},
{C_text("f_20945:irregex_2escm"),(void*)f_20945},
{C_text("f_20963:irregex_2escm"),(void*)f_20963},
{C_text("f_20988:irregex_2escm"),(void*)f_20988},
{C_text("f_21015:irregex_2escm"),(void*)f_21015},
{C_text("f_21046:irregex_2escm"),(void*)f_21046},
{C_text("f_21053:irregex_2escm"),(void*)f_21053},
{C_text("f_21065:irregex_2escm"),(void*)f_21065},
{C_text("f_21095:irregex_2escm"),(void*)f_21095},
{C_text("f_21109:irregex_2escm"),(void*)f_21109},
{C_text("f_21112:irregex_2escm"),(void*)f_21112},
{C_text("f_21118:irregex_2escm"),(void*)f_21118},
{C_text("f_21132:irregex_2escm"),(void*)f_21132},
{C_text("f_21156:irregex_2escm"),(void*)f_21156},
{C_text("f_21159:irregex_2escm"),(void*)f_21159},
{C_text("f_21166:irregex_2escm"),(void*)f_21166},
{C_text("f_21183:irregex_2escm"),(void*)f_21183},
{C_text("f_21199:irregex_2escm"),(void*)f_21199},
{C_text("f_21202:irregex_2escm"),(void*)f_21202},
{C_text("f_21205:irregex_2escm"),(void*)f_21205},
{C_text("f_21211:irregex_2escm"),(void*)f_21211},
{C_text("f_21221:irregex_2escm"),(void*)f_21221},
{C_text("f_21225:irregex_2escm"),(void*)f_21225},
{C_text("f_21236:irregex_2escm"),(void*)f_21236},
{C_text("f_21240:irregex_2escm"),(void*)f_21240},
{C_text("f_21244:irregex_2escm"),(void*)f_21244},
{C_text("f_21252:irregex_2escm"),(void*)f_21252},
{C_text("f_21260:irregex_2escm"),(void*)f_21260},
{C_text("f_21275:irregex_2escm"),(void*)f_21275},
{C_text("f_21281:irregex_2escm"),(void*)f_21281},
{C_text("f_21295:irregex_2escm"),(void*)f_21295},
{C_text("f_21309:irregex_2escm"),(void*)f_21309},
{C_text("f_21315:irregex_2escm"),(void*)f_21315},
{C_text("f_21318:irregex_2escm"),(void*)f_21318},
{C_text("f_21345:irregex_2escm"),(void*)f_21345},
{C_text("f_21349:irregex_2escm"),(void*)f_21349},
{C_text("f_21369:irregex_2escm"),(void*)f_21369},
{C_text("f_21375:irregex_2escm"),(void*)f_21375},
{C_text("f_21378:irregex_2escm"),(void*)f_21378},
{C_text("f_21381:irregex_2escm"),(void*)f_21381},
{C_text("f_21397:irregex_2escm"),(void*)f_21397},
{C_text("f_21405:irregex_2escm"),(void*)f_21405},
{C_text("f_21417:irregex_2escm"),(void*)f_21417},
{C_text("f_21447:irregex_2escm"),(void*)f_21447},
{C_text("f_21453:irregex_2escm"),(void*)f_21453},
{C_text("f_21456:irregex_2escm"),(void*)f_21456},
{C_text("f_21459:irregex_2escm"),(void*)f_21459},
{C_text("f_21475:irregex_2escm"),(void*)f_21475},
{C_text("f_21483:irregex_2escm"),(void*)f_21483},
{C_text("f_21491:irregex_2escm"),(void*)f_21491},
{C_text("f_21495:irregex_2escm"),(void*)f_21495},
{C_text("f_21514:irregex_2escm"),(void*)f_21514},
{C_text("f_21532:irregex_2escm"),(void*)f_21532},
{C_text("f_21660:irregex_2escm"),(void*)f_21660},
{C_text("f_21666:irregex_2escm"),(void*)f_21666},
{C_text("f_21669:irregex_2escm"),(void*)f_21669},
{C_text("f_21677:irregex_2escm"),(void*)f_21677},
{C_text("f_21716:irregex_2escm"),(void*)f_21716},
{C_text("f_21751:irregex_2escm"),(void*)f_21751},
{C_text("f_21758:irregex_2escm"),(void*)f_21758},
{C_text("f_21784:irregex_2escm"),(void*)f_21784},
{C_text("f_21880:irregex_2escm"),(void*)f_21880},
{C_text("f_21884:irregex_2escm"),(void*)f_21884},
{C_text("f_21897:irregex_2escm"),(void*)f_21897},
{C_text("f_21909:irregex_2escm"),(void*)f_21909},
{C_text("f_21913:irregex_2escm"),(void*)f_21913},
{C_text("f_21920:irregex_2escm"),(void*)f_21920},
{C_text("f_21926:irregex_2escm"),(void*)f_21926},
{C_text("f_21930:irregex_2escm"),(void*)f_21930},
{C_text("f_21933:irregex_2escm"),(void*)f_21933},
{C_text("f_21935:irregex_2escm"),(void*)f_21935},
{C_text("f_21942:irregex_2escm"),(void*)f_21942},
{C_text("f_21949:irregex_2escm"),(void*)f_21949},
{C_text("f_21984:irregex_2escm"),(void*)f_21984},
{C_text("f_21997:irregex_2escm"),(void*)f_21997},
{C_text("f_22098:irregex_2escm"),(void*)f_22098},
{C_text("f_22104:irregex_2escm"),(void*)f_22104},
{C_text("f_22110:irregex_2escm"),(void*)f_22110},
{C_text("f_22163:irregex_2escm"),(void*)f_22163},
{C_text("f_22234:irregex_2escm"),(void*)f_22234},
{C_text("f_22287:irregex_2escm"),(void*)f_22287},
{C_text("f_22296:irregex_2escm"),(void*)f_22296},
{C_text("f_22316:irregex_2escm"),(void*)f_22316},
{C_text("f_22345:irregex_2escm"),(void*)f_22345},
{C_text("f_22369:irregex_2escm"),(void*)f_22369},
{C_text("f_22376:irregex_2escm"),(void*)f_22376},
{C_text("f_22379:irregex_2escm"),(void*)f_22379},
{C_text("f_22382:irregex_2escm"),(void*)f_22382},
{C_text("f_22385:irregex_2escm"),(void*)f_22385},
{C_text("f_22401:irregex_2escm"),(void*)f_22401},
{C_text("f_22418:irregex_2escm"),(void*)f_22418},
{C_text("f_22453:irregex_2escm"),(void*)f_22453},
{C_text("f_22457:irregex_2escm"),(void*)f_22457},
{C_text("f_22485:irregex_2escm"),(void*)f_22485},
{C_text("f_22488:irregex_2escm"),(void*)f_22488},
{C_text("f_22494:irregex_2escm"),(void*)f_22494},
{C_text("f_22580:irregex_2escm"),(void*)f_22580},
{C_text("f_22583:irregex_2escm"),(void*)f_22583},
{C_text("f_22588:irregex_2escm"),(void*)f_22588},
{C_text("f_22598:irregex_2escm"),(void*)f_22598},
{C_text("f_22601:irregex_2escm"),(void*)f_22601},
{C_text("f_22621:irregex_2escm"),(void*)f_22621},
{C_text("f_22629:irregex_2escm"),(void*)f_22629},
{C_text("f_22662:irregex_2escm"),(void*)f_22662},
{C_text("f_22664:irregex_2escm"),(void*)f_22664},
{C_text("f_22689:irregex_2escm"),(void*)f_22689},
{C_text("f_22702:irregex_2escm"),(void*)f_22702},
{C_text("f_22741:irregex_2escm"),(void*)f_22741},
{C_text("f_22756:irregex_2escm"),(void*)f_22756},
{C_text("f_22777:irregex_2escm"),(void*)f_22777},
{C_text("f_22795:irregex_2escm"),(void*)f_22795},
{C_text("f_22804:irregex_2escm"),(void*)f_22804},
{C_text("f_22818:irregex_2escm"),(void*)f_22818},
{C_text("f_22822:irregex_2escm"),(void*)f_22822},
{C_text("f_22825:irregex_2escm"),(void*)f_22825},
{C_text("f_22831:irregex_2escm"),(void*)f_22831},
{C_text("f_22834:irregex_2escm"),(void*)f_22834},
{C_text("f_22837:irregex_2escm"),(void*)f_22837},
{C_text("f_22854:irregex_2escm"),(void*)f_22854},
{C_text("f_22926:irregex_2escm"),(void*)f_22926},
{C_text("f_22930:irregex_2escm"),(void*)f_22930},
{C_text("f_22934:irregex_2escm"),(void*)f_22934},
{C_text("f_22936:irregex_2escm"),(void*)f_22936},
{C_text("f_22959:irregex_2escm"),(void*)f_22959},
{C_text("f_22988:irregex_2escm"),(void*)f_22988},
{C_text("f_22992:irregex_2escm"),(void*)f_22992},
{C_text("f_23016:irregex_2escm"),(void*)f_23016},
{C_text("f_23044:irregex_2escm"),(void*)f_23044},
{C_text("f_23054:irregex_2escm"),(void*)f_23054},
{C_text("f_23064:irregex_2escm"),(void*)f_23064},
{C_text("f_23069:irregex_2escm"),(void*)f_23069},
{C_text("f_23084:irregex_2escm"),(void*)f_23084},
{C_text("f_23104:irregex_2escm"),(void*)f_23104},
{C_text("f_23127:irregex_2escm"),(void*)f_23127},
{C_text("f_23159:irregex_2escm"),(void*)f_23159},
{C_text("f_23166:irregex_2escm"),(void*)f_23166},
{C_text("f_23168:irregex_2escm"),(void*)f_23168},
{C_text("f_23192:irregex_2escm"),(void*)f_23192},
{C_text("f_23236:irregex_2escm"),(void*)f_23236},
{C_text("f_23249:irregex_2escm"),(void*)f_23249},
{C_text("f_23263:irregex_2escm"),(void*)f_23263},
{C_text("f_23290:irregex_2escm"),(void*)f_23290},
{C_text("f_23371:irregex_2escm"),(void*)f_23371},
{C_text("f_23387:irregex_2escm"),(void*)f_23387},
{C_text("f_23397:irregex_2escm"),(void*)f_23397},
{C_text("f_23406:irregex_2escm"),(void*)f_23406},
{C_text("f_23446:irregex_2escm"),(void*)f_23446},
{C_text("f_23449:irregex_2escm"),(void*)f_23449},
{C_text("f_23472:irregex_2escm"),(void*)f_23472},
{C_text("f_23488:irregex_2escm"),(void*)f_23488},
{C_text("f_23495:irregex_2escm"),(void*)f_23495},
{C_text("f_23515:irregex_2escm"),(void*)f_23515},
{C_text("f_23535:irregex_2escm"),(void*)f_23535},
{C_text("f_23538:irregex_2escm"),(void*)f_23538},
{C_text("f_23539:irregex_2escm"),(void*)f_23539},
{C_text("f_23545:irregex_2escm"),(void*)f_23545},
{C_text("f_23561:irregex_2escm"),(void*)f_23561},
{C_text("f_23689:irregex_2escm"),(void*)f_23689},
{C_text("f_23708:irregex_2escm"),(void*)f_23708},
{C_text("f_23731:irregex_2escm"),(void*)f_23731},
{C_text("f_23732:irregex_2escm"),(void*)f_23732},
{C_text("f_23738:irregex_2escm"),(void*)f_23738},
{C_text("f_23757:irregex_2escm"),(void*)f_23757},
{C_text("f_23758:irregex_2escm"),(void*)f_23758},
{C_text("f_23764:irregex_2escm"),(void*)f_23764},
{C_text("f_23786:irregex_2escm"),(void*)f_23786},
{C_text("f_23792:irregex_2escm"),(void*)f_23792},
{C_text("f_23793:irregex_2escm"),(void*)f_23793},
{C_text("f_23799:irregex_2escm"),(void*)f_23799},
{C_text("f_23831:irregex_2escm"),(void*)f_23831},
{C_text("f_23837:irregex_2escm"),(void*)f_23837},
{C_text("f_23839:irregex_2escm"),(void*)f_23839},
{C_text("f_23845:irregex_2escm"),(void*)f_23845},
{C_text("f_23855:irregex_2escm"),(void*)f_23855},
{C_text("f_23861:irregex_2escm"),(void*)f_23861},
{C_text("f_23885:irregex_2escm"),(void*)f_23885},
{C_text("f_23891:irregex_2escm"),(void*)f_23891},
{C_text("f_23898:irregex_2escm"),(void*)f_23898},
{C_text("f_23904:irregex_2escm"),(void*)f_23904},
{C_text("f_23987:irregex_2escm"),(void*)f_23987},
{C_text("f_23990:irregex_2escm"),(void*)f_23990},
{C_text("f_23991:irregex_2escm"),(void*)f_23991},
{C_text("f_24004:irregex_2escm"),(void*)f_24004},
{C_text("f_24010:irregex_2escm"),(void*)f_24010},
{C_text("f_24023:irregex_2escm"),(void*)f_24023},
{C_text("f_24028:irregex_2escm"),(void*)f_24028},
{C_text("f_24059:irregex_2escm"),(void*)f_24059},
{C_text("f_24063:irregex_2escm"),(void*)f_24063},
{C_text("f_24068:irregex_2escm"),(void*)f_24068},
{C_text("f_24139:irregex_2escm"),(void*)f_24139},
{C_text("f_24142:irregex_2escm"),(void*)f_24142},
{C_text("f_24143:irregex_2escm"),(void*)f_24143},
{C_text("f_24156:irregex_2escm"),(void*)f_24156},
{C_text("f_24162:irregex_2escm"),(void*)f_24162},
{C_text("f_24172:irregex_2escm"),(void*)f_24172},
{C_text("f_24184:irregex_2escm"),(void*)f_24184},
{C_text("f_24197:irregex_2escm"),(void*)f_24197},
{C_text("f_24218:irregex_2escm"),(void*)f_24218},
{C_text("f_24228:irregex_2escm"),(void*)f_24228},
{C_text("f_24232:irregex_2escm"),(void*)f_24232},
{C_text("f_24238:irregex_2escm"),(void*)f_24238},
{C_text("f_24310:irregex_2escm"),(void*)f_24310},
{C_text("f_24360:irregex_2escm"),(void*)f_24360},
{C_text("f_24373:irregex_2escm"),(void*)f_24373},
{C_text("f_24374:irregex_2escm"),(void*)f_24374},
{C_text("f_24381:irregex_2escm"),(void*)f_24381},
{C_text("f_24389:irregex_2escm"),(void*)f_24389},
{C_text("f_24396:irregex_2escm"),(void*)f_24396},
{C_text("f_24408:irregex_2escm"),(void*)f_24408},
{C_text("f_24409:irregex_2escm"),(void*)f_24409},
{C_text("f_24416:irregex_2escm"),(void*)f_24416},
{C_text("f_24424:irregex_2escm"),(void*)f_24424},
{C_text("f_24431:irregex_2escm"),(void*)f_24431},
{C_text("f_24446:irregex_2escm"),(void*)f_24446},
{C_text("f_24447:irregex_2escm"),(void*)f_24447},
{C_text("f_24451:irregex_2escm"),(void*)f_24451},
{C_text("f_24457:irregex_2escm"),(void*)f_24457},
{C_text("f_24464:irregex_2escm"),(void*)f_24464},
{C_text("f_24473:irregex_2escm"),(void*)f_24473},
{C_text("f_24483:irregex_2escm"),(void*)f_24483},
{C_text("f_24487:irregex_2escm"),(void*)f_24487},
{C_text("f_24492:irregex_2escm"),(void*)f_24492},
{C_text("f_24509:irregex_2escm"),(void*)f_24509},
{C_text("f_24518:irregex_2escm"),(void*)f_24518},
{C_text("f_24529:irregex_2escm"),(void*)f_24529},
{C_text("f_24530:irregex_2escm"),(void*)f_24530},
{C_text("f_24534:irregex_2escm"),(void*)f_24534},
{C_text("f_24545:irregex_2escm"),(void*)f_24545},
{C_text("f_24552:irregex_2escm"),(void*)f_24552},
{C_text("f_24564:irregex_2escm"),(void*)f_24564},
{C_text("f_24567:irregex_2escm"),(void*)f_24567},
{C_text("f_24570:irregex_2escm"),(void*)f_24570},
{C_text("f_24579:irregex_2escm"),(void*)f_24579},
{C_text("f_24582:irregex_2escm"),(void*)f_24582},
{C_text("f_24583:irregex_2escm"),(void*)f_24583},
{C_text("f_24640:irregex_2escm"),(void*)f_24640},
{C_text("f_24641:irregex_2escm"),(void*)f_24641},
{C_text("f_24647:irregex_2escm"),(void*)f_24647},
{C_text("f_24683:irregex_2escm"),(void*)f_24683},
{C_text("f_24688:irregex_2escm"),(void*)f_24688},
{C_text("f_24720:irregex_2escm"),(void*)f_24720},
{C_text("f_24723:irregex_2escm"),(void*)f_24723},
{C_text("f_24724:irregex_2escm"),(void*)f_24724},
{C_text("f_24728:irregex_2escm"),(void*)f_24728},
{C_text("f_24746:irregex_2escm"),(void*)f_24746},
{C_text("f_24759:irregex_2escm"),(void*)f_24759},
{C_text("f_24773:irregex_2escm"),(void*)f_24773},
{C_text("f_24777:irregex_2escm"),(void*)f_24777},
{C_text("f_24791:irregex_2escm"),(void*)f_24791},
{C_text("f_24797:irregex_2escm"),(void*)f_24797},
{C_text("f_24810:irregex_2escm"),(void*)f_24810},
{C_text("f_24817:irregex_2escm"),(void*)f_24817},
{C_text("f_24824:irregex_2escm"),(void*)f_24824},
{C_text("f_24846:irregex_2escm"),(void*)f_24846},
{C_text("f_24850:irregex_2escm"),(void*)f_24850},
{C_text("f_24937:irregex_2escm"),(void*)f_24937},
{C_text("f_24938:irregex_2escm"),(void*)f_24938},
{C_text("f_24986:irregex_2escm"),(void*)f_24986},
{C_text("f_25046:irregex_2escm"),(void*)f_25046},
{C_text("f_25081:irregex_2escm"),(void*)f_25081},
{C_text("f_25281:irregex_2escm"),(void*)f_25281},
{C_text("f_25301:irregex_2escm"),(void*)f_25301},
{C_text("f_25310:irregex_2escm"),(void*)f_25310},
{C_text("f_25316:irregex_2escm"),(void*)f_25316},
{C_text("f_25322:irregex_2escm"),(void*)f_25322},
{C_text("f_25339:irregex_2escm"),(void*)f_25339},
{C_text("f_25376:irregex_2escm"),(void*)f_25376},
{C_text("f_25385:irregex_2escm"),(void*)f_25385},
{C_text("f_25391:irregex_2escm"),(void*)f_25391},
{C_text("f_25397:irregex_2escm"),(void*)f_25397},
{C_text("f_25431:irregex_2escm"),(void*)f_25431},
{C_text("f_25461:irregex_2escm"),(void*)f_25461},
{C_text("f_25468:irregex_2escm"),(void*)f_25468},
{C_text("f_25505:irregex_2escm"),(void*)f_25505},
{C_text("f_25512:irregex_2escm"),(void*)f_25512},
{C_text("f_25519:irregex_2escm"),(void*)f_25519},
{C_text("f_25528:irregex_2escm"),(void*)f_25528},
{C_text("f_25547:irregex_2escm"),(void*)f_25547},
{C_text("f_25564:irregex_2escm"),(void*)f_25564},
{C_text("f_25571:irregex_2escm"),(void*)f_25571},
{C_text("f_25592:irregex_2escm"),(void*)f_25592},
{C_text("f_25612:irregex_2escm"),(void*)f_25612},
{C_text("f_25619:irregex_2escm"),(void*)f_25619},
{C_text("f_25626:irregex_2escm"),(void*)f_25626},
{C_text("f_25645:irregex_2escm"),(void*)f_25645},
{C_text("f_25652:irregex_2escm"),(void*)f_25652},
{C_text("f_25659:irregex_2escm"),(void*)f_25659},
{C_text("f_25682:irregex_2escm"),(void*)f_25682},
{C_text("f_25703:irregex_2escm"),(void*)f_25703},
{C_text("f_25710:irregex_2escm"),(void*)f_25710},
{C_text("f_25717:irregex_2escm"),(void*)f_25717},
{C_text("f_25724:irregex_2escm"),(void*)f_25724},
{C_text("f_25733:irregex_2escm"),(void*)f_25733},
{C_text("f_25753:irregex_2escm"),(void*)f_25753},
{C_text("f_25769:irregex_2escm"),(void*)f_25769},
{C_text("f_25786:irregex_2escm"),(void*)f_25786},
{C_text("f_25806:irregex_2escm"),(void*)f_25806},
{C_text("f_25810:irregex_2escm"),(void*)f_25810},
{C_text("f_25813:irregex_2escm"),(void*)f_25813},
{C_text("f_25819:irregex_2escm"),(void*)f_25819},
{C_text("f_25864:irregex_2escm"),(void*)f_25864},
{C_text("f_25928:irregex_2escm"),(void*)f_25928},
{C_text("f_25945:irregex_2escm"),(void*)f_25945},
{C_text("f_25947:irregex_2escm"),(void*)f_25947},
{C_text("f_25957:irregex_2escm"),(void*)f_25957},
{C_text("f_25963:irregex_2escm"),(void*)f_25963},
{C_text("f_25969:irregex_2escm"),(void*)f_25969},
{C_text("f_25985:irregex_2escm"),(void*)f_25985},
{C_text("f_25991:irregex_2escm"),(void*)f_25991},
{C_text("f_26013:irregex_2escm"),(void*)f_26013},
{C_text("f_26027:irregex_2escm"),(void*)f_26027},
{C_text("f_26044:irregex_2escm"),(void*)f_26044},
{C_text("f_26046:irregex_2escm"),(void*)f_26046},
{C_text("f_26056:irregex_2escm"),(void*)f_26056},
{C_text("f_26062:irregex_2escm"),(void*)f_26062},
{C_text("f_26068:irregex_2escm"),(void*)f_26068},
{C_text("f_26084:irregex_2escm"),(void*)f_26084},
{C_text("f_26142:irregex_2escm"),(void*)f_26142},
{C_text("f_26168:irregex_2escm"),(void*)f_26168},
{C_text("f_26170:irregex_2escm"),(void*)f_26170},
{C_text("f_26183:irregex_2escm"),(void*)f_26183},
{C_text("f_26202:irregex_2escm"),(void*)f_26202},
{C_text("f_26211:irregex_2escm"),(void*)f_26211},
{C_text("f_26217:irregex_2escm"),(void*)f_26217},
{C_text("f_26223:irregex_2escm"),(void*)f_26223},
{C_text("f_26229:irregex_2escm"),(void*)f_26229},
{C_text("f_26381:irregex_2escm"),(void*)f_26381},
{C_text("f_26393:irregex_2escm"),(void*)f_26393},
{C_text("f_26443:irregex_2escm"),(void*)f_26443},
{C_text("f_26461:irregex_2escm"),(void*)f_26461},
{C_text("f_26472:irregex_2escm"),(void*)f_26472},
{C_text("f_26478:irregex_2escm"),(void*)f_26478},
{C_text("f_26490:irregex_2escm"),(void*)f_26490},
{C_text("f_26492:irregex_2escm"),(void*)f_26492},
{C_text("f_26502:irregex_2escm"),(void*)f_26502},
{C_text("f_26505:irregex_2escm"),(void*)f_26505},
{C_text("f_26531:irregex_2escm"),(void*)f_26531},
{C_text("f_26552:irregex_2escm"),(void*)f_26552},
{C_text("f_26556:irregex_2escm"),(void*)f_26556},
{C_text("f_26569:irregex_2escm"),(void*)f_26569},
{C_text("f_26571:irregex_2escm"),(void*)f_26571},
{C_text("f_26596:irregex_2escm"),(void*)f_26596},
{C_text("f_26620:irregex_2escm"),(void*)f_26620},
{C_text("f_26633:irregex_2escm"),(void*)f_26633},
{C_text("f_26635:irregex_2escm"),(void*)f_26635},
{C_text("f_26660:irregex_2escm"),(void*)f_26660},
{C_text("f_26682:irregex_2escm"),(void*)f_26682},
{C_text("f_26690:irregex_2escm"),(void*)f_26690},
{C_text("f_26703:irregex_2escm"),(void*)f_26703},
{C_text("f_26705:irregex_2escm"),(void*)f_26705},
{C_text("f_26730:irregex_2escm"),(void*)f_26730},
{C_text("f_26750:irregex_2escm"),(void*)f_26750},
{C_text("f_26760:irregex_2escm"),(void*)f_26760},
{C_text("f_26775:irregex_2escm"),(void*)f_26775},
{C_text("f_26788:irregex_2escm"),(void*)f_26788},
{C_text("f_26790:irregex_2escm"),(void*)f_26790},
{C_text("f_26815:irregex_2escm"),(void*)f_26815},
{C_text("f_26937:irregex_2escm"),(void*)f_26937},
{C_text("f_26939:irregex_2escm"),(void*)f_26939},
{C_text("f_26957:irregex_2escm"),(void*)f_26957},
{C_text("f_26959:irregex_2escm"),(void*)f_26959},
{C_text("f_26992:irregex_2escm"),(void*)f_26992},
{C_text("f_27044:irregex_2escm"),(void*)f_27044},
{C_text("f_27080:irregex_2escm"),(void*)f_27080},
{C_text("f_27088:irregex_2escm"),(void*)f_27088},
{C_text("f_27092:irregex_2escm"),(void*)f_27092},
{C_text("f_27094:irregex_2escm"),(void*)f_27094},
{C_text("f_27108:irregex_2escm"),(void*)f_27108},
{C_text("f_27112:irregex_2escm"),(void*)f_27112},
{C_text("f_27116:irregex_2escm"),(void*)f_27116},
{C_text("f_27129:irregex_2escm"),(void*)f_27129},
{C_text("f_27133:irregex_2escm"),(void*)f_27133},
{C_text("f_27137:irregex_2escm"),(void*)f_27137},
{C_text("f_27275:irregex_2escm"),(void*)f_27275},
{C_text("f_27283:irregex_2escm"),(void*)f_27283},
{C_text("f_27287:irregex_2escm"),(void*)f_27287},
{C_text("f_27289:irregex_2escm"),(void*)f_27289},
{C_text("f_27303:irregex_2escm"),(void*)f_27303},
{C_text("f_27316:irregex_2escm"),(void*)f_27316},
{C_text("f_27320:irregex_2escm"),(void*)f_27320},
{C_text("f_27366:irregex_2escm"),(void*)f_27366},
{C_text("f_27369:irregex_2escm"),(void*)f_27369},
{C_text("f_27376:irregex_2escm"),(void*)f_27376},
{C_text("f_27471:irregex_2escm"),(void*)f_27471},
{C_text("f_27479:irregex_2escm"),(void*)f_27479},
{C_text("f_27483:irregex_2escm"),(void*)f_27483},
{C_text("f_27485:irregex_2escm"),(void*)f_27485},
{C_text("f_27502:irregex_2escm"),(void*)f_27502},
{C_text("f_27597:irregex_2escm"),(void*)f_27597},
{C_text("f_27605:irregex_2escm"),(void*)f_27605},
{C_text("f_27607:irregex_2escm"),(void*)f_27607},
{C_text("f_27615:irregex_2escm"),(void*)f_27615},
{C_text("f_27617:irregex_2escm"),(void*)f_27617},
{C_text("f_27631:irregex_2escm"),(void*)f_27631},
{C_text("f_27637:irregex_2escm"),(void*)f_27637},
{C_text("f_27646:irregex_2escm"),(void*)f_27646},
{C_text("f_27650:irregex_2escm"),(void*)f_27650},
{C_text("f_27654:irregex_2escm"),(void*)f_27654},
{C_text("f_27658:irregex_2escm"),(void*)f_27658},
{C_text("f_27682:irregex_2escm"),(void*)f_27682},
{C_text("f_27692:irregex_2escm"),(void*)f_27692},
{C_text("f_27701:irregex_2escm"),(void*)f_27701},
{C_text("f_27705:irregex_2escm"),(void*)f_27705},
{C_text("f_27709:irregex_2escm"),(void*)f_27709},
{C_text("f_27713:irregex_2escm"),(void*)f_27713},
{C_text("f_27723:irregex_2escm"),(void*)f_27723},
{C_text("f_27738:irregex_2escm"),(void*)f_27738},
{C_text("f_27742:irregex_2escm"),(void*)f_27742},
{C_text("f_27745:irregex_2escm"),(void*)f_27745},
{C_text("f_27748:irregex_2escm"),(void*)f_27748},
{C_text("f_27757:irregex_2escm"),(void*)f_27757},
{C_text("f_27760:irregex_2escm"),(void*)f_27760},
{C_text("f_27769:irregex_2escm"),(void*)f_27769},
{C_text("f_27772:irregex_2escm"),(void*)f_27772},
{C_text("f_27780:irregex_2escm"),(void*)f_27780},
{C_text("f_27793:irregex_2escm"),(void*)f_27793},
{C_text("f_27828:irregex_2escm"),(void*)f_27828},
{C_text("f_27831:irregex_2escm"),(void*)f_27831},
{C_text("f_27939:irregex_2escm"),(void*)f_27939},
{C_text("f_27956:irregex_2escm"),(void*)f_27956},
{C_text("f_27960:irregex_2escm"),(void*)f_27960},
{C_text("f_27961:irregex_2escm"),(void*)f_27961},
{C_text("f_27969:irregex_2escm"),(void*)f_27969},
{C_text("f_27984:irregex_2escm"),(void*)f_27984},
{C_text("f_27988:irregex_2escm"),(void*)f_27988},
{C_text("f_27991:irregex_2escm"),(void*)f_27991},
{C_text("f_28000:irregex_2escm"),(void*)f_28000},
{C_text("f_28006:irregex_2escm"),(void*)f_28006},
{C_text("f_28014:irregex_2escm"),(void*)f_28014},
{C_text("f_28027:irregex_2escm"),(void*)f_28027},
{C_text("f_28077:irregex_2escm"),(void*)f_28077},
{C_text("f_28087:irregex_2escm"),(void*)f_28087},
{C_text("f_28101:irregex_2escm"),(void*)f_28101},
{C_text("f_28104:irregex_2escm"),(void*)f_28104},
{C_text("f_28107:irregex_2escm"),(void*)f_28107},
{C_text("f_28159:irregex_2escm"),(void*)f_28159},
{C_text("f_28166:irregex_2escm"),(void*)f_28166},
{C_text("f_28170:irregex_2escm"),(void*)f_28170},
{C_text("f_28171:irregex_2escm"),(void*)f_28171},
{C_text("f_28179:irregex_2escm"),(void*)f_28179},
{C_text("f_28194:irregex_2escm"),(void*)f_28194},
{C_text("f_28198:irregex_2escm"),(void*)f_28198},
{C_text("f_28201:irregex_2escm"),(void*)f_28201},
{C_text("f_28215:irregex_2escm"),(void*)f_28215},
{C_text("f_28219:irregex_2escm"),(void*)f_28219},
{C_text("f_28223:irregex_2escm"),(void*)f_28223},
{C_text("f_28231:irregex_2escm"),(void*)f_28231},
{C_text("f_28273:irregex_2escm"),(void*)f_28273},
{C_text("f_28277:irregex_2escm"),(void*)f_28277},
{C_text("f_28282:irregex_2escm"),(void*)f_28282},
{C_text("f_28316:irregex_2escm"),(void*)f_28316},
{C_text("f_28323:irregex_2escm"),(void*)f_28323},
{C_text("f_28331:irregex_2escm"),(void*)f_28331},
{C_text("f_28333:irregex_2escm"),(void*)f_28333},
{C_text("f_28354:irregex_2escm"),(void*)f_28354},
{C_text("f_28366:irregex_2escm"),(void*)f_28366},
{C_text("f_28372:irregex_2escm"),(void*)f_28372},
{C_text("f_28399:irregex_2escm"),(void*)f_28399},
{C_text("f_28424:irregex_2escm"),(void*)f_28424},
{C_text("f_28450:irregex_2escm"),(void*)f_28450},
{C_text("f_28514:irregex_2escm"),(void*)f_28514},
{C_text("f_28546:irregex_2escm"),(void*)f_28546},
{C_text("f_28576:irregex_2escm"),(void*)f_28576},
{C_text("f_28597:irregex_2escm"),(void*)f_28597},
{C_text("f_28601:irregex_2escm"),(void*)f_28601},
{C_text("f_28606:irregex_2escm"),(void*)f_28606},
{C_text("f_28614:irregex_2escm"),(void*)f_28614},
{C_text("f_28616:irregex_2escm"),(void*)f_28616},
{C_text("f_28632:irregex_2escm"),(void*)f_28632},
{C_text("f_28636:irregex_2escm"),(void*)f_28636},
{C_text("f_28642:irregex_2escm"),(void*)f_28642},
{C_text("f_28647:irregex_2escm"),(void*)f_28647},
{C_text("f_28661:irregex_2escm"),(void*)f_28661},
{C_text("f_28705:irregex_2escm"),(void*)f_28705},
{C_text("f_28713:irregex_2escm"),(void*)f_28713},
{C_text("f_28715:irregex_2escm"),(void*)f_28715},
{C_text("f_28756:irregex_2escm"),(void*)f_28756},
{C_text("f_28796:irregex_2escm"),(void*)f_28796},
{C_text("f_28830:irregex_2escm"),(void*)f_28830},
{C_text("f_28843:irregex_2escm"),(void*)f_28843},
{C_text("f_28873:irregex_2escm"),(void*)f_28873},
{C_text("f_28881:irregex_2escm"),(void*)f_28881},
{C_text("f_28885:irregex_2escm"),(void*)f_28885},
{C_text("f_28887:irregex_2escm"),(void*)f_28887},
{C_text("f_28906:irregex_2escm"),(void*)f_28906},
{C_text("f_28930:irregex_2escm"),(void*)f_28930},
{C_text("f_28947:irregex_2escm"),(void*)f_28947},
{C_text("f_28954:irregex_2escm"),(void*)f_28954},
{C_text("f_28977:irregex_2escm"),(void*)f_28977},
{C_text("f_28998:irregex_2escm"),(void*)f_28998},
{C_text("f_29000:irregex_2escm"),(void*)f_29000},
{C_text("f_29010:irregex_2escm"),(void*)f_29010},
{C_text("f_29037:irregex_2escm"),(void*)f_29037},
{C_text("f_29067:irregex_2escm"),(void*)f_29067},
{C_text("f_29072:irregex_2escm"),(void*)f_29072},
{C_text("f_29084:irregex_2escm"),(void*)f_29084},
{C_text("f_29104:irregex_2escm"),(void*)f_29104},
{C_text("f_29139:irregex_2escm"),(void*)f_29139},
{C_text("f_29147:irregex_2escm"),(void*)f_29147},
{C_text("f_29183:irregex_2escm"),(void*)f_29183},
{C_text("f_29197:irregex_2escm"),(void*)f_29197},
{C_text("f_29201:irregex_2escm"),(void*)f_29201},
{C_text("f_29204:irregex_2escm"),(void*)f_29204},
{C_text("f_29209:irregex_2escm"),(void*)f_29209},
{C_text("f_29219:irregex_2escm"),(void*)f_29219},
{C_text("f_29230:irregex_2escm"),(void*)f_29230},
{C_text("f_29233:irregex_2escm"),(void*)f_29233},
{C_text("f_29240:irregex_2escm"),(void*)f_29240},
{C_text("f_29252:irregex_2escm"),(void*)f_29252},
{C_text("f_29267:irregex_2escm"),(void*)f_29267},
{C_text("f_29277:irregex_2escm"),(void*)f_29277},
{C_text("f_29281:irregex_2escm"),(void*)f_29281},
{C_text("f_29284:irregex_2escm"),(void*)f_29284},
{C_text("f_29289:irregex_2escm"),(void*)f_29289},
{C_text("f_29312:irregex_2escm"),(void*)f_29312},
{C_text("f_29315:irregex_2escm"),(void*)f_29315},
{C_text("f_29322:irregex_2escm"),(void*)f_29322},
{C_text("f_29327:irregex_2escm"),(void*)f_29327},
{C_text("f_29337:irregex_2escm"),(void*)f_29337},
{C_text("f_29357:irregex_2escm"),(void*)f_29357},
{C_text("f_29367:irregex_2escm"),(void*)f_29367},
{C_text("f_29409:irregex_2escm"),(void*)f_29409},
{C_text("f_29416:irregex_2escm"),(void*)f_29416},
{C_text("f_29421:irregex_2escm"),(void*)f_29421},
{C_text("f_29431:irregex_2escm"),(void*)f_29431},
{C_text("f_29451:irregex_2escm"),(void*)f_29451},
{C_text("f_29454:irregex_2escm"),(void*)f_29454},
{C_text("f_29457:irregex_2escm"),(void*)f_29457},
{C_text("f_29466:irregex_2escm"),(void*)f_29466},
{C_text("f_29471:irregex_2escm"),(void*)f_29471},
{C_text("f_29481:irregex_2escm"),(void*)f_29481},
{C_text("f_29505:irregex_2escm"),(void*)f_29505},
{C_text("f_29508:irregex_2escm"),(void*)f_29508},
{C_text("f_29509:irregex_2escm"),(void*)f_29509},
{C_text("f_29513:irregex_2escm"),(void*)f_29513},
{C_text("f_29525:irregex_2escm"),(void*)f_29525},
{C_text("f_29530:irregex_2escm"),(void*)f_29530},
{C_text("f_29540:irregex_2escm"),(void*)f_29540},
{C_text("f_29564:irregex_2escm"),(void*)f_29564},
{C_text("f_29567:irregex_2escm"),(void*)f_29567},
{C_text("f_29578:irregex_2escm"),(void*)f_29578},
{C_text("f_29585:irregex_2escm"),(void*)f_29585},
{C_text("f_29590:irregex_2escm"),(void*)f_29590},
{C_text("f_29600:irregex_2escm"),(void*)f_29600},
{C_text("f_29634:irregex_2escm"),(void*)f_29634},
{C_text("f_29637:irregex_2escm"),(void*)f_29637},
{C_text("f_29640:irregex_2escm"),(void*)f_29640},
{C_text("f_29647:irregex_2escm"),(void*)f_29647},
{C_text("f_29651:irregex_2escm"),(void*)f_29651},
{C_text("f_29664:irregex_2escm"),(void*)f_29664},
{C_text("f_29677:irregex_2escm"),(void*)f_29677},
{C_text("f_29680:irregex_2escm"),(void*)f_29680},
{C_text("f_29687:irregex_2escm"),(void*)f_29687},
{C_text("f_29698:irregex_2escm"),(void*)f_29698},
{C_text("f_29707:irregex_2escm"),(void*)f_29707},
{C_text("f_29710:irregex_2escm"),(void*)f_29710},
{C_text("f_29713:irregex_2escm"),(void*)f_29713},
{C_text("f_29720:irregex_2escm"),(void*)f_29720},
{C_text("f_29745:irregex_2escm"),(void*)f_29745},
{C_text("f_29783:irregex_2escm"),(void*)f_29783},
{C_text("f_29786:irregex_2escm"),(void*)f_29786},
{C_text("f_29789:irregex_2escm"),(void*)f_29789},
{C_text("f_29796:irregex_2escm"),(void*)f_29796},
{C_text("f_29801:irregex_2escm"),(void*)f_29801},
{C_text("f_29811:irregex_2escm"),(void*)f_29811},
{C_text("f_29847:irregex_2escm"),(void*)f_29847},
{C_text("f_30011:irregex_2escm"),(void*)f_30011},
{C_text("f_30024:irregex_2escm"),(void*)f_30024},
{C_text("f_30033:irregex_2escm"),(void*)f_30033},
{C_text("f_30044:irregex_2escm"),(void*)f_30044},
{C_text("f_30048:irregex_2escm"),(void*)f_30048},
{C_text("f_30050:irregex_2escm"),(void*)f_30050},
{C_text("f_30081:irregex_2escm"),(void*)f_30081},
{C_text("f_30128:irregex_2escm"),(void*)f_30128},
{C_text("f_30148:irregex_2escm"),(void*)f_30148},
{C_text("f_30158:irregex_2escm"),(void*)f_30158},
{C_text("f_30185:irregex_2escm"),(void*)f_30185},
{C_text("f_30207:irregex_2escm"),(void*)f_30207},
{C_text("f_30309:irregex_2escm"),(void*)f_30309},
{C_text("f_30774:irregex_2escm"),(void*)f_30774},
{C_text("f_30936:irregex_2escm"),(void*)f_30936},
{C_text("f_30939:irregex_2escm"),(void*)f_30939},
{C_text("f_30973:irregex_2escm"),(void*)f_30973},
{C_text("f_30977:irregex_2escm"),(void*)f_30977},
{C_text("f_30980:irregex_2escm"),(void*)f_30980},
{C_text("f_30983:irregex_2escm"),(void*)f_30983},
{C_text("f_30986:irregex_2escm"),(void*)f_30986},
{C_text("f_30989:irregex_2escm"),(void*)f_30989},
{C_text("f_7535:irregex_2escm"),(void*)f_7535},
{C_text("f_7538:irregex_2escm"),(void*)f_7538},
{C_text("f_7540:irregex_2escm"),(void*)f_7540},
{C_text("f_7544:irregex_2escm"),(void*)f_7544},
{C_text("f_7547:irregex_2escm"),(void*)f_7547},
{C_text("f_7553:irregex_2escm"),(void*)f_7553},
{C_text("f_7595:irregex_2escm"),(void*)f_7595},
{C_text("f_7613:irregex_2escm"),(void*)f_7613},
{C_text("f_7622:irregex_2escm"),(void*)f_7622},
{C_text("f_7630:irregex_2escm"),(void*)f_7630},
{C_text("f_7643:irregex_2escm"),(void*)f_7643},
{C_text("f_7645:irregex_2escm"),(void*)f_7645},
{C_text("f_7652:irregex_2escm"),(void*)f_7652},
{C_text("f_7654:irregex_2escm"),(void*)f_7654},
{C_text("f_7668:irregex_2escm"),(void*)f_7668},
{C_text("f_7686:irregex_2escm"),(void*)f_7686},
{C_text("f_7692:irregex_2escm"),(void*)f_7692},
{C_text("f_7717:irregex_2escm"),(void*)f_7717},
{C_text("f_7763:irregex_2escm"),(void*)f_7763},
{C_text("f_7776:irregex_2escm"),(void*)f_7776},
{C_text("f_7778:irregex_2escm"),(void*)f_7778},
{C_text("f_7782:irregex_2escm"),(void*)f_7782},
{C_text("f_7803:irregex_2escm"),(void*)f_7803},
{C_text("f_7807:irregex_2escm"),(void*)f_7807},
{C_text("f_7832:irregex_2escm"),(void*)f_7832},
{C_text("f_7836:irregex_2escm"),(void*)f_7836},
{C_text("f_7861:irregex_2escm"),(void*)f_7861},
{C_text("f_7865:irregex_2escm"),(void*)f_7865},
{C_text("f_7954:irregex_2escm"),(void*)f_7954},
{C_text("f_8022:irregex_2escm"),(void*)f_8022},
{C_text("f_8064:irregex_2escm"),(void*)f_8064},
{C_text("f_8066:irregex_2escm"),(void*)f_8066},
{C_text("f_8136:irregex_2escm"),(void*)f_8136},
{C_text("f_8138:irregex_2escm"),(void*)f_8138},
{C_text("f_8142:irregex_2escm"),(void*)f_8142},
{C_text("f_8199:irregex_2escm"),(void*)f_8199},
{C_text("f_8201:irregex_2escm"),(void*)f_8201},
{C_text("f_8205:irregex_2escm"),(void*)f_8205},
{C_text("f_8282:irregex_2escm"),(void*)f_8282},
{C_text("f_8286:irregex_2escm"),(void*)f_8286},
{C_text("f_8372:irregex_2escm"),(void*)f_8372},
{C_text("f_8406:irregex_2escm"),(void*)f_8406},
{C_text("f_8412:irregex_2escm"),(void*)f_8412},
{C_text("f_8446:irregex_2escm"),(void*)f_8446},
{C_text("f_8460:irregex_2escm"),(void*)f_8460},
{C_text("f_8467:irregex_2escm"),(void*)f_8467},
{C_text("f_8473:irregex_2escm"),(void*)f_8473},
{C_text("f_8491:irregex_2escm"),(void*)f_8491},
{C_text("f_8495:irregex_2escm"),(void*)f_8495},
{C_text("f_8499:irregex_2escm"),(void*)f_8499},
{C_text("f_8506:irregex_2escm"),(void*)f_8506},
{C_text("f_8514:irregex_2escm"),(void*)f_8514},
{C_text("f_8518:irregex_2escm"),(void*)f_8518},
{C_text("f_8522:irregex_2escm"),(void*)f_8522},
{C_text("f_8526:irregex_2escm"),(void*)f_8526},
{C_text("f_8530:irregex_2escm"),(void*)f_8530},
{C_text("f_8534:irregex_2escm"),(void*)f_8534},
{C_text("f_8538:irregex_2escm"),(void*)f_8538},
{C_text("f_8547:irregex_2escm"),(void*)f_8547},
{C_text("f_8555:irregex_2escm"),(void*)f_8555},
{C_text("f_8564:irregex_2escm"),(void*)f_8564},
{C_text("f_8622:irregex_2escm"),(void*)f_8622},
{C_text("f_8626:irregex_2escm"),(void*)f_8626},
{C_text("f_8640:irregex_2escm"),(void*)f_8640},
{C_text("f_8644:irregex_2escm"),(void*)f_8644},
{C_text("f_8657:irregex_2escm"),(void*)f_8657},
{C_text("f_8668:irregex_2escm"),(void*)f_8668},
{C_text("f_8670:irregex_2escm"),(void*)f_8670},
{C_text("f_8677:irregex_2escm"),(void*)f_8677},
{C_text("f_8690:irregex_2escm"),(void*)f_8690},
{C_text("f_8697:irregex_2escm"),(void*)f_8697},
{C_text("f_8736:irregex_2escm"),(void*)f_8736},
{C_text("f_8763:irregex_2escm"),(void*)f_8763},
{C_text("f_8795:irregex_2escm"),(void*)f_8795},
{C_text("f_8802:irregex_2escm"),(void*)f_8802},
{C_text("f_8817:irregex_2escm"),(void*)f_8817},
{C_text("f_8830:irregex_2escm"),(void*)f_8830},
{C_text("f_8873:irregex_2escm"),(void*)f_8873},
{C_text("f_8930:irregex_2escm"),(void*)f_8930},
{C_text("f_8968:irregex_2escm"),(void*)f_8968},
{C_text("f_8971:irregex_2escm"),(void*)f_8971},
{C_text("f_8979:irregex_2escm"),(void*)f_8979},
{C_text("f_8992:irregex_2escm"),(void*)f_8992},
{C_text("f_9011:irregex_2escm"),(void*)f_9011},
{C_text("f_9108:irregex_2escm"),(void*)f_9108},
{C_text("f_9116:irregex_2escm"),(void*)f_9116},
{C_text("f_9118:irregex_2escm"),(void*)f_9118},
{C_text("f_9132:irregex_2escm"),(void*)f_9132},
{C_text("f_9137:irregex_2escm"),(void*)f_9137},
{C_text("f_9158:irregex_2escm"),(void*)f_9158},
{C_text("f_9174:irregex_2escm"),(void*)f_9174},
{C_text("f_9195:irregex_2escm"),(void*)f_9195},
{C_text("f_9201:irregex_2escm"),(void*)f_9201},
{C_text("f_9214:irregex_2escm"),(void*)f_9214},
{C_text("f_9256:irregex_2escm"),(void*)f_9256},
{C_text("f_9271:irregex_2escm"),(void*)f_9271},
{C_text("f_9294:irregex_2escm"),(void*)f_9294},
{C_text("f_9310:irregex_2escm"),(void*)f_9310},
{C_text("f_9323:irregex_2escm"),(void*)f_9323},
{C_text("f_9337:irregex_2escm"),(void*)f_9337},
{C_text("f_9355:irregex_2escm"),(void*)f_9355},
{C_text("f_9371:irregex_2escm"),(void*)f_9371},
{C_text("f_9382:irregex_2escm"),(void*)f_9382},
{C_text("f_9388:irregex_2escm"),(void*)f_9388},
{C_text("f_9406:irregex_2escm"),(void*)f_9406},
{C_text("f_9712:irregex_2escm"),(void*)f_9712},
{C_text("f_9721:irregex_2escm"),(void*)f_9721},
{C_text("f_9739:irregex_2escm"),(void*)f_9739},
{C_text("f_9746:irregex_2escm"),(void*)f_9746},
{C_text("f_9773:irregex_2escm"),(void*)f_9773},
{C_text("f_9779:irregex_2escm"),(void*)f_9779},
{C_text("f_9793:irregex_2escm"),(void*)f_9793},
{C_text("f_9813:irregex_2escm"),(void*)f_9813},
{C_text("f_9819:irregex_2escm"),(void*)f_9819},
{C_text("f_9836:irregex_2escm"),(void*)f_9836},
{C_text("f_9840:irregex_2escm"),(void*)f_9840},
{C_text("f_9887:irregex_2escm"),(void*)f_9887},
{C_text("f_9936:irregex_2escm"),(void*)f_9936},
{C_text("f_9964:irregex_2escm"),(void*)f_9964},
{C_text("f_9975:irregex_2escm"),(void*)f_9975},
{C_text("toplevel:irregex_2escm"),(void*)C_irregex_toplevel},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.irregex#d 
o|hiding unexported module binding: chicken.irregex#define-alias 
o|hiding unexported module binding: chicken.irregex#build-cache 
o|hiding unexported module binding: chicken.irregex#vector-copy 
o|hiding unexported module binding: chicken.irregex#irregex-dfa 
o|hiding unexported module binding: chicken.irregex#irregex-dfa/search 
o|hiding unexported module binding: chicken.irregex#irregex-nfa 
o|hiding unexported module binding: chicken.irregex#irregex-flags 
o|hiding unexported module binding: chicken.irregex#irregex-lengths 
o|hiding unexported module binding: chicken.irregex#irregex-new-matches 
o|hiding unexported module binding: chicken.irregex#irregex-reset-matches! 
o|hiding unexported module binding: chicken.irregex#irregex-copy-matches 
o|hiding unexported module binding: chicken.irregex#irregex-match-chunker 
o|hiding unexported module binding: chicken.irregex#irregex-match-chunker-set! 
o|hiding unexported module binding: chicken.irregex#%irregex-match-end-index 
o|hiding unexported module binding: chicken.irregex#%irregex-match-fail 
o|hiding unexported module binding: chicken.irregex#%irregex-match-fail-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-valid-named-index? 
o|hiding unexported module binding: chicken.irregex#irregex-match-start-chunk-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-start-index-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-end-chunk-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-end-index-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-chunk&index-from-tag-set! 
o|hiding unexported module binding: chicken.irregex#irregex-match-numeric-index 
o|hiding unexported module binding: chicken.irregex#chunker-get-next 
o|hiding unexported module binding: chicken.irregex#chunker-get-str 
o|hiding unexported module binding: chicken.irregex#chunker-get-start 
o|hiding unexported module binding: chicken.irregex#chunker-get-end 
o|hiding unexported module binding: chicken.irregex#chunker-get-substring 
o|hiding unexported module binding: chicken.irregex#chunker-get-subchunk 
o|hiding unexported module binding: chicken.irregex#chunker-prev-chunk 
o|hiding unexported module binding: chicken.irregex#chunker-prev-char 
o|hiding unexported module binding: chicken.irregex#chunker-next-char 
o|hiding unexported module binding: chicken.irregex#chunk-before? 
o|hiding unexported module binding: chicken.irregex#wrap-end-chunker 
o|hiding unexported module binding: chicken.irregex#*all-chars* 
o|hiding unexported module binding: chicken.irregex#*allow-utf8-mode?* 
o|hiding unexported module binding: chicken.irregex#string-scan-char 
o|hiding unexported module binding: chicken.irregex#string-scan-char-escape 
o|hiding unexported module binding: chicken.irregex#string-scan-pred 
o|hiding unexported module binding: chicken.irregex#string-split-char 
o|hiding unexported module binding: chicken.irregex#char-alphanumeric? 
o|hiding unexported module binding: chicken.irregex#%substring=? 
o|hiding unexported module binding: chicken.irregex#%%string-copy! 
o|hiding unexported module binding: chicken.irregex#string-cat-reverse 
o|hiding unexported module binding: chicken.irregex#string-cat-reverse/aux 
o|hiding unexported module binding: chicken.irregex#zero-to 
o|hiding unexported module binding: chicken.irregex#find 
o|hiding unexported module binding: chicken.irregex#find-tail 
o|hiding unexported module binding: chicken.irregex#last 
o|hiding unexported module binding: chicken.irregex#any 
o|hiding unexported module binding: chicken.irregex#every 
o|hiding unexported module binding: chicken.irregex#fold 
o|hiding unexported module binding: chicken.irregex#filter 
o|hiding unexported module binding: chicken.irregex#remove 
o|hiding unexported module binding: chicken.irregex#bit-shr 
o|hiding unexported module binding: chicken.irregex#bit-shl 
o|hiding unexported module binding: chicken.irregex#bit-not 
o|hiding unexported module binding: chicken.irregex#bit-ior 
o|hiding unexported module binding: chicken.irregex#bit-and 
o|hiding unexported module binding: chicken.irregex#integer-log 
o|hiding unexported module binding: chicken.irregex#flag-set? 
o|hiding unexported module binding: chicken.irregex#flag-join 
o|hiding unexported module binding: chicken.irregex#flag-clear 
o|hiding unexported module binding: chicken.irregex#~none 
o|hiding unexported module binding: chicken.irregex#~searcher? 
o|hiding unexported module binding: chicken.irregex#~consumer? 
o|hiding unexported module binding: chicken.irregex#close-token 
o|hiding unexported module binding: chicken.irregex#dot-token 
o|hiding unexported module binding: chicken.irregex#with-read-from-string 
o|hiding unexported module binding: chicken.irregex#~save? 
o|hiding unexported module binding: chicken.irregex#~case-insensitive? 
o|hiding unexported module binding: chicken.irregex#~multi-line? 
o|hiding unexported module binding: chicken.irregex#~single-line? 
o|hiding unexported module binding: chicken.irregex#~ignore-space? 
o|hiding unexported module binding: chicken.irregex#~utf8? 
o|hiding unexported module binding: chicken.irregex#symbol-list->flags 
o|hiding unexported module binding: chicken.irregex#posix-escape-sequences 
o|hiding unexported module binding: chicken.irregex#char-altcase 
o|hiding unexported module binding: chicken.irregex#char-mirror 
o|hiding unexported module binding: chicken.irregex#string-parse-hex-escape 
o|hiding unexported module binding: chicken.irregex#string-parse-cset 
o|hiding unexported module binding: chicken.irregex#high-char? 
o|hiding unexported module binding: chicken.irregex#utf8-start-char->length 
o|hiding unexported module binding: chicken.irregex#utf8-string-ref 
o|hiding unexported module binding: chicken.irregex#utf8-backup-to-initial-char 
o|hiding unexported module binding: chicken.irregex#utf8-lowest-digit-of-length 
o|hiding unexported module binding: chicken.irregex#utf8-highest-digit-of-length 
o|hiding unexported module binding: chicken.irregex#char->utf8-list 
o|hiding unexported module binding: chicken.irregex#unicode-range->utf8-pattern 
o|hiding unexported module binding: chicken.irregex#unicode-range-helper 
o|hiding unexported module binding: chicken.irregex#unicode-range-up-from 
o|hiding unexported module binding: chicken.irregex#unicode-range-up-to 
o|hiding unexported module binding: chicken.irregex#unicode-range-climb-digits 
o|hiding unexported module binding: chicken.irregex#unicode-range-middle 
o|hiding unexported module binding: chicken.irregex#cset->utf8-pattern 
o|hiding unexported module binding: chicken.irregex#sre-adjust-utf8 
o|hiding unexported module binding: chicken.irregex#cached 
o|hiding unexported module binding: chicken.irregex#sre-empty? 
o|hiding unexported module binding: chicken.irregex#sre-any? 
o|hiding unexported module binding: chicken.irregex#sre-repeater? 
o|hiding unexported module binding: chicken.irregex#sre-bos? 
o|hiding unexported module binding: chicken.irregex#sre-searcher? 
o|hiding unexported module binding: chicken.irregex#sre-consumer? 
o|hiding unexported module binding: chicken.irregex#sre-has-submatches? 
o|hiding unexported module binding: chicken.irregex#sre-count-submatches 
o|hiding unexported module binding: chicken.irregex#sre-length-ranges 
o|hiding unexported module binding: chicken.irregex#sre-sequence 
o|hiding unexported module binding: chicken.irregex#sre-alternate 
o|hiding unexported module binding: chicken.irregex#sre-strip-submatches 
o|hiding unexported module binding: chicken.irregex#sre-flatten-ranges 
o|hiding unexported module binding: chicken.irregex#sre-names 
o|hiding unexported module binding: chicken.irregex#sre-sequence-names 
o|hiding unexported module binding: chicken.irregex#sre-remove-initial-bos 
o|hiding unexported module binding: chicken.irregex#irregex-basic-string-chunker 
o|hiding unexported module binding: chicken.irregex#irregex-search/matches 
o|hiding unexported module binding: chicken.irregex#irregex-search/backtrack 
o|hiding unexported module binding: chicken.irregex#dfa-init-state 
o|hiding unexported module binding: chicken.irregex#dfa-next-state 
o|hiding unexported module binding: chicken.irregex#dfa-cell-commands 
o|hiding unexported module binding: chicken.irregex#dfa-finalizer 
o|hiding unexported module binding: chicken.irregex#dfa-match/shortest 
o|hiding unexported module binding: chicken.irregex#finalize! 
o|hiding unexported module binding: chicken.irregex#make-initial-memory 
o|hiding unexported module binding: chicken.irregex#dfa-match/longest 
o|hiding unexported module binding: chicken.irregex#sre-named-definitions 
o|hiding unexported module binding: chicken.irregex#*nfa-presize* 
o|hiding unexported module binding: chicken.irregex#*nfa-num-fields* 
o|hiding unexported module binding: chicken.irregex#nfa-num-states 
o|hiding unexported module binding: chicken.irregex#nfa-start-state 
o|hiding unexported module binding: chicken.irregex#nfa-num-tags 
o|hiding unexported module binding: chicken.irregex#nfa-highest-map-index 
o|hiding unexported module binding: chicken.irregex#nfa-set-highest-map-index! 
o|hiding unexported module binding: chicken.irregex#nfa-get-state-trans 
o|hiding unexported module binding: chicken.irregex#nfa-set-state-trans! 
o|hiding unexported module binding: chicken.irregex#nfa-get-epsilons 
o|hiding unexported module binding: chicken.irregex#nfa-set-epsilons! 
o|hiding unexported module binding: chicken.irregex#nfa-add-epsilon! 
o|hiding unexported module binding: chicken.irregex#nfa-get-reorder-commands 
o|hiding unexported module binding: chicken.irregex#nfa-set-reorder-commands! 
o|hiding unexported module binding: chicken.irregex#nfa-get-closure 
o|hiding unexported module binding: chicken.irregex#nfa-add-closure! 
o|hiding unexported module binding: chicken.irregex#sre->nfa 
o|hiding unexported module binding: chicken.irregex#*mst-first-state-index* 
o|hiding unexported module binding: chicken.irregex#mst-mappings-summary 
o|hiding unexported module binding: chicken.irregex#mst-num-states 
o|hiding unexported module binding: chicken.irregex#mst-num-states-set! 
o|hiding unexported module binding: chicken.irregex#mst-hash 
o|hiding unexported module binding: chicken.irregex#mst-hash-set! 
o|hiding unexported module binding: chicken.irregex#mst-state-mappings 
o|hiding unexported module binding: chicken.irregex#mst-state-mappings-set! 
o|hiding unexported module binding: chicken.irregex#make-mst 
o|hiding unexported module binding: chicken.irregex#mst-copy 
o|hiding unexported module binding: chicken.irregex#nfa-state->mst 
o|hiding unexported module binding: chicken.irregex#mst-add! 
o|hiding unexported module binding: chicken.irregex#mst-add/fast! 
o|hiding unexported module binding: chicken.irregex#mst-add-tagged! 
o|hiding unexported module binding: chicken.irregex#mst-same-states? 
o|hiding unexported module binding: chicken.irregex#mst-fold 
o|hiding unexported module binding: chicken.irregex#next-index-for-tag! 
o|hiding unexported module binding: chicken.irregex#nfa->dfa 
o|hiding unexported module binding: chicken.irregex#dfa-renumber 
o|hiding unexported module binding: chicken.irregex#get-distinct-transitions 
o|hiding unexported module binding: chicken.irregex#nfa-epsilon-closure-internal 
o|hiding unexported module binding: chicken.irregex#nfa-epsilon-closure 
o|hiding unexported module binding: chicken.irregex#tag-set-commands-for-closure 
o|hiding unexported module binding: chicken.irregex#find-reorder-commands-internal 
o|hiding unexported module binding: chicken.irregex#find-reorder-commands 
o|hiding unexported module binding: chicken.irregex#sre->procedure 
o|hiding unexported module binding: chicken.irregex#sre-cset->procedure 
o|hiding unexported module binding: chicken.irregex#make-cset 
o|hiding unexported module binding: chicken.irregex#range->cset 
o|hiding unexported module binding: chicken.irregex#char->cset 
o|hiding unexported module binding: chicken.irregex#cset-empty? 
o|hiding unexported module binding: chicken.irregex#maybe-cset->char 
o|hiding unexported module binding: chicken.irregex#cset=? 
o|hiding unexported module binding: chicken.irregex#cset-size 
o|hiding unexported module binding: chicken.irregex#cset->plist 
o|hiding unexported module binding: chicken.irregex#plist->cset 
o|hiding unexported module binding: chicken.irregex#string->cset 
o|hiding unexported module binding: chicken.irregex#sre->cset 
o|hiding unexported module binding: chicken.irregex#cset->sre 
o|hiding unexported module binding: chicken.irregex#cset-contains? 
o|hiding unexported module binding: chicken.irregex#char-ranges-union 
o|hiding unexported module binding: chicken.irregex#cset-union 
o|hiding unexported module binding: chicken.irregex#cset-adjoin 
o|hiding unexported module binding: chicken.irregex#next-char 
o|hiding unexported module binding: chicken.irregex#prev-char 
o|hiding unexported module binding: chicken.irregex#cset-difference 
o|hiding unexported module binding: chicken.irregex#min-char 
o|hiding unexported module binding: chicken.irregex#max-char 
o|hiding unexported module binding: chicken.irregex#cset-intersection 
o|hiding unexported module binding: chicken.irregex#cset-complement 
o|hiding unexported module binding: chicken.irregex#cset-case-insensitive 
o|hiding unexported module binding: chicken.irregex#irregex-fold/fast 
o|hiding unexported module binding: chicken.irregex#irregex-fold/chunked/fast 
o|hiding unexported module binding: chicken.irregex#irregex-apply-match 
o|hiding unexported module binding: chicken.irregex#rx-special-chars 
o|hiding unexported module binding: chicken.irregex#string-scan-char 
o|hiding unexported module binding: chicken.irregex#cset->string 
S|applied compiler syntax:
S|  irregex-match-chunk&index-from-tag-set!		1
S|  scheme#for-each		12
S|  irregex-match-end-index-set!		7
S|  irregex-match-end-chunk-set!		7
S|  irregex-match-start-index-set!		7
S|  irregex-match-start-chunk-set!		7
S|  make-irregex		2
S|  scheme#map		24
S|  bit-not		1
S|  bit-shr		10
S|  bit-shl		11
S|  bit-and		18
S|  bit-ior		11
S|  %%string-copy!		1
S|  scheme#reverse		35
S|  make-irregex-match		1
o|eliminated procedure checks: 537 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#integer->char (quote 55295)) 
o|folded constant expression: (scheme#integer->char (quote 57344)) 
o|folded constant expression: (scheme#integer->char (quote 1114111)) 
o|folded constant expression: (scheme#string->symbol (quote ".")) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 3)) 
o|folded constant expression: (scheme#integer->char (quote 13)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#- (quote 10) (quote 1)) 
o|folded constant expression: (scheme#integer->char (quote 9)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#- (quote 10) (quote 3)) 
o|folded constant expression: (scheme#integer->char (quote 7)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 17)) 
o|folded constant expression: (scheme#integer->char (quote 27)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 2)) 
o|folded constant expression: (scheme#integer->char (quote 12)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#- (quote 32) (quote 23)) 
o|folded constant expression: (scheme#integer->char (quote 9)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#- (quote 32) (quote 32)) 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#- (quote 32) (quote 1)) 
o|folded constant expression: (scheme#integer->char (quote 31)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#- (quote 32) (quote 32)) 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 95)) 
o|folded constant expression: (scheme#integer->char (quote 127)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#- (quote 32) (quote 32)) 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#- (quote 10) (quote 1)) 
o|folded constant expression: (scheme#integer->char (quote 9)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 1)) 
o|folded constant expression: (scheme#integer->char (quote 11)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 95)) 
o|folded constant expression: (scheme#integer->char (quote 127)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 3)) 
o|folded constant expression: (scheme#integer->char (quote 13)) 
o|folded constant expression: (scheme#char->integer (quote #\newline)) 
o|folded constant expression: (scheme#+ (quote 10) (quote 3)) 
o|folded constant expression: (scheme#integer->char (quote 13)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 96)) 
o|folded constant expression: (scheme#integer->char (quote 128)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 161)) 
o|folded constant expression: (scheme#integer->char (quote 193)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 162)) 
o|folded constant expression: (scheme#integer->char (quote 194)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 191)) 
o|folded constant expression: (scheme#integer->char (quote 223)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 192)) 
o|folded constant expression: (scheme#integer->char (quote 224)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 207)) 
o|folded constant expression: (scheme#integer->char (quote 239)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 208)) 
o|folded constant expression: (scheme#integer->char (quote 240)) 
o|folded constant expression: (scheme#char->integer (quote #\space)) 
o|folded constant expression: (scheme#+ (quote 32) (quote 215)) 
o|folded constant expression: (scheme#integer->char (quote 247)) 
o|eliminated procedure checks: 1 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|folded constant expression: (scheme#+ (quote 3) (quote 0)) 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|specializations:
o|  1 (scheme#> fixnum fixnum)
o|  1 (scheme#> integer integer)
o|  1 (scheme#< integer integer)
o|  1 (scheme#char<? char char)
o|  2 (scheme#char<=? char char)
o|  2 (scheme#zero? integer)
o|  1 (scheme#eqv? * *)
o|  1 (scheme#set-car! pair *)
o|  1 (scheme#equal? (or eof null fixnum char boolean symbol keyword) *)
o|  1 (scheme#caar (pair pair *))
o|  1 (scheme#quotient fixnum fixnum)
o|  1 (chicken.irregex#irregex-nfa (struct regexp))
o|  1 (chicken.irregex#irregex-dfa/search (struct regexp))
o|  4 (chicken.irregex#irregex-dfa (struct regexp))
o|  4 (chicken.irregex#irregex-flags (struct regexp))
o|  2 (scheme#string-length string)
o|  3 (scheme#cdddr (pair * (pair * pair)))
o|  5 (scheme#caddr (pair * (pair * pair)))
o|  3 (scheme#vector-length vector)
o|  31 (scheme#cadr (pair * pair))
o|  2 (scheme#- integer integer)
o|  2 (scheme#= fixnum fixnum)
o|  2 (scheme#length list)
o|  1 (scheme#>= fixnum fixnum)
o|  1 (scheme#< fixnum fixnum)
o|  5 (scheme#<= fixnum fixnum)
o|  7 (scheme#>= integer integer)
o|  6 (scheme#cddr (pair * pair))
o|  17 (scheme#memq * list)
o|  4 (##sys#check-list (or pair list) *)
o|  3 (scheme#memv (or symbol keyword procedure eof null fixnum char boolean) list)
o|  3 (scheme#string-ref string fixnum)
o|  1 (scheme#number->string *)
o|  128 (scheme#eqv? (or eof null fixnum char boolean symbol keyword) *)
o|  6 (scheme#+ integer integer)
o|  7 (scheme#* fixnum fixnum)
o|  4 (scheme#odd? *)
o|  13 (scheme#zero? *)
o|  4 (scheme#char=? char char)
o|  264 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  215 (scheme#cdr pair)
o|  5 (scheme#cdar (pair pair *))
o|  12 (scheme#integer? *)
o|  159 (scheme#car pair)
o|  16 (scheme#- fixnum fixnum)
o|  1 (chicken.irregex#irregex-names (struct regexp))
o|  1 (scheme#* integer integer)
o|  215 (scheme#+ fixnum fixnum)
(o e)|safe calls: 3628 
(o e)|assignments to immediate values: 13 
o|safe globals: (chicken.irregex#%irregex-match-fail-set! chicken.irregex#%irregex-match-fail chicken.irregex#%irregex-match-end-index chicken.irregex#irregex-match-chunker-set! chicken.irregex#irregex-match-names chicken.irregex#irregex-match-chunker chicken.irregex#irregex-match-num-submatches chicken.irregex#irregex-match-data? chicken.irregex#irregex-copy-matches chicken.irregex#irregex-reset-matches! chicken.irregex#irregex-new-matches chicken.irregex#irregex-names chicken.irregex#irregex-lengths chicken.irregex#irregex-num-submatches chicken.irregex#irregex-flags chicken.irregex#irregex-nfa chicken.irregex#irregex-dfa/search chicken.irregex#irregex-dfa chicken.irregex#irregex? chicken.irregex#vector-copy) 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-dfa/search 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-flags 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-lengths 
o|substituted constant variable: a7639 
o|substituted constant variable: a7637 
o|inlining procedure: k7656 
o|inlining procedure: k7656 
o|substituted constant variable: a7701 
o|inlining procedure: k7783 
o|contracted procedure: "(irregex-core.scm:233) g794795" 
o|inlining procedure: k7783 
o|inlining procedure: k7808 
o|contracted procedure: "(irregex-core.scm:236) g803804" 
o|inlining procedure: k7808 
o|inlining procedure: k7837 
o|contracted procedure: "(irregex-core.scm:239) g812813" 
o|inlining procedure: k7837 
o|inlining procedure: k7866 
o|inlining procedure: k7866 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-start-chunk-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-start-index-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-end-chunk-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-end-index-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-chunk&index-from-tag-set! 
o|inlining procedure: k7956 
o|inlining procedure: k7956 
o|inlining procedure: k7971 
o|inlining procedure: k7971 
o|inlining procedure: k7987 
o|inlining procedure: k8023 
o|inlining procedure: k8023 
o|contracted procedure: "(irregex-core.scm:278) g860861" 
o|inlining procedure: k8031 
o|inlining procedure: k8031 
o|contracted procedure: "(irregex-core.scm:277) g853854" 
o|inlining procedure: k8001 
o|substituted constant variable: a8019 
o|inlining procedure: k8001 
o|inlining procedure: k7987 
o|inlining procedure: k8068 
o|inlining procedure: k8068 
o|inlining procedure: k8086 
o|inlining procedure: k8086 
o|contracted procedure: "(irregex-core.scm:288) g874875" 
o|inlining procedure: k8143 
o|contracted procedure: "(irregex-core.scm:299) g882883" 
o|inlining procedure: k8161 
o|substituted constant variable: a8179 
o|inlining procedure: k8161 
o|inlining procedure: k8143 
o|contracted procedure: "(irregex-core.scm:300) chicken.irregex#irregex-match-valid-named-index?" 
o|inlining procedure: k7767 
o|inlining procedure: k7767 
o|inlining procedure: k8209 
o|contracted procedure: "(irregex-core.scm:309) g905906" 
o|contracted procedure: "(irregex-core.scm:308) g901902" 
o|contracted procedure: "(irregex-core.scm:307) g897898" 
o|inlining procedure: "(irregex-core.scm:306) chicken.irregex#chunker-get-substring" 
o|inlining procedure: k8209 
o|inlining procedure: "(irregex-core.scm:304) chicken.irregex#irregex-match-chunker" 
o|inlining procedure: k8293 
o|inlining procedure: k8293 
o|contracted procedure: "(irregex-core.scm:321) g924925" 
o|contracted procedure: "(irregex-core.scm:320) g920921" 
o|contracted procedure: "(irregex-core.scm:319) g916917" 
o|inlining procedure: "(irregex-core.scm:315) chicken.irregex#chunker-get-subchunk" 
o|inlining procedure: "(irregex-core.scm:314) chicken.irregex#irregex-match-chunker" 
o|inlining procedure: k8417 
o|inlining procedure: k8429 
o|inlining procedure: k8429 
o|inlining procedure: k8417 
o|inlining procedure: k8448 
o|inlining procedure: k8448 
o|inlining procedure: k8475 
o|inlining procedure: k8475 
o|inlining procedure: k8645 
o|inlining procedure: "(irregex-core.scm:380) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:379) chicken.irregex#chunker-get-str" 
o|inlining procedure: k8645 
o|contracted procedure: "(irregex-core.scm:377) chicken.irregex#chunker-prev-chunk" 
o|inlining procedure: k8609 
o|inlining procedure: k8609 
o|inlining procedure: k8627 
o|inlining procedure: k8627 
o|inlining procedure: "(irregex-core.scm:369) chicken.irregex#chunker-get-next" 
o|inlining procedure: k8678 
o|inlining procedure: "(irregex-core.scm:386) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:385) chicken.irregex#chunker-get-str" 
o|inlining procedure: k8678 
o|inlining procedure: "(irregex-core.scm:383) chicken.irregex#chunker-get-next" 
o|inlining procedure: k8832 
o|inlining procedure: k8832 
o|substituted constant variable: a8849 
o|inlining procedure: k9029 
o|inlining procedure: k9029 
o|removed side-effect free assignment to unused variable: chicken.irregex#%substring=? 
o|removed side-effect free assignment to unused variable: chicken.irregex#%%string-copy! 
o|contracted procedure: "(irregex-core.scm:483) chicken.irregex#string-cat-reverse/aux" 
o|substituted constant variable: u11411146 
o|inlining procedure: k9139 
o|inlining procedure: k9139 
o|inlining procedure: k9160 
o|inlining procedure: k9160 
o|inlining procedure: k9176 
o|inlining procedure: k9176 
o|inlining procedure: k9203 
o|inlining procedure: k9203 
o|removed side-effect free assignment to unused variable: chicken.irregex#find-tail 
o|inlining procedure: k9258 
o|inlining procedure: k9258 
o|inlining procedure: k9273 
o|inlining procedure: k9273 
o|inlining procedure: k9296 
o|inlining procedure: k9312 
o|inlining procedure: k9312 
o|inlining procedure: k9296 
o|inlining procedure: k9342 
o|inlining procedure: k9342 
o|inlining procedure: k9357 
o|inlining procedure: k9357 
o|inlining procedure: k9390 
o|inlining procedure: k9390 
o|removed side-effect free assignment to unused variable: chicken.irregex#filter 
o|removed side-effect free assignment to unused variable: chicken.irregex#remove 
o|removed side-effect free assignment to unused variable: chicken.irregex#bit-shr 
o|removed side-effect free assignment to unused variable: chicken.irregex#bit-shl 
o|removed side-effect free assignment to unused variable: chicken.irregex#bit-not 
o|removed side-effect free assignment to unused variable: chicken.irregex#bit-ior 
o|removed side-effect free assignment to unused variable: chicken.irregex#bit-and 
o|removed side-effect free assignment to unused variable: chicken.irregex#integer-log 
o|inlining procedure: k9714 
o|inlining procedure: k9714 
o|inlining procedure: k9748 
o|inlining procedure: k9748 
o|substituted constant variable: a9771 
o|inlining procedure: k9781 
o|inlining procedure: k9781 
o|inlining procedure: k9800 
o|inlining procedure: k9800 
o|inlining procedure: k9821 
o|inlining procedure: k9821 
o|substituted constant variable: chicken.irregex#dot-token 
o|inlining procedure: k9862 
o|substituted constant variable: a9873 
o|inlining procedure: k9862 
o|substituted constant variable: a9885 
o|inlining procedure: k9889 
o|inlining procedure: k9889 
o|substituted constant variable: a9915 
o|inlining procedure: k9920 
o|substituted constant variable: a9934 
o|substituted constant variable: a9951 
o|inlining procedure: k9920 
o|substituted constant variable: a9985 
o|substituted constant variable: a9987 
o|substituted constant variable: a9994 
o|substituted constant variable: a10001 
o|substituted constant variable: a10004 
o|inlining procedure: k10005 
o|substituted constant variable: a10016 
o|substituted constant variable: a10019 
o|inlining procedure: k10026 
o|inlining procedure: k10026 
o|inlining procedure: k10039 
o|inlining procedure: k10039 
o|substituted constant variable: a10066 
o|inlining procedure: k10071 
o|substituted constant variable: a10082 
o|substituted constant variable: a10085 
o|inlining procedure: k10071 
o|substituted constant variable: a10095 
o|substituted constant variable: a10097 
o|substituted constant variable: a10099 
o|inlining procedure: k10005 
o|inlining procedure: k10109 
o|substituted constant variable: a10120 
o|inlining procedure: k10109 
o|substituted constant variable: a10138 
o|inlining procedure: k10146 
o|inlining procedure: k10146 
o|substituted constant variable: a10162 
o|inlining procedure: k10158 
o|inlining procedure: k10158 
o|substituted constant variable: a10174 
o|substituted constant variable: a10176 
o|inlining procedure: k10177 
o|substituted constant variable: a10195 
o|substituted constant variable: a10196 
o|substituted constant variable: a10203 
o|inlining procedure: k10177 
o|substituted constant variable: a10211 
o|substituted constant variable: a10213 
o|substituted constant variable: a10215 
o|substituted constant variable: a10217 
o|substituted constant variable: a10220 
o|inlining procedure: k10221 
o|substituted constant variable: a10232 
o|inlining procedure: k10221 
o|substituted constant variable: a10238 
o|inlining procedure: k10242 
o|inlining procedure: k10261 
o|inlining procedure: k10261 
o|inlining procedure: k10242 
o|inlining procedure: k10280 
o|inlining procedure: k10280 
o|substituted constant variable: a10284 
o|substituted constant variable: a10289 
o|substituted constant variable: a10291 
o|substituted constant variable: a10293 
o|substituted constant variable: a10298 
o|substituted constant variable: a10300 
o|substituted constant variable: a10302 
o|substituted constant variable: a10304 
o|substituted constant variable: a10306 
o|inlining procedure: k10313 
o|inlining procedure: k10313 
o|inlining procedure: k10340 
o|inlining procedure: k10340 
o|inlining procedure: k10359 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|propagated global variable: r1036031189 chicken.irregex#~case-insensitive? 
o|inlining procedure: k10359 
o|substituted constant variable: chicken.irregex#~multi-line? 
o|inlining procedure: k10377 
o|substituted constant variable: chicken.irregex#~single-line? 
o|propagated global variable: r1037831193 chicken.irregex#~single-line? 
o|inlining procedure: k10377 
o|substituted constant variable: chicken.irregex#~ignore-space? 
o|inlining procedure: k10395 
o|substituted constant variable: chicken.irregex#*allow-utf8-mode?* 
o|substituted constant variable: chicken.irregex#~utf8? 
o|substituted constant variable: chicken.irregex#~none 
o|inlining procedure: k10359 
o|inlining procedure: k10395 
o|substituted constant variable: a10411 
o|substituted constant variable: a10413 
o|substituted constant variable: a10418 
o|substituted constant variable: a10420 
o|substituted constant variable: a10425 
o|substituted constant variable: a10427 
o|substituted constant variable: a10432 
o|substituted constant variable: a10434 
o|inlining procedure: k10438 
o|inlining procedure: k10438 
o|substituted constant variable: a10445 
o|substituted constant variable: a10447 
o|substituted constant variable: a10449 
o|substituted constant variable: chicken.irregex#~none 
o|inlining procedure: k10457 
o|inlining procedure: k10457 
o|inlining procedure: k10487 
o|inlining procedure: k10487 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k10509 
o|inlining procedure: k10531 
o|inlining procedure: k10531 
o|inlining procedure: k10509 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k10565 
o|inlining procedure: k10565 
o|inlining procedure: k10591 
o|inlining procedure: k10591 
o|inlining procedure: k10625 
o|inlining procedure: k10625 
o|contracted procedure: "(irregex-core.scm:772) chicken.irregex#utf8-backup-to-initial-char" 
o|inlining procedure: k14226 
o|inlining procedure: k14226 
o|substituted constant variable: a14249 
o|substituted constant variable: a14254 
o|substituted constant variable: a14256 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k10685 
o|inlining procedure: k10700 
o|substituted constant variable: a10719 
o|inlining procedure: k10712 
o|inlining procedure: k10712 
o|inlining procedure: k10700 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k10685 
o|substituted constant variable: a10827 
o|inlining procedure: k10820 
o|inlining procedure: k10820 
o|inlining procedure: k10864 
o|inlining procedure: k10864 
o|substituted constant variable: a10878 
o|substituted constant variable: a10886 
o|inlining procedure: k10901 
o|inlining procedure: k10901 
o|substituted constant variable: a10933 
o|substituted constant variable: a10936 
o|substituted constant variable: chicken.irregex#~single-line? 
o|inlining procedure: k10952 
o|substituted constant variable: a10978 
o|substituted constant variable: a10981 
o|inlining procedure: k10987 
o|inlining procedure: k11007 
o|inlining procedure: k11007 
o|inlining procedure: k11037 
o|inlining procedure: k11037 
o|inlining procedure: k11067 
o|inlining procedure: k11067 
o|substituted constant variable: a11095 
o|substituted constant variable: a11097 
o|substituted constant variable: a11099 
o|substituted constant variable: a11101 
o|substituted constant variable: a11103 
o|substituted constant variable: a11105 
o|inlining procedure: k10987 
o|inlining procedure: k10952 
o|inlining procedure: k11127 
o|inlining procedure: k11127 
o|substituted constant variable: a11150 
o|substituted constant variable: a11153 
o|inlining procedure: k11174 
o|substituted constant variable: a11185 
o|inlining procedure: k11190 
o|substituted constant variable: a11201 
o|substituted constant variable: a11204 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11190 
o|substituted constant variable: a11218 
o|inlining procedure: k11223 
o|substituted constant variable: a11240 
o|inlining procedure: k11244 
o|inlining procedure: k11244 
o|substituted constant variable: a11282 
o|substituted constant variable: a11290 
o|substituted constant variable: a11297 
o|inlining procedure: k11223 
o|substituted constant variable: a11327 
o|inlining procedure: k11328 
o|substituted constant variable: a11339 
o|substituted constant variable: a11342 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11328 
o|substituted constant variable: a11362 
o|substituted constant variable: a11365 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11374 
o|substituted constant variable: a11385 
o|substituted constant variable: a11388 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11374 
o|substituted constant variable: a11408 
o|inlining procedure: k11403 
o|inlining procedure: k11403 
o|substituted constant variable: a11427 
o|substituted constant variable: a11430 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11439 
o|substituted constant variable: a11450 
o|substituted constant variable: a11453 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11439 
o|substituted constant variable: a11501 
o|substituted constant variable: chicken.irregex#~save? 
o|substituted constant variable: a11513 
o|substituted constant variable: a11520 
o|substituted constant variable: a11522 
o|substituted constant variable: a11524 
o|substituted constant variable: a11527 
o|inlining procedure: k11528 
o|substituted constant variable: a11539 
o|substituted constant variable: a11542 
o|substituted constant variable: chicken.irregex#~save? 
o|inlining procedure: k11528 
o|substituted constant variable: a11562 
o|inlining procedure: k11557 
o|inlining procedure: k11557 
o|inlining procedure: k11579 
o|inlining procedure: k11579 
o|substituted constant variable: a11593 
o|substituted constant variable: a11596 
o|substituted constant variable: chicken.irregex#~save? 
o|substituted constant variable: a11615 
o|substituted constant variable: a11618 
o|inlining procedure: k11619 
o|substituted constant variable: a11636 
o|substituted constant variable: a11639 
o|substituted constant variable: chicken.irregex#~save? 
o|substituted constant variable: a11658 
o|substituted constant variable: a11661 
o|inlining procedure: k11619 
o|substituted constant variable: a11667 
o|substituted constant variable: a11670 
o|substituted constant variable: chicken.irregex#~save? 
o|substituted constant variable: a11685 
o|substituted constant variable: a11692 
o|inlining procedure: k11693 
o|inlining procedure: k11693 
o|substituted constant variable: a11707 
o|inlining procedure: k11714 
o|propagated global variable: r1171531276 chicken.irregex#flag-clear 
o|inlining procedure: k11714 
o|propagated global variable: r1171531281 chicken.irregex#flag-join 
o|inlining procedure: k11729 
o|inlining procedure: k11729 
o|inlining procedure: k11739 
o|inlining procedure: k11739 
o|substituted constant variable: chicken.irregex#~utf8? 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k11742 
o|inlining procedure: k11742 
o|substituted constant variable: a11765 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k11770 
o|substituted constant variable: a11781 
o|substituted constant variable: chicken.irregex#~multi-line? 
o|inlining procedure: k11770 
o|substituted constant variable: a11797 
o|substituted constant variable: chicken.irregex#~ignore-space? 
o|inlining procedure: k11802 
o|substituted constant variable: chicken.irregex#*allow-utf8-mode?* 
o|substituted constant variable: a11816 
o|substituted constant variable: chicken.irregex#~utf8? 
o|substituted constant variable: a11826 
o|inlining procedure: k11802 
o|substituted constant variable: a11838 
o|inlining procedure: k11843 
o|substituted constant variable: a11854 
o|substituted constant variable: a11857 
o|inlining procedure: k11843 
o|substituted constant variable: a11877 
o|substituted constant variable: a11880 
o|substituted constant variable: a11901 
o|substituted constant variable: a11903 
o|substituted constant variable: a11905 
o|substituted constant variable: a11907 
o|substituted constant variable: a11909 
o|substituted constant variable: a11911 
o|substituted constant variable: a11913 
o|substituted constant variable: a11915 
o|substituted constant variable: a11917 
o|substituted constant variable: a11919 
o|substituted constant variable: a11921 
o|substituted constant variable: a11923 
o|substituted constant variable: a11925 
o|substituted constant variable: a11927 
o|substituted constant variable: a11929 
o|substituted constant variable: a11932 
o|substituted constant variable: a11937 
o|substituted constant variable: a11940 
o|substituted constant variable: a11946 
o|substituted constant variable: a11949 
o|inlining procedure: k11174 
o|inlining procedure: k11956 
o|inlining procedure: k11956 
o|substituted constant variable: a11970 
o|substituted constant variable: a11973 
o|inlining procedure: k11992 
o|substituted constant variable: a12029 
o|contracted procedure: "(irregex-core.scm:1014) chicken.irregex#string-parse-cset" 
o|inlining procedure: k13406 
o|inlining procedure: k13406 
o|inlining procedure: k13424 
o|substituted constant variable: a13435 
o|inlining procedure: k13424 
o|contracted procedure: "(irregex-core.scm:1234) chicken.irregex#cset->sre" 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k13463 
o|substituted constant variable: a13483 
o|inlining procedure: k13488 
o|inlining procedure: k13488 
o|inlining procedure: k13506 
o|inlining procedure: k13506 
o|inlining procedure: k13530 
o|inlining procedure: k13530 
o|substituted constant variable: a13560 
o|inlining procedure: k13561 
o|substituted constant variable: a13583 
o|inlining procedure: k13561 
o|substituted constant variable: a13589 
o|substituted constant variable: a13607 
o|substituted constant variable: a13610 
o|substituted constant variable: a13611 
o|substituted constant variable: a13619 
o|substituted constant variable: a13623 
o|inlining procedure: k13627 
o|inlining procedure: k13627 
o|substituted constant variable: a13633 
o|substituted constant variable: a13640 
o|substituted constant variable: a13647 
o|inlining procedure: k13463 
o|inlining procedure: k13671 
o|substituted constant variable: a13703 
o|consed rest parameter at call site: "(irregex-core.scm:1270) chicken.irregex#sre->cset" 2 
o|substituted constant variable: a13721 
o|substituted constant variable: a13729 
o|substituted constant variable: a13740 
o|inlining procedure: k13671 
o|substituted constant variable: a13758 
o|substituted constant variable: a13767 
o|substituted constant variable: a13769 
o|substituted constant variable: a13771 
o|substituted constant variable: a13773 
o|substituted constant variable: a13775 
o|substituted constant variable: a13776 
o|substituted constant variable: a13783 
o|inlining procedure: k13784 
o|substituted constant variable: a13807 
o|consed rest parameter at call site: "(irregex-core.scm:1285) chicken.irregex#sre->cset" 2 
o|inlining procedure: k13824 
o|substituted constant variable: a13849 
o|inlining procedure: k13824 
o|substituted constant variable: a13861 
o|substituted constant variable: a13870 
o|inlining procedure: k13874 
o|inlining procedure: k13874 
o|inlining procedure: k13886 
o|inlining procedure: k13886 
o|substituted constant variable: a13899 
o|substituted constant variable: a13901 
o|substituted constant variable: a13903 
o|substituted constant variable: a13905 
o|substituted constant variable: a13907 
o|substituted constant variable: a13909 
o|substituted constant variable: a13912 
o|inlining procedure: k13784 
o|substituted constant variable: a13946 
o|substituted constant variable: a13959 
o|substituted constant variable: a13961 
o|substituted constant variable: a13963 
o|substituted constant variable: a13965 
o|inlining procedure: k13966 
o|inlining procedure: k13977 
o|inlining procedure: k13977 
o|inlining procedure: "(irregex-core.scm:1307) chicken.irregex#make-cset" 
o|substituted constant variable: chicken.irregex#~multi-line? 
o|inlining procedure: k13966 
o|inlining procedure: "(irregex-core.scm:1308) chicken.irregex#make-cset" 
o|substituted constant variable: chicken.irregex#~utf8? 
o|substituted constant variable: a14002 
o|inlining procedure: k11992 
o|substituted constant variable: a12041 
o|inlining procedure: k12036 
o|substituted constant variable: a12051 
o|inlining procedure: k12036 
o|inlining procedure: k12080 
o|inlining procedure: k12080 
o|substituted constant variable: a12103 
o|substituted constant variable: a12106 
o|inlining procedure: k12115 
o|substituted constant variable: a12123 
o|substituted constant variable: a12126 
o|inlining procedure: k12115 
o|substituted constant variable: a12140 
o|substituted constant variable: a12143 
o|inlining procedure: k12154 
o|inlining procedure: k12154 
o|contracted procedure: "(irregex-core.scm:1030) chicken.irregex#string-split-char" 
o|inlining procedure: k8980 
o|inlining procedure: k8980 
o|substituted constant variable: a9004 
o|substituted constant variable: a9007 
o|substituted constant variable: a9017 
o|substituted constant variable: a12201 
o|substituted constant variable: a12204 
o|inlining procedure: k12212 
o|inlining procedure: k12212 
o|substituted constant variable: a12218 
o|substituted constant variable: a12225 
o|substituted constant variable: a12232 
o|inlining procedure: k12233 
o|substituted constant variable: a12244 
o|inlining procedure: k12252 
o|substituted constant variable: a12263 
o|substituted constant variable: a12266 
o|inlining procedure: k12252 
o|substituted constant variable: a12286 
o|substituted constant variable: a12289 
o|inlining procedure: k12302 
o|substituted constant variable: a12313 
o|substituted constant variable: a12316 
o|inlining procedure: k12302 
o|substituted constant variable: a12336 
o|substituted constant variable: a12339 
o|inlining procedure: k12352 
o|substituted constant variable: a12363 
o|substituted constant variable: a12366 
o|inlining procedure: k12352 
o|substituted constant variable: a12394 
o|substituted constant variable: a12397 
o|inlining procedure: k12418 
o|substituted constant variable: a12429 
o|substituted constant variable: a12432 
o|inlining procedure: k12418 
o|substituted constant variable: a12456 
o|substituted constant variable: a12459 
o|inlining procedure: k12468 
o|substituted constant variable: a12479 
o|substituted constant variable: a12482 
o|inlining procedure: k12468 
o|substituted constant variable: a12502 
o|substituted constant variable: a12505 
o|inlining procedure: k12522 
o|substituted constant variable: a12533 
o|substituted constant variable: a12536 
o|inlining procedure: k12522 
o|substituted constant variable: a12556 
o|substituted constant variable: a12559 
o|inlining procedure: k12568 
o|substituted constant variable: a12579 
o|substituted constant variable: a12582 
o|inlining procedure: k12568 
o|substituted constant variable: a12602 
o|substituted constant variable: a12605 
o|inlining procedure: k12614 
o|substituted constant variable: a12625 
o|substituted constant variable: a12628 
o|inlining procedure: k12614 
o|substituted constant variable: a12674 
o|inlining procedure: k12675 
o|inlining procedure: k12705 
o|inlining procedure: k12705 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: a12749 
o|substituted constant variable: a12752 
o|contracted procedure: "(irregex-core.scm:1099) chicken.irregex#char-mirror" 
o|inlining procedure: k13262 
o|inlining procedure: k13262 
o|inlining procedure: k13274 
o|inlining procedure: k13274 
o|substituted constant variable: a13287 
o|substituted constant variable: a13289 
o|substituted constant variable: a13291 
o|substituted constant variable: a13293 
o|substituted constant variable: a12755 
o|substituted constant variable: a12758 
o|inlining procedure: k12675 
o|substituted constant variable: a12773 
o|inlining procedure: k12777 
o|inlining procedure: k12777 
o|substituted constant variable: a12801 
o|inlining procedure: k12796 
o|substituted constant variable: a12808 
o|inlining procedure: k12796 
o|substituted constant variable: a12824 
o|substituted constant variable: a12827 
o|substituted constant variable: a12845 
o|substituted constant variable: a12846 
o|substituted constant variable: a12853 
o|substituted constant variable: a12859 
o|substituted constant variable: a12860 
o|inlining procedure: k12865 
o|substituted constant variable: a12876 
o|inlining procedure: k12865 
o|substituted constant variable: a12930 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: a12946 
o|contracted procedure: "(irregex-core.scm:1140) chicken.irregex#string-scan-pred" 
o|inlining procedure: k8932 
o|inlining procedure: k8932 
o|substituted constant variable: a8949 
o|inlining procedure: k12947 
o|substituted constant variable: a12964 
o|substituted constant variable: a12967 
o|inlining procedure: k12947 
o|substituted constant variable: a12988 
o|substituted constant variable: a12991 
o|substituted constant variable: a12997 
o|substituted constant variable: a12999 
o|substituted constant variable: a13001 
o|substituted constant variable: a13003 
o|substituted constant variable: a13005 
o|substituted constant variable: a13007 
o|substituted constant variable: a13009 
o|substituted constant variable: a13011 
o|substituted constant variable: a13013 
o|substituted constant variable: a13015 
o|substituted constant variable: a13017 
o|substituted constant variable: a13019 
o|substituted constant variable: a13021 
o|substituted constant variable: a13023 
o|substituted constant variable: a13025 
o|substituted constant variable: a13027 
o|substituted constant variable: a13029 
o|substituted constant variable: a13031 
o|substituted constant variable: a13033 
o|substituted constant variable: a13036 
o|inlining procedure: k12233 
o|substituted constant variable: a13048 
o|substituted constant variable: a13051 
o|inlining procedure: k13060 
o|substituted constant variable: a13074 
o|substituted constant variable: a13077 
o|substituted constant variable: chicken.irregex#~multi-line? 
o|inlining procedure: k13060 
o|substituted constant variable: a13103 
o|substituted constant variable: a13106 
o|substituted constant variable: chicken.irregex#~multi-line? 
o|inlining procedure: k13118 
o|substituted constant variable: a13135 
o|substituted constant variable: a13138 
o|substituted constant variable: a13148 
o|substituted constant variable: chicken.irregex#~ignore-space? 
o|inlining procedure: k13118 
o|inlining procedure: k13155 
o|substituted constant variable: a13183 
o|substituted constant variable: a13186 
o|inlining procedure: k13155 
o|substituted constant variable: a13192 
o|substituted constant variable: chicken.irregex#~ignore-space? 
o|substituted constant variable: a13198 
o|substituted constant variable: a13200 
o|substituted constant variable: a13202 
o|substituted constant variable: a13204 
o|substituted constant variable: a13206 
o|substituted constant variable: a13208 
o|substituted constant variable: a13210 
o|substituted constant variable: a13212 
o|substituted constant variable: a13214 
o|substituted constant variable: a13216 
o|substituted constant variable: a13218 
o|substituted constant variable: a13223 
o|substituted constant variable: a13225 
o|substituted constant variable: a13227 
o|substituted constant variable: a13229 
o|inlining procedure: k13247 
o|inlining procedure: k13247 
o|inlining procedure: k13297 
o|inlining procedure: k13297 
o|inlining procedure: k13315 
o|inlining procedure: k13315 
o|substituted constant variable: a13345 
o|substituted constant variable: a13348 
o|contracted procedure: "(irregex-core.scm:1202) chicken.irregex#string-scan-char-escape" 
o|inlining procedure: k8875 
o|inlining procedure: k8875 
o|inlining procedure: k8887 
o|substituted constant variable: a8898 
o|inlining procedure: k8887 
o|substituted constant variable: a8904 
o|substituted constant variable: a8906 
o|inlining procedure: k13349 
o|inlining procedure: k13349 
o|substituted constant variable: a13376 
o|substituted constant variable: a13382 
o|substituted constant variable: a13385 
o|substituted constant variable: a13386 
o|substituted constant variable: a14013 
o|substituted constant variable: table1944 
o|inlining procedure: k14041 
o|inlining procedure: k14041 
o|inlining procedure: k14084 
o|inlining procedure: k14084 
o|substituted constant variable: a14210 
o|substituted constant variable: a14212 
o|substituted constant variable: a14214 
o|substituted constant variable: a14216 
o|inlining procedure: k14264 
o|inlining procedure: k14264 
o|inlining procedure: k14276 
o|inlining procedure: k14276 
o|substituted constant variable: a14292 
o|substituted constant variable: a14294 
o|substituted constant variable: a14296 
o|substituted constant variable: a14298 
o|inlining procedure: k14343 
o|inlining procedure: k14343 
o|inlining procedure: k14377 
o|inlining procedure: k14377 
o|substituted constant variable: a14467 
o|substituted constant variable: a14469 
o|substituted constant variable: a14471 
o|substituted constant variable: a14473 
o|inlining procedure: k14616 
o|inlining procedure: k14616 
o|inlining procedure: k14682 
o|contracted procedure: "(irregex-core.scm:1446) g20832092" 
o|inlining procedure: k14682 
o|inlining procedure: k14718 
o|inlining procedure: k14718 
o|inlining procedure: k14839 
o|inlining procedure: k14839 
o|inlining procedure: k14978 
o|inlining procedure: k14978 
o|inlining procedure: k15481 
o|inlining procedure: k15499 
o|inlining procedure: k15499 
o|inlining procedure: k15481 
o|contracted procedure: "(irregex-core.scm:1534) chicken.irregex#unicode-range->utf8-pattern" 
o|inlining procedure: k14483 
o|contracted procedure: "(irregex-core.scm:1419) chicken.irregex#unicode-range-climb-digits" 
o|inlining procedure: k15084 
o|contracted procedure: "(irregex-core.scm:1501) g22432252" 
o|inlining procedure: k15084 
o|contracted procedure: "(irregex-core.scm:1500) chicken.irregex#utf8-highest-digit-of-length" 
o|inlining procedure: k14302 
o|inlining procedure: k14302 
o|inlining procedure: k14314 
o|inlining procedure: k14314 
o|substituted constant variable: a14330 
o|substituted constant variable: a14332 
o|substituted constant variable: a14334 
o|substituted constant variable: a14336 
o|inlining procedure: k15187 
o|contracted procedure: "(irregex-core.scm:1512) g22832292" 
o|inlining procedure: k15187 
o|inlining procedure: k15255 
o|inlining procedure: k15255 
o|substituted constant variable: a15293 
o|inlining procedure: k15325 
o|contracted procedure: "(irregex-core.scm:1492) g21902199" 
o|inlining procedure: k15325 
o|inlining procedure: k15365 
o|inlining procedure: k15365 
o|inlining procedure: k14483 
o|inlining procedure: k14510 
o|inlining procedure: k14528 
o|inlining procedure: k14528 
o|inlining procedure: k14510 
o|contracted procedure: "(irregex-core.scm:1433) chicken.irregex#unicode-range-middle" 
o|inlining procedure: k15416 
o|contracted procedure: "(irregex-core.scm:1521) g23212330" 
o|inlining procedure: k15416 
o|inlining procedure: k16038 
o|inlining procedure: k16038 
o|inlining procedure: k16050 
o|inlining procedure: k16050 
o|inlining procedure: k16068 
o|inlining procedure: k16068 
o|inlining procedure: k16086 
o|inlining procedure: k16086 
o|inlining procedure: k16151 
o|inlining procedure: k16151 
o|inlining procedure: k16212 
o|inlining procedure: k16212 
o|contracted procedure: "(irregex-core.scm:1639) chicken.irregex#sre->procedure" 
o|inlining procedure: "(irregex-core.scm:3135) chicken.irregex#%irregex-match-fail-set!" 
o|inlining procedure: k23454 
o|consed rest parameter at call site: "(irregex-core.scm:3143) chicken.irregex#sre->cset" 2 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k23480 
o|consed rest parameter at call site: "(irregex-core.scm:3148) chicken.irregex#sre->cset" 2 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k23480 
o|inlining procedure: k23509 
o|inlining procedure: k23509 
o|substituted constant variable: a23575 
o|substituted constant variable: a23577 
o|inlining procedure: k23580 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k23580 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k23618 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k23618 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k23656 
o|inlining procedure: k23674 
o|inlining procedure: k23674 
o|substituted constant variable: a23718 
o|substituted constant variable: a23720 
o|inlining procedure: k23656 
o|inlining procedure: k23749 
o|inlining procedure: k23749 
o|inlining procedure: k23781 
o|inlining procedure: k23781 
o|inlining procedure: k23820 
o|inlining procedure: k23820 
o|inlining procedure: k23880 
o|inlining procedure: k23880 
o|inlining procedure: k23917 
o|inlining procedure: k23917 
o|inlining procedure: k23976 
o|inlining procedure: k24012 
o|inlining procedure: k24012 
o|inlining procedure: k24030 
o|inlining procedure: k24030 
o|inlining procedure: k24053 
o|inlining procedure: k24053 
o|inlining procedure: k24070 
o|inlining procedure: k24070 
o|inlining procedure: k24081 
o|inlining procedure: k24081 
o|inlining procedure: k24089 
o|inlining procedure: k24089 
o|inlining procedure: k24104 
o|inlining procedure: k24104 
o|inlining procedure: k23976 
o|inlining procedure: k24134 
o|inlining procedure: k24164 
o|inlining procedure: k24164 
o|inlining procedure: k24186 
o|inlining procedure: k24186 
o|inlining procedure: k24134 
o|inlining procedure: k24220 
o|inlining procedure: k24220 
o|inlining procedure: k24243 
o|inlining procedure: k24243 
o|inlining procedure: k24251 
o|inlining procedure: k24251 
o|inlining procedure: k24266 
o|inlining procedure: k24266 
o|inlining procedure: k24290 
o|inlining procedure: k24290 
o|inlining procedure: k24348 
o|inlining procedure: k24348 
o|inlining procedure: k24376 
o|inlining procedure: k24376 
o|inlining procedure: k24400 
o|inlining procedure: k24411 
o|inlining procedure: k24411 
o|inlining procedure: k24400 
o|inlining procedure: k24465 
o|inlining procedure: k24465 
o|inlining procedure: "(irregex-core.scm:3354) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3352) chicken.irregex#chunker-get-str" 
o|contracted procedure: "(irregex-core.scm:3351) chicken.irregex#wrap-end-chunker" 
o|inlining procedure: k8738 
o|inlining procedure: "(irregex-core.scm:400) chicken.irregex#chunker-get-next" 
o|inlining procedure: k8738 
o|inlining procedure: k8765 
o|inlining procedure: k8765 
o|inlining procedure: "(irregex-core.scm:412) chicken.irregex#chunker-get-end" 
o|inlining procedure: k8780 
o|inlining procedure: "(irregex-core.scm:410) chicken.irregex#chunker-get-next" 
o|inlining procedure: k8780 
o|inlining procedure: "(irregex-core.scm:409) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:414) chicken.irregex#chunker-get-subchunk" 
o|inlining procedure: "(irregex-core.scm:413) chicken.irregex#chunker-get-substring" 
o|inlining procedure: "(irregex-core.scm:402) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:401) chicken.irregex#chunker-get-str" 
o|inlining procedure: k24521 
o|inlining procedure: k24535 
o|inlining procedure: k24535 
o|inlining procedure: k24521 
o|inlining procedure: k24571 
o|inlining procedure: k24585 
o|inlining procedure: k24585 
o|contracted procedure: "(irregex-core.scm:3392) g48084809" 
o|inlining procedure: k24620 
o|inlining procedure: k24620 
o|inlining procedure: k24571 
o|inlining procedure: k24709 
o|inlining procedure: k24729 
o|inlining procedure: k24729 
o|inlining procedure: k24748 
o|inlining procedure: k24748 
o|inlining procedure: k24798 
o|inlining procedure: "(irregex-core.scm:3437) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3436) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3435) chicken.irregex#chunker-get-str" 
o|inlining procedure: k24798 
o|inlining procedure: "(irregex-core.scm:3432) chicken.irregex#chunker-get-next" 
o|inlining procedure: k24866 
o|inlining procedure: k24866 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k24882 
o|inlining procedure: k24882 
o|inlining procedure: k24709 
o|inlining procedure: k24926 
o|contracted procedure: "(irregex-core.scm:3466) g49214922" 
o|contracted procedure: "(irregex-core.scm:3465) g49174918" 
o|contracted procedure: "(irregex-core.scm:3452) g48874888" 
o|inlining procedure: k24926 
o|substituted constant variable: a25169 
o|substituted constant variable: a25171 
o|substituted constant variable: a25176 
o|substituted constant variable: a25178 
o|substituted constant variable: a25180 
o|substituted constant variable: a25185 
o|substituted constant variable: a25187 
o|substituted constant variable: a25189 
o|substituted constant variable: a25191 
o|substituted constant variable: a25196 
o|substituted constant variable: a25198 
o|substituted constant variable: a25200 
o|substituted constant variable: a25202 
o|substituted constant variable: a25204 
o|substituted constant variable: a25206 
o|substituted constant variable: a25208 
o|substituted constant variable: a25210 
o|substituted constant variable: a25212 
o|substituted constant variable: a25214 
o|substituted constant variable: a25216 
o|substituted constant variable: a25218 
o|substituted constant variable: a25220 
o|substituted constant variable: a25222 
o|substituted constant variable: a25224 
o|substituted constant variable: a25226 
o|substituted constant variable: a25231 
o|substituted constant variable: a25233 
o|substituted constant variable: a25235 
o|substituted constant variable: a25237 
o|substituted constant variable: a25239 
o|substituted constant variable: a25241 
o|substituted constant variable: a25243 
o|inlining procedure: k25247 
o|inlining procedure: k25247 
o|substituted constant variable: a25260 
o|substituted constant variable: a25262 
o|substituted constant variable: a25264 
o|substituted constant variable: a25266 
o|inlining procedure: k23454 
o|inlining procedure: k25275 
o|inlining procedure: k25283 
o|inlining procedure: k25283 
o|inlining procedure: "(irregex-core.scm:3490) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3489) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3488) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:3486) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25275 
o|inlining procedure: k25341 
o|substituted constant variable: a25358 
o|substituted constant variable: a25366 
o|inlining procedure: k25341 
o|inlining procedure: k25398 
o|substituted constant variable: a25409 
o|inlining procedure: k25398 
o|substituted constant variable: a25417 
o|inlining procedure: "(irregex-core.scm:3503) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3502) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3501) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:3499) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25425 
o|inlining procedure: k25433 
o|inlining procedure: k25433 
o|inlining procedure: k25425 
o|inlining procedure: k25463 
o|inlining procedure: k25463 
o|substituted constant variable: a25484 
o|inlining procedure: "(irregex-core.scm:3515) chicken.irregex#chunker-get-start" 
o|inlining procedure: k25506 
o|inlining procedure: k25514 
o|inlining procedure: k25514 
o|inlining procedure: k25529 
o|inlining procedure: k25529 
o|inlining procedure: "(irregex-core.scm:3533) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3532) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:3529) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25596 
o|inlining procedure: k25596 
o|inlining procedure: "(irregex-core.scm:3523) chicken.irregex#chunker-get-start" 
o|inlining procedure: k25506 
o|inlining procedure: k25621 
o|inlining procedure: k25621 
o|inlining procedure: "(irregex-core.scm:3538) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25646 
o|inlining procedure: k25654 
o|inlining procedure: k25654 
o|substituted constant variable: a25672 
o|inlining procedure: k25683 
o|inlining procedure: k25683 
o|substituted constant variable: a25692 
o|inlining procedure: "(irregex-core.scm:3550) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3549) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:3545) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25646 
o|inlining procedure: k25719 
o|inlining procedure: k25719 
o|inlining procedure: k25734 
o|inlining procedure: k25734 
o|inlining procedure: "(irregex-core.scm:3559) chicken.irregex#chunker-get-start" 
o|inlining procedure: k25790 
o|inlining procedure: k25790 
o|inlining procedure: k25800 
o|inlining procedure: k25814 
o|inlining procedure: k25814 
o|inlining procedure: k25826 
o|inlining procedure: k25826 
o|inlining procedure: "(irregex-core.scm:3570) chicken.irregex#chunker-get-start" 
o|inlining procedure: k25800 
o|inlining procedure: k25883 
o|inlining procedure: k25883 
o|substituted constant variable: a25897 
o|substituted constant variable: a25899 
o|substituted constant variable: a25901 
o|substituted constant variable: a25903 
o|substituted constant variable: a25905 
o|substituted constant variable: a25907 
o|substituted constant variable: a25909 
o|substituted constant variable: a25911 
o|substituted constant variable: a25913 
o|substituted constant variable: a25915 
o|inlining procedure: k25916 
o|inlining procedure: k25930 
o|inlining procedure: k25949 
o|inlining procedure: "(irregex-core.scm:3597) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25986 
o|substituted constant variable: a25997 
o|inlining procedure: k25986 
o|inlining procedure: "(irregex-core.scm:3595) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3594) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3593) chicken.irregex#chunker-get-str" 
o|inlining procedure: k25949 
o|inlining procedure: "(irregex-core.scm:3591) chicken.irregex#chunker-get-next" 
o|inlining procedure: k25930 
o|substituted constant variable: a26019 
o|inlining procedure: k26029 
o|inlining procedure: k26048 
o|inlining procedure: "(irregex-core.scm:3615) chicken.irregex#chunker-get-next" 
o|inlining procedure: k26085 
o|substituted constant variable: a26096 
o|inlining procedure: k26085 
o|inlining procedure: "(irregex-core.scm:3613) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3612) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3611) chicken.irregex#chunker-get-str" 
o|inlining procedure: k26048 
o|inlining procedure: "(irregex-core.scm:3609) chicken.irregex#chunker-get-next" 
o|inlining procedure: k26029 
o|substituted constant variable: a26118 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k25916 
o|substituted constant variable: chicken.irregex#~none 
o|inlining procedure: k16229 
o|substituted constant variable: chicken.irregex#~consumer? 
o|propagated global variable: r1623031862 chicken.irregex#~consumer? 
o|inlining procedure: k16229 
o|substituted constant variable: chicken.irregex#~none 
o|inlining procedure: k16236 
o|substituted constant variable: chicken.irregex#~searcher? 
o|propagated global variable: r1623731866 chicken.irregex#~searcher? 
o|substituted constant variable: chicken.irregex#~none 
o|inlining procedure: k16236 
o|substituted constant variable: chicken.irregex#~none 
o|contracted procedure: "(irregex-core.scm:1631) chicken.irregex#sre-length-ranges" 
o|inlining procedure: k17107 
o|inlining procedure: k17107 
o|inlining procedure: k17113 
o|inlining procedure: k17129 
o|inlining procedure: k17129 
o|inlining procedure: k17158 
o|inlining procedure: k17175 
o|inlining procedure: k17189 
o|inlining procedure: k17189 
o|inlining procedure: k17175 
o|inlining procedure: k17224 
o|inlining procedure: k17224 
o|inlining procedure: k17158 
o|inlining procedure: k17253 
o|inlining procedure: k17267 
o|inlining procedure: k17267 
o|inlining procedure: k17277 
o|inlining procedure: k17277 
o|inlining procedure: k17253 
o|inlining procedure: k17306 
o|inlining procedure: k17306 
o|inlining procedure: k17324 
o|inlining procedure: k17394 
o|inlining procedure: k17400 
o|inlining procedure: k17400 
o|inlining procedure: k17394 
o|inlining procedure: k17427 
o|inlining procedure: k17427 
o|inlining procedure: k17324 
o|inlining procedure: k17482 
o|inlining procedure: k17516 
o|inlining procedure: k17516 
o|inlining procedure: k17482 
o|inlining procedure: k17540 
o|inlining procedure: k17540 
o|inlining procedure: k17575 
o|inlining procedure: k17575 
o|inlining procedure: k17618 
o|inlining procedure: k17618 
o|inlining procedure: k17637 
o|inlining procedure: k17637 
o|inlining procedure: k17670 
o|inlining procedure: k17670 
o|inlining procedure: k17706 
o|inlining procedure: k17706 
o|inlining procedure: k17764 
o|inlining procedure: k17764 
o|inlining procedure: k17779 
o|inlining procedure: k17779 
o|inlining procedure: k17803 
o|inlining procedure: k17803 
o|inlining procedure: k17851 
o|inlining procedure: k17851 
o|inlining procedure: k17862 
o|substituted constant variable: a17899 
o|inlining procedure: k17891 
o|inlining procedure: k17891 
o|inlining procedure: k17862 
o|inlining procedure: k17917 
o|inlining procedure: k17917 
o|inlining procedure: k17948 
o|inlining procedure: k17948 
o|substituted constant variable: a17961 
o|substituted constant variable: a17963 
o|substituted constant variable: a17965 
o|substituted constant variable: a17967 
o|inlining procedure: k17971 
o|inlining procedure: k17971 
o|substituted constant variable: a17984 
o|substituted constant variable: a17986 
o|substituted constant variable: a17988 
o|substituted constant variable: a17990 
o|substituted constant variable: a17995 
o|substituted constant variable: a17997 
o|substituted constant variable: a17999 
o|substituted constant variable: a18004 
o|substituted constant variable: a18006 
o|substituted constant variable: a18011 
o|substituted constant variable: a18013 
o|substituted constant variable: a18018 
o|substituted constant variable: a18020 
o|inlining procedure: k18024 
o|inlining procedure: k18024 
o|substituted constant variable: a18037 
o|substituted constant variable: a18039 
o|substituted constant variable: a18041 
o|substituted constant variable: a18043 
o|substituted constant variable: a18045 
o|substituted constant variable: a18047 
o|substituted constant variable: a18049 
o|inlining procedure: k18053 
o|inlining procedure: k18053 
o|inlining procedure: k18065 
o|inlining procedure: k18065 
o|substituted constant variable: a18072 
o|substituted constant variable: a18074 
o|substituted constant variable: a18076 
o|substituted constant variable: a18078 
o|substituted constant variable: a18080 
o|substituted constant variable: a18082 
o|inlining procedure: k18086 
o|inlining procedure: k18086 
o|substituted constant variable: a18099 
o|substituted constant variable: a18101 
o|substituted constant variable: a18103 
o|substituted constant variable: a18105 
o|inlining procedure: k17113 
o|inlining procedure: k18117 
o|inlining procedure: k18117 
o|substituted constant variable: a18135 
o|substituted constant variable: a18143 
o|inlining procedure: k18139 
o|inlining procedure: k18139 
o|inlining procedure: k18157 
o|inlining procedure: k18157 
o|consed rest parameter at call site: "(irregex-core.scm:1627) chicken.irregex#nfa->dfa" 2 
o|substituted constant variable: a16258 
o|inlining procedure: k16260 
o|inlining procedure: k16260 
o|consed rest parameter at call site: "(irregex-core.scm:1623) chicken.irregex#nfa->dfa" 2 
o|substituted constant variable: a16293 
o|substituted constant variable: a16298 
o|inlining procedure: k16295 
o|inlining procedure: k16295 
o|substituted constant variable: chicken.irregex#*allow-utf8-mode?* 
o|contracted procedure: "(irregex-core.scm:1610) chicken.irregex#sre-adjust-utf8" 
o|inlining procedure: k15599 
o|inlining procedure: k15621 
o|inlining procedure: k15621 
o|inlining procedure: k15666 
o|inlining procedure: k15666 
o|inlining procedure: k15697 
o|inlining procedure: k15727 
o|inlining procedure: k15727 
o|inlining procedure: k15697 
o|inlining procedure: k15767 
o|inlining procedure: k15767 
o|inlining procedure: k15782 
o|inlining procedure: k15782 
o|consed rest parameter at call site: "(irregex-core.scm:1555) chicken.irregex#sre->cset" 2 
o|inlining procedure: k15799 
o|inlining procedure: k15814 
o|inlining procedure: k15814 
o|inlining procedure: k15837 
o|inlining procedure: k15837 
o|substituted constant variable: a15869 
o|substituted constant variable: a15871 
o|inlining procedure: k15799 
o|inlining procedure: k15893 
o|inlining procedure: k15893 
o|substituted constant variable: a15925 
o|inlining procedure: k15929 
o|inlining procedure: k15929 
o|substituted constant variable: a15942 
o|substituted constant variable: a15944 
o|substituted constant variable: a15946 
o|substituted constant variable: a15948 
o|substituted constant variable: a15950 
o|substituted constant variable: a15952 
o|substituted constant variable: a15954 
o|substituted constant variable: a15956 
o|inlining procedure: k15599 
o|inlining procedure: k15963 
o|inlining procedure: k15963 
o|inlining procedure: k15991 
o|inlining procedure: k15991 
o|inlining procedure: k16022 
o|inlining procedure: k16022 
o|substituted constant variable: a16032 
o|substituted constant variable: a16034 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k16309 
o|inlining procedure: k16325 
o|inlining procedure: k16325 
o|inlining procedure: k16359 
o|inlining procedure: k16359 
o|inlining procedure: k16376 
o|inlining procedure: k16376 
o|inlining procedure: k16388 
o|inlining procedure: k16388 
o|inlining procedure: k16400 
o|inlining procedure: k16400 
o|substituted constant variable: a16413 
o|substituted constant variable: a16415 
o|substituted constant variable: a16417 
o|substituted constant variable: a16419 
o|substituted constant variable: a16421 
o|substituted constant variable: a16423 
o|substituted constant variable: a16425 
o|substituted constant variable: a16427 
o|substituted constant variable: a16429 
o|substituted constant variable: a16431 
o|inlining procedure: k16435 
o|inlining procedure: k16435 
o|inlining procedure: k16447 
o|inlining procedure: k16447 
o|substituted constant variable: a16460 
o|substituted constant variable: a16462 
o|substituted constant variable: a16464 
o|substituted constant variable: a16466 
o|substituted constant variable: a16468 
o|substituted constant variable: a16470 
o|inlining procedure: k16309 
o|substituted constant variable: a16472 
o|inlining procedure: k16479 
o|inlining procedure: k16479 
o|inlining procedure: k16489 
o|inlining procedure: k16504 
o|inlining procedure: k16504 
o|inlining procedure: k16489 
o|substituted constant variable: a16534 
o|inlining procedure: k16538 
o|inlining procedure: k16538 
o|inlining procedure: k16550 
o|inlining procedure: k16550 
o|substituted constant variable: a16563 
o|substituted constant variable: a16565 
o|substituted constant variable: a16567 
o|substituted constant variable: a16569 
o|substituted constant variable: a16571 
o|substituted constant variable: a16573 
o|substituted constant variable: a16585 
o|inlining procedure: k16577 
o|substituted constant variable: a16594 
o|inlining procedure: k16589 
o|inlining procedure: k16601 
o|inlining procedure: k16601 
o|inlining procedure: k16589 
o|inlining procedure: k16577 
o|inlining procedure: k16622 
o|inlining procedure: k16638 
o|inlining procedure: k16638 
o|inlining procedure: k16653 
o|inlining procedure: k16653 
o|substituted constant variable: a16665 
o|inlining procedure: k16669 
o|inlining procedure: k16669 
o|inlining procedure: k16681 
o|inlining procedure: k16681 
o|substituted constant variable: a16694 
o|substituted constant variable: a16696 
o|substituted constant variable: a16698 
o|substituted constant variable: a16700 
o|substituted constant variable: a16702 
o|substituted constant variable: a16704 
o|inlining procedure: k16622 
o|inlining procedure: k16714 
o|inlining procedure: k16714 
o|inlining procedure: k16724 
o|inlining procedure: k16724 
o|inlining procedure: k16751 
o|inlining procedure: k16751 
o|inlining procedure: k16766 
o|inlining procedure: k16766 
o|substituted constant variable: a16778 
o|inlining procedure: k16782 
o|inlining procedure: k16782 
o|inlining procedure: k16794 
o|inlining procedure: k16794 
o|substituted constant variable: a16807 
o|substituted constant variable: a16809 
o|substituted constant variable: a16811 
o|substituted constant variable: a16813 
o|substituted constant variable: a16815 
o|substituted constant variable: a16817 
o|substituted constant variable: a16822 
o|substituted constant variable: a16824 
o|inlining procedure: k16831 
o|inlining procedure: k16831 
o|inlining procedure: k16841 
o|inlining procedure: k16841 
o|inlining procedure: k16868 
o|inlining procedure: k16868 
o|inlining procedure: k16883 
o|inlining procedure: k16883 
o|substituted constant variable: a16895 
o|inlining procedure: k16899 
o|inlining procedure: k16899 
o|inlining procedure: k16911 
o|inlining procedure: k16911 
o|substituted constant variable: a16924 
o|substituted constant variable: a16926 
o|substituted constant variable: a16928 
o|substituted constant variable: a16930 
o|substituted constant variable: a16932 
o|substituted constant variable: a16934 
o|substituted constant variable: a16939 
o|substituted constant variable: a16941 
o|substituted constant variable: a16953 
o|inlining procedure: k16945 
o|inlining procedure: k16957 
o|inlining procedure: k16957 
o|inlining procedure: k16945 
o|inlining procedure: k16990 
o|inlining procedure: k17007 
o|inlining procedure: k17007 
o|inlining procedure: k17033 
o|inlining procedure: k17033 
o|substituted constant variable: a17051 
o|substituted constant variable: a17053 
o|inlining procedure: k17057 
o|inlining procedure: k17057 
o|substituted constant variable: a17070 
o|substituted constant variable: a17072 
o|substituted constant variable: a17074 
o|substituted constant variable: a17076 
o|inlining procedure: k16990 
o|inlining procedure: k18193 
o|inlining procedure: k18193 
o|inlining procedure: k18216 
o|inlining procedure: k18216 
o|inlining procedure: k18239 
o|inlining procedure: k18239 
o|inlining procedure: k18266 
o|inlining procedure: k18266 
o|inlining procedure: k18312 
o|inlining procedure: k18312 
o|substituted constant variable: a18344 
o|substituted constant variable: a18349 
o|substituted constant variable: a18351 
o|substituted constant variable: a18356 
o|substituted constant variable: a18358 
o|inlining procedure: k18420 
o|inlining procedure: k18420 
o|inlining procedure: k18451 
o|inlining procedure: k18451 
o|inlining procedure: k18512 
o|inlining procedure: k18512 
o|inlining procedure: k18542 
o|inlining procedure: k18542 
o|substituted constant variable: a18562 
o|substituted constant variable: a18564 
o|substituted constant variable: a18569 
o|substituted constant variable: a18571 
o|inlining procedure: k18575 
o|inlining procedure: k18575 
o|inlining procedure: k18587 
o|inlining procedure: k18587 
o|inlining procedure: k18599 
o|inlining procedure: k18599 
o|inlining procedure: k18611 
o|inlining procedure: k18611 
o|inlining procedure: k18623 
o|inlining procedure: k18623 
o|inlining procedure: k18635 
o|inlining procedure: k18635 
o|inlining procedure: k18647 
o|inlining procedure: k18647 
o|substituted constant variable: a18654 
o|substituted constant variable: a18656 
o|substituted constant variable: a18658 
o|substituted constant variable: a18660 
o|substituted constant variable: a18662 
o|substituted constant variable: a18664 
o|substituted constant variable: a18666 
o|substituted constant variable: a18668 
o|substituted constant variable: a18670 
o|substituted constant variable: a18672 
o|substituted constant variable: a18674 
o|substituted constant variable: a18676 
o|substituted constant variable: a18678 
o|substituted constant variable: a18680 
o|substituted constant variable: a18682 
o|substituted constant variable: a18684 
o|substituted constant variable: a18689 
o|substituted constant variable: a18691 
o|substituted constant variable: a18696 
o|substituted constant variable: a18698 
o|inlining procedure: k18706 
o|inlining procedure: k18706 
o|inlining procedure: k18738 
o|inlining procedure: k18754 
o|inlining procedure: k18754 
o|inlining procedure: k18804 
o|inlining procedure: k18827 
o|inlining procedure: k18827 
o|inlining procedure: k18804 
o|substituted constant variable: a18859 
o|inlining procedure: k18863 
o|inlining procedure: k18863 
o|inlining procedure: k18875 
o|inlining procedure: k18875 
o|inlining procedure: k18887 
o|inlining procedure: k18887 
o|substituted constant variable: a18900 
o|substituted constant variable: a18902 
o|substituted constant variable: a18904 
o|substituted constant variable: a18906 
o|substituted constant variable: a18908 
o|substituted constant variable: a18910 
o|substituted constant variable: a18912 
o|substituted constant variable: a18914 
o|inlining procedure: k18738 
o|inlining procedure: k18957 
o|inlining procedure: k18957 
o|inlining procedure: k18971 
o|inlining procedure: k18971 
o|inlining procedure: k18984 
o|inlining procedure: k18984 
o|inlining procedure: "(irregex-core.scm:1983) chicken.irregex#irregex-match-chunker-set!" 
o|inlining procedure: "(irregex-core.scm:1981) chicken.irregex#chunker-get-start" 
o|inlining procedure: k19055 
o|inlining procedure: k19067 
o|inlining procedure: k19067 
o|inlining procedure: k19092 
o|inlining procedure: k19125 
o|inlining procedure: k19125 
o|inlining procedure: k19154 
o|inlining procedure: k19154 
o|inlining procedure: "(irregex-core.scm:2005) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:2004) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2003) chicken.irregex#chunker-get-start" 
o|inlining procedure: k19092 
o|contracted procedure: "(irregex-core.scm:2000) chicken.irregex#dfa-match/shortest" 
o|inlining procedure: k19680 
o|inlining procedure: k19695 
o|inlining procedure: k19695 
o|inlining procedure: k19746 
o|substituted constant variable: a19754 
o|inlining procedure: k19746 
o|inlining procedure: k19765 
o|inlining procedure: k19765 
o|inlining procedure: k19797 
o|inlining procedure: k19797 
o|inlining procedure: "(irregex-core.scm:2134) chicken.irregex#dfa-finalizer" 
o|inlining procedure: k19680 
o|inlining procedure: "(irregex-core.scm:2126) chicken.irregex#dfa-init-state" 
o|inlining procedure: "(irregex-core.scm:2124) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:2123) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2122) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:2121) chicken.irregex#chunker-get-str" 
o|substituted constant variable: chicken.irregex#~searcher? 
o|inlining procedure: k19055 
o|inlining procedure: "(irregex-core.scm:2023) chicken.irregex#%irregex-match-fail-set!" 
o|contracted procedure: "(irregex-core.scm:2022) chicken.irregex#irregex-search/backtrack" 
o|inlining procedure: k19212 
o|inlining procedure: k19212 
o|inlining procedure: k19230 
o|inlining procedure: k19230 
o|inlining procedure: k19266 
o|inlining procedure: "(irregex-core.scm:2050) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2049) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:2048) chicken.irregex#chunker-get-str" 
o|inlining procedure: k19266 
o|substituted constant variable: chicken.irregex#~searcher? 
o|inlining procedure: "(irregex-core.scm:2030) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:2029) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2028) chicken.irregex#chunker-get-str" 
o|contracted procedure: "(irregex-core.scm:2027) chicken.irregex#irregex-nfa" 
o|contracted procedure: "(irregex-core.scm:1990) chicken.irregex#irregex-dfa" 
o|inlining procedure: k19339 
o|inlining procedure: k19339 
o|inlining procedure: k19353 
o|inlining procedure: k19353 
o|inlining procedure: k19366 
o|inlining procedure: k19366 
o|inlining procedure: k19403 
o|inlining procedure: k19415 
o|inlining procedure: "(irregex-core.scm:2080) chicken.irregex#chunker-get-start" 
o|inlining procedure: k19415 
o|contracted procedure: "(irregex-core.scm:2074) g33953396" 
o|inlining procedure: "(irregex-core.scm:2074) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2073) chicken.irregex#chunker-get-start" 
o|inlining procedure: k19403 
o|inlining procedure: k19520 
o|inlining procedure: "(irregex-core.scm:2096) chicken.irregex#%irregex-match-fail-set!" 
o|inlining procedure: k19532 
o|inlining procedure: "(irregex-core.scm:2099) chicken.irregex#%irregex-match-fail" 
o|inlining procedure: k19532 
o|inlining procedure: "(irregex-core.scm:2098) chicken.irregex#%irregex-match-fail" 
o|contracted procedure: "(irregex-core.scm:2094) g34353436" 
o|inlining procedure: "(irregex-core.scm:2093) chicken.irregex#chunker-get-end" 
o|contracted procedure: "(irregex-core.scm:2092) g34293430" 
o|inlining procedure: "(irregex-core.scm:2091) chicken.irregex#chunker-get-next" 
o|inlining procedure: k19520 
o|inlining procedure: "(irregex-core.scm:2086) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2085) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:2084) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:2068) chicken.irregex#irregex-match-chunker-set!" 
o|inlining procedure: k19622 
o|inlining procedure: k19622 
o|removed unused formal parameters: (dfa3453) 
o|removed unused formal parameters: (dfa3456) 
o|substituted constant variable: a19843 
o|substituted constant variable: a19840 
o|inlining procedure: k19845 
o|inlining procedure: k19845 
o|substituted constant variable: a19859 
o|substituted constant variable: a19856 
o|inlining procedure: k19861 
o|inlining procedure: k19861 
o|inlining procedure: k19878 
o|inlining procedure: k19878 
o|inlining procedure: k20002 
o|inlining procedure: k20024 
o|inlining procedure: k20050 
o|inlining procedure: k20050 
o|contracted procedure: "(irregex-core.scm:2216) g36363637" 
o|inlining procedure: k20024 
o|inlining procedure: k20002 
o|inlining procedure: k20123 
o|substituted constant variable: a20131 
o|substituted constant variable: a20134 
o|inlining procedure: k20123 
o|substituted constant variable: a20140 
o|inlining procedure: k20228 
o|contracted procedure: "(irregex-core.scm:2246) g36793697" 
o|inlining procedure: k20228 
o|inlining procedure: k20251 
o|inlining procedure: k20251 
o|inlining procedure: k20285 
o|inlining procedure: k20285 
o|inlining procedure: "(irregex-core.scm:2232) chicken.irregex#dfa-cell-commands" 
o|inlining procedure: "(irregex-core.scm:2229) chicken.irregex#dfa-finalizer" 
o|inlining procedure: k20314 
o|inlining procedure: k20314 
o|inlining procedure: k20362 
o|inlining procedure: k20362 
o|contracted procedure: "(irregex-core.scm:2264) g37553756" 
o|inlining procedure: k20394 
o|inlining procedure: k20394 
o|inlining procedure: k20459 
o|inlining procedure: k20459 
o|inlining procedure: "(irregex-core.scm:2192) chicken.irregex#dfa-cell-commands" 
o|inlining procedure: "(irregex-core.scm:2183) chicken.irregex#dfa-finalizer" 
o|contracted procedure: "(irregex-core.scm:2180) chicken.irregex#make-initial-memory" 
o|substituted constant variable: a19905 
o|inlining procedure: k19915 
o|inlining procedure: k19915 
o|substituted constant variable: a19929 
o|inlining procedure: "(irregex-core.scm:2176) chicken.irregex#dfa-init-state" 
o|inlining procedure: "(irregex-core.scm:2175) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:2174) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:2173) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:2172) chicken.irregex#chunker-get-str" 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: k20584 
o|inlining procedure: k20584 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: k20621 
o|contracted procedure: "(irregex-core.scm:2423) chicken.irregex#nfa-set-epsilons!" 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: k20621 
o|contracted procedure: "(irregex-core.scm:2480) chicken.irregex#nfa-set-state-trans!" 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|substituted constant variable: a20802 
o|inlining procedure: k20806 
o|inlining procedure: k20806 
o|substituted constant variable: a20820 
o|substituted constant variable: a20826 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: k20842 
o|inlining procedure: k20842 
o|inlining procedure: k20862 
o|inlining procedure: k20862 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k20890 
o|inlining procedure: k20890 
o|inlining procedure: k20908 
o|inlining procedure: k20908 
o|inlining procedure: k20923 
o|substituted constant variable: a20943 
o|inlining procedure: k20947 
o|inlining procedure: k20947 
o|inlining procedure: k20923 
o|inlining procedure: k20993 
o|inlining procedure: k21013 
o|inlining procedure: k21013 
o|inlining procedure: k20993 
o|inlining procedure: k21038 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k21038 
o|inlining procedure: k21101 
o|substituted constant variable: a21140 
o|inlining procedure: k21134 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|propagated global variable: r2113532478 chicken.irregex#~case-insensitive? 
o|inlining procedure: k21134 
o|substituted constant variable: chicken.irregex#~utf8? 
o|propagated global variable: r2113532480 chicken.irregex#~utf8? 
o|substituted constant variable: a21145 
o|inlining procedure: k21101 
o|consed rest parameter at call site: "(irregex-core.scm:2547) chicken.irregex#sre->cset" 2 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k21175 
o|inlining procedure: k21187 
o|inlining procedure: k21187 
o|inlining procedure: k21175 
o|inlining procedure: k21276 
o|inlining procedure: k21276 
o|inlining procedure: k21298 
o|inlining procedure: k21319 
o|inlining procedure: k21319 
o|inlining procedure: k21298 
o|inlining procedure: k21370 
o|inlining procedure: k21370 
o|inlining procedure: k21430 
o|inlining procedure: k21460 
o|inlining procedure: k21460 
o|inlining procedure: k21430 
o|inlining procedure: k21516 
o|inlining procedure: k21516 
o|substituted constant variable: a21563 
o|substituted constant variable: a21565 
o|substituted constant variable: a21570 
o|substituted constant variable: a21572 
o|substituted constant variable: a21577 
o|substituted constant variable: a21579 
o|substituted constant variable: a21581 
o|substituted constant variable: a21583 
o|inlining procedure: k21587 
o|inlining procedure: k21587 
o|substituted constant variable: a21600 
o|substituted constant variable: a21602 
o|substituted constant variable: a21604 
o|substituted constant variable: a21606 
o|inlining procedure: k21610 
o|inlining procedure: k21610 
o|substituted constant variable: a21623 
o|substituted constant variable: a21625 
o|substituted constant variable: a21627 
o|substituted constant variable: a21629 
o|substituted constant variable: a21634 
o|substituted constant variable: a21636 
o|inlining procedure: k21661 
o|substituted constant variable: a21675 
o|inlining procedure: k21679 
o|inlining procedure: k21679 
o|substituted constant variable: a21693 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: k21661 
o|inlining procedure: k21718 
o|inlining procedure: k21718 
o|inlining procedure: k21743 
o|inlining procedure: k21743 
o|inlining procedure: k21792 
o|inlining procedure: k21792 
o|substituted constant variable: a21805 
o|substituted constant variable: a21807 
o|substituted constant variable: a21809 
o|substituted constant variable: a21811 
o|substituted constant variable: chicken.irregex#*nfa-presize* 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|substituted constant variable: chicken.irregex#*nfa-presize* 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|inlining procedure: "(irregex-core.scm:2738) chicken.irregex#nfa-num-tags" 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|inlining procedure: k21958 
o|inlining procedure: k21958 
o|inlining procedure: "(irregex-core.scm:2771) chicken.irregex#mst-hash-set!" 
o|inlining procedure: k21999 
o|inlining procedure: k21999 
o|inlining procedure: "(irregex-core.scm:2761) chicken.irregex#mst-mappings-summary" 
o|inlining procedure: "(irregex-core.scm:2759) chicken.irregex#mst-num-states-set!" 
o|inlining procedure: "(irregex-core.scm:2759) chicken.irregex#mst-num-states" 
o|inlining procedure: "(irregex-core.scm:2756) chicken.irregex#mst-hash" 
o|inlining procedure: k22298 
o|inlining procedure: k22298 
o|substituted constant variable: a22312 
o|inlining procedure: k22314 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|inlining procedure: k22314 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|merged explicitly consed rest parameter: o4198 
o|inlining procedure: k22403 
o|contracted procedure: "(irregex-core.scm:2889) chicken.irregex#dfa-renumber" 
o|inlining procedure: k22590 
o|inlining procedure: k22590 
o|substituted constant variable: a22610 
o|contracted procedure: "(irregex-core.scm:2941) chicken.irregex#maybe-cset->char" 
o|inlining procedure: k26285 
o|inlining procedure: k26285 
o|substituted constant variable: a26324 
o|inlining procedure: k22666 
o|inlining procedure: k22666 
o|inlining procedure: k22704 
o|inlining procedure: k22704 
o|inlining procedure: "(irregex-core.scm:2888) chicken.irregex#nfa-highest-map-index" 
o|inlining procedure: k22403 
o|inlining procedure: k22435 
o|inlining procedure: k22435 
o|inlining procedure: k22459 
o|inlining procedure: k22459 
o|contracted procedure: "(irregex-core.scm:2906) chicken.irregex#find-reorder-commands" 
o|inlining procedure: k23382 
o|inlining procedure: k23382 
o|contracted procedure: "(irregex-core.scm:3113) chicken.irregex#nfa-set-reorder-commands!" 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: "(irregex-core.scm:2430) chicken.irregex#mst-hash" 
o|contracted procedure: "(irregex-core.scm:3112) chicken.irregex#find-reorder-commands-internal" 
o|inlining procedure: k23238 
o|inlining procedure: k23238 
o|inlining procedure: k23265 
o|inlining procedure: k23265 
o|inlining procedure: k23292 
o|inlining procedure: k23292 
o|inlining procedure: k23343 
o|inlining procedure: k23343 
o|inlining procedure: "(irregex-core.scm:3091) chicken.irregex#mst-mappings-summary" 
o|contracted procedure: "(irregex-core.scm:3089) chicken.irregex#mst-same-states?" 
o|inlining procedure: k22215 
o|inlining procedure: k22239 
o|inlining procedure: k22239 
o|substituted constant variable: a22253 
o|substituted constant variable: chicken.irregex#*mst-first-state-index* 
o|inlining procedure: "(irregex-core.scm:2809) chicken.irregex#mst-num-states" 
o|inlining procedure: "(irregex-core.scm:2809) chicken.irregex#mst-num-states" 
o|inlining procedure: k22215 
o|inlining procedure: "(irregex-core.scm:2808) chicken.irregex#mst-hash" 
o|inlining procedure: "(irregex-core.scm:2808) chicken.irregex#mst-hash" 
o|inlining procedure: "(irregex-core.scm:3085) chicken.irregex#mst-mappings-summary" 
o|inlining procedure: "(irregex-core.scm:3084) chicken.irregex#nfa-num-tags" 
o|contracted procedure: "(irregex-core.scm:3111) chicken.irregex#nfa-get-reorder-commands" 
o|inlining procedure: k20648 
o|inlining procedure: k20648 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: "(irregex-core.scm:2426) chicken.irregex#mst-hash" 
o|contracted procedure: "(irregex-core.scm:2896) chicken.irregex#get-distinct-transitions" 
o|inlining procedure: k22761 
o|inlining procedure: k22761 
o|inlining procedure: k22779 
o|inlining procedure: k22779 
o|inlining procedure: k22838 
o|inlining procedure: k22838 
o|inlining procedure: k22815 
o|inlining procedure: k22815 
o|contracted procedure: "(irregex-core.scm:2974) csets-intersect?4286" 
o|inlining procedure: k22742 
o|inlining procedure: k22742 
o|contracted procedure: "(irregex-core.scm:2957) chicken.irregex#nfa-get-state-trans" 
o|inlining procedure: k20558 
o|inlining procedure: k20558 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|contracted procedure: "(irregex-core.scm:2873) chicken.irregex#nfa-start-state" 
o|substituted constant variable: a20536 
o|inlining procedure: k23059 
o|inlining procedure: k23059 
o|contracted procedure: "(irregex-core.scm:3040) chicken.irregex#nfa-add-closure!" 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: "(irregex-core.scm:2438) chicken.irregex#mst-hash" 
o|contracted procedure: "(irregex-core.scm:3039) chicken.irregex#nfa-epsilon-closure-internal" 
o|inlining procedure: k22938 
o|inlining procedure: k22938 
o|inlining procedure: k22961 
o|inlining procedure: k22961 
o|contracted procedure: "(irregex-core.scm:3028) chicken.irregex#mst-add-tagged!" 
o|inlining procedure: "(irregex-core.scm:2803) chicken.irregex#mst-hash-set!" 
o|inlining procedure: "(irregex-core.scm:2798) chicken.irregex#mst-num-states-set!" 
o|inlining procedure: "(irregex-core.scm:2798) chicken.irregex#mst-num-states" 
o|inlining procedure: "(irregex-core.scm:2795) chicken.irregex#mst-hash" 
o|inlining procedure: k22165 
o|inlining procedure: k22165 
o|inlining procedure: "(irregex-core.scm:2787) chicken.irregex#mst-mappings-summary" 
o|contracted procedure: "(irregex-core.scm:3026) chicken.irregex#next-index-for-tag!" 
o|inlining procedure: k22335 
o|contracted procedure: "(irregex-core.scm:2835) chicken.irregex#nfa-set-highest-map-index!" 
o|inlining procedure: k22335 
o|inlining procedure: k22347 
o|inlining procedure: k22347 
o|inlining procedure: "(irregex-core.scm:2831) chicken.irregex#mst-mappings-summary" 
o|inlining procedure: "(irregex-core.scm:2830) chicken.irregex#nfa-highest-map-index" 
o|inlining procedure: k22985 
o|inlining procedure: k22985 
o|contracted procedure: "(irregex-core.scm:3033) chicken.irregex#mst-add/fast!" 
o|inlining procedure: "(irregex-core.scm:2780) chicken.irregex#mst-num-states-set!" 
o|inlining procedure: "(irregex-core.scm:2780) chicken.irregex#mst-num-states" 
o|inlining procedure: "(irregex-core.scm:2777) chicken.irregex#mst-hash-set!" 
o|inlining procedure: "(irregex-core.scm:2778) chicken.irregex#mst-hash" 
o|contracted procedure: "(irregex-core.scm:3038) chicken.irregex#nfa-get-closure" 
o|inlining procedure: k20705 
o|inlining procedure: k20705 
o|substituted constant variable: chicken.irregex#*nfa-num-fields* 
o|inlining procedure: "(irregex-core.scm:2434) chicken.irregex#mst-hash" 
o|inlining procedure: k23086 
o|inlining procedure: k23086 
o|inlining procedure: k23106 
o|inlining procedure: k23106 
o|inlining procedure: k23148 
o|inlining procedure: k23170 
o|inlining procedure: k23170 
o|inlining procedure: k23194 
o|inlining procedure: k23194 
o|inlining procedure: k23148 
o|inlining procedure: "(irregex-core.scm:3048) chicken.irregex#mst-mappings-summary" 
o|inlining procedure: "(irregex-core.scm:3047) chicken.irregex#mst-mappings-summary" 
o|inlining procedure: "(irregex-core.scm:3046) chicken.irregex#nfa-num-tags" 
o|inlining procedure: k26172 
o|substituted constant variable: a26189 
o|inlining procedure: k26172 
o|inlining procedure: k26224 
o|substituted constant variable: a26235 
o|inlining procedure: k26224 
o|inlining procedure: "(irregex-core.scm:3657) chicken.irregex#chunker-get-end" 
o|inlining procedure: "(irregex-core.scm:3656) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3655) chicken.irregex#chunker-get-str" 
o|inlining procedure: "(irregex-core.scm:3653) chicken.irregex#chunker-get-next" 
o|removed side-effect free assignment to unused variable: chicken.irregex#cset-size 
o|substituted constant variable: a26391 
o|inlining procedure: k26395 
o|inlining procedure: k26395 
o|inlining procedure: "(irregex-core.scm:3702) chicken.irregex#make-cset" 
o|merged explicitly consed rest parameter: o5232 
o|inlining procedure: k26510 
o|inlining procedure: k26522 
o|inlining procedure: k26522 
o|inlining procedure: k26540 
o|inlining procedure: k26573 
o|inlining procedure: k26573 
o|inlining procedure: k26540 
o|inlining procedure: k26637 
o|inlining procedure: k26637 
o|inlining procedure: k26672 
o|inlining procedure: k26707 
o|inlining procedure: k26707 
o|inlining procedure: k26672 
o|inlining procedure: k26751 
o|inlining procedure: k26751 
o|contracted procedure: "(irregex-core.scm:3725) chicken.irregex#plist->cset" 
o|inlining procedure: k26445 
o|inlining procedure: k26445 
o|inlining procedure: "(irregex-core.scm:3694) chicken.irregex#make-cset" 
o|contracted procedure: "(irregex-core.scm:3725) chicken.irregex#sre-flatten-ranges" 
o|inlining procedure: k18372 
o|inlining procedure: k18372 
o|inlining procedure: k26763 
o|inlining procedure: k26792 
o|inlining procedure: k26792 
o|inlining procedure: k26763 
o|inlining procedure: k26842 
o|inlining procedure: k26842 
o|substituted constant variable: a26861 
o|substituted constant variable: a26863 
o|substituted constant variable: a26865 
o|substituted constant variable: a26867 
o|substituted constant variable: a26869 
o|substituted constant variable: a26871 
o|substituted constant variable: a26873 
o|inlining procedure: k26510 
o|inlining procedure: k26882 
o|inlining procedure: k26882 
o|inlining procedure: k26895 
o|inlining procedure: k26895 
o|inlining procedure: k26964 
o|inlining procedure: k26964 
o|inlining procedure: k26979 
o|inlining procedure: k26979 
o|substituted constant variable: a27007 
o|inlining procedure: k27000 
o|inlining procedure: k27000 
o|inlining procedure: k27023 
o|inlining procedure: k27023 
o|substituted constant variable: a27039 
o|substituted constant variable: a27041 
o|inlining procedure: k27054 
o|inlining procedure: k27054 
o|inlining procedure: k27096 
o|inlining procedure: k27096 
o|substituted constant variable: a27152 
o|inlining procedure: k27144 
o|inlining procedure: k27144 
o|substituted constant variable: a27178 
o|inlining procedure: k27188 
o|inlining procedure: k27188 
o|inlining procedure: k27291 
o|inlining procedure: k27291 
o|inlining procedure: k27327 
o|inlining procedure: k27327 
o|inlining procedure: k27353 
o|inlining procedure: k27353 
o|inlining procedure: k27378 
o|inlining procedure: k27378 
o|contracted procedure: "(irregex-core.scm:3831) chicken.irregex#next-char" 
o|substituted constant variable: a27261 
o|contracted procedure: "(irregex-core.scm:3829) chicken.irregex#prev-char" 
o|substituted constant variable: a27273 
o|inlining procedure: k27487 
o|inlining procedure: k27487 
o|inlining procedure: k27520 
o|inlining procedure: k27520 
o|contracted procedure: "(irregex-core.scm:3858) chicken.irregex#min-char" 
o|inlining procedure: k27455 
o|inlining procedure: k27455 
o|contracted procedure: "(irregex-core.scm:3857) chicken.irregex#max-char" 
o|inlining procedure: k27464 
o|inlining procedure: k27464 
o|consed rest parameter at call site: "(irregex-core.scm:3866) chicken.irregex#sre->cset" 2 
o|inlining procedure: k27619 
o|inlining procedure: k27619 
o|inlining procedure: k27782 
o|inlining procedure: k27782 
o|inlining procedure: k27832 
o|inlining procedure: k27832 
o|substituted constant variable: chicken.irregex#~consumer? 
o|contracted procedure: "(irregex-core.scm:3916) g55135514" 
o|inlining procedure: "(irregex-core.scm:3903) chicken.irregex#irregex-match-chunker-set!" 
o|propagated global variable: str74632877 chicken.irregex#irregex-basic-string-chunker 
o|inlining procedure: k27877 
o|inlining procedure: k27877 
o|inlining procedure: k27891 
o|inlining procedure: k27891 
o|inlining procedure: k27910 
o|inlining procedure: k27910 
o|inlining procedure: k28016 
o|inlining procedure: k28016 
o|inlining procedure: k28060 
o|inlining procedure: "(irregex-core.scm:3955) chicken.irregex#chunker-get-start" 
o|inlining procedure: "(irregex-core.scm:3954) chicken.irregex#chunker-get-next" 
o|inlining procedure: "(irregex-core.scm:3953) chicken.irregex#chunker-get-end" 
o|inlining procedure: k28060 
o|substituted constant variable: chicken.irregex#~consumer? 
o|contracted procedure: "(irregex-core.scm:3950) g55655566" 
o|inlining procedure: "(irregex-core.scm:3943) chicken.irregex#irregex-match-chunker-set!" 
o|inlining procedure: "(irregex-core.scm:3940) chicken.irregex#chunker-get-start" 
o|inlining procedure: k28202 
o|contracted procedure: "(irregex-core.scm:3975) g55975598" 
o|inlining procedure: k28202 
o|inlining procedure: k28304 
o|inlining procedure: k28304 
o|contracted procedure: "(irregex-core.scm:3984) g56105611" 
o|inlining procedure: k28342 
o|inlining procedure: k28342 
o|inlining procedure: k28374 
o|inlining procedure: k28374 
o|inlining procedure: k28400 
o|inlining procedure: k28400 
o|inlining procedure: k28405 
o|inlining procedure: k28405 
o|inlining procedure: k28432 
o|contracted procedure: "(irregex-core.scm:4013) g56435644" 
o|contracted procedure: "(irregex-core.scm:4011) g56395640" 
o|inlining procedure: k28432 
o|contracted procedure: "(irregex-core.scm:4016) g56485649" 
o|inlining procedure: k28543 
o|inlining procedure: k28543 
o|substituted constant variable: a28578 
o|substituted constant variable: a28580 
o|inlining procedure: k28649 
o|inlining procedure: k28649 
o|contracted procedure: "(irregex-core.scm:4052) g56905691" 
o|contracted procedure: "(irregex-core.scm:4049) g56865687" 
o|inlining procedure: k28717 
o|inlining procedure: k28717 
o|inlining procedure: k28735 
o|inlining procedure: k28735 
o|inlining procedure: k28845 
o|inlining procedure: k28845 
o|substituted constant variable: a28862 
o|inlining procedure: k28889 
o|inlining procedure: k28889 
o|substituted constant variable: chicken.irregex#rx-special-chars 
o|inlining procedure: k29052 
o|inlining procedure: k29052 
o|inlining procedure: k29074 
o|inlining procedure: k29086 
o|contracted procedure: "(irregex-utils.scm:47) make-alt5939" 
o|inlining procedure: k28935 
o|inlining procedure: k28935 
o|inlining procedure: k29086 
o|substituted constant variable: a29116 
o|inlining procedure: k29118 
o|inlining procedure: k29118 
o|contracted procedure: "(irregex-utils.scm:55) make-seq5940" 
o|inlining procedure: k28965 
o|inlining procedure: k28965 
o|inlining procedure: k29002 
o|contracted procedure: "(irregex-utils.scm:36) g59635972" 
o|inlining procedure: k28987 
o|inlining procedure: k28987 
o|inlining procedure: k29002 
o|inlining procedure: k29042 
o|inlining procedure: k29042 
o|inlining procedure: k29074 
o|substituted constant variable: a29190 
o|inlining procedure: k29211 
o|inlining procedure: k29211 
o|inlining procedure: k29291 
o|inlining procedure: k29307 
o|inlining procedure: k29329 
o|inlining procedure: k29329 
o|inlining procedure: k29307 
o|inlining procedure: k29359 
o|inlining procedure: k29359 
o|inlining procedure: k29382 
o|inlining procedure: k29382 
o|inlining procedure: k29401 
o|inlining procedure: k29423 
o|inlining procedure: k29423 
o|inlining procedure: k29401 
o|inlining procedure: k29473 
o|inlining procedure: k29473 
o|inlining procedure: k29497 
o|inlining procedure: k29532 
o|inlining procedure: k29532 
o|inlining procedure: k29497 
o|inlining procedure: k29592 
o|inlining procedure: k29592 
o|inlining procedure: k29623 
o|inlining procedure: k29623 
o|inlining procedure: k29690 
o|inlining procedure: k29722 
o|inlining procedure: k29722 
o|substituted constant variable: a29732 
o|consed rest parameter at call site: "(irregex-utils.scm:130) chicken.irregex#sre->cset" 2 
o|inlining procedure: k29746 
o|inlining procedure: k29746 
o|substituted constant variable: a29769 
o|inlining procedure: k29690 
o|inlining procedure: k29803 
o|inlining procedure: k29803 
o|inlining procedure: k29831 
o|inlining procedure: k29831 
o|substituted constant variable: a29859 
o|substituted constant variable: a29861 
o|inlining procedure: k29865 
o|inlining procedure: k29865 
o|substituted constant variable: a29878 
o|substituted constant variable: a29880 
o|substituted constant variable: a29882 
o|substituted constant variable: a29884 
o|substituted constant variable: a29886 
o|substituted constant variable: a29888 
o|inlining procedure: k29892 
o|inlining procedure: k29892 
o|inlining procedure: k29904 
o|inlining procedure: k29904 
o|substituted constant variable: a29911 
o|substituted constant variable: a29913 
o|substituted constant variable: a29915 
o|substituted constant variable: a29917 
o|substituted constant variable: a29919 
o|substituted constant variable: a29921 
o|substituted constant variable: a29923 
o|substituted constant variable: a29925 
o|substituted constant variable: a29930 
o|substituted constant variable: a29932 
o|inlining procedure: k29291 
o|inlining procedure: k29939 
o|inlining procedure: k29939 
o|inlining procedure: k29963 
o|inlining procedure: k29963 
o|substituted constant variable: a29982 
o|substituted constant variable: a29984 
o|substituted constant variable: a29989 
o|substituted constant variable: a29991 
o|substituted constant variable: a29996 
o|substituted constant variable: a29998 
o|inlining procedure: k29999 
o|inlining procedure: k29999 
o|inlining procedure: k30052 
o|inlining procedure: k30052 
o|inlining procedure: k30068 
o|inlining procedure: k30068 
o|substituted constant variable: a30141 
o|inlining procedure: k30137 
o|inlining procedure: k30137 
o|substituted constant variable: a30153 
o|inlining procedure: k30160 
o|inlining procedure: k30160 
o|substituted constant variable: a30201 
o|inlining procedure: k30202 
o|inlining procedure: k30202 
o|inlining procedure: k30277 
o|inlining procedure: k30277 
o|substituted constant variable: a30312 
o|inlining procedure: k30987 
o|inlining procedure: k30987 
o|simplifications: ((if . 8)) 
o|replaced variables: 7205 
o|removed binding forms: 699 
o|removed conditional forms: 3 
o|substituted constant variable: r765731006 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-chunker 
o|removed side-effect free assignment to unused variable: chicken.irregex#irregex-match-chunker-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#%irregex-match-fail 
o|removed side-effect free assignment to unused variable: chicken.irregex#%irregex-match-fail-set! 
o|substituted constant variable: r778431008 
o|substituted constant variable: r780931010 
o|substituted constant variable: r783831012 
o|substituted constant variable: r786731014 
o|substituted constant variable: r802431021 
o|substituted constant variable: r803231022 
o|substituted constant variable: r803231023 
o|substituted constant variable: r800231025 
o|substituted constant variable: r816231033 
o|substituted constant variable: r776831035 
o|substituted constant variable: r776831036 
o|substituted constant variable: r821031043 
o|inlining procedure: k8417 
o|substituted constant variable: r843031064 
o|inlining procedure: k8417 
o|substituted constant variable: r841831065 
o|substituted constant variable: r841831065 
o|folded constant expression: (scheme#not (quote #f)) 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-next 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-str 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-start 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-end 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-substring 
o|removed side-effect free assignment to unused variable: chicken.irregex#chunker-get-subchunk 
o|substituted constant variable: r864631082 
o|substituted constant variable: r861031083 
o|substituted constant variable: r867931103 
o|removed side-effect free assignment to unused variable: chicken.irregex#*allow-utf8-mode?* 
o|substituted constant variable: r883331109 
o|substituted constant variable: r916131115 
o|substituted constant variable: r920431119 
o|substituted constant variable: r929731128 
o|removed side-effect free assignment to unused variable: chicken.irregex#~none 
o|removed side-effect free assignment to unused variable: chicken.irregex#dot-token 
o|converted assignments to bindings: (collect1353) 
o|substituted constant variable: r1014731161 
o|substituted constant variable: r1014731161 
o|inlining procedure: k10146 
o|inlining procedure: k10146 
o|removed side-effect free assignment to unused variable: chicken.irregex#~save? 
o|removed side-effect free assignment to unused variable: chicken.irregex#~multi-line? 
o|removed side-effect free assignment to unused variable: chicken.irregex#~ignore-space? 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|inlining procedure: k10359 
o|inlining procedure: k10359 
o|substituted constant variable: chicken.irregex#~single-line? 
o|propagated global variable: r1036033104 chicken.irregex#~single-line? 
o|propagated global variable: r1036033104 chicken.irregex#~single-line? 
o|inlining procedure: k10359 
o|substituted constant variable: r1036031196 
o|substituted constant variable: r1036031196 
o|inlining procedure: k10359 
o|substituted constant variable: r1039631198 
o|substituted constant variable: r1422731217 
o|converted assignments to bindings: (shift1522) 
o|substituted constant variable: r1086531230 
o|substituted constant variable: r1174031288 
o|substituted constant variable: r1174031288 
o|substituted constant variable: r1174031290 
o|substituted constant variable: r1174031290 
o|substituted constant variable: r1215531354 
o|substituted constant variable: c1080 
o|converted assignments to bindings: (collect1090) 
o|substituted constant variable: r1326331381 
o|substituted constant variable: r1327531383 
o|substituted constant variable: r893331393 
o|substituted constant variable: r887631411 
o|substituted constant variable: c1056 
o|folded constant expression: (scheme#eq? (quote #\}) (quote #\\)) 
o|substituted constant variable: c1056 
o|converted assignments to bindings: (byte1955) 
o|substituted constant variable: r1426531421 
o|substituted constant variable: r1427731423 
o|substituted constant variable: r1550031440 
o|substituted constant variable: r1550031440 
o|substituted constant variable: r1430331448 
o|substituted constant variable: r1431531450 
o|substituted constant variable: r1536631458 
o|substituted constant variable: r1536631458 
o|folded constant expression: (scheme#integer->char (quote 127)) 
o|substituted constant variable: r1536631460 
o|substituted constant variable: r1536631460 
o|folded constant expression: (scheme#integer->char (quote 255)) 
o|substituted constant variable: r1452931464 
o|substituted constant variable: r1452931464 
o|substituted constant variable: r2408231524 
o|substituted constant variable: r2409031526 
o|substituted constant variable: r2410531528 
o|substituted constant variable: r2424431539 
o|substituted constant variable: r2425231541 
o|substituted constant variable: r2426731543 
o|substituted constant variable: r873931572 
o|substituted constant variable: r878131586 
o|substituted constant variable: r2568431756 
o|substituted constant variable: r2582731789 
o|converted assignments to bindings: (rec4438) 
o|substituted constant variable: chicken.irregex#~consumer? 
o|substituted constant variable: chicken.irregex#~consumer? 
o|substituted constant variable: r1623031864 
o|substituted constant variable: r1623031864 
o|substituted constant variable: chicken.irregex#~searcher? 
o|substituted constant variable: chicken.irregex#~searcher? 
o|substituted constant variable: r1623731868 
o|substituted constant variable: r1623731868 
o|substituted constant variable: r1710831873 
o|substituted constant variable: r1710831873 
o|inlining procedure: k17189 
o|substituted constant variable: r1719031882 
o|substituted constant variable: r1719031882 
o|inlining procedure: k17224 
o|substituted constant variable: r1722531887 
o|substituted constant variable: r1722531887 
o|inlining procedure: k17267 
o|substituted constant variable: r1726831893 
o|substituted constant variable: r1726831893 
o|substituted constant variable: r1727831897 
o|substituted constant variable: r1727831897 
o|inlining procedure: k17306 
o|substituted constant variable: r1730731902 
o|substituted constant variable: r1730731902 
o|inlining procedure: k17394 
o|substituted constant variable: r1740131908 
o|substituted constant variable: r1739531909 
o|substituted constant variable: r1739531909 
o|inlining procedure: k17575 
o|substituted constant variable: r1757631928 
o|substituted constant variable: r1757631928 
o|inlining procedure: k17706 
o|substituted constant variable: r1770731938 
o|substituted constant variable: r1770731938 
o|substituted constant variable: r1776531941 
o|substituted constant variable: r1778031943 
o|inlining procedure: k17851 
o|substituted constant variable: r1785231948 
o|substituted constant variable: r1785231948 
o|substituted constant variable: r1789231951 
o|substituted constant variable: r1789231951 
o|converted assignments to bindings: (grow2855) 
o|substituted constant variable: r1626131988 
o|substituted constant variable: r1629631990 
o|substituted constant variable: r1629631991 
o|substituted constant variable: r1581532006 
o|substituted constant variable: r1596432016 
o|substituted constant variable: r1602332021 
o|converted assignments to bindings: (rec2364) 
o|substituted constant variable: r1636032026 
o|substituted constant variable: r1650532042 
o|substituted constant variable: r1660232051 
o|substituted constant variable: r1659032052 
o|substituted constant variable: r1657832054 
o|substituted constant variable: r1663932057 
o|substituted constant variable: r1665432059 
o|substituted constant variable: r1675232070 
o|substituted constant variable: r1676732072 
o|substituted constant variable: r1686932082 
o|substituted constant variable: r1688432084 
o|substituted constant variable: r1694632092 
o|substituted constant variable: r1700832094 
o|substituted constant variable: r1700832094 
o|inlining procedure: k17007 
o|substituted constant variable: r1703432099 
o|substituted constant variable: r1819432103 
o|substituted constant variable: r1821732105 
o|substituted constant variable: r1895832153 
o|substituted constant variable: r1895832153 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1897232157 
o|substituted constant variable: r1897232157 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1898532160 
o|substituted constant variable: r1906832174 
o|substituted constant variable: r1915532179 
o|substituted constant variable: r1909332195 
o|substituted constant variable: r1969632197 
o|substituted constant variable: index3464 
o|substituted constant variable: index3464 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: index3464 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|inlining procedure: k19701 
o|inlining procedure: k19719 
o|inlining procedure: k19731 
o|substituted constant variable: r1974732200 
o|substituted constant variable: r1979832204 
o|substituted constant variable: r1968132211 
o|substituted constant variable: x75432240 
o|inlining procedure: k19185 
o|substituted constant variable: r1926732264 
o|substituted constant variable: r1934032282 
o|substituted constant variable: r1934032282 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1935432286 
o|substituted constant variable: r1935432286 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1936732289 
o|substituted constant variable: r1941632297 
o|substituted constant variable: n3398 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: x75432312 
o|substituted constant variable: r1953332322 
o|substituted constant variable: n3438 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n3432 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: r1952132338 
o|substituted constant variable: r1962332360 
o|substituted constant variable: r1962332361 
o|removed side-effect free assignment to unused variable: chicken.irregex#dfa-init-state 
o|removed side-effect free assignment to unused variable: chicken.irregex#dfa-cell-commands 
o|removed side-effect free assignment to unused variable: chicken.irregex#dfa-finalizer 
o|substituted constant variable: r1984632364 
o|substituted constant variable: r1984632364 
o|substituted constant variable: r1986232368 
o|substituted constant variable: r1986232368 
o|substituted constant variable: r2005132374 
o|substituted constant variable: r2005132375 
o|inlining procedure: k20056 
o|substituted constant variable: r2002532376 
o|substituted constant variable: r2031532398 
o|inlining procedure: k20320 
o|removed side-effect free assignment to unused variable: chicken.irregex#*nfa-presize* 
o|removed side-effect free assignment to unused variable: chicken.irregex#*nfa-num-fields* 
o|removed side-effect free assignment to unused variable: chicken.irregex#nfa-num-tags 
o|removed side-effect free assignment to unused variable: chicken.irregex#nfa-highest-map-index 
o|substituted constant variable: r2058532447 
o|substituted constant variable: r2084332454 
o|substituted constant variable: r2090932460 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: chicken.irregex#~case-insensitive? 
o|substituted constant variable: chicken.irregex#~utf8? 
o|substituted constant variable: chicken.irregex#~utf8? 
o|inlining procedure: k21282 
o|substituted constant variable: r2127732490 
o|inlining procedure: k21382 
o|substituted constant variable: r2137132498 
o|substituted constant variable: r2166232514 
o|removed side-effect free assignment to unused variable: chicken.irregex#*mst-first-state-index* 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-mappings-summary 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-num-states 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-num-states-set! 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-hash 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-hash-set! 
o|substituted constant variable: r2323932589 
o|substituted constant variable: r2221632617 
o|substituted constant variable: r2064932639 
o|substituted constant variable: r2274332654 
o|substituted constant variable: r2055932655 
o|substituted constant variable: r2070632738 
o|substituted constant variable: r2317132750 
o|substituted constant variable: r2319532752 
o|substituted constant variable: r2314932753 
o|removed side-effect free assignment to unused variable: chicken.irregex#make-cset 
o|removed side-effect free assignment to unused variable: chicken.irregex#cset=? 
o|converted assignments to bindings: (rec5240) 
o|substituted constant variable: r2696532833 
o|substituted constant variable: r2698032836 
o|substituted constant variable: r2702432840 
o|substituted constant variable: n5516 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: r2787832883 
o|substituted constant variable: r2787832883 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r2789232887 
o|substituted constant variable: r2789232887 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r2791132890 
o|substituted constant variable: n5568 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5600 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5613 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: r2840132933 
o|substituted constant variable: r2840132933 
o|substituted constant variable: n5646 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5642 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5651 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5693 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|substituted constant variable: n5689 
o|folded constant expression: (scheme#* (quote 0) (quote 4)) 
o|removed side-effect free assignment to unused variable: chicken.irregex#rx-special-chars 
o|substituted constant variable: r2884632949 
o|substituted constant variable: r2905332953 
o|substituted constant variable: r2938332983 
o|substituted constant variable: r2974733004 
o|substituted constant variable: r3005333023 
o|substituted constant variable: r3027833034 
o|simplifications: ((let . 8)) 
o|replaced variables: 1046 
o|removed binding forms: 6510 
o|removed conditional forms: 1 
o|inlining procedure: k8045 
o|inlining procedure: k8045 
o|inlining procedure: k8404 
o|substituted constant variable: r841833049 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r841833051 
o|folded constant expression: (scheme#not (quote #f)) 
o|removed side-effect free assignment to unused variable: chicken.irregex#~searcher? 
o|removed side-effect free assignment to unused variable: chicken.irregex#~consumer? 
o|substituted constant variable: r1014733096 
o|removed side-effect free assignment to unused variable: chicken.irregex#~case-insensitive? 
o|removed side-effect free assignment to unused variable: chicken.irregex#~utf8? 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|substituted constant variable: r1036033102 
o|substituted constant variable: chicken.irregex#~single-line? 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|substituted constant variable: chicken.irregex#~single-line? 
o|propagated global variable: b128733565 chicken.irregex#~single-line? 
o|propagated global variable: b128733565 chicken.irregex#~single-line? 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|substituted constant variable: r1036033106 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:734) chicken.irregex#flag-join" 
o|substituted constant variable: r1036033108 
o|inlining procedure: "(irregex-core.scm:756) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:761) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:770) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:814) chicken.irregex#flag-set?" 
o|inlining procedure: k10942 
o|inlining procedure: k10942 
o|inlining procedure: "(irregex-core.scm:849) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:891) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:908) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:910) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:913) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:922) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:925) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:932) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:938) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:953) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:958) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:961) chicken.irregex#flag-clear" 
o|inlining procedure: chicken.irregex#flag-clear 
o|inlining procedure: chicken.irregex#flag-join 
o|inlining procedure: "(irregex-core.scm:972) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:971) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1230) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1231) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1229) chicken.irregex#cset-empty?" 
o|inlining procedure: "(irregex-core.scm:1241) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1250) chicken.irregex#range->cset" 
o|inlining procedure: "(irregex-core.scm:1258) chicken.irregex#utf8-start-char->length" 
o|inlining procedure: "(irregex-core.scm:1278) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1289) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1294) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1300) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1297) chicken.irregex#utf8-start-char->length" 
o|inlining procedure: "(irregex-core.scm:1301) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1306) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:1305) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1222) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1103) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1146) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:451) a12934" 
o|inlining procedure: "(irregex-core.scm:1164) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1167) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1170) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1174) chicken.irregex#flag-set?" 
o|contracted procedure: k8890 
o|removed call to pure procedure with unused result: "(irregex-core.scm:1446) ##sys#slot" 
o|removed call to pure procedure with unused result: "(irregex-core.scm:1501) ##sys#slot" 
o|inlining procedure: k15128 
o|inlining procedure: k15128 
o|inlining procedure: k15128 
o|inlining procedure: k15128 
o|removed call to pure procedure with unused result: "(irregex-core.scm:1512) ##sys#slot" 
o|inlining procedure: "(irregex-core.scm:1509) chicken.irregex#utf8-start-char->length" 
o|removed call to pure procedure with unused result: "(irregex-core.scm:1492) ##sys#slot" 
o|inlining procedure: k15361 
o|inlining procedure: k15361 
o|removed call to pure procedure with unused result: "(irregex-core.scm:1521) ##sys#slot" 
o|inlining procedure: "(irregex-core.scm:1532) chicken.irregex#high-char?" 
o|inlining procedure: "(irregex-core.scm:1532) chicken.irregex#high-char?" 
o|inlining procedure: "(irregex-core.scm:3143) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:3148) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:3167) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:3172) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:3175) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:3177) chicken.irregex#flag-clear" 
o|inlining procedure: "(irregex-core.scm:3406) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:3587) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1632) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:1632) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:1633) chicken.irregex#flag-join" 
o|inlining procedure: "(irregex-core.scm:1633) chicken.irregex#flag-join" 
o|substituted constant variable: r1719033198 
o|substituted constant variable: r1722533200 
o|substituted constant variable: r1726833204 
o|substituted constant variable: r1730733206 
o|substituted constant variable: r1739533210 
o|substituted constant variable: r1757633212 
o|substituted constant variable: r1770733214 
o|substituted constant variable: r1785233216 
o|inlining procedure: "(irregex-core.scm:1627) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:1623) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:1574) chicken.irregex#high-char?" 
o|inlining procedure: "(irregex-core.scm:1541) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:1540) chicken.irregex#flag-set?" 
o|substituted constant variable: r1700833236 
o|inlining procedure: k18950 
o|inlining procedure: k18964 
o|substituted constant variable: r1972033264 
o|folded constant expression: (scheme#+ (quote 3) (quote 0)) 
o|substituted constant variable: r1973233266 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|inlining procedure: "(irregex-core.scm:2147) chicken.irregex#dfa-next-state" 
o|inlining procedure: "(irregex-core.scm:2126) chicken.irregex#dfa-next-state" 
o|inlining procedure: "(irregex-core.scm:1992) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:2031) chicken.irregex#flag-set?" 
o|inlining procedure: k19332 
o|inlining procedure: k19346 
o|contracted procedure: k19468 
o|contracted procedure: k19580 
o|contracted procedure: k19608 
o|substituted constant variable: r200513237433295 
o|substituted constant variable: r200513237533296 
o|inlining procedure: k20305 
o|inlining procedure: "(irregex-core.scm:2228) chicken.irregex#dfa-next-state" 
o|substituted constant variable: r203153239833306 
o|inlining procedure: "(irregex-core.scm:2182) chicken.irregex#dfa-next-state" 
o|inlining procedure: "(irregex-core.scm:2488) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:2488) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:2489) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:2487) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:2526) chicken.irregex#flag-set?" 
o|inlining procedure: "(irregex-core.scm:2548) chicken.irregex#flag-set?" 
o|inlining procedure: k21322 
o|inlining procedure: "(irregex-core.scm:2737) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:2771) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:2757) chicken.irregex#mst-state-mappings" 
o|inlining procedure: "(irregex-core.scm:2900) chicken.irregex#mst-state-mappings" 
o|inlining procedure: k23369 
o|inlining procedure: k23369 
o|inlining procedure: "(irregex-core.scm:2990) chicken.irregex#cset-empty?" 
o|inlining procedure: "(irregex-core.scm:2980) chicken.irregex#cset-empty?" 
o|inlining procedure: "(irregex-core.scm:2953) chicken.irregex#cset-empty?" 
o|inlining procedure: k22568 
o|inlining procedure: "(irregex-core.scm:2402) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:2803) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:2796) chicken.irregex#mst-state-mappings" 
o|inlining procedure: "(irregex-core.scm:2779) chicken.irregex#nfa-num-states" 
o|inlining procedure: k22044 
o|inlining procedure: "(irregex-core.scm:2776) chicken.irregex#mst-state-mappings" 
o|inlining procedure: "(irregex-core.scm:3007) chicken.irregex#nfa-num-states" 
o|inlining procedure: "(irregex-core.scm:3701) chicken.irregex#cset-adjoin" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: "(irregex-core.scm:3697) chicken.irregex#range->cset" 
o|inlining procedure: "(irregex-core.scm:3738) chicken.irregex#range->cset" 
o|inlining procedure: "(irregex-core.scm:3739) chicken.irregex#range->cset" 
o|inlining procedure: "(irregex-core.scm:3802) chicken.irregex#char->cset" 
o|inlining procedure: k27570 
o|inlining procedure: k27570 
o|inlining procedure: "(irregex-core.scm:3879) chicken.irregex#range->cset" 
o|inlining procedure: "(irregex-core.scm:3921) chicken.irregex#flag-set?" 
o|contracted procedure: k27817 
o|inlining procedure: k27870 
o|inlining procedure: k27884 
o|inlining procedure: k27898 
o|inlining procedure: k27898 
o|inlining procedure: k27925 
o|inlining procedure: "(irregex-core.scm:3959) chicken.irregex#flag-set?" 
o|contracted procedure: k28051 
o|contracted procedure: k28247 
o|contracted procedure: k28298 
o|contracted procedure: k28470 
o|contracted procedure: k28487 
o|contracted procedure: k28534 
o|contracted procedure: k28677 
o|contracted procedure: k28698 
o|inlining procedure: k28797 
o|inlining procedure: k29092 
o|inlining procedure: k29137 
o|inlining procedure: k29023 
o|replaced variables: 4 
o|removed binding forms: 1105 
o|substituted constant variable: r804633518 
o|substituted constant variable: r804633521 
o|removed side-effect free assignment to unused variable: chicken.irregex#flag-set? 
o|substituted constant variable: b128733557 
o|substituted constant variable: b128733557 
o|substituted constant variable: b128733561 
o|substituted constant variable: b128733561 
o|substituted constant variable: chicken.irregex#~single-line? 
o|substituted constant variable: chicken.irregex#~single-line? 
o|substituted constant variable: b128733569 
o|substituted constant variable: b128733569 
o|substituted constant variable: b128733573 
o|substituted constant variable: b128733573 
o|substituted constant variable: b128733577 
o|substituted constant variable: b128733577 
o|inlining procedure: k10354 
o|substituted constant variable: i128333583 
o|substituted constant variable: i128333583 
o|substituted constant variable: i128333591 
o|substituted constant variable: i128333591 
o|substituted constant variable: i128333605 
o|substituted constant variable: i128333605 
o|substituted constant variable: i128333611 
o|substituted constant variable: i128333611 
o|substituted constant variable: r1094333617 
o|substituted constant variable: r1094333621 
o|substituted constant variable: i128333627 
o|substituted constant variable: i128333627 
o|substituted constant variable: b128733633 
o|substituted constant variable: b128733633 
o|substituted constant variable: b129133637 
o|substituted constant variable: b129133643 
o|substituted constant variable: b129133649 
o|substituted constant variable: b129133655 
o|substituted constant variable: b129133661 
o|substituted constant variable: b129133667 
o|substituted constant variable: b129133673 
o|substituted constant variable: b129133679 
o|substituted constant variable: b129133685 
o|substituted constant variable: b129133691 
o|substituted constant variable: i128333707 
o|substituted constant variable: i128333707 
o|substituted constant variable: i128333713 
o|substituted constant variable: i128333713 
o|substituted constant variable: ch541533721 
o|substituted constant variable: ch541533721 
o|substituted constant variable: ch541533721 
o|substituted constant variable: i128333732 
o|substituted constant variable: i128333732 
o|substituted constant variable: ch541533765 
o|substituted constant variable: ch541533765 
o|substituted constant variable: ch541533765 
o|substituted constant variable: ch520033824 
o|substituted constant variable: ch520033824 
o|substituted constant variable: i128333830 
o|substituted constant variable: i128333830 
o|substituted constant variable: i128333838 
o|substituted constant variable: i128333838 
o|substituted constant variable: i128333846 
o|substituted constant variable: i128333846 
o|substituted constant variable: i128333854 
o|substituted constant variable: i128333854 
o|substituted constant variable: i128333867 
o|substituted constant variable: i128333867 
o|substituted constant variable: i128333873 
o|substituted constant variable: i128333873 
o|substituted constant variable: i128333879 
o|substituted constant variable: i128333879 
o|substituted constant variable: i128333885 
o|substituted constant variable: i128333885 
o|substituted constant variable: r8891 
o|removed side-effect free assignment to unused variable: chicken.irregex#utf8-start-char->length 
o|contracted procedure: k14707 
o|contracted procedure: k15109 
o|substituted constant variable: r1512933903 
o|folded constant expression: (scheme#integer->char (quote 127)) 
o|substituted constant variable: r1512933904 
o|folded constant expression: (scheme#integer->char (quote 223)) 
o|substituted constant variable: r1512933905 
o|folded constant expression: (scheme#integer->char (quote 239)) 
o|substituted constant variable: r1512933906 
o|folded constant expression: (scheme#integer->char (quote 247)) 
o|contracted procedure: k15212 
o|contracted procedure: k15350 
o|substituted constant variable: r1536233918 
o|substituted constant variable: r1536233919 
o|contracted procedure: k15441 
o|contracted procedure: k16206 
o|substituted constant variable: i128333938 
o|substituted constant variable: i128333938 
o|substituted constant variable: i128333944 
o|substituted constant variable: i128333944 
o|substituted constant variable: b129133950 
o|substituted constant variable: b128733956 
o|substituted constant variable: b128733956 
o|substituted constant variable: b128733960 
o|substituted constant variable: b128733960 
o|substituted constant variable: b129133964 
o|substituted constant variable: i128333972 
o|substituted constant variable: i128333972 
o|substituted constant variable: i128333982 
o|substituted constant variable: i128333982 
o|substituted constant variable: b128733990 
o|substituted constant variable: b128733990 
o|substituted constant variable: b128733994 
o|substituted constant variable: b128733994 
o|substituted constant variable: b128733998 
o|substituted constant variable: a128633997 
o|substituted constant variable: b128733998 
o|substituted constant variable: a128633997 
o|substituted constant variable: b128734002 
o|substituted constant variable: a128634001 
o|substituted constant variable: b128734002 
o|substituted constant variable: a128634001 
o|substituted constant variable: i128334039 
o|substituted constant variable: i128334039 
o|substituted constant variable: i128334045 
o|substituted constant variable: i128334045 
o|substituted constant variable: r1895134055 
o|substituted constant variable: r1896534056 
o|contracted procedure: k19715 
o|contracted procedure: k19727 
o|substituted constant variable: i128334079 
o|substituted constant variable: i128334079 
o|substituted constant variable: i128334087 
o|substituted constant variable: i128334087 
o|substituted constant variable: r1933334091 
o|substituted constant variable: r1934734092 
o|substituted constant variable: r19469 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|substituted constant variable: r19581 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|substituted constant variable: r19609 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|removed side-effect free assignment to unused variable: chicken.irregex#dfa-next-state 
o|substituted constant variable: r2030634103 
o|removed side-effect free assignment to unused variable: chicken.irregex#nfa-num-states 
o|substituted constant variable: i128334143 
o|substituted constant variable: i128334143 
o|substituted constant variable: i128334151 
o|substituted constant variable: i128334151 
o|substituted constant variable: i128334157 
o|substituted constant variable: i128334157 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-state-mappings 
o|substituted constant variable: state405134196 
o|folded constant expression: (scheme#+ (quote 0) (quote 3)) 
o|substituted constant variable: r2337034206 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r2337034207 
o|folded constant expression: (scheme#not (quote #f)) 
o|contracted procedure: k22993 
o|removed side-effect free assignment to unused variable: chicken.irregex#range->cset 
o|removed side-effect free assignment to unused variable: chicken.irregex#char->cset 
o|removed side-effect free assignment to unused variable: chicken.irregex#cset-empty? 
o|removed side-effect free assignment to unused variable: chicken.irregex#cset-adjoin 
o|substituted constant variable: i128334351 
o|substituted constant variable: i128334351 
o|substituted constant variable: r27818 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r2787134357 
o|substituted constant variable: r2788534358 
o|substituted constant variable: r2789934359 
o|substituted constant variable: r2789934360 
o|substituted constant variable: r2792634361 
o|substituted constant variable: i128334364 
o|substituted constant variable: i128334364 
o|substituted constant variable: r28052 
o|folded constant expression: (scheme#+ (quote 2) (quote 0)) 
o|substituted constant variable: r28248 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r28299 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r28471 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r28488 
o|substituted constant variable: r28535 
o|substituted constant variable: r28678 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r28699 
o|folded constant expression: (scheme#+ (quote 1) (quote 0)) 
o|substituted constant variable: r2879834374 
o|replaced variables: 273 
o|removed binding forms: 66 
o|removed conditional forms: 27 
o|removed side-effect free assignment to unused variable: chicken.irregex#~single-line? 
o|inlining procedure: k15124 
o|inlining procedure: k15124 
o|inlining procedure: k15124 
o|inlining procedure: k15124 
o|inlining procedure: k16249 
o|inlining procedure: k16279 
o|substituted constant variable: r19716 
o|substituted constant variable: r19728 
o|contracted procedure: k19464 
o|contracted procedure: k19576 
o|contracted procedure: k19604 
o|inlining procedure: k21904 
o|inlining procedure: k21993 
o|contracted procedure: k2186634199 
o|inlining procedure: k20533 
o|inlining procedure: k22119 
o|inlining procedure: k22076 
o|inlining procedure: k23040 
o|contracted procedure: k27813 
o|contracted procedure: k28047 
o|contracted procedure: k28243 
o|contracted procedure: k28294 
o|contracted procedure: k28466 
o|contracted procedure: k28673 
o|contracted procedure: k28694 
o|replaced variables: 4 
o|removed binding forms: 358 
o|substituted constant variable: r1512534719 
o|substituted constant variable: r1512534720 
o|substituted constant variable: r1512534721 
o|substituted constant variable: r1512534722 
o|substituted constant variable: r19465 
o|substituted constant variable: r19577 
o|substituted constant variable: r19605 
o|substituted constant variable: r2186734197 
o|substituted constant variable: r27814 
o|substituted constant variable: r28048 
o|substituted constant variable: r28244 
o|substituted constant variable: r28295 
o|substituted constant variable: r28467 
o|substituted constant variable: r28674 
o|substituted constant variable: r28695 
o|removed binding forms: 14 
o|removed binding forms: 15 
o|simplifications: ((let . 234) (if . 172) (##core#call . 2979)) 
o|  call simplifications:
o|    ##sys#check-string
o|    chicken.fixnum#fx>	2
o|    scheme#write-char
o|    scheme#list->string	2
o|    scheme#char>?	5
o|    scheme#char<=?	2
o|    scheme#char>=?	3
o|    scheme#quotient
o|    scheme#assoc	4
o|    scheme#modulo	3
o|    scheme#memv	4
o|    scheme#char?	11
o|    scheme#char=?	7
o|    scheme#cdddr	9
o|    scheme#cadddr	2
o|    scheme#symbol?	7
o|    scheme#caddr	14
o|    scheme#vector-set!	63
o|    chicken.fixnum#fx+	2
o|    chicken.fixnum#fxmod
o|    ##sys#setislot
o|    scheme#length	6
o|    ##sys#check-list	32
o|    chicken.fixnum#fxshr	6
o|    chicken.fixnum#fxshl	6
o|    chicken.fixnum#fx<=	8
o|    scheme#integer->char	27
o|    scheme#char-upper-case?
o|    scheme#char-upcase
o|    scheme#char-downcase
o|    scheme#equal?	9
o|    scheme#assv	4
o|    scheme#char<?	7
o|    scheme#string	10
o|    ##sys#cons	117
o|    scheme#cddr	31
o|    scheme#cadr	64
o|    scheme#>	11
o|    scheme#char->integer	19
o|    chicken.fixnum#fx<
o|    chicken.fixnum#fx>=
o|    scheme#string->list	7
o|    ##sys#setslot	34
o|    ##sys#list	179
o|    scheme#string?	23
o|    scheme#char-whitespace?	2
o|    chicken.fixnum#fxnot	14
o|    chicken.fixnum#fxand	58
o|    chicken.fixnum#fxior	21
o|    scheme#car	78
o|    scheme#<=	5
o|    scheme#char-alphabetic?	7
o|    scheme#char-numeric?	5
o|    scheme#=	66
o|    scheme#-	20
o|    scheme#string-ref	68
o|    scheme#string-length	24
o|    scheme#list	49
o|    scheme#cons	223
o|    scheme#procedure?	10
o|    scheme#vector	26
o|    scheme#vector-ref	170
o|    scheme#assq	13
o|    scheme#not	100
o|    scheme#cdr	60
o|    scheme#pair?	124
o|    scheme#number?	13
o|    scheme#null?	65
o|    scheme#caar	26
o|    scheme#eq?	433
o|    scheme#cdar	15
o|    scheme#exact?	8
o|    scheme#>=	27
o|    scheme#<	21
o|    scheme#apply	16
o|    scheme#*	52
o|    scheme#+	148
o|    ##sys#size	8
o|    chicken.fixnum#fx/
o|    ##sys#make-structure	4
o|    ##sys#check-structure	6
o|    ##sys#slot	220
o|    ##sys#structure?	3
o|    scheme#vector-length	20
o|contracted procedure: k7549 
o|contracted procedure: k7597 
o|contracted procedure: k7615 
o|contracted procedure: k7632 
o|contracted procedure: k7647 
o|contracted procedure: k7659 
o|contracted procedure: k7670 
o|contracted procedure: k7674 
o|contracted procedure: k7678 
o|contracted procedure: k7682 
o|contracted procedure: k7694 
o|contracted procedure: k7707 
o|contracted procedure: k7703 
o|contracted procedure: k7698 
o|contracted procedure: k7719 
o|contracted procedure: k7738 
o|contracted procedure: k7746 
o|contracted procedure: k7742 
o|contracted procedure: k7792 
o|contracted procedure: k7796 
o|contracted procedure: k7817 
o|contracted procedure: k7825 
o|contracted procedure: k7821 
o|contracted procedure: k7846 
o|contracted procedure: k7854 
o|contracted procedure: k7850 
o|contracted procedure: k7959 
o|contracted procedure: k8130 
o|contracted procedure: k7968 
o|contracted procedure: k8126 
o|contracted procedure: k7974 
o|contracted procedure: k7984 
o|contracted procedure: k7990 
o|contracted procedure: k8028 
o|contracted procedure: k8042 
o|contracted procedure: k8038 
o|contracted procedure: k8034 
o|contracted procedure: k7998 
o|contracted procedure: k8004 
o|substituted constant variable: g35658 
o|contracted procedure: k8011 
o|contracted procedure: k8016 
o|contracted procedure: k8071 
o|contracted procedure: k8122 
o|contracted procedure: k8083 
o|contracted procedure: k8095 
o|contracted procedure: k8099 
o|contracted procedure: k8102 
o|contracted procedure: k8184 
o|contracted procedure: k8150 
o|contracted procedure: k8158 
o|contracted procedure: k8164 
o|substituted constant variable: g35664 
o|contracted procedure: k8171 
o|contracted procedure: k8176 
o|contracted procedure: k7770 
o|contracted procedure: k8190 
o|contracted procedure: k8206 
o|contracted procedure: k8212 
o|contracted procedure: k8225 
o|contracted procedure: k8229 
o|contracted procedure: k8232 
o|contracted procedure: k8242 
o|contracted procedure: k8250 
o|contracted procedure: k8246 
o|contracted procedure: k8253 
o|contracted procedure: k8263 
o|contracted procedure: k8271 
o|contracted procedure: k8267 
o|contracted procedure: k8274 
o|contracted procedure: k8287 
o|contracted procedure: k8290 
o|contracted procedure: k8296 
o|contracted procedure: k8315 
o|contracted procedure: k8319 
o|contracted procedure: k8322 
o|contracted procedure: k8332 
o|contracted procedure: k8340 
o|contracted procedure: k8336 
o|contracted procedure: k8343 
o|contracted procedure: k8353 
o|contracted procedure: k8361 
o|contracted procedure: k8357 
o|contracted procedure: k8364 
o|contracted procedure: k8566 
o|contracted procedure: k8374 
o|contracted procedure: k8377 
o|contracted procedure: k8560 
o|contracted procedure: k8380 
o|contracted procedure: k8556 
o|contracted procedure: k8383 
o|contracted procedure: k8386 
o|contracted procedure: k8543 
o|contracted procedure: k8389 
o|contracted procedure: k8539 
o|contracted procedure: k8392 
o|contracted procedure: k8395 
o|contracted procedure: k8451 
o|contracted procedure: k8469 
o|contracted procedure: k8478 
o|contracted procedure: k8485 
o|contracted procedure: k8508 
o|contracted procedure: k8442 
o|contracted procedure: k8398 
o|contracted procedure: k8438 
o|contracted procedure: k8401 
o|contracted procedure: k8420 
o|contracted procedure: k8426 
o|contracted procedure: k8432 
o|contracted procedure: k8417 
o|contracted procedure: k8652 
o|contracted procedure: k8663 
o|contracted procedure: k8659 
o|contracted procedure: k8612 
o|contracted procedure: k8615 
o|contracted procedure: k8630 
o|contracted procedure: k8672 
o|contracted procedure: k8685 
o|contracted procedure: k8692 
o|contracted procedure: k8812 
o|contracted procedure: k8819 
o|contracted procedure: k8855 
o|contracted procedure: k8826 
o|contracted procedure: k8835 
o|contracted procedure: k8852 
o|contracted procedure: k8841 
o|contracted procedure: k9026 
o|contracted procedure: k9142 
o|contracted procedure: k9146 
o|contracted procedure: k9149 
o|contracted procedure: k9124 
o|contracted procedure: k9163 
o|substituted constant variable: g35678 
o|contracted procedure: k9170 
o|contracted procedure: k9187 
o|contracted procedure: k9191 
o|contracted procedure: k9206 
o|contracted procedure: k9222 
o|contracted procedure: k9290 
o|contracted procedure: k9261 
o|contracted procedure: k9286 
o|contracted procedure: k9276 
o|contracted procedure: k9299 
o|contracted procedure: k9315 
o|contracted procedure: k9331 
o|contracted procedure: k9339 
o|contracted procedure: k9349 
o|contracted procedure: k9360 
o|contracted procedure: k9376 
o|contracted procedure: k9393 
o|contracted procedure: k9400 
o|contracted procedure: k9727 
o|contracted procedure: k9734 
o|contracted procedure: k9742 
o|contracted procedure: k9751 
o|contracted procedure: k9757 
o|contracted procedure: k9763 
o|contracted procedure: k9784 
o|contracted procedure: k9797 
o|contracted procedure: k9803 
o|contracted procedure: k9852 
o|contracted procedure: k9824 
o|contracted procedure: k9842 
o|contracted procedure: k9848 
o|contracted procedure: k9859 
o|contracted procedure: k9865 
o|contracted procedure: k9877 
o|contracted procedure: k9892 
o|contracted procedure: k9902 
o|contracted procedure: k9909 
o|contracted procedure: k9917 
o|contracted procedure: k9923 
o|contracted procedure: k9926 
o|contracted procedure: k9953 
o|contracted procedure: k9948 
o|contracted procedure: k9938 
o|contracted procedure: k9945 
o|contracted procedure: k9959 
o|contracted procedure: k9965 
o|contracted procedure: k9968 
o|contracted procedure: k9981 
o|contracted procedure: k9988 
o|contracted procedure: k9996 
o|contracted procedure: k10008 
o|contracted procedure: k10029 
o|contracted procedure: k10042 
o|contracted procedure: k10048 
o|contracted procedure: k10054 
o|contracted procedure: k10074 
o|contracted procedure: k10103 
o|contracted procedure: k10106 
o|contracted procedure: k10112 
o|contracted procedure: k10130 
o|contracted procedure: k10149 
o|contracted procedure: k10155 
o|contracted procedure: k10163 
o|contracted procedure: k10180 
o|contracted procedure: k10183 
o|contracted procedure: k10198 
o|contracted procedure: k10190 
o|contracted procedure: k10224 
o|contracted procedure: k10245 
o|contracted procedure: k10271 
o|contracted procedure: k10274 
o|contracted procedure: k10277 
o|contracted procedure: k10316 
o|contracted procedure: k10451 
o|contracted procedure: k10343 
o|contracted procedure: k10350 
o|contracted procedure: k10362 
o|inlining procedure: k10354 
o|contracted procedure: k10371 
o|contracted procedure: k10374 
o|inlining procedure: k10354 
o|contracted procedure: k10380 
o|contracted procedure: k10383 
o|contracted procedure: k10389 
o|contracted procedure: k10392 
o|contracted procedure: k10398 
o|contracted procedure: k10401 
o|contracted procedure: k10435 
o|contracted procedure: k10460 
o|contracted procedure: k10472 
o|contracted procedure: k970833586 
o|contracted procedure: k10500 
o|substituted constant variable: g35711 
o|contracted procedure: k10490 
o|contracted procedure: k970833594 
o|contracted procedure: k10512 
o|substituted constant variable: g35715 
o|contracted procedure: k10519 
o|contracted procedure: k10534 
o|contracted procedure: k10537 
o|contracted procedure: k10540 
o|contracted procedure: k10548 
o|contracted procedure: k10556 
o|contracted procedure: k10568 
o|contracted procedure: k970833608 
o|contracted procedure: k10585 
o|substituted constant variable: g35729 
o|contracted procedure: k10594 
o|contracted procedure: k10603 
o|contracted procedure: k10613 
o|contracted procedure: k10632 
o|inlining procedure: k10625 
o|contracted procedure: k10645 
o|contracted procedure: k14229 
o|substituted constant variable: g35724 
o|contracted procedure: k14258 
o|contracted procedure: k14232 
o|contracted procedure: k14238 
o|contracted procedure: k14241 
o|substituted constant variable: g35728 
o|contracted procedure: k10688 
o|contracted procedure: k970833614 
o|contracted procedure: k10780 
o|substituted constant variable: g35731 
o|contracted procedure: k10694 
o|contracted procedure: k10777 
o|contracted procedure: k10703 
o|contracted procedure: k10766 
o|contracted procedure: k10709 
o|contracted procedure: k10716 
o|contracted procedure: k10723 
o|contracted procedure: k10730 
o|contracted procedure: k10756 
o|contracted procedure: k10773 
o|contracted procedure: k10817 
o|contracted procedure: k10793 
o|contracted procedure: k10811 
o|contracted procedure: k10824 
o|contracted procedure: k10832 
o|contracted procedure: k10843 
o|contracted procedure: k10857 
o|contracted procedure: k10871 
o|contracted procedure: k10875 
o|contracted procedure: k10879 
o|contracted procedure: k10894 
o|contracted procedure: k10904 
o|contracted procedure: k10910 
o|contracted procedure: k10919 
o|contracted procedure: k10925 
o|contracted procedure: k970833630 
o|contracted procedure: k10949 
o|substituted constant variable: g35735 
o|inlining procedure: k10938 
o|inlining procedure: k10938 
o|contracted procedure: k10955 
o|contracted procedure: k10964 
o|contracted procedure: k10970 
o|contracted procedure: k10983 
o|contracted procedure: k10992 
o|contracted procedure: k10999 
o|contracted procedure: k11010 
o|contracted procedure: k11021 
o|contracted procedure: k11017 
o|contracted procedure: k11029 
o|contracted procedure: k11040 
o|contracted procedure: k11051 
o|contracted procedure: k11062 
o|contracted procedure: k11058 
o|contracted procedure: k11070 
o|contracted procedure: k11081 
o|contracted procedure: k11085 
o|contracted procedure: k11077 
o|contracted procedure: k11112 
o|contracted procedure: k11115 
o|contracted procedure: k11170 
o|contracted procedure: k11121 
o|contracted procedure: k11159 
o|contracted procedure: k11163 
o|contracted procedure: k11155 
o|contracted procedure: k11167 
o|contracted procedure: k11177 
o|contracted procedure: k11943 
o|contracted procedure: k11193 
o|contracted procedure: k11206 
o|contracted procedure: k11934 
o|contracted procedure: k11226 
o|contracted procedure: k11292 
o|contracted procedure: k11232 
o|contracted procedure: k11247 
o|contracted procedure: k11260 
o|contracted procedure: k11264 
o|contracted procedure: k11268 
o|contracted procedure: k11284 
o|contracted procedure: k11279 
o|contracted procedure: k11298 
o|contracted procedure: k11304 
o|contracted procedure: k11314 
o|contracted procedure: k11318 
o|contracted procedure: k11331 
o|contracted procedure: k972733640 
o|contracted procedure: k11344 
o|contracted procedure: k11354 
o|contracted procedure: k972733646 
o|contracted procedure: k11367 
o|contracted procedure: k11377 
o|contracted procedure: k972733652 
o|contracted procedure: k11390 
o|contracted procedure: k11400 
o|contracted procedure: k11413 
o|contracted procedure: k11419 
o|contracted procedure: k972733658 
o|contracted procedure: k11432 
o|contracted procedure: k11442 
o|contracted procedure: k972733664 
o|contracted procedure: k11455 
o|contracted procedure: k11472 
o|contracted procedure: k11476 
o|contracted procedure: k972733670 
o|contracted procedure: k11480 
o|contracted procedure: k11484 
o|contracted procedure: k11515 
o|contracted procedure: k11505 
o|contracted procedure: k11531 
o|contracted procedure: k972733676 
o|contracted procedure: k11544 
o|contracted procedure: k11554 
o|contracted procedure: k11687 
o|contracted procedure: k11570 
o|contracted procedure: k11582 
o|contracted procedure: k972733682 
o|contracted procedure: k11598 
o|contracted procedure: k11602 
o|contracted procedure: k11680 
o|contracted procedure: k11622 
o|contracted procedure: k972733688 
o|contracted procedure: k11641 
o|contracted procedure: k11645 
o|contracted procedure: k972733694 
o|contracted procedure: k11672 
o|contracted procedure: k11696 
o|contracted procedure: k972733700 
o|contracted procedure: k970833716 
o|contracted procedure: k11723 
o|substituted constant variable: g35741 
o|contracted procedure: k970833710 
o|contracted procedure: k11726 
o|substituted constant variable: g35739 
o|contracted procedure: k11732 
o|contracted procedure: k11745 
o|contracted procedure: k11751 
o|contracted procedure: k11757 
o|contracted procedure: k11773 
o|contracted procedure: k11789 
o|contracted procedure: k11805 
o|contracted procedure: k11830 
o|contracted procedure: k11840 
o|contracted procedure: k11846 
o|contracted procedure: k11869 
o|contracted procedure: k11890 
o|contracted procedure: k11886 
o|contracted procedure: k11953 
o|contracted procedure: k11959 
o|contracted procedure: k11975 
o|contracted procedure: k11989 
o|contracted procedure: k11979 
o|contracted procedure: k11995 
o|contracted procedure: k12008 
o|contracted procedure: k12012 
o|contracted procedure: k12016 
o|contracted procedure: k13394 
o|contracted procedure: k970833841 
o|contracted procedure: k13400 
o|substituted constant variable: g35765 
o|contracted procedure: k13409 
o|contracted procedure: k13415 
o|contracted procedure: k13421 
o|contracted procedure: k2627933740 
o|contracted procedure: k13427 
o|contracted procedure: k2626933729 
o|contracted procedure: k2724733724 
o|contracted procedure: k970833735 
o|contracted procedure: k13440 
o|substituted constant variable: g35747 
o|contracted procedure: k13450 
o|contracted procedure: k26945 
o|contracted procedure: k26949 
o|contracted procedure: k13466 
o|contracted procedure: k13472 
o|contracted procedure: k2626933751 
o|contracted procedure: k2724733746 
o|contracted procedure: k13491 
o|contracted procedure: k13497 
o|contracted procedure: k13509 
o|contracted procedure: k2625933757 
o|contracted procedure: k13523 
o|contracted procedure: k13613 
o|contracted procedure: k13527 
o|contracted procedure: k13539 
o|contracted procedure: k13543 
o|contracted procedure: k1402533762 
o|contracted procedure: k13567 
o|contracted procedure: k13578 
o|inlining procedure: k13530 
o|inlining procedure: k13530 
o|contracted procedure: k13594 
o|substituted constant variable: g35749 
o|substituted constant variable: g35751 
o|contracted procedure: k13597 
o|contracted procedure: k13604 
o|contracted procedure: k13635 
o|contracted procedure: k13653 
o|contracted procedure: k13641 
o|contracted procedure: k13649 
o|contracted procedure: k13659 
o|contracted procedure: k13778 
o|contracted procedure: k13662 
o|contracted procedure: k13665 
o|contracted procedure: k13668 
o|contracted procedure: k13674 
o|contracted procedure: k13683 
o|contracted procedure: k13735 
o|contracted procedure: k13731 
o|contracted procedure: k13726 
o|contracted procedure: k13744 
o|contracted procedure: k13747 
o|contracted procedure: k2626933773 
o|contracted procedure: k2724733768 
o|contracted procedure: k13787 
o|contracted procedure: k13790 
o|contracted procedure: k13796 
o|contracted procedure: k13821 
o|contracted procedure: k13827 
o|contracted procedure: k2626933784 
o|contracted procedure: k2724733779 
o|contracted procedure: k13850 
o|contracted procedure: k13853 
o|contracted procedure: k2626933795 
o|contracted procedure: k2724733790 
o|contracted procedure: k13871 
o|contracted procedure: k13877 
o|contracted procedure: k13883 
o|contracted procedure: k13889 
o|contracted procedure: k1402533811 
o|contracted procedure: k13919 
o|contracted procedure: k13926 
o|contracted procedure: k2626933806 
o|contracted procedure: k2724733801 
o|contracted procedure: k2626933822 
o|contracted procedure: k2724733817 
o|contracted procedure: k13955 
o|substituted constant variable: g35756 
o|substituted constant variable: g35758 
o|contracted procedure: k13973 
o|contracted procedure: k970833833 
o|contracted procedure: k13980 
o|substituted constant variable: g35761 
o|contracted procedure: k2626933827 
o|inlining procedure: k13977 
o|inlining procedure: k13977 
o|contracted procedure: k13993 
o|contracted procedure: k13996 
o|contracted procedure: k14004 
o|contracted procedure: k12033 
o|contracted procedure: k12058 
o|contracted procedure: k12064 
o|contracted procedure: k12083 
o|contracted procedure: k12095 
o|contracted procedure: k12112 
o|contracted procedure: k12108 
o|contracted procedure: k12132 
o|contracted procedure: k12128 
o|contracted procedure: k12149 
o|contracted procedure: k12145 
o|contracted procedure: k12157 
o|contracted procedure: k12174 
o|contracted procedure: k12170 
o|contracted procedure: k12163 
o|contracted procedure: k12179 
o|contracted procedure: k12186 
o|contracted procedure: k12192 
o|contracted procedure: k8961 
o|contracted procedure: k8983 
o|contracted procedure: k9020 
o|contracted procedure: k8996 
o|contracted procedure: k12227 
o|contracted procedure: k12209 
o|contracted procedure: k12220 
o|contracted procedure: k12212 
o|contracted procedure: k12236 
o|contracted procedure: k12249 
o|contracted procedure: k12255 
o|contracted procedure: k12268 
o|contracted procedure: k12278 
o|contracted procedure: k12295 
o|contracted procedure: k12291 
o|contracted procedure: k12305 
o|contracted procedure: k12318 
o|contracted procedure: k12328 
o|contracted procedure: k12345 
o|contracted procedure: k12341 
o|contracted procedure: k12355 
o|contracted procedure: k12380 
o|contracted procedure: k12372 
o|contracted procedure: k12368 
o|contracted procedure: k12386 
o|contracted procedure: k12415 
o|contracted procedure: k12411 
o|contracted procedure: k12403 
o|contracted procedure: k12399 
o|contracted procedure: k12421 
o|contracted procedure: k12438 
o|contracted procedure: k12434 
o|contracted procedure: k12448 
o|contracted procedure: k12461 
o|contracted procedure: k12471 
o|contracted procedure: k12484 
o|contracted procedure: k12494 
o|contracted procedure: k12511 
o|contracted procedure: k12515 
o|contracted procedure: k12507 
o|contracted procedure: k12525 
o|contracted procedure: k12538 
o|contracted procedure: k12548 
o|contracted procedure: k12561 
o|contracted procedure: k12571 
o|contracted procedure: k12584 
o|contracted procedure: k12594 
o|contracted procedure: k12607 
o|contracted procedure: k12617 
o|contracted procedure: k12630 
o|contracted procedure: k12640 
o|contracted procedure: k12653 
o|contracted procedure: k12657 
o|contracted procedure: k12661 
o|contracted procedure: k12678 
o|contracted procedure: k12681 
o|contracted procedure: k12687 
o|contracted procedure: k970833849 
o|contracted procedure: k12741 
o|substituted constant variable: g35773 
o|contracted procedure: k12702 
o|contracted procedure: k12708 
o|contracted procedure: k12718 
o|contracted procedure: k12722 
o|contracted procedure: k12730 
o|contracted procedure: k12726 
o|contracted procedure: k13265 
o|contracted procedure: k13271 
o|contracted procedure: k13277 
o|contracted procedure: k13283 
o|contracted procedure: k12762 
o|contracted procedure: k12780 
o|contracted procedure: k12787 
o|contracted procedure: k12862 
o|contracted procedure: k12793 
o|contracted procedure: k12810 
o|contracted procedure: k12848 
o|contracted procedure: k12816 
o|contracted procedure: k12829 
o|contracted procedure: k12837 
o|contracted procedure: k12868 
o|contracted procedure: k12884 
o|contracted procedure: k12894 
o|contracted procedure: k12900 
o|contracted procedure: k970833857 
o|contracted procedure: k12931 
o|substituted constant variable: g35779 
o|contracted procedure: k12903 
o|contracted procedure: k12913 
o|contracted procedure: k12906 
o|contracted procedure: k8919 
o|contracted procedure: k8954 
o|contracted procedure: k8926 
o|contracted procedure: k8935 
o|contracted procedure: k8951 
o|contracted procedure: k1294133862 
o|contracted procedure: k8941 
o|contracted procedure: k12950 
o|contracted procedure: k12953 
o|contracted procedure: k12973 
o|contracted procedure: k12969 
o|contracted procedure: k13040 
o|contracted procedure: k13053 
o|contracted procedure: k13063 
o|contracted procedure: k970833870 
o|contracted procedure: k13086 
o|substituted constant variable: g35785 
o|contracted procedure: k13066 
o|contracted procedure: k13079 
o|contracted procedure: k13092 
o|contracted procedure: k970833876 
o|contracted procedure: k13115 
o|substituted constant variable: g35787 
o|contracted procedure: k13095 
o|contracted procedure: k13108 
o|contracted procedure: k13121 
o|contracted procedure: k970833882 
o|contracted procedure: k13127 
o|substituted constant variable: g35789 
o|contracted procedure: k13152 
o|contracted procedure: k970833888 
o|contracted procedure: k13158 
o|substituted constant variable: g35791 
o|contracted procedure: k13164 
o|contracted procedure: k13171 
o|contracted procedure: k13175 
o|contracted procedure: k13237 
o|contracted procedure: k13230 
o|contracted procedure: k30949 
o|contracted procedure: k30953 
o|contracted procedure: k30957 
o|contracted procedure: k30961 
o|contracted procedure: k30965 
o|contracted procedure: k30969 
o|contracted procedure: k13241 
o|contracted procedure: k13250 
o|contracted procedure: k13300 
o|contracted procedure: k13388 
o|contracted procedure: k13309 
o|contracted procedure: k13318 
o|contracted procedure: k13337 
o|contracted procedure: k8862 
o|contracted procedure: k8912 
o|contracted procedure: k8869 
o|contracted procedure: k8878 
o|contracted procedure: k8909 
o|contracted procedure: k8884 
o|contracted procedure: k13352 
o|contracted procedure: k13371 
o|contracted procedure: k14015 
o|contracted procedure: k14038 
o|contracted procedure: k14044 
o|contracted procedure: k14053 
o|contracted procedure: k14077 
o|contracted procedure: k14061 
o|contracted procedure: k14065 
o|contracted procedure: k14073 
o|contracted procedure: k14087 
o|contracted procedure: k14130 
o|contracted procedure: k14098 
o|contracted procedure: k14118 
o|contracted procedure: k14102 
o|contracted procedure: k14106 
o|contracted procedure: k14094 
o|contracted procedure: k14114 
o|contracted procedure: k14126 
o|contracted procedure: k14140 
o|contracted procedure: k14199 
o|contracted procedure: k14151 
o|contracted procedure: k14187 
o|contracted procedure: k14155 
o|contracted procedure: k14175 
o|contracted procedure: k14159 
o|contracted procedure: k14163 
o|contracted procedure: k14147 
o|contracted procedure: k14171 
o|contracted procedure: k14183 
o|contracted procedure: k14195 
o|contracted procedure: k14267 
o|contracted procedure: k14273 
o|contracted procedure: k14279 
o|contracted procedure: k14285 
o|contracted procedure: k14340 
o|contracted procedure: k14346 
o|contracted procedure: k14355 
o|contracted procedure: k14374 
o|contracted procedure: k14362 
o|contracted procedure: k14370 
o|contracted procedure: k14366 
o|contracted procedure: k14380 
o|contracted procedure: k14411 
o|contracted procedure: k14387 
o|contracted procedure: k14407 
o|contracted procedure: k14403 
o|contracted procedure: k14391 
o|contracted procedure: k14399 
o|contracted procedure: k14395 
o|contracted procedure: k14417 
o|contracted procedure: k14460 
o|contracted procedure: k14424 
o|contracted procedure: k14456 
o|contracted procedure: k14452 
o|contracted procedure: k14428 
o|contracted procedure: k14448 
o|contracted procedure: k14444 
o|contracted procedure: k14432 
o|contracted procedure: k14440 
o|contracted procedure: k14436 
o|contracted procedure: k14619 
o|contracted procedure: k14626 
o|contracted procedure: k14630 
o|contracted procedure: k14634 
o|contracted procedure: k14646 
o|contracted procedure: k14649 
o|contracted procedure: k14664 
o|contracted procedure: k14673 
o|contracted procedure: k14656 
o|contracted procedure: k14685 
o|contracted procedure: k14703 
o|contracted procedure: k14688 
o|contracted procedure: k14691 
o|contracted procedure: k14699 
o|contracted procedure: k14721 
o|contracted procedure: k14743 
o|contracted procedure: k14739 
o|contracted procedure: k14724 
o|contracted procedure: k14727 
o|contracted procedure: k14735 
o|contracted procedure: k14887 
o|contracted procedure: k14762 
o|contracted procedure: k14758 
o|contracted procedure: k14788 
o|contracted procedure: k14784 
o|contracted procedure: k14780 
o|contracted procedure: k14792 
o|contracted procedure: k14796 
o|contracted procedure: k14808 
o|contracted procedure: k14814 
o|contracted procedure: k14829 
o|contracted procedure: k14825 
o|contracted procedure: k14821 
o|contracted procedure: k14842 
o|contracted procedure: k14864 
o|contracted procedure: k14860 
o|contracted procedure: k14845 
o|contracted procedure: k14848 
o|contracted procedure: k14856 
o|contracted procedure: k14871 
o|contracted procedure: k15026 
o|contracted procedure: k14901 
o|contracted procedure: k14897 
o|contracted procedure: k14927 
o|contracted procedure: k14923 
o|contracted procedure: k14919 
o|contracted procedure: k14931 
o|contracted procedure: k14935 
o|contracted procedure: k14947 
o|contracted procedure: k14953 
o|contracted procedure: k14968 
o|contracted procedure: k14964 
o|contracted procedure: k14960 
o|contracted procedure: k14981 
o|contracted procedure: k15003 
o|contracted procedure: k14999 
o|contracted procedure: k14984 
o|contracted procedure: k14987 
o|contracted procedure: k14995 
o|contracted procedure: k15010 
o|contracted procedure: k15484 
o|contracted procedure: k15502 
o|contracted procedure: k15509 
o|contracted procedure: k15499 
o|contracted procedure: k15573 
o|contracted procedure: k1401533933 
o|contracted procedure: k15519 
o|contracted procedure: k15529 
o|contracted procedure: k15533 
o|contracted procedure: k14606 
o|contracted procedure: k14486 
o|contracted procedure: k14493 
o|contracted procedure: k15032 
o|contracted procedure: k15043 
o|contracted procedure: k15047 
o|contracted procedure: k15064 
o|contracted procedure: k15075 
o|contracted procedure: k15056 
o|contracted procedure: k15087 
o|contracted procedure: k15105 
o|contracted procedure: k15090 
o|contracted procedure: k15093 
o|contracted procedure: k15101 
o|contracted procedure: k15116 
o|contracted procedure: k15120 
o|contracted procedure: k15124 
o|contracted procedure: k15132 
o|contracted procedure: k14305 
o|contracted procedure: k14311 
o|contracted procedure: k14317 
o|contracted procedure: k14323 
o|contracted procedure: k15140 
o|contracted procedure: k15146 
o|contracted procedure: k15153 
o|contracted procedure: k15219 
o|contracted procedure: k15227 
o|contracted procedure: k15223 
o|contracted procedure: k15165 
o|contracted procedure: k15169 
o|contracted procedure: k15178 
o|contracted procedure: k15161 
o|contracted procedure: k15190 
o|contracted procedure: k15208 
o|contracted procedure: k15193 
o|contracted procedure: k15196 
o|contracted procedure: k15204 
o|contracted procedure: k15249 
o|contracted procedure: k15245 
o|contracted procedure: k15241 
o|contracted procedure: k1402533913 
o|contracted procedure: k15237 
o|contracted procedure: k15258 
o|contracted procedure: k15261 
o|contracted procedure: k15264 
o|contracted procedure: k15272 
o|contracted procedure: k15280 
o|contracted procedure: k15288 
o|contracted procedure: k15374 
o|contracted procedure: k15357 
o|contracted procedure: k15368 
o|substituted constant variable: g35822 
o|contracted procedure: k15303 
o|contracted procedure: k15307 
o|contracted procedure: k15316 
o|contracted procedure: k15299 
o|contracted procedure: k15328 
o|contracted procedure: k15346 
o|contracted procedure: k15331 
o|contracted procedure: k15334 
o|contracted procedure: k15342 
o|contracted procedure: k14598 
o|contracted procedure: k14602 
o|contracted procedure: k14513 
o|contracted procedure: k14524 
o|contracted procedure: k14531 
o|inlining procedure: k14520 
o|contracted procedure: k14528 
o|inlining procedure: k14520 
o|contracted procedure: k14590 
o|contracted procedure: k14552 
o|contracted procedure: k14559 
o|contracted procedure: k14574 
o|contracted procedure: k15465 
o|contracted procedure: k15461 
o|contracted procedure: k15380 
o|contracted procedure: k15457 
o|contracted procedure: k15453 
o|contracted procedure: k15383 
o|contracted procedure: k15394 
o|contracted procedure: k15398 
o|contracted procedure: k15407 
o|contracted procedure: k15390 
o|contracted procedure: k15419 
o|contracted procedure: k15437 
o|contracted procedure: k15422 
o|contracted procedure: k15425 
o|contracted procedure: k15433 
o|contracted procedure: k15549 
o|contracted procedure: k15560 
o|contracted procedure: k15553 
o|contracted procedure: k15569 
o|contracted procedure: k1401533928 
o|contracted procedure: k16047 
o|contracted procedure: k16148 
o|contracted procedure: k16053 
o|contracted procedure: k16144 
o|contracted procedure: k16062 
o|contracted procedure: k16140 
o|contracted procedure: k16071 
o|contracted procedure: k16136 
o|contracted procedure: k16080 
o|contracted procedure: k16132 
o|contracted procedure: k16089 
o|contracted procedure: k16098 
o|contracted procedure: k16101 
o|contracted procedure: k16119 
o|contracted procedure: k16104 
o|contracted procedure: k16115 
o|contracted procedure: k16111 
o|contracted procedure: k16107 
o|contracted procedure: k16122 
o|contracted procedure: k16154 
o|contracted procedure: k16188 
o|contracted procedure: k17086 
o|contracted procedure: k16225 
o|contracted procedure: k16209 
o|contracted procedure: k26146 
o|contracted procedure: k23402 
o|contracted procedure: k23438 
o|contracted procedure: k23442 
o|contracted procedure: k23408 
o|contracted procedure: k23432 
o|contracted procedure: k23411 
o|contracted procedure: k23428 
o|contracted procedure: k23414 
o|contracted procedure: k23424 
o|contracted procedure: k23417 
o|contracted procedure: k23420 
o|contracted procedure: k23457 
o|contracted procedure: k23463 
o|contracted procedure: k970833941 
o|contracted procedure: k23476 
o|substituted constant variable: g35833 
o|contracted procedure: k23483 
o|contracted procedure: k970833947 
o|contracted procedure: k23497 
o|substituted constant variable: g35835 
o|contracted procedure: k23503 
o|contracted procedure: k23506 
o|contracted procedure: k23512 
o|contracted procedure: k23523 
o|contracted procedure: k23530 
o|contracted procedure: k23555 
o|contracted procedure: k23563 
o|contracted procedure: k23567 
o|contracted procedure: k23571 
o|contracted procedure: k23583 
o|contracted procedure: k972733953 
o|contracted procedure: k23594 
o|contracted procedure: k23602 
o|contracted procedure: k23613 
o|contracted procedure: k23621 
o|contracted procedure: k23632 
o|contracted procedure: k23640 
o|contracted procedure: k972733967 
o|contracted procedure: k23651 
o|contracted procedure: k23659 
o|contracted procedure: k23662 
o|contracted procedure: k23665 
o|contracted procedure: k23671 
o|contracted procedure: k23677 
o|contracted procedure: k23684 
o|contracted procedure: k23694 
o|contracted procedure: k23702 
o|contracted procedure: k23710 
o|contracted procedure: k23714 
o|contracted procedure: k23726 
o|contracted procedure: k23752 
o|contracted procedure: k23778 
o|contracted procedure: k23805 
o|contracted procedure: k23823 
o|contracted procedure: k23877 
o|contracted procedure: k23920 
o|contracted procedure: k23935 
o|contracted procedure: k23939 
o|contracted procedure: k23931 
o|contracted procedure: k23927 
o|contracted procedure: k23951 
o|contracted procedure: k23966 
o|contracted procedure: k23970 
o|contracted procedure: k23962 
o|contracted procedure: k23958 
o|contracted procedure: k23979 
o|contracted procedure: k23998 
o|contracted procedure: k24033 
o|contracted procedure: k24043 
o|contracted procedure: k24092 
o|contracted procedure: k24125 
o|contracted procedure: k24101 
o|contracted procedure: k24121 
o|contracted procedure: k24107 
o|contracted procedure: k24131 
o|contracted procedure: k24150 
o|contracted procedure: k24167 
o|contracted procedure: k24177 
o|contracted procedure: k24202 
o|contracted procedure: k24254 
o|contracted procedure: k24287 
o|contracted procedure: k24263 
o|contracted procedure: k24283 
o|contracted procedure: k24269 
o|contracted procedure: k24293 
o|contracted procedure: k24304 
o|contracted procedure: k24300 
o|contracted procedure: k24314 
o|contracted procedure: k24320 
o|contracted procedure: k24339 
o|contracted procedure: k24343 
o|contracted procedure: k24335 
o|contracted procedure: k24331 
o|contracted procedure: k24327 
o|contracted procedure: k24351 
o|contracted procedure: k24362 
o|contracted procedure: k24368 
o|contracted procedure: k24403 
o|contracted procedure: k24438 
o|contracted procedure: k24441 
o|contracted procedure: k24452 
o|contracted procedure: k24459 
o|contracted procedure: k24495 
o|contracted procedure: k24489 
o|contracted procedure: k24468 
o|contracted procedure: k24501 
o|contracted procedure: k8751 
o|contracted procedure: k8741 
o|contracted procedure: k8744 
o|contracted procedure: k8755 
o|contracted procedure: k8759 
o|contracted procedure: k8768 
o|contracted procedure: k8774 
o|contracted procedure: k8797 
o|contracted procedure: k8783 
o|contracted procedure: k8790 
o|contracted procedure: k8804 
o|contracted procedure: k8808 
o|contracted procedure: k24512 
o|contracted procedure: k24524 
o|contracted procedure: k24559 
o|contracted procedure: k24663 
o|contracted procedure: k24574 
o|contracted procedure: k24594 
o|contracted procedure: k24602 
o|contracted procedure: k24598 
o|contracted procedure: k24605 
o|contracted procedure: k24614 
o|contracted procedure: k24617 
o|contracted procedure: k24653 
o|contracted procedure: k24694 
o|contracted procedure: k24666 
o|contracted procedure: k24673 
o|contracted procedure: k24677 
o|contracted procedure: k24698 
o|contracted procedure: k24702 
o|contracted procedure: k24706 
o|contracted procedure: k24712 
o|contracted procedure: k24715 
o|contracted procedure: k24732 
o|contracted procedure: k24742 
o|contracted procedure: k24860 
o|contracted procedure: k24751 
o|contracted procedure: k24764 
o|contracted procedure: k24779 
o|contracted procedure: k24783 
o|contracted procedure: k24792 
o|contracted procedure: k24805 
o|contracted procedure: k24812 
o|contracted procedure: k24819 
o|contracted procedure: k24831 
o|contracted procedure: k24856 
o|contracted procedure: k24852 
o|contracted procedure: k24863 
o|contracted procedure: k970833975 
o|contracted procedure: k24866 
o|substituted constant variable: g35853 
o|contracted procedure: k24898 
o|contracted procedure: k24874 
o|contracted procedure: k24879 
o|contracted procedure: k24904 
o|contracted procedure: k24919 
o|contracted procedure: k24915 
o|contracted procedure: k24923 
o|contracted procedure: k24929 
o|contracted procedure: k24932 
o|contracted procedure: k24946 
o|contracted procedure: k24950 
o|contracted procedure: k24953 
o|contracted procedure: k24962 
o|contracted procedure: k24970 
o|contracted procedure: k24966 
o|contracted procedure: k24973 
o|contracted procedure: k25030 
o|contracted procedure: k25034 
o|contracted procedure: k24976 
o|contracted procedure: k25018 
o|contracted procedure: k25026 
o|contracted procedure: k25022 
o|contracted procedure: k24979 
o|contracted procedure: k25010 
o|contracted procedure: k25014 
o|contracted procedure: k24988 
o|contracted procedure: k24998 
o|contracted procedure: k25006 
o|contracted procedure: k25002 
o|contracted procedure: k24991 
o|contracted procedure: k25042 
o|contracted procedure: k25054 
o|contracted procedure: k25062 
o|contracted procedure: k25058 
o|contracted procedure: k25065 
o|contracted procedure: k25129 
o|contracted procedure: k25137 
o|contracted procedure: k25133 
o|contracted procedure: k25071 
o|contracted procedure: k25117 
o|contracted procedure: k25125 
o|contracted procedure: k25121 
o|contracted procedure: k25074 
o|contracted procedure: k25105 
o|contracted procedure: k25113 
o|contracted procedure: k25109 
o|contracted procedure: k25083 
o|contracted procedure: k25093 
o|contracted procedure: k25101 
o|contracted procedure: k25097 
o|contracted procedure: k25086 
o|contracted procedure: k25145 
o|contracted procedure: k25148 
o|contracted procedure: k25159 
o|contracted procedure: k25155 
o|contracted procedure: k25244 
o|contracted procedure: k25250 
o|contracted procedure: k25272 
o|contracted procedure: k25278 
o|contracted procedure: k25286 
o|contracted procedure: k25293 
o|contracted procedure: k25296 
o|contracted procedure: k25305 
o|contracted procedure: k25311 
o|contracted procedure: k25317 
o|contracted procedure: k25327 
o|contracted procedure: k25336 
o|contracted procedure: k25344 
o|contracted procedure: k25368 
o|contracted procedure: k25363 
o|contracted procedure: k25350 
o|contracted procedure: k25371 
o|contracted procedure: k25380 
o|contracted procedure: k25386 
o|contracted procedure: k25392 
o|contracted procedure: k25419 
o|contracted procedure: k25414 
o|contracted procedure: k25401 
o|contracted procedure: k25428 
o|contracted procedure: k25452 
o|contracted procedure: k25445 
o|contracted procedure: k25436 
o|contracted procedure: k25458 
o|contracted procedure: k25469 
o|contracted procedure: k25472 
o|contracted procedure: k25500 
o|contracted procedure: k25486 
o|contracted procedure: k25493 
o|contracted procedure: k25509 
o|contracted procedure: k25532 
o|contracted procedure: k25539 
o|contracted procedure: k25542 
o|contracted procedure: k25559 
o|contracted procedure: k25566 
o|contracted procedure: k25555 
o|contracted procedure: k25607 
o|contracted procedure: k25572 
o|contracted procedure: k25587 
o|contracted procedure: k25583 
o|contracted procedure: k25593 
o|contracted procedure: k25616 
o|contracted procedure: k25633 
o|contracted procedure: k25640 
o|contracted procedure: k25649 
o|contracted procedure: k25666 
o|contracted procedure: k25674 
o|contracted procedure: k25677 
o|contracted procedure: k25686 
o|contracted procedure: k25698 
o|contracted procedure: k25705 
o|contracted procedure: k25694 
o|contracted procedure: k25714 
o|contracted procedure: k25764 
o|contracted procedure: k25737 
o|contracted procedure: k25748 
o|contracted procedure: k25744 
o|contracted procedure: k25754 
o|contracted procedure: k25770 
o|contracted procedure: k25781 
o|contracted procedure: k25787 
o|contracted procedure: k25803 
o|contracted procedure: k25859 
o|contracted procedure: k25845 
o|contracted procedure: k25852 
o|contracted procedure: k25865 
o|contracted procedure: k25877 
o|contracted procedure: k25880 
o|contracted procedure: k25890 
o|contracted procedure: k25919 
o|contracted procedure: k970833985 
o|contracted procedure: k25925 
o|substituted constant variable: g35889 
o|contracted procedure: k25933 
o|contracted procedure: k25940 
o|contracted procedure: k25952 
o|contracted procedure: k25958 
o|contracted procedure: k25964 
o|contracted procedure: k25973 
o|contracted procedure: k25980 
o|contracted procedure: k26002 
o|contracted procedure: k26024 
o|contracted procedure: k26032 
o|contracted procedure: k26039 
o|contracted procedure: k26051 
o|contracted procedure: k26057 
o|contracted procedure: k26063 
o|contracted procedure: k26072 
o|contracted procedure: k26079 
o|contracted procedure: k26101 
o|contracted procedure: k26088 
o|contracted procedure: k26123 
o|contracted procedure: k26110 
o|contracted procedure: k26129 
o|contracted procedure: k26159 
o|contracted procedure: k26150 
o|contracted procedure: k17103 
o|contracted procedure: k17107 
o|contracted procedure: k17116 
o|contracted procedure: k17122 
o|contracted procedure: k17132 
o|contracted procedure: k17144 
o|contracted procedure: k17155 
o|contracted procedure: k17161 
o|contracted procedure: k17178 
o|contracted procedure: k17185 
o|contracted procedure: k17189 
o|contracted procedure: k17202 
o|contracted procedure: k17212 
o|contracted procedure: k17216 
o|contracted procedure: k17220 
o|contracted procedure: k17224 
o|contracted procedure: k17242 
o|contracted procedure: k17256 
o|contracted procedure: k17263 
o|contracted procedure: k17267 
o|contracted procedure: k17284 
o|contracted procedure: k17294 
o|contracted procedure: k17298 
o|contracted procedure: k17327 
o|contracted procedure: k17333 
o|contracted procedure: k17362 
o|contracted procedure: k17366 
o|contracted procedure: k17421 
o|contracted procedure: k17414 
o|contracted procedure: k17376 
o|contracted procedure: k17380 
o|contracted procedure: k17390 
o|inlining procedure: k17394 
o|inlining procedure: k17394 
o|contracted procedure: k17439 
o|contracted procedure: k17424 
o|contracted procedure: k17427 
o|contracted procedure: k17443 
o|contracted procedure: k17452 
o|contracted procedure: k17460 
o|contracted procedure: k17475 
o|contracted procedure: k17471 
o|contracted procedure: k17479 
o|contracted procedure: k17485 
o|contracted procedure: k17499 
o|contracted procedure: k17512 
o|contracted procedure: k17505 
o|contracted procedure: k17519 
o|contracted procedure: k17516 
o|contracted procedure: k17531 
o|contracted procedure: k17534 
o|contracted procedure: k17543 
o|contracted procedure: k17593 
o|contracted procedure: k17555 
o|contracted procedure: k17589 
o|contracted procedure: k17561 
o|contracted procedure: k17585 
o|contracted procedure: k17564 
o|contracted procedure: k17571 
o|contracted procedure: k17575 
o|contracted procedure: k17604 
o|contracted procedure: k17600 
o|substituted constant variable: g35897 
o|contracted procedure: k17634 
o|contracted procedure: k17610 
o|contracted procedure: k17615 
o|contracted procedure: k17640 
o|contracted procedure: k17643 
o|contracted procedure: k17664 
o|contracted procedure: k17667 
o|contracted procedure: k17685 
o|contracted procedure: k17728 
o|contracted procedure: k17724 
o|contracted procedure: k17702 
o|contracted procedure: k17720 
o|contracted procedure: k17716 
o|contracted procedure: k17706 
o|contracted procedure: k17757 
o|contracted procedure: k17753 
o|contracted procedure: k17749 
o|contracted procedure: k17761 
o|contracted procedure: k17767 
o|contracted procedure: k17800 
o|contracted procedure: k17776 
o|contracted procedure: k17796 
o|contracted procedure: k17782 
o|contracted procedure: k17806 
o|contracted procedure: k17823 
o|contracted procedure: k17831 
o|contracted procedure: k17834 
o|contracted procedure: k17851 
o|contracted procedure: k17865 
o|contracted procedure: k17883 
o|contracted procedure: k17879 
o|contracted procedure: k17875 
o|contracted procedure: k17905 
o|contracted procedure: k17914 
o|contracted procedure: k17930 
o|contracted procedure: k17934 
o|contracted procedure: k17945 
o|contracted procedure: k17951 
o|contracted procedure: k17968 
o|contracted procedure: k17974 
o|contracted procedure: k18021 
o|contracted procedure: k18027 
o|contracted procedure: k18050 
o|contracted procedure: k18056 
o|contracted procedure: k18062 
o|contracted procedure: k18083 
o|contracted procedure: k18089 
o|contracted procedure: k18111 
o|contracted procedure: k18120 
o|contracted procedure: k18127 
o|contracted procedure: k18147 
o|contracted procedure: k18169 
o|contracted procedure: k18160 
o|contracted procedure: k18176 
o|contracted procedure: k18183 
o|contracted procedure: k2052534014 
o|contracted procedure: k16245 
o|contracted procedure: k2052534019 
o|contracted procedure: k16275 
o|contracted procedure: k16290 
o|contracted procedure: k16286 
o|contracted procedure: k970834048 
o|contracted procedure: k15583 
o|substituted constant variable: g35918 
o|contracted procedure: k970834042 
o|contracted procedure: k15587 
o|substituted constant variable: g35916 
o|contracted procedure: k15602 
o|contracted procedure: k15609 
o|contracted procedure: k15624 
o|contracted procedure: k15639 
o|contracted procedure: k15648 
o|contracted procedure: k15657 
o|contracted procedure: k15669 
o|contracted procedure: k15672 
o|contracted procedure: k15675 
o|contracted procedure: k15683 
o|contracted procedure: k15691 
o|contracted procedure: k15700 
o|contracted procedure: k15709 
o|contracted procedure: k15718 
o|contracted procedure: k15730 
o|contracted procedure: k15733 
o|contracted procedure: k15736 
o|contracted procedure: k15744 
o|contracted procedure: k15752 
o|contracted procedure: k15761 
o|contracted procedure: k15770 
o|contracted procedure: k15802 
o|contracted procedure: k15811 
o|contracted procedure: k15817 
o|contracted procedure: k15824 
o|contracted procedure: k15828 
o|contracted procedure: k15840 
o|contracted procedure: k15843 
o|contracted procedure: k15846 
o|contracted procedure: k15854 
o|contracted procedure: k15862 
o|contracted procedure: k15880 
o|contracted procedure: k15884 
o|contracted procedure: k15896 
o|contracted procedure: k15899 
o|contracted procedure: k15902 
o|contracted procedure: k15910 
o|contracted procedure: k15918 
o|contracted procedure: k15926 
o|contracted procedure: k15932 
o|contracted procedure: k15960 
o|contracted procedure: k15966 
o|contracted procedure: k15979 
o|contracted procedure: k15994 
o|contracted procedure: k16016 
o|contracted procedure: k16012 
o|contracted procedure: k15997 
o|contracted procedure: k16000 
o|contracted procedure: k16008 
o|contracted procedure: k16025 
o|contracted procedure: k1401534034 
o|contracted procedure: k16312 
o|contracted procedure: k16319 
o|contracted procedure: k16328 
o|contracted procedure: k16345 
o|contracted procedure: k16341 
o|contracted procedure: k16331 
o|contracted procedure: k16351 
o|contracted procedure: k16362 
o|inlining procedure: k16365 
o|contracted procedure: k16373 
o|inlining procedure: k16365 
o|contracted procedure: k16379 
o|contracted procedure: k16385 
o|contracted procedure: k16391 
o|contracted procedure: k16397 
o|contracted procedure: k16403 
o|contracted procedure: k16432 
o|contracted procedure: k16438 
o|contracted procedure: k16444 
o|contracted procedure: k16450 
o|contracted procedure: k16476 
o|contracted procedure: k16485 
o|contracted procedure: k16492 
o|contracted procedure: k16501 
o|contracted procedure: k16517 
o|contracted procedure: k16507 
o|contracted procedure: k16525 
o|contracted procedure: k16535 
o|contracted procedure: k16541 
o|contracted procedure: k16547 
o|contracted procedure: k16553 
o|contracted procedure: k16580 
o|contracted procedure: k16598 
o|contracted procedure: k16614 
o|contracted procedure: k16604 
o|contracted procedure: k16625 
o|contracted procedure: k16632 
o|contracted procedure: k16641 
o|contracted procedure: k16648 
o|contracted procedure: k16656 
o|contracted procedure: k16666 
o|contracted procedure: k16672 
o|contracted procedure: k16678 
o|contracted procedure: k16684 
o|contracted procedure: k16720 
o|contracted procedure: k16727 
o|contracted procedure: k16730 
o|contracted procedure: k16745 
o|contracted procedure: k16754 
o|contracted procedure: k16761 
o|contracted procedure: k16769 
o|contracted procedure: k16779 
o|contracted procedure: k16785 
o|contracted procedure: k16791 
o|contracted procedure: k16797 
o|contracted procedure: k16837 
o|contracted procedure: k16844 
o|contracted procedure: k16847 
o|contracted procedure: k16862 
o|contracted procedure: k16871 
o|contracted procedure: k16886 
o|contracted procedure: k16896 
o|contracted procedure: k16902 
o|contracted procedure: k16908 
o|contracted procedure: k16914 
o|contracted procedure: k16948 
o|contracted procedure: k16960 
o|contracted procedure: k16971 
o|contracted procedure: k16993 
o|inlining procedure: k17000 
o|contracted procedure: k17010 
o|inlining procedure: k17000 
o|contracted procedure: k17019 
o|contracted procedure: k17026 
o|contracted procedure: k17030 
o|contracted procedure: k17036 
o|contracted procedure: k17047 
o|inlining procedure: k17000 
o|contracted procedure: k17054 
o|contracted procedure: k17060 
o|contracted procedure: k18196 
o|contracted procedure: k18210 
o|contracted procedure: k18202 
o|contracted procedure: k18219 
o|contracted procedure: k18233 
o|contracted procedure: k18225 
o|contracted procedure: k18360 
o|contracted procedure: k18242 
o|contracted procedure: k18245 
o|contracted procedure: k18251 
o|contracted procedure: k18254 
o|contracted procedure: k18269 
o|contracted procedure: k18272 
o|contracted procedure: k18283 
o|contracted procedure: k18289 
o|contracted procedure: k18300 
o|contracted procedure: k18303 
o|contracted procedure: k18315 
o|contracted procedure: k18318 
o|contracted procedure: k18321 
o|contracted procedure: k18329 
o|contracted procedure: k18337 
o|contracted procedure: k18700 
o|contracted procedure: k18423 
o|contracted procedure: k18426 
o|contracted procedure: k18432 
o|contracted procedure: k18435 
o|contracted procedure: k18446 
o|contracted procedure: k18454 
o|contracted procedure: k18457 
o|contracted procedure: k18468 
o|contracted procedure: k18480 
o|contracted procedure: k18476 
o|contracted procedure: k18472 
o|contracted procedure: k18484 
o|contracted procedure: k18490 
o|contracted procedure: k18505 
o|contracted procedure: k18501 
o|contracted procedure: k18509 
o|contracted procedure: k18515 
o|contracted procedure: k18529 
o|contracted procedure: k18532 
o|contracted procedure: k18539 
o|contracted procedure: k18545 
o|contracted procedure: k18548 
o|contracted procedure: k18555 
o|contracted procedure: k18572 
o|contracted procedure: k18578 
o|contracted procedure: k18584 
o|contracted procedure: k18590 
o|contracted procedure: k18596 
o|contracted procedure: k18602 
o|contracted procedure: k18608 
o|contracted procedure: k18614 
o|contracted procedure: k18620 
o|contracted procedure: k18626 
o|contracted procedure: k18632 
o|contracted procedure: k18638 
o|contracted procedure: k18644 
o|contracted procedure: k18709 
o|contracted procedure: k18716 
o|contracted procedure: k18720 
o|contracted procedure: k18741 
o|contracted procedure: k18748 
o|contracted procedure: k18799 
o|contracted procedure: k18757 
o|contracted procedure: k18795 
o|contracted procedure: k18763 
o|contracted procedure: k18788 
o|contracted procedure: k18780 
o|contracted procedure: k18807 
o|contracted procedure: k18814 
o|contracted procedure: k18818 
o|contracted procedure: k18830 
o|contracted procedure: k18833 
o|contracted procedure: k18836 
o|contracted procedure: k18844 
o|contracted procedure: k18852 
o|contracted procedure: k18860 
o|contracted procedure: k18866 
o|contracted procedure: k18872 
o|contracted procedure: k18878 
o|contracted procedure: k18884 
o|contracted procedure: k18890 
o|contracted procedure: k18995 
o|contracted procedure: k18925 
o|contracted procedure: k18928 
o|contracted procedure: k18934 
o|contracted procedure: k18947 
o|contracted procedure: k18957 
o|contracted procedure: k18950 
o|contracted procedure: k18971 
o|contracted procedure: k18964 
o|contracted procedure: k18981 
o|contracted procedure: k18987 
o|contracted procedure: k19006 
o|contracted procedure: k18999 
o|contracted procedure: k19024 
o|contracted procedure: k19031 
o|contracted procedure: k19034 
o|contracted procedure: k19042 
o|contracted procedure: k19046 
o|contracted procedure: k7561 
o|contracted procedure: k19058 
o|contracted procedure: k19178 
o|contracted procedure: k970834082 
o|contracted procedure: k19064 
o|substituted constant variable: g35938 
o|contracted procedure: k19084 
o|contracted procedure: k19073 
o|contracted procedure: k19080 
o|contracted procedure: k19076 
o|contracted procedure: k19088 
o|contracted procedure: k19098 
o|contracted procedure: k19101 
o|contracted procedure: k19104 
o|contracted procedure: k19107 
o|contracted procedure: k19142 
o|contracted procedure: k19131 
o|contracted procedure: k19138 
o|contracted procedure: k19134 
o|contracted procedure: k19148 
o|contracted procedure: k19168 
o|contracted procedure: k19173 
o|contracted procedure: k19659 
o|contracted procedure: k19662 
o|contracted procedure: k19665 
o|contracted procedure: k19668 
o|contracted procedure: k19812 
o|contracted procedure: k19808 
o|contracted procedure: k1964134076 
o|contracted procedure: k19671 
o|contracted procedure: k19698 
o|contracted procedure: k19723 
o|contracted procedure: k19704 
o|contracted procedure: k19711 
o|contracted procedure: k19701 
o|contracted procedure: k19737 
o|contracted procedure: k19740 
o|contracted procedure: k1964134066 
o|contracted procedure: k19756 
o|contracted procedure: k19787 
o|contracted procedure: k19762 
o|contracted procedure: k19780 
o|contracted procedure: k19771 
o|contracted procedure: k19791 
o|contracted procedure: k19185 
o|contracted procedure: k7579 
o|contracted procedure: k19194 
o|contracted procedure: k19197 
o|contracted procedure: k19203 
o|contracted procedure: k19209 
o|contracted procedure: k19297 
o|contracted procedure: k970834090 
o|contracted procedure: k19215 
o|substituted constant variable: g35944 
o|contracted procedure: k19247 
o|contracted procedure: k19236 
o|contracted procedure: k19243 
o|contracted procedure: k19239 
o|contracted procedure: k19253 
o|contracted procedure: k19260 
o|contracted procedure: k19273 
o|contracted procedure: k19280 
o|contracted procedure: k19287 
o|contracted procedure: k19377 
o|contracted procedure: k19307 
o|contracted procedure: k19310 
o|contracted procedure: k19316 
o|contracted procedure: k19329 
o|contracted procedure: k19339 
o|contracted procedure: k19332 
o|contracted procedure: k19353 
o|contracted procedure: k19346 
o|contracted procedure: k19363 
o|contracted procedure: k19369 
o|contracted procedure: k19388 
o|contracted procedure: k19381 
o|contracted procedure: k19400 
o|contracted procedure: k19406 
o|contracted procedure: k19443 
o|contracted procedure: k19418 
o|contracted procedure: k19439 
o|contracted procedure: k19421 
o|contracted procedure: k19428 
o|contracted procedure: k19432 
o|contracted procedure: k19424 
o|contracted procedure: k19460 
o|contracted procedure: k19471 
o|contracted procedure: k19475 
o|contracted procedure: k19479 
o|contracted procedure: k19486 
o|contracted procedure: k19489 
o|contracted procedure: k19495 
o|contracted procedure: k19501 
o|contracted procedure: k19507 
o|contracted procedure: k19529 
o|contracted procedure: k19535 
o|contracted procedure: k19542 
o|contracted procedure: k19587 
o|contracted procedure: k19548 
o|contracted procedure: k19555 
o|contracted procedure: k19572 
o|contracted procedure: k19583 
o|contracted procedure: k19600 
o|contracted procedure: k19611 
o|contracted procedure: k19820 
o|contracted procedure: k19823 
o|contracted procedure: k19826 
o|contracted procedure: k19836 
o|contracted procedure: k19845 
o|contracted procedure: k19852 
o|contracted procedure: k19861 
o|contracted procedure: k19869 
o|contracted procedure: k19881 
o|contracted procedure: k19891 
o|contracted procedure: k19895 
o|contracted procedure: k19937 
o|contracted procedure: k19940 
o|contracted procedure: k19943 
o|contracted procedure: k19946 
o|contracted procedure: k19949 
o|contracted procedure: k19952 
o|contracted procedure: k19955 
o|contracted procedure: k19961 
o|contracted procedure: k1964134121 
o|contracted procedure: k19964 
o|contracted procedure: k19967 
o|contracted procedure: k19980 
o|contracted procedure: k19984 
o|contracted procedure: k20005 
o|contracted procedure: k20036 
o|contracted procedure: k20044 
o|contracted procedure: k20040 
o|contracted procedure: k20047 
o|contracted procedure: k20053 
o|contracted procedure: k20069 
o|contracted procedure: k20062 
o|contracted procedure: k20072 
o|contracted procedure: k20094 
o|contracted procedure: k20102 
o|contracted procedure: k20098 
o|contracted procedure: k20075 
o|contracted procedure: k20082 
o|contracted procedure: k20090 
o|contracted procedure: k20086 
o|contracted procedure: k20105 
o|contracted procedure: k1964134109 
o|contracted procedure: k20114 
o|contracted procedure: k20117 
o|contracted procedure: k20141 
o|contracted procedure: k20165 
o|contracted procedure: k20149 
o|contracted procedure: k20161 
o|contracted procedure: k20152 
o|contracted procedure: k20168 
o|contracted procedure: k20171 
o|contracted procedure: k20182 
o|contracted procedure: k20187 
o|contracted procedure: k20209 
o|contracted procedure: k20190 
o|contracted procedure: k20205 
o|contracted procedure: k20193 
o|contracted procedure: k20196 
o|contracted procedure: k20213 
o|contracted procedure: k20219 
o|contracted procedure: k20231 
o|contracted procedure: k20241 
o|contracted procedure: k20245 
o|contracted procedure: k20254 
o|contracted procedure: k20257 
o|contracted procedure: k20260 
o|contracted procedure: k20268 
o|contracted procedure: k20276 
o|contracted procedure: k20288 
o|contracted procedure: k20298 
o|contracted procedure: k20302 
o|contracted procedure: k20305 
o|contracted procedure: k20345 
o|contracted procedure: k20353 
o|contracted procedure: k20349 
o|contracted procedure: k20326 
o|contracted procedure: k20333 
o|contracted procedure: k20341 
o|contracted procedure: k20337 
o|contracted procedure: k20374 
o|contracted procedure: k20382 
o|contracted procedure: k20378 
o|contracted procedure: k20416 
o|contracted procedure: k20391 
o|contracted procedure: k20409 
o|contracted procedure: k20400 
o|contracted procedure: k20420 
o|contracted procedure: k20423 
o|contracted procedure: k20444 
o|contracted procedure: k20431 
o|contracted procedure: k20440 
o|contracted procedure: k20480 
o|contracted procedure: k20447 
o|contracted procedure: k20450 
o|contracted procedure: k20462 
o|contracted procedure: k20472 
o|contracted procedure: k20476 
o|contracted procedure: k20502 
o|contracted procedure: k20510 
o|contracted procedure: k20506 
o|contracted procedure: k20483 
o|contracted procedure: k20490 
o|contracted procedure: k20498 
o|contracted procedure: k20494 
o|contracted procedure: k19918 
o|contracted procedure: k19921 
o|contracted procedure: k30314 
o|contracted procedure: k30932 
o|contracted procedure: k30928 
o|contracted procedure: k30322 
o|contracted procedure: k30330 
o|contracted procedure: k30338 
o|contracted procedure: k30346 
o|contracted procedure: k30354 
o|contracted procedure: k30362 
o|contracted procedure: k30370 
o|contracted procedure: k30378 
o|contracted procedure: k30386 
o|contracted procedure: k30394 
o|contracted procedure: k30402 
o|contracted procedure: k30410 
o|contracted procedure: k30418 
o|contracted procedure: k30426 
o|contracted procedure: k30434 
o|contracted procedure: k30442 
o|contracted procedure: k30450 
o|contracted procedure: k30458 
o|contracted procedure: k30466 
o|contracted procedure: k30474 
o|contracted procedure: k30482 
o|contracted procedure: k30490 
o|contracted procedure: k30498 
o|contracted procedure: k30506 
o|contracted procedure: k30514 
o|contracted procedure: k30924 
o|contracted procedure: k30522 
o|contracted procedure: k30530 
o|contracted procedure: k30538 
o|contracted procedure: k30542 
o|contracted procedure: k30916 
o|contracted procedure: k30920 
o|contracted procedure: k30546 
o|contracted procedure: k30912 
o|contracted procedure: k30908 
o|contracted procedure: k30550 
o|contracted procedure: k30554 
o|contracted procedure: k30904 
o|contracted procedure: k30558 
o|contracted procedure: k30900 
o|contracted procedure: k30562 
o|contracted procedure: k30896 
o|contracted procedure: k30566 
o|contracted procedure: k30570 
o|contracted procedure: k30574 
o|contracted procedure: k30892 
o|contracted procedure: k30884 
o|contracted procedure: k30888 
o|contracted procedure: k30578 
o|contracted procedure: k30880 
o|contracted procedure: k30856 
o|contracted procedure: k30860 
o|contracted procedure: k30876 
o|contracted procedure: k30864 
o|contracted procedure: k30872 
o|contracted procedure: k30868 
o|contracted procedure: k30582 
o|contracted procedure: k30852 
o|contracted procedure: k30586 
o|contracted procedure: k30848 
o|contracted procedure: k30590 
o|contracted procedure: k30844 
o|contracted procedure: k30816 
o|contracted procedure: k30832 
o|contracted procedure: k30840 
o|contracted procedure: k30836 
o|contracted procedure: k30820 
o|contracted procedure: k30828 
o|contracted procedure: k30824 
o|contracted procedure: k30594 
o|contracted procedure: k30808 
o|contracted procedure: k30812 
o|contracted procedure: k30800 
o|contracted procedure: k30804 
o|contracted procedure: k30598 
o|contracted procedure: k30796 
o|contracted procedure: k30602 
o|contracted procedure: k30792 
o|contracted procedure: k30784 
o|contracted procedure: k30788 
o|contracted procedure: k30780 
o|contracted procedure: k30606 
o|contracted procedure: k30770 
o|contracted procedure: k30762 
o|contracted procedure: k30766 
o|contracted procedure: k30610 
o|contracted procedure: k30758 
o|contracted procedure: k30614 
o|contracted procedure: k30742 
o|contracted procedure: k30750 
o|contracted procedure: k30754 
o|contracted procedure: k30746 
o|contracted procedure: k30618 
o|contracted procedure: k30622 
o|contracted procedure: k30738 
o|contracted procedure: k30626 
o|contracted procedure: k30734 
o|contracted procedure: k30630 
o|contracted procedure: k30730 
o|contracted procedure: k30726 
o|contracted procedure: k30634 
o|contracted procedure: k30722 
o|contracted procedure: k30638 
o|contracted procedure: k30718 
o|contracted procedure: k30642 
o|contracted procedure: k30646 
o|contracted procedure: k30714 
o|contracted procedure: k30650 
o|contracted procedure: k30710 
o|contracted procedure: k30654 
o|contracted procedure: k30662 
o|contracted procedure: k30666 
o|contracted procedure: k30706 
o|contracted procedure: k30670 
o|contracted procedure: k30702 
o|contracted procedure: k30678 
o|contracted procedure: k30698 
o|contracted procedure: k30682 
o|contracted procedure: k30694 
o|contracted procedure: k30690 
o|contracted procedure: k30686 
o|contracted procedure: k30674 
o|contracted procedure: k30658 
o|contracted procedure: k30534 
o|contracted procedure: k30526 
o|contracted procedure: k30518 
o|contracted procedure: k30510 
o|contracted procedure: k30502 
o|contracted procedure: k30494 
o|contracted procedure: k30486 
o|contracted procedure: k30478 
o|contracted procedure: k30470 
o|contracted procedure: k30462 
o|contracted procedure: k30454 
o|contracted procedure: k30446 
o|contracted procedure: k30438 
o|contracted procedure: k30430 
o|contracted procedure: k30422 
o|contracted procedure: k30414 
o|contracted procedure: k30406 
o|contracted procedure: k30398 
o|contracted procedure: k30390 
o|contracted procedure: k30382 
o|contracted procedure: k30374 
o|contracted procedure: k30366 
o|contracted procedure: k30358 
o|contracted procedure: k30350 
o|contracted procedure: k30342 
o|contracted procedure: k30334 
o|contracted procedure: k30326 
o|contracted procedure: k30318 
o|contracted procedure: k20516 
o|contracted procedure: k20587 
o|substituted constant variable: g35973 
o|contracted procedure: k20598 
o|contracted procedure: k20594 
o|contracted procedure: k20639 
o|contracted procedure: k20624 
o|contracted procedure: k20635 
o|contracted procedure: k20631 
o|contracted procedure: k20612 
o|contracted procedure: k20608 
o|contracted procedure: k20775 
o|contracted procedure: k20578 
o|contracted procedure: k20784 
o|contracted procedure: k20832 
o|contracted procedure: k20836 
o|contracted procedure: k20787 
o|contracted procedure: k20799 
o|contracted procedure: k20809 
o|substituted constant variable: g35975 
o|contracted procedure: k20822 
o|contracted procedure: k20812 
o|contracted procedure: k20828 
o|contracted procedure: k20853 
o|contracted procedure: k970834146 
o|contracted procedure: k20865 
o|substituted constant variable: g35980 
o|contracted procedure: k2626934135 
o|contracted procedure: k20876 
o|contracted procedure: k2626934130 
o|contracted procedure: k20880 
o|contracted procedure: k2626934140 
o|contracted procedure: k20887 
o|contracted procedure: k20893 
o|contracted procedure: k21655 
o|contracted procedure: k20899 
o|contracted procedure: k20902 
o|contracted procedure: k20926 
o|contracted procedure: k20940 
o|contracted procedure: k20950 
o|substituted constant variable: g35983 
o|contracted procedure: k20957 
o|contracted procedure: k20969 
o|contracted procedure: k20965 
o|contracted procedure: k20979 
o|contracted procedure: k20996 
o|contracted procedure: k20999 
o|inlining procedure: k21009 
o|contracted procedure: k21027 
o|contracted procedure: k21018 
o|inlining procedure: k21009 
o|contracted procedure: k21035 
o|contracted procedure: k21638 
o|contracted procedure: k21041 
o|contracted procedure: k970834154 
o|contracted procedure: k21056 
o|substituted constant variable: g35986 
o|contracted procedure: k21067 
o|contracted procedure: k21074 
o|contracted procedure: k21077 
o|contracted procedure: k21083 
o|contracted procedure: k21086 
o|contracted procedure: k21104 
o|contracted procedure: k21142 
o|contracted procedure: k21113 
o|contracted procedure: k21126 
o|contracted procedure: k21151 
o|contracted procedure: k970834160 
o|contracted procedure: k21172 
o|substituted constant variable: g35988 
o|contracted procedure: k21178 
o|contracted procedure: k21262 
o|contracted procedure: k21190 
o|contracted procedure: k21230 
o|contracted procedure: k21246 
o|contracted procedure: k21270 
o|contracted procedure: k21289 
o|contracted procedure: k21301 
o|contracted procedure: k21304 
o|contracted procedure: k21335 
o|contracted procedure: k21328 
o|contracted procedure: k21339 
o|contracted procedure: k21355 
o|contracted procedure: k21358 
o|contracted procedure: k21425 
o|contracted procedure: k21421 
o|contracted procedure: k21361 
o|contracted procedure: k21364 
o|contracted procedure: k21399 
o|contracted procedure: k21411 
o|contracted procedure: k21433 
o|contracted procedure: k21436 
o|contracted procedure: k21503 
o|contracted procedure: k21499 
o|contracted procedure: k21439 
o|contracted procedure: k21442 
o|contracted procedure: k21477 
o|contracted procedure: k21556 
o|contracted procedure: k21508 
o|contracted procedure: k21549 
o|contracted procedure: k21519 
o|contracted procedure: k21526 
o|contracted procedure: k21538 
o|contracted procedure: k21545 
o|contracted procedure: k21584 
o|contracted procedure: k21590 
o|contracted procedure: k21607 
o|contracted procedure: k21613 
o|contracted procedure: k21672 
o|contracted procedure: k21682 
o|substituted constant variable: g35991 
o|contracted procedure: k21695 
o|contracted procedure: k21685 
o|contracted procedure: k21704 
o|contracted procedure: k21700 
o|contracted procedure: k21708 
o|contracted procedure: k21712 
o|contracted procedure: k21822 
o|contracted procedure: k21721 
o|contracted procedure: k21731 
o|contracted procedure: k21724 
o|contracted procedure: k21727 
o|contracted procedure: k21818 
o|contracted procedure: k21737 
o|contracted procedure: k21740 
o|contracted procedure: k21746 
o|contracted procedure: k21760 
o|contracted procedure: k21768 
o|contracted procedure: k21764 
o|contracted procedure: k21789 
o|contracted procedure: k21795 
o|contracted procedure: k21876 
o|contracted procedure: k21885 
o|contracted procedure: k21888 
o|contracted procedure: k21891 
o|contracted procedure: k21899 
o|contracted procedure: k2052534172 
o|contracted procedure: k21914 
o|contracted procedure: k21922 
o|contracted procedure: k21937 
o|contracted procedure: k21946 
o|contracted procedure: k21951 
o|contracted procedure: k21955 
o|contracted procedure: k21976 
o|contracted procedure: k21961 
o|contracted procedure: k21965 
o|contracted procedure: k21972 
o|contracted procedure: k21979 
o|contracted procedure: k2052534177 
o|contracted procedure: k21989 
o|contracted procedure: k22002 
o|contracted procedure: k22012 
o|contracted procedure: k22016 
o|contracted procedure: k2186634185 
o|contracted procedure: k22038 
o|contracted procedure: k22019 
o|contracted procedure: k22023 
o|contracted procedure: k22034 
o|contracted procedure: k22030 
o|contracted procedure: k22289 
o|contracted procedure: k22301 
o|contracted procedure: k22304 
o|contracted procedure: k22571 
o|contracted procedure: k22371 
o|contracted procedure: k22564 
o|contracted procedure: k22560 
o|contracted procedure: k22556 
o|contracted procedure: k22552 
o|contracted procedure: k22386 
o|contracted procedure: k22393 
o|contracted procedure: k22397 
o|contracted procedure: k22406 
o|contracted procedure: k22426 
o|contracted procedure: k22422 
o|contracted procedure: k22409 
o|contracted procedure: k22593 
o|contracted procedure: k22618 
o|contracted procedure: k22643 
o|contracted procedure: k22635 
o|contracted procedure: k22639 
o|contracted procedure: k22631 
o|contracted procedure: k22648 
o|contracted procedure: k26295 
o|contracted procedure: k26321 
o|contracted procedure: k26298 
o|contracted procedure: k26317 
o|contracted procedure: k26305 
o|contracted procedure: k26313 
o|contracted procedure: k26309 
o|contracted procedure: k22657 
o|contracted procedure: k22614 
o|contracted procedure: k22602 
o|contracted procedure: k22669 
o|contracted procedure: k22672 
o|contracted procedure: k22675 
o|contracted procedure: k22683 
o|contracted procedure: k22691 
o|contracted procedure: k22698 
o|contracted procedure: k22707 
o|contracted procedure: k22714 
o|contracted procedure: k22718 
o|contracted procedure: k22730 
o|contracted procedure: k22726 
o|contracted procedure: k22722 
o|contracted procedure: k22545 
o|contracted procedure: k22438 
o|contracted procedure: k22462 
o|contracted procedure: k22465 
o|contracted procedure: k22480 
o|contracted procedure: k22472 
o|contracted procedure: k22476 
o|contracted procedure: k22489 
o|contracted procedure: k22495 
o|contracted procedure: k22504 
o|contracted procedure: k22520 
o|contracted procedure: k22524 
o|contracted procedure: k22516 
o|contracted procedure: k22512 
o|contracted procedure: k22508 
o|contracted procedure: k22534 
o|contracted procedure: k20667 
o|contracted procedure: k20663 
o|contracted procedure: k20659 
o|contracted procedure: k20655 
o|contracted procedure: k20645 
o|contracted procedure: k23379 
o|contracted procedure: k20696 
o|contracted procedure: k20692 
o|contracted procedure: k20673 
o|contracted procedure: k20684 
o|contracted procedure: k20688 
o|contracted procedure: k20680 
o|contracted procedure: k23388 
o|contracted procedure: k23226 
o|contracted procedure: k23229 
o|contracted procedure: k23241 
o|contracted procedure: k23365 
o|contracted procedure: k23259 
o|contracted procedure: k23268 
o|contracted procedure: k23275 
o|contracted procedure: k23282 
o|contracted procedure: k23286 
o|contracted procedure: k23295 
o|contracted procedure: k23301 
o|contracted procedure: k23308 
o|contracted procedure: k23315 
o|contracted procedure: k23321 
o|contracted procedure: k23328 
o|contracted procedure: k23335 
o|contracted procedure: k23339 
o|contracted procedure: k23346 
o|contracted procedure: k23353 
o|contracted procedure: k23343 
o|contracted procedure: k23373 
o|contracted procedure: k22279 
o|contracted procedure: k22283 
o|contracted procedure: k22218 
o|contracted procedure: k22271 
o|contracted procedure: k22275 
o|contracted procedure: k22224 
o|contracted procedure: k22227 
o|contracted procedure: k22236 
o|contracted procedure: k22267 
o|contracted procedure: k22255 
o|contracted procedure: k22263 
o|contracted procedure: k22259 
o|contracted procedure: k22245 
o|contracted procedure: k22541 
o|contracted procedure: k22764 
o|contracted procedure: k22771 
o|contracted procedure: k22782 
o|contracted procedure: k22789 
o|contracted procedure: k22914 
o|contracted procedure: k22799 
o|contracted procedure: k22809 
o|contracted procedure: k22910 
o|contracted procedure: k2627934226 
o|contracted procedure: k22749 
o|contracted procedure: k22745 
o|contracted procedure: k22812 
o|contracted procedure: k22826 
o|contracted procedure: k2627934216 
o|contracted procedure: k22841 
o|contracted procedure: k22848 
o|contracted procedure: k22867 
o|contracted procedure: k22863 
o|contracted procedure: k22874 
o|contracted procedure: k2627934221 
o|contracted procedure: k22884 
o|contracted procedure: k22877 
o|contracted procedure: k22888 
o|contracted procedure: k22892 
o|contracted procedure: k22904 
o|contracted procedure: k20561 
o|substituted constant variable: g36033 
o|contracted procedure: k20568 
o|contracted procedure: k2052534236 
o|contracted procedure: k20724 
o|contracted procedure: k20720 
o|contracted procedure: k20716 
o|contracted procedure: k20712 
o|contracted procedure: k20702 
o|contracted procedure: k23056 
o|contracted procedure: k20753 
o|contracted procedure: k20749 
o|contracted procedure: k20730 
o|contracted procedure: k20741 
o|contracted procedure: k20745 
o|contracted procedure: k20737 
o|contracted procedure: k23065 
o|contracted procedure: k22941 
o|contracted procedure: k22944 
o|contracted procedure: k22964 
o|contracted procedure: k22970 
o|contracted procedure: k23036 
o|contracted procedure: k22976 
o|contracted procedure: k22979 
o|contracted procedure: k22982 
o|contracted procedure: k22090 
o|contracted procedure: k22093 
o|contracted procedure: k22099 
o|contracted procedure: k2052534241 
o|contracted procedure: k22115 
o|contracted procedure: k22111 
o|contracted procedure: k23000 
o|contracted procedure: k23008 
o|contracted procedure: k23004 
o|contracted procedure: k22137 
o|contracted procedure: k22122 
o|contracted procedure: k22126 
o|contracted procedure: k22133 
o|contracted procedure: k2186634247 
o|contracted procedure: k22159 
o|contracted procedure: k22140 
o|contracted procedure: k22144 
o|contracted procedure: k22155 
o|contracted procedure: k22151 
o|contracted procedure: k22168 
o|contracted procedure: k22175 
o|contracted procedure: k22209 
o|contracted procedure: k22181 
o|contracted procedure: k22192 
o|contracted procedure: k22188 
o|contracted procedure: k22203 
o|contracted procedure: k22326 
o|contracted procedure: k22365 
o|contracted procedure: k22329 
o|contracted procedure: k22338 
o|contracted procedure: k22335 
o|contracted procedure: k22361 
o|contracted procedure: k22350 
o|contracted procedure: k22357 
o|contracted procedure: k23027 
o|contracted procedure: k23023 
o|contracted procedure: k2186634263 
o|contracted procedure: k22084 
o|contracted procedure: k22050 
o|contracted procedure: k22080 
o|contracted procedure: k22072 
o|contracted procedure: k2052534256 
o|contracted procedure: k22068 
o|contracted procedure: k22053 
o|contracted procedure: k22064 
o|contracted procedure: k22060 
o|contracted procedure: k22044 
o|contracted procedure: k2052534272 
o|contracted procedure: k23050 
o|contracted procedure: k23071 
o|contracted procedure: k23074 
o|contracted procedure: k23077 
o|contracted procedure: k23089 
o|contracted procedure: k23096 
o|contracted procedure: k23100 
o|contracted procedure: k23109 
o|contracted procedure: k23116 
o|contracted procedure: k23220 
o|contracted procedure: k23122 
o|contracted procedure: k23143 
o|contracted procedure: k23139 
o|contracted procedure: k23216 
o|contracted procedure: k23151 
o|contracted procedure: k23188 
o|contracted procedure: k23173 
o|contracted procedure: k23180 
o|contracted procedure: k23184 
o|contracted procedure: k23212 
o|contracted procedure: k23197 
o|contracted procedure: k23204 
o|contracted procedure: k23208 
o|contracted procedure: k26175 
o|contracted procedure: k26194 
o|contracted procedure: k26197 
o|contracted procedure: k26206 
o|contracted procedure: k26212 
o|contracted procedure: k26218 
o|contracted procedure: k26240 
o|contracted procedure: k26388 
o|contracted procedure: k26398 
o|substituted constant variable: g36069 
o|contracted procedure: k26405 
o|contracted procedure: k26429 
o|contracted procedure: k26413 
o|contracted procedure: k26425 
o|contracted procedure: k26421 
o|contracted procedure: k26417 
o|contracted procedure: k26409 
o|contracted procedure: k2626934289 
o|contracted procedure: k2724734284 
o|contracted procedure: k26484 
o|contracted procedure: k26924 
o|contracted procedure: k26498 
o|contracted procedure: k26513 
o|contracted procedure: k26519 
o|contracted procedure: k26543 
o|contracted procedure: k26558 
o|contracted procedure: k26561 
o|contracted procedure: k26564 
o|contracted procedure: k26576 
o|contracted procedure: k26579 
o|contracted procedure: k26582 
o|contracted procedure: k26590 
o|contracted procedure: k26598 
o|contracted procedure: k26605 
o|contracted procedure: k26611 
o|contracted procedure: k26622 
o|contracted procedure: k26625 
o|contracted procedure: k26628 
o|contracted procedure: k26640 
o|contracted procedure: k26643 
o|contracted procedure: k26646 
o|contracted procedure: k26654 
o|contracted procedure: k26662 
o|contracted procedure: k26669 
o|contracted procedure: k26675 
o|contracted procedure: k26692 
o|contracted procedure: k26695 
o|contracted procedure: k26698 
o|contracted procedure: k26710 
o|contracted procedure: k26713 
o|contracted procedure: k26716 
o|contracted procedure: k26724 
o|contracted procedure: k26732 
o|contracted procedure: k26739 
o|contracted procedure: k26745 
o|contracted procedure: k26439 
o|contracted procedure: k26448 
o|contracted procedure: k26455 
o|contracted procedure: k2625934301 
o|contracted procedure: k26463 
o|contracted procedure: k18375 
o|contracted procedure: k18414 
o|contracted procedure: k18384 
o|contracted procedure: k18408 
o|contracted procedure: k26766 
o|contracted procedure: k26777 
o|contracted procedure: k26780 
o|contracted procedure: k26783 
o|contracted procedure: k26795 
o|contracted procedure: k26798 
o|contracted procedure: k26801 
o|contracted procedure: k26809 
o|contracted procedure: k26817 
o|contracted procedure: k26824 
o|contracted procedure: k26830 
o|contracted procedure: k26845 
o|contracted procedure: k26879 
o|contracted procedure: k2625934317 
o|contracted procedure: k26889 
o|contracted procedure: k2625934323 
o|contracted procedure: k26898 
o|contracted procedure: k26905 
o|contracted procedure: k26908 
o|contracted procedure: k26918 
o|contracted procedure: k26961 
o|contracted procedure: k26967 
o|contracted procedure: k26973 
o|contracted procedure: k26976 
o|contracted procedure: k27035 
o|contracted procedure: k26994 
o|contracted procedure: k26997 
o|contracted procedure: k27011 
o|contracted procedure: k27020 
o|contracted procedure: k27026 
o|contracted procedure: k27072 
o|contracted procedure: k27076 
o|contracted procedure: k27066 
o|contracted procedure: k27050 
o|contracted procedure: k27057 
o|contracted procedure: k27099 
o|contracted procedure: k27120 
o|contracted procedure: k27138 
o|contracted procedure: k27141 
o|contracted procedure: k27237 
o|contracted procedure: k27149 
o|contracted procedure: k27233 
o|contracted procedure: k27154 
o|contracted procedure: k27163 
o|contracted procedure: k27170 
o|contracted procedure: k27175 
o|contracted procedure: k27185 
o|contracted procedure: k27191 
o|contracted procedure: k27198 
o|contracted procedure: k27215 
o|contracted procedure: k27294 
o|contracted procedure: k27307 
o|contracted procedure: k27321 
o|contracted procedure: k27324 
o|contracted procedure: k27445 
o|contracted procedure: k27449 
o|contracted procedure: k27330 
o|contracted procedure: k27339 
o|contracted procedure: k27345 
o|contracted procedure: k27426 
o|contracted procedure: k27356 
o|contracted procedure: k27378 
o|contracted procedure: k27390 
o|contracted procedure: k27258 
o|contracted procedure: k27397 
o|contracted procedure: k27408 
o|contracted procedure: k27270 
o|contracted procedure: k27417 
o|contracted procedure: k27490 
o|contracted procedure: k27493 
o|contracted procedure: k27503 
o|contracted procedure: k27506 
o|contracted procedure: k27586 
o|contracted procedure: k27590 
o|contracted procedure: k27512 
o|contracted procedure: k27523 
o|contracted procedure: k27467 
o|contracted procedure: k27566 
o|contracted procedure: k27458 
o|contracted procedure: k27531 
o|contracted procedure: k27557 
o|contracted procedure: k27538 
o|contracted procedure: k27549 
o|contracted procedure: k27542 
o|contracted procedure: k27546 
o|contracted procedure: k27622 
o|contracted procedure: k2625934346 
o|contracted procedure: k27660 
o|contracted procedure: k27672 
o|contracted procedure: k27676 
o|contracted procedure: k27684 
o|contracted procedure: k27715 
o|contracted procedure: k27734 
o|contracted procedure: k27724 
o|contracted procedure: k27941 
o|contracted procedure: k27749 
o|contracted procedure: k27752 
o|contracted procedure: k27761 
o|contracted procedure: k27764 
o|contracted procedure: k27773 
o|contracted procedure: k27785 
o|contracted procedure: k27797 
o|contracted procedure: k27809 
o|contracted procedure: k27820 
o|contracted procedure: k27866 
o|contracted procedure: k970834354 
o|contracted procedure: k27835 
o|substituted constant variable: g36102 
o|contracted procedure: k27844 
o|contracted procedure: k27851 
o|contracted procedure: k27855 
o|contracted procedure: k27862 
o|contracted procedure: k27877 
o|contracted procedure: k27870 
o|contracted procedure: k27891 
o|contracted procedure: k27884 
o|contracted procedure: k27907 
o|contracted procedure: k27913 
o|contracted procedure: k27920 
o|contracted procedure: k27898 
o|contracted procedure: k27931 
o|contracted procedure: k27925 
o|contracted procedure: k27952 
o|contracted procedure: k27945 
o|contracted procedure: k27980 
o|contracted procedure: k27973 
o|contracted procedure: k28161 
o|contracted procedure: k27992 
o|contracted procedure: k27995 
o|contracted procedure: k28001 
o|contracted procedure: k28007 
o|contracted procedure: k28019 
o|contracted procedure: k28031 
o|contracted procedure: k28043 
o|contracted procedure: k28054 
o|contracted procedure: k28096 
o|contracted procedure: k28069 
o|contracted procedure: k28072 
o|contracted procedure: k28082 
o|contracted procedure: k28092 
o|contracted procedure: k28121 
o|contracted procedure: k970834367 
o|contracted procedure: k28111 
o|substituted constant variable: g36110 
o|contracted procedure: k28125 
o|contracted procedure: k28131 
o|contracted procedure: k28151 
o|contracted procedure: k28139 
o|contracted procedure: k28145 
o|contracted procedure: k28190 
o|contracted procedure: k28183 
o|contracted procedure: k28209 
o|contracted procedure: k28225 
o|contracted procedure: k28239 
o|contracted procedure: k28250 
o|contracted procedure: k28258 
o|contracted procedure: k28269 
o|contracted procedure: k28262 
o|contracted procedure: k28290 
o|contracted procedure: k28301 
o|contracted procedure: k28307 
o|contracted procedure: k28325 
o|contracted procedure: k28335 
o|contracted procedure: k28345 
o|contracted procedure: k28342 
o|contracted procedure: k28362 
o|contracted procedure: k28355 
o|contracted procedure: k28377 
o|contracted procedure: k28384 
o|contracted procedure: k28393 
o|contracted procedure: k28408 
o|contracted procedure: k28417 
o|contracted procedure: k28428 
o|contracted procedure: k28435 
o|contracted procedure: k28444 
o|contracted procedure: k28483 
o|contracted procedure: k28490 
o|contracted procedure: k28452 
o|contracted procedure: k28462 
o|contracted procedure: k28473 
o|contracted procedure: k28496 
o|contracted procedure: k28530 
o|contracted procedure: k28537 
o|contracted procedure: k28499 
o|contracted procedure: k28508 
o|contracted procedure: k28520 
o|contracted procedure: k28540 
o|contracted procedure: k28556 
o|contracted procedure: k28560 
o|contracted procedure: k28552 
o|contracted procedure: k28587 
o|contracted procedure: k28628 
o|contracted procedure: k28621 
o|contracted procedure: k28814 
o|contracted procedure: k28637 
o|contracted procedure: k28690 
o|contracted procedure: k28701 
o|contracted procedure: k28652 
o|contracted procedure: k28669 
o|contracted procedure: k28680 
o|contracted procedure: k28720 
o|contracted procedure: k28784 
o|contracted procedure: k28726 
o|contracted procedure: k28738 
o|contracted procedure: k28745 
o|inlining procedure: k28735 
o|inlining procedure: k28735 
o|contracted procedure: k28760 
o|contracted procedure: k28764 
o|contracted procedure: k28778 
o|contracted procedure: k28787 
o|contracted procedure: k28806 
o|contracted procedure: k28797 
o|contracted procedure: k28825 
o|contracted procedure: k28818 
o|contracted procedure: k28832 
o|contracted procedure: k28868 
o|contracted procedure: k28839 
o|contracted procedure: k28848 
o|contracted procedure: k28865 
o|contracted procedure: k28854 
o|contracted procedure: k28892 
o|contracted procedure: k28898 
o|contracted procedure: k28917 
o|contracted procedure: k28913 
o|contracted procedure: k28926 
o|contracted procedure: k29055 
o|contracted procedure: k29193 
o|contracted procedure: k29061 
o|contracted procedure: k29077 
o|contracted procedure: k29089 
o|substituted constant variable: g36137 
o|contracted procedure: k28959 
o|contracted procedure: k28938 
o|inlining procedure: k29092 
o|inlining procedure: k29092 
o|contracted procedure: k29105 
o|contracted procedure: k29108 
o|contracted procedure: k29121 
o|contracted procedure: k29127 
o|inlining procedure: k29118 
o|inlining procedure: k29118 
o|contracted procedure: k29141 
o|contracted procedure: k29049 
o|contracted procedure: k28968 
o|inlining procedure: k29118 
o|contracted procedure: k28982 
o|contracted procedure: k29005 
o|contracted procedure: k29011 
o|contracted procedure: k29019 
o|contracted procedure: k29027 
o|contracted procedure: k28990 
o|contracted procedure: k29023 
o|contracted procedure: k29039 
o|contracted procedure: k29148 
o|contracted procedure: k29151 
o|contracted procedure: k29157 
o|contracted procedure: k29165 
o|contracted procedure: k29185 
o|contracted procedure: k29177 
o|contracted procedure: k29168 
o|contracted procedure: k29214 
o|contracted procedure: k29225 
o|contracted procedure: k29246 
o|contracted procedure: k29242 
o|contracted procedure: k29258 
o|contracted procedure: k29254 
o|contracted procedure: k29269 
o|contracted procedure: k29294 
o|contracted procedure: k29301 
o|contracted procedure: k29304 
o|contracted procedure: k29317 
o|contracted procedure: k29332 
o|contracted procedure: k29342 
o|contracted procedure: k29346 
o|contracted procedure: k29350 
o|contracted procedure: k29362 
o|contracted procedure: k29372 
o|contracted procedure: k29376 
o|contracted procedure: k29379 
o|contracted procedure: k29396 
o|contracted procedure: k29385 
o|contracted procedure: k29392 
o|contracted procedure: k29404 
o|contracted procedure: k29411 
o|contracted procedure: k29426 
o|contracted procedure: k29436 
o|contracted procedure: k29440 
o|contracted procedure: k29446 
o|contracted procedure: k29458 
o|contracted procedure: k29461 
o|contracted procedure: k29476 
o|contracted procedure: k29486 
o|contracted procedure: k29490 
o|contracted procedure: k29494 
o|contracted procedure: k29500 
o|contracted procedure: k29517 
o|contracted procedure: k29520 
o|contracted procedure: k29535 
o|contracted procedure: k29545 
o|contracted procedure: k29549 
o|contracted procedure: k29553 
o|contracted procedure: k29559 
o|contracted procedure: k29620 
o|contracted procedure: k29573 
o|contracted procedure: k29580 
o|contracted procedure: k29595 
o|contracted procedure: k29605 
o|contracted procedure: k29609 
o|contracted procedure: k29616 
o|contracted procedure: k29626 
o|contracted procedure: k29666 
o|contracted procedure: k29655 
o|contracted procedure: k29672 
o|contracted procedure: k29693 
o|contracted procedure: k29702 
o|contracted procedure: k29725 
o|contracted procedure: k29722 
o|contracted procedure: k29739 
o|contracted procedure: k29749 
o|contracted procedure: k29755 
o|contracted procedure: k29762 
o|contracted procedure: k29775 
o|contracted procedure: k29778 
o|contracted procedure: k29791 
o|contracted procedure: k29806 
o|contracted procedure: k29816 
o|contracted procedure: k29820 
o|contracted procedure: k29823 
o|contracted procedure: k29834 
o|contracted procedure: k29841 
o|contracted procedure: k29862 
o|contracted procedure: k29868 
o|contracted procedure: k29889 
o|contracted procedure: k29895 
o|contracted procedure: k29901 
o|contracted procedure: k29936 
o|contracted procedure: k29942 
o|contracted procedure: k29945 
o|contracted procedure: k29954 
o|contracted procedure: k29957 
o|contracted procedure: k29966 
o|contracted procedure: k29969 
o|contracted procedure: k30002 
o|contracted procedure: k30015 
o|contracted procedure: k30026 
o|contracted procedure: k30035 
o|contracted procedure: k30055 
o|contracted procedure: k30058 
o|contracted procedure: k30065 
o|contracted procedure: k30115 
o|contracted procedure: k30099 
o|contracted procedure: k30111 
o|contracted procedure: k30107 
o|contracted procedure: k30103 
o|contracted procedure: k30083 
o|contracted procedure: k30095 
o|contracted procedure: k30091 
o|contracted procedure: k30087 
o|contracted procedure: k30075 
o|contracted procedure: k30134 
o|contracted procedure: k30130 
o|contracted procedure: k30122 
o|contracted procedure: k30300 
o|contracted procedure: k30163 
o|contracted procedure: k30296 
o|contracted procedure: k30172 
o|contracted procedure: k30198 
o|contracted procedure: k30188 
o|contracted procedure: k30179 
o|contracted procedure: k30212 
o|contracted procedure: k30220 
o|contracted procedure: k30216 
o|contracted procedure: k30264 
o|contracted procedure: k30232 
o|contracted procedure: k30239 
o|contracted procedure: k30247 
o|contracted procedure: k30243 
o|contracted procedure: k30259 
o|contracted procedure: k30274 
o|contracted procedure: k30290 
o|contracted procedure: k30280 
o|contracted procedure: k30945 
o|contracted procedure: k30993 
o|contracted procedure: k30996 
o|simplifications: ((if . 14) (let . 673)) 
o|replaced variables: 210 
o|removed binding forms: 2711 
o|inlining procedure: "(irregex-core.scm:242) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:310) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:322) chicken.irregex#%irregex-match-end-index" 
(o x)|known list op on rest arg sublist: ##core#rest-car o931 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr o931 0 
(o x)|known list op on rest arg sublist: ##core#rest-car o1045 0 
o|contracted procedure: k9895 
o|inlining procedure: "(irregex-core.scm:758) chicken.irregex#char-altcase" 
o|contracted procedure: k10850 
o|contracted procedure: k13397 
o|contracted procedure: k13624 
o|inlining procedure: k8771 
o|inlining procedure: k8771 
o|inlining procedure: k8771 
o|inlining procedure: "(irregex-core.scm:3454) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:3528) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3531) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3524) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3526) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3560) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3562) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3556) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3558) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3575) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3576) chicken.irregex#char-alphanumeric?" 
o|inlining procedure: "(irregex-core.scm:3574) chicken.irregex#char-alphanumeric?" 
o|contracted procedure: k17336 
o|substituted constant variable: r1739536263 
o|inlining procedure: k17007 
o|inlining procedure: k19322 
o|inlining procedure: "(irregex-core.scm:2075) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:2095) chicken.irregex#%irregex-match-end-index" 
o|contracted procedure: k19829 
o|inlining procedure: "(irregex-core.scm:2486) chicken.irregex#char-altcase" 
o|inlining procedure: "(irregex-core.scm:2760) chicken.irregex#mst-state-mappings-set!" 
o|inlining procedure: k26288 
o|inlining procedure: k26288 
o|contracted procedure: k22758 
o|inlining procedure: "(irregex-core.scm:2799) chicken.irregex#mst-state-mappings-set!" 
o|inlining procedure: "(irregex-core.scm:2781) chicken.irregex#mst-state-mappings-set!" 
o|inlining procedure: "(irregex-core.scm:2781) chicken.irregex#mst-state-mappings-set!" 
o|inlining procedure: "(irregex-core.scm:3880) chicken.irregex#char-altcase" 
o|inlining procedure: "(irregex-core.scm:3879) chicken.irregex#char-altcase" 
o|inlining procedure: "(irregex-core.scm:3917) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:3951) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:3973) chicken.irregex#%irregex-match-end-index" 
o|inlining procedure: "(irregex-core.scm:4019) chicken.irregex#%irregex-match-end-index" 
(o x)|known list op on rest arg sublist: ##core#rest-car o5917 0 
o|inlining procedure: k29137 
o|simplifications: ((if . 3) (let . 6)) 
o|replaced variables: 7 
o|removed binding forms: 270 
o|removed side-effect free assignment to unused variable: chicken.irregex#%irregex-match-end-index 
o|removed side-effect free assignment to unused variable: chicken.irregex#char-alphanumeric? 
o|contracted procedure: k10638 
o|removed side-effect free assignment to unused variable: chicken.irregex#char-altcase 
o|substituted constant variable: r877236533 
o|substituted constant variable: n74936620 
o|substituted constant variable: n74936625 
o|removed side-effect free assignment to unused variable: chicken.irregex#mst-state-mappings-set! 
o|substituted constant variable: r2628936668 
o|contracted procedure: k22432 
o|substituted constant variable: n74936747 
o|contracted procedure: k28063 
o|substituted constant variable: n74936754 
o|substituted constant variable: n74936761 
o|substituted constant variable: n74936766 
o|simplifications: ((if . 2) (let . 9)) 
o|replaced variables: 74 
o|removed binding forms: 14 
o|removed conditional forms: 1 
o|inlining procedure: k8278 
o|inlining procedure: k8368 
o|inlining procedure: k10497 
o|inlining procedure: k25579 
o|inlining procedure: k25603 
o|inlining procedure: k25777 
o|inlining procedure: k25797 
o|inlining procedure: k25842 
o|inlining procedure: k19562 
o|inlining procedure: k27668 
o|inlining procedure: k28254 
o|inlining procedure: k28516 
o|removed binding forms: 76 
o|contracted procedure: k25068 
o|contracted procedure: k25832 
o|contracted procedure: k19450 
o|contracted procedure: k20859 
o|contracted procedure: k21943 
o|contracted procedure: k22105 
o|contracted procedure: k27664 
o|contracted procedure: k27823 
o|contracted procedure: k28057 
o|simplifications: ((let . 5)) 
o|removed binding forms: 21 
o|direct leaf routine/allocation: lp1131 0 
o|direct leaf routine/allocation: lp1179 0 
o|direct leaf routine/allocation: cased-char1457 9 
o|direct leaf routine/allocation: join1643 0 
o|direct leaf routine/allocation: new-res1644 6 
o|direct leaf routine/allocation: g18841885 6 
o|direct leaf routine/allocation: byte1955 0 
o|direct leaf routine/allocation: chicken.irregex#sre-repeater? 0 
o|direct leaf routine/allocation: chicken.irregex#sre-sequence 3 
o|direct leaf routine/allocation: chicken.irregex#sre-alternate 3 
o|direct leaf routine/allocation: g35143521 20 
o|direct leaf routine/allocation: g36693687 3 
o|direct leaf routine/allocation: g35853592 3 
o|direct leaf routine/allocation: chicken.irregex#nfa-get-epsilons 0 
o|direct leaf routine/allocation: g40874094 3 
o|direct leaf routine/allocation: doloop41874188 0 
o|direct leaf routine/allocation: chicken.irregex#char-ranges-union 6 
o|contracted procedure: k9133 
o|converted assignments to bindings: (lp1131) 
o|converted assignments to bindings: (lp1179) 
o|contracted procedure: "(irregex-core.scm:762) k10552" 
o|contracted procedure: "(irregex-core.scm:810) k10682" 
o|contracted procedure: "(irregex-core.scm:813) k10691" 
o|contracted procedure: "(irregex-core.scm:828) k10734" 
o|contracted procedure: "(irregex-core.scm:831) k10748" 
o|contracted procedure: "(irregex-core.scm:832) k10752" 
o|contracted procedure: "(irregex-core.scm:878) k11130" 
o|contracted procedure: "(irregex-core.scm:982) k11767" 
o|contracted procedure: "(irregex-core.scm:984) k11783" 
o|contracted procedure: "(irregex-core.scm:986) k11799" 
o|contracted procedure: "(irregex-core.scm:989) k11818" 
o|contracted procedure: "(irregex-core.scm:994) k11859" 
o|contracted procedure: "(irregex-core.scm:997) k11882" 
o|inlining procedure: "(irregex-core.scm:1251) k13530" 
o|contracted procedure: "(irregex-core.scm:1362) k14081" 
o|contracted procedure: "(irregex-core.scm:1363) k14069" 
o|contracted procedure: "(irregex-core.scm:1366) k14134" 
o|contracted procedure: "(irregex-core.scm:1367) k14122" 
o|contracted procedure: "(irregex-core.scm:1368) k14110" 
o|contracted procedure: "(irregex-core.scm:1371) k14203" 
o|contracted procedure: "(irregex-core.scm:1372) k14191" 
o|contracted procedure: "(irregex-core.scm:1373) k14179" 
o|contracted procedure: "(irregex-core.scm:1374) k14167" 
o|contracted procedure: "(irregex-core.scm:1443) k14638" 
o|contracted procedure: "(irregex-core.scm:1455) k14766" 
o|contracted procedure: "(irregex-core.scm:1462) k14800" 
o|contracted procedure: "(irregex-core.scm:1471) k14905" 
o|contracted procedure: "(irregex-core.scm:1478) k14939" 
o|contracted procedure: "(irregex-core.scm:1486) k14497" 
o|inlining procedure: "(irregex-core.scm:1419) k15537" 
o|contracted procedure: "(irregex-core.scm:1489) k15295" 
o|contracted procedure: "(irregex-core.scm:1506) k15157" 
o|contracted procedure: "(irregex-core.scm:1519) k14582" 
o|contracted procedure: "(irregex-core.scm:3156) k23551" 
o|contracted procedure: "(irregex-core.scm:3165) k23590" 
o|contracted procedure: "(irregex-core.scm:3170) k23609" 
o|contracted procedure: "(irregex-core.scm:3175) k23628" 
o|contracted procedure: "(irregex-core.scm:3177) k23647" 
o|contracted procedure: "(irregex-core.scm:3183) k23698" 
o|contracted procedure: "(irregex-core.scm:3189) k23744" 
o|contracted procedure: "(irregex-core.scm:3194) k23770" 
o|contracted procedure: "(irregex-core.scm:3203) k23809" 
o|contracted procedure: "(irregex-core.scm:3200) k23815" 
o|contracted procedure: "(irregex-core.scm:3215) k23851" 
o|contracted procedure: "(irregex-core.scm:3210) k23869" 
o|contracted procedure: "(irregex-core.scm:3234) k23894" 
o|contracted procedure: "(irregex-core.scm:3229) k23912" 
o|contracted procedure: "(irregex-core.scm:3258) k24001" 
o|contracted procedure: "(irregex-core.scm:3292) k24153" 
o|contracted procedure: "(irregex-core.scm:3325) k24392" 
o|contracted procedure: "(irregex-core.scm:3335) k24427" 
o|contracted procedure: "(irregex-core.scm:3345) k24505" 
o|contracted procedure: "(irregex-core.scm:3362) k24548" 
o|contracted procedure: "(irregex-core.scm:3444) k24911" 
o|contracted procedure: "(irregex-core.scm:3447) k25038" 
o|contracted procedure: "(irregex-core.scm:3626) k26136" 
o|contracted procedure: "(irregex-core.scm:1786) k17467" 
o|contracted procedure: "(irregex-core.scm:1810) k17653" 
o|contracted procedure: "(irregex-core.scm:1821) k17692" 
o|contracted procedure: "(irregex-core.scm:1825) k17739" 
o|contracted procedure: "(irregex-core.scm:1829) k17813" 
o|contracted procedure: "(irregex-core.scm:1833) k17841" 
o|contracted procedure: "(irregex-core.scm:1546) k15616" 
o|contracted procedure: "(irregex-core.scm:1547) k15631" 
o|contracted procedure: "(irregex-core.scm:1562) k15805" 
o|contracted procedure: "(irregex-core.scm:1688) k16737" 
o|contracted procedure: "(irregex-core.scm:1699) k16854" 
o|contracted procedure: "(irregex-core.scm:1888) k18261" 
o|contracted procedure: "(irregex-core.scm:1889) k18279" 
o|contracted procedure: "(irregex-core.scm:1890) k18296" 
o|contracted procedure: "(irregex-core.scm:1910) k18442" 
o|contracted procedure: "(irregex-core.scm:1912) k18464" 
o|contracted procedure: "(irregex-core.scm:1916) k18497" 
o|contracted procedure: "(irregex-core.scm:2153) k19884" 
o|contracted procedure: "(irregex-core.scm:2237) k20291" 
o|contracted procedure: "(irregex-core.scm:2189) k20465" 
o|contracted procedure: "(irregex-core.scm:2421) k20618" 
o|contracted procedure: "(irregex-core.scm:2560) k21254" 
o|contracted procedure: "(irregex-core.scm:2633) k21407" 
o|contracted procedure: "(irregex-core.scm:2646) k21485" 
o|contracted procedure: "(irregex-core.scm:2762) k22005" 
o|contracted procedure: "(irregex-core.scm:3013) k22953" 
o|contracted procedure: k22332 
o|converted assignments to bindings: (doloop41874188) 
o|contracted procedure: "(irregex-core.scm:3732) k26837" 
o|contracted procedure: "(irregex-core.scm:3734) k26852" 
o|contracted procedure: "(irregex-core.scm:3795) k27204" 
o|contracted procedure: "(irregex-core.scm:3799) k27219" 
o|simplifications: ((if . 1) (let . 3)) 
o|removed binding forms: 86 
o|contracted procedure: "(irregex-core.scm:1788) k17495" 
o|replaced variables: 10 
o|removed binding forms: 1 
o|simplifications: ((if . 6)) 
o|removed binding forms: 10 
o|contracted procedure: k24015 
o|contracted procedure: k24056 
o|contracted procedure: k24189 
o|contracted procedure: k24215 
o|removed binding forms: 4 
o|direct leaf routine/allocation: shift1522 6 
o|direct leaf routine/allocation: chicken.irregex#nfa-add-epsilon! 6 
o|contracted procedure: "(irregex-core.scm:813) k10787" 
o|contracted procedure: "(irregex-core.scm:835) k10802" 
o|contracted procedure: "(irregex-core.scm:2498) k20914" 
o|contracted procedure: "(irregex-core.scm:2572) k21212" 
o|contracted procedure: "(irregex-core.scm:2573) k21215" 
o|contracted procedure: "(irregex-core.scm:2582) k21282" 
o|contracted procedure: "(irregex-core.scm:2597) k21322" 
o|inlining procedure: "(irregex-core.scm:2599) k21319" 
o|contracted procedure: "(irregex-core.scm:2637) k21385" 
o|contracted procedure: "(irregex-core.scm:2638) k21382" 
o|contracted procedure: "(irregex-core.scm:2650) k21463" 
o|contracted procedure: "(irregex-core.scm:2651) k21460" 
o|removed binding forms: 11 
o|contracted procedure: "(irregex-core.scm:2599) k21319" 
o|removed binding forms: 1 
o|customizable procedures: (k30205 loop26289 loop6274 k29562 k29696 for-each-loop62266236 k29705 k29632 chicken.irregex#cset->string for-each-loop61716181 g61346141 for-each-loop61336145 for-each-loop61126122 for-each-loop60936103 k29310 for-each-loop60766086 lp6034 for-each-loop60576067 lp6016 lp15983 k29008 map-loop59575975 lp25986 loop5928 scan5919 k28640 k28711 lp5696 g56585659 lp5621 chicken.irregex#irregex-apply-match lp5558 chicken.irregex#irregex-copy-matches k27755 k27758 chicken.irregex#irregex-reset-matches! lp5504 k27635 lp5471 intersect5452 k27364 k27367 k27374 diff5423 union-range5398 lp5379 map-loop53315348 lp3090 lp5222 map-loop53045321 map-loop52765293 rec5240 map-loop52505267 lp5233 lp5216 k23125 lp24373 lp4370 k23014 g43424343 k22990 lp4134 k22102 k22108 lp24328 lp4322 chicken.irregex#mst-fold g43074308 chicken.irregex#cset-difference chicken.irregex#mst-copy k22832 lp4295 chicken.irregex#nfa-state->mst chicken.irregex#nfa-epsilon-closure lp4163 k23247 lp34404 lp24400 lp4398 k22486 chicken.irregex#tag-set-commands-for-closure lp24220 lp4209 lp4239 g42584267 map-loop42524274 doloop42444245 k22627 lp4175 k21940 for-each-loop40864105 chicken.irregex#make-mst chicken.irregex#mst-add! k21749 lp3844 doloop40294030 k21107 k21154 g40194020 chicken.irregex#string->cset add-char-state!3878 lp23910 lp3870 extend-state!3877 new-state-number3875 add-state!3876 doloop38833884 k20781 doloop35443545 k19970 for-each-loop35843596 k20359 k20317 for-each-loop36683692 g37083717 map-loop37023724 for-each-loop36783731 lp23610 k20008 chicken.irregex#finalize! lp13601 for-each-loop35133534 k19526 lp3419 k19313 lp3335 lp13470 chicken.irregex#find chicken.irregex#cset-contains? lp23477 lp23301 lp13297 chicken.irregex#dfa-match/longest chicken.irregex#irregex-new-matches chicken.irregex#irregex-search/matches k18931 k18751 map-loop32053222 k18518 chicken.irregex#sre-sequence-names map-loop30643081 k17013 k16865 k16748 k16635 k16495 k16322 k16365 chicken.irregex#every k15972 map-loop25042521 k15764 map-loop24702487 rec2364 map-loop24442461 chicken.irregex#any chicken.irregex#cset->utf8-pattern g24072416 map-loop24012419 g23802389 map-loop23742392 adjust2356 g26142615 chicken.irregex#sre->nfa g26252626 chicken.irregex#nfa->dfa k17135 k17164 k17488 k17868 k17908 g30273028 k17887 k17673 k17676 k17546 k17352 lp22897 lp22885 lp2845 grow2855 chicken.irregex#sre-names lp5129 lp5105 k25817 chicken.irregex#chunker-next-char k25731 k25722 k25657 k25624 k25526 k25517 chicken.irregex#chunker-prev-char k23486 k24721 lp4844 k24577 k24137 k24140 k23985 k23988 chicken.irregex#sre-count-submatches rec4438 chicken.irregex#sre-cset->procedure lp4429 chicken.irregex#cset->plist k15522 chicken.irregex#char->utf8-list map-loop23152336 chicken.irregex#unicode-range-up-from lp2033 map-loop21842205 g22202229 map-loop22142265 map-loop22772298 chicken.irregex#utf8-lowest-digit-of-length k15060 chicken.irregex#zero-to map-loop22372258 chicken.irregex#unicode-range-up-to lp2345 map-loop21482165 map-loop21132130 map-loop20512068 map-loop20772098 chicken.irregex#unicode-range-helper scan1059 chicken.irregex#symbol-list->flags scan1071 lp21773 k12693 k12043 lp1082 collect1090 k12086 k13916 k13799 k13686 chicken.irregex#sre->cset k13475 k13552 k13564 chicken.irregex#string-parse-hex-escape chicken.irregex#cset-case-insensitive chicken.irregex#cset-complement go1850 lp21635 chicken.irregex#with-read-from-string k11250 save1462 collect/single1460 k10987 lp1451 collect/terms1461 collect1459 chicken.irregex#last k10660 k10663 lp1514 lp1977 chicken.irregex#utf8-string-ref k10597 cased-string1458 map-loop14711488 k10365 k10354 lp1416 k10248 scan1386 scan1349 collect1353 k9962 skip1331 lp1317 read1304 lp1208 lp1199 lp1186 lp1160 lp1153 chicken.irregex#fold scan1047 lp983 k8410 loop955 chicken.irregex#string-cat-reverse lp866 k8020 chicken.irregex#irregex-match-numeric-index chicken.irregex#vector-copy) 
o|calls to known targets: 1141 
o|identified direct recursive calls: f_8066 2 
o|identified direct recursive calls: f_8830 1 
o|identified direct recursive calls: f_9137 1 
o|identified direct recursive calls: f_9174 1 
o|identified direct recursive calls: f_9271 1 
o|identified direct recursive calls: f_9887 1 
o|identified direct recursive calls: f_10021 1 
o|identified direct recursive calls: f_9746 4 
o|identified direct recursive calls: f_10529 1 
o|identified direct recursive calls: f_14224 1 
o|identified direct recursive calls: f_10673 2 
o|identified direct recursive calls: f_11709 5 
o|identified direct recursive calls: f_8968 1 
o|identified direct recursive calls: f_12775 2 
o|identified direct recursive calls: f_8930 1 
o|identified direct recursive calls: f_10482 3 
o|identified direct recursive calls: f_8873 1 
o|identified direct recursive calls: f_14680 1 
o|identified direct recursive calls: f_14716 1 
o|identified direct recursive calls: f_14837 1 
o|identified direct recursive calls: f_14976 1 
o|identified direct recursive calls: f_15082 1 
o|identified direct recursive calls: f_15185 1 
o|identified direct recursive calls: f_15323 1 
o|identified direct recursive calls: f_15414 1 
o|identified direct recursive calls: f_14508 1 
o|identified direct recursive calls: f_17094 1 
o|identified direct recursive calls: f_15989 1 
o|identified direct recursive calls: f_15591 2 
o|identified direct recursive calls: f_16575 1 
o|identified direct recursive calls: f_18237 3 
o|identified direct recursive calls: f_18418 3 
o|identified direct recursive calls: f_19876 1 
o|identified direct recursive calls: f_20283 1 
o|identified direct recursive calls: f_20457 1 
o|identified direct recursive calls: f_20804 1 
o|identified direct recursive calls: f_20766 4 
o|identified direct recursive calls: f_21677 1 
o|identified direct recursive calls: f_21716 1 
o|identified direct recursive calls: f_21997 1 
o|identified direct recursive calls: f_22296 1 
o|identified direct recursive calls: f_22702 1 
o|identified direct recursive calls: f_23290 2 
o|identified direct recursive calls: f_22234 1 
o|identified direct recursive calls: f_22401 1 
o|identified direct recursive calls: f_22163 1 
o|identified direct recursive calls: f_22345 1 
o|identified direct recursive calls: f_22959 1 
o|identified direct recursive calls: f_26393 1 
o|identified direct recursive calls: f_18370 1 
o|identified direct recursive calls: f_26502 2 
o|identified direct recursive calls: f_26992 2 
o|identified direct recursive calls: f_27094 4 
o|identified direct recursive calls: f_27289 3 
o|identified direct recursive calls: f_27485 3 
o|identified direct recursive calls: f_28372 1 
o|identified direct recursive calls: f_28715 2 
o|identified direct recursive calls: f_28843 1 
o|identified direct recursive calls: f_29084 2 
o|identified direct recursive calls: f_29072 1 
o|identified direct recursive calls: f_30050 4 
o|fast box initializations: 141 
o|fast global references: 362 
o|fast global assignments: 77 
o|dropping unused closure argument: f_10332 
o|dropping unused closure argument: f_13295 
o|dropping unused closure argument: f_14029 
o|dropping unused closure argument: f_14262 
o|dropping unused closure argument: f_14338 
o|dropping unused closure argument: f_14614 
o|dropping unused closure argument: f_14752 
o|dropping unused closure argument: f_14891 
o|dropping unused closure argument: f_15469 
o|dropping unused closure argument: f_16575 
o|dropping unused closure argument: f_16982 
o|dropping unused closure argument: f_18191 
o|dropping unused closure argument: f_18214 
o|dropping unused closure argument: f_18418 
o|dropping unused closure argument: f_18704 
o|dropping unused closure argument: f_19053 
o|dropping unused closure argument: f_19816 
o|dropping unused closure argument: f_19935 
o|dropping unused closure argument: f_20582 
o|dropping unused closure argument: f_20616 
o|dropping unused closure argument: f_20757 
o|dropping unused closure argument: f_21880 
o|dropping unused closure argument: f_21909 
o|dropping unused closure argument: f_21926 
o|dropping unused closure argument: f_21935 
o|dropping unused closure argument: f_22287 
o|dropping unused closure argument: f_22369 
o|dropping unused closure argument: f_23054 
o|dropping unused closure argument: f_23069 
o|dropping unused closure argument: f_26168 
o|dropping unused closure argument: f_26381 
o|dropping unused closure argument: f_26472 
o|dropping unused closure argument: f_26492 
o|dropping unused closure argument: f_26959 
o|dropping unused closure argument: f_27044 
o|dropping unused closure argument: f_27275 
o|dropping unused closure argument: f_27597 
o|dropping unused closure argument: f_27607 
o|dropping unused closure argument: f_28366 
o|dropping unused closure argument: f_29197 
o|dropping unused closure argument: f_7540 
o|dropping unused closure argument: f_7622 
o|dropping unused closure argument: f_7645 
o|dropping unused closure argument: f_7654 
o|dropping unused closure argument: f_7954 
o|dropping unused closure argument: f_8640 
o|dropping unused closure argument: f_8670 
o|dropping unused closure argument: f_9108 
o|dropping unused closure argument: f_9158 
o|dropping unused closure argument: f_9195 
o|dropping unused closure argument: f_9256 
o|dropping unused closure argument: f_9271 
o|dropping unused closure argument: f_9294 
o|dropping unused closure argument: f_9337 
o|dropping unused closure argument: f_9382 
o|dropping unused closure argument: f_9739 
*/
/* end of file */
