/* Generated from optimizer.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: optimizer.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -no-lambda-info -no-trace -emit-import-library chicken.compiler.optimizer -output-file optimizer.c
   unit: optimizer
   uses: eval data-structures internal support library
*/
#include "chicken.h"

static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_eval_toplevel)
C_externimport void C_ccall C_eval_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_data_2dstructures_toplevel)
C_externimport void C_ccall C_data_2dstructures_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_internal_toplevel)
C_externimport void C_ccall C_internal_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_support_toplevel)
C_externimport void C_ccall C_support_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[245];
static double C_possibly_force_alignment;


C_noret_decl(f_10000)
static void C_ccall f_10000(C_word c,C_word *av) C_noret;
C_noret_decl(f_10015)
static void C_ccall f_10015(C_word c,C_word *av) C_noret;
C_noret_decl(f_10019)
static void C_ccall f_10019(C_word c,C_word *av) C_noret;
C_noret_decl(f_10029)
static void C_ccall f_10029(C_word c,C_word *av) C_noret;
C_noret_decl(f_10041)
static void C_ccall f_10041(C_word c,C_word *av) C_noret;
C_noret_decl(f_10053)
static void C_ccall f_10053(C_word c,C_word *av) C_noret;
C_noret_decl(f_10057)
static void C_ccall f_10057(C_word c,C_word *av) C_noret;
C_noret_decl(f_10099)
static void C_ccall f_10099(C_word c,C_word *av) C_noret;
C_noret_decl(f_10114)
static void C_ccall f_10114(C_word c,C_word *av) C_noret;
C_noret_decl(f_10122)
static void C_ccall f_10122(C_word c,C_word *av) C_noret;
C_noret_decl(f_10124)
static void C_ccall f_10124(C_word c,C_word *av) C_noret;
C_noret_decl(f_10132)
static void C_ccall f_10132(C_word c,C_word *av) C_noret;
C_noret_decl(f_10144)
static void C_ccall f_10144(C_word c,C_word *av) C_noret;
C_noret_decl(f_10171)
static void C_ccall f_10171(C_word c,C_word *av) C_noret;
C_noret_decl(f_10186)
static void C_ccall f_10186(C_word c,C_word *av) C_noret;
C_noret_decl(f_10207)
static void C_ccall f_10207(C_word c,C_word *av) C_noret;
C_noret_decl(f_10243)
static void C_ccall f_10243(C_word c,C_word *av) C_noret;
C_noret_decl(f_10245)
static void C_ccall f_10245(C_word c,C_word *av) C_noret;
C_noret_decl(f_10280)
static void C_ccall f_10280(C_word c,C_word *av) C_noret;
C_noret_decl(f_10302)
static void C_ccall f_10302(C_word c,C_word *av) C_noret;
C_noret_decl(f_10306)
static void C_ccall f_10306(C_word c,C_word *av) C_noret;
C_noret_decl(f_10342)
static void C_ccall f_10342(C_word c,C_word *av) C_noret;
C_noret_decl(f_10366)
static void C_ccall f_10366(C_word c,C_word *av) C_noret;
C_noret_decl(f_10406)
static void C_ccall f_10406(C_word c,C_word *av) C_noret;
C_noret_decl(f_10426)
static void C_ccall f_10426(C_word c,C_word *av) C_noret;
C_noret_decl(f_10430)
static void C_ccall f_10430(C_word c,C_word *av) C_noret;
C_noret_decl(f_10434)
static void C_ccall f_10434(C_word c,C_word *av) C_noret;
C_noret_decl(f_10436)
static void C_ccall f_10436(C_word c,C_word *av) C_noret;
C_noret_decl(f_10442)
static void C_ccall f_10442(C_word c,C_word *av) C_noret;
C_noret_decl(f_10450)
static void C_ccall f_10450(C_word c,C_word *av) C_noret;
C_noret_decl(f_10456)
static void C_fcall f_10456(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10476)
static void C_ccall f_10476(C_word c,C_word *av) C_noret;
C_noret_decl(f_10480)
static void C_ccall f_10480(C_word c,C_word *av) C_noret;
C_noret_decl(f_10503)
static void C_ccall f_10503(C_word c,C_word *av) C_noret;
C_noret_decl(f_10576)
static void C_ccall f_10576(C_word c,C_word *av) C_noret;
C_noret_decl(f_10579)
static void C_fcall f_10579(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_10583)
static void C_ccall f_10583(C_word c,C_word *av) C_noret;
C_noret_decl(f_10586)
static void C_ccall f_10586(C_word c,C_word *av) C_noret;
C_noret_decl(f_10589)
static void C_ccall f_10589(C_word c,C_word *av) C_noret;
C_noret_decl(f_10604)
static void C_ccall f_10604(C_word c,C_word *av) C_noret;
C_noret_decl(f_10635)
static void C_ccall f_10635(C_word c,C_word *av) C_noret;
C_noret_decl(f_10641)
static void C_ccall f_10641(C_word c,C_word *av) C_noret;
C_noret_decl(f_10647)
static void C_ccall f_10647(C_word c,C_word *av) C_noret;
C_noret_decl(f_10695)
static void C_ccall f_10695(C_word c,C_word *av) C_noret;
C_noret_decl(f_10703)
static void C_ccall f_10703(C_word c,C_word *av) C_noret;
C_noret_decl(f_10707)
static void C_ccall f_10707(C_word c,C_word *av) C_noret;
C_noret_decl(f_10733)
static void C_ccall f_10733(C_word c,C_word *av) C_noret;
C_noret_decl(f_10749)
static void C_fcall f_10749(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10761)
static void C_fcall f_10761(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10771)
static void C_ccall f_10771(C_word c,C_word *av) C_noret;
C_noret_decl(f_10790)
static void C_fcall f_10790(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_10793)
static void C_fcall f_10793(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_10797)
static void C_ccall f_10797(C_word c,C_word *av) C_noret;
C_noret_decl(f_10800)
static void C_ccall f_10800(C_word c,C_word *av) C_noret;
C_noret_decl(f_10803)
static void C_ccall f_10803(C_word c,C_word *av) C_noret;
C_noret_decl(f_10842)
static void C_ccall f_10842(C_word c,C_word *av) C_noret;
C_noret_decl(f_10860)
static void C_ccall f_10860(C_word c,C_word *av) C_noret;
C_noret_decl(f_10876)
static void C_ccall f_10876(C_word c,C_word *av) C_noret;
C_noret_decl(f_10897)
static void C_ccall f_10897(C_word c,C_word *av) C_noret;
C_noret_decl(f_10931)
static void C_ccall f_10931(C_word c,C_word *av) C_noret;
C_noret_decl(f_10947)
static void C_ccall f_10947(C_word c,C_word *av) C_noret;
C_noret_decl(f_10951)
static void C_fcall f_10951(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10956)
static void C_ccall f_10956(C_word c,C_word *av) C_noret;
C_noret_decl(f_10986)
static void C_ccall f_10986(C_word c,C_word *av) C_noret;
C_noret_decl(f_10990)
static void C_fcall f_10990(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10995)
static void C_ccall f_10995(C_word c,C_word *av) C_noret;
C_noret_decl(f_11037)
static void C_fcall f_11037(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11042)
static void C_ccall f_11042(C_word c,C_word *av) C_noret;
C_noret_decl(f_11079)
static void C_ccall f_11079(C_word c,C_word *av) C_noret;
C_noret_decl(f_11083)
static void C_ccall f_11083(C_word c,C_word *av) C_noret;
C_noret_decl(f_11090)
static void C_ccall f_11090(C_word c,C_word *av) C_noret;
C_noret_decl(f_11094)
static void C_ccall f_11094(C_word c,C_word *av) C_noret;
C_noret_decl(f_11122)
static void C_ccall f_11122(C_word c,C_word *av) C_noret;
C_noret_decl(f_11155)
static void C_ccall f_11155(C_word c,C_word *av) C_noret;
C_noret_decl(f_11166)
static void C_ccall f_11166(C_word c,C_word *av) C_noret;
C_noret_decl(f_11179)
static void C_ccall f_11179(C_word c,C_word *av) C_noret;
C_noret_decl(f_11212)
static void C_ccall f_11212(C_word c,C_word *av) C_noret;
C_noret_decl(f_11219)
static void C_ccall f_11219(C_word c,C_word *av) C_noret;
C_noret_decl(f_11221)
static void C_fcall f_11221(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_11225)
static void C_ccall f_11225(C_word c,C_word *av) C_noret;
C_noret_decl(f_11229)
static void C_ccall f_11229(C_word c,C_word *av) C_noret;
C_noret_decl(f_11235)
static void C_ccall f_11235(C_word c,C_word *av) C_noret;
C_noret_decl(f_11241)
static void C_fcall f_11241(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11247)
static void C_ccall f_11247(C_word c,C_word *av) C_noret;
C_noret_decl(f_11250)
static void C_ccall f_11250(C_word c,C_word *av) C_noret;
C_noret_decl(f_11256)
static void C_ccall f_11256(C_word c,C_word *av) C_noret;
C_noret_decl(f_11259)
static void C_ccall f_11259(C_word c,C_word *av) C_noret;
C_noret_decl(f_11260)
static void C_fcall f_11260(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11267)
static void C_ccall f_11267(C_word c,C_word *av) C_noret;
C_noret_decl(f_11270)
static void C_ccall f_11270(C_word c,C_word *av) C_noret;
C_noret_decl(f_11276)
static void C_ccall f_11276(C_word c,C_word *av) C_noret;
C_noret_decl(f_11291)
static void C_ccall f_11291(C_word c,C_word *av) C_noret;
C_noret_decl(f_11333)
static void C_ccall f_11333(C_word c,C_word *av) C_noret;
C_noret_decl(f_11339)
static void C_ccall f_11339(C_word c,C_word *av) C_noret;
C_noret_decl(f_11348)
static void C_ccall f_11348(C_word c,C_word *av) C_noret;
C_noret_decl(f_11351)
static void C_ccall f_11351(C_word c,C_word *av) C_noret;
C_noret_decl(f_11357)
static void C_ccall f_11357(C_word c,C_word *av) C_noret;
C_noret_decl(f_11360)
static void C_ccall f_11360(C_word c,C_word *av) C_noret;
C_noret_decl(f_11368)
static void C_ccall f_11368(C_word c,C_word *av) C_noret;
C_noret_decl(f_11375)
static void C_ccall f_11375(C_word c,C_word *av) C_noret;
C_noret_decl(f_11379)
static void C_ccall f_11379(C_word c,C_word *av) C_noret;
C_noret_decl(f_11387)
static void C_ccall f_11387(C_word c,C_word *av) C_noret;
C_noret_decl(f_11392)
static void C_fcall f_11392(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11415)
static void C_fcall f_11415(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11442)
static void C_ccall f_11442(C_word c,C_word *av) C_noret;
C_noret_decl(f_11446)
static void C_ccall f_11446(C_word c,C_word *av) C_noret;
C_noret_decl(f_11450)
static void C_ccall f_11450(C_word c,C_word *av) C_noret;
C_noret_decl(f_11454)
static void C_ccall f_11454(C_word c,C_word *av) C_noret;
C_noret_decl(f_11458)
static void C_ccall f_11458(C_word c,C_word *av) C_noret;
C_noret_decl(f_11475)
static void C_ccall f_11475(C_word c,C_word *av) C_noret;
C_noret_decl(f_11484)
static void C_fcall f_11484(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11494)
static void C_ccall f_11494(C_word c,C_word *av) C_noret;
C_noret_decl(f_11507)
static void C_ccall f_11507(C_word c,C_word *av) C_noret;
C_noret_decl(f_11518)
static void C_ccall f_11518(C_word c,C_word *av) C_noret;
C_noret_decl(f_11520)
static void C_ccall f_11520(C_word c,C_word *av) C_noret;
C_noret_decl(f_11534)
static void C_fcall f_11534(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11538)
static void C_ccall f_11538(C_word c,C_word *av) C_noret;
C_noret_decl(f_11541)
static void C_ccall f_11541(C_word c,C_word *av) C_noret;
C_noret_decl(f_11544)
static void C_ccall f_11544(C_word c,C_word *av) C_noret;
C_noret_decl(f_11559)
static void C_ccall f_11559(C_word c,C_word *av) C_noret;
C_noret_decl(f_11562)
static void C_ccall f_11562(C_word c,C_word *av) C_noret;
C_noret_decl(f_11587)
static void C_ccall f_11587(C_word c,C_word *av) C_noret;
C_noret_decl(f_11590)
static void C_ccall f_11590(C_word c,C_word *av) C_noret;
C_noret_decl(f_11608)
static void C_fcall f_11608(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11618)
static void C_ccall f_11618(C_word c,C_word *av) C_noret;
C_noret_decl(f_11621)
static void C_ccall f_11621(C_word c,C_word *av) C_noret;
C_noret_decl(f_11624)
static void C_ccall f_11624(C_word c,C_word *av) C_noret;
C_noret_decl(f_11635)
static void C_ccall f_11635(C_word c,C_word *av) C_noret;
C_noret_decl(f_11647)
static void C_ccall f_11647(C_word c,C_word *av) C_noret;
C_noret_decl(f_11655)
static void C_ccall f_11655(C_word c,C_word *av) C_noret;
C_noret_decl(f_11659)
static void C_ccall f_11659(C_word c,C_word *av) C_noret;
C_noret_decl(f_11682)
static void C_ccall f_11682(C_word c,C_word *av) C_noret;
C_noret_decl(f_11685)
static void C_ccall f_11685(C_word c,C_word *av) C_noret;
C_noret_decl(f_11707)
static void C_ccall f_11707(C_word c,C_word *av) C_noret;
C_noret_decl(f_11732)
static void C_ccall f_11732(C_word c,C_word *av) C_noret;
C_noret_decl(f_11747)
static void C_fcall f_11747(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11757)
static void C_ccall f_11757(C_word c,C_word *av) C_noret;
C_noret_decl(f_11776)
static void C_fcall f_11776(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11786)
static void C_ccall f_11786(C_word c,C_word *av) C_noret;
C_noret_decl(f_11805)
static void C_ccall f_11805(C_word c,C_word *av) C_noret;
C_noret_decl(f_11845)
static void C_ccall f_11845(C_word c,C_word *av) C_noret;
C_noret_decl(f_11847)
static void C_ccall f_11847(C_word c,C_word *av) C_noret;
C_noret_decl(f_11855)
static void C_ccall f_11855(C_word c,C_word *av) C_noret;
C_noret_decl(f_11857)
static void C_ccall f_11857(C_word c,C_word *av) C_noret;
C_noret_decl(f_11867)
static void C_ccall f_11867(C_word c,C_word *av) C_noret;
C_noret_decl(f_11870)
static void C_ccall f_11870(C_word c,C_word *av) C_noret;
C_noret_decl(f_11872)
static void C_ccall f_11872(C_word c,C_word *av) C_noret;
C_noret_decl(f_11875)
static C_word C_fcall f_11875(C_word *a,C_word t0);
C_noret_decl(f_11919)
static void C_ccall f_11919(C_word c,C_word *av) C_noret;
C_noret_decl(f_11923)
static void C_ccall f_11923(C_word c,C_word *av) C_noret;
C_noret_decl(f_11925)
static void C_fcall f_11925(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11929)
static void C_ccall f_11929(C_word c,C_word *av) C_noret;
C_noret_decl(f_11932)
static void C_ccall f_11932(C_word c,C_word *av) C_noret;
C_noret_decl(f_11935)
static void C_ccall f_11935(C_word c,C_word *av) C_noret;
C_noret_decl(f_11956)
static void C_fcall f_11956(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11970)
static void C_fcall f_11970(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11987)
static void C_ccall f_11987(C_word c,C_word *av) C_noret;
C_noret_decl(f_12039)
static void C_ccall f_12039(C_word c,C_word *av) C_noret;
C_noret_decl(f_12043)
static void C_ccall f_12043(C_word c,C_word *av) C_noret;
C_noret_decl(f_12047)
static void C_ccall f_12047(C_word c,C_word *av) C_noret;
C_noret_decl(f_12051)
static void C_ccall f_12051(C_word c,C_word *av) C_noret;
C_noret_decl(f_12055)
static void C_ccall f_12055(C_word c,C_word *av) C_noret;
C_noret_decl(f_12059)
static void C_ccall f_12059(C_word c,C_word *av) C_noret;
C_noret_decl(f_12069)
static void C_ccall f_12069(C_word c,C_word *av) C_noret;
C_noret_decl(f_12087)
static void C_ccall f_12087(C_word c,C_word *av) C_noret;
C_noret_decl(f_12093)
static void C_ccall f_12093(C_word c,C_word *av) C_noret;
C_noret_decl(f_12100)
static void C_ccall f_12100(C_word c,C_word *av) C_noret;
C_noret_decl(f_12110)
static void C_ccall f_12110(C_word c,C_word *av) C_noret;
C_noret_decl(f_12119)
static void C_fcall f_12119(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12131)
static void C_fcall f_12131(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12141)
static void C_ccall f_12141(C_word c,C_word *av) C_noret;
C_noret_decl(f_12164)
static void C_ccall f_12164(C_word c,C_word *av) C_noret;
C_noret_decl(f_12167)
static void C_ccall f_12167(C_word c,C_word *av) C_noret;
C_noret_decl(f_12168)
static void C_fcall f_12168(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12176)
static void C_ccall f_12176(C_word c,C_word *av) C_noret;
C_noret_decl(f_12179)
static void C_ccall f_12179(C_word c,C_word *av) C_noret;
C_noret_decl(f_12183)
static void C_fcall f_12183(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12193)
static void C_ccall f_12193(C_word c,C_word *av) C_noret;
C_noret_decl(f_12205)
static void C_ccall f_12205(C_word c,C_word *av) C_noret;
C_noret_decl(f_12210)
static void C_ccall f_12210(C_word c,C_word *av) C_noret;
C_noret_decl(f_12219)
static void C_ccall f_12219(C_word c,C_word *av) C_noret;
C_noret_decl(f_12226)
static void C_ccall f_12226(C_word c,C_word *av) C_noret;
C_noret_decl(f_12230)
static void C_ccall f_12230(C_word c,C_word *av) C_noret;
C_noret_decl(f_12232)
static void C_fcall f_12232(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12257)
static void C_ccall f_12257(C_word c,C_word *av) C_noret;
C_noret_decl(f_12268)
static void C_ccall f_12268(C_word c,C_word *av) C_noret;
C_noret_decl(f_12272)
static void C_ccall f_12272(C_word c,C_word *av) C_noret;
C_noret_decl(f_12278)
static void C_ccall f_12278(C_word c,C_word *av) C_noret;
C_noret_decl(f_12281)
static void C_ccall f_12281(C_word c,C_word *av) C_noret;
C_noret_decl(f_12284)
static void C_ccall f_12284(C_word c,C_word *av) C_noret;
C_noret_decl(f_12287)
static void C_ccall f_12287(C_word c,C_word *av) C_noret;
C_noret_decl(f_12290)
static void C_ccall f_12290(C_word c,C_word *av) C_noret;
C_noret_decl(f_12293)
static void C_ccall f_12293(C_word c,C_word *av) C_noret;
C_noret_decl(f_12294)
static void C_fcall f_12294(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12298)
static void C_ccall f_12298(C_word c,C_word *av) C_noret;
C_noret_decl(f_12299)
static void C_fcall f_12299(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12313)
static void C_ccall f_12313(C_word c,C_word *av) C_noret;
C_noret_decl(f_12317)
static void C_ccall f_12317(C_word c,C_word *av) C_noret;
C_noret_decl(f_12321)
static void C_ccall f_12321(C_word c,C_word *av) C_noret;
C_noret_decl(f_12325)
static void C_ccall f_12325(C_word c,C_word *av) C_noret;
C_noret_decl(f_12329)
static void C_ccall f_12329(C_word c,C_word *av) C_noret;
C_noret_decl(f_12333)
static void C_ccall f_12333(C_word c,C_word *av) C_noret;
C_noret_decl(f_12341)
static void C_ccall f_12341(C_word c,C_word *av) C_noret;
C_noret_decl(f_12351)
static void C_ccall f_12351(C_word c,C_word *av) C_noret;
C_noret_decl(f_12363)
static void C_ccall f_12363(C_word c,C_word *av) C_noret;
C_noret_decl(f_12371)
static void C_fcall f_12371(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12381)
static void C_ccall f_12381(C_word c,C_word *av) C_noret;
C_noret_decl(f_12401)
static void C_fcall f_12401(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12411)
static void C_ccall f_12411(C_word c,C_word *av) C_noret;
C_noret_decl(f_12424)
static void C_fcall f_12424(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12434)
static void C_fcall f_12434(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12444)
static void C_ccall f_12444(C_word c,C_word *av) C_noret;
C_noret_decl(f_12456)
static void C_ccall f_12456(C_word c,C_word *av) C_noret;
C_noret_decl(f_12460)
static void C_ccall f_12460(C_word c,C_word *av) C_noret;
C_noret_decl(f_12472)
static void C_ccall f_12472(C_word c,C_word *av) C_noret;
C_noret_decl(f_12476)
static void C_ccall f_12476(C_word c,C_word *av) C_noret;
C_noret_decl(f_12480)
static void C_ccall f_12480(C_word c,C_word *av) C_noret;
C_noret_decl(f_12484)
static void C_ccall f_12484(C_word c,C_word *av) C_noret;
C_noret_decl(f_12488)
static void C_ccall f_12488(C_word c,C_word *av) C_noret;
C_noret_decl(f_12492)
static void C_ccall f_12492(C_word c,C_word *av) C_noret;
C_noret_decl(f_12500)
static void C_ccall f_12500(C_word c,C_word *av) C_noret;
C_noret_decl(f_12502)
static void C_ccall f_12502(C_word c,C_word *av) C_noret;
C_noret_decl(f_12518)
static void C_ccall f_12518(C_word c,C_word *av) C_noret;
C_noret_decl(f_12530)
static void C_ccall f_12530(C_word c,C_word *av) C_noret;
C_noret_decl(f_12538)
static void C_ccall f_12538(C_word c,C_word *av) C_noret;
C_noret_decl(f_12546)
static void C_ccall f_12546(C_word c,C_word *av) C_noret;
C_noret_decl(f_12554)
static void C_ccall f_12554(C_word c,C_word *av) C_noret;
C_noret_decl(f_12556)
static void C_ccall f_12556(C_word c,C_word *av) C_noret;
C_noret_decl(f_12564)
static void C_ccall f_12564(C_word c,C_word *av) C_noret;
C_noret_decl(f_12568)
static void C_ccall f_12568(C_word c,C_word *av) C_noret;
C_noret_decl(f_12574)
static void C_fcall f_12574(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12599)
static void C_ccall f_12599(C_word c,C_word *av) C_noret;
C_noret_decl(f_12603)
static void C_ccall f_12603(C_word c,C_word *av) C_noret;
C_noret_decl(f_12623)
static void C_ccall f_12623(C_word c,C_word *av) C_noret;
C_noret_decl(f_12629)
static void C_ccall f_12629(C_word c,C_word *av) C_noret;
C_noret_decl(f_12633)
static void C_ccall f_12633(C_word c,C_word *av) C_noret;
C_noret_decl(f_12646)
static void C_ccall f_12646(C_word c,C_word *av) C_noret;
C_noret_decl(f_12650)
static void C_ccall f_12650(C_word c,C_word *av) C_noret;
C_noret_decl(f_12654)
static void C_ccall f_12654(C_word c,C_word *av) C_noret;
C_noret_decl(f_12658)
static void C_ccall f_12658(C_word c,C_word *av) C_noret;
C_noret_decl(f_12665)
static void C_ccall f_12665(C_word c,C_word *av) C_noret;
C_noret_decl(f_12667)
static void C_fcall f_12667(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12703)
static void C_ccall f_12703(C_word c,C_word *av) C_noret;
C_noret_decl(f_12709)
static void C_ccall f_12709(C_word c,C_word *av) C_noret;
C_noret_decl(f_12732)
static void C_ccall f_12732(C_word c,C_word *av) C_noret;
C_noret_decl(f_12734)
static void C_fcall f_12734(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12768)
static void C_fcall f_12768(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12793)
static void C_ccall f_12793(C_word c,C_word *av) C_noret;
C_noret_decl(f_12806)
static void C_ccall f_12806(C_word c,C_word *av) C_noret;
C_noret_decl(f_12815)
static void C_fcall f_12815(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12825)
static void C_ccall f_12825(C_word c,C_word *av) C_noret;
C_noret_decl(f_12854)
static void C_ccall f_12854(C_word c,C_word *av) C_noret;
C_noret_decl(f_12876)
static void C_ccall f_12876(C_word c,C_word *av) C_noret;
C_noret_decl(f_12894)
static void C_ccall f_12894(C_word c,C_word *av) C_noret;
C_noret_decl(f_12902)
static void C_ccall f_12902(C_word c,C_word *av) C_noret;
C_noret_decl(f_12904)
static void C_fcall f_12904(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12912)
static void C_fcall f_12912(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_12924)
static void C_ccall f_12924(C_word c,C_word *av) C_noret;
C_noret_decl(f_12928)
static void C_ccall f_12928(C_word c,C_word *av) C_noret;
C_noret_decl(f_12936)
static void C_ccall f_12936(C_word c,C_word *av) C_noret;
C_noret_decl(f_12940)
static void C_ccall f_12940(C_word c,C_word *av) C_noret;
C_noret_decl(f_12951)
static void C_ccall f_12951(C_word c,C_word *av) C_noret;
C_noret_decl(f_12970)
static void C_ccall f_12970(C_word c,C_word *av) C_noret;
C_noret_decl(f_12974)
static void C_ccall f_12974(C_word c,C_word *av) C_noret;
C_noret_decl(f_13000)
static void C_ccall f_13000(C_word c,C_word *av) C_noret;
C_noret_decl(f_13015)
static void C_ccall f_13015(C_word c,C_word *av) C_noret;
C_noret_decl(f_13019)
static void C_ccall f_13019(C_word c,C_word *av) C_noret;
C_noret_decl(f_13081)
static void C_ccall f_13081(C_word c,C_word *av) C_noret;
C_noret_decl(f_13096)
static void C_fcall f_13096(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13113)
static void C_ccall f_13113(C_word c,C_word *av) C_noret;
C_noret_decl(f_13119)
static void C_fcall f_13119(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13132)
static void C_ccall f_13132(C_word c,C_word *av) C_noret;
C_noret_decl(f_13161)
static void C_ccall f_13161(C_word c,C_word *av) C_noret;
C_noret_decl(f_13167)
static void C_ccall f_13167(C_word c,C_word *av) C_noret;
C_noret_decl(f_13180)
static void C_ccall f_13180(C_word c,C_word *av) C_noret;
C_noret_decl(f_13214)
static void C_ccall f_13214(C_word c,C_word *av) C_noret;
C_noret_decl(f_13241)
static void C_fcall f_13241(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13262)
static void C_ccall f_13262(C_word c,C_word *av) C_noret;
C_noret_decl(f_13266)
static void C_ccall f_13266(C_word c,C_word *av) C_noret;
C_noret_decl(f_13274)
static void C_ccall f_13274(C_word c,C_word *av) C_noret;
C_noret_decl(f_13280)
static void C_fcall f_13280(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13293)
static void C_ccall f_13293(C_word c,C_word *av) C_noret;
C_noret_decl(f_13322)
static void C_ccall f_13322(C_word c,C_word *av) C_noret;
C_noret_decl(f_13328)
static void C_ccall f_13328(C_word c,C_word *av) C_noret;
C_noret_decl(f_13341)
static void C_ccall f_13341(C_word c,C_word *av) C_noret;
C_noret_decl(f_13345)
static void C_ccall f_13345(C_word c,C_word *av) C_noret;
C_noret_decl(f_13399)
static void C_ccall f_13399(C_word c,C_word *av) C_noret;
C_noret_decl(f_13423)
static void C_ccall f_13423(C_word c,C_word *av) C_noret;
C_noret_decl(f_13436)
static void C_ccall f_13436(C_word c,C_word *av) C_noret;
C_noret_decl(f_13444)
static void C_ccall f_13444(C_word c,C_word *av) C_noret;
C_noret_decl(f_13482)
static void C_ccall f_13482(C_word c,C_word *av) C_noret;
C_noret_decl(f_13492)
static void C_fcall f_13492(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13496)
static void C_ccall f_13496(C_word c,C_word *av) C_noret;
C_noret_decl(f_13499)
static void C_ccall f_13499(C_word c,C_word *av) C_noret;
C_noret_decl(f_13502)
static void C_ccall f_13502(C_word c,C_word *av) C_noret;
C_noret_decl(f_13532)
static void C_ccall f_13532(C_word c,C_word *av) C_noret;
C_noret_decl(f_13535)
static void C_ccall f_13535(C_word c,C_word *av) C_noret;
C_noret_decl(f_13538)
static void C_ccall f_13538(C_word c,C_word *av) C_noret;
C_noret_decl(f_13568)
static void C_ccall f_13568(C_word c,C_word *av) C_noret;
C_noret_decl(f_13595)
static void C_fcall f_13595(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13599)
static void C_ccall f_13599(C_word c,C_word *av) C_noret;
C_noret_decl(f_13602)
static void C_ccall f_13602(C_word c,C_word *av) C_noret;
C_noret_decl(f_13605)
static void C_ccall f_13605(C_word c,C_word *av) C_noret;
C_noret_decl(f_13611)
static void C_fcall f_13611(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13634)
static void C_ccall f_13634(C_word c,C_word *av) C_noret;
C_noret_decl(f_13649)
static void C_ccall f_13649(C_word c,C_word *av) C_noret;
C_noret_decl(f_13657)
static void C_ccall f_13657(C_word c,C_word *av) C_noret;
C_noret_decl(f_13659)
static void C_ccall f_13659(C_word c,C_word *av) C_noret;
C_noret_decl(f_13709)
static void C_ccall f_13709(C_word c,C_word *av) C_noret;
C_noret_decl(f_13717)
static void C_ccall f_13717(C_word c,C_word *av) C_noret;
C_noret_decl(f_13725)
static void C_ccall f_13725(C_word c,C_word *av) C_noret;
C_noret_decl(f_13733)
static void C_ccall f_13733(C_word c,C_word *av) C_noret;
C_noret_decl(f_13761)
static void C_ccall f_13761(C_word c,C_word *av) C_noret;
C_noret_decl(f_13769)
static void C_ccall f_13769(C_word c,C_word *av) C_noret;
C_noret_decl(f_13795)
static void C_ccall f_13795(C_word c,C_word *av) C_noret;
C_noret_decl(f_13808)
static void C_ccall f_13808(C_word c,C_word *av) C_noret;
C_noret_decl(f_13825)
static void C_ccall f_13825(C_word c,C_word *av) C_noret;
C_noret_decl(f_13829)
static void C_ccall f_13829(C_word c,C_word *av) C_noret;
C_noret_decl(f_13833)
static void C_ccall f_13833(C_word c,C_word *av) C_noret;
C_noret_decl(f_13844)
static void C_ccall f_13844(C_word c,C_word *av) C_noret;
C_noret_decl(f_13848)
static void C_ccall f_13848(C_word c,C_word *av) C_noret;
C_noret_decl(f_13922)
static void C_ccall f_13922(C_word c,C_word *av) C_noret;
C_noret_decl(f_13935)
static void C_ccall f_13935(C_word c,C_word *av) C_noret;
C_noret_decl(f_13941)
static void C_ccall f_13941(C_word c,C_word *av) C_noret;
C_noret_decl(f_13964)
static void C_ccall f_13964(C_word c,C_word *av) C_noret;
C_noret_decl(f_13968)
static void C_ccall f_13968(C_word c,C_word *av) C_noret;
C_noret_decl(f_13972)
static void C_ccall f_13972(C_word c,C_word *av) C_noret;
C_noret_decl(f_13981)
static void C_ccall f_13981(C_word c,C_word *av) C_noret;
C_noret_decl(f_13990)
static void C_ccall f_13990(C_word c,C_word *av) C_noret;
C_noret_decl(f_13994)
static void C_ccall f_13994(C_word c,C_word *av) C_noret;
C_noret_decl(f_14092)
static void C_ccall f_14092(C_word c,C_word *av) C_noret;
C_noret_decl(f_14100)
static void C_ccall f_14100(C_word c,C_word *av) C_noret;
C_noret_decl(f_14105)
static void C_fcall f_14105(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14115)
static void C_ccall f_14115(C_word c,C_word *av) C_noret;
C_noret_decl(f_14119)
static C_word C_fcall f_14119(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_3387)
static void C_ccall f_3387(C_word c,C_word *av) C_noret;
C_noret_decl(f_3390)
static void C_ccall f_3390(C_word c,C_word *av) C_noret;
C_noret_decl(f_3393)
static void C_ccall f_3393(C_word c,C_word *av) C_noret;
C_noret_decl(f_3396)
static void C_ccall f_3396(C_word c,C_word *av) C_noret;
C_noret_decl(f_3399)
static void C_ccall f_3399(C_word c,C_word *av) C_noret;
C_noret_decl(f_3407)
static void C_fcall f_3407(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_3421)
static void C_ccall f_3421(C_word c,C_word *av) C_noret;
C_noret_decl(f_3425)
static void C_ccall f_3425(C_word c,C_word *av) C_noret;
C_noret_decl(f_3431)
static void C_ccall f_3431(C_word c,C_word *av) C_noret;
C_noret_decl(f_3509)
static void C_fcall f_3509(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3527)
static void C_ccall f_3527(C_word c,C_word *av) C_noret;
C_noret_decl(f_3561)
static void C_fcall f_3561(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3567)
static void C_fcall f_3567(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_3581)
static void C_ccall f_3581(C_word c,C_word *av) C_noret;
C_noret_decl(f_3600)
static void C_fcall f_3600(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_3615)
static void C_fcall f_3615(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3623)
static void C_fcall f_3623(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_3631)
static void C_ccall f_3631(C_word c,C_word *av) C_noret;
C_noret_decl(f_3642)
static void C_ccall f_3642(C_word c,C_word *av) C_noret;
C_noret_decl(f_3655)
static void C_fcall f_3655(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3662)
static void C_ccall f_3662(C_word c,C_word *av) C_noret;
C_noret_decl(f_3669)
static void C_ccall f_3669(C_word c,C_word *av) C_noret;
C_noret_decl(f_3673)
static void C_ccall f_3673(C_word c,C_word *av) C_noret;
C_noret_decl(f_3685)
static void C_ccall f_3685(C_word c,C_word *av) C_noret;
C_noret_decl(f_3687)
static void C_fcall f_3687(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3734)
static void C_ccall f_3734(C_word c,C_word *av) C_noret;
C_noret_decl(f_3736)
static void C_fcall f_3736(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3770)
static void C_fcall f_3770(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3776)
static void C_fcall f_3776(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3798)
static void C_ccall f_3798(C_word c,C_word *av) C_noret;
C_noret_decl(f_3804)
static void C_fcall f_3804(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3810)
static void C_fcall f_3810(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3820)
static void C_ccall f_3820(C_word c,C_word *av) C_noret;
C_noret_decl(f_3834)
static void C_fcall f_3834(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3840)
static void C_fcall f_3840(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_3854)
static void C_ccall f_3854(C_word c,C_word *av) C_noret;
C_noret_decl(f_3890)
static void C_fcall f_3890(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_3896)
static void C_fcall f_3896(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3909)
static void C_ccall f_3909(C_word c,C_word *av) C_noret;
C_noret_decl(f_3923)
static void C_ccall f_3923(C_word c,C_word *av) C_noret;
C_noret_decl(f_3931)
static void C_ccall f_3931(C_word c,C_word *av) C_noret;
C_noret_decl(f_3971)
static void C_fcall f_3971(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_3985)
static void C_ccall f_3985(C_word c,C_word *av) C_noret;
C_noret_decl(f_3998)
static void C_ccall f_3998(C_word c,C_word *av) C_noret;
C_noret_decl(f_4010)
static void C_fcall f_4010(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4019)
static void C_fcall f_4019(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4027)
static void C_fcall f_4027(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4034)
static void C_ccall f_4034(C_word c,C_word *av) C_noret;
C_noret_decl(f_4048)
static void C_ccall f_4048(C_word c,C_word *av) C_noret;
C_noret_decl(f_4063)
static void C_fcall f_4063(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4071)
static void C_fcall f_4071(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4075)
static void C_ccall f_4075(C_word c,C_word *av) C_noret;
C_noret_decl(f_4079)
static C_word C_fcall f_4079(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_4097)
static void C_ccall f_4097(C_word c,C_word *av) C_noret;
C_noret_decl(f_4136)
static void C_fcall f_4136(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4176)
static C_word C_fcall f_4176(C_word t0);
C_noret_decl(f_4235)
static void C_fcall f_4235(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4412)
static void C_fcall f_4412(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4418)
static void C_fcall f_4418(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4432)
static void C_ccall f_4432(C_word c,C_word *av) C_noret;
C_noret_decl(f_4436)
static void C_ccall f_4436(C_word c,C_word *av) C_noret;
C_noret_decl(f_4448)
static void C_ccall f_4448(C_word c,C_word *av) C_noret;
C_noret_decl(f_4466)
static void C_ccall f_4466(C_word c,C_word *av) C_noret;
C_noret_decl(f_4681)
static void C_ccall f_4681(C_word c,C_word *av) C_noret;
C_noret_decl(f_4685)
static void C_ccall f_4685(C_word c,C_word *av) C_noret;
C_noret_decl(f_4689)
static void C_ccall f_4689(C_word c,C_word *av) C_noret;
C_noret_decl(f_4693)
static void C_ccall f_4693(C_word c,C_word *av) C_noret;
C_noret_decl(f_4695)
static void C_ccall f_4695(C_word c,C_word *av) C_noret;
C_noret_decl(f_4698)
static C_word C_fcall f_4698(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_4721)
static void C_fcall f_4721(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4726)
static void C_ccall f_4726(C_word c,C_word *av) C_noret;
C_noret_decl(f_4728)
static C_word C_fcall f_4728(C_word t0);
C_noret_decl(f_4733)
static void C_fcall f_4733(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_4735)
static void C_fcall f_4735(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4751)
static void C_fcall f_4751(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_4761)
static void C_ccall f_4761(C_word c,C_word *av) C_noret;
C_noret_decl(f_4774)
static void C_fcall f_4774(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_4778)
static void C_ccall f_4778(C_word c,C_word *av) C_noret;
C_noret_decl(f_4781)
static void C_ccall f_4781(C_word c,C_word *av) C_noret;
C_noret_decl(f_4784)
static void C_ccall f_4784(C_word c,C_word *av) C_noret;
C_noret_decl(f_4796)
static void C_fcall f_4796(C_word t0,C_word t1) C_noret;
C_noret_decl(f_4800)
static void C_ccall f_4800(C_word c,C_word *av) C_noret;
C_noret_decl(f_4802)
static void C_ccall f_4802(C_word c,C_word *av) C_noret;
C_noret_decl(f_4844)
static void C_fcall f_4844(C_word t0,C_word t1) C_noret;
C_noret_decl(f_4847)
static void C_ccall f_4847(C_word c,C_word *av) C_noret;
C_noret_decl(f_4870)
static void C_ccall f_4870(C_word c,C_word *av) C_noret;
C_noret_decl(f_4881)
static void C_ccall f_4881(C_word c,C_word *av) C_noret;
C_noret_decl(f_4885)
static void C_ccall f_4885(C_word c,C_word *av) C_noret;
C_noret_decl(f_4918)
static void C_ccall f_4918(C_word c,C_word *av) C_noret;
C_noret_decl(f_4921)
static void C_ccall f_4921(C_word c,C_word *av) C_noret;
C_noret_decl(f_4924)
static void C_ccall f_4924(C_word c,C_word *av) C_noret;
C_noret_decl(f_4942)
static void C_ccall f_4942(C_word c,C_word *av) C_noret;
C_noret_decl(f_4949)
static void C_ccall f_4949(C_word c,C_word *av) C_noret;
C_noret_decl(f_4992)
static void C_ccall f_4992(C_word c,C_word *av) C_noret;
C_noret_decl(f_4995)
static void C_ccall f_4995(C_word c,C_word *av) C_noret;
C_noret_decl(f_4998)
static void C_ccall f_4998(C_word c,C_word *av) C_noret;
C_noret_decl(f_5011)
static void C_fcall f_5011(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5021)
static void C_ccall f_5021(C_word c,C_word *av) C_noret;
C_noret_decl(f_5042)
static void C_ccall f_5042(C_word c,C_word *av) C_noret;
C_noret_decl(f_5046)
static void C_ccall f_5046(C_word c,C_word *av) C_noret;
C_noret_decl(f_5051)
static void C_ccall f_5051(C_word c,C_word *av) C_noret;
C_noret_decl(f_5054)
static void C_fcall f_5054(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_5068)
static void C_ccall f_5068(C_word c,C_word *av) C_noret;
C_noret_decl(f_5078)
static void C_ccall f_5078(C_word c,C_word *av) C_noret;
C_noret_decl(f_5080)
static C_word C_fcall f_5080(C_word t0);
C_noret_decl(f_5084)
static void C_fcall f_5084(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5098)
static C_word C_fcall f_5098(C_word t0);
C_noret_decl(f_5121)
static void C_fcall f_5121(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5125)
static void C_ccall f_5125(C_word c,C_word *av) C_noret;
C_noret_decl(f_5128)
static void C_ccall f_5128(C_word c,C_word *av) C_noret;
C_noret_decl(f_5136)
static void C_ccall f_5136(C_word c,C_word *av) C_noret;
C_noret_decl(f_5146)
static void C_ccall f_5146(C_word c,C_word *av) C_noret;
C_noret_decl(f_5152)
static void C_ccall f_5152(C_word c,C_word *av) C_noret;
C_noret_decl(f_5191)
static C_word C_fcall f_5191(C_word t0,C_word t1);
C_noret_decl(f_5205)
static void C_ccall f_5205(C_word c,C_word *av) C_noret;
C_noret_decl(f_5207)
static void C_fcall f_5207(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5247)
static void C_ccall f_5247(C_word c,C_word *av) C_noret;
C_noret_decl(f_5249)
static void C_fcall f_5249(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5256)
static void C_ccall f_5256(C_word c,C_word *av) C_noret;
C_noret_decl(f_5268)
static void C_fcall f_5268(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5304)
static void C_ccall f_5304(C_word c,C_word *av) C_noret;
C_noret_decl(f_5326)
static void C_ccall f_5326(C_word c,C_word *av) C_noret;
C_noret_decl(f_5336)
static void C_ccall f_5336(C_word c,C_word *av) C_noret;
C_noret_decl(f_5348)
static void C_ccall f_5348(C_word c,C_word *av) C_noret;
C_noret_decl(f_5352)
static void C_ccall f_5352(C_word c,C_word *av) C_noret;
C_noret_decl(f_5356)
static void C_ccall f_5356(C_word c,C_word *av) C_noret;
C_noret_decl(f_5360)
static void C_ccall f_5360(C_word c,C_word *av) C_noret;
C_noret_decl(f_5364)
static void C_ccall f_5364(C_word c,C_word *av) C_noret;
C_noret_decl(f_5368)
static void C_ccall f_5368(C_word c,C_word *av) C_noret;
C_noret_decl(f_5372)
static void C_ccall f_5372(C_word c,C_word *av) C_noret;
C_noret_decl(f_5376)
static void C_ccall f_5376(C_word c,C_word *av) C_noret;
C_noret_decl(f_5380)
static void C_ccall f_5380(C_word c,C_word *av) C_noret;
C_noret_decl(f_5405)
static void C_ccall f_5405(C_word c,C_word *av) C_noret;
C_noret_decl(f_5409)
static void C_ccall f_5409(C_word c,C_word *av) C_noret;
C_noret_decl(f_5411)
static void C_fcall f_5411(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_5425)
static void C_ccall f_5425(C_word c,C_word *av) C_noret;
C_noret_decl(f_5428)
static void C_ccall f_5428(C_word c,C_word *av) C_noret;
C_noret_decl(f_5431)
static void C_ccall f_5431(C_word c,C_word *av) C_noret;
C_noret_decl(f_5434)
static void C_ccall f_5434(C_word c,C_word *av) C_noret;
C_noret_decl(f_5494)
static void C_ccall f_5494(C_word c,C_word *av) C_noret;
C_noret_decl(f_5507)
static void C_fcall f_5507(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5511)
static void C_ccall f_5511(C_word c,C_word *av) C_noret;
C_noret_decl(f_5518)
static void C_ccall f_5518(C_word c,C_word *av) C_noret;
C_noret_decl(f_5549)
static void C_fcall f_5549(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5553)
static void C_ccall f_5553(C_word c,C_word *av) C_noret;
C_noret_decl(f_5557)
static void C_fcall f_5557(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5561)
static void C_ccall f_5561(C_word c,C_word *av) C_noret;
C_noret_decl(f_5564)
static void C_ccall f_5564(C_word c,C_word *av) C_noret;
C_noret_decl(f_5569)
static void C_fcall f_5569(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_5573)
static void C_ccall f_5573(C_word c,C_word *av) C_noret;
C_noret_decl(f_5576)
static void C_ccall f_5576(C_word c,C_word *av) C_noret;
C_noret_decl(f_5579)
static void C_ccall f_5579(C_word c,C_word *av) C_noret;
C_noret_decl(f_5588)
static void C_ccall f_5588(C_word c,C_word *av) C_noret;
C_noret_decl(f_5594)
static void C_ccall f_5594(C_word c,C_word *av) C_noret;
C_noret_decl(f_5600)
static void C_ccall f_5600(C_word c,C_word *av) C_noret;
C_noret_decl(f_5611)
static void C_ccall f_5611(C_word c,C_word *av) C_noret;
C_noret_decl(f_5615)
static void C_ccall f_5615(C_word c,C_word *av) C_noret;
C_noret_decl(f_5638)
static void C_fcall f_5638(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5648)
static void C_fcall f_5648(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5651)
static void C_ccall f_5651(C_word c,C_word *av) C_noret;
C_noret_decl(f_5668)
static void C_ccall f_5668(C_word c,C_word *av) C_noret;
C_noret_decl(f_5701)
static void C_ccall f_5701(C_word c,C_word *av) C_noret;
C_noret_decl(f_5704)
static void C_fcall f_5704(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5719)
static void C_fcall f_5719(C_word t0,C_word t1) C_noret;
C_noret_decl(f_5727)
static void C_fcall f_5727(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5737)
static void C_ccall f_5737(C_word c,C_word *av) C_noret;
C_noret_decl(f_5739)
static void C_fcall f_5739(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5764)
static void C_ccall f_5764(C_word c,C_word *av) C_noret;
C_noret_decl(f_5774)
static void C_ccall f_5774(C_word c,C_word *av) C_noret;
C_noret_decl(f_5785)
static void C_ccall f_5785(C_word c,C_word *av) C_noret;
C_noret_decl(f_5803)
static void C_ccall f_5803(C_word c,C_word *av) C_noret;
C_noret_decl(f_5811)
static void C_ccall f_5811(C_word c,C_word *av) C_noret;
C_noret_decl(f_5821)
static void C_ccall f_5821(C_word c,C_word *av) C_noret;
C_noret_decl(f_5828)
static void C_ccall f_5828(C_word c,C_word *av) C_noret;
C_noret_decl(f_5846)
static void C_ccall f_5846(C_word c,C_word *av) C_noret;
C_noret_decl(f_5851)
static void C_ccall f_5851(C_word c,C_word *av) C_noret;
C_noret_decl(f_5857)
static void C_ccall f_5857(C_word c,C_word *av) C_noret;
C_noret_decl(f_5863)
static void C_fcall f_5863(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_5869)
static void C_ccall f_5869(C_word c,C_word *av) C_noret;
C_noret_decl(f_5876)
static void C_ccall f_5876(C_word c,C_word *av) C_noret;
C_noret_decl(f_5891)
static void C_ccall f_5891(C_word c,C_word *av) C_noret;
C_noret_decl(f_5911)
static void C_ccall f_5911(C_word c,C_word *av) C_noret;
C_noret_decl(f_5918)
static void C_ccall f_5918(C_word c,C_word *av) C_noret;
C_noret_decl(f_5921)
static void C_ccall f_5921(C_word c,C_word *av) C_noret;
C_noret_decl(f_5938)
static void C_ccall f_5938(C_word c,C_word *av) C_noret;
C_noret_decl(f_5943)
static void C_ccall f_5943(C_word c,C_word *av) C_noret;
C_noret_decl(f_5950)
static void C_ccall f_5950(C_word c,C_word *av) C_noret;
C_noret_decl(f_5965)
static void C_ccall f_5965(C_word c,C_word *av) C_noret;
C_noret_decl(f_5985)
static void C_ccall f_5985(C_word c,C_word *av) C_noret;
C_noret_decl(f_6019)
static void C_ccall f_6019(C_word c,C_word *av) C_noret;
C_noret_decl(f_6031)
static void C_ccall f_6031(C_word c,C_word *av) C_noret;
C_noret_decl(f_6034)
static void C_ccall f_6034(C_word c,C_word *av) C_noret;
C_noret_decl(f_6041)
static void C_fcall f_6041(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6044)
static void C_ccall f_6044(C_word c,C_word *av) C_noret;
C_noret_decl(f_6053)
static void C_ccall f_6053(C_word c,C_word *av) C_noret;
C_noret_decl(f_6056)
static void C_ccall f_6056(C_word c,C_word *av) C_noret;
C_noret_decl(f_6060)
static void C_fcall f_6060(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6070)
static void C_ccall f_6070(C_word c,C_word *av) C_noret;
C_noret_decl(f_6077)
static void C_ccall f_6077(C_word c,C_word *av) C_noret;
C_noret_decl(f_6085)
static void C_ccall f_6085(C_word c,C_word *av) C_noret;
C_noret_decl(f_6087)
static void C_fcall f_6087(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6097)
static void C_ccall f_6097(C_word c,C_word *av) C_noret;
C_noret_decl(f_6111)
static void C_ccall f_6111(C_word c,C_word *av) C_noret;
C_noret_decl(f_6120)
static void C_ccall f_6120(C_word c,C_word *av) C_noret;
C_noret_decl(f_6123)
static void C_fcall f_6123(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6126)
static void C_ccall f_6126(C_word c,C_word *av) C_noret;
C_noret_decl(f_6145)
static void C_ccall f_6145(C_word c,C_word *av) C_noret;
C_noret_decl(f_6151)
static void C_fcall f_6151(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6154)
static void C_ccall f_6154(C_word c,C_word *av) C_noret;
C_noret_decl(f_6162)
static void C_ccall f_6162(C_word c,C_word *av) C_noret;
C_noret_decl(f_6169)
static void C_ccall f_6169(C_word c,C_word *av) C_noret;
C_noret_decl(f_6175)
static void C_fcall f_6175(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6181)
static void C_ccall f_6181(C_word c,C_word *av) C_noret;
C_noret_decl(f_6184)
static void C_ccall f_6184(C_word c,C_word *av) C_noret;
C_noret_decl(f_6185)
static void C_fcall f_6185(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6195)
static void C_ccall f_6195(C_word c,C_word *av) C_noret;
C_noret_decl(f_6198)
static void C_ccall f_6198(C_word c,C_word *av) C_noret;
C_noret_decl(f_6203)
static void C_ccall f_6203(C_word c,C_word *av) C_noret;
C_noret_decl(f_6206)
static void C_ccall f_6206(C_word c,C_word *av) C_noret;
C_noret_decl(f_6210)
static void C_ccall f_6210(C_word c,C_word *av) C_noret;
C_noret_decl(f_6217)
static void C_ccall f_6217(C_word c,C_word *av) C_noret;
C_noret_decl(f_6220)
static void C_ccall f_6220(C_word c,C_word *av) C_noret;
C_noret_decl(f_6242)
static void C_ccall f_6242(C_word c,C_word *av) C_noret;
C_noret_decl(f_6244)
static void C_fcall f_6244(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6254)
static void C_ccall f_6254(C_word c,C_word *av) C_noret;
C_noret_decl(f_6276)
static void C_ccall f_6276(C_word c,C_word *av) C_noret;
C_noret_decl(f_6285)
static void C_ccall f_6285(C_word c,C_word *av) C_noret;
C_noret_decl(f_6299)
static void C_fcall f_6299(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_6316)
static void C_fcall f_6316(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6326)
static void C_ccall f_6326(C_word c,C_word *av) C_noret;
C_noret_decl(f_6329)
static void C_ccall f_6329(C_word c,C_word *av) C_noret;
C_noret_decl(f_6334)
static void C_fcall f_6334(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6359)
static void C_ccall f_6359(C_word c,C_word *av) C_noret;
C_noret_decl(f_6370)
static void C_ccall f_6370(C_word c,C_word *av) C_noret;
C_noret_decl(f_6374)
static void C_ccall f_6374(C_word c,C_word *av) C_noret;
C_noret_decl(f_6380)
static void C_ccall f_6380(C_word c,C_word *av) C_noret;
C_noret_decl(f_6386)
static void C_ccall f_6386(C_word c,C_word *av) C_noret;
C_noret_decl(f_6392)
static void C_ccall f_6392(C_word c,C_word *av) C_noret;
C_noret_decl(f_6407)
static void C_ccall f_6407(C_word c,C_word *av) C_noret;
C_noret_decl(f_6411)
static void C_ccall f_6411(C_word c,C_word *av) C_noret;
C_noret_decl(f_6423)
static void C_ccall f_6423(C_word c,C_word *av) C_noret;
C_noret_decl(f_6473)
static void C_ccall f_6473(C_word c,C_word *av) C_noret;
C_noret_decl(f_6485)
static void C_ccall f_6485(C_word c,C_word *av) C_noret;
C_noret_decl(f_6490)
static void C_ccall f_6490(C_word c,C_word *av) C_noret;
C_noret_decl(f_6496)
static void C_ccall f_6496(C_word c,C_word *av) C_noret;
C_noret_decl(f_6500)
static void C_ccall f_6500(C_word c,C_word *av) C_noret;
C_noret_decl(f_6507)
static void C_ccall f_6507(C_word c,C_word *av) C_noret;
C_noret_decl(f_6512)
static void C_fcall f_6512(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6522)
static void C_ccall f_6522(C_word c,C_word *av) C_noret;
C_noret_decl(f_6524)
static void C_fcall f_6524(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6549)
static void C_ccall f_6549(C_word c,C_word *av) C_noret;
C_noret_decl(f_6560)
static void C_ccall f_6560(C_word c,C_word *av) C_noret;
C_noret_decl(f_6568)
static void C_ccall f_6568(C_word c,C_word *av) C_noret;
C_noret_decl(f_6597)
static void C_ccall f_6597(C_word c,C_word *av) C_noret;
C_noret_decl(f_6610)
static void C_ccall f_6610(C_word c,C_word *av) C_noret;
C_noret_decl(f_6628)
static void C_ccall f_6628(C_word c,C_word *av) C_noret;
C_noret_decl(f_6634)
static void C_ccall f_6634(C_word c,C_word *av) C_noret;
C_noret_decl(f_6640)
static void C_ccall f_6640(C_word c,C_word *av) C_noret;
C_noret_decl(f_6665)
static void C_ccall f_6665(C_word c,C_word *av) C_noret;
C_noret_decl(f_6669)
static void C_ccall f_6669(C_word c,C_word *av) C_noret;
C_noret_decl(f_6673)
static void C_ccall f_6673(C_word c,C_word *av) C_noret;
C_noret_decl(f_6679)
static void C_ccall f_6679(C_word c,C_word *av) C_noret;
C_noret_decl(f_6682)
static void C_ccall f_6682(C_word c,C_word *av) C_noret;
C_noret_decl(f_6689)
static void C_ccall f_6689(C_word c,C_word *av) C_noret;
C_noret_decl(f_6703)
static void C_ccall f_6703(C_word c,C_word *av) C_noret;
C_noret_decl(f_6723)
static void C_ccall f_6723(C_word c,C_word *av) C_noret;
C_noret_decl(f_6727)
static void C_ccall f_6727(C_word c,C_word *av) C_noret;
C_noret_decl(f_6734)
static void C_ccall f_6734(C_word c,C_word *av) C_noret;
C_noret_decl(f_6749)
static void C_ccall f_6749(C_word c,C_word *av) C_noret;
C_noret_decl(f_6767)
static void C_ccall f_6767(C_word c,C_word *av) C_noret;
C_noret_decl(f_6770)
static void C_fcall f_6770(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6780)
static void C_ccall f_6780(C_word c,C_word *av) C_noret;
C_noret_decl(f_6782)
static void C_ccall f_6782(C_word c,C_word *av) C_noret;
C_noret_decl(f_6801)
static void C_ccall f_6801(C_word c,C_word *av) C_noret;
C_noret_decl(f_6807)
static void C_ccall f_6807(C_word c,C_word *av) C_noret;
C_noret_decl(f_6817)
static void C_ccall f_6817(C_word c,C_word *av) C_noret;
C_noret_decl(f_6821)
static void C_ccall f_6821(C_word c,C_word *av) C_noret;
C_noret_decl(f_6831)
static void C_ccall f_6831(C_word c,C_word *av) C_noret;
C_noret_decl(f_6835)
static void C_ccall f_6835(C_word c,C_word *av) C_noret;
C_noret_decl(f_6841)
static void C_ccall f_6841(C_word c,C_word *av) C_noret;
C_noret_decl(f_6848)
static void C_ccall f_6848(C_word c,C_word *av) C_noret;
C_noret_decl(f_6861)
static void C_ccall f_6861(C_word c,C_word *av) C_noret;
C_noret_decl(f_6869)
static void C_ccall f_6869(C_word c,C_word *av) C_noret;
C_noret_decl(f_6873)
static void C_ccall f_6873(C_word c,C_word *av) C_noret;
C_noret_decl(f_6879)
static void C_ccall f_6879(C_word c,C_word *av) C_noret;
C_noret_decl(f_6889)
static void C_ccall f_6889(C_word c,C_word *av) C_noret;
C_noret_decl(f_6893)
static void C_ccall f_6893(C_word c,C_word *av) C_noret;
C_noret_decl(f_6911)
static void C_ccall f_6911(C_word c,C_word *av) C_noret;
C_noret_decl(f_6914)
static void C_ccall f_6914(C_word c,C_word *av) C_noret;
C_noret_decl(f_6923)
static void C_fcall f_6923(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6930)
static void C_ccall f_6930(C_word c,C_word *av) C_noret;
C_noret_decl(f_6932)
static void C_fcall f_6932(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6957)
static void C_ccall f_6957(C_word c,C_word *av) C_noret;
C_noret_decl(f_6990)
static void C_ccall f_6990(C_word c,C_word *av) C_noret;
C_noret_decl(f_6996)
static void C_ccall f_6996(C_word c,C_word *av) C_noret;
C_noret_decl(f_7002)
static void C_ccall f_7002(C_word c,C_word *av) C_noret;
C_noret_decl(f_7011)
static void C_ccall f_7011(C_word c,C_word *av) C_noret;
C_noret_decl(f_7023)
static void C_fcall f_7023(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7029)
static void C_ccall f_7029(C_word c,C_word *av) C_noret;
C_noret_decl(f_7035)
static void C_ccall f_7035(C_word c,C_word *av) C_noret;
C_noret_decl(f_7044)
static void C_ccall f_7044(C_word c,C_word *av) C_noret;
C_noret_decl(f_7046)
static void C_fcall f_7046(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7056)
static void C_ccall f_7056(C_word c,C_word *av) C_noret;
C_noret_decl(f_7070)
static void C_ccall f_7070(C_word c,C_word *av) C_noret;
C_noret_decl(f_7071)
static void C_ccall f_7071(C_word c,C_word *av) C_noret;
C_noret_decl(f_7088)
static void C_ccall f_7088(C_word c,C_word *av) C_noret;
C_noret_decl(f_7112)
static void C_ccall f_7112(C_word c,C_word *av) C_noret;
C_noret_decl(f_7122)
static void C_fcall f_7122(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7141)
static void C_ccall f_7141(C_word c,C_word *av) C_noret;
C_noret_decl(f_7149)
static void C_ccall f_7149(C_word c,C_word *av) C_noret;
C_noret_decl(f_7153)
static void C_ccall f_7153(C_word c,C_word *av) C_noret;
C_noret_decl(f_7160)
static void C_ccall f_7160(C_word c,C_word *av) C_noret;
C_noret_decl(f_7164)
static void C_ccall f_7164(C_word c,C_word *av) C_noret;
C_noret_decl(f_7173)
static void C_fcall f_7173(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7182)
static void C_ccall f_7182(C_word c,C_word *av) C_noret;
C_noret_decl(f_7188)
static void C_ccall f_7188(C_word c,C_word *av) C_noret;
C_noret_decl(f_7195)
static void C_ccall f_7195(C_word c,C_word *av) C_noret;
C_noret_decl(f_7238)
static void C_fcall f_7238(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8) C_noret;
C_noret_decl(f_7244)
static void C_fcall f_7244(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_7254)
static void C_ccall f_7254(C_word c,C_word *av) C_noret;
C_noret_decl(f_7264)
static void C_ccall f_7264(C_word c,C_word *av) C_noret;
C_noret_decl(f_7270)
static void C_ccall f_7270(C_word c,C_word *av) C_noret;
C_noret_decl(f_7298)
static void C_fcall f_7298(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7304)
static void C_ccall f_7304(C_word c,C_word *av) C_noret;
C_noret_decl(f_7308)
static void C_ccall f_7308(C_word c,C_word *av) C_noret;
C_noret_decl(f_7311)
static void C_ccall f_7311(C_word c,C_word *av) C_noret;
C_noret_decl(f_7314)
static void C_ccall f_7314(C_word c,C_word *av) C_noret;
C_noret_decl(f_7317)
static void C_ccall f_7317(C_word c,C_word *av) C_noret;
C_noret_decl(f_7320)
static void C_ccall f_7320(C_word c,C_word *av) C_noret;
C_noret_decl(f_7323)
static void C_ccall f_7323(C_word c,C_word *av) C_noret;
C_noret_decl(f_7352)
static void C_ccall f_7352(C_word c,C_word *av) C_noret;
C_noret_decl(f_7356)
static void C_ccall f_7356(C_word c,C_word *av) C_noret;
C_noret_decl(f_7361)
static void C_ccall f_7361(C_word c,C_word *av) C_noret;
C_noret_decl(f_7389)
static void C_fcall f_7389(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7399)
static void C_ccall f_7399(C_word c,C_word *av) C_noret;
C_noret_decl(f_7427)
static void C_fcall f_7427(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7467)
static C_word C_fcall f_7467(C_word t0);
C_noret_decl(f_7471)
static void C_fcall f_7471(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7478)
static void C_ccall f_7478(C_word c,C_word *av) C_noret;
C_noret_decl(f_7481)
static void C_ccall f_7481(C_word c,C_word *av) C_noret;
C_noret_decl(f_7484)
static void C_ccall f_7484(C_word c,C_word *av) C_noret;
C_noret_decl(f_7493)
static void C_ccall f_7493(C_word c,C_word *av) C_noret;
C_noret_decl(f_7494)
static void C_fcall f_7494(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7501)
static void C_ccall f_7501(C_word c,C_word *av) C_noret;
C_noret_decl(f_7507)
static void C_ccall f_7507(C_word c,C_word *av) C_noret;
C_noret_decl(f_7510)
static void C_ccall f_7510(C_word c,C_word *av) C_noret;
C_noret_decl(f_7516)
static void C_fcall f_7516(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7525)
static void C_ccall f_7525(C_word c,C_word *av) C_noret;
C_noret_decl(f_7537)
static void C_ccall f_7537(C_word c,C_word *av) C_noret;
C_noret_decl(f_7543)
static void C_fcall f_7543(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7552)
static void C_fcall f_7552(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7559)
static void C_ccall f_7559(C_word c,C_word *av) C_noret;
C_noret_decl(f_7562)
static void C_ccall f_7562(C_word c,C_word *av) C_noret;
C_noret_decl(f_7565)
static void C_ccall f_7565(C_word c,C_word *av) C_noret;
C_noret_decl(f_7580)
static void C_ccall f_7580(C_word c,C_word *av) C_noret;
C_noret_decl(f_7600)
static void C_ccall f_7600(C_word c,C_word *av) C_noret;
C_noret_decl(f_7604)
static void C_ccall f_7604(C_word c,C_word *av) C_noret;
C_noret_decl(f_7608)
static void C_ccall f_7608(C_word c,C_word *av) C_noret;
C_noret_decl(f_7618)
static void C_ccall f_7618(C_word c,C_word *av) C_noret;
C_noret_decl(f_7637)
static void C_ccall f_7637(C_word c,C_word *av) C_noret;
C_noret_decl(f_7641)
static void C_ccall f_7641(C_word c,C_word *av) C_noret;
C_noret_decl(f_7660)
static void C_ccall f_7660(C_word c,C_word *av) C_noret;
C_noret_decl(f_7680)
static void C_ccall f_7680(C_word c,C_word *av) C_noret;
C_noret_decl(f_7684)
static void C_ccall f_7684(C_word c,C_word *av) C_noret;
C_noret_decl(f_7691)
static void C_ccall f_7691(C_word c,C_word *av) C_noret;
C_noret_decl(f_7702)
static void C_fcall f_7702(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7712)
static void C_ccall f_7712(C_word c,C_word *av) C_noret;
C_noret_decl(f_7732)
static void C_ccall f_7732(C_word c,C_word *av) C_noret;
C_noret_decl(f_7735)
static void C_ccall f_7735(C_word c,C_word *av) C_noret;
C_noret_decl(f_7738)
static void C_ccall f_7738(C_word c,C_word *av) C_noret;
C_noret_decl(f_7753)
static void C_ccall f_7753(C_word c,C_word *av) C_noret;
C_noret_decl(f_7755)
static void C_fcall f_7755(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7761)
static void C_fcall f_7761(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7785)
static void C_ccall f_7785(C_word c,C_word *av) C_noret;
C_noret_decl(f_7798)
static void C_fcall f_7798(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7807)
static void C_ccall f_7807(C_word c,C_word *av) C_noret;
C_noret_decl(f_7810)
static void C_ccall f_7810(C_word c,C_word *av) C_noret;
C_noret_decl(f_7811)
static void C_fcall f_7811(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7821)
static void C_ccall f_7821(C_word c,C_word *av) C_noret;
C_noret_decl(f_7829)
static void C_ccall f_7829(C_word c,C_word *av) C_noret;
C_noret_decl(f_7837)
static void C_ccall f_7837(C_word c,C_word *av) C_noret;
C_noret_decl(f_7843)
static void C_ccall f_7843(C_word c,C_word *av) C_noret;
C_noret_decl(f_7856)
static void C_ccall f_7856(C_word c,C_word *av) C_noret;
C_noret_decl(f_7870)
static void C_ccall f_7870(C_word c,C_word *av) C_noret;
C_noret_decl(f_7871)
static void C_fcall f_7871(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7878)
static void C_ccall f_7878(C_word c,C_word *av) C_noret;
C_noret_decl(f_7886)
static void C_ccall f_7886(C_word c,C_word *av) C_noret;
C_noret_decl(f_7888)
static void C_fcall f_7888(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7901)
static void C_ccall f_7901(C_word c,C_word *av) C_noret;
C_noret_decl(f_7912)
static void C_ccall f_7912(C_word c,C_word *av) C_noret;
C_noret_decl(f_7918)
static void C_ccall f_7918(C_word c,C_word *av) C_noret;
C_noret_decl(f_7930)
static void C_ccall f_7930(C_word c,C_word *av) C_noret;
C_noret_decl(f_7933)
static void C_ccall f_7933(C_word c,C_word *av) C_noret;
C_noret_decl(f_7936)
static void C_ccall f_7936(C_word c,C_word *av) C_noret;
C_noret_decl(f_7945)
static void C_ccall f_7945(C_word c,C_word *av) C_noret;
C_noret_decl(f_7953)
static void C_fcall f_7953(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7969)
static void C_fcall f_7969(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7982)
static void C_fcall f_7982(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8006)
static void C_fcall f_8006(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8030)
static void C_ccall f_8030(C_word c,C_word *av) C_noret;
C_noret_decl(f_8041)
static void C_ccall f_8041(C_word c,C_word *av) C_noret;
C_noret_decl(f_8053)
static void C_fcall f_8053(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8061)
static void C_fcall f_8061(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8077)
static void C_ccall f_8077(C_word c,C_word *av) C_noret;
C_noret_decl(f_8097)
static void C_ccall f_8097(C_word c,C_word *av) C_noret;
C_noret_decl(f_8108)
static void C_ccall f_8108(C_word c,C_word *av) C_noret;
C_noret_decl(f_8139)
static void C_ccall f_8139(C_word c,C_word *av) C_noret;
C_noret_decl(f_8145)
static void C_fcall f_8145(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8155)
static void C_ccall f_8155(C_word c,C_word *av) C_noret;
C_noret_decl(f_8168)
static void C_fcall f_8168(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8178)
static void C_ccall f_8178(C_word c,C_word *av) C_noret;
C_noret_decl(f_8191)
static void C_fcall f_8191(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8201)
static void C_ccall f_8201(C_word c,C_word *av) C_noret;
C_noret_decl(f_8228)
static void C_fcall f_8228(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8278)
static void C_ccall f_8278(C_word c,C_word *av) C_noret;
C_noret_decl(f_8280)
static void C_ccall f_8280(C_word c,C_word *av) C_noret;
C_noret_decl(f_8284)
static void C_ccall f_8284(C_word c,C_word *av) C_noret;
C_noret_decl(f_8294)
static void C_ccall f_8294(C_word c,C_word *av) C_noret;
C_noret_decl(f_8303)
static C_word C_fcall f_8303(C_word t0,C_word t1);
C_noret_decl(f_8393)
static void C_ccall f_8393(C_word c,C_word *av) C_noret;
C_noret_decl(f_8396)
static void C_ccall f_8396(C_word c,C_word *av) C_noret;
C_noret_decl(f_8417)
static void C_ccall f_8417(C_word c,C_word *av) C_noret;
C_noret_decl(f_8467)
static void C_ccall f_8467(C_word c,C_word *av) C_noret;
C_noret_decl(f_8471)
static void C_ccall f_8471(C_word c,C_word *av) C_noret;
C_noret_decl(f_8475)
static void C_ccall f_8475(C_word c,C_word *av) C_noret;
C_noret_decl(f_8479)
static void C_ccall f_8479(C_word c,C_word *av) C_noret;
C_noret_decl(f_8483)
static void C_ccall f_8483(C_word c,C_word *av) C_noret;
C_noret_decl(f_8512)
static void C_ccall f_8512(C_word c,C_word *av) C_noret;
C_noret_decl(f_8539)
static void C_ccall f_8539(C_word c,C_word *av) C_noret;
C_noret_decl(f_8570)
static void C_ccall f_8570(C_word c,C_word *av) C_noret;
C_noret_decl(f_8587)
static void C_fcall f_8587(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8611)
static void C_ccall f_8611(C_word c,C_word *av) C_noret;
C_noret_decl(f_8622)
static void C_ccall f_8622(C_word c,C_word *av) C_noret;
C_noret_decl(f_8641)
static void C_ccall f_8641(C_word c,C_word *av) C_noret;
C_noret_decl(f_8684)
static void C_ccall f_8684(C_word c,C_word *av) C_noret;
C_noret_decl(f_8699)
static void C_ccall f_8699(C_word c,C_word *av) C_noret;
C_noret_decl(f_8707)
static void C_ccall f_8707(C_word c,C_word *av) C_noret;
C_noret_decl(f_8743)
static void C_ccall f_8743(C_word c,C_word *av) C_noret;
C_noret_decl(f_8776)
static void C_ccall f_8776(C_word c,C_word *av) C_noret;
C_noret_decl(f_8792)
static void C_ccall f_8792(C_word c,C_word *av) C_noret;
C_noret_decl(f_8838)
static void C_ccall f_8838(C_word c,C_word *av) C_noret;
C_noret_decl(f_8853)
static void C_ccall f_8853(C_word c,C_word *av) C_noret;
C_noret_decl(f_8865)
static void C_ccall f_8865(C_word c,C_word *av) C_noret;
C_noret_decl(f_8912)
static void C_ccall f_8912(C_word c,C_word *av) C_noret;
C_noret_decl(f_8927)
static void C_ccall f_8927(C_word c,C_word *av) C_noret;
C_noret_decl(f_8935)
static void C_ccall f_8935(C_word c,C_word *av) C_noret;
C_noret_decl(f_8943)
static void C_ccall f_8943(C_word c,C_word *av) C_noret;
C_noret_decl(f_8974)
static void C_ccall f_8974(C_word c,C_word *av) C_noret;
C_noret_decl(f_8995)
static void C_ccall f_8995(C_word c,C_word *av) C_noret;
C_noret_decl(f_9016)
static void C_ccall f_9016(C_word c,C_word *av) C_noret;
C_noret_decl(f_9025)
static void C_fcall f_9025(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9036)
static void C_ccall f_9036(C_word c,C_word *av) C_noret;
C_noret_decl(f_9045)
static void C_ccall f_9045(C_word c,C_word *av) C_noret;
C_noret_decl(f_9050)
static void C_fcall f_9050(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9075)
static void C_ccall f_9075(C_word c,C_word *av) C_noret;
C_noret_decl(f_9077)
static void C_ccall f_9077(C_word c,C_word *av) C_noret;
C_noret_decl(f_9117)
static void C_ccall f_9117(C_word c,C_word *av) C_noret;
C_noret_decl(f_9127)
static void C_fcall f_9127(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9152)
static void C_ccall f_9152(C_word c,C_word *av) C_noret;
C_noret_decl(f_9161)
static void C_fcall f_9161(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9186)
static void C_ccall f_9186(C_word c,C_word *av) C_noret;
C_noret_decl(f_9250)
static void C_ccall f_9250(C_word c,C_word *av) C_noret;
C_noret_decl(f_9259)
static void C_ccall f_9259(C_word c,C_word *av) C_noret;
C_noret_decl(f_9274)
static void C_ccall f_9274(C_word c,C_word *av) C_noret;
C_noret_decl(f_9282)
static void C_ccall f_9282(C_word c,C_word *av) C_noret;
C_noret_decl(f_9286)
static void C_ccall f_9286(C_word c,C_word *av) C_noret;
C_noret_decl(f_9339)
static void C_ccall f_9339(C_word c,C_word *av) C_noret;
C_noret_decl(f_9362)
static void C_ccall f_9362(C_word c,C_word *av) C_noret;
C_noret_decl(f_9366)
static void C_ccall f_9366(C_word c,C_word *av) C_noret;
C_noret_decl(f_9400)
static void C_ccall f_9400(C_word c,C_word *av) C_noret;
C_noret_decl(f_9446)
static void C_ccall f_9446(C_word c,C_word *av) C_noret;
C_noret_decl(f_9450)
static void C_ccall f_9450(C_word c,C_word *av) C_noret;
C_noret_decl(f_9479)
static void C_ccall f_9479(C_word c,C_word *av) C_noret;
C_noret_decl(f_9504)
static void C_fcall f_9504(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9508)
static void C_ccall f_9508(C_word c,C_word *av) C_noret;
C_noret_decl(f_9512)
static void C_ccall f_9512(C_word c,C_word *av) C_noret;
C_noret_decl(f_9549)
static void C_ccall f_9549(C_word c,C_word *av) C_noret;
C_noret_decl(f_9579)
static void C_ccall f_9579(C_word c,C_word *av) C_noret;
C_noret_decl(f_9632)
static void C_ccall f_9632(C_word c,C_word *av) C_noret;
C_noret_decl(f_9649)
static void C_ccall f_9649(C_word c,C_word *av) C_noret;
C_noret_decl(f_9653)
static void C_ccall f_9653(C_word c,C_word *av) C_noret;
C_noret_decl(f_9733)
static void C_ccall f_9733(C_word c,C_word *av) C_noret;
C_noret_decl(f_9754)
static void C_ccall f_9754(C_word c,C_word *av) C_noret;
C_noret_decl(f_9762)
static void C_ccall f_9762(C_word c,C_word *av) C_noret;
C_noret_decl(f_9845)
static void C_ccall f_9845(C_word c,C_word *av) C_noret;
C_noret_decl(f_9860)
static void C_ccall f_9860(C_word c,C_word *av) C_noret;
C_noret_decl(f_9913)
static void C_ccall f_9913(C_word c,C_word *av) C_noret;
C_noret_decl(f_9928)
static void C_ccall f_9928(C_word c,C_word *av) C_noret;
C_noret_decl(f_9947)
static void C_ccall f_9947(C_word c,C_word *av) C_noret;
C_noret_decl(f_9956)
static void C_ccall f_9956(C_word c,C_word *av) C_noret;
C_noret_decl(f_9972)
static void C_fcall f_9972(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_9994)
static void C_ccall f_9994(C_word c,C_word *av) C_noret;
C_noret_decl(f_9997)
static void C_ccall f_9997(C_word c,C_word *av) C_noret;
C_noret_decl(C_optimizer_toplevel)
C_externexport void C_ccall C_optimizer_toplevel(C_word c,C_word *av) C_noret;

C_noret_decl(trf_10456)
static void C_ccall trf_10456(C_word c,C_word *av) C_noret;
static void C_ccall trf_10456(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10456(t0,t1,t2,t3);}

C_noret_decl(trf_10579)
static void C_ccall trf_10579(C_word c,C_word *av) C_noret;
static void C_ccall trf_10579(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_10579(t0,t1,t2,t3,t4);}

C_noret_decl(trf_10749)
static void C_ccall trf_10749(C_word c,C_word *av) C_noret;
static void C_ccall trf_10749(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10749(t0,t1,t2);}

C_noret_decl(trf_10761)
static void C_ccall trf_10761(C_word c,C_word *av) C_noret;
static void C_ccall trf_10761(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10761(t0,t1,t2);}

C_noret_decl(trf_10790)
static void C_ccall trf_10790(C_word c,C_word *av) C_noret;
static void C_ccall trf_10790(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_10790(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_10793)
static void C_ccall trf_10793(C_word c,C_word *av) C_noret;
static void C_ccall trf_10793(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_10793(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_10951)
static void C_ccall trf_10951(C_word c,C_word *av) C_noret;
static void C_ccall trf_10951(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10951(t0,t1);}

C_noret_decl(trf_10990)
static void C_ccall trf_10990(C_word c,C_word *av) C_noret;
static void C_ccall trf_10990(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10990(t0,t1);}

C_noret_decl(trf_11037)
static void C_ccall trf_11037(C_word c,C_word *av) C_noret;
static void C_ccall trf_11037(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11037(t0,t1);}

C_noret_decl(trf_11221)
static void C_ccall trf_11221(C_word c,C_word *av) C_noret;
static void C_ccall trf_11221(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_11221(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_11241)
static void C_ccall trf_11241(C_word c,C_word *av) C_noret;
static void C_ccall trf_11241(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11241(t0,t1);}

C_noret_decl(trf_11260)
static void C_ccall trf_11260(C_word c,C_word *av) C_noret;
static void C_ccall trf_11260(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11260(t0,t1,t2);}

C_noret_decl(trf_11392)
static void C_ccall trf_11392(C_word c,C_word *av) C_noret;
static void C_ccall trf_11392(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11392(t0,t1,t2);}

C_noret_decl(trf_11415)
static void C_ccall trf_11415(C_word c,C_word *av) C_noret;
static void C_ccall trf_11415(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11415(t0,t1,t2);}

C_noret_decl(trf_11484)
static void C_ccall trf_11484(C_word c,C_word *av) C_noret;
static void C_ccall trf_11484(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11484(t0,t1,t2);}

C_noret_decl(trf_11534)
static void C_ccall trf_11534(C_word c,C_word *av) C_noret;
static void C_ccall trf_11534(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11534(t0,t1,t2);}

C_noret_decl(trf_11608)
static void C_ccall trf_11608(C_word c,C_word *av) C_noret;
static void C_ccall trf_11608(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11608(t0,t1,t2);}

C_noret_decl(trf_11747)
static void C_ccall trf_11747(C_word c,C_word *av) C_noret;
static void C_ccall trf_11747(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11747(t0,t1,t2);}

C_noret_decl(trf_11776)
static void C_ccall trf_11776(C_word c,C_word *av) C_noret;
static void C_ccall trf_11776(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11776(t0,t1,t2);}

C_noret_decl(trf_11925)
static void C_ccall trf_11925(C_word c,C_word *av) C_noret;
static void C_ccall trf_11925(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11925(t0,t1,t2,t3);}

C_noret_decl(trf_11956)
static void C_ccall trf_11956(C_word c,C_word *av) C_noret;
static void C_ccall trf_11956(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11956(t0,t1);}

C_noret_decl(trf_11970)
static void C_ccall trf_11970(C_word c,C_word *av) C_noret;
static void C_ccall trf_11970(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11970(t0,t1);}

C_noret_decl(trf_12119)
static void C_ccall trf_12119(C_word c,C_word *av) C_noret;
static void C_ccall trf_12119(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12119(t0,t1,t2);}

C_noret_decl(trf_12131)
static void C_ccall trf_12131(C_word c,C_word *av) C_noret;
static void C_ccall trf_12131(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12131(t0,t1,t2);}

C_noret_decl(trf_12168)
static void C_ccall trf_12168(C_word c,C_word *av) C_noret;
static void C_ccall trf_12168(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12168(t0,t1,t2);}

C_noret_decl(trf_12183)
static void C_ccall trf_12183(C_word c,C_word *av) C_noret;
static void C_ccall trf_12183(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12183(t0,t1,t2);}

C_noret_decl(trf_12232)
static void C_ccall trf_12232(C_word c,C_word *av) C_noret;
static void C_ccall trf_12232(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12232(t0,t1,t2);}

C_noret_decl(trf_12294)
static void C_ccall trf_12294(C_word c,C_word *av) C_noret;
static void C_ccall trf_12294(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12294(t0,t1,t2);}

C_noret_decl(trf_12299)
static void C_ccall trf_12299(C_word c,C_word *av) C_noret;
static void C_ccall trf_12299(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12299(t0,t1,t2);}

C_noret_decl(trf_12371)
static void C_ccall trf_12371(C_word c,C_word *av) C_noret;
static void C_ccall trf_12371(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12371(t0,t1,t2);}

C_noret_decl(trf_12401)
static void C_ccall trf_12401(C_word c,C_word *av) C_noret;
static void C_ccall trf_12401(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12401(t0,t1,t2);}

C_noret_decl(trf_12424)
static void C_ccall trf_12424(C_word c,C_word *av) C_noret;
static void C_ccall trf_12424(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12424(t0,t1,t2);}

C_noret_decl(trf_12434)
static void C_ccall trf_12434(C_word c,C_word *av) C_noret;
static void C_ccall trf_12434(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12434(t0,t1);}

C_noret_decl(trf_12574)
static void C_ccall trf_12574(C_word c,C_word *av) C_noret;
static void C_ccall trf_12574(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12574(t0,t1,t2,t3);}

C_noret_decl(trf_12667)
static void C_ccall trf_12667(C_word c,C_word *av) C_noret;
static void C_ccall trf_12667(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12667(t0,t1,t2);}

C_noret_decl(trf_12734)
static void C_ccall trf_12734(C_word c,C_word *av) C_noret;
static void C_ccall trf_12734(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12734(t0,t1,t2);}

C_noret_decl(trf_12768)
static void C_ccall trf_12768(C_word c,C_word *av) C_noret;
static void C_ccall trf_12768(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12768(t0,t1,t2);}

C_noret_decl(trf_12815)
static void C_ccall trf_12815(C_word c,C_word *av) C_noret;
static void C_ccall trf_12815(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12815(t0,t1,t2);}

C_noret_decl(trf_12904)
static void C_ccall trf_12904(C_word c,C_word *av) C_noret;
static void C_ccall trf_12904(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12904(t0,t1,t2);}

C_noret_decl(trf_12912)
static void C_ccall trf_12912(C_word c,C_word *av) C_noret;
static void C_ccall trf_12912(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_12912(t0,t1,t2,t3);}

C_noret_decl(trf_13096)
static void C_ccall trf_13096(C_word c,C_word *av) C_noret;
static void C_ccall trf_13096(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13096(t0,t1,t2,t3,t4);}

C_noret_decl(trf_13119)
static void C_ccall trf_13119(C_word c,C_word *av) C_noret;
static void C_ccall trf_13119(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13119(t0,t1);}

C_noret_decl(trf_13241)
static void C_ccall trf_13241(C_word c,C_word *av) C_noret;
static void C_ccall trf_13241(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13241(t0,t1,t2,t3,t4);}

C_noret_decl(trf_13280)
static void C_ccall trf_13280(C_word c,C_word *av) C_noret;
static void C_ccall trf_13280(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13280(t0,t1);}

C_noret_decl(trf_13492)
static void C_ccall trf_13492(C_word c,C_word *av) C_noret;
static void C_ccall trf_13492(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13492(t0,t1,t2,t3);}

C_noret_decl(trf_13595)
static void C_ccall trf_13595(C_word c,C_word *av) C_noret;
static void C_ccall trf_13595(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13595(t0,t1,t2,t3,t4);}

C_noret_decl(trf_13611)
static void C_ccall trf_13611(C_word c,C_word *av) C_noret;
static void C_ccall trf_13611(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13611(t0,t1);}

C_noret_decl(trf_14105)
static void C_ccall trf_14105(C_word c,C_word *av) C_noret;
static void C_ccall trf_14105(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14105(t0,t1,t2);}

C_noret_decl(trf_3407)
static void C_ccall trf_3407(C_word c,C_word *av) C_noret;
static void C_ccall trf_3407(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_3407(t0,t1,t2,t3,t4);}

C_noret_decl(trf_3509)
static void C_ccall trf_3509(C_word c,C_word *av) C_noret;
static void C_ccall trf_3509(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3509(t0,t1,t2);}

C_noret_decl(trf_3561)
static void C_ccall trf_3561(C_word c,C_word *av) C_noret;
static void C_ccall trf_3561(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3561(t0,t1,t2);}

C_noret_decl(trf_3567)
static void C_ccall trf_3567(C_word c,C_word *av) C_noret;
static void C_ccall trf_3567(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_3567(t0,t1,t2,t3,t4);}

C_noret_decl(trf_3600)
static void C_ccall trf_3600(C_word c,C_word *av) C_noret;
static void C_ccall trf_3600(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_3600(t0,t1,t2,t3);}

C_noret_decl(trf_3615)
static void C_ccall trf_3615(C_word c,C_word *av) C_noret;
static void C_ccall trf_3615(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3615(t0,t1,t2);}

C_noret_decl(trf_3623)
static void C_ccall trf_3623(C_word c,C_word *av) C_noret;
static void C_ccall trf_3623(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_3623(t0,t1,t2,t3);}

C_noret_decl(trf_3655)
static void C_ccall trf_3655(C_word c,C_word *av) C_noret;
static void C_ccall trf_3655(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3655(t0,t1,t2);}

C_noret_decl(trf_3687)
static void C_ccall trf_3687(C_word c,C_word *av) C_noret;
static void C_ccall trf_3687(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3687(t0,t1,t2);}

C_noret_decl(trf_3736)
static void C_ccall trf_3736(C_word c,C_word *av) C_noret;
static void C_ccall trf_3736(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3736(t0,t1,t2);}

C_noret_decl(trf_3770)
static void C_ccall trf_3770(C_word c,C_word *av) C_noret;
static void C_ccall trf_3770(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3770(t0,t1,t2);}

C_noret_decl(trf_3776)
static void C_ccall trf_3776(C_word c,C_word *av) C_noret;
static void C_ccall trf_3776(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3776(t0,t1,t2);}

C_noret_decl(trf_3804)
static void C_ccall trf_3804(C_word c,C_word *av) C_noret;
static void C_ccall trf_3804(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3804(t0,t1,t2);}

C_noret_decl(trf_3810)
static void C_ccall trf_3810(C_word c,C_word *av) C_noret;
static void C_ccall trf_3810(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3810(t0,t1,t2);}

C_noret_decl(trf_3834)
static void C_ccall trf_3834(C_word c,C_word *av) C_noret;
static void C_ccall trf_3834(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3834(t0,t1,t2);}

C_noret_decl(trf_3840)
static void C_ccall trf_3840(C_word c,C_word *av) C_noret;
static void C_ccall trf_3840(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_3840(t0,t1,t2,t3);}

C_noret_decl(trf_3890)
static void C_ccall trf_3890(C_word c,C_word *av) C_noret;
static void C_ccall trf_3890(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_3890(t0,t1,t2,t3);}

C_noret_decl(trf_3896)
static void C_ccall trf_3896(C_word c,C_word *av) C_noret;
static void C_ccall trf_3896(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3896(t0,t1,t2);}

C_noret_decl(trf_3971)
static void C_ccall trf_3971(C_word c,C_word *av) C_noret;
static void C_ccall trf_3971(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_3971(t0,t1,t2);}

C_noret_decl(trf_4010)
static void C_ccall trf_4010(C_word c,C_word *av) C_noret;
static void C_ccall trf_4010(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4010(t0,t1,t2);}

C_noret_decl(trf_4019)
static void C_ccall trf_4019(C_word c,C_word *av) C_noret;
static void C_ccall trf_4019(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4019(t0,t1,t2);}

C_noret_decl(trf_4027)
static void C_ccall trf_4027(C_word c,C_word *av) C_noret;
static void C_ccall trf_4027(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4027(t0,t1,t2,t3);}

C_noret_decl(trf_4063)
static void C_ccall trf_4063(C_word c,C_word *av) C_noret;
static void C_ccall trf_4063(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4063(t0,t1,t2);}

C_noret_decl(trf_4071)
static void C_ccall trf_4071(C_word c,C_word *av) C_noret;
static void C_ccall trf_4071(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4071(t0,t1,t2,t3);}

C_noret_decl(trf_4136)
static void C_ccall trf_4136(C_word c,C_word *av) C_noret;
static void C_ccall trf_4136(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4136(t0,t1,t2);}

C_noret_decl(trf_4235)
static void C_ccall trf_4235(C_word c,C_word *av) C_noret;
static void C_ccall trf_4235(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4235(t0,t1,t2,t3);}

C_noret_decl(trf_4412)
static void C_ccall trf_4412(C_word c,C_word *av) C_noret;
static void C_ccall trf_4412(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4412(t0,t1,t2);}

C_noret_decl(trf_4418)
static void C_ccall trf_4418(C_word c,C_word *av) C_noret;
static void C_ccall trf_4418(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4418(t0,t1,t2);}

C_noret_decl(trf_4721)
static void C_ccall trf_4721(C_word c,C_word *av) C_noret;
static void C_ccall trf_4721(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4721(t0,t1,t2,t3);}

C_noret_decl(trf_4733)
static void C_ccall trf_4733(C_word c,C_word *av) C_noret;
static void C_ccall trf_4733(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_4733(t0,t1,t2,t3,t4);}

C_noret_decl(trf_4735)
static void C_ccall trf_4735(C_word c,C_word *av) C_noret;
static void C_ccall trf_4735(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4735(t0,t1,t2);}

C_noret_decl(trf_4751)
static void C_ccall trf_4751(C_word c,C_word *av) C_noret;
static void C_ccall trf_4751(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_4751(t0,t1,t2);}

C_noret_decl(trf_4774)
static void C_ccall trf_4774(C_word c,C_word *av) C_noret;
static void C_ccall trf_4774(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_4774(t0,t1,t2,t3);}

C_noret_decl(trf_4796)
static void C_ccall trf_4796(C_word c,C_word *av) C_noret;
static void C_ccall trf_4796(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_4796(t0,t1);}

C_noret_decl(trf_4844)
static void C_ccall trf_4844(C_word c,C_word *av) C_noret;
static void C_ccall trf_4844(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_4844(t0,t1);}

C_noret_decl(trf_5011)
static void C_ccall trf_5011(C_word c,C_word *av) C_noret;
static void C_ccall trf_5011(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5011(t0,t1,t2);}

C_noret_decl(trf_5054)
static void C_ccall trf_5054(C_word c,C_word *av) C_noret;
static void C_ccall trf_5054(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_5054(t0,t1,t2,t3);}

C_noret_decl(trf_5084)
static void C_ccall trf_5084(C_word c,C_word *av) C_noret;
static void C_ccall trf_5084(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5084(t0,t1);}

C_noret_decl(trf_5121)
static void C_ccall trf_5121(C_word c,C_word *av) C_noret;
static void C_ccall trf_5121(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5121(t0,t1,t2);}

C_noret_decl(trf_5207)
static void C_ccall trf_5207(C_word c,C_word *av) C_noret;
static void C_ccall trf_5207(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5207(t0,t1,t2);}

C_noret_decl(trf_5249)
static void C_ccall trf_5249(C_word c,C_word *av) C_noret;
static void C_ccall trf_5249(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5249(t0,t1,t2);}

C_noret_decl(trf_5268)
static void C_ccall trf_5268(C_word c,C_word *av) C_noret;
static void C_ccall trf_5268(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5268(t0,t1);}

C_noret_decl(trf_5411)
static void C_ccall trf_5411(C_word c,C_word *av) C_noret;
static void C_ccall trf_5411(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_5411(t0,t1,t2,t3,t4);}

C_noret_decl(trf_5507)
static void C_ccall trf_5507(C_word c,C_word *av) C_noret;
static void C_ccall trf_5507(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5507(t0,t1);}

C_noret_decl(trf_5549)
static void C_ccall trf_5549(C_word c,C_word *av) C_noret;
static void C_ccall trf_5549(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5549(t0,t1,t2);}

C_noret_decl(trf_5557)
static void C_ccall trf_5557(C_word c,C_word *av) C_noret;
static void C_ccall trf_5557(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5557(t0,t1,t2);}

C_noret_decl(trf_5569)
static void C_ccall trf_5569(C_word c,C_word *av) C_noret;
static void C_ccall trf_5569(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_5569(t0,t1,t2,t3,t4);}

C_noret_decl(trf_5638)
static void C_ccall trf_5638(C_word c,C_word *av) C_noret;
static void C_ccall trf_5638(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5638(t0,t1,t2);}

C_noret_decl(trf_5648)
static void C_ccall trf_5648(C_word c,C_word *av) C_noret;
static void C_ccall trf_5648(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5648(t0,t1);}

C_noret_decl(trf_5704)
static void C_ccall trf_5704(C_word c,C_word *av) C_noret;
static void C_ccall trf_5704(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5704(t0,t1);}

C_noret_decl(trf_5719)
static void C_ccall trf_5719(C_word c,C_word *av) C_noret;
static void C_ccall trf_5719(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_5719(t0,t1);}

C_noret_decl(trf_5727)
static void C_ccall trf_5727(C_word c,C_word *av) C_noret;
static void C_ccall trf_5727(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5727(t0,t1,t2);}

C_noret_decl(trf_5739)
static void C_ccall trf_5739(C_word c,C_word *av) C_noret;
static void C_ccall trf_5739(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5739(t0,t1,t2);}

C_noret_decl(trf_5863)
static void C_ccall trf_5863(C_word c,C_word *av) C_noret;
static void C_ccall trf_5863(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_5863(t0,t1,t2);}

C_noret_decl(trf_6041)
static void C_ccall trf_6041(C_word c,C_word *av) C_noret;
static void C_ccall trf_6041(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6041(t0,t1);}

C_noret_decl(trf_6060)
static void C_ccall trf_6060(C_word c,C_word *av) C_noret;
static void C_ccall trf_6060(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6060(t0,t1,t2);}

C_noret_decl(trf_6087)
static void C_ccall trf_6087(C_word c,C_word *av) C_noret;
static void C_ccall trf_6087(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6087(t0,t1,t2);}

C_noret_decl(trf_6123)
static void C_ccall trf_6123(C_word c,C_word *av) C_noret;
static void C_ccall trf_6123(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6123(t0,t1);}

C_noret_decl(trf_6151)
static void C_ccall trf_6151(C_word c,C_word *av) C_noret;
static void C_ccall trf_6151(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6151(t0,t1);}

C_noret_decl(trf_6175)
static void C_ccall trf_6175(C_word c,C_word *av) C_noret;
static void C_ccall trf_6175(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6175(t0,t1);}

C_noret_decl(trf_6185)
static void C_ccall trf_6185(C_word c,C_word *av) C_noret;
static void C_ccall trf_6185(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6185(t0,t1,t2);}

C_noret_decl(trf_6244)
static void C_ccall trf_6244(C_word c,C_word *av) C_noret;
static void C_ccall trf_6244(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6244(t0,t1,t2);}

C_noret_decl(trf_6299)
static void C_ccall trf_6299(C_word c,C_word *av) C_noret;
static void C_ccall trf_6299(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_6299(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_6316)
static void C_ccall trf_6316(C_word c,C_word *av) C_noret;
static void C_ccall trf_6316(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6316(t0,t1,t2);}

C_noret_decl(trf_6334)
static void C_ccall trf_6334(C_word c,C_word *av) C_noret;
static void C_ccall trf_6334(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6334(t0,t1,t2);}

C_noret_decl(trf_6512)
static void C_ccall trf_6512(C_word c,C_word *av) C_noret;
static void C_ccall trf_6512(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6512(t0,t1,t2);}

C_noret_decl(trf_6524)
static void C_ccall trf_6524(C_word c,C_word *av) C_noret;
static void C_ccall trf_6524(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6524(t0,t1,t2);}

C_noret_decl(trf_6770)
static void C_ccall trf_6770(C_word c,C_word *av) C_noret;
static void C_ccall trf_6770(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6770(t0,t1);}

C_noret_decl(trf_6923)
static void C_ccall trf_6923(C_word c,C_word *av) C_noret;
static void C_ccall trf_6923(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6923(t0,t1,t2);}

C_noret_decl(trf_6932)
static void C_ccall trf_6932(C_word c,C_word *av) C_noret;
static void C_ccall trf_6932(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6932(t0,t1,t2);}

C_noret_decl(trf_7023)
static void C_ccall trf_7023(C_word c,C_word *av) C_noret;
static void C_ccall trf_7023(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7023(t0,t1);}

C_noret_decl(trf_7046)
static void C_ccall trf_7046(C_word c,C_word *av) C_noret;
static void C_ccall trf_7046(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7046(t0,t1,t2);}

C_noret_decl(trf_7122)
static void C_ccall trf_7122(C_word c,C_word *av) C_noret;
static void C_ccall trf_7122(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7122(t0,t1);}

C_noret_decl(trf_7173)
static void C_ccall trf_7173(C_word c,C_word *av) C_noret;
static void C_ccall trf_7173(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7173(t0,t1);}

C_noret_decl(trf_7238)
static void C_ccall trf_7238(C_word c,C_word *av) C_noret;
static void C_ccall trf_7238(C_word c,C_word *av){
C_word t0=av[8];
C_word t1=av[7];
C_word t2=av[6];
C_word t3=av[5];
C_word t4=av[4];
C_word t5=av[3];
C_word t6=av[2];
C_word t7=av[1];
C_word t8=av[0];
f_7238(t0,t1,t2,t3,t4,t5,t6,t7,t8);}

C_noret_decl(trf_7244)
static void C_ccall trf_7244(C_word c,C_word *av) C_noret;
static void C_ccall trf_7244(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_7244(t0,t1,t2,t3,t4);}

C_noret_decl(trf_7298)
static void C_ccall trf_7298(C_word c,C_word *av) C_noret;
static void C_ccall trf_7298(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7298(t0,t1);}

C_noret_decl(trf_7389)
static void C_ccall trf_7389(C_word c,C_word *av) C_noret;
static void C_ccall trf_7389(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7389(t0,t1,t2);}

C_noret_decl(trf_7427)
static void C_ccall trf_7427(C_word c,C_word *av) C_noret;
static void C_ccall trf_7427(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7427(t0,t1,t2,t3);}

C_noret_decl(trf_7471)
static void C_ccall trf_7471(C_word c,C_word *av) C_noret;
static void C_ccall trf_7471(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7471(t0,t1,t2,t3);}

C_noret_decl(trf_7494)
static void C_ccall trf_7494(C_word c,C_word *av) C_noret;
static void C_ccall trf_7494(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7494(t0,t1,t2);}

C_noret_decl(trf_7516)
static void C_ccall trf_7516(C_word c,C_word *av) C_noret;
static void C_ccall trf_7516(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7516(t0,t1);}

C_noret_decl(trf_7543)
static void C_ccall trf_7543(C_word c,C_word *av) C_noret;
static void C_ccall trf_7543(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7543(t0,t1);}

C_noret_decl(trf_7552)
static void C_ccall trf_7552(C_word c,C_word *av) C_noret;
static void C_ccall trf_7552(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7552(t0,t1);}

C_noret_decl(trf_7702)
static void C_ccall trf_7702(C_word c,C_word *av) C_noret;
static void C_ccall trf_7702(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7702(t0,t1,t2);}

C_noret_decl(trf_7755)
static void C_ccall trf_7755(C_word c,C_word *av) C_noret;
static void C_ccall trf_7755(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7755(t0,t1,t2,t3);}

C_noret_decl(trf_7761)
static void C_ccall trf_7761(C_word c,C_word *av) C_noret;
static void C_ccall trf_7761(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7761(t0,t1,t2,t3);}

C_noret_decl(trf_7798)
static void C_ccall trf_7798(C_word c,C_word *av) C_noret;
static void C_ccall trf_7798(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7798(t0,t1,t2,t3);}

C_noret_decl(trf_7811)
static void C_ccall trf_7811(C_word c,C_word *av) C_noret;
static void C_ccall trf_7811(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7811(t0,t1,t2);}

C_noret_decl(trf_7871)
static void C_ccall trf_7871(C_word c,C_word *av) C_noret;
static void C_ccall trf_7871(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7871(t0,t1,t2);}

C_noret_decl(trf_7888)
static void C_ccall trf_7888(C_word c,C_word *av) C_noret;
static void C_ccall trf_7888(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7888(t0,t1,t2);}

C_noret_decl(trf_7953)
static void C_ccall trf_7953(C_word c,C_word *av) C_noret;
static void C_ccall trf_7953(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7953(t0,t1,t2,t3);}

C_noret_decl(trf_7969)
static void C_ccall trf_7969(C_word c,C_word *av) C_noret;
static void C_ccall trf_7969(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7969(t0,t1,t2,t3);}

C_noret_decl(trf_7982)
static void C_ccall trf_7982(C_word c,C_word *av) C_noret;
static void C_ccall trf_7982(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7982(t0,t1);}

C_noret_decl(trf_8006)
static void C_ccall trf_8006(C_word c,C_word *av) C_noret;
static void C_ccall trf_8006(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8006(t0,t1,t2);}

C_noret_decl(trf_8053)
static void C_ccall trf_8053(C_word c,C_word *av) C_noret;
static void C_ccall trf_8053(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8053(t0,t1,t2);}

C_noret_decl(trf_8061)
static void C_ccall trf_8061(C_word c,C_word *av) C_noret;
static void C_ccall trf_8061(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8061(t0,t1,t2,t3);}

C_noret_decl(trf_8145)
static void C_ccall trf_8145(C_word c,C_word *av) C_noret;
static void C_ccall trf_8145(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8145(t0,t1,t2);}

C_noret_decl(trf_8168)
static void C_ccall trf_8168(C_word c,C_word *av) C_noret;
static void C_ccall trf_8168(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8168(t0,t1,t2);}

C_noret_decl(trf_8191)
static void C_ccall trf_8191(C_word c,C_word *av) C_noret;
static void C_ccall trf_8191(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8191(t0,t1,t2,t3);}

C_noret_decl(trf_8228)
static void C_ccall trf_8228(C_word c,C_word *av) C_noret;
static void C_ccall trf_8228(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8228(t0,t1,t2,t3);}

C_noret_decl(trf_8587)
static void C_ccall trf_8587(C_word c,C_word *av) C_noret;
static void C_ccall trf_8587(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8587(t0,t1,t2);}

C_noret_decl(trf_9025)
static void C_ccall trf_9025(C_word c,C_word *av) C_noret;
static void C_ccall trf_9025(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9025(t0,t1);}

C_noret_decl(trf_9050)
static void C_ccall trf_9050(C_word c,C_word *av) C_noret;
static void C_ccall trf_9050(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9050(t0,t1,t2,t3);}

C_noret_decl(trf_9127)
static void C_ccall trf_9127(C_word c,C_word *av) C_noret;
static void C_ccall trf_9127(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9127(t0,t1,t2);}

C_noret_decl(trf_9161)
static void C_ccall trf_9161(C_word c,C_word *av) C_noret;
static void C_ccall trf_9161(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9161(t0,t1,t2);}

C_noret_decl(trf_9504)
static void C_ccall trf_9504(C_word c,C_word *av) C_noret;
static void C_ccall trf_9504(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9504(t0,t1);}

C_noret_decl(trf_9972)
static void C_ccall trf_9972(C_word c,C_word *av) C_noret;
static void C_ccall trf_9972(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_9972(t0,t1,t2,t3,t4,t5);}

/* k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(40,c,2)))){
C_save_and_reclaim((void *)f_10000,c,av);}
a=C_alloc(40);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10015,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[6]);
t5=C_a_i_list4(&a,4,t1,C_SCHEME_TRUE,t4,C_fix(0));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10041,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10053,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=t6,a[5]=t7,a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1405: chicken.compiler.support#varnode */
t9=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k10013 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_10015,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10019,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_car(((C_word*)t0)[4]);
t4=C_u_i_cdr(((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10029,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[6],a[5]=t3,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1409: chicken.compiler.support#varnode */
t6=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k10017 in k10013 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10019,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1398: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10027 in k10013 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10029,c,av);}
/* optimizer.scm:1406: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9972(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1);}

/* k10039 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10041,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1401: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[29];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10051 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10053,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10057,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1405: chicken.compiler.support#varnode */
t3=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10055 in k10051 in k9998 in k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10057(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_10057,c,av);}
a=C_alloc(12);
t2=C_a_i_list4(&a,4,((C_word*)t0)[2],((C_word*)t0)[3],t1,((C_word*)t0)[4]);
/* optimizer.scm:1403: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[6];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10099(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_10099,c,av);}
a=C_alloc(22);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10114,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_cadr(((C_word*)t0)[4]);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10122,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10124,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10132,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1423: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t6;
av2[2]=t7;
av2[3]=t8;
C_call_with_values(4,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k10112 in k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10114,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1418: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10120 in k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10122,c,av);}
/* optimizer.scm:1421: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a10123 in k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10124(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10124,c,av);}
a=C_alloc(5);
t2=C_a_i_fixnum_difference(&a,2,((C_word*)t0)[2],C_fix(1));
/* optimizer.scm:1423: split-at */
f_3561(t1,((C_word*)t0)[3],t2);}

/* a10131 in k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10132,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10144,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1425: chicken.compiler.support#qnode */
t5=*((C_word*)lf[77]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_caddr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k10142 in a10131 in k10097 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10144,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1424: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_10171,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cddddr(((C_word*)t0)[2]);
t4=C_i_car(t3);
t5=C_i_cadddr(((C_word*)t0)[2]);
t6=(C_truep(*((C_word*)lf[232]+1))?C_i_caddr(((C_word*)t0)[2]):C_i_cadr(((C_word*)t0)[2]));
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10186,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t6,a[6]=t5,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10280,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1441: filter */
f_4010(t7,t8,((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k10184 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_10186,c,av);}
a=C_alloc(13);
if(C_truep(C_i_nullp(t1))){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10207,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1446: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=C_i_car(t1);
t5=C_a_i_list2(&a,2,((C_word*)t0)[2],t4);
/* optimizer.scm:1448: chicken.compiler.support#make-node */
t6=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=t3;
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10243,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10245,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1454: chicken.compiler.support#fold-inner */
t6=*((C_word*)lf[242]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}

/* k10205 in k10184 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10207,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1446: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10241 in k10184 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10243,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1450: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a10244 in k10184 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_10245,c,av);}
a=C_alloc(12);
t4=C_eqp(*((C_word*)lf[233]+1),lf[235]);
if(C_truep(t4)){
t5=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t6=C_a_i_list2(&a,2,t2,t3);
/* optimizer.scm:1457: chicken.compiler.support#make-node */
t7=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[58];
av2[3]=t5;
av2[4]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t5=C_a_i_list2(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t6=C_a_i_list2(&a,2,t2,t3);
/* optimizer.scm:1458: chicken.compiler.support#make-node */
t7=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[121];
av2[3]=t5;
av2[4]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}}

/* a10279 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10280,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10306,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1443: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k10300 in k10304 in a10279 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10302,c,av);}
t2=C_i_car(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_not(C_eqp(((C_word*)t0)[3],t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10304 in a10279 in k10169 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10306,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[76],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10302,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1444: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10340 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_10342,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(C_truep(t2)?t2:*((C_word*)lf[232]+1));
if(C_truep(t3)){
t4=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10366,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_eqp(*((C_word*)lf[233]+1),lf[235]);
if(C_truep(t6)){
t7=C_i_cddddr(((C_word*)t0)[2]);
t8=C_i_car(t7);
t9=C_a_i_list1(&a,1,t8);
/* optimizer.scm:1475: chicken.compiler.support#make-node */
t10=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t5;
av2[2]=lf[58];
av2[3]=t9;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t7=C_i_cadr(((C_word*)t0)[2]);
t8=C_a_i_list2(&a,2,t7,((C_word*)t0)[6]);
/* optimizer.scm:1479: chicken.compiler.support#make-node */
t9=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t5;
av2[2]=lf[121];
av2[3]=t8;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k10364 in k10340 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10366,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1471: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_10406,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_length(((C_word*)t0)[3]);
t4=C_i_car(((C_word*)t0)[2]);
if(C_truep(C_i_greater_or_equalp(t3,t4))){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=C_a_i_list2(&a,2,C_SCHEME_TRUE,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10426,a[2]=((C_word*)t0)[4],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10430,a[2]=t7,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1496: chicken.compiler.support#varnode */
t9=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k10424 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10426(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10426,c,av);}
/* optimizer.scm:1493: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_10430,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10434,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10436,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10442,a[2]=((C_word*)t0)[6],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1498: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}

/* k10432 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10434,c,av);}
a=C_alloc(6);
/* optimizer.scm:1495: cons* */
f_3834(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,((C_word*)t0)[4],t1));}

/* a10435 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10436,c,av);}
/* optimizer.scm:1498: split-at */
f_3561(t1,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_10442,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10450,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=C_i_cddr(((C_word*)t0)[2]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10456,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_10456(t9,t4,t3,t5);}

/* k10448 in a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10450,c,av);}
/* optimizer.scm:1499: scheme#append */
t2=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* loop in a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10456(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_10456,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10476,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_car(t3);
if(C_truep(C_i_symbolp(t5))){
/* optimizer.scm:1107: chicken.compiler.support#varnode */
t6=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t6=C_i_pairp(t5);
t7=(C_truep(t6)?C_eqp(lf[76],C_u_i_car(t5)):C_SCHEME_FALSE);
if(C_truep(t7)){
/* optimizer.scm:1108: chicken.compiler.support#qnode */
t8=*((C_word*)lf[77]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t4;
av2[2]=C_i_cadr(t5);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
/* optimizer.scm:1109: chicken.compiler.support#qnode */
t8=*((C_word*)lf[77]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}}}
else{
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10503,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1508: loop */
t9=t5;
t10=C_u_i_cdr(t2);
t11=C_i_cdr(t3);
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* k10474 in loop in a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_10476,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10480,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1506: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_10456(t3,t2,C_SCHEME_END_OF_LIST,C_u_i_cdr(((C_word*)t0)[4]));}

/* k10478 in k10474 in loop in a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10480,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10501 in loop in a10441 in k10428 in k10404 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10503,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,8)))){
C_save_and_reclaim((void *)f_10576,c,av);}
a=C_alloc(39);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_END_OF_LIST;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_END_OF_LIST;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_fix(0);
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10579,a[2]=t17,a[3]=t7,a[4]=t9,a[5]=t11,a[6]=t13,a[7]=t15,a[8]=t3,tmp=(C_word)a,a+=9,tmp));
t19=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10790,a[2]=t11,a[3]=t3,a[4]=t9,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t20=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11221,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t21=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11867,a[2]=t1,a[3]=t5,a[4]=t13,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1746: chicken.compiler.support#debugging */
t22=*((C_word*)lf[35]+1);{
C_word *av2=av;
av2[0]=t22;
av2[1]=t21;
av2[2]=lf[48];
av2[3]=lf[185];
((C_proc)(void*)(*((C_word*)t22+1)))(4,av2);}}

/* walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10579(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_10579,5,t0,t1,t2,t3,t4);}
a=C_alloc(13);
t5=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10583,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,a[5]=t2,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=t4,a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],a[12]=((C_word*)t0)[8],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:1529: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_10583,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_10586,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* optimizer.scm:1530: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_10586,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_10589,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
/* optimizer.scm:1531: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10589(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_10589,c,av);}
a=C_alloc(22);
t2=C_eqp(t1,lf[29]);
if(C_truep(t2)){
t3=C_i_caddr(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10604,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[6])){
if(C_truep(C_i_cadr(((C_word*)t0)[2]))){
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10707,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[13],a[7]=t4,a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[14],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1536: chicken.compiler.support#db-get */
t6=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[14];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[131];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}
else{
t3=C_eqp(t1,lf[31]);
if(C_truep(t3)){
/* optimizer.scm:1550: walk */
t4=((C_word*)((C_word*)t0)[11])[1];
f_10579(t4,((C_word*)t0)[4],C_i_car(((C_word*)t0)[2]),C_i_car(((C_word*)t0)[12]),C_SCHEME_FALSE);}
else{
t4=C_eqp(t1,lf[26]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10733,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[12],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1552: walk */
t6=((C_word*)((C_word*)t0)[11])[1];
f_10579(t6,t5,C_i_car(((C_word*)t0)[2]),C_i_car(((C_word*)t0)[12]),((C_word*)t0)[5]);}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10749,a[2]=((C_word*)t0)[11],tmp=(C_word)a,a+=3,tmp);
t6=C_i_check_list_2(((C_word*)t0)[12],lf[23]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10761,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_10761(t10,((C_word*)t0)[4],((C_word*)t0)[12]);}}}}

/* k10602 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_10604,c,av);}
if(C_truep(t1)){
/* optimizer.scm:1548: transform */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11221(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)((C_word*)t0)[6])[1],((C_word*)((C_word*)t0)[7])[1],((C_word*)t0)[8],((C_word*)((C_word*)t0)[9])[1]);}
else{
/* optimizer.scm:1549: walk */
t2=((C_word*)((C_word*)t0)[10])[1];
f_10579(t2,((C_word*)t0)[3],C_SCHEME_FALSE,C_i_car(((C_word*)t0)[11]),C_SCHEME_FALSE);}}

/* k10633 in k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10635(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10635,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10641,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1539: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}

/* k10639 in k10633 in k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_10641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10641,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10647,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1540: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[157];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}

/* k10645 in k10639 in k10633 in k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_10647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_10647,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=C_eqp(((C_word*)t0)[2],((C_word*)t0)[3]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10695,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10703,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1544: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}
else{
t2=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}

/* k10693 in k10645 in k10639 in k10633 in k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_10695(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_10695,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=C_i_length(((C_word*)t0)[2]);
t3=C_i_length(((C_word*)t0)[3]);
t4=C_eqp(t2,t3);
if(C_truep(t4)){
t5=C_i_car(((C_word*)t0)[4]);
t6=C_i_car(((C_word*)t0)[5]);
t7=C_a_i_cons(&a,2,((C_word*)t0)[6],((C_word*)t0)[5]);
/* optimizer.scm:1547: scan */
t8=((C_word*)((C_word*)t0)[7])[1];
f_10790(t8,((C_word*)t0)[8],t5,t6,((C_word*)t0)[6],((C_word*)t0)[9],t7);}
else{
t5=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}

/* k10701 in k10645 in k10639 in k10633 in k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_10703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10703,c,av);}
/* optimizer.scm:1543: chicken.compiler.support#variable-mark */
t2=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
av2[3]=lf[163];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10705 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10707,c,av);}
a=C_alloc(10);
if(C_truep(C_i_not(t1))){
if(C_truep(C_i_listp(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10635,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1538: chicken.compiler.support#db-get */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[81];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_10604(2,av2);}}}

/* k10731 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10733,c,av);}
/* optimizer.scm:1553: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10579(t2,((C_word*)t0)[3],C_SCHEME_FALSE,C_i_cadr(((C_word*)t0)[4]),C_SCHEME_FALSE);}

/* g2229 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10749(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_10749,3,t0,t1,t2);}
/* optimizer.scm:1554: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10579(t3,t1,C_SCHEME_FALSE,t2,C_SCHEME_FALSE);}

/* for-each-loop2228 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10761(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_10761,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10771,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1554: g2229 */
t4=((C_word*)t0)[3];
f_10749(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10769 in for-each-loop2228 in k10587 in k10584 in k10581 in walk in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10771,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10761(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10790(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,6)))){
C_save_and_reclaim_args((void *)trf_10790,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(25);
t7=C_SCHEME_END_OF_LIST;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_FALSE;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10793,a[2]=t6,a[3]=t10,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=t8,a[7]=t12,a[8]=t5,a[9]=((C_word*)t0)[4],a[10]=t4,a[11]=((C_word*)t0)[5],a[12]=t3,tmp=(C_word)a,a+=13,tmp));
t14=C_set_block_item(((C_word*)t0)[5],0,C_SCHEME_END_OF_LIST);
t15=C_set_block_item(((C_word*)t0)[4],0,C_SCHEME_END_OF_LIST);
t16=C_set_block_item(((C_word*)t0)[2],0,C_fix(0));
t17=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11212,a[2]=t8,a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1633: rec */
t18=((C_word*)t12)[1];
f_10793(t18,t17,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,t6);}

/* rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_10793(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_10793,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(18);
t6=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_10797,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t3,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=t5,a[11]=t4,a[12]=((C_word*)t0)[8],a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[10],a[15]=((C_word*)t0)[11],a[16]=((C_word*)t0)[12],a[17]=t2,tmp=(C_word)a,a+=18,tmp);
/* optimizer.scm:1560: chicken.compiler.support#node-parameters */
t7=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_10797,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_10800,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
/* optimizer.scm:1561: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[17];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10800(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_10800,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_10803,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],tmp=(C_word)a,a+=20,tmp);
/* optimizer.scm:1562: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[18];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,5)))){
C_save_and_reclaim((void *)f_10803,c,av);}
a=C_alloc(33);
t2=C_eqp(t1,lf[24]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10842,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1565: chicken.compiler.support#db-get */
t5=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=t3;
av2[4]=lf[164];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_eqp(t1,lf[29]);
if(C_truep(t3)){
if(C_truep(((C_word*)t0)[8])){
t4=C_i_caddr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10860,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[12],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1573: ##sys#decompose-lambda-list */
t6=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[3];
av2[2]=t4;
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=C_eqp(t1,lf[121]);
if(C_truep(t4)){
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[6])[1],t5);
t7=C_mutate(((C_word *)((C_word*)t0)[6])+1,t6);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10897,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1582: every */
f_3770(((C_word*)t0)[3],t8,((C_word*)t0)[10]);}
else{
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=C_eqp(t1,lf[101]);
if(C_truep(t5)){
if(C_truep(((C_word*)t0)[13])){
if(C_truep(((C_word*)t0)[14])){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10931,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1585: chicken.compiler.support#scan-used-variables */
t7=*((C_word*)lf[165]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_car(((C_word*)t0)[10]);
av2[3]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t6=C_eqp(t1,lf[166]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10947,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1590: chicken.compiler.support#estimate-foreign-result-size */
t8=*((C_word*)lf[167]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=C_eqp(t1,lf[168]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10986,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1598: chicken.compiler.support#estimate-foreign-result-size */
t9=*((C_word*)lf[167]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=C_eqp(t1,lf[30]);
if(C_truep(t8)){
t9=C_i_car(((C_word*)t0)[10]);
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11094,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[16],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[17],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[18],a[11]=t9,tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:1607: chicken.compiler.support#node-class */
t11=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=t9;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t9=C_eqp(t1,lf[169]);
if(C_truep(t9)){
t10=C_i_cadddr(((C_word*)t0)[2]);
t11=C_i_zerop(t10);
if(C_truep(t11)){
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t12=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[6])[1],t10);
t13=C_mutate(((C_word *)((C_word*)t0)[6])+1,t12);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11122,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1624: every */
f_3770(((C_word*)t0)[3],t14,((C_word*)t0)[10]);}
else{
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}}
else{
t10=C_eqp(t1,lf[31]);
if(C_truep(t10)){
/* optimizer.scm:1625: rec */
t11=((C_word*)((C_word*)t0)[11])[1];
f_10793(t11,((C_word*)t0)[3],C_i_car(((C_word*)t0)[10]),C_i_car(((C_word*)t0)[2]),C_SCHEME_FALSE,((C_word*)t0)[12]);}
else{
t11=C_eqp(t1,lf[26]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11155,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[12],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1627: rec */
t13=((C_word*)((C_word*)t0)[11])[1];
f_10793(t13,t12,C_i_car(((C_word*)t0)[10]),C_i_car(((C_word*)t0)[2]),((C_word*)t0)[19],((C_word*)t0)[12]);}
else{
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11179,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1629: every */
f_3770(((C_word*)t0)[3],t12,((C_word*)t0)[10]);}}}}}}}}}}}

/* k10840 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_10842,c,av);}
a=C_alloc(29);
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_memq(((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_i_not(t3);
if(C_truep(t4)){
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t5=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[6])[1],C_fix(2));
t6=C_mutate(((C_word *)((C_word*)t0)[6])+1,t5);
t7=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}}

/* a10859 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10860,c,av);}
a=C_alloc(8);
t5=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t5);
t7=C_i_car(((C_word*)t0)[4]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10876,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=t7,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1577: scheme#append */
t9=*((C_word*)lf[7]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=t2;
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}

/* k10874 in a10859 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_10876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10876,c,av);}
/* optimizer.scm:1577: rec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10793(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,t1);}

/* a10896 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10897,c,av);}
/* optimizer.scm:1582: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k10929 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10931,c,av);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t1))){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k10945 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,2)))){
C_save_and_reclaim((void *)f_10947,c,av);}
a=C_alloc(35);
t2=C_i_zerop(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10951,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_10951(t4,t2);}
else{
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[6])[1]))){
t4=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[7])[1],t1);
t5=C_mutate(((C_word *)((C_word*)t0)[7])+1,t4);
t6=t3;
f_10951(t6,C_SCHEME_TRUE);}
else{
t4=t3;
f_10951(t4,C_SCHEME_FALSE);}}}

/* k10949 in k10945 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_fcall f_10951(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_10951,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10956,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1596: every */
f_3770(((C_word*)t0)[4],t2,((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a10955 in k10949 in k10945 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_10956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10956,c,av);}
/* optimizer.scm:1596: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k10984 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_10986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,2)))){
C_save_and_reclaim((void *)f_10986,c,av);}
a=C_alloc(35);
t2=C_i_zerop(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10990,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t2)){
t4=t3;
f_10990(t4,t2);}
else{
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[6])[1]))){
t4=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[7])[1],t1);
t5=C_mutate(((C_word *)((C_word*)t0)[7])+1,t4);
t6=t3;
f_10990(t6,C_SCHEME_TRUE);}
else{
t4=t3;
f_10990(t4,C_SCHEME_FALSE);}}}

/* k10988 in k10984 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_fcall f_10990(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_10990,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1604: every */
f_3770(((C_word*)t0)[4],t2,((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a10994 in k10988 in k10984 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_10995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_10995,c,av);}
/* optimizer.scm:1604: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k11035 in k11088 in k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_fcall f_11037(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_11037,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11042,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1617: every */
f_3770(((C_word*)t0)[4],t2,C_i_cdr(((C_word*)t0)[5]));}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a11041 in k11035 in k11088 in k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_11042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11042,c,av);}
/* optimizer.scm:1617: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k11077 in k11081 in k11088 in k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_11079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_11079,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)((C_word*)t0)[2])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t5=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_TRUE);
t6=((C_word*)t0)[4];
f_11037(t6,C_SCHEME_TRUE);}

/* k11081 in k11088 in k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_11083(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11083,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11079,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1613: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_TRUE);
t4=((C_word*)t0)[4];
f_11037(t4,C_SCHEME_TRUE);}}

/* k11088 in k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_11090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11090,c,av);}
a=C_alloc(12);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11037,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_eqp(t2,((C_word*)t0)[6]);
if(C_truep(t4)){
t5=((C_word*)((C_word*)t0)[7])[1];
if(C_truep(C_i_zerop(t5))){
t6=C_i_cadr(((C_word*)t0)[5]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11083,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[9],a[4]=t3,a[5]=t6,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1612: chicken.compiler.support#node-class */
t8=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t6=t3;
f_11037(t6,C_SCHEME_FALSE);}}
else{
t5=t3;
f_11037(t5,C_eqp(t2,((C_word*)t0)[10]));}}

/* k11092 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_11094,c,av);}
a=C_alloc(11);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11090,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1608: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a11121 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11122,c,av);}
/* optimizer.scm:1624: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k11153 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11155(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11155,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11166,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1628: scheme#append */
t4=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k11164 in k11153 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_11166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11166,c,av);}
/* optimizer.scm:1628: rec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_10793(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,t1);}

/* a11178 in k10801 in k10798 in k10795 in rec in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11179,c,av);}
/* optimizer.scm:1629: rec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_10793(t3,t1,t2,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[3]);}

/* k11210 in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11212,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11219,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1634: delete */
f_3890(t2,((C_word*)t0)[4],((C_word*)((C_word*)t0)[5])[1],*((C_word*)lf[47]+1));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k11217 in k11210 in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11219,c,av);}
a=C_alloc(3);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_i_length(t2);
t4=C_i_length(t1);
t5=C_eqp(t3,t4);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4466,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:197: every */
f_3770(((C_word*)t0)[3],t6,t2);}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_11221(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,5)))){
C_save_and_reclaim_args((void *)trf_11221,8,t0,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(19);
t8=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11225,a[2]=((C_word*)t0)[2],a[3]=t7,a[4]=t3,a[5]=t6,a[6]=t5,a[7]=t1,a[8]=t4,a[9]=t2,a[10]=((C_word*)t0)[3],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_pairp(t5))){
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11845,a[2]=t8,a[3]=t3,a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11847,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1638: ##sys#make-promise */
t11=*((C_word*)lf[182]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t9;
av2[2]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
/* optimizer.scm:1639: chicken.compiler.support#debugging */
t9=*((C_word*)lf[35]+1);{
C_word av2[6];
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[36];
av2[3]=lf[184];
av2[4]=t3;
av2[5]=t7;
((C_proc)(void*)(*((C_word*)t9+1)))(6,av2);}}}

/* k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_11225,c,av);}
a=C_alloc(10);
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11229,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1641: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_11229,c,av);}
a=C_alloc(14);
t2=C_i_caddr(t1);
t3=C_i_length(t2);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11235,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=t5,a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:1644: chicken.compiler.support#db-get-list */
t7=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[157];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_11235,c,av);}
a=C_alloc(16);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_11241,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t3,a[10]=t1,a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
if(C_truep(C_i_listp(((C_word*)t0)[2]))){
t5=C_eqp(C_u_i_length(((C_word*)t0)[2]),C_fix(4));
t6=t4;
f_11241(t6,(C_truep(t5)?C_i_listp(C_i_caddr(((C_word*)t0)[2])):C_SCHEME_FALSE));}
else{
t5=t4;
f_11241(t5,C_SCHEME_FALSE);}}

/* k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_11241(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,3)))){
C_save_and_reclaim_args((void *)trf_11241,2,t0,t1);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_11247,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
/* optimizer.scm:1648: scheme#caaddr */
t4=*((C_word*)lf[179]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* optimizer.scm:1744: chicken.compiler.support#bomb */
t2=*((C_word*)lf[174]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[8];
av2[2]=lf[180];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11247(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_11247,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_11250,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* optimizer.scm:1649: scheme#cdaddr */
t3=*((C_word*)lf[178]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_11250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_11250,c,av);}
a=C_alloc(15);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_i_setslot(C_u_i_cdr(t2),C_fix(0),t1);
t4=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_11256,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],tmp=(C_word)a,a+=15,tmp);
/* optimizer.scm:1653: chicken.compiler.support#node-class-set! */
t5=*((C_word*)lf[172]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[15];
av2[3]=lf[101];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_11256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_11256,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11259,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11805,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[12],a[7]=((C_word*)t0)[13],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1656: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[14];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_11259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_11259,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11260,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11333,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11507,a[2]=((C_word*)t0)[9],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1718: filter */
f_4010(t3,t4,((C_word*)t0)[10]);}

/* g2348 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_fcall f_11260(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_11260,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11267,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1703: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11265 in g2348 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_11267(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_11267,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11270,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1704: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11268 in k11265 in g2348 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_11270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_11270,c,av);}
a=C_alloc(8);
t2=C_i_cdr(t1);
t3=C_i_pairp(t2);
t4=(C_truep(t3)?C_i_cadr(t1):C_SCHEME_FALSE);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11276,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t6=C_i_cdr(((C_word*)t0)[2]);
t7=C_i_length(t6);
t8=C_eqp(((C_word*)t0)[7],t7);
if(C_truep(t8)){
t9=t5;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_UNDEFINED;
f_11276(2,av2);}}
else{
/* optimizer.scm:1708: chicken.compiler.support#quit-compiling */
t9=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t9;
av2[1]=t5;
av2[2]=lf[171];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}}

/* k11274 in k11268 in k11265 in g2348 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_11276(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,4)))){
C_save_and_reclaim((void *)f_11276,c,av);}
a=C_alloc(20);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11291,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_list4(&a,4,C_SCHEME_TRUE,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);
t5=C_u_i_car(((C_word*)t0)[2]);
t6=C_i_cddr(((C_word*)t0)[2]);
t7=C_a_i_cons(&a,2,t5,t6);
/* optimizer.scm:1714: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t3;
av2[2]=lf[169];
av2[3]=t4;
av2[4]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}

/* k11289 in k11274 in k11268 in k11265 in g2348 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_11291(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_11291,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1711: chicken.compiler.support#node-subexpressions-set! */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_11333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11333,c,av);}
a=C_alloc(11);
t2=C_i_check_list_2(t1,lf[23]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11339,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11484,a[2]=t5,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_11484(t7,t3,t1);}

/* k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_11339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11339,c,av);}
a=C_alloc(5);
t2=(C_truep(((C_word*)t0)[2])?C_i_pairp(((C_word*)t0)[3]):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11348,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1725: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_11348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_11348,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11351,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1726: chicken.compiler.support#copy-node! */
t3=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_11351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11351,c,av);}
a=C_alloc(11);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[6]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11357,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11415,a[2]=t5,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_11415(t7,t3,((C_word*)t0)[2]);}

/* k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_11357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11357,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11360,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1737: chicken.compiler.support#copy-node! */
t3=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_11360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11360,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11392,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_11392(t5,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* k11366 in for-each-loop2473 in k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11368(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11368,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11375,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1742: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11373 in k11366 in for-each-loop2473 in k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in ... */
static void C_ccall f_11375(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11375,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11379,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1742: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11377 in k11373 in k11366 in for-each-loop2473 in k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in ... */
static void C_ccall f_11379(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11379,c,av);}
t2=C_i_set_car(((C_word*)t0)[2],t1);
t3=((C_word*)((C_word*)t0)[3])[1];
f_11392(t3,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11385 in for-each-loop2473 in k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11387,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1741: chicken.compiler.support#node-parameters-set! */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* for-each-loop2473 in k11358 in k11355 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_fcall f_11392(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_11392,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cdr(t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11368,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11387,a[2]=t5,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1741: chicken.base#gensym */
t7=*((C_word*)lf[116]+1);{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_fcall f_11415(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_11415,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11475,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=t4;
t7=C_slot(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11440 in k11456 in k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11442,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:1730: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11444 in k11456 in k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11446,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11450,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1733: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11448 in k11444 in k11456 in k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in ... */
static void C_ccall f_11450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11450,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11454,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1733: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11452 in k11448 in k11444 in k11456 in k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in ... */
static void C_ccall f_11454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11454,c,av);}
/* optimizer.scm:1733: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11456 in k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_11458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_11458,c,av);}
a=C_alloc(9);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11442,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11446,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1733: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11473 in foldr2459 in k11349 in k11346 in k11337 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_11475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_11475,c,av);}
a=C_alloc(8);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11458,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1732: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_cdr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* for-each-loop2347 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_fcall f_11484(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11484,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11494,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1700: g2348 */
t4=((C_word*)t0)[3];
f_11260(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11492 in for-each-loop2347 in k11331 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_11494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11494,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11484(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a11506 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_11507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_11507,c,av);}
a=C_alloc(6);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11518,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11520,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1720: any */
f_3804(t4,t5,((C_word*)((C_word*)t0)[2])[1]);}

/* k11516 in a11506 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_11518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11518,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a11519 in a11506 in k11257 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_11520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11520,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(C_i_cdr(t2),((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_fcall f_11534(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_11534,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11538,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=t1,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1657: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_11538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11538,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11541,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:1658: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_11541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_11541,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11544,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:1659: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_11544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_11544,c,av);}
a=C_alloc(13);
t2=C_eqp(t1,lf[30]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=C_i_cadr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11559,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t3,a[12]=t4,tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:1663: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t3=C_eqp(t1,lf[26]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[11]);
t5=C_i_car(((C_word*)t0)[2]);
if(C_truep(C_i_memq(t4,((C_word*)t0)[12]))){
t6=((C_word*)((C_word*)t0)[9])[1];
t7=C_a_i_cons(&a,2,t4,t5);
t8=C_a_i_cons(&a,2,t7,t6);
t9=C_mutate(((C_word *)((C_word*)t0)[9])+1,t8);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11732,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1693: chicken.compiler.support#copy-node! */
t11=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}
else{
t6=C_i_check_list_2(((C_word*)t0)[2],lf[23]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11747,a[2]=t8,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_11747(t10,((C_word*)t0)[7],((C_word*)t0)[2]);}}
else{
t4=C_i_check_list_2(((C_word*)t0)[2],lf[23]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11776,a[2]=t6,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_11776(t8,((C_word*)t0)[7],((C_word*)t0)[2]);}}}

/* k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_11559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_11559,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11562,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:1664: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_11562(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_11562,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11707,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:1665: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11585 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11587,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11590,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_list2(&a,2,C_SCHEME_TRUE,((C_word*)t0)[5]);
/* optimizer.scm:1670: chicken.compiler.support#node-parameters-set! */
t4=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11588 in k11585 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in ... */
static void C_ccall f_11590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11590,c,av);}
/* optimizer.scm:1671: chicken.compiler.support#node-subexpressions-set! */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_cddr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_fcall f_11608(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_11608,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11659,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1675: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in ... */
static void C_ccall f_11618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_11618,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11647,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11655,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1677: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11619 in k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in ... */
static void C_ccall f_11621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_11621,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11624,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11635,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_list2(&a,2,C_SCHEME_FALSE,((C_word*)t0)[6]);
/* optimizer.scm:1680: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[173];
av2[3]=t4;
av2[4]=C_i_cddr(((C_word*)t0)[7]);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k11622 in k11619 in k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in ... */
static void C_ccall f_11624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11624,c,av);}
/* optimizer.scm:1681: rec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11534(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k11633 in k11619 in k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in ... */
static void C_ccall f_11635(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_11635,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:1678: chicken.compiler.support#node-subexpressions-set! */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11645 in k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in ... */
static void C_ccall f_11647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11647,c,av);}
/* optimizer.scm:1677: chicken.compiler.support#node-parameters-set! */
t2=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11653 in k11616 in k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in ... */
static void C_ccall f_11655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11655,c,av);}
/* optimizer.scm:1677: take */
f_3509(((C_word*)t0)[2],C_i_caddr(t1),C_fix(1));}

/* k11657 in g2381 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in ... */
static void C_ccall f_11659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_11659,c,av);}
a=C_alloc(9);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11618,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1676: chicken.compiler.support#node-class-set! */
t4=*((C_word*)lf[172]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[26];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11680 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_11682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11682,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11685,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1685: chicken.compiler.support#node-parameters-set! */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11683 in k11680 in k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in ... */
static void C_ccall f_11685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11685,c,av);}
/* optimizer.scm:1686: chicken.compiler.support#node-subexpressions-set! */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11705 in k11560 in k11557 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_11707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_11707,c,av);}
a=C_alloc(12);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=C_eqp(((C_word*)t0)[3],t3);
if(C_truep(t4)){
t5=((C_word*)((C_word*)t0)[4])[1];
t6=C_a_i_cons(&a,2,C_SCHEME_FALSE,((C_word*)t0)[5]);
t7=C_a_i_cons(&a,2,t6,t5);
t8=C_mutate(((C_word *)((C_word*)t0)[4])+1,t7);
t9=C_i_car(((C_word*)t0)[6]);
t10=C_eqp(((C_word*)t0)[7],t9);
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11587,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1669: chicken.compiler.support#node-class-set! */
t12=*((C_word*)lf[172]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t11;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[173];
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}
else{
t11=C_i_car(((C_word*)t0)[6]);
t12=C_i_assq(t11,((C_word*)((C_word*)t0)[11])[1]);
if(C_truep(t12)){
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11608,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1668: g2381 */
t14=t13;
f_11608(t14,((C_word*)t0)[8],t12);}
else{
/* optimizer.scm:1682: chicken.compiler.support#bomb */
t13=*((C_word*)lf[174]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t13;
av2[1]=((C_word*)t0)[8];
av2[2]=lf[175];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}}}
else{
t5=C_i_car(((C_word*)t0)[2]);
t6=C_eqp(((C_word*)t0)[7],t5);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11682,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1684: chicken.compiler.support#node-class-set! */
t8=*((C_word*)lf[172]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[176];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* optimizer.scm:1687: chicken.compiler.support#bomb */
t7=*((C_word*)lf[174]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[8];
av2[2]=lf[177];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}
else{
t3=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11730 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_11732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11732,c,av);}
/* optimizer.scm:1694: rec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11534(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* for-each-loop2402 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_fcall f_11747(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11747,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11757,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1695: g2403 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_11534(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11755 in for-each-loop2402 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_11757(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11757,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11747(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop2419 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_fcall f_11776(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_11776,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11786,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1697: g2420 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_11534(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k11784 in for-each-loop2419 in k11542 in k11539 in k11536 in rec in k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_11786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11786,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_11776(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k11803 in k11254 in k11248 in k11245 in k11239 in k11233 in k11227 in k11223 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_11805(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11805,c,av);}
a=C_alloc(11);
t2=C_i_car(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11534,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t4,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp));
t6=((C_word*)t4)[1];
f_11534(t6,((C_word*)t0)[8],t2);}

/* k11843 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_11845,c,av);}
/* optimizer.scm:1638: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[36];
av2[3]=lf[181];
av2[4]=((C_word*)t0)[3];
av2[5]=t1;
av2[6]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* a11846 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_11847,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11855,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11857,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1638: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=t3;
av2[3]=*((C_word*)lf[183]+1);
C_call_with_values(4,av2);}}

/* k11853 in a11846 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11855,c,av);}
/* optimizer.scm:1638: ##sys#make-promise */
t2=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a11856 in a11846 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_11857,c,av);}
a=C_alloc(12);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[8]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4136,a[2]=t4,a[3]=t8,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_4136(t10,t1,((C_word*)t0)[2]);}

/* k11865 in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11867,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11870,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1747: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_10579(t3,t2,C_SCHEME_FALSE,((C_word*)t0)[5],C_SCHEME_FALSE);}

/* k11868 in k11865 in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11870,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_11872,c,av);}
a=C_alloc(29);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_END_OF_LIST;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11875,a[2]=t9,a[3]=t7,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t15=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11925,a[2]=t7,a[3]=t13,a[4]=t9,a[5]=t11,a[6]=t3,tmp=(C_word)a,a+=7,tmp));
t16=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12164,a[2]=t3,a[3]=t5,a[4]=t1,a[5]=t2,a[6]=t13,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1831: chicken.compiler.support#debugging */
t17=*((C_word*)lf[35]+1);{
C_word *av2=av;
av2[0]=t17;
av2[1]=t16;
av2[2]=lf[48];
av2[3]=lf[196];
((C_proc)(void*)(*((C_word*)t17+1)))(4,av2);}}

/* close in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_11875(C_word *a,C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_stack_overflow_check;{}
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[2])[1]))){
t1=C_i_length(((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(C_fixnum_greaterp(t1,C_fix(1)))){
t2=((C_word*)((C_word*)t0)[3])[1];
t3=((C_word*)((C_word*)t0)[2])[1];
t4=((C_word*)((C_word*)t0)[4])[1];
t5=C_a_i_cons(&a,2,t2,t3);
t6=C_a_i_cons(&a,2,t5,t4);
t7=C_mutate(((C_word *)((C_word*)t0)[4])+1,t6);
t8=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_END_OF_LIST);
t9=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_FALSE);
return(t9);}
else{
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_END_OF_LIST);
t3=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_FALSE);
return(t3);}}
else{
t1=C_SCHEME_UNDEFINED;
return(t1);}}

/* k11917 in k11921 in k12037 in k12041 in k12045 in k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_11919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11919,c,av);}
t2=((C_word*)t0)[2];
f_11970(t2,C_i_listp(C_i_caddr(t1)));}

/* k11921 in k12037 in k12041 in k12045 in k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_11923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11923,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[29],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11919,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1784: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_11970(t3,C_SCHEME_FALSE);}}

/* walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_11925(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_11925,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11929,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=t1,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1787: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_11929,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11932,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1788: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11932,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11935,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:1789: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_11935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,5)))){
C_save_and_reclaim((void *)f_11935,c,av);}
a=C_alloc(16);
t2=C_eqp(t1,lf[26]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=C_i_car(((C_word*)t0)[3]);
t5=C_i_cadr(((C_word*)t0)[3]);
t6=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11956,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t5,a[9]=t4,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[4])[1]))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12069,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1796: chicken.compiler.support#node-class */
t8=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=t6;
f_11956(t7,C_SCHEME_FALSE);}}
else{
t3=C_eqp(t1,lf[29]);
t4=(C_truep(t3)?t3:C_eqp(t1,lf[101]));
if(C_truep(t4)){
t5=C_i_caddr(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12087,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1819: ##sys#decompose-lambda-list */
t7=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[8];
av2[2]=t5;
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t5=(
/* optimizer.scm:1828: close */
  f_11875(C_a_i(&a,6),((C_word*)((C_word*)t0)[10])[1])
);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12119,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(((C_word*)t0)[3],lf[23]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12131,a[2]=t9,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_12131(t11,((C_word*)t0)[8],((C_word*)t0)[3]);}}}

/* k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_11956(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,3)))){
C_save_and_reclaim_args((void *)trf_11956,2,t0,t1);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* optimizer.scm:1799: walk */
t4=((C_word*)((C_word*)t0)[6])[1];
f_11925(t4,((C_word*)t0)[7],((C_word*)t0)[8],t3);}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11970,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[11],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12059,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1801: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;
f_11970(t3,C_SCHEME_FALSE);}}}

/* k11968 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_11970(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_11970,2,t0,t1);}
a=C_alloc(13);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* optimizer.scm:1812: walk */
t5=((C_word*)((C_word*)t0)[6])[1];
f_11925(t5,((C_word*)t0)[7],((C_word*)t0)[8],t4);}
else{
t2=(
/* optimizer.scm:1815: close */
  f_11875(C_a_i(&a,6),((C_word*)((C_word*)t0)[9])[1])
);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11987,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1816: walk */
t4=((C_word*)((C_word*)t0)[6])[1];
f_11925(t4,t3,((C_word*)t0)[2],((C_word*)t0)[5]);}}

/* k11985 in k11968 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_11987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11987,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* optimizer.scm:1817: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_11925(t3,((C_word*)t0)[5],((C_word*)t0)[6],t2);}

/* k12037 in k12041 in k12045 in k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_12039(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12039,c,av);}
a=C_alloc(4);
t2=C_i_length(t1);
t3=C_eqp(((C_word*)t0)[2],t2);
if(C_truep(t3)){
if(C_truep(C_i_memq(((C_word*)t0)[3],((C_word*)t0)[4]))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11923,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1783: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[5];
f_11970(t4,C_SCHEME_FALSE);}}
else{
t4=((C_word*)t0)[5];
f_11970(t4,C_SCHEME_FALSE);}}

/* k12041 in k12045 in k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_12043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_12043,c,av);}
a=C_alloc(7);
t2=C_i_length(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12039,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1807: chicken.compiler.support#db-get-list */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[157];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12045 in k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_12047(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_12047,c,av);}
a=C_alloc(7);
t2=C_eqp(lf[29],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12043,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1806: chicken.compiler.support#db-get-list */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=((C_word*)t0)[4];
f_11970(t3,C_SCHEME_FALSE);}}

/* k12049 in k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_12051(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12051,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12047,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1805: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12053 in k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_12055(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_12055,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12051,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1803: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12057 in k11954 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_12059,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[31],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12055,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1802: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];
f_11970(t3,C_SCHEME_FALSE);}}

/* k12067 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12069,c,av);}
t2=((C_word*)t0)[2];
f_11956(t2,C_eqp(lf[34],t1));}

/* a12086 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_12087,c,av);}
a=C_alloc(29);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_FALSE;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_FALSE;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_FALSE;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12093,a[2]=t10,a[3]=((C_word*)t0)[2],a[4]=t12,a[5]=((C_word*)t0)[3],a[6]=t6,a[7]=t8,tmp=(C_word)a,a+=8,tmp);
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12100,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t15=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12110,a[2]=t6,a[3]=((C_word*)t0)[2],a[4]=t8,a[5]=((C_word*)t0)[3],a[6]=t10,a[7]=t12,tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1823: ##sys#dynamic-wind */
t16=*((C_word*)lf[187]+1);{
C_word *av2=av;
av2[0]=t16;
av2[1]=t1;
av2[2]=t13;
av2[3]=t14;
av2[4]=t15;
((C_proc)(void*)(*((C_word*)t16+1)))(5,av2);}}

/* a12092 in a12086 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12093,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)((C_word*)t0)[5])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[6])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[5])+1,((C_word*)((C_word*)t0)[7])[1]);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* a12099 in a12086 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12100,c,av);}
/* optimizer.scm:1825: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_11925(t2,t1,C_i_car(((C_word*)t0)[3]),((C_word*)t0)[4]);}

/* a12109 in a12086 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12110(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12110,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)((C_word*)t0)[5])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[6])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[5])+1,((C_word*)((C_word*)t0)[7])[1]);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* g2586 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12119(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_12119,3,t0,t1,t2);}
/* optimizer.scm:1829: g2601 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_11925(t3,t1,t2,((C_word*)t0)[3]);}

/* for-each-loop2585 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12131(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_12131,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12141,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1829: g2586 */
t4=((C_word*)t0)[3];
f_12119(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12139 in for-each-loop2585 in k11933 in k11930 in k11927 in walk in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12141,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12131(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_12164,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12167,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1834: walk */
t3=((C_word*)((C_word*)t0)[6])[1];
f_11925(t3,t2,((C_word*)t0)[5],C_SCHEME_END_OF_LIST);}

/* k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_12167,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12168,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_i_check_list_2(t3,lf[23]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12806,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12815,a[2]=t7,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_12815(t9,t5,t3);}

/* g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12168(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12168,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12176,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1841: chicken.base#gensym */
t6=*((C_word*)lf[116]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[195];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12176,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12179,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1842: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[106];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_12179,c,av);}
a=C_alloc(26);
t2=C_fix(1);
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12183,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t9=C_i_check_list_2(((C_word*)t0)[2],lf[8]);
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12278,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t3,a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12768,a[2]=t6,a[3]=t12,a[4]=t8,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_12768(t14,t10,((C_word*)t0)[2]);}

/* g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12183(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_12183,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12272,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1848: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_12193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_12193,c,av);}
a=C_alloc(20);
t2=C_i_caddr(t1);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(t2,lf[8]);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12205,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12232,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_12232(t12,t8,t2);}

/* k12203 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_12205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_12205,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12210,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1853: ##sys#decompose-lambda-list */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a12209 in k12203 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_12210(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12210,c,av);}
a=C_alloc(7);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12230,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1856: chicken.compiler.support#node-subexpressions */
t6=*((C_word*)lf[42]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k12217 in k12228 in a12209 in k12203 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_12219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_12219,c,av);}
a=C_alloc(12);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list4(&a,4,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12224 in k12228 in a12209 in k12203 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_12226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12226,c,av);}
/* optimizer.scm:1859: chicken.compiler.support#copy-node! */
t2=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k12228 in a12209 in k12203 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_12230(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(40,c,4)))){
C_save_and_reclaim((void *)f_12230,c,av);}
a=C_alloc(40);
t2=C_i_car(t1);
t3=((C_word*)((C_word*)t0)[2])[1];
t4=((C_word*)((C_word*)t0)[2])[1];
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t4,C_fix(1)));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12219,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12226,a[2]=t6,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1860: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}

/* map-loop2645 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_fcall f_12232(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12232,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12257,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1852: g2651 */
t4=*((C_word*)lf[116]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12255 in map-loop2645 in k12191 in k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_12257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12257,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_12232(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k12266 in k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_12268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12268,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12193,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1850: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12270 in g2626 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_12272(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_12272,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12268,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1849: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12278,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12281,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1864: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[194];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_12281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_12281,c,av);}
a=C_alloc(24);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12284,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[5],lf[8]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12732,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12734,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_12734(t12,t8,((C_word*)t0)[5]);}

/* k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_12284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_12284,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12287,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12703,a[2]=((C_word*)t0)[7],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12709,tmp=(C_word)a,a+=2,tmp);
/* optimizer.scm:1867: list-tabulate */
f_4412(t3,t1,t4);}

/* k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_12287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_12287,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12290,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=lf[14];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12665,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12667,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_12667(t12,t8,((C_word*)t0)[6]);}

/* k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_12290(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_12290,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12293,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12424,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp));
t6=((C_word*)t4)[1];
f_12424(t6,t2,((C_word*)t0)[10]);}

/* k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_12293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_12293,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12294,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12401,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_12401(t6,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_fcall f_12294(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_12294,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12298,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1932: chicken.compiler.support#db-get */
t4=*((C_word*)lf[55]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=C_i_car(t2);
av2[4]=lf[157];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_12298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_12298,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12299,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_i_check_list_2(t1,lf[23]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12371,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_12371(t7,((C_word*)t0)[5],t1);}

/* g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_fcall f_12299(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_12299,3,t0,t1,t2);}
a=C_alloc(7);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12363,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1936: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k12311 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_12313(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12313,c,av);}
/* optimizer.scm:1937: chicken.compiler.support#copy-node! */
t2=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_12317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_12317,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12321,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12325,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1940: chicken.compiler.support#varnode */
t4=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12319 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12321,c,av);}
/* optimizer.scm:1938: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12323 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(40,c,3)))){
C_save_and_reclaim((void *)f_12325,c,av);}
a=C_alloc(40);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12329,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12333,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_i_length(((C_word*)t0)[3]);
t5=C_s_a_i_minus(&a,2,((C_word*)t0)[5],t4);
t6=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12351,tmp=(C_word)a,a+=2,tmp);
/* optimizer.scm:1943: list-tabulate */
f_4412(t3,t5,t6);}

/* k12327 in k12323 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12329(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12329,c,av);}
a=C_alloc(3);
/* optimizer.scm:1940: cons* */
f_3834(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,1,t1));}

/* k12331 in k12323 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12333,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12341,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1946: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12339 in k12331 in k12323 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in ... */
static void C_ccall f_12341(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12341,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1941: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a12350 in k12323 in k12315 in k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12351,c,av);}
/* optimizer.scm:1945: chicken.compiler.support#qnode */
t2=*((C_word*)lf[77]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12361 in g2771 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_12363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_12363,c,av);}
a=C_alloc(11);
t2=C_i_cdr(t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12313,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12317,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1939: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* for-each-loop2770 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_fcall f_12371(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_12371,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12381,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1933: g2771 */
t4=((C_word*)t0)[3];
f_12299(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12379 in for-each-loop2770 in k12296 in g2719 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_12381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12381,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12371(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop2718 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_fcall f_12401(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_12401,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12411,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1930: g2719 */
t4=((C_word*)t0)[3];
f_12294(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12409 in for-each-loop2718 in k12291 in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_12411(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12411,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12401(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_fcall f_12424(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_12424,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12658,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1877: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_fcall f_12434(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,4)))){
C_save_and_reclaim_args((void *)trf_12434,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
/* optimizer.scm:1881: descend */
t2=((C_word*)((C_word*)t0)[2])[1];
f_12424(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12444,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_list1(&a,1,((C_word*)t0)[6]);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12456,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:1888: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k12442 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_12444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12444,c,av);}
/* optimizer.scm:1883: chicken.compiler.support#copy-node! */
t2=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_12456(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_12456,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12460,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12633,a[2]=t2,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1890: chicken.base#gensym */
t4=*((C_word*)lf[116]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k12458 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_12460(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12460,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1884: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_12472(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_12472,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12476,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12480,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12484,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1923: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k12474 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12476,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1889: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12478 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12480,c,av);}
/* optimizer.scm:1921: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=lf[188];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12482 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_12484,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12488,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12492,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12502,tmp=(C_word)a,a+=2,tmp);
/* optimizer.scm:1925: list-tabulate */
f_4412(t3,((C_word*)t0)[3],t4);}

/* k12486 in k12482 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12488,c,av);}
a=C_alloc(3);
/* optimizer.scm:1923: cons* */
f_3834(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,1,t1));}

/* k12490 in k12482 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12492(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12492,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12500,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1926: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12498 in k12490 in k12482 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in ... */
static void C_ccall f_12500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12500,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1924: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a12501 in k12482 in k12470 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12502,c,av);}
/* optimizer.scm:1925: chicken.compiler.support#qnode */
t2=*((C_word*)lf[77]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12516 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_12518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12518,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1892: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[31];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12528 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12530,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1895: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[29];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12536 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12538,c,av);}
/* optimizer.scm:1900: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[40];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12544 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12546,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12554,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1917: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12552 in k12544 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in ... */
static void C_ccall f_12554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12554,c,av);}
/* optimizer.scm:1903: scheme#append */
t2=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=C_i_cdr(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in ... */
static void C_ccall f_12556(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12556,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12564,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1907: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12562 in a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in ... */
static void C_ccall f_12564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_12564,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12568,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_i_caddr(((C_word*)t0)[3]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12574,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_12574(t7,t2,((C_word*)t0)[4],t3);}

/* k12566 in k12562 in a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in ... */
static void C_ccall f_12568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_12568,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k12562 in a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in ... */
static void C_fcall f_12574(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_12574,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_i_cadddr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t3);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12599,a[2]=t1,a[3]=t5,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1914: chicken.compiler.support#varnode */
t7=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k12597 in loop in k12562 in a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in ... */
static void C_ccall f_12599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12599,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12603,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1915: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_12574(t3,t2,C_u_i_cdr(((C_word*)t0)[5]),C_u_i_cdr(((C_word*)t0)[6]));}

/* k12601 in k12597 in loop in k12562 in a12555 in k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in ... */
static void C_ccall f_12603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12603,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1912: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12621 in k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in ... */
static void C_ccall f_12623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_12623,c,av);}
a=C_alloc(11);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12546,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12556,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1905: append-map */
f_3600(t3,t4,((C_word*)t0)[5],C_SCHEME_END_OF_LIST);}

/* k12627 in k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in ... */
static void C_ccall f_12629(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,2)))){
C_save_and_reclaim((void *)f_12629,c,av);}
a=C_alloc(58);
t2=C_a_i_list4(&a,4,t1,C_SCHEME_TRUE,((C_word*)t0)[2],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12530,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=((C_word*)((C_word*)t0)[4])[1];
t5=C_s_a_i_minus(&a,2,t4,C_fix(1));
t6=C_a_i_list1(&a,1,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12538,a[2]=t3,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12623,a[2]=t7,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1904: chicken.compiler.support#varnode */
t9=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k12631 in k12454 in k12432 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_12633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,2)))){
C_save_and_reclaim((void *)f_12633,c,av);}
a=C_alloc(24);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12472,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12518,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12629,a[2]=((C_word*)t0)[5],a[3]=t5,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1897: chicken.base#gensym */
t7=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[189];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k12644 in k12648 in k12652 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_12646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12646,c,av);}
t2=((C_word*)t0)[2];
f_12434(t2,C_eqp(lf[34],t1));}

/* k12648 in k12652 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_12650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12650,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12646,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1880: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12652 in k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_12654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12654,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[26],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12650,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1879: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_12434(t3,C_SCHEME_FALSE);}}

/* k12656 in descend in k12288 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_12658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_12658,c,av);}
a=C_alloc(16);
t2=C_i_cadr(t1);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_12434,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12654,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1878: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k12663 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_12665(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12665,c,av);}
/* optimizer.scm:1870: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[190];
av2[3]=lf[191];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop2728 in k12285 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_fcall f_12667(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_12667,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12701 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_12703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12703,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
/* optimizer.scm:1866: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a12708 in k12282 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_12709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12709,c,av);}
/* optimizer.scm:1867: chicken.base#gensym */
t2=*((C_word*)lf[116]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[192];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12730 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_12732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12732,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[193]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop2687 in k12279 in k12276 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_fcall f_12734(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_12734,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_caddr(t3);
t5=C_i_length(t4);
t6=C_a_i_cons(&a,2,t5,C_SCHEME_END_OF_LIST);
t7=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t6);
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,t6);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop2620 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12768(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12768,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12793,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1845: g2626 */
t4=((C_word*)t0)[4];
f_12183(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12791 in map-loop2620 in k12177 in k12174 in g2516 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_12793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12793,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_12768(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k12804 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12806,c,av);}
/* optimizer.scm:1952: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_pairp(((C_word*)((C_word*)t0)[4])[1]);
C_values(4,av2);}}

/* for-each-loop2515 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12815(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_12815,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12825,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1837: g2516 */
t4=((C_word*)t0)[3];
f_12168(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k12823 in for-each-loop2515 in k12165 in k12162 in chicken.compiler.optimizer#determine-loop-and-dispatch in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12825,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_12815(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12854,c,av);}
a=C_alloc(9);
t10=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12974,a[2]=t5,a[3]=t7,a[4]=t6,a[5]=t1,a[6]=t8,a[7]=t9,a[8]=t4,tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:975: membership-test-operators */
t11=*((C_word*)lf[18]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}

/* k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_12876,c,av);}
a=C_alloc(24);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_list1(&a,1,t2);
t4=C_a_i_list1(&a,1,t1);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12894,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12902,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t5,a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12904,a[2]=t3,a[3]=t1,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_12904(t10,t6,((C_word*)t0)[8]);}

/* k12892 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12894(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12894,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:980: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12900 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_12902,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,t1,((C_word*)t0)[2],((C_word*)t0)[3]);
/* optimizer.scm:984: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[25];
av2[3]=((C_word*)t0)[5];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12904(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_12904,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12912,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12951,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
/* optimizer.scm:995: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* g1642 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_12912(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_12912,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12924,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12936,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:992: chicken.compiler.support#varnode */
t6=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k12922 in g1642 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12924,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12928,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:993: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12926 in k12922 in g1642 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_12928,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],t1,((C_word*)t0)[3]);
/* optimizer.scm:989: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[39];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12934 in g1642 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12936,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12940,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:992: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12938 in k12934 in g1642 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12940,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:992: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12949 in foldr1637 in k12874 in k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12951(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12951,c,av);}
/* optimizer.scm:987: g1642 */
t2=((C_word*)t0)[2];
f_12912(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k12968 in k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12970(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12970,c,av);}
a=C_alloc(9);
if(C_truep(C_i_lessp(((C_word*)t0)[2],t1))){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12876,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:978: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k12972 in a12853 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_12974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_12974,c,av);}
a=C_alloc(10);
t2=C_i_assoc(((C_word*)t0)[2],t1);
if(C_truep(t2)){
if(C_truep(C_i_listp(((C_word*)t0)[3]))){
t3=C_i_length(((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12970,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:977: membership-unfold-limit */
t5=*((C_word*)lf[19]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a12999 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10=av[10];
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_13000,c,av);}
a=C_alloc(7);
if(C_truep(t3)){
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13015,a[2]=t1,a[3]=t5,a[4]=t7,a[5]=t8,a[6]=t9,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:963: chicken.compiler.support#varnode */
t12=*((C_word*)lf[83]+1);{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=t10;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t11=t1;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}

/* k13013 in a12999 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_13015,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13019,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_list3(&a,3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);
/* optimizer.scm:964: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[39];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k13017 in k13013 in a12999 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_13019,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:961: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13081(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13081,c,av);}
a=C_alloc(8);
if(C_truep(t3)){
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13180,a[2]=t7,a[3]=t5,a[4]=t6,a[5]=t4,a[6]=t1,a[7]=t8,tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:924: chicken.compiler.support#db-get-list */
t10=*((C_word*)lf[155]+1);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
av2[3]=t4;
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}

/* loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_13096(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_13096,5,t0,t1,t2,t3,t4);}
a=C_alloc(14);
if(C_truep(C_i_nullp(t2))){
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13113,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:930: scheme#reverse */
t6=*((C_word*)lf[4]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13119,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=t4,tmp=(C_word)a,a+=9,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13167,a[2]=t5,a[3]=((C_word*)t0)[6],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:932: chicken.compiler.support#node-class */
t7=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k13111 in loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13113,c,av);}
/* optimizer.scm:928: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13117 in loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_13119(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_13119,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13132,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[6]);
/* optimizer.scm:936: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[58];
av2[3]=t4;
av2[4]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* optimizer.scm:942: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_13096(t5,((C_word*)t0)[5],t2,t4,((C_word*)t0)[8]);}}

/* k13130 in k13117 in loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13132(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_13132,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:935: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13096(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2,C_SCHEME_TRUE);}

/* k13159 in k13165 in loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13161,c,av);}
t2=((C_word*)t0)[2];
f_13119(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k13165 in loop in k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13167,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13161,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:934: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_13119(t3,C_SCHEME_FALSE);}}

/* k13178 in a13080 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_13180,c,av);}
a=C_alloc(9);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13096,a[2]=((C_word*)t0)[2],a[3]=t5,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp));
t7=((C_word*)t5)[1];
f_13096(t7,((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}
else{
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10=av[10];
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_13214,c,av);}
a=C_alloc(11);
if(C_truep(t3)){
t11=C_eqp(t4,t8);
if(C_truep(C_i_not(t11))){
t12=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_13345,a[2]=t7,a[3]=t9,a[4]=t8,a[5]=t5,a[6]=t6,a[7]=t4,a[8]=t1,a[9]=t10,a[10]=t2,tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:887: chicken.compiler.support#db-get */
t13=*((C_word*)lf[55]+1);{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
av2[2]=t2;
av2[3]=t8;
av2[4]=lf[91];
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}
else{
t12=t1;{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t11=t1;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}

/* loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_13241(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_13241,5,t0,t1,t2,t3,t4);}
a=C_alloc(14);
if(C_truep(C_i_nullp(t2))){
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13262,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:894: chicken.compiler.support#varnode */
t6=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13280,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t4,tmp=(C_word)a,a+=9,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13328,a[2]=t5,a[3]=((C_word*)t0)[8],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:900: chicken.compiler.support#node-class */
t7=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k13260 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_13262,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13266,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_list1(&a,1,((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13274,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:898: scheme#reverse */
t5=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13264 in k13260 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13266(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_13266,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:892: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13272 in k13260 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13274,c,av);}
/* optimizer.scm:895: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13278 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_13280(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_13280,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13293,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[6]);
/* optimizer.scm:904: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[58];
av2[3]=t4;
av2[4]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* optimizer.scm:910: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_13241(t5,((C_word*)t0)[5],t2,t4,((C_word*)t0)[8]);}}

/* k13291 in k13278 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_13293,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:903: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13241(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2,C_SCHEME_TRUE);}

/* k13320 in k13326 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13322,c,av);}
t2=((C_word*)t0)[2];
f_13280(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k13326 in loop in k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13328,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13322,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:902: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_13280(t3,C_SCHEME_FALSE);}}

/* k13339 in k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13341(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_13341,c,av);}
a=C_alloc(11);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13241,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t5,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp));
t7=((C_word*)t5)[1];
f_13241(t7,((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}
else{
t4=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13343 in a13213 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_13345,c,av);}
a=C_alloc(10);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13341,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:888: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=((C_word*)t0)[7];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a13398 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13399,c,av);}
a=C_alloc(10);
t10=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13444,a[2]=t5,a[3]=t8,a[4]=t9,a[5]=t1,a[6]=t7,a[7]=t6,a[8]=t2,a[9]=t4,tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:867: eq-inline-operator */
t11=*((C_word*)lf[17]+1);{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}

/* k13421 in k13434 in k13442 in a13398 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13423,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,t1,((C_word*)t0)[2],((C_word*)t0)[3]);
/* optimizer.scm:869: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[25];
av2[3]=((C_word*)t0)[5];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13434 in k13442 in a13398 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13436,c,av);}
a=C_alloc(9);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13423,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_list1(&a,1,((C_word*)t0)[6]);
/* optimizer.scm:871: chicken.compiler.support#make-node */
t6=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=lf[58];
av2[3]=t5;
av2[4]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13442 in a13398 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13444,c,av);}
a=C_alloc(8);
t2=C_i_equalp(((C_word*)t0)[2],t1);
if(C_truep(C_i_not(t2))){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13436,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:868: chicken.compiler.support#db-get-list */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)t0)[9];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13482(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13482,c,av);}
a=C_alloc(9);
t6=C_a_i_list1(&a,1,t4);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13492,a[2]=t8,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_13492(t10,t1,t6,t5);}

/* loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_13492(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_13492,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13496,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[3],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:802: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13496,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13499,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:803: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13499,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13502,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:804: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13502,c,av);}
a=C_alloc(8);
t2=C_eqp(((C_word*)t0)[2],lf[26]);
if(C_truep(t2)){
t3=C_i_cdr(((C_word*)t0)[3]);
if(C_truep(C_i_nullp(t3))){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13769,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:807: chicken.compiler.support#db-get */
t5=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=C_i_car(((C_word*)t0)[3]);
av2[4]=lf[136];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13532,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13535,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:811: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13535(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13535,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13538,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:812: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13538,c,av);}
a=C_alloc(7);
t2=C_eqp(t1,lf[34]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* optimizer.scm:813: loop1 */
t5=((C_word*)((C_word*)t0)[4])[1];
f_13492(t5,((C_word*)t0)[5],t4,C_i_cadr(((C_word*)t0)[6]));}
else{
t3=C_eqp(t1,lf[31]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13568,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:815: scheme#reverse */
t5=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_13568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_13568,c,av);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t1))){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_car(t1);
t4=C_eqp(t2,t3);
if(C_truep(t4)){
t5=C_i_car(((C_word*)t0)[3]);
t6=C_a_i_list1(&a,1,t5);
t7=C_u_i_cdr(t1);
t8=C_i_cadr(((C_word*)t0)[4]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13595,a[2]=t10,a[3]=t1,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_13595(t12,((C_word*)t0)[6],t6,t7,t8);}
else{
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_fcall f_13595(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_13595,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13599,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,a[6]=((C_word*)t0)[3],a[7]=t4,a[8]=((C_word*)t0)[4],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:821: chicken.compiler.support#node-class */
t6=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_13599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13599,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13602,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:822: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_13602(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_13602,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_13605,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:823: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_13605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_13605,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13611,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=C_eqp(((C_word*)t0)[8],lf[26]);
if(C_truep(t3)){
t4=C_i_cdr(((C_word*)t0)[9]);
if(C_truep(C_i_nullp(t4))){
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13733,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[9],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:826: chicken.compiler.support#db-get */
t6=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[10];
av2[3]=C_i_car(((C_word*)t0)[9]);
av2[4]=lf[136];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t2;
f_13611(t5,C_SCHEME_FALSE);}}
else{
t4=t2;
f_13611(t4,C_SCHEME_FALSE);}}

/* k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_fcall f_13611(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_13611,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13634,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:831: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_i_nullp(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13649,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13659,tmp=(C_word)a,a+=2,tmp);
/* optimizer.scm:835: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k13632 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_13634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_13634,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[2]);
/* optimizer.scm:831: loop2 */
t4=((C_word*)((C_word*)t0)[3])[1];
f_13595(t4,((C_word*)t0)[4],t3,C_i_cdr(((C_word*)t0)[5]),C_i_cadr(((C_word*)t0)[6]));}

/* a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_13649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_13649,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13657,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:836: scheme#reverse */
t3=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_13657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_13657,c,av);}
a=C_alloc(21);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=C_i_check_list_2(((C_word*)t0)[2],lf[8]);
t9=C_i_check_list_2(t1,lf[8]);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7753,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8228,a[2]=t6,a[3]=t12,a[4]=t7,tmp=(C_word)a,a+=5,tmp));
t14=((C_word*)t12)[1];
f_8228(t14,t10,((C_word*)t0)[2],t1);}

/* a13658 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_13659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13659,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?t2:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13707 in k13715 in k13723 in k13731 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_13709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13709,c,av);}
t2=((C_word*)t0)[2];
f_13611(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k13715 in k13723 in k13731 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in ... */
static void C_ccall f_13717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13717,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[31],t1);
if(C_truep(t2)){
t3=C_u_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13709,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:830: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];
f_13611(t3,C_SCHEME_FALSE);}}

/* k13723 in k13731 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_13725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_13725,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13717,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:829: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];
f_13611(t2,C_SCHEME_FALSE);}}
else{
t2=((C_word*)t0)[3];
f_13611(t2,C_SCHEME_FALSE);}}

/* k13731 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_13733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13733,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:827: chicken.compiler.support#db-get */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_i_car(((C_word*)t0)[6]);
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[3];
f_13611(t2,C_SCHEME_FALSE);}}

/* k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_13761,c,av);}
a=C_alloc(9);
if(C_truep(C_i_not(t1))){
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13532,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:810: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13769,c,av);}
a=C_alloc(8);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13761,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:808: chicken.compiler.support#db-get */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=C_i_car(((C_word*)t0)[3]);
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10=av[10];
C_word t11=av[11];
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_13795,c,av);}
a=C_alloc(11);
t12=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_13848,a[2]=t5,a[3]=t10,a[4]=t1,a[5]=t9,a[6]=t11,a[7]=t7,a[8]=t6,a[9]=t2,a[10]=t4,tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:776: eq-inline-operator */
t13=*((C_word*)lf[17]+1);{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}

/* k13806 in k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13808(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13808,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13844,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:778: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)t0)[9];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13823 in k13842 in k13806 in k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13825,c,av);}
/* optimizer.scm:779: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[40];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13827 in k13842 in k13806 in k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13829(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13829,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13833,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:783: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13831 in k13827 in k13842 in k13806 in k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13833(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_13833,c,av);}
a=C_alloc(9);
/* optimizer.scm:782: cons* */
f_3834(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,3,t1,((C_word*)t0)[4],((C_word*)t0)[5]));}

/* k13842 in k13806 in k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,2)))){
C_save_and_reclaim((void *)f_13844,c,av);}
a=C_alloc(42);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13825,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13829,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:782: chicken.compiler.support#varnode */
t8=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13846 in a13794 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13848,c,av);}
a=C_alloc(10);
if(C_truep(C_i_equalp(((C_word*)t0)[2],t1))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13808,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:777: chicken.compiler.support#immediate? */
t3=*((C_word*)lf[215]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9=av[9];
C_word t10=av[10];
C_word t11=av[11];
C_word t12=av[12];
C_word t13=av[13];
C_word t14=av[14];
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_13922,c,av);}
a=C_alloc(13);
t15=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_13994,a[2]=t7,a[3]=t10,a[4]=t11,a[5]=t14,a[6]=t1,a[7]=t9,a[8]=t8,a[9]=t4,a[10]=t2,a[11]=t6,a[12]=t5,tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:749: eq-inline-operator */
t16=*((C_word*)lf[17]+1);{
C_word *av2=av;
av2[0]=t16;
av2[1]=t15;
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}

/* k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_13935,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_13941,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:751: chicken.compiler.support#immediate? */
t3=*((C_word*)lf[215]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_13941,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_13990,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:752: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[11];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k13962 in k13979 in k13988 in k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13964,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13968,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:758: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13966 in k13962 in k13979 in k13988 in k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13968(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13968,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13972,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:760: chicken.compiler.support#qnode */
t3=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13970 in k13966 in k13962 in k13979 in k13988 in k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_13972,c,av);}
a=C_alloc(18);
t2=C_a_i_list6(&a,6,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6]);
/* optimizer.scm:754: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
av2[2]=lf[40];
av2[3]=lf[214];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13979 in k13988 in k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13981,c,av);}
a=C_alloc(8);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13964,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:757: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13988 in k13939 in k13933 in k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13990(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13990,c,av);}
a=C_alloc(9);
t2=C_i_length(t1);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13981,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:753: chicken.compiler.support#db-get-list */
t5=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k13992 in a13921 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_13994(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_13994,c,av);}
a=C_alloc(12);
if(C_truep(C_i_equalp(((C_word*)t0)[2],t1))){
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_13935,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:750: chicken.compiler.support#immediate? */
t3=*((C_word*)lf[215]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_14092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_14092,c,av);}
a=C_alloc(9);
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_14100,a[2]=t4,a[3]=t6,a[4]=t3,a[5]=t5,a[6]=t2,a[7]=t7,a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:717: chicken.internal#hash-table-ref */
t9=*((C_word*)lf[57]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[159];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}

/* k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_14100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_14100,c,av);}
a=C_alloc(11);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_14105,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp));
t6=((C_word*)t4)[1];
f_14105(t6,((C_word*)t0)[8],t2);}

/* loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_14105(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,2)))){
C_save_and_reclaim_args((void *)trf_14105,3,t0,t1,t2);}
a=C_alloc(19);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14115,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_i_caar(t2);
t5=C_u_i_car(t2);
t6=C_u_i_cdr(t5);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8303,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
switch(t4){
case C_fix(1):
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8393,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=t6,a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1116: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}
case C_fix(2):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_length(((C_word*)t0)[4]);
t9=C_i_car(t6);
if(C_truep(C_i_nequalp(t8,t9))){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8512,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1134: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(3):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8570,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1147: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(4):
if(C_truep(((C_word*)t0)[5])){
if(C_truep(*((C_word*)lf[232]+1))){
t8=C_i_length(((C_word*)t0)[4]);
t9=C_eqp(C_fix(2),t8);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8684,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1160: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(5):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8743,a[2]=((C_word*)t0)[4],a[3]=t6,a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1172: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(6):
t8=C_i_caddr(t6);
t9=(C_truep(t8)?t8:*((C_word*)lf[232]+1));
if(C_truep(t9)){
if(C_truep(((C_word*)t0)[5])){
t10=C_i_length(((C_word*)t0)[4]);
t11=C_eqp(C_fix(1),t10);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8838,a[2]=((C_word*)t0)[6],a[3]=t3,a[4]=t6,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1187: chicken.compiler.support#intrinsic? */
t13=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t12=t3;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(7):
t8=C_i_cadddr(t6);
t9=(C_truep(t8)?t8:*((C_word*)lf[232]+1));
if(C_truep(t9)){
if(C_truep(((C_word*)t0)[5])){
t10=C_i_length(((C_word*)t0)[4]);
t11=C_i_car(t6);
if(C_truep(C_i_nequalp(t10,t11))){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8912,a[2]=((C_word*)t0)[6],a[3]=t3,a[4]=t6,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1199: chicken.compiler.support#intrinsic? */
t13=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t12=t3;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(8):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8974,a[2]=t6,a[3]=t3,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1209: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(9):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8995,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[6],a[4]=t3,a[5]=t6,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1216: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(10):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_cadddr(t6);
t9=(C_truep(t8)?t8:*((C_word*)lf[232]+1));
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9250,a[2]=((C_word*)t0)[4],a[3]=t6,a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1247: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(11):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_caddr(t6);
t9=(C_truep(t8)?t8:*((C_word*)lf[232]+1));
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9339,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1264: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(12):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9400,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1277: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(13):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9479,a[2]=t6,a[3]=t7,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1291: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(14):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_cadr(t6);
t9=C_i_length(((C_word*)t0)[4]);
if(C_truep(C_i_nequalp(t8,t9))){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9549,a[2]=t6,a[3]=((C_word*)t0)[6],a[4]=t3,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1303: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(15):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_length(((C_word*)t0)[4]);
t9=C_eqp(C_fix(1),t8);
if(C_truep(t9)){
t10=*((C_word*)lf[232]+1);
t11=(C_truep(*((C_word*)lf[232]+1))?*((C_word*)lf[232]+1):C_i_cadddr(t6));
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9632,a[2]=t6,a[3]=t3,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1320: chicken.compiler.support#intrinsic? */
t13=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t12=t3;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(16):
t8=C_i_car(t6);
t9=C_i_length(((C_word*)t0)[4]);
t10=C_i_caddr(t6);
t11=C_i_cadddr(t6);
t12=C_i_cddddr(t6);
t13=C_i_pairp(t12);
t14=(C_truep(t13)?C_i_car(C_i_cddddr(t6)):C_SCHEME_FALSE);
if(C_truep(((C_word*)t0)[5])){
t15=C_i_not(t8);
t16=(C_truep(t15)?t15:C_i_nequalp(t9,t8));
if(C_truep(t16)){
t17=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9733,a[2]=t10,a[3]=((C_word*)t0)[6],a[4]=t3,a[5]=t11,a[6]=t9,a[7]=((C_word*)t0)[4],a[8]=t14,a[9]=t6,tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1345: chicken.compiler.support#intrinsic? */
t18=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t18;
av2[1]=t17;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t18+1)))(3,av2);}}
else{
t17=t3;{
C_word av2[2];
av2[0]=t17;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t15=t3;{
C_word av2[2];
av2[0]=t15;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(17):
if(C_truep(((C_word*)t0)[5])){
t8=C_i_length(((C_word*)t0)[4]);
t9=C_i_car(t6);
if(C_truep(C_i_nequalp(t8,t9))){
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9845,a[2]=((C_word*)t0)[6],a[3]=t3,a[4]=t6,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1365: chicken.compiler.support#intrinsic? */
t11=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=t3;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(18):
if(C_truep(((C_word*)t0)[5])){
if(C_truep(C_i_nullp(((C_word*)t0)[4]))){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9913,a[2]=((C_word*)t0)[6],a[3]=t3,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1379: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(19):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9947,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1385: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(20):
t8=C_i_length(((C_word*)t0)[4]);
t9=C_i_cadddr(t6);
t10=(C_truep(t9)?t9:*((C_word*)lf[232]+1));
if(C_truep(t10)){
if(C_truep(((C_word*)t0)[5])){
t11=C_i_car(t6);
if(C_truep(C_i_nequalp(t8,t11))){
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10099,a[2]=((C_word*)t0)[6],a[3]=t3,a[4]=t6,a[5]=t8,a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1417: chicken.compiler.support#intrinsic? */
t13=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t13;
av2[1]=t12;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t12=t3;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t11=t3;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t11=t3;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(21):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10171,a[2]=t6,a[3]=((C_word*)t0)[6],a[4]=t3,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1435: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(22):
t8=C_i_car(t6);
t9=C_i_length(((C_word*)t0)[4]);
t10=C_i_cadddr(t6);
if(C_truep(((C_word*)t0)[5])){
if(C_truep(C_i_nequalp(t9,t8))){
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10342,a[2]=t6,a[3]=((C_word*)t0)[6],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=t10,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1469: chicken.compiler.support#intrinsic? */
t12=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t12;
av2[1]=t11;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t11=t3;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t11=t3;{
C_word av2[2];
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
case C_fix(23):
if(C_truep(((C_word*)t0)[5])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10406,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1490: chicken.compiler.support#intrinsic? */
t9=*((C_word*)lf[129]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=t3;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}
default:
/* optimizer.scm:1510: chicken.compiler.support#bomb */
t8=*((C_word*)lf[174]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t3;
av2[2]=lf[243];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}}

/* k14113 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_14115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(38,c,2)))){
C_save_and_reclaim((void *)f_14115,c,av);}
a=C_alloc(38);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14119,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:718: g1440 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* optimizer.scm:718: g1440 */
  f_14119(C_a_i(&a,35),t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* optimizer.scm:727: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_14105(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[5]));}}

/* g1440 in k14113 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_14119(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;{}
t2=C_i_assq(((C_word*)t0)[2],lf[51]);
if(C_truep(t2)){
t3=C_i_cdr(t2);
t4=C_s_a_i_plus(&a,2,t3,C_fix(1));
t5=C_i_set_cdr(t2,t4);
return(t1);}
else{
t3=lf[51];
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],C_fix(1));
t5=C_a_i_cons(&a,2,t4,lf[51]);
t6=C_mutate(&lf[51] /* (set! chicken.compiler.optimizer#simplified-ops ...) */,t5);
return(t1);}}

/* k3385 */
static void C_ccall f_3387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3387,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3390,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_eval_toplevel(2,av2);}}

/* k3388 in k3385 */
static void C_ccall f_3390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3390,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3393,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_data_2dstructures_toplevel(2,av2);}}

/* k3391 in k3388 in k3385 */
static void C_ccall f_3393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3393,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3396,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_internal_toplevel(2,av2);}}

/* k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_3396,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3399,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_support_toplevel(2,av2);}}

/* k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,5)))){
C_save_and_reclaim((void *)f_3399,c,av);}
a=C_alloc(39);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=C_mutate(&lf[2] /* (set! chicken.compiler.optimizer#take ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3509,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate(&lf[3] /* (set! chicken.compiler.optimizer#split-at ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3561,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate(&lf[5] /* (set! chicken.compiler.optimizer#append-map ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3600,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[11] /* (set! chicken.compiler.optimizer#every ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3770,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate(&lf[9] /* (set! chicken.compiler.optimizer#any ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3804,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate(&lf[12] /* (set! chicken.compiler.optimizer#cons* ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3834,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate(&lf[13] /* (set! chicken.compiler.optimizer#delete ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3890,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate(&lf[14] /* (set! chicken.compiler.optimizer#first ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_3931,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate(&lf[15] /* (set! chicken.compiler.optimizer#filter ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4010,tmp=(C_word)a,a+=2,tmp));
t13=C_mutate(&lf[16] /* (set! chicken.compiler.optimizer#list-tabulate ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4412,tmp=(C_word)a,a+=2,tmp));
t14=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4681,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:52: chicken.base#make-parameter */
t15=*((C_word*)lf[244]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t15;
av2[1]=t14;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t15+1)))(3,av2);}}

/* loop in a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3407(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_3407,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3421,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:43: scheme#reverse */
t6=*((C_word*)lf[4]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_3431,a[2]=t4,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* mini-srfi-1.scm:44: pred */
t6=((C_word*)t0)[3];
f_5863(t6,t5,C_i_car(t4));}}

/* k3419 in loop in a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3421(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_3421,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3425,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:43: scheme#reverse */
t3=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k3423 in k3419 in loop in a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3425,c,av);}
/* mini-srfi-1.scm:43: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_values(4,av2);}}

/* k3429 in loop in a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_3431,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[3]);
/* mini-srfi-1.scm:44: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_3407(t4,((C_word*)t0)[5],t3,((C_word*)t0)[6],C_u_i_cdr(((C_word*)t0)[2]));}
else{
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[6]);
/* mini-srfi-1.scm:45: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_3407(t4,((C_word*)t0)[5],((C_word*)t0)[3],t3,C_u_i_cdr(((C_word*)t0)[2]));}}

/* chicken.compiler.optimizer#take in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3509(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_3509,3,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3527,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:56: take */
t7=t5;
t8=C_u_i_cdr(t2);
t9=C_fixnum_difference(t3,C_fix(1));
t1=t7;
t2=t8;
t3=t9;
goto loop;}}

/* k3525 in chicken.compiler.optimizer#take in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3527(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_3527,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.optimizer#split-at in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3561(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_3561,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3567,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_3567(t7,t1,t3,C_SCHEME_END_OF_LIST,t2);}

/* loop in chicken.compiler.optimizer#split-at in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3567(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_3567,5,t0,t1,t2,t3,t4);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3581,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:67: scheme#reverse */
t6=*((C_word*)lf[4]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_fixnum_difference(t2,C_fix(1));
t6=C_i_car(t4);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:68: loop */
t9=t1;
t10=t5;
t11=t7;
t12=C_u_i_cdr(t4);
t1=t9;
t2=t10;
t3=t11;
t4=t12;
goto loop;}}

/* k3579 in loop in chicken.compiler.optimizer#split-at in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3581,c,av);}
/* mini-srfi-1.scm:67: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3600(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_3600,4,t1,t2,t3,t4);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t4))){
t5=C_i_check_list_2(t3,lf[6]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3615,a[2]=t2,a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_3615(t9,t1,t3);}
else{
t5=C_a_i_cons(&a,2,t3,t4);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3655,a[2]=t7,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_3655(t9,t1,t5);}}

/* foldr113 in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3615(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_3615,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3623,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3642,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g118 in foldr113 in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3623(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_3623,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3631,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:72: proc */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k3629 in g118 in foldr113 in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3631,c,av);}
/* mini-srfi-1.scm:72: scheme#append */
t2=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k3640 in foldr113 in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3642,c,av);}
/* mini-srfi-1.scm:72: g118 */
t2=((C_word*)t0)[2];
f_3623(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3655(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_3655,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_3662,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* mini-srfi-1.scm:74: any */
f_3804(t3,*((C_word*)lf[10]+1),t2);}

/* k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_3662,c,av);}
a=C_alloc(21);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3669,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[4],lf[8]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3734,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3736,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_3736(t12,t8,((C_word*)t0)[4]);}}

/* k3667 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_3669,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3673,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3685,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3687,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_3687(t11,t7,((C_word*)t0)[4]);}

/* k3671 in k3667 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3673,c,av);}
/* mini-srfi-1.scm:76: scheme#append */
t2=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k3683 in k3667 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_3685,c,av);}
/* mini-srfi-1.scm:77: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_3655(t2,((C_word*)t0)[3],t1);}

/* map-loop155 in k3667 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3687(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_3687,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cdr(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k3732 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3734(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_3734,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop128 in k3660 in loop in chicken.compiler.optimizer#append-map in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3736(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_3736,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* chicken.compiler.optimizer#every in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3770(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_3770,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3776,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_3776(t7,t1,t3);}

/* loop in chicken.compiler.optimizer#every in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3776(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_3776,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3798,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:82: pred */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k3796 in loop in chicken.compiler.optimizer#every in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_3798,c,av);}
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:83: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_3776(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.compiler.optimizer#any in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3804(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_3804,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3810,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_3810(t7,t1,t3);}

/* loop in chicken.compiler.optimizer#any in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3810(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_3810,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3820,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:88: pred */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k3818 in loop in chicken.compiler.optimizer#any in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_3820,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:89: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_3810(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* chicken.compiler.optimizer#cons* in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3834(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_3834,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3840,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_3840(t7,t1,t2,t3);}

/* loop in chicken.compiler.optimizer#cons* in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3840(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_3840,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3854,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:95: loop */
t6=t4;
t7=C_i_car(t3);
t8=C_u_i_cdr(t3);
t1=t6;
t2=t7;
t3=t8;
goto loop;}}

/* k3852 in loop in chicken.compiler.optimizer#cons* in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_3854,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.optimizer#delete in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3890(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_3890,4,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3896,a[2]=t6,a[3]=t4,a[4]=t2,tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_3896(t8,t1,t3);}

/* loop in chicken.compiler.optimizer#delete in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3896(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_3896,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_3909,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:106: test */
t4=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k3907 in loop in chicken.compiler.optimizer#delete in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3909(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_3909,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
/* mini-srfi-1.scm:107: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_3896(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=C_u_i_car(((C_word*)t0)[4]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3923,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:109: loop */
t4=((C_word*)((C_word*)t0)[2])[1];
f_3896(t4,t3,C_u_i_cdr(((C_word*)t0)[4]));}}

/* k3921 in k3907 in loop in chicken.compiler.optimizer#delete in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_3923,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.optimizer#first in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_3931,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_3971(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_3971,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_3985,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3998,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:123: delete */
f_3890(t6,t3,t4,((C_word*)t0)[3]);}}

/* k3983 in loop in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_3985,c,av);}
a=C_alloc(3);
t2=C_i_equalp(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?((C_word*)t0)[4]:C_a_i_cons(&a,2,((C_word*)t0)[5],t1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k3996 in loop in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_3998(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_3998,c,av);}
/* mini-srfi-1.scm:123: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_3971(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.optimizer#filter in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4010(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_4010,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_check_list_2(t3,lf[6]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4019,a[2]=t2,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_4019(t8,t1,t3);}

/* foldr260 in chicken.compiler.optimizer#filter in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4019(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_4019,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4027,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4048,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g265 in foldr260 in chicken.compiler.optimizer#filter in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4027(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_4027,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4034,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:131: pred */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k4032 in g265 in foldr260 in chicken.compiler.optimizer#filter in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_4034,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]):((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k4046 in foldr260 in chicken.compiler.optimizer#filter in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4048(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4048,c,av);}
/* mini-srfi-1.scm:131: g265 */
t2=((C_word*)t0)[2];
f_4027(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* foldr278 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_fcall f_4063(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_4063,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4071,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4097,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g283 in foldr278 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_fcall f_4071(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_4071,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4075,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:135: pred */
t5=((C_word*)t0)[2];
f_7888(t5,t4,t2);}

/* k4073 in g283 in foldr278 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in ... */
static void C_ccall f_4075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_4075,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4079,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:135: g293 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* mini-srfi-1.scm:135: g293 */
  f_4079(C_a_i(&a,3),t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g293 in k4073 in g283 in foldr278 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in ... */
static C_word C_fcall f_4079(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_a_i_cons(&a,2,t1,((C_word*)t0)[2]));}

/* k4095 in foldr278 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_ccall f_4097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4097,c,av);}
/* mini-srfi-1.scm:134: g283 */
t2=((C_word*)t0)[2];
f_4071(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* map-loop306 in a11856 in a11846 in transform in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4136(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_4136,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* loop in k4868 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_4176(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
return(C_u_i_car(t1));}
else{
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}}

/* loop in k5505 in a5493 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4235(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_4235,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_memq(t4,t3))){
/* mini-srfi-1.scm:160: loop */
t9=t1;
t10=C_u_i_cdr(t2);
t11=t3;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:161: loop */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* chicken.compiler.optimizer#list-tabulate in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4412(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_4412,3,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4418,a[2]=t2,a[3]=t5,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_4418(t7,t1,C_fix(0));}

/* loop in chicken.compiler.optimizer#list-tabulate in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4418(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_4418,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4432,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:190: proc */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k4430 in loop in chicken.compiler.optimizer#list-tabulate in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4432(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_4432,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4436,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:190: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4418(t3,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k4434 in k4430 in loop in chicken.compiler.optimizer#list-tabulate in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_4436,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a4447 in a7887 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_ccall f_4448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4448,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a4465 in k11217 in k11210 in scan in chicken.compiler.optimizer#transform-direct-lambdas! in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4466(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4466,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(t2,((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_4681,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[17]+1 /* (set! chicken.compiler.optimizer#eq-inline-operator ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4685,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:53: chicken.base#make-parameter */
t4=*((C_word*)lf[244]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_4685,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.optimizer#membership-test-operators ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4689,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:54: chicken.base#make-parameter */
t4=*((C_word*)lf[244]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_4689,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.optimizer#membership-unfold-limit ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4693,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:55: chicken.base#make-parameter */
t4=*((C_word*)lf[244]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4693(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_4693,c,av);}
a=C_alloc(5);
t2=C_mutate((C_word*)lf[20]+1 /* (set! chicken.compiler.optimizer#default-optimization-passes ...) */,t1);
t3=C_mutate((C_word*)lf[21]+1 /* (set! chicken.compiler.optimizer#scan-toplevel-assignments ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4695,tmp=(C_word)a,a+=2,tmp));
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5046,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:150: scheme#make-vector */
t5=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_fix(301);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4695(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(49,c,5)))){
C_save_and_reclaim((void *)f_4695,c,av);}
a=C_alloc(49);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_FALSE;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_END_OF_LIST;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_SCHEME_UNDEFINED;
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_SCHEME_UNDEFINED;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4698,a[2]=t8,a[3]=t6,a[4]=t4,tmp=(C_word)a,a+=5,tmp));
t22=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4721,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t23=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4728,a[2]=t8,a[3]=t10,tmp=(C_word)a,a+=4,tmp));
t24=C_set_block_item(t18,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4733,a[2]=t10,a[3]=t20,tmp=(C_word)a,a+=4,tmp));
t25=C_set_block_item(t20,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_4774,a[2]=t10,a[3]=t6,a[4]=t16,a[5]=t18,a[6]=t20,a[7]=t14,a[8]=t12,tmp=(C_word)a,a+=9,tmp));
t26=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4992,a[2]=t4,a[3]=t1,a[4]=t20,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:127: chicken.compiler.support#debugging */
t27=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t27;
av2[1]=t26;
av2[2]=lf[48];
av2[3]=lf[49];
((C_proc)(void*)(*((C_word*)t27+1)))(4,av2);}}

/* mark in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_4698(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;{}
t2=C_i_not(((C_word*)((C_word*)t0)[2])[1]);
t3=(C_truep(t2)?C_i_not(C_i_memq(t1,((C_word*)((C_word*)t0)[3])[1])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_a_i_cons(&a,2,t1,((C_word*)((C_word*)t0)[4])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[4])+1,t4);
return(t5);}
else{
t4=C_SCHEME_UNDEFINED;
return(t4);}}

/* remember in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4721(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_4721,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4726,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:71: chicken.base#alist-update! */
t5=*((C_word*)lf[22]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=t3;
av2[4]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k4724 in remember in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4726,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* touch in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_4728(C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
t1=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
t2=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_END_OF_LIST);
return(t2);}

/* scan-each in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4733(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_4733,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_4735,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t6=C_i_check_list_2(t2,lf[23]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4751,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_4751(t10,t1,t2);}

/* g584 in scan-each in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4735(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_4735,3,t0,t1,t2);}
if(C_truep(((C_word*)t0)[2])){
t3=C_set_block_item(((C_word*)t0)[3],0,C_SCHEME_END_OF_LIST);
/* optimizer.scm:80: scan */
t4=((C_word*)((C_word*)t0)[4])[1];
f_4774(t4,t1,t2,((C_word*)t0)[5]);}
else{
/* optimizer.scm:80: scan */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4774(t3,t1,t2,((C_word*)t0)[5]);}}

/* for-each-loop583 in scan-each in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4751(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_4751,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4761,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:78: g584 */
t4=((C_word*)t0)[3];
f_4735(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k4759 in for-each-loop583 in scan-each in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_4761,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_4751(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4774(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_4774,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_4778,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=t2,a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:84: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_4778,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_4781,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:85: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_4781,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_4784,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* optimizer.scm:86: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_4784,c,av);}
a=C_alloc(15);
t2=C_eqp(t1,lf[24]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4796,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_memq(t3,((C_word*)t0)[5]);
t6=C_i_not(t5);
t7=(C_truep(t6)?C_i_not(C_i_memq(t3,((C_word*)((C_word*)t0)[6])[1])):C_SCHEME_FALSE);
if(C_truep(t7)){
t8=C_a_i_cons(&a,2,t3,((C_word*)((C_word*)t0)[6])[1]);
t9=C_mutate(((C_word *)((C_word*)t0)[6])+1,t8);
t10=t4;
f_4796(t10,t9);}
else{
t8=t4;
f_4796(t8,C_SCHEME_UNDEFINED);}}
else{
t3=C_eqp(t1,lf[25]);
t4=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_4844,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[10],a[8]=t1,a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[6],a[14]=((C_word*)t0)[3],tmp=(C_word)a,a+=15,tmp);
if(C_truep(t3)){
t5=t4;
f_4844(t5,t3);}
else{
t5=C_eqp(t1,lf[39]);
t6=t4;
f_4844(t6,(C_truep(t5)?t5:C_eqp(t1,lf[40])));}}}

/* k4794 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4796(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_4796,2,t0,t1);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4800,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4802,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:93: filter */
f_4010(t2,t3,((C_word*)((C_word*)t0)[2])[1]);}

/* k4798 in k4794 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4800(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4800,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a4801 in k4794 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4802(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_4802,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_not(C_eqp(t3,((C_word*)t0)[2]));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_4844(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_4844,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_4847,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:96: scan */
t3=((C_word*)((C_word*)t0)[7])[1];
f_4774(t3,t2,C_i_car(((C_word*)t0)[5]),((C_word*)t0)[6]);}
else{
t2=C_eqp(((C_word*)t0)[8],lf[26]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_4870,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4885,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:101: chicken.base#butlast */
t5=*((C_word*)lf[27]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=C_eqp(((C_word*)t0)[8],lf[28]);
t4=(C_truep(t3)?t3:C_eqp(((C_word*)t0)[8],lf[29]));
if(C_truep(t4)){
t5=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)t0)[8],lf[30]);
if(C_truep(t5)){
/* optimizer.scm:106: touch */
t6=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t6;
av2[1]=(
/* optimizer.scm:106: touch */
  f_4728(((C_word*)((C_word*)t0)[2])[1])
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_eqp(((C_word*)t0)[8],lf[31]);
if(C_truep(t6)){
t7=C_i_car(((C_word*)t0)[9]);
t8=C_i_car(((C_word*)t0)[5]);
t9=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_4918,a[2]=t7,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[13],a[9]=((C_word*)t0)[14],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:111: scan */
t10=((C_word*)((C_word*)t0)[7])[1];
f_4774(t10,t9,t8,((C_word*)t0)[6]);}
else{
/* optimizer.scm:125: scan-each */
t7=((C_word*)((C_word*)t0)[3])[1];
f_4733(t7,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE);}}}}}}

/* k4845 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_4847,c,av);}
t2=(
/* optimizer.scm:97: touch */
  f_4728(((C_word*)((C_word*)t0)[2])[1])
);
/* optimizer.scm:98: scan-each */
t3=((C_word*)((C_word*)t0)[3])[1];
f_4733(t3,((C_word*)t0)[4],C_i_cdr(((C_word*)t0)[5]),((C_word*)t0)[6],C_SCHEME_TRUE);}

/* k4868 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_4870,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_4176,tmp=(C_word)a,a+=2,tmp);
t3=(
  f_4176(((C_word*)t0)[2])
);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_4881,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:102: scheme#append */
t5=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k4879 in k4868 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4881,c,av);}
/* optimizer.scm:102: scan */
t2=((C_word*)((C_word*)t0)[2])[1];
f_4774(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k4883 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_4885,c,av);}
/* optimizer.scm:101: scan-each */
t2=((C_word*)((C_word*)t0)[2])[1];
f_4733(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],C_SCHEME_FALSE);}

/* k4916 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_4918,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_4921,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:112: chicken.base#alist-ref */
t3=*((C_word*)lf[38]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)((C_word*)t0)[9])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k4919 in k4916 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_4921,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_4924,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=(C_truep(t1)?C_i_not(C_i_memq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[8])[1])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4942,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:118: chicken.compiler.support#debugging */
t5=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[36];
av2[3]=lf[37];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_4924(2,av2);}}}

/* k4922 in k4919 in k4916 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_4924,c,av);}
a=C_alloc(3);
if(C_truep(C_i_memq(((C_word*)t0)[2],((C_word*)t0)[3]))){
/* optimizer.scm:123: remember */
t2=((C_word*)((C_word*)t0)[4])[1];
f_4721(t2,((C_word*)t0)[5],((C_word*)t0)[2],((C_word*)t0)[6]);}
else{
t2=(
/* optimizer.scm:122: mark */
  f_4698(C_a_i(&a,3),((C_word*)((C_word*)t0)[7])[1],((C_word*)t0)[2])
);
/* optimizer.scm:123: remember */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4721(t3,((C_word*)t0)[5],((C_word*)t0)[2],((C_word*)t0)[6]);}}

/* k4940 in k4919 in k4916 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_4942,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4949,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:120: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k4947 in k4940 in k4919 in k4916 in k4842 in k4782 in k4779 in k4776 in scan in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_4949,c,av);}
/* optimizer.scm:119: chicken.compiler.support#copy-node! */
t2=*((C_word*)lf[32]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_4992,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4995,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:128: scan */
t3=((C_word*)((C_word*)t0)[4])[1];
f_4774(t3,t2,((C_word*)t0)[5],C_SCHEME_END_OF_LIST);}

/* k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_4995,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4998,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[2])[1]))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5042,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=((C_word*)((C_word*)t0)[2])[1];
t5=*((C_word*)lf[47]+1);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3971,a[2]=t7,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_3971(t9,t3,t4);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_4998(2,av2);}}}

/* k4996 in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_4998(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_4998,c,av);}
a=C_alloc(5);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_i_check_list_2(t2,lf[23]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5011,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_5011(t7,((C_word*)t0)[3],t2);}

/* for-each-loop565 in k4996 in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5011(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_5011,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5021,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[44]+1);
/* optimizer.scm:131: g642 */
t6=*((C_word*)lf[44]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[45];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k5019 in for-each-loop565 in k4996 in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5021,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_5011(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k5040 in k4993 in k4990 in chicken.compiler.optimizer#scan-toplevel-assignments in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5042,c,av);}
/* optimizer.scm:130: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[36];
av2[3]=lf[46];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(52,c,9)))){
C_save_and_reclaim((void *)f_5046,c,av);}
a=C_alloc(52);
t2=C_mutate(&lf[50] /* (set! chicken.compiler.optimizer#simplifications ...) */,t1);
t3=lf[51] /* chicken.compiler.optimizer#simplified-ops */ =C_SCHEME_END_OF_LIST;;
t4=lf[52] /* chicken.compiler.optimizer#broken-constant-nodes */ =C_SCHEME_END_OF_LIST;;
t5=lf[53] /* chicken.compiler.optimizer#inline-history */ =C_SCHEME_END_OF_LIST;;
t6=C_mutate((C_word*)lf[54]+1 /* (set! chicken.compiler.optimizer#perform-high-level-optimizations ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5051,tmp=(C_word)a,a+=2,tmp));
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7732,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=C_a_i_list(&a,1,lf[192]);
t9=C_a_i_list(&a,2,lf[24],t8);
t10=C_a_i_cons(&a,2,lf[230],lf[231]);
t11=C_a_i_cons(&a,2,t9,t10);
t12=C_a_i_cons(&a,2,lf[219],t11);
t13=C_a_i_cons(&a,2,lf[30],t12);
t14=C_a_i_list(&a,4,lf[192],lf[230],lf[231],lf[219]);
t15=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_14092,tmp=(C_word)a,a+=2,tmp);
t16=C_a_i_list(&a,3,t13,t14,t15);
t17=C_a_i_list(&a,1,t16);
/* optimizer.scm:708: chicken.internal#hash-table-set! */
t18=*((C_word*)lf[161]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t18;
av2[1]=t7;
av2[2]=lf[50];
av2[3]=lf[30];
av2[4]=t17;
((C_proc)(void*)(*((C_word*)t18+1)))(5,av2);}}

/* chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5051(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7=av[7];
C_word t8=av[8];
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(116,c,9)))){
C_save_and_reclaim((void *)f_5051,c,av);}
a=C_alloc(116);
t9=C_fix(0);
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_fix(0);
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_fix(0);
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_END_OF_LIST;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_SCHEME_END_OF_LIST;
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_SCHEME_FALSE;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_SCHEME_UNDEFINED;
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_SCHEME_UNDEFINED;
t24=(*a=C_VECTOR_TYPE|1,a[1]=t23,tmp=(C_word)a,a+=2,tmp);
t25=C_SCHEME_UNDEFINED;
t26=(*a=C_VECTOR_TYPE|1,a[1]=t25,tmp=(C_word)a,a+=2,tmp);
t27=C_SCHEME_UNDEFINED;
t28=(*a=C_VECTOR_TYPE|1,a[1]=t27,tmp=(C_word)a,a+=2,tmp);
t29=C_SCHEME_UNDEFINED;
t30=(*a=C_VECTOR_TYPE|1,a[1]=t29,tmp=(C_word)a,a+=2,tmp);
t31=C_SCHEME_UNDEFINED;
t32=(*a=C_VECTOR_TYPE|1,a[1]=t31,tmp=(C_word)a,a+=2,tmp);
t33=C_SCHEME_UNDEFINED;
t34=(*a=C_VECTOR_TYPE|1,a[1]=t33,tmp=(C_word)a,a+=2,tmp);
t35=C_SCHEME_UNDEFINED;
t36=(*a=C_VECTOR_TYPE|1,a[1]=t35,tmp=(C_word)a,a+=2,tmp);
t37=C_SCHEME_UNDEFINED;
t38=(*a=C_VECTOR_TYPE|1,a[1]=t37,tmp=(C_word)a,a+=2,tmp);
t39=C_set_block_item(t22,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5054,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t40=C_set_block_item(t24,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5080,a[2]=t20,tmp=(C_word)a,a+=3,tmp));
t41=C_set_block_item(t26,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5084,tmp=(C_word)a,a+=2,tmp));
t42=C_set_block_item(t28,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5121,a[2]=t18,a[3]=t24,a[4]=t28,a[5]=t3,a[6]=t8,tmp=(C_word)a,a+=7,tmp));
t43=C_set_block_item(t30,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5249,a[2]=t24,a[3]=t22,a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t44=C_set_block_item(t32,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5411,a[2]=t20,a[3]=t28,a[4]=t12,a[5]=t24,a[6]=t32,a[7]=t30,a[8]=t36,tmp=(C_word)a,a+=9,tmp));
t45=C_set_block_item(t34,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5549,a[2]=t3,a[3]=t34,a[4]=t22,tmp=(C_word)a,a+=5,tmp));
t46=C_set_block_item(t36,0,(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_5569,a[2]=t24,a[3]=t22,a[4]=t14,a[5]=t34,a[6]=t10,a[7]=t32,a[8]=t38,a[9]=t3,a[10]=t26,a[11]=t16,a[12]=t5,a[13]=t6,a[14]=t7,a[15]=t4,tmp=(C_word)a,a+=16,tmp));
t47=C_set_block_item(t38,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7238,a[2]=t26,a[3]=t32,tmp=(C_word)a,a+=4,tmp));
t48=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7298,a[2]=t1,a[3]=t2,a[4]=t20,a[5]=t12,a[6]=t10,a[7]=t14,a[8]=t18,a[9]=t32,tmp=(C_word)a,a+=10,tmp);
t49=C_SCHEME_FALSE;
t50=(*a=C_VECTOR_TYPE|1,a[1]=t49,tmp=(C_word)a,a+=2,tmp);
t51=C_fix(0);
t52=(*a=C_VECTOR_TYPE|1,a[1]=t51,tmp=(C_word)a,a+=2,tmp);
t53=C_SCHEME_UNDEFINED;
t54=(*a=C_VECTOR_TYPE|1,a[1]=t53,tmp=(C_word)a,a+=2,tmp);
t55=C_SCHEME_UNDEFINED;
t56=(*a=C_VECTOR_TYPE|1,a[1]=t55,tmp=(C_word)a,a+=2,tmp);
t57=C_set_block_item(t54,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7467,a[2]=t50,tmp=(C_word)a,a+=3,tmp));
t58=C_set_block_item(t56,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7471,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t59=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7478,a[2]=t48,a[3]=t50,a[4]=t52,a[5]=t54,a[6]=t3,a[7]=t56,tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:660: chicken.compiler.support#debugging */
t60=*((C_word*)lf[35]+1);{
C_word *av2=av;
av2[0]=t60;
av2[1]=t59;
av2[2]=lf[48];
av2[3]=lf[158];
((C_proc)(void*)(*((C_word*)t60+1)))(4,av2);}}

/* test in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5054(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_5054,4,t0,t1,t2,t3);}
/* optimizer.scm:166: chicken.compiler.support#db-get */
t4=*((C_word*)lf[55]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k5066 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,2)))){
C_save_and_reclaim((void *)f_5068,c,av);}
a=C_alloc(36);
t2=C_eqp(lf[76],t1);
if(C_truep(t2)){
t3=((C_word*)((C_word*)t0)[2])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=(
/* optimizer.scm:235: touch */
  f_5080(((C_word*)((C_word*)t0)[3])[1])
);
t6=C_u_i_car(((C_word*)t0)[4]);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5078,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:168: chicken.compiler.support#node-parameters */
t8=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
/* optimizer.scm:226: simplify */
t3=((C_word*)((C_word*)t0)[9])[1];
f_5121(t3,((C_word*)t0)[10],((C_word*)t0)[11]);}}

/* k5076 in k5066 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5078(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5078,c,av);}
t2=C_i_car(t1);
t3=(C_truep(t2)?C_i_cadr(((C_word*)t0)[2]):C_i_caddr(((C_word*)t0)[2]));
/* optimizer.scm:236: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_5411(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* touch in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_5080(C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_stack_overflow_check;{}
t1=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
return(t1);}

/* invalidate-gae! in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5084(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(2,0,2)))){
C_save_and_reclaim_args((void *)trf_5084,2,t1,t2);}
a=C_alloc(2);
t3=C_i_check_list_2(t2,lf[23]);
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_5098,tmp=(C_word)a,a+=2,tmp);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(
  f_5098(t2)
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* for-each-loop693 in invalidate-gae! in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_5098(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_slot(t1,C_fix(0));
t3=C_i_set_cdr(t2,C_SCHEME_FALSE);
t5=C_slot(t1,C_fix(1));
t1=t5;
goto loop;}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5121(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_5121,3,t0,t1,t2);}
a=C_alloc(12);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5125,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5247,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:176: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_5125,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5128,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5136,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:177: any */
f_3804(t2,t3,t1);}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5126 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5128,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5136,c,av);}
a=C_alloc(10);
t3=C_i_cadr(t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5146,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t1,a[7]=t3,a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:179: chicken.compiler.support#match-node */
t5=*((C_word*)lf[56]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=C_i_car(t2);
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k5144 in a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5146(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_5146,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5152,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_i_caddr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5191,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t9=C_i_check_list_2(((C_word*)t0)[7],lf[8]);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5205,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5207,a[2]=t8,a[3]=t6,a[4]=t12,a[5]=t7,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_5207(t14,t10,((C_word*)t0)[7]);}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5150 in k5144 in a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5152(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_5152,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_i_caar(((C_word*)t0)[2]);
t3=C_i_assq(t2,((C_word*)((C_word*)t0)[3])[1]);
if(C_truep(t3)){
t4=C_i_cdr(t3);
t5=C_s_a_i_plus(&a,2,t4,C_fix(1));
t6=C_i_set_cdr(t3,t5);
t7=(
/* optimizer.scm:187: touch */
  f_5080(((C_word*)((C_word*)t0)[4])[1])
);
/* optimizer.scm:188: simplify */
t8=((C_word*)((C_word*)t0)[5])[1];
f_5121(t8,((C_word*)t0)[6],t1);}
else{
t4=((C_word*)((C_word*)t0)[3])[1];
t5=C_a_i_cons(&a,2,t2,C_fix(1));
t6=C_a_i_cons(&a,2,t5,t4);
t7=C_mutate(((C_word *)((C_word*)t0)[3])+1,t6);
t8=(
/* optimizer.scm:187: touch */
  f_5080(((C_word*)((C_word*)t0)[4])[1])
);
/* optimizer.scm:188: simplify */
t9=((C_word*)((C_word*)t0)[5])[1];
f_5121(t9,((C_word*)t0)[6],t1);}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g733 in k5144 in a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_5191(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_i_cdr(C_i_assq(t1,((C_word*)t0)[2])));}

/* k5203 in k5144 in a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_5205,c,av);}{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=t1;
C_apply(6,av2);}}

/* map-loop727 in k5144 in a5135 in k5123 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5207(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_5207,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=(
/* optimizer.scm:181: g733 */
  f_5191(((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5245 in simplify in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5247(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5247,c,av);}
/* optimizer.scm:175: chicken.internal#hash-table-ref */
t2=*((C_word*)lf[57]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[50];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5249(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_5249,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5409,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:197: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k5254 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5256,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?t1:((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5268(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_5268,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5380,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:205: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5304(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_5304,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_i_car(t1);
if(C_truep(t2)){
t3=C_u_i_cdr(t1);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5360,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t3,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:213: test */
t5=((C_word*)((C_word*)t0)[8])[1];
f_5054(t5,t4,((C_word*)t0)[6],lf[62]);}
else{
t4=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5324 in k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_5326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_5326,c,av);}
a=C_alloc(9);
t2=(
/* optimizer.scm:217: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5336,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5348,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:219: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k5334 in k5324 in k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_5336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5336,c,av);}
/* optimizer.scm:218: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k5346 in k5324 in k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_5348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_5348,c,av);}
a=C_alloc(6);
t2=C_i_cdr(t1);
/* optimizer.scm:219: cons* */
f_3834(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,((C_word*)t0)[4],t2));}

/* k5350 in k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_5352(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_5352,c,av);}
/* optimizer.scm:215: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[59];
av2[3]=lf[60];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
av2[6]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k5354 in k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_5356(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5356,c,av);}
/* optimizer.scm:216: chicken.compiler.support#call-info */
t2=*((C_word*)lf[61]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k5358 in k5302 in k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_5360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_5360,c,av);}
a=C_alloc(17);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5326,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5352,a[2]=t2,a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5356,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:216: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t2=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5362 in k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5364(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_5364,c,av);}
a=C_alloc(9);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5304,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:210: chicken.compiler.support#db-get */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[63];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5366 in k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5368(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5368,c,av);}
a=C_alloc(10);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5364,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:209: chicken.compiler.support#db-get */
t3=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[64];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k5370 in k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5372,c,av);}
a=C_alloc(10);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5368,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:208: chicken.compiler.support#db-get */
t4=*((C_word*)lf[55]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=t2;
av2[4]=lf[65];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5374 in k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_5376,c,av);}
a=C_alloc(9);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5372,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:207: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5378 in k5266 in k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_5380,c,av);}
a=C_alloc(10);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5376,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t2,tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:206: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5403 in k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_5405,c,av);}
a=C_alloc(8);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5268,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep((C_truep(C_i_equalp(t2,lf[66]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(t2,lf[67]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
t4=t3;
f_5268(t4,lf[68]);}
else{
if(C_truep((C_truep(C_i_equalp(t2,lf[69]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(t2,lf[70]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))){
t4=t3;
f_5268(t4,lf[71]);}
else{
if(C_truep((C_truep(C_i_equalp(t2,lf[72]))?C_SCHEME_TRUE:C_SCHEME_FALSE))){
t4=t3;
f_5268(t4,lf[73]);}
else{
t4=C_i_member(t2,lf[74]);
t5=t3;
f_5268(t5,(C_truep(t4)?lf[75]:C_SCHEME_FALSE));}}}}
else{
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5407 in maybe-replace-rest-arg-calls in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_5409,c,av);}
a=C_alloc(12);
t2=C_eqp(lf[58],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5256,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t2)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5405,a[2]=((C_word*)t0)[4],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:198: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5411(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_5411,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
if(C_truep(C_i_memq(t2,lf[52]))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=((C_word*)((C_word*)t0)[2])[1];
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5425,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t3,a[8]=t4,a[9]=((C_word*)t0)[7],a[10]=t5,a[11]=((C_word*)t0)[2],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:228: walk1 */
t7=((C_word*)((C_word*)t0)[8])[1];
f_5569(t7,t6,t2,t3,t4);}}

/* k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_5425,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_5428,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:229: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_5428,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_5431,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* optimizer.scm:230: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,5)))){
C_save_and_reclaim((void *)f_5431,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5434,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(t1,lf[25]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5068,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=t2,a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:167: chicken.compiler.support#node-class */
t6=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=C_eqp(t1,lf[58]);
if(C_truep(t4)){
/* optimizer.scm:243: maybe-replace-rest-arg-calls */
t5=((C_word*)((C_word*)t0)[11])[1];
f_5249(t5,t2,((C_word*)t0)[10]);}
else{
t5=C_eqp(t1,lf[30]);
if(C_truep(t5)){
t6=C_i_car(((C_word*)t0)[4]);
t7=C_i_cddr(((C_word*)t0)[4]);
t8=C_a_i_cons(&a,2,t6,t7);
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5494,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:246: chicken.compiler.support#maybe-constant-fold-call */
t10=*((C_word*)lf[78]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=t8;
av2[4]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
/* optimizer.scm:226: simplify */
t6=((C_word*)((C_word*)t0)[2])[1];
f_5121(t6,((C_word*)t0)[3],((C_word*)t0)[10]);}}}}

/* k5432 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5434,c,av);}
/* optimizer.scm:226: simplify */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5121(t2,((C_word*)t0)[3],t1);}

/* a5493 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_5494,c,av);}
a=C_alloc(4);
if(C_truep(C_i_not(t2))){
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5507,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[3])){
t6=t5;
f_5507(t6,C_SCHEME_UNDEFINED);}
else{
t6=C_set_block_item(((C_word*)t0)[4],0,C_SCHEME_FALSE);
t7=t5;
f_5507(t7,t6);}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=(
/* optimizer.scm:257: touch */
  f_5080(((C_word*)((C_word*)t0)[5])[1])
);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5518,a[2]=((C_word*)t0)[6],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:259: chicken.compiler.support#qnode */
t7=*((C_word*)lf[77]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k5505 in a5493 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5507(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_5507,2,t0,t1);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5511,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=lf[52];
t4=C_a_i_list(&a,1,((C_word*)t0)[3]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4235,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t8=((C_word*)t6)[1];
f_4235(t8,t2,t4,lf[52]);}

/* k5509 in k5505 in a5493 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5511,c,av);}
t2=C_mutate(&lf[52] /* (set! chicken.compiler.optimizer#broken-constant-nodes ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k5516 in a5493 in k5429 in k5426 in k5423 in walk in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5518(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_5518,c,av);}
a=C_alloc(9);
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_a_i_list2(&a,2,t3,t1);
/* optimizer.scm:260: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=t2;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* replace-var in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5549(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_5549,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5553,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:267: test */
t4=((C_word*)((C_word*)t0)[4])[1];
f_5054(t4,t3,t2,lf[80]);}

/* k5551 in replace-var in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_5553,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5557,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:267: g827 */
t3=t2;
f_5557(t3,((C_word*)t0)[5],t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g827 in k5551 in replace-var in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5557(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_5557,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5561,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:269: replace-var */
t4=((C_word*)((C_word*)t0)[4])[1];
f_5549(t4,t3,t2);}

/* k5559 in g827 in k5551 in replace-var in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_5561,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5564,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:271: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[79]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[80];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k5562 in k5559 in g827 in k5551 in replace-var in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5564,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5569(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,2)))){
C_save_and_reclaim_args((void *)trf_5569,5,t0,t1,t2,t3,t4);}
a=C_alloc(20);
t5=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_5573,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=t3,a[11]=((C_word*)t0)[8],a[12]=t2,a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[10],a[15]=((C_word*)t0)[11],a[16]=((C_word*)t0)[12],a[17]=((C_word*)t0)[13],a[18]=((C_word*)t0)[14],a[19]=((C_word*)t0)[15],tmp=(C_word)a,a+=20,tmp);
/* optimizer.scm:276: chicken.compiler.support#node-subexpressions */
t6=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5573(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_5573,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_5576,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
/* optimizer.scm:277: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_5576,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_5579,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
/* optimizer.scm:278: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[13];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,8)))){
C_save_and_reclaim((void *)f_5579,c,av);}
a=C_alloc(20);
t2=C_eqp(t1,lf[24]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5588,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:282: replace-var */
t4=((C_word*)((C_word*)t0)[8])[1];
f_5549(t4,t3,C_i_car(((C_word*)t0)[5]));}
else{
t3=C_eqp(t1,lf[26]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[5]);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5701,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[5],a[10]=t4,a[11]=((C_word*)t0)[4],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:303: test */
t6=((C_word*)((C_word*)t0)[4])[1];
f_5054(t6,t5,t4,lf[92]);}
else{
t4=C_eqp(t1,lf[29]);
if(C_truep(t4)){
t5=C_i_caddr(((C_word*)t0)[5]);
t6=C_i_car(((C_word*)t0)[5]);
t7=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_5846,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[11],a[6]=t6,a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[3],a[10]=t5,a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=t1,tmp=(C_word)a,a+=14,tmp);
/* optimizer.scm:321: test */
t8=((C_word*)((C_word*)t0)[4])[1];
f_5054(t8,t7,t6,lf[100]);}
else{
t5=C_eqp(t1,lf[101]);
if(C_truep(t5)){
/* optimizer.scm:354: walk-generic */
t6=((C_word*)((C_word*)t0)[13])[1];
f_7238(t6,((C_word*)t0)[3],((C_word*)t0)[14],t1,((C_word*)t0)[5],((C_word*)t0)[11],((C_word*)t0)[12],C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}
else{
t6=C_eqp(t1,lf[30]);
if(C_truep(t6)){
t7=C_i_car(((C_word*)t0)[11]);
t8=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_6019,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[15],a[5]=((C_word*)t0)[12],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[14],a[11]=t1,a[12]=((C_word*)t0)[5],a[13]=t7,a[14]=((C_word*)t0)[16],a[15]=((C_word*)t0)[4],a[16]=((C_word*)t0)[17],a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[19],a[19]=((C_word*)t0)[20],tmp=(C_word)a,a+=20,tmp);
/* optimizer.scm:358: chicken.compiler.support#node-class */
t9=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=t7;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t7=C_eqp(t1,lf[31]);
if(C_truep(t7)){
t8=C_i_car(((C_word*)t0)[5]);
t9=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_6990,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t8,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[15],a[12]=((C_word*)t0)[21],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:545: test */
t10=((C_word*)((C_word*)t0)[4])[1];
f_5054(t10,t9,t8,lf[91]);}
else{
t8=C_eqp(t1,lf[71]);
t9=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_7173,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[13],a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[4],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t8)){
t10=t9;
f_7173(t10,t8);}
else{
t10=C_eqp(t1,lf[68]);
if(C_truep(t10)){
t11=t9;
f_7173(t11,t10);}
else{
t11=C_eqp(t1,lf[73]);
t12=t9;
f_7173(t12,(C_truep(t11)?t11:C_eqp(t1,lf[75])));}}}}}}}}}

/* k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5588(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_5588,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5594,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:283: test */
t3=((C_word*)((C_word*)t0)[4])[1];
f_5054(t3,t2,t1,lf[88]);}

/* k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_5594,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=(
/* optimizer.scm:284: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5600,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:285: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[36];
av2[3]=lf[82];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=C_i_car(((C_word*)t0)[6]);
t3=C_eqp(((C_word*)t0)[5],t2);
if(C_truep(C_i_not(t3))){
t4=(
/* optimizer.scm:288: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t5=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[7])[1],C_fix(1));
t6=C_mutate(((C_word *)((C_word*)t0)[7])+1,t5);
/* optimizer.scm:290: chicken.compiler.support#varnode */
t7=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t4=C_i_assq(((C_word*)t0)[5],((C_word*)t0)[8]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5638,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:283: g855 */
t6=t5;
f_5638(t6,((C_word*)t0)[3],t4);}
else{
/* optimizer.scm:299: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}}

/* k5598 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_5600,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5611,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5615,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:286: test */
t4=((C_word*)((C_word*)t0)[3])[1];
f_5054(t4,t3,((C_word*)t0)[4],lf[81]);}

/* k5609 in k5598 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5611(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5611,c,av);}
/* optimizer.scm:286: chicken.compiler.support#qnode */
t2=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k5613 in k5598 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5615,c,av);}
/* optimizer.scm:286: chicken.compiler.support#node-parameters */
t2=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* g855 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5638(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_5638,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5648,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t3)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5668,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:295: chicken.compiler.support#variable-mark */
t6=*((C_word*)lf[86]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
av2[3]=lf[87];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=t4;
f_5648(t5,C_SCHEME_FALSE);}}

/* k5646 in g855 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5648(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,5)))){
C_save_and_reclaim_args((void *)trf_5648,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5651,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:296: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[84];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
/* optimizer.scm:298: chicken.compiler.support#varnode */
t2=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* k5649 in k5646 in g855 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_5651,c,av);}
/* optimizer.scm:297: chicken.compiler.support#varnode */
t2=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k5666 in g855 in k5592 in k5586 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5668,c,av);}
t2=((C_word*)t0)[2];
f_5648(t2,C_i_not(C_eqp(lf[85],t1)));}

/* k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5701(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_5701,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5704,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t1)){
t3=t2;
f_5704(t3,t1);}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5821,a[2]=t2,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[10],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:304: test */
t4=((C_word*)((C_word*)t0)[11])[1];
f_5054(t4,t3,((C_word*)t0)[10],lf[91]);}}

/* k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5704(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,4)))){
C_save_and_reclaim_args((void *)trf_5704,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=(
/* optimizer.scm:306: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
/* optimizer.scm:308: walk */
t5=((C_word*)((C_word*)t0)[4])[1];
f_5411(t5,((C_word*)t0)[5],C_i_cadr(((C_word*)t0)[6]),((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_5719,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[9],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5774,a[2]=((C_word*)t0)[10],a[3]=t2,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5811,a[2]=((C_word*)t0)[11],a[3]=t3,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:310: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k5717 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5719(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,0,3)))){
C_save_and_reclaim_args((void *)trf_5719,2,t0,t1);}
a=C_alloc(22);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5727,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t7=C_i_check_list_2(((C_word*)t0)[4],lf[8]);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5737,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5739,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_5739(t12,t8,((C_word*)t0)[4]);}

/* g886 in k5717 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5727(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_5727,3,t0,t1,t2);}
/* optimizer.scm:316: g903 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5411(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k5735 in k5717 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_5737,c,av);}
/* optimizer.scm:316: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop880 in k5717 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5739(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_5739,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5764,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:316: g886 */
t4=((C_word*)t0)[4];
f_5727(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k5762 in map-loop880 in k5717 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_5764,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_5739(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k5772 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_5774,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5785,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:313: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];
f_5719(t2,((C_word*)t0)[4]);}}

/* k5783 in k5772 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_5785,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
t4=((C_word*)t0)[3];
f_5719(t4,C_a_i_cons(&a,2,t3,((C_word*)t0)[4]));}

/* k5801 in k5809 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_5803,c,av);}
/* optimizer.scm:311: test */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5054(t2,((C_word*)t0)[3],C_i_car(t1),lf[89]);}

/* k5809 in k5702 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_5811,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5803,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:311: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_5774(2,av2);}}}

/* k5819 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_5821,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5828,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:305: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_5054(t3,t2,((C_word*)t0)[4],lf[90]);}
else{
t2=((C_word*)t0)[2];
f_5704(t2,C_SCHEME_FALSE);}}

/* k5826 in k5819 in k5699 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_5828,c,av);}
t2=((C_word*)t0)[2];
f_5704(t2,C_i_not(t1));}

/* k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_5846,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_5851,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:322: ##sys#decompose-lambda-list */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[9];
av2[2]=((C_word*)t0)[10];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_5938,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:338: test */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5054(t3,t2,((C_word*)t0)[6],lf[96]);}}

/* a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_5851,c,av);}
a=C_alloc(15);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5857,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_5869,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t3,a[9]=t4,a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:325: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}

/* a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_5857,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_5863,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_3407,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_3407(t6,t1,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST,((C_word*)t0)[3]);}

/* a5862 in a5856 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_5863(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_5863,3,t0,t1,t2);}
/* optimizer.scm:325: test */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5054(t3,t1,t2,lf[93]);}

/* a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_5869,c,av);}
a=C_alloc(12);
t4=(
/* optimizer.scm:326: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_5876,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t3,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:327: chicken.compiler.support#debugging */
t6=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[36];
av2[3]=lf[97];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k5874 in a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_5876,c,av);}
a=C_alloc(16);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5911,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_5918,a[2]=((C_word*)t0)[8],a[3]=t4,a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[10])){
/* optimizer.scm:331: test */
t6=((C_word*)((C_word*)t0)[11])[1];
f_5054(t6,t5,((C_word*)t0)[5],lf[96]);}
else{
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_5918(2,av2);}}}

/* k5889 in k5909 in k5874 in a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_5891,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:328: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[29];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k5909 in k5874 in a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_5911,c,av);}
a=C_alloc(19);
t2=C_i_cadddr(((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,((C_word*)t0)[3],((C_word*)t0)[4],t1,t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5891,a[2]=((C_word*)t0)[5],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_i_car(((C_word*)t0)[6]);
t6=C_a_i_cons(&a,2,((C_word*)t0)[7],((C_word*)t0)[8]);
/* optimizer.scm:337: walk */
t7=((C_word*)((C_word*)t0)[9])[1];
f_5411(t7,t4,t5,t6,C_SCHEME_END_OF_LIST);}

/* k5916 in k5874 in a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_5918,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_5921,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:332: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[95];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* optimizer.scm:335: chicken.compiler.support#build-lambda-list */
t2=*((C_word*)lf[94]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k5919 in k5916 in k5874 in a5868 in a5850 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_5921,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],C_fix(1));
/* optimizer.scm:334: chicken.compiler.support#build-lambda-list */
t3=*((C_word*)lf[94]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k5936 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,8)))){
C_save_and_reclaim((void *)f_5938,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_5943,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:339: ##sys#decompose-lambda-list */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[8];
av2[2]=((C_word*)t0)[9];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[5],((C_word*)t0)[6]);
/* optimizer.scm:351: walk-generic */
t3=((C_word*)((C_word*)t0)[10])[1];
f_7238(t3,((C_word*)t0)[8],((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[3],((C_word*)t0)[4],t2,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}}

/* a5942 in k5936 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_5943,c,av);}
a=C_alloc(10);
t5=(
/* optimizer.scm:342: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5950,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t3,a[9]=t2,tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:343: chicken.compiler.support#debugging */
t7=*((C_word*)lf[35]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[36];
av2[3]=lf[99];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k5948 in a5942 in k5936 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5950(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,4)))){
C_save_and_reclaim((void *)f_5950,c,av);}
a=C_alloc(39);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_5985,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
t5=C_s_a_i_plus(&a,2,((C_word*)t0)[8],C_fix(1));
/* optimizer.scm:348: chicken.compiler.support#build-lambda-list */
t6=*((C_word*)lf[94]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=t5;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k5963 in k5983 in k5948 in a5942 in k5936 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5965(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_5965,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:344: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[29];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k5983 in k5948 in a5942 in k5936 in k5844 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_5985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_5985,c,av);}
a=C_alloc(19);
t2=C_i_cadddr(((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,((C_word*)t0)[3],((C_word*)t0)[4],t1,t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_5965,a[2]=((C_word*)t0)[5],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_i_car(((C_word*)t0)[6]);
t6=C_a_i_cons(&a,2,((C_word*)t0)[7],((C_word*)t0)[8]);
/* optimizer.scm:350: walk */
t7=((C_word*)((C_word*)t0)[9])[1];
f_5411(t7,t4,t5,t6,C_SCHEME_END_OF_LIST);}

/* k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,8)))){
C_save_and_reclaim((void *)f_6019,c,av);}
a=C_alloc(30);
t2=C_eqp(t1,lf[24]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_6893,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],tmp=(C_word)a,a+=20,tmp);
/* optimizer.scm:362: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[13];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_eqp(t1,lf[29]);
if(C_truep(t3)){
if(C_truep(C_i_car(((C_word*)t0)[12]))){
/* optimizer.scm:536: walk-generic */
t4=((C_word*)((C_word*)t0)[9])[1];
f_7238(t4,((C_word*)t0)[7],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[2],((C_word*)t0)[5],((C_word*)t0)[8],C_SCHEME_FALSE);}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6911,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
t5=C_i_cdr(((C_word*)t0)[12]);
t6=C_a_i_cons(&a,2,C_SCHEME_TRUE,t5);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6923,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6930,a[2]=t4,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6932,a[2]=t9,a[3]=t14,a[4]=t11,a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_6932(t16,t12,((C_word*)t0)[2]);}}
else{
/* optimizer.scm:541: walk-generic */
t4=((C_word*)((C_word*)t0)[9])[1];
f_7238(t4,((C_word*)t0)[7],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[2],((C_word*)t0)[5],((C_word*)t0)[8],C_SCHEME_TRUE);}}}

/* k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6031(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_6031,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_6034,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6889,a[2]=t2,a[3]=((C_word*)t0)[16],a[4]=((C_word*)t0)[13],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:364: test */
t4=((C_word*)((C_word*)t0)[16])[1];
f_5054(t4,t3,((C_word*)t0)[13],lf[131]);}

/* k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,3)))){
C_save_and_reclaim((void *)f_6034,c,av);}
a=C_alloc(30);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_6041,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t2,a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[2],a[16]=((C_word*)t0)[14],a[17]=((C_word*)t0)[15],a[18]=((C_word*)t0)[16],a[19]=((C_word*)t0)[17],a[20]=((C_word*)t0)[18],a[21]=((C_word*)t0)[19],a[22]=((C_word*)t0)[20],a[23]=((C_word*)t0)[21],tmp=(C_word)a,a+=24,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6848,a[2]=t3,a[3]=((C_word*)t0)[17],a[4]=t1,a[5]=((C_word*)t0)[14],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:368: test */
t5=((C_word*)((C_word*)t0)[17])[1];
f_5054(t5,t4,((C_word*)t0)[14],lf[91]);}

/* k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_6041(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,0,3)))){
C_save_and_reclaim_args((void *)trf_6041,2,t0,t1);}
a=C_alloc(24);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_6044,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],tmp=(C_word)a,a+=17,tmp);
/* optimizer.scm:373: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_6120,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[12],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[14],a[11]=((C_word*)t0)[15],a[12]=((C_word*)t0)[7],a[13]=((C_word*)t0)[2],a[14]=((C_word*)t0)[5],a[15]=((C_word*)t0)[9],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],tmp=(C_word)a,a+=24,tmp);
/* optimizer.scm:392: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[86]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[16];
av2[3]=lf[132];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_6044,c,av);}
a=C_alloc(17);
t2=C_i_caddr(t1);
t3=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_6053,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t2,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],tmp=(C_word)a,a+=17,tmp);
/* optimizer.scm:375: chicken.compiler.support#check-signature */
t4=*((C_word*)lf[108]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[16];
av2[3]=((C_word*)t0)[8];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_6053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_6053,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6056,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:376: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[105];
av2[4]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6111,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[14],a[6]=((C_word*)t0)[15],a[7]=((C_word*)t0)[16],a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:387: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[106];
av2[3]=lf[107];
av2[4]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_6056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_6056,c,av);}
a=C_alloc(19);
t2=(
/* optimizer.scm:377: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6060,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=C_i_check_list_2(((C_word*)t0)[4],lf[23]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6070,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6087,a[2]=t7,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_6087(t9,t5,((C_word*)t0)[4]);}

/* g968 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_fcall f_6060(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_6060,3,t0,t1,t2);}
t3=*((C_word*)lf[79]+1);
/* optimizer.scm:378: g983 */
t4=*((C_word*)lf[79]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[102];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k6068 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_6070,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6077,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6085,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:382: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k6075 in k6068 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6077,c,av);}
/* optimizer.scm:380: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5411(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6083 in k6068 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6085,c,av);}
/* optimizer.scm:381: chicken.compiler.support#inline-lambda-bindings */
t2=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=C_i_car(t1);
av2[5]=C_SCHEME_FALSE;
av2[6]=((C_word*)t0)[5];
av2[7]=*((C_word*)lf[104]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(8,av2);}}

/* for-each-loop967 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_fcall f_6087(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6087,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6097,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:378: g968 */
t4=((C_word*)t0)[3];
f_6060(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k6095 in for-each-loop967 in k6054 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6097,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6087(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k6109 in k6051 in k6042 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_6111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_6111,c,av);}
/* optimizer.scm:391: walk-generic */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7238(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_TRUE);}

/* k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,2)))){
C_save_and_reclaim((void *)f_6120,c,av);}
a=C_alloc(30);
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_6123,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],tmp=(C_word)a,a+=24,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6841,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[19],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:393: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;
f_6123(t3,C_SCHEME_FALSE);}}

/* k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_fcall f_6123(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,0,4)))){
C_save_and_reclaim_args((void *)trf_6123,2,t0,t1);}
a=C_alloc(27);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6126,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:405: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[109];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_6151,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[4],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[2],a[14]=((C_word*)t0)[15],a[15]=((C_word*)t0)[16],a[16]=((C_word*)t0)[17],a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[19],a[19]=((C_word*)t0)[20],a[20]=((C_word*)t0)[21],a[21]=((C_word*)t0)[22],a[22]=((C_word*)t0)[23],a[23]=((C_word*)t0)[3],tmp=(C_word)a,a+=24,tmp);
if(C_truep(((C_word*)t0)[15])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6734,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:414: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[15];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;
f_6151(t3,C_SCHEME_FALSE);}}}

/* k6124 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_6126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_6126,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=C_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6145,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:412: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k6143 in k6124 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_6145,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:409: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_fcall f_6151(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,0,2)))){
C_save_and_reclaim_args((void *)trf_6151,2,t0,t1);}
a=C_alloc(24);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|23,a[1]=(C_word)f_6154,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],tmp=(C_word)a,a+=24,tmp);
/* optimizer.scm:416: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[14];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_6679,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[23],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[4],a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[6],tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)t0)[14])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6727,a[2]=t2,a[3]=((C_word*)t0)[14],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:522: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[14];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_6679(2,av2);}}}}

/* k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,5)))){
C_save_and_reclaim((void *)f_6154,c,av);}
a=C_alloc(25);
t2=C_i_caddr(t1);
t3=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_6162,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=t2,a[15]=((C_word*)t0)[13],a[16]=((C_word*)t0)[14],a[17]=((C_word*)t0)[15],a[18]=((C_word*)t0)[16],a[19]=((C_word*)t0)[17],a[20]=((C_word*)t0)[18],a[21]=((C_word*)t0)[19],a[22]=((C_word*)t0)[20],a[23]=((C_word*)t0)[21],a[24]=((C_word*)t0)[22],tmp=(C_word)a,a+=25,tmp);
/* optimizer.scm:418: ##sys#decompose-lambda-list */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[23];
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6162(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,3)))){
C_save_and_reclaim((void *)f_6162,c,av);}
a=C_alloc(32);
t5=C_i_car(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|28,a[1]=(C_word)f_6169,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=t5,a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=t1,a[18]=((C_word*)t0)[2],a[19]=((C_word*)t0)[17],a[20]=t3,a[21]=((C_word*)t0)[18],a[22]=((C_word*)t0)[19],a[23]=((C_word*)t0)[20],a[24]=t2,a[25]=((C_word*)t0)[21],a[26]=((C_word*)t0)[22],a[27]=((C_word*)t0)[23],a[28]=((C_word*)t0)[24],tmp=(C_word)a,a+=29,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6673,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:422: chicken.compiler.support#variable-mark */
t8=*((C_word*)lf[86]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[17];
av2[3]=lf[127];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}

/* k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_6169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,3)))){
C_save_and_reclaim((void *)f_6169,c,av);}
a=C_alloc(39);
t2=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_6175,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=t1,a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],tmp=(C_word)a,a+=27,tmp);
if(C_truep(((C_word*)t0)[26])){
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6610,a[2]=t1,a[3]=((C_word*)t0)[18],a[4]=((C_word*)t0)[27],a[5]=t2,a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[28],a[10]=((C_word*)t0)[19],a[11]=((C_word*)t0)[23],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:424: test */
t4=((C_word*)((C_word*)t0)[23])[1];
f_5054(t4,t3,((C_word*)t0)[19],lf[125]);}
else{
t3=t2;
f_6175(t3,C_SCHEME_FALSE);}}

/* k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_fcall f_6175(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,0,4)))){
C_save_and_reclaim_args((void *)trf_6175,2,t0,t1);}
a=C_alloc(23);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_6181,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],tmp=(C_word)a,a+=20,tmp);
/* optimizer.scm:436: chicken.compiler.support#check-signature */
t3=*((C_word*)lf[108]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[20];
av2[3]=((C_word*)t0)[15];
av2[4]=((C_word*)t0)[14];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_6285,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[21],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[17],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[22],a[15]=((C_word*)t0)[23],a[16]=((C_word*)t0)[2],a[17]=((C_word*)t0)[10],a[18]=((C_word*)t0)[24],a[19]=((C_word*)t0)[25],a[20]=((C_word*)t0)[26],a[21]=((C_word*)t0)[14],a[22]=((C_word*)t0)[11],tmp=(C_word)a,a+=23,tmp);
/* optimizer.scm:466: test */
t3=((C_word*)((C_word*)t0)[24])[1];
f_5054(t3,t2,((C_word*)t0)[11],lf[100]);}}

/* k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_6181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,6)))){
C_save_and_reclaim((void *)f_6181,c,av);}
a=C_alloc(18);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_6184,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
t3=(C_truep(((C_word*)t0)[18])?lf[113]:lf[114]);
/* optimizer.scm:437: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[106];
av2[3]=t3;
av2[4]=((C_word*)t0)[10];
av2[5]=((C_word*)t0)[11];
av2[6]=C_i_cadddr(((C_word*)t0)[19]);
((C_proc)(void*)(*((C_word*)t4+1)))(7,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6276,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[17],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:461: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[106];
av2[3]=lf[115];
av2[4]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_6184,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6185,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_i_check_list_2(((C_word*)t0)[3],lf[23]);
t4=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_6195,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],a[15]=((C_word*)t0)[2],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6244,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_6244(t8,t4,((C_word*)t0)[3]);}

/* g1061 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_fcall f_6185(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_6185,3,t0,t1,t2);}
t3=*((C_word*)lf[79]+1);
/* optimizer.scm:442: g1076 */
t4=*((C_word*)lf[79]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[102];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_6195,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_6198,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],tmp=(C_word)a,a+=18,tmp);
/* optimizer.scm:444: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[112];
av2[4]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6198(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_6198,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_6203,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],tmp=(C_word)a,a+=17,tmp);
/* optimizer.scm:445: chicken.base#call/cc */
t3=*((C_word*)lf[111]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[17];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6203(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_6203,c,av);}
a=C_alloc(26);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6206,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6220,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[12],a[6]=t1,a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6242,a[2]=t4,a[3]=((C_word*)t0)[13],a[4]=((C_word*)t0)[14],a[5]=((C_word*)t0)[15],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:454: chicken.compiler.support#node-subexpressions */
t6=*((C_word*)lf[42]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[16];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* cfk in a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_6206,c,av);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6210,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:448: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[106];
av2[3]=lf[110];
av2[4]=((C_word*)t0)[10];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k6208 in cfk in a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in ... */
static void C_ccall f_6210(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,8)))){
C_save_and_reclaim((void *)f_6210,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6217,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:452: walk-generic */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7238(t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_TRUE);}

/* k6215 in k6208 in cfk in a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in ... */
static void C_ccall f_6217(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6217,c,av);}
/* optimizer.scm:452: return */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k6218 in a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_6220,c,av);}
a=C_alloc(6);
t2=C_i_car(((C_word*)t0)[2]);
t3=lf[53];
t4=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
t5=C_a_i_cons(&a,2,t4,lf[53]);
t6=C_mutate(&lf[53] /* (set! chicken.compiler.optimizer#inline-history ...) */,t5);
t7=(
/* optimizer.scm:458: touch */
  f_5080(((C_word*)((C_word*)t0)[4])[1])
);
/* optimizer.scm:459: walk */
t8=((C_word*)((C_word*)t0)[5])[1];
f_5411(t8,((C_word*)t0)[6],t1,((C_word*)t0)[2],((C_word*)t0)[7]);}

/* k6240 in a6202 in k6196 in k6193 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,7)))){
C_save_and_reclaim((void *)f_6242,c,av);}
/* optimizer.scm:453: chicken.compiler.support#inline-lambda-bindings */
t2=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=C_i_car(t1);
av2[5]=C_SCHEME_TRUE;
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t2+1)))(8,av2);}}

/* for-each-loop1060 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_fcall f_6244(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6244,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6254,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:442: g1061 */
t4=((C_word*)t0)[3];
f_6185(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k6252 in for-each-loop1060 in k6182 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6254,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_6244(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k6274 in k6179 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6276(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_6276,c,av);}
/* optimizer.scm:465: walk-generic */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7238(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_TRUE);}

/* k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_6285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,8)))){
C_save_and_reclaim((void *)f_6285,c,av);}
a=C_alloc(17);
if(C_truep(t1)){
t2=C_i_length(((C_word*)t0)[2]);
if(C_truep(C_i_lessp(t2,((C_word*)t0)[3]))){
/* optimizer.scm:468: walk-generic */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7238(t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_TRUE);}
else{
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_6299,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[13],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[14],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[15],a[9]=t4,a[10]=((C_word*)t0)[16],a[11]=((C_word*)t0)[17],a[12]=((C_word*)t0)[18],tmp=(C_word)a,a+=13,tmp));
t6=((C_word*)t4)[1];
f_6299(t6,((C_word*)t0)[5],((C_word*)t0)[19],((C_word*)t0)[3],((C_word*)t0)[2],C_SCHEME_END_OF_LIST);}}
else{
t2=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_6597,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[20],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[15],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[17],a[16]=((C_word*)t0)[21],tmp=(C_word)a,a+=17,tmp);
/* optimizer.scm:495: test */
t3=((C_word*)((C_word*)t0)[18])[1];
f_5054(t3,t2,((C_word*)t0)[22],lf[96]);}}

/* loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_fcall f_6299(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,0,3)))){
C_save_and_reclaim_args((void *)trf_6299,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(24);
t6=C_i_nullp(t2);
t7=(C_truep(t6)?t6:C_i_zerop(t3));
if(C_truep(t7)){
t8=(
/* optimizer.scm:471: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t9=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t10=t9;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=((C_word*)t11)[1];
t13=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6316,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t14=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6370,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[5],a[7]=t11,a[8]=t13,a[9]=t12,tmp=(C_word)a,a+=10,tmp);
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6374,a[2]=t14,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:476: scheme#reverse */
t16=*((C_word*)lf[4]+1);{
C_word av2[3];
av2[0]=t16;
av2[1]=t15;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t16+1)))(3,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_6380,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[9],a[7]=t4,a[8]=t5,a[9]=((C_word*)t0)[3],a[10]=((C_word*)t0)[4],a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
/* optimizer.scm:479: test */
t9=((C_word*)((C_word*)t0)[12])[1];
f_5054(t9,t8,C_i_car(t2),lf[93]);}}

/* g1116 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_fcall f_6316(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_6316,3,t0,t1,t2);}
/* optimizer.scm:473: g1133 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5411(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k6324 in k6368 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6326(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_6326,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6329,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:477: invalidate-gae! */
f_5084(t2,((C_word*)t0)[5]);}

/* k6327 in k6324 in k6368 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6329(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6329,c,av);}
/* optimizer.scm:478: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1110 in k6368 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_fcall f_6334(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_6334,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6359,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:473: g1116 */
t4=((C_word*)t0)[4];
f_6316(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k6357 in map-loop1110 in k6368 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6359(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6359,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_6334(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k6368 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_6370,c,av);}
a=C_alloc(17);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6326,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6334,a[2]=((C_word*)t0)[7],a[3]=t5,a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[9],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_6334(t7,t3,t2);}

/* k6372 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6374,c,av);}
/* optimizer.scm:476: scheme#append */
t2=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,5)))){
C_save_and_reclaim((void *)f_6380,c,av);}
a=C_alloc(32);
if(C_truep(t1)){
t2=(
/* optimizer.scm:480: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6386,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:481: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[36];
av2[3]=lf[119];
av2[4]=C_u_i_car(((C_word*)t0)[4]);
av2[5]=((C_word*)t0)[13];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[4]);
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[5],C_fix(1));
t4=C_i_cdr(((C_word*)t0)[7]);
t5=C_u_i_car(((C_word*)t0)[7]);
t6=C_a_i_cons(&a,2,t5,((C_word*)t0)[8]);
/* optimizer.scm:491: loop */
t7=((C_word*)((C_word*)t0)[6])[1];
f_6299(t7,((C_word*)t0)[3],t2,t3,t4,t6);}}

/* k6384 in k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_6386,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6392,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:484: chicken.compiler.support#expression-has-side-effects? */
t3=*((C_word*)lf[118]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[6]);
av2[3]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k6390 in k6384 in k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,5)))){
C_save_and_reclaim((void *)f_6392,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6423,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:487: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[117];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[3]);
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[4],C_fix(1));
/* optimizer.scm:490: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_6299(t4,((C_word*)t0)[2],t2,t3,C_u_i_cdr(((C_word*)t0)[6]),((C_word*)t0)[7]);}}

/* k6405 in k6421 in k6390 in k6384 in k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in ... */
static void C_ccall f_6407(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,5)))){
C_save_and_reclaim((void *)f_6407,c,av);}
a=C_alloc(34);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6411,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_u_i_cdr(((C_word*)t0)[4]);
t4=C_s_a_i_minus(&a,2,((C_word*)t0)[5],C_fix(1));
/* optimizer.scm:489: loop */
t5=((C_word*)((C_word*)t0)[6])[1];
f_6299(t5,t2,t3,t4,C_u_i_cdr(((C_word*)t0)[7]),((C_word*)t0)[8]);}

/* k6409 in k6405 in k6421 in k6390 in k6384 in k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in ... */
static void C_ccall f_6411(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_6411,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:485: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k6421 in k6390 in k6384 in k6378 in loop in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_6423,c,av);}
a=C_alloc(12);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_6407,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:488: walk */
t4=((C_word*)((C_word*)t0)[8])[1];
f_5411(t4,t3,C_u_i_car(((C_word*)t0)[6]),((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,8)))){
C_save_and_reclaim((void *)f_6473,c,av);}
a=C_alloc(12);
t2=C_i_length(((C_word*)t0)[2]);
if(C_truep(C_i_lessp(t2,t1))){
/* optimizer.scm:499: walk-generic */
t3=((C_word*)((C_word*)t0)[3])[1];
f_7238(t3,((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_TRUE);}
else{
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6485,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[4],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:501: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[36];
av2[3]=lf[122];
av2[4]=((C_word*)t0)[14];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}}

/* k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_6485,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6490,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_6496,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:502: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[11];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a6489 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6490,c,av);}
/* optimizer.scm:502: split-at */
f_3561(t1,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,4)))){
C_save_and_reclaim((void *)f_6496,c,av);}
a=C_alloc(39);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6500,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6512,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_6560,a[2]=((C_word*)t0)[7],a[3]=t4,a[4]=((C_word*)t0)[8],a[5]=t7,a[6]=t9,a[7]=t8,tmp=(C_word)a,a+=8,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6568,a[2]=t10,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_nullp(t3))){
/* optimizer.scm:512: chicken.compiler.support#qnode */
t12=*((C_word*)lf[77]+1);{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
av2[2]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t12=C_i_length(t3);
t13=C_a_i_fixnum_times(&a,2,C_fix(3),t12);
t14=C_a_i_list2(&a,2,lf[120],t13);
/* optimizer.scm:513: chicken.compiler.support#make-node */
t15=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t15;
av2[1]=t11;
av2[2]=lf[121];
av2[3]=t14;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t15+1)))(5,av2);}}}

/* k6498 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_6500,c,av);}
a=C_alloc(7);
t2=C_a_i_cons(&a,2,t1,((C_word*)((C_word*)t0)[2])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6507,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:518: invalidate-gae! */
f_5084(t4,((C_word*)t0)[5]);}

/* k6505 in k6498 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in ... */
static void C_ccall f_6507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6507,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g1166 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_fcall f_6512(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_6512,3,t0,t1,t2);}
/* optimizer.scm:506: g1183 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5411(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k6520 in k6558 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in ... */
static void C_ccall f_6522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6522,c,av);}
/* optimizer.scm:503: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1160 in k6558 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in ... */
static void C_fcall f_6524(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_6524,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6549,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:506: g1166 */
t4=((C_word*)t0)[4];
f_6512(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k6547 in map-loop1160 in k6558 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in ... */
static void C_ccall f_6549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6549,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_6524(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k6558 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_6560,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6522,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6524,a[2]=((C_word*)t0)[5],a[3]=t5,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_6524(t7,t3,t2);}

/* k6566 in a6495 in k6483 in k6471 in k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in ... */
static void C_ccall f_6568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6568,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:508: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k6595 in k6283 in k6173 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,8)))){
C_save_and_reclaim((void *)f_6597,c,av);}
a=C_alloc(15);
t2=(C_truep(t1)?C_i_not(C_i_memq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1])):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_6473,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],tmp=(C_word)a,a+=15,tmp);
/* optimizer.scm:497: chicken.compiler.support#llist-length */
t4=*((C_word*)lf[123]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[16];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* optimizer.scm:520: walk-generic */
t3=((C_word*)((C_word*)t0)[5])[1];
f_7238(t3,((C_word*)t0)[6],((C_word*)t0)[2],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_TRUE);}}

/* k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_6610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_6610,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_6669,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:425: test */
t3=((C_word*)((C_word*)t0)[11])[1];
f_5054(t3,t2,((C_word*)t0)[7],lf[102]);}
else{
t2=((C_word*)t0)[5];
f_6175(t2,C_SCHEME_FALSE);}}

/* k6626 in k6663 in k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in ... */
static void C_ccall f_6628(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_6628,c,av);}
a=C_alloc(16);
t2=C_eqp(t1,lf[85]);
t3=(C_truep(t2)?C_SCHEME_FALSE:(C_truep(((C_word*)t0)[2])?((C_word*)t0)[2]:C_i_lessp(C_i_cadddr(((C_word*)t0)[3]),((C_word*)t0)[4])));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6634,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp);
t5=C_i_car(((C_word*)t0)[8]);
t6=C_a_i_cons(&a,2,((C_word*)t0)[7],t5);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7427,a[2]=t6,a[3]=((C_word*)t0)[9],a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_7427(t10,t4,lf[53],C_fix(0));}
else{
t4=((C_word*)t0)[5];
f_6175(t4,C_SCHEME_FALSE);}}

/* k6632 in k6626 in k6663 in k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_ccall f_6634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_6634,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];
f_6175(t2,t1);}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6640,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:433: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[106];
av2[3]=lf[124];
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=C_u_i_car(((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(7,av2);}}}

/* k6638 in k6632 in k6626 in k6663 in k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in ... */
static void C_ccall f_6640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6640,c,av);}
t2=((C_word*)t0)[2];
f_6175(t2,C_SCHEME_FALSE);}

/* k6663 in k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6665(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_6665,c,av);}
a=C_alloc(10);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_6628,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:427: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[86]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=lf[87];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[5];
f_6175(t2,C_SCHEME_FALSE);}}

/* k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_6669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_6669,c,av);}
a=C_alloc(11);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_6665,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* optimizer.scm:426: test */
t3=((C_word*)((C_word*)t0)[11])[1];
f_5054(t3,t2,((C_word*)t0)[7],lf[96]);}
else{
t2=((C_word*)t0)[5];
f_6175(t2,C_SCHEME_FALSE);}}

/* k6671 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_6673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6673,c,av);}
/* optimizer.scm:422: chicken.compiler.support#node? */
t2=*((C_word*)lf[126]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k6677 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,8)))){
C_save_and_reclaim((void *)f_6679,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_6682,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6703,a[2]=t2,a[3]=((C_word*)t0)[9],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:526: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* optimizer.scm:533: walk-generic */
t2=((C_word*)((C_word*)t0)[10])[1];
f_7238(t2,((C_word*)t0)[3],((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[8],((C_word*)t0)[6],((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_TRUE);}}

/* k6680 in k6677 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_6682,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6689,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_u_i_cdr(((C_word*)t0)[6]);
t4=C_a_i_cons(&a,2,((C_word*)t0)[7],t3);
/* optimizer.scm:528: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[30];
av2[3]=((C_word*)t0)[8];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k6687 in k6680 in k6677 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_6689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6689,c,av);}
/* optimizer.scm:527: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5411(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k6701 in k6677 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_6703,c,av);}
/* optimizer.scm:525: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[106];
av2[3]=lf[128];
av2[4]=((C_word*)t0)[3];
av2[5]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k6721 in k6725 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6723,c,av);}
/* optimizer.scm:523: chicken.compiler.support#intrinsic? */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k6725 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6727(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6727,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6723,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:523: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_6679(2,av2);}}}

/* k6732 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_6734(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6734,c,av);}
t2=((C_word*)t0)[2];
f_6151(t2,C_eqp(lf[29],t1));}

/* k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_6749,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6821,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:397: chicken.compiler.support#node-class */
t3=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_6123(t2,C_SCHEME_FALSE);}}

/* k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_6767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_6767,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6770,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=t2;
f_6770(t3,t1);}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6807,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:400: test */
t4=((C_word*)((C_word*)t0)[5])[1];
f_5054(t4,t3,C_u_i_car(((C_word*)t0)[6]),lf[130]);}}

/* k6768 in k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_fcall f_6770(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_6770,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6780,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6782,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:403: any */
f_3804(t2,t3,C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=((C_word*)t0)[2];
f_6123(t2,C_SCHEME_FALSE);}}

/* k6778 in k6768 in k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6780,c,av);}
t2=((C_word*)t0)[2];
f_6123(t2,C_i_not(t1));}

/* a6781 in k6768 in k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6782,c,av);}
t3=*((C_word*)lf[118]+1);
/* optimizer.scm:403: g1020 */
t4=*((C_word*)lf[118]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k6799 in k6805 in k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_6801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6801,c,av);}
t2=((C_word*)t0)[2];
f_6770(t2,C_i_not(t1));}

/* k6805 in k6765 in k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_6807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_6807,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6801,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:401: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_5054(t3,t2,C_u_i_car(((C_word*)t0)[4]),lf[62]);}
else{
t2=((C_word*)t0)[2];
f_6770(t2,C_SCHEME_FALSE);}}

/* k6815 in k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_6817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_6817,c,av);}
a=C_alloc(7);
t2=C_i_caddr(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_6767,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:399: test */
t4=((C_word*)((C_word*)t0)[5])[1];
f_5054(t4,t3,C_i_car(t2),lf[93]);}
else{
t3=((C_word*)t0)[2];
f_6123(t3,C_SCHEME_FALSE);}}

/* k6819 in k6747 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_6821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6821,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[29],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6817,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:398: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_6123(t3,C_SCHEME_FALSE);}}

/* k6829 in k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_6831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6831,c,av);}
if(C_truep(C_i_not(t1))){
/* optimizer.scm:396: test */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5054(t2,((C_word*)t0)[3],((C_word*)t0)[4],lf[81]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_6749(2,av2);}}}

/* k6833 in k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_6835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_6835,c,av);}
a=C_alloc(11);
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6749,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6831,a[2]=((C_word*)t0)[5],a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:395: test */
t5=((C_word*)((C_word*)t0)[5])[1];
f_5054(t5,t4,t2,lf[131]);}
else{
t3=((C_word*)t0)[2];
f_6123(t3,C_SCHEME_FALSE);}}

/* k6839 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_6841(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6841,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6835,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:394: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_6123(t3,C_SCHEME_FALSE);}}

/* k6846 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6848,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6873,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:369: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_5054(t3,t2,((C_word*)t0)[5],lf[90]);}
else{
t2=((C_word*)t0)[2];
f_6041(t2,C_SCHEME_FALSE);}}

/* k6859 in k6871 in k6846 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_6861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6861,c,av);}
t2=((C_word*)t0)[2];
f_6041(t2,C_i_not(t1));}

/* k6867 in k6871 in k6846 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_6869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6869,c,av);}
/* optimizer.scm:371: test */
t2=((C_word*)((C_word*)t0)[2])[1];
f_5054(t2,((C_word*)t0)[3],C_i_car(t1),lf[102]);}

/* k6871 in k6846 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_6873,c,av);}
a=C_alloc(7);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6861,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6869,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:371: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_6041(t2,C_SCHEME_FALSE);}}

/* k6877 in k6887 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6879(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6879,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_6034(2,av2);}}
else{
/* optimizer.scm:366: test */
t2=((C_word*)((C_word*)t0)[3])[1];
f_5054(t2,((C_word*)t0)[2],((C_word*)t0)[4],lf[133]);}}

/* k6887 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6889(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_6889,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6879,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:365: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_5054(t3,t2,((C_word*)t0)[4],lf[81]);}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_6034(2,av2);}}}

/* k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6893(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_6893,c,av);}
a=C_alloc(21);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_6031,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t2,a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
/* optimizer.scm:363: chicken.compiler.support#call-info */
t4=*((C_word*)lf[61]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[12];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k6909 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6911,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6914,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:539: invalidate-gae! */
f_5084(t2,((C_word*)t0)[4]);}

/* k6912 in k6909 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_6914,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* g1207 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_6923(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_6923,3,t0,t1,t2);}
/* optimizer.scm:538: g1224 */
t3=((C_word*)((C_word*)t0)[2])[1];
f_5411(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k6928 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6930,c,av);}
/* optimizer.scm:537: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1201 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_6932(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_6932,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6957,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:538: g1207 */
t4=((C_word*)t0)[4];
f_6923(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k6955 in map-loop1201 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6957,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_6932(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6990(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_6990,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(
/* optimizer.scm:546: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6996,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7002,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:547: test */
t5=((C_word*)((C_word*)t0)[5])[1];
f_5054(t5,t4,((C_word*)t0)[4],lf[89]);}
else{
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_7011,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* optimizer.scm:550: test */
t3=((C_word*)((C_word*)t0)[5])[1];
f_5054(t3,t2,((C_word*)t0)[4],lf[80]);}}

/* k6994 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_6996(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_6996,c,av);}
/* optimizer.scm:549: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k7000 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7002,c,av);}
if(C_truep(t1)){
/* optimizer.scm:548: chicken.compiler.support#debugging */
t2=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[106];
av2[3]=lf[134];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
/* optimizer.scm:549: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_7011,c,av);}
a=C_alloc(19);
if(C_truep(t1)){
t2=(
/* optimizer.scm:551: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
/* optimizer.scm:552: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_7023,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7164,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[12],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:553: test */
t4=((C_word*)((C_word*)t0)[6])[1];
f_5054(t4,t3,((C_word*)t0)[4],lf[89]);}}

/* k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7023(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_7023,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(
/* optimizer.scm:558: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7029,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:559: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[36];
av2[3]=lf[135];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7035,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7112,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:562: walk */
t4=((C_word*)((C_word*)t0)[8])[1];
f_5411(t4,t3,C_i_car(((C_word*)t0)[9]),((C_word*)t0)[10],((C_word*)t0)[5]);}}

/* k7027 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7029,c,av);}
/* optimizer.scm:560: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7035,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7070,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:564: test */
t3=((C_word*)((C_word*)t0)[5])[1];
f_5054(t3,t2,((C_word*)t0)[2],lf[89]);}

/* k7042 in k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7044,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* for-each-loop1253 in k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7046(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7046,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7056,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:563: g1254 */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7054 in for-each-loop1253 in k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_7056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7056,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7046(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_7070,c,av);}
a=C_alloc(13);
t2=(C_truep(t1)?(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7071,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp):(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7088,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp));
t3=C_i_check_list_2(((C_word*)t0)[3],lf[23]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7044,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7046,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_7046(t8,t4,((C_word*)t0)[3]);}

/* f_7071 in k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7071(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7071,c,av);}
t3=C_i_cdr(t2);
t4=C_eqp(((C_word*)t0)[2],t3);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_i_slot(t2,C_fix(1),C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* f_7088 in k7068 in k7033 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7088,c,av);}
t3=C_i_car(t2);
t4=C_eqp(((C_word*)t0)[2],t3);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_i_slot(t2,C_fix(1),C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k7110 in k7021 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_7112,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:562: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[31];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k7120 in k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7122(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_7122,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7153,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:555: test */
t3=((C_word*)((C_word*)t0)[5])[1];
f_5054(t3,t2,((C_word*)t0)[6],lf[136]);}
else{
t2=((C_word*)t0)[2];
f_7023(t2,C_SCHEME_FALSE);}}

/* k7139 in k7147 in k7151 in k7120 in k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_7141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7141,c,av);}
t2=((C_word*)t0)[2];
f_7023(t2,C_i_not(t1));}

/* k7147 in k7151 in k7120 in k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7149,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7141,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:557: chicken.compiler.support#expression-has-side-effects? */
t3=*((C_word*)lf[118]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[3]);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_7023(t2,C_SCHEME_FALSE);}}

/* k7151 in k7120 in k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7153,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7149,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:556: test */
t3=((C_word*)((C_word*)t0)[5])[1];
f_5054(t3,t2,((C_word*)t0)[6],lf[130]);}
else{
t2=((C_word*)t0)[2];
f_7023(t2,C_SCHEME_FALSE);}}

/* k7158 in k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7160,c,av);}
t2=((C_word*)t0)[2];
f_7122(t2,C_i_not(t1));}

/* k7162 in k7009 in k6988 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7164,c,av);}
a=C_alloc(10);
t2=C_i_not(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7122,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_7122(t4,t2);}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7160,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:554: chicken.compiler.support#variable-visible? */
t5=*((C_word*)lf[137]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k7171 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7173(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,8)))){
C_save_and_reclaim_args((void *)trf_7173,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_7182,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* optimizer.scm:582: test */
t4=((C_word*)((C_word*)t0)[11])[1];
f_5054(t4,t3,t2,lf[64]);}
else{
/* optimizer.scm:590: walk-generic */
t2=((C_word*)((C_word*)t0)[6])[1];
f_7238(t2,((C_word*)t0)[4],((C_word*)t0)[7],((C_word*)t0)[5],((C_word*)t0)[2],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}}

/* k7180 in k7171 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,8)))){
C_save_and_reclaim((void *)f_7182,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(
/* optimizer.scm:583: touch */
  f_5080(((C_word*)((C_word*)t0)[2])[1])
);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7188,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:584: chicken.compiler.support#debugging */
t4=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[36];
av2[3]=lf[139];
av2[4]=((C_word*)t0)[6];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
/* optimizer.scm:588: walk-generic */
t2=((C_word*)((C_word*)t0)[7])[1];
f_7238(t2,((C_word*)t0)[3],((C_word*)t0)[8],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_FALSE);}}

/* k7186 in k7180 in k7171 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7188(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7188,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7195,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:586: chicken.compiler.support#varnode */
t3=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7193 in k7186 in k7180 in k7171 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7195,c,av);}
/* optimizer.scm:586: chicken.compiler.support#replace-rest-op-with-list-ops */
t2=*((C_word*)lf[138]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* walk-generic in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7238(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8){
C_word tmp;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,5)))){
C_save_and_reclaim_args((void *)trf_7238,9,t0,t1,t2,t3,t4,t5,t6,t7,t8);}
a=C_alloc(13);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_7244,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t8,a[6]=((C_word*)t0)[2],a[7]=t7,a[8]=t10,a[9]=((C_word*)t0)[3],a[10]=t6,tmp=(C_word)a,a+=11,tmp));
t12=((C_word*)t10)[1];
f_7244(t12,t1,C_SCHEME_TRUE,t5,C_SCHEME_END_OF_LIST);}

/* lp in walk-generic in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7244(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_7244,5,t0,t1,t2,t3,t4);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t3))){
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7254,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=t4,tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[5])){
/* optimizer.scm:597: invalidate-gae! */
f_5084(t5,((C_word*)t0)[7]);}
else{
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_7254(2,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7270,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=((C_word*)t0)[8],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:601: walk */
t6=((C_word*)((C_word*)t0)[9])[1];
f_5411(t6,t5,C_i_car(t3),((C_word*)t0)[10],((C_word*)t0)[7]);}}

/* k7252 in lp in walk-generic in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7254,c,av);}
a=C_alloc(5);
if(C_truep(((C_word*)t0)[2])){
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7264,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:599: scheme#reverse */
t3=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k7262 in k7252 in lp in walk-generic in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7264,c,av);}
/* optimizer.scm:599: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k7268 in lp in walk-generic in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_7270,c,av);}
a=C_alloc(3);
t2=(C_truep(((C_word*)t0)[2])?C_eqp(t1,C_u_i_car(((C_word*)t0)[3])):C_SCHEME_FALSE);
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,t1,((C_word*)t0)[4]);
/* optimizer.scm:602: lp */
t5=((C_word*)((C_word*)t0)[5])[1];
f_7244(t5,((C_word*)t0)[6],t2,t3,t4);}

/* k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7298(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_7298,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
/* optimizer.scm:606: scheme#values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_TRUE;
C_values(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7304,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:608: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[48];
av2[3]=lf[150];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7304(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_7304,c,av);}
a=C_alloc(8);
t2=lf[51] /* chicken.compiler.optimizer#simplified-ops */ =C_SCHEME_END_OF_LIST;;
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7308,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:610: walk */
t4=((C_word*)((C_word*)t0)[8])[1];
f_5411(t4,t3,((C_word*)t0)[9],C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST);}

/* k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7308(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_7308,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7311,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[7])[1]))){
/* optimizer.scm:611: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[149];
av2[4]=((C_word*)((C_word*)t0)[7])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7311(2,av2);}}}

/* k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7311,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7314,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_pairp(lf[51]))){
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7352,tmp=(C_word)a,a+=2,tmp);
/* optimizer.scm:613: chicken.compiler.support#with-debugging-output */
t4=*((C_word*)lf[148]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7314(2,av2);}}}

/* k7312 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_7314,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7317,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_greaterp(((C_word*)((C_word*)t0)[7])[1],C_fix(0)))){
/* optimizer.scm:624: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[142];
av2[4]=((C_word*)((C_word*)t0)[7])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7317(2,av2);}}}

/* k7315 in k7312 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_7317,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7320,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_greaterp(((C_word*)((C_word*)t0)[6])[1],C_fix(0)))){
/* optimizer.scm:625: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[141];
av2[4]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_7320(2,av2);}}}

/* k7318 in k7315 in k7312 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_7320,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7323,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_greaterp(((C_word*)((C_word*)t0)[5])[1],C_fix(0)))){
/* optimizer.scm:626: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[140];
av2[4]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* optimizer.scm:627: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
C_values(4,av2);}}}

/* k7321 in k7318 in k7315 in k7312 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7323,c,av);}
/* optimizer.scm:627: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
C_values(4,av2);}}

/* a7351 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7352(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7352,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7356,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:616: chicken.base#print */
t3=*((C_word*)lf[143]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[147];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7354 in a7351 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7356(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_7356,c,av);}
a=C_alloc(5);
t2=lf[51];
t3=C_i_check_list_2(lf[51],lf[23]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7389,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7389(t7,((C_word*)t0)[2],lf[51]);}

/* k7359 in for-each-loop1309 in k7354 in a7351 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7361,c,av);}
if(C_truep(C_i_greaterp(C_u_i_cdr(((C_word*)t0)[2]),C_fix(1)))){
/* optimizer.scm:621: chicken.base#print */
t2=*((C_word*)lf[143]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_make_character(9);
av2[3]=C_u_i_cdr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
/* optimizer.scm:622: scheme#newline */
t2=*((C_word*)lf[144]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* for-each-loop1309 in k7354 in a7351 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7389(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_7389,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7399,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7361,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:619: chicken.base#print* */
t6=*((C_word*)lf[145]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[146];
av2[3]=C_i_car(t4);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7397 in for-each-loop1309 in k7354 in a7351 in k7309 in k7306 in k7302 in k7296 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7399,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7389(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* loop in k6626 in k6663 in k6667 in k6608 in k6167 in a6161 in k6152 in k6149 in k6121 in k6118 in k6039 in k6032 in k6029 in k6891 in k6017 in k5577 in k5574 in k5571 in walk1 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in ... */
static void C_fcall f_7427(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(29,0,3)))){
C_save_and_reclaim_args((void *)trf_7427,4,t0,t1,t2,t3);}
a=C_alloc(29);
t4=C_i_nullp(t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_car(t2);
if(C_truep(C_i_equalp(((C_word*)t0)[2],t5))){
if(C_truep(C_i_lessp(t3,((C_word*)t0)[3]))){
t6=C_u_i_cdr(t2);
t7=C_s_a_i_plus(&a,2,t3,C_fix(1));
/* optimizer.scm:643: loop */
t9=t1;
t10=t6;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
/* optimizer.scm:644: loop */
t9=t1;
t10=C_u_i_cdr(t2);
t11=t3;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* touch in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_7467(C_word t0){
C_word tmp;
C_word t1;
C_word t2;
C_stack_overflow_check;{}
t1=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
return(C_SCHEME_TRUE);}

/* test in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7471(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_7471,4,t0,t1,t2,t3);}
/* optimizer.scm:658: chicken.compiler.support#db-get */
t4=*((C_word*)lf[55]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7478(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_7478,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7481,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7493,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:663: chicken.compiler.support#intrinsic? */
t4=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[156];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7479 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_7481,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7484,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_greaterp(((C_word*)((C_word*)t0)[4])[1],C_fix(0)))){
/* optimizer.scm:701: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[151];
av2[4]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=((C_word*)t0)[2];
f_7298(t3,((C_word*)((C_word*)t0)[3])[1]);}}

/* k7482 in k7479 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7484,c,av);}
t2=((C_word*)t0)[2];
f_7298(t2,((C_word*)((C_word*)t0)[3])[1]);}

/* k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_7493,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7494,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7691,a[2]=t2,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:699: test */
t4=((C_word*)((C_word*)t0)[5])[1];
f_7471(t4,t3,lf[156],lf[157]);}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_7481(2,av2);}}}

/* g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7494(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_7494,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7501,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:667: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_7501,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7684,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:668: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(t1);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_7507,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7510,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:670: chicken.compiler.support#db-get-list */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[8];
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_7510,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7516,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[8])){
t3=C_i_length(t1);
t4=C_eqp(C_fix(1),t3);
if(C_truep(t4)){
t5=C_i_length(((C_word*)t0)[6]);
t6=C_eqp(C_fix(3),t5);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7660,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:674: chicken.compiler.support#node-class */
t8=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=t2;
f_7516(t7,C_SCHEME_FALSE);}}
else{
t5=t2;
f_7516(t5,C_SCHEME_FALSE);}}
else{
t3=t2;
f_7516(t3,C_SCHEME_FALSE);}}

/* k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7516(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_7516,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7641,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:675: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 in ... */
static void C_ccall f_7525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_7525,c,av);}
a=C_alloc(10);
t2=C_i_listp(((C_word*)t0)[2]);
t3=(C_truep(t2)?C_i_nullp(C_i_cdr(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7537,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t4,tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:681: chicken.compiler.support#db-get-list */
t6=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[9];
av2[3]=t4;
av2[4]=lf[130];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in ... */
static void C_ccall f_7537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_7537,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7543,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t3=C_i_length(t1);
t4=C_eqp(C_fix(1),t3);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7618,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:683: chicken.compiler.support#node-class */
t6=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t2;
f_7543(t5,C_SCHEME_FALSE);}}

/* k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_fcall f_7543(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_7543,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7608,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:685: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[42]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k7550 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_fcall f_7552(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(37,0,3)))){
C_save_and_reclaim_args((void *)trf_7552,2,t0,t1);}
a=C_alloc(37);
if(C_truep(t1)){
t2=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7559,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:693: chicken.compiler.support#node-parameters-set! */
t5=*((C_word*)lf[153]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
av2[3]=lf[154];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k7557 in k7550 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_7559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7559,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7562,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:694: chicken.compiler.support#node-subexpressions-set! */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=C_i_cdr(((C_word*)t0)[7]);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7560 in k7557 in k7550 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in ... */
static void C_ccall f_7562(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_7562,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7565,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_car(((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7580,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:697: scheme#reverse */
t5=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7563 in k7560 in k7557 in k7550 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_7565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7565,c,av);}
/* optimizer.scm:698: touch */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* optimizer.scm:698: touch */
  f_7467(((C_word*)((C_word*)t0)[3])[1])
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k7578 in k7560 in k7557 in k7550 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in ... */
static void C_ccall f_7580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7580,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:695: chicken.compiler.support#node-subexpressions-set! */
t3=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7598 in k7602 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in ... */
static void C_ccall f_7600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7600,c,av);}
t2=((C_word*)t0)[2];
f_7552(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k7602 in k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in ... */
static void C_ccall f_7604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_7604,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7600,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:688: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_7552(t3,C_SCHEME_FALSE);}}

/* k7606 in k7541 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in ... */
static void C_ccall f_7608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_7608,c,av);}
a=C_alloc(14);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7552,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7604,a[2]=t3,a[3]=((C_word*)t0)[9],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:687: chicken.compiler.support#node-class */
t5=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k7616 in k7535 in k7523 in k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in ... */
static void C_ccall f_7618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7618,c,av);}
t2=((C_word*)t0)[2];
f_7543(t2,C_eqp(lf[25],t1));}

/* k7635 in k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7637(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_7637,c,av);}
a=C_alloc(10);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_7525,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:677: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7639 in k7514 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_7641,c,av);}
a=C_alloc(9);
t2=C_i_caddr(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7637,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:676: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[42]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7658 in k7508 in k7505 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7660,c,av);}
t2=((C_word*)t0)[2];
f_7516(t2,C_eqp(lf[29],t1));}

/* k7678 in k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7680,c,av);}
if(C_truep(C_i_not(t1))){
/* optimizer.scm:669: test */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7471(t2,((C_word*)t0)[3],((C_word*)t0)[4],lf[81]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_7507(2,av2);}}}

/* k7682 in k7499 in g1375 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7684(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_7684,c,av);}
a=C_alloc(14);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7507,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7680,a[2]=((C_word*)t0)[8],a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:669: test */
t5=((C_word*)((C_word*)t0)[8])[1];
f_7471(t5,t4,t2,lf[131]);}

/* k7689 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7691(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7691,c,av);}
a=C_alloc(6);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=C_i_check_list_2(t2,lf[23]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7702,a[2]=t5,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_7702(t7,((C_word*)t0)[3],t2);}

/* for-each-loop1374 in k7689 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_7702(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7702,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7712,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:664: g1375 */
t4=((C_word*)t0)[3];
f_7494(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7710 in for-each-loop1374 in k7689 in k7491 in k7476 in chicken.compiler.optimizer#perform-high-level-optimizations in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7712,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7702(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(594,c,15)))){
C_save_and_reclaim((void *)f_7732,c,av);}
a=C_alloc(594);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7735,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list(&a,1,lf[206]);
t4=C_a_i_list(&a,1,lf[204]);
t5=C_a_i_list(&a,1,lf[207]);
t6=C_a_i_list(&a,2,lf[24],t5);
t7=C_a_i_list(&a,1,lf[208]);
t8=C_a_i_list(&a,2,lf[76],t7);
t9=C_a_i_list(&a,4,lf[58],t4,t6,t8);
t10=C_a_i_list(&a,1,lf[206]);
t11=C_a_i_list(&a,2,lf[24],t10);
t12=C_a_i_list(&a,1,lf[209]);
t13=C_a_i_list(&a,1,lf[204]);
t14=C_a_i_list(&a,1,lf[207]);
t15=C_a_i_list(&a,2,lf[24],t14);
t16=C_a_i_list(&a,1,lf[210]);
t17=C_a_i_list(&a,2,lf[76],t16);
t18=C_a_i_list(&a,4,lf[58],t13,t15,t17);
t19=C_a_i_list(&a,1,lf[209]);
t20=C_a_i_list(&a,2,lf[24],t19);
t21=C_a_i_list(&a,5,lf[25],lf[199],t20,lf[211],lf[212]);
t22=C_a_i_list(&a,4,lf[26],t12,t18,t21);
t23=C_a_i_list(&a,5,lf[25],lf[203],t11,lf[213],t22);
t24=C_a_i_list(&a,4,lf[26],t3,t9,t23);
t25=C_a_i_list(&a,11,lf[207],lf[206],lf[209],lf[204],lf[208],lf[210],lf[213],lf[211],lf[203],lf[199],lf[212]);
t26=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13922,tmp=(C_word)a,a+=2,tmp);
t27=C_a_i_list(&a,3,t24,t25,t26);
t28=C_a_i_list(&a,1,lf[198]);
t29=C_a_i_list(&a,1,lf[204]);
t30=C_a_i_list(&a,1,lf[207]);
t31=C_a_i_list(&a,2,lf[24],t30);
t32=C_a_i_list(&a,1,lf[216]);
t33=C_a_i_list(&a,2,lf[76],t32);
t34=C_a_i_list(&a,4,lf[58],t29,t31,t33);
t35=C_a_i_list(&a,1,lf[198]);
t36=C_a_i_list(&a,2,lf[24],t35);
t37=C_a_i_list(&a,1,lf[217]);
t38=C_a_i_list(&a,1,lf[207]);
t39=C_a_i_list(&a,2,lf[24],t38);
t40=C_a_i_cons(&a,2,t39,lf[218]);
t41=C_a_i_cons(&a,2,t37,t40);
t42=C_a_i_cons(&a,2,lf[40],t41);
t43=C_a_i_list(&a,5,lf[25],lf[219],t36,lf[220],t42);
t44=C_a_i_list(&a,4,lf[26],t28,t34,t43);
t45=C_a_i_list(&a,8,lf[198],lf[204],lf[207],lf[216],lf[219],lf[220],lf[217],lf[218]);
t46=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13795,tmp=(C_word)a,a+=2,tmp);
t47=C_a_i_list(&a,3,t44,t45,t46);
t48=C_a_i_list(&a,1,lf[206]);
t49=C_a_i_list(&a,2,lf[34],C_SCHEME_END_OF_LIST);
t50=C_a_i_list(&a,4,lf[26],t48,t49,lf[221]);
t51=C_a_i_list(&a,2,lf[206],lf[221]);
t52=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13482,tmp=(C_word)a,a+=2,tmp);
t53=C_a_i_list(&a,3,t50,t51,t52);
t54=C_a_i_list(&a,1,lf[198]);
t55=C_a_i_list(&a,1,lf[204]);
t56=C_a_i_cons(&a,2,t55,lf[224]);
t57=C_a_i_cons(&a,2,lf[58],t56);
t58=C_a_i_list(&a,1,lf[198]);
t59=C_a_i_list(&a,2,lf[24],t58);
t60=C_a_i_list(&a,5,lf[25],lf[219],t59,lf[190],lf[200]);
t61=C_a_i_list(&a,4,lf[26],t54,t57,t60);
t62=C_a_i_list(&a,6,lf[198],lf[204],lf[224],lf[219],lf[190],lf[200]);
t63=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13399,tmp=(C_word)a,a+=2,tmp);
t64=C_a_i_list(&a,3,t61,t62,t63);
t65=C_a_i_list(&a,1,lf[198]);
t66=C_a_i_list(&a,1,lf[225]);
t67=C_a_i_cons(&a,2,t66,lf[226]);
t68=C_a_i_cons(&a,2,lf[58],t67);
t69=C_a_i_list(&a,1,lf[227]);
t70=C_a_i_list(&a,2,lf[24],t69);
t71=C_a_i_list(&a,1,lf[228]);
t72=C_a_i_cons(&a,2,t71,lf[229]);
t73=C_a_i_cons(&a,2,lf[58],t72);
t74=C_a_i_list(&a,4,lf[30],lf[48],t70,t73);
t75=C_a_i_list(&a,4,lf[26],t65,t68,t74);
t76=C_a_i_list(&a,7,lf[198],lf[225],lf[226],lf[48],lf[227],lf[228],lf[229]);
t77=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13214,tmp=(C_word)a,a+=2,tmp);
t78=C_a_i_list(&a,3,t75,t76,t77);
t79=C_a_i_list(&a,1,lf[198]);
t80=C_a_i_list(&a,1,lf[204]);
t81=C_a_i_cons(&a,2,t80,lf[226]);
t82=C_a_i_cons(&a,2,lf[58],t81);
t83=C_a_i_cons(&a,2,lf[48],lf[229]);
t84=C_a_i_cons(&a,2,lf[30],t83);
t85=C_a_i_list(&a,4,lf[26],t79,t82,t84);
t86=C_a_i_list(&a,5,lf[198],lf[204],lf[226],lf[48],lf[229]);
t87=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13081,tmp=(C_word)a,a+=2,tmp);
t88=C_a_i_list(&a,3,t85,t86,t87);
t89=C_a_i_list(&a,6,t27,t47,t53,t64,t78,t88);
/* optimizer.scm:708: chicken.internal#hash-table-set! */
t90=*((C_word*)lf[161]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t90;
av2[1]=t2;
av2[2]=lf[50];
av2[3]=lf[26];
av2[4]=t89;
((C_proc)(void*)(*((C_word*)t90+1)))(5,av2);}}

/* k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7735(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(166,c,11)))){
C_save_and_reclaim((void *)f_7735,c,av);}
a=C_alloc(166);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7738,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list(&a,1,lf[198]);
t4=C_a_i_list(&a,2,lf[24],t3);
t5=C_a_i_list(&a,4,lf[30],lf[199],t4,lf[200]);
t6=C_a_i_list(&a,1,lf[198]);
t7=C_a_i_list(&a,2,lf[24],t6);
t8=C_a_i_list(&a,4,lf[30],lf[201],t7,lf[202]);
t9=C_a_i_list(&a,5,lf[25],lf[203],lf[190],t5,t8);
t10=C_a_i_list(&a,7,lf[203],lf[199],lf[201],lf[190],lf[200],lf[202],lf[198]);
t11=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13000,tmp=(C_word)a,a+=2,tmp);
t12=C_a_i_list(&a,3,t9,t10,t11);
t13=C_a_i_list(&a,1,lf[204]);
t14=C_a_i_list(&a,1,lf[205]);
t15=C_a_i_list(&a,2,lf[76],t14);
t16=C_a_i_list(&a,4,lf[58],t13,lf[190],t15);
t17=C_a_i_list(&a,5,lf[25],lf[203],t16,lf[200],lf[202]);
t18=C_a_i_list(&a,6,lf[203],lf[204],lf[190],lf[205],lf[200],lf[202]);
t19=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_12854,tmp=(C_word)a,a+=2,tmp);
t20=C_a_i_list(&a,3,t17,t18,t19);
t21=C_a_i_list(&a,2,t12,t20);
/* optimizer.scm:708: chicken.internal#hash-table-set! */
t22=*((C_word*)lf[161]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t22;
av2[1]=t2;
av2[2]=lf[50];
av2[3]=lf[25];
av2[4]=t21;
((C_proc)(void*)(*((C_word*)t22+1)))(5,av2);}}

/* k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_7738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7738,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8278,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1090: scheme#make-vector */
t3=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fix(301);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in ... */
static void C_ccall f_7753(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_7753,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7755,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7798,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7810,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8191,a[2]=t6,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_8191(t8,t4,((C_word*)t0)[3],((C_word*)t0)[6]);}

/* find-path in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in ... */
static void C_fcall f_7755(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_7755,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7761,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_7761(t7,t1,t2,C_SCHEME_END_OF_LIST);}

/* find in find-path in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_fcall f_7761(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_7761,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_memq(t2,t3);
if(C_truep(C_i_not(t4))){
t5=C_i_assq(t2,((C_word*)((C_word*)t0)[2])[1]);
t6=C_i_cdr(t5);
t7=C_i_memq(((C_word*)t0)[3],t6);
if(C_truep(t7)){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=C_a_i_cons(&a,2,t2,t3);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7785,a[2]=((C_word*)t0)[4],a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1013: any */
f_3804(t1,t9,t6);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* a7784 in find in find-path in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_ccall f_7785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7785,c,av);}
/* optimizer.scm:1013: find */
t3=((C_word*)((C_word*)t0)[2])[1];
f_7761(t3,t1,t2,((C_word*)t0)[3]);}

/* g1689 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in ... */
static void C_fcall f_7798(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_7798,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7807,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1017: chicken.compiler.support#scan-used-variables */
t5=*((C_word*)lf[165]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k7805 in g1689 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_ccall f_7807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_7807,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in ... */
static void C_ccall f_7810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,3)))){
C_save_and_reclaim((void *)f_7810,c,av);}
a=C_alloc(25);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7811,a[2]=t5,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_7870,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8168,a[2]=t9,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_8168(t11,t7,((C_word*)t0)[3]);}

/* g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_fcall f_7811(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_7811,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_memq(t2,((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7821,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7843,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1026: filter */
f_4010(t4,t5,((C_word*)t0)[5]);}
else{
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k7819 in g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_ccall f_7821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_7821,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7837,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1029: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7827 in k7835 in k7819 in g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_ccall f_7829(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7829,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7835 in k7819 in g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_7837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_7837,c,av);}
a=C_alloc(16);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=((C_word*)((C_word*)t0)[4])[1];
t4=C_a_i_cons(&a,2,t1,t2);
t5=C_a_i_cons(&a,2,t4,t3);
t6=C_mutate(((C_word *)((C_word*)t0)[4])+1,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7829,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t8=C_a_i_list1(&a,1,((C_word*)t0)[2]);
/* optimizer.scm:1030: scheme#append */
t9=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t7;
av2[2]=t8;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a7842 in g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_ccall f_7843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7843,c,av);}
a=C_alloc(6);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7856,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1027: find-path */
t5=((C_word*)t0)[3];
f_7755(t5,t4,((C_word*)t0)[2],t2);}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7854 in a7842 in g1729 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_7856(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7856,c,av);}
if(C_truep(t1)){
/* optimizer.scm:1027: find-path */
t2=((C_word*)t0)[2];
f_7755(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_ccall f_7870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_7870,c,av);}
a=C_alloc(22);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7871,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t5=((C_word*)((C_word*)t0)[2])[1];
t6=C_i_check_list_2(t5,lf[23]);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7930,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8145,a[2]=t9,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_8145(t11,t7,t5);}

/* g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_fcall f_7871(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_7871,3,t0,t1,t2);}
a=C_alloc(11);
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7878,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7912,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1039: append-map */
f_3600(t4,t5,C_u_i_cdr(t2),C_SCHEME_END_OF_LIST);}

/* k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_7878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_7878,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7886,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7888,a[2]=((C_word*)t0)[5],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=((C_word*)((C_word*)t0)[6])[1];
t5=C_i_check_list_2(t4,lf[6]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_4063,a[2]=t3,a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_4063(t9,t2,t4);}

/* k7884 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_ccall f_7886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_7886,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* a7887 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_fcall f_7888(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_7888,3,t0,t1,t2);}
a=C_alloc(7);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7901,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=C_i_cdr(t2);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_4448,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:193: every */
f_3770(t4,t6,t5);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7899 in a7887 in k7876 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_ccall f_7901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7901,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_u_i_car(((C_word*)t0)[3]):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a7911 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_7912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_7912,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7918,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1040: filter */
f_4010(t1,t3,((C_word*)t0)[3]);}

/* a7917 in a7911 in g1753 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_ccall f_7918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7918,c,av);}
/* optimizer.scm:1040: find-path */
t3=((C_word*)t0)[2];
f_7755(t3,t1,((C_word*)t0)[3],t2);}

/* k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_ccall f_7930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_7930,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7933,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1052: chicken.sort#topological-sort */
t3=*((C_word*)lf[223]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[7])[1];
av2[3]=*((C_word*)lf[47]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_7933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_7933,c,av);}
a=C_alloc(15);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7936,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7953,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t6,tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_7953(t8,t4,t1,((C_word*)t0)[6]);}

/* k7934 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_ccall f_7936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_7936,c,av);}
a=C_alloc(4);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[2])[1]))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7945,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1083: chicken.compiler.support#debugging */
t3=*((C_word*)lf[35]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[36];
av2[3]=lf[222];
av2[4]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* optimizer.scm:1085: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
C_values(4,av2);}}}

/* k7943 in k7934 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_ccall f_7945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7945,c,av);}
/* optimizer.scm:1084: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_TRUE;
C_values(4,av2);}}

/* foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in ... */
static void C_fcall f_7953(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_7953,4,t0,t1,t2,t3);}
a=C_alloc(11);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7969,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8139,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1057: g1790 */
t7=t5;
f_7969(t7,t6,t3,C_slot(t2,C_fix(0)));}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_fcall f_7969(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_7969,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=C_i_assq(t3,((C_word*)((C_word*)t0)[2])[1]);
t5=C_i_cdr(t4);
t6=C_i_car(t5);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7982,a[2]=t6,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t1,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_nullp(C_u_i_cdr(t5)))){
t8=C_i_assq(t6,((C_word*)((C_word*)t0)[5])[1]);
t9=C_i_cdr(t8);
t10=t7;
f_7982(t10,C_i_not(C_i_memq(t6,t9)));}
else{
t8=t7;
f_7982(t8,C_SCHEME_FALSE);}}

/* k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in ... */
static void C_fcall f_7982(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_7982,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=C_i_assq(((C_word*)t0)[2],((C_word*)t0)[4]);
t5=C_i_cdr(t4);
t6=C_a_i_list2(&a,2,t5,((C_word*)t0)[5]);
/* optimizer.scm:1064: chicken.compiler.support#make-node */
t7=*((C_word*)lf[33]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=((C_word*)t0)[6];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[7];
av2[4]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8006,a[2]=t3,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_8006(t5,((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in ... */
static void C_fcall f_8006(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_8006,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8041,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t8=t4;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=C_i_check_list_2(((C_word*)t0)[3],lf[6]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8053,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_8053(t7,t1,((C_word*)t0)[3]);}}

/* k8028 in k8039 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in ... */
static void C_ccall f_8030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8030,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:1068: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8039 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in ... */
static void C_ccall f_8041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_8041,c,av);}
a=C_alloc(8);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8030,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1070: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[34];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* foldr1819 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in ... */
static void C_fcall f_8053(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_8053,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8061,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8108,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g1824 in foldr1819 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in ... */
static void C_fcall f_8061(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8061,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8097,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1074: chicken.base#gensym */
t5=*((C_word*)lf[116]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k8075 in k8095 in g1824 in foldr1819 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in ... */
static void C_ccall f_8077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8077,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:1073: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8095 in g1824 in foldr1819 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in ... */
static void C_ccall f_8097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_8097,c,av);}
a=C_alloc(14);
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8077,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[4]);
t5=C_i_assq(((C_word*)t0)[4],((C_word*)t0)[5]);
t6=C_i_cdr(t5);
t7=C_a_i_list1(&a,1,t6);
/* optimizer.scm:1075: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t3;
av2[2]=lf[31];
av2[3]=t4;
av2[4]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}

/* k8106 in foldr1819 in foldr1806 in k7980 in g1790 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in ... */
static void C_ccall f_8108(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8108,c,av);}
/* optimizer.scm:1071: g1824 */
t2=((C_word*)t0)[2];
f_8061(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k8137 in foldl1783 in k7931 in k7928 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in ... */
static void C_ccall f_8139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8139,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7953(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* for-each-loop1752 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_fcall f_8145(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8145,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8155,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1035: g1753 */
t4=((C_word*)t0)[3];
f_7871(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k8153 in for-each-loop1752 in k7868 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in ... */
static void C_ccall f_8155(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8155,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8145(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop1728 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_fcall f_8168(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8168,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8178,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1023: g1729 */
t4=((C_word*)t0)[3];
f_7811(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k8176 in for-each-loop1728 in k7808 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in ... */
static void C_ccall f_8178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8178,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8168(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop1688 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in ... */
static void C_fcall f_8191(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_8191,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8201,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1016: g1689 */
t7=((C_word*)t0)[3];
f_7798(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k8199 in for-each-loop1688 in k7751 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in ... */
static void C_ccall f_8201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8201,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8191(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* map-loop1657 in k13655 in a13648 in k13609 in k13603 in k13600 in k13597 in loop2 in k13566 in k13536 in k13533 in k13530 in k13759 in k13767 in k13500 in k13497 in k13494 in loop1 in a13481 in k7730 in k5044 in k4691 in ... */
static void C_fcall f_8228(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_8228,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_cons(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8278,c,av);}
a=C_alloc(6);
t2=C_mutate(&lf[159] /* (set! chicken.compiler.optimizer#substitution-table ...) */,t1);
t3=C_mutate((C_word*)lf[160]+1 /* (set! chicken.compiler.optimizer#rewrite ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_8280,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate((C_word*)lf[162]+1 /* (set! chicken.compiler.optimizer#transform-direct-lambdas! ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_10576,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate((C_word*)lf[186]+1 /* (set! chicken.compiler.optimizer#determine-loop-and-dispatch ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11872,tmp=(C_word)a,a+=2,tmp));
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* chicken.compiler.optimizer#rewrite in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_8280,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8284,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1093: chicken.internal#hash-table-ref */
t5=*((C_word*)lf[57]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[159];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k8282 in chicken.compiler.optimizer#rewrite in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_8284,c,av);}
a=C_alloc(7);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8294,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_a_i_list1(&a,1,((C_word*)t0)[4]);
/* optimizer.scm:1094: scheme#append */
t5=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t2;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k8292 in k8282 in chicken.compiler.optimizer#rewrite in k8276 in k7736 in k7733 in k7730 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8294,c,av);}
/* optimizer.scm:1094: chicken.internal#hash-table-set! */
t2=*((C_word*)lf[161]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[159];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* argc-ok? in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static C_word C_fcall f_8303(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;
loop:{}
t2=C_i_not(t1);
if(C_truep(t2)){
return(t2);}
else{
t3=C_fixnump(t1);
t4=(C_truep(t3)?C_eqp(t1,C_i_length(((C_word*)t0)[2])):C_SCHEME_FALSE);
if(C_truep(t4)){
return(t4);}
else{
if(C_truep(C_i_pairp(t1))){
t5=f_8303(t0,C_u_i_car(t1));
if(C_truep(t5)){
t7=C_u_i_cdr(t1);
t1=t7;
goto loop;}
else{
return(C_SCHEME_FALSE);}}
else{
return(C_SCHEME_FALSE);}}}}

/* k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_8393,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8396,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=C_i_length(((C_word*)t0)[6]);
t4=C_i_car(((C_word*)t0)[5]);
if(C_truep(C_i_nequalp(t3,t4))){
t5=C_i_car(((C_word*)t0)[6]);
t6=C_i_cadr(((C_word*)t0)[6]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8483,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1120: chicken.compiler.support#node-class */
t8=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t5=t2;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_8396(2,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8394 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8396(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_8396,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_14115(2,av2);}}
else{
if(C_truep(((C_word*)t0)[3])){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8417,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_cadr(((C_word*)t0)[5]);
t5=C_a_i_list1(&a,1,t4);
/* optimizer.scm:1127: chicken.compiler.support#make-node */
t6=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=lf[58];
av2[3]=t5;
av2[4]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}}

/* k8415 in k8394 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8417,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1125: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8465 in k8473 in k8469 in k8477 in k8481 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8467,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1123: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8469 in k8477 in k8481 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8471,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8475,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1122: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8473 in k8469 in k8477 in k8481 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_8475,c,av);}
a=C_alloc(8);
if(C_truep(C_i_equalp(((C_word*)t0)[2],t1))){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8467,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1123: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_8396(2,av2);}}}

/* k8477 in k8481 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8479,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8471,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1122: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[43]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_8396(2,av2);}}}

/* k8481 in k8391 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_8483,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[24],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8479,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1121: chicken.compiler.support#node-class */
t4=*((C_word*)lf[41]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_8396(2,av2);}}}

/* k8510 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_8512,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(C_truep(t2)?t2:*((C_word*)lf[232]+1));
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[3]);
t5=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8539,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
t7=C_i_cadr(((C_word*)t0)[2]);
t8=C_a_i_list1(&a,1,t7);
/* optimizer.scm:1141: chicken.compiler.support#make-node */
t9=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t6;
av2[2]=lf[58];
av2[3]=t8;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8537 in k8510 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8539,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1137: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8568 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_8570,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_not(t2);
t4=(C_truep(t3)?t3:C_i_nequalp(C_i_length(((C_word*)t0)[3]),C_i_cadr(((C_word*)t0)[2])));
if(C_truep(t4)){
t5=C_i_check_list_2(((C_word*)t0)[3],lf[6]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8587,a[2]=t7,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_8587(t9,((C_word*)t0)[5],((C_word*)t0)[3]);}
else{
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* foldr1913 in k8568 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_8587(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_8587,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8622,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=t4;
t7=C_slot(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}
else{
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8641,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1152: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k8609 in k8620 in foldr1913 in k8568 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8611(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_8611,c,av);}
a=C_alloc(9);
t2=C_a_i_list1(&a,1,t1);
t3=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* optimizer.scm:1151: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[26];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k8620 in foldr1913 in k8568 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8622,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8611,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1151: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k8639 in foldr1913 in k8568 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8641,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1152: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8682 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8684(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_8684,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_list2(&a,2,C_SCHEME_FALSE,t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8699,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t3,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1162: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8697 in k8682 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_8699,c,av);}
a=C_alloc(8);
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8707,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* optimizer.scm:1165: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8705 in k8697 in k8682 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_8707,c,av);}
a=C_alloc(15);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_a_i_list5(&a,5,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,t2);
/* optimizer.scm:1161: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[6];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[7];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k8741 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_8743,c,av);}
a=C_alloc(16);
if(C_truep(t1)){
t2=C_i_length(((C_word*)t0)[2]);
t3=C_eqp(C_fix(1),t2);
if(C_truep(t3)){
t4=C_i_caddr(((C_word*)t0)[3]);
t5=C_i_not(t4);
t6=(C_truep(t5)?t5:C_eqp(t4,*((C_word*)lf[233]+1)));
if(C_truep(t6)){
t7=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8776,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t9=C_i_car(((C_word*)t0)[3]);
t10=C_a_i_list1(&a,1,t9);
t11=C_i_car(((C_word*)t0)[2]);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8792,a[2]=t11,a[3]=t8,a[4]=t10,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1180: chicken.compiler.support#qnode */
t13=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t13;
av2[1]=t12;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t13+1)))(3,av2);}}
else{
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8774 in k8741 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8776,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1176: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8790 in k8741 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8792,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1178: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8836 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8838(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_8838,c,av);}
a=C_alloc(18);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8853,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_car(((C_word*)t0)[4]);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8865,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=C_i_cadr(((C_word*)t0)[4]);
t8=C_a_i_list1(&a,1,t7);
/* optimizer.scm:1191: chicken.compiler.support#make-node */
t9=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t6;
av2[2]=lf[58];
av2[3]=t8;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8851 in k8836 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8853,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1188: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8863 in k8836 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_8865,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1190: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8910 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_8912,c,av);}
a=C_alloc(19);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8927,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_cadr(((C_word*)t0)[4]);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8935,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8943,a[2]=t6,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* optimizer.scm:1204: chicken.compiler.support#qnode */
t8=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_i_caddr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8925 in k8910 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8927(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8927,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1200: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8933 in k8910 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8935,c,av);}
/* optimizer.scm:1202: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[58];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8941 in k8910 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_8943,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* optimizer.scm:1203: scheme#append */
t3=*((C_word*)lf[7]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k8972 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_8974,c,av);}
if(C_truep(t1)){
/* optimizer.scm:1208: g1952 */
t2=C_i_car(((C_word*)t0)[2]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_8995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_8995,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_length(((C_word*)t0)[2]);
if(C_truep(C_fixnum_lessp(t2,C_fix(2)))){
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9016,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1218: chicken.compiler.support#qnode */
t5=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=(C_truep(*((C_word*)lf[232]+1))?C_i_not(C_eqp(*((C_word*)lf[233]+1),lf[234])):C_SCHEME_FALSE);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9025,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t3)){
t5=t4;
f_9025(t5,t3);}
else{
t5=C_eqp(*((C_word*)lf[233]+1),lf[235]);
t6=(C_truep(t5)?C_i_caddr(((C_word*)t0)[5]):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t4;
f_9025(t7,t6);}
else{
t7=C_eqp(*((C_word*)lf[233]+1),lf[237]);
t8=t4;
f_9025(t8,(C_truep(t7)?C_i_cadddr(((C_word*)t0)[5]):C_SCHEME_FALSE));}}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9014 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9016(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9016,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1218: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9025(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_9025,2,t0,t1);}
a=C_alloc(18);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9036,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9161,a[2]=t4,a[3]=t8,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_9161(t10,t6,((C_word*)t0)[5]);}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_9036,c,av);}
a=C_alloc(20);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=*((C_word*)lf[83]+1);
t7=C_i_check_list_2(t1,lf[8]);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9045,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9127,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_9127(t12,t8,t1);}

/* k9043 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_9045,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9050,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_9050(t5,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* loop in k9043 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9050(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_9050,4,t0,t1,t2,t3);}
a=C_alloc(14);
if(C_truep(C_i_nullp(t2))){
t4=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t5=C_eqp(*((C_word*)lf[233]+1),lf[235]);
t6=(C_truep(t5)?C_i_car(((C_word*)t0)[2]):C_i_cadr(((C_word*)t0)[2]));
t7=C_a_i_list1(&a,1,t6);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9075,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9077,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
/* optimizer.scm:1235: chicken.compiler.support#fold-boolean */
t10=*((C_word*)lf[236]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t8;
av2[2]=t9;
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t4=C_i_car(t3);
t5=C_a_i_list1(&a,1,t4);
t6=C_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9117,a[2]=t6,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1241: loop */
t11=t7;
t12=C_u_i_cdr(t2);
t13=C_u_i_cdr(t3);
t1=t11;
t2=t12;
t3=t13;
goto loop;}}

/* k9073 in loop in k9043 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9075,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1227: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a9076 in loop in k9043 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9077,c,av);}
a=C_alloc(6);
t4=C_a_i_list2(&a,2,t2,t3);
/* optimizer.scm:1236: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[58];
av2[3]=((C_word*)t0)[2];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k9115 in loop in k9043 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9117(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9117,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1238: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[26];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* map-loop1997 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9127(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9127,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9152,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1223: g2003 */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9150 in map-loop1997 in k9034 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9152(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9152,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9127(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* map-loop1969 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9161(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9161,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9186,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1222: chicken.base#gensym */
t4=*((C_word*)lf[116]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9184 in map-loop1969 in k9023 in k8993 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9186,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9161(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9248 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9250,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_length(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9259,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* optimizer.scm:1249: scheme#< */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t3;
av2[2]=C_fix(0);
av2[3]=t2;
av2[4]=C_fix(3);
C_lessp(5,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9257 in k9248 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_9259,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_list2(&a,2,C_SCHEME_FALSE,t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9274,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t3,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1251: chicken.compiler.support#varnode */
t5=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9272 in k9257 in k9248 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_9274,c,av);}
a=C_alloc(9);
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9282,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1254: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k9280 in k9272 in k9257 in k9248 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9282(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_9282,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9286,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=C_i_cdr(((C_word*)t0)[7]);
if(C_truep(C_i_nullp(t3))){
/* optimizer.scm:1256: chicken.compiler.support#varnode */
t4=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_i_caddr(((C_word*)t0)[8]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cadr(((C_word*)t0)[7]);
f_9286(2,av2);}}}

/* k9284 in k9280 in k9272 in k9257 in k9248 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_9286,c,av);}
a=C_alloc(15);
t2=C_a_i_list5(&a,5,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);
/* optimizer.scm:1250: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[6];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[7];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9337 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_9339,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_not(t2);
t4=(C_truep(t3)?t3:C_i_nequalp(C_i_length(((C_word*)t0)[3]),C_i_car(((C_word*)t0)[2])));
if(C_truep(t4)){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=C_a_i_list2(&a,2,C_SCHEME_TRUE,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9362,a[2]=((C_word*)t0)[4],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9366,a[2]=t7,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1269: chicken.compiler.support#varnode */
t9=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9360 in k9337 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9362(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9362,c,av);}
/* optimizer.scm:1268: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k9364 in k9337 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9366(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9366,c,av);}
a=C_alloc(6);
/* optimizer.scm:1269: cons* */
f_3834(((C_word*)t0)[2],t1,C_a_i_list(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k9398 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_9400,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_i_cadr(((C_word*)t0)[2]);
t3=(C_truep(t2)?t2:*((C_word*)lf[232]+1));
if(C_truep(t3)){
t4=C_i_length(((C_word*)t0)[3]);
t5=C_i_caddr(((C_word*)t0)[2]);
if(C_truep(C_i_less_or_equalp(t4,t5))){
t6=C_eqp(t4,C_fix(1));
if(C_truep(t6)){
t7=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t8=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[3]);
/* optimizer.scm:1282: chicken.compiler.support#make-node */
t9=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[30];
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t7=C_i_car(((C_word*)t0)[2]);
t8=C_a_i_list2(&a,2,C_SCHEME_TRUE,t7);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9446,a[2]=((C_word*)t0)[5],a[3]=t8,tmp=(C_word)a,a+=4,tmp);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9450,a[2]=t9,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1284: chicken.compiler.support#varnode */
t11=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=C_i_car(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}}
else{
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9444 in k9398 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9446,c,av);}
/* optimizer.scm:1283: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k9448 in k9398 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9450(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9450,c,av);}
a=C_alloc(6);
/* optimizer.scm:1284: cons* */
f_3834(((C_word*)t0)[2],t1,C_a_i_list(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k9477 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_9479,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(C_truep(t2)?t2:*((C_word*)lf[232]+1));
if(C_truep(t3)){
t4=(
/* optimizer.scm:1293: argc-ok? */
  f_8303(((C_word*)t0)[3],C_i_car(((C_word*)t0)[2]))
);
if(C_truep(t4)){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9504,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=t5,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[7]))){
t7=C_u_i_cdr(((C_word*)t0)[7]);
t8=t6;
f_9504(t8,C_a_i_cons(&a,2,C_SCHEME_TRUE,t7));}
else{
t7=t6;
f_9504(t7,((C_word*)t0)[7]);}}
else{
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9502 in k9477 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9504(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_9504,2,t0,t1);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9508,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9512,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_list2(&a,2,((C_word*)t0)[5],C_SCHEME_TRUE);
/* optimizer.scm:1296: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[238];
av2[3]=t4;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k9506 in k9502 in k9477 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9508,c,av);}
/* optimizer.scm:1295: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k9510 in k9502 in k9477 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9512,c,av);}
a=C_alloc(6);
/* optimizer.scm:1296: cons* */
f_3834(((C_word*)t0)[2],t1,C_a_i_list(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k9547 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_9549,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_eqp(*((C_word*)lf[233]+1),t2);
if(C_truep(t3)){
t4=C_i_cadddr(((C_word*)t0)[2]);
t5=(C_truep(t4)?t4:*((C_word*)lf[232]+1));
if(C_truep(t5)){
t6=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9579,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t6,tmp=(C_word)a,a+=5,tmp);
t8=(C_truep(*((C_word*)lf[232]+1))?C_i_cadddr(((C_word*)t0)[2]):C_i_caddr(((C_word*)t0)[2]));
t9=C_a_i_list1(&a,1,t8);
/* optimizer.scm:1309: chicken.compiler.support#make-node */
t10=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t7;
av2[2]=lf[58];
av2[3]=t9;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9577 in k9547 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9579,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1306: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9630 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9632(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_9632,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_eqp(*((C_word*)lf[233]+1),t2);
if(C_truep(t3)){
t4=C_i_caddr(((C_word*)t0)[2]);
t5=C_a_i_list2(&a,2,C_SCHEME_TRUE,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9649,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9653,a[2]=t6,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1323: chicken.compiler.support#varnode */
t8=*((C_word*)lf[83]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_i_caddr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_eqp(*((C_word*)lf[233]+1),t4);
if(C_truep(t5)){
t6=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t7=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* optimizer.scm:1325: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=t6;
av2[4]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9647 in k9630 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9649,c,av);}
/* optimizer.scm:1322: chicken.compiler.support#make-node */
t2=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k9651 in k9630 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9653,c,av);}
a=C_alloc(6);
/* optimizer.scm:1323: cons* */
f_3834(((C_word*)t0)[2],t1,C_a_i_list(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]));}

/* k9731 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_9733,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=*((C_word*)lf[232]+1);
t3=(C_truep(*((C_word*)lf[232]+1))?*((C_word*)lf[232]+1):((C_word*)t0)[2]);
if(C_truep(t3)){
t4=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9754,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9762,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t5,a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t7=(C_truep(((C_word*)t0)[8])?(C_truep(C_i_fixnum_positivep(((C_word*)t0)[6]))?C_fixnum_less_or_equal_p(((C_word*)t0)[6],C_fix(8)):C_SCHEME_FALSE):C_SCHEME_FALSE);
if(C_truep(t7)){
/* optimizer.scm:1353: chicken.string#conc */
t8=*((C_word*)lf[239]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t6;
av2[2]=C_i_cadr(((C_word*)t0)[9]);
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t8=t6;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_i_cadr(((C_word*)t0)[9]);
f_9762(2,av2);}}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9752 in k9731 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9754,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1347: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9760 in k9731 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(68,c,4)))){
C_save_and_reclaim((void *)f_9762,c,av);}
a=C_alloc(68);
t2=C_eqp(C_SCHEME_TRUE,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_a_i_fixnum_plus(&a,2,((C_word*)t0)[3],C_fix(1));
t4=C_a_i_list2(&a,2,t1,t3);
/* optimizer.scm:1350: chicken.compiler.support#make-node */
t5=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[121];
av2[3]=t4;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_i_cadr(((C_word*)t0)[2]);
t5=C_s_a_i_times(&a,2,((C_word*)t0)[3],t4);
t6=C_s_a_i_plus(&a,2,t3,t5);
t7=C_a_i_list2(&a,2,t1,t6);
/* optimizer.scm:1350: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[121];
av2[3]=t7;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t3=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* optimizer.scm:1350: chicken.compiler.support#make-node */
t4=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[121];
av2[3]=t3;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}}

/* k9843 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_9845,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9860,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[232]+1))){
t4=C_i_cddr(((C_word*)t0)[4]);
t5=C_i_pairp(t4);
t6=(C_truep(t5)?C_i_caddr(((C_word*)t0)[4]):C_i_cadr(((C_word*)t0)[4]));
t7=C_a_i_list1(&a,1,t6);
/* optimizer.scm:1369: chicken.compiler.support#make-node */
t8=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t3;
av2[2]=lf[58];
av2[3]=t7;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t4=C_i_cadr(((C_word*)t0)[4]);
t5=C_a_i_list1(&a,1,t4);
/* optimizer.scm:1369: chicken.compiler.support#make-node */
t6=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=lf[58];
av2[3]=t5;
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9858 in k9843 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9860,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1366: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9911 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_9913,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9928,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1380: chicken.compiler.support#qnode */
t4=*((C_word*)lf[77]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9926 in k9911 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9928,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* optimizer.scm:1380: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9947,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_i_length(((C_word*)t0)[2]);
if(C_truep(C_fixnum_greaterp(t2,C_fix(2)))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9956,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* optimizer.scm:1387: scheme#reverse */
t4=*((C_word*)lf[4]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_14115(2,av2);}}}

/* k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_9956,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=C_i_cadr(t1);
t4=C_u_i_cdr(t1);
t5=C_u_i_cdr(t4);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9972,a[2]=((C_word*)t0)[2],a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_9972(t9,((C_word*)t0)[3],t2,t3,t5,((C_word*)t0)[4]);}

/* lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_fcall f_9972(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_9972,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t4))){
t6=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9994,a[2]=t5,a[3]=t3,a[4]=t2,a[5]=t1,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* optimizer.scm:1395: chicken.compiler.support#varnode */
t8=*((C_word*)lf[83]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9997,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=t5,a[7]=t2,a[8]=((C_word*)t0)[2],tmp=(C_word)a,a+=9,tmp);
/* optimizer.scm:1396: chicken.base#gensym */
t7=*((C_word*)lf[116]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[241];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k9992 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9994(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_9994,c,av);}
a=C_alloc(12);
t2=C_a_i_list4(&a,4,t1,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
/* optimizer.scm:1393: chicken.compiler.support#make-node */
t3=*((C_word*)lf[33]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[30];
av2[3]=((C_word*)t0)[6];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9995 in lp in k9954 in k9945 in loop in k14098 in a14091 in k5044 in k4691 in k4687 in k4683 in k4679 in k3397 in k3394 in k3391 in k3388 in k3385 */
static void C_ccall f_9997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_9997,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10000,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* optimizer.scm:1397: chicken.base#gensym */
t3=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[240];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_optimizer_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("optimizer"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_optimizer_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(1260))){
C_save(t1);
C_rereclaim2(1260*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,245);
lf[0]=C_h_intern(&lf[0],9, C_text("optimizer"));
lf[1]=C_h_intern(&lf[1],27, C_text("chicken.compiler.optimizer#"));
lf[4]=C_h_intern(&lf[4],14, C_text("scheme#reverse"));
lf[6]=C_h_intern(&lf[6],5, C_text("foldr"));
lf[7]=C_h_intern(&lf[7],13, C_text("scheme#append"));
lf[8]=C_h_intern(&lf[8],3, C_text("map"));
lf[10]=C_h_intern(&lf[10],12, C_text("scheme#null\077"));
lf[17]=C_h_intern(&lf[17],45, C_text("chicken.compiler.optimizer#eq-inline-operator"));
lf[18]=C_h_intern(&lf[18],52, C_text("chicken.compiler.optimizer#membership-test-operators"));
lf[19]=C_h_intern(&lf[19],50, C_text("chicken.compiler.optimizer#membership-unfold-limit"));
lf[20]=C_h_intern(&lf[20],54, C_text("chicken.compiler.optimizer#default-optimization-passes"));
lf[21]=C_h_intern(&lf[21],52, C_text("chicken.compiler.optimizer#scan-toplevel-assignments"));
lf[22]=C_h_intern(&lf[22],26, C_text("chicken.base#alist-update!"));
lf[23]=C_h_intern(&lf[23],8, C_text("for-each"));
lf[24]=C_h_intern(&lf[24],15, C_text("##core#variable"));
lf[25]=C_h_intern(&lf[25],2, C_text("if"));
lf[26]=C_h_intern(&lf[26],3, C_text("let"));
lf[27]=C_h_intern(&lf[27],20, C_text("chicken.base#butlast"));
lf[28]=C_h_intern(&lf[28],6, C_text("lambda"));
lf[29]=C_h_intern(&lf[29],13, C_text("##core#lambda"));
lf[30]=C_h_intern(&lf[30],11, C_text("##core#call"));
lf[31]=C_h_intern(&lf[31],4, C_text("set!"));
lf[32]=C_h_intern(&lf[32],35, C_text("chicken.compiler.support#copy-node!"));
lf[33]=C_h_intern(&lf[33],34, C_text("chicken.compiler.support#make-node"));
lf[34]=C_h_intern(&lf[34],16, C_text("##core#undefined"));
lf[35]=C_h_intern(&lf[35],34, C_text("chicken.compiler.support#debugging"));
lf[36]=C_h_intern(&lf[36],1, C_text("o"));
lf[37]=C_decode_literal(C_heaptop,C_text("\376B\000\000&dropping redundant toplevel assignment"));
lf[38]=C_h_intern(&lf[38],22, C_text("chicken.base#alist-ref"));
lf[39]=C_h_intern(&lf[39],11, C_text("##core#cond"));
lf[40]=C_h_intern(&lf[40],13, C_text("##core#switch"));
lf[41]=C_h_intern(&lf[41],35, C_text("chicken.compiler.support#node-class"));
lf[42]=C_h_intern(&lf[42],44, C_text("chicken.compiler.support#node-subexpressions"));
lf[43]=C_h_intern(&lf[43],40, C_text("chicken.compiler.support#node-parameters"));
lf[44]=C_h_intern(&lf[44],38, C_text("chicken.compiler.support#mark-variable"));
lf[45]=C_h_intern(&lf[45],23, C_text("##compiler#always-bound"));
lf[46]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014safe globals"));
lf[47]=C_h_intern(&lf[47],10, C_text("scheme#eq\077"));
lf[48]=C_h_intern(&lf[48],1, C_text("p"));
lf[49]=C_decode_literal(C_heaptop,C_text("\376B\000\000 scanning toplevel assignments..."));
lf[54]=C_h_intern(&lf[54],59, C_text("chicken.compiler.optimizer#perform-high-level-optimizations"));
lf[55]=C_h_intern(&lf[55],31, C_text("chicken.compiler.support#db-get"));
lf[56]=C_h_intern(&lf[56],35, C_text("chicken.compiler.support#match-node"));
lf[57]=C_h_intern(&lf[57],31, C_text("chicken.internal#hash-table-ref"));
lf[58]=C_h_intern(&lf[58],13, C_text("##core#inline"));
lf[59]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001o\376\003\000\000\002\376\001\000\000\001\001x\376\377\016"));
lf[60]=C_decode_literal(C_heaptop,C_text("\376B\000\000!known list op on rest arg sublist"));
lf[61]=C_h_intern(&lf[61],34, C_text("chicken.compiler.support#call-info"));
lf[62]=C_h_intern(&lf[62],8, C_text("assigned"));
lf[63]=C_h_intern(&lf[63],8, C_text("rest-cdr"));
lf[64]=C_h_intern(&lf[64],15, C_text("consed-rest-arg"));
lf[65]=C_h_intern(&lf[65],8, C_text("captured"));
lf[66]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_i_car"));
lf[67]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_u_i_car"));
lf[68]=C_h_intern(&lf[68],15, C_text("##core#rest-car"));
lf[69]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007C_i_cdr"));
lf[70]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_u_i_cdr"));
lf[71]=C_h_intern(&lf[71],15, C_text("##core#rest-cdr"));
lf[72]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011C_i_nullp"));
lf[73]=C_h_intern(&lf[73],17, C_text("##core#rest-null\077"));
lf[74]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376B\000\000\012C_i_length\376\003\000\000\002\376B\000\000\014C_u_i_length\376\377\016"));
lf[75]=C_h_intern(&lf[75],18, C_text("##core#rest-length"));
lf[76]=C_h_intern(&lf[76],5, C_text("quote"));
lf[77]=C_h_intern(&lf[77],30, C_text("chicken.compiler.support#qnode"));
lf[78]=C_h_intern(&lf[78],49, C_text("chicken.compiler.support#maybe-constant-fold-call"));
lf[79]=C_h_intern(&lf[79],32, C_text("chicken.compiler.support#db-put!"));
lf[80]=C_h_intern(&lf[80],10, C_text("replacable"));
lf[81]=C_h_intern(&lf[81],5, C_text("value"));
lf[82]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035substituted constant variable"));
lf[83]=C_h_intern(&lf[83],32, C_text("chicken.compiler.support#varnode"));
lf[84]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032propagated global variable"));
lf[85]=C_h_intern(&lf[85],2, C_text("no"));
lf[86]=C_h_intern(&lf[86],38, C_text("chicken.compiler.support#variable-mark"));
lf[87]=C_h_intern(&lf[87],17, C_text("##compiler#inline"));
lf[88]=C_h_intern(&lf[88],11, C_text("collapsable"));
lf[89]=C_h_intern(&lf[89],6, C_text("global"));
lf[90]=C_h_intern(&lf[90],9, C_text("replacing"));
lf[91]=C_h_intern(&lf[91],12, C_text("contractable"));
lf[92]=C_h_intern(&lf[92],9, C_text("removable"));
lf[93]=C_h_intern(&lf[93],6, C_text("unused"));
lf[94]=C_h_intern(&lf[94],42, C_text("chicken.compiler.support#build-lambda-list"));
lf[95]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047merged explicitly consed rest parameter"));
lf[96]=C_h_intern(&lf[96],13, C_text("explicit-rest"));
lf[97]=C_decode_literal(C_heaptop,C_text("\376B\000\000 removed unused formal parameters"));
lf[98]=C_h_intern(&lf[98],27, C_text("##sys#decompose-lambda-list"));
lf[99]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047merged explicitly consed rest parameter"));
lf[100]=C_h_intern(&lf[100],21, C_text("has-unused-parameters"));
lf[101]=C_h_intern(&lf[101],20, C_text("##core#direct_lambda"));
lf[102]=C_h_intern(&lf[102],13, C_text("inline-target"));
lf[103]=C_h_intern(&lf[103],47, C_text("chicken.compiler.support#inline-lambda-bindings"));
lf[104]=C_h_intern(&lf[104],17, C_text("chicken.base#void"));
lf[105]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024contracted procedure"));
lf[106]=C_h_intern(&lf[106],1, C_text("i"));
lf[107]=C_decode_literal(C_heaptop,C_text("\376B\000\000>not contracting procedure because argument list does not match"));
lf[108]=C_h_intern(&lf[108],40, C_text("chicken.compiler.support#check-signature"));
lf[109]=C_decode_literal(C_heaptop,C_text("\376B\000\0001removed call to pure procedure with unused result"));
lf[110]=C_decode_literal(C_heaptop,C_text("\376B\000\0008not inlining procedure because it refers to contractable"));
lf[111]=C_h_intern(&lf[111],20, C_text("chicken.base#call/cc"));
lf[112]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022inlining procedure"));
lf[113]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017global inlining"));
lf[114]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010inlining"));
lf[115]=C_decode_literal(C_heaptop,C_text("\376B\000\000;not inlining procedure because argument list does not match"));
lf[116]=C_h_intern(&lf[116],19, C_text("chicken.base#gensym"));
lf[117]=C_h_intern(&lf[117],1, C_text("t"));
lf[118]=C_h_intern(&lf[118],53, C_text("chicken.compiler.support#expression-has-side-effects\077"));
lf[119]=C_decode_literal(C_heaptop,C_text("\376B\000\000+removed unused parameter to known procedure"));
lf[120]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012C_a_i_list"));
lf[121]=C_h_intern(&lf[121],22, C_text("##core#inline_allocate"));
lf[122]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042consed rest parameter at call site"));
lf[123]=C_h_intern(&lf[123],37, C_text("chicken.compiler.support#llist-length"));
lf[124]=C_decode_literal(C_heaptop,C_text("\376B\000\000(not inlining as unroll-limit is exceeded"));
lf[125]=C_h_intern(&lf[125],9, C_text("inlinable"));
lf[126]=C_h_intern(&lf[126],30, C_text("chicken.compiler.support#node\077"));
lf[127]=C_h_intern(&lf[127],24, C_text("##compiler#inline-global"));
lf[128]=C_decode_literal(C_heaptop,C_text("\376B\000\000 inlining call to intrinsic alias"));
lf[129]=C_h_intern(&lf[129],35, C_text("chicken.compiler.support#intrinsic\077"));
lf[130]=C_h_intern(&lf[130],10, C_text("references"));
lf[131]=C_h_intern(&lf[131],7, C_text("unknown"));
lf[132]=C_h_intern(&lf[132],15, C_text("##compiler#pure"));
lf[133]=C_h_intern(&lf[133],11, C_text("local-value"));
lf[134]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034removing global contractable"));
lf[135]=C_decode_literal(C_heaptop,C_text("\376B\000\0006removed side-effect free assignment to unused variable"));
lf[136]=C_h_intern(&lf[136],16, C_text("inline-transient"));
lf[137]=C_h_intern(&lf[137],42, C_text("chicken.compiler.support#variable-visible\077"));
lf[138]=C_h_intern(&lf[138],54, C_text("chicken.compiler.support#replace-rest-op-with-list-ops"));
lf[139]=C_decode_literal(C_heaptop,C_text("\376B\000\0006resetting rest op for explicitly consed rest parameter"));
lf[140]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031removed conditional forms"));
lf[141]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025removed binding forms"));
lf[142]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022replaced variables"));
lf[143]=C_h_intern(&lf[143],18, C_text("chicken.base#print"));
lf[144]=C_h_intern(&lf[144],14, C_text("scheme#newline"));
lf[145]=C_h_intern(&lf[145],19, C_text("chicken.base#print\052"));
lf[146]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004    "));
lf[147]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027  call simplifications:"));
lf[148]=C_h_intern(&lf[148],46, C_text("chicken.compiler.support#with-debugging-output"));
lf[149]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017simplifications"));
lf[150]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022traversal phase..."));
lf[151]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023Removed `not\047 forms"));
lf[152]=C_h_intern(&lf[152],49, C_text("chicken.compiler.support#node-subexpressions-set!"));
lf[153]=C_h_intern(&lf[153],45, C_text("chicken.compiler.support#node-parameters-set!"));
lf[154]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\006\001\376\377\016"));
lf[155]=C_h_intern(&lf[155],36, C_text("chicken.compiler.support#db-get-list"));
lf[156]=C_h_intern(&lf[156],3, C_text("not"));
lf[157]=C_h_intern(&lf[157],10, C_text("call-sites"));
lf[158]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031pre-optimization phase..."));
lf[160]=C_h_intern(&lf[160],34, C_text("chicken.compiler.optimizer#rewrite"));
lf[161]=C_h_intern(&lf[161],32, C_text("chicken.internal#hash-table-set!"));
lf[162]=C_h_intern(&lf[162],52, C_text("chicken.compiler.optimizer#transform-direct-lambdas!"));
lf[163]=C_h_intern(&lf[163],26, C_text("##compiler#callback-lambda"));
lf[164]=C_h_intern(&lf[164],5, C_text("boxed"));
lf[165]=C_h_intern(&lf[165],44, C_text("chicken.compiler.support#scan-used-variables"));
lf[166]=C_h_intern(&lf[166],17, C_text("##core#inline_ref"));
lf[167]=C_h_intern(&lf[167],53, C_text("chicken.compiler.support#estimate-foreign-result-size"));
lf[168]=C_h_intern(&lf[168],21, C_text("##core#inline_loc_ref"));
lf[169]=C_h_intern(&lf[169],18, C_text("##core#direct_call"));
lf[170]=C_h_intern(&lf[170],39, C_text("chicken.compiler.support#quit-compiling"));
lf[171]=C_decode_literal(C_heaptop,C_text("\376B\000\000;known procedure called with wrong number of arguments: `~A\047"));
lf[172]=C_h_intern(&lf[172],40, C_text("chicken.compiler.support#node-class-set!"));
lf[173]=C_h_intern(&lf[173],14, C_text("##core#recurse"));
lf[174]=C_h_intern(&lf[174],29, C_text("chicken.compiler.support#bomb"));
lf[175]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014missing kvar"));
lf[176]=C_h_intern(&lf[176],13, C_text("##core#return"));
lf[177]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017bad call (leaf)"));
lf[178]=C_h_intern(&lf[178],13, C_text("scheme#cdaddr"));
lf[179]=C_h_intern(&lf[179],13, C_text("scheme#caaddr"));
lf[180]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026invalid parameter list"));
lf[181]=C_decode_literal(C_heaptop,C_text("\376B\000\0006direct leaf routine with hoistable closures/allocation"));
lf[182]=C_h_intern(&lf[182],18, C_text("##sys#make-promise"));
lf[183]=C_h_intern(&lf[183],10, C_text("##sys#list"));
lf[184]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036direct leaf routine/allocation"));
lf[185]=C_decode_literal(C_heaptop,C_text("\376B\000\000(direct leaf routine optimization pass..."));
lf[186]=C_h_intern(&lf[186],54, C_text("chicken.compiler.optimizer#determine-loop-and-dispatch"));
lf[187]=C_h_intern(&lf[187],18, C_text("##sys#dynamic-wind"));
lf[188]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\006\001\376\377\016"));
lf[189]=C_h_intern(&lf[189],2, C_text("f_"));
lf[190]=C_h_intern(&lf[190],1, C_text("x"));
lf[191]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012clustering"));
lf[192]=C_h_intern(&lf[192],1, C_text("a"));
lf[193]=C_h_intern(&lf[193],10, C_text("scheme#max"));
lf[194]=C_h_intern(&lf[194],1, C_text("k"));
lf[195]=C_h_intern(&lf[195],8, C_text("dispatch"));
lf[196]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027collecting clusters ..."));
lf[197]=C_h_intern(&lf[197],18, C_text("scheme#make-vector"));
lf[198]=C_h_intern(&lf[198],3, C_text("var"));
lf[199]=C_h_intern(&lf[199],2, C_text("d2"));
lf[200]=C_h_intern(&lf[200],1, C_text("y"));
lf[201]=C_h_intern(&lf[201],2, C_text("d3"));
lf[202]=C_h_intern(&lf[202],1, C_text("z"));
lf[203]=C_h_intern(&lf[203],2, C_text("d1"));
lf[204]=C_h_intern(&lf[204],2, C_text("op"));
lf[205]=C_h_intern(&lf[205],5, C_text("clist"));
lf[206]=C_h_intern(&lf[206],4, C_text("var1"));
lf[207]=C_h_intern(&lf[207],4, C_text("var0"));
lf[208]=C_h_intern(&lf[208],6, C_text("const1"));
lf[209]=C_h_intern(&lf[209],4, C_text("var2"));
lf[210]=C_h_intern(&lf[210],6, C_text("const2"));
lf[211]=C_h_intern(&lf[211],5, C_text("body2"));
lf[212]=C_h_intern(&lf[212],4, C_text("rest"));
lf[213]=C_h_intern(&lf[213],5, C_text("body1"));
lf[214]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\001\000\000\000\002\376\377\016"));
lf[215]=C_h_intern(&lf[215],35, C_text("chicken.compiler.support#immediate\077"));
lf[216]=C_h_intern(&lf[216],5, C_text("const"));
lf[217]=C_h_intern(&lf[217],1, C_text("n"));
lf[218]=C_h_intern(&lf[218],7, C_text("clauses"));
lf[219]=C_h_intern(&lf[219],1, C_text("d"));
lf[220]=C_h_intern(&lf[220],4, C_text("body"));
lf[221]=C_h_intern(&lf[221],4, C_text("more"));
lf[222]=C_decode_literal(C_heaptop,C_text("\376B\000\000!converted assignments to bindings"));
lf[223]=C_h_intern(&lf[223],29, C_text("chicken.sort#topological-sort"));
lf[224]=C_h_intern(&lf[224],4, C_text("args"));
lf[225]=C_h_intern(&lf[225],3, C_text("op1"));
lf[226]=C_h_intern(&lf[226],5, C_text("args1"));
lf[227]=C_h_intern(&lf[227],4, C_text("kvar"));
lf[228]=C_h_intern(&lf[228],3, C_text("op2"));
lf[229]=C_h_intern(&lf[229],5, C_text("args2"));
lf[230]=C_h_intern(&lf[230],1, C_text("b"));
lf[231]=C_h_intern(&lf[231],1, C_text("c"));
lf[232]=C_h_intern(&lf[232],31, C_text("chicken.compiler.support#unsafe"));
lf[233]=C_h_intern(&lf[233],36, C_text("chicken.compiler.support#number-type"));
lf[234]=C_h_intern(&lf[234],7, C_text("generic"));
lf[235]=C_h_intern(&lf[235],6, C_text("fixnum"));
lf[236]=C_h_intern(&lf[236],37, C_text("chicken.compiler.support#fold-boolean"));
lf[237]=C_h_intern(&lf[237],6, C_text("flonum"));
lf[238]=C_h_intern(&lf[238],11, C_text("##core#proc"));
lf[239]=C_h_intern(&lf[239],19, C_text("chicken.string#conc"));
lf[240]=C_h_intern(&lf[240],2, C_text("va"));
lf[241]=C_h_intern(&lf[241],1, C_text("r"));
lf[242]=C_h_intern(&lf[242],35, C_text("chicken.compiler.support#fold-inner"));
lf[243]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023bad type (optimize)"));
lf[244]=C_h_intern(&lf[244],27, C_text("chicken.base#make-parameter"));
C_register_lf2(lf,245,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_3387,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_library_toplevel(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[830] = {
{C_text("f_10000:optimizer_2escm"),(void*)f_10000},
{C_text("f_10015:optimizer_2escm"),(void*)f_10015},
{C_text("f_10019:optimizer_2escm"),(void*)f_10019},
{C_text("f_10029:optimizer_2escm"),(void*)f_10029},
{C_text("f_10041:optimizer_2escm"),(void*)f_10041},
{C_text("f_10053:optimizer_2escm"),(void*)f_10053},
{C_text("f_10057:optimizer_2escm"),(void*)f_10057},
{C_text("f_10099:optimizer_2escm"),(void*)f_10099},
{C_text("f_10114:optimizer_2escm"),(void*)f_10114},
{C_text("f_10122:optimizer_2escm"),(void*)f_10122},
{C_text("f_10124:optimizer_2escm"),(void*)f_10124},
{C_text("f_10132:optimizer_2escm"),(void*)f_10132},
{C_text("f_10144:optimizer_2escm"),(void*)f_10144},
{C_text("f_10171:optimizer_2escm"),(void*)f_10171},
{C_text("f_10186:optimizer_2escm"),(void*)f_10186},
{C_text("f_10207:optimizer_2escm"),(void*)f_10207},
{C_text("f_10243:optimizer_2escm"),(void*)f_10243},
{C_text("f_10245:optimizer_2escm"),(void*)f_10245},
{C_text("f_10280:optimizer_2escm"),(void*)f_10280},
{C_text("f_10302:optimizer_2escm"),(void*)f_10302},
{C_text("f_10306:optimizer_2escm"),(void*)f_10306},
{C_text("f_10342:optimizer_2escm"),(void*)f_10342},
{C_text("f_10366:optimizer_2escm"),(void*)f_10366},
{C_text("f_10406:optimizer_2escm"),(void*)f_10406},
{C_text("f_10426:optimizer_2escm"),(void*)f_10426},
{C_text("f_10430:optimizer_2escm"),(void*)f_10430},
{C_text("f_10434:optimizer_2escm"),(void*)f_10434},
{C_text("f_10436:optimizer_2escm"),(void*)f_10436},
{C_text("f_10442:optimizer_2escm"),(void*)f_10442},
{C_text("f_10450:optimizer_2escm"),(void*)f_10450},
{C_text("f_10456:optimizer_2escm"),(void*)f_10456},
{C_text("f_10476:optimizer_2escm"),(void*)f_10476},
{C_text("f_10480:optimizer_2escm"),(void*)f_10480},
{C_text("f_10503:optimizer_2escm"),(void*)f_10503},
{C_text("f_10576:optimizer_2escm"),(void*)f_10576},
{C_text("f_10579:optimizer_2escm"),(void*)f_10579},
{C_text("f_10583:optimizer_2escm"),(void*)f_10583},
{C_text("f_10586:optimizer_2escm"),(void*)f_10586},
{C_text("f_10589:optimizer_2escm"),(void*)f_10589},
{C_text("f_10604:optimizer_2escm"),(void*)f_10604},
{C_text("f_10635:optimizer_2escm"),(void*)f_10635},
{C_text("f_10641:optimizer_2escm"),(void*)f_10641},
{C_text("f_10647:optimizer_2escm"),(void*)f_10647},
{C_text("f_10695:optimizer_2escm"),(void*)f_10695},
{C_text("f_10703:optimizer_2escm"),(void*)f_10703},
{C_text("f_10707:optimizer_2escm"),(void*)f_10707},
{C_text("f_10733:optimizer_2escm"),(void*)f_10733},
{C_text("f_10749:optimizer_2escm"),(void*)f_10749},
{C_text("f_10761:optimizer_2escm"),(void*)f_10761},
{C_text("f_10771:optimizer_2escm"),(void*)f_10771},
{C_text("f_10790:optimizer_2escm"),(void*)f_10790},
{C_text("f_10793:optimizer_2escm"),(void*)f_10793},
{C_text("f_10797:optimizer_2escm"),(void*)f_10797},
{C_text("f_10800:optimizer_2escm"),(void*)f_10800},
{C_text("f_10803:optimizer_2escm"),(void*)f_10803},
{C_text("f_10842:optimizer_2escm"),(void*)f_10842},
{C_text("f_10860:optimizer_2escm"),(void*)f_10860},
{C_text("f_10876:optimizer_2escm"),(void*)f_10876},
{C_text("f_10897:optimizer_2escm"),(void*)f_10897},
{C_text("f_10931:optimizer_2escm"),(void*)f_10931},
{C_text("f_10947:optimizer_2escm"),(void*)f_10947},
{C_text("f_10951:optimizer_2escm"),(void*)f_10951},
{C_text("f_10956:optimizer_2escm"),(void*)f_10956},
{C_text("f_10986:optimizer_2escm"),(void*)f_10986},
{C_text("f_10990:optimizer_2escm"),(void*)f_10990},
{C_text("f_10995:optimizer_2escm"),(void*)f_10995},
{C_text("f_11037:optimizer_2escm"),(void*)f_11037},
{C_text("f_11042:optimizer_2escm"),(void*)f_11042},
{C_text("f_11079:optimizer_2escm"),(void*)f_11079},
{C_text("f_11083:optimizer_2escm"),(void*)f_11083},
{C_text("f_11090:optimizer_2escm"),(void*)f_11090},
{C_text("f_11094:optimizer_2escm"),(void*)f_11094},
{C_text("f_11122:optimizer_2escm"),(void*)f_11122},
{C_text("f_11155:optimizer_2escm"),(void*)f_11155},
{C_text("f_11166:optimizer_2escm"),(void*)f_11166},
{C_text("f_11179:optimizer_2escm"),(void*)f_11179},
{C_text("f_11212:optimizer_2escm"),(void*)f_11212},
{C_text("f_11219:optimizer_2escm"),(void*)f_11219},
{C_text("f_11221:optimizer_2escm"),(void*)f_11221},
{C_text("f_11225:optimizer_2escm"),(void*)f_11225},
{C_text("f_11229:optimizer_2escm"),(void*)f_11229},
{C_text("f_11235:optimizer_2escm"),(void*)f_11235},
{C_text("f_11241:optimizer_2escm"),(void*)f_11241},
{C_text("f_11247:optimizer_2escm"),(void*)f_11247},
{C_text("f_11250:optimizer_2escm"),(void*)f_11250},
{C_text("f_11256:optimizer_2escm"),(void*)f_11256},
{C_text("f_11259:optimizer_2escm"),(void*)f_11259},
{C_text("f_11260:optimizer_2escm"),(void*)f_11260},
{C_text("f_11267:optimizer_2escm"),(void*)f_11267},
{C_text("f_11270:optimizer_2escm"),(void*)f_11270},
{C_text("f_11276:optimizer_2escm"),(void*)f_11276},
{C_text("f_11291:optimizer_2escm"),(void*)f_11291},
{C_text("f_11333:optimizer_2escm"),(void*)f_11333},
{C_text("f_11339:optimizer_2escm"),(void*)f_11339},
{C_text("f_11348:optimizer_2escm"),(void*)f_11348},
{C_text("f_11351:optimizer_2escm"),(void*)f_11351},
{C_text("f_11357:optimizer_2escm"),(void*)f_11357},
{C_text("f_11360:optimizer_2escm"),(void*)f_11360},
{C_text("f_11368:optimizer_2escm"),(void*)f_11368},
{C_text("f_11375:optimizer_2escm"),(void*)f_11375},
{C_text("f_11379:optimizer_2escm"),(void*)f_11379},
{C_text("f_11387:optimizer_2escm"),(void*)f_11387},
{C_text("f_11392:optimizer_2escm"),(void*)f_11392},
{C_text("f_11415:optimizer_2escm"),(void*)f_11415},
{C_text("f_11442:optimizer_2escm"),(void*)f_11442},
{C_text("f_11446:optimizer_2escm"),(void*)f_11446},
{C_text("f_11450:optimizer_2escm"),(void*)f_11450},
{C_text("f_11454:optimizer_2escm"),(void*)f_11454},
{C_text("f_11458:optimizer_2escm"),(void*)f_11458},
{C_text("f_11475:optimizer_2escm"),(void*)f_11475},
{C_text("f_11484:optimizer_2escm"),(void*)f_11484},
{C_text("f_11494:optimizer_2escm"),(void*)f_11494},
{C_text("f_11507:optimizer_2escm"),(void*)f_11507},
{C_text("f_11518:optimizer_2escm"),(void*)f_11518},
{C_text("f_11520:optimizer_2escm"),(void*)f_11520},
{C_text("f_11534:optimizer_2escm"),(void*)f_11534},
{C_text("f_11538:optimizer_2escm"),(void*)f_11538},
{C_text("f_11541:optimizer_2escm"),(void*)f_11541},
{C_text("f_11544:optimizer_2escm"),(void*)f_11544},
{C_text("f_11559:optimizer_2escm"),(void*)f_11559},
{C_text("f_11562:optimizer_2escm"),(void*)f_11562},
{C_text("f_11587:optimizer_2escm"),(void*)f_11587},
{C_text("f_11590:optimizer_2escm"),(void*)f_11590},
{C_text("f_11608:optimizer_2escm"),(void*)f_11608},
{C_text("f_11618:optimizer_2escm"),(void*)f_11618},
{C_text("f_11621:optimizer_2escm"),(void*)f_11621},
{C_text("f_11624:optimizer_2escm"),(void*)f_11624},
{C_text("f_11635:optimizer_2escm"),(void*)f_11635},
{C_text("f_11647:optimizer_2escm"),(void*)f_11647},
{C_text("f_11655:optimizer_2escm"),(void*)f_11655},
{C_text("f_11659:optimizer_2escm"),(void*)f_11659},
{C_text("f_11682:optimizer_2escm"),(void*)f_11682},
{C_text("f_11685:optimizer_2escm"),(void*)f_11685},
{C_text("f_11707:optimizer_2escm"),(void*)f_11707},
{C_text("f_11732:optimizer_2escm"),(void*)f_11732},
{C_text("f_11747:optimizer_2escm"),(void*)f_11747},
{C_text("f_11757:optimizer_2escm"),(void*)f_11757},
{C_text("f_11776:optimizer_2escm"),(void*)f_11776},
{C_text("f_11786:optimizer_2escm"),(void*)f_11786},
{C_text("f_11805:optimizer_2escm"),(void*)f_11805},
{C_text("f_11845:optimizer_2escm"),(void*)f_11845},
{C_text("f_11847:optimizer_2escm"),(void*)f_11847},
{C_text("f_11855:optimizer_2escm"),(void*)f_11855},
{C_text("f_11857:optimizer_2escm"),(void*)f_11857},
{C_text("f_11867:optimizer_2escm"),(void*)f_11867},
{C_text("f_11870:optimizer_2escm"),(void*)f_11870},
{C_text("f_11872:optimizer_2escm"),(void*)f_11872},
{C_text("f_11875:optimizer_2escm"),(void*)f_11875},
{C_text("f_11919:optimizer_2escm"),(void*)f_11919},
{C_text("f_11923:optimizer_2escm"),(void*)f_11923},
{C_text("f_11925:optimizer_2escm"),(void*)f_11925},
{C_text("f_11929:optimizer_2escm"),(void*)f_11929},
{C_text("f_11932:optimizer_2escm"),(void*)f_11932},
{C_text("f_11935:optimizer_2escm"),(void*)f_11935},
{C_text("f_11956:optimizer_2escm"),(void*)f_11956},
{C_text("f_11970:optimizer_2escm"),(void*)f_11970},
{C_text("f_11987:optimizer_2escm"),(void*)f_11987},
{C_text("f_12039:optimizer_2escm"),(void*)f_12039},
{C_text("f_12043:optimizer_2escm"),(void*)f_12043},
{C_text("f_12047:optimizer_2escm"),(void*)f_12047},
{C_text("f_12051:optimizer_2escm"),(void*)f_12051},
{C_text("f_12055:optimizer_2escm"),(void*)f_12055},
{C_text("f_12059:optimizer_2escm"),(void*)f_12059},
{C_text("f_12069:optimizer_2escm"),(void*)f_12069},
{C_text("f_12087:optimizer_2escm"),(void*)f_12087},
{C_text("f_12093:optimizer_2escm"),(void*)f_12093},
{C_text("f_12100:optimizer_2escm"),(void*)f_12100},
{C_text("f_12110:optimizer_2escm"),(void*)f_12110},
{C_text("f_12119:optimizer_2escm"),(void*)f_12119},
{C_text("f_12131:optimizer_2escm"),(void*)f_12131},
{C_text("f_12141:optimizer_2escm"),(void*)f_12141},
{C_text("f_12164:optimizer_2escm"),(void*)f_12164},
{C_text("f_12167:optimizer_2escm"),(void*)f_12167},
{C_text("f_12168:optimizer_2escm"),(void*)f_12168},
{C_text("f_12176:optimizer_2escm"),(void*)f_12176},
{C_text("f_12179:optimizer_2escm"),(void*)f_12179},
{C_text("f_12183:optimizer_2escm"),(void*)f_12183},
{C_text("f_12193:optimizer_2escm"),(void*)f_12193},
{C_text("f_12205:optimizer_2escm"),(void*)f_12205},
{C_text("f_12210:optimizer_2escm"),(void*)f_12210},
{C_text("f_12219:optimizer_2escm"),(void*)f_12219},
{C_text("f_12226:optimizer_2escm"),(void*)f_12226},
{C_text("f_12230:optimizer_2escm"),(void*)f_12230},
{C_text("f_12232:optimizer_2escm"),(void*)f_12232},
{C_text("f_12257:optimizer_2escm"),(void*)f_12257},
{C_text("f_12268:optimizer_2escm"),(void*)f_12268},
{C_text("f_12272:optimizer_2escm"),(void*)f_12272},
{C_text("f_12278:optimizer_2escm"),(void*)f_12278},
{C_text("f_12281:optimizer_2escm"),(void*)f_12281},
{C_text("f_12284:optimizer_2escm"),(void*)f_12284},
{C_text("f_12287:optimizer_2escm"),(void*)f_12287},
{C_text("f_12290:optimizer_2escm"),(void*)f_12290},
{C_text("f_12293:optimizer_2escm"),(void*)f_12293},
{C_text("f_12294:optimizer_2escm"),(void*)f_12294},
{C_text("f_12298:optimizer_2escm"),(void*)f_12298},
{C_text("f_12299:optimizer_2escm"),(void*)f_12299},
{C_text("f_12313:optimizer_2escm"),(void*)f_12313},
{C_text("f_12317:optimizer_2escm"),(void*)f_12317},
{C_text("f_12321:optimizer_2escm"),(void*)f_12321},
{C_text("f_12325:optimizer_2escm"),(void*)f_12325},
{C_text("f_12329:optimizer_2escm"),(void*)f_12329},
{C_text("f_12333:optimizer_2escm"),(void*)f_12333},
{C_text("f_12341:optimizer_2escm"),(void*)f_12341},
{C_text("f_12351:optimizer_2escm"),(void*)f_12351},
{C_text("f_12363:optimizer_2escm"),(void*)f_12363},
{C_text("f_12371:optimizer_2escm"),(void*)f_12371},
{C_text("f_12381:optimizer_2escm"),(void*)f_12381},
{C_text("f_12401:optimizer_2escm"),(void*)f_12401},
{C_text("f_12411:optimizer_2escm"),(void*)f_12411},
{C_text("f_12424:optimizer_2escm"),(void*)f_12424},
{C_text("f_12434:optimizer_2escm"),(void*)f_12434},
{C_text("f_12444:optimizer_2escm"),(void*)f_12444},
{C_text("f_12456:optimizer_2escm"),(void*)f_12456},
{C_text("f_12460:optimizer_2escm"),(void*)f_12460},
{C_text("f_12472:optimizer_2escm"),(void*)f_12472},
{C_text("f_12476:optimizer_2escm"),(void*)f_12476},
{C_text("f_12480:optimizer_2escm"),(void*)f_12480},
{C_text("f_12484:optimizer_2escm"),(void*)f_12484},
{C_text("f_12488:optimizer_2escm"),(void*)f_12488},
{C_text("f_12492:optimizer_2escm"),(void*)f_12492},
{C_text("f_12500:optimizer_2escm"),(void*)f_12500},
{C_text("f_12502:optimizer_2escm"),(void*)f_12502},
{C_text("f_12518:optimizer_2escm"),(void*)f_12518},
{C_text("f_12530:optimizer_2escm"),(void*)f_12530},
{C_text("f_12538:optimizer_2escm"),(void*)f_12538},
{C_text("f_12546:optimizer_2escm"),(void*)f_12546},
{C_text("f_12554:optimizer_2escm"),(void*)f_12554},
{C_text("f_12556:optimizer_2escm"),(void*)f_12556},
{C_text("f_12564:optimizer_2escm"),(void*)f_12564},
{C_text("f_12568:optimizer_2escm"),(void*)f_12568},
{C_text("f_12574:optimizer_2escm"),(void*)f_12574},
{C_text("f_12599:optimizer_2escm"),(void*)f_12599},
{C_text("f_12603:optimizer_2escm"),(void*)f_12603},
{C_text("f_12623:optimizer_2escm"),(void*)f_12623},
{C_text("f_12629:optimizer_2escm"),(void*)f_12629},
{C_text("f_12633:optimizer_2escm"),(void*)f_12633},
{C_text("f_12646:optimizer_2escm"),(void*)f_12646},
{C_text("f_12650:optimizer_2escm"),(void*)f_12650},
{C_text("f_12654:optimizer_2escm"),(void*)f_12654},
{C_text("f_12658:optimizer_2escm"),(void*)f_12658},
{C_text("f_12665:optimizer_2escm"),(void*)f_12665},
{C_text("f_12667:optimizer_2escm"),(void*)f_12667},
{C_text("f_12703:optimizer_2escm"),(void*)f_12703},
{C_text("f_12709:optimizer_2escm"),(void*)f_12709},
{C_text("f_12732:optimizer_2escm"),(void*)f_12732},
{C_text("f_12734:optimizer_2escm"),(void*)f_12734},
{C_text("f_12768:optimizer_2escm"),(void*)f_12768},
{C_text("f_12793:optimizer_2escm"),(void*)f_12793},
{C_text("f_12806:optimizer_2escm"),(void*)f_12806},
{C_text("f_12815:optimizer_2escm"),(void*)f_12815},
{C_text("f_12825:optimizer_2escm"),(void*)f_12825},
{C_text("f_12854:optimizer_2escm"),(void*)f_12854},
{C_text("f_12876:optimizer_2escm"),(void*)f_12876},
{C_text("f_12894:optimizer_2escm"),(void*)f_12894},
{C_text("f_12902:optimizer_2escm"),(void*)f_12902},
{C_text("f_12904:optimizer_2escm"),(void*)f_12904},
{C_text("f_12912:optimizer_2escm"),(void*)f_12912},
{C_text("f_12924:optimizer_2escm"),(void*)f_12924},
{C_text("f_12928:optimizer_2escm"),(void*)f_12928},
{C_text("f_12936:optimizer_2escm"),(void*)f_12936},
{C_text("f_12940:optimizer_2escm"),(void*)f_12940},
{C_text("f_12951:optimizer_2escm"),(void*)f_12951},
{C_text("f_12970:optimizer_2escm"),(void*)f_12970},
{C_text("f_12974:optimizer_2escm"),(void*)f_12974},
{C_text("f_13000:optimizer_2escm"),(void*)f_13000},
{C_text("f_13015:optimizer_2escm"),(void*)f_13015},
{C_text("f_13019:optimizer_2escm"),(void*)f_13019},
{C_text("f_13081:optimizer_2escm"),(void*)f_13081},
{C_text("f_13096:optimizer_2escm"),(void*)f_13096},
{C_text("f_13113:optimizer_2escm"),(void*)f_13113},
{C_text("f_13119:optimizer_2escm"),(void*)f_13119},
{C_text("f_13132:optimizer_2escm"),(void*)f_13132},
{C_text("f_13161:optimizer_2escm"),(void*)f_13161},
{C_text("f_13167:optimizer_2escm"),(void*)f_13167},
{C_text("f_13180:optimizer_2escm"),(void*)f_13180},
{C_text("f_13214:optimizer_2escm"),(void*)f_13214},
{C_text("f_13241:optimizer_2escm"),(void*)f_13241},
{C_text("f_13262:optimizer_2escm"),(void*)f_13262},
{C_text("f_13266:optimizer_2escm"),(void*)f_13266},
{C_text("f_13274:optimizer_2escm"),(void*)f_13274},
{C_text("f_13280:optimizer_2escm"),(void*)f_13280},
{C_text("f_13293:optimizer_2escm"),(void*)f_13293},
{C_text("f_13322:optimizer_2escm"),(void*)f_13322},
{C_text("f_13328:optimizer_2escm"),(void*)f_13328},
{C_text("f_13341:optimizer_2escm"),(void*)f_13341},
{C_text("f_13345:optimizer_2escm"),(void*)f_13345},
{C_text("f_13399:optimizer_2escm"),(void*)f_13399},
{C_text("f_13423:optimizer_2escm"),(void*)f_13423},
{C_text("f_13436:optimizer_2escm"),(void*)f_13436},
{C_text("f_13444:optimizer_2escm"),(void*)f_13444},
{C_text("f_13482:optimizer_2escm"),(void*)f_13482},
{C_text("f_13492:optimizer_2escm"),(void*)f_13492},
{C_text("f_13496:optimizer_2escm"),(void*)f_13496},
{C_text("f_13499:optimizer_2escm"),(void*)f_13499},
{C_text("f_13502:optimizer_2escm"),(void*)f_13502},
{C_text("f_13532:optimizer_2escm"),(void*)f_13532},
{C_text("f_13535:optimizer_2escm"),(void*)f_13535},
{C_text("f_13538:optimizer_2escm"),(void*)f_13538},
{C_text("f_13568:optimizer_2escm"),(void*)f_13568},
{C_text("f_13595:optimizer_2escm"),(void*)f_13595},
{C_text("f_13599:optimizer_2escm"),(void*)f_13599},
{C_text("f_13602:optimizer_2escm"),(void*)f_13602},
{C_text("f_13605:optimizer_2escm"),(void*)f_13605},
{C_text("f_13611:optimizer_2escm"),(void*)f_13611},
{C_text("f_13634:optimizer_2escm"),(void*)f_13634},
{C_text("f_13649:optimizer_2escm"),(void*)f_13649},
{C_text("f_13657:optimizer_2escm"),(void*)f_13657},
{C_text("f_13659:optimizer_2escm"),(void*)f_13659},
{C_text("f_13709:optimizer_2escm"),(void*)f_13709},
{C_text("f_13717:optimizer_2escm"),(void*)f_13717},
{C_text("f_13725:optimizer_2escm"),(void*)f_13725},
{C_text("f_13733:optimizer_2escm"),(void*)f_13733},
{C_text("f_13761:optimizer_2escm"),(void*)f_13761},
{C_text("f_13769:optimizer_2escm"),(void*)f_13769},
{C_text("f_13795:optimizer_2escm"),(void*)f_13795},
{C_text("f_13808:optimizer_2escm"),(void*)f_13808},
{C_text("f_13825:optimizer_2escm"),(void*)f_13825},
{C_text("f_13829:optimizer_2escm"),(void*)f_13829},
{C_text("f_13833:optimizer_2escm"),(void*)f_13833},
{C_text("f_13844:optimizer_2escm"),(void*)f_13844},
{C_text("f_13848:optimizer_2escm"),(void*)f_13848},
{C_text("f_13922:optimizer_2escm"),(void*)f_13922},
{C_text("f_13935:optimizer_2escm"),(void*)f_13935},
{C_text("f_13941:optimizer_2escm"),(void*)f_13941},
{C_text("f_13964:optimizer_2escm"),(void*)f_13964},
{C_text("f_13968:optimizer_2escm"),(void*)f_13968},
{C_text("f_13972:optimizer_2escm"),(void*)f_13972},
{C_text("f_13981:optimizer_2escm"),(void*)f_13981},
{C_text("f_13990:optimizer_2escm"),(void*)f_13990},
{C_text("f_13994:optimizer_2escm"),(void*)f_13994},
{C_text("f_14092:optimizer_2escm"),(void*)f_14092},
{C_text("f_14100:optimizer_2escm"),(void*)f_14100},
{C_text("f_14105:optimizer_2escm"),(void*)f_14105},
{C_text("f_14115:optimizer_2escm"),(void*)f_14115},
{C_text("f_14119:optimizer_2escm"),(void*)f_14119},
{C_text("f_3387:optimizer_2escm"),(void*)f_3387},
{C_text("f_3390:optimizer_2escm"),(void*)f_3390},
{C_text("f_3393:optimizer_2escm"),(void*)f_3393},
{C_text("f_3396:optimizer_2escm"),(void*)f_3396},
{C_text("f_3399:optimizer_2escm"),(void*)f_3399},
{C_text("f_3407:optimizer_2escm"),(void*)f_3407},
{C_text("f_3421:optimizer_2escm"),(void*)f_3421},
{C_text("f_3425:optimizer_2escm"),(void*)f_3425},
{C_text("f_3431:optimizer_2escm"),(void*)f_3431},
{C_text("f_3509:optimizer_2escm"),(void*)f_3509},
{C_text("f_3527:optimizer_2escm"),(void*)f_3527},
{C_text("f_3561:optimizer_2escm"),(void*)f_3561},
{C_text("f_3567:optimizer_2escm"),(void*)f_3567},
{C_text("f_3581:optimizer_2escm"),(void*)f_3581},
{C_text("f_3600:optimizer_2escm"),(void*)f_3600},
{C_text("f_3615:optimizer_2escm"),(void*)f_3615},
{C_text("f_3623:optimizer_2escm"),(void*)f_3623},
{C_text("f_3631:optimizer_2escm"),(void*)f_3631},
{C_text("f_3642:optimizer_2escm"),(void*)f_3642},
{C_text("f_3655:optimizer_2escm"),(void*)f_3655},
{C_text("f_3662:optimizer_2escm"),(void*)f_3662},
{C_text("f_3669:optimizer_2escm"),(void*)f_3669},
{C_text("f_3673:optimizer_2escm"),(void*)f_3673},
{C_text("f_3685:optimizer_2escm"),(void*)f_3685},
{C_text("f_3687:optimizer_2escm"),(void*)f_3687},
{C_text("f_3734:optimizer_2escm"),(void*)f_3734},
{C_text("f_3736:optimizer_2escm"),(void*)f_3736},
{C_text("f_3770:optimizer_2escm"),(void*)f_3770},
{C_text("f_3776:optimizer_2escm"),(void*)f_3776},
{C_text("f_3798:optimizer_2escm"),(void*)f_3798},
{C_text("f_3804:optimizer_2escm"),(void*)f_3804},
{C_text("f_3810:optimizer_2escm"),(void*)f_3810},
{C_text("f_3820:optimizer_2escm"),(void*)f_3820},
{C_text("f_3834:optimizer_2escm"),(void*)f_3834},
{C_text("f_3840:optimizer_2escm"),(void*)f_3840},
{C_text("f_3854:optimizer_2escm"),(void*)f_3854},
{C_text("f_3890:optimizer_2escm"),(void*)f_3890},
{C_text("f_3896:optimizer_2escm"),(void*)f_3896},
{C_text("f_3909:optimizer_2escm"),(void*)f_3909},
{C_text("f_3923:optimizer_2escm"),(void*)f_3923},
{C_text("f_3931:optimizer_2escm"),(void*)f_3931},
{C_text("f_3971:optimizer_2escm"),(void*)f_3971},
{C_text("f_3985:optimizer_2escm"),(void*)f_3985},
{C_text("f_3998:optimizer_2escm"),(void*)f_3998},
{C_text("f_4010:optimizer_2escm"),(void*)f_4010},
{C_text("f_4019:optimizer_2escm"),(void*)f_4019},
{C_text("f_4027:optimizer_2escm"),(void*)f_4027},
{C_text("f_4034:optimizer_2escm"),(void*)f_4034},
{C_text("f_4048:optimizer_2escm"),(void*)f_4048},
{C_text("f_4063:optimizer_2escm"),(void*)f_4063},
{C_text("f_4071:optimizer_2escm"),(void*)f_4071},
{C_text("f_4075:optimizer_2escm"),(void*)f_4075},
{C_text("f_4079:optimizer_2escm"),(void*)f_4079},
{C_text("f_4097:optimizer_2escm"),(void*)f_4097},
{C_text("f_4136:optimizer_2escm"),(void*)f_4136},
{C_text("f_4176:optimizer_2escm"),(void*)f_4176},
{C_text("f_4235:optimizer_2escm"),(void*)f_4235},
{C_text("f_4412:optimizer_2escm"),(void*)f_4412},
{C_text("f_4418:optimizer_2escm"),(void*)f_4418},
{C_text("f_4432:optimizer_2escm"),(void*)f_4432},
{C_text("f_4436:optimizer_2escm"),(void*)f_4436},
{C_text("f_4448:optimizer_2escm"),(void*)f_4448},
{C_text("f_4466:optimizer_2escm"),(void*)f_4466},
{C_text("f_4681:optimizer_2escm"),(void*)f_4681},
{C_text("f_4685:optimizer_2escm"),(void*)f_4685},
{C_text("f_4689:optimizer_2escm"),(void*)f_4689},
{C_text("f_4693:optimizer_2escm"),(void*)f_4693},
{C_text("f_4695:optimizer_2escm"),(void*)f_4695},
{C_text("f_4698:optimizer_2escm"),(void*)f_4698},
{C_text("f_4721:optimizer_2escm"),(void*)f_4721},
{C_text("f_4726:optimizer_2escm"),(void*)f_4726},
{C_text("f_4728:optimizer_2escm"),(void*)f_4728},
{C_text("f_4733:optimizer_2escm"),(void*)f_4733},
{C_text("f_4735:optimizer_2escm"),(void*)f_4735},
{C_text("f_4751:optimizer_2escm"),(void*)f_4751},
{C_text("f_4761:optimizer_2escm"),(void*)f_4761},
{C_text("f_4774:optimizer_2escm"),(void*)f_4774},
{C_text("f_4778:optimizer_2escm"),(void*)f_4778},
{C_text("f_4781:optimizer_2escm"),(void*)f_4781},
{C_text("f_4784:optimizer_2escm"),(void*)f_4784},
{C_text("f_4796:optimizer_2escm"),(void*)f_4796},
{C_text("f_4800:optimizer_2escm"),(void*)f_4800},
{C_text("f_4802:optimizer_2escm"),(void*)f_4802},
{C_text("f_4844:optimizer_2escm"),(void*)f_4844},
{C_text("f_4847:optimizer_2escm"),(void*)f_4847},
{C_text("f_4870:optimizer_2escm"),(void*)f_4870},
{C_text("f_4881:optimizer_2escm"),(void*)f_4881},
{C_text("f_4885:optimizer_2escm"),(void*)f_4885},
{C_text("f_4918:optimizer_2escm"),(void*)f_4918},
{C_text("f_4921:optimizer_2escm"),(void*)f_4921},
{C_text("f_4924:optimizer_2escm"),(void*)f_4924},
{C_text("f_4942:optimizer_2escm"),(void*)f_4942},
{C_text("f_4949:optimizer_2escm"),(void*)f_4949},
{C_text("f_4992:optimizer_2escm"),(void*)f_4992},
{C_text("f_4995:optimizer_2escm"),(void*)f_4995},
{C_text("f_4998:optimizer_2escm"),(void*)f_4998},
{C_text("f_5011:optimizer_2escm"),(void*)f_5011},
{C_text("f_5021:optimizer_2escm"),(void*)f_5021},
{C_text("f_5042:optimizer_2escm"),(void*)f_5042},
{C_text("f_5046:optimizer_2escm"),(void*)f_5046},
{C_text("f_5051:optimizer_2escm"),(void*)f_5051},
{C_text("f_5054:optimizer_2escm"),(void*)f_5054},
{C_text("f_5068:optimizer_2escm"),(void*)f_5068},
{C_text("f_5078:optimizer_2escm"),(void*)f_5078},
{C_text("f_5080:optimizer_2escm"),(void*)f_5080},
{C_text("f_5084:optimizer_2escm"),(void*)f_5084},
{C_text("f_5098:optimizer_2escm"),(void*)f_5098},
{C_text("f_5121:optimizer_2escm"),(void*)f_5121},
{C_text("f_5125:optimizer_2escm"),(void*)f_5125},
{C_text("f_5128:optimizer_2escm"),(void*)f_5128},
{C_text("f_5136:optimizer_2escm"),(void*)f_5136},
{C_text("f_5146:optimizer_2escm"),(void*)f_5146},
{C_text("f_5152:optimizer_2escm"),(void*)f_5152},
{C_text("f_5191:optimizer_2escm"),(void*)f_5191},
{C_text("f_5205:optimizer_2escm"),(void*)f_5205},
{C_text("f_5207:optimizer_2escm"),(void*)f_5207},
{C_text("f_5247:optimizer_2escm"),(void*)f_5247},
{C_text("f_5249:optimizer_2escm"),(void*)f_5249},
{C_text("f_5256:optimizer_2escm"),(void*)f_5256},
{C_text("f_5268:optimizer_2escm"),(void*)f_5268},
{C_text("f_5304:optimizer_2escm"),(void*)f_5304},
{C_text("f_5326:optimizer_2escm"),(void*)f_5326},
{C_text("f_5336:optimizer_2escm"),(void*)f_5336},
{C_text("f_5348:optimizer_2escm"),(void*)f_5348},
{C_text("f_5352:optimizer_2escm"),(void*)f_5352},
{C_text("f_5356:optimizer_2escm"),(void*)f_5356},
{C_text("f_5360:optimizer_2escm"),(void*)f_5360},
{C_text("f_5364:optimizer_2escm"),(void*)f_5364},
{C_text("f_5368:optimizer_2escm"),(void*)f_5368},
{C_text("f_5372:optimizer_2escm"),(void*)f_5372},
{C_text("f_5376:optimizer_2escm"),(void*)f_5376},
{C_text("f_5380:optimizer_2escm"),(void*)f_5380},
{C_text("f_5405:optimizer_2escm"),(void*)f_5405},
{C_text("f_5409:optimizer_2escm"),(void*)f_5409},
{C_text("f_5411:optimizer_2escm"),(void*)f_5411},
{C_text("f_5425:optimizer_2escm"),(void*)f_5425},
{C_text("f_5428:optimizer_2escm"),(void*)f_5428},
{C_text("f_5431:optimizer_2escm"),(void*)f_5431},
{C_text("f_5434:optimizer_2escm"),(void*)f_5434},
{C_text("f_5494:optimizer_2escm"),(void*)f_5494},
{C_text("f_5507:optimizer_2escm"),(void*)f_5507},
{C_text("f_5511:optimizer_2escm"),(void*)f_5511},
{C_text("f_5518:optimizer_2escm"),(void*)f_5518},
{C_text("f_5549:optimizer_2escm"),(void*)f_5549},
{C_text("f_5553:optimizer_2escm"),(void*)f_5553},
{C_text("f_5557:optimizer_2escm"),(void*)f_5557},
{C_text("f_5561:optimizer_2escm"),(void*)f_5561},
{C_text("f_5564:optimizer_2escm"),(void*)f_5564},
{C_text("f_5569:optimizer_2escm"),(void*)f_5569},
{C_text("f_5573:optimizer_2escm"),(void*)f_5573},
{C_text("f_5576:optimizer_2escm"),(void*)f_5576},
{C_text("f_5579:optimizer_2escm"),(void*)f_5579},
{C_text("f_5588:optimizer_2escm"),(void*)f_5588},
{C_text("f_5594:optimizer_2escm"),(void*)f_5594},
{C_text("f_5600:optimizer_2escm"),(void*)f_5600},
{C_text("f_5611:optimizer_2escm"),(void*)f_5611},
{C_text("f_5615:optimizer_2escm"),(void*)f_5615},
{C_text("f_5638:optimizer_2escm"),(void*)f_5638},
{C_text("f_5648:optimizer_2escm"),(void*)f_5648},
{C_text("f_5651:optimizer_2escm"),(void*)f_5651},
{C_text("f_5668:optimizer_2escm"),(void*)f_5668},
{C_text("f_5701:optimizer_2escm"),(void*)f_5701},
{C_text("f_5704:optimizer_2escm"),(void*)f_5704},
{C_text("f_5719:optimizer_2escm"),(void*)f_5719},
{C_text("f_5727:optimizer_2escm"),(void*)f_5727},
{C_text("f_5737:optimizer_2escm"),(void*)f_5737},
{C_text("f_5739:optimizer_2escm"),(void*)f_5739},
{C_text("f_5764:optimizer_2escm"),(void*)f_5764},
{C_text("f_5774:optimizer_2escm"),(void*)f_5774},
{C_text("f_5785:optimizer_2escm"),(void*)f_5785},
{C_text("f_5803:optimizer_2escm"),(void*)f_5803},
{C_text("f_5811:optimizer_2escm"),(void*)f_5811},
{C_text("f_5821:optimizer_2escm"),(void*)f_5821},
{C_text("f_5828:optimizer_2escm"),(void*)f_5828},
{C_text("f_5846:optimizer_2escm"),(void*)f_5846},
{C_text("f_5851:optimizer_2escm"),(void*)f_5851},
{C_text("f_5857:optimizer_2escm"),(void*)f_5857},
{C_text("f_5863:optimizer_2escm"),(void*)f_5863},
{C_text("f_5869:optimizer_2escm"),(void*)f_5869},
{C_text("f_5876:optimizer_2escm"),(void*)f_5876},
{C_text("f_5891:optimizer_2escm"),(void*)f_5891},
{C_text("f_5911:optimizer_2escm"),(void*)f_5911},
{C_text("f_5918:optimizer_2escm"),(void*)f_5918},
{C_text("f_5921:optimizer_2escm"),(void*)f_5921},
{C_text("f_5938:optimizer_2escm"),(void*)f_5938},
{C_text("f_5943:optimizer_2escm"),(void*)f_5943},
{C_text("f_5950:optimizer_2escm"),(void*)f_5950},
{C_text("f_5965:optimizer_2escm"),(void*)f_5965},
{C_text("f_5985:optimizer_2escm"),(void*)f_5985},
{C_text("f_6019:optimizer_2escm"),(void*)f_6019},
{C_text("f_6031:optimizer_2escm"),(void*)f_6031},
{C_text("f_6034:optimizer_2escm"),(void*)f_6034},
{C_text("f_6041:optimizer_2escm"),(void*)f_6041},
{C_text("f_6044:optimizer_2escm"),(void*)f_6044},
{C_text("f_6053:optimizer_2escm"),(void*)f_6053},
{C_text("f_6056:optimizer_2escm"),(void*)f_6056},
{C_text("f_6060:optimizer_2escm"),(void*)f_6060},
{C_text("f_6070:optimizer_2escm"),(void*)f_6070},
{C_text("f_6077:optimizer_2escm"),(void*)f_6077},
{C_text("f_6085:optimizer_2escm"),(void*)f_6085},
{C_text("f_6087:optimizer_2escm"),(void*)f_6087},
{C_text("f_6097:optimizer_2escm"),(void*)f_6097},
{C_text("f_6111:optimizer_2escm"),(void*)f_6111},
{C_text("f_6120:optimizer_2escm"),(void*)f_6120},
{C_text("f_6123:optimizer_2escm"),(void*)f_6123},
{C_text("f_6126:optimizer_2escm"),(void*)f_6126},
{C_text("f_6145:optimizer_2escm"),(void*)f_6145},
{C_text("f_6151:optimizer_2escm"),(void*)f_6151},
{C_text("f_6154:optimizer_2escm"),(void*)f_6154},
{C_text("f_6162:optimizer_2escm"),(void*)f_6162},
{C_text("f_6169:optimizer_2escm"),(void*)f_6169},
{C_text("f_6175:optimizer_2escm"),(void*)f_6175},
{C_text("f_6181:optimizer_2escm"),(void*)f_6181},
{C_text("f_6184:optimizer_2escm"),(void*)f_6184},
{C_text("f_6185:optimizer_2escm"),(void*)f_6185},
{C_text("f_6195:optimizer_2escm"),(void*)f_6195},
{C_text("f_6198:optimizer_2escm"),(void*)f_6198},
{C_text("f_6203:optimizer_2escm"),(void*)f_6203},
{C_text("f_6206:optimizer_2escm"),(void*)f_6206},
{C_text("f_6210:optimizer_2escm"),(void*)f_6210},
{C_text("f_6217:optimizer_2escm"),(void*)f_6217},
{C_text("f_6220:optimizer_2escm"),(void*)f_6220},
{C_text("f_6242:optimizer_2escm"),(void*)f_6242},
{C_text("f_6244:optimizer_2escm"),(void*)f_6244},
{C_text("f_6254:optimizer_2escm"),(void*)f_6254},
{C_text("f_6276:optimizer_2escm"),(void*)f_6276},
{C_text("f_6285:optimizer_2escm"),(void*)f_6285},
{C_text("f_6299:optimizer_2escm"),(void*)f_6299},
{C_text("f_6316:optimizer_2escm"),(void*)f_6316},
{C_text("f_6326:optimizer_2escm"),(void*)f_6326},
{C_text("f_6329:optimizer_2escm"),(void*)f_6329},
{C_text("f_6334:optimizer_2escm"),(void*)f_6334},
{C_text("f_6359:optimizer_2escm"),(void*)f_6359},
{C_text("f_6370:optimizer_2escm"),(void*)f_6370},
{C_text("f_6374:optimizer_2escm"),(void*)f_6374},
{C_text("f_6380:optimizer_2escm"),(void*)f_6380},
{C_text("f_6386:optimizer_2escm"),(void*)f_6386},
{C_text("f_6392:optimizer_2escm"),(void*)f_6392},
{C_text("f_6407:optimizer_2escm"),(void*)f_6407},
{C_text("f_6411:optimizer_2escm"),(void*)f_6411},
{C_text("f_6423:optimizer_2escm"),(void*)f_6423},
{C_text("f_6473:optimizer_2escm"),(void*)f_6473},
{C_text("f_6485:optimizer_2escm"),(void*)f_6485},
{C_text("f_6490:optimizer_2escm"),(void*)f_6490},
{C_text("f_6496:optimizer_2escm"),(void*)f_6496},
{C_text("f_6500:optimizer_2escm"),(void*)f_6500},
{C_text("f_6507:optimizer_2escm"),(void*)f_6507},
{C_text("f_6512:optimizer_2escm"),(void*)f_6512},
{C_text("f_6522:optimizer_2escm"),(void*)f_6522},
{C_text("f_6524:optimizer_2escm"),(void*)f_6524},
{C_text("f_6549:optimizer_2escm"),(void*)f_6549},
{C_text("f_6560:optimizer_2escm"),(void*)f_6560},
{C_text("f_6568:optimizer_2escm"),(void*)f_6568},
{C_text("f_6597:optimizer_2escm"),(void*)f_6597},
{C_text("f_6610:optimizer_2escm"),(void*)f_6610},
{C_text("f_6628:optimizer_2escm"),(void*)f_6628},
{C_text("f_6634:optimizer_2escm"),(void*)f_6634},
{C_text("f_6640:optimizer_2escm"),(void*)f_6640},
{C_text("f_6665:optimizer_2escm"),(void*)f_6665},
{C_text("f_6669:optimizer_2escm"),(void*)f_6669},
{C_text("f_6673:optimizer_2escm"),(void*)f_6673},
{C_text("f_6679:optimizer_2escm"),(void*)f_6679},
{C_text("f_6682:optimizer_2escm"),(void*)f_6682},
{C_text("f_6689:optimizer_2escm"),(void*)f_6689},
{C_text("f_6703:optimizer_2escm"),(void*)f_6703},
{C_text("f_6723:optimizer_2escm"),(void*)f_6723},
{C_text("f_6727:optimizer_2escm"),(void*)f_6727},
{C_text("f_6734:optimizer_2escm"),(void*)f_6734},
{C_text("f_6749:optimizer_2escm"),(void*)f_6749},
{C_text("f_6767:optimizer_2escm"),(void*)f_6767},
{C_text("f_6770:optimizer_2escm"),(void*)f_6770},
{C_text("f_6780:optimizer_2escm"),(void*)f_6780},
{C_text("f_6782:optimizer_2escm"),(void*)f_6782},
{C_text("f_6801:optimizer_2escm"),(void*)f_6801},
{C_text("f_6807:optimizer_2escm"),(void*)f_6807},
{C_text("f_6817:optimizer_2escm"),(void*)f_6817},
{C_text("f_6821:optimizer_2escm"),(void*)f_6821},
{C_text("f_6831:optimizer_2escm"),(void*)f_6831},
{C_text("f_6835:optimizer_2escm"),(void*)f_6835},
{C_text("f_6841:optimizer_2escm"),(void*)f_6841},
{C_text("f_6848:optimizer_2escm"),(void*)f_6848},
{C_text("f_6861:optimizer_2escm"),(void*)f_6861},
{C_text("f_6869:optimizer_2escm"),(void*)f_6869},
{C_text("f_6873:optimizer_2escm"),(void*)f_6873},
{C_text("f_6879:optimizer_2escm"),(void*)f_6879},
{C_text("f_6889:optimizer_2escm"),(void*)f_6889},
{C_text("f_6893:optimizer_2escm"),(void*)f_6893},
{C_text("f_6911:optimizer_2escm"),(void*)f_6911},
{C_text("f_6914:optimizer_2escm"),(void*)f_6914},
{C_text("f_6923:optimizer_2escm"),(void*)f_6923},
{C_text("f_6930:optimizer_2escm"),(void*)f_6930},
{C_text("f_6932:optimizer_2escm"),(void*)f_6932},
{C_text("f_6957:optimizer_2escm"),(void*)f_6957},
{C_text("f_6990:optimizer_2escm"),(void*)f_6990},
{C_text("f_6996:optimizer_2escm"),(void*)f_6996},
{C_text("f_7002:optimizer_2escm"),(void*)f_7002},
{C_text("f_7011:optimizer_2escm"),(void*)f_7011},
{C_text("f_7023:optimizer_2escm"),(void*)f_7023},
{C_text("f_7029:optimizer_2escm"),(void*)f_7029},
{C_text("f_7035:optimizer_2escm"),(void*)f_7035},
{C_text("f_7044:optimizer_2escm"),(void*)f_7044},
{C_text("f_7046:optimizer_2escm"),(void*)f_7046},
{C_text("f_7056:optimizer_2escm"),(void*)f_7056},
{C_text("f_7070:optimizer_2escm"),(void*)f_7070},
{C_text("f_7071:optimizer_2escm"),(void*)f_7071},
{C_text("f_7088:optimizer_2escm"),(void*)f_7088},
{C_text("f_7112:optimizer_2escm"),(void*)f_7112},
{C_text("f_7122:optimizer_2escm"),(void*)f_7122},
{C_text("f_7141:optimizer_2escm"),(void*)f_7141},
{C_text("f_7149:optimizer_2escm"),(void*)f_7149},
{C_text("f_7153:optimizer_2escm"),(void*)f_7153},
{C_text("f_7160:optimizer_2escm"),(void*)f_7160},
{C_text("f_7164:optimizer_2escm"),(void*)f_7164},
{C_text("f_7173:optimizer_2escm"),(void*)f_7173},
{C_text("f_7182:optimizer_2escm"),(void*)f_7182},
{C_text("f_7188:optimizer_2escm"),(void*)f_7188},
{C_text("f_7195:optimizer_2escm"),(void*)f_7195},
{C_text("f_7238:optimizer_2escm"),(void*)f_7238},
{C_text("f_7244:optimizer_2escm"),(void*)f_7244},
{C_text("f_7254:optimizer_2escm"),(void*)f_7254},
{C_text("f_7264:optimizer_2escm"),(void*)f_7264},
{C_text("f_7270:optimizer_2escm"),(void*)f_7270},
{C_text("f_7298:optimizer_2escm"),(void*)f_7298},
{C_text("f_7304:optimizer_2escm"),(void*)f_7304},
{C_text("f_7308:optimizer_2escm"),(void*)f_7308},
{C_text("f_7311:optimizer_2escm"),(void*)f_7311},
{C_text("f_7314:optimizer_2escm"),(void*)f_7314},
{C_text("f_7317:optimizer_2escm"),(void*)f_7317},
{C_text("f_7320:optimizer_2escm"),(void*)f_7320},
{C_text("f_7323:optimizer_2escm"),(void*)f_7323},
{C_text("f_7352:optimizer_2escm"),(void*)f_7352},
{C_text("f_7356:optimizer_2escm"),(void*)f_7356},
{C_text("f_7361:optimizer_2escm"),(void*)f_7361},
{C_text("f_7389:optimizer_2escm"),(void*)f_7389},
{C_text("f_7399:optimizer_2escm"),(void*)f_7399},
{C_text("f_7427:optimizer_2escm"),(void*)f_7427},
{C_text("f_7467:optimizer_2escm"),(void*)f_7467},
{C_text("f_7471:optimizer_2escm"),(void*)f_7471},
{C_text("f_7478:optimizer_2escm"),(void*)f_7478},
{C_text("f_7481:optimizer_2escm"),(void*)f_7481},
{C_text("f_7484:optimizer_2escm"),(void*)f_7484},
{C_text("f_7493:optimizer_2escm"),(void*)f_7493},
{C_text("f_7494:optimizer_2escm"),(void*)f_7494},
{C_text("f_7501:optimizer_2escm"),(void*)f_7501},
{C_text("f_7507:optimizer_2escm"),(void*)f_7507},
{C_text("f_7510:optimizer_2escm"),(void*)f_7510},
{C_text("f_7516:optimizer_2escm"),(void*)f_7516},
{C_text("f_7525:optimizer_2escm"),(void*)f_7525},
{C_text("f_7537:optimizer_2escm"),(void*)f_7537},
{C_text("f_7543:optimizer_2escm"),(void*)f_7543},
{C_text("f_7552:optimizer_2escm"),(void*)f_7552},
{C_text("f_7559:optimizer_2escm"),(void*)f_7559},
{C_text("f_7562:optimizer_2escm"),(void*)f_7562},
{C_text("f_7565:optimizer_2escm"),(void*)f_7565},
{C_text("f_7580:optimizer_2escm"),(void*)f_7580},
{C_text("f_7600:optimizer_2escm"),(void*)f_7600},
{C_text("f_7604:optimizer_2escm"),(void*)f_7604},
{C_text("f_7608:optimizer_2escm"),(void*)f_7608},
{C_text("f_7618:optimizer_2escm"),(void*)f_7618},
{C_text("f_7637:optimizer_2escm"),(void*)f_7637},
{C_text("f_7641:optimizer_2escm"),(void*)f_7641},
{C_text("f_7660:optimizer_2escm"),(void*)f_7660},
{C_text("f_7680:optimizer_2escm"),(void*)f_7680},
{C_text("f_7684:optimizer_2escm"),(void*)f_7684},
{C_text("f_7691:optimizer_2escm"),(void*)f_7691},
{C_text("f_7702:optimizer_2escm"),(void*)f_7702},
{C_text("f_7712:optimizer_2escm"),(void*)f_7712},
{C_text("f_7732:optimizer_2escm"),(void*)f_7732},
{C_text("f_7735:optimizer_2escm"),(void*)f_7735},
{C_text("f_7738:optimizer_2escm"),(void*)f_7738},
{C_text("f_7753:optimizer_2escm"),(void*)f_7753},
{C_text("f_7755:optimizer_2escm"),(void*)f_7755},
{C_text("f_7761:optimizer_2escm"),(void*)f_7761},
{C_text("f_7785:optimizer_2escm"),(void*)f_7785},
{C_text("f_7798:optimizer_2escm"),(void*)f_7798},
{C_text("f_7807:optimizer_2escm"),(void*)f_7807},
{C_text("f_7810:optimizer_2escm"),(void*)f_7810},
{C_text("f_7811:optimizer_2escm"),(void*)f_7811},
{C_text("f_7821:optimizer_2escm"),(void*)f_7821},
{C_text("f_7829:optimizer_2escm"),(void*)f_7829},
{C_text("f_7837:optimizer_2escm"),(void*)f_7837},
{C_text("f_7843:optimizer_2escm"),(void*)f_7843},
{C_text("f_7856:optimizer_2escm"),(void*)f_7856},
{C_text("f_7870:optimizer_2escm"),(void*)f_7870},
{C_text("f_7871:optimizer_2escm"),(void*)f_7871},
{C_text("f_7878:optimizer_2escm"),(void*)f_7878},
{C_text("f_7886:optimizer_2escm"),(void*)f_7886},
{C_text("f_7888:optimizer_2escm"),(void*)f_7888},
{C_text("f_7901:optimizer_2escm"),(void*)f_7901},
{C_text("f_7912:optimizer_2escm"),(void*)f_7912},
{C_text("f_7918:optimizer_2escm"),(void*)f_7918},
{C_text("f_7930:optimizer_2escm"),(void*)f_7930},
{C_text("f_7933:optimizer_2escm"),(void*)f_7933},
{C_text("f_7936:optimizer_2escm"),(void*)f_7936},
{C_text("f_7945:optimizer_2escm"),(void*)f_7945},
{C_text("f_7953:optimizer_2escm"),(void*)f_7953},
{C_text("f_7969:optimizer_2escm"),(void*)f_7969},
{C_text("f_7982:optimizer_2escm"),(void*)f_7982},
{C_text("f_8006:optimizer_2escm"),(void*)f_8006},
{C_text("f_8030:optimizer_2escm"),(void*)f_8030},
{C_text("f_8041:optimizer_2escm"),(void*)f_8041},
{C_text("f_8053:optimizer_2escm"),(void*)f_8053},
{C_text("f_8061:optimizer_2escm"),(void*)f_8061},
{C_text("f_8077:optimizer_2escm"),(void*)f_8077},
{C_text("f_8097:optimizer_2escm"),(void*)f_8097},
{C_text("f_8108:optimizer_2escm"),(void*)f_8108},
{C_text("f_8139:optimizer_2escm"),(void*)f_8139},
{C_text("f_8145:optimizer_2escm"),(void*)f_8145},
{C_text("f_8155:optimizer_2escm"),(void*)f_8155},
{C_text("f_8168:optimizer_2escm"),(void*)f_8168},
{C_text("f_8178:optimizer_2escm"),(void*)f_8178},
{C_text("f_8191:optimizer_2escm"),(void*)f_8191},
{C_text("f_8201:optimizer_2escm"),(void*)f_8201},
{C_text("f_8228:optimizer_2escm"),(void*)f_8228},
{C_text("f_8278:optimizer_2escm"),(void*)f_8278},
{C_text("f_8280:optimizer_2escm"),(void*)f_8280},
{C_text("f_8284:optimizer_2escm"),(void*)f_8284},
{C_text("f_8294:optimizer_2escm"),(void*)f_8294},
{C_text("f_8303:optimizer_2escm"),(void*)f_8303},
{C_text("f_8393:optimizer_2escm"),(void*)f_8393},
{C_text("f_8396:optimizer_2escm"),(void*)f_8396},
{C_text("f_8417:optimizer_2escm"),(void*)f_8417},
{C_text("f_8467:optimizer_2escm"),(void*)f_8467},
{C_text("f_8471:optimizer_2escm"),(void*)f_8471},
{C_text("f_8475:optimizer_2escm"),(void*)f_8475},
{C_text("f_8479:optimizer_2escm"),(void*)f_8479},
{C_text("f_8483:optimizer_2escm"),(void*)f_8483},
{C_text("f_8512:optimizer_2escm"),(void*)f_8512},
{C_text("f_8539:optimizer_2escm"),(void*)f_8539},
{C_text("f_8570:optimizer_2escm"),(void*)f_8570},
{C_text("f_8587:optimizer_2escm"),(void*)f_8587},
{C_text("f_8611:optimizer_2escm"),(void*)f_8611},
{C_text("f_8622:optimizer_2escm"),(void*)f_8622},
{C_text("f_8641:optimizer_2escm"),(void*)f_8641},
{C_text("f_8684:optimizer_2escm"),(void*)f_8684},
{C_text("f_8699:optimizer_2escm"),(void*)f_8699},
{C_text("f_8707:optimizer_2escm"),(void*)f_8707},
{C_text("f_8743:optimizer_2escm"),(void*)f_8743},
{C_text("f_8776:optimizer_2escm"),(void*)f_8776},
{C_text("f_8792:optimizer_2escm"),(void*)f_8792},
{C_text("f_8838:optimizer_2escm"),(void*)f_8838},
{C_text("f_8853:optimizer_2escm"),(void*)f_8853},
{C_text("f_8865:optimizer_2escm"),(void*)f_8865},
{C_text("f_8912:optimizer_2escm"),(void*)f_8912},
{C_text("f_8927:optimizer_2escm"),(void*)f_8927},
{C_text("f_8935:optimizer_2escm"),(void*)f_8935},
{C_text("f_8943:optimizer_2escm"),(void*)f_8943},
{C_text("f_8974:optimizer_2escm"),(void*)f_8974},
{C_text("f_8995:optimizer_2escm"),(void*)f_8995},
{C_text("f_9016:optimizer_2escm"),(void*)f_9016},
{C_text("f_9025:optimizer_2escm"),(void*)f_9025},
{C_text("f_9036:optimizer_2escm"),(void*)f_9036},
{C_text("f_9045:optimizer_2escm"),(void*)f_9045},
{C_text("f_9050:optimizer_2escm"),(void*)f_9050},
{C_text("f_9075:optimizer_2escm"),(void*)f_9075},
{C_text("f_9077:optimizer_2escm"),(void*)f_9077},
{C_text("f_9117:optimizer_2escm"),(void*)f_9117},
{C_text("f_9127:optimizer_2escm"),(void*)f_9127},
{C_text("f_9152:optimizer_2escm"),(void*)f_9152},
{C_text("f_9161:optimizer_2escm"),(void*)f_9161},
{C_text("f_9186:optimizer_2escm"),(void*)f_9186},
{C_text("f_9250:optimizer_2escm"),(void*)f_9250},
{C_text("f_9259:optimizer_2escm"),(void*)f_9259},
{C_text("f_9274:optimizer_2escm"),(void*)f_9274},
{C_text("f_9282:optimizer_2escm"),(void*)f_9282},
{C_text("f_9286:optimizer_2escm"),(void*)f_9286},
{C_text("f_9339:optimizer_2escm"),(void*)f_9339},
{C_text("f_9362:optimizer_2escm"),(void*)f_9362},
{C_text("f_9366:optimizer_2escm"),(void*)f_9366},
{C_text("f_9400:optimizer_2escm"),(void*)f_9400},
{C_text("f_9446:optimizer_2escm"),(void*)f_9446},
{C_text("f_9450:optimizer_2escm"),(void*)f_9450},
{C_text("f_9479:optimizer_2escm"),(void*)f_9479},
{C_text("f_9504:optimizer_2escm"),(void*)f_9504},
{C_text("f_9508:optimizer_2escm"),(void*)f_9508},
{C_text("f_9512:optimizer_2escm"),(void*)f_9512},
{C_text("f_9549:optimizer_2escm"),(void*)f_9549},
{C_text("f_9579:optimizer_2escm"),(void*)f_9579},
{C_text("f_9632:optimizer_2escm"),(void*)f_9632},
{C_text("f_9649:optimizer_2escm"),(void*)f_9649},
{C_text("f_9653:optimizer_2escm"),(void*)f_9653},
{C_text("f_9733:optimizer_2escm"),(void*)f_9733},
{C_text("f_9754:optimizer_2escm"),(void*)f_9754},
{C_text("f_9762:optimizer_2escm"),(void*)f_9762},
{C_text("f_9845:optimizer_2escm"),(void*)f_9845},
{C_text("f_9860:optimizer_2escm"),(void*)f_9860},
{C_text("f_9913:optimizer_2escm"),(void*)f_9913},
{C_text("f_9928:optimizer_2escm"),(void*)f_9928},
{C_text("f_9947:optimizer_2escm"),(void*)f_9947},
{C_text("f_9956:optimizer_2escm"),(void*)f_9956},
{C_text("f_9972:optimizer_2escm"),(void*)f_9972},
{C_text("f_9994:optimizer_2escm"),(void*)f_9994},
{C_text("f_9997:optimizer_2escm"),(void*)f_9997},
{C_text("toplevel:optimizer_2escm"),(void*)C_optimizer_toplevel},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.compiler.optimizer#partition 
o|hiding unexported module binding: chicken.compiler.optimizer#span 
o|hiding unexported module binding: chicken.compiler.optimizer#take 
o|hiding unexported module binding: chicken.compiler.optimizer#drop 
o|hiding unexported module binding: chicken.compiler.optimizer#split-at 
o|hiding unexported module binding: chicken.compiler.optimizer#append-map 
o|hiding unexported module binding: chicken.compiler.optimizer#every 
o|hiding unexported module binding: chicken.compiler.optimizer#any 
o|hiding unexported module binding: chicken.compiler.optimizer#cons* 
o|hiding unexported module binding: chicken.compiler.optimizer#concatenate 
o|hiding unexported module binding: chicken.compiler.optimizer#delete 
o|hiding unexported module binding: chicken.compiler.optimizer#first 
o|hiding unexported module binding: chicken.compiler.optimizer#second 
o|hiding unexported module binding: chicken.compiler.optimizer#third 
o|hiding unexported module binding: chicken.compiler.optimizer#fourth 
o|hiding unexported module binding: chicken.compiler.optimizer#fifth 
o|hiding unexported module binding: chicken.compiler.optimizer#delete-duplicates 
o|hiding unexported module binding: chicken.compiler.optimizer#alist-cons 
o|hiding unexported module binding: chicken.compiler.optimizer#filter 
o|hiding unexported module binding: chicken.compiler.optimizer#filter-map 
o|hiding unexported module binding: chicken.compiler.optimizer#remove 
o|hiding unexported module binding: chicken.compiler.optimizer#unzip1 
o|hiding unexported module binding: chicken.compiler.optimizer#last 
o|hiding unexported module binding: chicken.compiler.optimizer#list-index 
o|hiding unexported module binding: chicken.compiler.optimizer#lset-adjoin/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#lset-difference/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#lset-union/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#lset-intersection/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#list-tabulate 
o|hiding unexported module binding: chicken.compiler.optimizer#lset<=/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#lset=/eq? 
o|hiding unexported module binding: chicken.compiler.optimizer#length+ 
o|hiding unexported module binding: chicken.compiler.optimizer#find 
o|hiding unexported module binding: chicken.compiler.optimizer#find-tail 
o|hiding unexported module binding: chicken.compiler.optimizer#iota 
o|hiding unexported module binding: chicken.compiler.optimizer#make-list 
o|hiding unexported module binding: chicken.compiler.optimizer#posq 
o|hiding unexported module binding: chicken.compiler.optimizer#posv 
o|hiding unexported module binding: chicken.compiler.optimizer#simplifications 
o|hiding unexported module binding: chicken.compiler.optimizer#simplified-ops 
o|hiding unexported module binding: chicken.compiler.optimizer#broken-constant-nodes 
o|hiding unexported module binding: chicken.compiler.optimizer#inline-history 
o|hiding unexported module binding: chicken.compiler.optimizer#within-unrolling-limit 
o|hiding unexported module binding: chicken.compiler.optimizer#perform-pre-optimization! 
o|hiding unexported module binding: chicken.compiler.optimizer#register-simplifications 
o|hiding unexported module binding: chicken.compiler.optimizer#reorganize-recursive-bindings 
o|hiding unexported module binding: chicken.compiler.optimizer#substitution-table 
o|hiding unexported module binding: chicken.compiler.optimizer#simplify-named-call 
S|applied compiler syntax:
S|  chicken.base#o		1
S|  scheme#for-each		20
S|  chicken.base#foldl		4
S|  scheme#map		15
S|  chicken.base#foldr		8
o|eliminated procedure checks: 239 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|eliminated procedure checks: 1 
o|specializations:
o|  1 (scheme#set-car! pair *)
o|  1 (scheme#length list)
o|  1 (chicken.base#sub1 fixnum)
o|  2 (scheme#cddr (pair * pair))
o|  2 (scheme#> fixnum fixnum)
o|  1 (chicken.base#add1 fixnum)
o|  1 (scheme#<= fixnum fixnum)
o|  1 (scheme#positive? fixnum)
o|  1 (scheme#eqv? (or eof null fixnum char boolean symbol keyword) *)
o|  1 (scheme#< fixnum fixnum)
o|  1 (scheme#cdar (pair pair *))
o|  17 (scheme#= fixnum fixnum)
o|  2 (scheme#set-cdr! pair *)
o|  1 (scheme#* fixnum fixnum)
o|  4 (chicken.base#sub1 *)
o|  5 (scheme#zero? *)
o|  9 (chicken.base#add1 *)
o|  68 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  1 (scheme#eqv? * *)
o|  16 (##sys#check-list (or pair list) *)
o|  56 (scheme#cdr pair)
o|  26 (scheme#car pair)
(o e)|safe calls: 1509 
(o e)|assignments to immediate values: 10 
o|safe globals: (chicken.compiler.optimizer#posv chicken.compiler.optimizer#posq chicken.compiler.optimizer#make-list chicken.compiler.optimizer#iota chicken.compiler.optimizer#find-tail chicken.compiler.optimizer#find chicken.compiler.optimizer#length+ chicken.compiler.optimizer#lset=/eq? chicken.compiler.optimizer#lset<=/eq? chicken.compiler.optimizer#list-tabulate chicken.compiler.optimizer#lset-intersection/eq? chicken.compiler.optimizer#lset-union/eq? chicken.compiler.optimizer#lset-difference/eq? chicken.compiler.optimizer#lset-adjoin/eq? chicken.compiler.optimizer#list-index chicken.compiler.optimizer#last chicken.compiler.optimizer#unzip1 chicken.compiler.optimizer#remove chicken.compiler.optimizer#filter-map chicken.compiler.optimizer#filter chicken.compiler.optimizer#alist-cons chicken.compiler.optimizer#delete-duplicates chicken.compiler.optimizer#fifth chicken.compiler.optimizer#fourth chicken.compiler.optimizer#third chicken.compiler.optimizer#second chicken.compiler.optimizer#first chicken.compiler.optimizer#delete chicken.compiler.optimizer#concatenate chicken.compiler.optimizer#cons* chicken.compiler.optimizer#any chicken.compiler.optimizer#every chicken.compiler.optimizer#append-map chicken.compiler.optimizer#split-at chicken.compiler.optimizer#drop chicken.compiler.optimizer#take chicken.compiler.optimizer#span chicken.compiler.optimizer#partition) 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#span 
o|inlining procedure: k3511 
o|inlining procedure: k3511 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#drop 
o|inlining procedure: k3569 
o|inlining procedure: k3569 
o|merged explicitly consed rest parameter: lsts108 
o|inlining procedure: k3602 
o|inlining procedure: k3617 
o|inlining procedure: k3617 
o|inlining procedure: k3602 
o|inlining procedure: k3657 
o|inlining procedure: k3657 
o|inlining procedure: k3689 
o|contracted procedure: "(mini-srfi-1.scm:77) g161170" 
o|inlining procedure: k3689 
o|inlining procedure: k3738 
o|contracted procedure: "(mini-srfi-1.scm:76) g134143" 
o|inlining procedure: k3738 
o|inlining procedure: k3781 
o|inlining procedure: k3781 
o|inlining procedure: k3812 
o|inlining procedure: k3812 
o|merged explicitly consed rest parameter: xs209 
o|inlining procedure: k3842 
o|inlining procedure: k3842 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#concatenate 
o|inlining procedure: k3898 
o|inlining procedure: k3898 
o|inlining procedure: k4029 
o|inlining procedure: k4029 
o|inlining procedure: k4021 
o|inlining procedure: k4021 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#remove 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#list-index 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#lset-difference/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#lset-union/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#lset-intersection/eq? 
o|inlining procedure: k4420 
o|inlining procedure: k4420 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#length+ 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#find 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#find-tail 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#iota 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#make-list 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#posq 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#posv 
o|inlining procedure: k4700 
o|inlining procedure: k4700 
o|inlining procedure: k4737 
o|inlining procedure: k4737 
o|inlining procedure: k4753 
o|inlining procedure: k4753 
o|inlining procedure: k4785 
o|inlining procedure: "(optimizer.scm:89) chicken.compiler.optimizer#first" 
o|inlining procedure: k4785 
o|inlining procedure: "(optimizer.scm:96) chicken.compiler.optimizer#first" 
o|inlining procedure: k4862 
o|contracted procedure: "(optimizer.scm:102) chicken.compiler.optimizer#last" 
o|inlining procedure: k4178 
o|inlining procedure: k4178 
o|inlining procedure: k4862 
o|inlining procedure: k4895 
o|inlining procedure: k4895 
o|inlining procedure: k4925 
o|inlining procedure: k4925 
o|inlining procedure: "(optimizer.scm:110) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:109) chicken.compiler.optimizer#first" 
o|substituted constant variable: a4961 
o|substituted constant variable: a4963 
o|substituted constant variable: a4968 
o|substituted constant variable: a4970 
o|substituted constant variable: a4972 
o|inlining procedure: k4976 
o|inlining procedure: k4976 
o|substituted constant variable: a4983 
o|substituted constant variable: a4985 
o|substituted constant variable: a4987 
o|substituted constant variable: a4989 
o|inlining procedure: k5013 
o|contracted procedure: "(optimizer.scm:131) g566634" 
o|propagated global variable: g642643 chicken.compiler.support#mark-variable 
o|inlining procedure: k5013 
o|contracted procedure: "(optimizer.scm:130) chicken.compiler.optimizer#delete-duplicates" 
o|inlining procedure: k3973 
o|inlining procedure: k3973 
o|inlining procedure: k5100 
o|contracted procedure: "(optimizer.scm:172) g694701" 
o|inlining procedure: k5100 
o|inlining procedure: k5129 
o|inlining procedure: k5129 
o|inlining procedure: k5141 
o|inlining procedure: k5153 
o|inlining procedure: k5153 
o|inlining procedure: k5209 
o|inlining procedure: k5209 
o|inlining procedure: "(optimizer.scm:180) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:179) chicken.compiler.optimizer#first" 
o|inlining procedure: k5141 
o|inlining procedure: "(optimizer.scm:178) chicken.compiler.optimizer#second" 
o|inlining procedure: k5257 
o|inlining procedure: k5257 
o|inlining procedure: k5263 
o|inlining procedure: k5275 
o|inlining procedure: k5287 
o|inlining procedure: k5299 
o|inlining procedure: k5311 
o|inlining procedure: k5321 
o|consed rest parameter at call site: "(optimizer.scm:219) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: k5321 
o|inlining procedure: k5311 
o|inlining procedure: k5299 
o|inlining procedure: k5287 
o|inlining procedure: "(optimizer.scm:207) chicken.compiler.optimizer#first" 
o|inlining procedure: k5275 
o|inlining procedure: "(optimizer.scm:205) chicken.compiler.optimizer#first" 
o|inlining procedure: k5384 
o|inlining procedure: k5384 
o|inlining procedure: k5396 
o|inlining procedure: k5396 
o|inlining procedure: k5263 
o|inlining procedure: k5413 
o|inlining procedure: k5413 
o|inlining procedure: k5432 
o|inlining procedure: k5453 
o|inlining procedure: k5453 
o|contracted procedure: "(optimizer.scm:236) node-value677" 
o|inlining procedure: "(optimizer.scm:168) chicken.compiler.optimizer#first" 
o|contracted procedure: "(optimizer.scm:233) constant-node?676" 
o|inlining procedure: k5432 
o|inlining procedure: k5480 
o|inlining procedure: k5496 
o|contracted procedure: "(optimizer.scm:254) chicken.compiler.optimizer#lset-adjoin/eq?" 
o|inlining procedure: k4237 
o|inlining procedure: k4237 
o|propagated global variable: lst348 chicken.compiler.optimizer#broken-constant-nodes 
o|inlining procedure: k5496 
o|inlining procedure: k5480 
o|substituted constant variable: a5543 
o|substituted constant variable: a5545 
o|substituted constant variable: a5547 
o|inlining procedure: k5554 
o|inlining procedure: k5554 
o|inlining procedure: k5580 
o|inlining procedure: k5616 
o|inlining procedure: k5616 
o|inlining procedure: k5643 
o|inlining procedure: k5643 
o|inlining procedure: "(optimizer.scm:287) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:282) chicken.compiler.optimizer#first" 
o|inlining procedure: k5580 
o|inlining procedure: k5696 
o|inlining procedure: "(optimizer.scm:308) chicken.compiler.optimizer#second" 
o|inlining procedure: k5696 
o|inlining procedure: k5741 
o|inlining procedure: k5741 
o|inlining procedure: "(optimizer.scm:313) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:313) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:311) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:311) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:310) chicken.compiler.optimizer#first" 
o|inlining procedure: k5816 
o|inlining procedure: k5816 
o|inlining procedure: "(optimizer.scm:302) chicken.compiler.optimizer#first" 
o|inlining procedure: k5829 
o|contracted procedure: "(optimizer.scm:325) chicken.compiler.optimizer#partition" 
o|inlining procedure: k3409 
o|inlining procedure: k3409 
o|inlining procedure: "(optimizer.scm:337) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:336) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:330) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:330) chicken.compiler.optimizer#first" 
o|inlining procedure: k5933 
o|inlining procedure: "(optimizer.scm:350) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:349) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:347) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:346) chicken.compiler.optimizer#first" 
o|inlining procedure: k5933 
o|inlining procedure: "(optimizer.scm:320) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:319) chicken.compiler.optimizer#third" 
o|inlining procedure: k5829 
o|inlining procedure: k6008 
o|inlining procedure: k6036 
o|propagated global variable: g983984 chicken.compiler.support#db-put! 
o|inlining procedure: "(optimizer.scm:382) chicken.compiler.optimizer#first" 
o|inlining procedure: k6089 
o|inlining procedure: k6089 
o|inlining procedure: "(optimizer.scm:374) chicken.compiler.optimizer#third" 
o|inlining procedure: k6036 
o|inlining procedure: k6146 
o|inlining procedure: k6170 
o|propagated global variable: g10761077 chicken.compiler.support#db-put! 
o|inlining procedure: "(optimizer.scm:454) chicken.compiler.optimizer#first" 
o|inlining procedure: k6246 
o|inlining procedure: k6246 
o|inlining procedure: "(optimizer.scm:441) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k6170 
o|inlining procedure: k6286 
o|inlining procedure: k6286 
o|inlining procedure: k6301 
o|inlining procedure: k6336 
o|inlining procedure: k6336 
o|inlining procedure: k6301 
o|inlining procedure: k6387 
o|inlining procedure: k6387 
o|inlining procedure: k6465 
o|inlining procedure: k6526 
o|inlining procedure: k6526 
o|inlining procedure: k6566 
o|inlining procedure: k6566 
o|substituted constant variable: a6583 
o|inlining procedure: k6465 
o|inlining procedure: k6605 
o|inlining procedure: k6617 
o|inlining procedure: k6635 
o|inlining procedure: k6635 
o|contracted procedure: "(optimizer.scm:431) chicken.compiler.optimizer#within-unrolling-limit" 
o|inlining procedure: k7432 
o|inlining procedure: k7432 
o|inlining procedure: k7441 
o|inlining procedure: k7441 
o|inlining procedure: k6650 
o|inlining procedure: k6650 
o|inlining procedure: "(optimizer.scm:430) chicken.compiler.optimizer#fourth" 
o|substituted constant variable: a6661 
o|inlining procedure: k6617 
o|inlining procedure: k6605 
o|inlining procedure: "(optimizer.scm:421) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:417) chicken.compiler.optimizer#third" 
o|inlining procedure: k6146 
o|inlining procedure: "(optimizer.scm:526) chicken.compiler.optimizer#first" 
o|inlining procedure: k6707 
o|inlining procedure: "(optimizer.scm:523) chicken.compiler.optimizer#first" 
o|inlining procedure: k6707 
o|inlining procedure: k6738 
o|inlining procedure: k6750 
o|inlining procedure: k6762 
o|propagated global variable: g10201021 chicken.compiler.support#expression-has-side-effects? 
o|inlining procedure: k6789 
o|inlining procedure: k6789 
o|inlining procedure: k6762 
o|inlining procedure: "(optimizer.scm:398) chicken.compiler.optimizer#third" 
o|inlining procedure: k6750 
o|inlining procedure: "(optimizer.scm:394) chicken.compiler.optimizer#first" 
o|inlining procedure: k6738 
o|inlining procedure: k6849 
o|inlining procedure: "(optimizer.scm:371) chicken.compiler.optimizer#first" 
o|inlining procedure: k6849 
o|inlining procedure: k6880 
o|inlining procedure: k6880 
o|inlining procedure: "(optimizer.scm:362) chicken.compiler.optimizer#first" 
o|inlining procedure: k6894 
o|inlining procedure: k6934 
o|inlining procedure: k6934 
o|inlining procedure: "(optimizer.scm:535) chicken.compiler.optimizer#first" 
o|inlining procedure: k6894 
o|substituted constant variable: a6973 
o|substituted constant variable: a6975 
o|inlining procedure: k6008 
o|inlining procedure: k6985 
o|inlining procedure: k6985 
o|inlining procedure: k7018 
o|inlining procedure: k7018 
o|inlining procedure: k7048 
o|inlining procedure: k7048 
o|inlining procedure: k7073 
o|substituted constant variable: a7083 
o|inlining procedure: k7073 
o|inlining procedure: k7090 
o|substituted constant variable: a7100 
o|inlining procedure: k7090 
o|inlining procedure: k7123 
o|inlining procedure: "(optimizer.scm:557) chicken.compiler.optimizer#first" 
o|inlining procedure: k7123 
o|inlining procedure: "(optimizer.scm:544) chicken.compiler.optimizer#first" 
o|inlining procedure: k7165 
o|inlining procedure: "(optimizer.scm:575) chicken.compiler.optimizer#first" 
o|inlining procedure: k7165 
o|inlining procedure: k7205 
o|inlining procedure: k7205 
o|substituted constant variable: a7218 
o|substituted constant variable: a7220 
o|substituted constant variable: a7222 
o|substituted constant variable: a7224 
o|substituted constant variable: a7226 
o|substituted constant variable: a7228 
o|substituted constant variable: a7230 
o|substituted constant variable: a7232 
o|substituted constant variable: a7234 
o|substituted constant variable: a7236 
o|inlining procedure: k7246 
o|inlining procedure: k7246 
o|inlining procedure: k7293 
o|inlining procedure: k7293 
o|inlining procedure: k7391 
o|contracted procedure: "(optimizer.scm:617) g13101317" 
o|inlining procedure: k7362 
o|inlining procedure: k7362 
o|inlining procedure: k7391 
o|propagated global variable: g13161318 chicken.compiler.optimizer#simplified-ops 
o|contracted procedure: "(optimizer.scm:605) chicken.compiler.optimizer#perform-pre-optimization!" 
o|inlining procedure: k7482 
o|inlining procedure: k7482 
o|inlining procedure: k7511 
o|inlining procedure: k7538 
o|inlining procedure: "(optimizer.scm:688) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:685) chicken.compiler.optimizer#first" 
o|inlining procedure: k7538 
o|substituted constant variable: a7619 
o|inlining procedure: "(optimizer.scm:676) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:675) chicken.compiler.optimizer#third" 
o|inlining procedure: k7511 
o|inlining procedure: k7642 
o|substituted constant variable: a7661 
o|inlining procedure: k7642 
o|substituted constant variable: a7666 
o|inlining procedure: "(optimizer.scm:668) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:668) chicken.compiler.optimizer#second" 
o|inlining procedure: k7704 
o|inlining procedure: k7704 
o|merged explicitly consed rest parameter: ss1422 
o|inlining procedure: k10590 
o|inlining procedure: "(optimizer.scm:1549) chicken.compiler.optimizer#first" 
o|inlining procedure: k10615 
o|inlining procedure: k10627 
o|inlining procedure: k10642 
o|inlining procedure: k10651 
o|inlining procedure: k10663 
o|inlining procedure: "(optimizer.scm:1547) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1547) chicken.compiler.optimizer#first" 
o|inlining procedure: k10663 
o|inlining procedure: "(optimizer.scm:1544) chicken.compiler.optimizer#first" 
o|inlining procedure: k10651 
o|inlining procedure: k10642 
o|inlining procedure: k10627 
o|inlining procedure: k10615 
o|inlining procedure: "(optimizer.scm:1535) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1533) chicken.compiler.optimizer#third" 
o|inlining procedure: k10590 
o|inlining procedure: "(optimizer.scm:1550) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1550) chicken.compiler.optimizer#first" 
o|inlining procedure: k10725 
o|inlining procedure: "(optimizer.scm:1553) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1552) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1552) chicken.compiler.optimizer#first" 
o|inlining procedure: k10725 
o|inlining procedure: k10763 
o|inlining procedure: k10763 
o|substituted constant variable: a10784 
o|substituted constant variable: a10786 
o|substituted constant variable: a10788 
o|inlining procedure: k10804 
o|inlining procedure: k10822 
o|inlining procedure: k10822 
o|inlining procedure: "(optimizer.scm:1564) chicken.compiler.optimizer#first" 
o|inlining procedure: k10804 
o|inlining procedure: k10849 
o|inlining procedure: "(optimizer.scm:1577) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1574) chicken.compiler.optimizer#third" 
o|inlining procedure: k10849 
o|inlining procedure: k10877 
o|inlining procedure: "(optimizer.scm:1581) chicken.compiler.optimizer#second" 
o|inlining procedure: k10877 
o|inlining procedure: k10912 
o|inlining procedure: k10918 
o|inlining procedure: k10918 
o|inlining procedure: "(optimizer.scm:1585) chicken.compiler.optimizer#first" 
o|inlining procedure: k10912 
o|inlining procedure: k10936 
o|inlining procedure: k10961 
o|inlining procedure: k10961 
o|inlining procedure: "(optimizer.scm:1590) chicken.compiler.optimizer#second" 
o|inlining procedure: k10936 
o|inlining procedure: k10981 
o|inlining procedure: k10981 
o|inlining procedure: k11000 
o|inlining procedure: k11000 
o|inlining procedure: "(optimizer.scm:1598) chicken.compiler.optimizer#first" 
o|inlining procedure: k11014 
o|inlining procedure: k11029 
o|inlining procedure: k11029 
o|inlining procedure: k11054 
o|inlining procedure: "(optimizer.scm:1613) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1611) chicken.compiler.optimizer#second" 
o|inlining procedure: k11054 
o|inlining procedure: "(optimizer.scm:1608) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1606) chicken.compiler.optimizer#first" 
o|inlining procedure: k11014 
o|inlining procedure: k11105 
o|inlining procedure: k11105 
o|inlining procedure: "(optimizer.scm:1619) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k11127 
o|inlining procedure: "(optimizer.scm:1625) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1625) chicken.compiler.optimizer#first" 
o|inlining procedure: k11127 
o|inlining procedure: k11150 
o|inlining procedure: "(optimizer.scm:1628) chicken.compiler.optimizer#second" 
o|inlining procedure: k11150 
o|inlining procedure: "(optimizer.scm:1627) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1627) chicken.compiler.optimizer#first" 
o|substituted constant variable: a11185 
o|substituted constant variable: a11187 
o|substituted constant variable: a11189 
o|substituted constant variable: a11191 
o|substituted constant variable: a11193 
o|substituted constant variable: a11195 
o|substituted constant variable: a11197 
o|substituted constant variable: a11199 
o|substituted constant variable: a11201 
o|substituted constant variable: a11203 
o|inlining procedure: k11207 
o|contracted procedure: "(optimizer.scm:1634) chicken.compiler.optimizer#lset=/eq?" 
o|inlining procedure: k4456 
o|inlining procedure: k4456 
o|inlining procedure: k11207 
o|inlining procedure: "(optimizer.scm:1713) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1706) chicken.compiler.optimizer#second" 
o|inlining procedure: k11236 
o|inlining procedure: k11394 
o|contracted procedure: "(optimizer.scm:1738) g24742481" 
o|inlining procedure: k11394 
o|inlining procedure: k11417 
o|contracted procedure: "(optimizer.scm:1728) g24642465" 
o|inlining procedure: "(optimizer.scm:1732) chicken.compiler.optimizer#first" 
o|inlining procedure: k11417 
o|inlining procedure: k11486 
o|inlining procedure: k11486 
o|inlining procedure: k11545 
o|inlining procedure: k11569 
o|inlining procedure: "(optimizer.scm:1677) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1675) chicken.compiler.optimizer#first" 
o|inlining procedure: k11605 
o|inlining procedure: k11605 
o|inlining procedure: "(optimizer.scm:1672) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1668) chicken.compiler.optimizer#first" 
o|inlining procedure: k11569 
o|inlining procedure: "(optimizer.scm:1683) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1666) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1662) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1661) chicken.compiler.optimizer#first" 
o|inlining procedure: k11545 
o|inlining procedure: k11720 
o|inlining procedure: "(optimizer.scm:1693) chicken.compiler.optimizer#second" 
o|inlining procedure: k11720 
o|inlining procedure: k11749 
o|inlining procedure: k11749 
o|inlining procedure: "(optimizer.scm:1690) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1689) chicken.compiler.optimizer#first" 
o|inlining procedure: k11778 
o|inlining procedure: k11778 
o|substituted constant variable: a11799 
o|substituted constant variable: a11801 
o|inlining procedure: "(optimizer.scm:1656) chicken.compiler.optimizer#first" 
o|inlining procedure: k11236 
o|inlining procedure: k11816 
o|inlining procedure: k11816 
o|substituted constant variable: a11831 
o|inlining procedure: "(optimizer.scm:1642) chicken.compiler.optimizer#third" 
o|contracted procedure: "(optimizer.scm:1638) chicken.compiler.optimizer#unzip1" 
o|inlining procedure: k4138 
o|contracted procedure: "(mini-srfi-1.scm:143) g312321" 
o|inlining procedure: k4138 
o|inlining procedure: k11877 
o|substituted constant variable: a11899 
o|inlining procedure: k11877 
o|inlining procedure: k11936 
o|inlining procedure: k11965 
o|inlining procedure: k11965 
o|inlining procedure: k11995 
o|inlining procedure: k12013 
o|contracted procedure: "(optimizer.scm:1809) user-lambda?2511" 
o|inlining procedure: k11903 
o|inlining procedure: "(optimizer.scm:1784) chicken.compiler.optimizer#third" 
o|inlining procedure: k11903 
o|inlining procedure: k12013 
o|inlining procedure: "(optimizer.scm:1803) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1802) chicken.compiler.optimizer#first" 
o|inlining procedure: k11995 
o|inlining procedure: "(optimizer.scm:1794) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1793) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1792) chicken.compiler.optimizer#first" 
o|inlining procedure: k11936 
o|inlining procedure: "(optimizer.scm:1825) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1820) chicken.compiler.optimizer#third" 
o|inlining procedure: k12133 
o|inlining procedure: k12133 
o|substituted constant variable: a12157 
o|substituted constant variable: a12159 
o|substituted constant variable: a12161 
o|inlining procedure: "(optimizer.scm:1856) chicken.compiler.optimizer#first" 
o|inlining procedure: k12234 
o|inlining procedure: k12234 
o|inlining procedure: "(optimizer.scm:1851) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1849) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1848) chicken.compiler.optimizer#first" 
o|consed rest parameter at call site: "(optimizer.scm:1940) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: "(optimizer.scm:1946) chicken.compiler.optimizer#second" 
o|inlining procedure: k12373 
o|inlining procedure: k12373 
o|inlining procedure: k12403 
o|inlining procedure: k12403 
o|inlining procedure: k12429 
o|inlining procedure: k12429 
o|consed rest parameter at call site: "(optimizer.scm:1923) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: k12576 
o|inlining procedure: "(optimizer.scm:1911) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k12576 
o|inlining procedure: "(optimizer.scm:1909) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1907) chicken.compiler.optimizer#second" 
o|consed rest parameter at call site: "(optimizer.scm:1905) chicken.compiler.optimizer#append-map" 3 
o|inlining procedure: "(optimizer.scm:1879) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1877) chicken.compiler.optimizer#second" 
o|inlining procedure: k12669 
o|inlining procedure: k12669 
o|inlining procedure: k12736 
o|contracted procedure: "(optimizer.scm:1865) g26932702" 
o|inlining procedure: "(optimizer.scm:1865) chicken.compiler.optimizer#third" 
o|inlining procedure: k12736 
o|inlining procedure: k12770 
o|inlining procedure: k12770 
o|inlining procedure: k12817 
o|inlining procedure: k12817 
o|consed rest parameter at call site: "(optimizer.scm:947) chicken.compiler.optimizer#register-simplifications" 2 
o|inlining procedure: k12859 
o|inlining procedure: k12871 
o|inlining procedure: k12906 
o|inlining procedure: k12906 
o|inlining procedure: k12871 
o|inlining procedure: k12859 
o|inlining procedure: k13002 
o|inlining procedure: k13002 
o|consed rest parameter at call site: "(optimizer.scm:730) chicken.compiler.optimizer#register-simplifications" 2 
o|inlining procedure: k13083 
o|inlining procedure: k13098 
o|inlining procedure: k13098 
o|substituted constant variable: a13172 
o|inlining procedure: k13083 
o|inlining procedure: k13216 
o|inlining procedure: k13225 
o|inlining procedure: k13243 
o|inlining procedure: k13243 
o|substituted constant variable: a13333 
o|inlining procedure: k13225 
o|inlining procedure: k13216 
o|inlining procedure: k13401 
o|substituted constant variable: a13428 
o|inlining procedure: k13401 
o|inlining procedure: k13503 
o|inlining procedure: k13515 
o|inlining procedure: k13539 
o|inlining procedure: "(optimizer.scm:813) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:813) chicken.compiler.optimizer#first" 
o|inlining procedure: k13539 
o|inlining procedure: k13569 
o|inlining procedure: k13606 
o|inlining procedure: "(optimizer.scm:833) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:831) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:831) chicken.compiler.optimizer#first" 
o|inlining procedure: k13606 
o|contracted procedure: "(optimizer.scm:836) chicken.compiler.optimizer#reorganize-recursive-bindings" 
o|inlining procedure: k7763 
o|inlining procedure: k7763 
o|inlining procedure: k7813 
o|inlining procedure: k7845 
o|inlining procedure: k7845 
o|inlining procedure: k7813 
o|inlining procedure: k7890 
o|contracted procedure: "(optimizer.scm:1046) chicken.compiler.optimizer#lset<=/eq?" 
o|inlining procedure: k7890 
o|contracted procedure: "(optimizer.scm:1045) chicken.compiler.optimizer#filter-map" 
o|inlining procedure: k4076 
o|inlining procedure: k4076 
o|inlining procedure: k4065 
o|inlining procedure: k4065 
o|consed rest parameter at call site: "(optimizer.scm:1039) chicken.compiler.optimizer#append-map" 3 
o|inlining procedure: k7937 
o|inlining procedure: k7937 
o|inlining procedure: k7955 
o|inlining procedure: k7977 
o|inlining procedure: k7977 
o|inlining procedure: k8008 
o|contracted procedure: "(optimizer.scm:1066) g18111812" 
o|inlining procedure: k8008 
o|inlining procedure: k8055 
o|inlining procedure: k8055 
o|inlining procedure: k7955 
o|inlining procedure: k8147 
o|inlining procedure: k8147 
o|inlining procedure: k8170 
o|inlining procedure: k8170 
o|inlining procedure: k8193 
o|inlining procedure: k8193 
o|inlining procedure: k8230 
o|inlining procedure: k8230 
o|inlining procedure: k13661 
o|inlining procedure: k13661 
o|inlining procedure: k13667 
o|inlining procedure: k13679 
o|inlining procedure: k13691 
o|inlining procedure: "(optimizer.scm:830) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:830) chicken.compiler.optimizer#first" 
o|inlining procedure: k13691 
o|inlining procedure: "(optimizer.scm:829) chicken.compiler.optimizer#first" 
o|inlining procedure: k13679 
o|inlining procedure: "(optimizer.scm:827) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:826) chicken.compiler.optimizer#first" 
o|inlining procedure: k13667 
o|inlining procedure: "(optimizer.scm:820) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:818) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:817) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:817) chicken.compiler.optimizer#first" 
o|inlining procedure: k13569 
o|substituted constant variable: a13755 
o|substituted constant variable: a13757 
o|inlining procedure: "(optimizer.scm:809) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:808) chicken.compiler.optimizer#first" 
o|inlining procedure: k13515 
o|inlining procedure: "(optimizer.scm:807) chicken.compiler.optimizer#first" 
o|inlining procedure: k13503 
o|inlining procedure: k13797 
o|inlining procedure: k13809 
o|consed rest parameter at call site: "(optimizer.scm:782) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: k13809 
o|substituted constant variable: a13836 
o|inlining procedure: k13797 
o|inlining procedure: k13924 
o|inlining procedure: k13936 
o|inlining procedure: k13948 
o|inlining procedure: k13948 
o|substituted constant variable: a13973 
o|substituted constant variable: a13982 
o|inlining procedure: k13936 
o|inlining procedure: k13924 
o|consed rest parameter at call site: "(optimizer.scm:711) chicken.compiler.optimizer#register-simplifications" 2 
o|inlining procedure: k14107 
o|inlining procedure: k14107 
o|inlining procedure: k14124 
o|inlining procedure: k14124 
o|contracted procedure: "(optimizer.scm:719) chicken.compiler.optimizer#simplify-named-call" 
o|inlining procedure: k8308 
o|inlining procedure: k8308 
o|inlining procedure: k8317 
o|inlining procedure: k8317 
o|inlining procedure: k8382 
o|inlining procedure: k8397 
o|inlining procedure: k8397 
o|inlining procedure: "(optimizer.scm:1127) chicken.compiler.optimizer#second" 
o|inlining procedure: k8435 
o|inlining procedure: k8447 
o|inlining procedure: k8447 
o|inlining procedure: k8435 
o|inlining procedure: "(optimizer.scm:1119) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1118) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1117) chicken.compiler.optimizer#first" 
o|inlining procedure: k8382 
o|inlining procedure: k8498 
o|inlining procedure: k8507 
o|inlining procedure: "(optimizer.scm:1141) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1136) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1135) chicken.compiler.optimizer#third" 
o|inlining procedure: k8507 
o|inlining procedure: "(optimizer.scm:1133) chicken.compiler.optimizer#first" 
o|inlining procedure: k8498 
o|inlining procedure: k8556 
o|inlining procedure: k8565 
o|inlining procedure: k8589 
o|contracted procedure: "(optimizer.scm:1149) g19181919" 
o|inlining procedure: k8589 
o|inlining procedure: "(optimizer.scm:1152) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1148) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1148) chicken.compiler.optimizer#second" 
o|inlining procedure: k8565 
o|inlining procedure: k8556 
o|inlining procedure: k8667 
o|inlining procedure: k8673 
o|inlining procedure: "(optimizer.scm:1166) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1165) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1164) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1162) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1161) chicken.compiler.optimizer#first" 
o|inlining procedure: k8673 
o|substituted constant variable: a8724 
o|inlining procedure: k8667 
o|inlining procedure: k8729 
o|inlining procedure: k8738 
o|inlining procedure: k8750 
o|inlining procedure: "(optimizer.scm:1180) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1179) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1178) chicken.compiler.optimizer#first" 
o|inlining procedure: k8750 
o|inlining procedure: "(optimizer.scm:1174) chicken.compiler.optimizer#third" 
o|substituted constant variable: a8804 
o|inlining procedure: k8738 
o|inlining procedure: k8729 
o|inlining procedure: k8815 
o|inlining procedure: k8827 
o|inlining procedure: "(optimizer.scm:1191) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1190) chicken.compiler.optimizer#first" 
o|inlining procedure: k8827 
o|substituted constant variable: a8878 
o|inlining procedure: k8815 
o|inlining procedure: "(optimizer.scm:1184) chicken.compiler.optimizer#third" 
o|inlining procedure: k8883 
o|inlining procedure: k8898 
o|inlining procedure: k8907 
o|inlining procedure: "(optimizer.scm:1204) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1202) chicken.compiler.optimizer#second" 
o|inlining procedure: k8907 
o|inlining procedure: "(optimizer.scm:1198) chicken.compiler.optimizer#first" 
o|inlining procedure: k8898 
o|inlining procedure: "(optimizer.scm:1196) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k8883 
o|inlining procedure: k8966 
o|inlining procedure: "(optimizer.scm:1210) chicken.compiler.optimizer#first" 
o|inlining procedure: k8966 
o|inlining procedure: k8981 
o|inlining procedure: k8990 
o|inlining procedure: k9017 
o|inlining procedure: k9052 
o|inlining procedure: k9087 
o|inlining procedure: "(optimizer.scm:1233) chicken.compiler.optimizer#first" 
o|inlining procedure: k9087 
o|inlining procedure: "(optimizer.scm:1234) chicken.compiler.optimizer#second" 
o|inlining procedure: k9052 
o|inlining procedure: k9129 
o|inlining procedure: k9129 
o|inlining procedure: k9163 
o|contracted procedure: "(optimizer.scm:1222) g19751984" 
o|inlining procedure: k9163 
o|inlining procedure: k9017 
o|inlining procedure: k9197 
o|inlining procedure: k9197 
o|inlining procedure: "(optimizer.scm:1221) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:1220) chicken.compiler.optimizer#third" 
o|substituted constant variable: a9226 
o|inlining procedure: k8990 
o|inlining procedure: k8981 
o|inlining procedure: k9233 
o|inlining procedure: k9245 
o|inlining procedure: k9284 
o|inlining procedure: "(optimizer.scm:1256) chicken.compiler.optimizer#third" 
o|inlining procedure: k9284 
o|inlining procedure: "(optimizer.scm:1257) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1254) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1253) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1251) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1250) chicken.compiler.optimizer#first" 
o|inlining procedure: k9245 
o|inlining procedure: "(optimizer.scm:1246) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k9233 
o|inlining procedure: k9316 
o|inlining procedure: k9325 
o|inlining procedure: k9343 
o|consed rest parameter at call site: "(optimizer.scm:1269) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: "(optimizer.scm:1269) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1268) chicken.compiler.optimizer#second" 
o|inlining procedure: k9343 
o|inlining procedure: "(optimizer.scm:1267) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1265) chicken.compiler.optimizer#first" 
o|inlining procedure: k9325 
o|inlining procedure: "(optimizer.scm:1263) chicken.compiler.optimizer#third" 
o|inlining procedure: k9316 
o|inlining procedure: k9392 
o|inlining procedure: k9401 
o|inlining procedure: k9419 
o|inlining procedure: k9419 
o|consed rest parameter at call site: "(optimizer.scm:1284) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: "(optimizer.scm:1284) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1283) chicken.compiler.optimizer#first" 
o|substituted constant variable: a9460 
o|inlining procedure: "(optimizer.scm:1280) chicken.compiler.optimizer#third" 
o|inlining procedure: k9401 
o|inlining procedure: "(optimizer.scm:1278) chicken.compiler.optimizer#second" 
o|inlining procedure: k9392 
o|inlining procedure: k9465 
o|inlining procedure: k9474 
o|inlining procedure: k9489 
o|consed rest parameter at call site: "(optimizer.scm:1296) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: "(optimizer.scm:1294) chicken.compiler.optimizer#second" 
o|inlining procedure: k9489 
o|inlining procedure: "(optimizer.scm:1293) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1292) chicken.compiler.optimizer#third" 
o|inlining procedure: k9474 
o|inlining procedure: k9465 
o|inlining procedure: k9535 
o|inlining procedure: k9544 
o|inlining procedure: k9556 
o|inlining procedure: k9585 
o|inlining procedure: "(optimizer.scm:1311) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k9585 
o|inlining procedure: "(optimizer.scm:1311) chicken.compiler.optimizer#third" 
o|inlining procedure: k9556 
o|inlining procedure: "(optimizer.scm:1305) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:1304) chicken.compiler.optimizer#first" 
o|inlining procedure: k9544 
o|inlining procedure: "(optimizer.scm:1302) chicken.compiler.optimizer#second" 
o|inlining procedure: k9535 
o|inlining procedure: k9606 
o|inlining procedure: k9615 
o|inlining procedure: k9627 
o|consed rest parameter at call site: "(optimizer.scm:1323) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: "(optimizer.scm:1323) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1322) chicken.compiler.optimizer#third" 
o|inlining procedure: k9662 
o|inlining procedure: k9662 
o|inlining procedure: "(optimizer.scm:1324) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1321) chicken.compiler.optimizer#first" 
o|inlining procedure: k9627 
o|propagated global variable: tmp20802082 chicken.compiler.support#unsafe 
o|propagated global variable: tmp20802082 chicken.compiler.support#unsafe 
o|inlining procedure: "(optimizer.scm:1319) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k9615 
o|substituted constant variable: a9690 
o|inlining procedure: k9606 
o|inlining procedure: k9716 
o|inlining procedure: k9728 
o|inlining procedure: k9764 
o|inlining procedure: k9764 
o|inlining procedure: "(optimizer.scm:1353) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1354) chicken.compiler.optimizer#second" 
o|inlining procedure: k9803 
o|substituted constant variable: a9811 
o|inlining procedure: k9803 
o|propagated global variable: tmp21012103 chicken.compiler.support#unsafe 
o|propagated global variable: tmp21012103 chicken.compiler.support#unsafe 
o|inlining procedure: k9728 
o|inlining procedure: k9716 
o|inlining procedure: "(optimizer.scm:1341) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:1340) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1338) chicken.compiler.optimizer#first" 
o|inlining procedure: k9825 
o|inlining procedure: k9834 
o|inlining procedure: k9866 
o|inlining procedure: "(optimizer.scm:1371) chicken.compiler.optimizer#third" 
o|inlining procedure: k9866 
o|inlining procedure: "(optimizer.scm:1372) chicken.compiler.optimizer#second" 
o|inlining procedure: k9834 
o|inlining procedure: "(optimizer.scm:1364) chicken.compiler.optimizer#first" 
o|inlining procedure: k9825 
o|inlining procedure: k9899 
o|inlining procedure: k9908 
o|inlining procedure: "(optimizer.scm:1380) chicken.compiler.optimizer#first" 
o|inlining procedure: k9908 
o|inlining procedure: k9899 
o|inlining procedure: k9933 
o|inlining procedure: k9942 
o|inlining procedure: k9974 
o|inlining procedure: k9974 
o|substituted constant variable: a10066 
o|inlining procedure: k9942 
o|inlining procedure: k9933 
o|inlining procedure: k10076 
o|inlining procedure: k10088 
o|inlining procedure: "(optimizer.scm:1425) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1422) chicken.compiler.optimizer#second" 
o|inlining procedure: k10088 
o|inlining procedure: "(optimizer.scm:1416) chicken.compiler.optimizer#first" 
o|inlining procedure: k10076 
o|inlining procedure: "(optimizer.scm:1414) chicken.compiler.optimizer#fourth" 
o|inlining procedure: k10157 
o|inlining procedure: k10166 
o|inlining procedure: k10208 
o|inlining procedure: "(optimizer.scm:1448) chicken.compiler.optimizer#first" 
o|inlining procedure: k10208 
o|inlining procedure: k10247 
o|inlining procedure: k10247 
o|inlining procedure: k10286 
o|inlining procedure: "(optimizer.scm:1444) chicken.compiler.optimizer#first" 
o|inlining procedure: k10286 
o|inlining procedure: "(optimizer.scm:1439) chicken.compiler.optimizer#third" 
o|inlining procedure: "(optimizer.scm:1439) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1438) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:1436) chicken.compiler.optimizer#first" 
o|inlining procedure: k10166 
o|inlining procedure: k10157 
o|inlining procedure: k10328 
o|inlining procedure: k10337 
o|inlining procedure: k10364 
o|inlining procedure: k10364 
o|inlining procedure: "(optimizer.scm:1481) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1470) chicken.compiler.optimizer#third" 
o|inlining procedure: k10337 
o|inlining procedure: k10328 
o|inlining procedure: "(optimizer.scm:1466) chicken.compiler.optimizer#fourth" 
o|inlining procedure: "(optimizer.scm:1464) chicken.compiler.optimizer#first" 
o|inlining procedure: k10392 
o|inlining procedure: k10401 
o|consed rest parameter at call site: "(optimizer.scm:1495) chicken.compiler.optimizer#cons*" 2 
o|inlining procedure: k10458 
o|contracted procedure: "(optimizer.scm:1506) defarg1866" 
o|inlining procedure: k8349 
o|inlining procedure: k8349 
o|inlining procedure: k10458 
o|inlining procedure: "(optimizer.scm:1496) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1494) chicken.compiler.optimizer#second" 
o|inlining procedure: "(optimizer.scm:1492) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:1491) chicken.compiler.optimizer#first" 
o|inlining procedure: k10401 
o|inlining procedure: k10392 
o|substituted constant variable: a10530 
o|substituted constant variable: a10532 
o|substituted constant variable: a10534 
o|substituted constant variable: a10536 
o|substituted constant variable: a10538 
o|substituted constant variable: a10540 
o|substituted constant variable: a10542 
o|substituted constant variable: a10544 
o|substituted constant variable: a10546 
o|substituted constant variable: a10548 
o|substituted constant variable: a10550 
o|substituted constant variable: a10552 
o|substituted constant variable: a10554 
o|substituted constant variable: a10556 
o|substituted constant variable: a10558 
o|substituted constant variable: a10560 
o|substituted constant variable: a10562 
o|substituted constant variable: a10564 
o|substituted constant variable: a10566 
o|substituted constant variable: a10568 
o|substituted constant variable: a10570 
o|substituted constant variable: a10572 
o|substituted constant variable: a10574 
o|replaced variables: 3057 
o|removed binding forms: 453 
o|substituted constant variable: r351214174 
o|substituted constant variable: r361814180 
o|substituted constant variable: r365814182 
o|substituted constant variable: r381314190 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#second 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#third 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#fourth 
o|substituted constant variable: r402214199 
o|substituted constant variable: r442114200 
o|substituted constant variable: r515414266 
o|substituted constant variable: r514214279 
o|substituted constant variable: r532214293 
o|substituted constant variable: r531214294 
o|substituted constant variable: r530014295 
o|substituted constant variable: r528814296 
o|substituted constant variable: r527614302 
o|substituted constant variable: r538514308 
o|substituted constant variable: r539714310 
o|substituted constant variable: r539714311 
o|substituted constant variable: r526414312 
o|inlining procedure: k5432 
o|inlining procedure: k5502 
o|inlining procedure: k5502 
o|inlining procedure: k5432 
o|substituted constant variable: r581714388 
o|converted assignments to bindings: (cfk1084) 
o|substituted constant variable: r663614499 
o|substituted constant variable: r744214503 
o|substituted constant variable: r661814511 
o|substituted constant variable: r660614512 
o|substituted constant variable: r670814535 
o|substituted constant variable: r679014540 
o|substituted constant variable: r676314541 
o|substituted constant variable: r675114547 
o|substituted constant variable: r673914553 
o|substituted constant variable: r685014560 
o|inlining procedure: k6994 
o|substituted constant variable: r712414594 
o|substituted constant variable: r764314646 
o|substituted constant variable: r1066414680 
o|substituted constant variable: r1065214686 
o|substituted constant variable: r1064314687 
o|substituted constant variable: r1062814688 
o|substituted constant variable: r1061614689 
o|substituted constant variable: r1085014750 
o|substituted constant variable: r1091914759 
o|substituted constant variable: r1091914760 
o|substituted constant variable: r1091314766 
o|substituted constant variable: r1096214768 
o|substituted constant variable: r1096214769 
o|substituted constant variable: r1098214777 
o|substituted constant variable: r1100114778 
o|substituted constant variable: r1100114779 
o|substituted constant variable: r1103014787 
o|substituted constant variable: r1105514788 
o|inlining procedure: k11061 
o|inlining procedure: k11061 
o|substituted constant variable: r1105514799 
o|substituted constant variable: r1115114836 
o|substituted constant variable: r445714849 
o|substituted constant variable: r1120814850 
o|substituted constant variable: r1181714947 
o|inlining procedure: k11883 
o|inlining procedure: k11883 
o|substituted constant variable: r1190414968 
o|substituted constant variable: r1201414969 
o|substituted constant variable: r1199614980 
o|inlining procedure: "(optimizer.scm:1870) chicken.compiler.optimizer#first" 
o|inlining procedure: "(optimizer.scm:947) chicken.compiler.optimizer#register-simplifications" 
o|substituted constant variable: r1287215086 
o|substituted constant variable: r1286015087 
o|substituted constant variable: r1300315089 
o|inlining procedure: "(optimizer.scm:730) chicken.compiler.optimizer#register-simplifications" 
o|substituted constant variable: r1308415093 
o|substituted constant variable: r1322615098 
o|substituted constant variable: r1321715099 
o|substituted constant variable: r1340215101 
o|substituted constant variable: r776415135 
o|substituted constant variable: r784615138 
o|substituted constant variable: r789115141 
o|substituted constant variable: r406615145 
o|converted assignments to bindings: (find-path1697) 
o|substituted constant variable: r1366215165 
o|substituted constant variable: r1369215179 
o|substituted constant variable: r1368015185 
o|substituted constant variable: r1366815196 
o|substituted constant variable: r1357015217 
o|substituted constant variable: r1351615228 
o|substituted constant variable: r1350415234 
o|substituted constant variable: r1381015237 
o|substituted constant variable: r1379815238 
o|substituted constant variable: r1394915242 
o|substituted constant variable: r1393715243 
o|substituted constant variable: r1392515244 
o|inlining procedure: "(optimizer.scm:711) chicken.compiler.optimizer#register-simplifications" 
o|substituted constant variable: r1410815245 
o|substituted constant variable: r831815254 
o|substituted constant variable: r844815265 
o|substituted constant variable: r843615266 
o|substituted constant variable: r850815300 
o|substituted constant variable: r849915306 
o|substituted constant variable: r856615326 
o|substituted constant variable: r867415355 
o|substituted constant variable: r866815356 
o|substituted constant variable: r875115375 
o|substituted constant variable: r873915381 
o|substituted constant variable: r882815395 
o|substituted constant variable: r881615396 
o|substituted constant variable: r890815415 
o|substituted constant variable: r889915421 
o|substituted constant variable: r896715434 
o|substituted constant variable: r901815458 
o|substituted constant variable: r899115471 
o|substituted constant variable: r924615509 
o|substituted constant variable: r923415515 
o|substituted constant variable: r934415529 
o|substituted constant variable: r932615540 
o|substituted constant variable: r940215566 
o|substituted constant variable: r939315572 
o|substituted constant variable: r949015581 
o|substituted constant variable: r947515592 
o|substituted constant variable: r955715611 
o|substituted constant variable: r954515622 
o|substituted constant variable: r953615628 
o|substituted constant variable: r966315643 
o|substituted constant variable: r962815654 
o|substituted constant variable: r961615660 
o|inlining procedure: k9764 
o|substituted constant variable: r980415679 
o|substituted constant variable: r972915680 
o|substituted constant variable: r971715681 
o|substituted constant variable: r983515713 
o|substituted constant variable: r990915727 
o|substituted constant variable: r990015728 
o|substituted constant variable: r994315733 
o|substituted constant variable: r1008915747 
o|substituted constant variable: r1007715753 
o|substituted constant variable: r1028715777 
o|substituted constant variable: r1028715777 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1016715799 
o|substituted constant variable: r1033815817 
o|substituted constant variable: r1032915818 
o|substituted constant variable: r1040215855 
o|simplifications: ((let . 2)) 
o|replaced variables: 1142 
o|removed binding forms: 2467 
o|inlining procedure: k5162 
o|inlining procedure: "(optimizer.scm:186) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: k5126 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: k5254 
o|inlining procedure: "(optimizer.scm:313) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: "(optimizer.scm:457) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: k7321 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#register-simplifications 
o|inlining procedure: "(optimizer.scm:1587) chicken.compiler.optimizer#alist-cons" 
o|substituted constant variable: r110551478815983 
o|substituted constant variable: r110551478815986 
o|inlining procedure: "(optimizer.scm:1667) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: "(optimizer.scm:1692) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: "(optimizer.scm:1778) chicken.compiler.optimizer#alist-cons" 
o|substituted constant variable: class142116059 
o|substituted constant variable: class142116067 
o|inlining procedure: "(optimizer.scm:1017) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: "(optimizer.scm:1029) chicken.compiler.optimizer#alist-cons" 
o|inlining procedure: "(optimizer.scm:1043) chicken.compiler.optimizer#alist-cons" 
o|substituted constant variable: class142116118 
o|inlining procedure: "(optimizer.scm:725) chicken.compiler.optimizer#alist-cons" 
o|removed call to pure procedure with unused result: "(optimizer.scm:1222) ##sys#slot" 
o|inlining procedure: "(optimizer.scm:1342) chicken.compiler.optimizer#fifth" 
o|inlining procedure: "(optimizer.scm:1437) chicken.compiler.optimizer#fifth" 
o|inlining procedure: "(optimizer.scm:1477) chicken.compiler.optimizer#fifth" 
o|replaced variables: 12 
o|removed binding forms: 915 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#fifth 
o|removed side-effect free assignment to unused variable: chicken.compiler.optimizer#alist-cons 
o|substituted constant variable: y25116193 
o|substituted constant variable: r512716200 
o|substituted constant variable: r512716200 
o|substituted constant variable: r525516203 
o|substituted constant variable: r525516203 
o|substituted constant variable: r525516206 
o|substituted constant variable: r525516206 
o|substituted constant variable: r525516209 
o|substituted constant variable: r525516209 
o|substituted constant variable: r525516212 
o|substituted constant variable: r525516212 
o|substituted constant variable: r525516215 
o|substituted constant variable: r525516215 
o|substituted constant variable: r525516218 
o|substituted constant variable: r525516218 
o|substituted constant variable: r525516221 
o|substituted constant variable: r525516221 
o|substituted constant variable: r525516224 
o|substituted constant variable: r525516224 
o|substituted constant variable: r525516227 
o|substituted constant variable: r525516227 
o|substituted constant variable: r525516230 
o|substituted constant variable: r525516230 
o|substituted constant variable: r525516233 
o|substituted constant variable: r525516233 
o|substituted constant variable: r525516236 
o|substituted constant variable: r525516236 
o|substituted constant variable: x25016306 
o|substituted constant variable: y25116405 
o|contracted procedure: k9188 
o|inlining procedure: k9869 
o|replaced variables: 35 
o|removed binding forms: 47 
o|removed conditional forms: 13 
o|substituted constant variable: r987016595 
o|removed binding forms: 54 
o|removed conditional forms: 1 
o|removed binding forms: 1 
o|simplifications: ((if . 46) (let . 41) (##core#call . 1305)) 
o|  call simplifications:
o|    scheme#>=
o|    scheme#symbol?
o|    scheme#cddddr	4
o|    scheme#*
o|    scheme#<=
o|    chicken.fixnum#fx<
o|    scheme#=	10
o|    chicken.base#fixnum?
o|    chicken.fixnum#fx=
o|    ##sys#cons	17
o|    ##sys#list	87
o|    scheme#assoc
o|    scheme#-
o|    chicken.fixnum#fx>	2
o|    scheme#set-car!
o|    scheme#list?	6
o|    scheme#>	5
o|    scheme#length	44
o|    scheme#<	6
o|    ##sys#call-with-values	6
o|    scheme#cadddr	16
o|    scheme#+	8
o|    scheme#cddr	6
o|    scheme#member	4
o|    scheme#list	150
o|    scheme#cadr	55
o|    scheme#caddr	34
o|    scheme#caar	2
o|    scheme#assq	10
o|    scheme#set-cdr!	3
o|    scheme#equal?	6
o|    scheme#eq?	149
o|    scheme#memq	17
o|    chicken.fixnum#fx>=
o|    chicken.fixnum#fx+
o|    scheme#not	60
o|    scheme#null?	32
o|    scheme#apply	3
o|    scheme#cdr	39
o|    ##sys#setslot	18
o|    ##sys#check-list	32
o|    scheme#pair?	63
o|    ##sys#slot	106
o|    scheme#values	8
o|    chicken.fixnum#fx<=	3
o|    scheme#car	178
o|    chicken.fixnum#fx-	2
o|    scheme#cons	102
o|contracted procedure: k3514 
o|contracted procedure: k3521 
o|contracted procedure: k3531 
o|contracted procedure: k3572 
o|contracted procedure: k3586 
o|contracted procedure: k3596 
o|contracted procedure: k3590 
o|contracted procedure: k3605 
o|contracted procedure: k3608 
o|contracted procedure: k3620 
o|contracted procedure: k3636 
o|contracted procedure: k3644 
o|contracted procedure: k3651 
o|contracted procedure: k3675 
o|contracted procedure: k3692 
o|contracted procedure: k3714 
o|contracted procedure: k3710 
o|contracted procedure: k3695 
o|contracted procedure: k3698 
o|contracted procedure: k3706 
o|contracted procedure: k3721 
o|contracted procedure: k3729 
o|contracted procedure: k3741 
o|contracted procedure: k3763 
o|contracted procedure: k3759 
o|contracted procedure: k3744 
o|contracted procedure: k3747 
o|contracted procedure: k3755 
o|contracted procedure: k3778 
o|contracted procedure: k3787 
o|contracted procedure: k3800 
o|contracted procedure: k3815 
o|contracted procedure: k3830 
o|contracted procedure: k3845 
o|contracted procedure: k3856 
o|contracted procedure: k3901 
o|contracted procedure: k3927 
o|contracted procedure: k4012 
o|contracted procedure: k4024 
o|contracted procedure: k4042 
o|contracted procedure: k4050 
o|contracted procedure: k4423 
o|contracted procedure: k4438 
o|contracted procedure: k4707 
o|contracted procedure: k4710 
o|contracted procedure: k4717 
o|contracted procedure: k4744 
o|contracted procedure: k4756 
o|contracted procedure: k4766 
o|contracted procedure: k4770 
o|contracted procedure: k4788 
o|contracted procedure: k4791 
o|contracted procedure: k4812 
o|contracted procedure: k4808 
o|contracted procedure: k4819 
o|contracted procedure: k4833 
o|contracted procedure: k4822 
o|contracted procedure: k4829 
o|contracted procedure: k4839 
o|contracted procedure: k4855 
o|contracted procedure: k4859 
o|contracted procedure: k4865 
o|contracted procedure: k4191 
o|contracted procedure: k4181 
o|contracted procedure: k4889 
o|contracted procedure: k4892 
o|contracted procedure: k4898 
o|contracted procedure: k4907 
o|contracted procedure: k4910 
o|contracted procedure: k4913 
o|contracted procedure: k4931 
o|contracted procedure: k4954 
o|contracted procedure: k4973 
o|contracted procedure: k5004 
o|contracted procedure: k5016 
o|contracted procedure: k5026 
o|contracted procedure: k5030 
o|contracted procedure: k5033 
o|contracted procedure: k3976 
o|contracted procedure: k3979 
o|contracted procedure: k3989 
o|contracted procedure: k5091 
o|contracted procedure: k5103 
o|contracted procedure: k5117 
o|contracted procedure: k5106 
o|contracted procedure: k5113 
o|contracted procedure: k5138 
o|contracted procedure: k5156 
o|contracted procedure: k5159 
o|contracted procedure: k5176 
o|contracted procedure: k5162 
o|contracted procedure: k400616197 
o|contracted procedure: k5180 
o|contracted procedure: k5184 
o|contracted procedure: k5188 
o|contracted procedure: k5197 
o|contracted procedure: k5200 
o|contracted procedure: k5212 
o|contracted procedure: k5215 
o|contracted procedure: k5218 
o|contracted procedure: k5226 
o|contracted procedure: k5234 
o|contracted procedure: k5241 
o|contracted procedure: k5251 
o|contracted procedure: k5260 
o|contracted procedure: k5272 
o|contracted procedure: k5278 
o|contracted procedure: k5284 
o|contracted procedure: k5290 
o|contracted procedure: k5296 
o|contracted procedure: k5308 
o|contracted procedure: k5318 
o|contracted procedure: k5338 
o|contracted procedure: k5342 
o|contracted procedure: k5381 
o|contracted procedure: k5387 
o|contracted procedure: k5393 
o|contracted procedure: k5399 
o|contracted procedure: k5416 
o|contracted procedure: k5435 
o|contracted procedure: k5468 
o|contracted procedure: k5441 
o|contracted procedure: k5456 
o|contracted procedure: k5453 
o|contracted procedure: k5474 
o|contracted procedure: k5483 
o|contracted procedure: k5535 
o|contracted procedure: k5539 
o|contracted procedure: k5490 
o|contracted procedure: k5499 
o|contracted procedure: k4240 
o|contracted procedure: k4266 
o|contracted procedure: k4246 
o|contracted procedure: k4260 
o|contracted procedure: k5523 
o|contracted procedure: k5531 
o|contracted procedure: k5527 
o|contracted procedure: k5583 
o|contracted procedure: k5605 
o|contracted procedure: k5680 
o|contracted procedure: k5676 
o|contracted procedure: k5619 
o|contracted procedure: k5626 
o|contracted procedure: k5632 
o|contracted procedure: k5640 
o|contracted procedure: k5662 
o|contracted procedure: k5684 
o|contracted procedure: k5690 
o|contracted procedure: k5693 
o|contracted procedure: k5714 
o|contracted procedure: k5724 
o|contracted procedure: k5732 
o|contracted procedure: k5744 
o|contracted procedure: k5747 
o|contracted procedure: k5750 
o|contracted procedure: k5758 
o|contracted procedure: k5766 
o|contracted procedure: k5779 
o|contracted procedure: k400616249 
o|contracted procedure: k5787 
o|contracted procedure: k5790 
o|contracted procedure: k5797 
o|contracted procedure: k5805 
o|contracted procedure: k5813 
o|contracted procedure: k5832 
o|contracted procedure: k5835 
o|contracted procedure: k5838 
o|contracted procedure: k3412 
o|contracted procedure: k3436 
o|contracted procedure: k3447 
o|contracted procedure: k3455 
o|contracted procedure: k5901 
o|contracted procedure: k5905 
o|contracted procedure: k5913 
o|contracted procedure: k5881 
o|contracted procedure: k5885 
o|contracted procedure: k5893 
o|contracted procedure: k5897 
o|contracted procedure: k5975 
o|contracted procedure: k5979 
o|contracted procedure: k5987 
o|contracted procedure: k5955 
o|contracted procedure: k5959 
o|contracted procedure: k5967 
o|contracted procedure: k5971 
o|contracted procedure: k5996 
o|contracted procedure: k6002 
o|contracted procedure: k6011 
o|contracted procedure: k6014 
o|contracted procedure: k6023 
o|contracted procedure: k6026 
o|contracted procedure: k6045 
o|contracted procedure: k6065 
o|contracted procedure: k6079 
o|contracted procedure: k6092 
o|contracted procedure: k6102 
o|contracted procedure: k6106 
o|contracted procedure: k6131 
o|contracted procedure: k6139 
o|contracted procedure: k6135 
o|contracted procedure: k6155 
o|contracted procedure: k6164 
o|contracted procedure: k6190 
o|contracted procedure: k6232 
o|contracted procedure: k400616266 
o|contracted procedure: k6222 
o|propagated global variable: z25216263 chicken.compiler.optimizer#inline-history 
o|contracted procedure: k6236 
o|contracted procedure: k6249 
o|contracted procedure: k6259 
o|contracted procedure: k6263 
o|contracted procedure: k6267 
o|contracted procedure: k6271 
o|contracted procedure: k6462 
o|contracted procedure: k6289 
o|contracted procedure: k6304 
o|contracted procedure: k6307 
o|contracted procedure: k6313 
o|contracted procedure: k6321 
o|contracted procedure: k6339 
o|contracted procedure: k6342 
o|contracted procedure: k6345 
o|contracted procedure: k6353 
o|contracted procedure: k6361 
o|contracted procedure: k6397 
o|contracted procedure: k6401 
o|contracted procedure: k6434 
o|contracted procedure: k6447 
o|contracted procedure: k6451 
o|contracted procedure: k6457 
o|contracted procedure: k6589 
o|contracted procedure: k6477 
o|contracted procedure: k6502 
o|contracted procedure: k6509 
o|contracted procedure: k6517 
o|contracted procedure: k6529 
o|contracted procedure: k6532 
o|contracted procedure: k6535 
o|contracted procedure: k6543 
o|contracted procedure: k6551 
o|contracted procedure: k6562 
o|contracted procedure: k6569 
o|contracted procedure: k6585 
o|contracted procedure: k6579 
o|contracted procedure: k6602 
o|contracted procedure: k6614 
o|contracted procedure: k6620 
o|contracted procedure: k6644 
o|contracted procedure: k7420 
o|contracted procedure: k7429 
o|contracted procedure: k7460 
o|contracted procedure: k7438 
o|contracted procedure: k7444 
o|contracted procedure: k6647 
o|contracted procedure: k6657 
o|contracted procedure: k6691 
o|contracted procedure: k6697 
o|contracted procedure: k6710 
o|contracted procedure: k6717 
o|contracted procedure: k6735 
o|contracted procedure: k6741 
o|contracted procedure: k6753 
o|contracted procedure: k6759 
o|contracted procedure: k6792 
o|contracted procedure: k6811 
o|contracted procedure: k6822 
o|contracted procedure: k6843 
o|contracted procedure: k6852 
o|contracted procedure: k6863 
o|contracted procedure: k6874 
o|contracted procedure: k6897 
o|contracted procedure: k6903 
o|contracted procedure: k6966 
o|contracted procedure: k6916 
o|contracted procedure: k6920 
o|contracted procedure: k6937 
o|contracted procedure: k6940 
o|contracted procedure: k6943 
o|contracted procedure: k6951 
o|contracted procedure: k6959 
o|contracted procedure: k6979 
o|contracted procedure: k6982 
o|contracted procedure: k7036 
o|contracted procedure: k7085 
o|contracted procedure: k7076 
o|contracted procedure: k7102 
o|contracted procedure: k7093 
o|contracted procedure: k7039 
o|contracted procedure: k7051 
o|contracted procedure: k7061 
o|contracted procedure: k7065 
o|contracted procedure: k7106 
o|contracted procedure: k7114 
o|contracted procedure: k7117 
o|contracted procedure: k7126 
o|contracted procedure: k7132 
o|contracted procedure: k7143 
o|contracted procedure: k7168 
o|contracted procedure: k7174 
o|contracted procedure: k7202 
o|contracted procedure: k7208 
o|contracted procedure: k7249 
o|contracted procedure: k7275 
o|contracted procedure: k7281 
o|contracted procedure: k7290 
o|contracted procedure: k7327 
o|contracted procedure: k7333 
o|contracted procedure: k7339 
o|contracted procedure: k7345 
o|contracted procedure: k7382 
o|contracted procedure: k7394 
o|contracted procedure: k7404 
o|contracted procedure: k7408 
o|contracted procedure: k7365 
o|contracted procedure: k7379 
o|propagated global variable: g13161318 chicken.compiler.optimizer#simplified-ops 
o|contracted procedure: k7411 
o|contracted procedure: k7485 
o|contracted procedure: k7496 
o|contracted procedure: k7502 
o|contracted procedure: k7517 
o|contracted procedure: k7520 
o|contracted procedure: k7532 
o|contracted procedure: k7544 
o|contracted procedure: k7554 
o|contracted procedure: k7574 
o|contracted procedure: k7570 
o|contracted procedure: k7584 
o|contracted procedure: k7587 
o|contracted procedure: k7594 
o|contracted procedure: k7621 
o|contracted procedure: k7609 
o|contracted procedure: k7624 
o|contracted procedure: k7631 
o|contracted procedure: k7668 
o|contracted procedure: k7645 
o|contracted procedure: k7663 
o|contracted procedure: k7651 
o|contracted procedure: k7671 
o|contracted procedure: k7686 
o|contracted procedure: k7692 
o|contracted procedure: k7695 
o|contracted procedure: k7707 
o|contracted procedure: k7717 
o|contracted procedure: k7721 
o|contracted procedure: k8285 
o|contracted procedure: k8296 
o|contracted procedure: k10593 
o|contracted procedure: k10596 
o|contracted procedure: k10612 
o|contracted procedure: k10618 
o|contracted procedure: k10624 
o|contracted procedure: k10630 
o|contracted procedure: k10654 
o|contracted procedure: k10660 
o|contracted procedure: k10685 
o|contracted procedure: k10689 
o|contracted procedure: k10666 
o|contracted procedure: k10673 
o|contracted procedure: k10677 
o|contracted procedure: k10681 
o|contracted procedure: k10697 
o|contracted procedure: k10711 
o|contracted procedure: k10718 
o|contracted procedure: k10722 
o|contracted procedure: k10728 
o|contracted procedure: k10738 
o|contracted procedure: k10742 
o|contracted procedure: k10746 
o|contracted procedure: k10754 
o|contracted procedure: k10766 
o|contracted procedure: k10776 
o|contracted procedure: k10780 
o|contracted procedure: k10807 
o|contracted procedure: k10810 
o|contracted procedure: k10813 
o|contracted procedure: k10836 
o|contracted procedure: k10819 
o|contracted procedure: k10828 
o|contracted procedure: k10832 
o|contracted procedure: k10846 
o|contracted procedure: k10856 
o|contracted procedure: k10863 
o|contracted procedure: k10870 
o|contracted procedure: k10880 
o|contracted procedure: k10886 
o|contracted procedure: k10903 
o|contracted procedure: k10890 
o|contracted procedure: k10909 
o|contracted procedure: k10921 
o|contracted procedure: k400616296 
o|contracted procedure: k10925 
o|contracted procedure: k10933 
o|contracted procedure: k10939 
o|contracted procedure: k10964 
o|contracted procedure: k10968 
o|contracted procedure: k10972 
o|contracted procedure: k10978 
o|contracted procedure: k11003 
o|contracted procedure: k11007 
o|contracted procedure: k11011 
o|contracted procedure: k11017 
o|contracted procedure: k11020 
o|contracted procedure: k11026 
o|contracted procedure: k11032 
o|contracted procedure: k11048 
o|contracted procedure: k11051 
o|contracted procedure: k11058 
o|contracted procedure: k11065 
o|contracted procedure: k11073 
o|contracted procedure: k11069 
o|contracted procedure: k11098 
o|contracted procedure: k11101 
o|contracted procedure: k11111 
o|contracted procedure: k11115 
o|contracted procedure: k11130 
o|contracted procedure: k11137 
o|contracted procedure: k11141 
o|contracted procedure: k11147 
o|contracted procedure: k11160 
o|contracted procedure: k11168 
o|contracted procedure: k11172 
o|contracted procedure: k4472 
o|contracted procedure: k4476 
o|contracted procedure: k4459 
o|contracted procedure: k11833 
o|contracted procedure: k11230 
o|contracted procedure: k11242 
o|contracted procedure: k11251 
o|contracted procedure: k11262 
o|contracted procedure: k11328 
o|contracted procedure: k11321 
o|contracted procedure: k11271 
o|contracted procedure: k11285 
o|contracted procedure: k11281 
o|contracted procedure: k11293 
o|contracted procedure: k11303 
o|contracted procedure: k11297 
o|contracted procedure: k11318 
o|contracted procedure: k11314 
o|contracted procedure: k11306 
o|contracted procedure: k11334 
o|contracted procedure: k11343 
o|contracted procedure: k11352 
o|contracted procedure: k11397 
o|contracted procedure: k11411 
o|contracted procedure: k11363 
o|contracted procedure: k11400 
o|contracted procedure: k11407 
o|contracted procedure: k11381 
o|contracted procedure: k11420 
o|contracted procedure: k11469 
o|contracted procedure: k11462 
o|contracted procedure: k11429 
o|contracted procedure: k11433 
o|contracted procedure: k11436 
o|contracted procedure: k11477 
o|contracted procedure: k11489 
o|contracted procedure: k11499 
o|contracted procedure: k11503 
o|contracted procedure: k11509 
o|contracted procedure: k11526 
o|contracted procedure: k11530 
o|contracted procedure: k11548 
o|contracted procedure: k11551 
o|contracted procedure: k11554 
o|contracted procedure: k11566 
o|contracted procedure: k11701 
o|contracted procedure: k11572 
o|contracted procedure: k400616311 
o|contracted procedure: k11576 
o|contracted procedure: k11671 
o|contracted procedure: k11582 
o|contracted procedure: k11595 
o|contracted procedure: k11599 
o|contracted procedure: k11667 
o|contracted procedure: k11602 
o|contracted procedure: k11610 
o|contracted procedure: k11613 
o|contracted procedure: k11629 
o|contracted procedure: k11637 
o|contracted procedure: k11641 
o|contracted procedure: k11649 
o|contracted procedure: k11697 
o|contracted procedure: k11677 
o|contracted procedure: k11690 
o|contracted procedure: k11711 
o|contracted procedure: k11714 
o|contracted procedure: k11717 
o|contracted procedure: k11723 
o|contracted procedure: k400616318 
o|contracted procedure: k11727 
o|contracted procedure: k11737 
o|contracted procedure: k11740 
o|contracted procedure: k11752 
o|contracted procedure: k11762 
o|contracted procedure: k11766 
o|contracted procedure: k11769 
o|contracted procedure: k11781 
o|contracted procedure: k11791 
o|contracted procedure: k11795 
o|contracted procedure: k11813 
o|contracted procedure: k11819 
o|contracted procedure: k11826 
o|contracted procedure: k11836 
o|contracted procedure: k4121 
o|contracted procedure: k4129 
o|contracted procedure: k4141 
o|contracted procedure: k4163 
o|contracted procedure: k4159 
o|contracted procedure: k4144 
o|contracted procedure: k4147 
o|contracted procedure: k4155 
o|contracted procedure: k11880 
o|contracted procedure: k11896 
o|contracted procedure: k11888 
o|contracted procedure: k400616333 
o|contracted procedure: k11892 
o|contracted procedure: k11939 
o|contracted procedure: k11942 
o|contracted procedure: k11945 
o|contracted procedure: k11948 
o|contracted procedure: k11962 
o|contracted procedure: k11972 
o|contracted procedure: k11979 
o|contracted procedure: k11992 
o|contracted procedure: k11998 
o|contracted procedure: k12001 
o|contracted procedure: k12004 
o|contracted procedure: k12010 
o|contracted procedure: k12029 
o|contracted procedure: k12033 
o|contracted procedure: k12016 
o|contracted procedure: k12022 
o|contracted procedure: k11906 
o|contracted procedure: k11913 
o|contracted procedure: k12060 
o|contracted procedure: k12073 
o|contracted procedure: k12076 
o|contracted procedure: k12083 
o|contracted procedure: k12106 
o|contracted procedure: k12124 
o|contracted procedure: k12136 
o|contracted procedure: k12146 
o|contracted procedure: k12150 
o|contracted procedure: k12170 
o|contracted procedure: k12180 
o|contracted procedure: k12185 
o|contracted procedure: k12188 
o|contracted procedure: k12194 
o|contracted procedure: k12197 
o|contracted procedure: k12200 
o|contracted procedure: k12212 
o|contracted procedure: k12237 
o|contracted procedure: k12240 
o|contracted procedure: k12243 
o|contracted procedure: k12251 
o|contracted procedure: k12259 
o|contracted procedure: k12273 
o|contracted procedure: k12301 
o|contracted procedure: k12304 
o|contracted procedure: k12335 
o|contracted procedure: k12343 
o|contracted procedure: k12357 
o|contracted procedure: k12347 
o|contracted procedure: k12364 
o|contracted procedure: k12376 
o|contracted procedure: k12386 
o|contracted procedure: k12390 
o|contracted procedure: k12394 
o|contracted procedure: k12406 
o|contracted procedure: k12416 
o|contracted procedure: k12420 
o|contracted procedure: k12426 
o|contracted procedure: k12446 
o|contracted procedure: k12450 
o|contracted procedure: k12462 
o|contracted procedure: k12466 
o|contracted procedure: k12494 
o|contracted procedure: k12508 
o|contracted procedure: k12512 
o|contracted procedure: k12520 
o|contracted procedure: k12524 
o|contracted procedure: k12532 
o|contracted procedure: k12540 
o|contracted procedure: k12548 
o|contracted procedure: k12570 
o|contracted procedure: k12579 
o|contracted procedure: k12613 
o|contracted procedure: k12589 
o|contracted procedure: k12593 
o|contracted procedure: k12609 
o|contracted procedure: k12617 
o|contracted procedure: k12634 
o|contracted procedure: k12637 
o|contracted procedure: k12660 
o|contracted procedure: k12672 
o|contracted procedure: k12694 
o|contracted procedure: k12690 
o|contracted procedure: k12675 
o|contracted procedure: k12678 
o|contracted procedure: k12686 
o|contracted procedure: k12705 
o|contracted procedure: k12715 
o|contracted procedure: k12727 
o|contracted procedure: k12739 
o|contracted procedure: k12761 
o|contracted procedure: k12724 
o|contracted procedure: k12757 
o|contracted procedure: k12742 
o|contracted procedure: k12745 
o|contracted procedure: k12753 
o|contracted procedure: k12773 
o|contracted procedure: k12776 
o|contracted procedure: k12779 
o|contracted procedure: k12787 
o|contracted procedure: k12795 
o|contracted procedure: k12801 
o|contracted procedure: k12811 
o|contracted procedure: k12820 
o|contracted procedure: k12830 
o|contracted procedure: k12834 
o|contracted procedure: k13045 
o|contracted procedure: k13041 
o|contracted procedure: k13025 
o|contracted procedure: k13037 
o|contracted procedure: k13033 
o|contracted procedure: k13029 
o|contracted procedure: k12992 
o|contracted procedure: k12996 
o|contracted procedure: k13009 
o|contracted procedure: k13021 
o|contracted procedure: k12838 
o|contracted procedure: k12980 
o|contracted procedure: k12988 
o|contracted procedure: k12984 
o|contracted procedure: k12976 
o|contracted procedure: k12846 
o|contracted procedure: k12850 
o|contracted procedure: k12856 
o|contracted procedure: k12862 
o|contracted procedure: k12964 
o|contracted procedure: k12868 
o|contracted procedure: k12960 
o|contracted procedure: k12877 
o|contracted procedure: k12884 
o|contracted procedure: k12888 
o|contracted procedure: k12896 
o|contracted procedure: k12909 
o|contracted procedure: k12918 
o|contracted procedure: k12930 
o|contracted procedure: k12945 
o|contracted procedure: k12953 
o|contracted procedure: k12842 
o|contracted procedure: k13996 
o|contracted procedure: k14060 
o|contracted procedure: k14076 
o|contracted procedure: k14064 
o|contracted procedure: k14072 
o|contracted procedure: k14068 
o|contracted procedure: k14000 
o|contracted procedure: k14056 
o|contracted procedure: k14008 
o|contracted procedure: k14016 
o|contracted procedure: k14036 
o|contracted procedure: k14052 
o|contracted procedure: k14040 
o|contracted procedure: k14048 
o|contracted procedure: k14044 
o|contracted procedure: k14020 
o|contracted procedure: k14032 
o|contracted procedure: k14028 
o|contracted procedure: k14024 
o|contracted procedure: k14012 
o|contracted procedure: k14004 
o|contracted procedure: k13914 
o|contracted procedure: k13918 
o|contracted procedure: k13927 
o|contracted procedure: k13984 
o|contracted procedure: k13945 
o|contracted procedure: k13975 
o|contracted procedure: k13951 
o|contracted procedure: k13958 
o|contracted procedure: k13049 
o|contracted procedure: k13850 
o|contracted procedure: k13894 
o|contracted procedure: k13910 
o|contracted procedure: k13898 
o|contracted procedure: k13906 
o|contracted procedure: k13902 
o|contracted procedure: k13854 
o|contracted procedure: k13890 
o|contracted procedure: k13862 
o|contracted procedure: k13874 
o|contracted procedure: k13886 
o|contracted procedure: k13882 
o|contracted procedure: k13878 
o|contracted procedure: k13870 
o|contracted procedure: k13866 
o|contracted procedure: k13858 
o|contracted procedure: k13787 
o|contracted procedure: k13791 
o|contracted procedure: k13800 
o|contracted procedure: k13838 
o|contracted procedure: k13812 
o|contracted procedure: k13819 
o|contracted procedure: k13053 
o|contracted procedure: k13779 
o|contracted procedure: k13783 
o|contracted procedure: k13474 
o|contracted procedure: k13478 
o|contracted procedure: k13488 
o|contracted procedure: k13506 
o|contracted procedure: k13775 
o|contracted procedure: k13512 
o|contracted procedure: k13518 
o|contracted procedure: k13524 
o|contracted procedure: k13527 
o|contracted procedure: k13542 
o|contracted procedure: k13557 
o|contracted procedure: k13549 
o|contracted procedure: k13553 
o|contracted procedure: k13563 
o|contracted procedure: k13572 
o|contracted procedure: k13747 
o|contracted procedure: k13751 
o|contracted procedure: k13578 
o|contracted procedure: k13743 
o|contracted procedure: k13585 
o|contracted procedure: k13591 
o|contracted procedure: k13628 
o|contracted procedure: k13616 
o|contracted procedure: k13620 
o|contracted procedure: k13624 
o|contracted procedure: k13636 
o|contracted procedure: k13642 
o|contracted procedure: k7742 
o|contracted procedure: k7745 
o|contracted procedure: k7748 
o|contracted procedure: k7795 
o|contracted procedure: k7766 
o|contracted procedure: k7791 
o|contracted procedure: k7769 
o|contracted procedure: k7772 
o|contracted procedure: k7778 
o|contracted procedure: k400616368 
o|contracted procedure: k7801 
o|contracted procedure: k7865 
o|contracted procedure: k7816 
o|contracted procedure: k7839 
o|contracted procedure: k400616375 
o|contracted procedure: k7823 
o|contracted procedure: k7831 
o|contracted procedure: k7861 
o|contracted procedure: k7848 
o|contracted procedure: k7873 
o|contracted procedure: k400616382 
o|contracted procedure: k7880 
o|contracted procedure: k7908 
o|contracted procedure: k7893 
o|contracted procedure: k7904 
o|contracted procedure: k4056 
o|contracted procedure: k4068 
o|contracted procedure: k4091 
o|contracted procedure: k4099 
o|contracted procedure: k7925 
o|contracted procedure: k7940 
o|contracted procedure: k7958 
o|contracted procedure: k7965 
o|contracted procedure: k8134 
o|contracted procedure: k7971 
o|contracted procedure: k7974 
o|contracted procedure: k7984 
o|contracted procedure: k7999 
o|contracted procedure: k7995 
o|contracted procedure: k7991 
o|contracted procedure: k8011 
o|contracted procedure: k8035 
o|contracted procedure: k8020 
o|contracted procedure: k8024 
o|contracted procedure: k8043 
o|contracted procedure: k8046 
o|contracted procedure: k8058 
o|contracted procedure: k8067 
o|contracted procedure: k8071 
o|contracted procedure: k8079 
o|contracted procedure: k8091 
o|contracted procedure: k8087 
o|contracted procedure: k8083 
o|contracted procedure: k8102 
o|contracted procedure: k8110 
o|contracted procedure: k8113 
o|contracted procedure: k8128 
o|contracted procedure: k8124 
o|contracted procedure: k8120 
o|contracted procedure: k8141 
o|contracted procedure: k8150 
o|contracted procedure: k8160 
o|contracted procedure: k8164 
o|contracted procedure: k8173 
o|contracted procedure: k8183 
o|contracted procedure: k8187 
o|contracted procedure: k8221 
o|contracted procedure: k8196 
o|contracted procedure: k8206 
o|contracted procedure: k8210 
o|contracted procedure: k8214 
o|contracted procedure: k8218 
o|contracted procedure: k8269 
o|contracted procedure: k8233 
o|contracted procedure: k8259 
o|contracted procedure: k8263 
o|contracted procedure: k8255 
o|contracted procedure: k8236 
o|contracted procedure: k8239 
o|contracted procedure: k8247 
o|contracted procedure: k8251 
o|contracted procedure: k13664 
o|contracted procedure: k13739 
o|contracted procedure: k13670 
o|contracted procedure: k13676 
o|contracted procedure: k13682 
o|contracted procedure: k13688 
o|contracted procedure: k13694 
o|contracted procedure: k13703 
o|contracted procedure: k13711 
o|contracted procedure: k13719 
o|contracted procedure: k13727 
o|contracted procedure: k13735 
o|contracted procedure: k13763 
o|contracted procedure: k13771 
o|contracted procedure: k13057 
o|contracted procedure: k13446 
o|contracted procedure: k13470 
o|contracted procedure: k13466 
o|contracted procedure: k13450 
o|contracted procedure: k13462 
o|contracted procedure: k13458 
o|contracted procedure: k13454 
o|contracted procedure: k13391 
o|contracted procedure: k13395 
o|contracted procedure: k13438 
o|contracted procedure: k13404 
o|contracted procedure: k13430 
o|contracted procedure: k13410 
o|contracted procedure: k13417 
o|contracted procedure: k13425 
o|contracted procedure: k13061 
o|contracted procedure: k13351 
o|contracted procedure: k13387 
o|contracted procedure: k13383 
o|contracted procedure: k13355 
o|contracted procedure: k13379 
o|contracted procedure: k13363 
o|contracted procedure: k13375 
o|contracted procedure: k13371 
o|contracted procedure: k13367 
o|contracted procedure: k13359 
o|contracted procedure: k13206 
o|contracted procedure: k13210 
o|contracted procedure: k13347 
o|contracted procedure: k13222 
o|contracted procedure: k13228 
o|contracted procedure: k13335 
o|contracted procedure: k13234 
o|contracted procedure: k13246 
o|contracted procedure: k13256 
o|contracted procedure: k13268 
o|contracted procedure: k13287 
o|contracted procedure: k13295 
o|contracted procedure: k13304 
o|contracted procedure: k13309 
o|contracted procedure: k13316 
o|contracted procedure: k13330 
o|contracted procedure: k13065 
o|contracted procedure: k13182 
o|contracted procedure: k13202 
o|contracted procedure: k13198 
o|contracted procedure: k13186 
o|contracted procedure: k13194 
o|contracted procedure: k13190 
o|contracted procedure: k13073 
o|contracted procedure: k13077 
o|contracted procedure: k13174 
o|contracted procedure: k13089 
o|contracted procedure: k13101 
o|contracted procedure: k13126 
o|contracted procedure: k13134 
o|contracted procedure: k13143 
o|contracted procedure: k13148 
o|contracted procedure: k13155 
o|contracted procedure: k13169 
o|contracted procedure: k13069 
o|contracted procedure: k14171 
o|contracted procedure: k14163 
o|contracted procedure: k14167 
o|contracted procedure: k14159 
o|contracted procedure: k14155 
o|contracted procedure: k14084 
o|contracted procedure: k14088 
o|contracted procedure: k14101 
o|contracted procedure: k14110 
o|contracted procedure: k14121 
o|contracted procedure: k14132 
o|contracted procedure: k14124 
o|contracted procedure: k400616409 
o|contracted procedure: k14136 
o|propagated global variable: z25216406 chicken.compiler.optimizer#simplified-ops 
o|contracted procedure: k14148 
o|contracted procedure: k8305 
o|contracted procedure: k8320 
o|contracted procedure: k8336 
o|contracted procedure: k8343 
o|contracted procedure: k8385 
o|contracted procedure: k8407 
o|contracted procedure: k8411 
o|contracted procedure: k8423 
o|contracted procedure: k8419 
o|contracted procedure: k8485 
o|contracted procedure: k8489 
o|contracted procedure: k8426 
o|contracted procedure: k8429 
o|contracted procedure: k8432 
o|contracted procedure: k8438 
o|contracted procedure: k8444 
o|contracted procedure: k8450 
o|contracted procedure: k8457 
o|contracted procedure: k8461 
o|contracted procedure: k8495 
o|contracted procedure: k8549 
o|contracted procedure: k8553 
o|contracted procedure: k8504 
o|contracted procedure: k8516 
o|contracted procedure: k8519 
o|contracted procedure: k8522 
o|contracted procedure: k8529 
o|contracted procedure: k8533 
o|contracted procedure: k8545 
o|contracted procedure: k8541 
o|contracted procedure: k8559 
o|contracted procedure: k8658 
o|contracted procedure: k8574 
o|contracted procedure: k8580 
o|contracted procedure: k8592 
o|contracted procedure: k8616 
o|contracted procedure: k8601 
o|contracted procedure: k8605 
o|contracted procedure: k8624 
o|contracted procedure: k8631 
o|contracted procedure: k8635 
o|contracted procedure: k8643 
o|contracted procedure: k8650 
o|contracted procedure: k8654 
o|contracted procedure: k8664 
o|contracted procedure: k8726 
o|contracted procedure: k8676 
o|contracted procedure: k8721 
o|contracted procedure: k8689 
o|contracted procedure: k8701 
o|contracted procedure: k8709 
o|contracted procedure: k8693 
o|contracted procedure: k8713 
o|contracted procedure: k8717 
o|contracted procedure: k8732 
o|contracted procedure: k8806 
o|contracted procedure: k8747 
o|contracted procedure: k8753 
o|contracted procedure: k8756 
o|contracted procedure: k8759 
o|contracted procedure: k8766 
o|contracted procedure: k8770 
o|contracted procedure: k8798 
o|contracted procedure: k8778 
o|contracted procedure: k8786 
o|contracted procedure: k8782 
o|contracted procedure: k8794 
o|contracted procedure: k8812 
o|contracted procedure: k8818 
o|contracted procedure: k8821 
o|contracted procedure: k8880 
o|contracted procedure: k8830 
o|contracted procedure: k8843 
o|contracted procedure: k8847 
o|contracted procedure: k8875 
o|contracted procedure: k8855 
o|contracted procedure: k8859 
o|contracted procedure: k8871 
o|contracted procedure: k8867 
o|contracted procedure: k8886 
o|contracted procedure: k8892 
o|contracted procedure: k8895 
o|contracted procedure: k8953 
o|contracted procedure: k8957 
o|contracted procedure: k8904 
o|contracted procedure: k8917 
o|contracted procedure: k8921 
o|contracted procedure: k8949 
o|contracted procedure: k8929 
o|contracted procedure: k8937 
o|contracted procedure: k8945 
o|contracted procedure: k8963 
o|contracted procedure: k8975 
o|contracted procedure: k8984 
o|contracted procedure: k9223 
o|contracted procedure: k8999 
o|contracted procedure: k9006 
o|contracted procedure: k9010 
o|contracted procedure: k9026 
o|contracted procedure: k9037 
o|contracted procedure: k9040 
o|contracted procedure: k9055 
o|contracted procedure: k9062 
o|contracted procedure: k9090 
o|contracted procedure: k9087 
o|contracted procedure: k9070 
o|contracted procedure: k9066 
o|contracted procedure: k9083 
o|contracted procedure: k9123 
o|contracted procedure: k9103 
o|contracted procedure: k9111 
o|contracted procedure: k9107 
o|contracted procedure: k9132 
o|contracted procedure: k9135 
o|contracted procedure: k9138 
o|contracted procedure: k9146 
o|contracted procedure: k9154 
o|contracted procedure: k9166 
o|contracted procedure: k9169 
o|contracted procedure: k9172 
o|contracted procedure: k9180 
o|contracted procedure: k9209 
o|contracted procedure: k9194 
o|contracted procedure: k9203 
o|contracted procedure: k9219 
o|contracted procedure: k9230 
o|contracted procedure: k9239 
o|contracted procedure: k9242 
o|contracted procedure: k9251 
o|contracted procedure: k9313 
o|contracted procedure: k9264 
o|contracted procedure: k9276 
o|contracted procedure: k9268 
o|contracted procedure: k9301 
o|contracted procedure: k9287 
o|contracted procedure: k9294 
o|contracted procedure: k9305 
o|contracted procedure: k9309 
o|contracted procedure: k9319 
o|contracted procedure: k9328 
o|contracted procedure: k9331 
o|contracted procedure: k9340 
o|contracted procedure: k9346 
o|contracted procedure: k9372 
o|contracted procedure: k9356 
o|contracted procedure: k9368 
o|contracted procedure: k9379 
o|contracted procedure: k9383 
o|contracted procedure: k9389 
o|contracted procedure: k9404 
o|contracted procedure: k9407 
o|contracted procedure: k9410 
o|contracted procedure: k9462 
o|contracted procedure: k9416 
o|contracted procedure: k9422 
o|contracted procedure: k9429 
o|contracted procedure: k9433 
o|contracted procedure: k9456 
o|contracted procedure: k9440 
o|contracted procedure: k9452 
o|contracted procedure: k9468 
o|contracted procedure: k9483 
o|contracted procedure: k9486 
o|contracted procedure: k9495 
o|contracted procedure: k9514 
o|contracted procedure: k9517 
o|contracted procedure: k9526 
o|contracted procedure: k9532 
o|contracted procedure: k9599 
o|contracted procedure: k9603 
o|contracted procedure: k9541 
o|contracted procedure: k9595 
o|contracted procedure: k9553 
o|contracted procedure: k9559 
o|contracted procedure: k9562 
o|contracted procedure: k9569 
o|contracted procedure: k9573 
o|contracted procedure: k9585 
o|contracted procedure: k9581 
o|contracted procedure: k9609 
o|contracted procedure: k9692 
o|contracted procedure: k9618 
o|contracted procedure: k9624 
o|contracted procedure: k9684 
o|contracted procedure: k9636 
o|contracted procedure: k9659 
o|contracted procedure: k9643 
o|contracted procedure: k9655 
o|contracted procedure: k9680 
o|contracted procedure: k9665 
o|contracted procedure: k9672 
o|contracted procedure: k9676 
o|contracted procedure: k9698 
o|contracted procedure: k9701 
o|contracted procedure: k9704 
o|contracted procedure: k9707 
o|contracted procedure: k9710 
o|contracted procedure: k9722 
o|contracted procedure: k9725 
o|contracted procedure: k9737 
o|contracted procedure: k9744 
o|contracted procedure: k9748 
o|contracted procedure: k9767 
o|inlining procedure: k9756 
o|contracted procedure: k9774 
o|contracted procedure: k9787 
o|contracted procedure: k9783 
o|contracted procedure: k9764 
o|inlining procedure: k9756 
o|contracted procedure: k9790 
o|contracted procedure: k9797 
o|contracted procedure: k9822 
o|contracted procedure: k9815 
o|contracted procedure: k396116422 
o|contracted procedure: k9828 
o|contracted procedure: k9886 
o|contracted procedure: k9890 
o|contracted procedure: k9837 
o|contracted procedure: k9850 
o|contracted procedure: k9854 
o|contracted procedure: k9862 
o|contracted procedure: k9882 
o|contracted procedure: k9869 
o|contracted procedure: k9896 
o|contracted procedure: k9905 
o|contracted procedure: k9918 
o|contracted procedure: k9922 
o|contracted procedure: k9930 
o|contracted procedure: k9936 
o|contracted procedure: k10063 
o|contracted procedure: k9951 
o|contracted procedure: k9961 
o|contracted procedure: k9965 
o|contracted procedure: k9977 
o|contracted procedure: k9984 
o|contracted procedure: k9988 
o|contracted procedure: k10005 
o|contracted procedure: k10009 
o|contracted procedure: k10021 
o|contracted procedure: k10059 
o|contracted procedure: k10031 
o|contracted procedure: k10035 
o|contracted procedure: k10043 
o|contracted procedure: k10047 
o|contracted procedure: k10070 
o|contracted procedure: k10073 
o|contracted procedure: k10079 
o|contracted procedure: k10082 
o|contracted procedure: k10154 
o|contracted procedure: k10091 
o|contracted procedure: k10104 
o|contracted procedure: k10108 
o|contracted procedure: k10150 
o|contracted procedure: k10116 
o|contracted procedure: k10138 
o|contracted procedure: k10146 
o|contracted procedure: k10160 
o|contracted procedure: k10172 
o|contracted procedure: k396116429 
o|contracted procedure: k10175 
o|contracted procedure: k10178 
o|contracted procedure: k10181 
o|contracted procedure: k10190 
o|contracted procedure: k10197 
o|contracted procedure: k10201 
o|contracted procedure: k10276 
o|contracted procedure: k10211 
o|contracted procedure: k10218 
o|contracted procedure: k10226 
o|contracted procedure: k10222 
o|contracted procedure: k10233 
o|contracted procedure: k10237 
o|contracted procedure: k10250 
o|contracted procedure: k10257 
o|contracted procedure: k10261 
o|contracted procedure: k10268 
o|contracted procedure: k10272 
o|contracted procedure: k10289 
o|contracted procedure: k10296 
o|contracted procedure: k10286 
o|contracted procedure: k10316 
o|contracted procedure: k10319 
o|contracted procedure: k10322 
o|contracted procedure: k10325 
o|contracted procedure: k10334 
o|contracted procedure: k10346 
o|contracted procedure: k10349 
o|contracted procedure: k10356 
o|contracted procedure: k10360 
o|contracted procedure: k10367 
o|contracted procedure: k396116434 
o|contracted procedure: k10378 
o|contracted procedure: k10374 
o|contracted procedure: k10389 
o|contracted procedure: k10385 
o|contracted procedure: k10395 
o|contracted procedure: k10407 
o|contracted procedure: k10519 
o|contracted procedure: k10523 
o|contracted procedure: k10413 
o|contracted procedure: k10515 
o|contracted procedure: k10420 
o|contracted procedure: k10452 
o|contracted procedure: k10461 
o|contracted procedure: k10467 
o|contracted procedure: k10484 
o|contracted procedure: k8352 
o|contracted procedure: k8374 
o|contracted procedure: k8361 
o|contracted procedure: k8368 
o|contracted procedure: k10490 
o|contracted procedure: k10497 
o|contracted procedure: k10507 
o|contracted procedure: k10511 
o|contracted procedure: k14080 
o|simplifications: ((if . 16) (let . 368)) 
o|removed binding forms: 1219 
o|contracted procedure: k4703 
o|contracted procedure: k4815 
o|contracted procedure: k4937 
o|substituted constant variable: r5339 
o|contracted procedure: k6468 
o|contracted procedure: k6629 
o|contracted procedure: k7529 
o|contracted procedure: k8311 
o|contracted procedure: k8577 
o|contracted procedure: k9020 
o|contracted procedure: k9349 
o|contracted procedure: k9713 
o|contracted procedure: k9756 
o|inlining procedure: k9866 
o|inlining procedure: k9866 
o|replaced variables: 5 
o|removed binding forms: 12 
o|removed binding forms: 5 
o|direct leaf routine/allocation: mark558 3 
o|direct leaf routine/allocation: touch560 0 
o|direct leaf routine/allocation: loop333 0 
o|direct leaf routine/allocation: touch678 0 
o|direct leaf routine/allocation: for-each-loop693711 0 
o|direct leaf routine/allocation: g733742 0 
o|direct leaf routine/allocation: touch1367 0 
o|direct leaf routine/allocation: close2510 6 
o|direct leaf routine/allocation: g293294 3 
o|direct leaf routine/allocation: g14401441 35 
o|direct leaf routine/allocation: argc-ok?1865 0 
o|contracted procedure: "(optimizer.scm:97) k4848" 
o|contracted procedure: k4875 
o|converted assignments to bindings: (loop333) 
o|contracted procedure: "(optimizer.scm:122) k4925" 
o|converted assignments to bindings: (for-each-loop693711) 
o|contracted procedure: "(optimizer.scm:187) k5165" 
o|contracted procedure: "(optimizer.scm:187) k516516190" 
o|contracted procedure: "(optimizer.scm:181) k5230" 
o|contracted procedure: "(optimizer.scm:217) k5327" 
o|contracted procedure: "(optimizer.scm:235) k5446" 
o|contracted procedure: "(optimizer.scm:257) k5513" 
o|contracted procedure: "(optimizer.scm:284) k5595" 
o|contracted procedure: "(optimizer.scm:288) k5622" 
o|contracted procedure: "(optimizer.scm:306) k5705" 
o|contracted procedure: "(optimizer.scm:326) k5871" 
o|contracted procedure: "(optimizer.scm:342) k5945" 
o|contracted procedure: "(optimizer.scm:377) k6057" 
o|contracted procedure: "(optimizer.scm:458) k6225" 
o|contracted procedure: "(optimizer.scm:471) k6310" 
o|contracted procedure: "(optimizer.scm:480) k6381" 
o|contracted procedure: "(optimizer.scm:546) k6991" 
o|contracted procedure: "(optimizer.scm:551) k7012" 
o|contracted procedure: "(optimizer.scm:558) k7024" 
o|contracted procedure: "(optimizer.scm:583) k7183" 
o|contracted procedure: "(optimizer.scm:1815) k11982" 
o|contracted procedure: "(optimizer.scm:1828) k12116" 
o|contracted procedure: "(optimizer.scm:1293) k9492" 
o|converted assignments to bindings: (argc-ok?1865) 
o|simplifications: ((let . 3)) 
o|removed binding forms: 25 
o|customizable procedures: (loop2182 lp2121 k9502 k9023 map-loop19691987 map-loop19972014 loop2021 foldr19131916 loop1432 k13117 loop1589 k13278 loop1566 k13609 map-loop16571676 g16891711 for-each-loop16881716 g17291736 for-each-loop17281744 g17531760 for-each-loop17521770 g17901791 foldl17831787 k7980 foldr18191822 g18241825 foldr18061809 foldr278281 g283284 a7887 find-path1697 find1700 loop21515 loop11493 foldr16371640 g16421643 g25162609 for-each-loop25152796 g26262635 map-loop26202676 map-loop26872707 map-loop27282745 k12432 chicken.compiler.optimizer#append-map loop2758 descend2752 g27192764 for-each-loop27182789 g27712778 for-each-loop27702784 chicken.compiler.optimizer#list-tabulate map-loop26452662 g25862593 for-each-loop25852603 k11954 k11968 walk2512 map-loop306324 k11239 for-each-loop24192429 for-each-loop24022412 g23812382 rec2355 g23482435 for-each-loop23472447 foldr24592462 for-each-loop24732486 k11035 k10988 k10949 chicken.compiler.optimizer#every rec2251 g22292236 for-each-loop22282239 scan2200 walk2199 transform2201 g13751382 for-each-loop13741410 test1368 k7514 k7541 k7550 k7296 for-each-loop13091321 lp1294 k7171 k7120 k7021 for-each-loop12531265 g12071216 map-loop12011226 k6039 k6768 k6121 k6149 loop1350 k6173 g11661175 map-loop11601185 chicken.compiler.optimizer#split-at loop1095 g11161125 map-loop11101135 invalidate-gae!679 g10611068 for-each-loop10601078 g968975 for-each-loop967985 walk-generic685 a5862 loop67 k5702 k5717 g886895 map-loop880905 g855856 k5646 g827828 replace-var683 walk1684 k5505 loop350 maybe-replace-rest-arg-calls681 walk682 k5266 test675 chicken.compiler.optimizer#cons* map-loop727745 simplify680 chicken.compiler.optimizer#delete loop243 for-each-loop565644 k4842 remember559 scan-each561 k4794 chicken.compiler.optimizer#filter g584591 for-each-loop583595 scan562 loop428 foldr260263 g265266 loop223 loop210 loop197 loop184 chicken.compiler.optimizer#any map-loop128146 map-loop155173 loop124 foldr113116 g118119 loop100 chicken.compiler.optimizer#take) 
o|calls to known targets: 553 
o|identified direct recursive calls: f_3509 1 
o|identified direct recursive calls: f_3567 1 
o|identified direct recursive calls: f_3615 1 
o|identified direct recursive calls: f_3687 1 
o|identified direct recursive calls: f_3736 1 
o|identified direct recursive calls: f_3840 1 
o|identified direct recursive calls: f_4019 1 
o|identified direct recursive calls: f_4176 1 
o|identified direct recursive calls: f_5098 1 
o|identified direct recursive calls: f_5207 1 
o|identified direct recursive calls: f_4235 2 
o|identified direct recursive calls: f_7427 2 
o|identified direct recursive calls: f_11415 1 
o|identified direct recursive calls: f_4136 1 
o|unused rest argument: _2783 f_12351 
o|unused rest argument: _2762 f_12502 
o|identified direct recursive calls: f_12667 1 
o|unused rest argument: _2715 f_12709 
o|identified direct recursive calls: f_12734 1 
o|identified direct recursive calls: f_12904 1 
o|identified direct recursive calls: f_4063 1 
o|identified direct recursive calls: f_8053 1 
o|identified direct recursive calls: f_8006 1 
o|identified direct recursive calls: f_8228 1 
o|identified direct recursive calls: f_8303 1 
o|identified direct recursive calls: f_8587 1 
o|identified direct recursive calls: f_9050 1 
o|identified direct recursive calls: f_10456 1 
o|fast box initializations: 90 
o|fast global references: 60 
o|fast global assignments: 19 
o|dropping unused closure argument: f_3509 
o|dropping unused closure argument: f_3561 
o|dropping unused closure argument: f_3600 
o|dropping unused closure argument: f_3770 
o|dropping unused closure argument: f_3804 
o|dropping unused closure argument: f_3834 
o|dropping unused closure argument: f_3890 
o|dropping unused closure argument: f_4010 
o|dropping unused closure argument: f_4176 
o|dropping unused closure argument: f_4412 
o|dropping unused closure argument: f_5084 
o|dropping unused closure argument: f_5098 
*/
/* end of file */
