/* Generated from library.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: library.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -explicit-use -no-trace -output-file library.c -no-module-registration -emit-import-library chicken.bitwise -emit-import-library chicken.blob -emit-import-library chicken.fixnum -emit-import-library chicken.flonum -emit-import-library chicken.gc -emit-import-library chicken.keyword -emit-import-library chicken.platform -emit-import-library chicken.plist -emit-import-library chicken.process-context
   unit: library
   uses: build-version library
*/
#include "chicken.h"

#include <errno.h>
#include <float.h>

#ifdef HAVE_SYSEXITS_H
# include <sysexits.h>
#endif

#ifndef EX_SOFTWARE
# define EX_SOFTWARE	70
#endif

#define C_close_file(p)	      (C_fclose((C_FILEPTR)(C_port_file(p))), C_SCHEME_UNDEFINED)
#define C_a_f64peek(ptr, c, b, i)  C_flonum(ptr, ((double *)C_data_pointer(b))[ C_unfix(i) ])
#define C_fetch_c_strlen(b, i) C_fix(strlen((C_char *)C_block_item(b, C_unfix(i))))
#define C_asciiz_strlen(str) C_fix(strlen(C_c_string(str)))
#define C_peek_c_string(b, i, to, len) (C_memcpy(C_data_pointer(to), (C_char *)C_block_item(b, C_unfix(i)), C_unfix(len)), C_SCHEME_UNDEFINED)
#define C_free_mptr(p, i)     (C_free((void *)C_block_item(p, C_unfix(i))), C_SCHEME_UNDEFINED)
#define C_free_sptr(p, i)     (C_free((void *)(((C_char **)C_block_item(p, 0))[ C_unfix(i) ])), C_SCHEME_UNDEFINED)

#define C_a_get_current_seconds(ptr, c, dummy)  C_int64_to_num(ptr, time(NULL))
#define C_peek_c_string_at(ptr, i)    ((C_char *)(((C_char **)ptr)[ i ]))

static C_word
fast_read_line_from_file(C_word str, C_word port, C_word size) {
  int n = C_unfix(size);
  int i;
  int c;
  char *buf = C_c_string(str);
  C_FILEPTR fp = C_port_file(port);

  if ((c = C_getc(fp)) == EOF) {
    if (ferror(fp)) {
      clearerr(fp);
      return C_fix(-1);
    } else { /* feof (fp) */
      return C_SCHEME_END_OF_FILE;
    }
  }

  C_ungetc(c, fp);

  for (i = 0; i < n; i++) {
    c = C_getc(fp);

    if(c == EOF && ferror(fp)) {
      clearerr(fp);
      return C_fix(-(i + 1));
    }

    switch (c) {
    case '\r':	if ((c = C_getc(fp)) != '\n') C_ungetc(c, fp);
    case EOF:	clearerr(fp);
    case '\n':	return C_fix(i);
    }
    buf[i] = c;
  }
  return C_SCHEME_FALSE;
}

static C_word
fast_read_string_from_file(C_word dest, C_word port, C_word len, C_word pos)
{
  size_t m;
  int n = C_unfix (len);
  char * buf = ((char *)C_data_pointer (dest) + C_unfix (pos));
  C_FILEPTR fp = C_port_file (port);

  if(feof(fp)) return C_SCHEME_END_OF_FILE;

  m = fread (buf, sizeof (char), n, fp);

  if (m < n) {
    if (ferror(fp)) /* Report to Scheme, which may retry, so clear errors */
      clearerr(fp);
    else if (feof(fp) && 0 == m) /* eof but m > 0? Return data first, below */
      return C_SCHEME_END_OF_FILE; /* Calling again will get us here */
  }

  return C_fix (m);
}

static C_word
shallow_equal(C_word x, C_word y)
{
  /* assumes x and y are non-immediate */
  int i, len = C_header_size(x);

  if(C_header_size(y) != len) return C_SCHEME_FALSE;      
  else return C_mk_bool(!C_memcmp((void *)x, (void *)y, len * sizeof(C_word)));
}

static C_word
signal_debug_event(C_word mode, C_word msg, C_word args)
{
  C_DEBUG_INFO cell;
  C_word av[ 3 ];
  cell.enabled = 1;
  cell.event = C_DEBUG_SIGNAL;
  cell.loc = "";
  cell.val = "";
  av[ 0 ] = mode;
  av[ 1 ] = msg;
  av[ 2 ] = args;
  C_debugger(&cell, 3, av);
  return C_SCHEME_UNDEFINED;
}

#ifdef NO_DLOAD2
# define HAVE_DLOAD 0
#else
# define HAVE_DLOAD 1
#endif

#ifdef C_ENABLE_PTABLES
# define HAVE_PTABLES 1
#else
# define HAVE_PTABLES 0
#endif

#ifdef C_GC_HOOKS
# define HAVE_GCHOOKS 1
#else
# define HAVE_GCHOOKS 0
#endif

#if defined(C_CROSS_CHICKEN) && C_CROSS_CHICKEN
# define IS_CROSS_CHICKEN 1
#else
# define IS_CROSS_CHICKEN 0
#endif


#define C_chdir(str) C_fix(chdir(C_c_string(str)))
#define C_curdir(buf) (getcwd(C_c_string(buf), 1024) ? C_fix(strlen(C_c_string(buf))) : C_SCHEME_FALSE)
#define C_getenventry(i) (environ[ i ])

#ifdef HAVE_CRT_EXTERNS_H
# include <crt_externs.h>
# define environ (*_NSGetEnviron())
#else
extern char **environ;
#endif

#ifdef HAVE_SETENV
# define C_unsetenv(s)    (unsetenv((char *)C_data_pointer(s)), C_SCHEME_TRUE)
# define C_setenv(x, y)   C_fix(setenv((char *)C_data_pointer(x), (char *)C_data_pointer(y), 1))
#else
# if defined(_WIN32) && !defined(__CYGWIN__)
#  define C_unsetenv(s)   C_setenv(s, C_SCHEME_FALSE)
# else
#  define C_unsetenv(s)   C_fix(putenv((char *)C_data_pointer(s)))
# endif
static C_word C_fcall C_setenv(C_word x, C_word y) {
  char *sx = C_c_string(x),
       *sy = (y == C_SCHEME_FALSE ? "" : C_c_string(y));
  int n1 = C_strlen(sx), n2 = C_strlen(sy);
  int buf_len = n1 + n2 + 2;
  char *buf = (char *)C_malloc(buf_len);
  if(buf == NULL) return(C_fix(0));
  else {
    C_strlcpy(buf, sx, buf_len);
    C_strlcat(buf, "=", buf_len);
    C_strlcat(buf, sy, buf_len);
    return(C_fix(putenv(buf)));
  }
}
#endif



static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_build_2dversion_toplevel)
C_externimport void C_ccall C_build_2dversion_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[1405];
static double C_possibly_force_alignment;
static C_char C_TLS li0[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,110,111,116,32,120,41,0,0};
static C_char C_TLS li1[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,98,111,111,108,101,97,110,63,32,120,41,0,0,0,0,0};
static C_char C_TLS li2[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,101,113,63,32,120,32,121,41};
static C_char C_TLS li3[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,101,113,118,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li4[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,101,113,117,97,108,63,32,120,32,121,41,0,0,0,0,0};
static C_char C_TLS li5[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,112,97,105,114,63,32,120,41};
static C_char C_TLS li6[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,111,110,115,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li7[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,99,97,114,32,120,41,0,0};
static C_char C_TLS li8[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,99,100,114,32,120,41,0,0};
static C_char C_TLS li9[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,115,101,116,45,99,97,114,33,32,120,32,121,41,0,0,0};
static C_char C_TLS li10[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,115,101,116,45,99,100,114,33,32,120,32,121,41,0,0,0};
static C_char C_TLS li11[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,99,97,100,114,32,120,41,0};
static C_char C_TLS li12[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,97,100,100,114,32,120,41};
static C_char C_TLS li13[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,100,100,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li14[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,100,100,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li15[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,99,97,97,114,32,120,41,0};
static C_char C_TLS li16[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,99,100,97,114,32,120,41,0};
static C_char C_TLS li17[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,99,100,100,114,32,120,41,0};
static C_char C_TLS li18[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,97,97,97,114,32,120,41};
static C_char C_TLS li19[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,97,97,100,114,32,120,41};
static C_char C_TLS li20[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,97,100,97,114,32,120,41};
static C_char C_TLS li21[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,100,97,97,114,32,120,41};
static C_char C_TLS li22[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,100,97,100,114,32,120,41};
static C_char C_TLS li23[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,100,100,97,114,32,120,41};
static C_char C_TLS li24[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,100,100,100,114,32,120,41};
static C_char C_TLS li25[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,97,97,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li26[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,97,97,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li27[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,97,100,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li28[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,97,100,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li29[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,100,97,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li30[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,100,97,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li31[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,97,100,100,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li32[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,97,97,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li33[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,97,97,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li34[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,97,100,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li35[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,97,100,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li36[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,100,97,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li37[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,100,97,100,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li38[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,99,100,100,100,97,114,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li39[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,110,117,108,108,63,32,120,41};
static C_char C_TLS li40[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,108,105,115,116,32,46,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li41[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,108,101,110,103,116,104,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li42[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,108,105,115,116,45,116,97,105,108,32,108,115,116,32,105,41};
static C_char C_TLS li43[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,108,105,115,116,45,114,101,102,32,108,115,116,32,105,41,0};
static C_char C_TLS li44[] C_aligned={C_lihdr(0,0,15),40,108,111,111,112,32,108,115,116,32,114,101,115,116,41,0};
static C_char C_TLS li45[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,114,101,118,101,114,115,101,32,108,115,116,48,41,0,0,0};
static C_char C_TLS li46[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,109,101,109,113,32,120,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li47[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,109,101,109,118,32,120,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li48[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,109,101,109,98,101,114,32,120,32,108,115,116,41,0,0,0};
static C_char C_TLS li49[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,97,115,115,113,32,120,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li50[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,97,115,115,118,32,120,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li51[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,97,115,115,111,99,32,120,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li52[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,108,105,115,116,63,32,120,41};
static C_char C_TLS li53[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,115,116,114,105,110,103,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li54[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,108,101,110,103,116,104,32,115,41};
static C_char C_TLS li55[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,114,101,102,32,115,32,105,41,0};
static C_char C_TLS li56[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,115,101,116,33,32,115,32,105,32,99,41,0,0,0,0,0,0};
static C_char C_TLS li57[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,115,116,114,105,110,103,61,63,32,120,32,121,41,0,0,0};
static C_char C_TLS li58[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,105,61,63,32,120,32,121,41};
static C_char C_TLS li59[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,101,120,97,99,116,45,62,105,110,101,120,97,99,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li60[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,97,98,115,32,120,41,0,0};
static C_char C_TLS li61[] C_aligned={C_lihdr(0,0,6),67,95,112,108,117,115,0,0};
static C_char C_TLS li62[] C_aligned={C_lihdr(0,0,7),67,95,109,105,110,117,115,0};
static C_char C_TLS li63[] C_aligned={C_lihdr(0,0,7),67,95,116,105,109,101,115,0};
static C_char C_TLS li64[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,113,117,111,116,105,101,110,116,32,97,32,98,41,0,0,0};
static C_char C_TLS li65[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,114,101,109,97,105,110,100,101,114,32,97,32,98,41,0,0};
static C_char C_TLS li66[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,109,111,100,117,108,111,32,97,32,98,41,0,0,0,0,0};
static C_char C_TLS li67[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,101,118,101,110,63,32,110,41};
static C_char C_TLS li68[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,111,100,100,63,32,110,41,0};
static C_char C_TLS li69[] C_aligned={C_lihdr(0,0,9),67,95,110,101,113,117,97,108,112,0,0,0,0,0,0,0};
static C_char C_TLS li70[] C_aligned={C_lihdr(0,0,10),67,95,103,114,101,97,116,101,114,112,0,0,0,0,0,0};
static C_char C_TLS li71[] C_aligned={C_lihdr(0,0,7),67,95,108,101,115,115,112,0};
static C_char C_TLS li72[] C_aligned={C_lihdr(0,0,20),67,95,103,114,101,97,116,101,114,95,111,114,95,101,113,117,97,108,95,112,0,0,0,0};
static C_char C_TLS li73[] C_aligned={C_lihdr(0,0,17),67,95,108,101,115,115,95,111,114,95,101,113,117,97,108,95,112,0,0,0,0,0,0,0};
static C_char C_TLS li74[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,110,117,109,98,101,114,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li75[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,114,101,97,108,63,32,120,41};
static C_char C_TLS li76[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,114,97,116,105,111,110,97,108,63,32,110,41,0,0,0,0};
static C_char C_TLS li77[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,105,110,116,101,103,101,114,63,32,120,41,0,0,0,0,0};
static C_char C_TLS li78[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,101,120,97,99,116,63,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li79[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,105,110,101,120,97,99,116,63,32,120,41,0,0,0,0,0};
static C_char C_TLS li80[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,122,101,114,111,63,32,110,41};
static C_char C_TLS li81[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,112,111,115,105,116,105,118,101,63,32,110,41,0,0,0,0};
static C_char C_TLS li82[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,110,101,103,97,116,105,118,101,63,32,110,41,0,0,0,0};
static C_char C_TLS li83[] C_aligned={C_lihdr(0,0,18),67,95,110,117,109,98,101,114,95,116,111,95,115,116,114,105,110,103,0,0,0,0,0,0};
static C_char C_TLS li84[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,115,121,109,98,111,108,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li85[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,118,101,99,116,111,114,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li86[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,118,101,99,116,111,114,45,108,101,110,103,116,104,32,118,41};
static C_char C_TLS li87[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,118,101,99,116,111,114,45,114,101,102,32,118,32,105,41,0};
static C_char C_TLS li88[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,118,101,99,116,111,114,45,115,101,116,33,32,118,32,105,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li89[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,99,104,97,114,63,32,120,41};
static C_char C_TLS li90[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,99,104,97,114,45,62,105,110,116,101,103,101,114,32,99,41};
static C_char C_TLS li91[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,105,110,116,101,103,101,114,45,62,99,104,97,114,32,110,41};
static C_char C_TLS li92[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,99,104,97,114,61,63,32,99,49,32,99,50,41,0,0,0};
static C_char C_TLS li93[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,99,104,97,114,62,63,32,99,49,32,99,50,41,0,0,0};
static C_char C_TLS li94[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,99,104,97,114,60,63,32,99,49,32,99,50,41,0,0,0};
static C_char C_TLS li95[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,62,61,63,32,99,49,32,99,50,41,0,0};
static C_char C_TLS li96[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,60,61,63,32,99,49,32,99,50,41,0,0};
static C_char C_TLS li97[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,45,117,112,99,97,115,101,32,99,41,0,0};
static C_char C_TLS li98[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,99,104,97,114,45,100,111,119,110,99,97,115,101,32,99,41};
static C_char C_TLS li99[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,99,104,97,114,45,117,112,112,101,114,45,99,97,115,101,63,32,99,41,0,0,0,0,0};
static C_char C_TLS li100[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,99,104,97,114,45,108,111,119,101,114,45,99,97,115,101,63,32,99,41,0,0,0,0,0};
static C_char C_TLS li101[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,99,104,97,114,45,110,117,109,101,114,105,99,63,32,99,41};
static C_char C_TLS li102[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,99,104,97,114,45,119,104,105,116,101,115,112,97,99,101,63,32,99,41,0,0,0,0,0};
static C_char C_TLS li103[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,99,104,97,114,45,97,108,112,104,97,98,101,116,105,99,63,32,99,41,0,0,0,0,0};
static C_char C_TLS li104[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,112,114,111,99,101,100,117,114,101,63,32,120,41,0,0,0};
static C_char C_TLS li105[] C_aligned={C_lihdr(0,0,7),67,95,97,112,112,108,121,0};
static C_char C_TLS li106[] C_aligned={C_lihdr(0,0,8),67,95,118,97,108,117,101,115};
static C_char C_TLS li107[] C_aligned={C_lihdr(0,0,18),67,95,99,97,108,108,95,119,105,116,104,95,118,97,108,117,101,115,0,0,0,0,0,0};
static C_char C_TLS li108[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,105,110,112,117,116,45,112,111,114,116,63,32,120,41,0,0};
static C_char C_TLS li109[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,111,117,116,112,117,116,45,112,111,114,116,63,32,120,41,0};
static C_char C_TLS li110[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,101,111,102,45,111,98,106,101,99,116,63,32,120,41,0,0};
static C_char C_TLS li111[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,101,118,97,108,32,120,32,46,32,101,110,118,41,0,0,0};
static C_char C_TLS li112[] C_aligned={C_lihdr(0,0,32),40,115,99,104,101,109,101,35,105,110,116,101,114,97,99,116,105,111,110,45,101,110,118,105,114,111,110,109,101,110,116,41};
static C_char C_TLS li113[] C_aligned={C_lihdr(0,0,36),40,115,99,104,101,109,101,35,115,99,104,101,109,101,45,114,101,112,111,114,116,45,101,110,118,105,114,111,110,109,101,110,116,32,110,41,0,0,0,0};
static C_char C_TLS li114[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,110,117,108,108,45,101,110,118,105,114,111,110,109,101,110,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li115[] C_aligned={C_lihdr(0,0,34),40,115,99,104,101,109,101,35,108,111,97,100,32,102,105,108,101,110,97,109,101,32,46,32,101,118,97,108,117,97,116,111,114,41,0,0,0,0,0,0};
static C_char C_TLS li116[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,105,120,110,117,109,63,32,120,41};
static C_char C_TLS li117[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,108,111,110,117,109,63,32,120,41};
static C_char C_TLS li118[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,98,105,103,110,117,109,63,32,120,41};
static C_char C_TLS li119[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,114,97,116,110,117,109,63,32,120,41};
static C_char C_TLS li120[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,112,108,120,110,117,109,63,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li121[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,120,97,99,116,45,105,110,116,101,103,101,114,63,32,120,41,0};
static C_char C_TLS li122[] C_aligned={C_lihdr(0,0,24),67,95,113,117,111,116,105,101,110,116,95,97,110,100,95,114,101,109,97,105,110,100,101,114};
static C_char C_TLS li123[] C_aligned={C_lihdr(0,0,7),40,97,57,50,49,56,41,0};
static C_char C_TLS li124[] C_aligned={C_lihdr(0,0,15),40,97,57,50,50,52,32,100,105,118,32,114,101,109,41,0};
static C_char C_TLS li125[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,97,115,101,35,113,117,111,116,105,101,110,116,38,109,111,100,117,108,111,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li126[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,105,110,105,116,101,63,32,120,41};
static C_char C_TLS li127[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,98,97,115,101,35,105,110,102,105,110,105,116,101,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li128[] C_aligned={C_lihdr(0,0,21),40,99,104,105,99,107,101,110,46,98,97,115,101,35,110,97,110,63,32,120,41,0,0,0};
static C_char C_TLS li129[] C_aligned={C_lihdr(0,0,8),67,95,115,105,103,110,117,109};
static C_char C_TLS li130[] C_aligned={C_lihdr(0,0,21),40,99,104,105,99,107,101,110,46,98,97,115,101,35,97,100,100,49,32,110,41,0,0,0};
static C_char C_TLS li131[] C_aligned={C_lihdr(0,0,21),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,117,98,49,32,110,41,0,0,0};
static C_char C_TLS li132[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,114,114,111,114,32,46,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li133[] C_aligned={C_lihdr(0,0,23),40,99,104,105,99,107,101,110,46,98,97,115,101,35,118,111,105,100,32,46,32,95,41,0};
static C_char C_TLS li134[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,114,111,109,105,115,101,63,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li135[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,109,97,107,101,45,112,114,111,109,105,115,101,32,112,114,111,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li136[] C_aligned={C_lihdr(0,0,7),40,97,57,51,52,56,41,0};
static C_char C_TLS li137[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,98,97,115,101,35,109,97,107,101,45,112,114,111,109,105,115,101,32,111,98,106,41,0};
static C_char C_TLS li138[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,108,115,116,32,122,41,0,0,0,0};
static C_char C_TLS li139[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,111,108,100,108,32,102,32,122,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li140[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li141[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,111,108,100,114,32,102,32,122,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li142[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,111,110,45,101,120,105,116,32,116,104,117,110,107,41,0,0,0,0};
static C_char C_TLS li143[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,120,105,116,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li144[] C_aligned={C_lihdr(0,0,36),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,109,101,114,103,101,110,99,121,45,101,120,105,116,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li145[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,98,97,115,101,35,105,100,101,110,116,105,116,121,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li146[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,114,101,100,115,41,0,0,0,0};
static C_char C_TLS li147[] C_aligned={C_lihdr(0,0,10),40,102,95,57,52,55,56,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li148[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,111,110,106,111,105,110,32,46,32,112,114,101,100,115,41,0,0};
static C_char C_TLS li149[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,114,101,100,115,41,0,0,0,0};
static C_char C_TLS li150[] C_aligned={C_lihdr(0,0,10),40,102,95,57,53,49,49,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li151[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,97,115,101,35,100,105,115,106,111,105,110,32,46,32,112,114,101,100,115,41,0,0};
static C_char C_TLS li152[] C_aligned={C_lihdr(0,0,12),40,102,95,57,53,53,55,32,46,32,95,41,0,0,0,0};
static C_char C_TLS li153[] C_aligned={C_lihdr(0,0,12),40,102,95,57,53,53,57,32,46,32,95,41,0,0,0,0};
static C_char C_TLS li154[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,111,110,115,116,97,110,116,108,121,32,46,32,120,115,41,0,0};
static C_char C_TLS li155[] C_aligned={C_lihdr(0,0,12),40,102,95,57,53,54,57,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li156[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,108,105,112,32,112,114,111,99,41};
static C_char C_TLS li157[] C_aligned={C_lihdr(0,0,15),40,102,95,57,53,55,55,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li158[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,111,109,112,108,101,109,101,110,116,32,112,41,0,0,0,0,0};
static C_char C_TLS li159[] C_aligned={C_lihdr(0,0,7),40,97,57,54,48,51,41,0};
static C_char C_TLS li160[] C_aligned={C_lihdr(0,0,15),40,102,95,57,53,57,56,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li161[] C_aligned={C_lihdr(0,0,14),40,114,101,99,32,102,48,32,46,32,102,110,115,41,0,0};
static C_char C_TLS li162[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,111,109,112,111,115,101,32,46,32,102,110,115,41,0,0,0,0};
static C_char C_TLS li163[] C_aligned={C_lihdr(0,0,10),40,102,95,57,54,52,57,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li164[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,102,110,115,41,0,0,0,0,0,0};
static C_char C_TLS li165[] C_aligned={C_lihdr(0,0,22),40,99,104,105,99,107,101,110,46,98,97,115,101,35,111,32,46,32,102,110,115,41,0,0};
static C_char C_TLS li166[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li167[] C_aligned={C_lihdr(0,0,12),40,102,95,57,54,54,52,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li168[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,108,105,115,116,45,111,102,63,32,112,114,101,100,41,0,0,0,0};
static C_char C_TLS li169[] C_aligned={C_lihdr(0,0,12),40,102,95,57,55,49,52,32,46,32,95,41,0,0,0,0};
static C_char C_TLS li170[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,114,111,99,115,41,0,0,0,0};
static C_char C_TLS li171[] C_aligned={C_lihdr(0,0,15),40,102,95,57,55,50,56,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li172[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,97,99,104,32,46,32,112,114,111,99,115,41,0,0,0,0,0};
static C_char C_TLS li173[] C_aligned={C_lihdr(0,0,22),40,99,104,105,99,107,101,110,46,98,97,115,101,35,97,116,111,109,63,32,120,41,0,0};
static C_char C_TLS li174[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li175[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,98,97,115,101,35,116,97,105,108,63,32,120,32,121,41};
static C_char C_TLS li176[] C_aligned={C_lihdr(0,0,9),40,108,111,111,112,32,110,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li177[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,98,97,115,101,35,105,110,116,101,114,115,112,101,114,115,101,32,108,115,116,32,120,41};
static C_char C_TLS li178[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li179[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,98,97,115,101,35,98,117,116,108,97,115,116,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li180[] C_aligned={C_lihdr(0,0,17),40,108,111,111,112,32,108,105,115,116,115,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li181[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,108,97,116,116,101,110,32,46,32,108,105,115,116,115,48,41,0};
static C_char C_TLS li182[] C_aligned={C_lihdr(0,0,11),40,108,111,111,112,32,108,115,116,115,41,0,0,0,0,0};
static C_char C_TLS li183[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,97,115,101,35,106,111,105,110,32,108,115,116,115,32,46,32,108,115,116,41,0,0};
static C_char C_TLS li184[] C_aligned={C_lihdr(0,0,15),40,108,111,111,112,32,98,108,115,116,32,108,115,116,41,0};
static C_char C_TLS li185[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,111,109,112,114,101,115,115,32,98,108,115,116,32,108,115,116,41};
static C_char C_TLS li186[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li187[] C_aligned={C_lihdr(0,0,15),40,102,95,49,48,48,55,53,32,120,32,108,115,116,41,0};
static C_char C_TLS li188[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,98,97,115,101,35,97,108,105,115,116,45,117,112,100,97,116,101,33,32,120,32,121,32,108,115,116,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li189[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li190[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,98,97,115,101,35,97,108,105,115,116,45,117,112,100,97,116,101,32,107,32,118,32,108,115,116,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li191[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li192[] C_aligned={C_lihdr(0,0,15),40,102,95,49,48,50,54,55,32,120,32,108,115,116,41,0};
static C_char C_TLS li193[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,98,97,115,101,35,97,108,105,115,116,45,114,101,102,32,120,32,108,115,116,32,46,32,114,101,115,116,41,0,0,0};
static C_char C_TLS li194[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,108,41};
static C_char C_TLS li195[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,98,97,115,101,35,114,97,115,115,111,99,32,120,32,108,115,116,32,46,32,116,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li196[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,43,32,120,32,121,41};
static C_char C_TLS li197[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,45,32,120,32,121,41};
static C_char C_TLS li198[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,42,32,120,32,121,41};
static C_char C_TLS li199[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,61,32,120,32,121,41};
static C_char C_TLS li200[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,62,32,120,32,121,41};
static C_char C_TLS li201[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,60,32,120,32,121,41};
static C_char C_TLS li202[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,62,61,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li203[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,60,61,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li204[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,109,105,110,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li205[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,109,97,120,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li206[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,110,101,103,32,120,41};
static C_char C_TLS li207[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,97,110,100,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li208[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,105,111,114,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li209[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,120,111,114,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li210[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,110,111,116,32,120,41};
static C_char C_TLS li211[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,115,104,108,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li212[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,115,104,114,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li213[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,111,100,100,63,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li214[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,101,118,101,110,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li215[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,108,101,110,32,120,41};
static C_char C_TLS li216[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,47,32,120,32,121,41};
static C_char C_TLS li217[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,103,99,100,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li218[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,109,111,100,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li219[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,114,101,109,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li220[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,43,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li221[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,45,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li222[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,42,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li223[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,105,120,110,117,109,35,102,120,47,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li224[] C_aligned={C_lihdr(0,0,19),40,35,35,115,121,115,35,100,101,98,117,103,45,109,111,100,101,63,41,0,0,0,0,0};
static C_char C_TLS li225[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,98,97,115,101,35,119,97,114,110,105,110,103,32,109,115,103,32,46,32,97,114,103,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li226[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,98,97,115,101,35,110,111,116,105,99,101,32,109,115,103,32,46,32,97,114,103,115,41};
static C_char C_TLS li227[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,110,97,98,108,101,45,119,97,114,110,105,110,103,115,32,46,32,98,111,111,108,41,0,0,0};
static C_char C_TLS li228[] C_aligned={C_lihdr(0,0,4),67,95,103,99,0,0,0,0};
static C_char C_TLS li229[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,115,101,116,115,108,111,116,32,120,32,105,32,121,41,0,0,0};
static C_char C_TLS li230[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,115,101,116,105,115,108,111,116,32,120,32,105,32,121,41,0,0};
static C_char C_TLS li231[] C_aligned={C_lihdr(0,0,17),67,95,97,108,108,111,99,97,116,101,95,118,101,99,116,111,114,0,0,0,0,0,0,0};
static C_char C_TLS li232[] C_aligned={C_lihdr(0,0,16),67,95,109,97,107,101,95,115,116,114,117,99,116,117,114,101};
static C_char C_TLS li233[] C_aligned={C_lihdr(0,0,21),67,95,101,110,115,117,114,101,95,104,101,97,112,95,114,101,115,101,114,118,101,0,0,0};
static C_char C_TLS li234[] C_aligned={C_lihdr(0,0,23),67,95,103,101,116,95,115,121,109,98,111,108,95,116,97,98,108,101,95,105,110,102,111,0};
static C_char C_TLS li235[] C_aligned={C_lihdr(0,0,17),67,95,103,101,116,95,109,101,109,111,114,121,95,105,110,102,111,0,0,0,0,0,0,0};
static C_char C_TLS li236[] C_aligned={C_lihdr(0,0,19),40,35,35,115,121,115,35,115,116,97,114,116,45,116,105,109,101,114,41,0,0,0,0,0};
static C_char C_TLS li237[] C_aligned={C_lihdr(0,0,12),67,95,115,116,111,112,95,116,105,109,101,114,0,0,0,0};
static C_char C_TLS li238[] C_aligned={C_lihdr(0,0,12),67,95,115,116,111,112,95,116,105,109,101,114,0,0,0,0};
static C_char C_TLS li239[] C_aligned={C_lihdr(0,0,18),40,35,35,115,121,115,35,115,116,111,112,45,116,105,109,101,114,41,0,0,0,0,0,0};
static C_char C_TLS li240[] C_aligned={C_lihdr(0,0,20),40,35,35,115,121,115,35,105,109,109,101,100,105,97,116,101,63,32,120,41,0,0,0,0};
static C_char C_TLS li241[] C_aligned={C_lihdr(0,0,19),40,35,35,115,121,115,35,109,101,115,115,97,103,101,32,115,116,114,41,0,0,0,0,0};
static C_char C_TLS li242[] C_aligned={C_lihdr(0,0,16),40,35,35,115,121,115,35,98,121,116,101,32,120,32,105,41};
static C_char C_TLS li243[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,115,101,116,98,121,116,101,32,120,32,105,32,110,41,0,0,0};
static C_char C_TLS li244[] C_aligned={C_lihdr(0,0,16),40,35,35,115,121,115,35,104,97,108,116,32,109,115,103,41};
static C_char C_TLS li245[] C_aligned={C_lihdr(0,0,8),67,95,98,101,99,111,109,101};
static C_char C_TLS li246[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,98,108,111,99,107,45,114,101,102,32,120,32,105,41,0,0,0};
static C_char C_TLS li247[] C_aligned={C_lihdr(0,0,14),67,95,97,112,112,108,121,95,118,97,108,117,101,115,0,0};
static C_char C_TLS li248[] C_aligned={C_lihdr(0,0,14),67,95,99,111,112,121,95,99,108,111,115,117,114,101,0,0};
static C_char C_TLS li249[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,98,108,111,99,107,45,115,101,116,33,32,120,32,105,32,121,41};
static C_char C_TLS li250[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,116,105,109,101,35,99,117,114,114,101,110,116,45,109,105,108,108,105,115,101,99,111,110,100,115,41,0,0,0,0,0};
static C_char C_TLS li251[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,116,105,109,101,35,99,117,114,114,101,110,116,45,112,114,111,99,101,115,115,45,109,105,108,108,105,115,101,99,111,110,100,115,41,0,0,0,0,0};
static C_char C_TLS li252[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,116,105,109,101,35,99,117,114,114,101,110,116,45,115,101,99,111,110,100,115,41,0,0};
static C_char C_TLS li253[] C_aligned={C_lihdr(0,0,23),40,99,104,105,99,107,101,110,46,116,105,109,101,35,99,112,117,45,116,105,109,101,41,0};
static C_char C_TLS li254[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,99,104,101,99,107,45,115,116,114,117,99,116,117,114,101,32,120,32,121,32,46,32,108,111,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li255[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,98,108,111,98,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li256[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,112,97,105,114,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li257[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,108,105,115,116,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li258[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,99,104,101,99,107,45,115,116,114,105,110,103,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li259[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,99,104,101,99,107,45,110,117,109,98,101,114,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li260[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,99,104,101,99,107,45,102,105,120,110,117,109,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li261[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,99,104,101,99,107,45,101,120,97,99,116,32,120,32,46,32,108,111,99,41,0,0,0,0,0};
static C_char C_TLS li262[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,99,104,101,99,107,45,105,110,101,120,97,99,116,32,120,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li263[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,99,104,101,99,107,45,115,121,109,98,111,108,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li264[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,99,104,101,99,107,45,107,101,121,119,111,114,100,32,120,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li265[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,99,104,101,99,107,45,118,101,99,116,111,114,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li266[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,99,104,97,114,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li267[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,99,104,101,99,107,45,98,111,111,108,101,97,110,32,120,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li268[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,99,104,101,99,107,45,108,111,99,97,116,105,118,101,32,120,32,46,32,108,111,99,41,0,0};
static C_char C_TLS li269[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,99,104,101,99,107,45,105,110,116,101,103,101,114,32,120,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li270[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,99,104,101,99,107,45,101,120,97,99,116,45,105,110,116,101,103,101,114,32,120,32,46,32,108,111,99,41,0,0,0,0,0};
static C_char C_TLS li271[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,99,104,101,99,107,45,101,120,97,99,116,45,117,105,110,116,101,103,101,114,32,120,32,46,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li272[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,114,101,97,108,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li273[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,99,104,101,99,107,45,114,97,110,103,101,32,105,32,102,114,111,109,32,116,111,32,46,32,108,111,99,41,0,0,0,0,0};
static C_char C_TLS li274[] C_aligned={C_lihdr(0,0,31),40,35,35,115,121,115,35,99,104,101,99,107,45,115,112,101,99,105,97,108,32,112,116,114,32,46,32,108,111,99,41,0};
static C_char C_TLS li275[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,99,104,101,99,107,45,99,108,111,115,117,114,101,32,120,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li276[] C_aligned={C_lihdr(0,0,20),40,108,112,32,112,114,111,109,105,115,101,32,102,111,114,119,97,114,100,41,0,0,0,0};
static C_char C_TLS li277[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,102,111,114,99,101,32,111,98,106,41,0,0,0,0,0,0};
static C_char C_TLS li278[] C_aligned={C_lihdr(0,0,7),67,95,100,108,111,97,100,0};
static C_char C_TLS li279[] C_aligned={C_lihdr(0,0,18),67,95,115,101,116,95,100,108,111,112,101,110,95,102,108,97,103,115,0,0,0,0,0,0};
static C_char C_TLS li280[] C_aligned={C_lihdr(0,0,42),40,35,35,115,121,115,35,101,114,114,111,114,45,110,111,116,45,97,45,112,114,111,112,101,114,45,108,105,115,116,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li281[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,110,117,109,98,101,114,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li282[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,105,110,116,101,103,101,114,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li283[] C_aligned={C_lihdr(0,0,42),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,101,120,97,99,116,45,105,110,116,101,103,101,114,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li284[] C_aligned={C_lihdr(0,0,43),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,101,120,97,99,116,45,117,105,110,116,101,103,101,114,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li285[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,105,110,101,120,97,99,116,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li286[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,114,101,97,108,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li287[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,101,114,114,111,114,45,98,97,100,45,98,97,115,101,32,97,114,103,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li288[] C_aligned={C_lihdr(0,0,11),40,99,111,112,121,32,110,111,100,101,41,0,0,0,0,0};
static C_char C_TLS li289[] C_aligned={C_lihdr(0,0,11),40,108,111,111,112,32,108,115,116,115,41,0,0,0,0,0};
static C_char C_TLS li290[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,97,112,112,101,110,100,32,46,32,108,115,116,115,41,0,0};
static C_char C_TLS li291[] C_aligned={C_lihdr(0,0,15),40,108,111,111,112,32,108,115,116,32,114,101,115,116,41,0};
static C_char C_TLS li292[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,102,97,115,116,45,114,101,118,101,114,115,101,32,108,115,116,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li293[] C_aligned={C_lihdr(0,0,31),40,35,35,115,121,115,35,109,97,107,101,45,115,116,114,105,110,103,32,115,105,122,101,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li294[] C_aligned={C_lihdr(0,0,32),40,115,99,104,101,109,101,35,109,97,107,101,45,115,116,114,105,110,103,32,115,105,122,101,32,46,32,102,105,108,108,41};
static C_char C_TLS li295[] C_aligned={C_lihdr(0,0,11),40,108,111,111,112,32,105,32,108,115,41,0,0,0,0,0};
static C_char C_TLS li296[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,62,108,105,115,116,32,115,41,0};
static C_char C_TLS li297[] C_aligned={C_lihdr(0,0,16),40,100,111,108,111,111,112,49,53,50,56,32,108,115,116,41};
static C_char C_TLS li298[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,108,105,115,116,45,62,115,116,114,105,110,103,32,108,115,116,48,41,0,0,0,0,0,0};
static C_char C_TLS li299[] C_aligned={C_lihdr(0,0,9),40,105,116,101,114,32,110,50,41,0,0,0,0,0,0,0};
static C_char C_TLS li300[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,114,101,118,101,114,115,101,45,108,105,115,116,45,62,115,116,114,105,110,103,32,108,41,0,0};
static C_char C_TLS li301[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,102,105,108,108,33,32,115,32,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li302[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,111,112,121,32,115,41,0,0};
static C_char C_TLS li303[] C_aligned={C_lihdr(0,0,32),40,115,99,104,101,109,101,35,115,117,98,115,116,114,105,110,103,32,115,32,115,116,97,114,116,32,46,32,101,110,100,41};
static C_char C_TLS li304[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,115,117,98,115,116,114,105,110,103,32,115,32,115,116,97,114,116,32,101,110,100,41,0,0,0};
static C_char C_TLS li305[] C_aligned={C_lihdr(0,0,25),40,99,111,109,112,97,114,101,49,53,56,49,32,115,49,32,115,50,32,108,111,99,32,107,41,0,0,0,0,0,0,0};
static C_char C_TLS li306[] C_aligned={C_lihdr(0,0,22),40,97,49,49,54,49,57,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li307[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,115,116,114,105,110,103,60,63,32,115,49,32,115,50,41,0};
static C_char C_TLS li308[] C_aligned={C_lihdr(0,0,22),40,97,49,49,54,52,51,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li309[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,115,116,114,105,110,103,62,63,32,115,49,32,115,50,41,0};
static C_char C_TLS li310[] C_aligned={C_lihdr(0,0,22),40,97,49,49,54,54,55,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li311[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,115,116,114,105,110,103,60,61,63,32,115,49,32,115,50,41};
static C_char C_TLS li312[] C_aligned={C_lihdr(0,0,22),40,97,49,49,54,56,56,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li313[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,115,116,114,105,110,103,62,61,63,32,115,49,32,115,50,41};
static C_char C_TLS li314[] C_aligned={C_lihdr(0,0,25),40,99,111,109,112,97,114,101,49,54,50,52,32,115,49,32,115,50,32,108,111,99,32,107,41,0,0,0,0,0,0,0};
static C_char C_TLS li315[] C_aligned={C_lihdr(0,0,22),40,97,49,49,55,50,57,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li316[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,105,60,63,32,115,49,32,115,50,41,0,0,0,0,0,0};
static C_char C_TLS li317[] C_aligned={C_lihdr(0,0,22),40,97,49,49,55,53,51,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li318[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,105,62,63,32,115,49,32,115,50,41,0,0,0,0,0,0};
static C_char C_TLS li319[] C_aligned={C_lihdr(0,0,22),40,97,49,49,55,55,55,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li320[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,105,60,61,63,32,115,49,32,115,50,41,0,0,0,0,0};
static C_char C_TLS li321[] C_aligned={C_lihdr(0,0,22),40,97,49,49,55,57,56,32,108,101,110,49,32,108,101,110,50,32,99,109,112,41,0,0};
static C_char C_TLS li322[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,99,105,62,61,63,32,115,49,32,115,50,41,0,0,0,0,0};
static C_char C_TLS li323[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,115,116,114,105,110,103,45,97,112,112,101,110,100,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li324[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,115,116,114,115,32,110,41,0,0,0};
static C_char C_TLS li325[] C_aligned={C_lihdr(0,0,28),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,97,112,112,101,110,100,32,46,32,97,108,108,41,0,0,0,0};
static C_char C_TLS li326[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,115,116,114,105,110,103,32,46,32,99,104,97,114,115,41,0};
static C_char C_TLS li327[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,112,111,115,41,0,0,0,0,0,0};
static C_char C_TLS li328[] C_aligned={C_lihdr(0,0,34),40,35,35,115,121,115,35,102,114,97,103,109,101,110,116,115,45,62,115,116,114,105,110,103,32,116,111,116,97,108,32,102,115,41,0,0,0,0,0,0};
static C_char C_TLS li329[] C_aligned={C_lihdr(0,0,20),40,100,111,108,111,111,112,49,55,48,57,32,104,100,32,116,108,32,99,41,0,0,0,0};
static C_char C_TLS li330[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,108,115,116,32,105,41,0,0,0,0};
static C_char C_TLS li331[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,104,111,112,32,108,115,116,32,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li332[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,43,32,120,32,121,41};
static C_char C_TLS li333[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,45,32,120,32,121,41};
static C_char C_TLS li334[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,42,32,120,32,121,41};
static C_char C_TLS li335[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,47,32,120,32,121,41};
static C_char C_TLS li336[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,103,99,100,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li337[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,47,63,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li338[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,61,32,120,32,121,41};
static C_char C_TLS li339[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,62,32,120,32,121,41};
static C_char C_TLS li340[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,60,32,120,32,121,41};
static C_char C_TLS li341[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,62,61,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li342[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,60,61,32,120,32,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li343[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,110,101,103,32,120,41};
static C_char C_TLS li344[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,109,97,120,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li345[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,109,105,110,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li346[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,102,108,111,111,114,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li347[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,116,114,117,110,99,97,116,101,32,120,41,0,0,0};
static C_char C_TLS li348[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,114,111,117,110,100,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li349[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,99,101,105,108,105,110,103,32,120,41,0,0,0,0};
static C_char C_TLS li350[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,115,105,110,32,120,41};
static C_char C_TLS li351[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,99,111,115,32,120,41};
static C_char C_TLS li352[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,116,97,110,32,120,41};
static C_char C_TLS li353[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,97,115,105,110,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li354[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,97,99,111,115,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li355[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,97,116,97,110,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li356[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,97,116,97,110,50,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li357[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,101,120,112,32,120,41};
static C_char C_TLS li358[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,101,120,112,116,32,120,32,121,41,0,0,0,0,0};
static C_char C_TLS li359[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,108,111,103,32,120,41};
static C_char C_TLS li360[] C_aligned={C_lihdr(0,0,25),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,115,113,114,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li361[] C_aligned={C_lihdr(0,0,24),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,97,98,115,32,120,41};
static C_char C_TLS li362[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,112,105,110,116,101,103,101,114,63,32,120,41,0,0,0};
static C_char C_TLS li363[] C_aligned={C_lihdr(0,0,46),40,99,104,105,99,107,101,110,46,102,108,111,110,117,109,35,102,108,111,110,117,109,45,112,114,105,110,116,45,112,114,101,99,105,115,105,111,110,32,46,32,114,101,115,116,41,0,0};
static C_char C_TLS li364[] C_aligned={C_lihdr(0,0,18),40,109,97,107,101,45,99,111,109,112,108,101,120,32,114,32,105,41,0,0,0,0,0,0};
static C_char C_TLS li365[] C_aligned={C_lihdr(0,0,29),40,115,99,104,101,109,101,35,109,97,107,101,45,114,101,99,116,97,110,103,117,108,97,114,32,114,32,105,41,0,0,0};
static C_char C_TLS li366[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,109,97,107,101,45,112,111,108,97,114,32,114,32,112,104,105,41,0,0,0,0,0,0,0};
static C_char C_TLS li367[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,114,101,97,108,45,112,97,114,116,32,120,41,0,0,0,0};
static C_char C_TLS li368[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,105,109,97,103,45,112,97,114,116,32,120,41,0,0,0,0};
static C_char C_TLS li369[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,97,110,103,108,101,32,110,41};
static C_char C_TLS li370[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,109,97,103,110,105,116,117,100,101,32,120,41,0,0,0,0};
static C_char C_TLS li371[] C_aligned={C_lihdr(0,0,10),40,114,97,116,110,117,109,32,110,41,0,0,0,0,0,0};
static C_char C_TLS li372[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,110,117,109,101,114,97,116,111,114,32,110,41,0,0,0,0};
static C_char C_TLS li373[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,100,101,110,111,109,105,110,97,116,111,114,32,110,41,0,0};
static C_char C_TLS li374[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,101,120,116,101,110,100,101,100,45,115,105,103,110,117,109,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li375[] C_aligned={C_lihdr(0,0,16),40,100,111,108,111,111,112,50,50,50,48,32,120,32,105,41};
static C_char C_TLS li376[] C_aligned={C_lihdr(0,0,13),40,100,101,108,105,118,101,114,32,121,32,100,41,0,0,0};
static C_char C_TLS li377[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,105,110,101,120,97,99,116,45,62,101,120,97,99,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li378[] C_aligned={C_lihdr(0,0,13),67,95,98,105,116,119,105,115,101,95,97,110,100,0,0,0};
static C_char C_TLS li379[] C_aligned={C_lihdr(0,0,13),67,95,98,105,116,119,105,115,101,95,105,111,114,0,0,0};
static C_char C_TLS li380[] C_aligned={C_lihdr(0,0,13),67,95,98,105,116,119,105,115,101,95,120,111,114,0,0,0};
static C_char C_TLS li381[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,98,105,116,119,105,115,101,35,98,105,116,119,105,115,101,45,110,111,116,32,110,41,0};
static C_char C_TLS li382[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,105,116,119,105,115,101,35,98,105,116,45,62,98,111,111,108,101,97,110,32,110,32,105,41,0,0,0,0,0,0};
static C_char C_TLS li383[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,105,116,119,105,115,101,35,105,110,116,101,103,101,114,45,108,101,110,103,116,104,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li384[] C_aligned={C_lihdr(0,0,38),40,99,104,105,99,107,101,110,46,98,105,116,119,105,115,101,35,97,114,105,116,104,109,101,116,105,99,45,115,104,105,102,116,32,110,32,109,41,0,0};
static C_char C_TLS li385[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,97,114,103,115,32,120,41,0,0,0};
static C_char C_TLS li386[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,47,32,97,114,103,49,32,46,32,97,114,103,115,41,0,0};
static C_char C_TLS li387[] C_aligned={C_lihdr(0,0,15),40,35,35,115,121,115,35,47,45,50,32,120,32,121,41,0};
static C_char C_TLS li388[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,102,108,111,111,114,32,120,41};
static C_char C_TLS li389[] C_aligned={C_lihdr(0,0,18),40,115,99,104,101,109,101,35,99,101,105,108,105,110,103,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li390[] C_aligned={C_lihdr(0,0,19),40,115,99,104,101,109,101,35,116,114,117,110,99,97,116,101,32,120,41,0,0,0,0,0};
static C_char C_TLS li391[] C_aligned={C_lihdr(0,0,16),40,115,99,104,101,109,101,35,114,111,117,110,100,32,120,41};
static C_char C_TLS li392[] C_aligned={C_lihdr(0,0,8),40,115,114,32,120,32,121,41};
static C_char C_TLS li393[] C_aligned={C_lihdr(0,0,24),40,102,105,110,100,45,114,97,116,105,111,45,98,101,116,119,101,101,110,32,120,32,121,41};
static C_char C_TLS li394[] C_aligned={C_lihdr(0,0,24),40,115,99,104,101,109,101,35,114,97,116,105,111,110,97,108,105,122,101,32,120,32,101,41};
static C_char C_TLS li395[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,105,32,109,32,120,115,41,0,0,0};
static C_char C_TLS li396[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,109,97,120,32,120,49,32,46,32,120,115,41,0,0,0,0};
static C_char C_TLS li397[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,105,32,109,32,120,115,41,0,0,0};
static C_char C_TLS li398[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,109,105,110,32,120,49,32,46,32,120,115,41,0,0,0,0};
static C_char C_TLS li399[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,101,120,112,32,110,41,0,0};
static C_char C_TLS li400[] C_aligned={C_lihdr(0,0,15),40,35,35,115,121,115,35,108,111,103,45,49,32,120,41,0};
static C_char C_TLS li401[] C_aligned={C_lihdr(0,0,21),40,115,99,104,101,109,101,35,108,111,103,32,97,32,46,32,114,101,115,116,41,0,0,0};
static C_char C_TLS li402[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,115,105,110,32,110,41,0,0};
static C_char C_TLS li403[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,99,111,115,32,110,41,0,0};
static C_char C_TLS li404[] C_aligned={C_lihdr(0,0,14),40,115,99,104,101,109,101,35,116,97,110,32,110,41,0,0};
static C_char C_TLS li405[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,97,115,105,110,32,110,41,0};
static C_char C_TLS li406[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,97,99,111,115,32,110,41,0};
static C_char C_TLS li407[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,97,116,97,110,32,110,32,46,32,114,101,115,116,41,0,0};
static C_char C_TLS li408[] C_aligned={C_lihdr(0,0,8),40,97,49,52,52,49,57,41};
static C_char C_TLS li409[] C_aligned={C_lihdr(0,0,34),67,95,117,95,105,110,116,101,103,101,114,95,113,117,111,116,105,101,110,116,95,97,110,100,95,114,101,109,97,105,110,100,101,114,0,0,0,0,0,0};
static C_char C_TLS li410[] C_aligned={C_lihdr(0,0,8),40,97,49,52,52,53,48,41};
static C_char C_TLS li411[] C_aligned={C_lihdr(0,0,20),40,97,49,52,52,54,54,32,113,50,53,56,56,32,117,50,53,57,48,41,0,0,0,0};
static C_char C_TLS li412[] C_aligned={C_lihdr(0,0,22),40,97,49,52,52,51,51,32,115,94,50,53,54,56,32,114,94,50,53,55,48,41,0,0};
static C_char C_TLS li413[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,101,120,97,99,116,45,105,110,116,101,103,101,114,45,115,113,114,116,32,97,41,0,0,0,0};
static C_char C_TLS li414[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,120,97,99,116,45,105,110,116,101,103,101,114,45,115,113,114,116,32,120,41,0,0,0,0,0};
static C_char C_TLS li415[] C_aligned={C_lihdr(0,0,8),40,97,49,52,54,48,55,41};
static C_char C_TLS li416[] C_aligned={C_lihdr(0,0,14),40,97,49,52,54,49,51,32,115,94,50,32,114,41,0,0};
static C_char C_TLS li417[] C_aligned={C_lihdr(0,0,8),40,97,49,52,54,51,53,41};
static C_char C_TLS li418[] C_aligned={C_lihdr(0,0,8),40,97,49,52,54,53,57,41};
static C_char C_TLS li419[] C_aligned={C_lihdr(0,0,16),40,97,49,52,54,55,49,32,100,115,94,50,32,100,114,41};
static C_char C_TLS li420[] C_aligned={C_lihdr(0,0,16),40,97,49,52,54,52,55,32,110,115,94,50,32,110,114,41};
static C_char C_TLS li421[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,115,113,114,116,47,108,111,99,32,108,111,99,32,110,41,0,0};
static C_char C_TLS li422[] C_aligned={C_lihdr(0,0,15),40,115,99,104,101,109,101,35,115,113,114,116,32,120,41,0};
static C_char C_TLS li423[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,120,97,99,116,45,105,110,116,101,103,101,114,45,110,116,104,45,114,111,111,116,32,107,32,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li424[] C_aligned={C_lihdr(0,0,10),40,108,112,32,103,48,32,103,49,41,0,0,0,0,0,0};
static C_char C_TLS li425[] C_aligned={C_lihdr(0,0,42),40,35,35,115,121,115,35,101,120,97,99,116,45,105,110,116,101,103,101,114,45,110,116,104,45,114,111,111,116,47,108,111,99,32,108,111,99,32,107,32,110,41,0,0,0,0,0,0};
static C_char C_TLS li426[] C_aligned={C_lihdr(0,0,11),40,108,112,32,114,101,115,32,101,50,41,0,0,0,0,0};
static C_char C_TLS li427[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,105,110,116,101,103,101,114,45,112,111,119,101,114,32,98,97,115,101,32,101,41,0,0,0,0};
static C_char C_TLS li428[] C_aligned={C_lihdr(0,0,14),40,108,111,103,45,101,120,112,116,32,97,32,98,41,0,0};
static C_char C_TLS li429[] C_aligned={C_lihdr(0,0,15),40,115,108,111,119,45,101,120,112,116,32,97,32,98,41,0};
static C_char C_TLS li430[] C_aligned={C_lihdr(0,0,8),40,97,49,53,48,50,56,41};
static C_char C_TLS li431[] C_aligned={C_lihdr(0,0,15),40,97,49,53,48,52,48,32,100,115,94,110,32,114,41,0};
static C_char C_TLS li432[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,101,120,112,116,32,97,32,98,41,0,0,0,0,0,0,0};
static C_char C_TLS li433[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,105,110,116,101,114,110,97,108,45,103,99,100,32,108,111,99,32,97,32,98,41,0,0,0,0};
static C_char C_TLS li434[] C_aligned={C_lihdr(0,0,15),40,35,35,115,121,115,35,103,99,100,32,97,32,98,41,0};
static C_char C_TLS li435[] C_aligned={C_lihdr(0,0,16),40,108,111,111,112,32,104,101,97,100,32,110,101,120,116,41};
static C_char C_TLS li436[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,103,99,100,32,46,32,110,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li437[] C_aligned={C_lihdr(0,0,15),40,35,35,115,121,115,35,108,99,109,32,120,32,121,41,0};
static C_char C_TLS li438[] C_aligned={C_lihdr(0,0,16),40,108,111,111,112,32,104,101,97,100,32,110,101,120,116,41};
static C_char C_TLS li439[] C_aligned={C_lihdr(0,0,17),40,115,99,104,101,109,101,35,108,99,109,32,46,32,110,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li440[] C_aligned={C_lihdr(0,0,34),67,95,117,95,105,110,116,101,103,101,114,95,113,117,111,116,105,101,110,116,95,97,110,100,95,114,101,109,97,105,110,100,101,114,0,0,0,0,0,0};
static C_char C_TLS li441[] C_aligned={C_lihdr(0,0,8),40,97,49,53,51,57,49,41};
static C_char C_TLS li442[] C_aligned={C_lihdr(0,0,22),40,97,49,53,51,57,55,32,104,105,50,55,55,51,32,108,111,50,55,55,53,41,0,0};
static C_char C_TLS li443[] C_aligned={C_lihdr(0,0,61),40,35,35,115,121,115,35,105,110,116,101,103,101,114,45,62,115,116,114,105,110,103,47,114,101,99,117,114,115,105,118,101,32,110,32,98,97,115,101,32,101,120,112,101,99,116,101,100,45,115,116,114,105,110,103,45,115,105,122,101,41,0,0,0};
static C_char C_TLS li444[] C_aligned={C_lihdr(0,0,38),40,35,35,115,121,115,35,101,120,116,101,110,100,101,100,45,110,117,109,98,101,114,45,62,115,116,114,105,110,103,32,110,32,98,97,115,101,41,0,0};
static C_char C_TLS li445[] C_aligned={C_lihdr(0,0,20),40,114,111,117,110,100,45,113,117,111,116,105,101,110,116,32,110,32,100,41,0,0,0,0};
static C_char C_TLS li446[] C_aligned={C_lihdr(0,0,13),40,103,111,45,105,110,101,120,97,99,116,33,41,0,0,0};
static C_char C_TLS li447[] C_aligned={C_lihdr(0,0,23),40,115,97,102,101,45,101,120,112,111,110,101,110,116,32,118,97,108,117,101,32,101,41,0};
static C_char C_TLS li448[] C_aligned={C_lihdr(0,0,6),40,108,112,32,105,41,0,0};
static C_char C_TLS li449[] C_aligned={C_lihdr(0,0,19),40,115,99,97,110,45,100,105,103,105,116,115,32,115,116,97,114,116,41,0,0,0,0,0};
static C_char C_TLS li450[] C_aligned={C_lihdr(0,0,6),40,108,112,32,105,41,0,0};
static C_char C_TLS li451[] C_aligned={C_lihdr(0,0,19),40,115,99,97,110,45,104,97,115,104,101,115,32,115,116,97,114,116,41,0,0,0,0,0};
static C_char C_TLS li452[] C_aligned={C_lihdr(0,0,46),40,115,99,97,110,45,100,105,103,105,116,115,43,104,97,115,104,101,115,32,115,116,97,114,116,32,110,101,103,63,32,97,108,108,45,104,97,115,104,101,115,45,111,107,63,41,0,0};
static C_char C_TLS li453[] C_aligned={C_lihdr(0,0,21),40,115,99,97,110,45,101,120,112,111,110,101,110,116,32,115,116,97,114,116,41,0,0,0};
static C_char C_TLS li454[] C_aligned={C_lihdr(0,0,43),40,115,99,97,110,45,100,101,99,105,109,97,108,45,116,97,105,108,32,115,116,97,114,116,32,110,101,103,63,32,100,101,99,105,109,97,108,45,104,101,97,100,41,0,0,0,0,0};
static C_char C_TLS li455[] C_aligned={C_lihdr(0,0,23),40,115,99,97,110,45,117,114,101,97,108,32,115,116,97,114,116,32,110,101,103,63,41,0};
static C_char C_TLS li456[] C_aligned={C_lihdr(0,0,17),40,115,99,97,110,45,114,101,97,108,32,115,116,97,114,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li457[] C_aligned={C_lihdr(0,0,50),40,35,35,115,121,115,35,115,116,114,105,110,103,45,62,99,111,109,112,110,117,109,32,114,97,100,105,120,32,115,116,114,32,111,102,102,115,101,116,32,101,120,97,99,116,110,101,115,115,41,0,0,0,0,0,0};
static C_char C_TLS li458[] C_aligned={C_lihdr(0,0,32),40,115,99,97,110,45,112,114,101,102,105,120,32,105,32,101,120,110,101,115,115,32,114,97,100,105,120,32,108,101,110,41};
static C_char C_TLS li459[] C_aligned={C_lihdr(0,0,34),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,62,110,117,109,98,101,114,32,115,116,114,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li460[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,115,116,114,105,110,103,45,62,110,117,109,98,101,114,32,115,116,114,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li461[] C_aligned={C_lihdr(0,0,18),67,95,102,105,120,110,117,109,95,116,111,95,115,116,114,105,110,103,0,0,0,0,0,0};
static C_char C_TLS li462[] C_aligned={C_lihdr(0,0,18),67,95,102,108,111,110,117,109,95,116,111,95,115,116,114,105,110,103,0,0,0,0,0,0};
static C_char C_TLS li463[] C_aligned={C_lihdr(0,0,19),67,95,105,110,116,101,103,101,114,95,116,111,95,115,116,114,105,110,103,0,0,0,0,0};
static C_char C_TLS li464[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li465[] C_aligned={C_lihdr(0,0,25),40,99,111,109,112,97,114,101,45,115,108,111,116,115,32,120,32,121,32,115,116,97,114,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li466[] C_aligned={C_lihdr(0,0,10),40,119,97,108,107,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li467[] C_aligned={C_lihdr(0,0,26),40,99,104,105,99,107,101,110,46,98,97,115,101,35,101,113,117,97,108,61,63,32,120,32,121,41,0,0,0,0,0,0};
static C_char C_TLS li468[] C_aligned={C_lihdr(0,0,18),67,95,115,116,114,105,110,103,95,116,111,95,115,121,109,98,111,108,0,0,0,0,0,0};
static C_char C_TLS li469[] C_aligned={C_lihdr(0,0,19),67,95,115,116,114,105,110,103,95,116,111,95,107,101,121,119,111,114,100,0,0,0,0,0};
static C_char C_TLS li470[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,105,110,116,101,114,110,101,100,45,115,121,109,98,111,108,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li471[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,115,116,114,105,110,103,45,62,115,121,109,98,111,108,32,115,116,114,41,0,0,0,0,0,0};
static C_char C_TLS li472[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,115,121,109,98,111,108,45,62,115,116,114,105,110,103,32,115,41};
static C_char C_TLS li473[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,115,121,109,98,111,108,45,62,115,116,114,105,110,103,32,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li474[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,115,116,114,105,110,103,45,62,115,121,109,98,111,108,32,115,116,114,41,0,0,0,0,0};
static C_char C_TLS li475[] C_aligned={C_lihdr(0,0,13),67,95,109,97,107,101,95,115,121,109,98,111,108,0,0,0};
static C_char C_TLS li476[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,116,114,105,110,103,45,62,117,110,105,110,116,101,114,110,101,100,45,115,121,109,98,111,108,32,115,116,114,41,0,0,0,0};
static C_char C_TLS li477[] C_aligned={C_lihdr(0,0,13),67,95,109,97,107,101,95,115,121,109,98,111,108,0,0,0};
static C_char C_TLS li478[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,97,115,101,35,103,101,110,115,121,109,32,46,32,115,116,114,45,111,114,45,115,121,109,41,0,0,0,0,0,0};
static C_char C_TLS li479[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,51,51,56,53,32,103,51,51,57,55,41,0,0,0,0};
static C_char C_TLS li480[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,121,109,98,111,108,45,97,112,112,101,110,100,32,46,32,115,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li481[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,107,101,121,119,111,114,100,35,107,101,121,119,111,114,100,63,32,120,41,0,0,0,0};
static C_char C_TLS li482[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,107,101,121,119,111,114,100,35,115,116,114,105,110,103,45,62,107,101,121,119,111,114,100,32,115,41,0,0,0,0,0};
static C_char C_TLS li483[] C_aligned={C_lihdr(0,0,36),40,99,104,105,99,107,101,110,46,107,101,121,119,111,114,100,35,107,101,121,119,111,114,100,45,62,115,116,114,105,110,103,32,107,119,41,0,0,0,0};
static C_char C_TLS li484[] C_aligned={C_lihdr(0,0,45),40,99,104,105,99,107,101,110,46,107,101,121,119,111,114,100,35,103,101,116,45,107,101,121,119,111,114,100,32,107,101,121,32,97,114,103,115,32,46,32,114,101,115,116,41,0,0,0};
static C_char C_TLS li485[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,109,97,107,101,45,98,108,111,98,32,115,105,122,101,41,0,0};
static C_char C_TLS li486[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,98,108,111,98,35,109,97,107,101,45,98,108,111,98,32,115,105,122,101,41,0,0,0};
static C_char C_TLS li487[] C_aligned={C_lihdr(0,0,22),40,99,104,105,99,107,101,110,46,98,108,111,98,35,98,108,111,98,63,32,120,41,0,0};
static C_char C_TLS li488[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,108,111,98,35,98,108,111,98,45,115,105,122,101,32,98,118,41,0,0,0,0,0};
static C_char C_TLS li489[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,98,108,111,98,35,115,116,114,105,110,103,45,62,98,108,111,98,32,115,41,0,0,0};
static C_char C_TLS li490[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,108,111,98,35,98,108,111,98,45,62,115,116,114,105,110,103,32,98,118,41,0,0};
static C_char C_TLS li491[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,108,111,98,35,98,108,111,98,61,63,32,98,49,32,98,50,41,0,0,0,0,0};
static C_char C_TLS li492[] C_aligned={C_lihdr(0,0,32),40,115,99,104,101,109,101,35,109,97,107,101,45,118,101,99,116,111,114,32,115,105,122,101,32,46,32,102,105,108,108,41};
static C_char C_TLS li493[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li494[] C_aligned={C_lihdr(0,0,26),40,115,99,104,101,109,101,35,108,105,115,116,45,62,118,101,99,116,111,114,32,108,115,116,48,41,0,0,0,0,0,0};
static C_char C_TLS li495[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li496[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,118,101,99,116,111,114,45,62,108,105,115,116,32,118,41,0};
static C_char C_TLS li497[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,118,101,99,116,111,114,32,46,32,120,115,41,0,0,0,0};
static C_char C_TLS li498[] C_aligned={C_lihdr(0,0,12),40,100,111,108,111,111,112,51,53,50,51,41,0,0,0,0};
static C_char C_TLS li499[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,118,101,99,116,111,114,45,102,105,108,108,33,32,118,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li500[] C_aligned={C_lihdr(0,0,12),40,100,111,108,111,111,112,51,53,51,53,41,0,0,0,0};
static C_char C_TLS li501[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,98,97,115,101,35,118,101,99,116,111,114,45,99,111,112,121,33,32,102,114,111,109,32,116,111,32,46,32,110,41,0};
static C_char C_TLS li502[] C_aligned={C_lihdr(0,0,12),40,100,111,108,111,111,112,51,53,54,54,41,0,0,0,0};
static C_char C_TLS li503[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,117,98,118,101,99,116,111,114,32,118,32,105,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li504[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,98,97,115,101,35,118,101,99,116,111,114,45,114,101,115,105,122,101,32,118,32,110,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li505[] C_aligned={C_lihdr(0,0,12),40,100,111,108,111,111,112,51,53,57,52,41,0,0,0,0};
static C_char C_TLS li506[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,118,101,99,116,111,114,45,114,101,115,105,122,101,32,118,32,110,32,105,110,105,116,41,0,0};
static C_char C_TLS li507[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,45,99,105,61,63,32,120,32,121,41,0,0};
static C_char C_TLS li508[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,45,99,105,62,63,32,120,32,121,41,0,0};
static C_char C_TLS li509[] C_aligned={C_lihdr(0,0,22),40,115,99,104,101,109,101,35,99,104,97,114,45,99,105,60,63,32,120,32,121,41,0,0};
static C_char C_TLS li510[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,99,104,97,114,45,99,105,62,61,63,32,120,32,121,41,0};
static C_char C_TLS li511[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,99,104,97,114,45,99,105,60,61,63,32,120,32,121,41,0};
static C_char C_TLS li512[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li513[] C_aligned={C_lihdr(0,0,15),40,108,111,111,107,117,112,45,99,104,97,114,32,99,41,0};
static C_char C_TLS li514[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,104,97,114,45,110,97,109,101,32,120,32,46,32,121,41,0,0};
static C_char C_TLS li515[] C_aligned={C_lihdr(0,0,9),67,95,99,97,108,108,95,99,99,0,0,0,0,0,0,0};
static C_char C_TLS li516[] C_aligned={C_lihdr(0,0,19),67,95,99,97,108,108,95,119,105,116,104,95,99,116,104,117,108,104,117,0,0,0,0,0};
static C_char C_TLS li517[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li518[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,102,111,114,45,101,97,99,104,32,112,32,108,115,116,48,41,0};
static C_char C_TLS li519[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li520[] C_aligned={C_lihdr(0,0,18),40,35,35,115,121,115,35,109,97,112,32,112,32,108,115,116,48,41,0,0,0,0,0,0};
static C_char C_TLS li521[] C_aligned={C_lihdr(0,0,9),40,108,112,32,108,115,116,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li522[] C_aligned={C_lihdr(0,0,14),40,97,49,56,53,52,56,32,101,109,112,116,121,41,0,0};
static C_char C_TLS li523[] C_aligned={C_lihdr(0,0,24),40,109,97,112,115,97,102,101,51,54,55,48,32,112,32,108,115,116,115,32,108,111,99,41};
static C_char C_TLS li524[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,51,54,57,48,32,103,51,54,57,55,41,0,0,0,0,0,0,0};
static C_char C_TLS li525[] C_aligned={C_lihdr(0,0,10),40,97,49,56,54,55,48,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li526[] C_aligned={C_lihdr(0,0,10),40,97,49,56,54,55,54,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li527[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,97,108,108,41,0,0,0,0,0,0};
static C_char C_TLS li528[] C_aligned={C_lihdr(0,0,32),40,115,99,104,101,109,101,35,102,111,114,45,101,97,99,104,32,102,110,32,108,115,116,49,32,46,32,108,115,116,115,41};
static C_char C_TLS li529[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,51,55,49,56,32,103,51,55,51,48,41,0,0,0,0};
static C_char C_TLS li530[] C_aligned={C_lihdr(0,0,10),40,97,49,56,55,55,49,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li531[] C_aligned={C_lihdr(0,0,10),40,97,49,56,55,55,55,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li532[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,97,108,108,41,0,0,0,0,0,0};
static C_char C_TLS li533[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,109,97,112,32,102,110,32,108,115,116,49,32,46,32,108,115,116,115,41,0,0,0,0,0};
static C_char C_TLS li534[] C_aligned={C_lihdr(0,0,18),40,97,49,56,55,57,55,32,46,32,114,101,115,117,108,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li535[] C_aligned={C_lihdr(0,0,40),40,115,99,104,101,109,101,35,100,121,110,97,109,105,99,45,119,105,110,100,32,98,101,102,111,114,101,32,116,104,117,110,107,32,97,102,116,101,114,41};
static C_char C_TLS li536[] C_aligned={C_lihdr(0,0,24),40,99,111,110,116,105,110,117,97,116,105,111,110,32,46,32,114,101,115,117,108,116,115,41};
static C_char C_TLS li537[] C_aligned={C_lihdr(0,0,13),40,97,49,56,56,50,49,32,99,111,110,116,41,0,0,0};
static C_char C_TLS li538[] C_aligned={C_lihdr(0,0,44),40,115,99,104,101,109,101,35,99,97,108,108,45,119,105,116,104,45,99,117,114,114,101,110,116,45,99,111,110,116,105,110,117,97,116,105,111,110,32,112,114,111,99,41,0,0,0,0};
static C_char C_TLS li539[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,100,121,110,97,109,105,99,45,117,110,119,105,110,100,32,119,105,110,100,115,32,110,41,0,0};
static C_char C_TLS li540[] C_aligned={C_lihdr(0,0,22),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,111,114,116,63,32,120,41,0,0};
static C_char C_TLS li541[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,98,97,115,101,35,105,110,112,117,116,45,112,111,114,116,45,111,112,101,110,63,32,112,41,0,0,0,0,0,0,0};
static C_char C_TLS li542[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,97,115,101,35,111,117,116,112,117,116,45,112,111,114,116,45,111,112,101,110,63,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li543[] C_aligned={C_lihdr(0,0,29),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,111,114,116,45,99,108,111,115,101,100,63,32,112,41,0,0,0};
static C_char C_TLS li544[] C_aligned={C_lihdr(0,0,37),40,35,35,115,121,115,35,109,97,107,101,45,112,111,114,116,32,105,47,111,32,99,108,97,115,115,32,110,97,109,101,32,116,121,112,101,41,0,0,0};
static C_char C_TLS li545[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li546[] C_aligned={C_lihdr(0,0,10),40,97,51,50,49,53,57,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li547[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li548[] C_aligned={C_lihdr(0,0,10),40,97,51,50,49,57,57,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li549[] C_aligned={C_lihdr(0,0,12),40,97,51,50,50,51,57,32,112,32,99,41,0,0,0,0};
static C_char C_TLS li550[] C_aligned={C_lihdr(0,0,12),40,97,51,50,50,52,50,32,112,32,115,41,0,0,0,0};
static C_char C_TLS li551[] C_aligned={C_lihdr(0,0,12),40,97,51,50,50,52,53,32,112,32,100,41,0,0,0,0};
static C_char C_TLS li552[] C_aligned={C_lihdr(0,0,10),40,97,51,50,50,53,49,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li553[] C_aligned={C_lihdr(0,0,10),40,97,51,50,50,53,52,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li554[] C_aligned={C_lihdr(0,0,8),40,97,51,50,50,57,49,41};
static C_char C_TLS li555[] C_aligned={C_lihdr(0,0,20),40,108,111,111,112,32,114,101,109,32,97,99,116,32,115,116,97,114,116,41,0,0,0,0};
static C_char C_TLS li556[] C_aligned={C_lihdr(0,0,23),40,97,51,50,50,53,55,32,112,32,110,32,100,101,115,116,32,115,116,97,114,116,41,0};
static C_char C_TLS li557[] C_aligned={C_lihdr(0,0,8),40,97,51,50,52,53,50,41};
static C_char C_TLS li558[] C_aligned={C_lihdr(0,0,32),40,108,111,111,112,32,108,101,110,32,108,105,109,105,116,32,98,117,102,102,101,114,32,114,101,115,117,108,116,32,102,41};
static C_char C_TLS li559[] C_aligned={C_lihdr(0,0,17),40,97,51,50,51,53,54,32,112,32,114,108,105,109,105,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li560[] C_aligned={C_lihdr(0,0,16),67,95,111,112,101,110,95,102,105,108,101,95,112,111,114,116};
static C_char C_TLS li561[] C_aligned={C_lihdr(0,0,37),40,35,35,115,121,115,35,99,104,101,99,107,45,105,110,112,117,116,45,112,111,114,116,32,120,32,111,112,101,110,32,46,32,108,111,99,41,0,0,0};
static C_char C_TLS li562[] C_aligned={C_lihdr(0,0,38),40,35,35,115,121,115,35,99,104,101,99,107,45,111,117,116,112,117,116,45,112,111,114,116,32,120,32,111,112,101,110,32,46,32,108,111,99,41,0,0};
static C_char C_TLS li563[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,99,104,101,99,107,45,112,111,114,116,32,120,32,46,32,108,111,99,41,0,0,0,0,0,0};
static C_char C_TLS li564[] C_aligned={C_lihdr(0,0,31),40,35,35,115,121,115,35,99,104,101,99,107,45,111,112,101,110,45,112,111,114,116,32,120,32,46,32,108,111,99,41,0};
static C_char C_TLS li565[] C_aligned={C_lihdr(0,0,10),40,98,111,100,121,51,56,57,56,41,0,0,0,0,0,0};
static C_char C_TLS li566[] C_aligned={C_lihdr(0,0,34),40,115,99,104,101,109,101,35,99,117,114,114,101,110,116,45,105,110,112,117,116,45,112,111,114,116,32,46,32,97,114,103,115,41,0,0,0,0,0,0};
static C_char C_TLS li567[] C_aligned={C_lihdr(0,0,10),40,98,111,100,121,51,57,50,51,41,0,0,0,0,0,0};
static C_char C_TLS li568[] C_aligned={C_lihdr(0,0,35),40,115,99,104,101,109,101,35,99,117,114,114,101,110,116,45,111,117,116,112,117,116,45,112,111,114,116,32,46,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li569[] C_aligned={C_lihdr(0,0,10),40,98,111,100,121,51,57,52,56,41,0,0,0,0,0,0};
static C_char C_TLS li570[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,98,97,115,101,35,99,117,114,114,101,110,116,45,101,114,114,111,114,45,112,111,114,116,32,46,32,97,114,103,115,41};
static C_char C_TLS li571[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,116,116,121,45,112,111,114,116,63,32,112,111,114,116,41,0,0};
static C_char C_TLS li572[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,112,111,114,116,45,100,97,116,97,32,112,111,114,116,41,0,0};
static C_char C_TLS li573[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,115,101,116,45,112,111,114,116,45,100,97,116,97,33,32,112,111,114,116,32,100,97,116,97,41};
static C_char C_TLS li574[] C_aligned={C_lihdr(0,0,18),40,100,111,108,111,111,112,51,57,56,53,32,109,111,100,101,115,41,0,0,0,0,0,0};
static C_char C_TLS li575[] C_aligned={C_lihdr(0,0,25),40,111,112,101,110,32,110,97,109,101,32,105,110,112,32,109,111,100,101,115,32,108,111,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li576[] C_aligned={C_lihdr(0,0,20),40,99,108,111,115,101,32,112,111,114,116,32,105,110,112,32,108,111,99,41,0,0,0,0};
static C_char C_TLS li577[] C_aligned={C_lihdr(0,0,36),40,115,99,104,101,109,101,35,111,112,101,110,45,105,110,112,117,116,45,102,105,108,101,32,110,97,109,101,32,46,32,109,111,100,101,41,0,0,0,0};
static C_char C_TLS li578[] C_aligned={C_lihdr(0,0,37),40,115,99,104,101,109,101,35,111,112,101,110,45,111,117,116,112,117,116,45,102,105,108,101,32,110,97,109,101,32,46,32,109,111,100,101,41,0,0,0};
static C_char C_TLS li579[] C_aligned={C_lihdr(0,0,30),40,115,99,104,101,109,101,35,99,108,111,115,101,45,105,110,112,117,116,45,112,111,114,116,32,112,111,114,116,41,0,0};
static C_char C_TLS li580[] C_aligned={C_lihdr(0,0,31),40,115,99,104,101,109,101,35,99,108,111,115,101,45,111,117,116,112,117,116,45,112,111,114,116,32,112,111,114,116,41,0};
static C_char C_TLS li581[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,49,51,41};
static C_char C_TLS li582[] C_aligned={C_lihdr(0,0,18),40,97,49,57,52,49,57,32,46,32,114,101,115,117,108,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li583[] C_aligned={C_lihdr(0,0,43),40,115,99,104,101,109,101,35,99,97,108,108,45,119,105,116,104,45,105,110,112,117,116,45,102,105,108,101,32,110,97,109,101,32,112,32,46,32,109,111,100,101,41,0,0,0,0,0};
static C_char C_TLS li584[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,51,55,41};
static C_char C_TLS li585[] C_aligned={C_lihdr(0,0,18),40,97,49,57,52,52,51,32,46,32,114,101,115,117,108,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li586[] C_aligned={C_lihdr(0,0,44),40,115,99,104,101,109,101,35,99,97,108,108,45,119,105,116,104,45,111,117,116,112,117,116,45,102,105,108,101,32,110,97,109,101,32,112,32,46,32,109,111,100,101,41,0,0,0,0};
static C_char C_TLS li587[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,54,49,41};
static C_char C_TLS li588[] C_aligned={C_lihdr(0,0,18),40,97,49,57,52,55,50,32,46,32,114,101,115,117,108,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li589[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,54,54,41};
static C_char C_TLS li590[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,56,49,41};
static C_char C_TLS li591[] C_aligned={C_lihdr(0,0,46),40,115,99,104,101,109,101,35,119,105,116,104,45,105,110,112,117,116,45,102,114,111,109,45,102,105,108,101,32,115,116,114,32,116,104,117,110,107,32,46,32,109,111,100,101,41,0,0};
static C_char C_TLS li592[] C_aligned={C_lihdr(0,0,8),40,97,49,57,52,57,53,41};
static C_char C_TLS li593[] C_aligned={C_lihdr(0,0,18),40,97,49,57,53,48,54,32,46,32,114,101,115,117,108,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li594[] C_aligned={C_lihdr(0,0,8),40,97,49,57,53,48,48,41};
static C_char C_TLS li595[] C_aligned={C_lihdr(0,0,8),40,97,49,57,53,49,53,41};
static C_char C_TLS li596[] C_aligned={C_lihdr(0,0,45),40,115,99,104,101,109,101,35,119,105,116,104,45,111,117,116,112,117,116,45,116,111,45,102,105,108,101,32,115,116,114,32,116,104,117,110,107,32,46,32,109,111,100,101,41,0,0,0};
static C_char C_TLS li597[] C_aligned={C_lihdr(0,0,40),40,35,35,115,121,115,35,102,105,108,101,45,101,120,105,115,116,115,63,32,110,97,109,101,32,102,105,108,101,63,32,100,105,114,63,32,108,111,99,41};
static C_char C_TLS li598[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,102,108,117,115,104,45,111,117,116,112,117,116,32,112,111,114,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li599[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,97,115,101,35,102,108,117,115,104,45,111,117,116,112,117,116,32,46,32,114,101,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li600[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,112,111,114,116,45,108,105,110,101,32,112,111,114,116,41,0,0};
static C_char C_TLS li601[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,52,49,49,53,32,105,41,0,0};
static C_char C_TLS li602[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li603[] C_aligned={C_lihdr(0,0,43),40,35,35,115,121,115,35,100,101,99,111,114,97,116,101,45,108,97,109,98,100,97,32,112,114,111,99,32,112,114,101,100,32,100,101,99,111,114,97,116,111,114,41,0,0,0,0,0};
static C_char C_TLS li604[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li605[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,108,97,109,98,100,97,45,100,101,99,111,114,97,116,105,111,110,32,112,114,111,99,32,112,114,101,100,41,0,0,0,0,0};
static C_char C_TLS li606[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,109,97,107,101,45,108,97,109,98,100,97,45,105,110,102,111,32,115,116,114,41,0,0,0,0};
static C_char C_TLS li607[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,108,97,109,98,100,97,45,105,110,102,111,63,32,120,41,0,0};
static C_char C_TLS li608[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,108,97,109,98,100,97,45,105,110,102,111,32,112,114,111,99,41};
static C_char C_TLS li609[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,108,97,109,98,100,97,45,105,110,102,111,45,62,115,116,114,105,110,103,32,105,110,102,111,41};
static C_char C_TLS li610[] C_aligned={C_lihdr(0,0,38),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,114,111,99,101,100,117,114,101,45,105,110,102,111,114,109,97,116,105,111,110,32,120,41,0,0};
static C_char C_TLS li611[] C_aligned={C_lihdr(0,0,12),40,97,49,57,55,57,51,32,112,32,105,41,0,0,0,0};
static C_char C_TLS li612[] C_aligned={C_lihdr(0,0,10),40,97,49,57,56,48,51,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li613[] C_aligned={C_lihdr(0,0,15),40,97,49,57,56,49,57,32,112,114,111,99,32,105,41,0};
static C_char C_TLS li614[] C_aligned={C_lihdr(0,0,48),40,99,104,105,99,107,101,110,46,98,97,115,101,35,103,101,116,116,101,114,45,119,105,116,104,45,115,101,116,116,101,114,32,103,101,116,32,115,101,116,32,46,32,114,101,115,116,41};
static C_char C_TLS li615[] C_aligned={C_lihdr(0,0,11),40,102,95,50,48,48,55,54,32,120,41,0,0,0,0,0};
static C_char C_TLS li616[] C_aligned={C_lihdr(0,0,28),40,97,115,115,105,103,110,32,118,97,108,32,110,32,99,111,110,118,101,114,116,63,32,115,101,116,63,41,0,0,0,0};
static C_char C_TLS li617[] C_aligned={C_lihdr(0,0,24),40,98,111,100,121,52,50,54,54,32,99,111,110,118,101,114,116,63,32,115,101,116,63,41};
static C_char C_TLS li618[] C_aligned={C_lihdr(0,0,24),40,100,101,102,45,115,101,116,63,52,50,54,57,32,37,99,111,110,118,101,114,116,63,41};
static C_char C_TLS li619[] C_aligned={C_lihdr(0,0,18),40,100,101,102,45,99,111,110,118,101,114,116,63,52,50,54,56,41,0,0,0,0,0,0};
static C_char C_TLS li620[] C_aligned={C_lihdr(0,0,15),40,97,49,57,57,54,55,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li621[] C_aligned={C_lihdr(0,0,12),40,97,50,48,48,52,57,32,118,97,108,41,0,0,0,0};
static C_char C_TLS li622[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,98,97,115,101,35,109,97,107,101,45,112,97,114,97,109,101,116,101,114,32,105,110,105,116,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li623[] C_aligned={C_lihdr(0,0,27),40,115,99,104,101,109,101,35,99,104,97,114,45,114,101,97,100,121,63,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li624[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,114,101,97,100,45,99,104,97,114,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li625[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,114,101,97,100,45,99,104,97,114,45,48,32,112,41,0,0,0};
static C_char C_TLS li626[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,114,101,97,100,45,99,104,97,114,47,112,111,114,116,32,112,111,114,116,41,0,0,0,0,0};
static C_char C_TLS li627[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,112,101,101,107,45,99,104,97,114,45,48,32,112,41,0,0,0};
static C_char C_TLS li628[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,112,101,101,107,45,99,104,97,114,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li629[] C_aligned={C_lihdr(0,0,20),40,115,99,104,101,109,101,35,114,101,97,100,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li630[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,114,101,97,100,45,112,114,111,109,112,116,45,104,111,111,107,41};
static C_char C_TLS li631[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,105,110,102,105,120,45,108,105,115,116,45,104,111,111,107,32,108,115,116,41,0,0,0,0,0};
static C_char C_TLS li632[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,115,104,97,114,112,45,110,117,109,98,101,114,45,104,111,111,107,32,112,111,114,116,32,110,41};
static C_char C_TLS li633[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,114,101,97,100,45,119,97,114,110,105,110,103,32,112,111,114,116,32,109,115,103,32,46,32,97,114,103,115,41,0,0,0,0};
static C_char C_TLS li634[] C_aligned={C_lihdr(0,0,34),40,35,35,115,121,115,35,114,101,97,100,45,101,114,114,111,114,32,112,111,114,116,32,109,115,103,32,46,32,97,114,103,115,41,0,0,0,0,0,0};
static C_char C_TLS li635[] C_aligned={C_lihdr(0,0,11),40,102,95,50,48,51,53,51,32,99,41,0,0,0,0,0};
static C_char C_TLS li636[] C_aligned={C_lihdr(0,0,21),40,105,110,102,111,32,99,108,97,115,115,32,100,97,116,97,32,118,97,108,41,0,0,0};
static C_char C_TLS li637[] C_aligned={C_lihdr(0,0,8),40,115,107,105,112,32,99,41};
static C_char C_TLS li638[] C_aligned={C_lihdr(0,0,13),40,115,107,105,112,45,116,111,45,101,111,108,41,0,0,0};
static C_char C_TLS li639[] C_aligned={C_lihdr(0,0,22),40,114,101,115,101,114,118,101,100,45,99,104,97,114,97,99,116,101,114,32,99,41,0,0};
static C_char C_TLS li640[] C_aligned={C_lihdr(0,0,29),40,114,101,97,100,45,117,110,114,101,115,101,114,118,101,100,45,99,104,97,114,45,48,32,112,111,114,116,41,0,0,0};
static C_char C_TLS li641[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,99,41};
static C_char C_TLS li642[] C_aligned={C_lihdr(0,0,10),40,114,45,115,112,97,99,101,115,41,0,0,0,0,0,0};
static C_char C_TLS li643[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,115,101,113,32,110,41,0,0,0,0};
static C_char C_TLS li644[] C_aligned={C_lihdr(0,0,22),40,114,45,117,115,101,113,117,101,110,99,101,32,117,32,110,32,98,97,115,101,41,0,0};
static C_char C_TLS li645[] C_aligned={C_lihdr(0,0,10),40,108,112,32,105,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li646[] C_aligned={C_lihdr(0,0,25),40,114,45,99,111,110,115,45,99,111,100,101,112,111,105,110,116,32,99,112,32,108,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li647[] C_aligned={C_lihdr(0,0,14),40,101,97,116,45,119,115,32,99,32,110,108,63,41,0,0};
static C_char C_TLS li648[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,99,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li649[] C_aligned={C_lihdr(0,0,15),40,114,45,115,116,114,105,110,103,32,116,101,114,109,41,0};
static C_char C_TLS li650[] C_aligned={C_lihdr(0,0,19),40,115,116,97,114,116,105,110,103,45,108,105,110,101,32,109,115,103,41,0,0,0,0,0};
static C_char C_TLS li651[] C_aligned={C_lihdr(0,0,11),40,102,95,50,49,49,55,51,32,99,41,0,0,0,0,0};
static C_char C_TLS li652[] C_aligned={C_lihdr(0,0,15),40,97,50,49,50,56,56,32,116,111,107,32,107,119,41,0};
static C_char C_TLS li653[] C_aligned={C_lihdr(0,0,11),40,108,111,111,112,32,108,97,115,116,41,0,0,0,0,0};
static C_char C_TLS li654[] C_aligned={C_lihdr(0,0,15),40,97,50,49,49,54,57,32,114,101,116,117,114,110,41,0};
static C_char C_TLS li655[] C_aligned={C_lihdr(0,0,18),40,114,45,108,105,115,116,32,115,116,97,114,116,32,101,110,100,41,0,0,0,0,0,0};
static C_char C_TLS li656[] C_aligned={C_lihdr(0,0,10),40,114,45,118,101,99,116,111,114,41,0,0,0,0,0,0};
static C_char C_TLS li657[] C_aligned={C_lihdr(0,0,15),40,97,50,49,52,49,54,32,116,111,107,32,107,119,41,0};
static C_char C_TLS li658[] C_aligned={C_lihdr(0,0,26),40,114,45,110,117,109,98,101,114,32,114,97,100,105,120,32,101,120,97,99,116,110,101,115,115,41,0,0,0,0,0,0};
static C_char C_TLS li659[] C_aligned={C_lihdr(0,0,31),40,114,45,110,117,109,98,101,114,45,119,105,116,104,45,101,120,97,99,116,110,101,115,115,32,114,97,100,105,120,41,0};
static C_char C_TLS li660[] C_aligned={C_lihdr(0,0,31),40,114,45,110,117,109,98,101,114,45,119,105,116,104,45,114,97,100,105,120,32,101,120,97,99,116,110,101,115,115,41,0};
static C_char C_TLS li661[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,99,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li662[] C_aligned={C_lihdr(0,0,9),40,114,45,116,111,107,101,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li663[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,99,32,108,115,116,41,0,0,0,0};
static C_char C_TLS li664[] C_aligned={C_lihdr(0,0,10),40,114,45,100,105,103,105,116,115,41,0,0,0,0,0,0};
static C_char C_TLS li665[] C_aligned={C_lihdr(0,0,15),40,97,50,49,55,49,52,32,115,116,114,32,107,119,41,0};
static C_char C_TLS li666[] C_aligned={C_lihdr(0,0,10),40,114,45,115,121,109,98,111,108,41,0,0,0,0,0,0};
static C_char C_TLS li667[] C_aligned={C_lihdr(0,0,18),40,108,111,111,112,32,108,115,116,32,115,107,119,32,113,116,100,41,0,0,0,0,0,0};
static C_char C_TLS li668[] C_aligned={C_lihdr(0,0,12),40,114,45,120,116,111,107,101,110,32,107,41,0,0,0,0};
static C_char C_TLS li669[] C_aligned={C_lihdr(0,0,8),40,114,45,99,104,97,114,41};
static C_char C_TLS li670[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li671[] C_aligned={C_lihdr(0,0,11),40,114,45,99,111,109,109,101,110,116,41,0,0,0,0,0};
static C_char C_TLS li672[] C_aligned={C_lihdr(0,0,14),40,114,45,101,120,116,45,115,121,109,98,111,108,41,0,0};
static C_char C_TLS li673[] C_aligned={C_lihdr(0,0,11),40,114,45,113,117,111,116,101,32,113,41,0,0,0,0,0};
static C_char C_TLS li674[] C_aligned={C_lihdr(0,0,8),40,97,50,50,51,52,52,41};
static C_char C_TLS li675[] C_aligned={C_lihdr(0,0,13),40,97,50,50,51,53,48,32,46,32,120,115,41,0,0,0};
static C_char C_TLS li676[] C_aligned={C_lihdr(0,0,8),40,97,50,50,52,55,49,41};
static C_char C_TLS li677[] C_aligned={C_lihdr(0,0,13),40,97,50,50,52,55,55,32,46,32,120,115,41,0,0,0};
static C_char C_TLS li678[] C_aligned={C_lihdr(0,0,8),40,97,50,50,53,52,50,41};
static C_char C_TLS li679[] C_aligned={C_lihdr(0,0,13),40,97,50,50,53,52,56,32,46,32,120,115,41,0,0,0};
static C_char C_TLS li680[] C_aligned={C_lihdr(0,0,8),40,97,50,50,54,53,54,41};
static C_char C_TLS li681[] C_aligned={C_lihdr(0,0,8),40,97,50,50,54,54,49,41};
static C_char C_TLS li682[] C_aligned={C_lihdr(0,0,8),40,97,50,50,54,54,55,41};
static C_char C_TLS li683[] C_aligned={C_lihdr(0,0,8),40,97,50,50,56,49,48,41};
static C_char C_TLS li684[] C_aligned={C_lihdr(0,0,15),40,97,50,50,56,50,49,32,115,116,114,32,107,119,41,0};
static C_char C_TLS li685[] C_aligned={C_lihdr(0,0,8),40,97,50,50,56,49,53,41};
static C_char C_TLS li686[] C_aligned={C_lihdr(0,0,8),40,97,50,50,56,53,48,41};
static C_char C_TLS li687[] C_aligned={C_lihdr(0,0,8),40,97,50,50,57,54,52,41};
static C_char C_TLS li688[] C_aligned={C_lihdr(0,0,13),40,97,50,50,57,55,48,32,46,32,120,115,41,0,0,0};
static C_char C_TLS li689[] C_aligned={C_lihdr(0,0,9),40,114,101,97,100,114,101,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li690[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,114,101,97,100,32,112,111,114,116,32,105,110,102,111,104,97,110,100,108,101,114,41,0,0,0};
static C_char C_TLS li691[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,99,104,97,114,45,62,117,116,102,56,45,115,116,114,105,110,103,32,99,41,0,0,0,0,0};
static C_char C_TLS li692[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,117,110,105,99,111,100,101,45,115,117,114,114,111,103,97,116,101,63,32,110,41,0,0,0,0};
static C_char C_TLS li693[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,115,117,114,114,111,103,97,116,101,115,45,62,99,111,100,101,112,111,105,110,116,32,104,105,32,108,111,41,0,0,0,0,0};
static C_char C_TLS li694[] C_aligned={C_lihdr(0,0,7),40,104,101,120,32,99,41,0};
static C_char C_TLS li695[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,108,115,116,32,104,41,0,0,0,0};
static C_char C_TLS li696[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,114,101,97,100,45,98,121,116,101,118,101,99,116,111,114,45,108,105,116,101,114,97,108,32,112,111,114,116,41,0,0,0,0};
static C_char C_TLS li697[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,117,115,101,114,45,114,101,97,100,45,104,111,111,107,32,99,104,97,114,32,112,111,114,116,41};
static C_char C_TLS li698[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,119,114,105,116,101,45,99,104,97,114,45,48,32,99,32,112,41};
static C_char C_TLS li699[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,119,114,105,116,101,45,99,104,97,114,47,112,111,114,116,32,99,32,112,111,114,116,41,0,0};
static C_char C_TLS li700[] C_aligned={C_lihdr(0,0,28),40,115,99,104,101,109,101,35,119,114,105,116,101,45,99,104,97,114,32,99,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li701[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,110,101,119,108,105,110,101,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li702[] C_aligned={C_lihdr(0,0,23),40,115,99,104,101,109,101,35,119,114,105,116,101,32,120,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li703[] C_aligned={C_lihdr(0,0,25),40,115,99,104,101,109,101,35,100,105,115,112,108,97,121,32,120,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li704[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,48,55,49,32,103,53,48,55,56,41,0,0,0,0,0,0,0};
static C_char C_TLS li705[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,114,105,110,116,32,46,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li706[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,49,48,51,32,103,53,49,49,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li707[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,114,105,110,116,42,32,46,32,97,114,103,115,41,0,0,0,0};
static C_char C_TLS li708[] C_aligned={C_lihdr(0,0,17),40,111,117,116,115,116,114,32,112,111,114,116,32,115,116,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li709[] C_aligned={C_lihdr(0,0,17),40,111,117,116,99,104,114,32,112,111,114,116,32,99,104,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li710[] C_aligned={C_lihdr(0,0,17),40,111,117,116,115,121,109,32,112,111,114,116,32,115,121,109,41,0,0,0,0,0,0,0};
static C_char C_TLS li711[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li712[] C_aligned={C_lihdr(0,0,25),40,111,117,116,114,101,97,100,97,98,108,101,115,121,109,32,112,111,114,116,32,115,116,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li713[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li714[] C_aligned={C_lihdr(0,0,22),40,115,121,109,45,105,115,45,114,101,97,100,97,98,108,101,63,32,115,116,114,41,0,0};
static C_char C_TLS li715[] C_aligned={C_lihdr(0,0,10),40,103,53,50,54,57,32,99,110,41,0,0,0,0,0,0};
static C_char C_TLS li716[] C_aligned={C_lihdr(0,0,16),40,100,111,108,111,111,112,53,50,57,48,32,105,32,99,41};
static C_char C_TLS li717[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,53,51,50,48,32,120,41,0,0};
static C_char C_TLS li718[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,53,51,51,52,32,105,41,0,0};
static C_char C_TLS li719[] C_aligned={C_lihdr(0,0,16),40,100,111,108,111,111,112,53,51,53,56,32,105,32,99,41};
static C_char C_TLS li720[] C_aligned={C_lihdr(0,0,7),40,111,117,116,32,120,41,0};
static C_char C_TLS li721[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,112,114,105,110,116,32,120,32,114,101,97,100,97,98,108,101,32,112,111,114,116,41,0,0,0};
static C_char C_TLS li722[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,112,114,111,99,101,100,117,114,101,45,62,115,116,114,105,110,103,32,120,41,0,0,0,0,0};
static C_char C_TLS li723[] C_aligned={C_lihdr(0,0,34),40,99,104,105,99,107,101,110,46,98,97,115,101,35,114,101,99,111,114,100,45,112,114,105,110,116,101,114,32,116,121,112,101,41,0,0,0,0,0,0};
static C_char C_TLS li724[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,101,116,45,114,101,99,111,114,100,45,112,114,105,110,116,101,114,33,32,116,121,112,101,32,112,114,111,99,41,0,0,0,0};
static C_char C_TLS li725[] C_aligned={C_lihdr(0,0,8),40,97,50,53,48,52,56,41};
static C_char C_TLS li726[] C_aligned={C_lihdr(0,0,11),40,97,50,53,48,52,50,32,101,120,41,0,0,0,0,0};
static C_char C_TLS li727[] C_aligned={C_lihdr(0,0,8),40,97,50,53,48,57,57,41};
static C_char C_TLS li728[] C_aligned={C_lihdr(0,0,8),40,97,50,53,49,49,52,41};
static C_char C_TLS li729[] C_aligned={C_lihdr(0,0,15),40,97,50,53,49,48,56,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li730[] C_aligned={C_lihdr(0,0,8),40,97,50,53,48,57,51,41};
static C_char C_TLS li731[] C_aligned={C_lihdr(0,0,10),40,97,50,53,48,51,54,32,107,41,0,0,0,0,0,0};
static C_char C_TLS li732[] C_aligned={C_lihdr(0,0,39),40,35,35,115,121,115,35,117,115,101,114,45,112,114,105,110,116,45,104,111,111,107,32,120,32,114,101,97,100,97,98,108,101,32,112,111,114,116,41,0};
static C_char C_TLS li733[] C_aligned={C_lihdr(0,0,8),40,97,50,53,49,56,49,41};
static C_char C_TLS li734[] C_aligned={C_lihdr(0,0,8),40,97,50,53,50,50,52,41};
static C_char C_TLS li735[] C_aligned={C_lihdr(0,0,8),40,97,50,53,50,51,48,41};
static C_char C_TLS li736[] C_aligned={C_lihdr(0,0,15),40,97,50,53,49,55,53,32,114,101,116,117,114,110,41,0};
static C_char C_TLS li737[] C_aligned={C_lihdr(0,0,43),40,35,35,115,121,115,35,119,105,116,104,45,112,114,105,110,116,45,108,101,110,103,116,104,45,108,105,109,105,116,32,108,105,109,105,116,32,116,104,117,110,107,41,0,0,0,0,0};
static C_char C_TLS li738[] C_aligned={C_lihdr(0,0,15),40,99,104,101,99,107,53,52,55,56,32,112,32,110,41,0};
static C_char C_TLS li739[] C_aligned={C_lihdr(0,0,10),40,97,51,49,54,53,54,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li740[] C_aligned={C_lihdr(0,0,10),40,97,51,49,54,56,49,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li741[] C_aligned={C_lihdr(0,0,12),40,97,51,49,54,57,57,32,112,32,99,41,0,0,0,0};
static C_char C_TLS li742[] C_aligned={C_lihdr(0,0,14),40,97,51,49,55,49,56,32,112,32,115,116,114,41,0,0};
static C_char C_TLS li743[] C_aligned={C_lihdr(0,0,10),40,97,51,49,55,51,55,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li744[] C_aligned={C_lihdr(0,0,10),40,97,51,49,55,52,48,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li745[] C_aligned={C_lihdr(0,0,23),40,97,51,49,55,52,51,32,112,32,110,32,100,101,115,116,32,115,116,97,114,116,41,0};
static C_char C_TLS li746[] C_aligned={C_lihdr(0,0,12),40,97,51,49,56,49,56,32,112,111,115,41,0,0,0,0};
static C_char C_TLS li747[] C_aligned={C_lihdr(0,0,8),40,97,51,49,56,48,56,41};
static C_char C_TLS li748[] C_aligned={C_lihdr(0,0,29),40,97,51,49,56,50,55,32,110,101,120,116,32,108,105,110,101,32,102,117,108,108,45,108,105,110,101,63,41,0,0,0};
static C_char C_TLS li749[] C_aligned={C_lihdr(0,0,16),40,97,51,49,55,56,52,32,112,32,108,105,109,105,116,41};
static C_char C_TLS li750[] C_aligned={C_lihdr(0,0,10),40,97,51,49,56,54,56,32,112,41,0,0,0,0,0,0};
static C_char C_TLS li751[] C_aligned={C_lihdr(0,0,37),40,99,111,112,121,38,97,112,112,101,110,100,32,98,117,102,32,111,102,102,115,101,116,32,112,111,115,32,111,108,100,45,108,105,110,101,41,0,0,0};
static C_char C_TLS li752[] C_aligned={C_lihdr(0,0,8),40,97,50,53,51,52,56,41};
static C_char C_TLS li753[] C_aligned={C_lihdr(0,0,25),40,97,50,53,51,53,52,32,98,117,102,32,111,102,102,115,101,116,32,108,105,109,105,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li754[] C_aligned={C_lihdr(0,0,8),40,97,50,53,52,49,50,41};
static C_char C_TLS li755[] C_aligned={C_lihdr(0,0,25),40,97,50,53,52,49,56,32,98,117,102,32,111,102,102,115,101,116,32,108,105,109,105,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li756[] C_aligned={C_lihdr(0,0,32),40,108,111,111,112,32,98,117,102,32,111,102,102,115,101,116,32,112,111,115,32,108,105,109,105,116,32,108,105,110,101,41};
static C_char C_TLS li757[] C_aligned={C_lihdr(0,0,56),40,35,35,115,121,115,35,115,99,97,110,45,98,117,102,102,101,114,45,108,105,110,101,32,98,117,102,32,108,105,109,105,116,32,115,116,97,114,116,45,112,111,115,32,101,111,115,45,104,97,110,100,108,101,114,41};
static C_char C_TLS li758[] C_aligned={C_lihdr(0,0,39),40,99,104,105,99,107,101,110,46,98,97,115,101,35,111,112,101,110,45,105,110,112,117,116,45,115,116,114,105,110,103,32,115,116,114,105,110,103,41,0};
static C_char C_TLS li759[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,98,97,115,101,35,111,112,101,110,45,111,117,116,112,117,116,45,115,116,114,105,110,103,41,0,0,0,0,0,0,0};
static C_char C_TLS li760[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,98,97,115,101,35,103,101,116,45,111,117,116,112,117,116,45,115,116,114,105,110,103,32,112,111,114,116,41,0,0,0};
static C_char C_TLS li761[] C_aligned={C_lihdr(0,0,9),40,103,53,54,48,56,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li762[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,54,48,55,32,103,53,54,49,52,41,0,0,0,0,0,0,0};
static C_char C_TLS li763[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,112,114,105,110,116,45,116,111,45,115,116,114,105,110,103,32,120,115,41,0,0,0,0,0,0};
static C_char C_TLS li764[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,112,111,105,110,116,101,114,45,62,115,116,114,105,110,103,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li765[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li766[] C_aligned={C_lihdr(0,0,36),40,99,104,105,99,107,101,110,46,98,97,115,101,35,103,101,116,45,99,97,108,108,45,99,104,97,105,110,32,46,32,114,101,115,116,41,0,0,0,0};
static C_char C_TLS li767[] C_aligned={C_lihdr(0,0,8),40,97,50,53,56,54,48,41};
static C_char C_TLS li768[] C_aligned={C_lihdr(0,0,12),40,103,53,54,55,52,32,105,110,102,111,41,0,0,0,0};
static C_char C_TLS li769[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,54,55,51,32,103,53,54,56,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li770[] C_aligned={C_lihdr(0,0,49),40,35,35,115,121,115,35,114,101,97,108,108,121,45,112,114,105,110,116,45,99,97,108,108,45,99,104,97,105,110,32,112,111,114,116,32,99,104,97,105,110,32,104,101,97,100,101,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li771[] C_aligned={C_lihdr(0,0,38),40,99,104,105,99,107,101,110,46,98,97,115,101,35,112,114,105,110,116,45,99,97,108,108,45,99,104,97,105,110,32,46,32,114,101,115,116,41,0,0};
static C_char C_TLS li772[] C_aligned={C_lihdr(0,0,7),40,98,114,101,97,107,41,0};
static C_char C_TLS li773[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,117,115,101,114,45,105,110,116,101,114,114,117,112,116,45,104,111,111,107,41,0,0,0,0,0};
static C_char C_TLS li774[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,55,51,51,32,103,53,55,52,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li775[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li776[] C_aligned={C_lihdr(0,0,21),40,99,108,101,97,110,117,112,45,98,101,102,111,114,101,45,101,120,105,116,41,0,0,0};
static C_char C_TLS li777[] C_aligned={C_lihdr(0,0,8),40,97,50,54,49,56,57,41};
static C_char C_TLS li778[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,56,48,51,32,103,53,56,49,48,41,0,0,0,0,0,0,0};
static C_char C_TLS li779[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,115,105,103,110,97,108,45,104,111,111,107,32,109,111,100,101,32,109,115,103,32,46,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li780[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,97,98,111,114,116,32,120,41,0,0,0,0,0};
static C_char C_TLS li781[] C_aligned={C_lihdr(0,0,28),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,115,105,103,110,97,108,32,120,41,0,0,0,0};
static C_char C_TLS li782[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,99,117,114,114,101,110,116,45,101,120,99,101,112,116,105,111,110,45,104,97,110,100,108,101,114,32,99,41,0,0,0,0,0};
static C_char C_TLS li783[] C_aligned={C_lihdr(0,0,8),40,97,50,54,54,50,57,41};
static C_char C_TLS li784[] C_aligned={C_lihdr(0,0,8),40,97,50,54,54,51,51,41};
static C_char C_TLS li785[] C_aligned={C_lihdr(0,0,56),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,119,105,116,104,45,101,120,99,101,112,116,105,111,110,45,104,97,110,100,108,101,114,32,104,97,110,100,108,101,114,32,116,104,117,110,107,41};
static C_char C_TLS li786[] C_aligned={C_lihdr(0,0,10),40,98,111,100,121,53,57,54,51,41,0,0,0,0,0,0};
static C_char C_TLS li787[] C_aligned={C_lihdr(0,0,52),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,117,114,114,101,110,116,45,101,120,99,101,112,116,105,111,110,45,104,97,110,100,108,101,114,32,46,32,97,114,103,115,41,0,0,0,0};
static C_char C_TLS li788[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,114,111,112,115,41,0,0,0,0};
static C_char C_TLS li789[] C_aligned={C_lihdr(0,0,69),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,112,114,111,112,45,108,105,115,116,45,62,107,105,110,100,45,112,114,101,102,105,120,101,100,45,112,114,111,112,45,108,105,115,116,32,108,111,99,32,107,105,110,100,32,112,108,105,115,116,41,0,0,0};
static C_char C_TLS li790[] C_aligned={C_lihdr(0,0,56),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,109,97,107,101,45,112,114,111,112,101,114,116,121,45,99,111,110,100,105,116,105,111,110,32,107,105,110,100,32,46,32,112,114,111,112,115,41};
static C_char C_TLS li791[] C_aligned={C_lihdr(0,0,19),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,48,48,55,41,0,0,0,0,0};
static C_char C_TLS li792[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,54,48,53,50,32,103,54,48,54,52,41,0,0,0,0};
static C_char C_TLS li793[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,54,48,50,53,32,103,54,48,51,55,41,0,0,0,0};
static C_char C_TLS li794[] C_aligned={C_lihdr(0,0,55),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,109,97,107,101,45,99,111,109,112,111,115,105,116,101,45,99,111,110,100,105,116,105,111,110,32,99,49,32,46,32,99,111,110,100,115,41,0};
static C_char C_TLS li795[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,54,49,49,50,32,103,54,49,50,52,41,0,0,0,0};
static C_char C_TLS li796[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,54,48,56,53,32,103,54,48,57,55,41,0,0,0,0};
static C_char C_TLS li797[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,111,110,100,105,116,105,111,110,32,97,114,103,49,32,46,32,97,114,103,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li798[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,111,110,100,105,116,105,111,110,63,32,120,41};
static C_char C_TLS li799[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,114,111,112,115,41,0,0,0,0};
static C_char C_TLS li800[] C_aligned={C_lihdr(0,0,9),40,103,54,49,52,56,32,107,41,0,0,0,0,0,0,0};
static C_char C_TLS li801[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,54,49,52,50,32,103,54,49,53,52,41,0,0,0,0};
static C_char C_TLS li802[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,111,110,100,105,116,105,111,110,45,62,108,105,115,116,32,120,41,0,0,0};
static C_char C_TLS li803[] C_aligned={C_lihdr(0,0,11),40,102,95,50,55,49,53,50,32,99,41,0,0,0,0,0};
static C_char C_TLS li804[] C_aligned={C_lihdr(0,0,44),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,111,110,100,105,116,105,111,110,45,112,114,101,100,105,99,97,116,101,32,107,105,110,100,41,0,0,0,0};
static C_char C_TLS li805[] C_aligned={C_lihdr(0,0,11),40,102,95,50,55,49,55,57,32,99,41,0,0,0,0,0};
static C_char C_TLS li806[] C_aligned={C_lihdr(0,0,67),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,99,111,110,100,105,116,105,111,110,45,112,114,111,112,101,114,116,121,45,97,99,99,101,115,115,111,114,32,107,105,110,100,32,112,114,111,112,32,46,32,101,114,114,45,100,101,102,41,0,0,0,0,0};
static C_char C_TLS li807[] C_aligned={C_lihdr(0,0,64),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,103,101,116,45,99,111,110,100,105,116,105,111,110,45,112,114,111,112,101,114,116,121,32,99,32,107,105,110,100,32,112,114,111,112,32,46,32,101,114,114,45,100,101,102,41};
static C_char C_TLS li808[] C_aligned={C_lihdr(0,0,8),40,97,50,55,50,52,53,41};
static C_char C_TLS li809[] C_aligned={C_lihdr(0,0,9),40,103,54,50,49,51,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li810[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,54,50,49,50,32,103,54,50,49,57,41,0,0,0,0,0,0,0};
static C_char C_TLS li811[] C_aligned={C_lihdr(0,0,21),40,119,114,105,116,101,97,114,103,115,32,97,114,103,115,32,112,111,114,116,41,0,0,0};
static C_char C_TLS li812[] C_aligned={C_lihdr(0,0,11),40,103,54,50,53,54,32,109,115,103,41,0,0,0,0,0};
static C_char C_TLS li813[] C_aligned={C_lihdr(0,0,22),40,98,111,100,121,54,50,51,51,32,112,111,114,116,32,104,101,97,100,101,114,41,0,0};
static C_char C_TLS li814[] C_aligned={C_lihdr(0,0,22),40,100,101,102,45,104,101,97,100,101,114,54,50,51,54,32,37,112,111,114,116,41,0,0};
static C_char C_TLS li815[] C_aligned={C_lihdr(0,0,14),40,100,101,102,45,112,111,114,116,54,50,51,53,41,0,0};
static C_char C_TLS li816[] C_aligned={C_lihdr(0,0,49),40,99,104,105,99,107,101,110,46,99,111,110,100,105,116,105,111,110,35,112,114,105,110,116,45,101,114,114,111,114,45,109,101,115,115,97,103,101,32,101,120,32,46,32,97,114,103,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li817[] C_aligned={C_lihdr(0,0,47),40,35,35,115,121,115,35,115,104,111,119,45,101,120,99,101,112,116,105,111,110,45,119,97,114,110,105,110,103,32,101,120,110,32,99,97,117,115,101,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li818[] C_aligned={C_lihdr(0,0,34),40,35,35,115,121,115,35,101,114,114,111,114,45,104,111,111,107,32,99,111,100,101,32,108,111,99,32,46,32,97,114,103,115,41,0,0,0,0,0,0};
static C_char C_TLS li819[] C_aligned={C_lihdr(0,0,22),40,35,35,115,121,115,35,115,116,114,117,99,116,117,114,101,63,32,120,32,115,41,0,0};
static C_char C_TLS li820[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,103,101,110,101,114,105,99,45,115,116,114,117,99,116,117,114,101,63,32,120,41,0,0,0,0};
static C_char C_TLS li821[] C_aligned={C_lihdr(0,0,16),40,35,35,115,121,115,35,115,108,111,116,32,120,32,105,41};
static C_char C_TLS li822[] C_aligned={C_lihdr(0,0,14),40,35,35,115,121,115,35,115,105,122,101,32,120,41,0,0};
static C_char C_TLS li823[] C_aligned={C_lihdr(0,0,14),67,95,109,97,107,101,95,112,111,105,110,116,101,114,0,0};
static C_char C_TLS li824[] C_aligned={C_lihdr(0,0,21),67,95,109,97,107,101,95,116,97,103,103,101,100,95,112,111,105,110,116,101,114,0,0,0};
static C_char C_TLS li825[] C_aligned={C_lihdr(0,0,18),40,35,35,115,121,115,35,112,111,105,110,116,101,114,63,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li826[] C_aligned={C_lihdr(0,0,37),40,35,35,115,121,115,35,115,101,116,45,112,111,105,110,116,101,114,45,97,100,100,114,101,115,115,33,32,112,116,114,32,97,100,100,114,41,0,0,0};
static C_char C_TLS li827[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,98,121,116,101,118,101,99,116,111,114,63,32,120,41,0,0,0};
static C_char C_TLS li828[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,115,116,114,105,110,103,45,62,112,98,121,116,101,118,101,99,116,111,114,32,115,41,0,0,0};
static C_char C_TLS li829[] C_aligned={C_lihdr(0,0,20),40,35,35,115,121,115,35,112,101,114,109,97,110,101,110,116,63,32,120,41,0,0,0,0};
static C_char C_TLS li830[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,98,108,111,99,107,45,97,100,100,114,101,115,115,32,120,41,0};
static C_char C_TLS li831[] C_aligned={C_lihdr(0,0,19),40,35,35,115,121,115,35,108,111,99,97,116,105,118,101,63,32,120,41,0,0,0,0,0};
static C_char C_TLS li832[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,115,114,102,105,45,52,45,118,101,99,116,111,114,63,32,120,41};
static C_char C_TLS li833[] C_aligned={C_lihdr(0,0,20),40,35,35,115,121,115,35,110,117,108,108,45,112,111,105,110,116,101,114,41,0,0,0,0};
static C_char C_TLS li834[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,110,117,108,108,45,112,111,105,110,116,101,114,63,32,120,41,0};
static C_char C_TLS li835[] C_aligned={C_lihdr(0,0,29),40,35,35,115,121,115,35,97,100,100,114,101,115,115,45,62,112,111,105,110,116,101,114,32,97,100,100,114,41,0,0,0};
static C_char C_TLS li836[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,112,111,105,110,116,101,114,45,62,97,100,100,114,101,115,115,32,112,116,114,41,0,0,0,0};
static C_char C_TLS li837[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,109,97,107,101,45,99,45,115,116,114,105,110,103,32,115,116,114,32,46,32,114,101,115,116,41};
static C_char C_TLS li838[] C_aligned={C_lihdr(0,0,21),67,95,112,101,101,107,95,115,105,103,110,101,100,95,105,110,116,101,103,101,114,0,0,0};
static C_char C_TLS li839[] C_aligned={C_lihdr(0,0,23),67,95,112,101,101,107,95,117,110,115,105,103,110,101,100,95,105,110,116,101,103,101,114,0};
static C_char C_TLS li840[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,112,101,101,107,45,102,105,120,110,117,109,32,98,32,105,41,0};
static C_char C_TLS li841[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,112,101,101,107,45,98,121,116,101,32,112,116,114,32,105,41,0};
static C_char C_TLS li842[] C_aligned={C_lihdr(0,0,30),40,35,35,115,121,115,35,118,101,99,116,111,114,45,62,115,116,114,117,99,116,117,114,101,33,32,118,101,99,41,0,0};
static C_char C_TLS li843[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,112,101,101,107,45,100,111,117,98,108,101,32,98,32,105,41,0};
static C_char C_TLS li844[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,112,101,101,107,45,99,45,115,116,114,105,110,103,32,98,32,105,41,0,0,0,0,0,0,0};
static C_char C_TLS li845[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,112,101,101,107,45,110,111,110,110,117,108,108,45,99,45,115,116,114,105,110,103,32,98,32,105,41,0,0,0,0,0,0,0};
static C_char C_TLS li846[] C_aligned={C_lihdr(0,0,34),40,35,35,115,121,115,35,112,101,101,107,45,97,110,100,45,102,114,101,101,45,99,45,115,116,114,105,110,103,32,98,32,105,41,0,0,0,0,0,0};
static C_char C_TLS li847[] C_aligned={C_lihdr(0,0,42),40,35,35,115,121,115,35,112,101,101,107,45,97,110,100,45,102,114,101,101,45,110,111,110,110,117,108,108,45,99,45,115,116,114,105,110,103,32,98,32,105,41,0,0,0,0,0,0};
static C_char C_TLS li848[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,112,111,107,101,45,99,45,115,116,114,105,110,103,32,98,32,105,32,115,41,0,0,0,0,0};
static C_char C_TLS li849[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,112,111,107,101,45,105,110,116,101,103,101,114,32,98,32,105,32,110,41,0,0,0,0,0,0};
static C_char C_TLS li850[] C_aligned={C_lihdr(0,0,25),40,35,35,115,121,115,35,112,111,107,101,45,100,111,117,98,108,101,32,98,32,105,32,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li851[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li852[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,112,101,101,107,45,99,45,115,116,114,105,110,103,45,108,105,115,116,32,112,116,114,32,110,41};
static C_char C_TLS li853[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li854[] C_aligned={C_lihdr(0,0,41),40,35,35,115,121,115,35,112,101,101,107,45,97,110,100,45,102,114,101,101,45,99,45,115,116,114,105,110,103,45,108,105,115,116,32,112,116,114,32,110,41,0,0,0,0,0,0,0};
static C_char C_TLS li855[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,118,101,99,116,111,114,45,62,99,108,111,115,117,114,101,33,32,118,101,99,32,97,100,100,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li856[] C_aligned={C_lihdr(0,0,38),40,35,35,115,121,115,35,115,121,109,98,111,108,45,104,97,115,45,116,111,112,108,101,118,101,108,45,98,105,110,100,105,110,103,63,32,115,41,0,0};
static C_char C_TLS li857[] C_aligned={C_lihdr(0,0,48),40,35,35,115,121,115,35,99,111,112,121,45,98,121,116,101,115,32,102,114,111,109,32,116,111,32,111,102,102,115,101,116,49,32,111,102,102,115,101,116,50,32,98,121,116,101,115,41};
static C_char C_TLS li858[] C_aligned={C_lihdr(0,0,48),40,35,35,115,121,115,35,99,111,112,121,45,119,111,114,100,115,32,102,114,111,109,32,116,111,32,111,102,102,115,101,116,49,32,111,102,102,115,101,116,50,32,119,111,114,100,115,41};
static C_char C_TLS li859[] C_aligned={C_lihdr(0,0,51),40,35,35,115,121,115,35,99,111,109,112,97,114,101,45,98,121,116,101,115,32,102,114,111,109,32,116,111,32,111,102,102,115,101,116,49,32,111,102,102,115,101,116,50,32,98,121,116,101,115,41,0,0,0,0,0};
static C_char C_TLS li860[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,98,108,111,99,107,45,112,111,105,110,116,101,114,32,120,41,0};
static C_char C_TLS li861[] C_aligned={C_lihdr(0,0,31),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,99,104,97,114,45,97,114,103,117,109,101,110,116,32,120,41,0};
static C_char C_TLS li862[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,102,105,120,110,117,109,45,97,114,103,117,109,101,110,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li863[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,102,108,111,110,117,109,45,97,114,103,117,109,101,110,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li864[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,98,108,111,99,107,45,97,114,103,117,109,101,110,116,32,120,41};
static C_char C_TLS li865[] C_aligned={C_lihdr(0,0,43),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,115,116,114,117,99,116,45,119,114,97,112,112,101,114,45,97,114,103,117,109,101,110,116,32,116,32,120,41,0,0,0,0,0};
static C_char C_TLS li866[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,115,116,114,105,110,103,45,97,114,103,117,109,101,110,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li867[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,115,121,109,98,111,108,45,97,114,103,117,109,101,110,116,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li868[] C_aligned={C_lihdr(0,0,34),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,112,111,105,110,116,101,114,45,97,114,103,117,109,101,110,116,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li869[] C_aligned={C_lihdr(0,0,44),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,116,97,103,103,101,100,45,112,111,105,110,116,101,114,45,97,114,103,117,109,101,110,116,32,120,32,116,120,41,0,0,0,0};
static C_char C_TLS li870[] C_aligned={C_lihdr(0,0,48),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,114,97,110,103,101,100,45,105,110,116,101,103,101,114,45,97,114,103,117,109,101,110,116,32,111,98,106,32,115,105,122,101,41};
static C_char C_TLS li871[] C_aligned={C_lihdr(0,0,57),40,35,35,115,121,115,35,102,111,114,101,105,103,110,45,117,110,115,105,103,110,101,100,45,114,97,110,103,101,100,45,105,110,116,101,103,101,114,45,97,114,103,117,109,101,110,116,32,111,98,106,32,115,105,122,101,41,0,0,0,0,0,0,0};
static C_char C_TLS li872[] C_aligned={C_lihdr(0,0,37),40,35,35,115,121,115,35,100,101,102,97,117,108,116,45,101,120,99,101,112,116,105,111,110,45,104,97,110,100,108,101,114,32,97,114,103,41,0,0,0};
static C_char C_TLS li873[] C_aligned={C_lihdr(0,0,38),40,35,35,115,121,115,35,109,97,107,101,45,116,104,114,101,97,100,32,116,104,117,110,107,32,115,116,97,116,101,32,110,97,109,101,32,113,41,0,0};
static C_char C_TLS li874[] C_aligned={C_lihdr(0,0,27),40,35,35,115,121,115,35,109,97,107,101,45,109,117,116,101,120,32,105,100,32,111,119,110,101,114,41,0,0,0,0,0};
static C_char C_TLS li875[] C_aligned={C_lihdr(0,0,16),40,35,35,115,121,115,35,115,99,104,101,100,117,108,101,41};
static C_char C_TLS li876[] C_aligned={C_lihdr(0,0,8),40,97,50,56,54,51,53,41};
static C_char C_TLS li877[] C_aligned={C_lihdr(0,0,15),40,97,50,56,54,50,54,32,114,101,116,117,114,110,41,0};
static C_char C_TLS li878[] C_aligned={C_lihdr(0,0,21),40,35,35,115,121,115,35,116,104,114,101,97,100,45,121,105,101,108,100,33,41,0,0,0};
static C_char C_TLS li879[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,107,105,108,108,45,111,116,104,101,114,45,116,104,114,101,97,100,115,32,116,104,117,110,107,41};
static C_char C_TLS li880[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,108,101,101,112,45,104,111,111,107,32,110,41,0,0,0,0,0};
static C_char C_TLS li881[] C_aligned={C_lihdr(0,0,22),40,99,104,105,99,107,101,110,46,98,97,115,101,35,115,108,101,101,112,32,110,41,0,0};
static C_char C_TLS li882[] C_aligned={C_lihdr(0,0,16),67,95,99,111,110,116,101,120,116,95,115,119,105,116,99,104};
static C_char C_TLS li883[] C_aligned={C_lihdr(0,0,13),40,108,111,111,112,32,114,101,97,115,111,110,41,0,0,0};
static C_char C_TLS li884[] C_aligned={C_lihdr(0,0,35),40,35,35,115,121,115,35,105,110,116,101,114,114,117,112,116,45,104,111,111,107,32,114,101,97,115,111,110,32,115,116,97,116,101,41,0,0,0,0,0};
static C_char C_TLS li885[] C_aligned={C_lihdr(0,0,28),40,35,35,115,121,115,35,100,105,115,112,97,116,99,104,45,105,110,116,101,114,114,117,112,116,32,107,41,0,0,0,0};
static C_char C_TLS li886[] C_aligned={C_lihdr(0,0,20),40,35,35,115,121,115,35,117,112,100,97,116,101,45,101,114,114,110,111,41,0,0,0,0};
static C_char C_TLS li887[] C_aligned={C_lihdr(0,0,13),40,35,35,115,121,115,35,101,114,114,110,111,41,0,0,0};
static C_char C_TLS li888[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,54,54,51,54,32,99,41,0,0};
static C_char C_TLS li889[] C_aligned={C_lihdr(0,0,13),40,114,101,97,100,108,110,32,112,111,114,116,41,0,0,0};
static C_char C_TLS li890[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li891[] C_aligned={C_lihdr(0,0,36),40,114,101,97,100,45,101,115,99,97,112,101,100,45,115,101,120,112,32,112,111,114,116,32,115,107,105,112,45,98,114,97,99,101,63,41,0,0,0,0};
static C_char C_TLS li892[] C_aligned={C_lihdr(0,0,15),40,100,111,108,111,111,112,54,54,55,49,32,108,110,41,0};
static C_char C_TLS li893[] C_aligned={C_lihdr(0,0,15),40,103,101,116,47,99,108,101,97,114,45,115,116,114,41,0};
static C_char C_TLS li894[] C_aligned={C_lihdr(0,0,27),40,108,111,111,112,50,32,108,115,116,32,110,101,120,116,45,115,116,114,105,110,103,32,97,99,99,41,0,0,0,0,0};
static C_char C_TLS li895[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,108,115,116,41,0,0,0,0,0,0};
static C_char C_TLS li896[] C_aligned={C_lihdr(0,0,32),40,35,35,115,121,115,35,117,115,101,114,45,114,101,97,100,45,104,111,111,107,32,99,104,97,114,32,112,111,114,116,41};
static C_char C_TLS li897[] C_aligned={C_lihdr(0,0,47),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,99,104,97,110,103,101,45,100,105,114,101,99,116,111,114,121,32,110,97,109,101,41,0};
static C_char C_TLS li898[] C_aligned={C_lihdr(0,0,33),40,35,35,115,121,115,35,99,104,97,110,103,101,45,100,105,114,101,99,116,111,114,121,45,104,111,111,107,32,100,105,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li899[] C_aligned={C_lihdr(0,0,71),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,103,101,116,45,101,110,118,105,114,111,110,109,101,110,116,45,118,97,114,105,97,98,108,101,32,110,111,110,110,117,108,108,45,99,45,115,116,114,105,110,103,54,55,56,57,41,0};
static C_char C_TLS li900[] C_aligned={C_lihdr(0,0,59),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,115,101,116,45,101,110,118,105,114,111,110,109,101,110,116,45,118,97,114,105,97,98,108,101,33,32,118,97,114,32,118,97,108,41,0,0,0,0,0};
static C_char C_TLS li901[] C_aligned={C_lihdr(0,0,57),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,117,110,115,101,116,45,101,110,118,105,114,111,110,109,101,110,116,45,118,97,114,105,97,98,108,101,33,32,118,97,114,41,0,0,0,0,0,0,0};
static C_char C_TLS li902[] C_aligned={C_lihdr(0,0,8),40,115,99,97,110,32,106,41};
static C_char C_TLS li903[] C_aligned={C_lihdr(0,0,8),40,108,111,111,112,32,105,41};
static C_char C_TLS li904[] C_aligned={C_lihdr(0,0,51),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,103,101,116,45,101,110,118,105,114,111,110,109,101,110,116,45,118,97,114,105,97,98,108,101,115,41,0,0,0,0,0};
static C_char C_TLS li905[] C_aligned={C_lihdr(0,0,45),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,101,120,101,99,117,116,97,98,108,101,45,112,97,116,104,110,97,109,101,41,0,0,0};
static C_char C_TLS li906[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,97,114,103,99,43,97,114,103,118,41,0,0,0,0,0};
static C_char C_TLS li907[] C_aligned={C_lihdr(0,0,16),40,100,111,108,111,111,112,54,56,51,49,32,105,32,118,41};
static C_char C_TLS li908[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,112,114,111,99,101,115,115,45,99,111,110,116,101,120,116,35,97,114,103,118,41,0,0};
static C_char C_TLS li909[] C_aligned={C_lihdr(0,0,36),40,99,104,105,99,107,101,110,46,103,99,35,99,117,114,114,101,110,116,45,103,99,45,109,105,108,108,105,115,101,99,111,110,100,115,41,0,0,0,0};
static C_char C_TLS li910[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,103,99,35,115,101,116,45,103,99,45,114,101,112,111,114,116,33,32,102,108,97,103,41};
static C_char C_TLS li911[] C_aligned={C_lihdr(0,0,30),40,99,104,105,99,107,101,110,46,103,99,35,109,101,109,111,114,121,45,115,116,97,116,105,115,116,105,99,115,41,0,0};
static C_char C_TLS li912[] C_aligned={C_lihdr(0,0,20),67,95,114,101,103,105,115,116,101,114,95,102,105,110,97,108,105,122,101,114,0,0,0,0};
static C_char C_TLS li913[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,103,99,35,115,101,116,45,102,105,110,97,108,105,122,101,114,33,32,120,32,121,41,0};
static C_char C_TLS li914[] C_aligned={C_lihdr(0,0,8),40,97,50,57,54,55,52,41};
static C_char C_TLS li915[] C_aligned={C_lihdr(0,0,11),40,97,50,57,54,54,56,32,101,120,41,0,0,0,0,0};
static C_char C_TLS li916[] C_aligned={C_lihdr(0,0,8),40,97,50,57,54,56,54,41};
static C_char C_TLS li917[] C_aligned={C_lihdr(0,0,8),40,97,50,57,55,48,57,41};
static C_char C_TLS li918[] C_aligned={C_lihdr(0,0,15),40,97,50,57,55,48,51,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li919[] C_aligned={C_lihdr(0,0,8),40,97,50,57,54,56,48,41};
static C_char C_TLS li920[] C_aligned={C_lihdr(0,0,10),40,97,50,57,54,54,50,32,107,41,0,0,0,0,0,0};
static C_char C_TLS li921[] C_aligned={C_lihdr(0,0,14),40,100,111,108,111,111,112,54,57,49,50,32,105,41,0,0};
static C_char C_TLS li922[] C_aligned={C_lihdr(0,0,36),40,35,35,115,121,115,35,114,117,110,45,112,101,110,100,105,110,103,45,102,105,110,97,108,105,122,101,114,115,32,115,116,97,116,101,41,0,0,0,0};
static C_char C_TLS li923[] C_aligned={C_lihdr(0,0,6),40,108,111,111,112,41,0,0};
static C_char C_TLS li924[] C_aligned={C_lihdr(0,0,24),40,35,35,115,121,115,35,102,111,114,99,101,45,102,105,110,97,108,105,122,101,114,115,41};
static C_char C_TLS li925[] C_aligned={C_lihdr(0,0,21),40,99,104,105,99,107,101,110,46,103,99,35,103,99,32,46,32,97,114,103,41,0,0,0};
static C_char C_TLS li926[] C_aligned={C_lihdr(0,0,41),40,35,35,115,121,115,35,109,97,107,101,45,108,111,99,97,116,105,118,101,32,111,98,106,32,105,110,100,101,120,32,119,101,97,107,63,32,108,111,99,41,0,0,0,0,0,0,0};
static C_char C_TLS li927[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,112,108,105,115,116,35,112,117,116,33,32,115,121,109,32,112,114,111,112,32,118,97,108,41,0,0,0,0,0,0,0};
static C_char C_TLS li928[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,112,108,105,115,116,35,103,101,116,32,115,121,109,32,112,114,111,112,32,46,32,114,101,115,116,41,0,0,0,0,0};
static C_char C_TLS li929[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,112,116,108,41,0,0,0,0,0,0};
static C_char C_TLS li930[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,112,108,105,115,116,35,114,101,109,112,114,111,112,33,32,115,121,109,32,112,114,111,112,41,0,0,0,0,0,0,0};
static C_char C_TLS li931[] C_aligned={C_lihdr(0,0,12),40,108,111,111,112,32,112,108,105,115,116,41,0,0,0,0};
static C_char C_TLS li932[] C_aligned={C_lihdr(0,0,40),40,99,104,105,99,107,101,110,46,112,108,105,115,116,35,103,101,116,45,112,114,111,112,101,114,116,105,101,115,32,115,121,109,32,112,114,111,112,115,41};
static C_char C_TLS li933[] C_aligned={C_lihdr(0,0,10),40,112,110,117,109,32,110,117,109,41,0,0,0,0,0,0};
static C_char C_TLS li934[] C_aligned={C_lihdr(0,0,14),40,114,111,117,110,100,45,116,111,32,120,32,121,41,0,0};
static C_char C_TLS li935[] C_aligned={C_lihdr(0,0,12),40,112,109,101,109,32,98,121,116,101,115,41,0,0,0,0};
static C_char C_TLS li936[] C_aligned={C_lihdr(0,0,26),40,35,35,115,121,115,35,100,105,115,112,108,97,121,45,116,105,109,101,115,32,105,110,102,111,41,0,0,0,0,0,0};
static C_char C_TLS li937[] C_aligned={C_lihdr(0,0,17),67,95,100,117,109,112,95,104,101,97,112,95,115,116,97,116,101,0,0,0,0,0,0,0};
static C_char C_TLS li938[] C_aligned={C_lihdr(0,0,21),67,95,102,105,108,116,101,114,95,104,101,97,112,95,111,98,106,101,99,116,115,0,0,0};
static C_char C_TLS li939[] C_aligned={C_lihdr(0,0,32),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,115,111,102,116,119,97,114,101,45,116,121,112,101,41};
static C_char C_TLS li940[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,109,97,99,104,105,110,101,45,116,121,112,101,41,0};
static C_char C_TLS li941[] C_aligned={C_lihdr(0,0,37),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,109,97,99,104,105,110,101,45,98,121,116,101,45,111,114,100,101,114,41,0,0,0};
static C_char C_TLS li942[] C_aligned={C_lihdr(0,0,35),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,115,111,102,116,119,97,114,101,45,118,101,114,115,105,111,110,41,0,0,0,0,0};
static C_char C_TLS li943[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,98,117,105,108,100,45,112,108,97,116,102,111,114,109,41,0,0,0,0,0,0,0};
static C_char C_TLS li944[] C_aligned={C_lihdr(0,0,7),40,115,116,114,32,120,41,0};
static C_char C_TLS li945[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,99,104,105,99,107,101,110,45,118,101,114,115,105,111,110,32,46,32,114,101,115,116,41,0,0,0,0,0,0,0};
static C_char C_TLS li946[] C_aligned={C_lihdr(0,0,31),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,99,104,105,99,107,101,110,45,104,111,109,101,41,0};
static C_char C_TLS li947[] C_aligned={C_lihdr(0,0,16),40,108,111,111,112,32,115,116,97,114,116,32,112,111,115,41};
static C_char C_TLS li948[] C_aligned={C_lihdr(0,0,23),40,35,35,115,121,115,35,115,112,108,105,116,45,112,97,116,104,32,112,97,116,104,41,0};
static C_char C_TLS li949[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,45,62,102,101,97,116,117,114,101,45,105,100,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li950[] C_aligned={C_lihdr(0,0,9),40,99,104,101,99,107,32,102,41,0,0,0,0,0,0,0};
static C_char C_TLS li951[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,55,51,50,52,32,103,55,51,51,49,41,0,0,0,0,0,0,0};
static C_char C_TLS li952[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,114,101,103,105,115,116,101,114,45,102,101,97,116,117,114,101,33,32,46,32,102,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li953[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,102,102,115,41,0,0,0,0,0,0};
static C_char C_TLS li954[] C_aligned={C_lihdr(0,0,20),40,109,97,112,45,108,111,111,112,55,51,52,55,32,103,55,51,53,57,41,0,0,0,0};
static C_char C_TLS li955[] C_aligned={C_lihdr(0,0,43),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,117,110,114,101,103,105,115,116,101,114,45,102,101,97,116,117,114,101,33,32,46,32,102,115,41,0,0,0,0,0};
static C_char C_TLS li956[] C_aligned={C_lihdr(0,0,27),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,102,101,97,116,117,114,101,115,41,0,0,0,0,0};
static C_char C_TLS li957[] C_aligned={C_lihdr(0,0,10),40,108,111,111,112,32,105,100,115,41,0,0,0,0,0,0};
static C_char C_TLS li958[] C_aligned={C_lihdr(0,0,33),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,102,101,97,116,117,114,101,63,32,46,32,105,100,115,41,0,0,0,0,0,0,0};
static C_char C_TLS li959[] C_aligned={C_lihdr(0,0,16),67,95,114,101,116,117,114,110,95,116,111,95,104,111,115,116};
static C_char C_TLS li960[] C_aligned={C_lihdr(0,0,42),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,115,121,115,116,101,109,45,99,111,110,102,105,103,45,100,105,114,101,99,116,111,114,121,41,0,0,0,0,0,0};
static C_char C_TLS li961[] C_aligned={C_lihdr(0,0,41),40,99,104,105,99,107,101,110,46,112,108,97,116,102,111,114,109,35,115,121,115,116,101,109,45,99,97,99,104,101,45,100,105,114,101,99,116,111,114,121,41,0,0,0,0,0,0,0};
static C_char C_TLS li962[] C_aligned={C_lihdr(0,0,19),40,102,111,114,45,101,97,99,104,45,108,111,111,112,55,50,54,54,41,0,0,0,0,0};
static C_char C_TLS li963[] C_aligned={C_lihdr(0,0,12),40,97,51,49,49,57,54,32,110,101,119,41,0,0,0,0};
static C_char C_TLS li964[] C_aligned={C_lihdr(0,0,16),67,95,98,117,105,108,100,95,112,108,97,116,102,111,114,109};
static C_char C_TLS li965[] C_aligned={C_lihdr(0,0,18),67,95,115,111,102,116,119,97,114,101,95,118,101,114,115,105,111,110,0,0,0,0,0,0};
static C_char C_TLS li966[] C_aligned={C_lihdr(0,0,20),67,95,109,97,99,104,105,110,101,95,98,121,116,101,95,111,114,100,101,114,0,0,0,0};
static C_char C_TLS li967[] C_aligned={C_lihdr(0,0,14),67,95,109,97,99,104,105,110,101,95,116,121,112,101,0,0};
static C_char C_TLS li968[] C_aligned={C_lihdr(0,0,15),67,95,115,111,102,116,119,97,114,101,95,116,121,112,101,0};
static C_char C_TLS li969[] C_aligned={C_lihdr(0,0,12),40,97,51,49,50,57,53,32,115,121,109,41,0,0,0,0};
static C_char C_TLS li970[] C_aligned={C_lihdr(0,0,16),40,97,51,49,51,48,52,32,115,121,109,32,108,115,116,41};
static C_char C_TLS li971[] C_aligned={C_lihdr(0,0,10),40,97,51,49,51,52,52,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li972[] C_aligned={C_lihdr(0,0,11),40,108,111,111,112,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li973[] C_aligned={C_lihdr(0,0,19),40,102,51,55,49,50,55,32,120,54,56,52,56,51,55,49,50,54,41,0,0,0,0,0};
static C_char C_TLS li974[] C_aligned={C_lihdr(0,0,19),40,102,51,55,49,51,51,32,120,54,56,51,56,51,55,49,51,50,41,0,0,0,0,0};
static C_char C_TLS li975[] C_aligned={C_lihdr(0,0,19),40,102,51,55,49,51,57,32,120,54,56,51,56,51,55,49,51,56,41,0,0,0,0,0};
static C_char C_TLS li976[] C_aligned={C_lihdr(0,0,8),40,97,51,49,52,50,57,41};
static C_char C_TLS li977[] C_aligned={C_lihdr(0,0,12),40,97,51,49,52,53,49,32,100,105,114,41,0,0,0,0};
static C_char C_TLS li978[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,56,54,56,32,103,53,56,55,53,41,0,0,0,0,0,0,0};
static C_char C_TLS li979[] C_aligned={C_lihdr(0,0,8),40,97,51,49,53,48,50,41};
static C_char C_TLS li980[] C_aligned={C_lihdr(0,0,9),40,103,53,57,48,48,32,120,41,0,0,0,0,0,0,0};
static C_char C_TLS li981[] C_aligned={C_lihdr(0,0,25),40,102,111,114,45,101,97,99,104,45,108,111,111,112,53,56,57,57,32,103,53,57,48,54,41,0,0,0,0,0,0,0};
static C_char C_TLS li982[] C_aligned={C_lihdr(0,0,15),40,97,51,49,54,49,52,32,46,32,97,114,103,115,41,0};
static C_char C_TLS li983[] C_aligned={C_lihdr(0,0,19),40,97,51,49,52,53,55,32,109,115,103,32,46,32,97,114,103,115,41,0,0,0,0,0};
static C_char C_TLS li984[] C_aligned={C_lihdr(0,0,8),40,97,51,49,54,49,55,41};
static C_char C_TLS li985[] C_aligned={C_lihdr(0,0,8),40,97,51,49,54,50,54,41};
static C_char C_TLS li986[] C_aligned={C_lihdr(0,0,15),40,97,51,49,54,51,50,32,46,32,114,101,115,116,41,0};
static C_char C_TLS li987[] C_aligned={C_lihdr(0,0,10),40,97,51,49,56,57,54,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li988[] C_aligned={C_lihdr(0,0,14),40,97,51,49,57,48,53,32,120,32,105,32,121,41,0,0};
static C_char C_TLS li989[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,49,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li990[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,50,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li991[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,51,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li992[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,52,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li993[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,53,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li994[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,54,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li995[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,55,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li996[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,56,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li997[] C_aligned={C_lihdr(0,0,12),40,97,51,49,57,57,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li998[] C_aligned={C_lihdr(0,0,12),40,97,51,50,48,48,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li999[] C_aligned={C_lihdr(0,0,12),40,97,51,50,48,49,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li1000[] C_aligned={C_lihdr(0,0,12),40,97,51,50,48,50,53,32,120,32,121,41,0,0,0,0};
static C_char C_TLS li1001[] C_aligned={C_lihdr(0,0,10),40,97,51,50,48,53,57,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li1002[] C_aligned={C_lihdr(0,0,13),40,97,51,50,48,51,53,32,112,114,111,99,41,0,0,0};
static C_char C_TLS li1003[] C_aligned={C_lihdr(0,0,10),40,97,51,50,48,55,53,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li1004[] C_aligned={C_lihdr(0,0,10),40,97,51,50,49,51,48,32,120,41,0,0,0,0,0,0};
static C_char C_TLS li1005[] C_aligned={C_lihdr(0,0,15),40,97,51,50,49,52,54,32,112,114,111,99,32,105,41,0};
static C_char C_TLS li1006[] C_aligned={C_lihdr(0,0,16),40,97,51,50,49,48,49,32,103,101,116,32,115,101,116,41};
static C_char C_TLS li1007[] C_aligned={C_lihdr(0,0,15),40,97,51,50,48,57,49,32,112,114,111,99,32,105,41,0};
static C_char C_TLS li1008[] C_aligned={C_lihdr(0,0,10),40,116,111,112,108,101,118,101,108,41,0,0,0,0,0,0};


#define return(x) C_cblock C_r = (C_mpointer(&C_a,(void*)(x))); goto C_ret; C_cblockend
C_regparm static C_word C_fcall stub6827(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_word i=(C_word )(C_a0);
C_return(C_main_argv[C_unfix(i)]);
C_ret:
#undef return

return C_r;}

/* from chicken.process-context#executable-pathname in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
C_regparm static C_word C_fcall stub6819(C_word C_buf){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_r=C_mpointer(&C_a,(void*)C_executable_pathname());
return C_r;}

/* from k29351 */
C_regparm static C_word C_fcall stub6807(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
int t0=(int )C_unfix(C_a0);
C_r=C_mpointer(&C_a,(void*)C_getenventry(t0));
return C_r;}

/* from k29309 in chicken.process-context#get-environment-variable in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
C_regparm static C_word C_fcall stub6790(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
char * t0=(char * )C_c_string(C_a0);
C_r=C_mpointer(&C_a,(void*)C_getenv(t0));
return C_r;}

/* from k28469 */
C_regparm static C_word C_fcall stub6503(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
void * t0=(void * )C_c_pointer_or_null(C_a0);
C_free(t0);
return C_r;}

/* from k28460 */
C_regparm static C_word C_fcall stub6495(C_word C_buf,C_word C_a0,C_word C_a1){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
void * t0=(void * )C_c_pointer_or_null(C_a0);
int t1=(int )C_unfix(C_a1);
C_r=C_mpointer(&C_a,(void*)C_peek_c_string_at(t0,t1));
return C_r;}

/* from k28408 */
C_regparm static C_word C_fcall stub6476(C_word C_buf,C_word C_a0,C_word C_a1){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
void * t0=(void * )C_c_pointer_or_null(C_a0);
int t1=(int )C_unfix(C_a1);
C_r=C_mpointer(&C_a,(void*)C_peek_c_string_at(t0,t1));
return C_r;}

#define return(x) C_cblock C_r = (C_mpointer(&C_a,(void*)(x))); goto C_ret; C_cblockend
C_regparm static C_word C_fcall stub5630(C_word C_buf,C_word C_a0){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
C_word x=(C_word )(C_a0);
C_return((C_char *)x);
C_ret:
#undef return

return C_r;}

/* from k15603 */
C_regparm static C_word C_fcall stub2831(C_word C_buf,C_word C_a0,C_word C_a1){
C_word C_r=C_SCHEME_UNDEFINED,*C_a=(C_word*)C_buf;
double t0=(double )C_c_double(C_a0);
int t1=(int )C_unfix(C_a1);
C_r=C_flonum(&C_a,ldexp(t0,t1));
return C_r;}

C_noret_decl(f34270)
static void C_ccall f34270(C_word c,C_word *av) C_noret;
C_noret_decl(f34823)
static void C_ccall f34823(C_word c,C_word *av) C_noret;
C_noret_decl(f35032)
static void C_ccall f35032(C_word c,C_word *av) C_noret;
C_noret_decl(f35036)
static void C_ccall f35036(C_word c,C_word *av) C_noret;
C_noret_decl(f37127)
static void C_ccall f37127(C_word c,C_word *av) C_noret;
C_noret_decl(f37133)
static void C_ccall f37133(C_word c,C_word *av) C_noret;
C_noret_decl(f37139)
static void C_ccall f37139(C_word c,C_word *av) C_noret;
C_noret_decl(f_10007)
static void C_ccall f_10007(C_word c,C_word *av) C_noret;
C_noret_decl(f_10036)
static void C_ccall f_10036(C_word c,C_word *av) C_noret;
C_noret_decl(f_10043)
static void C_fcall f_10043(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10046)
static void C_ccall f_10046(C_word c,C_word *av) C_noret;
C_noret_decl(f_10075)
static void C_ccall f_10075(C_word c,C_word *av) C_noret;
C_noret_decl(f_10081)
static void C_fcall f_10081(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10097)
static void C_ccall f_10097(C_word c,C_word *av) C_noret;
C_noret_decl(f_10122)
static void C_ccall f_10122(C_word c,C_word *av) C_noret;
C_noret_decl(f_10131)
static void C_fcall f_10131(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10172)
static void C_ccall f_10172(C_word c,C_word *av) C_noret;
C_noret_decl(f_10194)
static void C_ccall f_10194(C_word c,C_word *av) C_noret;
C_noret_decl(f_10226)
static void C_ccall f_10226(C_word c,C_word *av) C_noret;
C_noret_decl(f_10242)
static void C_fcall f_10242(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10245)
static void C_ccall f_10245(C_word c,C_word *av) C_noret;
C_noret_decl(f_10267)
static void C_ccall f_10267(C_word c,C_word *av) C_noret;
C_noret_decl(f_10273)
static void C_fcall f_10273(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10298)
static void C_ccall f_10298(C_word c,C_word *av) C_noret;
C_noret_decl(f_10338)
static void C_ccall f_10338(C_word c,C_word *av) C_noret;
C_noret_decl(f_10350)
static void C_fcall f_10350(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10369)
static void C_ccall f_10369(C_word c,C_word *av) C_noret;
C_noret_decl(f_10391)
static void C_ccall f_10391(C_word c,C_word *av) C_noret;
C_noret_decl(f_10394)
static void C_ccall f_10394(C_word c,C_word *av) C_noret;
C_noret_decl(f_10397)
static void C_ccall f_10397(C_word c,C_word *av) C_noret;
C_noret_decl(f_10400)
static void C_ccall f_10400(C_word c,C_word *av) C_noret;
C_noret_decl(f_10406)
static void C_ccall f_10406(C_word c,C_word *av) C_noret;
C_noret_decl(f_10409)
static void C_ccall f_10409(C_word c,C_word *av) C_noret;
C_noret_decl(f_10412)
static void C_ccall f_10412(C_word c,C_word *av) C_noret;
C_noret_decl(f_10415)
static void C_ccall f_10415(C_word c,C_word *av) C_noret;
C_noret_decl(f_10418)
static void C_ccall f_10418(C_word c,C_word *av) C_noret;
C_noret_decl(f_10421)
static void C_ccall f_10421(C_word c,C_word *av) C_noret;
C_noret_decl(f_10424)
static void C_ccall f_10424(C_word c,C_word *av) C_noret;
C_noret_decl(f_10427)
static void C_ccall f_10427(C_word c,C_word *av) C_noret;
C_noret_decl(f_10430)
static void C_ccall f_10430(C_word c,C_word *av) C_noret;
C_noret_decl(f_10433)
static void C_ccall f_10433(C_word c,C_word *av) C_noret;
C_noret_decl(f_10436)
static void C_ccall f_10436(C_word c,C_word *av) C_noret;
C_noret_decl(f_10439)
static void C_ccall f_10439(C_word c,C_word *av) C_noret;
C_noret_decl(f_10442)
static void C_ccall f_10442(C_word c,C_word *av) C_noret;
C_noret_decl(f_10445)
static void C_ccall f_10445(C_word c,C_word *av) C_noret;
C_noret_decl(f_10448)
static void C_ccall f_10448(C_word c,C_word *av) C_noret;
C_noret_decl(f_10451)
static void C_ccall f_10451(C_word c,C_word *av) C_noret;
C_noret_decl(f_10454)
static void C_ccall f_10454(C_word c,C_word *av) C_noret;
C_noret_decl(f_10457)
static void C_ccall f_10457(C_word c,C_word *av) C_noret;
C_noret_decl(f_10460)
static void C_ccall f_10460(C_word c,C_word *av) C_noret;
C_noret_decl(f_10463)
static void C_ccall f_10463(C_word c,C_word *av) C_noret;
C_noret_decl(f_10466)
static void C_ccall f_10466(C_word c,C_word *av) C_noret;
C_noret_decl(f_10469)
static void C_ccall f_10469(C_word c,C_word *av) C_noret;
C_noret_decl(f_10472)
static void C_ccall f_10472(C_word c,C_word *av) C_noret;
C_noret_decl(f_10475)
static void C_ccall f_10475(C_word c,C_word *av) C_noret;
C_noret_decl(f_10479)
static void C_ccall f_10479(C_word c,C_word *av) C_noret;
C_noret_decl(f_10481)
static void C_ccall f_10481(C_word c,C_word *av) C_noret;
C_noret_decl(f_10486)
static void C_ccall f_10486(C_word c,C_word *av) C_noret;
C_noret_decl(f_10495)
static void C_ccall f_10495(C_word c,C_word *av) C_noret;
C_noret_decl(f_10507)
static void C_ccall f_10507(C_word c,C_word *av) C_noret;
C_noret_decl(f_10522)
static void C_ccall f_10522(C_word c,C_word *av) C_noret;
C_noret_decl(f_10525)
static void C_ccall f_10525(C_word c,C_word *av) C_noret;
C_noret_decl(f_10533)
static void C_ccall f_10533(C_word c,C_word *av) C_noret;
C_noret_decl(f_10537)
static void C_ccall f_10537(C_word c,C_word *av) C_noret;
C_noret_decl(f_10539)
static void C_ccall f_10539(C_word c,C_word *av) C_noret;
C_noret_decl(f_10543)
static void C_ccall f_10543(C_word c,C_word *av) C_noret;
C_noret_decl(f_10546)
static void C_ccall f_10546(C_word c,C_word *av) C_noret;
C_noret_decl(f_10557)
static void C_ccall f_10557(C_word c,C_word *av) C_noret;
C_noret_decl(f_10559)
static void C_ccall f_10559(C_word c,C_word *av) C_noret;
C_noret_decl(f_10566)
static void C_ccall f_10566(C_word c,C_word *av) C_noret;
C_noret_decl(f_10569)
static void C_ccall f_10569(C_word c,C_word *av) C_noret;
C_noret_decl(f_10572)
static void C_ccall f_10572(C_word c,C_word *av) C_noret;
C_noret_decl(f_10577)
static void C_ccall f_10577(C_word c,C_word *av) C_noret;
C_noret_decl(f_10581)
static void C_ccall f_10581(C_word c,C_word *av) C_noret;
C_noret_decl(f_10586)
static void C_ccall f_10586(C_word c,C_word *av) C_noret;
C_noret_decl(f_10590)
static void C_ccall f_10590(C_word c,C_word *av) C_noret;
C_noret_decl(f_10593)
static void C_ccall f_10593(C_word c,C_word *av) C_noret;
C_noret_decl(f_10621)
static void C_ccall f_10621(C_word c,C_word *av) C_noret;
C_noret_decl(f_10624)
static void C_ccall f_10624(C_word c,C_word *av) C_noret;
C_noret_decl(f_10627)
static void C_ccall f_10627(C_word c,C_word *av) C_noret;
C_noret_decl(f_10630)
static void C_ccall f_10630(C_word c,C_word *av) C_noret;
C_noret_decl(f_10647)
static void C_ccall f_10647(C_word c,C_word *av) C_noret;
C_noret_decl(f_10658)
static void C_ccall f_10658(C_word c,C_word *av) C_noret;
C_noret_decl(f_10670)
static void C_ccall f_10670(C_word c,C_word *av) C_noret;
C_noret_decl(f_10681)
static void C_ccall f_10681(C_word c,C_word *av) C_noret;
C_noret_decl(f_10692)
static void C_ccall f_10692(C_word c,C_word *av) C_noret;
C_noret_decl(f_10703)
static void C_ccall f_10703(C_word c,C_word *av) C_noret;
C_noret_decl(f_10714)
static void C_ccall f_10714(C_word c,C_word *av) C_noret;
C_noret_decl(f_10725)
static void C_ccall f_10725(C_word c,C_word *av) C_noret;
C_noret_decl(f_10736)
static void C_ccall f_10736(C_word c,C_word *av) C_noret;
C_noret_decl(f_10747)
static void C_ccall f_10747(C_word c,C_word *av) C_noret;
C_noret_decl(f_10758)
static void C_ccall f_10758(C_word c,C_word *av) C_noret;
C_noret_decl(f_10769)
static void C_ccall f_10769(C_word c,C_word *av) C_noret;
C_noret_decl(f_10780)
static void C_ccall f_10780(C_word c,C_word *av) C_noret;
C_noret_decl(f_10791)
static void C_ccall f_10791(C_word c,C_word *av) C_noret;
C_noret_decl(f_10802)
static void C_ccall f_10802(C_word c,C_word *av) C_noret;
C_noret_decl(f_10813)
static void C_ccall f_10813(C_word c,C_word *av) C_noret;
C_noret_decl(f_10830)
static void C_ccall f_10830(C_word c,C_word *av) C_noret;
C_noret_decl(f_10847)
static void C_ccall f_10847(C_word c,C_word *av) C_noret;
C_noret_decl(f_10871)
static void C_ccall f_10871(C_word c,C_word *av) C_noret;
C_noret_decl(f_10888)
static void C_ccall f_10888(C_word c,C_word *av) C_noret;
C_noret_decl(f_10892)
static void C_ccall f_10892(C_word c,C_word *av) C_noret;
C_noret_decl(f_10918)
static void C_ccall f_10918(C_word c,C_word *av) C_noret;
C_noret_decl(f_10938)
static void C_ccall f_10938(C_word c,C_word *av) C_noret;
C_noret_decl(f_10949)
static void C_ccall f_10949(C_word c,C_word *av) C_noret;
C_noret_decl(f_10961)
static void C_fcall f_10961(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_10995)
static void C_ccall f_10995(C_word c,C_word *av) C_noret;
C_noret_decl(f_11010)
static void C_fcall f_11010(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11077)
static void C_ccall f_11077(C_word c,C_word *av) C_noret;
C_noret_decl(f_11093)
static void C_ccall f_11093(C_word c,C_word *av) C_noret;
C_noret_decl(f_11109)
static void C_ccall f_11109(C_word c,C_word *av) C_noret;
C_noret_decl(f_11125)
static void C_ccall f_11125(C_word c,C_word *av) C_noret;
C_noret_decl(f_11141)
static void C_ccall f_11141(C_word c,C_word *av) C_noret;
C_noret_decl(f_11157)
static void C_ccall f_11157(C_word c,C_word *av) C_noret;
C_noret_decl(f_11173)
static void C_ccall f_11173(C_word c,C_word *av) C_noret;
C_noret_decl(f_11189)
static void C_ccall f_11189(C_word c,C_word *av) C_noret;
C_noret_decl(f_11205)
static void C_ccall f_11205(C_word c,C_word *av) C_noret;
C_noret_decl(f_11217)
static void C_fcall f_11217(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11236)
static void C_fcall f_11236(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11267)
static void C_ccall f_11267(C_word c,C_word *av) C_noret;
C_noret_decl(f_11284)
static void C_ccall f_11284(C_word c,C_word *av) C_noret;
C_noret_decl(f_11290)
static void C_fcall f_11290(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11314)
static void C_ccall f_11314(C_word c,C_word *av) C_noret;
C_noret_decl(f_11334)
static void C_ccall f_11334(C_word c,C_word *av) C_noret;
C_noret_decl(f_11338)
static void C_ccall f_11338(C_word c,C_word *av) C_noret;
C_noret_decl(f_11341)
static void C_ccall f_11341(C_word c,C_word *av) C_noret;
C_noret_decl(f_11370)
static void C_ccall f_11370(C_word c,C_word *av) C_noret;
C_noret_decl(f_11386)
static void C_fcall f_11386(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11408)
static void C_ccall f_11408(C_word c,C_word *av) C_noret;
C_noret_decl(f_11424)
static void C_ccall f_11424(C_word c,C_word *av) C_noret;
C_noret_decl(f_11429)
static C_word C_fcall f_11429(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_11460)
static void C_ccall f_11460(C_word c,C_word *av) C_noret;
C_noret_decl(f_11471)
static void C_ccall f_11471(C_word c,C_word *av) C_noret;
C_noret_decl(f_11480)
static C_word C_fcall f_11480(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_11509)
static void C_ccall f_11509(C_word c,C_word *av) C_noret;
C_noret_decl(f_11522)
static void C_ccall f_11522(C_word c,C_word *av) C_noret;
C_noret_decl(f_11532)
static void C_ccall f_11532(C_word c,C_word *av) C_noret;
C_noret_decl(f_11534)
static void C_ccall f_11534(C_word c,C_word *av) C_noret;
C_noret_decl(f_11541)
static void C_ccall f_11541(C_word c,C_word *av) C_noret;
C_noret_decl(f_11544)
static void C_fcall f_11544(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11553)
static void C_fcall f_11553(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11579)
static void C_ccall f_11579(C_word c,C_word *av) C_noret;
C_noret_decl(f_11584)
static void C_ccall f_11584(C_word c,C_word *av) C_noret;
C_noret_decl(f_11588)
static void C_ccall f_11588(C_word c,C_word *av) C_noret;
C_noret_decl(f_11593)
static void C_ccall f_11593(C_word c,C_word *av) C_noret;
C_noret_decl(f_11614)
static void C_ccall f_11614(C_word c,C_word *av) C_noret;
C_noret_decl(f_11620)
static void C_ccall f_11620(C_word c,C_word *av) C_noret;
C_noret_decl(f_11638)
static void C_ccall f_11638(C_word c,C_word *av) C_noret;
C_noret_decl(f_11644)
static void C_ccall f_11644(C_word c,C_word *av) C_noret;
C_noret_decl(f_11662)
static void C_ccall f_11662(C_word c,C_word *av) C_noret;
C_noret_decl(f_11668)
static void C_ccall f_11668(C_word c,C_word *av) C_noret;
C_noret_decl(f_11683)
static void C_ccall f_11683(C_word c,C_word *av) C_noret;
C_noret_decl(f_11689)
static void C_ccall f_11689(C_word c,C_word *av) C_noret;
C_noret_decl(f_11703)
static void C_ccall f_11703(C_word c,C_word *av) C_noret;
C_noret_decl(f_11724)
static void C_ccall f_11724(C_word c,C_word *av) C_noret;
C_noret_decl(f_11730)
static void C_ccall f_11730(C_word c,C_word *av) C_noret;
C_noret_decl(f_11748)
static void C_ccall f_11748(C_word c,C_word *av) C_noret;
C_noret_decl(f_11754)
static void C_ccall f_11754(C_word c,C_word *av) C_noret;
C_noret_decl(f_11772)
static void C_ccall f_11772(C_word c,C_word *av) C_noret;
C_noret_decl(f_11778)
static void C_ccall f_11778(C_word c,C_word *av) C_noret;
C_noret_decl(f_11793)
static void C_ccall f_11793(C_word c,C_word *av) C_noret;
C_noret_decl(f_11799)
static void C_ccall f_11799(C_word c,C_word *av) C_noret;
C_noret_decl(f_11814)
static void C_ccall f_11814(C_word c,C_word *av) C_noret;
C_noret_decl(f_11824)
static void C_ccall f_11824(C_word c,C_word *av) C_noret;
C_noret_decl(f_11830)
static void C_ccall f_11830(C_word c,C_word *av) C_noret;
C_noret_decl(f_11834)
static void C_ccall f_11834(C_word c,C_word *av) C_noret;
C_noret_decl(f_11836)
static void C_fcall f_11836(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11847)
static void C_ccall f_11847(C_word c,C_word *av) C_noret;
C_noret_decl(f_11859)
static void C_ccall f_11859(C_word c,C_word *av) C_noret;
C_noret_decl(f_11869)
static void C_ccall f_11869(C_word c,C_word *av) C_noret;
C_noret_decl(f_11875)
static void C_ccall f_11875(C_word c,C_word *av) C_noret;
C_noret_decl(f_11879)
static void C_ccall f_11879(C_word c,C_word *av) C_noret;
C_noret_decl(f_11884)
static C_word C_fcall f_11884(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_11910)
static void C_ccall f_11910(C_word c,C_word *av) C_noret;
C_noret_decl(f_11914)
static void C_ccall f_11914(C_word c,C_word *av) C_noret;
C_noret_decl(f_11917)
static void C_ccall f_11917(C_word c,C_word *av) C_noret;
C_noret_decl(f_11925)
static void C_fcall f_11925(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11946)
static void C_fcall f_11946(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_11960)
static void C_ccall f_11960(C_word c,C_word *av) C_noret;
C_noret_decl(f_11964)
static void C_ccall f_11964(C_word c,C_word *av) C_noret;
C_noret_decl(f_12005)
static void C_ccall f_12005(C_word c,C_word *av) C_noret;
C_noret_decl(f_12027)
static void C_ccall f_12027(C_word c,C_word *av) C_noret;
C_noret_decl(f_12029)
static void C_ccall f_12029(C_word c,C_word *av) C_noret;
C_noret_decl(f_12051)
static void C_ccall f_12051(C_word c,C_word *av) C_noret;
C_noret_decl(f_12053)
static void C_ccall f_12053(C_word c,C_word *av) C_noret;
C_noret_decl(f_12075)
static void C_ccall f_12075(C_word c,C_word *av) C_noret;
C_noret_decl(f_12077)
static void C_ccall f_12077(C_word c,C_word *av) C_noret;
C_noret_decl(f_12099)
static void C_ccall f_12099(C_word c,C_word *av) C_noret;
C_noret_decl(f_12101)
static void C_ccall f_12101(C_word c,C_word *av) C_noret;
C_noret_decl(f_12123)
static void C_ccall f_12123(C_word c,C_word *av) C_noret;
C_noret_decl(f_12125)
static void C_ccall f_12125(C_word c,C_word *av) C_noret;
C_noret_decl(f_12147)
static void C_ccall f_12147(C_word c,C_word *av) C_noret;
C_noret_decl(f_12149)
static void C_ccall f_12149(C_word c,C_word *av) C_noret;
C_noret_decl(f_12171)
static void C_ccall f_12171(C_word c,C_word *av) C_noret;
C_noret_decl(f_12173)
static void C_ccall f_12173(C_word c,C_word *av) C_noret;
C_noret_decl(f_12195)
static void C_ccall f_12195(C_word c,C_word *av) C_noret;
C_noret_decl(f_12197)
static void C_ccall f_12197(C_word c,C_word *av) C_noret;
C_noret_decl(f_12219)
static void C_ccall f_12219(C_word c,C_word *av) C_noret;
C_noret_decl(f_12221)
static void C_ccall f_12221(C_word c,C_word *av) C_noret;
C_noret_decl(f_12243)
static void C_ccall f_12243(C_word c,C_word *av) C_noret;
C_noret_decl(f_12245)
static void C_ccall f_12245(C_word c,C_word *av) C_noret;
C_noret_decl(f_12267)
static void C_ccall f_12267(C_word c,C_word *av) C_noret;
C_noret_decl(f_12269)
static void C_ccall f_12269(C_word c,C_word *av) C_noret;
C_noret_decl(f_12285)
static void C_ccall f_12285(C_word c,C_word *av) C_noret;
C_noret_decl(f_12287)
static void C_ccall f_12287(C_word c,C_word *av) C_noret;
C_noret_decl(f_12309)
static void C_ccall f_12309(C_word c,C_word *av) C_noret;
C_noret_decl(f_12311)
static void C_ccall f_12311(C_word c,C_word *av) C_noret;
C_noret_decl(f_12333)
static void C_ccall f_12333(C_word c,C_word *av) C_noret;
C_noret_decl(f_12335)
static void C_ccall f_12335(C_word c,C_word *av) C_noret;
C_noret_decl(f_12351)
static void C_ccall f_12351(C_word c,C_word *av) C_noret;
C_noret_decl(f_12353)
static void C_ccall f_12353(C_word c,C_word *av) C_noret;
C_noret_decl(f_12369)
static void C_ccall f_12369(C_word c,C_word *av) C_noret;
C_noret_decl(f_12371)
static void C_ccall f_12371(C_word c,C_word *av) C_noret;
C_noret_decl(f_12387)
static void C_ccall f_12387(C_word c,C_word *av) C_noret;
C_noret_decl(f_12389)
static void C_ccall f_12389(C_word c,C_word *av) C_noret;
C_noret_decl(f_12405)
static void C_ccall f_12405(C_word c,C_word *av) C_noret;
C_noret_decl(f_12407)
static void C_ccall f_12407(C_word c,C_word *av) C_noret;
C_noret_decl(f_12423)
static void C_ccall f_12423(C_word c,C_word *av) C_noret;
C_noret_decl(f_12425)
static void C_ccall f_12425(C_word c,C_word *av) C_noret;
C_noret_decl(f_12441)
static void C_ccall f_12441(C_word c,C_word *av) C_noret;
C_noret_decl(f_12443)
static void C_ccall f_12443(C_word c,C_word *av) C_noret;
C_noret_decl(f_12459)
static void C_ccall f_12459(C_word c,C_word *av) C_noret;
C_noret_decl(f_12461)
static void C_ccall f_12461(C_word c,C_word *av) C_noret;
C_noret_decl(f_12477)
static void C_ccall f_12477(C_word c,C_word *av) C_noret;
C_noret_decl(f_12479)
static void C_ccall f_12479(C_word c,C_word *av) C_noret;
C_noret_decl(f_12495)
static void C_ccall f_12495(C_word c,C_word *av) C_noret;
C_noret_decl(f_12497)
static void C_ccall f_12497(C_word c,C_word *av) C_noret;
C_noret_decl(f_12513)
static void C_ccall f_12513(C_word c,C_word *av) C_noret;
C_noret_decl(f_12515)
static void C_ccall f_12515(C_word c,C_word *av) C_noret;
C_noret_decl(f_12537)
static void C_ccall f_12537(C_word c,C_word *av) C_noret;
C_noret_decl(f_12539)
static void C_ccall f_12539(C_word c,C_word *av) C_noret;
C_noret_decl(f_12555)
static void C_ccall f_12555(C_word c,C_word *av) C_noret;
C_noret_decl(f_12557)
static void C_ccall f_12557(C_word c,C_word *av) C_noret;
C_noret_decl(f_12579)
static void C_ccall f_12579(C_word c,C_word *av) C_noret;
C_noret_decl(f_12581)
static void C_ccall f_12581(C_word c,C_word *av) C_noret;
C_noret_decl(f_12597)
static void C_ccall f_12597(C_word c,C_word *av) C_noret;
C_noret_decl(f_12599)
static void C_ccall f_12599(C_word c,C_word *av) C_noret;
C_noret_decl(f_12615)
static void C_ccall f_12615(C_word c,C_word *av) C_noret;
C_noret_decl(f_12617)
static void C_ccall f_12617(C_word c,C_word *av) C_noret;
C_noret_decl(f_12633)
static void C_ccall f_12633(C_word c,C_word *av) C_noret;
C_noret_decl(f_12635)
static void C_ccall f_12635(C_word c,C_word *av) C_noret;
C_noret_decl(f_12651)
static void C_ccall f_12651(C_word c,C_word *av) C_noret;
C_noret_decl(f_12653)
static void C_ccall f_12653(C_word c,C_word *av) C_noret;
C_noret_decl(f_12663)
static void C_ccall f_12663(C_word c,C_word *av) C_noret;
C_noret_decl(f_12675)
static void C_fcall f_12675(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12685)
static void C_ccall f_12685(C_word c,C_word *av) C_noret;
C_noret_decl(f_12689)
static void C_ccall f_12689(C_word c,C_word *av) C_noret;
C_noret_decl(f_12711)
static void C_ccall f_12711(C_word c,C_word *av) C_noret;
C_noret_decl(f_12715)
static void C_ccall f_12715(C_word c,C_word *av) C_noret;
C_noret_decl(f_12718)
static void C_ccall f_12718(C_word c,C_word *av) C_noret;
C_noret_decl(f_12723)
static void C_ccall f_12723(C_word c,C_word *av) C_noret;
C_noret_decl(f_12727)
static void C_ccall f_12727(C_word c,C_word *av) C_noret;
C_noret_decl(f_12730)
static void C_ccall f_12730(C_word c,C_word *av) C_noret;
C_noret_decl(f_12733)
static void C_ccall f_12733(C_word c,C_word *av) C_noret;
C_noret_decl(f_12746)
static void C_ccall f_12746(C_word c,C_word *av) C_noret;
C_noret_decl(f_12769)
static void C_ccall f_12769(C_word c,C_word *av) C_noret;
C_noret_decl(f_12795)
static void C_ccall f_12795(C_word c,C_word *av) C_noret;
C_noret_decl(f_12804)
static void C_ccall f_12804(C_word c,C_word *av) C_noret;
C_noret_decl(f_12809)
static void C_ccall f_12809(C_word c,C_word *av) C_noret;
C_noret_decl(f_12811)
static void C_ccall f_12811(C_word c,C_word *av) C_noret;
C_noret_decl(f_12855)
static C_word C_fcall f_12855(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_12901)
static void C_ccall f_12901(C_word c,C_word *av) C_noret;
C_noret_decl(f_12925)
static void C_ccall f_12925(C_word c,C_word *av) C_noret;
C_noret_decl(f_12929)
static void C_ccall f_12929(C_word c,C_word *av) C_noret;
C_noret_decl(f_12947)
static void C_ccall f_12947(C_word c,C_word *av) C_noret;
C_noret_decl(f_12971)
static void C_ccall f_12971(C_word c,C_word *av) C_noret;
C_noret_decl(f_12975)
static void C_ccall f_12975(C_word c,C_word *av) C_noret;
C_noret_decl(f_12993)
static void C_ccall f_12993(C_word c,C_word *av) C_noret;
C_noret_decl(f_13034)
static void C_fcall f_13034(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13045)
static void C_ccall f_13045(C_word c,C_word *av) C_noret;
C_noret_decl(f_13051)
static void C_fcall f_13051(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13055)
static void C_ccall f_13055(C_word c,C_word *av) C_noret;
C_noret_decl(f_13069)
static void C_ccall f_13069(C_word c,C_word *av) C_noret;
C_noret_decl(f_13082)
static void C_ccall f_13082(C_word c,C_word *av) C_noret;
C_noret_decl(f_13086)
static void C_ccall f_13086(C_word c,C_word *av) C_noret;
C_noret_decl(f_13103)
static void C_ccall f_13103(C_word c,C_word *av) C_noret;
C_noret_decl(f_13107)
static void C_ccall f_13107(C_word c,C_word *av) C_noret;
C_noret_decl(f_13113)
static void C_ccall f_13113(C_word c,C_word *av) C_noret;
C_noret_decl(f_13117)
static void C_ccall f_13117(C_word c,C_word *av) C_noret;
C_noret_decl(f_13157)
static void C_ccall f_13157(C_word c,C_word *av) C_noret;
C_noret_decl(f_13161)
static void C_ccall f_13161(C_word c,C_word *av) C_noret;
C_noret_decl(f_13183)
static void C_ccall f_13183(C_word c,C_word *av) C_noret;
C_noret_decl(f_13186)
static void C_ccall f_13186(C_word c,C_word *av) C_noret;
C_noret_decl(f_13189)
static void C_ccall f_13189(C_word c,C_word *av) C_noret;
C_noret_decl(f_13192)
static void C_ccall f_13192(C_word c,C_word *av) C_noret;
C_noret_decl(f_13195)
static void C_ccall f_13195(C_word c,C_word *av) C_noret;
C_noret_decl(f_13216)
static void C_ccall f_13216(C_word c,C_word *av) C_noret;
C_noret_decl(f_13218)
static void C_fcall f_13218(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13236)
static void C_ccall f_13236(C_word c,C_word *av) C_noret;
C_noret_decl(f_13246)
static void C_ccall f_13246(C_word c,C_word *av) C_noret;
C_noret_decl(f_13250)
static void C_ccall f_13250(C_word c,C_word *av) C_noret;
C_noret_decl(f_13288)
static void C_ccall f_13288(C_word c,C_word *av) C_noret;
C_noret_decl(f_13291)
static void C_ccall f_13291(C_word c,C_word *av) C_noret;
C_noret_decl(f_13294)
static void C_ccall f_13294(C_word c,C_word *av) C_noret;
C_noret_decl(f_13297)
static void C_ccall f_13297(C_word c,C_word *av) C_noret;
C_noret_decl(f_13303)
static void C_ccall f_13303(C_word c,C_word *av) C_noret;
C_noret_decl(f_13306)
static void C_ccall f_13306(C_word c,C_word *av) C_noret;
C_noret_decl(f_13354)
static void C_ccall f_13354(C_word c,C_word *av) C_noret;
C_noret_decl(f_13358)
static void C_ccall f_13358(C_word c,C_word *av) C_noret;
C_noret_decl(f_13435)
static void C_ccall f_13435(C_word c,C_word *av) C_noret;
C_noret_decl(f_13474)
static void C_ccall f_13474(C_word c,C_word *av) C_noret;
C_noret_decl(f_13529)
static void C_ccall f_13529(C_word c,C_word *av) C_noret;
C_noret_decl(f_13574)
static void C_ccall f_13574(C_word c,C_word *av) C_noret;
C_noret_decl(f_13619)
static void C_ccall f_13619(C_word c,C_word *av) C_noret;
C_noret_decl(f_13656)
static void C_ccall f_13656(C_word c,C_word *av) C_noret;
C_noret_decl(f_13676)
static void C_ccall f_13676(C_word c,C_word *av) C_noret;
C_noret_decl(f_13700)
static void C_fcall f_13700(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13703)
static void C_fcall f_13703(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13707)
static void C_ccall f_13707(C_word c,C_word *av) C_noret;
C_noret_decl(f_13710)
static void C_ccall f_13710(C_word c,C_word *av) C_noret;
C_noret_decl(f_13728)
static void C_ccall f_13728(C_word c,C_word *av) C_noret;
C_noret_decl(f_13751)
static void C_ccall f_13751(C_word c,C_word *av) C_noret;
C_noret_decl(f_13755)
static void C_ccall f_13755(C_word c,C_word *av) C_noret;
C_noret_decl(f_13778)
static void C_ccall f_13778(C_word c,C_word *av) C_noret;
C_noret_decl(f_13782)
static void C_ccall f_13782(C_word c,C_word *av) C_noret;
C_noret_decl(f_13814)
static void C_ccall f_13814(C_word c,C_word *av) C_noret;
C_noret_decl(f_13850)
static void C_ccall f_13850(C_word c,C_word *av) C_noret;
C_noret_decl(f_13854)
static void C_ccall f_13854(C_word c,C_word *av) C_noret;
C_noret_decl(f_13873)
static void C_ccall f_13873(C_word c,C_word *av) C_noret;
C_noret_decl(f_13875)
static void C_ccall f_13875(C_word c,C_word *av) C_noret;
C_noret_decl(f_13882)
static void C_fcall f_13882(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13919)
static void C_ccall f_13919(C_word c,C_word *av) C_noret;
C_noret_decl(f_13926)
static void C_fcall f_13926(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_13963)
static void C_ccall f_13963(C_word c,C_word *av) C_noret;
C_noret_decl(f_13981)
static void C_ccall f_13981(C_word c,C_word *av) C_noret;
C_noret_decl(f_13990)
static void C_ccall f_13990(C_word c,C_word *av) C_noret;
C_noret_decl(f_13995)
static void C_ccall f_13995(C_word c,C_word *av) C_noret;
C_noret_decl(f_14008)
static void C_ccall f_14008(C_word c,C_word *av) C_noret;
C_noret_decl(f_14034)
static void C_ccall f_14034(C_word c,C_word *av) C_noret;
C_noret_decl(f_14042)
static void C_ccall f_14042(C_word c,C_word *av) C_noret;
C_noret_decl(f_14046)
static void C_ccall f_14046(C_word c,C_word *av) C_noret;
C_noret_decl(f_14050)
static void C_ccall f_14050(C_word c,C_word *av) C_noret;
C_noret_decl(f_14054)
static void C_ccall f_14054(C_word c,C_word *av) C_noret;
C_noret_decl(f_14057)
static void C_ccall f_14057(C_word c,C_word *av) C_noret;
C_noret_decl(f_14071)
static void C_ccall f_14071(C_word c,C_word *av) C_noret;
C_noret_decl(f_14075)
static void C_ccall f_14075(C_word c,C_word *av) C_noret;
C_noret_decl(f_14086)
static void C_ccall f_14086(C_word c,C_word *av) C_noret;
C_noret_decl(f_14110)
static void C_ccall f_14110(C_word c,C_word *av) C_noret;
C_noret_decl(f_14114)
static void C_ccall f_14114(C_word c,C_word *av) C_noret;
C_noret_decl(f_14120)
static void C_ccall f_14120(C_word c,C_word *av) C_noret;
C_noret_decl(f_14144)
static void C_ccall f_14144(C_word c,C_word *av) C_noret;
C_noret_decl(f_14148)
static void C_ccall f_14148(C_word c,C_word *av) C_noret;
C_noret_decl(f_14154)
static void C_ccall f_14154(C_word c,C_word *av) C_noret;
C_noret_decl(f_14171)
static void C_ccall f_14171(C_word c,C_word *av) C_noret;
C_noret_decl(f_14175)
static void C_ccall f_14175(C_word c,C_word *av) C_noret;
C_noret_decl(f_14179)
static void C_ccall f_14179(C_word c,C_word *av) C_noret;
C_noret_decl(f_14189)
static void C_fcall f_14189(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14195)
static void C_fcall f_14195(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14202)
static void C_ccall f_14202(C_word c,C_word *av) C_noret;
C_noret_decl(f_14214)
static void C_ccall f_14214(C_word c,C_word *av) C_noret;
C_noret_decl(f_14237)
static void C_ccall f_14237(C_word c,C_word *av) C_noret;
C_noret_decl(f_14241)
static void C_ccall f_14241(C_word c,C_word *av) C_noret;
C_noret_decl(f_14251)
static void C_fcall f_14251(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14257)
static void C_fcall f_14257(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14264)
static void C_ccall f_14264(C_word c,C_word *av) C_noret;
C_noret_decl(f_14279)
static void C_ccall f_14279(C_word c,C_word *av) C_noret;
C_noret_decl(f_14283)
static void C_ccall f_14283(C_word c,C_word *av) C_noret;
C_noret_decl(f_14316)
static void C_ccall f_14316(C_word c,C_word *av) C_noret;
C_noret_decl(f_14320)
static void C_ccall f_14320(C_word c,C_word *av) C_noret;
C_noret_decl(f_14337)
static void C_ccall f_14337(C_word c,C_word *av) C_noret;
C_noret_decl(f_14347)
static void C_ccall f_14347(C_word c,C_word *av) C_noret;
C_noret_decl(f_14420)
static void C_ccall f_14420(C_word c,C_word *av) C_noret;
C_noret_decl(f_14434)
static void C_ccall f_14434(C_word c,C_word *av) C_noret;
C_noret_decl(f_14451)
static void C_ccall f_14451(C_word c,C_word *av) C_noret;
C_noret_decl(f_14467)
static void C_ccall f_14467(C_word c,C_word *av) C_noret;
C_noret_decl(f_14538)
static void C_ccall f_14538(C_word c,C_word *av) C_noret;
C_noret_decl(f_14542)
static void C_ccall f_14542(C_word c,C_word *av) C_noret;
C_noret_decl(f_14547)
static void C_ccall f_14547(C_word c,C_word *av) C_noret;
C_noret_decl(f_14557)
static void C_ccall f_14557(C_word c,C_word *av) C_noret;
C_noret_decl(f_14561)
static void C_ccall f_14561(C_word c,C_word *av) C_noret;
C_noret_decl(f_14576)
static void C_ccall f_14576(C_word c,C_word *av) C_noret;
C_noret_decl(f_14580)
static void C_ccall f_14580(C_word c,C_word *av) C_noret;
C_noret_decl(f_14597)
static void C_ccall f_14597(C_word c,C_word *av) C_noret;
C_noret_decl(f_14608)
static void C_ccall f_14608(C_word c,C_word *av) C_noret;
C_noret_decl(f_14614)
static void C_ccall f_14614(C_word c,C_word *av) C_noret;
C_noret_decl(f_14625)
static void C_ccall f_14625(C_word c,C_word *av) C_noret;
C_noret_decl(f_14636)
static void C_ccall f_14636(C_word c,C_word *av) C_noret;
C_noret_decl(f_14648)
static void C_ccall f_14648(C_word c,C_word *av) C_noret;
C_noret_decl(f_14660)
static void C_ccall f_14660(C_word c,C_word *av) C_noret;
C_noret_decl(f_14672)
static void C_ccall f_14672(C_word c,C_word *av) C_noret;
C_noret_decl(f_14689)
static void C_ccall f_14689(C_word c,C_word *av) C_noret;
C_noret_decl(f_14696)
static void C_ccall f_14696(C_word c,C_word *av) C_noret;
C_noret_decl(f_14700)
static void C_ccall f_14700(C_word c,C_word *av) C_noret;
C_noret_decl(f_14702)
static void C_ccall f_14702(C_word c,C_word *av) C_noret;
C_noret_decl(f_14708)
static void C_ccall f_14708(C_word c,C_word *av) C_noret;
C_noret_decl(f_14712)
static void C_ccall f_14712(C_word c,C_word *av) C_noret;
C_noret_decl(f_14715)
static void C_ccall f_14715(C_word c,C_word *av) C_noret;
C_noret_decl(f_14720)
static void C_ccall f_14720(C_word c,C_word *av) C_noret;
C_noret_decl(f_14730)
static void C_fcall f_14730(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14750)
static void C_ccall f_14750(C_word c,C_word *av) C_noret;
C_noret_decl(f_14765)
static void C_fcall f_14765(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14795)
static void C_ccall f_14795(C_word c,C_word *av) C_noret;
C_noret_decl(f_14806)
static void C_ccall f_14806(C_word c,C_word *av) C_noret;
C_noret_decl(f_14822)
static void C_ccall f_14822(C_word c,C_word *av) C_noret;
C_noret_decl(f_14826)
static void C_ccall f_14826(C_word c,C_word *av) C_noret;
C_noret_decl(f_14830)
static void C_ccall f_14830(C_word c,C_word *av) C_noret;
C_noret_decl(f_14846)
static void C_ccall f_14846(C_word c,C_word *av) C_noret;
C_noret_decl(f_14864)
static void C_ccall f_14864(C_word c,C_word *av) C_noret;
C_noret_decl(f_14875)
static void C_fcall f_14875(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_14897)
static void C_ccall f_14897(C_word c,C_word *av) C_noret;
C_noret_decl(f_14914)
static void C_ccall f_14914(C_word c,C_word *av) C_noret;
C_noret_decl(f_14917)
static void C_fcall f_14917(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14929)
static void C_ccall f_14929(C_word c,C_word *av) C_noret;
C_noret_decl(f_14931)
static void C_fcall f_14931(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14952)
static void C_ccall f_14952(C_word c,C_word *av) C_noret;
C_noret_decl(f_14983)
static void C_ccall f_14983(C_word c,C_word *av) C_noret;
C_noret_decl(f_14987)
static void C_ccall f_14987(C_word c,C_word *av) C_noret;
C_noret_decl(f_15022)
static void C_ccall f_15022(C_word c,C_word *av) C_noret;
C_noret_decl(f_15029)
static void C_ccall f_15029(C_word c,C_word *av) C_noret;
C_noret_decl(f_15041)
static void C_ccall f_15041(C_word c,C_word *av) C_noret;
C_noret_decl(f_15061)
static void C_ccall f_15061(C_word c,C_word *av) C_noret;
C_noret_decl(f_15065)
static void C_ccall f_15065(C_word c,C_word *av) C_noret;
C_noret_decl(f_15085)
static void C_fcall f_15085(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15105)
static void C_ccall f_15105(C_word c,C_word *av) C_noret;
C_noret_decl(f_15109)
static void C_ccall f_15109(C_word c,C_word *av) C_noret;
C_noret_decl(f_15113)
static void C_ccall f_15113(C_word c,C_word *av) C_noret;
C_noret_decl(f_15120)
static void C_ccall f_15120(C_word c,C_word *av) C_noret;
C_noret_decl(f_15136)
static void C_ccall f_15136(C_word c,C_word *av) C_noret;
C_noret_decl(f_15140)
static void C_ccall f_15140(C_word c,C_word *av) C_noret;
C_noret_decl(f_15181)
static void C_fcall f_15181(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15214)
static void C_ccall f_15214(C_word c,C_word *av) C_noret;
C_noret_decl(f_15243)
static void C_ccall f_15243(C_word c,C_word *av) C_noret;
C_noret_decl(f_15251)
static void C_ccall f_15251(C_word c,C_word *av) C_noret;
C_noret_decl(f_15257)
static void C_ccall f_15257(C_word c,C_word *av) C_noret;
C_noret_decl(f_15277)
static void C_fcall f_15277(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15302)
static void C_ccall f_15302(C_word c,C_word *av) C_noret;
C_noret_decl(f_15308)
static void C_ccall f_15308(C_word c,C_word *av) C_noret;
C_noret_decl(f_15312)
static void C_ccall f_15312(C_word c,C_word *av) C_noret;
C_noret_decl(f_15322)
static void C_ccall f_15322(C_word c,C_word *av) C_noret;
C_noret_decl(f_15342)
static void C_fcall f_15342(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15363)
static void C_ccall f_15363(C_word c,C_word *av) C_noret;
C_noret_decl(f_15380)
static void C_ccall f_15380(C_word c,C_word *av) C_noret;
C_noret_decl(f_15387)
static void C_ccall f_15387(C_word c,C_word *av) C_noret;
C_noret_decl(f_15392)
static void C_ccall f_15392(C_word c,C_word *av) C_noret;
C_noret_decl(f_15398)
static void C_ccall f_15398(C_word c,C_word *av) C_noret;
C_noret_decl(f_15402)
static void C_ccall f_15402(C_word c,C_word *av) C_noret;
C_noret_decl(f_15405)
static void C_ccall f_15405(C_word c,C_word *av) C_noret;
C_noret_decl(f_15412)
static void C_ccall f_15412(C_word c,C_word *av) C_noret;
C_noret_decl(f_15433)
static void C_ccall f_15433(C_word c,C_word *av) C_noret;
C_noret_decl(f_15447)
static void C_ccall f_15447(C_word c,C_word *av) C_noret;
C_noret_decl(f_15451)
static void C_ccall f_15451(C_word c,C_word *av) C_noret;
C_noret_decl(f_15488)
static void C_ccall f_15488(C_word c,C_word *av) C_noret;
C_noret_decl(f_15496)
static void C_ccall f_15496(C_word c,C_word *av) C_noret;
C_noret_decl(f_15525)
static void C_ccall f_15525(C_word c,C_word *av) C_noret;
C_noret_decl(f_15528)
static void C_ccall f_15528(C_word c,C_word *av) C_noret;
C_noret_decl(f_15543)
static void C_ccall f_15543(C_word c,C_word *av) C_noret;
C_noret_decl(f_15546)
static void C_fcall f_15546(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15553)
static void C_ccall f_15553(C_word c,C_word *av) C_noret;
C_noret_decl(f_15564)
static void C_ccall f_15564(C_word c,C_word *av) C_noret;
C_noret_decl(f_15582)
static void C_ccall f_15582(C_word c,C_word *av) C_noret;
C_noret_decl(f_15607)
static void C_fcall f_15607(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15622)
static void C_ccall f_15622(C_word c,C_word *av) C_noret;
C_noret_decl(f_15650)
static void C_fcall f_15650(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15654)
static C_word C_fcall f_15654(C_word t0,C_word t1);
C_noret_decl(f_15668)
static void C_fcall f_15668(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15728)
static void C_ccall f_15728(C_word c,C_word *av) C_noret;
C_noret_decl(f_15748)
static void C_fcall f_15748(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15754)
static void C_fcall f_15754(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15817)
static void C_fcall f_15817(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15856)
static void C_fcall f_15856(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15862)
static void C_fcall f_15862(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15906)
static void C_fcall f_15906(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_15910)
static void C_ccall f_15910(C_word c,C_word *av) C_noret;
C_noret_decl(f_15913)
static void C_ccall f_15913(C_word c,C_word *av) C_noret;
C_noret_decl(f_15962)
static void C_fcall f_15962(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15977)
static void C_fcall f_15977(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15986)
static void C_ccall f_15986(C_word c,C_word *av) C_noret;
C_noret_decl(f_16020)
static void C_fcall f_16020(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_16030)
static void C_ccall f_16030(C_word c,C_word *av) C_noret;
C_noret_decl(f_16039)
static void C_fcall f_16039(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16051)
static void C_fcall f_16051(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16060)
static void C_ccall f_16060(C_word c,C_word *av) C_noret;
C_noret_decl(f_16072)
static void C_ccall f_16072(C_word c,C_word *av) C_noret;
C_noret_decl(f_16078)
static void C_fcall f_16078(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16084)
static void C_ccall f_16084(C_word c,C_word *av) C_noret;
C_noret_decl(f_16126)
static void C_ccall f_16126(C_word c,C_word *av) C_noret;
C_noret_decl(f_16234)
static void C_fcall f_16234(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16241)
static void C_fcall f_16241(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16254)
static void C_ccall f_16254(C_word c,C_word *av) C_noret;
C_noret_decl(f_16312)
static void C_fcall f_16312(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16330)
static void C_ccall f_16330(C_word c,C_word *av) C_noret;
C_noret_decl(f_16340)
static void C_ccall f_16340(C_word c,C_word *av) C_noret;
C_noret_decl(f_16376)
static void C_ccall f_16376(C_word c,C_word *av) C_noret;
C_noret_decl(f_16402)
static void C_ccall f_16402(C_word c,C_word *av) C_noret;
C_noret_decl(f_16413)
static void C_ccall f_16413(C_word c,C_word *av) C_noret;
C_noret_decl(f_16580)
static void C_fcall f_16580(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16595)
static void C_fcall f_16595(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16653)
static void C_fcall f_16653(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16694)
static void C_ccall f_16694(C_word c,C_word *av) C_noret;
C_noret_decl(f_16772)
static void C_ccall f_16772(C_word c,C_word *av) C_noret;
C_noret_decl(f_16820)
static void C_ccall f_16820(C_word c,C_word *av) C_noret;
C_noret_decl(f_16823)
static void C_ccall f_16823(C_word c,C_word *av) C_noret;
C_noret_decl(f_16835)
static void C_ccall f_16835(C_word c,C_word *av) C_noret;
C_noret_decl(f_16922)
static void C_ccall f_16922(C_word c,C_word *av) C_noret;
C_noret_decl(f_16990)
static void C_ccall f_16990(C_word c,C_word *av) C_noret;
C_noret_decl(f_17070)
static void C_ccall f_17070(C_word c,C_word *av) C_noret;
C_noret_decl(f_17080)
static void C_ccall f_17080(C_word c,C_word *av) C_noret;
C_noret_decl(f_17089)
static void C_fcall f_17089(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_17325)
static void C_ccall f_17325(C_word c,C_word *av) C_noret;
C_noret_decl(f_17371)
static void C_ccall f_17371(C_word c,C_word *av) C_noret;
C_noret_decl(f_17374)
static void C_fcall f_17374(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_17401)
static void C_fcall f_17401(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17425)
static void C_ccall f_17425(C_word c,C_word *av) C_noret;
C_noret_decl(f_17442)
static void C_fcall f_17442(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17543)
static void C_ccall f_17543(C_word c,C_word *av) C_noret;
C_noret_decl(f_17546)
static void C_ccall f_17546(C_word c,C_word *av) C_noret;
C_noret_decl(f_17555)
static void C_ccall f_17555(C_word c,C_word *av) C_noret;
C_noret_decl(f_17561)
static void C_ccall f_17561(C_word c,C_word *av) C_noret;
C_noret_decl(f_17572)
static void C_ccall f_17572(C_word c,C_word *av) C_noret;
C_noret_decl(f_17574)
static void C_ccall f_17574(C_word c,C_word *av) C_noret;
C_noret_decl(f_17585)
static void C_ccall f_17585(C_word c,C_word *av) C_noret;
C_noret_decl(f_17587)
static void C_ccall f_17587(C_word c,C_word *av) C_noret;
C_noret_decl(f_17598)
static void C_ccall f_17598(C_word c,C_word *av) C_noret;
C_noret_decl(f_17600)
static void C_ccall f_17600(C_word c,C_word *av) C_noret;
C_noret_decl(f_17617)
static void C_ccall f_17617(C_word c,C_word *av) C_noret;
C_noret_decl(f_17621)
static void C_ccall f_17621(C_word c,C_word *av) C_noret;
C_noret_decl(f_17625)
static void C_ccall f_17625(C_word c,C_word *av) C_noret;
C_noret_decl(f_17634)
static void C_ccall f_17634(C_word c,C_word *av) C_noret;
C_noret_decl(f_17654)
static void C_ccall f_17654(C_word c,C_word *av) C_noret;
C_noret_decl(f_17662)
static void C_ccall f_17662(C_word c,C_word *av) C_noret;
C_noret_decl(f_17677)
static void C_ccall f_17677(C_word c,C_word *av) C_noret;
C_noret_decl(f_17679)
static void C_fcall f_17679(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17704)
static void C_ccall f_17704(C_word c,C_word *av) C_noret;
C_noret_decl(f_17713)
static void C_ccall f_17713(C_word c,C_word *av) C_noret;
C_noret_decl(f_17716)
static void C_ccall f_17716(C_word c,C_word *av) C_noret;
C_noret_decl(f_17725)
static void C_ccall f_17725(C_word c,C_word *av) C_noret;
C_noret_decl(f_17732)
static void C_ccall f_17732(C_word c,C_word *av) C_noret;
C_noret_decl(f_17743)
static void C_ccall f_17743(C_word c,C_word *av) C_noret;
C_noret_decl(f_17750)
static void C_ccall f_17750(C_word c,C_word *av) C_noret;
C_noret_decl(f_17774)
static void C_ccall f_17774(C_word c,C_word *av) C_noret;
C_noret_decl(f_17778)
static void C_ccall f_17778(C_word c,C_word *av) C_noret;
C_noret_decl(f_17780)
static void C_ccall f_17780(C_word c,C_word *av) C_noret;
C_noret_decl(f_17784)
static void C_ccall f_17784(C_word c,C_word *av) C_noret;
C_noret_decl(f_17789)
static void C_ccall f_17789(C_word c,C_word *av) C_noret;
C_noret_decl(f_17795)
static void C_ccall f_17795(C_word c,C_word *av) C_noret;
C_noret_decl(f_17799)
static void C_ccall f_17799(C_word c,C_word *av) C_noret;
C_noret_decl(f_17804)
static void C_ccall f_17804(C_word c,C_word *av) C_noret;
C_noret_decl(f_17814)
static void C_ccall f_17814(C_word c,C_word *av) C_noret;
C_noret_decl(f_17816)
static void C_ccall f_17816(C_word c,C_word *av) C_noret;
C_noret_decl(f_17820)
static void C_ccall f_17820(C_word c,C_word *av) C_noret;
C_noret_decl(f_17826)
static void C_ccall f_17826(C_word c,C_word *av) C_noret;
C_noret_decl(f_17828)
static void C_ccall f_17828(C_word c,C_word *av) C_noret;
C_noret_decl(f_17832)
static void C_ccall f_17832(C_word c,C_word *av) C_noret;
C_noret_decl(f_17835)
static void C_ccall f_17835(C_word c,C_word *av) C_noret;
C_noret_decl(f_17851)
static void C_ccall f_17851(C_word c,C_word *av) C_noret;
C_noret_decl(f_17855)
static void C_ccall f_17855(C_word c,C_word *av) C_noret;
C_noret_decl(f_17858)
static void C_ccall f_17858(C_word c,C_word *av) C_noret;
C_noret_decl(f_17880)
static void C_ccall f_17880(C_word c,C_word *av) C_noret;
C_noret_decl(f_17896)
static void C_ccall f_17896(C_word c,C_word *av) C_noret;
C_noret_decl(f_17901)
static C_word C_fcall f_17901(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_17932)
static void C_ccall f_17932(C_word c,C_word *av) C_noret;
C_noret_decl(f_17941)
static void C_fcall f_17941(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17959)
static void C_ccall f_17959(C_word c,C_word *av) C_noret;
C_noret_decl(f_17965)
static void C_ccall f_17965(C_word c,C_word *av) C_noret;
C_noret_decl(f_17971)
static void C_ccall f_17971(C_word c,C_word *av) C_noret;
C_noret_decl(f_17980)
static C_word C_fcall f_17980(C_word t0,C_word t1);
C_noret_decl(f_17999)
static void C_ccall f_17999(C_word c,C_word *av) C_noret;
C_noret_decl(f_18018)
static void C_ccall f_18018(C_word c,C_word *av) C_noret;
C_noret_decl(f_18021)
static void C_ccall f_18021(C_word c,C_word *av) C_noret;
C_noret_decl(f_18026)
static C_word C_fcall f_18026(C_word t0,C_word t1);
C_noret_decl(f_18068)
static void C_ccall f_18068(C_word c,C_word *av) C_noret;
C_noret_decl(f_18087)
static void C_ccall f_18087(C_word c,C_word *av) C_noret;
C_noret_decl(f_18090)
static void C_ccall f_18090(C_word c,C_word *av) C_noret;
C_noret_decl(f_18093)
static void C_ccall f_18093(C_word c,C_word *av) C_noret;
C_noret_decl(f_18098)
static C_word C_fcall f_18098(C_word t0,C_word t1);
C_noret_decl(f_18139)
static void C_ccall f_18139(C_word c,C_word *av) C_noret;
C_noret_decl(f_18149)
static void C_ccall f_18149(C_word c,C_word *av) C_noret;
C_noret_decl(f_18160)
static void C_fcall f_18160(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18164)
static void C_ccall f_18164(C_word c,C_word *av) C_noret;
C_noret_decl(f_18167)
static void C_ccall f_18167(C_word c,C_word *av) C_noret;
C_noret_decl(f_18172)
static C_word C_fcall f_18172(C_word t0,C_word t1);
C_noret_decl(f_18199)
static void C_ccall f_18199(C_word c,C_word *av) C_noret;
C_noret_decl(f_18213)
static void C_ccall f_18213(C_word c,C_word *av) C_noret;
C_noret_decl(f_18224)
static void C_ccall f_18224(C_word c,C_word *av) C_noret;
C_noret_decl(f_18235)
static void C_ccall f_18235(C_word c,C_word *av) C_noret;
C_noret_decl(f_18246)
static void C_ccall f_18246(C_word c,C_word *av) C_noret;
C_noret_decl(f_18259)
static void C_ccall f_18259(C_word c,C_word *av) C_noret;
C_noret_decl(f_18261)
static void C_fcall f_18261(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18274)
static C_word C_fcall f_18274(C_word t0,C_word t1);
C_noret_decl(f_18302)
static void C_ccall f_18302(C_word c,C_word *av) C_noret;
C_noret_decl(f_18315)
static void C_ccall f_18315(C_word c,C_word *av) C_noret;
C_noret_decl(f_18333)
static void C_ccall f_18333(C_word c,C_word *av) C_noret;
C_noret_decl(f_18336)
static void C_ccall f_18336(C_word c,C_word *av) C_noret;
C_noret_decl(f_18419)
static void C_ccall f_18419(C_word c,C_word *av) C_noret;
C_noret_decl(f_18422)
static void C_ccall f_18422(C_word c,C_word *av) C_noret;
C_noret_decl(f_18425)
static void C_ccall f_18425(C_word c,C_word *av) C_noret;
C_noret_decl(f_18428)
static void C_ccall f_18428(C_word c,C_word *av) C_noret;
C_noret_decl(f_18431)
static void C_ccall f_18431(C_word c,C_word *av) C_noret;
C_noret_decl(f_18434)
static void C_ccall f_18434(C_word c,C_word *av) C_noret;
C_noret_decl(f_18437)
static void C_ccall f_18437(C_word c,C_word *av) C_noret;
C_noret_decl(f_18440)
static void C_ccall f_18440(C_word c,C_word *av) C_noret;
C_noret_decl(f_18443)
static void C_ccall f_18443(C_word c,C_word *av) C_noret;
C_noret_decl(f_18446)
static void C_ccall f_18446(C_word c,C_word *av) C_noret;
C_noret_decl(f_18449)
static void C_ccall f_18449(C_word c,C_word *av) C_noret;
C_noret_decl(f_18452)
static void C_ccall f_18452(C_word c,C_word *av) C_noret;
C_noret_decl(f_18455)
static void C_ccall f_18455(C_word c,C_word *av) C_noret;
C_noret_decl(f_18458)
static void C_ccall f_18458(C_word c,C_word *av) C_noret;
C_noret_decl(f_18463)
static void C_ccall f_18463(C_word c,C_word *av) C_noret;
C_noret_decl(f_18469)
static void C_fcall f_18469(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18485)
static void C_ccall f_18485(C_word c,C_word *av) C_noret;
C_noret_decl(f_18501)
static void C_ccall f_18501(C_word c,C_word *av) C_noret;
C_noret_decl(f_18507)
static void C_fcall f_18507(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18527)
static void C_ccall f_18527(C_word c,C_word *av) C_noret;
C_noret_decl(f_18531)
static void C_ccall f_18531(C_word c,C_word *av) C_noret;
C_noret_decl(f_18543)
static void C_ccall f_18543(C_word c,C_word *av) C_noret;
C_noret_decl(f_18549)
static void C_ccall f_18549(C_word c,C_word *av) C_noret;
C_noret_decl(f_18555)
static void C_fcall f_18555(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18587)
static void C_ccall f_18587(C_word c,C_word *av) C_noret;
C_noret_decl(f_18591)
static void C_ccall f_18591(C_word c,C_word *av) C_noret;
C_noret_decl(f_18601)
static void C_ccall f_18601(C_word c,C_word *av) C_noret;
C_noret_decl(f_18616)
static void C_fcall f_18616(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18626)
static void C_ccall f_18626(C_word c,C_word *av) C_noret;
C_noret_decl(f_18646)
static void C_fcall f_18646(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18653)
static void C_ccall f_18653(C_word c,C_word *av) C_noret;
C_noret_decl(f_18662)
static void C_ccall f_18662(C_word c,C_word *av) C_noret;
C_noret_decl(f_18669)
static void C_ccall f_18669(C_word c,C_word *av) C_noret;
C_noret_decl(f_18671)
static void C_ccall f_18671(C_word c,C_word *av) C_noret;
C_noret_decl(f_18677)
static void C_ccall f_18677(C_word c,C_word *av) C_noret;
C_noret_decl(f_18683)
static void C_ccall f_18683(C_word c,C_word *av) C_noret;
C_noret_decl(f_18701)
static void C_fcall f_18701(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18726)
static void C_ccall f_18726(C_word c,C_word *av) C_noret;
C_noret_decl(f_18742)
static void C_fcall f_18742(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18749)
static void C_ccall f_18749(C_word c,C_word *av) C_noret;
C_noret_decl(f_18762)
static void C_ccall f_18762(C_word c,C_word *av) C_noret;
C_noret_decl(f_18766)
static void C_ccall f_18766(C_word c,C_word *av) C_noret;
C_noret_decl(f_18770)
static void C_ccall f_18770(C_word c,C_word *av) C_noret;
C_noret_decl(f_18772)
static void C_ccall f_18772(C_word c,C_word *av) C_noret;
C_noret_decl(f_18778)
static void C_ccall f_18778(C_word c,C_word *av) C_noret;
C_noret_decl(f_18785)
static void C_ccall f_18785(C_word c,C_word *av) C_noret;
C_noret_decl(f_18789)
static void C_ccall f_18789(C_word c,C_word *av) C_noret;
C_noret_decl(f_18798)
static void C_ccall f_18798(C_word c,C_word *av) C_noret;
C_noret_decl(f_18806)
static void C_ccall f_18806(C_word c,C_word *av) C_noret;
C_noret_decl(f_18816)
static void C_ccall f_18816(C_word c,C_word *av) C_noret;
C_noret_decl(f_18822)
static void C_ccall f_18822(C_word c,C_word *av) C_noret;
C_noret_decl(f_18825)
static void C_ccall f_18825(C_word c,C_word *av) C_noret;
C_noret_decl(f_18829)
static void C_ccall f_18829(C_word c,C_word *av) C_noret;
C_noret_decl(f_18856)
static void C_fcall f_18856(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18872)
static void C_ccall f_18872(C_word c,C_word *av) C_noret;
C_noret_decl(f_18878)
static void C_ccall f_18878(C_word c,C_word *av) C_noret;
C_noret_decl(f_18901)
static void C_ccall f_18901(C_word c,C_word *av) C_noret;
C_noret_decl(f_18914)
static void C_ccall f_18914(C_word c,C_word *av) C_noret;
C_noret_decl(f_18920)
static void C_ccall f_18920(C_word c,C_word *av) C_noret;
C_noret_decl(f_18925)
static void C_ccall f_18925(C_word c,C_word *av) C_noret;
C_noret_decl(f_18930)
static void C_ccall f_18930(C_word c,C_word *av) C_noret;
C_noret_decl(f_18934)
static void C_ccall f_18934(C_word c,C_word *av) C_noret;
C_noret_decl(f_18943)
static void C_ccall f_18943(C_word c,C_word *av) C_noret;
C_noret_decl(f_18974)
static void C_ccall f_18974(C_word c,C_word *av) C_noret;
C_noret_decl(f_18978)
static void C_ccall f_18978(C_word c,C_word *av) C_noret;
C_noret_decl(f_18982)
static void C_ccall f_18982(C_word c,C_word *av) C_noret;
C_noret_decl(f_18985)
static void C_ccall f_18985(C_word c,C_word *av) C_noret;
C_noret_decl(f_18988)
static void C_ccall f_18988(C_word c,C_word *av) C_noret;
C_noret_decl(f_18991)
static void C_ccall f_18991(C_word c,C_word *av) C_noret;
C_noret_decl(f_18993)
static void C_ccall f_18993(C_word c,C_word *av) C_noret;
C_noret_decl(f_19004)
static void C_ccall f_19004(C_word c,C_word *av) C_noret;
C_noret_decl(f_19015)
static void C_ccall f_19015(C_word c,C_word *av) C_noret;
C_noret_decl(f_19026)
static void C_ccall f_19026(C_word c,C_word *av) C_noret;
C_noret_decl(f_19037)
static void C_ccall f_19037(C_word c,C_word *av) C_noret;
C_noret_decl(f_19050)
static void C_ccall f_19050(C_word c,C_word *av) C_noret;
C_noret_decl(f_19052)
static C_word C_fcall f_19052(C_word t0,C_word t1);
C_noret_decl(f_19058)
static C_word C_fcall f_19058(C_word t0);
C_noret_decl(f_19063)
static C_word C_fcall f_19063(C_word t0);
C_noret_decl(f_19098)
static void C_ccall f_19098(C_word c,C_word *av) C_noret;
C_noret_decl(f_19111)
static void C_ccall f_19111(C_word c,C_word *av) C_noret;
C_noret_decl(f_19113)
static C_word C_fcall f_19113(C_word t0,C_word t1);
C_noret_decl(f_19119)
static C_word C_fcall f_19119(C_word t0);
C_noret_decl(f_19124)
static C_word C_fcall f_19124(C_word t0);
C_noret_decl(f_19159)
static void C_ccall f_19159(C_word c,C_word *av) C_noret;
C_noret_decl(f_19172)
static void C_ccall f_19172(C_word c,C_word *av) C_noret;
C_noret_decl(f_19174)
static C_word C_fcall f_19174(C_word t0,C_word t1);
C_noret_decl(f_19180)
static C_word C_fcall f_19180(C_word t0);
C_noret_decl(f_19185)
static C_word C_fcall f_19185(C_word t0);
C_noret_decl(f_19220)
static void C_ccall f_19220(C_word c,C_word *av) C_noret;
C_noret_decl(f_19232)
static void C_ccall f_19232(C_word c,C_word *av) C_noret;
C_noret_decl(f_19234)
static void C_ccall f_19234(C_word c,C_word *av) C_noret;
C_noret_decl(f_19240)
static void C_ccall f_19240(C_word c,C_word *av) C_noret;
C_noret_decl(f_19246)
static void C_fcall f_19246(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_19256)
static void C_ccall f_19256(C_word c,C_word *av) C_noret;
C_noret_decl(f_19259)
static void C_ccall f_19259(C_word c,C_word *av) C_noret;
C_noret_decl(f_19262)
static void C_ccall f_19262(C_word c,C_word *av) C_noret;
C_noret_decl(f_19265)
static void C_ccall f_19265(C_word c,C_word *av) C_noret;
C_noret_decl(f_19268)
static void C_ccall f_19268(C_word c,C_word *av) C_noret;
C_noret_decl(f_19275)
static void C_ccall f_19275(C_word c,C_word *av) C_noret;
C_noret_decl(f_19279)
static void C_ccall f_19279(C_word c,C_word *av) C_noret;
C_noret_decl(f_19283)
static void C_ccall f_19283(C_word c,C_word *av) C_noret;
C_noret_decl(f_19289)
static void C_fcall f_19289(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19302)
static void C_ccall f_19302(C_word c,C_word *av) C_noret;
C_noret_decl(f_19344)
static void C_fcall f_19344(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19348)
static void C_ccall f_19348(C_word c,C_word *av) C_noret;
C_noret_decl(f_19381)
static void C_ccall f_19381(C_word c,C_word *av) C_noret;
C_noret_decl(f_19387)
static void C_ccall f_19387(C_word c,C_word *av) C_noret;
C_noret_decl(f_19393)
static void C_ccall f_19393(C_word c,C_word *av) C_noret;
C_noret_decl(f_19399)
static void C_ccall f_19399(C_word c,C_word *av) C_noret;
C_noret_decl(f_19405)
static void C_ccall f_19405(C_word c,C_word *av) C_noret;
C_noret_decl(f_19409)
static void C_ccall f_19409(C_word c,C_word *av) C_noret;
C_noret_decl(f_19414)
static void C_ccall f_19414(C_word c,C_word *av) C_noret;
C_noret_decl(f_19420)
static void C_ccall f_19420(C_word c,C_word *av) C_noret;
C_noret_decl(f_19424)
static void C_ccall f_19424(C_word c,C_word *av) C_noret;
C_noret_decl(f_19429)
static void C_ccall f_19429(C_word c,C_word *av) C_noret;
C_noret_decl(f_19433)
static void C_ccall f_19433(C_word c,C_word *av) C_noret;
C_noret_decl(f_19438)
static void C_ccall f_19438(C_word c,C_word *av) C_noret;
C_noret_decl(f_19444)
static void C_ccall f_19444(C_word c,C_word *av) C_noret;
C_noret_decl(f_19448)
static void C_ccall f_19448(C_word c,C_word *av) C_noret;
C_noret_decl(f_19453)
static void C_ccall f_19453(C_word c,C_word *av) C_noret;
C_noret_decl(f_19457)
static void C_ccall f_19457(C_word c,C_word *av) C_noret;
C_noret_decl(f_19462)
static void C_ccall f_19462(C_word c,C_word *av) C_noret;
C_noret_decl(f_19467)
static void C_ccall f_19467(C_word c,C_word *av) C_noret;
C_noret_decl(f_19473)
static void C_ccall f_19473(C_word c,C_word *av) C_noret;
C_noret_decl(f_19477)
static void C_ccall f_19477(C_word c,C_word *av) C_noret;
C_noret_decl(f_19482)
static void C_ccall f_19482(C_word c,C_word *av) C_noret;
C_noret_decl(f_19487)
static void C_ccall f_19487(C_word c,C_word *av) C_noret;
C_noret_decl(f_19491)
static void C_ccall f_19491(C_word c,C_word *av) C_noret;
C_noret_decl(f_19496)
static void C_ccall f_19496(C_word c,C_word *av) C_noret;
C_noret_decl(f_19501)
static void C_ccall f_19501(C_word c,C_word *av) C_noret;
C_noret_decl(f_19507)
static void C_ccall f_19507(C_word c,C_word *av) C_noret;
C_noret_decl(f_19511)
static void C_ccall f_19511(C_word c,C_word *av) C_noret;
C_noret_decl(f_19516)
static void C_ccall f_19516(C_word c,C_word *av) C_noret;
C_noret_decl(f_19521)
static void C_ccall f_19521(C_word c,C_word *av) C_noret;
C_noret_decl(f_19526)
static void C_ccall f_19526(C_word c,C_word *av) C_noret;
C_noret_decl(f_19547)
static void C_ccall f_19547(C_word c,C_word *av) C_noret;
C_noret_decl(f_19554)
static void C_ccall f_19554(C_word c,C_word *av) C_noret;
C_noret_decl(f_19560)
static void C_ccall f_19560(C_word c,C_word *av) C_noret;
C_noret_decl(f_19578)
static void C_ccall f_19578(C_word c,C_word *av) C_noret;
C_noret_decl(f_19587)
static void C_ccall f_19587(C_word c,C_word *av) C_noret;
C_noret_decl(f_19600)
static void C_fcall f_19600(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19608)
static void C_ccall f_19608(C_word c,C_word *av) C_noret;
C_noret_decl(f_19613)
static void C_fcall f_19613(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19650)
static void C_ccall f_19650(C_word c,C_word *av) C_noret;
C_noret_decl(f_19662)
static void C_ccall f_19662(C_word c,C_word *av) C_noret;
C_noret_decl(f_19672)
static void C_fcall f_19672(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19688)
static void C_ccall f_19688(C_word c,C_word *av) C_noret;
C_noret_decl(f_19701)
static void C_ccall f_19701(C_word c,C_word *av) C_noret;
C_noret_decl(f_19708)
static void C_ccall f_19708(C_word c,C_word *av) C_noret;
C_noret_decl(f_19710)
static void C_ccall f_19710(C_word c,C_word *av) C_noret;
C_noret_decl(f_19723)
static void C_ccall f_19723(C_word c,C_word *av) C_noret;
C_noret_decl(f_19729)
static void C_ccall f_19729(C_word c,C_word *av) C_noret;
C_noret_decl(f_19736)
static void C_ccall f_19736(C_word c,C_word *av) C_noret;
C_noret_decl(f_19738)
static void C_ccall f_19738(C_word c,C_word *av) C_noret;
C_noret_decl(f_19742)
static void C_ccall f_19742(C_word c,C_word *av) C_noret;
C_noret_decl(f_19745)
static void C_ccall f_19745(C_word c,C_word *av) C_noret;
C_noret_decl(f_19755)
static void C_ccall f_19755(C_word c,C_word *av) C_noret;
C_noret_decl(f_19759)
static void C_ccall f_19759(C_word c,C_word *av) C_noret;
C_noret_decl(f_19767)
static void C_ccall f_19767(C_word c,C_word *av) C_noret;
C_noret_decl(f_19770)
static void C_ccall f_19770(C_word c,C_word *av) C_noret;
C_noret_decl(f_19777)
static void C_ccall f_19777(C_word c,C_word *av) C_noret;
C_noret_decl(f_19780)
static void C_ccall f_19780(C_word c,C_word *av) C_noret;
C_noret_decl(f_19783)
static void C_ccall f_19783(C_word c,C_word *av) C_noret;
C_noret_decl(f_19786)
static void C_ccall f_19786(C_word c,C_word *av) C_noret;
C_noret_decl(f_19794)
static void C_ccall f_19794(C_word c,C_word *av) C_noret;
C_noret_decl(f_19802)
static void C_ccall f_19802(C_word c,C_word *av) C_noret;
C_noret_decl(f_19804)
static void C_ccall f_19804(C_word c,C_word *av) C_noret;
C_noret_decl(f_19820)
static void C_ccall f_19820(C_word c,C_word *av) C_noret;
C_noret_decl(f_19847)
static void C_ccall f_19847(C_word c,C_word *av) C_noret;
C_noret_decl(f_19851)
static void C_ccall f_19851(C_word c,C_word *av) C_noret;
C_noret_decl(f_19855)
static void C_ccall f_19855(C_word c,C_word *av) C_noret;
C_noret_decl(f_19859)
static void C_ccall f_19859(C_word c,C_word *av) C_noret;
C_noret_decl(f_19863)
static void C_ccall f_19863(C_word c,C_word *av) C_noret;
C_noret_decl(f_19867)
static void C_ccall f_19867(C_word c,C_word *av) C_noret;
C_noret_decl(f_19871)
static void C_ccall f_19871(C_word c,C_word *av) C_noret;
C_noret_decl(f_19875)
static void C_ccall f_19875(C_word c,C_word *av) C_noret;
C_noret_decl(f_19879)
static void C_ccall f_19879(C_word c,C_word *av) C_noret;
C_noret_decl(f_19883)
static void C_ccall f_19883(C_word c,C_word *av) C_noret;
C_noret_decl(f_19887)
static void C_ccall f_19887(C_word c,C_word *av) C_noret;
C_noret_decl(f_19891)
static void C_ccall f_19891(C_word c,C_word *av) C_noret;
C_noret_decl(f_19895)
static void C_ccall f_19895(C_word c,C_word *av) C_noret;
C_noret_decl(f_19899)
static void C_ccall f_19899(C_word c,C_word *av) C_noret;
C_noret_decl(f_19903)
static void C_ccall f_19903(C_word c,C_word *av) C_noret;
C_noret_decl(f_19907)
static void C_ccall f_19907(C_word c,C_word *av) C_noret;
C_noret_decl(f_19911)
static void C_ccall f_19911(C_word c,C_word *av) C_noret;
C_noret_decl(f_19918)
static void C_ccall f_19918(C_word c,C_word *av) C_noret;
C_noret_decl(f_19925)
static void C_ccall f_19925(C_word c,C_word *av) C_noret;
C_noret_decl(f_19926)
static void C_fcall f_19926(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_19930)
static void C_fcall f_19930(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19933)
static void C_ccall f_19933(C_word c,C_word *av) C_noret;
C_noret_decl(f_19949)
static void C_ccall f_19949(C_word c,C_word *av) C_noret;
C_noret_decl(f_19960)
static void C_fcall f_19960(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19968)
static void C_ccall f_19968(C_word c,C_word *av) C_noret;
C_noret_decl(f_19980)
static void C_fcall f_19980(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19989)
static void C_fcall f_19989(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19994)
static void C_fcall f_19994(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20050)
static void C_ccall f_20050(C_word c,C_word *av) C_noret;
C_noret_decl(f_20064)
static void C_ccall f_20064(C_word c,C_word *av) C_noret;
C_noret_decl(f_20076)
static void C_ccall f_20076(C_word c,C_word *av) C_noret;
C_noret_decl(f_20082)
static void C_ccall f_20082(C_word c,C_word *av) C_noret;
C_noret_decl(f_20107)
static void C_ccall f_20107(C_word c,C_word *av) C_noret;
C_noret_decl(f_20125)
static void C_ccall f_20125(C_word c,C_word *av) C_noret;
C_noret_decl(f_20129)
static void C_ccall f_20129(C_word c,C_word *av) C_noret;
C_noret_decl(f_20185)
static void C_ccall f_20185(C_word c,C_word *av) C_noret;
C_noret_decl(f_20194)
static void C_ccall f_20194(C_word c,C_word *av) C_noret;
C_noret_decl(f_20207)
static void C_ccall f_20207(C_word c,C_word *av) C_noret;
C_noret_decl(f_20219)
static void C_ccall f_20219(C_word c,C_word *av) C_noret;
C_noret_decl(f_20237)
static void C_ccall f_20237(C_word c,C_word *av) C_noret;
C_noret_decl(f_20257)
static void C_ccall f_20257(C_word c,C_word *av) C_noret;
C_noret_decl(f_20260)
static void C_ccall f_20260(C_word c,C_word *av) C_noret;
C_noret_decl(f_20263)
static void C_ccall f_20263(C_word c,C_word *av) C_noret;
C_noret_decl(f_20271)
static void C_ccall f_20271(C_word c,C_word *av) C_noret;
C_noret_decl(f_20275)
static void C_ccall f_20275(C_word c,C_word *av) C_noret;
C_noret_decl(f_20279)
static void C_ccall f_20279(C_word c,C_word *av) C_noret;
C_noret_decl(f_20283)
static void C_ccall f_20283(C_word c,C_word *av) C_noret;
C_noret_decl(f_20287)
static void C_ccall f_20287(C_word c,C_word *av) C_noret;
C_noret_decl(f_20289)
static void C_ccall f_20289(C_word c,C_word *av) C_noret;
C_noret_decl(f_20297)
static void C_ccall f_20297(C_word c,C_word *av) C_noret;
C_noret_decl(f_20300)
static void C_ccall f_20300(C_word c,C_word *av) C_noret;
C_noret_decl(f_20310)
static void C_ccall f_20310(C_word c,C_word *av) C_noret;
C_noret_decl(f_20312)
static void C_ccall f_20312(C_word c,C_word *av) C_noret;
C_noret_decl(f_20320)
static void C_ccall f_20320(C_word c,C_word *av) C_noret;
C_noret_decl(f_20323)
static void C_ccall f_20323(C_word c,C_word *av) C_noret;
C_noret_decl(f_20333)
static void C_ccall f_20333(C_word c,C_word *av) C_noret;
C_noret_decl(f_20335)
static void C_ccall f_20335(C_word c,C_word *av) C_noret;
C_noret_decl(f_20339)
static void C_ccall f_20339(C_word c,C_word *av) C_noret;
C_noret_decl(f_20342)
static void C_ccall f_20342(C_word c,C_word *av) C_noret;
C_noret_decl(f_20345)
static void C_ccall f_20345(C_word c,C_word *av) C_noret;
C_noret_decl(f_20348)
static void C_ccall f_20348(C_word c,C_word *av) C_noret;
C_noret_decl(f_20351)
static void C_ccall f_20351(C_word c,C_word *av) C_noret;
C_noret_decl(f_20353)
static void C_ccall f_20353(C_word c,C_word *av) C_noret;
C_noret_decl(f_20359)
static void C_fcall f_20359(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_20368)
static void C_fcall f_20368(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20376)
static void C_ccall f_20376(C_word c,C_word *av) C_noret;
C_noret_decl(f_20378)
static void C_fcall f_20378(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20392)
static void C_ccall f_20392(C_word c,C_word *av) C_noret;
C_noret_decl(f_20405)
static void C_fcall f_20405(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20409)
static void C_ccall f_20409(C_word c,C_word *av) C_noret;
C_noret_decl(f_20414)
static void C_fcall f_20414(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20418)
static void C_ccall f_20418(C_word c,C_word *av) C_noret;
C_noret_decl(f_20429)
static void C_fcall f_20429(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20432)
static void C_fcall f_20432(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20440)
static void C_ccall f_20440(C_word c,C_word *av) C_noret;
C_noret_decl(f_20442)
static void C_fcall f_20442(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20455)
static void C_ccall f_20455(C_word c,C_word *av) C_noret;
C_noret_decl(f_20462)
static void C_ccall f_20462(C_word c,C_word *av) C_noret;
C_noret_decl(f_20471)
static void C_ccall f_20471(C_word c,C_word *av) C_noret;
C_noret_decl(f_20478)
static void C_ccall f_20478(C_word c,C_word *av) C_noret;
C_noret_decl(f_20480)
static void C_fcall f_20480(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_20486)
static void C_fcall f_20486(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20496)
static void C_ccall f_20496(C_word c,C_word *av) C_noret;
C_noret_decl(f_20499)
static void C_ccall f_20499(C_word c,C_word *av) C_noret;
C_noret_decl(f_20509)
static void C_ccall f_20509(C_word c,C_word *av) C_noret;
C_noret_decl(f_20512)
static void C_ccall f_20512(C_word c,C_word *av) C_noret;
C_noret_decl(f_20540)
static void C_fcall f_20540(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20544)
static void C_ccall f_20544(C_word c,C_word *av) C_noret;
C_noret_decl(f_20552)
static void C_fcall f_20552(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20577)
static void C_fcall f_20577(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20585)
static void C_ccall f_20585(C_word c,C_word *av) C_noret;
C_noret_decl(f_20587)
static void C_fcall f_20587(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20604)
static void C_ccall f_20604(C_word c,C_word *av) C_noret;
C_noret_decl(f_20617)
static void C_ccall f_20617(C_word c,C_word *av) C_noret;
C_noret_decl(f_20634)
static void C_ccall f_20634(C_word c,C_word *av) C_noret;
C_noret_decl(f_20651)
static void C_ccall f_20651(C_word c,C_word *av) C_noret;
C_noret_decl(f_20668)
static void C_ccall f_20668(C_word c,C_word *av) C_noret;
C_noret_decl(f_20685)
static void C_ccall f_20685(C_word c,C_word *av) C_noret;
C_noret_decl(f_20702)
static void C_ccall f_20702(C_word c,C_word *av) C_noret;
C_noret_decl(f_20719)
static void C_ccall f_20719(C_word c,C_word *av) C_noret;
C_noret_decl(f_20739)
static void C_ccall f_20739(C_word c,C_word *av) C_noret;
C_noret_decl(f_20747)
static void C_ccall f_20747(C_word c,C_word *av) C_noret;
C_noret_decl(f_20756)
static void C_ccall f_20756(C_word c,C_word *av) C_noret;
C_noret_decl(f_20762)
static void C_ccall f_20762(C_word c,C_word *av) C_noret;
C_noret_decl(f_20768)
static void C_fcall f_20768(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20771)
static void C_ccall f_20771(C_word c,C_word *av) C_noret;
C_noret_decl(f_20774)
static void C_ccall f_20774(C_word c,C_word *av) C_noret;
C_noret_decl(f_20784)
static void C_ccall f_20784(C_word c,C_word *av) C_noret;
C_noret_decl(f_20788)
static void C_ccall f_20788(C_word c,C_word *av) C_noret;
C_noret_decl(f_20805)
static void C_ccall f_20805(C_word c,C_word *av) C_noret;
C_noret_decl(f_20810)
static void C_ccall f_20810(C_word c,C_word *av) C_noret;
C_noret_decl(f_20817)
static void C_ccall f_20817(C_word c,C_word *av) C_noret;
C_noret_decl(f_20821)
static void C_ccall f_20821(C_word c,C_word *av) C_noret;
C_noret_decl(f_20830)
static void C_ccall f_20830(C_word c,C_word *av) C_noret;
C_noret_decl(f_20836)
static void C_ccall f_20836(C_word c,C_word *av) C_noret;
C_noret_decl(f_20846)
static void C_ccall f_20846(C_word c,C_word *av) C_noret;
C_noret_decl(f_20850)
static void C_ccall f_20850(C_word c,C_word *av) C_noret;
C_noret_decl(f_20859)
static void C_fcall f_20859(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20866)
static void C_ccall f_20866(C_word c,C_word *av) C_noret;
C_noret_decl(f_20879)
static void C_fcall f_20879(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20884)
static void C_fcall f_20884(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20901)
static void C_ccall f_20901(C_word c,C_word *av) C_noret;
C_noret_decl(f_20916)
static void C_ccall f_20916(C_word c,C_word *av) C_noret;
C_noret_decl(f_20929)
static void C_ccall f_20929(C_word c,C_word *av) C_noret;
C_noret_decl(f_20951)
static void C_ccall f_20951(C_word c,C_word *av) C_noret;
C_noret_decl(f_20954)
static void C_ccall f_20954(C_word c,C_word *av) C_noret;
C_noret_decl(f_20983)
static void C_fcall f_20983(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20993)
static void C_ccall f_20993(C_word c,C_word *av) C_noret;
C_noret_decl(f_21009)
static void C_ccall f_21009(C_word c,C_word *av) C_noret;
C_noret_decl(f_21020)
static void C_ccall f_21020(C_word c,C_word *av) C_noret;
C_noret_decl(f_21027)
static void C_ccall f_21027(C_word c,C_word *av) C_noret;
C_noret_decl(f_21125)
static void C_ccall f_21125(C_word c,C_word *av) C_noret;
C_noret_decl(f_21131)
static void C_fcall f_21131(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21140)
static void C_fcall f_21140(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21154)
static void C_ccall f_21154(C_word c,C_word *av) C_noret;
C_noret_decl(f_21157)
static void C_ccall f_21157(C_word c,C_word *av) C_noret;
C_noret_decl(f_21168)
static void C_ccall f_21168(C_word c,C_word *av) C_noret;
C_noret_decl(f_21170)
static void C_ccall f_21170(C_word c,C_word *av) C_noret;
C_noret_decl(f_21173)
static void C_ccall f_21173(C_word c,C_word *av) C_noret;
C_noret_decl(f_21190)
static void C_ccall f_21190(C_word c,C_word *av) C_noret;
C_noret_decl(f_21195)
static void C_fcall f_21195(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21199)
static void C_ccall f_21199(C_word c,C_word *av) C_noret;
C_noret_decl(f_21202)
static void C_fcall f_21202(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21205)
static void C_ccall f_21205(C_word c,C_word *av) C_noret;
C_noret_decl(f_21215)
static void C_ccall f_21215(C_word c,C_word *av) C_noret;
C_noret_decl(f_21233)
static void C_ccall f_21233(C_word c,C_word *av) C_noret;
C_noret_decl(f_21236)
static void C_ccall f_21236(C_word c,C_word *av) C_noret;
C_noret_decl(f_21245)
static void C_fcall f_21245(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21248)
static void C_ccall f_21248(C_word c,C_word *av) C_noret;
C_noret_decl(f_21251)
static void C_ccall f_21251(C_word c,C_word *av) C_noret;
C_noret_decl(f_21257)
static void C_ccall f_21257(C_word c,C_word *av) C_noret;
C_noret_decl(f_21270)
static void C_ccall f_21270(C_word c,C_word *av) C_noret;
C_noret_decl(f_21274)
static void C_ccall f_21274(C_word c,C_word *av) C_noret;
C_noret_decl(f_21278)
static void C_ccall f_21278(C_word c,C_word *av) C_noret;
C_noret_decl(f_21289)
static void C_ccall f_21289(C_word c,C_word *av) C_noret;
C_noret_decl(f_21293)
static void C_ccall f_21293(C_word c,C_word *av) C_noret;
C_noret_decl(f_21296)
static void C_ccall f_21296(C_word c,C_word *av) C_noret;
C_noret_decl(f_21324)
static void C_ccall f_21324(C_word c,C_word *av) C_noret;
C_noret_decl(f_21380)
static void C_ccall f_21380(C_word c,C_word *av) C_noret;
C_noret_decl(f_21384)
static void C_ccall f_21384(C_word c,C_word *av) C_noret;
C_noret_decl(f_21391)
static void C_ccall f_21391(C_word c,C_word *av) C_noret;
C_noret_decl(f_21393)
static void C_fcall f_21393(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21397)
static void C_ccall f_21397(C_word c,C_word *av) C_noret;
C_noret_decl(f_21411)
static void C_fcall f_21411(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21417)
static void C_ccall f_21417(C_word c,C_word *av) C_noret;
C_noret_decl(f_21424)
static void C_ccall f_21424(C_word c,C_word *av) C_noret;
C_noret_decl(f_21431)
static void C_ccall f_21431(C_word c,C_word *av) C_noret;
C_noret_decl(f_21452)
static void C_ccall f_21452(C_word c,C_word *av) C_noret;
C_noret_decl(f_21481)
static void C_fcall f_21481(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21491)
static void C_ccall f_21491(C_word c,C_word *av) C_noret;
C_noret_decl(f_21494)
static void C_ccall f_21494(C_word c,C_word *av) C_noret;
C_noret_decl(f_21530)
static void C_ccall f_21530(C_word c,C_word *av) C_noret;
C_noret_decl(f_21532)
static void C_fcall f_21532(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21542)
static void C_ccall f_21542(C_word c,C_word *av) C_noret;
C_noret_decl(f_21545)
static void C_ccall f_21545(C_word c,C_word *av) C_noret;
C_noret_decl(f_21597)
static void C_ccall f_21597(C_word c,C_word *av) C_noret;
C_noret_decl(f_21599)
static void C_fcall f_21599(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21607)
static void C_ccall f_21607(C_word c,C_word *av) C_noret;
C_noret_decl(f_21609)
static void C_fcall f_21609(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21619)
static void C_fcall f_21619(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21634)
static void C_ccall f_21634(C_word c,C_word *av) C_noret;
C_noret_decl(f_21641)
static void C_ccall f_21641(C_word c,C_word *av) C_noret;
C_noret_decl(f_21663)
static void C_fcall f_21663(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21671)
static void C_ccall f_21671(C_word c,C_word *av) C_noret;
C_noret_decl(f_21673)
static void C_fcall f_21673(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21689)
static void C_ccall f_21689(C_word c,C_word *av) C_noret;
C_noret_decl(f_21696)
static void C_ccall f_21696(C_word c,C_word *av) C_noret;
C_noret_decl(f_21709)
static void C_fcall f_21709(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21715)
static void C_ccall f_21715(C_word c,C_word *av) C_noret;
C_noret_decl(f_21719)
static void C_ccall f_21719(C_word c,C_word *av) C_noret;
C_noret_decl(f_21726)
static void C_ccall f_21726(C_word c,C_word *av) C_noret;
C_noret_decl(f_21734)
static void C_fcall f_21734(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21739)
static void C_fcall f_21739(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21744)
static void C_fcall f_21744(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_21748)
static void C_ccall f_21748(C_word c,C_word *av) C_noret;
C_noret_decl(f_21757)
static void C_fcall f_21757(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21763)
static void C_fcall f_21763(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21770)
static void C_ccall f_21770(C_word c,C_word *av) C_noret;
C_noret_decl(f_21787)
static void C_ccall f_21787(C_word c,C_word *av) C_noret;
C_noret_decl(f_21793)
static void C_fcall f_21793(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21803)
static void C_ccall f_21803(C_word c,C_word *av) C_noret;
C_noret_decl(f_21854)
static void C_ccall f_21854(C_word c,C_word *av) C_noret;
C_noret_decl(f_21863)
static void C_ccall f_21863(C_word c,C_word *av) C_noret;
C_noret_decl(f_21870)
static void C_ccall f_21870(C_word c,C_word *av) C_noret;
C_noret_decl(f_21874)
static void C_ccall f_21874(C_word c,C_word *av) C_noret;
C_noret_decl(f_21878)
static void C_ccall f_21878(C_word c,C_word *av) C_noret;
C_noret_decl(f_21887)
static void C_ccall f_21887(C_word c,C_word *av) C_noret;
C_noret_decl(f_21898)
static void C_ccall f_21898(C_word c,C_word *av) C_noret;
C_noret_decl(f_21920)
static void C_ccall f_21920(C_word c,C_word *av) C_noret;
C_noret_decl(f_21979)
static void C_ccall f_21979(C_word c,C_word *av) C_noret;
C_noret_decl(f_21983)
static void C_ccall f_21983(C_word c,C_word *av) C_noret;
C_noret_decl(f_21985)
static void C_fcall f_21985(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21989)
static void C_ccall f_21989(C_word c,C_word *av) C_noret;
C_noret_decl(f_21992)
static void C_ccall f_21992(C_word c,C_word *av) C_noret;
C_noret_decl(f_22004)
static void C_ccall f_22004(C_word c,C_word *av) C_noret;
C_noret_decl(f_22024)
static void C_fcall f_22024(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22030)
static void C_ccall f_22030(C_word c,C_word *av) C_noret;
C_noret_decl(f_22040)
static void C_ccall f_22040(C_word c,C_word *av) C_noret;
C_noret_decl(f_22169)
static void C_fcall f_22169(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22176)
static void C_ccall f_22176(C_word c,C_word *av) C_noret;
C_noret_decl(f_22194)
static void C_fcall f_22194(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22200)
static void C_fcall f_22200(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22204)
static void C_ccall f_22204(C_word c,C_word *av) C_noret;
C_noret_decl(f_22240)
static void C_ccall f_22240(C_word c,C_word *av) C_noret;
C_noret_decl(f_22263)
static void C_ccall f_22263(C_word c,C_word *av) C_noret;
C_noret_decl(f_22281)
static void C_fcall f_22281(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22285)
static void C_ccall f_22285(C_word c,C_word *av) C_noret;
C_noret_decl(f_22292)
static void C_ccall f_22292(C_word c,C_word *av) C_noret;
C_noret_decl(f_22294)
static void C_fcall f_22294(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22298)
static void C_ccall f_22298(C_word c,C_word *av) C_noret;
C_noret_decl(f_22309)
static void C_ccall f_22309(C_word c,C_word *av) C_noret;
C_noret_decl(f_22325)
static void C_ccall f_22325(C_word c,C_word *av) C_noret;
C_noret_decl(f_22328)
static void C_ccall f_22328(C_word c,C_word *av) C_noret;
C_noret_decl(f_22331)
static void C_ccall f_22331(C_word c,C_word *av) C_noret;
C_noret_decl(f_22345)
static void C_ccall f_22345(C_word c,C_word *av) C_noret;
C_noret_decl(f_22351)
static void C_ccall f_22351(C_word c,C_word *av) C_noret;
C_noret_decl(f_22373)
static void C_ccall f_22373(C_word c,C_word *av) C_noret;
C_noret_decl(f_22385)
static void C_ccall f_22385(C_word c,C_word *av) C_noret;
C_noret_decl(f_22397)
static void C_ccall f_22397(C_word c,C_word *av) C_noret;
C_noret_decl(f_22406)
static void C_ccall f_22406(C_word c,C_word *av) C_noret;
C_noret_decl(f_22416)
static void C_ccall f_22416(C_word c,C_word *av) C_noret;
C_noret_decl(f_22425)
static void C_ccall f_22425(C_word c,C_word *av) C_noret;
C_noret_decl(f_22428)
static void C_ccall f_22428(C_word c,C_word *av) C_noret;
C_noret_decl(f_22446)
static void C_ccall f_22446(C_word c,C_word *av) C_noret;
C_noret_decl(f_22449)
static void C_ccall f_22449(C_word c,C_word *av) C_noret;
C_noret_decl(f_22472)
static void C_ccall f_22472(C_word c,C_word *av) C_noret;
C_noret_decl(f_22478)
static void C_ccall f_22478(C_word c,C_word *av) C_noret;
C_noret_decl(f_22506)
static void C_ccall f_22506(C_word c,C_word *av) C_noret;
C_noret_decl(f_22529)
static void C_ccall f_22529(C_word c,C_word *av) C_noret;
C_noret_decl(f_22543)
static void C_ccall f_22543(C_word c,C_word *av) C_noret;
C_noret_decl(f_22549)
static void C_ccall f_22549(C_word c,C_word *av) C_noret;
C_noret_decl(f_22574)
static void C_ccall f_22574(C_word c,C_word *av) C_noret;
C_noret_decl(f_22586)
static void C_ccall f_22586(C_word c,C_word *av) C_noret;
C_noret_decl(f_22598)
static void C_ccall f_22598(C_word c,C_word *av) C_noret;
C_noret_decl(f_22610)
static void C_ccall f_22610(C_word c,C_word *av) C_noret;
C_noret_decl(f_22622)
static void C_ccall f_22622(C_word c,C_word *av) C_noret;
C_noret_decl(f_22634)
static void C_ccall f_22634(C_word c,C_word *av) C_noret;
C_noret_decl(f_22646)
static void C_ccall f_22646(C_word c,C_word *av) C_noret;
C_noret_decl(f_22649)
static void C_ccall f_22649(C_word c,C_word *av) C_noret;
C_noret_decl(f_22652)
static void C_ccall f_22652(C_word c,C_word *av) C_noret;
C_noret_decl(f_22657)
static void C_ccall f_22657(C_word c,C_word *av) C_noret;
C_noret_decl(f_22662)
static void C_ccall f_22662(C_word c,C_word *av) C_noret;
C_noret_decl(f_22668)
static void C_ccall f_22668(C_word c,C_word *av) C_noret;
C_noret_decl(f_22710)
static void C_ccall f_22710(C_word c,C_word *av) C_noret;
C_noret_decl(f_22722)
static void C_ccall f_22722(C_word c,C_word *av) C_noret;
C_noret_decl(f_22725)
static void C_ccall f_22725(C_word c,C_word *av) C_noret;
C_noret_decl(f_22737)
static void C_ccall f_22737(C_word c,C_word *av) C_noret;
C_noret_decl(f_22749)
static void C_ccall f_22749(C_word c,C_word *av) C_noret;
C_noret_decl(f_22752)
static void C_ccall f_22752(C_word c,C_word *av) C_noret;
C_noret_decl(f_22764)
static void C_ccall f_22764(C_word c,C_word *av) C_noret;
C_noret_decl(f_22776)
static void C_ccall f_22776(C_word c,C_word *av) C_noret;
C_noret_decl(f_22779)
static void C_ccall f_22779(C_word c,C_word *av) C_noret;
C_noret_decl(f_22788)
static void C_ccall f_22788(C_word c,C_word *av) C_noret;
C_noret_decl(f_22803)
static void C_ccall f_22803(C_word c,C_word *av) C_noret;
C_noret_decl(f_22806)
static void C_ccall f_22806(C_word c,C_word *av) C_noret;
C_noret_decl(f_22811)
static void C_ccall f_22811(C_word c,C_word *av) C_noret;
C_noret_decl(f_22816)
static void C_ccall f_22816(C_word c,C_word *av) C_noret;
C_noret_decl(f_22822)
static void C_ccall f_22822(C_word c,C_word *av) C_noret;
C_noret_decl(f_22851)
static void C_ccall f_22851(C_word c,C_word *av) C_noret;
C_noret_decl(f_22863)
static void C_ccall f_22863(C_word c,C_word *av) C_noret;
C_noret_decl(f_22866)
static void C_ccall f_22866(C_word c,C_word *av) C_noret;
C_noret_decl(f_22869)
static void C_ccall f_22869(C_word c,C_word *av) C_noret;
C_noret_decl(f_22884)
static void C_ccall f_22884(C_word c,C_word *av) C_noret;
C_noret_decl(f_22893)
static void C_ccall f_22893(C_word c,C_word *av) C_noret;
C_noret_decl(f_22896)
static void C_ccall f_22896(C_word c,C_word *av) C_noret;
C_noret_decl(f_22902)
static void C_fcall f_22902(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22905)
static void C_ccall f_22905(C_word c,C_word *av) C_noret;
C_noret_decl(f_22911)
static void C_ccall f_22911(C_word c,C_word *av) C_noret;
C_noret_decl(f_22930)
static void C_ccall f_22930(C_word c,C_word *av) C_noret;
C_noret_decl(f_22949)
static void C_ccall f_22949(C_word c,C_word *av) C_noret;
C_noret_decl(f_22965)
static void C_ccall f_22965(C_word c,C_word *av) C_noret;
C_noret_decl(f_22971)
static void C_ccall f_22971(C_word c,C_word *av) C_noret;
C_noret_decl(f_23043)
static void C_ccall f_23043(C_word c,C_word *av) C_noret;
C_noret_decl(f_23055)
static void C_ccall f_23055(C_word c,C_word *av) C_noret;
C_noret_decl(f_23133)
static void C_ccall f_23133(C_word c,C_word *av) C_noret;
C_noret_decl(f_23209)
static void C_ccall f_23209(C_word c,C_word *av) C_noret;
C_noret_decl(f_23374)
static void C_ccall f_23374(C_word c,C_word *av) C_noret;
C_noret_decl(f_23386)
static void C_ccall f_23386(C_word c,C_word *av) C_noret;
C_noret_decl(f_23448)
static void C_ccall f_23448(C_word c,C_word *av) C_noret;
C_noret_decl(f_23451)
static void C_fcall f_23451(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23497)
static void C_fcall f_23497(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_23501)
static void C_ccall f_23501(C_word c,C_word *av) C_noret;
C_noret_decl(f_23519)
static void C_ccall f_23519(C_word c,C_word *av) C_noret;
C_noret_decl(f_23583)
static void C_ccall f_23583(C_word c,C_word *av) C_noret;
C_noret_decl(f_23594)
static void C_ccall f_23594(C_word c,C_word *av) C_noret;
C_noret_decl(f_23596)
static void C_ccall f_23596(C_word c,C_word *av) C_noret;
C_noret_decl(f_23609)
static void C_ccall f_23609(C_word c,C_word *av) C_noret;
C_noret_decl(f_23621)
static void C_ccall f_23621(C_word c,C_word *av) C_noret;
C_noret_decl(f_23641)
static void C_ccall f_23641(C_word c,C_word *av) C_noret;
C_noret_decl(f_23648)
static void C_ccall f_23648(C_word c,C_word *av) C_noret;
C_noret_decl(f_23657)
static void C_ccall f_23657(C_word c,C_word *av) C_noret;
C_noret_decl(f_23669)
static void C_ccall f_23669(C_word c,C_word *av) C_noret;
C_noret_decl(f_23690)
static void C_ccall f_23690(C_word c,C_word *av) C_noret;
C_noret_decl(f_23705)
static void C_ccall f_23705(C_word c,C_word *av) C_noret;
C_noret_decl(f_23723)
static void C_ccall f_23723(C_word c,C_word *av) C_noret;
C_noret_decl(f_23741)
static void C_ccall f_23741(C_word c,C_word *av) C_noret;
C_noret_decl(f_23760)
static void C_fcall f_23760(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23770)
static void C_ccall f_23770(C_word c,C_word *av) C_noret;
C_noret_decl(f_23784)
static void C_ccall f_23784(C_word c,C_word *av) C_noret;
C_noret_decl(f_23787)
static void C_ccall f_23787(C_word c,C_word *av) C_noret;
C_noret_decl(f_23792)
static void C_ccall f_23792(C_word c,C_word *av) C_noret;
C_noret_decl(f_23811)
static void C_fcall f_23811(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_23821)
static void C_ccall f_23821(C_word c,C_word *av) C_noret;
C_noret_decl(f_23835)
static void C_ccall f_23835(C_word c,C_word *av) C_noret;
C_noret_decl(f_23838)
static void C_ccall f_23838(C_word c,C_word *av) C_noret;
C_noret_decl(f_23845)
static void C_ccall f_23845(C_word c,C_word *av) C_noret;
C_noret_decl(f_23849)
static void C_ccall f_23849(C_word c,C_word *av) C_noret;
C_noret_decl(f_23853)
static void C_ccall f_23853(C_word c,C_word *av) C_noret;
C_noret_decl(f_23855)
static void C_ccall f_23855(C_word c,C_word *av) C_noret;
C_noret_decl(f_23862)
static void C_ccall f_23862(C_word c,C_word *av) C_noret;
C_noret_decl(f_23865)
static void C_ccall f_23865(C_word c,C_word *av) C_noret;
C_noret_decl(f_23868)
static void C_ccall f_23868(C_word c,C_word *av) C_noret;
C_noret_decl(f_23870)
static void C_fcall f_23870(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_23880)
static void C_ccall f_23880(C_word c,C_word *av) C_noret;
C_noret_decl(f_23886)
static void C_ccall f_23886(C_word c,C_word *av) C_noret;
C_noret_decl(f_23898)
static void C_ccall f_23898(C_word c,C_word *av) C_noret;
C_noret_decl(f_23901)
static void C_ccall f_23901(C_word c,C_word *av) C_noret;
C_noret_decl(f_23904)
static void C_ccall f_23904(C_word c,C_word *av) C_noret;
C_noret_decl(f_23921)
static void C_ccall f_23921(C_word c,C_word *av) C_noret;
C_noret_decl(f_23942)
static void C_fcall f_23942(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_23946)
static void C_ccall f_23946(C_word c,C_word *av) C_noret;
C_noret_decl(f_23959)
static void C_ccall f_23959(C_word c,C_word *av) C_noret;
C_noret_decl(f_23962)
static void C_ccall f_23962(C_word c,C_word *av) C_noret;
C_noret_decl(f_23971)
static void C_ccall f_23971(C_word c,C_word *av) C_noret;
C_noret_decl(f_23974)
static void C_ccall f_23974(C_word c,C_word *av) C_noret;
C_noret_decl(f_24002)
static void C_fcall f_24002(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_24006)
static void C_ccall f_24006(C_word c,C_word *av) C_noret;
C_noret_decl(f_24015)
static void C_ccall f_24015(C_word c,C_word *av) C_noret;
C_noret_decl(f_24026)
static void C_fcall f_24026(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_24033)
static void C_ccall f_24033(C_word c,C_word *av) C_noret;
C_noret_decl(f_24038)
static void C_fcall f_24038(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24060)
static void C_ccall f_24060(C_word c,C_word *av) C_noret;
C_noret_decl(f_24066)
static void C_ccall f_24066(C_word c,C_word *av) C_noret;
C_noret_decl(f_24069)
static void C_ccall f_24069(C_word c,C_word *av) C_noret;
C_noret_decl(f_24080)
static void C_ccall f_24080(C_word c,C_word *av) C_noret;
C_noret_decl(f_24089)
static void C_ccall f_24089(C_word c,C_word *av) C_noret;
C_noret_decl(f_24092)
static void C_ccall f_24092(C_word c,C_word *av) C_noret;
C_noret_decl(f_24115)
static void C_fcall f_24115(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24164)
static void C_fcall f_24164(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24180)
static void C_fcall f_24180(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24187)
static void C_ccall f_24187(C_word c,C_word *av) C_noret;
C_noret_decl(f_24199)
static void C_fcall f_24199(C_word t0,C_word t1) C_noret;
C_noret_decl(f_24304)
static void C_ccall f_24304(C_word c,C_word *av) C_noret;
C_noret_decl(f_24309)
static void C_fcall f_24309(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24358)
static void C_ccall f_24358(C_word c,C_word *av) C_noret;
C_noret_decl(f_24364)
static void C_ccall f_24364(C_word c,C_word *av) C_noret;
C_noret_decl(f_24368)
static void C_fcall f_24368(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24391)
static void C_ccall f_24391(C_word c,C_word *av) C_noret;
C_noret_decl(f_24398)
static void C_ccall f_24398(C_word c,C_word *av) C_noret;
C_noret_decl(f_24417)
static void C_ccall f_24417(C_word c,C_word *av) C_noret;
C_noret_decl(f_24453)
static void C_ccall f_24453(C_word c,C_word *av) C_noret;
C_noret_decl(f_24465)
static void C_ccall f_24465(C_word c,C_word *av) C_noret;
C_noret_decl(f_24471)
static void C_ccall f_24471(C_word c,C_word *av) C_noret;
C_noret_decl(f_24493)
static void C_ccall f_24493(C_word c,C_word *av) C_noret;
C_noret_decl(f_24500)
static void C_ccall f_24500(C_word c,C_word *av) C_noret;
C_noret_decl(f_24510)
static void C_ccall f_24510(C_word c,C_word *av) C_noret;
C_noret_decl(f_24519)
static void C_ccall f_24519(C_word c,C_word *av) C_noret;
C_noret_decl(f_24525)
static void C_fcall f_24525(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_24538)
static void C_ccall f_24538(C_word c,C_word *av) C_noret;
C_noret_decl(f_24576)
static void C_ccall f_24576(C_word c,C_word *av) C_noret;
C_noret_decl(f_24642)
static void C_ccall f_24642(C_word c,C_word *av) C_noret;
C_noret_decl(f_24645)
static void C_ccall f_24645(C_word c,C_word *av) C_noret;
C_noret_decl(f_24652)
static void C_ccall f_24652(C_word c,C_word *av) C_noret;
C_noret_decl(f_24692)
static void C_ccall f_24692(C_word c,C_word *av) C_noret;
C_noret_decl(f_24695)
static void C_ccall f_24695(C_word c,C_word *av) C_noret;
C_noret_decl(f_24704)
static void C_fcall f_24704(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24717)
static void C_ccall f_24717(C_word c,C_word *av) C_noret;
C_noret_decl(f_24726)
static void C_ccall f_24726(C_word c,C_word *av) C_noret;
C_noret_decl(f_24736)
static void C_ccall f_24736(C_word c,C_word *av) C_noret;
C_noret_decl(f_24739)
static void C_ccall f_24739(C_word c,C_word *av) C_noret;
C_noret_decl(f_24765)
static void C_ccall f_24765(C_word c,C_word *av) C_noret;
C_noret_decl(f_24771)
static void C_ccall f_24771(C_word c,C_word *av) C_noret;
C_noret_decl(f_24776)
static void C_fcall f_24776(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_24789)
static void C_ccall f_24789(C_word c,C_word *av) C_noret;
C_noret_decl(f_24792)
static void C_ccall f_24792(C_word c,C_word *av) C_noret;
C_noret_decl(f_24803)
static void C_ccall f_24803(C_word c,C_word *av) C_noret;
C_noret_decl(f_24825)
static void C_ccall f_24825(C_word c,C_word *av) C_noret;
C_noret_decl(f_24837)
static void C_ccall f_24837(C_word c,C_word *av) C_noret;
C_noret_decl(f_24840)
static void C_ccall f_24840(C_word c,C_word *av) C_noret;
C_noret_decl(f_24847)
static void C_ccall f_24847(C_word c,C_word *av) C_noret;
C_noret_decl(f_24856)
static void C_ccall f_24856(C_word c,C_word *av) C_noret;
C_noret_decl(f_24859)
static void C_ccall f_24859(C_word c,C_word *av) C_noret;
C_noret_decl(f_24903)
static void C_ccall f_24903(C_word c,C_word *av) C_noret;
C_noret_decl(f_24906)
static void C_ccall f_24906(C_word c,C_word *av) C_noret;
C_noret_decl(f_24915)
static void C_fcall f_24915(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_24928)
static void C_ccall f_24928(C_word c,C_word *av) C_noret;
C_noret_decl(f_24931)
static void C_ccall f_24931(C_word c,C_word *av) C_noret;
C_noret_decl(f_24956)
static void C_ccall f_24956(C_word c,C_word *av) C_noret;
C_noret_decl(f_24960)
static void C_ccall f_24960(C_word c,C_word *av) C_noret;
C_noret_decl(f_24970)
static void C_ccall f_24970(C_word c,C_word *av) C_noret;
C_noret_decl(f_24973)
static void C_ccall f_24973(C_word c,C_word *av) C_noret;
C_noret_decl(f_24988)
static void C_ccall f_24988(C_word c,C_word *av) C_noret;
C_noret_decl(f_24995)
static void C_ccall f_24995(C_word c,C_word *av) C_noret;
C_noret_decl(f_25017)
static void C_ccall f_25017(C_word c,C_word *av) C_noret;
C_noret_decl(f_25019)
static void C_ccall f_25019(C_word c,C_word *av) C_noret;
C_noret_decl(f_25032)
static void C_ccall f_25032(C_word c,C_word *av) C_noret;
C_noret_decl(f_25037)
static void C_ccall f_25037(C_word c,C_word *av) C_noret;
C_noret_decl(f_25043)
static void C_ccall f_25043(C_word c,C_word *av) C_noret;
C_noret_decl(f_25049)
static void C_ccall f_25049(C_word c,C_word *av) C_noret;
C_noret_decl(f_25053)
static void C_ccall f_25053(C_word c,C_word *av) C_noret;
C_noret_decl(f_25056)
static void C_ccall f_25056(C_word c,C_word *av) C_noret;
C_noret_decl(f_25071)
static void C_ccall f_25071(C_word c,C_word *av) C_noret;
C_noret_decl(f_25074)
static void C_ccall f_25074(C_word c,C_word *av) C_noret;
C_noret_decl(f_25092)
static void C_ccall f_25092(C_word c,C_word *av) C_noret;
C_noret_decl(f_25094)
static void C_ccall f_25094(C_word c,C_word *av) C_noret;
C_noret_decl(f_25100)
static void C_ccall f_25100(C_word c,C_word *av) C_noret;
C_noret_decl(f_25109)
static void C_ccall f_25109(C_word c,C_word *av) C_noret;
C_noret_decl(f_25115)
static void C_ccall f_25115(C_word c,C_word *av) C_noret;
C_noret_decl(f_25122)
static void C_ccall f_25122(C_word c,C_word *av) C_noret;
C_noret_decl(f_25125)
static void C_ccall f_25125(C_word c,C_word *av) C_noret;
C_noret_decl(f_25128)
static void C_ccall f_25128(C_word c,C_word *av) C_noret;
C_noret_decl(f_25137)
static void C_ccall f_25137(C_word c,C_word *av) C_noret;
C_noret_decl(f_25153)
static void C_ccall f_25153(C_word c,C_word *av) C_noret;
C_noret_decl(f_25168)
static void C_ccall f_25168(C_word c,C_word *av) C_noret;
C_noret_decl(f_25170)
static void C_ccall f_25170(C_word c,C_word *av) C_noret;
C_noret_decl(f_25176)
static void C_ccall f_25176(C_word c,C_word *av) C_noret;
C_noret_decl(f_25182)
static void C_ccall f_25182(C_word c,C_word *av) C_noret;
C_noret_decl(f_25186)
static void C_ccall f_25186(C_word c,C_word *av) C_noret;
C_noret_decl(f_25189)
static void C_ccall f_25189(C_word c,C_word *av) C_noret;
C_noret_decl(f_25192)
static void C_ccall f_25192(C_word c,C_word *av) C_noret;
C_noret_decl(f_25196)
static void C_ccall f_25196(C_word c,C_word *av) C_noret;
C_noret_decl(f_25200)
static void C_ccall f_25200(C_word c,C_word *av) C_noret;
C_noret_decl(f_25204)
static void C_ccall f_25204(C_word c,C_word *av) C_noret;
C_noret_decl(f_25207)
static void C_ccall f_25207(C_word c,C_word *av) C_noret;
C_noret_decl(f_25210)
static void C_ccall f_25210(C_word c,C_word *av) C_noret;
C_noret_decl(f_25213)
static void C_ccall f_25213(C_word c,C_word *av) C_noret;
C_noret_decl(f_25225)
static void C_ccall f_25225(C_word c,C_word *av) C_noret;
C_noret_decl(f_25231)
static void C_ccall f_25231(C_word c,C_word *av) C_noret;
C_noret_decl(f_25235)
static void C_ccall f_25235(C_word c,C_word *av) C_noret;
C_noret_decl(f_25238)
static void C_ccall f_25238(C_word c,C_word *av) C_noret;
C_noret_decl(f_25241)
static void C_ccall f_25241(C_word c,C_word *av) C_noret;
C_noret_decl(f_25244)
static void C_ccall f_25244(C_word c,C_word *av) C_noret;
C_noret_decl(f_25247)
static void C_ccall f_25247(C_word c,C_word *av) C_noret;
C_noret_decl(f_25250)
static void C_ccall f_25250(C_word c,C_word *av) C_noret;
C_noret_decl(f_25255)
static void C_ccall f_25255(C_word c,C_word *av) C_noret;
C_noret_decl(f_25277)
static void C_ccall f_25277(C_word c,C_word *av) C_noret;
C_noret_decl(f_25283)
static void C_ccall f_25283(C_word c,C_word *av) C_noret;
C_noret_decl(f_25286)
static void C_ccall f_25286(C_word c,C_word *av) C_noret;
C_noret_decl(f_25311)
static void C_ccall f_25311(C_word c,C_word *av) C_noret;
C_noret_decl(f_25314)
static void C_fcall f_25314(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_25321)
static void C_ccall f_25321(C_word c,C_word *av) C_noret;
C_noret_decl(f_25334)
static void C_fcall f_25334(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_25344)
static void C_ccall f_25344(C_word c,C_word *av) C_noret;
C_noret_decl(f_25349)
static void C_ccall f_25349(C_word c,C_word *av) C_noret;
C_noret_decl(f_25355)
static void C_ccall f_25355(C_word c,C_word *av) C_noret;
C_noret_decl(f_25382)
static void C_ccall f_25382(C_word c,C_word *av) C_noret;
C_noret_decl(f_25388)
static void C_fcall f_25388(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25399)
static void C_ccall f_25399(C_word c,C_word *av) C_noret;
C_noret_decl(f_25408)
static void C_ccall f_25408(C_word c,C_word *av) C_noret;
C_noret_decl(f_25413)
static void C_ccall f_25413(C_word c,C_word *av) C_noret;
C_noret_decl(f_25419)
static void C_ccall f_25419(C_word c,C_word *av) C_noret;
C_noret_decl(f_25443)
static void C_ccall f_25443(C_word c,C_word *av) C_noret;
C_noret_decl(f_25451)
static void C_ccall f_25451(C_word c,C_word *av) C_noret;
C_noret_decl(f_25468)
static void C_ccall f_25468(C_word c,C_word *av) C_noret;
C_noret_decl(f_25508)
static void C_ccall f_25508(C_word c,C_word *av) C_noret;
C_noret_decl(f_25515)
static void C_ccall f_25515(C_word c,C_word *av) C_noret;
C_noret_decl(f_25527)
static void C_ccall f_25527(C_word c,C_word *av) C_noret;
C_noret_decl(f_25531)
static void C_ccall f_25531(C_word c,C_word *av) C_noret;
C_noret_decl(f_25544)
static void C_ccall f_25544(C_word c,C_word *av) C_noret;
C_noret_decl(f_25546)
static void C_ccall f_25546(C_word c,C_word *av) C_noret;
C_noret_decl(f_25580)
static void C_ccall f_25580(C_word c,C_word *av) C_noret;
C_noret_decl(f_25584)
static void C_ccall f_25584(C_word c,C_word *av) C_noret;
C_noret_decl(f_25585)
static void C_fcall f_25585(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25595)
static void C_ccall f_25595(C_word c,C_word *av) C_noret;
C_noret_decl(f_25600)
static void C_fcall f_25600(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25610)
static void C_ccall f_25610(C_word c,C_word *av) C_noret;
C_noret_decl(f_25623)
static void C_ccall f_25623(C_word c,C_word *av) C_noret;
C_noret_decl(f_25634)
static void C_ccall f_25634(C_word c,C_word *av) C_noret;
C_noret_decl(f_25638)
static void C_ccall f_25638(C_word c,C_word *av) C_noret;
C_noret_decl(f_25642)
static void C_ccall f_25642(C_word c,C_word *av) C_noret;
C_noret_decl(f_25664)
static void C_ccall f_25664(C_word c,C_word *av) C_noret;
C_noret_decl(f_25668)
static void C_ccall f_25668(C_word c,C_word *av) C_noret;
C_noret_decl(f_25676)
static void C_ccall f_25676(C_word c,C_word *av) C_noret;
C_noret_decl(f_25692)
static void C_ccall f_25692(C_word c,C_word *av) C_noret;
C_noret_decl(f_25703)
static void C_fcall f_25703(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25722)
static void C_fcall f_25722(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25733)
static void C_ccall f_25733(C_word c,C_word *av) C_noret;
C_noret_decl(f_25741)
static void C_ccall f_25741(C_word c,C_word *av) C_noret;
C_noret_decl(f_25820)
static void C_ccall f_25820(C_word c,C_word *av) C_noret;
C_noret_decl(f_25830)
static void C_ccall f_25830(C_word c,C_word *av) C_noret;
C_noret_decl(f_25831)
static void C_fcall f_25831(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25844)
static void C_ccall f_25844(C_word c,C_word *av) C_noret;
C_noret_decl(f_25847)
static void C_ccall f_25847(C_word c,C_word *av) C_noret;
C_noret_decl(f_25850)
static void C_ccall f_25850(C_word c,C_word *av) C_noret;
C_noret_decl(f_25853)
static void C_ccall f_25853(C_word c,C_word *av) C_noret;
C_noret_decl(f_25861)
static void C_ccall f_25861(C_word c,C_word *av) C_noret;
C_noret_decl(f_25868)
static void C_fcall f_25868(C_word t0,C_word t1) C_noret;
C_noret_decl(f_25871)
static void C_ccall f_25871(C_word c,C_word *av) C_noret;
C_noret_decl(f_25874)
static void C_ccall f_25874(C_word c,C_word *av) C_noret;
C_noret_decl(f_25897)
static void C_ccall f_25897(C_word c,C_word *av) C_noret;
C_noret_decl(f_25902)
static void C_fcall f_25902(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_25912)
static void C_ccall f_25912(C_word c,C_word *av) C_noret;
C_noret_decl(f_25925)
static void C_ccall f_25925(C_word c,C_word *av) C_noret;
C_noret_decl(f_25956)
static void C_ccall f_25956(C_word c,C_word *av) C_noret;
C_noret_decl(f_25966)
static void C_ccall f_25966(C_word c,C_word *av) C_noret;
C_noret_decl(f_26016)
static void C_ccall f_26016(C_word c,C_word *av) C_noret;
C_noret_decl(f_26019)
static void C_ccall f_26019(C_word c,C_word *av) C_noret;
C_noret_decl(f_26038)
static void C_fcall f_26038(C_word t0) C_noret;
C_noret_decl(f_26043)
static void C_ccall f_26043(C_word c,C_word *av) C_noret;
C_noret_decl(f_26049)
static void C_ccall f_26049(C_word c,C_word *av) C_noret;
C_noret_decl(f_26052)
static void C_ccall f_26052(C_word c,C_word *av) C_noret;
C_noret_decl(f_26061)
static void C_ccall f_26061(C_word c,C_word *av) C_noret;
C_noret_decl(f_26067)
static void C_ccall f_26067(C_word c,C_word *av) C_noret;
C_noret_decl(f_26086)
static void C_fcall f_26086(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26105)
static void C_ccall f_26105(C_word c,C_word *av) C_noret;
C_noret_decl(f_26110)
static void C_fcall f_26110(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26120)
static void C_ccall f_26120(C_word c,C_word *av) C_noret;
C_noret_decl(f_26134)
static void C_ccall f_26134(C_word c,C_word *av) C_noret;
C_noret_decl(f_26141)
static void C_ccall f_26141(C_word c,C_word *av) C_noret;
C_noret_decl(f_26145)
static void C_ccall f_26145(C_word c,C_word *av) C_noret;
C_noret_decl(f_26149)
static void C_ccall f_26149(C_word c,C_word *av) C_noret;
C_noret_decl(f_26151)
static void C_ccall f_26151(C_word c,C_word *av) C_noret;
C_noret_decl(f_26177)
static void C_ccall f_26177(C_word c,C_word *av) C_noret;
C_noret_decl(f_26180)
static void C_ccall f_26180(C_word c,C_word *av) C_noret;
C_noret_decl(f_26183)
static void C_ccall f_26183(C_word c,C_word *av) C_noret;
C_noret_decl(f_26190)
static void C_ccall f_26190(C_word c,C_word *av) C_noret;
C_noret_decl(f_26194)
static void C_ccall f_26194(C_word c,C_word *av) C_noret;
C_noret_decl(f_26200)
static void C_ccall f_26200(C_word c,C_word *av) C_noret;
C_noret_decl(f_26205)
static void C_fcall f_26205(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26215)
static void C_ccall f_26215(C_word c,C_word *av) C_noret;
C_noret_decl(f_26253)
static void C_fcall f_26253(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26256)
static void C_fcall f_26256(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26276)
static void C_fcall f_26276(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26284)
static void C_ccall f_26284(C_word c,C_word *av) C_noret;
C_noret_decl(f_26418)
static void C_ccall f_26418(C_word c,C_word *av) C_noret;
C_noret_decl(f_26435)
static void C_ccall f_26435(C_word c,C_word *av) C_noret;
C_noret_decl(f_26439)
static void C_ccall f_26439(C_word c,C_word *av) C_noret;
C_noret_decl(f_26452)
static void C_ccall f_26452(C_word c,C_word *av) C_noret;
C_noret_decl(f_26461)
static void C_ccall f_26461(C_word c,C_word *av) C_noret;
C_noret_decl(f_26464)
static void C_ccall f_26464(C_word c,C_word *av) C_noret;
C_noret_decl(f_26468)
static void C_ccall f_26468(C_word c,C_word *av) C_noret;
C_noret_decl(f_26507)
static void C_ccall f_26507(C_word c,C_word *av) C_noret;
C_noret_decl(f_26510)
static void C_ccall f_26510(C_word c,C_word *av) C_noret;
C_noret_decl(f_26517)
static void C_ccall f_26517(C_word c,C_word *av) C_noret;
C_noret_decl(f_26521)
static void C_ccall f_26521(C_word c,C_word *av) C_noret;
C_noret_decl(f_26547)
static void C_ccall f_26547(C_word c,C_word *av) C_noret;
C_noret_decl(f_26578)
static void C_ccall f_26578(C_word c,C_word *av) C_noret;
C_noret_decl(f_26581)
static void C_ccall f_26581(C_word c,C_word *av) C_noret;
C_noret_decl(f_26593)
static void C_ccall f_26593(C_word c,C_word *av) C_noret;
C_noret_decl(f_26596)
static void C_ccall f_26596(C_word c,C_word *av) C_noret;
C_noret_decl(f_26599)
static void C_ccall f_26599(C_word c,C_word *av) C_noret;
C_noret_decl(f_26624)
static void C_ccall f_26624(C_word c,C_word *av) C_noret;
C_noret_decl(f_26630)
static void C_ccall f_26630(C_word c,C_word *av) C_noret;
C_noret_decl(f_26634)
static void C_ccall f_26634(C_word c,C_word *av) C_noret;
C_noret_decl(f_26638)
static void C_ccall f_26638(C_word c,C_word *av) C_noret;
C_noret_decl(f_26651)
static void C_ccall f_26651(C_word c,C_word *av) C_noret;
C_noret_decl(f_26653)
static C_word C_fcall f_26653(C_word t0,C_word t1);
C_noret_decl(f_26659)
static C_word C_fcall f_26659(C_word t0);
C_noret_decl(f_26664)
static C_word C_fcall f_26664(C_word t0);
C_noret_decl(f_26699)
static void C_fcall f_26699(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_26705)
static void C_fcall f_26705(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26721)
static void C_fcall f_26721(C_word t0,C_word t1) C_noret;
C_noret_decl(f_26743)
static void C_ccall f_26743(C_word c,C_word *av) C_noret;
C_noret_decl(f_26767)
static void C_ccall f_26767(C_word c,C_word *av) C_noret;
C_noret_decl(f_26779)
static void C_ccall f_26779(C_word c,C_word *av) C_noret;
C_noret_decl(f_26781)
static void C_ccall f_26781(C_word c,C_word *av) C_noret;
C_noret_decl(f_26800)
static void C_ccall f_26800(C_word c,C_word *av) C_noret;
C_noret_decl(f_26804)
static void C_ccall f_26804(C_word c,C_word *av) C_noret;
C_noret_decl(f_26816)
static void C_ccall f_26816(C_word c,C_word *av) C_noret;
C_noret_decl(f_26818)
static void C_fcall f_26818(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26862)
static void C_ccall f_26862(C_word c,C_word *av) C_noret;
C_noret_decl(f_26864)
static void C_fcall f_26864(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26898)
static C_word C_fcall f_26898(C_word t0);
C_noret_decl(f_26921)
static void C_ccall f_26921(C_word c,C_word *av) C_noret;
C_noret_decl(f_26928)
static void C_ccall f_26928(C_word c,C_word *av) C_noret;
C_noret_decl(f_26938)
static void C_ccall f_26938(C_word c,C_word *av) C_noret;
C_noret_decl(f_26940)
static void C_fcall f_26940(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_26990)
static void C_ccall f_26990(C_word c,C_word *av) C_noret;
C_noret_decl(f_26992)
static void C_fcall f_26992(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27017)
static void C_ccall f_27017(C_word c,C_word *av) C_noret;
C_noret_decl(f_27026)
static void C_ccall f_27026(C_word c,C_word *av) C_noret;
C_noret_decl(f_27032)
static void C_ccall f_27032(C_word c,C_word *av) C_noret;
C_noret_decl(f_27036)
static void C_ccall f_27036(C_word c,C_word *av) C_noret;
C_noret_decl(f_27040)
static void C_fcall f_27040(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27048)
static void C_ccall f_27048(C_word c,C_word *av) C_noret;
C_noret_decl(f_27054)
static void C_fcall f_27054(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27085)
static void C_ccall f_27085(C_word c,C_word *av) C_noret;
C_noret_decl(f_27110)
static void C_fcall f_27110(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27135)
static void C_ccall f_27135(C_word c,C_word *av) C_noret;
C_noret_decl(f_27145)
static void C_ccall f_27145(C_word c,C_word *av) C_noret;
C_noret_decl(f_27150)
static void C_ccall f_27150(C_word c,C_word *av) C_noret;
C_noret_decl(f_27152)
static void C_ccall f_27152(C_word c,C_word *av) C_noret;
C_noret_decl(f_27159)
static void C_ccall f_27159(C_word c,C_word *av) C_noret;
C_noret_decl(f_27171)
static void C_ccall f_27171(C_word c,C_word *av) C_noret;
C_noret_decl(f_27179)
static void C_ccall f_27179(C_word c,C_word *av) C_noret;
C_noret_decl(f_27217)
static void C_ccall f_27217(C_word c,C_word *av) C_noret;
C_noret_decl(f_27221)
static void C_ccall f_27221(C_word c,C_word *av) C_noret;
C_noret_decl(f_27228)
static void C_ccall f_27228(C_word c,C_word *av) C_noret;
C_noret_decl(f_27231)
static void C_ccall f_27231(C_word c,C_word *av) C_noret;
C_noret_decl(f_27234)
static void C_ccall f_27234(C_word c,C_word *av) C_noret;
C_noret_decl(f_27235)
static void C_fcall f_27235(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27237)
static void C_fcall f_27237(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27241)
static void C_ccall f_27241(C_word c,C_word *av) C_noret;
C_noret_decl(f_27246)
static void C_ccall f_27246(C_word c,C_word *av) C_noret;
C_noret_decl(f_27258)
static void C_fcall f_27258(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27268)
static void C_ccall f_27268(C_word c,C_word *av) C_noret;
C_noret_decl(f_27280)
static void C_ccall f_27280(C_word c,C_word *av) C_noret;
C_noret_decl(f_27282)
static void C_fcall f_27282(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_27289)
static void C_ccall f_27289(C_word c,C_word *av) C_noret;
C_noret_decl(f_27292)
static void C_ccall f_27292(C_word c,C_word *av) C_noret;
C_noret_decl(f_27301)
static void C_ccall f_27301(C_word c,C_word *av) C_noret;
C_noret_decl(f_27304)
static void C_ccall f_27304(C_word c,C_word *av) C_noret;
C_noret_decl(f_27307)
static void C_ccall f_27307(C_word c,C_word *av) C_noret;
C_noret_decl(f_27322)
static void C_ccall f_27322(C_word c,C_word *av) C_noret;
C_noret_decl(f_27328)
static void C_ccall f_27328(C_word c,C_word *av) C_noret;
C_noret_decl(f_27336)
static void C_fcall f_27336(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27340)
static void C_ccall f_27340(C_word c,C_word *av) C_noret;
C_noret_decl(f_27343)
static void C_ccall f_27343(C_word c,C_word *av) C_noret;
C_noret_decl(f_27346)
static void C_ccall f_27346(C_word c,C_word *av) C_noret;
C_noret_decl(f_27359)
static void C_ccall f_27359(C_word c,C_word *av) C_noret;
C_noret_decl(f_27363)
static void C_ccall f_27363(C_word c,C_word *av) C_noret;
C_noret_decl(f_27384)
static void C_ccall f_27384(C_word c,C_word *av) C_noret;
C_noret_decl(f_27400)
static void C_ccall f_27400(C_word c,C_word *av) C_noret;
C_noret_decl(f_27403)
static void C_ccall f_27403(C_word c,C_word *av) C_noret;
C_noret_decl(f_27409)
static void C_ccall f_27409(C_word c,C_word *av) C_noret;
C_noret_decl(f_27431)
static void C_fcall f_27431(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_27436)
static void C_fcall f_27436(C_word t0,C_word t1) C_noret;
C_noret_decl(f_27471)
static void C_ccall f_27471(C_word c,C_word *av) C_noret;
C_noret_decl(f_27481)
static void C_ccall f_27481(C_word c,C_word *av) C_noret;
C_noret_decl(f_27484)
static void C_ccall f_27484(C_word c,C_word *av) C_noret;
C_noret_decl(f_27487)
static void C_ccall f_27487(C_word c,C_word *av) C_noret;
C_noret_decl(f_27490)
static void C_ccall f_27490(C_word c,C_word *av) C_noret;
C_noret_decl(f_27493)
static void C_ccall f_27493(C_word c,C_word *av) C_noret;
C_noret_decl(f_27496)
static void C_ccall f_27496(C_word c,C_word *av) C_noret;
C_noret_decl(f_27503)
static void C_ccall f_27503(C_word c,C_word *av) C_noret;
C_noret_decl(f_27506)
static void C_ccall f_27506(C_word c,C_word *av) C_noret;
C_noret_decl(f_27509)
static void C_ccall f_27509(C_word c,C_word *av) C_noret;
C_noret_decl(f_27520)
static void C_ccall f_27520(C_word c,C_word *av) C_noret;
C_noret_decl(f_27543)
static void C_ccall f_27543(C_word c,C_word *av) C_noret;
C_noret_decl(f_27554)
static void C_ccall f_27554(C_word c,C_word *av) C_noret;
C_noret_decl(f_27558)
static void C_ccall f_27558(C_word c,C_word *av) C_noret;
C_noret_decl(f_27580)
static void C_ccall f_27580(C_word c,C_word *av) C_noret;
C_noret_decl(f_27591)
static void C_ccall f_27591(C_word c,C_word *av) C_noret;
C_noret_decl(f_27595)
static void C_ccall f_27595(C_word c,C_word *av) C_noret;
C_noret_decl(f_28094)
static void C_ccall f_28094(C_word c,C_word *av) C_noret;
C_noret_decl(f_28105)
static void C_ccall f_28105(C_word c,C_word *av) C_noret;
C_noret_decl(f_28109)
static void C_ccall f_28109(C_word c,C_word *av) C_noret;
C_noret_decl(f_28226)
static void C_ccall f_28226(C_word c,C_word *av) C_noret;
C_noret_decl(f_28229)
static void C_ccall f_28229(C_word c,C_word *av) C_noret;
C_noret_decl(f_28232)
static void C_ccall f_28232(C_word c,C_word *av) C_noret;
C_noret_decl(f_28235)
static void C_ccall f_28235(C_word c,C_word *av) C_noret;
C_noret_decl(f_28240)
static void C_ccall f_28240(C_word c,C_word *av) C_noret;
C_noret_decl(f_28243)
static void C_ccall f_28243(C_word c,C_word *av) C_noret;
C_noret_decl(f_28246)
static void C_ccall f_28246(C_word c,C_word *av) C_noret;
C_noret_decl(f_28249)
static void C_ccall f_28249(C_word c,C_word *av) C_noret;
C_noret_decl(f_28252)
static void C_ccall f_28252(C_word c,C_word *av) C_noret;
C_noret_decl(f_28255)
static void C_ccall f_28255(C_word c,C_word *av) C_noret;
C_noret_decl(f_28258)
static void C_ccall f_28258(C_word c,C_word *av) C_noret;
C_noret_decl(f_28261)
static void C_ccall f_28261(C_word c,C_word *av) C_noret;
C_noret_decl(f_28264)
static void C_ccall f_28264(C_word c,C_word *av) C_noret;
C_noret_decl(f_28268)
static void C_ccall f_28268(C_word c,C_word *av) C_noret;
C_noret_decl(f_28270)
static void C_ccall f_28270(C_word c,C_word *av) C_noret;
C_noret_decl(f_28278)
static void C_ccall f_28278(C_word c,C_word *av) C_noret;
C_noret_decl(f_28280)
static void C_ccall f_28280(C_word c,C_word *av) C_noret;
C_noret_decl(f_28284)
static void C_ccall f_28284(C_word c,C_word *av) C_noret;
C_noret_decl(f_28286)
static void C_ccall f_28286(C_word c,C_word *av) C_noret;
C_noret_decl(f_28293)
static void C_ccall f_28293(C_word c,C_word *av) C_noret;
C_noret_decl(f_28303)
static void C_ccall f_28303(C_word c,C_word *av) C_noret;
C_noret_decl(f_28328)
static void C_ccall f_28328(C_word c,C_word *av) C_noret;
C_noret_decl(f_28331)
static void C_ccall f_28331(C_word c,C_word *av) C_noret;
C_noret_decl(f_28334)
static void C_ccall f_28334(C_word c,C_word *av) C_noret;
C_noret_decl(f_28337)
static void C_ccall f_28337(C_word c,C_word *av) C_noret;
C_noret_decl(f_28340)
static void C_ccall f_28340(C_word c,C_word *av) C_noret;
C_noret_decl(f_28350)
static void C_ccall f_28350(C_word c,C_word *av) C_noret;
C_noret_decl(f_28356)
static void C_ccall f_28356(C_word c,C_word *av) C_noret;
C_noret_decl(f_28360)
static void C_ccall f_28360(C_word c,C_word *av) C_noret;
C_noret_decl(f_28362)
static void C_ccall f_28362(C_word c,C_word *av) C_noret;
C_noret_decl(f_28372)
static void C_ccall f_28372(C_word c,C_word *av) C_noret;
C_noret_decl(f_28378)
static void C_ccall f_28378(C_word c,C_word *av) C_noret;
C_noret_decl(f_28382)
static void C_ccall f_28382(C_word c,C_word *av) C_noret;
C_noret_decl(f_28384)
static void C_ccall f_28384(C_word c,C_word *av) C_noret;
C_noret_decl(f_28389)
static void C_ccall f_28389(C_word c,C_word *av) C_noret;
C_noret_decl(f_28391)
static void C_ccall f_28391(C_word c,C_word *av) C_noret;
C_noret_decl(f_28394)
static void C_ccall f_28394(C_word c,C_word *av) C_noret;
C_noret_decl(f_28414)
static void C_ccall f_28414(C_word c,C_word *av) C_noret;
C_noret_decl(f_28420)
static void C_fcall f_28420(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28430)
static void C_ccall f_28430(C_word c,C_word *av) C_noret;
C_noret_decl(f_28440)
static void C_ccall f_28440(C_word c,C_word *av) C_noret;
C_noret_decl(f_28475)
static void C_ccall f_28475(C_word c,C_word *av) C_noret;
C_noret_decl(f_28479)
static void C_ccall f_28479(C_word c,C_word *av) C_noret;
C_noret_decl(f_28484)
static void C_fcall f_28484(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28494)
static void C_ccall f_28494(C_word c,C_word *av) C_noret;
C_noret_decl(f_28504)
static void C_ccall f_28504(C_word c,C_word *av) C_noret;
C_noret_decl(f_28513)
static void C_ccall f_28513(C_word c,C_word *av) C_noret;
C_noret_decl(f_28516)
static void C_ccall f_28516(C_word c,C_word *av) C_noret;
C_noret_decl(f_28519)
static void C_ccall f_28519(C_word c,C_word *av) C_noret;
C_noret_decl(f_28526)
static void C_ccall f_28526(C_word c,C_word *av) C_noret;
C_noret_decl(f_28533)
static void C_ccall f_28533(C_word c,C_word *av) C_noret;
C_noret_decl(f_28536)
static void C_ccall f_28536(C_word c,C_word *av) C_noret;
C_noret_decl(f_28540)
static void C_ccall f_28540(C_word c,C_word *av) C_noret;
C_noret_decl(f_28542)
static void C_ccall f_28542(C_word c,C_word *av) C_noret;
C_noret_decl(f_28545)
static void C_ccall f_28545(C_word c,C_word *av) C_noret;
C_noret_decl(f_28548)
static void C_ccall f_28548(C_word c,C_word *av) C_noret;
C_noret_decl(f_28551)
static void C_ccall f_28551(C_word c,C_word *av) C_noret;
C_noret_decl(f_28554)
static void C_ccall f_28554(C_word c,C_word *av) C_noret;
C_noret_decl(f_28557)
static void C_ccall f_28557(C_word c,C_word *av) C_noret;
C_noret_decl(f_28560)
static void C_ccall f_28560(C_word c,C_word *av) C_noret;
C_noret_decl(f_28563)
static void C_ccall f_28563(C_word c,C_word *av) C_noret;
C_noret_decl(f_28566)
static void C_ccall f_28566(C_word c,C_word *av) C_noret;
C_noret_decl(f_28569)
static void C_ccall f_28569(C_word c,C_word *av) C_noret;
C_noret_decl(f_28572)
static void C_ccall f_28572(C_word c,C_word *av) C_noret;
C_noret_decl(f_28576)
static void C_ccall f_28576(C_word c,C_word *av) C_noret;
C_noret_decl(f_28579)
static void C_ccall f_28579(C_word c,C_word *av) C_noret;
C_noret_decl(f_28595)
static void C_ccall f_28595(C_word c,C_word *av) C_noret;
C_noret_decl(f_28603)
static void C_ccall f_28603(C_word c,C_word *av) C_noret;
C_noret_decl(f_28606)
static void C_ccall f_28606(C_word c,C_word *av) C_noret;
C_noret_decl(f_28612)
static void C_ccall f_28612(C_word c,C_word *av) C_noret;
C_noret_decl(f_28621)
static void C_ccall f_28621(C_word c,C_word *av) C_noret;
C_noret_decl(f_28627)
static void C_ccall f_28627(C_word c,C_word *av) C_noret;
C_noret_decl(f_28636)
static void C_ccall f_28636(C_word c,C_word *av) C_noret;
C_noret_decl(f_28642)
static void C_ccall f_28642(C_word c,C_word *av) C_noret;
C_noret_decl(f_28648)
static void C_ccall f_28648(C_word c,C_word *av) C_noret;
C_noret_decl(f_28651)
static void C_ccall f_28651(C_word c,C_word *av) C_noret;
C_noret_decl(f_28655)
static void C_ccall f_28655(C_word c,C_word *av) C_noret;
C_noret_decl(f_28658)
static void C_ccall f_28658(C_word c,C_word *av) C_noret;
C_noret_decl(f_28663)
static void C_ccall f_28663(C_word c,C_word *av) C_noret;
C_noret_decl(f_28665)
static void C_ccall f_28665(C_word c,C_word *av) C_noret;
C_noret_decl(f_28669)
static void C_ccall f_28669(C_word c,C_word *av) C_noret;
C_noret_decl(f_28696)
static void C_fcall f_28696(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28706)
static void C_ccall f_28706(C_word c,C_word *av) C_noret;
C_noret_decl(f_28715)
static void C_ccall f_28715(C_word c,C_word *av) C_noret;
C_noret_decl(f_28722)
static void C_ccall f_28722(C_word c,C_word *av) C_noret;
C_noret_decl(f_28726)
static void C_ccall f_28726(C_word c,C_word *av) C_noret;
C_noret_decl(f_28740)
static void C_fcall f_28740(C_word t0,C_word t1) C_noret;
C_noret_decl(f_28744)
static void C_ccall f_28744(C_word c,C_word *av) C_noret;
C_noret_decl(f_28751)
static void C_ccall f_28751(C_word c,C_word *av) C_noret;
C_noret_decl(f_28753)
static void C_fcall f_28753(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28775)
static void C_ccall f_28775(C_word c,C_word *av) C_noret;
C_noret_decl(f_28782)
static void C_ccall f_28782(C_word c,C_word *av) C_noret;
C_noret_decl(f_28787)
static void C_fcall f_28787(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28791)
static void C_ccall f_28791(C_word c,C_word *av) C_noret;
C_noret_decl(f_28794)
static void C_ccall f_28794(C_word c,C_word *av) C_noret;
C_noret_decl(f_28797)
static void C_ccall f_28797(C_word c,C_word *av) C_noret;
C_noret_decl(f_28802)
static void C_fcall f_28802(C_word t0,C_word t1) C_noret;
C_noret_decl(f_28806)
static void C_ccall f_28806(C_word c,C_word *av) C_noret;
C_noret_decl(f_28832)
static void C_ccall f_28832(C_word c,C_word *av) C_noret;
C_noret_decl(f_28845)
static void C_ccall f_28845(C_word c,C_word *av) C_noret;
C_noret_decl(f_28848)
static void C_ccall f_28848(C_word c,C_word *av) C_noret;
C_noret_decl(f_28857)
static void C_ccall f_28857(C_word c,C_word *av) C_noret;
C_noret_decl(f_28860)
static void C_ccall f_28860(C_word c,C_word *av) C_noret;
C_noret_decl(f_28863)
static void C_ccall f_28863(C_word c,C_word *av) C_noret;
C_noret_decl(f_28869)
static void C_ccall f_28869(C_word c,C_word *av) C_noret;
C_noret_decl(f_28876)
static void C_ccall f_28876(C_word c,C_word *av) C_noret;
C_noret_decl(f_28878)
static void C_fcall f_28878(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_28891)
static void C_ccall f_28891(C_word c,C_word *av) C_noret;
C_noret_decl(f_28904)
static void C_ccall f_28904(C_word c,C_word *av) C_noret;
C_noret_decl(f_28907)
static void C_ccall f_28907(C_word c,C_word *av) C_noret;
C_noret_decl(f_28910)
static void C_ccall f_28910(C_word c,C_word *av) C_noret;
C_noret_decl(f_28917)
static void C_ccall f_28917(C_word c,C_word *av) C_noret;
C_noret_decl(f_28970)
static void C_ccall f_28970(C_word c,C_word *av) C_noret;
C_noret_decl(f_28973)
static void C_ccall f_28973(C_word c,C_word *av) C_noret;
C_noret_decl(f_28976)
static void C_ccall f_28976(C_word c,C_word *av) C_noret;
C_noret_decl(f_28978)
static void C_fcall f_28978(C_word t0,C_word t1) C_noret;
C_noret_decl(f_28982)
static void C_ccall f_28982(C_word c,C_word *av) C_noret;
C_noret_decl(f_28986)
static void C_ccall f_28986(C_word c,C_word *av) C_noret;
C_noret_decl(f_28992)
static void C_ccall f_28992(C_word c,C_word *av) C_noret;
C_noret_decl(f_28997)
static void C_fcall f_28997(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29001)
static void C_ccall f_29001(C_word c,C_word *av) C_noret;
C_noret_decl(f_29013)
static void C_ccall f_29013(C_word c,C_word *av) C_noret;
C_noret_decl(f_29025)
static void C_ccall f_29025(C_word c,C_word *av) C_noret;
C_noret_decl(f_29032)
static void C_ccall f_29032(C_word c,C_word *av) C_noret;
C_noret_decl(f_29038)
static void C_fcall f_29038(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_29052)
static void C_ccall f_29052(C_word c,C_word *av) C_noret;
C_noret_decl(f_29091)
static void C_ccall f_29091(C_word c,C_word *av) C_noret;
C_noret_decl(f_29110)
static void C_ccall f_29110(C_word c,C_word *av) C_noret;
C_noret_decl(f_29133)
static void C_ccall f_29133(C_word c,C_word *av) C_noret;
C_noret_decl(f_29142)
static void C_ccall f_29142(C_word c,C_word *av) C_noret;
C_noret_decl(f_29149)
static void C_ccall f_29149(C_word c,C_word *av) C_noret;
C_noret_decl(f_29166)
static void C_ccall f_29166(C_word c,C_word *av) C_noret;
C_noret_decl(f_29174)
static void C_ccall f_29174(C_word c,C_word *av) C_noret;
C_noret_decl(f_29185)
static void C_ccall f_29185(C_word c,C_word *av) C_noret;
C_noret_decl(f_29193)
static void C_ccall f_29193(C_word c,C_word *av) C_noret;
C_noret_decl(f_29200)
static void C_ccall f_29200(C_word c,C_word *av) C_noret;
C_noret_decl(f_29262)
static void C_ccall f_29262(C_word c,C_word *av) C_noret;
C_noret_decl(f_29269)
static void C_ccall f_29269(C_word c,C_word *av) C_noret;
C_noret_decl(f_29272)
static void C_ccall f_29272(C_word c,C_word *av) C_noret;
C_noret_decl(f_29278)
static void C_ccall f_29278(C_word c,C_word *av) C_noret;
C_noret_decl(f_29285)
static void C_ccall f_29285(C_word c,C_word *av) C_noret;
C_noret_decl(f_29289)
static void C_ccall f_29289(C_word c,C_word *av) C_noret;
C_noret_decl(f_29292)
static void C_ccall f_29292(C_word c,C_word *av) C_noret;
C_noret_decl(f_29300)
static void C_ccall f_29300(C_word c,C_word *av) C_noret;
C_noret_decl(f_29302)
static void C_ccall f_29302(C_word c,C_word *av) C_noret;
C_noret_decl(f_29311)
static void C_ccall f_29311(C_word c,C_word *av) C_noret;
C_noret_decl(f_29317)
static void C_ccall f_29317(C_word c,C_word *av) C_noret;
C_noret_decl(f_29328)
static void C_ccall f_29328(C_word c,C_word *av) C_noret;
C_noret_decl(f_29332)
static void C_ccall f_29332(C_word c,C_word *av) C_noret;
C_noret_decl(f_29334)
static void C_ccall f_29334(C_word c,C_word *av) C_noret;
C_noret_decl(f_29342)
static void C_ccall f_29342(C_word c,C_word *av) C_noret;
C_noret_decl(f_29354)
static void C_ccall f_29354(C_word c,C_word *av) C_noret;
C_noret_decl(f_29360)
static void C_fcall f_29360(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29364)
static void C_ccall f_29364(C_word c,C_word *av) C_noret;
C_noret_decl(f_29372)
static void C_fcall f_29372(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29390)
static void C_ccall f_29390(C_word c,C_word *av) C_noret;
C_noret_decl(f_29398)
static void C_ccall f_29398(C_word c,C_word *av) C_noret;
C_noret_decl(f_29402)
static void C_ccall f_29402(C_word c,C_word *av) C_noret;
C_noret_decl(f_29420)
static void C_ccall f_29420(C_word c,C_word *av) C_noret;
C_noret_decl(f_29427)
static void C_ccall f_29427(C_word c,C_word *av) C_noret;
C_noret_decl(f_29439)
static void C_ccall f_29439(C_word c,C_word *av) C_noret;
C_noret_decl(f_29447)
static void C_ccall f_29447(C_word c,C_word *av) C_noret;
C_noret_decl(f_29453)
static void C_fcall f_29453(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_29475)
static void C_ccall f_29475(C_word c,C_word *av) C_noret;
C_noret_decl(f_29479)
static void C_ccall f_29479(C_word c,C_word *av) C_noret;
C_noret_decl(f_29483)
static void C_ccall f_29483(C_word c,C_word *av) C_noret;
C_noret_decl(f_29485)
static void C_ccall f_29485(C_word c,C_word *av) C_noret;
C_noret_decl(f_29488)
static void C_ccall f_29488(C_word c,C_word *av) C_noret;
C_noret_decl(f_29491)
static void C_ccall f_29491(C_word c,C_word *av) C_noret;
C_noret_decl(f_29495)
static void C_ccall f_29495(C_word c,C_word *av) C_noret;
C_noret_decl(f_29498)
static void C_ccall f_29498(C_word c,C_word *av) C_noret;
C_noret_decl(f_29520)
static void C_ccall f_29520(C_word c,C_word *av) C_noret;
C_noret_decl(f_29526)
static void C_ccall f_29526(C_word c,C_word *av) C_noret;
C_noret_decl(f_29530)
static void C_ccall f_29530(C_word c,C_word *av) C_noret;
C_noret_decl(f_29547)
static void C_ccall f_29547(C_word c,C_word *av) C_noret;
C_noret_decl(f_29557)
static void C_ccall f_29557(C_word c,C_word *av) C_noret;
C_noret_decl(f_29561)
static void C_ccall f_29561(C_word c,C_word *av) C_noret;
C_noret_decl(f_29565)
static void C_ccall f_29565(C_word c,C_word *av) C_noret;
C_noret_decl(f_29577)
static void C_ccall f_29577(C_word c,C_word *av) C_noret;
C_noret_decl(f_29587)
static void C_ccall f_29587(C_word c,C_word *av) C_noret;
C_noret_decl(f_29591)
static void C_ccall f_29591(C_word c,C_word *av) C_noret;
C_noret_decl(f_29595)
static void C_ccall f_29595(C_word c,C_word *av) C_noret;
C_noret_decl(f_29626)
static void C_ccall f_29626(C_word c,C_word *av) C_noret;
C_noret_decl(f_29629)
static void C_ccall f_29629(C_word c,C_word *av) C_noret;
C_noret_decl(f_29632)
static void C_ccall f_29632(C_word c,C_word *av) C_noret;
C_noret_decl(f_29638)
static void C_fcall f_29638(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_29651)
static void C_ccall f_29651(C_word c,C_word *av) C_noret;
C_noret_decl(f_29654)
static void C_ccall f_29654(C_word c,C_word *av) C_noret;
C_noret_decl(f_29663)
static void C_ccall f_29663(C_word c,C_word *av) C_noret;
C_noret_decl(f_29669)
static void C_ccall f_29669(C_word c,C_word *av) C_noret;
C_noret_decl(f_29675)
static void C_ccall f_29675(C_word c,C_word *av) C_noret;
C_noret_decl(f_29681)
static void C_ccall f_29681(C_word c,C_word *av) C_noret;
C_noret_decl(f_29687)
static void C_ccall f_29687(C_word c,C_word *av) C_noret;
C_noret_decl(f_29704)
static void C_ccall f_29704(C_word c,C_word *av) C_noret;
C_noret_decl(f_29710)
static void C_ccall f_29710(C_word c,C_word *av) C_noret;
C_noret_decl(f_29725)
static void C_ccall f_29725(C_word c,C_word *av) C_noret;
C_noret_decl(f_29729)
static void C_ccall f_29729(C_word c,C_word *av) C_noret;
C_noret_decl(f_29733)
static void C_ccall f_29733(C_word c,C_word *av) C_noret;
C_noret_decl(f_29737)
static void C_ccall f_29737(C_word c,C_word *av) C_noret;
C_noret_decl(f_29758)
static void C_ccall f_29758(C_word c,C_word *av) C_noret;
C_noret_decl(f_29760)
static void C_ccall f_29760(C_word c,C_word *av) C_noret;
C_noret_decl(f_29766)
static void C_fcall f_29766(C_word t0,C_word t1) C_noret;
C_noret_decl(f_29770)
static void C_ccall f_29770(C_word c,C_word *av) C_noret;
C_noret_decl(f_29779)
static void C_ccall f_29779(C_word c,C_word *av) C_noret;
C_noret_decl(f_29788)
static void C_ccall f_29788(C_word c,C_word *av) C_noret;
C_noret_decl(f_29831)
static void C_ccall f_29831(C_word c,C_word *av) C_noret;
C_noret_decl(f_29853)
static void C_ccall f_29853(C_word c,C_word *av) C_noret;
C_noret_decl(f_29866)
static void C_ccall f_29866(C_word c,C_word *av) C_noret;
C_noret_decl(f_29891)
static void C_ccall f_29891(C_word c,C_word *av) C_noret;
C_noret_decl(f_29907)
static void C_ccall f_29907(C_word c,C_word *av) C_noret;
C_noret_decl(f_29923)
static void C_ccall f_29923(C_word c,C_word *av) C_noret;
C_noret_decl(f_29939)
static void C_ccall f_29939(C_word c,C_word *av) C_noret;
C_noret_decl(f_29955)
static void C_ccall f_29955(C_word c,C_word *av) C_noret;
C_noret_decl(f_29971)
static void C_ccall f_29971(C_word c,C_word *av) C_noret;
C_noret_decl(f_29987)
static void C_ccall f_29987(C_word c,C_word *av) C_noret;
C_noret_decl(f_30003)
static void C_ccall f_30003(C_word c,C_word *av) C_noret;
C_noret_decl(f_30019)
static void C_ccall f_30019(C_word c,C_word *av) C_noret;
C_noret_decl(f_30035)
static void C_ccall f_30035(C_word c,C_word *av) C_noret;
C_noret_decl(f_30042)
static void C_ccall f_30042(C_word c,C_word *av) C_noret;
C_noret_decl(f_30083)
static void C_ccall f_30083(C_word c,C_word *av) C_noret;
C_noret_decl(f_30095)
static void C_ccall f_30095(C_word c,C_word *av) C_noret;
C_noret_decl(f_30101)
static void C_ccall f_30101(C_word c,C_word *av) C_noret;
C_noret_decl(f_30120)
static void C_ccall f_30120(C_word c,C_word *av) C_noret;
C_noret_decl(f_30122)
static void C_ccall f_30122(C_word c,C_word *av) C_noret;
C_noret_decl(f_30145)
static C_word C_fcall f_30145(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_30191)
static void C_ccall f_30191(C_word c,C_word *av) C_noret;
C_noret_decl(f_30193)
static void C_ccall f_30193(C_word c,C_word *av) C_noret;
C_noret_decl(f_30200)
static void C_fcall f_30200(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30212)
static void C_fcall f_30212(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30256)
static void C_ccall f_30256(C_word c,C_word *av) C_noret;
C_noret_decl(f_30271)
static void C_fcall f_30271(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30279)
static void C_ccall f_30279(C_word c,C_word *av) C_noret;
C_noret_decl(f_30285)
static void C_fcall f_30285(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30293)
static void C_ccall f_30293(C_word c,C_word *av) C_noret;
C_noret_decl(f_30297)
static void C_ccall f_30297(C_word c,C_word *av) C_noret;
C_noret_decl(f_30305)
static void C_ccall f_30305(C_word c,C_word *av) C_noret;
C_noret_decl(f_30307)
static void C_fcall f_30307(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30317)
static void C_ccall f_30317(C_word c,C_word *av) C_noret;
C_noret_decl(f_30324)
static void C_ccall f_30324(C_word c,C_word *av) C_noret;
C_noret_decl(f_30328)
static void C_ccall f_30328(C_word c,C_word *av) C_noret;
C_noret_decl(f_30339)
static void C_ccall f_30339(C_word c,C_word *av) C_noret;
C_noret_decl(f_30346)
static void C_ccall f_30346(C_word c,C_word *av) C_noret;
C_noret_decl(f_30350)
static void C_ccall f_30350(C_word c,C_word *av) C_noret;
C_noret_decl(f_30361)
static void C_ccall f_30361(C_word c,C_word *av) C_noret;
C_noret_decl(f_30368)
static void C_ccall f_30368(C_word c,C_word *av) C_noret;
C_noret_decl(f_30372)
static void C_ccall f_30372(C_word c,C_word *av) C_noret;
C_noret_decl(f_30377)
static void C_ccall f_30377(C_word c,C_word *av) C_noret;
C_noret_decl(f_30383)
static void C_ccall f_30383(C_word c,C_word *av) C_noret;
C_noret_decl(f_30386)
static void C_ccall f_30386(C_word c,C_word *av) C_noret;
C_noret_decl(f_30389)
static void C_ccall f_30389(C_word c,C_word *av) C_noret;
C_noret_decl(f_30395)
static void C_ccall f_30395(C_word c,C_word *av) C_noret;
C_noret_decl(f_30404)
static void C_ccall f_30404(C_word c,C_word *av) C_noret;
C_noret_decl(f_30413)
static void C_ccall f_30413(C_word c,C_word *av) C_noret;
C_noret_decl(f_30419)
static void C_ccall f_30419(C_word c,C_word *av) C_noret;
C_noret_decl(f_30422)
static void C_ccall f_30422(C_word c,C_word *av) C_noret;
C_noret_decl(f_30425)
static void C_ccall f_30425(C_word c,C_word *av) C_noret;
C_noret_decl(f_30437)
static void C_ccall f_30437(C_word c,C_word *av) C_noret;
C_noret_decl(f_30440)
static void C_ccall f_30440(C_word c,C_word *av) C_noret;
C_noret_decl(f_30443)
static void C_ccall f_30443(C_word c,C_word *av) C_noret;
C_noret_decl(f_30446)
static void C_ccall f_30446(C_word c,C_word *av) C_noret;
C_noret_decl(f_30458)
static void C_ccall f_30458(C_word c,C_word *av) C_noret;
C_noret_decl(f_30461)
static void C_ccall f_30461(C_word c,C_word *av) C_noret;
C_noret_decl(f_30464)
static void C_ccall f_30464(C_word c,C_word *av) C_noret;
C_noret_decl(f_30467)
static void C_ccall f_30467(C_word c,C_word *av) C_noret;
C_noret_decl(f_30476)
static void C_ccall f_30476(C_word c,C_word *av) C_noret;
C_noret_decl(f_30479)
static void C_ccall f_30479(C_word c,C_word *av) C_noret;
C_noret_decl(f_30492)
static void C_ccall f_30492(C_word c,C_word *av) C_noret;
C_noret_decl(f_30493)
static void C_ccall f_30493(C_word c,C_word *av) C_noret;
C_noret_decl(f_30498)
static void C_ccall f_30498(C_word c,C_word *av) C_noret;
C_noret_decl(f_30499)
static void C_ccall f_30499(C_word c,C_word *av) C_noret;
C_noret_decl(f_30504)
static void C_ccall f_30504(C_word c,C_word *av) C_noret;
C_noret_decl(f_30505)
static void C_ccall f_30505(C_word c,C_word *av) C_noret;
C_noret_decl(f_30510)
static void C_ccall f_30510(C_word c,C_word *av) C_noret;
C_noret_decl(f_30511)
static void C_ccall f_30511(C_word c,C_word *av) C_noret;
C_noret_decl(f_30516)
static void C_ccall f_30516(C_word c,C_word *av) C_noret;
C_noret_decl(f_30517)
static void C_ccall f_30517(C_word c,C_word *av) C_noret;
C_noret_decl(f_30522)
static void C_fcall f_30522(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30524)
static void C_ccall f_30524(C_word c,C_word *av) C_noret;
C_noret_decl(f_30534)
static void C_ccall f_30534(C_word c,C_word *av) C_noret;
C_noret_decl(f_30537)
static void C_ccall f_30537(C_word c,C_word *av) C_noret;
C_noret_decl(f_30540)
static void C_ccall f_30540(C_word c,C_word *av) C_noret;
C_noret_decl(f_30543)
static void C_ccall f_30543(C_word c,C_word *av) C_noret;
C_noret_decl(f_30545)
static void C_fcall f_30545(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30559)
static void C_ccall f_30559(C_word c,C_word *av) C_noret;
C_noret_decl(f_30567)
static void C_ccall f_30567(C_word c,C_word *av) C_noret;
C_noret_decl(f_30571)
static void C_ccall f_30571(C_word c,C_word *av) C_noret;
C_noret_decl(f_30575)
static void C_ccall f_30575(C_word c,C_word *av) C_noret;
C_noret_decl(f_30579)
static void C_ccall f_30579(C_word c,C_word *av) C_noret;
C_noret_decl(f_30585)
static void C_ccall f_30585(C_word c,C_word *av) C_noret;
C_noret_decl(f_30592)
static void C_ccall f_30592(C_word c,C_word *av) C_noret;
C_noret_decl(f_30596)
static void C_ccall f_30596(C_word c,C_word *av) C_noret;
C_noret_decl(f_30600)
static void C_ccall f_30600(C_word c,C_word *av) C_noret;
C_noret_decl(f_30604)
static void C_ccall f_30604(C_word c,C_word *av) C_noret;
C_noret_decl(f_30624)
static void C_ccall f_30624(C_word c,C_word *av) C_noret;
C_noret_decl(f_30643)
static void C_ccall f_30643(C_word c,C_word *av) C_noret;
C_noret_decl(f_30646)
static void C_ccall f_30646(C_word c,C_word *av) C_noret;
C_noret_decl(f_30649)
static void C_ccall f_30649(C_word c,C_word *av) C_noret;
C_noret_decl(f_30652)
static void C_ccall f_30652(C_word c,C_word *av) C_noret;
C_noret_decl(f_30661)
static void C_ccall f_30661(C_word c,C_word *av) C_noret;
C_noret_decl(f_30665)
static void C_ccall f_30665(C_word c,C_word *av) C_noret;
C_noret_decl(f_30668)
static void C_ccall f_30668(C_word c,C_word *av) C_noret;
C_noret_decl(f_30683)
static void C_ccall f_30683(C_word c,C_word *av) C_noret;
C_noret_decl(f_30705)
static void C_ccall f_30705(C_word c,C_word *av) C_noret;
C_noret_decl(f_30711)
static void C_fcall f_30711(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_30731)
static void C_ccall f_30731(C_word c,C_word *av) C_noret;
C_noret_decl(f_30746)
static void C_ccall f_30746(C_word c,C_word *av) C_noret;
C_noret_decl(f_30750)
static void C_ccall f_30750(C_word c,C_word *av) C_noret;
C_noret_decl(f_30773)
static void C_ccall f_30773(C_word c,C_word *av) C_noret;
C_noret_decl(f_30777)
static void C_ccall f_30777(C_word c,C_word *av) C_noret;
C_noret_decl(f_30794)
static void C_ccall f_30794(C_word c,C_word *av) C_noret;
C_noret_decl(f_30801)
static void C_ccall f_30801(C_word c,C_word *av) C_noret;
C_noret_decl(f_30823)
static void C_ccall f_30823(C_word c,C_word *av) C_noret;
C_noret_decl(f_30828)
static void C_fcall f_30828(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30843)
static void C_ccall f_30843(C_word c,C_word *av) C_noret;
C_noret_decl(f_30846)
static void C_ccall f_30846(C_word c,C_word *av) C_noret;
C_noret_decl(f_30849)
static void C_ccall f_30849(C_word c,C_word *av) C_noret;
C_noret_decl(f_30852)
static void C_ccall f_30852(C_word c,C_word *av) C_noret;
C_noret_decl(f_30855)
static void C_ccall f_30855(C_word c,C_word *av) C_noret;
C_noret_decl(f_30858)
static void C_ccall f_30858(C_word c,C_word *av) C_noret;
C_noret_decl(f_30861)
static void C_fcall f_30861(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30864)
static void C_fcall f_30864(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30867)
static void C_fcall f_30867(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30870)
static void C_fcall f_30870(C_word t0,C_word t1) C_noret;
C_noret_decl(f_30878)
static void C_ccall f_30878(C_word c,C_word *av) C_noret;
C_noret_decl(f_30881)
static void C_ccall f_30881(C_word c,C_word *av) C_noret;
C_noret_decl(f_30886)
static void C_ccall f_30886(C_word c,C_word *av) C_noret;
C_noret_decl(f_30892)
static void C_ccall f_30892(C_word c,C_word *av) C_noret;
C_noret_decl(f_30905)
static void C_ccall f_30905(C_word c,C_word *av) C_noret;
C_noret_decl(f_30907)
static void C_fcall f_30907(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30930)
static void C_ccall f_30930(C_word c,C_word *av) C_noret;
C_noret_decl(f_30937)
static void C_ccall f_30937(C_word c,C_word *av) C_noret;
C_noret_decl(f_30941)
static void C_ccall f_30941(C_word c,C_word *av) C_noret;
C_noret_decl(f_30943)
static void C_fcall f_30943(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30972)
static void C_ccall f_30972(C_word c,C_word *av) C_noret;
C_noret_decl(f_30974)
static void C_fcall f_30974(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_30999)
static void C_ccall f_30999(C_word c,C_word *av) C_noret;
C_noret_decl(f_31008)
static void C_ccall f_31008(C_word c,C_word *av) C_noret;
C_noret_decl(f_31011)
static void C_ccall f_31011(C_word c,C_word *av) C_noret;
C_noret_decl(f_31017)
static void C_fcall f_31017(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_31041)
static void C_ccall f_31041(C_word c,C_word *av) C_noret;
C_noret_decl(f_31048)
static void C_ccall f_31048(C_word c,C_word *av) C_noret;
C_noret_decl(f_31052)
static void C_ccall f_31052(C_word c,C_word *av) C_noret;
C_noret_decl(f_31064)
static void C_ccall f_31064(C_word c,C_word *av) C_noret;
C_noret_decl(f_31072)
static void C_ccall f_31072(C_word c,C_word *av) C_noret;
C_noret_decl(f_31076)
static void C_ccall f_31076(C_word c,C_word *av) C_noret;
C_noret_decl(f_31085)
static void C_ccall f_31085(C_word c,C_word *av) C_noret;
C_noret_decl(f_31094)
static void C_ccall f_31094(C_word c,C_word *av) C_noret;
C_noret_decl(f_31104)
static void C_ccall f_31104(C_word c,C_word *av) C_noret;
C_noret_decl(f_31112)
static void C_ccall f_31112(C_word c,C_word *av) C_noret;
C_noret_decl(f_31116)
static void C_ccall f_31116(C_word c,C_word *av) C_noret;
C_noret_decl(f_31120)
static void C_ccall f_31120(C_word c,C_word *av) C_noret;
C_noret_decl(f_31126)
static void C_ccall f_31126(C_word c,C_word *av) C_noret;
C_noret_decl(f_31130)
static void C_ccall f_31130(C_word c,C_word *av) C_noret;
C_noret_decl(f_31134)
static void C_ccall f_31134(C_word c,C_word *av) C_noret;
C_noret_decl(f_31156)
static void C_ccall f_31156(C_word c,C_word *av) C_noret;
C_noret_decl(f_31160)
static void C_ccall f_31160(C_word c,C_word *av) C_noret;
C_noret_decl(f_31164)
static void C_ccall f_31164(C_word c,C_word *av) C_noret;
C_noret_decl(f_31168)
static void C_ccall f_31168(C_word c,C_word *av) C_noret;
C_noret_decl(f_31172)
static void C_ccall f_31172(C_word c,C_word *av) C_noret;
C_noret_decl(f_31176)
static void C_ccall f_31176(C_word c,C_word *av) C_noret;
C_noret_decl(f_31179)
static void C_ccall f_31179(C_word c,C_word *av) C_noret;
C_noret_decl(f_31182)
static void C_ccall f_31182(C_word c,C_word *av) C_noret;
C_noret_decl(f_31192)
static void C_ccall f_31192(C_word c,C_word *av) C_noret;
C_noret_decl(f_31195)
static void C_ccall f_31195(C_word c,C_word *av) C_noret;
C_noret_decl(f_31197)
static void C_ccall f_31197(C_word c,C_word *av) C_noret;
C_noret_decl(f_31214)
static C_word C_fcall f_31214(C_word t0);
C_noret_decl(f_31241)
static void C_ccall f_31241(C_word c,C_word *av) C_noret;
C_noret_decl(f_31250)
static void C_ccall f_31250(C_word c,C_word *av) C_noret;
C_noret_decl(f_31270)
static void C_ccall f_31270(C_word c,C_word *av) C_noret;
C_noret_decl(f_31274)
static void C_ccall f_31274(C_word c,C_word *av) C_noret;
C_noret_decl(f_31278)
static void C_ccall f_31278(C_word c,C_word *av) C_noret;
C_noret_decl(f_31282)
static void C_ccall f_31282(C_word c,C_word *av) C_noret;
C_noret_decl(f_31286)
static void C_ccall f_31286(C_word c,C_word *av) C_noret;
C_noret_decl(f_31290)
static void C_ccall f_31290(C_word c,C_word *av) C_noret;
C_noret_decl(f_31294)
static void C_ccall f_31294(C_word c,C_word *av) C_noret;
C_noret_decl(f_31296)
static void C_ccall f_31296(C_word c,C_word *av) C_noret;
C_noret_decl(f_31305)
static void C_ccall f_31305(C_word c,C_word *av) C_noret;
C_noret_decl(f_31315)
static void C_ccall f_31315(C_word c,C_word *av) C_noret;
C_noret_decl(f_31340)
static void C_ccall f_31340(C_word c,C_word *av) C_noret;
C_noret_decl(f_31343)
static void C_ccall f_31343(C_word c,C_word *av) C_noret;
C_noret_decl(f_31345)
static void C_ccall f_31345(C_word c,C_word *av) C_noret;
C_noret_decl(f_31361)
static void C_fcall f_31361(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_31380)
static void C_fcall f_31380(C_word t0,C_word t1) C_noret;
C_noret_decl(f_31390)
static void C_ccall f_31390(C_word c,C_word *av) C_noret;
C_noret_decl(f_31400)
static void C_ccall f_31400(C_word c,C_word *av) C_noret;
C_noret_decl(f_31424)
static void C_ccall f_31424(C_word c,C_word *av) C_noret;
C_noret_decl(f_31428)
static void C_ccall f_31428(C_word c,C_word *av) C_noret;
C_noret_decl(f_31430)
static void C_ccall f_31430(C_word c,C_word *av) C_noret;
C_noret_decl(f_31434)
static void C_ccall f_31434(C_word c,C_word *av) C_noret;
C_noret_decl(f_31437)
static void C_ccall f_31437(C_word c,C_word *av) C_noret;
C_noret_decl(f_31452)
static void C_ccall f_31452(C_word c,C_word *av) C_noret;
C_noret_decl(f_31458)
static void C_ccall f_31458(C_word c,C_word *av) C_noret;
C_noret_decl(f_31462)
static void C_ccall f_31462(C_word c,C_word *av) C_noret;
C_noret_decl(f_31471)
static void C_ccall f_31471(C_word c,C_word *av) C_noret;
C_noret_decl(f_31474)
static void C_ccall f_31474(C_word c,C_word *av) C_noret;
C_noret_decl(f_31477)
static void C_ccall f_31477(C_word c,C_word *av) C_noret;
C_noret_decl(f_31480)
static void C_ccall f_31480(C_word c,C_word *av) C_noret;
C_noret_decl(f_31483)
static void C_ccall f_31483(C_word c,C_word *av) C_noret;
C_noret_decl(f_31489)
static void C_ccall f_31489(C_word c,C_word *av) C_noret;
C_noret_decl(f_31495)
static void C_ccall f_31495(C_word c,C_word *av) C_noret;
C_noret_decl(f_31498)
static void C_ccall f_31498(C_word c,C_word *av) C_noret;
C_noret_decl(f_31503)
static void C_ccall f_31503(C_word c,C_word *av) C_noret;
C_noret_decl(f_31513)
static void C_ccall f_31513(C_word c,C_word *av) C_noret;
C_noret_decl(f_31525)
static void C_ccall f_31525(C_word c,C_word *av) C_noret;
C_noret_decl(f_31533)
static void C_fcall f_31533(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_31543)
static void C_ccall f_31543(C_word c,C_word *av) C_noret;
C_noret_decl(f_31559)
static void C_ccall f_31559(C_word c,C_word *av) C_noret;
C_noret_decl(f_31565)
static void C_ccall f_31565(C_word c,C_word *av) C_noret;
C_noret_decl(f_31568)
static void C_ccall f_31568(C_word c,C_word *av) C_noret;
C_noret_decl(f_31571)
static void C_ccall f_31571(C_word c,C_word *av) C_noret;
C_noret_decl(f_31572)
static void C_fcall f_31572(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_31576)
static void C_ccall f_31576(C_word c,C_word *av) C_noret;
C_noret_decl(f_31582)
static void C_ccall f_31582(C_word c,C_word *av) C_noret;
C_noret_decl(f_31586)
static void C_ccall f_31586(C_word c,C_word *av) C_noret;
C_noret_decl(f_31588)
static void C_fcall f_31588(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_31598)
static void C_ccall f_31598(C_word c,C_word *av) C_noret;
C_noret_decl(f_31615)
static void C_ccall f_31615(C_word c,C_word *av) C_noret;
C_noret_decl(f_31618)
static void C_ccall f_31618(C_word c,C_word *av) C_noret;
C_noret_decl(f_31622)
static void C_ccall f_31622(C_word c,C_word *av) C_noret;
C_noret_decl(f_31627)
static void C_ccall f_31627(C_word c,C_word *av) C_noret;
C_noret_decl(f_31633)
static void C_ccall f_31633(C_word c,C_word *av) C_noret;
C_noret_decl(f_31640)
static void C_ccall f_31640(C_word c,C_word *av) C_noret;
C_noret_decl(f_31649)
static void C_ccall f_31649(C_word c,C_word *av) C_noret;
C_noret_decl(f_31657)
static void C_ccall f_31657(C_word c,C_word *av) C_noret;
C_noret_decl(f_31682)
static void C_ccall f_31682(C_word c,C_word *av) C_noret;
C_noret_decl(f_31700)
static void C_ccall f_31700(C_word c,C_word *av) C_noret;
C_noret_decl(f_31704)
static void C_ccall f_31704(C_word c,C_word *av) C_noret;
C_noret_decl(f_31719)
static void C_ccall f_31719(C_word c,C_word *av) C_noret;
C_noret_decl(f_31723)
static void C_ccall f_31723(C_word c,C_word *av) C_noret;
C_noret_decl(f_31738)
static void C_ccall f_31738(C_word c,C_word *av) C_noret;
C_noret_decl(f_31741)
static void C_ccall f_31741(C_word c,C_word *av) C_noret;
C_noret_decl(f_31744)
static void C_ccall f_31744(C_word c,C_word *av) C_noret;
C_noret_decl(f_31754)
static void C_fcall f_31754(C_word t0,C_word t1) C_noret;
C_noret_decl(f_31785)
static void C_ccall f_31785(C_word c,C_word *av) C_noret;
C_noret_decl(f_31809)
static void C_ccall f_31809(C_word c,C_word *av) C_noret;
C_noret_decl(f_31819)
static void C_ccall f_31819(C_word c,C_word *av) C_noret;
C_noret_decl(f_31828)
static void C_ccall f_31828(C_word c,C_word *av) C_noret;
C_noret_decl(f_31869)
static void C_ccall f_31869(C_word c,C_word *av) C_noret;
C_noret_decl(f_31888)
static void C_ccall f_31888(C_word c,C_word *av) C_noret;
C_noret_decl(f_31897)
static void C_ccall f_31897(C_word c,C_word *av) C_noret;
C_noret_decl(f_31901)
static void C_ccall f_31901(C_word c,C_word *av) C_noret;
C_noret_decl(f_31906)
static void C_ccall f_31906(C_word c,C_word *av) C_noret;
C_noret_decl(f_31916)
static void C_ccall f_31916(C_word c,C_word *av) C_noret;
C_noret_decl(f_31926)
static void C_ccall f_31926(C_word c,C_word *av) C_noret;
C_noret_decl(f_31936)
static void C_ccall f_31936(C_word c,C_word *av) C_noret;
C_noret_decl(f_31946)
static void C_ccall f_31946(C_word c,C_word *av) C_noret;
C_noret_decl(f_31956)
static void C_ccall f_31956(C_word c,C_word *av) C_noret;
C_noret_decl(f_31966)
static void C_ccall f_31966(C_word c,C_word *av) C_noret;
C_noret_decl(f_31976)
static void C_ccall f_31976(C_word c,C_word *av) C_noret;
C_noret_decl(f_31986)
static void C_ccall f_31986(C_word c,C_word *av) C_noret;
C_noret_decl(f_31996)
static void C_ccall f_31996(C_word c,C_word *av) C_noret;
C_noret_decl(f_32006)
static void C_ccall f_32006(C_word c,C_word *av) C_noret;
C_noret_decl(f_32016)
static void C_ccall f_32016(C_word c,C_word *av) C_noret;
C_noret_decl(f_32026)
static void C_ccall f_32026(C_word c,C_word *av) C_noret;
C_noret_decl(f_32036)
static void C_ccall f_32036(C_word c,C_word *av) C_noret;
C_noret_decl(f_32052)
static void C_ccall f_32052(C_word c,C_word *av) C_noret;
C_noret_decl(f_32060)
static void C_ccall f_32060(C_word c,C_word *av) C_noret;
C_noret_decl(f_32076)
static void C_ccall f_32076(C_word c,C_word *av) C_noret;
C_noret_decl(f_32092)
static void C_ccall f_32092(C_word c,C_word *av) C_noret;
C_noret_decl(f_32102)
static void C_ccall f_32102(C_word c,C_word *av) C_noret;
C_noret_decl(f_32112)
static void C_ccall f_32112(C_word c,C_word *av) C_noret;
C_noret_decl(f_32131)
static void C_ccall f_32131(C_word c,C_word *av) C_noret;
C_noret_decl(f_32147)
static void C_ccall f_32147(C_word c,C_word *av) C_noret;
C_noret_decl(f_32160)
static void C_ccall f_32160(C_word c,C_word *av) C_noret;
C_noret_decl(f_32166)
static void C_ccall f_32166(C_word c,C_word *av) C_noret;
C_noret_decl(f_32189)
static void C_ccall f_32189(C_word c,C_word *av) C_noret;
C_noret_decl(f_32193)
static void C_ccall f_32193(C_word c,C_word *av) C_noret;
C_noret_decl(f_32197)
static void C_ccall f_32197(C_word c,C_word *av) C_noret;
C_noret_decl(f_32200)
static void C_ccall f_32200(C_word c,C_word *av) C_noret;
C_noret_decl(f_32206)
static void C_ccall f_32206(C_word c,C_word *av) C_noret;
C_noret_decl(f_32229)
static void C_ccall f_32229(C_word c,C_word *av) C_noret;
C_noret_decl(f_32233)
static void C_ccall f_32233(C_word c,C_word *av) C_noret;
C_noret_decl(f_32237)
static void C_ccall f_32237(C_word c,C_word *av) C_noret;
C_noret_decl(f_32240)
static void C_ccall f_32240(C_word c,C_word *av) C_noret;
C_noret_decl(f_32243)
static void C_ccall f_32243(C_word c,C_word *av) C_noret;
C_noret_decl(f_32246)
static void C_ccall f_32246(C_word c,C_word *av) C_noret;
C_noret_decl(f_32252)
static void C_ccall f_32252(C_word c,C_word *av) C_noret;
C_noret_decl(f_32255)
static void C_ccall f_32255(C_word c,C_word *av) C_noret;
C_noret_decl(f_32258)
static void C_ccall f_32258(C_word c,C_word *av) C_noret;
C_noret_decl(f_32268)
static void C_fcall f_32268(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_32292)
static void C_ccall f_32292(C_word c,C_word *av) C_noret;
C_noret_decl(f_32315)
static void C_ccall f_32315(C_word c,C_word *av) C_noret;
C_noret_decl(f_32319)
static void C_ccall f_32319(C_word c,C_word *av) C_noret;
C_noret_decl(f_32323)
static void C_ccall f_32323(C_word c,C_word *av) C_noret;
C_noret_decl(f_32357)
static void C_ccall f_32357(C_word c,C_word *av) C_noret;
C_noret_decl(f_32361)
static void C_ccall f_32361(C_word c,C_word *av) C_noret;
C_noret_decl(f_32364)
static void C_fcall f_32364(C_word t0,C_word t1) C_noret;
C_noret_decl(f_32377)
static void C_fcall f_32377(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_32410)
static void C_ccall f_32410(C_word c,C_word *av) C_noret;
C_noret_decl(f_32425)
static void C_ccall f_32425(C_word c,C_word *av) C_noret;
C_noret_decl(f_32429)
static void C_ccall f_32429(C_word c,C_word *av) C_noret;
C_noret_decl(f_32453)
static void C_ccall f_32453(C_word c,C_word *av) C_noret;
C_noret_decl(f_32461)
static void C_ccall f_32461(C_word c,C_word *av) C_noret;
C_noret_decl(f_32465)
static void C_ccall f_32465(C_word c,C_word *av) C_noret;
C_noret_decl(f_32476)
static void C_ccall f_32476(C_word c,C_word *av) C_noret;
C_noret_decl(f_32480)
static void C_ccall f_32480(C_word c,C_word *av) C_noret;
C_noret_decl(f_32484)
static void C_ccall f_32484(C_word c,C_word *av) C_noret;
C_noret_decl(f_32498)
static void C_ccall f_32498(C_word c,C_word *av) C_noret;
C_noret_decl(f_32530)
static void C_ccall f_32530(C_word c,C_word *av) C_noret;
C_noret_decl(f_8571)
static void C_ccall f_8571(C_word c,C_word *av) C_noret;
C_noret_decl(f_8573)
static void C_ccall f_8573(C_word c,C_word *av) C_noret;
C_noret_decl(f_8576)
static void C_ccall f_8576(C_word c,C_word *av) C_noret;
C_noret_decl(f_8579)
static void C_ccall f_8579(C_word c,C_word *av) C_noret;
C_noret_decl(f_8582)
static void C_ccall f_8582(C_word c,C_word *av) C_noret;
C_noret_decl(f_8585)
static void C_ccall f_8585(C_word c,C_word *av) C_noret;
C_noret_decl(f_8588)
static void C_ccall f_8588(C_word c,C_word *av) C_noret;
C_noret_decl(f_8591)
static void C_ccall f_8591(C_word c,C_word *av) C_noret;
C_noret_decl(f_8594)
static void C_ccall f_8594(C_word c,C_word *av) C_noret;
C_noret_decl(f_8597)
static void C_ccall f_8597(C_word c,C_word *av) C_noret;
C_noret_decl(f_8600)
static void C_ccall f_8600(C_word c,C_word *av) C_noret;
C_noret_decl(f_8603)
static void C_ccall f_8603(C_word c,C_word *av) C_noret;
C_noret_decl(f_8606)
static void C_ccall f_8606(C_word c,C_word *av) C_noret;
C_noret_decl(f_8609)
static void C_ccall f_8609(C_word c,C_word *av) C_noret;
C_noret_decl(f_8612)
static void C_ccall f_8612(C_word c,C_word *av) C_noret;
C_noret_decl(f_8615)
static void C_ccall f_8615(C_word c,C_word *av) C_noret;
C_noret_decl(f_8618)
static void C_ccall f_8618(C_word c,C_word *av) C_noret;
C_noret_decl(f_8621)
static void C_ccall f_8621(C_word c,C_word *av) C_noret;
C_noret_decl(f_8624)
static void C_ccall f_8624(C_word c,C_word *av) C_noret;
C_noret_decl(f_8627)
static void C_ccall f_8627(C_word c,C_word *av) C_noret;
C_noret_decl(f_8641)
static void C_ccall f_8641(C_word c,C_word *av) C_noret;
C_noret_decl(f_8648)
static void C_ccall f_8648(C_word c,C_word *av) C_noret;
C_noret_decl(f_8655)
static void C_ccall f_8655(C_word c,C_word *av) C_noret;
C_noret_decl(f_8669)
static void C_ccall f_8669(C_word c,C_word *av) C_noret;
C_noret_decl(f_8676)
static void C_ccall f_8676(C_word c,C_word *av) C_noret;
C_noret_decl(f_8690)
static void C_ccall f_8690(C_word c,C_word *av) C_noret;
C_noret_decl(f_8704)
static void C_ccall f_8704(C_word c,C_word *av) C_noret;
C_noret_decl(f_8722)
static void C_ccall f_8722(C_word c,C_word *av) C_noret;
C_noret_decl(f_8733)
static void C_ccall f_8733(C_word c,C_word *av) C_noret;
C_noret_decl(f_8744)
static void C_ccall f_8744(C_word c,C_word *av) C_noret;
C_noret_decl(f_8751)
static void C_ccall f_8751(C_word c,C_word *av) C_noret;
C_noret_decl(f_8762)
static void C_ccall f_8762(C_word c,C_word *av) C_noret;
C_noret_decl(f_8766)
static void C_ccall f_8766(C_word c,C_word *av) C_noret;
C_noret_decl(f_8773)
static void C_ccall f_8773(C_word c,C_word *av) C_noret;
C_noret_decl(f_8791)
static void C_ccall f_8791(C_word c,C_word *av) C_noret;
C_noret_decl(f_8802)
static void C_ccall f_8802(C_word c,C_word *av) C_noret;
C_noret_decl(f_8813)
static void C_ccall f_8813(C_word c,C_word *av) C_noret;
C_noret_decl(f_8820)
static void C_ccall f_8820(C_word c,C_word *av) C_noret;
C_noret_decl(f_8838)
static void C_ccall f_8838(C_word c,C_word *av) C_noret;
C_noret_decl(f_8849)
static void C_ccall f_8849(C_word c,C_word *av) C_noret;
C_noret_decl(f_8867)
static void C_ccall f_8867(C_word c,C_word *av) C_noret;
C_noret_decl(f_8873)
static void C_ccall f_8873(C_word c,C_word *av) C_noret;
C_noret_decl(f_8876)
static void C_ccall f_8876(C_word c,C_word *av) C_noret;
C_noret_decl(f_8879)
static void C_ccall f_8879(C_word c,C_word *av) C_noret;
C_noret_decl(f_8882)
static void C_ccall f_8882(C_word c,C_word *av) C_noret;
C_noret_decl(f_8885)
static void C_ccall f_8885(C_word c,C_word *av) C_noret;
C_noret_decl(f_8891)
static void C_fcall f_8891(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8924)
static void C_ccall f_8924(C_word c,C_word *av) C_noret;
C_noret_decl(f_8927)
static void C_ccall f_8927(C_word c,C_word *av) C_noret;
C_noret_decl(f_8930)
static void C_ccall f_8930(C_word c,C_word *av) C_noret;
C_noret_decl(f_8933)
static void C_ccall f_8933(C_word c,C_word *av) C_noret;
C_noret_decl(f_8936)
static void C_ccall f_8936(C_word c,C_word *av) C_noret;
C_noret_decl(f_8939)
static void C_ccall f_8939(C_word c,C_word *av) C_noret;
C_noret_decl(f_8942)
static void C_ccall f_8942(C_word c,C_word *av) C_noret;
C_noret_decl(f_8945)
static void C_ccall f_8945(C_word c,C_word *av) C_noret;
C_noret_decl(f_8948)
static void C_ccall f_8948(C_word c,C_word *av) C_noret;
C_noret_decl(f_8951)
static void C_ccall f_8951(C_word c,C_word *av) C_noret;
C_noret_decl(f_8954)
static void C_ccall f_8954(C_word c,C_word *av) C_noret;
C_noret_decl(f_8957)
static void C_ccall f_8957(C_word c,C_word *av) C_noret;
C_noret_decl(f_8960)
static void C_ccall f_8960(C_word c,C_word *av) C_noret;
C_noret_decl(f_8964)
static void C_ccall f_8964(C_word c,C_word *av) C_noret;
C_noret_decl(f_8967)
static void C_ccall f_8967(C_word c,C_word *av) C_noret;
C_noret_decl(f_8973)
static void C_ccall f_8973(C_word c,C_word *av) C_noret;
C_noret_decl(f_8976)
static void C_ccall f_8976(C_word c,C_word *av) C_noret;
C_noret_decl(f_8979)
static void C_ccall f_8979(C_word c,C_word *av) C_noret;
C_noret_decl(f_8982)
static void C_ccall f_8982(C_word c,C_word *av) C_noret;
C_noret_decl(f_8985)
static void C_ccall f_8985(C_word c,C_word *av) C_noret;
C_noret_decl(f_8993)
static void C_ccall f_8993(C_word c,C_word *av) C_noret;
C_noret_decl(f_8997)
static void C_ccall f_8997(C_word c,C_word *av) C_noret;
C_noret_decl(f_9000)
static void C_ccall f_9000(C_word c,C_word *av) C_noret;
C_noret_decl(f_9003)
static void C_ccall f_9003(C_word c,C_word *av) C_noret;
C_noret_decl(f_9006)
static void C_ccall f_9006(C_word c,C_word *av) C_noret;
C_noret_decl(f_9009)
static void C_ccall f_9009(C_word c,C_word *av) C_noret;
C_noret_decl(f_9012)
static void C_ccall f_9012(C_word c,C_word *av) C_noret;
C_noret_decl(f_9015)
static void C_ccall f_9015(C_word c,C_word *av) C_noret;
C_noret_decl(f_9018)
static void C_ccall f_9018(C_word c,C_word *av) C_noret;
C_noret_decl(f_9022)
static void C_ccall f_9022(C_word c,C_word *av) C_noret;
C_noret_decl(f_9025)
static void C_ccall f_9025(C_word c,C_word *av) C_noret;
C_noret_decl(f_9028)
static void C_ccall f_9028(C_word c,C_word *av) C_noret;
C_noret_decl(f_9031)
static void C_ccall f_9031(C_word c,C_word *av) C_noret;
C_noret_decl(f_9034)
static void C_ccall f_9034(C_word c,C_word *av) C_noret;
C_noret_decl(f_9042)
static void C_ccall f_9042(C_word c,C_word *av) C_noret;
C_noret_decl(f_9045)
static void C_ccall f_9045(C_word c,C_word *av) C_noret;
C_noret_decl(f_9052)
static void C_ccall f_9052(C_word c,C_word *av) C_noret;
C_noret_decl(f_9056)
static void C_ccall f_9056(C_word c,C_word *av) C_noret;
C_noret_decl(f_9059)
static void C_ccall f_9059(C_word c,C_word *av) C_noret;
C_noret_decl(f_9062)
static void C_ccall f_9062(C_word c,C_word *av) C_noret;
C_noret_decl(f_9065)
static void C_ccall f_9065(C_word c,C_word *av) C_noret;
C_noret_decl(f_9068)
static void C_ccall f_9068(C_word c,C_word *av) C_noret;
C_noret_decl(f_9071)
static void C_ccall f_9071(C_word c,C_word *av) C_noret;
C_noret_decl(f_9074)
static void C_ccall f_9074(C_word c,C_word *av) C_noret;
C_noret_decl(f_9080)
static void C_ccall f_9080(C_word c,C_word *av) C_noret;
C_noret_decl(f_9091)
static void C_ccall f_9091(C_word c,C_word *av) C_noret;
C_noret_decl(f_9097)
static void C_ccall f_9097(C_word c,C_word *av) C_noret;
C_noret_decl(f_9103)
static void C_ccall f_9103(C_word c,C_word *av) C_noret;
C_noret_decl(f_9109)
static void C_ccall f_9109(C_word c,C_word *av) C_noret;
C_noret_decl(f_9115)
static void C_ccall f_9115(C_word c,C_word *av) C_noret;
C_noret_decl(f_9121)
static void C_ccall f_9121(C_word c,C_word *av) C_noret;
C_noret_decl(f_9128)
static void C_ccall f_9128(C_word c,C_word *av) C_noret;
C_noret_decl(f_9134)
static void C_ccall f_9134(C_word c,C_word *av) C_noret;
C_noret_decl(f_9150)
static void C_ccall f_9150(C_word c,C_word *av) C_noret;
C_noret_decl(f_9161)
static void C_ccall f_9161(C_word c,C_word *av) C_noret;
C_noret_decl(f_9167)
static void C_ccall f_9167(C_word c,C_word *av) C_noret;
C_noret_decl(f_9173)
static void C_ccall f_9173(C_word c,C_word *av) C_noret;
C_noret_decl(f_9179)
static void C_ccall f_9179(C_word c,C_word *av) C_noret;
C_noret_decl(f_9185)
static void C_ccall f_9185(C_word c,C_word *av) C_noret;
C_noret_decl(f_9194)
static void C_ccall f_9194(C_word c,C_word *av) C_noret;
C_noret_decl(f_9197)
static void C_ccall f_9197(C_word c,C_word *av) C_noret;
C_noret_decl(f_9200)
static void C_ccall f_9200(C_word c,C_word *av) C_noret;
C_noret_decl(f_9203)
static void C_ccall f_9203(C_word c,C_word *av) C_noret;
C_noret_decl(f_9206)
static void C_ccall f_9206(C_word c,C_word *av) C_noret;
C_noret_decl(f_9209)
static void C_ccall f_9209(C_word c,C_word *av) C_noret;
C_noret_decl(f_9213)
static void C_ccall f_9213(C_word c,C_word *av) C_noret;
C_noret_decl(f_9219)
static void C_ccall f_9219(C_word c,C_word *av) C_noret;
C_noret_decl(f_9225)
static void C_ccall f_9225(C_word c,C_word *av) C_noret;
C_noret_decl(f_9260)
static void C_ccall f_9260(C_word c,C_word *av) C_noret;
C_noret_decl(f_9263)
static void C_ccall f_9263(C_word c,C_word *av) C_noret;
C_noret_decl(f_9266)
static void C_ccall f_9266(C_word c,C_word *av) C_noret;
C_noret_decl(f_9274)
static void C_ccall f_9274(C_word c,C_word *av) C_noret;
C_noret_decl(f_9280)
static void C_ccall f_9280(C_word c,C_word *av) C_noret;
C_noret_decl(f_9287)
static void C_ccall f_9287(C_word c,C_word *av) C_noret;
C_noret_decl(f_9302)
static void C_ccall f_9302(C_word c,C_word *av) C_noret;
C_noret_decl(f_9325)
static void C_ccall f_9325(C_word c,C_word *av) C_noret;
C_noret_decl(f_9331)
static void C_ccall f_9331(C_word c,C_word *av) C_noret;
C_noret_decl(f_9337)
static void C_ccall f_9337(C_word c,C_word *av) C_noret;
C_noret_decl(f_9344)
static void C_ccall f_9344(C_word c,C_word *av) C_noret;
C_noret_decl(f_9349)
static void C_ccall f_9349(C_word c,C_word *av) C_noret;
C_noret_decl(f_9352)
static void C_ccall f_9352(C_word c,C_word *av) C_noret;
C_noret_decl(f_9361)
static void C_fcall f_9361(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9379)
static void C_ccall f_9379(C_word c,C_word *av) C_noret;
C_noret_decl(f_9389)
static void C_ccall f_9389(C_word c,C_word *av) C_noret;
C_noret_decl(f_9398)
static void C_fcall f_9398(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9416)
static void C_ccall f_9416(C_word c,C_word *av) C_noret;
C_noret_decl(f_9429)
static void C_ccall f_9429(C_word c,C_word *av) C_noret;
C_noret_decl(f_9436)
static void C_ccall f_9436(C_word c,C_word *av) C_noret;
C_noret_decl(f_9443)
static void C_ccall f_9443(C_word c,C_word *av) C_noret;
C_noret_decl(f_9454)
static void C_ccall f_9454(C_word c,C_word *av) C_noret;
C_noret_decl(f_9461)
static void C_ccall f_9461(C_word c,C_word *av) C_noret;
C_noret_decl(f_9473)
static void C_ccall f_9473(C_word c,C_word *av) C_noret;
C_noret_decl(f_9476)
static void C_ccall f_9476(C_word c,C_word *av) C_noret;
C_noret_decl(f_9478)
static void C_ccall f_9478(C_word c,C_word *av) C_noret;
C_noret_decl(f_9484)
static void C_fcall f_9484(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9500)
static void C_ccall f_9500(C_word c,C_word *av) C_noret;
C_noret_decl(f_9509)
static void C_ccall f_9509(C_word c,C_word *av) C_noret;
C_noret_decl(f_9511)
static void C_ccall f_9511(C_word c,C_word *av) C_noret;
C_noret_decl(f_9517)
static void C_fcall f_9517(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9530)
static void C_ccall f_9530(C_word c,C_word *av) C_noret;
C_noret_decl(f_9546)
static void C_ccall f_9546(C_word c,C_word *av) C_noret;
C_noret_decl(f_9557)
static void C_ccall f_9557(C_word c,C_word *av) C_noret;
C_noret_decl(f_9559)
static void C_ccall f_9559(C_word c,C_word *av) C_noret;
C_noret_decl(f_9567)
static void C_ccall f_9567(C_word c,C_word *av) C_noret;
C_noret_decl(f_9569)
static void C_ccall f_9569(C_word c,C_word *av) C_noret;
C_noret_decl(f_9575)
static void C_ccall f_9575(C_word c,C_word *av) C_noret;
C_noret_decl(f_9577)
static void C_ccall f_9577(C_word c,C_word *av) C_noret;
C_noret_decl(f_9585)
static void C_ccall f_9585(C_word c,C_word *av) C_noret;
C_noret_decl(f_9587)
static void C_ccall f_9587(C_word c,C_word *av) C_noret;
C_noret_decl(f_9590)
static void C_ccall f_9590(C_word c,C_word *av) C_noret;
C_noret_decl(f_9598)
static void C_ccall f_9598(C_word c,C_word *av) C_noret;
C_noret_decl(f_9604)
static void C_ccall f_9604(C_word c,C_word *av) C_noret;
C_noret_decl(f_9612)
static void C_ccall f_9612(C_word c,C_word *av) C_noret;
C_noret_decl(f_9623)
static void C_ccall f_9623(C_word c,C_word *av) C_noret;
C_noret_decl(f_9635)
static void C_fcall f_9635(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9649)
static void C_ccall f_9649(C_word c,C_word *av) C_noret;
C_noret_decl(f_9657)
static void C_ccall f_9657(C_word c,C_word *av) C_noret;
C_noret_decl(f_9660)
static void C_ccall f_9660(C_word c,C_word *av) C_noret;
C_noret_decl(f_9662)
static void C_ccall f_9662(C_word c,C_word *av) C_noret;
C_noret_decl(f_9664)
static void C_ccall f_9664(C_word c,C_word *av) C_noret;
C_noret_decl(f_9670)
static void C_fcall f_9670(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9689)
static void C_ccall f_9689(C_word c,C_word *av) C_noret;
C_noret_decl(f_9706)
static void C_ccall f_9706(C_word c,C_word *av) C_noret;
C_noret_decl(f_9714)
static void C_ccall f_9714(C_word c,C_word *av) C_noret;
C_noret_decl(f_9728)
static void C_ccall f_9728(C_word c,C_word *av) C_noret;
C_noret_decl(f_9734)
static void C_fcall f_9734(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9753)
static void C_ccall f_9753(C_word c,C_word *av) C_noret;
C_noret_decl(f_9762)
static void C_ccall f_9762(C_word c,C_word *av) C_noret;
C_noret_decl(f_9765)
static void C_ccall f_9765(C_word c,C_word *av) C_noret;
C_noret_decl(f_9777)
static C_word C_fcall f_9777(C_word t0,C_word t1);
C_noret_decl(f_9793)
static void C_ccall f_9793(C_word c,C_word *av) C_noret;
C_noret_decl(f_9799)
static void C_fcall f_9799(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9824)
static void C_ccall f_9824(C_word c,C_word *av) C_noret;
C_noret_decl(f_9826)
static void C_ccall f_9826(C_word c,C_word *av) C_noret;
C_noret_decl(f_9835)
static void C_fcall f_9835(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9856)
static void C_ccall f_9856(C_word c,C_word *av) C_noret;
C_noret_decl(f_9858)
static void C_ccall f_9858(C_word c,C_word *av) C_noret;
C_noret_decl(f_9864)
static void C_fcall f_9864(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9890)
static void C_ccall f_9890(C_word c,C_word *av) C_noret;
C_noret_decl(f_9897)
static void C_ccall f_9897(C_word c,C_word *av) C_noret;
C_noret_decl(f_9899)
static void C_ccall f_9899(C_word c,C_word *av) C_noret;
C_noret_decl(f_9911)
static void C_fcall f_9911(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9946)
static void C_ccall f_9946(C_word c,C_word *av) C_noret;
C_noret_decl(f_9956)
static void C_ccall f_9956(C_word c,C_word *av) C_noret;
C_noret_decl(f_9965)
static void C_fcall f_9965(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(C_library_toplevel)
C_externexport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

C_noret_decl(trf_10043)
static void C_ccall trf_10043(C_word c,C_word *av) C_noret;
static void C_ccall trf_10043(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10043(t0,t1);}

C_noret_decl(trf_10081)
static void C_ccall trf_10081(C_word c,C_word *av) C_noret;
static void C_ccall trf_10081(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10081(t0,t1,t2);}

C_noret_decl(trf_10131)
static void C_ccall trf_10131(C_word c,C_word *av) C_noret;
static void C_ccall trf_10131(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10131(t0,t1,t2);}

C_noret_decl(trf_10242)
static void C_ccall trf_10242(C_word c,C_word *av) C_noret;
static void C_ccall trf_10242(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10242(t0,t1);}

C_noret_decl(trf_10273)
static void C_ccall trf_10273(C_word c,C_word *av) C_noret;
static void C_ccall trf_10273(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10273(t0,t1,t2);}

C_noret_decl(trf_10350)
static void C_ccall trf_10350(C_word c,C_word *av) C_noret;
static void C_ccall trf_10350(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10350(t0,t1,t2);}

C_noret_decl(trf_10961)
static void C_ccall trf_10961(C_word c,C_word *av) C_noret;
static void C_ccall trf_10961(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_10961(t0,t1,t2,t3);}

C_noret_decl(trf_11010)
static void C_ccall trf_11010(C_word c,C_word *av) C_noret;
static void C_ccall trf_11010(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11010(t0,t1);}

C_noret_decl(trf_11217)
static void C_ccall trf_11217(C_word c,C_word *av) C_noret;
static void C_ccall trf_11217(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11217(t0,t1,t2);}

C_noret_decl(trf_11236)
static void C_ccall trf_11236(C_word c,C_word *av) C_noret;
static void C_ccall trf_11236(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11236(t0,t1,t2);}

C_noret_decl(trf_11290)
static void C_ccall trf_11290(C_word c,C_word *av) C_noret;
static void C_ccall trf_11290(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11290(t0,t1,t2,t3);}

C_noret_decl(trf_11386)
static void C_ccall trf_11386(C_word c,C_word *av) C_noret;
static void C_ccall trf_11386(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11386(t0,t1,t2,t3);}

C_noret_decl(trf_11544)
static void C_ccall trf_11544(C_word c,C_word *av) C_noret;
static void C_ccall trf_11544(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11544(t0,t1);}

C_noret_decl(trf_11553)
static void C_ccall trf_11553(C_word c,C_word *av) C_noret;
static void C_ccall trf_11553(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11553(t0,t1);}

C_noret_decl(trf_11836)
static void C_ccall trf_11836(C_word c,C_word *av) C_noret;
static void C_ccall trf_11836(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11836(t0,t1,t2,t3);}

C_noret_decl(trf_11925)
static void C_ccall trf_11925(C_word c,C_word *av) C_noret;
static void C_ccall trf_11925(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11925(t0,t1,t2,t3);}

C_noret_decl(trf_11946)
static void C_ccall trf_11946(C_word c,C_word *av) C_noret;
static void C_ccall trf_11946(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_11946(t0,t1,t2,t3,t4);}

C_noret_decl(trf_12675)
static void C_ccall trf_12675(C_word c,C_word *av) C_noret;
static void C_ccall trf_12675(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12675(t0,t1,t2);}

C_noret_decl(trf_13034)
static void C_ccall trf_13034(C_word c,C_word *av) C_noret;
static void C_ccall trf_13034(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13034(t0,t1,t2,t3);}

C_noret_decl(trf_13051)
static void C_ccall trf_13051(C_word c,C_word *av) C_noret;
static void C_ccall trf_13051(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13051(t0,t1,t2,t3);}

C_noret_decl(trf_13218)
static void C_ccall trf_13218(C_word c,C_word *av) C_noret;
static void C_ccall trf_13218(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13218(t0,t1,t2,t3);}

C_noret_decl(trf_13700)
static void C_ccall trf_13700(C_word c,C_word *av) C_noret;
static void C_ccall trf_13700(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13700(t0,t1,t2);}

C_noret_decl(trf_13703)
static void C_ccall trf_13703(C_word c,C_word *av) C_noret;
static void C_ccall trf_13703(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13703(t0,t1,t2,t3);}

C_noret_decl(trf_13882)
static void C_ccall trf_13882(C_word c,C_word *av) C_noret;
static void C_ccall trf_13882(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13882(t0,t1,t2,t3,t4);}

C_noret_decl(trf_13926)
static void C_ccall trf_13926(C_word c,C_word *av) C_noret;
static void C_ccall trf_13926(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_13926(t0,t1,t2,t3,t4);}

C_noret_decl(trf_14189)
static void C_ccall trf_14189(C_word c,C_word *av) C_noret;
static void C_ccall trf_14189(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14189(t0,t1);}

C_noret_decl(trf_14195)
static void C_ccall trf_14195(C_word c,C_word *av) C_noret;
static void C_ccall trf_14195(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14195(t0,t1);}

C_noret_decl(trf_14251)
static void C_ccall trf_14251(C_word c,C_word *av) C_noret;
static void C_ccall trf_14251(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14251(t0,t1);}

C_noret_decl(trf_14257)
static void C_ccall trf_14257(C_word c,C_word *av) C_noret;
static void C_ccall trf_14257(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14257(t0,t1);}

C_noret_decl(trf_14730)
static void C_ccall trf_14730(C_word c,C_word *av) C_noret;
static void C_ccall trf_14730(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14730(t0,t1);}

C_noret_decl(trf_14765)
static void C_ccall trf_14765(C_word c,C_word *av) C_noret;
static void C_ccall trf_14765(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14765(t0,t1,t2,t3);}

C_noret_decl(trf_14875)
static void C_ccall trf_14875(C_word c,C_word *av) C_noret;
static void C_ccall trf_14875(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_14875(t0,t1,t2,t3);}

C_noret_decl(trf_14917)
static void C_ccall trf_14917(C_word c,C_word *av) C_noret;
static void C_ccall trf_14917(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14917(t0,t1,t2);}

C_noret_decl(trf_14931)
static void C_ccall trf_14931(C_word c,C_word *av) C_noret;
static void C_ccall trf_14931(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14931(t0,t1,t2);}

C_noret_decl(trf_15085)
static void C_ccall trf_15085(C_word c,C_word *av) C_noret;
static void C_ccall trf_15085(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15085(t0,t1);}

C_noret_decl(trf_15181)
static void C_ccall trf_15181(C_word c,C_word *av) C_noret;
static void C_ccall trf_15181(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15181(t0,t1,t2,t3);}

C_noret_decl(trf_15277)
static void C_ccall trf_15277(C_word c,C_word *av) C_noret;
static void C_ccall trf_15277(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15277(t0,t1,t2,t3);}

C_noret_decl(trf_15342)
static void C_ccall trf_15342(C_word c,C_word *av) C_noret;
static void C_ccall trf_15342(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15342(t0,t1,t2,t3);}

C_noret_decl(trf_15546)
static void C_ccall trf_15546(C_word c,C_word *av) C_noret;
static void C_ccall trf_15546(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15546(t0,t1);}

C_noret_decl(trf_15607)
static void C_ccall trf_15607(C_word c,C_word *av) C_noret;
static void C_ccall trf_15607(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15607(t0,t1,t2);}

C_noret_decl(trf_15650)
static void C_ccall trf_15650(C_word c,C_word *av) C_noret;
static void C_ccall trf_15650(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15650(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15668)
static void C_ccall trf_15668(C_word c,C_word *av) C_noret;
static void C_ccall trf_15668(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15668(t0,t1,t2,t3);}

C_noret_decl(trf_15748)
static void C_ccall trf_15748(C_word c,C_word *av) C_noret;
static void C_ccall trf_15748(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15748(t0,t1,t2);}

C_noret_decl(trf_15754)
static void C_ccall trf_15754(C_word c,C_word *av) C_noret;
static void C_ccall trf_15754(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15754(t0,t1,t2);}

C_noret_decl(trf_15817)
static void C_ccall trf_15817(C_word c,C_word *av) C_noret;
static void C_ccall trf_15817(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15817(t0,t1);}

C_noret_decl(trf_15856)
static void C_ccall trf_15856(C_word c,C_word *av) C_noret;
static void C_ccall trf_15856(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15856(t0,t1,t2);}

C_noret_decl(trf_15862)
static void C_ccall trf_15862(C_word c,C_word *av) C_noret;
static void C_ccall trf_15862(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15862(t0,t1,t2);}

C_noret_decl(trf_15906)
static void C_ccall trf_15906(C_word c,C_word *av) C_noret;
static void C_ccall trf_15906(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_15906(t0,t1,t2,t3,t4);}

C_noret_decl(trf_15962)
static void C_ccall trf_15962(C_word c,C_word *av) C_noret;
static void C_ccall trf_15962(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15962(t0,t1,t2);}

C_noret_decl(trf_15977)
static void C_ccall trf_15977(C_word c,C_word *av) C_noret;
static void C_ccall trf_15977(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15977(t0,t1);}

C_noret_decl(trf_16020)
static void C_ccall trf_16020(C_word c,C_word *av) C_noret;
static void C_ccall trf_16020(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_16020(t0,t1,t2,t3,t4);}

C_noret_decl(trf_16039)
static void C_ccall trf_16039(C_word c,C_word *av) C_noret;
static void C_ccall trf_16039(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16039(t0,t1);}

C_noret_decl(trf_16051)
static void C_ccall trf_16051(C_word c,C_word *av) C_noret;
static void C_ccall trf_16051(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16051(t0,t1);}

C_noret_decl(trf_16078)
static void C_ccall trf_16078(C_word c,C_word *av) C_noret;
static void C_ccall trf_16078(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16078(t0,t1);}

C_noret_decl(trf_16234)
static void C_ccall trf_16234(C_word c,C_word *av) C_noret;
static void C_ccall trf_16234(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16234(t0,t1,t2,t3);}

C_noret_decl(trf_16241)
static void C_ccall trf_16241(C_word c,C_word *av) C_noret;
static void C_ccall trf_16241(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16241(t0,t1);}

C_noret_decl(trf_16312)
static void C_ccall trf_16312(C_word c,C_word *av) C_noret;
static void C_ccall trf_16312(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16312(t0,t1);}

C_noret_decl(trf_16580)
static void C_ccall trf_16580(C_word c,C_word *av) C_noret;
static void C_ccall trf_16580(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16580(t0,t1,t2);}

C_noret_decl(trf_16595)
static void C_ccall trf_16595(C_word c,C_word *av) C_noret;
static void C_ccall trf_16595(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16595(t0,t1);}

C_noret_decl(trf_16653)
static void C_ccall trf_16653(C_word c,C_word *av) C_noret;
static void C_ccall trf_16653(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16653(t0,t1);}

C_noret_decl(trf_17089)
static void C_ccall trf_17089(C_word c,C_word *av) C_noret;
static void C_ccall trf_17089(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_17089(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_17374)
static void C_ccall trf_17374(C_word c,C_word *av) C_noret;
static void C_ccall trf_17374(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_17374(t0,t1,t2,t3,t4);}

C_noret_decl(trf_17401)
static void C_ccall trf_17401(C_word c,C_word *av) C_noret;
static void C_ccall trf_17401(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17401(t0,t1,t2);}

C_noret_decl(trf_17442)
static void C_ccall trf_17442(C_word c,C_word *av) C_noret;
static void C_ccall trf_17442(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17442(t0,t1,t2,t3);}

C_noret_decl(trf_17679)
static void C_ccall trf_17679(C_word c,C_word *av) C_noret;
static void C_ccall trf_17679(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17679(t0,t1,t2);}

C_noret_decl(trf_17941)
static void C_ccall trf_17941(C_word c,C_word *av) C_noret;
static void C_ccall trf_17941(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17941(t0,t1,t2);}

C_noret_decl(trf_18160)
static void C_ccall trf_18160(C_word c,C_word *av) C_noret;
static void C_ccall trf_18160(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18160(t0,t1,t2,t3);}

C_noret_decl(trf_18261)
static void C_ccall trf_18261(C_word c,C_word *av) C_noret;
static void C_ccall trf_18261(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18261(t0,t1,t2);}

C_noret_decl(trf_18469)
static void C_ccall trf_18469(C_word c,C_word *av) C_noret;
static void C_ccall trf_18469(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18469(t0,t1,t2);}

C_noret_decl(trf_18507)
static void C_ccall trf_18507(C_word c,C_word *av) C_noret;
static void C_ccall trf_18507(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18507(t0,t1,t2);}

C_noret_decl(trf_18555)
static void C_ccall trf_18555(C_word c,C_word *av) C_noret;
static void C_ccall trf_18555(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18555(t0,t1,t2);}

C_noret_decl(trf_18616)
static void C_ccall trf_18616(C_word c,C_word *av) C_noret;
static void C_ccall trf_18616(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18616(t0,t1,t2);}

C_noret_decl(trf_18646)
static void C_ccall trf_18646(C_word c,C_word *av) C_noret;
static void C_ccall trf_18646(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18646(t0,t1,t2);}

C_noret_decl(trf_18701)
static void C_ccall trf_18701(C_word c,C_word *av) C_noret;
static void C_ccall trf_18701(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18701(t0,t1,t2);}

C_noret_decl(trf_18742)
static void C_ccall trf_18742(C_word c,C_word *av) C_noret;
static void C_ccall trf_18742(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18742(t0,t1,t2);}

C_noret_decl(trf_18856)
static void C_ccall trf_18856(C_word c,C_word *av) C_noret;
static void C_ccall trf_18856(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18856(t0,t1,t2);}

C_noret_decl(trf_19246)
static void C_ccall trf_19246(C_word c,C_word *av) C_noret;
static void C_ccall trf_19246(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_19246(t0,t1,t2,t3,t4);}

C_noret_decl(trf_19289)
static void C_ccall trf_19289(C_word c,C_word *av) C_noret;
static void C_ccall trf_19289(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19289(t0,t1,t2);}

C_noret_decl(trf_19344)
static void C_ccall trf_19344(C_word c,C_word *av) C_noret;
static void C_ccall trf_19344(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19344(t0,t1,t2,t3);}

C_noret_decl(trf_19600)
static void C_ccall trf_19600(C_word c,C_word *av) C_noret;
static void C_ccall trf_19600(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19600(t0,t1,t2);}

C_noret_decl(trf_19613)
static void C_ccall trf_19613(C_word c,C_word *av) C_noret;
static void C_ccall trf_19613(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19613(t0,t1,t2);}

C_noret_decl(trf_19672)
static void C_ccall trf_19672(C_word c,C_word *av) C_noret;
static void C_ccall trf_19672(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19672(t0,t1,t2);}

C_noret_decl(trf_19926)
static void C_ccall trf_19926(C_word c,C_word *av) C_noret;
static void C_ccall trf_19926(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_19926(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_19930)
static void C_ccall trf_19930(C_word c,C_word *av) C_noret;
static void C_ccall trf_19930(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19930(t0,t1);}

C_noret_decl(trf_19960)
static void C_ccall trf_19960(C_word c,C_word *av) C_noret;
static void C_ccall trf_19960(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19960(t0,t1);}

C_noret_decl(trf_19980)
static void C_ccall trf_19980(C_word c,C_word *av) C_noret;
static void C_ccall trf_19980(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19980(t0,t1,t2,t3);}

C_noret_decl(trf_19989)
static void C_ccall trf_19989(C_word c,C_word *av) C_noret;
static void C_ccall trf_19989(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19989(t0,t1,t2);}

C_noret_decl(trf_19994)
static void C_ccall trf_19994(C_word c,C_word *av) C_noret;
static void C_ccall trf_19994(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19994(t0,t1);}

C_noret_decl(trf_20359)
static void C_ccall trf_20359(C_word c,C_word *av) C_noret;
static void C_ccall trf_20359(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_20359(t0,t1,t2,t3,t4);}

C_noret_decl(trf_20368)
static void C_ccall trf_20368(C_word c,C_word *av) C_noret;
static void C_ccall trf_20368(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20368(t0,t1);}

C_noret_decl(trf_20378)
static void C_ccall trf_20378(C_word c,C_word *av) C_noret;
static void C_ccall trf_20378(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20378(t0,t1,t2);}

C_noret_decl(trf_20405)
static void C_ccall trf_20405(C_word c,C_word *av) C_noret;
static void C_ccall trf_20405(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20405(t0,t1,t2);}

C_noret_decl(trf_20414)
static void C_ccall trf_20414(C_word c,C_word *av) C_noret;
static void C_ccall trf_20414(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20414(t0,t1,t2);}

C_noret_decl(trf_20429)
static void C_ccall trf_20429(C_word c,C_word *av) C_noret;
static void C_ccall trf_20429(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20429(t0,t1);}

C_noret_decl(trf_20432)
static void C_ccall trf_20432(C_word c,C_word *av) C_noret;
static void C_ccall trf_20432(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20432(t0,t1);}

C_noret_decl(trf_20442)
static void C_ccall trf_20442(C_word c,C_word *av) C_noret;
static void C_ccall trf_20442(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20442(t0,t1,t2);}

C_noret_decl(trf_20480)
static void C_ccall trf_20480(C_word c,C_word *av) C_noret;
static void C_ccall trf_20480(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_20480(t0,t1,t2,t3,t4);}

C_noret_decl(trf_20486)
static void C_ccall trf_20486(C_word c,C_word *av) C_noret;
static void C_ccall trf_20486(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20486(t0,t1,t2,t3);}

C_noret_decl(trf_20540)
static void C_ccall trf_20540(C_word c,C_word *av) C_noret;
static void C_ccall trf_20540(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20540(t0,t1,t2);}

C_noret_decl(trf_20552)
static void C_ccall trf_20552(C_word c,C_word *av) C_noret;
static void C_ccall trf_20552(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20552(t0,t1,t2,t3);}

C_noret_decl(trf_20577)
static void C_ccall trf_20577(C_word c,C_word *av) C_noret;
static void C_ccall trf_20577(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20577(t0,t1,t2);}

C_noret_decl(trf_20587)
static void C_ccall trf_20587(C_word c,C_word *av) C_noret;
static void C_ccall trf_20587(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20587(t0,t1,t2,t3);}

C_noret_decl(trf_20768)
static void C_ccall trf_20768(C_word c,C_word *av) C_noret;
static void C_ccall trf_20768(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20768(t0,t1);}

C_noret_decl(trf_20859)
static void C_ccall trf_20859(C_word c,C_word *av) C_noret;
static void C_ccall trf_20859(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20859(t0,t1);}

C_noret_decl(trf_20879)
static void C_ccall trf_20879(C_word c,C_word *av) C_noret;
static void C_ccall trf_20879(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20879(t0,t1);}

C_noret_decl(trf_20884)
static void C_ccall trf_20884(C_word c,C_word *av) C_noret;
static void C_ccall trf_20884(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20884(t0,t1,t2,t3);}

C_noret_decl(trf_20983)
static void C_ccall trf_20983(C_word c,C_word *av) C_noret;
static void C_ccall trf_20983(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20983(t0,t1);}

C_noret_decl(trf_21131)
static void C_ccall trf_21131(C_word c,C_word *av) C_noret;
static void C_ccall trf_21131(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21131(t0,t1,t2,t3);}

C_noret_decl(trf_21140)
static void C_ccall trf_21140(C_word c,C_word *av) C_noret;
static void C_ccall trf_21140(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21140(t0,t1,t2);}

C_noret_decl(trf_21195)
static void C_ccall trf_21195(C_word c,C_word *av) C_noret;
static void C_ccall trf_21195(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21195(t0,t1,t2);}

C_noret_decl(trf_21202)
static void C_ccall trf_21202(C_word c,C_word *av) C_noret;
static void C_ccall trf_21202(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21202(t0,t1);}

C_noret_decl(trf_21245)
static void C_ccall trf_21245(C_word c,C_word *av) C_noret;
static void C_ccall trf_21245(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21245(t0,t1);}

C_noret_decl(trf_21393)
static void C_ccall trf_21393(C_word c,C_word *av) C_noret;
static void C_ccall trf_21393(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21393(t0,t1);}

C_noret_decl(trf_21411)
static void C_ccall trf_21411(C_word c,C_word *av) C_noret;
static void C_ccall trf_21411(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21411(t0,t1,t2,t3);}

C_noret_decl(trf_21481)
static void C_ccall trf_21481(C_word c,C_word *av) C_noret;
static void C_ccall trf_21481(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21481(t0,t1,t2);}

C_noret_decl(trf_21532)
static void C_ccall trf_21532(C_word c,C_word *av) C_noret;
static void C_ccall trf_21532(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21532(t0,t1,t2);}

C_noret_decl(trf_21599)
static void C_ccall trf_21599(C_word c,C_word *av) C_noret;
static void C_ccall trf_21599(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21599(t0,t1);}

C_noret_decl(trf_21609)
static void C_ccall trf_21609(C_word c,C_word *av) C_noret;
static void C_ccall trf_21609(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21609(t0,t1,t2,t3);}

C_noret_decl(trf_21619)
static void C_ccall trf_21619(C_word c,C_word *av) C_noret;
static void C_ccall trf_21619(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21619(t0,t1);}

C_noret_decl(trf_21663)
static void C_ccall trf_21663(C_word c,C_word *av) C_noret;
static void C_ccall trf_21663(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21663(t0,t1);}

C_noret_decl(trf_21673)
static void C_ccall trf_21673(C_word c,C_word *av) C_noret;
static void C_ccall trf_21673(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21673(t0,t1,t2,t3);}

C_noret_decl(trf_21709)
static void C_ccall trf_21709(C_word c,C_word *av) C_noret;
static void C_ccall trf_21709(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21709(t0,t1);}

C_noret_decl(trf_21734)
static void C_ccall trf_21734(C_word c,C_word *av) C_noret;
static void C_ccall trf_21734(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21734(t0,t1,t2);}

C_noret_decl(trf_21739)
static void C_ccall trf_21739(C_word c,C_word *av) C_noret;
static void C_ccall trf_21739(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21739(t0,t1);}

C_noret_decl(trf_21744)
static void C_ccall trf_21744(C_word c,C_word *av) C_noret;
static void C_ccall trf_21744(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_21744(t0,t1,t2,t3,t4);}

C_noret_decl(trf_21757)
static void C_ccall trf_21757(C_word c,C_word *av) C_noret;
static void C_ccall trf_21757(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21757(t0,t1);}

C_noret_decl(trf_21763)
static void C_ccall trf_21763(C_word c,C_word *av) C_noret;
static void C_ccall trf_21763(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21763(t0,t1);}

C_noret_decl(trf_21793)
static void C_ccall trf_21793(C_word c,C_word *av) C_noret;
static void C_ccall trf_21793(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21793(t0,t1);}

C_noret_decl(trf_21985)
static void C_ccall trf_21985(C_word c,C_word *av) C_noret;
static void C_ccall trf_21985(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21985(t0,t1);}

C_noret_decl(trf_22024)
static void C_ccall trf_22024(C_word c,C_word *av) C_noret;
static void C_ccall trf_22024(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22024(t0,t1);}

C_noret_decl(trf_22169)
static void C_ccall trf_22169(C_word c,C_word *av) C_noret;
static void C_ccall trf_22169(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22169(t0,t1);}

C_noret_decl(trf_22194)
static void C_ccall trf_22194(C_word c,C_word *av) C_noret;
static void C_ccall trf_22194(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22194(t0,t1);}

C_noret_decl(trf_22200)
static void C_ccall trf_22200(C_word c,C_word *av) C_noret;
static void C_ccall trf_22200(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22200(t0,t1,t2);}

C_noret_decl(trf_22281)
static void C_ccall trf_22281(C_word c,C_word *av) C_noret;
static void C_ccall trf_22281(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22281(t0,t1);}

C_noret_decl(trf_22294)
static void C_ccall trf_22294(C_word c,C_word *av) C_noret;
static void C_ccall trf_22294(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22294(t0,t1,t2);}

C_noret_decl(trf_22902)
static void C_ccall trf_22902(C_word c,C_word *av) C_noret;
static void C_ccall trf_22902(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22902(t0,t1);}

C_noret_decl(trf_23451)
static void C_ccall trf_23451(C_word c,C_word *av) C_noret;
static void C_ccall trf_23451(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23451(t0,t1,t2);}

C_noret_decl(trf_23497)
static void C_ccall trf_23497(C_word c,C_word *av) C_noret;
static void C_ccall trf_23497(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_23497(t0,t1,t2,t3);}

C_noret_decl(trf_23760)
static void C_ccall trf_23760(C_word c,C_word *av) C_noret;
static void C_ccall trf_23760(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23760(t0,t1,t2);}

C_noret_decl(trf_23811)
static void C_ccall trf_23811(C_word c,C_word *av) C_noret;
static void C_ccall trf_23811(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_23811(t0,t1,t2);}

C_noret_decl(trf_23870)
static void C_ccall trf_23870(C_word c,C_word *av) C_noret;
static void C_ccall trf_23870(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_23870(t0,t1,t2,t3);}

C_noret_decl(trf_23942)
static void C_ccall trf_23942(C_word c,C_word *av) C_noret;
static void C_ccall trf_23942(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_23942(t0,t1,t2,t3);}

C_noret_decl(trf_24002)
static void C_ccall trf_24002(C_word c,C_word *av) C_noret;
static void C_ccall trf_24002(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_24002(t0,t1,t2,t3);}

C_noret_decl(trf_24026)
static void C_ccall trf_24026(C_word c,C_word *av) C_noret;
static void C_ccall trf_24026(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_24026(t0,t1,t2,t3);}

C_noret_decl(trf_24038)
static void C_ccall trf_24038(C_word c,C_word *av) C_noret;
static void C_ccall trf_24038(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24038(t0,t1,t2);}

C_noret_decl(trf_24115)
static void C_ccall trf_24115(C_word c,C_word *av) C_noret;
static void C_ccall trf_24115(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24115(t0,t1,t2);}

C_noret_decl(trf_24164)
static void C_ccall trf_24164(C_word c,C_word *av) C_noret;
static void C_ccall trf_24164(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24164(t0,t1,t2);}

C_noret_decl(trf_24180)
static void C_ccall trf_24180(C_word c,C_word *av) C_noret;
static void C_ccall trf_24180(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24180(t0,t1);}

C_noret_decl(trf_24199)
static void C_ccall trf_24199(C_word c,C_word *av) C_noret;
static void C_ccall trf_24199(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_24199(t0,t1);}

C_noret_decl(trf_24309)
static void C_ccall trf_24309(C_word c,C_word *av) C_noret;
static void C_ccall trf_24309(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24309(t0,t1,t2);}

C_noret_decl(trf_24368)
static void C_ccall trf_24368(C_word c,C_word *av) C_noret;
static void C_ccall trf_24368(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24368(t0,t1,t2);}

C_noret_decl(trf_24525)
static void C_ccall trf_24525(C_word c,C_word *av) C_noret;
static void C_ccall trf_24525(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_24525(t0,t1,t2,t3);}

C_noret_decl(trf_24704)
static void C_ccall trf_24704(C_word c,C_word *av) C_noret;
static void C_ccall trf_24704(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24704(t0,t1,t2);}

C_noret_decl(trf_24776)
static void C_ccall trf_24776(C_word c,C_word *av) C_noret;
static void C_ccall trf_24776(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_24776(t0,t1,t2);}

C_noret_decl(trf_24915)
static void C_ccall trf_24915(C_word c,C_word *av) C_noret;
static void C_ccall trf_24915(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_24915(t0,t1,t2,t3);}

C_noret_decl(trf_25314)
static void C_ccall trf_25314(C_word c,C_word *av) C_noret;
static void C_ccall trf_25314(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_25314(t0,t1,t2,t3,t4);}

C_noret_decl(trf_25334)
static void C_ccall trf_25334(C_word c,C_word *av) C_noret;
static void C_ccall trf_25334(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_25334(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_25388)
static void C_ccall trf_25388(C_word c,C_word *av) C_noret;
static void C_ccall trf_25388(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25388(t0,t1);}

C_noret_decl(trf_25585)
static void C_ccall trf_25585(C_word c,C_word *av) C_noret;
static void C_ccall trf_25585(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25585(t0,t1,t2);}

C_noret_decl(trf_25600)
static void C_ccall trf_25600(C_word c,C_word *av) C_noret;
static void C_ccall trf_25600(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25600(t0,t1,t2);}

C_noret_decl(trf_25703)
static void C_ccall trf_25703(C_word c,C_word *av) C_noret;
static void C_ccall trf_25703(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25703(t0,t1,t2);}

C_noret_decl(trf_25722)
static void C_ccall trf_25722(C_word c,C_word *av) C_noret;
static void C_ccall trf_25722(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25722(t0,t1);}

C_noret_decl(trf_25831)
static void C_ccall trf_25831(C_word c,C_word *av) C_noret;
static void C_ccall trf_25831(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25831(t0,t1,t2);}

C_noret_decl(trf_25868)
static void C_ccall trf_25868(C_word c,C_word *av) C_noret;
static void C_ccall trf_25868(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_25868(t0,t1);}

C_noret_decl(trf_25902)
static void C_ccall trf_25902(C_word c,C_word *av) C_noret;
static void C_ccall trf_25902(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_25902(t0,t1,t2);}

C_noret_decl(trf_26038)
static void C_ccall trf_26038(C_word c,C_word *av) C_noret;
static void C_ccall trf_26038(C_word c,C_word *av){
C_word t0=av[0];
f_26038(t0);}

C_noret_decl(trf_26086)
static void C_ccall trf_26086(C_word c,C_word *av) C_noret;
static void C_ccall trf_26086(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26086(t0,t1);}

C_noret_decl(trf_26110)
static void C_ccall trf_26110(C_word c,C_word *av) C_noret;
static void C_ccall trf_26110(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26110(t0,t1,t2);}

C_noret_decl(trf_26205)
static void C_ccall trf_26205(C_word c,C_word *av) C_noret;
static void C_ccall trf_26205(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26205(t0,t1,t2);}

C_noret_decl(trf_26253)
static void C_ccall trf_26253(C_word c,C_word *av) C_noret;
static void C_ccall trf_26253(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26253(t0,t1);}

C_noret_decl(trf_26256)
static void C_ccall trf_26256(C_word c,C_word *av) C_noret;
static void C_ccall trf_26256(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26256(t0,t1);}

C_noret_decl(trf_26276)
static void C_ccall trf_26276(C_word c,C_word *av) C_noret;
static void C_ccall trf_26276(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26276(t0,t1);}

C_noret_decl(trf_26699)
static void C_ccall trf_26699(C_word c,C_word *av) C_noret;
static void C_ccall trf_26699(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_26699(t0,t1,t2,t3);}

C_noret_decl(trf_26705)
static void C_ccall trf_26705(C_word c,C_word *av) C_noret;
static void C_ccall trf_26705(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26705(t0,t1,t2);}

C_noret_decl(trf_26721)
static void C_ccall trf_26721(C_word c,C_word *av) C_noret;
static void C_ccall trf_26721(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_26721(t0,t1);}

C_noret_decl(trf_26818)
static void C_ccall trf_26818(C_word c,C_word *av) C_noret;
static void C_ccall trf_26818(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26818(t0,t1,t2);}

C_noret_decl(trf_26864)
static void C_ccall trf_26864(C_word c,C_word *av) C_noret;
static void C_ccall trf_26864(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26864(t0,t1,t2);}

C_noret_decl(trf_26940)
static void C_ccall trf_26940(C_word c,C_word *av) C_noret;
static void C_ccall trf_26940(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26940(t0,t1,t2);}

C_noret_decl(trf_26992)
static void C_ccall trf_26992(C_word c,C_word *av) C_noret;
static void C_ccall trf_26992(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_26992(t0,t1,t2);}

C_noret_decl(trf_27040)
static void C_ccall trf_27040(C_word c,C_word *av) C_noret;
static void C_ccall trf_27040(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27040(t0,t1,t2);}

C_noret_decl(trf_27054)
static void C_ccall trf_27054(C_word c,C_word *av) C_noret;
static void C_ccall trf_27054(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27054(t0,t1,t2);}

C_noret_decl(trf_27110)
static void C_ccall trf_27110(C_word c,C_word *av) C_noret;
static void C_ccall trf_27110(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27110(t0,t1,t2);}

C_noret_decl(trf_27235)
static void C_ccall trf_27235(C_word c,C_word *av) C_noret;
static void C_ccall trf_27235(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27235(t0,t1,t2);}

C_noret_decl(trf_27237)
static void C_ccall trf_27237(C_word c,C_word *av) C_noret;
static void C_ccall trf_27237(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27237(t0,t1,t2);}

C_noret_decl(trf_27258)
static void C_ccall trf_27258(C_word c,C_word *av) C_noret;
static void C_ccall trf_27258(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27258(t0,t1,t2);}

C_noret_decl(trf_27282)
static void C_ccall trf_27282(C_word c,C_word *av) C_noret;
static void C_ccall trf_27282(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_27282(t0,t1,t2,t3);}

C_noret_decl(trf_27336)
static void C_ccall trf_27336(C_word c,C_word *av) C_noret;
static void C_ccall trf_27336(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27336(t0,t1,t2);}

C_noret_decl(trf_27431)
static void C_ccall trf_27431(C_word c,C_word *av) C_noret;
static void C_ccall trf_27431(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_27431(t0,t1,t2);}

C_noret_decl(trf_27436)
static void C_ccall trf_27436(C_word c,C_word *av) C_noret;
static void C_ccall trf_27436(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_27436(t0,t1);}

C_noret_decl(trf_28420)
static void C_ccall trf_28420(C_word c,C_word *av) C_noret;
static void C_ccall trf_28420(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28420(t0,t1,t2);}

C_noret_decl(trf_28484)
static void C_ccall trf_28484(C_word c,C_word *av) C_noret;
static void C_ccall trf_28484(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28484(t0,t1,t2);}

C_noret_decl(trf_28696)
static void C_ccall trf_28696(C_word c,C_word *av) C_noret;
static void C_ccall trf_28696(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28696(t0,t1,t2);}

C_noret_decl(trf_28740)
static void C_ccall trf_28740(C_word c,C_word *av) C_noret;
static void C_ccall trf_28740(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_28740(t0,t1);}

C_noret_decl(trf_28753)
static void C_ccall trf_28753(C_word c,C_word *av) C_noret;
static void C_ccall trf_28753(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28753(t0,t1,t2);}

C_noret_decl(trf_28787)
static void C_ccall trf_28787(C_word c,C_word *av) C_noret;
static void C_ccall trf_28787(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28787(t0,t1,t2);}

C_noret_decl(trf_28802)
static void C_ccall trf_28802(C_word c,C_word *av) C_noret;
static void C_ccall trf_28802(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_28802(t0,t1);}

C_noret_decl(trf_28878)
static void C_ccall trf_28878(C_word c,C_word *av) C_noret;
static void C_ccall trf_28878(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28878(t0,t1,t2);}

C_noret_decl(trf_28978)
static void C_ccall trf_28978(C_word c,C_word *av) C_noret;
static void C_ccall trf_28978(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_28978(t0,t1);}

C_noret_decl(trf_28997)
static void C_ccall trf_28997(C_word c,C_word *av) C_noret;
static void C_ccall trf_28997(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_28997(t0,t1,t2);}

C_noret_decl(trf_29038)
static void C_ccall trf_29038(C_word c,C_word *av) C_noret;
static void C_ccall trf_29038(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_29038(t0,t1,t2,t3,t4);}

C_noret_decl(trf_29360)
static void C_ccall trf_29360(C_word c,C_word *av) C_noret;
static void C_ccall trf_29360(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29360(t0,t1,t2);}

C_noret_decl(trf_29372)
static void C_ccall trf_29372(C_word c,C_word *av) C_noret;
static void C_ccall trf_29372(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29372(t0,t1,t2);}

C_noret_decl(trf_29453)
static void C_ccall trf_29453(C_word c,C_word *av) C_noret;
static void C_ccall trf_29453(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_29453(t0,t1,t2,t3);}

C_noret_decl(trf_29638)
static void C_ccall trf_29638(C_word c,C_word *av) C_noret;
static void C_ccall trf_29638(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_29638(t0,t1,t2);}

C_noret_decl(trf_29766)
static void C_ccall trf_29766(C_word c,C_word *av) C_noret;
static void C_ccall trf_29766(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_29766(t0,t1);}

C_noret_decl(trf_30200)
static void C_ccall trf_30200(C_word c,C_word *av) C_noret;
static void C_ccall trf_30200(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30200(t0,t1);}

C_noret_decl(trf_30212)
static void C_ccall trf_30212(C_word c,C_word *av) C_noret;
static void C_ccall trf_30212(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30212(t0,t1,t2);}

C_noret_decl(trf_30271)
static void C_ccall trf_30271(C_word c,C_word *av) C_noret;
static void C_ccall trf_30271(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30271(t0,t1);}

C_noret_decl(trf_30285)
static void C_ccall trf_30285(C_word c,C_word *av) C_noret;
static void C_ccall trf_30285(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30285(t0,t1,t2);}

C_noret_decl(trf_30307)
static void C_ccall trf_30307(C_word c,C_word *av) C_noret;
static void C_ccall trf_30307(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30307(t0,t1,t2);}

C_noret_decl(trf_30522)
static void C_ccall trf_30522(C_word c,C_word *av) C_noret;
static void C_ccall trf_30522(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30522(t0,t1);}

C_noret_decl(trf_30545)
static void C_ccall trf_30545(C_word c,C_word *av) C_noret;
static void C_ccall trf_30545(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30545(t0,t1);}

C_noret_decl(trf_30711)
static void C_ccall trf_30711(C_word c,C_word *av) C_noret;
static void C_ccall trf_30711(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_30711(t0,t1,t2,t3);}

C_noret_decl(trf_30828)
static void C_ccall trf_30828(C_word c,C_word *av) C_noret;
static void C_ccall trf_30828(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30828(t0,t1);}

C_noret_decl(trf_30861)
static void C_ccall trf_30861(C_word c,C_word *av) C_noret;
static void C_ccall trf_30861(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30861(t0,t1);}

C_noret_decl(trf_30864)
static void C_ccall trf_30864(C_word c,C_word *av) C_noret;
static void C_ccall trf_30864(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30864(t0,t1);}

C_noret_decl(trf_30867)
static void C_ccall trf_30867(C_word c,C_word *av) C_noret;
static void C_ccall trf_30867(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30867(t0,t1);}

C_noret_decl(trf_30870)
static void C_ccall trf_30870(C_word c,C_word *av) C_noret;
static void C_ccall trf_30870(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_30870(t0,t1);}

C_noret_decl(trf_30907)
static void C_ccall trf_30907(C_word c,C_word *av) C_noret;
static void C_ccall trf_30907(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30907(t0,t1,t2);}

C_noret_decl(trf_30943)
static void C_ccall trf_30943(C_word c,C_word *av) C_noret;
static void C_ccall trf_30943(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30943(t0,t1,t2);}

C_noret_decl(trf_30974)
static void C_ccall trf_30974(C_word c,C_word *av) C_noret;
static void C_ccall trf_30974(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_30974(t0,t1,t2);}

C_noret_decl(trf_31017)
static void C_ccall trf_31017(C_word c,C_word *av) C_noret;
static void C_ccall trf_31017(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_31017(t0,t1,t2);}

C_noret_decl(trf_31361)
static void C_ccall trf_31361(C_word c,C_word *av) C_noret;
static void C_ccall trf_31361(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_31361(t0,t1,t2);}

C_noret_decl(trf_31380)
static void C_ccall trf_31380(C_word c,C_word *av) C_noret;
static void C_ccall trf_31380(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_31380(t0,t1);}

C_noret_decl(trf_31533)
static void C_ccall trf_31533(C_word c,C_word *av) C_noret;
static void C_ccall trf_31533(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_31533(t0,t1,t2);}

C_noret_decl(trf_31572)
static void C_ccall trf_31572(C_word c,C_word *av) C_noret;
static void C_ccall trf_31572(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_31572(t0,t1,t2);}

C_noret_decl(trf_31588)
static void C_ccall trf_31588(C_word c,C_word *av) C_noret;
static void C_ccall trf_31588(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_31588(t0,t1,t2);}

C_noret_decl(trf_31754)
static void C_ccall trf_31754(C_word c,C_word *av) C_noret;
static void C_ccall trf_31754(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_31754(t0,t1);}

C_noret_decl(trf_32268)
static void C_ccall trf_32268(C_word c,C_word *av) C_noret;
static void C_ccall trf_32268(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_32268(t0,t1,t2,t3,t4);}

C_noret_decl(trf_32364)
static void C_ccall trf_32364(C_word c,C_word *av) C_noret;
static void C_ccall trf_32364(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_32364(t0,t1);}

C_noret_decl(trf_32377)
static void C_ccall trf_32377(C_word c,C_word *av) C_noret;
static void C_ccall trf_32377(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_32377(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_8891)
static void C_ccall trf_8891(C_word c,C_word *av) C_noret;
static void C_ccall trf_8891(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8891(t0,t1,t2,t3);}

C_noret_decl(trf_9361)
static void C_ccall trf_9361(C_word c,C_word *av) C_noret;
static void C_ccall trf_9361(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9361(t0,t1,t2,t3);}

C_noret_decl(trf_9398)
static void C_ccall trf_9398(C_word c,C_word *av) C_noret;
static void C_ccall trf_9398(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9398(t0,t1,t2);}

C_noret_decl(trf_9484)
static void C_ccall trf_9484(C_word c,C_word *av) C_noret;
static void C_ccall trf_9484(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9484(t0,t1,t2);}

C_noret_decl(trf_9517)
static void C_ccall trf_9517(C_word c,C_word *av) C_noret;
static void C_ccall trf_9517(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9517(t0,t1,t2);}

C_noret_decl(trf_9635)
static void C_ccall trf_9635(C_word c,C_word *av) C_noret;
static void C_ccall trf_9635(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9635(t0,t1,t2);}

C_noret_decl(trf_9670)
static void C_ccall trf_9670(C_word c,C_word *av) C_noret;
static void C_ccall trf_9670(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9670(t0,t1,t2);}

C_noret_decl(trf_9734)
static void C_ccall trf_9734(C_word c,C_word *av) C_noret;
static void C_ccall trf_9734(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9734(t0,t1,t2);}

C_noret_decl(trf_9799)
static void C_ccall trf_9799(C_word c,C_word *av) C_noret;
static void C_ccall trf_9799(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9799(t0,t1,t2);}

C_noret_decl(trf_9835)
static void C_ccall trf_9835(C_word c,C_word *av) C_noret;
static void C_ccall trf_9835(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9835(t0,t1,t2);}

C_noret_decl(trf_9864)
static void C_ccall trf_9864(C_word c,C_word *av) C_noret;
static void C_ccall trf_9864(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9864(t0,t1,t2,t3);}

C_noret_decl(trf_9911)
static void C_ccall trf_9911(C_word c,C_word *av) C_noret;
static void C_ccall trf_9911(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9911(t0,t1,t2);}

C_noret_decl(trf_9965)
static void C_ccall trf_9965(C_word c,C_word *av) C_noret;
static void C_ccall trf_9965(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9965(t0,t1,t2,t3);}

/* f34270 in chicken.base#emergency-exit in k8569 */
static void C_ccall f34270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f34270,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_exit_runtime(C_fix(0));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f34823 in chicken.base#exit in k8569 */
static void C_ccall f34823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f34823,c,av);}
/* library.scm:724: g673 */
t2=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* f35032 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f35032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f35032,c,av);}
/* library.scm:2723: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[641];
av2[3]=t1;
tp(4,av2);}}

/* f35036 in k17632 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f35036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f35036,c,av);}
/* library.scm:2723: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* f37127 in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f37127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f37127,c,av);}
t3=C_i_check_list_2(t2,lf[1367]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* f37133 in k31426 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f37133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f37133,c,av);}
t3=C_i_check_string_2(t2,lf[1369]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* f37139 in k31422 in k31426 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f37139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f37139,c,av);}
t3=C_i_check_string_2(t2,lf[1369]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10005 in loop in chicken.base#compress in k8569 */
static void C_ccall f_10007(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10007,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#alist-update! in k8569 */
static void C_ccall f_10036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(c<5) C_bad_min_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10036,c,av);}
a=C_alloc(10);
t5=C_rest_nullp(c,5);
t6=(C_truep(t5)?*((C_word*)lf[5]+1):C_get_rest_arg(c,5,av,5,t0));
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10043,a[2]=t3,a[3]=t1,a[4]=t4,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t8=C_eqp(*((C_word*)lf[4]+1),t6);
if(C_truep(t8)){
t9=t7;
f_10043(t9,*((C_word*)lf[52]+1));}
else{
t9=C_eqp(*((C_word*)lf[5]+1),t6);
if(C_truep(t9)){
t10=t7;
f_10043(t10,*((C_word*)lf[53]+1));}
else{
t10=C_eqp(*((C_word*)lf[6]+1),t6);
t11=t7;
f_10043(t11,(C_truep(t10)?*((C_word*)lf[54]+1):(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10075,a[2]=t6,a[3]=((C_word)li187),tmp=(C_word)a,a+=4,tmp)));}}}

/* k10041 in chicken.base#alist-update! in k8569 */
static void C_fcall f_10043(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_10043,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10046,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:898: aq */
t3=t1;{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10044 in k10041 in chicken.base#alist-update! in k8569 */
static void C_ccall f_10046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10046,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_setslot(t1,C_fix(1),((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[5],((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* f_10075 in chicken.base#alist-update! in k8569 */
static void C_ccall f_10075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10075,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10081,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word)li186),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_10081(t7,t1,t3);}

/* loop */
static void C_fcall f_10081(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_10081,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10097,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(t3))){
/* library.scm:895: cmp */
t5=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
av2[3]=C_slot(t3,C_fix(0));
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}
else{
/* library.scm:897: loop */
t6=t1;
t7=C_slot(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10095 in loop */
static void C_ccall f_10097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10097,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:897: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_10081(t2,((C_word*)t0)[2],C_slot(((C_word*)t0)[5],C_fix(1)));}}

/* chicken.base#alist-update in k8569 */
static void C_ccall f_10122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c<5) C_bad_min_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10122,c,av);}
a=C_alloc(9);
t5=C_rest_nullp(c,5);
t6=(C_truep(t5)?*((C_word*)lf[5]+1):C_get_rest_arg(c,5,av,5,t0));
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10131,a[2]=t2,a[3]=t3,a[4]=t8,a[5]=t6,a[6]=((C_word)li189),tmp=(C_word)a,a+=7,tmp));
t10=((C_word*)t8)[1];
f_10131(t10,t1,t4);}

/* loop in chicken.base#alist-update in k8569 */
static void C_fcall f_10131(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_10131,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list1(&a,1,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
/* library.scm:910: chicken.base#error */
t4=*((C_word*)lf[193]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[263];
av2[3]=lf[264];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=C_i_pairp(t4);
if(C_truep(C_i_not(t5))){
/* library.scm:914: chicken.base#error */
t6=*((C_word*)lf[193]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[263];
av2[3]=lf[265];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10172,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t1,a[6]=t4,a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
/* library.scm:915: cmp */
t7=((C_word*)t0)[5];{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=C_slot(t4,C_fix(0));
((C_proc)C_fast_retrieve_proc(t7))(4,av2);}}}}}

/* k10170 in loop in chicken.base#alist-update in k8569 */
static void C_ccall f_10172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10172,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_slot(((C_word*)t0)[4],C_fix(1));
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_slot(((C_word*)t0)[6],C_fix(0));
t3=C_slot(((C_word*)t0)[6],C_fix(1));
t4=C_a_i_cons(&a,2,t2,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10194,a[2]=((C_word*)t0)[5],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:919: loop */
t6=((C_word*)((C_word*)t0)[7])[1];
f_10131(t6,t5,C_slot(((C_word*)t0)[4],C_fix(1)));}}

/* k10192 in k10170 in loop in chicken.base#alist-update in k8569 */
static void C_ccall f_10194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10194,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#alist-ref in k8569 */
static void C_ccall f_10226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10226,c,av);}
a=C_alloc(10);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?*((C_word*)lf[5]+1):C_get_rest_arg(c,4,av,4,t0));
t6=C_rest_nullp(c,4);
t7=C_rest_nullp(c,5);
t8=(C_truep(t7)?C_SCHEME_FALSE:C_get_rest_arg(c,5,av,4,t0));
t9=C_rest_nullp(c,5);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10242,a[2]=t1,a[3]=t8,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t11=C_eqp(*((C_word*)lf[4]+1),t5);
if(C_truep(t11)){
t12=t10;
f_10242(t12,*((C_word*)lf[52]+1));}
else{
t12=C_eqp(*((C_word*)lf[5]+1),t5);
if(C_truep(t12)){
t13=t10;
f_10242(t13,*((C_word*)lf[53]+1));}
else{
t13=C_eqp(*((C_word*)lf[6]+1),t5);
t14=t10;
f_10242(t14,(C_truep(t13)?*((C_word*)lf[54]+1):(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10267,a[2]=t5,a[3]=((C_word)li192),tmp=(C_word)a,a+=4,tmp)));}}}

/* k10240 in chicken.base#alist-ref in k8569 */
static void C_fcall f_10242(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_10242,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10245,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:937: aq */
t3=t1;{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10243 in k10240 in chicken.base#alist-ref in k8569 */
static void C_ccall f_10245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10245,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_slot(t1,C_fix(1)):((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_10267 in chicken.base#alist-ref in k8569 */
static void C_ccall f_10267(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10267,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10273,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word)li191),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_10273(t7,t1,t3);}

/* loop */
static void C_fcall f_10273(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_10273,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_check_pair_2(t3,lf[267]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10298,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:933: cmp */
t6=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
av2[3]=C_slot(t3,C_fix(0));
((C_proc)C_fast_retrieve_proc(t6))(4,av2);}}
else{
/* library.scm:936: chicken.base#error */
t3=*((C_word*)lf[193]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[267];
av2[3]=lf[268];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}}

/* k10296 in loop */
static void C_ccall f_10298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10298,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:935: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_10273(t2,((C_word*)t0)[2],C_slot(((C_word*)t0)[5],C_fix(1)));}}

/* chicken.base#rassoc in k8569 */
static void C_ccall f_10338(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +8,c,3)))){
C_save_and_reclaim((void*)f_10338,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+8);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
t5=C_i_check_list_2(t3,lf[270]);
t6=C_i_pairp(t4);
t7=(C_truep(t6)?C_get_rest_arg(c,4,av,4,t0):*((C_word*)lf[5]+1));
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10350,a[2]=t9,a[3]=t7,a[4]=t2,a[5]=((C_word)li194),tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_10350(t11,t1,t3);}

/* loop in chicken.base#rassoc in k8569 */
static void C_fcall f_10350(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_10350,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_check_pair_2(t3,lf[270]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10369,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:950: tst */
t6=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
av2[3]=C_slot(t3,C_fix(1));
((C_proc)C_fast_retrieve_proc(t6))(4,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10367 in loop in chicken.base#rassoc in k8569 */
static void C_ccall f_10369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10369,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:952: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_10350(t2,((C_word*)t0)[2],C_slot(((C_word*)t0)[5],C_fix(1)));}}

/* chicken.fixnum#fx+ in k8569 */
static void C_ccall f_10391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10391,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_plus(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx- in k8569 */
static void C_ccall f_10394(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10394,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_difference(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx* in k8569 */
static void C_ccall f_10397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10397,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_times(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx= in k8569 */
static void C_ccall f_10400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10400,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_eqp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx> in k8569 */
static void C_ccall f_10406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10406,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_greaterp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx< in k8569 */
static void C_ccall f_10409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10409,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_lessp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx>= in k8569 */
static void C_ccall f_10412(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10412,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_greater_or_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx<= in k8569 */
static void C_ccall f_10415(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10415,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_less_or_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxmin in k8569 */
static void C_ccall f_10418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10418,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_fixnum_min(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxmax in k8569 */
static void C_ccall f_10421(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10421,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_fixnum_max(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxneg in k8569 */
static void C_ccall f_10424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10424,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fixnum_negate(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.fixnum#fxand in k8569 */
static void C_ccall f_10427(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10427,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_and(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxior in k8569 */
static void C_ccall f_10430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10430,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_or(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxxor in k8569 */
static void C_ccall f_10433(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10433,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_xor(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxnot in k8569 */
static void C_ccall f_10436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10436,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fixnum_not(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.fixnum#fxshl in k8569 */
static void C_ccall f_10439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10439,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_shift_left(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxshr in k8569 */
static void C_ccall f_10442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10442,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_shift_right(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxodd? in k8569 */
static void C_ccall f_10445(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10445,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_fixnumoddp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.fixnum#fxeven? in k8569 */
static void C_ccall f_10448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10448,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_fixnumevenp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.fixnum#fxlen in k8569 */
static void C_ccall f_10451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10451,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_fixnum_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.fixnum#fx/ in k8569 */
static void C_ccall f_10454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10454,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_divide(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxgcd in k8569 */
static void C_ccall f_10457(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10457,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_fixnum_gcd(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxmod in k8569 */
static void C_ccall f_10460(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10460,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fixnum_modulo(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fxrem in k8569 */
static void C_ccall f_10463(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10463,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_fixnum_remainder_checked(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx+? in k8569 */
static void C_ccall f_10466(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10466,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_o_fixnum_plus(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx-? in k8569 */
static void C_ccall f_10469(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10469,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_o_fixnum_difference(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx*? in k8569 */
static void C_ccall f_10472(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10472,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_o_fixnum_times(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.fixnum#fx/? in k8569 */
static void C_ccall f_10475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10475,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_o_fixnum_quotient(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10477 in k8569 */
static void C_ccall f_10479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word t91;
C_word t92;
C_word t93;
C_word t94;
C_word t95;
C_word t96;
C_word t97;
C_word t98;
C_word t99;
C_word t100;
C_word t101;
C_word t102;
C_word t103;
C_word t104;
C_word t105;
C_word t106;
C_word t107;
C_word t108;
C_word t109;
C_word t110;
C_word t111;
C_word t112;
C_word t113;
C_word t114;
C_word t115;
C_word t116;
C_word t117;
C_word t118;
C_word t119;
C_word t120;
C_word t121;
C_word t122;
C_word t123;
C_word t124;
C_word t125;
C_word t126;
C_word t127;
C_word t128;
C_word t129;
C_word t130;
C_word t131;
C_word t132;
C_word t133;
C_word t134;
C_word t135;
C_word t136;
C_word t137;
C_word t138;
C_word t139;
C_word t140;
C_word t141;
C_word t142;
C_word t143;
C_word t144;
C_word t145;
C_word t146;
C_word t147;
C_word t148;
C_word t149;
C_word t150;
C_word t151;
C_word t152;
C_word t153;
C_word t154;
C_word t155;
C_word t156;
C_word t157;
C_word t158;
C_word t159;
C_word t160;
C_word t161;
C_word t162;
C_word t163;
C_word t164;
C_word t165;
C_word t166;
C_word t167;
C_word t168;
C_word t169;
C_word t170;
C_word t171;
C_word t172;
C_word t173;
C_word t174;
C_word t175;
C_word t176;
C_word t177;
C_word t178;
C_word t179;
C_word t180;
C_word t181;
C_word t182;
C_word t183;
C_word t184;
C_word t185;
C_word t186;
C_word t187;
C_word t188;
C_word t189;
C_word t190;
C_word t191;
C_word t192;
C_word t193;
C_word t194;
C_word t195;
C_word t196;
C_word t197;
C_word t198;
C_word t199;
C_word t200;
C_word t201;
C_word t202;
C_word t203;
C_word t204;
C_word t205;
C_word t206;
C_word t207;
C_word t208;
C_word t209;
C_word t210;
C_word t211;
C_word t212;
C_word t213;
C_word t214;
C_word t215;
C_word t216;
C_word t217;
C_word t218;
C_word t219;
C_word t220;
C_word t221;
C_word t222;
C_word t223;
C_word t224;
C_word t225;
C_word t226;
C_word t227;
C_word t228;
C_word t229;
C_word t230;
C_word t231;
C_word t232;
C_word t233;
C_word t234;
C_word t235;
C_word t236;
C_word t237;
C_word t238;
C_word t239;
C_word t240;
C_word t241;
C_word t242;
C_word t243;
C_word t244;
C_word t245;
C_word t246;
C_word t247;
C_word t248;
C_word t249;
C_word t250;
C_word t251;
C_word t252;
C_word t253;
C_word t254;
C_word t255;
C_word t256;
C_word t257;
C_word t258;
C_word t259;
C_word t260;
C_word t261;
C_word t262;
C_word t263;
C_word t264;
C_word t265;
C_word t266;
C_word t267;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(728,c,6)))){
C_save_and_reclaim((void *)f_10479,c,av);}
a=C_alloc(728);
t2=C_mutate((C_word*)lf[305]+1 /* (set! ##sys#debug-mode? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10481,a[2]=((C_word)li224),tmp=(C_word)a,a+=3,tmp));
t3=C_set_block_item(lf[306] /* ##sys#warnings-enabled */,0,C_SCHEME_TRUE);
t4=C_set_block_item(lf[307] /* ##sys#notices-enabled */,0,C_i_debug_modep());
t5=C_mutate((C_word*)lf[308]+1 /* (set! chicken.base#warning ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10486,a[2]=((C_word)li225),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[310]+1 /* (set! chicken.base#notice ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10495,a[2]=((C_word)li226),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[312]+1 /* (set! chicken.base#enable-warnings ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10507,a[2]=((C_word)li227),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[155]+1 /* (set! ##sys#error ...) */,*((C_word*)lf[193]+1));
t9=C_mutate((C_word*)lf[313]+1 /* (set! ##sys#warn ...) */,*((C_word*)lf[308]+1));
t10=C_mutate((C_word*)lf[314]+1 /* (set! ##sys#notice ...) */,*((C_word*)lf[310]+1));
t11=C_mutate((C_word*)lf[315]+1 /* (set! ##sys#gc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_gc,a[2]=((C_word)li228),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[316]+1 /* (set! ##sys#setslot ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10522,a[2]=((C_word)li229),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[317]+1 /* (set! ##sys#setislot ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10525,a[2]=((C_word)li230),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[318]+1 /* (set! ##sys#allocate-vector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_allocate_vector,a[2]=((C_word)li231),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[319]+1 /* (set! ##sys#make-structure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_make_structure,a[2]=((C_word)li232),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate((C_word*)lf[320]+1 /* (set! ##sys#ensure-heap-reserve ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_ensure_heap_reserve,a[2]=((C_word)li233),tmp=(C_word)a,a+=3,tmp));
t17=C_mutate((C_word*)lf[321]+1 /* (set! ##sys#symbol-table-info ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_get_symbol_table_info,a[2]=((C_word)li234),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate((C_word*)lf[322]+1 /* (set! ##sys#memory-info ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_get_memory_info,a[2]=((C_word)li235),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[323]+1 /* (set! ##sys#start-timer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10533,a[2]=((C_word)li236),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[324]+1 /* (set! ##sys#stop-timer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10539,a[2]=((C_word)li239),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[325]+1 /* (set! ##sys#immediate? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10559,a[2]=((C_word)li240),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[326]+1 /* (set! ##sys#message ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10566,a[2]=((C_word)li241),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[327]+1 /* (set! ##sys#byte ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10569,a[2]=((C_word)li242),tmp=(C_word)a,a+=3,tmp));
t24=C_mutate((C_word*)lf[328]+1 /* (set! ##sys#setbyte ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10572,a[2]=((C_word)li243),tmp=(C_word)a,a+=3,tmp));
t25=C_mutate((C_word*)lf[329]+1 /* (set! ##sys#void ...) */,*((C_word*)lf[196]+1));
t26=C_set_block_item(lf[246] /* ##sys#undefined-value */,0,C_SCHEME_UNDEFINED);
t27=C_mutate((C_word*)lf[330]+1 /* (set! ##sys#halt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10577,a[2]=((C_word)li244),tmp=(C_word)a,a+=3,tmp));
t28=C_mutate((C_word*)lf[331]+1 /* (set! ##sys#become! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_become,a[2]=((C_word)li245),tmp=(C_word)a,a+=3,tmp));
t29=C_mutate((C_word*)lf[332]+1 /* (set! ##sys#block-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10581,a[2]=((C_word)li246),tmp=(C_word)a,a+=3,tmp));
t30=C_mutate((C_word*)lf[333]+1 /* (set! ##sys#apply-values ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_apply_values,a[2]=((C_word)li247),tmp=(C_word)a,a+=3,tmp));
t31=C_mutate((C_word*)lf[334]+1 /* (set! ##sys#copy-closure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_copy_closure,a[2]=((C_word)li248),tmp=(C_word)a,a+=3,tmp));
t32=C_mutate((C_word*)lf[335]+1 /* (set! ##sys#block-set! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10586,a[2]=((C_word)li249),tmp=(C_word)a,a+=3,tmp));
t33=C_a_i_provide(&a,1,lf[338]);
t34=C_mutate((C_word*)lf[339]+1 /* (set! chicken.time#current-milliseconds ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10621,a[2]=((C_word)li250),tmp=(C_word)a,a+=3,tmp));
t35=C_mutate((C_word*)lf[340]+1 /* (set! chicken.time#current-process-milliseconds ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10624,a[2]=((C_word)li251),tmp=(C_word)a,a+=3,tmp));
t36=C_mutate((C_word*)lf[341]+1 /* (set! chicken.time#current-seconds ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10627,a[2]=((C_word)li252),tmp=(C_word)a,a+=3,tmp));
t37=C_mutate((C_word*)lf[342]+1 /* (set! chicken.time#cpu-time ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10630,a[2]=((C_word)li253),tmp=(C_word)a,a+=3,tmp));
t38=C_mutate((C_word*)lf[343]+1 /* (set! ##sys#check-structure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10647,a[2]=((C_word)li254),tmp=(C_word)a,a+=3,tmp));
t39=C_mutate((C_word*)lf[344]+1 /* (set! ##sys#check-blob ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10658,a[2]=((C_word)li255),tmp=(C_word)a,a+=3,tmp));
t40=C_mutate((C_word*)lf[345]+1 /* (set! ##sys#check-byte-vector ...) */,*((C_word*)lf[344]+1));
t41=C_mutate((C_word*)lf[346]+1 /* (set! ##sys#check-pair ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10670,a[2]=((C_word)li256),tmp=(C_word)a,a+=3,tmp));
t42=C_mutate((C_word*)lf[347]+1 /* (set! ##sys#check-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10681,a[2]=((C_word)li257),tmp=(C_word)a,a+=3,tmp));
t43=C_mutate((C_word*)lf[348]+1 /* (set! ##sys#check-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10692,a[2]=((C_word)li258),tmp=(C_word)a,a+=3,tmp));
t44=C_mutate((C_word*)lf[349]+1 /* (set! ##sys#check-number ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10703,a[2]=((C_word)li259),tmp=(C_word)a,a+=3,tmp));
t45=C_mutate((C_word*)lf[102]+1 /* (set! ##sys#check-fixnum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10714,a[2]=((C_word)li260),tmp=(C_word)a,a+=3,tmp));
t46=C_mutate((C_word*)lf[350]+1 /* (set! ##sys#check-exact ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10725,a[2]=((C_word)li261),tmp=(C_word)a,a+=3,tmp));
t47=C_mutate((C_word*)lf[351]+1 /* (set! ##sys#check-inexact ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10736,a[2]=((C_word)li262),tmp=(C_word)a,a+=3,tmp));
t48=C_mutate((C_word*)lf[352]+1 /* (set! ##sys#check-symbol ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10747,a[2]=((C_word)li263),tmp=(C_word)a,a+=3,tmp));
t49=C_mutate((C_word*)lf[353]+1 /* (set! ##sys#check-keyword ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10758,a[2]=((C_word)li264),tmp=(C_word)a,a+=3,tmp));
t50=C_mutate((C_word*)lf[354]+1 /* (set! ##sys#check-vector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10769,a[2]=((C_word)li265),tmp=(C_word)a,a+=3,tmp));
t51=C_mutate((C_word*)lf[355]+1 /* (set! ##sys#check-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10780,a[2]=((C_word)li266),tmp=(C_word)a,a+=3,tmp));
t52=C_mutate((C_word*)lf[356]+1 /* (set! ##sys#check-boolean ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10791,a[2]=((C_word)li267),tmp=(C_word)a,a+=3,tmp));
t53=C_mutate((C_word*)lf[357]+1 /* (set! ##sys#check-locative ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10802,a[2]=((C_word)li268),tmp=(C_word)a,a+=3,tmp));
t54=C_mutate((C_word*)lf[358]+1 /* (set! ##sys#check-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10813,a[2]=((C_word)li269),tmp=(C_word)a,a+=3,tmp));
t55=C_mutate((C_word*)lf[360]+1 /* (set! ##sys#check-exact-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10830,a[2]=((C_word)li270),tmp=(C_word)a,a+=3,tmp));
t56=C_mutate((C_word*)lf[362]+1 /* (set! ##sys#check-exact-uinteger ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10847,a[2]=((C_word)li271),tmp=(C_word)a,a+=3,tmp));
t57=C_mutate((C_word*)lf[364]+1 /* (set! ##sys#check-real ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10871,a[2]=((C_word)li272),tmp=(C_word)a,a+=3,tmp));
t58=C_mutate((C_word*)lf[336]+1 /* (set! ##sys#check-range ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10888,a[2]=((C_word)li273),tmp=(C_word)a,a+=3,tmp));
t59=C_mutate((C_word*)lf[367]+1 /* (set! ##sys#check-special ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10918,a[2]=((C_word)li274),tmp=(C_word)a,a+=3,tmp));
t60=C_mutate((C_word*)lf[369]+1 /* (set! ##sys#check-closure ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10938,a[2]=((C_word)li275),tmp=(C_word)a,a+=3,tmp));
t61=C_mutate((C_word*)lf[370]+1 /* (set! scheme#force ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10949,a[2]=((C_word)li277),tmp=(C_word)a,a+=3,tmp));
t62=C_mutate((C_word*)lf[372]+1 /* (set! ##sys#dload ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_dload,a[2]=((C_word)li278),tmp=(C_word)a,a+=3,tmp));
t63=C_mutate((C_word*)lf[373]+1 /* (set! ##sys#set-dlopen-flags! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_set_dlopen_flags,a[2]=((C_word)li279),tmp=(C_word)a,a+=3,tmp));
t64=C_mutate((C_word*)lf[47]+1 /* (set! ##sys#error-not-a-proper-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11077,a[2]=((C_word)li280),tmp=(C_word)a,a+=3,tmp));
t65=C_mutate((C_word*)lf[374]+1 /* (set! ##sys#error-bad-number ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11093,a[2]=((C_word)li281),tmp=(C_word)a,a+=3,tmp));
t66=C_mutate((C_word*)lf[359]+1 /* (set! ##sys#error-bad-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11109,a[2]=((C_word)li282),tmp=(C_word)a,a+=3,tmp));
t67=C_mutate((C_word*)lf[361]+1 /* (set! ##sys#error-bad-exact-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11125,a[2]=((C_word)li283),tmp=(C_word)a,a+=3,tmp));
t68=C_mutate((C_word*)lf[363]+1 /* (set! ##sys#error-bad-exact-uinteger ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11141,a[2]=((C_word)li284),tmp=(C_word)a,a+=3,tmp));
t69=C_mutate((C_word*)lf[375]+1 /* (set! ##sys#error-bad-inexact ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11157,a[2]=((C_word)li285),tmp=(C_word)a,a+=3,tmp));
t70=C_mutate((C_word*)lf[365]+1 /* (set! ##sys#error-bad-real ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11173,a[2]=((C_word)li286),tmp=(C_word)a,a+=3,tmp));
t71=C_mutate((C_word*)lf[376]+1 /* (set! ##sys#error-bad-base ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11189,a[2]=((C_word)li287),tmp=(C_word)a,a+=3,tmp));
t72=C_mutate((C_word*)lf[377]+1 /* (set! scheme#append ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11205,a[2]=((C_word)li290),tmp=(C_word)a,a+=3,tmp));
t73=C_mutate((C_word*)lf[379]+1 /* (set! ##sys#fast-reverse ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11284,a[2]=((C_word)li292),tmp=(C_word)a,a+=3,tmp));
t74=C_mutate((C_word*)lf[380]+1 /* (set! ##sys#make-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11314,a[2]=((C_word)li293),tmp=(C_word)a,a+=3,tmp));
t75=C_mutate((C_word*)lf[381]+1 /* (set! scheme#make-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11334,a[2]=((C_word)li294),tmp=(C_word)a,a+=3,tmp));
t76=C_mutate((C_word*)lf[385]+1 /* (set! scheme#string->list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11370,a[2]=((C_word)li296),tmp=(C_word)a,a+=3,tmp));
t77=C_mutate((C_word*)lf[387]+1 /* (set! ##sys#string->list ...) */,*((C_word*)lf[385]+1));
t78=C_mutate((C_word*)lf[388]+1 /* (set! scheme#list->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11408,a[2]=((C_word)li298),tmp=(C_word)a,a+=3,tmp));
t79=C_mutate((C_word*)lf[390]+1 /* (set! ##sys#list->string ...) */,*((C_word*)lf[388]+1));
t80=C_mutate((C_word*)lf[391]+1 /* (set! ##sys#reverse-list->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11460,a[2]=((C_word)li300),tmp=(C_word)a,a+=3,tmp));
t81=C_mutate((C_word*)lf[393]+1 /* (set! scheme#string-fill! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11509,a[2]=((C_word)li301),tmp=(C_word)a,a+=3,tmp));
t82=C_mutate((C_word*)lf[395]+1 /* (set! scheme#string-copy ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11522,a[2]=((C_word)li302),tmp=(C_word)a,a+=3,tmp));
t83=C_mutate((C_word*)lf[397]+1 /* (set! scheme#substring ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11534,a[2]=((C_word)li303),tmp=(C_word)a,a+=3,tmp));
t84=C_mutate((C_word*)lf[399]+1 /* (set! ##sys#substring ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11584,a[2]=((C_word)li304),tmp=(C_word)a,a+=3,tmp));
t85=C_SCHEME_UNDEFINED;
t86=(*a=C_VECTOR_TYPE|1,a[1]=t85,tmp=(C_word)a,a+=2,tmp);
t87=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11593,a[2]=((C_word)li305),tmp=(C_word)a,a+=3,tmp);
t88=C_set_block_item(t86,0,t87);
t89=C_mutate((C_word*)lf[400]+1 /* (set! scheme#string<? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11614,a[2]=t86,a[3]=((C_word)li307),tmp=(C_word)a,a+=4,tmp));
t90=C_mutate((C_word*)lf[402]+1 /* (set! scheme#string>? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11638,a[2]=t86,a[3]=((C_word)li309),tmp=(C_word)a,a+=4,tmp));
t91=C_mutate((C_word*)lf[404]+1 /* (set! scheme#string<=? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11662,a[2]=t86,a[3]=((C_word)li311),tmp=(C_word)a,a+=4,tmp));
t92=C_mutate((C_word*)lf[406]+1 /* (set! scheme#string>=? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11683,a[2]=t86,a[3]=((C_word)li313),tmp=(C_word)a,a+=4,tmp));
t93=C_SCHEME_UNDEFINED;
t94=(*a=C_VECTOR_TYPE|1,a[1]=t93,tmp=(C_word)a,a+=2,tmp);
t95=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11703,a[2]=((C_word)li314),tmp=(C_word)a,a+=3,tmp);
t96=C_set_block_item(t94,0,t95);
t97=C_mutate((C_word*)lf[408]+1 /* (set! scheme#string-ci<? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11724,a[2]=t94,a[3]=((C_word)li316),tmp=(C_word)a,a+=4,tmp));
t98=C_mutate((C_word*)lf[410]+1 /* (set! scheme#string-ci>? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11748,a[2]=t94,a[3]=((C_word)li318),tmp=(C_word)a,a+=4,tmp));
t99=C_mutate((C_word*)lf[412]+1 /* (set! scheme#string-ci<=? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11772,a[2]=t94,a[3]=((C_word)li320),tmp=(C_word)a,a+=4,tmp));
t100=C_mutate((C_word*)lf[414]+1 /* (set! scheme#string-ci>=? ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11793,a[2]=t94,a[3]=((C_word)li322),tmp=(C_word)a,a+=4,tmp));
t101=C_mutate((C_word*)lf[416]+1 /* (set! ##sys#string-append ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11814,a[2]=((C_word)li323),tmp=(C_word)a,a+=3,tmp));
t102=C_mutate((C_word*)lf[417]+1 /* (set! scheme#string-append ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11830,a[2]=((C_word)li325),tmp=(C_word)a,a+=3,tmp));
t103=C_mutate((C_word*)lf[419]+1 /* (set! scheme#string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11869,a[2]=((C_word)li326),tmp=(C_word)a,a+=3,tmp));
t104=C_mutate((C_word*)lf[420]+1 /* (set! ##sys#fragments->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11875,a[2]=((C_word)li328),tmp=(C_word)a,a+=3,tmp));
t105=C_mutate((C_word*)lf[421]+1 /* (set! chicken.base#chop ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11910,a[2]=((C_word)li331),tmp=(C_word)a,a+=3,tmp));
t106=C_a_i_provide(&a,1,lf[424]);
t107=C_mutate((C_word*)lf[425]+1 /* (set! chicken.flonum#maximum-flonum ...) */,C_flonum(&a,DBL_MAX));
t108=C_mutate((C_word*)lf[426]+1 /* (set! chicken.flonum#minimum-flonum ...) */,C_flonum(&a,DBL_MIN));
t109=C_set_block_item(lf[427] /* chicken.flonum#flonum-radix */,0,C_fix((C_word)FLT_RADIX));
t110=C_mutate((C_word*)lf[428]+1 /* (set! chicken.flonum#flonum-epsilon ...) */,C_flonum(&a,DBL_EPSILON));
t111=C_set_block_item(lf[429] /* chicken.flonum#flonum-precision */,0,C_fix((C_word)DBL_MANT_DIG));
t112=C_set_block_item(lf[430] /* chicken.flonum#flonum-decimal-precision */,0,C_fix((C_word)DBL_DIG));
t113=C_set_block_item(lf[431] /* chicken.flonum#flonum-maximum-exponent */,0,C_fix((C_word)DBL_MAX_EXP));
t114=C_set_block_item(lf[432] /* chicken.flonum#flonum-minimum-exponent */,0,C_fix((C_word)DBL_MIN_EXP));
t115=C_set_block_item(lf[433] /* chicken.flonum#flonum-maximum-decimal-exponent */,0,C_fix((C_word)DBL_MAX_10_EXP));
t116=C_set_block_item(lf[434] /* chicken.flonum#flonum-minimum-decimal-exponent */,0,C_fix((C_word)DBL_MIN_10_EXP));
t117=C_mutate((C_word*)lf[435]+1 /* (set! chicken.flonum#fp+ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12005,a[2]=((C_word)li332),tmp=(C_word)a,a+=3,tmp));
t118=C_mutate((C_word*)lf[437]+1 /* (set! chicken.flonum#fp- ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12029,a[2]=((C_word)li333),tmp=(C_word)a,a+=3,tmp));
t119=C_mutate((C_word*)lf[439]+1 /* (set! chicken.flonum#fp* ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12053,a[2]=((C_word)li334),tmp=(C_word)a,a+=3,tmp));
t120=C_mutate((C_word*)lf[441]+1 /* (set! chicken.flonum#fp/ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12077,a[2]=((C_word)li335),tmp=(C_word)a,a+=3,tmp));
t121=C_mutate((C_word*)lf[443]+1 /* (set! chicken.flonum#fpgcd ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12101,a[2]=((C_word)li336),tmp=(C_word)a,a+=3,tmp));
t122=C_mutate((C_word*)lf[445]+1 /* (set! chicken.flonum#fp/? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12125,a[2]=((C_word)li337),tmp=(C_word)a,a+=3,tmp));
t123=C_mutate((C_word*)lf[447]+1 /* (set! chicken.flonum#fp= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12149,a[2]=((C_word)li338),tmp=(C_word)a,a+=3,tmp));
t124=C_mutate((C_word*)lf[449]+1 /* (set! chicken.flonum#fp> ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12173,a[2]=((C_word)li339),tmp=(C_word)a,a+=3,tmp));
t125=C_mutate((C_word*)lf[451]+1 /* (set! chicken.flonum#fp< ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12197,a[2]=((C_word)li340),tmp=(C_word)a,a+=3,tmp));
t126=C_mutate((C_word*)lf[453]+1 /* (set! chicken.flonum#fp>= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12221,a[2]=((C_word)li341),tmp=(C_word)a,a+=3,tmp));
t127=C_mutate((C_word*)lf[455]+1 /* (set! chicken.flonum#fp<= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12245,a[2]=((C_word)li342),tmp=(C_word)a,a+=3,tmp));
t128=C_mutate((C_word*)lf[457]+1 /* (set! chicken.flonum#fpneg ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12269,a[2]=((C_word)li343),tmp=(C_word)a,a+=3,tmp));
t129=C_mutate((C_word*)lf[459]+1 /* (set! chicken.flonum#fpmax ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12287,a[2]=((C_word)li344),tmp=(C_word)a,a+=3,tmp));
t130=C_mutate((C_word*)lf[461]+1 /* (set! chicken.flonum#fpmin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12311,a[2]=((C_word)li345),tmp=(C_word)a,a+=3,tmp));
t131=C_mutate((C_word*)lf[463]+1 /* (set! chicken.flonum#fpfloor ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12335,a[2]=((C_word)li346),tmp=(C_word)a,a+=3,tmp));
t132=C_mutate((C_word*)lf[465]+1 /* (set! chicken.flonum#fptruncate ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12353,a[2]=((C_word)li347),tmp=(C_word)a,a+=3,tmp));
t133=C_mutate((C_word*)lf[467]+1 /* (set! chicken.flonum#fpround ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12371,a[2]=((C_word)li348),tmp=(C_word)a,a+=3,tmp));
t134=C_mutate((C_word*)lf[469]+1 /* (set! chicken.flonum#fpceiling ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12389,a[2]=((C_word)li349),tmp=(C_word)a,a+=3,tmp));
t135=C_mutate((C_word*)lf[471]+1 /* (set! chicken.flonum#fpsin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12407,a[2]=((C_word)li350),tmp=(C_word)a,a+=3,tmp));
t136=C_mutate((C_word*)lf[473]+1 /* (set! chicken.flonum#fpcos ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12425,a[2]=((C_word)li351),tmp=(C_word)a,a+=3,tmp));
t137=C_mutate((C_word*)lf[475]+1 /* (set! chicken.flonum#fptan ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12443,a[2]=((C_word)li352),tmp=(C_word)a,a+=3,tmp));
t138=C_mutate((C_word*)lf[477]+1 /* (set! chicken.flonum#fpasin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12461,a[2]=((C_word)li353),tmp=(C_word)a,a+=3,tmp));
t139=C_mutate((C_word*)lf[479]+1 /* (set! chicken.flonum#fpacos ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12479,a[2]=((C_word)li354),tmp=(C_word)a,a+=3,tmp));
t140=C_mutate((C_word*)lf[481]+1 /* (set! chicken.flonum#fpatan ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12497,a[2]=((C_word)li355),tmp=(C_word)a,a+=3,tmp));
t141=C_mutate((C_word*)lf[483]+1 /* (set! chicken.flonum#fpatan2 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12515,a[2]=((C_word)li356),tmp=(C_word)a,a+=3,tmp));
t142=C_mutate((C_word*)lf[485]+1 /* (set! chicken.flonum#fpexp ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12539,a[2]=((C_word)li357),tmp=(C_word)a,a+=3,tmp));
t143=C_mutate((C_word*)lf[487]+1 /* (set! chicken.flonum#fpexpt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12557,a[2]=((C_word)li358),tmp=(C_word)a,a+=3,tmp));
t144=C_mutate((C_word*)lf[489]+1 /* (set! chicken.flonum#fplog ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12581,a[2]=((C_word)li359),tmp=(C_word)a,a+=3,tmp));
t145=C_mutate((C_word*)lf[491]+1 /* (set! chicken.flonum#fpsqrt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12599,a[2]=((C_word)li360),tmp=(C_word)a,a+=3,tmp));
t146=C_mutate((C_word*)lf[493]+1 /* (set! chicken.flonum#fpabs ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12617,a[2]=((C_word)li361),tmp=(C_word)a,a+=3,tmp));
t147=C_mutate((C_word*)lf[495]+1 /* (set! chicken.flonum#fpinteger? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12635,a[2]=((C_word)li362),tmp=(C_word)a,a+=3,tmp));
t148=C_mutate((C_word*)lf[497]+1 /* (set! chicken.flonum#flonum-print-precision ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12653,a[2]=((C_word)li363),tmp=(C_word)a,a+=3,tmp));
t149=C_mutate((C_word*)lf[499]+1 /* (set! ##sys#number? ...) */,*((C_word*)lf[77]+1));
t150=C_mutate((C_word*)lf[500]+1 /* (set! ##sys#integer? ...) */,*((C_word*)lf[81]+1));
t151=C_mutate((C_word*)lf[501]+1 /* (set! ##sys#exact? ...) */,*((C_word*)lf[82]+1));
t152=C_mutate((C_word*)lf[502]+1 /* (set! ##sys#inexact? ...) */,*((C_word*)lf[83]+1));
t153=C_mutate(&lf[503] /* (set! make-complex ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12675,a[2]=((C_word)li364),tmp=(C_word)a,a+=3,tmp));
t154=C_mutate((C_word*)lf[505]+1 /* (set! scheme#make-rectangular ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12711,a[2]=((C_word)li365),tmp=(C_word)a,a+=3,tmp));
t155=C_mutate((C_word*)lf[507]+1 /* (set! scheme#make-polar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12723,a[2]=((C_word)li366),tmp=(C_word)a,a+=3,tmp));
t156=C_mutate((C_word*)lf[509]+1 /* (set! scheme#real-part ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12746,a[2]=((C_word)li367),tmp=(C_word)a,a+=3,tmp));
t157=C_mutate((C_word*)lf[511]+1 /* (set! scheme#imag-part ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12769,a[2]=((C_word)li368),tmp=(C_word)a,a+=3,tmp));
t158=C_mutate((C_word*)lf[513]+1 /* (set! scheme#angle ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12795,a[2]=((C_word)li369),tmp=(C_word)a,a+=3,tmp));
t159=C_mutate((C_word*)lf[515]+1 /* (set! scheme#magnitude ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12811,a[2]=((C_word)li370),tmp=(C_word)a,a+=3,tmp));
t160=C_mutate(&lf[518] /* (set! ratnum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12855,a[2]=((C_word)li371),tmp=(C_word)a,a+=3,tmp));
t161=C_mutate((C_word*)lf[519]+1 /* (set! scheme#numerator ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12901,a[2]=((C_word)li372),tmp=(C_word)a,a+=3,tmp));
t162=C_mutate((C_word*)lf[523]+1 /* (set! scheme#denominator ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12947,a[2]=((C_word)li373),tmp=(C_word)a,a+=3,tmp));
t163=C_mutate((C_word*)lf[527]+1 /* (set! ##sys#extended-signum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12993,a[2]=((C_word)li374),tmp=(C_word)a,a+=3,tmp));
t164=C_mutate((C_word*)lf[521]+1 /* (set! scheme#inexact->exact ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13117,a[2]=((C_word)li377),tmp=(C_word)a,a+=3,tmp));
t165=C_mutate((C_word*)lf[534]+1 /* (set! ##sys#exact->inexact ...) */,*((C_word*)lf[62]+1));
t166=C_mutate((C_word*)lf[535]+1 /* (set! ##sys#inexact->exact ...) */,*((C_word*)lf[521]+1));
t167=C_a_i_provide(&a,1,lf[536]);
t168=C_mutate((C_word*)lf[537]+1 /* (set! chicken.bitwise#bitwise-and ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_bitwise_and,a[2]=((C_word)li378),tmp=(C_word)a,a+=3,tmp));
t169=C_mutate((C_word*)lf[538]+1 /* (set! chicken.bitwise#bitwise-ior ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_bitwise_ior,a[2]=((C_word)li379),tmp=(C_word)a,a+=3,tmp));
t170=C_mutate((C_word*)lf[539]+1 /* (set! chicken.bitwise#bitwise-xor ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_bitwise_xor,a[2]=((C_word)li380),tmp=(C_word)a,a+=3,tmp));
t171=C_mutate((C_word*)lf[540]+1 /* (set! chicken.bitwise#bitwise-not ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13183,a[2]=((C_word)li381),tmp=(C_word)a,a+=3,tmp));
t172=C_mutate((C_word*)lf[541]+1 /* (set! chicken.bitwise#bit->boolean ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13186,a[2]=((C_word)li382),tmp=(C_word)a,a+=3,tmp));
t173=C_mutate((C_word*)lf[542]+1 /* (set! chicken.bitwise#integer-length ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13189,a[2]=((C_word)li383),tmp=(C_word)a,a+=3,tmp));
t174=C_mutate((C_word*)lf[543]+1 /* (set! chicken.bitwise#arithmetic-shift ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13192,a[2]=((C_word)li384),tmp=(C_word)a,a+=3,tmp));
t175=C_mutate((C_word*)lf[544]+1 /* (set! scheme#/ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13195,a[2]=((C_word)li386),tmp=(C_word)a,a+=3,tmp));
t176=C_mutate((C_word*)lf[529]+1 /* (set! ##sys#/-2 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13246,a[2]=((C_word)li387),tmp=(C_word)a,a+=3,tmp));
t177=C_mutate((C_word*)lf[547]+1 /* (set! scheme#floor ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13529,a[2]=((C_word)li388),tmp=(C_word)a,a+=3,tmp));
t178=C_mutate((C_word*)lf[549]+1 /* (set! scheme#ceiling ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13574,a[2]=((C_word)li389),tmp=(C_word)a,a+=3,tmp));
t179=C_mutate((C_word*)lf[551]+1 /* (set! scheme#truncate ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13619,a[2]=((C_word)li390),tmp=(C_word)a,a+=3,tmp));
t180=C_mutate((C_word*)lf[553]+1 /* (set! scheme#round ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13656,a[2]=((C_word)li391),tmp=(C_word)a,a+=3,tmp));
t181=C_mutate(&lf[555] /* (set! find-ratio-between ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13700,a[2]=((C_word)li393),tmp=(C_word)a,a+=3,tmp));
t182=C_mutate((C_word*)lf[557]+1 /* (set! scheme#rationalize ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13850,a[2]=((C_word)li394),tmp=(C_word)a,a+=3,tmp));
t183=C_mutate((C_word*)lf[558]+1 /* (set! scheme#max ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13875,a[2]=((C_word)li396),tmp=(C_word)a,a+=3,tmp));
t184=C_mutate((C_word*)lf[560]+1 /* (set! scheme#min ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13919,a[2]=((C_word)li398),tmp=(C_word)a,a+=3,tmp));
t185=C_mutate((C_word*)lf[562]+1 /* (set! scheme#exp ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13963,a[2]=((C_word)li399),tmp=(C_word)a,a+=3,tmp));
t186=C_mutate((C_word*)lf[564]+1 /* (set! ##sys#log-1 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14008,a[2]=((C_word)li400),tmp=(C_word)a,a+=3,tmp));
t187=C_mutate((C_word*)lf[568]+1 /* (set! scheme#log ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14057,a[2]=((C_word)li401),tmp=(C_word)a,a+=3,tmp));
t188=C_mutate((C_word*)lf[569]+1 /* (set! scheme#sin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14086,a[2]=((C_word)li402),tmp=(C_word)a,a+=3,tmp));
t189=C_mutate((C_word*)lf[573]+1 /* (set! scheme#cos ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14120,a[2]=((C_word)li403),tmp=(C_word)a,a+=3,tmp));
t190=C_mutate((C_word*)lf[575]+1 /* (set! scheme#tan ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14154,a[2]=((C_word)li404),tmp=(C_word)a,a+=3,tmp));
t191=C_mutate((C_word*)lf[577]+1 /* (set! scheme#asin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14179,a[2]=((C_word)li405),tmp=(C_word)a,a+=3,tmp));
t192=C_a_i_asin(&a,1,C_fix(1));
t193=C_mutate((C_word*)lf[582]+1 /* (set! scheme#acos ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14241,a[2]=t192,a[3]=((C_word)li406),tmp=(C_word)a,a+=4,tmp));
t194=C_mutate((C_word*)lf[584]+1 /* (set! scheme#atan ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14283,a[2]=((C_word)li407),tmp=(C_word)a,a+=3,tmp));
t195=C_mutate((C_word*)lf[586]+1 /* (set! ##sys#exact-integer-sqrt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14347,a[2]=((C_word)li413),tmp=(C_word)a,a+=3,tmp));
t196=C_mutate((C_word*)lf[588]+1 /* (set! chicken.base#exact-integer-sqrt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14538,a[2]=((C_word)li414),tmp=(C_word)a,a+=3,tmp));
t197=C_mutate((C_word*)lf[580]+1 /* (set! ##sys#sqrt/loc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14547,a[2]=((C_word)li421),tmp=(C_word)a,a+=3,tmp));
t198=C_mutate((C_word*)lf[516]+1 /* (set! scheme#sqrt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14702,a[2]=((C_word)li422),tmp=(C_word)a,a+=3,tmp));
t199=C_mutate((C_word*)lf[591]+1 /* (set! chicken.base#exact-integer-nth-root ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14708,a[2]=((C_word)li423),tmp=(C_word)a,a+=3,tmp));
t200=C_mutate((C_word*)lf[592]+1 /* (set! ##sys#exact-integer-nth-root/loc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14720,a[2]=((C_word)li425),tmp=(C_word)a,a+=3,tmp));
t201=C_mutate((C_word*)lf[531]+1 /* (set! ##sys#integer-power ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14846,a[2]=((C_word)li427),tmp=(C_word)a,a+=3,tmp));
t202=C_mutate((C_word*)lf[533]+1 /* (set! scheme#expt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14914,a[2]=((C_word)li432),tmp=(C_word)a,a+=3,tmp));
t203=C_mutate(&lf[545] /* (set! ##sys#internal-gcd ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15181,a[2]=((C_word)li433),tmp=(C_word)a,a+=3,tmp));
t204=C_mutate((C_word*)lf[596]+1 /* (set! ##sys#gcd ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15251,a[2]=((C_word)li434),tmp=(C_word)a,a+=3,tmp));
t205=C_mutate((C_word*)lf[598]+1 /* (set! scheme#gcd ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15257,a[2]=((C_word)li436),tmp=(C_word)a,a+=3,tmp));
t206=C_mutate((C_word*)lf[599]+1 /* (set! ##sys#lcm ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15308,a[2]=((C_word)li437),tmp=(C_word)a,a+=3,tmp));
t207=C_mutate((C_word*)lf[601]+1 /* (set! scheme#lcm ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15322,a[2]=((C_word)li439),tmp=(C_word)a,a+=3,tmp));
t208=C_mutate((C_word*)lf[602]+1 /* (set! ##sys#integer->string/recursive ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15380,a[2]=((C_word)li443),tmp=(C_word)a,a+=3,tmp));
t209=C_mutate((C_word*)lf[604]+1 /* (set! ##sys#extended-number->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15433,a[2]=((C_word)li444),tmp=(C_word)a,a+=3,tmp));
t210=C_mutate(&lf[610] /* (set! round-quotient ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15607,a[2]=((C_word)li445),tmp=(C_word)a,a+=3,tmp));
t211=C_mutate(&lf[611] /* (set! ##sys#string->compnum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15650,a[2]=((C_word)li457),tmp=(C_word)a,a+=3,tmp));
t212=C_mutate((C_word*)lf[620]+1 /* (set! scheme#string->number ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17070,a[2]=((C_word)li459),tmp=(C_word)a,a+=3,tmp));
t213=C_mutate((C_word*)lf[622]+1 /* (set! ##sys#string->number ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17325,a[2]=((C_word)li460),tmp=(C_word)a,a+=3,tmp));
t214=C_mutate((C_word*)lf[623]+1 /* (set! ##sys#fixnum->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_fixnum_to_string,a[2]=((C_word)li461),tmp=(C_word)a,a+=3,tmp));
t215=C_mutate((C_word*)lf[624]+1 /* (set! ##sys#flonum->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_flonum_to_string,a[2]=((C_word)li462),tmp=(C_word)a,a+=3,tmp));
t216=C_mutate((C_word*)lf[625]+1 /* (set! ##sys#integer->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_integer_to_string,a[2]=((C_word)li463),tmp=(C_word)a,a+=3,tmp));
t217=C_mutate((C_word*)lf[603]+1 /* (set! ##sys#number->string ...) */,*((C_word*)lf[87]+1));
t218=C_mutate((C_word*)lf[626]+1 /* (set! chicken.base#equal=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17371,a[2]=((C_word)li467),tmp=(C_word)a,a+=3,tmp));
t219=C_mutate((C_word*)lf[627]+1 /* (set! ##sys#snafu ...) */,lf[628]);
t220=C_mutate((C_word*)lf[629]+1 /* (set! ##sys#intern-symbol ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_string_to_symbol,a[2]=((C_word)li468),tmp=(C_word)a,a+=3,tmp));
t221=C_mutate((C_word*)lf[630]+1 /* (set! ##sys#intern-keyword ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_string_to_keyword,a[2]=((C_word)li469),tmp=(C_word)a,a+=3,tmp));
t222=C_mutate((C_word*)lf[631]+1 /* (set! ##sys#interned-symbol? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17543,a[2]=((C_word)li470),tmp=(C_word)a,a+=3,tmp));
t223=C_mutate((C_word*)lf[632]+1 /* (set! ##sys#string->symbol ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17546,a[2]=((C_word)li471),tmp=(C_word)a,a+=3,tmp));
t224=C_mutate((C_word*)lf[633]+1 /* (set! ##sys#symbol->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17555,a[2]=((C_word)li472),tmp=(C_word)a,a+=3,tmp));
t225=C_mutate((C_word*)lf[634]+1 /* (set! scheme#symbol->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17561,a[2]=((C_word)li473),tmp=(C_word)a,a+=3,tmp));
t226=*((C_word*)lf[395]+1);
t227=C_mutate((C_word*)lf[636]+1 /* (set! scheme#string->symbol ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17574,a[2]=t226,a[3]=((C_word)li474),tmp=(C_word)a,a+=4,tmp));
t228=*((C_word*)lf[395]+1);
t229=C_mutate((C_word*)lf[638]+1 /* (set! chicken.base#string->uninterned-symbol ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17587,a[2]=t228,a[3]=((C_word)li476),tmp=(C_word)a,a+=4,tmp));
t230=C_fix(-1);
t231=(*a=C_VECTOR_TYPE|1,a[1]=t230,tmp=(C_word)a,a+=2,tmp);
t232=C_mutate((C_word*)lf[640]+1 /* (set! chicken.base#gensym ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17600,a[2]=t231,a[3]=((C_word)li478),tmp=(C_word)a,a+=4,tmp));
t233=*((C_word*)lf[417]+1);
t234=C_mutate((C_word*)lf[644]+1 /* (set! chicken.base#symbol-append ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17654,a[2]=t233,a[3]=((C_word)li480),tmp=(C_word)a,a+=4,tmp));
t235=C_a_i_provide(&a,1,lf[646]);
t236=C_mutate((C_word*)lf[647]+1 /* (set! chicken.keyword#keyword? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17713,a[2]=((C_word)li481),tmp=(C_word)a,a+=3,tmp));
t237=C_mutate((C_word*)lf[648]+1 /* (set! chicken.keyword#string->keyword ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17716,a[2]=((C_word)li482),tmp=(C_word)a,a+=3,tmp));
t238=*((C_word*)lf[647]+1);
t239=C_mutate((C_word*)lf[650]+1 /* (set! chicken.keyword#keyword->string ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17725,a[2]=t238,a[3]=((C_word)li483),tmp=(C_word)a,a+=4,tmp));
t240=C_a_i_list1(&a,1,lf[653]);
t241=C_mutate((C_word*)lf[654]+1 /* (set! chicken.keyword#get-keyword ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17743,a[2]=t240,a[3]=((C_word)li484),tmp=(C_word)a,a+=4,tmp));
t242=C_mutate((C_word*)lf[656]+1 /* (set! ##sys#get-keyword ...) */,*((C_word*)lf[654]+1));
t243=C_a_i_provide(&a,1,lf[657]);
t244=C_mutate((C_word*)lf[658]+1 /* (set! ##sys#make-blob ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17774,a[2]=((C_word)li485),tmp=(C_word)a,a+=3,tmp));
t245=C_mutate((C_word*)lf[659]+1 /* (set! chicken.blob#make-blob ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17780,a[2]=((C_word)li486),tmp=(C_word)a,a+=3,tmp));
t246=C_mutate((C_word*)lf[661]+1 /* (set! chicken.blob#blob? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17789,a[2]=((C_word)li487),tmp=(C_word)a,a+=3,tmp));
t247=C_mutate((C_word*)lf[662]+1 /* (set! chicken.blob#blob-size ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17795,a[2]=((C_word)li488),tmp=(C_word)a,a+=3,tmp));
t248=C_mutate((C_word*)lf[664]+1 /* (set! chicken.blob#string->blob ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17804,a[2]=((C_word)li489),tmp=(C_word)a,a+=3,tmp));
t249=C_mutate((C_word*)lf[666]+1 /* (set! chicken.blob#blob->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17816,a[2]=((C_word)li490),tmp=(C_word)a,a+=3,tmp));
t250=C_mutate((C_word*)lf[668]+1 /* (set! chicken.blob#blob=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17828,a[2]=((C_word)li491),tmp=(C_word)a,a+=3,tmp));
t251=C_mutate((C_word*)lf[93]+1 /* (set! scheme#make-vector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17851,a[2]=((C_word)li492),tmp=(C_word)a,a+=3,tmp));
t252=C_mutate((C_word*)lf[672]+1 /* (set! ##sys#make-vector ...) */,*((C_word*)lf[93]+1));
t253=C_mutate((C_word*)lf[94]+1 /* (set! scheme#list->vector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17880,a[2]=((C_word)li494),tmp=(C_word)a,a+=3,tmp));
t254=C_mutate((C_word*)lf[95]+1 /* (set! scheme#vector->list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17932,a[2]=((C_word)li496),tmp=(C_word)a,a+=3,tmp));
t255=C_mutate((C_word*)lf[96]+1 /* (set! scheme#vector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17965,a[2]=((C_word)li497),tmp=(C_word)a,a+=3,tmp));
t256=C_mutate((C_word*)lf[97]+1 /* (set! scheme#vector-fill! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17971,a[2]=((C_word)li499),tmp=(C_word)a,a+=3,tmp));
t257=C_mutate((C_word*)lf[206]+1 /* (set! chicken.base#vector-copy! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17999,a[2]=((C_word)li501),tmp=(C_word)a,a+=3,tmp));
t258=C_mutate((C_word*)lf[207]+1 /* (set! chicken.base#subvector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18068,a[2]=((C_word)li503),tmp=(C_word)a,a+=3,tmp));
t259=C_mutate((C_word*)lf[208]+1 /* (set! chicken.base#vector-resize ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18139,a[2]=((C_word)li504),tmp=(C_word)a,a+=3,tmp));
t260=C_mutate(&lf[680] /* (set! ##sys#vector-resize ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18160,a[2]=((C_word)li506),tmp=(C_word)a,a+=3,tmp));
t261=C_mutate((C_word*)lf[113]+1 /* (set! scheme#char-ci=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18199,a[2]=((C_word)li507),tmp=(C_word)a,a+=3,tmp));
t262=C_mutate((C_word*)lf[114]+1 /* (set! scheme#char-ci>? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18213,a[2]=((C_word)li508),tmp=(C_word)a,a+=3,tmp));
t263=C_mutate((C_word*)lf[115]+1 /* (set! scheme#char-ci<? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18224,a[2]=((C_word)li509),tmp=(C_word)a,a+=3,tmp));
t264=C_mutate((C_word*)lf[116]+1 /* (set! scheme#char-ci>=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18235,a[2]=((C_word)li510),tmp=(C_word)a,a+=3,tmp));
t265=C_mutate((C_word*)lf[117]+1 /* (set! scheme#char-ci<=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18246,a[2]=((C_word)li511),tmp=(C_word)a,a+=3,tmp));
t266=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18259,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2946: scheme#make-vector */
t267=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t267;
av2[1]=t266;
av2[2]=C_fix(37);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t267+1)))(4,av2);}}

/* ##sys#debug-mode? in k10477 in k8569 */
static void C_ccall f_10481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10481,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_debug_modep();
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#warning in k10477 in k8569 */
static void C_ccall f_10486(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,5)))){
C_save_and_reclaim((void*)f_10486,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(*((C_word*)lf[306]+1))){{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[309];
av2[4]=t2;
av2[5]=t3;
C_apply(6,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* chicken.base#notice in k10477 in k8569 */
static void C_ccall f_10495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,5)))){
C_save_and_reclaim((void*)f_10495,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=(C_truep(*((C_word*)lf[307]+1))?*((C_word*)lf[306]+1):C_SCHEME_FALSE);
if(C_truep(t4)){{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[311];
av2[4]=t2;
av2[5]=t3;
C_apply(6,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* chicken.base#enable-warnings in k10477 in k8569 */
static void C_ccall f_10507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10507,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
if(C_truep(C_i_pairp(t2))){
t3=C_mutate((C_word*)lf[306]+1 /* (set! ##sys#warnings-enabled ...) */,C_get_rest_arg(c,2,av,2,t0));
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=*((C_word*)lf[306]+1);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=*((C_word*)lf[306]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* ##sys#setslot in k10477 in k8569 */
static void C_ccall f_10522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10522,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_setslot(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#setislot in k10477 in k8569 */
static void C_ccall f_10525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10525,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_i_slot(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#start-timer in k10477 in k8569 */
static void C_ccall f_10533(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10533,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10537,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1051: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
tp(3,av2);}}

/* k10535 in ##sys#start-timer in k10477 in k8569 */
static void C_ccall f_10537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10537,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_start_timer();
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#stop-timer in k10477 in k8569 */
static void C_ccall f_10539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_10539,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_stop_timer,a[2]=((C_word)li237),tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10543,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1055: g1206 */
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)C_fast_retrieve_proc(t4))(2,av2);}}

/* k10541 in ##sys#stop-timer in k10477 in k8569 */
static void C_ccall f_10543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10543,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10546,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:1058: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
tp(3,av2);}}

/* k10544 in k10541 in ##sys#stop-timer in k10477 in k8569 */
static void C_ccall f_10546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10546,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_stop_timer,a[2]=((C_word)li238),tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10557,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:1059: g1208 */
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)C_fast_retrieve_proc(t4))(2,av2);}}

/* k10555 in k10544 in k10541 in ##sys#stop-timer in k10477 in k8569 */
static void C_ccall f_10557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10557,c,av);}
t2=C_slot(t1,C_fix(6));
t3=C_i_setslot(((C_word*)t0)[2],C_fix(6),t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#immediate? in k10477 in k8569 */
static void C_ccall f_10559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10559,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_not(C_blockp(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#message in k10477 in k8569 */
static void C_ccall f_10566(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10566,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_message(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#byte in k10477 in k8569 */
static void C_ccall f_10569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10569,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_subbyte(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#setbyte in k10477 in k8569 */
static void C_ccall f_10572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10572,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_setbyte(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#halt in k10477 in k8569 */
static void C_ccall f_10577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10577,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_halt(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#block-ref in k10477 in k8569 */
static void C_ccall f_10581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10581,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#block-set! in k10477 in k8569 */
static void C_ccall f_10586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_10586,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10590,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_i_not(C_blockp(t2));
if(C_truep(t6)){
if(C_truep(t6)){
/* library.scm:1078: ##sys#signal-hook */
t7=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=lf[260];
av2[3]=lf[335];
av2[4]=lf[337];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}
else{
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_10590(2,av2);}}}
else{
t7=(C_truep(C_specialp(t2))?C_eqp(t3,C_fix(0)):C_SCHEME_FALSE);
if(C_truep(t7)){
if(C_truep(t7)){
/* library.scm:1078: ##sys#signal-hook */
t8=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t8;
av2[1]=t5;
av2[2]=lf[260];
av2[3]=lf[335];
av2[4]=lf[337];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(6,av2);}}
else{
t8=t5;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_10590(2,av2);}}}
else{
if(C_truep(C_byteblockp(t2))){
/* library.scm:1078: ##sys#signal-hook */
t8=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t8;
av2[1]=t5;
av2[2]=lf[260];
av2[3]=lf[335];
av2[4]=lf[337];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(6,av2);}}
else{
t8=t5;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_10590(2,av2);}}}}}

/* k10588 in ##sys#block-set! in k10477 in k8569 */
static void C_ccall f_10590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_10590,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10593,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:1079: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_fix(0);
av2[4]=C_block_size(((C_word*)t0)[3]);
av2[5]=lf[335];
tp(6,av2);}}

/* k10591 in k10588 in ##sys#block-set! in k10477 in k8569 */
static void C_ccall f_10593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10593,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_setslot(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.time#current-milliseconds in k10477 in k8569 */
static void C_ccall f_10621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_10621,c,av);}
a=C_alloc(7);
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_current_milliseconds(&a,1,C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.time#current-process-milliseconds in k10477 in k8569 */
static void C_ccall f_10624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_10624,c,av);}
a=C_alloc(7);
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_current_process_milliseconds(&a,1,C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.time#current-seconds in k10477 in k8569 */
static void C_ccall f_10627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_10627,c,av);}
a=C_alloc(7);
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_get_current_seconds(&a,1,C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.time#cpu-time in k10477 in k8569 */
static void C_ccall f_10630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_10630,c,av);}
a=C_alloc(11);
t2=C_a_i_vector2(&a,2,C_SCHEME_FALSE,C_SCHEME_FALSE);
t3=C_a_i_cpu_time(&a,1,t2);
/* library.scm:1112: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_slot(t2,C_fix(0));
av2[3]=C_slot(t2,C_fix(1));
C_values(4,av2);}}

/* ##sys#check-structure in k10477 in k8569 */
static void C_ccall f_10647(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10647,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+0);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=C_i_pairp(t4);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?C_i_check_structure_2(t2,t3,C_get_rest_arg(c,4,av,4,t0)):C_i_check_structure(t2,t3));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* ##sys#check-blob in k10477 in k8569 */
static void C_ccall f_10658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10658,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_bytevector_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_bytevector(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-pair in k10477 in k8569 */
static void C_ccall f_10670(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10670,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_pair_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_pair(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-list in k10477 in k8569 */
static void C_ccall f_10681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10681,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_list_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_list(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-string in k10477 in k8569 */
static void C_ccall f_10692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10692,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_string_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_string(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-number in k10477 in k8569 */
static void C_ccall f_10703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10703,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_number_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_number(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-fixnum in k10477 in k8569 */
static void C_ccall f_10714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10714,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_fixnum_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_fixnum(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-exact in k10477 in k8569 */
static void C_ccall f_10725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10725,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_exact_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_exact(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-inexact in k10477 in k8569 */
static void C_ccall f_10736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10736,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_inexact_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_inexact(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-symbol in k10477 in k8569 */
static void C_ccall f_10747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10747,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_symbol_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_symbol(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-keyword in k10477 in k8569 */
static void C_ccall f_10758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10758,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_keyword_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_keyword(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-vector in k10477 in k8569 */
static void C_ccall f_10769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10769,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_vector_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_vector(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-char in k10477 in k8569 */
static void C_ccall f_10780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10780,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_char_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_char(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-boolean in k10477 in k8569 */
static void C_ccall f_10791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10791,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_boolean_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_boolean(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-locative in k10477 in k8569 */
static void C_ccall f_10802(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10802,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_locative_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_locative(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-integer in k10477 in k8569 */
static void C_ccall f_10813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,3)))){
C_save_and_reclaim((void*)f_10813,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(C_i_integerp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t3))){
/* library.scm:1195: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_get_rest_arg(c,3,av,3,t0);
tp(4,av2);}}
else{
/* library.scm:1195: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}}}

/* ##sys#check-exact-integer in k10477 in k8569 */
static void C_ccall f_10830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,3)))){
C_save_and_reclaim((void*)f_10830,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(C_i_exact_integerp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t3))){
/* library.scm:1199: ##sys#error-bad-exact-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[361]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[361]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_get_rest_arg(c,3,av,3,t0);
tp(4,av2);}}
else{
/* library.scm:1199: ##sys#error-bad-exact-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[361]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[361]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}}}

/* ##sys#check-exact-uinteger in k10477 in k8569 */
static void C_ccall f_10847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,3)))){
C_save_and_reclaim((void*)f_10847,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
t4=C_i_not(C_i_exact_integerp(t2));
t5=(C_truep(t4)?t4:C_i_integer_negativep(t2));
if(C_truep(t5)){
if(C_truep(C_i_pairp(t3))){
/* library.scm:1204: ##sys#error-bad-exact-uinteger */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[363]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[363]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_get_rest_arg(c,3,av,3,t0);
tp(4,av2);}}
else{
/* library.scm:1204: ##sys#error-bad-exact-uinteger */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[363]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[363]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* ##sys#check-real in k10477 in k8569 */
static void C_ccall f_10871(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,3)))){
C_save_and_reclaim((void*)f_10871,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(C_i_realp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t3))){
/* library.scm:1208: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_get_rest_arg(c,3,av,3,t0);
tp(4,av2);}}
else{
/* library.scm:1208: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}}}

/* ##sys#check-range in k10477 in k8569 */
static void C_ccall f_10888(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c<5) C_bad_min_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-5)*C_SIZEOF_PAIR +7,c,3)))){
C_save_and_reclaim((void*)f_10888,c,av);}
a=C_alloc((c-5)*C_SIZEOF_PAIR+7);
t5=C_build_rest(&a,c,5,av);
C_word t6;
C_word t7;
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10892,a[2]=t3,a[3]=t2,a[4]=t4,a[5]=t1,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* library.scm:1211: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t6;
av2[2]=t2;
av2[3]=t5;
tp(4,av2);}}

/* k10890 in ##sys#check-range in k10477 in k8569 */
static void C_ccall f_10892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_10892,c,av);}
t2=C_fixnum_less_or_equal_p(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(C_truep(t2)?C_fixnum_lessp(((C_word*)t0)[3],((C_word*)t0)[4]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_OUT_OF_RANGE_ERROR);
if(C_truep(C_i_pairp(((C_word*)t0)[6]))){
/* library.scm:1213: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=t4;
av2[3]=C_u_i_car(((C_word*)t0)[6]);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[2];
av2[6]=((C_word*)t0)[4];
tp(7,av2);}}
else{
/* library.scm:1213: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=t4;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[2];
av2[6]=((C_word*)t0)[4];
tp(7,av2);}}}}

/* ##sys#check-special in k10477 in k8569 */
static void C_ccall f_10918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,5)))){
C_save_and_reclaim((void*)f_10918,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=(C_truep(C_blockp(t2))?C_specialp(t2):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t3))){
/* library.scm:1219: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word av2[6];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=C_get_rest_arg(c,3,av,3,t0);
av2[4]=lf[368];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
/* library.scm:1219: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=C_SCHEME_FALSE;
av2[4]=lf[368];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}}}

/* ##sys#check-closure in k10477 in k8569 */
static void C_ccall f_10938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_10938,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_closure_2(t2,C_get_rest_arg(c,3,av,3,t0)):C_i_check_closure(t2));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#force in k10477 in k8569 */
static void C_ccall f_10949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10949,c,av);}
a=C_alloc(6);
if(C_truep(C_i_structurep(t2,lf[218]))){
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10961,a[2]=t4,a[3]=((C_word)li276),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_10961(t6,t1,t2,C_SCHEME_FALSE);}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* lp in scheme#force in k10477 in k8569 */
static void C_fcall f_10961(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_10961,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_nullp(t4))){
/* library.scm:1232: ##sys#values */{
C_word av2[2];
av2[0]=0;
av2[1]=t1;
C_values(2,av2);}}
else{
if(C_truep(C_i_pairp(t4))){{
C_word av2[3];
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
C_apply_values(3,av2);}}
else{
if(C_truep(C_i_closurep(t4))){
t5=(C_truep(t3)?C_i_setslot(t3,C_fix(1),t2):C_SCHEME_UNDEFINED);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10995,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* library.scm:1236: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=t6;
av2[2]=t4;
av2[3]=*((C_word*)lf[371]+1);
C_call_with_values(4,av2);}}
else{
if(C_truep(C_i_structurep(t4,lf[218]))){
/* library.scm:1248: lp */
t8=t1;
t9=t4;
t10=t3;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}}}

/* k10993 in lp in scheme#force in k10477 in k8569 */
static void C_ccall f_10995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_10995,c,av);}
a=C_alloc(6);
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_closurep(t2);
if(C_truep(C_i_not(t3))){
/* library.scm:1238: lp */
t4=((C_word*)((C_word*)t0)[3])[1];
f_10961(t4,((C_word*)t0)[4],((C_word*)t0)[2],((C_word*)t0)[5]);}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11010,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t5=C_i_nullp(t1);
if(C_truep(C_i_not(t5))){
t6=C_i_cdr(t1);
t7=C_i_nullp(t6);
t8=t4;
f_11010(t8,(C_truep(t7)?C_i_structurep(C_slot(t1,C_fix(0)),lf[218]):C_SCHEME_FALSE));}
else{
t6=t4;
f_11010(t6,C_SCHEME_FALSE);}}}

/* k11008 in k10993 in lp in scheme#force in k10477 in k8569 */
static void C_fcall f_11010(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_11010,2,t0,t1);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(0));
t3=C_slot(t2,C_fix(1));
t4=C_i_setslot(((C_word*)t0)[3],C_fix(1),t3);
/* library.scm:1243: lp */
t5=((C_word*)((C_word*)t0)[4])[1];
f_10961(t5,((C_word*)t0)[5],((C_word*)t0)[3],t2);}
else{
t2=C_i_setslot(((C_word*)t0)[3],C_fix(1),((C_word*)t0)[2]);{
C_word av2[3];
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}}

/* ##sys#error-not-a-proper-list in k10477 in k8569 */
static void C_ccall f_11077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11077,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_NOT_A_PROPER_LIST_ERROR);
/* library.scm:1258: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_NOT_A_PROPER_LIST_ERROR);
/* library.scm:1258: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-number in k10477 in k8569 */
static void C_ccall f_11093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11093,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_NUMBER_ERROR);
/* library.scm:1262: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_NUMBER_ERROR);
/* library.scm:1262: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-integer in k10477 in k8569 */
static void C_ccall f_11109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11109,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_INTEGER_ERROR);
/* library.scm:1266: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_INTEGER_ERROR);
/* library.scm:1266: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-exact-integer in k10477 in k8569 */
static void C_ccall f_11125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11125,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_INTEGER_ERROR);
/* library.scm:1270: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_INTEGER_ERROR);
/* library.scm:1270: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-exact-uinteger in k10477 in k8569 */
static void C_ccall f_11141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11141,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_UINTEGER_ERROR);
/* library.scm:1274: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_UINTEGER_ERROR);
/* library.scm:1274: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-inexact in k10477 in k8569 */
static void C_ccall f_11157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11157,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_CANT_REPRESENT_INEXACT_ERROR);
/* library.scm:1278: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_CANT_REPRESENT_INEXACT_ERROR);
/* library.scm:1278: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-real in k10477 in k8569 */
static void C_ccall f_11173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11173,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_REAL_ERROR);
/* library.scm:1282: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_REAL_ERROR);
/* library.scm:1282: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* ##sys#error-bad-base in k10477 in k8569 */
static void C_ccall f_11189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11189,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=C_fix((C_word)C_BAD_ARGUMENT_TYPE_BAD_BASE_ERROR);
/* library.scm:1286: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
tp(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_BAD_BASE_ERROR);
/* library.scm:1286: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[5];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t3;
av2[4]=t2;
tp(5,av2);}}}

/* scheme#append in k10477 in k8569 */
static void C_ccall f_11205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_11205,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11217,a[2]=t5,a[3]=((C_word)li289),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_11217(t7,t1,t2);}}

/* loop in scheme#append in k10477 in k8569 */
static void C_fcall f_11217(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_11217,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_slot(t2,C_fix(1));
t4=C_eqp(t3,C_SCHEME_END_OF_LIST);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_slot(t2,C_fix(0));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11236,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t7,a[5]=((C_word)li288),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_11236(t9,t1,t5);}}

/* copy in loop in scheme#append in k10477 in k8569 */
static void C_fcall f_11236(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_11236,3,t0,t1,t2);}
a=C_alloc(4);
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
/* library.scm:1297: loop */
t4=((C_word*)((C_word*)t0)[2])[1];
f_11217(t4,t1,C_slot(((C_word*)t0)[3],C_fix(1)));}
else{
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11267,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1299: copy */
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
/* library.scm:1301: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=C_slot(((C_word*)t0)[3],C_fix(0));
av2[3]=lf[378];
tp(4,av2);}}}}

/* k11265 in copy in loop in scheme#append in k10477 in k8569 */
static void C_ccall f_11267(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_11267,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#fast-reverse in k10477 in k8569 */
static void C_ccall f_11284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11284,c,av);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11290,a[2]=t4,a[3]=((C_word)li291),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_11290(t6,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in ##sys#fast-reverse in k10477 in k8569 */
static void C_fcall f_11290(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_11290,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=C_slot(t2,C_fix(0));
t6=C_a_i_cons(&a,2,t5,t3);
/* library.scm:1307: loop */
t8=t1;
t9=t4;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* ##sys#make-string in k10477 in k8569 */
static void C_ccall f_11314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11314,c,av);}
if(C_truep(C_rest_nullp(c,3))){
/* library.scm:1314: ##sys#allocate-vector */
t3=*((C_word*)lf[318]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=C_make_character(32);
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
/* library.scm:1314: ##sys#allocate-vector */
t3=*((C_word*)lf[318]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=C_get_rest_arg(c,3,av,3,t0);
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}

/* scheme#make-string in k10477 in k8569 */
static void C_ccall f_11334(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_11334,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11338,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1321: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[382];
tp(4,av2);}}

/* k11336 in scheme#make-string in k10477 in k8569 */
static void C_ccall f_11338(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_11338,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11341,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnum_lessp(((C_word*)t0)[4],C_fix(0)))){
/* library.scm:1323: ##sys#signal-hook */
t3=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[383];
av2[3]=lf[382];
av2[4]=lf[384];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_11341(2,av2);}}}

/* k11339 in k11336 in scheme#make-string in k10477 in k8569 */
static void C_ccall f_11341(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_11341,c,av);}
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
/* library.scm:1314: ##sys#allocate-vector */
t2=*((C_word*)lf[318]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_make_character(32);
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_check_char_2(t2,lf[382]);
/* library.scm:1314: ##sys#allocate-vector */
t4=*((C_word*)lf[318]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=t2;
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}}

/* scheme#string->list in k10477 in k8569 */
static void C_ccall f_11370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_11370,c,av);}
a=C_alloc(7);
t3=C_i_check_string_2(t2,lf[386]);
t4=C_block_size(t2);
t5=C_fixnum_difference(t4,C_fix(1));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11386,a[2]=t2,a[3]=t7,a[4]=((C_word)li295),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_11386(t9,t1,t5,C_SCHEME_END_OF_LIST);}

/* loop in scheme#string->list in k10477 in k8569 */
static void C_fcall f_11386(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_11386,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_fixnum_lessp(t2,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fixnum_difference(t2,C_fix(1));
t5=C_subchar(((C_word*)t0)[2],t2);
t6=C_a_i_cons(&a,2,t5,t3);
/* library.scm:1339: loop */
t8=t1;
t9=t4;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}}

/* scheme#list->string in k10477 in k8569 */
static void C_ccall f_11408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11408,c,av);}
a=C_alloc(5);
t3=C_i_listp(t2);
if(C_truep(C_i_not(t3))){
/* library.scm:1347: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[389];
tp(4,av2);}}
else{
t4=C_i_length(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11424,a[2]=t4,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1349: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}}

/* k11422 in scheme#list->string in k10477 in k8569 */
static void C_ccall f_11424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11424,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11429,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li297),tmp=(C_word)a,a+=5,tmp);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_11429(t2,C_fix(0),((C_word*)t0)[4])
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* doloop1528 in k11422 in scheme#list->string in k10477 in k8569 */
static C_word C_fcall f_11429(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t1,((C_word*)t0)[2]))){
return(((C_word*)t0)[3]);}
else{
t3=C_slot(t2,C_fix(0));
t4=C_i_check_char_2(t3,lf[389]);
t5=C_setsubchar(((C_word*)t0)[3],t1,t3);
t7=C_fixnum_plus(t1,C_fix(1));
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}}

/* ##sys#reverse-list->string in k10477 in k8569 */
static void C_ccall f_11460(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11460,c,av);}
a=C_alloc(5);
if(C_truep(C_i_listp(t2))){
t3=C_u_i_length(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11471,a[2]=t3,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1364: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}
else{
/* library.scm:1372: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[392];
tp(4,av2);}}}

/* k11469 in ##sys#reverse-list->string in k10477 in k8569 */
static void C_ccall f_11471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11471,c,av);}
a=C_alloc(4);
t2=C_fixnum_difference(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11480,a[2]=t1,a[3]=((C_word)li299),tmp=(C_word)a,a+=4,tmp);
t4=(
  f_11480(t3,((C_word*)t0)[3],t2)
);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* iter in k11469 in ##sys#reverse-list->string in k10477 in k8569 */
static C_word C_fcall f_11480(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t2,C_fix(0)))){
t3=C_slot(t1,C_fix(0));
t4=C_i_check_char_2(t3,lf[392]);
t5=C_setsubchar(((C_word*)t0)[2],t2,t3);
t7=C_slot(t1,C_fix(1));
t8=C_fixnum_difference(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=C_SCHEME_UNDEFINED;
return(t3);}}

/* scheme#string-fill! in k10477 in k8569 */
static void C_ccall f_11509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11509,c,av);}
t4=C_i_check_string_2(t2,lf[394]);
t5=C_i_check_char_2(t3,lf[394]);
t6=C_block_size(t2);
t7=C_set_memory(t2,t3,t6);
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* scheme#string-copy in k10477 in k8569 */
static void C_ccall f_11522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11522,c,av);}
a=C_alloc(5);
t3=C_i_check_string_2(t2,lf[396]);
t4=C_block_size(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11532,a[2]=t2,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1385: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k11530 in scheme#string-copy in k10477 in k8569 */
static void C_ccall f_11532(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11532,c,av);}
t2=C_copy_memory(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#substring in k10477 in k8569 */
static void C_ccall f_11534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_11534,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+6);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=C_i_check_string_2(t2,lf[398]);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11541,a[2]=t2,a[3]=t1,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:1392: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t6;
av2[2]=t3;
av2[3]=lf[398];
tp(4,av2);}}

/* k11539 in scheme#substring in k10477 in k8569 */
static void C_ccall f_11541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_11541,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11544,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[5]))){
t3=C_u_i_car(((C_word*)t0)[5]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11579,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1395: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[398];
tp(4,av2);}}
else{
t3=t2;
f_11544(t3,C_block_size(((C_word*)t0)[2]));}}

/* k11542 in k11539 in scheme#substring in k10477 in k8569 */
static void C_fcall f_11544(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11544,2,t0,t1);}
a=C_alloc(6);
t2=C_block_size(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11553,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnum_less_or_equal_p(((C_word*)t0)[4],t1))){
t4=C_fixnum_greater_or_equal_p(((C_word*)t0)[4],C_fix(0));
t5=t3;
f_11553(t5,(C_truep(t4)?C_fixnum_less_or_equal_p(t1,t2):C_SCHEME_FALSE));}
else{
t4=t3;
f_11553(t4,C_SCHEME_FALSE);}}

/* k11551 in k11542 in k11539 in scheme#substring in k10477 in k8569 */
static void C_fcall f_11553(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_11553,2,t0,t1);}
if(C_truep(t1)){
/* library.scm:1402: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
tp(5,av2);}}
else{
t2=C_fix((C_word)C_OUT_OF_RANGE_ERROR);
/* library.scm:1403: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word av2[6];
av2[0]=*((C_word*)lf[366]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=lf[398];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
tp(6,av2);}}}

/* k11577 in k11539 in scheme#substring in k10477 in k8569 */
static void C_ccall f_11579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11579,c,av);}
t2=((C_word*)t0)[2];
f_11544(t2,((C_word*)t0)[3]);}

/* ##sys#substring in k10477 in k8569 */
static void C_ccall f_11584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_11584,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11588,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:1408: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=C_fixnum_difference(t4,t3);
tp(3,av2);}}

/* k11586 in ##sys#substring in k10477 in k8569 */
static void C_ccall f_11588(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11588,c,av);}
t2=C_substring_copy(((C_word*)t0)[2],t1,((C_word*)t0)[3],((C_word*)t0)[4],C_fix(0));
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* compare1581 in k10477 in k8569 */
static void C_ccall f_11593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11593,c,av);}
t6=C_i_check_string_2(t2,t4);
t7=C_i_check_string_2(t3,t4);
t8=C_block_size(t2);
t9=C_block_size(t3);
if(C_truep(C_fixnum_lessp(t8,t9))){
/* library.scm:1418: k */
t10=t5;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
av2[4]=C_string_compare(t2,t3,t8);
((C_proc)C_fast_retrieve_proc(t10))(5,av2);}}
else{
/* library.scm:1418: k */
t10=t5;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
av2[4]=C_string_compare(t2,t3,t9);
((C_proc)C_fast_retrieve_proc(t10))(5,av2);}}}

/* scheme#string<? in k10477 in k8569 */
static void C_ccall f_11614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11614,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11620,a[2]=((C_word)li306),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1426: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[401];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11619 in scheme#string<? in k10477 in k8569 */
static void C_ccall f_11620(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11620,c,av);}
t5=C_fixnum_lessp(t4,C_fix(0));
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_fixnum_lessp(t2,t3);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_eqp(t4,C_fix(0)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* scheme#string>? in k10477 in k8569 */
static void C_ccall f_11638(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11638,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11644,a[2]=((C_word)li308),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1433: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[403];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11643 in scheme#string>? in k10477 in k8569 */
static void C_ccall f_11644(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11644,c,av);}
t5=C_fixnum_greaterp(t4,C_fix(0));
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_fixnum_lessp(t3,t2);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_eqp(t4,C_fix(0)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* scheme#string<=? in k10477 in k8569 */
static void C_ccall f_11662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11662,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11668,a[2]=((C_word)li310),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1440: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[405];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11667 in scheme#string<=? in k10477 in k8569 */
static void C_ccall f_11668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11668,c,av);}
t5=C_eqp(t4,C_fix(0));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_fixnum_less_or_equal_p(t2,t3):C_fixnum_lessp(t4,C_fix(0)));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* scheme#string>=? in k10477 in k8569 */
static void C_ccall f_11683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11683,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11689,a[2]=((C_word)li312),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1447: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[407];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11688 in scheme#string>=? in k10477 in k8569 */
static void C_ccall f_11689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11689,c,av);}
t5=C_eqp(t4,C_fix(0));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_fixnum_greater_or_equal_p(t2,t3):C_fixnum_greaterp(t4,C_fix(0)));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* compare1624 in k10477 in k8569 */
static void C_ccall f_11703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11703,c,av);}
t6=C_i_check_string_2(t2,t4);
t7=C_i_check_string_2(t3,t4);
t8=C_block_size(t2);
t9=C_block_size(t3);
if(C_truep(C_fixnum_lessp(t8,t9))){
/* library.scm:1460: k */
t10=t5;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
av2[4]=C_string_compare_case_insensitive(t2,t3,t8);
((C_proc)C_fast_retrieve_proc(t10))(5,av2);}}
else{
/* library.scm:1460: k */
t10=t5;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
av2[4]=C_string_compare_case_insensitive(t2,t3,t9);
((C_proc)C_fast_retrieve_proc(t10))(5,av2);}}}

/* scheme#string-ci<? in k10477 in k8569 */
static void C_ccall f_11724(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11724,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11730,a[2]=((C_word)li315),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1468: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[409];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11729 in scheme#string-ci<? in k10477 in k8569 */
static void C_ccall f_11730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11730,c,av);}
t5=C_fixnum_lessp(t4,C_fix(0));
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_fixnum_lessp(t2,t3);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_eqp(t4,C_fix(0)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* scheme#string-ci>? in k10477 in k8569 */
static void C_ccall f_11748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11748,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11754,a[2]=((C_word)li317),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1475: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[411];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11753 in scheme#string-ci>? in k10477 in k8569 */
static void C_ccall f_11754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11754,c,av);}
t5=C_fixnum_greaterp(t4,C_fix(0));
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_fixnum_lessp(t3,t2);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_eqp(t4,C_fix(0)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* scheme#string-ci<=? in k10477 in k8569 */
static void C_ccall f_11772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11772,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11778,a[2]=((C_word)li319),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1482: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[413];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11777 in scheme#string-ci<=? in k10477 in k8569 */
static void C_ccall f_11778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11778,c,av);}
t5=C_eqp(t4,C_fix(0));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_fixnum_less_or_equal_p(t2,t3):C_fixnum_lessp(t4,C_fix(0)));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* scheme#string-ci>=? in k10477 in k8569 */
static void C_ccall f_11793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_11793,c,av);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11799,a[2]=((C_word)li321),tmp=(C_word)a,a+=3,tmp);
/* library.scm:1489: compare */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=lf[415];
av2[5]=t4;
((C_proc)C_fast_retrieve_proc(t5))(6,av2);}}

/* a11798 in scheme#string-ci>=? in k10477 in k8569 */
static void C_ccall f_11799(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11799,c,av);}
t5=C_eqp(t4,C_fix(0));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_fixnum_greater_or_equal_p(t2,t3):C_fixnum_greaterp(t4,C_fix(0)));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* ##sys#string-append in k10477 in k8569 */
static void C_ccall f_11814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_11814,c,av);}
a=C_alloc(7);
t4=C_block_size(t2);
t5=C_block_size(t3);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11824,a[2]=t2,a[3]=t4,a[4]=t3,a[5]=t5,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:1499: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t6;
av2[2]=C_fixnum_plus(t4,t5);
tp(3,av2);}}

/* k11822 in ##sys#string-append in k10477 in k8569 */
static void C_ccall f_11824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11824,c,av);}
t2=C_substring_copy(((C_word*)t0)[2],t1,C_fix(0),((C_word*)t0)[3],C_fix(0));
t3=C_substring_copy(((C_word*)t0)[4],t1,C_fix(0),((C_word*)t0)[5],((C_word*)t0)[3]);
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#string-append in k10477 in k8569 */
static void C_ccall f_11830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +13,c,4)))){
C_save_and_reclaim((void*)f_11830,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+13);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
t3=C_SCHEME_FALSE;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11834,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11836,a[2]=t4,a[3]=t7,a[4]=((C_word)li324),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_11836(t9,t5,t2,C_fix(0));}

/* k11832 in scheme#string-append in k10477 in k8569 */
static void C_ccall f_11834(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11834,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in scheme#string-append in k10477 in k8569 */
static void C_fcall f_11836(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_11836,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11847,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1509: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=t3;
tp(3,av2);}}
else{
t5=C_slot(t2,C_fix(0));
t6=C_i_check_string_2(t5,lf[418]);
t7=C_block_size(t5);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11859,a[2]=t1,a[3]=t5,a[4]=((C_word*)t0)[2],a[5]=t7,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* library.scm:1513: loop */
t10=t8;
t11=C_slot(t2,C_fix(1));
t12=C_fixnum_plus(t3,t7);
t1=t10;
t2=t11;
t3=t12;
goto loop;}}

/* k11845 in loop in scheme#string-append in k10477 in k8569 */
static void C_ccall f_11847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11847,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11857 in loop in scheme#string-append in k10477 in k8569 */
static void C_ccall f_11859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11859,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_substring_copy(((C_word*)t0)[3],((C_word*)((C_word*)t0)[4])[1],C_fix(0),((C_word*)t0)[5],((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#string in k10477 in k8569 */
static void C_ccall f_11869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,2)))){
C_save_and_reclaim((void*)f_11869,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
/* ##sys#list->string */
t3=*((C_word*)lf[390]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* ##sys#fragments->string in k10477 in k8569 */
static void C_ccall f_11875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11875,c,av);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11879,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1522: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}

/* k11877 in ##sys#fragments->string in k10477 in k8569 */
static void C_ccall f_11879(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11879,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11884,a[2]=t1,a[3]=((C_word)li327),tmp=(C_word)a,a+=4,tmp);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_11884(t2,((C_word*)t0)[3],C_fix(0))
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k11877 in ##sys#fragments->string in k10477 in k8569 */
static C_word C_fcall f_11884(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t1))){
return(((C_word*)t0)[2]);}
else{
t3=C_slot(t1,C_fix(0));
t4=C_block_size(t3);
t5=C_substring_copy(t3,((C_word*)t0)[2],C_fix(0),t4,t2);
t7=C_slot(t1,C_fix(1));
t8=C_fixnum_plus(t2,t4);
t1=t7;
t2=t8;
goto loop;}}

/* chicken.base#chop in k10477 in k8569 */
static void C_ccall f_11910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11910,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11914,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1533: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[422];
tp(4,av2);}}

/* k11912 in chicken.base#chop in k10477 in k8569 */
static void C_ccall f_11914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11914,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11917,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnum_less_or_equal_p(((C_word*)t0)[3],C_fix(0)))){
/* library.scm:1534: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[422];
av2[3]=lf[423];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_11917(2,av2);}}}

/* k11915 in k11912 in chicken.base#chop in k10477 in k8569 */
static void C_ccall f_11917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_11917,c,av);}
a=C_alloc(7);
t2=C_i_length(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11925,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word)li330),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_11925(t6,((C_word*)t0)[4],((C_word*)t0)[2],t2);}

/* loop in k11915 in k11912 in chicken.base#chop in k10477 in k8569 */
static void C_fcall f_11925(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_11925,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(t3,((C_word*)t0)[2]))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_list1(&a,1,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11946,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t5,a[6]=((C_word)li329),tmp=(C_word)a,a+=7,tmp));
t7=((C_word*)t5)[1];
f_11946(t7,t1,C_SCHEME_END_OF_LIST,t2,((C_word*)t0)[2]);}}}

/* doloop1709 in loop in k11915 in k11912 in chicken.base#chop in k10477 in k8569 */
static void C_fcall f_11946(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_11946,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_eqp(t4,C_fix(0));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11960,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* library.scm:1544: scheme#reverse */
t7=*((C_word*)lf[46]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=C_slot(t3,C_fix(0));
t7=C_a_i_cons(&a,2,t6,t2);
t9=t1;
t10=t7;
t11=C_slot(t3,C_fix(1));
t12=C_fixnum_difference(t4,C_fix(1));
t1=t9;
t2=t10;
t3=t11;
t4=t12;
goto loop;}}

/* k11958 in doloop1709 in loop in k11915 in k11912 in chicken.base#chop in k10477 in k8569 */
static void C_ccall f_11960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11960,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11964,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1544: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_11925(t3,t2,((C_word*)t0)[4],C_fixnum_difference(((C_word*)t0)[5],((C_word*)t0)[6]));}

/* k11962 in k11958 in doloop1709 in loop in k11915 in k11912 in chicken.base#chop in k10477 in k8569 */
static void C_ccall f_11964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_11964,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp+ in k10477 in k8569 */
static void C_ccall f_12005(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12005,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12027,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_flonum(&a,C_ub_i_flonum_plus(C_flonum_magnitude(t2),C_flonum_magnitude(t3)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[436];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12025 in chicken.flonum#fp+ in k10477 in k8569 */
static void C_ccall f_12027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12027,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_ub_i_flonum_plus(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp- in k10477 in k8569 */
static void C_ccall f_12029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12029,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12051,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_flonum(&a,C_ub_i_flonum_difference(C_flonum_magnitude(t2),C_flonum_magnitude(t3)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[438];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12049 in chicken.flonum#fp- in k10477 in k8569 */
static void C_ccall f_12051(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12051,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_ub_i_flonum_difference(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp* in k10477 in k8569 */
static void C_ccall f_12053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12053,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12075,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_flonum(&a,C_ub_i_flonum_times(C_flonum_magnitude(t2),C_flonum_magnitude(t3)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[440];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12073 in chicken.flonum#fp* in k10477 in k8569 */
static void C_ccall f_12075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12075,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_ub_i_flonum_times(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp/ in k10477 in k8569 */
static void C_ccall f_12077(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12077,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12099,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_flonum(&a,C_ub_i_flonum_quotient(C_flonum_magnitude(t2),C_flonum_magnitude(t3)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[442];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12097 in chicken.flonum#fp/ in k10477 in k8569 */
static void C_ccall f_12099(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12099,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_ub_i_flonum_quotient(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpgcd in k10477 in k8569 */
static void C_ccall f_12101(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12101,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12123,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_flonum_gcd(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[444];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12121 in chicken.flonum#fpgcd in k10477 in k8569 */
static void C_ccall f_12123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12123,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_flonum_gcd(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp/? in k10477 in k8569 */
static void C_ccall f_12125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12125,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12147,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_flonum_quotient_checked(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[446];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12145 in chicken.flonum#fp/? in k10477 in k8569 */
static void C_ccall f_12147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12147,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_flonum_quotient_checked(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp= in k10477 in k8569 */
static void C_ccall f_12149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12149,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12171,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_ub_i_flonum_equalp(C_flonum_magnitude(t2),C_flonum_magnitude(t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[448];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12169 in chicken.flonum#fp= in k10477 in k8569 */
static void C_ccall f_12171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12171,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_ub_i_flonum_equalp(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp> in k10477 in k8569 */
static void C_ccall f_12173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12173,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12195,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_ub_i_flonum_greaterp(C_flonum_magnitude(t2),C_flonum_magnitude(t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[450];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12193 in chicken.flonum#fp> in k10477 in k8569 */
static void C_ccall f_12195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12195,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_ub_i_flonum_greaterp(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp< in k10477 in k8569 */
static void C_ccall f_12197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12197,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12219,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_ub_i_flonum_lessp(C_flonum_magnitude(t2),C_flonum_magnitude(t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[452];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12217 in chicken.flonum#fp< in k10477 in k8569 */
static void C_ccall f_12219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12219,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_ub_i_flonum_lessp(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp>= in k10477 in k8569 */
static void C_ccall f_12221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12221,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12243,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_ub_i_flonum_greater_or_equal_p(C_flonum_magnitude(t2),C_flonum_magnitude(t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[454];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12241 in chicken.flonum#fp>= in k10477 in k8569 */
static void C_ccall f_12243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12243,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_ub_i_flonum_greater_or_equal_p(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fp<= in k10477 in k8569 */
static void C_ccall f_12245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12245,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12267,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_ub_i_flonum_less_or_equal_p(C_flonum_magnitude(t2),C_flonum_magnitude(t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[456];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12265 in chicken.flonum#fp<= in k10477 in k8569 */
static void C_ccall f_12267(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12267,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_ub_i_flonum_less_or_equal_p(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpneg in k10477 in k8569 */
static void C_ccall f_12269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12269,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12285,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_flonum_negate(&a,1,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[458];
av2[4]=t2;
tp(5,av2);}}}

/* k12283 in chicken.flonum#fpneg in k10477 in k8569 */
static void C_ccall f_12285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12285,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_flonum_negate(&a,1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpmax in k10477 in k8569 */
static void C_ccall f_12287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12287,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12309,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_flonum_max(t2,t3);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[460];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12307 in chicken.flonum#fpmax in k10477 in k8569 */
static void C_ccall f_12309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12309,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_flonum_max(((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpmin in k10477 in k8569 */
static void C_ccall f_12311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12311,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12333,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_flonum_min(t2,t3);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[462];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12331 in chicken.flonum#fpmin in k10477 in k8569 */
static void C_ccall f_12333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12333,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_flonum_min(((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpfloor in k10477 in k8569 */
static void C_ccall f_12335(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12335,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12351,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_floor(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[464];
av2[4]=t2;
tp(5,av2);}}}

/* k12349 in chicken.flonum#fpfloor in k10477 in k8569 */
static void C_ccall f_12351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12351,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_floor(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fptruncate in k10477 in k8569 */
static void C_ccall f_12353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12353,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12369,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_trunc(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[466];
av2[4]=t2;
tp(5,av2);}}}

/* k12367 in chicken.flonum#fptruncate in k10477 in k8569 */
static void C_ccall f_12369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12369,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_trunc(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpround in k10477 in k8569 */
static void C_ccall f_12371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12371,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12387,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_round(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[468];
av2[4]=t2;
tp(5,av2);}}}

/* k12385 in chicken.flonum#fpround in k10477 in k8569 */
static void C_ccall f_12387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12387,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_round(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpceiling in k10477 in k8569 */
static void C_ccall f_12389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12389,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12405,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_ceil(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[470];
av2[4]=t2;
tp(5,av2);}}}

/* k12403 in chicken.flonum#fpceiling in k10477 in k8569 */
static void C_ccall f_12405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12405,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_ceil(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpsin in k10477 in k8569 */
static void C_ccall f_12407(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12407,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12423,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_sin(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[472];
av2[4]=t2;
tp(5,av2);}}}

/* k12421 in chicken.flonum#fpsin in k10477 in k8569 */
static void C_ccall f_12423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12423,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_sin(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpcos in k10477 in k8569 */
static void C_ccall f_12425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12425,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12441,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_cos(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[474];
av2[4]=t2;
tp(5,av2);}}}

/* k12439 in chicken.flonum#fpcos in k10477 in k8569 */
static void C_ccall f_12441(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12441,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_cos(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fptan in k10477 in k8569 */
static void C_ccall f_12443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12443,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12459,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_tan(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[476];
av2[4]=t2;
tp(5,av2);}}}

/* k12457 in chicken.flonum#fptan in k10477 in k8569 */
static void C_ccall f_12459(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12459,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_tan(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpasin in k10477 in k8569 */
static void C_ccall f_12461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12461,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12477,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_asin(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[478];
av2[4]=t2;
tp(5,av2);}}}

/* k12475 in chicken.flonum#fpasin in k10477 in k8569 */
static void C_ccall f_12477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12477,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_asin(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpacos in k10477 in k8569 */
static void C_ccall f_12479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12479,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12495,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_acos(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[480];
av2[4]=t2;
tp(5,av2);}}}

/* k12493 in chicken.flonum#fpacos in k10477 in k8569 */
static void C_ccall f_12495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12495,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_acos(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpatan in k10477 in k8569 */
static void C_ccall f_12497(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12497,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12513,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_atan(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[482];
av2[4]=t2;
tp(5,av2);}}}

/* k12511 in chicken.flonum#fpatan in k10477 in k8569 */
static void C_ccall f_12513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12513,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_atan(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpatan2 in k10477 in k8569 */
static void C_ccall f_12515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12515,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12537,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_flonum(&a,C_atan2(C_flonum_magnitude(t2),C_flonum_magnitude(t3)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[484];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12535 in chicken.flonum#fpatan2 in k10477 in k8569 */
static void C_ccall f_12537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12537,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_atan2(C_flonum_magnitude(((C_word*)t0)[3]),C_flonum_magnitude(((C_word*)t0)[4])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpexp in k10477 in k8569 */
static void C_ccall f_12539(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12539,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12555,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_exp(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[486];
av2[4]=t2;
tp(5,av2);}}}

/* k12553 in chicken.flonum#fpexp in k10477 in k8569 */
static void C_ccall f_12555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12555,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_exp(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpexpt in k10477 in k8569 */
static void C_ccall f_12557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_12557,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12579,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=C_i_flonump(t2);
t6=(C_truep(t5)?C_i_flonump(t3):C_SCHEME_FALSE);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_flonum_expt(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1575: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t7;
av2[3]=lf[488];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}}

/* k12577 in chicken.flonum#fpexpt in k10477 in k8569 */
static void C_ccall f_12579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12579,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_flonum_expt(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fplog in k10477 in k8569 */
static void C_ccall f_12581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12581,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12597,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_log(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[490];
av2[4]=t2;
tp(5,av2);}}}

/* k12595 in chicken.flonum#fplog in k10477 in k8569 */
static void C_ccall f_12597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12597,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_log(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpsqrt in k10477 in k8569 */
static void C_ccall f_12599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12599,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12615,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_sqrt(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[492];
av2[4]=t2;
tp(5,av2);}}}

/* k12613 in chicken.flonum#fpsqrt in k10477 in k8569 */
static void C_ccall f_12615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12615,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_sqrt(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpabs in k10477 in k8569 */
static void C_ccall f_12617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_12617,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12633,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_flonum(&a,C_fabs(C_flonum_magnitude(t2)));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[494];
av2[4]=t2;
tp(5,av2);}}}

/* k12631 in chicken.flonum#fpabs in k10477 in k8569 */
static void C_ccall f_12633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_12633,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_flonum(&a,C_fabs(C_flonum_magnitude(((C_word*)t0)[3])));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#fpinteger? in k10477 in k8569 */
static void C_ccall f_12635(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12635,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12651,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_fpintegerp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fix((C_word)C_BAD_ARGUMENT_TYPE_NO_FLONUM_ERROR);
/* library.scm:1571: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[496];
av2[4]=t2;
tp(5,av2);}}}

/* k12649 in chicken.flonum#fpinteger? in k10477 in k8569 */
static void C_ccall f_12651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12651,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_u_i_fpintegerp(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.flonum#flonum-print-precision in k10477 in k8569 */
static void C_ccall f_12653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12653,c,av);}
a=C_alloc(5);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?C_SCHEME_FALSE:C_get_rest_arg(c,2,av,2,t0));
t4=C_get_print_precision();
if(C_truep(t3)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12663,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1704: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t5;
av2[2]=t3;
av2[3]=lf[498];
tp(4,av2);}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k12661 in chicken.flonum#flonum-print-precision in k10477 in k8569 */
static void C_ccall f_12663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12663,c,av);}
t2=C_set_print_precision(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* make-complex in k10477 in k8569 */
static void C_fcall f_12675(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_12675,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_eqp(t3,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12685,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=t4;
f_12685(2,av2);}}
else{
if(C_truep(C_i_flonump(t3))){
/* library.scm:1724: chicken.flonum#fp= */
t6=*((C_word*)lf[447]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
av2[3]=lf[504];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_12685(2,av2);}}}}

/* k12683 in make-complex in k10477 in k8569 */
static void C_ccall f_12685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12685,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12689,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_inexactp(((C_word*)t0)[4]))){
/* library.scm:1727: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
f_12689(2,av2);}}}}

/* k12687 in k12683 in make-complex in k10477 in k8569 */
static void C_ccall f_12689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,1)))){
C_save_and_reclaim((void *)f_12689,c,av);}
a=C_alloc(14);
if(C_truep(C_i_inexactp(((C_word*)t0)[2]))){
t2=C_a_i_exact_to_inexact(&a,1,((C_word*)t0)[3]);
t3=C_a_i_cplxnum(&a,2,t1,t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_a_i_cplxnum(&a,2,t1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* scheme#make-rectangular in k10477 in k8569 */
static void C_ccall f_12711(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12711,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12715,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1732: ##sys#check-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[364]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[364]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[506];
tp(4,av2);}}

/* k12713 in scheme#make-rectangular in k10477 in k8569 */
static void C_ccall f_12715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12715,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12718,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:1733: ##sys#check-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[364]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[364]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[506];
tp(4,av2);}}

/* k12716 in k12713 in scheme#make-rectangular in k10477 in k8569 */
static void C_ccall f_12718(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12718,c,av);}
/* library.scm:1734: make-complex */
f_12675(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);}

/* scheme#make-polar in k10477 in k8569 */
static void C_ccall f_12723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12723,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12727,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1738: ##sys#check-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[364]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[364]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[508];
tp(4,av2);}}

/* k12725 in scheme#make-polar in k10477 in k8569 */
static void C_ccall f_12727(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12727,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12730,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:1739: ##sys#check-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[364]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[364]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[508];
tp(4,av2);}}

/* k12728 in k12725 in scheme#make-polar in k10477 in k8569 */
static void C_ccall f_12730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12730,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12733,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:1740: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12731 in k12728 in k12725 in scheme#make-polar in k10477 in k8569 */
static void C_ccall f_12733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(74,c,3)))){
C_save_and_reclaim((void *)f_12733,c,av);}
a=C_alloc(74);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],C_a_i_cos(&a,1,t1));
t3=C_s_a_i_times(&a,2,((C_word*)t0)[2],C_a_i_sin(&a,1,t1));
/* library.scm:1741: make-complex */
f_12675(((C_word*)t0)[3],t2,t3);}

/* scheme#real-part in k10477 in k8569 */
static void C_ccall f_12746(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12746,c,av);}
if(C_truep(C_i_cplxnump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_cplxnum_real(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_numberp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:1749: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[510];
tp(4,av2);}}}}

/* scheme#imag-part in k10477 in k8569 */
static void C_ccall f_12769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12769,c,av);}
if(C_truep(C_i_cplxnump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_cplxnum_imag(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[504];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_numberp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:1756: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[512];
tp(4,av2);}}}}}

/* scheme#angle in k10477 in k8569 */
static void C_ccall f_12795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12795,c,av);}
a=C_alloc(4);
t3=C_i_check_number_2(t2,lf[514]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12804,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1762: scheme#imag-part */
t5=*((C_word*)lf[511]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k12802 in scheme#angle in k10477 in k8569 */
static void C_ccall f_12804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_12804,c,av);}
a=C_alloc(15);
t2=C_a_i_exact_to_inexact(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12809,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1763: scheme#real-part */
t4=*((C_word*)lf[509]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12807 in k12802 in scheme#angle in k10477 in k8569 */
static void C_ccall f_12809(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,1)))){
C_save_and_reclaim((void *)f_12809,c,av);}
a=C_alloc(15);
t2=C_a_i_exact_to_inexact(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_atan2(&a,2,((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#magnitude in k10477 in k8569 */
static void C_ccall f_12811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(95,c,3)))){
C_save_and_reclaim((void *)f_12811,c,av);}
a=C_alloc(95);
if(C_truep(C_i_cplxnump(t2))){
t3=C_u_i_cplxnum_real(t2);
t4=C_u_i_cplxnum_imag(t2);
t5=C_s_a_i_times(&a,2,t3,t3);
t6=C_s_a_i_times(&a,2,t4,t4);
t7=C_s_a_i_plus(&a,2,t5,t6);
/* library.scm:1770: scheme#sqrt */
t8=*((C_word*)lf[516]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t1;
av2[2]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
if(C_truep(C_i_numberp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_abs(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:1772: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[517];
tp(4,av2);}}}}

/* ratnum in k10477 in k8569 */
static C_word C_fcall f_12855(C_word *a,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;{}
switch(t2){
case C_fix(1):
return(t1);
case C_fix(-1):
return(C_s_a_u_i_integer_negate(&a,1,t1));
default:
if(C_truep(C_i_negativep(t2))){
t3=C_s_a_u_i_integer_negate(&a,1,t1);
t4=C_s_a_u_i_integer_negate(&a,1,t2);
return(C_a_i_ratnum(&a,2,t3,t4));}
else{
return(C_a_i_ratnum(&a,2,t1,t2));}}}

/* scheme#numerator in k10477 in k8569 */
static void C_ccall f_12901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_12901,c,av);}
a=C_alloc(6);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
if(C_truep(C_i_not(C_i_finitep(t2)))){
/* library.scm:1792: ##sys#error-bad-inexact */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[375]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[375]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[520];
tp(4,av2);}}
else{
if(C_truep(C_u_i_fpintegerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12925,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12929,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1794: scheme#inexact->exact */
t5=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_ratnum_num(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:1796: ##sys#signal-hook */
t3=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[520];
av2[4]=lf[522];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}}}

/* k12923 in scheme#numerator in k10477 in k8569 */
static void C_ccall f_12925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_12925,c,av);}
a=C_alloc(11);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_exact_to_inexact(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12927 in scheme#numerator in k10477 in k8569 */
static void C_ccall f_12929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12929,c,av);}
/* library.scm:1794: scheme#numerator */
t2=*((C_word*)lf[519]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* scheme#denominator in k10477 in k8569 */
static void C_ccall f_12947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_12947,c,av);}
a=C_alloc(6);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fix(1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
if(C_truep(C_i_not(C_i_finitep(t2)))){
/* library.scm:1804: ##sys#error-bad-inexact */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[375]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[375]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[524];
tp(4,av2);}}
else{
if(C_truep(C_u_i_fpintegerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[525];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12971,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12975,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1806: scheme#inexact->exact */
t5=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_ratnum_denom(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:1808: ##sys#signal-hook */
t3=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[520];
av2[4]=lf[526];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}}}

/* k12969 in scheme#denominator in k10477 in k8569 */
static void C_ccall f_12971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,1)))){
C_save_and_reclaim((void *)f_12971,c,av);}
a=C_alloc(11);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_exact_to_inexact(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12973 in scheme#denominator in k10477 in k8569 */
static void C_ccall f_12975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12975,c,av);}
/* library.scm:1806: scheme#denominator */
t2=*((C_word*)lf[523]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* ##sys#extended-signum in k10477 in k8569 */
static void C_ccall f_12993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_12993,c,av);}
a=C_alloc(26);
if(C_truep(C_i_ratnump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_u_i_integer_signum(C_u_i_ratnum_num(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_cplxnump(t2))){
t3=C_u_i_cplxnum_imag(t2);
t4=C_a_i_exact_to_inexact(&a,1,t3);
t5=C_u_i_cplxnum_real(t2);
t6=C_a_i_exact_to_inexact(&a,1,t5);
t7=C_flonum(&a,C_atan2(C_flonum_magnitude(t4),C_flonum_magnitude(t6)));
/* library.scm:1815: scheme#make-polar */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[507]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[507]+1);
av2[1]=t1;
av2[2]=C_fix(1);
av2[3]=t7;
tp(4,av2);}}
else{
/* library.scm:1816: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[528];
tp(4,av2);}}}}

/* doloop2220 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_fcall f_13034(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_13034,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_u_i_fpintegerp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13045,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1824: chicken.flonum#fp* */
t5=*((C_word*)lf[439]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[532];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k13043 in doloop2220 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13045,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_13034(t2,((C_word*)t0)[3],t1,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_fcall f_13051(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_13051,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13055,a[2]=t2,a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13086,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13034,a[2]=t7,a[3]=((C_word)li375),tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_13034(t9,t5,t2,C_fix(0));}

/* k13053 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13055(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_13055,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13082,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:1830: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13067 in k13080 in k13053 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13069,c,av);}
/* library.scm:1832: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k13080 in k13053 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,3)))){
C_save_and_reclaim((void *)f_13082,c,av);}
a=C_alloc(42);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
if(C_truep(C_i_finitep(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13069,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=C_s_a_u_i_flo_to_int(&a,1,t2);
/* library.scm:1832: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}
else{
/* library.scm:1833: ##sys#error-bad-inexact */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[375]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[375]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[530];
tp(4,av2);}}}

/* k13084 in deliver in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13086,c,av);}
/* library.scm:1829: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[531]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(2);
av2[3]=t1;
tp(4,av2);}}

/* k13101 in k13105 in k13111 in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13103,c,av);}
/* library.scm:1837: deliver */
t2=((C_word*)t0)[2];
f_13051(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k13105 in k13111 in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13107(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(38,c,3)))){
C_save_and_reclaim((void *)f_13107,c,av);}
a=C_alloc(38);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13103,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1840: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t3;
av2[2]=C_fix(2);
av2[3]=*((C_word*)lf[429]+1);
tp(4,av2);}}

/* k13111 in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13113,c,av);}
a=C_alloc(5);
t2=(C_truep(t1)?C_ub_i_flonum_greaterp(C_flonum_magnitude(((C_word*)t0)[2]),(double)-1.0):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13107,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:1837: scheme#expt */
t4=*((C_word*)lf[533]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[532];
av2[3]=*((C_word*)lf[429]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
/* library.scm:1841: deliver */
t3=((C_word*)t0)[3];
f_13051(t3,((C_word*)t0)[4],((C_word*)t0)[2],C_fix(1));}}

/* scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13117(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_13117,c,av);}
a=C_alloc(9);
if(C_truep(C_i_exactp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
if(C_truep(C_u_i_fpintegerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_u_i_flo_to_int(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_u_i_flonum_finitep(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13051,a[2]=t2,a[3]=((C_word)li376),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13113,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1835: chicken.flonum#fp< */
t5=*((C_word*)lf[451]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[525];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
/* library.scm:1849: ##sys#error-bad-inexact */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[375]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[375]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[530];
tp(4,av2);}}}}
else{
if(C_truep(C_i_cplxnump(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13157,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1851: scheme#inexact->exact */
t4=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_cplxnum_real(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
/* library.scm:1853: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[530];
tp(4,av2);}}}}}

/* k13155 in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13157,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13161,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1852: scheme#inexact->exact */
t3=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_cplxnum_imag(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13159 in k13155 in scheme#inexact->exact in k10477 in k8569 */
static void C_ccall f_13161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13161,c,av);}
/* library.scm:1851: make-complex */
f_12675(((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* chicken.bitwise#bitwise-not in k10477 in k8569 */
static void C_ccall f_13183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_13183,c,av);}
a=C_alloc(5);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_bitwise_not(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.bitwise#bit->boolean in k10477 in k8569 */
static void C_ccall f_13186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13186,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_bit_to_bool(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.bitwise#integer-length in k10477 in k8569 */
static void C_ccall f_13189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13189,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_integer_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.bitwise#arithmetic-shift in k10477 in k8569 */
static void C_ccall f_13192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_13192,c,av);}
a=C_alloc(5);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_s_a_i_arithmetic_shift(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#/ in k10477 in k8569 */
static void C_ccall f_13195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +4,c,3)))){
C_save_and_reclaim((void*)f_13195,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+4);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
if(C_truep(C_rest_nullp(c,3))){
/* library.scm:1888: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t1;
av2[2]=C_fix(1);
av2[3]=t2;
tp(4,av2);}}
else{
t4=C_slot(t3,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13216,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1890: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=C_slot(t3,C_fix(0));
tp(4,av2);}}}

/* k13214 in scheme#/ in k10477 in k8569 */
static void C_ccall f_13216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_13216,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13218,a[2]=t3,a[3]=((C_word)li385),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_13218(t5,((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* loop in k13214 in scheme#/ in k10477 in k8569 */
static void C_fcall f_13218(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_13218,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13236,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* library.scm:1894: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t3;
av2[3]=C_slot(t2,C_fix(0));
tp(4,av2);}}}

/* k13234 in loop in k13214 in scheme#/ in k10477 in k8569 */
static void C_ccall f_13236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13236,c,av);}
/* library.scm:1893: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13218(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_13246,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13250,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t5=C_eqp(t3,C_fix(0));
if(C_truep(t5)){
t6=C_fix((C_word)C_DIVISION_BY_ZERO_ERROR);
/* library.scm:1901: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=t4;
av2[2]=t6;
av2[3]=lf[546];
av2[4]=t2;
av2[5]=t3;
tp(6,av2);}}
else{
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_13250(2,av2);}}}

/* k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(117,c,4)))){
C_save_and_reclaim((void *)f_13250,c,av);}
a=C_alloc(117);
t2=(C_truep(C_i_exact_integerp(((C_word*)t0)[2]))?C_i_exact_integerp(((C_word*)t0)[3]):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=C_s_a_u_i_integer_gcd(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=C_s_a_u_i_integer_quotient(&a,2,((C_word*)t0)[2],t3);
t5=C_s_a_u_i_integer_quotient(&a,2,((C_word*)t0)[3],t3);
/* library.scm:1904: ratnum */
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(
/* library.scm:1904: ratnum */
  f_12855(C_a_i(&a,21),t4,t5)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t3=C_i_cplxnump(((C_word*)t0)[2]);
t4=(C_truep(t3)?t3:C_i_cplxnump(((C_word*)t0)[3]));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13288,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:1907: scheme#real-part */
t6=*((C_word*)lf[509]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_i_flonump(((C_word*)t0)[2]);
t6=(C_truep(t5)?t5:C_i_flonump(((C_word*)t0)[3]));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13354,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:1915: scheme#exact->inexact */
t8=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
if(C_truep(C_i_ratnump(((C_word*)t0)[2]))){
if(C_truep(C_i_ratnump(((C_word*)t0)[3]))){
t7=C_u_i_ratnum_num(((C_word*)t0)[2]);
t8=C_u_i_ratnum_denom(((C_word*)t0)[2]);
t9=C_u_i_ratnum_num(((C_word*)t0)[3]);
t10=C_u_i_ratnum_denom(((C_word*)t0)[3]);
t11=C_s_a_u_i_integer_gcd(&a,2,t7,t9);
t12=C_s_a_u_i_integer_gcd(&a,2,t8,t10);
t13=C_s_a_i_quotient(&a,2,t7,t11);
t14=C_s_a_i_quotient(&a,2,t10,t12);
t15=C_s_a_i_times(&a,2,t13,t14);
t16=C_s_a_i_quotient(&a,2,t8,t12);
t17=C_s_a_i_quotient(&a,2,t9,t11);
t18=C_s_a_i_times(&a,2,t16,t17);
/* library.scm:1925: ratnum */
t19=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t19;
av2[1]=(
/* library.scm:1925: ratnum */
  f_12855(C_a_i(&a,21),t15,t18)
);
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t7=C_u_i_ratnum_num(((C_word*)t0)[2]);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13435,a[2]=t7,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:1931: ##sys#internal-gcd */
f_15181(t8,lf[546],t7,((C_word*)t0)[3]);}}
else{
if(C_truep(C_i_ratnump(((C_word*)t0)[3]))){
t7=C_u_i_ratnum_num(((C_word*)t0)[3]);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13474,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t7,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:1942: ##sys#internal-gcd */
f_15181(t8,lf[546],((C_word*)t0)[2],t7);}
else{
t7=C_i_numberp(((C_word*)t0)[2]);
if(C_truep(C_i_not(t7))){
/* library.scm:1948: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[2];
av2[3]=lf[546];
tp(4,av2);}}
else{
/* library.scm:1949: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[374]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[546];
tp(4,av2);}}}}}}}}

/* k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13288(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_13288,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13291,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:1907: scheme#imag-part */
t3=*((C_word*)lf[511]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13289 in k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13291(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13291,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13294,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:1908: scheme#real-part */
t3=*((C_word*)lf[509]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13292 in k13289 in k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13294,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13297,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:1908: scheme#imag-part */
t3=*((C_word*)lf[511]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13295 in k13292 in k13289 in k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(198,c,3)))){
C_save_and_reclaim((void *)f_13297,c,av);}
a=C_alloc(198);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],((C_word*)t0)[2]);
t3=C_s_a_i_times(&a,2,t1,t1);
t4=C_s_a_i_plus(&a,2,t2,t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13303,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
t6=C_s_a_i_times(&a,2,((C_word*)t0)[5],((C_word*)t0)[2]);
t7=C_s_a_i_times(&a,2,((C_word*)t0)[4],t1);
t8=C_s_a_i_plus(&a,2,t6,t7);
/* library.scm:1910: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t8;
av2[3]=t4;
tp(4,av2);}}

/* k13301 in k13295 in k13292 in k13289 in k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(99,c,3)))){
C_save_and_reclaim((void *)f_13303,c,av);}
a=C_alloc(99);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13306,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_s_a_i_times(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_s_a_i_times(&a,2,((C_word*)t0)[5],((C_word*)t0)[6]);
t5=C_s_a_i_minus(&a,2,t3,t4);
/* library.scm:1911: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t2;
av2[2]=t5;
av2[3]=((C_word*)t0)[7];
tp(4,av2);}}

/* k13304 in k13301 in k13295 in k13292 in k13289 in k13286 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13306,c,av);}
/* library.scm:1912: make-complex */
f_12675(((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* k13352 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13354,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13358,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1915: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13356 in k13352 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13358(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13358,c,av);}
/* library.scm:1915: chicken.flonum#fp/ */
t2=*((C_word*)lf[441]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13433 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13435(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(64,c,3)))){
C_save_and_reclaim((void *)f_13435,c,av);}
a=C_alloc(64);
t2=C_s_a_i_quotient(&a,2,((C_word*)t0)[2],t1);
t3=C_u_i_ratnum_denom(((C_word*)t0)[3]);
t4=C_s_a_i_quotient(&a,2,((C_word*)t0)[4],t1);
t5=C_s_a_i_times(&a,2,t3,t4);
if(C_truep(C_i_flonump(t5))){
/* library.scm:1935: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t5;
tp(4,av2);}}
else{
/* library.scm:1936: ratnum */
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(
/* library.scm:1936: ratnum */
  f_12855(C_a_i(&a,21),t2,t5)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k13472 in k13248 in ##sys#/-2 in k10477 in k8569 */
static void C_ccall f_13474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(64,c,3)))){
C_save_and_reclaim((void *)f_13474,c,av);}
a=C_alloc(64);
t2=C_s_a_i_quotient(&a,2,((C_word*)t0)[2],t1);
t3=C_u_i_ratnum_denom(((C_word*)t0)[3]);
t4=C_s_a_i_times(&a,2,t2,t3);
t5=C_s_a_i_quotient(&a,2,((C_word*)t0)[4],t1);
if(C_truep(C_i_flonump(t5))){
/* library.scm:1946: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=t4;
av2[3]=t5;
tp(4,av2);}}
else{
/* library.scm:1947: ratnum */
t6=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(
/* library.scm:1947: ratnum */
  f_12855(C_a_i(&a,21),t4,t5)
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* scheme#floor in k10477 in k8569 */
static void C_ccall f_13529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(34,c,3)))){
C_save_and_reclaim((void *)f_13529,c,av);}
a=C_alloc(34);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
/* library.scm:1954: chicken.flonum#fpfloor */
t3=*((C_word*)lf[463]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=C_u_i_ratnum_num(t2);
t4=C_u_i_ratnum_denom(t2);
t5=C_s_a_i_quotient(&a,2,t3,t4);
t6=C_i_greater_or_equalp(t3,C_fix(0));
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?t5:C_s_a_i_minus(&a,2,t5,C_fix(1)));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* library.scm:1959: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[548];
tp(4,av2);}}}}}

/* scheme#ceiling in k10477 in k8569 */
static void C_ccall f_13574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(34,c,3)))){
C_save_and_reclaim((void *)f_13574,c,av);}
a=C_alloc(34);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
/* library.scm:1964: chicken.flonum#fpceiling */
t3=*((C_word*)lf[469]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=C_u_i_ratnum_num(t2);
t4=C_u_i_ratnum_denom(t2);
t5=C_s_a_i_quotient(&a,2,t3,t4);
t6=C_i_greater_or_equalp(t3,C_fix(0));
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_s_a_i_plus(&a,2,t5,C_fix(1)):t5);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* library.scm:1969: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[550];
tp(4,av2);}}}}}

/* scheme#truncate in k10477 in k8569 */
static void C_ccall f_13619(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13619,c,av);}
a=C_alloc(5);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
/* library.scm:1974: chicken.flonum#fptruncate */
t3=*((C_word*)lf[465]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=C_u_i_ratnum_num(t2);
t4=C_u_i_ratnum_denom(t2);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_s_a_i_quotient(&a,2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* library.scm:1978: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[552];
tp(4,av2);}}}}}

/* scheme#round in k10477 in k8569 */
static void C_ccall f_13656(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(36,c,3)))){
C_save_and_reclaim((void *)f_13656,c,av);}
a=C_alloc(36);
if(C_truep(C_i_exact_integerp(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_flonump(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_flonum_round_proper(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_ratnump(t2))){
t3=C_a_i_ratnum(&a,2,C_fix(1),C_fix(2));
t4=C_s_a_i_plus(&a,2,t2,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13676,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:1987: scheme#floor */
t6=*((C_word*)lf[547]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
/* library.scm:1989: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[554];
tp(4,av2);}}}}}

/* k13674 in scheme#round in k10477 in k8569 */
static void C_ccall f_13676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_13676,c,av);}
a=C_alloc(29);
t2=C_i_nequalp(t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?(C_truep(C_i_oddp(t1))?C_s_a_i_minus(&a,2,t1,C_fix(1)):t1):t1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* find-ratio-between in k10477 in k8569 */
static void C_fcall f_13700(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(67,0,4)))){
C_save_and_reclaim_args((void *)trf_13700,3,t1,t2,t3);}
a=C_alloc(67);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13703,a[2]=t5,a[3]=((C_word)li392),tmp=(C_word)a,a+=4,tmp));
if(C_truep(C_i_lessp(t3,t2))){
/* library.scm:2002: find-ratio-between */
t12=t1;
t13=t3;
t14=t2;
t1=t12;
t2=t13;
t3=t14;
goto loop;}
else{
t7=C_i_lessp(t2,t3);
if(C_truep(C_i_not(t7))){
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_a_i_list2(&a,2,t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
if(C_truep(C_i_positivep(t2))){
/* library.scm:2004: sr */
t8=((C_word*)t5)[1];
f_13703(t8,t1,t2,t3);}
else{
if(C_truep(C_i_negativep(t3))){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13814,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t9=C_s_a_i_negate(&a,1,t3);
t10=C_s_a_i_negate(&a,1,t2);
/* library.scm:2005: sr */
t11=((C_word*)t5)[1];
f_13703(t11,t8,t9,t10);}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=lf[556];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}}}

/* sr in find-ratio-between in k10477 in k8569 */
static void C_fcall f_13703(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_13703,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13707,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13782,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1993: scheme#floor */
t6=*((C_word*)lf[547]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13707,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13710,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13778,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:1994: scheme#floor */
t4=*((C_word*)lf[547]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k13708 in k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,3)))){
C_save_and_reclaim((void *)f_13710,c,av);}
a=C_alloc(39);
t2=C_i_lessp(((C_word*)t0)[2],((C_word*)t0)[3]);
if(C_truep(C_i_not(t2))){
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[2],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_nequalp(((C_word*)t0)[2],t1))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13728,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13751,a[2]=((C_word*)t0)[5],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t5=C_s_a_i_minus(&a,2,((C_word*)t0)[6],t1);
/* library.scm:1997: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t4;
av2[2]=C_fix(1);
av2[3]=t5;
tp(4,av2);}}
else{
t3=C_s_a_i_plus(&a,2,C_fix(1),((C_word*)t0)[2]);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list2(&a,2,t3,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k13726 in k13708 in k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(68,c,1)))){
C_save_and_reclaim((void *)f_13728,c,av);}
a=C_alloc(68);
t2=C_i_cadr(t1);
t3=C_u_i_car(t1);
t4=C_s_a_i_times(&a,2,((C_word*)t0)[2],t3);
t5=C_s_a_i_plus(&a,2,t2,t4);
t6=C_u_i_car(t1);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_list2(&a,2,t5,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k13749 in k13708 in k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13751(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,3)))){
C_save_and_reclaim((void *)f_13751,c,av);}
a=C_alloc(34);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13755,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* library.scm:1998: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t2;
av2[2]=C_fix(1);
av2[3]=t3;
tp(4,av2);}}

/* k13753 in k13749 in k13708 in k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13755,c,av);}
/* library.scm:1997: sr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13703(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k13776 in k13705 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13778,c,av);}
/* library.scm:1994: scheme#inexact->exact */
t2=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k13780 in sr in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13782,c,av);}
/* library.scm:1993: scheme#inexact->exact */
t2=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k13812 in find-ratio-between in k10477 in k8569 */
static void C_ccall f_13814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,1)))){
C_save_and_reclaim((void *)f_13814,c,av);}
a=C_alloc(35);
t2=C_i_car(t1);
t3=C_s_a_i_negate(&a,1,t2);
t4=C_i_cadr(t1);
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_a_i_list2(&a,2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#rationalize in k10477 in k8569 */
static void C_ccall f_13850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(66,c,3)))){
C_save_and_reclaim((void *)f_13850,c,av);}
a=C_alloc(66);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13854,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13873,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_s_a_i_minus(&a,2,t2,t3);
t7=C_s_a_i_plus(&a,2,t2,t3);
/* library.scm:2009: find-ratio-between */
f_13700(t5,t6,t7);}

/* k13852 in scheme#rationalize in k10477 in k8569 */
static void C_ccall f_13854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13854,c,av);}
t2=C_i_inexactp(((C_word*)t0)[2]);
if(C_truep(t2)){
if(C_truep(t2)){
/* library.scm:2015: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
if(C_truep(C_i_inexactp(((C_word*)t0)[4]))){
/* library.scm:2015: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k13871 in scheme#rationalize in k10477 in k8569 */
static void C_ccall f_13873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13873,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[529]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* scheme#max in k10477 in k8569 */
static void C_ccall f_13875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +6,c,5)))){
C_save_and_reclaim((void*)f_13875,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+6);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
t4=C_i_flonump(t2);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13882,a[2]=t6,a[3]=((C_word)li395),tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_13882(t8,t1,t4,t2,t3);}

/* loop in scheme#max in k10477 in k8569 */
static void C_fcall f_13882(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_13882,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=C_i_check_number_2(t3,lf[559]);
if(C_truep(C_i_nullp(t4))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t2)?C_a_i_exact_to_inexact(&a,1,t3):t3);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_slot(t4,C_fix(0));
t7=(C_truep(t2)?t2:C_i_flonump(t6));
t8=C_i_greaterp(t6,t3);
t9=(C_truep(t8)?t6:t3);
/* library.scm:2025: loop */
t11=t1;
t12=t7;
t13=t9;
t14=C_slot(t4,C_fix(1));
t1=t11;
t2=t12;
t3=t13;
t4=t14;
goto loop;}}

/* scheme#min in k10477 in k8569 */
static void C_ccall f_13919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +6,c,5)))){
C_save_and_reclaim((void*)f_13919,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+6);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
t4=C_i_flonump(t2);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13926,a[2]=t6,a[3]=((C_word)li397),tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_13926(t8,t1,t4,t2,t3);}

/* loop in scheme#min in k10477 in k8569 */
static void C_fcall f_13926(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_13926,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=C_i_check_number_2(t3,lf[561]);
if(C_truep(C_i_nullp(t4))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t2)?C_a_i_exact_to_inexact(&a,1,t3):t3);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_slot(t4,C_fix(0));
t7=(C_truep(t2)?t2:C_i_flonump(t6));
t8=C_i_lessp(t6,t3);
t9=(C_truep(t8)?t6:t3);
/* library.scm:2036: loop */
t11=t1;
t12=t7;
t13=t9;
t14=C_slot(t4,C_fix(1));
t1=t11;
t2=t12;
t3=t13;
t4=t14;
goto loop;}}

/* scheme#exp in k10477 in k8569 */
static void C_ccall f_13963(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_13963,c,av);}
a=C_alloc(15);
t3=C_i_check_number_2(t2,lf[563]);
if(C_truep(C_i_cplxnump(t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13981,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2045: scheme#exact->inexact */
t5=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_cplxnum_real(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_a_i_exact_to_inexact(&a,1,t2);
t5=C_flonum(&a,C_exp(C_flonum_magnitude(t4)));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k13979 in scheme#exp in k10477 in k8569 */
static void C_ccall f_13981(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_13981,c,av);}
a=C_alloc(12);
t2=C_a_i_exp(&a,1,t1);
t3=C_u_i_cplxnum_imag(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13990,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13995,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2048: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k13988 in k13979 in scheme#exp in k10477 in k8569 */
static void C_ccall f_13990(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,1)))){
C_save_and_reclaim((void *)f_13990,c,av);}
a=C_alloc(33);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_times(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13993 in k13979 in scheme#exp in k10477 in k8569 */
static void C_ccall f_13995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_13995,c,av);}
a=C_alloc(19);
t2=C_a_i_cos(&a,1,t1);
t3=C_a_i_exact_to_inexact(&a,1,((C_word*)t0)[2]);
t4=C_a_i_sin(&a,1,t3);
/* library.scm:2047: make-complex */
f_12675(((C_word*)t0)[3],t2,t4);}

/* ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_14008,c,av);}
a=C_alloc(7);
t3=C_eqp(t2,C_fix(0));
if(C_truep(t3)){
/* library.scm:2055: ##sys#signal-hook */
t4=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[565];
av2[3]=lf[566];
av2[4]=lf[567];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t4=C_i_cplxnump(t2);
t5=(C_truep(t4)?t4:C_i_negativep(t2));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14034,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14050,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2058: scheme#magnitude */
t8=*((C_word*)lf[515]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14054,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2061: scheme#exact->inexact */
t7=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}

/* k14032 in ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14034,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14042,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2059: make-complex */
f_12675(t2,C_fix(0),C_fix(1));}

/* k14040 in k14032 in ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14042,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14046,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2059: scheme#angle */
t3=*((C_word*)lf[513]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14044 in k14040 in k14032 in ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14046(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(62,c,1)))){
C_save_and_reclaim((void *)f_14046,c,av);}
a=C_alloc(62);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_plus(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14048 in ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14050(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14050,c,av);}
/* library.scm:2058: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[564]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k14052 in ##sys#log-1 in k10477 in k8569 */
static void C_ccall f_14054(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_14054,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_log(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#log in k10477 in k8569 */
static void C_ccall f_14057(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14057,c,av);}
a=C_alloc(4);
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?C_SCHEME_FALSE:C_get_rest_arg(c,3,av,3,t0));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14071,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2065: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}
else{
/* library.scm:2065: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}

/* k14069 in scheme#log in k10477 in k8569 */
static void C_ccall f_14071(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14071,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14075,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2065: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k14073 in k14069 in scheme#log in k10477 in k8569 */
static void C_ccall f_14075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14075,c,av);}
/* library.scm:2065: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* scheme#sin in k10477 in k8569 */
static void C_ccall f_14086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(37,c,2)))){
C_save_and_reclaim((void *)f_14086,c,av);}
a=C_alloc(37);
t3=C_i_check_number_2(t2,lf[570]);
if(C_truep(C_i_cplxnump(t2))){
t4=C_s_a_i_times(&a,2,lf[571],t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14110,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2072: scheme#exp */
t6=*((C_word*)lf[562]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=C_a_i_exact_to_inexact(&a,1,t2);
t5=C_a_i_sin(&a,1,t4);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k14108 in scheme#sin in k10477 in k8569 */
static void C_ccall f_14110(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_14110,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14114,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_s_a_i_negate(&a,1,((C_word*)t0)[3]);
/* library.scm:2072: scheme#exp */
t4=*((C_word*)lf[562]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k14112 in k14108 in scheme#sin in k10477 in k8569 */
static void C_ccall f_14114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_14114,c,av);}
a=C_alloc(29);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2072: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=lf[572];
tp(4,av2);}}

/* scheme#cos in k10477 in k8569 */
static void C_ccall f_14120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(37,c,2)))){
C_save_and_reclaim((void *)f_14120,c,av);}
a=C_alloc(37);
t3=C_i_check_number_2(t2,lf[574]);
if(C_truep(C_i_cplxnump(t2))){
t4=C_s_a_i_times(&a,2,lf[571],t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14144,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2080: scheme#exp */
t6=*((C_word*)lf[562]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=C_a_i_exact_to_inexact(&a,1,t2);
t5=C_a_i_cos(&a,1,t4);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k14142 in scheme#cos in k10477 in k8569 */
static void C_ccall f_14144(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_14144,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14148,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_s_a_i_negate(&a,1,((C_word*)t0)[3]);
/* library.scm:2080: scheme#exp */
t4=*((C_word*)lf[562]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k14146 in k14142 in scheme#cos in k10477 in k8569 */
static void C_ccall f_14148(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_14148,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2080: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=C_fix(2);
tp(4,av2);}}

/* scheme#tan in k10477 in k8569 */
static void C_ccall f_14154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_14154,c,av);}
a=C_alloc(15);
t3=C_i_check_number_2(t2,lf[576]);
if(C_truep(C_i_cplxnump(t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14171,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2087: scheme#sin */
t5=*((C_word*)lf[569]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_a_i_exact_to_inexact(&a,1,t2);
t5=C_a_i_tan(&a,1,t4);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k14169 in scheme#tan in k10477 in k8569 */
static void C_ccall f_14171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14171,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14175,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2087: scheme#cos */
t3=*((C_word*)lf[573]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14173 in k14169 in scheme#tan in k10477 in k8569 */
static void C_ccall f_14175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14175,c,av);}
/* library.scm:2087: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* scheme#asin in k10477 in k8569 */
static void C_ccall f_14179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_14179,c,av);}
a=C_alloc(8);
t3=C_i_check_number_2(t2,lf[578]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14189,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_flonump(t2))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14237,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2094: chicken.flonum#fp>= */
t6=*((C_word*)lf[453]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=lf[581];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=t4;
f_14189(t5,C_SCHEME_FALSE);}}

/* k14187 in scheme#asin in k10477 in k8569 */
static void C_fcall f_14189(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_14189,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_a_i_asin(&a,1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14195,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fixnump(((C_word*)t0)[2]))){
t3=C_fixnum_greater_or_equal_p(((C_word*)t0)[2],C_fix(-1));
t4=t2;
f_14195(t4,(C_truep(t3)?C_fixnum_less_or_equal_p(((C_word*)t0)[2],C_fix(1)):C_SCHEME_FALSE));}
else{
t3=t2;
f_14195(t3,C_SCHEME_FALSE);}}}

/* k14193 in k14187 in scheme#asin in k10477 in k8569 */
static void C_fcall f_14195(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(102,0,3)))){
C_save_and_reclaim_args((void *)trf_14195,2,t0,t1);}
a=C_alloc(102);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_a_i_asin(&a,1,C_a_i_fix_to_flo(&a,1,((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14202,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_s_a_i_times(&a,2,lf[571],((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14214,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=C_s_a_i_times(&a,2,((C_word*)t0)[3],((C_word*)t0)[3]);
t6=C_s_a_i_minus(&a,2,C_fix(1),t5);
/* library.scm:2103: ##sys#sqrt/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[580]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[580]+1);
av2[1]=t4;
av2[2]=lf[578];
av2[3]=t6;
tp(4,av2);}}}

/* k14200 in k14193 in k14187 in scheme#asin in k10477 in k8569 */
static void C_ccall f_14202(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,1)))){
C_save_and_reclaim((void *)f_14202,c,av);}
a=C_alloc(33);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_times(&a,2,lf[579],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k14212 in k14193 in k14187 in scheme#asin in k10477 in k8569 */
static void C_ccall f_14214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_14214,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2101: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[564]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
tp(3,av2);}}

/* k14235 in scheme#asin in k10477 in k8569 */
static void C_ccall f_14237(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14237,c,av);}
t2=((C_word*)t0)[2];
f_14189(t2,(C_truep(t1)?C_ub_i_flonum_less_or_equal_p(C_flonum_magnitude(((C_word*)t0)[3]),(double)1.0):C_SCHEME_FALSE));}

/* scheme#acos in k10477 in k8569 */
static void C_ccall f_14241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_14241,c,av);}
a=C_alloc(9);
t3=C_i_check_number_2(t2,lf[583]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14251,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_flonump(t2))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14279,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2111: chicken.flonum#fp>= */
t6=*((C_word*)lf[453]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=lf[581];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=t4;
f_14251(t5,C_SCHEME_FALSE);}}

/* k14249 in scheme#acos in k10477 in k8569 */
static void C_fcall f_14251(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14251,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=C_a_i_acos(&a,1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14257,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnump(((C_word*)t0)[2]))){
t3=C_fixnum_greater_or_equal_p(((C_word*)t0)[2],C_fix(-1));
t4=t2;
f_14257(t4,(C_truep(t3)?C_fixnum_less_or_equal_p(((C_word*)t0)[2],C_fix(1)):C_SCHEME_FALSE));}
else{
t3=t2;
f_14257(t3,C_SCHEME_FALSE);}}}

/* k14255 in k14249 in scheme#acos in k10477 in k8569 */
static void C_fcall f_14257(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_14257,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_a_i_acos(&a,1,C_a_i_fix_to_flo(&a,1,((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14264,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2118: scheme#asin */
t3=*((C_word*)lf[577]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k14262 in k14255 in k14249 in scheme#acos in k10477 in k8569 */
static void C_ccall f_14264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_14264,c,av);}
a=C_alloc(29);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_minus(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k14277 in scheme#acos in k10477 in k8569 */
static void C_ccall f_14279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14279,c,av);}
t2=((C_word*)t0)[2];
f_14251(t2,(C_truep(t1)?C_ub_i_flonum_less_or_equal_p(C_flonum_magnitude(((C_word*)t0)[3]),(double)1.0):C_SCHEME_FALSE));}

/* scheme#atan in k10477 in k8569 */
static void C_ccall f_14283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(66,c,3)))){
C_save_and_reclaim((void *)f_14283,c,av);}
a=C_alloc(66);
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?C_SCHEME_FALSE:C_get_rest_arg(c,3,av,3,t0));
t5=C_i_check_number_2(t2,lf[585]);
if(C_truep(C_i_cplxnump(t2))){
if(C_truep(t4)){
/* library.scm:2125: ##sys#error-bad-real */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[365]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[365]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[585];
tp(4,av2);}}
else{
t6=C_s_a_i_times(&a,2,lf[571],t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14316,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
t8=C_s_a_i_plus(&a,2,C_fix(1),t6);
/* library.scm:2127: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t7;
av2[2]=t8;
tp(3,av2);}}}
else{
if(C_truep(t4)){
t6=C_a_i_exact_to_inexact(&a,1,t2);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14337,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2131: scheme#exact->inexact */
t8=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t6=C_a_i_exact_to_inexact(&a,1,t2);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_a_i_atan(&a,1,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}

/* k14314 in scheme#atan in k10477 in k8569 */
static void C_ccall f_14316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_14316,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14320,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_s_a_i_minus(&a,2,C_fix(1),((C_word*)t0)[3]);
/* library.scm:2128: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t2;
av2[2]=t3;
tp(3,av2);}}

/* k14318 in k14314 in scheme#atan in k10477 in k8569 */
static void C_ccall f_14320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_14320,c,av);}
a=C_alloc(29);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2127: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=lf[572];
tp(4,av2);}}

/* k14335 in scheme#atan in k10477 in k8569 */
static void C_ccall f_14337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_14337,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_atan2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14347(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_14347,c,av);}
a=C_alloc(10);
t3=C_fixnump(t2);
t4=(C_truep(t3)?C_fixnum_less_or_equal_p(t2,C_fix(4)):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_eqp(t2,C_fix(0));
t6=(C_truep(t5)?t5:C_eqp(t2,C_fix(1)));
if(C_truep(t6)){
/* library.scm:2143: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fix(0);
C_values(4,av2);}}
else{
switch(t2){
case C_fix(2):
/* library.scm:2144: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fix(1);
av2[3]=C_fix(1);
C_values(4,av2);}
case C_fix(3):
/* library.scm:2145: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fix(1);
av2[3]=C_fix(2);
C_values(4,av2);}
case C_fix(4):
/* library.scm:2146: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fix(2);
av2[3]=C_fix(0);
C_values(4,av2);}
default:
/* library.scm:2147: chicken.base#error */
t7=*((C_word*)lf[193]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[587];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}
else{
t5=C_fixnum_plus(C_i_integer_length(t2),C_fix(1));
t6=C_fixnum_shift_right(t5,C_fix(2));
t7=C_fixnum_shift_left(t6,C_fix(1));
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14420,a[2]=t7,a[3]=t2,a[4]=((C_word)li408),tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14434,a[2]=t6,a[3]=t2,a[4]=((C_word)li412),tmp=(C_word)a,a+=5,tmp);
/* library.scm:2148: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
C_call_with_values(4,av2);}}}

/* a14419 in ##sys#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14420,c,av);}
a=C_alloc(5);
t2=C_fixnum_negate(((C_word*)t0)[2]);
t3=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[3],t2);
/* library.scm:2151: ##sys#exact-integer-sqrt */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[586]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[586]+1);
av2[1]=t1;
av2[2]=t3;
tp(3,av2);}}

/* a14433 in ##sys#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(38,c,4)))){
C_save_and_reclaim((void *)f_14434,c,av);}
a=C_alloc(38);
t4=C_s_a_i_arithmetic_shift(&a,2,C_fix(1),((C_word*)t0)[2]);
t5=C_s_a_u_i_integer_minus(&a,2,t4,C_fix(1));
t6=C_s_a_i_bitwise_and(&a,2,((C_word*)t0)[3],t5);
t7=C_fixnum_negate(((C_word*)t0)[2]);
t8=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[3],t7);
t9=C_s_a_i_bitwise_and(&a,2,t8,t5);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14451,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t9,a[5]=t2,a[6]=((C_word)li410),tmp=(C_word)a,a+=7,tmp);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14467,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t6,a[5]=((C_word)li411),tmp=(C_word)a,a+=6,tmp);
/* library.scm:2148: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t10;
av2[3]=t11;
C_call_with_values(4,av2);}}

/* a14450 in a14433 in ##sys#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_14451,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_u_integer_quotient_and_remainder,a[2]=((C_word)li409),tmp=(C_word)a,a+=3,tmp);
t3=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=C_s_a_u_i_integer_plus(&a,2,t3,((C_word*)t0)[4]);
t5=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[5],C_fix(1));
/* library.scm:2148: g2592 */
t6=t2;{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
((C_proc)C_fast_retrieve_proc(t6))(4,av2);}}

/* a14466 in a14433 in ##sys#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(222,c,3)))){
C_save_and_reclaim((void *)f_14467,c,av);}
a=C_alloc(222);
t4=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t5=C_s_a_i_plus(&a,2,t4,t2);
t6=C_s_a_i_arithmetic_shift(&a,2,t3,((C_word*)t0)[3]);
t7=C_s_a_i_times(&a,2,t2,t2);
t8=C_s_a_i_minus(&a,2,((C_word*)t0)[4],t7);
t9=C_s_a_i_plus(&a,2,t6,t8);
if(C_truep(C_i_negativep(t9))){
t10=C_s_a_i_minus(&a,2,t5,C_fix(1));
t11=C_s_a_i_arithmetic_shift(&a,2,t5,C_fix(1));
t12=C_s_a_i_plus(&a,2,t9,t11);
t13=C_s_a_i_minus(&a,2,t12,C_fix(1));
/* library.scm:2162: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t10;
av2[3]=t13;
C_values(4,av2);}}
else{
/* library.scm:2164: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t5;
av2[3]=t9;
C_values(4,av2);}}}

/* chicken.base#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14538,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14542,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2168: ##sys#check-exact-uinteger */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[362]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[362]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[589];
tp(4,av2);}}

/* k14540 in chicken.base#exact-integer-sqrt in k10477 in k8569 */
static void C_ccall f_14542(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14542,c,av);}
/* library.scm:2169: ##sys#exact-integer-sqrt */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[586]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[586]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(32,c,4)))){
C_save_and_reclaim((void *)f_14547,c,av);}
a=C_alloc(32);
if(C_truep(C_i_cplxnump(t3))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14557,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_u_i_cplxnum_imag(t3);
t6=C_a_i_exact_to_inexact(&a,1,t5);
t7=C_u_i_cplxnum_real(t3);
t8=C_a_i_exact_to_inexact(&a,1,t7);
t9=C_flonum(&a,C_atan2(C_flonum_magnitude(t6),C_flonum_magnitude(t8)));
/* library.scm:2175: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t4;
av2[2]=t9;
av2[3]=C_fix(2);
tp(4,av2);}}
else{
if(C_truep(C_i_negativep(t3))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14597,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=C_s_a_i_negate(&a,1,t3);
/* library.scm:2180: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
if(C_truep(C_i_exact_integerp(t3))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14608,a[2]=t3,a[3]=((C_word)li415),tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14614,a[2]=t3,a[3]=((C_word)li416),tmp=(C_word)a,a+=4,tmp);
/* library.scm:2182: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
C_call_with_values(4,av2);}}
else{
if(C_truep(C_i_ratnump(t3))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14636,a[2]=t3,a[3]=((C_word)li417),tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14648,a[2]=t3,a[3]=t2,a[4]=((C_word)li420),tmp=(C_word)a,a+=5,tmp);
/* library.scm:2187: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
C_call_with_values(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14700,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2195: scheme#exact->inexact */
t5=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}}}}

/* k14555 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14557,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14561,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2176: scheme#magnitude */
t3=*((C_word*)lf[515]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14559 in k14555 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_14561,c,av);}
a=C_alloc(9);
t2=C_a_i_sqrt(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14580,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2177: scheme#cos */
t4=*((C_word*)lf[573]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k14574 in k14578 in k14559 in k14555 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,3)))){
C_save_and_reclaim((void *)f_14576,c,av);}
a=C_alloc(33);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2177: make-complex */
f_12675(((C_word*)t0)[3],((C_word*)t0)[4],t2);}

/* k14578 in k14559 in k14555 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(38,c,2)))){
C_save_and_reclaim((void *)f_14580,c,av);}
a=C_alloc(38);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14576,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2177: scheme#sin */
t4=*((C_word*)lf[569]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k14595 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14597,c,av);}
a=C_alloc(4);
t2=C_a_i_sqrt(&a,1,t1);
/* library.scm:2179: make-complex */
f_12675(((C_word*)t0)[2],lf[504],t2);}

/* a14607 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14608,c,av);}
/* library.scm:2182: ##sys#exact-integer-sqrt */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[586]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[586]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* a14613 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14614,c,av);}
a=C_alloc(3);
t4=C_eqp(C_fix(0),t3);
if(C_truep(t4)){
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14625,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2185: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k14623 in a14613 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_14625,c,av);}
a=C_alloc(4);
t2=C_a_i_sqrt(&a,1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a14635 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14636,c,av);}
/* library.scm:2187: ##sys#exact-integer-sqrt */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[586]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[586]+1);
av2[1]=t1;
av2[2]=C_u_i_ratnum_num(((C_word*)t0)[2]);
tp(3,av2);}}

/* a14647 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_14648,c,av);}
a=C_alloc(10);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14660,a[2]=((C_word*)t0)[2],a[3]=((C_word)li418),tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14672,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=((C_word)li419),tmp=(C_word)a,a+=6,tmp);
/* library.scm:2189: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14696,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2194: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* a14659 in a14647 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14660,c,av);}
/* library.scm:2190: ##sys#exact-integer-sqrt */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[586]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[586]+1);
av2[1]=t1;
av2[2]=C_u_i_ratnum_denom(((C_word*)t0)[2]);
tp(3,av2);}}

/* a14671 in a14647 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_14672,c,av);}
a=C_alloc(4);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
/* library.scm:2192: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
tp(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14689,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2193: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k14687 in a14671 in a14647 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14689,c,av);}
/* library.scm:2193: ##sys#sqrt/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[580]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[580]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k14694 in a14647 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14696,c,av);}
/* library.scm:2194: ##sys#sqrt/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[580]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[580]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k14698 in ##sys#sqrt/loc in k10477 in k8569 */
static void C_ccall f_14700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_14700,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_sqrt(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#sqrt in k10477 in k8569 */
static void C_ccall f_14702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14702,c,av);}
/* library.scm:2197: ##sys#sqrt/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[580]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[580]+1);
av2[1]=t1;
av2[2]=lf[590];
av2[3]=t2;
tp(4,av2);}}

/* chicken.base#exact-integer-nth-root in k10477 in k8569 */
static void C_ccall f_14708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14708,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14712,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2201: ##sys#check-exact-uinteger */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[362]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[362]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[593];
tp(4,av2);}}

/* k14710 in chicken.base#exact-integer-nth-root in k10477 in k8569 */
static void C_ccall f_14712(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14712,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14715,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2202: ##sys#check-exact-uinteger */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[362]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[362]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[593];
tp(4,av2);}}

/* k14713 in k14710 in chicken.base#exact-integer-nth-root in k10477 in k8569 */
static void C_ccall f_14715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_14715,c,av);}
/* library.scm:2203: ##sys#exact-integer-nth-root/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[592]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[592]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[593];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}

/* ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_14720,c,av);}
a=C_alloc(5);
t5=C_eqp(C_fix(0),t3);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14730,a[2]=t1,a[3]=t3,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t5)){
t7=t6;
f_14730(t7,t5);}
else{
t7=C_eqp(C_fix(1),t3);
t8=t6;
f_14730(t8,(C_truep(t7)?t7:C_eqp(C_fix(1),t4)));}}

/* k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_fcall f_14730(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,3)))){
C_save_and_reclaim_args((void *)trf_14730,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
/* library.scm:2209: scheme#values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_fix(0);
C_values(4,av2);}}
else{
t2=C_i_integer_length(((C_word*)t0)[3]);
if(C_truep(C_i_lessp(t2,((C_word*)t0)[4]))){
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[3],C_fix(1));
/* library.scm:2212: scheme#values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
av2[3]=t3;
C_values(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14750,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14826,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14830,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=C_fixnum_plus(t2,C_fix(1));
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}}}

/* k14748 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(75,c,3)))){
C_save_and_reclaim((void *)f_14750,c,av);}
a=C_alloc(75);
t2=C_s_a_i_arithmetic_shift(&a,2,C_fix(1),t1);
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[2],C_fix(1));
t4=C_s_a_i_times(&a,2,t3,t2);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_14822,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=((C_word*)t0)[4],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* library.scm:2220: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=t3;
tp(4,av2);}}

/* lp in k14820 in k14748 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_fcall f_14765(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(41,0,3)))){
C_save_and_reclaim_args((void *)trf_14765,4,t0,t1,t2,t3);}
a=C_alloc(41);
if(C_truep(C_i_lessp(t3,t2))){
t4=C_s_a_i_times(&a,2,((C_word*)t0)[2],t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_14795,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* library.scm:2225: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t5;
av2[2]=t3;
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14806,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2227: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}}

/* k14793 in lp in k14820 in k14748 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,3)))){
C_save_and_reclaim((void *)f_14795,c,av);}
a=C_alloc(39);
t2=C_s_a_i_quotient(&a,2,((C_word*)t0)[2],t1);
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[3],t2);
t4=C_s_a_i_quotient(&a,2,t3,((C_word*)t0)[4]);
/* library.scm:2223: lp */
t5=((C_word*)((C_word*)t0)[5])[1];
f_14765(t5,((C_word*)t0)[6],((C_word*)t0)[7],t4);}

/* k14804 in lp in k14820 in k14748 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_14806,c,av);}
a=C_alloc(29);
t2=C_s_a_i_minus(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2227: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
C_values(4,av2);}}

/* k14820 in k14748 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(48,c,4)))){
C_save_and_reclaim((void *)f_14822,c,av);}
a=C_alloc(48);
t2=C_s_a_i_quotient(&a,2,((C_word*)t0)[2],t1);
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[3],t2);
t4=C_s_a_i_quotient(&a,2,t3,((C_word*)t0)[4]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14765,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=t6,a[6]=((C_word)li424),tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_14765(t8,((C_word*)t0)[6],((C_word*)t0)[7],t4);}

/* k14824 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14826,c,av);}
/* library.scm:2214: scheme#inexact->exact */
t2=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k14828 in k14728 in ##sys#exact-integer-nth-root/loc in k10477 in k8569 */
static void C_ccall f_14830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14830,c,av);}
/* library.scm:2214: scheme#ceiling */
t2=*((C_word*)lf[549]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* ##sys#integer-power in k10477 in k8569 */
static void C_ccall f_14846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_14846,c,av);}
a=C_alloc(8);
if(C_truep(C_i_negativep(t3))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14864,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=C_s_a_u_i_integer_negate(&a,1,t3);
/* library.scm:2232: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=t5;
tp(4,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14875,a[2]=t5,a[3]=t2,a[4]=((C_word)li426),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_14875(t7,t1,C_fix(1),t3);}}

/* k14862 in ##sys#integer-power in k10477 in k8569 */
static void C_ccall f_14864(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14864,c,av);}
/* library.scm:2232: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
av2[3]=t1;
tp(4,av2);}}

/* lp in ##sys#integer-power in k10477 in k8569 */
static void C_fcall f_14875(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(62,0,3)))){
C_save_and_reclaim_args((void *)trf_14875,4,t0,t1,t2,t3);}
a=C_alloc(62);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_evenp(t3))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14897,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t6=C_s_a_i_arithmetic_shift(&a,2,t3,C_fix(-1));
/* library.scm:2237: lp */
t8=t5;
t9=C_fix(1);
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t5=C_s_a_i_times(&a,2,t2,((C_word*)t0)[3]);
t6=C_s_a_i_minus(&a,2,t3,C_fix(1));
/* library.scm:2239: lp */
t8=t1;
t9=t5;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}}}

/* k14895 in lp in ##sys#integer-power in k10477 in k8569 */
static void C_ccall f_14897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(66,c,1)))){
C_save_and_reclaim((void *)f_14897,c,av);}
a=C_alloc(66);
t2=C_s_a_i_times(&a,2,t1,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_times(&a,2,((C_word*)t0)[3],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#expt in k10477 in k8569 */
static void C_ccall f_14914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_14914,c,av);}
a=C_alloc(21);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14917,a[2]=((C_word)li428),tmp=(C_word)a,a+=3,tmp));
t9=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14931,a[2]=((C_word)li429),tmp=(C_word)a,a+=3,tmp));
t10=C_i_numberp(t2);
if(C_truep(C_i_not(t10))){
/* library.scm:2251: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[594];
tp(4,av2);}}
else{
t11=C_i_numberp(t3);
if(C_truep(C_i_not(t11))){
/* library.scm:2252: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=lf[594];
tp(4,av2);}}
else{
t12=C_i_ratnump(t2);
t13=(C_truep(t12)?C_i_not(C_i_inexactp(t3)):C_SCHEME_FALSE);
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14983,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2256: scheme#expt */
t15=*((C_word*)lf[533]+1);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t14;
av2[2]=C_u_i_ratnum_num(t2);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t15+1)))(4,av2);}}
else{
if(C_truep(C_i_ratnump(t3))){
if(C_truep(C_i_exact_integerp(t2))){
if(C_truep(C_i_negativep(t2))){
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15022,a[2]=t3,a[3]=t5,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2263: scheme#exact->inexact */
t15=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t14;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t15+1)))(3,av2);}}
else{
t14=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15029,a[2]=t2,a[3]=t3,a[4]=((C_word)li430),tmp=(C_word)a,a+=5,tmp);
t15=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15041,a[2]=t3,a[3]=t2,a[4]=((C_word)li431),tmp=(C_word)a,a+=5,tmp);
/* library.scm:2264: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t14;
av2[3]=t15;
C_call_with_values(4,av2);}}}
else{
if(C_truep(C_i_flonump(t2))){
t14=C_a_i_exact_to_inexact(&a,1,t3);
/* library.scm:2273: log-expt */
f_14917(t1,t2,t14);}
else{
/* library.scm:2274: slow-expt */
f_14931(t1,t2,t3);}}}
else{
t14=C_i_cplxnump(t3);
t15=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15085,a[2]=t7,a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
if(C_truep(t14)){
t16=t15;
f_15085(t16,t14);}
else{
t16=C_i_cplxnump(t2);
t17=t15;
f_15085(t17,(C_truep(t16)?C_i_not(C_i_integerp(t3)):C_SCHEME_FALSE));}}}}}}

/* log-expt in scheme#expt in k10477 in k8569 */
static void C_fcall f_14917(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_14917,3,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14929,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2244: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}

/* k14927 in log-expt in scheme#expt in k10477 in k8569 */
static void C_ccall f_14929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_14929,c,av);}
a=C_alloc(33);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2244: scheme#exp */
t3=*((C_word*)lf[562]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* slow-expt in scheme#expt in k10477 in k8569 */
static void C_fcall f_14931(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,6)))){
C_save_and_reclaim_args((void *)trf_14931,3,t1,t2,t3);}
a=C_alloc(4);
t4=C_eqp(C_fix(0),t2);
if(C_truep(t4)){
/* library.scm:2247: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word av2[7];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[565];
av2[3]=lf[594];
av2[4]=lf[595];
av2[5]=t2;
av2[6]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(7,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14952,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2250: ##sys#log-1 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[564]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[564]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}}

/* k14950 in slow-expt in scheme#expt in k10477 in k8569 */
static void C_ccall f_14952(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_14952,c,av);}
a=C_alloc(33);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2250: scheme#exp */
t3=*((C_word*)lf[562]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14981 in scheme#expt in k10477 in k8569 */
static void C_ccall f_14983(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,3)))){
C_save_and_reclaim((void *)f_14983,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14987,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_u_i_ratnum_denom(((C_word*)t0)[3]);
t4=C_s_a_i_negate(&a,1,((C_word*)t0)[4]);
/* library.scm:2257: scheme#expt */
t5=*((C_word*)lf[533]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k14985 in k14981 in scheme#expt in k10477 in k8569 */
static void C_ccall f_14987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,1)))){
C_save_and_reclaim((void *)f_14987,c,av);}
a=C_alloc(33);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_times(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k15020 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_15022,c,av);}
a=C_alloc(11);
t2=C_a_i_exact_to_inexact(&a,1,((C_word*)t0)[2]);
/* library.scm:2263: log-expt */
f_14917(((C_word*)t0)[4],t1,t2);}

/* a15028 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_15029,c,av);}
/* library.scm:2265: ##sys#exact-integer-nth-root/loc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[592]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[592]+1);
av2[1]=t1;
av2[2]=lf[594];
av2[3]=((C_word*)t0)[2];
av2[4]=C_u_i_ratnum_denom(((C_word*)t0)[3]);
tp(5,av2);}}

/* a15040 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_15041,c,av);}
a=C_alloc(4);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
/* library.scm:2268: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_u_i_ratnum_num(((C_word*)t0)[2]);
tp(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15061,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2270: scheme#exact->inexact */
t6=*((C_word*)lf[62]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k15059 in a15040 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15061(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_15061,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15065,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2271: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15063 in k15059 in a15040 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_15065,c,av);}
a=C_alloc(4);
t2=C_a_i_flonum_expt(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15083 in scheme#expt in k10477 in k8569 */
static void C_fcall f_15085(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_15085,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
/* library.scm:2276: slow-expt */
f_14931(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
t2=(C_truep(C_i_flonump(((C_word*)t0)[5]))?C_i_not(C_u_i_fpintegerp(((C_word*)t0)[5])):C_SCHEME_FALSE);
if(C_truep(t2)){
if(C_truep(C_i_negativep(((C_word*)t0)[4]))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15105,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2280: scheme#exact->inexact */
t4=*((C_word*)lf[62]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15113,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2282: scheme#exact->inexact */
t4=*((C_word*)lf[62]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}
else{
if(C_truep(C_i_flonump(((C_word*)t0)[4]))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15120,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2284: scheme#exact->inexact */
t4=*((C_word*)lf[62]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_i_inexactp(((C_word*)t0)[4]);
t4=(C_truep(t3)?t3:C_i_inexactp(((C_word*)t0)[5]));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15136,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15140,a[2]=t5,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2288: scheme#inexact->exact */
t7=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
/* library.scm:2289: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}}}}}

/* k15103 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15105,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15109,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2280: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15107 in k15103 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15109,c,av);}
/* library.scm:2280: log-expt */
f_14917(((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k15111 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_15113,c,av);}
a=C_alloc(4);
t2=C_a_i_flonum_expt(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15118 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_15120,c,av);}
a=C_alloc(4);
t2=C_a_i_flonum_expt(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15134 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15136(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15136,c,av);}
/* library.scm:2288: scheme#exact->inexact */
t2=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k15138 in k15083 in scheme#expt in k10477 in k8569 */
static void C_ccall f_15140(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15140,c,av);}
/* library.scm:2288: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[531]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* ##sys#internal-gcd in k10477 in k8569 */
static void C_fcall f_15181(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_15181,4,t1,t2,t3,t4);}
a=C_alloc(5);
if(C_truep(C_i_exact_integerp(t3))){
if(C_truep(C_i_exact_integerp(t4))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_s_a_u_i_integer_gcd(&a,2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(C_truep(C_i_flonump(t4))?C_u_i_fpintegerp(t4):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15214,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2297: scheme#inexact->exact */
t7=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
/* library.scm:2298: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t2;
tp(4,av2);}}}}
else{
t5=(C_truep(C_i_flonump(t3))?C_u_i_fpintegerp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
if(C_truep(C_i_flonump(t4))){
t6=C_a_i_flonum_gcd(&a,2,t3,t4);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
if(C_truep(C_i_exact_integerp(t4))){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15243,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2304: scheme#inexact->exact */
t7=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
/* library.scm:2305: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=t2;
tp(4,av2);}}}}
else{
/* library.scm:2306: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=t2;
tp(4,av2);}}}}

/* k15212 in ##sys#internal-gcd in k10477 in k8569 */
static void C_ccall f_15214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15214,c,av);}
a=C_alloc(5);
t2=C_s_a_u_i_integer_gcd(&a,2,((C_word*)t0)[2],t1);
/* library.scm:2297: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15241 in ##sys#internal-gcd in k10477 in k8569 */
static void C_ccall f_15243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_15243,c,av);}
a=C_alloc(5);
t2=C_s_a_u_i_integer_gcd(&a,2,t1,((C_word*)t0)[2]);
/* library.scm:2304: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* ##sys#gcd in k10477 in k8569 */
static void C_ccall f_15251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_15251,c,av);}
/* library.scm:2308: ##sys#internal-gcd */
f_15181(t1,lf[597],t2,t3);}

/* scheme#gcd in k10477 in k8569 */
static void C_ccall f_15257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_15257,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=C_slot(t2,C_fix(1));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15277,a[2]=t7,a[3]=((C_word)li435),tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_15277(t9,t1,t4,t5);}}

/* loop in scheme#gcd in k10477 in k8569 */
static void C_fcall f_15277(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_15277,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t3))){
if(C_truep(C_i_integerp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_s_a_i_abs(&a,1,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* library.scm:2317: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[597];
tp(4,av2);}}}
else{
t4=C_slot(t3,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15302,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2319: ##sys#internal-gcd */
f_15181(t5,lf[597],t2,t4);}}

/* k15300 in loop in scheme#gcd in k10477 in k8569 */
static void C_ccall f_15302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15302,c,av);}
/* library.scm:2319: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_15277(t2,((C_word*)t0)[3],t1,C_slot(((C_word*)t0)[4],C_fix(1)));}

/* ##sys#lcm in k10477 in k8569 */
static void C_ccall f_15308(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_15308,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15312,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2323: ##sys#internal-gcd */
f_15181(t4,lf[600],t2,t3);}

/* k15310 in ##sys#lcm in k10477 in k8569 */
static void C_ccall f_15312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(45,c,1)))){
C_save_and_reclaim((void *)f_15312,c,av);}
a=C_alloc(45);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_s_a_i_quotient(&a,2,t2,t1);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_s_a_i_abs(&a,1,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#lcm in k10477 in k8569 */
static void C_ccall f_15322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_15322,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
if(C_truep(C_rest_nullp(c,2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fix(1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15342,a[2]=t6,a[3]=((C_word)li438),tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_15342(t8,t1,t3,t4);}}

/* loop in scheme#lcm in k10477 in k8569 */
static void C_fcall f_15342(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_15342,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t3))){
if(C_truep(C_i_integerp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_s_a_i_abs(&a,1,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* library.scm:2333: ##sys#error-bad-integer */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[359]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[359]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[600];
tp(4,av2);}}}
else{
t4=C_slot(t3,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15363,a[2]=t2,a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* library.scm:2335: ##sys#internal-gcd */
f_15181(t5,lf[600],t2,t4);}}

/* k15361 in loop in scheme#lcm in k10477 in k8569 */
static void C_ccall f_15363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(38,c,3)))){
C_save_and_reclaim((void *)f_15363,c,av);}
a=C_alloc(38);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_s_a_i_quotient(&a,2,t2,t1);
/* library.scm:2336: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_15342(t4,((C_word*)t0)[5],t3,C_slot(((C_word*)t0)[6],C_fix(1)));}

/* ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_15380,c,av);}
a=C_alloc(6);
t5=C_fixnum_plus(t4,C_fix(1));
t6=C_fixnum_shift_right(t5,C_fix(1));
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15387,a[2]=t2,a[3]=t6,a[4]=t3,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:2343: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t7;
av2[2]=t3;
av2[3]=t6;
tp(4,av2);}}

/* k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_15387,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15392,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li441),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15398,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word)li442),tmp=(C_word)a,a+=5,tmp);
/* library.scm:2342: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a15391 in k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_15392,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_u_integer_quotient_and_remainder,a[2]=((C_word)li440),tmp=(C_word)a,a+=3,tmp);
/* library.scm:2342: g2777 */
t3=t2;{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t3))(4,av2);}}

/* a15397 in k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_15398,c,av);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15402,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k15400 in a15397 in k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_15402,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15405,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_s_a_i_abs(&a,1,((C_word*)t0)[4]);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=t3;
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}

/* k15403 in k15400 in a15397 in k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15405,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15412,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=C_block_size(t1);
/* ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t2;
av2[2]=C_fixnum_difference(((C_word*)t0)[4],t3);
av2[3]=C_make_character(48);
tp(4,av2);}}

/* k15410 in k15403 in k15400 in a15397 in k15385 in ##sys#integer->string/recursive in k10477 in k8569 */
static void C_ccall f_15412(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_15412,c,av);}
/* library.scm:2348: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* ##sys#extended-number->string in k10477 in k8569 */
static void C_ccall f_15433(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15433,c,av);}
a=C_alloc(5);
if(C_truep(C_i_ratnump(t2))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15447,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=C_u_i_ratnum_num(t2);
av2[3]=t3;
tp(4,av2);}}
else{
if(C_truep(C_i_cplxnump(t2))){
t4=C_u_i_cplxnum_real(t2);
t5=C_u_i_cplxnum_imag(t2);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15488,a[2]=t5,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t6;
av2[2]=t4;
av2[3]=t3;
tp(4,av2);}}
else{
/* library.scm:2370: ##sys#error-bad-number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[374]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[374]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[609];
tp(4,av2);}}}}

/* k15445 in ##sys#extended-number->string in k10477 in k8569 */
static void C_ccall f_15447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_15447,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15451,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=C_u_i_ratnum_denom(((C_word*)t0)[3]);
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}

/* k15449 in k15445 in ##sys#extended-number->string in k10477 in k8569 */
static void C_ccall f_15451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_15451,c,av);}
/* library.scm:2358: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[605];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k15486 in ##sys#extended-number->string in k10477 in k8569 */
static void C_ccall f_15488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15488,c,av);}
a=C_alloc(5);
t2=(C_truep(C_i_finitep(((C_word*)t0)[2]))?(C_truep(C_i_positivep(((C_word*)t0)[2]))?lf[606]:lf[607]):lf[607]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15496,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}

/* k15494 in k15486 in ##sys#extended-number->string in k10477 in k8569 */
static void C_ccall f_15496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_15496,c,av);}
/* library.scm:2365: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
av2[5]=lf[608];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k15523 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(44,c,1)))){
C_save_and_reclaim((void *)f_15525,c,av);}
a=C_alloc(44);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_exact_to_inexact(&a,1,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15528(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_15528,c,av);}
a=C_alloc(13);
t2=C_fixnum_difference(C_i_integer_length(((C_word*)t0)[2]),C_i_integer_length(t1));
t3=C_fixnum_difference(t2,*((C_word*)lf[429]+1));
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
if(C_truep(C_fixnum_lessp(((C_word*)t5)[1],C_fix(0)))){
t6=C_fixnum_negate(((C_word*)t5)[1]);
t7=C_s_a_i_arithmetic_shift(&a,2,((C_word*)t0)[2],t6);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15543,a[2]=t5,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t7,tmp=(C_word)a,a+=6,tmp);
/* library.scm:2389: round-quotient */
f_15607(t8,t7,t1);}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15582,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2396: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t6;
av2[2]=C_fix(10);
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}}

/* k15541 in k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(44,c,3)))){
C_save_and_reclaim((void *)f_15543,c,av);}
a=C_alloc(44);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15546,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t5=((C_word*)t3)[1];
t6=C_i_integer_length(t5);
t7=*((C_word*)lf[429]+1);
if(C_truep(C_fixnum_greaterp(t6,*((C_word*)lf[429]+1)))){
t8=C_fixnum_plus(C_fix(1),((C_word*)((C_word*)t0)[2])[1]);
t9=C_set_block_item(((C_word*)t0)[2],0,t8);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15564,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t11=C_s_a_i_times(&a,2,((C_word*)t0)[4],C_fix(2));
/* library.scm:2393: round-quotient */
f_15607(t10,((C_word*)t0)[5],t11);}
else{
t8=t4;
f_15546(t8,C_SCHEME_UNDEFINED);}}

/* k15544 in k15541 in k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15546(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_15546,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15553,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2394: scheme#exact->inexact */
t3=*((C_word*)lf[62]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15551 in k15544 in k15541 in k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_15553,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_bytevector(&a,1,C_fix(4));
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=stub2831(t3,C_i_foreign_flonum_argumentp(t1),C_i_foreign_fixnum_argumentp(t2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k15562 in k15541 in k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15564,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];
f_15546(t3,t2);}

/* k15580 in k15526 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15582(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,1)))){
C_save_and_reclaim((void *)f_15582,c,av);}
a=C_alloc(33);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_times(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* round-quotient in k10477 in k8569 */
static void C_fcall f_15607(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(63,0,3)))){
C_save_and_reclaim_args((void *)trf_15607,3,t1,t2,t3);}
a=C_alloc(63);
t4=C_s_a_u_i_integer_quotient(&a,2,t2,t3);
t5=(C_truep(C_i_evenp(t4))?*((C_word*)lf[73]+1):*((C_word*)lf[75]+1));
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15622,a[2]=t2,a[3]=t3,a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t7=C_s_a_i_remainder(&a,2,t2,t3);
t8=C_s_a_i_abs(&a,1,t7);
t9=C_s_a_i_times(&a,2,t8,C_fix(2));
t10=C_s_a_i_abs(&a,1,t3);
/* library.scm:2403: g2844 */
t11=t5;{
C_word av2[4];
av2[0]=t11;
av2[1]=t6;
av2[2]=t9;
av2[3]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(4,av2);}}

/* k15620 in round-quotient in k10477 in k8569 */
static void C_ccall f_15622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_15622,c,av);}
a=C_alloc(29);
if(C_truep(t1)){
t2=C_eqp(C_i_negativep(((C_word*)t0)[2]),C_i_negativep(((C_word*)t0)[3]));
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_s_a_i_plus(&a,2,((C_word*)t0)[5],C_fix(1)):C_s_a_i_plus(&a,2,((C_word*)t0)[5],C_fix(-1)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15650(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(87,0,5)))){
C_save_and_reclaim_args((void *)trf_15650,5,t1,t2,t3,t4,t5);}
a=C_alloc(87);
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t8,0,C_SCHEME_FALSE);
t14=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15654,a[2]=t6,a[3]=t8,a[4]=((C_word)li446),tmp=(C_word)a,a+=5,tmp));
t15=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15668,a[2]=t6,a[3]=((C_word)li447),tmp=(C_word)a,a+=4,tmp));
t16=C_block_size(t3);
t17=C_fixnum_difference(t2,C_fix(1));
t18=C_fixnum_plus(C_fix(48),t17);
t19=C_make_character(C_unfix(t18));
t20=C_fixnum_difference(t2,C_fix(11));
t21=C_fixnum_plus(C_fix(97),t20);
t22=C_make_character(C_unfix(t21));
t23=C_fixnum_difference(t2,C_fix(11));
t24=C_fixnum_plus(C_fix(65),t23);
t25=C_make_character(C_unfix(t24));
t26=C_SCHEME_FALSE;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15748,a[2]=t16,a[3]=t3,a[4]=t2,a[5]=t19,a[6]=t22,a[7]=t25,a[8]=((C_word)li449),tmp=(C_word)a,a+=9,tmp);
t29=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15856,a[2]=t16,a[3]=t3,a[4]=((C_word)li451),tmp=(C_word)a,a+=5,tmp);
t30=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15906,a[2]=t3,a[3]=t2,a[4]=t27,a[5]=t10,a[6]=t29,a[7]=t28,a[8]=((C_word)li452),tmp=(C_word)a,a+=9,tmp);
t31=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15962,a[2]=t16,a[3]=t3,a[4]=t2,a[5]=t28,a[6]=((C_word)li453),tmp=(C_word)a,a+=7,tmp);
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16020,a[2]=t16,a[3]=t3,a[4]=t12,a[5]=t31,a[6]=t30,a[7]=((C_word)li454),tmp=(C_word)a,a+=8,tmp);
t33=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_16234,a[2]=t10,a[3]=t32,a[4]=t3,a[5]=t2,a[6]=t16,a[7]=t12,a[8]=t31,a[9]=t27,a[10]=t6,a[11]=t30,a[12]=((C_word)li455),tmp=(C_word)a,a+=13,tmp);
t34=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16580,a[2]=t16,a[3]=t3,a[4]=t33,a[5]=t10,a[6]=((C_word)li456),tmp=(C_word)a,a+=7,tmp);
t35=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_16820,a[2]=t6,a[3]=t8,a[4]=t1,a[5]=t3,a[6]=t16,a[7]=t4,a[8]=t27,a[9]=t34,tmp=(C_word)a,a+=10,tmp);
/* library.scm:2589: scan-real */
t36=t34;
f_16580(t36,t35,t4);}

/* go-inexact! in ##sys#string->compnum in k10477 in k8569 */
static C_word C_fcall f_15654(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;{}
t2=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[612]);
if(C_truep(t2)){
t3=C_SCHEME_UNDEFINED;
return(t3);}
else{
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,lf[613]);
t4=((C_word*)((C_word*)t0)[3])[1];
if(C_truep(t4)){
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t4);
return(t5);}
else{
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t1);
return(t5);}}}

/* safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15668(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_15668,4,t0,t1,t2,t3);}
a=C_alloc(12);
if(C_truep(t3)){
if(C_truep(C_i_not(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_fix(0);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_greaterp(t3,C_fix(10000)))){
t4=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[613]);
if(C_truep(t4)){
if(C_truep(C_i_zerop(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[504];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_greaterp(t2,lf[504]);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?lf[614]:lf[615]);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t4=C_fixnum_negate(C_fix(10000));
if(C_truep(C_i_lessp(t3,t4))){
t5=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[613]);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?lf[504]:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[613]);
if(C_truep(t5)){
if(C_truep(C_i_not(C_i_negativep(t3)))){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15525,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2382: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t6;
av2[2]=C_fix(10);
av2[3]=t3;
tp(4,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15528,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t7=C_s_a_i_abs(&a,1,t3);
/* library.scm:2383: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t6;
av2[2]=C_fix(10);
av2[3]=t7;
tp(4,av2);}}}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15728,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2428: ##sys#integer-power */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[531]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[531]+1);
av2[1]=t6;
av2[2]=C_fix(10);
av2[3]=t3;
tp(4,av2);}}}}}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15726 in safe-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,1)))){
C_save_and_reclaim((void *)f_15728,c,av);}
a=C_alloc(33);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_s_a_i_times(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scan-digits in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15748(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_15748,3,t0,t1,t2);}
a=C_alloc(13);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_15754,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t4,a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word)li448),tmp=(C_word)a,a+=11,tmp));
t6=((C_word*)t4)[1];
f_15754(t6,t1,t2);}

/* lp in scan-digits in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15754(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15754,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=C_fixnum_greaterp(t2,((C_word*)t0)[3]);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_a_i_cons(&a,2,t2,C_SCHEME_FALSE):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_subchar(((C_word*)t0)[4],t2);
if(C_truep(C_fixnum_less_or_equal_p(((C_word*)t0)[5],C_fix(10)))){
t5=C_i_char_greater_or_equal_p(t4,C_make_character(48));
t6=(C_truep(t5)?C_u_i_char_less_or_equal_p(t4,((C_word*)t0)[6]):C_SCHEME_FALSE);
if(C_truep(t6)){
/* library.scm:2452: lp */
t10=t1;
t11=C_fixnum_plus(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t7=C_fixnum_greaterp(t2,((C_word*)t0)[3]);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?C_a_i_cons(&a,2,t2,t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}
else{
t5=C_i_char_greater_or_equal_p(t4,C_make_character(48));
t6=(C_truep(t5)?C_u_i_char_less_or_equal_p(t4,C_make_character(57)):C_SCHEME_FALSE);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15817,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t6)){
t8=t7;
f_15817(t8,t6);}
else{
t8=(C_truep(C_u_i_char_greater_or_equal_p(t4,C_make_character(97)))?C_u_i_char_less_or_equal_p(t4,((C_word*)t0)[8]):C_SCHEME_FALSE);
t9=t7;
f_15817(t9,(C_truep(t8)?t8:(C_truep(C_u_i_char_greater_or_equal_p(t4,C_make_character(65)))?C_u_i_char_less_or_equal_p(t4,((C_word*)t0)[9]):C_SCHEME_FALSE)));}}}}

/* k15815 in lp in scan-digits in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15817(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_15817,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
/* library.scm:2457: lp */
t2=((C_word*)((C_word*)t0)[2])[1];
f_15754(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}
else{
t2=C_fixnum_greaterp(((C_word*)t0)[4],((C_word*)t0)[5]);
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=(C_truep(t2)?C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[4]):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* scan-hashes in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15856(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_15856,3,t0,t1,t2);}
a=C_alloc(9);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15862,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t4,a[6]=((C_word)li450),tmp=(C_word)a,a+=7,tmp));
t6=((C_word*)t4)[1];
f_15862(t6,t1,t2);}

/* lp in scan-hashes in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15862(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_15862,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=C_fixnum_greaterp(t2,((C_word*)t0)[3]);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_a_i_cons(&a,2,t2,C_SCHEME_FALSE):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_subchar(((C_word*)t0)[4],t2);
t5=C_eqp(t4,C_make_character(35));
if(C_truep(t5)){
/* library.scm:2466: lp */
t8=t1;
t9=C_fixnum_plus(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t6=C_fixnum_greaterp(t2,((C_word*)t0)[3]);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=(C_truep(t6)?C_a_i_cons(&a,2,t2,t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}

/* scan-digits+hashes in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15906(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_15906,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_15910,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=t1,a[9]=((C_word*)t0)[6],a[10]=t4,tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[4])[1]))){
/* library.scm:2470: scan-digits */
t6=((C_word*)t0)[7];
f_15748(t6,t5,t2);}
else{
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_15910(2,av2);}}}

/* k15908 in scan-digits+hashes in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_15910,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_15913,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t1)){
if(C_truep(C_i_cdr(t1))){
/* library.scm:2472: scan-hashes */
t3=((C_word*)t0)[9];
f_15856(t3,t2,C_u_i_cdr(t1));}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_15913(2,av2);}}}
else{
if(C_truep(((C_word*)t0)[10])){
/* library.scm:2473: scan-hashes */
t3=((C_word*)t0)[9];
f_15856(t3,t2,((C_word*)t0)[3]);}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_15913(2,av2);}}}}

/* k15911 in k15908 in scan-digits+hashes in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_15913,c,av);}
a=C_alloc(9);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_i_car(t2);
t4=C_s_a_i_digits_to_integer(&a,5,((C_word*)t0)[3],((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6]);
if(C_truep(t4)){
if(C_truep(t1)){
t5=C_set_block_item(((C_word*)t0)[7],0,C_SCHEME_TRUE);
t6=(
/* library.scm:2481: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[8])[1],((C_word*)t0)[6])
);
t7=C_u_i_cdr(t2);
t8=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_cons(&a,2,t4,t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t5=C_u_i_cdr(t2);
t6=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_cons(&a,2,t4,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t5=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* scan-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15962(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_15962,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_fixnum_lessp(t2,((C_word*)t0)[2]))){
t3=C_subchar(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15977,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t5=C_eqp(t3,C_make_character(43));
if(C_truep(t5)){
t6=t4;
f_15977(t6,lf[617]);}
else{
t6=C_eqp(t3,C_make_character(45));
t7=t4;
f_15977(t7,(C_truep(t6)?lf[616]:C_SCHEME_FALSE));}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15975 in scan-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_15977(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_15977,2,t0,t1);}
a=C_alloc(7);
t2=(C_truep(t1)?C_fixnum_plus(((C_word*)t0)[2],C_fix(1)):((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15986,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:2489: scan-digits */
t4=((C_word*)t0)[6];
f_15748(t4,t3,t2);}
else{
t3=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15984 in k15975 in scan-exponent in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_15986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_15986,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_i_car(t1);
t3=C_eqp(((C_word*)t0)[2],lf[616]);
t4=C_s_a_i_digits_to_integer(&a,5,((C_word*)t0)[3],((C_word*)t0)[4],t2,((C_word*)t0)[5],t3);
t5=C_u_i_cdr(t1);
t6=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_cons(&a,2,t4,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t2=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16020(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_16020,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
if(C_truep(C_fixnum_lessp(t2,((C_word*)t0)[2]))){
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_16030,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=t4,a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* library.scm:2497: scan-digits+hashes */
t6=((C_word*)t0)[6];
f_15906(t6,t5,t2,t3,t4);}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_16030,c,av);}
a=C_alloc(11);
t2=(C_truep(t1)?C_i_cdr(t1):((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_16039,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[7])){
t4=t3;
f_16039(t4,((C_word*)t0)[7]);}
else{
t4=C_i_not(t2);
t5=t3;
f_16039(t5,(C_truep(t4)?t4:C_fixnum_greaterp(t2,((C_word*)t0)[2])));}}

/* k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16039(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_16039,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(C_truep(((C_word*)t0)[2])?C_subchar(((C_word*)t0)[3],((C_word*)t0)[2]):C_SCHEME_FALSE);
t3=C_eqp(t2,C_make_character(101));
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_16051,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t3)){
t5=t4;
f_16051(t5,t3);}
else{
t5=C_eqp(t2,C_make_character(115));
if(C_truep(t5)){
t6=t4;
f_16051(t6,t5);}
else{
t6=C_eqp(t2,C_make_character(102));
if(C_truep(t6)){
t7=t4;
f_16051(t7,t6);}
else{
t7=C_eqp(t2,C_make_character(100));
if(C_truep(t7)){
t8=t4;
f_16051(t8,t7);}
else{
t8=C_eqp(t2,C_make_character(108));
if(C_truep(t8)){
t9=t4;
f_16051(t9,t8);}
else{
t9=C_eqp(t2,C_make_character(69));
if(C_truep(t9)){
t10=t4;
f_16051(t10,t9);}
else{
t10=C_eqp(t2,C_make_character(83));
if(C_truep(t10)){
t11=t4;
f_16051(t11,t10);}
else{
t11=C_eqp(t2,C_make_character(70));
if(C_truep(t11)){
t12=t4;
f_16051(t12,t11);}
else{
t12=C_eqp(t2,C_make_character(68));
t13=t4;
f_16051(t13,(C_truep(t12)?t12:C_eqp(t2,C_make_character(76))));}}}}}}}}}
else{
t2=((C_word*)t0)[6];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16051(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_16051,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[2],((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16060,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
/* library.scm:2505: scan-exponent */
t3=((C_word*)t0)[9];
f_15962(t3,t2,C_fixnum_plus(((C_word*)t0)[3],C_fix(1)));}
else{
t2=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=(C_truep(((C_word*)t0)[3])?((C_word*)t0)[3]:((C_word*)t0)[2]);
t3=(C_truep(((C_word*)t0)[4])?C_fixnum_difference(((C_word*)t0)[7],t2):C_SCHEME_FALSE);
t4=(C_truep(((C_word*)t0)[4])?C_i_car(((C_word*)t0)[4]):C_SCHEME_FALSE);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16126,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2515: safe-exponent */
t6=((C_word*)((C_word*)t0)[6])[1];
f_15668(t6,t5,t4,t3);}}

/* k16058 in k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16060(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_16060,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_i_car(t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16072,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* library.scm:2507: safe-exponent */
t4=((C_word*)((C_word*)t0)[4])[1];
f_15668(t4,t3,((C_word*)t0)[6],t2);}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16070 in k16058 in k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16072,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16078,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[2])){
t3=C_i_cdr(((C_word*)t0)[2]);
t4=t2;
f_16078(t4,C_fixnum_difference(((C_word*)t0)[6],C_fixnum_difference(t3,((C_word*)t0)[7])));}
else{
t3=t2;
f_16078(t3,C_SCHEME_FALSE);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16076 in k16070 in k16058 in k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16078(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_16078,2,t0,t1);}
a=C_alloc(5);
t2=(C_truep(((C_word*)t0)[2])?C_i_car(((C_word*)t0)[2]):C_SCHEME_FALSE);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16084,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2510: safe-exponent */
t4=((C_word*)((C_word*)t0)[6])[1];
f_15668(t4,t3,t2,t1);}

/* k16082 in k16076 in k16070 in k16058 in k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16084(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_16084,c,av);}
a=C_alloc(32);
if(C_truep(t1)){
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
t3=C_u_i_cdr(((C_word*)t0)[3]);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=C_u_i_cdr(((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16124 in k16049 in k16037 in k16028 in scan-decimal-tail in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_16126,c,av);}
a=C_alloc(32);
t2=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[2]:C_fix(0));
if(C_truep(t1)){
t3=C_s_a_i_plus(&a,2,t2,t1);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t3,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16234(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_16234,4,t0,t1,t2,t3);}
a=C_alloc(15);
t4=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_16241,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],a[12]=((C_word*)t0)[9],a[13]=((C_word*)t0)[10],a[14]=((C_word*)t0)[11],tmp=(C_word)a,a+=15,tmp);
t5=C_fixnum_plus(t2,C_fix(1));
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[6],t5))){
t6=C_eqp(((C_word*)t0)[5],C_fix(10));
t7=t4;
f_16241(t7,(C_truep(t6)?C_eqp(C_subchar(((C_word*)t0)[4],t2),C_make_character(46)):C_SCHEME_FALSE));}
else{
t6=t4;
f_16241(t6,C_SCHEME_FALSE);}}

/* k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16241(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_16241,2,t0,t1);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(
/* library.scm:2523: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3])
);
/* library.scm:2524: scan-decimal-tail */
t3=((C_word*)t0)[4];
f_16020(t3,((C_word*)t0)[5],C_fixnum_plus(((C_word*)t0)[6],C_fix(1)),((C_word*)t0)[3],C_SCHEME_FALSE);}
else{
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_16254,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],tmp=(C_word)a,a+=14,tmp);
/* library.scm:2525: scan-digits+hashes */
t3=((C_word*)t0)[14];
f_15906(t3,t2,((C_word*)t0)[6],((C_word*)t0)[3],C_SCHEME_FALSE);}}

/* k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_16254,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=C_i_cdr(t1);
t3=(C_truep(t2)?C_subchar(((C_word*)t0)[2],C_u_i_cdr(t1)):C_SCHEME_FALSE);
t4=C_eqp(t3,C_make_character(46));
if(C_truep(t4)){
t5=(
/* library.scm:2528: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[4])
);
t6=C_eqp(((C_word*)t0)[5],C_fix(10));
if(C_truep(t6)){
t7=C_fixnum_plus(C_u_i_cdr(t1),C_fix(1));
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[6],t7))){
/* library.scm:2531: scan-decimal-tail */
t8=((C_word*)t0)[7];
f_16020(t8,((C_word*)t0)[8],C_fixnum_plus(C_u_i_cdr(t1),C_fix(1)),((C_word*)t0)[4],C_u_i_car(t1));}
else{
t8=C_u_i_car(t1);
t9=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_a_i_cons(&a,2,t8,C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t7=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t5=C_eqp(t3,C_make_character(101));
t6=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_16312,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=t3,a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t5)){
t7=t6;
f_16312(t7,t5);}
else{
t7=C_eqp(t3,C_make_character(115));
if(C_truep(t7)){
t8=t6;
f_16312(t8,t7);}
else{
t8=C_eqp(t3,C_make_character(102));
if(C_truep(t8)){
t9=t6;
f_16312(t9,t8);}
else{
t9=C_eqp(t3,C_make_character(100));
if(C_truep(t9)){
t10=t6;
f_16312(t10,t9);}
else{
t10=C_eqp(t3,C_make_character(108));
if(C_truep(t10)){
t11=t6;
f_16312(t11,t10);}
else{
t11=C_eqp(t3,C_make_character(69));
if(C_truep(t11)){
t12=t6;
f_16312(t12,t11);}
else{
t12=C_eqp(t3,C_make_character(83));
if(C_truep(t12)){
t13=t6;
f_16312(t13,t12);}
else{
t13=C_eqp(t3,C_make_character(70));
if(C_truep(t13)){
t14=t6;
f_16312(t14,t13);}
else{
t14=C_eqp(t3,C_make_character(68));
t15=t6;
f_16312(t15,(C_truep(t14)?t14:C_eqp(t3,C_make_character(76))));}}}}}}}}}}
else{
t2=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16312(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_16312,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(
/* library.scm:2535: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3])
);
t3=C_eqp(((C_word*)t0)[4],C_fix(10));
if(C_truep(t3)){
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[5],C_u_i_cdr(((C_word*)t0)[6])))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16330,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2538: scan-exponent */
t5=((C_word*)t0)[9];
f_15962(t5,t4,C_fixnum_plus(C_u_i_cdr(((C_word*)t0)[6]),C_fix(1)));}
else{
t4=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t2=C_eqp(((C_word*)t0)[10],C_make_character(47));
if(C_truep(t2)){
t3=C_set_block_item(((C_word*)t0)[11],0,C_SCHEME_FALSE);
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[5],C_u_i_cdr(((C_word*)t0)[6])))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16376,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[12],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2545: scan-digits+hashes */
t5=((C_word*)t0)[13];
f_15906(t5,t4,C_fixnum_plus(C_u_i_cdr(((C_word*)t0)[6]),C_fix(1)),C_SCHEME_FALSE,C_SCHEME_FALSE);}
else{
t4=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k16328 in k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16330(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16330,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16340,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2540: safe-exponent */
t4=((C_word*)((C_word*)t0)[4])[1];
f_15668(t4,t3,t2,C_i_car(t1));}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16338 in k16328 in k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16340,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16374 in k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_16376,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_u_i_car(((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_i_car(t1);
if(C_truep(t3)){
t4=C_eqp(t3,C_fix(0));
if(C_truep(C_i_not(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16402,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2549: ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=t3;
tp(4,av2);}}
else{
t5=C_eqp(((C_word*)((C_word*)t0)[4])[1],lf[612]);
if(C_truep(C_i_not(t5))){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16413,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2553: chicken.base#signum */
t7=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16400 in k16374 in k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_16402,c,av);}
a=C_alloc(3);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,t1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k16411 in k16374 in k16310 in k16252 in k16239 in scan-ureal in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_16413,c,av);}
a=C_alloc(7);
switch(t1){
case C_fix(-1):
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[615],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
case C_fix(0):
t2=C_flonum(&a,C_ub_i_flonum_quotient((double)0.0,(double)0.0));
t3=C_u_i_cdr(((C_word*)t0)[2]);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}
case C_fix(1):
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[614],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
default:
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* scan-real in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16580(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_16580,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_fixnum_lessp(t2,((C_word*)t0)[2]))){
t3=C_subchar(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16595,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t5=C_eqp(t3,C_make_character(43));
if(C_truep(t5)){
t6=t4;
f_16595(t6,lf[617]);}
else{
t6=C_eqp(t3,C_make_character(45));
t7=t4;
f_16595(t7,(C_truep(t6)?lf[616]:C_SCHEME_FALSE));}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16593 in scan-real in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16595(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_16595,2,t0,t1);}
a=C_alloc(11);
t2=(C_truep(t1)?C_fixnum_plus(((C_word*)t0)[2],C_fix(1)):((C_word*)t0)[2]);
if(C_truep(C_fixnum_lessp(t2,((C_word*)t0)[3]))){
t3=C_subchar(((C_word*)t0)[4],t2);
t4=C_eqp(t3,C_make_character(105));
t5=(C_truep(t4)?t4:C_eqp(t3,C_make_character(73)));
if(C_truep(t5)){
if(C_truep(t1)){
t6=C_fixnum_plus(t2,C_fix(1));
t7=C_eqp(t6,((C_word*)t0)[3]);
if(C_truep(t7)){
t8=C_eqp(t1,lf[616]);
t9=(C_truep(t8)?C_a_i_cons(&a,2,C_fix(-1),t2):C_a_i_cons(&a,2,C_fix(1),t2));
if(C_truep(t9)){
t10=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
/* library.scm:2578: scan-ureal */
t10=((C_word*)t0)[6];
f_16234(t10,((C_word*)t0)[5],t2,C_eqp(t1,lf[616]));}}
else{
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16653,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t9=C_fixnum_plus(t2,C_fix(5));
if(C_truep(C_fixnum_less_or_equal_p(t9,((C_word*)t0)[3]))){
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16694,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2572: scheme#substring */
t11=*((C_word*)lf[397]+1);{
C_word av2[5];
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=C_fixnum_plus(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}
else{
t10=t8;
f_16653(t10,C_SCHEME_FALSE);}}}
else{
/* library.scm:2578: scan-ureal */
t6=((C_word*)t0)[6];
f_16234(t6,((C_word*)t0)[5],t2,C_eqp(t1,lf[616]));}}
else{
t6=C_eqp(t3,C_make_character(110));
t7=(C_truep(t6)?t6:C_eqp(t3,C_make_character(78)));
if(C_truep(t7)){
if(C_truep(t1)){
t8=C_fixnum_plus(t2,C_fix(5));
if(C_truep(C_fixnum_less_or_equal_p(t8,((C_word*)t0)[3]))){
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16772,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:2582: scheme#substring */
t10=*((C_word*)lf[397]+1);{
C_word av2[5];
av2[0]=t10;
av2[1]=t9;
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=C_fixnum_plus(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
/* library.scm:2587: scan-ureal */
t9=((C_word*)t0)[6];
f_16234(t9,((C_word*)t0)[5],t2,C_eqp(t1,lf[616]));}}
else{
/* library.scm:2587: scan-ureal */
t8=((C_word*)t0)[6];
f_16234(t8,((C_word*)t0)[5],t2,C_eqp(t1,lf[616]));}}
else{
/* library.scm:2588: scan-ureal */
t8=((C_word*)t0)[6];
f_16234(t8,((C_word*)t0)[5],t2,C_eqp(t1,lf[616]));}}}
else{
t3=((C_word*)t0)[5];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16651 in k16593 in scan-real in ##sys#string->compnum in k10477 in k8569 */
static void C_fcall f_16653(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_16653,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(
/* library.scm:2573: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[2])[1],C_eqp(((C_word*)t0)[3],lf[616]))
);
t3=C_eqp(((C_word*)t0)[3],lf[616]);
t4=(C_truep(t3)?lf[615]:lf[614]);
t5=C_fixnum_plus(((C_word*)t0)[4],C_fix(5));
if(C_truep(C_fixnum_lessp(t5,((C_word*)t0)[5]))){
t6=C_fixnum_plus(((C_word*)t0)[4],C_fix(5));
t7=C_a_i_cons(&a,2,t4,t6);
if(C_truep(t7)){
t8=((C_word*)t0)[6];{
C_word av2[2];
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
/* library.scm:2578: scan-ureal */
t8=((C_word*)t0)[7];
f_16234(t8,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}
else{
t6=C_a_i_cons(&a,2,t4,C_SCHEME_FALSE);
if(C_truep(t6)){
t7=((C_word*)t0)[6];{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* library.scm:2578: scan-ureal */
t7=((C_word*)t0)[7];
f_16234(t7,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}}
else{
/* library.scm:2578: scan-ureal */
t2=((C_word*)t0)[7];
f_16234(t2,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}

/* k16692 in k16593 in scan-real in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16694,c,av);}
t2=((C_word*)t0)[2];
f_16653(t2,C_i_string_ci_equal_p(t1,lf[618]));}

/* k16770 in k16593 in scan-real in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_16772,c,av);}
a=C_alloc(7);
if(C_truep(C_i_string_ci_equal_p(t1,lf[619]))){
t2=(
/* library.scm:2583: go-inexact! */
  f_15654(((C_word*)((C_word*)t0)[2])[1],C_eqp(((C_word*)t0)[3],lf[616]))
);
t3=C_flonum(&a,C_ub_i_flonum_quotient((double)0.0,(double)0.0));
t4=C_fixnum_plus(((C_word*)t0)[4],C_fix(5));
if(C_truep(C_fixnum_lessp(t4,((C_word*)t0)[5]))){
t5=C_fixnum_plus(((C_word*)t0)[4],C_fix(5));
t6=C_a_i_cons(&a,2,t3,t5);
if(C_truep(t6)){
t7=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* library.scm:2587: scan-ureal */
t7=((C_word*)t0)[7];
f_16234(t7,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}
else{
t5=C_a_i_cons(&a,2,t3,C_SCHEME_FALSE);
if(C_truep(t5)){
t6=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* library.scm:2587: scan-ureal */
t6=((C_word*)t0)[7];
f_16234(t6,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}}
else{
/* library.scm:2587: scan-ureal */
t2=((C_word*)t0)[7];
f_16234(t2,((C_word*)t0)[6],((C_word*)t0)[4],C_eqp(((C_word*)t0)[3],lf[616]));}}

/* k16818 in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_16820,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16823,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t1)){
t3=C_i_cdr(t1);
t4=(C_truep(t3)?C_subchar(((C_word*)t0)[5],C_u_i_cdr(t1)):C_SCHEME_FALSE);
t5=C_eqp(t4,C_SCHEME_FALSE);
if(C_truep(t5)){
t6=t2;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_u_i_car(t1);
f_16823(2,av2);}}
else{
t6=C_eqp(t4,C_make_character(105));
t7=(C_truep(t6)?t6:C_eqp(t4,C_make_character(73)));
if(C_truep(t7)){
t8=C_fixnum_plus(C_u_i_cdr(t1),C_fix(1));
t9=C_eqp(((C_word*)t0)[6],t8);
if(C_truep(t9)){
t10=C_eqp(C_subchar(((C_word*)t0)[5],((C_word*)t0)[7]),C_make_character(43));
if(C_truep(t10)){
if(C_truep(t10)){
/* library.scm:2595: scheme#make-rectangular */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[505]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[505]+1);
av2[1]=t2;
av2[2]=C_fix(0);
av2[3]=C_u_i_car(t1);
tp(4,av2);}}
else{
t11=t2;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t11=C_eqp(C_subchar(((C_word*)t0)[5],((C_word*)t0)[7]),C_make_character(45));
if(C_truep(t11)){
/* library.scm:2595: scheme#make-rectangular */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[505]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[505]+1);
av2[1]=t2;
av2[2]=C_fix(0);
av2[3]=C_u_i_car(t1);
tp(4,av2);}}
else{
t12=t2;{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}}
else{
t10=t2;{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t8=C_eqp(t4,C_make_character(43));
t9=(C_truep(t8)?t8:C_eqp(t4,C_make_character(45)));
if(C_truep(t9)){
t10=C_set_block_item(((C_word*)t0)[8],0,C_SCHEME_FALSE);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16922,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:2598: scan-real */
t12=((C_word*)t0)[9];
f_16580(t12,t11,C_u_i_cdr(t1));}
else{
t10=C_eqp(t4,C_make_character(64));
if(C_truep(t10)){
t11=C_set_block_item(((C_word*)t0)[8],0,C_SCHEME_FALSE);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16990,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2606: scan-real */
t13=((C_word*)t0)[9];
f_16580(t13,t12,C_fixnum_plus(C_u_i_cdr(t1),C_fix(1)));}
else{
t11=t2;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}

/* k16821 in k16818 in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16823,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[613]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16835,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:2611: scheme#exact->inexact */
t4=*((C_word*)lf[62]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_i_finitep(t1))?t1:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16833 in k16821 in k16818 in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16835,c,av);}
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
if(C_truep(C_u_i_zerop2(t1))){
/* library.scm:2614: chicken.flonum#fpneg */
t2=*((C_word*)lf[457]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k16920 in k16818 in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16922,c,av);}
if(C_truep(t1)){
if(C_truep(C_i_cdr(t1))){
t2=C_fixnum_plus(C_u_i_cdr(t1),C_fix(1));
t3=C_eqp(((C_word*)t0)[2],t2);
if(C_truep(t3)){
t4=C_u_i_cdr(t1);
t5=C_eqp(C_subchar(((C_word*)t0)[3],t4),C_make_character(105));
if(C_truep(t5)){
if(C_truep(t5)){
/* library.scm:2603: scheme#make-rectangular */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[505]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[505]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=C_u_i_car(((C_word*)t0)[5]);
av2[3]=C_u_i_car(t1);
tp(4,av2);}}
else{
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t6=C_u_i_cdr(t1);
t7=C_eqp(C_subchar(((C_word*)t0)[3],t6),C_make_character(73));
if(C_truep(t7)){
/* library.scm:2603: scheme#make-rectangular */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[505]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[505]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=C_u_i_car(((C_word*)t0)[5]);
av2[3]=C_u_i_car(t1);
tp(4,av2);}}
else{
t8=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}

/* k16988 in k16818 in ##sys#string->compnum in k10477 in k8569 */
static void C_ccall f_16990(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16990,c,av);}
if(C_truep(t1)){
t2=C_i_cdr(t1);
if(C_truep(C_i_not(t2))){
/* library.scm:2608: scheme#make-polar */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[507]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[507]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_u_i_car(((C_word*)t0)[3]);
av2[3]=C_u_i_car(t1);
tp(4,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_16823(2,av2);}}}

/* scheme#string->number in k10477 in k8569 */
static void C_ccall f_17070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17070,c,av);}
a=C_alloc(5);
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?C_fix(10):C_get_rest_arg(c,3,av,3,t0));
t5=C_i_check_string_2(t2,lf[621]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17080,a[2]=t2,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnump(t4))){
if(C_truep(C_fixnum_lessp(C_fix(1),t4))){
if(C_truep(C_fixnum_lessp(t4,C_fix(37)))){
t7=t6;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_17080(2,av2);}}
else{
/* library.scm:2623: ##sys#error-bad-base */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[376]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[376]+1);
av2[1]=t6;
av2[2]=t4;
av2[3]=lf[621];
tp(4,av2);}}}
else{
/* library.scm:2623: ##sys#error-bad-base */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[376]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[376]+1);
av2[1]=t6;
av2[2]=t4;
av2[3]=lf[621];
tp(4,av2);}}}
else{
/* library.scm:2623: ##sys#error-bad-base */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[376]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[376]+1);
av2[1]=t6;
av2[2]=t4;
av2[3]=lf[621];
tp(4,av2);}}}

/* k17078 in scheme#string->number in k10477 in k8569 */
static void C_ccall f_17080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,6)))){
C_save_and_reclaim((void *)f_17080,c,av);}
a=C_alloc(8);
t2=C_block_size(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17089,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word)li458),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_17089(t6,((C_word*)t0)[4],C_fix(0),C_SCHEME_FALSE,C_SCHEME_FALSE,t2);}

/* scan-prefix in k17078 in scheme#string->number in k10477 in k8569 */
static void C_fcall f_17089(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_17089,6,t0,t1,t2,t3,t4,t5);}
t6=C_fixnum_plus(t2,C_fix(2));
t7=C_fixnum_lessp(t6,t5);
t8=(C_truep(t7)?C_eqp(C_subchar(((C_word*)t0)[2],t2),C_make_character(35)):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=C_fixnum_plus(t2,C_fix(1));
t10=C_subchar(((C_word*)t0)[2],t9);
t11=C_eqp(t10,C_make_character(105));
t12=(C_truep(t11)?t11:C_eqp(t10,C_make_character(73)));
if(C_truep(t12)){
if(C_truep(C_i_not(t3))){
/* library.scm:2630: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=lf[613];
t27=t4;
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t13=t1;{
C_word av2[2];
av2[0]=t13;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}}
else{
t13=C_eqp(t10,C_make_character(101));
t14=(C_truep(t13)?t13:C_eqp(t10,C_make_character(69)));
if(C_truep(t14)){
if(C_truep(C_i_not(t3))){
/* library.scm:2631: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=lf[612];
t27=t4;
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t15=t1;{
C_word av2[2];
av2[0]=t15;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}}
else{
t15=C_eqp(t10,C_make_character(98));
t16=(C_truep(t15)?t15:C_eqp(t10,C_make_character(66)));
if(C_truep(t16)){
if(C_truep(C_i_not(t4))){
/* library.scm:2632: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=t3;
t27=C_fix(2);
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t17=t1;{
C_word av2[2];
av2[0]=t17;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}}
else{
t17=C_eqp(t10,C_make_character(111));
t18=(C_truep(t17)?t17:C_eqp(t10,C_make_character(79)));
if(C_truep(t18)){
if(C_truep(C_i_not(t4))){
/* library.scm:2633: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=t3;
t27=C_fix(8);
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t19=t1;{
C_word av2[2];
av2[0]=t19;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}}
else{
t19=C_eqp(t10,C_make_character(100));
t20=(C_truep(t19)?t19:C_eqp(t10,C_make_character(68)));
if(C_truep(t20)){
if(C_truep(C_i_not(t4))){
/* library.scm:2634: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=t3;
t27=C_fix(10);
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t21=t1;{
C_word av2[2];
av2[0]=t21;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}}
else{
t21=C_eqp(t10,C_make_character(120));
t22=(C_truep(t21)?t21:C_eqp(t10,C_make_character(88)));
if(C_truep(t22)){
if(C_truep(C_i_not(t4))){
/* library.scm:2635: scan-prefix */
t24=t1;
t25=C_fixnum_plus(t2,C_fix(2));
t26=t3;
t27=C_fix(16);
t28=t5;
t1=t24;
t2=t25;
t3=t26;
t4=t27;
t5=t28;
goto loop;}
else{
t23=t1;{
C_word av2[2];
av2[0]=t23;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}}
else{
t23=t1;{
C_word av2[2];
av2[0]=t23;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}}}}}}}
else{
if(C_truep(t4)){
/* library.scm:2637: ##sys#string->compnum */
f_15650(t1,t4,((C_word*)t0)[2],t2,t3);}
else{
/* library.scm:2637: ##sys#string->compnum */
f_15650(t1,((C_word*)t0)[4],((C_word*)t0)[2],t2,t3);}}}

/* ##sys#string->number in k10477 in k8569 */
static void C_ccall f_17325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17325,c,av);}
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?C_fix(10):C_get_rest_arg(c,3,av,3,t0));
t5=C_rest_nullp(c,3);
t6=C_rest_nullp(c,4);
t7=(C_truep(t6)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,3,t0));
if(C_truep(C_rest_nullp(c,4))){
/* library.scm:2640: ##sys#string->compnum */
f_15650(t1,t4,t2,C_fix(0),t7);}
else{
/* library.scm:2640: ##sys#string->compnum */
f_15650(t1,t4,t2,C_fix(0),t7);}}

/* chicken.base#equal=? in k10477 in k8569 */
static void C_ccall f_17371(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(12,c,5)))){
C_save_and_reclaim((void *)f_17371,c,av);}
a=C_alloc(12);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17374,a[2]=t7,a[3]=((C_word)li465),tmp=(C_word)a,a+=4,tmp));
t9=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17442,a[2]=t5,a[3]=((C_word)li466),tmp=(C_word)a,a+=4,tmp));
/* library.scm:2683: walk */
t10=((C_word*)t7)[1];
f_17442(t10,t1,t2,t3);}

/* compare-slots in chicken.base#equal=? in k10477 in k8569 */
static void C_fcall f_17374(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_17374,5,t0,t1,t2,t3,t4);}
a=C_alloc(10);
t5=C_block_size(t2);
t6=C_block_size(t3);
t7=C_eqp(t5,t6);
if(C_truep(t7)){
t8=C_fixnum_less_or_equal_p(t5,t4);
if(C_truep(t8)){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_fixnum_difference(t5,C_fix(1));
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17401,a[2]=t9,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t3,a[6]=t11,a[7]=((C_word)li464),tmp=(C_word)a,a+=8,tmp));
t13=((C_word*)t11)[1];
f_17401(t13,t1,t4);}}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}

/* loop in compare-slots in chicken.base#equal=? in k10477 in k8569 */
static void C_fcall f_17401(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_17401,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
/* library.scm:2657: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_17442(t4,t1,C_slot(((C_word*)t0)[4],t2),C_slot(((C_word*)t0)[5],t2));}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17425,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2658: walk */
t5=((C_word*)((C_word*)t0)[3])[1];
f_17442(t5,t4,C_slot(((C_word*)t0)[4],t2),C_slot(((C_word*)t0)[5],t2));}}

/* k17423 in loop in compare-slots in chicken.base#equal=? in k10477 in k8569 */
static void C_ccall f_17425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17425,c,av);}
if(C_truep(t1)){
/* library.scm:2659: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17401(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* walk in chicken.base#equal=? in k10477 in k8569 */
static void C_fcall f_17442(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_17442,4,t0,t1,t2,t3);}
t4=C_eqp(t2,t3);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_numberp(t2))){
t5=C_i_numberp(t3);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?C_i_nequalp(t2,t3):C_eqp(t2,t3));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_i_not(C_blockp(t2)))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_not(C_blockp(t3)))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_not(C_sametypep(t2,t3)))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_specialp(t2))){
if(C_truep(C_specialp(t3))){
if(C_truep(C_closurep(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=shallow_equal(t2,t3);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* library.scm:2673: compare-slots */
t5=((C_word*)((C_word*)t0)[2])[1];
f_17374(t5,t1,t2,t3,C_fix(1));}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
if(C_truep(C_byteblockp(t2))){
if(C_truep(C_byteblockp(t3))){
t5=C_block_size(t2);
t6=C_block_size(t3);
t7=C_eqp(t5,t6);
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=(C_truep(t7)?C_substring_compare(t2,t3,C_fix(0),C_fix(0),t5):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=C_block_size(t2);
t6=C_block_size(t3);
t7=C_eqp(t5,t6);
if(C_truep(t7)){
/* library.scm:2682: compare-slots */
t8=((C_word*)((C_word*)t0)[2])[1];
f_17374(t8,t1,t2,t3,C_fix(0));}
else{
t8=t1;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}}}}}}}

/* ##sys#interned-symbol? in k10477 in k8569 */
static void C_ccall f_17543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17543,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_lookup_symbol(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#string->symbol in k10477 in k8569 */
static void C_ccall f_17546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17546,c,av);}
t3=C_i_check_string(t2);
/* library.scm:2695: ##sys#intern-symbol */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
C_string_to_symbol(3,av2);}}

/* ##sys#symbol->string in k10477 in k8569 */
static void C_ccall f_17555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17555,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_slot(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#symbol->string in k10477 in k8569 */
static void C_ccall f_17561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17561,c,av);}
a=C_alloc(3);
t3=C_i_check_symbol_2(t2,lf[635]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17572,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2703: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}

/* k17570 in scheme#symbol->string in k10477 in k8569 */
static void C_ccall f_17572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17572,c,av);}
/* library.scm:2703: scheme#string-copy */
t2=*((C_word*)lf[395]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* scheme#string->symbol in k10477 in k8569 */
static void C_ccall f_17574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17574,c,av);}
a=C_alloc(3);
t3=C_i_check_string_2(t2,lf[637]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17585,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2709: string-copy */
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k17583 in scheme#string->symbol in k10477 in k8569 */
static void C_ccall f_17585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17585,c,av);}
/* library.scm:2709: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
C_string_to_symbol(3,av2);}}

/* chicken.base#string->uninterned-symbol in k10477 in k8569 */
static void C_ccall f_17587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_17587,c,av);}
a=C_alloc(7);
t3=C_i_check_string_2(t2,lf[639]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_make_symbol,a[2]=((C_word)li475),tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17598,a[2]=t4,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2715: string-copy */
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k17596 in chicken.base#string->uninterned-symbol in k10477 in k8569 */
static void C_ccall f_17598(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17598,c,av);}
/* library.scm:2713: g3362 */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* chicken.base#gensym in k10477 in k8569 */
static void C_ccall f_17600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +17,c,5)))){
C_save_and_reclaim((void*)f_17600,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+17);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
t3=C_fixnum_plus(((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t4=C_set_block_item(((C_word*)t0)[2],0,t3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_make_symbol,a[2]=((C_word)li477),tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17617,a[2]=t5,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17621,a[2]=t6,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t8=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f35032,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2732: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t9;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
tp(3,av2);}}
else{
t9=C_get_rest_arg(c,2,av,2,t0);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17634,a[2]=t6,a[3]=((C_word*)t0)[2],a[4]=t7,a[5]=t9,tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_blockp(t9))){
if(C_truep(C_stringp(t9))){
t11=t10;{
C_word *av2=av;
av2[0]=t11;
av2[1]=t9;
f_17634(2,av2);}}
else{
if(C_truep(C_symbolp(t9))){
/* library.scm:2729: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t10;
av2[2]=t9;
tp(3,av2);}}
else{
/* library.scm:2720: ##sys#signal-hook */
t11=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=lf[260];
av2[3]=lf[642];
av2[4]=lf[643];
av2[5]=t9;
((C_proc)(void*)(*((C_word*)t11+1)))(6,av2);}}}}
else{
t11=t10;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_FALSE;
f_17634(2,av2);}}}}

/* k17615 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f_17617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17617,c,av);}
/* library.scm:2720: g3369 */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* k17619 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f_17621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17621,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17625,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2732: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
tp(3,av2);}}

/* k17623 in k17619 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f_17625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17625,c,av);}
/* library.scm:2723: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k17632 in chicken.base#gensym in k10477 in k8569 */
static void C_ccall f_17634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_17634,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f35036,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2732: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
tp(3,av2);}}
else{
/* library.scm:2720: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[260];
av2[3]=lf[642];
av2[4]=lf[643];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* chicken.base#symbol-append in k10477 in k8569 */
static void C_ccall f_17654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +20,c,3)))){
C_save_and_reclaim((void*)f_17654,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+20);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17662,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t5=t4;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)t6)[1];
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17677,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17679,a[2]=t6,a[3]=t10,a[4]=t7,a[5]=((C_word)li479),tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_17679(t12,t8,t2);}

/* k17660 in chicken.base#symbol-append in k10477 in k8569 */
static void C_ccall f_17662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17662,c,av);}
/* library.scm:2737: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
C_string_to_symbol(3,av2);}}

/* k17675 in chicken.base#symbol-append in k10477 in k8569 */
static void C_ccall f_17677(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17677,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop3385 in chicken.base#symbol-append in k10477 in k8569 */
static void C_fcall f_17679(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_17679,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17704,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=C_i_check_symbol_2(t4,lf[645]);
/* library.scm:2742: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t3;
av2[2]=t4;
tp(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17702 in map-loop3385 in chicken.base#symbol-append in k10477 in k8569 */
static void C_ccall f_17704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_17704,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_17679(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.keyword#keyword? in k10477 in k8569 */
static void C_ccall f_17713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17713,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_keywordp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.keyword#string->keyword in k10477 in k8569 */
static void C_ccall f_17716(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17716,c,av);}
t3=C_i_check_string_2(t2,lf[649]);
/* library.scm:2759: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}

/* chicken.keyword#keyword->string in k10477 in k8569 */
static void C_ccall f_17725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17725,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17732,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2764: keyword? */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17730 in chicken.keyword#keyword->string in k10477 in k8569 */
static void C_ccall f_17732(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17732,c,av);}
if(C_truep(t1)){
/* library.scm:2765: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
/* library.scm:2766: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[260];
av2[3]=lf[651];
av2[4]=lf[652];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* chicken.keyword#get-keyword in k10477 in k8569 */
static void C_ccall f_17743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17743,c,av);}
a=C_alloc(7);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17750,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t5,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:2771: ##sys#check-keyword */
t7=*((C_word*)lf[353]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=lf[655];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k17748 in chicken.keyword#get-keyword in k10477 in k8569 */
static void C_ccall f_17750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17750,c,av);}
t2=C_i_check_list_2(((C_word*)t0)[2],lf[655]);
t3=C_i_get_keyword(((C_word*)t0)[3],((C_word*)t0)[2],((C_word*)t0)[4]);
t4=C_eqp(t3,((C_word*)t0)[4]);
if(C_truep(t4)){
if(C_truep(((C_word*)t0)[5])){
/* library.scm:2775: thunk */
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=((C_word*)t0)[6];
((C_proc)C_fast_retrieve_proc(t5))(2,av2);}}
else{
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* ##sys#make-blob in k10477 in k8569 */
static void C_ccall f_17774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_17774,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17778,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2791: ##sys#allocate-vector */
t4=*((C_word*)lf[318]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k17776 in ##sys#make-blob in k10477 in k8569 */
static void C_ccall f_17778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17778,c,av);}
t2=C_string_to_bytevector(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.blob#make-blob in k10477 in k8569 */
static void C_ccall f_17780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17780,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17784,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2796: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[660];
tp(4,av2);}}

/* k17782 in chicken.blob#make-blob in k10477 in k8569 */
static void C_ccall f_17784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17784,c,av);}
/* library.scm:2797: ##sys#make-blob */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[658]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[658]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* chicken.blob#blob? in k10477 in k8569 */
static void C_ccall f_17789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17789,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_blockp(t2))?C_bytevectorp(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.blob#blob-size in k10477 in k8569 */
static void C_ccall f_17795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17795,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17799,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2804: ##sys#check-blob */
t4=*((C_word*)lf[344]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[663];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k17797 in chicken.blob#blob-size in k10477 in k8569 */
static void C_ccall f_17799(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17799,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_block_size(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.blob#string->blob in k10477 in k8569 */
static void C_ccall f_17804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17804,c,av);}
a=C_alloc(5);
t3=C_i_check_string_2(t2,lf[665]);
t4=C_block_size(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17814,a[2]=t2,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2810: ##sys#make-blob */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[658]);
C_word *av2=av;
av2[0]=*((C_word*)lf[658]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k17812 in chicken.blob#string->blob in k10477 in k8569 */
static void C_ccall f_17814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17814,c,av);}
t2=C_copy_memory(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.blob#blob->string in k10477 in k8569 */
static void C_ccall f_17816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17816,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17820,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2815: ##sys#check-blob */
t4=*((C_word*)lf[344]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[667];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k17818 in chicken.blob#blob->string in k10477 in k8569 */
static void C_ccall f_17820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17820,c,av);}
a=C_alloc(5);
t2=C_block_size(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17826,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2817: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k17824 in k17818 in chicken.blob#blob->string in k10477 in k8569 */
static void C_ccall f_17826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17826,c,av);}
t2=C_copy_memory(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.blob#blob=? in k10477 in k8569 */
static void C_ccall f_17828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17828,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17832,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2822: ##sys#check-blob */
t5=*((C_word*)lf[344]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[669];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k17830 in chicken.blob#blob=? in k10477 in k8569 */
static void C_ccall f_17832(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17832,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17835,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2823: ##sys#check-blob */
t3=*((C_word*)lf[344]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[669];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k17833 in k17830 in chicken.blob#blob=? in k10477 in k8569 */
static void C_ccall f_17835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17835,c,av);}
t2=C_block_size(((C_word*)t0)[2]);
t3=C_block_size(((C_word*)t0)[3]);
t4=C_eqp(t3,t2);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_zerop(C_string_compare(((C_word*)t0)[2],((C_word*)t0)[3],t2)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#make-vector in k10477 in k8569 */
static void C_ccall f_17851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_17851,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+5);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17855,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2835: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[670];
tp(4,av2);}}

/* k17853 in scheme#make-vector in k10477 in k8569 */
static void C_ccall f_17855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17855,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17858,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnum_lessp(((C_word*)t0)[4],C_fix(0)))){
/* library.scm:2836: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[670];
av2[3]=lf[671];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_17858(2,av2);}}}

/* k17856 in k17853 in scheme#make-vector in k10477 in k8569 */
static void C_ccall f_17858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17858,c,av);}
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
t2=C_SCHEME_UNDEFINED;
/* library.scm:2837: ##sys#allocate-vector */
t3=*((C_word*)lf[318]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
/* library.scm:2837: ##sys#allocate-vector */
t2=*((C_word*)lf[318]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_i_car(((C_word*)t0)[2]);
av2[5]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* scheme#list->vector in k10477 in k8569 */
static void C_ccall f_17880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17880,c,av);}
a=C_alloc(4);
t3=C_i_listp(t2);
if(C_truep(C_i_not(t3))){
/* library.scm:2849: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[673];
tp(4,av2);}}
else{
t4=C_i_length(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17896,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2851: ##sys#make-vector */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[672]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[672]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}}

/* k17894 in scheme#list->vector in k10477 in k8569 */
static void C_ccall f_17896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17896,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17901,a[2]=t1,a[3]=((C_word)li493),tmp=(C_word)a,a+=4,tmp);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_17901(t2,((C_word*)t0)[3],C_fix(0))
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k17894 in scheme#list->vector in k10477 in k8569 */
static C_word C_fcall f_17901(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t1))){
return(((C_word*)t0)[2]);}
else{
t3=C_slot(t1,C_fix(0));
t4=C_i_setslot(((C_word*)t0)[2],t2,t3);
t6=C_slot(t1,C_fix(1));
t7=C_fixnum_plus(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}

/* scheme#vector->list in k10477 in k8569 */
static void C_ccall f_17932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17932,c,av);}
a=C_alloc(8);
t3=C_i_check_vector_2(t2,lf[674]);
t4=C_block_size(t2);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17941,a[2]=t4,a[3]=t2,a[4]=t6,a[5]=((C_word)li495),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_17941(t8,t1,C_fix(0));}

/* loop in scheme#vector->list in k10477 in k8569 */
static void C_fcall f_17941(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_17941,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_slot(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17959,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:2868: loop */
t6=t4;
t7=C_fixnum_plus(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}

/* k17957 in loop in scheme#vector->list in k10477 in k8569 */
static void C_ccall f_17959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_17959,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#vector in k10477 in k8569 */
static void C_ccall f_17965(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,2)))){
C_save_and_reclaim((void*)f_17965,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
/* library.scm:2870: scheme#list->vector */
t3=*((C_word*)lf[94]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* scheme#vector-fill! in k10477 in k8569 */
static void C_ccall f_17971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_17971,c,av);}
a=C_alloc(6);
t4=C_i_check_vector_2(t2,lf[675]);
t5=C_block_size(t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17980,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=((C_word)li498),tmp=(C_word)a,a+=6,tmp);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(
  f_17980(t6,C_fix(0))
);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* doloop3523 in scheme#vector-fill! in k10477 in k8569 */
static C_word C_fcall f_17980(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t1,((C_word*)t0)[2]))){
t2=C_SCHEME_UNDEFINED;
return(t2);}
else{
t2=C_i_setslot(((C_word*)t0)[3],t1,((C_word*)t0)[4]);
t4=C_fixnum_plus(t1,C_fix(1));
t1=t4;
goto loop;}}

/* chicken.base#vector-copy! in k10477 in k8569 */
static void C_ccall f_17999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +8,c,3)))){
C_save_and_reclaim((void*)f_17999,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+8);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
t5=C_i_check_vector_2(t2,lf[676]);
t6=C_i_check_vector_2(t3,lf[676]);
t7=C_block_size(t2);
t8=C_block_size(t3);
t9=C_i_pairp(t4);
t10=(C_truep(t9)?C_get_rest_arg(c,4,av,4,t0):C_i_fixnum_min(t8,t7));
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18018,a[2]=t10,a[3]=t2,a[4]=t3,a[5]=t1,a[6]=t8,a[7]=t7,tmp=(C_word)a,a+=8,tmp);
/* library.scm:2887: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t11;
av2[2]=t10;
av2[3]=lf[676];
tp(4,av2);}}

/* k18016 in chicken.base#vector-copy! in k10477 in k8569 */
static void C_ccall f_18018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,7)))){
C_save_and_reclaim((void *)f_18018,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18021,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_fixnum_greaterp(((C_word*)t0)[2],((C_word*)t0)[6]);
t4=(C_truep(t3)?t3:C_fixnum_greaterp(((C_word*)t0)[2],((C_word*)t0)[7]));
if(C_truep(t4)){
/* library.scm:2889: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 8) {
  av2=av;
} else {
  av2=C_alloc(8);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[383];
av2[3]=lf[676];
av2[4]=lf[677];
av2[5]=((C_word*)t0)[3];
av2[6]=((C_word*)t0)[4];
av2[7]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(8,av2);}}
else{
t5=t2;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_18021(2,av2);}}}

/* k18019 in k18016 in chicken.base#vector-copy! in k10477 in k8569 */
static void C_ccall f_18021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18021,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18026,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word)li500),tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_18026(t2,C_fix(0))
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* doloop3535 in k18019 in k18016 in chicken.base#vector-copy! in k10477 in k8569 */
static C_word C_fcall f_18026(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t1,((C_word*)t0)[2]))){
t2=C_SCHEME_UNDEFINED;
return(t2);}
else{
t2=C_slot(((C_word*)t0)[3],t1);
t3=C_i_setslot(((C_word*)t0)[4],t1,t2);
t5=C_fixnum_plus(t1,C_fix(1));
t1=t5;
goto loop;}}

/* chicken.base#subvector in k10477 in k8569 */
static void C_ccall f_18068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_18068,c,av);}
a=C_alloc(8);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=C_i_check_vector_2(t2,lf[678]);
t7=C_block_size(t2);
t8=(C_truep(t5)?t5:t7);
t9=C_fixnum_difference(t8,t3);
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18087,a[2]=t9,a[3]=t3,a[4]=t2,a[5]=t1,a[6]=t8,a[7]=t7,tmp=(C_word)a,a+=8,tmp);
/* library.scm:2902: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word av2[6];
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t10;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_fixnum_plus(t7,C_fix(1));
av2[5]=lf[678];
tp(6,av2);}}

/* k18085 in chicken.base#subvector in k10477 in k8569 */
static void C_ccall f_18087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_18087,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18090,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:2903: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=C_fix(0);
av2[4]=C_fixnum_plus(((C_word*)t0)[7],C_fix(1));
av2[5]=lf[678];
tp(6,av2);}}

/* k18088 in k18085 in chicken.base#subvector in k10477 in k8569 */
static void C_ccall f_18090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18090,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18093,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:2904: scheme#make-vector */
t3=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k18091 in k18088 in k18085 in chicken.base#subvector in k10477 in k8569 */
static void C_ccall f_18093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_18093,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18098,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word)li502),tmp=(C_word)a,a+=7,tmp);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_18098(t2,C_fix(0))
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* doloop3566 in k18091 in k18088 in k18085 in chicken.base#subvector in k10477 in k8569 */
static C_word C_fcall f_18098(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t1,((C_word*)t0)[2]))){
return(((C_word*)t0)[3]);}
else{
t2=C_fixnum_plus(t1,((C_word*)t0)[4]);
t3=C_slot(((C_word*)t0)[5],t2);
t4=C_i_setslot(((C_word*)t0)[3],t1,t3);
t6=C_fixnum_plus(t1,C_fix(1));
t1=t6;
goto loop;}}

/* chicken.base#vector-resize in k10477 in k8569 */
static void C_ccall f_18139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18139,c,av);}
a=C_alloc(6);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=C_i_check_vector_2(t2,lf[679]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18149,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* library.scm:2912: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t7;
av2[2]=t3;
av2[3]=lf[679];
tp(4,av2);}}

/* k18147 in chicken.base#vector-resize in k10477 in k8569 */
static void C_ccall f_18149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18149,c,av);}
/* library.scm:2913: ##sys#vector-resize */
f_18160(((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* ##sys#vector-resize in k10477 in k8569 */
static void C_fcall f_18160(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18160,4,t1,t2,t3,t4);}
a=C_alloc(5);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18164,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:2916: ##sys#make-vector */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[672]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[672]+1);
av2[1]=t5;
av2[2]=t3;
av2[3]=t4;
tp(4,av2);}}

/* k18162 in ##sys#vector-resize in k10477 in k8569 */
static void C_ccall f_18164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18164,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18167,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:2917: scheme#min */
t3=*((C_word*)lf[560]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_block_size(((C_word*)t0)[2]);
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18165 in k18162 in ##sys#vector-resize in k10477 in k8569 */
static void C_ccall f_18167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18167,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18172,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li505),tmp=(C_word)a,a+=6,tmp);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
  f_18172(t2,C_fix(0))
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* doloop3594 in k18165 in k18162 in ##sys#vector-resize in k10477 in k8569 */
static C_word C_fcall f_18172(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
if(C_truep(C_fixnum_greater_or_equal_p(t1,((C_word*)t0)[2]))){
return(((C_word*)t0)[3]);}
else{
t2=C_slot(((C_word*)t0)[4],t1);
t3=C_i_setslot(((C_word*)t0)[3],t1,t2);
t5=C_fixnum_plus(t1,C_fix(1));
t1=t5;
goto loop;}}

/* scheme#char-ci=? in k10477 in k8569 */
static void C_ccall f_18199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18199,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_eqp(C_u_i_char_downcase(t2),C_u_i_char_downcase(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-ci>? in k10477 in k8569 */
static void C_ccall f_18213(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18213,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_greaterp(C_u_i_char_downcase(t2),C_u_i_char_downcase(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-ci<? in k10477 in k8569 */
static void C_ccall f_18224(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18224,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_lessp(C_u_i_char_downcase(t2),C_u_i_char_downcase(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-ci>=? in k10477 in k8569 */
static void C_ccall f_18235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18235,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_greater_or_equal_p(C_u_i_char_downcase(t2),C_u_i_char_downcase(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-ci<=? in k10477 in k8569 */
static void C_ccall f_18246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18246,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_less_or_equal_p(C_u_i_char_downcase(t2),C_u_i_char_downcase(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k18257 in k10477 in k8569 */
static void C_ccall f_18259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_18259,c,av);}
a=C_alloc(15);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18261,a[2]=t1,a[3]=((C_word)li513),tmp=(C_word)a,a+=4,tmp);
t5=C_mutate((C_word*)lf[199]+1 /* (set! chicken.base#char-name ...) */,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18302,a[2]=t4,a[3]=t3,a[4]=t1,a[5]=((C_word)li514),tmp=(C_word)a,a+=6,tmp));
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18419,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2986: chicken.base#char-name */
t7=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[1404];
av2[3]=C_make_character(32);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* lookup-char in k18257 in k10477 in k8569 */
static void C_fcall f_18261(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_18261,3,t0,t1,t2);}
a=C_alloc(4);
t3=C_fix(C_character_code(t2));
t4=C_fixnum_modulo(t3,C_fix(37));
t5=C_slot(((C_word*)t0)[2],t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18274,a[2]=t2,a[3]=((C_word)li512),tmp=(C_word)a,a+=4,tmp);
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=(
  f_18274(t6,t5)
);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* loop in lookup-char in k18257 in k10477 in k8569 */
static C_word C_fcall f_18274(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_slot(t1,C_fix(0));
t3=C_slot(t2,C_fix(0));
t4=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t4)){
return(t2);}
else{
t6=C_slot(t1,C_fix(1));
t1=t6;
goto loop;}}
else{
return(C_SCHEME_FALSE);}}

/* chicken.base#char-name in k18257 in k10477 in k8569 */
static void C_ccall f_18302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +8,c,5)))){
C_save_and_reclaim((void*)f_18302,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+8);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
t4=C_i_pairp(t3);
t5=(C_truep(t4)?C_get_rest_arg(c,3,av,3,t0):C_SCHEME_FALSE);
if(C_truep(C_charp(t2))){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18315,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:2960: lookup-char */
t7=((C_word*)t0)[2];
f_18261(t7,t6,t2);}
else{
if(C_truep(t5)){
t6=C_i_check_symbol_2(t2,lf[681]);
t7=C_i_check_char_2(t5,lf[681]);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18333,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t5,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
t9=C_slot(t2,C_fix(1));
t10=C_block_size(t9);
if(C_truep(C_fixnum_lessp(t10,C_fix(2)))){
/* library.scm:2966: ##sys#signal-hook */
t11=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t11;
av2[1]=t8;
av2[2]=lf[260];
av2[3]=lf[681];
av2[4]=lf[682];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t11+1)))(6,av2);}}
else{
t11=t8;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_SCHEME_UNDEFINED;
f_18333(2,av2);}}}
else{
t6=C_i_check_symbol_2(t2,lf[681]);
t7=C_i_assq(t2,((C_word*)((C_word*)t0)[3])[1]);
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=(C_truep(t7)?C_slot(t7,C_fix(1)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}}}

/* k18313 in chicken.base#char-name in k18257 in k10477 in k8569 */
static void C_ccall f_18315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18315,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_slot(t1,C_fix(1)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k18331 in chicken.base#char-name in k18257 in k10477 in k8569 */
static void C_ccall f_18333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_18333,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18336,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:2967: lookup-char */
t3=((C_word*)t0)[7];
f_18261(t3,t2,((C_word*)t0)[5]);}

/* k18334 in k18331 in chicken.base#char-name in k18257 in k10477 in k8569 */
static void C_ccall f_18336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_18336,c,av);}
a=C_alloc(12);
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=C_i_setslot(t1,C_fix(1),((C_word*)t0)[2]);
if(C_truep(t2)){
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_i_slot(t2,C_fix(1),((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[5]);
t5=C_a_i_cons(&a,2,t4,((C_word*)((C_word*)t0)[3])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t5);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}
else{
t2=C_fix(C_character_code(((C_word*)t0)[5]));
t3=C_fixnum_modulo(t2,C_fix(37));
t4=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[5]);
t5=C_a_i_cons(&a,2,t4,((C_word*)((C_word*)t0)[3])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t5);
t7=C_a_i_cons(&a,2,((C_word*)t0)[5],((C_word*)t0)[2]);
t8=C_slot(((C_word*)t0)[6],t3);
t9=C_a_i_cons(&a,2,t7,t8);
t10=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_i_setslot(((C_word*)t0)[6],t3,t9);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}

/* k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18419,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18422,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2987: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1403];
av2[3]=C_make_character(9);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18422,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18425,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2988: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1402];
av2[3]=C_make_character(10);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18425,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18428,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2989: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1401];
av2[3]=C_make_character(10);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18428,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18431,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2990: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1400];
av2[3]=C_make_character(11);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18431,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18434,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2991: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1399];
av2[3]=C_make_character(127);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18434,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18437,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2992: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1398];
av2[3]=C_make_character(27);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18437,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18440,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2993: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1397];
av2[3]=C_make_character(27);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18440,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18443,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2994: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1396];
av2[3]=C_make_character(7);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18443,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18446,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2995: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1395];
av2[3]=C_make_character(0);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18446,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18449,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2996: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1394];
av2[3]=C_make_character(0);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18449(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18449,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18452,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2997: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1393];
av2[3]=C_make_character(13);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18452,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18455,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2998: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1392];
av2[3]=C_make_character(12);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18455,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18458,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:2999: chicken.base#char-name */
t3=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1391];
av2[3]=C_make_character(8);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(93,c,6)))){
C_save_and_reclaim((void *)f_18458,c,av);}
a=C_alloc(93);
t2=C_mutate((C_word*)lf[683]+1 /* (set! ##sys#call-with-current-continuation ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_call_cc,a[2]=((C_word)li515),tmp=(C_word)a,a+=3,tmp));
t3=C_mutate((C_word*)lf[684]+1 /* (set! ##sys#call-with-cthulhu ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_call_with_cthulhu,a[2]=((C_word)li516),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[685]+1 /* (set! ##sys#call-with-values ...) */,*((C_word*)lf[131]+1));
t5=C_mutate((C_word*)lf[686]+1 /* (set! ##sys#for-each ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18463,a[2]=((C_word)li518),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[688]+1 /* (set! ##sys#map ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18501,a[2]=((C_word)li520),tmp=(C_word)a,a+=3,tmp));
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18543,a[2]=((C_word)li523),tmp=(C_word)a,a+=3,tmp);
t10=C_set_block_item(t8,0,t9);
t11=C_mutate((C_word*)lf[170]+1 /* (set! scheme#for-each ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18601,a[2]=t8,a[3]=((C_word)li528),tmp=(C_word)a,a+=4,tmp));
t12=C_mutate((C_word*)lf[171]+1 /* (set! scheme#map ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18683,a[2]=t8,a[3]=((C_word)li533),tmp=(C_word)a,a+=4,tmp));
t13=C_set_block_item(lf[690] /* ##sys#dynamic-winds */,0,C_SCHEME_END_OF_LIST);
t14=C_mutate((C_word*)lf[172]+1 /* (set! scheme#dynamic-wind ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18785,a[2]=((C_word)li535),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[691]+1 /* (set! ##sys#dynamic-wind ...) */,*((C_word*)lf[172]+1));
t16=C_mutate((C_word*)lf[132]+1 /* (set! scheme#call-with-current-continuation ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18816,a[2]=((C_word)li538),tmp=(C_word)a,a+=3,tmp));
t17=C_mutate((C_word*)lf[198]+1 /* (set! chicken.base#call/cc ...) */,*((C_word*)lf[132]+1));
t18=C_mutate(&lf[692] /* (set! ##sys#dynamic-unwind ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18856,a[2]=((C_word)li539),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[209]+1 /* (set! chicken.base#port? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18914,a[2]=((C_word)li540),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[211]+1 /* (set! chicken.base#input-port-open? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18920,a[2]=((C_word)li541),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[212]+1 /* (set! chicken.base#output-port-open? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18925,a[2]=((C_word)li542),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[210]+1 /* (set! chicken.base#port-closed? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18930,a[2]=((C_word)li543),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[697]+1 /* (set! ##sys#make-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18943,a[2]=((C_word)li544),tmp=(C_word)a,a+=3,tmp));
t24=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32160,a[2]=((C_word)li546),tmp=(C_word)a,a+=3,tmp);
t25=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32200,a[2]=((C_word)li548),tmp=(C_word)a,a+=3,tmp);
t26=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32240,a[2]=((C_word)li549),tmp=(C_word)a,a+=3,tmp);
t27=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32243,a[2]=((C_word)li550),tmp=(C_word)a,a+=3,tmp);
t28=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32246,a[2]=((C_word)li551),tmp=(C_word)a,a+=3,tmp);
t29=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32252,a[2]=((C_word)li552),tmp=(C_word)a,a+=3,tmp);
t30=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32255,a[2]=((C_word)li553),tmp=(C_word)a,a+=3,tmp);
t31=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32258,a[2]=((C_word)li556),tmp=(C_word)a,a+=3,tmp);
t32=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32357,a[2]=((C_word)li559),tmp=(C_word)a,a+=3,tmp);
t33=C_a_i_vector(&a,10,t24,t25,t26,t27,t28,t29,t30,t31,t32,C_SCHEME_FALSE);
t34=C_mutate((C_word*)lf[711]+1 /* (set! ##sys#stream-port-class ...) */,t33);
t35=C_mutate((C_word*)lf[712]+1 /* (set! ##sys#open-file-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_open_file_port,a[2]=((C_word)li560),tmp=(C_word)a,a+=3,tmp));
t36=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18974,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3277: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t36;
av2[2]=C_fix(1);
av2[3]=C_fast_retrieve(lf[711]);
av2[4]=lf[1390];
av2[5]=lf[730];
tp(6,av2);}}

/* ##sys#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18463(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_18463,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18469,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=((C_word)li517),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_18469(t7,t1,t3);}

/* loop in ##sys#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18469(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18469,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18485,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3012: p */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}
else{
/* library.scm:3014: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[687];
tp(4,av2);}}}}

/* k18483 in loop in ##sys#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18485,c,av);}
/* library.scm:3013: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18469(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* ##sys#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_18501,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18507,a[2]=t5,a[3]=t2,a[4]=t3,a[5]=((C_word)li519),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_18507(t7,t1,t3);}

/* loop in ##sys#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18507(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18507,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18527,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3020: p */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}
else{
/* library.scm:3021: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[689];
tp(4,av2);}}}}

/* k18525 in loop in ##sys#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18527(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_18527,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18531,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3020: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_18507(t3,t2,C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k18529 in k18525 in loop in ##sys#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18531,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* mapsafe3670 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18543,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18549,a[2]=t2,a[3]=t4,a[4]=t3,a[5]=((C_word)li522),tmp=(C_word)a,a+=6,tmp);
/* library.scm:3025: scheme#call-with-current-continuation */
t6=*((C_word*)lf[132]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* a18548 in mapsafe3670 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_18549,c,av);}
a=C_alloc(9);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18555,a[2]=t2,a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word)li521),tmp=(C_word)a,a+=7,tmp));
t6=((C_word*)t4)[1];
f_18555(t6,t1,((C_word*)t0)[4]);}

/* lp in a18548 in mapsafe3670 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18555(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18555,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=C_eqp(t4,C_SCHEME_END_OF_LIST);
if(C_truep(t5)){
/* library.scm:3031: empty */
t6=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t6;
av2[1]=t1;
av2[2]=C_SCHEME_END_OF_LIST;
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}
else{
if(C_truep(C_i_pairp(t4))){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18587,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3033: p */
t7=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)C_fast_retrieve_proc(t7))(3,av2);}}
else{
/* library.scm:3034: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}}}}

/* k18585 in lp in a18548 in mapsafe3670 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_18587,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18591,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3033: lp */
t3=((C_word*)((C_word*)t0)[3])[1];
f_18555(t3,t2,C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k18589 in k18585 in lp in a18548 in mapsafe3670 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18591,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +11,c,3)))){
C_save_and_reclaim((void*)f_18601,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+11);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
if(C_truep(C_rest_nullp(c,4))){
t5=C_i_check_list_2(t3,lf[687]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18616,a[2]=t7,a[3]=t2,a[4]=((C_word)li524),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_18616(t9,t1,t3);}
else{
t5=C_a_i_cons(&a,2,t3,t4);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18646,a[2]=t7,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word)li527),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_18646(t9,t1,t5);}}

/* for-each-loop3690 in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18616(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_18616,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18626,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3039: g3691 */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k18624 in for-each-loop3690 in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18626(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18626,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_18616(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18646(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_18646,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18653,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18677,a[2]=((C_word)li526),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3042: mapsafe */
t5=((C_word*)((C_word*)t0)[3])[1];{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
av2[3]=t2;
av2[4]=lf[687];
((C_proc)C_fast_retrieve_proc(t5))(5,av2);}}

/* k18651 in loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18653,c,av);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t1))){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18662,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=t1;
C_apply(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18660 in k18651 in loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_18662,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18669,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18671,a[2]=((C_word)li525),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3045: mapsafe */
t4=((C_word*)((C_word*)t0)[4])[1];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
av2[3]=((C_word*)t0)[5];
av2[4]=lf[687];
((C_proc)C_fast_retrieve_proc(t4))(5,av2);}}

/* k18667 in k18660 in k18651 in loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18669,c,av);}
/* library.scm:3045: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18646(t2,((C_word*)t0)[3],t1);}

/* a18670 in k18660 in k18651 in loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18671,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a18676 in loop in scheme#for-each in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18677(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18677,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +14,c,3)))){
C_save_and_reclaim((void*)f_18683,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+14);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
if(C_truep(C_rest_nullp(c,4))){
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_i_check_list_2(t3,lf[689]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18701,a[2]=t7,a[3]=t11,a[4]=t2,a[5]=t8,a[6]=((C_word)li529),tmp=(C_word)a,a+=7,tmp));
t13=((C_word*)t11)[1];
f_18701(t13,t1,t3);}
else{
t5=C_a_i_cons(&a,2,t3,t4);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18742,a[2]=t7,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word)li532),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_18742(t9,t1,t5);}}

/* map-loop3718 in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18701(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18701,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18726,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3050: g3724 */
t4=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18724 in map-loop3718 in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_18726,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_18701(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18742(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_18742,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18749,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18778,a[2]=((C_word)li531),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3053: mapsafe */
t5=((C_word*)((C_word*)t0)[3])[1];{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
av2[3]=t2;
av2[4]=lf[689];
((C_proc)C_fast_retrieve_proc(t5))(5,av2);}}

/* k18747 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18749,c,av);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t1))){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18762,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=t1;
C_apply(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k18760 in k18747 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_18762,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18766,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18770,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18772,a[2]=((C_word)li530),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3056: mapsafe */
t5=((C_word*)((C_word*)t0)[4])[1];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
av2[3]=((C_word*)t0)[5];
av2[4]=lf[689];
((C_proc)C_fast_retrieve_proc(t5))(5,av2);}}

/* k18764 in k18760 in k18747 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18766(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_18766,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k18768 in k18760 in k18747 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18770,c,av);}
/* library.scm:3056: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18742(t2,((C_word*)t0)[3],t1);}

/* a18771 in k18760 in k18747 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18772,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a18777 in loop in scheme#map in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18778(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18778,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#dynamic-wind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18785,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18789,a[2]=t2,a[3]=t4,a[4]=t1,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3076: before */
t6=t2;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* k18787 in scheme#dynamic-wind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_18789,c,av);}
a=C_alloc(10);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,C_fast_retrieve(lf[690]));
t4=C_mutate((C_word*)lf[690]+1 /* (set! ##sys#dynamic-winds ...) */,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18798,a[2]=((C_word*)t0)[3],a[3]=((C_word)li534),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3078: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t5;
C_call_with_values(4,av2);}}

/* a18797 in k18787 in scheme#dynamic-wind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_18798,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_slot(C_fast_retrieve(lf[690]),C_fix(1));
t4=C_mutate((C_word*)lf[690]+1 /* (set! ##sys#dynamic-winds ...) */,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18806,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3082: after */
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* k18804 in a18797 in k18787 in scheme#dynamic-wind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18806,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_apply_values(3,av2);}}

/* scheme#call-with-current-continuation in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18816,c,av);}
a=C_alloc(5);
t3=C_fast_retrieve(lf[690]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18822,a[2]=t3,a[3]=t2,a[4]=((C_word)li537),tmp=(C_word)a,a+=5,tmp);
/* library.scm:3090: ##sys#call-with-current-continuation */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
C_call_cc(3,av2);}}

/* a18821 in scheme#call-with-current-continuation in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_18822,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18825,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word)li536),tmp=(C_word)a,a+=5,tmp);
/* library.scm:3096: proc */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}

/* continuation in a18821 in scheme#call-with-current-continuation in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_18825,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18829,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_eqp(C_fast_retrieve(lf[690]),((C_word*)t0)[3]);
if(C_truep(t4)){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
C_apply(4,av2);}}
else{
t5=C_i_length(C_fast_retrieve(lf[690]));
t6=C_i_length(((C_word*)t0)[3]);
/* library.scm:3094: ##sys#dynamic-unwind */
f_18856(t3,((C_word*)t0)[3],C_fixnum_difference(t5,t6));}}

/* k18827 in continuation in a18821 in scheme#call-with-current-continuation in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18829(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18829,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
C_apply(4,av2);}}

/* ##sys#dynamic-unwind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_18856(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_18856,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_eqp(C_fast_retrieve(lf[690]),t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(t3,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18872,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3103: ##sys#dynamic-unwind */
t11=t5;
t12=C_slot(t2,C_fix(1));
t13=C_fixnum_plus(t3,C_fix(1));
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
t5=C_slot(C_fast_retrieve(lf[690]),C_fix(0));
t6=C_slot(t5,C_fix(1));
t7=C_slot(C_fast_retrieve(lf[690]),C_fix(1));
t8=C_mutate((C_word*)lf[690]+1 /* (set! ##sys#dynamic-winds ...) */,t7);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18901,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3109: after */
t10=t6;{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)C_fast_retrieve_proc(t10))(2,av2);}}}}

/* k18870 in ##sys#dynamic-unwind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_18872,c,av);}
a=C_alloc(4);
t2=C_slot(((C_word*)t0)[2],C_fix(0));
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18878,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3101: g3778 */
t5=t3;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)C_fast_retrieve_proc(t5))(2,av2);}}

/* k18876 in k18870 in ##sys#dynamic-unwind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18878,c,av);}
t2=C_mutate((C_word*)lf[690]+1 /* (set! ##sys#dynamic-winds ...) */,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k18899 in ##sys#dynamic-unwind in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_18901,c,av);}
/* library.scm:3110: ##sys#dynamic-unwind */
f_18856(((C_word*)t0)[2],((C_word*)t0)[3],C_fixnum_difference(((C_word*)t0)[4],C_fix(1)));}

/* chicken.base#port? in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18914,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_blockp(t2))?C_portp(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#input-port-open? in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18920,c,av);}
t3=C_i_check_port(t2,C_fix(1),lf[693]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_input_port_openp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.base#output-port-open? in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18925,c,av);}
t3=C_i_check_port(t2,C_fix(2),lf[694]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_output_port_openp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.base#port-closed? in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18930,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18934,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3132: ##sys#check-port */
t4=*((C_word*)lf[695]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[696];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k18932 in chicken.base#port-closed? in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18934,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(C_slot(((C_word*)t0)[3],C_fix(8)),C_fix(0));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#make-port in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18943(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(17,c,1)))){
C_save_and_reclaim((void *)f_18943,c,av);}
a=C_alloc(17);
t6=C_a_i_port(&a,0);
t7=C_i_set_i_slot(t6,C_fix(1),t2);
t8=C_i_setslot(t6,C_fix(2),t3);
t9=C_i_setslot(t6,C_fix(3),t4);
t10=C_i_set_i_slot(t6,C_fix(4),C_fix(1));
t11=C_i_set_i_slot(t6,C_fix(5),C_fix(0));
t12=C_i_setslot(t6,C_fix(7),t5);
t13=C_i_setslot(t6,C_fix(8),t2);
t14=t1;{
C_word *av2=av;
av2[0]=t14;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}

/* k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_18974,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[713]+1 /* (set! ##sys#standard-input ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18978,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3278: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t3;
av2[2]=C_fix(2);
av2[3]=C_fast_retrieve(lf[711]);
av2[4]=lf[1389];
av2[5]=lf[730];
tp(6,av2);}}

/* k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18978(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_18978,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[714]+1 /* (set! ##sys#standard-output ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18982,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3279: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t3;
av2[2]=C_fix(2);
av2[3]=C_fast_retrieve(lf[711]);
av2[4]=lf[1388];
av2[5]=lf[730];
tp(6,av2);}}

/* k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_18982,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[715]+1 /* (set! ##sys#standard-error ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18985,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3281: ##sys#open-file-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[712]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[712]+1);
av2[1]=t3;
av2[2]=*((C_word*)lf[713]+1);
av2[3]=C_fix(0);
av2[4]=C_SCHEME_FALSE;
tp(5,av2);}}

/* k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_18985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_18985,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18988,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3282: ##sys#open-file-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[712]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[712]+1);
av2[1]=t2;
av2[2]=*((C_word*)lf[714]+1);
av2[3]=C_fix(1);
av2[4]=C_SCHEME_FALSE;
tp(5,av2);}}

/* k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_18988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_18988,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18991,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3283: ##sys#open-file-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[712]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[712]+1);
av2[1]=t2;
av2[2]=*((C_word*)lf[715]+1);
av2[3]=C_fix(2);
av2[4]=C_SCHEME_FALSE;
tp(5,av2);}}

/* k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in ... */
static void C_ccall f_18991(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(119,c,6)))){
C_save_and_reclaim((void *)f_18991,c,av);}
a=C_alloc(119);
t2=C_mutate((C_word*)lf[716]+1 /* (set! ##sys#check-input-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18993,a[2]=((C_word)li561),tmp=(C_word)a,a+=3,tmp));
t3=C_mutate((C_word*)lf[717]+1 /* (set! ##sys#check-output-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19004,a[2]=((C_word)li562),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[695]+1 /* (set! ##sys#check-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19015,a[2]=((C_word)li563),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[718]+1 /* (set! ##sys#check-open-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19026,a[2]=((C_word)li564),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[135]+1 /* (set! scheme#current-input-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19037,a[2]=((C_word)li566),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[136]+1 /* (set! scheme#current-output-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19098,a[2]=((C_word)li568),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[192]+1 /* (set! chicken.base#current-error-port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19159,a[2]=((C_word)li570),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[722]+1 /* (set! ##sys#tty-port? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19220,a[2]=((C_word)li571),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[724]+1 /* (set! ##sys#port-data ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19234,a[2]=((C_word)li572),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[725]+1 /* (set! ##sys#set-port-data! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19240,a[2]=((C_word)li573),tmp=(C_word)a,a+=3,tmp));
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19246,a[2]=((C_word)li575),tmp=(C_word)a,a+=3,tmp));
t17=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19344,a[2]=((C_word)li576),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate((C_word*)lf[137]+1 /* (set! scheme#open-input-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19381,a[2]=t13,a[3]=((C_word)li577),tmp=(C_word)a,a+=4,tmp));
t19=C_mutate((C_word*)lf[138]+1 /* (set! scheme#open-output-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19387,a[2]=t13,a[3]=((C_word)li578),tmp=(C_word)a,a+=4,tmp));
t20=C_mutate((C_word*)lf[139]+1 /* (set! scheme#close-input-port ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19393,a[2]=t15,a[3]=((C_word)li579),tmp=(C_word)a,a+=4,tmp));
t21=C_mutate((C_word*)lf[140]+1 /* (set! scheme#close-output-port ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19399,a[2]=t15,a[3]=((C_word)li580),tmp=(C_word)a,a+=4,tmp));
t22=*((C_word*)lf[137]+1);
t23=C_mutate((C_word*)lf[141]+1 /* (set! scheme#call-with-input-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19405,a[2]=t22,a[3]=((C_word)li583),tmp=(C_word)a,a+=4,tmp));
t24=*((C_word*)lf[138]+1);
t25=C_mutate((C_word*)lf[142]+1 /* (set! scheme#call-with-output-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19429,a[2]=t24,a[3]=((C_word)li586),tmp=(C_word)a,a+=4,tmp));
t26=*((C_word*)lf[137]+1);
t27=C_mutate((C_word*)lf[143]+1 /* (set! scheme#with-input-from-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19453,a[2]=t26,a[3]=((C_word)li591),tmp=(C_word)a,a+=4,tmp));
t28=*((C_word*)lf[138]+1);
t29=C_mutate((C_word*)lf[144]+1 /* (set! scheme#with-output-to-file ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19487,a[2]=t28,a[3]=((C_word)li596),tmp=(C_word)a,a+=4,tmp));
t30=C_mutate((C_word*)lf[743]+1 /* (set! ##sys#file-exists? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19521,a[2]=((C_word)li597),tmp=(C_word)a,a+=3,tmp));
t31=C_mutate((C_word*)lf[746]+1 /* (set! ##sys#flush-output ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19547,a[2]=((C_word)li598),tmp=(C_word)a,a+=3,tmp));
t32=C_mutate((C_word*)lf[216]+1 /* (set! chicken.base#flush-output ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19560,a[2]=((C_word)li599),tmp=(C_word)a,a+=3,tmp));
t33=C_mutate((C_word*)lf[748]+1 /* (set! ##sys#port-line ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19578,a[2]=((C_word)li600),tmp=(C_word)a,a+=3,tmp));
t34=C_mutate((C_word*)lf[749]+1 /* (set! ##sys#decorate-lambda ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19587,a[2]=((C_word)li603),tmp=(C_word)a,a+=3,tmp));
t35=C_mutate((C_word*)lf[750]+1 /* (set! ##sys#lambda-decoration ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19662,a[2]=((C_word)li605),tmp=(C_word)a,a+=3,tmp));
t36=C_mutate((C_word*)lf[751]+1 /* (set! ##sys#make-lambda-info ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19701,a[2]=((C_word)li606),tmp=(C_word)a,a+=3,tmp));
t37=C_mutate((C_word*)lf[752]+1 /* (set! ##sys#lambda-info? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19710,a[2]=((C_word)li607),tmp=(C_word)a,a+=3,tmp));
t38=C_mutate((C_word*)lf[753]+1 /* (set! ##sys#lambda-info ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19723,a[2]=((C_word)li608),tmp=(C_word)a,a+=3,tmp));
t39=C_mutate((C_word*)lf[754]+1 /* (set! ##sys#lambda-info->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19729,a[2]=((C_word)li609),tmp=(C_word)a,a+=3,tmp));
t40=C_mutate((C_word*)lf[202]+1 /* (set! chicken.base#procedure-information ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19738,a[2]=((C_word)li610),tmp=(C_word)a,a+=3,tmp));
t41=C_a_i_vector1(&a,1,lf[757]);
t42=C_mutate(&lf[758] /* (set! setter-tag ...) */,t41);
t43=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19767,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t44=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32036,a[2]=((C_word)li1002),tmp=(C_word)a,a+=3,tmp);
t45=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32076,a[2]=((C_word)li1003),tmp=(C_word)a,a+=3,tmp);
t46=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32092,a[2]=((C_word)li1007),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3512: ##sys#decorate-lambda */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[749]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[749]+1);
av2[1]=t43;
av2[2]=t44;
av2[3]=t45;
av2[4]=t46;
tp(5,av2);}}

/* ##sys#check-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_18993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_18993,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+0);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=C_i_pairp(t4);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?C_i_check_port_2(t2,C_fix(1),t3,C_get_rest_arg(c,4,av,4,t0)):C_i_check_port(t2,C_fix(1),t3));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* ##sys#check-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_19004,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+0);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=C_i_pairp(t4);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=(C_truep(t5)?C_i_check_port_2(t2,C_fix(2),t3,C_get_rest_arg(c,4,av,4,t0)):C_i_check_port(t2,C_fix(2),t3));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* ##sys#check-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_19015,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_port_2(t2,C_fix(0),C_SCHEME_FALSE,C_get_rest_arg(c,3,av,3,t0)):C_i_check_port(t2,C_fix(0),C_SCHEME_FALSE));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#check-open-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_19026,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
t4=C_i_pairp(t3);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_check_port_2(t2,C_fix(0),C_SCHEME_TRUE,C_get_rest_arg(c,3,av,3,t0)):C_i_check_port(t2,C_fix(0),C_SCHEME_TRUE));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#current-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19037(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_19037,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t2))){
t3=*((C_word*)lf[713]+1);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=*((C_word*)lf[713]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19050,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3310: ##sys#check-port */
t5=*((C_word*)lf[695]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[719];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k19048 in scheme#current-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19050(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_19050,c,av);}
a=C_alloc(10);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19052,a[2]=((C_word*)t0)[3],a[3]=((C_word)li565),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19058,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19063,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_nullp(t2))){
t6=(
/* library.scm:3311: def-convert?3900 */
  f_19063(t5)
);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_u_i_cdr(t2);
if(C_truep(C_i_nullp(t7))){
t8=(
/* library.scm:3311: def-set?3901 */
  f_19058(t4)
);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_i_car(t7);
t9=C_u_i_cdr(t7);
t10=(
/* library.scm:3311: body3898 */
  f_19052(t3,t8)
);
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}

/* body3898 in k19048 in scheme#current-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19052(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
if(C_truep(t1)){
t2=C_mutate((C_word*)lf[713]+1 /* (set! ##sys#standard-input ...) */,((C_word*)t0)[2]);
return(t2);}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* def-set?3901 in k19048 in scheme#current-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19058(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3311: body3898 */
  f_19052(((C_word*)t0)[2],C_SCHEME_TRUE)
));}

/* def-convert?3900 in k19048 in scheme#current-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19063(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3311: def-set?3901 */
  f_19058(((C_word*)t0)[2])
));}

/* scheme#current-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19098(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_19098,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t2))){
t3=*((C_word*)lf[714]+1);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=*((C_word*)lf[714]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19111,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3320: ##sys#check-port */
t5=*((C_word*)lf[695]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[720];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k19109 in scheme#current-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_19111,c,av);}
a=C_alloc(10);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19113,a[2]=((C_word*)t0)[3],a[3]=((C_word)li567),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19119,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19124,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_nullp(t2))){
t6=(
/* library.scm:3321: def-convert?3925 */
  f_19124(t5)
);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_u_i_cdr(t2);
if(C_truep(C_i_nullp(t7))){
t8=(
/* library.scm:3321: def-set?3926 */
  f_19119(t4)
);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_i_car(t7);
t9=C_u_i_cdr(t7);
t10=(
/* library.scm:3321: body3923 */
  f_19113(t3,t8)
);
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}

/* body3923 in k19109 in scheme#current-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19113(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
if(C_truep(t1)){
t2=C_mutate((C_word*)lf[714]+1 /* (set! ##sys#standard-output ...) */,((C_word*)t0)[2]);
return(t2);}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* def-set?3926 in k19109 in scheme#current-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19119(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3321: body3923 */
  f_19113(((C_word*)t0)[2],C_SCHEME_TRUE)
));}

/* def-convert?3925 in k19109 in scheme#current-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19124(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3321: def-set?3926 */
  f_19119(((C_word*)t0)[2])
));}

/* chicken.base#current-error-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_19159,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t2))){
t3=*((C_word*)lf[715]+1);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19172,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3330: ##sys#check-port */
t5=*((C_word*)lf[695]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[721];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k19170 in chicken.base#current-error-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_19172,c,av);}
a=C_alloc(10);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19174,a[2]=((C_word*)t0)[3],a[3]=((C_word)li569),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19180,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19185,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_nullp(t2))){
t6=(
/* library.scm:3331: def-convert?3950 */
  f_19185(t5)
);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_u_i_cdr(t2);
if(C_truep(C_i_nullp(t7))){
t8=(
/* library.scm:3331: def-set?3951 */
  f_19180(t4)
);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_i_car(t7);
t9=C_u_i_cdr(t7);
t10=(
/* library.scm:3331: body3948 */
  f_19174(t3,t8)
);
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}

/* body3948 in k19170 in chicken.base#current-error-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19174(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
if(C_truep(t1)){
t2=C_mutate((C_word*)lf[715]+1 /* (set! ##sys#standard-error ...) */,((C_word*)t0)[2]);
return(t2);}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* def-set?3951 in k19170 in chicken.base#current-error-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19180(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3331: body3948 */
  f_19174(((C_word*)t0)[2],C_SCHEME_TRUE)
));}

/* def-convert?3950 in k19170 in chicken.base#current-error-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static C_word C_fcall f_19185(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:3331: def-set?3951 */
  f_19180(((C_word*)t0)[2])
));}

/* ##sys#tty-port? in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_19220,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19232,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3336: ##sys#peek-unsigned-integer */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[723]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[723]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=C_fix(0);
tp(4,av2);}}

/* k19230 in ##sys#tty-port? in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19232(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19232,c,av);}
t2=C_i_not(C_i_zerop(t1));
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_tty_portp(((C_word*)t0)[3]):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#port-data in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19234(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19234,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_slot(t2,C_fix(9));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#set-port-data! in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19240,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_setslot(t2,C_fix(9),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_fcall f_19246(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,0,3)))){
C_save_and_reclaim_args((void *)trf_19246,5,t1,t2,t3,t4,t5);}
a=C_alloc(22);
t6=C_i_check_string_2(t2,t5);
t7=(C_truep(t3)?lf[726]:lf[727]);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=lf[728];
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19256,a[2]=t1,a[3]=t5,a[4]=t2,a[5]=t9,a[6]=t11,a[7]=t3,tmp=(C_word)a,a+=8,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19289,a[2]=t14,a[3]=t11,a[4]=t3,a[5]=t5,a[6]=t9,a[7]=((C_word)li574),tmp=(C_word)a,a+=8,tmp));
t16=((C_word*)t14)[1];
f_19289(t16,t12,t4);}

/* k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_19256,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19259,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[7])){
/* library.scm:3358: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t2;
av2[2]=C_fix(1);
av2[3]=C_fast_retrieve(lf[711]);
av2[4]=((C_word*)t0)[4];
av2[5]=lf[730];
tp(6,av2);}}
else{
/* library.scm:3358: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t2;
av2[2]=C_fix(2);
av2[3]=C_fast_retrieve(lf[711]);
av2[4]=((C_word*)t0)[4];
av2[5]=lf[730];
tp(6,av2);}}}

/* k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_19259,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19262,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19265,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19283,a[2]=t3,a[3]=t1,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3359: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
av2[3]=((C_word*)((C_word*)t0)[6])[1];
tp(4,av2);}}

/* k19260 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19262,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k19263 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19265(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19265,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19268,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3360: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t2;
tp(2,av2);}}}

/* k19266 in k19263 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_19268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_19268,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19275,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19279,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t4=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k19273 in k19266 in k19263 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_19275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19275,c,av);}
/* library.scm:3361: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k19277 in k19266 in k19263 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_19279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19279,c,av);}
/* library.scm:3361: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[729];
av2[3]=t1;
tp(4,av2);}}

/* k19281 in k19257 in k19254 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19283,c,av);}
/* library.scm:3359: ##sys#open-file-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[712]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[712]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
tp(5,av2);}}

/* doloop3985 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_fcall f_19289(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_19289,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19302,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=C_eqp(t3,lf[731]);
if(C_truep(t5)){
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,lf[732]);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t6=C_eqp(t3,lf[733]);
if(C_truep(t6)){
t7=C_mutate(((C_word *)((C_word*)t0)[3])+1,lf[734]);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t7=C_eqp(t3,lf[735]);
if(C_truep(t7)){
if(C_truep(((C_word*)t0)[4])){
/* library.scm:3355: ##sys#error */
t8=*((C_word*)lf[155]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[736];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t8=C_mutate(((C_word *)((C_word*)t0)[6])+1,lf[737]);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}}
else{
/* library.scm:3357: ##sys#error */
t8=*((C_word*)lf[155]+1);{
C_word av2[5];
av2[0]=t8;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[738];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}}}}}

/* k19300 in doloop3985 in open in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19302,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_19289(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* close in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_fcall f_19344(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_19344,4,t1,t2,t3,t4);}
a=C_alloc(5);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19348,a[2]=t3,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3365: ##sys#check-port */
t6=*((C_word*)lf[695]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k19346 in close in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19348,c,av);}
t2=(C_truep(((C_word*)t0)[2])?C_fix(1):C_fix(2));
if(C_truep(C_port_openp(((C_word*)t0)[3],t2))){
t3=C_slot(((C_word*)t0)[3],C_fix(8));
t4=C_fixnum_not(t2);
t5=C_fixnum_and(t3,t4);
t6=C_i_set_i_slot(((C_word*)t0)[3],C_fix(8),t5);
t7=C_slot(((C_word*)t0)[3],C_fix(2));
/* library.scm:3368: g4006 */
t8=C_slot(t7,C_fix(4));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)C_fast_retrieve_proc(t8))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* scheme#open-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,5)))){
C_save_and_reclaim((void*)f_19381,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
/* library.scm:3372: open */
f_19246(t1,t2,C_SCHEME_TRUE,t3,lf[739]);}

/* scheme#open-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +0,c,5)))){
C_save_and_reclaim((void*)f_19387,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+0);
t3=C_build_rest(&a,c,3,av);
C_word t4;
/* library.scm:3373: open */
f_19246(t1,t2,C_SCHEME_FALSE,t3,lf[740]);}

/* scheme#close-input-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19393,c,av);}
/* library.scm:3374: close */
f_19344(t1,t2,C_SCHEME_TRUE,lf[741]);}

/* scheme#close-output-port in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19399,c,av);}
/* library.scm:3375: close */
f_19344(t1,t2,C_SCHEME_FALSE,lf[742]);}

/* scheme#call-with-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_19405,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+4);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19409,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t4;
C_apply(5,av2);}}

/* k19407 in scheme#call-with-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_19409,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19414,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li581),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19420,a[2]=t1,a[3]=((C_word)li582),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3382: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a19413 in k19407 in scheme#call-with-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19414,c,av);}
/* library.scm:3383: p */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* a19419 in k19407 in scheme#call-with-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_19420,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19424,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3385: close-input-port */
t4=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19422 in a19419 in k19407 in scheme#call-with-input-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19424,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_apply_values(3,av2);}}

/* scheme#call-with-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19429(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_19429,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+4);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19433,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t4;
C_apply(5,av2);}}

/* k19431 in scheme#call-with-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19433(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_19433,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19438,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word)li584),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19444,a[2]=t1,a[3]=((C_word)li585),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3393: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a19437 in k19431 in scheme#call-with-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19438(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19438,c,av);}
/* library.scm:3394: p */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* a19443 in k19431 in scheme#call-with-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_19444,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19448,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3396: close-output-port */
t4=*((C_word*)lf[140]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19446 in a19443 in k19431 in scheme#call-with-output-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19448,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_apply_values(3,av2);}}

/* scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_19453,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+4);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19457,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t4;
C_apply(5,av2);}}

/* k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19457(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_19457,c,av);}
a=C_alloc(19);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19462,a[2]=t5,a[3]=t3,a[4]=((C_word)li587),tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19467,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word)li589),tmp=(C_word)a,a+=5,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19482,a[2]=t3,a[3]=t5,a[4]=((C_word)li590),tmp=(C_word)a,a+=5,tmp);
/* library.scm:3404: ##sys#dynamic-wind */
t9=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a19461 in k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19462,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[713]+1));
t3=C_mutate((C_word*)lf[713]+1 /* (set! ##sys#standard-input ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a19466 in k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_19467,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19473,a[2]=((C_word*)t0)[2],a[3]=((C_word)li588),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3405: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
C_call_with_values(4,av2);}}

/* a19472 in a19466 in k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_19473,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19477,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3407: close-input-port */
t4=*((C_word*)lf[139]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19475 in a19472 in a19466 in k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_19477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19477,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_apply_values(3,av2);}}

/* a19481 in k19455 in scheme#with-input-from-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19482(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19482,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[713]+1));
t3=C_mutate((C_word*)lf[713]+1 /* (set! ##sys#standard-input ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +4,c,4)))){
C_save_and_reclaim((void*)f_19487,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+4);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19491,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t4;
C_apply(5,av2);}}

/* k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_19491,c,av);}
a=C_alloc(19);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19496,a[2]=t5,a[3]=t3,a[4]=((C_word)li592),tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19501,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word)li594),tmp=(C_word)a,a+=5,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19516,a[2]=t3,a[3]=t5,a[4]=((C_word)li595),tmp=(C_word)a,a+=5,tmp);
/* library.scm:3415: ##sys#dynamic-wind */
t9=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a19495 in k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19496,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[714]+1));
t3=C_mutate((C_word*)lf[714]+1 /* (set! ##sys#standard-output ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a19500 in k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_19501,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19507,a[2]=((C_word*)t0)[2],a[3]=((C_word)li593),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3416: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
C_call_with_values(4,av2);}}

/* a19506 in a19500 in k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_19507,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19511,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3418: close-output-port */
t4=*((C_word*)lf[140]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19509 in a19506 in a19500 in k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_19511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19511,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_apply_values(3,av2);}}

/* a19515 in k19489 in scheme#with-output-to-file in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19516(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19516,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[714]+1));
t3=C_mutate((C_word*)lf[714]+1 /* (set! ##sys#standard-output ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#file-exists? in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_19521,c,av);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19526,a[2]=t3,a[3]=t4,a[4]=t1,a[5]=t5,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* library.scm:3422: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2=av;
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t6;
av2[2]=t2;
av2[3]=t5;
tp(4,av2);}}

/* k19524 in ##sys#file-exists? in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19526,c,av);}
t2=C_i_file_exists_p(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
switch(t2){
case C_SCHEME_FALSE:
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
case C_SCHEME_TRUE:
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}
default:
/* library.scm:3426: ##sys#signal-hook */
t3=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[699];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[744];
av2[5]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}

/* ##sys#flush-output in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19547,c,av);}
a=C_alloc(3);
t3=C_slot(t2,C_fix(2));
t4=C_slot(t3,C_fix(5));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19554,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:3430: g4086 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}

/* k19552 in ##sys#flush-output in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19554,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#flush-output in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19560,c,av);}
if(C_truep(C_rest_nullp(c,2))){
t2=*((C_word*)lf[714]+1);
t3=C_i_check_port_2(*((C_word*)lf[714]+1),C_fix(2),C_SCHEME_TRUE,lf[747]);
/* library.scm:3437: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word av2[3];
av2[0]=*((C_word*)lf[746]+1);
av2[1]=t1;
av2[2]=*((C_word*)lf[714]+1);
tp(3,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=C_i_check_port_2(t2,C_fix(2),C_SCHEME_TRUE,lf[747]);
/* library.scm:3437: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word av2[3];
av2[0]=*((C_word*)lf[746]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}

/* ##sys#port-line in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19578,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_input_portp(t2))?C_slot(t2,C_fix(4)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#decorate-lambda in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_19587,c,av);}
a=C_alloc(10);
t5=C_block_size(t2);
t6=C_fixnum_difference(t5,C_fix(1));
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19600,a[2]=t5,a[3]=t2,a[4]=t4,a[5]=t8,a[6]=t3,a[7]=((C_word)li602),tmp=(C_word)a,a+=8,tmp));
t10=((C_word*)t8)[1];
f_19600(t10,t1,t6);}

/* loop in ##sys#decorate-lambda in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_fcall f_19600(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_19600,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_zerop(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19608,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3451: scheme#make-vector */
t4=*((C_word*)lf[93]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_slot(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19650,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3460: pred */
t5=((C_word*)t0)[6];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}}

/* k19606 in loop in ##sys#decorate-lambda in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_19608,c,av);}
a=C_alloc(10);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19613,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word)li601),tmp=(C_word)a,a+=8,tmp));
t5=((C_word*)t3)[1];
f_19613(t5,((C_word*)t0)[5],C_fix(1));}

/* doloop4115 in k19606 in loop in ##sys#decorate-lambda in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_fcall f_19613(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_19613,3,t0,t1,t2);}
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=C_vector_to_closure(((C_word*)t0)[3]);
t4=C_copy_pointer(((C_word*)t0)[4],((C_word*)t0)[3]);
/* library.scm:3456: decorator */
t5=((C_word*)t0)[5];{
C_word av2[4];
av2[0]=t5;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}
else{
t3=C_slot(((C_word*)t0)[4],t2);
t4=C_i_setslot(((C_word*)t0)[3],t2,t3);
t6=t1;
t7=C_fixnum_plus(t2,C_fix(1));
t1=t6;
t2=t7;
goto loop;}}

/* k19648 in loop in ##sys#decorate-lambda in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19650,c,av);}
if(C_truep(t1)){
/* library.scm:3461: decorator */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}
else{
/* library.scm:3462: loop */
t2=((C_word*)((C_word*)t0)[6])[1];
f_19600(t2,((C_word*)t0)[3],C_fixnum_difference(((C_word*)t0)[5],C_fix(1)));}}

/* ##sys#lambda-decoration in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_19662,c,av);}
a=C_alloc(8);
t4=C_block_size(t2);
t5=C_fixnum_difference(t4,C_fix(1));
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19672,a[2]=t2,a[3]=t7,a[4]=t3,a[5]=((C_word)li604),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_19672(t9,t1,t5);}

/* loop in ##sys#lambda-decoration in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_fcall f_19672(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19672,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_fixnum_greaterp(t2,C_fix(0)))){
t3=C_slot(((C_word*)t0)[2],t2);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19688,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3468: pred */
t5=((C_word*)t0)[4];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19686 in loop in ##sys#lambda-decoration in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19688(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19688,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:3470: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_19672(t2,((C_word*)t0)[2],C_fixnum_difference(((C_word*)t0)[5],C_fix(1)));}}

/* ##sys#make-lambda-info in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19701(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19701,c,av);}
a=C_alloc(5);
t3=C_block_size(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19708,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3477: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k19706 in ##sys#make-lambda-info in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19708,c,av);}
t2=C_copy_memory(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_string_to_lambdainfo(t1);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#lambda-info? in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19710,c,av);}
t3=C_immp(t2);
t4=C_i_not(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_lambdainfop(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#lambda-info in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19723,c,av);}
/* library.scm:3489: ##sys#lambda-decoration */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[750]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[750]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fast_retrieve(lf[752]);
tp(4,av2);}}

/* ##sys#lambda-info->string in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19729(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19729,c,av);}
a=C_alloc(5);
t3=C_block_size(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19736,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3493: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k19734 in ##sys#lambda-info->string in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19736,c,av);}
t2=C_copy_memory(t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#procedure-information in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_19738,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19742,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3499: ##sys#check-closure */
t4=*((C_word*)lf[369]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[756];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k19740 in chicken.base#procedure-information in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19742,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19745,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3500: ##sys#lambda-info */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[753]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[753]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k19743 in k19740 in chicken.base#procedure-information in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19745,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19755,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19759,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:3501: ##sys#lambda-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[754]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[754]+1);
av2[1]=t3;
av2[2]=t1;
tp(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19753 in k19743 in k19740 in chicken.base#procedure-information in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19755,c,av);}
/* library.scm:3501: ##sys#read */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[755]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[755]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}

/* k19757 in k19743 in k19740 in chicken.base#procedure-information in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19759(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19759,c,av);}
/* library.scm:3501: chicken.base#open-input-string */
t2=*((C_word*)lf[214]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_19767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19767,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[203]+1 /* (set! chicken.base#setter ...) */,t1);
t3=C_mutate((C_word*)lf[759]+1 /* (set! ##sys#setter ...) */,*((C_word*)lf[203]+1));
t4=C_mutate((C_word*)lf[200]+1 /* (set! chicken.base#getter-with-setter ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19770,a[2]=((C_word)li614),tmp=(C_word)a,a+=3,tmp));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19847,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3561: chicken.base#getter-with-setter */
t6=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=*((C_word*)lf[9]+1);
av2[3]=*((C_word*)lf[11]+1);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_19770,c,av);}
a=C_alloc(6);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19777,a[2]=t1,a[3]=t3,a[4]=t2,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3540: ##sys#check-closure */
t7=*((C_word*)lf[369]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t2;
av2[3]=lf[760];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_19777,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19780,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3541: ##sys#check-closure */
t3=*((C_word*)lf[369]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[760];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19780,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19783,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
t3=C_i_check_string_2(((C_word*)t0)[5],lf[760]);
/* library.scm:3544: ##sys#make-lambda-info */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[751]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[751]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}
else{
/* library.scm:3545: ##sys#lambda-info */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[753]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[753]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}}

/* k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_19783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_19783,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19786,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19802,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3547: ##sys#copy-closure */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[334]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[334]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k19784 in k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_19786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_19786,c,av);}
a=C_alloc(4);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19794,a[2]=((C_word*)t0)[2],a[3]=((C_word)li611),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3553: ##sys#decorate-lambda */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[749]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[749]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_fast_retrieve(lf[752]);
av2[4]=t2;
tp(5,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* a19793 in k19784 in k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in ... */
static void C_ccall f_19794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19794,c,av);}
t4=C_i_setslot(t2,t3,((C_word*)t0)[2]);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k19800 in k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_19802(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_19802,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19804,a[2]=((C_word)li612),tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19820,a[2]=((C_word*)t0)[2],a[3]=((C_word)li613),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3546: ##sys#decorate-lambda */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[749]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[749]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t2;
av2[4]=t3;
tp(5,av2);}}

/* a19803 in k19800 in k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in ... */
static void C_ccall f_19804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19804,c,av);}
t3=C_i_pairp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_eqp(C_retrieve2(lf[758],C_text("setter-tag")),C_slot(t2,C_fix(0))):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a19819 in k19800 in k19781 in k19778 in k19775 in chicken.base#getter-with-setter in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in ... */
static void C_ccall f_19820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19820,c,av);}
a=C_alloc(3);
t4=C_a_i_cons(&a,2,C_retrieve2(lf[758],C_text("setter-tag")),((C_word*)t0)[2]);
t5=C_i_setslot(t2,t3,t4);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_19847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_19847,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[9]+1 /* (set! scheme#car ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19851,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3562: chicken.base#getter-with-setter */
t4=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[10]+1);
av2[3]=*((C_word*)lf[12]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_19851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19851,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[10]+1 /* (set! scheme#cdr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19855,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32026,a[2]=((C_word)li1000),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3563: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[17]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_19855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19855,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[17]+1 /* (set! scheme#caar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19859,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32016,a[2]=((C_word)li999),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3564: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[13]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_19859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19859,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[13]+1 /* (set! scheme#cadr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19863,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32006,a[2]=((C_word)li998),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3565: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[18]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_19863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19863,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[18]+1 /* (set! scheme#cdar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19867,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31996,a[2]=((C_word)li997),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3566: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[19]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in ... */
static void C_ccall f_19867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19867,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[19]+1 /* (set! scheme#cddr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19871,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31986,a[2]=((C_word)li996),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3567: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[20]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in ... */
static void C_ccall f_19871(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19871,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[20]+1 /* (set! scheme#caaar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19875,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31976,a[2]=((C_word)li995),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3568: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[21]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in ... */
static void C_ccall f_19875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19875,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[21]+1 /* (set! scheme#caadr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19879,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31966,a[2]=((C_word)li994),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3569: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[22]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in ... */
static void C_ccall f_19879(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19879,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[22]+1 /* (set! scheme#cadar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19883,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31956,a[2]=((C_word)li993),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3570: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[14]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in ... */
static void C_ccall f_19883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19883,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[14]+1 /* (set! scheme#caddr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19887,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31946,a[2]=((C_word)li992),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3571: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[23]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in ... */
static void C_ccall f_19887(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19887,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[23]+1 /* (set! scheme#cdaar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19891,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31936,a[2]=((C_word)li991),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3572: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[24]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in ... */
static void C_ccall f_19891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19891,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[24]+1 /* (set! scheme#cdadr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19895,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31926,a[2]=((C_word)li990),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3573: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[25]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in ... */
static void C_ccall f_19895(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_19895,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[25]+1 /* (set! scheme#cddar ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19899,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31916,a[2]=((C_word)li989),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3574: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[26]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in ... */
static void C_ccall f_19899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_19899,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[26]+1 /* (set! scheme#cdddr ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19903,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3575: chicken.base#getter-with-setter */
t4=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[58]+1);
av2[3]=*((C_word*)lf[59]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in ... */
static void C_ccall f_19903(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_19903,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[58]+1 /* (set! scheme#string-ref ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19907,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3576: chicken.base#getter-with-setter */
t4=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[91]+1);
av2[3]=*((C_word*)lf[92]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in ... */
static void C_ccall f_19907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_19907,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[91]+1 /* (set! scheme#vector-ref ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19911,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31906,a[2]=((C_word)li988),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3579: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=*((C_word*)lf[45]+1);
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in ... */
static void C_ccall f_19911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(56,c,4)))){
C_save_and_reclaim((void *)f_19911,c,av);}
a=C_alloc(56);
t2=C_mutate((C_word*)lf[45]+1 /* (set! scheme#list-ref ...) */,t1);
t3=C_SCHEME_UNDEFINED;
t4=C_a_i_vector(&a,16,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3,t3);
t5=C_mutate(&lf[761] /* (set! ##sys#default-parameter-vector ...) */,t4);
t6=C_mutate((C_word*)lf[762]+1 /* (set! ##sys#current-parameter-vector ...) */,lf[763]);
t7=C_fix(0);
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_mutate((C_word*)lf[201]+1 /* (set! chicken.base#make-parameter ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19918,a[2]=t8,a[3]=((C_word)li622),tmp=(C_word)a,a+=4,tmp));
t10=C_mutate((C_word*)lf[146]+1 /* (set! scheme#char-ready? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20082,a[2]=((C_word)li623),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[147]+1 /* (set! scheme#read-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20107,a[2]=((C_word)li624),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[765]+1 /* (set! ##sys#read-char-0 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20125,a[2]=((C_word)li625),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[766]+1 /* (set! ##sys#read-char/port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20185,a[2]=((C_word)li626),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[767]+1 /* (set! ##sys#peek-char-0 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20194,a[2]=((C_word)li627),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[148]+1 /* (set! scheme#peek-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20219,a[2]=((C_word)li628),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate((C_word*)lf[149]+1 /* (set! scheme#read ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20237,a[2]=((C_word)li629),tmp=(C_word)a,a+=3,tmp));
t17=C_set_block_item(lf[769] /* ##sys#default-read-info-hook */,0,C_SCHEME_FALSE);
t18=C_set_block_item(lf[770] /* ##sys#read-error-with-line-number */,0,C_SCHEME_FALSE);
t19=C_mutate((C_word*)lf[771]+1 /* (set! ##sys#read-prompt-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20257,a[2]=((C_word)li630),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[772]+1 /* (set! ##sys#infix-list-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20260,a[2]=((C_word)li631),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[773]+1 /* (set! ##sys#sharp-number-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20263,a[2]=((C_word)li632),tmp=(C_word)a,a+=3,tmp));
t22=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20271,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3689: chicken.base#make-parameter */
t23=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t23;
av2[1]=t22;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t23+1)))(3,av2);}}

/* chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_19918(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_19918,c,av);}
a=C_alloc(8);
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20076,a[2]=((C_word)li615),tmp=(C_word)a,a+=3,tmp):C_get_rest_arg(c,3,av,3,t0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19925,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3592: guard */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}

/* k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in ... */
static void C_ccall f_19925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,6)))){
C_save_and_reclaim((void *)f_19925,c,av);}
a=C_alloc(14);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19926,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word)li616),tmp=(C_word)a,a+=5,tmp);
t4=C_fixnum_plus(((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t5=C_set_block_item(((C_word*)t0)[2],0,t4);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19960,a[2]=t2,a[3]=t1,a[4]=t3,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t7=C_block_size(C_retrieve2(lf[761],C_text("##sys#default-parameter-vector")));
if(C_truep(C_fixnum_greater_or_equal_p(t2,t7))){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20064,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* library.scm:3609: ##sys#vector-resize */
f_18160(t8,C_retrieve2(lf[761],C_text("##sys#default-parameter-vector")),C_fixnum_plus(t2,C_fix(1)),C_SCHEME_UNDEFINED);}
else{
t8=t6;
f_19960(t8,C_SCHEME_UNDEFINED);}}

/* assign in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in ... */
static void C_fcall f_19926(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_19926,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(11);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19930,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,a[6]=((C_word*)t0)[3],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],t3))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19949,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* library.scm:3597: ##sys#vector-resize */
f_18160(t7,C_fast_retrieve(lf[762]),C_fixnum_plus(((C_word*)t0)[2],C_fix(1)),*((C_word*)lf[627]+1));}
else{
t7=t6;
f_19930(t7,C_SCHEME_UNDEFINED);}}

/* k19928 in assign in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_fcall f_19930(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_19930,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19933,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
/* library.scm:3601: guard */
t3=((C_word*)t0)[6];{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[7];
f_19933(2,av2);}}}

/* k19931 in k19928 in assign in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_ccall f_19933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19933,c,av);}
if(C_truep(((C_word*)t0)[2])){
t2=C_i_setslot(C_fast_retrieve(lf[762]),((C_word*)t0)[3],t1);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19947 in assign in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_ccall f_19949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19949,c,av);}
t2=C_mutate((C_word*)lf[762]+1 /* (set! ##sys#current-parameter-vector ...) */,t1);
t3=((C_word*)t0)[2];
f_19930(t3,t2);}

/* k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in ... */
static void C_fcall f_19960(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_19960,2,t0,t1);}
a=C_alloc(9);
t2=C_i_setslot(C_retrieve2(lf[761],C_text("##sys#default-parameter-vector")),((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19968,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word)li620),tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20050,a[2]=((C_word*)t0)[4],a[3]=((C_word)li621),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3615: chicken.base#getter-with-setter */
t5=*((C_word*)lf[200]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=((C_word*)t0)[5];
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a19967 in k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_ccall f_19968(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +14,c,4)))){
C_save_and_reclaim((void*)f_19968,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+14);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
t3=C_block_size(C_fast_retrieve(lf[762]));
if(C_truep(C_i_pairp(t2))){
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19980,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t3,a[5]=((C_word)li617),tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19989,a[2]=t5,a[3]=((C_word)li618),tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19994,a[2]=t6,a[3]=((C_word)li619),tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_nullp(t4))){
/* library.scm:3619: def-convert?4268 */
t8=t7;
f_19994(t8,t1);}
else{
t8=C_i_car(t4);
t9=C_u_i_cdr(t4);
if(C_truep(C_i_nullp(t9))){
/* library.scm:3619: def-set?4269 */
t10=t6;
f_19989(t10,t1,t8);}
else{
t10=C_i_car(t9);
t11=C_u_i_cdr(t9);
/* library.scm:3619: body4266 */
t12=t5;
f_19980(t12,t1,t8,t10);}}}
else{
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[3],t3))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(C_retrieve2(lf[761],C_text("##sys#default-parameter-vector")),((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(C_fast_retrieve(lf[762]),((C_word*)t0)[3]);
t5=C_eqp(t4,*((C_word*)lf[627]+1));
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_slot(C_retrieve2(lf[761],C_text("##sys#default-parameter-vector")),((C_word*)t0)[3]):t4);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* body4266 in a19967 in k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_fcall f_19980(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_19980,4,t0,t1,t2,t3);}
/* library.scm:3621: assign */
t4=((C_word*)t0)[2];
f_19926(t4,t1,C_i_car(((C_word*)t0)[3]),((C_word*)t0)[4],t2,t3);}

/* def-set?4269 in a19967 in k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_fcall f_19989(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_19989,3,t0,t1,t2);}
/* library.scm:3619: body4266 */
t3=((C_word*)t0)[2];
f_19980(t3,t1,t2,C_SCHEME_TRUE);}

/* def-convert?4268 in a19967 in k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_fcall f_19994(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_19994,2,t0,t1);}
/* library.scm:3619: def-set?4269 */
t2=((C_word*)t0)[2];
f_19989(t2,t1,C_SCHEME_TRUE);}

/* a20049 in k19958 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_ccall f_20050(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_20050,c,av);}
/* library.scm:3631: assign */
t3=((C_word*)t0)[2];
f_19926(t3,t1,t2,C_block_size(C_fast_retrieve(lf[762])),C_SCHEME_FALSE,C_SCHEME_TRUE);}

/* k20062 in k19923 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in ... */
static void C_ccall f_20064(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20064,c,av);}
t2=C_mutate(&lf[761] /* (set! ##sys#default-parameter-vector ...) */,t1);
t3=((C_word*)t0)[2];
f_19960(t3,t2);}

/* f_20076 in chicken.base#make-parameter in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in ... */
static void C_ccall f_20076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20076,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#char-ready? in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20082(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20082,c,av);}
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?*((C_word*)lf[713]+1):C_get_rest_arg(c,2,av,2,t0));
t4=C_i_check_port_2(t3,C_fix(1),C_SCHEME_TRUE,lf[764]);
t5=C_slot(t3,C_fix(2));
/* library.scm:3637: g4301 */
t6=C_slot(t5,C_fix(6));{
C_word av2[3];
av2[0]=t6;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}

/* scheme#read-char in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20107(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20107,c,av);}
if(C_truep(C_rest_nullp(c,2))){
t2=*((C_word*)lf[713]+1);
t3=C_i_check_port_2(*((C_word*)lf[713]+1),C_fix(1),C_SCHEME_TRUE,lf[700]);
/* library.scm:3644: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t1;
av2[2]=*((C_word*)lf[713]+1);
tp(3,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=C_i_check_port_2(t2,C_fix(1),C_SCHEME_TRUE,lf[700]);
/* library.scm:3644: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}

/* ##sys#read-char-0 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_20125,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20129,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_slot(t2,C_fix(6)))){
t4=C_i_set_i_slot(t2,C_fix(6),C_SCHEME_FALSE);
t5=t3;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_END_OF_FILE;
f_20129(2,av2);}}
else{
t4=C_slot(t2,C_fix(2));
/* library.scm:3647: g4319 */
t5=C_slot(t4,C_fix(0));{
C_word *av2=av;
av2[0]=t5;
av2[1]=t3;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}}

/* k20127 in ##sys#read-char-0 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in ... */
static void C_ccall f_20129(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20129,c,av);}
t2=C_eqp(t1,C_make_character(10));
if(C_truep(t2)){
t3=C_slot(((C_word*)t0)[2],C_fix(4));
t4=C_fixnum_plus(t3,C_fix(1));
t5=C_i_set_i_slot(((C_word*)t0)[2],C_fix(4),t4);
t6=C_i_set_i_slot(((C_word*)t0)[2],C_fix(5),C_fix(0));
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
if(C_truep(C_i_not(C_eofp(t1)))){
t3=C_slot(((C_word*)t0)[2],C_fix(5));
t4=C_fixnum_plus(t3,C_fix(1));
t5=C_i_set_i_slot(((C_word*)t0)[2],C_fix(5),t4);
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* ##sys#read-char/port in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20185(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20185,c,av);}
t3=C_i_check_port_2(t2,C_fix(1),C_SCHEME_TRUE,lf[700]);
/* library.scm:3661: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2=av;
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}

/* ##sys#peek-char-0 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_20194,c,av);}
a=C_alloc(4);
if(C_truep(C_slot(t2,C_fix(6)))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_FILE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(2));
t4=C_slot(t3,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20207,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3666: g4333 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}}

/* k20205 in ##sys#peek-char-0 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in ... */
static void C_ccall f_20207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20207,c,av);}
if(C_truep(C_eofp(t1))){
t2=C_i_set_i_slot(((C_word*)t0)[2],C_fix(6),C_SCHEME_TRUE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* scheme#peek-char in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20219,c,av);}
if(C_truep(C_rest_nullp(c,2))){
t2=*((C_word*)lf[713]+1);
t3=C_i_check_port_2(*((C_word*)lf[713]+1),C_fix(1),C_SCHEME_TRUE,lf[704]);
/* library.scm:3674: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t1;
av2[2]=*((C_word*)lf[713]+1);
tp(3,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=C_i_check_port_2(t2,C_fix(1),C_SCHEME_TRUE,lf[704]);
/* library.scm:3674: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}

/* scheme#read in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20237(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20237,c,av);}
if(C_truep(C_rest_nullp(c,2))){
t2=*((C_word*)lf[713]+1);
t3=C_i_check_port_2(*((C_word*)lf[713]+1),C_fix(1),C_SCHEME_TRUE,lf[768]);
/* library.scm:3679: ##sys#read */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[755]);
C_word av2[4];
av2[0]=*((C_word*)lf[755]+1);
av2[1]=t1;
av2[2]=*((C_word*)lf[713]+1);
av2[3]=C_fast_retrieve(lf[769]);
tp(4,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=C_i_check_port_2(t2,C_fix(1),C_SCHEME_TRUE,lf[768]);
/* library.scm:3679: ##sys#read */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[755]);
C_word av2[4];
av2[0]=*((C_word*)lf[755]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=C_fast_retrieve(lf[769]);
tp(4,av2);}}}

/* ##sys#read-prompt-hook in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20257,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#infix-list-hook in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20260,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#sharp-number-hook in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20263,c,av);}
/* library.scm:3687: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=lf[775];
av2[4]=t3;
tp(5,av2);}}

/* k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in ... */
static void C_ccall f_20271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20271,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[232]+1 /* (set! chicken.base#case-sensitive ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20275,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3690: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in ... */
static void C_ccall f_20275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20275,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[234]+1 /* (set! chicken.base#parentheses-synonyms ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20279,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3691: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in ... */
static void C_ccall f_20279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_20279,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[235]+1 /* (set! chicken.base#symbol-escape ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20283,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31897,a[2]=((C_word)li987),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3694: chicken.base#make-parameter */
t5=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[811];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_ccall f_20283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_20283,c,av);}
a=C_alloc(8);
t2=C_mutate((C_word*)lf[233]+1 /* (set! chicken.base#keyword-style ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20287,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_a_i_record4(&a,4,lf[1384],C_SCHEME_FALSE,C_SCHEME_FALSE,C_SCHEME_FALSE);
/* library.scm:3696: chicken.base#make-parameter */
t5=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_ccall f_20287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(56,c,4)))){
C_save_and_reclaim((void *)f_20287,c,av);}
a=C_alloc(56);
t2=C_mutate((C_word*)lf[776]+1 /* (set! ##sys#current-read-table ...) */,t1);
t3=C_mutate((C_word*)lf[777]+1 /* (set! ##sys#read-warning ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20289,a[2]=((C_word)li633),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[774]+1 /* (set! ##sys#read-error ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20312,a[2]=((C_word)li634),tmp=(C_word)a,a+=3,tmp));
t5=*((C_word*)lf[233]+1);
t6=*((C_word*)lf[232]+1);
t7=*((C_word*)lf[234]+1);
t8=*((C_word*)lf[235]+1);
t9=C_fast_retrieve(lf[776]);
t10=C_mutate((C_word*)lf[755]+1 /* (set! ##sys#read ...) */,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20335,a[2]=t9,a[3]=t8,a[4]=t7,a[5]=t5,a[6]=t6,a[7]=((C_word)li690),tmp=(C_word)a,a+=8,tmp));
t11=C_mutate((C_word*)lf[789]+1 /* (set! ##sys#char->utf8-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23209,a[2]=((C_word)li691),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[796]+1 /* (set! ##sys#unicode-surrogate? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23374,a[2]=((C_word)li692),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[793]+1 /* (set! ##sys#surrogates->codepoint ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23386,a[2]=((C_word)li693),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[844]+1 /* (set! ##sys#read-bytevector-literal ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23448,a[2]=((C_word)li696),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[856]+1 /* (set! ##sys#user-read-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23596,a[2]=((C_word)li697),tmp=(C_word)a,a+=3,tmp));
t16=C_set_block_item(lf[854] /* ##sys#read-marks */,0,C_SCHEME_END_OF_LIST);
t17=C_mutate((C_word*)lf[863]+1 /* (set! ##sys#write-char-0 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23641,a[2]=((C_word)li698),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate((C_word*)lf[864]+1 /* (set! ##sys#write-char/port ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23657,a[2]=((C_word)li699),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[150]+1 /* (set! scheme#write-char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23669,a[2]=((C_word)li700),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[151]+1 /* (set! scheme#newline ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23690,a[2]=((C_word)li701),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[152]+1 /* (set! scheme#write ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23705,a[2]=((C_word)li702),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[153]+1 /* (set! scheme#display ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23723,a[2]=((C_word)li703),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[188]+1 /* (set! chicken.base#print ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23741,a[2]=((C_word)li705),tmp=(C_word)a,a+=3,tmp));
t24=C_mutate((C_word*)lf[189]+1 /* (set! chicken.base#print* ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23792,a[2]=((C_word)li707),tmp=(C_word)a,a+=3,tmp));
t25=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23845,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4438: chicken.base#make-parameter */
t26=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t26;
av2[1]=t25;
av2[2]=C_fix(0);
((C_proc)(void*)(*((C_word*)t26+1)))(3,av2);}}

/* ##sys#read-warning in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_20289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +5,c,2)))){
C_save_and_reclaim((void*)f_20289,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+5);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20297,a[2]=t1,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3703: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2=av;
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}

/* k20295 in ##sys#read-warning in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_20297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_20297,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20300,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(C_truep(C_fast_retrieve(lf[770]))?t1:C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20310,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3705: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=t1;
tp(3,av2);}}
else{{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[313]+1);
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[3];
C_apply(5,av2);}}}

/* k20298 in k20295 in ##sys#read-warning in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_20300(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20300,c,av);}{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[313]+1);
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
C_apply(5,av2);}}

/* k20308 in k20295 in ##sys#read-warning in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_20310(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_20310,c,av);}
/* library.scm:3705: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[778];
av2[3]=t1;
av2[4]=lf[779];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* ##sys#read-error in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_20312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +5,c,2)))){
C_save_and_reclaim((void*)f_20312,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+5);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20320,a[2]=t1,a[3]=t4,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3715: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2=av;
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}

/* k20318 in ##sys#read-error in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_20320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_20320,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20323,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(C_truep(C_fast_retrieve(lf[770]))?t1:C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20333,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3717: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=t1;
tp(3,av2);}}
else{{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[780];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[3];
C_apply(6,av2);}}}

/* k20321 in k20318 in ##sys#read-error in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_20323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_20323,c,av);}{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[780];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
C_apply(6,av2);}}

/* k20331 in k20318 in ##sys#read-error in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_20333(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_20333,c,av);}
/* library.scm:3717: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[781];
av2[3]=t1;
av2[4]=lf[782];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_20335(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20335,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20339,a[2]=t2,a[3]=t3,a[4]=t1,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3729: case-sensitive */
t5=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_20339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_20339,c,av);}
a=C_alloc(11);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20342,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3730: keyword-style */
t5=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_20342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_20342,c,av);}
a=C_alloc(11);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20345,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3731: parentheses-synonyms */
t5=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_20345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20345,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20348,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3732: symbol-escape */
t3=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_20348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20348,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20351,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3733: current-read-table */
t3=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)C_fast_retrieve_proc(t3))(2,av2);}}

/* k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_20351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(52,c,5)))){
C_save_and_reclaim((void *)f_20351,c,av);}
a=C_alloc(52);
t2=lf[783];
t3=C_SCHEME_FALSE;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20353,a[2]=((C_word*)t0)[2],a[3]=((C_word)li635),tmp=(C_word)a,a+=4,tmp));
t18=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20359,a[2]=((C_word*)t0)[3],a[3]=((C_word)li636),tmp=(C_word)a,a+=4,tmp));
t19=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20368,a[2]=((C_word*)t0)[2],a[3]=((C_word)li638),tmp=(C_word)a,a+=4,tmp));
t20=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20405,a[2]=((C_word*)t0)[2],a[3]=((C_word)li639),tmp=(C_word)a,a+=4,tmp));
t21=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20414,a[2]=t4,a[3]=t12,a[4]=((C_word)li640),tmp=(C_word)a,a+=5,tmp));
t22=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_20429,a[2]=((C_word*)t0)[2],a[3]=t10,a[4]=t6,a[5]=t8,a[6]=t16,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=t14,a[10]=t2,a[11]=t4,a[12]=t12,a[13]=t1,a[14]=((C_word*)t0)[6],a[15]=((C_word*)t0)[7],a[16]=((C_word)li689),tmp=(C_word)a,a+=17,tmp));
/* library.scm:4305: readrec */
t23=((C_word*)t16)[1];
f_20429(t23,((C_word*)t0)[8]);}

/* f_20353 in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_20353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20353,c,av);}
/* library.scm:3739: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[784];
av2[4]=t2;
tp(5,av2);}}

/* info in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_20359(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_20359,5,t0,t1,t2,t3,t4);}
if(C_truep(((C_word*)t0)[2])){
/* library.scm:3743: infohandler */
t5=((C_word*)t0)[2];{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
((C_proc)C_fast_retrieve_proc(t5))(5,av2);}}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* skip-to-eol in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_20368(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_20368,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20376,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3747: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k20374 in skip-to-eol in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_20376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_20376,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20378,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word)li637),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_20378(t5,((C_word*)t0)[3],t1);}

/* skip in k20374 in skip-to-eol in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_20378(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_20378,3,t0,t1,t2);}
a=C_alloc(4);
t3=C_i_not(C_eofp(t2));
t4=(C_truep(t3)?C_i_not(C_eqp(C_make_character(10),t2)):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20392,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3749: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k20390 in skip in k20374 in skip-to-eol in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_20392(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20392,c,av);}
/* library.scm:3749: skip */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20378(t2,((C_word*)t0)[3],t1);}

/* reserved-character in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_20405(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20405,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20409,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3752: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k20407 in reserved-character in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_20409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20409,c,av);}
/* library.scm:3753: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[785];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}

/* read-unreserved-char-0 in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_20414(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20414,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20418,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3756: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k20416 in read-unreserved-char-0 in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_20418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20418,c,av);}
if(C_truep(C_i_memq(t1,((C_word*)((C_word*)t0)[2])[1]))){
/* library.scm:3758: reserved-character */
t2=((C_word*)((C_word*)t0)[3])[1];
f_20405(t2,((C_word*)t0)[4],t1);}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_20429(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(158,0,5)))){
C_save_and_reclaim_args((void *)trf_20429,2,t0,t1);}
a=C_alloc(158);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_UNDEFINED;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_SCHEME_UNDEFINED;
t25=(*a=C_VECTOR_TYPE|1,a[1]=t24,tmp=(C_word)a,a+=2,tmp);
t26=C_SCHEME_UNDEFINED;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=C_SCHEME_UNDEFINED;
t29=(*a=C_VECTOR_TYPE|1,a[1]=t28,tmp=(C_word)a,a+=2,tmp);
t30=C_SCHEME_UNDEFINED;
t31=(*a=C_VECTOR_TYPE|1,a[1]=t30,tmp=(C_word)a,a+=2,tmp);
t32=C_SCHEME_UNDEFINED;
t33=(*a=C_VECTOR_TYPE|1,a[1]=t32,tmp=(C_word)a,a+=2,tmp);
t34=C_SCHEME_UNDEFINED;
t35=(*a=C_VECTOR_TYPE|1,a[1]=t34,tmp=(C_word)a,a+=2,tmp);
t36=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20432,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li642),tmp=(C_word)a,a+=5,tmp));
t37=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20480,a[2]=((C_word*)t0)[2],a[3]=((C_word)li644),tmp=(C_word)a,a+=4,tmp));
t38=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20540,a[2]=((C_word)li646),tmp=(C_word)a,a+=3,tmp));
t39=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20577,a[2]=((C_word*)t0)[2],a[3]=t5,a[4]=t7,a[5]=((C_word)li649),tmp=(C_word)a,a+=6,tmp));
t40=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21131,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t27,a[9]=((C_word)li655),tmp=(C_word)a,a+=10,tmp));
t41=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21393,a[2]=((C_word*)t0)[2],a[3]=t11,a[4]=((C_word)li656),tmp=(C_word)a,a+=5,tmp));
t42=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21411,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t27,a[5]=((C_word)li658),tmp=(C_word)a,a+=6,tmp));
t43=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21481,a[2]=((C_word*)t0)[2],a[3]=t15,a[4]=((C_word)li659),tmp=(C_word)a,a+=5,tmp));
t44=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21532,a[2]=((C_word*)t0)[2],a[3]=t15,a[4]=((C_word)li660),tmp=(C_word)a,a+=5,tmp));
t45=C_set_block_item(t21,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21599,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],a[6]=((C_word)li662),tmp=(C_word)a,a+=7,tmp));
t46=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21663,a[2]=((C_word*)t0)[2],a[3]=((C_word)li664),tmp=(C_word)a,a+=4,tmp));
t47=C_set_block_item(t25,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21709,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t27,a[5]=((C_word)li666),tmp=(C_word)a,a+=6,tmp));
t48=C_set_block_item(t27,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_21734,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[12],a[5]=t9,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[10],a[9]=((C_word)li668),tmp=(C_word)a,a+=10,tmp));
t49=C_set_block_item(t29,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21985,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[10],a[4]=t21,a[5]=((C_word)li669),tmp=(C_word)a,a+=6,tmp));
t50=C_set_block_item(t31,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22194,a[2]=((C_word*)t0)[2],a[3]=((C_word)li671),tmp=(C_word)a,a+=4,tmp));
t51=C_set_block_item(t33,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22281,a[2]=t21,a[3]=((C_word)li672),tmp=(C_word)a,a+=4,tmp));
t52=C_set_block_item(t35,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22294,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word)li673),tmp=(C_word)a,a+=6,tmp));
t53=(*a=C_CLOSURE_TYPE|27,a[1]=(C_word)f_22325,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[13],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=t35,a[8]=t23,a[9]=t17,a[10]=t19,a[11]=((C_word*)t0)[8],a[12]=t13,a[13]=t29,a[14]=t31,a[15]=t33,a[16]=((C_word*)t0)[7],a[17]=t27,a[18]=((C_word*)t0)[5],a[19]=((C_word*)t0)[3],a[20]=t21,a[21]=t11,a[22]=((C_word*)t0)[4],a[23]=t9,a[24]=t15,a[25]=((C_word*)t0)[12],a[26]=t25,a[27]=t3,tmp=(C_word)a,a+=28,tmp);
t54=C_i_not(((C_word*)t0)[14]);
t55=(C_truep(t54)?lf[857]:C_SCHEME_END_OF_LIST);
if(C_truep(C_i_not(((C_word*)t0)[15]))){
/* library.scm:4145: scheme#append */
t56=*((C_word*)lf[377]+1);{
C_word av2[4];
av2[0]=t56;
av2[1]=t53;
av2[2]=t55;
av2[3]=lf[858];
((C_proc)(void*)(*((C_word*)t56+1)))(4,av2);}}
else{
/* library.scm:4145: scheme#append */
t56=*((C_word*)lf[377]+1);{
C_word av2[4];
av2[0]=t56;
av2[1]=t53;
av2[2]=t55;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t56+1)))(4,av2);}}}

/* r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_20432(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20432,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20440,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3764: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_20440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_20440,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20442,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li641),tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_20442(t5,((C_word*)t0)[4],t1);}

/* loop in k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_20442(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_20442,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eofp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_eqp(C_make_character(59),t2);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20455,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3767: skip-to-eol */
t6=((C_word*)((C_word*)t0)[4])[1];
f_20368(t6,t5);}
else{
if(C_truep(C_u_i_char_whitespacep(t2))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20471,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3770: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}

/* k20453 in loop in k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_20455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_20455,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20462,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3768: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k20460 in k20453 in loop in k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20462,c,av);}
/* library.scm:3768: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20442(t2,((C_word*)t0)[3],t1);}

/* k20469 in loop in k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_20471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_20471,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20478,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3771: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k20476 in k20469 in loop in k20438 in r-spaces in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20478(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20478,c,av);}
/* library.scm:3771: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20442(t2,((C_word*)t0)[3],t1);}

/* r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_20480(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_20480,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20486,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t4,a[5]=t6,a[6]=((C_word)li643),tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_20486(t8,t1,C_SCHEME_END_OF_LIST,t3);}

/* loop in r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_20486(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_20486,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20496,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3776: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t5;
av2[2]=t2;
tp(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20512,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* library.scm:3783: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}}

/* k20494 in loop in r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_20496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_20496,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20499,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3777: scheme#string->number */
t3=*((C_word*)lf[620]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k20497 in k20494 in loop in r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_20499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_20499,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20509,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3781: string-append */
t3=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[786];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=lf[787];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}

/* k20507 in k20497 in k20494 in loop in r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20509,c,av);}
/* library.scm:3779: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k20510 in loop in r-usequence in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_20512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20512,c,av);}
a=C_alloc(3);
t2=C_eofp(t1);
t3=(C_truep(t2)?t2:C_i_char_equalp(C_make_character(34),t1));
if(C_truep(t3)){
/* library.scm:3785: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[788];
tp(4,av2);}}
else{
t4=C_a_i_cons(&a,2,t1,((C_word*)t0)[4]);
/* library.scm:3786: loop */
t5=((C_word*)((C_word*)t0)[5])[1];
f_20486(t5,((C_word*)t0)[2],t4,C_fixnum_difference(((C_word*)t0)[6],C_fix(1)));}}

/* r-cons-codepoint in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_20540(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_20540,3,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20544,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3789: ##sys#char->utf8-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[789]);
C_word av2[3];
av2[0]=*((C_word*)lf[789]+1);
av2[1]=t4;
av2[2]=C_make_character(C_unfix(t2));
tp(3,av2);}}

/* k20542 in r-cons-codepoint in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_20544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_20544,c,av);}
a=C_alloc(8);
t2=C_block_size(t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20552,a[2]=t2,a[3]=t1,a[4]=t4,a[5]=((C_word)li645),tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_20552(t6,((C_word*)t0)[2],C_fix(0),((C_word*)t0)[3]);}

/* lp in k20542 in r-cons-codepoint in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_20552(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_20552,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fixnum_plus(t2,C_fix(1));
t5=C_subchar(((C_word*)t0)[3],t2);
t6=C_a_i_cons(&a,2,t5,t3);
/* library.scm:3794: lp */
t8=t1;
t9=t4;
t10=t6;
t1=t8;
t2=t9;
t3=t10;
goto loop;}}

/* r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_20577(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_20577,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20585,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:3797: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_20585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_20585,c,av);}
a=C_alloc(10);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20587,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li648),tmp=(C_word)a,a+=8,tmp));
t5=((C_word*)t3)[1];
f_20587(t5,((C_word*)t0)[6],t1,C_SCHEME_END_OF_LIST);}

/* loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_20587(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_20587,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
if(C_truep(C_eofp(((C_word*)t4)[1]))){
/* library.scm:3799: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[4];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[790];
tp(4,av2);}}
else{
t5=C_eqp(C_make_character(92),((C_word*)t4)[1]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20604,a[2]=t4,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3801: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}
else{
t6=C_eqp(((C_word*)t0)[6],((C_word*)t4)[1]);
if(C_truep(t6)){
/* library.scm:3872: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t1;
av2[2]=t3;
tp(3,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21125,a[2]=t4,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3873: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}}}}

/* k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_20604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_20604,c,av);}
a=C_alloc(9);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)((C_word*)t0)[2])[1];
switch(t3){
case C_make_character(116):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20617,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3803: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(114):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20634,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3804: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(98):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20651,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3805: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(110):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20668,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3806: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(97):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20685,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3807: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(118):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20702,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3808: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(102):
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20719,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3809: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(120):
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20747,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3811: r-usequence */
t5=((C_word*)((C_word*)t0)[7])[1];
f_20480(t5,t4,lf[791],C_fix(2),C_fix(16));
case C_make_character(117):
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20756,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:3814: r-usequence */
t5=((C_word*)((C_word*)t0)[7])[1];
f_20480(t5,t4,lf[797],C_fix(4),C_fix(16));
case C_make_character(85):
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20830,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3827: r-usequence */
t5=((C_word*)((C_word*)t0)[7])[1];
f_20480(t5,t4,lf[799],C_fix(8),C_fix(16));
default:
t4=C_eqp(t3,C_make_character(92));
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20859,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t3,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t4)){
t6=t5;
f_20859(t6,t4);}
else{
t6=C_eqp(t3,C_make_character(39));
if(C_truep(t6)){
t7=t5;
f_20859(t7,t6);}
else{
t7=C_eqp(t3,C_make_character(34));
t8=t5;
f_20859(t8,(C_truep(t7)?t7:C_eqp(t3,C_make_character(124))));}}}}

/* k20615 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20617,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(9),((C_word*)t0)[2]);
/* library.scm:3803: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20632 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20634,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(13),((C_word*)t0)[2]);
/* library.scm:3804: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20649 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20651,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(8),((C_word*)t0)[2]);
/* library.scm:3805: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20666 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20668,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(10),((C_word*)t0)[2]);
/* library.scm:3806: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20683 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20685(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20685,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(7),((C_word*)t0)[2]);
/* library.scm:3807: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20700 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20702,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(11),((C_word*)t0)[2]);
/* library.scm:3808: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20717 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20719(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20719,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(12),((C_word*)t0)[2]);
/* library.scm:3809: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20587(t3,((C_word*)t0)[4],t1,t2);}

/* k20737 in k20745 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_20739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20739,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* library.scm:3812: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20587(t3,((C_word*)t0)[5],t1,t2);}

/* k20745 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_20747,c,av);}
a=C_alloc(6);
t2=C_make_character(C_unfix(t1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20739,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3812: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20756,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20762,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3815: ##sys#unicode-surrogate? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[796]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[796]+1);
av2[1]=t2;
av2[2]=t1;
tp(3,av2);}}

/* k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_20762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_20762,c,av);}
a=C_alloc(13);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20768,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20810,a[2]=t2,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3816: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20817,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3825: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}}

/* k20766 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_20768(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_20768,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20771,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:3818: r-usequence */
t3=((C_word*)((C_word*)t0)[8])[1];
f_20480(t3,t2,lf[794],C_fix(4),C_fix(16));}
else{
/* library.scm:3824: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[5];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[795];
av2[4]=((C_word*)t0)[7];
tp(5,av2);}}}

/* k20769 in k20766 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_20771,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20774,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* library.scm:3819: ##sys#surrogates->codepoint */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[793]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[793]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=t1;
tp(4,av2);}}

/* k20772 in k20769 in k20766 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_20774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_20774,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20784,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3821: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}
else{
/* library.scm:3823: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[6];
av2[3]=lf[792];
av2[4]=((C_word*)t0)[7];
av2[5]=((C_word*)t0)[8];
tp(6,av2);}}}

/* k20782 in k20772 in k20769 in k20766 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_20784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20784,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20788,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3822: r-cons-codepoint */
f_20540(t2,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k20786 in k20782 in k20772 in k20769 in k20766 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_20788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20788,c,av);}
/* library.scm:3821: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20587(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k20803 in k20808 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20805(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_20805,c,av);}
t2=((C_word*)t0)[2];
f_20768(t2,C_eqp(C_make_character(117),t1));}

/* k20808 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_20810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20810,c,av);}
a=C_alloc(3);
t2=C_eqp(C_make_character(92),t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20805,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:3817: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_20768(t3,C_SCHEME_FALSE);}}

/* k20815 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_20817(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20817,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20821,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3825: r-cons-codepoint */
f_20540(t2,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k20819 in k20815 in k20760 in k20754 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20821,c,av);}
/* library.scm:3825: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20587(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k20828 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_20830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_20830,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20836,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:3828: ##sys#unicode-surrogate? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[796]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[796]+1);
av2[1]=t2;
av2[2]=t1;
tp(3,av2);}}

/* k20834 in k20828 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_20836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_20836,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
/* library.scm:3829: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[798];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20846,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3830: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k20844 in k20834 in k20828 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_20846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_20846,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20850,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3830: r-cons-codepoint */
f_20540(t2,((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k20848 in k20844 in k20834 in k20828 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20850,c,av);}
/* library.scm:3830: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20587(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_20859(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_20859,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20866,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3832: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[7],C_make_character(10));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20879,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_20879(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[7],C_make_character(13));
if(C_truep(t4)){
t5=t3;
f_20879(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[7],C_make_character(32));
t6=t3;
f_20879(t6,(C_truep(t5)?t5:C_eqp(((C_word*)t0)[7],C_make_character(9))));}}}}

/* k20864 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_20866(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20866,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3]);
/* library.scm:3832: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20587(t3,((C_word*)t0)[5],t1,t2);}

/* k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_20879(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_20879,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20884,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word)li647),tmp=(C_word)a,a+=7,tmp));
t5=((C_word*)t3)[1];
f_20884(t5,((C_word*)t0)[5],((C_word*)((C_word*)t0)[6])[1],C_SCHEME_FALSE);}
else{
if(C_truep(C_eofp(((C_word*)((C_word*)t0)[6])[1]))){
/* library.scm:3858: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[4];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[2];
av2[3]=lf[801];
tp(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20983,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_u_i_char_numericp(((C_word*)((C_word*)t0)[6])[1]))){
t3=C_i_char_greater_or_equal_p(((C_word*)((C_word*)t0)[6])[1],C_make_character(48));
t4=t2;
f_20983(t4,(C_truep(t3)?C_i_char_less_or_equal_p(((C_word*)((C_word*)t0)[6])[1],C_make_character(55)):C_SCHEME_FALSE));}
else{
t3=t2;
f_20983(t3,C_SCHEME_FALSE);}}}}

/* eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_20884(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_20884,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_eqp(t2,C_make_character(32));
t5=(C_truep(t4)?t4:C_eqp(t2,C_make_character(9)));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20901,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3838: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
switch(t2){
case C_make_character(13):
if(C_truep(t3)){
/* library.scm:3841: loop */
t6=((C_word*)((C_word*)t0)[4])[1];
f_20587(t6,t1,t2,((C_word*)t0)[5]);}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20916,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3842: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
case C_make_character(10):
if(C_truep(t3)){
/* library.scm:3848: loop */
t6=((C_word*)((C_word*)t0)[4])[1];
f_20587(t6,t1,t2,((C_word*)t0)[5]);}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20951,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3849: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
default:
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20954,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t3)){
/* library.scm:3855: loop */
t7=((C_word*)((C_word*)t0)[4])[1];
f_20587(t7,t1,t2,((C_word*)t0)[5]);}
else{
/* library.scm:3852: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word av2[4];
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[800];
tp(4,av2);}}}}}

/* k20899 in eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20901,c,av);}
/* library.scm:3838: eat-ws */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20884(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4]);}

/* k20914 in eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20916(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_20916,c,av);}
a=C_alloc(4);
t2=C_eqp(t1,C_make_character(10));
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20929,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3844: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
/* library.scm:3845: eat-ws */
t3=((C_word*)((C_word*)t0)[2])[1];
f_20884(t3,((C_word*)t0)[3],t1,C_SCHEME_TRUE);}}

/* k20927 in k20914 in eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_20929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20929,c,av);}
/* library.scm:3844: eat-ws */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20884(t2,((C_word*)t0)[3],t1,C_SCHEME_TRUE);}

/* k20949 in eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20951(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20951,c,av);}
/* library.scm:3849: eat-ws */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20884(t2,((C_word*)t0)[3],t1,C_SCHEME_TRUE);}

/* k20952 in eat-ws in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_20954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20954,c,av);}
/* library.scm:3855: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20587(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k20981 in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_20983(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_20983,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=C_fix(C_character_code(((C_word*)((C_word*)t0)[2])[1]));
t3=C_fixnum_difference(t2,C_fix(48));
t4=C_fixnum_times(t3,C_fix(64));
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21009,a[2]=t4,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3864: r-usequence */
t6=((C_word*)((C_word*)t0)[7])[1];
f_20480(t6,t5,lf[802],C_fix(2),C_fix(8));}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21020,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:3867: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word av2[5];
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=lf[803];
av2[4]=((C_word*)((C_word*)t0)[2])[1];
tp(5,av2);}}}

/* k20991 in k21007 in k20981 in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_20993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20993,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* library.scm:3865: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20587(t3,((C_word*)t0)[5],t1,t2);}

/* k21007 in k20981 in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21009(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21009,c,av);}
a=C_alloc(6);
t2=C_fixnum_plus(((C_word*)t0)[2],t1);
t3=C_make_character(C_unfix(t2));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20993,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3865: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k21018 in k20981 in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21020(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21020,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21027,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3871: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k21025 in k21018 in k20981 in k20877 in k20857 in k20602 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_21027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21027,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3]);
/* library.scm:3871: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20587(t3,((C_word*)t0)[5],t1,t2);}

/* k21123 in loop in k20583 in r-string in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21125,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3]);
/* library.scm:3873: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20587(t3,((C_word*)t0)[5],t1,t2);}

/* r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21131(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_21131,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21391,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t3,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
/* library.scm:3876: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* starting-line in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_21140(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_21140,3,t0,t1,t2);}
a=C_alloc(4);
t3=(C_truep(((C_word*)((C_word*)t0)[2])[1])?C_fast_retrieve(lf[770]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21154,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3884: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
tp(3,av2);}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k21152 in starting-line in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21154,c,av);}
/* library.scm:3882: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[804];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21155 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21157,c,av);}
a=C_alloc(5);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
if(C_truep(((C_word*)((C_word*)t0)[4])[1])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21168,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3951: ##sys#infix-list-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[772]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[772]+1);
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
tp(3,av2);}}
else{
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21166 in k21155 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21168,c,av);}
/* library.scm:3951: info */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20359(t2,((C_word*)t0)[3],lf[805],t1,((C_word*)((C_word*)t0)[4])[1]);}

/* a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_21170,c,av);}
a=C_alloc(22);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21173,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word)li651),tmp=(C_word)a,a+=7,tmp));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21195,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t5,a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word)li653),tmp=(C_word)a,a+=13,tmp));
t7=((C_word*)t5)[1];
f_21195(t7,t1,C_SCHEME_END_OF_LIST);}

/* f_21173 in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21173,c,av);}
a=C_alloc(6);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
/* library.scm:3891: return */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21190,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3894: starting-line */
t5=((C_word*)t0)[5];
f_21140(t5,t4,lf[806]);}}

/* k21188 */
static void C_ccall f_21190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_21190,c,av);}
/* library.scm:3892: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
tp(6,av2);}}

/* loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_21195(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_21195,3,t0,t1,t2);}
a=C_alloc(14);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21199,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
/* library.scm:3897: r-spaces */
t4=((C_word*)((C_word*)t0)[5])[1];
f_20432(t4,t3);}

/* k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_21199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_21199,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21202,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)((C_word*)t0)[9])[1])){
t3=t2;
f_21202(t3,C_SCHEME_UNDEFINED);}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21384,a[2]=((C_word*)t0)[13],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3898: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_21202(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_21202,2,t0,t1);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21205,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* library.scm:3899: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21205(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21205,c,av);}
a=C_alloc(13);
if(C_truep(C_eofp(t1))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21215,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3903: starting-line */
t3=((C_word*)t0)[4];
f_21140(t3,t2,lf[807]);}
else{
t2=C_eqp(t1,((C_word*)t0)[5]);
if(C_truep(t2)){
/* library.scm:3905: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t3=C_eqp(t1,C_make_character(46));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21233,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* library.scm:3907: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21380,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3944: readrec */
t5=((C_word*)((C_word*)t0)[8])[1];
f_20429(t5,t4);}}}}

/* k21213 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21215(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21215,c,av);}
/* library.scm:3901: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21233(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_21233,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21236,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* library.scm:3908: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_21236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_21236,c,av);}
a=C_alloc(14);
t2=C_u_i_char_whitespacep(t1);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_21245,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t2)){
t4=t3;
f_21245(t4,t2);}
else{
t4=C_eqp(t1,C_make_character(40));
if(C_truep(t4)){
t5=t3;
f_21245(t5,t4);}
else{
t5=C_eqp(t1,C_make_character(41));
if(C_truep(t5)){
t6=t3;
f_21245(t6,t5);}
else{
t6=C_eqp(t1,C_make_character(34));
t7=t3;
f_21245(t7,(C_truep(t6)?t6:C_eqp(t1,C_make_character(59))));}}}}

/* k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_fcall f_21245(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_21245,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21248,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_21248(2,av2);}}
else{
/* library.scm:3915: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[4];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[809];
tp(4,av2);}}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21289,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[12],a[7]=((C_word)li652),tmp=(C_word)a,a+=8,tmp);
/* library.scm:3925: r-xtoken */
t3=((C_word*)((C_word*)t0)[13])[1];
f_21734(t3,((C_word*)t0)[4],t2);}}

/* k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_21248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_21248,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21251,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3916: r-spaces */
t3=((C_word*)((C_word*)t0)[7])[1];
f_20432(t3,t2);}

/* k21249 in k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in ... */
static void C_ccall f_21251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_21251,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21278,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:3917: readrec */
t3=((C_word*)((C_word*)t0)[8])[1];
f_20429(t3,t2);}

/* k21255 in k21276 in k21249 in k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in ... */
static void C_ccall f_21257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21257,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21274,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3919: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k21268 in k21272 in k21255 in k21276 in k21249 in k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ... */
static void C_ccall f_21270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21270,c,av);}
/* library.scm:3920: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}

/* k21272 in k21255 in k21276 in k21249 in k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in ... */
static void C_ccall f_21274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21274,c,av);}
a=C_alloc(5);
t2=C_eqp(t1,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21270,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3922: starting-line */
t4=((C_word*)t0)[5];
f_21140(t4,t3,lf[808]);}}

/* k21276 in k21249 in k21246 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in ... */
static void C_ccall f_21278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21278,c,av);}
a=C_alloc(6);
t2=C_i_setslot(((C_word*)t0)[2],C_fix(1),t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21257,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3918: r-spaces */
t4=((C_word*)((C_word*)t0)[7])[1];
f_20432(t4,t3);}

/* a21288 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_21289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_21289,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21293,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=t3,a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* library.scm:3927: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t4;
av2[2]=lf[813];
av2[3]=t2;
tp(4,av2);}}

/* k21291 in a21288 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in ... */
static void C_ccall f_21293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_21293,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21296,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_i_string_equal_p(t1,lf[810]);
t4=(C_truep(t3)?C_eqp(((C_word*)((C_word*)t0)[6])[1],lf[811]):C_SCHEME_FALSE);
if(C_truep(t4)){
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t2;
av2[2]=lf[812];
tp(3,av2);}}
else{
if(C_truep(((C_word*)t0)[7])){
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t2;
av2[2]=t1;
tp(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21324,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_u_i_char_numericp(((C_word*)t0)[8]))){
/* library.scm:3936: ##sys#string->number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[622]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[622]+1);
av2[1]=t5;
av2[2]=t1;
tp(3,av2);}}
else{
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t2;
av2[2]=t1;
C_string_to_symbol(3,av2);}}}}}

/* k21294 in k21291 in a21288 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in ... */
static void C_ccall f_21296(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21296,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t3=C_i_setslot(((C_word*)t0)[3],C_fix(1),t2);
/* library.scm:3942: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_21195(t4,((C_word*)t0)[5],t2);}
else{
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
/* library.scm:3942: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_21195(t4,((C_word*)t0)[5],t2);}}

/* k21322 in k21291 in a21288 in k21243 in k21234 in k21231 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in ... */
static void C_ccall f_21324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21324,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_21296(2,av2);}}
else{
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_string_to_symbol(3,av2);}}}

/* k21378 in k21203 in k21200 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21380,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t3=C_i_setslot(((C_word*)t0)[3],C_fix(1),t2);
/* library.scm:3948: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_21195(t4,((C_word*)t0)[5],t2);}
else{
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
/* library.scm:3948: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_21195(t4,((C_word*)t0)[5],t2);}}

/* k21382 in k21197 in loop in a21169 in k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_21384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21384,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];
f_21202(t3,t2);}

/* k21389 in r-list in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_21391,c,av);}
a=C_alloc(29);
t2=C_eqp(t1,((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_SCHEME_FALSE;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_FALSE;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=((C_word*)((C_word*)t0)[3])[1];
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21140,a[2]=t6,a[3]=((C_word)li650),tmp=(C_word)a,a+=4,tmp);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21157,a[2]=((C_word*)t0)[3],a[3]=t7,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t6,tmp=(C_word)a,a+=8,tmp);
t10=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21170,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=t8,a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=t4,a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=t6,a[12]=((C_word)li654),tmp=(C_word)a,a+=13,tmp);
/* library.scm:3886: ##sys#call-with-current-continuation */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t9;
av2[2]=t10;
C_call_cc(3,av2);}}
else{
/* library.scm:3953: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[5];
av2[2]=((C_word*)t0)[7];
av2[3]=lf[814];
av2[4]=((C_word*)t0)[2];
tp(5,av2);}}}

/* r-vector in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21393(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_21393,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21397,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3956: r-list */
t3=((C_word*)((C_word*)t0)[3])[1];
f_21131(t3,t2,C_make_character(40),C_make_character(41));}

/* k21395 in r-vector in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21397,c,av);}
if(C_truep(C_i_listp(t1))){
/* library.scm:3958: ##sys#list->vector */
t2=*((C_word*)lf[815]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
/* library.scm:3959: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[816];
av2[4]=t1;
tp(5,av2);}}}

/* r-number in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21411(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_21411,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21417,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t3,a[6]=((C_word)li657),tmp=(C_word)a,a+=7,tmp);
/* library.scm:3962: r-xtoken */
t5=((C_word*)((C_word*)t0)[4])[1];
f_21734(t5,t1,t4);}

/* a21416 in r-number in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_21417,c,av);}
a=C_alloc(6);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21424,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}
else{
if(C_truep(C_i_string_equal_p(t2,lf[818]))){
/* library.scm:3968: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2=av;
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[819];
tp(4,av2);}}
else{
t4=C_block_size(t2);
t5=C_fixnum_greaterp(t4,C_fix(0));
t6=(C_truep(t5)?C_u_i_char_equalp(C_i_string_ref(t2,C_fix(0)),C_make_character(35)):C_SCHEME_FALSE);
if(C_truep(t6)){
/* library.scm:3970: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[820];
av2[4]=t2;
tp(5,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21452,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[4])){
/* library.scm:3971: ##sys#string->number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[622]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[622]+1);
av2[1]=t7;
av2[2]=t2;
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
tp(5,av2);}}
else{
/* library.scm:3971: ##sys#string->number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[622]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[622]+1);
av2[1]=t7;
av2[2]=t2;
av2[3]=C_fix(10);
av2[4]=((C_word*)t0)[5];
tp(5,av2);}}}}}}

/* k21422 in a21416 in r-number in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21424,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21431,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:3966: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k21429 in k21422 in a21416 in r-number in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21431,c,av);}
/* library.scm:3966: info */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20359(t2,((C_word*)t0)[3],lf[817],((C_word*)t0)[4],t1);}

/* k21450 in a21416 in r-number in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21452,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(((C_word*)t0)[3])){
/* library.scm:3972: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[4];
av2[3]=lf[821];
av2[4]=((C_word*)t0)[5];
tp(5,av2);}}
else{
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[5];
C_string_to_symbol(3,av2);}}}}

/* r-number-with-exactness in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21481(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21481,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21530,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3976: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k21489 in k21528 in r-number-with-exactness in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21491,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21494,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3978: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k21492 in k21489 in k21528 in r-number-with-exactness in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21494,c,av);}
if(C_truep(C_eofp(t1))){
/* library.scm:3980: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[822];
tp(4,av2);}}
else{
if(C_truep(C_i_char_equalp(t1,C_make_character(105)))){
/* library.scm:3981: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],((C_word*)t0)[5],lf[613]);}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(101)))){
/* library.scm:3982: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],((C_word*)t0)[5],lf[612]);}
else{
/* library.scm:3984: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[823];
av2[4]=t1;
tp(5,av2);}}}}}

/* k21528 in r-number-with-exactness in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21530,c,av);}
a=C_alloc(6);
t2=C_eqp(C_make_character(35),t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21491,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3977: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
/* library.scm:3987: r-number */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21411(t3,((C_word*)t0)[2],((C_word*)t0)[5],C_SCHEME_FALSE);}}

/* r-number-with-radix in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21532(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21532,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21597,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:3990: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k21540 in k21595 in r-number-with-radix in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21542(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21542,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21545,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3992: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k21543 in k21540 in k21595 in r-number-with-radix in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21545,c,av);}
if(C_truep(C_eofp(t1))){
/* library.scm:3993: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[824];
tp(4,av2);}}
else{
if(C_truep(C_i_char_equalp(t1,C_make_character(120)))){
/* library.scm:3994: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],C_fix(16),((C_word*)t0)[5]);}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(100)))){
/* library.scm:3995: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],C_fix(10),((C_word*)t0)[5]);}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(111)))){
/* library.scm:3996: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],C_fix(8),((C_word*)t0)[5]);}
else{
if(C_truep(C_u_i_char_equalp(t1,C_make_character(98)))){
/* library.scm:3997: r-number */
t2=((C_word*)((C_word*)t0)[4])[1];
f_21411(t2,((C_word*)t0)[2],C_fix(2),((C_word*)t0)[5]);}
else{
/* library.scm:3998: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[825];
av2[4]=t1;
tp(5,av2);}}}}}}}

/* k21595 in r-number-with-radix in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21597,c,av);}
a=C_alloc(6);
t2=C_eqp(C_make_character(35),t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21542,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:3991: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
/* library.scm:3999: r-number */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21411(t3,((C_word*)t0)[2],C_fix(10),((C_word*)t0)[5]);}}

/* r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21599(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_21599,2,t0,t1);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21607,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4002: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k21605 in r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21607(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_21607,c,av);}
a=C_alloc(10);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21609,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li661),tmp=(C_word)a,a+=8,tmp));
t5=((C_word*)t3)[1];
f_21609(t5,((C_word*)t0)[6],t1,C_SCHEME_END_OF_LIST);}

/* loop in k21605 in r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_21609(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_21609,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=C_eofp(t2);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21619,a[2]=t1,a[3]=t3,a[4]=t2,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t4)){
t6=t5;
f_21619(t6,t4);}
else{
t6=C_u_i_char_whitespacep(t2);
t7=t5;
f_21619(t7,(C_truep(t6)?t6:C_i_memq(t2,((C_word*)t0)[6])));}}

/* k21617 in loop in k21605 in r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_21619(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_21619,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
/* library.scm:4006: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
if(C_truep(C_i_char_equalp(((C_word*)t0)[4],C_make_character(0)))){
/* library.scm:4008: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[4];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[5];
av2[3]=lf[826];
tp(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21634,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4010: read-unreserved-char-0 */
t3=((C_word*)((C_word*)t0)[8])[1];
f_20414(t3,t2,((C_word*)t0)[5]);}}}

/* k21632 in k21617 in loop in k21605 in r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_21634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21634,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21641,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4011: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}}

/* k21639 in k21632 in k21617 in loop in k21605 in r-token in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_21641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21641,c,av);}
a=C_alloc(3);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
/* library.scm:4011: loop */
t3=((C_word*)((C_word*)t0)[5])[1];
f_21609(t3,((C_word*)t0)[6],t1,t2);}
else{
t2=C_u_i_char_downcase(((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
/* library.scm:4011: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_21609(t4,((C_word*)t0)[6],t1,t3);}}

/* r-digits in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21663(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_21663,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21671,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4015: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k21669 in r-digits in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_21671,c,av);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21673,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word)li663),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_21673(t5,((C_word*)t0)[3],t1,C_SCHEME_END_OF_LIST);}

/* loop in k21669 in r-digits in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_21673(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_21673,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_eofp(t2);
t5=(C_truep(t4)?t4:C_i_not(C_u_i_char_numericp(t2)));
if(C_truep(t5)){
/* library.scm:4017: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t1;
av2[2]=t3;
tp(3,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21689,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4019: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}}

/* k21687 in loop in k21669 in r-digits in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21689,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21696,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4020: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k21694 in k21687 in loop in k21669 in r-digits in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_21696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21696,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* library.scm:4020: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21673(t3,((C_word*)t0)[5],t1,t2);}

/* r-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21709(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_21709,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21715,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li665),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4023: r-xtoken */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21734(t3,t1,t2);}

/* a21714 in r-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21715,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21719,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t3)){
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}
else{
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2=av;
av2[0]=0;
av2[1]=t4;
av2[2]=t2;
C_string_to_symbol(3,av2);}}}

/* k21717 in a21714 in r-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21719(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_21719,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21726,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4026: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k21724 in k21717 in a21714 in r-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21726,c,av);}
/* library.scm:4026: info */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20359(t2,((C_word*)t0)[3],lf[817],((C_word*)t0)[4],t1);}

/* r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21734(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_21734,3,t0,t1,t2);}
a=C_alloc(18);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21739,a[2]=t4,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=t1,tmp=(C_word)a,a+=12,tmp);
t6=C_eqp(((C_word*)((C_word*)t0)[2])[1],lf[830]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21983,a[2]=t5,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4031: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}
else{
t7=t5;
f_21739(t7,C_SCHEME_FALSE);}}

/* k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_21739(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,5)))){
C_save_and_reclaim_args((void *)trf_21739,2,t0,t1);}
a=C_alloc(15);
t2=C_set_block_item(((C_word*)t0)[2],0,t1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21744,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t4,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word)li667),tmp=(C_word)a,a+=13,tmp));
t6=((C_word*)t4)[1];
f_21744(t6,((C_word*)t0)[11],C_SCHEME_END_OF_LIST,C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_21744(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_21744,5,t0,t1,t2,t3,t4);}
a=C_alloc(16);
t5=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_21748,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=t4,a[7]=t3,a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],a[12]=((C_word*)t0)[8],a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[10],a[15]=((C_word*)t0)[11],tmp=(C_word)a,a+=16,tmp);
/* library.scm:4034: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[9];
tp(3,av2);}}

/* k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_21748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_21748,c,av);}
a=C_alloc(16);
t2=C_eofp(t1);
t3=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_21757,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
if(C_truep(t2)){
t4=t3;
f_21757(t4,t2);}
else{
t4=C_u_i_char_whitespacep(t1);
t5=t3;
f_21757(t5,(C_truep(t4)?t4:C_i_memq(t1,((C_word*)t0)[15])));}}

/* k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_21757(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_21757,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21763,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(((C_word*)t0)[7])){
t3=C_eqp(((C_word*)((C_word*)t0)[8])[1],lf[811]);
if(C_truep(t3)){
if(C_truep(((C_word*)t0)[6])){
t4=t2;
f_21763(t4,((C_word*)t0)[6]);}
else{
t4=C_i_cdr(((C_word*)t0)[4]);
t5=t2;
f_21763(t5,C_i_not(C_i_nullp(t4)));}}
else{
t4=t2;
f_21763(t4,C_SCHEME_FALSE);}}
else{
t3=t2;
f_21763(t3,C_SCHEME_FALSE);}}
else{
if(C_truep(C_i_memq(((C_word*)t0)[9],((C_word*)((C_word*)t0)[10])[1]))){
/* library.scm:4051: reserved-character */
t2=((C_word*)((C_word*)t0)[11])[1];
f_20405(t2,((C_word*)t0)[3],((C_word*)t0)[9]);}
else{
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21854,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[13],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[15],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4053: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[14];
tp(3,av2);}}}}

/* k21761 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_21763(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_21763,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21770,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4043: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t2;
av2[2]=C_i_cdr(((C_word*)t0)[4]);
tp(3,av2);}}
else{
t2=(C_truep(((C_word*)((C_word*)t0)[5])[1])?(C_truep(((C_word*)t0)[6])?((C_word*)t0)[6]:C_i_not(C_i_nullp(((C_word*)t0)[4]))):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21787,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4045: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21793,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t4=C_i_not(((C_word*)t0)[6]);
t5=t3;
f_21793(t5,(C_truep(t4)?C_i_nullp(((C_word*)t0)[4]):C_SCHEME_FALSE));}
else{
t4=t3;
f_21793(t4,C_SCHEME_FALSE);}}}}

/* k21768 in k21761 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21770,c,av);}
/* library.scm:4043: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* k21785 in k21761 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21787(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21787,c,av);}
/* library.scm:4045: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* k21791 in k21761 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_21793(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_21793,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
/* library.scm:4047: k */
t2=((C_word*)t0)[2];{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[827];
av2[3]=C_SCHEME_FALSE;
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21803,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4049: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}}

/* k21801 in k21791 in k21761 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21803,c,av);}
/* library.scm:4049: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_21854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_21854,c,av);}
a=C_alloc(10);
switch(t1){
case C_make_character(124):
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21863,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4056: r-string */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20577(t3,t2,C_make_character(124));
case C_make_character(10):
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21887,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21898,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4062: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}
case C_make_character(58):
t2=C_a_i_cons(&a,2,C_make_character(58),((C_word*)t0)[4]);
/* library.scm:4065: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_21744(t3,((C_word*)t0)[3],t2,C_SCHEME_TRUE,((C_word*)t0)[6]);
case C_make_character(92):
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21920,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4067: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
tp(3,av2);}
default:
if(C_truep(((C_word*)((C_word*)t0)[8])[1])){
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[4]);
/* library.scm:4074: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_21744(t3,((C_word*)t0)[3],t2,C_SCHEME_FALSE,((C_word*)t0)[6]);}
else{
t2=C_u_i_char_downcase(t1);
t3=C_a_i_cons(&a,2,t2,((C_word*)t0)[4]);
/* library.scm:4074: loop */
t4=((C_word*)((C_word*)t0)[2])[1];
f_21744(t4,((C_word*)t0)[3],t3,C_SCHEME_FALSE,((C_word*)t0)[6]);}}}

/* k21861 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21863,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21870,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21874,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21878,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4057: ##sys#string->list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[387]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[387]+1);
av2[1]=t4;
av2[2]=t1;
tp(3,av2);}}

/* k21868 in k21861 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21870(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21870,c,av);}
/* library.scm:4057: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21744(t2,((C_word*)t0)[3],t1,C_SCHEME_FALSE,C_SCHEME_TRUE);}

/* k21872 in k21861 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_21874,c,av);}
/* library.scm:4057: scheme#append */
t2=*((C_word*)lf[377]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k21876 in k21861 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_21878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21878,c,av);}
/* library.scm:4057: ##sys#fast-reverse */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[379]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[379]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}

/* k21885 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21887(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21887,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,C_make_character(10),((C_word*)t0)[2]);
/* library.scm:4063: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_21744(t3,((C_word*)t0)[4],t2,C_SCHEME_FALSE,((C_word*)t0)[5]);}

/* k21896 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21898,c,av);}
/* library.scm:4060: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[828];
av2[4]=t1;
tp(5,av2);}}

/* k21918 in k21852 in k21755 in k21746 in loop in k21737 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_21920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21920,c,av);}
a=C_alloc(3);
if(C_truep(C_eofp(t1))){
/* library.scm:4069: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[829];
tp(4,av2);}}
else{
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[4]);
/* library.scm:4072: loop */
t3=((C_word*)((C_word*)t0)[5])[1];
f_21744(t3,((C_word*)t0)[2],t2,C_SCHEME_FALSE,((C_word*)t0)[6]);}}

/* k21977 in k21981 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21979,c,av);}
t2=((C_word*)t0)[2];
f_21739(t2,C_SCHEME_TRUE);}

/* k21981 in r-xtoken in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21983(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21983,c,av);}
a=C_alloc(3);
t2=C_eqp(C_make_character(58),t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21979,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4032: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_21739(t3,C_SCHEME_FALSE);}}

/* r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_21985(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21985,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21989,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4080: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word av2[3];
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_21989(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_21989,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21992,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4081: r-token */
t3=((C_word*)((C_word*)t0)[5])[1];
f_21599(t3,t2);}

/* k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_21992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21992,c,av);}
a=C_alloc(11);
t2=C_block_size(t1);
if(C_truep(C_fixnum_greaterp(t2,C_fix(1)))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22004,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_i_char_equalp(C_make_character(120),((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22169,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
t6=t5;
f_22169(t6,t4);}
else{
t6=C_u_i_char_equalp(C_make_character(117),((C_word*)t0)[4]);
t7=t5;
f_22169(t7,(C_truep(t6)?t6:C_u_i_char_equalp(C_make_character(85),((C_word*)t0)[4])));}}
else{
if(C_truep(C_i_memq(((C_word*)t0)[4],((C_word*)t0)[5]))){
/* library.scm:4111: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k22002 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22004,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_make_character(C_unfix(t1));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_subchar(((C_word*)t0)[3],C_fix(0));
t3=C_fix(C_character_code(t2));
t4=C_fixnum_less_or_equal_p(C_fix(192),t3);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22024,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(247)))){
t6=C_fixnum_shift_right(t3,C_fix(4));
t7=C_fixnum_and(t6,C_fix(3));
t8=C_fixnum_shift_right(t7,C_fix(1));
t9=C_fixnum_or(t7,t8);
t10=C_fixnum_difference(t7,C_fix(1));
t11=C_fixnum_and(t9,t10);
t12=C_fixnum_plus(C_fix(2),t11);
t13=C_eqp(((C_word*)t0)[5],t12);
if(C_truep(t13)){
t14=C_fixnum_difference(C_fix(8),t12);
t15=C_fixnum_shift_left(C_fix(1),t14);
t16=C_fixnum_difference(t15,C_fix(1));
t17=C_fixnum_and(t3,t16);
t18=C_fixnum_shift_left(t17,C_fix(6));
t19=C_subchar(((C_word*)t0)[3],C_fix(1));
t20=C_fix(C_character_code(t19));
t21=C_fixnum_and(t20,C_fix(63));
t22=C_fixnum_plus(t18,t21);
t23=t22;
if(C_truep(t23)){
if(C_truep(C_fixnum_greater_or_equal_p(t12,C_fix(3)))){
t24=C_fixnum_shift_left(t23,C_fix(6));
t25=C_subchar(((C_word*)t0)[3],C_fix(2));
t26=C_fix(C_character_code(t25));
t27=C_fixnum_and(t26,C_fix(63));
t28=C_fixnum_plus(t24,t27);
t29=t23=t28;
t30=C_eqp(t12,C_fix(4));
if(C_truep(t30)){
t31=C_fixnum_shift_left(t23,C_fix(6));
t32=C_subchar(((C_word*)t0)[3],C_fix(3));
t33=C_fix(C_character_code(t32));
t34=C_fixnum_and(t33,C_fix(63));
t35=C_fixnum_plus(t31,t34);
t36=t23=t35;
t37=t5;
f_22024(t37,C_make_character(C_unfix(t23)));}
else{
t31=t5;
f_22024(t31,C_make_character(C_unfix(t23)));}}
else{
t24=t5;
f_22024(t24,C_make_character(C_unfix(t23)));}}
else{
t24=t5;
f_22024(t24,C_SCHEME_FALSE);}}
else{
t14=t5;
f_22024(t14,C_SCHEME_FALSE);}}
else{
t6=t5;
f_22024(t6,C_SCHEME_FALSE);}}
else{
t6=t5;
f_22024(t6,C_SCHEME_FALSE);}}}

/* k22022 in k22002 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_22024(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_22024,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22030,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22040,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4109: ##sys#intern-symbol */{
C_word av2[3];
av2[0]=0;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
C_string_to_symbol(3,av2);}}}

/* k22028 in k22022 in k22002 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22030,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:4110: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[831];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}}

/* k22038 in k22022 in k22002 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22040(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22040,c,av);}
/* library.scm:4109: chicken.base#char-name */
t2=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22167 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_22169(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_22169,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22176,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4085: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_fix(1);
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_22004(2,av2);}}}

/* k22174 in k22167 in k21990 in k21987 in r-char in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_22176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22176,c,av);}
/* library.scm:4085: ##sys#string->number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[622]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[622]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_fix(16);
tp(4,av2);}}

/* r-comment in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_22194(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_22194,2,t0,t1);}
a=C_alloc(7);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22200,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word)li670),tmp=(C_word)a,a+=5,tmp));
t5=((C_word*)t3)[1];
f_22200(t5,t1,C_fix(0));}

/* loop in r-comment in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_22200(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_22200,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22204,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4116: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22202 in loop in r-comment in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_22204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_22204,c,av);}
a=C_alloc(5);
switch(t1){
case C_make_character(124):
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22240,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4118: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}
case C_make_character(35):
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22263,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4122: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}
default:
if(C_truep(C_eofp(t1))){
/* library.scm:4126: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=lf[832];
tp(4,av2);}}
else{
/* library.scm:4127: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_22200(t2,((C_word*)t0)[4],((C_word*)t0)[2]);}}}

/* k22238 in k22202 in loop in r-comment in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22240,c,av);}
t2=C_eqp(C_make_character(35),t1);
if(C_truep(t2)){
t3=C_eqp(((C_word*)t0)[2],C_fix(0));
if(C_truep(C_i_not(t3))){
/* library.scm:4120: loop */
t4=((C_word*)((C_word*)t0)[3])[1];
f_22200(t4,((C_word*)t0)[4],C_fixnum_difference(((C_word*)t0)[2],C_fix(1)));}
else{
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
/* library.scm:4121: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_22200(t3,((C_word*)t0)[4],((C_word*)t0)[2]);}}

/* k22261 in k22202 in loop in r-comment in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22263,c,av);}
t2=C_eqp(C_make_character(124),t1);
if(C_truep(t2)){
/* library.scm:4122: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_22200(t3,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}
else{
/* library.scm:4122: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_22200(t3,((C_word*)t0)[3],((C_word*)t0)[4]);}}

/* r-ext-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_22281(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_22281,2,t0,t1);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22285,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4130: r-token */
t3=((C_word*)((C_word*)t0)[2])[1];
f_21599(t3,t2);}

/* k22283 in r-ext-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_22285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_22285,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22292,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4131: string-append */
t3=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[833];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22290 in k22283 in r-ext-symbol in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_22292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22292,c,av);}
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
C_string_to_symbol(3,av2);}}

/* r-quote in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_22294(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_22294,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22298,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4134: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word av2[3];
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k22296 in r-quote in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_22298(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22298,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22309,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4135: readrec */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20429(t3,t2);}

/* k22307 in k22296 in r-quote in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_22309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22309,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* library.scm:4135: info */
t3=((C_word*)((C_word*)t0)[3])[1];
f_20359(t3,((C_word*)t0)[4],lf[805],t2,((C_word*)t0)[5]);}

/* k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_22325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,2)))){
C_save_and_reclaim((void *)f_22325,c,av);}
a=C_alloc(27);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_22328,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],a[15]=((C_word*)t0)[16],a[16]=((C_word*)t0)[17],a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[19],a[19]=((C_word*)t0)[20],a[20]=((C_word*)t0)[21],a[21]=((C_word*)t0)[22],a[22]=((C_word*)t0)[23],a[23]=((C_word*)t0)[24],a[24]=((C_word*)t0)[2],a[25]=((C_word*)t0)[25],a[26]=((C_word*)t0)[26],tmp=(C_word)a,a+=27,tmp);
/* library.scm:4148: r-spaces */
t4=((C_word*)((C_word*)t0)[27])[1];
f_20432(t4,t3);}

/* k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_22328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,2)))){
C_save_and_reclaim((void *)f_22328,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_22331,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],a[23]=((C_word*)t0)[23],a[24]=((C_word*)t0)[24],a[25]=((C_word*)t0)[25],a[26]=((C_word*)t0)[26],tmp=(C_word)a,a+=27,tmp);
/* library.scm:4149: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_22331(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_22331,c,av);}
a=C_alloc(21);
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_eofp(t1);
t4=C_i_not(t3);
t5=(C_truep(t4)?(C_truep(t2)?C_slot(t2,C_fix(C_character_code(t1))):C_SCHEME_FALSE):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22345,a[2]=t5,a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word)li674),tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22351,a[2]=((C_word*)t0)[4],a[3]=((C_word)li675),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4155: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=t6;
av2[3]=t7;
C_call_with_values(4,av2);}}
else{
switch(t1){
case C_make_character(39):
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22373,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4161: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(96):
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22385,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4164: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(44):
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22397,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4167: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(35):
t6=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_22425,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[6],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
/* library.scm:4173: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(40):
/* library.scm:4288: r-list */
t6=((C_word*)((C_word*)t0)[20])[1];
f_21131(t6,((C_word*)t0)[5],C_make_character(40),C_make_character(41));
case C_make_character(41):
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23043,a[2]=((C_word*)t0)[21],a[3]=((C_word*)t0)[5],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4289: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(34):
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23055,a[2]=((C_word*)t0)[22],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4290: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(46):
/* library.scm:4291: r-number */
t6=((C_word*)((C_word*)t0)[23])[1];
f_21411(t6,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE);
default:
t6=C_eqp(t1,C_make_character(45));
t7=(C_truep(t6)?t6:C_eqp(t1,C_make_character(43)));
if(C_truep(t7)){
/* library.scm:4292: r-number */
t8=((C_word*)((C_word*)t0)[23])[1];
f_21411(t8,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE);}
else{
if(C_truep(C_eofp(t1))){
t8=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t8;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
if(C_truep(C_u_i_char_numericp(t1))){
/* library.scm:4295: r-number */
t8=((C_word*)((C_word*)t0)[23])[1];
f_21411(t8,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE);}
else{
if(C_truep(C_i_memq(t1,((C_word*)((C_word*)t0)[24])[1]))){
/* library.scm:4297: reserved-character */
t8=((C_word*)((C_word*)t0)[25])[1];
f_20405(t8,((C_word*)t0)[5],t1);}
else{
switch(t1){
case C_make_character(91):
/* library.scm:4300: r-list */
t8=((C_word*)((C_word*)t0)[20])[1];
f_21131(t8,((C_word*)t0)[5],C_make_character(91),C_make_character(93));
case C_make_character(123):
/* library.scm:4301: r-list */
t8=((C_word*)((C_word*)t0)[20])[1];
f_21131(t8,((C_word*)t0)[5],C_make_character(123),C_make_character(125));
default:
t8=C_eqp(t1,C_make_character(93));
t9=(C_truep(t8)?t8:C_eqp(t1,C_make_character(125)));
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23133,a[2]=((C_word*)t0)[21],a[3]=((C_word*)t0)[5],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4302: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t10;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
/* library.scm:4303: r-symbol */
t10=((C_word*)((C_word*)t0)[26])[1];
f_21709(t10,((C_word*)t0)[5]);}}}}}}}}}

/* a22344 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22345,c,av);}
/* library.scm:4156: h */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* a22350 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22351(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22351,c,av);}
if(C_truep(C_rest_nullp(c,2))){
/* library.scm:4157: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,t1);}
else{
t2=t1;{
C_word av2[2];
av2[0]=t2;
av2[1]=C_get_rest_arg(c,2,av,2,t0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k22371 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22373,c,av);}
/* library.scm:4162: r-quote */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22294(t2,((C_word*)t0)[3],lf[834]);}

/* k22383 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22385(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22385,c,av);}
/* library.scm:4165: r-quote */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22294(t2,((C_word*)t0)[3],lf[835]);}

/* k22395 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22397(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22397,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22416,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4168: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k22404 in k22414 in k22395 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22406,c,av);}
/* library.scm:4170: r-quote */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22294(t2,((C_word*)t0)[3],lf[836]);}

/* k22414 in k22395 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_22416(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22416,c,av);}
a=C_alloc(4);
t2=C_eqp(t1,C_make_character(64));
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22406,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4169: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}
else{
/* library.scm:4171: r-quote */
t3=((C_word*)((C_word*)t0)[2])[1];
f_22294(t3,((C_word*)t0)[3],lf[837]);}}

/* k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_22425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_22425,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_22428,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],tmp=(C_word)a,a+=21,tmp);
/* library.scm:4174: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_22428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_22428,c,av);}
a=C_alloc(10);
if(C_truep(C_eofp(t1))){
/* library.scm:4177: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[838];
tp(4,av2);}}
else{
if(C_truep(C_u_i_char_numericp(t1))){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22446,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22529,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4180: r-digits */
t4=((C_word*)((C_word*)t0)[7])[1];
f_21663(t4,t3);}
else{
t2=C_slot(((C_word*)t0)[4],C_fix(2));
t3=(C_truep(t2)?C_slot(t2,C_fix(C_character_code(t1))):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22543,a[2]=t3,a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word)li678),tmp=(C_word)a,a+=6,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22549,a[2]=((C_word*)t0)[6],a[3]=((C_word)li679),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4205: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t4;
av2[3]=t5;
C_call_with_values(4,av2);}}
else{
t4=C_u_i_char_downcase(t1);
switch(t4){
case C_make_character(120):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22574,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4210: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(100):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22586,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4211: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(111):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22598,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4212: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(98):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22610,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4213: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(105):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22622,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4214: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(101):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22634,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4215: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(99):
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22646,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4217: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(40):
/* library.scm:4226: r-vector */
t5=((C_word*)((C_word*)t0)[11])[1];
f_21393(t5,((C_word*)t0)[2]);
case C_make_character(92):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22710,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4227: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(124):
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22722,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[13],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4229: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(35):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22737,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4232: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(59):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22749,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4235: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(96):
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22764,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4238: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(36):
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22776,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[15],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4241: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(58):
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22803,a[2]=((C_word*)t0)[16],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[17],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4250: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(43):
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22863,a[2]=((C_word*)t0)[18],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4260: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
case C_make_character(33):
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22893,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[19],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[20],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4267: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}
default:
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22965,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word)li687),tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22971,a[2]=((C_word*)t0)[6],a[3]=((C_word)li688),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4286: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}}}}}

/* k22444 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22446,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22449,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4181: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k22447 in k22444 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22449(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_22449,c,av);}
a=C_alloc(11);
t2=C_slot(((C_word*)t0)[2],C_fix(3));
t3=C_charp(t1);
t4=(C_truep(t3)?(C_truep(t2)?C_slot(t2,C_fix(C_character_code(t1))):C_SCHEME_FALSE):C_SCHEME_FALSE);
if(C_truep(C_eofp(t1))){
/* library.scm:4187: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=lf[839];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
tp(6,av2);}}
else{
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22472,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word)li676),tmp=(C_word)a,a+=7,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22478,a[2]=((C_word*)t0)[7],a[3]=((C_word)li677),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4190: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t5;
av2[3]=t6;
C_call_with_values(4,av2);}}
else{
t5=C_eqp(t1,C_make_character(41));
t6=(C_truep(t5)?t5:C_u_i_char_whitespacep(t1));
if(C_truep(t6)){
/* library.scm:4195: ##sys#sharp-number-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[773]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[773]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[6];
tp(4,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22506,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4196: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}}}}

/* a22471 in k22447 in k22444 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22472(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22472,c,av);}
/* library.scm:4191: h */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
((C_proc)C_fast_retrieve_proc(t2))(5,av2);}}

/* a22477 in k22447 in k22444 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22478(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22478,c,av);}
if(C_truep(C_rest_nullp(c,2))){
/* library.scm:4192: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,t1);}
else{
t2=t1;{
C_word av2[2];
av2[0]=t2;
av2[1]=C_get_rest_arg(c,2,av,2,t0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k22504 in k22447 in k22444 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_22506,c,av);}
/* library.scm:4197: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[840];
av2[4]=((C_word*)t0)[4];
av2[5]=((C_word*)t0)[5];
av2[6]=((C_word*)t0)[6];
tp(7,av2);}}

/* k22527 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22529,c,av);}
/* library.scm:4180: scheme#string->number */
t2=*((C_word*)lf[620]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a22542 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22543,c,av);}
/* library.scm:4206: h */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* a22548 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22549,c,av);}
if(C_truep(C_rest_nullp(c,2))){
/* library.scm:4207: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,t1);}
else{
t2=t1;{
C_word av2[2];
av2[0]=t2;
av2[1]=C_get_rest_arg(c,2,av,2,t0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k22572 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22574,c,av);}
/* library.scm:4210: r-number-with-exactness */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21481(t2,((C_word*)t0)[3],C_fix(16));}

/* k22584 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22586,c,av);}
/* library.scm:4211: r-number-with-exactness */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21481(t2,((C_word*)t0)[3],C_fix(10));}

/* k22596 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22598(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22598,c,av);}
/* library.scm:4212: r-number-with-exactness */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21481(t2,((C_word*)t0)[3],C_fix(8));}

/* k22608 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22610,c,av);}
/* library.scm:4213: r-number-with-exactness */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21481(t2,((C_word*)t0)[3],C_fix(2));}

/* k22620 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22622,c,av);}
/* library.scm:4214: r-number-with-radix */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21532(t2,((C_word*)t0)[3],lf[613]);}

/* k22632 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22634,c,av);}
/* library.scm:4215: r-number-with-radix */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21532(t2,((C_word*)t0)[3],lf[612]);}

/* k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22646,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22649,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4218: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k22647 in k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22649,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22652,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_eofp(t1))){
/* library.scm:4221: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[841];
tp(4,av2);}}
else{
switch(t1){
case C_make_character(105):
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_22652(2,av2);}
case C_make_character(115):
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
f_22652(2,av2);}
default:
/* library.scm:4224: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[842];
av2[4]=t1;
tp(5,av2);}}}}

/* k22650 in k22647 in k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,4)))){
C_save_and_reclaim((void *)f_22652,c,av);}
a=C_alloc(20);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22657,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word)li680),tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22662,a[2]=((C_word*)t0)[3],a[3]=((C_word)li681),tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22668,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word)li682),tmp=(C_word)a,a+=6,tmp);
/* library.scm:4219: ##sys#dynamic-wind */
t9=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[4];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a22656 in k22650 in k22647 in k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22657,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a22661 in k22650 in k22647 in k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22662,c,av);}
/* library.scm:4225: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,t1);}

/* a22667 in k22650 in k22647 in k22644 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22668,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k22708 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22710,c,av);}
/* library.scm:4227: r-char */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21985(t2,((C_word*)t0)[3]);}

/* k22720 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22722,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4230: r-comment */
t3=((C_word*)((C_word*)t0)[4])[1];
f_22194(t3,t2);}

/* k22723 in k22720 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22725,c,av);}
/* library.scm:4230: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,((C_word*)t0)[3]);}

/* k22735 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22737,c,av);}
/* library.scm:4233: r-ext-symbol */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22281(t2,((C_word*)t0)[3]);}

/* k22747 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22749,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22752,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4236: readrec */
t3=((C_word*)((C_word*)t0)[2])[1];
f_20429(t3,t2);}

/* k22750 in k22747 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22752,c,av);}
/* library.scm:4236: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,((C_word*)t0)[3]);}

/* k22762 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22764,c,av);}
/* library.scm:4239: r-quote */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22294(t2,((C_word*)t0)[3],lf[843]);}

/* k22774 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22776,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22779,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4242: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22777 in k22774 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22779,c,av);}
a=C_alloc(4);
if(C_truep(C_i_char_equalp(t1,C_make_character(123)))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22788,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4244: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
/* library.scm:4248: r-quote */
t2=((C_word*)((C_word*)t0)[4])[1];
f_22294(t2,((C_word*)t0)[2],lf[845]);}}

/* k22786 in k22777 in k22774 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22788,c,av);}
/* library.scm:4245: ##sys#read-bytevector-literal */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[844]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[844]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22803,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4251: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k22804 in k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_22806,c,av);}
a=C_alloc(22);
t2=C_SCHEME_FALSE;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22811,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=((C_word)li683),tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22816,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word)li685),tmp=(C_word)a,a+=6,tmp);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22851,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word)li686),tmp=(C_word)a,a+=6,tmp);
/* library.scm:4252: ##sys#dynamic-wind */
t9=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[5];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a22810 in k22804 in k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22811,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a22815 in k22804 in k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22816,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22822,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li684),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4253: r-xtoken */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21734(t3,t1,t2);}

/* a22821 in a22815 in k22804 in k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22822,c,av);}
t4=C_block_size(t2);
t5=C_eqp(C_fix(0),t4);
if(C_truep(t5)){
t6=C_i_char_equalp(((C_word*)t0)[2],C_make_character(124));
if(C_truep(C_i_not(t6))){
/* library.scm:4257: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2=av;
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[846];
tp(4,av2);}}
else{
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}
else{
/* library.scm:4141: ##sys#intern-keyword */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[630]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[630]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}}

/* a22850 in k22804 in k22801 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22851,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k22861 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_22863,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22866,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4261: ##sys#port-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[748]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[748]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k22864 in k22861 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22866(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22866,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22869,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4262: readrec */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20429(t3,t2);}

/* k22867 in k22864 in k22861 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_22869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_22869,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22884,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4264: readrec */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20429(t3,t2);}

/* k22882 in k22867 in k22864 in k22861 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_22884,c,av);}
a=C_alloc(15);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_list3(&a,3,lf[847],t2,lf[848]);
/* library.scm:4263: info */
t4=((C_word*)((C_word*)t0)[3])[1];
f_20359(t4,((C_word*)t0)[4],lf[805],t3,((C_word*)t0)[5]);}

/* k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22893(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22893,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22896,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4268: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_22896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22896,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22902,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_charp(t1))){
t3=C_u_i_char_whitespacep(t1);
t4=t2;
f_22902(t4,(C_truep(t3)?t3:C_u_i_char_equalp(C_make_character(47),t1)));}
else{
t3=t2;
f_22902(t3,C_SCHEME_FALSE);}}

/* k22900 in k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_fcall f_22902(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_22902,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22905,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4271: skip-to-eol */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20368(t3,t2);}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22911,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4274: r-token */
t3=((C_word*)((C_word*)t0)[6])[1];
f_21599(t3,t2);}}

/* k22903 in k22900 in k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22905,c,av);}
/* library.scm:4272: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,((C_word*)t0)[3]);}

/* k22909 in k22900 in k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_22911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_22911,c,av);}
a=C_alloc(5);
if(C_truep(C_i_string_equal_p(lf[849],t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_FILE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep((C_truep(C_i_equalp(t1,lf[850]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(t1,lf[851]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(t1,lf[852]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22930,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4277: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t2;
av2[2]=lf[853];
av2[3]=t1;
tp(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22949,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4279: scheme#string->symbol */
t3=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k22928 in k22909 in k22900 in k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_22930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22930,c,av);}
/* library.scm:4138: ##sys#intern-symbol */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
C_string_to_symbol(3,av2);}}

/* k22947 in k22909 in k22900 in k22894 in k22891 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_22949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22949,c,av);}
t2=C_i_assq(t1,C_fast_retrieve(lf[854]));
if(C_truep(t2)){
/* library.scm:4280: g4898 */
t3=C_slot(t2,C_fix(1));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}
else{
/* library.scm:4282: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[855];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}}

/* a22964 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22965(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22965,c,av);}
/* library.scm:4286: ##sys#user-read-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[856]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[856]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* a22970 in k22426 in k22423 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_22971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22971,c,av);}
if(C_truep(C_rest_nullp(c,2))){
/* library.scm:4287: readrec */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20429(t2,t1);}
else{
t2=t1;{
C_word av2[2];
av2[0]=t2;
av2[1]=C_get_rest_arg(c,2,av,2,t0);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k23041 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_23043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23043,c,av);}
/* library.scm:4289: container */
t2=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* k23053 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_23055(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23055,c,av);}
/* library.scm:4290: r-string */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20577(t2,((C_word*)t0)[3],C_make_character(34));}

/* k23131 in k22329 in k22326 in k22323 in readrec in k20349 in k20346 in k20343 in k20340 in k20337 in ##sys#read in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_23133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23133,c,av);}
/* library.scm:4302: container */
t2=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* ##sys#char->utf8-string in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23209(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_23209,c,av);}
a=C_alloc(5);
t3=C_fix(C_character_code(t2));
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(127)))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_string(&a,1,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(2047)))){
t4=C_fixnum_shift_right(t3,C_fix(6));
t5=C_fixnum_or(C_fix(192),t4);
t6=C_make_character(C_unfix(t5));
t7=C_fixnum_and(t3,C_fix(63));
t8=C_fixnum_or(C_fix(128),t7);
t9=C_make_character(C_unfix(t8));
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_a_i_string(&a,2,t6,t9);
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(65535)))){
t4=C_fixnum_shift_right(t3,C_fix(12));
t5=C_fixnum_or(C_fix(224),t4);
t6=C_make_character(C_unfix(t5));
t7=C_fixnum_shift_right(t3,C_fix(6));
t8=C_fixnum_and(t7,C_fix(63));
t9=C_fixnum_or(C_fix(128),t8);
t10=C_make_character(C_unfix(t9));
t11=C_fixnum_and(t3,C_fix(63));
t12=C_fixnum_or(C_fix(128),t11);
t13=C_make_character(C_unfix(t12));
t14=t1;{
C_word *av2=av;
av2[0]=t14;
av2[1]=C_a_i_string(&a,3,t6,t10,t13);
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(2097151)))){
t4=C_fixnum_shift_right(t3,C_fix(18));
t5=C_fixnum_or(C_fix(240),t4);
t6=C_make_character(C_unfix(t5));
t7=C_fixnum_shift_right(t3,C_fix(12));
t8=C_fixnum_and(t7,C_fix(63));
t9=C_fixnum_or(C_fix(128),t8);
t10=C_make_character(C_unfix(t9));
t11=C_fixnum_shift_right(t3,C_fix(6));
t12=C_fixnum_and(t11,C_fix(63));
t13=C_fixnum_or(C_fix(128),t12);
t14=C_make_character(C_unfix(t13));
t15=C_fixnum_and(t3,C_fix(63));
t16=C_fixnum_or(C_fix(128),t15);
t17=C_make_character(C_unfix(t16));
t18=t1;{
C_word *av2=av;
av2[0]=t18;
av2[1]=C_a_i_string(&a,4,t6,t10,t14,t17);
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
/* library.scm:4327: chicken.base#error */
t4=*((C_word*)lf[193]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[859];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}}}}

/* ##sys#unicode-surrogate? in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23374,c,av);}
t3=C_fixnum_less_or_equal_p(C_fix(55296),t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_fixnum_less_or_equal_p(t2,C_fix(57343)):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#surrogates->codepoint in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23386,c,av);}
if(C_truep(C_fixnum_less_or_equal_p(C_fix(55296),t2))){
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(56319)))){
if(C_truep(C_fixnum_less_or_equal_p(C_fix(56320),t3))){
if(C_truep(C_fixnum_less_or_equal_p(t3,C_fix(57343)))){
t4=C_fixnum_shift_right(t2,C_fix(6));
t5=C_fixnum_and(t4,C_fix(31));
t6=C_fixnum_plus(C_fix(1),t5);
t7=C_fixnum_shift_left(t6,C_fix(16));
t8=C_fixnum_and(t2,C_fix(63));
t9=C_fixnum_shift_left(t8,C_fix(10));
t10=C_fixnum_and(t3,C_fix(1023));
t11=t1;{
C_word *av2=av;
av2[0]=t11;
av2[1]=C_fixnum_or(t7,C_fixnum_or(t9,t10));
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23448(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_23448,c,av);}
a=C_alloc(12);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23451,a[2]=t2,a[3]=((C_word)li694),tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23497,a[2]=t2,a[3]=t5,a[4]=t3,a[5]=((C_word)li695),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_23497(t7,t1,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}

/* hex in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_fcall f_23451(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_23451,3,t0,t1,t2);}
t3=C_u_i_char_downcase(t2);
t4=(C_truep(C_u_i_char_greater_or_equal_p(t3,C_make_character(97)))?C_u_i_char_less_or_equal_p(t3,C_make_character(102)):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_fixnum_difference(C_fix(C_character_code(t3)),C_fix(87));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(C_truep(C_u_i_char_greater_or_equal_p(t3,C_make_character(48)))?C_u_i_char_less_or_equal_p(t3,C_make_character(57)):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_fixnum_difference(C_fix(C_character_code(t3)),C_fix(48));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* library.scm:4347: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word av2[4];
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[860];
tp(4,av2);}}}}

/* loop in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_fcall f_23497(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_23497,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23501,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4349: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k23499 in loop in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_23501,c,av);}
a=C_alloc(6);
if(C_truep(C_eofp(t1))){
/* library.scm:4351: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[861];
tp(4,av2);}}
else{
if(C_truep(C_i_char_equalp(C_make_character(125),t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23519,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=C_fixnum_shift_right(((C_word*)t0)[4],C_fix(4));
t4=C_make_character(C_unfix(t3));
t5=C_a_i_cons(&a,2,t4,((C_word*)t0)[5]);
/* library.scm:4353: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t2;
av2[2]=t5;
tp(3,av2);}}
else{
/* library.scm:4353: ##sys#reverse-list->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[391]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[391]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}}
else{
if(C_truep(C_u_i_char_whitespacep(t1))){
if(C_truep(((C_word*)t0)[4])){
t2=C_fixnum_shift_right(((C_word*)t0)[4],C_fix(4));
t3=C_make_character(C_unfix(t2));
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[5]);
/* library.scm:4361: loop */
t5=((C_word*)((C_word*)t0)[6])[1];
f_23497(t5,((C_word*)t0)[2],t4,C_SCHEME_FALSE);}
else{
/* library.scm:4362: loop */
t2=((C_word*)((C_word*)t0)[6])[1];
f_23497(t2,((C_word*)t0)[2],((C_word*)t0)[5],((C_word*)t0)[4]);}}
else{
if(C_truep(((C_word*)t0)[4])){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_23583,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4363: hex */
t3=((C_word*)t0)[7];
f_23451(t3,t2,t1);}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23594,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4364: hex */
t3=((C_word*)t0)[7];
f_23451(t3,t2,t1);}}}}}

/* k23517 in k23499 in loop in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_23519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23519,c,av);}
t2=C_string_to_bytevector(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k23581 in k23499 in loop in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_23583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_23583,c,av);}
a=C_alloc(3);
t2=C_fixnum_or(((C_word*)t0)[2],t1);
t3=C_make_character(C_unfix(t2));
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* library.scm:4363: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_23497(t5,((C_word*)t0)[5],t4,C_SCHEME_FALSE);}

/* k23592 in k23499 in loop in ##sys#read-bytevector-literal in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_23594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23594,c,av);}
/* library.scm:4364: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23497(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_fixnum_shift_left(t1,C_fix(4)));}

/* ##sys#user-read-hook in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_23596,c,av);}
a=C_alloc(3);
t4=C_eqp(t2,C_make_character(102));
t5=(C_truep(t4)?t4:C_eqp(t2,C_make_character(70)));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23609,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4376: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2=av;
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t6;
av2[2]=t3;
tp(3,av2);}}
else{
t6=C_eqp(t2,C_make_character(116));
t7=(C_truep(t6)?t6:C_eqp(t2,C_make_character(84)));
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23621,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4377: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2=av;
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t8;
av2[2]=t3;
tp(3,av2);}}
else{
/* library.scm:4378: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=t1;
av2[2]=t3;
av2[3]=lf[862];
av2[4]=t2;
tp(5,av2);}}}}

/* k23607 in ##sys#user-read-hook in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23609,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k23619 in ##sys#user-read-hook in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23621,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#write-char-0 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_23641,c,av);}
a=C_alloc(3);
t4=C_slot(t3,C_fix(2));
t5=C_slot(t4,C_fix(2));
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23648,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4392: g5010 */
t7=t5;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
av2[3]=t2;
((C_proc)C_fast_retrieve_proc(t7))(4,av2);}}

/* k23646 in ##sys#write-char-0 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23648,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=*((C_word*)lf[246]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#write-char/port in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23657,c,av);}
t4=C_i_check_port_2(t3,C_fix(2),C_SCHEME_TRUE,lf[865]);
t5=C_i_check_char_2(t2,lf[865]);
/* library.scm:4399: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2=av;
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
tp(4,av2);}}

/* scheme#write-char in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23669,c,av);}
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?*((C_word*)lf[714]+1):C_get_rest_arg(c,3,av,3,t0));
t5=C_i_check_char_2(t2,lf[865]);
t6=C_i_check_port_2(t4,C_fix(2),C_SCHEME_TRUE,lf[865]);
/* library.scm:4405: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word av2[4];
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=t4;
tp(4,av2);}}

/* scheme#newline in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23690,c,av);}
if(C_truep(C_rest_nullp(c,2))){
t2=*((C_word*)lf[714]+1);
t3=*((C_word*)lf[714]+1);
/* library.scm:4409: ##sys#write-char/port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[864]);
C_word av2[4];
av2[0]=*((C_word*)lf[864]+1);
av2[1]=t1;
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[714]+1);
tp(4,av2);}}
else{
/* library.scm:4409: ##sys#write-char/port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[864]);
C_word av2[4];
av2[0]=*((C_word*)lf[864]+1);
av2[1]=t1;
av2[2]=C_make_character(10);
av2[3]=C_get_rest_arg(c,2,av,2,t0);
tp(4,av2);}}}

/* scheme#write in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_23705,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=*((C_word*)lf[714]+1);
t4=C_i_check_port_2(*((C_word*)lf[714]+1),C_fix(2),C_SCHEME_TRUE,lf[866]);
/* library.scm:4414: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[714]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_i_check_port_2(t3,C_fix(2),C_SCHEME_TRUE,lf[866]);
/* library.scm:4414: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* scheme#display in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_23723,c,av);}
if(C_truep(C_rest_nullp(c,3))){
t3=*((C_word*)lf[714]+1);
t4=C_i_check_port_2(*((C_word*)lf[714]+1),C_fix(2),C_SCHEME_TRUE,lf[868]);
/* library.scm:4419: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[714]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_get_rest_arg(c,3,av,3,t0);
t4=C_i_check_port_2(t3,C_fix(2),C_SCHEME_TRUE,lf[868]);
/* library.scm:4419: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* chicken.base#print in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_23741,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+9);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
t3=*((C_word*)lf[714]+1);
t4=C_i_check_port_2(*((C_word*)lf[714]+1),C_fix(2),C_SCHEME_TRUE,lf[869]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23784,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=C_i_check_list_2(t2,lf[687]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23760,a[2]=t8,a[3]=((C_word)li704),tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_23760(t10,t5,t2);}

/* for-each-loop5071 in chicken.base#print in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_fcall f_23760(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_23760,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23770,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[867]+1);
/* library.scm:4422: g5087 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[867]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[867]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[714]+1);
tp(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k23768 in for-each-loop5071 in chicken.base#print in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23770,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_23760(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k23782 in chicken.base#print in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_23784,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23787,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4428: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[714]+1);
tp(4,av2);}}

/* k23785 in k23782 in chicken.base#print in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23787(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23787,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=*((C_word*)lf[246]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#print* in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_23792,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+9);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
t3=*((C_word*)lf[714]+1);
t4=C_i_check_port_2(*((C_word*)lf[714]+1),C_fix(2),C_SCHEME_TRUE,lf[869]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23835,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=C_i_check_list_2(t2,lf[687]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23811,a[2]=t8,a[3]=((C_word)li706),tmp=(C_word)a,a+=4,tmp));
t10=((C_word*)t8)[1];
f_23811(t10,t5,t2);}

/* for-each-loop5103 in chicken.base#print* in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_fcall f_23811(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_23811,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23821,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[867]+1);
/* library.scm:4422: g5119 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[867]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[867]+1);
av2[1]=t3;
av2[2]=t4;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[714]+1);
tp(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k23819 in for-each-loop5103 in chicken.base#print* in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23821,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_23811(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k23833 in chicken.base#print* in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_23835,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23838,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4435: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[746]+1);
av2[1]=t2;
av2[2]=*((C_word*)lf[714]+1);
tp(3,av2);}}

/* k23836 in k23833 in chicken.base#print* in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23838(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_23838,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=*((C_word*)lf[246]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in ... */
static void C_ccall f_23845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_23845,c,av);}
a=C_alloc(3);
t2=C_mutate(&lf[870] /* (set! current-print-length ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23849,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4439: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in ... */
static void C_ccall f_23849(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_23849,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[871]+1 /* (set! ##sys#print-length-limit ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23853,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4440: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in ... */
static void C_ccall f_23853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,5)))){
C_save_and_reclaim((void *)f_23853,c,av);}
a=C_alloc(17);
t2=C_mutate(&lf[872] /* (set! ##sys#print-exit ...) */,t1);
t3=*((C_word*)lf[232]+1);
t4=*((C_word*)lf[233]+1);
t5=C_mutate((C_word*)lf[867]+1 /* (set! ##sys#print ...) */,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23855,a[2]=t4,a[3]=t3,a[4]=((C_word)li721),tmp=(C_word)a,a+=5,tmp));
t6=C_mutate((C_word*)lf[894]+1 /* (set! ##sys#procedure->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24956,a[2]=((C_word)li722),tmp=(C_word)a,a+=3,tmp));
t7=C_set_block_item(lf[907] /* ##sys#record-printers */,0,C_SCHEME_END_OF_LIST);
t8=C_mutate((C_word*)lf[204]+1 /* (set! chicken.base#record-printer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24973,a[2]=((C_word)li723),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[205]+1 /* (set! chicken.base#set-record-printer! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24988,a[2]=((C_word)li724),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[910]+1 /* (set! ##sys#register-record-printer ...) */,*((C_word*)lf[205]+1));
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25017,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4689: chicken.base#getter-with-setter */
t12=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t11;
av2[2]=*((C_word*)lf[204]+1);
av2[3]=*((C_word*)lf[205]+1);
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}

/* ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_23855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23855,c,av);}
a=C_alloc(7);
t5=C_i_check_port_2(t4,C_fix(2),C_SCHEME_TRUE,C_SCHEME_FALSE);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23862,a[2]=t3,a[3]=t1,a[4]=t4,a[5]=t2,a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4447: case-sensitive */
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_23862(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23862,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23865,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4448: keyword-style */
t3=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_23865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_23865,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23868,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4449: ##sys#print-length-limit */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[871]);
C_word *av2=av;
av2[0]=*((C_word*)lf[871]+1);
av2[1]=t2;
tp(2,av2);}}

/* k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_23868(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(49,c,4)))){
C_save_and_reclaim((void *)f_23868,c,av);}
a=C_alloc(49);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23870,a[2]=t1,a[3]=((C_word)li708),tmp=(C_word)a,a+=4,tmp));
t13=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23942,a[2]=t1,a[3]=((C_word)li709),tmp=(C_word)a,a+=4,tmp));
t14=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24002,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t9,a[5]=t11,a[6]=((C_word)li710),tmp=(C_word)a,a+=7,tmp));
t15=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24026,a[2]=t5,a[3]=t3,a[4]=((C_word)li712),tmp=(C_word)a,a+=5,tmp));
t16=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_24115,a[2]=((C_word*)t0)[3],a[3]=((C_word)li714),tmp=(C_word)a,a+=4,tmp));
t17=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24304,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_24309,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=t5,a[6]=((C_word*)t0)[6],a[7]=t7,a[8]=t19,a[9]=((C_word)li720),tmp=(C_word)a,a+=10,tmp));
t21=((C_word*)t19)[1];
f_24309(t21,t17,((C_word*)t0)[7]);}

/* outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_23870(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_23870,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(((C_word*)t0)[2])){
t4=C_block_size(t3);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23880,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4455: current-print-length */
t6=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(2));
/* g51625163 */
t5=C_slot(t4,C_fix(3));{
C_word av2[4];
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}}

/* k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_23880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_23880,c,av);}
a=C_alloc(12);
t2=C_fixnum_plus(t1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23886,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fixnum_greaterp(t2,((C_word*)t0)[4]))){
t4=C_fixnum_difference(((C_word*)t0)[4],t1);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23898,a[2]=t3,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fixnum_greaterp(t4,C_fix(0)))){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_23921,a[2]=((C_word*)t0)[5],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4459: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[6];
av2[3]=C_fix(0);
av2[4]=t4;
tp(5,av2);}}
else{
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_23898(2,av2);}}}
else{
t4=C_slot(((C_word*)t0)[5],C_fix(2));
/* g51625163 */
t5=C_slot(t4,C_fix(3));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}}

/* k23884 in k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_23886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23886,c,av);}
/* library.scm:4463: current-print-length */
t2=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k23896 in k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_23898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_23898,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23901,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_slot(((C_word*)t0)[3],C_fix(2));
/* g51625163 */
t4=C_slot(t3,C_fix(3));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[873];
((C_proc)C_fast_retrieve_proc(t4))(4,av2);}}

/* k23899 in k23896 in k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_23901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_23901,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23904,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4461: ##sys#print-exit */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k23902 in k23899 in k23896 in k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_23904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23904,c,av);}
t2=*((C_word*)lf[246]+1);
/* library.scm:4458: g5155 */
t3=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[246]+1);
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}

/* k23919 in k23878 in outstr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_23921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23921,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(2));
/* g51625163 */
t3=C_slot(t2,C_fix(3));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[2];
av2[3]=t1;
((C_proc)C_fast_retrieve_proc(t3))(4,av2);}}

/* outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_23942(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_23942,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_23946,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[2])){
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23959,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=t2,a[5]=t1,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4471: current-print-length */
t6=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=C_slot(t2,C_fix(2));
/* g51715172 */
t6=C_slot(t5,C_fix(2));{
C_word av2[4];
av2[0]=t6;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)C_fast_retrieve_proc(t6))(4,av2);}}}

/* k23944 in outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_23946(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_23946,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(2));
/* g51715172 */
t3=C_slot(t2,C_fix(2));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t3))(4,av2);}}

/* k23957 in outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_23959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_23959,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23962,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4472: current-print-length */
t3=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fixnum_plus(t1,C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k23960 in k23957 in outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_23962(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_23962,c,av);}
a=C_alloc(3);
if(C_truep(C_fixnum_greater_or_equal_p(((C_word*)t0)[2],((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23971,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t3=C_slot(((C_word*)t0)[5],C_fix(2));
/* g51625163 */
t4=C_slot(t3,C_fix(3));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[874];
((C_proc)C_fast_retrieve_proc(t4))(4,av2);}}
else{
t2=C_slot(((C_word*)t0)[5],C_fix(2));
/* g51715172 */
t3=C_slot(t2,C_fix(2));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[6];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[7];
((C_proc)C_fast_retrieve_proc(t3))(4,av2);}}}

/* k23969 in k23960 in k23957 in outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_23971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_23971,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_23974,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4475: ##sys#print-exit */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k23972 in k23969 in k23960 in k23957 in outchr in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_23974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_23974,c,av);}
t2=*((C_word*)lf[246]+1);
/* library.scm:4473: g5167 */
t3=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[246]+1);
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}

/* outsym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_24002(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_24002,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24006,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4484: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k24004 in outsym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_24006,c,av);}
a=C_alloc(7);
t2=C_i_not(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24015,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
f_24015(2,av2);}}
else{
/* library.scm:4485: sym-is-readable? */
t4=((C_word*)((C_word*)t0)[7])[1];
f_24115(t4,t3,t1);}}

/* k24013 in k24004 in outsym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24015,c,av);}
if(C_truep(t1)){
/* library.scm:4486: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}
else{
/* library.scm:4487: outreadablesym */
t2=((C_word*)((C_word*)t0)[6])[1];
f_24026(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}}

/* outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_24026(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_24026,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=C_block_size(t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24033,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[3],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
/* library.scm:4491: outchr */
t6=((C_word*)((C_word*)t0)[2])[1];
f_23942(t6,t5,t2,C_make_character(124));}

/* k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24033(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_24033,c,av);}
a=C_alloc(11);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24038,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t3,a[7]=((C_word*)t0)[6],a[8]=((C_word)li711),tmp=(C_word)a,a+=9,tmp));
t5=((C_word*)t3)[1];
f_24038(t5,((C_word*)t0)[7],C_fix(0));}

/* loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_24038(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_24038,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
/* library.scm:4494: outchr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_23942(t3,t1,((C_word*)t0)[4],C_make_character(124));}
else{
t3=C_subchar(((C_word*)t0)[5],t2);
t4=C_i_char_lessp(t3,C_make_character(32));
t5=(C_truep(t4)?t4:C_u_i_char_greaterp(t3,C_make_character(126)));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24060,a[2]=t3,a[3]=((C_word*)t0)[6],a[4]=t1,a[5]=t2,a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[3],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4497: outstr */
t7=((C_word*)((C_word*)t0)[7])[1];
f_23870(t7,t6,((C_word*)t0)[4],lf[875]);}
else{
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24089,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
t7=C_eqp(t3,C_make_character(124));
if(C_truep(t7)){
if(C_truep(t7)){
/* library.scm:4503: outchr */
t8=((C_word*)((C_word*)t0)[3])[1];
f_23942(t8,t6,((C_word*)t0)[4],C_make_character(92));}
else{
t8=t6;{
C_word av2[2];
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_24089(2,av2);}}}
else{
t8=C_eqp(t3,C_make_character(92));
if(C_truep(t8)){
/* library.scm:4503: outchr */
t9=((C_word*)((C_word*)t0)[3])[1];
f_23942(t9,t6,((C_word*)t0)[4],C_make_character(92));}
else{
t9=t6;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_UNDEFINED;
f_24089(2,av2);}}}}}}

/* k24058 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24060(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_24060,c,av);}
a=C_alloc(8);
t2=C_fix(C_character_code(((C_word*)t0)[2]));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24066,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_fixnum_lessp(t2,C_fix(16)))){
/* library.scm:4499: outchr */
t4=((C_word*)((C_word*)t0)[8])[1];
f_23942(t4,t3,((C_word*)t0)[7],C_make_character(48));}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_24066(2,av2);}}}

/* k24064 in k24058 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24066(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_24066,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24069,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24080,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4500: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=C_fix(16);
tp(4,av2);}}

/* k24067 in k24064 in k24058 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24069,c,av);}
/* library.scm:4501: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_24038(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k24078 in k24064 in k24058 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24080,c,av);}
/* library.scm:4500: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24087 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24089(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_24089,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24092,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4504: outchr */
t3=((C_word*)((C_word*)t0)[5])[1];
f_23942(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k24090 in k24087 in loop in k24031 in outreadablesym in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24092,c,av);}
/* library.scm:4505: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_24038(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* sym-is-readable? in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_24115(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_24115,3,t0,t1,t2);}
a=C_alloc(9);
t3=C_block_size(t2);
switch(t3){
case C_fix(0):
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}
case C_fix(1):
t4=C_subchar(t2,C_fix(0));
t5=C_eqp(C_make_character(35),t4);
t6=(C_truep(t5)?t5:C_eqp(C_make_character(46),t4));
if(C_truep(t6)){
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t7=C_fix(C_character_code(t4));
t8=C_fixnum_less_or_equal_p(t7,C_fix(32));
t9=(C_truep(t8)?t8:C_i_memq(t4,lf[876]));
if(C_truep(t9)){
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t10=C_u_i_char_numericp(t4);
t11=t1;{
C_word av2[2];
av2[0]=t11;
av2[1]=(C_truep(t10)?C_SCHEME_FALSE:C_SCHEME_TRUE);
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}
default:
t4=C_fixnum_difference(t3,C_fix(1));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24164,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t6,a[6]=((C_word)li713),tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_24164(t8,t1,t4);}}

/* loop in sym-is-readable? in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_24164(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_24164,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_eqp(t2,C_fix(0));
if(C_truep(t3)){
t4=C_subchar(((C_word*)t0)[2],C_fix(0));
t5=C_u_i_char_numericp(t4);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24180,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t4,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t5)){
t7=t6;
f_24180(t7,t5);}
else{
t7=C_eqp(t4,C_make_character(43));
if(C_truep(t7)){
t8=t6;
f_24180(t8,t7);}
else{
t8=C_eqp(t4,C_make_character(46));
t9=t6;
f_24180(t9,(C_truep(t8)?t8:C_eqp(t4,C_make_character(45))));}}}
else{
t4=C_subchar(((C_word*)t0)[2],t2);
t5=(C_truep(((C_word*)t0)[4])?((C_word*)t0)[4]:C_i_not(C_u_i_char_upper_casep(t4)));
if(C_truep(t5)){
t6=C_fix(C_character_code(t4));
t7=C_fixnum_less_or_equal_p(t6,C_fix(32));
t8=(C_truep(t7)?t7:C_i_memq(t4,lf[876]));
if(C_truep(C_i_not(t8))){
t9=C_eqp(t4,C_make_character(58));
t10=C_i_not(t9);
t11=(C_truep(t10)?t10:C_fixnum_lessp(t2,C_fixnum_difference(((C_word*)t0)[3],C_fix(1))));
if(C_truep(t11)){
/* library.scm:4539: loop */
t13=t1;
t14=C_fixnum_difference(t2,C_fix(1));
t1=t13;
t2=t14;
goto loop;}
else{
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}}
else{
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}

/* k24178 in loop in sym-is-readable? in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_24180(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_24180,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24187,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4524: ##sys#string->number */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[622]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[622]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],C_make_character(58));
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24199,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=C_eqp(((C_word*)t0)[4],C_make_character(35));
if(C_truep(t4)){
if(C_truep(C_fixnum_greaterp(((C_word*)t0)[5],C_fix(2)))){
t5=C_eqp(C_subchar(((C_word*)t0)[3],C_fix(1)),C_make_character(35));
if(C_truep(t5)){
t6=C_eqp(C_subchar(((C_word*)t0)[3],C_fix(2)),C_make_character(35));
t7=t3;
f_24199(t7,C_i_not(C_i_not(t6)));}
else{
t6=t3;
f_24199(t6,C_SCHEME_TRUE);}}
else{
t5=t3;
f_24199(t5,C_SCHEME_TRUE);}}
else{
t5=t3;
f_24199(t5,C_SCHEME_FALSE);}}}}

/* k24185 in k24178 in loop in sym-is-readable? in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24187,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k24197 in k24178 in loop in sym-is-readable? in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_24199(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,1)))){
C_save_and_reclaim_args((void *)trf_24199,2,t0,t1);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_i_member(((C_word*)t0)[3],lf[877]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_fix(C_character_code(((C_word*)t0)[4]));
t3=C_fixnum_less_or_equal_p(t2,C_fix(32));
t4=(C_truep(t3)?t3:C_i_memq(((C_word*)t0)[4],lf[876]));
t5=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?C_SCHEME_FALSE:C_SCHEME_TRUE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k24302 in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_24304(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24304,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=*((C_word*)lf[246]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_24309(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_24309,3,t0,t1,t2);}
a=C_alloc(9);
switch(t2){
case C_SCHEME_END_OF_LIST:
/* library.scm:4542: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[878]);
case C_SCHEME_TRUE:
/* library.scm:4543: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[879]);
case C_SCHEME_FALSE:
/* library.scm:4544: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[880]);
default:
if(C_truep(C_eofp(t2))){
/* library.scm:4545: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[881]);}
else{
if(C_truep(C_undefinedp(t2))){
/* library.scm:4546: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[882]);}
else{
if(C_truep(C_charp(t2))){
if(C_truep(((C_word*)t0)[4])){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24358,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4549: outstr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_23870(t4,t3,((C_word*)t0)[3],lf[883]);}
else{
/* library.scm:4558: outchr */
t3=((C_word*)((C_word*)t0)[5])[1];
f_23942(t3,t1,((C_word*)t0)[3],t2);}}
else{
if(C_truep(C_fixnump(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24417,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4559: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}
else{
if(C_truep(C_unboundvaluep(t2))){
/* library.scm:4560: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[884]);}
else{
if(C_truep(C_i_not(C_blockp(t2)))){
/* library.scm:4561: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[885]);}
else{
if(C_truep(C_forwardedp(t2))){
/* library.scm:4562: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],lf[886]);}
else{
if(C_truep(C_i_keywordp(t2))){
t3=C_i_not(((C_word*)t0)[4]);
t4=(C_truep(t3)?((C_word*)t0)[6]:C_SCHEME_FALSE);
t5=C_eqp(t4,lf[830]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24453,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4567: outchr */
t7=((C_word*)((C_word*)t0)[5])[1];
f_23942(t7,t6,((C_word*)t0)[3],C_make_character(58));}
else{
t6=C_eqp(t4,lf[811]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24465,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4570: outsym */
t8=((C_word*)((C_word*)t0)[7])[1];
f_24002(t8,t7,((C_word*)t0)[3],t2);}
else{
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24471,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4573: outstr */
t8=((C_word*)((C_word*)t0)[2])[1];
f_23870(t8,t7,((C_word*)t0)[3],lf[887]);}}}
else{
if(C_truep(C_i_symbolp(t2))){
/* library.scm:4575: outsym */
t3=((C_word*)((C_word*)t0)[7])[1];
f_24002(t3,t1,((C_word*)t0)[3],t2);}
else{
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24493,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4576: ##sys#number? */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[499]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[499]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}}}}}}}}}}}

/* k24356 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24358(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_24358,c,av);}
a=C_alloc(8);
t2=C_fix(C_character_code(((C_word*)t0)[2]));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24364,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4551: chicken.base#char-name */
t4=*((C_word*)lf[199]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k24362 in k24356 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24364(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_24364,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24368,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li715),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4551: g5269 */
t3=t2;
f_24368(t3,((C_word*)t0)[4],t1);}
else{
t2=C_fixnum_lessp(((C_word*)t0)[5],C_fix(32));
t3=(C_truep(t2)?t2:C_fixnum_greaterp(((C_word*)t0)[5],C_fix(255)));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24391,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4555: outchr */
t5=((C_word*)((C_word*)t0)[6])[1];
f_23942(t5,t4,((C_word*)t0)[3],C_make_character(120));}
else{
/* library.scm:4557: outchr */
t4=((C_word*)((C_word*)t0)[6])[1];
f_23942(t4,((C_word*)t0)[4],((C_word*)t0)[3],((C_word*)t0)[7]);}}}

/* g5269 in k24362 in k24356 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_24368(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_24368,3,t0,t1,t2);}
/* library.scm:4553: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t1,((C_word*)t0)[3],C_slot(t2,C_fix(1)));}

/* k24389 in k24362 in k24356 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_24391,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24398,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4556: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_fix(16);
tp(4,av2);}}

/* k24396 in k24389 in k24362 in k24356 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24398,c,av);}
/* library.scm:4556: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24415 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24417,c,av);}
/* library.scm:4559: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24451 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24453,c,av);}
/* library.scm:4568: outsym */
t2=((C_word*)((C_word*)t0)[2])[1];
f_24002(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k24463 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24465,c,av);}
/* library.scm:4571: outchr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23942(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(58));}

/* k24469 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24471,c,av);}
/* library.scm:4574: outsym */
t2=((C_word*)((C_word*)t0)[2])[1];
f_24002(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_24493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_24493,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24500,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4576: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}
else{
if(C_truep(C_anypointerp(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24510,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4577: ##sys#pointer->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[888]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[888]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}
else{
if(C_truep(C_stringp(((C_word*)t0)[5]))){
if(C_truep(((C_word*)t0)[6])){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24519,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4580: outchr */
t3=((C_word*)((C_word*)t0)[7])[1];
f_23942(t3,t2,((C_word*)t0)[4],C_make_character(34));}
else{
/* library.scm:4606: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}}
else{
if(C_truep(C_pairp(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24692,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4608: outchr */
t3=((C_word*)((C_word*)t0)[7])[1];
f_23942(t3,t2,((C_word*)t0)[4],C_make_character(40));}
else{
if(C_truep(C_bytevectorp(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24765,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4620: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t2,((C_word*)t0)[4],lf[892]);}
else{
if(C_truep(C_structurep(((C_word*)t0)[5]))){
/* library.scm:4629: ##sys#user-print-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[893]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[893]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}
else{
if(C_truep(C_closurep(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24825,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4630: ##sys#procedure->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[894]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[894]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}
else{
if(C_truep(C_locativep(((C_word*)t0)[5]))){
/* library.scm:4631: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],lf[895]);}
else{
if(C_truep(C_lambdainfop(((C_word*)t0)[5]))){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24837,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4633: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t2,((C_word*)t0)[4],lf[896]);}
else{
if(C_truep(C_portp(((C_word*)t0)[5]))){
t2=C_slot(((C_word*)t0)[5],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24856,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
switch(t2){
case C_fix(1):
/* library.scm:4638: outstr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_23870(t4,t3,((C_word*)t0)[4],lf[898]);
case C_fix(2):
/* library.scm:4639: outstr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_23870(t4,t3,((C_word*)t0)[4],lf[899]);
default:
/* library.scm:4640: outstr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_23870(t4,t3,((C_word*)t0)[4],lf[900]);}}
else{
if(C_truep(C_vectorp(((C_word*)t0)[5]))){
t2=C_block_size(((C_word*)t0)[5]);
t3=C_eqp(C_fix(0),t2);
if(C_truep(t3)){
/* library.scm:4646: outstr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_23870(t4,((C_word*)t0)[3],((C_word*)t0)[4],lf[901]);}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24903,a[2]=t2,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4648: outstr */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23870(t5,t4,((C_word*)t0)[4],lf[902]);}}
else{
/* library.scm:4656: ##sys#error */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[903];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}}}}}}}}}}}

/* k24498 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24500,c,av);}
/* library.scm:4576: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24508 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24510,c,av);}
/* library.scm:4577: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_24519,c,av);}
a=C_alloc(10);
t2=C_block_size(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24525,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],a[5]=t4,a[6]=((C_word*)t0)[5],a[7]=((C_word)li716),tmp=(C_word)a,a+=8,tmp));
t6=((C_word*)t4)[1];
f_24525(t6,((C_word*)t0)[6],C_fix(0),t2);}

/* doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_24525(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_24525,4,t0,t1,t2,t3);}
a=C_alloc(13);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
/* library.scm:4584: outchr */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23942(t5,t1,((C_word*)t0)[3],C_make_character(34));}
else{
t5=C_subbyte(((C_word*)t0)[4],t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24538,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
switch(t5){
case C_fix(34):
/* library.scm:4587: outstr */
t7=((C_word*)((C_word*)t0)[6])[1];
f_23870(t7,t6,((C_word*)t0)[3],lf[889]);
case C_fix(92):
/* library.scm:4588: outstr */
t7=((C_word*)((C_word*)t0)[6])[1];
f_23870(t7,t6,((C_word*)t0)[3],lf[890]);
default:
t7=C_fixnum_lessp(t5,C_fix(32));
t8=(C_truep(t7)?t7:C_eqp(t5,C_fix(127)));
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24576,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t6,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4592: outchr */
t10=((C_word*)((C_word*)t0)[2])[1];
f_23942(t10,t9,((C_word*)t0)[3],C_make_character(92));}
else{
/* library.scm:4605: outchr */
t9=((C_word*)((C_word*)t0)[2])[1];
f_23942(t9,t6,((C_word*)t0)[3],C_fix_to_char(t5));}}}}

/* k24536 in doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24538(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24538,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_24525(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),C_fixnum_difference(((C_word*)t0)[5],C_fix(1)));}

/* k24574 in doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_24576,c,av);}
a=C_alloc(7);
switch(((C_word*)t0)[2]){
case C_fix(7):
/* library.scm:4594: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(97));
case C_fix(8):
/* library.scm:4595: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(98));
case C_fix(9):
/* library.scm:4596: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(116));
case C_fix(10):
/* library.scm:4597: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(110));
case C_fix(11):
/* library.scm:4598: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(118));
case C_fix(12):
/* library.scm:4599: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(102));
case C_fix(13):
/* library.scm:4600: outchr */
t2=((C_word*)((C_word*)t0)[3])[1];
f_23942(t2,((C_word*)t0)[4],((C_word*)t0)[5],C_make_character(114));
default:
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_24642,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4602: outchr */
t3=((C_word*)((C_word*)t0)[3])[1];
f_23942(t3,t2,((C_word*)t0)[5],C_make_character(120));}}

/* k24640 in k24574 in doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_24642,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24645,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnum_lessp(((C_word*)t0)[5],C_fix(16)))){
/* library.scm:4603: outchr */
t3=((C_word*)((C_word*)t0)[6])[1];
f_23942(t3,t2,((C_word*)t0)[4],C_make_character(48));}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_24645(2,av2);}}}

/* k24643 in k24640 in k24574 in doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_24645(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_24645,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24652,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4604: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_fix(16);
tp(4,av2);}}

/* k24650 in k24643 in k24640 in k24574 in doloop5290 in k24517 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_24652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24652,c,av);}
/* library.scm:4604: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_24692,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24695,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4609: out */
t3=((C_word*)((C_word*)t0)[5])[1];
f_24309(t3,t2,C_slot(((C_word*)t0)[2],C_fix(0)));}

/* k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24695(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_24695,c,av);}
a=C_alloc(10);
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24704,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t4,a[7]=((C_word)li717),tmp=(C_word)a,a+=8,tmp));
t6=((C_word*)t4)[1];
f_24704(t6,((C_word*)t0)[7],t2);}

/* doloop5320 in k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_24704(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_24704,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_not(C_blockp(t2));
t4=(C_truep(t3)?t3:C_i_not(C_pairp(t2)));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24717,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t6=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(C_i_not(t6))){
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24726,a[2]=((C_word*)t0)[4],a[3]=t5,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4614: outstr */
t8=((C_word*)((C_word*)t0)[5])[1];
f_23870(t8,t7,((C_word*)t0)[3],lf[891]);}
else{
/* library.scm:4616: outchr */
t7=((C_word*)((C_word*)t0)[2])[1];
f_23942(t7,t1,((C_word*)t0)[3],C_make_character(41));}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24736,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4617: outchr */
t6=((C_word*)((C_word*)t0)[2])[1];
f_23942(t6,t5,((C_word*)t0)[3],C_make_character(32));}}

/* k24715 in doloop5320 in k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24717,c,av);}
/* library.scm:4616: outchr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23942(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(41));}

/* k24724 in doloop5320 in k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24726,c,av);}
/* library.scm:4615: out */
t2=((C_word*)((C_word*)t0)[2])[1];
f_24309(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k24734 in doloop5320 in k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_24736,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24739,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4618: out */
t3=((C_word*)((C_word*)t0)[5])[1];
f_24309(t3,t2,C_slot(((C_word*)t0)[4],C_fix(0)));}

/* k24737 in k24734 in doloop5320 in k24693 in k24690 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_24739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24739,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_24704(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_24765,c,av);}
a=C_alloc(16);
t2=C_block_size(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24771,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_24776,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[3],a[8]=((C_word)li718),tmp=(C_word)a,a+=9,tmp));
t7=((C_word*)t5)[1];
f_24776(t7,t3,C_fix(0));}

/* k24769 in k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24771,c,av);}
/* library.scm:4628: outchr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23942(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(125));}

/* doloop5334 in k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_24776(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_24776,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_subbyte(((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24789,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t3,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_fixnum_lessp(t3,C_fix(16)))){
/* library.scm:4626: outchr */
t5=((C_word*)((C_word*)t0)[7])[1];
f_23942(t5,t4,((C_word*)t0)[6],C_make_character(48));}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_24789(2,av2);}}}}

/* k24787 in doloop5334 in k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_24789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_24789,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24792,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24803,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4627: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=C_fix(16);
tp(4,av2);}}

/* k24790 in k24787 in doloop5334 in k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_24792,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_24776(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k24801 in k24787 in doloop5334 in k24763 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24803(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24803,c,av);}
/* library.scm:4627: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24823 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24825,c,av);}
/* library.scm:4630: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24835 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_24837,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24840,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24847,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4634: ##sys#lambda-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[754]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[754]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}}

/* k24838 in k24835 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24840(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24840,c,av);}
/* library.scm:4635: outchr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23942(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_make_character(62));}

/* k24845 in k24835 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24847,c,av);}
/* library.scm:4634: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k24854 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24856(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_24856,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24859,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4641: outstr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_23870(t3,t2,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(3)));}

/* k24857 in k24854 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24859,c,av);}
/* library.scm:4642: outstr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_23870(t2,((C_word*)t0)[3],((C_word*)t0)[4],lf[897]);}

/* k24901 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_24903(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_24903,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24906,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4649: out */
t3=((C_word*)((C_word*)t0)[5])[1];
f_24309(t3,t2,C_slot(((C_word*)t0)[6],C_fix(0)));}

/* k24904 in k24901 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_24906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_24906,c,av);}
a=C_alloc(10);
t2=C_fixnum_difference(((C_word*)t0)[2],C_fix(1));
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24915,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t4,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word)li719),tmp=(C_word)a,a+=8,tmp));
t6=((C_word*)t4)[1];
f_24915(t6,((C_word*)t0)[7],C_fix(1),t2);}

/* doloop5358 in k24904 in k24901 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_24915(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_24915,4,t0,t1,t2,t3);}
a=C_alloc(8);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
/* library.scm:4653: outchr */
t5=((C_word*)((C_word*)t0)[2])[1];
f_23942(t5,t1,((C_word*)t0)[3],C_make_character(41));}
else{
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_24928,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4654: outchr */
t6=((C_word*)((C_word*)t0)[2])[1];
f_23942(t6,t5,((C_word*)t0)[3],C_make_character(32));}}

/* k24926 in doloop5358 in k24904 in k24901 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_24928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_24928,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_24931,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4655: out */
t3=((C_word*)((C_word*)t0)[6])[1];
f_24309(t3,t2,C_slot(((C_word*)t0)[7],((C_word*)t0)[4]));}

/* k24929 in k24926 in doloop5358 in k24904 in k24901 in k24491 in out in k23866 in k23863 in k23860 in ##sys#print in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_24931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_24931,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_24915(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),C_fixnum_difference(((C_word*)t0)[5],C_fix(1)));}

/* ##sys#procedure->string in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_24956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_24956,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24960,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4662: ##sys#lambda-info */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[753]);
C_word *av2=av;
av2[0]=*((C_word*)lf[753]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k24958 in ##sys#procedure->string in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_24960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_24960,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_24970,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:4664: ##sys#lambda-info->string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[754]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[754]+1);
av2[1]=t2;
av2[2]=t1;
tp(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[906];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k24968 in k24958 in ##sys#procedure->string in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_24970(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_24970,c,av);}
/* library.scm:4664: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[904];
av2[3]=t1;
av2[4]=lf[905];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.base#record-printer in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_24973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_24973,c,av);}
t3=C_i_check_symbol_2(t2,lf[908]);
t4=C_i_assq(t2,C_fast_retrieve(lf[907]));
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_i_cdr(t4):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.base#set-record-printer! in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_24988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_24988,c,av);}
a=C_alloc(5);
t4=C_i_check_symbol_2(t2,lf[909]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_24995,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4678: ##sys#check-closure */
t6=*((C_word*)lf[369]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
av2[3]=lf[909];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k24993 in chicken.base#set-record-printer! in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_24995(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_24995,c,av);}
a=C_alloc(6);
t2=C_i_assq(((C_word*)t0)[2],C_fast_retrieve(lf[907]));
if(C_truep(t2)){
t3=C_i_setslot(t2,C_fix(1),((C_word*)t0)[3]);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,t3,C_fast_retrieve(lf[907]));
t5=C_mutate((C_word*)lf[907]+1 /* (set! ##sys#record-printers ...) */,t4);
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in ... */
static void C_ccall f_25017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(93,c,6)))){
C_save_and_reclaim((void *)f_25017,c,av);}
a=C_alloc(93);
t2=C_mutate((C_word*)lf[204]+1 /* (set! chicken.base#record-printer ...) */,t1);
t3=C_mutate((C_word*)lf[893]+1 /* (set! ##sys#user-print-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25019,a[2]=((C_word)li732),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[921]+1 /* (set! ##sys#with-print-length-limit ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25170,a[2]=((C_word)li737),tmp=(C_word)a,a+=3,tmp));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25255,a[2]=t6,a[3]=((C_word)li738),tmp=(C_word)a,a+=4,tmp);
t8=C_set_block_item(t6,0,t7);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31657,a[2]=((C_word)li739),tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31682,a[2]=((C_word)li740),tmp=(C_word)a,a+=3,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31700,a[2]=t6,a[3]=((C_word)li741),tmp=(C_word)a,a+=4,tmp);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31719,a[2]=t6,a[3]=((C_word)li742),tmp=(C_word)a,a+=4,tmp);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31738,a[2]=((C_word)li743),tmp=(C_word)a,a+=3,tmp);
t14=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31741,a[2]=((C_word)li744),tmp=(C_word)a,a+=3,tmp);
t15=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31744,a[2]=((C_word)li745),tmp=(C_word)a,a+=3,tmp);
t16=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31785,a[2]=((C_word)li749),tmp=(C_word)a,a+=3,tmp);
t17=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31869,a[2]=((C_word)li750),tmp=(C_word)a,a+=3,tmp);
t18=C_a_i_vector(&a,10,t9,t10,t11,t12,*((C_word*)lf[196]+1),t13,t14,t15,t16,t17);
t19=C_mutate((C_word*)lf[926]+1 /* (set! ##sys#string-port-class ...) */,t18);
t20=C_mutate((C_word*)lf[924]+1 /* (set! ##sys#scan-buffer-line ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25311,a[2]=((C_word)li757),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[214]+1 /* (set! chicken.base#open-input-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25508,a[2]=((C_word)li758),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[215]+1 /* (set! chicken.base#open-output-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25527,a[2]=((C_word)li759),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[213]+1 /* (set! chicken.base#get-output-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25546,a[2]=((C_word)li760),tmp=(C_word)a,a+=3,tmp));
t24=*((C_word*)lf[213]+1);
t25=*((C_word*)lf[215]+1);
t26=C_mutate((C_word*)lf[936]+1 /* (set! ##sys#print-to-string ...) */,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25580,a[2]=t24,a[3]=t25,a[4]=((C_word)li763),tmp=(C_word)a,a+=5,tmp));
t27=C_mutate((C_word*)lf[888]+1 /* (set! ##sys#pointer->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25623,a[2]=((C_word)li764),tmp=(C_word)a,a+=3,tmp));
t28=C_mutate((C_word*)lf[186]+1 /* (set! chicken.base#get-call-chain ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25676,a[2]=((C_word)li766),tmp=(C_word)a,a+=3,tmp));
t29=C_mutate((C_word*)lf[945]+1 /* (set! ##sys#really-print-call-chain ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25820,a[2]=((C_word)li770),tmp=(C_word)a,a+=3,tmp));
t30=C_mutate((C_word*)lf[187]+1 /* (set! chicken.base#print-call-chain ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25925,a[2]=((C_word)li771),tmp=(C_word)a,a+=3,tmp));
t31=C_mutate((C_word*)lf[953]+1 /* (set! ##sys#user-interrupt-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26016,a[2]=((C_word)li773),tmp=(C_word)a,a+=3,tmp));
t32=lf[956] /* exit-in-progress */ =C_SCHEME_FALSE;;
t33=C_mutate(&lf[957] /* (set! cleanup-before-exit ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26038,a[2]=((C_word)li776),tmp=(C_word)a,a+=3,tmp));
t34=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26141,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t35=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31633,a[2]=((C_word)li986),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5023: chicken.base#make-parameter */
t36=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t36;
av2[1]=t34;
av2[2]=t35;
((C_proc)(void*)(*((C_word*)t36+1)))(3,av2);}}

/* ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_25019,c,av);}
a=C_alloc(10);
t5=C_slot(t2,C_fix(0));
t6=C_i_assq(t5,C_fast_retrieve(lf[907]));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25032,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25037,a[2]=t4,a[3]=t5,a[4]=t6,a[5]=t2,a[6]=((C_word)li731),tmp=(C_word)a,a+=7,tmp);
/* library.scm:4694: scheme#call-with-current-continuation */
t9=*((C_word*)lf[132]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t7;
av2[2]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25122,a[2]=t1,a[3]=t4,a[4]=t5,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4706: ##sys#print */
t8=*((C_word*)lf[867]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[920];
av2[3]=C_SCHEME_FALSE;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}}

/* k25030 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25032,c,av);}
/* library.scm:4694: g5404 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25037(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_25037,c,av);}
a=C_alloc(13);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25043,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word)li726),tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25094,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=((C_word)li730),tmp=(C_word)a,a+=7,tmp);
/* library.scm:4694: chicken.condition#with-exception-handler */
t5=C_fast_retrieve(lf[916]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_25043,c,av);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25049,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li725),tmp=(C_word)a,a+=6,tmp);
/* library.scm:4694: k5401 */
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}

/* a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_25049,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25053,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4696: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[915];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25051 in a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25053,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25056,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25092,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4697: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k25054 in k25051 in a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_25056,c,av);}
a=C_alloc(5);
if(C_truep(C_i_structurep(((C_word*)t0)[2],lf[911]))){
t2=C_slot(((C_word*)t0)[2],C_fix(2));
t3=C_i_member(lf[912],t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25071,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4700: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[913];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
/* library.scm:4703: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[914];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k25069 in k25054 in k25051 in a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_25071(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_25071,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25074,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4701: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25072 in k25069 in k25054 in k25051 in a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_25074(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25074,c,av);}
/* library.scm:4702: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(62);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k25090 in k25051 in a25048 in a25042 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25092,c,av);}
/* library.scm:4697: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a25093 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_25094,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25100,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word)li727),tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25109,a[2]=((C_word*)t0)[5],a[3]=((C_word)li729),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4694: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a25099 in a25093 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25100,c,av);}
/* library.scm:4694: g5414 */
t2=C_slot(((C_word*)t0)[2],C_fix(1));{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* a25108 in a25093 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_25109,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25115,a[2]=t2,a[3]=((C_word)li728),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4694: k5401 */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}

/* a25114 in a25108 in a25093 in a25036 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25115,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_25122,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25125,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25168,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4707: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k25123 in k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_25125,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25128,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(((C_word*)t0)[4],lf[911]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25137,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4710: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[917];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[918]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25153,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4713: ##sys#print */
t6=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[919];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
/* library.scm:4715: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(62);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}}}

/* k25126 in k25123 in k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25128,c,av);}
/* library.scm:4715: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(62);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}

/* k25135 in k25123 in k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25137(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25137,c,av);}
/* library.scm:4711: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_slot(((C_word*)t0)[3],C_fix(1));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k25151 in k25123 in k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25153(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25153,c,av);}
/* library.scm:4714: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_slot(((C_word*)t0)[3],C_fix(6));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k25166 in k25120 in ##sys#user-print-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25168,c,av);}
/* library.scm:4707: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_25170,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25176,a[2]=t2,a[3]=t3,a[4]=((C_word)li736),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4720: call-with-current-continuation */
t5=*((C_word*)lf[132]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_25176,c,av);}
a=C_alloc(27);
t3=C_fast_retrieve(lf[871]);
t4=C_retrieve2(lf[872],C_text("##sys#print-exit"));
t5=C_retrieve2(lf[870],C_text("current-print-length"));
t6=((C_word*)t0)[2];
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=t2;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_fix(0);
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_TRUE;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25182,a[2]=t7,a[3]=t9,a[4]=t11,a[5]=t13,a[6]=t3,a[7]=((C_word)li733),tmp=(C_word)a,a+=8,tmp);
t15=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25225,a[2]=((C_word*)t0)[3],a[3]=((C_word)li734),tmp=(C_word)a,a+=4,tmp);
t16=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25231,a[2]=t7,a[3]=t9,a[4]=t11,a[5]=t3,a[6]=((C_word)li735),tmp=(C_word)a,a+=7,tmp);
/* library.scm:4722: ##sys#dynamic-wind */
t17=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t17;
av2[1]=t1;
av2[2]=t14;
av2[3]=t15;
av2[4]=t16;
((C_proc)(void*)(*((C_word*)t17+1)))(5,av2);}}

/* a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_25182,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25186,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
/* library.scm:4722: ##sys#print-length-limit5434 */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)C_fast_retrieve_proc(t3))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_25186(2,av2);}}}

/* k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_25186,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25189,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
/* library.scm:4722: ##sys#print-exit5435 */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
f_25189(2,av2);}}}

/* k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_25189,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_25192,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
/* library.scm:4722: current-print-length54365437 */
t3=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[4])[1];
f_25192(2,av2);}}}

/* k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_25192,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_25196,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* library.scm:4722: ##sys#print-length-limit5434 */
t3=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)C_fast_retrieve_proc(t3))(2,av2);}}

/* k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_25196(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_25196,c,av);}
a=C_alloc(10);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_25200,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* library.scm:4722: ##sys#print-exit5435 */
t4=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k25198 in k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_25200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25200,c,av);}
a=C_alloc(9);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25204,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4722: current-print-length54365437 */
t4=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k25202 in k25198 in k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_25204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_25204,c,av);}
a=C_alloc(6);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25207,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4722: ##sys#print-length-limit5434 */
t4=((C_word*)t0)[7];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)C_fast_retrieve_proc(t4))(5,av2);}}

/* k25205 in k25202 in k25198 in k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_25207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_25207,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25210,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4722: ##sys#print-exit5435 */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25208 in k25205 in k25202 in k25198 in k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_25210(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_25210,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25213,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4722: current-print-length54365437 */
t3=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25211 in k25208 in k25205 in k25202 in k25198 in k25194 in k25190 in k25187 in k25184 in a25181 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_25213(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25213,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a25224 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25225,c,av);}
/* library.scm:4725: thunk */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_25231,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25235,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4722: ##sys#print-length-limit5434 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)C_fast_retrieve_proc(t3))(2,av2);}}

/* k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_25235,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25238,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4722: ##sys#print-exit5435 */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k25236 in k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25238(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25238,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25241,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4722: current-print-length54365437 */
t3=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k25239 in k25236 in k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_25241,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25244,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4722: ##sys#print-length-limit5434 */
t3=((C_word*)t0)[8];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)C_fast_retrieve_proc(t3))(5,av2);}}

/* k25242 in k25239 in k25236 in k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_25244(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_25244,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25247,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4722: ##sys#print-exit5435 */
t3=C_retrieve2(lf[872],C_text("##sys#print-exit"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25245 in k25242 in k25239 in k25236 in k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_25247(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_25247,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25250,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4722: current-print-length54365437 */
t3=C_retrieve2(lf[870],C_text("current-print-length"));{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[6])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25248 in k25245 in k25242 in k25239 in k25236 in k25233 in a25230 in a25175 in ##sys#with-print-length-limit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_25250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25250,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)t0)[5]);
t4=C_mutate(((C_word *)((C_word*)t0)[6])+1,((C_word*)t0)[7]);
t5=((C_word*)t0)[8];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* check5478 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25255(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_25255,c,av);}
a=C_alloc(9);
t4=C_slot(t2,C_fix(10));
t5=C_slot(t2,C_fix(11));
t6=C_slot(t2,C_fix(12));
t7=C_fixnum_plus(t4,t3);
if(C_truep(C_fixnum_greater_or_equal_p(t7,t5))){
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25277,a[2]=t5,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t3,a[7]=t6,a[8]=t4,tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(t7,lf[271]))){
/* library.scm:4753: ##sys#error */
t9=*((C_word*)lf[155]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[923];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t9=t8;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_SCHEME_UNDEFINED;
f_25277(2,av2);}}}
else{
t8=C_SCHEME_UNDEFINED;
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}

/* k25275 in check5478 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_25277,c,av);}
a=C_alloc(9);
t2=C_fixnum_plus(((C_word*)t0)[2],((C_word*)t0)[2]);
t3=C_i_fixnum_min(lf[271],t2);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_25283,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:4755: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k25281 in k25275 in check5478 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,6)))){
C_save_and_reclaim((void *)f_25283,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25286,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:4756: ##sys#copy-bytes */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[922]);
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=*((C_word*)lf[922]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=t1;
av2[4]=C_fix(0);
av2[5]=C_fix(0);
av2[6]=((C_word*)t0)[8];
tp(7,av2);}}

/* k25284 in k25281 in k25275 in check5478 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25286,c,av);}
t2=C_i_setslot(((C_word*)t0)[2],C_fix(12),((C_word*)t0)[3]);
t3=C_i_set_i_slot(((C_word*)t0)[2],C_fix(11),((C_word*)t0)[4]);
/* library.scm:4759: check */
t4=((C_word*)((C_word*)t0)[5])[1];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[6];
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[7];
((C_proc)C_fast_retrieve_proc(t4))(4,av2);}}

/* ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,7)))){
C_save_and_reclaim((void *)f_25311,c,av);}
a=C_alloc(11);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25314,a[2]=((C_word)li751),tmp=(C_word)a,a+=3,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25334,a[2]=t5,a[3]=t8,a[4]=t6,a[5]=((C_word)li756),tmp=(C_word)a,a+=6,tmp));
t10=((C_word*)t8)[1];
f_25334(t10,t1,t2,t4,t4,t3,lf[929]);}

/* copy&append in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_fcall f_25314(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_25314,5,t1,t2,t3,t4,t5);}
a=C_alloc(8);
t6=C_block_size(t5);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25321,a[2]=t5,a[3]=t6,a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t8=C_fixnum_difference(t4,t3);
/* library.scm:4836: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t7;
av2[2]=C_fixnum_plus(t6,t8);
tp(3,av2);}}

/* k25319 in copy&append in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25321,c,av);}
t2=C_substring_copy(((C_word*)t0)[2],t1,C_fix(0),((C_word*)t0)[3],C_fix(0));
t3=C_substring_copy(((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[3]);
t4=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_fcall f_25334(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,5)))){
C_save_and_reclaim_args((void *)trf_25334,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(12);
t7=C_eqp(t4,t5);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25344,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4846: copy&append */
f_25314(t8,t2,t3,t4,t6);}
else{
t8=C_subchar(t2,t4);
t9=C_eqp(t8,C_make_character(10));
if(C_truep(t9)){
t10=C_fixnum_plus(t4,C_fix(1));
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25382,a[2]=t1,a[3]=t10,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4853: copy&append */
f_25314(t11,t2,t3,t4,t6);}
else{
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_25388,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=t3,a[7]=t6,a[8]=t8,a[9]=t5,a[10]=((C_word*)t0)[2],a[11]=((C_word*)t0)[3],tmp=(C_word)a,a+=12,tmp);
t11=C_eqp(t8,C_make_character(13));
if(C_truep(t11)){
t12=C_fixnum_plus(t4,C_fix(1));
if(C_truep(C_fixnum_greaterp(t5,t12))){
t13=C_fixnum_plus(t4,C_fix(1));
t14=t10;
f_25388(t14,C_eqp(C_subchar(t2,t13),C_make_character(10)));}
else{
t13=t10;
f_25388(t13,C_SCHEME_FALSE);}}
else{
t12=t10;
f_25388(t12,C_SCHEME_FALSE);}}}}

/* k25342 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25344(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_25344,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25349,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li752),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25355,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word)li753),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4847: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a25348 in k25342 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25349,c,av);}
/* library.scm:4847: eos-handler */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* a25354 in k25342 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_25355,c,av);}
if(C_truep(t2)){
/* library.scm:4849: loop */
t5=((C_word*)((C_word*)t0)[2])[1];
f_25334(t5,t1,t2,t3,t3,t4,((C_word*)t0)[3]);}
else{
/* library.scm:4850: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=((C_word*)t0)[3];
av2[4]=C_SCHEME_FALSE;
C_values(5,av2);}}}

/* k25380 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25382,c,av);}
/* library.scm:4853: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=C_SCHEME_TRUE;
C_values(5,av2);}}

/* k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25388(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,6)))){
C_save_and_reclaim_args((void *)trf_25388,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(2));
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25399,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4857: copy&append */
f_25314(t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[2],((C_word*)t0)[7]);}
else{
t2=C_eqp(((C_word*)t0)[8],C_make_character(13));
t3=(C_truep(t2)?C_eqp(((C_word*)t0)[9],C_fixnum_plus(((C_word*)t0)[2],C_fix(1))):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25408,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4860: copy&append */
f_25314(t4,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[2],((C_word*)t0)[7]);}
else{
t4=C_eqp(((C_word*)t0)[8],C_make_character(13));
if(C_truep(t4)){
t5=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25468,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4871: copy&append */
f_25314(t6,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[2],((C_word*)t0)[7]);}
else{
/* library.scm:4872: loop */
t5=((C_word*)((C_word*)t0)[11])[1];
f_25334(t5,((C_word*)t0)[3],((C_word*)t0)[5],((C_word*)t0)[6],C_fixnum_plus(((C_word*)t0)[2],C_fix(1)),((C_word*)t0)[9],((C_word*)t0)[7]);}}}}

/* k25397 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25399,c,av);}
/* library.scm:4857: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=C_SCHEME_TRUE;
C_values(5,av2);}}

/* k25406 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,5)))){
C_save_and_reclaim((void *)f_25408,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25413,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li754),tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25419,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=((C_word)li755),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4861: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a25412 in k25406 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25413,c,av);}
/* library.scm:4861: eos-handler */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* a25418 in k25406 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_25419,c,av);}
a=C_alloc(7);
if(C_truep(t2)){
t5=C_eqp(C_subchar(t2,t3),C_make_character(10));
if(C_truep(t5)){
/* library.scm:4864: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=C_fixnum_plus(t3,C_fix(1));
av2[3]=((C_word*)t0)[2];
av2[4]=C_SCHEME_TRUE;
C_values(5,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25443,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=t4,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4867: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[927];
tp(4,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25451,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4869: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[928];
tp(4,av2);}}}

/* k25441 in a25418 in k25406 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_25443,c,av);}
/* library.scm:4866: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_25334(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[5],((C_word*)t0)[6],t1);}

/* k25449 in a25418 in k25406 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25451(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25451,c,av);}
/* library.scm:4869: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=C_SCHEME_TRUE;
C_values(5,av2);}}

/* k25466 in k25386 in loop in ##sys#scan-buffer-line in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25468,c,av);}
/* library.scm:4871: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=C_SCHEME_TRUE;
C_values(5,av2);}}

/* chicken.base#open-input-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25508(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,5)))){
C_save_and_reclaim((void *)f_25508,c,av);}
a=C_alloc(4);
t3=C_i_check_string_2(t2,lf[930]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25515,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4877: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t4;
av2[2]=C_fix(1);
av2[3]=C_fast_retrieve(lf[926]);
av2[4]=lf[931];
av2[5]=lf[932];
tp(6,av2);}}

/* k25513 in chicken.base#open-input-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25515,c,av);}
t2=C_i_set_i_slot(t1,C_fix(11),C_block_size(((C_word*)t0)[2]));
t3=C_i_set_i_slot(t1,C_fix(10),C_fix(0));
t4=C_i_setslot(t1,C_fix(12),((C_word*)t0)[2]);
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.base#open-output-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25527(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_25527,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25531,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4885: ##sys#make-port */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[697]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[697]+1);
av2[1]=t2;
av2[2]=C_fix(2);
av2[3]=C_fast_retrieve(lf[926]);
av2[4]=lf[933];
av2[5]=lf[932];
tp(6,av2);}}

/* k25529 in chicken.base#open-output-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_25531,c,av);}
a=C_alloc(4);
t2=C_i_set_i_slot(t1,C_fix(10),C_fix(0));
t3=C_i_set_i_slot(t1,C_fix(11),C_fix(256));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25544,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:4888: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t4;
av2[2]=C_fix(256);
tp(3,av2);}}

/* k25542 in k25529 in chicken.base#open-output-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_25544,c,av);}
t2=C_i_setslot(((C_word*)t0)[2],C_fix(12),t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#get-output-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_25546,c,av);}
t3=C_i_check_port_2(t2,C_fix(2),C_SCHEME_FALSE,lf[934]);
t4=C_slot(t2,C_fix(7));
t5=C_eqp(lf[932],t4);
if(C_truep(C_i_not(t5))){
/* library.scm:4895: ##sys#signal-hook */
t6=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[934];
av2[4]=lf[935];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(6,av2);}}
else{
/* library.scm:4897: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t1;
av2[2]=C_slot(t2,C_fix(12));
av2[3]=C_fix(0);
av2[4]=C_slot(t2,C_fix(10));
tp(5,av2);}}}

/* ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_25580,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25584,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4903: open-output-string */
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k25582 in ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_25584,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25585,a[2]=t1,a[3]=((C_word)li761),tmp=(C_word)a,a+=4,tmp);
t3=C_i_check_list_2(((C_word*)t0)[2],lf[687]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25595,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25600,a[2]=t6,a[3]=t2,a[4]=((C_word)li762),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_25600(t8,t4,((C_word*)t0)[2]);}

/* g5608 in k25582 in ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25585(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_25585,3,t0,t1,t2);}
/* library.scm:4904: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25593 in k25582 in ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25595,c,av);}
/* library.scm:4905: get-output-string */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* for-each-loop5607 in k25582 in ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25600(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_25600,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25610,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4904: g5608 */
t4=((C_word*)t0)[3];
f_25585(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k25608 in for-each-loop5607 in k25582 in ##sys#print-to-string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25610,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_25600(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_25623,c,av);}
a=C_alloc(7);
if(C_truep(C_taggedpointerp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25634,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_pairp(t4))){
t5=C_u_i_car(t4);
t6=C_a_i_list1(&a,1,t5);
/* library.scm:4913: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word *av2=av;
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t3;
av2[2]=t6;
tp(3,av2);}}
else{
t5=C_a_i_list1(&a,1,t4);
/* library.scm:4913: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word *av2=av;
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t3;
av2[2]=t5;
tp(3,av2);}}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25664,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25668,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4919: ##sys#pointer->address */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[940]);
C_word *av2=av;
av2[0]=*((C_word*)lf[940]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}}

/* k25632 in ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_25634,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25638,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_25642,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:4917: ##sys#pointer->address */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[940]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[940]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k25636 in k25632 in ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25638(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_25638,c,av);}
/* library.scm:4911: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[937];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[938];
av2[5]=t1;
av2[6]=lf[939];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k25640 in k25632 in ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25642,c,av);}
/* library.scm:4917: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_fix(16);
tp(4,av2);}}

/* k25662 in ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25664(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25664,c,av);}
/* library.scm:4919: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[941];
av2[3]=t1;
av2[4]=lf[942];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k25666 in ##sys#pointer->string in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_25668,c,av);}
/* library.scm:4919: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_fix(16);
tp(4,av2);}}

/* chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_25676,c,av);}
a=C_alloc(6);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?C_fix(0):C_get_rest_arg(c,2,av,2,t0));
t4=C_rest_nullp(c,2);
t5=C_rest_nullp(c,3);
t6=(C_truep(t5)?C_fast_retrieve(lf[943]):C_get_rest_arg(c,3,av,2,t0));
t7=C_rest_nullp(c,3);
t8=C_fix((C_word)C_trace_buffer_size);
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25692,a[2]=t3,a[3]=t8,a[4]=t6,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4933: ##sys#make-vector */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[672]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[672]+1);
av2[1]=t9;
av2[2]=C_fixnum_times(C_fix(4),t8);
av2[3]=C_SCHEME_FALSE;
tp(4,av2);}}

/* k25690 in chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_25692,c,av);}
a=C_alloc(10);
t2=C_fetch_trace(((C_word*)t0)[2],t1);
t3=C_fixnump(t2);
t4=(C_truep(t3)?t2:C_fixnum_times(C_fix(4),((C_word*)t0)[3]));
t5=(C_truep(((C_word*)t0)[4])?C_slot(((C_word*)t0)[4],C_fix(14)):C_SCHEME_FALSE);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25703,a[2]=t4,a[3]=t1,a[4]=t7,a[5]=((C_word*)t0)[4],a[6]=t5,a[7]=((C_word)li765),tmp=(C_word)a,a+=8,tmp));
t9=((C_word*)t7)[1];
f_25703(t9,((C_word*)t0)[5],C_fix(0));}

/* loop in k25690 in chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25703(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_25703,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_fixnum_plus(t2,C_fix(3));
t4=C_slot(((C_word*)t0)[3],t3);
t5=C_i_not(t4);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25722,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(t5)){
t7=t6;
f_25722(t7,t5);}
else{
t7=C_i_not(((C_word*)t0)[5]);
t8=t6;
f_25722(t8,(C_truep(t7)?t7:C_eqp(((C_word*)t0)[6],t4)));}}}

/* k25720 in loop in k25690 in chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_25722(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_25722,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25741,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_slot(((C_word*)t0)[3],((C_word*)t0)[2]);
t4=C_a_i_bytevector(&a,1,C_fix(3));
/* library.scm:4928: ##sys#peek-nonnull-c-string */
t5=*((C_word*)lf[944]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t2;
av2[2]=stub5630(t4,t3);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
/* library.scm:4947: loop */
t2=((C_word*)((C_word*)t0)[5])[1];
f_25703(t2,((C_word*)t0)[4],C_fixnum_plus(((C_word*)t0)[2],C_fix(4)));}}

/* k25731 in k25739 in k25720 in loop in k25690 in chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_25733,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k25739 in k25720 in loop in k25690 in chicken.base#get-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_25741,c,av);}
a=C_alloc(8);
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
t3=C_slot(((C_word*)t0)[3],t2);
t4=C_fixnum_plus(((C_word*)t0)[2],C_fix(2));
t5=C_slot(((C_word*)t0)[3],t4);
t6=C_a_i_vector3(&a,3,t1,t3,t5);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25733,a[2]=((C_word*)t0)[4],a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* library.scm:4946: loop */
t8=((C_word*)((C_word*)t0)[5])[1];
f_25703(t8,t7,C_fixnum_plus(((C_word*)t0)[2],C_fix(4)));}

/* ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_25820,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t3))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25830,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4951: ##sys#print */
t6=*((C_word*)lf[867]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
av2[3]=C_SCHEME_FALSE;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_25830,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25831,a[2]=((C_word*)t0)[2],a[3]=((C_word)li768),tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25897,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25902,a[2]=t5,a[3]=t2,a[4]=((C_word)li769),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_25902(t7,t3,((C_word*)t0)[4]);}

/* g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25831(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_25831,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_slot(t2,C_fix(1));
t4=C_slot(t2,C_fix(2));
t5=C_i_structurep(t4,lf[946]);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_25844,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t5,a[6]=t4,a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* library.scm:4957: ##sys#print */
t7=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[949];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25844(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_25844,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25847,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4958: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[7],C_fix(0));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_25847(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_25847,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25850,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* library.scm:4959: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[948];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_25850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_25850,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25853,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25868,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
if(C_truep(((C_word*)t0)[5])){
t4=t3;
f_25868(t4,C_slot(((C_word*)t0)[6],C_fix(1)));}
else{
t4=C_SCHEME_UNDEFINED;
t5=t3;
f_25868(t5,t4);}}
else{
t4=t3;
f_25868(t4,C_SCHEME_FALSE);}}

/* k25851 in k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_25853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_25853,c,av);}
a=C_alloc(5);
if(C_truep(((C_word*)t0)[2])){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25861,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word)li767),tmp=(C_word)a,a+=5,tmp);
/* library.scm:4969: ##sys#with-print-length-limit */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[921]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[921]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=C_fix(100);
av2[3]=t2;
tp(4,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a25860 in k25851 in k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_25861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25861,c,av);}
/* library.scm:4972: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k25866 in k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_25868(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_25868,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_25871,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:4961: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word av2[4];
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(91);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_25853(2,av2);}}}

/* k25869 in k25866 in k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_25871(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_25871,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_25874,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
/* library.scm:4962: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_slot(((C_word*)t0)[5],C_fix(1));
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* library.scm:4962: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k25872 in k25869 in k25866 in k25848 in k25845 in k25842 in g5674 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_25874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25874,c,av);}
/* library.scm:4967: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[947];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k25895 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25897,c,av);}
/* library.scm:4974: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[950];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* for-each-loop5673 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_25902(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_25902,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25912,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4952: g5674 */
t4=((C_word*)t0)[3];
f_25831(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k25910 in for-each-loop5673 in k25828 in ##sys#really-print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_25912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_25912,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_25902(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.base#print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_25925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_25925,c,av);}
a=C_alloc(7);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?*((C_word*)lf[714]+1):C_get_rest_arg(c,2,av,2,t0));
t4=C_rest_nullp(c,2);
t5=C_rest_nullp(c,3);
t6=(C_truep(t5)?C_fix(0):C_get_rest_arg(c,3,av,2,t0));
t7=C_rest_nullp(c,3);
t8=C_rest_nullp(c,4);
t9=(C_truep(t8)?C_fast_retrieve(lf[943]):C_get_rest_arg(c,4,av,2,t0));
t10=C_rest_nullp(c,4);
t11=C_rest_nullp(c,5);
t12=(C_truep(t11)?lf[951]:C_get_rest_arg(c,5,av,2,t0));
t13=C_rest_nullp(c,5);
t14=C_i_check_port_2(t3,C_fix(2),C_SCHEME_TRUE,lf[952]);
t15=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_25956,a[2]=t12,a[3]=t1,a[4]=t3,a[5]=t6,a[6]=t9,tmp=(C_word)a,a+=7,tmp);
/* library.scm:4981: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t15;
av2[2]=t6;
av2[3]=lf[952];
tp(4,av2);}}

/* k25954 in chicken.base#print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_25956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_25956,c,av);}
a=C_alloc(5);
t2=C_i_check_string_2(((C_word*)t0)[2],lf[952]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_25966,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:4983: chicken.base#get-call-chain */
t4=*((C_word*)lf[186]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k25964 in k25954 in chicken.base#print-call-chain in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_25966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_25966,c,av);}
/* library.scm:4983: ##sys#really-print-call-chain */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[945]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[945]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
tp(5,av2);}}

/* ##sys#user-interrupt-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_26016(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_26016,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26019,a[2]=((C_word)li772),tmp=(C_word)a,a+=3,tmp);
t3=C_eqp(C_fast_retrieve(lf[943]),C_fast_retrieve(lf[955]));
if(C_truep(t3)){
/* library.scm:4989: ##sys#signal-hook */
t4=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[954];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_setslot(C_fast_retrieve(lf[955]),C_fix(1),t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* break in ##sys#user-interrupt-hook in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_26019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26019,c,av);}
/* library.scm:4989: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[954];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_fcall f_26038(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_26038,1,t1);}
a=C_alloc(6);
t2=lf[956] /* exit-in-progress */ =C_SCHEME_TRUE;;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26043,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_dump_heap_on_exitp())){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26134,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5004: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[964];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=t3;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_26043(2,av2);}}}

/* k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_26043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_26043,c,av);}
a=C_alloc(9);
t2=(C_truep(C_i_profilingp())?C_i_dump_statistical_profile():C_SCHEME_UNDEFINED);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26049,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26086,a[2]=t5,a[3]=((C_word)li775),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_26086(t7,t3);}

/* k26047 in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_26049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26049,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26052,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_slot(C_fast_retrieve(lf[961]),C_fix(0));
if(C_truep(C_fixnum_greaterp(t3,C_fix(0)))){
/* library.scm:5015: ##sys#run-pending-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[962]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[962]+1);
av2[1]=t2;
av2[2]=C_SCHEME_FALSE;
tp(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_26052(2,av2);}}}

/* k26050 in k26047 in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_26052,c,av);}
a=C_alloc(3);
if(C_truep(C_fixnum_greaterp(C_i_live_finalizer_count(),C_fix(0)))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26061,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_debug_modep())){
/* library.scm:5018: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[960];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_26061(2,av2);}}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26059 in k26050 in k26047 in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26061(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26061,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26067,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5019: chicken.gc#force-finalizers */
t3=C_fast_retrieve(lf[959]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k26065 in k26059 in k26050 in k26047 in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26067(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26067,c,av);}
if(C_truep(t1)){
/* library.scm:5020: ##sys#force-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[958]);
C_word *av2=av;
av2[0]=*((C_word*)lf[958]+1);
av2[1]=((C_word*)t0)[2];
tp(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* loop in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_fcall f_26086(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_26086,2,t0,t1);}
a=C_alloc(10);
t2=*((C_word*)lf[227]+1);
t3=C_set_block_item(lf[227] /* chicken.base#cleanup-tasks */,0,C_SCHEME_END_OF_LIST);
if(C_truep(C_i_nullp(t2))){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=C_i_check_list_2(t2,lf[687]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26105,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26110,a[2]=t7,a[3]=((C_word)li774),tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_26110(t9,t5,t2);}}

/* k26103 in loop in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26105,c,av);}
/* library.scm:5013: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_26086(t2,((C_word*)t0)[3]);}

/* for-each-loop5733 in loop in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_fcall f_26110(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_26110,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26120,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5012: t */
t4=C_slot(t2,C_fix(0));{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)C_fast_retrieve_proc(t4))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k26118 in for-each-loop5733 in loop in k26041 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26120,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_26110(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k26132 in cleanup-before-exit in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_26134(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26134,c,av);}
/* library.scm:5005: ##sys#dump-heap-state */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[963]);
C_word *av2=av;
av2[0]=*((C_word*)lf[963]+1);
av2[1]=((C_word*)t0)[2];
tp(2,av2);}}

/* k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_26141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_26141,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[226]+1 /* (set! chicken.base#exit-handler ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26145,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31627,a[2]=((C_word)li985),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5033: chicken.base#make-parameter */
t5=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_26145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_26145,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[225]+1 /* (set! chicken.base#implicit-exit-handler ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26149,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31618,a[2]=((C_word)li984),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5038: chicken.base#make-parameter */
t5=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_26149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_26149,c,av);}
a=C_alloc(23);
t2=C_mutate((C_word*)lf[965]+1 /* (set! ##sys#reset-handler ...) */,t1);
t3=C_a_i_provide(&a,1,lf[966]);
t4=C_mutate((C_word*)lf[194]+1 /* (set! ##sys#signal-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26151,a[2]=((C_word)li779),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[968]+1 /* (set! chicken.condition#abort ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26435,a[2]=((C_word)li780),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1006]+1 /* (set! chicken.condition#signal ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26452,a[2]=((C_word)li781),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[1007]+1 /* (set! ##sys#break-on-error ...) */,C_mk_bool(C_enable_repl));
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26461,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31458,a[2]=((C_word)li983),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5135: chicken.base#make-parameter */
t10=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t10;
av2[1]=t8;
av2[2]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +12,c,4)))){
C_save_and_reclaim((void*)f_26151,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+12);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t6=C_dbg_hook(C_SCHEME_FALSE);
t7=signal_debug_event(t2,((C_word*)t5)[1],t4);
t8=C_eqp(t2,lf[954]);
if(C_truep(t8)){
t9=C_a_i_record3(&a,3,lf[911],lf[967],C_SCHEME_END_OF_LIST);
/* library.scm:5066: abort */
t10=C_fast_retrieve(lf[968]);{
C_word *av2=av;
av2[0]=t10;
av2[1]=t1;
av2[2]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t9=C_eqp(t2,lf[309]);
t10=(C_truep(t9)?t9:C_eqp(t2,lf[311]));
if(C_truep(t10)){
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26177,a[2]=t1,a[3]=t4,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
t12=C_eqp(t2,lf[309]);
if(C_truep(t12)){
/* library.scm:5072: ##sys#print */
t13=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t13;
av2[1]=t11;
av2[2]=lf[970];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}
else{
/* library.scm:5072: ##sys#print */
t13=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t13;
av2[1]=t11;
av2[2]=lf[971];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}}
else{
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26253,a[2]=t5,a[3]=t4,a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t12=C_i_symbolp(((C_word*)t5)[1]);
t13=(C_truep(t12)?C_i_nullp(t4):C_SCHEME_FALSE);
if(C_truep(t13)){
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26418,a[2]=t5,a[3]=t11,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5090: scheme#symbol->string */
t15=*((C_word*)lf[634]+1);{
C_word *av2=av;
av2[0]=t15;
av2[1]=t14;
av2[2]=((C_word*)t5)[1];
((C_proc)(void*)(*((C_word*)t15+1)))(3,av2);}}
else{
t14=t11;
f_26253(t14,C_SCHEME_UNDEFINED);}}}}

/* k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26177(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_26177,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26180,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5075: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_26180,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26183,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_nullp(((C_word*)t0)[3]);
t4=(C_truep(t3)?t3:C_fixnum_greaterp(C_u_i_length(((C_word*)t0)[3]),C_fix(1)));
if(C_truep(t4)){
/* library.scm:5077: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[715]+1);
tp(4,av2);}}
else{
/* library.scm:5078: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[969];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_26183,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26200,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26205,a[2]=t4,a[3]=((C_word)li778),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_26205(t6,t2,((C_word*)t0)[3]);}

/* a26189 in for-each-loop5803 in k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_26190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_26190,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26194,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5084: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k26192 in a26189 in for-each-loop5803 in k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_26194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26194,c,av);}
/* library.scm:5085: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[715]+1);
tp(4,av2);}}

/* k26198 in k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_26200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26200,c,av);}
/* library.scm:5087: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[746]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[715]+1);
tp(3,av2);}}

/* for-each-loop5803 in k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_26205(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_26205,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26215,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26190,a[2]=t4,a[3]=((C_word)li777),tmp=(C_word)a,a+=4,tmp);
/* library.scm:5081: ##sys#with-print-length-limit */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[921]);
C_word av2[4];
av2[0]=*((C_word*)lf[921]+1);
av2[1]=t3;
av2[2]=C_fix(400);
av2[3]=t5;
tp(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k26213 in for-each-loop5803 in k26181 in k26178 in k26175 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_26215(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26215,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_26205(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k26251 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_26253(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26253,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26256,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_i_not(((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(t3)){
t4=t2;
f_26256(t4,(C_truep(t3)?C_i_pairp(((C_word*)t0)[3]):C_SCHEME_FALSE));}
else{
t4=C_i_symbolp(((C_word*)((C_word*)t0)[2])[1]);
t5=t2;
f_26256(t5,(C_truep(t4)?C_i_pairp(((C_word*)t0)[3]):C_SCHEME_FALSE));}}

/* k26254 in k26251 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_26256(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_26256,2,t0,t1);}
a=C_alloc(6);
t2=(C_truep(t1)?((C_word*)((C_word*)t0)[2])[1]:C_SCHEME_FALSE);
t3=(C_truep(t1)?C_slot(((C_word*)t0)[3],C_fix(0)):((C_word*)((C_word*)t0)[2])[1]);
t4=(C_truep(t1)?C_slot(((C_word*)t0)[3],C_fix(1)):((C_word*)t0)[3]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26276,a[2]=t3,a[3]=t4,a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t6=C_eqp(((C_word*)t0)[5],lf[260]);
if(C_truep(t6)){
t7=t5;
f_26276(t7,lf[976]);}
else{
t7=C_eqp(((C_word*)t0)[5],lf[780]);
if(C_truep(t7)){
t8=t5;
f_26276(t8,lf[977]);}
else{
t8=C_eqp(((C_word*)t0)[5],lf[383]);
if(C_truep(t8)){
t9=t5;
f_26276(t9,lf[978]);}
else{
t9=C_eqp(((C_word*)t0)[5],lf[565]);
if(C_truep(t9)){
t10=t5;
f_26276(t10,lf[979]);}
else{
t10=C_eqp(((C_word*)t0)[5],lf[699]);
if(C_truep(t10)){
t11=t5;
f_26276(t11,lf[980]);}
else{
t11=C_eqp(((C_word*)t0)[5],lf[981]);
if(C_truep(t11)){
t12=t5;
f_26276(t12,lf[982]);}
else{
t12=C_eqp(((C_word*)t0)[5],lf[983]);
if(C_truep(t12)){
t13=t5;
f_26276(t13,lf[984]);}
else{
t13=C_eqp(((C_word*)t0)[5],lf[985]);
if(C_truep(t13)){
t14=t5;
f_26276(t14,lf[986]);}
else{
t14=C_eqp(((C_word*)t0)[5],lf[987]);
if(C_truep(t14)){
t15=t5;
f_26276(t15,lf[988]);}
else{
t15=C_eqp(((C_word*)t0)[5],lf[989]);
if(C_truep(t15)){
t16=t5;
f_26276(t16,lf[990]);}
else{
t16=C_eqp(((C_word*)t0)[5],lf[991]);
if(C_truep(t16)){
t17=t5;
f_26276(t17,lf[992]);}
else{
t17=C_eqp(((C_word*)t0)[5],lf[993]);
if(C_truep(t17)){
t18=t5;
f_26276(t18,lf[994]);}
else{
t18=C_eqp(((C_word*)t0)[5],lf[995]);
if(C_truep(t18)){
t19=t5;
f_26276(t19,lf[996]);}
else{
t19=C_eqp(((C_word*)t0)[5],lf[997]);
t20=t5;
f_26276(t20,(C_truep(t19)?lf[998]:lf[999]));}}}}}}}}}}}}}}

/* k26274 in k26254 in k26251 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_26276(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_26276,2,t0,t1);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26284,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:5116: chicken.base#get-call-chain */
t3=*((C_word*)lf[186]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k26282 in k26274 in k26254 in k26251 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_26284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,2)))){
C_save_and_reclaim((void *)f_26284,c,av);}
a=C_alloc(28);
t2=C_a_i_list8(&a,8,lf[972],((C_word*)t0)[2],lf[973],((C_word*)t0)[3],lf[974],t1,lf[975],((C_word*)t0)[4]);
t3=C_a_i_record3(&a,3,lf[911],((C_word*)t0)[5],t2);
/* library.scm:5095: abort */
t4=C_fast_retrieve(lf[968]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[6];
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26416 in ##sys#signal-hook in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26418,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];
f_26253(t3,t2);}

/* chicken.condition#abort in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26435(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26435,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26439,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5120: ##sys#current-exception-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1005]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1005]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k26437 in chicken.condition#abort in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_26439,c,av);}
a=C_alloc(22);
t2=C_a_i_list6(&a,6,lf[1000],lf[1001],lf[1002],C_SCHEME_END_OF_LIST,lf[1003],C_SCHEME_FALSE);
t3=C_a_i_record3(&a,3,lf[911],lf[1004],t2);
/* library.scm:5121: abort */
t4=C_fast_retrieve(lf[968]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* chicken.condition#signal in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_26452,c,av);}
/* library.scm:5130: ##sys#current-exception-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1005]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1005]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}

/* k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_26461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(39,c,5)))){
C_save_and_reclaim((void *)f_26461,c,av);}
a=C_alloc(39);
t2=C_mutate((C_word*)lf[1008]+1 /* (set! ##sys#error-handler ...) */,t1);
t3=C_set_block_item(lf[1009] /* ##sys#last-exception */,0,C_SCHEME_FALSE);
t4=C_mutate((C_word*)lf[1005]+1 /* (set! ##sys#current-exception-handler ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26464,a[2]=((C_word)li782),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[916]+1 /* (set! chicken.condition#with-exception-handler ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26624,a[2]=((C_word)li785),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1025]+1 /* (set! chicken.condition#current-exception-handler ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26638,a[2]=((C_word)li787),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate(&lf[1027] /* (set! chicken.condition#prop-list->kind-prefixed-prop-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26699,a[2]=((C_word)li789),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[1029]+1 /* (set! chicken.condition#make-property-condition ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26767,a[2]=((C_word)li790),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[1031]+1 /* (set! chicken.condition#make-composite-condition ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26781,a[2]=((C_word)li794),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[1033]+1 /* (set! chicken.condition#condition ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26921,a[2]=((C_word)li797),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[1034]+1 /* (set! chicken.condition#condition? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27026,a[2]=((C_word)li798),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[1035]+1 /* (set! chicken.condition#condition->list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27032,a[2]=((C_word)li802),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[1038]+1 /* (set! chicken.condition#condition-predicate ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27150,a[2]=((C_word)li804),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[1039]+1 /* (set! chicken.condition#condition-property-accessor ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27171,a[2]=((C_word)li806),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[1042]+1 /* (set! chicken.condition#get-condition-property ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27217,a[2]=((C_word)li807),tmp=(C_word)a,a+=3,tmp));
t16=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27228,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5318: condition-property-accessor */
t17=C_fast_retrieve(lf[1039]);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t17;
av2[1]=t16;
av2[2]=lf[1012];
av2[3]=lf[1377];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t17+1)))(5,av2);}}

/* ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26464(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_26464,c,av);}
a=C_alloc(14);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26468,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_structurep(t2,lf[911]))){
t4=C_mutate((C_word*)lf[1009]+1 /* (set! ##sys#last-exception ...) */,t2);
t5=C_slot(t2,C_fix(1));
if(C_truep(C_i_memq(lf[1012],t5))){
t6=C_slot(t2,C_fix(2));
t7=C_i_member(lf[1013],t6);
t8=C_i_member(lf[1014],t6);
t9=C_i_member(lf[1015],t6);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26507,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26517,a[2]=t8,a[3]=t10,a[4]=t7,a[5]=t9,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5187: ##sys#error-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1008]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1008]+1);
av2[1]=t11;
tp(2,av2);}}
else{
t6=C_slot(t5,C_fix(0));
t7=C_eqp(lf[1020],t6);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26578,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5205: ##sys#print */
t9=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[1021];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t8=C_slot(t5,C_fix(0));
t9=C_eqp(lf[1022],t8);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26593,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5208: ##sys#error-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1008]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1008]+1);
av2[1]=t10;
tp(2,av2);}}
else{
t10=C_a_i_list2(&a,2,lf[1010],t2);
t11=C_a_i_record3(&a,3,lf[911],lf[1011],t10);
/* library.scm:5212: abort */
t12=C_fast_retrieve(lf[968]);{
C_word *av2=av;
av2[0]=t12;
av2[1]=t1;
av2[2]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}}}}
else{
t4=C_a_i_list2(&a,2,lf[1010],t2);
t5=C_a_i_record3(&a,3,lf[911],lf[1011],t4);
/* library.scm:5212: abort */
t6=C_fast_retrieve(lf[968]);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k26466 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_26468,c,av);}
a=C_alloc(10);
t2=C_a_i_list2(&a,2,lf[1010],((C_word*)t0)[2]);
t3=C_a_i_record3(&a,3,lf[911],lf[1011],t2);
/* library.scm:5212: abort */
t4=C_fast_retrieve(lf[968]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k26505 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26507,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26510,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5203: ##sys#reset-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[965]);
C_word *av2=av;
av2[0]=*((C_word*)lf[965]+1);
av2[1]=t2;
tp(2,av2);}}

/* k26508 in k26505 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26510,c,av);}
/* library.scm:5182: g5941 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* k26515 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26517(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_26517,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26521,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=C_i_cadr(((C_word*)t0)[4]);
t4=(C_truep(((C_word*)t0)[5])?C_i_cadr(((C_word*)t0)[5]):C_SCHEME_FALSE);
t5=(C_truep(t4)?C_i_symbolp(t4):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26547,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5193: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t6;
av2[2]=t4;
tp(3,av2);}}
else{
t6=t2;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t3;
f_26521(2,av2);}}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[1019];
f_26521(2,av2);}}}

/* k26519 in k26515 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_26521,c,av);}
if(C_truep(((C_word*)t0)[2])){{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
av2[4]=C_i_cadr(((C_word*)t0)[2]);
C_apply(5,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}}

/* k26545 in k26515 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_26547,c,av);}
if(C_truep(C_i_symbolp(((C_word*)t0)[2]))){
/* library.scm:5192: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[1016];
av2[3]=t1;
av2[4]=lf[1017];
av2[5]=C_slot(((C_word*)t0)[2],C_fix(1));
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[2]))){
/* library.scm:5192: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[1016];
av2[3]=t1;
av2[4]=lf[1017];
av2[5]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
/* library.scm:5192: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[1016];
av2[3]=t1;
av2[4]=lf[1017];
av2[5]=lf[1018];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}}

/* k26576 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26578,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26581,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5206: ##sys#reset-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[965]);
C_word *av2=av;
av2[0]=*((C_word*)lf[965]+1);
av2[1]=t2;
tp(2,av2);}}

/* k26579 in k26576 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26581,c,av);}
/* library.scm:5181: g5944 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* k26591 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_26593,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26596,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_slot(((C_word*)t0)[3],C_fix(2));
t4=C_i_member(lf[1023],t3);
/* library.scm:5181: g5947 */
t5=t1;{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[1024];
av2[3]=C_i_cadr(t4);
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}

/* k26594 in k26591 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_26596,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26599,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5211: ##sys#reset-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[965]);
C_word *av2=av;
av2[0]=*((C_word*)lf[965]+1);
av2[1]=t2;
tp(2,av2);}}

/* k26597 in k26594 in k26591 in ##sys#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_26599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26599,c,av);}
/* library.scm:5181: g5949 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* chicken.condition#with-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_26624,c,av);}
a=C_alloc(8);
t4=C_fast_retrieve(lf[1005]);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26630,a[2]=t2,a[3]=((C_word)li783),tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26634,a[2]=t4,a[3]=((C_word)li784),tmp=(C_word)a,a+=4,tmp);
/* library.scm:5220: ##sys#dynamic-wind */
t7=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t1;
av2[2]=t5;
av2[3]=t3;
av2[4]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* a26629 in chicken.condition#with-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26630,c,av);}
t2=C_mutate((C_word*)lf[1005]+1 /* (set! ##sys#current-exception-handler ...) */,((C_word*)t0)[2]);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a26633 in chicken.condition#with-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_26634,c,av);}
t2=C_mutate((C_word*)lf[1005]+1 /* (set! ##sys#current-exception-handler ...) */,((C_word*)t0)[2]);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.condition#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26638(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_26638,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t2))){
t3=C_fast_retrieve(lf[1005]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fast_retrieve(lf[1005]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26651,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5230: ##sys#check-closure */
t5=*((C_word*)lf[369]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[1026];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}

/* k26649 in chicken.condition#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_26651,c,av);}
a=C_alloc(10);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26653,a[2]=((C_word*)t0)[3],a[3]=((C_word)li786),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26659,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26664,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_nullp(t2))){
t6=(
/* library.scm:5231: def-convert?5965 */
  f_26664(t5)
);
t7=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_i_car(t2);
t7=C_u_i_cdr(t2);
if(C_truep(C_i_nullp(t7))){
t8=(
/* library.scm:5231: def-set?5966 */
  f_26659(t4)
);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t8=C_i_car(t7);
t9=C_u_i_cdr(t7);
t10=(
/* library.scm:5231: body5963 */
  f_26653(t3,t8)
);
t11=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t11;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t11+1)))(2,av2);}}}}

/* body5963 in k26649 in chicken.condition#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static C_word C_fcall f_26653(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
if(C_truep(t1)){
t2=C_mutate((C_word*)lf[1005]+1 /* (set! ##sys#current-exception-handler ...) */,((C_word*)t0)[2]);
return(t2);}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* def-set?5966 in k26649 in chicken.condition#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static C_word C_fcall f_26659(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:5231: body5963 */
  f_26653(((C_word*)t0)[2],C_SCHEME_TRUE)
));}

/* def-convert?5965 in k26649 in chicken.condition#current-exception-handler in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static C_word C_fcall f_26664(C_word t0){
C_word tmp;
C_word t1;
C_stack_overflow_check;{}
return((
/* library.scm:5231: def-set?5966 */
  f_26659(((C_word*)t0)[2])
));}

/* chicken.condition#prop-list->kind-prefixed-prop-list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_fcall f_26699(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_26699,4,t1,t2,t3,t4);}
a=C_alloc(9);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_26705,a[2]=t2,a[3]=t4,a[4]=t3,a[5]=t6,a[6]=((C_word)li788),tmp=(C_word)a,a+=7,tmp));
t8=((C_word*)t6)[1];
f_26705(t8,t1,t4);}

/* loop in chicken.condition#prop-list->kind-prefixed-prop-list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_26705(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_26705,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
t4=C_i_not(t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_26721,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t4)){
t6=t5;
f_26721(t6,t4);}
else{
t6=C_i_cdr(t2);
t7=t5;
f_26721(t7,C_i_not(C_i_pairp(t6)));}}}

/* k26719 in loop in chicken.condition#prop-list->kind-prefixed-prop-list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_26721(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,5)))){
C_save_and_reclaim_args((void *)trf_26721,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
/* library.scm:5241: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[260];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1028];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=C_i_car(((C_word*)t0)[5]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[6],t2);
t4=C_i_cadr(((C_word*)t0)[5]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_26743,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t6=C_u_i_cdr(((C_word*)t0)[5]);
/* library.scm:5245: loop */
t7=((C_word*)((C_word*)t0)[7])[1];
f_26705(t7,t5,C_u_i_cdr(t6));}}

/* k26741 in k26719 in loop in chicken.condition#prop-list->kind-prefixed-prop-list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_26743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_26743,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.condition#make-property-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +7,c,4)))){
C_save_and_reclaim((void*)f_26767,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+7);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=C_a_i_list1(&a,1,t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26779,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5250: prop-list->kind-prefixed-prop-list */
f_26699(t5,lf[1030],t2,t3);}

/* k26777 in chicken.condition#make-property-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_26779,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record3(&a,3,lf[911],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +26,c,3)))){
C_save_and_reclaim((void*)f_26781,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+26);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
t4=C_a_i_cons(&a,2,t2,t3);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26898,a[2]=((C_word)li791),tmp=(C_word)a,a+=3,tmp);
t6=(
  f_26898(t4)
);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26800,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t8=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t9=t8;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=((C_word*)t10)[1];
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26862,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26864,a[2]=t10,a[3]=t14,a[4]=t11,a[5]=((C_word)li793),tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_26864(t16,t12,t4);}

/* k26798 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26800(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_26800,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26804,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26816,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26818,a[2]=t5,a[3]=t9,a[4]=t6,a[5]=((C_word)li792),tmp=(C_word)a,a+=6,tmp));
t11=((C_word*)t9)[1];
f_26818(t11,t7,((C_word*)t0)[3]);}

/* k26802 in k26798 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_26804,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record3(&a,3,lf[911],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k26814 in k26798 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26816(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26816,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[256]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop6052 in k26798 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_26818(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_26818,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_slot(t3,C_fix(2));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26860 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26862(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26862,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[256]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop6025 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_26864(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_26864,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_slot(t3,C_fix(1));
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* for-each-loop6007 in chicken.condition#make-composite-condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static C_word C_fcall f_26898(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_slot(t1,C_fix(0));
t3=C_i_check_structure_2(t2,lf[911],lf[1032]);
t5=C_slot(t1,C_fix(1));
t1=t5;
goto loop;}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_26921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +23,c,3)))){
C_save_and_reclaim((void*)f_26921,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+23);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
t4=C_a_i_cons(&a,2,t2,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26928,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_26990,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26992,a[2]=t8,a[3]=t12,a[4]=t9,a[5]=((C_word)li796),tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_26992(t14,t10,t4);}

/* k26926 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26928(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_26928,c,av);}
a=C_alloc(17);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_26938,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_26940,a[2]=t4,a[3]=t8,a[4]=t5,a[5]=((C_word)li795),tmp=(C_word)a,a+=6,tmp));
t10=((C_word*)t8)[1];
f_26940(t10,t6,((C_word*)t0)[3]);}

/* k26936 in k26926 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_26938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_26938,c,av);}
a=C_alloc(4);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_record3(&a,3,lf[911],t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop6112 in k26926 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_26940(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_26940,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k26988 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_26990(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_26990,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[256]+1);
av2[3]=t1;
C_apply(4,av2);}}

/* map-loop6085 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_fcall f_26992(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,4)))){
C_save_and_reclaim_args((void *)trf_26992,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27017,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
/* library.scm:5265: prop-list->kind-prefixed-prop-list */
f_26699(t3,lf[911],C_i_car(t4),C_u_i_cdr(t4));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k27015 in map-loop6085 in chicken.condition#condition in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_27017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_27017,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_26992(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.condition#condition? in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_27026,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[911]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_27032,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27036,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27145,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5273: condition? */
t5=C_fast_retrieve(lf[1034]);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_27036,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27040,a[2]=((C_word*)t0)[2],a[3]=((C_word)li800),tmp=(C_word)a,a+=4,tmp);
t7=C_slot(((C_word*)t0)[2],C_fix(1));
t8=C_i_check_list_2(t7,lf[689]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27110,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,a[6]=((C_word)li801),tmp=(C_word)a,a+=7,tmp));
t12=((C_word*)t10)[1];
f_27110(t12,((C_word*)t0)[3],t7);}

/* g6148 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_27040(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_27040,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27048,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_slot(((C_word*)t0)[2],C_fix(2));
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27054,a[2]=t2,a[3]=t6,a[4]=((C_word)li799),tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_27054(t8,t3,t4);}

/* k27046 in g6148 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_27048(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_27048,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in g6148 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_27054(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_27054,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_caar(t2);
t4=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t4)){
t5=C_u_i_car(t2);
t6=C_u_i_cdr(t5);
t7=C_i_cadr(t2);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27085,a[2]=t7,a[3]=t1,a[4]=t6,tmp=(C_word)a,a+=5,tmp);
t9=C_u_i_cdr(t2);
/* library.scm:5283: loop */
t11=t8;
t12=C_u_i_cdr(t9);
t1=t11;
t2=t12;
goto loop;}
else{
/* library.scm:5285: loop */
t11=t1;
t12=C_i_cddr(t2);
t1=t11;
t2=t12;
goto loop;}}}

/* k27083 in loop in g6148 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_27085,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop6142 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_fcall f_27110(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_27110,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27135,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5277: g6148 */
t4=((C_word*)t0)[4];
f_27040(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k27133 in map-loop6142 in k27034 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_27135(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_27135,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_27110(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k27143 in chicken.condition#condition->list in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_27145,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_27036(2,av2);}}
else{
/* library.scm:5274: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[260];
av2[3]=lf[1036];
av2[4]=lf[1037];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* chicken.condition#condition-predicate in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27150(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_27150,c,av);}
a=C_alloc(4);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27152,a[2]=t2,a[3]=((C_word)li803),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_27152 in chicken.condition#condition-predicate in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27152(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_27152,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27159,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5290: condition? */
t4=C_fast_retrieve(lf[1034]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k27157 */
static void C_ccall f_27159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_27159,c,av);}
if(C_truep(t1)){
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=C_i_memv(((C_word*)t0)[3],t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_SCHEME_TRUE:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.condition#condition-property-accessor in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +11,c,3)))){
C_save_and_reclaim((void*)f_27171,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+11);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
t5=C_i_nullp(t4);
t6=C_a_i_cons(&a,2,t2,t3);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27179,a[2]=t2,a[3]=t6,a[4]=t5,a[5]=t3,a[6]=t4,a[7]=((C_word)li805),tmp=(C_word)a,a+=8,tmp);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* f_27179 in chicken.condition#condition-property-accessor in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_27179,c,av);}
t3=C_i_check_structure(t2,lf[911]);
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_memv(((C_word*)t0)[2],t4))){
t5=C_slot(t2,C_fix(2));
t6=C_i_member(((C_word*)t0)[3],t5);
if(C_truep(t6)){
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_cadr(t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
if(C_truep(((C_word*)t0)[4])){
/* library.scm:5301: ##sys#signal-hook */
t7=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[1040];
av2[4]=lf[1041];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_car(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* chicken.condition#get-condition-property in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27217(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c<5) C_bad_min_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-5)*C_SIZEOF_PAIR +4,c,5)))){
C_save_and_reclaim((void*)f_27217,c,av);}
a=C_alloc((c-5)*C_SIZEOF_PAIR+4);
t5=C_build_rest(&a,c,5,av);
C_word t6;
C_word t7;
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27221,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=t6;
av2[2]=C_fast_retrieve(lf[1039]);
av2[3]=t3;
av2[4]=t4;
av2[5]=t5;
C_apply(6,av2);}}

/* k27219 in chicken.condition#get-condition-property in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27221,c,av);}
/* library.scm:5307: g6197 */
t2=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_27228(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_27228,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27231,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5319: condition-property-accessor */
t3=C_fast_retrieve(lf[1039]);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1012];
av2[3]=lf[845];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_27231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_27231,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27234,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5320: condition-property-accessor */
t3=C_fast_retrieve(lf[1039]);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1012];
av2[3]=lf[1376];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_27234(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(182,c,7)))){
C_save_and_reclaim((void *)f_27234,c,av);}
a=C_alloc(182);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27235,a[2]=((C_word)li811),tmp=(C_word)a,a+=3,tmp);
t3=C_mutate((C_word*)lf[1043]+1 /* (set! chicken.condition#print-error-message ...) */,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27280,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word)li816),tmp=(C_word)a,a+=7,tmp));
t4=C_fast_retrieve(lf[1043]);
t5=*((C_word*)lf[187]+1);
t6=*((C_word*)lf[215]+1);
t7=*((C_word*)lf[213]+1);
t8=C_mutate((C_word*)lf[1055]+1 /* (set! ##sys#show-exception-warning ...) */,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27471,a[2]=t5,a[3]=t4,a[4]=t7,a[5]=t6,a[6]=((C_word)li817),tmp=(C_word)a,a+=7,tmp));
t9=C_mutate((C_word*)lf[366]+1 /* (set! ##sys#error-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27520,a[2]=((C_word)li818),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[1119]+1 /* (set! ##sys#structure? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28226,a[2]=((C_word)li819),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[1120]+1 /* (set! ##sys#generic-structure? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28229,a[2]=((C_word)li820),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[1121]+1 /* (set! ##sys#slot ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28232,a[2]=((C_word)li821),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[1122]+1 /* (set! ##sys#size ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28235,a[2]=((C_word)li822),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[1123]+1 /* (set! ##sys#make-pointer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_make_pointer,a[2]=((C_word)li823),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[1124]+1 /* (set! ##sys#make-tagged-pointer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_make_tagged_pointer,a[2]=((C_word)li824),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate((C_word*)lf[1125]+1 /* (set! ##sys#pointer? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28240,a[2]=((C_word)li825),tmp=(C_word)a,a+=3,tmp));
t17=C_mutate((C_word*)lf[1126]+1 /* (set! ##sys#set-pointer-address! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28243,a[2]=((C_word)li826),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate((C_word*)lf[1127]+1 /* (set! ##sys#bytevector? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28246,a[2]=((C_word)li827),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[1128]+1 /* (set! ##sys#string->pbytevector ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28249,a[2]=((C_word)li828),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[1129]+1 /* (set! ##sys#permanent? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28252,a[2]=((C_word)li829),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[1130]+1 /* (set! ##sys#block-address ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28255,a[2]=((C_word)li830),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[1131]+1 /* (set! ##sys#locative? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28258,a[2]=((C_word)li831),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[1132]+1 /* (set! ##sys#srfi-4-vector? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28261,a[2]=((C_word)li832),tmp=(C_word)a,a+=3,tmp));
t24=C_mutate((C_word*)lf[1133]+1 /* (set! ##sys#null-pointer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28264,a[2]=((C_word)li833),tmp=(C_word)a,a+=3,tmp));
t25=C_mutate((C_word*)lf[1134]+1 /* (set! ##sys#null-pointer? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28270,a[2]=((C_word)li834),tmp=(C_word)a,a+=3,tmp));
t26=C_mutate((C_word*)lf[1135]+1 /* (set! ##sys#address->pointer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28280,a[2]=((C_word)li835),tmp=(C_word)a,a+=3,tmp));
t27=C_mutate((C_word*)lf[940]+1 /* (set! ##sys#pointer->address ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28286,a[2]=((C_word)li836),tmp=(C_word)a,a+=3,tmp));
t28=C_mutate((C_word*)lf[745]+1 /* (set! ##sys#make-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28293,a[2]=((C_word)li837),tmp=(C_word)a,a+=3,tmp));
t29=C_mutate((C_word*)lf[1136]+1 /* (set! ##sys#peek-signed-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_peek_signed_integer,a[2]=((C_word)li838),tmp=(C_word)a,a+=3,tmp));
t30=C_mutate((C_word*)lf[723]+1 /* (set! ##sys#peek-unsigned-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_peek_unsigned_integer,a[2]=((C_word)li839),tmp=(C_word)a,a+=3,tmp));
t31=C_mutate((C_word*)lf[1137]+1 /* (set! ##sys#peek-fixnum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28328,a[2]=((C_word)li840),tmp=(C_word)a,a+=3,tmp));
t32=C_mutate((C_word*)lf[1138]+1 /* (set! ##sys#peek-byte ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28331,a[2]=((C_word)li841),tmp=(C_word)a,a+=3,tmp));
t33=C_mutate((C_word*)lf[1139]+1 /* (set! ##sys#vector->structure! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28334,a[2]=((C_word)li842),tmp=(C_word)a,a+=3,tmp));
t34=C_mutate((C_word*)lf[1140]+1 /* (set! ##sys#peek-double ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28337,a[2]=((C_word)li843),tmp=(C_word)a,a+=3,tmp));
t35=C_mutate((C_word*)lf[702]+1 /* (set! ##sys#peek-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28340,a[2]=((C_word)li844),tmp=(C_word)a,a+=3,tmp));
t36=C_mutate((C_word*)lf[944]+1 /* (set! ##sys#peek-nonnull-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28356,a[2]=((C_word)li845),tmp=(C_word)a,a+=3,tmp));
t37=C_mutate((C_word*)lf[1141]+1 /* (set! ##sys#peek-and-free-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28362,a[2]=((C_word)li846),tmp=(C_word)a,a+=3,tmp));
t38=C_mutate((C_word*)lf[1142]+1 /* (set! ##sys#peek-and-free-nonnull-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28378,a[2]=((C_word)li847),tmp=(C_word)a,a+=3,tmp));
t39=C_mutate((C_word*)lf[1143]+1 /* (set! ##sys#poke-c-string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28384,a[2]=((C_word)li848),tmp=(C_word)a,a+=3,tmp));
t40=C_mutate((C_word*)lf[1144]+1 /* (set! ##sys#poke-integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28391,a[2]=((C_word)li849),tmp=(C_word)a,a+=3,tmp));
t41=C_mutate((C_word*)lf[1145]+1 /* (set! ##sys#poke-double ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28394,a[2]=((C_word)li850),tmp=(C_word)a,a+=3,tmp));
t42=C_mutate((C_word*)lf[1146]+1 /* (set! ##sys#peek-c-string-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28414,a[2]=((C_word)li852),tmp=(C_word)a,a+=3,tmp));
t43=C_mutate((C_word*)lf[1147]+1 /* (set! ##sys#peek-and-free-c-string-list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28475,a[2]=((C_word)li854),tmp=(C_word)a,a+=3,tmp));
t44=C_mutate((C_word*)lf[1148]+1 /* (set! ##sys#vector->closure! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28513,a[2]=((C_word)li855),tmp=(C_word)a,a+=3,tmp));
t45=C_mutate((C_word*)lf[1149]+1 /* (set! ##sys#symbol-has-toplevel-binding? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28516,a[2]=((C_word)li856),tmp=(C_word)a,a+=3,tmp));
t46=C_mutate((C_word*)lf[922]+1 /* (set! ##sys#copy-bytes ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28519,a[2]=((C_word)li857),tmp=(C_word)a,a+=3,tmp));
t47=C_mutate((C_word*)lf[1150]+1 /* (set! ##sys#copy-words ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28526,a[2]=((C_word)li858),tmp=(C_word)a,a+=3,tmp));
t48=C_mutate((C_word*)lf[1151]+1 /* (set! ##sys#compare-bytes ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28533,a[2]=((C_word)li859),tmp=(C_word)a,a+=3,tmp));
t49=C_mutate((C_word*)lf[1152]+1 /* (set! ##sys#block-pointer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28536,a[2]=((C_word)li860),tmp=(C_word)a,a+=3,tmp));
t50=C_mutate((C_word*)lf[1153]+1 /* (set! ##sys#foreign-char-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28542,a[2]=((C_word)li861),tmp=(C_word)a,a+=3,tmp));
t51=C_mutate((C_word*)lf[1154]+1 /* (set! ##sys#foreign-fixnum-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28545,a[2]=((C_word)li862),tmp=(C_word)a,a+=3,tmp));
t52=C_mutate((C_word*)lf[1155]+1 /* (set! ##sys#foreign-flonum-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28548,a[2]=((C_word)li863),tmp=(C_word)a,a+=3,tmp));
t53=C_mutate((C_word*)lf[1156]+1 /* (set! ##sys#foreign-block-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28551,a[2]=((C_word)li864),tmp=(C_word)a,a+=3,tmp));
t54=C_mutate((C_word*)lf[1157]+1 /* (set! ##sys#foreign-struct-wrapper-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28554,a[2]=((C_word)li865),tmp=(C_word)a,a+=3,tmp));
t55=C_mutate((C_word*)lf[1158]+1 /* (set! ##sys#foreign-string-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28557,a[2]=((C_word)li866),tmp=(C_word)a,a+=3,tmp));
t56=C_mutate((C_word*)lf[1159]+1 /* (set! ##sys#foreign-symbol-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28560,a[2]=((C_word)li867),tmp=(C_word)a,a+=3,tmp));
t57=C_mutate((C_word*)lf[1160]+1 /* (set! ##sys#foreign-pointer-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28563,a[2]=((C_word)li868),tmp=(C_word)a,a+=3,tmp));
t58=C_mutate((C_word*)lf[1161]+1 /* (set! ##sys#foreign-tagged-pointer-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28566,a[2]=((C_word)li869),tmp=(C_word)a,a+=3,tmp));
t59=C_mutate((C_word*)lf[1162]+1 /* (set! ##sys#foreign-ranged-integer-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28569,a[2]=((C_word)li870),tmp=(C_word)a,a+=3,tmp));
t60=C_mutate((C_word*)lf[1163]+1 /* (set! ##sys#foreign-unsigned-ranged-integer-argument ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28572,a[2]=((C_word)li871),tmp=(C_word)a,a+=3,tmp));
t61=C_set_block_item(lf[1164] /* ##sys#default-thread-quantum */,0,C_fix(10000));
t62=C_mutate((C_word*)lf[1165]+1 /* (set! ##sys#default-exception-handler ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28576,a[2]=((C_word)li872),tmp=(C_word)a,a+=3,tmp));
t63=C_mutate((C_word*)lf[1167]+1 /* (set! ##sys#make-thread ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28579,a[2]=((C_word)li873),tmp=(C_word)a,a+=3,tmp));
t64=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28603,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5683: ##sys#make-thread */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1167]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[1167]+1);
av2[1]=t64;
av2[2]=C_SCHEME_FALSE;
av2[3]=lf[1374];
av2[4]=lf[1375];
av2[5]=C_fast_retrieve(lf[1164]);
tp(6,av2);}}

/* writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_fcall f_27235(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_27235,3,t1,t2,t3);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27237,a[2]=t3,a[3]=((C_word)li809),tmp=(C_word)a,a+=4,tmp);
t5=C_i_check_list_2(t2,lf[687]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27258,a[2]=t7,a[3]=t4,a[4]=((C_word)li810),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_27258(t9,t1,t2);}

/* g6213 in writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_27237(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_27237,3,t0,t1,t2);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27241,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27246,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word)li808),tmp=(C_word)a,a+=5,tmp);
/* library.scm:5325: ##sys#with-print-length-limit */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[921]);
C_word av2[4];
av2[0]=*((C_word*)lf[921]+1);
av2[1]=t3;
av2[2]=C_fix(80);
av2[3]=t4;
tp(4,av2);}}

/* k27239 in g6213 in writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27241,c,av);}
/* library.scm:5326: newline */
t2=*((C_word*)lf[151]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a27245 in g6213 in writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27246,c,av);}
/* library.scm:5325: write */
t2=*((C_word*)lf[152]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop6212 in writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_27258(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_27258,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27268,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5323: g6213 */
t4=((C_word*)t0)[3];
f_27237(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k27266 in for-each-loop6212 in writeargs in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27268,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_27258(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_27280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_27280,c,av);}
a=C_alloc(16);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27282,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word)li813),tmp=(C_word)a,a+=8,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27431,a[2]=t3,a[3]=((C_word)li814),tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27436,a[2]=t4,a[3]=((C_word)li815),tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_rest_nullp(c,3))){
/* library.scm:5329: def-port6235 */
t6=t5;
f_27436(t6,t1);}
else{
t6=C_get_rest_arg(c,3,av,3,t0);
if(C_truep(C_rest_nullp(c,4))){
/* library.scm:5329: def-header6236 */
t7=t4;
f_27431(t7,t1,t6);}
else{
t7=C_get_rest_arg(c,4,av,3,t0);
/* library.scm:5329: body6233 */
t8=t3;
f_27282(t8,t1,t6,t7);}}}

/* body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_27282(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_27282,4,t0,t1,t2,t3);}
a=C_alloc(10);
t4=C_i_check_port_2(t2,C_fix(2),C_SCHEME_TRUE,lf[1044]);
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27289,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=t3,tmp=(C_word)a,a+=10,tmp);
/* library.scm:5332: newline */
t6=*((C_word*)lf[151]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_27289,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_27292,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:5333: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_27292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_27292,c,av);}
a=C_alloc(8);
t2=C_immp(((C_word*)t0)[2]);
t3=C_i_not(t2);
t4=(C_truep(t3)?C_eqp(lf[911],C_slot(((C_word*)t0)[2],C_fix(0))):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_27301,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:5335: errmsg */
t6=((C_word*)t0)[8];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[2]))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27400,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5359: display */
t6=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[1052];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27409,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5363: display */
t6=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[1053];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}

/* k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_27301(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_27301,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27304,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27336,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[6],a[5]=((C_word)li812),tmp=(C_word)a,a+=6,tmp);
/* library.scm:5335: g6256 */
t4=t3;
f_27336(t4,t2,t1);}
else{
t3=C_slot(((C_word*)t0)[6],C_fix(1));
if(C_truep(C_i_equalp(lf[1049],t3))){
/* library.scm:5345: display */
t4=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[1050];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27384,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5347: display */
t5=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[1051];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}}

/* k27302 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_27304(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_27304,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27307,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5349: errargs */
t3=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27305 in k27302 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_27307(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_27307,c,av);}
a=C_alloc(6);
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_length(t1);
t4=C_eqp(C_fix(1),t3);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27322,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5353: display */
t6=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[1045];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_27328,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5356: newline */
t6=*((C_word*)lf[151]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}

/* k27320 in k27305 in k27302 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_27322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27322,c,av);}
/* library.scm:5354: writeargs */
f_27235(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k27326 in k27305 in k27302 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_27328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27328,c,av);}
/* library.scm:5357: writeargs */
f_27235(((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);}

/* g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_fcall f_27336(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_27336,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_27340,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* library.scm:5337: display */
t4=*((C_word*)lf[153]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1048];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k27338 in g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_27340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_27340,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27343,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5338: errloc */
t3=((C_word*)t0)[5];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k27341 in k27338 in g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_27343(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_27343,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27346,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(C_truep(t1)?C_i_symbolp(t1):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27359,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_27363,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5340: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t5;
av2[2]=t1;
tp(3,av2);}}
else{
/* library.scm:5341: display */
t4=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k27344 in k27341 in k27338 in g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_27346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27346,c,av);}
/* library.scm:5341: display */
t2=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k27357 in k27341 in k27338 in g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_27359(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27359,c,av);}
/* library.scm:5340: display */
t2=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k27361 in k27341 in k27338 in g6256 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_27363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_27363,c,av);}
/* library.scm:5340: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1046];
av2[3]=t1;
av2[4]=lf[1047];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k27382 in k27299 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_27384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27384,c,av);}
/* library.scm:5348: display */
t2=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_slot(((C_word*)t0)[3],C_fix(1));
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k27398 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_27400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_27400,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27403,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5360: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27401 in k27398 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_27403(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_27403,c,av);}
/* library.scm:5361: newline */
t2=*((C_word*)lf[151]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k27407 in k27290 in k27287 in body6233 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_27409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_27409,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
/* library.scm:5364: writeargs */
f_27235(((C_word*)t0)[4],t2,((C_word*)t0)[5]);}

/* def-header6236 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_27431(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_27431,3,t0,t1,t2);}
/* library.scm:5329: body6233 */
t3=((C_word*)t0)[2];
f_27282(t3,t1,t2,lf[1054]);}

/* def-port6235 in chicken.condition#print-error-message in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_27436(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,2)))){
C_save_and_reclaim_args((void *)trf_27436,2,t0,t1);}
/* library.scm:5329: def-header6236 */
t2=((C_word*)t0)[2];
f_27431(t2,t1,*((C_word*)lf[714]+1));}

/* ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_27471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_27471,c,av);}
a=C_alloc(9);
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_fast_retrieve(lf[943]):C_get_rest_arg(c,4,av,4,t0));
if(C_truep(*((C_word*)lf[306]+1))){
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_27481,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,a[5]=((C_word*)t0)[3],a[6]=t2,a[7]=((C_word*)t0)[4],a[8]=t3,tmp=(C_word)a,a+=9,tmp);
/* library.scm:5379: open-output-string */
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_27481,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27484,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* library.scm:5380: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1058];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_27484,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27487,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27506,a[2]=t2,a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5382: display */
t4=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1057];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_27487(2,av2);}}}

/* k27485 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_27487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_27487,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_27490,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* library.scm:5385: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1056];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27488 in k27485 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_27490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_27490,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_27493,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* library.scm:5386: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27491 in k27488 in k27485 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_27493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_27493,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27496,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27503,a[2]=((C_word*)t0)[5],a[3]=t2,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5387: get-output-string */
t4=((C_word*)t0)[7];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k27494 in k27491 in k27488 in k27485 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_27496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_27496,c,av);}
/* library.scm:5388: print-call-chain */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=*((C_word*)lf[715]+1);
av2[3]=C_fix(0);
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k27501 in k27491 in k27488 in k27485 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_27503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_27503,c,av);}
/* library.scm:5387: print-error-message */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=*((C_word*)lf[715]+1);
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k27504 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_27506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_27506,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27509,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5383: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k27507 in k27504 in k27482 in k27479 in ##sys#show-exception-warning in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_27509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_27509,c,av);}
/* ##sys#write-char/port */
t2=C_fast_retrieve(lf[864]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(41);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_27520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-4)*C_SIZEOF_PAIR +9,c,6)))){
C_save_and_reclaim((void*)f_27520,c,av);}
a=C_alloc((c-4)*C_SIZEOF_PAIR+9);
t4=C_build_rest(&a,c,4,av);
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
switch(t2){
case C_fix(1):
t5=C_get_rest_arg(c,4,av,4,t0);
t6=C_i_cadr(t4);
t7=C_i_caddr(t4);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27543,a[2]=t7,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27554,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5403: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t9;
av2[2]=t6;
tp(3,av2);}
case C_fix(2):
t5=C_get_rest_arg(c,4,av,4,t0);
t6=C_i_cadr(t4);
t7=C_i_caddr(t4);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_27580,a[2]=t7,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27591,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5412: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t9;
av2[2]=t6;
tp(3,av2);}
case C_fix(3):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1063];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(4):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[981];
av2[4]=t3;
av2[5]=lf[1064];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(5):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1065];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(6):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[989];
av2[4]=t3;
av2[5]=lf[1066];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(7):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[565];
av2[4]=t3;
av2[5]=lf[1067];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(8):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[383];
av2[4]=t3;
av2[5]=lf[1068];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(9):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1069];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(10):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[991];
av2[4]=t3;
av2[5]=lf[1070];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(11):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1071];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(12):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[989];
av2[4]=t3;
av2[5]=lf[1072];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(13):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1073];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(14):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1074];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(15):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1075];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(16):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1076];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(17):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1077];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(18):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1078];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(19):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1079];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(20):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1080];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(21):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1081];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(22):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1082];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(23):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[989];
av2[4]=t3;
av2[5]=lf[1083];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(24):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1084];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(25):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1085];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(26):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1086];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(27):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1087];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(28):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1088];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(29):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1089];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(30):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1090];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(31):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1091];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(32):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1092];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(33):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1093];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(34):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1094];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(35):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1095];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(36):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[989];
av2[4]=t3;
av2[5]=lf[1096];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(37):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1097];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(38):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1098];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(39):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1099];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(40):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1100];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(41):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1101];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(42):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1102];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(43):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[699];
av2[4]=t3;
av2[5]=lf[1103];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(44):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1104];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(45):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[997];
av2[4]=t3;
av2[5]=lf[1105];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(46):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[565];
av2[4]=t3;
av2[5]=lf[1106];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(47):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[981];
av2[4]=t3;
av2[5]=lf[1107];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(48):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[997];
av2[4]=t3;
av2[5]=lf[1108];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(49):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1109];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(50):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1110];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(51):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1111];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(52):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1112];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(53):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1113];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(54):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1114];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(55):{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=t3;
av2[5]=lf[1115];
av2[6]=t4;
C_apply(7,av2);}
case C_fix(56):
t5=C_get_rest_arg(c,4,av,4,t0);
t6=C_i_cadr(t4);
t7=C_i_caddr(t4);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28094,a[2]=t7,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28105,a[2]=t8,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5474: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t9;
av2[2]=t6;
tp(3,av2);}
default:{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[981];
av2[4]=t3;
av2[5]=lf[1118];
av2[6]=t4;
C_apply(7,av2);}}}

/* k27541 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_27543,c,av);}
a=C_alloc(3);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[991];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
av2[6]=t2;
C_apply(7,av2);}}
else{
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[194]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[194]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[991];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
tp(5,av2);}}}

/* k27552 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27554,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27558,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5404: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k27556 in k27552 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27558(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_27558,c,av);}
/* library.scm:5403: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1059];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1060];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k27578 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_27580,c,av);}
a=C_alloc(3);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[991];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
av2[6]=t2;
C_apply(7,av2);}}
else{
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[194]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[194]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[991];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
tp(5,av2);}}}

/* k27589 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_27591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_27591,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_27595,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5413: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k27593 in k27589 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_27595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_27595,c,av);}
/* library.scm:5412: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1061];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1062];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k28092 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,6)))){
C_save_and_reclaim((void *)f_28094,c,av);}
a=C_alloc(3);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[383];
av2[4]=((C_word*)t0)[4];
av2[5]=t1;
av2[6]=t2;
C_apply(7,av2);}}
else{
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[194]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[194]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=lf[383];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
tp(5,av2);}}}

/* k28103 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28105,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28109,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5475: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k28107 in k28103 in ##sys#error-hook in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_28109,c,av);}
/* library.scm:5474: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1116];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1117];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* ##sys#structure? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28226,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_structurep(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#generic-structure? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28229,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_structurep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#slot in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28232(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28232,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#size in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28235,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_block_size(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#pointer? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28240,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_anypointerp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#set-pointer-address! in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28243,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_update_pointer(t3,t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#bytevector? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28246,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_bytevectorp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#string->pbytevector in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28249(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28249,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_string_to_pbytevector(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#permanent? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28252(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28252,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_permanentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#block-address in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28255(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_28255,c,av);}
a=C_alloc(6);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_block_address(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#locative? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28258,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_locativep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#srfi-4-vector? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28261(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28261,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_srfi_4_vectorp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#null-pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_28264,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28268,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5501: ##sys#make-pointer */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1123]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1123]+1);
av2[1]=t2;
tp(2,av2);}}

/* k28266 in ##sys#null-pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28268,c,av);}
t2=C_update_pointer(C_fix(0),t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#null-pointer? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_28270,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28278,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5506: ##sys#pointer->address */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[940]);
C_word *av2=av;
av2[0]=*((C_word*)lf[940]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}

/* k28276 in ##sys#null-pointer? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28278,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(C_fix(0),t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#address->pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28280,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28284,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5509: ##sys#make-pointer */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1123]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1123]+1);
av2[1]=t3;
tp(2,av2);}}

/* k28282 in ##sys#address->pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28284(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28284,c,av);}
t2=C_update_pointer(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#pointer->address in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_28286,c,av);}
a=C_alloc(6);
t3=C_slot(t2,C_fix(0));
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_unsigned_int_to_num(&a,1,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#make-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28293,c,av);}
a=C_alloc(6);
t3=C_rest_nullp(c,3);
t4=(C_truep(t3)?lf[745]:C_get_rest_arg(c,3,av,3,t0));
t5=C_block_size(t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28303,a[2]=t2,a[3]=t5,a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5519: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t6;
av2[2]=C_fixnum_plus(t5,C_fix(1));
tp(3,av2);}}

/* k28301 in ##sys#make-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_28303,c,av);}
t2=C_substring_copy(((C_word*)t0)[2],t1,C_fix(0),((C_word*)t0)[3],C_fix(0));
t3=C_setsubchar(t1,((C_word*)t0)[3],C_make_character(0));
t4=C_eqp(C_asciiz_strlen(t1),((C_word*)t0)[3]);
if(C_truep(t4)){
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_fix((C_word)C_ASCIIZ_REPRESENTATION_ERROR);
/* library.scm:5524: ##sys#error-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[366]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[366]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=t5;
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[2];
tp(5,av2);}}}

/* ##sys#peek-fixnum in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28328,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_peek_fixnum(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#peek-byte in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28331(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28331,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_peek_byte(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#vector->structure! in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28334(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28334,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_vector_to_structure(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#peek-double in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_28337,c,av);}
a=C_alloc(4);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_f64peek(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#peek-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28340,c,av);}
a=C_alloc(6);
t4=C_null_pointerp(t2);
if(C_truep(C_i_not(t4))){
t5=C_fetch_c_strlen(t2,t3);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28350,a[2]=t2,a[3]=t3,a[4]=t5,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5540: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t6;
av2[2]=t5;
tp(3,av2);}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k28348 in ##sys#peek-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28350,c,av);}
t2=C_peek_c_string(((C_word*)t0)[2],((C_word*)t0)[3],t1,((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#peek-nonnull-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28356(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28356,c,av);}
a=C_alloc(6);
t4=C_fetch_c_strlen(t2,t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28360,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5546: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k28358 in ##sys#peek-nonnull-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28360(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28360,c,av);}
t2=C_peek_c_string(((C_word*)t0)[2],((C_word*)t0)[3],t1,((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#peek-and-free-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28362(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28362,c,av);}
a=C_alloc(6);
t4=C_null_pointerp(t2);
if(C_truep(C_i_not(t4))){
t5=C_fetch_c_strlen(t2,t3);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28372,a[2]=t2,a[3]=t3,a[4]=t5,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5553: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t6;
av2[2]=t5;
tp(3,av2);}}
else{
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k28370 in ##sys#peek-and-free-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28372,c,av);}
t2=C_peek_c_string(((C_word*)t0)[2],((C_word*)t0)[3],t1,((C_word*)t0)[4]);
t3=C_free_mptr(((C_word*)t0)[2],((C_word*)t0)[3]);
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#peek-and-free-nonnull-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28378,c,av);}
a=C_alloc(6);
t4=C_fetch_c_strlen(t2,t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28382,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5560: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k28380 in ##sys#peek-and-free-nonnull-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28382,c,av);}
t2=C_peek_c_string(((C_word*)t0)[2],((C_word*)t0)[3],t1,((C_word*)t0)[4]);
t3=C_free_mptr(((C_word*)t0)[2],((C_word*)t0)[3]);
t4=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#poke-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28384,c,av);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28389,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:5566: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2=av;
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t5;
av2[2]=t4;
tp(3,av2);}}

/* k28387 in ##sys#poke-c-string in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28389,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_poke_c_string(((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#poke-integer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28391,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_poke_integer(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#poke-double in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28394(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28394,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_poke_double(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#peek-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_28414,c,av);}
a=C_alloc(8);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28420,a[2]=t3,a[3]=t5,a[4]=t2,a[5]=((C_word)li851),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_28420(t7,t1,C_fix(0));}

/* loop in ##sys#peek-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_28420(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_28420,3,t0,t1,t2);}
a=C_alloc(10);
t3=(C_truep(((C_word*)t0)[2])?C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28430,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=C_a_i_bytevector(&a,1,C_fix(3));
t6=(C_truep(((C_word*)t0)[4])?C_i_foreign_pointer_argumentp(((C_word*)t0)[4]):C_SCHEME_FALSE);
t7=C_i_foreign_fixnum_argumentp(t2);
/* library.scm:5572: ##sys#peek-c-string */
t8=*((C_word*)lf[702]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t4;
av2[2]=stub6476(t5,t6,t7);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}

/* k28428 in loop in ##sys#peek-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28430,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28440,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5579: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_28420(t3,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k28438 in k28428 in loop in ##sys#peek-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_28440,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#peek-and-free-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_28475,c,av);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28479,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28484,a[2]=t3,a[3]=t2,a[4]=t6,a[5]=((C_word)li853),tmp=(C_word)a,a+=6,tmp));
t8=((C_word*)t6)[1];
f_28484(t8,t4,C_fix(0));}

/* k28477 in ##sys#peek-and-free-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28479,c,av);}
if(C_truep(((C_word*)t0)[2])){
t2=C_i_foreign_pointer_argumentp(((C_word*)t0)[2]);
t3=stub6503(C_SCHEME_UNDEFINED,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=stub6503(C_SCHEME_UNDEFINED,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* loop in ##sys#peek-and-free-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_28484(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_28484,3,t0,t1,t2);}
a=C_alloc(11);
t3=(C_truep(((C_word*)t0)[2])?C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28494,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_bytevector(&a,1,C_fix(3));
t6=(C_truep(((C_word*)t0)[3])?C_i_foreign_pointer_argumentp(((C_word*)t0)[3]):C_SCHEME_FALSE);
t7=C_i_foreign_fixnum_argumentp(t2);
/* library.scm:5583: ##sys#peek-c-string */
t8=*((C_word*)lf[702]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t4;
av2[2]=stub6495(t5,t6,t7);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}

/* k28492 in loop in ##sys#peek-and-free-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28494,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=C_free_sptr(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28504,a[2]=((C_word*)t0)[4],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5592: loop */
t4=((C_word*)((C_word*)t0)[5])[1];
f_28484(t4,t3,C_fixnum_plus(((C_word*)t0)[3],C_fix(1)));}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k28502 in k28492 in loop in ##sys#peek-and-free-c-string-list in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_28504,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* ##sys#vector->closure! in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28513,c,av);}
t4=C_vector_to_closure(t2);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_update_pointer(t3,t2);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* ##sys#symbol-has-toplevel-binding? in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28516(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28516,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_boundp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#copy-bytes in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28519(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(c!=7) C_bad_argc_2(c,7,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28519,c,av);}
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_substring_copy(t2,t3,t4,C_fixnum_plus(t4,t6),t5);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* ##sys#copy-words in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(c!=7) C_bad_argc_2(c,7,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28526,c,av);}
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_subvector_copy(t2,t3,t4,C_fixnum_plus(t4,t6),t5);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* ##sys#compare-bytes in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28533(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6=av[6];
C_word t7;
C_word *a;
if(c!=7) C_bad_argc_2(c,7,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28533,c,av);}
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_substring_compare(t2,t3,t4,t5,t6);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* ##sys#block-pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28536(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28536,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28540,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5625: ##sys#make-pointer */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1123]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1123]+1);
av2[1]=t3;
tp(2,av2);}}

/* k28538 in ##sys#block-pointer in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28540,c,av);}
t2=C_pointer_to_block(t1,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-char-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28542(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28542,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_char_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-fixnum-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28545,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_fixnum_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-flonum-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28548,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_flonum_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-block-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28551(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28551,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_block_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-struct-wrapper-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28554,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_foreign_struct_wrapper_argumentp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#foreign-string-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28557,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_string_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-symbol-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28560,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_symbol_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-pointer-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28563,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_foreign_pointer_argumentp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#foreign-tagged-pointer-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28566(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28566,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_foreign_tagged_pointer_argumentp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#foreign-ranged-integer-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28569,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_foreign_ranged_integer_argumentp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#foreign-unsigned-ranged-integer-argument in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28572(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28572,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_foreign_unsigned_ranged_integer_argumentp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#default-exception-handler in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28576,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_halt(lf[1166]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#make-thread in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_28579,c,av);}
a=C_alloc(7);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_28595,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* library.scm:5670: ##sys#vector-resize */
f_18160(t6,C_fast_retrieve(lf[762]),C_block_size(C_fast_retrieve(lf[762])),C_SCHEME_FALSE);}

/* k28593 in ##sys#make-thread in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,1)))){
C_save_and_reclaim((void *)f_28595,c,av);}
a=C_alloc(26);
t2=C_a_i_vector6(&a,6,C_fast_retrieve(lf[690]),*((C_word*)lf[713]+1),*((C_word*)lf[714]+1),*((C_word*)lf[715]+1),C_fast_retrieve(lf[1165]),t1);
t3=C_a_i_cons(&a,2,C_SCHEME_FALSE,C_SCHEME_FALSE);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_record(&a,15,lf[918],((C_word*)t0)[3],C_SCHEME_FALSE,((C_word*)t0)[4],C_SCHEME_FALSE,t2,((C_word*)t0)[5],C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST,((C_word*)t0)[6],C_SCHEME_UNDEFINED,C_SCHEME_FALSE,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_28603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,4)))){
C_save_and_reclaim((void *)f_28603,c,av);}
a=C_alloc(24);
t2=C_mutate((C_word*)lf[955]+1 /* (set! ##sys#primordial-thread ...) */,t1);
t3=C_mutate((C_word*)lf[943]+1 /* (set! ##sys#current-thread ...) */,C_fast_retrieve(lf[955]));
t4=C_mutate((C_word*)lf[1168]+1 /* (set! ##sys#make-mutex ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28606,a[2]=((C_word)li874),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[1170]+1 /* (set! ##sys#schedule ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28612,a[2]=((C_word)li875),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1171]+1 /* (set! ##sys#thread-yield! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28621,a[2]=((C_word)li878),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[1172]+1 /* (set! ##sys#kill-other-threads ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28642,a[2]=((C_word)li879),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[1173]+1 /* (set! chicken.base#sleep-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28648,a[2]=((C_word)li880),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[197]+1 /* (set! chicken.base#sleep ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28651,a[2]=((C_word)li881),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[1175]+1 /* (set! ##sys#context-switch ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_context_switch,a[2]=((C_word)li882),tmp=(C_word)a,a+=3,tmp));
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28663,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5726: scheme#make-vector */
t12=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t11;
av2[2]=C_fix(256);
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}

/* ##sys#make-mutex in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_28606,c,av);}
a=C_alloc(8);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_record7(&a,7,lf[1169],t2,t3,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE,C_SCHEME_FALSE,C_SCHEME_UNDEFINED);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* ##sys#schedule in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28612,c,av);}
/* library.scm:5697: g6589 */
t2=C_slot(C_fast_retrieve(lf[943]),C_fix(1));{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)C_fast_retrieve_proc(t2))(2,av2);}}

/* ##sys#thread-yield! in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_28621,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28627,a[2]=((C_word)li877),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5700: ##sys#call-with-current-continuation */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
C_call_cc(3,av2);}}

/* a28626 in ##sys#thread-yield! in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28627,c,av);}
a=C_alloc(4);
t3=C_fast_retrieve(lf[943]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28636,a[2]=t2,a[3]=((C_word)li876),tmp=(C_word)a,a+=4,tmp);
t5=C_i_setslot(C_fast_retrieve(lf[943]),C_fix(1),t4);
/* library.scm:5704: ##sys#schedule */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1170]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1170]+1);
av2[1]=t1;
tp(2,av2);}}

/* a28635 in a28626 in ##sys#thread-yield! in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28636(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28636,c,av);}
/* library.scm:5703: return */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=C_SCHEME_UNDEFINED;
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* ##sys#kill-other-threads in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28642,c,av);}
/* library.scm:5707: thunk */
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)C_fast_retrieve_proc(t3))(2,av2);}}

/* chicken.base#sleep-hook in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28648,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_process_sleep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#sleep in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_28651,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28655,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5717: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[1174];
tp(4,av2);}}

/* k28653 in chicken.base#sleep in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_28655,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28658,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5718: chicken.base#sleep-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1173]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[1173]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k28656 in k28653 in chicken.base#sleep in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28658,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_28663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(55,c,4)))){
C_save_and_reclaim((void *)f_28663,c,av);}
a=C_alloc(55);
t2=C_mutate((C_word*)lf[1176]+1 /* (set! ##sys#signal-vector ...) */,t1);
t3=C_mutate((C_word*)lf[1177]+1 /* (set! ##sys#interrupt-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28665,a[2]=((C_word)li884),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[698]+1 /* (set! ##sys#dispatch-interrupt ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28715,a[2]=((C_word)li885),tmp=(C_word)a,a+=3,tmp));
t5=C_fix(0);
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_mutate((C_word*)lf[703]+1 /* (set! ##sys#update-errno ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28722,a[2]=t6,a[3]=((C_word)li886),tmp=(C_word)a,a+=4,tmp));
t8=C_mutate((C_word*)lf[1178]+1 /* (set! ##sys#errno ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28726,a[2]=t6,a[3]=((C_word)li887),tmp=(C_word)a,a+=4,tmp));
t9=C_fast_retrieve(lf[856]);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28740,a[2]=((C_word)li889),tmp=(C_word)a,a+=3,tmp));
t15=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_28787,a[2]=((C_word)li891),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate((C_word*)lf[856]+1 /* (set! ##sys#user-read-hook ...) */,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28832,a[2]=t9,a[3]=t11,a[4]=t13,a[5]=((C_word)li896),tmp=(C_word)a,a+=6,tmp));
t17=C_a_i_provide(&a,1,lf[1189]);
t18=C_mutate((C_word*)lf[1190]+1 /* (set! chicken.process-context#change-directory ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29262,a[2]=((C_word)li897),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[1193]+1 /* (set! ##sys#change-directory-hook ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29292,a[2]=((C_word)li898),tmp=(C_word)a,a+=3,tmp));
t20=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29300,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t21=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31430,a[2]=((C_word)li976),tmp=(C_word)a,a+=3,tmp);
t22=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31452,a[2]=((C_word)li977),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5944: chicken.base#getter-with-setter */
t23=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t23;
av2[1]=t20;
av2[2]=t21;
av2[3]=t22;
av2[4]=lf[1373];
((C_proc)(void*)(*((C_word*)t23+1)))(5,av2);}}

/* ##sys#interrupt-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28665(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_28665,c,av);}
a=C_alloc(10);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28669,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28696,a[2]=t6,a[3]=((C_word)li883),tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_28696(t8,t4,t2);}

/* k28667 in ##sys#interrupt-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28669,c,av);}
t2=C_slot(C_fast_retrieve(lf[961]),C_fix(0));
if(C_truep(C_fixnum_greaterp(t2,C_fix(0)))){
/* library.scm:5736: ##sys#run-pending-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[962]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[962]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
if(C_truep(C_i_closurep(((C_word*)t0)[3]))){
/* library.scm:5737: state */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:5738: ##sys#context-switch */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
C_context_switch(3,av2);}}}}

/* loop in ##sys#interrupt-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_fcall f_28696(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_28696,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(t2)){
t3=C_slot(C_fast_retrieve(lf[1176]),t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28706,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
/* library.scm:5733: handler */
t5=t3;{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}
else{
/* library.scm:5734: loop */
t6=t1;
t7=C_i_pending_interrupt(C_SCHEME_FALSE);
t1=t6;
t2=t7;
goto loop;}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k28704 in loop in ##sys#interrupt-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_28706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28706,c,av);}
/* library.scm:5734: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_28696(t2,((C_word*)t0)[3],C_i_pending_interrupt(C_SCHEME_FALSE));}

/* ##sys#dispatch-interrupt in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28715(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_28715,c,av);}
/* library.scm:5741: ##sys#interrupt-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1177]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[1177]+1);
av2[1]=t1;
av2[2]=C_i_pending_interrupt(C_SCHEME_FALSE);
av2[3]=t2;
tp(4,av2);}}

/* ##sys#update-errno in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28722,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_fix((C_word)errno));
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#errno in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28726,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_28740(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_28740,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28744,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5770: chicken.base#open-output-string */
t4=*((C_word*)lf[215]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k28742 in readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_28744,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28751,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5771: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k28749 in k28742 in readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_28751(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_28751,c,av);}
a=C_alloc(8);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28753,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word)li888),tmp=(C_word)a,a+=6,tmp));
t5=((C_word*)t3)[1];
f_28753(t5,((C_word*)t0)[4],t1);}

/* doloop6636 in k28749 in k28742 in readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_fcall f_28753(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_28753,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eofp(t2);
t4=(C_truep(t3)?t3:C_i_char_equalp(C_make_character(10),t2));
if(C_truep(t4)){
if(C_truep(C_eofp(t2))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* library.scm:5773: chicken.base#get-output-string */
t5=*((C_word*)lf[213]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28775,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5774: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word av2[4];
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=((C_word*)t0)[2];
tp(4,av2);}}}

/* k28773 in doloop6636 in k28749 in k28742 in readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_28775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28775,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28782,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5771: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
tp(3,av2);}}

/* k28780 in k28773 in doloop6636 in k28749 in k28742 in readln in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_28782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28782,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_28753(t2,((C_word*)t0)[3],t1);}

/* read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_fcall f_28787(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_28787,3,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28791,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t3)){
/* library.scm:5776: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t4;
av2[2]=t2;
tp(3,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_28791(2,av2);}}}

/* k28789 in read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_28791,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28794,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5777: read */
t3=*((C_word*)lf[149]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k28792 in k28789 in read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_28794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_28794,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28797,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[3])){
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28802,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word)li890),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_28802(t6,t2);}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k28795 in k28792 in k28789 in read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_28797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28797,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k28792 in k28789 in read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_fcall f_28802(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_28802,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28806,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5781: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k28804 in loop in k28792 in k28789 in read-escaped-sexp in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_28806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_28806,c,av);}
if(C_truep(C_eofp(t1))){
/* library.scm:5783: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1179];
tp(4,av2);}}
else{
t2=C_i_char_equalp(C_make_character(125),t1);
if(C_truep(C_i_not(t2))){
/* library.scm:5784: loop */
t3=((C_word*)((C_word*)t0)[4])[1];
f_28802(t3,((C_word*)t0)[2]);}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_28832(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_28832,c,av);}
a=C_alloc(6);
t4=C_i_char_equalp(C_make_character(60),t2);
if(C_truep(C_i_not(t4))){
/* library.scm:5787: old */
t5=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28845,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* ##sys#read-char/port */
t6=C_fast_retrieve(lf[766]);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_28845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28845,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28848,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5790: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}}

/* k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_28848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_28848,c,av);}
a=C_alloc(6);
switch(t1){
case C_make_character(60):
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28857,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5792: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}
case C_make_character(35):
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28970,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5817: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
tp(3,av2);}
default:
/* library.scm:5872: ##sys#read-error */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[774]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[774]+1);
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[2];
av2[3]=lf[1188];
tp(4,av2);}}}

/* k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_28857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_28857,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28860,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5793: chicken.base#open-output-string */
t3=*((C_word*)lf[215]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_28860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28860,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28863,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5794: readln */
f_28740(t2,((C_word*)t0)[2]);}

/* k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_28863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_28863,c,av);}
a=C_alloc(10);
t2=C_SCHEME_FALSE;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_eofp(t1);
t5=(C_truep(t4)?C_fix(0):C_i_string_length(t1));
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28869,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t3,a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t7=C_eqp(t5,C_fix(0));
if(C_truep(t7)){
/* library.scm:5799: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1182];
tp(4,av2);}}
else{
t8=C_fixnum_difference(t5,C_fix(1));
t9=C_i_string_ref(t1,t8);
t10=C_u_i_char_equalp(t9,C_make_character(32));
if(C_truep(t10)){
if(C_truep(t10)){
/* library.scm:5803: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1183];
tp(4,av2);}}
else{
t11=C_SCHEME_UNDEFINED;
t12=t6;{
C_word *av2=av;
av2[0]=t12;
av2[1]=t11;
f_28869(2,av2);}}}
else{
t11=C_fixnum_difference(t5,C_fix(1));
t12=C_i_string_ref(t1,t11);
if(C_truep(C_u_i_char_equalp(t12,C_make_character(9)))){
/* library.scm:5803: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1183];
tp(4,av2);}}
else{
t13=C_SCHEME_UNDEFINED;
t14=t6;{
C_word *av2=av;
av2[0]=t14;
av2[1]=t13;
f_28869(2,av2);}}}}}

/* k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_28869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_28869,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28876,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* library.scm:5806: readln */
f_28740(t2,((C_word*)t0)[4]);}

/* k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_28876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_28876,c,av);}
a=C_alloc(11);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_28878,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word)li892),tmp=(C_word)a,a+=9,tmp));
t5=((C_word*)t3)[1];
f_28878(t5,((C_word*)t0)[7],t1);}

/* doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_fcall f_28878(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,0,3)))){
C_save_and_reclaim_args((void *)trf_28878,3,t0,t1,t2);}
a=C_alloc(17);
t3=C_eofp(t2);
t4=(C_truep(t3)?t3:C_i_string_equal_p(((C_word*)t0)[2],t2));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28891,a[2]=t1,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_eofp(t2))){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28904,a[2]=t5,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t7=C_a_i_list(&a,3,lf[1180],((C_word*)t0)[2],lf[1181]);
/* library.scm:5761: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word av2[3];
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t6;
av2[2]=t7;
tp(3,av2);}}
else{
/* library.scm:5811: chicken.base#get-output-string */
t6=*((C_word*)lf[213]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28907,a[2]=((C_word*)t0)[5],a[3]=t1,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[4],a[6]=t2,a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)((C_word*)t0)[7])[1])){
/* library.scm:5813: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word av2[4];
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t5;
av2[2]=C_make_character(10);
av2[3]=((C_word*)t0)[3];
tp(4,av2);}}
else{
t6=C_set_block_item(((C_word*)t0)[7],0,C_SCHEME_TRUE);
t7=t5;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
f_28907(2,av2);}}}}

/* k28889 in doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_28891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28891,c,av);}
/* library.scm:5811: chicken.base#get-output-string */
t2=*((C_word*)lf[213]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k28902 in doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_28904(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_28904,c,av);}
/* library.scm:5809: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k28905 in doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_28907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_28907,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28910,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5815: display */
t3=*((C_word*)lf[153]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k28908 in k28905 in doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_28910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_28910,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28917,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5806: readln */
f_28740(t2,((C_word*)t0)[5]);}

/* k28915 in k28908 in k28905 in doloop6671 in k28874 in k28867 in k28861 in k28858 in k28855 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_28917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_28917,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_28878(t2,((C_word*)t0)[3],t1);}

/* k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_28970(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_28970,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28973,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5818: readln */
f_28740(t2,((C_word*)t0)[2]);}

/* k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_28973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_28973,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_28976,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5819: chicken.base#open-output-string */
t3=*((C_word*)lf[215]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_28976(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_28976,c,av);}
a=C_alloc(14);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28978,a[2]=t3,a[3]=((C_word)li893),tmp=(C_word)a,a+=4,tmp);
t5=C_eofp(((C_word*)t0)[2]);
t6=(C_truep(t5)?C_fix(0):C_i_string_length(((C_word*)t0)[2]));
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_28992,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t4,a[5]=t3,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
t8=C_eqp(t6,C_fix(0));
if(C_truep(t8)){
/* library.scm:5828: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1186];
tp(4,av2);}}
else{
t9=C_fixnum_difference(t6,C_fix(1));
t10=C_i_string_ref(((C_word*)t0)[2],t9);
t11=C_u_i_char_equalp(t10,C_make_character(32));
if(C_truep(t11)){
if(C_truep(t11)){
/* library.scm:5832: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1187];
tp(4,av2);}}
else{
t12=C_SCHEME_UNDEFINED;
t13=t7;{
C_word *av2=av;
av2[0]=t13;
av2[1]=t12;
f_28992(2,av2);}}}
else{
t12=C_fixnum_difference(t6,C_fix(1));
t13=C_i_string_ref(((C_word*)t0)[2],t12);
if(C_truep(C_u_i_char_equalp(t13,C_make_character(9)))){
/* library.scm:5832: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=t7;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1187];
tp(4,av2);}}
else{
t14=C_SCHEME_UNDEFINED;
t15=t7;{
C_word *av2=av;
av2[0]=t15;
av2[1]=t14;
f_28992(2,av2);}}}}}

/* get/clear-str in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_fcall f_28978(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_28978,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_28982,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5821: chicken.base#get-output-string */
t3=*((C_word*)lf[213]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k28980 in get/clear-str in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_28982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_28982,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_28986,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5822: chicken.base#open-output-string */
t3=*((C_word*)lf[215]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k28984 in k28980 in get/clear-str in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_28986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_28986,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_28992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_28992,c,av);}
a=C_alloc(11);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_28997,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word)li895),tmp=(C_word)a,a+=9,tmp));
t5=((C_word*)t3)[1];
f_28997(t5,((C_word*)t0)[7],C_SCHEME_END_OF_LIST);}

/* loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_fcall f_28997(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_28997,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_29001,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* library.scm:5837: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word av2[3];
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}

/* k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_29001(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_29001,c,av);}
a=C_alloc(9);
t2=C_eqp(t1,C_make_character(10));
t3=(C_truep(t2)?t2:C_eqp(t1,C_SCHEME_END_OF_FILE));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_29013,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* library.scm:5840: get/clear-str */
t5=((C_word*)t0)[7];
f_28978(t5,t4);}
else{
t4=C_eqp(t1,C_make_character(35));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_29133,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
/* library.scm:5860: ##sys#peek-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[767]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[767]+1);
av2[1]=t5;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29200,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5870: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t5;
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[8])[1];
tp(4,av2);}}}}

/* k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29013(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_29013,c,av);}
a=C_alloc(17);
t2=C_eofp(((C_word*)t0)[2]);
t3=(C_truep(t2)?t2:C_i_string_equal_p(((C_word*)t0)[3],t1));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29025,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_eofp(((C_word*)t0)[2]))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29110,a[2]=t4,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t6=C_a_i_list(&a,3,lf[1180],((C_word*)t0)[3],lf[1181]);
/* library.scm:5761: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t5;
av2[2]=t6;
tp(3,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_29025(2,av2);}}}
else{
t4=C_a_i_cons(&a,2,t1,((C_word*)t0)[5]);
t5=C_a_i_cons(&a,2,C_make_character(10),t4);
/* library.scm:5858: loop */
t6=((C_word*)((C_word*)t0)[7])[1];
f_28997(t6,((C_word*)t0)[4],t5);}}

/* k29023 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29025(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_29025,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29032,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_i_cdr(((C_word*)t0)[3]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29038,a[2]=t5,a[3]=((C_word)li894),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_29038(t7,t2,t3,C_SCHEME_END_OF_LIST,lf[1185]);}

/* k29030 in k29023 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_29032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_29032,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[936],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop2 in k29023 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_fcall f_29038(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_29038,5,t0,t1,t2,t3,t4);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29052,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5850: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word av2[3];
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t5;
av2[2]=t3;
tp(3,av2);}}
else{
t5=C_i_car(t2);
t6=C_i_stringp(t5);
t7=(C_truep(t6)?t6:C_charp(C_u_i_car(t2)));
if(C_truep(t7)){
t8=C_u_i_cdr(t2);
t9=C_u_i_car(t2);
t10=C_a_i_cons(&a,2,t9,t3);
/* library.scm:5852: loop2 */
t12=t1;
t13=t8;
t14=t10;
t15=t4;
t1=t12;
t2=t13;
t3=t14;
t4=t15;
goto loop;}
else{
t8=C_u_i_cdr(t2);
t9=C_u_i_car(t2);
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_29091,a[2]=t4,a[3]=t9,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=t8,tmp=(C_word)a,a+=7,tmp);
/* library.scm:5857: ##sys#print-to-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[936]);
C_word av2[3];
av2[0]=*((C_word*)lf[936]+1);
av2[1]=t10;
av2[2]=t3;
tp(3,av2);}}}}

/* k29050 in loop2 in k29023 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_29052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_29052,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[1184],t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29089 in loop2 in k29023 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_29091(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_29091,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,3,lf[1184],t1,((C_word*)t0)[2]);
t3=C_a_i_list(&a,3,lf[1184],((C_word*)t0)[3],t2);
/* library.scm:5854: loop2 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_29038(t4,((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_END_OF_LIST,t3);}

/* k29108 in k29011 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29110(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29110,c,av);}
/* library.scm:5843: ##sys#read-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[777]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[777]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29133(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_29133,c,av);}
a=C_alloc(9);
switch(t1){
case C_make_character(35):
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29142,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29149,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5863: ##sys#read-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[765]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[765]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
tp(3,av2);}
case C_make_character(123):
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29166,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5865: read-escaped-sexp */
f_28787(t2,((C_word*)t0)[6],C_SCHEME_TRUE);
default:
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29185,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5867: read-escaped-sexp */
f_28787(t2,((C_word*)t0)[6],C_SCHEME_FALSE);}}

/* k29140 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29142(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29142,c,av);}
/* library.scm:5864: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_28997(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k29147 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29149,c,av);}
/* library.scm:5863: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[3])[1];
tp(4,av2);}}

/* k29164 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29166,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29174,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5866: get/clear-str */
t3=((C_word*)t0)[5];
f_28978(t3,t2);}

/* k29172 in k29164 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_29174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29174,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
/* library.scm:5865: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_28997(t4,((C_word*)t0)[5],t3);}

/* k29183 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29185(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29185,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29193,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5868: get/clear-str */
t3=((C_word*)t0)[5];
f_28978(t3,t2);}

/* k29191 in k29183 in k29131 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_29193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29193,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
/* library.scm:5867: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_28997(t4,((C_word*)t0)[5],t3);}

/* k29198 in k28999 in loop in k28990 in k28974 in k28971 in k28968 in k28846 in k28843 in ##sys#user-read-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29200,c,av);}
/* library.scm:5871: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_28997(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_29262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_29262,c,av);}
a=C_alloc(4);
t3=C_i_check_string_2(t2,lf[1191]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29269,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5933: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[1191];
tp(4,av2);}}

/* k29267 in chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_29269,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29272,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_eqp(C_chdir(t1),C_fix(0));
if(C_truep(t3)){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29278,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5935: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t4;
tp(2,av2);}}}

/* k29270 in k29267 in chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29272(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29272,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29276 in k29267 in chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_29278,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29285,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29289,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t4=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k29283 in k29276 in k29267 in chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29285(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_29285,c,av);}
/* library.scm:5936: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=lf[1191];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k29287 in k29276 in k29267 in chicken.process-context#change-directory in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29289,c,av);}
/* library.scm:5937: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1192];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* ##sys#change-directory-hook in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_29292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29292,c,av);}
/* library.scm:5941: change-directory */
t3=C_fast_retrieve(lf[1190]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_29300(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,4)))){
C_save_and_reclaim((void *)f_29300,c,av);}
a=C_alloc(30);
t2=C_mutate((C_word*)lf[1194]+1 /* (set! chicken.process-context#current-directory ...) */,t1);
t3=C_mutate((C_word*)lf[1195]+1 /* (set! chicken.process-context#get-environment-variable ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29302,a[2]=((C_word)li899),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[1196]+1 /* (set! chicken.process-context#set-environment-variable! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29317,a[2]=((C_word)li900),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[1198]+1 /* (set! chicken.process-context#unset-environment-variable! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29334,a[2]=((C_word)li901),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1200]+1 /* (set! chicken.process-context#get-environment-variables ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29354,a[2]=((C_word)li904),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[1201]+1 /* (set! chicken.process-context#executable-pathname ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29420,a[2]=((C_word)li905),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[1202]+1 /* (set! chicken.process-context#argc+argv ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29427,a[2]=((C_word)li906),tmp=(C_word)a,a+=3,tmp));
t9=C_SCHEME_FALSE;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_mutate((C_word*)lf[1203]+1 /* (set! chicken.process-context#argv ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29439,a[2]=t10,a[3]=((C_word)li908),tmp=(C_word)a,a+=4,tmp));
t12=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29479,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31428,a[2]=t12,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6018: argv */
t14=C_fast_retrieve(lf[1203]);{
C_word *av2=av;
av2[0]=t14;
av2[1]=t13;
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}

/* chicken.process-context#get-environment-variable in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_29302,c,av);}
a=C_alloc(9);
t3=C_a_i_bytevector(&a,1,C_fix(3));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29311,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5963: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2=av;
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t4;
av2[2]=C_i_foreign_string_argumentp(t2);
tp(3,av2);}}

/* k29309 in chicken.process-context#get-environment-variable in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29311,c,av);}
/* library.scm:5963: ##sys#peek-c-string */
t2=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=stub6790(((C_word*)t0)[3],t1);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.process-context#set-environment-variable! in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_29317,c,av);}
a=C_alloc(4);
t4=C_i_check_string_2(t2,lf[1197]);
t5=C_i_check_string_2(t3,lf[1197]);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29328,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5969: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2=av;
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t6;
av2[2]=t2;
av2[3]=lf[1197];
tp(4,av2);}}

/* k29326 in chicken.process-context#set-environment-variable! in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_29328,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29332,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5970: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[1197];
tp(4,av2);}}

/* k29330 in k29326 in chicken.process-context#set-environment-variable! in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29332,c,av);}
t2=C_setenv(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.process-context#unset-environment-variable! in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29334(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_29334,c,av);}
a=C_alloc(3);
t3=C_i_check_string_2(t2,lf[1199]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29342,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5976: ##sys#make-c-string */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[745]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[745]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=lf[1199];
tp(4,av2);}}

/* k29340 in chicken.process-context#unset-environment-variable! in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29342(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29342,c,av);}
t2=C_unsetenv(t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29354,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29360,a[2]=t3,a[3]=((C_word)li903),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_29360(t5,t1,C_fix(0));}

/* loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_fcall f_29360(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_29360,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29364,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t4=C_a_i_bytevector(&a,1,C_fix(3));
t5=C_i_foreign_fixnum_argumentp(t2);
/* library.scm:5980: ##sys#peek-c-string */
t6=*((C_word*)lf[702]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=stub6807(t4,t5);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k29362 in loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29364(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_29364,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_29372,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t3,a[6]=((C_word)li902),tmp=(C_word)a,a+=7,tmp));
t5=((C_word*)t3)[1];
f_29372(t5,((C_word*)t0)[4],C_fix(0));}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* scan in k29362 in loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_fcall f_29372(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_29372,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_char_equalp(C_make_character(61),C_subchar(((C_word*)t0)[2],t2)))){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_29398,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* library.scm:5987: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=C_fix(0);
av2[4]=t2;
tp(5,av2);}}
else{
/* library.scm:5990: scan */
t5=t1;
t6=C_fixnum_plus(t2,C_fix(1));
t1=t5;
t2=t6;
goto loop;}}

/* k29388 in k29400 in k29396 in scan in k29362 in loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_29390,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29396 in scan in k29362 in loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29398(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_29398,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29402,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:5988: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_fixnum_plus(((C_word*)t0)[6],C_fix(1));
av2[4]=C_block_size(((C_word*)t0)[5]);
tp(5,av2);}}

/* k29400 in k29396 in scan in k29362 in loop in chicken.process-context#get-environment-variables in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_29402,c,av);}
a=C_alloc(7);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29390,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5989: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_29360(t4,t3,C_fixnum_plus(((C_word*)t0)[5],C_fix(1)));}

/* chicken.process-context#executable-pathname in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_29420,c,av);}
a=C_alloc(5);
t2=C_a_i_bytevector(&a,1,C_fix(3));
/* library.scm:6000: ##sys#peek-and-free-c-string */
t3=*((C_word*)lf[1141]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=stub6819(t2);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.process-context#argc+argv in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29427(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_29427,c,av);}
a=C_alloc(3);
/* library.scm:6003: ##sys#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fix((C_word)C_main_argc);
av2[3]=C_mpointer_or_false(&a,(void*)C_main_argv);
C_values(4,av2);}}

/* chicken.process-context#argv in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_29439,c,av);}
a=C_alloc(10);
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29447,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_fixnum_difference(C_fix((C_word)C_main_argc),C_fix(1));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29453,a[2]=t5,a[3]=((C_word)li907),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_29453(t7,t2,t3,C_SCHEME_END_OF_LIST);}}

/* k29445 in chicken.process-context#argv in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29447,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* doloop6831 in chicken.process-context#argv in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_fcall f_29453(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_29453,4,t0,t1,t2,t3);}
a=C_alloc(11);
if(C_truep(C_fixnum_lessp(t2,C_fix(0)))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_fixnum_difference(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29475,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_a_i_bytevector(&a,1,C_fix(3));
/* library.scm:6007: ##sys#peek-c-string */
t7=*((C_word*)lf[702]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t5;
av2[2]=stub6827(t6,t2);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}

/* k29473 in doloop6831 in chicken.process-context#argv in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_29475,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
t3=((C_word*)((C_word*)t0)[3])[1];
f_29453(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_29479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29479,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[1204]+1 /* (set! chicken.process-context#program-name ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29483,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31340,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6027: argv */
t5=C_fast_retrieve(lf[1203]);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_29483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_29483,c,av);}
a=C_alloc(20);
t2=C_mutate((C_word*)lf[1205]+1 /* (set! chicken.process-context#command-line-arguments ...) */,t1);
t3=C_a_i_provide(&a,1,lf[1206]);
t4=C_mutate((C_word*)lf[1207]+1 /* (set! chicken.gc#current-gc-milliseconds ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29485,a[2]=((C_word)li909),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[1208]+1 /* (set! chicken.gc#set-gc-report! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29488,a[2]=((C_word)li910),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1209]+1 /* (set! chicken.gc#memory-statistics ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29491,a[2]=((C_word)li911),tmp=(C_word)a,a+=3,tmp));
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29520,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=C_fixnum_times(C_fix(2),C_fix((C_word)C_max_pending_finalizers));
/* library.scm:6075: ##sys#make-vector */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[672]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[672]+1);
av2[1]=t7;
av2[2]=C_fixnum_plus(t8,C_fix(1));
av2[3]=C_SCHEME_UNDEFINED;
tp(4,av2);}}

/* chicken.gc#current-gc-milliseconds in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29485,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_accumulated_gc_time();
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.gc#set-gc-report! in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29488,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_set_gc_report(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.gc#memory-statistics in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_29491,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29495,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6065: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t2;
av2[2]=C_SCHEME_TRUE;
tp(3,av2);}}

/* k29493 in chicken.gc#memory-statistics in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_29495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29495,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29498,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:6066: ##sys#memory-info */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[322]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[322]+1);
av2[1]=t2;
tp(2,av2);}}

/* k29496 in k29493 in chicken.gc#memory-statistics in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,1)))){
C_save_and_reclaim((void *)f_29498,c,av);}
a=C_alloc(4);
t2=C_slot(t1,C_fix(0));
t3=C_fixnum_divide(t2,C_fix(2));
t4=C_fixnum_difference(t3,((C_word*)t0)[2]);
t5=C_slot(t1,C_fix(1));
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_vector3(&a,3,t3,t4,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_29520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_29520,c,av);}
a=C_alloc(15);
t2=C_mutate((C_word*)lf[961]+1 /* (set! ##sys#pending-finalizers ...) */,t1);
t3=C_i_set_i_slot(C_fast_retrieve(lf[961]),C_fix(0),C_fix(0));
t4=C_mutate((C_word*)lf[1210]+1 /* (set! ##sys#set-finalizer! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_register_finalizer,a[2]=((C_word)li912),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[1211]+1 /* (set! chicken.gc#set-finalizer! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29526,a[2]=((C_word)li913),tmp=(C_word)a,a+=3,tmp));
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_mutate((C_word*)lf[962]+1 /* (set! ##sys#run-pending-finalizers ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29591,a[2]=t7,a[3]=((C_word)li922),tmp=(C_word)a,a+=4,tmp));
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29758,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6153: chicken.base#make-parameter */
t10=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t10;
av2[1]=t9;
av2[2]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_29526(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_29526,c,av);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29530,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fixnum_greater_or_equal_p(C_i_live_finalizer_count(),C_fix((C_word)C_max_pending_finalizers)))){
t5=C_fixnum_times(C_fix(2),C_fix((C_word)C_max_pending_finalizers));
if(C_truep(C_resize_pending_finalizers(t5))){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29547,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t7=C_fixnum_times(C_fix(2),C_fix((C_word)C_max_pending_finalizers));
/* library.scm:6087: ##sys#vector-resize */
f_18160(t6,C_fast_retrieve(lf[961]),C_fixnum_plus(t7,C_fix(1)),C_SCHEME_UNDEFINED);}
else{
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29577,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_debug_modep())){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29587,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6103: string-append */
t8=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[1215];
av2[3]=C_i_live_finalizer_count();
av2[4]=lf[1216];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
/* library.scm:6108: ##sys#force-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[958]);
C_word *av2=av;
av2[0]=*((C_word*)lf[958]+1);
av2[1]=t4;
tp(2,av2);}}}}
else{
/* library.scm:6109: ##sys#set-finalizer! */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1210]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1210]+1);
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
tp(4,av2);}}}

/* k29528 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_29530,c,av);}
/* library.scm:6109: ##sys#set-finalizer! */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1210]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[1210]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
tp(4,av2);}}

/* k29545 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_29547,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[961]+1 /* (set! ##sys#pending-finalizers ...) */,t1);
if(C_truep(C_i_debug_modep())){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29557,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29561,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6094: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=C_i_live_finalizer_count();
tp(3,av2);}}
else{
/* library.scm:6109: ##sys#set-finalizer! */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1210]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[1210]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
tp(4,av2);}}}

/* k29555 in k29545 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_29557,c,av);}
/* library.scm:6091: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k29559 in k29545 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29561,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29565,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6097: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=C_fix((C_word)C_max_pending_finalizers);
tp(3,av2);}}

/* k29563 in k29559 in k29545 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_29565,c,av);}
/* library.scm:6092: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1212];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1213];
av2[5]=t1;
av2[6]=lf[1214];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k29575 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29577,c,av);}
/* library.scm:6108: ##sys#force-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[958]);
C_word *av2=av;
av2[0]=*((C_word*)lf[958]+1);
av2[1]=((C_word*)t0)[2];
tp(2,av2);}}

/* k29585 in chicken.gc#set-finalizer! in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_29587,c,av);}
/* library.scm:6102: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_29591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_29591,c,av);}
a=C_alloc(15);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29595,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[2])[1]))){
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_fast_retrieve(lf[943]));
t5=C_slot(C_fast_retrieve(lf[961]),C_fix(0));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29626,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_debug_modep())){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29725,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29729,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6122: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t8;
av2[2]=t5;
tp(3,av2);}}
else{
t7=t6;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_29626(2,av2);}}}
else{
if(C_truep(t2)){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
f_29595(2,av2);}}
else{
t4=C_eqp(((C_word*)((C_word*)t0)[2])[1],C_fast_retrieve(lf[943]));
if(C_truep(t4)){
/* library.scm:6143: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[195];
av2[3]=lf[962];
av2[4]=lf[1222];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
/* library.scm:6148: ##sys#thread-yield! */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1171]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1171]+1);
av2[1]=t3;
tp(2,av2);}}}}}

/* k29593 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29595,c,av);}
t2=C_i_not(((C_word*)t0)[2]);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_closurep(((C_word*)t0)[2]))){
/* library.scm:6150: state */
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(((C_word*)t0)[2])){
/* library.scm:6151: ##sys#context-switch */{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[2];
C_context_switch(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}}

/* k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29626(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_29626,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29629,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29638,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=((C_word)li921),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_29638(t6,t2,C_fix(0));}

/* k29627 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29629(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_29629,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29632,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:6138: vector-fill! */
t3=*((C_word*)lf[97]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_fast_retrieve(lf[961]);
av2[3]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k29630 in k29627 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29632(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29632,c,av);}
t2=C_i_set_i_slot(C_fast_retrieve(lf[961]),C_fix(0),C_fix(0));
t3=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_29595(2,av2);}}

/* doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_fcall f_29638(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_29638,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_fixnum_greater_or_equal_p(t2,((C_word*)t0)[2]))){
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_fixnum_times(t2,C_fix(2));
t4=C_fixnum_plus(C_fix(1),t3);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29651,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29663,a[2]=t4,a[3]=((C_word)li920),tmp=(C_word)a,a+=4,tmp);
/* library.scm:6134: scheme#call-with-current-continuation */
t7=*((C_word*)lf[132]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k29649 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_29651,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29654,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:6133: g6919 */
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)C_fast_retrieve_proc(t3))(2,av2);}}

/* k29652 in k29649 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_29654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29654,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_29638(t2,((C_word*)t0)[3],C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_29663,c,av);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29669,a[2]=t2,a[3]=((C_word)li915),tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29681,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word)li919),tmp=(C_word)a,a+=5,tmp);
/* library.scm:6134: chicken.condition#with-exception-handler */
t5=C_fast_retrieve(lf[916]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a29668 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_29669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29669,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29675,a[2]=t2,a[3]=((C_word)li914),tmp=(C_word)a,a+=4,tmp);
/* library.scm:6134: k6916 */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}

/* a29674 in a29668 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_29675,c,av);}
/* library.scm:6135: ##sys#show-exception-warning */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1055]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[1055]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[1217];
av2[4]=C_SCHEME_FALSE;
tp(5,av2);}}

/* a29680 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_29681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_29681,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29687,a[2]=((C_word*)t0)[2],a[3]=((C_word)li916),tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29704,a[2]=((C_word*)t0)[3],a[3]=((C_word)li918),tmp=(C_word)a,a+=4,tmp);
/* library.scm:6134: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a29686 in a29680 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29687,c,av);}
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
/* library.scm:6134: g6923 */
t3=C_slot(C_fast_retrieve(lf[961]),t2);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=C_slot(C_fast_retrieve(lf[961]),((C_word*)t0)[2]);
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}

/* a29703 in a29680 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_29704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_29704,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29710,a[2]=t2,a[3]=((C_word)li917),tmp=(C_word)a,a+=4,tmp);
/* library.scm:6134: k6916 */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)C_fast_retrieve_proc(t4))(3,av2);}}

/* a29709 in a29703 in a29680 in a29662 in doloop6912 in k29624 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_29710(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_29710,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k29723 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_29725,c,av);}
/* library.scm:6121: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k29727 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29729(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29729,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29733,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6124: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=C_i_live_finalizer_count();
tp(3,av2);}}

/* k29731 in k29727 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_29733,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_29737,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6127: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=C_i_allocated_finalizer_count();
tp(3,av2);}}

/* k29735 in k29731 in k29727 in ##sys#run-pending-finalizers in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29737(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_29737,c,av);}
/* library.scm:6122: string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1218];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1219];
av2[5]=((C_word*)t0)[4];
av2[6]=lf[1220];
av2[7]=t1;
av2[8]=lf[1221];
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}

/* k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_29758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,6)))){
C_save_and_reclaim((void *)f_29758,c,av);}
a=C_alloc(26);
t2=C_mutate((C_word*)lf[959]+1 /* (set! chicken.gc#force-finalizers ...) */,t1);
t3=C_mutate((C_word*)lf[958]+1 /* (set! ##sys#force-finalizers ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29760,a[2]=((C_word)li924),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[1223]+1 /* (set! chicken.gc#gc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29788,a[2]=((C_word)li925),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[815]+1 /* (set! ##sys#list->vector ...) */,*((C_word*)lf[94]+1));
t6=C_mutate((C_word*)lf[371]+1 /* (set! ##sys#list ...) */,*((C_word*)lf[42]+1));
t7=C_mutate((C_word*)lf[1224]+1 /* (set! ##sys#length ...) */,*((C_word*)lf[43]+1));
t8=C_mutate((C_word*)lf[1225]+1 /* (set! ##sys#cons ...) */,*((C_word*)lf[8]+1));
t9=C_mutate((C_word*)lf[256]+1 /* (set! ##sys#append ...) */,*((C_word*)lf[377]+1));
t10=C_mutate((C_word*)lf[1226]+1 /* (set! ##sys#vector ...) */,*((C_word*)lf[96]+1));
t11=C_mutate((C_word*)lf[1227]+1 /* (set! ##sys#apply ...) */,*((C_word*)lf[129]+1));
t12=C_mutate((C_word*)lf[1228]+1 /* (set! ##sys#values ...) */,*((C_word*)lf[130]+1));
t13=C_mutate((C_word*)lf[1229]+1 /* (set! ##sys#equal? ...) */,*((C_word*)lf[6]+1));
t14=C_mutate((C_word*)lf[1230]+1 /* (set! ##sys#car ...) */,*((C_word*)lf[9]+1));
t15=C_mutate((C_word*)lf[1231]+1 /* (set! ##sys#cdr ...) */,*((C_word*)lf[10]+1));
t16=C_mutate((C_word*)lf[1232]+1 /* (set! ##sys#pair? ...) */,*((C_word*)lf[7]+1));
t17=C_mutate((C_word*)lf[1233]+1 /* (set! ##sys#vector? ...) */,*((C_word*)lf[89]+1));
t18=C_mutate((C_word*)lf[1234]+1 /* (set! ##sys#vector->list ...) */,*((C_word*)lf[95]+1));
t19=C_mutate((C_word*)lf[1235]+1 /* (set! ##sys#vector-length ...) */,*((C_word*)lf[90]+1));
t20=C_mutate((C_word*)lf[1236]+1 /* (set! ##sys#vector-ref ...) */,*((C_word*)lf[91]+1));
t21=C_mutate((C_word*)lf[1237]+1 /* (set! ##sys#>= ...) */,*((C_word*)lf[75]+1));
t22=C_mutate((C_word*)lf[1238]+1 /* (set! ##sys#= ...) */,*((C_word*)lf[72]+1));
t23=C_mutate((C_word*)lf[1239]+1 /* (set! ##sys#+ ...) */,*((C_word*)lf[64]+1));
t24=C_mutate((C_word*)lf[1240]+1 /* (set! ##sys#eq? ...) */,*((C_word*)lf[4]+1));
t25=C_mutate((C_word*)lf[1241]+1 /* (set! ##sys#eqv? ...) */,*((C_word*)lf[5]+1));
t26=C_mutate((C_word*)lf[1242]+1 /* (set! ##sys#list? ...) */,*((C_word*)lf[55]+1));
t27=C_mutate((C_word*)lf[1243]+1 /* (set! ##sys#null? ...) */,*((C_word*)lf[41]+1));
t28=C_mutate((C_word*)lf[1244]+1 /* (set! ##sys#map-n ...) */,*((C_word*)lf[171]+1));
t29=C_mutate((C_word*)lf[1245]+1 /* (set! ##sys#make-locative ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_29831,a[2]=((C_word)li926),tmp=(C_word)a,a+=3,tmp));
t30=C_a_i_provide(&a,1,lf[1258]);
t31=C_mutate((C_word*)lf[1259]+1 /* (set! chicken.plist#put! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30095,a[2]=((C_word)li927),tmp=(C_word)a,a+=3,tmp));
t32=C_mutate((C_word*)lf[1261]+1 /* (set! chicken.plist#get ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30101,a[2]=((C_word)li928),tmp=(C_word)a,a+=3,tmp));
t33=C_mutate((C_word*)lf[1263]+1 /* (set! ##sys#put! ...) */,C_fast_retrieve(lf[1259]));
t34=C_mutate((C_word*)lf[1264]+1 /* (set! ##sys#get ...) */,C_fast_retrieve(lf[1261]));
t35=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30120,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6287: chicken.base#getter-with-setter */
t36=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t36;
av2[1]=t35;
av2[2]=C_fast_retrieve(lf[1261]);
av2[3]=C_fast_retrieve(lf[1259]);
((C_proc)(void*)(*((C_word*)t36+1)))(4,av2);}}

/* ##sys#force-finalizers in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29760(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_29760,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29766,a[2]=t3,a[3]=((C_word)li923),tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_29766(t5,t1);}

/* loop in ##sys#force-finalizers in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_fcall f_29766(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_29766,2,t0,t1);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29770,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6157: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word av2[2];
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t2;
tp(2,av2);}}

/* k29768 in loop in ##sys#force-finalizers in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_29770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_29770,c,av);}
a=C_alloc(4);
t2=C_slot(C_fast_retrieve(lf[961]),C_fix(0));
if(C_truep(C_fixnum_greaterp(t2,C_fix(0)))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_29779,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:6159: ##sys#run-pending-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[962]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[962]+1);
av2[1]=t3;
av2[2]=C_SCHEME_FALSE;
tp(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k29777 in k29768 in loop in ##sys#force-finalizers in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_29779(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_29779,c,av);}
/* library.scm:6160: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_29766(t2,((C_word*)t0)[3]);}

/* chicken.gc#gc in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,2)))){
C_save_and_reclaim((void*)f_29788,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
if(C_truep(C_i_pairp(t2))){
t3=C_get_rest_arg(c,2,av,2,t0);
if(C_truep(t3)){
/* library.scm:6166: ##sys#force-finalizers */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[958]);
C_word *av2=av;
av2[0]=*((C_word*)lf[958]+1);
av2[1]=t1;
tp(2,av2);}}
else{
/* library.scm:6167: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t1;
av2[2]=t3;
tp(3,av2);}}}
else{
/* library.scm:6167: ##sys#gc */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[315]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[315]+1);
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
tp(3,av2);}}}

/* ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_29831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_29831,c,av);}
a=C_alloc(6);
if(C_truep(C_immp(t2))){
/* library.scm:6200: ##sys#signal-hook */
t6=C_fast_retrieve(lf[194]);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=t5;
av2[4]=lf[1246];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(6,av2);}}
else{
t6=C_i_vectorp(t2);
t7=(C_truep(t6)?t6:C_i_pairp(t2));
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29853,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6202: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t8;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t2);
av2[5]=t5;
tp(6,av2);}}
else{
t8=(C_truep(C_blockp(t2))?C_bytevectorp(t2):C_SCHEME_FALSE);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29866,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6209: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t9;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t2);
av2[5]=t5;
tp(6,av2);}}
else{
if(C_truep(C_structurep(t2))){
t9=C_slot(t2,C_fix(0));
t10=C_eqp(t9,lf[1247]);
if(C_truep(t10)){
t11=C_slot(t2,C_fix(1));
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29891,a[2]=t11,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6215: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t12;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t11);
av2[5]=t5;
tp(6,av2);}}
else{
t11=C_eqp(t9,lf[1248]);
if(C_truep(t11)){
t12=C_slot(t2,C_fix(1));
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29907,a[2]=t1,a[3]=t12,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6219: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t13;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t12);
av2[5]=t5;
tp(6,av2);}}
else{
t12=C_eqp(t9,lf[1249]);
if(C_truep(t12)){
t13=C_slot(t2,C_fix(1));
t14=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29923,a[2]=t13,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6223: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t14;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t13);
av2[5]=t5;
tp(6,av2);}}
else{
t13=C_eqp(t9,lf[1250]);
if(C_truep(t13)){
t14=C_slot(t2,C_fix(1));
t15=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29939,a[2]=t1,a[3]=t14,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6227: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t15;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t14);
av2[5]=t5;
tp(6,av2);}}
else{
t14=C_eqp(t9,lf[1251]);
if(C_truep(t14)){
t15=C_slot(t2,C_fix(1));
t16=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29955,a[2]=t15,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6231: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t16;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t15);
av2[5]=t5;
tp(6,av2);}}
else{
t15=C_eqp(t9,lf[1252]);
if(C_truep(t15)){
t16=C_slot(t2,C_fix(1));
t17=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29971,a[2]=t1,a[3]=t16,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6235: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t17;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t16);
av2[5]=t5;
tp(6,av2);}}
else{
t16=C_eqp(t9,lf[1253]);
if(C_truep(t16)){
t17=C_slot(t2,C_fix(1));
t18=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_29987,a[2]=t17,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6239: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t18;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t17);
av2[5]=t5;
tp(6,av2);}}
else{
t17=C_eqp(t9,lf[1254]);
if(C_truep(t17)){
t18=C_slot(t2,C_fix(1));
t19=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30003,a[2]=t1,a[3]=t18,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6243: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t19;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t18);
av2[5]=t5;
tp(6,av2);}}
else{
t18=C_eqp(t9,lf[1255]);
if(C_truep(t18)){
t19=C_slot(t2,C_fix(1));
t20=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30019,a[2]=t19,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6247: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t20;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t19);
av2[5]=t5;
tp(6,av2);}}
else{
t19=C_eqp(t9,lf[1256]);
if(C_truep(t19)){
t20=C_slot(t2,C_fix(1));
t21=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30035,a[2]=t1,a[3]=t20,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6251: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t21;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t20);
av2[5]=t5;
tp(6,av2);}}
else{
t20=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30042,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t21=C_block_size(t2);
/* library.scm:6255: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t20;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_fixnum_difference(t21,C_fix(1));
av2[5]=t5;
tp(6,av2);}}}}}}}}}}}}
else{
if(C_truep(C_i_stringp(t2))){
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30083,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6258: ##sys#check-range */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[336]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[336]+1);
av2[1]=t9;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_block_size(t2);
av2[5]=t5;
tp(6,av2);}}
else{
/* library.scm:6261: ##sys#signal-hook */
t9=C_fast_retrieve(lf[194]);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=t5;
av2[4]=lf[1257];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(6,av2);}}}}}}}

/* k29851 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29853,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(0),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29864 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29866(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29866,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(2),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k29889 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29891,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(2),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k29905 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29907(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29907,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(3),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29921 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29923,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(4),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k29937 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29939,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(5),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29953 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29955(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29955,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(6),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k29969 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29971,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(7),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k29985 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_29987(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_29987,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(8),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30001 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30003(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_30003,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(9),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30017 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_30019,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(10),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30033 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_30035,c,av);}
a=C_alloc(5);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_make_locative(&a,4,C_fix(11),((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30040 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_30042,c,av);}
a=C_alloc(5);
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_make_locative(&a,4,C_fix(0),((C_word*)t0)[4],t2,((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30081 in ##sys#make-locative in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30083(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_30083,c,av);}
a=C_alloc(5);
t2=C_a_i_make_locative(&a,4,C_fix(1),((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.plist#put! in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_30095(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,1)))){
C_save_and_reclaim((void *)f_30095,c,av);}
a=C_alloc(8);
t5=C_i_check_symbol_2(t2,lf[1260]);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_a_i_putprop(&a,3,t2,t3,t4);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* chicken.plist#get in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_30101(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c<4) C_bad_min_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30101,c,av);}
t4=C_rest_nullp(c,4);
t5=(C_truep(t4)?C_SCHEME_FALSE:C_get_rest_arg(c,4,av,4,t0));
t6=C_i_check_symbol_2(t2,lf[1262]);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_getprop(t2,t3,t5);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_30120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_30120,c,av);}
a=C_alloc(12);
t2=C_mutate((C_word*)lf[1261]+1 /* (set! chicken.plist#get ...) */,t1);
t3=C_mutate((C_word*)lf[1265]+1 /* (set! chicken.plist#remprop! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30122,a[2]=((C_word)li930),tmp=(C_word)a,a+=3,tmp));
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30191,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31296,a[2]=((C_word)li969),tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31305,a[2]=((C_word)li970),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6307: chicken.base#getter-with-setter */
t7=*((C_word*)lf[200]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=t5;
av2[3]=t6;
av2[4]=lf[1366];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}

/* chicken.plist#remprop! in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30122,c,av);}
a=C_alloc(5);
t4=C_i_check_symbol_2(t2,lf[1266]);
t5=C_slot(t2,C_fix(2));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30145,a[2]=t3,a[3]=t2,a[4]=((C_word)li929),tmp=(C_word)a,a+=5,tmp);
t7=(
  f_30145(t6,t5,C_SCHEME_FALSE)
);
t8=C_slot(t2,C_fix(2));
if(C_truep(C_i_nullp(t8))){
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_i_unpersist_symbol(t2);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t9=C_SCHEME_UNDEFINED;
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}

/* loop in chicken.plist#remprop! in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static C_word C_fcall f_30145(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_stack_overflow_check;
loop:{}
t3=C_i_nullp(t1);
if(C_truep(C_i_not(t3))){
t4=C_slot(t1,C_fix(1));
t5=C_slot(t4,C_fix(1));
t6=C_slot(t1,C_fix(0));
t7=C_eqp(t6,((C_word*)t0)[2]);
if(C_truep(t7)){
t8=(C_truep(t2)?C_i_setslot(t2,C_fix(1),t5):C_i_setslot(((C_word*)t0)[3],C_fix(2),t5));
return(C_SCHEME_TRUE);}
else{
t10=t5;
t11=t4;
t1=t10;
t2=t11;
goto loop;}}
else{
return(C_SCHEME_FALSE);}}

/* k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_30191(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_30191,c,av);}
a=C_alloc(29);
t2=C_mutate((C_word*)lf[1267]+1 /* (set! chicken.plist#symbol-plist ...) */,t1);
t3=C_mutate((C_word*)lf[1268]+1 /* (set! chicken.plist#get-properties ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30193,a[2]=((C_word)li932),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[1270]+1 /* (set! ##sys#display-times ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30256,a[2]=((C_word)li936),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[963]+1 /* (set! ##sys#dump-heap-state ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_dump_heap_state,a[2]=((C_word)li937),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1286]+1 /* (set! ##sys#filter-heap-objects ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_filter_heap_objects,a[2]=((C_word)li938),tmp=(C_word)a,a+=3,tmp));
t7=C_a_i_provide(&a,1,lf[1287]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30492,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_software_type,a[2]=((C_word)li968),tmp=(C_word)a,a+=3,tmp);
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31294,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6413: g7185 */
t11=t9;{
C_word *av2=av;
av2[0]=t11;
av2[1]=t10;
((C_proc)C_fast_retrieve_proc(t11))(2,av2);}}

/* chicken.plist#get-properties in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_30193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_30193,c,av);}
a=C_alloc(10);
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_i_check_symbol_2(t2,lf[1269]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30200,a[2]=t4,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_symbolp(((C_word*)t4)[1]))){
t7=C_a_i_list1(&a,1,((C_word*)t4)[1]);
t8=C_set_block_item(t4,0,t7);
t9=t6;
f_30200(t9,t8);}
else{
t7=t6;
f_30200(t7,C_SCHEME_UNDEFINED);}}

/* k30198 in chicken.plist#get-properties in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_fcall f_30200(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_30200,2,t0,t1);}
a=C_alloc(7);
t2=C_i_check_list_2(((C_word*)((C_word*)t0)[2])[1],lf[1269]);
t3=C_slot(((C_word*)t0)[3],C_fix(2));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30212,a[2]=((C_word*)t0)[2],a[3]=t5,a[4]=((C_word)li931),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_30212(t7,((C_word*)t0)[4],t3);}

/* loop in k30198 in chicken.plist#get-properties in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_fcall f_30212(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_30212,3,t0,t1,t2);}
if(C_truep(C_i_nullp(t2))){
/* library.scm:6331: scheme#values */{
C_word av2[5];
av2[0]=0;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
C_values(5,av2);}}
else{
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
t5=C_slot(t4,C_fix(1));
if(C_truep(C_i_memq(t3,((C_word*)((C_word*)t0)[2])[1]))){
/* library.scm:6336: scheme#values */{
C_word av2[5];
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=C_slot(t4,C_fix(0));
av2[4]=t5;
C_values(5,av2);}}
else{
/* library.scm:6337: loop */
t7=t1;
t8=t5;
t1=t7;
t2=t8;
goto loop;}}}

/* ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_30256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_30256,c,av);}
a=C_alloc(23);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30271,a[2]=((C_word)li933),tmp=(C_word)a,a+=3,tmp));
t10=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30285,a[2]=((C_word)li934),tmp=(C_word)a,a+=3,tmp));
t11=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30307,a[2]=t4,a[3]=t6,a[4]=((C_word)li935),tmp=(C_word)a,a+=5,tmp));
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30383,a[2]=t2,a[3]=t1,a[4]=t8,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6359: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word *av2=av;
av2[0]=*((C_word*)lf[746]+1);
av2[1]=t12;
av2[2]=*((C_word*)lf[714]+1);
tp(3,av2);}}

/* pnum in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_fcall f_30271(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_30271,2,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30279,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_zerop(t2))){
/* library.scm:6348: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[1271];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* library.scm:6348: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t3;
av2[2]=t2;
tp(3,av2);}}}

/* k30277 in pnum in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30279,c,av);}
/* library.scm:6348: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* round-to in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_fcall f_30285(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_30285,3,t1,t2,t3);}
a=C_alloc(8);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30293,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30305,a[2]=t2,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6350: scheme#expt */
t6=*((C_word*)lf[533]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_fix(10);
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k30291 in round-to in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_30293,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30297,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6350: scheme#expt */
t3=*((C_word*)lf[533]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1272];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k30295 in k30291 in round-to in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_30297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30297,c,av);}
/* ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[529]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k30303 in round-to in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_30305,c,av);}
a=C_alloc(33);
t2=C_s_a_i_times(&a,2,((C_word*)t0)[2],t1);
/* library.scm:6350: scheme#round */
t3=*((C_word*)lf[553]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_fcall f_30307(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_30307,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(C_i_greaterp(t2,lf[1273]))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30317,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30324,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30328,a[2]=((C_word*)t0)[3],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=lf[1273];
tp(4,av2);}}
else{
if(C_truep(C_i_greaterp(t2,C_fix(1048576)))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30339,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30346,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30350,a[2]=((C_word*)t0)[3],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=C_fix(1048576);
tp(4,av2);}}
else{
if(C_truep(C_i_greaterp(t2,C_fix(1024)))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30361,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30368,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30372,a[2]=((C_word*)t0)[3],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* ##sys#/-2 */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[529]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[529]+1);
av2[1]=t5;
av2[2]=t2;
av2[3]=C_fix(1024);
tp(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30377,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6358: pnum */
f_30271(t3,t2);}}}}

/* k30315 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30317,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1274];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30322 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_30324,c,av);}
/* library.scm:6353: pnum */
f_30271(((C_word*)t0)[3],t1);}

/* k30326 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30328(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30328,c,av);}
/* library.scm:6353: round-to */
f_30285(((C_word*)t0)[3],t1,C_fix(2));}

/* k30337 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30339,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1275];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30344 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_30346,c,av);}
/* library.scm:6355: pnum */
f_30271(((C_word*)t0)[3],t1);}

/* k30348 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30350,c,av);}
/* library.scm:6355: round-to */
f_30285(((C_word*)t0)[3],t1,C_fix(2));}

/* k30359 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30361,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1276];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30366 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30368(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_30368,c,av);}
/* library.scm:6357: pnum */
f_30271(((C_word*)t0)[3],t1);}

/* k30370 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30372,c,av);}
/* library.scm:6357: round-to */
f_30285(((C_word*)t0)[3],t1,C_fix(2));}

/* k30375 in pmem in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30377,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1277];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_30383(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30383,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30386,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:6360: pnum */
f_30271(t2,C_slot(((C_word*)t0)[2],C_fix(0)));}

/* k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30386(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_30386,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30389,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* library.scm:6345: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1285];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_30389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_30389,c,av);}
a=C_alloc(11);
t2=C_slot(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30395,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_greaterp(t2,C_fix(0)))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30476,a[2]=t3,a[3]=((C_word*)t0)[5],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6345: ##sys#print */
t5=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[1284];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_30395(2,av2);}}}

/* k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_30395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_30395,c,av);}
a=C_alloc(12);
t2=C_slot(((C_word*)t0)[2],C_fix(2));
t3=C_slot(((C_word*)t0)[2],C_fix(3));
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30404,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnum_greaterp(t2,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30458,a[2]=t4,a[3]=((C_word*)t0)[5],a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6345: ##sys#print */
t6=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[1282];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_30404(2,av2);}}}

/* k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_30404(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_30404,c,av);}
a=C_alloc(11);
t2=C_slot(((C_word*)t0)[2],C_fix(4));
t3=C_slot(((C_word*)t0)[2],C_fix(5));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30413,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=C_fixnum_greaterp(t2,C_fix(0));
t6=(C_truep(t5)?t5:C_fixnum_greaterp(t3,C_fix(0)));
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30437,a[2]=t4,a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6345: ##sys#print */
t8=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[1280];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}
else{
t7=t4;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_30413(2,av2);}}}

/* k30411 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_30413,c,av);}
a=C_alloc(5);
t2=C_slot(((C_word*)t0)[2],C_fix(6));
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30419,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6345: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1278];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k30417 in k30411 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30419,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30422,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6385: pmem */
t3=((C_word*)((C_word*)t0)[3])[1];
f_30307(t3,t2,((C_word*)t0)[4]);}

/* k30420 in k30417 in k30411 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_30422,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30425,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6386: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=*((C_word*)lf[715]+1);
tp(4,av2);}}

/* k30423 in k30420 in k30417 in k30411 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_30425,c,av);}
/* library.scm:6387: ##sys#flush-output */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[746]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[746]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[715]+1);
tp(3,av2);}}

/* k30435 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_30437,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30440,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:6379: pnum */
f_30271(t2,((C_word*)t0)[5]);}

/* k30438 in k30435 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30440,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30443,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:6346: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(47);
av2[3]=*((C_word*)lf[715]+1);
tp(4,av2);}}

/* k30441 in k30438 in k30435 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30443,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30446,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6381: pnum */
f_30271(t2,((C_word*)t0)[4]);}

/* k30444 in k30441 in k30438 in k30435 in k30402 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30446,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1279];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30456 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_30458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_30458,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30461,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:6371: pnum */
f_30271(t2,((C_word*)t0)[5]);}

/* k30459 in k30456 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_30461,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30464,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:6346: ##sys#write-char-0 */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[863]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[863]+1);
av2[1]=t2;
av2[2]=C_make_character(47);
av2[3]=*((C_word*)lf[715]+1);
tp(4,av2);}}

/* k30462 in k30459 in k30456 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30464(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30464,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30467,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6373: pnum */
f_30271(t2,((C_word*)t0)[4]);}

/* k30465 in k30462 in k30459 in k30456 in k30393 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30467,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1281];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30474 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_30476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30476,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30479,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6365: pnum */
f_30271(t2,((C_word*)t0)[4]);}

/* k30477 in k30474 in k30387 in k30384 in k30381 in ##sys#display-times in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_30479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_30479,c,av);}
/* library.scm:6345: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1283];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_30492(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_30492,c,av);}
a=C_alloc(13);
t2=C_mutate((C_word*)lf[1288]+1 /* (set! chicken.platform#software-type ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30493,a[2]=t1,a[3]=((C_word)li939),tmp=(C_word)a,a+=4,tmp));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30498,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_machine_type,a[2]=((C_word)li967),tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31290,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6417: g7189 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* chicken.platform#software-type in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_30493(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30493,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_30498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_30498,c,av);}
a=C_alloc(13);
t2=C_mutate((C_word*)lf[1289]+1 /* (set! chicken.platform#machine-type ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30499,a[2]=t1,a[3]=((C_word)li940),tmp=(C_word)a,a+=4,tmp));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30504,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_machine_byte_order,a[2]=((C_word)li966),tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31286,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6421: g7193 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* chicken.platform#machine-type in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30499,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_30504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_30504,c,av);}
a=C_alloc(13);
t2=C_mutate((C_word*)lf[1290]+1 /* (set! chicken.platform#machine-byte-order ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30505,a[2]=t1,a[3]=((C_word)li941),tmp=(C_word)a,a+=4,tmp));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30510,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_software_version,a[2]=((C_word)li965),tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31282,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6425: g7197 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* chicken.platform#machine-byte-order in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_30505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30505,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_30510(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_30510,c,av);}
a=C_alloc(13);
t2=C_mutate((C_word*)lf[1291]+1 /* (set! chicken.platform#software-version ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30511,a[2]=t1,a[3]=((C_word)li942),tmp=(C_word)a,a+=4,tmp));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30516,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_build_platform,a[2]=((C_word)li964),tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31278,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6429: g7201 */
t6=t4;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)C_fast_retrieve_proc(t6))(2,av2);}}

/* chicken.platform#software-version in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_30511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30511,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_30516(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_30516,c,av);}
a=C_alloc(10);
t2=C_mutate((C_word*)lf[1292]+1 /* (set! chicken.platform#build-platform ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30517,a[2]=t1,a[3]=((C_word)li943),tmp=(C_word)a,a+=4,tmp));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30522,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31274,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6433: software-type */
t5=C_fast_retrieve(lf[1288]);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.platform#build-platform in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_30517(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30517,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_fcall f_30522(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_30522,2,t0,t1);}
a=C_alloc(21);
t2=C_set_block_item(lf[1293] /* ##sys#windows-platform */,0,t1);
t3=C_mutate((C_word*)lf[1294]+1 /* (set! chicken.platform#chicken-version ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30524,a[2]=((C_word)li945),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[1327]+1 /* (set! chicken.platform#chicken-home ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30661,a[2]=((C_word)li946),tmp=(C_word)a,a+=3,tmp));
t5=(C_truep(C_fast_retrieve(lf[1293]))?C_make_character(59):C_make_character(58));
t6=lf[1330] /* chicken.platform#path-list-separator */ =t5;;
t7=lf[1331];
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_mutate((C_word*)lf[1332]+1 /* (set! ##sys#split-path ...) */,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30683,a[2]=t8,a[3]=((C_word)li948),tmp=(C_word)a,a+=4,tmp));
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30773,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31192,a[2]=t10,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t12=*((C_word*)lf[702]+1);{
C_word av2[4];
av2[0]=t12;
av2[1]=t11;
av2[2]=C_mpointer(&a,(void*)C_private_repository_path());
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}

/* chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30524(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_30524,c,av);}
a=C_alloc(6);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?C_SCHEME_FALSE:C_get_rest_arg(c,2,av,2,t0));
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30585,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30624,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_fix((C_word)C_WORD_SIZE);
/* ##sys#fixnum->string */
t7=*((C_word*)lf[623]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
av2[3]=C_fix(10);
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t4=C_fast_retrieve(lf[1296]);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_fast_retrieve(lf[1296]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_30534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_30534,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30537,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6440: software-type */
t3=C_fast_retrieve(lf[1288]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in ... */
static void C_ccall f_30537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_30537,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30540,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6441: software-version */
t3=C_fast_retrieve(lf[1291]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in ... */
static void C_ccall f_30540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30540,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30543,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6442: machine-type */
t3=C_fast_retrieve(lf[1289]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in ... */
static void C_ccall f_30543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_30543,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30545,a[2]=((C_word)li944),tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_30567,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* library.scm:6447: str */
f_30545(t3,((C_word*)t0)[5]);}

/* str in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in ... */
static void C_fcall f_30545(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_30545,2,t1,t2);}
a=C_alloc(3);
t3=C_eqp(lf[1301],t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=lf[1302];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30559,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6446: scheme#symbol->string */
t5=*((C_word*)lf[634]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k30557 in str in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in ... */
static void C_ccall f_30559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_30559,c,av);}
/* ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[1303];
tp(4,av2);}}

/* k30565 in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in ... */
static void C_ccall f_30567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_30567,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_30571,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* library.scm:6447: str */
f_30545(t2,((C_word*)t0)[6]);}

/* k30569 in k30565 in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in ... */
static void C_ccall f_30571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30571,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30575,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* library.scm:6447: str */
f_30545(t2,((C_word*)t0)[6]);}

/* k30573 in k30569 in k30565 in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in ... */
static void C_ccall f_30575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30575,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30579,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6447: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
tp(3,av2);}}

/* k30577 in k30573 in k30569 in k30565 in k30541 in k30538 in k30535 in k30532 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in ... */
static void C_ccall f_30579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_30579,c,av);}
/* library.scm:6447: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_30585,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30592,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_fast_retrieve(lf[1308]))){
/* library.scm:6457: scheme#string-append */
t3=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1309];
av2[3]=C_fast_retrieve(lf[1308]);
av2[4]=lf[1310];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[1311];
f_30592(2,av2);}}}

/* k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_30592,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30596,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fast_retrieve(lf[1304]))){
/* library.scm:6458: scheme#string-append */
t3=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1305];
av2[3]=C_fast_retrieve(lf[1304]);
av2[4]=lf[1306];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[1307];
f_30596(2,av2);}}}

/* k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_30596,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30600,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30534,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6439: build-platform */
t4=C_fast_retrieve(lf[1292]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30598 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_30600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,8)))){
C_save_and_reclaim((void *)f_30600,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30604,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t3=C_block_size(((C_word*)t0)[5]);
t4=C_eqp(t3,C_fix(0));
if(C_truep(t4)){
/* library.scm:6455: scheme#string-append */
t5=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1295];
av2[3]=C_fast_retrieve(lf[1296]);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=lf[1297];
av2[7]=t1;
av2[8]=lf[1298];
((C_proc)(void*)(*((C_word*)t5+1)))(9,av2);}}
else{
/* library.scm:6463: scheme#string-append */
t5=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[1299];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[1300];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k30602 in k30598 in k30594 in k30590 in k30583 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in ... */
static void C_ccall f_30604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_30604,c,av);}
/* library.scm:6455: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1295];
av2[3]=C_fast_retrieve(lf[1296]);
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
av2[6]=lf[1297];
av2[7]=((C_word*)t0)[5];
av2[8]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}

/* k30622 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_30624,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30652,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6451: feature? */
t3=C_fast_retrieve(lf[1322]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1326];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k30641 in k30644 in k30647 in k30650 in k30622 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in ... */
static void C_ccall f_30643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_30643,c,av);}
if(C_truep(t1)){
/* library.scm:6449: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1318];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1319];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=lf[1320];
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}
else{
/* library.scm:6449: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1318];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1319];
av2[5]=((C_word*)t0)[4];
av2[6]=((C_word*)t0)[5];
av2[7]=((C_word*)t0)[6];
av2[8]=lf[1321];
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}}

/* k30644 in k30647 in k30650 in k30622 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_30646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_30646,c,av);}
a=C_alloc(7);
t2=(C_truep(t1)?lf[1316]:lf[1317]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_30643,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* library.scm:6454: feature? */
t4=C_fast_retrieve(lf[1322]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1323];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k30647 in k30650 in k30622 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30649,c,av);}
a=C_alloc(6);
t2=(C_truep(t1)?lf[1314]:lf[1315]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30646,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6453: feature? */
t4=C_fast_retrieve(lf[1322]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1324];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k30650 in k30622 in chicken.platform#chicken-version in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_30652,c,av);}
a=C_alloc(5);
t2=(C_truep(t1)?lf[1312]:lf[1313]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30649,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6452: feature? */
t4=C_fast_retrieve(lf[1322]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1325];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* chicken.platform#chicken-home in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30661,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30665,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6473: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1329];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k30663 in chicken.platform#chicken-home in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30665(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_30665,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30668,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
/* library.scm:6474: scheme#string-append */
t3=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=lf[1328];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* ##sys#peek-c-string */
t3=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=C_mpointer(&a,(void*)C_INSTALL_SHARE_HOME);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k30666 in k30663 in chicken.platform#chicken-home in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_30668,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* ##sys#peek-c-string */
t2=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_mpointer(&a,(void*)C_INSTALL_SHARE_HOME);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_30683,c,av);}
a=C_alloc(13);
if(C_truep(C_i_not(t2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(((C_word*)((C_word*)t0)[2])[1]);
if(C_truep(C_i_equalp(t2,t3))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(((C_word*)((C_word*)t0)[2])[1]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_string_length(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30705,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30711,a[2]=t4,a[3]=t2,a[4]=t7,a[5]=((C_word)li947),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_30711(t9,t5,C_fix(0),C_fix(0));}}}

/* k30703 in ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30705,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* loop in ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_fcall f_30711(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_30711,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_fixnum_greater_or_equal_p(t3,((C_word*)t0)[2]))){
t4=C_eqp(t3,t2);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30731,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6492: scheme#substring */
t6=*((C_word*)lf[397]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}
else{
t4=C_i_string_ref(((C_word*)t0)[3],t3);
t5=C_retrieve2(lf[1330],C_text("chicken.platform#path-list-separator"));
if(C_truep(C_u_i_char_equalp(t4,C_retrieve2(lf[1330],C_text("chicken.platform#path-list-separator"))))){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30746,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6495: scheme#substring */
t7=*((C_word*)lf[397]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
/* library.scm:6499: loop */
t8=t1;
t9=t2;
t10=C_fixnum_plus(t3,C_fix(1));
t1=t8;
t2=t9;
t3=t10;
goto loop;}}}

/* k30729 in loop in ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30731,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list1(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30744 in loop in ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30746(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_30746,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30750,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6496: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_30711(t3,t2,C_fixnum_plus(((C_word*)t0)[4],C_fix(1)),C_fixnum_plus(((C_word*)t0)[4],C_fix(1)));}

/* k30748 in k30744 in loop in ##sys#split-path in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30750,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_30773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_30773,c,av);}
a=C_alloc(9);
t2=C_mutate((C_word*)lf[1333]+1 /* (set! chicken.platform#repository-path ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30777,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31176,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t5=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_mpointer(&a,(void*)C_private_repository_path());
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_30777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_30777,c,av);}
a=C_alloc(14);
t2=C_mutate((C_word*)lf[1334]+1 /* (set! chicken.platform#installation-repository ...) */,t1);
t3=C_mutate(&lf[1335] /* (set! chicken.platform#->feature-id ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30794,a[2]=((C_word)li949),tmp=(C_word)a,a+=3,tmp));
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,lf[1338]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30828,a[2]=((C_word)li950),tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30846,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31172,a[2]=t5,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6550: software-type */
t8=C_fast_retrieve(lf[1288]);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* chicken.platform#->feature-id in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_30794,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30801,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6535: chicken.keyword#keyword? */
t4=*((C_word*)lf[647]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k30799 in chicken.platform#->feature-id in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_30801,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_stringp(((C_word*)t0)[3]))){
/* library.scm:6536: chicken.keyword#string->keyword */
t2=*((C_word*)lf[648]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30823,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6537: ##sys#symbol->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[633]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[633]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
tp(3,av2);}}
else{
t2=C_a_i_list(&a,1,((C_word*)t0)[3]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[260];
av2[4]=lf[1336];
av2[5]=t2;
C_apply(6,av2);}}}}}

/* k30821 in k30799 in chicken.platform#->feature-id in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_30823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_30823,c,av);}
/* library.scm:6537: chicken.keyword#string->keyword */
t2=*((C_word*)lf[648]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* check in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_fcall f_30828(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_30828,2,t1,t2);}
a=C_alloc(3);
t3=C_eqp(lf[1301],t2);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30843,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6549: ->feature-id */
t5=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
f_30794(3,av2);}}}

/* k30841 in check in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30843(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30843,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_fast_retrieve(lf[1337]));
t3=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t2);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_30846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_30846,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30849,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31168,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6551: software-version */
t4=C_fast_retrieve(lf[1291]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_30849(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_30849,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30852,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31164,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6552: build-platform */
t4=C_fast_retrieve(lf[1292]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_30852(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_30852,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30855,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31160,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6553: machine-type */
t4=C_fast_retrieve(lf[1289]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in ... */
static void C_ccall f_30855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_30855,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30858,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31156,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6554: machine-byte-order */
t4=C_fast_retrieve(lf[1290]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in ... */
static void C_ccall f_30858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_30858,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30861,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_mk_bool(HAVE_DLOAD))){
t3=C_a_i_cons(&a,2,lf[1326],C_fast_retrieve(lf[1337]));
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t3);
t5=t2;
f_30861(t5,t4);}
else{
t3=t2;
f_30861(t3,C_SCHEME_UNDEFINED);}}

/* k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in ... */
static void C_fcall f_30861(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_30861,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30864,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_mk_bool(HAVE_PTABLES))){
t3=C_a_i_cons(&a,2,lf[1325],C_fast_retrieve(lf[1337]));
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t3);
t5=t2;
f_30864(t5,t4);}
else{
t3=t2;
f_30864(t3,C_SCHEME_UNDEFINED);}}

/* k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in ... */
static void C_fcall f_30864(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_30864,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30867,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_mk_bool(HAVE_GCHOOKS))){
t3=C_a_i_cons(&a,2,lf[1324],C_fast_retrieve(lf[1337]));
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t3);
t5=t2;
f_30867(t5,t4);}
else{
t3=t2;
f_30867(t3,C_SCHEME_UNDEFINED);}}

/* k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in ... */
static void C_fcall f_30867(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_30867,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30870,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_mk_bool(IS_CROSS_CHICKEN))){
t3=C_a_i_cons(&a,2,lf[1323],C_fast_retrieve(lf[1337]));
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t3);
t5=t2;
f_30870(t5,t4);}
else{
t3=t2;
f_30870(t3,C_SCHEME_UNDEFINED);}}

/* k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in ... */
static void C_fcall f_30870(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_30870,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31126,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31130,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31134,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=C_fix((C_word)C_WORD_SIZE);
/* ##sys#fixnum->string */
t6=*((C_word*)lf[623]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
av2[3]=C_fix(10);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in ... */
static void C_ccall f_30878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_30878,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30881,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=C_fix((C_word)C_MINOR_VERSION);
/* library.scm:6575: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t2;
av2[2]=t3;
tp(3,av2);}}

/* k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in ... */
static void C_ccall f_30881(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_30881,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31104,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31120,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6576: scheme#string-append */
t4=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1356];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* chicken.platform#register-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_30886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +9,c,3)))){
C_save_and_reclaim((void*)f_30886,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+9);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30905,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30907,a[2]=t5,a[3]=((C_word)li951),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_30907(t7,t3,t2);}

/* k30890 in for-each-loop7324 in chicken.platform#register-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_30892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30892,c,av);}
a=C_alloc(3);
if(C_truep(C_i_memq(t1,C_fast_retrieve(lf[1337])))){
t2=((C_word*)((C_word*)t0)[2])[1];
f_30907(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}
else{
t2=C_a_i_cons(&a,2,t1,C_fast_retrieve(lf[1337]));
t3=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t2);
t4=((C_word*)((C_word*)t0)[2])[1];
f_30907(t4,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}}

/* k30903 in chicken.platform#register-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_ccall f_30905(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30905,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* for-each-loop7324 in chicken.platform#register-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_fcall f_30907(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_30907,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30892,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6583: ->feature-id */
t5=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
f_30794(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_30930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +16,c,3)))){
C_save_and_reclaim((void*)f_30930,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+16);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30937,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30974,a[2]=t5,a[3]=t10,a[4]=t6,a[5]=((C_word)li954),tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_30974(t12,t8,t2);}

/* k30935 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_ccall f_30937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_30937,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30941,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_30943,a[2]=t1,a[3]=t4,a[4]=((C_word)li953),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_30943(t6,t2,C_fast_retrieve(lf[1337]));}

/* k30939 in k30935 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_30941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_30941,c,av);}
t2=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* loop in k30935 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_fcall f_30943(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_30943,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_memq(t3,((C_word*)t0)[2]))){
/* library.scm:6597: loop */
t7=t1;
t8=t4;
t1=t7;
t2=t8;
goto loop;}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_30972,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6598: loop */
t7=t5;
t8=t4;
t1=t7;
t2=t8;
goto loop;}}}

/* k30970 in loop in k30935 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in ... */
static void C_ccall f_30972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_30972,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* map-loop7347 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_fcall f_30974(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_30974,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_30999,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* library.scm:6589: g7353 */
t4=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
f_30794(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k30997 in map-loop7347 in chicken.platform#unregister-feature! in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_30999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_30999,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_30974(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.platform#features in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_31008(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31008,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_fast_retrieve(lf[1337]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.platform#feature? in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_31011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_31011,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31017,a[2]=t4,a[3]=((C_word)li957),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_31017(t6,t1,t2);}

/* loop in chicken.platform#feature? in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_fcall f_31017(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_31017,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31041,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:6606: ->feature-id */
t5=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_slot(t2,C_fix(0));
f_30794(3,av2);}}}

/* k31039 in loop in chicken.platform#feature? in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_31041(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31041,c,av);}
if(C_truep(C_i_memq(t1,C_fast_retrieve(lf[1337])))){
/* library.scm:6607: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_31017(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.platform#system-config-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_31048(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31048,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31052,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6613: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1347];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k31050 in chicken.platform#system-config-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_ccall f_31052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31052,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_fast_retrieve(lf[1293]))){
/* library.scm:6615: chicken.process-context#get-environment-variable */
t2=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1344];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31064,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6616: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1346];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k31062 in k31050 in chicken.platform#system-config-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_31064(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_31064,c,av);}
if(C_truep(t1)){
/* library.scm:6617: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[1345];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.platform#system-cache-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in ... */
static void C_ccall f_31072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31072,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31076,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6620: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1353];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k31074 in chicken.platform#system-cache-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in ... */
static void C_ccall f_31076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31076,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_fast_retrieve(lf[1293]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31085,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6622: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1350];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31094,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6624: chicken.process-context#get-environment-variable */
t3=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[1352];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k31083 in k31074 in chicken.platform#system-cache-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_31085(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31085,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:6623: chicken.process-context#get-environment-variable */
t2=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1349];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* k31092 in k31074 in chicken.platform#system-cache-directory in k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in ... */
static void C_ccall f_31094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_31094,c,av);}
if(C_truep(t1)){
/* library.scm:6625: scheme#string-append */
t2=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[1351];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in ... */
static void C_ccall f_31104(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,5)))){
C_save_and_reclaim((void *)f_31104,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31112,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31116,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6577: scheme#string-append */
t4=*((C_word*)lf[417]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1354];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[1355];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k31110 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in ... */
static void C_ccall f_31112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,2)))){
C_save_and_reclaim((void *)f_31112,c,av);}
a=C_alloc(27);
t2=C_a_i_cons(&a,2,t1,C_fast_retrieve(lf[1337]));
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
t4=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t3);
t5=C_mutate((C_word*)lf[1339]+1 /* (set! chicken.platform#register-feature! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30886,a[2]=((C_word)li952),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[1340]+1 /* (set! chicken.platform#unregister-feature! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30930,a[2]=((C_word)li955),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[1341]+1 /* (set! chicken.platform#features ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31008,a[2]=((C_word)li956),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[1322]+1 /* (set! chicken.platform#feature? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31011,a[2]=((C_word)li958),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[1342]+1 /* (set! chicken.platform#return-to-host ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_return_to_host,a[2]=((C_word)li959),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[1343]+1 /* (set! chicken.platform#system-config-directory ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31048,a[2]=((C_word)li960),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[1348]+1 /* (set! chicken.platform#system-cache-directory ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31072,a[2]=((C_word)li961),tmp=(C_word)a,a+=3,tmp));
t12=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t12;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}

/* k31114 in k31102 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in ... */
static void C_ccall f_31116(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31116,c,av);}
/* library.scm:6577: ->feature-id */
t2=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_30794(3,av2);}}

/* k31118 in k30879 in k30876 in k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in ... */
static void C_ccall f_31120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31120,c,av);}
/* library.scm:6576: ->feature-id */
t2=C_retrieve2(lf[1335],C_text("chicken.platform#->feature-id"));{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
f_30794(3,av2);}}

/* k31124 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in ... */
static void C_ccall f_31126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_31126,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,t1,C_fast_retrieve(lf[1337]));
t3=C_mutate((C_word*)lf[1337]+1 /* (set! ##sys#features ...) */,t2);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_30878,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t5=C_fix((C_word)C_MAJOR_VERSION);
/* library.scm:6574: ##sys#number->string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[603]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[603]+1);
av2[1]=t4;
av2[2]=t5;
tp(3,av2);}}

/* k31128 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in ... */
static void C_ccall f_31130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31130,c,av);}
/* library.scm:6567: chicken.keyword#string->keyword */
t2=*((C_word*)lf[648]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31132 in k30868 in k30865 in k30862 in k30859 in k30856 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in ... */
static void C_ccall f_31134(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_31134,c,av);}
/* ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[1357];
tp(4,av2);}}

/* k31154 in k30853 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in ... */
static void C_ccall f_31156(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31156,c,av);}
/* library.scm:6554: check */
f_30828(((C_word*)t0)[3],t1);}

/* k31158 in k30850 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in ... */
static void C_ccall f_31160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31160,c,av);}
/* library.scm:6553: check */
f_30828(((C_word*)t0)[3],t1);}

/* k31162 in k30847 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in ... */
static void C_ccall f_31164(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31164,c,av);}
/* library.scm:6552: check */
f_30828(((C_word*)t0)[3],t1);}

/* k31166 in k30844 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static void C_ccall f_31168(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31168,c,av);}
/* library.scm:6551: check */
f_30828(((C_word*)t0)[3],t1);}

/* k31170 in k30775 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_31172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31172,c,av);}
/* library.scm:6550: check */
f_30828(((C_word*)t0)[3],t1);}

/* k31174 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_31176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_31176,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31179,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
/* library.scm:6520: chicken.base#make-parameter */
t3=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31182,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:6522: chicken.process-context#get-environment-variable */
t4=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1358];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k31177 in k31174 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_31179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31179,c,av);}
/* library.scm:6520: chicken.base#make-parameter */
t2=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31180 in k31174 in k30771 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_31182(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31182,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
/* library.scm:6520: chicken.base#make-parameter */
t2=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
/* ##sys#peek-c-string */
t2=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=C_mpointer(&a,(void*)C_INSTALL_EGG_HOME);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_31192(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_31192,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31195,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t1)){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list1(&a,1,t1);
f_31195(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31241,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6507: chicken.process-context#get-environment-variable */
t4=C_fast_retrieve(lf[1195]);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1360];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k31193 in k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_31195(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31195,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31197,a[2]=((C_word)li963),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6504: chicken.base#make-parameter */
t3=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a31196 in k31193 in k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_31197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31197,c,av);}
a=C_alloc(3);
if(C_truep(t2)){
t3=C_i_check_list_2(t2,lf[1359]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31214,a[2]=((C_word)li962),tmp=(C_word)a,a+=3,tmp);
t5=(
  f_31214(t2)
);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* for-each-loop7266 in a31196 in k31193 in k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in ... */
static C_word C_fcall f_31214(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=C_slot(t1,C_fix(0));
t3=C_i_check_string_2(t2,lf[1359]);
t5=C_slot(t1,C_fix(1));
t1=t5;
goto loop;}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* k31239 in k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in ... */
static void C_ccall f_31241(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_31241,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
/* library.scm:6505: ##sys#split-path */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1332]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[1332]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
tp(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31250,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t3=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_mpointer(&a,(void*)C_INSTALL_EGG_HOME);
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k31248 in k31239 in k31190 in k30520 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in ... */
static void C_ccall f_31250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_31250,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_list1(&a,1,t1):C_SCHEME_FALSE);
f_31195(2,av2);}}

/* k31268 in k31272 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in ... */
static void C_ccall f_31270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31270,c,av);}
t2=((C_word*)t0)[2];
f_30522(t2,C_i_not(C_eqp(lf[1362],t1)));}

/* k31272 in k30514 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in ... */
static void C_ccall f_31274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31274,c,av);}
a=C_alloc(3);
t2=C_eqp(lf[1361],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31270,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6435: software-version */
t4=C_fast_retrieve(lf[1291]);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=((C_word*)t0)[2];
f_30522(t3,C_SCHEME_FALSE);}}

/* k31276 in k30508 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in ... */
static void C_ccall f_31278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31278,c,av);}
/* library.scm:6429: scheme#string->symbol */
t2=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31280 in k30502 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in ... */
static void C_ccall f_31282(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31282,c,av);}
/* library.scm:6425: scheme#string->symbol */
t2=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31284 in k30496 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in ... */
static void C_ccall f_31286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31286,c,av);}
/* library.scm:6421: scheme#string->symbol */
t2=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31288 in k30490 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in ... */
static void C_ccall f_31290(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31290,c,av);}
/* library.scm:6417: scheme#string->symbol */
t2=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k31292 in k30189 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_31294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31294,c,av);}
/* library.scm:6413: scheme#string->symbol */
t2=*((C_word*)lf[636]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a31295 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_31296(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31296,c,av);}
t3=C_i_check_symbol_2(t2,lf[1363]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_slot(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31304 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in ... */
static void C_ccall f_31305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_31305,c,av);}
a=C_alloc(5);
t4=C_i_check_symbol_2(t2,lf[1363]);
t5=C_i_check_list_2(t3,lf[1364]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31315,a[2]=t3,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t7=C_i_length(t3);
if(C_truep(C_i_fixnumevenp(t7))){
t8=C_i_setslot(t2,C_fix(2),t3);
t9=C_i_nullp(t3);
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=(C_truep(t9)?C_i_unpersist_symbol(t2):C_i_persist_symbol(t2));
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
/* library.scm:6316: ##sys#signal-hook */
t8=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t8;
av2[1]=t6;
av2[2]=lf[260];
av2[3]=lf[1365];
av2[4]=t3;
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(6,av2);}}}

/* k31313 in a31304 in k30118 in k29756 in k29518 in k29481 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in ... */
static void C_ccall f_31315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31315,c,av);}
t2=C_i_nullp(((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_i_unpersist_symbol(((C_word*)t0)[4]):C_i_persist_symbol(((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_31340(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_31340,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31343,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_i_pairp(t1))){
t3=C_slot(t1,C_fix(1));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31361,a[2]=t5,a[3]=((C_word)li972),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_31361(t7,t2,t3);}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f37127,a[2]=((C_word)li973),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6026: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k31341 in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_31343(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31343,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31345,a[2]=((C_word)li971),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6026: chicken.base#make-parameter */
t3=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a31344 in k31341 in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_31345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31345,c,av);}
t3=C_i_check_list_2(t2,lf[1367]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* loop in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_fcall f_31361(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_31361,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_31380,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t6=C_block_size(t3);
if(C_truep(C_fixnum_greater_or_equal_p(t6,C_fix(3)))){
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31400,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* library.scm:6035: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t7;
av2[2]=t3;
av2[3]=C_fix(0);
av2[4]=C_fix(2);
tp(5,av2);}}
else{
t7=t5;
f_31380(t7,C_SCHEME_FALSE);}}}

/* k31378 in loop in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_fcall f_31380(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_31380,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
/* library.scm:6036: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_31361(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31390,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* library.scm:6037: loop */
t3=((C_word*)((C_word*)t0)[2])[1];
f_31361(t3,t2,((C_word*)t0)[4]);}}

/* k31388 in k31378 in loop in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in ... */
static void C_ccall f_31390(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_31390,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k31398 in loop in k31338 in k29477 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in ... */
static void C_ccall f_31400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31400,c,av);}
t2=((C_word*)t0)[2];
f_31380(t2,C_i_string_equal_p(lf[1368],t1));}

/* k31422 in k31426 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_31424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31424,c,av);}
a=C_alloc(3);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f37139,a[2]=((C_word)li975),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6017: chicken.base#make-parameter */
t4=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k31426 in k29298 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_31428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31428,c,av);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f37133,a[2]=((C_word)li974),tmp=(C_word)a,a+=3,tmp);
/* library.scm:6017: chicken.base#make-parameter */
t3=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1370];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31424,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:6020: argv */
t3=C_fast_retrieve(lf[1203]);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a31429 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_31430,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31434,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t2;
av2[2]=C_fix(1024);
av2[3]=C_make_character(32);
tp(4,av2);}}

/* k31432 in a31429 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_31434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_31434,c,av);}
a=C_alloc(5);
t2=C_curdir(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31437,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_fast_retrieve(lf[1293]))){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_31437(2,av2);}}
else{
/* library.scm:5949: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t3;
tp(2,av2);}}}

/* k31435 in k31432 in a31429 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_31437(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31437,c,av);}
if(C_truep(((C_word*)t0)[2])){
/* library.scm:5951: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_fix(0);
av2[4]=((C_word*)t0)[2];
tp(5,av2);}}
else{
/* library.scm:5952: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[699];
av2[3]=lf[1371];
av2[4]=lf[1372];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* a31451 in k28661 in k28601 in k27232 in k27229 in k27226 in k26459 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31452,c,av);}
/* library.scm:5956: ##sys#change-directory-hook */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1193]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1193]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}

/* a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_31458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +8,c,2)))){
C_save_and_reclaim((void*)f_31458,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+8);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31462,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31615,a[2]=((C_word)li982),tmp=(C_word)a,a+=3,tmp);
/* library.scm:5138: ##sys#error-handler */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1008]);
C_word *av2=av;
av2[0]=*((C_word*)lf[1008]+1);
av2[1]=t4;
av2[2]=t5;
tp(3,av2);}}

/* k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_31462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_31462,c,av);}
a=C_alloc(5);
t2=C_mk_bool(C_gui_mode);
if(C_truep(C_i_not(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31471,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5140: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1381];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31565,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5165: chicken.base#open-output-string */
t4=*((C_word*)lf[215]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_31471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_31471,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31474,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31559,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5142: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1380];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_31474(2,av2);}}}

/* k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_31474(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_31474,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31477,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31503,a[2]=((C_word*)t0)[3],a[3]=((C_word)li979),tmp=(C_word)a,a+=4,tmp);
/* library.scm:5144: ##sys#with-print-length-limit */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[921]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[921]+1);
av2[1]=t2;
av2[2]=C_fix(400);
av2[3]=t3;
tp(4,av2);}}

/* k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_31477(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_31477,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31480,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5156: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k31478 in k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_31480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31480,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31483,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5157: chicken.base#print-call-chain */
t3=*((C_word*)lf[187]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k31481 in k31478 in k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31483,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31489,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_fast_retrieve(lf[1007]))){
/* library.scm:5158: ##sys#symbol-has-toplevel-binding? */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[1149]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[1149]+1);
av2[1]=t2;
av2[2]=lf[1378];
tp(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_31489(2,av2);}}}

/* k31487 in k31481 in k31478 in k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in ... */
static void C_ccall f_31489(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31489,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_slot(lf[1378],C_fix(0));
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31495,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5158: g5886 */
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)C_fast_retrieve_proc(t4))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_halt(C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k31493 in k31487 in k31481 in k31478 in k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in ... */
static void C_ccall f_31495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_31495,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31498,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5161: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k31496 in k31493 in k31487 in k31481 in k31478 in k31475 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in ... */
static void C_ccall f_31498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31498,c,av);}
t2=C_exit_runtime(C_fix((C_word)EX_SOFTWARE));
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_halt(C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a31502 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_31503(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_31503,c,av);}
a=C_alloc(6);
t2=C_eqp(C_fix(1),C_u_i_length(((C_word*)t0)[2]));
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31513,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5148: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[1379];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31533,a[2]=t4,a[3]=((C_word)li978),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_31533(t6,t1,((C_word*)t0)[2]);}}

/* k31511 in a31502 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_31513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31513,c,av);}
/* library.scm:5149: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_slot(((C_word*)t0)[3],C_fix(0));
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k31523 in for-each-loop5868 in a31502 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31525,c,av);}
/* library.scm:5154: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_TRUE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* for-each-loop5868 in a31502 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_31533(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_31533,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31543,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31525,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5153: ##sys#print */
t6=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_make_character(10);
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k31541 in for-each-loop5868 in a31502 in k31472 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31543,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_31533(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k31557 in k31469 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_31559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31559,c,av);}
/* library.scm:5143: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_FALSE;
av2[4]=*((C_word*)lf[715]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in ... */
static void C_ccall f_31565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_31565,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31568,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[4])){
/* library.scm:5166: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_31568(2,av2);}}}

/* k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in ... */
static void C_ccall f_31568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_31568,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31571,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* library.scm:5167: ##sys#print */
t3=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(10);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in ... */
static void C_ccall f_31571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_31571,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31572,a[2]=((C_word*)t0)[2],a[3]=((C_word)li980),tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31582,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31588,a[2]=t5,a[3]=t2,a[4]=((C_word)li981),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_31588(t7,t3,((C_word*)t0)[4]);}

/* g5900 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_31572(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_31572,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31576,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5168: ##sys#print */
t4=*((C_word*)lf[867]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k31574 in g5900 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31576,c,av);}
/* library.scm:5168: ##sys#print */
t2=*((C_word*)lf[867]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_make_character(10);
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k31580 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_ccall f_31582(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31582,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31586,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* library.scm:5169: chicken.base#get-output-string */
t3=*((C_word*)lf[213]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k31584 in k31580 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31586(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31586,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_halt(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* for-each-loop5899 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in ... */
static void C_fcall f_31588(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_31588,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31598,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:5168: g5900 */
t4=((C_word*)t0)[3];
f_31572(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k31596 in for-each-loop5899 in k31569 in k31566 in k31563 in k31460 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in ... */
static void C_ccall f_31598(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31598,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_31588(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a31614 in a31457 in k26147 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in ... */
static void C_ccall f_31615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31615,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_halt(lf[1382]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a31617 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_31618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_31618,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31622,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:5040: chicken.base#exit-handler */
t3=*((C_word*)lf[226]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k31620 in a31617 in k26143 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in ... */
static void C_ccall f_31622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_31622,c,av);}
/* library.scm:5039: g5774 */
t2=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix((C_word)EX_SOFTWARE);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a31626 in k26139 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31627,c,av);}
/* library.scm:5035: cleanup-before-exit */
f_26038(t1);}

/* a31632 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31633(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_31633,c,av);}
a=C_alloc(4);
t2=C_rest_nullp(c,2);
t3=(C_truep(t2)?C_fix(0):C_get_rest_arg(c,2,av,2,t0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31640,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* library.scm:5025: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t3;
tp(3,av2);}}

/* k31638 in a31632 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_31640,c,av);}
a=C_alloc(4);
if(C_truep(C_retrieve2(lf[956],C_text("exit-in-progress")))){
/* library.scm:5027: ##sys#warn */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[313]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[313]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[1383];
tp(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31649,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:5029: cleanup-before-exit */
f_26038(t2);}}

/* k31647 in k31638 in a31632 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_31649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31649,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_exit_runtime(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a31656 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31657,c,av);}
t3=C_slot(t2,C_fix(10));
t4=C_slot(t2,C_fix(12));
t5=C_slot(t2,C_fix(11));
if(C_truep(C_fixnum_greater_or_equal_p(t3,t5))){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_END_OF_FILE;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=C_subchar(t4,t3);
t7=C_fixnum_plus(t3,C_fix(1));
t8=C_i_set_i_slot(t2,C_fix(10),t7);
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}

/* a31681 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31682(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31682,c,av);}
t3=C_slot(t2,C_fix(10));
t4=C_slot(t2,C_fix(12));
t5=C_slot(t2,C_fix(11));
t6=C_fixnum_greater_or_equal_p(t3,t5);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(C_truep(t6)?C_SCHEME_END_OF_FILE:C_subchar(t4,t3));
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* a31699 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_31700,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31704,a[2]=t2,a[3]=t3,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4778: check */
t5=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
av2[3]=C_fix(1);
((C_proc)C_fast_retrieve_proc(t5))(4,av2);}}

/* k31702 in a31699 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31704,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(10));
t3=C_slot(((C_word*)t0)[2],C_fix(12));
t4=C_setsubchar(t3,t2,((C_word*)t0)[3]);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),C_fixnum_plus(t2,C_fix(1)));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* a31718 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31719(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_31719,c,av);}
a=C_alloc(6);
t4=C_block_size(t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_31723,a[2]=t2,a[3]=t3,a[4]=t4,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* library.scm:4785: check */
t6=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=t4;
((C_proc)C_fast_retrieve_proc(t6))(4,av2);}}

/* k31721 in a31718 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31723(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31723,c,av);}
t2=C_slot(((C_word*)t0)[2],C_fix(10));
t3=C_slot(((C_word*)t0)[2],C_fix(12));
t4=C_substring_copy(((C_word*)t0)[3],t3,C_fix(0),((C_word*)t0)[4],t2);
t5=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),C_fixnum_plus(t2,((C_word*)t0)[4]));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* a31737 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31738,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a31740 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31741,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a31743 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_31744,c,av);}
a=C_alloc(10);
t6=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t7=C_slot(t2,C_fix(10));
t8=C_slot(t2,C_fix(11));
t9=C_fixnum_difference(t8,t7);
t10=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_31754,a[2]=t2,a[3]=t7,a[4]=t6,a[5]=t4,a[6]=t5,a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t11=C_i_not(((C_word*)t6)[1]);
if(C_truep(t11)){
if(C_truep(t11)){
t12=C_set_block_item(t6,0,t9);
t13=t10;
f_31754(t13,t12);}
else{
t12=t10;
f_31754(t12,C_SCHEME_UNDEFINED);}}
else{
if(C_truep(C_fixnum_greaterp(((C_word*)t6)[1],t9))){
t12=C_set_block_item(t6,0,t9);
t13=t10;
f_31754(t13,t12);}
else{
t12=t10;
f_31754(t12,C_SCHEME_UNDEFINED);}}}

/* k31752 in a31743 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_fcall f_31754(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,1)))){
C_save_and_reclaim_args((void *)trf_31754,2,t0,t1);}
t2=C_slot(((C_word*)t0)[2],C_fix(12));
t3=C_fixnum_plus(((C_word*)t0)[3],((C_word*)((C_word*)t0)[4])[1]);
t4=C_substring_copy(t2,((C_word*)t0)[5],((C_word*)t0)[3],t3,((C_word*)t0)[6]);
t5=C_fixnum_plus(((C_word*)t0)[3],((C_word*)((C_word*)t0)[4])[1]);
t6=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),t5);
t7=((C_word*)t0)[7];{
C_word av2[2];
av2[0]=t7;
av2[1]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* a31784 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_31785,c,av);}
a=C_alloc(11);
t4=C_slot(t2,C_fix(10));
t5=C_slot(t2,C_fix(11));
t6=C_slot(t2,C_fix(12));
t7=(C_truep(t3)?C_fixnum_plus(t4,t3):t5);
if(C_truep(C_fixnum_greater_or_equal_p(t4,t5))){
t8=t1;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_END_OF_FILE;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_31809,a[2]=t7,a[3]=t5,a[4]=t6,a[5]=t4,a[6]=((C_word)li747),tmp=(C_word)a,a+=7,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31828,a[2]=t2,a[3]=((C_word)li748),tmp=(C_word)a,a+=4,tmp);
/* library.scm:4807: ##sys#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t8;
av2[3]=t9;
C_call_with_values(4,av2);}}}

/* a31808 in a31784 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31809(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_31809,c,av);}
a=C_alloc(3);
t2=C_fixnum_greaterp(((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(C_truep(t2)?((C_word*)t0)[3]:((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_31819,a[2]=((C_word)li746),tmp=(C_word)a,a+=3,tmp);
/* library.scm:4808: ##sys#scan-buffer-line */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[924]);
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=*((C_word*)lf[924]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[4];
av2[3]=t3;
av2[4]=((C_word*)t0)[5];
av2[5]=t4;
tp(6,av2);}}

/* a31818 in a31808 in a31784 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in ... */
static void C_ccall f_31819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_31819,c,av);}
/* library.scm:4810: scheme#values */{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_SCHEME_FALSE;
av2[3]=t2;
av2[4]=C_SCHEME_FALSE;
C_values(5,av2);}}

/* a31827 in a31784 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31828,c,av);}
if(C_truep(t4)){
t5=C_slot(((C_word*)t0)[2],C_fix(4));
t6=C_fixnum_plus(t5,C_fix(1));
t7=C_i_set_i_slot(((C_word*)t0)[2],C_fix(4),t6);
t8=C_i_set_i_slot(((C_word*)t0)[2],C_fix(5),C_fix(0));
t9=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),t2);
t10=t1;{
C_word *av2=av;
av2[0]=t10;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t5=C_slot(((C_word*)t0)[2],C_fix(5));
t6=C_block_size(t3);
t7=C_i_set_i_slot(((C_word*)t0)[2],C_fix(5),C_fixnum_plus(t5,t6));
t8=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),t2);
t9=t1;{
C_word *av2=av;
av2[0]=t9;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}

/* a31868 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in ... */
static void C_ccall f_31869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_31869,c,av);}
a=C_alloc(5);
t3=C_slot(t2,C_fix(10));
t4=C_slot(t2,C_fix(12));
t5=C_slot(t2,C_fix(11));
if(C_truep(C_fixnum_greater_or_equal_p(t3,t5))){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=lf[925];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_31888,a[2]=t2,a[3]=t5,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* library.scm:4825: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t6;
av2[2]=t4;
av2[3]=t3;
av2[4]=t5;
tp(5,av2);}}}

/* k31886 in a31868 in k25015 in k23851 in k23847 in k23843 in k20285 in k20281 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in ... */
static void C_ccall f_31888(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31888,c,av);}
t2=C_i_set_i_slot(((C_word*)t0)[2],C_fix(10),((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a31896 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in ... */
static void C_ccall f_31897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_31897,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_31901,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t2)){
/* library.scm:3694: ##sys#check-keyword */
t4=*((C_word*)lf[353]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[1385];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k31899 in a31896 in k20277 in k20273 in k20269 in k19909 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in ... */
static void C_ccall f_31901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31901,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a31905 in k19905 in k19901 in k19897 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in ... */
static void C_ccall f_31906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31906,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_car(C_i_list_tail(t2,t3),t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* a31915 in k19893 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in ... */
static void C_ccall f_31916(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31916,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_cddr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31925 in k19889 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in ... */
static void C_ccall f_31926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31926,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_cdar(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31935 in k19885 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in ... */
static void C_ccall f_31936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31936,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_cadr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31945 in k19881 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in ... */
static void C_ccall f_31946(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31946,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_caar(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31955 in k19877 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in ... */
static void C_ccall f_31956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31956,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_cddr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31965 in k19873 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in ... */
static void C_ccall f_31966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31966,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_cdar(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31975 in k19869 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in ... */
static void C_ccall f_31976(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31976,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_cadr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31985 in k19865 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in ... */
static void C_ccall f_31986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31986,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_caar(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a31995 in k19861 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in ... */
static void C_ccall f_31996(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_31996,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_cdr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32005 in k19857 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in ... */
static void C_ccall f_32006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32006,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(C_i_car(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32015 in k19853 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in ... */
static void C_ccall f_32016(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32016,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_cdr(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32025 in k19849 in k19845 in k19765 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in ... */
static void C_ccall f_32026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32026,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(C_i_car(t2),t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32035 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_32036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_32036,c,av);}
a=C_alloc(7);
if(C_truep(C_i_closurep(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32052,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32060,a[2]=((C_word)li1001),tmp=(C_word)a,a+=3,tmp);
/* library.scm:3515: ##sys#lambda-decoration */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[750]);
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[750]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=t4;
tp(4,av2);}}
else{
/* library.scm:3517: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[757];
av2[3]=lf[1386];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k32050 in a32035 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_32052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_32052,c,av);}
if(C_truep(t1)){
t2=C_slot(t1,C_fix(1));
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* library.scm:3517: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[757];
av2[3]=lf[1386];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}
else{
/* library.scm:3517: ##sys#error */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[757];
av2[3]=lf[1386];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* a32059 in a32035 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_32060(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32060,c,av);}
t3=C_i_pairp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_eqp(C_retrieve2(lf[758],C_text("setter-tag")),C_slot(t2,C_fix(0))):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32075 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_32076(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32076,c,av);}
t3=C_i_pairp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_eqp(C_retrieve2(lf[758],C_text("setter-tag")),C_slot(t2,C_fix(0))):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32091 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_32092(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_32092,c,av);}
a=C_alloc(6);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32102,a[2]=((C_word)li1006),tmp=(C_word)a,a+=3,tmp);
t5=C_a_i_cons(&a,2,C_retrieve2(lf[758],C_text("setter-tag")),t4);
t6=C_i_setslot(t2,t3,t5);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* a32101 in a32091 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in ... */
static void C_ccall f_32102(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_32102,c,av);}
a=C_alloc(11);
if(C_truep(C_i_closurep(t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32112,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32131,a[2]=((C_word)li1004),tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32147,a[2]=t3,a[3]=((C_word)li1005),tmp=(C_word)a,a+=4,tmp);
/* library.scm:3526: ##sys#decorate-lambda */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[749]);
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[749]+1);
av2[1]=t4;
av2[2]=t2;
av2[3]=t5;
av2[4]=t6;
tp(5,av2);}}
else{
/* library.scm:3533: chicken.base#error */
t4=*((C_word*)lf[193]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[1387];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k32110 in a32101 in a32091 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_32112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_32112,c,av);}
a=C_alloc(6);
t2=C_eqp(((C_word*)t0)[2],t1);
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t4=C_a_i_list1(&a,1,t3);
/* library.scm:3532: ##sys#become! */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[331]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[331]+1);
av2[1]=((C_word*)t0)[3];
av2[2]=t4;
tp(3,av2);}}}

/* a32130 in a32101 in a32091 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_32131(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32131,c,av);}
t3=C_i_pairp(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?C_eqp(C_retrieve2(lf[758],C_text("setter-tag")),C_slot(t2,C_fix(0))):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32146 in a32101 in a32091 in k18989 in k18986 in k18983 in k18980 in k18976 in k18972 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in ... */
static void C_ccall f_32147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_32147,c,av);}
a=C_alloc(3);
t4=C_a_i_cons(&a,2,C_retrieve2(lf[758],C_text("setter-tag")),((C_word*)t0)[2]);
t5=C_i_setslot(t2,t3,t4);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* a32159 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_32160,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32166,a[2]=t2,a[3]=t4,a[4]=((C_word)li545),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
f_32166(2,av2);}}

/* loop in a32159 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32166(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_32166,c,av);}
a=C_alloc(5);
t2=C_read_char(((C_word*)t0)[2]);
t3=C_eqp(C_fix(-1),t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32197,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3185: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t4;
tp(2,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k32187 in k32195 in loop in a32159 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32189(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_32189,c,av);}
/* library.scm:3187: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=lf[700];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k32191 in k32195 in loop in a32159 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32193,c,av);}
/* library.scm:3189: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[701];
av2[3]=t1;
tp(4,av2);}}

/* k32195 in loop in a32159 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_32197,c,av);}
a=C_alloc(10);
t2=C_fix((C_word)EINTR);
t3=C_eqp(t1,t2);
if(C_truep(t3)){
/* library.scm:3186: ##sys#dispatch-interrupt */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[698]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[698]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
tp(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32189,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32193,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t6=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* a32199 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_32200,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32206,a[2]=t2,a[3]=t4,a[4]=((C_word)li547),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
f_32206(2,av2);}}

/* loop in a32199 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_32206,c,av);}
a=C_alloc(5);
t2=C_peek_char(((C_word*)t0)[2]);
t3=C_eqp(C_fix(-1),t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32237,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:3197: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t4;
tp(2,av2);}}
else{
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k32227 in k32235 in loop in a32199 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32229(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_32229,c,av);}
/* library.scm:3199: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=lf[704];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k32231 in k32235 in loop in a32199 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32233(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32233,c,av);}
/* library.scm:3201: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[705];
av2[3]=t1;
tp(4,av2);}}

/* k32235 in loop in a32199 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32237(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_32237,c,av);}
a=C_alloc(10);
t2=C_fix((C_word)EINTR);
t3=C_eqp(t1,t2);
if(C_truep(t3)){
/* library.scm:3198: ##sys#dispatch-interrupt */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[698]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[698]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
tp(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32229,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32233,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t6=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* a32239 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32240(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32240,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_display_char(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32242 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32243,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_display_string(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a32245 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32246,c,av);}
t4=C_close_file(t2);
/* library.scm:3210: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word *av2=av;
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t1;
tp(2,av2);}}

/* a32251 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32252(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32252,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_flush_output(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a32254 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32255(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32255,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_char_ready_p(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5=av[5];
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(c!=6) C_bad_argc_2(c,6,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_32258,c,av);}
a=C_alloc(9);
t6=(C_truep(t3)?t3:C_fixnum_difference(C_block_size(t4),t5));
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_32268,a[2]=t4,a[3]=t2,a[4]=t8,a[5]=t3,a[6]=((C_word)li555),tmp=(C_word)a,a+=7,tmp));
t10=((C_word*)t8)[1];
f_32268(t10,t1,t6,C_fix(0),t5);}

/* loop in a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_32268(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_32268,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=fast_read_string_from_file(((C_word*)t0)[2],((C_word*)t0)[3],t2,t4);
if(C_truep(C_eofp(t5))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(t5,C_fix(0)))){
t6=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_32323,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=t5,a[5]=t3,a[6]=t4,a[7]=t1,a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
/* library.scm:3221: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word av2[2];
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t6;
tp(2,av2);}}
else{
if(C_truep(C_fixnum_lessp(t5,t2))){
/* library.scm:3230: loop */
t8=t1;
t9=C_fixnum_difference(t2,t5);
t10=C_fixnum_plus(t3,t5);
t11=C_fixnum_plus(t4,t5);
t1=t8;
t2=t9;
t3=t10;
t4=t11;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_fixnum_plus(t3,t5);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}

/* a32291 in k32321 in loop in a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_32292,c,av);}
/* library.scm:3224: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_32268(t2,t1,C_fixnum_difference(((C_word*)t0)[3],((C_word*)t0)[4]),C_fixnum_plus(((C_word*)t0)[5],((C_word*)t0)[4]),C_fixnum_plus(((C_word*)t0)[6],((C_word*)t0)[4]));}

/* k32313 in k32321 in loop in a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_32315,c,av);}
/* library.scm:3225: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=lf[706];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
av2[6]=((C_word*)t0)[4];
av2[7]=((C_word*)t0)[5];
av2[8]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t2+1)))(9,av2);}}

/* k32317 in k32321 in loop in a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32319(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32319,c,av);}
/* library.scm:3227: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[707];
av2[3]=t1;
tp(4,av2);}}

/* k32321 in loop in a32257 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32323(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_32323,c,av);}
a=C_alloc(13);
t2=C_fix((C_word)EINTR);
t3=C_eqp(t1,t2);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_32292,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word)li554),tmp=(C_word)a,a+=8,tmp);
/* library.scm:3222: ##sys#dispatch-interrupt */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[698]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[698]+1);
av2[1]=((C_word*)t0)[7];
av2[2]=t4;
tp(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_32315,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32319,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t6=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_32357,c,av);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32361,a[2]=t3,a[3]=t2,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t3)){
/* library.scm:3234: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2=av;
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t4;
av2[2]=t3;
av2[3]=lf[708];
tp(4,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_32361(2,av2);}}}

/* k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_32361,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32364,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_slot(((C_word*)t0)[3],C_fix(12)))){
t3=t2;
f_32364(t3,C_SCHEME_UNDEFINED);}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32530,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3237: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t3;
av2[2]=C_fix(1024);
tp(3,av2);}}}

/* k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_32364(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,7)))){
C_save_and_reclaim_args((void *)trf_32364,2,t0,t1);}
a=C_alloc(8);
t2=(C_truep(((C_word*)t0)[2])?((C_word*)t0)[2]:lf[271]);
t3=C_slot(((C_word*)t0)[3],C_fix(12));
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_32377,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=((C_word*)t0)[2],a[5]=((C_word)li558),tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_32377(t7,((C_word*)t0)[4],C_fix(1024),t2,t3,lf[710],C_SCHEME_FALSE);}

/* loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_fcall f_32377(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_32377,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(11);
t7=C_i_fixnum_min(t3,t2);
t8=fast_read_line_from_file(t4,((C_word*)t0)[2],t7);
if(C_truep(C_eofp(t8))){
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=(C_truep(t6)?t5:C_SCHEME_END_OF_FILE);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
if(C_truep(C_i_not(t8))){
if(C_truep(C_fixnum_lessp(t3,t2))){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32410,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3248: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t9;
av2[2]=t4;
av2[3]=C_fix(0);
av2[4]=t3;
tp(5,av2);}}
else{
t9=C_fixnum_times(t2,C_fix(2));
t10=C_fixnum_difference(t3,t2);
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_32425,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t9,a[5]=t10,a[6]=t5,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
/* library.scm:3251: ##sys#make-string */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[380]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[380]+1);
av2[1]=t11;
av2[2]=C_fixnum_times(t2,C_fix(2));
tp(3,av2);}}}
else{
if(C_truep(C_fixnum_lessp(t8,C_fix(0)))){
t9=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_32484,a[2]=t8,a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t1,a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[4],tmp=(C_word)a,a+=11,tmp);
/* library.scm:3255: ##sys#update-errno */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[703]);
C_word av2[2];
av2[0]=*((C_word*)lf[703]+1);
av2[1]=t9;
tp(2,av2);}}
else{
if(C_truep(t6)){
t9=C_slot(((C_word*)t0)[2],C_fix(4));
t10=C_fixnum_plus(t9,C_fix(1));
t11=C_i_set_i_slot(((C_word*)t0)[2],C_fix(4),t10);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32498,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:3268: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t12;
av2[2]=t4;
av2[3]=C_fix(0);
av2[4]=t8;
tp(5,av2);}}
else{
t9=C_slot(((C_word*)t0)[2],C_fix(4));
t10=C_fixnum_plus(t9,C_fix(1));
t11=C_i_set_i_slot(((C_word*)t0)[2],C_fix(4),t10);
/* library.scm:3271: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word av2[5];
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t1;
av2[2]=t4;
av2[3]=C_fix(0);
av2[4]=t8;
tp(5,av2);}}}}}}

/* k32408 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_32410(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32410,c,av);}
/* library.scm:3248: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k32423 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_32425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_32425,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_32429,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* library.scm:3252: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
tp(4,av2);}}

/* k32427 in k32423 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in ... */
static void C_ccall f_32429(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_32429,c,av);}
/* library.scm:3249: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_32377(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,C_SCHEME_TRUE);}

/* a32452 in k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in ... */
static void C_ccall f_32453(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_32453,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_32461,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_32465,a[2]=t2,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* library.scm:3261: ##sys#substring */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[399]+1));
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=*((C_word*)lf[399]+1);
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=C_fix(0);
av2[4]=((C_word*)t0)[7];
tp(5,av2);}}

/* k32459 in a32452 in k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_32461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_32461,c,av);}
/* library.scm:3259: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_32377(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,C_SCHEME_TRUE);}

/* k32463 in a32452 in k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in ... */
static void C_ccall f_32465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32465,c,av);}
/* library.scm:3260: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k32474 in k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in ... */
static void C_ccall f_32476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_32476,c,av);}
/* library.scm:3263: ##sys#signal-hook */
t2=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[699];
av2[3]=lf[708];
av2[4]=t1;
av2[5]=((C_word*)t0)[3];
av2[6]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k32478 in k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in ... */
static void C_ccall f_32480(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32480,c,av);}
/* library.scm:3265: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=lf[709];
av2[3]=t1;
tp(4,av2);}}

/* k32482 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_32484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_32484,c,av);}
a=C_alloc(11);
t2=C_fix((C_word)EINTR);
t3=C_eqp(t1,t2);
if(C_truep(t3)){
t4=C_fixnum_negate(((C_word*)t0)[2]);
t5=C_fixnum_difference(t4,C_fix(1));
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_32453,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t5,a[8]=((C_word)li557),tmp=(C_word)a,a+=9,tmp);
/* library.scm:3257: ##sys#dispatch-interrupt */
{C_proc tp=(C_proc)C_fast_retrieve_symbol_proc(lf[698]);
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[698]+1);
av2[1]=((C_word*)t0)[8];
av2[2]=t6;
tp(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_32476,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[10],tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_32480,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* ##sys#peek-c-string */
t6=*((C_word*)lf[702]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_mpointer(&a,(void*)strerror(errno));
av2[3]=C_fix(0);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* k32496 in loop in k32362 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 in ... */
static void C_ccall f_32498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_32498,c,av);}
/* library.scm:3268: ##sys#string-append */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[416]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[416]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
tp(4,av2);}}

/* k32528 in k32359 in a32356 in k18456 in k18453 in k18450 in k18447 in k18444 in k18441 in k18438 in k18435 in k18432 in k18429 in k18426 in k18423 in k18420 in k18417 in k18257 in k10477 in k8569 */
static void C_ccall f_32530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_32530,c,av);}
t2=((C_word*)t0)[2];
f_32364(t2,C_i_setslot(((C_word*)t0)[3],C_fix(12),t1));}

/* k8569 */
static void C_ccall f_8571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word t91;
C_word t92;
C_word t93;
C_word t94;
C_word t95;
C_word t96;
C_word t97;
C_word t98;
C_word t99;
C_word t100;
C_word t101;
C_word t102;
C_word t103;
C_word t104;
C_word t105;
C_word t106;
C_word t107;
C_word t108;
C_word t109;
C_word t110;
C_word t111;
C_word t112;
C_word t113;
C_word t114;
C_word t115;
C_word t116;
C_word t117;
C_word t118;
C_word t119;
C_word t120;
C_word t121;
C_word t122;
C_word t123;
C_word t124;
C_word t125;
C_word t126;
C_word t127;
C_word t128;
C_word t129;
C_word t130;
C_word t131;
C_word t132;
C_word t133;
C_word t134;
C_word t135;
C_word t136;
C_word t137;
C_word t138;
C_word t139;
C_word t140;
C_word t141;
C_word t142;
C_word t143;
C_word t144;
C_word t145;
C_word t146;
C_word t147;
C_word t148;
C_word t149;
C_word t150;
C_word t151;
C_word t152;
C_word t153;
C_word t154;
C_word t155;
C_word t156;
C_word t157;
C_word t158;
C_word t159;
C_word t160;
C_word t161;
C_word t162;
C_word t163;
C_word t164;
C_word t165;
C_word t166;
C_word t167;
C_word t168;
C_word t169;
C_word t170;
C_word t171;
C_word t172;
C_word t173;
C_word t174;
C_word t175;
C_word t176;
C_word t177;
C_word t178;
C_word t179;
C_word t180;
C_word t181;
C_word t182;
C_word t183;
C_word t184;
C_word t185;
C_word t186;
C_word t187;
C_word t188;
C_word t189;
C_word t190;
C_word t191;
C_word t192;
C_word t193;
C_word t194;
C_word t195;
C_word t196;
C_word t197;
C_word t198;
C_word t199;
C_word t200;
C_word t201;
C_word t202;
C_word t203;
C_word t204;
C_word t205;
C_word t206;
C_word t207;
C_word t208;
C_word t209;
C_word t210;
C_word t211;
C_word t212;
C_word t213;
C_word t214;
C_word t215;
C_word t216;
C_word t217;
C_word t218;
C_word t219;
C_word t220;
C_word t221;
C_word t222;
C_word t223;
C_word t224;
C_word t225;
C_word t226;
C_word t227;
C_word t228;
C_word t229;
C_word t230;
C_word t231;
C_word t232;
C_word t233;
C_word t234;
C_word t235;
C_word t236;
C_word t237;
C_word t238;
C_word t239;
C_word t240;
C_word t241;
C_word t242;
C_word t243;
C_word t244;
C_word t245;
C_word t246;
C_word t247;
C_word t248;
C_word t249;
C_word t250;
C_word t251;
C_word t252;
C_word t253;
C_word t254;
C_word t255;
C_word t256;
C_word t257;
C_word t258;
C_word t259;
C_word t260;
C_word t261;
C_word t262;
C_word t263;
C_word t264;
C_word t265;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(605,c,5)))){
C_save_and_reclaim((void *)f_8571,c,av);}
a=C_alloc(605);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=C_mutate((C_word*)lf[2]+1 /* (set! scheme#not ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8573,a[2]=((C_word)li0),tmp=(C_word)a,a+=3,tmp));
t5=C_mutate((C_word*)lf[3]+1 /* (set! scheme#boolean? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8576,a[2]=((C_word)li1),tmp=(C_word)a,a+=3,tmp));
t6=C_mutate((C_word*)lf[4]+1 /* (set! scheme#eq? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8579,a[2]=((C_word)li2),tmp=(C_word)a,a+=3,tmp));
t7=C_mutate((C_word*)lf[5]+1 /* (set! scheme#eqv? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8582,a[2]=((C_word)li3),tmp=(C_word)a,a+=3,tmp));
t8=C_mutate((C_word*)lf[6]+1 /* (set! scheme#equal? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8585,a[2]=((C_word)li4),tmp=(C_word)a,a+=3,tmp));
t9=C_mutate((C_word*)lf[7]+1 /* (set! scheme#pair? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8588,a[2]=((C_word)li5),tmp=(C_word)a,a+=3,tmp));
t10=C_mutate((C_word*)lf[8]+1 /* (set! scheme#cons ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8591,a[2]=((C_word)li6),tmp=(C_word)a,a+=3,tmp));
t11=C_mutate((C_word*)lf[9]+1 /* (set! scheme#car ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8594,a[2]=((C_word)li7),tmp=(C_word)a,a+=3,tmp));
t12=C_mutate((C_word*)lf[10]+1 /* (set! scheme#cdr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8597,a[2]=((C_word)li8),tmp=(C_word)a,a+=3,tmp));
t13=C_mutate((C_word*)lf[11]+1 /* (set! scheme#set-car! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8600,a[2]=((C_word)li9),tmp=(C_word)a,a+=3,tmp));
t14=C_mutate((C_word*)lf[12]+1 /* (set! scheme#set-cdr! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8603,a[2]=((C_word)li10),tmp=(C_word)a,a+=3,tmp));
t15=C_mutate((C_word*)lf[13]+1 /* (set! scheme#cadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8606,a[2]=((C_word)li11),tmp=(C_word)a,a+=3,tmp));
t16=C_mutate((C_word*)lf[14]+1 /* (set! scheme#caddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8609,a[2]=((C_word)li12),tmp=(C_word)a,a+=3,tmp));
t17=C_mutate((C_word*)lf[15]+1 /* (set! scheme#cadddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8612,a[2]=((C_word)li13),tmp=(C_word)a,a+=3,tmp));
t18=C_mutate((C_word*)lf[16]+1 /* (set! scheme#cddddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8615,a[2]=((C_word)li14),tmp=(C_word)a,a+=3,tmp));
t19=C_mutate((C_word*)lf[17]+1 /* (set! scheme#caar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8618,a[2]=((C_word)li15),tmp=(C_word)a,a+=3,tmp));
t20=C_mutate((C_word*)lf[18]+1 /* (set! scheme#cdar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8621,a[2]=((C_word)li16),tmp=(C_word)a,a+=3,tmp));
t21=C_mutate((C_word*)lf[19]+1 /* (set! scheme#cddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8624,a[2]=((C_word)li17),tmp=(C_word)a,a+=3,tmp));
t22=C_mutate((C_word*)lf[20]+1 /* (set! scheme#caaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8627,a[2]=((C_word)li18),tmp=(C_word)a,a+=3,tmp));
t23=C_mutate((C_word*)lf[21]+1 /* (set! scheme#caadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8641,a[2]=((C_word)li19),tmp=(C_word)a,a+=3,tmp));
t24=C_mutate((C_word*)lf[22]+1 /* (set! scheme#cadar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8648,a[2]=((C_word)li20),tmp=(C_word)a,a+=3,tmp));
t25=C_mutate((C_word*)lf[23]+1 /* (set! scheme#cdaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8655,a[2]=((C_word)li21),tmp=(C_word)a,a+=3,tmp));
t26=C_mutate((C_word*)lf[24]+1 /* (set! scheme#cdadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8669,a[2]=((C_word)li22),tmp=(C_word)a,a+=3,tmp));
t27=C_mutate((C_word*)lf[25]+1 /* (set! scheme#cddar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8676,a[2]=((C_word)li23),tmp=(C_word)a,a+=3,tmp));
t28=C_mutate((C_word*)lf[26]+1 /* (set! scheme#cdddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8690,a[2]=((C_word)li24),tmp=(C_word)a,a+=3,tmp));
t29=C_mutate((C_word*)lf[27]+1 /* (set! scheme#caaaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8704,a[2]=((C_word)li25),tmp=(C_word)a,a+=3,tmp));
t30=C_mutate((C_word*)lf[28]+1 /* (set! scheme#caaadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8722,a[2]=((C_word)li26),tmp=(C_word)a,a+=3,tmp));
t31=C_mutate((C_word*)lf[29]+1 /* (set! scheme#caadar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8733,a[2]=((C_word)li27),tmp=(C_word)a,a+=3,tmp));
t32=C_mutate((C_word*)lf[30]+1 /* (set! scheme#caaddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8744,a[2]=((C_word)li28),tmp=(C_word)a,a+=3,tmp));
t33=C_mutate((C_word*)lf[31]+1 /* (set! scheme#cadaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8751,a[2]=((C_word)li29),tmp=(C_word)a,a+=3,tmp));
t34=C_mutate((C_word*)lf[32]+1 /* (set! scheme#cadadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8762,a[2]=((C_word)li30),tmp=(C_word)a,a+=3,tmp));
t35=C_mutate((C_word*)lf[33]+1 /* (set! scheme#caddar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8766,a[2]=((C_word)li31),tmp=(C_word)a,a+=3,tmp));
t36=C_mutate((C_word*)lf[34]+1 /* (set! scheme#cdaaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8773,a[2]=((C_word)li32),tmp=(C_word)a,a+=3,tmp));
t37=C_mutate((C_word*)lf[35]+1 /* (set! scheme#cdaadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8791,a[2]=((C_word)li33),tmp=(C_word)a,a+=3,tmp));
t38=C_mutate((C_word*)lf[36]+1 /* (set! scheme#cdadar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8802,a[2]=((C_word)li34),tmp=(C_word)a,a+=3,tmp));
t39=C_mutate((C_word*)lf[37]+1 /* (set! scheme#cdaddr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8813,a[2]=((C_word)li35),tmp=(C_word)a,a+=3,tmp));
t40=C_mutate((C_word*)lf[38]+1 /* (set! scheme#cddaar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8820,a[2]=((C_word)li36),tmp=(C_word)a,a+=3,tmp));
t41=C_mutate((C_word*)lf[39]+1 /* (set! scheme#cddadr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8838,a[2]=((C_word)li37),tmp=(C_word)a,a+=3,tmp));
t42=C_mutate((C_word*)lf[40]+1 /* (set! scheme#cdddar ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8849,a[2]=((C_word)li38),tmp=(C_word)a,a+=3,tmp));
t43=C_mutate((C_word*)lf[41]+1 /* (set! scheme#null? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8867,a[2]=((C_word)li39),tmp=(C_word)a,a+=3,tmp));
t44=C_mutate((C_word*)lf[42]+1 /* (set! scheme#list ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8873,a[2]=((C_word)li40),tmp=(C_word)a,a+=3,tmp));
t45=C_mutate((C_word*)lf[43]+1 /* (set! scheme#length ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8876,a[2]=((C_word)li41),tmp=(C_word)a,a+=3,tmp));
t46=C_mutate((C_word*)lf[44]+1 /* (set! scheme#list-tail ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8879,a[2]=((C_word)li42),tmp=(C_word)a,a+=3,tmp));
t47=C_mutate((C_word*)lf[45]+1 /* (set! scheme#list-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8882,a[2]=((C_word)li43),tmp=(C_word)a,a+=3,tmp));
t48=C_mutate((C_word*)lf[46]+1 /* (set! scheme#reverse ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8885,a[2]=((C_word)li45),tmp=(C_word)a,a+=3,tmp));
t49=C_mutate((C_word*)lf[49]+1 /* (set! scheme#memq ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8924,a[2]=((C_word)li46),tmp=(C_word)a,a+=3,tmp));
t50=C_mutate((C_word*)lf[50]+1 /* (set! scheme#memv ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8927,a[2]=((C_word)li47),tmp=(C_word)a,a+=3,tmp));
t51=C_mutate((C_word*)lf[51]+1 /* (set! scheme#member ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8930,a[2]=((C_word)li48),tmp=(C_word)a,a+=3,tmp));
t52=C_mutate((C_word*)lf[52]+1 /* (set! scheme#assq ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8933,a[2]=((C_word)li49),tmp=(C_word)a,a+=3,tmp));
t53=C_mutate((C_word*)lf[53]+1 /* (set! scheme#assv ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8936,a[2]=((C_word)li50),tmp=(C_word)a,a+=3,tmp));
t54=C_mutate((C_word*)lf[54]+1 /* (set! scheme#assoc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8939,a[2]=((C_word)li51),tmp=(C_word)a,a+=3,tmp));
t55=C_mutate((C_word*)lf[55]+1 /* (set! scheme#list? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8942,a[2]=((C_word)li52),tmp=(C_word)a,a+=3,tmp));
t56=C_mutate((C_word*)lf[56]+1 /* (set! scheme#string? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8945,a[2]=((C_word)li53),tmp=(C_word)a,a+=3,tmp));
t57=C_mutate((C_word*)lf[57]+1 /* (set! scheme#string-length ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8948,a[2]=((C_word)li54),tmp=(C_word)a,a+=3,tmp));
t58=C_mutate((C_word*)lf[58]+1 /* (set! scheme#string-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8951,a[2]=((C_word)li55),tmp=(C_word)a,a+=3,tmp));
t59=C_mutate((C_word*)lf[59]+1 /* (set! scheme#string-set! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8954,a[2]=((C_word)li56),tmp=(C_word)a,a+=3,tmp));
t60=C_mutate((C_word*)lf[60]+1 /* (set! scheme#string=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8957,a[2]=((C_word)li57),tmp=(C_word)a,a+=3,tmp));
t61=C_mutate((C_word*)lf[61]+1 /* (set! scheme#string-ci=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8960,a[2]=((C_word)li58),tmp=(C_word)a,a+=3,tmp));
t62=C_mutate((C_word*)lf[62]+1 /* (set! scheme#exact->inexact ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8964,a[2]=((C_word)li59),tmp=(C_word)a,a+=3,tmp));
t63=C_mutate((C_word*)lf[63]+1 /* (set! scheme#abs ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8967,a[2]=((C_word)li60),tmp=(C_word)a,a+=3,tmp));
t64=C_mutate((C_word*)lf[64]+1 /* (set! scheme#+ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_plus,a[2]=((C_word)li61),tmp=(C_word)a,a+=3,tmp));
t65=C_mutate((C_word*)lf[65]+1 /* (set! scheme#- ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_minus,a[2]=((C_word)li62),tmp=(C_word)a,a+=3,tmp));
t66=C_mutate((C_word*)lf[66]+1 /* (set! scheme#* ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_times,a[2]=((C_word)li63),tmp=(C_word)a,a+=3,tmp));
t67=C_mutate((C_word*)lf[67]+1 /* (set! scheme#quotient ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8973,a[2]=((C_word)li64),tmp=(C_word)a,a+=3,tmp));
t68=C_mutate((C_word*)lf[68]+1 /* (set! scheme#remainder ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8976,a[2]=((C_word)li65),tmp=(C_word)a,a+=3,tmp));
t69=C_mutate((C_word*)lf[69]+1 /* (set! scheme#modulo ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8979,a[2]=((C_word)li66),tmp=(C_word)a,a+=3,tmp));
t70=C_mutate((C_word*)lf[70]+1 /* (set! scheme#even? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8982,a[2]=((C_word)li67),tmp=(C_word)a,a+=3,tmp));
t71=C_mutate((C_word*)lf[71]+1 /* (set! scheme#odd? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8985,a[2]=((C_word)li68),tmp=(C_word)a,a+=3,tmp));
t72=C_mutate((C_word*)lf[72]+1 /* (set! scheme#= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_nequalp,a[2]=((C_word)li69),tmp=(C_word)a,a+=3,tmp));
t73=C_mutate((C_word*)lf[73]+1 /* (set! scheme#> ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_greaterp,a[2]=((C_word)li70),tmp=(C_word)a,a+=3,tmp));
t74=C_mutate((C_word*)lf[74]+1 /* (set! scheme#< ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_lessp,a[2]=((C_word)li71),tmp=(C_word)a,a+=3,tmp));
t75=C_mutate((C_word*)lf[75]+1 /* (set! scheme#>= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_greater_or_equal_p,a[2]=((C_word)li72),tmp=(C_word)a,a+=3,tmp));
t76=C_mutate((C_word*)lf[76]+1 /* (set! scheme#<= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_less_or_equal_p,a[2]=((C_word)li73),tmp=(C_word)a,a+=3,tmp));
t77=C_mutate((C_word*)lf[77]+1 /* (set! scheme#number? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8993,a[2]=((C_word)li74),tmp=(C_word)a,a+=3,tmp));
t78=C_mutate((C_word*)lf[78]+1 /* (set! scheme#complex? ...) */,*((C_word*)lf[77]+1));
t79=C_mutate((C_word*)lf[79]+1 /* (set! scheme#real? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8997,a[2]=((C_word)li75),tmp=(C_word)a,a+=3,tmp));
t80=C_mutate((C_word*)lf[80]+1 /* (set! scheme#rational? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9000,a[2]=((C_word)li76),tmp=(C_word)a,a+=3,tmp));
t81=C_mutate((C_word*)lf[81]+1 /* (set! scheme#integer? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9003,a[2]=((C_word)li77),tmp=(C_word)a,a+=3,tmp));
t82=C_mutate((C_word*)lf[82]+1 /* (set! scheme#exact? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9006,a[2]=((C_word)li78),tmp=(C_word)a,a+=3,tmp));
t83=C_mutate((C_word*)lf[83]+1 /* (set! scheme#inexact? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9009,a[2]=((C_word)li79),tmp=(C_word)a,a+=3,tmp));
t84=C_mutate((C_word*)lf[84]+1 /* (set! scheme#zero? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9012,a[2]=((C_word)li80),tmp=(C_word)a,a+=3,tmp));
t85=C_mutate((C_word*)lf[85]+1 /* (set! scheme#positive? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9015,a[2]=((C_word)li81),tmp=(C_word)a,a+=3,tmp));
t86=C_mutate((C_word*)lf[86]+1 /* (set! scheme#negative? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9018,a[2]=((C_word)li82),tmp=(C_word)a,a+=3,tmp));
t87=C_mutate((C_word*)lf[87]+1 /* (set! scheme#number->string ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_number_to_string,a[2]=((C_word)li83),tmp=(C_word)a,a+=3,tmp));
t88=C_mutate((C_word*)lf[88]+1 /* (set! scheme#symbol? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9022,a[2]=((C_word)li84),tmp=(C_word)a,a+=3,tmp));
t89=C_mutate((C_word*)lf[89]+1 /* (set! scheme#vector? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9025,a[2]=((C_word)li85),tmp=(C_word)a,a+=3,tmp));
t90=C_mutate((C_word*)lf[90]+1 /* (set! scheme#vector-length ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9028,a[2]=((C_word)li86),tmp=(C_word)a,a+=3,tmp));
t91=C_mutate((C_word*)lf[91]+1 /* (set! scheme#vector-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9031,a[2]=((C_word)li87),tmp=(C_word)a,a+=3,tmp));
t92=C_mutate((C_word*)lf[92]+1 /* (set! scheme#vector-set! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9034,a[2]=((C_word)li88),tmp=(C_word)a,a+=3,tmp));
t93=C_set_block_item(lf[93] /* scheme#make-vector */,0,C_SCHEME_UNDEFINED);
t94=C_set_block_item(lf[94] /* scheme#list->vector */,0,C_SCHEME_UNDEFINED);
t95=C_set_block_item(lf[95] /* scheme#vector->list */,0,C_SCHEME_UNDEFINED);
t96=C_set_block_item(lf[96] /* scheme#vector */,0,C_SCHEME_UNDEFINED);
t97=C_set_block_item(lf[97] /* scheme#vector-fill! */,0,C_SCHEME_UNDEFINED);
t98=C_mutate((C_word*)lf[98]+1 /* (set! scheme#char? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9042,a[2]=((C_word)li89),tmp=(C_word)a,a+=3,tmp));
t99=C_mutate((C_word*)lf[99]+1 /* (set! scheme#char->integer ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9045,a[2]=((C_word)li90),tmp=(C_word)a,a+=3,tmp));
t100=C_mutate((C_word*)lf[101]+1 /* (set! scheme#integer->char ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9052,a[2]=((C_word)li91),tmp=(C_word)a,a+=3,tmp));
t101=C_mutate((C_word*)lf[104]+1 /* (set! scheme#char=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9059,a[2]=((C_word)li92),tmp=(C_word)a,a+=3,tmp));
t102=C_mutate((C_word*)lf[105]+1 /* (set! scheme#char>? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9062,a[2]=((C_word)li93),tmp=(C_word)a,a+=3,tmp));
t103=C_mutate((C_word*)lf[106]+1 /* (set! scheme#char<? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9065,a[2]=((C_word)li94),tmp=(C_word)a,a+=3,tmp));
t104=C_mutate((C_word*)lf[107]+1 /* (set! scheme#char>=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9068,a[2]=((C_word)li95),tmp=(C_word)a,a+=3,tmp));
t105=C_mutate((C_word*)lf[108]+1 /* (set! scheme#char<=? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9071,a[2]=((C_word)li96),tmp=(C_word)a,a+=3,tmp));
t106=C_mutate((C_word*)lf[109]+1 /* (set! scheme#char-upcase ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9074,a[2]=((C_word)li97),tmp=(C_word)a,a+=3,tmp));
t107=C_mutate((C_word*)lf[111]+1 /* (set! scheme#char-downcase ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9080,a[2]=((C_word)li98),tmp=(C_word)a,a+=3,tmp));
t108=C_set_block_item(lf[113] /* scheme#char-ci=? */,0,C_SCHEME_UNDEFINED);
t109=C_set_block_item(lf[114] /* scheme#char-ci>? */,0,C_SCHEME_UNDEFINED);
t110=C_set_block_item(lf[115] /* scheme#char-ci<? */,0,C_SCHEME_UNDEFINED);
t111=C_set_block_item(lf[116] /* scheme#char-ci>=? */,0,C_SCHEME_UNDEFINED);
t112=C_set_block_item(lf[117] /* scheme#char-ci<=? */,0,C_SCHEME_UNDEFINED);
t113=C_mutate((C_word*)lf[118]+1 /* (set! scheme#char-upper-case? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9091,a[2]=((C_word)li99),tmp=(C_word)a,a+=3,tmp));
t114=C_mutate((C_word*)lf[120]+1 /* (set! scheme#char-lower-case? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9097,a[2]=((C_word)li100),tmp=(C_word)a,a+=3,tmp));
t115=C_mutate((C_word*)lf[122]+1 /* (set! scheme#char-numeric? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9103,a[2]=((C_word)li101),tmp=(C_word)a,a+=3,tmp));
t116=C_mutate((C_word*)lf[124]+1 /* (set! scheme#char-whitespace? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9109,a[2]=((C_word)li102),tmp=(C_word)a,a+=3,tmp));
t117=C_mutate((C_word*)lf[126]+1 /* (set! scheme#char-alphabetic? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9115,a[2]=((C_word)li103),tmp=(C_word)a,a+=3,tmp));
t118=C_mutate((C_word*)lf[128]+1 /* (set! scheme#procedure? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9121,a[2]=((C_word)li104),tmp=(C_word)a,a+=3,tmp));
t119=C_mutate((C_word*)lf[129]+1 /* (set! scheme#apply ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_apply,a[2]=((C_word)li105),tmp=(C_word)a,a+=3,tmp));
t120=C_mutate((C_word*)lf[130]+1 /* (set! scheme#values ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_values,a[2]=((C_word)li106),tmp=(C_word)a,a+=3,tmp));
t121=C_mutate((C_word*)lf[131]+1 /* (set! scheme#call-with-values ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_call_with_values,a[2]=((C_word)li107),tmp=(C_word)a,a+=3,tmp));
t122=C_set_block_item(lf[132] /* scheme#call-with-current-continuation */,0,C_SCHEME_UNDEFINED);
t123=C_mutate((C_word*)lf[133]+1 /* (set! scheme#input-port? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9128,a[2]=((C_word)li108),tmp=(C_word)a,a+=3,tmp));
t124=C_mutate((C_word*)lf[134]+1 /* (set! scheme#output-port? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9134,a[2]=((C_word)li109),tmp=(C_word)a,a+=3,tmp));
t125=C_set_block_item(lf[135] /* scheme#current-input-port */,0,C_SCHEME_UNDEFINED);
t126=C_set_block_item(lf[136] /* scheme#current-output-port */,0,C_SCHEME_UNDEFINED);
t127=C_set_block_item(lf[137] /* scheme#open-input-file */,0,C_SCHEME_UNDEFINED);
t128=C_set_block_item(lf[138] /* scheme#open-output-file */,0,C_SCHEME_UNDEFINED);
t129=C_set_block_item(lf[139] /* scheme#close-input-port */,0,C_SCHEME_UNDEFINED);
t130=C_set_block_item(lf[140] /* scheme#close-output-port */,0,C_SCHEME_UNDEFINED);
t131=C_set_block_item(lf[141] /* scheme#call-with-input-file */,0,C_SCHEME_UNDEFINED);
t132=C_set_block_item(lf[142] /* scheme#call-with-output-file */,0,C_SCHEME_UNDEFINED);
t133=C_set_block_item(lf[143] /* scheme#with-input-from-file */,0,C_SCHEME_UNDEFINED);
t134=C_set_block_item(lf[144] /* scheme#with-output-to-file */,0,C_SCHEME_UNDEFINED);
t135=C_mutate((C_word*)lf[145]+1 /* (set! scheme#eof-object? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9150,a[2]=((C_word)li110),tmp=(C_word)a,a+=3,tmp));
t136=C_set_block_item(lf[146] /* scheme#char-ready? */,0,C_SCHEME_UNDEFINED);
t137=C_set_block_item(lf[147] /* scheme#read-char */,0,C_SCHEME_UNDEFINED);
t138=C_set_block_item(lf[148] /* scheme#peek-char */,0,C_SCHEME_UNDEFINED);
t139=C_set_block_item(lf[149] /* scheme#read */,0,C_SCHEME_UNDEFINED);
t140=C_set_block_item(lf[150] /* scheme#write-char */,0,C_SCHEME_UNDEFINED);
t141=C_set_block_item(lf[151] /* scheme#newline */,0,C_SCHEME_UNDEFINED);
t142=C_set_block_item(lf[152] /* scheme#write */,0,C_SCHEME_UNDEFINED);
t143=C_set_block_item(lf[153] /* scheme#display */,0,C_SCHEME_UNDEFINED);
t144=C_mutate((C_word*)lf[154]+1 /* (set! scheme#eval ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9161,a[2]=((C_word)li111),tmp=(C_word)a,a+=3,tmp));
t145=C_mutate((C_word*)lf[158]+1 /* (set! scheme#interaction-environment ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9167,a[2]=((C_word)li112),tmp=(C_word)a,a+=3,tmp));
t146=C_mutate((C_word*)lf[161]+1 /* (set! scheme#scheme-report-environment ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9173,a[2]=((C_word)li113),tmp=(C_word)a,a+=3,tmp));
t147=C_mutate((C_word*)lf[164]+1 /* (set! scheme#null-environment ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9179,a[2]=((C_word)li114),tmp=(C_word)a,a+=3,tmp));
t148=C_mutate((C_word*)lf[167]+1 /* (set! scheme#load ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9185,a[2]=((C_word)li115),tmp=(C_word)a,a+=3,tmp));
t149=C_set_block_item(lf[170] /* scheme#for-each */,0,C_SCHEME_UNDEFINED);
t150=C_set_block_item(lf[171] /* scheme#map */,0,C_SCHEME_UNDEFINED);
t151=C_set_block_item(lf[172] /* scheme#dynamic-wind */,0,C_SCHEME_UNDEFINED);
t152=C_a_i_provide(&a,1,lf[173]);
t153=C_mutate((C_word*)lf[174]+1 /* (set! chicken.base#fixnum? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9194,a[2]=((C_word)li116),tmp=(C_word)a,a+=3,tmp));
t154=C_mutate((C_word*)lf[175]+1 /* (set! chicken.base#flonum? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9197,a[2]=((C_word)li117),tmp=(C_word)a,a+=3,tmp));
t155=C_mutate((C_word*)lf[176]+1 /* (set! chicken.base#bignum? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9200,a[2]=((C_word)li118),tmp=(C_word)a,a+=3,tmp));
t156=C_mutate((C_word*)lf[177]+1 /* (set! chicken.base#ratnum? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9203,a[2]=((C_word)li119),tmp=(C_word)a,a+=3,tmp));
t157=C_mutate((C_word*)lf[178]+1 /* (set! chicken.base#cplxnum? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9206,a[2]=((C_word)li120),tmp=(C_word)a,a+=3,tmp));
t158=C_mutate((C_word*)lf[179]+1 /* (set! chicken.base#exact-integer? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9209,a[2]=((C_word)li121),tmp=(C_word)a,a+=3,tmp));
t159=C_mutate((C_word*)lf[180]+1 /* (set! chicken.base#quotient&remainder ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_quotient_and_remainder,a[2]=((C_word)li122),tmp=(C_word)a,a+=3,tmp));
t160=C_mutate((C_word*)lf[181]+1 /* (set! chicken.base#quotient&modulo ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9213,a[2]=((C_word)li125),tmp=(C_word)a,a+=3,tmp));
t161=C_mutate((C_word*)lf[182]+1 /* (set! chicken.base#finite? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9260,a[2]=((C_word)li126),tmp=(C_word)a,a+=3,tmp));
t162=C_mutate((C_word*)lf[183]+1 /* (set! chicken.base#infinite? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9263,a[2]=((C_word)li127),tmp=(C_word)a,a+=3,tmp));
t163=C_mutate((C_word*)lf[184]+1 /* (set! chicken.base#nan? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9266,a[2]=((C_word)li128),tmp=(C_word)a,a+=3,tmp));
t164=C_mutate((C_word*)lf[185]+1 /* (set! chicken.base#signum ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)C_signum,a[2]=((C_word)li129),tmp=(C_word)a,a+=3,tmp));
t165=C_set_block_item(lf[186] /* chicken.base#get-call-chain */,0,C_SCHEME_UNDEFINED);
t166=C_set_block_item(lf[187] /* chicken.base#print-call-chain */,0,C_SCHEME_UNDEFINED);
t167=C_set_block_item(lf[188] /* chicken.base#print */,0,C_SCHEME_UNDEFINED);
t168=C_set_block_item(lf[189] /* chicken.base#print* */,0,C_SCHEME_UNDEFINED);
t169=C_mutate((C_word*)lf[190]+1 /* (set! chicken.base#add1 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9274,a[2]=((C_word)li130),tmp=(C_word)a,a+=3,tmp));
t170=C_mutate((C_word*)lf[191]+1 /* (set! chicken.base#sub1 ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9280,a[2]=((C_word)li131),tmp=(C_word)a,a+=3,tmp));
t171=C_set_block_item(lf[192] /* chicken.base#current-error-port */,0,C_SCHEME_UNDEFINED);
t172=C_mutate((C_word*)lf[193]+1 /* (set! chicken.base#error ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9287,a[2]=((C_word)li132),tmp=(C_word)a,a+=3,tmp));
t173=C_mutate((C_word*)lf[196]+1 /* (set! chicken.base#void ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9302,a[2]=((C_word)li133),tmp=(C_word)a,a+=3,tmp));
t174=C_set_block_item(lf[197] /* chicken.base#sleep */,0,C_SCHEME_UNDEFINED);
t175=C_set_block_item(lf[198] /* chicken.base#call/cc */,0,C_SCHEME_UNDEFINED);
t176=C_set_block_item(lf[199] /* chicken.base#char-name */,0,C_SCHEME_UNDEFINED);
t177=C_set_block_item(lf[200] /* chicken.base#getter-with-setter */,0,C_SCHEME_UNDEFINED);
t178=C_set_block_item(lf[201] /* chicken.base#make-parameter */,0,C_SCHEME_UNDEFINED);
t179=C_set_block_item(lf[202] /* chicken.base#procedure-information */,0,C_SCHEME_UNDEFINED);
t180=C_set_block_item(lf[203] /* chicken.base#setter */,0,C_SCHEME_UNDEFINED);
t181=C_set_block_item(lf[204] /* chicken.base#record-printer */,0,C_SCHEME_UNDEFINED);
t182=C_set_block_item(lf[205] /* chicken.base#set-record-printer! */,0,C_SCHEME_UNDEFINED);
t183=C_set_block_item(lf[206] /* chicken.base#vector-copy! */,0,C_SCHEME_UNDEFINED);
t184=C_set_block_item(lf[207] /* chicken.base#subvector */,0,C_SCHEME_UNDEFINED);
t185=C_set_block_item(lf[208] /* chicken.base#vector-resize */,0,C_SCHEME_UNDEFINED);
t186=C_set_block_item(lf[209] /* chicken.base#port? */,0,C_SCHEME_UNDEFINED);
t187=C_set_block_item(lf[210] /* chicken.base#port-closed? */,0,C_SCHEME_UNDEFINED);
t188=C_set_block_item(lf[211] /* chicken.base#input-port-open? */,0,C_SCHEME_UNDEFINED);
t189=C_set_block_item(lf[212] /* chicken.base#output-port-open? */,0,C_SCHEME_UNDEFINED);
t190=C_set_block_item(lf[213] /* chicken.base#get-output-string */,0,C_SCHEME_UNDEFINED);
t191=C_set_block_item(lf[214] /* chicken.base#open-input-string */,0,C_SCHEME_UNDEFINED);
t192=C_set_block_item(lf[215] /* chicken.base#open-output-string */,0,C_SCHEME_UNDEFINED);
t193=C_set_block_item(lf[216] /* chicken.base#flush-output */,0,C_SCHEME_UNDEFINED);
t194=C_mutate((C_word*)lf[217]+1 /* (set! chicken.base#promise? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9325,a[2]=((C_word)li134),tmp=(C_word)a,a+=3,tmp));
t195=C_mutate((C_word*)lf[219]+1 /* (set! ##sys#make-promise ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9331,a[2]=((C_word)li135),tmp=(C_word)a,a+=3,tmp));
t196=C_mutate((C_word*)lf[220]+1 /* (set! chicken.base#make-promise ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9337,a[2]=((C_word)li137),tmp=(C_word)a,a+=3,tmp));
t197=C_mutate((C_word*)lf[221]+1 /* (set! chicken.base#foldl ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9352,a[2]=((C_word)li139),tmp=(C_word)a,a+=3,tmp));
t198=C_mutate((C_word*)lf[223]+1 /* (set! chicken.base#foldr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9389,a[2]=((C_word)li141),tmp=(C_word)a,a+=3,tmp));
t199=C_set_block_item(lf[225] /* chicken.base#implicit-exit-handler */,0,C_SCHEME_UNDEFINED);
t200=C_set_block_item(lf[226] /* chicken.base#exit-handler */,0,C_SCHEME_UNDEFINED);
t201=C_set_block_item(lf[227] /* chicken.base#cleanup-tasks */,0,C_SCHEME_END_OF_LIST);
t202=C_mutate((C_word*)lf[228]+1 /* (set! chicken.base#on-exit ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9429,a[2]=((C_word)li142),tmp=(C_word)a,a+=3,tmp));
t203=C_mutate((C_word*)lf[229]+1 /* (set! chicken.base#exit ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9436,a[2]=((C_word)li143),tmp=(C_word)a,a+=3,tmp));
t204=C_mutate((C_word*)lf[230]+1 /* (set! chicken.base#emergency-exit ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9454,a[2]=((C_word)li144),tmp=(C_word)a,a+=3,tmp));
t205=C_set_block_item(lf[232] /* chicken.base#case-sensitive */,0,C_SCHEME_UNDEFINED);
t206=C_set_block_item(lf[233] /* chicken.base#keyword-style */,0,C_SCHEME_UNDEFINED);
t207=C_set_block_item(lf[234] /* chicken.base#parentheses-synonyms */,0,C_SCHEME_UNDEFINED);
t208=C_set_block_item(lf[235] /* chicken.base#symbol-escape */,0,C_SCHEME_UNDEFINED);
t209=C_mutate((C_word*)lf[236]+1 /* (set! chicken.base#identity ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9473,a[2]=((C_word)li145),tmp=(C_word)a,a+=3,tmp));
t210=C_mutate((C_word*)lf[237]+1 /* (set! chicken.base#conjoin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9476,a[2]=((C_word)li148),tmp=(C_word)a,a+=3,tmp));
t211=C_mutate((C_word*)lf[238]+1 /* (set! chicken.base#disjoin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9509,a[2]=((C_word)li151),tmp=(C_word)a,a+=3,tmp));
t212=C_mutate((C_word*)lf[239]+1 /* (set! chicken.base#constantly ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9546,a[2]=((C_word)li154),tmp=(C_word)a,a+=3,tmp));
t213=C_mutate((C_word*)lf[240]+1 /* (set! chicken.base#flip ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9567,a[2]=((C_word)li156),tmp=(C_word)a,a+=3,tmp));
t214=C_mutate((C_word*)lf[241]+1 /* (set! chicken.base#complement ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9575,a[2]=((C_word)li158),tmp=(C_word)a,a+=3,tmp));
t215=C_mutate((C_word*)lf[242]+1 /* (set! chicken.base#compose ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9587,a[2]=((C_word)li162),tmp=(C_word)a,a+=3,tmp));
t216=C_mutate((C_word*)lf[243]+1 /* (set! chicken.base#o ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9623,a[2]=((C_word)li165),tmp=(C_word)a,a+=3,tmp));
t217=C_mutate((C_word*)lf[244]+1 /* (set! chicken.base#list-of? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9662,a[2]=((C_word)li168),tmp=(C_word)a,a+=3,tmp));
t218=C_mutate((C_word*)lf[245]+1 /* (set! chicken.base#each ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9706,a[2]=((C_word)li172),tmp=(C_word)a,a+=3,tmp));
t219=C_mutate((C_word*)lf[247]+1 /* (set! chicken.base#atom? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9762,a[2]=((C_word)li173),tmp=(C_word)a,a+=3,tmp));
t220=C_mutate((C_word*)lf[248]+1 /* (set! chicken.base#tail? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9765,a[2]=((C_word)li175),tmp=(C_word)a,a+=3,tmp));
t221=C_mutate((C_word*)lf[250]+1 /* (set! chicken.base#intersperse ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9793,a[2]=((C_word)li177),tmp=(C_word)a,a+=3,tmp));
t222=C_mutate((C_word*)lf[251]+1 /* (set! chicken.base#butlast ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9826,a[2]=((C_word)li179),tmp=(C_word)a,a+=3,tmp));
t223=C_mutate((C_word*)lf[253]+1 /* (set! chicken.base#flatten ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9858,a[2]=((C_word)li181),tmp=(C_word)a,a+=3,tmp));
t224=C_mutate((C_word*)lf[254]+1 /* (set! chicken.base#join ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9899,a[2]=((C_word)li183),tmp=(C_word)a,a+=3,tmp));
t225=C_mutate((C_word*)lf[257]+1 /* (set! chicken.base#compress ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9956,a[2]=((C_word)li185),tmp=(C_word)a,a+=3,tmp));
t226=C_mutate((C_word*)lf[261]+1 /* (set! chicken.base#alist-update! ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10036,a[2]=((C_word)li188),tmp=(C_word)a,a+=3,tmp));
t227=C_mutate((C_word*)lf[262]+1 /* (set! chicken.base#alist-update ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10122,a[2]=((C_word)li190),tmp=(C_word)a,a+=3,tmp));
t228=C_mutate((C_word*)lf[266]+1 /* (set! chicken.base#alist-ref ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10226,a[2]=((C_word)li193),tmp=(C_word)a,a+=3,tmp));
t229=C_mutate((C_word*)lf[269]+1 /* (set! chicken.base#rassoc ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10338,a[2]=((C_word)li195),tmp=(C_word)a,a+=3,tmp));
t230=C_mutate(&lf[271] /* (set! maximal-string-length ...) */,C_unsigned_long_to_num(&a,C_HEADER_SIZE_MASK));
t231=C_a_i_provide(&a,1,lf[272]);
t232=C_set_block_item(lf[273] /* chicken.fixnum#most-positive-fixnum */,0,C_fix((C_word)C_MOST_POSITIVE_FIXNUM));
t233=C_set_block_item(lf[274] /* chicken.fixnum#most-negative-fixnum */,0,C_fix((C_word)C_MOST_NEGATIVE_FIXNUM));
t234=C_set_block_item(lf[275] /* chicken.fixnum#fixnum-bits */,0,C_fix((C_word)(C_WORD_SIZE - 1)));
t235=C_set_block_item(lf[276] /* chicken.fixnum#fixnum-precision */,0,C_fix((C_word)(C_WORD_SIZE - (1 + 1))));
t236=C_mutate((C_word*)lf[277]+1 /* (set! chicken.fixnum#fx+ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10391,a[2]=((C_word)li196),tmp=(C_word)a,a+=3,tmp));
t237=C_mutate((C_word*)lf[278]+1 /* (set! chicken.fixnum#fx- ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10394,a[2]=((C_word)li197),tmp=(C_word)a,a+=3,tmp));
t238=C_mutate((C_word*)lf[279]+1 /* (set! chicken.fixnum#fx* ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10397,a[2]=((C_word)li198),tmp=(C_word)a,a+=3,tmp));
t239=C_mutate((C_word*)lf[280]+1 /* (set! chicken.fixnum#fx= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10400,a[2]=((C_word)li199),tmp=(C_word)a,a+=3,tmp));
t240=C_mutate((C_word*)lf[281]+1 /* (set! chicken.fixnum#fx> ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10406,a[2]=((C_word)li200),tmp=(C_word)a,a+=3,tmp));
t241=C_mutate((C_word*)lf[282]+1 /* (set! chicken.fixnum#fx< ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10409,a[2]=((C_word)li201),tmp=(C_word)a,a+=3,tmp));
t242=C_mutate((C_word*)lf[283]+1 /* (set! chicken.fixnum#fx>= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10412,a[2]=((C_word)li202),tmp=(C_word)a,a+=3,tmp));
t243=C_mutate((C_word*)lf[284]+1 /* (set! chicken.fixnum#fx<= ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10415,a[2]=((C_word)li203),tmp=(C_word)a,a+=3,tmp));
t244=C_mutate((C_word*)lf[285]+1 /* (set! chicken.fixnum#fxmin ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10418,a[2]=((C_word)li204),tmp=(C_word)a,a+=3,tmp));
t245=C_mutate((C_word*)lf[286]+1 /* (set! chicken.fixnum#fxmax ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10421,a[2]=((C_word)li205),tmp=(C_word)a,a+=3,tmp));
t246=C_mutate((C_word*)lf[287]+1 /* (set! chicken.fixnum#fxneg ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10424,a[2]=((C_word)li206),tmp=(C_word)a,a+=3,tmp));
t247=C_mutate((C_word*)lf[288]+1 /* (set! chicken.fixnum#fxand ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10427,a[2]=((C_word)li207),tmp=(C_word)a,a+=3,tmp));
t248=C_mutate((C_word*)lf[289]+1 /* (set! chicken.fixnum#fxior ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10430,a[2]=((C_word)li208),tmp=(C_word)a,a+=3,tmp));
t249=C_mutate((C_word*)lf[290]+1 /* (set! chicken.fixnum#fxxor ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10433,a[2]=((C_word)li209),tmp=(C_word)a,a+=3,tmp));
t250=C_mutate((C_word*)lf[291]+1 /* (set! chicken.fixnum#fxnot ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10436,a[2]=((C_word)li210),tmp=(C_word)a,a+=3,tmp));
t251=C_mutate((C_word*)lf[292]+1 /* (set! chicken.fixnum#fxshl ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10439,a[2]=((C_word)li211),tmp=(C_word)a,a+=3,tmp));
t252=C_mutate((C_word*)lf[293]+1 /* (set! chicken.fixnum#fxshr ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10442,a[2]=((C_word)li212),tmp=(C_word)a,a+=3,tmp));
t253=C_mutate((C_word*)lf[294]+1 /* (set! chicken.fixnum#fxodd? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10445,a[2]=((C_word)li213),tmp=(C_word)a,a+=3,tmp));
t254=C_mutate((C_word*)lf[295]+1 /* (set! chicken.fixnum#fxeven? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10448,a[2]=((C_word)li214),tmp=(C_word)a,a+=3,tmp));
t255=C_mutate((C_word*)lf[296]+1 /* (set! chicken.fixnum#fxlen ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10451,a[2]=((C_word)li215),tmp=(C_word)a,a+=3,tmp));
t256=C_mutate((C_word*)lf[297]+1 /* (set! chicken.fixnum#fx/ ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10454,a[2]=((C_word)li216),tmp=(C_word)a,a+=3,tmp));
t257=C_mutate((C_word*)lf[298]+1 /* (set! chicken.fixnum#fxgcd ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10457,a[2]=((C_word)li217),tmp=(C_word)a,a+=3,tmp));
t258=C_mutate((C_word*)lf[299]+1 /* (set! chicken.fixnum#fxmod ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10460,a[2]=((C_word)li218),tmp=(C_word)a,a+=3,tmp));
t259=C_mutate((C_word*)lf[300]+1 /* (set! chicken.fixnum#fxrem ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10463,a[2]=((C_word)li219),tmp=(C_word)a,a+=3,tmp));
t260=C_mutate((C_word*)lf[301]+1 /* (set! chicken.fixnum#fx+? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10466,a[2]=((C_word)li220),tmp=(C_word)a,a+=3,tmp));
t261=C_mutate((C_word*)lf[302]+1 /* (set! chicken.fixnum#fx-? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10469,a[2]=((C_word)li221),tmp=(C_word)a,a+=3,tmp));
t262=C_mutate((C_word*)lf[303]+1 /* (set! chicken.fixnum#fx*? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10472,a[2]=((C_word)li222),tmp=(C_word)a,a+=3,tmp));
t263=C_mutate((C_word*)lf[304]+1 /* (set! chicken.fixnum#fx/? ...) */,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10475,a[2]=((C_word)li223),tmp=(C_word)a,a+=3,tmp));
t264=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10479,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t264;
C_library_toplevel(2,av2);}}

/* scheme#not in k8569 */
static void C_ccall f_8573(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8573,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_not(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#boolean? in k8569 */
static void C_ccall f_8576(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8576,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_booleanp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#eq? in k8569 */
static void C_ccall f_8579(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8579,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_eqp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#eqv? in k8569 */
static void C_ccall f_8582(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8582,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_eqvp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#equal? in k8569 */
static void C_ccall f_8585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8585,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_equalp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#pair? in k8569 */
static void C_ccall f_8588(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8588,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_pairp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cons in k8569 */
static void C_ccall f_8591(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_8591,c,av);}
a=C_alloc(3);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#car in k8569 */
static void C_ccall f_8594(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8594,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdr in k8569 */
static void C_ccall f_8597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8597,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#set-car! in k8569 */
static void C_ccall f_8600(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8600,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#set-cdr! in k8569 */
static void C_ccall f_8603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8603,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_cdr(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#cadr in k8569 */
static void C_ccall f_8606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8606,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cadr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#caddr in k8569 */
static void C_ccall f_8609(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8609,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_caddr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cadddr in k8569 */
static void C_ccall f_8612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8612,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cadddr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cddddr in k8569 */
static void C_ccall f_8615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8615,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cddddr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#caar in k8569 */
static void C_ccall f_8618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8618,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_caar(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdar in k8569 */
static void C_ccall f_8621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8621,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdar(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cddr in k8569 */
static void C_ccall f_8624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8624,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cddr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#caaar in k8569 */
static void C_ccall f_8627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8627,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_car(C_i_car(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#caadr in k8569 */
static void C_ccall f_8641(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8641,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(C_i_cadr(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cadar in k8569 */
static void C_ccall f_8648(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8648,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cadr(C_i_car(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdaar in k8569 */
static void C_ccall f_8655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8655,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(C_i_car(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#cdadr in k8569 */
static void C_ccall f_8669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8669,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(C_i_cadr(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cddar in k8569 */
static void C_ccall f_8676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8676,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(C_i_cdr(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#cdddr in k8569 */
static void C_ccall f_8690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8690,c,av);}
t3=C_i_cdr(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(C_i_cdr(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#caaaar in k8569 */
static void C_ccall f_8704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8704,c,av);}
t3=C_i_car(t2);
t4=C_i_car(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_car(C_i_car(t4));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#caaadr in k8569 */
static void C_ccall f_8722(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8722,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(C_i_car(C_i_cadr(t2)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#caadar in k8569 */
static void C_ccall f_8733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8733,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_car(C_i_cadr(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#caaddr in k8569 */
static void C_ccall f_8744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8744,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_car(C_i_caddr(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cadaar in k8569 */
static void C_ccall f_8751(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8751,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cadr(C_i_car(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#cadadr in k8569 */
static void C_ccall f_8762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8762,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cadr(C_i_cadr(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#caddar in k8569 */
static void C_ccall f_8766(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8766,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_caddr(C_i_car(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdaaar in k8569 */
static void C_ccall f_8773(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8773,c,av);}
t3=C_i_car(t2);
t4=C_i_car(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_cdr(C_i_car(t4));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#cdaadr in k8569 */
static void C_ccall f_8791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8791,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(C_i_car(C_i_cadr(t2)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdadar in k8569 */
static void C_ccall f_8802(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8802,c,av);}
t3=C_i_car(t2);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cdr(C_i_cadr(t3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#cdaddr in k8569 */
static void C_ccall f_8813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8813,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(C_i_caddr(t2));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cddaar in k8569 */
static void C_ccall f_8820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8820,c,av);}
t3=C_i_car(t2);
t4=C_i_car(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_cdr(C_i_cdr(t4));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#cddadr in k8569 */
static void C_ccall f_8838(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8838,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(C_i_cdr(C_i_cadr(t2)));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#cdddar in k8569 */
static void C_ccall f_8849(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8849,c,av);}
t3=C_i_car(t2);
t4=C_i_cdr(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_cdr(C_i_cdr(t4));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#null? in k8569 */
static void C_ccall f_8867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8867,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eqp(t2,C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#list in k8569 */
static void C_ccall f_8873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_8873,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#length in k8569 */
static void C_ccall f_8876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8876,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#list-tail in k8569 */
static void C_ccall f_8879(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8879,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_list_tail(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#list-ref in k8569 */
static void C_ccall f_8882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8882,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_list_ref(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#reverse in k8569 */
static void C_ccall f_8885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_8885,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8891,a[2]=t4,a[3]=t2,a[4]=((C_word)li44),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_8891(t6,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in scheme#reverse in k8569 */
static void C_fcall f_8891(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_8891,4,t0,t1,t2,t3);}
a=C_alloc(3);
t4=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
if(C_truep(C_i_pairp(t2))){
t5=C_slot(t2,C_fix(1));
t6=C_slot(t2,C_fix(0));
t7=C_a_i_cons(&a,2,t6,t3);
/* library.scm:313: loop */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
/* library.scm:314: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[48];
tp(4,av2);}}}}

/* scheme#memq in k8569 */
static void C_ccall f_8924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8924,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_memq(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#memv in k8569 */
static void C_ccall f_8927(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8927,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_memv(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#member in k8569 */
static void C_ccall f_8930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8930,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_member(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#assq in k8569 */
static void C_ccall f_8933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8933,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_assq(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#assv in k8569 */
static void C_ccall f_8936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8936,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_assv(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#assoc in k8569 */
static void C_ccall f_8939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8939,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_assoc(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#list? in k8569 */
static void C_ccall f_8942(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8942,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_listp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#string? in k8569 */
static void C_ccall f_8945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8945,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_stringp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#string-length in k8569 */
static void C_ccall f_8948(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8948,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_string_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#string-ref in k8569 */
static void C_ccall f_8951(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8951,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_string_ref(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#string-set! in k8569 */
static void C_ccall f_8954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8954,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_string_set(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#string=? in k8569 */
static void C_ccall f_8957(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8957,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_string_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#string-ci=? in k8569 */
static void C_ccall f_8960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8960,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_string_ci_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#exact->inexact in k8569 */
static void C_ccall f_8964(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_8964,c,av);}
a=C_alloc(12);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_exact_to_inexact(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#abs in k8569 */
static void C_ccall f_8967(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,1)))){
C_save_and_reclaim((void *)f_8967,c,av);}
a=C_alloc(7);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_abs(&a,1,t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#quotient in k8569 */
static void C_ccall f_8973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8973,c,av);}
a=C_alloc(5);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_s_a_i_quotient(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#remainder in k8569 */
static void C_ccall f_8976(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8976,c,av);}
a=C_alloc(5);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_s_a_i_remainder(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#modulo in k8569 */
static void C_ccall f_8979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,1)))){
C_save_and_reclaim((void *)f_8979,c,av);}
a=C_alloc(5);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_s_a_i_modulo(&a,2,t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#even? in k8569 */
static void C_ccall f_8982(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8982,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_evenp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#odd? in k8569 */
static void C_ccall f_8985(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8985,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_oddp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#number? in k8569 */
static void C_ccall f_8993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8993,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_numberp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#real? in k8569 */
static void C_ccall f_8997(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8997,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_realp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#rational? in k8569 */
static void C_ccall f_9000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9000,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_rationalp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#integer? in k8569 */
static void C_ccall f_9003(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9003,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_integerp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#exact? in k8569 */
static void C_ccall f_9006(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9006,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_exactp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#inexact? in k8569 */
static void C_ccall f_9009(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9009,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_inexactp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#zero? in k8569 */
static void C_ccall f_9012(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9012,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_zerop(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#positive? in k8569 */
static void C_ccall f_9015(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9015,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_positivep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#negative? in k8569 */
static void C_ccall f_9018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9018,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_negativep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#symbol? in k8569 */
static void C_ccall f_9022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9022,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_symbolp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#vector? in k8569 */
static void C_ccall f_9025(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9025,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_vectorp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#vector-length in k8569 */
static void C_ccall f_9028(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9028,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_vector_length(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#vector-ref in k8569 */
static void C_ccall f_9031(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9031,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_vector_ref(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#vector-set! in k8569 */
static void C_ccall f_9034(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9034,c,av);}
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_vector_set(t2,t3,t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* scheme#char? in k8569 */
static void C_ccall f_9042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9042,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_charp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#char->integer in k8569 */
static void C_ccall f_9045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9045,c,av);}
t3=C_i_check_char_2(t2,lf[100]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_fix(C_character_code(t2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#integer->char in k8569 */
static void C_ccall f_9052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9052,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9056,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:454: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[103];
tp(4,av2);}}

/* k9054 in scheme#integer->char in k8569 */
static void C_ccall f_9056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9056,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_make_character(C_unfix(((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* scheme#char=? in k8569 */
static void C_ccall f_9059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9059,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_char_equalp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char>? in k8569 */
static void C_ccall f_9062(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9062,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_char_greaterp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char<? in k8569 */
static void C_ccall f_9065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9065,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_char_lessp(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char>=? in k8569 */
static void C_ccall f_9068(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9068,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_char_greater_or_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char<=? in k8569 */
static void C_ccall f_9071(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9071,c,av);}
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_char_less_or_equal_p(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-upcase in k8569 */
static void C_ccall f_9074(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9074,c,av);}
t3=C_i_check_char_2(t2,lf[110]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_upcase(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-downcase in k8569 */
static void C_ccall f_9080(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9080,c,av);}
t3=C_i_check_char_2(t2,lf[112]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_downcase(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-upper-case? in k8569 */
static void C_ccall f_9091(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9091,c,av);}
t3=C_i_check_char_2(t2,lf[119]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_upper_casep(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-lower-case? in k8569 */
static void C_ccall f_9097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9097,c,av);}
t3=C_i_check_char_2(t2,lf[121]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_lower_casep(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-numeric? in k8569 */
static void C_ccall f_9103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9103,c,av);}
t3=C_i_check_char_2(t2,lf[123]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_numericp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-whitespace? in k8569 */
static void C_ccall f_9109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9109,c,av);}
t3=C_i_check_char_2(t2,lf[125]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_whitespacep(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#char-alphabetic? in k8569 */
static void C_ccall f_9115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9115,c,av);}
t3=C_i_check_char_2(t2,lf[127]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_char_alphabeticp(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* scheme#procedure? in k8569 */
static void C_ccall f_9121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9121,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_closurep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#input-port? in k8569 */
static void C_ccall f_9128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9128,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_blockp(t2))?C_input_portp(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#output-port? in k8569 */
static void C_ccall f_9134(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9134,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(C_blockp(t2))?C_output_portp(t2):C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#eof-object? in k8569 */
static void C_ccall f_9150(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9150,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_eofp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* scheme#eval in k8569 */
static void C_ccall f_9161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9161,c,av);}
/* library.scm:547: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[156];
av2[3]=lf[157];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* scheme#interaction-environment in k8569 */
static void C_ccall f_9167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9167,c,av);}
/* library.scm:550: ##sys#error */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[159];
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* scheme#scheme-report-environment in k8569 */
static void C_ccall f_9173(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9173,c,av);}
/* library.scm:553: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[162];
av2[3]=lf[163];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* scheme#null-environment in k8569 */
static void C_ccall f_9179(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9179,c,av);}
/* library.scm:556: ##sys#error */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[165];
av2[3]=lf[166];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* scheme#load in k8569 */
static void C_ccall f_9185(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9185,c,av);}
/* library.scm:559: ##sys#error */
t3=*((C_word*)lf[155]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[168];
av2[3]=lf[169];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* chicken.base#fixnum? in k8569 */
static void C_ccall f_9194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9194,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_fixnump(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#flonum? in k8569 */
static void C_ccall f_9197(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9197,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_flonump(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#bignum? in k8569 */
static void C_ccall f_9200(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9200,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_bignump(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#ratnum? in k8569 */
static void C_ccall f_9203(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9203,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_ratnump(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#cplxnum? in k8569 */
static void C_ccall f_9206(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9206,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cplxnump(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#exact-integer? in k8569 */
static void C_ccall f_9209(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9209,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_exact_integerp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#quotient&modulo in k8569 */
static void C_ccall f_9213(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_9213,c,av);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9219,a[2]=t2,a[3]=t3,a[4]=((C_word)li123),tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9225,a[2]=t3,a[3]=((C_word)li124),tmp=(C_word)a,a+=4,tmp);
/* library.scm:620: scheme#call-with-values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t4;
av2[3]=t5;
C_call_with_values(4,av2);}}

/* a9218 in chicken.base#quotient&modulo in k8569 */
static void C_ccall f_9219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9219,c,av);}
/* library.scm:620: quotient&remainder */
t2=*((C_word*)lf[180]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a9224 in chicken.base#quotient&modulo in k8569 */
static void C_ccall f_9225(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(29,c,3)))){
C_save_and_reclaim((void *)f_9225,c,av);}
a=C_alloc(29);
if(C_truep(C_i_positivep(((C_word*)t0)[2]))){
if(C_truep(C_i_negativep(t3))){
t4=C_s_a_i_plus(&a,2,t3,((C_word*)t0)[2]);
/* library.scm:624: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t4;
C_values(4,av2);}}
else{
/* library.scm:625: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_values(4,av2);}}}
else{
if(C_truep(C_i_positivep(t3))){
t4=C_s_a_i_plus(&a,2,t3,((C_word*)t0)[2]);
/* library.scm:627: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t4;
C_values(4,av2);}}
else{
/* library.scm:628: scheme#values */{
C_word *av2=av;
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_values(4,av2);}}}}

/* chicken.base#finite? in k8569 */
static void C_ccall f_9260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9260,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_finitep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#infinite? in k8569 */
static void C_ccall f_9263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9263,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_infinitep(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#nan? in k8569 */
static void C_ccall f_9266(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9266,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_nanp(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#add1 in k8569 */
static void C_ccall f_9274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_9274,c,av);}
a=C_alloc(29);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_plus(&a,2,t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#sub1 in k8569 */
static void C_ccall f_9280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_9280,c,av);}
a=C_alloc(29);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_s_a_i_minus(&a,2,t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#error in k8569 */
static void C_ccall f_9287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,4)))){
C_save_and_reclaim((void*)f_9287,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
if(C_truep(C_i_pairp(t2))){{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=0;
av2[1]=t1;
av2[2]=C_fast_retrieve(lf[194]);
av2[3]=lf[195];
av2[4]=t2;
C_apply(5,av2);}}
else{
/* library.scm:649: ##sys#signal-hook */
t3=C_fast_retrieve(lf[194]);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=lf[195];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* chicken.base#void in k8569 */
static void C_ccall f_9302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9302,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#promise? in k8569 */
static void C_ccall f_9325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9325,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_structurep(t2,lf[218]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* ##sys#make-promise in k8569 */
static void C_ccall f_9331(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9331,c,av);}
a=C_alloc(3);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_record2(&a,2,lf[218],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#make-promise in k8569 */
static void C_ccall f_9337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9337,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9344,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:695: promise? */
t4=*((C_word*)lf[217]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k9342 in chicken.base#make-promise in k8569 */
static void C_ccall f_9344(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9344,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9349,a[2]=((C_word*)t0)[3],a[3]=((C_word)li136),tmp=(C_word)a,a+=4,tmp);
/* library.scm:696: ##sys#make-promise */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[219]+1));
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=*((C_word*)lf[219]+1);
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
tp(3,av2);}}}

/* a9348 in k9342 in chicken.base#make-promise in k8569 */
static void C_ccall f_9349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9349,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#foldl in k8569 */
static void C_ccall f_9352(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_9352,c,av);}
a=C_alloc(7);
t5=C_i_check_list_2(t4,lf[222]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9361,a[2]=t7,a[3]=t2,a[4]=((C_word)li138),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_9361(t9,t1,t4,t3);}

/* loop in chicken.base#foldl in k8569 */
static void C_fcall f_9361(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9361,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=C_i_pairp(t2);
if(C_truep(C_i_not(t4))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_slot(t2,C_fix(1));
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9379,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* library.scm:705: f */
t7=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
av2[3]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t7))(4,av2);}}}

/* k9377 in loop in chicken.base#foldl in k8569 */
static void C_ccall f_9379(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9379,c,av);}
/* library.scm:705: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9361(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.base#foldr in k8569 */
static void C_ccall f_9389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=5) C_bad_argc_2(c,5,t0);
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_9389,c,av);}
a=C_alloc(8);
t5=C_i_check_list_2(t4,lf[224]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9398,a[2]=t3,a[3]=t2,a[4]=t7,a[5]=((C_word)li140),tmp=(C_word)a,a+=6,tmp));
t9=((C_word*)t7)[1];
f_9398(t9,t1,t4);}

/* loop in chicken.base#foldr in k8569 */
static void C_fcall f_9398(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9398,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9416,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* library.scm:712: loop */
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}}

/* k9414 in loop in chicken.base#foldr in k8569 */
static void C_ccall f_9416(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9416,c,av);}
/* library.scm:712: f */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t1;
((C_proc)C_fast_retrieve_proc(t2))(4,av2);}}

/* chicken.base#on-exit in k8569 */
static void C_ccall f_9429(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9429,c,av);}
a=C_alloc(3);
t3=C_a_i_cons(&a,2,t2,*((C_word*)lf[227]+1));
t4=C_mutate((C_word*)lf[227]+1 /* (set! chicken.base#cleanup-tasks ...) */,t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* chicken.base#exit in k8569 */
static void C_ccall f_9436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9436,c,av);}
a=C_alloc(4);
if(C_truep(C_rest_nullp(c,2))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f34823,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:725: chicken.base#exit-handler */
t3=*((C_word*)lf[226]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9443,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:725: chicken.base#exit-handler */
t4=*((C_word*)lf[226]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9441 in chicken.base#exit in k8569 */
static void C_ccall f_9443(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9443,c,av);}
/* library.scm:724: g673 */
t2=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.base#emergency-exit in k8569 */
static void C_ccall f_9454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9454,c,av);}
a=C_alloc(4);
if(C_truep(C_rest_nullp(c,2))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f34270,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* library.scm:728: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t2;
av2[2]=C_fix(0);
av2[3]=lf[231];
tp(4,av2);}}
else{
t2=C_get_rest_arg(c,2,av,2,t0);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9461,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* library.scm:728: ##sys#check-fixnum */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[102]+1));
C_word av2[4];
av2[0]=*((C_word*)lf[102]+1);
av2[1]=t3;
av2[2]=t2;
av2[3]=lf[231];
tp(4,av2);}}}

/* k9459 in chicken.base#emergency-exit in k8569 */
static void C_ccall f_9461(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9461,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_exit_runtime(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#identity in k8569 */
static void C_ccall f_9473(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9473,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#conjoin in k8569 */
static void C_ccall f_9476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,3)))){
C_save_and_reclaim((void*)f_9476,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9478,a[2]=t2,a[3]=((C_word)li147),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_9478 in chicken.base#conjoin in k8569 */
static void C_ccall f_9478(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_9478,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9484,a[2]=t4,a[3]=t2,a[4]=((C_word)li146),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_9484(t6,t1,((C_word*)t0)[2]);}

/* loop */
static void C_fcall f_9484(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9484,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9500,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:746: g702 */
t6=t4;{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}}

/* k9498 in loop */
static void C_ccall f_9500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9500,c,av);}
if(C_truep(t1)){
/* library.scm:747: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9484(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.base#disjoin in k8569 */
static void C_ccall f_9509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,3)))){
C_save_and_reclaim((void*)f_9509,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9511,a[2]=t2,a[3]=((C_word)li150),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_9511 in chicken.base#disjoin in k8569 */
static void C_ccall f_9511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_9511,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9517,a[2]=t4,a[3]=t2,a[4]=((C_word)li149),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_9517(t6,t1,((C_word*)t0)[2]);}

/* loop */
static void C_fcall f_9517(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9517,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(C_i_not(t3))){
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9530,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:753: g714 */
t6=t4;{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)C_fast_retrieve_proc(t6))(3,av2);}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k9528 in loop */
static void C_ccall f_9530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9530,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* library.scm:754: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_9517(t2,((C_word*)t0)[2],C_slot(((C_word*)t0)[4],C_fix(1)));}}

/* chicken.base#constantly in k8569 */
static void C_ccall f_9546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_9546,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_eqp(C_fix(1),C_u_i_length(t2));
if(C_truep(t3)){
t4=C_i_car(t2);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9557,a[2]=t4,a[3]=((C_word)li152),tmp=(C_word)a,a+=4,tmp);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9559,a[2]=t2,a[3]=((C_word)li153),tmp=(C_word)a,a+=4,tmp);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* f_9557 in chicken.base#constantly in k8569 */
static void C_ccall f_9557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9557,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_9559 in chicken.base#constantly in k8569 */
static void C_ccall f_9559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9559,c,av);}{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=*((C_word*)lf[130]+1);
av2[3]=((C_word*)t0)[2];
C_apply(4,av2);}}

/* chicken.base#flip in k8569 */
static void C_ccall f_9567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_9567,c,av);}
a=C_alloc(4);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9569,a[2]=t2,a[3]=((C_word)li155),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_9569 in chicken.base#flip in k8569 */
static void C_ccall f_9569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9569,c,av);}
/* library.scm:762: proc */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
av2[3]=t2;
((C_proc)C_fast_retrieve_proc(t4))(4,av2);}}

/* chicken.base#complement in k8569 */
static void C_ccall f_9575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9575,c,av);}
a=C_alloc(4);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9577,a[2]=t2,a[3]=((C_word)li157),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_9577 in chicken.base#complement in k8569 */
static void C_ccall f_9577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +3,c,3)))){
C_save_and_reclaim((void*)f_9577,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+3);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9585,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
C_apply(4,av2);}}

/* k9583 */
static void C_ccall f_9585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9585,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#compose in k8569 */
static void C_ccall f_9587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_9587,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9590,a[2]=t4,a[3]=((C_word)li161),tmp=(C_word)a,a+=4,tmp));
if(C_truep(C_rest_nullp(c,2))){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=*((C_word*)lf[130]+1);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t4)[1];
av2[3]=t2;
C_apply(4,av2);}}}

/* rec in chicken.base#compose in k8569 */
static void C_ccall f_9590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +6,c,2)))){
C_save_and_reclaim((void*)f_9590,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+6);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9598,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t2,a[5]=((C_word)li160),tmp=(C_word)a,a+=6,tmp);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* f_9598 in rec in chicken.base#compose in k8569 */
static void C_ccall f_9598(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_9598,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9604,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word)li159),tmp=(C_word)a,a+=6,tmp);
/* library.scm:773: scheme#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=((C_word*)t0)[4];
C_call_with_values(4,av2);}}

/* a9603 */
static void C_ccall f_9604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(c!=2) C_bad_argc_2(c,2,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9604,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9612,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=((C_word*)t0)[4];
C_apply(4,av2);}}

/* k9610 in a9603 */
static void C_ccall f_9612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9612,c,av);}{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_apply(4,av2);}}

/* chicken.base#o in k8569 */
static void C_ccall f_9623(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,3)))){
C_save_and_reclaim((void*)f_9623,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
if(C_truep(C_rest_nullp(c,2))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=*((C_word*)lf[236]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9635,a[2]=t4,a[3]=((C_word)li164),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_9635(t6,t1,t2);}}

/* loop in chicken.base#o in k8569 */
static void C_fcall f_9635(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_9635,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_nullp(t4))){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9649,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t4,a[5]=((C_word)li163),tmp=(C_word)a,a+=6,tmp);
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* f_9649 in loop in chicken.base#o in k8569 */
static void C_ccall f_9649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9649,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9657,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:788: loop */
t4=((C_word*)((C_word*)t0)[3])[1];
f_9635(t4,t3,((C_word*)t0)[4]);}

/* k9655 */
static void C_ccall f_9657(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9657,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9660,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* library.scm:788: g743 */
t3=t1;{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)C_fast_retrieve_proc(t3))(3,av2);}}

/* k9658 in k9655 */
static void C_ccall f_9660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9660,c,av);}
/* library.scm:788: h */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)C_fast_retrieve_proc(t2))(3,av2);}}

/* chicken.base#list-of? in k8569 */
static void C_ccall f_9662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_9662,c,av);}
a=C_alloc(4);
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9664,a[2]=t2,a[3]=((C_word)li167),tmp=(C_word)a,a+=4,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_9664 in chicken.base#list-of? in k8569 */
static void C_ccall f_9664(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_9664,c,av);}
a=C_alloc(7);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9670,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=((C_word)li166),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_9670(t6,t1,t2);}

/* loop */
static void C_fcall f_9670(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9670,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9689,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* library.scm:795: pred */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)C_fast_retrieve_proc(t5))(3,av2);}}}}

/* k9687 in loop */
static void C_ccall f_9689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9689,c,av);}
if(C_truep(t1)){
/* library.scm:795: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9670(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* chicken.base#each in k8569 */
static void C_ccall f_9706(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +4,c,2)))){
C_save_and_reclaim((void*)f_9706,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+4);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
if(C_truep(C_i_nullp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9714,a[2]=((C_word)li169),tmp=(C_word)a,a+=3,tmp);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t3=C_slot(t2,C_fix(1));
t4=C_i_nullp(t3);
t5=t1;{
C_word *av2=av;
av2[0]=t5;
av2[1]=(C_truep(t4)?C_slot(t2,C_fix(0)):(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9728,a[2]=t2,a[3]=((C_word)li171),tmp=(C_word)a,a+=4,tmp));
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* f_9714 in chicken.base#each in k8569 */
static void C_ccall f_9714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9714,c,av);}
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=*((C_word*)lf[246]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* f_9728 in chicken.base#each in k8569 */
static void C_ccall f_9728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +7,c,3)))){
C_save_and_reclaim((void*)f_9728,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+7);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9734,a[2]=t2,a[3]=t4,a[4]=((C_word)li170),tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_9734(t6,t1,((C_word*)t0)[2]);}

/* loop */
static void C_fcall f_9734(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9734,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_slot(t2,C_fix(0));
t4=C_slot(t2,C_fix(1));
if(C_truep(C_i_nullp(t4))){{
C_word av2[4];
av2[0]=0;
av2[1]=t1;
av2[2]=t3;
av2[3]=((C_word*)t0)[2];
C_apply(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9753,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);{
C_word av2[4];
av2[0]=0;
av2[1]=t5;
av2[2]=t3;
av2[3]=((C_word*)t0)[2];
C_apply(4,av2);}}}

/* k9751 in loop */
static void C_ccall f_9753(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_9753,c,av);}
/* library.scm:810: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9734(t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* chicken.base#atom? in k8569 */
static void C_ccall f_9762(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9762,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_not_pair_p(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#tail? in k8569 */
static void C_ccall f_9765(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9765,c,av);}
a=C_alloc(4);
t4=C_i_check_list_2(t3,lf[249]);
t5=C_eqp(t2,C_SCHEME_END_OF_LIST);
if(C_truep(t5)){
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9777,a[2]=t2,a[3]=((C_word)li174),tmp=(C_word)a,a+=4,tmp);
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=(
  f_9777(t6,t3)
);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* loop in chicken.base#tail? in k8569 */
static C_word C_fcall f_9777(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;
loop:{}
if(C_truep(C_eqp(t1,C_SCHEME_END_OF_LIST))){
return(C_SCHEME_FALSE);}
else{
if(C_truep(C_eqp(((C_word*)t0)[2],t1))){
return(C_SCHEME_TRUE);}
else{
t3=C_slot(t1,C_fix(1));
t1=t3;
goto loop;}}}

/* chicken.base#intersperse in k8569 */
static void C_ccall f_9793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_9793,c,av);}
a=C_alloc(7);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9799,a[2]=t3,a[3]=t5,a[4]=((C_word)li176),tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_9799(t7,t1,t2);}

/* loop in chicken.base#intersperse in k8569 */
static void C_fcall f_9799(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9799,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_eqp(t2,C_SCHEME_END_OF_LIST))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_cdr(t2);
if(C_truep(C_eqp(t3,C_SCHEME_END_OF_LIST))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9824,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* library.scm:833: loop */
t7=t5;
t8=t3;
t1=t7;
t2=t8;
goto loop;}}}

/* k9822 in loop in chicken.base#intersperse in k8569 */
static void C_ccall f_9824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_9824,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.base#butlast in k8569 */
static void C_ccall f_9826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(c!=3) C_bad_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_9826,c,av);}
a=C_alloc(6);
t3=C_i_check_pair_2(t2,lf[252]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9835,a[2]=t5,a[3]=((C_word)li178),tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_9835(t7,t1,t2);}

/* loop in chicken.base#butlast in k8569 */
static void C_fcall f_9835(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_9835,3,t0,t1,t2);}
a=C_alloc(4);
t3=C_slot(t2,C_fix(1));
t4=(C_truep(C_blockp(t3))?C_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t4)){
t5=C_slot(t2,C_fix(0));
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9856,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* library.scm:840: loop */
t8=t6;
t9=t3;
t1=t8;
t2=t9;
goto loop;}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}

/* k9854 in loop in chicken.base#butlast in k8569 */
static void C_ccall f_9856(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9856,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#flatten in k8569 */
static void C_ccall f_9858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +6,c,4)))){
C_save_and_reclaim((void*)f_9858,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+6);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
C_word t5;
C_word t6;
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9864,a[2]=t4,a[3]=((C_word)li180),tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_9864(t6,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in chicken.base#flatten in k8569 */
static void C_fcall f_9864(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_9864,4,t0,t1,t2,t3);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=C_slot(t2,C_fix(1));
if(C_truep(C_i_listp(t4))){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9890,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* library.scm:850: loop */
t8=t6;
t9=t5;
t10=t3;
t1=t8;
t2=t9;
t3=t10;
goto loop;}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9897,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* library.scm:851: loop */
t8=t6;
t9=t5;
t10=t3;
t1=t8;
t2=t9;
t3=t10;
goto loop;}}}

/* k9888 in loop in chicken.base#flatten in k8569 */
static void C_ccall f_9890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9890,c,av);}
/* library.scm:850: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_9864(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* k9895 in loop in chicken.base#flatten in k8569 */
static void C_ccall f_9897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9897,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.base#join in k8569 */
static void C_ccall f_9899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(c<3) C_bad_min_argc_2(c,3,t0);
if(C_unlikely(!C_demand(C_calculate_demand((c-3)*C_SIZEOF_PAIR +7,c,3)))){
C_save_and_reclaim((void*)f_9899,c,av);}
a=C_alloc((c-3)*C_SIZEOF_PAIR+7);
t3=C_build_rest(&a,c,3,av);
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
t4=C_i_pairp(t3);
t5=(C_truep(t4)?C_get_rest_arg(c,3,av,3,t0):C_SCHEME_END_OF_LIST);
t6=C_i_check_list_2(t5,lf[255]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9911,a[2]=t5,a[3]=t8,a[4]=((C_word)li182),tmp=(C_word)a,a+=5,tmp));
t10=((C_word*)t8)[1];
f_9911(t10,t1,t2);}

/* loop in chicken.base#join in k8569 */
static void C_fcall f_9911(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_9911,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_pairp(t2);
if(C_truep(C_i_not(t3))){
/* library.scm:861: ##sys#error-not-a-proper-list */
{C_proc tp=(C_proc)C_fast_retrieve_proc(*((C_word*)lf[47]+1));
C_word av2[3];
av2[0]=*((C_word*)lf[47]+1);
av2[1]=t1;
av2[2]=t2;
tp(3,av2);}}
else{
t4=C_slot(t2,C_fix(0));
t5=C_slot(t2,C_fix(1));
if(C_truep(C_i_nullp(t5))){
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9946,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* library.scm:867: loop */
t8=t6;
t9=t5;
t1=t8;
t2=t9;
goto loop;}}}}

/* k9944 in loop in chicken.base#join in k8569 */
static void C_ccall f_9946(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9946,c,av);}
/* library.scm:867: ##sys#append */
t2=*((C_word*)lf[256]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* chicken.base#compress in k8569 */
static void C_ccall f_9956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(c!=4) C_bad_argc_2(c,4,t0);
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_9956,c,av);}
a=C_alloc(7);
t4=lf[258];
t5=C_i_check_list_2(t3,lf[259]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9965,a[2]=t4,a[3]=t7,a[4]=((C_word)li184),tmp=(C_word)a,a+=5,tmp));
t9=((C_word*)t7)[1];
f_9965(t9,t1,t2,t3);}

/* loop in chicken.base#compress in k8569 */
static void C_fcall f_9965(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,5)))){
C_save_and_reclaim_args((void *)trf_9965,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_pairp(t2);
if(C_truep(C_i_not(t4))){
/* library.scm:876: ##sys#signal-hook */
t5=C_fast_retrieve(lf[194]);{
C_word av2[6];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[259];
av2[4]=((C_word*)t0)[2];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
t5=C_i_pairp(t3);
if(C_truep(C_i_not(t5))){
/* library.scm:878: ##sys#signal-hook */
t6=C_fast_retrieve(lf[194]);{
C_word av2[6];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[260];
av2[3]=lf[259];
av2[4]=((C_word*)t0)[2];
av2[5]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(6,av2);}}
else{
if(C_truep(C_slot(t2,C_fix(0)))){
t6=C_slot(t3,C_fix(0));
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10007,a[2]=t1,a[3]=t6,tmp=(C_word)a,a+=4,tmp);
/* library.scm:880: loop */
t9=t7;
t10=C_slot(t2,C_fix(1));
t11=C_slot(t3,C_fix(1));
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
/* library.scm:881: loop */
t9=t1;
t10=C_slot(t2,C_fix(1));
t11=C_slot(t3,C_fix(1));
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_library_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("library"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_library_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(7830))){
C_save(t1);
C_rereclaim2(7830*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,1405);
lf[0]=C_h_intern(&lf[0],7, C_text("library"));
lf[1]=C_h_intern(&lf[1],7, C_text("scheme#"));
lf[2]=C_h_intern(&lf[2],10, C_text("scheme#not"));
lf[3]=C_h_intern(&lf[3],15, C_text("scheme#boolean\077"));
lf[4]=C_h_intern(&lf[4],10, C_text("scheme#eq\077"));
lf[5]=C_h_intern(&lf[5],11, C_text("scheme#eqv\077"));
lf[6]=C_h_intern(&lf[6],13, C_text("scheme#equal\077"));
lf[7]=C_h_intern(&lf[7],12, C_text("scheme#pair\077"));
lf[8]=C_h_intern(&lf[8],11, C_text("scheme#cons"));
lf[9]=C_h_intern(&lf[9],10, C_text("scheme#car"));
lf[10]=C_h_intern(&lf[10],10, C_text("scheme#cdr"));
lf[11]=C_h_intern(&lf[11],15, C_text("scheme#set-car!"));
lf[12]=C_h_intern(&lf[12],15, C_text("scheme#set-cdr!"));
lf[13]=C_h_intern(&lf[13],11, C_text("scheme#cadr"));
lf[14]=C_h_intern(&lf[14],12, C_text("scheme#caddr"));
lf[15]=C_h_intern(&lf[15],13, C_text("scheme#cadddr"));
lf[16]=C_h_intern(&lf[16],13, C_text("scheme#cddddr"));
lf[17]=C_h_intern(&lf[17],11, C_text("scheme#caar"));
lf[18]=C_h_intern(&lf[18],11, C_text("scheme#cdar"));
lf[19]=C_h_intern(&lf[19],11, C_text("scheme#cddr"));
lf[20]=C_h_intern(&lf[20],12, C_text("scheme#caaar"));
lf[21]=C_h_intern(&lf[21],12, C_text("scheme#caadr"));
lf[22]=C_h_intern(&lf[22],12, C_text("scheme#cadar"));
lf[23]=C_h_intern(&lf[23],12, C_text("scheme#cdaar"));
lf[24]=C_h_intern(&lf[24],12, C_text("scheme#cdadr"));
lf[25]=C_h_intern(&lf[25],12, C_text("scheme#cddar"));
lf[26]=C_h_intern(&lf[26],12, C_text("scheme#cdddr"));
lf[27]=C_h_intern(&lf[27],13, C_text("scheme#caaaar"));
lf[28]=C_h_intern(&lf[28],13, C_text("scheme#caaadr"));
lf[29]=C_h_intern(&lf[29],13, C_text("scheme#caadar"));
lf[30]=C_h_intern(&lf[30],13, C_text("scheme#caaddr"));
lf[31]=C_h_intern(&lf[31],13, C_text("scheme#cadaar"));
lf[32]=C_h_intern(&lf[32],13, C_text("scheme#cadadr"));
lf[33]=C_h_intern(&lf[33],13, C_text("scheme#caddar"));
lf[34]=C_h_intern(&lf[34],13, C_text("scheme#cdaaar"));
lf[35]=C_h_intern(&lf[35],13, C_text("scheme#cdaadr"));
lf[36]=C_h_intern(&lf[36],13, C_text("scheme#cdadar"));
lf[37]=C_h_intern(&lf[37],13, C_text("scheme#cdaddr"));
lf[38]=C_h_intern(&lf[38],13, C_text("scheme#cddaar"));
lf[39]=C_h_intern(&lf[39],13, C_text("scheme#cddadr"));
lf[40]=C_h_intern(&lf[40],13, C_text("scheme#cdddar"));
lf[41]=C_h_intern(&lf[41],12, C_text("scheme#null\077"));
lf[42]=C_h_intern(&lf[42],11, C_text("scheme#list"));
lf[43]=C_h_intern(&lf[43],13, C_text("scheme#length"));
lf[44]=C_h_intern(&lf[44],16, C_text("scheme#list-tail"));
lf[45]=C_h_intern(&lf[45],15, C_text("scheme#list-ref"));
lf[46]=C_h_intern(&lf[46],14, C_text("scheme#reverse"));
lf[47]=C_h_intern(&lf[47],29, C_text("##sys#error-not-a-proper-list"));
lf[48]=C_h_intern(&lf[48],7, C_text("reverse"));
lf[49]=C_h_intern(&lf[49],11, C_text("scheme#memq"));
lf[50]=C_h_intern(&lf[50],11, C_text("scheme#memv"));
lf[51]=C_h_intern(&lf[51],13, C_text("scheme#member"));
lf[52]=C_h_intern(&lf[52],11, C_text("scheme#assq"));
lf[53]=C_h_intern(&lf[53],11, C_text("scheme#assv"));
lf[54]=C_h_intern(&lf[54],12, C_text("scheme#assoc"));
lf[55]=C_h_intern(&lf[55],12, C_text("scheme#list\077"));
lf[56]=C_h_intern(&lf[56],14, C_text("scheme#string\077"));
lf[57]=C_h_intern(&lf[57],20, C_text("scheme#string-length"));
lf[58]=C_h_intern(&lf[58],17, C_text("scheme#string-ref"));
lf[59]=C_h_intern(&lf[59],18, C_text("scheme#string-set!"));
lf[60]=C_h_intern(&lf[60],15, C_text("scheme#string=\077"));
lf[61]=C_h_intern(&lf[61],18, C_text("scheme#string-ci=\077"));
lf[62]=C_h_intern(&lf[62],21, C_text("scheme#exact->inexact"));
lf[63]=C_h_intern(&lf[63],10, C_text("scheme#abs"));
lf[64]=C_h_intern(&lf[64],8, C_text("scheme#+"));
lf[65]=C_h_intern(&lf[65],8, C_text("scheme#-"));
lf[66]=C_h_intern(&lf[66],8, C_text("scheme#\052"));
lf[67]=C_h_intern(&lf[67],15, C_text("scheme#quotient"));
lf[68]=C_h_intern(&lf[68],16, C_text("scheme#remainder"));
lf[69]=C_h_intern(&lf[69],13, C_text("scheme#modulo"));
lf[70]=C_h_intern(&lf[70],12, C_text("scheme#even\077"));
lf[71]=C_h_intern(&lf[71],11, C_text("scheme#odd\077"));
lf[72]=C_h_intern(&lf[72],8, C_text("scheme#="));
lf[73]=C_h_intern(&lf[73],8, C_text("scheme#>"));
lf[74]=C_h_intern(&lf[74],8, C_text("scheme#<"));
lf[75]=C_h_intern(&lf[75],9, C_text("scheme#>="));
lf[76]=C_h_intern(&lf[76],9, C_text("scheme#<="));
lf[77]=C_h_intern(&lf[77],14, C_text("scheme#number\077"));
lf[78]=C_h_intern(&lf[78],15, C_text("scheme#complex\077"));
lf[79]=C_h_intern(&lf[79],12, C_text("scheme#real\077"));
lf[80]=C_h_intern(&lf[80],16, C_text("scheme#rational\077"));
lf[81]=C_h_intern(&lf[81],15, C_text("scheme#integer\077"));
lf[82]=C_h_intern(&lf[82],13, C_text("scheme#exact\077"));
lf[83]=C_h_intern(&lf[83],15, C_text("scheme#inexact\077"));
lf[84]=C_h_intern(&lf[84],12, C_text("scheme#zero\077"));
lf[85]=C_h_intern(&lf[85],16, C_text("scheme#positive\077"));
lf[86]=C_h_intern(&lf[86],16, C_text("scheme#negative\077"));
lf[87]=C_h_intern(&lf[87],21, C_text("scheme#number->string"));
lf[88]=C_h_intern(&lf[88],14, C_text("scheme#symbol\077"));
lf[89]=C_h_intern(&lf[89],14, C_text("scheme#vector\077"));
lf[90]=C_h_intern(&lf[90],20, C_text("scheme#vector-length"));
lf[91]=C_h_intern(&lf[91],17, C_text("scheme#vector-ref"));
lf[92]=C_h_intern(&lf[92],18, C_text("scheme#vector-set!"));
lf[93]=C_h_intern(&lf[93],18, C_text("scheme#make-vector"));
lf[94]=C_h_intern(&lf[94],19, C_text("scheme#list->vector"));
lf[95]=C_h_intern(&lf[95],19, C_text("scheme#vector->list"));
lf[96]=C_h_intern(&lf[96],13, C_text("scheme#vector"));
lf[97]=C_h_intern(&lf[97],19, C_text("scheme#vector-fill!"));
lf[98]=C_h_intern(&lf[98],12, C_text("scheme#char\077"));
lf[99]=C_h_intern(&lf[99],20, C_text("scheme#char->integer"));
lf[100]=C_h_intern(&lf[100],13, C_text("char->integer"));
lf[101]=C_h_intern(&lf[101],20, C_text("scheme#integer->char"));
lf[102]=C_h_intern(&lf[102],18, C_text("##sys#check-fixnum"));
lf[103]=C_h_intern(&lf[103],13, C_text("integer->char"));
lf[104]=C_h_intern(&lf[104],13, C_text("scheme#char=\077"));
lf[105]=C_h_intern(&lf[105],13, C_text("scheme#char>\077"));
lf[106]=C_h_intern(&lf[106],13, C_text("scheme#char<\077"));
lf[107]=C_h_intern(&lf[107],14, C_text("scheme#char>=\077"));
lf[108]=C_h_intern(&lf[108],14, C_text("scheme#char<=\077"));
lf[109]=C_h_intern(&lf[109],18, C_text("scheme#char-upcase"));
lf[110]=C_h_intern(&lf[110],11, C_text("char-upcase"));
lf[111]=C_h_intern(&lf[111],20, C_text("scheme#char-downcase"));
lf[112]=C_h_intern(&lf[112],13, C_text("char-downcase"));
lf[113]=C_h_intern(&lf[113],16, C_text("scheme#char-ci=\077"));
lf[114]=C_h_intern(&lf[114],16, C_text("scheme#char-ci>\077"));
lf[115]=C_h_intern(&lf[115],16, C_text("scheme#char-ci<\077"));
lf[116]=C_h_intern(&lf[116],17, C_text("scheme#char-ci>=\077"));
lf[117]=C_h_intern(&lf[117],17, C_text("scheme#char-ci<=\077"));
lf[118]=C_h_intern(&lf[118],23, C_text("scheme#char-upper-case\077"));
lf[119]=C_h_intern(&lf[119],16, C_text("char-upper-case\077"));
lf[120]=C_h_intern(&lf[120],23, C_text("scheme#char-lower-case\077"));
lf[121]=C_h_intern(&lf[121],16, C_text("char-lower-case\077"));
lf[122]=C_h_intern(&lf[122],20, C_text("scheme#char-numeric\077"));
lf[123]=C_h_intern(&lf[123],13, C_text("char-numeric\077"));
lf[124]=C_h_intern(&lf[124],23, C_text("scheme#char-whitespace\077"));
lf[125]=C_h_intern(&lf[125],16, C_text("char-whitespace\077"));
lf[126]=C_h_intern(&lf[126],23, C_text("scheme#char-alphabetic\077"));
lf[127]=C_h_intern(&lf[127],16, C_text("char-alphabetic\077"));
lf[128]=C_h_intern(&lf[128],17, C_text("scheme#procedure\077"));
lf[129]=C_h_intern(&lf[129],12, C_text("scheme#apply"));
lf[130]=C_h_intern(&lf[130],13, C_text("scheme#values"));
lf[131]=C_h_intern(&lf[131],23, C_text("scheme#call-with-values"));
lf[132]=C_h_intern(&lf[132],37, C_text("scheme#call-with-current-continuation"));
lf[133]=C_h_intern(&lf[133],18, C_text("scheme#input-port\077"));
lf[134]=C_h_intern(&lf[134],19, C_text("scheme#output-port\077"));
lf[135]=C_h_intern(&lf[135],25, C_text("scheme#current-input-port"));
lf[136]=C_h_intern(&lf[136],26, C_text("scheme#current-output-port"));
lf[137]=C_h_intern(&lf[137],22, C_text("scheme#open-input-file"));
lf[138]=C_h_intern(&lf[138],23, C_text("scheme#open-output-file"));
lf[139]=C_h_intern(&lf[139],23, C_text("scheme#close-input-port"));
lf[140]=C_h_intern(&lf[140],24, C_text("scheme#close-output-port"));
lf[141]=C_h_intern(&lf[141],27, C_text("scheme#call-with-input-file"));
lf[142]=C_h_intern(&lf[142],28, C_text("scheme#call-with-output-file"));
lf[143]=C_h_intern(&lf[143],27, C_text("scheme#with-input-from-file"));
lf[144]=C_h_intern(&lf[144],26, C_text("scheme#with-output-to-file"));
lf[145]=C_h_intern(&lf[145],18, C_text("scheme#eof-object\077"));
lf[146]=C_h_intern(&lf[146],18, C_text("scheme#char-ready\077"));
lf[147]=C_h_intern(&lf[147],16, C_text("scheme#read-char"));
lf[148]=C_h_intern(&lf[148],16, C_text("scheme#peek-char"));
lf[149]=C_h_intern(&lf[149],11, C_text("scheme#read"));
lf[150]=C_h_intern(&lf[150],17, C_text("scheme#write-char"));
lf[151]=C_h_intern(&lf[151],14, C_text("scheme#newline"));
lf[152]=C_h_intern(&lf[152],12, C_text("scheme#write"));
lf[153]=C_h_intern(&lf[153],14, C_text("scheme#display"));
lf[154]=C_h_intern(&lf[154],11, C_text("scheme#eval"));
lf[155]=C_h_intern(&lf[155],11, C_text("##sys#error"));
lf[156]=C_h_intern(&lf[156],4, C_text("eval"));
lf[157]=C_decode_literal(C_heaptop,C_text("\376B\000\000T`eval\047 is not defined - the `eval\047 unit was probably not linked with this e"
"xecutable"));
lf[158]=C_h_intern(&lf[158],30, C_text("scheme#interaction-environment"));
lf[159]=C_h_intern(&lf[159],23, C_text("interaction-environment"));
lf[160]=C_decode_literal(C_heaptop,C_text("\376B\000\000g`interaction-environment\047 is not defined - the `eval\047 unit was probably not"
" linked with this executable"));
lf[161]=C_h_intern(&lf[161],32, C_text("scheme#scheme-report-environment"));
lf[162]=C_h_intern(&lf[162],25, C_text("scheme-report-environment"));
lf[163]=C_decode_literal(C_heaptop,C_text("\376B\000\000i`scheme-report-environment\047 is not defined - the `eval\047 unit was probably n"
"ot linked with this executable"));
lf[164]=C_h_intern(&lf[164],23, C_text("scheme#null-environment"));
lf[165]=C_h_intern(&lf[165],16, C_text("null-environment"));
lf[166]=C_decode_literal(C_heaptop,C_text("\376B\000\000``null-environment\047 is not defined - the `eval\047 unit was probably not linked"
" with this executable"));
lf[167]=C_h_intern(&lf[167],11, C_text("scheme#load"));
lf[168]=C_h_intern(&lf[168],4, C_text("load"));
lf[169]=C_decode_literal(C_heaptop,C_text("\376B\000\000T`load\047 is not defined - the `eval\047 unit was probably not linked with this e"
"xecutable"));
lf[170]=C_h_intern(&lf[170],15, C_text("scheme#for-each"));
lf[171]=C_h_intern(&lf[171],10, C_text("scheme#map"));
lf[172]=C_h_intern(&lf[172],19, C_text("scheme#dynamic-wind"));
lf[173]=C_h_intern(&lf[173],13, C_text("chicken.base#"));
lf[174]=C_h_intern(&lf[174],20, C_text("chicken.base#fixnum\077"));
lf[175]=C_h_intern(&lf[175],20, C_text("chicken.base#flonum\077"));
lf[176]=C_h_intern(&lf[176],20, C_text("chicken.base#bignum\077"));
lf[177]=C_h_intern(&lf[177],20, C_text("chicken.base#ratnum\077"));
lf[178]=C_h_intern(&lf[178],21, C_text("chicken.base#cplxnum\077"));
lf[179]=C_h_intern(&lf[179],27, C_text("chicken.base#exact-integer\077"));
lf[180]=C_h_intern(&lf[180],31, C_text("chicken.base#quotient&remainder"));
lf[181]=C_h_intern(&lf[181],28, C_text("chicken.base#quotient&modulo"));
lf[182]=C_h_intern(&lf[182],20, C_text("chicken.base#finite\077"));
lf[183]=C_h_intern(&lf[183],22, C_text("chicken.base#infinite\077"));
lf[184]=C_h_intern(&lf[184],17, C_text("chicken.base#nan\077"));
lf[185]=C_h_intern(&lf[185],19, C_text("chicken.base#signum"));
lf[186]=C_h_intern(&lf[186],27, C_text("chicken.base#get-call-chain"));
lf[187]=C_h_intern(&lf[187],29, C_text("chicken.base#print-call-chain"));
lf[188]=C_h_intern(&lf[188],18, C_text("chicken.base#print"));
lf[189]=C_h_intern(&lf[189],19, C_text("chicken.base#print\052"));
lf[190]=C_h_intern(&lf[190],17, C_text("chicken.base#add1"));
lf[191]=C_h_intern(&lf[191],17, C_text("chicken.base#sub1"));
lf[192]=C_h_intern(&lf[192],31, C_text("chicken.base#current-error-port"));
lf[193]=C_h_intern(&lf[193],18, C_text("chicken.base#error"));
lf[194]=C_h_intern(&lf[194],17, C_text("##sys#signal-hook"));
lf[195]=C_h_intern_kw(&lf[195],5, C_text("error"));
lf[196]=C_h_intern(&lf[196],17, C_text("chicken.base#void"));
lf[197]=C_h_intern(&lf[197],18, C_text("chicken.base#sleep"));
lf[198]=C_h_intern(&lf[198],20, C_text("chicken.base#call/cc"));
lf[199]=C_h_intern(&lf[199],22, C_text("chicken.base#char-name"));
lf[200]=C_h_intern(&lf[200],31, C_text("chicken.base#getter-with-setter"));
lf[201]=C_h_intern(&lf[201],27, C_text("chicken.base#make-parameter"));
lf[202]=C_h_intern(&lf[202],34, C_text("chicken.base#procedure-information"));
lf[203]=C_h_intern(&lf[203],19, C_text("chicken.base#setter"));
lf[204]=C_h_intern(&lf[204],27, C_text("chicken.base#record-printer"));
lf[205]=C_h_intern(&lf[205],32, C_text("chicken.base#set-record-printer!"));
lf[206]=C_h_intern(&lf[206],25, C_text("chicken.base#vector-copy!"));
lf[207]=C_h_intern(&lf[207],22, C_text("chicken.base#subvector"));
lf[208]=C_h_intern(&lf[208],26, C_text("chicken.base#vector-resize"));
lf[209]=C_h_intern(&lf[209],18, C_text("chicken.base#port\077"));
lf[210]=C_h_intern(&lf[210],25, C_text("chicken.base#port-closed\077"));
lf[211]=C_h_intern(&lf[211],29, C_text("chicken.base#input-port-open\077"));
lf[212]=C_h_intern(&lf[212],30, C_text("chicken.base#output-port-open\077"));
lf[213]=C_h_intern(&lf[213],30, C_text("chicken.base#get-output-string"));
lf[214]=C_h_intern(&lf[214],30, C_text("chicken.base#open-input-string"));
lf[215]=C_h_intern(&lf[215],31, C_text("chicken.base#open-output-string"));
lf[216]=C_h_intern(&lf[216],25, C_text("chicken.base#flush-output"));
lf[217]=C_h_intern(&lf[217],21, C_text("chicken.base#promise\077"));
lf[218]=C_h_intern(&lf[218],7, C_text("promise"));
lf[219]=C_h_intern(&lf[219],18, C_text("##sys#make-promise"));
lf[220]=C_h_intern(&lf[220],25, C_text("chicken.base#make-promise"));
lf[221]=C_h_intern(&lf[221],18, C_text("chicken.base#foldl"));
lf[222]=C_h_intern(&lf[222],5, C_text("foldl"));
lf[223]=C_h_intern(&lf[223],18, C_text("chicken.base#foldr"));
lf[224]=C_h_intern(&lf[224],5, C_text("foldr"));
lf[225]=C_h_intern(&lf[225],34, C_text("chicken.base#implicit-exit-handler"));
lf[226]=C_h_intern(&lf[226],25, C_text("chicken.base#exit-handler"));
lf[227]=C_h_intern(&lf[227],26, C_text("chicken.base#cleanup-tasks"));
lf[228]=C_h_intern(&lf[228],20, C_text("chicken.base#on-exit"));
lf[229]=C_h_intern(&lf[229],17, C_text("chicken.base#exit"));
lf[230]=C_h_intern(&lf[230],27, C_text("chicken.base#emergency-exit"));
lf[231]=C_h_intern(&lf[231],14, C_text("emergency-exit"));
lf[232]=C_h_intern(&lf[232],27, C_text("chicken.base#case-sensitive"));
lf[233]=C_h_intern(&lf[233],26, C_text("chicken.base#keyword-style"));
lf[234]=C_h_intern(&lf[234],33, C_text("chicken.base#parentheses-synonyms"));
lf[235]=C_h_intern(&lf[235],26, C_text("chicken.base#symbol-escape"));
lf[236]=C_h_intern(&lf[236],21, C_text("chicken.base#identity"));
lf[237]=C_h_intern(&lf[237],20, C_text("chicken.base#conjoin"));
lf[238]=C_h_intern(&lf[238],20, C_text("chicken.base#disjoin"));
lf[239]=C_h_intern(&lf[239],23, C_text("chicken.base#constantly"));
lf[240]=C_h_intern(&lf[240],17, C_text("chicken.base#flip"));
lf[241]=C_h_intern(&lf[241],23, C_text("chicken.base#complement"));
lf[242]=C_h_intern(&lf[242],20, C_text("chicken.base#compose"));
lf[243]=C_h_intern(&lf[243],14, C_text("chicken.base#o"));
lf[244]=C_h_intern(&lf[244],21, C_text("chicken.base#list-of\077"));
lf[245]=C_h_intern(&lf[245],17, C_text("chicken.base#each"));
lf[246]=C_h_intern(&lf[246],21, C_text("##sys#undefined-value"));
lf[247]=C_h_intern(&lf[247],18, C_text("chicken.base#atom\077"));
lf[248]=C_h_intern(&lf[248],18, C_text("chicken.base#tail\077"));
lf[249]=C_h_intern(&lf[249],5, C_text("tail\077"));
lf[250]=C_h_intern(&lf[250],24, C_text("chicken.base#intersperse"));
lf[251]=C_h_intern(&lf[251],20, C_text("chicken.base#butlast"));
lf[252]=C_h_intern(&lf[252],7, C_text("butlast"));
lf[253]=C_h_intern(&lf[253],20, C_text("chicken.base#flatten"));
lf[254]=C_h_intern(&lf[254],17, C_text("chicken.base#join"));
lf[255]=C_h_intern(&lf[255],4, C_text("join"));
lf[256]=C_h_intern(&lf[256],12, C_text("##sys#append"));
lf[257]=C_h_intern(&lf[257],21, C_text("chicken.base#compress"));
lf[258]=C_decode_literal(C_heaptop,C_text("\376B\000\000%bad argument type - not a proper list"));
lf[259]=C_h_intern(&lf[259],8, C_text("compress"));
lf[260]=C_h_intern_kw(&lf[260],10, C_text("type-error"));
lf[261]=C_h_intern(&lf[261],26, C_text("chicken.base#alist-update!"));
lf[262]=C_h_intern(&lf[262],25, C_text("chicken.base#alist-update"));
lf[263]=C_h_intern(&lf[263],12, C_text("alist-update"));
lf[264]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021bad argument type"));
lf[265]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021bad argument type"));
lf[266]=C_h_intern(&lf[266],22, C_text("chicken.base#alist-ref"));
lf[267]=C_h_intern(&lf[267],9, C_text("alist-ref"));
lf[268]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021bad argument type"));
lf[269]=C_h_intern(&lf[269],19, C_text("chicken.base#rassoc"));
lf[270]=C_h_intern(&lf[270],6, C_text("rassoc"));
lf[272]=C_h_intern(&lf[272],15, C_text("chicken.fixnum#"));
lf[273]=C_h_intern(&lf[273],35, C_text("chicken.fixnum#most-positive-fixnum"));
lf[274]=C_h_intern(&lf[274],35, C_text("chicken.fixnum#most-negative-fixnum"));
lf[275]=C_h_intern(&lf[275],26, C_text("chicken.fixnum#fixnum-bits"));
lf[276]=C_h_intern(&lf[276],31, C_text("chicken.fixnum#fixnum-precision"));
lf[277]=C_h_intern(&lf[277],18, C_text("chicken.fixnum#fx+"));
lf[278]=C_h_intern(&lf[278],18, C_text("chicken.fixnum#fx-"));
lf[279]=C_h_intern(&lf[279],18, C_text("chicken.fixnum#fx\052"));
lf[280]=C_h_intern(&lf[280],18, C_text("chicken.fixnum#fx="));
lf[281]=C_h_intern(&lf[281],18, C_text("chicken.fixnum#fx>"));
lf[282]=C_h_intern(&lf[282],18, C_text("chicken.fixnum#fx<"));
lf[283]=C_h_intern(&lf[283],19, C_text("chicken.fixnum#fx>="));
lf[284]=C_h_intern(&lf[284],19, C_text("chicken.fixnum#fx<="));
lf[285]=C_h_intern(&lf[285],20, C_text("chicken.fixnum#fxmin"));
lf[286]=C_h_intern(&lf[286],20, C_text("chicken.fixnum#fxmax"));
lf[287]=C_h_intern(&lf[287],20, C_text("chicken.fixnum#fxneg"));
lf[288]=C_h_intern(&lf[288],20, C_text("chicken.fixnum#fxand"));
lf[289]=C_h_intern(&lf[289],20, C_text("chicken.fixnum#fxior"));
lf[290]=C_h_intern(&lf[290],20, C_text("chicken.fixnum#fxxor"));
lf[291]=C_h_intern(&lf[291],20, C_text("chicken.fixnum#fxnot"));
lf[292]=C_h_intern(&lf[292],20, C_text("chicken.fixnum#fxshl"));
lf[293]=C_h_intern(&lf[293],20, C_text("chicken.fixnum#fxshr"));
lf[294]=C_h_intern(&lf[294],21, C_text("chicken.fixnum#fxodd\077"));
lf[295]=C_h_intern(&lf[295],22, C_text("chicken.fixnum#fxeven\077"));
lf[296]=C_h_intern(&lf[296],20, C_text("chicken.fixnum#fxlen"));
lf[297]=C_h_intern(&lf[297],18, C_text("chicken.fixnum#fx/"));
lf[298]=C_h_intern(&lf[298],20, C_text("chicken.fixnum#fxgcd"));
lf[299]=C_h_intern(&lf[299],20, C_text("chicken.fixnum#fxmod"));
lf[300]=C_h_intern(&lf[300],20, C_text("chicken.fixnum#fxrem"));
lf[301]=C_h_intern(&lf[301],19, C_text("chicken.fixnum#fx+\077"));
lf[302]=C_h_intern(&lf[302],19, C_text("chicken.fixnum#fx-\077"));
lf[303]=C_h_intern(&lf[303],19, C_text("chicken.fixnum#fx\052\077"));
lf[304]=C_h_intern(&lf[304],19, C_text("chicken.fixnum#fx/\077"));
lf[305]=C_h_intern(&lf[305],17, C_text("##sys#debug-mode\077"));
lf[306]=C_h_intern(&lf[306],22, C_text("##sys#warnings-enabled"));
lf[307]=C_h_intern(&lf[307],21, C_text("##sys#notices-enabled"));
lf[308]=C_h_intern(&lf[308],20, C_text("chicken.base#warning"));
lf[309]=C_h_intern_kw(&lf[309],7, C_text("warning"));
lf[310]=C_h_intern(&lf[310],19, C_text("chicken.base#notice"));
lf[311]=C_h_intern_kw(&lf[311],6, C_text("notice"));
lf[312]=C_h_intern(&lf[312],28, C_text("chicken.base#enable-warnings"));
lf[313]=C_h_intern(&lf[313],10, C_text("##sys#warn"));
lf[314]=C_h_intern(&lf[314],12, C_text("##sys#notice"));
lf[315]=C_h_intern(&lf[315],8, C_text("##sys#gc"));
lf[316]=C_h_intern(&lf[316],13, C_text("##sys#setslot"));
lf[317]=C_h_intern(&lf[317],14, C_text("##sys#setislot"));
lf[318]=C_h_intern(&lf[318],21, C_text("##sys#allocate-vector"));
lf[319]=C_h_intern(&lf[319],20, C_text("##sys#make-structure"));
lf[320]=C_h_intern(&lf[320],25, C_text("##sys#ensure-heap-reserve"));
lf[321]=C_h_intern(&lf[321],23, C_text("##sys#symbol-table-info"));
lf[322]=C_h_intern(&lf[322],17, C_text("##sys#memory-info"));
lf[323]=C_h_intern(&lf[323],17, C_text("##sys#start-timer"));
lf[324]=C_h_intern(&lf[324],16, C_text("##sys#stop-timer"));
lf[325]=C_h_intern(&lf[325],16, C_text("##sys#immediate\077"));
lf[326]=C_h_intern(&lf[326],13, C_text("##sys#message"));
lf[327]=C_h_intern(&lf[327],10, C_text("##sys#byte"));
lf[328]=C_h_intern(&lf[328],13, C_text("##sys#setbyte"));
lf[329]=C_h_intern(&lf[329],10, C_text("##sys#void"));
lf[330]=C_h_intern(&lf[330],10, C_text("##sys#halt"));
lf[331]=C_h_intern(&lf[331],13, C_text("##sys#become!"));
lf[332]=C_h_intern(&lf[332],15, C_text("##sys#block-ref"));
lf[333]=C_h_intern(&lf[333],18, C_text("##sys#apply-values"));
lf[334]=C_h_intern(&lf[334],18, C_text("##sys#copy-closure"));
lf[335]=C_h_intern(&lf[335],16, C_text("##sys#block-set!"));
lf[336]=C_h_intern(&lf[336],17, C_text("##sys#check-range"));
lf[337]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023slot not accessible"));
lf[338]=C_h_intern(&lf[338],13, C_text("chicken.time#"));
lf[339]=C_h_intern(&lf[339],33, C_text("chicken.time#current-milliseconds"));
lf[340]=C_h_intern(&lf[340],41, C_text("chicken.time#current-process-milliseconds"));
lf[341]=C_h_intern(&lf[341],28, C_text("chicken.time#current-seconds"));
lf[342]=C_h_intern(&lf[342],21, C_text("chicken.time#cpu-time"));
lf[343]=C_h_intern(&lf[343],21, C_text("##sys#check-structure"));
lf[344]=C_h_intern(&lf[344],16, C_text("##sys#check-blob"));
lf[345]=C_h_intern(&lf[345],23, C_text("##sys#check-byte-vector"));
lf[346]=C_h_intern(&lf[346],16, C_text("##sys#check-pair"));
lf[347]=C_h_intern(&lf[347],16, C_text("##sys#check-list"));
lf[348]=C_h_intern(&lf[348],18, C_text("##sys#check-string"));
lf[349]=C_h_intern(&lf[349],18, C_text("##sys#check-number"));
lf[350]=C_h_intern(&lf[350],17, C_text("##sys#check-exact"));
lf[351]=C_h_intern(&lf[351],19, C_text("##sys#check-inexact"));
lf[352]=C_h_intern(&lf[352],18, C_text("##sys#check-symbol"));
lf[353]=C_h_intern(&lf[353],19, C_text("##sys#check-keyword"));
lf[354]=C_h_intern(&lf[354],18, C_text("##sys#check-vector"));
lf[355]=C_h_intern(&lf[355],16, C_text("##sys#check-char"));
lf[356]=C_h_intern(&lf[356],19, C_text("##sys#check-boolean"));
lf[357]=C_h_intern(&lf[357],20, C_text("##sys#check-locative"));
lf[358]=C_h_intern(&lf[358],19, C_text("##sys#check-integer"));
lf[359]=C_h_intern(&lf[359],23, C_text("##sys#error-bad-integer"));
lf[360]=C_h_intern(&lf[360],25, C_text("##sys#check-exact-integer"));
lf[361]=C_h_intern(&lf[361],29, C_text("##sys#error-bad-exact-integer"));
lf[362]=C_h_intern(&lf[362],26, C_text("##sys#check-exact-uinteger"));
lf[363]=C_h_intern(&lf[363],30, C_text("##sys#error-bad-exact-uinteger"));
lf[364]=C_h_intern(&lf[364],16, C_text("##sys#check-real"));
lf[365]=C_h_intern(&lf[365],20, C_text("##sys#error-bad-real"));
lf[366]=C_h_intern(&lf[366],16, C_text("##sys#error-hook"));
lf[367]=C_h_intern(&lf[367],19, C_text("##sys#check-special"));
lf[368]=C_decode_literal(C_heaptop,C_text("\376B\000\000-bad argument type - not a pointer-like object"));
lf[369]=C_h_intern(&lf[369],19, C_text("##sys#check-closure"));
lf[370]=C_h_intern(&lf[370],12, C_text("scheme#force"));
lf[371]=C_h_intern(&lf[371],10, C_text("##sys#list"));
lf[372]=C_h_intern(&lf[372],11, C_text("##sys#dload"));
lf[373]=C_h_intern(&lf[373],23, C_text("##sys#set-dlopen-flags!"));
lf[374]=C_h_intern(&lf[374],22, C_text("##sys#error-bad-number"));
lf[375]=C_h_intern(&lf[375],23, C_text("##sys#error-bad-inexact"));
lf[376]=C_h_intern(&lf[376],20, C_text("##sys#error-bad-base"));
lf[377]=C_h_intern(&lf[377],13, C_text("scheme#append"));
lf[378]=C_h_intern(&lf[378],6, C_text("append"));
lf[379]=C_h_intern(&lf[379],18, C_text("##sys#fast-reverse"));
lf[380]=C_h_intern(&lf[380],17, C_text("##sys#make-string"));
lf[381]=C_h_intern(&lf[381],18, C_text("scheme#make-string"));
lf[382]=C_h_intern(&lf[382],11, C_text("make-string"));
lf[383]=C_h_intern_kw(&lf[383],12, C_text("bounds-error"));
lf[384]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020size is negative"));
lf[385]=C_h_intern(&lf[385],19, C_text("scheme#string->list"));
lf[386]=C_h_intern(&lf[386],12, C_text("string->list"));
lf[387]=C_h_intern(&lf[387],18, C_text("##sys#string->list"));
lf[388]=C_h_intern(&lf[388],19, C_text("scheme#list->string"));
lf[389]=C_h_intern(&lf[389],12, C_text("list->string"));
lf[390]=C_h_intern(&lf[390],18, C_text("##sys#list->string"));
lf[391]=C_h_intern(&lf[391],26, C_text("##sys#reverse-list->string"));
lf[392]=C_h_intern(&lf[392],20, C_text("reverse-list->string"));
lf[393]=C_h_intern(&lf[393],19, C_text("scheme#string-fill!"));
lf[394]=C_h_intern(&lf[394],12, C_text("string-fill!"));
lf[395]=C_h_intern(&lf[395],18, C_text("scheme#string-copy"));
lf[396]=C_h_intern(&lf[396],11, C_text("string-copy"));
lf[397]=C_h_intern(&lf[397],16, C_text("scheme#substring"));
lf[398]=C_h_intern(&lf[398],9, C_text("substring"));
lf[399]=C_h_intern(&lf[399],15, C_text("##sys#substring"));
lf[400]=C_h_intern(&lf[400],15, C_text("scheme#string<\077"));
lf[401]=C_h_intern(&lf[401],8, C_text("string<\077"));
lf[402]=C_h_intern(&lf[402],15, C_text("scheme#string>\077"));
lf[403]=C_h_intern(&lf[403],8, C_text("string>\077"));
lf[404]=C_h_intern(&lf[404],16, C_text("scheme#string<=\077"));
lf[405]=C_h_intern(&lf[405],9, C_text("string<=\077"));
lf[406]=C_h_intern(&lf[406],16, C_text("scheme#string>=\077"));
lf[407]=C_h_intern(&lf[407],9, C_text("string>=\077"));
lf[408]=C_h_intern(&lf[408],18, C_text("scheme#string-ci<\077"));
lf[409]=C_h_intern(&lf[409],11, C_text("string-ci<\077"));
lf[410]=C_h_intern(&lf[410],18, C_text("scheme#string-ci>\077"));
lf[411]=C_h_intern(&lf[411],11, C_text("string-ci>\077"));
lf[412]=C_h_intern(&lf[412],19, C_text("scheme#string-ci<=\077"));
lf[413]=C_h_intern(&lf[413],12, C_text("string-ci<=\077"));
lf[414]=C_h_intern(&lf[414],19, C_text("scheme#string-ci>=\077"));
lf[415]=C_h_intern(&lf[415],12, C_text("string-ci>=\077"));
lf[416]=C_h_intern(&lf[416],19, C_text("##sys#string-append"));
lf[417]=C_h_intern(&lf[417],20, C_text("scheme#string-append"));
lf[418]=C_h_intern(&lf[418],13, C_text("string-append"));
lf[419]=C_h_intern(&lf[419],13, C_text("scheme#string"));
lf[420]=C_h_intern(&lf[420],23, C_text("##sys#fragments->string"));
lf[421]=C_h_intern(&lf[421],17, C_text("chicken.base#chop"));
lf[422]=C_h_intern(&lf[422],4, C_text("chop"));
lf[423]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030invalid numeric argument"));
lf[424]=C_h_intern(&lf[424],15, C_text("chicken.flonum#"));
lf[425]=C_h_intern(&lf[425],29, C_text("chicken.flonum#maximum-flonum"));
lf[426]=C_h_intern(&lf[426],29, C_text("chicken.flonum#minimum-flonum"));
lf[427]=C_h_intern(&lf[427],27, C_text("chicken.flonum#flonum-radix"));
lf[428]=C_h_intern(&lf[428],29, C_text("chicken.flonum#flonum-epsilon"));
lf[429]=C_h_intern(&lf[429],31, C_text("chicken.flonum#flonum-precision"));
lf[430]=C_h_intern(&lf[430],39, C_text("chicken.flonum#flonum-decimal-precision"));
lf[431]=C_h_intern(&lf[431],38, C_text("chicken.flonum#flonum-maximum-exponent"));
lf[432]=C_h_intern(&lf[432],38, C_text("chicken.flonum#flonum-minimum-exponent"));
lf[433]=C_h_intern(&lf[433],46, C_text("chicken.flonum#flonum-maximum-decimal-exponent"));
lf[434]=C_h_intern(&lf[434],46, C_text("chicken.flonum#flonum-minimum-decimal-exponent"));
lf[435]=C_h_intern(&lf[435],18, C_text("chicken.flonum#fp+"));
lf[436]=C_h_intern(&lf[436],3, C_text("fp+"));
lf[437]=C_h_intern(&lf[437],18, C_text("chicken.flonum#fp-"));
lf[438]=C_h_intern(&lf[438],3, C_text("fp-"));
lf[439]=C_h_intern(&lf[439],18, C_text("chicken.flonum#fp\052"));
lf[440]=C_h_intern(&lf[440],3, C_text("fp\052"));
lf[441]=C_h_intern(&lf[441],18, C_text("chicken.flonum#fp/"));
lf[442]=C_h_intern(&lf[442],3, C_text("fp/"));
lf[443]=C_h_intern(&lf[443],20, C_text("chicken.flonum#fpgcd"));
lf[444]=C_h_intern(&lf[444],5, C_text("fpgcd"));
lf[445]=C_h_intern(&lf[445],19, C_text("chicken.flonum#fp/\077"));
lf[446]=C_h_intern(&lf[446],4, C_text("fp/\077"));
lf[447]=C_h_intern(&lf[447],18, C_text("chicken.flonum#fp="));
lf[448]=C_h_intern(&lf[448],3, C_text("fp="));
lf[449]=C_h_intern(&lf[449],18, C_text("chicken.flonum#fp>"));
lf[450]=C_h_intern(&lf[450],3, C_text("fp>"));
lf[451]=C_h_intern(&lf[451],18, C_text("chicken.flonum#fp<"));
lf[452]=C_h_intern(&lf[452],3, C_text("fp<"));
lf[453]=C_h_intern(&lf[453],19, C_text("chicken.flonum#fp>="));
lf[454]=C_h_intern(&lf[454],4, C_text("fp>="));
lf[455]=C_h_intern(&lf[455],19, C_text("chicken.flonum#fp<="));
lf[456]=C_h_intern(&lf[456],4, C_text("fp<="));
lf[457]=C_h_intern(&lf[457],20, C_text("chicken.flonum#fpneg"));
lf[458]=C_h_intern(&lf[458],5, C_text("fpneg"));
lf[459]=C_h_intern(&lf[459],20, C_text("chicken.flonum#fpmax"));
lf[460]=C_h_intern(&lf[460],5, C_text("fpmax"));
lf[461]=C_h_intern(&lf[461],20, C_text("chicken.flonum#fpmin"));
lf[462]=C_h_intern(&lf[462],5, C_text("fpmin"));
lf[463]=C_h_intern(&lf[463],22, C_text("chicken.flonum#fpfloor"));
lf[464]=C_h_intern(&lf[464],7, C_text("fpfloor"));
lf[465]=C_h_intern(&lf[465],25, C_text("chicken.flonum#fptruncate"));
lf[466]=C_h_intern(&lf[466],10, C_text("fptruncate"));
lf[467]=C_h_intern(&lf[467],22, C_text("chicken.flonum#fpround"));
lf[468]=C_h_intern(&lf[468],7, C_text("fpround"));
lf[469]=C_h_intern(&lf[469],24, C_text("chicken.flonum#fpceiling"));
lf[470]=C_h_intern(&lf[470],9, C_text("fpceiling"));
lf[471]=C_h_intern(&lf[471],20, C_text("chicken.flonum#fpsin"));
lf[472]=C_h_intern(&lf[472],5, C_text("fpsin"));
lf[473]=C_h_intern(&lf[473],20, C_text("chicken.flonum#fpcos"));
lf[474]=C_h_intern(&lf[474],5, C_text("fpcos"));
lf[475]=C_h_intern(&lf[475],20, C_text("chicken.flonum#fptan"));
lf[476]=C_h_intern(&lf[476],5, C_text("fptan"));
lf[477]=C_h_intern(&lf[477],21, C_text("chicken.flonum#fpasin"));
lf[478]=C_h_intern(&lf[478],6, C_text("fpasin"));
lf[479]=C_h_intern(&lf[479],21, C_text("chicken.flonum#fpacos"));
lf[480]=C_h_intern(&lf[480],6, C_text("fpacos"));
lf[481]=C_h_intern(&lf[481],21, C_text("chicken.flonum#fpatan"));
lf[482]=C_h_intern(&lf[482],6, C_text("fpatan"));
lf[483]=C_h_intern(&lf[483],22, C_text("chicken.flonum#fpatan2"));
lf[484]=C_h_intern(&lf[484],7, C_text("fpatan2"));
lf[485]=C_h_intern(&lf[485],20, C_text("chicken.flonum#fpexp"));
lf[486]=C_h_intern(&lf[486],5, C_text("fpexp"));
lf[487]=C_h_intern(&lf[487],21, C_text("chicken.flonum#fpexpt"));
lf[488]=C_h_intern(&lf[488],6, C_text("fpexpt"));
lf[489]=C_h_intern(&lf[489],20, C_text("chicken.flonum#fplog"));
lf[490]=C_h_intern(&lf[490],5, C_text("fplog"));
lf[491]=C_h_intern(&lf[491],21, C_text("chicken.flonum#fpsqrt"));
lf[492]=C_h_intern(&lf[492],6, C_text("fpsqrt"));
lf[493]=C_h_intern(&lf[493],20, C_text("chicken.flonum#fpabs"));
lf[494]=C_h_intern(&lf[494],5, C_text("fpabs"));
lf[495]=C_h_intern(&lf[495],25, C_text("chicken.flonum#fpinteger\077"));
lf[496]=C_h_intern(&lf[496],10, C_text("fpinteger\077"));
lf[497]=C_h_intern(&lf[497],37, C_text("chicken.flonum#flonum-print-precision"));
lf[498]=C_h_intern(&lf[498],22, C_text("flonum-print-precision"));
lf[499]=C_h_intern(&lf[499],13, C_text("##sys#number\077"));
lf[500]=C_h_intern(&lf[500],14, C_text("##sys#integer\077"));
lf[501]=C_h_intern(&lf[501],12, C_text("##sys#exact\077"));
lf[502]=C_h_intern(&lf[502],14, C_text("##sys#inexact\077"));
lf[504]=C_decode_literal(C_heaptop,C_text("\376U0.0\000"));
lf[505]=C_h_intern(&lf[505],23, C_text("scheme#make-rectangular"));
lf[506]=C_h_intern(&lf[506],16, C_text("make-rectangular"));
lf[507]=C_h_intern(&lf[507],17, C_text("scheme#make-polar"));
lf[508]=C_h_intern(&lf[508],10, C_text("make-polar"));
lf[509]=C_h_intern(&lf[509],16, C_text("scheme#real-part"));
lf[510]=C_h_intern(&lf[510],9, C_text("real-part"));
lf[511]=C_h_intern(&lf[511],16, C_text("scheme#imag-part"));
lf[512]=C_h_intern(&lf[512],9, C_text("imag-part"));
lf[513]=C_h_intern(&lf[513],12, C_text("scheme#angle"));
lf[514]=C_h_intern(&lf[514],5, C_text("angle"));
lf[515]=C_h_intern(&lf[515],16, C_text("scheme#magnitude"));
lf[516]=C_h_intern(&lf[516],11, C_text("scheme#sqrt"));
lf[517]=C_h_intern(&lf[517],9, C_text("magnitude"));
lf[519]=C_h_intern(&lf[519],16, C_text("scheme#numerator"));
lf[520]=C_h_intern(&lf[520],9, C_text("numerator"));
lf[521]=C_h_intern(&lf[521],21, C_text("scheme#inexact->exact"));
lf[522]=C_decode_literal(C_heaptop,C_text("\376B\000\000)bad argument type - not a rational number"));
lf[523]=C_h_intern(&lf[523],18, C_text("scheme#denominator"));
lf[524]=C_h_intern(&lf[524],11, C_text("denominator"));
lf[525]=C_decode_literal(C_heaptop,C_text("\376U1.0\000"));
lf[526]=C_decode_literal(C_heaptop,C_text("\376B\000\000)bad argument type - not a rational number"));
lf[527]=C_h_intern(&lf[527],21, C_text("##sys#extended-signum"));
lf[528]=C_h_intern(&lf[528],6, C_text("signum"));
lf[529]=C_h_intern(&lf[529],9, C_text("##sys#/-2"));
lf[530]=C_h_intern(&lf[530],14, C_text("inexact->exact"));
lf[531]=C_h_intern(&lf[531],19, C_text("##sys#integer-power"));
lf[532]=C_decode_literal(C_heaptop,C_text("\376U2.0\000"));
lf[533]=C_h_intern(&lf[533],11, C_text("scheme#expt"));
lf[534]=C_h_intern(&lf[534],20, C_text("##sys#exact->inexact"));
lf[535]=C_h_intern(&lf[535],20, C_text("##sys#inexact->exact"));
lf[536]=C_h_intern(&lf[536],16, C_text("chicken.bitwise#"));
lf[537]=C_h_intern(&lf[537],27, C_text("chicken.bitwise#bitwise-and"));
lf[538]=C_h_intern(&lf[538],27, C_text("chicken.bitwise#bitwise-ior"));
lf[539]=C_h_intern(&lf[539],27, C_text("chicken.bitwise#bitwise-xor"));
lf[540]=C_h_intern(&lf[540],27, C_text("chicken.bitwise#bitwise-not"));
lf[541]=C_h_intern(&lf[541],28, C_text("chicken.bitwise#bit->boolean"));
lf[542]=C_h_intern(&lf[542],30, C_text("chicken.bitwise#integer-length"));
lf[543]=C_h_intern(&lf[543],32, C_text("chicken.bitwise#arithmetic-shift"));
lf[544]=C_h_intern(&lf[544],8, C_text("scheme#/"));
lf[546]=C_h_intern(&lf[546],1, C_text("/"));
lf[547]=C_h_intern(&lf[547],12, C_text("scheme#floor"));
lf[548]=C_h_intern(&lf[548],5, C_text("floor"));
lf[549]=C_h_intern(&lf[549],14, C_text("scheme#ceiling"));
lf[550]=C_h_intern(&lf[550],7, C_text("ceiling"));
lf[551]=C_h_intern(&lf[551],15, C_text("scheme#truncate"));
lf[552]=C_h_intern(&lf[552],8, C_text("truncate"));
lf[553]=C_h_intern(&lf[553],12, C_text("scheme#round"));
lf[554]=C_h_intern(&lf[554],5, C_text("round"));
lf[556]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\001\000\000\000\000\376\003\000\000\002\376\377\001\000\000\000\001\376\377\016"));
lf[557]=C_h_intern(&lf[557],18, C_text("scheme#rationalize"));
lf[558]=C_h_intern(&lf[558],10, C_text("scheme#max"));
lf[559]=C_h_intern(&lf[559],3, C_text("max"));
lf[560]=C_h_intern(&lf[560],10, C_text("scheme#min"));
lf[561]=C_h_intern(&lf[561],3, C_text("min"));
lf[562]=C_h_intern(&lf[562],10, C_text("scheme#exp"));
lf[563]=C_h_intern(&lf[563],3, C_text("exp"));
lf[564]=C_h_intern(&lf[564],11, C_text("##sys#log-1"));
lf[565]=C_h_intern_kw(&lf[565],16, C_text("arithmetic-error"));
lf[566]=C_h_intern(&lf[566],3, C_text("log"));
lf[567]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033log of exact 0 is undefined"));
lf[568]=C_h_intern(&lf[568],10, C_text("scheme#log"));
lf[569]=C_h_intern(&lf[569],10, C_text("scheme#sin"));
lf[570]=C_h_intern(&lf[570],3, C_text("sin"));
lf[571]=C_decode_literal(C_heaptop,C_text("\376\016\000\000\002\376\377\001\000\000\000\000\376\377\001\000\000\000\001"));
lf[572]=C_decode_literal(C_heaptop,C_text("\376\016\000\000\002\376\377\001\000\000\000\000\376\377\001\000\000\000\002"));
lf[573]=C_h_intern(&lf[573],10, C_text("scheme#cos"));
lf[574]=C_h_intern(&lf[574],3, C_text("cos"));
lf[575]=C_h_intern(&lf[575],10, C_text("scheme#tan"));
lf[576]=C_h_intern(&lf[576],3, C_text("tan"));
lf[577]=C_h_intern(&lf[577],11, C_text("scheme#asin"));
lf[578]=C_h_intern(&lf[578],4, C_text("asin"));
lf[579]=C_decode_literal(C_heaptop,C_text("\376\016\000\000\002\376\377\001\000\000\000\000\376\377\001\377\377\377\377"));
lf[580]=C_h_intern(&lf[580],14, C_text("##sys#sqrt/loc"));
lf[581]=C_decode_literal(C_heaptop,C_text("\376U-1.0\000"));
lf[582]=C_h_intern(&lf[582],11, C_text("scheme#acos"));
lf[583]=C_h_intern(&lf[583],4, C_text("acos"));
lf[584]=C_h_intern(&lf[584],11, C_text("scheme#atan"));
lf[585]=C_h_intern(&lf[585],4, C_text("atan"));
lf[586]=C_h_intern(&lf[586],24, C_text("##sys#exact-integer-sqrt"));
lf[587]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030this should never happen"));
lf[588]=C_h_intern(&lf[588],31, C_text("chicken.base#exact-integer-sqrt"));
lf[589]=C_h_intern(&lf[589],18, C_text("exact-integer-sqrt"));
lf[590]=C_h_intern(&lf[590],4, C_text("sqrt"));
lf[591]=C_h_intern(&lf[591],35, C_text("chicken.base#exact-integer-nth-root"));
lf[592]=C_h_intern(&lf[592],32, C_text("##sys#exact-integer-nth-root/loc"));
lf[593]=C_h_intern(&lf[593],22, C_text("exact-integer-nth-root"));
lf[594]=C_h_intern(&lf[594],4, C_text("expt"));
lf[595]=C_decode_literal(C_heaptop,C_text("\376B\000\0006exponent of exact 0 with complex argument is undefined"));
lf[596]=C_h_intern(&lf[596],9, C_text("##sys#gcd"));
lf[597]=C_h_intern(&lf[597],3, C_text("gcd"));
lf[598]=C_h_intern(&lf[598],10, C_text("scheme#gcd"));
lf[599]=C_h_intern(&lf[599],9, C_text("##sys#lcm"));
lf[600]=C_h_intern(&lf[600],3, C_text("lcm"));
lf[601]=C_h_intern(&lf[601],10, C_text("scheme#lcm"));
lf[602]=C_h_intern(&lf[602],31, C_text("##sys#integer->string/recursive"));
lf[603]=C_h_intern(&lf[603],20, C_text("##sys#number->string"));
lf[604]=C_h_intern(&lf[604],29, C_text("##sys#extended-number->string"));
lf[605]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001/"));
lf[606]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001+"));
lf[607]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[608]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001i"));
lf[609]=C_h_intern(&lf[609],14, C_text("number->string"));
lf[612]=C_h_intern(&lf[612],1, C_text("e"));
lf[613]=C_h_intern(&lf[613],1, C_text("i"));
lf[614]=C_decode_literal(C_heaptop,C_text("\376U+inf.0\000"));
lf[615]=C_decode_literal(C_heaptop,C_text("\376U-inf.0\000"));
lf[616]=C_h_intern(&lf[616],3, C_text("neg"));
lf[617]=C_h_intern(&lf[617],3, C_text("pos"));
lf[618]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005inf.0"));
lf[619]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005nan.0"));
lf[620]=C_h_intern(&lf[620],21, C_text("scheme#string->number"));
lf[621]=C_h_intern(&lf[621],14, C_text("string->number"));
lf[622]=C_h_intern(&lf[622],20, C_text("##sys#string->number"));
lf[623]=C_h_intern(&lf[623],20, C_text("##sys#fixnum->string"));
lf[624]=C_h_intern(&lf[624],20, C_text("##sys#flonum->string"));
lf[625]=C_h_intern(&lf[625],21, C_text("##sys#integer->string"));
lf[626]=C_h_intern(&lf[626],20, C_text("chicken.base#equal=\077"));
lf[627]=C_h_intern(&lf[627],11, C_text("##sys#snafu"));
lf[628]=C_h_intern(&lf[628],11, C_text("##sys#fnord"));
lf[629]=C_h_intern(&lf[629],19, C_text("##sys#intern-symbol"));
lf[630]=C_h_intern(&lf[630],20, C_text("##sys#intern-keyword"));
lf[631]=C_h_intern(&lf[631],22, C_text("##sys#interned-symbol\077"));
lf[632]=C_h_intern(&lf[632],20, C_text("##sys#string->symbol"));
lf[633]=C_h_intern(&lf[633],20, C_text("##sys#symbol->string"));
lf[634]=C_h_intern(&lf[634],21, C_text("scheme#symbol->string"));
lf[635]=C_h_intern(&lf[635],14, C_text("symbol->string"));
lf[636]=C_h_intern(&lf[636],21, C_text("scheme#string->symbol"));
lf[637]=C_h_intern(&lf[637],14, C_text("string->symbol"));
lf[638]=C_h_intern(&lf[638],38, C_text("chicken.base#string->uninterned-symbol"));
lf[639]=C_h_intern(&lf[639],25, C_text("string->uninterned-symbol"));
lf[640]=C_h_intern(&lf[640],19, C_text("chicken.base#gensym"));
lf[641]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001g"));
lf[642]=C_h_intern(&lf[642],6, C_text("gensym"));
lf[643]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042argument is not a string or symbol"));
lf[644]=C_h_intern(&lf[644],26, C_text("chicken.base#symbol-append"));
lf[645]=C_h_intern(&lf[645],13, C_text("symbol-append"));
lf[646]=C_h_intern(&lf[646],16, C_text("chicken.keyword#"));
lf[647]=C_h_intern(&lf[647],24, C_text("chicken.keyword#keyword\077"));
lf[648]=C_h_intern(&lf[648],31, C_text("chicken.keyword#string->keyword"));
lf[649]=C_h_intern(&lf[649],15, C_text("string->keyword"));
lf[650]=C_h_intern(&lf[650],31, C_text("chicken.keyword#keyword->string"));
lf[651]=C_h_intern(&lf[651],15, C_text("keyword->string"));
lf[652]=C_decode_literal(C_heaptop,C_text("\376B\000\000!bad argument type - not a keyword"));
lf[653]=C_h_intern(&lf[653],3, C_text("tag"));
lf[654]=C_h_intern(&lf[654],27, C_text("chicken.keyword#get-keyword"));
lf[655]=C_h_intern(&lf[655],11, C_text("get-keyword"));
lf[656]=C_h_intern(&lf[656],17, C_text("##sys#get-keyword"));
lf[657]=C_h_intern(&lf[657],13, C_text("chicken.blob#"));
lf[658]=C_h_intern(&lf[658],15, C_text("##sys#make-blob"));
lf[659]=C_h_intern(&lf[659],22, C_text("chicken.blob#make-blob"));
lf[660]=C_h_intern(&lf[660],9, C_text("make-blob"));
lf[661]=C_h_intern(&lf[661],18, C_text("chicken.blob#blob\077"));
lf[662]=C_h_intern(&lf[662],22, C_text("chicken.blob#blob-size"));
lf[663]=C_h_intern(&lf[663],9, C_text("blob-size"));
lf[664]=C_h_intern(&lf[664],25, C_text("chicken.blob#string->blob"));
lf[665]=C_h_intern(&lf[665],12, C_text("string->blob"));
lf[666]=C_h_intern(&lf[666],25, C_text("chicken.blob#blob->string"));
lf[667]=C_h_intern(&lf[667],12, C_text("blob->string"));
lf[668]=C_h_intern(&lf[668],19, C_text("chicken.blob#blob=\077"));
lf[669]=C_h_intern(&lf[669],6, C_text("blob=\077"));
lf[670]=C_h_intern(&lf[670],11, C_text("make-vector"));
lf[671]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020size is negative"));
lf[672]=C_h_intern(&lf[672],17, C_text("##sys#make-vector"));
lf[673]=C_h_intern(&lf[673],12, C_text("list->vector"));
lf[674]=C_h_intern(&lf[674],12, C_text("vector->list"));
lf[675]=C_h_intern(&lf[675],12, C_text("vector-fill!"));
lf[676]=C_h_intern(&lf[676],12, C_text("vector-copy!"));
lf[677]=C_decode_literal(C_heaptop,C_text("\376B\000\000)cannot copy vector - count exceeds length"));
lf[678]=C_h_intern(&lf[678],9, C_text("subvector"));
lf[679]=C_h_intern(&lf[679],13, C_text("vector-resize"));
lf[681]=C_h_intern(&lf[681],9, C_text("char-name"));
lf[682]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026invalid character name"));
lf[683]=C_h_intern(&lf[683],36, C_text("##sys#call-with-current-continuation"));
lf[684]=C_h_intern(&lf[684],23, C_text("##sys#call-with-cthulhu"));
lf[685]=C_h_intern(&lf[685],22, C_text("##sys#call-with-values"));
lf[686]=C_h_intern(&lf[686],14, C_text("##sys#for-each"));
lf[687]=C_h_intern(&lf[687],8, C_text("for-each"));
lf[688]=C_h_intern(&lf[688],9, C_text("##sys#map"));
lf[689]=C_h_intern(&lf[689],3, C_text("map"));
lf[690]=C_h_intern(&lf[690],19, C_text("##sys#dynamic-winds"));
lf[691]=C_h_intern(&lf[691],18, C_text("##sys#dynamic-wind"));
lf[693]=C_h_intern(&lf[693],16, C_text("input-port-open\077"));
lf[694]=C_h_intern(&lf[694],17, C_text("output-port-open\077"));
lf[695]=C_h_intern(&lf[695],16, C_text("##sys#check-port"));
lf[696]=C_h_intern(&lf[696],12, C_text("port-closed\077"));
lf[697]=C_h_intern(&lf[697],15, C_text("##sys#make-port"));
lf[698]=C_h_intern(&lf[698],24, C_text("##sys#dispatch-interrupt"));
lf[699]=C_h_intern_kw(&lf[699],10, C_text("file-error"));
lf[700]=C_h_intern(&lf[700],9, C_text("read-char"));
lf[701]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030cannot read from port - "));
lf[702]=C_h_intern(&lf[702],19, C_text("##sys#peek-c-string"));
lf[703]=C_h_intern(&lf[703],18, C_text("##sys#update-errno"));
lf[704]=C_h_intern(&lf[704],9, C_text("peek-char"));
lf[705]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030cannot read from port - "));
lf[706]=C_h_intern(&lf[706],12, C_text("read-string!"));
lf[707]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030cannot read from port - "));
lf[708]=C_h_intern(&lf[708],9, C_text("read-line"));
lf[709]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030cannot read from port - "));
lf[710]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[711]=C_h_intern(&lf[711],23, C_text("##sys#stream-port-class"));
lf[712]=C_h_intern(&lf[712],20, C_text("##sys#open-file-port"));
lf[713]=C_h_intern(&lf[713],20, C_text("##sys#standard-input"));
lf[714]=C_h_intern(&lf[714],21, C_text("##sys#standard-output"));
lf[715]=C_h_intern(&lf[715],20, C_text("##sys#standard-error"));
lf[716]=C_h_intern(&lf[716],22, C_text("##sys#check-input-port"));
lf[717]=C_h_intern(&lf[717],23, C_text("##sys#check-output-port"));
lf[718]=C_h_intern(&lf[718],21, C_text("##sys#check-open-port"));
lf[719]=C_h_intern(&lf[719],18, C_text("current-input-port"));
lf[720]=C_h_intern(&lf[720],19, C_text("current-output-port"));
lf[721]=C_h_intern(&lf[721],18, C_text("current-error-port"));
lf[722]=C_h_intern(&lf[722],15, C_text("##sys#tty-port\077"));
lf[723]=C_h_intern(&lf[723],27, C_text("##sys#peek-unsigned-integer"));
lf[724]=C_h_intern(&lf[724],15, C_text("##sys#port-data"));
lf[725]=C_h_intern(&lf[725],20, C_text("##sys#set-port-data!"));
lf[726]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001r"));
lf[727]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001w"));
lf[728]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[729]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023cannot open file - "));
lf[730]=C_h_intern(&lf[730],6, C_text("stream"));
lf[731]=C_h_intern_kw(&lf[731],6, C_text("binary"));
lf[732]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001b"));
lf[733]=C_h_intern_kw(&lf[733],4, C_text("text"));
lf[734]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[735]=C_h_intern_kw(&lf[735],6, C_text("append"));
lf[736]=C_decode_literal(C_heaptop,C_text("\376B\000\000&cannot use append mode with input file"));
lf[737]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001a"));
lf[738]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid file option"));
lf[739]=C_h_intern(&lf[739],15, C_text("open-input-file"));
lf[740]=C_h_intern(&lf[740],16, C_text("open-output-file"));
lf[741]=C_h_intern(&lf[741],16, C_text("close-input-port"));
lf[742]=C_h_intern(&lf[742],17, C_text("close-output-port"));
lf[743]=C_h_intern(&lf[743],18, C_text("##sys#file-exists\077"));
lf[744]=C_decode_literal(C_heaptop,C_text("\376B\000\000(system error while trying to access file"));
lf[745]=C_h_intern(&lf[745],19, C_text("##sys#make-c-string"));
lf[746]=C_h_intern(&lf[746],18, C_text("##sys#flush-output"));
lf[747]=C_h_intern(&lf[747],12, C_text("flush-output"));
lf[748]=C_h_intern(&lf[748],15, C_text("##sys#port-line"));
lf[749]=C_h_intern(&lf[749],21, C_text("##sys#decorate-lambda"));
lf[750]=C_h_intern(&lf[750],23, C_text("##sys#lambda-decoration"));
lf[751]=C_h_intern(&lf[751],22, C_text("##sys#make-lambda-info"));
lf[752]=C_h_intern(&lf[752],18, C_text("##sys#lambda-info\077"));
lf[753]=C_h_intern(&lf[753],17, C_text("##sys#lambda-info"));
lf[754]=C_h_intern(&lf[754],25, C_text("##sys#lambda-info->string"));
lf[755]=C_h_intern(&lf[755],10, C_text("##sys#read"));
lf[756]=C_h_intern(&lf[756],21, C_text("procedure-information"));
lf[757]=C_h_intern(&lf[757],6, C_text("setter"));
lf[759]=C_h_intern(&lf[759],12, C_text("##sys#setter"));
lf[760]=C_h_intern(&lf[760],18, C_text("getter-with-setter"));
lf[762]=C_h_intern(&lf[762],30, C_text("##sys#current-parameter-vector"));
lf[763]=C_decode_literal(C_heaptop,C_text("\376\000\000\000\000"));
lf[764]=C_h_intern(&lf[764],11, C_text("char-ready\077"));
lf[765]=C_h_intern(&lf[765],17, C_text("##sys#read-char-0"));
lf[766]=C_h_intern(&lf[766],20, C_text("##sys#read-char/port"));
lf[767]=C_h_intern(&lf[767],17, C_text("##sys#peek-char-0"));
lf[768]=C_h_intern(&lf[768],4, C_text("read"));
lf[769]=C_h_intern(&lf[769],28, C_text("##sys#default-read-info-hook"));
lf[770]=C_h_intern(&lf[770],33, C_text("##sys#read-error-with-line-number"));
lf[771]=C_h_intern(&lf[771],22, C_text("##sys#read-prompt-hook"));
lf[772]=C_h_intern(&lf[772],21, C_text("##sys#infix-list-hook"));
lf[773]=C_h_intern(&lf[773],23, C_text("##sys#sharp-number-hook"));
lf[774]=C_h_intern(&lf[774],16, C_text("##sys#read-error"));
lf[775]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032invalid `#...\047 read syntax"));
lf[776]=C_h_intern(&lf[776],24, C_text("##sys#current-read-table"));
lf[777]=C_h_intern(&lf[777],18, C_text("##sys#read-warning"));
lf[778]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006(line "));
lf[779]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002) "));
lf[780]=C_h_intern_kw(&lf[780],12, C_text("syntax-error"));
lf[781]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006(line "));
lf[782]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002) "));
lf[783]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000,\376\003\000\000\002\376\377\012\000\000;\376\003\000\000\002\376\377\012\000\000(\376\003\000\000\002\376\377\012\000\000)\376\003\000\000\002\376\377\012\000\000\047\376\003\000\000\002\376\377\012\000\000\042\376\003\000\000\002\376\377\012\000\000[\376\003\000"
"\000\002\376\377\012\000\000]\376\003\000\000\002\376\377\012\000\000{\376\003\000\000\002\376\377\012\000\000}\376\377\016"));
lf[784]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032unexpected list terminator"));
lf[785]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022reserved character"));
lf[786]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032invalid escape-sequence \047\134"));
lf[787]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\047"));
lf[788]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034unterminated string constant"));
lf[789]=C_h_intern(&lf[789],23, C_text("##sys#char->utf8-string"));
lf[790]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023unterminated string"));
lf[791]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001x"));
lf[792]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022bad surrogate pair"));
lf[793]=C_h_intern(&lf[793],27, C_text("##sys#surrogates->codepoint"));
lf[794]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001u"));
lf[795]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032unpaired escaped surrogate"));
lf[796]=C_h_intern(&lf[796],24, C_text("##sys#unicode-surrogate\077"));
lf[797]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001u"));
lf[798]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032invalid escape (surrogate)"));
lf[799]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001U"));
lf[800]=C_decode_literal(C_heaptop,C_text("\376B\000\0006escaped whitespace, but no newline - collapsing anyway"));
lf[801]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023unterminated string"));
lf[802]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[803]=C_decode_literal(C_heaptop,C_text("\376B\000\000\077undefined escape sequence in string - probably forgot backslash"));
lf[804]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023, starting in line "));
lf[805]=C_h_intern(&lf[805],9, C_text("list-info"));
lf[806]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030list-terminator mismatch"));
lf[807]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021unterminated list"));
lf[808]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027missing list terminator"));
lf[809]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022invalid use of `.\047"));
lf[810]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002.:"));
lf[811]=C_h_intern_kw(&lf[811],6, C_text("suffix"));
lf[812]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[813]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[814]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015missing token"));
lf[815]=C_h_intern(&lf[815],18, C_text("##sys#list->vector"));
lf[816]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025invalid vector syntax"));
lf[817]=C_h_intern(&lf[817],11, C_text("symbol-info"));
lf[818]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[819]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022invalid use of `.\047"));
lf[820]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042unexpected prefix in number syntax"));
lf[821]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025illegal number syntax"));
lf[822]=C_decode_literal(C_heaptop,C_text("\376B\000\000!unexpected end of numeric literal"));
lf[823]=C_decode_literal(C_heaptop,C_text("\376B\000\0000illegal number syntax - invalid exactness prefix"));
lf[824]=C_decode_literal(C_heaptop,C_text("\376B\000\000!unexpected end of numeric literal"));
lf[825]=C_decode_literal(C_heaptop,C_text("\376B\000\000%illegal number syntax - invalid radix"));
lf[826]=C_decode_literal(C_heaptop,C_text("\376B\000\000Eattempt to read expression from something that looks like binary data"));
lf[827]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001:"));
lf[828]=C_decode_literal(C_heaptop,C_text("\376B\000\000\052escaped symbol syntax spans multiple lines"));
lf[829]=C_decode_literal(C_heaptop,C_text("\376B\000\0006unexpected end of file while reading escaped character"));
lf[830]=C_h_intern_kw(&lf[830],6, C_text("prefix"));
lf[831]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027unknown named character"));
lf[832]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032unterminated block-comment"));
lf[833]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002##"));
lf[834]=C_h_intern(&lf[834],5, C_text("quote"));
lf[835]=C_h_intern(&lf[835],10, C_text("quasiquote"));
lf[836]=C_h_intern(&lf[836],16, C_text("unquote-splicing"));
lf[837]=C_h_intern(&lf[837],7, C_text("unquote"));
lf[838]=C_decode_literal(C_heaptop,C_text("\376B\000\000,unexpected end of input after reading #-sign"));
lf[839]=C_decode_literal(C_heaptop,C_text("\376B\000\000%unexpected end of input after reading"));
lf[840]=C_decode_literal(C_heaptop,C_text("\376B\000\000!invalid parameterized read syntax"));
lf[841]=C_decode_literal(C_heaptop,C_text("\376B\000\0006unexpected end of input while reading `#c...\047 sequence"));
lf[842]=C_decode_literal(C_heaptop,C_text("\376B\000\000\052invalid case specifier in `#c...\047 sequence"));
lf[843]=C_h_intern(&lf[843],11, C_text("quasisyntax"));
lf[844]=C_h_intern(&lf[844],29, C_text("##sys#read-bytevector-literal"));
lf[845]=C_h_intern(&lf[845],8, C_text("location"));
lf[846]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015empty keyword"));
lf[847]=C_h_intern(&lf[847],11, C_text("cond-expand"));
lf[848]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\004\001else\376\377\016"));
lf[849]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003eof"));
lf[850]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010optional"));
lf[851]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004rest"));
lf[852]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003key"));
lf[853]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#!"));
lf[854]=C_h_intern(&lf[854],16, C_text("##sys#read-marks"));
lf[855]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022invalid `#!\047 token"));
lf[856]=C_h_intern(&lf[856],20, C_text("##sys#user-read-hook"));
lf[857]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000[\376\003\000\000\002\376\377\012\000\000]\376\003\000\000\002\376\377\012\000\000{\376\003\000\000\002\376\377\012\000\000}\376\377\016"));
lf[858]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000|\376\377\016"));
lf[859]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035UTF-8 codepoint out of range:"));
lf[860]=C_decode_literal(C_heaptop,C_text("\376B\000\000 invalid hex-code in blob-literal"));
lf[861]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036unexpected end of blob literal"));
lf[862]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036invalid sharp-sign read syntax"));
lf[863]=C_h_intern(&lf[863],18, C_text("##sys#write-char-0"));
lf[864]=C_h_intern(&lf[864],21, C_text("##sys#write-char/port"));
lf[865]=C_h_intern(&lf[865],10, C_text("write-char"));
lf[866]=C_h_intern(&lf[866],5, C_text("write"));
lf[867]=C_h_intern(&lf[867],11, C_text("##sys#print"));
lf[868]=C_h_intern(&lf[868],7, C_text("display"));
lf[869]=C_h_intern(&lf[869],5, C_text("print"));
lf[871]=C_h_intern(&lf[871],24, C_text("##sys#print-length-limit"));
lf[873]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003..."));
lf[874]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003..."));
lf[875]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\134x"));
lf[876]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\012\000\000(\376\003\000\000\002\376\377\012\000\000)\376\003\000\000\002\376\377\012\000\000,\376\003\000\000\002\376\377\012\000\000[\376\003\000\000\002\376\377\012\000\000]\376\003\000\000\002\376\377\012\000\000{\376\003\000\000\002\376\377\012\000\000}\376\003\000"
"\000\002\376\377\012\000\000\047\376\003\000\000\002\376\377\012\000\000\042\376\003\000\000\002\376\377\012\000\000;\376\003\000\000\002\376\377\012\000\000 \376\003\000\000\002\376\377\012\000\000`\376\003\000\000\002\376\377\012\000\000|\376\003\000\000\002\376\377\012\000\000\134\376\377\016"));
lf[877]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376B\000\000\006#!rest\376\003\000\000\002\376B\000\000\005#!key\376\003\000\000\002\376B\000\000\012#!optional\376\377\016"));
lf[878]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002()"));
lf[879]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#t"));
lf[880]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#f"));
lf[881]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005#!eof"));
lf[882]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016#<unspecified>"));
lf[883]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#\134"));
lf[884]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020#<unbound value>"));
lf[885]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033#<invalid immediate object>"));
lf[886]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033#<invalid forwarded object>"));
lf[887]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#:"));
lf[888]=C_h_intern(&lf[888],21, C_text("##sys#pointer->string"));
lf[889]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\134\042"));
lf[890]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\134\134"));
lf[891]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003 . "));
lf[892]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003#${"));
lf[893]=C_h_intern(&lf[893],21, C_text("##sys#user-print-hook"));
lf[894]=C_h_intern(&lf[894],23, C_text("##sys#procedure->string"));
lf[895]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013#<locative>"));
lf[896]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016#<lambda info "));
lf[897]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\042>"));
lf[898]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016#<input port \042"));
lf[899]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017#<output port \042"));
lf[900]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010#<port \042"));
lf[901]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003#()"));
lf[902]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#("));
lf[903]=C_decode_literal(C_heaptop,C_text("\376B\000\000$unprintable block object encountered"));
lf[904]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014#<procedure "));
lf[905]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001>"));
lf[906]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014#<procedure>"));
lf[907]=C_h_intern(&lf[907],21, C_text("##sys#record-printers"));
lf[908]=C_h_intern(&lf[908],14, C_text("record-printer"));
lf[909]=C_h_intern(&lf[909],19, C_text("set-record-printer!"));
lf[910]=C_h_intern(&lf[910],29, C_text("##sys#register-record-printer"));
lf[911]=C_h_intern(&lf[911],9, C_text("condition"));
lf[912]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\007\001message"));
lf[913]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\047: "));
lf[914]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\047>"));
lf[915]=C_decode_literal(C_heaptop,C_text("\376B\000\000##<Error in printer of record type `"));
lf[916]=C_h_intern(&lf[916],40, C_text("chicken.condition#with-exception-handler"));
lf[917]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[918]=C_h_intern(&lf[918],6, C_text("thread"));
lf[919]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[920]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002#<"));
lf[921]=C_h_intern(&lf[921],29, C_text("##sys#with-print-length-limit"));
lf[922]=C_h_intern(&lf[922],16, C_text("##sys#copy-bytes"));
lf[923]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022string buffer full"));
lf[924]=C_h_intern(&lf[924],22, C_text("##sys#scan-buffer-line"));
lf[925]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[926]=C_h_intern(&lf[926],23, C_text("##sys#string-port-class"));
lf[927]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\015"));
lf[928]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\015"));
lf[929]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[930]=C_h_intern(&lf[930],17, C_text("open-input-string"));
lf[931]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(string)"));
lf[932]=C_h_intern(&lf[932],6, C_text("string"));
lf[933]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(string)"));
lf[934]=C_h_intern(&lf[934],17, C_text("get-output-string"));
lf[935]=C_decode_literal(C_heaptop,C_text("\376B\000\000$argument is not a string-output-port"));
lf[936]=C_h_intern(&lf[936],21, C_text("##sys#print-to-string"));
lf[937]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021#<tagged pointer "));
lf[938]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[939]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001>"));
lf[940]=C_h_intern(&lf[940],22, C_text("##sys#pointer->address"));
lf[941]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014#<pointer 0x"));
lf[942]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001>"));
lf[943]=C_h_intern(&lf[943],20, C_text("##sys#current-thread"));
lf[944]=C_h_intern(&lf[944],27, C_text("##sys#peek-nonnull-c-string"));
lf[945]=C_h_intern(&lf[945],29, C_text("##sys#really-print-call-chain"));
lf[946]=C_h_intern(&lf[946],9, C_text("frameinfo"));
lf[947]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002] "));
lf[948]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003\011  "));
lf[949]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002\012\011"));
lf[950]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005\011<--\012"));
lf[951]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020\012\011Call history:\012"));
lf[952]=C_h_intern(&lf[952],16, C_text("print-call-chain"));
lf[953]=C_h_intern(&lf[953],25, C_text("##sys#user-interrupt-hook"));
lf[954]=C_h_intern_kw(&lf[954],14, C_text("user-interrupt"));
lf[955]=C_h_intern(&lf[955],23, C_text("##sys#primordial-thread"));
lf[958]=C_h_intern(&lf[958],22, C_text("##sys#force-finalizers"));
lf[959]=C_h_intern(&lf[959],27, C_text("chicken.gc#force-finalizers"));
lf[960]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036[debug] forcing finalizers...\012"));
lf[961]=C_h_intern(&lf[961],24, C_text("##sys#pending-finalizers"));
lf[962]=C_h_intern(&lf[962],28, C_text("##sys#run-pending-finalizers"));
lf[963]=C_h_intern(&lf[963],21, C_text("##sys#dump-heap-state"));
lf[964]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[965]=C_h_intern(&lf[965],19, C_text("##sys#reset-handler"));
lf[966]=C_h_intern(&lf[966],18, C_text("chicken.condition#"));
lf[967]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\016\001user-interrupt\376\377\016"));
lf[968]=C_h_intern(&lf[968],23, C_text("chicken.condition#abort"));
lf[969]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[970]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012\012Warning: "));
lf[971]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007\012Note: "));
lf[972]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\007\001message"));
lf[973]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\011\001arguments"));
lf[974]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\012\001call-chain"));
lf[975]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\010\001location"));
lf[976]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\004\001type\376\377\016"));
lf[977]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\006\001syntax\376\377\016"));
lf[978]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\006\001bounds\376\377\016"));
lf[979]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\012\001arithmetic\376\377\016"));
lf[980]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\003\001i/o\376\003\000\000\002\376\001\000\000\004\001file\376\377\016"));
lf[981]=C_h_intern_kw(&lf[981],13, C_text("runtime-error"));
lf[982]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\007\001runtime\376\377\016"));
lf[983]=C_h_intern_kw(&lf[983],13, C_text("process-error"));
lf[984]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\007\001process\376\377\016"));
lf[985]=C_h_intern_kw(&lf[985],13, C_text("network-error"));
lf[986]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\003\001i/o\376\003\000\000\002\376\001\000\000\003\001net\376\377\016"));
lf[987]=C_h_intern_kw(&lf[987],21, C_text("network-timeout-error"));
lf[988]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\003\001i/o\376\003\000\000\002\376\001\000\000\003\001net\376\003\000\000\002\376\001\000\000\007\001timeout\376\377\016"));
lf[989]=C_h_intern_kw(&lf[989],11, C_text("limit-error"));
lf[990]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\007\001runtime\376\003\000\000\002\376\001\000\000\005\001limit\376\377\016"));
lf[991]=C_h_intern_kw(&lf[991],11, C_text("arity-error"));
lf[992]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\005\001arity\376\377\016"));
lf[993]=C_h_intern_kw(&lf[993],12, C_text("access-error"));
lf[994]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\006\001access\376\377\016"));
lf[995]=C_h_intern_kw(&lf[995],12, C_text("domain-error"));
lf[996]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\006\001domain\376\377\016"));
lf[997]=C_h_intern_kw(&lf[997],12, C_text("memory-error"));
lf[998]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\003\000\000\002\376\001\000\000\006\001memory\376\377\016"));
lf[999]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\377\016"));
lf[1000]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\007\001message"));
lf[1001]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032exception handler returned"));
lf[1002]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\011\001arguments"));
lf[1003]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\010\001location"));
lf[1004]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\377\016"));
lf[1005]=C_h_intern(&lf[1005],31, C_text("##sys#current-exception-handler"));
lf[1006]=C_h_intern(&lf[1006],24, C_text("chicken.condition#signal"));
lf[1007]=C_h_intern(&lf[1007],20, C_text("##sys#break-on-error"));
lf[1008]=C_h_intern(&lf[1008],19, C_text("##sys#error-handler"));
lf[1009]=C_h_intern(&lf[1009],20, C_text("##sys#last-exception"));
lf[1010]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\022\001uncaught-exception\376\001\000\000\006\001reason"));
lf[1011]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\022\001uncaught-exception\376\377\016"));
lf[1012]=C_h_intern(&lf[1012],3, C_text("exn"));
lf[1013]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\007\001message"));
lf[1014]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\011\001arguments"));
lf[1015]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001exn\376\001\000\000\010\001location"));
lf[1016]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[1017]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002) "));
lf[1018]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1019]=C_decode_literal(C_heaptop,C_text("\376B\000\000 <exn: has no `message\047 property>"));
lf[1020]=C_h_intern(&lf[1020],14, C_text("user-interrupt"));
lf[1021]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030\012\052\052\052 user interrupt \052\052\052\012"));
lf[1022]=C_h_intern(&lf[1022],18, C_text("uncaught-exception"));
lf[1023]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\022\001uncaught-exception\376\001\000\000\006\001reason"));
lf[1024]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022uncaught exception"));
lf[1025]=C_h_intern(&lf[1025],43, C_text("chicken.condition#current-exception-handler"));
lf[1026]=C_h_intern(&lf[1026],25, C_text("current-exception-handler"));
lf[1028]=C_decode_literal(C_heaptop,C_text("\376B\000\000%argument is not an even property list"));
lf[1029]=C_h_intern(&lf[1029],41, C_text("chicken.condition#make-property-condition"));
lf[1030]=C_h_intern(&lf[1030],23, C_text("make-property-condition"));
lf[1031]=C_h_intern(&lf[1031],42, C_text("chicken.condition#make-composite-condition"));
lf[1032]=C_h_intern(&lf[1032],24, C_text("make-composite-condition"));
lf[1033]=C_h_intern(&lf[1033],27, C_text("chicken.condition#condition"));
lf[1034]=C_h_intern(&lf[1034],28, C_text("chicken.condition#condition\077"));
lf[1035]=C_h_intern(&lf[1035],33, C_text("chicken.condition#condition->list"));
lf[1036]=C_h_intern(&lf[1036],15, C_text("condition->list"));
lf[1037]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042argument is not a condition object"));
lf[1038]=C_h_intern(&lf[1038],37, C_text("chicken.condition#condition-predicate"));
lf[1039]=C_h_intern(&lf[1039],45, C_text("chicken.condition#condition-property-accessor"));
lf[1040]=C_h_intern(&lf[1040],27, C_text("condition-property-accessor"));
lf[1041]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036condition has no such property"));
lf[1042]=C_h_intern(&lf[1042],40, C_text("chicken.condition#get-condition-property"));
lf[1043]=C_h_intern(&lf[1043],37, C_text("chicken.condition#print-error-message"));
lf[1044]=C_h_intern(&lf[1044],19, C_text("print-error-message"));
lf[1045]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1046]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001("));
lf[1047]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002) "));
lf[1048]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1049]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\016\001user-interrupt\376\377\016"));
lf[1050]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030: \052\052\052 user interrupt \052\052\052"));
lf[1051]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016: <condition> "));
lf[1052]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1053]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026: uncaught exception: "));
lf[1054]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005Error"));
lf[1055]=C_h_intern(&lf[1055],28, C_text("##sys#show-exception-warning"));
lf[1056]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1057]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002 ("));
lf[1058]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007Warning"));
lf[1059]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument count - received "));
lf[1060]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016 but expected "));
lf[1061]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035too few arguments - received "));
lf[1062]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016 but expected "));
lf[1063]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021bad argument type"));
lf[1064]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020unbound variable"));
lf[1065]=C_decode_literal(C_heaptop,C_text("\376B\000\000!bad argument type - not a keyword"));
lf[1066]=C_decode_literal(C_heaptop,C_text("\376B\000\000\015out of memory"));
lf[1067]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020division by zero"));
lf[1068]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014out of range"));
lf[1069]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025call of non-procedure"));
lf[1070]=C_decode_literal(C_heaptop,C_text("\376B\000\000+continuation cannot receive multiple values"));
lf[1071]=C_decode_literal(C_heaptop,C_text("\376B\000\000)bad argument type - not a non-cyclic list"));
lf[1072]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022recursion too deep"));
lf[1073]=C_decode_literal(C_heaptop,C_text("\376B\000\0007inexact number cannot be represented as an exact number"));
lf[1074]=C_decode_literal(C_heaptop,C_text("\376B\000\000%bad argument type - not a proper list"));
lf[1075]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a fixnum"));
lf[1076]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a number"));
lf[1077]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a string"));
lf[1078]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument type - not a pair"));
lf[1079]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument type - not a list"));
lf[1080]=C_decode_literal(C_heaptop,C_text("\376B\000\000#bad argument type - not a character"));
lf[1081]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a vector"));
lf[1082]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a symbol"));
lf[1083]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016stack overflow"));
lf[1084]=C_decode_literal(C_heaptop,C_text("\376B\000\0008bad argument type - not a structure of the required type"));
lf[1085]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument type - not a blob"));
lf[1086]=C_decode_literal(C_heaptop,C_text("\376B\000\000#locative refers to reclaimed object"));
lf[1087]=C_decode_literal(C_heaptop,C_text("\376B\000\000&bad argument type - not a block object"));
lf[1088]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047bad argument type - not a number vector"));
lf[1089]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042bad argument type - not an integer"));
lf[1090]=C_decode_literal(C_heaptop,C_text("\376B\000\000+bad argument type - not an unsigned integer"));
lf[1091]=C_decode_literal(C_heaptop,C_text("\376B\000\000!bad argument type - not a pointer"));
lf[1092]=C_decode_literal(C_heaptop,C_text("\376B\000\000(bad argument type - not a tagged pointer"));
lf[1093]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - not a flonum"));
lf[1094]=C_decode_literal(C_heaptop,C_text("\376B\000\000#bad argument type - not a procedure"));
lf[1095]=C_decode_literal(C_heaptop,C_text("\376B\000\000 bad argument type - invalid base"));
lf[1096]=C_decode_literal(C_heaptop,C_text("\376B\000\000/recursion too deep or circular data encountered"));
lf[1097]=C_decode_literal(C_heaptop,C_text("\376B\000\000!bad argument type - not a boolean"));
lf[1098]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042bad argument type - not a locative"));
lf[1099]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument type - not a port"));
lf[1100]=C_decode_literal(C_heaptop,C_text("\376B\000\0002bad argument type - not a port of the correct type"));
lf[1101]=C_decode_literal(C_heaptop,C_text("\376B\000\000%bad argument type - not an input-port"));
lf[1102]=C_decode_literal(C_heaptop,C_text("\376B\000\000&bad argument type - not an output-port"));
lf[1103]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023port already closed"));
lf[1104]=C_decode_literal(C_heaptop,C_text("\376B\000\0002cannot represent string with NUL bytes as C string"));
lf[1105]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026segmentation violation"));
lf[1106]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030floating-point exception"));
lf[1107]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023illegal instruction"));
lf[1108]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011bus error"));
lf[1109]=C_decode_literal(C_heaptop,C_text("\376B\000\000\047bad argument type - not an exact number"));
lf[1110]=C_decode_literal(C_heaptop,C_text("\376B\000\000)bad argument type - not an inexact number"));
lf[1111]=C_decode_literal(C_heaptop,C_text("\376B\000\000\036bad argument type - not a real"));
lf[1112]=C_decode_literal(C_heaptop,C_text("\376B\000\0002bad argument type - complex number has no ordering"));
lf[1113]=C_decode_literal(C_heaptop,C_text("\376B\000\000(bad argument type - not an exact integer"));
lf[1114]=C_decode_literal(C_heaptop,C_text("\376B\000\000#number does not fit in foreign type"));
lf[1115]=C_decode_literal(C_heaptop,C_text("\376B\000\000/cannot compute absolute value of complex number"));
lf[1116]=C_decode_literal(C_heaptop,C_text("\376B\000\000(attempted rest argument access at index "));
lf[1117]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031 but rest list length is "));
lf[1118]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026unknown internal error"));
lf[1119]=C_h_intern(&lf[1119],16, C_text("##sys#structure\077"));
lf[1120]=C_h_intern(&lf[1120],24, C_text("##sys#generic-structure\077"));
lf[1121]=C_h_intern(&lf[1121],10, C_text("##sys#slot"));
lf[1122]=C_h_intern(&lf[1122],10, C_text("##sys#size"));
lf[1123]=C_h_intern(&lf[1123],18, C_text("##sys#make-pointer"));
lf[1124]=C_h_intern(&lf[1124],25, C_text("##sys#make-tagged-pointer"));
lf[1125]=C_h_intern(&lf[1125],14, C_text("##sys#pointer\077"));
lf[1126]=C_h_intern(&lf[1126],26, C_text("##sys#set-pointer-address!"));
lf[1127]=C_h_intern(&lf[1127],17, C_text("##sys#bytevector\077"));
lf[1128]=C_h_intern(&lf[1128],25, C_text("##sys#string->pbytevector"));
lf[1129]=C_h_intern(&lf[1129],16, C_text("##sys#permanent\077"));
lf[1130]=C_h_intern(&lf[1130],19, C_text("##sys#block-address"));
lf[1131]=C_h_intern(&lf[1131],15, C_text("##sys#locative\077"));
lf[1132]=C_h_intern(&lf[1132],20, C_text("##sys#srfi-4-vector\077"));
lf[1133]=C_h_intern(&lf[1133],18, C_text("##sys#null-pointer"));
lf[1134]=C_h_intern(&lf[1134],19, C_text("##sys#null-pointer\077"));
lf[1135]=C_h_intern(&lf[1135],22, C_text("##sys#address->pointer"));
lf[1136]=C_h_intern(&lf[1136],25, C_text("##sys#peek-signed-integer"));
lf[1137]=C_h_intern(&lf[1137],17, C_text("##sys#peek-fixnum"));
lf[1138]=C_h_intern(&lf[1138],15, C_text("##sys#peek-byte"));
lf[1139]=C_h_intern(&lf[1139],24, C_text("##sys#vector->structure!"));
lf[1140]=C_h_intern(&lf[1140],17, C_text("##sys#peek-double"));
lf[1141]=C_h_intern(&lf[1141],28, C_text("##sys#peek-and-free-c-string"));
lf[1142]=C_h_intern(&lf[1142],36, C_text("##sys#peek-and-free-nonnull-c-string"));
lf[1143]=C_h_intern(&lf[1143],19, C_text("##sys#poke-c-string"));
lf[1144]=C_h_intern(&lf[1144],18, C_text("##sys#poke-integer"));
lf[1145]=C_h_intern(&lf[1145],17, C_text("##sys#poke-double"));
lf[1146]=C_h_intern(&lf[1146],24, C_text("##sys#peek-c-string-list"));
lf[1147]=C_h_intern(&lf[1147],33, C_text("##sys#peek-and-free-c-string-list"));
lf[1148]=C_h_intern(&lf[1148],22, C_text("##sys#vector->closure!"));
lf[1149]=C_h_intern(&lf[1149],34, C_text("##sys#symbol-has-toplevel-binding\077"));
lf[1150]=C_h_intern(&lf[1150],16, C_text("##sys#copy-words"));
lf[1151]=C_h_intern(&lf[1151],19, C_text("##sys#compare-bytes"));
lf[1152]=C_h_intern(&lf[1152],19, C_text("##sys#block-pointer"));
lf[1153]=C_h_intern(&lf[1153],27, C_text("##sys#foreign-char-argument"));
lf[1154]=C_h_intern(&lf[1154],29, C_text("##sys#foreign-fixnum-argument"));
lf[1155]=C_h_intern(&lf[1155],29, C_text("##sys#foreign-flonum-argument"));
lf[1156]=C_h_intern(&lf[1156],28, C_text("##sys#foreign-block-argument"));
lf[1157]=C_h_intern(&lf[1157],37, C_text("##sys#foreign-struct-wrapper-argument"));
lf[1158]=C_h_intern(&lf[1158],29, C_text("##sys#foreign-string-argument"));
lf[1159]=C_h_intern(&lf[1159],29, C_text("##sys#foreign-symbol-argument"));
lf[1160]=C_h_intern(&lf[1160],30, C_text("##sys#foreign-pointer-argument"));
lf[1161]=C_h_intern(&lf[1161],37, C_text("##sys#foreign-tagged-pointer-argument"));
lf[1162]=C_h_intern(&lf[1162],37, C_text("##sys#foreign-ranged-integer-argument"));
lf[1163]=C_h_intern(&lf[1163],46, C_text("##sys#foreign-unsigned-ranged-integer-argument"));
lf[1164]=C_h_intern(&lf[1164],28, C_text("##sys#default-thread-quantum"));
lf[1165]=C_h_intern(&lf[1165],31, C_text("##sys#default-exception-handler"));
lf[1166]=C_decode_literal(C_heaptop,C_text("\376B\000\000>internal error: default exception handler shouldn\047t be called!"));
lf[1167]=C_h_intern(&lf[1167],17, C_text("##sys#make-thread"));
lf[1168]=C_h_intern(&lf[1168],16, C_text("##sys#make-mutex"));
lf[1169]=C_h_intern(&lf[1169],5, C_text("mutex"));
lf[1170]=C_h_intern(&lf[1170],14, C_text("##sys#schedule"));
lf[1171]=C_h_intern(&lf[1171],19, C_text("##sys#thread-yield!"));
lf[1172]=C_h_intern(&lf[1172],24, C_text("##sys#kill-other-threads"));
lf[1173]=C_h_intern(&lf[1173],23, C_text("chicken.base#sleep-hook"));
lf[1174]=C_h_intern(&lf[1174],5, C_text("sleep"));
lf[1175]=C_h_intern(&lf[1175],20, C_text("##sys#context-switch"));
lf[1176]=C_h_intern(&lf[1176],19, C_text("##sys#signal-vector"));
lf[1177]=C_h_intern(&lf[1177],20, C_text("##sys#interrupt-hook"));
lf[1178]=C_h_intern(&lf[1178],11, C_text("##sys#errno"));
lf[1179]=C_decode_literal(C_heaptop,C_text("\376B\000\000Lunexpected end of file - unterminated `#{...}\047 item in `here\047 string litera"
"l"));
lf[1180]=C_decode_literal(C_heaptop,C_text("\376B\000\000&unterminated here-doc string literal `"));
lf[1181]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\047"));
lf[1182]=C_decode_literal(C_heaptop,C_text("\376B\000\000$Missing tag after #<< here-doc token"));
lf[1183]=C_decode_literal(C_heaptop,C_text("\376B\000\000!Whitespace after #<< here-doc tag"));
lf[1184]=C_h_intern(&lf[1184],4, C_text("cons"));
lf[1185]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001quote\376\003\000\000\002\376\377\016\376\377\016"));
lf[1186]=C_decode_literal(C_heaptop,C_text("\376B\000\000$Missing tag after #<# here-doc token"));
lf[1187]=C_decode_literal(C_heaptop,C_text("\376B\000\000!Whitespace after #<# here-doc tag"));
lf[1188]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021unreadable object"));
lf[1189]=C_h_intern(&lf[1189],24, C_text("chicken.process-context#"));
lf[1190]=C_h_intern(&lf[1190],40, C_text("chicken.process-context#change-directory"));
lf[1191]=C_h_intern(&lf[1191],16, C_text("change-directory"));
lf[1192]=C_decode_literal(C_heaptop,C_text("\376B\000\000\042cannot change current directory - "));
lf[1193]=C_h_intern(&lf[1193],27, C_text("##sys#change-directory-hook"));
lf[1194]=C_h_intern(&lf[1194],41, C_text("chicken.process-context#current-directory"));
lf[1195]=C_h_intern(&lf[1195],48, C_text("chicken.process-context#get-environment-variable"));
lf[1196]=C_h_intern(&lf[1196],49, C_text("chicken.process-context#set-environment-variable!"));
lf[1197]=C_h_intern(&lf[1197],25, C_text("set-environment-variable!"));
lf[1198]=C_h_intern(&lf[1198],51, C_text("chicken.process-context#unset-environment-variable!"));
lf[1199]=C_h_intern(&lf[1199],27, C_text("unset-environment-variable!"));
lf[1200]=C_h_intern(&lf[1200],49, C_text("chicken.process-context#get-environment-variables"));
lf[1201]=C_h_intern(&lf[1201],43, C_text("chicken.process-context#executable-pathname"));
lf[1202]=C_h_intern(&lf[1202],33, C_text("chicken.process-context#argc+argv"));
lf[1203]=C_h_intern(&lf[1203],28, C_text("chicken.process-context#argv"));
lf[1204]=C_h_intern(&lf[1204],36, C_text("chicken.process-context#program-name"));
lf[1205]=C_h_intern(&lf[1205],46, C_text("chicken.process-context#command-line-arguments"));
lf[1206]=C_h_intern(&lf[1206],11, C_text("chicken.gc#"));
lf[1207]=C_h_intern(&lf[1207],34, C_text("chicken.gc#current-gc-milliseconds"));
lf[1208]=C_h_intern(&lf[1208],25, C_text("chicken.gc#set-gc-report!"));
lf[1209]=C_h_intern(&lf[1209],28, C_text("chicken.gc#memory-statistics"));
lf[1210]=C_h_intern(&lf[1210],20, C_text("##sys#set-finalizer!"));
lf[1211]=C_h_intern(&lf[1211],25, C_text("chicken.gc#set-finalizer!"));
lf[1212]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035[debug] too many finalizers ("));
lf[1213]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035), resized max finalizers to "));
lf[1214]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[1215]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035[debug] too many finalizers ("));
lf[1216]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017), forcing ...\012"));
lf[1217]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014in finalizer"));
lf[1218]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020[debug] running "));
lf[1219]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017 finalizer(s) ("));
lf[1220]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007 live, "));
lf[1221]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020 allocated) ...\012"));
lf[1222]=C_decode_literal(C_heaptop,C_text("\376B\000\000Jre-entry from finalizer thread (maybe (gc #t) was called from a finalizer)"));
lf[1223]=C_h_intern(&lf[1223],13, C_text("chicken.gc#gc"));
lf[1224]=C_h_intern(&lf[1224],12, C_text("##sys#length"));
lf[1225]=C_h_intern(&lf[1225],10, C_text("##sys#cons"));
lf[1226]=C_h_intern(&lf[1226],12, C_text("##sys#vector"));
lf[1227]=C_h_intern(&lf[1227],11, C_text("##sys#apply"));
lf[1228]=C_h_intern(&lf[1228],12, C_text("##sys#values"));
lf[1229]=C_h_intern(&lf[1229],12, C_text("##sys#equal\077"));
lf[1230]=C_h_intern(&lf[1230],9, C_text("##sys#car"));
lf[1231]=C_h_intern(&lf[1231],9, C_text("##sys#cdr"));
lf[1232]=C_h_intern(&lf[1232],11, C_text("##sys#pair\077"));
lf[1233]=C_h_intern(&lf[1233],13, C_text("##sys#vector\077"));
lf[1234]=C_h_intern(&lf[1234],18, C_text("##sys#vector->list"));
lf[1235]=C_h_intern(&lf[1235],19, C_text("##sys#vector-length"));
lf[1236]=C_h_intern(&lf[1236],16, C_text("##sys#vector-ref"));
lf[1237]=C_h_intern(&lf[1237],8, C_text("##sys#>="));
lf[1238]=C_h_intern(&lf[1238],7, C_text("##sys#="));
lf[1239]=C_h_intern(&lf[1239],7, C_text("##sys#+"));
lf[1240]=C_h_intern(&lf[1240],9, C_text("##sys#eq\077"));
lf[1241]=C_h_intern(&lf[1241],10, C_text("##sys#eqv\077"));
lf[1242]=C_h_intern(&lf[1242],11, C_text("##sys#list\077"));
lf[1243]=C_h_intern(&lf[1243],11, C_text("##sys#null\077"));
lf[1244]=C_h_intern(&lf[1244],11, C_text("##sys#map-n"));
lf[1245]=C_h_intern(&lf[1245],19, C_text("##sys#make-locative"));
lf[1246]=C_decode_literal(C_heaptop,C_text("\376B\000\000)locative cannot refer to immediate object"));
lf[1247]=C_h_intern(&lf[1247],8, C_text("u8vector"));
lf[1248]=C_h_intern(&lf[1248],8, C_text("s8vector"));
lf[1249]=C_h_intern(&lf[1249],9, C_text("u16vector"));
lf[1250]=C_h_intern(&lf[1250],9, C_text("s16vector"));
lf[1251]=C_h_intern(&lf[1251],9, C_text("u32vector"));
lf[1252]=C_h_intern(&lf[1252],9, C_text("s32vector"));
lf[1253]=C_h_intern(&lf[1253],9, C_text("u64vector"));
lf[1254]=C_h_intern(&lf[1254],9, C_text("s64vector"));
lf[1255]=C_h_intern(&lf[1255],9, C_text("f32vector"));
lf[1256]=C_h_intern(&lf[1256],9, C_text("f64vector"));
lf[1257]=C_decode_literal(C_heaptop,C_text("\376B\000\000Abad argument type - locative cannot refer to objects of this type"));
lf[1258]=C_h_intern(&lf[1258],14, C_text("chicken.plist#"));
lf[1259]=C_h_intern(&lf[1259],18, C_text("chicken.plist#put!"));
lf[1260]=C_h_intern(&lf[1260],4, C_text("put!"));
lf[1261]=C_h_intern(&lf[1261],17, C_text("chicken.plist#get"));
lf[1262]=C_h_intern(&lf[1262],3, C_text("get"));
lf[1263]=C_h_intern(&lf[1263],10, C_text("##sys#put!"));
lf[1264]=C_h_intern(&lf[1264],9, C_text("##sys#get"));
lf[1265]=C_h_intern(&lf[1265],22, C_text("chicken.plist#remprop!"));
lf[1266]=C_h_intern(&lf[1266],8, C_text("remprop!"));
lf[1267]=C_h_intern(&lf[1267],26, C_text("chicken.plist#symbol-plist"));
lf[1268]=C_h_intern(&lf[1268],28, C_text("chicken.plist#get-properties"));
lf[1269]=C_h_intern(&lf[1269],14, C_text("get-properties"));
lf[1270]=C_h_intern(&lf[1270],19, C_text("##sys#display-times"));
lf[1271]=C_decode_literal(C_heaptop,C_text("\376B\000\000\0010"));
lf[1272]=C_decode_literal(C_heaptop,C_text("\376U10.0\000"));
lf[1273]=C_decode_literal(C_heaptop,C_text("\376\302\000\000\01040000000"));
lf[1274]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 GiB"));
lf[1275]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 MiB"));
lf[1276]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004 KiB"));
lf[1277]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 bytes"));
lf[1278]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025, maximum live heap: "));
lf[1279]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022 GCs (major/minor)"));
lf[1280]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002, "));
lf[1281]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032 mutations (total/tracked)"));
lf[1282]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002, "));
lf[1283]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021s GC time (major)"));
lf[1284]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002, "));
lf[1285]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012s CPU time"));
lf[1286]=C_h_intern(&lf[1286],25, C_text("##sys#filter-heap-objects"));
lf[1287]=C_h_intern(&lf[1287],17, C_text("chicken.platform#"));
lf[1288]=C_h_intern(&lf[1288],30, C_text("chicken.platform#software-type"));
lf[1289]=C_h_intern(&lf[1289],29, C_text("chicken.platform#machine-type"));
lf[1290]=C_h_intern(&lf[1290],35, C_text("chicken.platform#machine-byte-order"));
lf[1291]=C_h_intern(&lf[1291],33, C_text("chicken.platform#software-version"));
lf[1292]=C_h_intern(&lf[1292],31, C_text("chicken.platform#build-platform"));
lf[1293]=C_h_intern(&lf[1293],22, C_text("##sys#windows-platform"));
lf[1294]=C_h_intern(&lf[1294],32, C_text("chicken.platform#chicken-version"));
lf[1295]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010Version "));
lf[1296]=C_h_intern(&lf[1296],19, C_text("##sys#build-version"));
lf[1297]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[1298]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1299]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002 ["));
lf[1300]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002 ]"));
lf[1301]=C_h_intern(&lf[1301],7, C_text("unknown"));
lf[1302]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1303]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001-"));
lf[1304]=C_h_intern(&lf[1304],14, C_text("##sys#build-id"));
lf[1305]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 (rev "));
lf[1306]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[1307]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1308]=C_h_intern(&lf[1308],18, C_text("##sys#build-branch"));
lf[1309]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002 ("));
lf[1310]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001)"));
lf[1311]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1312]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 dload"));
lf[1313]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1314]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 ptables"));
lf[1315]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1316]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010 gchooks"));
lf[1317]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1318]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001 "));
lf[1319]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003bit"));
lf[1320]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006 cross"));
lf[1321]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[1322]=C_h_intern(&lf[1322],25, C_text("chicken.platform#feature\077"));
lf[1323]=C_h_intern_kw(&lf[1323],13, C_text("cross-chicken"));
lf[1324]=C_h_intern_kw(&lf[1324],7, C_text("gchooks"));
lf[1325]=C_h_intern_kw(&lf[1325],7, C_text("ptables"));
lf[1326]=C_h_intern_kw(&lf[1326],5, C_text("dload"));
lf[1327]=C_h_intern(&lf[1327],29, C_text("chicken.platform#chicken-home"));
lf[1328]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006/share"));
lf[1329]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026CHICKEN_INSTALL_PREFIX"));
lf[1331]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\006\000\376\377\016"));
lf[1332]=C_h_intern(&lf[1332],16, C_text("##sys#split-path"));
lf[1333]=C_h_intern(&lf[1333],32, C_text("chicken.platform#repository-path"));
lf[1334]=C_h_intern(&lf[1334],40, C_text("chicken.platform#installation-repository"));
lf[1336]=C_decode_literal(C_heaptop,C_text("\376B\000\0001bad argument type - not a valid feature identifer"));
lf[1337]=C_h_intern(&lf[1337],14, C_text("##sys#features"));
lf[1338]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\002chicken\376\003\000\000\002\376\001\000\000\006\002srfi-6\376\003\000\000\002\376\001\000\000\006\002srfi-8\376\003\000\000\002\376\001\000\000\007\002srfi-12\376\003\000\000\002\376\001\000\000\007"
"\002srfi-17\376\003\000\000\002\376\001\000\000\007\002srfi-23\376\003\000\000\002\376\001\000\000\007\002srfi-30\376\003\000\000\002\376\001\000\000\007\002srfi-39\376\003\000\000\002\376\001\000\000\007\002srfi-62"
"\376\003\000\000\002\376\001\000\000\007\002srfi-87\376\003\000\000\002\376\001\000\000\007\002srfi-88\376\003\000\000\002\376\001\000\000\022\002full-numeric-tower\376\377\016"));
lf[1339]=C_h_intern(&lf[1339],34, C_text("chicken.platform#register-feature!"));
lf[1340]=C_h_intern(&lf[1340],36, C_text("chicken.platform#unregister-feature!"));
lf[1341]=C_h_intern(&lf[1341],25, C_text("chicken.platform#features"));
lf[1342]=C_h_intern(&lf[1342],31, C_text("chicken.platform#return-to-host"));
lf[1343]=C_h_intern(&lf[1343],40, C_text("chicken.platform#system-config-directory"));
lf[1344]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007APPDATA"));
lf[1345]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010/.config"));
lf[1346]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004HOME"));
lf[1347]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017XDG_CONFIG_HOME"));
lf[1348]=C_h_intern(&lf[1348],39, C_text("chicken.platform#system-cache-directory"));
lf[1349]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007APPDATA"));
lf[1350]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014LOCALAPPDATA"));
lf[1351]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007/.cache"));
lf[1352]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004HOME"));
lf[1353]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016XDG_CACHE_HOME"));
lf[1354]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010chicken-"));
lf[1355]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001."));
lf[1356]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010chicken-"));
lf[1357]=C_decode_literal(C_heaptop,C_text("\376B\000\000\003bit"));
lf[1358]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032CHICKEN_INSTALL_REPOSITORY"));
lf[1359]=C_h_intern(&lf[1359],15, C_text("repository-path"));
lf[1360]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027CHICKEN_REPOSITORY_PATH"));
lf[1361]=C_h_intern(&lf[1361],7, C_text("windows"));
lf[1362]=C_h_intern(&lf[1362],6, C_text("cygwin"));
lf[1363]=C_h_intern(&lf[1363],12, C_text("symbol-plist"));
lf[1364]=C_h_intern(&lf[1364],19, C_text("symbol-plist/setter"));
lf[1365]=C_decode_literal(C_heaptop,C_text("\376B\000\000$property-list must be of even length"));
lf[1366]=C_decode_literal(C_heaptop,C_text("\376B\000\000 (chicken.plist#symbol-plist sym)"));
lf[1367]=C_h_intern(&lf[1367],22, C_text("command-line-arguments"));
lf[1368]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002-:"));
lf[1369]=C_h_intern(&lf[1369],12, C_text("program-name"));
lf[1370]=C_decode_literal(C_heaptop,C_text("\376B\000\000\011<unknown>"));
lf[1371]=C_h_intern(&lf[1371],17, C_text("current-directory"));
lf[1372]=C_decode_literal(C_heaptop,C_text("\376B\000\000!cannot retrieve current directory"));
lf[1373]=C_decode_literal(C_heaptop,C_text("\376B\000\000+(chicken.process-context#current-directory)"));
lf[1374]=C_h_intern(&lf[1374],7, C_text("running"));
lf[1375]=C_h_intern(&lf[1375],10, C_text("primordial"));
lf[1376]=C_h_intern(&lf[1376],9, C_text("arguments"));
lf[1377]=C_h_intern(&lf[1377],7, C_text("message"));
lf[1378]=C_h_intern(&lf[1378],17, C_text("chicken.repl#repl"));
lf[1379]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1380]=C_decode_literal(C_heaptop,C_text("\376B\000\000\002: "));
lf[1381]=C_decode_literal(C_heaptop,C_text("\376B\000\000\006\012Error"));
lf[1382]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016error in error"));
lf[1383]=C_decode_literal(C_heaptop,C_text("\376B\000\000,\042exit\042 called while processing on-exit tasks"));
lf[1384]=C_h_intern(&lf[1384],10, C_text("read-table"));
lf[1385]=C_h_intern(&lf[1385],13, C_text("keyword-style"));
lf[1386]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021no setter defined"));
lf[1387]=C_decode_literal(C_heaptop,C_text("\376B\000\000#can not set setter of non-procedure"));
lf[1388]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(stderr)"));
lf[1389]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010(stdout)"));
lf[1390]=C_decode_literal(C_heaptop,C_text("\376B\000\000\007(stdin)"));
lf[1391]=C_h_intern(&lf[1391],9, C_text("backspace"));
lf[1392]=C_h_intern(&lf[1392],4, C_text("page"));
lf[1393]=C_h_intern(&lf[1393],6, C_text("return"));
lf[1394]=C_h_intern(&lf[1394],4, C_text("null"));
lf[1395]=C_h_intern(&lf[1395],3, C_text("nul"));
lf[1396]=C_h_intern(&lf[1396],5, C_text("alarm"));
lf[1397]=C_h_intern(&lf[1397],6, C_text("escape"));
lf[1398]=C_h_intern(&lf[1398],3, C_text("esc"));
lf[1399]=C_h_intern(&lf[1399],6, C_text("delete"));
lf[1400]=C_h_intern(&lf[1400],4, C_text("vtab"));
lf[1401]=C_h_intern(&lf[1401],7, C_text("newline"));
lf[1402]=C_h_intern(&lf[1402],8, C_text("linefeed"));
lf[1403]=C_h_intern(&lf[1403],3, C_text("tab"));
lf[1404]=C_h_intern(&lf[1404],5, C_text("space"));
C_register_lf2(lf,1405,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8571,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_build_2dversion_toplevel(2,av2);}}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[2101] = {
{C_text("f34270:library_2escm"),(void*)f34270},
{C_text("f34823:library_2escm"),(void*)f34823},
{C_text("f35032:library_2escm"),(void*)f35032},
{C_text("f35036:library_2escm"),(void*)f35036},
{C_text("f37127:library_2escm"),(void*)f37127},
{C_text("f37133:library_2escm"),(void*)f37133},
{C_text("f37139:library_2escm"),(void*)f37139},
{C_text("f_10007:library_2escm"),(void*)f_10007},
{C_text("f_10036:library_2escm"),(void*)f_10036},
{C_text("f_10043:library_2escm"),(void*)f_10043},
{C_text("f_10046:library_2escm"),(void*)f_10046},
{C_text("f_10075:library_2escm"),(void*)f_10075},
{C_text("f_10081:library_2escm"),(void*)f_10081},
{C_text("f_10097:library_2escm"),(void*)f_10097},
{C_text("f_10122:library_2escm"),(void*)f_10122},
{C_text("f_10131:library_2escm"),(void*)f_10131},
{C_text("f_10172:library_2escm"),(void*)f_10172},
{C_text("f_10194:library_2escm"),(void*)f_10194},
{C_text("f_10226:library_2escm"),(void*)f_10226},
{C_text("f_10242:library_2escm"),(void*)f_10242},
{C_text("f_10245:library_2escm"),(void*)f_10245},
{C_text("f_10267:library_2escm"),(void*)f_10267},
{C_text("f_10273:library_2escm"),(void*)f_10273},
{C_text("f_10298:library_2escm"),(void*)f_10298},
{C_text("f_10338:library_2escm"),(void*)f_10338},
{C_text("f_10350:library_2escm"),(void*)f_10350},
{C_text("f_10369:library_2escm"),(void*)f_10369},
{C_text("f_10391:library_2escm"),(void*)f_10391},
{C_text("f_10394:library_2escm"),(void*)f_10394},
{C_text("f_10397:library_2escm"),(void*)f_10397},
{C_text("f_10400:library_2escm"),(void*)f_10400},
{C_text("f_10406:library_2escm"),(void*)f_10406},
{C_text("f_10409:library_2escm"),(void*)f_10409},
{C_text("f_10412:library_2escm"),(void*)f_10412},
{C_text("f_10415:library_2escm"),(void*)f_10415},
{C_text("f_10418:library_2escm"),(void*)f_10418},
{C_text("f_10421:library_2escm"),(void*)f_10421},
{C_text("f_10424:library_2escm"),(void*)f_10424},
{C_text("f_10427:library_2escm"),(void*)f_10427},
{C_text("f_10430:library_2escm"),(void*)f_10430},
{C_text("f_10433:library_2escm"),(void*)f_10433},
{C_text("f_10436:library_2escm"),(void*)f_10436},
{C_text("f_10439:library_2escm"),(void*)f_10439},
{C_text("f_10442:library_2escm"),(void*)f_10442},
{C_text("f_10445:library_2escm"),(void*)f_10445},
{C_text("f_10448:library_2escm"),(void*)f_10448},
{C_text("f_10451:library_2escm"),(void*)f_10451},
{C_text("f_10454:library_2escm"),(void*)f_10454},
{C_text("f_10457:library_2escm"),(void*)f_10457},
{C_text("f_10460:library_2escm"),(void*)f_10460},
{C_text("f_10463:library_2escm"),(void*)f_10463},
{C_text("f_10466:library_2escm"),(void*)f_10466},
{C_text("f_10469:library_2escm"),(void*)f_10469},
{C_text("f_10472:library_2escm"),(void*)f_10472},
{C_text("f_10475:library_2escm"),(void*)f_10475},
{C_text("f_10479:library_2escm"),(void*)f_10479},
{C_text("f_10481:library_2escm"),(void*)f_10481},
{C_text("f_10486:library_2escm"),(void*)f_10486},
{C_text("f_10495:library_2escm"),(void*)f_10495},
{C_text("f_10507:library_2escm"),(void*)f_10507},
{C_text("f_10522:library_2escm"),(void*)f_10522},
{C_text("f_10525:library_2escm"),(void*)f_10525},
{C_text("f_10533:library_2escm"),(void*)f_10533},
{C_text("f_10537:library_2escm"),(void*)f_10537},
{C_text("f_10539:library_2escm"),(void*)f_10539},
{C_text("f_10543:library_2escm"),(void*)f_10543},
{C_text("f_10546:library_2escm"),(void*)f_10546},
{C_text("f_10557:library_2escm"),(void*)f_10557},
{C_text("f_10559:library_2escm"),(void*)f_10559},
{C_text("f_10566:library_2escm"),(void*)f_10566},
{C_text("f_10569:library_2escm"),(void*)f_10569},
{C_text("f_10572:library_2escm"),(void*)f_10572},
{C_text("f_10577:library_2escm"),(void*)f_10577},
{C_text("f_10581:library_2escm"),(void*)f_10581},
{C_text("f_10586:library_2escm"),(void*)f_10586},
{C_text("f_10590:library_2escm"),(void*)f_10590},
{C_text("f_10593:library_2escm"),(void*)f_10593},
{C_text("f_10621:library_2escm"),(void*)f_10621},
{C_text("f_10624:library_2escm"),(void*)f_10624},
{C_text("f_10627:library_2escm"),(void*)f_10627},
{C_text("f_10630:library_2escm"),(void*)f_10630},
{C_text("f_10647:library_2escm"),(void*)f_10647},
{C_text("f_10658:library_2escm"),(void*)f_10658},
{C_text("f_10670:library_2escm"),(void*)f_10670},
{C_text("f_10681:library_2escm"),(void*)f_10681},
{C_text("f_10692:library_2escm"),(void*)f_10692},
{C_text("f_10703:library_2escm"),(void*)f_10703},
{C_text("f_10714:library_2escm"),(void*)f_10714},
{C_text("f_10725:library_2escm"),(void*)f_10725},
{C_text("f_10736:library_2escm"),(void*)f_10736},
{C_text("f_10747:library_2escm"),(void*)f_10747},
{C_text("f_10758:library_2escm"),(void*)f_10758},
{C_text("f_10769:library_2escm"),(void*)f_10769},
{C_text("f_10780:library_2escm"),(void*)f_10780},
{C_text("f_10791:library_2escm"),(void*)f_10791},
{C_text("f_10802:library_2escm"),(void*)f_10802},
{C_text("f_10813:library_2escm"),(void*)f_10813},
{C_text("f_10830:library_2escm"),(void*)f_10830},
{C_text("f_10847:library_2escm"),(void*)f_10847},
{C_text("f_10871:library_2escm"),(void*)f_10871},
{C_text("f_10888:library_2escm"),(void*)f_10888},
{C_text("f_10892:library_2escm"),(void*)f_10892},
{C_text("f_10918:library_2escm"),(void*)f_10918},
{C_text("f_10938:library_2escm"),(void*)f_10938},
{C_text("f_10949:library_2escm"),(void*)f_10949},
{C_text("f_10961:library_2escm"),(void*)f_10961},
{C_text("f_10995:library_2escm"),(void*)f_10995},
{C_text("f_11010:library_2escm"),(void*)f_11010},
{C_text("f_11077:library_2escm"),(void*)f_11077},
{C_text("f_11093:library_2escm"),(void*)f_11093},
{C_text("f_11109:library_2escm"),(void*)f_11109},
{C_text("f_11125:library_2escm"),(void*)f_11125},
{C_text("f_11141:library_2escm"),(void*)f_11141},
{C_text("f_11157:library_2escm"),(void*)f_11157},
{C_text("f_11173:library_2escm"),(void*)f_11173},
{C_text("f_11189:library_2escm"),(void*)f_11189},
{C_text("f_11205:library_2escm"),(void*)f_11205},
{C_text("f_11217:library_2escm"),(void*)f_11217},
{C_text("f_11236:library_2escm"),(void*)f_11236},
{C_text("f_11267:library_2escm"),(void*)f_11267},
{C_text("f_11284:library_2escm"),(void*)f_11284},
{C_text("f_11290:library_2escm"),(void*)f_11290},
{C_text("f_11314:library_2escm"),(void*)f_11314},
{C_text("f_11334:library_2escm"),(void*)f_11334},
{C_text("f_11338:library_2escm"),(void*)f_11338},
{C_text("f_11341:library_2escm"),(void*)f_11341},
{C_text("f_11370:library_2escm"),(void*)f_11370},
{C_text("f_11386:library_2escm"),(void*)f_11386},
{C_text("f_11408:library_2escm"),(void*)f_11408},
{C_text("f_11424:library_2escm"),(void*)f_11424},
{C_text("f_11429:library_2escm"),(void*)f_11429},
{C_text("f_11460:library_2escm"),(void*)f_11460},
{C_text("f_11471:library_2escm"),(void*)f_11471},
{C_text("f_11480:library_2escm"),(void*)f_11480},
{C_text("f_11509:library_2escm"),(void*)f_11509},
{C_text("f_11522:library_2escm"),(void*)f_11522},
{C_text("f_11532:library_2escm"),(void*)f_11532},
{C_text("f_11534:library_2escm"),(void*)f_11534},
{C_text("f_11541:library_2escm"),(void*)f_11541},
{C_text("f_11544:library_2escm"),(void*)f_11544},
{C_text("f_11553:library_2escm"),(void*)f_11553},
{C_text("f_11579:library_2escm"),(void*)f_11579},
{C_text("f_11584:library_2escm"),(void*)f_11584},
{C_text("f_11588:library_2escm"),(void*)f_11588},
{C_text("f_11593:library_2escm"),(void*)f_11593},
{C_text("f_11614:library_2escm"),(void*)f_11614},
{C_text("f_11620:library_2escm"),(void*)f_11620},
{C_text("f_11638:library_2escm"),(void*)f_11638},
{C_text("f_11644:library_2escm"),(void*)f_11644},
{C_text("f_11662:library_2escm"),(void*)f_11662},
{C_text("f_11668:library_2escm"),(void*)f_11668},
{C_text("f_11683:library_2escm"),(void*)f_11683},
{C_text("f_11689:library_2escm"),(void*)f_11689},
{C_text("f_11703:library_2escm"),(void*)f_11703},
{C_text("f_11724:library_2escm"),(void*)f_11724},
{C_text("f_11730:library_2escm"),(void*)f_11730},
{C_text("f_11748:library_2escm"),(void*)f_11748},
{C_text("f_11754:library_2escm"),(void*)f_11754},
{C_text("f_11772:library_2escm"),(void*)f_11772},
{C_text("f_11778:library_2escm"),(void*)f_11778},
{C_text("f_11793:library_2escm"),(void*)f_11793},
{C_text("f_11799:library_2escm"),(void*)f_11799},
{C_text("f_11814:library_2escm"),(void*)f_11814},
{C_text("f_11824:library_2escm"),(void*)f_11824},
{C_text("f_11830:library_2escm"),(void*)f_11830},
{C_text("f_11834:library_2escm"),(void*)f_11834},
{C_text("f_11836:library_2escm"),(void*)f_11836},
{C_text("f_11847:library_2escm"),(void*)f_11847},
{C_text("f_11859:library_2escm"),(void*)f_11859},
{C_text("f_11869:library_2escm"),(void*)f_11869},
{C_text("f_11875:library_2escm"),(void*)f_11875},
{C_text("f_11879:library_2escm"),(void*)f_11879},
{C_text("f_11884:library_2escm"),(void*)f_11884},
{C_text("f_11910:library_2escm"),(void*)f_11910},
{C_text("f_11914:library_2escm"),(void*)f_11914},
{C_text("f_11917:library_2escm"),(void*)f_11917},
{C_text("f_11925:library_2escm"),(void*)f_11925},
{C_text("f_11946:library_2escm"),(void*)f_11946},
{C_text("f_11960:library_2escm"),(void*)f_11960},
{C_text("f_11964:library_2escm"),(void*)f_11964},
{C_text("f_12005:library_2escm"),(void*)f_12005},
{C_text("f_12027:library_2escm"),(void*)f_12027},
{C_text("f_12029:library_2escm"),(void*)f_12029},
{C_text("f_12051:library_2escm"),(void*)f_12051},
{C_text("f_12053:library_2escm"),(void*)f_12053},
{C_text("f_12075:library_2escm"),(void*)f_12075},
{C_text("f_12077:library_2escm"),(void*)f_12077},
{C_text("f_12099:library_2escm"),(void*)f_12099},
{C_text("f_12101:library_2escm"),(void*)f_12101},
{C_text("f_12123:library_2escm"),(void*)f_12123},
{C_text("f_12125:library_2escm"),(void*)f_12125},
{C_text("f_12147:library_2escm"),(void*)f_12147},
{C_text("f_12149:library_2escm"),(void*)f_12149},
{C_text("f_12171:library_2escm"),(void*)f_12171},
{C_text("f_12173:library_2escm"),(void*)f_12173},
{C_text("f_12195:library_2escm"),(void*)f_12195},
{C_text("f_12197:library_2escm"),(void*)f_12197},
{C_text("f_12219:library_2escm"),(void*)f_12219},
{C_text("f_12221:library_2escm"),(void*)f_12221},
{C_text("f_12243:library_2escm"),(void*)f_12243},
{C_text("f_12245:library_2escm"),(void*)f_12245},
{C_text("f_12267:library_2escm"),(void*)f_12267},
{C_text("f_12269:library_2escm"),(void*)f_12269},
{C_text("f_12285:library_2escm"),(void*)f_12285},
{C_text("f_12287:library_2escm"),(void*)f_12287},
{C_text("f_12309:library_2escm"),(void*)f_12309},
{C_text("f_12311:library_2escm"),(void*)f_12311},
{C_text("f_12333:library_2escm"),(void*)f_12333},
{C_text("f_12335:library_2escm"),(void*)f_12335},
{C_text("f_12351:library_2escm"),(void*)f_12351},
{C_text("f_12353:library_2escm"),(void*)f_12353},
{C_text("f_12369:library_2escm"),(void*)f_12369},
{C_text("f_12371:library_2escm"),(void*)f_12371},
{C_text("f_12387:library_2escm"),(void*)f_12387},
{C_text("f_12389:library_2escm"),(void*)f_12389},
{C_text("f_12405:library_2escm"),(void*)f_12405},
{C_text("f_12407:library_2escm"),(void*)f_12407},
{C_text("f_12423:library_2escm"),(void*)f_12423},
{C_text("f_12425:library_2escm"),(void*)f_12425},
{C_text("f_12441:library_2escm"),(void*)f_12441},
{C_text("f_12443:library_2escm"),(void*)f_12443},
{C_text("f_12459:library_2escm"),(void*)f_12459},
{C_text("f_12461:library_2escm"),(void*)f_12461},
{C_text("f_12477:library_2escm"),(void*)f_12477},
{C_text("f_12479:library_2escm"),(void*)f_12479},
{C_text("f_12495:library_2escm"),(void*)f_12495},
{C_text("f_12497:library_2escm"),(void*)f_12497},
{C_text("f_12513:library_2escm"),(void*)f_12513},
{C_text("f_12515:library_2escm"),(void*)f_12515},
{C_text("f_12537:library_2escm"),(void*)f_12537},
{C_text("f_12539:library_2escm"),(void*)f_12539},
{C_text("f_12555:library_2escm"),(void*)f_12555},
{C_text("f_12557:library_2escm"),(void*)f_12557},
{C_text("f_12579:library_2escm"),(void*)f_12579},
{C_text("f_12581:library_2escm"),(void*)f_12581},
{C_text("f_12597:library_2escm"),(void*)f_12597},
{C_text("f_12599:library_2escm"),(void*)f_12599},
{C_text("f_12615:library_2escm"),(void*)f_12615},
{C_text("f_12617:library_2escm"),(void*)f_12617},
{C_text("f_12633:library_2escm"),(void*)f_12633},
{C_text("f_12635:library_2escm"),(void*)f_12635},
{C_text("f_12651:library_2escm"),(void*)f_12651},
{C_text("f_12653:library_2escm"),(void*)f_12653},
{C_text("f_12663:library_2escm"),(void*)f_12663},
{C_text("f_12675:library_2escm"),(void*)f_12675},
{C_text("f_12685:library_2escm"),(void*)f_12685},
{C_text("f_12689:library_2escm"),(void*)f_12689},
{C_text("f_12711:library_2escm"),(void*)f_12711},
{C_text("f_12715:library_2escm"),(void*)f_12715},
{C_text("f_12718:library_2escm"),(void*)f_12718},
{C_text("f_12723:library_2escm"),(void*)f_12723},
{C_text("f_12727:library_2escm"),(void*)f_12727},
{C_text("f_12730:library_2escm"),(void*)f_12730},
{C_text("f_12733:library_2escm"),(void*)f_12733},
{C_text("f_12746:library_2escm"),(void*)f_12746},
{C_text("f_12769:library_2escm"),(void*)f_12769},
{C_text("f_12795:library_2escm"),(void*)f_12795},
{C_text("f_12804:library_2escm"),(void*)f_12804},
{C_text("f_12809:library_2escm"),(void*)f_12809},
{C_text("f_12811:library_2escm"),(void*)f_12811},
{C_text("f_12855:library_2escm"),(void*)f_12855},
{C_text("f_12901:library_2escm"),(void*)f_12901},
{C_text("f_12925:library_2escm"),(void*)f_12925},
{C_text("f_12929:library_2escm"),(void*)f_12929},
{C_text("f_12947:library_2escm"),(void*)f_12947},
{C_text("f_12971:library_2escm"),(void*)f_12971},
{C_text("f_12975:library_2escm"),(void*)f_12975},
{C_text("f_12993:library_2escm"),(void*)f_12993},
{C_text("f_13034:library_2escm"),(void*)f_13034},
{C_text("f_13045:library_2escm"),(void*)f_13045},
{C_text("f_13051:library_2escm"),(void*)f_13051},
{C_text("f_13055:library_2escm"),(void*)f_13055},
{C_text("f_13069:library_2escm"),(void*)f_13069},
{C_text("f_13082:library_2escm"),(void*)f_13082},
{C_text("f_13086:library_2escm"),(void*)f_13086},
{C_text("f_13103:library_2escm"),(void*)f_13103},
{C_text("f_13107:library_2escm"),(void*)f_13107},
{C_text("f_13113:library_2escm"),(void*)f_13113},
{C_text("f_13117:library_2escm"),(void*)f_13117},
{C_text("f_13157:library_2escm"),(void*)f_13157},
{C_text("f_13161:library_2escm"),(void*)f_13161},
{C_text("f_13183:library_2escm"),(void*)f_13183},
{C_text("f_13186:library_2escm"),(void*)f_13186},
{C_text("f_13189:library_2escm"),(void*)f_13189},
{C_text("f_13192:library_2escm"),(void*)f_13192},
{C_text("f_13195:library_2escm"),(void*)f_13195},
{C_text("f_13216:library_2escm"),(void*)f_13216},
{C_text("f_13218:library_2escm"),(void*)f_13218},
{C_text("f_13236:library_2escm"),(void*)f_13236},
{C_text("f_13246:library_2escm"),(void*)f_13246},
{C_text("f_13250:library_2escm"),(void*)f_13250},
{C_text("f_13288:library_2escm"),(void*)f_13288},
{C_text("f_13291:library_2escm"),(void*)f_13291},
{C_text("f_13294:library_2escm"),(void*)f_13294},
{C_text("f_13297:library_2escm"),(void*)f_13297},
{C_text("f_13303:library_2escm"),(void*)f_13303},
{C_text("f_13306:library_2escm"),(void*)f_13306},
{C_text("f_13354:library_2escm"),(void*)f_13354},
{C_text("f_13358:library_2escm"),(void*)f_13358},
{C_text("f_13435:library_2escm"),(void*)f_13435},
{C_text("f_13474:library_2escm"),(void*)f_13474},
{C_text("f_13529:library_2escm"),(void*)f_13529},
{C_text("f_13574:library_2escm"),(void*)f_13574},
{C_text("f_13619:library_2escm"),(void*)f_13619},
{C_text("f_13656:library_2escm"),(void*)f_13656},
{C_text("f_13676:library_2escm"),(void*)f_13676},
{C_text("f_13700:library_2escm"),(void*)f_13700},
{C_text("f_13703:library_2escm"),(void*)f_13703},
{C_text("f_13707:library_2escm"),(void*)f_13707},
{C_text("f_13710:library_2escm"),(void*)f_13710},
{C_text("f_13728:library_2escm"),(void*)f_13728},
{C_text("f_13751:library_2escm"),(void*)f_13751},
{C_text("f_13755:library_2escm"),(void*)f_13755},
{C_text("f_13778:library_2escm"),(void*)f_13778},
{C_text("f_13782:library_2escm"),(void*)f_13782},
{C_text("f_13814:library_2escm"),(void*)f_13814},
{C_text("f_13850:library_2escm"),(void*)f_13850},
{C_text("f_13854:library_2escm"),(void*)f_13854},
{C_text("f_13873:library_2escm"),(void*)f_13873},
{C_text("f_13875:library_2escm"),(void*)f_13875},
{C_text("f_13882:library_2escm"),(void*)f_13882},
{C_text("f_13919:library_2escm"),(void*)f_13919},
{C_text("f_13926:library_2escm"),(void*)f_13926},
{C_text("f_13963:library_2escm"),(void*)f_13963},
{C_text("f_13981:library_2escm"),(void*)f_13981},
{C_text("f_13990:library_2escm"),(void*)f_13990},
{C_text("f_13995:library_2escm"),(void*)f_13995},
{C_text("f_14008:library_2escm"),(void*)f_14008},
{C_text("f_14034:library_2escm"),(void*)f_14034},
{C_text("f_14042:library_2escm"),(void*)f_14042},
{C_text("f_14046:library_2escm"),(void*)f_14046},
{C_text("f_14050:library_2escm"),(void*)f_14050},
{C_text("f_14054:library_2escm"),(void*)f_14054},
{C_text("f_14057:library_2escm"),(void*)f_14057},
{C_text("f_14071:library_2escm"),(void*)f_14071},
{C_text("f_14075:library_2escm"),(void*)f_14075},
{C_text("f_14086:library_2escm"),(void*)f_14086},
{C_text("f_14110:library_2escm"),(void*)f_14110},
{C_text("f_14114:library_2escm"),(void*)f_14114},
{C_text("f_14120:library_2escm"),(void*)f_14120},
{C_text("f_14144:library_2escm"),(void*)f_14144},
{C_text("f_14148:library_2escm"),(void*)f_14148},
{C_text("f_14154:library_2escm"),(void*)f_14154},
{C_text("f_14171:library_2escm"),(void*)f_14171},
{C_text("f_14175:library_2escm"),(void*)f_14175},
{C_text("f_14179:library_2escm"),(void*)f_14179},
{C_text("f_14189:library_2escm"),(void*)f_14189},
{C_text("f_14195:library_2escm"),(void*)f_14195},
{C_text("f_14202:library_2escm"),(void*)f_14202},
{C_text("f_14214:library_2escm"),(void*)f_14214},
{C_text("f_14237:library_2escm"),(void*)f_14237},
{C_text("f_14241:library_2escm"),(void*)f_14241},
{C_text("f_14251:library_2escm"),(void*)f_14251},
{C_text("f_14257:library_2escm"),(void*)f_14257},
{C_text("f_14264:library_2escm"),(void*)f_14264},
{C_text("f_14279:library_2escm"),(void*)f_14279},
{C_text("f_14283:library_2escm"),(void*)f_14283},
{C_text("f_14316:library_2escm"),(void*)f_14316},
{C_text("f_14320:library_2escm"),(void*)f_14320},
{C_text("f_14337:library_2escm"),(void*)f_14337},
{C_text("f_14347:library_2escm"),(void*)f_14347},
{C_text("f_14420:library_2escm"),(void*)f_14420},
{C_text("f_14434:library_2escm"),(void*)f_14434},
{C_text("f_14451:library_2escm"),(void*)f_14451},
{C_text("f_14467:library_2escm"),(void*)f_14467},
{C_text("f_14538:library_2escm"),(void*)f_14538},
{C_text("f_14542:library_2escm"),(void*)f_14542},
{C_text("f_14547:library_2escm"),(void*)f_14547},
{C_text("f_14557:library_2escm"),(void*)f_14557},
{C_text("f_14561:library_2escm"),(void*)f_14561},
{C_text("f_14576:library_2escm"),(void*)f_14576},
{C_text("f_14580:library_2escm"),(void*)f_14580},
{C_text("f_14597:library_2escm"),(void*)f_14597},
{C_text("f_14608:library_2escm"),(void*)f_14608},
{C_text("f_14614:library_2escm"),(void*)f_14614},
{C_text("f_14625:library_2escm"),(void*)f_14625},
{C_text("f_14636:library_2escm"),(void*)f_14636},
{C_text("f_14648:library_2escm"),(void*)f_14648},
{C_text("f_14660:library_2escm"),(void*)f_14660},
{C_text("f_14672:library_2escm"),(void*)f_14672},
{C_text("f_14689:library_2escm"),(void*)f_14689},
{C_text("f_14696:library_2escm"),(void*)f_14696},
{C_text("f_14700:library_2escm"),(void*)f_14700},
{C_text("f_14702:library_2escm"),(void*)f_14702},
{C_text("f_14708:library_2escm"),(void*)f_14708},
{C_text("f_14712:library_2escm"),(void*)f_14712},
{C_text("f_14715:library_2escm"),(void*)f_14715},
{C_text("f_14720:library_2escm"),(void*)f_14720},
{C_text("f_14730:library_2escm"),(void*)f_14730},
{C_text("f_14750:library_2escm"),(void*)f_14750},
{C_text("f_14765:library_2escm"),(void*)f_14765},
{C_text("f_14795:library_2escm"),(void*)f_14795},
{C_text("f_14806:library_2escm"),(void*)f_14806},
{C_text("f_14822:library_2escm"),(void*)f_14822},
{C_text("f_14826:library_2escm"),(void*)f_14826},
{C_text("f_14830:library_2escm"),(void*)f_14830},
{C_text("f_14846:library_2escm"),(void*)f_14846},
{C_text("f_14864:library_2escm"),(void*)f_14864},
{C_text("f_14875:library_2escm"),(void*)f_14875},
{C_text("f_14897:library_2escm"),(void*)f_14897},
{C_text("f_14914:library_2escm"),(void*)f_14914},
{C_text("f_14917:library_2escm"),(void*)f_14917},
{C_text("f_14929:library_2escm"),(void*)f_14929},
{C_text("f_14931:library_2escm"),(void*)f_14931},
{C_text("f_14952:library_2escm"),(void*)f_14952},
{C_text("f_14983:library_2escm"),(void*)f_14983},
{C_text("f_14987:library_2escm"),(void*)f_14987},
{C_text("f_15022:library_2escm"),(void*)f_15022},
{C_text("f_15029:library_2escm"),(void*)f_15029},
{C_text("f_15041:library_2escm"),(void*)f_15041},
{C_text("f_15061:library_2escm"),(void*)f_15061},
{C_text("f_15065:library_2escm"),(void*)f_15065},
{C_text("f_15085:library_2escm"),(void*)f_15085},
{C_text("f_15105:library_2escm"),(void*)f_15105},
{C_text("f_15109:library_2escm"),(void*)f_15109},
{C_text("f_15113:library_2escm"),(void*)f_15113},
{C_text("f_15120:library_2escm"),(void*)f_15120},
{C_text("f_15136:library_2escm"),(void*)f_15136},
{C_text("f_15140:library_2escm"),(void*)f_15140},
{C_text("f_15181:library_2escm"),(void*)f_15181},
{C_text("f_15214:library_2escm"),(void*)f_15214},
{C_text("f_15243:library_2escm"),(void*)f_15243},
{C_text("f_15251:library_2escm"),(void*)f_15251},
{C_text("f_15257:library_2escm"),(void*)f_15257},
{C_text("f_15277:library_2escm"),(void*)f_15277},
{C_text("f_15302:library_2escm"),(void*)f_15302},
{C_text("f_15308:library_2escm"),(void*)f_15308},
{C_text("f_15312:library_2escm"),(void*)f_15312},
{C_text("f_15322:library_2escm"),(void*)f_15322},
{C_text("f_15342:library_2escm"),(void*)f_15342},
{C_text("f_15363:library_2escm"),(void*)f_15363},
{C_text("f_15380:library_2escm"),(void*)f_15380},
{C_text("f_15387:library_2escm"),(void*)f_15387},
{C_text("f_15392:library_2escm"),(void*)f_15392},
{C_text("f_15398:library_2escm"),(void*)f_15398},
{C_text("f_15402:library_2escm"),(void*)f_15402},
{C_text("f_15405:library_2escm"),(void*)f_15405},
{C_text("f_15412:library_2escm"),(void*)f_15412},
{C_text("f_15433:library_2escm"),(void*)f_15433},
{C_text("f_15447:library_2escm"),(void*)f_15447},
{C_text("f_15451:library_2escm"),(void*)f_15451},
{C_text("f_15488:library_2escm"),(void*)f_15488},
{C_text("f_15496:library_2escm"),(void*)f_15496},
{C_text("f_15525:library_2escm"),(void*)f_15525},
{C_text("f_15528:library_2escm"),(void*)f_15528},
{C_text("f_15543:library_2escm"),(void*)f_15543},
{C_text("f_15546:library_2escm"),(void*)f_15546},
{C_text("f_15553:library_2escm"),(void*)f_15553},
{C_text("f_15564:library_2escm"),(void*)f_15564},
{C_text("f_15582:library_2escm"),(void*)f_15582},
{C_text("f_15607:library_2escm"),(void*)f_15607},
{C_text("f_15622:library_2escm"),(void*)f_15622},
{C_text("f_15650:library_2escm"),(void*)f_15650},
{C_text("f_15654:library_2escm"),(void*)f_15654},
{C_text("f_15668:library_2escm"),(void*)f_15668},
{C_text("f_15728:library_2escm"),(void*)f_15728},
{C_text("f_15748:library_2escm"),(void*)f_15748},
{C_text("f_15754:library_2escm"),(void*)f_15754},
{C_text("f_15817:library_2escm"),(void*)f_15817},
{C_text("f_15856:library_2escm"),(void*)f_15856},
{C_text("f_15862:library_2escm"),(void*)f_15862},
{C_text("f_15906:library_2escm"),(void*)f_15906},
{C_text("f_15910:library_2escm"),(void*)f_15910},
{C_text("f_15913:library_2escm"),(void*)f_15913},
{C_text("f_15962:library_2escm"),(void*)f_15962},
{C_text("f_15977:library_2escm"),(void*)f_15977},
{C_text("f_15986:library_2escm"),(void*)f_15986},
{C_text("f_16020:library_2escm"),(void*)f_16020},
{C_text("f_16030:library_2escm"),(void*)f_16030},
{C_text("f_16039:library_2escm"),(void*)f_16039},
{C_text("f_16051:library_2escm"),(void*)f_16051},
{C_text("f_16060:library_2escm"),(void*)f_16060},
{C_text("f_16072:library_2escm"),(void*)f_16072},
{C_text("f_16078:library_2escm"),(void*)f_16078},
{C_text("f_16084:library_2escm"),(void*)f_16084},
{C_text("f_16126:library_2escm"),(void*)f_16126},
{C_text("f_16234:library_2escm"),(void*)f_16234},
{C_text("f_16241:library_2escm"),(void*)f_16241},
{C_text("f_16254:library_2escm"),(void*)f_16254},
{C_text("f_16312:library_2escm"),(void*)f_16312},
{C_text("f_16330:library_2escm"),(void*)f_16330},
{C_text("f_16340:library_2escm"),(void*)f_16340},
{C_text("f_16376:library_2escm"),(void*)f_16376},
{C_text("f_16402:library_2escm"),(void*)f_16402},
{C_text("f_16413:library_2escm"),(void*)f_16413},
{C_text("f_16580:library_2escm"),(void*)f_16580},
{C_text("f_16595:library_2escm"),(void*)f_16595},
{C_text("f_16653:library_2escm"),(void*)f_16653},
{C_text("f_16694:library_2escm"),(void*)f_16694},
{C_text("f_16772:library_2escm"),(void*)f_16772},
{C_text("f_16820:library_2escm"),(void*)f_16820},
{C_text("f_16823:library_2escm"),(void*)f_16823},
{C_text("f_16835:library_2escm"),(void*)f_16835},
{C_text("f_16922:library_2escm"),(void*)f_16922},
{C_text("f_16990:library_2escm"),(void*)f_16990},
{C_text("f_17070:library_2escm"),(void*)f_17070},
{C_text("f_17080:library_2escm"),(void*)f_17080},
{C_text("f_17089:library_2escm"),(void*)f_17089},
{C_text("f_17325:library_2escm"),(void*)f_17325},
{C_text("f_17371:library_2escm"),(void*)f_17371},
{C_text("f_17374:library_2escm"),(void*)f_17374},
{C_text("f_17401:library_2escm"),(void*)f_17401},
{C_text("f_17425:library_2escm"),(void*)f_17425},
{C_text("f_17442:library_2escm"),(void*)f_17442},
{C_text("f_17543:library_2escm"),(void*)f_17543},
{C_text("f_17546:library_2escm"),(void*)f_17546},
{C_text("f_17555:library_2escm"),(void*)f_17555},
{C_text("f_17561:library_2escm"),(void*)f_17561},
{C_text("f_17572:library_2escm"),(void*)f_17572},
{C_text("f_17574:library_2escm"),(void*)f_17574},
{C_text("f_17585:library_2escm"),(void*)f_17585},
{C_text("f_17587:library_2escm"),(void*)f_17587},
{C_text("f_17598:library_2escm"),(void*)f_17598},
{C_text("f_17600:library_2escm"),(void*)f_17600},
{C_text("f_17617:library_2escm"),(void*)f_17617},
{C_text("f_17621:library_2escm"),(void*)f_17621},
{C_text("f_17625:library_2escm"),(void*)f_17625},
{C_text("f_17634:library_2escm"),(void*)f_17634},
{C_text("f_17654:library_2escm"),(void*)f_17654},
{C_text("f_17662:library_2escm"),(void*)f_17662},
{C_text("f_17677:library_2escm"),(void*)f_17677},
{C_text("f_17679:library_2escm"),(void*)f_17679},
{C_text("f_17704:library_2escm"),(void*)f_17704},
{C_text("f_17713:library_2escm"),(void*)f_17713},
{C_text("f_17716:library_2escm"),(void*)f_17716},
{C_text("f_17725:library_2escm"),(void*)f_17725},
{C_text("f_17732:library_2escm"),(void*)f_17732},
{C_text("f_17743:library_2escm"),(void*)f_17743},
{C_text("f_17750:library_2escm"),(void*)f_17750},
{C_text("f_17774:library_2escm"),(void*)f_17774},
{C_text("f_17778:library_2escm"),(void*)f_17778},
{C_text("f_17780:library_2escm"),(void*)f_17780},
{C_text("f_17784:library_2escm"),(void*)f_17784},
{C_text("f_17789:library_2escm"),(void*)f_17789},
{C_text("f_17795:library_2escm"),(void*)f_17795},
{C_text("f_17799:library_2escm"),(void*)f_17799},
{C_text("f_17804:library_2escm"),(void*)f_17804},
{C_text("f_17814:library_2escm"),(void*)f_17814},
{C_text("f_17816:library_2escm"),(void*)f_17816},
{C_text("f_17820:library_2escm"),(void*)f_17820},
{C_text("f_17826:library_2escm"),(void*)f_17826},
{C_text("f_17828:library_2escm"),(void*)f_17828},
{C_text("f_17832:library_2escm"),(void*)f_17832},
{C_text("f_17835:library_2escm"),(void*)f_17835},
{C_text("f_17851:library_2escm"),(void*)f_17851},
{C_text("f_17855:library_2escm"),(void*)f_17855},
{C_text("f_17858:library_2escm"),(void*)f_17858},
{C_text("f_17880:library_2escm"),(void*)f_17880},
{C_text("f_17896:library_2escm"),(void*)f_17896},
{C_text("f_17901:library_2escm"),(void*)f_17901},
{C_text("f_17932:library_2escm"),(void*)f_17932},
{C_text("f_17941:library_2escm"),(void*)f_17941},
{C_text("f_17959:library_2escm"),(void*)f_17959},
{C_text("f_17965:library_2escm"),(void*)f_17965},
{C_text("f_17971:library_2escm"),(void*)f_17971},
{C_text("f_17980:library_2escm"),(void*)f_17980},
{C_text("f_17999:library_2escm"),(void*)f_17999},
{C_text("f_18018:library_2escm"),(void*)f_18018},
{C_text("f_18021:library_2escm"),(void*)f_18021},
{C_text("f_18026:library_2escm"),(void*)f_18026},
{C_text("f_18068:library_2escm"),(void*)f_18068},
{C_text("f_18087:library_2escm"),(void*)f_18087},
{C_text("f_18090:library_2escm"),(void*)f_18090},
{C_text("f_18093:library_2escm"),(void*)f_18093},
{C_text("f_18098:library_2escm"),(void*)f_18098},
{C_text("f_18139:library_2escm"),(void*)f_18139},
{C_text("f_18149:library_2escm"),(void*)f_18149},
{C_text("f_18160:library_2escm"),(void*)f_18160},
{C_text("f_18164:library_2escm"),(void*)f_18164},
{C_text("f_18167:library_2escm"),(void*)f_18167},
{C_text("f_18172:library_2escm"),(void*)f_18172},
{C_text("f_18199:library_2escm"),(void*)f_18199},
{C_text("f_18213:library_2escm"),(void*)f_18213},
{C_text("f_18224:library_2escm"),(void*)f_18224},
{C_text("f_18235:library_2escm"),(void*)f_18235},
{C_text("f_18246:library_2escm"),(void*)f_18246},
{C_text("f_18259:library_2escm"),(void*)f_18259},
{C_text("f_18261:library_2escm"),(void*)f_18261},
{C_text("f_18274:library_2escm"),(void*)f_18274},
{C_text("f_18302:library_2escm"),(void*)f_18302},
{C_text("f_18315:library_2escm"),(void*)f_18315},
{C_text("f_18333:library_2escm"),(void*)f_18333},
{C_text("f_18336:library_2escm"),(void*)f_18336},
{C_text("f_18419:library_2escm"),(void*)f_18419},
{C_text("f_18422:library_2escm"),(void*)f_18422},
{C_text("f_18425:library_2escm"),(void*)f_18425},
{C_text("f_18428:library_2escm"),(void*)f_18428},
{C_text("f_18431:library_2escm"),(void*)f_18431},
{C_text("f_18434:library_2escm"),(void*)f_18434},
{C_text("f_18437:library_2escm"),(void*)f_18437},
{C_text("f_18440:library_2escm"),(void*)f_18440},
{C_text("f_18443:library_2escm"),(void*)f_18443},
{C_text("f_18446:library_2escm"),(void*)f_18446},
{C_text("f_18449:library_2escm"),(void*)f_18449},
{C_text("f_18452:library_2escm"),(void*)f_18452},
{C_text("f_18455:library_2escm"),(void*)f_18455},
{C_text("f_18458:library_2escm"),(void*)f_18458},
{C_text("f_18463:library_2escm"),(void*)f_18463},
{C_text("f_18469:library_2escm"),(void*)f_18469},
{C_text("f_18485:library_2escm"),(void*)f_18485},
{C_text("f_18501:library_2escm"),(void*)f_18501},
{C_text("f_18507:library_2escm"),(void*)f_18507},
{C_text("f_18527:library_2escm"),(void*)f_18527},
{C_text("f_18531:library_2escm"),(void*)f_18531},
{C_text("f_18543:library_2escm"),(void*)f_18543},
{C_text("f_18549:library_2escm"),(void*)f_18549},
{C_text("f_18555:library_2escm"),(void*)f_18555},
{C_text("f_18587:library_2escm"),(void*)f_18587},
{C_text("f_18591:library_2escm"),(void*)f_18591},
{C_text("f_18601:library_2escm"),(void*)f_18601},
{C_text("f_18616:library_2escm"),(void*)f_18616},
{C_text("f_18626:library_2escm"),(void*)f_18626},
{C_text("f_18646:library_2escm"),(void*)f_18646},
{C_text("f_18653:library_2escm"),(void*)f_18653},
{C_text("f_18662:library_2escm"),(void*)f_18662},
{C_text("f_18669:library_2escm"),(void*)f_18669},
{C_text("f_18671:library_2escm"),(void*)f_18671},
{C_text("f_18677:library_2escm"),(void*)f_18677},
{C_text("f_18683:library_2escm"),(void*)f_18683},
{C_text("f_18701:library_2escm"),(void*)f_18701},
{C_text("f_18726:library_2escm"),(void*)f_18726},
{C_text("f_18742:library_2escm"),(void*)f_18742},
{C_text("f_18749:library_2escm"),(void*)f_18749},
{C_text("f_18762:library_2escm"),(void*)f_18762},
{C_text("f_18766:library_2escm"),(void*)f_18766},
{C_text("f_18770:library_2escm"),(void*)f_18770},
{C_text("f_18772:library_2escm"),(void*)f_18772},
{C_text("f_18778:library_2escm"),(void*)f_18778},
{C_text("f_18785:library_2escm"),(void*)f_18785},
{C_text("f_18789:library_2escm"),(void*)f_18789},
{C_text("f_18798:library_2escm"),(void*)f_18798},
{C_text("f_18806:library_2escm"),(void*)f_18806},
{C_text("f_18816:library_2escm"),(void*)f_18816},
{C_text("f_18822:library_2escm"),(void*)f_18822},
{C_text("f_18825:library_2escm"),(void*)f_18825},
{C_text("f_18829:library_2escm"),(void*)f_18829},
{C_text("f_18856:library_2escm"),(void*)f_18856},
{C_text("f_18872:library_2escm"),(void*)f_18872},
{C_text("f_18878:library_2escm"),(void*)f_18878},
{C_text("f_18901:library_2escm"),(void*)f_18901},
{C_text("f_18914:library_2escm"),(void*)f_18914},
{C_text("f_18920:library_2escm"),(void*)f_18920},
{C_text("f_18925:library_2escm"),(void*)f_18925},
{C_text("f_18930:library_2escm"),(void*)f_18930},
{C_text("f_18934:library_2escm"),(void*)f_18934},
{C_text("f_18943:library_2escm"),(void*)f_18943},
{C_text("f_18974:library_2escm"),(void*)f_18974},
{C_text("f_18978:library_2escm"),(void*)f_18978},
{C_text("f_18982:library_2escm"),(void*)f_18982},
{C_text("f_18985:library_2escm"),(void*)f_18985},
{C_text("f_18988:library_2escm"),(void*)f_18988},
{C_text("f_18991:library_2escm"),(void*)f_18991},
{C_text("f_18993:library_2escm"),(void*)f_18993},
{C_text("f_19004:library_2escm"),(void*)f_19004},
{C_text("f_19015:library_2escm"),(void*)f_19015},
{C_text("f_19026:library_2escm"),(void*)f_19026},
{C_text("f_19037:library_2escm"),(void*)f_19037},
{C_text("f_19050:library_2escm"),(void*)f_19050},
{C_text("f_19052:library_2escm"),(void*)f_19052},
{C_text("f_19058:library_2escm"),(void*)f_19058},
{C_text("f_19063:library_2escm"),(void*)f_19063},
{C_text("f_19098:library_2escm"),(void*)f_19098},
{C_text("f_19111:library_2escm"),(void*)f_19111},
{C_text("f_19113:library_2escm"),(void*)f_19113},
{C_text("f_19119:library_2escm"),(void*)f_19119},
{C_text("f_19124:library_2escm"),(void*)f_19124},
{C_text("f_19159:library_2escm"),(void*)f_19159},
{C_text("f_19172:library_2escm"),(void*)f_19172},
{C_text("f_19174:library_2escm"),(void*)f_19174},
{C_text("f_19180:library_2escm"),(void*)f_19180},
{C_text("f_19185:library_2escm"),(void*)f_19185},
{C_text("f_19220:library_2escm"),(void*)f_19220},
{C_text("f_19232:library_2escm"),(void*)f_19232},
{C_text("f_19234:library_2escm"),(void*)f_19234},
{C_text("f_19240:library_2escm"),(void*)f_19240},
{C_text("f_19246:library_2escm"),(void*)f_19246},
{C_text("f_19256:library_2escm"),(void*)f_19256},
{C_text("f_19259:library_2escm"),(void*)f_19259},
{C_text("f_19262:library_2escm"),(void*)f_19262},
{C_text("f_19265:library_2escm"),(void*)f_19265},
{C_text("f_19268:library_2escm"),(void*)f_19268},
{C_text("f_19275:library_2escm"),(void*)f_19275},
{C_text("f_19279:library_2escm"),(void*)f_19279},
{C_text("f_19283:library_2escm"),(void*)f_19283},
{C_text("f_19289:library_2escm"),(void*)f_19289},
{C_text("f_19302:library_2escm"),(void*)f_19302},
{C_text("f_19344:library_2escm"),(void*)f_19344},
{C_text("f_19348:library_2escm"),(void*)f_19348},
{C_text("f_19381:library_2escm"),(void*)f_19381},
{C_text("f_19387:library_2escm"),(void*)f_19387},
{C_text("f_19393:library_2escm"),(void*)f_19393},
{C_text("f_19399:library_2escm"),(void*)f_19399},
{C_text("f_19405:library_2escm"),(void*)f_19405},
{C_text("f_19409:library_2escm"),(void*)f_19409},
{C_text("f_19414:library_2escm"),(void*)f_19414},
{C_text("f_19420:library_2escm"),(void*)f_19420},
{C_text("f_19424:library_2escm"),(void*)f_19424},
{C_text("f_19429:library_2escm"),(void*)f_19429},
{C_text("f_19433:library_2escm"),(void*)f_19433},
{C_text("f_19438:library_2escm"),(void*)f_19438},
{C_text("f_19444:library_2escm"),(void*)f_19444},
{C_text("f_19448:library_2escm"),(void*)f_19448},
{C_text("f_19453:library_2escm"),(void*)f_19453},
{C_text("f_19457:library_2escm"),(void*)f_19457},
{C_text("f_19462:library_2escm"),(void*)f_19462},
{C_text("f_19467:library_2escm"),(void*)f_19467},
{C_text("f_19473:library_2escm"),(void*)f_19473},
{C_text("f_19477:library_2escm"),(void*)f_19477},
{C_text("f_19482:library_2escm"),(void*)f_19482},
{C_text("f_19487:library_2escm"),(void*)f_19487},
{C_text("f_19491:library_2escm"),(void*)f_19491},
{C_text("f_19496:library_2escm"),(void*)f_19496},
{C_text("f_19501:library_2escm"),(void*)f_19501},
{C_text("f_19507:library_2escm"),(void*)f_19507},
{C_text("f_19511:library_2escm"),(void*)f_19511},
{C_text("f_19516:library_2escm"),(void*)f_19516},
{C_text("f_19521:library_2escm"),(void*)f_19521},
{C_text("f_19526:library_2escm"),(void*)f_19526},
{C_text("f_19547:library_2escm"),(void*)f_19547},
{C_text("f_19554:library_2escm"),(void*)f_19554},
{C_text("f_19560:library_2escm"),(void*)f_19560},
{C_text("f_19578:library_2escm"),(void*)f_19578},
{C_text("f_19587:library_2escm"),(void*)f_19587},
{C_text("f_19600:library_2escm"),(void*)f_19600},
{C_text("f_19608:library_2escm"),(void*)f_19608},
{C_text("f_19613:library_2escm"),(void*)f_19613},
{C_text("f_19650:library_2escm"),(void*)f_19650},
{C_text("f_19662:library_2escm"),(void*)f_19662},
{C_text("f_19672:library_2escm"),(void*)f_19672},
{C_text("f_19688:library_2escm"),(void*)f_19688},
{C_text("f_19701:library_2escm"),(void*)f_19701},
{C_text("f_19708:library_2escm"),(void*)f_19708},
{C_text("f_19710:library_2escm"),(void*)f_19710},
{C_text("f_19723:library_2escm"),(void*)f_19723},
{C_text("f_19729:library_2escm"),(void*)f_19729},
{C_text("f_19736:library_2escm"),(void*)f_19736},
{C_text("f_19738:library_2escm"),(void*)f_19738},
{C_text("f_19742:library_2escm"),(void*)f_19742},
{C_text("f_19745:library_2escm"),(void*)f_19745},
{C_text("f_19755:library_2escm"),(void*)f_19755},
{C_text("f_19759:library_2escm"),(void*)f_19759},
{C_text("f_19767:library_2escm"),(void*)f_19767},
{C_text("f_19770:library_2escm"),(void*)f_19770},
{C_text("f_19777:library_2escm"),(void*)f_19777},
{C_text("f_19780:library_2escm"),(void*)f_19780},
{C_text("f_19783:library_2escm"),(void*)f_19783},
{C_text("f_19786:library_2escm"),(void*)f_19786},
{C_text("f_19794:library_2escm"),(void*)f_19794},
{C_text("f_19802:library_2escm"),(void*)f_19802},
{C_text("f_19804:library_2escm"),(void*)f_19804},
{C_text("f_19820:library_2escm"),(void*)f_19820},
{C_text("f_19847:library_2escm"),(void*)f_19847},
{C_text("f_19851:library_2escm"),(void*)f_19851},
{C_text("f_19855:library_2escm"),(void*)f_19855},
{C_text("f_19859:library_2escm"),(void*)f_19859},
{C_text("f_19863:library_2escm"),(void*)f_19863},
{C_text("f_19867:library_2escm"),(void*)f_19867},
{C_text("f_19871:library_2escm"),(void*)f_19871},
{C_text("f_19875:library_2escm"),(void*)f_19875},
{C_text("f_19879:library_2escm"),(void*)f_19879},
{C_text("f_19883:library_2escm"),(void*)f_19883},
{C_text("f_19887:library_2escm"),(void*)f_19887},
{C_text("f_19891:library_2escm"),(void*)f_19891},
{C_text("f_19895:library_2escm"),(void*)f_19895},
{C_text("f_19899:library_2escm"),(void*)f_19899},
{C_text("f_19903:library_2escm"),(void*)f_19903},
{C_text("f_19907:library_2escm"),(void*)f_19907},
{C_text("f_19911:library_2escm"),(void*)f_19911},
{C_text("f_19918:library_2escm"),(void*)f_19918},
{C_text("f_19925:library_2escm"),(void*)f_19925},
{C_text("f_19926:library_2escm"),(void*)f_19926},
{C_text("f_19930:library_2escm"),(void*)f_19930},
{C_text("f_19933:library_2escm"),(void*)f_19933},
{C_text("f_19949:library_2escm"),(void*)f_19949},
{C_text("f_19960:library_2escm"),(void*)f_19960},
{C_text("f_19968:library_2escm"),(void*)f_19968},
{C_text("f_19980:library_2escm"),(void*)f_19980},
{C_text("f_19989:library_2escm"),(void*)f_19989},
{C_text("f_19994:library_2escm"),(void*)f_19994},
{C_text("f_20050:library_2escm"),(void*)f_20050},
{C_text("f_20064:library_2escm"),(void*)f_20064},
{C_text("f_20076:library_2escm"),(void*)f_20076},
{C_text("f_20082:library_2escm"),(void*)f_20082},
{C_text("f_20107:library_2escm"),(void*)f_20107},
{C_text("f_20125:library_2escm"),(void*)f_20125},
{C_text("f_20129:library_2escm"),(void*)f_20129},
{C_text("f_20185:library_2escm"),(void*)f_20185},
{C_text("f_20194:library_2escm"),(void*)f_20194},
{C_text("f_20207:library_2escm"),(void*)f_20207},
{C_text("f_20219:library_2escm"),(void*)f_20219},
{C_text("f_20237:library_2escm"),(void*)f_20237},
{C_text("f_20257:library_2escm"),(void*)f_20257},
{C_text("f_20260:library_2escm"),(void*)f_20260},
{C_text("f_20263:library_2escm"),(void*)f_20263},
{C_text("f_20271:library_2escm"),(void*)f_20271},
{C_text("f_20275:library_2escm"),(void*)f_20275},
{C_text("f_20279:library_2escm"),(void*)f_20279},
{C_text("f_20283:library_2escm"),(void*)f_20283},
{C_text("f_20287:library_2escm"),(void*)f_20287},
{C_text("f_20289:library_2escm"),(void*)f_20289},
{C_text("f_20297:library_2escm"),(void*)f_20297},
{C_text("f_20300:library_2escm"),(void*)f_20300},
{C_text("f_20310:library_2escm"),(void*)f_20310},
{C_text("f_20312:library_2escm"),(void*)f_20312},
{C_text("f_20320:library_2escm"),(void*)f_20320},
{C_text("f_20323:library_2escm"),(void*)f_20323},
{C_text("f_20333:library_2escm"),(void*)f_20333},
{C_text("f_20335:library_2escm"),(void*)f_20335},
{C_text("f_20339:library_2escm"),(void*)f_20339},
{C_text("f_20342:library_2escm"),(void*)f_20342},
{C_text("f_20345:library_2escm"),(void*)f_20345},
{C_text("f_20348:library_2escm"),(void*)f_20348},
{C_text("f_20351:library_2escm"),(void*)f_20351},
{C_text("f_20353:library_2escm"),(void*)f_20353},
{C_text("f_20359:library_2escm"),(void*)f_20359},
{C_text("f_20368:library_2escm"),(void*)f_20368},
{C_text("f_20376:library_2escm"),(void*)f_20376},
{C_text("f_20378:library_2escm"),(void*)f_20378},
{C_text("f_20392:library_2escm"),(void*)f_20392},
{C_text("f_20405:library_2escm"),(void*)f_20405},
{C_text("f_20409:library_2escm"),(void*)f_20409},
{C_text("f_20414:library_2escm"),(void*)f_20414},
{C_text("f_20418:library_2escm"),(void*)f_20418},
{C_text("f_20429:library_2escm"),(void*)f_20429},
{C_text("f_20432:library_2escm"),(void*)f_20432},
{C_text("f_20440:library_2escm"),(void*)f_20440},
{C_text("f_20442:library_2escm"),(void*)f_20442},
{C_text("f_20455:library_2escm"),(void*)f_20455},
{C_text("f_20462:library_2escm"),(void*)f_20462},
{C_text("f_20471:library_2escm"),(void*)f_20471},
{C_text("f_20478:library_2escm"),(void*)f_20478},
{C_text("f_20480:library_2escm"),(void*)f_20480},
{C_text("f_20486:library_2escm"),(void*)f_20486},
{C_text("f_20496:library_2escm"),(void*)f_20496},
{C_text("f_20499:library_2escm"),(void*)f_20499},
{C_text("f_20509:library_2escm"),(void*)f_20509},
{C_text("f_20512:library_2escm"),(void*)f_20512},
{C_text("f_20540:library_2escm"),(void*)f_20540},
{C_text("f_20544:library_2escm"),(void*)f_20544},
{C_text("f_20552:library_2escm"),(void*)f_20552},
{C_text("f_20577:library_2escm"),(void*)f_20577},
{C_text("f_20585:library_2escm"),(void*)f_20585},
{C_text("f_20587:library_2escm"),(void*)f_20587},
{C_text("f_20604:library_2escm"),(void*)f_20604},
{C_text("f_20617:library_2escm"),(void*)f_20617},
{C_text("f_20634:library_2escm"),(void*)f_20634},
{C_text("f_20651:library_2escm"),(void*)f_20651},
{C_text("f_20668:library_2escm"),(void*)f_20668},
{C_text("f_20685:library_2escm"),(void*)f_20685},
{C_text("f_20702:library_2escm"),(void*)f_20702},
{C_text("f_20719:library_2escm"),(void*)f_20719},
{C_text("f_20739:library_2escm"),(void*)f_20739},
{C_text("f_20747:library_2escm"),(void*)f_20747},
{C_text("f_20756:library_2escm"),(void*)f_20756},
{C_text("f_20762:library_2escm"),(void*)f_20762},
{C_text("f_20768:library_2escm"),(void*)f_20768},
{C_text("f_20771:library_2escm"),(void*)f_20771},
{C_text("f_20774:library_2escm"),(void*)f_20774},
{C_text("f_20784:library_2escm"),(void*)f_20784},
{C_text("f_20788:library_2escm"),(void*)f_20788},
{C_text("f_20805:library_2escm"),(void*)f_20805},
{C_text("f_20810:library_2escm"),(void*)f_20810},
{C_text("f_20817:library_2escm"),(void*)f_20817},
{C_text("f_20821:library_2escm"),(void*)f_20821},
{C_text("f_20830:library_2escm"),(void*)f_20830},
{C_text("f_20836:library_2escm"),(void*)f_20836},
{C_text("f_20846:library_2escm"),(void*)f_20846},
{C_text("f_20850:library_2escm"),(void*)f_20850},
{C_text("f_20859:library_2escm"),(void*)f_20859},
{C_text("f_20866:library_2escm"),(void*)f_20866},
{C_text("f_20879:library_2escm"),(void*)f_20879},
{C_text("f_20884:library_2escm"),(void*)f_20884},
{C_text("f_20901:library_2escm"),(void*)f_20901},
{C_text("f_20916:library_2escm"),(void*)f_20916},
{C_text("f_20929:library_2escm"),(void*)f_20929},
{C_text("f_20951:library_2escm"),(void*)f_20951},
{C_text("f_20954:library_2escm"),(void*)f_20954},
{C_text("f_20983:library_2escm"),(void*)f_20983},
{C_text("f_20993:library_2escm"),(void*)f_20993},
{C_text("f_21009:library_2escm"),(void*)f_21009},
{C_text("f_21020:library_2escm"),(void*)f_21020},
{C_text("f_21027:library_2escm"),(void*)f_21027},
{C_text("f_21125:library_2escm"),(void*)f_21125},
{C_text("f_21131:library_2escm"),(void*)f_21131},
{C_text("f_21140:library_2escm"),(void*)f_21140},
{C_text("f_21154:library_2escm"),(void*)f_21154},
{C_text("f_21157:library_2escm"),(void*)f_21157},
{C_text("f_21168:library_2escm"),(void*)f_21168},
{C_text("f_21170:library_2escm"),(void*)f_21170},
{C_text("f_21173:library_2escm"),(void*)f_21173},
{C_text("f_21190:library_2escm"),(void*)f_21190},
{C_text("f_21195:library_2escm"),(void*)f_21195},
{C_text("f_21199:library_2escm"),(void*)f_21199},
{C_text("f_21202:library_2escm"),(void*)f_21202},
{C_text("f_21205:library_2escm"),(void*)f_21205},
{C_text("f_21215:library_2escm"),(void*)f_21215},
{C_text("f_21233:library_2escm"),(void*)f_21233},
{C_text("f_21236:library_2escm"),(void*)f_21236},
{C_text("f_21245:library_2escm"),(void*)f_21245},
{C_text("f_21248:library_2escm"),(void*)f_21248},
{C_text("f_21251:library_2escm"),(void*)f_21251},
{C_text("f_21257:library_2escm"),(void*)f_21257},
{C_text("f_21270:library_2escm"),(void*)f_21270},
{C_text("f_21274:library_2escm"),(void*)f_21274},
{C_text("f_21278:library_2escm"),(void*)f_21278},
{C_text("f_21289:library_2escm"),(void*)f_21289},
{C_text("f_21293:library_2escm"),(void*)f_21293},
{C_text("f_21296:library_2escm"),(void*)f_21296},
{C_text("f_21324:library_2escm"),(void*)f_21324},
{C_text("f_21380:library_2escm"),(void*)f_21380},
{C_text("f_21384:library_2escm"),(void*)f_21384},
{C_text("f_21391:library_2escm"),(void*)f_21391},
{C_text("f_21393:library_2escm"),(void*)f_21393},
{C_text("f_21397:library_2escm"),(void*)f_21397},
{C_text("f_21411:library_2escm"),(void*)f_21411},
{C_text("f_21417:library_2escm"),(void*)f_21417},
{C_text("f_21424:library_2escm"),(void*)f_21424},
{C_text("f_21431:library_2escm"),(void*)f_21431},
{C_text("f_21452:library_2escm"),(void*)f_21452},
{C_text("f_21481:library_2escm"),(void*)f_21481},
{C_text("f_21491:library_2escm"),(void*)f_21491},
{C_text("f_21494:library_2escm"),(void*)f_21494},
{C_text("f_21530:library_2escm"),(void*)f_21530},
{C_text("f_21532:library_2escm"),(void*)f_21532},
{C_text("f_21542:library_2escm"),(void*)f_21542},
{C_text("f_21545:library_2escm"),(void*)f_21545},
{C_text("f_21597:library_2escm"),(void*)f_21597},
{C_text("f_21599:library_2escm"),(void*)f_21599},
{C_text("f_21607:library_2escm"),(void*)f_21607},
{C_text("f_21609:library_2escm"),(void*)f_21609},
{C_text("f_21619:library_2escm"),(void*)f_21619},
{C_text("f_21634:library_2escm"),(void*)f_21634},
{C_text("f_21641:library_2escm"),(void*)f_21641},
{C_text("f_21663:library_2escm"),(void*)f_21663},
{C_text("f_21671:library_2escm"),(void*)f_21671},
{C_text("f_21673:library_2escm"),(void*)f_21673},
{C_text("f_21689:library_2escm"),(void*)f_21689},
{C_text("f_21696:library_2escm"),(void*)f_21696},
{C_text("f_21709:library_2escm"),(void*)f_21709},
{C_text("f_21715:library_2escm"),(void*)f_21715},
{C_text("f_21719:library_2escm"),(void*)f_21719},
{C_text("f_21726:library_2escm"),(void*)f_21726},
{C_text("f_21734:library_2escm"),(void*)f_21734},
{C_text("f_21739:library_2escm"),(void*)f_21739},
{C_text("f_21744:library_2escm"),(void*)f_21744},
{C_text("f_21748:library_2escm"),(void*)f_21748},
{C_text("f_21757:library_2escm"),(void*)f_21757},
{C_text("f_21763:library_2escm"),(void*)f_21763},
{C_text("f_21770:library_2escm"),(void*)f_21770},
{C_text("f_21787:library_2escm"),(void*)f_21787},
{C_text("f_21793:library_2escm"),(void*)f_21793},
{C_text("f_21803:library_2escm"),(void*)f_21803},
{C_text("f_21854:library_2escm"),(void*)f_21854},
{C_text("f_21863:library_2escm"),(void*)f_21863},
{C_text("f_21870:library_2escm"),(void*)f_21870},
{C_text("f_21874:library_2escm"),(void*)f_21874},
{C_text("f_21878:library_2escm"),(void*)f_21878},
{C_text("f_21887:library_2escm"),(void*)f_21887},
{C_text("f_21898:library_2escm"),(void*)f_21898},
{C_text("f_21920:library_2escm"),(void*)f_21920},
{C_text("f_21979:library_2escm"),(void*)f_21979},
{C_text("f_21983:library_2escm"),(void*)f_21983},
{C_text("f_21985:library_2escm"),(void*)f_21985},
{C_text("f_21989:library_2escm"),(void*)f_21989},
{C_text("f_21992:library_2escm"),(void*)f_21992},
{C_text("f_22004:library_2escm"),(void*)f_22004},
{C_text("f_22024:library_2escm"),(void*)f_22024},
{C_text("f_22030:library_2escm"),(void*)f_22030},
{C_text("f_22040:library_2escm"),(void*)f_22040},
{C_text("f_22169:library_2escm"),(void*)f_22169},
{C_text("f_22176:library_2escm"),(void*)f_22176},
{C_text("f_22194:library_2escm"),(void*)f_22194},
{C_text("f_22200:library_2escm"),(void*)f_22200},
{C_text("f_22204:library_2escm"),(void*)f_22204},
{C_text("f_22240:library_2escm"),(void*)f_22240},
{C_text("f_22263:library_2escm"),(void*)f_22263},
{C_text("f_22281:library_2escm"),(void*)f_22281},
{C_text("f_22285:library_2escm"),(void*)f_22285},
{C_text("f_22292:library_2escm"),(void*)f_22292},
{C_text("f_22294:library_2escm"),(void*)f_22294},
{C_text("f_22298:library_2escm"),(void*)f_22298},
{C_text("f_22309:library_2escm"),(void*)f_22309},
{C_text("f_22325:library_2escm"),(void*)f_22325},
{C_text("f_22328:library_2escm"),(void*)f_22328},
{C_text("f_22331:library_2escm"),(void*)f_22331},
{C_text("f_22345:library_2escm"),(void*)f_22345},
{C_text("f_22351:library_2escm"),(void*)f_22351},
{C_text("f_22373:library_2escm"),(void*)f_22373},
{C_text("f_22385:library_2escm"),(void*)f_22385},
{C_text("f_22397:library_2escm"),(void*)f_22397},
{C_text("f_22406:library_2escm"),(void*)f_22406},
{C_text("f_22416:library_2escm"),(void*)f_22416},
{C_text("f_22425:library_2escm"),(void*)f_22425},
{C_text("f_22428:library_2escm"),(void*)f_22428},
{C_text("f_22446:library_2escm"),(void*)f_22446},
{C_text("f_22449:library_2escm"),(void*)f_22449},
{C_text("f_22472:library_2escm"),(void*)f_22472},
{C_text("f_22478:library_2escm"),(void*)f_22478},
{C_text("f_22506:library_2escm"),(void*)f_22506},
{C_text("f_22529:library_2escm"),(void*)f_22529},
{C_text("f_22543:library_2escm"),(void*)f_22543},
{C_text("f_22549:library_2escm"),(void*)f_22549},
{C_text("f_22574:library_2escm"),(void*)f_22574},
{C_text("f_22586:library_2escm"),(void*)f_22586},
{C_text("f_22598:library_2escm"),(void*)f_22598},
{C_text("f_22610:library_2escm"),(void*)f_22610},
{C_text("f_22622:library_2escm"),(void*)f_22622},
{C_text("f_22634:library_2escm"),(void*)f_22634},
{C_text("f_22646:library_2escm"),(void*)f_22646},
{C_text("f_22649:library_2escm"),(void*)f_22649},
{C_text("f_22652:library_2escm"),(void*)f_22652},
{C_text("f_22657:library_2escm"),(void*)f_22657},
{C_text("f_22662:library_2escm"),(void*)f_22662},
{C_text("f_22668:library_2escm"),(void*)f_22668},
{C_text("f_22710:library_2escm"),(void*)f_22710},
{C_text("f_22722:library_2escm"),(void*)f_22722},
{C_text("f_22725:library_2escm"),(void*)f_22725},
{C_text("f_22737:library_2escm"),(void*)f_22737},
{C_text("f_22749:library_2escm"),(void*)f_22749},
{C_text("f_22752:library_2escm"),(void*)f_22752},
{C_text("f_22764:library_2escm"),(void*)f_22764},
{C_text("f_22776:library_2escm"),(void*)f_22776},
{C_text("f_22779:library_2escm"),(void*)f_22779},
{C_text("f_22788:library_2escm"),(void*)f_22788},
{C_text("f_22803:library_2escm"),(void*)f_22803},
{C_text("f_22806:library_2escm"),(void*)f_22806},
{C_text("f_22811:library_2escm"),(void*)f_22811},
{C_text("f_22816:library_2escm"),(void*)f_22816},
{C_text("f_22822:library_2escm"),(void*)f_22822},
{C_text("f_22851:library_2escm"),(void*)f_22851},
{C_text("f_22863:library_2escm"),(void*)f_22863},
{C_text("f_22866:library_2escm"),(void*)f_22866},
{C_text("f_22869:library_2escm"),(void*)f_22869},
{C_text("f_22884:library_2escm"),(void*)f_22884},
{C_text("f_22893:library_2escm"),(void*)f_22893},
{C_text("f_22896:library_2escm"),(void*)f_22896},
{C_text("f_22902:library_2escm"),(void*)f_22902},
{C_text("f_22905:library_2escm"),(void*)f_22905},
{C_text("f_22911:library_2escm"),(void*)f_22911},
{C_text("f_22930:library_2escm"),(void*)f_22930},
{C_text("f_22949:library_2escm"),(void*)f_22949},
{C_text("f_22965:library_2escm"),(void*)f_22965},
{C_text("f_22971:library_2escm"),(void*)f_22971},
{C_text("f_23043:library_2escm"),(void*)f_23043},
{C_text("f_23055:library_2escm"),(void*)f_23055},
{C_text("f_23133:library_2escm"),(void*)f_23133},
{C_text("f_23209:library_2escm"),(void*)f_23209},
{C_text("f_23374:library_2escm"),(void*)f_23374},
{C_text("f_23386:library_2escm"),(void*)f_23386},
{C_text("f_23448:library_2escm"),(void*)f_23448},
{C_text("f_23451:library_2escm"),(void*)f_23451},
{C_text("f_23497:library_2escm"),(void*)f_23497},
{C_text("f_23501:library_2escm"),(void*)f_23501},
{C_text("f_23519:library_2escm"),(void*)f_23519},
{C_text("f_23583:library_2escm"),(void*)f_23583},
{C_text("f_23594:library_2escm"),(void*)f_23594},
{C_text("f_23596:library_2escm"),(void*)f_23596},
{C_text("f_23609:library_2escm"),(void*)f_23609},
{C_text("f_23621:library_2escm"),(void*)f_23621},
{C_text("f_23641:library_2escm"),(void*)f_23641},
{C_text("f_23648:library_2escm"),(void*)f_23648},
{C_text("f_23657:library_2escm"),(void*)f_23657},
{C_text("f_23669:library_2escm"),(void*)f_23669},
{C_text("f_23690:library_2escm"),(void*)f_23690},
{C_text("f_23705:library_2escm"),(void*)f_23705},
{C_text("f_23723:library_2escm"),(void*)f_23723},
{C_text("f_23741:library_2escm"),(void*)f_23741},
{C_text("f_23760:library_2escm"),(void*)f_23760},
{C_text("f_23770:library_2escm"),(void*)f_23770},
{C_text("f_23784:library_2escm"),(void*)f_23784},
{C_text("f_23787:library_2escm"),(void*)f_23787},
{C_text("f_23792:library_2escm"),(void*)f_23792},
{C_text("f_23811:library_2escm"),(void*)f_23811},
{C_text("f_23821:library_2escm"),(void*)f_23821},
{C_text("f_23835:library_2escm"),(void*)f_23835},
{C_text("f_23838:library_2escm"),(void*)f_23838},
{C_text("f_23845:library_2escm"),(void*)f_23845},
{C_text("f_23849:library_2escm"),(void*)f_23849},
{C_text("f_23853:library_2escm"),(void*)f_23853},
{C_text("f_23855:library_2escm"),(void*)f_23855},
{C_text("f_23862:library_2escm"),(void*)f_23862},
{C_text("f_23865:library_2escm"),(void*)f_23865},
{C_text("f_23868:library_2escm"),(void*)f_23868},
{C_text("f_23870:library_2escm"),(void*)f_23870},
{C_text("f_23880:library_2escm"),(void*)f_23880},
{C_text("f_23886:library_2escm"),(void*)f_23886},
{C_text("f_23898:library_2escm"),(void*)f_23898},
{C_text("f_23901:library_2escm"),(void*)f_23901},
{C_text("f_23904:library_2escm"),(void*)f_23904},
{C_text("f_23921:library_2escm"),(void*)f_23921},
{C_text("f_23942:library_2escm"),(void*)f_23942},
{C_text("f_23946:library_2escm"),(void*)f_23946},
{C_text("f_23959:library_2escm"),(void*)f_23959},
{C_text("f_23962:library_2escm"),(void*)f_23962},
{C_text("f_23971:library_2escm"),(void*)f_23971},
{C_text("f_23974:library_2escm"),(void*)f_23974},
{C_text("f_24002:library_2escm"),(void*)f_24002},
{C_text("f_24006:library_2escm"),(void*)f_24006},
{C_text("f_24015:library_2escm"),(void*)f_24015},
{C_text("f_24026:library_2escm"),(void*)f_24026},
{C_text("f_24033:library_2escm"),(void*)f_24033},
{C_text("f_24038:library_2escm"),(void*)f_24038},
{C_text("f_24060:library_2escm"),(void*)f_24060},
{C_text("f_24066:library_2escm"),(void*)f_24066},
{C_text("f_24069:library_2escm"),(void*)f_24069},
{C_text("f_24080:library_2escm"),(void*)f_24080},
{C_text("f_24089:library_2escm"),(void*)f_24089},
{C_text("f_24092:library_2escm"),(void*)f_24092},
{C_text("f_24115:library_2escm"),(void*)f_24115},
{C_text("f_24164:library_2escm"),(void*)f_24164},
{C_text("f_24180:library_2escm"),(void*)f_24180},
{C_text("f_24187:library_2escm"),(void*)f_24187},
{C_text("f_24199:library_2escm"),(void*)f_24199},
{C_text("f_24304:library_2escm"),(void*)f_24304},
{C_text("f_24309:library_2escm"),(void*)f_24309},
{C_text("f_24358:library_2escm"),(void*)f_24358},
{C_text("f_24364:library_2escm"),(void*)f_24364},
{C_text("f_24368:library_2escm"),(void*)f_24368},
{C_text("f_24391:library_2escm"),(void*)f_24391},
{C_text("f_24398:library_2escm"),(void*)f_24398},
{C_text("f_24417:library_2escm"),(void*)f_24417},
{C_text("f_24453:library_2escm"),(void*)f_24453},
{C_text("f_24465:library_2escm"),(void*)f_24465},
{C_text("f_24471:library_2escm"),(void*)f_24471},
{C_text("f_24493:library_2escm"),(void*)f_24493},
{C_text("f_24500:library_2escm"),(void*)f_24500},
{C_text("f_24510:library_2escm"),(void*)f_24510},
{C_text("f_24519:library_2escm"),(void*)f_24519},
{C_text("f_24525:library_2escm"),(void*)f_24525},
{C_text("f_24538:library_2escm"),(void*)f_24538},
{C_text("f_24576:library_2escm"),(void*)f_24576},
{C_text("f_24642:library_2escm"),(void*)f_24642},
{C_text("f_24645:library_2escm"),(void*)f_24645},
{C_text("f_24652:library_2escm"),(void*)f_24652},
{C_text("f_24692:library_2escm"),(void*)f_24692},
{C_text("f_24695:library_2escm"),(void*)f_24695},
{C_text("f_24704:library_2escm"),(void*)f_24704},
{C_text("f_24717:library_2escm"),(void*)f_24717},
{C_text("f_24726:library_2escm"),(void*)f_24726},
{C_text("f_24736:library_2escm"),(void*)f_24736},
{C_text("f_24739:library_2escm"),(void*)f_24739},
{C_text("f_24765:library_2escm"),(void*)f_24765},
{C_text("f_24771:library_2escm"),(void*)f_24771},
{C_text("f_24776:library_2escm"),(void*)f_24776},
{C_text("f_24789:library_2escm"),(void*)f_24789},
{C_text("f_24792:library_2escm"),(void*)f_24792},
{C_text("f_24803:library_2escm"),(void*)f_24803},
{C_text("f_24825:library_2escm"),(void*)f_24825},
{C_text("f_24837:library_2escm"),(void*)f_24837},
{C_text("f_24840:library_2escm"),(void*)f_24840},
{C_text("f_24847:library_2escm"),(void*)f_24847},
{C_text("f_24856:library_2escm"),(void*)f_24856},
{C_text("f_24859:library_2escm"),(void*)f_24859},
{C_text("f_24903:library_2escm"),(void*)f_24903},
{C_text("f_24906:library_2escm"),(void*)f_24906},
{C_text("f_24915:library_2escm"),(void*)f_24915},
{C_text("f_24928:library_2escm"),(void*)f_24928},
{C_text("f_24931:library_2escm"),(void*)f_24931},
{C_text("f_24956:library_2escm"),(void*)f_24956},
{C_text("f_24960:library_2escm"),(void*)f_24960},
{C_text("f_24970:library_2escm"),(void*)f_24970},
{C_text("f_24973:library_2escm"),(void*)f_24973},
{C_text("f_24988:library_2escm"),(void*)f_24988},
{C_text("f_24995:library_2escm"),(void*)f_24995},
{C_text("f_25017:library_2escm"),(void*)f_25017},
{C_text("f_25019:library_2escm"),(void*)f_25019},
{C_text("f_25032:library_2escm"),(void*)f_25032},
{C_text("f_25037:library_2escm"),(void*)f_25037},
{C_text("f_25043:library_2escm"),(void*)f_25043},
{C_text("f_25049:library_2escm"),(void*)f_25049},
{C_text("f_25053:library_2escm"),(void*)f_25053},
{C_text("f_25056:library_2escm"),(void*)f_25056},
{C_text("f_25071:library_2escm"),(void*)f_25071},
{C_text("f_25074:library_2escm"),(void*)f_25074},
{C_text("f_25092:library_2escm"),(void*)f_25092},
{C_text("f_25094:library_2escm"),(void*)f_25094},
{C_text("f_25100:library_2escm"),(void*)f_25100},
{C_text("f_25109:library_2escm"),(void*)f_25109},
{C_text("f_25115:library_2escm"),(void*)f_25115},
{C_text("f_25122:library_2escm"),(void*)f_25122},
{C_text("f_25125:library_2escm"),(void*)f_25125},
{C_text("f_25128:library_2escm"),(void*)f_25128},
{C_text("f_25137:library_2escm"),(void*)f_25137},
{C_text("f_25153:library_2escm"),(void*)f_25153},
{C_text("f_25168:library_2escm"),(void*)f_25168},
{C_text("f_25170:library_2escm"),(void*)f_25170},
{C_text("f_25176:library_2escm"),(void*)f_25176},
{C_text("f_25182:library_2escm"),(void*)f_25182},
{C_text("f_25186:library_2escm"),(void*)f_25186},
{C_text("f_25189:library_2escm"),(void*)f_25189},
{C_text("f_25192:library_2escm"),(void*)f_25192},
{C_text("f_25196:library_2escm"),(void*)f_25196},
{C_text("f_25200:library_2escm"),(void*)f_25200},
{C_text("f_25204:library_2escm"),(void*)f_25204},
{C_text("f_25207:library_2escm"),(void*)f_25207},
{C_text("f_25210:library_2escm"),(void*)f_25210},
{C_text("f_25213:library_2escm"),(void*)f_25213},
{C_text("f_25225:library_2escm"),(void*)f_25225},
{C_text("f_25231:library_2escm"),(void*)f_25231},
{C_text("f_25235:library_2escm"),(void*)f_25235},
{C_text("f_25238:library_2escm"),(void*)f_25238},
{C_text("f_25241:library_2escm"),(void*)f_25241},
{C_text("f_25244:library_2escm"),(void*)f_25244},
{C_text("f_25247:library_2escm"),(void*)f_25247},
{C_text("f_25250:library_2escm"),(void*)f_25250},
{C_text("f_25255:library_2escm"),(void*)f_25255},
{C_text("f_25277:library_2escm"),(void*)f_25277},
{C_text("f_25283:library_2escm"),(void*)f_25283},
{C_text("f_25286:library_2escm"),(void*)f_25286},
{C_text("f_25311:library_2escm"),(void*)f_25311},
{C_text("f_25314:library_2escm"),(void*)f_25314},
{C_text("f_25321:library_2escm"),(void*)f_25321},
{C_text("f_25334:library_2escm"),(void*)f_25334},
{C_text("f_25344:library_2escm"),(void*)f_25344},
{C_text("f_25349:library_2escm"),(void*)f_25349},
{C_text("f_25355:library_2escm"),(void*)f_25355},
{C_text("f_25382:library_2escm"),(void*)f_25382},
{C_text("f_25388:library_2escm"),(void*)f_25388},
{C_text("f_25399:library_2escm"),(void*)f_25399},
{C_text("f_25408:library_2escm"),(void*)f_25408},
{C_text("f_25413:library_2escm"),(void*)f_25413},
{C_text("f_25419:library_2escm"),(void*)f_25419},
{C_text("f_25443:library_2escm"),(void*)f_25443},
{C_text("f_25451:library_2escm"),(void*)f_25451},
{C_text("f_25468:library_2escm"),(void*)f_25468},
{C_text("f_25508:library_2escm"),(void*)f_25508},
{C_text("f_25515:library_2escm"),(void*)f_25515},
{C_text("f_25527:library_2escm"),(void*)f_25527},
{C_text("f_25531:library_2escm"),(void*)f_25531},
{C_text("f_25544:library_2escm"),(void*)f_25544},
{C_text("f_25546:library_2escm"),(void*)f_25546},
{C_text("f_25580:library_2escm"),(void*)f_25580},
{C_text("f_25584:library_2escm"),(void*)f_25584},
{C_text("f_25585:library_2escm"),(void*)f_25585},
{C_text("f_25595:library_2escm"),(void*)f_25595},
{C_text("f_25600:library_2escm"),(void*)f_25600},
{C_text("f_25610:library_2escm"),(void*)f_25610},
{C_text("f_25623:library_2escm"),(void*)f_25623},
{C_text("f_25634:library_2escm"),(void*)f_25634},
{C_text("f_25638:library_2escm"),(void*)f_25638},
{C_text("f_25642:library_2escm"),(void*)f_25642},
{C_text("f_25664:library_2escm"),(void*)f_25664},
{C_text("f_25668:library_2escm"),(void*)f_25668},
{C_text("f_25676:library_2escm"),(void*)f_25676},
{C_text("f_25692:library_2escm"),(void*)f_25692},
{C_text("f_25703:library_2escm"),(void*)f_25703},
{C_text("f_25722:library_2escm"),(void*)f_25722},
{C_text("f_25733:library_2escm"),(void*)f_25733},
{C_text("f_25741:library_2escm"),(void*)f_25741},
{C_text("f_25820:library_2escm"),(void*)f_25820},
{C_text("f_25830:library_2escm"),(void*)f_25830},
{C_text("f_25831:library_2escm"),(void*)f_25831},
{C_text("f_25844:library_2escm"),(void*)f_25844},
{C_text("f_25847:library_2escm"),(void*)f_25847},
{C_text("f_25850:library_2escm"),(void*)f_25850},
{C_text("f_25853:library_2escm"),(void*)f_25853},
{C_text("f_25861:library_2escm"),(void*)f_25861},
{C_text("f_25868:library_2escm"),(void*)f_25868},
{C_text("f_25871:library_2escm"),(void*)f_25871},
{C_text("f_25874:library_2escm"),(void*)f_25874},
{C_text("f_25897:library_2escm"),(void*)f_25897},
{C_text("f_25902:library_2escm"),(void*)f_25902},
{C_text("f_25912:library_2escm"),(void*)f_25912},
{C_text("f_25925:library_2escm"),(void*)f_25925},
{C_text("f_25956:library_2escm"),(void*)f_25956},
{C_text("f_25966:library_2escm"),(void*)f_25966},
{C_text("f_26016:library_2escm"),(void*)f_26016},
{C_text("f_26019:library_2escm"),(void*)f_26019},
{C_text("f_26038:library_2escm"),(void*)f_26038},
{C_text("f_26043:library_2escm"),(void*)f_26043},
{C_text("f_26049:library_2escm"),(void*)f_26049},
{C_text("f_26052:library_2escm"),(void*)f_26052},
{C_text("f_26061:library_2escm"),(void*)f_26061},
{C_text("f_26067:library_2escm"),(void*)f_26067},
{C_text("f_26086:library_2escm"),(void*)f_26086},
{C_text("f_26105:library_2escm"),(void*)f_26105},
{C_text("f_26110:library_2escm"),(void*)f_26110},
{C_text("f_26120:library_2escm"),(void*)f_26120},
{C_text("f_26134:library_2escm"),(void*)f_26134},
{C_text("f_26141:library_2escm"),(void*)f_26141},
{C_text("f_26145:library_2escm"),(void*)f_26145},
{C_text("f_26149:library_2escm"),(void*)f_26149},
{C_text("f_26151:library_2escm"),(void*)f_26151},
{C_text("f_26177:library_2escm"),(void*)f_26177},
{C_text("f_26180:library_2escm"),(void*)f_26180},
{C_text("f_26183:library_2escm"),(void*)f_26183},
{C_text("f_26190:library_2escm"),(void*)f_26190},
{C_text("f_26194:library_2escm"),(void*)f_26194},
{C_text("f_26200:library_2escm"),(void*)f_26200},
{C_text("f_26205:library_2escm"),(void*)f_26205},
{C_text("f_26215:library_2escm"),(void*)f_26215},
{C_text("f_26253:library_2escm"),(void*)f_26253},
{C_text("f_26256:library_2escm"),(void*)f_26256},
{C_text("f_26276:library_2escm"),(void*)f_26276},
{C_text("f_26284:library_2escm"),(void*)f_26284},
{C_text("f_26418:library_2escm"),(void*)f_26418},
{C_text("f_26435:library_2escm"),(void*)f_26435},
{C_text("f_26439:library_2escm"),(void*)f_26439},
{C_text("f_26452:library_2escm"),(void*)f_26452},
{C_text("f_26461:library_2escm"),(void*)f_26461},
{C_text("f_26464:library_2escm"),(void*)f_26464},
{C_text("f_26468:library_2escm"),(void*)f_26468},
{C_text("f_26507:library_2escm"),(void*)f_26507},
{C_text("f_26510:library_2escm"),(void*)f_26510},
{C_text("f_26517:library_2escm"),(void*)f_26517},
{C_text("f_26521:library_2escm"),(void*)f_26521},
{C_text("f_26547:library_2escm"),(void*)f_26547},
{C_text("f_26578:library_2escm"),(void*)f_26578},
{C_text("f_26581:library_2escm"),(void*)f_26581},
{C_text("f_26593:library_2escm"),(void*)f_26593},
{C_text("f_26596:library_2escm"),(void*)f_26596},
{C_text("f_26599:library_2escm"),(void*)f_26599},
{C_text("f_26624:library_2escm"),(void*)f_26624},
{C_text("f_26630:library_2escm"),(void*)f_26630},
{C_text("f_26634:library_2escm"),(void*)f_26634},
{C_text("f_26638:library_2escm"),(void*)f_26638},
{C_text("f_26651:library_2escm"),(void*)f_26651},
{C_text("f_26653:library_2escm"),(void*)f_26653},
{C_text("f_26659:library_2escm"),(void*)f_26659},
{C_text("f_26664:library_2escm"),(void*)f_26664},
{C_text("f_26699:library_2escm"),(void*)f_26699},
{C_text("f_26705:library_2escm"),(void*)f_26705},
{C_text("f_26721:library_2escm"),(void*)f_26721},
{C_text("f_26743:library_2escm"),(void*)f_26743},
{C_text("f_26767:library_2escm"),(void*)f_26767},
{C_text("f_26779:library_2escm"),(void*)f_26779},
{C_text("f_26781:library_2escm"),(void*)f_26781},
{C_text("f_26800:library_2escm"),(void*)f_26800},
{C_text("f_26804:library_2escm"),(void*)f_26804},
{C_text("f_26816:library_2escm"),(void*)f_26816},
{C_text("f_26818:library_2escm"),(void*)f_26818},
{C_text("f_26862:library_2escm"),(void*)f_26862},
{C_text("f_26864:library_2escm"),(void*)f_26864},
{C_text("f_26898:library_2escm"),(void*)f_26898},
{C_text("f_26921:library_2escm"),(void*)f_26921},
{C_text("f_26928:library_2escm"),(void*)f_26928},
{C_text("f_26938:library_2escm"),(void*)f_26938},
{C_text("f_26940:library_2escm"),(void*)f_26940},
{C_text("f_26990:library_2escm"),(void*)f_26990},
{C_text("f_26992:library_2escm"),(void*)f_26992},
{C_text("f_27017:library_2escm"),(void*)f_27017},
{C_text("f_27026:library_2escm"),(void*)f_27026},
{C_text("f_27032:library_2escm"),(void*)f_27032},
{C_text("f_27036:library_2escm"),(void*)f_27036},
{C_text("f_27040:library_2escm"),(void*)f_27040},
{C_text("f_27048:library_2escm"),(void*)f_27048},
{C_text("f_27054:library_2escm"),(void*)f_27054},
{C_text("f_27085:library_2escm"),(void*)f_27085},
{C_text("f_27110:library_2escm"),(void*)f_27110},
{C_text("f_27135:library_2escm"),(void*)f_27135},
{C_text("f_27145:library_2escm"),(void*)f_27145},
{C_text("f_27150:library_2escm"),(void*)f_27150},
{C_text("f_27152:library_2escm"),(void*)f_27152},
{C_text("f_27159:library_2escm"),(void*)f_27159},
{C_text("f_27171:library_2escm"),(void*)f_27171},
{C_text("f_27179:library_2escm"),(void*)f_27179},
{C_text("f_27217:library_2escm"),(void*)f_27217},
{C_text("f_27221:library_2escm"),(void*)f_27221},
{C_text("f_27228:library_2escm"),(void*)f_27228},
{C_text("f_27231:library_2escm"),(void*)f_27231},
{C_text("f_27234:library_2escm"),(void*)f_27234},
{C_text("f_27235:library_2escm"),(void*)f_27235},
{C_text("f_27237:library_2escm"),(void*)f_27237},
{C_text("f_27241:library_2escm"),(void*)f_27241},
{C_text("f_27246:library_2escm"),(void*)f_27246},
{C_text("f_27258:library_2escm"),(void*)f_27258},
{C_text("f_27268:library_2escm"),(void*)f_27268},
{C_text("f_27280:library_2escm"),(void*)f_27280},
{C_text("f_27282:library_2escm"),(void*)f_27282},
{C_text("f_27289:library_2escm"),(void*)f_27289},
{C_text("f_27292:library_2escm"),(void*)f_27292},
{C_text("f_27301:library_2escm"),(void*)f_27301},
{C_text("f_27304:library_2escm"),(void*)f_27304},
{C_text("f_27307:library_2escm"),(void*)f_27307},
{C_text("f_27322:library_2escm"),(void*)f_27322},
{C_text("f_27328:library_2escm"),(void*)f_27328},
{C_text("f_27336:library_2escm"),(void*)f_27336},
{C_text("f_27340:library_2escm"),(void*)f_27340},
{C_text("f_27343:library_2escm"),(void*)f_27343},
{C_text("f_27346:library_2escm"),(void*)f_27346},
{C_text("f_27359:library_2escm"),(void*)f_27359},
{C_text("f_27363:library_2escm"),(void*)f_27363},
{C_text("f_27384:library_2escm"),(void*)f_27384},
{C_text("f_27400:library_2escm"),(void*)f_27400},
{C_text("f_27403:library_2escm"),(void*)f_27403},
{C_text("f_27409:library_2escm"),(void*)f_27409},
{C_text("f_27431:library_2escm"),(void*)f_27431},
{C_text("f_27436:library_2escm"),(void*)f_27436},
{C_text("f_27471:library_2escm"),(void*)f_27471},
{C_text("f_27481:library_2escm"),(void*)f_27481},
{C_text("f_27484:library_2escm"),(void*)f_27484},
{C_text("f_27487:library_2escm"),(void*)f_27487},
{C_text("f_27490:library_2escm"),(void*)f_27490},
{C_text("f_27493:library_2escm"),(void*)f_27493},
{C_text("f_27496:library_2escm"),(void*)f_27496},
{C_text("f_27503:library_2escm"),(void*)f_27503},
{C_text("f_27506:library_2escm"),(void*)f_27506},
{C_text("f_27509:library_2escm"),(void*)f_27509},
{C_text("f_27520:library_2escm"),(void*)f_27520},
{C_text("f_27543:library_2escm"),(void*)f_27543},
{C_text("f_27554:library_2escm"),(void*)f_27554},
{C_text("f_27558:library_2escm"),(void*)f_27558},
{C_text("f_27580:library_2escm"),(void*)f_27580},
{C_text("f_27591:library_2escm"),(void*)f_27591},
{C_text("f_27595:library_2escm"),(void*)f_27595},
{C_text("f_28094:library_2escm"),(void*)f_28094},
{C_text("f_28105:library_2escm"),(void*)f_28105},
{C_text("f_28109:library_2escm"),(void*)f_28109},
{C_text("f_28226:library_2escm"),(void*)f_28226},
{C_text("f_28229:library_2escm"),(void*)f_28229},
{C_text("f_28232:library_2escm"),(void*)f_28232},
{C_text("f_28235:library_2escm"),(void*)f_28235},
{C_text("f_28240:library_2escm"),(void*)f_28240},
{C_text("f_28243:library_2escm"),(void*)f_28243},
{C_text("f_28246:library_2escm"),(void*)f_28246},
{C_text("f_28249:library_2escm"),(void*)f_28249},
{C_text("f_28252:library_2escm"),(void*)f_28252},
{C_text("f_28255:library_2escm"),(void*)f_28255},
{C_text("f_28258:library_2escm"),(void*)f_28258},
{C_text("f_28261:library_2escm"),(void*)f_28261},
{C_text("f_28264:library_2escm"),(void*)f_28264},
{C_text("f_28268:library_2escm"),(void*)f_28268},
{C_text("f_28270:library_2escm"),(void*)f_28270},
{C_text("f_28278:library_2escm"),(void*)f_28278},
{C_text("f_28280:library_2escm"),(void*)f_28280},
{C_text("f_28284:library_2escm"),(void*)f_28284},
{C_text("f_28286:library_2escm"),(void*)f_28286},
{C_text("f_28293:library_2escm"),(void*)f_28293},
{C_text("f_28303:library_2escm"),(void*)f_28303},
{C_text("f_28328:library_2escm"),(void*)f_28328},
{C_text("f_28331:library_2escm"),(void*)f_28331},
{C_text("f_28334:library_2escm"),(void*)f_28334},
{C_text("f_28337:library_2escm"),(void*)f_28337},
{C_text("f_28340:library_2escm"),(void*)f_28340},
{C_text("f_28350:library_2escm"),(void*)f_28350},
{C_text("f_28356:library_2escm"),(void*)f_28356},
{C_text("f_28360:library_2escm"),(void*)f_28360},
{C_text("f_28362:library_2escm"),(void*)f_28362},
{C_text("f_28372:library_2escm"),(void*)f_28372},
{C_text("f_28378:library_2escm"),(void*)f_28378},
{C_text("f_28382:library_2escm"),(void*)f_28382},
{C_text("f_28384:library_2escm"),(void*)f_28384},
{C_text("f_28389:library_2escm"),(void*)f_28389},
{C_text("f_28391:library_2escm"),(void*)f_28391},
{C_text("f_28394:library_2escm"),(void*)f_28394},
{C_text("f_28414:library_2escm"),(void*)f_28414},
{C_text("f_28420:library_2escm"),(void*)f_28420},
{C_text("f_28430:library_2escm"),(void*)f_28430},
{C_text("f_28440:library_2escm"),(void*)f_28440},
{C_text("f_28475:library_2escm"),(void*)f_28475},
{C_text("f_28479:library_2escm"),(void*)f_28479},
{C_text("f_28484:library_2escm"),(void*)f_28484},
{C_text("f_28494:library_2escm"),(void*)f_28494},
{C_text("f_28504:library_2escm"),(void*)f_28504},
{C_text("f_28513:library_2escm"),(void*)f_28513},
{C_text("f_28516:library_2escm"),(void*)f_28516},
{C_text("f_28519:library_2escm"),(void*)f_28519},
{C_text("f_28526:library_2escm"),(void*)f_28526},
{C_text("f_28533:library_2escm"),(void*)f_28533},
{C_text("f_28536:library_2escm"),(void*)f_28536},
{C_text("f_28540:library_2escm"),(void*)f_28540},
{C_text("f_28542:library_2escm"),(void*)f_28542},
{C_text("f_28545:library_2escm"),(void*)f_28545},
{C_text("f_28548:library_2escm"),(void*)f_28548},
{C_text("f_28551:library_2escm"),(void*)f_28551},
{C_text("f_28554:library_2escm"),(void*)f_28554},
{C_text("f_28557:library_2escm"),(void*)f_28557},
{C_text("f_28560:library_2escm"),(void*)f_28560},
{C_text("f_28563:library_2escm"),(void*)f_28563},
{C_text("f_28566:library_2escm"),(void*)f_28566},
{C_text("f_28569:library_2escm"),(void*)f_28569},
{C_text("f_28572:library_2escm"),(void*)f_28572},
{C_text("f_28576:library_2escm"),(void*)f_28576},
{C_text("f_28579:library_2escm"),(void*)f_28579},
{C_text("f_28595:library_2escm"),(void*)f_28595},
{C_text("f_28603:library_2escm"),(void*)f_28603},
{C_text("f_28606:library_2escm"),(void*)f_28606},
{C_text("f_28612:library_2escm"),(void*)f_28612},
{C_text("f_28621:library_2escm"),(void*)f_28621},
{C_text("f_28627:library_2escm"),(void*)f_28627},
{C_text("f_28636:library_2escm"),(void*)f_28636},
{C_text("f_28642:library_2escm"),(void*)f_28642},
{C_text("f_28648:library_2escm"),(void*)f_28648},
{C_text("f_28651:library_2escm"),(void*)f_28651},
{C_text("f_28655:library_2escm"),(void*)f_28655},
{C_text("f_28658:library_2escm"),(void*)f_28658},
{C_text("f_28663:library_2escm"),(void*)f_28663},
{C_text("f_28665:library_2escm"),(void*)f_28665},
{C_text("f_28669:library_2escm"),(void*)f_28669},
{C_text("f_28696:library_2escm"),(void*)f_28696},
{C_text("f_28706:library_2escm"),(void*)f_28706},
{C_text("f_28715:library_2escm"),(void*)f_28715},
{C_text("f_28722:library_2escm"),(void*)f_28722},
{C_text("f_28726:library_2escm"),(void*)f_28726},
{C_text("f_28740:library_2escm"),(void*)f_28740},
{C_text("f_28744:library_2escm"),(void*)f_28744},
{C_text("f_28751:library_2escm"),(void*)f_28751},
{C_text("f_28753:library_2escm"),(void*)f_28753},
{C_text("f_28775:library_2escm"),(void*)f_28775},
{C_text("f_28782:library_2escm"),(void*)f_28782},
{C_text("f_28787:library_2escm"),(void*)f_28787},
{C_text("f_28791:library_2escm"),(void*)f_28791},
{C_text("f_28794:library_2escm"),(void*)f_28794},
{C_text("f_28797:library_2escm"),(void*)f_28797},
{C_text("f_28802:library_2escm"),(void*)f_28802},
{C_text("f_28806:library_2escm"),(void*)f_28806},
{C_text("f_28832:library_2escm"),(void*)f_28832},
{C_text("f_28845:library_2escm"),(void*)f_28845},
{C_text("f_28848:library_2escm"),(void*)f_28848},
{C_text("f_28857:library_2escm"),(void*)f_28857},
{C_text("f_28860:library_2escm"),(void*)f_28860},
{C_text("f_28863:library_2escm"),(void*)f_28863},
{C_text("f_28869:library_2escm"),(void*)f_28869},
{C_text("f_28876:library_2escm"),(void*)f_28876},
{C_text("f_28878:library_2escm"),(void*)f_28878},
{C_text("f_28891:library_2escm"),(void*)f_28891},
{C_text("f_28904:library_2escm"),(void*)f_28904},
{C_text("f_28907:library_2escm"),(void*)f_28907},
{C_text("f_28910:library_2escm"),(void*)f_28910},
{C_text("f_28917:library_2escm"),(void*)f_28917},
{C_text("f_28970:library_2escm"),(void*)f_28970},
{C_text("f_28973:library_2escm"),(void*)f_28973},
{C_text("f_28976:library_2escm"),(void*)f_28976},
{C_text("f_28978:library_2escm"),(void*)f_28978},
{C_text("f_28982:library_2escm"),(void*)f_28982},
{C_text("f_28986:library_2escm"),(void*)f_28986},
{C_text("f_28992:library_2escm"),(void*)f_28992},
{C_text("f_28997:library_2escm"),(void*)f_28997},
{C_text("f_29001:library_2escm"),(void*)f_29001},
{C_text("f_29013:library_2escm"),(void*)f_29013},
{C_text("f_29025:library_2escm"),(void*)f_29025},
{C_text("f_29032:library_2escm"),(void*)f_29032},
{C_text("f_29038:library_2escm"),(void*)f_29038},
{C_text("f_29052:library_2escm"),(void*)f_29052},
{C_text("f_29091:library_2escm"),(void*)f_29091},
{C_text("f_29110:library_2escm"),(void*)f_29110},
{C_text("f_29133:library_2escm"),(void*)f_29133},
{C_text("f_29142:library_2escm"),(void*)f_29142},
{C_text("f_29149:library_2escm"),(void*)f_29149},
{C_text("f_29166:library_2escm"),(void*)f_29166},
{C_text("f_29174:library_2escm"),(void*)f_29174},
{C_text("f_29185:library_2escm"),(void*)f_29185},
{C_text("f_29193:library_2escm"),(void*)f_29193},
{C_text("f_29200:library_2escm"),(void*)f_29200},
{C_text("f_29262:library_2escm"),(void*)f_29262},
{C_text("f_29269:library_2escm"),(void*)f_29269},
{C_text("f_29272:library_2escm"),(void*)f_29272},
{C_text("f_29278:library_2escm"),(void*)f_29278},
{C_text("f_29285:library_2escm"),(void*)f_29285},
{C_text("f_29289:library_2escm"),(void*)f_29289},
{C_text("f_29292:library_2escm"),(void*)f_29292},
{C_text("f_29300:library_2escm"),(void*)f_29300},
{C_text("f_29302:library_2escm"),(void*)f_29302},
{C_text("f_29311:library_2escm"),(void*)f_29311},
{C_text("f_29317:library_2escm"),(void*)f_29317},
{C_text("f_29328:library_2escm"),(void*)f_29328},
{C_text("f_29332:library_2escm"),(void*)f_29332},
{C_text("f_29334:library_2escm"),(void*)f_29334},
{C_text("f_29342:library_2escm"),(void*)f_29342},
{C_text("f_29354:library_2escm"),(void*)f_29354},
{C_text("f_29360:library_2escm"),(void*)f_29360},
{C_text("f_29364:library_2escm"),(void*)f_29364},
{C_text("f_29372:library_2escm"),(void*)f_29372},
{C_text("f_29390:library_2escm"),(void*)f_29390},
{C_text("f_29398:library_2escm"),(void*)f_29398},
{C_text("f_29402:library_2escm"),(void*)f_29402},
{C_text("f_29420:library_2escm"),(void*)f_29420},
{C_text("f_29427:library_2escm"),(void*)f_29427},
{C_text("f_29439:library_2escm"),(void*)f_29439},
{C_text("f_29447:library_2escm"),(void*)f_29447},
{C_text("f_29453:library_2escm"),(void*)f_29453},
{C_text("f_29475:library_2escm"),(void*)f_29475},
{C_text("f_29479:library_2escm"),(void*)f_29479},
{C_text("f_29483:library_2escm"),(void*)f_29483},
{C_text("f_29485:library_2escm"),(void*)f_29485},
{C_text("f_29488:library_2escm"),(void*)f_29488},
{C_text("f_29491:library_2escm"),(void*)f_29491},
{C_text("f_29495:library_2escm"),(void*)f_29495},
{C_text("f_29498:library_2escm"),(void*)f_29498},
{C_text("f_29520:library_2escm"),(void*)f_29520},
{C_text("f_29526:library_2escm"),(void*)f_29526},
{C_text("f_29530:library_2escm"),(void*)f_29530},
{C_text("f_29547:library_2escm"),(void*)f_29547},
{C_text("f_29557:library_2escm"),(void*)f_29557},
{C_text("f_29561:library_2escm"),(void*)f_29561},
{C_text("f_29565:library_2escm"),(void*)f_29565},
{C_text("f_29577:library_2escm"),(void*)f_29577},
{C_text("f_29587:library_2escm"),(void*)f_29587},
{C_text("f_29591:library_2escm"),(void*)f_29591},
{C_text("f_29595:library_2escm"),(void*)f_29595},
{C_text("f_29626:library_2escm"),(void*)f_29626},
{C_text("f_29629:library_2escm"),(void*)f_29629},
{C_text("f_29632:library_2escm"),(void*)f_29632},
{C_text("f_29638:library_2escm"),(void*)f_29638},
{C_text("f_29651:library_2escm"),(void*)f_29651},
{C_text("f_29654:library_2escm"),(void*)f_29654},
{C_text("f_29663:library_2escm"),(void*)f_29663},
{C_text("f_29669:library_2escm"),(void*)f_29669},
{C_text("f_29675:library_2escm"),(void*)f_29675},
{C_text("f_29681:library_2escm"),(void*)f_29681},
{C_text("f_29687:library_2escm"),(void*)f_29687},
{C_text("f_29704:library_2escm"),(void*)f_29704},
{C_text("f_29710:library_2escm"),(void*)f_29710},
{C_text("f_29725:library_2escm"),(void*)f_29725},
{C_text("f_29729:library_2escm"),(void*)f_29729},
{C_text("f_29733:library_2escm"),(void*)f_29733},
{C_text("f_29737:library_2escm"),(void*)f_29737},
{C_text("f_29758:library_2escm"),(void*)f_29758},
{C_text("f_29760:library_2escm"),(void*)f_29760},
{C_text("f_29766:library_2escm"),(void*)f_29766},
{C_text("f_29770:library_2escm"),(void*)f_29770},
{C_text("f_29779:library_2escm"),(void*)f_29779},
{C_text("f_29788:library_2escm"),(void*)f_29788},
{C_text("f_29831:library_2escm"),(void*)f_29831},
{C_text("f_29853:library_2escm"),(void*)f_29853},
{C_text("f_29866:library_2escm"),(void*)f_29866},
{C_text("f_29891:library_2escm"),(void*)f_29891},
{C_text("f_29907:library_2escm"),(void*)f_29907},
{C_text("f_29923:library_2escm"),(void*)f_29923},
{C_text("f_29939:library_2escm"),(void*)f_29939},
{C_text("f_29955:library_2escm"),(void*)f_29955},
{C_text("f_29971:library_2escm"),(void*)f_29971},
{C_text("f_29987:library_2escm"),(void*)f_29987},
{C_text("f_30003:library_2escm"),(void*)f_30003},
{C_text("f_30019:library_2escm"),(void*)f_30019},
{C_text("f_30035:library_2escm"),(void*)f_30035},
{C_text("f_30042:library_2escm"),(void*)f_30042},
{C_text("f_30083:library_2escm"),(void*)f_30083},
{C_text("f_30095:library_2escm"),(void*)f_30095},
{C_text("f_30101:library_2escm"),(void*)f_30101},
{C_text("f_30120:library_2escm"),(void*)f_30120},
{C_text("f_30122:library_2escm"),(void*)f_30122},
{C_text("f_30145:library_2escm"),(void*)f_30145},
{C_text("f_30191:library_2escm"),(void*)f_30191},
{C_text("f_30193:library_2escm"),(void*)f_30193},
{C_text("f_30200:library_2escm"),(void*)f_30200},
{C_text("f_30212:library_2escm"),(void*)f_30212},
{C_text("f_30256:library_2escm"),(void*)f_30256},
{C_text("f_30271:library_2escm"),(void*)f_30271},
{C_text("f_30279:library_2escm"),(void*)f_30279},
{C_text("f_30285:library_2escm"),(void*)f_30285},
{C_text("f_30293:library_2escm"),(void*)f_30293},
{C_text("f_30297:library_2escm"),(void*)f_30297},
{C_text("f_30305:library_2escm"),(void*)f_30305},
{C_text("f_30307:library_2escm"),(void*)f_30307},
{C_text("f_30317:library_2escm"),(void*)f_30317},
{C_text("f_30324:library_2escm"),(void*)f_30324},
{C_text("f_30328:library_2escm"),(void*)f_30328},
{C_text("f_30339:library_2escm"),(void*)f_30339},
{C_text("f_30346:library_2escm"),(void*)f_30346},
{C_text("f_30350:library_2escm"),(void*)f_30350},
{C_text("f_30361:library_2escm"),(void*)f_30361},
{C_text("f_30368:library_2escm"),(void*)f_30368},
{C_text("f_30372:library_2escm"),(void*)f_30372},
{C_text("f_30377:library_2escm"),(void*)f_30377},
{C_text("f_30383:library_2escm"),(void*)f_30383},
{C_text("f_30386:library_2escm"),(void*)f_30386},
{C_text("f_30389:library_2escm"),(void*)f_30389},
{C_text("f_30395:library_2escm"),(void*)f_30395},
{C_text("f_30404:library_2escm"),(void*)f_30404},
{C_text("f_30413:library_2escm"),(void*)f_30413},
{C_text("f_30419:library_2escm"),(void*)f_30419},
{C_text("f_30422:library_2escm"),(void*)f_30422},
{C_text("f_30425:library_2escm"),(void*)f_30425},
{C_text("f_30437:library_2escm"),(void*)f_30437},
{C_text("f_30440:library_2escm"),(void*)f_30440},
{C_text("f_30443:library_2escm"),(void*)f_30443},
{C_text("f_30446:library_2escm"),(void*)f_30446},
{C_text("f_30458:library_2escm"),(void*)f_30458},
{C_text("f_30461:library_2escm"),(void*)f_30461},
{C_text("f_30464:library_2escm"),(void*)f_30464},
{C_text("f_30467:library_2escm"),(void*)f_30467},
{C_text("f_30476:library_2escm"),(void*)f_30476},
{C_text("f_30479:library_2escm"),(void*)f_30479},
{C_text("f_30492:library_2escm"),(void*)f_30492},
{C_text("f_30493:library_2escm"),(void*)f_30493},
{C_text("f_30498:library_2escm"),(void*)f_30498},
{C_text("f_30499:library_2escm"),(void*)f_30499},
{C_text("f_30504:library_2escm"),(void*)f_30504},
{C_text("f_30505:library_2escm"),(void*)f_30505},
{C_text("f_30510:library_2escm"),(void*)f_30510},
{C_text("f_30511:library_2escm"),(void*)f_30511},
{C_text("f_30516:library_2escm"),(void*)f_30516},
{C_text("f_30517:library_2escm"),(void*)f_30517},
{C_text("f_30522:library_2escm"),(void*)f_30522},
{C_text("f_30524:library_2escm"),(void*)f_30524},
{C_text("f_30534:library_2escm"),(void*)f_30534},
{C_text("f_30537:library_2escm"),(void*)f_30537},
{C_text("f_30540:library_2escm"),(void*)f_30540},
{C_text("f_30543:library_2escm"),(void*)f_30543},
{C_text("f_30545:library_2escm"),(void*)f_30545},
{C_text("f_30559:library_2escm"),(void*)f_30559},
{C_text("f_30567:library_2escm"),(void*)f_30567},
{C_text("f_30571:library_2escm"),(void*)f_30571},
{C_text("f_30575:library_2escm"),(void*)f_30575},
{C_text("f_30579:library_2escm"),(void*)f_30579},
{C_text("f_30585:library_2escm"),(void*)f_30585},
{C_text("f_30592:library_2escm"),(void*)f_30592},
{C_text("f_30596:library_2escm"),(void*)f_30596},
{C_text("f_30600:library_2escm"),(void*)f_30600},
{C_text("f_30604:library_2escm"),(void*)f_30604},
{C_text("f_30624:library_2escm"),(void*)f_30624},
{C_text("f_30643:library_2escm"),(void*)f_30643},
{C_text("f_30646:library_2escm"),(void*)f_30646},
{C_text("f_30649:library_2escm"),(void*)f_30649},
{C_text("f_30652:library_2escm"),(void*)f_30652},
{C_text("f_30661:library_2escm"),(void*)f_30661},
{C_text("f_30665:library_2escm"),(void*)f_30665},
{C_text("f_30668:library_2escm"),(void*)f_30668},
{C_text("f_30683:library_2escm"),(void*)f_30683},
{C_text("f_30705:library_2escm"),(void*)f_30705},
{C_text("f_30711:library_2escm"),(void*)f_30711},
{C_text("f_30731:library_2escm"),(void*)f_30731},
{C_text("f_30746:library_2escm"),(void*)f_30746},
{C_text("f_30750:library_2escm"),(void*)f_30750},
{C_text("f_30773:library_2escm"),(void*)f_30773},
{C_text("f_30777:library_2escm"),(void*)f_30777},
{C_text("f_30794:library_2escm"),(void*)f_30794},
{C_text("f_30801:library_2escm"),(void*)f_30801},
{C_text("f_30823:library_2escm"),(void*)f_30823},
{C_text("f_30828:library_2escm"),(void*)f_30828},
{C_text("f_30843:library_2escm"),(void*)f_30843},
{C_text("f_30846:library_2escm"),(void*)f_30846},
{C_text("f_30849:library_2escm"),(void*)f_30849},
{C_text("f_30852:library_2escm"),(void*)f_30852},
{C_text("f_30855:library_2escm"),(void*)f_30855},
{C_text("f_30858:library_2escm"),(void*)f_30858},
{C_text("f_30861:library_2escm"),(void*)f_30861},
{C_text("f_30864:library_2escm"),(void*)f_30864},
{C_text("f_30867:library_2escm"),(void*)f_30867},
{C_text("f_30870:library_2escm"),(void*)f_30870},
{C_text("f_30878:library_2escm"),(void*)f_30878},
{C_text("f_30881:library_2escm"),(void*)f_30881},
{C_text("f_30886:library_2escm"),(void*)f_30886},
{C_text("f_30892:library_2escm"),(void*)f_30892},
{C_text("f_30905:library_2escm"),(void*)f_30905},
{C_text("f_30907:library_2escm"),(void*)f_30907},
{C_text("f_30930:library_2escm"),(void*)f_30930},
{C_text("f_30937:library_2escm"),(void*)f_30937},
{C_text("f_30941:library_2escm"),(void*)f_30941},
{C_text("f_30943:library_2escm"),(void*)f_30943},
{C_text("f_30972:library_2escm"),(void*)f_30972},
{C_text("f_30974:library_2escm"),(void*)f_30974},
{C_text("f_30999:library_2escm"),(void*)f_30999},
{C_text("f_31008:library_2escm"),(void*)f_31008},
{C_text("f_31011:library_2escm"),(void*)f_31011},
{C_text("f_31017:library_2escm"),(void*)f_31017},
{C_text("f_31041:library_2escm"),(void*)f_31041},
{C_text("f_31048:library_2escm"),(void*)f_31048},
{C_text("f_31052:library_2escm"),(void*)f_31052},
{C_text("f_31064:library_2escm"),(void*)f_31064},
{C_text("f_31072:library_2escm"),(void*)f_31072},
{C_text("f_31076:library_2escm"),(void*)f_31076},
{C_text("f_31085:library_2escm"),(void*)f_31085},
{C_text("f_31094:library_2escm"),(void*)f_31094},
{C_text("f_31104:library_2escm"),(void*)f_31104},
{C_text("f_31112:library_2escm"),(void*)f_31112},
{C_text("f_31116:library_2escm"),(void*)f_31116},
{C_text("f_31120:library_2escm"),(void*)f_31120},
{C_text("f_31126:library_2escm"),(void*)f_31126},
{C_text("f_31130:library_2escm"),(void*)f_31130},
{C_text("f_31134:library_2escm"),(void*)f_31134},
{C_text("f_31156:library_2escm"),(void*)f_31156},
{C_text("f_31160:library_2escm"),(void*)f_31160},
{C_text("f_31164:library_2escm"),(void*)f_31164},
{C_text("f_31168:library_2escm"),(void*)f_31168},
{C_text("f_31172:library_2escm"),(void*)f_31172},
{C_text("f_31176:library_2escm"),(void*)f_31176},
{C_text("f_31179:library_2escm"),(void*)f_31179},
{C_text("f_31182:library_2escm"),(void*)f_31182},
{C_text("f_31192:library_2escm"),(void*)f_31192},
{C_text("f_31195:library_2escm"),(void*)f_31195},
{C_text("f_31197:library_2escm"),(void*)f_31197},
{C_text("f_31214:library_2escm"),(void*)f_31214},
{C_text("f_31241:library_2escm"),(void*)f_31241},
{C_text("f_31250:library_2escm"),(void*)f_31250},
{C_text("f_31270:library_2escm"),(void*)f_31270},
{C_text("f_31274:library_2escm"),(void*)f_31274},
{C_text("f_31278:library_2escm"),(void*)f_31278},
{C_text("f_31282:library_2escm"),(void*)f_31282},
{C_text("f_31286:library_2escm"),(void*)f_31286},
{C_text("f_31290:library_2escm"),(void*)f_31290},
{C_text("f_31294:library_2escm"),(void*)f_31294},
{C_text("f_31296:library_2escm"),(void*)f_31296},
{C_text("f_31305:library_2escm"),(void*)f_31305},
{C_text("f_31315:library_2escm"),(void*)f_31315},
{C_text("f_31340:library_2escm"),(void*)f_31340},
{C_text("f_31343:library_2escm"),(void*)f_31343},
{C_text("f_31345:library_2escm"),(void*)f_31345},
{C_text("f_31361:library_2escm"),(void*)f_31361},
{C_text("f_31380:library_2escm"),(void*)f_31380},
{C_text("f_31390:library_2escm"),(void*)f_31390},
{C_text("f_31400:library_2escm"),(void*)f_31400},
{C_text("f_31424:library_2escm"),(void*)f_31424},
{C_text("f_31428:library_2escm"),(void*)f_31428},
{C_text("f_31430:library_2escm"),(void*)f_31430},
{C_text("f_31434:library_2escm"),(void*)f_31434},
{C_text("f_31437:library_2escm"),(void*)f_31437},
{C_text("f_31452:library_2escm"),(void*)f_31452},
{C_text("f_31458:library_2escm"),(void*)f_31458},
{C_text("f_31462:library_2escm"),(void*)f_31462},
{C_text("f_31471:library_2escm"),(void*)f_31471},
{C_text("f_31474:library_2escm"),(void*)f_31474},
{C_text("f_31477:library_2escm"),(void*)f_31477},
{C_text("f_31480:library_2escm"),(void*)f_31480},
{C_text("f_31483:library_2escm"),(void*)f_31483},
{C_text("f_31489:library_2escm"),(void*)f_31489},
{C_text("f_31495:library_2escm"),(void*)f_31495},
{C_text("f_31498:library_2escm"),(void*)f_31498},
{C_text("f_31503:library_2escm"),(void*)f_31503},
{C_text("f_31513:library_2escm"),(void*)f_31513},
{C_text("f_31525:library_2escm"),(void*)f_31525},
{C_text("f_31533:library_2escm"),(void*)f_31533},
{C_text("f_31543:library_2escm"),(void*)f_31543},
{C_text("f_31559:library_2escm"),(void*)f_31559},
{C_text("f_31565:library_2escm"),(void*)f_31565},
{C_text("f_31568:library_2escm"),(void*)f_31568},
{C_text("f_31571:library_2escm"),(void*)f_31571},
{C_text("f_31572:library_2escm"),(void*)f_31572},
{C_text("f_31576:library_2escm"),(void*)f_31576},
{C_text("f_31582:library_2escm"),(void*)f_31582},
{C_text("f_31586:library_2escm"),(void*)f_31586},
{C_text("f_31588:library_2escm"),(void*)f_31588},
{C_text("f_31598:library_2escm"),(void*)f_31598},
{C_text("f_31615:library_2escm"),(void*)f_31615},
{C_text("f_31618:library_2escm"),(void*)f_31618},
{C_text("f_31622:library_2escm"),(void*)f_31622},
{C_text("f_31627:library_2escm"),(void*)f_31627},
{C_text("f_31633:library_2escm"),(void*)f_31633},
{C_text("f_31640:library_2escm"),(void*)f_31640},
{C_text("f_31649:library_2escm"),(void*)f_31649},
{C_text("f_31657:library_2escm"),(void*)f_31657},
{C_text("f_31682:library_2escm"),(void*)f_31682},
{C_text("f_31700:library_2escm"),(void*)f_31700},
{C_text("f_31704:library_2escm"),(void*)f_31704},
{C_text("f_31719:library_2escm"),(void*)f_31719},
{C_text("f_31723:library_2escm"),(void*)f_31723},
{C_text("f_31738:library_2escm"),(void*)f_31738},
{C_text("f_31741:library_2escm"),(void*)f_31741},
{C_text("f_31744:library_2escm"),(void*)f_31744},
{C_text("f_31754:library_2escm"),(void*)f_31754},
{C_text("f_31785:library_2escm"),(void*)f_31785},
{C_text("f_31809:library_2escm"),(void*)f_31809},
{C_text("f_31819:library_2escm"),(void*)f_31819},
{C_text("f_31828:library_2escm"),(void*)f_31828},
{C_text("f_31869:library_2escm"),(void*)f_31869},
{C_text("f_31888:library_2escm"),(void*)f_31888},
{C_text("f_31897:library_2escm"),(void*)f_31897},
{C_text("f_31901:library_2escm"),(void*)f_31901},
{C_text("f_31906:library_2escm"),(void*)f_31906},
{C_text("f_31916:library_2escm"),(void*)f_31916},
{C_text("f_31926:library_2escm"),(void*)f_31926},
{C_text("f_31936:library_2escm"),(void*)f_31936},
{C_text("f_31946:library_2escm"),(void*)f_31946},
{C_text("f_31956:library_2escm"),(void*)f_31956},
{C_text("f_31966:library_2escm"),(void*)f_31966},
{C_text("f_31976:library_2escm"),(void*)f_31976},
{C_text("f_31986:library_2escm"),(void*)f_31986},
{C_text("f_31996:library_2escm"),(void*)f_31996},
{C_text("f_32006:library_2escm"),(void*)f_32006},
{C_text("f_32016:library_2escm"),(void*)f_32016},
{C_text("f_32026:library_2escm"),(void*)f_32026},
{C_text("f_32036:library_2escm"),(void*)f_32036},
{C_text("f_32052:library_2escm"),(void*)f_32052},
{C_text("f_32060:library_2escm"),(void*)f_32060},
{C_text("f_32076:library_2escm"),(void*)f_32076},
{C_text("f_32092:library_2escm"),(void*)f_32092},
{C_text("f_32102:library_2escm"),(void*)f_32102},
{C_text("f_32112:library_2escm"),(void*)f_32112},
{C_text("f_32131:library_2escm"),(void*)f_32131},
{C_text("f_32147:library_2escm"),(void*)f_32147},
{C_text("f_32160:library_2escm"),(void*)f_32160},
{C_text("f_32166:library_2escm"),(void*)f_32166},
{C_text("f_32189:library_2escm"),(void*)f_32189},
{C_text("f_32193:library_2escm"),(void*)f_32193},
{C_text("f_32197:library_2escm"),(void*)f_32197},
{C_text("f_32200:library_2escm"),(void*)f_32200},
{C_text("f_32206:library_2escm"),(void*)f_32206},
{C_text("f_32229:library_2escm"),(void*)f_32229},
{C_text("f_32233:library_2escm"),(void*)f_32233},
{C_text("f_32237:library_2escm"),(void*)f_32237},
{C_text("f_32240:library_2escm"),(void*)f_32240},
{C_text("f_32243:library_2escm"),(void*)f_32243},
{C_text("f_32246:library_2escm"),(void*)f_32246},
{C_text("f_32252:library_2escm"),(void*)f_32252},
{C_text("f_32255:library_2escm"),(void*)f_32255},
{C_text("f_32258:library_2escm"),(void*)f_32258},
{C_text("f_32268:library_2escm"),(void*)f_32268},
{C_text("f_32292:library_2escm"),(void*)f_32292},
{C_text("f_32315:library_2escm"),(void*)f_32315},
{C_text("f_32319:library_2escm"),(void*)f_32319},
{C_text("f_32323:library_2escm"),(void*)f_32323},
{C_text("f_32357:library_2escm"),(void*)f_32357},
{C_text("f_32361:library_2escm"),(void*)f_32361},
{C_text("f_32364:library_2escm"),(void*)f_32364},
{C_text("f_32377:library_2escm"),(void*)f_32377},
{C_text("f_32410:library_2escm"),(void*)f_32410},
{C_text("f_32425:library_2escm"),(void*)f_32425},
{C_text("f_32429:library_2escm"),(void*)f_32429},
{C_text("f_32453:library_2escm"),(void*)f_32453},
{C_text("f_32461:library_2escm"),(void*)f_32461},
{C_text("f_32465:library_2escm"),(void*)f_32465},
{C_text("f_32476:library_2escm"),(void*)f_32476},
{C_text("f_32480:library_2escm"),(void*)f_32480},
{C_text("f_32484:library_2escm"),(void*)f_32484},
{C_text("f_32498:library_2escm"),(void*)f_32498},
{C_text("f_32530:library_2escm"),(void*)f_32530},
{C_text("f_8571:library_2escm"),(void*)f_8571},
{C_text("f_8573:library_2escm"),(void*)f_8573},
{C_text("f_8576:library_2escm"),(void*)f_8576},
{C_text("f_8579:library_2escm"),(void*)f_8579},
{C_text("f_8582:library_2escm"),(void*)f_8582},
{C_text("f_8585:library_2escm"),(void*)f_8585},
{C_text("f_8588:library_2escm"),(void*)f_8588},
{C_text("f_8591:library_2escm"),(void*)f_8591},
{C_text("f_8594:library_2escm"),(void*)f_8594},
{C_text("f_8597:library_2escm"),(void*)f_8597},
{C_text("f_8600:library_2escm"),(void*)f_8600},
{C_text("f_8603:library_2escm"),(void*)f_8603},
{C_text("f_8606:library_2escm"),(void*)f_8606},
{C_text("f_8609:library_2escm"),(void*)f_8609},
{C_text("f_8612:library_2escm"),(void*)f_8612},
{C_text("f_8615:library_2escm"),(void*)f_8615},
{C_text("f_8618:library_2escm"),(void*)f_8618},
{C_text("f_8621:library_2escm"),(void*)f_8621},
{C_text("f_8624:library_2escm"),(void*)f_8624},
{C_text("f_8627:library_2escm"),(void*)f_8627},
{C_text("f_8641:library_2escm"),(void*)f_8641},
{C_text("f_8648:library_2escm"),(void*)f_8648},
{C_text("f_8655:library_2escm"),(void*)f_8655},
{C_text("f_8669:library_2escm"),(void*)f_8669},
{C_text("f_8676:library_2escm"),(void*)f_8676},
{C_text("f_8690:library_2escm"),(void*)f_8690},
{C_text("f_8704:library_2escm"),(void*)f_8704},
{C_text("f_8722:library_2escm"),(void*)f_8722},
{C_text("f_8733:library_2escm"),(void*)f_8733},
{C_text("f_8744:library_2escm"),(void*)f_8744},
{C_text("f_8751:library_2escm"),(void*)f_8751},
{C_text("f_8762:library_2escm"),(void*)f_8762},
{C_text("f_8766:library_2escm"),(void*)f_8766},
{C_text("f_8773:library_2escm"),(void*)f_8773},
{C_text("f_8791:library_2escm"),(void*)f_8791},
{C_text("f_8802:library_2escm"),(void*)f_8802},
{C_text("f_8813:library_2escm"),(void*)f_8813},
{C_text("f_8820:library_2escm"),(void*)f_8820},
{C_text("f_8838:library_2escm"),(void*)f_8838},
{C_text("f_8849:library_2escm"),(void*)f_8849},
{C_text("f_8867:library_2escm"),(void*)f_8867},
{C_text("f_8873:library_2escm"),(void*)f_8873},
{C_text("f_8876:library_2escm"),(void*)f_8876},
{C_text("f_8879:library_2escm"),(void*)f_8879},
{C_text("f_8882:library_2escm"),(void*)f_8882},
{C_text("f_8885:library_2escm"),(void*)f_8885},
{C_text("f_8891:library_2escm"),(void*)f_8891},
{C_text("f_8924:library_2escm"),(void*)f_8924},
{C_text("f_8927:library_2escm"),(void*)f_8927},
{C_text("f_8930:library_2escm"),(void*)f_8930},
{C_text("f_8933:library_2escm"),(void*)f_8933},
{C_text("f_8936:library_2escm"),(void*)f_8936},
{C_text("f_8939:library_2escm"),(void*)f_8939},
{C_text("f_8942:library_2escm"),(void*)f_8942},
{C_text("f_8945:library_2escm"),(void*)f_8945},
{C_text("f_8948:library_2escm"),(void*)f_8948},
{C_text("f_8951:library_2escm"),(void*)f_8951},
{C_text("f_8954:library_2escm"),(void*)f_8954},
{C_text("f_8957:library_2escm"),(void*)f_8957},
{C_text("f_8960:library_2escm"),(void*)f_8960},
{C_text("f_8964:library_2escm"),(void*)f_8964},
{C_text("f_8967:library_2escm"),(void*)f_8967},
{C_text("f_8973:library_2escm"),(void*)f_8973},
{C_text("f_8976:library_2escm"),(void*)f_8976},
{C_text("f_8979:library_2escm"),(void*)f_8979},
{C_text("f_8982:library_2escm"),(void*)f_8982},
{C_text("f_8985:library_2escm"),(void*)f_8985},
{C_text("f_8993:library_2escm"),(void*)f_8993},
{C_text("f_8997:library_2escm"),(void*)f_8997},
{C_text("f_9000:library_2escm"),(void*)f_9000},
{C_text("f_9003:library_2escm"),(void*)f_9003},
{C_text("f_9006:library_2escm"),(void*)f_9006},
{C_text("f_9009:library_2escm"),(void*)f_9009},
{C_text("f_9012:library_2escm"),(void*)f_9012},
{C_text("f_9015:library_2escm"),(void*)f_9015},
{C_text("f_9018:library_2escm"),(void*)f_9018},
{C_text("f_9022:library_2escm"),(void*)f_9022},
{C_text("f_9025:library_2escm"),(void*)f_9025},
{C_text("f_9028:library_2escm"),(void*)f_9028},
{C_text("f_9031:library_2escm"),(void*)f_9031},
{C_text("f_9034:library_2escm"),(void*)f_9034},
{C_text("f_9042:library_2escm"),(void*)f_9042},
{C_text("f_9045:library_2escm"),(void*)f_9045},
{C_text("f_9052:library_2escm"),(void*)f_9052},
{C_text("f_9056:library_2escm"),(void*)f_9056},
{C_text("f_9059:library_2escm"),(void*)f_9059},
{C_text("f_9062:library_2escm"),(void*)f_9062},
{C_text("f_9065:library_2escm"),(void*)f_9065},
{C_text("f_9068:library_2escm"),(void*)f_9068},
{C_text("f_9071:library_2escm"),(void*)f_9071},
{C_text("f_9074:library_2escm"),(void*)f_9074},
{C_text("f_9080:library_2escm"),(void*)f_9080},
{C_text("f_9091:library_2escm"),(void*)f_9091},
{C_text("f_9097:library_2escm"),(void*)f_9097},
{C_text("f_9103:library_2escm"),(void*)f_9103},
{C_text("f_9109:library_2escm"),(void*)f_9109},
{C_text("f_9115:library_2escm"),(void*)f_9115},
{C_text("f_9121:library_2escm"),(void*)f_9121},
{C_text("f_9128:library_2escm"),(void*)f_9128},
{C_text("f_9134:library_2escm"),(void*)f_9134},
{C_text("f_9150:library_2escm"),(void*)f_9150},
{C_text("f_9161:library_2escm"),(void*)f_9161},
{C_text("f_9167:library_2escm"),(void*)f_9167},
{C_text("f_9173:library_2escm"),(void*)f_9173},
{C_text("f_9179:library_2escm"),(void*)f_9179},
{C_text("f_9185:library_2escm"),(void*)f_9185},
{C_text("f_9194:library_2escm"),(void*)f_9194},
{C_text("f_9197:library_2escm"),(void*)f_9197},
{C_text("f_9200:library_2escm"),(void*)f_9200},
{C_text("f_9203:library_2escm"),(void*)f_9203},
{C_text("f_9206:library_2escm"),(void*)f_9206},
{C_text("f_9209:library_2escm"),(void*)f_9209},
{C_text("f_9213:library_2escm"),(void*)f_9213},
{C_text("f_9219:library_2escm"),(void*)f_9219},
{C_text("f_9225:library_2escm"),(void*)f_9225},
{C_text("f_9260:library_2escm"),(void*)f_9260},
{C_text("f_9263:library_2escm"),(void*)f_9263},
{C_text("f_9266:library_2escm"),(void*)f_9266},
{C_text("f_9274:library_2escm"),(void*)f_9274},
{C_text("f_9280:library_2escm"),(void*)f_9280},
{C_text("f_9287:library_2escm"),(void*)f_9287},
{C_text("f_9302:library_2escm"),(void*)f_9302},
{C_text("f_9325:library_2escm"),(void*)f_9325},
{C_text("f_9331:library_2escm"),(void*)f_9331},
{C_text("f_9337:library_2escm"),(void*)f_9337},
{C_text("f_9344:library_2escm"),(void*)f_9344},
{C_text("f_9349:library_2escm"),(void*)f_9349},
{C_text("f_9352:library_2escm"),(void*)f_9352},
{C_text("f_9361:library_2escm"),(void*)f_9361},
{C_text("f_9379:library_2escm"),(void*)f_9379},
{C_text("f_9389:library_2escm"),(void*)f_9389},
{C_text("f_9398:library_2escm"),(void*)f_9398},
{C_text("f_9416:library_2escm"),(void*)f_9416},
{C_text("f_9429:library_2escm"),(void*)f_9429},
{C_text("f_9436:library_2escm"),(void*)f_9436},
{C_text("f_9443:library_2escm"),(void*)f_9443},
{C_text("f_9454:library_2escm"),(void*)f_9454},
{C_text("f_9461:library_2escm"),(void*)f_9461},
{C_text("f_9473:library_2escm"),(void*)f_9473},
{C_text("f_9476:library_2escm"),(void*)f_9476},
{C_text("f_9478:library_2escm"),(void*)f_9478},
{C_text("f_9484:library_2escm"),(void*)f_9484},
{C_text("f_9500:library_2escm"),(void*)f_9500},
{C_text("f_9509:library_2escm"),(void*)f_9509},
{C_text("f_9511:library_2escm"),(void*)f_9511},
{C_text("f_9517:library_2escm"),(void*)f_9517},
{C_text("f_9530:library_2escm"),(void*)f_9530},
{C_text("f_9546:library_2escm"),(void*)f_9546},
{C_text("f_9557:library_2escm"),(void*)f_9557},
{C_text("f_9559:library_2escm"),(void*)f_9559},
{C_text("f_9567:library_2escm"),(void*)f_9567},
{C_text("f_9569:library_2escm"),(void*)f_9569},
{C_text("f_9575:library_2escm"),(void*)f_9575},
{C_text("f_9577:library_2escm"),(void*)f_9577},
{C_text("f_9585:library_2escm"),(void*)f_9585},
{C_text("f_9587:library_2escm"),(void*)f_9587},
{C_text("f_9590:library_2escm"),(void*)f_9590},
{C_text("f_9598:library_2escm"),(void*)f_9598},
{C_text("f_9604:library_2escm"),(void*)f_9604},
{C_text("f_9612:library_2escm"),(void*)f_9612},
{C_text("f_9623:library_2escm"),(void*)f_9623},
{C_text("f_9635:library_2escm"),(void*)f_9635},
{C_text("f_9649:library_2escm"),(void*)f_9649},
{C_text("f_9657:library_2escm"),(void*)f_9657},
{C_text("f_9660:library_2escm"),(void*)f_9660},
{C_text("f_9662:library_2escm"),(void*)f_9662},
{C_text("f_9664:library_2escm"),(void*)f_9664},
{C_text("f_9670:library_2escm"),(void*)f_9670},
{C_text("f_9689:library_2escm"),(void*)f_9689},
{C_text("f_9706:library_2escm"),(void*)f_9706},
{C_text("f_9714:library_2escm"),(void*)f_9714},
{C_text("f_9728:library_2escm"),(void*)f_9728},
{C_text("f_9734:library_2escm"),(void*)f_9734},
{C_text("f_9753:library_2escm"),(void*)f_9753},
{C_text("f_9762:library_2escm"),(void*)f_9762},
{C_text("f_9765:library_2escm"),(void*)f_9765},
{C_text("f_9777:library_2escm"),(void*)f_9777},
{C_text("f_9793:library_2escm"),(void*)f_9793},
{C_text("f_9799:library_2escm"),(void*)f_9799},
{C_text("f_9824:library_2escm"),(void*)f_9824},
{C_text("f_9826:library_2escm"),(void*)f_9826},
{C_text("f_9835:library_2escm"),(void*)f_9835},
{C_text("f_9856:library_2escm"),(void*)f_9856},
{C_text("f_9858:library_2escm"),(void*)f_9858},
{C_text("f_9864:library_2escm"),(void*)f_9864},
{C_text("f_9890:library_2escm"),(void*)f_9890},
{C_text("f_9897:library_2escm"),(void*)f_9897},
{C_text("f_9899:library_2escm"),(void*)f_9899},
{C_text("f_9911:library_2escm"),(void*)f_9911},
{C_text("f_9946:library_2escm"),(void*)f_9946},
{C_text("f_9956:library_2escm"),(void*)f_9956},
{C_text("f_9965:library_2escm"),(void*)f_9965},
{C_text("toplevel:library_2escm"),(void*)C_library_toplevel},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: scheme#string-fill 
o|hiding unexported module binding: chicken.condition#prop-list->kind-prefixed-prop-list 
o|hiding unexported module binding: chicken.platform#path-list-separator 
o|hiding unexported module binding: chicken.platform#->feature-id 
S|applied compiler syntax:
S|  scheme#for-each		9
S|  ##sys#map		1
S|  ##sys#for-each		4
S|  scheme#map		7
o|eliminated procedure checks: 524 
o|folded constant expression: (scheme#char->integer (quote #\0)) 
o|folded constant expression: (scheme#char->integer (quote #\a)) 
o|folded constant expression: (scheme#char->integer (quote #\A)) 
o|folded constant expression: (scheme#integer->char (quote 11)) 
o|folded constant expression: (scheme#integer->char (quote 127)) 
o|folded constant expression: (scheme#integer->char (quote 27)) 
o|folded constant expression: (scheme#integer->char (quote 27)) 
o|folded constant expression: (scheme#integer->char (quote 7)) 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#integer->char (quote 0)) 
o|folded constant expression: (scheme#integer->char (quote 12)) 
o|folded constant expression: (scheme#integer->char (quote 8)) 
o|folded constant expression: (scheme#integer->char (quote 7)) 
o|folded constant expression: (scheme#integer->char (quote 11)) 
o|folded constant expression: (scheme#integer->char (quote 12)) 
o|folded constant expression: (scheme#expt (quote 1024) (quote 3)) 
o|folded constant expression: (scheme#expt (quote 1024) (quote 3)) 
o|folded constant expression: (scheme#expt (quote 1024) (quote 2)) 
o|folded constant expression: (scheme#expt (quote 1024) (quote 2)) 
o|specializations:
o|  1 (scheme#zero? integer)
o|  2 (scheme#number->string fixnum)
o|  2 (scheme#string-append string string)
o|  1 (scheme#make-string fixnum)
o|  1 (scheme#cdar (pair pair *))
o|  2 (scheme#cddr (pair * pair))
o|  1 (scheme#char>? char char)
o|  13 (scheme#char=? char char)
o|  3 (scheme#string-ref string fixnum)
o|  5 (##sys#check-input-port * * *)
o|  11 (##sys#check-output-port * * *)
o|  1 (##sys#check-output-port * *)
o|  1 (##sys#check-input-port * *)
o|  13 (##sys#check-list (or pair list) *)
o|  1 (scheme#zero? number)
o|  43 (scheme#cdr pair)
o|  4 (scheme#char>=? char char)
o|  6 (scheme#char<=? char char)
o|  5 (scheme#zero? *)
o|  24 (scheme#eqv? (or eof null fixnum char boolean symbol keyword) *)
o|  1 (scheme#> fixnum fixnum)
o|  1 (scheme#make-string fixnum char)
o|  1 (scheme#string-length string)
o|  6 (scheme#number->string * *)
o|  3 (scheme#integer? *)
o|  2 (scheme#even? *)
o|  5 (scheme#/ * *)
o|  1 (scheme#+ integer integer)
o|  1 (scheme#- integer integer)
o|  5 (chicken.bitwise#integer-length *)
o|  208 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  2 (chicken.flonum#fp<= float float)
o|  7 (scheme#- *)
o|  1 (scheme#odd? *)
o|  1 (chicken.flonum#fp> float float)
o|  2 (scheme#angle cplxnum)
o|  5 (chicken.base#finite? *)
o|  13 (chicken.base#exact-integer? *)
o|  8 (scheme#abs *)
o|  17 (scheme#exact->inexact number)
o|  5 (##sys#debug-mode?)
o|  49 (scheme#car pair)
o|  4 (scheme#length list)
o|  12 (scheme#negative? *)
o|  4 (scheme#positive? *)
(o e)|safe calls: 3265 
(o e)|dropped branches: 3 
(o e)|assignments to immediate values: 28 
o|dropping redundant toplevel assignment: chicken.base#warning 
o|dropping redundant toplevel assignment: chicken.base#notice 
o|dropping redundant toplevel assignment: chicken.base#enable-warnings 
o|dropping redundant toplevel assignment: scheme#force 
o|dropping redundant toplevel assignment: scheme#append 
o|dropping redundant toplevel assignment: scheme#make-string 
o|dropping redundant toplevel assignment: scheme#string->list 
o|dropping redundant toplevel assignment: scheme#list->string 
o|dropping redundant toplevel assignment: scheme#string-fill! 
o|dropping redundant toplevel assignment: scheme#string-copy 
o|dropping redundant toplevel assignment: scheme#substring 
o|dropping redundant toplevel assignment: scheme#string<? 
o|dropping redundant toplevel assignment: scheme#string>? 
o|dropping redundant toplevel assignment: scheme#string<=? 
o|dropping redundant toplevel assignment: scheme#string>=? 
o|dropping redundant toplevel assignment: scheme#string-ci<? 
o|dropping redundant toplevel assignment: scheme#string-ci>? 
o|dropping redundant toplevel assignment: scheme#string-ci<=? 
o|dropping redundant toplevel assignment: scheme#string-ci>=? 
o|dropping redundant toplevel assignment: scheme#string-append 
o|dropping redundant toplevel assignment: scheme#string 
o|dropping redundant toplevel assignment: chicken.base#chop 
o|dropping redundant toplevel assignment: scheme#make-rectangular 
o|dropping redundant toplevel assignment: scheme#make-polar 
o|dropping redundant toplevel assignment: scheme#real-part 
o|dropping redundant toplevel assignment: scheme#imag-part 
o|dropping redundant toplevel assignment: scheme#angle 
o|dropping redundant toplevel assignment: scheme#magnitude 
o|dropping redundant toplevel assignment: scheme#numerator 
o|dropping redundant toplevel assignment: scheme#denominator 
o|dropping redundant toplevel assignment: scheme#inexact->exact 
o|dropping redundant toplevel assignment: scheme#/ 
o|dropping redundant toplevel assignment: scheme#floor 
o|dropping redundant toplevel assignment: scheme#ceiling 
o|dropping redundant toplevel assignment: scheme#truncate 
o|dropping redundant toplevel assignment: scheme#round 
o|dropping redundant toplevel assignment: scheme#rationalize 
o|dropping redundant toplevel assignment: scheme#max 
o|dropping redundant toplevel assignment: scheme#min 
o|dropping redundant toplevel assignment: scheme#exp 
o|dropping redundant toplevel assignment: scheme#log 
o|dropping redundant toplevel assignment: scheme#sin 
o|dropping redundant toplevel assignment: scheme#cos 
o|dropping redundant toplevel assignment: scheme#tan 
o|dropping redundant toplevel assignment: scheme#asin 
o|dropping redundant toplevel assignment: scheme#acos 
o|dropping redundant toplevel assignment: scheme#atan 
o|dropping redundant toplevel assignment: chicken.base#exact-integer-sqrt 
o|dropping redundant toplevel assignment: scheme#sqrt 
o|dropping redundant toplevel assignment: chicken.base#exact-integer-nth-root 
o|dropping redundant toplevel assignment: scheme#expt 
o|dropping redundant toplevel assignment: scheme#gcd 
o|dropping redundant toplevel assignment: scheme#lcm 
o|dropping redundant toplevel assignment: scheme#string->number 
o|dropping redundant toplevel assignment: ##sys#number->string 
o|dropping redundant toplevel assignment: chicken.base#equal=? 
o|dropping redundant toplevel assignment: scheme#symbol->string 
o|dropping redundant toplevel assignment: scheme#string->symbol 
o|dropping redundant toplevel assignment: chicken.base#string->uninterned-symbol 
o|dropping redundant toplevel assignment: chicken.base#gensym 
o|dropping redundant toplevel assignment: chicken.base#symbol-append 
o|dropping redundant toplevel assignment: ##sys#update-errno 
o|dropping redundant toplevel assignment: ##sys#errno 
o|safe globals: (chicken.keyword#keyword->string chicken.keyword#string->keyword chicken.keyword#keyword? chicken.base#symbol-append chicken.base#gensym chicken.base#string->uninterned-symbol scheme#string->symbol scheme#symbol->string ##sys#symbol->string ##sys#string->symbol ##sys#interned-symbol? ##sys#intern-keyword ##sys#intern-symbol ##sys#snafu chicken.base#equal=? ##sys#number->string ##sys#integer->string ##sys#flonum->string ##sys#fixnum->string ##sys#string->number scheme#string->number ##sys#string->compnum round-quotient ldexp mantexp->dbl ##sys#extended-number->string ##sys#integer->string/recursive scheme#lcm ##sys#lcm scheme#gcd ##sys#gcd ##sys#internal-gcd scheme#expt ##sys#integer-power ##sys#exact-integer-nth-root/loc chicken.base#exact-integer-nth-root scheme#sqrt ##sys#sqrt/loc chicken.base#exact-integer-sqrt ##sys#exact-integer-sqrt scheme#atan scheme#acos scheme#asin scheme#tan scheme#cos scheme#sin scheme#log ##sys#log-1 scheme#exp scheme#min scheme#max scheme#rationalize find-ratio find-ratio-between scheme#round scheme#truncate scheme#ceiling scheme#floor ##sys#/-2 scheme#/ chicken.bitwise#arithmetic-shift chicken.bitwise#integer-length chicken.bitwise#bit->boolean chicken.bitwise#bitwise-not chicken.bitwise#bitwise-xor chicken.bitwise#bitwise-ior chicken.bitwise#bitwise-and ##sys#inexact->exact ##sys#exact->inexact scheme#inexact->exact flonum->ratnum ##sys#extended-signum scheme#denominator scheme#numerator ratnum scheme#magnitude scheme#angle scheme#imag-part scheme#real-part scheme#make-polar scheme#make-rectangular make-complex ##sys#inexact? ##sys#exact? ##sys#integer? ##sys#number? chicken.flonum#flonum-print-precision chicken.flonum#fpinteger? chicken.flonum#fpabs chicken.flonum#fpsqrt chicken.flonum#fplog chicken.flonum#fpexpt chicken.flonum#fpexp chicken.flonum#fpatan2 chicken.flonum#fpatan chicken.flonum#fpacos chicken.flonum#fpasin chicken.flonum#fptan chicken.flonum#fpcos chicken.flonum#fpsin chicken.flonum#fpceiling chicken.flonum#fpround chicken.flonum#fptruncate chicken.flonum#fpfloor chicken.flonum#fpmin chicken.flonum#fpmax chicken.flonum#fpneg chicken.flonum#fp<= chicken.flonum#fp>= chicken.flonum#fp< chicken.flonum#fp> chicken.flonum#fp= chicken.flonum#fp/? chicken.flonum#fpgcd chicken.flonum#fp/ chicken.flonum#fp* chicken.flonum#fp- chicken.flonum#fp+ chicken.flonum#flonum-minimum-decimal-exponent chicken.flonum#flonum-maximum-decimal-exponent chicken.flonum#flonum-minimum-exponent chicken.flonum#flonum-maximum-exponent chicken.flonum#flonum-decimal-precision chicken.flonum#flonum-precision chicken.flonum#flonum-epsilon chicken.flonum#flonum-radix chicken.flonum#minimum-flonum chicken.flonum#maximum-flonum chicken.base#chop ##sys#fragments->string scheme#string scheme#string-append ##sys#string-append scheme#string-ci>=? scheme#string-ci<=? scheme#string-ci>? scheme#string-ci<? scheme#string>=? scheme#string<=? scheme#string>? scheme#string<? ##sys#substring scheme#substring scheme#string-copy scheme#string-fill! ##sys#reverse-list->string ##sys#list->string scheme#list->string ##sys#string->list scheme#string->list scheme#make-string ##sys#make-string ##sys#fast-reverse scheme#append ##sys#error-bad-base ##sys#error-bad-real ##sys#error-bad-inexact ##sys#error-bad-exact-uinteger ##sys#error-bad-exact-integer ##sys#error-bad-integer ##sys#error-bad-number ##sys#error-not-a-proper-list ##sys#set-dlopen-flags! ##sys#dload scheme#force ##sys#check-closure ##sys#check-special ##sys#check-range ##sys#check-real ##sys#check-exact-uinteger ##sys#check-exact-integer ##sys#check-integer ##sys#check-locative ##sys#check-boolean ##sys#check-char ##sys#check-vector ##sys#check-keyword ##sys#check-symbol ##sys#check-inexact ##sys#check-exact ##sys#check-fixnum ##sys#check-number ##sys#check-string ##sys#check-list ##sys#check-pair ##sys#check-byte-vector ##sys#check-blob ##sys#check-structure chicken.time#cpu-time chicken.time#current-seconds chicken.time#current-process-milliseconds chicken.time#current-milliseconds ##sys#block-set! ##sys#copy-closure ##sys#apply-values ##sys#block-ref ##sys#become! ##sys#halt ##sys#undefined-value ##sys#void ##sys#setbyte ##sys#byte ##sys#message ##sys#immediate? ##sys#stop-timer ##sys#start-timer ##sys#memory-info ##sys#symbol-table-info ##sys#ensure-heap-reserve ##sys#make-structure ##sys#allocate-vector ##sys#setislot ##sys#setslot ##sys#gc ##sys#notice ##sys#warn ##sys#error chicken.base#enable-warnings chicken.base#notice chicken.base#warning ##sys#notices-enabled ##sys#warnings-enabled ##sys#debug-mode? chicken.fixnum#fx/? chicken.fixnum#fx*? chicken.fixnum#fx-? chicken.fixnum#fx+? chicken.fixnum#fxrem chicken.fixnum#fxmod chicken.fixnum#fxgcd chicken.fixnum#fx/ chicken.fixnum#fxlen chicken.fixnum#fxeven? chicken.fixnum#fxodd? chicken.fixnum#fxshr chicken.fixnum#fxshl chicken.fixnum#fxnot chicken.fixnum#fxxor chicken.fixnum#fxior chicken.fixnum#fxand chicken.fixnum#fxneg chicken.fixnum#fxmax chicken.fixnum#fxmin chicken.fixnum#fx<= chicken.fixnum#fx>= chicken.fixnum#fx< chicken.fixnum#fx> chicken.fixnum#fx= chicken.fixnum#fx* chicken.fixnum#fx- chicken.fixnum#fx+ chicken.fixnum#fixnum-precision chicken.fixnum#fixnum-bits chicken.fixnum#most-negative-fixnum chicken.fixnum#most-positive-fixnum maximal-string-length chicken.base#rassoc chicken.base#alist-ref chicken.base#alist-update chicken.base#alist-update! chicken.base#compress chicken.base#join chicken.base#flatten chicken.base#butlast chicken.base#intersperse chicken.base#tail? chicken.base#atom? chicken.base#each chicken.base#list-of? chicken.base#o chicken.base#compose chicken.base#complement chicken.base#flip chicken.base#constantly chicken.base#disjoin chicken.base#conjoin chicken.base#identity chicken.base#symbol-escape chicken.base#parentheses-synonyms chicken.base#keyword-style chicken.base#case-sensitive chicken.base#emergency-exit chicken.base#exit chicken.base#on-exit chicken.base#cleanup-tasks chicken.base#exit-handler chicken.base#implicit-exit-handler chicken.base#foldr chicken.base#foldl chicken.base#make-promise ##sys#make-promise chicken.base#promise? chicken.base#flush-output chicken.base#open-output-string chicken.base#open-input-string chicken.base#get-output-string chicken.base#output-port-open? chicken.base#input-port-open? chicken.base#port-closed? chicken.base#port? chicken.base#vector-resize chicken.base#subvector chicken.base#vector-copy! chicken.base#set-record-printer! chicken.base#record-printer chicken.base#setter chicken.base#procedure-information chicken.base#make-parameter chicken.base#getter-with-setter chicken.base#char-name chicken.base#call/cc chicken.base#sleep chicken.base#void chicken.base#error chicken.base#current-error-port chicken.base#sub1 chicken.base#add1 chicken.base#print* chicken.base#print chicken.base#print-call-chain chicken.base#get-call-chain chicken.base#signum chicken.base#nan? chicken.base#infinite? chicken.base#finite? chicken.base#quotient&modulo chicken.base#quotient&remainder chicken.base#exact-integer? chicken.base#cplxnum? chicken.base#ratnum? chicken.base#bignum? chicken.base#flonum? chicken.base#fixnum? scheme#dynamic-wind scheme#map scheme#for-each scheme#load scheme#null-environment scheme#scheme-report-environment scheme#interaction-environment scheme#eval scheme#display scheme#write scheme#newline scheme#write-char scheme#read scheme#peek-char scheme#read-char scheme#char-ready? scheme#eof-object? scheme#with-output-to-file scheme#with-input-from-file scheme#call-with-output-file scheme#call-with-input-file scheme#close-output-port scheme#close-input-port scheme#open-output-file scheme#open-input-file scheme#current-output-port scheme#current-input-port scheme#output-port? scheme#input-port? scheme#call-with-current-continuation scheme#call-with-values scheme#values scheme#apply scheme#procedure? scheme#char-alphabetic? scheme#char-whitespace? scheme#char-numeric? scheme#char-lower-case? scheme#char-upper-case? scheme#char-ci<=? scheme#char-ci>=? scheme#char-ci<? scheme#char-ci>? scheme#char-ci=? scheme#char-downcase scheme#char-upcase scheme#char<=? scheme#char>=? scheme#char<? scheme#char>? scheme#char=? scheme#integer->char scheme#char->integer scheme#char? scheme#vector-fill! scheme#vector scheme#vector->list scheme#list->vector scheme#make-vector scheme#vector-set! scheme#vector-ref scheme#vector-length scheme#vector? scheme#symbol? scheme#number->string scheme#negative? scheme#positive? scheme#zero? scheme#inexact? scheme#exact? scheme#integer? scheme#rational? scheme#real? scheme#complex? scheme#number? scheme#<= scheme#>= scheme#< scheme#> scheme#= scheme#odd? scheme#even? scheme#modulo scheme#remainder scheme#quotient scheme#* scheme#- scheme#+ scheme#abs scheme#exact->inexact scheme#string-fill scheme#string-ci=? scheme#string=? scheme#string-set! scheme#string-ref scheme#string-length scheme#string? scheme#list? scheme#assoc scheme#assv scheme#assq scheme#member scheme#memv scheme#memq scheme#reverse scheme#list-ref scheme#list-tail scheme#length scheme#list scheme#null? scheme#cdddar scheme#cddadr scheme#cddaar scheme#cdaddr scheme#cdadar scheme#cdaadr scheme#cdaaar scheme#caddar scheme#cadadr scheme#cadaar scheme#caaddr scheme#caadar scheme#caaadr scheme#caaaar scheme#cdddr scheme#cddar scheme#cdadr scheme#cdaar scheme#cadar scheme#caadr scheme#caaar scheme#cddr scheme#cdar scheme#caar scheme#cddddr scheme#cadddr scheme#caddr scheme#cadr scheme#set-cdr! scheme#set-car! scheme#cdr scheme#car scheme#cons scheme#pair? scheme#equal? scheme#eqv? scheme#eq? scheme#boolean? scheme#not) 
o|inlining procedure: k8893 
o|inlining procedure: k8893 
o|removed side-effect free assignment to unused variable: scheme#string-fill 
o|inlining procedure: k9130 
o|inlining procedure: k9130 
o|inlining procedure: k9136 
o|inlining procedure: k9136 
o|inlining procedure: k9227 
o|inlining procedure: k9227 
o|inlining procedure: k9289 
o|inlining procedure: k9289 
o|inlining procedure: k9339 
o|inlining procedure: k9339 
o|inlining procedure: k9363 
o|inlining procedure: k9363 
o|inlining procedure: k9400 
o|inlining procedure: k9400 
o|inlining procedure: k9489 
o|inlining procedure: k9489 
o|inlining procedure: k9519 
o|inlining procedure: k9519 
o|inlining procedure: k9548 
o|inlining procedure: k9548 
o|inlining procedure: k9592 
o|inlining procedure: k9592 
o|inlining procedure: k9613 
o|propagated global variable: r961432558 scheme#values 
o|inlining procedure: k9613 
o|inlining procedure: k9625 
o|propagated global variable: r962632560 chicken.base#identity 
o|inlining procedure: k9625 
o|inlining procedure: k9643 
o|inlining procedure: k9643 
o|inlining procedure: k9672 
o|inlining procedure: k9672 
o|inlining procedure: k9684 
o|inlining procedure: k9684 
o|inlining procedure: k9708 
o|inlining procedure: k9708 
o|inlining procedure: k9742 
o|inlining procedure: k9742 
o|inlining procedure: k9770 
o|inlining procedure: k9770 
o|inlining procedure: k9779 
o|inlining procedure: k9779 
o|inlining procedure: k9801 
o|inlining procedure: k9801 
o|inlining procedure: k9840 
o|inlining procedure: k9840 
o|inlining procedure: k9866 
o|inlining procedure: k9866 
o|inlining procedure: k9913 
o|inlining procedure: k9913 
o|inlining procedure: k9934 
o|inlining procedure: k9934 
o|inlining procedure: k9967 
o|inlining procedure: k9967 
o|inlining procedure: k9982 
o|inlining procedure: k9982 
o|inlining procedure: k10047 
o|inlining procedure: k10047 
o|inlining procedure: k10063 
o|propagated global variable: r1006432592 scheme#assv 
o|inlining procedure: k10063 
o|inlining procedure: k10083 
o|inlining procedure: k10083 
o|inlining procedure: k10133 
o|inlining procedure: k10133 
o|inlining procedure: k10158 
o|inlining procedure: k10158 
o|inlining procedure: k10246 
o|inlining procedure: k10246 
o|inlining procedure: k10255 
o|propagated global variable: r1025632602 scheme#assv 
o|inlining procedure: k10255 
o|inlining procedure: k10275 
o|inlining procedure: k10275 
o|inlining procedure: k10293 
o|inlining procedure: k10293 
o|inlining procedure: k10352 
o|inlining procedure: k10352 
o|inlining procedure: k10488 
o|inlining procedure: k10488 
o|inlining procedure: k10497 
o|inlining procedure: k10497 
o|inlining procedure: k10509 
o|inlining procedure: k10509 
o|propagated global variable: r1051032615 ##sys#warnings-enabled 
o|inlining procedure: k10613 
o|inlining procedure: k10613 
o|inlining procedure: k10649 
o|inlining procedure: k10649 
o|inlining procedure: k10660 
o|inlining procedure: k10660 
o|inlining procedure: k10672 
o|inlining procedure: k10672 
o|inlining procedure: k10683 
o|inlining procedure: k10683 
o|inlining procedure: k10694 
o|inlining procedure: k10694 
o|inlining procedure: k10705 
o|inlining procedure: k10705 
o|inlining procedure: k10716 
o|inlining procedure: k10716 
o|inlining procedure: k10727 
o|inlining procedure: k10727 
o|inlining procedure: k10738 
o|inlining procedure: k10738 
o|inlining procedure: k10749 
o|inlining procedure: k10749 
o|inlining procedure: k10760 
o|inlining procedure: k10760 
o|inlining procedure: k10771 
o|inlining procedure: k10771 
o|inlining procedure: k10782 
o|inlining procedure: k10782 
o|inlining procedure: k10793 
o|inlining procedure: k10793 
o|inlining procedure: k10804 
o|inlining procedure: k10804 
o|inlining procedure: k10815 
o|inlining procedure: k10815 
o|inlining procedure: k10822 
o|inlining procedure: k10822 
o|inlining procedure: k10832 
o|inlining procedure: k10832 
o|inlining procedure: k10839 
o|inlining procedure: k10839 
o|inlining procedure: k10849 
o|inlining procedure: k10862 
o|inlining procedure: k10862 
o|inlining procedure: k10849 
o|inlining procedure: k10873 
o|inlining procedure: k10873 
o|inlining procedure: k10880 
o|inlining procedure: k10880 
o|inlining procedure: k10893 
o|inlining procedure: k10893 
o|inlining procedure: k10904 
o|inlining procedure: k10904 
o|inlining procedure: k10920 
o|inlining procedure: k10920 
o|inlining procedure: k10930 
o|inlining procedure: k10930 
o|inlining procedure: k10940 
o|inlining procedure: k10940 
o|inlining procedure: k10951 
o|inlining procedure: k10966 
o|inlining procedure: k10966 
o|inlining procedure: k10984 
o|inlining procedure: k11005 
o|inlining procedure: k11005 
o|inlining procedure: k11033 
o|inlining procedure: k11033 
o|inlining procedure: k10984 
o|inlining procedure: k10951 
o|inlining procedure: k11207 
o|inlining procedure: k11207 
o|inlining procedure: k11219 
o|inlining procedure: k11219 
o|inlining procedure: k11238 
o|inlining procedure: k11238 
o|inlining procedure: k11292 
o|inlining procedure: k11292 
o|contracted procedure: "(library.scm:1317) g15031504" 
o|contracted procedure: "(library.scm:1324) g15091510" 
o|inlining procedure: k11388 
o|inlining procedure: k11388 
o|inlining procedure: k11410 
o|inlining procedure: k11410 
o|inlining procedure: k11431 
o|inlining procedure: k11431 
o|inlining procedure: k11462 
o|inlining procedure: k11482 
o|inlining procedure: k11482 
o|inlining procedure: k11462 
o|inlining procedure: k11548 
o|inlining procedure: k11548 
o|inlining procedure: k11564 
o|inlining procedure: k11564 
o|inlining procedure: k11625 
o|inlining procedure: k11625 
o|inlining procedure: k11649 
o|inlining procedure: k11649 
o|inlining procedure: k11670 
o|inlining procedure: k11670 
o|inlining procedure: k11691 
o|inlining procedure: k11691 
o|inlining procedure: k11735 
o|inlining procedure: k11735 
o|inlining procedure: k11759 
o|inlining procedure: k11759 
o|inlining procedure: k11780 
o|inlining procedure: k11780 
o|inlining procedure: k11801 
o|inlining procedure: k11801 
o|inlining procedure: k11838 
o|inlining procedure: k11838 
o|inlining procedure: k11886 
o|inlining procedure: k11886 
o|inlining procedure: k11927 
o|inlining procedure: k11927 
o|inlining procedure: k11948 
o|inlining procedure: k11948 
o|contracted procedure: "(library.scm:1578) g17571758" 
o|inlining procedure: k12009 
o|inlining procedure: k12009 
o|contracted procedure: "(library.scm:1582) g17681769" 
o|inlining procedure: k12033 
o|inlining procedure: k12033 
o|contracted procedure: "(library.scm:1586) g17791780" 
o|inlining procedure: k12057 
o|inlining procedure: k12057 
o|contracted procedure: "(library.scm:1590) g17901791" 
o|inlining procedure: k12081 
o|inlining procedure: k12081 
o|contracted procedure: "(library.scm:1594) g18011802" 
o|inlining procedure: k12105 
o|inlining procedure: k12105 
o|contracted procedure: "(library.scm:1598) g18121813" 
o|inlining procedure: k12129 
o|inlining procedure: k12129 
o|contracted procedure: "(library.scm:1602) g18231824" 
o|inlining procedure: k12153 
o|inlining procedure: k12153 
o|contracted procedure: "(library.scm:1606) g18341835" 
o|inlining procedure: k12177 
o|inlining procedure: k12177 
o|contracted procedure: "(library.scm:1610) g18451846" 
o|inlining procedure: k12201 
o|inlining procedure: k12201 
o|contracted procedure: "(library.scm:1614) g18561857" 
o|inlining procedure: k12225 
o|inlining procedure: k12225 
o|contracted procedure: "(library.scm:1618) g18671868" 
o|inlining procedure: k12249 
o|inlining procedure: k12249 
o|contracted procedure: "(library.scm:1622) g18771878" 
o|inlining procedure: k12273 
o|inlining procedure: k12273 
o|contracted procedure: "(library.scm:1626) g18861887" 
o|inlining procedure: k12291 
o|inlining procedure: k12291 
o|contracted procedure: "(library.scm:1630) g18971898" 
o|inlining procedure: k12315 
o|inlining procedure: k12315 
o|contracted procedure: "(library.scm:1634) g19071908" 
o|inlining procedure: k12339 
o|inlining procedure: k12339 
o|contracted procedure: "(library.scm:1638) g19151916" 
o|inlining procedure: k12357 
o|inlining procedure: k12357 
o|contracted procedure: "(library.scm:1642) g19231924" 
o|inlining procedure: k12375 
o|inlining procedure: k12375 
o|contracted procedure: "(library.scm:1646) g19311932" 
o|inlining procedure: k12393 
o|inlining procedure: k12393 
o|contracted procedure: "(library.scm:1650) g19391940" 
o|inlining procedure: k12411 
o|inlining procedure: k12411 
o|contracted procedure: "(library.scm:1654) g19471948" 
o|inlining procedure: k12429 
o|inlining procedure: k12429 
o|contracted procedure: "(library.scm:1658) g19551956" 
o|inlining procedure: k12447 
o|inlining procedure: k12447 
o|contracted procedure: "(library.scm:1662) g19631964" 
o|inlining procedure: k12465 
o|inlining procedure: k12465 
o|contracted procedure: "(library.scm:1666) g19711972" 
o|inlining procedure: k12483 
o|inlining procedure: k12483 
o|contracted procedure: "(library.scm:1670) g19791980" 
o|inlining procedure: k12501 
o|inlining procedure: k12501 
o|contracted procedure: "(library.scm:1674) g19881989" 
o|inlining procedure: k12519 
o|inlining procedure: k12519 
o|contracted procedure: "(library.scm:1678) g19981999" 
o|inlining procedure: k12543 
o|inlining procedure: k12543 
o|contracted procedure: "(library.scm:1682) g20072008" 
o|inlining procedure: k12561 
o|inlining procedure: k12561 
o|contracted procedure: "(library.scm:1686) g20172018" 
o|inlining procedure: k12585 
o|inlining procedure: k12585 
o|contracted procedure: "(library.scm:1690) g20252026" 
o|inlining procedure: k12603 
o|inlining procedure: k12603 
o|contracted procedure: "(library.scm:1694) g20332034" 
o|inlining procedure: k12621 
o|inlining procedure: k12621 
o|contracted procedure: "(library.scm:1698) g20412042" 
o|inlining procedure: k12639 
o|inlining procedure: k12639 
o|inlining procedure: k12658 
o|inlining procedure: k12658 
o|inlining procedure: k12677 
o|inlining procedure: k12677 
o|inlining procedure: k12704 
o|inlining procedure: k12704 
o|inlining procedure: k12748 
o|contracted procedure: "(library.scm:1747) g21322133" 
o|inlining procedure: k12748 
o|inlining procedure: k12771 
o|contracted procedure: "(library.scm:1753) g21402141" 
o|inlining procedure: k12771 
o|inlining procedure: k12785 
o|inlining procedure: k12785 
o|inlining procedure: k12813 
o|contracted procedure: "(library.scm:1769) g21552156" 
o|contracted procedure: "(library.scm:1768) g21522153" 
o|inlining procedure: k12813 
o|inlining procedure: k12857 
o|inlining procedure: k12857 
o|contracted procedure: "(library.scm:1783) g21652166" 
o|inlining procedure: k12874 
o|contracted procedure: "(library.scm:1785) g21682169" 
o|contracted procedure: "(library.scm:1785) g21752176" 
o|contracted procedure: "(library.scm:1785) g21722173" 
o|inlining procedure: k12874 
o|contracted procedure: "(library.scm:1786) g21782179" 
o|inlining procedure: k12903 
o|inlining procedure: k12903 
o|inlining procedure: k12910 
o|inlining procedure: k12910 
o|inlining procedure: k12932 
o|contracted procedure: "(library.scm:1795) g21912192" 
o|inlining procedure: k12932 
o|inlining procedure: k12949 
o|inlining procedure: k12949 
o|inlining procedure: k12956 
o|inlining procedure: k12956 
o|inlining procedure: k12978 
o|contracted procedure: "(library.scm:1807) g22032204" 
o|inlining procedure: k12978 
o|inlining procedure: k12995 
o|contracted procedure: "(library.scm:1814) g22122213" 
o|inlining procedure: k12995 
o|inlining procedure: k13119 
o|inlining procedure: k13119 
o|inlining procedure: k13128 
o|contracted procedure: "(library.scm:1847) g22442245" 
o|inlining procedure: k13128 
o|contracted procedure: "(library.scm:1848) flonum->ratnum" 
o|inlining procedure: k13059 
o|contracted procedure: "(library.scm:1832) g22292230" 
o|inlining procedure: k13059 
o|contracted procedure: "(library.scm:1829) float-fraction-length2217" 
o|inlining procedure: k13036 
o|inlining procedure: k13036 
o|inlining procedure: k13087 
o|inlining procedure: k13087 
o|substituted constant variable: a13115 
o|inlining procedure: k13145 
o|contracted procedure: "(library.scm:1852) g22502251" 
o|contracted procedure: "(library.scm:1851) g22472248" 
o|inlining procedure: k13145 
o|inlining procedure: k13197 
o|inlining procedure: k13197 
o|inlining procedure: k13220 
o|inlining procedure: k13220 
o|inlining procedure: k13251 
o|contracted procedure: "(library.scm:1904) g23072308" 
o|contracted procedure: "(library.scm:1904) g23032304" 
o|contracted procedure: "(library.scm:1903) g22992300" 
o|inlining procedure: k13251 
o|inlining procedure: k13342 
o|inlining procedure: k13342 
o|inlining procedure: k13365 
o|contracted procedure: "(library.scm:1924) g23462347" 
o|contracted procedure: "(library.scm:1923) g23412342" 
o|contracted procedure: "(library.scm:1922) g23372338" 
o|contracted procedure: "(library.scm:1922) g23332334" 
o|contracted procedure: "(library.scm:1921) g23292330" 
o|contracted procedure: "(library.scm:1921) g23252326" 
o|inlining procedure: k13365 
o|contracted procedure: "(library.scm:1933) g23572358" 
o|contracted procedure: "(library.scm:1930) g23512352" 
o|inlining procedure: k13461 
o|contracted procedure: "(library.scm:1943) g23662367" 
o|contracted procedure: "(library.scm:1941) g23612362" 
o|inlining procedure: k13461 
o|inlining procedure: k13531 
o|inlining procedure: k13531 
o|inlining procedure: k13541 
o|contracted procedure: "(library.scm:1957) g23812382" 
o|contracted procedure: "(library.scm:1956) g23772378" 
o|inlining procedure: k13541 
o|inlining procedure: k13576 
o|inlining procedure: k13576 
o|inlining procedure: k13586 
o|contracted procedure: "(library.scm:1967) g23942395" 
o|contracted procedure: "(library.scm:1966) g23902391" 
o|inlining procedure: k13586 
o|inlining procedure: k13621 
o|inlining procedure: k13621 
o|inlining procedure: k13631 
o|contracted procedure: "(library.scm:1977) g24052406" 
o|contracted procedure: "(library.scm:1976) g24022403" 
o|inlining procedure: k13631 
o|inlining procedure: k13658 
o|inlining procedure: k13658 
o|inlining procedure: k13665 
o|contracted procedure: "(library.scm:1986) g24142415" 
o|inlining procedure: k13665 
o|inlining procedure: k13711 
o|inlining procedure: k13711 
o|inlining procedure: k13783 
o|inlining procedure: k13783 
o|inlining procedure: k13801 
o|inlining procedure: k13801 
o|inlining procedure: k13855 
o|inlining procedure: k13855 
o|contracted procedure: "(library.scm:2013) find-ratio" 
o|inlining procedure: k13887 
o|inlining procedure: k13887 
o|inlining procedure: k13931 
o|inlining procedure: k13931 
o|inlining procedure: k13968 
o|contracted procedure: "(library.scm:2046) g24772478" 
o|contracted procedure: "(library.scm:2045) g24732474" 
o|inlining procedure: k13968 
o|inlining procedure: k14010 
o|inlining procedure: k14010 
o|inlining procedure: k14062 
o|inlining procedure: k14062 
o|inlining procedure: k14091 
o|inlining procedure: k14091 
o|inlining procedure: k14125 
o|inlining procedure: k14125 
o|inlining procedure: k14159 
o|inlining procedure: k14159 
o|inlining procedure: k14184 
o|inlining procedure: k14184 
o|inlining procedure: k14223 
o|inlining procedure: k14223 
o|inlining procedure: k14232 
o|substituted constant variable: a14239 
o|inlining procedure: k14232 
o|inlining procedure: k14246 
o|inlining procedure: k14246 
o|inlining procedure: k14265 
o|inlining procedure: k14265 
o|inlining procedure: k14274 
o|substituted constant variable: a14281 
o|inlining procedure: k14274 
o|inlining procedure: k14291 
o|inlining procedure: k14291 
o|inlining procedure: k14349 
o|inlining procedure: k14367 
o|inlining procedure: k14367 
o|inlining procedure: k14385 
o|inlining procedure: k14385 
o|substituted constant variable: a14398 
o|substituted constant variable: a14400 
o|substituted constant variable: a14402 
o|substituted constant variable: a14407 
o|substituted constant variable: a14409 
o|inlining procedure: k14349 
o|substituted constant variable: a14440 
o|inlining procedure: k14475 
o|inlining procedure: k14475 
o|inlining procedure: k14549 
o|inlining procedure: k14549 
o|inlining procedure: k14600 
o|inlining procedure: k14616 
o|inlining procedure: k14616 
o|inlining procedure: k14600 
o|contracted procedure: "(library.scm:2187) g26192620" 
o|inlining procedure: k14650 
o|contracted procedure: "(library.scm:2190) g26242625" 
o|inlining procedure: k14674 
o|inlining procedure: k14674 
o|inlining procedure: k14650 
o|inlining procedure: k14722 
o|inlining procedure: k14722 
o|inlining procedure: k14767 
o|inlining procedure: k14767 
o|inlining procedure: k14839 
o|inlining procedure: k14839 
o|inlining procedure: k14854 
o|contracted procedure: "(library.scm:2232) g26572658" 
o|inlining procedure: k14854 
o|inlining procedure: k14877 
o|inlining procedure: k14877 
o|contracted procedure: "(library.scm:2237) square2655" 
o|inlining procedure: k14933 
o|inlining procedure: k14933 
o|inlining procedure: k14953 
o|inlining procedure: k14953 
o|inlining procedure: k14971 
o|contracted procedure: "(library.scm:2257) g26852686" 
o|contracted procedure: "(library.scm:2256) g26822683" 
o|inlining procedure: k14971 
o|inlining procedure: k15008 
o|contracted procedure: "(library.scm:2266) g26922693" 
o|inlining procedure: k15043 
o|contracted procedure: "(library.scm:2268) g26972698" 
o|inlining procedure: k15043 
o|inlining procedure: k15008 
o|inlining procedure: k15077 
o|inlining procedure: k15077 
o|inlining procedure: k15095 
o|inlining procedure: k15095 
o|inlining procedure: k15114 
o|inlining procedure: k15114 
o|inlining procedure: k15151 
o|inlining procedure: k15151 
o|inlining procedure: k15183 
o|contracted procedure: "(library.scm:2294) g27222723" 
o|inlining procedure: k15196 
o|contracted procedure: "(library.scm:2297) g27272728" 
o|inlining procedure: k15196 
o|inlining procedure: k15183 
o|inlining procedure: k15224 
o|inlining procedure: k15224 
o|contracted procedure: "(library.scm:2304) g27362737" 
o|inlining procedure: k15259 
o|inlining procedure: k15259 
o|inlining procedure: k15279 
o|inlining procedure: k15279 
o|inlining procedure: k15324 
o|inlining procedure: k15324 
o|inlining procedure: k15344 
o|inlining procedure: k15344 
o|substituted constant variable: a15417 
o|inlining procedure: k15435 
o|contracted procedure: "(library.scm:2360) g28022803" 
o|contracted procedure: "(library.scm:2358) g27992800" 
o|inlining procedure: k15435 
o|inlining procedure: k15499 
o|inlining procedure: k15499 
o|contracted procedure: "(library.scm:2364) g28102811" 
o|contracted procedure: "(library.scm:2363) g28072808" 
o|inlining procedure: k15614 
o|inlining procedure: k15627 
o|inlining procedure: k15627 
o|inlining procedure: k15614 
o|contracted procedure: "(library.scm:2402) g28402841" 
o|inlining procedure: k15656 
o|inlining procedure: k15656 
o|inlining procedure: k15670 
o|inlining procedure: k15679 
o|inlining procedure: k15691 
o|inlining procedure: k15691 
o|inlining procedure: k15679 
o|inlining procedure: k15707 
o|inlining procedure: k15707 
o|inlining procedure: k15713 
o|contracted procedure: "(library.scm:2427) mantexp->dbl" 
o|inlining procedure: k15512 
o|inlining procedure: k15512 
o|contracted procedure: "(library.scm:2394) ldexp" 
o|propagated global variable: a15571 chicken.flonum#flonum-precision 
o|inlining procedure: k15713 
o|inlining procedure: k15670 
o|inlining procedure: k15756 
o|inlining procedure: k15756 
o|inlining procedure: k15782 
o|inlining procedure: k15782 
o|inlining procedure: k15809 
o|inlining procedure: k15809 
o|inlining procedure: k15837 
o|inlining procedure: k15837 
o|substituted constant variable: a15844 
o|substituted constant variable: a15848 
o|substituted constant variable: a15855 
o|contracted procedure: "(library.scm:2449) g28842885" 
o|inlining procedure: k15864 
o|inlining procedure: k15864 
o|inlining procedure: k15897 
o|inlining procedure: k15897 
o|contracted procedure: "(library.scm:2464) g29072908" 
o|inlining procedure: k15917 
o|inlining procedure: k15917 
o|inlining procedure: k15939 
o|inlining procedure: k15939 
o|inlining procedure: k15950 
o|inlining procedure: k15950 
o|inlining procedure: k15964 
o|inlining procedure: k15987 
o|inlining procedure: k15987 
o|inlining procedure: k16010 
o|inlining procedure: k16010 
o|substituted constant variable: a16017 
o|substituted constant variable: a16019 
o|contracted procedure: "(library.scm:2486) g29432944" 
o|inlining procedure: k15964 
o|inlining procedure: k16022 
o|inlining procedure: k16043 
o|inlining procedure: k16061 
o|inlining procedure: k16073 
o|inlining procedure: k16089 
o|inlining procedure: k16089 
o|inlining procedure: k16073 
o|inlining procedure: k16061 
o|inlining procedure: k16043 
o|inlining procedure: k16134 
o|inlining procedure: k16134 
o|inlining procedure: k16149 
o|inlining procedure: k16149 
o|inlining procedure: k16161 
o|inlining procedure: k16161 
o|inlining procedure: k16173 
o|inlining procedure: k16173 
o|inlining procedure: k16185 
o|inlining procedure: k16185 
o|substituted constant variable: a16198 
o|substituted constant variable: a16200 
o|substituted constant variable: a16202 
o|substituted constant variable: a16204 
o|substituted constant variable: a16206 
o|substituted constant variable: a16208 
o|substituted constant variable: a16210 
o|substituted constant variable: a16212 
o|substituted constant variable: a16214 
o|substituted constant variable: a16216 
o|contracted procedure: "(library.scm:2501) g29712972" 
o|inlining procedure: k16225 
o|inlining procedure: k16225 
o|inlining procedure: k16022 
o|inlining procedure: k16236 
o|inlining procedure: k16236 
o|inlining procedure: k16261 
o|inlining procedure: k16276 
o|inlining procedure: k16276 
o|inlining procedure: k16261 
o|inlining procedure: k16319 
o|inlining procedure: k16331 
o|inlining procedure: k16341 
o|inlining procedure: k16341 
o|inlining procedure: k16331 
o|inlining procedure: k16319 
o|inlining procedure: k16361 
o|inlining procedure: k16377 
o|inlining procedure: k16387 
o|inlining procedure: k16405 
o|inlining procedure: k16425 
o|inlining procedure: "(library.scm:2555) make-nan2854" 
o|inlining procedure: k16425 
o|substituted constant variable: a16452 
o|substituted constant variable: a16454 
o|substituted constant variable: a16456 
o|inlining procedure: k16405 
o|inlining procedure: k16387 
o|inlining procedure: k16377 
o|inlining procedure: k16361 
o|substituted constant variable: a16474 
o|inlining procedure: k16478 
o|inlining procedure: k16478 
o|inlining procedure: k16490 
o|inlining procedure: k16490 
o|inlining procedure: k16502 
o|inlining procedure: k16502 
o|inlining procedure: k16514 
o|inlining procedure: k16514 
o|substituted constant variable: a16527 
o|substituted constant variable: a16529 
o|substituted constant variable: a16531 
o|substituted constant variable: a16533 
o|substituted constant variable: a16535 
o|substituted constant variable: a16537 
o|substituted constant variable: a16539 
o|substituted constant variable: a16541 
o|substituted constant variable: a16543 
o|substituted constant variable: a16545 
o|substituted constant variable: a16547 
o|contracted procedure: "(library.scm:2526) g30453046" 
o|inlining procedure: k16561 
o|contracted procedure: "(library.scm:2521) g30323033" 
o|inlining procedure: k16561 
o|inlining procedure: k16582 
o|inlining procedure: k16610 
o|inlining procedure: k16632 
o|inlining procedure: k16642 
o|inlining procedure: k16642 
o|inlining procedure: k16632 
o|inlining procedure: k16665 
o|inlining procedure: k16665 
o|inlining procedure: k16610 
o|inlining procedure: k16719 
o|inlining procedure: k16719 
o|inlining procedure: k16729 
o|inlining procedure: k16752 
o|inlining procedure: k16752 
o|inlining procedure: "(library.scm:2584) make-nan2854" 
o|inlining procedure: k16729 
o|substituted constant variable: a16792 
o|substituted constant variable: a16794 
o|substituted constant variable: a16799 
o|substituted constant variable: a16801 
o|contracted procedure: "(library.scm:2565) g31273128" 
o|inlining procedure: k16808 
o|inlining procedure: k16808 
o|substituted constant variable: a16815 
o|substituted constant variable: a16817 
o|contracted procedure: "(library.scm:2561) g31153116" 
o|inlining procedure: k16582 
o|inlining procedure: k16824 
o|inlining procedure: k16836 
o|inlining procedure: k16836 
o|inlining procedure: k16846 
o|inlining procedure: k16846 
o|inlining procedure: k16824 
o|inlining procedure: k16853 
o|inlining procedure: k16853 
o|inlining procedure: k16869 
o|contracted procedure: "(library.scm:2594) g31813182" 
o|contracted procedure: "(library.scm:2593) g31773178" 
o|inlining procedure: k16869 
o|inlining procedure: k16910 
o|inlining procedure: k16929 
o|inlining procedure: k16944 
o|inlining procedure: k16944 
o|contracted procedure: "(library.scm:2602) g32053206" 
o|contracted procedure: "(library.scm:2601) g32013202" 
o|inlining procedure: k16929 
o|inlining procedure: k16910 
o|inlining procedure: k16991 
o|inlining procedure: k16991 
o|substituted constant variable: a17018 
o|substituted constant variable: a17023 
o|substituted constant variable: a17025 
o|substituted constant variable: a17030 
o|substituted constant variable: a17032 
o|substituted constant variable: a17034 
o|contracted procedure: "(library.scm:2590) g31653166" 
o|inlining procedure: k17091 
o|inlining procedure: k17111 
o|inlining procedure: k17111 
o|inlining procedure: k17124 
o|inlining procedure: k17124 
o|inlining procedure: k17155 
o|inlining procedure: k17155 
o|inlining procedure: k17168 
o|inlining procedure: k17168 
o|inlining procedure: k17199 
o|inlining procedure: k17199 
o|inlining procedure: k17212 
o|inlining procedure: k17212 
o|substituted constant variable: a17238 
o|substituted constant variable: a17240 
o|substituted constant variable: a17245 
o|substituted constant variable: a17247 
o|substituted constant variable: a17252 
o|substituted constant variable: a17254 
o|substituted constant variable: a17259 
o|substituted constant variable: a17261 
o|substituted constant variable: a17266 
o|substituted constant variable: a17268 
o|substituted constant variable: a17273 
o|substituted constant variable: a17275 
o|contracted procedure: "(library.scm:2629) g32553256" 
o|inlining procedure: k17091 
o|inlining procedure: k17284 
o|inlining procedure: k17284 
o|contracted procedure: "(library.scm:2628) g32453246" 
o|inlining procedure: k17309 
o|inlining procedure: k17309 
o|inlining procedure: k17382 
o|inlining procedure: k17403 
o|inlining procedure: k17403 
o|inlining procedure: k17382 
o|inlining procedure: k17447 
o|inlining procedure: k17447 
o|inlining procedure: k17456 
o|inlining procedure: k17456 
o|inlining procedure: k17468 
o|inlining procedure: k17468 
o|inlining procedure: k17480 
o|inlining procedure: k17480 
o|inlining procedure: k17489 
o|inlining procedure: k17489 
o|inlining procedure: k17498 
o|inlining procedure: k17507 
o|inlining procedure: k17507 
o|inlining procedure: k17498 
o|inlining procedure: k17635 
o|inlining procedure: k17635 
o|inlining procedure: "(library.scm:2731) err3367" 
o|inlining procedure: k17641 
o|inlining procedure: k17641 
o|inlining procedure: "(library.scm:2730) err3367" 
o|inlining procedure: k17681 
o|contracted procedure: "(library.scm:2740) g33913400" 
o|inlining procedure: k17681 
o|inlining procedure: k17727 
o|inlining procedure: k17727 
o|inlining procedure: k17754 
o|inlining procedure: k17754 
o|inlining procedure: k17791 
o|inlining procedure: k17791 
o|inlining procedure: k17839 
o|inlining procedure: k17839 
o|inlining procedure: k17863 
o|inlining procedure: k17863 
o|inlining procedure: k17882 
o|inlining procedure: k17882 
o|inlining procedure: k17903 
o|inlining procedure: k17903 
o|inlining procedure: k17943 
o|inlining procedure: k17943 
o|inlining procedure: k17982 
o|inlining procedure: k17982 
o|inlining procedure: k18028 
o|inlining procedure: k18028 
o|inlining procedure: k18100 
o|inlining procedure: k18100 
o|inlining procedure: k18174 
o|inlining procedure: k18174 
o|inlining procedure: k18276 
o|inlining procedure: k18276 
o|inlining procedure: k18307 
o|inlining procedure: k18307 
o|inlining procedure: k18337 
o|inlining procedure: k18337 
o|inlining procedure: k18407 
o|inlining procedure: k18407 
o|inlining procedure: k18471 
o|inlining procedure: k18471 
o|inlining procedure: k18509 
o|inlining procedure: k18509 
o|inlining procedure: k18557 
o|inlining procedure: k18557 
o|inlining procedure: k18575 
o|inlining procedure: k18575 
o|inlining procedure: k18603 
o|inlining procedure: k18618 
o|inlining procedure: k18618 
o|inlining procedure: k18603 
o|inlining procedure: k18654 
o|inlining procedure: k18654 
o|inlining procedure: k18685 
o|inlining procedure: k18703 
o|inlining procedure: k18703 
o|inlining procedure: k18685 
o|inlining procedure: k18750 
o|inlining procedure: k18750 
o|inlining procedure: k18827 
o|inlining procedure: k18827 
o|inlining procedure: k18861 
o|inlining procedure: k18861 
o|inlining procedure: k18916 
o|inlining procedure: k18916 
o|substituted constant variable: a18923 
o|substituted constant variable: a18928 
o|inlining procedure: k18995 
o|inlining procedure: k18995 
o|inlining procedure: k19006 
o|inlining procedure: k19006 
o|inlining procedure: k19017 
o|inlining procedure: k19017 
o|inlining procedure: k19028 
o|inlining procedure: k19028 
o|inlining procedure: k19039 
o|propagated global variable: r1904033260 ##sys#standard-input 
o|removed unused formal parameters: (convert?3908) 
o|inlining procedure: k19054 
o|inlining procedure: k19054 
o|removed unused parameter to known procedure: convert?3908 "(library.scm:3311) body38983907" 
o|inlining procedure: k19039 
o|inlining procedure: k19081 
o|inlining procedure: k19081 
o|removed unused parameter to known procedure: convert?3908 "(library.scm:3311) body38983907" 
o|inlining procedure: k19100 
o|propagated global variable: r1910133266 ##sys#standard-output 
o|removed unused formal parameters: (convert?3933) 
o|inlining procedure: k19115 
o|inlining procedure: k19115 
o|removed unused parameter to known procedure: convert?3933 "(library.scm:3321) body39233932" 
o|inlining procedure: k19100 
o|inlining procedure: k19142 
o|inlining procedure: k19142 
o|removed unused parameter to known procedure: convert?3933 "(library.scm:3321) body39233932" 
o|inlining procedure: k19161 
o|propagated global variable: r1916233272 ##sys#standard-error 
o|removed unused formal parameters: (convert?3958) 
o|inlining procedure: k19176 
o|inlining procedure: k19176 
o|removed unused parameter to known procedure: convert?3958 "(library.scm:3331) body39483957" 
o|inlining procedure: k19161 
o|inlining procedure: k19203 
o|inlining procedure: k19203 
o|removed unused parameter to known procedure: convert?3958 "(library.scm:3331) body39483957" 
o|inlining procedure: k19222 
o|inlining procedure: k19222 
o|inlining procedure: k19260 
o|inlining procedure: k19260 
o|inlining procedure: k19285 
o|inlining procedure: k19285 
o|inlining procedure: k19291 
o|inlining procedure: k19291 
o|inlining procedure: k19314 
o|inlining procedure: k19314 
o|inlining procedure: k19327 
o|inlining procedure: k19327 
o|substituted constant variable: a19338 
o|substituted constant variable: a19340 
o|substituted constant variable: a19342 
o|inlining procedure: k19352 
o|inlining procedure: k19352 
o|inlining procedure: k19527 
o|inlining procedure: k19527 
o|substituted constant variable: a19543 
o|substituted constant variable: a19545 
o|substituted constant variable: a19566 
o|substituted constant variable: a19567 
o|inlining procedure: k19580 
o|inlining procedure: k19580 
o|inlining procedure: k19602 
o|inlining procedure: k19615 
o|inlining procedure: k19615 
o|inlining procedure: k19602 
o|inlining procedure: k19674 
o|inlining procedure: k19674 
o|inlining procedure: k19712 
o|inlining procedure: k19712 
o|inlining procedure: k19746 
o|inlining procedure: k19746 
o|inlining procedure: k19787 
o|inlining procedure: k19787 
o|inlining procedure: k19806 
o|inlining procedure: k19806 
o|inlining procedure: k19934 
o|inlining procedure: k19934 
o|inlining procedure: k19973 
o|inlining procedure: k20012 
o|inlining procedure: k20012 
o|inlining procedure: k19973 
o|inlining procedure: k20040 
o|inlining procedure: k20040 
o|substituted constant variable: a20088 
o|substituted constant variable: a20089 
o|substituted constant variable: a20113 
o|substituted constant variable: a20114 
o|inlining procedure: k20130 
o|inlining procedure: k20130 
o|substituted constant variable: a20188 
o|substituted constant variable: a20189 
o|inlining procedure: k20196 
o|inlining procedure: k20196 
o|substituted constant variable: a20225 
o|substituted constant variable: a20226 
o|substituted constant variable: a20243 
o|substituted constant variable: a20244 
o|inlining procedure: k20298 
o|inlining procedure: k20298 
o|inlining procedure: k20321 
o|inlining procedure: k20321 
o|inlining procedure: k20361 
o|inlining procedure: k20361 
o|inlining procedure: k20380 
o|inlining procedure: k20380 
o|inlining procedure: k20419 
o|inlining procedure: k20419 
o|inlining procedure: k20444 
o|inlining procedure: k20444 
o|inlining procedure: k20463 
o|inlining procedure: k20463 
o|inlining procedure: k20488 
o|inlining procedure: k20488 
o|inlining procedure: k20554 
o|inlining procedure: k20554 
o|inlining procedure: k20589 
o|inlining procedure: k20589 
o|inlining procedure: k20605 
o|inlining procedure: k20605 
o|inlining procedure: k20639 
o|inlining procedure: k20639 
o|inlining procedure: k20673 
o|inlining procedure: k20673 
o|inlining procedure: k20707 
o|inlining procedure: k20707 
o|inlining procedure: k20748 
o|inlining procedure: k20763 
o|inlining procedure: k20763 
o|substituted constant variable: a20801 
o|substituted constant variable: a20806 
o|inlining procedure: k20748 
o|inlining procedure: k20831 
o|inlining procedure: k20831 
o|inlining procedure: k20851 
o|inlining procedure: k20851 
o|inlining procedure: k20886 
o|inlining procedure: k20886 
o|inlining procedure: k20908 
o|inlining procedure: k20908 
o|inlining procedure: k20933 
o|inlining procedure: k20933 
o|substituted constant variable: a20962 
o|substituted constant variable: a20964 
o|substituted constant variable: a20969 
o|substituted constant variable: a20971 
o|inlining procedure: k20972 
o|inlining procedure: k20972 
o|inlining procedure: k21035 
o|inlining procedure: k21035 
o|inlining procedure: k21047 
o|inlining procedure: k21047 
o|substituted constant variable: a21060 
o|substituted constant variable: a21062 
o|substituted constant variable: a21064 
o|substituted constant variable: a21066 
o|inlining procedure: k21070 
o|inlining procedure: k21070 
o|substituted constant variable: a21083 
o|substituted constant variable: a21085 
o|substituted constant variable: a21087 
o|substituted constant variable: a21089 
o|substituted constant variable: a21091 
o|substituted constant variable: a21093 
o|substituted constant variable: a21095 
o|substituted constant variable: a21097 
o|substituted constant variable: a21099 
o|substituted constant variable: a21101 
o|substituted constant variable: a21103 
o|substituted constant variable: a21105 
o|substituted constant variable: a21107 
o|substituted constant variable: a21109 
o|inlining procedure: k21110 
o|inlining procedure: k21110 
o|inlining procedure: k21142 
o|inlining procedure: k21142 
o|inlining procedure: k21133 
o|inlining procedure: k21175 
o|inlining procedure: k21175 
o|inlining procedure: k21206 
o|inlining procedure: k21206 
o|inlining procedure: k21225 
o|inlining procedure: k21258 
o|inlining procedure: k21258 
o|inlining procedure: k21300 
o|inlining procedure: k21300 
o|inlining procedure: "(library.scm:3933) build-keyword4435" 
o|inlining procedure: k21316 
o|inlining procedure: "(library.scm:3934) build-keyword4435" 
o|inlining procedure: k21316 
o|inlining procedure: "(library.scm:3937) build-symbol4434" 
o|inlining procedure: k21346 
o|inlining procedure: k21346 
o|inlining procedure: k21358 
o|inlining procedure: k21358 
o|inlining procedure: k21225 
o|inlining procedure: k21133 
o|inlining procedure: k21398 
o|inlining procedure: k21398 
o|inlining procedure: k21419 
o|inlining procedure: "(library.scm:3965) build-keyword4435" 
o|inlining procedure: k21419 
o|inlining procedure: k21441 
o|inlining procedure: k21441 
o|inlining procedure: k21456 
o|inlining procedure: k21456 
o|inlining procedure: "(library.scm:3973) build-symbol4434" 
o|inlining procedure: k21466 
o|inlining procedure: k21466 
o|substituted constant variable: a21474 
o|substituted constant variable: a21475 
o|inlining procedure: k21483 
o|inlining procedure: k21504 
o|inlining procedure: k21504 
o|substituted constant variable: a21517 
o|inlining procedure: k21483 
o|inlining procedure: k21534 
o|inlining procedure: k21555 
o|inlining procedure: k21555 
o|substituted constant variable: a21568 
o|substituted constant variable: a21576 
o|inlining procedure: k21572 
o|inlining procedure: k21572 
o|substituted constant variable: a21584 
o|inlining procedure: k21534 
o|inlining procedure: k21611 
o|inlining procedure: k21611 
o|inlining procedure: k21647 
o|inlining procedure: k21647 
o|inlining procedure: k21656 
o|inlining procedure: k21656 
o|inlining procedure: k21675 
o|inlining procedure: k21675 
o|inlining procedure: "(library.scm:4025) build-keyword4435" 
o|inlining procedure: "(library.scm:4025) build-symbol4434" 
o|inlining procedure: k21749 
o|inlining procedure: k21775 
o|inlining procedure: k21775 
o|inlining procedure: k21804 
o|inlining procedure: k21804 
o|inlining procedure: k21813 
o|inlining procedure: k21813 
o|inlining procedure: k21823 
o|inlining procedure: k21823 
o|inlining procedure: k21749 
o|inlining procedure: k21855 
o|inlining procedure: k21855 
o|inlining procedure: k21899 
o|inlining procedure: k21899 
o|inlining procedure: k21921 
o|inlining procedure: k21921 
o|inlining procedure: k21945 
o|inlining procedure: k21945 
o|substituted constant variable: a21952 
o|substituted constant variable: a21954 
o|substituted constant variable: a21956 
o|substituted constant variable: a21958 
o|inlining procedure: k21962 
o|inlining procedure: k21962 
o|inlining procedure: k21971 
o|inlining procedure: k21971 
o|inlining procedure: k21996 
o|contracted procedure: "(library.scm:4084) g47344735" 
o|inlining procedure: k22025 
o|inlining procedure: k22025 
o|inlining procedure: k22044 
o|inlining procedure: k22062 
o|inlining procedure: k22078 
o|inlining procedure: k22078 
o|inlining procedure: k22062 
o|inlining procedure: k22044 
o|substituted constant variable: a22177 
o|inlining procedure: k22179 
o|inlining procedure: k22179 
o|substituted constant variable: a22182 
o|inlining procedure: k21996 
o|inlining procedure: k22205 
o|inlining procedure: k22217 
o|inlining procedure: k22217 
o|inlining procedure: k22205 
o|inlining procedure: k22251 
o|inlining procedure: k22251 
o|inlining procedure: k22264 
o|inlining procedure: k22264 
o|substituted constant variable: a22277 
o|substituted constant variable: a22279 
o|inlining procedure: "(library.scm:4131) build-symbol4434" 
o|inlining procedure: k22338 
o|inlining procedure: k22353 
o|inlining procedure: k22353 
o|inlining procedure: k22338 
o|inlining procedure: k22377 
o|inlining procedure: k22377 
o|inlining procedure: k22398 
o|inlining procedure: k22398 
o|inlining procedure: k22417 
o|inlining procedure: k22438 
o|inlining procedure: k22465 
o|inlining procedure: k22480 
o|inlining procedure: k22480 
o|inlining procedure: k22465 
o|inlining procedure: k22516 
o|inlining procedure: k22516 
o|inlining procedure: k22438 
o|inlining procedure: k22551 
o|inlining procedure: k22551 
o|inlining procedure: k22566 
o|inlining procedure: k22566 
o|inlining procedure: k22590 
o|inlining procedure: k22590 
o|inlining procedure: k22614 
o|inlining procedure: k22614 
o|inlining procedure: k22638 
o|inlining procedure: k22678 
o|inlining procedure: k22678 
o|inlining procedure: k22638 
o|inlining procedure: k22702 
o|inlining procedure: k22702 
o|inlining procedure: k22729 
o|inlining procedure: k22729 
o|inlining procedure: k22756 
o|inlining procedure: k22756 
o|inlining procedure: k22780 
o|inlining procedure: k22780 
o|inlining procedure: k22795 
o|inlining procedure: k22824 
o|inlining procedure: k22824 
o|inlining procedure: "(library.scm:4258) build-keyword4435" 
o|inlining procedure: k22795 
o|inlining procedure: k22885 
o|inlining procedure: k22912 
o|inlining procedure: k22912 
o|inlining procedure: "(library.scm:4277) build-symbol4434" 
o|inlining procedure: k22934 
o|inlining procedure: k22934 
o|inlining procedure: k22956 
o|inlining procedure: k22956 
o|substituted constant variable: a22959 
o|inlining procedure: k22885 
o|inlining procedure: k22973 
o|inlining procedure: k22973 
o|substituted constant variable: a22986 
o|substituted constant variable: a22988 
o|substituted constant variable: a22990 
o|substituted constant variable: a22992 
o|substituted constant variable: a22994 
o|substituted constant variable: a22996 
o|substituted constant variable: a22998 
o|substituted constant variable: a23000 
o|substituted constant variable: a23002 
o|substituted constant variable: a23004 
o|substituted constant variable: a23006 
o|substituted constant variable: a23008 
o|substituted constant variable: a23010 
o|substituted constant variable: a23012 
o|substituted constant variable: a23014 
o|substituted constant variable: a23016 
o|substituted constant variable: a23018 
o|inlining procedure: k22417 
o|inlining procedure: k23035 
o|inlining procedure: k23035 
o|inlining procedure: k23059 
o|inlining procedure: k23059 
o|inlining procedure: k23080 
o|inlining procedure: k23080 
o|inlining procedure: k23095 
o|inlining procedure: k23095 
o|inlining procedure: k23113 
o|inlining procedure: k23113 
o|substituted constant variable: a23144 
o|substituted constant variable: a23146 
o|substituted constant variable: a23148 
o|substituted constant variable: a23150 
o|substituted constant variable: a23155 
o|substituted constant variable: a23157 
o|substituted constant variable: a23159 
o|substituted constant variable: a23161 
o|substituted constant variable: a23163 
o|substituted constant variable: a23165 
o|substituted constant variable: a23167 
o|substituted constant variable: a23169 
o|substituted constant variable: a23171 
o|substituted constant variable: a23173 
o|inlining procedure: k23177 
o|inlining procedure: k23177 
o|inlining procedure: k23196 
o|inlining procedure: k23196 
o|inlining procedure: k23214 
o|inlining procedure: k23214 
o|inlining procedure: k23256 
o|inlining procedure: k23256 
o|inlining procedure: k23376 
o|inlining procedure: k23376 
o|inlining procedure: k23388 
o|inlining procedure: k23400 
o|inlining procedure: k23400 
o|inlining procedure: k23388 
o|inlining procedure: k23456 
o|inlining procedure: k23456 
o|substituted constant variable: a23486 
o|substituted constant variable: a23488 
o|substituted constant variable: a23490 
o|substituted constant variable: a23492 
o|inlining procedure: k23502 
o|inlining procedure: k23502 
o|inlining procedure: k23521 
o|inlining procedure: k23521 
o|inlining procedure: k23535 
o|inlining procedure: k23535 
o|inlining procedure: k23598 
o|inlining procedure: k23598 
o|substituted constant variable: a23629 
o|substituted constant variable: a23631 
o|substituted constant variable: a23636 
o|substituted constant variable: a23638 
o|substituted constant variable: a23660 
o|substituted constant variable: a23661 
o|substituted constant variable: a23678 
o|substituted constant variable: a23679 
o|inlining procedure: k23692 
o|propagated global variable: r2369333632 ##sys#standard-output 
o|inlining procedure: k23692 
o|substituted constant variable: a23711 
o|substituted constant variable: a23712 
o|substituted constant variable: a23729 
o|substituted constant variable: a23730 
o|propagated global variable: a23743 ##sys#standard-output 
o|substituted constant variable: a23744 
o|substituted constant variable: a23745 
o|contracted procedure: "(library.scm:4427) g50665067" 
o|inlining procedure: k23762 
o|contracted procedure: "(library.scm:4422) g50725079" 
o|propagated global variable: g50875088 ##sys#print 
o|inlining procedure: k23762 
o|propagated global variable: a23794 ##sys#standard-output 
o|substituted constant variable: a23795 
o|substituted constant variable: a23796 
o|contracted procedure: "(library.scm:4434) g50985099" 
o|inlining procedure: k23813 
o|contracted procedure: "(library.scm:4422) g51045111" 
o|propagated global variable: g51195120 ##sys#print 
o|inlining procedure: k23813 
o|substituted constant variable: a23858 
o|substituted constant variable: a23859 
o|inlining procedure: k23872 
o|inlining procedure: k23872 
o|inlining procedure: k23963 
o|inlining procedure: k23963 
o|inlining procedure: k23995 
o|inlining procedure: k23995 
o|substituted constant variable: special-characters5141 
o|inlining procedure: k24007 
o|inlining procedure: k24007 
o|inlining procedure: k24040 
o|inlining procedure: k24040 
o|substituted constant variable: a24113 
o|inlining procedure: k24120 
o|inlining procedure: k24120 
o|inlining procedure: k24132 
o|inlining procedure: k24132 
o|inlining procedure: k24147 
o|inlining procedure: k24147 
o|inlining procedure: k24166 
o|inlining procedure: k24188 
o|inlining procedure: k24188 
o|inlining procedure: k24203 
o|inlining procedure: k24203 
o|inlining procedure: k24216 
o|inlining procedure: k24216 
o|inlining procedure: k24240 
o|inlining procedure: k24240 
o|inlining procedure: k24166 
o|inlining procedure: k24258 
o|inlining procedure: k24258 
o|inlining procedure: k24311 
o|inlining procedure: k24311 
o|inlining procedure: k24329 
o|inlining procedure: k24329 
o|inlining procedure: k24344 
o|inlining procedure: k24344 
o|inlining procedure: k24353 
o|inlining procedure: k24380 
o|inlining procedure: k24380 
o|inlining procedure: k24353 
o|inlining procedure: k24408 
o|inlining procedure: k24408 
o|inlining procedure: k24424 
o|inlining procedure: k24424 
o|inlining procedure: k24439 
o|inlining procedure: k24457 
o|inlining procedure: k24457 
o|substituted constant variable: a24476 
o|substituted constant variable: a24478 
o|inlining procedure: k24439 
o|inlining procedure: k24488 
o|inlining procedure: k24488 
o|inlining procedure: k24511 
o|inlining procedure: k24527 
o|inlining procedure: k24527 
o|inlining procedure: k24556 
o|inlining procedure: k24556 
o|inlining procedure: k24577 
o|inlining procedure: k24577 
o|inlining procedure: k24595 
o|inlining procedure: k24595 
o|inlining procedure: k24613 
o|inlining procedure: k24613 
o|inlining procedure: k24631 
o|inlining procedure: k24631 
o|substituted constant variable: a24660 
o|substituted constant variable: a24662 
o|substituted constant variable: a24664 
o|substituted constant variable: a24666 
o|substituted constant variable: a24668 
o|substituted constant variable: a24670 
o|substituted constant variable: a24672 
o|substituted constant variable: a24681 
o|substituted constant variable: a24683 
o|inlining procedure: k24511 
o|inlining procedure: k24706 
o|inlining procedure: k24706 
o|inlining procedure: k24760 
o|inlining procedure: k24778 
o|inlining procedure: k24778 
o|inlining procedure: k24760 
o|inlining procedure: k24816 
o|inlining procedure: k24816 
o|inlining procedure: k24832 
o|inlining procedure: k24832 
o|inlining procedure: k24873 
o|inlining procedure: k24873 
o|substituted constant variable: a24886 
o|substituted constant variable: a24888 
o|inlining procedure: k24889 
o|inlining procedure: k24917 
o|inlining procedure: k24917 
o|inlining procedure: k24889 
o|inlining procedure: k24961 
o|inlining procedure: k24961 
o|inlining procedure: k24981 
o|inlining procedure: k24981 
o|inlining procedure: k24999 
o|inlining procedure: k24999 
o|inlining procedure: k25027 
o|inlining procedure: k25057 
o|inlining procedure: k25057 
o|inlining procedure: k25027 
o|inlining procedure: k25145 
o|inlining procedure: k25145 
o|substituted constant variable: a25162 
o|substituted constant variable: a25164 
o|substituted constant variable: saved544254435455 
o|inlining procedure: k25269 
o|inlining procedure: k25269 
o|inlining procedure: k25336 
o|inlining procedure: k25357 
o|inlining procedure: k25357 
o|inlining procedure: k25336 
o|inlining procedure: k25383 
o|inlining procedure: k25383 
o|inlining procedure: k25421 
o|inlining procedure: k25421 
o|inlining procedure: k25452 
o|inlining procedure: k25452 
o|inlining procedure: k25489 
o|inlining procedure: k25489 
o|substituted constant variable: a25549 
o|substituted constant variable: a25550 
o|inlining procedure: k25551 
o|inlining procedure: k25551 
o|inlining procedure: k25602 
o|inlining procedure: k25602 
o|inlining procedure: k25625 
o|inlining procedure: k25651 
o|inlining procedure: k25651 
o|inlining procedure: k25625 
o|inlining procedure: k25705 
o|inlining procedure: k25705 
o|substituted constant variable: c5651 
o|contracted procedure: "(library.scm:4943) extract5628" 
o|substituted constant variable: c5651 
o|inlining procedure: k25772 
o|inlining procedure: k25772 
o|substituted constant variable: c5651 
o|substituted constant variable: c5651 
o|inlining procedure: k25854 
o|inlining procedure: k25854 
o|inlining procedure: k25879 
o|inlining procedure: k25879 
o|inlining procedure: k25885 
o|inlining procedure: k25885 
o|inlining procedure: k25822 
o|inlining procedure: k25904 
o|inlining procedure: k25904 
o|inlining procedure: k25822 
o|substituted constant variable: a25952 
o|substituted constant variable: a25953 
o|inlining procedure: k26024 
o|inlining procedure: "(library.scm:4991) break5724" 
o|inlining procedure: k26024 
o|inlining procedure: k26053 
o|inlining procedure: k26053 
o|inlining procedure: k26089 
o|inlining procedure: k26089 
o|inlining procedure: k26112 
o|contracted procedure: "(library.scm:5012) g57345741" 
o|inlining procedure: k26112 
o|inlining procedure: k26153 
o|inlining procedure: k26153 
o|inlining procedure: k26207 
o|contracted procedure: "(library.scm:5079) g58045811" 
o|inlining procedure: k26207 
o|inlining procedure: k26245 
o|inlining procedure: k26245 
o|inlining procedure: k26288 
o|inlining procedure: k26288 
o|inlining procedure: k26300 
o|inlining procedure: k26300 
o|inlining procedure: k26312 
o|inlining procedure: k26312 
o|inlining procedure: k26324 
o|inlining procedure: k26324 
o|inlining procedure: k26336 
o|inlining procedure: k26336 
o|inlining procedure: k26348 
o|inlining procedure: k26348 
o|inlining procedure: k26360 
o|inlining procedure: k26360 
o|substituted constant variable: a26367 
o|substituted constant variable: a26369 
o|substituted constant variable: a26371 
o|substituted constant variable: a26373 
o|substituted constant variable: a26375 
o|substituted constant variable: a26377 
o|substituted constant variable: a26379 
o|substituted constant variable: a26381 
o|substituted constant variable: a26383 
o|substituted constant variable: a26385 
o|substituted constant variable: a26387 
o|substituted constant variable: a26389 
o|substituted constant variable: a26391 
o|substituted constant variable: a26393 
o|substituted constant variable: a26429 
o|substituted constant variable: a26431 
o|substituted constant variable: a26433 
o|inlining procedure: k26487 
o|inlining procedure: k26523 
o|inlining procedure: k26523 
o|inlining procedure: k26535 
o|inlining procedure: k26549 
o|inlining procedure: k26549 
o|inlining procedure: k26535 
o|inlining procedure: k26487 
o|inlining procedure: k26585 
o|inlining procedure: k26585 
o|inlining procedure: k26640 
o|propagated global variable: r2664133828 ##sys#current-exception-handler 
o|removed unused formal parameters: (convert?5973) 
o|inlining procedure: k26655 
o|inlining procedure: k26655 
o|removed unused parameter to known procedure: convert?5973 "(library.scm:5231) body59635972" 
o|inlining procedure: k26640 
o|inlining procedure: k26682 
o|inlining procedure: k26682 
o|removed unused parameter to known procedure: convert?5973 "(library.scm:5231) body59635972" 
o|inlining procedure: k26707 
o|inlining procedure: k26707 
o|inlining procedure: k26820 
o|contracted procedure: "(library.scm:5259) g60586067" 
o|inlining procedure: k26820 
o|inlining procedure: k26866 
o|contracted procedure: "(library.scm:5258) g60316040" 
o|inlining procedure: k26866 
o|inlining procedure: k26900 
o|contracted procedure: "(library.scm:5255) g60086015" 
o|inlining procedure: k26900 
o|inlining procedure: k26942 
o|inlining procedure: k26942 
o|inlining procedure: k26994 
o|contracted procedure: "(library.scm:5264) g60916100" 
o|inlining procedure: k26994 
o|inlining procedure: k27056 
o|inlining procedure: k27056 
o|inlining procedure: k27112 
o|inlining procedure: k27112 
o|inlining procedure: k27154 
o|inlining procedure: k27154 
o|inlining procedure: k27184 
o|inlining procedure: k27199 
o|inlining procedure: k27199 
o|inlining procedure: k27184 
o|inlining procedure: k27260 
o|inlining procedure: k27260 
o|substituted constant variable: a27285 
o|substituted constant variable: a27286 
o|inlining procedure: k27293 
o|inlining procedure: k27314 
o|inlining procedure: k27314 
o|inlining procedure: k27344 
o|inlining procedure: k27344 
o|inlining procedure: k27373 
o|inlining procedure: k27373 
o|inlining procedure: k27293 
o|inlining procedure: k27441 
(o x)|known list op on rest arg sublist: ##core#rest-cdr args6230 0 
o|inlining procedure: k27441 
o|inlining procedure: k27476 
o|inlining procedure: k27476 
o|inlining procedure: k27522 
o|inlining procedure: k27545 
o|inlining procedure: k27545 
o|inlining procedure: k27522 
o|inlining procedure: k27582 
o|inlining procedure: k27582 
o|inlining procedure: k27596 
o|inlining procedure: k27596 
o|inlining procedure: k27614 
o|inlining procedure: k27614 
o|inlining procedure: k27632 
o|inlining procedure: k27632 
o|inlining procedure: k27650 
o|inlining procedure: k27650 
o|inlining procedure: k27668 
o|inlining procedure: k27668 
o|inlining procedure: k27686 
o|inlining procedure: k27686 
o|inlining procedure: k27704 
o|inlining procedure: k27704 
o|inlining procedure: k27722 
o|inlining procedure: k27722 
o|inlining procedure: k27740 
o|inlining procedure: k27740 
o|inlining procedure: k27758 
o|inlining procedure: k27758 
o|inlining procedure: k27776 
o|inlining procedure: k27776 
o|inlining procedure: k27794 
o|inlining procedure: k27794 
o|inlining procedure: k27812 
o|inlining procedure: k27812 
o|inlining procedure: k27830 
o|inlining procedure: k27830 
o|inlining procedure: k27848 
o|inlining procedure: k27848 
o|inlining procedure: k27866 
o|inlining procedure: k27866 
o|inlining procedure: k27884 
o|inlining procedure: k27884 
o|inlining procedure: k27902 
o|inlining procedure: k27902 
o|inlining procedure: k27920 
o|inlining procedure: k27920 
o|inlining procedure: k27938 
o|inlining procedure: k27938 
o|inlining procedure: k27956 
o|inlining procedure: k27956 
o|inlining procedure: k27974 
o|inlining procedure: k27974 
o|inlining procedure: k27992 
o|inlining procedure: k27992 
o|inlining procedure: k28010 
o|inlining procedure: k28010 
o|inlining procedure: k28028 
o|inlining procedure: k28028 
o|inlining procedure: k28046 
o|inlining procedure: k28046 
o|inlining procedure: k28064 
o|inlining procedure: k28064 
o|inlining procedure: k28096 
o|inlining procedure: k28096 
o|substituted constant variable: a28114 
o|substituted constant variable: a28116 
o|substituted constant variable: a28118 
o|substituted constant variable: a28120 
o|substituted constant variable: a28122 
o|substituted constant variable: a28124 
o|substituted constant variable: a28126 
o|substituted constant variable: a28128 
o|substituted constant variable: a28130 
o|substituted constant variable: a28132 
o|substituted constant variable: a28134 
o|substituted constant variable: a28136 
o|substituted constant variable: a28138 
o|substituted constant variable: a28140 
o|substituted constant variable: a28142 
o|substituted constant variable: a28144 
o|substituted constant variable: a28146 
o|substituted constant variable: a28148 
o|substituted constant variable: a28150 
o|substituted constant variable: a28152 
o|substituted constant variable: a28154 
o|substituted constant variable: a28156 
o|substituted constant variable: a28158 
o|substituted constant variable: a28160 
o|substituted constant variable: a28162 
o|substituted constant variable: a28164 
o|substituted constant variable: a28166 
o|substituted constant variable: a28168 
o|substituted constant variable: a28170 
o|substituted constant variable: a28172 
o|substituted constant variable: a28174 
o|substituted constant variable: a28176 
o|substituted constant variable: a28178 
o|substituted constant variable: a28180 
o|substituted constant variable: a28182 
o|substituted constant variable: a28184 
o|substituted constant variable: a28186 
o|substituted constant variable: a28188 
o|substituted constant variable: a28190 
o|substituted constant variable: a28192 
o|substituted constant variable: a28194 
o|substituted constant variable: a28196 
o|substituted constant variable: a28198 
o|substituted constant variable: a28200 
o|substituted constant variable: a28202 
o|substituted constant variable: a28204 
o|substituted constant variable: a28206 
o|substituted constant variable: a28208 
o|substituted constant variable: a28210 
o|substituted constant variable: a28212 
o|substituted constant variable: a28214 
o|substituted constant variable: a28216 
o|substituted constant variable: a28218 
o|substituted constant variable: a28220 
o|substituted constant variable: a28222 
o|substituted constant variable: a28224 
o|inlining procedure: k28304 
o|inlining procedure: k28304 
o|inlining procedure: k28342 
o|inlining procedure: k28342 
o|inlining procedure: k28364 
o|inlining procedure: k28364 
o|inlining procedure: k28422 
o|inlining procedure: k28422 
o|contracted procedure: "(library.scm:5577) fetch6473" 
o|contracted procedure: "(library.scm:5594) free6492" 
o|inlining procedure: k28469 
o|inlining procedure: k28469 
o|inlining procedure: k28486 
o|inlining procedure: k28486 
o|contracted procedure: "(library.scm:5589) fetch6491" 
o|propagated global variable: ct6593 ##sys#current-thread 
o|inlining procedure: k28670 
o|inlining procedure: k28670 
o|inlining procedure: k28698 
o|inlining procedure: k28698 
o|inlining procedure: k28755 
o|inlining procedure: k28755 
o|inlining procedure: k28795 
o|inlining procedure: k28807 
o|inlining procedure: k28807 
o|inlining procedure: k28795 
o|inlining procedure: k28834 
o|inlining procedure: k28834 
o|inlining procedure: k28880 
o|inlining procedure: k28880 
o|substituted constant variable: a28938 
o|inlining procedure: k28931 
o|inlining procedure: k28931 
o|substituted constant variable: a28951 
o|inlining procedure: k28962 
o|inlining procedure: k29002 
o|inlining procedure: k29040 
o|inlining procedure: k29040 
o|inlining procedure: k29002 
o|inlining procedure: k29134 
o|inlining procedure: k29134 
o|substituted constant variable: a29195 
o|substituted constant variable: a29197 
o|substituted constant variable: a29205 
o|substituted constant variable: a29210 
o|substituted constant variable: a29212 
o|substituted constant variable: a29226 
o|inlining procedure: k29219 
o|inlining procedure: k29219 
o|substituted constant variable: a29239 
o|inlining procedure: k28962 
o|substituted constant variable: a29254 
o|substituted constant variable: a29256 
o|inlining procedure: k29270 
o|inlining procedure: k29270 
o|inlining procedure: k29365 
o|inlining procedure: k29374 
o|inlining procedure: k29374 
o|inlining procedure: k29365 
o|contracted procedure: "(library.scm:5983) get6805" 
o|inlining procedure: k29441 
o|inlining procedure: k29441 
o|inlining procedure: k29455 
o|inlining procedure: k29455 
o|contracted procedure: "(library.scm:6012) fetch-arg6825" 
o|inlining procedure: k29528 
o|inlining procedure: k29548 
o|inlining procedure: k29548 
o|inlining procedure: k29575 
o|inlining procedure: k29575 
o|inlining procedure: k29528 
o|inlining procedure: k29599 
o|inlining procedure: k29599 
o|inlining procedure: k29611 
o|inlining procedure: k29611 
o|inlining procedure: k29640 
o|inlining procedure: k29640 
o|inlining procedure: k29740 
o|inlining procedure: k29740 
o|inlining procedure: k29771 
o|inlining procedure: k29771 
o|inlining procedure: k29793 
o|inlining procedure: k29793 
o|inlining procedure: k29833 
o|inlining procedure: k29833 
o|inlining procedure: k29858 
o|inlining procedure: k29858 
o|inlining procedure: k29880 
o|inlining procedure: k29880 
o|inlining procedure: k29912 
o|inlining procedure: k29912 
o|inlining procedure: k29944 
o|inlining procedure: k29944 
o|inlining procedure: k29976 
o|inlining procedure: k29976 
o|inlining procedure: k30008 
o|inlining procedure: k30008 
o|substituted constant variable: a30056 
o|substituted constant variable: a30058 
o|substituted constant variable: a30060 
o|substituted constant variable: a30062 
o|substituted constant variable: a30064 
o|substituted constant variable: a30066 
o|substituted constant variable: a30068 
o|substituted constant variable: a30070 
o|substituted constant variable: a30072 
o|substituted constant variable: a30074 
o|inlining procedure: k30075 
o|inlining procedure: k30075 
o|inlining procedure: k30130 
o|inlining procedure: k30130 
o|inlining procedure: k30147 
o|inlining procedure: k30171 
o|inlining procedure: k30171 
o|inlining procedure: k30147 
o|inlining procedure: k30214 
o|inlining procedure: k30214 
o|inlining procedure: k30277 
o|inlining procedure: k30277 
o|inlining procedure: k30309 
o|inlining procedure: "(library.scm:6353) pstr7111" 
o|substituted constant variable: a30330 
o|inlining procedure: k30309 
o|inlining procedure: "(library.scm:6355) pstr7111" 
o|substituted constant variable: a30352 
o|inlining procedure: k30353 
o|inlining procedure: "(library.scm:6357) pstr7111" 
o|substituted constant variable: a30374 
o|inlining procedure: k30353 
o|inlining procedure: "(library.scm:6358) pstr7111" 
o|inlining procedure: "(library.scm:6384) pstr7111" 
o|inlining procedure: "(library.scm:6382) pstr7111" 
o|inlining procedure: "(library.scm:6380) pchr7112" 
o|inlining procedure: "(library.scm:6378) pstr7111" 
o|inlining procedure: "(library.scm:6374) pstr7111" 
o|inlining procedure: "(library.scm:6372) pchr7112" 
o|inlining procedure: "(library.scm:6370) pstr7111" 
o|inlining procedure: "(library.scm:6366) pstr7111" 
o|inlining procedure: "(library.scm:6364) pstr7111" 
o|inlining procedure: "(library.scm:6361) pstr7111" 
o|inlining procedure: k30580 
o|inlining procedure: k30602 
o|inlining procedure: k30602 
o|contracted procedure: "(library.scm:6460) get-config7216" 
o|inlining procedure: k30547 
o|inlining procedure: k30547 
o|substituted constant variable: a30560 
o|inlining procedure: k30638 
o|inlining procedure: k30638 
o|inlining procedure: k30580 
o|propagated global variable: r3058134151 ##sys#build-version 
o|inlining procedure: k30669 
o|inlining procedure: k30669 
o|inlining procedure: k30685 
o|inlining procedure: k30685 
o|inlining procedure: k30713 
o|inlining procedure: k30713 
o|removed side-effect free assignment to unused variable: prefix7294 
o|inlining procedure: k30796 
o|inlining procedure: k30796 
o|inlining procedure: k30811 
o|inlining procedure: k30811 
o|contracted procedure: "(library.scm:6538) err7293" 
o|inlining procedure: k30830 
o|inlining procedure: k30830 
o|inlining procedure: k30909 
o|contracted procedure: "(library.scm:6581) g73257332" 
o|inlining procedure: k30893 
o|inlining procedure: k30893 
o|inlining procedure: k30909 
o|inlining procedure: k30945 
o|inlining procedure: k30945 
o|inlining procedure: k30976 
o|inlining procedure: k30976 
o|inlining procedure: k31022 
o|inlining procedure: k31022 
o|inlining procedure: k31053 
o|inlining procedure: k31053 
o|inlining procedure: k31065 
o|inlining procedure: k31065 
o|inlining procedure: k31077 
o|inlining procedure: k31077 
o|inlining procedure: k31086 
o|inlining procedure: k31086 
o|inlining procedure: k31095 
o|inlining procedure: k31095 
o|substituted constant variable: a31135 
o|inlining procedure: k31177 
o|inlining procedure: k31177 
o|inlining procedure: k31199 
o|inlining procedure: k31216 
o|contracted procedure: "(library.scm:6516) g72677274" 
o|inlining procedure: k31216 
o|inlining procedure: k31199 
o|inlining procedure: k31242 
o|inlining procedure: k31242 
o|inlining procedure: k31316 
o|inlining procedure: k31316 
o|inlining procedure: k31363 
o|inlining procedure: k31363 
o|inlining procedure: k31438 
o|inlining procedure: k31438 
o|substituted constant variable: a31450 
o|inlining procedure: k31463 
o|inlining procedure: k31505 
o|inlining procedure: k31505 
o|inlining procedure: k31535 
o|contracted procedure: "(library.scm:5151) g58695876" 
o|inlining procedure: k31535 
o|inlining procedure: k31463 
o|inlining procedure: k31590 
o|inlining procedure: k31590 
o|inlining procedure: k31641 
o|inlining procedure: k31641 
o|inlining procedure: k31668 
o|inlining procedure: k31668 
o|inlining procedure: k31693 
o|inlining procedure: k31693 
o|inlining procedure: k31773 
o|inlining procedure: k31773 
o|inlining procedure: k31799 
o|inlining procedure: k31799 
o|inlining procedure: k31880 
o|inlining procedure: k31880 
o|inlining procedure: k31899 
o|inlining procedure: k31899 
o|inlining procedure: k32044 
o|inlining procedure: k32044 
o|inlining procedure: k32053 
o|inlining procedure: k32053 
o|inlining procedure: k32062 
o|inlining procedure: k32062 
o|inlining procedure: k32078 
o|inlining procedure: k32078 
o|inlining procedure: k32104 
o|inlining procedure: k32133 
o|inlining procedure: k32133 
o|inlining procedure: k32104 
o|inlining procedure: k32168 
o|inlining procedure: k32168 
o|inlining procedure: k32208 
o|inlining procedure: k32208 
o|inlining procedure: k32270 
o|inlining procedure: k32270 
o|inlining procedure: k32282 
o|inlining procedure: k32282 
o|inlining procedure: k32325 
o|inlining procedure: k32325 
o|inlining procedure: k32383 
o|inlining procedure: k32383 
o|inlining procedure: k32398 
o|inlining procedure: k32398 
o|inlining procedure: k32434 
o|inlining procedure: k32434 
o|substituted constant variable: sblen3854 
o|substituted constant variable: sblen3854 
o|replaced variables: 6428 
o|removed binding forms: 1888 
o|substituted constant variable: r913132537 
o|substituted constant variable: r913732539 
o|inlining procedure: k9456 
o|substituted constant variable: r952032553 
o|substituted constant variable: r967332564 
o|substituted constant variable: r968532567 
o|substituted constant variable: r978032574 
o|substituted constant variable: r984132579 
o|substituted constant variable: r991432582 
o|substituted constant variable: r996832586 
o|substituted constant variable: r1008432595 
o|substituted constant variable: r1027632604 
o|substituted constant variable: r1035332609 
o|substituted constant variable: r1082332652 
o|substituted constant variable: r1082332652 
o|substituted constant variable: r1084032658 
o|substituted constant variable: r1084032658 
o|substituted constant variable: r1086332663 
o|substituted constant variable: r1086332663 
o|substituted constant variable: r1088132670 
o|substituted constant variable: r1088132670 
o|substituted constant variable: r1090532676 
o|substituted constant variable: r1090532676 
o|substituted constant variable: r1093132682 
o|substituted constant variable: r1093132682 
o|substituted constant variable: r1103432693 
o|substituted constant variable: r1156532717 
o|substituted constant variable: r1192832738 
o|substituted constant variable: loc1761 
o|substituted constant variable: loc1772 
o|substituted constant variable: loc1783 
o|substituted constant variable: loc1794 
o|substituted constant variable: loc1805 
o|substituted constant variable: loc1816 
o|substituted constant variable: loc1827 
o|substituted constant variable: loc1838 
o|substituted constant variable: loc1849 
o|substituted constant variable: loc1860 
o|substituted constant variable: loc1871 
o|substituted constant variable: loc1880 
o|substituted constant variable: loc1890 
o|substituted constant variable: loc1901 
o|substituted constant variable: loc1910 
o|substituted constant variable: loc1918 
o|substituted constant variable: loc1926 
o|substituted constant variable: loc1934 
o|substituted constant variable: loc1942 
o|substituted constant variable: loc1950 
o|substituted constant variable: loc1958 
o|substituted constant variable: loc1966 
o|substituted constant variable: loc1974 
o|substituted constant variable: loc1982 
o|substituted constant variable: loc1992 
o|substituted constant variable: loc2001 
o|substituted constant variable: loc2011 
o|substituted constant variable: loc2020 
o|substituted constant variable: loc2028 
o|substituted constant variable: loc2036 
o|substituted constant variable: loc2044 
o|inlining procedure: k12691 
o|inlining procedure: k12691 
o|substituted constant variable: r1270532811 
o|substituted constant variable: r1278632816 
o|substituted constant variable: r1295032830 
o|converted assignments to bindings: (deliver2218) 
o|substituted constant variable: n2416 
o|substituted constant variable: d2417 
o|substituted constant variable: r1422432905 
o|substituted constant variable: r1423332907 
o|substituted constant variable: r1426632911 
o|substituted constant variable: r1427532913 
o|substituted constant variable: r1515232961 
o|substituted constant variable: r1526032968 
o|substituted constant variable: r1532532972 
o|substituted constant variable: r1550032979 
o|substituted constant variable: r1562832981 
o|substituted constant variable: r1562832981 
o|substituted constant variable: r1562832983 
o|substituted constant variable: r1562832983 
o|inlining procedure: k15664 
o|inlining procedure: k15664 
o|substituted constant variable: r1569232990 
o|substituted constant variable: r1570832993 
o|substituted constant variable: r1570832994 
o|substituted constant variable: r1567132999 
o|removed side-effect free assignment to unused variable: make-nan2854 
o|substituted constant variable: r1589833011 
o|substituted constant variable: r1591833013 
o|substituted constant variable: r1594033015 
o|substituted constant variable: r1595133017 
o|substituted constant variable: r1598833020 
o|substituted constant variable: r1601133021 
o|substituted constant variable: r1601133022 
o|substituted constant variable: r1596533023 
o|substituted constant variable: r1607433036 
o|substituted constant variable: r1606233037 
o|substituted constant variable: r1602333053 
o|substituted constant variable: r1634233063 
o|substituted constant variable: r1633233064 
o|substituted constant variable: r1632033065 
o|substituted constant variable: r1640633073 
o|substituted constant variable: r1638833074 
o|substituted constant variable: r1637833075 
o|substituted constant variable: r1656233086 
o|substituted constant variable: r1664333090 
o|substituted constant variable: r1664333090 
o|substituted constant variable: r1664333092 
o|substituted constant variable: r1664333092 
o|substituted constant variable: r1666633097 
o|substituted constant variable: r1666633097 
o|substituted constant variable: r1675333105 
o|substituted constant variable: r1675333105 
o|substituted constant variable: r1673033108 
o|substituted constant variable: r1680933109 
o|substituted constant variable: r1680933110 
o|substituted constant variable: r1658333111 
o|substituted constant variable: r1684733116 
o|substituted constant variable: r1682533117 
o|substituted constant variable: r1687033121 
o|substituted constant variable: r1694533125 
o|substituted constant variable: r1693033126 
o|substituted constant variable: r1699233129 
o|substituted constant variable: r1711233132 
o|substituted constant variable: r1715633136 
o|substituted constant variable: r1720033140 
o|substituted constant variable: r1721333142 
o|substituted constant variable: r1731033149 
o|substituted constant variable: r1738333153 
o|substituted constant variable: r1746933158 
o|substituted constant variable: r1748133160 
o|substituted constant variable: r1749033163 
o|substituted constant variable: r1750833166 
o|substituted constant variable: r1779233189 
o|substituted constant variable: r1784033191 
o|substituted constant variable: r1794433200 
o|substituted constant variable: r1827733211 
o|substituted constant variable: r1840833217 
o|removed call to pure procedure with unused result: "(library.scm:3041) ##sys#slot" 
o|substituted constant variable: r1875133237 
o|removed call to pure procedure with unused result: "(library.scm:3052) ##sys#slot" 
o|converted assignments to bindings: (continuation3764) 
o|substituted constant variable: r1891733251 
o|removed unused formal parameters: (%convert?38963911) 
o|removed unused parameter to known procedure: %convert?38963911 "(library.scm:3311) def-set?39013910" 
o|removed unused parameter to known procedure: %convert?38963911 "(library.scm:3311) def-set?39013910" 
o|removed unused formal parameters: (%convert?39213936) 
o|removed unused parameter to known procedure: %convert?39213936 "(library.scm:3321) def-set?39263935" 
o|removed unused parameter to known procedure: %convert?39213936 "(library.scm:3321) def-set?39263935" 
o|removed unused formal parameters: (%convert?39463961) 
o|removed unused parameter to known procedure: %convert?39463961 "(library.scm:3331) def-set?39513960" 
o|removed unused parameter to known procedure: %convert?39463961 "(library.scm:3331) def-set?39513960" 
o|substituted constant variable: r1922333279 
o|substituted constant variable: r1928633284 
o|substituted constant variable: r1928633284 
o|substituted constant variable: r1928633286 
o|substituted constant variable: r1928633286 
o|substituted constant variable: r1952833296 
o|substituted constant variable: r1958133299 
o|substituted constant variable: r1967533305 
o|substituted constant variable: r1971333307 
o|substituted constant variable: r1974733309 
o|substituted constant variable: r1980733313 
o|inlining procedure: k20130 
o|substituted constant variable: r2019733328 
o|inlining procedure: k20208 
o|substituted constant variable: r2103633379 
o|substituted constant variable: tok477733407 
o|inlining procedure: k21367 
o|converted assignments to bindings: (starting-line4540) 
o|substituted constant variable: r2146733449 
o|substituted constant variable: r2146733449 
o|substituted constant variable: r2180533485 
o|substituted constant variable: r2182433489 
o|substituted constant variable: r2197233503 
o|substituted constant variable: r2197233504 
o|inlining procedure: k22065 
o|inlining procedure: k22065 
o|inlining procedure: k22065 
o|substituted constant variable: r2206333512 
o|substituted constant variable: r2204533513 
o|removed side-effect free assignment to unused variable: build-symbol4434 
o|removed side-effect free assignment to unused variable: build-keyword4435 
o|substituted constant variable: r2251733547 
o|substituted constant variable: r2267933558 
o|substituted constant variable: r2291333579 
o|substituted constant variable: r2317833605 
o|substituted constant variable: r2319733606 
o|substituted constant variable: r2319733606 
o|substituted constant variable: r2319733608 
o|substituted constant variable: r2319733608 
o|substituted constant variable: r2337733615 
o|substituted constant variable: r2340133618 
o|substituted constant variable: r2338933619 
o|converted assignments to bindings: (hex4968) 
o|substituted constant variable: r2359933630 
o|propagated global variable: port503633633 ##sys#standard-output 
o|substituted constant variable: r2412133656 
o|substituted constant variable: r2413333658 
o|substituted constant variable: r2414833660 
o|substituted constant variable: r2414833661 
o|substituted constant variable: r2418933663 
o|substituted constant variable: r2420433665 
o|substituted constant variable: r2420433666 
o|inlining procedure: k24216 
o|substituted constant variable: r2421733669 
o|substituted constant variable: r2421733669 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r2425933675 
o|inlining procedure: k24715 
o|substituted constant variable: r2496233727 
o|substituted constant variable: r2498233729 
o|inlining procedure: k25126 
o|substituted constant variable: r2549033753 
o|converted assignments to bindings: (copy&append5555) 
o|substituted constant variable: r2570633764 
o|converted assignments to bindings: (break5724) 
o|substituted constant variable: r2624633796 
o|substituted constant variable: r2624633796 
o|substituted constant variable: r2624633798 
o|substituted constant variable: r2624633798 
o|substituted constant variable: r2628933800 
o|substituted constant variable: r2630133802 
o|substituted constant variable: r2631333804 
o|substituted constant variable: r2632533806 
o|substituted constant variable: r2633733808 
o|substituted constant variable: r2634933810 
o|substituted constant variable: r2636133812 
o|substituted constant variable: r2636133813 
o|substituted constant variable: r2652433817 
o|substituted constant variable: r2652433817 
o|inlining procedure: k26549 
o|inlining procedure: k26549 
o|removed unused formal parameters: (%convert?59615976) 
o|removed unused parameter to known procedure: %convert?59615976 "(library.scm:5231) def-set?59665975" 
o|removed unused parameter to known procedure: %convert?59615976 "(library.scm:5231) def-set?59665975" 
o|substituted constant variable: r2670833834 
o|substituted constant variable: r2705733846 
o|substituted constant variable: r2715533851 
o|substituted constant variable: r2718533855 
o|substituted constant variable: r2754633881 
o|substituted constant variable: r2754633881 
o|substituted constant variable: r2758333886 
o|substituted constant variable: r2758333886 
o|substituted constant variable: r2809733944 
o|substituted constant variable: r2809733944 
o|substituted constant variable: r2834333949 
o|substituted constant variable: r2836533951 
o|substituted constant variable: r2842333952 
o|substituted constant variable: r2847033956 
o|substituted constant variable: r2847033956 
o|substituted constant variable: r2848733958 
o|inlining procedure: k28889 
o|converted assignments to bindings: (get/clear-str6693) 
o|substituted constant variable: r2936633995 
o|inlining procedure: k29528 
o|inlining procedure: k29528 
o|substituted constant variable: r3014834059 
o|removed side-effect free assignment to unused variable: pstr7111 
o|removed side-effect free assignment to unused variable: pchr7112 
o|substituted constant variable: r3027834062 
o|substituted constant variable: r3027834062 
o|substituted constant variable: str711634068 
o|substituted constant variable: str711634074 
o|substituted constant variable: str711634080 
o|substituted constant variable: str711634086 
o|substituted constant variable: str711634091 
o|substituted constant variable: str711634096 
o|substituted constant variable: chr711734101 
o|substituted constant variable: str711634106 
o|substituted constant variable: str711634111 
o|substituted constant variable: chr711734116 
o|substituted constant variable: str711634121 
o|substituted constant variable: str711634126 
o|substituted constant variable: str711634131 
o|substituted constant variable: str711634136 
o|substituted constant variable: r3060334141 
o|substituted constant variable: r3060334141 
o|substituted constant variable: r3054834145 
o|converted assignments to bindings: (str7221) 
o|substituted constant variable: r3063934147 
o|substituted constant variable: r3063934147 
o|substituted constant variable: r3063934149 
o|substituted constant variable: r3063934149 
o|substituted constant variable: r3068634154 
o|substituted constant variable: r3094634169 
o|substituted constant variable: r3106634178 
o|substituted constant variable: r3109634184 
o|inlining procedure: k31177 
o|substituted constant variable: r3120034192 
o|substituted constant variable: r3136434197 
o|inlining procedure: k31484 
o|inlining procedure: k31484 
o|substituted constant variable: r3166934211 
o|substituted constant variable: r3169434213 
o|substituted constant variable: r3180034219 
o|substituted constant variable: r3188134221 
o|substituted constant variable: r3205434230 
o|substituted constant variable: r3206334232 
o|substituted constant variable: r3207934234 
o|substituted constant variable: r3213434237 
o|converted assignments to bindings: (lookup-char3616) 
o|simplifications: ((let . 10)) 
o|replaced variables: 641 
o|removed binding forms: 5387 
o|removed conditional forms: 1 
o|inlining procedure: k9438 
o|substituted constant variable: r945734267 
o|substituted constant variable: r945734267 
o|inlining procedure: k10604 
o|inlining procedure: k10604 
o|inlining procedure: k10604 
o|inlining procedure: k11079 
o|inlining procedure: k11095 
o|inlining procedure: k11111 
o|inlining procedure: k11127 
o|inlining procedure: k11143 
o|inlining procedure: k11159 
o|inlining procedure: k11175 
o|inlining procedure: k11191 
o|inlining procedure: k11316 
o|inlining procedure: k11351 
o|inlining procedure: k11351 
o|inlining procedure: k11606 
o|inlining procedure: k11606 
o|inlining procedure: k11716 
o|inlining procedure: k11716 
o|inlining procedure: k12025 
o|inlining procedure: k12049 
o|inlining procedure: k12073 
o|inlining procedure: k12097 
o|inlining procedure: k12121 
o|inlining procedure: k12145 
o|inlining procedure: k12169 
o|inlining procedure: k12193 
o|inlining procedure: k12217 
o|inlining procedure: k12241 
o|inlining procedure: k12265 
o|inlining procedure: k12283 
o|inlining procedure: k12307 
o|inlining procedure: k12331 
o|inlining procedure: k12349 
o|inlining procedure: k12367 
o|inlining procedure: k12385 
o|inlining procedure: k12403 
o|inlining procedure: k12421 
o|inlining procedure: k12439 
o|inlining procedure: k12457 
o|inlining procedure: k12475 
o|inlining procedure: k12493 
o|inlining procedure: k12511 
o|inlining procedure: k12535 
o|inlining procedure: k12553 
o|inlining procedure: k12577 
o|inlining procedure: k12595 
o|inlining procedure: k12613 
o|inlining procedure: k12631 
o|inlining procedure: k12649 
o|inlining procedure: k12892 
o|inlining procedure: k13004 
o|inlining procedure: k13073 
o|inlining procedure: k13165 
o|inlining procedure: k13171 
o|inlining procedure: k13274 
o|inlining procedure: k13497 
o|inlining procedure: k13567 
o|inlining procedure: k13612 
o|inlining procedure: k13649 
o|inlining procedure: k13680 
o|inlining procedure: k13680 
o|inlining procedure: k13693 
o|inlining procedure: k13861 
o|inlining procedure: k14002 
o|inlining procedure: k14644 
o|inlining procedure: k14668 
o|inlining procedure: k14868 
o|inlining procedure: k14991 
o|inlining procedure: k14999 
o|inlining procedure: k15037 
o|inlining procedure: k15055 
o|inlining procedure: k15208 
o|inlining procedure: k15237 
o|inlining procedure: k15455 
o|inlining procedure: k15462 
o|inlining procedure: k15927 
o|inlining procedure: k16435 
o|inlining procedure: k16573 
o|inlining procedure: k16839 
o|inlining procedure: k16839 
o|inlining procedure: k16881 
o|inlining procedure: k16895 
o|inlining procedure: k16901 
o|inlining procedure: k16960 
o|inlining procedure: k16968 
o|inlining procedure: k17296 
o|inlining procedure: k17303 
o|inlining procedure: k17303 
o|inlining procedure: k17336 
o|inlining procedure: k17619 
o|inlining procedure: k17619 
o|contracted procedure: k18648 
o|contracted procedure: k18744 
o|inlining procedure: k19300 
o|inlining procedure: k19300 
o|inlining procedure: k19300 
o|inlining procedure: k19562 
o|propagated global variable: r1956335082 ##sys#standard-output 
o|propagated global variable: r1956335082 ##sys#standard-output 
o|inlining procedure: k20109 
o|propagated global variable: r2011035090 ##sys#standard-input 
o|propagated global variable: r2011035090 ##sys#standard-input 
o|inlining procedure: k20221 
o|propagated global variable: r2022235092 ##sys#standard-input 
o|propagated global variable: r2022235092 ##sys#standard-input 
o|inlining procedure: k20239 
o|propagated global variable: r2024035094 ##sys#standard-input 
o|propagated global variable: r2024035094 ##sys#standard-input 
o|inlining procedure: k20952 
o|inlining procedure: k23707 
o|propagated global variable: r2370835121 ##sys#standard-output 
o|propagated global variable: r2370835121 ##sys#standard-output 
o|inlining procedure: k23725 
o|propagated global variable: r2372635123 ##sys#standard-output 
o|propagated global variable: r2372635123 ##sys#standard-output 
o|inlining procedure: k24103 
o|substituted constant variable: r2421734615 
o|folded constant expression: (scheme#not (quote #f)) 
o|inlining procedure: k26403 
o|inlining procedure: k28480 
o|inlining procedure: k28480 
o|inlining procedure: k28704 
o|inlining procedure: "(library.scm:5810) ##sys#format-here-doc-warning" 
o|inlining procedure: k28939 
o|inlining procedure: k28939 
o|inlining procedure: "(library.scm:5844) ##sys#format-here-doc-warning" 
o|inlining procedure: k29227 
o|inlining procedure: k29227 
o|inlining procedure: k29790 
o|inlining procedure: k29790 
o|inlining procedure: k30159 
o|inlining procedure: k30159 
o|inlining procedure: k30666 
o|inlining procedure: k30915 
o|inlining procedure: k30915 
o|inlining procedure: k32041 
o|inlining procedure: k32041 
o|replaced variables: 8 
o|removed binding forms: 843 
o|substituted constant variable: r943934821 
o|substituted constant variable: r1108034862 
o|substituted constant variable: r1109634864 
o|substituted constant variable: r1111234866 
o|substituted constant variable: r1112834868 
o|substituted constant variable: r1114434870 
o|substituted constant variable: r1116034872 
o|substituted constant variable: r1117634874 
o|substituted constant variable: r1119234876 
o|substituted constant variable: r1131734884 
o|substituted constant variable: r1135234888 
o|contracted procedure: k12821 
o|contracted procedure: k12826 
o|contracted procedure: k12886 
o|contracted procedure: k13259 
o|contracted procedure: k13268 
o|contracted procedure: k13373 
o|contracted procedure: k13378 
o|contracted procedure: k13383 
o|contracted procedure: k13388 
o|contracted procedure: k13393 
o|contracted procedure: k13398 
o|contracted procedure: k13430 
o|contracted procedure: k13454 
o|contracted procedure: k13469 
o|contracted procedure: k13549 
o|contracted procedure: k13594 
o|contracted procedure: k13643 
o|substituted constant variable: r1368134972 
o|contracted procedure: k13985 
o|contracted procedure: k15474 
o|contracted procedure: k15479 
o|contracted procedure: k15611 
o|contracted procedure: k15773 
o|contracted procedure: k15881 
o|contracted procedure: k15972 
o|contracted procedure: k16590 
o|contracted procedure: k16607 
o|contracted procedure: k16748 
o|substituted constant variable: r1684035014 
o|contracted procedure: k17099 
o|substituted constant variable: r1730435025 
o|substituted constant variable: r1730435026 
o|substituted constant variable: r1762035030 
o|inlining procedure: k21322 
o|inlining procedure: k22827 
o|substituted constant variable: r2655034669 
o|removed side-effect free assignment to unused variable: ##sys#format-here-doc-warning 
o|substituted constant variable: r2979135226 
o|substituted constant variable: r2979135226 
o|substituted constant variable: r3016035227 
o|substituted constant variable: r3016035227 
o|substituted constant variable: r3016035228 
o|substituted constant variable: r3016035228 
o|substituted constant variable: r3066735233 
o|substituted constant variable: r3066735233 
o|substituted constant variable: r3204235261 
o|substituted constant variable: r3204235261 
o|substituted constant variable: r3204235263 
o|substituted constant variable: r3204235263 
o|simplifications: ((let . 4)) 
o|replaced variables: 19 
o|removed binding forms: 143 
o|removed conditional forms: 10 
o|substituted constant variable: r2132335409 
o|substituted constant variable: r2132335409 
o|substituted constant variable: r2282835420 
o|removed binding forms: 75 
o|removed conditional forms: 2 
o|simplifications: ((let . 1)) 
o|removed binding forms: 2 
o|simplifications: ((if . 267) (let . 198) (##core#call . 2706)) 
o|  call simplifications:
o|    scheme#cdar	2
o|    scheme#set-cdr!	6
o|    scheme#list-tail
o|    scheme#set-car!	7
o|    ##sys#make-vector
o|    scheme#vector?
o|    ##sys#generic-structure?
o|    chicken.fixnum#fx/
o|    ##sys#foreign-string-argument
o|    scheme#read-char
o|    scheme#string-length	3
o|    scheme#string-ref	3
o|    ##sys#list	6
o|    ##sys#context-switch	2
o|    ##sys#foreign-pointer-argument	3
o|    ##sys#null-pointer?	2
o|    scheme#caddr	3
o|    scheme#write-char
o|    scheme#equal?	2
o|    scheme#memv	2
o|    scheme#caar	3
o|    scheme#cddr	3
o|    ##sys#check-structure	2
o|    scheme#string?	5
o|    scheme#symbol?	7
o|    ##sys#apply	2
o|    ##sys#byte
o|    scheme#char-upper-case?
o|    scheme#char<?
o|    ##sys#void	4
o|    scheme#string	4
o|    scheme#member	8
o|    chicken.fixnum#fxior	13
o|    scheme#string=?	6
o|    scheme#char-numeric?	7
o|    scheme#char<=?
o|    chicken.fixnum#fx*	9
o|    scheme#eof-object?	25
o|    scheme#char=?	12
o|    scheme#char-whitespace?	7
o|    scheme#memq	12
o|    ##sys#immediate?	3
o|    chicken.fixnum#fxnot
o|    chicken.fixnum#fxand	16
o|    ##sys#call-with-current-continuation	3
o|    scheme#char?	4
o|    scheme#assq	6
o|    ##sys#setislot	28
o|    scheme#char->integer	16
o|    scheme#char-downcase	14
o|    chicken.fixnum#fxmin	3
o|    ##sys#check-vector	6
o|    ##sys#check-symbol	12
o|    ##sys#intern-symbol	10
o|    scheme#integer->char	22
o|    scheme#string-ci=?	2
o|    scheme#char>=?	3
o|    ##sys#foreign-flonum-argument
o|    ##sys#foreign-fixnum-argument	4
o|    scheme#remainder
o|    chicken.base#fixnum?	2
o|    chicken.fixnum#fxshr	13
o|    chicken.fixnum#fxshl	8
o|    chicken.bitwise#bitwise-and	2
o|    chicken.fixnum#fxneg	5
o|    chicken.bitwise#arithmetic-shift	11
o|    scheme#>	7
o|    scheme#<	7
o|    scheme#cadr	15
o|    scheme#=	3
o|    scheme#>=	2
o|    scheme#quotient	17
o|    scheme#exact?
o|    chicken.base#ratnum?	14
o|    ##sys#check-number	10
o|    chicken.base#cplxnum?	17
o|    scheme#number?	8
o|    scheme#*	44
o|    scheme#inexact?	7
o|    chicken.base#flonum?	46
o|    scheme#list->string
o|    chicken.fixnum#fx>	36
o|    scheme#length	6
o|    chicken.fixnum#fx>=	34
o|    chicken.fixnum#fx+	126
o|    ##sys#check-string	25
o|    chicken.fixnum#fx-	54
o|    ##sys#call-with-values	22
o|    scheme#procedure?	6
o|    ##sys#values	2
o|    chicken.fixnum#fx<=	26
o|    chicken.fixnum#fx<	38
o|    scheme#vector	7
o|    chicken.fixnum#fx=	20
o|    ##sys#size	66
o|    scheme#list	26
o|    ##sys#setslot	46
o|    scheme#list?	5
o|    ##sys#check-pair	3
o|    chicken.base#void	3
o|    scheme#null?	128
o|    ##sys#check-list	19
o|    scheme#not	89
o|    ##sys#make-structure	11
o|    ##sys#structure?	8
o|    scheme#apply	98
o|    scheme#-	19
o|    scheme#call-with-values	2
o|    scheme#+	24
o|    scheme#values	23
o|    ##sys#check-char	15
o|    scheme#pair?	91
o|    ##sys#slot	344
o|    scheme#cons	142
o|    scheme#eq?	397
o|    scheme#cdr	47
o|    scheme#car	121
o|contracted procedure: k8637 
o|contracted procedure: k8633 
o|contracted procedure: k8651 
o|contracted procedure: k8665 
o|contracted procedure: k8661 
o|contracted procedure: k8686 
o|contracted procedure: k8682 
o|contracted procedure: k8700 
o|contracted procedure: k8696 
o|contracted procedure: k8718 
o|contracted procedure: k8714 
o|contracted procedure: k8710 
o|contracted procedure: k8728 
o|contracted procedure: k8740 
o|contracted procedure: k8758 
o|contracted procedure: k8754 
o|contracted procedure: k8769 
o|contracted procedure: k8787 
o|contracted procedure: k8783 
o|contracted procedure: k8779 
o|contracted procedure: k8797 
o|contracted procedure: k8809 
o|contracted procedure: k8834 
o|contracted procedure: k8830 
o|contracted procedure: k8826 
o|contracted procedure: k8844 
o|contracted procedure: k8863 
o|contracted procedure: k8859 
o|contracted procedure: k8855 
o|contracted procedure: k8896 
o|contracted procedure: k8902 
o|contracted procedure: k8909 
o|contracted procedure: k8917 
o|contracted procedure: k8913 
o|contracted procedure: k9047 
o|contracted procedure: k9076 
o|contracted procedure: k9082 
o|contracted procedure: k9093 
o|contracted procedure: k9099 
o|contracted procedure: k9105 
o|contracted procedure: k9111 
o|contracted procedure: k9117 
o|contracted procedure: k9239 
o|contracted procedure: k9253 
o|contracted procedure: k9292 
o|contracted procedure: k9354 
o|contracted procedure: k9385 
o|contracted procedure: k9366 
o|contracted procedure: k9373 
o|contracted procedure: k9381 
o|contracted procedure: k9391 
o|contracted procedure: k9422 
o|contracted procedure: k9403 
o|contracted procedure: k9410 
o|contracted procedure: k9418 
o|contracted procedure: k9432 
o|contracted procedure: k9447 
o|contracted procedure: k9438 
o|contracted procedure: k9462 
o|contracted procedure: k9456 
o|contracted procedure: k9486 
o|contracted procedure: k9495 
o|contracted procedure: k9505 
o|contracted procedure: k9542 
o|contracted procedure: k9522 
o|contracted procedure: k9525 
o|contracted procedure: k9538 
o|contracted procedure: k9551 
o|contracted procedure: k9554 
o|contracted procedure: k9595 
o|contracted procedure: k9616 
o|contracted procedure: k9628 
o|contracted procedure: k9637 
o|contracted procedure: k9640 
o|contracted procedure: k9646 
o|contracted procedure: k9675 
o|contracted procedure: k9702 
o|contracted procedure: k9681 
o|contracted procedure: k9694 
o|contracted procedure: k9698 
o|contracted procedure: k9711 
o|contracted procedure: k9758 
o|contracted procedure: k9722 
o|contracted procedure: k9736 
o|contracted procedure: k9739 
o|contracted procedure: k9745 
o|contracted procedure: k9767 
o|contracted procedure: k9789 
o|contracted procedure: k9804 
o|contracted procedure: k9814 
o|contracted procedure: k9818 
o|contracted procedure: k9828 
o|contracted procedure: k9837 
o|contracted procedure: k9843 
o|contracted procedure: k9850 
o|contracted procedure: k9869 
o|contracted procedure: k9872 
o|contracted procedure: k9875 
o|contracted procedure: k9881 
o|contracted procedure: k9951 
o|contracted procedure: k9901 
o|contracted procedure: k9904 
o|contracted procedure: k9916 
o|contracted procedure: k9948 
o|contracted procedure: k9922 
o|contracted procedure: k9928 
o|contracted procedure: k9931 
o|contracted procedure: k9937 
o|contracted procedure: k9958 
o|contracted procedure: k9970 
o|contracted procedure: k10032 
o|contracted procedure: k9976 
o|contracted procedure: k10028 
o|contracted procedure: k9985 
o|contracted procedure: k9994 
o|contracted procedure: k10001 
o|contracted procedure: k10009 
o|contracted procedure: k10013 
o|contracted procedure: k10020 
o|contracted procedure: k10024 
o|contracted procedure: k10115 
o|contracted procedure: k10038 
o|contracted procedure: k10050 
o|contracted procedure: k10057 
o|contracted procedure: k10060 
o|contracted procedure: k10066 
o|contracted procedure: k10072 
o|contracted procedure: k10086 
o|contracted procedure: k10089 
o|contracted procedure: k10102 
o|contracted procedure: k10105 
o|contracted procedure: k10112 
o|contracted procedure: k10219 
o|contracted procedure: k10124 
o|contracted procedure: k10136 
o|contracted procedure: k10143 
o|contracted procedure: k10216 
o|contracted procedure: k10149 
o|contracted procedure: k10155 
o|contracted procedure: k10212 
o|contracted procedure: k10161 
o|contracted procedure: k10177 
o|contracted procedure: k10181 
o|contracted procedure: k10200 
o|contracted procedure: k10204 
o|contracted procedure: k10188 
o|contracted procedure: k10196 
o|contracted procedure: k10208 
o|contracted procedure: k10331 
o|contracted procedure: k10228 
o|contracted procedure: k10325 
o|contracted procedure: k10231 
o|contracted procedure: k10319 
o|contracted procedure: k10234 
o|contracted procedure: k10313 
o|contracted procedure: k10237 
o|contracted procedure: k10252 
o|contracted procedure: k10258 
o|contracted procedure: k10264 
o|contracted procedure: k10278 
o|contracted procedure: k10284 
o|contracted procedure: k10287 
o|contracted procedure: k10290 
o|contracted procedure: k10303 
o|contracted procedure: k10307 
o|contracted procedure: k10340 
o|contracted procedure: k10381 
o|contracted procedure: k10343 
o|contracted procedure: k10355 
o|contracted procedure: k10358 
o|contracted procedure: k10361 
o|contracted procedure: k10374 
o|contracted procedure: k10378 
o|contracted procedure: k10500 
o|contracted procedure: k10512 
o|contracted procedure: k10551 
o|contracted procedure: k10547 
o|contracted procedure: k10598 
o|contracted procedure: k10601 
o|contracted procedure: k10610 
o|contracted procedure: k10632 
o|contracted procedure: k10639 
o|contracted procedure: k10643 
o|contracted procedure: k10652 
o|contracted procedure: k10663 
o|contracted procedure: k10675 
o|contracted procedure: k10686 
o|contracted procedure: k10697 
o|contracted procedure: k10708 
o|contracted procedure: k10719 
o|contracted procedure: k10730 
o|contracted procedure: k10741 
o|contracted procedure: k10752 
o|contracted procedure: k10763 
o|contracted procedure: k10774 
o|contracted procedure: k10785 
o|contracted procedure: k10796 
o|contracted procedure: k10807 
o|contracted procedure: k10825 
o|contracted procedure: k10842 
o|contracted procedure: k10852 
o|contracted procedure: k10855 
o|contracted procedure: k10865 
o|contracted procedure: k10883 
o|contracted procedure: k10911 
o|contracted procedure: k10896 
o|contracted procedure: k10907 
o|contracted procedure: k10923 
o|contracted procedure: k10933 
o|contracted procedure: k10943 
o|contracted procedure: k10954 
o|contracted procedure: k10963 
o|contracted procedure: k10969 
o|contracted procedure: k10978 
o|contracted procedure: k10987 
o|contracted procedure: k10990 
o|contracted procedure: k11059 
o|contracted procedure: k11055 
o|contracted procedure: k10999 
o|contracted procedure: k11011 
o|contracted procedure: k11021 
o|contracted procedure: k11014 
o|contracted procedure: k11024 
o|contracted procedure: k11051 
o|contracted procedure: k11030 
o|contracted procedure: k11047 
o|contracted procedure: k11036 
o|contracted procedure: k11043 
o|contracted procedure: k11068 
o|contracted procedure: k11086 
o|contracted procedure: k11079 
o|contracted procedure: k11102 
o|contracted procedure: k11095 
o|contracted procedure: k11118 
o|contracted procedure: k11111 
o|contracted procedure: k11134 
o|contracted procedure: k11127 
o|contracted procedure: k11150 
o|contracted procedure: k11143 
o|contracted procedure: k11166 
o|contracted procedure: k11159 
o|contracted procedure: k11182 
o|contracted procedure: k11175 
o|contracted procedure: k11198 
o|contracted procedure: k11191 
o|contracted procedure: k11210 
o|contracted procedure: k11280 
o|contracted procedure: k11222 
o|contracted procedure: k11232 
o|contracted procedure: k11241 
o|contracted procedure: k11248 
o|contracted procedure: k11254 
o|contracted procedure: k11261 
o|contracted procedure: k11269 
o|contracted procedure: k11276 
o|contracted procedure: k11295 
o|contracted procedure: k11302 
o|contracted procedure: k11310 
o|contracted procedure: k11306 
o|contracted procedure: k11327 
o|contracted procedure: k11316 
o|contracted procedure: k11354 
o|contracted procedure: k11357 
o|contracted procedure: k11360 
o|contracted procedure: k11363 
o|contracted procedure: k11372 
o|contracted procedure: k11375 
o|contracted procedure: k11382 
o|contracted procedure: k11391 
o|contracted procedure: k11398 
o|contracted procedure: k11402 
o|contracted procedure: k11455 
o|contracted procedure: k11413 
o|contracted procedure: k11419 
o|contracted procedure: k11434 
o|contracted procedure: k11437 
o|contracted procedure: k11440 
o|contracted procedure: k11447 
o|contracted procedure: k11451 
o|contracted procedure: k11465 
o|contracted procedure: k11476 
o|contracted procedure: k11485 
o|contracted procedure: k11488 
o|contracted procedure: k11491 
o|contracted procedure: k11498 
o|contracted procedure: k11502 
o|contracted procedure: k11511 
o|contracted procedure: k11514 
o|contracted procedure: k11518 
o|contracted procedure: k11524 
o|contracted procedure: k11527 
o|contracted procedure: k11536 
o|contracted procedure: k11545 
o|contracted procedure: k11561 
o|contracted procedure: k11567 
o|contracted procedure: k11573 
o|contracted procedure: k11590 
o|contracted procedure: k11595 
o|contracted procedure: k11598 
o|contracted procedure: k11609 
o|contracted procedure: k11622 
o|contracted procedure: k11631 
o|contracted procedure: k11646 
o|contracted procedure: k11655 
o|contracted procedure: k11673 
o|contracted procedure: k11694 
o|contracted procedure: k11705 
o|contracted procedure: k11708 
o|contracted procedure: k11719 
o|contracted procedure: k11732 
o|contracted procedure: k11741 
o|contracted procedure: k11756 
o|contracted procedure: k11765 
o|contracted procedure: k11783 
o|contracted procedure: k11804 
o|contracted procedure: k11816 
o|contracted procedure: k11819 
o|contracted procedure: k11826 
o|contracted procedure: k11841 
o|contracted procedure: k11848 
o|contracted procedure: k11851 
o|contracted procedure: k11854 
o|contracted procedure: k11861 
o|contracted procedure: k11865 
o|contracted procedure: k11889 
o|contracted procedure: k11892 
o|contracted procedure: k11895 
o|contracted procedure: k11902 
o|contracted procedure: k11906 
o|contracted procedure: k11918 
o|contracted procedure: k11930 
o|contracted procedure: k11936 
o|contracted procedure: k11951 
o|contracted procedure: k11966 
o|contracted procedure: k11985 
o|contracted procedure: k11973 
o|contracted procedure: k11977 
o|contracted procedure: k11981 
o|contracted procedure: k11988 
o|contracted procedure: k12019 
o|contracted procedure: k12012 
o|contracted procedure: k12043 
o|contracted procedure: k12036 
o|contracted procedure: k12067 
o|contracted procedure: k12060 
o|contracted procedure: k12091 
o|contracted procedure: k12084 
o|contracted procedure: k12115 
o|contracted procedure: k12108 
o|contracted procedure: k12139 
o|contracted procedure: k12132 
o|contracted procedure: k12163 
o|contracted procedure: k12156 
o|contracted procedure: k12187 
o|contracted procedure: k12180 
o|contracted procedure: k12211 
o|contracted procedure: k12204 
o|contracted procedure: k12235 
o|contracted procedure: k12228 
o|contracted procedure: k12259 
o|contracted procedure: k12252 
o|contracted procedure: k12276 
o|contracted procedure: k12301 
o|contracted procedure: k12294 
o|contracted procedure: k12325 
o|contracted procedure: k12318 
o|contracted procedure: k12342 
o|contracted procedure: k12360 
o|contracted procedure: k12378 
o|contracted procedure: k12396 
o|contracted procedure: k12414 
o|contracted procedure: k12432 
o|contracted procedure: k12450 
o|contracted procedure: k12468 
o|contracted procedure: k12486 
o|contracted procedure: k12504 
o|contracted procedure: k12529 
o|contracted procedure: k12522 
o|contracted procedure: k12546 
o|contracted procedure: k12571 
o|contracted procedure: k12564 
o|contracted procedure: k12588 
o|contracted procedure: k12606 
o|contracted procedure: k12624 
o|contracted procedure: k12642 
o|contracted procedure: k12664 
o|contracted procedure: k12655 
o|contracted procedure: k12680 
o|contracted procedure: k12694 
o|contracted procedure: k12698 
o|contracted procedure: k12738 
o|contracted procedure: k12742 
o|contracted procedure: k12751 
o|contracted procedure: k12762 
o|contracted procedure: k12774 
o|contracted procedure: k12788 
o|contracted procedure: k12797 
o|contracted procedure: k12816 
o|contracted procedure: k12837 
o|contracted procedure: k12841 
o|contracted procedure: k12833 
o|contracted procedure: k12847 
o|contracted procedure: k12860 
o|contracted procedure: k12866 
o|contracted procedure: k12913 
o|contracted procedure: k12935 
o|contracted procedure: k12959 
o|contracted procedure: k12981 
o|contracted procedure: k12998 
o|contracted procedure: k13010 
o|contracted procedure: k13122 
o|contracted procedure: k13056 
o|contracted procedure: k13047 
o|contracted procedure: k13090 
o|contracted procedure: k13097 
o|contracted procedure: k13148 
o|contracted procedure: k13200 
o|contracted procedure: k13210 
o|contracted procedure: k13223 
o|contracted procedure: k13230 
o|contracted procedure: k13238 
o|contracted procedure: k13242 
o|contracted procedure: k13254 
o|contracted procedure: k13280 
o|contracted procedure: k13283 
o|contracted procedure: k13335 
o|contracted procedure: k13339 
o|contracted procedure: k13298 
o|contracted procedure: k13315 
o|contracted procedure: k13319 
o|contracted procedure: k13311 
o|contracted procedure: k13327 
o|contracted procedure: k13331 
o|contracted procedure: k13323 
o|contracted procedure: k13345 
o|contracted procedure: k13362 
o|contracted procedure: k13368 
o|contracted procedure: k13421 
o|contracted procedure: k13425 
o|contracted procedure: k13405 
o|contracted procedure: k13413 
o|contracted procedure: k13417 
o|contracted procedure: k13409 
o|contracted procedure: k13436 
o|contracted procedure: k13458 
o|contracted procedure: k13439 
o|contracted procedure: k13464 
o|contracted procedure: k13491 
o|contracted procedure: k13475 
o|contracted procedure: k13478 
o|contracted procedure: k13513 
o|contracted procedure: k13503 
o|contracted procedure: k13521 
o|contracted procedure: k13544 
o|contracted procedure: k13552 
o|contracted procedure: k13558 
o|contracted procedure: k13589 
o|contracted procedure: k13597 
o|contracted procedure: k13603 
o|contracted procedure: k13634 
o|contracted procedure: k13668 
o|contracted procedure: k13671 
o|contracted procedure: k13686 
o|contracted procedure: k13772 
o|contracted procedure: k13714 
o|contracted procedure: k13723 
o|contracted procedure: k13739 
o|contracted procedure: k13743 
o|contracted procedure: k13733 
o|contracted procedure: k13757 
o|contracted procedure: k13761 
o|contracted procedure: k13768 
o|contracted procedure: k13786 
o|contracted procedure: k13832 
o|contracted procedure: k13795 
o|contracted procedure: k13820 
o|contracted procedure: k13824 
o|contracted procedure: k13858 
o|contracted procedure: k13861 
o|contracted procedure: k13842 
o|contracted procedure: k13846 
o|contracted procedure: k13884 
o|contracted procedure: k13890 
o|contracted procedure: k13897 
o|contracted procedure: k13904 
o|contracted procedure: k13915 
o|contracted procedure: k13908 
o|contracted procedure: k13912 
o|contracted procedure: k13928 
o|contracted procedure: k13934 
o|contracted procedure: k13941 
o|contracted procedure: k13948 
o|contracted procedure: k13959 
o|contracted procedure: k13952 
o|contracted procedure: k13956 
o|contracted procedure: k13965 
o|contracted procedure: k13971 
o|contracted procedure: k14013 
o|contracted procedure: k14022 
o|contracted procedure: k14025 
o|contracted procedure: k14036 
o|contracted procedure: k14079 
o|contracted procedure: k14059 
o|contracted procedure: k14088 
o|contracted procedure: k14094 
o|contracted procedure: k14097 
o|contracted procedure: k14104 
o|contracted procedure: k14122 
o|contracted procedure: k14128 
o|contracted procedure: k14131 
o|contracted procedure: k14138 
o|contracted procedure: k14156 
o|contracted procedure: k14162 
o|contracted procedure: k14181 
o|contracted procedure: k14208 
o|contracted procedure: k14204 
o|contracted procedure: k14220 
o|contracted procedure: k14216 
o|contracted procedure: k14226 
o|contracted procedure: k14243 
o|contracted procedure: k14268 
o|contracted procedure: k14340 
o|contracted procedure: k14285 
o|contracted procedure: k14288 
o|contracted procedure: k14294 
o|contracted procedure: k14303 
o|contracted procedure: k14310 
o|contracted procedure: k14322 
o|contracted procedure: k14326 
o|contracted procedure: k14531 
o|contracted procedure: k14352 
o|contracted procedure: k14358 
o|contracted procedure: k14361 
o|contracted procedure: k14370 
o|contracted procedure: k14379 
o|contracted procedure: k14388 
o|contracted procedure: k14526 
o|contracted procedure: k14410 
o|contracted procedure: k14413 
o|contracted procedure: k14430 
o|contracted procedure: k14426 
o|contracted procedure: k14437 
o|contracted procedure: k14441 
o|contracted procedure: k14522 
o|contracted procedure: k14518 
o|contracted procedure: k14444 
o|contracted procedure: k14458 
o|contracted procedure: k14463 
o|contracted procedure: k14514 
o|contracted procedure: k14469 
o|contracted procedure: k14502 
o|contracted procedure: k14510 
o|contracted procedure: k14506 
o|contracted procedure: k14472 
o|contracted procedure: k14483 
o|contracted procedure: k14495 
o|contracted procedure: k14491 
o|contracted procedure: k14487 
o|contracted procedure: k14552 
o|contracted procedure: k14566 
o|contracted procedure: k14570 
o|contracted procedure: k14619 
o|contracted procedure: k14629 
o|contracted procedure: k14653 
o|contracted procedure: k14677 
o|contracted procedure: k14725 
o|contracted procedure: k14738 
o|contracted procedure: k14745 
o|contracted procedure: k14751 
o|contracted procedure: k14754 
o|contracted procedure: k14812 
o|contracted procedure: k14816 
o|contracted procedure: k14808 
o|contracted procedure: k14761 
o|contracted procedure: k14770 
o|contracted procedure: k14785 
o|contracted procedure: k14789 
o|contracted procedure: k14781 
o|contracted procedure: k14777 
o|contracted procedure: k14800 
o|contracted procedure: k14832 
o|contracted procedure: k14836 
o|contracted procedure: k14880 
o|contracted procedure: k14891 
o|contracted procedure: k14899 
o|contracted procedure: k14906 
o|contracted procedure: k14910 
o|contracted procedure: k14923 
o|contracted procedure: k14936 
o|contracted procedure: k14946 
o|contracted procedure: k15177 
o|contracted procedure: k14956 
o|contracted procedure: k15173 
o|contracted procedure: k14965 
o|contracted procedure: k15005 
o|contracted procedure: k15046 
o|contracted procedure: k15080 
o|contracted procedure: k15092 
o|contracted procedure: k15124 
o|contracted procedure: k15127 
o|contracted procedure: k15154 
o|contracted procedure: k15162 
o|contracted procedure: k15169 
o|contracted procedure: k15199 
o|contracted procedure: k15221 
o|contracted procedure: k15262 
o|contracted procedure: k15269 
o|contracted procedure: k15273 
o|contracted procedure: k15282 
o|contracted procedure: k15293 
o|contracted procedure: k15304 
o|contracted procedure: k15318 
o|contracted procedure: k15314 
o|contracted procedure: k15327 
o|contracted procedure: k15334 
o|contracted procedure: k15338 
o|contracted procedure: k15347 
o|contracted procedure: k15358 
o|contracted procedure: k15376 
o|contracted procedure: k15368 
o|contracted procedure: k15372 
o|contracted procedure: k15429 
o|contracted procedure: k15382 
o|contracted procedure: k15419 
o|contracted procedure: k15414 
o|contracted procedure: k15438 
o|contracted procedure: k15469 
o|contracted procedure: k15490 
o|contracted procedure: k15617 
o|contracted procedure: k15630 
o|contracted procedure: k15645 
o|contracted procedure: k15638 
o|contracted procedure: k15659 
o|contracted procedure: k15676 
o|contracted procedure: k15682 
o|contracted procedure: k15688 
o|contracted procedure: k15698 
o|contracted procedure: k15730 
o|contracted procedure: k15704 
o|contracted procedure: k15710 
o|contracted procedure: k15716 
o|contracted procedure: k15515 
o|contracted procedure: k15519 
o|contracted procedure: k15584 
o|contracted procedure: k15529 
o|contracted procedure: k15535 
o|contracted procedure: k15573 
o|contracted procedure: k15538 
o|contracted procedure: k15599 
o|contracted procedure: k15603 
o|contracted procedure: k15554 
o|contracted procedure: k15558 
o|contracted procedure: k15566 
o|contracted procedure: k15736 
o|contracted procedure: k17066 
o|contracted procedure: k17062 
o|contracted procedure: k15739 
o|contracted procedure: k17058 
o|contracted procedure: k17054 
o|contracted procedure: k15742 
o|contracted procedure: k17050 
o|contracted procedure: k17046 
o|contracted procedure: k15745 
o|contracted procedure: k15759 
o|contracted procedure: k15765 
o|contracted procedure: k15779 
o|contracted procedure: k15804 
o|contracted procedure: k15785 
o|contracted procedure: k15792 
o|contracted procedure: k15798 
o|contracted procedure: k15851 
o|contracted procedure: k15812 
o|contracted procedure: k15822 
o|contracted procedure: k15828 
o|contracted procedure: k15834 
o|contracted procedure: k15867 
o|contracted procedure: k15873 
o|contracted procedure: k15887 
o|contracted procedure: k15894 
o|contracted procedure: k15900 
o|contracted procedure: k15914 
o|contracted procedure: k15921 
o|contracted procedure: k15942 
o|contracted procedure: k15956 
o|contracted procedure: k15967 
o|contracted procedure: k15978 
o|contracted procedure: k15995 
o|contracted procedure: k15999 
o|contracted procedure: k16007 
o|contracted procedure: k16013 
o|contracted procedure: k16025 
o|contracted procedure: k16031 
o|contracted procedure: k16040 
o|contracted procedure: k16046 
o|contracted procedure: k16052 
o|contracted procedure: k16064 
o|contracted procedure: k16079 
o|contracted procedure: k16089 
o|contracted procedure: k16108 
o|contracted procedure: k16104 
o|contracted procedure: k16112 
o|contracted procedure: k16115 
o|contracted procedure: k16118 
o|contracted procedure: k16121 
o|contracted procedure: k16127 
o|contracted procedure: k16134 
o|contracted procedure: k16146 
o|contracted procedure: k16152 
o|contracted procedure: k16158 
o|contracted procedure: k16164 
o|contracted procedure: k16170 
o|contracted procedure: k16176 
o|contracted procedure: k16182 
o|contracted procedure: k16188 
o|contracted procedure: k16222 
o|contracted procedure: k16249 
o|contracted procedure: k16548 
o|contracted procedure: k16258 
o|contracted procedure: k16264 
o|contracted procedure: k16273 
o|contracted procedure: k16299 
o|contracted procedure: k16279 
o|contracted procedure: k16286 
o|contracted procedure: k16307 
o|contracted procedure: k16316 
o|contracted procedure: k16322 
o|contracted procedure: k16350 
o|contracted procedure: k16354 
o|contracted procedure: k16364 
o|contracted procedure: k16368 
o|contracted procedure: k16384 
o|contracted procedure: k16462 
o|contracted procedure: k16393 
o|contracted procedure: k16458 
o|contracted procedure: k16408 
o|contracted procedure: k16417 
o|contracted procedure: k16428 
o|contracted procedure: k16443 
o|contracted procedure: k16466 
o|contracted procedure: k16475 
o|contracted procedure: k16481 
o|contracted procedure: k16487 
o|contracted procedure: k16493 
o|contracted procedure: k16499 
o|contracted procedure: k16505 
o|contracted procedure: k16511 
o|contracted procedure: k16517 
o|contracted procedure: k16577 
o|contracted procedure: k16558 
o|contracted procedure: k16564 
o|contracted procedure: k16585 
o|contracted procedure: k16596 
o|contracted procedure: k16602 
o|contracted procedure: k16613 
o|contracted procedure: k16616 
o|contracted procedure: k16629 
o|contracted procedure: k16704 
o|contracted procedure: k16635 
o|contracted procedure: k16645 
o|contracted procedure: k16678 
o|contracted procedure: k16661 
o|contracted procedure: k16675 
o|contracted procedure: k16668 
o|contracted procedure: k16665 
o|contracted procedure: k16682 
o|contracted procedure: k16700 
o|contracted procedure: k16685 
o|contracted procedure: k16696 
o|contracted procedure: k16710 
o|contracted procedure: k16713 
o|contracted procedure: k16726 
o|contracted procedure: k16778 
o|contracted procedure: k16732 
o|contracted procedure: k16738 
o|contracted procedure: k16762 
o|contracted procedure: k16755 
o|contracted procedure: k16752 
o|contracted procedure: k16766 
o|contracted procedure: k16774 
o|contracted procedure: k16785 
o|contracted procedure: k16805 
o|contracted procedure: k16811 
o|contracted procedure: k16830 
o|contracted procedure: k17035 
o|contracted procedure: k16850 
o|contracted procedure: k16856 
o|contracted procedure: k16863 
o|contracted procedure: k16866 
o|contracted procedure: k16905 
o|contracted procedure: k16872 
o|contracted procedure: k16878 
o|contracted procedure: k16881 
o|contracted procedure: k16913 
o|contracted procedure: k16916 
o|contracted procedure: k16926 
o|contracted procedure: k16974 
o|contracted procedure: k16932 
o|contracted procedure: k16938 
o|inlining procedure: k16941 
o|inlining procedure: k16941 
o|contracted procedure: k16984 
o|contracted procedure: k17008 
o|contracted procedure: k16994 
o|contracted procedure: k17012 
o|contracted procedure: k17318 
o|contracted procedure: k17072 
o|contracted procedure: k17075 
o|contracted procedure: k17085 
o|contracted procedure: k17300 
o|contracted procedure: k17287 
o|contracted procedure: k17094 
o|contracted procedure: k17277 
o|contracted procedure: k17105 
o|contracted procedure: k17108 
o|contracted procedure: k17114 
o|contracted procedure: k17121 
o|contracted procedure: k17127 
o|contracted procedure: k17130 
o|contracted procedure: k17136 
o|contracted procedure: k17143 
o|contracted procedure: k17149 
o|contracted procedure: k17152 
o|contracted procedure: k17158 
o|contracted procedure: k17165 
o|contracted procedure: k17171 
o|contracted procedure: k17174 
o|contracted procedure: k17180 
o|contracted procedure: k17187 
o|contracted procedure: k17193 
o|contracted procedure: k17196 
o|contracted procedure: k17202 
o|contracted procedure: k17209 
o|contracted procedure: k17215 
o|contracted procedure: k17218 
o|contracted procedure: k17224 
o|contracted procedure: k17231 
o|contracted procedure: k17312 
o|contracted procedure: k17303 
o|contracted procedure: k17360 
o|contracted procedure: k17327 
o|contracted procedure: k17354 
o|contracted procedure: k17330 
o|contracted procedure: k17348 
o|contracted procedure: k17333 
o|contracted procedure: k17342 
o|contracted procedure: k17336 
o|contracted procedure: k17376 
o|contracted procedure: k17379 
o|contracted procedure: k17385 
o|contracted procedure: k17388 
o|contracted procedure: k17394 
o|contracted procedure: k17406 
o|contracted procedure: k17413 
o|contracted procedure: k17417 
o|contracted procedure: k17430 
o|contracted procedure: k17434 
o|contracted procedure: k17438 
o|contracted procedure: k17444 
o|contracted procedure: k17453 
o|contracted procedure: k17459 
o|contracted procedure: k17471 
o|contracted procedure: k17477 
o|contracted procedure: k17483 
o|contracted procedure: k17504 
o|contracted procedure: k17514 
o|contracted procedure: k17510 
o|contracted procedure: k17517 
o|contracted procedure: k17530 
o|contracted procedure: k17523 
o|contracted procedure: k17548 
o|contracted procedure: k17563 
o|contracted procedure: k17576 
o|contracted procedure: k17589 
o|contracted procedure: k17608 
o|contracted procedure: k17626 
o|contracted procedure: k17629 
o|contracted procedure: k17664 
o|contracted procedure: k17684 
o|contracted procedure: k17687 
o|contracted procedure: k17690 
o|contracted procedure: k17698 
o|contracted procedure: k17706 
o|contracted procedure: k17669 
o|contracted procedure: k17718 
o|contracted procedure: k17740 
o|contracted procedure: k17766 
o|contracted procedure: k17745 
o|contracted procedure: k17751 
o|contracted procedure: k17757 
o|contracted procedure: k17806 
o|contracted procedure: k17809 
o|contracted procedure: k17821 
o|contracted procedure: k17836 
o|contracted procedure: k17847 
o|contracted procedure: k17842 
o|contracted procedure: k17866 
o|contracted procedure: k17863 
o|contracted procedure: k17872 
o|contracted procedure: k17928 
o|contracted procedure: k17885 
o|contracted procedure: k17891 
o|contracted procedure: k17906 
o|contracted procedure: k17924 
o|contracted procedure: k17909 
o|contracted procedure: k17916 
o|contracted procedure: k17920 
o|contracted procedure: k17934 
o|contracted procedure: k17946 
o|contracted procedure: k17953 
o|contracted procedure: k17961 
o|contracted procedure: k17973 
o|contracted procedure: k17985 
o|contracted procedure: k17988 
o|contracted procedure: k17995 
o|contracted procedure: k18001 
o|contracted procedure: k18004 
o|contracted procedure: k18007 
o|contracted procedure: k18010 
o|contracted procedure: k18060 
o|contracted procedure: k18013 
o|contracted procedure: k18031 
o|contracted procedure: k18045 
o|contracted procedure: k18034 
o|contracted procedure: k18041 
o|contracted procedure: k18048 
o|contracted procedure: k18051 
o|contracted procedure: k18132 
o|contracted procedure: k18070 
o|contracted procedure: k18073 
o|contracted procedure: k18076 
o|contracted procedure: k18079 
o|contracted procedure: k18082 
o|contracted procedure: k18103 
o|contracted procedure: k18121 
o|contracted procedure: k18117 
o|contracted procedure: k18106 
o|contracted procedure: k18113 
o|contracted procedure: k18125 
o|contracted procedure: k18129 
o|contracted procedure: k18153 
o|contracted procedure: k18141 
o|contracted procedure: k18144 
o|contracted procedure: k18177 
o|contracted procedure: k18191 
o|contracted procedure: k18180 
o|contracted procedure: k18187 
o|contracted procedure: k18195 
o|contracted procedure: k18205 
o|contracted procedure: k18209 
o|contracted procedure: k18216 
o|contracted procedure: k18220 
o|contracted procedure: k18227 
o|contracted procedure: k18231 
o|contracted procedure: k18238 
o|contracted procedure: k18242 
o|contracted procedure: k18249 
o|contracted procedure: k18253 
o|contracted procedure: k18263 
o|contracted procedure: k18270 
o|contracted procedure: k18279 
o|contracted procedure: k18282 
o|contracted procedure: k18299 
o|contracted procedure: k18288 
o|contracted procedure: k18295 
o|contracted procedure: k18413 
o|contracted procedure: k18304 
o|contracted procedure: k18310 
o|contracted procedure: k18325 
o|contracted procedure: k18328 
o|contracted procedure: k18340 
o|contracted procedure: k18343 
o|contracted procedure: k18357 
o|contracted procedure: k18353 
o|contracted procedure: k18361 
o|contracted procedure: k18384 
o|contracted procedure: k18365 
o|contracted procedure: k18376 
o|contracted procedure: k18380 
o|contracted procedure: k18372 
o|contracted procedure: k18398 
o|contracted procedure: k18394 
o|contracted procedure: k18387 
o|contracted procedure: k18401 
o|contracted procedure: k18404 
o|contracted procedure: k18474 
o|contracted procedure: k18480 
o|contracted procedure: k18490 
o|contracted procedure: k18494 
o|contracted procedure: k18512 
o|contracted procedure: k18518 
o|contracted procedure: k18533 
o|contracted procedure: k18537 
o|contracted procedure: k18560 
o|contracted procedure: k18563 
o|contracted procedure: k18569 
o|contracted procedure: k18578 
o|contracted procedure: k18593 
o|contracted procedure: k18606 
o|contracted procedure: k18609 
o|contracted procedure: k18621 
o|contracted procedure: k18631 
o|contracted procedure: k18635 
o|contracted procedure: k18642 
o|contracted procedure: k18657 
o|contracted procedure: k18688 
o|contracted procedure: k18691 
o|contracted procedure: k18694 
o|contracted procedure: k18706 
o|contracted procedure: k18709 
o|contracted procedure: k18712 
o|contracted procedure: k18720 
o|contracted procedure: k18728 
o|contracted procedure: k18738 
o|contracted procedure: k18753 
o|contracted procedure: k18811 
o|contracted procedure: k18791 
o|contracted procedure: k18801 
o|contracted procedure: k18833 
o|contracted procedure: k18844 
o|contracted procedure: k18848 
o|contracted procedure: k18840 
o|contracted procedure: k18858 
o|contracted procedure: k18867 
o|contracted procedure: k18881 
o|contracted procedure: k18873 
o|contracted procedure: k18885 
o|contracted procedure: k18889 
o|contracted procedure: k18910 
o|contracted procedure: k18892 
o|contracted procedure: k18896 
o|contracted procedure: k18906 
o|contracted procedure: k18939 
o|contracted procedure: k18945 
o|contracted procedure: k18948 
o|contracted procedure: k18951 
o|contracted procedure: k18954 
o|contracted procedure: k18957 
o|contracted procedure: k18960 
o|contracted procedure: k18963 
o|contracted procedure: k32171 
o|contracted procedure: k32177 
o|contracted procedure: k32211 
o|contracted procedure: k32217 
o|contracted procedure: k32273 
o|contracted procedure: k32279 
o|contracted procedure: k32285 
o|contracted procedure: k32298 
o|contracted procedure: k32302 
o|contracted procedure: k32306 
o|contracted procedure: k32328 
o|contracted procedure: k32335 
o|contracted procedure: k32339 
o|contracted procedure: k32343 
o|contracted procedure: k32353 
o|contracted procedure: k32369 
o|contracted procedure: k32373 
o|contracted procedure: k32380 
o|contracted procedure: k32386 
o|contracted procedure: k32395 
o|contracted procedure: k32401 
o|contracted procedure: k32415 
o|contracted procedure: k32419 
o|contracted procedure: k32431 
o|contracted procedure: k32437 
o|contracted procedure: k32443 
o|contracted procedure: k32467 
o|contracted procedure: k32446 
o|contracted procedure: k32504 
o|contracted procedure: k32500 
o|contracted procedure: k32489 
o|contracted procedure: k32518 
o|contracted procedure: k32514 
o|contracted procedure: k32507 
o|contracted procedure: k32521 
o|contracted procedure: k18967 
o|contracted procedure: k18998 
o|contracted procedure: k19009 
o|contracted procedure: k19020 
o|contracted procedure: k19031 
o|contracted procedure: k19042 
o|contracted procedure: k19045 
o|contracted procedure: k19071 
o|contracted procedure: k19077 
o|contracted procedure: k19084 
o|contracted procedure: k19090 
o|contracted procedure: k19103 
o|contracted procedure: k19106 
o|contracted procedure: k19132 
o|contracted procedure: k19138 
o|contracted procedure: k19145 
o|contracted procedure: k19151 
o|contracted procedure: k19164 
o|contracted procedure: k19167 
o|contracted procedure: k19193 
o|contracted procedure: k19199 
o|contracted procedure: k19206 
o|contracted procedure: k19212 
o|contracted procedure: k19225 
o|contracted procedure: k19248 
o|contracted procedure: k19251 
o|contracted procedure: k19294 
o|contracted procedure: k19297 
o|contracted procedure: k19307 
o|contracted procedure: k19310 
o|contracted procedure: k1930735071 
o|contracted procedure: k19317 
o|contracted procedure: k1930735075 
o|contracted procedure: k19324 
o|contracted procedure: k1930735079 
o|contracted procedure: k19349 
o|contracted procedure: k19373 
o|contracted procedure: k19377 
o|contracted procedure: k19369 
o|contracted procedure: k19355 
o|contracted procedure: k19365 
o|contracted procedure: k19358 
o|contracted procedure: k19530 
o|contracted procedure: k19536 
o|contracted procedure: k19556 
o|contracted procedure: k19549 
o|contracted procedure: k19571 
o|contracted procedure: k19562 
o|contracted procedure: k19589 
o|contracted procedure: k19596 
o|contracted procedure: k19618 
o|contracted procedure: k19635 
o|contracted procedure: k19624 
o|contracted procedure: k19631 
o|contracted procedure: k19639 
o|contracted procedure: k19642 
o|contracted procedure: k19658 
o|contracted procedure: k19697 
o|contracted procedure: k19668 
o|contracted procedure: k19677 
o|contracted procedure: k19680 
o|contracted procedure: k19693 
o|contracted procedure: k19703 
o|contracted procedure: k19719 
o|contracted procedure: k19715 
o|contracted procedure: k19731 
o|contracted procedure: k19761 
o|contracted procedure: k19838 
o|contracted procedure: k19772 
o|contracted procedure: k19796 
o|contracted procedure: k19809 
o|contracted procedure: k19816 
o|contracted procedure: k19826 
o|contracted procedure: k19822 
o|contracted procedure: k19829 
o|contracted procedure: k19913 
o|contracted procedure: k20073 
o|contracted procedure: k19920 
o|contracted procedure: k19934 
o|contracted procedure: k19943 
o|contracted procedure: k19951 
o|contracted procedure: k19955 
o|contracted procedure: k19961 
o|contracted procedure: k19970 
o|contracted procedure: k19976 
o|contracted procedure: k19986 
o|contracted procedure: k20002 
o|contracted procedure: k20008 
o|contracted procedure: k20015 
o|contracted procedure: k20021 
o|contracted procedure: k20031 
o|contracted procedure: k20037 
o|contracted procedure: k20043 
o|contracted procedure: k20052 
o|contracted procedure: k20070 
o|contracted procedure: k20058 
o|contracted procedure: k20066 
o|contracted procedure: k20100 
o|contracted procedure: k20084 
o|contracted procedure: k20097 
o|contracted procedure: k20090 
o|contracted procedure: k20118 
o|contracted procedure: k20109 
o|contracted procedure: k20133 
o|contracted procedure: k20147 
o|contracted procedure: k20143 
o|contracted procedure: k20136 
o|inlining procedure: k20130 
o|contracted procedure: k20153 
o|contracted procedure: k20164 
o|contracted procedure: k20160 
o|inlining procedure: k20130 
o|contracted procedure: k20168 
o|contracted procedure: k20171 
o|contracted procedure: k20181 
o|contracted procedure: k20174 
o|contracted procedure: k20199 
o|contracted procedure: k20215 
o|contracted procedure: k20202 
o|contracted procedure: k20208 
o|contracted procedure: k20230 
o|contracted procedure: k20221 
o|contracted procedure: k20248 
o|contracted procedure: k20239 
o|contracted procedure: k20301 
o|contracted procedure: k20324 
o|contracted procedure: k20393 
o|contracted procedure: k20400 
o|contracted procedure: k20422 
o|contracted procedure: k20450 
o|contracted procedure: k20466 
o|contracted procedure: k20491 
o|contracted procedure: k20516 
o|contracted procedure: k20519 
o|contracted procedure: k20529 
o|contracted procedure: k20533 
o|contracted procedure: k20545 
o|contracted procedure: k20557 
o|contracted procedure: k20564 
o|contracted procedure: k20568 
o|contracted procedure: k20573 
o|contracted procedure: k20598 
o|contracted procedure: k20608 
o|contracted procedure: k20619 
o|contracted procedure: k20625 
o|contracted procedure: k20636 
o|contracted procedure: k20642 
o|contracted procedure: k20653 
o|contracted procedure: k20659 
o|contracted procedure: k20670 
o|contracted procedure: k20676 
o|contracted procedure: k20687 
o|contracted procedure: k20693 
o|contracted procedure: k20704 
o|contracted procedure: k20710 
o|contracted procedure: k20721 
o|contracted procedure: k20727 
o|contracted procedure: k20730 
o|contracted procedure: k20741 
o|contracted procedure: k20751 
o|contracted procedure: k20795 
o|contracted procedure: k20825 
o|contracted procedure: k20854 
o|contracted procedure: k20868 
o|contracted procedure: k20874 
o|contracted procedure: k20889 
o|contracted procedure: k20892 
o|contracted procedure: k20905 
o|contracted procedure: k20920 
o|contracted procedure: k20936 
o|contracted procedure: k21015 
o|contracted procedure: k21011 
o|contracted procedure: k21003 
o|contracted procedure: k20999 
o|contracted procedure: k20984 
o|contracted procedure: k20995 
o|contracted procedure: k21029 
o|contracted procedure: k21032 
o|contracted procedure: k21038 
o|contracted procedure: k21044 
o|contracted procedure: k21050 
o|contracted procedure: k21067 
o|contracted procedure: k21073 
o|contracted procedure: k21113 
o|contracted procedure: k21127 
o|contracted procedure: k21136 
o|contracted procedure: k21145 
o|contracted procedure: k21178 
o|contracted procedure: k21219 
o|contracted procedure: k21228 
o|contracted procedure: k21240 
o|contracted procedure: k21252 
o|contracted procedure: k21261 
o|contracted procedure: k21279 
o|contracted procedure: k21297 
o|contracted procedure: k21300 
o|contracted procedure: k21337 
o|contracted procedure: k21310 
o|contracted procedure: k21331 
o|contracted procedure: k21343 
o|contracted procedure: k21349 
o|contracted procedure: k21355 
o|contracted procedure: k21364 
o|contracted procedure: k21367 
o|contracted procedure: k21401 
o|contracted procedure: k21435 
o|contracted procedure: k21477 
o|contracted procedure: k21469 
o|contracted procedure: k21444 
o|contracted procedure: k21486 
o|contracted procedure: k21498 
o|contracted procedure: k21507 
o|contracted procedure: k21537 
o|contracted procedure: k21549 
o|contracted procedure: k21558 
o|contracted procedure: k21614 
o|contracted procedure: k21626 
o|inlining procedure: k21643 
o|contracted procedure: k21647 
o|inlining procedure: k21643 
o|contracted procedure: k21653 
o|contracted procedure: k21678 
o|contracted procedure: k21698 
o|contracted procedure: k21705 
o|contracted procedure: k21752 
o|contracted procedure: k21772 
o|contracted procedure: k21807 
o|contracted procedure: k21820 
o|contracted procedure: k21826 
o|contracted procedure: k21840 
o|contracted procedure: k21836 
o|contracted procedure: k21846 
o|contracted procedure: k21858 
o|contracted procedure: k21882 
o|contracted procedure: k21892 
o|contracted procedure: k21902 
o|contracted procedure: k21909 
o|contracted procedure: k21915 
o|contracted procedure: k21924 
o|contracted procedure: k21934 
o|inlining procedure: k21941 
o|contracted procedure: k21945 
o|inlining procedure: k21941 
o|contracted procedure: k21959 
o|contracted procedure: k21968 
o|contracted procedure: k21974 
o|contracted procedure: k21993 
o|contracted procedure: k21999 
o|contracted procedure: k22016 
o|contracted procedure: k22019 
o|contracted procedure: k22041 
o|contracted procedure: k22160 
o|contracted procedure: k22047 
o|contracted procedure: k22156 
o|contracted procedure: k22148 
o|contracted procedure: k22152 
o|contracted procedure: k22144 
o|contracted procedure: k22050 
o|contracted procedure: k22053 
o|contracted procedure: k22140 
o|contracted procedure: k22136 
o|contracted procedure: k22132 
o|contracted procedure: k22128 
o|contracted procedure: k22115 
o|contracted procedure: k22123 
o|contracted procedure: k22119 
o|contracted procedure: k22059 
o|contracted procedure: k22071 
o|contracted procedure: k22102 
o|contracted procedure: k22110 
o|contracted procedure: k22106 
o|contracted procedure: k22075 
o|contracted procedure: k22081 
o|contracted procedure: k22089 
o|contracted procedure: k22097 
o|contracted procedure: k22093 
o|contracted procedure: k22085 
o|contracted procedure: k22164 
o|contracted procedure: k22187 
o|contracted procedure: k22208 
o|contracted procedure: k22214 
o|contracted procedure: k22231 
o|contracted procedure: k22220 
o|contracted procedure: k22227 
o|contracted procedure: k22244 
o|contracted procedure: k22254 
o|contracted procedure: k22251 
o|contracted procedure: k22267 
o|contracted procedure: k22303 
o|contracted procedure: k22332 
o|contracted procedure: k22356 
o|contracted procedure: k22368 
o|contracted procedure: k22380 
o|contracted procedure: k22392 
o|contracted procedure: k22401 
o|contracted procedure: k22420 
o|contracted procedure: k22432 
o|contracted procedure: k22441 
o|contracted procedure: k22450 
o|contracted procedure: k22459 
o|contracted procedure: k22483 
o|contracted procedure: k22495 
o|contracted procedure: k22498 
o|contracted procedure: k22513 
o|contracted procedure: k22523 
o|contracted procedure: k22530 
o|contracted procedure: k22554 
o|contracted procedure: k22563 
o|contracted procedure: k22569 
o|contracted procedure: k22581 
o|contracted procedure: k22593 
o|contracted procedure: k22605 
o|contracted procedure: k22617 
o|contracted procedure: k22629 
o|contracted procedure: k22641 
o|contracted procedure: k22672 
o|contracted procedure: k22681 
o|contracted procedure: k22687 
o|contracted procedure: k22696 
o|contracted procedure: k22705 
o|contracted procedure: k22717 
o|contracted procedure: k22732 
o|contracted procedure: k22744 
o|contracted procedure: k22759 
o|contracted procedure: k22771 
o|contracted procedure: k22783 
o|contracted procedure: k22798 
o|contracted procedure: k22847 
o|contracted procedure: k22836 
o|contracted procedure: k22843 
o|contracted procedure: k22827 
o|contracted procedure: k22858 
o|contracted procedure: k22878 
o|contracted procedure: k22874 
o|contracted procedure: k22888 
o|contracted procedure: k22915 
o|contracted procedure: k22921 
o|contracted procedure: k22931 
o|contracted procedure: k22937 
o|contracted procedure: k22950 
o|contracted procedure: k22953 
o|contracted procedure: k22976 
o|contracted procedure: k23023 
o|contracted procedure: k23029 
o|contracted procedure: k23038 
o|contracted procedure: k23050 
o|contracted procedure: k23062 
o|contracted procedure: k23071 
o|contracted procedure: k23074 
o|contracted procedure: k23083 
o|contracted procedure: k23089 
o|contracted procedure: k23098 
o|contracted procedure: k23107 
o|contracted procedure: k23116 
o|contracted procedure: k23125 
o|contracted procedure: k23128 
o|contracted procedure: k23188 
o|contracted procedure: k23174 
o|contracted procedure: k23184 
o|contracted procedure: k23202 
o|contracted procedure: k23192 
o|contracted procedure: k23199 
o|contracted procedure: k23211 
o|contracted procedure: k23217 
o|contracted procedure: k23226 
o|contracted procedure: k23253 
o|contracted procedure: k23249 
o|contracted procedure: k23233 
o|contracted procedure: k23245 
o|contracted procedure: k23241 
o|contracted procedure: k23237 
o|contracted procedure: k23259 
o|contracted procedure: k23302 
o|contracted procedure: k23298 
o|contracted procedure: k23266 
o|contracted procedure: k23294 
o|contracted procedure: k23290 
o|contracted procedure: k23286 
o|contracted procedure: k23270 
o|contracted procedure: k23282 
o|contracted procedure: k23278 
o|contracted procedure: k23274 
o|contracted procedure: k23308 
o|contracted procedure: k23367 
o|contracted procedure: k23363 
o|contracted procedure: k23315 
o|contracted procedure: k23359 
o|contracted procedure: k23355 
o|contracted procedure: k23351 
o|contracted procedure: k23319 
o|contracted procedure: k23347 
o|contracted procedure: k23343 
o|contracted procedure: k23339 
o|contracted procedure: k23323 
o|contracted procedure: k23335 
o|contracted procedure: k23331 
o|contracted procedure: k23327 
o|contracted procedure: k23379 
o|contracted procedure: k23391 
o|contracted procedure: k23397 
o|contracted procedure: k23403 
o|contracted procedure: k23409 
o|contracted procedure: k23444 
o|contracted procedure: k23440 
o|contracted procedure: k23436 
o|contracted procedure: k23416 
o|contracted procedure: k23432 
o|contracted procedure: k23424 
o|contracted procedure: k23428 
o|contracted procedure: k23420 
o|contracted procedure: k23453 
o|contracted procedure: k23459 
o|contracted procedure: k23466 
o|contracted procedure: k23472 
o|contracted procedure: k23479 
o|contracted procedure: k23505 
o|contracted procedure: k23514 
o|contracted procedure: k23532 
o|contracted procedure: k23528 
o|contracted procedure: k23521 
o|contracted procedure: k23538 
o|contracted procedure: k23556 
o|contracted procedure: k23552 
o|contracted procedure: k23548 
o|contracted procedure: k23577 
o|contracted procedure: k23573 
o|contracted procedure: k23569 
o|contracted procedure: k23588 
o|contracted procedure: k23601 
o|contracted procedure: k23604 
o|contracted procedure: k23613 
o|contracted procedure: k23616 
o|contracted procedure: k23653 
o|contracted procedure: k23643 
o|contracted procedure: k23662 
o|contracted procedure: k23683 
o|contracted procedure: k23671 
o|contracted procedure: k23674 
o|contracted procedure: k23698 
o|contracted procedure: k23692 
o|contracted procedure: k23716 
o|contracted procedure: k23707 
o|contracted procedure: k23734 
o|contracted procedure: k23725 
o|contracted procedure: k23753 
o|contracted procedure: k23765 
o|contracted procedure: k23775 
o|contracted procedure: k23779 
o|contracted procedure: k23804 
o|contracted procedure: k23816 
o|contracted procedure: k23826 
o|contracted procedure: k23830 
o|contracted procedure: k23875 
o|contracted procedure: k23881 
o|contracted procedure: k23890 
o|contracted procedure: k23893 
o|contracted procedure: k23909 
o|propagated global variable: r23910 ##sys#undefined-value 
o|contracted procedure: k23912 
o|contracted procedure: k23938 
o|contracted procedure: k23931 
o|contracted procedure: k23954 
o|contracted procedure: k23947 
o|contracted procedure: k23966 
o|contracted procedure: k23979 
o|propagated global variable: r23980 ##sys#undefined-value 
o|contracted procedure: k23983 
o|contracted procedure: k23989 
o|contracted procedure: k23992 
o|contracted procedure: k24010 
o|contracted procedure: k24028 
o|contracted procedure: k24043 
o|contracted procedure: k24052 
o|contracted procedure: k24055 
o|contracted procedure: k24061 
o|contracted procedure: k24074 
o|contracted procedure: k24081 
o|contracted procedure: k24097 
o|contracted procedure: k24100 
o|contracted procedure: k24103 
o|contracted procedure: k24117 
o|contracted procedure: k24123 
o|contracted procedure: k24129 
o|contracted procedure: k24135 
o|contracted procedure: k24138 
o|contracted procedure: k24150 
o|contracted procedure: k24160 
o|contracted procedure: k24169 
o|contracted procedure: k24175 
o|contracted procedure: k24191 
o|contracted procedure: k24209 
o|contracted procedure: k24219 
o|contracted procedure: k24225 
o|contracted procedure: k24232 
o|contracted procedure: k24216 
o|contracted procedure: k24237 
o|contracted procedure: k24243 
o|contracted procedure: k24261 
o|contracted procedure: k24288 
o|contracted procedure: k24267 
o|contracted procedure: k24277 
o|contracted procedure: k24284 
o|contracted procedure: k24299 
o|contracted procedure: k24314 
o|contracted procedure: k24323 
o|contracted procedure: k24332 
o|contracted procedure: k24359 
o|contracted procedure: k24374 
o|contracted procedure: k24383 
o|contracted procedure: k24386 
o|contracted procedure: k24427 
o|contracted procedure: k24479 
o|contracted procedure: k24442 
o|contracted procedure: k24448 
o|contracted procedure: k24460 
o|contracted procedure: k24530 
o|contracted procedure: k24543 
o|contracted procedure: k24547 
o|contracted procedure: k24550 
o|contracted procedure: k24559 
o|contracted procedure: k24568 
o|contracted procedure: k24571 
o|contracted procedure: k24580 
o|contracted procedure: k24589 
o|contracted procedure: k24598 
o|contracted procedure: k24607 
o|contracted procedure: k24616 
o|contracted procedure: k24625 
o|contracted procedure: k24634 
o|contracted procedure: k24653 
o|contracted procedure: k24700 
o|contracted procedure: k24709 
o|contracted procedure: k24712 
o|contracted procedure: k24731 
o|contracted procedure: k24721 
o|contracted procedure: k24744 
o|contracted procedure: k24748 
o|contracted procedure: k24757 
o|contracted procedure: k24766 
o|contracted procedure: k24781 
o|contracted procedure: k24784 
o|contracted procedure: k24797 
o|contracted procedure: k24804 
o|contracted procedure: k24851 
o|contracted procedure: k24864 
o|contracted procedure: k24867 
o|contracted procedure: k24876 
o|contracted procedure: k24895 
o|contracted procedure: k24911 
o|contracted procedure: k24920 
o|contracted procedure: k24936 
o|contracted procedure: k24940 
o|contracted procedure: k24944 
o|contracted procedure: k24948 
o|contracted procedure: k24975 
o|contracted procedure: k24978 
o|contracted procedure: k24990 
o|contracted procedure: k24996 
o|contracted procedure: k24999 
o|contracted procedure: k25010 
o|contracted procedure: k25006 
o|contracted procedure: k25021 
o|contracted procedure: k25024 
o|contracted procedure: k25060 
o|contracted procedure: k25083 
o|contracted procedure: k25063 
o|contracted procedure: k25079 
o|contracted procedure: k25102 
o|contracted procedure: k25132 
o|contracted procedure: k25142 
o|contracted procedure: k25148 
o|contracted procedure: k25158 
o|contracted procedure: k25257 
o|contracted procedure: k25260 
o|contracted procedure: k25263 
o|contracted procedure: k25266 
o|contracted procedure: k25272 
o|contracted procedure: k25297 
o|contracted procedure: k25278 
o|contracted procedure: k25287 
o|contracted procedure: k25290 
o|contracted procedure: k25300 
o|contracted procedure: k31659 
o|contracted procedure: k31662 
o|contracted procedure: k31665 
o|contracted procedure: k31671 
o|contracted procedure: k31678 
o|contracted procedure: k31674 
o|contracted procedure: k31684 
o|contracted procedure: k31687 
o|contracted procedure: k31690 
o|contracted procedure: k31696 
o|contracted procedure: k31705 
o|contracted procedure: k31708 
o|contracted procedure: k31715 
o|contracted procedure: k31724 
o|contracted procedure: k31727 
o|contracted procedure: k31734 
o|contracted procedure: k31746 
o|contracted procedure: k31781 
o|contracted procedure: k31749 
o|contracted procedure: k31756 
o|contracted procedure: k31760 
o|contracted procedure: k31767 
o|contracted procedure: k31763 
o|contracted procedure: k31770 
o|contracted procedure: k31773 
o|contracted procedure: k31787 
o|contracted procedure: k31790 
o|contracted procedure: k31793 
o|contracted procedure: k31796 
o|contracted procedure: k31802 
o|contracted procedure: k31824 
o|contracted procedure: k31815 
o|contracted procedure: k31833 
o|contracted procedure: k31847 
o|contracted procedure: k31843 
o|contracted procedure: k31836 
o|contracted procedure: k31858 
o|contracted procedure: k31862 
o|contracted procedure: k31854 
o|contracted procedure: k31871 
o|contracted procedure: k31874 
o|contracted procedure: k31877 
o|contracted procedure: k31883 
o|contracted procedure: k31889 
o|contracted procedure: k25307 
o|contracted procedure: k25316 
o|contracted procedure: k25327 
o|contracted procedure: k25323 
o|contracted procedure: k25339 
o|contracted procedure: k25369 
o|contracted procedure: k25376 
o|contracted procedure: k25393 
o|contracted procedure: k25427 
o|contracted procedure: k25434 
o|contracted procedure: k25455 
o|contracted procedure: k25462 
o|contracted procedure: k25473 
o|contracted procedure: k25476 
o|contracted procedure: k25483 
o|contracted procedure: k25486 
o|contracted procedure: k25504 
o|contracted procedure: k25492 
o|contracted procedure: k25500 
o|contracted procedure: k25510 
o|contracted procedure: k25516 
o|contracted procedure: k25519 
o|contracted procedure: k25522 
o|contracted procedure: k25532 
o|contracted procedure: k25535 
o|contracted procedure: k25538 
o|contracted procedure: k25576 
o|contracted procedure: k25572 
o|contracted procedure: k25554 
o|contracted procedure: k25564 
o|contracted procedure: k25568 
o|contracted procedure: k25590 
o|contracted procedure: k25605 
o|contracted procedure: k25615 
o|contracted procedure: k25619 
o|contracted procedure: k25644 
o|contracted procedure: k25654 
o|inlining procedure: k25647 
o|inlining procedure: k25647 
o|contracted procedure: k25813 
o|contracted procedure: k25678 
o|contracted procedure: k25807 
o|contracted procedure: k25681 
o|contracted procedure: k25801 
o|contracted procedure: k25684 
o|contracted procedure: k25795 
o|contracted procedure: k25687 
o|contracted procedure: k25785 
o|contracted procedure: k25693 
o|contracted procedure: k25696 
o|contracted procedure: k25708 
o|contracted procedure: k25779 
o|contracted procedure: k25711 
o|contracted procedure: k25717 
o|contracted procedure: k25755 
o|contracted procedure: k25743 
o|contracted procedure: k25751 
o|contracted procedure: k25747 
o|contracted procedure: k25727 
o|contracted procedure: k25735 
o|contracted procedure: k25759 
o|contracted procedure: k25766 
o|contracted procedure: k25769 
o|contracted procedure: k25792 
o|contracted procedure: k25825 
o|contracted procedure: k25833 
o|contracted procedure: k25836 
o|contracted procedure: k25839 
o|contracted procedure: k25879 
o|contracted procedure: k25892 
o|contracted procedure: k25907 
o|contracted procedure: k25917 
o|contracted procedure: k25921 
o|contracted procedure: k26009 
o|contracted procedure: k25927 
o|contracted procedure: k26003 
o|contracted procedure: k25930 
o|contracted procedure: k25997 
o|contracted procedure: k25933 
o|contracted procedure: k25991 
o|contracted procedure: k25936 
o|contracted procedure: k25985 
o|contracted procedure: k25939 
o|contracted procedure: k25979 
o|contracted procedure: k25942 
o|contracted procedure: k25973 
o|contracted procedure: k25945 
o|contracted procedure: k25967 
o|contracted procedure: k25948 
o|contracted procedure: k25957 
o|contracted procedure: k26027 
o|contracted procedure: k26044 
o|contracted procedure: k26056 
o|contracted procedure: k26082 
o|contracted procedure: k26075 
o|contracted procedure: k26092 
o|contracted procedure: k26100 
o|contracted procedure: k26115 
o|contracted procedure: k26125 
o|contracted procedure: k26129 
o|contracted procedure: k26156 
o|contracted procedure: k26163 
o|contracted procedure: k26169 
o|contracted procedure: k26172 
o|contracted procedure: k26210 
o|contracted procedure: k26220 
o|contracted procedure: k26224 
o|contracted procedure: k26227 
o|contracted procedure: k26230 
o|contracted procedure: k26248 
o|contracted procedure: k26257 
o|contracted procedure: k26260 
o|contracted procedure: k26263 
o|contracted procedure: k26278 
o|contracted procedure: k26270 
o|contracted procedure: k26285 
o|contracted procedure: k26291 
o|contracted procedure: k26297 
o|contracted procedure: k26303 
o|contracted procedure: k26309 
o|contracted procedure: k26315 
o|contracted procedure: k26321 
o|contracted procedure: k26327 
o|contracted procedure: k26333 
o|contracted procedure: k26339 
o|contracted procedure: k26345 
o|contracted procedure: k26351 
o|contracted procedure: k26357 
o|contracted procedure: k26363 
o|contracted procedure: k26400 
o|contracted procedure: k26403 
o|contracted procedure: k26419 
o|contracted procedure: k26412 
o|contracted procedure: k26448 
o|contracted procedure: k26444 
o|contracted procedure: k26477 
o|contracted procedure: k26473 
o|contracted procedure: k26480 
o|contracted procedure: k26484 
o|contracted procedure: k26490 
o|contracted procedure: k26493 
o|contracted procedure: k26496 
o|contracted procedure: k26499 
o|contracted procedure: k26502 
o|contracted procedure: k26523 
o|contracted procedure: k26529 
o|contracted procedure: k26532 
o|contracted procedure: k26538 
o|contracted procedure: k26552 
o|contracted procedure: k26549 
o|contracted procedure: k26561 
o|contracted procedure: k26620 
o|contracted procedure: k26573 
o|contracted procedure: k26616 
o|contracted procedure: k26588 
o|contracted procedure: k26612 
o|contracted procedure: k26608 
o|contracted procedure: k26604 
o|contracted procedure: k26643 
o|contracted procedure: k26646 
o|contracted procedure: k26672 
o|contracted procedure: k26678 
o|contracted procedure: k26685 
o|contracted procedure: k26691 
o|contracted procedure: k26710 
o|contracted procedure: k26763 
o|contracted procedure: k26716 
o|contracted procedure: k26748 
o|contracted procedure: k26729 
o|contracted procedure: k26737 
o|contracted procedure: k26733 
o|contracted procedure: k26759 
o|contracted procedure: k26755 
o|contracted procedure: k26773 
o|contracted procedure: k26783 
o|contracted procedure: k26806 
o|contracted procedure: k26823 
o|contracted procedure: k26845 
o|contracted procedure: k26841 
o|contracted procedure: k26826 
o|contracted procedure: k26829 
o|contracted procedure: k26837 
o|contracted procedure: k26852 
o|contracted procedure: k26869 
o|contracted procedure: k26891 
o|contracted procedure: k26887 
o|contracted procedure: k26872 
o|contracted procedure: k26875 
o|contracted procedure: k26883 
o|contracted procedure: k26903 
o|contracted procedure: k26917 
o|contracted procedure: k26906 
o|contracted procedure: k26913 
o|contracted procedure: k26923 
o|contracted procedure: k26933 
o|contracted procedure: k26945 
o|contracted procedure: k26967 
o|contracted procedure: k26963 
o|contracted procedure: k26948 
o|contracted procedure: k26951 
o|contracted procedure: k26959 
o|contracted procedure: k26974 
o|contracted procedure: k26997 
o|contracted procedure: k27000 
o|contracted procedure: k27003 
o|contracted procedure: k27011 
o|contracted procedure: k27019 
o|contracted procedure: k26983 
o|contracted procedure: k27037 
o|contracted procedure: k27050 
o|contracted procedure: k27059 
o|contracted procedure: k27097 
o|contracted procedure: k27065 
o|contracted procedure: k27079 
o|contracted procedure: k27075 
o|contracted procedure: k27093 
o|contracted procedure: k27100 
o|contracted procedure: k27103 
o|contracted procedure: k27115 
o|contracted procedure: k27118 
o|contracted procedure: k27121 
o|contracted procedure: k27129 
o|contracted procedure: k27137 
o|contracted procedure: k27167 
o|contracted procedure: k27163 
o|contracted procedure: k27173 
o|contracted procedure: k27176 
o|contracted procedure: k27181 
o|contracted procedure: k27213 
o|contracted procedure: k27187 
o|contracted procedure: k27209 
o|contracted procedure: k27190 
o|contracted procedure: k27251 
o|contracted procedure: k27263 
o|contracted procedure: k27273 
o|contracted procedure: k27277 
o|contracted procedure: k27308 
o|contracted procedure: k27333 
o|contracted procedure: k27317 
o|contracted procedure: k27350 
o|contracted procedure: k27370 
o|contracted procedure: k27376 
o|contracted procedure: k27389 
o|contracted procedure: k27395 
o|contracted procedure: k27414 
o|contracted procedure: k27428 
o|contracted procedure: k27417 
o|contracted procedure: k27424 
o|contracted procedure: k27444 
o|contracted procedure: k27450 
o|contracted procedure: k27457 
o|contracted procedure: k27463 
o|contracted procedure: k27513 
o|contracted procedure: k27473 
o|contracted procedure: k27525 
o|contracted procedure: k27528 
o|contracted procedure: k27531 
o|contracted procedure: k27534 
o|contracted procedure: k27545 
o|contracted procedure: k27562 
o|contracted procedure: k27565 
o|contracted procedure: k27568 
o|contracted procedure: k27571 
o|contracted procedure: k27582 
o|contracted procedure: k27599 
o|contracted procedure: k27608 
o|contracted procedure: k27617 
o|contracted procedure: k27626 
o|contracted procedure: k27635 
o|contracted procedure: k27644 
o|contracted procedure: k27653 
o|contracted procedure: k27662 
o|contracted procedure: k27671 
o|contracted procedure: k27680 
o|contracted procedure: k27689 
o|contracted procedure: k27698 
o|contracted procedure: k27707 
o|contracted procedure: k27716 
o|contracted procedure: k27725 
o|contracted procedure: k27734 
o|contracted procedure: k27743 
o|contracted procedure: k27752 
o|contracted procedure: k27761 
o|contracted procedure: k27770 
o|contracted procedure: k27779 
o|contracted procedure: k27788 
o|contracted procedure: k27797 
o|contracted procedure: k27806 
o|contracted procedure: k27815 
o|contracted procedure: k27824 
o|contracted procedure: k27833 
o|contracted procedure: k27842 
o|contracted procedure: k27851 
o|contracted procedure: k27860 
o|contracted procedure: k27869 
o|contracted procedure: k27878 
o|contracted procedure: k27887 
o|contracted procedure: k27896 
o|contracted procedure: k27905 
o|contracted procedure: k27914 
o|contracted procedure: k27923 
o|contracted procedure: k27932 
o|contracted procedure: k27941 
o|contracted procedure: k27950 
o|contracted procedure: k27959 
o|contracted procedure: k27968 
o|contracted procedure: k27977 
o|contracted procedure: k27986 
o|contracted procedure: k27995 
o|contracted procedure: k28004 
o|contracted procedure: k28013 
o|contracted procedure: k28022 
o|contracted procedure: k28031 
o|contracted procedure: k28040 
o|contracted procedure: k28049 
o|contracted procedure: k28058 
o|contracted procedure: k28067 
o|contracted procedure: k28076 
o|contracted procedure: k28079 
o|contracted procedure: k28082 
o|contracted procedure: k28085 
o|contracted procedure: k28096 
o|contracted procedure: k28289 
o|contracted procedure: k28319 
o|contracted procedure: k28295 
o|contracted procedure: k28298 
o|contracted procedure: k28307 
o|contracted procedure: k28316 
o|contracted procedure: k28352 
o|contracted procedure: k28345 
o|contracted procedure: k28374 
o|contracted procedure: k28367 
o|contracted procedure: k28425 
o|contracted procedure: k28442 
o|contracted procedure: k28404 
o|contracted procedure: k28408 
o|contracted procedure: k28469 
o|contracted procedure: k28489 
o|contracted procedure: k28506 
o|contracted procedure: k28456 
o|contracted procedure: k28460 
o|contracted procedure: k28522 
o|contracted procedure: k28529 
o|contracted procedure: k28585 
o|contracted procedure: k28589 
o|contracted procedure: k28597 
o|contracted procedure: k28614 
o|contracted procedure: k28629 
o|contracted procedure: k28692 
o|contracted procedure: k28673 
o|contracted procedure: k28682 
o|contracted procedure: k28701 
o|contracted procedure: k28758 
o|contracted procedure: k28761 
o|contracted procedure: k28767 
o|contracted procedure: k28810 
o|contracted procedure: k28826 
o|contracted procedure: k28819 
o|contracted procedure: k29258 
o|contracted procedure: k28837 
o|contracted procedure: k28852 
o|contracted procedure: k28956 
o|contracted procedure: k28864 
o|contracted procedure: k28883 
o|contracted procedure: k28886 
o|contracted procedure: k28895 
o|contracted procedure: k2873535195 
o|contracted procedure: k28925 
o|contracted procedure: k28953 
o|contracted procedure: k28935 
o|contracted procedure: k28948 
o|contracted procedure: k28965 
o|contracted procedure: k29244 
o|contracted procedure: k28987 
o|contracted procedure: k29005 
o|contracted procedure: k29008 
o|contracted procedure: k29017 
o|contracted procedure: k29020 
o|contracted procedure: k29034 
o|contracted procedure: k29043 
o|contracted procedure: k29098 
o|contracted procedure: k29056 
o|contracted procedure: k29059 
o|contracted procedure: k29068 
o|contracted procedure: k29085 
o|contracted procedure: k29079 
o|contracted procedure: k29101 
o|contracted procedure: k2873535208 
o|contracted procedure: k29119 
o|contracted procedure: k29115 
o|contracted procedure: k29128 
o|contracted procedure: k29137 
o|contracted procedure: k29153 
o|contracted procedure: k29168 
o|contracted procedure: k29160 
o|contracted procedure: k29187 
o|contracted procedure: k29179 
o|contracted procedure: k29213 
o|contracted procedure: k29241 
o|contracted procedure: k29223 
o|contracted procedure: k29236 
o|contracted procedure: k29264 
o|contracted procedure: k29273 
o|contracted procedure: k29313 
o|contracted procedure: k29319 
o|contracted procedure: k29322 
o|contracted procedure: k29336 
o|contracted procedure: k29377 
o|contracted procedure: k29384 
o|contracted procedure: k29392 
o|contracted procedure: k29404 
o|contracted procedure: k29408 
o|contracted procedure: k29415 
o|contracted procedure: k29351 
o|contracted procedure: k29449 
o|contracted procedure: k29458 
o|contracted procedure: k29465 
o|contracted procedure: k29469 
o|contracted procedure: k29514 
o|contracted procedure: k29499 
o|contracted procedure: k29506 
o|contracted procedure: k29510 
o|contracted procedure: k29521 
o|contracted procedure: k29534 
o|contracted procedure: k29541 
o|contracted procedure: k29572 
o|contracted procedure: k29568 
o|contracted procedure: k29596 
o|contracted procedure: k29605 
o|contracted procedure: k29617 
o|contracted procedure: k29621 
o|contracted procedure: k29633 
o|contracted procedure: k29643 
o|contracted procedure: k29716 
o|contracted procedure: k29646 
o|contracted procedure: k29659 
o|contracted procedure: k29700 
o|contracted procedure: k29689 
o|contracted procedure: k29696 
o|contracted procedure: k29746 
o|contracted procedure: k29784 
o|contracted procedure: k29774 
o|contracted procedure: k29802 
o|contracted procedure: k29836 
o|contracted procedure: k29845 
o|contracted procedure: k29848 
o|contracted procedure: k29855 
o|contracted procedure: k29861 
o|contracted procedure: k29868 
o|contracted procedure: k29874 
o|contracted procedure: k29877 
o|contracted procedure: k29883 
o|contracted procedure: k29886 
o|contracted procedure: k29893 
o|contracted procedure: k29899 
o|contracted procedure: k29902 
o|contracted procedure: k29909 
o|contracted procedure: k29915 
o|contracted procedure: k29918 
o|contracted procedure: k29925 
o|contracted procedure: k29931 
o|contracted procedure: k29934 
o|contracted procedure: k29941 
o|contracted procedure: k29947 
o|contracted procedure: k29950 
o|contracted procedure: k29957 
o|contracted procedure: k29963 
o|contracted procedure: k29966 
o|contracted procedure: k29973 
o|contracted procedure: k29979 
o|contracted procedure: k29982 
o|contracted procedure: k29989 
o|contracted procedure: k29995 
o|contracted procedure: k29998 
o|contracted procedure: k30005 
o|contracted procedure: k30011 
o|contracted procedure: k30014 
o|contracted procedure: k30021 
o|contracted procedure: k30027 
o|contracted procedure: k30030 
o|contracted procedure: k30037 
o|contracted procedure: k30044 
o|contracted procedure: k30052 
o|contracted procedure: k30048 
o|contracted procedure: k30078 
o|contracted procedure: k30085 
o|contracted procedure: k30097 
o|contracted procedure: k30109 
o|contracted procedure: k30103 
o|contracted procedure: k30106 
o|contracted procedure: k30124 
o|contracted procedure: k30137 
o|contracted procedure: k30133 
o|contracted procedure: k30141 
o|contracted procedure: k30185 
o|contracted procedure: k30150 
o|contracted procedure: k30153 
o|contracted procedure: k30156 
o|contracted procedure: k30181 
o|contracted procedure: k30168 
o|contracted procedure: k30171 
o|contracted procedure: k30195 
o|contracted procedure: k30201 
o|contracted procedure: k30208 
o|contracted procedure: k30217 
o|contracted procedure: k30223 
o|contracted procedure: k30226 
o|contracted procedure: k30229 
o|contracted procedure: k30235 
o|contracted procedure: k30242 
o|contracted procedure: k30248 
o|contracted procedure: k30252 
o|contracted procedure: k30299 
o|contracted procedure: k30312 
o|contracted procedure: k30334 
o|contracted procedure: k30356 
o|contracted procedure: k30390 
o|contracted procedure: k30396 
o|contracted procedure: k30399 
o|contracted procedure: k30405 
o|contracted procedure: k30408 
o|contracted procedure: k30414 
o|contracted procedure: k30429 
o|contracted procedure: k30432 
o|contracted procedure: k30453 
o|contracted procedure: k30471 
o|contracted procedure: k30484 
o|contracted procedure: k30654 
o|contracted procedure: k30526 
o|contracted procedure: k30612 
o|contracted procedure: k30605 
o|contracted procedure: k30550 
o|contracted procedure: k30626 
o|contracted procedure: k30630 
o|contracted procedure: k30634 
o|contracted procedure: k30679 
o|contracted procedure: k30688 
o|contracted procedure: k30767 
o|contracted procedure: k30694 
o|contracted procedure: k30700 
o|contracted procedure: k30707 
o|contracted procedure: k30716 
o|contracted procedure: k30722 
o|contracted procedure: k30736 
o|contracted procedure: k30752 
o|contracted procedure: k30756 
o|contracted procedure: k30763 
o|contracted procedure: k30805 
o|contracted procedure: k30814 
o|contracted procedure: k30833 
o|contracted procedure: k30837 
o|contracted procedure: k30872 
o|contracted procedure: k31106 
o|contracted procedure: k30882 
o|contracted procedure: k30912 
o|contracted procedure: k30926 
o|contracted procedure: k30896 
o|contracted procedure: k3092235240 
o|contracted procedure: k30900 
o|contracted procedure: k3092235244 
o|contracted procedure: k30932 
o|contracted procedure: k30948 
o|contracted procedure: k30951 
o|contracted procedure: k30954 
o|contracted procedure: k30960 
o|contracted procedure: k30979 
o|contracted procedure: k30982 
o|contracted procedure: k30985 
o|contracted procedure: k30993 
o|contracted procedure: k31001 
o|contracted procedure: k31019 
o|contracted procedure: k31028 
o|contracted procedure: k31035 
o|contracted procedure: k31043 
o|contracted procedure: k31138 
o|contracted procedure: k31142 
o|contracted procedure: k31146 
o|contracted procedure: k31150 
o|contracted procedure: k31202 
o|contracted procedure: k31219 
o|contracted procedure: k31233 
o|contracted procedure: k31222 
o|contracted procedure: k31229 
o|contracted procedure: k31257 
o|contracted procedure: k31264 
o|contracted procedure: k31298 
o|contracted procedure: k31307 
o|contracted procedure: k31310 
o|contracted procedure: k31319 
o|contracted procedure: k31334 
o|contracted procedure: k31330 
o|contracted procedure: k31347 
o|contracted procedure: k31350 
o|contracted procedure: k31357 
o|contracted procedure: k31366 
o|contracted procedure: k31369 
o|contracted procedure: k31372 
o|contracted procedure: k31402 
o|contracted procedure: k31391 
o|contracted procedure: k31412 
o|contracted procedure: k31415 
o|contracted procedure: k31466 
o|contracted procedure: k31490 
o|contracted procedure: k31508 
o|contracted procedure: k31518 
o|contracted procedure: k31538 
o|contracted procedure: k31548 
o|contracted procedure: k31552 
o|contracted procedure: k31593 
o|contracted procedure: k31603 
o|contracted procedure: k31607 
o|contracted procedure: k31650 
o|contracted procedure: k31635 
o|contracted procedure: k31893 
o|contracted procedure: k31912 
o|contracted procedure: k31922 
o|contracted procedure: k31932 
o|contracted procedure: k31942 
o|contracted procedure: k31952 
o|contracted procedure: k31962 
o|contracted procedure: k31972 
o|contracted procedure: k31982 
o|contracted procedure: k31992 
o|contracted procedure: k32002 
o|contracted procedure: k32012 
o|contracted procedure: k32022 
o|contracted procedure: k32032 
o|contracted procedure: k32038 
o|contracted procedure: k32041 
o|contracted procedure: k32065 
o|contracted procedure: k32072 
o|contracted procedure: k32081 
o|contracted procedure: k32088 
o|contracted procedure: k32107 
o|contracted procedure: k32116 
o|contracted procedure: k32127 
o|contracted procedure: k32123 
o|contracted procedure: k32136 
o|contracted procedure: k32143 
o|contracted procedure: k32153 
o|contracted procedure: k32149 
o|contracted procedure: k32098 
o|contracted procedure: k32094 
o|simplifications: ((if . 21) (let . 809)) 
o|removed binding forms: 2331 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest666667 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest666667 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest679680 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest679680 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? fns730 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? fns737 0 
(o x)|known list op on rest arg sublist: ##core#rest-car lst816 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest845849 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest845849 0 
o|inlining procedure: k10095 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest874878 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest874878 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest900903 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest900903 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest900903 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr rest900903 0 
(o x)|known list op on rest arg sublist: ##core#rest-car tst934 0 
(o x)|known list op on rest arg sublist: ##core#rest-car bool1184 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1267 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1270 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1274 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1277 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1280 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1283 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1286 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1289 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1292 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1295 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1298 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1301 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1304 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1307 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1310 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1313 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1317 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1321 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1328 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1341 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc1346 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest13741376 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest13741376 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest13871389 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest13871389 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14001402 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14001402 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14131415 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14131415 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14261428 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14261428 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14391441 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14391441 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14521454 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14521454 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14651467 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14651467 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest14951497 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest14951497 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest20512052 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest20512052 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? args2284 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest24932495 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest24932495 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest25332535 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest25332535 0 
o|contracted procedure: k14974 
(o x)|known list op on rest arg sublist: ##core#rest-null? ns2753 0 
o|inlining procedure: k16619 
o|inlining procedure: k16619 
o|inlining procedure: k16619 
o|inlining procedure: k16619 
o|inlining procedure: k16619 
o|inlining procedure: k16716 
o|inlining procedure: k16716 
o|inlining procedure: k16716 
o|inlining procedure: k16716 
o|inlining procedure: k16716 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest32293231 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest32293231 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest32933295 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest32933295 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest32933295 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr rest32933295 0 
(o x)|known list op on rest arg sublist: ##core#rest-car str-or-sym3366 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest34333436 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest34333436 0 
(o x)|known list op on rest arg sublist: ##core#rest-car n3531 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest35503553 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest35503553 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest35773580 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest35773580 0 
(o x)|known list op on rest arg sublist: ##core#rest-car y3626 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? lsts3687 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? lsts3715 0 
o|contracted procedure: k32264 
(o x)|known list op on rest arg sublist: ##core#rest-car loc3883 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc3887 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc3890 0 
(o x)|known list op on rest arg sublist: ##core#rest-car loc3893 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest40924093 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest40924093 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest41814184 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest41814184 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest42394241 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest42394241 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest42944295 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest42944295 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest43074308 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest43074308 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest43394340 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest43394340 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest43504351 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest43504351 0 
o|contracted procedure: k20383 
o|contracted procedure: k21681 
o|contracted procedure: k21778 
o|contracted procedure: k22335 
(o x)|known list op on rest arg sublist: ##core#rest-null? xs4783 0 
(o x)|known list op on rest arg sublist: ##core#rest-car xs4783 0 
o|contracted procedure: k22453 
(o x)|known list op on rest arg sublist: ##core#rest-null? xs4813 0 
(o x)|known list op on rest arg sublist: ##core#rest-car xs4813 0 
o|contracted procedure: k22533 
(o x)|known list op on rest arg sublist: ##core#rest-null? xs4821 0 
(o x)|known list op on rest arg sublist: ##core#rest-car xs4821 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? xs4901 0 
(o x)|known list op on rest arg sublist: ##core#rest-car xs4901 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest50215023 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest50215023 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest50345035 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest50345035 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest50445046 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest50445046 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest50565058 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest50565058 0 
o|inlining procedure: "(library.scm:4460) outstr05143" 
o|inlining procedure: "(library.scm:4459) outstr05143" 
o|inlining procedure: "(library.scm:4462) outstr05143" 
o|inlining procedure: "(library.scm:4464) outstr05143" 
o|inlining procedure: "(library.scm:4474) outstr05143" 
o|inlining procedure: k23944 
o|inlining procedure: k23944 
o|inlining procedure: "(library.scm:4513) specialchar?5145" 
o|inlining procedure: "(library.scm:4532) specialchar?5145" 
o|contracted procedure: k24255 
o|contracted procedure: k24270 
o|inlining procedure: "(library.scm:4536) specialchar?5145" 
o|inlining procedure: k31830 
o|inlining procedure: k31830 
o|contracted procedure: k25403 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest56375638 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest56375638 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest56375638 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr rest56375638 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest57045705 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest57045705 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest57045705 0 
(o x)|known list op on rest arg sublist: ##core#rest-cdr rest57045705 0 
o|inlining procedure: k26466 
o|inlining procedure: k26466 
o|contracted procedure: k27296 
(o x)|known list op on rest arg sublist: ##core#rest-null? args6230 0 
(o x)|known list op on rest arg sublist: ##core#rest-car args6230 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? _%rest62346286 1 
(o x)|known list op on rest arg sublist: ##core#rest-car _%rest62346286 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr _%rest62346286 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest62996302 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest62996302 0 
(o x)|known list op on rest arg sublist: ##core#rest-car args6320 0 
(o x)|known list op on rest arg sublist: ##core#rest-car args6320 0 
(o x)|known list op on rest arg sublist: ##core#rest-car args6320 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest64066408 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest64066408 0 
(o x)|known list op on rest arg sublist: ##core#rest-car arg6949 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest70527055 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest70527055 0 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest72097210 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest72097210 0 
o|inlining procedure: k31313 
o|inlining procedure: k31341 
o|inlining procedure: k31406 
o|inlining procedure: k31406 
(o x)|known list op on rest arg sublist: ##core#rest-null? rest57615762 0 
(o x)|known list op on rest arg sublist: ##core#rest-car rest57615762 0 
o|replaced variables: 4 
o|removed binding forms: 17 
o|substituted constant variable: r1009636845 
(o x)|known list op on rest arg sublist: ##core#rest-null? r10232 1 
(o x)|known list op on rest arg sublist: ##core#rest-car r10232 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r10232 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r10232 1 
o|substituted constant variable: r1662036901 
o|substituted constant variable: r1662036901 
o|substituted constant variable: r1662036902 
o|substituted constant variable: r1662036902 
o|substituted constant variable: r1671736905 
o|substituted constant variable: r1671736905 
o|substituted constant variable: r1671736906 
o|substituted constant variable: r1671736906 
o|substituted constant variable: r1671736907 
o|substituted constant variable: r1671736907 
(o x)|known list op on rest arg sublist: ##core#rest-null? r17331 1 
(o x)|known list op on rest arg sublist: ##core#rest-car r17331 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r17331 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r17331 1 
o|substituted constant variable: str516136988 
o|removed side-effect free assignment to unused variable: outstr05143 
o|substituted constant variable: str516137004 
o|removed side-effect free assignment to unused variable: specialchar?5145 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25682 1 
(o x)|known list op on rest arg sublist: ##core#rest-car r25682 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25682 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r25682 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25931 1 
(o x)|known list op on rest arg sublist: ##core#rest-car r25931 1 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25931 1 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r25931 1 
o|substituted constant variable: r3140737130 
o|replaced variables: 28 
o|removed binding forms: 12 
o|removed conditional forms: 6 
o|inlining procedure: k24144 
o|inlining procedure: k24206 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25937 2 
(o x)|known list op on rest arg sublist: ##core#rest-car r25937 2 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25937 2 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r25937 2 
o|removed binding forms: 42 
o|contracted procedure: k24292 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25943 3 
(o x)|known list op on rest arg sublist: ##core#rest-car r25943 3 
(o x)|known list op on rest arg sublist: ##core#rest-null? r25943 3 
(o x)|known list op on rest arg sublist: ##core#rest-cdr r25943 3 
o|removed binding forms: 4 
o|removed binding forms: 2 
o|direct leaf routine/allocation: loop778 0 
o|direct leaf routine/allocation: doloop15281529 0 
o|direct leaf routine/allocation: iter1541 0 
o|direct leaf routine/allocation: loop1692 0 
o|direct leaf routine/allocation: ratnum 21 
o|direct leaf routine/allocation: go-inexact!2852 0 
o|direct leaf routine/allocation: loop3508 0 
o|direct leaf routine/allocation: doloop35233524 0 
o|direct leaf routine/allocation: doloop35353539 0 
o|direct leaf routine/allocation: doloop35663567 0 
o|direct leaf routine/allocation: doloop35943595 0 
o|direct leaf routine/allocation: loop3620 0 
o|direct leaf routine/allocation: body38983907 0 
o|direct leaf routine/allocation: def-set?39013910 0 
o|direct leaf routine/allocation: def-convert?39003912 0 
o|direct leaf routine/allocation: body39233932 0 
o|direct leaf routine/allocation: def-set?39263935 0 
o|direct leaf routine/allocation: def-convert?39253937 0 
o|direct leaf routine/allocation: body39483957 0 
o|direct leaf routine/allocation: def-set?39513960 0 
o|direct leaf routine/allocation: def-convert?39503962 0 
o|direct leaf routine/allocation: body59635972 0 
o|direct leaf routine/allocation: def-set?59665975 0 
o|direct leaf routine/allocation: def-convert?59655977 0 
o|direct leaf routine/allocation: for-each-loop60076018 0 
o|direct leaf routine/allocation: loop7067 0 
o|direct leaf routine/allocation: for-each-loop72667277 0 
o|converted assignments to bindings: (loop778) 
o|converted assignments to bindings: (doloop15281529) 
o|contracted procedure: k11472 
o|converted assignments to bindings: (iter1541) 
o|converted assignments to bindings: (loop1692) 
o|contracted procedure: "(library.scm:2481) k15927" 
o|contracted procedure: "(library.scm:2523) k16242" 
o|contracted procedure: "(library.scm:2528) k16267" 
o|contracted procedure: "(library.scm:2535) k16313" 
o|contracted procedure: "(library.scm:2573) k16654" 
o|contracted procedure: "(library.scm:2583) k16741" 
o|converted assignments to bindings: (loop3508) 
o|converted assignments to bindings: (doloop35233524) 
o|converted assignments to bindings: (doloop35353539) 
o|converted assignments to bindings: (doloop35663567) 
o|converted assignments to bindings: (doloop35943595) 
o|converted assignments to bindings: (loop3620) 
o|inlining procedure: "(library.scm:3311) k19068" 
o|inlining procedure: "(library.scm:3311) k19068" 
o|inlining procedure: "(library.scm:3321) k19129" 
o|inlining procedure: "(library.scm:3321) k19129" 
o|inlining procedure: "(library.scm:3331) k19190" 
o|inlining procedure: "(library.scm:3331) k19190" 
o|inlining procedure: "(library.scm:5231) k26669" 
o|inlining procedure: "(library.scm:5231) k26669" 
o|contracted procedure: k26791 
o|converted assignments to bindings: (for-each-loop60076018) 
o|contracted procedure: k30127 
o|converted assignments to bindings: (loop7067) 
o|contracted procedure: k31210 
o|converted assignments to bindings: (for-each-loop72667277) 
o|simplifications: ((let . 13)) 
o|removed binding forms: 10 
o|contracted procedure: "(library.scm:3311) k19068" 
o|contracted procedure: "(library.scm:3321) k19129" 
o|contracted procedure: "(library.scm:3331) k19190" 
o|contracted procedure: "(library.scm:5231) k26669" 
o|removed binding forms: 4 
x|number of unboxed float variables: 0
x|number of inline operations replaced with unboxed ones: 56
o|customizable procedures: (cleanup-before-exit g59005907 for-each-loop58995911 for-each-loop58685880 k31378 loop6842 k30520 check7305 k30859 k30862 k30865 k30868 loop7380 map-loop73477364 loop7371 for-each-loop73247336 loop7243 str7221 pmem7115 round-to7114 pnum7113 k30198 loop7092 loop6940 doloop69126913 doloop68316832 scan6814 loop6811 read-escaped-sexp6633 get/clear-str6693 loop6704 loop26727 readln6632 doloop66716680 loop6647 doloop66366637 loop6611 loop6511 loop6485 def-port62356284 def-header62366282 body62336242 g62566257 writeargs6207 g62136220 for-each-loop62126224 g61486157 map-loop61426167 loop6160 map-loop60856103 map-loop61126129 map-loop60256043 map-loop60526070 chicken.condition#prop-list->kind-prefixed-prop-list k26719 loop5988 k26251 k26254 k26274 for-each-loop58035815 for-each-loop57335744 loop5729 g56745681 for-each-loop56735694 k25866 k25720 loop5657 g56085615 for-each-loop56075618 k25386 copy&append5555 loop5564 k31752 doloop53585359 doloop53345335 doloop53205321 out5253 doloop52905291 outsym5146 g52695270 loop5223 k24178 k24197 loop5188 outchr5144 sym-is-readable?5148 outreadablesym5147 outstr5142 for-each-loop51035121 for-each-loop50715089 hex4968 loop4977 r-symbol4428 k22900 r-ext-symbol4432 r-comment4431 r-char4430 r-vector4422 r-number-with-radix4425 r-number-with-exactness4424 r-digits4427 r-quote4433 loop4763 r-token4426 k22167 k22022 k21737 k21755 r-string4420 loop4672 k21761 k21791 loop4652 k21617 read-unreserved-char-04403 loop4637 r-number4423 r-list4421 k21200 k21243 r-xtoken4429 loop4545 readrec4404 r-spaces4417 starting-line4540 info4400 k20857 k20877 k20981 eat-ws4510 k20766 r-cons-codepoint4419 r-usequence4418 loop4473 lp4468 loop4451 skip-to-eol4401 loop4436 reserved-character4402 skip4409 k19958 def-convert?42684280 def-set?42694278 body42664275 assign4250 k19928 loop4127 loop4108 doloop41154116 close3978 open3977 doloop39853986 k32362 loop3855 loop3838 ##sys#dynamic-unwind loop3742 map-loop37183735 loop3705 for-each-loop36903700 lp3677 loop3663 loop3652 lookup-char3616 ##sys#vector-resize loop3515 map-loop33853404 compare-slots3312 loop3324 walk3313 ##sys#string->compnum scan-prefix3239 scan-real3105 k16593 k16651 scan-ureal3027 k16239 k16310 scan-decimal-tail2950 scan-digits+hashes2913 k16037 k16049 scan-exponent2933 k16076 safe-exponent2853 k15975 scan-digits2878 scan-hashes2901 lp2903 k15815 lp2880 round-quotient k15544 loop2754 loop2744 k15083 slow-expt2676 log-expt2675 lp2660 k14728 lp2648 k14249 k14255 k14187 k14193 loop2462 loop2450 find-ratio-between sr2427 ##sys#internal-gcd loop2285 deliver2218 doloop22202221 make-complex doloop17091710 loop1702 loop1677 k11542 k11551 loop1519 loop1487 copy1477 loop1475 k11008 lp1348 loop936 loop921 k10240 loop884 loop862 k10041 loop832 loop818 loop803 loop795 loop789 loop764 loop749 loop738 loop708 loop696 loop653 loop644 loop108) 
o|calls to known targets: 841 
o|identified direct recursive calls: f_8891 1 
o|unused rest argument: env348 f_9161 
o|unused rest argument: evaluator355 f_9185 
o|unused rest argument: _607 f_9302 
o|identified direct recursive calls: f_9398 1 
o|unused rest argument: rest666667 f_9436 
o|unused rest argument: rest679680 f_9454 
o|unused rest argument: _720 f_9557 
o|unused rest argument: _721 f_9559 
o|unused rest argument: _762 f_9714 
o|identified direct recursive calls: f_9777 1 
o|identified direct recursive calls: f_9799 1 
o|identified direct recursive calls: f_9835 1 
o|identified direct recursive calls: f_9864 2 
o|identified direct recursive calls: f_9911 1 
o|identified direct recursive calls: f_9965 2 
o|identified direct recursive calls: f_10081 1 
o|unused rest argument: rest845849 f_10036 
o|unused rest argument: rest874878 f_10122 
o|unused rest argument: rest900903 f_10226 
o|identified direct recursive calls: f_10961 1 
o|unused rest argument: rest13741376 f_11077 
o|unused rest argument: rest13871389 f_11093 
o|unused rest argument: rest14001402 f_11109 
o|unused rest argument: rest14131415 f_11125 
o|unused rest argument: rest14261428 f_11141 
o|unused rest argument: rest14391441 f_11157 
o|unused rest argument: rest14521454 f_11173 
o|unused rest argument: rest14651467 f_11189 
o|identified direct recursive calls: f_11236 1 
o|identified direct recursive calls: f_11290 1 
o|unused rest argument: rest14951497 f_11314 
o|identified direct recursive calls: f_11386 1 
o|identified direct recursive calls: f_11429 1 
o|identified direct recursive calls: f_11480 1 
o|identified direct recursive calls: f_11836 1 
o|identified direct recursive calls: f_11884 1 
o|identified direct recursive calls: f_11946 1 
o|unused rest argument: rest20512052 f_12653 
o|identified direct recursive calls: f_13700 1 
o|identified direct recursive calls: f_13882 1 
o|identified direct recursive calls: f_13926 1 
o|unused rest argument: rest24932495 f_14057 
o|unused rest argument: rest25332535 f_14283 
o|identified direct recursive calls: f_14875 2 
o|identified direct recursive calls: f_15754 1 
o|identified direct recursive calls: f_15862 1 
o|identified direct recursive calls: f_17089 6 
o|unused rest argument: rest32293231 f_17070 
o|unused rest argument: rest32933295 f_17325 
o|unused rest argument: rest34333436 f_17743 
o|identified direct recursive calls: f_17901 1 
o|identified direct recursive calls: f_17941 1 
o|identified direct recursive calls: f_17980 1 
o|identified direct recursive calls: f_18026 1 
o|identified direct recursive calls: f_18098 1 
o|unused rest argument: rest35503553 f_18068 
o|unused rest argument: rest35773580 f_18139 
o|identified direct recursive calls: f_18172 1 
o|identified direct recursive calls: f_18274 1 
o|identified direct recursive calls: f_18856 1 
o|identified direct recursive calls: f_32268 1 
o|identified direct recursive calls: f_19289 3 
o|unused rest argument: rest40924093 f_19560 
o|identified direct recursive calls: f_19613 1 
o|unused rest argument: rest41814184 f_19770 
o|unused rest argument: rest42394241 f_19918 
o|unused rest argument: rest42944295 f_20082 
o|unused rest argument: rest43074308 f_20107 
o|unused rest argument: rest43394340 f_20219 
o|unused rest argument: rest43504351 f_20237 
o|identified direct recursive calls: f_20552 1 
o|unused rest argument: xs4783 f_22351 
o|unused rest argument: xs4813 f_22478 
o|unused rest argument: xs4821 f_22549 
o|unused rest argument: xs4901 f_22971 
o|unused rest argument: rest50215023 f_23669 
o|unused rest argument: rest50345035 f_23690 
o|unused rest argument: rest50445046 f_23705 
o|unused rest argument: rest50565058 f_23723 
o|identified direct recursive calls: f_24164 1 
o|unused rest argument: rest56375638 f_25676 
o|unused rest argument: rest57045705 f_25925 
o|identified direct recursive calls: f_26898 1 
o|identified direct recursive calls: f_26818 1 
o|identified direct recursive calls: f_26864 1 
o|identified direct recursive calls: f_26940 1 
o|identified direct recursive calls: f_27054 2 
o|unused rest argument: args6230 f_27280 
o|unused rest argument: rest62996302 f_27471 
o|unused rest argument: rest64066408 f_28293 
o|identified direct recursive calls: f_28696 1 
o|identified direct recursive calls: f_29038 1 
o|identified direct recursive calls: f_29372 1 
o|unused rest argument: rest70527055 f_30101 
o|identified direct recursive calls: f_30145 1 
o|identified direct recursive calls: f_30212 1 
o|unused rest argument: rest72097210 f_30524 
o|identified direct recursive calls: f_30711 1 
o|identified direct recursive calls: f_30943 2 
o|identified direct recursive calls: f_31214 1 
o|unused rest argument: args5858 f_31615 
o|unused rest argument: rest57615762 f_31633 
o|fast box initializations: 161 
o|fast global references: 81 
o|fast global assignments: 20 
o|dropping unused closure argument: f_12675 
o|dropping unused closure argument: f_12855 
o|dropping unused closure argument: f_13700 
o|dropping unused closure argument: f_14917 
o|dropping unused closure argument: f_14931 
o|dropping unused closure argument: f_15181 
o|dropping unused closure argument: f_15607 
o|dropping unused closure argument: f_15650 
o|dropping unused closure argument: f_18160 
o|dropping unused closure argument: f_18856 
o|dropping unused closure argument: f_19246 
o|dropping unused closure argument: f_19344 
o|dropping unused closure argument: f_20540 
o|dropping unused closure argument: f_25314 
o|dropping unused closure argument: f_26038 
o|dropping unused closure argument: f_26699 
o|dropping unused closure argument: f_26898 
o|dropping unused closure argument: f_27235 
o|dropping unused closure argument: f_28740 
o|dropping unused closure argument: f_28787 
o|dropping unused closure argument: f_30271 
o|dropping unused closure argument: f_30285 
o|dropping unused closure argument: f_30545 
o|dropping unused closure argument: f_30828 
o|dropping unused closure argument: f_31214 
*/
/* end of file */
